#!/usr/bin/env python
# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2015 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################


import os
import re
import datetime
from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import *
from library.Utils import anyToUnicode
from database import *
from Import import *



class Cimport():
    def __init__(self, log=None, edtFile=None, db = None, progressBar = None, btnImport=None, btnClose=None):
        if not hasattr(self, 'edtFileName'):
            self.edtFileName = edtFile
        if not hasattr(self, 'db'):
            self.db = db
        if not hasattr(self, 'log'):
            self.log = log
        if not hasattr(self, 'progressBar'):
            self.progressBar = progressBar
        if self.progressBar:
            self.progressBar.setValue(0)
        if not hasattr(self, 'btnImport'):
            self.btnImport = btnImport
        if not hasattr(self, 'btnClose'):
            self.btnClose = btnClose
        self.n=0
        self.row=None
        self.abort=False
        self.importRun=False
        if self.edtFileName:
            QtCore.QObject.connect(
                self.edtFileName, QtCore.SIGNAL("textChanged(QString)"), self.checkName)

    def checkName(self):
        self.btnImport.setEnabled(self.edtFileName.text()!='')

    @QtCore.pyqtSignature('')
    def on_btnClose_clicked(self):
        if self.importRun:
            self.abort=True
        else:
            self.close()

    def err2log(self, e):
        if self.log:
            self.log.append(u'запись '+str(self.n)+' (id=\"'+self.row['ID']+u'\"): '+e)

    @QtCore.pyqtSignature('')
    def on_btnImport_clicked(self):
        self.btnImport.setEnabled(False)
        self.btnClose.setText(u'Прервать')
        if self.progressBar:
            self.progressBar.setValue(0)
        self.abort=False
        self.importRun=True
        try:
            self.startImport()
        except IOError, e:
            #QtGui.qApp.logCurrentException()
            msg=''
            if hasattr(e, 'filename'):
                msg = u'%s:\n[Errno %s] %s' % (e.filename, e.errno, anyToUnicode(e.strerror))
            else:
                msg = u'[Errno %s] %s' % (e.errno, anyToUnicode(e.strerror))
            QtGui.QMessageBox.critical (self,
                u'Произошла ошибка ввода-вывода', msg, QtGui.QMessageBox.Close)
        except Exception, e:
            #QtGui.qApp.logCurrentException()
            if self.db:
                self.db.rollback()
            QtGui.QMessageBox.critical (self,
                u'Произошла ошибка', unicode(e), QtGui.QMessageBox.Close)
            self.abort = True
        if self.progressBar:
            self.progressBar.setValue(0)
            self.progressBar.setFormat(u'прервано' if self.abort else u'готово')
        self.btnImport.setEnabled(True)
        self.btnClose.setText(u'Закрыть')
        self.abort=False
        self.importRun=False

    def logQueryResult(self, query):
        u"""Записывает результат выполнения запроса query в лог по строкам"""
        if self.log:
            logQueryResult(self.log, query)

    def setSqlVariable(self, name, value):
        u"""Устанавливает значение переменной в запросе на SQL"""
        setSqlVariable(self.db, name, value)

    def runScript(self, instream,  dict = {}):
        u"""Выполняет последовательность запросов к базе данных self.db, считанную из входного потока instream.
        instream - это произвольный контейнер со строками
        dict - набор пар (имя, значение) для установки переменных в SQL
        """
        return runScript(self.db, self.log, instream, dict)


class CDBFimport(Cimport):
    @QtCore.pyqtSignature('')
    def on_btnView_clicked(self):
        fname=unicode(forceStringEx(self.edtFileName.text()))
        if fname:
            CDbfViewDialog(self, fileName=fname).exec_()

    def insertTableDataFromDbf(self, tablename, filename, encoding, binaries=[], fields=None, packet=1000):
        dbf = Dbf(filename, readOnly=True, encoding=encoding, enableFieldNameDups=True)
#    for i in binaries:
#        dbf.fieldDefs[i].encoding = 'latin-1'
        self.log.append(u'В таблице найдено %d записей.'%len(dbf))
        #TODO: сделать загрузку пакетами
        for i in xrange(len(dbf)):
            record = dbf[i]
            for j in binaries:
                record[j] = binaryToInt(record[j])
            insertRecordFromDbf(self.db, tablename, record, fields)
            if (i+1) % packet == 0:
                self.log.append(u'Загружено %d записей.'%(i+1))
            QtGui.qApp.processEvents()
        dbf.close()

    def fullTable(self, filename, tablename, binaries, fields=None, packet=1000):
        self.log.append(u'Заполняется таблица %s из файла %s...'%(tablename, filename))
        QtGui.qApp.processEvents()
        self.insertTableDataFromDbf(tablename, unicode(filename), 'ibm866', binaries, fields, packet)

