# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2015 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import *

import database
from Utils import *


AgeSelectorUnits = u'днмг'

def parseAgeSelectorPart(val):
    if val:
        strCount, strUnit = re.match(r'^(\d*)(\D*)', val).groups()
        count = int(strCount) if strCount else 0
        unit  = AgeSelectorUnits.find((strUnit[:1]).lower())+1
        return (unit, count)
    else:
        return(0,0)


def parseAgeSelector(val):
    """ selector syntax: "{NNN{д|н|м|г}-{MMM{д|н|м|г}}" -
    с NNN дней/недель/месяцев/лет по MMM дней/недель/месяцев/лет;
    пустая нижняя или верхняя граница - нет ограничения снизу или сверху"""
    parts = forceString(val).split('-')
    if len(parts) >= 2:
        (begUnit, begCount) = parseAgeSelectorPart(parts[0])
        (endUnit, endCount) = parseAgeSelectorPart(parts[1])
        return (begUnit, begCount, endUnit, endCount)
    elif len(parts) == 1 and parts[0]:
        (begUnit, begCount) = parseAgeSelectorPart(parts[0])
        return (begUnit, begCount, 0, 0)
    else:
        return (0, 0, 0, 0)


def composeAgeSelectorPart(unit, count):
    if unit == 0 or count == 0:
        return ''
    else:
        return str(count)+AgeSelectorUnits[unit-1]


def composeAgeSelector(begUnit, begCount, endUnit, endCount):
    if (begUnit == 0 or begCount == 0) and (endUnit == 0 or endCount == 0):
        return ''
    else:
        return composeAgeSelectorPart(begUnit,begCount)+'-'+composeAgeSelectorPart(endUnit,endCount)


def checkAgeSelector((begUnit, begCount, endUnit, endCount), ageTuple):
    if begUnit != 0:
        if ageTuple[begUnit-1]<begCount:
            return False
    if endUnit != 0:
        if ageTuple[endUnit-1]>endCount:
            return False
    return True




def recordAcceptable(clientSex, clientAge, record):
    sex = forceInt(record.value('sex'))
    if sex != 0:
        if sex != clientSex:
            return False
    age = forceStringEx(record.value('age'))
    if age:
        if clientAge is None:
            return False
        ageSelector = parseAgeSelector(age)
        return checkAgeSelector(ageSelector, clientAge)
    return True


def checkDiagnosis(db, parentWidget, MKB, diagFilter, clientId, clientSex, clientAge):
    # 1) диагноз должен существовать
    tableMKB = db.table('MKB')
    record  = db.getRecordEx(tableMKB, 'DiagName, sex, age, MKBSubclass_id', tableMKB['DiagID'].eq(MKB[:5]))
    if not record:
        message = u'Кода %s не существует' % (MKB)
        QtGui.QMessageBox.critical(parentWidget, u'Внимание!', message)
        return False
    # 1.1) и субклассификация - тоже
    subclassMKB = MKB[5:]
    if subclassMKB:
        subclassId = forceInt(record.value('MKBSubclass_id'))
        tableSubclass = db.table('rbMKBSubclass_Item')
        record  = db.getRecordEx(tableSubclass, 'id', [tableSubclass['master_id'].eq(subclassId), tableSubclass['code'].eq(subclassMKB)])
        if not record:
            message = u'В коде %s указана недопустимая субклассификация' % (MKB)
            QtGui.QMessageBox.critical(parentWidget, u'Внимание!', message)
            return False
    # 2) диагноз должен быть применим по полу и возрасту
    elif not recordAcceptable(clientSex, clientAge, record):
        message = u'Код %s (%s)\n в данном случае не применим' % (MKB, forceString(record.value('DiagName')))
        QtGui.QMessageBox.critical(parentWidget, u'Внимание!', message)
        return False
    # 3) диагноз должен быть применим по специалисту
    if diagFilter and diagFilter.find(MKB[:1])>=0:
        message = u'Код %s (%s)\nпо правилам контроля не должен применяться данным специалистом\nВвод верен?' % (MKB, forceString(record.value('DiagName')))
        if QtGui.QMessageBox.Yes != QtGui.QMessageBox.question(parentWidget, u'Внимание!', message, QtGui.QMessageBox.Yes|QtGui.QMessageBox.No, QtGui.QMessageBox.Yes):
            return False
    return True


def specifyDiagnosis(db, parentWidget, MKB, diagFilter, clientId, clientSex, clientAge):
    specifiedMKB = MKB
    specifiedMKBEx = ''
    specifiedCharacterId = None
    specifiedTraumaTypeId = None
    acceptable = checkDiagnosis(db, parentWidget, MKB, diagFilter, clientId, clientSex, clientAge)
    if acceptable and MKB[:1] != 'Z':
        # 4) диагноз должен быть подтверждён, если есть похожий
        table = db.table('Diagnosis')
        cols = 'id, MKB, MKBEx, character_id, traumaType_id'
        condClient = table['client_id'].eq(clientId)
        record = db.getRecordEx(table, cols, [condClient, table['MKB'].eq(MKB)]) # точно такой
        if not record:
            record = db.getRecordEx(table, cols, [condClient, table['MKB'].like(MKB+'%')]) # такой, но с субклассификацией
        if not record:
            record = db.getRecordEx(table, cols, [condClient, table['MKB'].like(MKB[:5]+'%')]) # такой, без субклассификацией (другой субклассификацией)
        if not record:
            record = db.getRecordEx(table, cols, [condClient, table['MKB'].like(MKB[:3]+'%')]) # такой, без классификации (другой классификацией)
        if not record:
            pass # другой в этом-же блоке
        if record:
            suggestedMKB = forceString(record.value('MKB'))
            suggestedMKBEx = forceString(record.value('MKBEx'))
            suggestedCharacterId = forceRef(record.value('character_id'))
            suggestedTraumaTypeId = forceRef(record.value('traumaType_id'))
            if suggestedMKB != MKB:
                message = u'Ранее для этого пациента был указан код %s\nИзменить на этот код?' % (suggestedMKB)
                if QtGui.QMessageBox.Yes == QtGui.QMessageBox.question(parentWidget, u'Внимание!', message, QtGui.QMessageBox.Yes|QtGui.QMessageBox.No, QtGui.QMessageBox.Yes):
                    specifiedMKB = suggestedMKB
                    specifiedMKBEx = suggestedMKBEx
                    specifiedCharacterId = suggestedCharacterId
                    specifiedTraumaTypeId = suggestedTraumaTypeId
            else:
                specifiedMKB = suggestedMKB
                specifiedMKBEx = suggestedMKBEx
                specifiedCharacterId = suggestedCharacterId
                specifiedTraumaTypeId = suggestedTraumaTypeId
    return (acceptable, specifiedMKB, specifiedMKBEx, specifiedCharacterId, specifiedTraumaTypeId)


def getDiagnosisPrimacy(db, averageDuration, clientId, date, MKB, characterId):
    table = db.table('Diagnosis')
    cond  = [table['client_id'].eq(clientId), table['MKB'].eq(MKB), table['endDate'].le(date)]
    characterCode = forceString(db.translate('rbDiseaseCharacter', 'id', characterId, 'code'))
    if characterCode == '1':
        duration = forceInt(db.translate('MKB', 'DiagID', MKB[:5], 'duration'))
        if not duration:
            duration = averageDuration
        cond.append(table['endDate'].ge(date.addDays(-duration)))
    else:
        cond.append(table['endDate'].ge(firstYearDay(date)))
    record = db.getRecordEx(table, 'id', cond)
    return not record


#######################################################################
#
# второй подход к ведению ЛУД
#
#######################################################################

# считаем количество Diagnostic, TempInvalid и TempInvalid_Period ссылающихся на данный diagnosis
def countUsageDiagnosis(db, diagnosisId, excludeDiagnosticId, countTempInvalid):
    result = 0
    if diagnosisId:
        table = db.table('Diagnostic')
        cond = [table['diagnosis_id'].eq(diagnosisId)]
        if excludeDiagnosticId:
            cond.append(table['id'].ne(excludeDiagnosticId))
        record = db.getRecordEx(table, 'COUNT(id)', cond)
        if record:
            result += forceInt(record.value(0))

        if countTempInvalid:
            for tableName in ('TempInvalid', 'TempInvalid_Period'):
                table = db.table(tableName)
                cond = [table['diagnosis_id'].eq(diagnosisId)]
                record = db.getRecordEx(table, 'COUNT(id)', cond)
                if record:
                    result += forceInt(record.value(0))
    return result


def createDiagnosisRecord(db, clientId, diagnosisTypeCode, characterCode, MKB, MKBEx, dispanserId, traumaTypeId, setDate, endDate, personId):
    # создаём новую запись Diagnosis
    diagnosisTypeId = db.translate('rbDiagnosisType', 'code', diagnosisTypeCode, 'id')
    characterId = db.translate('rbDiseaseCharacter', 'code', characterCode, 'id')

    table = db.table('Diagnosis')
    record = table.newRecord()
    record.setValue('client_id',        toVariant(clientId))
    record.setValue('diagnosisType_id', toVariant(diagnosisTypeId))
    record.setValue('character_id',     toVariant(characterId))
    record.setValue('MKB',              toVariant(MKB))
    record.setValue('MKBEx',            toVariant(MKBEx))
    record.setValue('dispanser_id',     toVariant(dispanserId))
    record.setValue('traumaType_id',    toVariant(traumaTypeId))
    record.setValue('setDate',          toVariant(setDate))
    record.setValue('endDate',          toVariant(endDate))
    record.setValue('person_id',        toVariant(personId))
    return db.insertRecord(table, record)


def modifyDiagnosisRecord(db, record, diagnosisTypeCode, characterCode, MKB, MKBEx, dispanserId, traumaTypeId, setDate, endDate, personId):
    # меняем существующую запись Diagnosis, из предположения что она использована только в одном
    # месте и при этом исправляется ошибочный ввод.
    diagnosisTypeId = db.translate('rbDiagnosisType', 'code', diagnosisTypeCode, 'id')
    characterId = db.translate('rbDiseaseCharacter', 'code', characterCode, 'id')
    table = db.table('Diagnosis')
    record.setValue('diagnosisType_id', toVariant(diagnosisTypeId))
    record.setValue('character_id',     toVariant(characterId))
    record.setValue('MKB',              toVariant(MKB))
    record.setValue('MKBEx',            toVariant(MKBEx))
    record.setValue('dispanser_id',     toVariant(dispanserId))
    record.setValue('traumaType_id',    toVariant(traumaTypeId))
    record.setValue('setDate',          toVariant(setDate))
    record.setValue('endDate',          toVariant(endDate))
    record.setValue('person_id',        toVariant(personId))
    return db.updateRecord(table, record)



def updateDiagnosisRecord(db, record, setDate, endDate, MKBEx, dispanserId, traumaTypeId, personId):
    # меняем существующую запись Diagnosis, из предположения что она использована во многих
    # местах и при этом "расширяется"
    oldSetDate = forceDate(record.value('setDate'))
    oldEndDate = forceDate(record.value('endDate'))
    if oldEndDate<=endDate:
        record.setValue('endDate', QVariant(endDate))
        record.setValue('MKBEx', toVariant(MKBEx))
        if dispanserId:
            record.setValue('dispanser_id', toVariant(dispanserId))
        if traumaTypeId:
            record.setValue('traumaType_id',toVariant(traumaTypeId))
        if personId:
            record.setValue('person_id',toVariant(personId))

    if not setDate.isNull():
        if oldSetDate.isNull():
#            if setDate<oldEndDate:
#                record.setValue('setDate', QVariant(setDate))
            pass
        else:
            if setDate<oldSetDate:
                record.setValue('setDate', QVariant(setDate))
    db.updateRecord('Diagnosis', record)


def findDiagnosisRecord(db, table, permanentCond, date, delta):
    # ищем в периоде
    record = db.getRecordEx(table, '*', [permanentCond,
                                         db.joinOr([table['setDate'].isNull(), table['setDate'].le(date.addDays(delta))]),
                                         table['endDate'].ge(date.addDays(-delta))], 'endDate DESC')
    if not record:
        # ищем после периода
        record = db.getRecordEx(table, '*', [permanentCond, table['endDate'].le(date)], 'endDate DESC')
    if not record:
        # ищем до периода
        record = db.getRecordEx(table, '*', [permanentCond], 'endDate DESC')
    return record


def getDiagnosisId2(db,
                    averageDuration,
            date,
            personId,
            clientId,
            diagnosisTypeId,
            MKB,
            MKBEx,
            characterId,
            dispanserId,
            traumaTypeId,
            suggestedDiagnosisId=None,
            ownerId=None):
    if MKB[:1] == 'Z':
        diagnosisTypeCode = '98' # magic!!!
    else:
        diagnosisTypeCode = forceString(db.translate('rbDiagnosisType', 'id', diagnosisTypeId, 'replaceInDiagnosis'))
    characterCode = forceString(db.translate('rbDiseaseCharacter', 'id', characterId, 'replaceInDiagnosis'))
    table = db.table('Diagnosis')
    result = None

    if diagnosisTypeCode == '98':
        result = createDiagnosisRecord(db, clientId, diagnosisTypeCode, characterCode, MKB, MKBEx, dispanserId, traumaTypeId, date, date, personId)
    else:
        assert diagnosisTypeCode in ('2', '9')

        clientCond = table['client_id'].eq(clientId)
        diagCond = table['MKB'].eq(MKB)
        commonCond = db.joinAnd([table['deleted'].eq(0), table['mod_id'].isNull(), clientCond, diagCond])
        setDate = date

        if characterCode == '3' : # хроническое
            record = findDiagnosisRecord(db, table, commonCond, date, 0)
            docCharacterCode = forceString(db.translate('rbDiseaseCharacter', 'id', characterId, 'code'))
            if docCharacterCode in ('3', '4'): # "хроническое" или "обострение хронического"
                setDate = QDate()
            if record:
                oldCharacterId = forceRef(record.value('character_id'))
                oldCharacterCode = forceString(db.translate('rbDiseaseCharacter', 'id', oldCharacterId, 'code'))
                oldSetDate = forceDate(record.value('setDate'))
                oldEndDate = forceDate(record.value('endDate'))
#                countUsage = countUsageDiagnosis(db, suggestedDiagnosisId, ownerId, False)
                countUsage = countUsageDiagnosis(db, forceRef(record.value('id')), ownerId, False)
                if countUsage:
                    if oldCharacterCode == '3':
                        # и раньше было хроническим, обновляем документ
                        if (oldSetDate.isNull() or oldSetDate<date) and docCharacterCode == '2':
                            # при наличии уже установленного хрон. и выставлении впервые установленного
                            characterId = oldCharacterId # меняем в документе на хроническое
                    else:
                        # раньше было острым
                        if oldEndDate.year()>=date.year():
                            # меняем на хроническое
                            characterId = db.translate('rbDiseaseCharacter', 'code', characterCode, 'id')
                            record.setValue('character_id', toVariant(characterId))
                            characterId = db.translate('rbDiseaseCharacter', 'code', '4', 'id') # меняем в документе на обострение
                        else:
                            # прошлогодние данные не меняем.
                            result = createDiagnosisRecord(db, clientId, diagnosisTypeCode, characterCode, MKB, MKBEx, dispanserId, traumaTypeId, setDate, date, personId)
                else:
                    result = modifyDiagnosisRecord(db, record, diagnosisTypeCode, characterCode, MKB, MKBEx, dispanserId, traumaTypeId, setDate, date, personId)
            else:
                result = createDiagnosisRecord(db, clientId, diagnosisTypeCode, characterCode, MKB, MKBEx, dispanserId, traumaTypeId, setDate, date, personId)
        else: # это острое заболевание
            duration = forceInt(db.translate('MKB', 'DiagID', MKB[:5], 'duration'))
            if not duration:
                duration = averageDuration
            record = findDiagnosisRecord(db, table, commonCond, date, duration)
            if record:
                oldCharacterId = forceRef(record.value('character_id'))
                oldCharacterCode = forceString(db.translate('rbDiseaseCharacter', 'id', oldCharacterId, 'code'))
#                countUsage = countUsageDiagnosis(db, suggestedDiagnosisId, ownerId, False)
                countUsage = countUsageDiagnosis(db, forceRef(record.value('id')), ownerId, False)
                if countUsage:
                    if oldCharacterCode == '3':
                        # раньше было хроническим, обновляем в документе на обострение
                        characterId = db.translate('rbDiseaseCharacter', 'code', '4', 'id') # меняем в документе на обострение
                        updateDiagnosisRecord(db, record, date, date, MKBEx, dispanserId, traumaTypeId, personId)
                    else:
                        oldSetDate = forceDate(record.value('setDate'))
                        oldEndDate = forceDate(record.value('endDate'))
                        if oldSetDate.addDays(-duration)>date or date>oldEndDate.addDays(duration):
                            result = createDiagnosisRecord(db, clientId, diagnosisTypeCode, characterCode, MKB, MKBEx, dispanserId, traumaTypeId, date, date, personId)
                else:
                    result = modifyDiagnosisRecord(db, record, diagnosisTypeCode, characterCode, MKB, MKBEx, dispanserId, traumaTypeId, setDate, date, personId)
            else:
                result = createDiagnosisRecord(db, clientId, diagnosisTypeCode, characterCode, MKB, MKBEx, dispanserId, traumaTypeId, date, date, personId)
        if result == None:
            if diagnosisTypeCode == '2':
                diagnosisTypeId = db.translate('rbDiagnosisType', 'code', diagnosisTypeCode, 'id')
                record.setValue('diagnosisType_id', diagnosisTypeId)
            updateDiagnosisRecord(db, record, setDate, date, MKBEx, dispanserId, traumaTypeId, personId)
            result = forceRef(record.value('id'))
    if suggestedDiagnosisId and suggestedDiagnosisId != result:
        if countUsageDiagnosis(db, suggestedDiagnosisId, ownerId, True) == 0:
            db.deleteRecord(table, table['id'].eq(suggestedDiagnosisId))
    return result, characterId