# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2015 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtCore, QtGui, QtSql
from dbfpy.dbf import Dbf
from dbfpy.record import DbfRecord
from binascii import hexlify

u"""Утилиты для импорта данных в БД"""

class CLogBrowser(QtGui.QTextBrowser):
    u"""Лог для записей при импорте; поддерживает синхронную запись в файл"""
    def __init__(self, filename=None):
        QtGui.QTextBrowser.__init__(self)
        self.logfile = None
        self.logstream = None

    def setLogFile(self, filename):
        if filename:
            self.logfile = QtCore.QFile(filename) # файл протокола
            self.logfile.open(QtCore.QFile.Text | QtCore.QFile.WriteOnly)
            self.logstream = QtCore.QTextStream(self.logfile)
        else:
            if self.logfile:
                self.logfile.close()
            self.logfile = None
            self.logstream = None

    def append(self, str):
        QtGui.QTextBrowser.append(self, str)
        QtGui.qApp.processEvents()
        if self.logstream:
            self.logstream << QtCore.QDateTime.currentDateTime().toString("yyyy-MM-dd hh:mm:ss\t");
            self.logstream << str << '\n'
            self.logstream.flush()

    def close(self):
        if self.logfile:
            self.logfile.close()
        QtGui.QWidget.close(self)

def binaryToInt(str):
    str = str.encode('cp866')
    return int('0' + hexlify(str), 16)

def insertRecordFromDbf(db, tablename, record, fields = None):
    u"""
    Конвертирует запись DBF в запись в любой БД
    db - БД
    tablename - имя таблицы БД, куда вставлять запись
    record - запись DBF
    fields - номера полей, которые надо конвертировать (None - все поля). Если номер равен -1, нужно вставить значение по умолчанию.
    """
    db_record = db.record(tablename)
    values = record.asList()
    strvalues = []
    if fields is None:
        fields = xrange(len(values))
    for i in xrange(len(fields)):
        field = db_record.field(i)
        if fields[i] == -1:
            field.setValue(field.defaultValue())
        else:
            if values[fields[i]]:
                field.setValue(QtCore.QVariant(values[fields[i]]))
            else:
                field.clear()
        # fields.append(unicode('`'+record.fieldName(i)+'`'))
        strvalues = strvalues + [unicode(db.db.driver().formatValue(field, False)), ]
    stmt = ('INSERT INTO ' +  tablename +
#          '(' + (', '.join(fields)) + ') '+
           ' VALUES (' + (', '.join(strvalues)) + ')')
    #print stmt
    querier =  QtSql.QSqlQuery(db.db)
    if not querier.exec_(stmt):
        return querier.lastError()

def insertTableDataFromDbf(db, tablename, filename, encoding, binaries=[], fields=None):
    u"""
    Выкачивает данные из DBF и добавляет в таблицу БД
    tablename - имя таблицы БД
    filename - имя файла DBF
    encoding - кодировка, в которой хранится DBF
    binaries - список номеров полей, которые нужно перекодировать из бинарных в целые
    fields - номера полей, которые надо конвертировать (None - все поля). Если номер равен -1, нужно вставить значение по умолчанию.
    """
    dbf = Dbf(filename, readOnly=True, encoding=encoding, enableFieldNameDups=True)
#    for i in binaries:
#        dbf.fieldDefs[i].encoding = 'latin-1'
    for record in dbf:
        for i in binaries:
            record[i] = binaryToInt(record[i])
        insertRecordFromDbf(db, tablename, record, fields)
        QtGui.qApp.processEvents()
    dbf.close()


def logQueryResult(log, query):
    u"""Записывает результат выполнения запроса query в лог по строкам"""
    while query.next():
        rec = query.record()
        str = query.value(0).toString()
        for i in xrange(1,  rec.count()):
            str = str + '\t' + query.value(i).toString()
        log.append(str)

def setSqlVariable(db, name, value):
    u"""Устанавливает значение переменной в запросе на SQL"""
    if value:
        if str(type(value)) == "<type 'string'>" or str(type(value)) == "<type 'unicode'>":
            strvalue = '\'' + value + '\''
        else:
            strvalue = str(value)
    else:
        strvalue = 'NULL'
    querier =  QtSql.QSqlQuery(db.db)
    if not querier.exec_('SELECT @' + name + ' := ' + strvalue):
        return querier.lastError()
    return None


def execQuery(db, s, log):
    if QtCore.QRegExp('[ \n\r\t]*').exactMatch(s): # пустой запрос
        return None
    #print "QUERY: " + s
    querier =  QtSql.QSqlQuery(db.db)
    if not querier.exec_(s):
        return querier.lastError()
    if log:
        logQueryResult(log, querier) # записываем результат выполнения
    return None

def runScript(db, log, instream, dict = {}):
    u"""Выполняет последовательность запросов к базе данных db, считанную из входного потока instream.
    instream - это произвольный контейнер со строками
    Записывает результаты в окно log
    dict - набор пар (имя, значение) для установки переменных в SQL
    """
    # устанавливаем значения переменных:
    for (key,  value) in dict.items():
        setSqlVariable(db, key,  value)

    # анализируем запросы:
    fullstr = '' # текущий запрос
    delimiter = ';' # разделитель запросов
    for str in instream:
        delimexp = QtCore.QRegExp('delimiter', QtCore.Qt.CaseInsensitive)
        if delimexp.indexIn(str.strip()) == 0: # в этой строке меняется разделитель
            delimiter = str[delimexp.matchedLength():].strip()
        else:
            pos = QtCore.QRegExp('--|\'|\"|%s'%QtCore.QRegExp.escape(delimiter)).indexIn(str, 0) # позиция, с которой начинается комментарий или открывается строка
            while pos != -1:
                if str[pos] == '\'':
                    pos = str.find('\'', pos+1)+1
                elif str[pos] == '\"':
                    pos = str.find('\"', pos+1)+1
                elif str[pos:pos+len(delimiter)] == delimiter: # здесь запрос закончился
                    fullstr = fullstr + ' ' + str[:pos]
                    result = execQuery(db, fullstr, log) # выполняем его
                    if result:
                        return result
                    fullstr = '' # и ждем нового
                    str = str[pos+len(delimiter):]
                    pos = 0
                else: # найден комментарий
                    str = str[:pos]
                    break
                pos = QtCore.QRegExp('--|\'|\"|%s'%QtCore.QRegExp.escape(delimiter)).indexIn(str, pos)
            fullstr = fullstr + ' ' + str
    result = execQuery(db, fullstr, log) # выполняем всё, что осталось в конце
    return result
