# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2015 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################


from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import *

import database
from Utils              import *

MapMKBCharactersToCharacterCode = [
    (u'нет', []),                                      #0
    (u'острое', ['1']),                                #1
    (u'хроническое впервые выявленное', ['2']),        #2
    (u'хроническое ранее известное', ['3']),           #3
    (u'обострение хронического', ['4']),               #4
    (u'хроническое', ['3', '2', '4']),                 #5 такой порядок - для выбора 3 "по умолчанию"
    (u'хроническое или острое', ['3', '1', '2', '4']), #6 idem
                                  ]

MapMKBCharactersToCharacterID = None



def getMapMKBCharactersToCharacterID(db):
    u'''
        получить список id записей характеров в соответствии с
        характером указанным в МКБ
    '''
    global MapMKBCharactersToCharacterID

    if not MapMKBCharactersToCharacterID:
        mapCodeToId = {}
        for record in db.getRecordList('rbDiseaseCharacter', 'id, code'):
            id   = forceRef(record.value(0))
            code = forceString(record.value(1))
            mapCodeToId[code] = id
        result = []
        for (title,  codes) in MapMKBCharactersToCharacterCode:
            idList = []
            for code in codes:
                idList.append(mapCodeToId[code])
            result.append(idList)
        MapMKBCharactersToCharacterID = result
    return MapMKBCharactersToCharacterID


def getAvailableCharacterIdByMKB(db, MKB):
    u'''
        получить список id записей характеров в соответствии с кодом МКБ
    '''
    MKBCharacters = forceInt(db.translate('MKB', 'DiagID', MKB, 'characters'))
    return getMapMKBCharactersToCharacterID(db)[MKBCharacters]


def getRightCharacterId(db, MKB, id): #TODO: переписать всё это на SQL с помощью вспомогательных таблиц
    u"""Проверяет характер заболевания на правильность.
    Если характер неправильный, исправляет на наиболее правдоподобный"""
    characters = getAvailableCharacterIdByMKB(db, MKB)
    if id in characters or (id == 0 and not len(characters)):
        return id
    else:
        return characters[0] if len(characters) else 0