# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2015 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import *

import database
from Utils import *


class CPayStatus:
    initial    = 0 # ни выставления, ни оплаты, ни отказа
    exposed    = 1 # выставлено, но не оплачено и не отказано
    refused    = 2 # отказано
    payed      = 3 # оплачено

    exposedBits = 0x55555555
    refusedBits = 0xAAAAAAAA
    payedBits   = 0xFFFFFFFF


def getPayStatusValueByCode(status, financeCode):
    return status << (2*financeCode)

def getPayStatusMaskByCode(financeCode):
    return 3 << (2*financeCode)

def getPayStatusMask(db, financeId):
    code = forceInt(db.translate('rbFinance', 'id', financeId, 'code'))
    return 3 << (2*code)


def getExposed(payStatusMask):
    return CPayStatus.exposedBits & payStatusMask


def getRefused(payStatusMask):
    return CPayStatus.refusedBits & payStatusMask


def getPayed(payStatusMask):
    return payStatusMask


def getWorstPayStatus(payStatus):
    statusSet = set([CPayStatus.initial])
    while payStatus>0:
        status = payStatus & 0x3
        statusSet.add(status)
        payStatus = payStatus >> 2
    for status in (CPayStatus.exposed, CPayStatus.payed, CPayStatus.refused, CPayStatus.initial):
        if status in statusSet:
            return status


def payStatusText(db, payStatus):
    resultList = []
    for record in db.getRecordList('rbFinance', 'code, name', order='code'):
        code = forceInt(record.value('code'))
        payCode = (payStatus >> (2*code)) & 3
        if payCode:
            name = forceString(record.value('name'))
            resultList.append( name + ': ' + ['-', u'выставлено', u'отказано', u'оплачено'][payCode] )
    return ', '.join(resultList)