# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2015 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################


import re

# import string
from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import *



def getPref(dict, key, default):
    return dict.get(unicode(key).lower(), default)

def getPrefDate(dict, key, default):
    return forceDate(dict.get(unicode(key).lower(), default))

def getPrefRef(dict, key, default):
    return forceRef(dict.get(unicode(key).lower(), default))

def getPrefBool(dict, key, default):
    return forceBool(dict.get(unicode(key).lower(), default))

def getPrefInt(dict, key, default):
    return forceInt(dict.get(unicode(key).lower(), default))

def getPrefString(dict, key, default):
    return forceString(dict.get(unicode(key).lower(), default))


def setPref(dict, key, value):
    dict[unicode(key).lower()] = value


def trim(s):
    return u' '.join(unicode(s).split())


def nameCase(s):
    r = u''
    up = True
    for c in s:
        if c.isalpha():
            if up:
                r += c.upper()
                up = False
            else:
                r += c.lower()
        else:
            up = True
            r += c
    return r


def isNameValid(name):
    return not re.search(r'''[0-9a-zA-Z`~!@#$%^&*_=+\\|{}[\];:'"<>?/]''', forceStringEx(name))


def addDots(s):
    if len(s) != 0 and s[-3:] != '...':
        return s + '...'
    else:
        return s


def addDotsBefore(s):
    if len(s) != 0 and s[:3] != '...':
        return '...' + s
    else:
        return s


def addDotsEx(s):
    if len(s) != 0:
        result = s
        if s[:3]  != '...': result = '...' + result
        if s[-3:] != '...': result = result + '...'
        return result
    else:
        return s


def toVariant(v):
    if v is None:
        return QVariant()
    elif type(v) == QVariant:
        return v
    return QVariant(v)

def variantEq(value1, value2):
    return (value1.isNull() and value2.isNull()) or (value1 == value2)


###################################################

def forceBool(val):
    if type(val) == QVariant:
        return val.toBool()
    return bool(val)


def forceDate(val):
    if type(val) == QVariant:
        return val.toDate()
    return val


def forceTime(val):
    if type(val) == QVariant:
        return val.toTime()
    return val


def forceDateTime(val):
    if type(val) == QVariant:
        return val.toDateTime()
    return val


def forceInt(val):
    if type(val) == QVariant:
        return val.toInt()[0]
    elif val == None:
        return 0
    return int(val)


def forceRef(val):
    if type(val) == QVariant:
        if val.isNull():
            val = None
        else:
            val = val.toInt()[0]
            if val == 0:
                val = None
    return val


def forceString(val):
    if type(val) == QVariant:
        if  val.type() == QVariant.Date:
            val = val.toDate()
        elif val.type() == QVariant.DateTime:
            val = val.toDateTime()
        else:
            val = val.toString()
    if type(val) == QDate:
        return unicode(val.toString('dd.MM.yyyy'))
    elif type(val) == QDateTime:
        return unicode(val.toString('dd.MM.yyyy hh:mm:ss'))
    elif val == None:
        return u''
    else:
        return unicode(val)


def forceStringEx(val):
    return trim(forceString(val))


def forceDouble(val):
    if type(val) == QVariant:
        return val.toDouble()[0]
    else:
        return float(val)


def formatBool(val):
    if forceBool(val):
        return u'да'
    else:
        return u'нет'


def pyDate(date):
    if date and date.isValid():
        return date.toPyDate()
    else:
        return None


###################################################

def formatDate(date):
    return forceString(forceDate(date))


def formatTime(time):
    return unicode(time.toString('H:mm'))


def formatNameInt(lastName, firstName, patrName):
    return trim(lastName+' '+firstName+' '+patrName)


def formatName(lastName, firstName, patrName):
    lastName  = nameCase(forceStringEx(lastName))
    firstName = nameCase(forceStringEx(firstName))
    patrName  = nameCase(forceStringEx(patrName))
    return formatNameInt(lastName, firstName, patrName)


def formatShortNameInt(lastName, firstName, patrName):
    return trim(lastName + ' ' +((firstName[:1]+'.') if firstName else '') + ((patrName[:1]+'.') if patrName else ''))


def formatShortName(lastName, firstName, patrName):
    lastName  = nameCase(forceStringEx(lastName))
    firstName = nameCase(forceStringEx(firstName))
    patrName  = nameCase(forceStringEx(patrName))
    return formatShortNameInt(lastName, firstName, patrName)


def formatSex(sex):
    sex = forceInt(sex)
    if sex == 1:
        return u'М'
    elif sex == 2:
        return u'Ж'
    else:
        return u''


def formatSNILS(SNILS):
    if SNILS:
        s = forceString(SNILS)+' '*14
        return s[0:3]+'-'+s[3:6]+'-'+s[6:9]+' '+s[9:11]
    else:
        return u''


def unformatSNILS(SNILS):
    return forceString(SNILS).replace('-', '').replace(' ', '')


def calcSNILSCheckCode(SNILS):
    result = 0
    for i in xrange(9):
        result += (9-i)*int(SNILS[i])
    result = result % 101
    if result == 100:
        result = 0
    return '%02.2d' % result


def checkSNILS(SNILS):
    raw = unformatSNILS(SNILS)
    if len(raw) == 11:
        return raw[:9]<='001001998' or raw[-2:] == calcSNILSCheckCode(raw)
    return False


def fixSNILS(SNILS):
    raw = unformatSNILS(SNILS)
    return (raw+'0'*11)[:9] + calcSNILSCheckCode(raw)


def agreeNumberAndWord(num, words):
    u"""
        Согласовать число и слово:
        num - число, слово = (один, два, много)
        agreeNumberAndWord(12, (u'год', u'года', u'лет'))
    """
    if num<0: num = -num
    if (num/10)%10 != 1:
        if num%10 == 1:
            return words[0]
        elif 1 < num%10 < 5:
            return words[1]
    return words[-1]


def formatNum(num, words):
    return u'%d %s' % (num, agreeNumberAndWord(num,words))


def formatNum1(num, words):
    if num == 1:
        return agreeNumberAndWord(num,words)
    else:
        return u'%d %s' % (num, agreeNumberAndWord(num,words))


def formatYears(years):
    return '%d %s' % (years, agreeNumberAndWord(years, (u'год', u'года', u'лет')))


def formatMonths(months):
    return '%d %s' % (months, agreeNumberAndWord(months, (u'месяц', u'месяца', u'месяцев')))


def formatWeeks(weeks):
    return '%d %s' % (weeks, agreeNumberAndWord(weeks, (u'неделя', u'недели', u'недель')))


def formatDays(days):
    return '%d %s' % (days, agreeNumberAndWord(days, (u'день', u'дня', u'дней')))


def formatYearsMonths(years, months):
    if years == 0:
        return formatMonths(months)
    elif months == 0:
        return formatYears(years)
    else:
        return formatYears(years) + ' ' + formatMonths(months)


def formatMonthsWeeks(months, weeks):
    if  months == 0:
        return formatWeeks(weeks)
    elif weeks == 0:
        return formatMonths(months)
    else:
        return formatMonths(months) + ' ' + formatWeeks(weeks)


def formatList(list):
    if len(list)>2:
        return u' и '.join([', '.join(list[:-1]), list[-1]])
    else:
        return u' и '.join(list)


def splitText(text, widths):
    result = []
    if not text:
        return result

    width = widths[0]

    lines = text.splitlines()
    count = 0

    for line in lines:
        p = 0
        l = len(line)
        while p<l:
            while line[p:p+1].isspace():
                p += 1
            s = p + width
            if s>=l:
                breakpos = s
            else:
                breakpos = line.rfind(' ', p, s+1)
            if breakpos<0:
                breakpos = max([line.rfind(sep, p, s) for sep in '-,.;:!?)]}\\|/'])
                if breakpos>= 0:
                  breakpos+=1
            if breakpos<0:
                breakpos = s
            result.append(line[p:breakpos])
            p = breakpos
            count += 1
            width = widths[count if count<len(widths) else -1]
    return result


def foldText(text, widths):
    return '\n'.join(splitText(text, widths))


def filter(alist):
    return [x for x in alist if x]


####################################################

def calcAgeInDays(birthDay, today):
    assert isinstance(birthDay, QDate)
    assert isinstance(today, QDate)
    return birthDay.daysTo(today)


def calcAgeInWeeks(birthDay, today):
    return calcAgeInDays(birthDay, today)/7


def calcAgeInMonths(birthDay, today):
    assert isinstance(birthDay, QDate)
    assert isinstance(today, QDate)

    bYear  = birthDay.year()
    bMonth = birthDay.month()
    bDay   = birthDay.day()

    tYear  = today.year()
    tMonth = today.month()
    tDay   = today.day()

    result = (tYear-bYear)*12+(tMonth-bMonth)
    if bDay > tDay:
        result -= 1
    return result


def calcAgeInYears(birthDay, today):
    assert isinstance(birthDay, QDate)
    assert isinstance(today, QDate)

    bYear  = birthDay.year()
    bMonth = birthDay.month()
    bDay   = birthDay.day()

    tYear  = today.year()
    tMonth = today.month()
    tDay   = today.day()

    result = tYear-bYear
    if bMonth>tMonth or (bMonth == tMonth and bDay > tDay):
        result -= 1
    return result


def calcAgeTuple(birthDay, today):
    if not today or today.isNull():
        today_ = QDate.currentDate()
    elif isinstance(today, QDateTime):
        today_ = today.date()
    else:
        today_ = today
    d = calcAgeInDays(birthDay, today_)
    if d>=0:
        return (d,
                d/7,
                calcAgeInMonths(birthDay, today_),
                calcAgeInYears(birthDay, today_)
               )
    else:
        return None


def calcAge(birthDay, today=None):
    bd = forceDate(birthDay)
    td = forceDate(today) if today else QDate.currentDate()

    ageTuple = calcAgeTuple(bd, td)
    if not ageTuple:
        return u'ещё не родился'
    (days, weeks, months, years) = ageTuple
    if years>7:
        return formatYears(years)
    elif years>1:
        return formatYearsMonths(years, months-12*years)
    elif months>1:
        return formatMonthsWeeks(months, birthDay.addMonths(months).daysTo(td)/7)
    else:
        return formatDays(days)


def firstWeekDay(date):
#    return date.addDays(-(date.dayOfWeek()-1))
    return date.addDays(1-date.dayOfWeek())


def lastWeekDay(date):
#    return firstWeekDay.addDays(6)
    return date.addDays(7-date.dayOfWeek())


def firstMonthDay(date):
    return QDate(date.year(), date.month(), 1)

def lastMonthDay(date):
    return firstMonthDay(date).addMonths(1).addDays(-1)


def lastQuarterDay(date):
    return firstQuarterDay(date).addMonths(3).addDays(-1)


def firstQuarterDay(date):
    month = ((date.month()-1)/3)*3+1
    return QDate(date.year(), month, 1)


def lastQuarterDay(date):
    return firstQuarterDay(date).addMonths(3).addDays(-1)


def lastQuarterDay(date):
    month = 6 if date.month() < 7 else 12
    return QDate(date.year(), month, 30 if month == 6 else 31)


def firstHalfYearDay(date):
    month = 1 if date.month() < 7 else 7
    return QDate(date.year(), month, 1)


def lastHalfYearDay(date):
    month = 6 if date.month() < 7 else 12
    return QDate(date.year(), month, 30 if month == 6 else 31)


def lastHalfYearDay(date):
    return QDate(date.year(), 12, 31)


def firstYearDay(date):
    return QDate(date.year(), 1, 1)


def lastYearDay(date):
    return QDate(date.year(), 12, 31)


def getPeriodLength(startDate, stopDate, countRedDays):
    # определение длительности периода в днях (включая или исключая выходные)
    # учитывается и начальная и конечная дата
    # необходимо доделать для работы с праздничными днями
    def determineMondayAndWorkDaysCount(date):
        doy = date.dayOfWeek()
        monday = date.addDays(-doy+Qt.Monday)
        workDaysCount = doy if doy<Qt.Saturday else Qt.Friday
        return monday, workDaysCount, doy in (Qt.Saturday, Qt.Sunday)
    if startDate>stopDate:
        return 0
    if countRedDays:
        return startDate.daysTo(stopDate)+1
    else:
        startMonday, startWDC, startIsRed = determineMondayAndWorkDaysCount(startDate)
        stopMonday, stopWDC, stopIsRed = determineMondayAndWorkDaysCount(stopDate)
        return startMonday.daysTo(stopMonday)*5//7+stopWDC-startWDC+(0 if startIsRed else 1)


####################################################


def setBits(val, mask, bits):
    return ( val & ~mask ) | ( bits & mask )


def checkBits(val, mask, bits):
    return ( val & mask ) == ( bits & mask )


####################################################

def sorry():
    widget = QtGui.qApp.focusWidget() or QtGui.qApp.activeWindow() or QtGui.qApp.mainWindow
    QtGui.QMessageBox.information(widget,
                                u'не реализовано',
                                u'к сожалению не реализовано :(',
                                QtGui.QMessageBox.Ok,
                                QtGui.QMessageBox.Ok
                                )

def oops():
    widget = QtGui.qApp.focusWidget() or QtGui.qApp.activeWindow() or QtGui.qApp.mainWindow
    QtGui.QMessageBox.information(widget,
                                u'ой',
                                u'косяк, однако! :\'(',
                                QtGui.QMessageBox.Ok,
                                QtGui.QMessageBox.Ok
                                )

def get_date(d):
    d=forceDate(d)
    if d and 1800<=d.year()<=2030:
        return d.toPyDate()
    else:
        return None
