# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2015 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import *

import database
from library.Utils import *
from Utils import *
from MKB import *


def parseModifier(modifier):
    m = forceStringEx(modifier)
    if not(m):
        return (0, m)
    elif m.startswith('-'):
        return (1,  u'')
    elif m.startswith('+'):
        return (2,  m[1:])
    else:
        return (3,  m)

def applyModifier(serviceCode, (modifierAction, modifierText)):
    if modifierAction == 0:
        return serviceCode
    elif modifierAction == 1:
        return ''
    elif modifierAction == 2:
        return modifierText
    elif modifierAction == 3:
        return modifierText + serviceCode[1:]
    else:
        raise CException(u'unknown service modifier (%s,%s)' % (modifierAction, modifierText))



def getExactServiceId(db, diagnosisServiceId, eventServiceId, personServiceId, visitTypeId, sceneId):
    u'''
        получить id услуги с учётом дополнительных факторов
    '''
    baseServiceId = diagnosisServiceId or eventServiceId or personServiceId
    baseServiceCode = forceString(db.translate('rbService', 'id', baseServiceId, 'code'))
    serviceCode = baseServiceCode
    visitTypeServiceModifier = parseModifier(db.translate('rbVisitType', 'id', visitTypeId, 'serviceModifier'))
    sceneServiceModifier     = parseModifier(db.translate('rbScene',     'id', sceneId,     'serviceModifier'))
    serviceCode = applyModifier(serviceCode, visitTypeServiceModifier)
    serviceCode = applyModifier(serviceCode, sceneServiceModifier)
    if serviceCode == baseServiceCode:
        return baseServiceId
    elif serviceCode:
        return forceRef(db.translate('rbService', 'code', serviceCode, 'id'))
    else:
        return None


def createVisits(db, eventId, eventTypeId=None):
    sceneId = forceRef(db.translate('rbScene', 'code',  '1', 'id'))
    if not eventTypeId:
        eventTypeId = forceRef(db.translate('Event', 'id',  eventId, 'eventType_id'))
    eventServiceId = forceRef(db.translate('EventType', 'id',  eventTypeId, 'service_id'))
    tableDiagnostic = db.table('Diagnostic')
    tableVisit      = db.table('Visit')
    tableEventTypeDiagnostic = db.table('EventType_Diagnostic')
    diagnosisTypes  = db.getIdList('rbDiagnosisType', where='code in (1, 2)') # magic
    diagnostics     = db.getRecordList(tableDiagnostic, '*', where=tableDiagnostic['event_id'].eq(eventId))
    db.deleteRecord(tableVisit, where=tableVisit['event_id'].eq(eventId))
    if sceneId and eventTypeId:
        for diagnostic in diagnostics:
            diagnosisTypeId = forceRef(diagnostic.value('diagnosisType_id'))
            if diagnosisTypeId in  diagnosisTypes:
                setDate = forceDate(diagnostic.value('setDate'))
                endDate = forceDate(diagnostic.value('endDate'))
                specialityId = forceRef(diagnostic.value('speciality_id'))
                personId     = forceRef(diagnostic.value('person_id'))
                financeId    = forceRef(db.translate('Person', 'id', personId, 'finance_id'))
                serviceId    = forceRef(db.translate('rbSpeciality', 'id', specialityId, 'service_id'))
#                visitTypeId  = forceRef(diagnostic.value('visitType_id'))
                visitTypeCond = [tableEventTypeDiagnostic['eventType_id'].eq(eventTypeId),
                                 tableEventTypeDiagnostic['speciality_id'].eq(specialityId)]
                visitTypeIdList  = db.getIdList(tableEventTypeDiagnostic, 'visitType_id', where=visitTypeCond)
                if visitTypeIdList:
                    visitTypeId = visitTypeIdList[0]
                else:
                    visitTypeId = None
                serviceId    = getExactServiceId(db, None, eventServiceId, serviceId, visitTypeId, sceneId)
                if not endDate.isNull() and (setDate.isNull() or setDate<=endDate) \
                   and personId     \
                   and financeId    \
                   and serviceId    \
                   and visitTypeId:
                    record = tableVisit.newRecord()
                    record.setValue('event_id',     toVariant(eventId))
                    record.setValue('scene_id',     toVariant(sceneId))
                    record.setValue('date',         toVariant(endDate))
                    record.setValue('visitType_id', toVariant(visitTypeId))
                    record.setValue('person_id',    toVariant(personId))
                    record.setValue('isPrimary',    toVariant(1))
                    record.setValue('finance_id',   toVariant(financeId))
                    record.setValue('service_id',   toVariant(serviceId))
                    record.setValue('payStatus',    toVariant(0))
                    db.insertRecord(tableVisit, record)