# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2015 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtCore, QtGui, QtSql
from PyQt4.QtCore import *

from Utils import toVariant

queryLog={}

class CException(Exception):

    def __init__(self, message):
        m = unicode(message)
        Exception.__init__(self, m)
        self._message = m

    def __str__(self):
        return self._message

class CDatabaseException(CException):
    """
       Root of all database exceptions
    """
    def __init__(self, message, sqlError=None):
#        if self.sqlError and sqlError.type() != QtSql.QSqlError.NoError:
        if sqlError:
            message = message + '\n' + sqlError.driverText() + '\n' + sqlError.databaseText()
        CException.__init__(self, message)
        self.sqlError = sqlError


#    def __str__(self):
#        return unicode(self._message)


class CField(object):
    def __init__(self, driver, tableName, field):
        self.driver    = driver
        self.tableName = unicode(tableName)
        if driver:
#            self.fieldName = unicode(driver.escapeIdentifier(field.name(), QtSql.QSqlDriver.FieldName))
            self.fieldName = unicode('`'+field.name()+'`')
        else:
            self.fieldName = field.name()
        self.field = field


    def name(self):
        return self.tableName+'.'+self.fieldName


    def alias(self, name=''):
        if name:
            return self.name()+' AS `'+unicode(name)+'`'
        else:
            return self.name()+' AS '+self.fieldName


    def toTable(self, tableName):
        return CField(self.driver, tableName, self.field)


    def signEx(self, sign, expr):
        return unicode(self.name()+sign+expr)


    def formatValue(self, val):
        if isinstance(val, CField):
            return val.name()
        else:
            tmpField = QtSql.QSqlField(self.field)
            tmpField.setValue(toVariant(val))
            return self.driver.formatValue(tmpField, False)


    def sign(self, sign, val):
        return self.signEx(sign,self.formatValue(val))


    def eq(self, val):
        if val is None:
            return self.isNull()
        else:
            return self.sign('=', val)


    def eqEx(self, val):
        if val is None:
            return self.isNull()
        else:
            return self.signEx('=', val)


    def lt(self, val):
        return self.sign('<', val)


    def le(self, val):
        return self.sign('<=', val)


    def gt(self, val):
        return self.sign('>', val)


    def ge(self, val):
        return self.sign('>=', val)


    def ne(self, val):
        if val is None:
            return self.isNotNull()
        else:
            return self.sign('!=', val)


    def isNull(self):
        return unicode(self.name()+' IS NULL')


    def isNotNull(self):
        return unicode(self.name()+' IS NOT NULL')


    def inlist(self, list):
        if not list:
            return '0' # false
        else:
            tmpField = QtSql.QSqlField(self.field)
            decoratedList = []
            for val in list:
                tmpField.setValue(toVariant(val))
                decoratedList.append(unicode(self.driver.formatValue(tmpField, False)))
            return unicode(self.name()+' IN ('+(','.join(decoratedList))+')')


    def notInlist(self, list):
        if not list:
            return '0' # false
        else:
            tmpField = QtSql.QSqlField(self.field)
            decoratedList = []
            for val in list:
                tmpField.setValue(toVariant(val))
                decoratedList.append(unicode(self.driver.formatValue(tmpField, False)))
            return unicode(self.name()+' NOT IN ('+(','.join(decoratedList))+')')


    def like(self, val):
        tmp = val
        if tmp.endswith('...'):
            tmp = tmp[:-3]+'%'
        return self.sign(' LIKE ', tmp.replace('...','%'))


    def between(self, low, high):
        return u'(%s BETWEEN %s AND %s)' % (self.name(), self.formatValue(low), self.formatValue(high))



class CTable(object):
    def __init__(self, tableName, dataBase):
        self.fields = []
        self.fieldsDict = {}
        self.database = dataBase
        driver = dataBase.db.driver()
        self.tableName = unicode(tableName)
        self._idField = None
        self._idFieldName = None
        record = dataBase.record(self.tableName)
        for i in range(record.count()):
            qtfield = record.field(i)
            field = CField(driver, self.tableName, qtfield)
            self.fields.append(field)
            self.fieldsDict[str(qtfield.name())] = field


    def name(self, alias=''):
        if alias:
            return self.tableName+' AS '+unicode(alias)
        else:
            return self.tableName


    def alias(self, name):
        return CTableAlias(self, name)


    def setIdFieldName(self, name):
        self._idField = self.__getitem__(name)
        self._idFieldName = name


    def idField(self):
        if not self._idField:
            self.setIdFieldName('id')
##            raise CDatabaseException(CDatabase.errNoIdField % (self.tableName))
        return self._idField


    def idFieldName(self):
        if not self._idField:
            self.setIdFieldName('id')
##            raise CDatabaseException(CDatabase.errNoIdField % (self.tableName))
        return self._idFieldName


    def __getitem__(self, key):
        key = str(key)
        result = self.fieldsDict.get(key, None)
        if result:
            return result
        else:
            raise CDatabaseException(CDatabase.errFieldNotFound % (self.tableName, key) )


    def hasField(self, fieldName):
        return self.fieldsDict.has_key(fieldName)


    def newRecord(self, fields=None):
        record = QtSql.QSqlRecord()
        if fields:
            for field in self.fields:
                if field.field.name() in fields:
                    record.append( QtSql.QSqlField(field.field) )
        else:
            for field in self.fields:
                record.append( QtSql.QSqlField(field.field) )
        return record


    def beforeInsert(self, record):
        return


    def beforeUpdate(self, record):
        return


    def beforeDelete(self, record):
        return


    def join(self, table, onCond):
        return self.database.join(self, table, onCond)


    def leftJoin(self, table, onCond):
        return self.database.leftJoin(self, table, onCond)


class CTableAlias(object):
    def __init__(self, table, name):
        self.table = table
        self.aliasName = str(name)
        self.fieldsDict = {}
        self.database = table.database
        self._idField = None
        self._idFieldName = None


    def name(self):
        return self.table.name()+' AS '+self.aliasName


    def hasField(self, fieldName):
        return self.table.hasField(fieldName)


    def newRecord(self, fields=None):
        return self.table.newRecord(fields)


    def setIdFieldName(self, name):
        self._idField = self.__getitem__(name)
        self._idFieldName = name


    def idField(self):
        if not self._idField:
            self.setIdFieldName('id')
        return self._idField


    def idFieldName(self):
        if not self._idField:
            self.setIdFieldName('id')
        return self._idFieldName


    def __getitem__(self, key):
        key = str(key)
        if self.fieldsDict.has_key(key):
            return self.fieldsDict[str(key)]
        else:
            tableField = self.table.__getitem__(key).toTable(self.aliasName)
            self.fieldsDict[key] = tableField
            return tableField


    def beforeInsert(self, record):
        self.table.beforeInsert(record)


    def beforeUpdate(self, record):
        self.table.beforeUpdate(record)


    def beforeDelete(self, record):
        self.table.beforeDelete(record)
        return


    def join(self, table, onCond):
        return self.database.join(self, table, onCond)


    def leftJoin(self, table, onCond):
        return self.database.leftJoin(self, table, onCond)



class CDocumentTable(CTable):
    # fields in DocumentTables:
    dtfCreateDatetime = 'createDatetime'
    dtfCreateUserId   = 'createPerson_id'
    dtfModifyDatetime = 'modifyDatetime'
    dtfModifyUserId   = 'modifyPerson_id'
    dtfDeleted        = 'deleted'

    def __init__(self, tableName, dataBase):
        CTable.__init__(self, tableName, dataBase)


    def beforeInsert(self, record):
        now = toVariant(QDateTime.currentDateTime())
        userId = toVariant(None) #TODO: сделать поддержку user id
        if record.indexOf(CDocumentTable.dtfCreateDatetime)<0:
            record.append(QtSql.QSqlField(CDocumentTable.dtfCreateDatetime, QVariant.DateTime))
        if record.indexOf(CDocumentTable.dtfCreateUserId)<0:
            record.append(QtSql.QSqlField(CDocumentTable.dtfCreateUserId, QVariant.Int))
        if record.indexOf(CDocumentTable.dtfModifyDatetime)<0:
            record.append(QtSql.QSqlField(CDocumentTable.dtfModifyDatetime, QVariant.DateTime))
        if record.indexOf(CDocumentTable.dtfModifyUserId)<0:
            record.append(QtSql.QSqlField(CDocumentTable.dtfModifyUserId,  QVariant.Int))

        record.setValue(CDocumentTable.dtfCreateDatetime, now)
        record.setValue(CDocumentTable.dtfCreateUserId,   userId)
        record.setValue(CDocumentTable.dtfModifyDatetime, now)
        record.setValue(CDocumentTable.dtfModifyUserId,   userId)
        return

    def beforeUpdate(self, record):
        now = toVariant(QDateTime.currentDateTime())
        userId = toVariant(None) #TODO: сделать поддержку user id

        if record.isNull(CDocumentTable.dtfCreateDatetime):
            pos = record.indexOf(CDocumentTable.dtfCreateDatetime)
            record.remove(pos)
        if record.isNull(CDocumentTable.dtfCreateUserId):
            pos = record.indexOf(CDocumentTable.dtfCreateUserId)
            record.remove(pos)

        if record.indexOf(CDocumentTable.dtfModifyDatetime)<0:
            record.append(QtSql.QSqlField(CDocumentTable.dtfModifyDatetime, QVariant.DateTime))
        if record.indexOf(CDocumentTable.dtfModifyUserId)<0:
            record.append(QtSql.QSqlField(CDocumentTable.dtfModifyUserId, QVariant.Int))

        record.setValue(CDocumentTable.dtfModifyDatetime, now)
        record.setValue(CDocumentTable.dtfModifyUserId,   userId)
        return


#    def beforeDelete(self, record):
#        record.setValue(CDocumentTable.dtfModifyDatetime, toVariant(Now()))
#        record.setValue(CDocumentTable.dtfModifyUserId,   toVariant(???))
#        record.setValue(CDocumentTable.dtfModifyUserId,   toVariant(True))
#        return

class CJoin(object):
    def __init__(self, firstTable, secondTable, onCond, stmt='JOIN'):
        self.firstTable = firstTable
        self.secondTable = secondTable
        self.onCond = onCond
        self.stmt = stmt
        self.database = firstTable.database
        assert firstTable.database == secondTable.database

    def name(self):
        return u'%s %s %s ON %s ' % (self.firstTable.name(), self.stmt, self.secondTable.name(), self.onCond)


    def join(self, table, onCond):
        return self.database.join(self, table, onCond)


    def leftJoin(self, table, onCond):
        return self.database.leftJoin(self, table, onCond)


    def getMainTable(self):
        if isinstance(self.firstTable, CJoin):
            return self.firstTable.getMainTable()
        else:
            return self.firstTable

    def idField(self):
        return self.firstTable.idField()

################################################################
#

DocumentTables = []

def registerDocumentTable(tableName):
    DocumentTables.append(tableName)



class CDatabase(object):
    errUndefinedDriver         = u'Драйвер базы данных "%s" не зарегистрирован'
    errCannotConnectToDatabase = u'Невозможно подключиться к базе данных "%s"'
    errDatabaseIsNotOpen       = u'База данных не открыта'
    errTransactionError        = u'Ошибка открытия тразнакции'
    errCommitError             = u'Ошибка закрытия тразнакции'
    errRollbackError           = u'Ошибка отмены тразнакции'
    errTableNotFound           = u'Таблица "%s" не найдена'
    errFieldNotFound           = u'В таблице %s не найдено поле "%s"'
    errQueryError              = u'Ошибка выполнения запроса\n%s'
    errNoIdField               = u'В таблице %s не определен первичный ключ'


    def __init__(self):
        self.db = None
        self.tables = {}


    def createConnection(self, driverName,  connectionName):
        if connectionName:
            db = QtSql.QSqlDatabase.addDatabase(driverName, connectionName)
        else:
            db = QtSql.QSqlDatabase.addDatabase(driverName)
        if not db.isValid():
            raise CDatabaseException(CDatabase.errUndefinedDriver % driverName)
        return db


    def connect(self, db, serverName, serverPort, databaseName, userName, password):
        db.setHostName(serverName)
        if serverPort != 0:
            db.setPort(serverPort)
        db.setDatabaseName(databaseName)
        db.setUserName(userName)
        db.setPassword(password)
        if not db.open():
            raise CDatabaseException(
                CDatabase.errCannotConnectToDatabase % databaseName, db.lastError())
        self.db = db


#    def __del__(self):
#        self.close()


    def close(self):
        self.tables = {}
        if self.db:
            self.db.close()
            self.db = None
            self.tables = None


    def checkdb(self):
        if not self.db or not self.db.isValid() or not self.db.isOpen():
            raise CDatabaseException(CDatabase.errDatabaseIsNotOpen)

    def forceTable(self, table):
        if isinstance(table, (CTable, CTableAlias, CJoin)):
            return table
        elif isinstance(table, basestring):
            return self.table(table)
        else:
            raise AssertionError, u'Недопустимый тип'


    def formatDate(self, val):
        return '\''+val.toString(Qt.ISODate)+'\''


    def joinAnd(self, list):
        if list:
            return '('+(') AND ('.join(list))+')'
        else:
            return ''


    def joinOr(self, list):
        if list:
            return '(('+(') OR ('.join(list))+'))'
        else:
            return ''


    def prepareFieldList(self, fields):
        if isinstance(fields, (list,  tuple)):
            return  ', '.join([field.name() if isinstance(field, CField) else field for field in fields])
        return fields


    def prepareWhere(self, cond):
        if isinstance(cond, (list,  tuple)):
            cond = self.joinAnd(cond)
        if cond:
            return ' WHERE '+ cond
        else:
            return ''


    def prepareOrder(self, order):
        if isinstance(order, (list,  tuple)):
            order = ', '.join(order)
        if order:
            return ' ORDER BY '+ order
        else:
            return ''


    def prepareLimit(self, limit):
        assert False


    def selectStmt(self, table, fields='*', where='', order='', limit=None):
        self.checkdb()
        table = self.forceTable(table)
        return ' '.join([
            'SELECT', self.prepareFieldList(fields),
            'FROM', table.name(),
            self.prepareWhere(where),
            self.prepareOrder(order)])

#        return 'SELECT %s FROM %s %s %s %s' % (self.prepareFieldList(fields),
#                                          table.name(),
#                                          self.prepareWhere(where),
#                                          self.prepareOrder(order),
#                                          self.prepareLimit(limit))


    def existsStmt(self, table, where):
        self.checkdb()
        table = self.forceTable(table)
        if isinstance(table, CJoin):
            mainTable = table.getMainTable()
        else:
            mainTable = table
        if mainTable.hasField('id'):
            field = mainTable['id'].name()
        else:
            field = '*'
        return 'EXISTS (%s)' % self.selectStmt(table, field, where)


    def transaction(self):
        self.checkdb()
        if not self.db.transaction():
            raise CDatabaseException(CDatabase.errTransactionError, self.db.lastError())


    def commit(self):
        self.checkdb()
        if not self.db.commit():
            raise CDatabaseException(CDatabase.errCommitError, self.db.lastError())


    def rollback(self):
        self.checkdb()
        if not self.db.rollback():
            raise CDatabaseException(CDatabase.errRollbackError, self.db.lastError())


    def table(self, tableName):
        self.checkdb()
        if self.tables.has_key(tableName):
            return self.tables[tableName]
        else:
            if tableName in DocumentTables:
                table = CDocumentTable(tableName, self)
            else:
                table = CTable(tableName, self)
            self.tables[tableName] = table
            return table


    def join(self, firstTable, secondTable, onCond, stmt='JOIN'):
        self.checkdb()
        if isinstance(onCond, (list,  tuple)):
            onCond = self.joinAnd(onCond)
        return CJoin(self.forceTable(firstTable), self.forceTable(secondTable), onCond, stmt)


    def leftJoin(self, firstTable, secondTable, onCond):
        return self.join(firstTable, secondTable, onCond, 'LEFT JOIN')


    def record(self, tableName):
        parts = tableName.split('.',1)
        if len(parts) <= 1:
            res = self.db.record(tableName)
        else:
            databaseName  = parts[0]
            try:
                tmpdb = self.db.cloneDatabase(self.db, databaseName)
                tmpdb.setDatabaseName(databaseName)
                if not tmpdb.open():
                    raise CDatabaseException(
                        CDatabase.errCannotConnectToDatabase % databaseName, self.db.lastError())
                res = tmpdb.record(parts[1])
                tmpdb.close()
            finally:
                tmpdb = None
                QtSql.QSqlDatabase.removeDatabase(databaseName)
        if res.isEmpty():
            raise CDatabaseException(CDatabase.errTableNotFound % tableName)
        return res


#    def query(self, stmt):
##        global queryLog
##        queryInfo = queryLog.get(stmt, None)
##        if queryInfo is None:
##            queryInfo = [0, True]
##            queryLog[stmt] = queryInfo
##        queryInfo[0] = queryInfo[0] + 1
##        if queryInfo[0]>1 and queryInfo[1]:
##            pass
#
#        self.checkdb()
#        result = QtSql.QSqlQuery(self.db)
#        result.setForwardOnly(True)
##        print stmt #test
##        dt=QTime()
##        dt.start()
#        if not result.exec_(stmt):
#            raise CDatabaseException(CDatabase.errQueryError % stmt, result.lastError())
##        if dt.elapsed()>100 and self.db.driverName()=='QMYSQL':
##            print stmt
#        return result
#

    def query(self, stmt):
        self.checkdb()
        result = QtSql.QSqlQuery(self.db)
        result.setForwardOnly(True)
        if not result.exec_(stmt):
            self.onError(stmt, result.lastError())
        return result


    def onError(self, stmt, sqlError):
        raise CDatabaseException(CDatabase.errQueryError % stmt, sqlError)


    def getRecordEx(self, table, cols, where, order=''):
        stmt = self.selectStmt(table, cols, where, order, 1)
        query = self.query(stmt)
        if query.first():
            record=query.record()
#            del query
            return record
        else:
            return None


    def getRecord(self, table, cols, id):
        self.checkdb()
        table = self.forceTable(table)
        idCol = table.idField()
        return self.getRecordEx(table, cols, idCol.eq(id))


    def updateRecord(self, table, record):
        self.checkdb()
        table = self.forceTable(table)
        table.beforeUpdate(record)
        driver = self.db.driver()
        fieldsCount = record.count()
        idFieldName = table.idFieldName()
        values = []
        cond   = ''
        id     = None
        for i in range(fieldsCount):
#            pair = driver.escapeIdentifier(record.fieldName(i), QtSql.QSqlDriver.FieldName) + '=' + driver.formatValue(record.field(i), False)
            pair = '`'+record.fieldName(i) + '`=' + driver.formatValue(record.field(i), False)
            if record.fieldName(i) == idFieldName:
                cond = pair
                id = record.value(i).toInt()[0]
            else:
                values.append(unicode(pair))
        stmt = 'UPDATE ' + table.name() + ' SET ' + (', '.join(values)) + ' WHERE ' + cond
###        print unicode(stmt).encode('cp866')
        self.query(stmt)
        return id


    def insertRecord(self, table, record):
        self.checkdb()
        table = self.forceTable(table)
        table.beforeInsert(record)
        driver = self.db.driver()
        fieldsCount = record.count()
        fields = []
        values = []
        for i in range(fieldsCount):
            if not record.value(i).isNull():
#                fields.append(unicode(driver.escapeIdentifier(record.fieldName(i), QtSql.QSqlDriver.FieldName)))
                fields.append(unicode('`'+record.fieldName(i)+'`'))
                values.append(unicode(driver.formatValue(record.field(i), False)))
        stmt = ('INSERT INTO ' +  table.name() +
                '(' + (', '.join(fields)) + ') '+
                'VALUES (' + (', '.join(values)) + ')')
###        print unicode(stmt).encode('cp866')
        id = self.query(stmt).lastInsertId().toInt()[0]
        idFieldName = table.idFieldName()
        record.setValue(idFieldName, QVariant(id))
        return id


    def insertOrUpdate(self, table, record):
        table = self.forceTable(table)
        idFieldName = table.idFieldName()
        if record.isNull(idFieldName):
            return self.insertRecord(table, record)
        else:
            return self.updateRecord(table, record)


    def deleteRecord(self, table, where):
        self.checkdb()
        table = self.forceTable(table)
#        table.beforeUpdate(record)
        stmt = 'DELETE FROM  '+table.name()+self.prepareWhere(where)
        self.query(stmt)


    def getCount(self, table, countCol='*', where=''):
        stmt = self.selectStmt(table, 'COUNT(%s)'%countCol, where)
        query = self.query(stmt)
        if query.first():
            return query.value(0).toInt()[0]
        else:
            return 0


    def getIdList(self, table, idCol='id', where='', order='', limit=None):
        stmt = self.selectStmt(table, idCol, where, order, limit)
        query = self.query(stmt)
        result = []
        while query.next():
            result.append(query.value(0).toInt()[0])
        return result



#    def getIdList(self, table, idCol='id', where='', order=''):
#        stmt = self.selectStmt(table, idCol, where,  order)
#        query = self.query(stmt)
#        idColIdx = query.record().indexOf(idCol)
#        assert idColIdx>=0
#        result = []
#        while query.next():
#            result.append(query.value(idColIdx).toInt()[0])
#        return result


    def getRecordList(self, table, cols='*', where='', order=''):
        stmt = self.selectStmt(table, cols, where,  order)
        res = []
        query = self.query(stmt)
        while query.next():
            res.append(query.record())
        return res


    def translate(self, table, keyCol, keyVal, valCol):
        if keyCol == 'id' and keyVal is None:
            return None
        self.checkdb()
        table = self.forceTable(table)
        if not isinstance(keyCol, CField):
            keyCol = table[keyCol]
        record = self.getRecordEx(table, valCol, keyCol.eq(keyVal))
        if record:
            return record.value(0)
        else:
            return None


    def copyDepended(self, table, masterKeyCol, currentId, newId):
        self.checkdb()
        table = self.forceTable(table)
        if not isinstance(masterKeyCol, CField):
            masterKeyCol = table[masterKeyCol]
        masterKeyColName = masterKeyCol.field.name()
        result = []
        stmt = self.selectStmt(table, '*', masterKeyCol.eq(currentId), 'id')
        qquery = self.query(stmt)
        while qquery.next():
            record = qquery.record()
            record.setNull('id')
            record.setValue(masterKeyColName, toVariant(newId))
            result.append( self.insertRecord(table, record) )
        return result


    def getDescendants(self, table, groupCol, id):
        self.checkdb()
        table = self.forceTable(table)
        group = table[groupCol]

        result = set([id])
        parents = [id]

        while parents:
            children = set(self.getIdList(table, where=group.inlist(parents)))
            newChildren = children-result
            result |= newChildren
            parents = newChildren
        return list(result)


class CMySqlDatabase(CDatabase):
    limit1 = 'LIMIT 0, %d'
    limit2 = 'LIMIT %d, %d'
    CR_SERVER_GONE_ERROR = 2006
    CR_SERVER_LOST = 2013

    def __init__(self, serverName, serverPort, databaseName, userName, password, connectionName=None):
        CDatabase.__init__(self)
        db = self.createConnection('QMYSQL', connectionName)
        self.connect(db, serverName, serverPort, databaseName, userName, password)
        self.query(u'SET SQL_AUTO_IS_NULL=0;')

    def onError(self, stmt, sqlError=None):
        if sqlError and sqlError.number() in (CMySqlDatabase.CR_SERVER_GONE_ERROR, CMySqlDatabase.CR_SERVER_LOST):
            app = QtGui.qApp
            QtGui.QMessageBox.critical(None,
                u'Критическая ошибка',
                u'Потеряна связь с сервером.\nНеобходимо перезапустить приложение')
            try:
                CDatabase.onError(self, stmt, sqlError)
            finally:
                #app.doneTrace()
                self.db = None
        else:
            CDatabase.onError(self, stmt, sqlError)



    def prepareLimit(self, limit):
        if isinstance(limit, (list,  tuple)):
            assert len(limit) == 2
            return self.limit2 % limit
        elif isinstance(limit, int):
            return self.limit1 % limit
        else:
            return ''


    def selectStmt(self, table, fields='*', where='', order='', limit=None):
        self.checkdb()
        table = self.forceTable(table)
        return ' '.join([
            'SELECT', self.prepareFieldList(fields),
            'FROM', table.name(),
            self.prepareWhere(where),
            self.prepareOrder(order),
            self.prepareLimit(limit)])




class CRecordCache(object):
    def __init__(self, capacity=200):
        self.map = {}
        self.queue = []
        self.capacity = capacity


    def invalidate(self):
        self.map = {}
        self.queue = []


    def get(self, key):
        if self.map.has_key(key):
            return self.map[key]
        return None


    def put(self, key, record):
        if self.map.has_key(key):
            self.map[key] = record
        else:
            if len(self.queue) >= self.capacity:
                top = self.queue[0]
                del(self.queue[0])
                del(self.map[top])
            self.queue.append(key)
            self.map[key] = record


class CTableRecordCache(CRecordCache):
    def __init__(self, database, table, cols='*', capacity=200):
        CRecordCache.__init__(self, capacity)
        self.database = database
        self.table = table
        self.cols = cols if not isinstance(cols, (list,  tuple)) else ', '.join(cols)


    def get(self, id):
        if type(id) == QVariant:
            id = id.toInt()[0]
        res = CRecordCache.get(self, id)
        if not res:
#            try:
            res = self.database.getRecord(self.table, self.cols, id)
#            except:
#                res = self.database.getRecord(table)
            self.put(id, res)
        return res

# # # # #

def addCondLike(cond, field, val):
    if val.strip(' .'):
        if val.find('...') != -1:
            cond.append(field.like(val.strip()))
        else:
            cond.append(field.eq(val.strip()))

# # # # #

def addCondEq(cond, field, val):
    if val:
        cond.append(field.eq(val))

def addDateInRange(cond, field, begDate, endDate):
    if begDate and not begDate.isNull():
        cond.append(field.ge(begDate))
    if endDate and not endDate.isNull():
        cond.append(field.lt(endDate.addDays(1)))