select 'Импорт данных о пациентах (stattal)...' as ' ';

drop function if exists getStreetCode;

DELIMITER//

create function getStreetCode(region varchar(10), name varchar(10))
returns varchar(17)
COMMENT 'Возвращает код улицы по ее имени в стат. талоне и имени региона'
begin
	declare name_infis, name_socr varchar(10);

	if position('.' in name) != 0 then
		set name_infis = substring(name, 1, position('.' in name)-1);
		set name_socr = substring(name, position('.' in name)+1);
	else
		set name_infis = name;
		set name_socr = '';
	end if;
	return (select kladr.STREET.CODE
		from kladr.STREET
		left join kladr.SOCRBASE on kladr.STREET.SOCR = kladr.SOCRBASE.SCNAME
		-- проверяем все районы, через  кот. проходит улица, независимо от реального № дома!:
		left join kladr.DOMA on kladr.DOMA.CODE like '78%' -- для ускорения процесса берём питерские дома
					and left(kladr.STREET.CODE, 15) = left(kladr.DOMA.CODE, 15), 
		kladr.OKATO 
		where kladr.OKATO.infis = region
		and kladr.STREET.infis = name_infis -- совпадает имя улицы
		and kladr.SOCRBASE.infisCode = name_socr -- совпадает сокращение
		and kladr.OKATO.P1 = if(kladr.STREET.OCATD != '', substring(kladr.STREET.OCATD, 3, 3),
								substring(kladr.DOMA.OCATD, 3, 3)) -- совпадает район
		and kladr.OKATO.P2 = if(kladr.OKATO.P2 = '', '', if(kladr.STREET.OCATD != '', 
								substring(kladr.STREET.OCATD, 6, 3),
								substring(kladr.DOMA.OCATD, 6, 3))
								) -- если указан пригород, он тоже должен совпадать
		limit 1);
end//


DELIMITER;


select 'Определение пациентов...' as ' ';

truncate table tmpClient;
insert into tmpClient
select distinct
	0 as id,
	Pin,
	lastName,
	firstName,
	patrName,
	sex,
	birthDate,
	howOld,
	NULL as `category`,
	NULL as `work`,
	NULL as territory,
	NULL as region,
	NULL as street,
	NULL as house,
	NULL as korpus,
	NULL as flat,
	NULL as documentType,
	NULL as documentSerial,
	NULL as documentNumber,
	NULL as SMO,
	NULL as policySerial,
	NULL as policyNumber,
	NULL as streetCode,
	NULL as Client_id,
	0 as Client_type,
	NULL as Address_id,
	0 as Address_type,
	NULL as AddressHouse_id,
	0 as AddressHouse_type
from tmpStattal;

-- внутренние соответствия для пациентов устанавливаются:
-- 1. По Pin
-- 2. По ФИО и дате рождения.
-- 3. Если дата рождения пуста, по ФИО и различию возрастов не более, чем на 1.
update tmpStattal, tmpClient
set tmpStattal.tmpClient_id = tmpClient.id
where tmpStattal.Pin = tmpClient.Pin;

update tmpStattal, tmpClient
set tmpStattal.tmpClient_id = tmpClient.id
where tmpStattal.lastName = tmpClient.lastName
and tmpStattal.firstName = tmpClient.firstName
and tmpStattal.patrName = tmpClient.patrName
and tmpStattal.birthDate = tmpClient.birthDate;

update tmpStattal, tmpClient
set tmpStattal.tmpClient_id = tmpClient.id
where tmpStattal.birthDate is null
and tmpClient.birthDate is null
and tmpStattal.lastName = tmpClient.lastName
and tmpStattal.firstName = tmpClient.firstName
and tmpStattal.patrName = tmpClient.patrName
and (tmpStattal.howOld - tmpClient.howOld) in (-1, 0, 1);

-- устанавливаем дубли (по тем же 3 показателям)
update tmpClient A,
tmpStattal
left join tmpClient B on tmpStattal.tmpClient_id = B.id
set tmpStattal.tmpClient_id = A.id
WHERE A.Pin = B.Pin
AND A.id != B.id
AND A.id = (select max(`id`) -- в последнем варианте, скорее всего, меньше ошибок
		from tmpClient C
		where A.Pin = C.Pin);

update tmpClient A,
tmpStattal
left join tmpClient B on tmpStattal.tmpClient_id = B.id
set tmpStattal.tmpClient_id = A.id
WHERE A.lastName = B.lastName
and A.firstName = B.firstName
and A.patrName = B.patrName
and A.birthDate = B.birthDate
AND A.id != B.id
AND A.id = (select max(`id`)
		from tmpClient C
		where A.lastName = C.lastName
		and A.firstName = C.firstName
		and A.patrName = C.patrName
		and A.birthDate = C.birthDate);

update tmpClient A,
tmpStattal
left join tmpClient B on tmpStattal.tmpClient_id = B.id
set tmpStattal.tmpClient_id = A.id
WHERE A.birthDate is null
and B.birthDate is null
and A.lastName = B.lastName
and A.firstName = B.firstName
and A.patrName = B.patrName
and (A.howOld - B.howOld) in (-1, 0, 1)
AND A.id != B.id
AND A.id = (select max(`id`)
		from tmpClient C
		where C.birthDate is null
		and A.lastName = C.lastName
		and A.firstName = C.firstName
		and A.patrName = C.patrName
		and (A.howOld - C.howOld) in (-1, 0, 1));

-- теперь дубли тупо удаляем:
delete FROM `tmpClient` 
WHERE NOT EXISTS (
	SELECT * 
	FROM tmpStattal
	WHERE tmpStattal.tmpClient_id = tmpClient.id
);

-- докачиваем в таблицу клиентов остальную информацию:
update tmpClient
left join tmpStattal on tmpStattal.tmpClient_id = tmpClient.id
set tmpClient.`category` = tmpStattal.`category`,
	tmpClient.`work` = tmpStattal.`work`,
	tmpClient.territory = tmpStattal.territory,
	tmpClient.region = tmpStattal.region,
	tmpClient.street = tmpStattal.street,
	tmpClient.house = tmpStattal.house,
	tmpClient.korpus = tmpStattal.korpus,
	tmpClient.flat = tmpStattal.flat,
	tmpClient.documentType = tmpStattal.documentType,
	tmpClient.documentSerial = tmpStattal.documentSerial,
	tmpClient.documentNumber = tmpStattal.documentNumber,
	tmpClient.SMO = tmpStattal.SMO,
	tmpClient.policySerial = tmpStattal.policySerial,
	tmpClient.policyNumber = tmpStattal.policyNumber;

-- вот теперь определяем коды улиц:
update tmpClient
set streetCode = getStreetCode(region, street)
where territory = 'г';

select concat('Для ', convert(count(*) using utf8), ' пациентов улица указана неправильно или не соответствует региону.')
from tmpClient
where streetCode is null
and territory = 'г';



select 'Установка соответствий для адресов...' as ' ';
-- сперва для AddressHouse:
update tmpClient, AddressHouse
set tmpClient.AddressHouse_id = AddressHouse.id,
tmpClient.AddressHouse_type = 2
where AddressHouse.KLADRCode = '7800000000000' and tmpClient.territory = 'г'
and AddressHouse.KLADRStreetCode = tmpClient.streetCode
and AddressHouse.`number` = tmpClient.house
and ((AddressHouse.corpus = '' and tmpClient.korpus is null)
	or AddressHouse.corpus = tmpClient.korpus)
and tmpClient.AddressHouse_type = 0;

-- теперь для Address:
update tmpClient, Address
set tmpClient.Address_id = Address.id,
tmpClient.Address_type = 2
where tmpClient.AddressHouse_id = Address.house_id and tmpClient.AddressHouse_type = 2
and tmpClient.flat = Address.flat
and tmpClient.Address_type = 0;




select 'Установка соответствий для пациентов...' as ' ';
select 'by birth date...' as ' ';
update tmpClient, Client
set tmpClient.Client_id = Client.id,
    tmpClient.Client_type = 2
where tmpClient.birthDate = Client.birthDate
    and tmpClient.sex != Client.sex -- в источнике мужской и женский пол нумеруются наоборот!
    and strcmp(tmpClient.lastName, Client.lastName) = 0
    and strcmp(tmpClient.firstName, Client.firstName) = 0
    and strcmp(tmpClient.patrName, Client.patrName) = 0
AND tmpClient.Client_type = 0;

--select 'by how old...' as ' ';
update tmpClient
left join tmpStattal on tmpStattal.tmpClient_id = tmpClient.id,
Client
set tmpClient.Client_id = Client.id,
    tmpClient.Client_type = 2
where tmpClient.birthDate is NULL
    AND tmpClient.Client_type = 0
    and tmpClient.sex != Client.sex -- в источнике мужской и женский пол нумеруются наоборот!
    and Client.lastName = concat(upper(left(tmpClient.lastName, 1)), lower(substr(tmpClient.lastName, 2)))
    and strcmp(tmpClient.firstName, Client.firstName) = 0
    and strcmp(tmpClient.patrName, Client.patrName) = 0
    and date_add(Client.birthDate, interval tmpClient.howOld year) <= tmpStattal.endDate
    and date_add(Client.birthDate, interval (tmpClient.howOld +1) year) > tmpStattal.endDate;

-- по документу:
--select 'by document...' as ' ';
update tmpClient
left join tmpDoctype on tmpClient.documentType = tmpDoctype.svDocumentType
left join tmpStattal on tmpStattal.tmpClient_id = tmpClient.id,
Client
LEFT JOIN ClientDocument ON ClientDocument.client_id = Client.id
set tmpClient.Client_id = Client.id,
    tmpClient.Client_type = 2
WHERE tmpDoctype.rbDocumentType = ClientDocument.documentType_id
AND tmpClient.documentSerial = ClientDocument.serial
AND tmpClient.documentNumber = ClientDocument.number
and date_add(Client.birthDate, interval tmpClient.howOld year) <= tmpStattal.endDate
and date_add(Client.birthDate, interval (tmpClient.howOld +1) year) > tmpStattal.endDate
AND tmpClient.Client_type = 0;

-- по полису:
--select 'by policy...' as ' ';
update tmpClient, Client
LEFT JOIN ClientPolicy ON ClientPolicy.client_id = Client.id
LEFT JOIN Organisation ON ClientPolicy.insurer_id = Organisation.id
set tmpClient.Client_id = Client.id,
    tmpClient.Client_type = 2
WHERE tmpClient.SMO = Organisation.infisCode
AND tmpClient.policySerial = ClientPolicy.serial
AND tmpClient.policyNumber = ClientPolicy.number
AND tmpClient.Client_type = 0;

--количество не найденных клиентов - в протокол.
select concat('Не найдено ', convert(count(*) using utf8), ' пациентов.')
from tmpClient
where tmpClient.Client_type = 0; 