update tmpNetrsop
set TempInvalid_Period_type = 0
where TempInvalid_Period_type is NULL;
update tmpTempInvalid
set TempInvalid_type = 0
where TempInvalid_type is NULL;

select 'Определение документов нетрудоспособности и исправление ошибок...' as ' ';

drop function if exists getOpenPeriod;
drop function if exists getBegDate;

DELIMITER//

--create function periodIntersects(openDate1 date, closeDate1 date, openDate2 date, closeDate2 date)
--returns tinyint(1)
--DETERMINISTIC
--COMMENT 'Проверяет периоды на пересечение, причём направленно: предполагаем, что openDate1 <= openDate2'
--    return closeDate1 > openDate2;

--create function periodTouches(openDate1 date, closeDate1 date, openDate2 date, closeDate2 date)
--returns tinyint(1)
--DETERMINISTIC
--COMMENT 'Проверяет периоды на касание, причём направленно: предполагаем, что openDate1 <= openDate2'
--	return closeDate1 = openDate2;

create function getOpenPeriod(key1 bigint(20), Client_id int(11), openDate1 date, closeDate1 date)
returns bigint(20)
COMMENT 'Возвращает ключ предполагаемого начала периода с учётом пересечений'
begin
	declare key2 bigint(20);
	declare	openDate2, closeDate2 date;

	-- ищем лексикографически меньший по (openDate, key) пересекающийся интервал
	set key2 = key1, openDate2 = openDate1, closeDate2 = closeDate1;
    	repeat
		set key1 = key2, openDate1 = openDate2, closeDate1 = closeDate2;
		select `key`, openDate, closeDate into key2, openDate2, closeDate2
			from tmpNetrsop
			where tmpNetrsop.Client_id = Client_id
			and openDate < openDate1
			and closeDate > openDate1 -- периоды пересекаются
			limit 1;
	until key2 = key1 or key2 is null
	end repeat;
	select min(`key`) into `key2`
		from tmpNetrsop
		where tmpNetrsop.Client_id = Client_id
		and openDate = openDate1
		and closeDate > openDate1; -- периоды пересекаются
	return ifnull(key2, key1);
end//


create function getBegDate(key1 bigint(20), Client_id int(11), openDate1 date, closeDate1 date)
returns bigint(20)
COMMENT 'Возвращает ключ начала больняка'
begin
	declare key2, key3 bigint(20);
	declare	openDate2, closeDate2, openDate3, closeDate3 date;
	declare documentType1 tinyint(1);
	declare documentSerial varchar(8);
	declare documentNumber varchar(16);

	select tmpNetrsop.documentSerial, tmpNetrsop.documentNumber
	into documentSerial, documentNumber
	from tmpNetrsop
	where `key` = key1;
	-- сперва прыгаем на начало периода
	set key2 = getOpenPeriod(key1, Client_id, openDate1, closeDate1);
	select openDate, closeDate, documentType into openDate2, closeDate2, documentType1
	from tmpNetrsop
	where `key` = key2;
	-- теперь проверяем, не касается ли он другого периода; если касается, действуем рекурсивно
	-- опять же, движемся назад лексикографически по (openDate, key)
	set key3 = key2, openDate3 = openDate2, closeDate3 = closeDate2;
	repeat
		set key1 = key3, openDate1 = openDate3, closeDate1 = closeDate3;
		select tmpNetrsop.`key`, openDate, closeDate into key2, openDate2, closeDate2
			from tmpNetrsop
			where tmpNetrsop.Client_id = Client_id
			and ( (tmpNetrsop.documentSerial = documentSerial or (tmpNetrsop.documentSerial is null and documentSerial is null))
				and (tmpNetrsop.documentNumber = documentNumber or (tmpNetrsop.documentNumber is null and documentNumber is null)) -- серия и номер совпадают
				or (tmpNetrsop.documentType = 3 and documentType1 = 1)-- или это внешний период, который продолжается внутренним
			 )
			and (closeDate + 1 = openDate1
			 or   (openDate < openDate1 and closeDate = openDate1)
			 or   (openDate = openDate1 and closeDate = openDate1 and `key` < key1)) -- периоды соприкасаются
			limit 1;
		set key3 = getOpenPeriod(key2, Client_id, openDate2, closeDate2);
		select openDate, closeDate into openDate3, closeDate3
		from tmpNetrsop
		where `key` = key3;
	until key3 = key1 or key3 is null
	end repeat;
	return key1;
end//

DELIMITER;
















select concat('Количество записей, для которых нет связанных стат. талонов: ', convert(count(*) using utf8))
from tmpNetrsop
where not exists(select * from tmpStattal
		where tmpNetrsop.`key` = tmpStattal.`key`);

-- Для повышения эффективности не создаем таблицу клиентов источника, а больняк указывает сразу на клиента приёмника:
update tmpNetrsop
left join tmpStattal on tmpNetrsop.`key` = tmpStattal.`key`
set tmpNetrsop.Client_id = tmpStattal.Client_id
where tmpStattal.Client_id is not null;

-- то, что есть заключительные диагнозы, а первых нет - это полный бред. Этот бред надо исправить:
update tmpNetrsop
set begMKB = endMKB
where begMKB is NULL;

-- если МКБ стат. талона не совп. с МКБ больняка, фикс. в протокол; МКБ брать из стат. талона
select 'Для следующих документов временной нетрудоспособности диагноз не совпадает с указанным в стат. талоне:' as ' ';
select concat('Ключ ', convert(tmpNetrsop.`key` using utf8)), tmpNetrsop.begMKB, tmpStattal.MKB from tmpNetrsop
left join tmpStattal on tmpNetrsop.`key` = tmpStattal.`key`
where tmpNetrsop.begMKB != tmpStattal.MKB;

select 'Исправляем диагнозы из документов нетрудоспособности на диагнозы из стат. талонов...' as ' ';
update tmpNetrsop
left join tmpStattal on tmpNetrsop.`key` = tmpStattal.`key`
set tmpNetrsop.begMKB = tmpStattal.MKB
where tmpNetrsop.begMKB != tmpStattal.MKB;

-- теперь устанавливаем уход:
update tmpNetrsop
left join Client on tmpNetrsop.Client_id = Client.id
set care = 1
where date_add(Client.birthDate, interval 18 year) > tmpNetrsop.closeDate; -- если чувак мелкий, больничный выдан не ему, а по уходу

-- определим правильное начало каждого периода (getOpenPeriod) и запишем его в openPeriodKey
update tmpNetrsop
set openPeriodKey = getOpenPeriod(`key`, Client_id, openDate, closeDate)
where Client_id is not null;

-- записать пересекающиеся периоды в протокол (в периоде в этом случае запис. 1 период):
select concat(convert(count(*) using utf8), ' периодов пересекаются с предыдущими периодами:')
from tmpNetrsop
where `key` != openPeriodKey;
select concat('Ключ: ', convert(a1.`key` using utf8),
 '[', convert(a1.openDate using utf8), ' ; ', convert(a1.closeDate using utf8), '], меняем дату начала на: ', convert(a2.openDate using utf8))
from tmpNetrsop a1, tmpNetrsop a2
where a1.`key` != a2.`key`
and a1.openPeriodKey = a2.key;

-- теперь для пересекающихся периодов установим максимально возможный конец периода: 
-- низя так делать, компутер ругается:
--update tmpNetrsop t1
--set closeDate = (select max(closeDate) from tmpNetrsop t2
--		where t2.openPeriodKey = t1.`key`);

-- а теперь таким же макаром определяем начало каждого больняка (getBegDate) и запишем его в begKey
update tmpNetrsop
set begKey = getBegDate(`key`, Client_id, openDate, closeDate)
where Client_id is not null;

-- проверяем, есть ли больничные, не имеющие продолжений. Такую гадость мы удаляем:
select 'Внешние больничные листы, не связанные с данным ЛПУ (не имеющие продолжения в данном ЛПУ): ' as ' ';
select concat('Ключ ', convert(TBeg.`key` using utf8), ', с ', convert(TBeg.openDate using utf8), ' по ', convert(TBeg.closeDate using utf8))
from tmpNetrsop TBeg
where documentType = 3
and Client_id is not null
and TBeg.`key` = TBeg.`openPeriodKey`
and not exists (select * from tmpNetrsop T
		where T.begKey = TBeg.`key`
		and T.openPeriodKey = T.`key` -- продолжения должны быть правильными
		and T.documentType != 3);
select 'Такие больничные листы игнорируются.' as ' ';
update tmpNetrsop
set Client_id = NULL
where documentType = 3;
update tmpNetrsop TBeg, tmpNetrsop T
set TBeg.Client_id = T.Client_id
where T.begKey = TBeg.`key`
and T.openPeriodKey = T.`key` -- продолжения должны быть правильными
and T.documentType != 3;
update tmpNetrsop TOpen, tmpNetrsop T
set T.Client_id = TOpen.Client_id -- периоды с неправильными ключами пока не зануляем, они нам ещё понадобятся
where T.openPeriodKey = TOpen.`key`
and T.documentType = 3;
 -- сначала зануляем всех клиентов, а потом хороших возвращаем.

















truncate table tmpTempInvalid;
insert into tmpTempInvalid
SELECT distinct	0 as id,
		Client_id,
		documentSerial,
		documentNumber,
		openDate as begDate, -- в качестве даты начала периода берем openDate
		NULL as endDate,		
		NULL as TempInvalid_id,
		0 as TempInvalid_type
FROM tmpNetrsop
WHERE Client_id is not null
and `key` = begKey; 

-- исправляем серию и номер для внешних периодов на последующие внутренние:
update tmpTempInvalid
left join tmpNetrsop Period1 on Period1.tmpTempInvalid_id = tmpTempInvalid.id
			and Period1.begKey = Period1.`key`
left join tmpNetrsop Period2 on Period2.begKey = Period1.`key`
			and Period2.`key` != Period1.`key`
			and Period2.openDate = Period1.closeDate + 1
			and Period2.openPeriodKey = Period2.`key`
set tmpTempInvalid.documentSerial = Period2.documentSerial,
tmpTempInvalid.documentNumber = Period2.documentNumber
where Period1.documentType = 3;

-- устанавливаем соответствия между tmpNetrsop и tmpTempInvalid
-- устанавливаем соответствие по клиенту и начальной дате (getBegDate(tmpNersop.openDate) = tmpTempInvalid.begDate)
update tmpNetrsop T
left join tmpNetrsop TBeg on T.begKey = TBeg.`key`,
tmpTempInvalid
set T.tmpTempInvalid_id = tmpTempInvalid.id
where T.Client_id = tmpTempInvalid.Client_id
and T.key = T.openPeriodKey -- берём только правильные периоды
and TBeg.openDate = tmpTempInvalid.begDate;

-- устанавливаем endDate для tmpTempInvalid:
update tmpNetrsop T, tmpTempInvalid
set tmpTempInvalid.endDate = (select max(closeDate)
				 from tmpNetrsop T1
				 where T1.begKey = T.key)
where T.tmpTempInvalid_id = tmpTempInvalid.id;














select 'Установление соответствий для документов нетрудоспособности (netrsop)...' as ' ';

-- Больняки грузим даже для незагруженных талонов - главное, чтобы клиент и диагноз были.
update tmpTempInvalid
left join tmpNetrsop on tmpNetrsop.tmpTempInvalid_id = tmpTempInvalid.id and tmpNetrsop.`key` = tmpNetrsop.begKey
left join tmpStattal on tmpNetrsop.`key` = tmpStattal.`key`
set tmpTempInvalid.TempInvalid_type = 5
where tmpTempInvalid.TempInvalid_type = 0
and tmpTempInvalid.endDate is not null
and tmpStattal.tmpDiagnosis_id is not null;

update tmpNetrsop
left join tmpStattal on tmpNetrsop.`key` = tmpStattal.`key`
set tmpNetrsop.TempInvalid_Period_type = 5
where tmpNetrsop.TempInvalid_Period_type = 0
and tmpNetrsop.Client_id is not null
and tmpNetrsop.`key` = tmpNetrsop.openPeriodKey -- только те, у которых период не пересекается с другими
and tmpStattal.tmpDiagnosis_id is not null;


-- устанавливаем соответствие между tmpTempInvalid и TempInvalid по клиенту и пересечению периодов (предполагаем, что загружаемые данные относятся к более ранним периодам):
update tmpTempInvalid, TempInvalid
set tmpTempInvalid.TempInvalid_id = TempInvalid.id,
tmpTempInvalid.TempInvalid_type = 2
where tmpTempInvalid.Client_id = TempInvalid.client_id
and tmpTempInvalid.endDate is not null
and tmpTempInvalid.begDate <= TempInvalid.begDate
and tmpTempInvalid.endDate + 1 >= TempInvalid.begDate;


-- устанавливаем соответствие между tmpNetrsop и TempInvalid_Period по объемлющему больняку и начальной дате:
update tmpNetrsop
left join tmpTempInvalid on tmpNetrsop.tmpTempInvalid_id = tmpTempInvalid.id, 
TempInvalid_Period
left join TempInvalid on TempInvalid_Period.master_id = TempInvalid.id
set tmpNetrsop.TempInvalid_Period_id = TempInvalid_Period.id,
tmpNetrsop.TempInvalid_Period_type = 2
where tmpTempInvalid.TempInvalid_id = TempInvalid.id
and tmpNetrsop.openDate = TempInvalid_Period.begDate;



drop function getOpenPeriod;
drop function getBegDate;