drop function if exists applyModifier;
drop function if exists getService_id;

DELIMITER//

create function applyModifier(service int(11), modifier varchar(16))
returns int(11)
DETERMINISTIC
begin
    declare service_code varchar(8);
    
    select code into service_code
    from rbService
    where id = service;
    if left(modifier, 1) = '-' then
	return null;
    end if;
    if left(modifier, 1) = '+' then
	set service_code = substr(modifier from 2);
    else
	set service_code = concat(modifier, substr(service_code from 2));
    end if; 
    select id into service
    from rbService
    where strcmp(code, service_code) = 0
    limit 1;
    return service;
end//

create function getService_id(stattalKey bigint(20), visitType int(11), scene int(11))
returns int(11)
READS SQL DATA
DETERMINISTIC
COMMENT 'return service id'
begin
    declare service int(11);
    declare modifier varchar(16);

    select MKB.service_id into service -- сперва пытаемся определить по диагнозу
    from tmpStattal
    left join MKB on tmpStattal.MKB = MKB.DiagID
    where tmpStattal.`key` = stattalKey;
    if service is null then
	select EventType.service_id into service -- если не получается, пытаемся определить по типу события
	from tmpStattal
	left join Event on tmpStattal.Event_id = Event.id
	left join EventType on Event.eventType_id = EventType.id
	where tmpStattal.`key` = stattalKey;
    end if;
    if service is null then
	select rbSpeciality.service_id into service -- если и это не получается, пытаемся определить по специальности
	from tmpStattal
	left join tmpSpeciality on tmpStattal.speciality = tmpSpeciality.code
	left join rbSpeciality on tmpSpeciality.s11code = rbSpeciality.id
	where tmpStattal.`key` = stattalKey;
    end if;
    select serviceModifier into modifier
    from rbVisitType
    where rbVisitType.id = visitType;
    if modifier != '' then -- в зависимости от значения modifier, тип услуги может модифицироваться
	set service = applyModifier(service, modifier);
    end if;
    select serviceModifier into modifier
    from rbScene
    where rbScene.id = scene;
    if modifier != '' then -- в зависимости от значения modifier, тип услуги может модифицироваться
	set service = applyModifier(service, modifier);    
    end if;
    return service;
end//

DELIMITER;



select 'Установление соответствий для посещений...' as ' ';

update tmpGosop
set Visit_type = 0 -- неизвестно
where Visit_type is NULL;

--посещение определяем по событию, дате и врачу:
update tmpGosop
left join tmpMed on tmpGosop.vrach = tmpMed.code
left join tmpStattal on tmpGosop.`key` = tmpStattal.`key`,
Visit
set tmpGosop.Visit_id = Visit.id,
    tmpGosop.Visit_type = 2
where tmpGosop.Visit_type = 0
and tmpStattal.Event_id = Visit.event_id
and tmpGosop.`date` = date(Visit.`date`)
and tmpMed.Person_id = Visit.person_id;


select 'Добавление посещений...' as ' ';
-- добавляем все посещения?????????????????
update tmpGosop
set Visit_type = 5
where Visit_type = 0; -- добавляем те, которых ещё нет


insert into Visit
select  0 as id,
	now() as createDatetime,
	@person_id as createPerson_id,
	now() as modifyDatetime,
	@person_id as modifyPerson_id,
	0 as deleted,
	tmpStattal.Event_id as event_id,
	tmpPlaces.rbScene as scene_id,
	`date` as `date`,
	1 as visitType_id,
	ifnull(tmpMedV.Person_id, tmpMedE.Person_id) as person_id,
	if(tmpStattal.isPrimary = 1 and tmpGosop.`date` = tmpStattal.firstVisitDate, 1, 2) as isPrimary,
	ifnull(tmpFinance.rbFinance, 2) as finance_id, -- если финансирование не указано - ОМС
	getService_id(tmpGosop.`key`, 1, tmpPlaces.rbScene) as service_id,
	if(tmpFinance.rbFinance is not null, 1 << (tmpFinance.rbFinance*2), 0) as payStatus
from tmpGosop
left join tmpMed tmpMedV on tmpGosop.vrach = tmpMedV.code
left join tmpStattal on tmpGosop.`key` = tmpStattal.`key`
left join tmpMed tmpMedE on tmpStattal.vrach = tmpMedE.code
left join tmpFinance on tmpStattal.pay = tmpFinance.svPay
left join tmpPlaces on tmpGosop.place = tmpPlaces.svPlace
where tmpGosop.Visit_type = 5
and tmpStattal.Event_type = 2
and tmpPlaces.rbScene is not null
and (tmpMedV.Person_id is not null or tmpMedE.Person_id is not null);


select 'Установка соответствий для добавленных посещений...' as ' ';

--посещение определяем по событию, дате и врачу:
update tmpGosop
left join tmpMed on tmpGosop.vrach = tmpMed.code
left join tmpStattal on tmpGosop.`key` = tmpStattal.`key`,
Visit
set tmpGosop.Visit_id = Visit.id,
    tmpGosop.Visit_type = 2
where tmpGosop.Visit_type = 5
and tmpStattal.Event_id = Visit.event_id
and tmpGosop.`date` = date(Visit.`date`)
and tmpMed.Person_id = Visit.person_id;

select 'Удаление дублирующихся стат. талонов и перенос посещений...' as ' ';
--Если есть дубли, переносить все визиты в более ранний талон:
update Visit
left join tmpGosop on tmpGosop.Visit_id = Visit.id
			and tmpGosop.Visit_type = 2
left join tmpStattal C on tmpGosop.`key` = C.`key`
left join tmpStattal A on A.Event_type = 2
			and A.Client_id = C.Client_id
			and A.endDate = C.endDate
			and A.vrach = C.vrach
set Visit.event_id = A.Event_id
where A.begDate = (select min(B.begDate) from tmpStattal B -- сортируем лексикографически по начальной дате и ключу
		where B.Event_type = 2
		and A.Client_id = B.Client_id
		and A.endDate = B.endDate
		and A.vrach = B.vrach
		and A.`key` != B.`key` 
		)
and A.`key` = (select min(B.`key`) from tmpStattal B -- сортируем лексикографически по начальной дате и ключу
		where B.Event_type = 2
		and A.Client_id = B.Client_id
		and A.begDate = B.begDate
		and A.endDate = B.endDate
		and A.vrach = B.vrach);
--... и удалять дубли:
update Event
left join tmpStattal A on A.Event_id = Event.id
			and A.Event_type = 2
left join tmpStattal B on B.Event_type = 2
	and A.Client_id = B.Client_id
	and A.endDate = B.endDate
	and A.vrach = B.vrach
set Event.deleted = 1
where A.`key` != B.`key`
and Event.note='Импортировано из SV';
update Event
left join tmpStattal A on A.Event_id = Event.id
			and A.Event_type = 2
set Event.deleted = 0
where Event.note='Импортировано из SV'
and A.begDate = (select min(B.begDate) from tmpStattal B -- сортируем лексикографически по начальной дате и ключу
		where B.Event_type = 2
		and A.Client_id = B.Client_id
		and A.endDate = B.endDate
		and A.vrach = B.vrach
		and A.`key` != B.`key` 
		)
and A.`key` = (select min(B.`key`) from tmpStattal B -- сортируем лексикографически по начальной дате и ключу
		where B.Event_type = 2
		and A.Client_id = B.Client_id
		and A.begDate = B.begDate
		and A.endDate = B.endDate
		and A.vrach = B.vrach);
update tmpStattal
left join Event on tmpStattal.Event_id = Event.id and tmpStattal.Event_type = 2
set Event_type = 4,
Diagnostic_type = 4
where Event.deleted = 1;
delete from Event
where deleted = 1
and note='Импортировано из SV';


drop function applyModifier;
drop function getService_id;

select 'Импорт данных о посещениях завершён.' as ' ';