# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2015 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import sys
from sys import *
import codecs

from PyQt4 import QtCore, QtGui, QtSql
from PyQt4.QtCore import *
from PyQt4.QtSql import *

from library.Utils impot *
from Utils import *
from database import *
from Import import *
from CImport import CDBFimport
from setlinks import SetLinksDialog

from MKB import getRightCharacterId
from Diagnosis import getDiagnosisId2

from Ui_main import Ui_MainWindow

DefaultAverageDuration = 28

class mainWin(QtGui.QMainWindow, CDBFimport, Ui_MainWindow):
    def __init__(self):
        QtGui.QMainWindow.__init__(self)
        self.setupUi(self)
        CDBFimport.__init__(self)
        self.initSettings()

    def initSettings(self):
        settings = QSettings(QSettings.IniFormat, QSettings.UserScope, 'samson-vista', 'S11App')
        (self.orgId, ok) = settings.value('appPrefs/orgId').toInt()
        if not ok: #FIXME: что делать, если текущая организация не указана????
            pass
        (self.averageDuration, ok) = settings.value('appPrefs/averageDuration').toInt()
        if not ok:
            self.averageDuration = DefaultAverageDuration

        settings = QSettings(QSettings.IniFormat, QSettings.UserScope, 'samson-vista', 'importSV')
        self.editServer.setText(settings.value('server', QtCore.QVariant('localhost')).toString())
        self.editDatabase.setText(settings.value('database', QtCore.QVariant('s11')).toString())
        self.editUser.setText(settings.value('user', QtCore.QVariant('root')).toString())
        self.editPassword.setText(settings.value('password', QtCore.QVariant('')).toString())
        self.edtFileName.setText(settings.value('directory', QtCore.QVariant('')).toString())

    def saveSettings(self):
        settings = QSettings(QSettings.IniFormat, QSettings.UserScope, 'samson-vista', 'importSV')
        settings.setValue('server', QtCore.QVariant(self.editServer.text()))
        settings.setValue('database', QtCore.QVariant(self.editDatabase.text()))
        settings.setValue('user', QtCore.QVariant(self.editUser.text()))
        settings.setValue('password', QtCore.QVariant(self.editPassword.text()))
        settings.setValue('directory', QtCore.QVariant(self.edtFileName.text()))


    def registerDocumentTables(self):
        registerDocumentTable('Address')
        registerDocumentTable('AddressHouse')
        registerDocumentTable('Client')
        registerDocumentTable('ClientAddress')
        registerDocumentTable('ClientDocument')
        registerDocumentTable('ClientPolicy')
        registerDocumentTable('Diagnosis')
        registerDocumentTable('Diagnostic')
        registerDocumentTable('Event')
        registerDocumentTable('OrgStructure')
        registerDocumentTable('Person')
        registerDocumentTable('TempInvalid')
        registerDocumentTable('TempInvalid_Period')
        registerDocumentTable('Visit')

    def openDirectory(self):
        dialog = QtGui.QFileDialog(self)
        dialog.setFileMode(QtGui.QFileDialog.Directory)
        dialog.exec_()
        filename = dialog.selectedFiles()[0]
        self.edtFileName.setText(filename)

    def runProgramScript(self, name, title):
        file = codecs.open(name, encoding='utf-8', mode='rt')
        if file is None:
            QtGui.QMessageBox.warning(self, title,
                        u'Не могу открыть файл %s:\n%s.'%(name, file.errorString( )))
            return False
        QtGui.qApp.processEvents()
        error = self.runScript(file, {'person_id': None, 'org_id': self.orgId, 'averageDuration': self.averageDuration})
        if error != None:
            QtGui.QMessageBox.warning(self, title,
                        u'Ошибка при выполнении файла %s:\n%s.'%(name, error.text()))
            self.log.append(unicode(error.text()))
            return False
        file.close()
        return True

    def fullTables(self, filename, tables):
        for (name, fields, binaries) in tables:
            self.fullTable(filename + '/' + name + '.dbf', 'tmp' + name[0].upper() + name[1:].lower(), binaries, fields)

    def fullMedTables(self, filename):
        self.fullTables(filename, [('med', range(5) + [-1, -1]*2, []),])

    def fullOtherTables(self, filename):
#        tables = [('dop', range(20), [0]),
#                  ('hospsop', range(8), [0, 7]),
#                  ]
        tables = []
        if self.checkMain.isChecked():
            tables = tables + [('gosop', range(7) + [-1, -1], [0, 6]),
                               ('netrsop', range(25) + [-1]*2 + [-1, -1] + [-1]*2, [0]),
                               ('stattal', range(60)+ [-1] + [-1] + [-1, -1]*2 + [-1]*2, [0])
                            ]
        if self.checkDead.isChecked():
            tables = tables + [('dead', range(29), []),]
        self.fullTables(filename, tables)


    def checkCharacters(self): #TODO: переписать на SQL
        u"""Проверяем характеры заболеваний на соответствие МКБ в tmpDiagnosis"""
        for record in self.db.getRecordList('tmpDiagnosis'):
            MKB = forceString(record.value('MKB'))
            character = forceInt(record.value('diseaseCharacter'))
            new_character = getRightCharacterId(self.db, MKB, character)
            if new_character != character:
                client_id = forceInt(record.value('Client_id'))
                record.setValue('diseaseCharacter', toVariant(new_character))
                self.db.updateRecord('tmpDiagnosis', record)
                self.log.append(u'У клиента id=%d характер заболевания %d не соответствует МКБ %s; исправлен на %d'%(client_id, character, MKB, new_character))


    def addDiagnosises(self): #TODO: начальный запрос реализовать с помощью возвращаемых значений в runScript
        u"""Добавляет последовательно диагнозы из таблицы tmpDiagnosis"""
        self.log.append(u'Добавление диагнозов...')
        query = self.db.query("""SELECT tmpDiagnosis.id,
                                        date,
                                        tmpMed.Person_id,
                                        Client_id,
                                        rbDiagnosisType.id,
                                        MKB,
                                        MKBEx,
                                        diseaseCharacter,
                                        dispanser,
                                        tmpTraumaType.rbTrauma
                                FROM tmpDiagnosis
                                LEFT JOIN rbDiagnosisType on tmpDiagnosis.type = rbDiagnosisType.code
                                LEFT JOIN tmpMed ON tmpDiagnosis.person = tmpMed.code
                                LEFT JOIN tmpTraumaType ON tmpDiagnosis.traumaType = tmpTraumaType.svTrauma
                                WHERE Diagnosis_type = 5
                                ORDER BY date
                                        """)
        while query.next():
            record = query.record()
            id = forceInt(record.value(0))
            (diagid, character) = getDiagnosisId2(self.db,
                                                  self.averageDuration,
                                forceDate(record.value(1)),
                                forceRef(record.value(2)),
                                forceRef(record.value(3)),
                                forceRef(record.value(4)),
                                forceString(record.value(5)),
                                forceString(record.value(6)),
                                forceInt(record.value(7)),
                                forceRef(record.value(8)),
                                forceRef(record.value(9)))
            if diagid:
                self.db.query("""UPDATE tmpDiagnosis
                            SET Diagnosis_id = %d,
                            Diagnosis_type = 2
                            WHERE id = %d"""%(diagid, id))


    def startImport(self):
        try:
            self.db = CMySqlDatabase(self.editServer.text(), 0, self.editDatabase.text(), self.editUser.text(), self.editPassword.text())
        except Exception, e:
            QtGui.QMessageBox.critical(self,
                                            u'Произошла ошибка',
                                            unicode(e),
                                            QtGui.QMessageBox.Close)
            return
        self.log.setLogFile('import.log')
        self.registerDocumentTables()
        result = self.runProgramScript('create_links.sql',  u'Cкрипт создания временных таблиц соответствий')
        result = result and self.runProgramScript('create_data.sql',  u'Cкрипт создания временных таблиц для данных')
        filename = self.edtFileName.text()
        if filename.isEmpty():
            QtGui.QMessageBox.warning(None, u'Ошибка', u'Введите имя каталога с данными!')
            result = False
        if result:
            self.saveSettings()
            self.fullMedTables(filename)
            result = result and self.runProgramScript('explore_med.sql',  u'Скрипт установления соответствий для врачей')
            dialog = SetLinksDialog(self, self.db)
            dialog.exec_()
            self.fullOtherTables(filename)
            result = result and self.runProgramScript('import_med.sql',  u'Скрипт импортирования данных о врачах')
            result = result and self.runProgramScript('explore_client.sql',  u'Скрипт определения данных о пациентах')
            result = result and self.runProgramScript('import_client.sql',  u'Скрипт импортирования данных о пациентах')
            result = result and self.runProgramScript('import_stattal.sql',  u'Скрипт импортирования данных о статистических талонах')
            result = result and self.runProgramScript('import_dop.sql',  u'Скрипт импортирования дополнительных данных о пациентах')
            result = result and self.runProgramScript('explore_diagnosis.sql',  u'Скрипт определения данных о диагнозах')
            self.checkCharacters()
            self.addDiagnosises()
            result = result and self.runProgramScript('import_diagnosis.sql',  u'Скрипт импортирования данных о диагнозах и диагностике')
            result = result and self.runProgramScript('import_gosop.sql',  u'Скрипт импортирования данных о посещениях')
            result = result and self.runProgramScript('import_hospsop.sql',  u'Скрипт импортирования данных о госпитализации')
            result = result and self.runProgramScript('import_dead.sql',  u'Скрипт импортирования данных о смерти')
            result = result and self.runProgramScript('explore_netrsop.sql',  u'Скрипт установления соответствий для документов нетрудоспособности')
            result = result and self.runProgramScript('import_netrsop.sql',  u'Скрипт импортирования данных о документах нетрудоспособности')
        result = self.runProgramScript('deltemp.sql',  u'Скрипт удаления временных таблиц') and result
        if result:
            self.log.append(u'Обновление выполнено!')
            QtGui.QMessageBox.information( None,
                                    u'Всё хорошо!',
                                    u'Обновление выполнено.')
        else:
            QtGui.QMessageBox.information( None,
                                    u'Всё плохо!',
                                    u'Обновление не выполнено.')
        QtGui.qApp.processEvents()
        self.log.append(u'Результаты сохранены в файле import.log')
        self.db.close()


    def test(self):
        try:
            self.db = CMySqlDatabase(self.editServer.text(), 0, self.editDatabase.text(), self.editUser.text(), self.editPassword.text())
        except Exception, e:
            QtGui.QMessageBox.critical(self,
                                            u'Произошла ошибка',
                                            unicode(e),
                                            QtGui.QMessageBox.Close)
            return
        self.registerDocumentTables()
        self.runProgramScript('test.sql',  u'Проверяющий целостность БД скрипт')
        errorstxt = QtCore.QFile('errors.txt')
        errorstxt.open(QtCore.QFile.Text | QtCore.QFile.WriteOnly)
        outstream = QtCore.QTextStream(errorstxt)
        testsql = codecs.open('test.sql', encoding='utf-8', mode='rt')
        #self.runScript(self.db, testsql, None,  50, outstream)
        errorstxt.close()
        self.log.append(u'Результаты сохранены в файле errors.txt')
        self.db.close()



if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)
    try:
        mw=mainWin()
        mw.show()
        app.exec_()
    except Exception, e:
        QtGui.QMessageBox.critical( None,
                                    'error',
                                    unicode(e),
                                    QtGui.QMessageBox.Close)
    mw = None
    app = None
