# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2015 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import sys
from sys import *

from PyQt4 import QtCore, QtGui, QtSql

from Ui_link import Ui_Dialog
from Utils import *

class SetLinksDialog(QtGui.QDialog, Ui_Dialog):
    def __init__(self, parent, db):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)
        self.db = db
        self.setWindowTitle(self.getTitle())
        sourceModel = QtSql.QSqlQueryModel(self)
        sourceModel.setQuery(self.getSourceQuery(), self.db.db)
        self.sourceView.setModel(sourceModel)
        self.sourceView.hideColumn(0) # скрываем первичный ключ
        self.sourceView.hideColumn(1) # скрываем внешний ключ
        targetModel = QtSql.QSqlQueryModel(self)
        targetModel.setQuery(self.getTargetQuery(), self.db.db)
        self.targetView.setModel(targetModel)
        self.targetView.hideColumn(0) # скрываем первичный ключ

    def getTitle(self):
        return u'Установка соответствий для врачей'

    def getSourceQuery(self, filter='1'):
        return u"""
        SELECT tmpMed.code, tmpMed.Person_id,
           tmpMed.FIO as 'ФИО',
           tmpMed.code as 'Код',
           tmpSpeciality.name as 'Специальность',
           tmpMed.`section` as 'Отделение',
           tmpLinkType.name as 'Соответствие'
        FROM tmpMed
        LEFT JOIN tmpSpeciality ON tmpSpeciality.code = tmpMed.speciality
        LEFT JOIN tmpLinkType ON tmpLinkType.code = tmpMed.Person_type
        WHERE %s
        ORDER by tmpMed.FIO
        """%(filter)

    def getTargetQuery(self, filter='1'):
        return u"""
        SELECT Person.id,
            lastName as 'Фамилия',
            firstName as 'Имя',
            patrName as 'Отчество',
            Person.code as 'Код',
            rbSpeciality.name as 'Специальность',
            OrgStructure.name as 'Отделение'
        FROM Person
        LEFT JOIN rbSpeciality ON Person.speciality_id = rbSpeciality.id
        LEFT JOIN OrgStructure ON Person.orgStructure_id = OrgStructure.id
        WHERE %s
        ORDER by lastName
        """%(filter)

    def currentSourceValue(self, field):
        row = self.sourceView.currentIndex().row()
        if row >= 0:
            currentRecord = self.sourceView.model().record(row)
            return currentRecord.value(field)
        return QtCore.QVariant(None)
    def currentTargetValue(self, field):
        row = self.targetView.currentIndex().row()
        if row >= 0:
            currentRecord = self.targetView.model().record(row)
            return currentRecord.value(field)
        return QtCore.QVariant(None)

    def recordChanged(self):
        value = self.currentSourceValue('Person_id')
        if not value.isNull():
            (id, ok) = value.toInt()
            row2 = 0
            for i in xrange(self.targetView.model().rowCount()):
                (id2, ok) = self.targetView.model().record(i).value('id').toInt()
                if id2 == id:
                    row2 = i
            self.targetView.setCurrentIndex(self.targetView.model().createIndex(row2, 0))
        #print 'record changed!'

    def changeType(self, type, setNull=False):
        row = self.sourceView.currentIndex().row()
        id = self.currentSourceValue('code').toString()
        (id2, ok) = self.currentTargetValue('id').toInt()
        id2 = str(id2)
        msg = """
        UPDATE tmpMed
        SET Person_type = %d,
        Person_id = %s
        WHERE code = '%s'
        """%(type, 'NULL' if setNull else id2, id)
        self.db.query(msg)
        self.sourceView.model().setQuery(self.getSourceQuery(), self.db.db)
        self.sourceView.setCurrentIndex(self.sourceView.model().createIndex(row, 0))

    @QtCore.pyqtSignature('')
    def on_btnSet_clicked(self):
        self.changeType(3)

    @QtCore.pyqtSignature('')
    def on_btnAdd_clicked(self):
        self.changeType(5, True)

    @QtCore.pyqtSignature('')
    def on_btnDelete_clicked(self):
        self.changeType(0, True)

    @QtCore.pyqtSignature('')
    def on_btnImport_clicked(self):
        self.close()
