# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

u"""
Описание объекта Action и его свойств
"""
import sip

from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import QDateTime, QVariant, QDate, QTime

from library.Attach.AttachedFile import CAttachedFilesLoader
from library.Calendar         import wpFiveDays, wpSixDays, wpSevenDays, addWorkDays
from library.exception        import CException
from library.DbEntityCache    import CDbEntityCache
from library.Utils            import (forceBool, forceDate, forceDateTime, forceDouble, forceInt, forceRef,
                                      forceString, forceStringEx, formatNum1, toVariant)

from Users.Rights             import urEditAfterInvoicingEvent, urEditOtherpeopleAction

from Orgs.Utils               import getPersonChiefs
from Orgs.OrgComboBox         import CContractDbModel
from ActionProperty           import CActionProperty, CActionPropertyType
from Events.Utils             import (getActionTypeIdListByFlatCode,
                                      getExistsNomenclature,
                                      getEventTypeForm,
                                      getEventDuration,
                                      getEventActionContract,
                                      syncNomenclature,
                                     )
from Events.ActionServiceType import CActionServiceType
from Events.ActionStatus      import CActionStatus
from Events.Models.Event import CEvent
from Events.ActionExecutionPlan import CActionExecutionPlan

from Stock.Service import CStockService

from Stock.Utils import getNomenclatureUnitRatio, applyNomenclatureUnitRatio, findFinanceBatchShelfTime, getStockMotionNumberCounterId, getExistsNomenclatureAmount


class CActionType(object):
    # режимы подсчёта количества в действии
    userInput                            = 0
    eventVisitCount                      = 1
    eventDurationWithFiveDayWorking      = 2
    eventDurationWithSixDayWorking       = 3
    eventDurationWithSevenDayWorking     = 4
    actionFilledPropsCount               = 5
    actionAssignedPropsCount             = 6
    actionDurationWithFiveDayWorking     = 7
    actionDurationWithSixDayWorking      = 8
    actionDurationWithSevenDayWorking    = 9
    actionPredefinedNumber               = 10

    # их названия
    amountEvaluation = [
                        u'Количество вводится непосредственно',
                        u'Число визитов',
                        u'Длительность события при пятидневной рабочей неделе',
                        u'Длительность события при шестидневной рабочей неделе',
                        u'Длительность события при семидневной рабочей неделе',
                        u'Количество заполненных свойств действия',
                        u'Количество назначенных свойств действия',
                        u'Длительность действия при пятидневной рабочей неделе',
                        u'Длительность действия при шестидневной рабочей неделе',
                        u'Длительность действия при семидневной рабочей неделе',
                        u'Предопределенное количество'
                       ]

#    amountEvaluationFreeInput = 0

    # дата выполнения(default end date)
    dedUndefined      = 0
    dedCurrentDate    = 1
    dedEventSetDate   = 2
    dedEventExecDate  = 3
    dedActionBegDate  = 4
    dedSyncActionBegDate  = 5

    # дата начала действия(default beg date)
    dbdUndefined      = 0
    dbdCurrentDate    = 1
    dbdEventSetDate   = 2
    dbdEventExecDate  = 3
    dbdActionEndDate  = 4
    dbdJobTicketTime  = 5

    # планируемая дата выполнения(default planned end date)
    dpedUndefined           = 0
    dpedNextDay             = 1
    dpedNextWorkDay         = 2
    dpedJobTicketDate       = 3
    dpedBegDatePlusAmount   = 4
    dpedBegDatePlusDuration = 5

    # заполнение планируемой даты выполнения(isPlannedEndDateRequired)
    dpedControlNoNeed       = 0
    dpedControlMild         = 1
    dpedControlHard         = 2

    # дата назначения(default direction date)
    dddUndefined      = 0
    dddEventSetDate   = 1
    dddCurrentDate    = 2
    dddActionExecDate = 3

    # ответственный (default person)
    dpUndefined       = 0
    dpEmpty           = 1
    dpSetPerson       = 2
    dpEventExecPerson = 3
    dpCurrentUser     = 4
    dpCurrentMedUser  = 5
    dpUserExecPerson  = 6

    #MKB (default MKB)
    dmkbNotUsed           = 0
    dmkbByFinalDiag       = 1
    dmkbBySetPersonDiag   = 2
    dmkbSyncFinalDiag     = 3
    dmkbSyncSetPersonDiag = 4
    dmkbUserInput         = 5

    def __init__(self, record):
        self._propertiesByName = {}
        self._propertiesById = {}
        self._propertiesByType = {}
        self.initByRecord(record)


    def initByRecord(self, record):
        self.id   = forceRef(record.value('id'))
        self.groupId = forceRef(record.value('group_id'))
        self.class_ = forceInt(record.value('class'))
        self.code = forceString(record.value('code'))
        self.name = forceString(record.value('name'))
        self.title = forceString(record.value('title'))
        self.flatCode = forceString(record.value('flatCode'))
        self.isRequiredCoordination = forceBool(record.value('isRequiredCoordination'))
#        self.serviceId = forceRef(record.value('service_id'))
        self.amount = forceDouble(record.value('amount'))
        self.amountEvaluation = forceInt(record.value('amountEvaluation'))
        self.defaultStatus  = forceInt(record.value('defaultStatus'))
        self.defaultDirectionDate = forceInt(record.value('defaultDirectionDate'))
        self.defaultPlannedEndDate = forceInt(record.value('defaultPlannedEndDate'))
        self.isPlannedEndDateRequired = forceInt(record.value('isPlannedEndDateRequired'))
        self.defaultBegDate = forceInt(record.value('defaultBegDate'))
        self.defaultEndDate = forceInt(record.value('defaultEndDate'))
        self.defaultExecPersonId   = forceRef(record.value('defaultExecPerson_id'))
        self.defaultPersonInEvent  = forceInt(record.value('defaultPersonInEvent'))
        self.defaultPersonInEditor = forceInt(record.value('defaultPersonInEditor'))
        self.defaultOrgId          = forceRef(record.value('defaultOrg_id'))
        self.defaultMKB            = forceInt(record.value('defaultMKB'))
        self.isMorphologyRequired  = forceInt(record.value('isMorphologyRequired'))
        self.office = forceString(record.value('office'))
        self.showTime = forceBool(record.value('showTime'))
        self.maxOccursInEvent = forceInt(record.value('maxOccursInEvent'))
        self.isMes = forceBool(record.value('isMES'))
        self.nomenclativeServiceId = forceRef(record.value('nomenclativeService_id'))
        self.context = forceString(record.value('context'))
        self.prescribedTypeId = forceRef(record.value('prescribedType_id'))
        self.sheduleId = forceRef(record.value('shedule_id'))
        self.isNomenclatureExpense = forceBool(record.value('isNomenclatureExpense'))
        self.nomenclatureCounterId = forceRef(record.value('nomenclatureCounter_id'))
        self.generateStockMotionReason = forceBool(record.value('generateStockMotionReason'))
        self.hasAssistant = forceBool(record.value('hasAssistant'))
        self.propertyAssignedVisible = forceBool(record.value('propertyAssignedVisible'))
        self.propertyUnitVisible = forceBool(record.value('propertyUnitVisible'))
        self.propertyNormVisible = forceBool(record.value('propertyNormVisible'))
        self.propertyEvaluationVisible = forceBool(record.value('propertyEvaluationVisible'))
        self.serviceType = forceInt(record.value('serviceType'))
        self.actualAppointmentDuration = forceInt(record.value('actualAppointmentDuration'))
        self.ticketDuration = forceInt(record.value('ticketDuration'))
        self.closeEvent = forceInt(record.value('closeEvent'))
        self.addVisit = forceInt(record.value('addVisit'))
        self.addVisitSceneId = forceRef(record.value('addVisitScene_id'))
        self.addVisitTypeId = forceRef(record.value('addVisitType_id'))
        self.generateAfterEventExecDate = forceBool(record.value('generateAfterEventExecDate'))

        self._nomenclatureRecordList = None
        self._hasJobTicketPropertyType = False

        self._initProperties()
        self._loadPFOrgStructureRecordList()
        self._loadPFSpecialityRecordList()
        self._loadEquipmentRecordList()
        self._loadQuotaTypeList()
        if self.isNomenclatureExpense:
            self._loadNomenclatureRecordList()


    def isServiceTypeProcedure(self):
        return self.serviceType == CActionServiceType.procedure


    def isServiceTypeResearch(self):
        return self.serviceType in (CActionServiceType.research, CActionServiceType.labResearch)


    def hasJobTicketPropertyType(self):
        return self._hasJobTicketPropertyType


    def _loadNomenclatureRecordList(self):
        self._nomenclatureRecordList = QtGui.qApp.db.getRecordList('ActionType_Nomenclature', '*', 'master_id=%d'%self.id)


    def _loadPFSpecialityRecordList(self):
        self._pfSpecialityRecordList = QtGui.qApp.db.getRecordList('ActionType_PFSpeciality', '*', 'master_id=%d'%self.id)


    def _loadPFOrgStructureRecordList(self):
        self._pfOrgStructureRecordList = QtGui.qApp.db.getRecordList('ActionType_PFOrgStructure', '*', 'master_id=%d'%self.id)


    def _loadEquipmentRecordList(self):
        self._equipmentRecordList = QtGui.qApp.db.getRecordList('ActionType_Equipment', '*', 'master_id=%d'%self.id)
        self._equipmentIdList = []
        for record in self._equipmentRecordList:
            self._equipmentIdList.append(forceRef(record.value('equipment_id')))


    def _loadQuotaTypeList(self):
        self._quotaTypeList = QtGui.qApp.db.getRecordList('ActionType_QuotaType', '*', 'master_id=%d'%self.id)
        self._quotaType = []
        for record in self._quotaTypeList:
            self._quotaType.append(forceRef(record.value('quotaType_id')))


    def getNomenclatureRecordList(self):
        return self._nomenclatureRecordList


    def getPFSpecialityRecordList(self):
        return self._pfSpecialityRecordList


    def getPFOrgStructureRecordList(self):
        return self._pfOrgStructureRecordList


    def _initProperties(self):
        db = QtGui.qApp.db
        tablePropertyType = db.table('ActionPropertyType')
        stmt = db.selectStmt(tablePropertyType, '*', [tablePropertyType['actionType_id'].eq(self.id), tablePropertyType['deleted'].eq(0)], 'name')
        query = db.query(stmt)
        while query.next():
            propertyType = CActionPropertyType(query.record())
            self._propertiesByName[propertyType.name] = propertyType
            self._propertiesById[propertyType.id] = propertyType
            propsList = self._propertiesByType.setdefault(propertyType.typeName, [])
            propsList.append(propertyType)
            if not self._hasJobTicketPropertyType and propertyType.isJobTicketValueType():
                self._hasJobTicketPropertyType = True


    def getPropertiesById(self):
        return self._propertiesById


    def getPropertiesByName(self):
        return self._propertiesByName


    def hasProperty(self, name):
        return name in self._propertiesByName


    def getPropertyType(self, name):
        return self._propertiesByName[name]


    def getPropertyTypeById(self, id):
        return self._propertiesById[id]


    def getPropertiesTypeByTypeName(self, typeName):
        return self._propertiesByType.get(typeName, [])


    def propertyTypeIdPresent(self, id):
        return id in self._propertiesById


    def checkMaxOccursLimit(self, count, displayMessage=True):
        result = self.maxOccursInEvent == 0 or count < self.maxOccursInEvent
        if not result and displayMessage:
            widget = QtGui.qApp.activeModalWidget() or QtGui.qApp.mainWindow or None
            QtGui.QMessageBox.critical( widget,
                                        u'Произошла ошибка',
                                        u'Действие типа "%s" должно применяться в осмотре не более %s' % (self.name, formatNum1(self.maxOccursInEvent, (u'раза', u'раз', u'раз'))),
                                        QtGui.QMessageBox.Close)
        return result


    def checkReceivedMovingLeaved(self, message):
        widget = QtGui.qApp.activeModalWidget() or QtGui.qApp.mainWindow or None
        QtGui.QMessageBox.critical( widget,
                                    u'Произошла ошибка',
                                    message,
                                    QtGui.QMessageBox.Close)
        return False



class CActionTypeCache(CDbEntityCache):
    mapIdToActionType = {}
    mapCodeToActionType = {}
    mapFlatCodeToActionType = {}

    @classmethod
    def purge(cls):
        cls.mapIdToActionType.clear()
        cls.mapCodeToActionType.clear()
        cls.mapFlatCodeToActionType.clear()


    @classmethod
    def getById(cls, actionTypeId):
        result = cls.mapIdToActionType.get(actionTypeId, None)
        if not result:
            cls.connect()
            db = QtGui.qApp.db
            actionTypeRecord = db.getRecord('ActionType', '*', actionTypeId)
            result = CActionType(actionTypeRecord)
            cls.register(result)
        return result


    @classmethod
    def getByCode(cls, actionTypeCode):
        result = cls.mapCodeToActionType.get(actionTypeCode, None)
        if not result:
            cls.connect()
            db = QtGui.qApp.db
            tableActionType = db.table('ActionType')
            actionTypeRecord = db.getRecordEx(tableActionType, '*', tableActionType['code'].eq(actionTypeCode))
            assert actionTypeRecord
            result = CActionType(actionTypeRecord)
            cls.register(result)
        return result


    @classmethod
    def getByFlatCode(cls, actionTypeFlatCode):
        result = cls.mapFlatCodeToActionType.get(actionTypeFlatCode, None)
        if not result:
            cls.connect()
            db = QtGui.qApp.db
            tableActionType = db.table('ActionType')
            actionTypeRecord = db.getRecordEx(tableActionType, '*', tableActionType['flatCode'].eq(actionTypeFlatCode))
            assert actionTypeRecord
            result = CActionType(actionTypeRecord)
            cls.register(result)
        return result


    @classmethod
    def register(cls, actionType):
        cls.mapIdToActionType[actionType.id]     = actionType
        cls.mapCodeToActionType[actionType.code] = actionType
        cls.mapFlatCodeToActionType[actionType.flatCode] = actionType


# ################################################



class CAction(object):
    # __slots__ = ['_actionType', '_record', '_properties']

    def __init__(self, actionType=None, record=None):
        self._actionType = actionType
        self._record = None
        self._propertiesByName  = {}
        self._propertiesById    = {}
        self._executionPlan = CActionExecutionPlan(self)
        self._presetValuesConditions = {}
        self._properties = []
        self._locked = False
        self.nomenclatureExpense = None
        self.nomenclatureClientReservation = None
        self._specifiedName = ''
        self._clientId = None
        self._event = None
        self._financeId = None
        self._attachedFileItemList = []
        self.checkModifyDate = True
        if record:
            self.setRecord(record)

    @property
    def presetValuesConditions(self):
        return self._presetValuesConditions

    @classmethod
    def createByTypeId(cls, actionTypeId):
        actionType = CActionTypeCache.getById(actionTypeId)
        return cls(actionType=actionType)


    @classmethod
    def createByTypeCode(cls, actionTypeCode):
        actionType = CActionTypeCache.getByCode(actionTypeCode)
        return cls(actionType=actionType)


    @classmethod
    def getActionById(cls, actionId):
        db = QtGui.qApp.db
        actionRecord = db.getRecord('Action', '*', actionId)
        return cls(record=actionRecord) if actionRecord else None


    @classmethod
    def getAction(cls, eventId, actionTypeCode):
        db = QtGui.qApp.db
        tableAction = db.table('Action')
        actionType = CActionTypeCache.getByCode(actionTypeCode)
        cond = [tableAction['event_id'].eq(eventId), tableAction['actionType_id'].eq(actionType.id)]
        record = db.getRecordEx(tableAction, '*', cond)
        return cls(record=record, actionType=actionType)


    def setRecord(self, record):
        # установить тип
        actionTypeId = forceRef(record.value('actionType_id'))
        if self._actionType:
            assert actionTypeId == self._actionType.id
        else:
            self._actionType = CActionTypeCache.getById(actionTypeId)
        self._record = record
        # инициализировать properties
        actionId = record.value('id')
        if forceRef(actionId):
            db = QtGui.qApp.db
            tableActionProperty = db.table('ActionProperty')
            stmt = db.selectStmt(tableActionProperty, '*', [tableActionProperty['action_id'].eq(actionId), tableActionProperty['deleted'].eq(0)])
            query = db.query(stmt)
            while query.next():
                propertyRecord = query.record()
                propertyTypeId = forceRef(propertyRecord.value('type_id'))
                if self._actionType.propertyTypeIdPresent(propertyTypeId):
                    prop = CActionProperty(self._actionType, propertyRecord)
#???                    if prop.isValid():
                    self._propertiesByName[prop._type.name] = prop
                    self._propertiesById[prop._type.id] = prop
            self._properties = self._propertiesById.values()
            self._properties.sort(key=lambda prop:prop._type.idx)

            self._executionPlan.load()

            self.nomenclatureClientReservation = CNomenclatureClientReservation(self).load()
        # спец. отметки
        status = forceInt(record.value('status'))
        personId = forceRef(record.value('person_id'))
        if not record.isNull('id') and status == CActionStatus.finished and personId:
            self._locked = not ( QtGui.qApp.userId == personId
                             or QtGui.qApp.userHasRight(urEditOtherpeopleAction)
                             or QtGui.qApp.userId in getPersonChiefs(personId)
                           )
        else:
            self._locked = False
        # списание ЛСиИМН
        if forceRef(actionId) and self._actionType.isNomenclatureExpense:
            self.nomenclatureExpense = CNomenclatureExpense(self, forceRef(actionId), self._actionType,  financeId=forceRef(record.value('finance_id')))
        self._specifiedName = forceString(record.value('specifiedName'))
        # прикреплённые файлы
        self._attachedFileItemList = []
        if hasattr(QtGui.qApp, 'webDAVInterface'): # у скриптов типа labExchange нет таких штук, а попользоваться этими функциями хочется
            storageInterface = QtGui.qApp.webDAVInterface
            self._attachedFileItemList = CAttachedFilesLoader.loadItems(storageInterface, 'Action_FileAttach', actionId)


    def addNomenclature(self, nomenclatureId):
        if self.nomenclatureExpense:
            self._financeId = self.getFinanceId()
            self.nomenclatureExpense.addNomenclature(nomenclatureId, financeId=self._financeId)

    def updateNomenclatureDosageValue(self, nomenclatureId, dosageValue):
        if self.nomenclatureExpense:
            self.nomenclatureExpense.updateNomenclatureDosageValue(nomenclatureId, dosageValue)


    def removeNomenclature(self, nomenclatureId):
        if self.nomenclatureExpense:
            self.nomenclatureExpense.removeNomenclature(nomenclatureId)


    def findDosagePropertyValue(self):
        actionType = self.getType()

        for propertyType in actionType.getPropertiesById().values():
            if propertyType.inActionsSelectionTable:
                if propertyType.inActionsSelectionTable == 2:
                    return self.getPropertyById(propertyType.id).getValue()

    def getStockMotionRecord(self):
        return self.nomenclatureExpense.stockMotionRecord() if self.nomenclatureExpense else None


    def getStockMotionItemList(self):
        return self.nomenclatureExpense.stockMotionItems() if self.nomenclatureExpense else []


    def initNomenclature(self, clientId, financeId=None):
        if QtGui.qApp.controlSMFinance() == 0:
            self.nomenclatureExpense = CNomenclatureExpense(self, None, self._actionType, clientId=clientId) if self._actionType and self._actionType.isNomenclatureExpense else None
        else:
            self.nomenclatureExpense = CNomenclatureExpense(self, None, self._actionType, clientId=clientId, financeId=financeId) if self._actionType and self._actionType.isNomenclatureExpense else None

    def _getNewRecord(self):
        db = QtGui.qApp.db
        tableAction = db.table('Action')
        record = tableAction.newRecord()
        record.setValue('actionType_id', QVariant(self._actionType.id))
        record.setValue('specifiedName', QVariant(self._specifiedName))
        return record

    def cancel(self):
        # Вот в случае разных отмен, не сохранения действия, тут нужно зависимости подчищать.
        if self.nomenclatureClientReservation:
            self.nomenclatureClientReservation.cancel()

    def save(self, eventId=None, idx=0, checkModifyDate = True):
        db = QtGui.qApp.db
        id = forceRef(self._record.value('id')) if self._record else None
        if id and checkModifyDate and self.checkModifyDate:
            curRec = db.getRecord('Action', ['modifyDatetime', 'modifyPerson_id'], id)
            if curRec:
                modDT = forceDateTime(curRec.value(0))
                modPerson = forceInt(curRec.value(1))
                if modDT != forceDateTime(self._record.value('modifyDatetime')):
                    #TODO Не согласен, что объект класса CAction при невалидном состоянии должен писать что-то в лог.
                    # Мне кажется, что тут должно бросаться специальное исключение,
                    # а уже вызваший данный метод как хочет так пусть и обрабатывает такую ситуацию.
                    QtGui.qApp.log(u'Предупреждение ', u'Действие с id %i не сохранено, так как дата модификации записи отличается. Изменивший пользователь на момент открытия %i, на момент сохранения %i'
                                   %(id, forceInt(self._record.value('modifyPerson_id')), modPerson))
                    return id
        if self._locked:
            # для заблокированной записи сохраняем только idx (позиция в списке на экране)
            id = forceRef(self._record.value('id'))
            if forceInt(self._record.value('idx')) != idx:
                self._record.setValue('idx', QVariant(idx))
                db.query('UPDATE Action SET idx=%d WHERE id=%d' % (idx, id))
            return id
        else:
            # сохранить основную запись
            tableAction = db.table('Action')
            if not self._record:
                self._record = self._getNewRecord()
            if eventId:
                self._record.setValue('event_id', QVariant(eventId))
            else:
                eventId = forceRef(self._record.value('event_id'))
            if idx != -1:
                #TODO как насчет использовать None для определения случая не изменять порядковый номер?
                self._record.setValue('idx', QVariant(idx))

            # Это хак: удаляем payStatus из записи
            tmpRecord = type(self._record)(self._record) # copy record
            tmpRecord.remove(tmpRecord.indexOf('payStatus'))
            id = db.insertOrUpdate(tableAction, tmpRecord)
            self._record.setValue('id', toVariant(id))
            propertiesIdList = []

            # сохранить записи свойств
            actionTypeIdList = None
            tableEvent = db.table('Event')
            tableEventType = db.table('EventType')
            tableRBMedicalAidKind = db.table('rbMedicalAidKind')
            queryTable = tableEvent.innerJoin(tableEventType, tableEventType['id'].eq(tableEvent['eventType_id']))
            queryTable = queryTable.innerJoin(tableRBMedicalAidKind, tableRBMedicalAidKind['id'].eq(tableEventType['medicalAidKind_id']))
            condMAK = [tableEvent['id'].eq(eventId),
                       tableEvent['deleted'].eq(0),
                       tableEventType['deleted'].eq(0),
                       tableRBMedicalAidKind['code'].inlist([u'1', u'2', u'3', u'7'])
                       ]
            recordMAK = db.getRecordEx(queryTable, [tableRBMedicalAidKind['id']], condMAK)
            makId = forceRef(recordMAK.value('id')) if recordMAK else None
            if makId:
                actionTypeList = getActionTypeIdListByFlatCode(u'moving%')
                actionTypeIdList = (','.join(str(actionTypeId) for actionTypeId in actionTypeList if actionTypeId))
            for property in self._propertiesById.itervalues():
                changed = property._changed or property._normChanged
                propertyId = property.save(id)
                propertiesIdList.append(propertyId)
                if changed:
                    if property.type().isJobTicketValueType() and property.getValue() and eventId:
                        #WFT?!!! смотреть страшно...
                        recordJT = db.getRecord('Job_Ticket', '*', property.getValue())
                        if recordJT:
                            begDateJT = forceDateTime(recordJT.value('dateTime'))
                            execOrgStructureId = None
                            if actionTypeIdList and makId:
                                nameProperty = u'''Отделение пребывания'''
                                stmt = '''
                                        SELECT (SELECT APOS.value
                                        FROM ActionPropertyType AS APT
                                        INNER JOIN ActionProperty AS AP ON AP.type_id=APT.id
                                        INNER JOIN ActionProperty_OrgStructure AS APOS ON APOS.id=AP.id
                                        WHERE APT.actionType_id=Action.actionType_id AND AP.action_id=Action.id AND APT.deleted=0
                                        AND APT.name = '%s') AS execOrgStructureId
                                        FROM Event
                                        INNER JOIN Action ON Event.id = Action.event_id
                                        WHERE Event.id = %d AND Action.deleted = 0 AND Event.deleted = 0 AND Action.actionType_id IN (%s)
                                        AND Action.begDate <= %s AND (Action.endDate IS NULL OR Action.endDate > %s)
                                        ORDER BY Action.id DESC
                                        LIMIT 1
                                        '''%(nameProperty, eventId, actionTypeIdList, db.formatDate(begDateJT), db.formatDate(begDateJT))
                                query = db.query(stmt)
                                if query.next():
                                    execOSRecord = query.record()
                                    execOrgStructureId = forceRef(execOSRecord.value('execOrgStructureId')) if execOSRecord else None
                            if not execOrgStructureId and makId:
                                execOrgStructureId = QtGui.qApp.currentOrgStructureId()
#                            if not execOrgStructureId:
#                                setPersonId = forceRef(self._record.value('setPerson_id'))
#                                if setPersonId:
#                                    stmt = '''SELECT Person.orgStructure_id
#                                                FROM Person
#                                                WHERE Person.id = %d AND Person.deleted = 0
#                                                LIMIT 1'''%(setPersonId)
#                                    query = db.query(stmt)
#                                    if query.next():
#                                        execOSRecord = query.record()
#                                        execOrgStructureId = forceRef(execOSRecord.value('orgStructure_id')) if execOSRecord else None
                            if not execOrgStructureId:
                                execOrgStructureId = QtGui.qApp.userOrgStructureId
                            #TODO А давайте закоментированное удалать? Код всегда можно будет востановить через SVN
#                            if not execOrgStructureId:
#                                eventId = forceRef(self._record.value('event_id'))
#                                if eventId:
#                                    tableEvent = db.table('Event')
#                                    cond = [tableEvent['id'].eq(eventId),
#                                            tableEvent['deleted'].eq(0)
#                                            ]
#                                    recordCl = db.getRecordEx(tableEvent, [tableEvent['client_id']], cond)
#                                    clientId = forceRef(recordCl.value('client_id')) if recordCl else None
#                                    if clientId:
#                                        tableCAT = db.table('ClientAttach')
#                                        cond = [u'ClientAttach.id = getClientAttachIdForDate(%s, 0, DATE(%s))'%(clientId, db.formatDate(begDateJT)),
#                                                tableCAT['id'].isNotNull(),
#                                                tableCAT['deleted'].eq(0)
#                                                ]
#                                        cols = [tableCAT['orgStructure_id']]
#                                        recordCAT = db.getRecordEx(tableCAT, cols, cond)
#                                        execOrgStructureId = forceRef(recordCAT.value('orgStructure_id')) if recordCAT else None
#                                        if not execOrgStructureId:
#                                            tableCA = db.table('ClientAddress')
#                                            tableAddress = db.table('Address')
#                                            tableOSAddress = db.table('OrgStructure_Address')
#                                            queryTable = tableCA.innerJoin(tableAddress, tableAddress['id'].eq(tableCA['address_id']))
#                                            queryTable = queryTable.innerJoin(tableOSAddress, tableOSAddress['house_id'].eq(tableAddress['house_id']))
#                                            cond = [u'ClientAddress.id = getClientRegAddressId(%s)'%(clientId),
#                                                    tableCA['id'].isNotNull(),
#                                                    tableCA['deleted'].eq(0),
#                                                    tableAddress['deleted'].eq(0)
#                                                    ]
#                                            cols = [tableOSAddress['master_id']]
#                                            recordOSAddress = db.getRecordEx(queryTable, cols, cond)
#                                            execOrgStructureId = forceRef(recordOSAddress.value('master_id')) if recordOSAddress else None
#                                            if not execOrgStructureId:
#                                                cond = [u'ClientAddress.id = getClientLocAddressId(%s)'%(clientId),
#                                                        tableCA['id'].isNotNull(),
#                                                        tableCA['deleted'].eq(0),
#                                                        tableAddress['deleted'].eq(0)
#                                                        ]
#                                                recordOSAddress = db.getRecordEx(queryTable, cols, cond)
#                                                execOrgStructureId = forceRef(recordOSAddress.value('master_id')) if recordOSAddress else None
#                            if not execOrgStructureId:
#                                jobId = forceRef(recordJT.value('master_id'))
#                                if jobId:
#                                    recordJob = db.getRecord('Job', 'orgStructure_id', jobId)
#                                    execOrgStructureId = forceRef(recordJob.value('orgStructure_id')) if recordJob else None
                            if execOrgStructureId:
                                recordJT.setValue('orgStructure_id', toVariant(execOrgStructureId))
                                db.updateRecord('Job_Ticket', recordJT)

            # сохранить план выполнения
#            executionPlanIdList = []
            self._executionPlan.save()
                # tableActionExecutionPlan = db.table('Action_ExecutionPlan')
                #
                # def _saveExecutionPlanRecord(planRecord):
                #     planRecord.setValue('master_id', QVariant(id))
                #     executionPlanId = db.insertOrUpdate(tableActionExecutionPlan, planRecord)
                #     planRecord.setValue('id', toVariant(executionPlanId))
                #     if executionPlanId not in executionPlanIdList:
                #         executionPlanIdList.append(executionPlanId)
                #
                # for execDate, execTimeDict in self._executionPlan.items():
                #     # Это значит что открывали планировщие назначений по F2 в окне выбора действия F9
                #     # И заплонировали по дням и по времени
                #     if execTimeDict:
                #         for execTime, planRecord in execTimeDict.items():
                #             _saveExecutionPlanRecord(planRecord)
                #     else:
                #         # Планирование назначений не открывали, но на основе
                #         # Action свойств duration и periodicity был сформирован план по дням.
                #         for _ in range(forceInt(self._record.value('aliquoticity')) or 1):
                #             planRecord = tableActionExecutionPlan.newRecord()
                #             planRecord.setValue('execDate', toVariant(execDate))
                #             _saveExecutionPlanRecord(planRecord)

            # удалить записи свойств кроме сохранённых
            tableActionProperty = db.table('ActionProperty')
            if propertiesIdList:
                db.deleteRecord(tableActionProperty,
                                 [ tableActionProperty['action_id'].eq(id),
                                   tableActionProperty['id'].notInlist(propertiesIdList)])
            else:
                db.deleteRecord(tableActionProperty,
                                 [ tableActionProperty['action_id'].eq(id) ])

            # # удалить записи плана выполнения кроме сохранённых
            # if self._executionPlan:
            #     tableActionExecutionPlan = db.table('Action_ExecutionPlan')
            #     if executionPlanIdList:
            #         db.deleteRecord(tableActionExecutionPlan,
            #                          [ tableActionExecutionPlan['master_id'].eq(id),
            #                            tableActionExecutionPlan['id'].notInlist(executionPlanIdList)])
            #     else:
            #         db.deleteRecord(tableActionExecutionPlan,
            #                          [ tableActionExecutionPlan['master_id'].eq(id) ])

            # списание ЛСиИМН
            if self.saveNomenclatureExpense():
                db.updateRecord(tableAction, self._record)


            self.updateNomenclatureClientReservation()

            if hasattr(QtGui.qApp, 'webDAVInterface'): # у скриптов типа labExchange нет таких штук, а попользоваться этими функциями хочется
                storageInterface = QtGui.qApp.webDAVInterface
                CAttachedFilesLoader.saveItems(storageInterface, 'Action_FileAttach', id, self._attachedFileItemList)
            return id

    def updateNomenclatureClientReservation(self):
        if not self.nomenclatureClientReservation:
            return

        self.nomenclatureClientReservation.linkAction(self)

    def saveNomenclatureExpense(self):
        status = forceInt(self.getRecord().value('status'))
        if self.nomenclatureExpense:
            if (self._actionType.generateAfterEventExecDate and self.event.execDate) or (not self._actionType.generateAfterEventExecDate and not self.event.execDate):
                if self.nomenclatureExpense and status == CActionStatus.finished and (
                        self.nomenclatureExpense.getStockMotionId() or self.nomenclatureExpense.stockMotionItems()
                ):
                    if QtGui.qApp.controlSMFinance() in (1, 2):
                        self._financeId = self.getFinanceId()
                    stockMotionId = self.nomenclatureExpense.save()
                    if stockMotionId:
                        if not self._record:
                            self._record = self._getNewRecord()
                        self._record.setValue('stockMotion_id', toVariant(stockMotionId))
                        return True
                elif status == CActionStatus.canceled:
                    self.cancel()

        return False


    def getType(self):
        return self._actionType


    def getRecord(self):
        return self._record

    @property
    def idx(self):
        return forceInt(self._record.value('idx')) if self._record else None

    def getId(self):
        return forceRef(self._record.value('id')) if self._record else None


    def getEventId(self):
        return forceRef(self._record.value('event_id')) if self._record else None


    def getFinanceId(self):
        return forceRef(self._record.value('finance_id')) if self._record else None


    @property
    def event(self):
        if self._event:
            return self._event

        eventId = self.getEventId()
        if not eventId:
            return None

        eventRecord = QtGui.qApp.db.getRecord(CEvent.tableName, '*', eventId)
        if not eventRecord:
            return None

        self._event = CEvent(eventRecord)
        return self._event


    def isLocked(self):
        return self._locked


    def isExposed(self):
        payStatus = forceInt(self._record.value('payStatus')) if self._record else 0
        actionId = forceRef(self._record.value('id')) if self._record else None
        eventId = forceRef(self._record.value('event_id')) if self._record else None
        if not payStatus and actionId and eventId:
            db = QtGui.qApp.db
            table = db.table('Action')
            cond = [ table['event_id'].eq(eventId),
                     table['id'].eq(actionId),
                     table['payStatus'].ne(0),
                     table['deleted'].eq(0)
                   ]
            record = db.getRecordEx(table, [table['payStatus']], where=cond)
            payStatus = forceInt(record.value('payStatus')) if record else 0
        if payStatus and not QtGui.qApp.userHasRight(urEditAfterInvoicingEvent):
            return True
        return False


    def getSpecifiedName(self):
        return self._specifiedName


    def getAttachedFileItemList(self):
        return self._attachedFileItemList


    def getProperties(self):
        return self._properties


    def getPropertiesById(self):
        return self._propertiesById


    def getPropertiesByName(self):
        return self._propertiesByName


    def getFilledPropertiesCount(self):
        count = 0
        for prop in self._propertiesById.itervalues():
            if prop.getValue():
                count += 1
        return count


    def getAssignedPropertiesCount(self):
        count = 0
        for prop in self._propertiesById.itervalues():
            if prop.isAssigned():
                count += 1
        return count


    def hasProperty(self, name):
        return self._actionType.hasProperty(name)


    def getProperty(self, name):
        result = self._propertiesByName.get(name, None)
        if not result:
            propertyType = self._actionType.getPropertyType(name)
            result = CActionProperty(self._actionType, type=propertyType)
            self._propertiesByName[propertyType.name] = result
            self._propertiesById[propertyType.id] = result
            self._properties.append(result)
            self._properties.sort(key=lambda prop:prop._type.idx)
        return result


    def getPropertyById(self, id):
        result = self._propertiesById.get(id, None)
        if not result:
            propertyType = self._actionType.getPropertyTypeById(id)
            result = CActionProperty(self._actionType, type=propertyType)
            self._propertiesByName[propertyType.name] = result
            self._propertiesById[propertyType.id] = result
            self._properties.append(result)
            self._properties.sort(key=lambda prop:prop._type.idx)
        return result


    def getPropertyByIndex(self, index):
        return self._properties[index]


    def getExecutionPlan(self):
        return self._executionPlan

    def getDataForNewExecutionPlanAction(self, begDate, periodicity):
        dt = self._executionPlan.getDataForNewExecutionPlanAction(begDate.toPyDateTime())
        return QDateTime(dt), periodicity

    def getExecutionPlanForNewAction(self, begDate):
        return self._executionPlan.getExecutionPlanForNewAction()

    def addExecutionPlanByRecord(self):
        self._executionPlan.addExecutionPlanByAction()

    def updateDosageFromExecutionPlan(self):
        dosage = self._executionPlan.getActualDosage()
        if dosage is None:
            return

        actionType = self.getType()
        for propertyType in actionType.getPropertiesById().values():
            if propertyType.inActionsSelectionTable == 2:
                self.getPropertyById(propertyType.id).setValue(dosage)
                return

    def minExecutionPlanDate(self):
        return self._executionPlan.minExecutionPlanDate()

    def minExecutionPlanTime(self):
        return self._executionPlan.minExecutionPlanTime()

    def minExecutionPlanTimeByDate(self, date):
        return self._executionPlan.minExecutionPlanTimeByDate(date)

    def minExecutionPlanDateTime(self):
        return self._executionPlan.minExecutionPlanDateTime()

    def isStarted(self):
        return forceInt(self._record.value('status')) == CActionStatus.started

    def defaultExecutionPlanCount(self):
        return self._executionPlan.defaultExecutionPlanCount()

    def updateExecutionPlanByRecord(self, forceDuration=False):
        # Только до того как мы создали запись в БД. Есть id - руки прочь от комиссарского тела.
        if self._record is None or forceRef(self._record.value('id')) is not None:
            return

        self._executionPlan.updateExecutionPlanByAction(forceDuration)

    def setExecutionPlan(self, executionPlan):
        self._executionPlan.set(executionPlan)


    def __getitem__(self, name):
        return self.getProperty(name).getValue()


    def __setitem__(self, name, value):
        property = self.getProperty(name)
        property.setValue(value)
#        if property.isActionNameSpecifier():
#            self.setSpecifiedName(property.getText())


    def __delitem__(self, name):
        if self._propertiesByName.has_key(name):
            property = self._propertiesByName[name]
#            if property.isActionNameSpecifier():
#                self.setSpecifiedName(property.getText())
            del self._propertiesByName[name]
            del self._propertiesById[property._type.id]


    def get(self, name, default=None):
        if self.hasProperty(name):
            return self.getProperty(name).getValue()
        else:
            return default


    def updateByTemplate(self, templateId, checkPropsOnOwner=True, clientSex = None, clientAge = None):
        db = QtGui.qApp.db
        templateActionId = forceRef(db.translate('ActionTemplate', 'id', templateId, 'action_id'))
        self.updateByActionId(templateActionId, checkPropsOnOwner, clientSex, clientAge)


    def updateByActionId(self, actionId, checkPropsOnOwner=False, clientSex = None, clientAge = None):
        db = QtGui.qApp.db
        templateRecord = db.getRecord('Action', '*', actionId)
        if templateRecord:
            templateAction = CAction(record=templateRecord)
            self.updateByAction(templateAction, checkPropsOnOwner, clientSex, clientAge)


    def updateByAction(self, templateAction, checkPropsOnOwner=False, clientSex = None, clientAge = None):
        if checkPropsOnOwner:
            setPersonId = forceRef(self.getRecord().value('setPerson_id'))
        self.nomenclatureExpense = templateAction.nomenclatureExpense

        for id, targetPropertyType in self._actionType._propertiesById.iteritems():
            sourceProperty =(targetPropertyType.applicable(clientSex, clientAge) and
                            ( templateAction._propertiesById.get(id) or
                               templateAction._propertiesByName.get(targetPropertyType.name)
                             ))
            canCopy = (targetPropertyType.valueType.isCopyable
                       and sourceProperty
                       and targetPropertyType.valueType.variantType == sourceProperty.type().valueType.variantType
                      )
            if checkPropsOnOwner and canCopy and targetPropertyType.canChangeOnlyOwner == 1:
                canCopy = setPersonId == QtGui.qApp.userId
            if canCopy:
                targetProperty = self.getPropertyById(id)
                targetProperty.copyIfNotEmpty(sourceProperty)


    def clone(self):
        newAction = CAction(actionType=self.getType())
        for id in self._propertiesById:
            property = newAction.getPropertyById(id)
            if property.type().valueType.isCopyable:
                property.copy(self._propertiesById[id])
        return newAction


    def findFireableJobTicketId(self):
        for property in self._properties:
            if property.type().isJobTicketValueType():
                jobTicketId = property.getValue()
                if jobTicketId:
                    return jobTicketId
        return None


    def getTestProperties(self):
        return [property
                for property in self._properties
                if property.type().testId and property.isAssigned()
               ]


    def updateSpecifiedName(self):
        name = ' '.join([forceString(property.getText())
                         for property in self._properties
                         if property.isActionNameSpecifier() and property.getValue()
                        ]
                       )
        self.setSpecifiedName(name)


    def setSpecifiedName(self, name):
        if self._record:
            self._record.setValue('specifiedName', QVariant(name))
        self._specifiedName = name


    def initPresetValues(self):
        actionType = self.getType()
        if actionType:
            propertyTypeList = actionType.getPropertiesById().values()
            for propertyType in propertyTypeList:
                if propertyType.valueType.initPresetValue:
                    prop = self.getPropertyById(propertyType.id)
                    prop.setPresetValue(self)

    def maxPropertiesCourse(self):
        actionType = self.getType()
        if actionType:
            propertyTypeList = actionType.getPropertiesById().values()
            return max(pt.course for pt in propertyTypeList)
        return None

    def initNomenclatureReservation(self, clientId, dosage=None, financeId=None):
        if self.nomenclatureClientReservation is None:
            self.nomenclatureClientReservation = CNomenclatureClientReservation(self, clientId=clientId, financeId=financeId).init(dosage)

    def nomenclatureReservationFromAction(self, anotherAction, financeId=None):
        if self.nomenclatureClientReservation is None:
            self.nomenclatureClientReservation = CNomenclatureClientReservation(self, financeId=financeId).fromAction(anotherAction)

    def updatePresetValuesConditions(self, conditions):
        self._presetValuesConditions.update(conditions)


    def getJobEndDate(self, jobTicketId=None):
        actionType = self.getType()
        if jobTicketId is None:
            propertyTypeList = actionType.getPropertiesById().values() if actionType else []
            for propertyType in propertyTypeList:
                if propertyType.isJobTicketValueType():
                    property = self.getPropertyById(propertyType.id)
                    jobTicketId = property.getValue()
                    break

        if jobTicketId:
            db = QtGui.qApp.db
            date = forceDate(db.translate('Job_Ticket', 'id', jobTicketId, 'datetime'))
            ticketDuration = actionType.ticketDuration
            if ticketDuration:
                return date.addDays(ticketDuration)

            jobId = forceRef(db.translate('Job_Ticket', 'id', jobTicketId, 'master_id'))
            jobTypeId = forceRef(db.translate('Job', 'id', jobId, 'jobType_id'))
            ticketDuration = forceInt(db.translate('rbJobType', 'id', jobTypeId, 'ticketDuration'))
            return date.addDays(ticketDuration)
        return None

    def checkOptimisticLock(self):
        id = self.getId()
        if not id:
            return True

        record = QtGui.qApp.db.getRecord('Action', 'modifyDatetime', id)
        if not record:
            return True

        modifyDatetime = forceDateTime(record.value('modifyDatetime'))

        return modifyDatetime == forceDateTime(self._record.value('modifyDatetime'))


    def finishAction(self, clientId, date=None):
        if not self._record:
            return None

        if date:
            self._record.setValue('endDate', toVariant(date))
        else:
            self._record.setValue('endDate', toVariant(QtCore.QDateTime.currentDateTime()))

        self._record.setValue('status', toVariant(CActionStatus.finished))

        duration = forceInt(self._record.value('duration'))
        if duration < 1:
            return None

        db = QtGui.qApp.db

        record = self._record

        plannedEndDate = forceDateTime(record.value('plannedEndDate'))
        aliquoticity = forceInt(record.value('aliquoticity')) or 1
        begDate = forceDateTime(record.value('begDate'))
        periodicity = forceInt(record.value('periodicty')) or 0
        actionTypeId = forceRef(record.value('actionType_id'))
        specifiedName = forceString(record.value('specifiedName'))
        if QtGui.qApp.userId and QtGui.qApp.userSpecialityId:
            execPersonId = QtGui.qApp.userId
        else:
            execPersonId = forceRef(record.value('setPerson_id'))

        begDate, periodicity = self.getDataForNewExecutionPlanAction(begDate, periodicity)

        record.setValue('person_id', toVariant(execPersonId))
        record.setValue('aliquoticity', toVariant(aliquoticity))
        record.setValue('begDate', toVariant(begDate))

        newExecutionPlan = self.getExecutionPlanForNewAction(begDate)

        if not newExecutionPlan:
            return None

        newExecutionPlanKeys = newExecutionPlan.keys()
        newExecutionPlanKeys.sort()

        newExecDate = newExecutionPlanKeys[0]
        newExecTimeList = newExecutionPlan.get(newExecDate, {})

        if newExecTimeList:
            begDateNew = QDateTime(QDate(newExecDate), newExecTimeList.keys()[0])

        else:
            begDateNew = QDateTime(QDate(newExecDate), QTime(0, 0))

        duration = max(begDateNew.daysTo(plannedEndDate) + 1, 1)

        record.setValue('periodicity', toVariant(periodicity))

        tableAction = db.table('Action')

        newRecord = tableAction.newRecord()
        newRecord.setValue('event_id', toVariant(record.value('event_id')))
        newRecord.setValue('begDate', toVariant(begDateNew))
        newRecord.setValue('status', toVariant(CActionStatus.started))
        newRecord.setValue('specifiedName', toVariant(specifiedName))
        newRecord.setValue('duration', toVariant(duration))
        newRecord.setValue('periodicity', toVariant(periodicity))
        newRecord.setValue('aliquoticity', toVariant(aliquoticity))
        newRecord.setValue('plannedEndDate', toVariant(plannedEndDate))
        newRecord.setValue('actionType_id', toVariant(actionTypeId))
        newRecord.setValue('directionDate', toVariant(record.value('directionDate')))
        newRecord.setValue('setPerson_id', toVariant(record.value('setPerson_id')))
        newRecord.setValue('person_id', toVariant(
            QtGui.qApp.userId if (QtGui.qApp.userId and QtGui.qApp.userSpecialityId) else execPersonId))
        newRecord.setValue('org_id', toVariant(record.value('org_id')))
        newRecord.setValue('amount', toVariant(record.value('amount')))
        newAction = CAction(record=newRecord)
        if QtGui.qApp.controlSMFinance() == 0:
            newAction.initNomenclature(clientId)
            newAction.nomenclatureReservationFromAction(self)
        else:
            newAction.initNomenclature(clientId, financeId=record.value('finance_id'))
            newAction.nomenclatureReservationFromAction(self, financeId=record.value('finance_id'))
        if newExecutionPlan:
            newAction.setExecutionPlan(newExecutionPlan)

        for property in self._properties:
            type = property.type()
            name = type.name
            if (type.canChangeOnlyOwner > 0 or type.inActionsSelectionTable > 0 or type.isActionNameSpecifier) and name:
                newAction[name] = self[name]
                newAction.getProperty(name).applyDependents(newAction)
                if type.isActionNameSpecifier:
                    newAction.updateSpecifiedName()

        return newAction

    def countIdx(self, eventId):
        db = QtGui.qApp.db

        actionTypeClass = self.getType().class_

        tableAction = db.table('Action')
        tableActionType = db.table('ActionType')

        queryTable = tableAction.innerJoin(tableActionType, tableActionType['id'].eq(tableAction['actionType_id']))

        cond = [
            tableAction['deleted'].eq(0),
            tableAction['event_id'].eq(eventId),
            tableActionType['deleted'].eq(0),
            tableActionType['class'].eq(actionTypeClass)
        ]

        return forceInt(
            QtGui.qApp.db.getMax(queryTable, tableAction['idx'], where=cond)
        )

    def getJobTicketDateTime(self):
        actionType = self.getType()
        propertyTypeList = actionType.getPropertiesById().values() if actionType else []
        for propertyType in propertyTypeList:
            if propertyType.isJobTicketValueType():
                property = self.getPropertyById(propertyType.id)
                jobTicketId = property.getValue()
                if jobTicketId:
                    db = QtGui.qApp.db
                    table = db.table('Job_Ticket')
                    record = db.getRecordEx(table, [table['datetime']], [table['id'].eq(jobTicketId)])
                    if record:
                        return forceDateTime(record.value('datetime'))
        return None


    @classmethod
    def getFilledAction(cls, eventEditor, record, actionTypeId,
                        amount=None, financeId=None, contractId=None, initPresetValues=True):

        cls.preFillingActionRecord(eventEditor, record, actionTypeId, amount, financeId, contractId)
        if actionTypeId is None:
            return None

        action = cls(record=record)
        action.fillAction(eventEditor, initPresetValues=initPresetValues)
        return action


    @staticmethod
    def preFillingActionRecord(eventEditor, record, actionTypeId, amount, financeId, contractId):
        # сюда - инициализация, которая никак не зависит от свойств действия
        if actionTypeId:
            actionType = CActionTypeCache.getById(actionTypeId)
            defaultStatus = actionType.defaultStatus
            defaultDirectionDate = actionType.defaultDirectionDate
            defaultBegDate = actionType.defaultBegDate
            defaultEndDate = actionType.defaultEndDate
            defaultExecPersonId = actionType.defaultExecPersonId
            defaultPerson = actionType.defaultPersonInEvent
            defaultMKB = actionType.defaultMKB
            defaultOrgId = actionType.defaultOrgId
            office = actionType.office
            record.setValue('actionType_id', toVariant(actionTypeId))
        else:
            defaultStatus = CActionStatus.finished  # Закончено
            defaultDirectionDate = CActionType.dddUndefined
            defaultBegDate = 0
            defaultEndDate = CActionType.dedEventExecDate  # Дата события
            defaultExecPersonId = None
            defaultPerson = CActionType.dpEmpty
            defaultMKB = None
            defaultOrgId = None
            office = ''

        if defaultEndDate == CActionType.dedEventExecDate:
            endDate = forceDateTime(eventEditor.eventDate)
        elif defaultEndDate == CActionType.dedEventSetDate:
            endDate = eventEditor.eventSetDateTime
        elif defaultEndDate == CActionType.dedCurrentDate:
            endDate = QDateTime.currentDateTime()
        else:
            if defaultStatus in (CActionStatus.finished, CActionStatus.withoutResult):
                endDate = QDateTime.currentDateTime()
            else:
                endDate = QDateTime()

        begDate = eventEditor.eventSetDateTime
        if defaultDirectionDate == CActionType.dddEventSetDate:
            directionDate = eventEditor.eventSetDateTime
        elif defaultDirectionDate == CActionType.dddCurrentDate:
            directionDate = QDateTime.currentDateTime()
        elif defaultDirectionDate == CActionType.dddActionExecDate:
            if endDate:
                if endDate < eventEditor.eventSetDateTime:
                    directionDate = eventEditor.eventSetDateTime
                    begDate = QDateTime()
                else:
                    directionDate = begDate = endDate
            else:
                directionDate = eventEditor.eventSetDateTime
        else:
            directionDate = eventEditor.eventSetDateTime

        if defaultBegDate == CActionType.dbdEventExecDate:
            begDate = forceDateTime(eventEditor.eventDate)
        elif defaultBegDate == CActionType.dbdEventSetDate:
            begDate = eventEditor.eventSetDateTime
        elif defaultBegDate == CActionType.dbdCurrentDate:
            begDate = QDateTime.currentDateTime()

        if defaultEndDate == CActionType.dedActionBegDate or defaultEndDate == CActionType.dedSyncActionBegDate:
            endDate = max(begDate, directionDate)

        if getEventTypeForm(eventEditor.eventTypeId) == '001':  # WTF?
            setPersonId = eventEditor.cmbSetPerson.value()
        else:
            setPersonId = eventEditor.getSuggestedPersonId()

        if defaultExecPersonId:
            personId = defaultExecPersonId
        else:
            if defaultPerson == CActionType.dpCurrentMedUser:
                if QtGui.qApp.userSpecialityId:
                    personId = QtGui.qApp.userId
                else:
                    personId = None
            elif defaultPerson == CActionType.dpCurrentUser:
                personId = QtGui.qApp.userId
            elif defaultPerson == CActionType.dpEventExecPerson:
                personId = eventEditor.personId
            elif defaultPerson == CActionType.dpSetPerson:
                personId = setPersonId
            else:
                personId = None

        if defaultBegDate == CActionType.dbdActionEndDate:
            begDate = max(endDate, directionDate)

        defaultMKBValue = ''
        defaultMorphologyMKBValue = ''
        if defaultMKB in (CActionType.dmkbByFinalDiag,
                          CActionType.dmkbBySetPersonDiag,
                          CActionType.dmkbSyncFinalDiag,
                          CActionType.dmkbSyncSetPersonDiag,
                          ):
            defaultMKBValue, defaultMorphologyMKBValue = eventEditor.getDefaultMKBValue(defaultMKB, setPersonId)

        record.setValue('directionDate', toVariant(directionDate))
        record.setValue('setPerson_id', toVariant(setPersonId))
        record.setValue('begDate', toVariant(max(begDate, directionDate)))
        record.setValue('endDate', toVariant(endDate))
        record.setValue('status', toVariant(defaultStatus))
        record.setValue('office', toVariant(office))
        record.setValue('person_id', toVariant(personId))
        record.setValue('MKB', toVariant(defaultMKBValue))
        record.setValue('morphologyMKB', toVariant(defaultMorphologyMKBValue))
        record.setValue('org_id', toVariant(defaultOrgId))
        if actionTypeId:
            if not (amount and actionType.amountEvaluation == CActionType.userInput):
                amount = getActionDefaultAmountEx(eventEditor, actionType, record, None)
        else:
            amount = 0
        record.setValue('amount', toVariant(amount))
        if not financeId:
            financeId = eventEditor.getActionFinanceId(record)
        if getEventActionContract(eventEditor.eventTypeId):
            contractId = getActionDefaultContractId(eventEditor, actionTypeId, financeId, begDate, endDate,
                                                    contractId or eventEditor.contractId)
        else:
            contractId = None
        record.setValue('uet',
                        toVariant(amount * eventEditor.getUet(actionTypeId, personId, financeId, contractId)))
        record.setValue('finance_id', toVariant(financeId))
        record.setValue('contract_id', toVariant(contractId))
        if eventEditor._id:
            record.setValue('event_id', toVariant(eventEditor._id))


    def fillAction(self, eventEditor, initPresetValues=True):
        self.updatePresetValuesConditions({'clientId': eventEditor.clientId,
                                           'eventTypeId': eventEditor.eventTypeId})
        if initPresetValues:
            self.initPresetValues()
        self.finalFillingActionRecord()
        if QtGui.qApp.controlSMFinance() == 0:
            self.initNomenclature(eventEditor.clientId)
        else:
            self.initNomenclature(eventEditor.clientId, financeId=self._financeId)


    def finalFillingActionRecord(self):
        # сюда - инициализация, которая может зависеть от свойств
        record = self.getRecord()
        directionDate = forceDate(record.value('directionDate'))
        plannedEndDate = None
        if directionDate:
            defaultPlannedEndDate = self.getType().defaultPlannedEndDate
            if defaultPlannedEndDate == CActionType.dpedNextDay:
                plannedEndDate = directionDate.addDays(1)
            elif defaultPlannedEndDate == CActionType.dpedNextWorkDay:
                plannedEndDate = addWorkDays(directionDate, 1, wpFiveDays)
            elif defaultPlannedEndDate == CActionType.dpedJobTicketDate:
                plannedEndDate = self.getJobEndDate()
        actionType = self._actionType
        if actionType:
            defaultBegDate = actionType.defaultBegDate
            if defaultBegDate == CActionType.dbdJobTicketTime:
                dateTimeJT = self.getJobTicketDateTime()
                if dateTimeJT:
                    record.setValue('begDate', toVariant(dateTimeJT))
        record.setValue('plannedEndDate', toVariant(plannedEndDate))

    def setPlannedEndDateOnJobTicketChanged(self, jobTicketId):
        # Если плановая дата выполнения действия зависит от номерка на работу согласно умолчаниям
        record = self.getRecord()
        defaultPlannedEndDate = self.getType().defaultPlannedEndDate
        if defaultPlannedEndDate == CActionType.dpedJobTicketDate and forceDate(record.value('directionDate')):
            if jobTicketId:
                plannedEndDate = self.getJobEndDate(jobTicketId)
            else:
                plannedEndDate = None

            record.setValue('plannedEndDate', toVariant(plannedEndDate))

    def getCreateDateTime(self):
        if self._record:
            return forceDateTime(self._record.value('createDatetime'))
        return None


def getActionDefaultAmountEx(eventEditor, actionType, record, action):
    result = actionType.amount
    if actionType.amountEvaluation == CActionType.eventVisitCount:
        result = result * eventEditor.getVisitCount()
    elif actionType.amountEvaluation == CActionType.eventDurationWithFiveDayWorking:
        result = result * eventEditor.getEventDuration(wpFiveDays)
    elif actionType.amountEvaluation == CActionType.eventDurationWithSixDayWorking:
        result = result * eventEditor.getEventDuration(wpSixDays)
    elif actionType.amountEvaluation == CActionType.eventDurationWithSevenDayWorking:
        result = result * eventEditor.getEventDuration(wpSevenDays)
    elif actionType.amountEvaluation == CActionType.actionDurationWithFiveDayWorking:
        result = result * getActionDuration(eventEditor.eventTypeId, record, wpFiveDays)
    elif actionType.amountEvaluation == CActionType.actionDurationWithSixDayWorking:
        result = result * getActionDuration(eventEditor.eventTypeId, record, wpSixDays)
    elif actionType.amountEvaluation == CActionType.actionDurationWithSevenDayWorking:
        result = result * getActionDuration(eventEditor.eventTypeId, record, wpSevenDays)
    elif actionType.amountEvaluation == CActionType.actionFilledPropsCount:
        result = result * action.getFilledPropertiesCount() if action else 0
    elif actionType.amountEvaluation == CActionType.actionAssignedPropsCount:
        result = result * action.getAssignedPropertiesCount() if action else 0
    return result


def getActionDuration(eventTypeId, record, weekProfile):
    if record:
        startDate = forceDate(record.value('begDate'))
        stopDate  = forceDate(record.value('endDate'))
        if startDate and stopDate:
            return getEventDuration(startDate, stopDate, weekProfile, eventTypeId)
    return 0


def getActionDefaultContractId(eventEditor, actionTypeId, financeId, begDate, endDate, contractId):
    model = CContractDbModel(None)
    try:
        model.setOrgId(eventEditor.orgId)
        model.setEventTypeId(eventEditor.eventTypeId)
        model.setClientInfo(eventEditor.clientId,
                            eventEditor.clientSex,
                            eventEditor.clientAge,
                            eventEditor.clientWorkOrgId,
                            eventEditor.clientPolicyInfoList)
        model.setFinanceId(financeId)
        model.setActionTypeId(actionTypeId)
        model.setBegDate(begDate or QtCore.QDate.currentDate())
        model.setEndDate(endDate or QtCore.QDate.currentDate())
        model.initDbData()
        return contractId if contractId and model.searchId(contractId)>=0 else model.getId(0)
    finally:
        sip.delete(model)


####################################################
#TODO вынести в отдельный модуль
class CNomenclatureClientReservation(object):
    _RECIPE_PROPERTY = 1

    def __init__(self, action, clientId=None, financeId=None):
        self._tableStockMotion = QtGui.qApp.db.table('StockMotion')
        self._tableStockMotionItem = QtGui.qApp.db.table('StockMotion_Item')
        self._linkTable = QtGui.qApp.db.table('Action_NomenclatureReservation')
        self._action = action
        self._clientId = clientId
        self._financeId = financeId
        self._record = None
        self._items = []
        self._actionId = action.getId()
        self.markToDelete = False

    def cancel(self):
        if not self._record:
            return
        reservationId = forceRef(self._record.value('id'))
        if not reservationId:
            return

        QtGui.qApp.db.deleteRecord(self._tableStockMotion, self._tableStockMotion['id'].eq(reservationId))

    def releaseNomenclature(self, clientStockMotionItem):
        nomenclatureId = forceRef(clientStockMotionItem.value('nomenclature_id'))
        csmQnt = forceDouble(clientStockMotionItem.value('qnt'))
        unitId = forceRef(clientStockMotionItem.value('unit_id'))

        for item in self._items:
            if nomenclatureId != forceRef(item.value('nomenclature_id')):
                continue

            ratio = getNomenclatureUnitRatio(nomenclatureId, unitId, forceRef(item.value('unit_id')))

            csmQnt *= ratio

            reservedQnt = forceDouble(item.value('qnt'))
            if csmQnt > 0:
                if reservedQnt!=0:
                    if reservedQnt < csmQnt:
                        csmQnt = reservedQnt
                        clientStockMotionItem.setValue('qnt', toVariant(csmQnt / ratio))

                    item.setValue('qnt', toVariant(reservedQnt-csmQnt))
                    csmQnt = csmQnt - reservedQnt

                    QtGui.qApp.db.updateRecord(self._tableStockMotionItem, item)

    def _calcQnt(self, dosageValue=None):
        actionRecord = self._action.getRecord()
        total = (forceInt(actionRecord.value('duration')) or 1)
        perodicity = forceInt(actionRecord.value('periodicity'))
        if not perodicity:
            qnt = total * (forceInt(actionRecord.value('aliquoticity')) or 1)
        else:
            qnt = len(range(perodicity, total + perodicity, perodicity + 1)) * (
            forceInt(actionRecord.value('aliquoticity')) or 1)

        if qnt == 0:
            return None

        doses = None
        for properties in self._action._properties:
            type = properties._type
            name = type.name
            if type.inActionsSelectionTable == 2:  # doses
                property = self._action.getProperty(name)
                doses = property.getValue()
                if isinstance(doses, (float, int)):
                    break
                else:
                    doses = None

        coeff = 1
        if dosageValue:
            coeff = doses / dosageValue

        return qnt * coeff

    def init(self, dosage=None):
        executionPlan = self._action.getExecutionPlan()
        if not executionPlan or not self._clientId:
            return None

        db = QtGui.qApp.db

        stockMotionId = None

        stockService = None

        db.transaction()

        try:
            actionRecord = self._action.getRecord()
            actionType = self._action.getType()
            propertyTypeList = actionType.getPropertiesById().values() if actionType else []
            for propertyType in propertyTypeList:
                if propertyType.isNomenclatureValueType() and propertyType.inActionsSelectionTable == self._RECIPE_PROPERTY:
                    property = self._action.getPropertyById(propertyType.id)
                    nomenclatureId = property.getValue()
                    if not nomenclatureId:
                        continue

                    nomenclatureRecord = db.getRecord(
                        'rbNomenclature', 'defaultClientUnit_id, defaultStockUnit_id, dosageValue', nomenclatureId
                    )
                    dosageValue = forceDouble(nomenclatureRecord.value('dosageValue'))
                    clientUnitId = forceRef(nomenclatureRecord.value('defaultClientUnit_id'))
                    stockUnitId = forceRef(nomenclatureRecord.value('defaultStockUnit_id'))

                    if not stockMotionId:
                        # TODO Это все через сервис
                        self._record = stockMotionRecord = self._tableStockMotion.newRecord()
                        currentDateTime = QDateTime.currentDateTime()
                        stockMotionRecord.setValue('type', toVariant(6))  # client reservation type
                        stockMotionRecord.setValue('client_id', toVariant(self._clientId))
                        stockMotionRecord.setValue('supplier_id', toVariant(QtGui.qApp.currentOrgStructureId()))
                        stockMotionRecord.setValue('date', toVariant(currentDateTime))
                        stockMotionRecord.setValue('reasonDate', toVariant(currentDateTime))
                        stockMotionRecord.setValue(
                            'supplierPerson_id',
                            toVariant(
                                QtGui.qApp.userId if QtGui.qApp.userSpecialityId
                                else forceRef(actionRecord.value('person_id'))
                            )
                        )

                        stockMotionId = db.insertRecord(self._tableStockMotion, stockMotionRecord)

                        stockMotion = CStockService.getStockMotionByRecord(stockMotionRecord)
                        stockService = CStockService(stockMotion)

                    if dosage is None:
                        qnt = self._calcQnt(dosageValue)
                    elif dosageValue:
                        qnt = dosage/dosageValue
                    else:
                        qnt = self._calcQnt()

                    while qnt!=0:

                        itemRecord = self._tableStockMotionItem.newRecord()
                        stockMotionItem = stockService.getStockMotionItemByRecord(itemRecord)
                        stockMotionItem.nomenclature_id = nomenclatureId
                        stockMotionItem.unit_id = clientUnitId

                        stockMotionItem.sum = self.calcSum(itemRecord, stockUnitId)
                        stockMotionItem.note = u''
                        stockService.setFinanceBatchShelfTime(stockMotionItem, self._financeId)

                        existQnt = getExistsNomenclatureAmount(nomenclatureId, financeId=self._financeId, batch=stockMotionItem.batch, orgStructureId=QtGui.qApp.currentOrgStructureId(), unitId=clientUnitId)
                        if existQnt:
                            stockMotionItem.qnt = existQnt if existQnt <= qnt else qnt

                            db.insertRecord(self._tableStockMotionItem, itemRecord)
                            self._items.append(itemRecord)
                            qnt = qnt - (existQnt if existQnt <= qnt else qnt)
                        else:
                            qnt = 0


        except:
            db.rollback()
            raise

        else:
            db.commit()

        if not stockMotionId:
            return None

        return self

    def getRatio(self, nomenclatureId, oldUnitId, newUnitId):
        if oldUnitId == newUnitId:
            return 1

        ratio = getNomenclatureUnitRatio(nomenclatureId, oldUnitId, newUnitId)
        if ratio is None:
            raise ValueError()

        return ratio

    def calcSum(self, item, stockUnitId):
        from Stock.Utils import CStockCache

        nomenclatureId = forceRef(item.value('nomenclature_id'))
        stockPrice = CStockCache().getPrice(QtGui.qApp.currentOrgStructureId(), nomenclatureId, None)

        qnt = forceDouble(item.value('qnt'))
        unit_id = forceRef(item.value('unit_id'))
        ratio = self.getRatio(nomenclatureId, unit_id, stockUnitId)
        stockQnt = qnt*ratio
        return stockQnt*stockPrice


    def linkAction(self, action):
        if self._record is None:
            return None

        elif self.markToDelete:
            if self._actionId:
                QtGui.qApp.db.query(
                    'DELETE FROM {0} WHERE action_id = {1}'.format(self._linkTable.tableName, self._actionId)

                )
            return

        elif self._actionId == action.getId():
            return

        self._actionId = action.getId()

        db = QtGui.qApp.db
        tableStockMotion = db.table('StockMotion')
        cond = [tableStockMotion['id'].eq(self._record.value('id'))]
        exists = db.getRecordEx(tableStockMotion, tableStockMotion['id'], cond)
        if exists:
            linkRecord = self._linkTable.newRecord()
            linkRecord.setValue('action_id', self._actionId)
            linkRecord.setValue('reservation_id', self._record.value('id'))

            db.insertRecord(self._linkTable, linkRecord)


    def fromAction(self, anotherAction):
        if not anotherAction.nomenclatureClientReservation:
            return None

        anotherNCR = anotherAction.nomenclatureClientReservation
        self._record = anotherNCR._record
        self._items = anotherNCR._items

        anotherNCR.markToDelete = True

        return self


    def load(self):
        actionId = self._action.getId()
        if actionId:
            db = QtGui.qApp.db
            reservation_id = db.translate(self._linkTable, 'action_id', actionId, 'reservation_id')
            if not reservation_id:
                return None

            self._record = db.getRecord(self._tableStockMotion, '*', reservation_id)
            if not self._record:
                return None

            self._clientId = forceRef(self._record.value('client_id'))
            if not self._clientId:
                db.deleteRecord(self._tableStockMotionItem, self._record)
                db.query('DELETE FROM {0} WHERE action_id = {1}'.format(self._linkTable.tableName, actionId))
                return None

            self._items = db.getRecordList(
                self._tableStockMotionItem, '*', self._tableStockMotionItem['master_id'].eq(reservation_id)
            )
            return self

        return None


class CNomenclatureExpense:
    def __init__(self, action=None, actionId = None, actionType = None, actionTypeId = None, clientId=None, financeId=None):
        self._tableStockMotion = QtGui.qApp.db.table('StockMotion')
        self._tableStockMotionItem = QtGui.qApp.db.table('StockMotion_Item')
        self._dirty = False
        self._stockMotionId = None
        self._clientId = clientId
        self._financeId = financeId
        self._items = []
        self._exists = {}
        self._stockMotionRecord = None
        self._action = action
        self.set(actionId, actionType, actionTypeId)
        if not self._actionType and action:
            self._actionType = action.getType()


    def getStockMotionId(self):
        return self._stockMotionId


    def setClientId(self, clientId):
        if self._stockMotionRecord:
            self._stockMotionRecord.setValue('client_id', QVariant(clientId))


    def stockMotionRecord(self):
        return self._stockMotionRecord


    def set(self, actionId = None, actionType = None, actionTypeId = None):
        self._actionId = actionId
        self._actionType = actionType
        if actionId:
            if QtGui.qApp.controlSMFinance() != 0:
                self._financeId = self.setFinanceId(actionId)
            self._clientId = self.getClientId(actionId)
            self._load(actionId)
            return
        if actionType:
            self._setActionType(actionType)
        elif actionTypeId:
            actionType = CActionTypeCache.getById(actionTypeId)
            self._setActionType(actionType)
        else:
            raise CException(u'Неправильные настройки списания ЛСиИМН')


    def setFinanceId(self, actionId):
        db = QtGui.qApp.db
        tableAction = db.table('Action')
        cond = [tableAction['id'].eq(actionId)]
        record = db.getRecordEx(tableAction, tableAction['finance_id'], cond)
        if record:
            return forceInt(record.value('finance_id'))
        return None

    def getClientId(self, actionId):
        db = QtGui.qApp.db
        tableAction = db.table('Action')
        tableEvent = db.table('Event')
        queryTable = tableAction
        queryTable = queryTable.leftJoin(tableEvent, tableEvent['id'].eq(tableAction['event_id']))
        cond = [tableAction['id'].eq(actionId)]
        record = db.getRecordEx(queryTable, tableEvent['client_id'], cond)
        if record:
            return forceInt(record.value('client_id'))
        return None

    def getNewRecord(self):
        record = self._tableStockMotion.newRecord()
        record.setValue('type', toVariant(4)) # списание на пациента
        record.setValue('client_id', toVariant(self._clientId))
        if self._actionType.nomenclatureCounterId:
            record.setValue('number', toVariant(
                    QtGui.qApp.getDocumentNumber(self._clientId, self._actionType.nomenclatureCounterId)))
        record.setValue('date', toVariant(QDateTime.currentDateTime()))
        record.setValue('supplier_id', toVariant(QtGui.qApp.currentOrgStructureId()))
        record.setValue('supplierPerson_id', toVariant(QtGui.qApp.userId if QtGui.qApp.userSpecialityId else forceRef(self._action.getRecord().value('person_id'))))
        return record


    def stockMotionItems(self):
        return self._items

    @staticmethod
    def _getExists(nomenclatureIdList, filter={}):
        return getExistsNomenclature(nomenclatureIdList, filter)


    @staticmethod
    def _sync(exists, targetRecord, unset=False, translateExistsQnt=False, financeId=None, clientId=None):
        return syncNomenclature(exists, targetRecord, unset, translateExistsQnt, financeId=financeId, clientId=clientId)


    def _setActionType(self, actionType):
        def _getNomenclatureIdList(recordList):
            return [forceRef(record.value('nomenclature_id')) for record in recordList]

        _defaultsCache = {}

        def _getDefaultUnitId(nomenclatureId):
            if not nomenclatureId:
                return None
            if nomenclatureId not in _defaultsCache:
                unitId = forceRef(
                    QtGui.qApp.db.translate('rbNomenclature', 'id', nomenclatureId, 'defaultClientUnit_id')
                )
                _defaultsCache[nomenclatureId] = unitId
            return _defaultsCache[nomenclatureId]


        self._stockMotionRecord = self.getNewRecord()
        self._items = []

        nomenclatureRecordList = actionType.getNomenclatureRecordList()
        exists = self._getExists(_getNomenclatureIdList(nomenclatureRecordList), filter={'financeId': self._financeId, 'clientId':self._clientId})
        for idx, nomenclativeRecord in enumerate(nomenclatureRecordList):
            stockMotionItem = QtGui.qApp.db.table('StockMotion_Item').newRecord()
            nomenclatureId = forceRef(nomenclativeRecord.value('nomenclature_id'))
            amount = forceDouble(nomenclativeRecord.value('amount'))
#            writeoffTime = forceInt(nomenclativeRecord.value('writeoffTime'))
            stockMotionItem.setValue('nomenclature_id', toVariant(nomenclatureId))
            stockMotionItem.setValue('qnt', toVariant(amount))
            stockMotionItem.setValue('unit_id', toVariant(_getDefaultUnitId(nomenclatureId)))
            self._sync(exists, stockMotionItem, translateExistsQnt=True, financeId=self._financeId, clientId=self._clientId)
            self._items.append(stockMotionItem)
        self._exists = exists

    def updateNomenclatureDosageValue(self, nomenclatureId, dosageValue):
        if not nomenclatureId:
            return

        target = None
        for item in self._items:
            if nomenclatureId == forceRef(item.value('nomenclature_id')):
                target = item
                break

        if not target:
            return

        nomenclatureRecord = QtGui.qApp.db.getRecord(
            'rbNomenclature', 'defaultClientUnit_id, dosageValue', nomenclatureId
        )

        defaultValue = forceDouble(nomenclatureRecord.value('dosageValue'))
        if defaultValue:
            qnt = dosageValue / defaultValue
            target.setValue('qnt', toVariant(qnt))


    def addNomenclature(self, nomenclatureId, financeId=None):
        if not nomenclatureId:
            return

        if financeId:
            self._financeId = financeId

        for item in self._items:
            if nomenclatureId == forceRef(item.value('nomenclature_id')):
                return

        if not self._stockMotionRecord:
            self._stockMotionRecord = self.getNewRecord()

        nomenclatureRecord = QtGui.qApp.db.getRecord(
            'rbNomenclature', 'defaultClientUnit_id, dosageValue', nomenclatureId
        )
        unitId = forceRef(forceRef(nomenclatureRecord.value('defaultClientUnit_id')))
        dosageValue = forceDouble(nomenclatureRecord.value('dosageValue'))
        qnt = 1
        if dosageValue:
            for properties in self._action._properties:
                type = properties._type
                name = type.name
                if type.inActionsSelectionTable == 2:  # doses
                    property = self._action.getProperty(name)
                    doses = property.getValue()
                    if isinstance(doses, (float, int)):
                        qnt = doses / dosageValue
                        break

        stockMotionItem = self._tableStockMotionItem.newRecord()
        stockMotionItem.setValue('nomenclature_id', toVariant(nomenclatureId))
        stockMotionItem.setValue('qnt', toVariant(qnt))
        stockMotionItem.setValue('unit_id', toVariant(unitId))

        self._applyBatchFinanceIdShelfTime(stockMotionItem)

        self._items.insert(0, stockMotionItem)

    def _applyBatchFinanceIdShelfTime(self, item):
        qnt = forceDouble(item.value('qnt'))
        nomenclatureId = forceRef(item.value('nomenclature_id'))
        unitId = forceRef(item.value('unit_id'))

        qnt = applyNomenclatureUnitRatio(qnt, nomenclatureId, unitId, revert=True)

        orgStructureId = QtGui.qApp.currentOrgStructureId()

        if QtGui.qApp.controlSMFinance() in (1, 2):
            self._financeId = self._action.getFinanceId()

        financeId, batch, shelfTime = findFinanceBatchShelfTime(orgStructureId, nomenclatureId, qnt, clientId=self._clientId, financeId=self._financeId)

        item.setValue('finance_id', toVariant(financeId))
        item.setValue('batch', toVariant(batch))
        item.setValue('shelfTime', toVariant(shelfTime))

    def removeNomenclature(self, nomenclatureId):
        if nomenclatureId:
            for item in self._items:
                if forceRef(item.value('nomenclature_id')) == nomenclatureId:
                    self._items.remove(item)
                    break


    def _load(self, actionId):
        db = QtGui.qApp.db
        self._stockMotionId = forceRef(db.translate('Action', 'id', actionId, 'stockMotion_id'))
        if self._stockMotionId:
            self._stockMotionRecord = db.getRecord(self._tableStockMotion, '*', self._stockMotionId)
            self._items = db.getRecordList(self._tableStockMotionItem, '*', 'master_id=%d'%self._stockMotionId)
            return bool(self._stockMotionRecord)
        else:
            self._setActionType(self._actionType)
            for properties in self._action._properties:
                type = properties._type
                name = type.name
                if type.isNomenclatureValueType():
                    property = self._action.getProperty(name)
                    nomencalureId = property.getValue()
                    if nomencalureId:
                        self.addNomenclature(nomencalureId)
        return False


    def _getReason(self):
        if self._actionType and self._actionType.generateStockMotionReason:
            actionId = forceRef(self._action.getRecord().value('id'))
            if actionId:
                return u'/'.join([self._actionType.code, unicode(actionId)])
        return None


    def _generateStockMotionNumber(self):
        if not self._stockMotionRecord or not QtGui.qApp.counterController():
            return

        number = forceString(self._stockMotionRecord.value('number'))
        if number:
            return

        counterId = getStockMotionNumberCounterId(4) #списание на пациента
        if not counterId:
            return

        number = QtGui.qApp.getDocumentNumber(None, counterId, date=QtCore.QDate.currentDate())

        self._stockMotionRecord.setValue('number', number)


    def save(self):
        if self._stockMotionRecord:
            self._generateStockMotionNumber()
            db = QtGui.qApp.db
            if not forceStringEx(self._stockMotionRecord.value('reason')):
                reason = self._getReason()
                if reason:
                    self._stockMotionRecord.setValue('reason', toVariant(reason))
            self._stockMotionId = db.insertOrUpdate(self._tableStockMotion, self._stockMotionRecord)
            self._releaseReservation()
            stockMotionItemIdList = []
            for stockMotionItem in self._items:
                stockMotionItem.setValue('master_id', toVariant(self._stockMotionId))
                self._applyBatchFinanceIdShelfTime(stockMotionItem)
                stockMotionItemIdList.append(db.insertOrUpdate(self._tableStockMotionItem, stockMotionItem))
            filter = [self._tableStockMotionItem['master_id'].eq(self._stockMotionId),
                      'NOT ('+self._tableStockMotionItem['id'].inlist(stockMotionItemIdList)+')']
            db.deleteRecord(self._tableStockMotionItem, filter)
            return self._stockMotionId

    def _releaseReservation(self):
        if not self._action.nomenclatureClientReservation:
            return

        actionId = self._action.getId()
        if not actionId:
            return

        for item in self._items:
            if forceRef(item.value('id')):
                continue

            self._action.nomenclatureClientReservation.releaseNomenclature(item)



def initActionProperties(action):
    actionType = action.getType()
    propertyTypeList = actionType.getPropertiesById().values()
    for propertyType in propertyTypeList:
        action.getPropertyById(propertyType.id)


####################################################

#def ensureActionTypePresence(actionTypeCode, propTypeList=[]):
#    db = QtGui.qApp.db
#    tableActionType = db.table('ActionType')
#    tablePropertyType = db.table('ActionPropertyType')
#    actionTypeRecord = db.getRecordEx(tableActionType, '*', tableActionType['code'].eq(actionTypeCode))
#    if actionTypeRecord:
#        actionTypeId = forceRef(actionTypeRecord.value('id'))
#    else:
#        actionTypeRecord = tableActionType.newRecord()
#        actionTypeRecord.setValue('code',  toVariant(actionTypeCode))
#        actionTypeId = db.insertOrUpdate(tableActionType, actionTypeRecord)
#    if propTypeList:
#        for propTypeDef in propTypeList:
#            name = propTypeDef.get('name', None)
#            descr = propTypeDef.get('descr', None)
#            unit = propTypeDef.get('unit', None)
#            typeName = propTypeDef.get('typeName', 'String')
#            valueDomain = propTypeDef.get('valueDomain', None)
#            isVector = propTypeDef.get('isVector', False)
#            norm = propTypeDef.get('norm', None)
#            sex = propTypeDef.get('sex', None)
#            age = propTypeDef.get('age', None)
#
#            propertyTypeRecord = db.getRecordEx(tablePropertyType, '*', [tablePropertyType['actionType_id'].eq(actionTypeId), tablePropertyType['name'].eq(name)])
#            if propertyTypeRecord:
#                pass
#            else:
#                propertyTypeRecord = tablePropertyType.newRecord()
#                propertyTypeRecord.setValue('actionType_id', toVariant(actionTypeId))
#                propertyTypeRecord.setValue('name', toVariant(name))
#                propertyTypeRecord.setValue('descr', toVariant(descr))
#                if unit:
#                    unitId = db.translate('rbUnit', 'code', unit, 'id')
#                    if unitId:
#                        propertyTypeRecord.setValue('descr', unitId)
#                propertyTypeRecord.setValue('typeName', toVariant(CActionPropertyValueTypeRegistry.normTypeName(typeName)))
#                propertyTypeRecord.setValue('valueDomain', toVariant(valueDomain))
#                propertyTypeRecord.setValue('isVector',    toVariant(isVector))
#                propertyTypeRecord.setValue('norm',        toVariant(norm))
#                propertyTypeRecord.setValue('sex',         toVariant(sex))
#                propertyTypeRecord.setValue('age',         toVariant(age))
#                db.insertRecord(tablePropertyType, propertyTypeRecord)

####################################################



####################################################################
# сценарий работы c Actions:
#
# 1) создание Action
#    a1 = CAction.createByTypeCode(10)
# 2) загрузка Action
#    a2 = CAction(record=query.record())
# 3) сохранение Action
#    a1.save(eventId)
#    a2.save()
# 4) доступ к свойствам Action
#    x = a2['value']
# 5) изменение свойства Action
#    a2['value'] = x
# 6) очистка/удаление свойства Action
#    del a2['value']
# 7) векторные свойства?
