# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

# Редактор одного action
import sip
from PyQt4 import QtGui
from PyQt4.QtCore import Qt, QDate, QDateTime, QMimeData, QString, QTime, QVariant, pyqtSignature, SIGNAL

from library.Attach.AttachAction     import getAttachAction
from library.Attach.AttachButton     import CAttachButton
from library.Calendar                import wpFiveDays, wpSixDays, wpSevenDays
from library.Counter                 import CCounterController
from library.ICDUtils                import MKBwithoutSubclassification
from library.interchange             import getDatetimeEditValue, getDoubleBoxValue, getLineEditValue, getRBComboBoxValue, setCheckBoxValue, setDatetimeEditValue, setDoubleBoxValue, setLabelText, setLineEditValue, setRBComboBoxValue
from library.ItemsListDialog         import CItemEditorBaseDialog
from library.PrintInfo               import CInfoContext
from library.PrintTemplates          import applyTemplate, customizePrintButton, getPrintButton
from library.Utils                   import calcAgeTuple, forceDate, forceDateTime, forceInt, forceRef, forceString, formatName, toDateTimeWithoutSeconds, toVariant

from Events.Action                   import CAction, CActionType
from Events.ActionInfo               import CCookedActionInfo
from Events.ActionPropertiesTable    import CActionPropertiesTableModel
from Events.ActionStatus             import CActionStatus
from Events.ActionTemplateChoose     import CActionTemplateChooseButton, CActionTemplateCache
from Events.ActionTemplateSaveDialog import CActionTemplateSaveDialog
from Events.EventInfo                import CEventInfo
from Events.ExecTimeNextActionDialog import CExecTimeNextActionDialog
from Events.ExecutionPlanDialog      import CGetExecutionPlan
from Events.Utils                    import checkAttachOnDate, checkPolicyOnDate, checkTissueJournalStatusByActions, getEventDuration, getEventShowTime, setActionPropertiesColumnVisible, validCalculatorSettings
from Orgs.Orgs                       import selectOrganisation
from Registry.ClientEditDialog       import CClientEditDialog
from Registry.Utils                  import formatClientBanner, getClientInfo
from Stock.ClientInvoiceEditDialog   import CClientInvoiceEditDialog, CClientRefundInvoiceEditDialog
from Users.Rights                    import urAdmin, urCanUseNomenclatureButton, urCopyPrevAction, urEditCoordinationAction, urLoadActionTemplate, urEditOtherpeopleAction, urRegTabWriteRegistry, urRegTabReadRegistry, urSaveActionTemplate

from Events.Ui_ActionEditDialog      import Ui_ActionDialog


class CActionEditDialog(CItemEditorBaseDialog, Ui_ActionDialog):
    _saveAsExecutionPlan = False


    def __init__(self, parent):
# ctor
        CItemEditorBaseDialog.__init__(self, parent, 'Action')
        self.eventId     = None
        self.eventRecord = None
        self._eventExecDate2Change = None
        self.eventTypeId = None
        self.eventSetDate = None
        self.clientId    = None
        self.forceClientId = None
        self.clientSex   = None
        self.clientAge   = None
        self.personId    = None
        self.personSpecialityId = None
        self.newActionId = None
        self.clientInfo = None
        self._mainWindowState = QtGui.qApp.mainWindow.windowState()

# create models
        self.addModels('ActionProperties', CActionPropertiesTableModel(self))

# ui
        self.addObject('actEditClient', QtGui.QAction(u'Открыть регистрационную карточку', self))
        self.addObject('actShowAttachedToClientFiles', getAttachAction('Client_FileAttach',  self))
        self.addObject('btnPrint', getPrintButton(self, ''))
        self.addObject('btnLoadTemplate', CActionTemplateChooseButton(self))
        self.addObject('btnAttachedFiles', CAttachButton(self, u'Прикреплённые файлы'))
        self.btnLoadTemplate.setText(u'Загрузить шаблон')
        self.addObject('btnSaveAsTemplate', QtGui.QPushButton(u'Сохранить шаблон', self))
#        self.addObject('btnLoadPrevAction', QPushButton(u'Копировать из предыдущего', self))
        self.setupUi(self)
        self.setWindowFlags(self.windowFlags() | Qt.WindowMaximizeButtonHint)
        self.setWindowTitleEx(u'Мероприятие')
        self.edtDirectionDate.canBeEmpty(True)
        self.edtEndDate.canBeEmpty(True)
        self.edtBegDate.canBeEmpty(True)
        self.buttonBox.addButton(self.btnPrint, QtGui.QDialogButtonBox.ActionRole)
        self.buttonBox.addButton(self.btnLoadTemplate, QtGui.QDialogButtonBox.ActionRole)
        self.buttonBox.addButton(self.btnSaveAsTemplate, QtGui.QDialogButtonBox.ActionRole)
#        self.buttonBox.addButton(self.btnLoadPrevAction, QtGui.QDialogButtonBox.ActionRole)
        self.buttonBox.addButton(self.btnAttachedFiles, QtGui.QDialogButtonBox.ActionRole)
        self.setModels(self.tblProps, self.modelActionProperties, self.selectionModelActionProperties)
# popup menus
        self.txtClientInfoBrowser.actions.append(self.actEditClient)
        self.txtClientInfoBrowser.actions.append(self.actShowAttachedToClientFiles)
        self.actEditClient.setEnabled(QtGui.qApp.userHasAnyRight([urAdmin, urRegTabWriteRegistry, urRegTabReadRegistry]))

# default values
        self.setupDirtyCather()
        self.setIsDirty(False)

        self.actionTemplateCache = CActionTemplateCache(self, self.cmbPerson)

        action = QtGui.QAction(self)
        self.addObject('actSetLaboratoryCalculatorInfo', action)
        action.setShortcut('F3')
        self.addAction(action)
        self.connect(self.actSetLaboratoryCalculatorInfo, SIGNAL('triggered()'), self.on_actSetLaboratoryCalculatorInfo)
        self.connect(self.btnCloseWidgets, SIGNAL('arrowTypeChanged(bool)'), self.on_arrowTypeChanged)
        self.connect(self.modelActionProperties, SIGNAL('actionAmountChanged(double)'), self.on_actionAmountChanged)
        self.setVisibleBtnCloseWidgets(False)
        self._canUseLaboratoryCalculatorPropertyTypeList = None
# done

    def on_actionAmountChanged(self, value):
        self.edtAmount.setValue(value)


    def on_arrowTypeChanged(self, value):
        self.frmWidgets.setVisible(value)


    def setActionCoordinationEnable(self, isRequiredCoordination):
        canEdit = QtGui.qApp.userHasRight(urEditCoordinationAction) and isRequiredCoordination
        editWidgets = [self.edtCoordDate,
                       self.edtCoordTime,
                       self.lblCoordText
                      ]
        for widget in editWidgets:
            widget.setEnabled(canEdit)


    def setVisibleBtnCloseWidgets(self, value):
        self.btnCloseWidgets.setVisible(value)
        if value:
            self.btnCloseWidgets.applayArrow()


    def setEventDate(self, date):
        eventRecord = self._getEventRecord()
        if eventRecord:
            execDate = forceDate(eventRecord.value('execDate'))
            if not execDate:
                self._eventExecDate2Change = date


    def _getEventRecord(self):
        if not self.eventRecord and self.eventId:
            self.eventRecord = QtGui.qApp.db.getRecordEx('Event', 'execDate, id, isClosed', 'id=%d'%self.eventId)
        return self.eventRecord


    def setReduced(self, value):
        self.txtClientInfoBrowser.setVisible(not value)
        if self.clientInfo is None:
            self.clientInfo = getClientInfo(self.clientId, date=self.edtDirectionDate.date())
        name = formatName(self.clientInfo.lastName, self.clientInfo.firstName, self.clientInfo.patrName)
        self.setWindowTitle(self.windowTitle()+' : '+ name)
        self.setVisibleBtnCloseWidgets(value)


    def exec_(self):
        QtGui.qApp.setCounterController(CCounterController(self))
        QtGui.qApp.setJTR(self)
        try:
            result = CItemEditorBaseDialog.exec_(self)
        finally:
            QtGui.qApp.unsetJTR(self)
        if result:
            QtGui.qApp.delAllCounterValueIdReservation()
        else:
            QtGui.qApp.resetAllCounterValueIdReservation()
        QtGui.qApp.setCounterController(None)
        QtGui.qApp.disconnectClipboard()
        return result


    def setForceClientId(self, clientId):
        self.forceClientId = clientId


    def checkNeedLaboratoryCalculator(self, propertyTypeList, clipboardSlot):
        actualPropertyTypeList = [propType for propType in propertyTypeList if validCalculatorSettings(propType.laboratoryCalculator)]
        if actualPropertyTypeList:
            QtGui.qApp.connectClipboard(clipboardSlot)
        else:
            QtGui.qApp.disconnectClipboard()
        return actualPropertyTypeList


    @pyqtSignature('')
    def on_btnPlanNextAction_clicked(self):
        record = self.getRecord()
        if record:
            dialog = CGetExecutionPlan(self, record, self.action.getExecutionPlan())
            dialog.exec_()
            self.action.setExecutionPlan(dialog.model.executionPlan)


    @pyqtSignature('')
    def on_btnNextAction_clicked(self):
        if self.cmbStatus.value() == CActionStatus.canceled:
            currentDateTime = QDateTime.currentDateTime()
            db = QtGui.qApp.db
            table = db.table('vrbPersonWithSpeciality')
            personId = QtGui.qApp.userId if (QtGui.qApp.userId and QtGui.qApp.userSpecialityId) else self.cmbSetPerson.value()
            record = db.getRecordEx(table, [table['name']], [table['id'].eq(personId)]) if personId else None
            personName = forceString(record.value('name')) if record else ''
            self.edtNote.setText(u'Отменить: %s %s' % (currentDateTime.toString('dd-MM-yyyy hh:mm'), personName))
        else:
            self.newActionId = None
            prevRecord = self.getRecord()
            if prevRecord:
                actionTypeId = forceRef(prevRecord.value('actionType_id')) if prevRecord else None
                plannedEndDate = forceDateTime(prevRecord.value('plannedEndDate')) if prevRecord else None
                duration = self.edtDuration.value()
                aliquoticity = self.edtAliquoticity.value() or 1
                if duration > 0 or aliquoticity > 0:
                    recordList = {}
                    periodicity = self.edtPeriodicity.value()
                    if aliquoticity and actionTypeId:
                        db = QtGui.qApp.db
                        tableAction = db.table('Action')
                        eventId = forceRef(prevRecord.value('event_id'))
                        directionDate = forceDateTime(prevRecord.value('directionDate'))
                        setPersonId = forceRef(prevRecord.value('setPerson_id'))
                        if eventId:
                            showTime = self.action.getType().showTime
                            condAction = [tableAction['deleted'].eq(0),
                                          tableAction['actionType_id'].eq(actionTypeId),
                                          tableAction['event_id'].eq(eventId),
                                          tableAction['directionDate'].eq(directionDate) if showTime else tableAction['directionDate'].dateEq(directionDate),
                                          tableAction['setPerson_id'].eq(setPersonId)
                                          ]
                            records = db.getRecordList(tableAction, 'Action.*', condAction, 'Action.begDate')
                            for recordOld in records:
                                id = forceRef(recordOld.value('id'))
                                begDateOld = forceDateTime(recordOld.value('begDate'))
                                recordList[(id, begDateOld)] = recordOld
                            keysList = recordList.keys()
                            keysList.sort()
                            keyList = keysList[-1]

                            lastRecord = recordList.get(keyList, None)

                            if lastRecord:
                                begDate = forceDateTime(prevRecord.value('begDate'))
                                if QtGui.qApp.userId and QtGui.qApp.userSpecialityId:
                                    execPersonId = QtGui.qApp.userId
                                else:
                                    execPersonId = self.cmbSetPerson.value()
                                if not self._openNomenclatureEditor(requireItems=True):
                                    return
                                dialog = CExecTimeNextActionDialog(self, begDate, execPersonId)
                                try:
                                    if dialog.exec_():
                                        execTime = dialog.getExecTime()
                                        execPersonId = dialog.getExecPersonId()
                                        specifiedName = forceString(prevRecord.value('specifiedName'))
                                        prevRecord.setValue('person_id', toVariant(execPersonId))

                                        begDate, periodicity = self.action.getDataForNewExecutionPlanAction(
                                            begDate, periodicity
                                        )
                                        prevRecord.setValue('aliquoticity',  toVariant(aliquoticity))
                                        prevRecord.setValue('begDate', toVariant(begDate))
                                        prevRecord.setValue('endDate', toVariant(QDateTime(begDate.date(), execTime)))
                                        prevRecord.setValue('status', toVariant(CActionStatus.finished))

                                        newExecutionPlan = self.action.getExecutionPlanForNewAction(begDate)

                                        if newExecutionPlan:
                                            newExecutionPlanKeys = newExecutionPlan.keys()
                                            newExecutionPlanKeys.sort()

                                            newExecDate = newExecutionPlanKeys[0]
                                            newExecTimeList = newExecutionPlan.get(newExecDate, {})

                                            if newExecTimeList:
                                                begDateNew = QDateTime(QDate(newExecDate), min(newExecTimeList.keys()))

                                            else:
                                                begDateNew = QDateTime(QDate(newExecDate), QTime(0, 0))

                                            duration = max(begDateNew.daysTo(plannedEndDate) + 1, 1)
                                        else:
                                            if self.action.nomenclatureClientReservation:
                                                self.action.nomenclatureClientReservation.cancel()
                                            self.setRecordByNext(prevRecord)
                                            # self.saveData()
                                            try:
                                                self._saveAsExecutionPlan = True
                                                self.done(self.cdSave)
                                            finally:
                                                self._saveAsExecutionPlan = False
                                            # self.close()
                                            return

                                        prevRecord.setValue('periodicity',   toVariant(periodicity))


                                        newRecord = tableAction.newRecord()
                                        newRecord.setValue('event_id',      toVariant(prevRecord.value('event_id')))
                                        newRecord.setValue('begDate',       toVariant(begDateNew))
                                        newRecord.setValue('status',        toVariant(CActionStatus.started))
                                        newRecord.setValue('specifiedName', toVariant(specifiedName))
                                        newRecord.setValue('duration',      toVariant(duration))
                                        newRecord.setValue('periodicity',   toVariant(periodicity))
                                        newRecord.setValue('aliquoticity',  toVariant(aliquoticity))
                                        newRecord.setValue('plannedEndDate', toVariant(plannedEndDate))
                                        newRecord.setValue('actionType_id', toVariant(actionTypeId))
                                        newRecord.setValue('directionDate', toVariant(prevRecord.value('directionDate')))
                                        newRecord.setValue('setPerson_id',  toVariant(prevRecord.value('setPerson_id')))
                                        newRecord.setValue('person_id',     toVariant(QtGui.qApp.userId if (QtGui.qApp.userId and QtGui.qApp.userSpecialityId) else self.cmbSetPerson.value()))
                                        newRecord.setValue('org_id',        toVariant(prevRecord.value('org_id')))
                                        newRecord.setValue('amount',        toVariant(prevRecord.value('amount')))
                                        newAction = CAction(record=newRecord)
                                        newAction.initNomenclature(self.clientId)
                                        newAction.nomenclatureReservationFromAction(self.action)
                                        if newExecutionPlan:
                                            newAction.setExecutionPlan(newExecutionPlan)

                                        newAction.updateDosageFromExecutionPlan()

                                        for properties in self.action._properties:
                                            type = properties._type
                                            name = type.name
                                            if (type.canChangeOnlyOwner > 0 or type.inActionsSelectionTable > 0 or type.isActionNameSpecifier) and name:
                                                if not newAction[name]:
                                                    newAction[name] = self.action[name]
                                                newAction.getProperty(name).applyDependents(newAction)
                                                if type.isActionNameSpecifier:
                                                    newAction.updateSpecifiedName()

                                        idx = db.getCount('Action', where='deleted=0 and event_id=%d' % eventId)
                                        self.newActionId = newAction.save(idx=idx)

                                        self.setRecordByNext(prevRecord)

                                        try:
                                            self._saveAsExecutionPlan = True
                                            self.done(self.cdSave)
                                        finally:
                                            self._saveAsExecutionPlan = False
                                finally:
                                    dialog.destroy()
                                    sip.delete(dialog)
                                    del dialog


    def getCurrentTimeAction(self, actionDate):
        currentDateTime = QDateTime.currentDateTime()
        if currentDateTime == actionDate:
            currentTime = currentDateTime.time()
            return currentTime.addSecs(60)
        else:
            return currentDateTime.time()


    def setRecordByNext(self, record):
        self.eventId = forceRef(record.value('event_id'))
        self.eventTypeId = forceRef(QtGui.qApp.db.translate('Event', 'id', self.eventId, 'eventType_id'))
        self.eventSetDate = forceDate(QtGui.qApp.db.translate('Event', 'id', self.eventId, 'setDate'))
        self.idx = forceInt(record.value('idx'))
        self.clientId = forceRef(QtGui.qApp.db.translate('Event', 'id', self.eventId, 'client_id'))
        self.action._record = record
        actionType = self.action.getType()
        setActionPropertiesColumnVisible(actionType, self.tblProps)
        showTime = actionType.showTime
        self.edtDirectionTime.setVisible(showTime)
        self.edtPlannedEndTime.setVisible(showTime)
        self.edtCoordTime.setVisible(showTime)
        self.edtBegTime.setVisible(showTime)
        self.edtEndTime.setVisible(showTime)
        self.lblAssistant.setVisible(actionType.hasAssistant)
        self.cmbAssistant.setVisible(actionType.hasAssistant)
        self.setWindowTitle(actionType.code + '|' + actionType.name)
        setCheckBoxValue(self.chkIsUrgent, record, 'isUrgent')
        setDatetimeEditValue(self.edtDirectionDate,    self.edtDirectionTime,    record, 'directionDate')
        setDatetimeEditValue(self.edtPlannedEndDate,   self.edtPlannedEndTime,   record, 'plannedEndDate')
        setDatetimeEditValue(self.edtCoordDate, self.edtCoordTime, record, 'coordDate')
        setLabelText( self.lblCoordText, record, 'coordText')
        setDatetimeEditValue(self.edtBegDate,          self.edtBegTime,          record, 'begDate')
        setDatetimeEditValue(self.edtEndDate,          self.edtEndTime,          record, 'endDate')
        setRBComboBoxValue(self.cmbStatus,      record, 'status')
        setDoubleBoxValue(self.edtAmount,       record, 'amount')
        setDoubleBoxValue(self.edtUet,          record, 'uet')
        setRBComboBoxValue(self.cmbPerson,      record, 'person_id')
        setRBComboBoxValue(self.cmbSetPerson,   record, 'setPerson_id')
        setLineEditValue(self.edtOffice,        record, 'office')
        setRBComboBoxValue(self.cmbAssistant,   record, 'assistant_id')
        setLineEditValue(self.edtNote,          record, 'note')
        self.edtDuration.setValue(forceInt(record.value('duration')))
        self.edtPeriodicity.setValue(forceInt(record.value('periodicity')))
        self.edtAliquoticity.setValue(forceInt(record.value('aliquoticity')))

        mkbVisible = bool(actionType.defaultMKB)
        mkbEnabled = actionType.defaultMKB in (CActionType.dmkbByFinalDiag,
                                               CActionType.dmkbBySetPersonDiag,
                                               CActionType.dmkbUserInput
                                              )
        self.cmbMKB.setVisible(mkbVisible)
        self.lblMKB.setVisible(mkbVisible)
        self.cmbMKB.setEnabled(mkbEnabled)
        self.cmbMKB.setText(forceString(record.value('MKB')))

        morphologyMKBVisible = mkbVisible and QtGui.qApp.defaultMorphologyMKBIsVisible()
        self.cmbMorphologyMKB.setVisible(morphologyMKBVisible)
        self.lblMorphologyMKB.setVisible(morphologyMKBVisible)
        self.cmbMorphologyMKB.setEnabled(mkbEnabled)
        self.cmbMorphologyMKB.setText(forceString(record.value('morphologyMKB')))

        self.cmbOrg.setValue(forceRef(record.value('org_id')))
        if (self.cmbPerson.value() is None
                and actionType.defaultPersonInEditor in (CActionType.dpUndefined, CActionType.dpCurrentUser, CActionType.dpCurrentMedUser)
                and QtGui.qApp.userSpecialityId):
            self.cmbPerson.setValue(QtGui.qApp.userId)
        self.setPersonId(self.cmbPerson.value())
        self.modelActionProperties.setAction(self.action, self.clientId, self.clientSex, self.clientAge, self.eventTypeId)
        self.modelActionProperties.reset()
        self.tblProps.resizeRowsToContents()

        canEdit = not self.action.isLocked() if self.action else True
        canEditPlannedEndDate = canEdit and actionType.defaultPlannedEndDate != CActionType.dpedBegDatePlusAmount
        self.edtPlannedEndDate.setEnabled(canEditPlannedEndDate)
        self.edtPlannedEndTime.setEnabled(canEditPlannedEndDate)
        if not self.action.nomenclatureExpense:
            self.btnAPNomenclatureExpense.setVisible(False)
        self.on_edtDuration_valueChanged(forceInt(record.value('duration')))
        self.cmbMKB.setFilter('''IF(MKB.endDate IS NOT NULL, MKB.endDate >= %s, 1)'''%(QtGui.qApp.db.formatDate(self.edtBegDate.date())))
        self.setActionCoordinationEnable(actionType.isRequiredCoordination)

    def btnNextActionMustBeEnabled(self):
        return (
            (self.edtDuration.value() >= 1 or self.edtAliquoticity.value() >= 1)
            and not self.edtEndDate.date() and self.cmbStatus.value() != CActionStatus.canceled
        )

    @pyqtSignature('int')
    def on_edtDuration_valueChanged(self, value):
        self.btnNextAction.setEnabled(self.btnNextActionMustBeEnabled())

        if value > 0:
            self.btnPlanNextAction.setEnabled(True)
        else:
            self.btnPlanNextAction.setEnabled(False)
        canEdit = not self.action.isLocked() if self.action else True
        self.edtDirectionDate.setEnabled(bool(not value > 0) and canEdit)
        self.edtDirectionTime.setEnabled(bool(not value > 0) and canEdit)
        self.cmbSetPerson.setEnabled(bool(not value > 0) and canEdit)
        self.edtPlannedEndDate.setEnabled(bool(not value > 0) and canEdit)
        self.edtPlannedEndTime.setEnabled(bool(not value > 0) and canEdit)
        self.edtBegDate.setEnabled(bool(not value > 0) and canEdit)
        self.edtBegTime.setEnabled(bool(not value > 0) and canEdit)
        self.edtEndDate.setEnabled(bool(not value > 0) and canEdit)
        self.edtEndTime.setEnabled(bool(not value > 0) and canEdit)
        actionType = self.action.getType()
        if actionType.defaultPlannedEndDate == CActionType.dpedBegDatePlusDuration:
            begDate = self.edtBegDate.date()
            date = begDate.addDays(value-1) if begDate and value else QDate()
            self.edtPlannedEndDate.setDate(date)


    def on_actSetLaboratoryCalculatorInfo(self):
        result = self.checkNeedLaboratoryCalculator(self.modelActionProperties.propertyTypeList,
                                                    self.on_laboratoryCalculatorClipboard)
        self._canUseLaboratoryCalculatorPropertyTypeList = result
        self.setInfoToLaboratoryCalculatorClipboard()


    def setInfoToLaboratoryCalculatorClipboard(self):
        def chooseLabCode(propType):
            code = propType.laboratoryCalculator
            if code in ('LL*', 'GG*', 'EE*', 'CI*'):
                return '%s%s'%(code, unicode(self.modelActionProperties.action.getPropertyById(propType.id).getValue()))
            else:
                return code
        if self._canUseLaboratoryCalculatorPropertyTypeList:
            propertyTypeList = self._canUseLaboratoryCalculatorPropertyTypeList
            #actual = unicode('; '.join(['('+','.join([forceString(propType.id), propType.laboratoryCalculator, propType.name])+')' for propType in propertyTypeList]))
            actual = unicode('; '.join(['('+','.join([forceString(propType.id),
                chooseLabCode(propType),
                propType.name])+')' for propType in propertyTypeList]))
            mimeData = QMimeData()
            mimeData.setData(QtGui.qApp.inputCalculatorMimeDataType,
                             QString(actual).toUtf8())
            QtGui.qApp.clipboard().setMimeData(mimeData)
            self._mainWindowState = QtGui.qApp.mainWindow.windowState()
            QtGui.qApp.mainWindow.showMinimized()


    def on_laboratoryCalculatorClipboard(self):
        mimeData = QtGui.qApp.clipboard().mimeData()
        baData = mimeData.data(QtGui.qApp.outputCalculatorMimeDataType)
        if baData:
            QtGui.qApp.mainWindow.setWindowState(self._mainWindowState)
            data = forceString(QString.fromUtf8(baData))
            self.modelActionProperties.setLaboratoryCalculatorData(data)

    def destroy(self):
        pass

    def getClientId(self, eventId):
        if self.forceClientId:
            return self.forceClientId
        return forceRef(QtGui.qApp.db.translate('Event', 'id', eventId, 'client_id'))

    def setRecord(self, record):
        CItemEditorBaseDialog.setRecord(self, record)
        self.eventId = forceRef(record.value('event_id'))
        self.eventTypeId = forceRef(QtGui.qApp.db.translate('Event', 'id', self.eventId, 'eventType_id'))
        self.eventSetDate = forceDate(QtGui.qApp.db.translate('Event', 'id', self.eventId, 'setDate'))
        self.idx = forceInt(record.value('idx'))
        self.clientId = self.getClientId(self.eventId)
        self.action = CAction(record=record)
        actionType = self.action.getType()
        setActionPropertiesColumnVisible(actionType, self.tblProps)
        showTime = actionType.showTime
        self.edtDirectionTime.setVisible(showTime)
        self.edtPlannedEndTime.setVisible(showTime)
        self.edtCoordTime.setVisible(showTime)
        self.edtBegTime.setVisible(showTime)
        self.edtEndTime.setVisible(showTime)
        self.lblAssistant.setVisible(actionType.hasAssistant)
        self.cmbAssistant.setVisible(actionType.hasAssistant)
        self.setWindowTitle(actionType.code + '|' + actionType.name)
        setCheckBoxValue(self.chkIsUrgent, record, 'isUrgent')
        setDatetimeEditValue(self.edtDirectionDate,    self.edtDirectionTime,    record, 'directionDate')
        setDatetimeEditValue(self.edtPlannedEndDate,   self.edtPlannedEndTime,   record, 'plannedEndDate')
        setDatetimeEditValue(self.edtCoordDate, self.edtCoordTime, record, 'coordDate')
        setLabelText( self.lblCoordText, record, 'coordText')
        setDatetimeEditValue(self.edtBegDate,          self.edtBegTime,          record, 'begDate')
        setDatetimeEditValue(self.edtEndDate,          self.edtEndTime,          record, 'endDate')
        setRBComboBoxValue(self.cmbStatus,      record, 'status')
        setDoubleBoxValue(self.edtAmount,       record, 'amount')
        setDoubleBoxValue(self.edtUet,          record, 'uet')
        setRBComboBoxValue(self.cmbPerson,      record, 'person_id')
        setRBComboBoxValue(self.cmbSetPerson,   record, 'setPerson_id')
        setLineEditValue(self.edtOffice,        record, 'office')
        setRBComboBoxValue(self.cmbAssistant,   record, 'assistant_id')
        setLineEditValue(self.edtNote,          record, 'note')
        self.edtDuration.setValue(forceInt(record.value('duration')))
        self.edtPeriodicity.setValue(forceInt(record.value('periodicity')))
        self.edtAliquoticity.setValue(forceInt(record.value('aliquoticity')))

        mkbVisible = bool(actionType.defaultMKB)
        mkbEnabled = actionType.defaultMKB in (CActionType.dmkbByFinalDiag,
                                               CActionType.dmkbBySetPersonDiag,
                                               CActionType.dmkbUserInput
                                              )
        self.cmbMKB.setVisible(mkbVisible)
        self.lblMKB.setVisible(mkbVisible)
        self.cmbMKB.setEnabled(mkbEnabled)
        self.cmbMKB.setText(forceString(record.value('MKB')))

        morphologyMKBVisible = mkbVisible and QtGui.qApp.defaultMorphologyMKBIsVisible()
        self.cmbMorphologyMKB.setVisible(morphologyMKBVisible)
        self.lblMorphologyMKB.setVisible(morphologyMKBVisible)
        self.cmbMorphologyMKB.setEnabled(mkbEnabled)
        self.cmbMorphologyMKB.setText(forceString(record.value('morphologyMKB')))

        self.cmbOrg.setValue(forceRef(record.value('org_id')))
#        if not actionType.isRequiredCoordination:
#            self.lblCoordDate.setVisible(False)
#            self.frmCoordDate.setVisible(False)
#            self.lblCoordText.setVisible(False)
        if (self.cmbPerson.value() is None
                and actionType.defaultPersonInEditor in (CActionType.dpUndefined, CActionType.dpCurrentUser, CActionType.dpCurrentMedUser)
                and QtGui.qApp.userSpecialityId):
            self.cmbPerson.setValue(QtGui.qApp.userId)

        self.setPersonId(self.cmbPerson.value())
        self.updateClientInfo()

        self.modelActionProperties.setAction(self.action, self.clientId, self.clientSex, self.clientAge, self.eventTypeId)
        self.modelActionProperties.reset()
        self.tblProps.resizeRowsToContents()

        context = actionType.context if actionType else ''
        customizePrintButton(self.btnPrint, context)
        self.btnAttachedFiles.setAttachedFileItemList(self.action.getAttachedFileItemList())

        if QtGui.qApp.userHasRight(urLoadActionTemplate) and (self.cmbStatus.value() != CActionStatus.finished or not self.cmbPerson.value() or QtGui.qApp.userId == self.cmbPerson.value() or QtGui.qApp.userHasRight(urEditOtherpeopleAction)):
            actionTemplateTreeModel = self.actionTemplateCache.getModel(actionType.id)
            self.btnLoadTemplate.setModel(actionTemplateTreeModel)
        else:
            self.btnLoadTemplate.setEnabled(False)
        self.btnSaveAsTemplate.setEnabled(QtGui.qApp.userHasRight(urSaveActionTemplate))
        self.edtDuration.setEnabled(QtGui.qApp.userHasRight(urCopyPrevAction)
                                    and QtGui.qApp.userId == self.cmbSetPerson.value()
                                    and not self.action.getExecutionPlan()
                                    and self.cmbStatus.value() in (CActionStatus.started, CActionStatus.appointed)
                                    and not self.edtEndDate.date())
        self.edtPeriodicity.setEnabled(QtGui.qApp.userHasRight(urCopyPrevAction)
                                       and QtGui.qApp.userId == self.cmbSetPerson.value()
                                       and not self.action.getExecutionPlan()
                                       and self.cmbStatus.value() in (CActionStatus.started, CActionStatus.appointed)
                                       and not self.edtEndDate.date())
        self.edtAliquoticity.setEnabled(QtGui.qApp.userHasRight(urCopyPrevAction)
                                        and QtGui.qApp.userId == self.cmbSetPerson.value()
                                        and not self.action.getExecutionPlan()
                                        and self.cmbStatus.value() in (CActionStatus.started, CActionStatus.appointed)
                                        and not self.edtEndDate.date())

        canEdit = not self.action.isLocked() if self.action else True
        for widget in (self.edtPlannedEndDate, self.edtPlannedEndTime,
                       self.cmbStatus, self.edtBegDate, self.edtBegTime,
                       self.edtEndDate, self.edtEndTime,
                       self.cmbPerson, self.edtOffice,
                       self.cmbAssistant,
                       self.edtUet,
                       self.edtNote, self.cmbOrg,
                       self.buttonBox.button(QtGui.QDialogButtonBox.Ok)
                      ):
                widget.setEnabled(canEdit)
        self.edtAmount.setEnabled(actionType.amountEvaluation == 0 and canEdit)
        if not QtGui.qApp.userHasRight(urLoadActionTemplate) and not (self.cmbStatus.value() != CActionStatus.finished or not self.cmbPerson.value() or QtGui.qApp.userId == self.cmbPerson.value() or QtGui.qApp.userHasRight(urEditOtherpeopleAction)) and not canEdit:
            self.btnLoadTemplate.setEnabled(False)

        canEditPlannedEndDate = canEdit and actionType.defaultPlannedEndDate not in (CActionType.dpedBegDatePlusAmount,
                                                                                     CActionType.dpedBegDatePlusDuration)
        self.edtPlannedEndDate.setEnabled(canEditPlannedEndDate)
        self.edtPlannedEndTime.setEnabled(canEditPlannedEndDate and bool(self.edtPlannedEndDate.date()))

        self.btnNextAction.setEnabled(self.btnNextActionMustBeEnabled())

        if self.edtDuration.value() > 0:
            self.btnPlanNextAction.setEnabled(True)
        else:
            self.btnPlanNextAction.setEnabled(False)
#        self.initFocus()
        self.on_edtDuration_valueChanged(forceInt(record.value('duration')))
        self.edtBegTime.setEnabled(bool(self.edtBegDate.date()) and canEdit)
        self.edtEndTime.setEnabled(bool(self.edtEndDate.date()) and canEdit)
        self.edtPlannedEndTime.setEnabled(bool(self.edtPlannedEndDate.date()) and canEdit)
        if not self.action.nomenclatureExpense:
            self.btnAPNomenclatureExpense.setVisible(False)
        elif not QtGui.qApp.userHasAnyRight([urCanUseNomenclatureButton]):
            self.btnAPNomenclatureExpense.setEnabled(False)
        self.setActionCoordinationEnable(actionType.isRequiredCoordination)


#    def loadDialogPreferences(self):
#        CItemEditorBaseDialog.loadDialogPreferences(self)


    def getRecord(self):
        record = self.record()
        showTime = self.action.getType().showTime
        eventRecord = self._getEventRecord()
        if eventRecord:
            eventExecDate = forceDate(eventRecord.value('execDate'))
            if not eventExecDate:
                getDatetimeEditValue(self.edtDirectionDate, self.edtDirectionTime, record, 'directionDate', showTime)
        getDatetimeEditValue(self.edtPlannedEndDate, self.edtPlannedEndTime, record, 'plannedEndDate', showTime)
        getDatetimeEditValue(self.edtBegDate, self.edtBegTime, record, 'begDate', showTime)
        getDatetimeEditValue(self.edtEndDate, self.edtEndTime, record, 'endDate', showTime)
        getDatetimeEditValue(self.edtCoordDate, self.edtCoordTime, record, 'coordDate', showTime)
        record.setValue('coordText', QVariant(self.lblCoordText.text()))
        getRBComboBoxValue(self.cmbStatus,      record, 'status')
        getDoubleBoxValue(self.edtAmount,       record, 'amount')
        getDoubleBoxValue(self.edtUet,          record, 'uet')
        getRBComboBoxValue(self.cmbPerson,      record, 'person_id')
        getRBComboBoxValue(self.cmbSetPerson,   record, 'setPerson_id')
        getRBComboBoxValue(self.cmbAssistant,   record, 'assistant_id')
        getLineEditValue(self.edtOffice,        record, 'office')
        getLineEditValue(self.edtNote,          record, 'note')
        record.setValue('MKB', QVariant(self.cmbMKB.text()))
        record.setValue('morphologyMKB', QVariant(self.cmbMorphologyMKB.validText()))
        record.setValue('org_id', QVariant(self.cmbOrg.value()))
        record.setValue('duration', toVariant(self.edtDuration.value()))
        record.setValue('periodicity', toVariant(self.edtPeriodicity.value()))
        record.setValue('aliquoticity', toVariant(self.edtAliquoticity.value()))
        # Это хак: удаляем payStatus из записи
        result = type(record)(record) # copy record
        result.remove(result.indexOf('payStatus'))
        return result


    def saveInternals(self, id):
        # TODO: Почему так? saveInternals должно вызываться уже после того как checkDataEntered вызвано.
        if self.checkDataEntered(secondTry=True):
            id = self.action.save(self.eventId, self.idx, checkModifyDate=False)
            checkTissueJournalStatusByActions([(self.action.getRecord(), self.action)])
            if self.action.getType().closeEvent:
                eventRecord = self._getEventRecord()
                if eventRecord:
                    eventExecDate = forceDate(eventRecord.value('execDate'))
                    if not eventExecDate and self._eventExecDate2Change:
                        eventRecord.setValue('execDate', QVariant(self._eventExecDate2Change))
                        eventRecord.setValue('isClosed', QVariant(1))
                        QtGui.qApp.db.updateRecord('Event', eventRecord)
        return id


    def updateClientInfo(self):
        db = QtGui.qApp.db
        self.clientInfo = getClientInfo(self.clientId, date=self.edtDirectionDate.date())
        self.txtClientInfoBrowser.setHtml(formatClientBanner(self.clientInfo))
        table  = db.table('Client')
        record = db.getRecord(table, '*', self.clientId)
        if record:
            directionDate = self.edtDirectionDate.date()
            self.clientSex       = forceInt(record.value('sex'))
            self.clientBirthDate = forceDate(record.value('birthDate'))
            self.clientAge       = calcAgeTuple(self.clientBirthDate, directionDate)
#            self.actionTemplateCache.reset()
        self.actShowAttachedToClientFiles.setMasterId(self.clientId)


    def updateAmount(self):
        def getActionDuration(weekProfile):
            startDate = self.edtBegDate.date()
            stopDate  = self.edtEndDate.date()
            if startDate and stopDate:
                return getEventDuration(startDate, stopDate, weekProfile, self.eventTypeId)
            else:
                return 0

        def setAmount(amount):
            self.edtAmount.setValue(amount)

        actionType = self.action.getType()

        if actionType.amountEvaluation == CActionType.actionPredefinedNumber:
            setAmount(actionType.amount)
        elif actionType.amountEvaluation == CActionType.actionDurationWithFiveDayWorking:
            setAmount(actionType.amount*getActionDuration(wpFiveDays))
        elif actionType.amountEvaluation == CActionType.actionDurationWithSixDayWorking:
            setAmount(actionType.amount*getActionDuration(wpSixDays))
        elif actionType.amountEvaluation == CActionType.actionDurationWithSevenDayWorking:
            setAmount(actionType.amount*getActionDuration(wpSevenDays))
        elif actionType.amountEvaluation == CActionType.actionFilledPropsCount:
            setAmount(actionType.amount*self.action.getFilledPropertiesCount())
        elif actionType.amountEvaluation == CActionType.actionAssignedPropsCount:
            setAmount(actionType.amount*self.action.getAssignedPropertiesCount())

        if actionType.defaultPlannedEndDate == CActionType.dpedBegDatePlusAmount:
            begDate = self.edtBegDate.date()
            amountValue = int(self.edtAmount.value())
            date = begDate.addDays(amountValue-1) if begDate and amountValue else QDate()
            self.edtPlannedEndDate.setDate(date)
        elif actionType.defaultPlannedEndDate == CActionType.dpedBegDatePlusDuration:
            begDate = self.edtBegDate.date()
            durationValue = self.edtDuration.value()
            date = begDate.addDays(durationValue) if begDate else QDate()
            self.edtPlannedEndDate.setDate(date)


    def checkDataEntered(self, secondTry=False):
        result = True
        begDate = self.edtBegDate.date()
        endDate = self.edtEndDate.date()
        if endDate:
            if QtGui.qApp.isStrictCheckPolicyOnEndAction() in (0, 1):
                if not checkPolicyOnDate(self.clientId, endDate):
                    skippable = QtGui.qApp.isStrictCheckPolicyOnEndAction() == 0
                    result = result and self.checkInputMessage(u'время закрытия действия соответствующее полису',
                                                               skippable, self.edtEndDate)
            if QtGui.qApp.isStrictCheckAttachOnEndAction() in (0, 1):
                if not checkAttachOnDate(self.clientId, endDate):
                    skippable = QtGui.qApp.isStrictCheckAttachOnEndAction() == 0
                    result = result and self.checkInputMessage(u'время закрытия действия соответствующее прикреплению',
                                                               skippable, self.edtEndDate)
        if begDate and endDate:
            showTime = self.action._actionType.showTime and getEventShowTime(self.eventTypeId)
            if showTime:
                begDateTime = toDateTimeWithoutSeconds(QDateTime(begDate, self.edtBegTime.time()))
                endDateTime = toDateTimeWithoutSeconds(QDateTime(endDate, self.edtEndTime.time()))
                result = result and (endDateTime >= begDateTime or self.checkValueMessage(u'Дата выполнения действия %s не должна быть раньше даты начала действия % s'%(forceString(endDateTime), forceString(begDateTime)), False, self.edtEndTime))
            else:
                result = result and (endDate >= begDate or self.checkValueMessage(u'Дата выполнения действия %s не должна быть раньше даты начала действия % s'%(forceString(endDate), forceString(begDate)), False, self.edtEndDate))
        if result and self.eventId:
            record = QtGui.qApp.db.getRecord('Event', 'setDate, execDate',  self.eventId)
            if record:
                setDate = forceDate(record.value('setDate'))
                execDate = forceDate(record.value('execDate'))
                if setDate and endDate:
                    result = result and (endDate >= setDate or self.checkValueMessage(u'Дата выполнения действия %s не должна быть раньше даты начала события % s'%(forceString(endDate), forceString(setDate)), True, self.edtEndDate))
                if execDate and endDate:
                    result = result and (execDate >= endDate or self.checkValueMessage(u'Дата выполнения действия %s не должна быть позже даты выполнения события % s'%(forceString(endDate), forceString(execDate)), True, self.edtEndDate))
                elif setDate and endDate:
                    currentDate = QDate.currentDate()
                    if currentDate and not secondTry:
                        # secondTry - это заглушка потому что я "ниасилил" по каким причинам сделали чтобы checkDataEntered вызывается два раза.
                        # Поправте, пожалуйста, чтобы вызывался один раз.
                        # Я верю что были серьезные причины, но это не правильно.
                        if not self._saveAsExecutionPlan:
                            result = result and (currentDate >= endDate or self.checkValueMessage(
                                u'Дата выполнения действия %s не должна быть позже текущей даты % s'%(forceString(endDate), forceString(currentDate)), True, self.edtEndDate)
                            )
#        result = result and (begDate or self.checkInputMessage(u'дату назначения', False, self.edtBegDate))
#        result = result and (endDate or self.checkInputMessage(u'дату выполнения', False, self.edtEndDate))
#        if not endDate:
#            maxEndDate = self.getMaxEndDateByVisits()
#            if maxEndDate:
#                if QtGui.QMessageBox.question(self,
#                                    u'Внимание!',
#                                    u'Дата выполнения обращения не указана.\nУстановить дату завершения по максимальной дате посещений',
#                                    QtGui.QMessageBox.No|QtGui.QMessageBox.Yes,
#                                    QtGui.QMessageBox.No) == QtGui.QMessageBox.Yes:
#                    self.edtEndDate.setDate(maxEndDate)
#                    endDate = maxEndDate
        result = result and self.checkActualMKB()
        if not self.checkActionMorphology():
            return False
        if not self.checkPlannedEndDate():
            return False
        return result


    def checkPlannedEndDate(self):
        action = self.action
        if action:
            actionType = action.getType()
            if actionType and actionType.isPlannedEndDateRequired in [CActionType.dpedControlMild, CActionType.dpedControlHard]:
                if not self.edtAPPlannedEndDate.date():
                    skippable = True if actionType.isPlannedEndDateRequired == CActionType.dpedControlMild else False
                    message = u'Необходимо указать Плановую дату выполнения у действия %s'%(actionType.name)
                    return self.checkValueMessage(message, skippable, self.edtAPPlannedEndDate)
        return True


    def checkActionMorphology(self):
        actionStatus = self.cmbStatus.value()
        if QtGui.qApp.defaultMorphologyMKBIsVisible() \
           and actionStatus in (CActionStatus.finished, CActionStatus.withoutResult):
            action = self.action
            actionType = action.getType()
            defaultMKB = actionType.defaultMKB
            isMorphologyRequired = actionType.isMorphologyRequired
            morphologyMKB = self.cmbMorphologyMKB.text()
            if not self.cmbMorphologyMKB.isValid(morphologyMKB) and defaultMKB > 0 and isMorphologyRequired > 0:
                if actionStatus == CActionStatus.withoutResult and isMorphologyRequired == 2:
                    return True
                skippable = True if isMorphologyRequired == 1 else False
                message = u'Необходимо ввести корректную морфологию диагноза действия `%s`' % actionType.name
                return self.checkValueMessage(message, skippable, self.cmbMorphologyMKB)
        return True


    def checkActualMKB(self):
            result = True
            MKB = unicode(self.cmbMKB.text())
            if MKB:
                db = QtGui.qApp.db
                tableMKB = db.table('MKB')
                cond = [tableMKB['DiagID'].eq(MKBwithoutSubclassification(MKB))]
                cond.append(db.joinOr([tableMKB['endDate'].isNull(), tableMKB['endDate'].dateGe(self.edtBegDate.date())] ))
                recordMKB = db.getRecordEx(tableMKB, [tableMKB['DiagID']], cond)
                result = result and (forceString(recordMKB.value('DiagID')) == MKBwithoutSubclassification(MKB) if recordMKB else False) or self.checkValueMessage(u'Диагноз %s не доступен для применения'%MKB, False, self.cmbMKB)
            return result


    def setPersonId(self, personId):
        self.personId = personId
        record = QtGui.qApp.db.getRecord('Person', 'speciality_id, tariffCategory_id',  self.personId)
        if record:
            self.personSpecialityId     = forceRef(record.value('speciality_id'))
            self.personTariffCategoryId = forceRef(record.value('tariffCategory_id'))
#        self.actionTemplateCache.reset()

    def getEventInfo(self, context):
        return None


    def setCurrentProperty(self, row, column=1):
        index = self.modelActionProperties.index(row, column)
        self.tblProps.setCurrentIndex(index)
# # #


    @pyqtSignature('')
    def on_actEditClient_triggered(self):
        if QtGui.qApp.userHasAnyRight([urAdmin, urRegTabWriteRegistry, urRegTabReadRegistry]):
            dialog = CClientEditDialog(self)
            try:
                dialog.load(self.clientId)
                if dialog.exec_():
                    self.updateClientInfo()
            finally:
                dialog.destroy()
                sip.delete(dialog)
                del dialog


    @pyqtSignature('double')
    def on_edtAmount_valueChanged(self, value):
        actionType = self.action.getType()
        if actionType.defaultPlannedEndDate == CActionType.dpedBegDatePlusAmount:
            begDate = self.edtBegDate.date()
            amountValue = int(value)
            date = begDate.addDays(amountValue-1) if begDate and amountValue else QDate()
            self.edtPlannedEndDate.setDate(date)



    @pyqtSignature('QString')
    def on_cmbMKB_textChanged(self, value):
        if value[-1:] == '.':
            value = value[:-1]
        diagName = forceString(QtGui.qApp.db.translate('MKB', 'DiagID', value, 'DiagName'))
        if diagName:
            self.lblMKBText.setText(diagName)
        else:
            self.lblMKBText.clear()
        self.cmbMorphologyMKB.setMKBFilter(self.cmbMorphologyMKB.getMKBFilter(unicode(value)))


    @pyqtSignature('')
    def on_btnSelectOrg_clicked(self):
        orgId = selectOrganisation(self, self.cmbOrg.value(), False, self.cmbOrg.filter)
        self.cmbOrg.updateModel()
        if orgId:
            self.cmbOrg.setValue(orgId)


    @pyqtSignature('int')
    def on_cmbPerson_currentIndexChanged(self):
        self.setPersonId(self.cmbPerson.value())
        actionTemplateTreeModel = self.actionTemplateCache.getModel(self.action.getType().id)
        self.btnLoadTemplate.setModel(actionTemplateTreeModel)
        if not(QtGui.qApp.userHasRight(urLoadActionTemplate) and (self.cmbStatus.value() != CActionStatus.finished or not self.cmbPerson.value() or QtGui.qApp.userId == self.cmbPerson.value() or QtGui.qApp.userHasRight(urEditOtherpeopleAction))):
            self.btnLoadTemplate.setEnabled(False)


    @pyqtSignature('QDate')
    def on_edtDirectionDate_dateChanged(self, date):
        self.edtDirectionTime.setEnabled(bool(date))


    @pyqtSignature('QDate')
    def on_edtBegDate_dateChanged(self, date):
        self.edtBegTime.setEnabled(bool(date))
        self.updateAmount()
        self.cmbMKB.setFilter('''IF(MKB.endDate IS NOT NULL, MKB.endDate >= %s, 1)'''%(QtGui.qApp.db.formatDate(self.edtBegDate.date())))


    @pyqtSignature('QDate')
    def on_edtEndDate_dateChanged(self, date):
        self.edtEndTime.setEnabled(bool(date))
        self.updateAmount()
        if date:
            self.cmbStatus.setValue(CActionStatus.finished)
            if self.eventSetDate and date < self.eventSetDate:
                self.edtBegDate.setDate(QDate())
                self.edtBegTime.setTime(QTime())
                self.edtDirectionDate.setDate(QDate())
                self.edtDirectionTime.setTime(QTime())
        else:
            if self.cmbStatus.value() == CActionStatus.finished:
                self.cmbStatus.setValue(CActionStatus.canceled)

        self.btnNextAction.setEnabled(self.btnNextActionMustBeEnabled())

        if self.edtDuration.value() > 0:
            self.btnPlanNextAction.setEnabled(True)
        else:
            self.btnPlanNextAction.setEnabled(False)

        if self.action.getType().closeEvent:
            self.setEventDate(date)


    @pyqtSignature('int')
    def on_cmbStatus_currentIndexChanged(self, index):
        actionStatus = self.cmbStatus.value()
        if actionStatus in (CActionStatus.finished, CActionStatus.withoutResult):
            if not self.edtEndDate.date():
                now = QDateTime.currentDateTime()
                self.edtEndDate.setDate(now.date())
                if self.edtEndTime.isVisible():
                    self.edtEndTime.setTime(now.time())
        elif actionStatus in (CActionStatus.canceled, CActionStatus.refused):
            self.edtEndDate.setDate(QDate())
            self.edtEndTime.setTime(QTime())
            if QtGui.qApp.userId and QtGui.qApp.userSpecialityId:
                self.cmbPerson.setValue(QtGui.qApp.userId)
            else:
                self.cmbPerson.setValue(self.cmbSetPerson.value())
        else:
            self.edtEndDate.setDate(QDate())
            self.edtEndTime.setTime(QTime())

        self.btnNextAction.setEnabled(self.btnNextActionMustBeEnabled())


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_modelActionProperties_dataChanged(self, topLeft, bottomRight):
        self.updateAmount()


    @pyqtSignature('int')
    def on_btnPrint_printByTemplate(self, templateId):
        context = CInfoContext()
        eventInfo = context.getInstance(CEventInfo, self.eventId)

        eventActions = eventInfo.actions
        eventActions._idList = [self.itemId()]
        eventActions._items  = [CCookedActionInfo(context, self.getRecord(), self.action)]
        eventActions._loaded = True

        action = eventInfo.actions[0]
        action._isDirty = self.isDirty()
        action.setCurrentPropertyIndex(self.tblProps.currentIndex().row())
        data = { 'event' : eventInfo,
                 'action': action,
                 'client': eventInfo.client,
                 'actions':eventActions,
                 'currentActionIndex': 0,
                 'tempInvalid': None
               }
        applyTemplate(self, templateId, data)


    @pyqtSignature('int')
    def on_btnLoadTemplate_templateSelected(self, templateId):
        if QtGui.qApp.userHasRight(urLoadActionTemplate) and (self.cmbStatus.value() != CActionStatus.finished or not self.cmbPerson.value() or QtGui.qApp.userId == self.cmbPerson.value() or QtGui.qApp.userHasRight(urEditOtherpeopleAction)):
            self.action.updateByTemplate(templateId)
            self.tblProps.model().reset()
            self.tblProps.resizeRowsToContents()
            self.updateAmount()


    @pyqtSignature('')
    def on_btnSaveAsTemplate_clicked(self):
        if QtGui.qApp.userHasRight(urSaveActionTemplate):
            record = self.getRecord()
            dlg = CActionTemplateSaveDialog(self, record, self.action, self.clientSex, self.clientAge, self.personId, self.personSpecialityId)
            dlg.exec_()
            sip.delete(dlg)
            actionType = self.action.getType()
            self.actionTemplateCache.reset()
            actionTemplateTreeModel = self.actionTemplateCache.getModel(actionType.id)
            self.btnLoadTemplate.setModel(actionTemplateTreeModel)
            if not(QtGui.qApp.userHasRight(urLoadActionTemplate) and (self.cmbStatus.value() != CActionStatus.finished or not self.cmbPerson.value() or QtGui.qApp.userId == self.cmbPerson.value() or QtGui.qApp.userHasRight(urEditOtherpeopleAction))):
                self.btnLoadTemplate.setEnabled(False)


    @pyqtSignature('')
    def on_btnAPNomenclatureExpense_clicked(self):
        self._openNomenclatureEditor()


    def _openNomenclatureEditor(self, requireItems=False):
        # Результат может влиять на продолжение работы функционала "Выполнить"
        action = self.action
        if not action.nomenclatureExpense:
            return True

        if action.nomenclatureExpense.stockMotionRecord():
            if requireItems and not action.nomenclatureExpense.stockMotionItems():
                return True

            if QtGui.qApp.keyboardModifiers() & Qt.ShiftModifier:
                dlg = CClientRefundInvoiceEditDialog(self)
                dlg.setData(action.nomenclatureExpense)
            else:
                dlg = CClientInvoiceEditDialog(self, fromAction=True)
                dlg.setFinanceId(forceRef(self.record().value('finance_id')))
                dlg.setData(action.nomenclatureExpense)
                dlg.setClientId(self.clientId)
            try:
                dlg.exec_()
                dlg.getRecord()

                result = dlg.closeByOkButton

            finally:
                sip.delete(dlg)

            return result
        else:
            QtGui.QMessageBox.warning(self,
                                      u'Внимание!',
                                      u'Не удалось инициализировать записи ЛСиИМН \nсвязанные с данным действием!',
                                      QtGui.QMessageBox.Ok,
                                      QtGui.QMessageBox.Ok)
