# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui
from PyQt4.QtCore import QVariant


from library.crbcombobox import CRBComboBox
from library.Utils       import forceRef, forceString, forceDate
from ActionPropertyValueType       import CActionPropertyValueType


class CHospitalBedProfileActionPropertyValueType(CActionPropertyValueType):
    name         = 'rbHospitalBedProfile'
    variantType  = QVariant.Int

    class CPropEditor(CRBComboBox):
        def __init__(self, action, domain, parent, clientId, eventTypeId):
            CRBComboBox.__init__(self, parent)
            self.domain = domain
            orgStructureId = None
            setFilter=''
            setFilterAnd = u''
            db = QtGui.qApp.db
            if action:
                record = action.getRecord()
                if record:
                    begDate = forceDate(record.value('begDate'))
                    if begDate:
                        setFilter = u'''(endDate IS NULL OR endDate >= %s)'''%(db.formatDate(begDate))
                        setFilterAnd = u' AND '
            if u'received' in action._actionType.flatCode.lower():
                orgStructureId = action[u'Направлен в отделение'] if u'Направлен в отделение' in action._actionType._propertiesByName else None
            elif u'leaved' in action._actionType.flatCode.lower():
                orgStructureId = action[u'Отделение'] if u'Отделение' in action._actionType._propertiesByName else None
            if orgStructureId:
                orgStructureIdList = db.getDescendants('OrgStructure', 'parent_id', orgStructureId)
                if orgStructureIdList:
                    table = db.table('vHospitalBed')
                    hospitalBedProfileIdList = db.getDistinctIdList(table, [table['profile_id']], [table['master_id'].inlist(orgStructureIdList)])
                    if hospitalBedProfileIdList:
                        setFilter += setFilterAnd + u'id IN (%s)'%(u', '.join(str(profileId) for profileId in hospitalBedProfileIdList if profileId))
            self.setTable('rbHospitalBedProfile', addNone=True, filter=setFilter, order='code')

        def setValue(self, value):
            CRBComboBox.setValue(self, forceRef(value))


    @staticmethod
    def convertDBValueToPyValue(value):
        return forceRef(value)


    convertQVariantToPyValue = convertDBValueToPyValue


    def toInfo(self, context, v):
        from HospitalBeds.HospitalBedInfo import CHospitalBedProfileInfo
        return CHospitalBedProfileInfo(context, v)


    def toText(self, v):
        return forceString(QtGui.qApp.db.translate('rbHospitalBedProfile', 'id', v, 'CONCAT(code,\' | \',name)'))

