# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################


from PyQt4 import QtGui
from PyQt4.QtCore import QVariant

from library.crbcombobox        import CRBModelDataCache
from library.Utils              import forceDate, forceRef, trim
from Resources.JobTicketChooser import CJobTicketChooserComboBox, getJobTicketAsText
from Resources.JobTicketChooserHelper import CJobTicketChooserHelper, TODAY, ANYDAY, ANYNEXTDAY
from Users.Rights               import urEditJobTicket
from ActionPropertyValueType    import CActionPropertyValueType


class CJobTicketActionPropertyValueType(CActionPropertyValueType):
    name         = 'JobTicket'
    variantType  = QVariant.Int
    isCopyable   = False
    initPresetValue = True

    def __init__(self, domain = None):
        CActionPropertyValueType.__init__(self, domain)
        self.domain = self.parseDomain(domain)
        self._presetValueAllowed = self.domain['presetValueAllowed']
        self._forDays            = self.domain['forDays']
        self._jobTypeCode        = self.domain['jobTypeCode']
        self._chainLength        = self.domain['chainLength']


    @staticmethod
    def parseDomain(domain):
        # синтаксис:
        #   <код>[;A[0]],<длина цепочки>]]
        # <код> - код типа работы
        # A - возможность выбирать автоматом только на тек.дату
        # A0 - возможность выбирать автоматом на любую дату
        # <длина цепочки> - это длина цепочки талонов.

        presetValueAllowed = False
        forDays = TODAY
        chainLength = 1
        specialValues = presetSpecialValues = ''
        domain = domain.split(';')
        jobTypeCode = trim(domain[0])
        if len(domain) > 1:
            specialValuesStr = trim(domain[1]).lower()
            specialValuesList = specialValuesStr.split(',')

            if len(specialValuesList) == 1:
                specialValues = specialValuesList[0]
                if specialValues:
                    if specialValues[0] in ('a', u'а'):
                        presetSpecialValues = specialValues
                    else:
                        chainLength = specialValues
            elif len(specialValuesList) == 2:
                presetSpecialValues, chainLength = specialValuesList
            else:
                return {}

            if len(presetSpecialValues) == 1:
                presetValueAllowed = presetSpecialValues in ('a', u'а') # en & ru
            elif len(presetSpecialValues) == 2:
                presetValueAllowed = presetSpecialValues[0] in ('a', u'а') # en & ru
                if presetSpecialValues[1] == '0':
                    forDays = ANYDAY
                elif presetSpecialValues[1] == '1':
                    forDays = ANYNEXTDAY
        return {'jobTypeCode'       : jobTypeCode,
                'presetValueAllowed': presetValueAllowed,
                'forDays'           : forDays,
                'chainLength'       : int(chainLength)
               }


    def chainLength(self):
        return self._chainLength


    def getjobTypeCode(self):
        return self._jobTypeCode


    def getJobTypeId(self):
        if self._jobTypeCode:
            data = CRBModelDataCache.getData('rbJobType')
            return data.getIdByCode(self._jobTypeCode)
        else:
            return None



    @staticmethod
    def getParsedDomain(domain):
        return CJobTicketActionPropertyValueType.parseDomain(domain)


    class CPropEditor(CJobTicketChooserComboBox):
        def __init__(self, action, domain, parent, clientId, eventTypeId, reservedJobTickets={}):
            CJobTicketChooserComboBox.__init__(self, parent)

            jobTypeCode = domain['jobTypeCode']
#            chainLength = domain['chainLength']
#            self.setchainLengthValue(chainLength)
            self.setEventTypeId(eventTypeId)
            self.setClientId(clientId)
            self.setReservedJobTickets(reservedJobTickets)
            self.setDefaultJobTypeCode(jobTypeCode)
            date = forceDate(action._record.value('begDate')) if action else None
            if date:
                self.setDefaultDate(date)

            enabled = QtGui.qApp.userHasRight(urEditJobTicket)
            if jobTypeCode:
                enabled = enabled and QtGui.qApp.isUserAvailableJobType(jobTypeCode)
            self.setEnabled(enabled)


        def setValue(self, value):
            CJobTicketChooserComboBox.setValue(self, forceRef(value))


    def getPresetValue(self, action):
        if self._presetValueAllowed:
            jobTypeId = self.getJobTypeId()
            if jobTypeId:
                jobTypeIdList = QtGui.qApp.db.getDescendants('rbJobType', 'group_id', jobTypeId)
                if len(jobTypeIdList) == 1 and jobTypeIdList[0] == jobTypeId:
                    conditions = action.presetValuesConditions
                    clientId = conditions.get('clientId')
                    eventTypeId = conditions.get('eventTypeId')
                    courseDate = conditions.get('courseDate')
                    courseTime = conditions.get('courseTime')
                    firstInCourse = conditions.get('firstInCourse', False)
                    return CJobTicketChooserHelper(jobTypeId, clientId, eventTypeId, self._chainLength, self._forDays,
                                                   courseDate=courseDate, courseTime=courseTime,
                                                   firstInCourse=firstInCourse).get()
        return None

    @staticmethod
    def convertDBValueToPyValue(value):
        return forceRef(value)


    convertQVariantToPyValue = convertDBValueToPyValue

    @classmethod
    def getTableName(cls):
        return cls.tableNamePrefix+'Job_Ticket'


    def toText(self, v):
        return getJobTicketAsText(forceRef(v))


    def toInfo(self, context, v):
        from Resources.JobTicketInfo import CJobTicketInfo
        return CJobTicketInfo(context, forceRef(v))
