# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import json
from PyQt4 import QtGui
from PyQt4.QtCore import QVariant

from Stock.NomenclatureComboBox import CNomenclatureComboBox
from library.Utils              import forceRef, forceString
from ActionPropertyValueType    import CActionPropertyValueType
from FeatureActionPropertyValueType import CFeatureActionPropertyValueType

from Stock.StockMotionInfo import CLSiIMNInfo


class CNomenclatureActionPropertyValueType(CActionPropertyValueType):
    variantType  = QVariant.Int
    name         = u'Номенклатура ЛСиИМН'
    cacheText = True

    class CPropEditor(CNomenclatureComboBox):
        def __init__(self, action, domain, parent, clientId, eventTypeId, eventEditor=None):
            CNomenclatureComboBox.__init__(self, parent)
            self.setUseClientUnitId()
            actionTypeId = action.getType().id

            cols = ['nomenclatureClass_id', 'nomenclatureKind_id', 'nomenclatureType_id'] # wtf
            record = QtGui.qApp.db.getRecord('ActionType', cols, actionTypeId)

            nomenclatureClassId = forceRef(record.value('nomenclatureClass_id'))
            nomenclatureKindId  = forceRef(record.value('nomenclatureKind_id'))
            nomenclatureTypeId  = forceRef(record.value('nomenclatureType_id'))

            try:
                domainObj = json.loads(domain)
                featuresPropertyName = domainObj.get('featuresPropertyName', '')
                if featuresPropertyName:
                    featuresStr = action[featuresPropertyName]
                else:
                    featuresStr = ''
            except:
                featuresStr = ''

            self.setDefaultIds(nomenclatureClassId, nomenclatureKindId, nomenclatureTypeId)
            if eventEditor:
                self.setFinanceId(eventEditor.eventFinanceId)

            if featuresStr:
                try:
                    features = CFeatureActionPropertyValueType.parseValue(featuresStr)
                    self.setDefaultFeatures(features)
                except:
                    pass

            self.setOrgStructureId(QtGui.qApp.currentOrgStructureId())
            self.setOnlyExists()
            self.setPreferredWidth(QtGui.QApplication.desktop().width() - self.mapToGlobal(self.rect().bottomLeft()).x())


        def setValue(self, value):
            CNomenclatureComboBox.setValue(self, forceRef(value))


    @staticmethod
    def convertDBValueToPyValue(value):
        return forceRef(value)


    convertQVariantToPyValue = convertDBValueToPyValue


    def toText(self, nomenclatureId):
        if not nomenclatureId:
            return ''

        db = QtGui.qApp.db
        fields = 'mnnLatin, originName, internationalNonproprietaryName, name, dosageValue'
        record = db.getRecord('rbNomenclature', fields, nomenclatureId)
        if record:
            dosageValue = forceString(record.value('dosageValue'))
            mnnLatin = forceString(record.value('mnnLatin'))
            originName = forceString(record.value('originName'))
            internationalNonproprietaryName = forceString(record.value('internationalNonproprietaryName'))
            name = forceString(record.value('name'))

            if mnnLatin:
                result = mnnLatin
            elif originName:
                result = originName
            elif internationalNonproprietaryName:
                result = internationalNonproprietaryName
            else:
                result = name
            result = ' | '.join([result, dosageValue]) if dosageValue else result
        else:
            result = ''

        return result


    def toInfo(self, context, v):
        return context.getInstance(CLSiIMNInfo, forceRef(v))


    @classmethod
    def getTableName(cls):
        return cls.tableNamePrefix+'rbNomenclature'

