# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui
from PyQt4.QtCore import QEvent, QVariant, SIGNAL

from Orgs.OrgComboBox        import COrgComboBox
from Orgs.Orgs               import selectOrganisation
from Orgs.Utils              import getOrganisationShortName,  COrgInfo

from library.Utils           import forceRef

from ActionPropertyValueType import CActionPropertyValueType



class COrganisationActionPropertyValueType(CActionPropertyValueType):
    name        = 'Organisation'
    variantType = QVariant.Int
    badDomain   = u'Неверное описание области определения значения свойства действия типа Organisation:\n%(domain)s'
    badKey      = u'Недопустимый ключ "%(key)s" в описание области определения значения свойства действия типа Organisation:\n%(domain)s'
    badValue    = u'Недопустимое значение "%(val)s" ключа "%(key)s" в описание области определения значения свойства действия типа Organisation:\n%(domain)s'

    class CPropEditor(QtGui.QWidget):
        __pyqtSignals__ = ('editingFinished()',
                           'commit()',
                          )


        def __init__(self, action, domain, parent, clientId, eventTypeId):
            QtGui.QWidget.__init__(self, parent)
            self.boxlayout = QtGui.QHBoxLayout(self)
            self.boxlayout.setMargin(0)
            self.boxlayout.setSpacing(0)
            self.boxlayout.setObjectName('boxlayout')
            self.cmbOrganisation = COrgComboBox(self)
            self.cmbOrganisation.setObjectName('cmbOrganisation')

            self.cmbOrganisation.setFilter(domain)
            self.boxlayout.addWidget(self.cmbOrganisation)
            self.btnSelect = QtGui.QPushButton(self)
            self.btnSelect.setObjectName('btnSelect')
#            self.btnSelect.setText(u'…')
            self.btnSelect.setText(u'...')
            self.btnSelect.setSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Ignored)
            self.btnSelect.setFixedWidth(20)
            self.boxlayout.addWidget(self.btnSelect)
            self.setFocusProxy(self.cmbOrganisation)
            self.connect(self.btnSelect, SIGNAL('clicked()'), self.on_btnSelect_clicked)
            self.cmbOrganisation.installEventFilter(self)
            self.btnSelect.installEventFilter(self)
            self.action = action


        def eventFilter(self, widget, event):
            et = event.type()
            if et == QEvent.FocusOut:
                fw = QtGui.qApp.focusWidget()
                while fw and fw != self:
                    fw = fw.parentWidget()
                if not fw:
                    self.emit(SIGNAL('editingFinished()'))
            elif et == QEvent.Hide and widget == self.cmbOrganisation:
                self.emit(SIGNAL('commit()'))
            return QtGui.QWidget.eventFilter(self, widget, event)


        def on_btnSelect_clicked(self):
            sqlFilter = self.cmbOrganisation.filter()
            hospitalBedProfileId = None
            if 'isMedical' in sqlFilter:
                try:
                    hospitalBedProfileId = self.action[u'Профиль']
                except:
                    pass
            orgId = selectOrganisation(self, self.cmbOrganisation.value(), False, self.cmbOrganisation.filter(), hospitalBedProfileId)
            self.cmbOrganisation.updateModel()
            if orgId:
                self.cmbOrganisation.setValue(orgId)


        def setValue(self, value):
            self.cmbOrganisation.setValue(forceRef(value))


        def value(self):
            return self.cmbOrganisation.value()


    def __init__(self, domain = None):
        CActionPropertyValueType.__init__(self, domain)
        self.rawDomain = domain
        self.domain = self.parseDomain(domain)


    def parseDomain(self, domain):
        isInsurer = None
        isHospital = None
        isMed = None
        netCodes = []
        for word in domain.split(','):
            if word:
                parts = word.split(':')
                if len(parts) == 1:
                    key, val = u'тип', parts[0].strip()
                elif len(parts) == 2:
                    key, val = parts[0].strip(), parts[1].strip()
                else:
                    raise ValueError, self.badDomain % locals()
                keylower = key.lower()
                vallower = val.lower()
                if keylower == u'тип':
                    if vallower == u'смо':
                        isInsurer = True
                    elif vallower in (u'стац', u'стационар'):
                        isHospital = True
                    elif vallower == u'лпу':
                        isMed = True
                    else:
                        raise ValueError, self.badValue % locals()
                elif keylower == u'сеть':
                    netCodes.append(val)
#                elif keylower == u'профиль-койки':
#                    if val.startswith('@'):
#                        pass
#                    else:
#
                else:
                    raise ValueError, self.badKey % locals()
        db = QtGui.qApp.db
        table = db.table('Organisation')
        cond = []
        if isInsurer:
            cond.append('isInsurer')
        if isHospital:
            cond.append('isMedical=2')
        if isMed:
            cond.append('isMedical!=0')
        if netCodes:
            tableNet = db.table('rbNet')
            contNet  = [ tableNet['code'].inlist(netCodes), tableNet['name'].inlist(netCodes)]
            netIdList = db.getIdList(tableNet, 'id', db.joinOr(contNet))
            cond.append(table['net_id'].inlist(netIdList))
        return db.joinAnd(cond)


    @staticmethod
    def convertDBValueToPyValue(value):
        return forceRef(value)


    convertQVariantToPyValue = convertDBValueToPyValue


    def toText(self, v):
        return getOrganisationShortName(forceRef(v))


    def toInfo(self, context, v):
        return context.getInstance(COrgInfo, forceRef(v))


