# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui
from PyQt4.QtCore import QDate, QVariant, QUrl, SIGNAL
import json

from library.Pacs.RestToolbox  import postRequest, getRequest
from library.Pacs.Explorer     import CPacsExplorer
from library.DbEntityCache     import CDbEntityCache
from library.Utils             import forceString, forceDate, forceInt

from ActionPropertyValueType   import CActionPropertyValueType


class CPacsActionPropertyValueType(CActionPropertyValueType):
    name         = 'PacsImages'
    variantType  = QVariant.String


    def __init__(self, domain = None):
        CActionPropertyValueType.__init__(self, domain)


    class CPropEditor(QtGui.QPushButton):
        def __init__(self, action, domain, parent, clientId, eventTypeId):
            QtGui.QPushButton.__init__(self, parent)
            self.setText(u'Открыть')
            self.connect(self, SIGNAL('clicked()'), self.onClick)
            self._value = None

        def setValue(self, value):
            self._value = value
            self.setEnabled(bool(value))


        def value(self):
            return self._value


        def onClick(self):
            try:
                values = json.loads(forceString(self._value))
            except:
                QtGui.QMessageBox.critical(self,
                    u'Внимание!',
                    u'Ошибка данных пакс сервера',
                    QtGui.QMessageBox.Ok,
                    QtGui.QMessageBox.Ok)
                return
            appPrefs = QtGui.qApp.preferences.appPrefs
            type = forceInt(appPrefs.get('pacsType'))
            path = forceString(appPrefs.get('pacsParams'))
            if not path:
                path = forceString(appPrefs.get('pacsImagesBrowser'))
            if not path:
                QtGui.QMessageBox.critical(self,
                    u'Внимание!',
                    u'Просмотр изображений не настроен',
                    QtGui.QMessageBox.Ok,
                    QtGui.QMessageBox.Ok)
                return
            path = path.replace('{StudyUID}', values.get('StudyUID', ''))
            if type == 0:
                path.replace('\\', '/')
                p = subprocess.Popen(path)
            elif type == 2:
                if not QtGui.QDesktopServices.openUrl(QUrl.fromEncoded(path)):
                    QtGui.QMessageBox.critical(self,
                    u'Внимание!',
                    u'Невозможно открыть браузер',
                    QtGui.QMessageBox.Ok,
                    QtGui.QMessageBox.Ok)
                return


    @staticmethod
    def convertDBValueToPyValue(value):
        return forceString(value)


    convertQVariantToPyValue = convertDBValueToPyValue


    def toInfo(self, context, v):
        return v


    def toText(self, v):
        try:
            d = json.loads(v)
            return d.get('ShowText', u'Снимки получены')
        except:
            return u'Снимки не зарегистрированы'
