# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.actAPDuplicateAction
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

# Страница редактирования action одного класса в пределах event'а
import sip
from PyQt4 import QtGui
from PyQt4.QtCore import Qt, QDate, QDateTime, QMimeData, QString, QTime, QVariant, pyqtSignature, SIGNAL

#from library.Attach.AttachButton     import CAttachButton
from library.Calendar                import wpFiveDays, addWorkDays, getNextWorkDay, wpSixDays
from library.DialogBase              import CConstructHelperMixin, CDialogBase
from library.ICDUtils                import MKBwithoutSubclassification
from library.interchange             import setDatetimeEditValue
from library.PrintTemplates          import applyTemplate, customizePrintButton
from library.Utils                   import forceBool, forceDate, forceDateTime, forceDouble, forceInt, forceRef, forceString, toVariant

from Events.Action                   import CActionTypeCache, CAction, CActionType
from Events.ActionPropertiesTable    import CActionPropertiesTableModel
from Events.ActionsModel             import CGroupActionsProxyModel
from Events.ActionsSelector          import selectActionTypes
from Events.ActionStatus             import CActionStatus
from Events.ActionTemplateChoose     import CActionTemplateCache
from Events.ActionTemplateSaveDialog import CActionTemplateSaveDialog
from Events.ActionTypeDialog         import CActionTypeDialogTableModel
from Events.ExecTimeNextActionDialog import CExecTimeNextActionDialog
from Events.ExecutionPlanDialog      import CGetExecutionPlan
from Events.GetPrevActionIdHelper    import CGetPrevActionIdHelper
from Events.InputDialog              import CInputDialog
from Events.Utils                    import CFinanceType, getDiagnosisId2, getChiefId, getActionTypeIdListByFlatCode, checkAttachOnDate, checkPolicyOnDate, cutFeed, getEventContextData, getEventShowTime, setActionPropertiesColumnVisible, getEventCSGRequired, getEventTypeForm
from Orgs.Orgs                       import selectOrganisation
from Stock.ClientInvoiceEditDialog   import CClientInvoiceEditDialog, CClientRefundInvoiceEditDialog
from Users.Rights                    import urCopyPrevAction, urEditClosedEvent, urLoadActionTemplate, urSaveActionTemplate, urEditOtherpeopleAction

from Events.Ui_ActionsPage import Ui_ActionsPageWidget

class CActionsPage(QtGui.QWidget, CConstructHelperMixin, Ui_ActionsPageWidget):
    def __init__(self, parent=None):
        QtGui.QWidget.__init__(self, parent)
        self.eventEditor = None

        self.addModels('APActions',            CGroupActionsProxyModel(self))
        self.addModels('APActionProperties',   CActionPropertiesTableModel(self))
        self.addObject('actAPAddLikeAction',   QtGui.QAction(u'Добавить такой-же', self))
        self.addObject('actAPDuplicateAction', QtGui.QAction(u'Дублировать', self))
        self.addObject('actAPReplicateAction', QtGui.QAction(u'Тиражировать', self))
        self.addObject('actAPAddAction',       QtGui.QAction(u'Добавить', self))
        self.addObject('actAPActionsAdd',      QtGui.QAction(u'Добавить ...', self))
        self.addObject('actAPActionSplitEvent',QtGui.QAction(u'В новое событие', self))

        self.addObject('mnuAPLoadPrevAction',  QtGui.QMenu(self))
        self.addObject('actAPLoadSameSpecialityPrevAction', QtGui.QAction(u'Той же самой специальности', self))
        self.addObject('actAPLoadOwnPrevAction',            QtGui.QAction(u'Только свои', self))
        self.addObject('actAPLoadAnyPrevAction',            QtGui.QAction(u'Любое', self))
        self.mnuAPLoadPrevAction.addAction(self.actAPLoadSameSpecialityPrevAction)
        self.mnuAPLoadPrevAction.addAction(self.actAPLoadOwnPrevAction)
        self.mnuAPLoadPrevAction.addAction(self.actAPLoadAnyPrevAction)

        self.actAPActionsAdd.setShortcut(Qt.Key_F9)
        self.addAction(self.actAPActionsAdd)
        self.setupUi(self)
        self.tblAPActions.setParentWidget(self)
        self.setFocusProxy(self.tblAPActions)
        self.edtAPDirectionDate.canBeEmpty(True)
        self.edtAPBegDate.canBeEmpty(True)
        self.edtAPEndDate.canBeEmpty(True)
        self.setupActionPopupMenu()
        self.commonTakenTissueJournalRecordId = None
        self.allowedActionTypesByTissue = []
        self.defaultDirectionDate = CActionType.dddUndefined
        self.defaultEndDate = CActionType.dedUndefined
        self.actionTemplateCache = CActionTemplateCache(self.eventEditor, self.cmbAPPerson)
        self.setModels(self.tblAPActions, self.modelAPActions, self.selectionModelAPActions)
        self.setModels(self.tblAPProps, self.modelAPActionProperties, self.selectionModelAPActionProperties)
        self.btnAPLoadPrevAction.setMenu(self.mnuAPLoadPrevAction)
        self._visibleMorphologyMKB = QtGui.qApp.defaultMorphologyMKBIsVisible()
        self._canUseLaboratoryCalculator = False
        self.receivedFinanceId = None
        self.eventActionFinance = None
        self.getActionTypeHBIdList()

        action = QtGui.QAction(self)
        self.addObject('actSetLaboratoryCalculatorInfo', action)
        action.setShortcut('F3')
        self.addAction(action)
        self.connect(self.actSetLaboratoryCalculatorInfo, SIGNAL('triggered()'), self.on_actSetLaboratoryCalculatorInfo)

        self._canUseLaboratoryCalculatorPropertyTypeList = None
        self._mainWindowState = QtGui.qApp.mainWindow.windowState()

        self.connect(self.modelAPActionProperties,
                     SIGNAL('actionNameChanged()'), self.updateCurrentActionName)
        self.connect(self.modelAPActionProperties,
                     SIGNAL('setCurrentActionEndDate(QDate)'), self.setCurrentActionEndDate)
        self.connect(self.modelAPActionProperties,
                     SIGNAL('actionAmountChanged(double)'), self.on_actionAmountChanged)

        self.connect(self.tblAPActions,
                     SIGNAL('delRows()'), self.modelAPActions.emitItemsCountChanged)


    def getActionTypeHBIdList(self):
        self.actionTypeHBIdList = []
        self.actionTypeHBIdList.extend(getActionTypeIdListByFlatCode(u'received%'))
        self.actionTypeHBIdList.extend(getActionTypeIdListByFlatCode(u'moving%'))
        self.actionTypeHBIdList.extend(getActionTypeIdListByFlatCode(u'leaved%'))


    def protectFromEdit(self, isProtected):
        editWidgets = [self.edtAPDirectionDate,
                       self.edtAPDirectionTime,
                       self.edtAPPlannedEndDate,
                       self.edtAPPlannedEndTime,
                       self.edtAPBegDate,
                       self.edtAPBegTime,
                       self.edtAPEndDate,
                       self.edtAPEndTime,
                       self.edtAPOffice,
                       self.edtAPAmount,
                       self.edtAPUet,
                       self.edtAPNote,
                       self.edtAPCoordDate,
                       self.edtAPCoordTime,
                       self.edtAPDuration,
                       self.edtAPPeriodicity,
                       self.edtAPAliquoticity,
                       self.cmbAPSetPerson,
                       self.cmbAPStatus,
                       self.cmbAPOrg,
                       self.cmbAPPerson,
                       self.cmbAPAssistant,
                       self.cmbAPMKB,
                       self.cmbAPMorphologyMKB
                       ]
        for widget in editWidgets:
            widget.setReadOnly(isProtected)
        editWidgets = [self.chkAPIsUrgent,
                       self.btnAPLoadTemplate,
                       self.btnAPLoadPrevAction,
                       self.btnAPSaveAsTemplate,
                       self.btnPlanNextAction,
                       self.btnAPNomenclatureExpense,
                       self.btnAPSelectOrg,
                       self.actAPActionsAdd
                       ]
        for widget in editWidgets:
            widget.setEnabled(not isProtected)
        if self.btnNextAction.isEnabled():
            self.btnNextAction.setEnabled(not isProtected)


    def on_actionAmountChanged(self, value):
        self.edtAPAmount.setValue(value)


    def updateCurrentActionName(self):
        index = self.tblAPActions.currentIndex()
        # Правильно было бы сделать emit dataChanged,
        #self.modelAPActions.emit(SIGNAL('dataChanged(const QModelIndex &, const QModelIndex &)'), index, index)
        # но это приводит к перескоку фокуса в неведомые места
        self.tblAPActions.dataChanged(index, index)


    def setCurrentActionEndDate(self, date):
        self.edtAPPlannedEndDate.setDate(date)


    def destroy(self):
        self.tblAPActions.setModel(None)
        del self.modelAPActions


    def setActionTypeClass(self, actionTypeClass):
        self.modelAPActions.setActionTypeClass(actionTypeClass)


    def setEventEditor(self, eventEditor):
        self.eventEditor = eventEditor
        self.modelAPActions.eventEditor = eventEditor
        self.actionTemplateCache.eventEditor = eventEditor
        self.connect(eventEditor, SIGNAL('updateActionsAmount()'), self.modelAPActions.updateActionsAmount)
        self.cmbCSG.setEventEditor(eventEditor)
        if hasattr(self.eventEditor, 'tabMes'):
            self.eventEditor.tabMes.csgRowAboutToBeRemoved.connect(self.onCsgRowAboutToBeRemoved)
            self.eventEditor.tabMes.csgRowRemoved.connect(self.onCsgRowRemoved)

    def onCsgRowRemoved(self):
        self.cmbCSG.setItems()

    def onCsgRowAboutToBeRemoved(self, record):
        for action in self.cmbCSG.mapActionToCSG:
            if self.cmbCSG.mapActionToCSG[action] == record:
                del self.cmbCSG.mapActionToCSG[action]
                action.setValue('eventCSG_id', QVariant(None))
                break
        csgId = forceRef(record.value('id'))
        if csgId:
            for actionRecord in self.eventEditor.modelActionsSummary._items:
                actionCsgId = forceRef(actionRecord.value('eventCSG_id'))
                if actionCsgId == csgId:
                    actionRecord.setValue('eventCSG_id', QVariant(None))

    def updateCmbCSG(self):
        model = self.tblAPActions.model()
        items = model.items()
        row = self.tblAPActions.currentIndex().row()

        if 0<=row<len(items):# and (row == len(items) - 1):
            record, action = items[row]
            self.cmbCSG.setCurrentActionRecord(record)

    def setupActionPopupMenu(self):
        tbl = self.tblAPActions
        tbl.createPopupMenu([
                             self.actAPAddLikeAction,
                             self.actAPDuplicateAction,
                             self.actAPReplicateAction,
                             '-',
                             self.actAPAddAction,
                             self.actAPActionsAdd,
                             '-',
                             self.actAPActionSplitEvent
                             ])
        tbl.popupMenu().addSeparator()
        tbl.addMoveRow()
        tbl.popupMenu().addSeparator()
        tbl.addOpenInEditor()
        tbl.popupMenu().addSeparator()
        tbl.addPopupDelRow()
        tbl.setDelRowsChecker(lambda rows: not any(map(self.modelAPActions.isLockedOrExposed, rows)))
        tbl.setDelRowsIsExposed(lambda rowsExp: not any(map(self.modelAPActions.isExposed, rowsExp)))
        tbl.setActionsWithCheckers([(self.actAPActionSplitEvent, self.actAPActionSplitEventChecker)])
        tbl.addPopupRecordProperies()


    def setOrgId(self, orgId):
        self.cmbAPSetPerson.setOrgId(orgId)
        self.cmbAPPerson.setOrgId(orgId)
        self.cmbAPAssistant.setOrgId(orgId)


    def setBegDate(self, date):
        self.cmbAPSetPerson.setBegDate(date)
        self.cmbAPPerson.setBegDate(date)
        self.cmbAPAssistant.setBegDate(date)


    def setEndDate(self, date):
        self.cmbAPSetPerson.setEndDate(date)
        self.cmbAPPerson.setEndDate(date)
        self.cmbAPAssistant.setEndDate(date)

    def setCommonTakenTissueJournalRecordIdToActions(self):
        for record, action in self.modelAPActions.items():
            actionTypeId = forceRef(record.value('actionType_id'))
            if actionTypeId in self.allowedActionTypesByTissue:
                record.setValue('takenTissueJournal_id', toVariant(self.commonTakenTissueJournalRecordId))

    def updatePersonId(self, oldPersonId, newPersonId):
        self.modelAPActions.updatePersonId(oldPersonId, newPersonId)


    def loadActions(self, eventId):
        self.modelAPActions.loadItems(eventId)
        self.updateActionEditor()

    def setCommonTakenTissueJournalRecordId(self, id=None, actionTypeIdList=[]):
        self.commonTakenTissueJournalRecordId = id
        self.allowedActionTypesByTissue = actionTypeIdList

    def saveActions(self, eventId):
        if self.commonTakenTissueJournalRecordId:
            self.setCommonTakenTissueJournalRecordIdToActions()
        self.cmbCSG.saveCSG()
        self.modelAPActions.saveItems(eventId)


    def updateActionEditor(self):
        model = self.tblAPActions.model()
        self.cmbCSG.setItems()
        if model.rowCount() > 0:
            self.tblAPActions.setCurrentIndex(model.index(0, 0))
        else:
            for widget in (self.edtAPDirectionDate, self.edtAPDirectionTime,
                           self.edtAPPlannedEndDate, self.edtAPPlannedEndTime,
                           self.cmbAPSetPerson,
                           self.cmbAPStatus, self.edtAPBegDate, self.edtAPBegTime,
                           self.edtAPEndDate, self.edtAPEndTime,
                           self.cmbAPPerson, self.edtAPOffice,
                           self.cmbAPAssistant,
                           self.edtAPAmount, # self.edtAPUet,
                           self.edtAPNote,
                           self.tblAPProps,
                           self.btnAPPrint,
                           self.btnAPLoadTemplate, self.btnAPSaveAsTemplate, self.btnAPLoadPrevAction,
                           self.btnAPAttachedFiles,
                           self.lblCSG,  self.cmbCSG):
                widget.setEnabled(False)


    def updatePrintButton(self, actionType):
        context = actionType.context if actionType else ''
        customizePrintButton(self.btnAPPrint, context)


    def setCSGEditEnable(self, actionTypeId, canEdit):
        required = getEventCSGRequired(actionTypeId) if canEdit else False
        self.lblCSG.setEnabled(required)
        self.cmbCSG.setEnabled(required)


    def onActionCurrentChanged(self, previous=None):
        model = self.tblAPActions.model()
        items = model.items()
        row = self.tblAPActions.currentIndex().row()
        editWidgets = [self.edtAPDirectionDate, self.edtAPDirectionTime,
                       self.edtAPPlannedEndDate, self.edtAPPlannedEndTime,
                       self.cmbAPSetPerson,
                       self.cmbAPStatus, self.edtAPBegDate, self.edtAPBegTime,
                       self.edtAPEndDate, self.edtAPEndTime, self.cmbAPOrg,
                       self.cmbAPPerson, self.edtAPOffice,
                       self.cmbAPAssistant,
                       self.edtAPAmount, # self.edtAPUet,
                       self.edtAPNote,
                       self.btnAPLoadTemplate, self.btnAPLoadPrevAction,
                       self.btnAPAttachedFiles,
                       self.cmbCSG, self.lblCSG]
        otherWidgets = [self.tblAPProps, self.btnAPPrint, self.btnAPSaveAsTemplate]
        mkbWidgets   = [self.cmbAPMKB, self.cmbAPMorphologyMKB]
        if 0<=row<len(items):
            record, action = items[row]
        else:
            record = action = None
        if record:
            actionTypeId = forceRef(record.value('actionType_id'))
            actionType = CActionTypeCache.getById(actionTypeId) if actionTypeId else None
            personId = forceRef(record.value('person_id'))
            financeId = forceRef(record.value('finance_id'))
            contractId = forceRef(record.value('contract_id'))
        else:
            actionType = actionTypeId = None
            self.defaultDirectionDate = CActionType.dddUndefined
            self.defaultEndDate = CActionType.dedUndefined
        if actionType:
            self.cmbCSG.setCurrentActionRecord(record)
            if previous:
                self.tblAPProps.savePreferencesLoc(previous.row())
            setActionPropertiesColumnVisible(actionType, self.tblAPProps)
            self.tblAPProps.resizeColumnsToContents()
            self.tblAPProps.resizeRowsToContents()
            self.tblAPProps.loadPreferencesLoc(self.tblAPProps.preferencesLocal, row)
            self.defaultDirectionDate = actionType.defaultDirectionDate
            self.defaultEndDate = actionType.defaultEndDate
            visibleMKB = True
            if actionType.defaultMKB == CActionType.dmkbNotUsed:
                enableMKB  = False
                visibleMKB = False
            elif actionType.defaultMKB in (CActionType.dmkbSyncFinalDiag,
                                           CActionType.dmkbSyncSetPersonDiag):
                enableMKB = not forceString(record.value('MKB'))
            else:
                enableMKB = True
            dentitionCanEdit = True
            if ((u'dentitionInspection'.lower() in actionType.flatCode.lower()) or (u'parodentInsp'.lower() in actionType.flatCode.lower())):
                dentitionCanEdit = False
            if action:
                canEditIsLocked  = not action.isLocked()
                canEditIsExposed = not action.isExposed()
            else:
                canEditIsLocked = True
                canEditIsExposed = True
            enableEdit = True
            enableClosed = True
            if self.eventEditor:
                isClosed = self.eventEditor.tabNotes.isEventClosed()
                enableClosed = not isClosed or QtGui.qApp.userHasRight(urEditClosedEvent)
            canEdit = canEditIsLocked and canEditIsExposed and dentitionCanEdit and enableEdit
            if enableClosed:
                editWidgets.append(self.chkAPIsUrgent)
            for widget in editWidgets:
                widget.setEnabled(canEdit)
            self.setCSGEditEnable(self.eventEditor.eventTypeId, canEdit)
            self.lblAPMKB.setVisible(visibleMKB)
            self.cmbAPMKB.setVisible(visibleMKB)
            self.cmbAPMKB.setEnabled(canEdit and enableMKB)

            self.lblAPMorphologyMKB.setVisible(self._visibleMorphologyMKB and visibleMKB)
            self.cmbAPMorphologyMKB.setVisible(self._visibleMorphologyMKB and visibleMKB)
            self.cmbAPMorphologyMKB.setEnabled(canEdit and enableMKB)

            for widget in otherWidgets:
                widget.setEnabled(canEditIsExposed and dentitionCanEdit and enableEdit)
            self.btnAPNomenclatureExpense.setEnabled(canEdit and actionType.isNomenclatureExpense)

            orgStructureList = actionType.getPFOrgStructureRecordList()
            specialityList = actionType.getPFSpecialityRecordList()
            tablePerson = QtGui.qApp.db.table('vrbPersonWithSpeciality')
            orgStructureIdList = [forceInt(orgStructureRecord.value('orgStructure_id')) for orgStructureRecord in orgStructureList]
            specialityIdList      = [forceInt(specialityRecord.value('speciality_id'))     for specialityRecord in specialityList]
            pFilter= []
            if orgStructureIdList:
                pFilter.append(tablePerson['orgStructure_id'].inlist(orgStructureIdList))
            if specialityIdList:
                pFilter.append(tablePerson['speciality_id'].inlist(specialityIdList))
            self.cmbAPPerson.blockSignals(True)
            self.cmbAPPerson.setFilter(QtGui.qApp.db.joinAnd(pFilter) if pFilter else None)
            self.cmbAPPerson.blockSignals(False)

            try:
                relegateOrgId = self.eventEditor.tabNotes.cmbRelegateOrg.value()
                for widget in editWidgets+mkbWidgets:
                    widget.blockSignals(True)
#                self.cmbAPMKB.blockSignals(True)
#                if ((u'moving' in actionType.flatCode.lower()) or (u'received' in actionType.flatCode.lower())) and (row == len(items) - 1):
                if ((u'moving' in actionType.flatCode.lower()) or (u'received' in actionType.flatCode.lower())):
                    self.onCurrentActionAdd(actionType.flatCode, action)
                elif relegateOrgId and (relegateOrgId != QtGui.qApp.currentOrgId()
                    and ('recoveryDirection'.lower() in actionType.flatCode.lower()
                    or u'inspectionDirection'.lower() in actionType.flatCode.lower()
                    or u'researchDirection'.lower() in actionType.flatCode.lower()
                    or u'consultationDirection'.lower() in actionType.flatCode.lower())
                   ):
                    pass
                else:
                    if (self.edtAPDuration.value() > 0 or self.edtAPAliquoticity.value() > 0) and self.edtAPEndDate.date().isNull() and self.cmbAPStatus.value() != CActionStatus.canceled:
                        self.btnNextAction.setText(u'Выполнить')
                        self.btnNextAction.setEnabled(dentitionCanEdit and True and enableEdit)
                    else:
                        self.btnNextAction.setText(u'Действие')
                        self.btnNextAction.setEnabled(dentitionCanEdit and False and enableEdit)
                    if self.edtAPDuration.value() > 0:
                        self.btnPlanNextAction.setEnabled(canEditIsExposed and dentitionCanEdit and True and enableEdit)
                    else:
                        self.btnPlanNextAction.setEnabled(canEditIsExposed and dentitionCanEdit and False and enableEdit)
                if action and u'Серия и номер бланка' in actionType._propertiesByName:
                    actionPropertyType = actionType._propertiesByName[u'Серия и номер бланка']
                    if actionPropertyType.valueDomain.lower() in [u'a', u'а']: #wtf
                        action[u'Серия и номер бланка'] = self.getBlankSerialNumberParams(actionType.id)
                self.on_edtAPDuration_valueChanged(forceInt(record.value('duration')))
                showTime = actionType.showTime if actionType else False
                self.edtAPDirectionTime.setVisible(showTime)
                self.edtAPPlannedEndTime.setVisible(showTime)
                self.edtAPCoordTime.setVisible(showTime)
                self.edtAPBegTime.setVisible(showTime)
                self.edtAPEndTime.setVisible(showTime)
                showAssistant = actionType.hasAssistant if actionType else False
                self.lblAPAssistant.setVisible(showAssistant)
                self.cmbAPAssistant.setVisible(showAssistant)
                setDatetimeEditValue(self.edtAPDirectionDate,    self.edtAPDirectionTime,    record, 'directionDate')
                setDatetimeEditValue(self.edtAPPlannedEndDate,   self.edtAPPlannedEndTime,   record, 'plannedEndDate')
                self.chkAPIsUrgent.setChecked(forceBool(record.value('isUrgent')))
                setDatetimeEditValue(self.edtAPCoordDate, self.edtAPCoordTime, record, 'coordDate')
                self.lblAPCoordText.setText(forceString(record.value('coordText')))
                setDatetimeEditValue(self.edtAPBegDate, self.edtAPBegTime, record, 'begDate')
                setDatetimeEditValue(self.edtAPEndDate, self.edtAPEndTime, record, 'endDate')
                self.APEndDate = forceDate(record.value('endDate'))
                self.APPerson = personId
                self.edtAPDirectionTime.setEnabled(canEdit and bool(self.edtAPDirectionDate.date()))
                canEditPlannedEndDate = canEdit and bool(actionType.defaultPlannedEndDate not in (CActionType.dpedBegDatePlusAmount,
                                                                                             CActionType.dpedBegDatePlusDuration))
                self.edtAPPlannedEndDate.setEnabled(canEditIsExposed and canEditPlannedEndDate and enableEdit)
                self.edtAPPlannedEndTime.setEnabled(canEditIsExposed and canEditPlannedEndDate and bool(self.edtAPPlannedEndDate.date()) and enableEdit)
                self.cmbAPSetPerson.setValue(forceRef(record.value('setPerson_id')))
                self.edtAPBegTime.setEnabled(canEdit and bool(self.edtAPBegDate.date()))
                self.edtAPEndTime.setEnabled(canEdit and bool(self.edtAPEndDate.date()))
                self.cmbAPStatus.setValue(forceInt(record.value('status')))
                self.cmbAPPerson.setValue(personId)
                self.edtAPOffice.setText(forceString(record.value('office')))
                self.cmbAPAssistant.setValue(forceRef(record.value('assistant_id')))
                self.edtAPDuration.setValue(forceInt(record.value('duration')))
                self.edtAPPeriodicity.setValue(forceInt(record.value('periodicity')))
                self.edtAPAliquoticity.setValue(forceInt(record.value('aliquoticity')))
                amount = forceDouble(record.value('amount'))
                self.edtAPAmount.setValue(amount)
                self.edtAPAmount.setEnabled(canEdit and bool(actionType) and actionType.amountEvaluation == 0)
                uet = forceDouble(record.value('uet'))
                if amount and uet == 0:
                    uet = amount*self.eventEditor.getUet(actionTypeId, personId, financeId, contractId)
                self.edtAPUet.setValue(uet)
#                self.edtAPUet.setEnabled(canEditIsExposed and False)
                self.edtAPNote.setText(forceString(record.value('note')))
                MKB = forceString(record.value('MKB'))
                self.cmbAPMKB.setText(MKB)
                self.cmbAPMorphologyMKB.setText(forceString(record.value('morphologyMKB')))
                self.cmbAPOrg.setValue(forceRef(record.value('org_id')))
                self.on_cmbAPMKB_textChanged(MKB)
                self.tblAPProps.setEnabled(canEditIsExposed and bool(action) and dentitionCanEdit and enableEdit)
                #self.tblAPProps.setEnabled(True)
                self.updatePropTable(action)
                self.updatePrintButton(actionType)
                self.btnAPAttachedFiles.setAttachedFileItemList(action.getAttachedFileItemList())

                if QtGui.qApp.userHasRight(urLoadActionTemplate) and action and (self.cmbAPStatus.value() != CActionStatus.finished or not self.cmbAPPerson.value() or QtGui.qApp.userId == self.cmbAPPerson.value() or QtGui.qApp.userHasRight(urEditOtherpeopleAction)):
                    actionTemplateTreeModel = self.actionTemplateCache.getModel(action.getType().id)
                    self.btnAPLoadTemplate.setModel(actionTemplateTreeModel)
                else:
                    self.btnAPLoadTemplate.setEnabled(False)
                self.btnAPSaveAsTemplate.setEnabled(canEditIsExposed
                                                    and dentitionCanEdit
                                                    and QtGui.qApp.userHasRight(urSaveActionTemplate)
                                                    and enableEdit)
                self.btnAPLoadPrevAction.setEnabled(canEdit
                                                    and QtGui.qApp.userHasRight(urCopyPrevAction)
                                                    and bool(action))
                self.edtAPDuration.setEnabled(canEditIsExposed
                                                    and dentitionCanEdit
                                                    and QtGui.qApp.userHasRight(urCopyPrevAction)
                                                    and QtGui.qApp.userId == self.cmbAPSetPerson.value()
                                                    and not action.getExecutionPlan()
                                                    and self.cmbAPStatus.value() in (CActionStatus.started, CActionStatus.finished)
                                                    and self.edtAPEndDate.date().isNull()
                                                    and enableEdit)
                self.edtAPPeriodicity.setEnabled(canEditIsExposed
                                                    and dentitionCanEdit
                                                    and QtGui.qApp.userHasRight(urCopyPrevAction)
                                                    and QtGui.qApp.userId == self.cmbAPSetPerson.value()
                                                    and not action.getExecutionPlan()
                                                    and self.cmbAPStatus.value() in (CActionStatus.started, CActionStatus.appointed)
                                                    and self.edtAPEndDate.date().isNull()
                                                    and enableEdit)
                self.edtAPAliquoticity.setEnabled(canEditIsExposed
                                                    and dentitionCanEdit
                                                    and QtGui.qApp.userHasRight(urCopyPrevAction)
                                                    and QtGui.qApp.userId == self.cmbAPSetPerson.value()
                                                    and not action.getExecutionPlan()
                                                    and self.cmbAPStatus.value() in (CActionStatus.started, CActionStatus.appointed)
                                                    and self.edtAPEndDate.date().isNull()
                                                    and enableEdit)
                self._canUseLaboratoryCalculator = False
                QtGui.qApp.disconnectClipboard()
            finally:
                for widget in editWidgets+mkbWidgets:
                    widget.blockSignals(False)
                self.edtAPPlannedEndDate.setEnabled(canEditIsExposed and dentitionCanEdit and self.edtAPPlannedEndDate.isEnabled() and enableEdit)
                self.edtAPDirectionDate.setEnabled(canEditIsExposed and dentitionCanEdit and self.edtAPDirectionDate.isEnabled() and enableEdit)
                self.edtAPBegDate.setEnabled(canEditIsExposed and dentitionCanEdit and self.edtAPBegDate.isEnabled() and enableEdit)
                self.edtAPEndDate.setEnabled(canEditIsExposed and dentitionCanEdit and self.edtAPEndDate.isEnabled() and enableEdit)
                self.cmbAPSetPerson.setEnabled(canEditIsExposed and dentitionCanEdit and self.cmbAPSetPerson.isEnabled() and enableEdit)
                self.btnAPSelectOrg.setEnabled(canEditIsExposed and dentitionCanEdit and self.btnAPSelectOrg.isEnabled() and enableEdit)
#                self.cmbAPMKB.blockSignals(False)
        else:
            self.cmbAPPerson.blockSignals(True)
            self.cmbAPPerson.setFilter(None)
            self.cmbAPPerson.blockSignals(False)
            self.defaultDirectionDate = CActionType.dddUndefined
            self.defaultEndDate = CActionType.dedUndefined
            for widget in editWidgets+mkbWidgets:
                widget.setEnabled(False)
#            self.cmbAPMKB.setEnabled(False)
            for widget in otherWidgets:
                widget.setEnabled(False)
            self.edtAPDirectionTime.setVisible(False)
            self.edtAPPlannedEndTime.setVisible(False)
            self.edtAPBegTime.setVisible(False)
            self.edtAPEndTime.setVisible(False)
            if (self.edtAPDuration.value() > 0 or self.edtAPAliquoticity.value() > 0) and self.edtAPEndDate.date().isNull() and self.cmbAPStatus.value() != CActionStatus.canceled:
                self.btnNextAction.setText(u'Выполнить')
                self.btnNextAction.setEnabled(True)
            else:
                self.btnNextAction.setText(u'Действие')
                self.btnNextAction.setEnabled(False)
            if self.edtAPDuration.value() > 0:
                self.btnPlanNextAction.setEnabled(True)
            else:
                self.btnPlanNextAction.setEnabled(False)
            self.btnAPNomenclatureExpense.setEnabled(False)
        self.cmbAPMKB.setFilter('''case when MKB.endDate IS NOT NULL then MKB.endDate >= %s else true end'''%(QtGui.qApp.db.formatDate(self.edtAPBegDate.date())))


    def getBlankSerialNumberParams(self, docTypeId):
        blankParams = {}
        blankIdList = []
        text = ''
        if docTypeId:
            db = QtGui.qApp.db
            personId = QtGui.qApp.userId
            tableRBBlankTempInvalids = db.table('rbBlankActions')
            tableBlankTempInvalidParty = db.table('BlankActions_Party')
            tableBlankTempInvalidMoving = db.table('BlankActions_Moving')
            tableTempInvalid = db.table('TempInvalid')
            tablePerson = db.table('Person')
            orgStructureId = None
            if personId:
                orgStructRecord = db.getRecordEx(tablePerson, [tablePerson['orgStructure_id']], [tablePerson['deleted'].eq(0), tablePerson['id'].eq(personId)])
                orgStructureId = forceRef(orgStructRecord.value('orgStructure_id')) if orgStructRecord else None

            cols = [tableBlankTempInvalidMoving['id'].alias('blankMovingId'),
                    tableBlankTempInvalidParty['serial'],
                    tableBlankTempInvalidMoving['numberFrom'],
                    tableBlankTempInvalidMoving['numberTo'],
                    tableBlankTempInvalidMoving['returnAmount'],
                    tableBlankTempInvalidMoving['used'],
                    tableBlankTempInvalidMoving['received']
                    ]
            cond = [tableRBBlankTempInvalids['doctype_id'].eq(docTypeId),
                    tableBlankTempInvalidParty['deleted'].eq(0),
                    tableBlankTempInvalidMoving['deleted'].eq(0)
                    ]
            cond.append('''BlankActions_Moving.received > (BlankActions_Moving.used - BlankActions_Moving.returnAmount)''')
            order = []
            queryTable = tableBlankTempInvalidParty.innerJoin(tableBlankTempInvalidMoving, tableBlankTempInvalidMoving['blankParty_id'].eq(tableBlankTempInvalidParty['id']))
            order = []
            orgStructureIdList = db.getTheseAndParents('OrgStructure', 'parent_id', [orgStructureId]) if orgStructureId else []
            if personId and orgStructureIdList:
                cond.append(db.joinOr([tableBlankTempInvalidMoving['person_id'].eq(personId), tableBlankTempInvalidMoving['orgStructure_id'].inlist(orgStructureIdList)]))
                order.append(u'BlankActions_Moving.person_id')
            elif personId:
                cond.append(tableBlankTempInvalidMoving['person_id'].eq(personId))
            elif orgStructureIdList:
                cond.append(tableBlankTempInvalidMoving['orgStructure_id'].inlist(orgStructureIdList))
            queryTable = tableRBBlankTempInvalids.innerJoin(tableBlankTempInvalidParty, tableBlankTempInvalidParty['doctype_id'].eq(tableRBBlankTempInvalids['id']))
            queryTable = queryTable.innerJoin(tableBlankTempInvalidMoving, tableBlankTempInvalidMoving['blankParty_id'].eq(tableBlankTempInvalidParty['id']))
            order.append(u'BlankActions_Moving.numberFrom ASC')
            records = db.getRecordList(queryTable, cols, cond, order)
            for record in records:
                blankInfo = {}
                blankMovingId = forceRef(record.value('blankMovingId'))
                serial = forceString(record.value('serial'))
                numberFromString = forceString(record.value('numberFrom'))
                numberFromFirstChar = numberFromString[:1]
                numberFrom = int(numberFromString) if numberFromString else 0
                numberToString = forceString(record.value('numberTo'))
                numberTo = int(numberToString) if numberToString else 0
                returnAmount = forceInt(record.value('returnAmount'))
                used = forceInt(record.value('used'))
                received = forceInt(record.value('received'))
                blankInfo['serial'] = serial
                blankInfo['numberFromFirstChar'] = numberFromFirstChar
                blankInfo['numberFrom'] = numberFrom
                blankInfo['numberTo'] = numberTo
                blankInfo['returnAmount'] = returnAmount
                blankInfo['used'] = used
                blankInfo['received'] = received
                blankParams[blankMovingId] = blankInfo
                blankIdList.append(blankMovingId)
            if blankIdList:
                movingId = blankIdList[0]
            else:
                movingId = None
            if movingId:
                blankInfo = blankParams.get(movingId, None)
                if blankInfo:
                    serial = forceString(blankInfo.get('serial', u''))
                    numberFromFirstChar = blankInfo.get('numberFromFirstChar', u'')
                    numberFrom = blankInfo.get('numberFrom', 0)
                    numberTo = blankInfo.get('numberTo', 0)
                    returnAmount = forceInt(blankInfo.get('returnAmount', 0))
                    used = forceInt(blankInfo.get('used', 0))
                    received = forceInt(blankInfo.get('received', 0))
                    balance = received - used - returnAmount
                    if balance > 0:
                        number = numberFrom + used + returnAmount
                        if number <= numberTo:
                            numberBlank = number
                            if numberFromFirstChar and numberFromFirstChar == u'0':
                                numberBlank = numberFromFirstChar + forceString(number)
                            record = db.getRecordEx(tableTempInvalid, [tableTempInvalid['id']], [tableTempInvalid['deleted'].eq(0), tableTempInvalid['serial'].eq(serial), tableTempInvalid['number'].eq(numberBlank)])
                            if not record:
                                text = forceString(serial) + u' ' + forceString(numberBlank)
        return  text


    def on_actSetLaboratoryCalculatorInfo(self):
        result = self.eventEditor.checkNeedLaboratoryCalculator(self.modelAPActionProperties.propertyTypeList,
                                                                self.on_laboratoryCalculatorClipboard)
        self._canUseLaboratoryCalculatorPropertyTypeList = result
        self.setInfoToLaboratoryCalculatorClipboard()


    def setInfoToLaboratoryCalculatorClipboard(self):
        def chooseLabCode(propType):
            code = propType.laboratoryCalculator
            if code in ('LL*', 'GG*', 'EE*', 'CI*'):
                return '%s%s'%(code, forceString(self.modelAPActionProperties.action.getPropertyById(propType.id).getValue()))
            else:
                return code
        if self._canUseLaboratoryCalculatorPropertyTypeList:
            propertyTypeList = self._canUseLaboratoryCalculatorPropertyTypeList
            actual = unicode('; '.join(['('+','.join([forceString(propType.id),
                chooseLabCode(propType),#'LL*%s'%forceString(self.modelAPActionProperties.action.getPropertyByIndex(i).getValue()) if propType.laboratoryCalculator == 'LL*' else propType.laboratoryCalculator,
                propType.name])+')' for propType in propertyTypeList]))
            QtGui.qApp.log(u'Передача в лаб кальк', actual)
            mimeData = QMimeData()
            mimeData.setData(QtGui.qApp.inputCalculatorMimeDataType,
                             QString(actual).toUtf8())
            QtGui.qApp.clipboard().setMimeData(mimeData)
            self._mainWindowState = QtGui.qApp.mainWindow.windowState()
            QtGui.qApp.mainWindow.showMinimized()

    def on_laboratoryCalculatorClipboard(self):
        QtGui.qApp.setActiveWindow(self.eventEditor)
        mimeData = QtGui.qApp.clipboard().mimeData()
        baData = mimeData.data(QtGui.qApp.outputCalculatorMimeDataType)
        if baData:
            QtGui.qApp.mainWindow.setWindowState(self._mainWindowState)
            data = forceString(QString.fromUtf8(baData))
            self.modelAPActionProperties.setLaboratoryCalculatorData(data)


    def onCurrentActionAdd(self, flatCode, action):
        model = self.tblAPActions.model()
        items = model.items()
        relegateOrgId = self.eventEditor.tabNotes.cmbRelegateOrg.value()
        if relegateOrgId and (relegateOrgId != QtGui.qApp.currentOrgId()
            and self.cmbAPStatus.currentIndex() in (CActionStatus.started, CActionStatus.wait, CActionStatus.withoutResult, CActionStatus.appointed)
            and ('recoveryDirection'.lower() in flatCode.lower()
            or u'inspectionDirection'.lower() in flatCode.lower()
            or u'researchDirection'.lower() in flatCode.lower()
            or u'consultationDirection'.lower() in flatCode.lower())
           ):
            self.btnNextAction.setText(u'Выполнить')
            self.btnNextAction.setEnabled(True)
        else:
            noPresentLeaved = True
            for item in items:
                record, actionItem = item
                if record:
                    actionTypeId = forceRef(record.value('actionType_id'))
                    actionType = CActionTypeCache.getById(actionTypeId) if actionTypeId else None
                    if u'leaved' in actionType.flatCode.lower():
                        noPresentLeaved = False
                        break
            if noPresentLeaved:
                if u'received' in flatCode.lower():
                    if action[u'Направлен в отделение']:
                        self.btnNextAction.setText(u'Перевод')
                        self.btnNextAction.setEnabled(self.edtAPEndDate.date().isNull())
                    else:
                        self.btnNextAction.setText(u'Выписка')
                        self.btnNextAction.setEnabled(self.edtAPEndDate.date().isNull())
                elif u'moving' in flatCode.lower():
                    if action[u'Переведен в отделение']:
                        self.btnNextAction.setText(u'Перевод')
                        self.btnNextAction.setEnabled(self.edtAPEndDate.date().isNull())
                    else:
                        self.btnNextAction.setText(u'Выписка')
                        self.btnNextAction.setEnabled(self.edtAPEndDate.date().isNull())
                elif u'inspectPigeonHole'.lower() in flatCode.lower():
                    self.btnNextAction.setText(u'Закончить')
                    self.btnNextAction.setEnabled(self.edtAPEndDate.date().isNull())
                else:
                    if (self.edtAPDuration.value() > 0 or self.edtAPAliquoticity.value() > 0) and self.edtAPEndDate.date().isNull() and self.cmbAPStatus.value() != CActionStatus.canceled:
                        self.btnNextAction.setText(u'Выполнить')
                        self.btnNextAction.setEnabled(True)
                    else:
                        self.btnNextAction.setText(u'Действие')
                        self.btnNextAction.setEnabled(False)
                    if self.edtAPDuration.value() > 0:
                        self.btnPlanNextAction.setEnabled(True)
                    else:
                        self.btnPlanNextAction.setEnabled(False)
            else:
                if (self.edtAPDuration.value() > 0 or self.edtAPAliquoticity.value() > 0) and self.edtAPEndDate.date().isNull() and self.cmbAPStatus.value() != CActionStatus.canceled:
                    self.btnNextAction.setText(u'Выполнить')
                    self.btnNextAction.setEnabled(True)
                else:
                    self.btnNextAction.setText(u'Действие')
                    self.btnNextAction.setEnabled(False)
                if self.edtAPDuration.value() > 0:
                    self.btnPlanNextAction.setEnabled(True)
                else:
                    self.btnPlanNextAction.setEnabled(False)


    def getIdListActionType(self, flatCode):
        actionTypeId = None
        db = QtGui.qApp.db
        tableActionType = db.table('ActionType')
        if flatCode==u'moving':
            idList = db.getIdList(tableActionType, [tableActionType['id']], [tableActionType['flatCode'].like(flatCode), tableActionType['deleted'].eq(0)])
            idListActionType = []
            for actionTypeId in idList:
                actionType = CActionTypeCache.getById(actionTypeId) if actionTypeId else None
                orgStructureList = actionType.getPFOrgStructureRecordList()
                specialityList = actionType.getPFSpecialityRecordList()
                orgStructureIdList = [forceInt(orgStructureRecord.value('orgStructure_id')) for orgStructureRecord in orgStructureList]
                specialityIdList      = [forceInt(specialityRecord.value('speciality_id'))     for specialityRecord in specialityList]
                if len(orgStructureIdList) or len(specialityIdList):
                    for orgStructureId in orgStructureIdList:
                        if orgStructureId == QtGui.qApp.userOrgStructureId and actionTypeId not in idListActionType:
                            idListActionType.append(actionTypeId)
                    for specialityId in specialityIdList:
                        if specialityId == QtGui.qApp.userSpecialityId and actionTypeId not in idListActionType:
                            idListActionType.append(actionTypeId)
                else:
                    if actionTypeId not in idListActionType:
                        idListActionType.append(actionTypeId)
        else:
            idListActionType = db.getIdList(tableActionType, [tableActionType['id']], [tableActionType['flatCode'].like(flatCode), tableActionType['deleted'].eq(0)])
        if len(idListActionType) > 1:
            dialogActionType = CActionTypeDialogTableModel(self, idListActionType)
            if dialogActionType.exec_():
                actionTypeId= dialogActionType.currentItemId()
        else:
            actionTypeId = idListActionType[0] if idListActionType else None
        return actionTypeId


    def checkActionByNextEventCreation(self):
        if QtGui.qApp.isNextEventCreationFromAction() == 1:
            res = QtGui.QMessageBox.warning(self,
                                   u'Внимание!',
                                   u'Добавить движение во вновь зарегистрированное Событие?',
                                   QtGui.QMessageBox.Yes|QtGui.QMessageBox.No|QtGui.QMessageBox.Help,
                                   QtGui.QMessageBox.No)
            if res == QtGui.QMessageBox.Yes:
                return True
            elif res == QtGui.QMessageBox.Help:
                QtGui.QMessageBox.information(self,
                            u'Справка',
                            u'При согласии, действие "Движение" будет добавлено в новое событие, текущее событие будет закрыто. Это необходимо в том случае, если перевод пациента связан с изменением МЭСа.')
                res = self.checkActionByNextEventCreation()
                return res
            else:
                return False
        elif QtGui.qApp.isNextEventCreationFromAction() == 2:
            return True
        return False


    def checkPolicyOnActionsEndDate(self):
        for row, (record, action) in enumerate(self.modelAPActions.items()):
            actionId = forceRef(record.value('id'))
            if not actionId in self.modelAPActions.loadedActionIdListWithEndDate():
                if not checkPolicyOnDate(self.clientId(), forceDate(record.value('endDate'))):
                    skippable = QtGui.qApp.isStrictCheckPolicyOnEndAction() == 0
                    if not self.eventEditor.checkInputMessage(u'время закрытия действия соответствующее полису',
                                                              skippable, self.tblAPActions, row=row, column=0):
                        self.edtAPEndDate.setFocus(Qt.ShortcutFocusReason)
                        return False
        return True


    def checkAttachOnActionsEndDate(self):
        for row, (record, action) in enumerate(self.modelAPActions.items()):
            actionId = forceRef(record.value('id'))
            if not actionId in self.modelAPActions.loadedActionIdListWithEndDate():
                if not checkAttachOnDate(self.clientId(), forceDate(record.value('endDate'))):
                    skippable = QtGui.qApp.isStrictCheckAttachOnEndAction() == 0
                    if not self.eventEditor.checkInputMessage(u'время закрытия действия соответствующее прикреплению',
                                                              skippable, self.tblAPActions, row=row, column=0):
                        self.edtAPEndDate.setFocus(Qt.ShortcutFocusReason)
                        return False
        return True


    def checkAPDataEntered(self):
        isStrictCheckPolicyOnEndAction = QtGui.qApp.isStrictCheckPolicyOnEndAction()
        isStrictCheckAttachOnEndAction = QtGui.qApp.isStrictCheckAttachOnEndAction()
        checkPolicyOnEndAction = isStrictCheckPolicyOnEndAction in (0, 1) or isStrictCheckAttachOnEndAction in (0, 1)
        for row, (record, action) in enumerate(self.modelAPActions.items()):
            actionId = forceRef(record.value('id'))
            if checkPolicyOnEndAction:
                if not actionId in self.modelAPActions.loadedActionIdListWithEndDate():
                    if isStrictCheckPolicyOnEndAction in (0, 1):
                        if not checkPolicyOnDate(self.clientId(), forceDate(record.value('endDate'))):
                            skippable = isStrictCheckPolicyOnEndAction == 0
                            if not self.eventEditor.checkInputMessage(u'время закрытия действия соответствующее полису',
                                                                      skippable, self.tblAPActions, row=row, column=0):
                                self.edtAPEndDate.setFocus(Qt.ShortcutFocusReason)
                                return False
                    if isStrictCheckAttachOnEndAction in (0, 1):
                        if not checkAttachOnDate(self.clientId(), forceDate(record.value('endDate'))):
                            skippable = isStrictCheckAttachOnEndAction == 0
                            if not self.eventEditor.checkInputMessage(u'время закрытия действия соответствующее прикреплению',
                                                                      skippable, self.tblAPActions, row=row, column=0):
                                self.edtAPEndDate.setFocus(Qt.ShortcutFocusReason)
                                return False
            if action.nomenclatureExpense:
                if not action.nomenclatureExpense.stockMotionItems():
                    if not self.eventEditor.checkInputMessage(u'списываемые ЛСиИМН',
                                                              True, self.tblAPActions, row=row, column=0):
                        self.btnAPNomenclatureExpense.setFocus(Qt.ShortcutFocusReason)
                        return False
        return self.checkActualMKB()


    def checkActualMKB(self):
            result = True
            MKB = unicode(self.cmbAPMKB.text())
            if MKB:
                db = QtGui.qApp.db
                tableMKB = db.table('MKB')
                cond = [tableMKB['DiagID'].eq(MKBwithoutSubclassification(MKB))]
                cond.append(db.joinOr([tableMKB['endDate'].isNull(), tableMKB['endDate'].dateGe(self.edtAPBegDate.date())] ))
                recordMKB = db.getRecordEx(tableMKB, [tableMKB['DiagID']], cond)
                result = result and (forceString(recordMKB.value('DiagID')) == MKBwithoutSubclassification(MKB) if recordMKB else False) or self.eventEditor.checkValueMessage(u'Диагноз %s не доступен для применения'%MKB, False, self.cmbAPMKB)
            return result


    @pyqtSignature('')
    def on_btnPlanNextAction_clicked(self):
        model = self.tblAPActions.model()
        items = model.items()
        row = self.tblAPActions.currentIndex().row()
        if 0<=row<len(items):
            record, action = items[row]
            dialog = CGetExecutionPlan(self, record, action.getExecutionPlan())
            dialog.exec_()
            action.setExecutionPlan(dialog.model.executionPlan)

    def _onNextActionIsInspectPigeonHole(self, record, endDate):
        endDate = forceDateTime(record.value('endDate'))
        if not endDate:
            record.setValue('endDate', toVariant(endDate))
        record.setValue('status', toVariant(2))
        if hasattr(self.eventEditor, 'btnNextActionSetFocus'):
            self.eventEditor.btnNextActionSetFocus()

    def _onNextActionIsReceived(self, action, actionType, record, jobTicketEndDateAskingIsRequired, nextChiefId, dateTime):
        db = QtGui.qApp.db
        tableEventType = db.table('EventType')

        model = self.tblAPActions.model()

        orgStructureMoving = action[u'Направлен в отделение']
        hospitalBedProfileId = action[u'Профиль'] if u'Профиль' in action._actionType._propertiesByName else None
        receivedQuoting = action[u'Квота'] if u'Квота' in action._actionType._propertiesByName else None
        currentOrgStructureId = None
        if orgStructureMoving:
            actionTypeId = self.getIdListActionType(u'moving')
            if not actionTypeId:
                        QtGui.QMessageBox.critical(self,
                        u'Внимание!',
                        u'Не найдено подходящего действия "Движение" для Ваших специальности и подразделения',
                        QtGui.QMessageBox.Ok,
                        QtGui.QMessageBox.Ok)
            chiefId = getChiefId(orgStructureMoving) if not jobTicketEndDateAskingIsRequired else nextChiefId
            if chiefId:
                setPersonTRId = forceRef(record.value('person_id'))
                self.eventEditor.mapJournalInfoTransfer = [
                    chiefId, forceDateTime(record.value('endDate')), setPersonTRId if setPersonTRId else self.eventEditor.cmbPerson.value()
                ]
                self.eventEditor.cmbPerson.setValue(chiefId)
        else:
            actionTypeId = self.getIdListActionType(u'leaved')
            currentOrgStructureId = QtGui.qApp.currentOrgStructureId()
        if actionTypeId:
            oldBegDate = forceDateTime(record.value('endDate'))
            if not oldBegDate:
                record.setValue('endDate', toVariant(dateTime))
            record.setValue('status', toVariant(2))
            self.receivedFinanceId = forceRef(record.value('finance_id'))
            self.updateAmount()
            model.addRow(actionTypeId, actionType.amount)
            record, action = model.items()[-1]
            eventTypeId = self.eventEditor.getEventTypeId()
            if eventTypeId:
                recordET = db.getRecordEx(
                    tableEventType, [tableEventType['actionFinance']],
                    [tableEventType['deleted'].eq(0), tableEventType['id'].eq(eventTypeId)]
                )
                self.eventActionFinance = forceInt(recordET.value('actionFinance')) if recordET else None
                if self.eventActionFinance == 0:
                    record.setValue('finance_id', toVariant(self.receivedFinanceId))
            if orgStructureMoving:
                action[u'Отделение пребывания'] = orgStructureMoving
            elif currentOrgStructureId:
                typeRecord = db.getRecordEx(
                    'OrgStructure', 'type', 'id = %d AND type = 4 AND deleted = 0' % (currentOrgStructureId)
                )
                if typeRecord and (typeRecord.value('type')) == 4:
                    if u'Отделение' in action._actionType._propertiesByName:
                        action[u'Отделение'] = currentOrgStructureId
            if u'Профиль' in action._actionType._propertiesByName and hospitalBedProfileId:
                action[u'Профиль'] = hospitalBedProfileId
            if oldBegDate:
                record.setValue('begDate', toVariant(oldBegDate))
                record.setValue('directionDate', toVariant(oldBegDate))
            else:
                record.setValue('begDate', toVariant(dateTime))
                record.setValue('directionDate', toVariant(dateTime))
            actionTypeOfNewItem = action.getType()
            if actionTypeOfNewItem.defaultEndDate == 4:
                record.setValue('endDate', toVariant(dateTime))
            if u'Квота' in action._actionType._propertiesByName and receivedQuoting:
                action[u'Квота'] = receivedQuoting

    def _onNextActionIsMoving(self, action, actionType, record, jobTicketEndDateAskingIsRequired, nextChiefId, dateTime, needCutFeed):
        db = QtGui.qApp.db
        model = self.tblAPActions.model()
        eventId = forceRef(record.value('event_id')) if record else None
        orgStructureTransfer = action[u'Переведен в отделение']
        orgStructurePresence = action[u'Отделение пребывания']
        hospitalBedId = action[u'койка']
        oldBegDate = forceDateTime(record.value('endDate'))
        if not oldBegDate:
            record.setValue('endDate', toVariant(dateTime))
            oldBegDate = dateTime
        record.setValue('status', toVariant(2))
        movingQuoting = action[u'Квота'] if u'Квота' in action._actionType._propertiesByName else None
        if orgStructureTransfer:
            actionTypeId = self.getIdListActionType(u'moving')
            if actionTypeId:
                chiefId = getChiefId(orgStructureTransfer) if not jobTicketEndDateAskingIsRequired else nextChiefId
                if chiefId:
                    setPersonTRId = forceRef(record.value('person_id'))
                    self.eventEditor.mapJournalInfoTransfer = [
                        chiefId, forceDateTime(record.value('endDate')), setPersonTRId if setPersonTRId else self.eventEditor.cmbPerson.value()
                    ]
                if self.checkActionByNextEventCreation():
                    self.eventEditor.eventDate = forceDate(record.value('endDate'))
                    self.eventEditor.edtEndDate.setDate(self.eventEditor.eventDate)
                    personId = forceRef(record.value('person_id'))
                    prevEndDate = oldBegDate
                    self.requestNewEvent(
                        orgStructureTransfer, chiefId if chiefId else personId, self.receivedFinanceId,
                        orgStructurePresence, prevEndDate, movingQuoting, actionTypeId,
                        [self.eventActionFinance, self.receivedFinanceId, orgStructureTransfer,
                         orgStructurePresence, prevEndDate, movingQuoting, personId],
                        mapJournalInfoTransfer=self.eventEditor.mapJournalInfoTransfer)
                    return
                if chiefId:
                    self.eventEditor.cmbPerson.setValue(chiefId)
                self.updateAmount()
                model.addRow(actionTypeId, actionType.amount)
                record, action = model.items()[-1]
                if eventId:
                    record.setValue('event_id', toVariant(eventId))
                if self.eventActionFinance == 0:
                    record.setValue('finance_id', toVariant(self.receivedFinanceId))
                action[u'Отделение пребывания'] = orgStructureTransfer
                if orgStructurePresence:
                    action[u'Переведен из отделения'] = orgStructurePresence
                if oldBegDate:
                    record.setValue('begDate', toVariant(oldBegDate))
                else:
                    record.setValue('begDate', toVariant(dateTime))
                if u'Квота' in action._actionType._propertiesByName and movingQuoting:
                    action[u'Квота'] = movingQuoting
                if hasattr(self.eventEditor, 'tabFeed') and needCutFeed:
                    cutFeed(eventId, forceDateTime(record.value('begDate')))
                    self.eventEditor.tabFeed.load(eventId)
        else:
            actionTypeId = self.getIdListActionType(u'leaved')
            if actionTypeId:
                self.updateAmount()
                model.addRow(actionTypeId, actionType.amount)
                record, action = model.items()[-1]
                if self.eventActionFinance == 0:
                    record.setValue('finance_id', toVariant(self.receivedFinanceId))
                if orgStructurePresence:
                    if u'Отделение' in action._actionType._propertiesByName:
                        action[u'Отделение'] = orgStructurePresence
                if oldBegDate:
                    record.setValue('begDate', toVariant(oldBegDate))
                    record.setValue('directionDate', toVariant(oldBegDate))
                else:
                    record.setValue('begDate', toVariant(dateTime))
                    record.setValue('directionDate', toVariant(dateTime))
                actionTypeOfNewItem = action.getType()
                if actionTypeOfNewItem.defaultEndDate == CActionType.dedActionBegDate:
                    record.setValue('endDate', toVariant(dateTime))
                if u'Квота' in action._actionType._propertiesByName and movingQuoting:
                    action[u'Квота'] = movingQuoting
                if hospitalBedId:
                    tableOSHB = db.table('OrgStructure_HospitalBed')
                    recordProfile = db.getRecordEx(
                        tableOSHB, [tableOSHB['profile_id']], [tableOSHB['id'].eq(hospitalBedId)]
                    )
                    if recordProfile:
                        profileId = forceRef(recordProfile.value('profile_id'))
                        if u'Профиль' in action._actionType._propertiesByName and profileId:
                            action[u'Профиль'] = profileId
                if hasattr(self.eventEditor, 'tabFeed'):
                    cutFeed(eventId, forceDateTime(record.value('begDate')))
                    self.eventEditor.tabFeed.load(eventId)


    def _onNextActionDurationOrAliquoticity(self, action, record, actionType, duration, aliquoticity, actionTypeId, directionDate, setPersonId):
        def _checkNewRecordAction(_duration, newRecordAction):
            if _duration == 1 and not newRecordAction:
                begDate = forceDateTime(record.value('begDate'))
                record.setValue('status', toVariant(CActionStatus.finished))
                endDate = forceDateTime(record.value('endDate'))
                if not endDate:
                    record.setValue('endDate', toVariant(begDate))
                record.setValue('person_id', toVariant(
                    QtGui.qApp.userId if (QtGui.qApp.userId and QtGui.qApp.userSpecialityId) else self.cmbAPSetPerson.value())
                )
                record.setValue('plannedEndDate', toVariant(begDate.addDays(duration - 1)))

        aliquoticity = aliquoticity or 1

#        db = QtGui.qApp.db
        newRecordAction = False

        noExecTimeNextDialog = True
        recordList = {}
        periodicity = self.edtAPPeriodicity.value()
        if not (aliquoticity and actionTypeId and directionDate and setPersonId):
            _checkNewRecordAction(duration, newRecordAction)
            return noExecTimeNextDialog

        model = self.tblAPActions.model()
        for recordOld, actionOld in model.items():
            actionTypeIdOld = forceRef(recordOld.value('actionType_id'))
            id = forceRef(recordOld.value('id'))
            begDateOld = forceDateTime(recordOld.value('begDate'))
            directionDateOld = forceDateTime(recordOld.value('directionDate'))
            setPersonIdOld = forceRef(recordOld.value('setPerson_id'))
            if actionTypeIdOld == actionTypeId and setPersonIdOld == setPersonId and directionDate == directionDateOld:
                recordList[(begDateOld, id)] = (recordOld, actionOld)
        keysList = recordList.keys()
        keysList.sort()
        lastDataKey = keysList[-1]
#        quoticity = 0
#        if aliquoticity > 1:
#            condDate = lastDataKey[0].date()
#            quoticity = len([k for k in keysList if k[0].date() == condDate])
#
#        # if not (duration > 1 or (aliquoticity > 1 and aliquoticity > quoticity)):
#        #     _checkNewRecordAction(duration, newRecordAction)
#        #     return noExecTimeNextDialog

        lastRecord = recordList[lastDataKey]
        if lastRecord:
            begDate = forceDateTime(record.value('begDate'))
            if QtGui.qApp.userId and QtGui.qApp.userSpecialityId:
                execPersonId = QtGui.qApp.userId
            else:
                execPersonId = self.cmbAPSetPerson.value()

            if action.nomenclatureExpense:
                if not self._openNomenclatureEditor(action, record, requireItems=True):
                    return

            dialog = CExecTimeNextActionDialog(self, begDate, execPersonId)
            try:
                if dialog.exec_():
                    execTime = dialog.getExecTime()
                    execPersonId = dialog.getExecPersonId()
                    specifiedName = forceString(record.value('specifiedName'))
                    record.setValue('person_id', toVariant(execPersonId))

                    begDate, periodicity = action.getDataForNewExecutionPlanAction(begDate, periodicity)

                    record.setValue('begDate', toVariant(begDate))
                    record.setValue('endDate', toVariant(QDateTime(begDate.date(), execTime)))
                    record.setValue('status', toVariant(CActionStatus.finished))
                    record.setValue('periodicity', toVariant(periodicity))
                    plannedEndDate = forceDateTime(record.value('plannedEndDate'))
                    newExecutionPlan = action.getExecutionPlanForNewAction(begDate)

                    if newExecutionPlan:
                        newExecutionPlanKeys = newExecutionPlan.keys()
                        newExecutionPlanKeys.sort()

                        newExecDate = newExecutionPlanKeys[0]
                        newExecTimeList = newExecutionPlan.get(newExecDate, {})

                        if newExecTimeList:
                            begDateNew = QDateTime(QDate(newExecDate), min(newExecTimeList.keys()))

                        else:
                            begDateNew = QDateTime(QDate(newExecDate), QTime(0, 0))

                        duration = max(begDateNew.daysTo(plannedEndDate) + 1, 1)

                    else:
                        if action.nomenclatureClientReservation:
                            action.nomenclatureClientReservation.cancel()
                        self.onActionCurrentChanged()
                        return

                    newRecord = QtGui.qApp.db.table('Action').newRecord()
                    newRecord.setValue('plannedEndDate', toVariant(plannedEndDate))
                    newRecord.setValue('directionDate', toVariant(directionDate))
                    newRecord.setValue('begDate', toVariant(begDateNew))
                    newAction = model.getFilledAction(newRecord, actionTypeId, actionType.amount, saveDirectionDates=True)

                    if newExecutionPlan:
                        newAction.setExecutionPlan(newExecutionPlan)

                    newAction.initNomenclature(self.clientId())
                    newAction.nomenclatureReservationFromAction(action)
                    newAction.updateDosageFromExecutionPlan()

                    for property in action._properties:
                        propertyType = property._type
                        name = propertyType.name
                        if (propertyType.canChangeOnlyOwner > 0
                                or propertyType.inActionsSelectionTable > 0
                                or propertyType.isActionNameSpecifier
                                ) and name:

                            if not newAction[name]:
                                newAction[name] = action[name]

                            newAction.getProperty(name).applyDependents(newAction)
                            if propertyType.isActionNameSpecifier:
                                newAction.updateSpecifiedName()

                    model.addRow(presetAction=newAction)

                    self.updateAmount()

                    newRecord.setValue('event_id', toVariant(record.value('event_id')))
                    newRecord.setValue('begDate', toVariant(begDateNew))
                    newRecord.setValue('status', toVariant(CActionStatus.started))
                    newRecord.setValue('specifiedName', toVariant(specifiedName))
                    newRecord.setValue('duration', toVariant(duration))
                    newRecord.setValue('periodicity', toVariant(periodicity))
                    newRecord.setValue('aliquoticity', toVariant(aliquoticity))
                    newRecord.setValue('plannedEndDate', toVariant(plannedEndDate))
                    newRecord.setValue('actionType_id', toVariant(actionTypeId))
                    newRecord.setValue('directionDate', toVariant(directionDate))
                    newRecord.setValue('setPerson_id', toVariant(setPersonId))
                    newRecord.setValue('person_id', toVariant(QtGui.qApp.userId if (
                    QtGui.qApp.userId and QtGui.qApp.userSpecialityId) else self.cmbAPSetPerson.value()))
                    newRecord.setValue('org_id', toVariant(record.value('org_id')))
                    newRecord.setValue('amount', toVariant(record.value('amount')))

                    newRecordAction = True
            finally:
                dialog.destroy()
                sip.delete(dialog)
                del dialog
        _checkNewRecordAction(duration, newRecordAction)
        return noExecTimeNextDialog


    def _onNextEventAfterDirection(self):
        eventId = self.requestNewEvent(None, None, None, None, None, None, None, [],
                                       srcDate = self.eventEditor.getSrcDate(), srcNumber = self.eventEditor.getSrcNumber(),
                                       relegatePersonId = self.eventEditor.getRelegatePersonId(), isMoving = False)
        if eventId:
            db = QtGui.qApp.db
            tableEvent = db.table('Event')
            recordNext = db.getRecordEx(tableEvent,
                        [tableEvent['prevEvent_id'],tableEvent['setDate'], tableEvent['execPerson_id']],
                        [tableEvent['id'].eq(eventId), tableEvent['deleted'].eq(0)])
            prevEventId = forceRef(recordNext.value('prevEvent_id'))
            if prevEventId:
                tableEventType = db.table('EventType')
                record = db.getRecordEx(tableEvent,
                                        u'*',
                                        [tableEvent['id'].eq(prevEventId), tableEvent['deleted'].eq(0)])
                if record:
                    eventTypeId = forceRef(record.value('eventType_id'))
                    recordEventType = db.getRecordEx(tableEventType,
                                            [tableEventType['purpose_id']],
                                            [tableEventType['id'].eq(eventTypeId), tableEventType['deleted'].eq(0)])
                    execDate = forceDateTime(record.value('execDate'))
                    resultId = forceRef(record.value('result_id'))
                    execPersonId = forceRef(record.value('execPerson_id'))
                    purposeId = forceRef(recordEventType.value('purpose_id')) if recordEventType else None
                    execPersonId = forceRef(recordNext.value('execPerson_id')) if recordNext else None
                    setDate = forceDateTime(recordNext.value('setDate'))  if recordNext else None
                    isUpdateEvent = False
                    if not execPersonId and recordNext:
                        record.setValue('execPerson_id', toVariant(execPersonId))
                        isUpdateEvent = True
                    if not execDate and recordNext:
                        record.setValue('execDate', toVariant(setDate))
                        isUpdateEvent = True
                    if not resultId:
                        tableResult = db.table('rbResult')
                        cond = [tableResult['name'].like(u'динам%')]
                        if purposeId:
                            cond.append(tableResult['eventPurpose_id'].eq(purposeId))
                        recordResult = db.getRecordEx(tableResult, [tableResult['id']], cond)
                        eventResultId = forceRef(recordResult.value('id')) if recordResult else None
                        record.setValue('result_id', toVariant(eventResultId))
                        isUpdateEvent = True
                    if isUpdateEvent:
                        db.transaction()
                        try:
                            db.updateRecord(tableEvent, record)
                            db.commit()
                        except:
                            db.rollback()
                    isUpdateDiagnosis = False
                    isDiagnosis = False
                    tableDiagnosis = db.table('Diagnosis')
                    tableDiagnostic = db.table('Diagnostic')
                    tableDiagnosticResult = db.table('rbDiagnosticResult')
                    cond = [tableDiagnosticResult['name'].like(u'динам%')]
                    if purposeId:
                        cond.append(tableDiagnosticResult['eventPurpose_id'].eq(purposeId))
                    recordDiagnosticResult = db.getRecordEx(tableDiagnosticResult, [tableDiagnosticResult['id']], cond)
                    diagnosticResultId = forceRef(recordDiagnosticResult.value('id')) if recordDiagnosticResult else None
                    query = db.query('SELECT getEventDiagnosis(%d)'%prevEventId)
                    if query.next():
                        diagnosisId = forceRef(query.record().value(0))
                        if diagnosisId:
                            recordDiagnosis = db.getRecordEx(tableDiagnosis, '*', tableDiagnosis['id'].eq(diagnosisId))
                            if recordDiagnosis:
                                MKB = forceString(recordDiagnosis.value('MKB'))
                                if not MKB:
                                    recordDiagnosis.setValue('MKB', toVariant(u' Z00.0'))
                                    isUpdateDiagnosis = True
                                    db.transaction()
                                    try:
                                        db.updateRecord(tableDiagnosis, recordDiagnosis)
                                        db.commit()
                                    except:
                                        db.rollback()
                                else:
                                    isDiagnosis = True
                                diagnosisId = forceRef(recordDiagnosis.value('id'))
                                recordDiagnostic = db.getRecordEx(tableDiagnostic, '*', [tableDiagnostic['event_id'].eq(prevEventId),
                                                                                         tableDiagnostic['diagnosis_id'].eq(diagnosisId),
                                                                                         tableDiagnostic['deleted'].eq(0)])
                                if recordDiagnostic:
                                    resultId = forceRef(recordDiagnostic.value('result_id'))
                                    if not resultId:
                                        recordDiagnostic.setValue('result_id', toVariant(diagnosticResultId))
                                        db.transaction()
                                        try:
                                            db.updateRecord(tableDiagnostic, recordDiagnostic)
                                            db.commit()
                                        except:
                                            db.rollback()
                    if not isDiagnosis and not isUpdateDiagnosis:
                        recordDiagnostic = self.setDiagnosDirection(diagnosticResultId, prevEventId)
                        db.transaction()
                        try:
                            db.insertOrUpdate(tableDiagnostic, recordDiagnostic)
                            db.commit()
                        except:
                            db.rollback()
                    tableAction = db.table('Action')
                    tableActionType = db.table('ActionType')
                    table = tableAction.innerJoin(tableActionType, tableActionType['id'].eq(tableAction['actionType_id']))
                    cond = [tableAction['event_id'].eq(prevEventId),
                            tableAction['deleted'].eq(0),
                            tableActionType['deleted'].eq(0),
                            db.joinOr([tableActionType['flatCode'].like(u'recoveryDirection%'),
                                       tableActionType['flatCode'].like(u'inspectionDirection%'),
                                       tableActionType['flatCode'].like(u'researchDirection%'),
                                       tableActionType['flatCode'].like(u'consultationDirection%')])
                            ]
                    cols = [tableAction['id'],
                            tableAction['person_id'],
                            tableAction['endDate'],
                            tableAction['status']
                            ]
                    record = db.getRecordEx(table, cols, cond, tableAction['id'].name())
                    if record:
                        actionId = forceRef(record.value('id'))
                        if actionId:
                            isUpdateAction = False
                            endDate = forceDateTime(record.value('endDate'))
                            personId = forceRef(record.value('person_id'))
                            status = forceInt(record.value('status'))
                            recordAction = db.getRecordEx(tableAction, u'*', [tableAction['id'].eq(actionId), tableAction['deleted'].eq(0)])
                            if not endDate:
                                recordAction.setValue('endDate', toVariant(setDate))
                                isUpdateAction = True
                            if not personId:
                                recordAction.setValue('person_id', toVariant(execPersonId))
                                isUpdateAction = True
                            if status != CActionStatus.finished:
                                recordAction.setValue('status', toVariant(CActionStatus.finished))
                                isUpdateAction = True
                            if isUpdateAction:
                                db.transaction()
                                try:
                                    db.updateRecord(tableAction, recordAction)
                                    db.commit()
                                except:
                                    db.rollback()


    def setDiagnosDirection(self, diagnosticResultId, eventId):
        db = QtGui.qApp.db
        tableEvent = db.table('Event')
        tableDiagnostic = db.table('Diagnostic')
        record = db.getRecordEx(tableEvent,
                                u'*',
                                [tableEvent['id'].eq(eventId), tableEvent['deleted'].eq(0)])
        execDate = forceDateTime(record.value('execDate')) if record else None
        execPersonId = forceRef(record.value('execPerson_id')) if record else None
        clientId = forceRef(record.value('client_id')) if record else None
        MKB           = u' Z00.0'
        MKBEx         = U''
        TNMS          = u''
        morphologyMKB = u''
        diagnosisTypeId = forceRef(QtGui.qApp.db.translate('rbDiagnosisType', 'code', '1', 'id'))
        specialityId = forceRef(QtGui.qApp.db.translate('Person', 'id', execPersonId, 'speciality_id'))
        recordDiagnostic = tableDiagnostic.newRecord()
        recordDiagnostic.setValue('diagnosisType_id', toVariant(diagnosisTypeId))
        recordDiagnostic.setValue('speciality_id', toVariant(specialityId))
        recordDiagnostic.setValue('person_id', toVariant(execPersonId))
        recordDiagnostic.setValue('setDate', toVariant(execDate))
        recordDiagnostic.setValue('endDate', toVariant(execDate))
        diagnosisId = None
        characterId = None
        diagnosisId, characterId = getDiagnosisId2(
                execDate,
                execPersonId,
                clientId,
                diagnosisTypeId,
                MKB,
                MKBEx,
                None,
                None,
                None,
                diagnosisId,
                forceRef(recordDiagnostic.value('id')),
                QtGui.qApp.defaultIsManualSwitchDiagnosis(),
                forceBool(recordDiagnostic.value('handleDiagnosis')),
                TNMS=TNMS,
                morphologyMKB=morphologyMKB
                )
        recordDiagnostic.setValue('diagnosis_id', toVariant(diagnosisId))
        recordDiagnostic.setValue('TNMS', toVariant(TNMS))
        recordDiagnostic.setValue('character_id', toVariant(characterId))
        recordDiagnostic.setValue('event_id', toVariant(eventId))
        recordDiagnostic.setValue('result_id', toVariant(diagnosticResultId))
        return recordDiagnostic


    @pyqtSignature('')
    def on_btnNextAction_clicked(self):
        db = QtGui.qApp.db
        nextChiefId = None
        isNextEventAfterDirection = False
        relegateOrgId = self.eventEditor.tabNotes.cmbRelegateOrg.value()
        if relegateOrgId and (relegateOrgId != QtGui.qApp.currentOrgId()
            and self.cmbAPStatus.currentIndex() in (CActionStatus.started, CActionStatus.wait, CActionStatus.withoutResult, CActionStatus.appointed)):
            model = self.tblAPActions.model()
            items = model.items()
            row = self.tblAPActions.currentIndex().row()
            if 0<= row <len(items):
                record, action = items[row]
                actionTypeId = forceRef(record.value('actionType_id')) if record else None
                actionType = CActionTypeCache.getById(actionTypeId) if actionTypeId else None
                flatCode = actionType.flatCode.lower()
                if ('recoveryDirection'.lower() in flatCode
                    or u'inspectionDirection'.lower() in flatCode
                    or u'researchDirection'.lower() in flatCode
                    or u'consultationDirection'.lower() in flatCode
                    ):
                    self._onNextEventAfterDirection()
                    isNextEventAfterDirection = True
        if self.cmbAPStatus.value() == CActionStatus.canceled:
            currentDateTime = QDateTime.currentDateTime()
            table = db.table('vrbPersonWithSpeciality')
            personId = QtGui.qApp.userId if (QtGui.qApp.userId and QtGui.qApp.userSpecialityId) else self.cmbAPSetPerson.value()
            record = db.getRecordEx(table, [table['name']], [table['id'].eq(personId)]) if personId else None
            personName = forceString(record.value('name')) if record else ''
            self.edtAPNote.setText(u'Отменить: %s %s'%(currentDateTime.toString('dd-MM-yyyy hh:mm'), personName))
        if isNextEventAfterDirection:
            pass
        else:
            model = self.tblAPActions.model()
            items = model.items()
            row = self.tblAPActions.currentIndex().row()

            if not (0 <= row < len(items)):
                return

            record, action = items[row]

            needCutFeed = True
            actionTypeId = forceRef(record.value('actionType_id')) if record else None
            actionType = CActionTypeCache.getById(actionTypeId) if actionTypeId else None

            if any([actionType and i in actionType.flatCode.lower() for i in ['moving', 'received', 'inspectpigeonhole']]):
                jobTicketEndDateAskingIsRequired = forceBool(
                    QtGui.qApp.preferences.appPrefs.get('jobTicketEndDateAskingIsRequired', QVariant(True))
                )
            else:
                jobTicketEndDateAskingIsRequired = False

            if jobTicketEndDateAskingIsRequired:
                dlg = CInputCutFeedDialog(action, actionType, record, self.eventEditor.cmbPerson.value(), self)
                if dlg.exec_():
                    currentDateTime = dlg.dateTime()
                    record.setValue('person_id', dlg.person())
                    nextChiefId = dlg.execPerson()
                    needCutFeed = dlg.cutFeed()
                else:
                    return
            else:
                currentDateTime = QDateTime.currentDateTime()
            noExecTimeNextDialog = True
            directionDate = forceDateTime(record.value('directionDate')) if record else None
            setPersonId = forceRef(record.value('setPerson_id')) if record else None
            duration = self.edtAPDuration.value()
            aliquoticity = self.edtAPAliquoticity.value()
            if u'inspectPigeonHole'.lower() in actionType.flatCode.lower():
                self._onNextActionIsInspectPigeonHole(record, currentDateTime)

            elif u'received' in actionType.flatCode.lower():
                self._onNextActionIsReceived(
                    action, actionType, record, jobTicketEndDateAskingIsRequired, nextChiefId, currentDateTime
                )

            elif u'moving' in actionType.flatCode.lower():
                self._onNextActionIsMoving(
                    action, actionType, record, jobTicketEndDateAskingIsRequired, nextChiefId, currentDateTime, needCutFeed
                )

            elif duration > 0 or aliquoticity > 0:
                noExecTimeNextDialog = self._onNextActionDurationOrAliquoticity(
                    action, record, actionType, duration, aliquoticity, actionTypeId, directionDate, setPersonId
                )

            if noExecTimeNextDialog:
                row = (row + 1) if (row + 1) <= model.rowCount()-1 else model.rowCount()-1
                self.tblAPActions.setCurrentIndex(model.index(row, 0))
                self.edtAPBegDate.setFocus(Qt.OtherFocusReason)

            row = self.tblAPActions.currentIndex().row()
            actionsSummaryRow = self.eventEditor.translate2ActionsSummaryRow(model, row)
            if actionsSummaryRow is not None:
                self.eventEditor.onActionChanged(actionsSummaryRow)


    def getCurrentTimeAction(self, actionDate):
        currentDateTime = QDateTime.currentDateTime()
        if currentDateTime == actionDate:
            currentTime = currentDateTime.time()
            return currentTime.addSecs(60)
        else:
            return currentDateTime.time()


    def clientId(self):
        return self.eventEditor.clientId


    def requestNewEvent(self, orgStructureTransfer, personId, financeId, orgStructurePresence, actionDate, movingQuoting,
                        actionTypeId, actionByNewEvent, actionListToNewEvent = [], endDateTime = None, result = None, mapJournalInfoTransfer = [],
                        srcDate = QDate(), srcNumber = u'', relegatePersonId = None, isMoving = True):
        from Events.CreateEvent import requestNewEvent
        clientId = self.clientId()
        self.eventEditor.done(1)
        eventId = self.eventEditor._id
        if not self.eventEditor.isDirty() and clientId:
            params = {}
            params['widget'] = self.eventEditor
            params['clientId'] = clientId
            params['flagHospitalization'] = True
            params['actionTypeId'] = actionTypeId
            params['valueProperties'] = [orgStructureTransfer]
            if isMoving:
                params['eventTypeFilterHospitalization'] = 2
            params['dateTime'] = actionDate
            params['personId'] = personId
            params['planningEventId'] = eventId
            params['diagnos'] = None
            params['financeId'] = financeId
            params['protocolQuoteId'] = movingQuoting
            params['actionByNewEvent'] = actionByNewEvent
            params['externalId'] = self.eventEditor.getExternalId()
            params['relegateOrgId'] = self.eventEditor.tabNotes.cmbRelegateOrg.value()
            params['srcNumber'] = srcNumber
            params['srcDate'] = srcDate
            params['relegatePersonId'] = relegatePersonId
            params['order'] = self.eventEditor.cmbOrder.currentIndex()
            params['moving'] = True
            params['actionListToNewEvent'] = actionListToNewEvent
            params['endDateTime'] = endDateTime
            params['prevEventId'] = eventId
            params['result'] = result
            params['mapJournalInfoTransfer'] = mapJournalInfoTransfer
            for action in actionListToNewEvent:
                action.checkModifyDate = False
            return requestNewEvent(params)
        return None


    def updatePropTable(self, action):
        self.tblAPProps.model().setAction(action, self.eventEditor.clientId, self.eventEditor.clientSex, self.eventEditor.clientAge, self.eventEditor.eventTypeId)
        self.tblAPProps.resizeRowsToContents()


    def onActionDataChanged(self, name, value):
        model = self.tblAPActions.model()
        items = model.items()
        row = self.tblAPActions.currentIndex().row()
        if 0<=row<len(items):
            #record = items[row][0]
            record, action = items[row]
            record.setValue(name, toVariant(value))
            actionTypeId = forceRef(record.value('actionType_id')) if record else None
            actionType = CActionTypeCache.getById(actionTypeId) if actionTypeId else None
            if (u'moving' in actionType.flatCode.lower()
                or u'received' in actionType.flatCode.lower()
                or u'inspectPigeonHole'.lower() in actionType.flatCode.lower()
                or u'recoveryDirection'.lower() in actionType.flatCode.lower()
                or u'inspectionDirection'.lower() in actionType.flatCode.lower()
                or u'researchDirection'.lower() in actionType.flatCode.lower()
                or u'consultationDirection'.lower() in actionType.flatCode.lower()
                ):
                self.onCurrentActionAdd(actionType.flatCode, action)
            else:
                if (self.edtAPDuration.value() > 0 or self.edtAPAliquoticity.value() > 0) and self.edtAPEndDate.date().isNull() and self.cmbAPStatus.value() != CActionStatus.canceled:
                    self.btnNextAction.setText(u'Выполнить')
                    self.btnNextAction.setEnabled(True)
                else:
                    self.btnNextAction.setText(u'Действие')
                    self.btnNextAction.setEnabled(False)
                if self.edtAPDuration.value() > 0:
                    self.btnPlanNextAction.setEnabled(True)
                else:
                    self.btnPlanNextAction.setEnabled(False)
            if actionType.defaultPlannedEndDate == CActionType.dpedBegDatePlusAmount and name in ('amount', 'begDate'):
                begDate = self.edtAPBegDate.date()
                amountValue = int(self.edtAPAmount.value())
                date = begDate.addDays(amountValue-1) if begDate and amountValue else QDate()
                self.edtAPPlannedEndDate.setDate(date)
            elif actionType.defaultPlannedEndDate == CActionType.dpedBegDatePlusDuration and name in ('duration', 'begDate'):
                begDate = self.edtAPBegDate.date()
                durationValue = self.edtAPDuration.value()
                date = begDate.addDays(durationValue-1) if begDate and durationValue else QDate()
                self.edtAPPlannedEndDate.setDate(date)


    def updateAmount(self):
        model = self.tblAPActions.model()
        row = self.tblAPActions.currentIndex().row()
        model.updateActionAmount(row)


    def loadPrevAction(self, type):
        model = self.tblAPActions.model()
        items = model.items()
        row = self.tblAPActions.currentIndex().row()
        if QtGui.qApp.userHasRight(urCopyPrevAction) and 0<=row<len(items):
                action = items[row][1]
                prevActionId = self.eventEditor.getPrevActionId(action, type)
                if prevActionId:
                    action.updateByActionId(prevActionId)
                    self.tblAPProps.model().reset()
                    self.tblAPProps.resizeRowsToContents()
                    model.updateActionAmount(row)


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelAPActions_currentChanged(self, current, previous):
        self.onActionCurrentChanged(previous)
        #self.tblAPProps.updatePropertiesTable(current, previous)


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_modelAPActions_dataChanged(self, topLeft, bottomRight):
        topLeftRow = topLeft.row()
        bottomRightRow = bottomRight.row()
        if topLeftRow<=self.tblAPActions.currentIndex().row()<=bottomRightRow:
            self.onActionCurrentChanged()


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_modelAPActions_personChanged(self, topLeft, bottomRight):
#        print u"Человек сменился\n"
        pass


    @pyqtSignature('int')
    def on_modelAPActions_amountChanged(self, row):
        if row == self.tblAPActions.currentIndex().row():
            model = self.modelAPActions
            record = model._items[row][0]
            actionTypeId = forceRef(record.value('actionType_id'))
            amount = forceDouble(record.value('amount'))
            personId = forceRef(record.value('person_id'))
            financeId = forceRef(record.value('finance_id'))
            contractId = forceRef(record.value('contract_id'))
            self.edtAPAmount.setValue(amount)
            uet = amount*self.eventEditor.getUet(actionTypeId, personId, financeId, contractId)
            self.edtAPUet.setValue(uet)
            self.onActionDataChanged('uet', uet)
            self.emit(SIGNAL('actionUetChanged()'))


    def findPrevAction(self, actionBegDate):
        model = self.tblAPActions.model()
        found = [None, None, None]
        for item in model._items:
            record, action = item
            actionTypeId = forceRef(record.value('actionType_id'))
            actionType = CActionTypeCache.getById(actionTypeId) if actionTypeId else None
            if u'moving' in actionType.flatCode.lower():
                if action[u'Переведен в отделение']:
                    endDate = forceDateTime(record.value('endDate'))
                    if endDate and endDate <= actionBegDate:
                        if found:
                           if endDate > found[2]:
                                found = [record, action, endDate]
                        else:
                            found = [record, action, endDate]
        return found[0]


    def findAllAfterCurrent(self, actionBegDate):
        model = self.tblAPActions.model()
        found = []
        for item in model._items:
            record, action = item
            begDate = forceDateTime(record.value('begDate'))
            if begDate and begDate >= actionBegDate:
                found.append(action)
        return found


    def actAPActionSplitEventChecker(self):
        if not self.eventEditor.mesRequired:
            return False
        model = self.tblAPActions.model()
        row = self.tblAPActions.currentIndex().row()
        if 0<=row<model.rowCount()-1:
            record = model._items[row][0]
            action = model._items[row][1]
            actionTypeId = forceRef(record.value('actionType_id'))
            actionType = CActionTypeCache.getById(actionTypeId) if actionTypeId else None
            if u'moving' in actionType.flatCode.lower():
                if action[u'Переведен из отделения']:
                    return True
        return False


    @pyqtSignature('')
    def on_actAPActionSplitEvent_triggered(self):
        if QtGui.QMessageBox.question(self,
            u'Внимание!',
            u'Выполнить перенос действий в новое событие?',
            QtGui.QMessageBox.No|QtGui.QMessageBox.Yes,
            QtGui.QMessageBox.No) != QtGui.QMessageBox.Yes:
                return
        resultId = forceRef(QtGui.qApp.db.translate('rbResult', 'federalCode', '104', 'id'))
        model = self.tblAPActions.model()
        row = self.tblAPActions.currentIndex().row()
        if 0<=row<model.rowCount()-1:
            from F003.F003Dialog import CF003Dialog
            if type(self.eventEditor) == CF003Dialog and not  self.eventEditor.getExecDateTime().isValid():
                result = True
                showTime = getEventShowTime(self.eventEditor.eventTypeId)
                if showTime:
                    endDate = QDateTime(self.eventEditor.edtEndDate.date(), self.eventEditor.edtEndTime.time())
                else:
                    endDate = self.eventEditor.edtEndDate.date()
                result = result and (len(self.eventEditor.modelFinalDiagnostics.items())>0 or self.eventEditor.checkInputMessage(u'диагноз', False, self.eventEditor.tblFinalDiagnostics))
                #WTF? какое дело ActionPage до МЭСа?
                if self.eventEditor.mesRequired:
                    result = result and self.eventEditor.tabMes.checkMesAndSpecification()
                    result = result and (self.eventEditor.tabMes.chechMesDuration() or self.eventEditor.checkValueMessage(u'Длительность события должна попадать в интервал минимальной и максимальной длительности по требованию к МЭС', True, self.eventEditor.edtBegDate))
                    result = result and self.eventEditor.checkDiagnosticsMKBForMes(self.eventEditor.tblFinalDiagnostics, self.eventEditor.tabMes.cmbMes.value())
                result = result and self.eventEditor.checkDiagnosticsDataEntered(endDate)
                if not resultId:
                    result = result and (self.cmbResult.value() or self.checkInputMessage(u'результат', False, self.cmbResult))
                if not result:
                    return
            record = model._items[row][0]
            action = model._items[row][1]
            actionTypeId = forceRef(record.value('actionType_id'))
            actionType = CActionTypeCache.getById(actionTypeId) if actionTypeId else None
            if u'moving' in actionType.flatCode.lower():
                if action[u'Переведен из отделения']:
                    begDate = forceDateTime(record.value('begDate'))
                    prevActionRecord = self.findPrevAction(begDate)
                    if prevActionRecord:
                        personId = forceRef(record.value('person_id'))
                        prevActionEndDate = forceDateTime(prevActionRecord.value('endDate'))
                        movingQuoting = action[u'Квота'] if u'Квота' in action._actionType._propertiesByName else None
                        actions = self.findAllAfterCurrent(begDate)
                        if hasattr(self.eventEditor, 'tabNotes'):
                            self.eventEditor.tabNotes.chkIsClosed.setChecked(0)
                        eventId = self.requestNewEvent(None, personId, None, \
                                             None, prevActionEndDate, movingQuoting, actionTypeId, [], actions, self.eventEditor.getExecDateTime())
                        if eventId:
                            record = self.eventEditor.getRecord()
                            record.setValue('execDate', toVariant(prevActionEndDate))
                            if resultId:
                                record.setValue('result_id', toVariant(resultId))
                            QtGui.qApp.db.transaction()
                            try:
                                QtGui.qApp.db.insertOrUpdate('Event', record)
                                QtGui.qApp.db.commit()
                            except:
                                QtGui.qApp.db.rollback()


    @pyqtSignature('')
    def on_actAPAddLikeAction_triggered(self):
        model = self.tblAPActions.model()
        row = self.tblAPActions.currentIndex().row()
        if 0<=row<model.rowCount()-1:
            record = model._items[row][0]
            actionTypeId = forceRef(record.value('actionType_id'))
            index = model.index(model.rowCount()-1, 0)
            if model.setData(index, toVariant(actionTypeId)):
                self.tblAPActions.setCurrentIndex(index)


    @pyqtSignature('')
    def on_actAPDuplicateAction_triggered(self):
        model = self.tblAPActions.model()
        row = self.tblAPActions.currentIndex().row()
        if 0<=row<model.rowCount()-1:
            record, action = model._items[row]
            actionTypeId = forceRef(record.value('actionType_id'))
            index = model.index(model.rowCount()-1, 0)
            if model.setData(index, toVariant(actionTypeId)):
                newRecord, newAction = model._items[index.row()]
                newAction.updateByAction(action)
                self.tblAPActions.setCurrentIndex(index)


    @pyqtSignature('')
    def on_actAPReplicateAction_triggered(self):
        model = self.tblAPActions.model()
        row = self.tblAPActions.currentIndex().row()
        if 0<=row<model.rowCount()-1:
            action = model._items[row][1]
#WFT?
# диалог НЕ должен создавать/изменять Actions
# он должен ввести что-то, а код для копирования должен быть здесь
            dlg = CActionReplicateDialog(self, action, self.eventEditor)
            dlg.exec_()
            for action in dlg.getResult():
                actionTypeId = action._actionType.id
                index = model.index(model.rowCount()-1, 0)
                model.setData(index, toVariant(actionTypeId), presetAction=action)


    @pyqtSignature('')
    def on_actAPAddAction_triggered(self):
        model = self.tblAPActions.model()
        model.addRow()
        row = model.rowCount()-1
        self.tblAPActions.setCurrentIndex(model.index(row, 0))


    @pyqtSignature('')
    def on_actAPActionsAdd_triggered(self):
        orgStructureId = QtGui.qApp.currentOrgStructureId()
        eventFinanceId = self.eventEditor.eventFinanceId
        financeCode = CFinanceType.getCode(eventFinanceId)
        existsActionTypesList = []
        for item in self.modelAPActions.items():
            existsActionTypesList.append(forceRef(item[0].value('actionType_id')))
        actionTypes = selectActionTypes(self,
                                self.eventEditor,
                                [self.modelAPActions.actionTypeClass],
                                 orgStructureId,
                                 self.eventEditor.getEventTypeId(),
                                 self.eventEditor.getContractId(),
                                 self.eventEditor.getMesId(),
                                 financeCode in (CFinanceType.VMI, CFinanceType.cash),
                                 self.eventEditor._id,
                                 existsActionTypesList,
                                 visibleTblSelected=True,
                                 contractTariffCache=self.eventEditor.contractTariffCache
                               )
        model = self.tblAPActions.model()
        isEventCSGRequired = getEventCSGRequired(self.eventEditor.eventTypeId)
        for actionTypeId, action, csgRecord in actionTypes:
            index = model.index(model.rowCount()-1, 0)
            model.setData(index, toVariant(actionTypeId), presetAction=action)
            if isEventCSGRequired:
                self.cmbCSG.addActionToCSG(action.getRecord(), csgRecord)
                self.cmbCSG.setItems()
                self.updateCmbCSG()
#        self.onActionCurrentChanged()


    @pyqtSignature('QDate')
    def on_edtAPDirectionDate_dateChanged(self, date):
        self.edtAPDirectionTime.setEnabled(bool(date))
        time = self.edtAPDirectionTime.time() if date and self.edtAPDirectionTime.isVisible() else QTime()
        self.onActionDataChanged('directionDate', QDateTime(date, time))
        row = self.tblAPActions.currentIndex().row()
        model = self.tblAPActions.model()
        action = model.items()[row][1]
        if action:
            defaultPlannedEndDate = action.getType().defaultPlannedEndDate
            if defaultPlannedEndDate == CActionType.dpedNextDay:
                plannedEndDate = date.addDays(1)
                self.edtAPPlannedEndDate.setDate(plannedEndDate)
            elif defaultPlannedEndDate == CActionType.dpedNextWorkDay:
                plannedEndDate = addWorkDays(date, 1, wpFiveDays)
                self.edtAPPlannedEndDate.setDate(plannedEndDate)


    @pyqtSignature('QString')
    def on_cmbAPMKB_textChanged(self, value):
        if value[-1:] == '.':
            value = value[:-1]
        diagName = forceString(QtGui.qApp.db.translate('MKB', 'DiagID', value, 'DiagName'))
        if diagName:
            self.lblAPMKBText.setText(diagName)
        else:
            self.lblAPMKBText.clear()
        self.cmbAPMorphologyMKB.setMKBFilter(self.cmbAPMorphologyMKB.getMKBFilter(unicode(value)))
        self.onActionDataChanged('MKB', value)


    @pyqtSignature('QString')
    def on_cmbAPMorphologyMKB_textChanged(self, value):
        self.onActionDataChanged('morphologyMKB', value)


    @pyqtSignature('QTime')
    def on_edtAPDirectionTime_timeChanged(self, time):
        date = self.edtAPDirectionDate.date()
        self.onActionDataChanged('directionDate', QDateTime(date, time if date else QTime()))


    @pyqtSignature('bool')
    def on_chkAPIsUrgent_toggled(self, checked):
        self.onActionDataChanged('isUrgent', checked)


    @pyqtSignature('QDate')
    def on_edtAPPlannedEndDate_dateChanged(self, date):
        self.edtAPPlannedEndTime.setEnabled(bool(date))
        time = self.edtAPPlannedEndTime.time() if date and self.edtAPPlannedEndTime.isVisible() else QTime()
        self.onActionDataChanged('plannedEndDate', QDateTime(date, time))


    @pyqtSignature('QTime')
    def on_edtAPPlannedEndTime_timeChanged(self, time):
        date = self.edtAPPlannedEndDate.date()
        self.onActionDataChanged('plannedEndDate', QDateTime(date, time if date else QTime()))


    @pyqtSignature('int')
    def on_cmbAPSetPerson_currentIndexChanged(self, index):
        self.onActionDataChanged('setPerson_id', self.cmbAPSetPerson.value())


    @pyqtSignature('int')
    def on_cmbAPOrg_currentIndexChanged(self, index):
        self.onActionDataChanged('org_id', self.cmbAPOrg.value())


    @pyqtSignature('')
    def on_btnAPSelectOrg_clicked(self):
        orgId = selectOrganisation(self, self.cmbAPOrg.value(), False)
        self.cmbAPOrg.updateModel()
        if orgId:
            self.cmbAPOrg.setValue(orgId)


    @pyqtSignature('QDate')
    def on_edtAPBegDate_dateChanged(self, date):
        self.edtAPBegTime.setEnabled(bool(date))
        time = self.edtAPBegTime.time() if date and self.edtAPBegTime.isVisible() else QTime()
        self.onActionDataChanged('begDate', QDateTime(date, time))
        self.updateAmount()
        self.cmbAPMKB.setFilter('''IF(MKB.endDate IS NOT NULL, MKB.endDate >= %s, 1)'''%(QtGui.qApp.db.formatDate(self.edtAPBegDate.date())))
        defaultEndDate = self.defaultEndDate
        if defaultEndDate == CActionType.dedSyncActionBegDate:
            if date and self.edtAPEndDate.date() != self.edtAPBegDate.date():
                self.edtAPEndDate.setDate(date)


    @pyqtSignature('QTime')
    def on_edtAPBegTime_timeChanged(self, time):
        date = self.edtAPBegDate.date()
        self.onActionDataChanged('begDate', QDateTime(date, time if date else QTime()))
        defaultEndDate = self.defaultEndDate
        if defaultEndDate == CActionType.dedSyncActionBegDate:
            if date and self.edtAPEndTime.time() != self.edtAPBegTime.time():
                self.edtAPEndTime.setTime(time)


    def checkDateTimeTOCurrentDateTime(self, isDate):
        if self.edtAPEndDate.date():
            model = self.tblAPActions.model()
            items = model.items()
            row = self.tblAPActions.currentIndex().row()
            if 0<=row<len(items):
                record, action = items[row]
                if record:
                    actionTypeId = forceRef(record.value('actionType_id'))
                    if actionTypeId and actionTypeId in self.actionTypeHBIdList:
                        time = self.edtAPEndTime.time()
                        if self.edtAPEndTime.isVisible() and bool(self.edtAPEndTime.time()) and self.edtAPEndTime.time().isValid():
                            date = QDateTime(self.edtAPEndDate.date(), time)
                            curDate = QDateTime.currentDateTime()
                        else:
                            date = self.edtAPEndDate.date()
                            curDate = QDate.currentDate()
                        if date > curDate:
                            return CDialogBase(self).checkValueMessage(u'Дата окончания действия %s не может быть больше текущей даты %s' % (forceString(date), forceString(curDate)), True, self.tblAPActions, row, 0, self.edtAPEndDate if isDate else self.edtAPEndTime)
        return True


    @pyqtSignature('QDate')
    def on_edtAPEndDate_dateChanged(self, date):
        if not self.checkDateTimeTOCurrentDateTime(True):
            return
        eventEditor = self.eventEditor
        defaultDirectionDate = self.defaultDirectionDate
        eventSetDate = eventEditor.eventSetDateTime.date() if eventEditor and eventEditor.eventSetDateTime else None
        #begDate = eventEditor.eventSetDateTime
        if defaultDirectionDate == CActionType.dddActionExecDate:
            if date:
                if date < eventSetDate:
                    directionDate = eventSetDate
                    #begDate = QDate()
                else:
                    directionDate = date
                    #begDate = date
            else:
                directionDate = eventSetDate
            #self.edtAPBegDate.setDate(begDate.date() if type(begDate) == QDateTime else begDate)
            self.edtAPDirectionDate.setDate(directionDate)
        else:
            if date and eventSetDate and date < eventSetDate:
                self.edtAPBegDate.setDate(QDate())
                self.edtAPBegTime.setTime(QTime())
                self.edtAPDirectionDate.setDate(QDate())
                self.edtAPDirectionTime.setTime(QTime())
            elif date and eventSetDate and date >= eventSetDate:
                if not self.edtAPDirectionDate.date():
                    self.edtAPDirectionDate.setDate(eventSetDate)
                #if not self.edtAPBegDate.date():
                    #self.edtAPBegDate.setDate(eventSetDate)
        self.edtAPEndTime.setEnabled(bool(date))
        time = self.edtAPEndTime.time() if date and self.edtAPEndTime.isVisible() else QTime()
        self.onActionDataChanged('endDate', QDateTime(date, time))
        self.updateAmount()
        if date:
            self.cmbAPStatus.setValue(CActionStatus.finished)
        else:
            if self.cmbAPStatus.value() == CActionStatus.finished:
                self.cmbAPStatus.setValue(CActionStatus.canceled)
        self._onActionChanged()
        if not self.cmbAPPerson.value() and not self.edtAPEndDate.date().isNull():
            self.setUserExecPerson()


    def setUserExecPerson(self):
        model = self.tblAPActions.model()
        items = model.items()
        row = self.tblAPActions.currentIndex().row()
        record, action = items[row]
        actionTypeId = forceRef(record.value('actionType_id')) if record else None
        actionType = CActionTypeCache.getById(actionTypeId) if actionTypeId else None
        if actionType:
            if actionType.defaultPersonInEvent == CActionType.dpUserExecPerson:
                if QtGui.qApp.userId and QtGui.qApp.userSpecialityId:
                    self.cmbAPPerson.setValue(QtGui.qApp.userId)


    def _onActionChanged(self):
        model = self.tblAPActions.model()
        row = self.tblAPActions.currentIndex().row()
        actionsSummaryRowRow = self.eventEditor.translate2ActionsSummaryRow(model, row)
        if actionsSummaryRowRow is not None:
            self.eventEditor.onActionChanged(actionsSummaryRowRow)


    @pyqtSignature('QTime')
    def on_edtAPEndTime_timeChanged(self, time):
        if not self.checkDateTimeTOCurrentDateTime(False):
            return
        date = self.edtAPEndDate.date()
        self.onActionDataChanged('endDate', QDateTime(date, time if date else QTime()))

#       TODO: с временем события по закрытию действия отложим пока.
#        if self.edtAPEndTime.isVisible() and bool(time) and time.isValid():
#            model = self.tblAPActions.model()
#            items = model.items()
#            row = self.tblAPActions.currentIndex().row()
#            if 0 <= row < len(items):
#                record, action = items[row]
#                actionType = action.getType()
#                if actionType and actionType.closeEvent:
#                    self.eventEditor.setEndTime(time)


    @pyqtSignature('int')
    def on_cmbAPStatus_currentIndexChanged(self, index):
        actionStatus = self.cmbAPStatus.value()
        self.onActionDataChanged('status', actionStatus)
        if actionStatus in (CActionStatus.finished, CActionStatus.withoutResult):
            if self.edtAPEndDate.date().isNull():
                now = QDateTime.currentDateTime()
                self.edtAPEndDate.setDate(now.date())
                if self.edtAPEndTime.isVisible():
                    self.edtAPEndTime.setTime(now.time())
                self._onActionChanged()
            if actionStatus == CActionStatus.finished \
               and not self.cmbAPPerson.value() \
               and not self.edtAPEndDate.date().isNull():
                self.setUserExecPerson()
        elif actionStatus in (CActionStatus.canceled, CActionStatus.refused):
            self.edtAPEndDate.setDate(QDate())
            self.edtAPEndTime.setTime(QTime())
            if QtGui.qApp.userId and QtGui.qApp.userSpecialityId:
                self.cmbAPPerson.setValue(QtGui.qApp.userId)
            else:
                self.cmbAPPerson.setValue(self.cmbAPSetPerson.value())
        else:
            self.edtAPEndDate.setDate(QDate())
            self.edtAPEndTime.setTime(QTime())


    @pyqtSignature('int')
    def on_edtAPDuration_valueChanged(self, value):
        self.onActionDataChanged('duration', value)
        record, action = self.tblAPActions.model().items()[self.tblAPActions.currentIndex().row()]
        canEdit = not action.isLocked() if action else True
        self.edtAPDirectionDate.setEnabled(bool(not value > 0) and canEdit)
        self.edtAPDirectionTime.setEnabled(bool(not value > 0) and canEdit)
        self.cmbAPSetPerson.setEnabled(bool(not value > 0) and canEdit)
        self.edtAPPlannedEndDate.setEnabled(bool(not value > 0) and canEdit)
        self.edtAPPlannedEndTime.setEnabled(bool(not value > 0) and canEdit)
        self.edtAPBegDate.setEnabled(bool(not value > 0) and canEdit)
        self.edtAPBegTime.setEnabled(bool(not value > 0) and canEdit)
        self.edtAPEndDate.setEnabled(bool(not value > 0) and canEdit)
        self.edtAPEndTime.setEnabled(bool(not value > 0) and canEdit)


    @pyqtSignature('int')
    def on_edtAPPeriodicity_valueChanged(self, value):
        self.onActionDataChanged('periodicity', value)


    @pyqtSignature('int')
    def on_edtAPAliquoticity_valueChanged(self, value):
        self.onActionDataChanged('aliquoticity', value)


    @pyqtSignature('double')
    def on_edtAPAmount_valueChanged(self, value):
        self.onActionDataChanged('amount', value)
        row = self.tblAPActions.currentIndex().row()
        model = self.modelAPActions
        model.emitAmountChanged(row)
#        record = model._items[row][0]
#        actionTypeId = forceRef(record.value('actionType_id'))
#        amount = forceDouble(record.value('amount'))
#        personId = forceRef(record.value('person_id'))
#        financeId = forceRef(record.value('finance_id'))
#        contractId = forceRef(record.value('contract_id'))
#        self.edtAPUet.setValue(value*self.eventEditor.getUet(actionTypeId, personId, financeId, contractId))



#    @pyqtSignature('double')
#    def on_edtAPUet_valueChanged(self, value):
#        self.onActionDataChanged('uet', value)


    @pyqtSignature('QString')
    def on_edtAPOffice_textChanged(self, text):
        self.onActionDataChanged('office', text)


    @pyqtSignature('int')
    def on_cmbAPPerson_currentIndexChanged(self, index):
        self.onActionDataChanged('person_id', self.cmbAPPerson.value())
        self._onActionChanged()
        items = self.tblAPActions.model().items()
        row = self.tblAPActions.currentIndex().row()
        if 0<=row<len(items):
            record, action = items[row]
            actionTemplateTreeModel = self.actionTemplateCache.getModel(action.getType().id)
            self.btnAPLoadTemplate.setModel(actionTemplateTreeModel)
            if not(QtGui.qApp.userHasRight(urLoadActionTemplate) and (self.cmbAPStatus.value() != CActionStatus.finished or not self.cmbAPPerson.value() or QtGui.qApp.userId == self.cmbAPPerson.value() or QtGui.qApp.userHasRight(urEditOtherpeopleAction))):
                self.btnAPLoadTemplate.setEnabled(False)


    @pyqtSignature('int')
    def on_cmbAPAssistant_currentIndexChanged(self, index):
        self.onActionDataChanged('assistant_id', self.cmbAPAssistant.value())


    @pyqtSignature('QString')
    def on_edtAPNote_textChanged(self, text):
        self.onActionDataChanged('note', text)


    @pyqtSignature('')
    def on_btnAPNomenclatureExpense_clicked(self):
        model = self.tblAPActions.model()
        items = model.items()
        row = self.tblAPActions.currentIndex().row()
        if 0 <= row < len(items):
            record, action = items[row]
            self._openNomenclatureEditor(action, record)


    def _openNomenclatureEditor(self, action, record, requireItems=False):
        # Результат может влиять на продолжение работы функционала "Выполнить"
        if not action.nomenclatureExpense:
            return True

        if action.nomenclatureExpense.stockMotionRecord():
            if requireItems and not action.nomenclatureExpense.stockMotionItems():
                return True

            if QtGui.qApp.keyboardModifiers() & Qt.ShiftModifier:
                dlg = CClientRefundInvoiceEditDialog(self)
                dlg.setData(action.nomenclatureExpense)
            else:
                dlg = CClientInvoiceEditDialog(self, fromAction=True)
                dlg.setFinanceId(self.eventEditor.eventFinanceId)
                dlg.setData(action.nomenclatureExpense)
                dlg.setClientId(self.eventEditor.clientId)
            try:
                dlg.exec_()
                dlg.getRecord()

                result = dlg.closeByOkButton

            finally:
                sip.delete(dlg)

            return result
        else:
            QtGui.QMessageBox.warning(self,
                                      u'Внимание!',
                                      u'Не удалось инициализировать записи ЛСиИМН \nсвязанные с данным действием!',
                                      QtGui.QMessageBox.Ok,
                                      QtGui.QMessageBox.Ok)



    @pyqtSignature('QModelIndex, QModelIndex')
    def on_modelAPActionProperties_dataChanged(self, topLeft, bottomRight):
        self.updateAmount()
        model = self.tblAPActions.model()
        items = model.items()
        row = self.tblAPActions.currentIndex().row()
        record, action = items[row]
        actionTypeId = forceRef(record.value('actionType_id')) if record else None
        actionType = CActionTypeCache.getById(actionTypeId) if actionTypeId else None
        if u'moving' in actionType.flatCode.lower():
            if action[u'Переведен в отделение']:
                self.btnNextAction.setText(u'Перевод')
                self.btnNextAction.setEnabled(self.edtAPEndDate.date().isNull())
            else:
                self.btnNextAction.setText(u'Выписка')
                self.btnNextAction.setEnabled(self.edtAPEndDate.date().isNull())
        elif u'received' in actionType.flatCode.lower():
            if action[u'Направлен в отделение']:
                self.btnNextAction.setText(u'Перевод')
                self.btnNextAction.setEnabled(self.edtAPEndDate.date().isNull())
            else:
                self.btnNextAction.setText(u'Выписка')
                self.btnNextAction.setEnabled(self.edtAPEndDate.date().isNull())
            if u'Доставлен по' in action._actionType._propertiesByName:
                if u'экстренным показаниям' in action[u'Доставлен по'].lower():
                    self.eventEditor.cmbOrder.setCurrentIndex(1)
                elif u'плановым показаниям' in action[u'Доставлен по'].lower():
                    self.eventEditor.cmbOrder.setCurrentIndex(0)
            if hasattr(self.eventEditor, 'tabNotes'):
                form = getEventTypeForm(self.eventEditor.eventTypeId)
                if form == '003':
                    if action and u'Кем направлен' in action._actionType._propertiesByName and u'Кем направлен' in action._properties[topLeft.row()].type().name:
                        if hasattr(self.eventEditor.tabNotes, 'cmbRelegateOrg'):
                            self.eventEditor.tabNotes.cmbRelegateOrg.setValue(forceRef(action[u'Кем направлен']))
                    if action and u'Номер направления' in action._actionType._propertiesByName and u'Номер направления' in action._properties[topLeft.row()].type().name:
                        if hasattr(self.eventEditor.tabNotes, 'edtEventSrcNumber'):
                            self.eventEditor.tabNotes.edtEventSrcNumber.setText(forceString(action[u'Номер направления']))
                    elif u'№ направления' in action._actionType._propertiesByName and u'№ направления' in action._properties[topLeft.row()].type().name:
                        if hasattr(self.eventEditor.tabNotes, 'edtEventSrcNumber'):
                            self.eventEditor.tabNotes.edtEventSrcNumber.setText(forceString(action[u'№ направления']))
                    if action and u'Дата направления' in action._actionType._propertiesByName and u'Дата направления' in action._properties[topLeft.row()].type().name:
                        if hasattr(self.eventEditor.tabNotes, 'edtEventSrcDate'):
                            self.eventEditor.tabNotes.edtEventSrcDate.setDate(forceDate(action[u'Дата направления']))
        elif u'planning' in actionType.flatCode.lower():
            if action and u'Кем направлен' in action._actionType._propertiesByName:
                if hasattr(self.eventEditor.tabNotes, 'cmbRelegateOrg'):
                    self.eventEditor.tabNotes.cmbRelegateOrg.setValue(forceRef(action[u'Кем направлен']))
            if action and u'Номер документа' in action._actionType._propertiesByName:
                if hasattr(self.eventEditor.tabNotes, 'edtEventSrcNumber'):
                    self.eventEditor.tabNotes.edtEventSrcNumber.setText(forceString(action[u'Номер документа']))
            elif u'№ документа' in action._actionType._propertiesByName:
                if hasattr(self.eventEditor.tabNotes, 'edtEventSrcNumber'):
                    self.eventEditor.tabNotes.edtEventSrcNumber.setText(forceString(action[u'№ документа']))
            if action and u'Дата направления' in action._actionType._propertiesByName:
                if hasattr(self.eventEditor.tabNotes, 'edtEventSrcDate'):
                    self.eventEditor.tabNotes.edtEventSrcDate.setDate(forceDate(action[u'Дата направления']))
        elif u'inspectPigeonHole'.lower() in actionType.flatCode.lower():
            self.btnNextAction.setText(u'Закончить')
            self.btnNextAction.setEnabled(self.edtAPEndDate.date().isNull())
        if actionType.defaultBegDate == CActionType.dbdJobTicketTime:
            if self.modelAPActionProperties.getPropertyType(self.tblAPProps.currentIndex().row()).isJobTicketValueType():
                dateTimeJT = action.getJobTicketDateTime()
                if dateTimeJT and forceDateTime(record.value('begDate')) != dateTimeJT:
                    record.setValue('begDate',toVariant(dateTimeJT))
                    self.edtAPBegDate.setDate(dateTimeJT.date())
                    self.edtAPBegTime.setTime(dateTimeJT.time())


    @pyqtSignature('int')
    def on_btnAPPrint_printByTemplate(self, templateId):
        model = self.tblAPActions.model()
        items = model.items()
        row = self.tblAPActions.currentIndex().row()
        if 0<=row<len(items):
            data = getEventContextData(self.eventEditor)
            eventInfo = data['event']
            currentActionIndex = eventInfo.actions._rawItems.index(items[row])
            action = eventInfo.actions[currentActionIndex]
            action.setCurrentPropertyIndex(self.tblAPProps.currentIndex().row())
            data['action'] = action
            data['actions'] = eventInfo.actions
            data['currentActionIndex'] = currentActionIndex
            applyTemplate(self.eventEditor, templateId, data)


    @pyqtSignature('int')
    def on_btnAPLoadTemplate_templateSelected(self, templateId):
        model = self.tblAPActions.model()
        items = model.items()
        row = self.tblAPActions.currentIndex().row()
        if QtGui.qApp.userHasRight(urLoadActionTemplate) and 0<=row<len(items) and (self.cmbAPStatus.value() != CActionStatus.finished or not self.cmbAPPerson.value() or QtGui.qApp.userId == self.cmbAPPerson.value() or QtGui.qApp.userHasRight(urEditOtherpeopleAction)):
            action = items[row][1]
            action.updateByTemplate(templateId, clientSex = self.eventEditor.clientSex, clientAge = self.eventEditor.clientAge)
            for propertie in action._properties:
                if propertie.isActionNameSpecifier():
                    action.updateSpecifiedName()
                    self.modelAPActionProperties.emit(SIGNAL('actionNameChanged()'))
                    self.modelAPActionProperties.emitDataChanged()
            self.tblAPProps.model().reset()
            self.tblAPProps.resizeRowsToContents()
            model.updateActionAmount(row)


    @pyqtSignature('')
    def on_btnAPSaveAsTemplate_clicked(self):
        model = self.tblAPActions.model()
        items = model.items()
        row = self.tblAPActions.currentIndex().row()
        if QtGui.qApp.userHasRight(urSaveActionTemplate) and 0<=row<len(items):
            actionRecord, action = items[row]
            dlg = CActionTemplateSaveDialog(self, actionRecord, action, self.eventEditor.clientSex, self.eventEditor.clientAge, self.eventEditor.personId, self.eventEditor.personSpecialityId)
            try:
                dlg.exec_()
            finally:
                sip.delete(dlg)
            self.actionTemplateCache.reset()
            actionTemplateTreeModel = self.actionTemplateCache.getModel(action.getType().id)
            self.btnAPLoadTemplate.setModel(actionTemplateTreeModel)
            if not(QtGui.qApp.userHasRight(urLoadActionTemplate) and (self.cmbAPStatus.value() != CActionStatus.finished or not self.cmbAPPerson.value() or QtGui.qApp.userId == self.cmbAPPerson.value() or QtGui.qApp.userHasRight(urEditOtherpeopleAction))):
                self.btnAPLoadTemplate.setEnabled(False)


    @pyqtSignature('')
    def on_mnuAPLoadPrevAction_aboutToShow(self):
        model = self.tblAPActions.model()
        items = model.items()
        row = self.tblAPActions.currentIndex().row()
        if 0<=row<len(items):
            record, action = items[row]
        else:
            action = None
        self.actAPLoadSameSpecialityPrevAction.setEnabled(bool(
                action and self.eventEditor.getPrevActionId(action, CGetPrevActionIdHelper.sameSpecialityPrevAction))
                                                         )
        self.actAPLoadOwnPrevAction.setEnabled(bool(
                action and self.eventEditor.getPrevActionId(action, CGetPrevActionIdHelper.ownPrevAction))
                                              )

        self.actAPLoadAnyPrevAction.setEnabled(bool(
                action and self.eventEditor.getPrevActionId(action, CGetPrevActionIdHelper.anyPrevAction))
                                              )


    @pyqtSignature('')
    def on_actAPLoadSameSpecialityPrevAction_triggered(self):
        self.loadPrevAction(CGetPrevActionIdHelper.sameSpecialityPrevAction)


    @pyqtSignature('')
    def on_actAPLoadOwnPrevAction_triggered(self):
        self.loadPrevAction(CGetPrevActionIdHelper.ownPrevAction)


    @pyqtSignature('')
    def on_actAPLoadAnyPrevAction_triggered(self):
        self.loadPrevAction(CGetPrevActionIdHelper.anyPrevAction)

# ##################################################################

#WFT?
class CActionReplicateDialog(QtGui.QDialog):
    def __init__(self, parent=None, sourceAction=None, eventEditor=None):
        QtGui.QDialog.__init__(self, parent)
        self.lblAmount = QtGui.QLabel(u'Количество', self)
        self.edtAmount = QtGui.QSpinBox(self)

        self.lblWeekProfile = QtGui.QLabel(u'Длительность рабочей недели', self)
        self.cmbWeekProfile = QtGui.QComboBox(self)

        self.buttonBox = QtGui.QDialogButtonBox(QtGui.QDialogButtonBox.Cancel|QtGui.QDialogButtonBox.Ok, Qt.Horizontal, self)

        self.layout = QtGui.QGridLayout(self)

        self.layout.addWidget(self.lblAmount, 0, 0)
        self.layout.addWidget(self.edtAmount, 0, 1)
        self.layout.addWidget(self.lblWeekProfile, 1, 0)
        self.layout.addWidget(self.cmbWeekProfile, 1, 1)
        self.layout.addWidget(self.buttonBox, 2, 1)

        self.setLayout(self.layout)

        self._result = []
        self._sourceAction = sourceAction
        self._eventEditor = eventEditor

        self._applyWidgetSettings()

        self.setWindowTitle(u'Тиражирование действия')

        self.connect(self.buttonBox, SIGNAL('accepted()'), self.accept)
        self.connect(self.buttonBox, SIGNAL('rejected()'), self.reject)


    def _applyWidgetSettings(self):
        self.edtAmount.setMinimum(1)

        self.cmbWeekProfile.addItem(u'пятидневная рабочая неделя')
        self.cmbWeekProfile.addItem(u'шестидневаня рабочая неделя')
        self.cmbWeekProfile.addItem(u'семидневная рабочая неделя')

        endDate = self._eventEditor.edtEndDate.date()
        begDate = forceDate(self._sourceAction.getRecord().value('begDate'))
        if endDate and begDate:
            amount = begDate.daysTo(endDate)
            amount = 1 if amount in (0, 1) else amount
        else:
            amount = 1
        self.edtAmount.setValue(amount)


    def getResult(self):
        return self._result


    def _countResult(self):
        from PyQt4.QtSql import QSqlRecord

        def _initActionProperties(action):
            actionType = action.getType()
            propertyTypeList = actionType.getPropertiesById().values()
            for propertyType in propertyTypeList:
                action.getPropertyById(propertyType.id)

        def _setNewDate(record, sourceDate, additional, index, weekType):
            def _getNewDate(newDate, weekType):
                weekProfile = wpFiveDays if weekType == 0 else wpSixDays
                workDay = getNextWorkDay(newDate, weekProfile)
                return workDay
            
            newDate = sourceDate.addDays(additional)
            targetDate = _getNewDate(newDate, weekType)
            additional = sourceDate.daysTo(targetDate)

            record.setValue('begDate', toVariant(targetDate))
            record.setValue('endDate', toVariant(targetDate))
            return additional

        sourceRecord = self._sourceAction.getRecord()
        endDate = forceDate(sourceRecord.value('endDate'))
        #actionType = self._sourceAction.getType()
        additional = 0
        for i in xrange(self.edtAmount.value()):
            newActionRecord = QSqlRecord(sourceRecord)
            newActionRecord.setValue('id', toVariant(None))
            newAction = CAction(record=newActionRecord)
            _initActionProperties(newAction)
            newAction.updateByAction(self._sourceAction)
            if endDate:
                additional = _setNewDate(newActionRecord, endDate, additional, i, self.cmbWeekProfile.currentIndex())
            self._result.append(newAction)


    def exec_(self):
        r = QtGui.QDialog.exec_(self)
        if r:
            self._countResult()
        return r

class CInputCutFeedDialog(CInputDialog):
    def __init__(self, action, actionType, actionRecord, eventPersonId, parent=None):
        CInputDialog.__init__(self, parent)
        self.chkCutFeed = QtGui.QCheckBox(self)
        self.chkCutFeed.setText(u'Отменить питание после перевода')
        self.chkCutFeed.setChecked(True)
        self.setExecPersonVisible(True)
        self.gridLayout.removeWidget(self.buttonBox)
        self.gridLayout.addWidget(self.chkCutFeed, 5, 0, 1, 2)
        self.gridLayout.addWidget(self.buttonBox, 6, 0, 1, 2)
        self.initEditors(action, actionType, actionRecord, eventPersonId)

    def initEditors(self, action, actionType, actionRecord, eventPersonId):
        self.setPerson(forceRef(actionRecord.value('person_id')))
        osTransfer = None
        osPresence = None
        if u'received' in actionType.flatCode.lower():
            osTransfer = action[u'Направлен в отделение']
            osPresence = action[u'Отделение'] if u'Отделение' in actionType._propertiesByName else None
        elif u'moving' in actionType.flatCode.lower():
            osTransfer = action[u'Переведен в отделение']
            osPresence = action[u'Отделение пребывания']
        self.cmbPerson.setOrgStructureId(osPresence, True)
        self.cmbExecPerson.setOrgStructureId(osTransfer, True)
        transferChiefId = getChiefId(osTransfer) if osTransfer else None
        self.setExecPerson(transferChiefId if transferChiefId else eventPersonId)

    def cutFeed(self):
        return self.chkCutFeed.isChecked()
