# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui
from PyQt4.QtCore import Qt, QObject, QVariant, pyqtSignature

from library.DialogBase          import CDialogBase
from library.InDocTable          import CInDocTableModel, CDateInDocTableCol, CFloatInDocTableCol, CInDocTableCol
from library.Utils               import forceDate, forceDouble, forceRef, forceString, pyDate

from Accounting.Tariff           import CTariff
from Orgs.ContractFindComboBox   import CContractTreeFindComboBox
from Registry.AmbCardMixin       import CAmbCardMixin


from Events.Ui_ClientDepositDialog import Ui_ClientDepositDialog


class CClientDepositDialog(CDialogBase, CAmbCardMixin, Ui_ClientDepositDialog):
    def __init__(self, parent, clientId, clientAge, clientSex, buttonBoxIgnore, title):
        CDialogBase.__init__(self, parent)
        self.addModels('Deposit', CClientDepositModel(self))
        self.setWindowTitle(title)
        self.setupUi(self)
        self.setWindowFlags(self.windowFlags() | Qt.WindowMaximizeButtonHint)
        self.setModels(self.tblDeposit,  self.modelDeposit, self.selectionModelDeposit)
        self.setWindowTitle(title)
        self.currentClientId = clientId
        self.clientAge = clientAge
        self.clientSex = clientSex
        self.buttonBoxIgnore = buttonBoxIgnore
        self.btnIgnore.setVisible(self.buttonBoxIgnore)
        self.modelDeposit.loadItems(self.currentClientId)
        self.on_btnUpdateFactSum_clicked()


    @pyqtSignature('')
    def on_btnClose_clicked(self):
        from Registry.ClientEditDialog   import CClientEditDialog
        from Events.EventEditDialog      import CEventEditDialog

        if isinstance(self.parent(), CClientEditDialog) or isinstance(self.parent(), CEventEditDialog):
            self.parent().btnExit = False
        self.close()


    @pyqtSignature('')
    def on_btnIgnore_clicked(self):
        from Registry.ClientEditDialog   import CClientEditDialog
        from Events.EventEditDialog      import CEventEditDialog

        if isinstance(self.parent(), CClientEditDialog) or isinstance(self.parent(), CEventEditDialog):
            self.parent().btnExit = True
        self.close()


    @pyqtSignature('')
    def on_btnUpdateFactSum_clicked(self):
        self.modelDeposit.factSum = []
        contractIdOfDateList = {}
        contractIdList = {}
        parentObj = QObject.parent(self)
        if not parentObj:
            return
        eventId = parentObj.itemId()
        if hasattr(parentObj, 'tabCash'):
            for row, record in enumerate(parentObj.tabCash.modelAccActions._items):
                accContractId = forceRef(record.value('contract_id'))
                if accContractId:
                    sum = parentObj.tabCash.modelAccActions.sums[row]
                    contractSumList = contractIdList.get(accContractId, 0.0)
                    contractSumList += sum
                    contractIdList[accContractId] = contractSumList

        for row, record in enumerate(self.modelDeposit._items):
            contractId = forceRef(record.value('contract_id'))
            contractDate = forceDate(record.value('contractDate'))
            dateList = contractIdOfDateList.get(contractId, [])
            if contractDate and contractDate not in dateList:
                dateList.append(pyDate(contractDate))
                contractIdOfDateList[contractId] = dateList

        for row, record in enumerate(self.modelDeposit._items):
            contractId = forceRef(record.value('contract_id'))
            contractDate = forceDate(record.value('contractDate'))
            dateList = contractIdOfDateList.get(contractId, [])
            dateList.sort()
            lenDateList = len(dateList)
            if lenDateList > 1:
                begDateContract = dateList[0]
                endDateContract = dateList[lenDateList-1]
            else:
                begDateContract = contractDate
                endDateContract = 0
            sumAction = 0.0
            sumItem = 0.0
            sumCurrentEvent = 0.0
            if contractId:
                db = QtGui.qApp.db
                tableAction = db.table('Action')
                tableEvent = db.table('Event')
                tableAccountItem = db.table('Account_Item')

            #  payStatus != 0
                cond = [tableAction['deleted'].eq(0),
                        tableEvent['deleted'].eq(0),
                        tableAccountItem['deleted'].eq(0),
                        tableAction['contract_id'].eq(contractId),
                        tableAction['payStatus'].ne(0),
                        tableAccountItem['refuseType_id'].isNull()
                        ]
                if eventId:
                    cond.append(tableEvent['id'].ne(eventId))
                if begDateContract:
                    cond.append(tableAction['begDate'].dateGe(begDateContract))
                if endDateContract:
                    cond.append(tableAction['begDate'].dateLt(endDateContract))
                cond.append(tableEvent['client_id'].eq(self.currentClientId))
                table = tableEvent.innerJoin(tableAction, tableEvent['id'].eq(tableAction['event_id']))
                table = table.innerJoin(tableAccountItem, tableAction['id'].eq(tableAccountItem['action_id']))
                records = db.getRecordListGroupBy(table, u'SUM(Account_Item.sum) AS sumAction, Action.contract_id', cond, u'Action.contract_id')
                for newRecord in records:
                    contractId = forceRef(newRecord.value('contract_id'))
                    sumAction += forceDouble(newRecord.value('sumAction'))

            #  payStatus == 0
                tableActionTypeService = db.table('ActionType_Service')
                tableRBService = db.table('rbService')
                tableContractTariff = db.table('Contract_Tariff')
                tableA = db.table('Action').alias('A')
                cols = u'''SUM(IF(A.amount > Contract_Tariff.amount AND Contract_Tariff.amount != 0, Contract_Tariff.amount * Contract_Tariff.price,
                A.amount * Contract_Tariff.price))'''
                cond = [tableContractTariff['deleted'].eq(0),
                        tableContractTariff['master_id'].eq(contractId),
                        tableContractTariff['tariffType'].eq(CTariff.ttActionAmount)
                        ]
                cond.append(u'A.id = Action.id')
                if self.clientAge:
                    cond.append(db.joinOr([tableContractTariff['age'].eq(''), tableContractTariff['age'].ge(self.clientAge[3])]))
                cond.append(u'''ActionType_Service.`service_id`=(SELECT ATS.`service_id`
                                FROM ActionType_Service AS ATS
                                WHERE ATS.`master_id`=ActionType_Service.`master_id`
                                AND (ATS.`finance_id` IS NULL OR A.`finance_id`=ATS.`finance_id`)
                                ORDER BY ATS.`finance_id` DESC
                                LIMIT 1)''')
                cond.append(db.joinOr([tableContractTariff['sex'].eq(0), tableContractTariff['sex'].eq(self.clientSex)]))
                table = tableA.innerJoin(tableActionTypeService, tableA['actionType_id'].eq(tableActionTypeService['master_id']))
                table = table.innerJoin(tableContractTariff, tableActionTypeService['service_id'].eq(tableContractTariff['service_id']))
                stmt = db.selectStmtGroupBy(table, cols, cond, u'A.contract_id')

                condQuery = [tableAction['deleted'].eq(0),
                            tableEvent['deleted'].eq(0),
                            tableAction['contract_id'].eq(contractId),
                            tableAction['payStatus'].eq(0)
                            ]
                if eventId:
                    condQuery.append(tableEvent['id'].ne(eventId))
                condQuery.append(tableEvent['client_id'].eq(self.currentClientId))
                tableQuery = tableEvent.innerJoin(tableAction, tableEvent['id'].eq(tableAction['event_id']))
                colsQuery = [u'(%s) AS sumItem'%stmt,
                             tableAction['contract_id']
                             ]
                records = db.getRecordList(tableQuery, colsQuery, condQuery)
                for newRecord in records:
                    contractId = forceRef(newRecord.value('contract_id'))
                    sumItem += forceDouble(newRecord.value('sumItem'))
                sumCurrentEvent = contractIdList.get(contractId, 0.0)

                self.modelDeposit.factSum.append(sumAction + sumItem + sumCurrentEvent)
                self.modelDeposit.reset()


class CClientDepositModel(CInDocTableModel):
    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'ClientDeposit', 'id', 'client_id', parent)
        self.addCol(CDateInDocTableCol(u'Дата', 'contractDate', 15, canBeEmpty=True)).setReadOnly(True)
        self.addCol(CClientDepositModel.CContractInDocTableCol(self)).setReadOnly(True)
        self.addCol(CFloatInDocTableCol(u'Сумма депозита', 'contractSum', 8, precision=4)).setReadOnly(True)
        self.addExtCol(CFloatInDocTableCol( u'Сумма фактическая',   'factSum', 10), QVariant.Int).setReadOnly(True)
        self.columnFactSum = self.getColIndex('factSum')
        self.factSum = []


    def data(self, index, role=Qt.DisplayRole):
        column = index.column()
        if role == Qt.DisplayRole:
            if column == self.columnFactSum:
                row = index.row()
                if 0 <= row < len(self.items()) and len(self.factSum) <= len(self.items()):
                    return QVariant(self.factSum[row])
        return CInDocTableModel.data(self, index, role)


    class CContractInDocTableCol(CInDocTableCol):

        def __init__(self, model):
            CInDocTableCol.__init__(self, u'Договор', 'contract_id', 20)
            self.model = model


        def toString(self, val, record):
            contractId = forceRef(val)
            if contractId:
                names = ['number', 'date', 'resolution']
                record = QtGui.qApp.db.getRecord('Contract', names, contractId)
                str = ' '.join([forceString(record.value(name)) for name in names])
            else:
                str = u'не задано'
            return QVariant(str)


        def createEditor(self, parent):
            filter = {}
            editor = CContractTreeFindComboBox(parent, filter)
            return editor


        def getEditorData(self, editor):
            return QVariant(editor.value())


        def setEditorData(self, editor, value, record):
            date = forceDate(record.value('contractDate'))
            editor.setDate(date)


