# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui
from PyQt4.QtCore import QDate

from library.Utils              import forceDouble, forceInt, forceRef

from Accounting.Tariff          import CTariff


class CContractTariffDescr:
    def __init__(self, contractId, tariffChecker):
        # tariffChecker должен предоставить 2 метода:
        # getEventTypeId() -> id типа события
        # recordAcceptable(record) -> запись подходит по полу и возрасту
        self.visitTariffMap = {}
        self.actionTariffMap = {}
        self.financeId = None

        if contractId:
            db = QtGui.qApp.db
            self.financeId = forceRef(db.translate('Contract', 'id', contractId, 'finance_id'))
            table = db.table('Contract_Tariff')
            priceListId = forceRef(db.translate('Contract', 'id', contractId, 'priceListExternal_id'))
            if priceListId is None:
                masterCond = table['master_id'].eq(contractId)
                order = 'id'
            else:
                masterCond = table['master_id'].inlist([contractId, priceListId])
                # Антон, такая сортировка подходит?
                order = 'master_id, id' if priceListId<contractId else 'master_id DESC, id'
            currentDate = QDate.currentDate()
            cond = [ masterCond,
                     table['deleted'].eq(0),
                     db.joinOr([table['begDate'].isNull(), table['begDate'].le(currentDate)]),
                     db.joinOr([table['endDate'].isNull(), table['endDate'].ge(currentDate)]),
                   ]
            eventTypeId = tariffChecker.getEventTypeId() if tariffChecker else None
            if eventTypeId:
                cond.append(db.joinOr([table['eventType_id'].eq(eventTypeId),
                                       table['eventType_id'].isNull()
                                     ]))
            records = db.getRecordList(table,
                                       'tariffType, service_id, tariffCategory_id, begDate, endDate, sex, age, price, uet',
                                       cond,
                                       order)
            for record in records:
                if not tariffChecker or tariffChecker.recordAcceptable(record):
                    tariffType = forceInt(record.value('tariffType'))
                    serviceId = forceRef(record.value('service_id'))
                    tariffCategoryId = forceRef(record.value('tariffCategory_id'))
                    price = forceDouble(record.value('price'))
                    uet = forceDouble(record.value('uet'))
                    if tariffType == CTariff.ttVisit:
                        self._register(self.visitTariffMap, serviceId, tariffCategoryId, price, uet)
                    elif tariffType in (CTariff.ttActionAmount, CTariff.ttActionUET,  CTariff.ttHospitalBedService):
                        self._register(self.actionTariffMap, serviceId, tariffCategoryId, price, uet)


    @staticmethod
    def _register(dataMap, serviceId, tariffCategoryId, price, uet):
        mapTariffCategoryToPriceAndUet = dataMap.setdefault(serviceId, {})
        priceAndUet = mapTariffCategoryToPriceAndUet.get(tariffCategoryId, None)
        if priceAndUet is None:
            mapTariffCategoryToPriceAndUet[tariffCategoryId] = price, uet
            if tariffCategoryId is None:
                mapTariffCategoryToPriceAndUet[0] = price, uet


class CContractTariffCache:
    def __init__(self):
        self.mapContractIdToDecr = {}

    def getTariffDescr(self, contractId, tariffChecker):
        if contractId in self.mapContractIdToDecr:
            return self.mapContractIdToDecr[contractId]
        else:
            result = CContractTariffDescr(contractId, tariffChecker)
            self.mapContractIdToDecr[contractId] = result
            return result


    @staticmethod
    def getServiceIdList(tariffMap):
         return tariffMap.keys()


    @staticmethod
    def getPriceAndUet(tariffMap, serviceIdList, tariffCategoryId):
        resultPrice = 0.0
        resultUet = 0.0
        for serviceId in serviceIdList:
            mapTariffCategoryToPriceAndUet = tariffMap.get(serviceId, None)
            if mapTariffCategoryToPriceAndUet:
                priceAndUet = mapTariffCategoryToPriceAndUet.get(tariffCategoryId, None)
                if priceAndUet is None:
                    priceAndUet = mapTariffCategoryToPriceAndUet.get(None, None)
                if priceAndUet:
                    resultPrice += priceAndUet[0]
                    resultUet += priceAndUet[1]
        return resultPrice, resultUet


    @staticmethod
    def getPrice(tariffMap, serviceIdList, tariffCategoryId):
        return CContractTariffCache.getPriceAndUet(tariffMap, serviceIdList, tariffCategoryId)[0]


    @staticmethod
    def getUet(tariffMap, serviceIdList, tariffCategoryId):
        return CContractTariffCache.getPriceAndUet(tariffMap, serviceIdList, tariffCategoryId)[1]
