# -*- coding: utf-8 -*-

#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import sip
from PyQt4 import QtGui
from PyQt4.QtCore  import QDate, QDateTime

from library.AgeSelector          import checkAgeSelector
from library.Utils                import calcAgeTuple, forceBool, forceDate, forceDateTime, forceRef, forceString, formatDateTime, toVariant

from Events.EditDispatcher        import getEventFormClass, getEventFormClassByType
from Events.PreCreateEventDialog  import CPreCreateEventDialog
from Events.Utils                 import CFinanceType, getEventShowTime, checkEventPosibility, getDeathDate, getEventAgeSelector, getEventFinanceCode, getEventTypeForm, isEventDeath,  getEventOrder
from Registry.CheckEnteredOpenEventsDialog import CCheckEnteredOpenEvents
from Registry.Utils               import getClientCompulsoryPolicy, getClientIdentification, getClientVoluntaryPolicy, getClientWork
from Users.Rights                 import urAdmin, urRegTabWriteEvents


def requestNewEvent(params):
    widget = params['widget']
    clientId = params['clientId']
    if QtGui.qApp.userHasAnyRight([urAdmin, urRegTabWriteEvents]) and clientId and checkClientCanHaveEvent(widget, clientId):
        flagHospitalization = params.get('flagHospitalization', False)
        actionTypeId        = params.get('actionTypeId', None)
        valueProperties     = params.get('valueProperties', [])
        planningEventId     = params.get('planningEventId', None)
        diagnos             = params.get('diagnos', '')
        financeId           = params.get('financeId', None)
        protocolQuoteId     = params.get('protocolQuoteId', None)
        actionByNewEvent    = params.get('actionByNewEvent', [])
        transferDataList    = params.get('transferDataList', [])
        relegateOrgId       = params.get('relegateOrgId', None)
        relegatePersonId    = params.get('relegatePersonId', None)
        prevEventId         = params.get('prevEventId', None)
        order               = params.get('order', None)
        isMoving            = params.get('moving', False)
        actionListToNewEvent = params.get('actionListToNewEvent', [])
        eventDatetime        = params.get('endDateTime', None)
        typeQueue            = params.get('typeQueue', -1)
        result               = params.get('result', None)
        docNum               = params.get('docNum', None)
        plannedEndDate       = params.get('plannedEndDate', None)
        mapJournalInfoTransfer = params.get('mapJournalInfoTransfer', None)

        while True:
            destroyCounterController()
            dlg = CPreCreateEventDialog(params)
            if transferDataList:
                externalId        = params.get('externalId', '')
            else:
                if not dlg.exec_():
                    return
                externalId        = dlg.externalId()
            orgId                 = dlg.orgId()
            relegateOrgId         = dlg.relegateOrgId()
            relegatePersonId      = dlg.relegatePersonId()
            srcPerson            = dlg.getSrcPerson()
            srcNumber            = dlg.getSrcNumber()
            srcDate              = dlg.getSrcDate()
            relegateInfo         = [srcPerson, srcNumber, srcDate, relegateOrgId, relegatePersonId]

            eventTypeId           = dlg.eventTypeId()
            tissueTypeId          = dlg.tissueTypeId()
            personId              = dlg.personId()
            eventSetDatetime      = dlg.eventSetDate()
            if not eventDatetime:
                eventDatetime         = dlg.eventDate()
            weekProfile           = dlg.weekProfile()
            selectPreviousActions = dlg.selectPreviousActions()
            days                  = dlg.days()
            assistantId           = dlg.assistantId()
            curatorId             = dlg.curatorId()
            eventSetDate = eventSetDatetime.date() if eventSetDatetime else None
            eventDate    = eventDatetime.date() if eventDatetime else None
            form         = getEventTypeForm(eventTypeId)
            deathDate = forceDateTime(QtGui.qApp.db.translate('Client', 'id', clientId, 'deathDate'))
            if order is None:
                order = getEventOrder(eventTypeId)
            if not eventSetDate and not eventDate:
                msg = u'Новый осмотр не может быть добавлен.\nНе введены даты начала и выполнения данного события.'
                boxResult = QtGui.QMessageBox.question(widget,
                    u'Внимание!',
                    msg,
                    QtGui.QMessageBox.Close|QtGui.QMessageBox.Retry,
                    QtGui.QMessageBox.Close
                    )
                if boxResult == QtGui.QMessageBox.Retry:
                    continue
                else:
                    break
            eventId = findCreateEventToDate(eventTypeId, personId, clientId, eventSetDate, eventDate)
            if eventId:
                msg = u'Новый осмотр указанного типа не может быть добавлен.\nОткрыть существующий?'
                boxResult = QtGui.QMessageBox.question(widget,
                    u'Внимание!',
                    msg,
                    QtGui.QMessageBox.Yes|QtGui.QMessageBox.No|QtGui.QMessageBox.Retry,
                    QtGui.QMessageBox.Yes
                    )
                if boxResult == QtGui.QMessageBox.Retry:
                    continue
                elif boxResult == QtGui.QMessageBox.Yes and eventId:
                    destroyCounterController()
                    editEvent(widget, eventId)
                    return eventId
                else:
                    break
            if deathDate and deathDate <= eventSetDatetime and not isEventDeath(eventTypeId):
                QtGui.QMessageBox.critical(widget,
                                                u'Внимание!',
                                                u'Пациент умер %s'%formatDateTime(deathDate),
                                                QtGui.QMessageBox.Ok,
                                                QtGui.QMessageBox.Ok)
                break
            resCheckSocStatus, isCheckSocStatus, socStatusNames = checkSocStatus(widget, clientId, eventSetDate)
            if not resCheckSocStatus and isCheckSocStatus:
                msg = u'''У пациента отсутствует обязательный социальный статус: '%s'!'''%(socStatusNames)
                if isCheckSocStatus == 1:
                    buttonBox = QtGui.QMessageBox.Ignore|QtGui.QMessageBox.Close
                else:
                    buttonBox = QtGui.QMessageBox.Close
                boxResult = QtGui.QMessageBox.question(widget,
                    u'Внимание!',
                    msg,
                    buttonBox,
                    QtGui.QMessageBox.Close
                    )
                if boxResult == QtGui.QMessageBox.Close:
                    break
            if not checkEventTypeAge(widget, clientId, eventTypeId, eventSetDate):
                msg = u'Новый осмотр указанного типа не может быть добавлен.\nВозраст пациента не попадает в диапазон возрастов данного события.'
                boxResult = QtGui.QMessageBox.question(widget,
                    u'Внимание!',
                    msg,
                    QtGui.QMessageBox.Close,
                    QtGui.QMessageBox.Close
                    )
                break
            if eventDate and eventSetDate > eventDate:
                if not confirmTrouble(widget, u'Дата окончания %s не может быть раньше даты начала %s' % (forceString(eventDate), forceString(eventSetDate))):
                    continue
            if not checkDatesRegardToClientLife(widget, clientId, eventSetDate, eventDate, eventTypeId):
                continue
            if not checkWorkHurts(widget, clientId, eventTypeId):
                continue
            if form != '001' and not isMoving:
                eventId = findSameEvent(eventTypeId, personId, clientId, eventSetDate, eventDate)
                if eventId:
                    msg = u'Новый осмотр указанного типа пересекается по времени с существующим.\nОткрыть существующий?'
                    boxResultExisting = QtGui.QMessageBox.question(widget,
                        u'Внимание!',
                        msg,
                        QtGui.QMessageBox.Yes|QtGui.QMessageBox.Cancel|QtGui.QMessageBox.Ignore,
                        QtGui.QMessageBox.Yes
                        )
                    if boxResultExisting == QtGui.QMessageBox.Cancel:
                        break
                    elif boxResultExisting == QtGui.QMessageBox.Yes and eventId:
                        destroyCounterController()
                        editEvent(widget, eventId)
                        return eventId
            if not isMoving:
                btnOpenEvent, eventId = checkClientHasOpenEvents(widget, eventTypeId, clientId, eventSetDate, eventDate, personId, form)
                if eventId and btnOpenEvent == 2:
                    destroyCounterController()
                    editEvent(widget, eventId)
                    return eventId
                elif btnOpenEvent == 0:
                    break
                elif btnOpenEvent == 1:
                    continue
            if form == '001' or isMoving:
                canEnterEvent = True
            else:
                canEnterEvent, priorEventId = checkEventPosibility(clientId, eventTypeId, personId, eventDate if eventDate else eventSetDate)
            if canEnterEvent:
                if (    widget.checkClientAttach(personId, clientId, eventDatetime if eventDatetime else eventSetDatetime, False)
                    and (    widget.checkClientAttendance(personId, clientId)
                          or widget.confirmClientAttendance(widget, personId, clientId)
                        )
                    and (    checkClientPolicyAttendance(eventTypeId, clientId, eventDate or eventSetDate)
                          or confirmClientPolicyAttendance(widget, eventTypeId, clientId)
                        )
                    and confirmClientPolicyConstraint(widget, eventTypeId, clientId, eventSetDate, eventDate)
                   ):
                        if transferDataList:
                            return saveTransferEvent(widget, transferDataList, clientId, eventTypeId, orgId, personId, eventDatetime, eventSetDatetime, weekProfile, days, externalId, assistantId, curatorId, flagHospitalization, actionTypeId, valueProperties, tissueTypeId, selectPreviousActions, relegateOrgId, relegatePersonId, planningEventId, diagnos, financeId, protocolQuoteId, actionByNewEvent, order)
                        else:
                            return createEvent(widget, form, clientId, eventTypeId, orgId, personId, eventDatetime, eventSetDatetime, weekProfile, days, externalId, assistantId, curatorId,
                                               flagHospitalization, actionTypeId, valueProperties, tissueTypeId, selectPreviousActions, relegateOrgId, relegatePersonId, planningEventId, diagnos, financeId,
                                               protocolQuoteId, actionByNewEvent, order, actionListToNewEvent, result, prevEventId, typeQueue, docNum, relegateInfo, plannedEndDate, mapJournalInfoTransfer)
                else:
                    continue
            else:
                msg = u'Новый осмотр указанного типа не может быть добавлен.'
                if priorEventId:
                    msg += u'\nОткрыть существующий?'
                boxResult = QtGui.QMessageBox.question(widget,
                    u'Внимание!',
                    msg,
                    QtGui.QMessageBox.Yes|QtGui.QMessageBox.No|QtGui.QMessageBox.Retry,
                    QtGui.QMessageBox.Yes
                    )
                if boxResult == QtGui.QMessageBox.Retry:
                    continue
                elif boxResult == QtGui.QMessageBox.Yes and priorEventId:
                    destroyCounterController()
                    editEvent(widget, priorEventId)
                    return priorEventId
                else:
                    break
    destroyCounterController()
    return None


def destroyCounterController():
    QtGui.qApp.resetAllCounterValueIdReservation()
    QtGui.qApp.setCounterController(None)


def saveTransferEvent(widget, transferDataList, clientId, eventTypeId, orgId, personId, eventDatetime, eventSetDatetime, weekProfile, days, externalId, assistantId, curatorId, flagHospitalization, actionTypeId, valueProperties, tissueTypeId, selectPreviousActions, relegateOrgId, relegatePersonId, planningEventId, diagnos, financeId, protocolQuoteId, actionByNewEvent, order):
    recordEvent, recordDiagnosis, newAction, oldEventId = transferDataList
    eventSetDateTime = eventSetDatetime if isinstance(eventSetDatetime, QDateTime) else QDateTime(eventSetDatetime)
    eventExecDateTime = eventDatetime if isinstance(eventDatetime, QDateTime) else QDateTime(eventDatetime)
    db = QtGui.qApp.db
    tableEvent = db.table('Event')
    newEventRecord = tableEvent.newRecord()
    for i in range(0, newEventRecord.count()):
        newEventRecord.setValue(newEventRecord.fieldName(i), toVariant(recordEvent.value(recordEvent.fieldName(i))))
    newEventRecord.setValue('id', toVariant(None))
    newEventRecord.setValue('isClosed', toVariant(0))
    newEventRecord.setValue('MES_id', toVariant(None))
    newEventRecord.setValue('mesSpecification_id', toVariant(None))
    newEventRecord.setValue('client_id', toVariant(clientId))
    newEventRecord.setValue('org_id', toVariant(orgId))
    newEventRecord.setValue('execPerson_id', toVariant(personId))
    newEventRecord.setValue('prevEventDate', toVariant(newEventRecord.value('setDate')))
    newEventRecord.setValue('setDate', toVariant(eventSetDateTime))
    newEventRecord.setValue('execDate', toVariant(eventExecDateTime))
    newEventRecord.setValue('externalId', toVariant(externalId))
    newEventRecord.setValue('assistant_id', toVariant(assistantId))
    newEventRecord.setValue('curator_id', toVariant(curatorId))
    newEventRecord.setValue('order', toVariant(order))
    newEventRecord.setValue('relegateOrg_id', toVariant(relegateOrgId))
    newEventRecord.setValue('relegatePerson_id', toVariant(relegatePersonId))
    newEventRecord.setValue('result_id', toVariant(None))
    newEventRecord.setValue('prevEvent_id', toVariant(oldEventId))

    eventId = saveTransferData(widget,'Event', newEventRecord)
    if eventId:
        newActionRecord = newAction._record
        newActionRecord.setValue('event_id', toVariant(eventId))
        newActionRecord.setValue('idx', toVariant(9999))
        newAction.save(eventId, -1)
    return eventId


def saveTransferData(widget, tableName, record):
    try:
        db = QtGui.qApp.db
        db.transaction()
        try:
            id = db.insertOrUpdate(db.table(tableName), record)
            db.commit()
        except:
            db.rollback()
            raise
        return id
    except Exception, e:
        QtGui.qApp.logCurrentException()
        QtGui.QMessageBox.critical( widget,
                                    u'',
                                    unicode(e),
                                    QtGui.QMessageBox.Close)
        return None


def confirmTrouble(widget, message):
    boxResult = QtGui.QMessageBox.question(widget,
                                    u'Внимание!',
                                    message,
                                    QtGui.QMessageBox.Ok,
                                    QtGui.QMessageBox.Ok)
    if boxResult == QtGui.QMessageBox.Ok:
        return False


def checkEventTypeAge(widget, clientId, eventTypeId, eventSetDate):
    if clientId:
        db = QtGui.qApp.db
        birthDate = forceDate(db.translate('Client', 'id', clientId, 'birthDate'))
        if birthDate:
            ageSelector = getEventAgeSelector(eventTypeId)
            if not eventSetDate:
                eventSetDate = QDate.currentDate()
            return checkAgeSelector(ageSelector, calcAgeTuple(birthDate, eventSetDate))
    return False


def checkSocStatus(widget, clientId, eventSetDate):
    isCheckSocStatus, socStatusRes = QtGui.qApp.getStrictCheckSocStatus()
    socStatusNames = u''
    if isCheckSocStatus:
        db = QtGui.qApp.db
        if socStatusRes:
            tableSSC = db.table('rbSocStatusClass')
            records = db.getRecordList(tableSSC, tableSSC['name'], [tableSSC['id'].inlist(socStatusRes)])
            for record in records:
                if socStatusNames:
                    socStatusNames += u', '
                socStatusNames += forceString(record.value('name'))
        if clientId:
#            socStatusClassIdList = []
            table = db.table('ClientSocStatus')
            cond = [table['deleted'].eq(0),
                    table['client_id'].eq(clientId),
                    ]
            if eventSetDate:
                cond.append(db.joinOr([table['endDate'].isNull(), table['endDate'].dateGe(eventSetDate)]))
                cond.append(db.joinOr([table['begDate'].isNull(), table['begDate'].dateLe(eventSetDate)]))
            records = db.getRecordList(table, table['socStatusClass_id'], cond)
            for record in records:
                socStatusClassId = forceRef(record.value('socStatusClass_id'))
                if socStatusClassId in socStatusRes:
                    return True, isCheckSocStatus, socStatusNames
    return False, isCheckSocStatus, socStatusNames


def checkWorkHurts(widget, clientId, eventTypeId):
    db = QtGui.qApp.db
    tableEventTypeDiagnostic = db.table('EventType_Diagnostic')
    cond = [tableEventTypeDiagnostic['eventType_id'].eq(eventTypeId),
            db.joinOr([tableEventTypeDiagnostic['hurtType'].ne(''), tableEventTypeDiagnostic['hurtFactorType'].ne('')])]
    if not db.getRecordEx(tableEventTypeDiagnostic, tableEventTypeDiagnostic['id'].name(), cond):
        return True

    clientWorkRecord = getClientWork(clientId)
    if clientWorkRecord:
        tableClientWorkHurt = db.table('ClientWork_Hurt')
        cond = [tableClientWorkHurt['master_id'].eq(forceRef(clientWorkRecord.value('id')))]
        if db.getRecordEx(tableClientWorkHurt, tableClientWorkHurt['id'].name(), cond):
            return True

    QtGui.QMessageBox.information(widget,
                            u'Внимание!',
                            u'У пациента отсутствуют записи о вредности работы.',
                            QtGui.QMessageBox.Ok,
                            QtGui.QMessageBox.Ok)

    return False

def checkClientCanHaveEvent(widget, clientId):
    eventControl = QtGui.qApp.getGlobalPreference('20') # WTF!!!!
    if eventControl in (u'мягкий', u'строгий'):
        identifiers = getClientIdentification(clientId)
        if u'Согласие на обследование' in identifiers and identifiers[u'Согласие на обследование'][0] == u'Да':
            lastDate = identifiers[u'Согласие на обследование'][1].addYears(1)
            if lastDate.daysTo(QDate.currentDate()) >= 0:
                if eventControl == u'строгий':
                    QtGui.QMessageBox.critical(widget,
                                        u'Ошибка',
                                        u'Согласие на обследование пациента просрочено',
                                        QtGui.QMessageBox.Ok,
                                        QtGui.QMessageBox.Ok)
                    return False
                else:
                    append = QtGui.QMessageBox.question(widget,
                        u'Внимание!',
                        u'Согласие на обследование пациента просрочено. Все равно добавить событие?',
                        QtGui.QMessageBox.Yes|QtGui.QMessageBox.No,
                        QtGui.QMessageBox.No)
                    return (append == QtGui.QMessageBox.Yes)
            else:
                return True
        else:
            if eventControl == u'строгий':
                QtGui.QMessageBox.critical(widget,
                                        u'Ошибка',
                                        u'У пациента нет согласия на обследование',
                                        QtGui.QMessageBox.Ok,
                                        QtGui.QMessageBox.Ok)
                return False
            else:
                append = QtGui.QMessageBox.question(widget,
                        u'Внимание!',
                        u'У пациента нет согласия на обследование. Все равно добавить событие?',
                        QtGui.QMessageBox.Yes|QtGui.QMessageBox.No,
                        QtGui.QMessageBox.No)
                return append == QtGui.QMessageBox.Yes
    else:
        return True


def checkDatesRegardToClientLife(widget, clientId, eventSetDate, eventDate, eventTypeId):
    result = True
    if clientId:
        db = QtGui.qApp.db
        birthDate   = forceDate(db.translate('Client', 'id', clientId, 'birthDate'))
        deathDate   = getDeathDate(clientId)
        possibleDeathDate = birthDate.addYears(QtGui.qApp.maxLifeDuration)
        if birthDate:
            postmortem = isEventDeath(eventTypeId)
            if eventSetDate:
                result = result and (eventSetDate >= birthDate or confirmTrouble(widget, u'Дата назначения %s не должна быть раньше даты рождения пациента %s' % (forceString(eventSetDate), forceString(birthDate))))
                if deathDate:
                    result = result and (eventSetDate <= deathDate or postmortem or confirmTrouble(widget, u'Дата назначения %s не должна быть позже имеющейся даты смерти пациента %s' % (forceString(eventSetDate), forceString(deathDate))))
                else:
                    result = result and (eventSetDate <= possibleDeathDate or postmortem or confirmTrouble(widget, u'Дата назначения %s не должна быть позже возможной даты смерти пациента %s' % (forceString(eventSetDate), forceString(possibleDeathDate))))

            if eventDate:
                result = result and (eventDate >= birthDate or confirmTrouble(widget, u'Дата выполнения (окончания) %s не должна быть раньше даты рождения пациента %s' % (forceString(eventDate), forceString(birthDate))))
                if deathDate:
                    result = result and (eventDate <= deathDate or postmortem or confirmTrouble(widget, u'Дата выполнения (окончания) %s не должна быть позже имеющейся даты смерти пациента %s' % (forceString(eventDate), forceString(deathDate))))
                else:
                    result = result and (eventDate <= possibleDeathDate or postmortem or confirmTrouble(widget, u'Дата выполнения (окончания) %s не должна быть позже возможной даты смерти пациента %s' % (forceString(eventDate), forceString(possibleDeathDate))))
    return result


def findCreateEventToDate(eventTypeId, personId, clientId, eventSetDate, eventDate):
    if clientId and eventTypeId and personId and (eventSetDate or eventDate):
        db = QtGui.qApp.db
        tableEvent  = db.table('Event')
        cols = [tableEvent['id']]
        cond = [tableEvent['client_id'].eq(clientId),
                tableEvent['eventType_id'].eq(eventTypeId),
                tableEvent['deleted'].eq(0)
                ]
        queryTable = tableEvent
        if QtGui.qApp.isCheckedEventForPersonOrSpeciality():
            cond.append(tableEvent['execPerson_id'].eq(personId))
        else:
            if personId:
                db = QtGui.qApp.db
                tablePerson = db.table('Person')
                record = db.getRecordEx(tablePerson, [tablePerson['speciality_id']], [tablePerson['deleted'].eq(0), tablePerson['id'].eq(personId)])
                specialityId = forceRef(record.value('speciality_id')) if record else None
                if specialityId:
                    queryTable = queryTable.innerJoin(tablePerson, tableEvent['execPerson_id'].eq(tablePerson['id']))
                    cond.append(tablePerson['speciality_id'].eq(specialityId))
                    cond.append(tablePerson['deleted'].eq(0))
        eventShowTime = getEventShowTime(eventTypeId)
        if eventSetDate:
            eventSetDateTime = db.formatDate(eventSetDate)
        else:
            eventSetDateTime = None
        if eventDate:
            eventDateTime = db.formatDate(eventDate)
        else:
            eventDateTime = None
        if eventSetDateTime:
            if eventShowTime:
                cond.append(u'''Event.setDate = %s''' % (eventSetDateTime))
            else:
                cond.append(u'''DATE(Event.setDate) = %s''' % (eventSetDateTime))
        elif eventDateTime:
            if eventShowTime:
                cond.append(u'''Event.execDate = %s''' % (eventDateTime))
            else:
                cond.append(u'''DATE(Event.execDate) = %s''' % (eventDateTime))
        recordList = db.getRecordList(queryTable, cols, cond, 'Event.setDate')
        if recordList:
            record = recordList[0]
            eventId = forceRef(record.value('id'))
            return eventId
    return None


def findSameEvent(eventTypeId, personId, clientId, eventSetDate, eventDate):
    if clientId and eventTypeId and personId and (eventSetDate or eventDate):
        db = QtGui.qApp.db
        tableEvent  = db.table('Event')
        cols = [tableEvent['id']]
        cond = [tableEvent['client_id'].eq(clientId),
                tableEvent['eventType_id'].eq(eventTypeId),
                tableEvent['deleted'].eq(0)
                ]
        queryTable = tableEvent
        if QtGui.qApp.isCheckedEventForPersonOrSpeciality():
            cond.append(tableEvent['execPerson_id'].eq(personId))
        else:
            if personId:
                db = QtGui.qApp.db
                tablePerson = db.table('Person')
                record = db.getRecordEx(tablePerson, [tablePerson['speciality_id']], [tablePerson['deleted'].eq(0), tablePerson['id'].eq(personId)])
                specialityId = forceRef(record.value('speciality_id')) if record else None
                if specialityId:
                    queryTable = queryTable.innerJoin(tablePerson, tableEvent['execPerson_id'].eq(tablePerson['id']))
                    cond.append(tablePerson['speciality_id'].eq(specialityId))
                    cond.append(tablePerson['deleted'].eq(0))
        if eventSetDate:
            eventSetDateTime = db.formatDate(eventSetDate)
        else:
            eventSetDateTime = None
        if eventDate:
            eventDateTime = db.formatDate(eventDate)
        else:
            eventDateTime = None
        if eventSetDateTime and eventDateTime:
            cond.append(u''' (DATE(Event.execDate) IS NOT NULL
            AND ((DATE(Event.setDate) <= %s AND DATE(Event.execDate) >= %s) OR (DATE(Event.setDate) >= %s
            AND DATE(Event.execDate) <= %s) OR (DATE(Event.setDate) >= %s AND (DATE(Event.setDate) <= %s)
            OR (DATE(Event.setDate) <= %s AND (DATE(Event.execDate) <= %s AND DATE(Event.execDate) >= %s))))
            OR (DATE(Event.execDate) IS NULL AND (DATE(Event.setDate) = %s OR DATE(Event.setDate) = %s
            OR (%s <= DATE(Event.setDate) AND DATE(Event.setDate) <= %s))))
            ''' % (eventSetDateTime, eventDateTime, eventSetDateTime, eventDateTime,
                   eventSetDateTime, eventDateTime, eventSetDateTime, eventDateTime,
                   eventSetDateTime, eventSetDateTime, eventDateTime, eventSetDateTime, eventDateTime))
        elif eventSetDateTime:
            cond.append(u''' ((DATE(Event.execDate) IS NULL AND DATE(Event.setDate) = %s)
            OR (DATE(Event.execDate) IS NOT NULL AND (DATE(Event.setDate) <= %s AND DATE(Event.execDate) >= %s)))
            ''' % (eventSetDateTime, eventSetDateTime, eventSetDateTime))
        elif eventDateTime:
            cond.append(u''' ((DATE(Event.execDate) IS NULL AND DATE(Event.setDate) = %s)
            OR (DATE(Event.execDate) IS NOT NULL AND (DATE(Event.setDate) <= %s AND DATE(Event.execDate) >= %s)))
            ''' % (eventDateTime, eventDateTime, eventDateTime))
        recordList = db.getRecordList(queryTable, cols, cond, 'Event.setDate')
        if recordList:
            record = recordList[0]
            eventId = forceRef(record.value('id'))
            return eventId
    return None


def checkClientHasOpenEvents(widget, eventTypeId, clientId, eventSetDate, eventDate, personId, form):
    result = (3, None)
    if clientId and eventTypeId and (eventSetDate or eventDate):
        db = QtGui.qApp.db
        tableEvent  = db.table('Event')
        tableEventType = db.table('EventType')
        table = tableEvent.leftJoin(tableEventType, tableEventType['id'].eq(tableEvent['eventType_id']))
        cols = [tableEvent['id']]
        cond = [tableEvent['client_id'].eq(clientId),
                tableEvent['deleted'].eq(0),
                tableEventType['purpose_id'].gt(1),
                tableEvent['execDate'].isNull()
                ]
#        if eventSetDate and eventDate:
#            cond.append(tableEvent['setDate'].le(max(eventSetDate, eventDate)))
#        elif eventSetDate:
#            cond.append(tableEvent['setDate'].le(eventSetDate))
#        elif eventDate:
#            cond.append(tableEvent['setDate'].le(eventDate))
        recordList = db.getIdList(table, cols, cond, 'Event.setDate')
        if recordList:
            dialog = CCheckEnteredOpenEvents(widget, recordList, clientId, eventDate or eventSetDate)
            try:
                dialog.exec_()
                result = (dialog.btnResult, dialog.resultEventId)
            finally:
                dialog.destroy()
                sip.delete(dialog)
                del dialog
    return result


def checkClientPolicyAttendance(eventTypeId, clientId, date):
    if QtGui.qApp.isStrictPolicyCheckOnEventCreation() != 2:
        financeCode = getEventFinanceCode(eventTypeId)
        if financeCode == CFinanceType.CMI:
            return bool(getClientCompulsoryPolicy(clientId, date))
        elif financeCode == CFinanceType.VMI:
            return bool(getClientVoluntaryPolicy(clientId, date))
    return True


def confirmClientPolicyAttendance(widget, eventTypeId, clientId):
    if QtGui.qApp.isStrictPolicyCheckOnEventCreation() != 2:
        if QtGui.qApp.isStrictPolicyCheckOnEventCreation(): # строгий контроль
            message = u'Пациент не имеет полиса, требуемого для данного типа обращения.\nРегистрация обращения запрещена.'
            QtGui.QMessageBox.critical(widget, u'Внимание!', message, QtGui.QMessageBox.Ok, QtGui.QMessageBox.Ok)
            return False
        else:
            message = u'Пациент не имеет полиса, требуемого для данного типа обращения.\nЭто может привести к затруднениям оплаты обращения.\nВсё равно продолжить?'
            return QtGui.QMessageBox.critical(widget,
                                        u'Внимание!',
                                        message,
                                        QtGui.QMessageBox.Yes|QtGui.QMessageBox.No,
                                        QtGui.QMessageBox.No) == QtGui.QMessageBox.Yes
    return True


def confirmClientPolicyConstraint(widget, eventTypeId, clientId, eventSetDate, eventDate):
    if QtGui.qApp.isStrictPolicyCheckOnEventCreation() != 2:
        today = QDate.currentDate()
        fixedEventDate = eventDate or today
        fixedEventSetDate = eventSetDate or fixedEventDate
        policyDate = fixedEventSetDate
        serviceName = ''
        serviceStopFieldName = ''
        financeCode = getEventFinanceCode(eventTypeId)
        recordPolicy = None
        if financeCode == 2:
            recordPolicy = getClientCompulsoryPolicy(clientId, policyDate)
            serviceName = u'ОМС'
            serviceStopFieldName = 'compulsoryServiceStop' # wtf
        elif financeCode == 3:
            recordPolicy = getClientVoluntaryPolicy(clientId, policyDate)
            serviceName = u'ДМС'
            serviceStopFieldName = 'voluntaryServiceStop' # wtf
        serviceStopped = False
        if recordPolicy:
            policyBegDate = forceDate(recordPolicy.value('begDate'))
            policyEndDate = forceDate(recordPolicy.value('endDate'))
            validPolicy = not policyBegDate or policyBegDate <= fixedEventSetDate
            validPolicy = validPolicy and (not policyEndDate or policyEndDate >= fixedEventDate)
            if not validPolicy:
                if not confirmPolicyAvailable(widget, u'Полис пациента не действителен по дате!'):
                    return False
            serviceStopped = forceBool(recordPolicy.value(serviceStopFieldName)) # wtf
        if serviceStopped:
            if QtGui.qApp.checkGlobalPreference('4', u'да'): # строгий контроль
                message = u'По данной СМО приостановлено обслуживание %s полисов.\nРегистрация обращения запрещена.' % serviceName
                QtGui.QMessageBox.critical(widget, u'Внимание!', message, QtGui.QMessageBox.Ok, QtGui.QMessageBox.Ok)
                return False
            else:
                message = u'По данной СМО приостановлено обслуживание %s полисов.\nЭто может привести к затруднениям оплаты обращения.\nВсё равно продолжить?' % serviceName
                return QtGui.QMessageBox.critical(widget,
                                        u'Внимание!',
                                        message,
                                        QtGui.QMessageBox.Yes|QtGui.QMessageBox.No,
                                        QtGui.QMessageBox.No) == QtGui.QMessageBox.Yes
    return True


def confirmPolicyAvailable(widget, message):
    if QtGui.qApp.checkGlobalPreference('4', u'да'): # строгий контроль
        default = buttons = QtGui.QMessageBox.Ok
        messageBox = QtGui.QMessageBox.critical
    else:
        buttons = QtGui.QMessageBox.Yes | QtGui.QMessageBox.No
        default = QtGui.QMessageBox.Yes
        message = '\n'.join([message, u'Все равно продолжить?'])
        messageBox = QtGui.QMessageBox.question
    result = messageBox(widget,
                        u'Внимание!',
                        message,
                        buttons,
                        default) == QtGui.QMessageBox.Yes
    return result


def createEvent(widget, form, clientId, eventTypeId, orgId, personId, eventDate, eventSetDate, weekProfile, numDays, externalId, assistantId, curatorId, flagHospitalization = False, actionTypeId = None,
                valueProperties = None, tissueTypeId=None, selectPreviousActions=False, relegateOrgId = None, relegatePersonId = None, planningEventId = None, diagnos = None, financeId = None, protocolQuoteId = None,
                actionByNewEvent = [], order = 1, actionListToNewEvent = [], result = None, prevEventId = None, typeQueue = -1, docNum=None, relegateInfo = [], plannedEndDate = None, mapJournalInfoTransfer=[]):
    formClass = getEventFormClassByType(eventTypeId)
    dialog = formClass(widget)
    QtGui.qApp.setJTR(dialog) # fucked shit!
    try:
#        QtGui.qApp.setCounterController(CCounterController(dialog))
        if dialog.prepare(clientId, eventTypeId, orgId, personId, eventSetDate, eventDate, weekProfile, numDays, externalId, assistantId,
                          curatorId, flagHospitalization, actionTypeId, valueProperties, tissueTypeId, selectPreviousActions,
                          relegateOrgId, relegatePersonId, diagnos, financeId, protocolQuoteId, actionByNewEvent, order, actionListToNewEvent, typeQueue, docNum,
                          relegateInfo, plannedEndDate, mapJournalInfoTransfer):
            dialog.initPrevEventId(prevEventId)
            if not dialog.initPrevEventTypeId(eventTypeId, clientId):
                return None
            dialog.addActions(actionListToNewEvent)
            if result:
                dialog.setCmbResult(result)
            if hasattr(dialog, 'getEventDataPlanning'):
                dialog.getEventDataPlanning(planningEventId)
            if dialog.exec_():
                updateEventListAfterEdit(dialog.itemId())
                return dialog.itemId()
        return None
    finally:
        QtGui.qApp.unsetJTR(dialog) # fuck it again!
        QtGui.qApp.resetAllCounterValueIdReservation()
        dialog.destroy()
        sip.delete(dialog)
        del dialog


def editEvent(widget, eventId):
    formClass = getEventFormClass(eventId)
    dialog = formClass(widget)
    try:
        dialog.load(eventId)
        if dialog.exec_():
            updateEventListAfterEdit(eventId)
            return dialog.itemId()
        return None
    finally:
        dialog.destroy()
        sip.delete(dialog)
        del dialog


def updateEventListAfterEdit(eventId):
    if QtGui.qApp.mainWindow.registry:
        QtGui.qApp.mainWindow.registry.updateEventListAfterEdit(eventId)

