# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui

from Events.Utils      import getEventTypeForm, isEventDeath, isEventPeriodic, isEventUrgent
from F000.F000Dialog   import CF000Dialog
from F001.F001Dialog   import CF001Dialog
from F003.F003Dialog   import CF003Dialog
from F025.F025Dialog   import CF025Dialog
from F025.F025TrDialog import CF025TrDialog
from F030.F030Dialog   import CF030Dialog
from F043.F043Dialog   import CF043Dialog
from F106.F106Dialog   import CF106Dialog
from F131.F131Dialog   import CF131Dialog
from F110.F110Dialog   import CF110Dialog
from F027.F027Dialog   import CF027Dialog


_eventForms = [
   ('000',   u'ф. 000/у (платное обслуживание)',      CF000Dialog),
   ('001',   u'ф. 001/у (ввод услуг)',                CF001Dialog),
   ('003',   u'ф. 003/у (стационарное лечение)',      CF003Dialog),
   ('025',   u'ф. 025/у (стат.талон)',                CF025Dialog),
   ('030',   u'ф. 030/у (диспансерное наблюдение)',   CF030Dialog),
   ('043',   u'ф. 043/у (стоматология)',              CF043Dialog),
   ('106',   u'ф. 106/у (констатация смерти)',        CF106Dialog),
   ('131',   u'ф. 131/у (доп.диспансеризация)',       CF131Dialog),
   ('110',   u'ф. 110/у (скорая медицинская помощь)', CF110Dialog),
   ('027',   u'ф. 027/у (протокол)',                  CF027Dialog)
 ]

_mapEventFormCodeToFormClass = dict([(descr[0], descr[2]) for descr in _eventForms if descr[2]])

def getEventFormClassByType(eventTypeId):
    form = getEventTypeForm(eventTypeId)
    result = _mapEventFormCodeToFormClass.get(form, None)
    if result:
        return result
    if isEventDeath(eventTypeId):
        return CF106Dialog
    if isEventPeriodic(eventTypeId):
        return CF131Dialog
    if isEventUrgent(eventTypeId):
        return CF025TrDialog
    return CF025Dialog


def getEventFormClass(eventId):
    eventTypeId = QtGui.qApp.db.translate('Event', 'id', eventId, 'eventType_id')
    return getEventFormClassByType(eventTypeId)


def getEventFormList():
    return [descr[:2] for descr in _eventForms if descr[2]]
