# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui

from library.PrintInfo import CInfo, CTemplatableInfoMixin, CInfoList, CInfoProxyList, CDateInfo, CDateTimeInfo, CRBInfo, CTimeInfo
from library.Utils     import forceBool, forceDate, forceDateTime, forceDouble, forceInt, forceRef, forceString, forceStringEx, forceTime, formatSex, formatServiceArea, formatAttachOrg
from Events.ActionInfo      import CActionInfoList
from Accounting.AccountInfo      import CRBAccountFormatInfo
from Events.TempInvalidInfo import CTempInvalidInfoList
from Events.ContractTariffCache import CContractTariffCache
from Events.Service    import CServiceInfo
from Events.MKBInfo    import CMKBInfo
from Events.MesInfo    import CMesInfo
from Events.Utils      import recordAcceptable, getEventShowTime, getActionDispansPhase, CCSGInfo
from Orgs.PersonInfo   import CSpecialityInfo, CPersonInfo, CPersonInfoList
from Orgs.Utils        import COrgInfo, COrgStructureInfo
from Registry.Utils    import CClientInfo, CClientDocumentInfo, CQuotaTypeInfo, CRBPolicyTypeInfo, CAddressInfo
from Registry.ClientDocumentTracking import CDocumentLocationInfo
from RefBooks.RBMedicalAidKind import CMedicalAidKindInfo
from RefBooks.RBMedicalAidType import CMedicalAidTypeInfo

__all__ = ( 'CActionDispansPhaseInfo',
            'CBankInfo',
            'CCashOperationInfo',
            'CCharacterInfo',
            'CContractInfo',
            'CContractInfoList',
            'CCureMethodInfo',
            'CCureTypeInfo',
            'CDagnosisTypeInfo',
            'CDiagnosisInfo',
            'CDiagnosisInfoList',
            'CDiagnosticInfo',
            'CDiagnosticInfoList',
            'CDiagnosticInfoIdList',
            'CDiagnosticInfoProxyList',
            'CDiagnosticResultInfo',
            'CDispanserInfo',
            'CEmergencyAccidentInfo',
            'CEmergencyBrigadeInfo',
            'CEmergencyCauseCallInfo',
            'CEmergencyDeathInfo',
            'CEmergencyDiseasedInfo',
            'CEmergencyEbrietyInfo',
            'CEmergencyEventInfo',
            'CEmergencyEventInfoList',
            'CEmergencyMethodTransportInfo',
            'CEmergencyPlaceCallInfo',
            'CEmergencyPlaceReceptionCallInfo',
            'CEmergencyReasondDelaysInfo',
            'CEmergencyReceivedCallInfo',
            'CEmergencyResultInfo',
            'CEmergencyTransferTransportInfo',
            'CEmergencyTypeAssetInfo',
            'CEventInfo',
            'CEventInfoList',
            'CEventLocalContractInfo',
            'CEventPaymentInfo',
            'CEventPaymentInfoList',
            'CEventTypeInfo',
            'CFinanceInfo',
            'CHealthGroupInfo',
            'CHospitalInfo',
            'CLocEventInfoList',
            'CMesSpecificationInfo',
            'COrgAccountInfo',
            'CPatientModelInfo',
            'CPhasesInfo',
            'CPurposeInfo',
            'CResultInfo',
            'CSceneInfo',
            'CStageInfo',
            'CTraumaTypeInfo',
            'CVisitInfo',
            'CVisitInfoList',
            'CVisitInfoListEx',
            'CVisitInfoProxyList',
            'CVisitPersonallInfo',
            'CVisitTypeInfo',
            'CFeedInfo',
            'CFeedInfoList',
            'CMealTimeInfo',
            'CDietInfo',
          )


class CMealTimeInfo(CRBInfo):
    tableName = 'rbMealTime'

    def _initByRecord(self, record):
        self._begTime = CTimeInfo( forceTime(record.value('begTime')) )
        self._endTime = CTimeInfo( forceTime(record.value('endTime')) )

    begtime = property(lambda self: self.load()._begTime)
    endTime = property(lambda self: self.load()._endTime)


class CDietInfo(CRBInfo):
    tableName = 'rbDiet'

    def _initByRecord(self, record):
        self._begDate = CDateInfo( forceDate(record.value('begDate')) )
        self._endDate = CDateInfo( forceDate(record.value('endDate')) )

    begDate = property(lambda self: self.load()._begDate)
    endDate = property(lambda self: self.load()._endDate)


class CFeedInfo(CInfo):
    def __init__(self, context, id):
        CInfo.__init__(self, context)
        self.id = id


    def _load(self):
        db = QtGui.qApp.db
        table = db.table('Event_Feed')
        record = db.getRecordEx(table, '*', [table['id'].eq(self.id), table['deleted'].eq(0)])
        if record:
            self.initByRecord(record)
            return True
        else:
            self.initByRecord(db.dummyRecord())
            return False


    def initByRecord(self, record):
        self._date = CDateInfo(forceDate(record.value('date')))
        self._eventInfo = self.getInstance(CEventInfo, forceRef(record.value('event_id')))
        self._mealTime = self.getInstance(CMealTimeInfo, forceRef(record.value('mealTime_id')))
        self._diet = self.getInstance(CDietInfo, forceRef(record.value('diet_id')))
        self._patron = self.getInstance(CClientInfo, forceRef(record.value('patron_id')))
        self._typeFeed = forceInt(record.value('typeFeed'))
        self._finance = self.getInstance(CFinanceInfo, forceRef(record.value('finance_id')))
        self._featuresToEat = forceString(record.value('featuresToEat'))
        self._refusalToEat = forceBool(record.value('refusalToEat'))

    date          = property(lambda self: self.load()._date)
    eventInfo     = property(lambda self: self.load()._eventInfo)
    mealTime      = property(lambda self: self.load()._mealTime)
    diet          = property(lambda self: self.load()._diet)
    patron        = property(lambda self: self.load()._patron)
    typeFeed      = property(lambda self: self.load()._typeFeed)
    finance       = property(lambda self: self.load()._finance)
    featuresToEat = property(lambda self: self.load()._featuresToEat)
    refusalToEat  = property(lambda self: self.load()._refusalToEat)


class CFeedInfoList(CInfoList):
    def __init__(self, context, eventId):
        CInfoList.__init__(self, context)
        self.eventId = eventId

    def _load(self):
        db = QtGui.qApp.db
        table = db.table('Event_Feed')
        idList = db.getIdList(table, 'id', [table['event_id'].eq(self.eventId), table['deleted'].eq(0)], 'id')
        self._items = [ self.getInstance(CFeedInfo, id) for id in idList ]
        return True


# ###################################################################################################################

class CPurposeInfo(CRBInfo):
    tableName = 'rbEventTypePurpose'


    def _initByRecord(self, record):
        self._purpose = forceInt(record.value('purpose'))


    def _initByNull(self):
        self._purpose = 0


    purpose = property(lambda self: self.load()._purpose)


class CFinanceInfo(CRBInfo):
    tableName = 'rbFinance'


class CEventTypeInfo(CRBInfo):
    tableName = 'EventType'


    def _initByRecord(self, record):
        self._purpose = self.getInstance(CPurposeInfo, forceRef(record.value('purpose_id')))
        self._finance = self.getInstance(CFinanceInfo, forceRef(record.value('finance_id')))
        self._service = self.getInstance(CServiceInfo, forceRef(record.value('service_id')))
        self._printContext = forceString(record.value('context'))
        self._regionalCode = forceString(record.value('regionalCode'))
        self._medicalAidType = self.getInstance(CMedicalAidTypeInfo, forceRef(record.value('medicalAidType_id')))
        self._medicalAidKind = self.getInstance(CMedicalAidKindInfo, forceRef(record.value('medicalAidKind_id')))


    def _initByNull(self):
        self._purpose = self.getInstance(CPurposeInfo, None)
        self._finance = self.getInstance(CFinanceInfo, None)
        self._service = self.getInstance(CServiceInfo, None)
        self._printContext = ''
        self._regionalCode = ''
        self._medicalAidType = self.getInstance(CMedicalAidTypeInfo, None)
        self._medicalAidKind = self.getInstance(CMedicalAidKindInfo, None)

    purpose = property(lambda self: self.load()._purpose)
    finance = property(lambda self: self.load()._finance)
    service = property(lambda self: self.load()._service)
    medicalAidType = property(lambda self: self.load()._medicalAidType)
    medicalAidKind = property(lambda self: self.load()._medicalAidKind)
    printContext = property(lambda self: self.load()._printContext)
    regionalCode = property(lambda self: self.load()._regionalCode)


class CResultInfo(CRBInfo):
    tableName = 'rbResult'

    def _initByRecord(self, record):
        self._continued = forceBool(record.value('continued'))
        self._regionalCode = forceString(record.value('regionalCode'))
        self._federalCode = forceString(record.value('federalCode'))


    def _initByNull(self):
        self._continued = False
        self._regionalCode = self._federalCode = ''

    continued = property(lambda self: self.load()._continued)
    regionalCode = property(lambda self: self.load()._regionalCode)
    federalCode = property(lambda self: self.load()._federalCode)


class CDiagnosticResultInfo(CResultInfo):
    tableName = 'rbDiagnosticResult'


class CContractInfoList(CInfoList):
    def __init__(self, context, idList):
        CInfoList.__init__(self, context)
        self.idList = idList


    def _load(self):
        self._items = [ self.getInstance(CContractInfo, id) for id in self.idList ]
        return True


class CContractInfo(CInfo):
    def __init__(self, context, id):
        CInfo.__init__(self, context)
        self.id = id

    def _load(self):
        db = QtGui.qApp.db
        record = db.getRecord('Contract', '*', self.id)
        if record:
            self._number = forceString(record.value('number'))
            self._date = CDateInfo(forceDate(record.value('date')))
            self._begDate = CDateInfo(forceDate(record.value('begDate')))
            self._endDate = CDateInfo(forceDate(record.value('endDate')))
            self._recipient = self.getInstance(COrgInfo, forceRef(record.value('recipient_id')))
            self._recipientAccount = self.getInstance(COrgAccountInfo, forceRef(record.value('recipientAccount_id')))
            self._recipientKBK = forceString(record.value('recipientKBK'))
            self._payer = self.getInstance(COrgInfo, forceRef(record.value('payer_id')))
            self._payerAccount = self.getInstance(COrgAccountInfo, forceRef(record.value('payerAccount_id')))
            self._payerKBK = forceString(record.value('payerKBK'))
            self._finance = self.getInstance(CFinanceInfo, forceRef(record.value('finance_id')))
            self._note = forceString(record.value('note'))
            self._contingent = self.getInstance(CContingentInfoList, self.id)
            self._resolution = forceString(record.value('resolution'))
            self._format = self.getInstance(CRBAccountFormatInfo, forceRef(record.value('format_id')))
            return True
        else:
            self._number = ''
            self._date = CDateInfo()
            self._begDate = CDateInfo()
            self._endDate = CDateInfo()
            self._recipient = self.getInstance(COrgInfo, None)
            self._recipientAccount = self.getInstance(COrgAccountInfo, None)
            self._recipientKBK = ''
            self._payer = self.getInstance(COrgInfo, None)
            self._payerAccount = self.getInstance(COrgAccountInfo, None)
            self._payerKBK = ''
            self._finance = self.getInstance(CFinanceInfo, None)
            self._note = ''
            self._contingent = self.getInstance(CContingentInfoList, None)
            self._resolution  = ''
            self._format = self.getInstance(CRBAccountFormatInfo, None)
            return False


    def __str__(self):
        self.load()
        return self._number + ' ' + self._date

    number           = property(lambda self: self.load()._number)
    date             = property(lambda self: self.load()._date)
    begDate          = property(lambda self: self.load()._begDate)
    endDate          = property(lambda self: self.load()._endDate)
    recipient        = property(lambda self: self.load()._recipient)
    recipientAccount = property(lambda self: self.load()._recipientAccount)
    recipientKBK     = property(lambda self: self.load()._recipientKBK)
    payer            = property(lambda self: self.load()._payer)
    payerAccount     = property(lambda self: self.load()._payerAccount)
    payerKBK         = property(lambda self: self.load()._payerKBK)
    finance          = property(lambda self: self.load()._finance)
    note             = property(lambda self: self.load()._note)
    contingent   = property(lambda self: self.load()._contingent)
    resolution           = property(lambda self: self.load()._resolution)
    format = property(lambda self: self.load()._format)


class CContingentInfoList(CInfoList):
    def __init__(self, context, masterId):
        CInfoList.__init__(self, context)
        self.masterId = masterId


    def _load(self):
        db = QtGui.qApp.db
        table = db.table('Contract_Contingent')
        idList = db.getIdList(table, 'id', [table['master_id'].eq(self.masterId), table['deleted'].eq(0)], 'id')
        self._items = [ self.getInstance(CContingentInfo, id) for id in idList ]
        return True


class CContingentInfo(CInfo):
    def __init__(self, context, id):
        CInfo.__init__(self, context)
        self.id = id

    def _load(self):
        db = QtGui.qApp.db
        record = db.getRecord('Contract_Contingent', '*', self.id)
        if record:
            self._attachType = self.getInstance(CAttachTypeInfo, forceRef(record.value('attachType_id')))
            self._attachOrg = formatAttachOrg(forceInt(record.value('attachOrg')))
            self._org = self.getInstance(COrgInfo, forceRef(record.value('org_id')))
            self._socStatusType = self.getInstance(CSocStatusType, forceRef(record.value('attachType_id')))
            self._insurer = self.getInstance(COrgInfo, forceRef(record.value('insurer_id')))
            self._policyType = self.getInstance(CRBPolicyTypeInfo, forceRef(record.value('policyType_id')))
            self._serviceArea = formatServiceArea(forceInt(record.value('serviceArea')))
            self._sex = formatSex(forceInt(record.value('sex')))
            self._age = forceString(record.value('age'))
            return True
        else:
            self._attachType = self.getInstance(CAttachTypeInfo, None)
            self._attachOrg = 0
            self._org = self.getInstance(COrgInfo, None)
            self._socStatusType = self.getInstance(CSocStatusType, None)
            self._insurer = self.getInstance(COrgInfo, None)
            self._policyType = self.getInstance(CRBPolicyTypeInfo, None)
            self._serviceArea = 0
            self._sex = 0
            self._age = u''
            return False


    attachType           = property(lambda self: self.load()._attachType)
    attachOrg             = property(lambda self: self.load()._attachOrg)
    org          = property(lambda self: self.load()._org)
    socStatusType          = property(lambda self: self.load()._socStatusType)
    insurer        = property(lambda self: self.load()._insurer)
    policyType = property(lambda self: self.load()._policyType)
    serviceArea     = property(lambda self: self.load()._serviceArea)
    sex            = property(lambda self: self.load()._sex)
    age     = property(lambda self: self.load()._age)



class CAttachTypeInfo(CRBInfo):
    tableName = 'rbAttachType'


class CSocStatusType(CRBInfo):
    tableName = 'rbSocStatusType'


class COrgAccountInfo(CInfo):
    def __init__(self, context, id):
        CInfo.__init__(self, context)
        self.id = id

    def _load(self):
        db = QtGui.qApp.db
        record = db.getRecord('Organisation_Account', '*', self.id)
        if record:
            self._org = self.getInstance(COrgInfo, forceRef(record.value('organisation_id')))
            self._bankName = forceString(record.value('bankName'))
            self._name = forceString(record.value('name'))
            self._notes = forceString(record.value('notes'))
            self._bank = self.getInstance(CBankInfo, forceRef(record.value('bank_id')))
            self._cash = forceBool(record.value('cash'))
            return True
        else:
            self._org = self.getInstance(COrgInfo, None)
            self._bankName = ''
            self._name = ''
            self._notes = ''
            self._bank = self.getInstance(CBankInfo, None)
            self._cash = False
            return False


    def __str__(self):
        self.load()
        return self._name

    organisation = property(lambda self: self.load()._org)
    org          = property(lambda self: self.load()._org)
    bankName     = property(lambda self: self.load()._bankName)
    name         = property(lambda self: self.load()._name)
    notes        = property(lambda self: self.load()._notes)
    bank         = property(lambda self: self.load()._bank)
    cash         = property(lambda self: self.load()._cash)


class CBankInfo(CInfo):
    def __init__(self, context, id):
        CInfo.__init__(self, context)
        self.id = id

    def _load(self):
        db = QtGui.qApp.db
        record = db.getRecord('Bank', '*', self.id)
        if record:
            self._BIK = forceString(record.value('BIK'))
            self._name = forceString(record.value('name'))
            self._branchName = forceString(record.value('branchName'))
            self._corrAccount = forceString(record.value('corrAccount'))
            self._subAccount = forceString(record.value('subAccount'))
            return True
        else:
            self._BIK = ''
            self._name = ''
            self._branchName = ''
            self._corrAccount = ''
            self._subAccount = ''
            return False


    def __str__(self):
        self.load()
        return self._name

    BIK        = property(lambda self: self.load()._BIK)
    name       = property(lambda self: self.load()._name)
    branchName = property(lambda self: self.load()._branchName)
    corrAccount= property(lambda self: self.load()._corrAccount)
    subAccount = property(lambda self: self.load()._subAccount)


class CRBInfoWithRegionalCode(CRBInfo):
    def _initByRecord(self, record):
        self._regionalCode = forceString(record.value('regionalCode'))


    def _initByNull(self):
        self._regionalCode = ''


class CMesSpecificationInfo(CRBInfoWithRegionalCode):
    tableName = 'rbMesSpecification'


    def _initByRecord(self, record):
        CRBInfoWithRegionalCode._initByRecord(self, record)
        self._level = forceInt(record.value('level'))


    def _initByNull(self):
        CRBInfoWithRegionalCode._initByNull(self)
        self._level = 0


    level = property(lambda self: self.load()._level)
    done  = property(lambda self: self.load()._level == 2)


class CPatientModelInfo(CRBInfo):
    tableName = 'rbPatientModel'


    def _initByRecord(self, record):
        self._MKB = self.getInstance(CMKBInfo, forceString(record.value('MKB')))
        self._quotaType = self.getInstance(CQuotaTypeInfo, forceRef(record.value('quotaType_id')))


    def _initByNull(self):
        self._MKB = self.getInstance(CMKBInfo, None)
        self._quotaType = self.getInstance(CQuotaTypeInfo, None)


    MKB       = property(lambda self: self.load()._MKB)
    quotaType = property(lambda self: self.load()._quotaType)


class CCureTypeInfo(CRBInfoWithRegionalCode):
    tableName = 'rbCureType'



class CCureMethodInfo(CRBInfoWithRegionalCode):
    tableName = 'rbCureMethod'


#WTF?
class CActionDispansPhaseInfo(CInfoList):
    def __init__(self, context, id, phase=0):
        CInfoList.__init__(self, context)
        self.id = id
        self.phase = phase
        self.row2Name = {}
        self.mapNumMesVisitCode2Row = {}


    def getCodeRowToPhase(self):
        if self.phase == 1:
            self.mapNumMesVisitCode2Row = {
                        u'1': [1],
                        u'3': [2],
                        u'2': [3],
                        u'4': [4],
                        u'5': [5],
                        u'6': [6],
                        u'86':[7],
                        u'15':[8],
                        u'18':[9],
                        u'14':[10],
                        u'19':[11],
                        u'8': [12],
                        u'9': [13],
                        u'11':[14],
                        u'10':[15],
                        u'12':[16],
                        u'87':[17],
                        u'13':[18],
                        u'7': [19],
                        u'17':[20]
                        }
            self.row2Name = {
                        1: u'Опрос(анкетирование), направленный на выявление хронических неинфекционных заболеваний, факторов риска их развития, потребления наркотических средств и психотропных веществ без назначения врача',
                        2: u'Антропометрия (измерение роста стоя, массы тела, окружности талии), расчет индекса массы тела',
                        3: u'Измерение артериального давления',
                        4: u'Определение уровня общего холестерина в крови',
                        5: u'Определение уровня глюкозы в крови экспресс-методом',
                        6: u'Определение относительного суммарного сердечно-сосудистого риска',
                        7: u'Определение абсолютного суммарного сердечно-сосудистого риска',
                        8: u'Электрокардиография (в покое)',
                        9: u'Осмотр фельдшером (акушеркой), включая взятие мазка (соскоба) с поверхности шейки матки (наружного маточного зева) и цервикального канала на цитологическое исследование',
                        10: u'Флюорография легких',
                        11: u'Маммография обеих молочных желез',
                        12: u'Клинический анализ крови',
                        13: u'Клинический анализ крови развернутый',
                        14: u'Анализ крови биохимический общетерапевтический',
                        15: u'Общий анализ мочи',
                        16: u'Исследование кала на скрытую кровь иммунохимическим методом',
                        17: u'Ультразвуковое исследование (УЗИ) на предмет исключения новообразований органов брюшной полости, малого таза и аневризмы брюшной аорты',
                        18: u'Ультразвуковое исследование (УЗИ) в целях исключения аневризмы брюшной аорты',
                        19: u'Измерение внутриглазного давления',
                        20: u'Прием (осмотр) врача-терапевта'
                        }
        elif self.phase == 2:
            self.mapNumMesVisitCode2Row = {u'55' : [1],
                                           u'44' : [2],
                                           u'56' : [3],
                                           u'50' : [4],
                                           u'45' : [5],
                                           u'46' : [5],
                                           u'58' : [6],
                                           u'53' : [7],
                                           u'90' : [8],
                                           u'48' : [9],
                                           u'54' : [10],
                                           u'91' : [11],
                                           u'92' : [12],
                                           u'47' : [13],
                                           u'52' : [14],
                                           u'59' : [15],
                                           u'17' : [16]
                                          }
            self.row2Name = {
                             1 : u'Дуплексное сканирование брахицефальных артерий',
                             2 : u'Осмотр (консультация) врачом-неврологом',
                             3 : u'Эзофагогастродуоденоскопия ',
                             4 : u'Осмотр (консультация) врачом-хирургом или врачом-урологом',
                             5 : u'Осмотр (консультация) врачом-хирургом или врачом-колопроктологом',
                             6 : u'Колоноскопия или ректороманоскопия',
                             7 : u'Определение липидного спектра крови',
                             8 : u'Спирометрия',
                             9 : u'Осмотр (консультация) врачом-акушером-гинекологом',
                             10: u'Определение концентрации гликированного гемоглобина в крови или тест на толерантность к глюкозе',
                             11: u'Осмотр (консультация) врачом-оториноларингологом',
                             12: u'Анализ крови на уровень содержания простатспецифического антигена',
                             13: u'Осмотр (консультация) врачом-офтальмологом',
                             14: u'Индивидуальное углубленное профилактическое консультирование',
                             15: u'Групповое профилактическое консультирование (школа пациента)',
                             16: u'Прием (осмотр) врача-терапевта'
                            }


    def getDefault(self):
        result = {}
        for key in self.row2Name.keys():
            result[key] = [u'-', u'-', u'-', u'-']
        return result


    def getName(self, num):
        return [self.row2Name.get(num, ''), num]


    def getReportData(self, query):
        self.getCodeRowToPhase()
        reportData = self.getDefault()
        uniqueSet = set()
        db = QtGui.qApp.db
        while query.next():
            record = query.record()
            eventId = forceRef(record.value('eventId'))
            actionId = forceRef(record.value('actionId'))
#            actionTypeName = forceString(record.value('actionTypeName'))
            actionMkb = forceString(record.value('actionMkb'))
#            clientId = forceRef(record.value('clientId'))
            serviceName = forceString(record.value('serviceName'))
            numMesVisitCode = forceString(record.value('numMesVisitCode'))
            actionExecNow = forceInt(record.value('actionExecNow'))
            actionExecPrev = forceInt(record.value('actionExecPrev'))
            actionExecRefusal = forceInt(record.value('actionExecRefusal'))
            propertyEvaluation = forceInt(record.value('propertyEvaluation'))
            endDate = forceDate(record.value('endDate'))
            directionDate = forceDate(record.value('directionDate'))
            if numMesVisitCode not in self.mapNumMesVisitCode2Row:
                continue
            key = (eventId, numMesVisitCode, serviceName)
            if key in uniqueSet:
                continue
            uniqueSet.add(key)
            numCode = self.mapNumMesVisitCode2Row.get(numMesVisitCode, [])
            numCodeKey = numCode[0] if numCode else None
            if numCodeKey not in self.row2Name.keys():
                continue
            data = reportData.get(numCodeKey, [u'-', u'-', u'-', u'-'])
            if eventId and actionId and not actionExecRefusal:
                if actionExecNow:
                    data[0] = u'%s' % db.formatDate(endDate)
                    data[3] = u'%s' % db.formatDate(directionDate)
                elif actionExecPrev:
                    data[2] = u'проведено ранее(%s)' % db.formatDate(endDate)
            elif eventId and actionExecRefusal:
                data[2] = u'отказ(%s)' % db.formatDate(directionDate)
            if actionMkb and actionMkb[0].isalpha() and actionMkb[0].upper() != 'Z':
                data[1] = u'+'
            elif propertyEvaluation:
                data[1] = u'+'
            else:
                data[1] = u'-'
            reportData[numCodeKey] = data
        return reportData


    def _load(self):
        reportData = self.getReportData(getActionDispansPhase(self.id, self.phase) if self.id else None)
        self._items = []
        if reportData:
            for numMesVisitCode, reportLine in reportData.items():
                name = self.getName(numMesVisitCode)
                item = [name[0], name[1]]
                for row, val in enumerate(reportLine):
                    item.append(val)
                self._items.append(item)
            return True
        else:
            self._items = []
            return False


    def _initByRecord(self, record=None):
        pass


    def _initByNull(self):
        pass


    def __str__(self):
        return u'Тут идёт какая-то фигня'

    dispansItems = property(lambda self: self.load()._items)


class CEventInfo(CInfo, CTemplatableInfoMixin):
    def __init__(self, context, id):
        CInfo.__init__(self, context)
        self.id = id
        self._clientId = None
        self._localContract = None
        self._contractTariffCache = None
        self._tempInvalidListLoaded = False
        self._clientEvents = []
        self._isDirty = False

    def _load(self):
        db = QtGui.qApp.db
        record = db.getRecord('Event', '*', self.id)
        if record:
            self._clientId = forceRef(record.value('client_id'))

            self._pregnancyWeek = forceInt(record.value('pregnancyWeek'))
            self._eventType = self.getInstance(CEventTypeInfo, forceRef(record.value('eventType_id')))
            self._externalId = forceString(record.value('externalId'))
            self._org = self.getInstance(COrgInfo, forceRef(record.value('org_id')))
            self._relegateOrg = self.getInstance(COrgInfo, forceRef(record.value('relegateOrg_id')))
            self._relegatePerson = self.getInstance(CPersonInfo, forceRef(record.value('relegatePerson_id')))
            self._client = self.getInstance(CClientInfo, forceRef(record.value('client_id')))
            self._contract = self.getInstance(CContractInfo, forceRef(record.value('contract_id')))
            self._prevEventDate = CDateInfo(forceDate(record.value('prevEventDate')))
            self._setDate = CDateInfo(forceDate(record.value('setDate')))
            self._setTime = CTimeInfo(forceTime(record.value('setDate')))
            self._setPerson = self.getInstance(CPersonInfo, forceRef(record.value('setPerson_id')))
            self._execDate = CDateInfo(forceDate(record.value('execDate')))
            self._execTime = CTimeInfo(forceTime(record.value('execDate')))
            self._execPerson = self.getInstance(CPersonInfo, forceRef(record.value('execPerson_id')))
            self._isPrimary = forceInt(record.value('isPrimary')) == 1
            self._order = forceInt(record.value('order'))
            self._result = self.getInstance(CResultInfo, forceRef(record.value('result_id')))
            self._nextEventDate = CDateInfo(forceDate(record.value('nextEventDate')))
            self._payStatus = forceInt(record.value('payStatus'))
            self._typeAsset = self.getInstance(CEmergencyTypeAssetInfo, forceRef(record.value('typeAsset_id')))
            self._note = self._notes = forceString(record.value('note'))
            self._curator = self.getInstance(CPersonInfo, forceRef(record.value('curator_id')))
            self._assistant = self.getInstance(CPersonInfo, forceRef(record.value('assistant_id')))
            self._actions = self.getInstance(CActionInfoList, self.id)
            self._diagnosises = self.getInstance(CDiagnosticInfoList, self.id)
            self._feeds = self.getInstance(CFeedInfoList, self.id)
            self._visits = self.getInstance(CVisitInfoList, self.id)
            self._dispansIPhase = self.getInstance(CActionDispansPhaseInfo, self.id, 1)
            self._dispansIIPhase = self.getInstance(CActionDispansPhaseInfo, self.id, 2)
            self._localContract = self.getInstance(CEventLocalContractInfo, self.id)
            self._mes = self.getInstance(CMesInfo, forceRef(record.value('MES_id')))
            self._mesSpecification = self.getInstance(CMesSpecificationInfo, forceRef(record.value('mesSpecification_id')))
            self._patientModel = self.getInstance(CPatientModelInfo, forceRef(record.value('patientModel_id')))
            self._cureType     = self.getInstance(CCureTypeInfo, forceRef(record.value('cureType_id')))
            self._cureMethod   = self.getInstance(CCureMethodInfo, forceRef(record.value('cureMethod_id')))
            self._prevEvent = self.getInstance(CEventInfo, forceRef(record.value('prevEvent_id')))
            #self._tempInvalidList = self.getTempInvalidList(forceDate(record.value('setDate')), forceDate(record.value('execDate')), forceRef(record.value('client_id')))
            self._tempInvalidList = self.getTempInvalidList()
            self._relative = self.getInstance(CClientInfo, forceRef(record.value('relative_id')))
            self._clientEvents = self.getEventCount(self._clientId)
            self._createDatetime = CDateTimeInfo(forceDateTime(record.value('createDatetime')))
            self._modifyDatetime = CDateTimeInfo(forceDateTime(record.value('modifyDatetime')))
            self._modifyPerson = self.getInstance(CPersonInfo, forceRef(record.value('modifyPerson_id')))
            self._createPerson = self.getInstance(CPersonInfo, forceRef(record.value('createPerson_id')))
            self._csgList = self.getInstance(CCSGInfoList, self.id)
            self._srcDate = CDateInfo(forceDate(record.value('srcDate')))
            self._srcNumber = forceString(record.value('srcNumber'))
            self._documentLocation = self.getInstance(CDocumentLocationInfo, forceInt(record.value('client_id')), forceString(record.value('externalId')))
            return True
        else:
            self._clientId = None
            self._clientEvents = []
            self._pregnancyWeek = 0
            self._eventType = self.getInstance(CEventTypeInfo, None)
            self._externalId = ''
            self._org = self.getInstance(COrgInfo, None)
            self._relegateOrg = self.getInstance(COrgInfo, None)
            self._relegatePerson = self.getInstance(CPersonInfo, None)
            self._client = self.getInstance(CClientInfo, None)
            self._contract = self.getInstance(CContractInfo, None)
            self._prevEventDate = CDateInfo()
            self._setDate = CDateInfo()
            self._setTime = CTimeInfo()
            self._setPerson = self.getInstance(CPersonInfo, None)
            self._execDate = CDateInfo()
            self._execTime = CTimeInfo()
            self._execPerson = self.getInstance(CPersonInfo, None)
            self._isPrimary = False
            self._order = 0
            self._result = self.getInstance(CResultInfo, None)
            self._nextEventDate = CDateInfo()
            self._payStatus = 0
            self._typeAsset = self.getInstance(CEmergencyTypeAssetInfo, None)
            self._note = self._notes = ''
            self._curator = self.getInstance(CPersonInfo, None)
            self._assistant = self.getInstance(CPersonInfo, None)
            self._actions = self.getInstance(CActionInfoList, None)
            self._diagnosises = self.getInstance(CDiagnosticInfoList, None)
            self._feeds = self.getInstance(CFeedInfoList, None)
            self._visits = self.getInstance(CVisitInfoList, None)
            self._localContract = self.getInstance(CEventLocalContractInfo, None)
            self._mes = self.getInstance(CMesInfo, None)
            self._mesSpecification = self.getInstance(CMesSpecificationInfo, None)
            self._patientModel = self.getInstance(CPatientModelInfo, None)
            self._cureType     = self.getInstance(CCureTypeInfo, None)
            self._cureMethod   = self.getInstance(CCureMethodInfo, None)
            self._prevEvent = self.getInstance(CEventInfo, None)
            self._tempInvalidList = {}
            self._relative = self.getInstance(CClientInfo, None)
            self._createDatetime = CDateTimeInfo()
            self._modifyDatetime = CDateTimeInfo()
            self._modifyPerson = self.getInstance(CPersonInfo, None)
            self._createPerson = self.getInstance(CPersonInfo, None)
            self._csgList = self.getInstance(CCSGInfoList, None)
            self._srcDate = CDateInfo()
            self._srcNumber = ''
            self._documentLocation = self.getInstance(CDocumentLocationInfo, None, None)
            return False


    # tariff checker interface
    def getEventTypeId(self):
        return self.eventType.id

    def getEventCount(self, clientId):
        idList=[]
        cond = []
        db = QtGui.qApp.db
        queryTable = db.table('Event')
        tableEventType = db.table('EventType')
        queryTable = queryTable.leftJoin(tableEventType, queryTable['eventType_id'].eq(tableEventType['id']))
        cond.append(queryTable['client_id'].eq(clientId))
        records = db.getRecordList(queryTable, ['Event.id as eventId'], cond)
        if records:
            for record in records:
                idList.append(forceInt(record.value('eventId')))
        clientEvents = CEventInfoList(self.context, idList)
        clientEvents._load()
        return clientEvents

    def recordAcceptable(self, record):
        client = self.client
        if client:
            return recordAcceptable(client.sexCode, client.ageTuple, record)
        else:
            return True


    def getTariffDescrEx(self, contractId):
        if not self._contractTariffCache:
            self._contractTariffCache = CContractTariffCache()
        return self._contractTariffCache.getTariffDescr(contractId, self)


    def getTariffDescr(self):
        return self.getTariffDescrEx(self.contract.id)


    def getPrintTemplateContext(self):
        return self.eventType.printContext


    def getData(self):
        return { 'event' : self,
                 'client': self.client,
                 'tempInvalid': None
               }


#    def __unicode__(self):
    def __str__(self):
        self.load()
        return unicode(self._eventType)


    def getTempInvalidList(self, begDate=None, endDate=None, clientId = None, types=None, currentTempInvalid = None):
        if self._tempInvalidListLoaded:
            return self._tempInvalidList
        if endDate is None:
            endDate = self._execDate
        if isinstance(endDate, CDateInfo):
            endDate = endDate.date
        if begDate is None:
            #begDate = endDate.addMonths(-12)
            begDate = self._setDate
        if isinstance(begDate, CDateInfo):
            begDate = begDate.date
        if clientId is None:
            clientId = self._clientId
        result = CTempInvalidInfoList._get(self.context, clientId, begDate, endDate, types)
        if currentTempInvalid:
            if currentTempInvalid.id:
                for (i, tempInvalid) in enumerate(result):
                    if tempInvalid.id == currentTempInvalid.id:
                        result._items[i] = currentTempInvalid
            else:
                result = result + [currentTempInvalid, ]
        self._invalidListLoaded = True
        self._tempInvalidList = result
        return result

    pregnancyWeek = property(lambda self: self.load()._pregnancyWeek)
    eventType   = property(lambda self: self.load()._eventType)
    externalId  = property(lambda self: self.load()._externalId)
    org         = property(lambda self: self.load()._org)
    relegateOrg = property(lambda self: self.load()._relegateOrg)
    relegatePerson = property(lambda self: self.load()._relegatePerson)
    client      = property(lambda self: self.load()._client)
    contract    = property(lambda self: self.load()._contract)
    prevEventDate = property(lambda self: self.load()._prevEventDate)
    setDate     = property(lambda self: self.load()._setDate)
    setTime     = property(lambda self: self.load()._setTime)
    setPerson   = property(lambda self: self.load()._setPerson)
    execDate    = property(lambda self: self.load()._execDate)
    execTime    = property(lambda self: self.load()._execTime)
    execPerson  = property(lambda self: self.load()._execPerson)
    isPrimary   = property(lambda self: self.load()._isPrimary)
    order       = property(lambda self: self.load()._order)
    result      = property(lambda self: self.load()._result)
    nextEventDate = property(lambda self: self.load()._nextEventDate)
    payStatus   = property(lambda self: self.load()._payStatus)
    typeAsset   = property(lambda self: self.load()._typeAsset)
    note        = property(lambda self: self.load()._note)
    notes       = property(lambda self: self.load()._notes)
    curator     = property(lambda self: self.load()._curator)
    assistant   = property(lambda self: self.load()._assistant)
    finance     = property(lambda self: self.contract.finance)
    actions     = property(lambda self: self.load()._actions)
    diagnosises = property(lambda self: self.load()._diagnosises)
    feeds       = property(lambda self: self.load()._feeds)
    visits      = property(lambda self: self.load()._visits)
    localContract = property(lambda self: self.load()._localContract)
    mes         = property(lambda self: self.load()._mes)
    mesSpecification = property(lambda self: self.load()._mesSpecification)
    patientModel= property(lambda self: self.load()._patientModel)
    cureType    = property(lambda self: self.load()._cureType)
    cureMethod  = property(lambda self: self.load()._cureMethod)
    prevEvent   = property(lambda self: self.load()._prevEvent)
    tempInvalidList = property(lambda self: self.load()._tempInvalidList)
    relative      = property(lambda self: self.load()._relative)
    clientEvents      = property(lambda self: self.load()._clientEvents)
    createDatetime      = property(lambda self: self.load()._createDatetime)
    modifyDatetime      = property(lambda self: self.load()._modifyDatetime)
    modifyPerson   = property(lambda self: self.load()._modifyPerson)
    createPerson   = property(lambda self: self.load()._createPerson)
    csgList = property(lambda self: self.load()._csgList)
    srcDate = property(lambda self: self.load()._srcDate)
    srcNumber = property(lambda self: self.load()._srcNumber)
    isDirty = property(lambda self: self._isDirty)
    documentLocation = property(lambda self: self.load()._documentLocation)


class CEventInfoList(CInfoList):
    def __init__(self, context, accountIdList):
        CInfoList.__init__(self, context)
        self.idList = accountIdList


    def _load(self):
        self._items = [ self.getInstance(CEventInfo, id) for id in self.idList ]
        return True


class CLocEventInfoList(CInfoProxyList):
    def __init__(self, context, idList):
        CInfoProxyList.__init__(self, context)
        self.idList = idList
        self._items = [ None ]*len(self.idList)


    def __getitem__(self, key):
        v = self._items[key]
        if v is None:
            v = self.getInstance(CEventInfo, self.idList[key])
            self._items[key] = v
        return v


class CEmergencyEventInfo(CEventInfo):
    def __init__(self, context, id):
        CEventInfo.__init__(self, context, id)

    def _load(self):
        db = QtGui.qApp.db
        if CEventInfo._load(self):
            recordEmergency = db.getRecordEx('EmergencyCall', '*', 'event_id=%d' % self.id)
        else:
            recordEmergency = None
        if recordEmergency:
            self._numberCardCall = forceString(recordEmergency.value('numberCardCall'))
            self._brigade = self.getInstance(CEmergencyBrigadeInfo, forceRef(recordEmergency.value('brigade_id')))
            self._causeCall = self.getInstance(CEmergencyCauseCallInfo, forceRef(recordEmergency.value('causeCall_id')))
            self._whoCallOnPhone = forceString(recordEmergency.value('whoCallOnPhone'))
            self._numberPhone = forceString(recordEmergency.value('numberPhone'))
            self._storey = forceString(recordEmergency.value('storey'))
            self._entrance = forceString(recordEmergency.value('entrance'))
            self._additional = forceString(recordEmergency.value('additional'))
            self._guidePerson_id = self.getInstance(CPersonInfo, forceString(recordEmergency.value('guidePerson_id')))
            self._numberEpidemic = forceString(recordEmergency.value('numberEpidemic'))
            self._numberOrder = forceString(recordEmergency.value('order'))
            self._orgStructure = self.getInstance(COrgStructureInfo, forceRef(recordEmergency.value('orgStructure_id')))
            if getEventShowTime(self._eventType.id):
                self._begDate = CDateTimeInfo(forceDateTime(recordEmergency.value('begDate')))
                self._passDate = CDateTimeInfo(forceDateTime(recordEmergency.value('passDate')))
                self._departureDate = CDateTimeInfo(forceDateTime(recordEmergency.value('departureDate')))
                self._arrivalDate = CDateTimeInfo(forceDateTime(recordEmergency.value('arrivalDate')))
                self._finishServiceDate = CDateTimeInfo(forceDateTime(recordEmergency.value('finishServiceDate')))
                self._endDate = CDateTimeInfo(forceDateTime(recordEmergency.value('endDate')))
            else:
                self._begDate = CDateInfo(forceDate(recordEmergency.value('begDate')))
                self._passDate = CDateInfo(forceDate(recordEmergency.value('passDate')))
                self._departureDate = CDateInfo(forceDate(recordEmergency.value('departureDate')))
                self._arrivalDate = CDateInfo(forceDate(recordEmergency.value('arrivalDate')))
                self._finishServiceDate = CDateInfo(forceDate(recordEmergency.value('finishServiceDate')))
                self._endDate = CDateInfo(forceDate(recordEmergency.value('endDate')))

            self._placeReceptionCall = self.getInstance(CEmergencyPlaceReceptionCallInfo, forceRef(recordEmergency.value('placeReceptionCall_id')))
            self._receivedCall = self.getInstance(CEmergencyReceivedCallInfo, forceRef(recordEmergency.value('receivedCall_id')))
            self._reasondDelays = self.getInstance(CEmergencyReasondDelaysInfo, forceRef(recordEmergency.value('reasondDelays_id')))
            self._resultCall = self.getInstance(CEmergencyResultInfo, forceRef(recordEmergency.value('resultCall_id')))
            self._accident = self.getInstance(CEmergencyAccidentInfo, forceRef(recordEmergency.value('accident_id')))
            self._death = self.getInstance(CEmergencyDeathInfo, forceRef(recordEmergency.value('death_id')))
            self._ebriety = self.getInstance(CEmergencyEbrietyInfo, forceRef(recordEmergency.value('ebriety_id')))
            self._diseased = self.getInstance(CEmergencyDiseasedInfo, forceRef(recordEmergency.value('diseased_id')))
            self._placeCall = self.getInstance(CEmergencyPlaceCallInfo, forceRef(recordEmergency.value('placeCall_id')))
            self._methodTransport = self.getInstance(CEmergencyMethodTransportInfo, forceRef(recordEmergency.value('methodTransport_id')))
            self._transfTransport = self.getInstance(CEmergencyTransferTransportInfo, forceRef(recordEmergency.value('transfTransport_id')))
            self._renunOfHospital = forceInt(recordEmergency.value('renunOfHospital'))
            self._faceRenunOfHospital = forceString(recordEmergency.value('faceRenunOfHospital'))
            self._disease = forceInt(recordEmergency.value('disease'))
            self._birth = forceInt(recordEmergency.value('birth'))
            self._pregnancyFailure = forceInt(recordEmergency.value('pregnancyFailure'))
            self._noteCall = forceString(recordEmergency.value('noteCall'))
            self._address = self.getInstance(CAddressInfo, forceRef(recordEmergency.value('address_id')))
            return True
        else:
            self._numberCardCall = ''
            self._brigade = self.getInstance(CEmergencyBrigadeInfo, None)
            self._causeCall = self.getInstance(CEmergencyCauseCallInfo, None)
            self._whoCallOnPhone = ''
            self._numberPhone = ''
            self._storey = ''
            self._entrance = ''
            self._additional = ''
            self._guidePerson_id = None
            self._numberEpidemic = ''
            self._numberOrder = ''
            self._begDate = CDateTimeInfo()
            self._passDate = CDateTimeInfo()
            self._departureDate = CDateTimeInfo()
            self._arrivalDate = CDateTimeInfo()
            self._finishServiceDate = CDateTimeInfo()
            self._endDate = CDateTimeInfo()
            self._placeReceptionCall = self.getInstance(CEmergencyPlaceReceptionCallInfo, None)
            self._receivedCall = self.getInstance(CEmergencyReceivedCallInfo, None)
            self._reasondDelays = self.getInstance(CEmergencyReasondDelaysInfo, None)
            self._resultCall = self.getInstance(CEmergencyResultInfo, None)
            self._accident = self.getInstance(CEmergencyAccidentInfo, None)
            self._death = self.getInstance(CEmergencyDeathInfo, None)
            self._ebriety = self.getInstance(CEmergencyEbrietyInfo, None)
            self._diseased = self.getInstance(CEmergencyDiseasedInfo, None)
            self._placeCall = self.getInstance(CEmergencyPlaceCallInfo, None)
            self._methodTransport = self.getInstance(CEmergencyMethodTransportInfo, None)
            self._transfTransport = self.getInstance(CEmergencyTransferTransportInfo, None)
            self._orgStructure = self.getInstance(COrgStructureInfo, None)
            self._renunOfHospital = 0
            self._faceRenunOfHospital = ''
            self._disease = 0
            self._birth = 0
            self._pregnancyFailure = 0
            self._noteCall = ''
            self._address = None
            return False

    numberCardCall      = property(lambda self: self.load()._numberCardCall)
    brigade             = property(lambda self: self.load()._brigade)
    causeCall           = property(lambda self: self.load()._causeCall)
    whoCallOnPhone      = property(lambda self: self.load()._whoCallOnPhone)
    numberPhone         = property(lambda self: self.load()._numberPhone)
    storey              = property(lambda self: self.load()._storey)
    entrance            = property(lambda self: self.load()._entrance)
    additional          = property(lambda self: self.load()._additional)
    guidePerson_id      = property(lambda self: self.load()._guidePerson_id)
    numberEpidemic      = property(lambda self: self.load()._numberEpidemic)
    numberOrder      = property(lambda self: self.load()._numberOrder)
    begDate             = property(lambda self: self.load()._begDate)
    passDate            = property(lambda self: self.load()._passDate)
    departureDate       = property(lambda self: self.load()._departureDate)
    arrivalDate         = property(lambda self: self.load()._arrivalDate)
    finishServiceDate   = property(lambda self: self.load()._finishServiceDate)
    endDate             = property(lambda self: self.load()._endDate)
    placeReceptionCall  = property(lambda self: self.load()._placeReceptionCall)
    receivedCall        = property(lambda self: self.load()._receivedCall)
    reasondDelays       = property(lambda self: self.load()._reasondDelays)
    resultCall          = property(lambda self: self.load()._resultCall)
    accident            = property(lambda self: self.load()._accident)
    death               = property(lambda self: self.load()._death)
    ebriety             = property(lambda self: self.load()._ebriety)
    diseased            = property(lambda self: self.load()._diseased)
    placeCall           = property(lambda self: self.load()._placeCall)
    methodTransport     = property(lambda self: self.load()._methodTransport)
    transfTransport     = property(lambda self: self.load()._transfTransport)
    renunOfHospital     = property(lambda self: self.load()._renunOfHospital)
    faceRenunOfHospital = property(lambda self: self.load()._faceRenunOfHospital)
    disease             = property(lambda self: self.load()._disease)
    birth               = property(lambda self: self.load()._birth)
    pregnancyFailure    = property(lambda self: self.load()._pregnancyFailure)
    noteCall            = property(lambda self: self.load()._noteCall)
    orgStructure        = property(lambda self: self.load()._orgStructure)
    address             = property(lambda self: self.load()._address)


class CEmergencyEventInfoList(CInfoList):
    def __init__(self, context, eventIdList):
        CInfoList.__init__(self, context)
        self.idList = eventIdList


    def _load(self):
        self._items = [ self.getInstance(CEmergencyEventInfo, id) for id in self.idList ]
        return True


class CEventLocalContractInfo(CInfo):
    def __init__(self, context, eventId):
        CInfo.__init__(self, context)
        self.eventId = eventId
        self._payment = []

    def _load(self):
        db = QtGui.qApp.db
        table = db.table('Event_LocalContract')
        record = db.getRecordEx(table, '*', [table['master_id'].eq(self.eventId),
                                             table['deleted'].eq(0)
                                            ]) if self.eventId else None
        if record:
            self.initByRecord(record)
            return True
        else:
            self.initByRecord(db.dummyRecord())
            return False


    def initByRecord(self, record):
        self._coordDate = CDateInfo(forceDate(record.value('coordDate')))
        self._coordAgent = forceString(record.value('coordAgent'))
        self._coordInspector = forceString(record.value('coordInspector'))
        self._coordText = forceString(record.value('coordText'))
        self._date   = CDateInfo(forceDate(record.value('dateContract')))
        self._number = forceString(record.value('numberContract'))
        self._sumLimit = forceDouble(record.value('sumLimit'))
        self._org = self.getInstance(COrgInfo, forceRef(record.value('org_id')))
        self._lastName = forceString(record.value('lastName'))
        self._firstName = forceString(record.value('firstName'))
        self._patrName = forceString(record.value('patrName'))
        self._birthDate = CDateInfo(forceDate(record.value('birthDate')))
        self._document = CClientDocumentInfo(self.context)
        self._document._documentType = forceString(QtGui.qApp.db.translate('rbDocumentType', 'id', record.value('documentType_id'), 'name'))
        self._document._serial = forceStringEx(record.value('serialLeft'))+' '+forceStringEx(record.value('serialRight'))
        self._document._number = forceString(record.value('number'))
        self._document._origin = forceString(record.value('docOrigin'))
        self._document._date = CDateInfo(record.value('docDate'))
        if self._document._documentType:
            self._document._loaded = True
            self._document._ok = True
        self._address = forceString(record.value('regAddress'))
        self._email   = forceString(record.value('email'))

        self._customerOrg = self.getInstance(COrgInfo, forceRef(record.value('customerOrg_id')))
        self._customerLastName = forceString(record.value('customerLastName'))
        self._customerFirstName = forceString(record.value('customerFirstName'))
        self._customerPatrName = forceString(record.value('customerPatrName'))
        self._customerBirthDate = CDateInfo(forceDate(record.value('customerBirthDate')))
        self._customerDocument = CClientDocumentInfo(self.context)
        self._customerDocument._documentType = forceString(QtGui.qApp.db.translate('rbDocumentType', 'id', record.value('customerDocumentType_id'), 'name'))
        self._customerDocument._serial = forceStringEx(record.value('customerSerialLeft'))+' '+forceStringEx(record.value('customerSerialRight'))
        self._customerDocument._number = forceString(record.value('customerNumber'))
        self._customerDocument._origin = forceString(record.value('customerDocOrigin'))
        self._customerDocument._date = CDateInfo(record.value('customerDocDate'))
        if self._customerDocument._documentType:
            self._customerDocument._loaded = True
            self._customerDocument._ok = True
        self._customerAddress = forceString(record.value('customerRegAddress'))
        self._customerEmail   = forceString(record.value('customerEmail'))
        self._customerEnabled = forceBool(record.value('customerEnabled'))
        self._payment =  self.getInstance(CEventPaymentInfoList, record.value('master_id'))

    coordDate   = property(lambda self: self.load()._coordDate)
    coordAgent  = property(lambda self: self.load()._coordAgent)
    coordInspector = property(lambda self: self.load()._coordInspector)
    coordText   = property(lambda self: self.load()._coordText)
    date        = property(lambda self: self.load()._date)
    number      = property(lambda self: self.load()._number)
    sumLimit    = property(lambda self: self.load()._sumLimit)
    lastName    = property(lambda self: self.load()._lastName)
    firstName   = property(lambda self: self.load()._firstName)
    patrName    = property(lambda self: self.load()._patrName)
    birthDate   = property(lambda self: self.load()._birthDate)
    document    = property(lambda self: self.load()._document)
    address     = property(lambda self: self.load()._address)
    org         = property(lambda self: self.load()._org)
    email       = property(lambda self: self.load()._email)
    customerEnabled      = property(lambda self: self.load()._customerEnabled)
    customerLastName    = property(lambda self: self.load()._customerLastName)
    customerFirstName   = property(lambda self: self.load()._customerFirstName)
    customerPatrName    = property(lambda self: self.load()._customerPatrName)
    customerBirthDate   = property(lambda self: self.load()._customerBirthDate)
    customerDocument    = property(lambda self: self.load()._customerDocument)
    customerAddress     = property(lambda self: self.load()._customerAddress)
    customerOrg         = property(lambda self: self.load()._customerOrg)
    customerEmail       = property(lambda self: self.load()._customerEmail)
    payment         = property(lambda self: self.load()._payment)

    def __str__(self):
        if self.load():
            parts = []
            if self._coordDate:
                parts.append(u'согласовано ' + self._coordDate)
            if self._coordText:
                parts.append(self._coordText)
            if self._number:
                parts.append(u'№ ' + self._number)
            if self._date:
                parts.append(u'от ' + forceString(self._date))
            if self._org:
                parts.append(unicode(self._org))
            else:
                parts.append(self._lastName)
                parts.append(self._firstName)
                parts.append(self._patrName)
            return ' '.join(parts)
        else:
            return ''


class CEventPaymentInfoList(CInfoList):
    def __init__(self, context, masterId):
        CInfoList.__init__(self, context)
        self.masterId = masterId

    def _load(self):
        idList = self.getEventPaymentIds()
        self._items = [ self.getInstance(CEventPaymentInfo, id) for id in idList ]

    def getEventPaymentIds(self):
        masterId = self.masterId
        db = QtGui.qApp.db
        table = db.table('Event_Payment')
        cond  = [table['master_id'].eq(masterId)]
        cond.append(table['deleted'].eq(0))
        return db.getIdList(table, idCol='id', where=cond)


class CEventPaymentInfo(CInfo):
    def __init__(self, context, paymentId):
        CInfo.__init__(self, context)
        self.paymentId = paymentId
        self._date = None
        self._cashOperation = None
        self._sum = None
        self._typePayment = None
        self._settlementAccount = None
        self._bank = None
        self._numberCreditCard = None
        self._cashBox = None


    def _load(self):
        db = QtGui.qApp.db
        table = db.table('Event_Payment')
        record = db.getRecord(table, '*', self.paymentId)
        if record:
            from Accounting.CashDialog import CCashOperationInfo
            cashOperationId = forceRef(record.value('cashOperation_id'))
            self._date = forceString(record.value('date'))
            self._cashOperation = self.getInstance(CCashOperationInfo, cashOperationId) if cashOperationId else None
            self._sum = forceDouble(record.value('sum'))
            self._typePayment = forceInt(record.value('typePayment'))
            self._settlementAccount = forceString(record.value('settlementAccount'))
            bankId = forceRef(record.value('bank_id'))
            self._bank = self.getInstance(COrgInfo, bankId) if bankId else None
            self._numberCreditCard = forceString(record.value('numberCreditCard'))
            self._cashBox = forceString(record.value('cashBox'))
            return True
        return False

    date = property(lambda self: self.load()._date)
    cashOperation = property(lambda self: self.load()._cashOperation)
    sum = property(lambda self: self.load()._sum)
    typePayment = property(lambda self: self.load()._typePayment)
    settlementAccount = property(lambda self: self.load()._settlementAccount)
    bank = property(lambda self: self.load()._bank)
    numberCreditCard = property(lambda self: self.load()._numberCreditCard)
    cashBox = property(lambda self: self.load()._cashBox)


class CDagnosisTypeInfo(CRBInfo):
    tableName = 'rbDiagnosisType'


class CCharacterInfo(CRBInfo):
    tableName = 'rbDiseaseCharacter'


class CPhasesInfo(CRBInfo):
    tableName = 'rbDiseasePhases'


class CStageInfo(CRBInfo):
    tableName = 'rbDiseaseStage'


class CDispanserInfo(CRBInfo):
    tableName = 'rbDispanser'

    def _initByRecord(self, record):
        self._observed = forceBool(record.value('observed'))


    def _initByNull(self):
        self._observed = False

    observed = property(lambda self: self.load()._observed)


class CHospitalInfo(CInfo):
    names = [u'не требуется', u'требуется', u'направлен', u'пролечен']

    def __init__(self, context, code):
        CInfo.__init__(self, context)
        self.code = code
        self.name = self.names[code] if 0<=code<len(self.names) else ('{%s}' % code)
        self._ok = True
        self._loaded = True

    def __str__(self):
        return self.name


class CTraumaTypeInfo(CRBInfo):
    tableName = 'rbTraumaType'


class CHealthGroupInfo(CRBInfo):
    tableName = 'rbHealthGroup'


class CDiagnosticInfo(CInfo):
    def __init__(self, context, id):
        CInfo.__init__(self, context)
        self.id = id

    def _load(self):
        db = QtGui.qApp.db
        tableDiagnostic = db.table('Diagnostic')
        tableDiagnosis = db.table('Diagnosis')
        record = db.getRecord(tableDiagnostic.leftJoin(tableDiagnosis, tableDiagnosis['id'].eq(tableDiagnostic['diagnosis_id'])),
                              'Diagnostic.*, Diagnosis.MKB, Diagnosis.MKBEx, Diagnosis.morphologyMKB, Diagnosis.mod_id',
                              self.id)
        if record:
            self.initByRecord(record)
            return True
        else:
            self.initByRecord(db.dummyRecord())
            return False


    def initByRecord(self, record):
        self._type = self.getInstance(CDagnosisTypeInfo, forceRef(record.value('diagnosisType_id')))
        self._MKB = self.getInstance(CMKBInfo, forceString(record.value('MKB')))
        self._MKBEx = self.getInstance(CMKBInfo, forceString(record.value('MKBEx')))
        self._morphologyMKB = forceString(record.value('morphologyMKB'))
        self._character = self.getInstance(CCharacterInfo, forceRef(record.value('character_id')))
        self._stage = self.getInstance(CStageInfo, forceRef(record.value('stage_id')))
        self._phase = self.getInstance(CPhasesInfo, forceRef(record.value('phase_id')))
        self._dispanser = self.getInstance(CDispanserInfo, forceRef(record.value('dispanser_id')))
        self._sanatorium = self.getInstance(CHospitalInfo, forceInt(record.value('sanatorium')))
        self._hospital = self.getInstance(CHospitalInfo, forceInt(record.value('hospital')))
        self._traumaType = self.getInstance(CTraumaTypeInfo, forceRef(record.value('traumaType_id')))
        self._speciality = self.getInstance(CSpecialityInfo, forceRef(record.value('speciality_id')))
        self._person = self.getInstance(CPersonInfo, forceRef(record.value('person_id')))
        self._healthGroup = self.getInstance(CHealthGroupInfo, forceRef(record.value('healthGroup_id')))
        self._result = self.getInstance(CDiagnosticResultInfo, forceRef(record.value('result_id')))
        self._setDate = CDateInfo(forceDate(record.value('setDate')))
        self._endDate = CDateInfo(forceDate(record.value('endDate')))
        self._notes = forceString(record.value('notes'))
        self._mod = self.getInstance(CDiagnosisInfo, forceRef(record.value('mod_id')))
        self._diagnosis = self.getInstance(CDiagnosisInfo, forceRef(record.value('diagnosis_id')))
        self._freeInput = forceString(record.value('freeInput'))
        self._eventInfo = self.getInstance(CEventInfo, forceRef(record.value('event_id')))

    type          = property(lambda self: self.load()._type)
    MKB           = property(lambda self: self.load()._MKB)
    MKBEx         = property(lambda self: self.load()._MKBEx)
    morphologyMKB = property(lambda self: self.load()._morphologyMKB)
    character     = property(lambda self: self.load()._character)
    stage         = property(lambda self: self.load()._stage)
    phase         = property(lambda self: self.load()._phase)
    dispanser     = property(lambda self: self.load()._dispanser)
    sanatorium    = property(lambda self: self.load()._sanatorium)
    hospital      = property(lambda self: self.load()._hospital)
    traumaType    = property(lambda self: self.load()._traumaType)
    speciality    = property(lambda self: self.load()._speciality)
    person        = property(lambda self: self.load()._person)
    healthGroup   = property(lambda self: self.load()._healthGroup)
    result        = property(lambda self: self.load()._result)
    setDate       = property(lambda self: self.load()._setDate)
    endDate       = property(lambda self: self.load()._endDate)
    notes         = property(lambda self: self.load()._notes)
    mod           = property(lambda self: self.load()._mod)
    diagnosis     = property(lambda self: self.load()._diagnosis)
    freeInput     = property(lambda self: self.load()._freeInput)
    eventInfo     = property(lambda self: self.load()._eventInfo)


class CDiagnosticInfoList(CInfoList):
    def __init__(self, context, eventId):
        CInfoList.__init__(self, context)
        self.eventId = eventId


    def _load(self):
        db = QtGui.qApp.db
        table = db.table('Diagnostic')
        idList = db.getIdList(table, 'id', [table['event_id'].eq(self.eventId), table['deleted'].eq(0)], 'id')
        self._items = [ self.getInstance(CDiagnosticInfo, id) for id in idList ]
        return True


class CDiagnosticInfoIdList(CInfoList):
    def __init__(self, context, idList):
        CInfoList.__init__(self, context)
        self.idList = idList


    def _load(self):
        self._items = [ self.getInstance(CDiagnosticInfo, id) for id in self.idList ]
        return True


class CDiagnosisInfo(CInfo):
    def __init__(self, context, id):
        CInfo.__init__(self, context)
        self.id = id

    def _load(self):
        db = QtGui.qApp.db
        table = db.table('Diagnosis')
        record = db.getRecord(table, '*', self.id)
        if record:
            self.initByRecord(record)
            return True
        else:
            self.initByRecord(db.dummyRecord())
            return False


    def initByRecord(self, record):
        self._type = self.getInstance(CDagnosisTypeInfo, forceRef(record.value('diagnosisType_id')))
        self._MKB = self.getInstance(CMKBInfo, forceString(record.value('MKB')))
        self._MKBEx = self.getInstance(CMKBInfo, forceString(record.value('MKBEx')))
        self._morphologyMKB = forceString(record.value('morphologyMKB'))
        self._character = self.getInstance(CCharacterInfo, forceRef(record.value('character_id')))
        self._dispanser = self.getInstance(CDispanserInfo, forceRef(record.value('dispanser_id')))
        self._traumaType = self.getInstance(CTraumaTypeInfo, forceRef(record.value('traumaType_id')))
        self._person = self.getInstance(CPersonInfo, forceRef(record.value('person_id')))
        self._setDate = CDateInfo(forceDate(record.value('setDate')))
        self._endDate = CDateInfo(forceDate(record.value('endDate')))
        self._mod = self.getInstance(CDiagnosisInfo, forceRef(record.value('mod_id')))
        self._client = self.getInstance(CClientInfo, forceRef(record.value('client_id')))

    type          = property(lambda self: self.load()._type)
    MKB           = property(lambda self: self.load()._MKB)
    MKBEx         = property(lambda self: self.load()._MKBEx)
    morphologyMKB = property(lambda self: self.load()._morphologyMKB)
    character     = property(lambda self: self.load()._character)
    dispanser     = property(lambda self: self.load()._dispanser)
    traumaType    = property(lambda self: self.load()._traumaType)
    person        = property(lambda self: self.load()._person)
#    healthGroup = property(lambda self: self.load()._healthGroup)
    setDate       = property(lambda self: self.load()._setDate)
    endDate       = property(lambda self: self.load()._endDate)
    mod           = property(lambda self: self.load()._mod)
    client        = property(lambda self: self.load()._client)


class CDiagnosisInfoList(CInfoList):
    def __init__(self, context, clientId):
        CInfoList.__init__(self, context)
        self.clientId = clientId

    def _load(self):
        db = QtGui.qApp.db
        table = db.table('Diagnosis')
        idList = db.getIdList(table,
                              'id',
                              [table['client_id'].eq(self.clientId), table['deleted'].eq(0), table['mod_id'].isNull()],
                              'endDate')
        self._items = [ self.getInstance(CDiagnosisInfo, id) for id in idList ]
        return True


class CSceneInfo(CRBInfo):
    tableName = 'rbScene'


class CVisitTypeInfo(CRBInfo):
    tableName = 'rbVisitType'


class CVisitInfo(CInfo):
    def __init__(self, context, id):
        CInfo.__init__(self, context)
        self.id = id


    def _load(self):
        db = QtGui.qApp.db
        table = db.table('Visit')
        record = db.getRecordEx(table, '*', [table['id'].eq(self.id), table['deleted'].eq(0)])
        if record:
            self.initByRecord(record)
            return True
        else:
            self.initByRecord(db.dummyRecord())
            return False


    def initByRecord(self, record):
        self._scene = self.getInstance(CSceneInfo, forceRef(record.value('scene_id')))
        self._date = CDateInfo(forceDate(record.value('date')))
        self._type = self.getInstance(CVisitTypeInfo, forceRef(record.value('visitType_id')))
        self._eventInfo = self.getInstance(CEventInfo, forceRef(record.value('event_id')))
        self._person = self.getInstance(CPersonInfo, forceRef(record.value('person_id')))
        self._assistant = self.getInstance(CPersonInfo, forceRef(record.value('assistant_id')))
        self._isPrimary = forceBool(record.value('isPrimary'))
        self._finance = self.getInstance(CFinanceInfo, forceRef(record.value('finance_id')))
        self._service = self.getInstance(CServiceInfo, forceRef(record.value('service_id')))
        self._payStatus = forceInt(record.value('payStatus'))

    scene       = property(lambda self: self.load()._scene)
    date        = property(lambda self: self.load()._date)
    type        = property(lambda self: self.load()._type)
    eventInfo   = property(lambda self: self.load()._eventInfo)
    person      = property(lambda self: self.load()._person)
    assistant   = property(lambda self: self.load()._assistant)
    isPrimary   = property(lambda self: self.load()._isPrimary)
    finance     = property(lambda self: self.load()._finance)
    service     = property(lambda self: self.load()._service)
    payStatus   = property(lambda self: self.load()._payStatus)


class CVisitInfoList(CInfoList):
    def __init__(self, context, eventId):
        CInfoList.__init__(self, context)
        self.eventId = eventId

    def _load(self):
        db = QtGui.qApp.db
        table = db.table('Visit')
        idList = db.getIdList(table, 'id', [table['event_id'].eq(self.eventId), table['deleted'].eq(0)], 'id')
        self._items = [ self.getInstance(CVisitInfo, id) for id in idList ]
        return True


class CVisitInfoListEx(CInfoList):
    def __init__(self, context, visitIdList):
        CInfoList.__init__(self, context)
        self.idList = visitIdList


    def _load(self):
        self._items = [ self.getInstance(CVisitInfo, id) for id in self.idList ]
        return True


class CDiagnosticInfoProxyList(CInfoProxyList):
    def __init__(self, context, models):
        CInfoProxyList.__init__(self, context)
        self._rawItems = []
        for model in models:
            self._rawItems.extend(model.items())
        self._items = [ None ] * len(self._rawItems)


    def __getitem__(self, key):
        v = self._items[key]
        if v is None:
            record = self._rawItems[key]
            v = self.getInstance(CDiagnosticInfo, 'tmp_%d'%key)
            v.initByRecord(record)
            v.setOkLoaded()
            self._items[key] = v
        return v


class CVisitInfoProxyList(CInfoProxyList):
    def __init__(self, context, modelVisits):
        CInfoProxyList.__init__(self, context)
        self._items = [ None ] * len(modelVisits.items())
        self.model = modelVisits


    def _getItemEx(self, key):
        record = self.model.items()[key]
        v = self.getInstance(CVisitInfo, 'tmp_%d'%key)
        v.initByRecord(record)
        v.setOkLoaded()
        return v

    def __getitem__(self, key):
        if isinstance(key, slice):
            for i in range(key.start or 0, key.stop or len(self._items), key.step or 1):
                val = self._items[i]
                if val is None:
                    self._items[i] = self._getItemEx(i)
        v = self._items[key]
        if v is None:
            v = self._getItemEx(key)
            self._items[key] = v
        return v


class CVisitPersonallInfo(CInfo):
    def __init__(self, context, item = []):
        CInfo.__init__(self, context)
        self.item = item


    def _load(self):
        if self.item != []:
            self._scene = self.getInstance(CSceneInfo, self.item[0])
            self._date = CDateTimeInfo(self.item[1])
            self._type = self.getInstance(CVisitTypeInfo, self.item[2])
            self._person = self.getInstance(CPersonInfo, self.item[3])
            self._isPrimary = forceBool(self.item[4])
            self._finance = self.getInstance(CFinanceInfo, self.item[5])
            self._service = self.getInstance(CServiceInfo, self.item[6])
            self._payStatus = self.item[7]

    scene       = property(lambda self: self.load()._scene)
    date        = property(lambda self: self.load()._date)
    type        = property(lambda self: self.load()._type)
    person      = property(lambda self: self.load()._person)
    isPrimary   = property(lambda self: self.load()._isPrimary)
    finance     = property(lambda self: self.load()._finance)
    service     = property(lambda self: self.load()._service)
    payStatus   = property(lambda self: self.load()._payStatus)


class CEmergencyBrigadeInfo(CInfo):
    def __init__(self, context, brigadeId):
        CInfo.__init__(self, context)
        self.brigadeId = brigadeId
        self._code = ''
        self._name = ''
        self._regionalCode = ''
        self._persons = context.getInstance(CPersonInfoList, 'EmergencyBrigade_Personnel', None)


    def _load(self):
        db = QtGui.qApp.db
        record = db.getRecord('EmergencyBrigade', '*', self.brigadeId)
        if record:
            self._code = forceString(record.value('code'))
            self._name = forceString(record.value('name'))
            self._regionalCode = forceString(record.value('regionalCode'))
            self._persons = self.getInstance(CPersonInfoList, 'EmergencyBrigade_Personnel', self.brigadeId)
            return True
        else:
            return False


    def __str__(self):
        return self.load()._name

    code     = property(lambda self: self.load()._code)
    name     = property(lambda self: self.load()._name)
    regionalCode = property(lambda self: self.load()._regionalCode)
    persons  = property(lambda self: self.load()._persons)


class CEmergencyCauseCallInfo(CRBInfoWithRegionalCode):
    tableName = 'rbEmergencyCauseCall'


class CEmergencyTransferTransportInfo(CRBInfoWithRegionalCode):
    tableName = 'rbEmergencyTransferredTransportation'


class CEmergencyPlaceReceptionCallInfo(CRBInfoWithRegionalCode):
    tableName = 'rbEmergencyPlaceReceptionCall'


class CEmergencyReceivedCallInfo(CRBInfoWithRegionalCode):
    tableName = 'rbEmergencyReceivedCall'


class CEmergencyReasondDelaysInfo(CRBInfoWithRegionalCode):
    tableName = 'rbEmergencyReasondDelays'


class CEmergencyResultInfo(CRBInfoWithRegionalCode):
    tableName = 'rbEmergencyResult'


class CEmergencyAccidentInfo(CRBInfoWithRegionalCode):
    tableName = 'rbEmergencyAccident'


class CEmergencyDeathInfo(CRBInfoWithRegionalCode):
    tableName = 'rbEmergencyDeath'


class CEmergencyEbrietyInfo(CRBInfoWithRegionalCode):
    tableName = 'rbEmergencyEbriety'


class CEmergencyDiseasedInfo(CRBInfoWithRegionalCode):
    tableName = 'rbEmergencyDiseased'


class CEmergencyPlaceCallInfo(CRBInfoWithRegionalCode):
    tableName = 'rbEmergencyPlaceCall'


class CEmergencyMethodTransportInfo(CRBInfoWithRegionalCode):
    tableName = 'rbEmergencyMethodTransportation'


class CEmergencyTypeAssetInfo(CRBInfoWithRegionalCode):
    tableName = 'rbEmergencyTypeAsset'


class CCashOperationInfo(CRBInfo):
    tableName = 'rbCashOperation'


class CCSGInfoList(CInfoList):
    def __init__(self, context, eventId):
        CInfoList.__init__(self, context)
        self.eventId = eventId

    def _load(self):
        db = QtGui.qApp.db
        table = db.table('Event_CSG')
        idList = db.getIdList(table, 'id', table['master_id'].eq(self.eventId), 'id')
        self._items = [ self.getInstance(CCSGInfo, id) for id in idList ]
        return True

