# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui, QtSql
from PyQt4.QtCore import Qt, QChar, QDate, QDateTime, QString, pyqtSignature, QModelIndex, SIGNAL, QVariant, pyqtSignal


from library.DialogBase          import CConstructHelperMixin
from library.interchange         import getRBComboBoxValue, setRBComboBoxValue
from library.ICDInDocTableCol    import CICDExInDocTableCol
from library.CSG.CSGInDocTableCol import CCSGInDocTableCol
from library.CSG.CSGComboBox import defaultFilters
from library.InDocTable          import CInDocTableModel, CIntInDocTableCol, CDateInDocTableCol, CRBInDocTableCol
from library.crbcombobox         import CRBComboBox

from library.Utils               import forceBool, forceInt, forceRef, forceString, forceDate, toVariant, forceDateTime

from Events.Utils                import getEvenMesServiceMask, getEventMesSpecificationId, getEventMesCodeMask, getEventMesNameMask, getEventProfileId, getEventCSGRequired, getEventMesRequired, getEventCSGCodeMask, getEventSubCSGCodeMask
from Reports.CheckMesDescription import showCheckMesDescription
from Reports.MesDescription      import showMesDescription


from Events.Ui_EventMesPage             import Ui_EventMesPageWidget
from Events.Ui_CheckMesParametersDialog import Ui_CheckMesParametersDialog


class CEventMesPage(QtGui.QWidget, CConstructHelperMixin, Ui_EventMesPageWidget):
    csgRowRemoved = pyqtSignal()
    csgRowAboutToBeRemoved = pyqtSignal(QtSql.QSqlRecord)
    def __init__(self, parent=None):
        QtGui.QWidget.__init__(self, parent)
        self.eventEditor = None

        self.addModels('CSGs', CCSGModel(self))
        self.addModels('CSGSubItems', CCSGSlaveModel(self))
        self.setupBtnCheckMes()

        self.setupUi(self)

        self.setFocusProxy(self.cmbMes)
        self.cmbMesSpecification.setTable('rbMesSpecification')
        self.setModels(self.tblCSGs, self.modelCSGs, self.selectionModelCSGs)
        self.setModels(self.tblCSGSubItems, self.modelCSGSubItems, self.selectionModelCSGSubItems)
        self.btnCheckMes.setMenu(self.mnuBtnCheckMes)
        self.eventId = None
        self.eventTypeId = None
        self.mesWidgets = [self.lblMes, self.cmbMes, self.lblMesSpecification, self.cmbMesSpecification, self.btnCheckMes, self.btnShowMes]
        self.csgWidgets = [self.grpCSG]
        self.tblCSGs.addPopupDelRow()
        self.tblCSGs.addPopupDuplicateCurrentRow()
        self.tblCSGSubItems.addPopupDelRow()
        self.tblCSGSubItems.addPopupDuplicateCurrentRow()
        self.tblCSGs.setDelRowsIsExposed(lambda rowsExp: not any(map(self.isExposed, rowsExp)))
        self.tblCSGSubItems.setDelRowsIsExposed(lambda rowsExp: not any(map(self.isExposed_sub, rowsExp)))
        self.addObject('actCreateSomeCSGformepls', QtGui.QAction(u'Подобрать КСГ', self))
        self.tblCSGs.addPopupAction(self.actCreateSomeCSGformepls)
        self.connect(self.actCreateSomeCSGformepls, SIGNAL('triggered()'), self.on_actCreateSomeCSGformepls)
        self.actCreateSomeCSGformepls.setEnabled(False)
        self.mesRequired = False
        for w in self.mesWidgets:
            w.setEnabled(False)
        for w in self.csgWidgets:
            w.setEnabled(False)


    def isExposed(self, row):
        items = self.modelCSGs.items()
        if 0 <= row < len(items):
            item = items[row]
            return True if forceInt(item.value('payStatus')) != 0 else False
        return True

    def isExposed_sub(self, row):
        items = self.modelCSGs.items()
        if 0 <= row < len(items):
            item = items[row]
            return True if forceInt(item.value('payStatus')) != 0 else False
        return True

    def protectFromEdit(self, isProtected):
        if self.mesRequired:
            editWidgets = [self.cmbMes, self.cmbMesSpecification]
            for widget in editWidgets:
                widget.setEnabled(not isProtected)


    def setupBtnCheckMes(self):
        self.addObject('mnuBtnCheckMes', QtGui.QMenu(self))
        self.addObject('actDecarationColor', QtGui.QAction(u'Оформление в цвете', self))
        self.addObject('actDecarationNoColor', QtGui.QAction(u'Оформление без цвета', self))
        self.mnuBtnCheckMes.addAction(self.actDecarationColor)
        self.mnuBtnCheckMes.addAction(self.actDecarationNoColor)


    def showCheckMes(self, decarationColor):
        mesId = self.cmbMes.value()
        if mesId:
            dialog = CCheckMesParametersDialog(self)
            #dialog.setParams(params)
            if not dialog.exec_():
                return
            params = dialog.params()
            showCheckMesDescription(self, mesId, decarationColor, params)


    def setEventEditor(self, eventEditor):
        self.eventEditor = eventEditor
        self.modelCSGs.csgCol.setEventEditor(eventEditor)
        self.modelCSGSubItems.csgCol.setEventEditor(eventEditor)
        if hasattr(self.eventEditor,  'modelPreliminaryDiagnostics') and hasattr(self.eventEditor,  'tabMisc'):
            self.actCreateSomeCSGformepls.setEnabled(True)


    def setRecord(self, record):
        setRBComboBoxValue(self.cmbMes, record, 'MES_id')
        setRBComboBoxValue(self.cmbMesSpecification, record, 'mesSpecification_id')
        self.eventId = forceRef(record.value('id'))
        self.modelCSGs.loadItems(self.eventId)
        for record in self.modelCSGs.items()[::-1]:
            self.modelCSGSubItems.setMasterRecord(record)


    def setEventTypeId(self, eventTypeId):
        self.eventTypeId = eventTypeId
        self.cmbMes.setEventProfile(getEventProfileId(eventTypeId))
        self.cmbMes.setMESCodeTemplate(getEventMesCodeMask(eventTypeId))
        self.cmbMes.setMESNameTemplate(getEventMesNameMask(eventTypeId))
        self.modelCSGs.setCsgCodeMask(getEventCSGCodeMask(eventTypeId))
        self.modelCSGSubItems.setSubCsgCodeMask(getEventSubCSGCodeMask(eventTypeId))
        self.cmbMesSpecification.setValue(getEventMesSpecificationId(eventTypeId))
        self.setMESServiceTemplate(eventTypeId)
        csgRequired = getEventCSGRequired(eventTypeId)
        self.mesRequired = getEventMesRequired(eventTypeId)
        if csgRequired:
            for w in self.csgWidgets:
                w.setEnabled(True)
        if self.mesRequired:
            for w in self.mesWidgets:
                w.setEnabled(True)


    def setClientId(self, clientId):
        pass


    def setMESServiceTemplate(self, eventTypeId):
        domainServicesCodeList = []
        domainRList = self.parseMesServiceMask(getEvenMesServiceMask(eventTypeId))
        servicesCodeList = self.eventEditor.getServiceActionCode()
        for domainR in domainRList:
            for servicesCode in servicesCodeList:
                if domainR in servicesCode and servicesCode not in domainServicesCodeList:
                    domainServicesCodeList.append(servicesCode)
        self.cmbMes.setMESServiceTemplate(domainServicesCodeList)
        self.modelCSGs.setCsgServicesTemplate(domainServicesCodeList)


    def parseMesServiceMask(self, mesServiceTemplate):
        domainRList = u''
        if mesServiceTemplate:
            domainAll = QString(mesServiceTemplate)
            domainList = domainAll.split(';')
            if len(domainList) > 0:
                domainR = domainList[0]
                if len(domainR) > 0:
                    if u'*' in domainR:
                        index = domainR.indexOf(u'*', 0, Qt.CaseInsensitive)
                        if domainR[index - 1] != u',':
                            domainR.replace(QString('*'), QString(','))
                        else:
                            domainR.remove(QChar('*'), Qt.CaseInsensitive)
                    domainRList = domainR.split(',')
                    for i, domainR in enumerate(domainRList):
                        domainR.remove(QChar('\''), Qt.CaseInsensitive)
                        domainRList[i] = domainR
        return  domainRList


    def setEventBegDate(self, date):
        self.cmbMes.setEventBegDate(date)
        self.modelCSGs.setCsgFilterEventBegDate(date)


    def setClientInfo(self, baseDate, clientSex, clientBirthDate, clientAge, clientAgePrevYearEnd, clientAgeCurrYearEnd):
        self.cmbMes.setClientSex(clientSex)
        self.cmbMes.setClientAge(baseDate, clientBirthDate, clientAge, clientAgePrevYearEnd, clientAgeCurrYearEnd)
        self.modelCSGs.setCsgFilterClientBirthDate(clientBirthDate)
        self.modelCSGs.setCsgFilterClientSex(clientSex)


    def setSpeciality(self, specialityId):
        self.cmbMes.setSpeciality(specialityId)


    def getRecord(self, record):
        getRBComboBoxValue(self.cmbMes, record, 'MES_id')
        getRBComboBoxValue(self.cmbMesSpecification, record, 'mesSpecification_id')


    def save(self, eventId):
        self.modelCSGs.saveItems(eventId)


    def setMKB(self, MKB):
        self.cmbMes.setMKB(MKB)
        self.modelCSGs.setCsgFilterMKB(MKB)


    def setMKBEx(self, MKBEx):
        self.cmbMes.setMKBEx(MKBEx)


    def checkCsg(self):
        result = len(self.modelCSGs.items())>0 or self.eventEditor.checkInputMessage(u'КСГ', False, self.tblCSGs)
        result = result and (self.modelCSGs.checkData() or self.eventEditor.checkInputMessage(u'Данные КСГ', False, self.tblCSGs))
        return result

    def checkMesAndSpecification(self):
        if self.eventEditor.mesRequired:
            result = self.cmbMes.value() or self.eventEditor.checkInputMessage(u'МЭС', False, self.cmbMes)
            result = result and (self.cmbMesSpecification.value() or self.eventEditor.checkInputMessage(u'Особенности выполнения МЭС', False, self.cmbMesSpecification))
            return result
        return True


    def chechMesDuration(self):
        result = True
        mesId = self.cmbMes.value()
        if mesId:
            db = QtGui.qApp.db
            mesRecord = db.getRecord('mes.MES', '*', mesId)
            minDuration = forceInt(mesRecord.value('minDuration'))
            maxDuration = forceInt(mesRecord.value('maxDuration'))
            eventSetDateTime = self.eventEditor.eventSetDateTime
            eventDate = self.eventEditor.eventDate
            begDateEvent = eventSetDateTime if isinstance(eventSetDateTime, QDateTime) else (QDateTime(eventSetDateTime) if eventDate  else QDateTime())
            endDateEvent = eventDate if isinstance(eventDate, QDateTime) else (QDateTime(eventDate) if eventDate  else QDateTime())
            currentDateTime = QDateTime.currentDateTime()
            if not endDateEvent:
                endDateEvent = currentDateTime
            begDate = begDateEvent.date()
            endDate = endDateEvent.date()
            if endDate != begDate:
                avgDurationDay = begDate.daysTo(endDate) + 1
            else:
                avgDurationDay = 1
            result = (avgDurationDay >= minDuration or avgDurationDay >= maxDuration)
        return result


    def on_actCreateSomeCSGformepls(self):
        db = QtGui.qApp.db
        diags = self.eventEditor.modelFinalDiagnostics._items
        mkbField = self.eventEditor.modelFinalDiagnostics._mapFieldNameToCol['MKB']
        personField = self.eventEditor.modelFinalDiagnostics._mapFieldNameToCol['person_id']
        personTable = db.table('Person')
        recordList = {}
        for (record, action) in self.eventEditor.tabMisc.modelAPActions._items:
            if action.getType().flatCode == 'moving':
                begDate = forceDateTime(record.value('begDate'))
                endDate = forceDateTime(record.value('endDate'))
                person_id = forceRef(record.value('person_id'))
                personIdList = db.getIdList(personTable, where=personTable['orgStructure_id'].signEx('=', '(select orgStructure_id from Person as P where id = %i)'%person_id)) if person_id else []
                mkb = forceString(record.value('MKB'))
                if not mkb:
                    for diag in diags:
                        if forceRef(diag.value(personField)) == person_id and forceInt(diag.value('diagnosisType_id')) == 2:
                            mkb = forceString(diag.value(mkbField))
                            break
                if not mkb:
                    for diag in diags:
                        if forceRef(diag.value(personField)) in personIdList and forceInt(diag.value('diagnosisType_id')) == 2:
                            mkb = forceString(diag.value(mkbField))
                            break
                if begDate and endDate:
                    addNew = True
                    for existsCSGRecord in self.modelCSGs._items:
                        exbegDate = forceDateTime(existsCSGRecord.value('begDate'))
                        exendDate = forceDateTime(existsCSGRecord.value('endDate'))
                        exMKB = forceString(existsCSGRecord.value('MKB'))
                        if not exbegDate:
                            exbegDate = QDate.currentDate()
                        if not exendDate:
                            exendDate = QDate.currentDate()
                        if len(exMKB)>0 and len(mkb)>0 and exMKB[0] == mkb[0]: # надеюсь они отсортированы по датам
                            existsCSGRecord.setValue('endDate', QVariant(endDate))
                            self.eventEditor.tabMisc.cmbCSG.mapActionToCSG[record] = existsCSGRecord
                            tabs = []
                            if self.eventEditor and hasattr(self.eventEditor, 'tabCure'):
                                tabs.append(self.eventEditor.tabCure)
                            if self.eventEditor and hasattr(self.eventEditor, 'tabDiagnostic'):
                                tabs.append(self.eventEditor.tabDiagnostic)
                            for tab in tabs:
                                for (cureRecord, cureAction) in tab.modelAPActions._items:
                                    if cureAction._actionType.nomenclativeServiceId:
                                        cureEndDT = forceDateTime(cureRecord.value('endDate'))
                                        if cureEndDT >= begDate and cureEndDT <= endDate:
                                            tab.cmbCSG.mapActionToCSG[cureRecord] = existsCSGRecord
                            code, cost = self.modelCSGs.csgCol.getMostExpensiveCSG(existsCSGRecord, filterMKBClass=True)
                            existsCSGRecord.setValue('CSGCode', QVariant(code))
                            addNew = False
                            break
                        if begDate < exendDate and endDate > exbegDate:
                            addNew = False
                            break
                    if addNew:
                        newCsg = self.modelCSGs.getEmptyRecord()
                        newCsg.setValue('begDate', toVariant(begDate))
                        newCsg.setValue('endDate', toVariant(endDate))
                        if mkb:
                            newCsg.setValue('MKB', toVariant(mkb))
                        self.eventEditor.tabMisc.cmbCSG.mapActionToCSG[record] = newCsg
                        tabs = []
                        if self.eventEditor and hasattr(self.eventEditor, 'tabCure'):
                            tabs.append(self.eventEditor.tabCure)
                        if self.eventEditor and hasattr(self.eventEditor, 'tabDiagnostic'):
                            tabs.append(self.eventEditor.tabDiagnostic)
                        for tab in tabs:
                            for (cureRecord, cureAction) in tab.modelAPActions._items:
                                if (cureAction.getType().serviceType in [0,3,4]) and cureAction._actionType.nomenclativeServiceId:
                                    cureEndDT = forceDateTime(cureRecord.value('endDate'))
                                    if cureEndDT >= begDate and cureEndDT <= endDate:
                                        tab.cmbCSG.mapActionToCSG[cureRecord] = newCsg
                        code, cost = self.modelCSGs.csgCol.getMostExpensiveCSG(newCsg)
                        newCsg.setValue('CSGCode', QVariant(code))
                        self.modelCSGs.addRecord(newCsg)
                        orgStructure_id = action[u'Отделение пребывания']
                        recordList.setdefault(orgStructure_id, []).append((mkb,  record))
#        for mkbList in recordList:
#            if len(mkbList) > 1:
#                coolestMkb = mkbList[0][0]
        self.eventEditor.tabMisc.cmbCSG.setItems()
        self.eventEditor.tabCure.cmbCSG.setItems()
        self.eventEditor.tabMisc.updateCmbCSG()
        self.eventEditor.tabCure.updateCmbCSG()

    @pyqtSignature('')
    def on_mnuBtnCheckMes_aboutToShow(self):
        self.actDecarationColor.setEnabled(True)
        self.actDecarationNoColor.setEnabled(True)


    @pyqtSignature('')
    def on_actDecarationColor_triggered(self):
        self.showCheckMes(0)


    @pyqtSignature('')
    def on_actDecarationNoColor_triggered(self):
        self.showCheckMes(1)


    @pyqtSignature('')
    def on_btnShowMes_pressed(self):
        mesId = self.cmbMes.value()
        if mesId:
            showMesDescription(self, mesId)


    @pyqtSignature('int')
    def on_cmbMes_currentIndexChanged(self, index):
        mesCode = u''
        mesId = self.cmbMes.value()
        if mesId:
            self.btnCheckMes.setEnabled(forceBool(mesId))
            self.btnShowMes.setEnabled(forceBool(mesId))
            mesCode = u'МЭС события: ' + self.cmbMes.code()
        if self.eventEditor:
            self.eventEditor.setMesInfo(mesCode)


    @pyqtSignature('QModelIndex')
    def on_tblCSGs_doubleClicked(self, index):
        if index.column() == 4: # CSG
            self.modelCSGs.setCsgFilterMKB(forceString(
                self.modelCSGs.value(index.row(),'MKB')))
            self.modelCSGs.setCsgFilterEventProfileId(forceRef(
                self.modelCSGs.value(index.row(),'eventProfile_id')))



    @pyqtSignature('QModelIndex')
    def on_tblCSGs_clicked(self, index):
        if index.column() == 4: # CSG
            self.modelCSGs.setCsgFilterMKB(forceString(
                self.modelCSGs.value(index.row(),'MKB')))
            self.modelCSGs.setCsgFilterEventProfileId(forceRef(
                self.modelCSGs.value(index.row(),'eventProfile_id')))


    @pyqtSignature('QModelIndex, int, int')
    def on_modelCSGs_rowsRemoved(self, index1, start, end):
        self.csgRowRemoved.emit()

    @pyqtSignature('QModelIndex, int, int')
    def on_modelCSGs_rowsAboutToBeRemoved(self, index1, start, end):
        record = self.modelCSGs.items()[start]
        self.modelCSGSubItems.masterRowRemove(record)
        self.csgRowAboutToBeRemoved.emit(record)

    @pyqtSignature('QModelIndex, QModelIndex')
    def on_modelCSGs_dataChanged(self, index1, index2):
        row = index1.row()
        if row == len(self.modelCSGs.items())-1:
            self.modelCSGSubItems.setMasterRecord(self.modelCSGs.items()[row])
            self.tblCSGSubItems.setEnabled(True)

    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelCSGs_currentRowChanged(self, current, previous):
        row = current.row()
        if row < len(self.modelCSGs.items()):
            currentCSGRecord = self.modelCSGs.items()[current.row()]
            self.modelCSGSubItems.setMasterRecord(currentCSGRecord)
            self.tblCSGSubItems.setEnabled(True)
        else:
            self.modelCSGSubItems.setMasterRecord(None)
            self.tblCSGSubItems.setEnabled(False)


class CCheckMesParametersDialog(QtGui.QDialog, Ui_CheckMesParametersDialog):
    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)


    def setParams(self, params):
        self.chkMandatoryServiceMes.setChecked(params.get('chkMandatoryServiceMes', False))
        self.chkServiceMes.setChecked(params.get('chkServiceMes', False))
        self.chkServiceNoMes.setChecked(params.get('chkServiceNoMes', False))
        self.chkMedicamentsSection.setChecked(params.get('chkMedicamentsSection', False))


    def params(self):
        result = {}
        result['chkMandatoryServiceMes'] = self.chkMandatoryServiceMes.isChecked()
        result['chkServiceMes'] = (self.chkServiceMes.isChecked() if self.chkServiceMes.isEnabled() else False)
        result['chkServiceNoMes'] = self.chkServiceNoMes.isChecked()
        result['chkMedicamentsSection'] = self.chkMedicamentsSection.isChecked()
        return result


class CCSGModel(CInDocTableModel):
    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'Event_CSG', 'id', 'master_id', parent)
        self._parent = parent
        csgFilter = defaultFilters
        if QtGui.qApp.getGlobalPreference('csgServiceFilter') == u'нет':
            csgFilter['csgServices'] = False
        self.csgCol = CCSGInDocTableCol( u'КСГ','CSGCode', csgFilter,         7)
        self.setFilter(self._table['parentCSG_id'].isNull())
        self.addCol(CDateInDocTableCol(  u'С',         'begDate',    15, canBeEmpty=False)).setToolTip(u'Дата начала')
        self.addCol(CDateInDocTableCol(  u'По',        'endDate',    15, canBeEmpty=False)).setToolTip(u'Дата окончания')
        self.addCol(CICDExInDocTableCol( u'МКБ',       'MKB',        7)).setToolTip(u'Код диагноза')
        self.addCol(CRBInDocTableCol(    u'Профиль',   'eventProfile_id', 20, 'rbEventProfile', showFields = CRBComboBox.showNameAndCode))
        self.addCol(self.csgCol).setToolTip(u'Код КСГ')
        self.addCol(CIntInDocTableCol(   u'Количество', 'amount', 10)).setToolTip(u'Количество')
        self.addHiddenCol('payStatus')

    def checkData(self):
        for record in self.items():
            if not (forceDate(record.value('begDate')) and forceDate(record.value('endDate')) and forceString(record.value('MKB')) and forceString(record.value('CSGCode'))):
                return False
        return True

    def getEmptyRecord(self):
        result = CInDocTableModel.getEmptyRecord(self)
        result.setValue('amount',  QVariant(1))
        return result


    def setCsgFilterEventBegDate(self, date):
        self.csgCol.setEventBegDate(date)


    def setCsgFilterClientBirthDate(self, date):
        self.csgCol.setClientBirthDate(date)


    def setCsgFilterClientSex(self, sex):
        self.csgCol.setClientSex(sex)

    def setCsgFilterMKB(self, MKB):
        self.csgCol.setMKB(MKB)


    def setCsgFilterEventProfileId(self, eventProfileId):
        self.csgCol.setEventProfileId(eventProfileId)


    def setCsgCodeMask(self, mask):
        self.csgCol.setCsgCodeMask(mask)

    def setCsgServicesTemplate(self, MESServiceTemplate):
        self.csgCol.setCsgServiceTemplate(MESServiceTemplate)

    def saveDependence(self, idx, id):
        record = self._items[idx]
        self._parent.modelCSGSubItems.saveItemsForRecord(record)

    def insertRecord(self, row, record):
        self.beginInsertRows(QModelIndex(), row, row)
        self._items.insert(row, record)
        self.endInsertRows()
        self._parent.modelCSGSubItems.setMasterRecord(record)

    def createEditor(self, index, parent):
        column = index.column()
        row = index.row()
        editor = self._cols[column].createEditor(parent)
        if type(editor).__name__ == 'CCSGComboBox':
            if row < len(self._items):
                record = self._items[row]
                editor.setCSGRecord(record)
            else:
                editor.setCSGRecord(None)
        return editor


class CCSGSlaveModel(CInDocTableModel):
    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'Event_CSG', 'id', 'parentCSG_id', parent)
        self._parent = parent
        csgFilter = defaultFilters
        if QtGui.qApp.getGlobalPreference('csgServiceFilter') == u'нет':
            csgFilter['csgServices'] = False
        self.csgCol = CCSGInDocTableCol( u'КСГ','CSGCode', csgFilter, 7)
        self.addCol(CDateInDocTableCol( u'С',   'begDate',    15, canBeEmpty=False)).setToolTip(u'Дата начала')
        self.addCol(CDateInDocTableCol( u'По',  'endDate',    15, canBeEmpty=False)).setToolTip(u'Дата окончания')
        self.addCol(CICDExInDocTableCol( u'МКБ','MKB',        7)).setToolTip(u'Код диагноза')
        self.addCol(self.csgCol).setToolTip(u'Код КСГ')
        self.addCol(CIntInDocTableCol( u'Количество', 'amount', 10)).setToolTip(u'Количество')
        self.addHiddenCol('master_id')
        self.addHiddenCol('payStatus')
        self.mapRecordToItems = {}
        self.currentMasterRecord = None

    def getEmptyRecord(self):
        result = CInDocTableModel.getEmptyRecord(self)
        result.setValue('amount',  QVariant(1))
        return result

    def setMasterRecord(self, record):
        if record == self.currentMasterRecord:
            pass
        if not record:
            self.currentMasterRecord = None
            self._items = []
            self.reset()
            return
        self.currentMasterRecord = record
        if record in self.mapRecordToItems:
            self._items = self.mapRecordToItems[record]
            self.reset()
        else:
            masterId = forceRef(record.value('id'))
            if masterId:
                self.loadItems(masterId)
            else:
                self.mapRecordToItems[record] = []
                self._items = self.mapRecordToItems[record]
                self.reset()

    def saveItemsForRecord(self, record):
        if record in self.mapRecordToItems:
            self._items = self.mapRecordToItems[record]
            masterId = forceRef(record.value('id'))
            eventId = forceRef(record.value('master_id'))
            for item in self._items:
                item.setValue('master_id',  toVariant(eventId))
            self.saveItems(masterId)


    def setSubCsgCodeMask(self, mask):
        self.csgCol.setCsgCodeMask(mask)

    def loadItems(self, masterId):
        db = QtGui.qApp.db
        cols = []
        for col in self._cols:
            if not col.external():
                cols.append(col.fieldName())
        cols.append(self._idFieldName)
        cols.append(self._masterIdFieldName)
        if self._idxFieldName:
            cols.append(self._idxFieldName)
        for col in self._hiddenCols:
            cols.append(col)
        table = self._table
        filter = [table[self._masterIdFieldName].eq(masterId)]
        if self._filter:
            filter.append(self._filter)
        if table.hasField('deleted'):
            filter.append(table['deleted'].eq(0))
        if self._idxFieldName:
            order = [self._idxFieldName, self._idFieldName]
        else:
            order = [self._idFieldName]
        self.mapRecordToItems[self.currentMasterRecord] = db.getRecordList(table, cols, filter, order)
        if self._extColsPresent:
            extSqlFields = []
            for col in self._cols:
                if col.external():
                    fieldName = col.fieldName()
                    if fieldName not in cols:
                        extSqlFields.append(QtSql.QSqlField(fieldName, col.valueType()))
            if extSqlFields:
                for item in self.mapRecordToItems[self.currentMasterRecord]:
                    for field in extSqlFields:
                        item.append(field)
        self._items = self.mapRecordToItems[self.currentMasterRecord]
        self.reset()

    def masterRowRemove(self, record):
        if record in self.mapRecordToItems:
            del self.mapRecordToItems[record]

    def createEditor(self, index, parent):
        column = index.column()
        row = index.row()
        editor = self._cols[column].createEditor(parent)
        if type(editor).__name__ == 'CCSGComboBox':
            record = self._items[row]
            editor.setCSGRecord(record)
        return editor
