# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import sip

from PyQt4 import QtGui
from PyQt4.QtCore import Qt, QAbstractTableModel, QDate, QDateTime, QModelIndex, QObject, QTime, QVariant, pyqtSignature, SIGNAL



from Users.Rights         import urEditExecutionPlanAction
from library.DialogBase   import CDialogBase
from library.Utils        import forceDateTime, forceInt, forceRef, forceString, pyDate, toVariant

from library.TableModel   import CCol, CDateCol, CTimeCol


from Reports.ReportView   import CReportViewDialog

from Events.Ui_ExecutionPlanDialog import Ui_ExecutionPlanDialog
from Events.Ui_ExecutionPlanEditor import Ui_ExecutionPlanEditor


class CExecutionPlanDialog(CDialogBase, Ui_ExecutionPlanDialog):
    def __init__(self, parent, cols, tableName, order, record, executionPlan, forSelect=False, filterClass=None):
        CDialogBase.__init__(self, parent)
        self.setupUi(self)
        self.setup(cols, tableName, order, record, executionPlan, forSelect, filterClass)


    def setup(self, cols, tableName, order, record, executionPlan, forSelect=False, filterClass=None):
        self.setWindowFlags(self.windowFlags() | Qt.WindowMaximizeButtonHint)
        self.forSelect = forSelect
        self.executionPlan = executionPlan
        self.record = record
        self.filterClass = filterClass
        self.props = {}
        self.order = order
        self.model = CExecutionPlanModel(self)
        self.tblExecutionPlan.setModel(self.model)
        self.btnEdit.setDefault(not self.forSelect)
        self.tblExecutionPlan.setFocus(Qt.OtherFocusReason)

        directionDateTime = forceDateTime(record.value('directionDate'))
        self.edtDirectionDate.setDate(directionDateTime.date())
        self.edtDirectionTime.setTime(directionDateTime.time())
        self.cmbSetPerson.setValue(forceRef(record.value('setPerson_id')))
        self.isEditor = False if forceDateTime(record.value('endDate')) else True
        self.specifiedName = forceString(record.value('specifiedName'))
        self.actionTypeId = forceRef(record.value('actionType_id'))
        self.aliquoticity = forceInt(record.value('aliquoticity'))
        begDateTime = forceDateTime(record.value('begDate'))
        begDate = begDateTime.date()
        self.edtBegDate.setDate(begDate)
        duration = forceInt(record.value('duration'))
        # periodicity - это интервал. Сколько полных дней между назначениями
        periodicity = forceInt(record.value('periodicity'))
        plannedEndDateTime = forceDateTime(record.value('plannedEndDate'))
        if not plannedEndDateTime:
            plannedEndDateTime = begDateTime.addDays(duration-1)
        plannedEndDate = plannedEndDateTime.date()
        self.edtEndDate.setDate(plannedEndDate)
        self.edtDuration.setValue(duration)
        self.actionId = forceRef(record.value('id'))
        begDateNext = begDate
        while begDateNext <= plannedEndDate:
            if begDateNext not in self.executionPlan.keys():
                self.executionPlan[pyDate(begDateNext)] = {}
            begDateNext = begDateNext.addDays(periodicity+1)
        self.model.loadData(self.executionPlan)
        self.label.setText(u'всего: %d' % len(self.model.items))

        QObject.connect(
            self.tblExecutionPlan.horizontalHeader(), SIGNAL('sectionClicked(int)'), self.setSort)


    def select(self, props):
        table = self.model.table()
        return QtGui.qApp.db.getIdList(table.name(), 'id', '', self.order)


    def getItemEditor(self):
        return CExecutionPlanEditor(self, self.aliquoticity)


    def selectItem(self):
        return self.exec_()


    @pyqtSignature('QModelIndex')
    def on_tblExecutionPlan_doubleClicked(self, index):
        self.selected = True
        self.on_btnEdit_clicked()


    @pyqtSignature('')
    def on_btnCancel_clicked(self):
        self.close()


    @pyqtSignature('')
    def on_btnEdit_clicked(self):
        if self.isEditor:
            if QtGui.qApp.userHasRight(urEditExecutionPlanAction):
                item = {}
                row = self.tblExecutionPlan.currentRow()
                if row >= 0:
                    items = self.tblExecutionPlan.model().items
                    item = items[row][2]
                    if item:
                        dialog = self.getItemEditor()
                        try:
                            dialog.load(item)
                            if dialog.exec_():
                                execDate = item.keys()[0]
                                item = dialog.getInfoDict()
                                self.tblExecutionPlan.model().loadDataItem(item, execDate, row)
                                if 0 < len(items) and (row + 1) < len(items):
                                    self.tblExecutionPlan.setCurrentRow(row + 1)
                                else:
                                    self.btnCancel.setFocus(Qt.OtherFocusReason)
                        finally:
                            sip.delete(dialog)
            else:
                QtGui.QMessageBox.information(
                    self,
                    u'Внимание!',
                    u'У вас нет прав на редактирование календаря выполнения назначений!')
        else:
            QtGui.QMessageBox.information(
                self,
                u'Внимание!',
                u'Вы не можете редактировать календарь выполнения назначений, так как действие выполнено!')


    def getReportHeader(self):
        return self.objectName()


    def getFilterAsText(self):
        return u''


    def contentToHTML(self):
        actionTypeName = ''
        if self.actionTypeId:
            db = QtGui.qApp.db
            table = db.table('ActionType')
            record = db.getRecordEx(table, [table['name']], [table['id'].eq(self.actionTypeId), table['deleted'].eq(0)])
            if record:
                actionTypeName = forceString(record.value('name'))
        reportHeader = self.getReportHeader() + u'\n'+ u'Действие: ' + actionTypeName + ((u'(' + self.specifiedName + u')') if self.specifiedName else u'')
        self.tblExecutionPlan.setReportHeader(reportHeader)
        reportDescription=self.getFilterAsText()
        self.tblExecutionPlan.setReportDescription(reportDescription)
        return self.tblExecutionPlan.contentToHTML()


    @pyqtSignature('')
    def on_btnPrint_clicked(self):
        html = self.contentToHTML()
        view = CReportViewDialog(self)
        view.setText(html)
        view.exec_()


    def setSort(self, col):
        name=self.model.cols()[col].fields()[0]
        self.order = name
        header=self.tblExecutionPlan.horizontalHeader()
        header.setSortIndicatorShown(True)
        header.setSortIndicator(col, Qt.AscendingOrder)


class CExecutionPlanEditor(CDialogBase, Ui_ExecutionPlanEditor):
    def __init__(self,  parent, aliquoticity):
        CDialogBase.__init__(self, parent)
        self.setupUi(self)
        self.setWindowFlags(self.windowFlags() | Qt.WindowMaximizeButtonHint)
        self.tblPlanEditor.setModel(CExecutionPlanEditorModel(self))
        self.item = {}
        self.aliquoticity = max(aliquoticity, 1)

    def load(self, item):
        self.item = item
        if self.item:
            self.tblPlanEditor.model().loadData(self.item)

    def getInfoDict(self):
        return self.tblPlanEditor.model().saveData()

    def saveData(self):
        modelItems = self.tblPlanEditor.model().items
        if modelItems and len(modelItems) != self.aliquoticity:
            return self.checkValueMessage(u'Запланированное время должно либо '
                                          u'отсутствовать либо иметь %d элемента' % self.aliquoticity,
                                          skipable=False,
                                          widget=self)
        return True


class CExecutionPlanEditorModel(QAbstractTableModel):
    column = [u'Выбранное время']

    def __init__(self, parent):
        QAbstractTableModel.__init__(self, parent)
        self.items = []
        self.executionPlan = {}


    def columnCount(self, index = None):
        return 1


    def rowCount(self, index = None):
        return len(self.items)+1


    def flags(self, index = None):
        return Qt.ItemIsSelectable|Qt.ItemIsEnabled|Qt.ItemIsEditable


    def headerData(self, section, orientation, role = Qt.DisplayRole):
        if orientation == Qt.Horizontal:
            if role == Qt.DisplayRole:
                return QVariant(self.column[section])
        return QVariant()


    def loadData(self, executionPlan):
        self.executionPlan = executionPlan
        self.items = []
        db = QtGui.qApp.db
        table = db.table('Action_ExecutionPlan')
        if self.executionPlan:
            execDateKeys = self.executionPlan.keys()
            execDateKeys.sort()
            for execDate in execDateKeys:
                self.column = [QDate(execDate).toString('dd-MM-yyyy')]
                execTimeDict = self.executionPlan.get(execDate, {})
                execTimeKeys = execTimeDict.keys()
                execTimeKeys.sort()
                for execTime in execTimeKeys:
                    record = execTimeDict.get(execTime, None)
                    if not record:
                        record = table.newRecord()
                    item = [execTime,
                            QDate(execDate),
                            record
                            ]
                    self.items.append(item)
        self.reset()


    def saveData(self):
        if len(self.items) < 1:
            return self.executionPlan
        self.executionPlan = {}
        db = QtGui.qApp.db
        table = db.table('Action_ExecutionPlan')
        for item in self.items:
            execTime = item[0]
            if execTime:
                execDate = pyDate(item[1])
                record = item[2]
                execTimeDict = self.executionPlan.get(execDate, {})
                if execTime in execTimeDict.keys():
                    record = execTimeDict.get(execTime, None)
                    if not record:
                        record = table.newRecord()
                else:
                    record = table.newRecord()
                record.setValue('execDate', toVariant(QDateTime(QDate(execDate), execTime)))
                execTimeDict[execTime] = record
                self.executionPlan[execDate] = execTimeDict
        return self.executionPlan


    def data(self, index, role=Qt.DisplayRole):
        column = index.column()
        row = index.row()
        if role == Qt.DisplayRole:
            if 0 <= row < len(self.items):
                if self.items[row][column]:
                    return toVariant(self.items[row][column])
        if role == Qt.EditRole:
            if 0 <= row < len(self.items):
                if self.items[row][column]:
                    return toVariant(self.items[row][column])
        return QVariant()


    def setData(self, index, value, role=Qt.EditRole):
        if role == Qt.EditRole:
            column = index.column()
            row = index.row()
            if row == len(self.items):
                if value.isNull():
                    return False
                execDateList = self.executionPlan.keys()
                if execDateList:
                    execDate = execDateList[0]
                    self.items.append([QTime(), QDate(execDate), None])
                    vCnt = len(self.items)
                    vIndex = QModelIndex()
                    self.beginInsertRows(vIndex, vCnt, vCnt)
                    self.insertRows(vCnt, 1, vIndex)
                    self.endInsertRows()
                else:
                    return False
            if 0 <= row < len(self.items):
                newTime = value.toTime()
                if newTime.isNull():
                    self.items[row][column] = None
                    self.items[row][2] = None
                else:
                    self.items[row][column] = value.toTime()
                    record = self.items[row][2]
                    if not record:
                        db = QtGui.qApp.db
                        table = db.table('Action_ExecutionPlan')
                        record = table.newRecord()
                    record.setValue('execDate', toVariant(QDateTime(self.items[row][1], self.items[row][column])))
                    self.items[row][2] = record
            self.emitCellChanged(row, column)
            return True
        return False


    def emitCellChanged(self, row, column):
        index = self.index(row, column)
        self.emit(SIGNAL('dataChanged(QModelIndex, QModelIndex)'), index, index)

    def confirmRemoveRow(self, view, row, multiple=False):
        return True

    def canRemoveRow(self, row):
        return 0 <= row < len(self.items)

    def removeRow(self, row, parentIndex=QModelIndex()):
        if not 0 <= row < len(self.items):
            return False

        self.beginRemoveRows(parentIndex, row, row)
        execTime, execDate, execRecord = self.items[row]
        self.executionPlan.get(execDate, {}).pop(execTime, None)
        del self.items[row]
        self.endRemoveRows()


class CGetExecutionPlan(CExecutionPlanDialog):
    def __init__(self, parent, record, executionPlan):
        CExecutionPlanDialog.__init__(self, parent, [
            CDateCol(u'Дата',  ['execDate'], 20),
            CTimeCol(u'Время', ['execDate'], 40),
            ], 'Action_ExecutionPlan', ['execDate'], record, executionPlan)
        self.setWindowTitleEx(u'План выполнения назначений')
        self.selected = False
        self.executionPlan = executionPlan


class CExecutionPlanModel(QAbstractTableModel):
    column = [u'Дата', u'Время']

    def __init__(self, parent):
        QAbstractTableModel.__init__(self, parent)
        self.items = []
        self._cols = []
        self.executionPlan = {}


    def cols(self):
        self._cols = [CCol(u'Дата',     ['execDate'], 20, 'l'),
                      CCol(u'Время',    ['execDate'], 20, 'l')
                      ]
        return self._cols


    def columnCount(self, index = None):
        return 2


    def rowCount(self, index = None):
        return len(self.items)


    def headerData(self, section, orientation, role = Qt.DisplayRole):
        if orientation == Qt.Horizontal:
            if role == Qt.DisplayRole:
                return QVariant(self.column[section])
        return QVariant()


    def data(self, index, role=Qt.DisplayRole):
        column = index.column()
        row = index.row()
        if role == Qt.DisplayRole:
            item = self.items[row]
            return toVariant(item[column])
        return QVariant()


    def loadData(self, executionPlan):
        self.items = []
        self.executionPlan = executionPlan
        execDateKeys = executionPlan.keys()
        execDateKeys.sort()
        for execDate in execDateKeys:
            resultDict = {}
            execTimeDict = executionPlan.get(execDate, {})
            execTimeKeys = execTimeDict.keys()
            execTimeKeys.sort()
            execTimeStr = u', '.join(str(execTime.toString('hh:mm')) for execTime in execTimeKeys if execTime)
            resultDict[execDate] = execTimeDict
            item = [QDate(execDate).toString('dd-MM-yyyy'),
                    execTimeStr,
                    resultDict
                    ]
            self.items.append(item)
        self.reset()


    def loadDataItem(self, executionPlan, execDate, row):
        execTimeDict = executionPlan.get(execDate, {})
        execTimeKeys = execTimeDict.keys()
        execTimeKeys.sort()
        execTimeStr = u', '.join(str(execTime.toString('hh:mm')) for execTime in execTimeKeys if execTime)
        item = [QDate(execDate).toString('dd-MM-yyyy'),
                execTimeStr,
                executionPlan
                ]
        self.executionPlan[execDate] = executionPlan.get(execDate, {})
        self.items[row] = item
        self.reset()
