# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import SIGNAL


from library.DialogBase import CDialogBase

from Events.NomenclatureExpense.Ui_NomenclatureExpenseDayDialog import Ui_NomenclatureExpenseDayDialog

from Events.NomenclatureExpense.NomenclatureExpenseItemDelegate import CLocItemDelegate
from Events.NomenclatureExpense.NomenclatureExpenseDayModel import CNomenclatureExpenseDayModel, TIME_INDEX, DOSES_INDEX


class CNomenclatureExpenseDayDialog(CDialogBase, Ui_NomenclatureExpenseDayDialog):
    def __init__(self, parent, date, dosageUnitName=''):
        CDialogBase.__init__(self, parent)
        self._parent = parent
        self._result = None
        self.addModels('NomenclatureExpense', CNomenclatureExpenseDayModel(self, dosageUnitName=dosageUnitName))
        self.setupUi(self)

        self.setWindowTitle(u'Суточные назначения: %d.%d.%d' % (date.year(), date.month(), date.day()))

        self.tblNomenlcatureExpense.contextMenuEvent = self._tblContextMenuEvent
        self.setModels(self.tblNomenlcatureExpense, self.modelNomenclatureExpense, self.selectionModelNomenclatureExpense)

        self.tblNomenlcatureExpense.setItemDelegate(CLocItemDelegate(self.tblNomenlcatureExpense))
        h = self.tblNomenlcatureExpense.fontMetrics().height()
        self.tblNomenlcatureExpense.verticalHeader().setDefaultSectionSize(3 * h / 2)
        self.tblNomenlcatureExpense.verticalHeader().hide()
        self.tblNomenlcatureExpense.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)

        self._popupMenu = QtGui.QMenu(self.tblNomenlcatureExpense)
        self._popupMenu.setObjectName('popupMenu')
        self.connect(self._popupMenu, SIGNAL('aboutToShow()'), self.on_popupMenu_aboutToShow)

        self._actDelete = QtGui.QAction(u'Удалить', self.tblNomenlcatureExpense)
        self._popupMenu.addAction(self._actDelete)
        self.connect(self._actDelete, SIGNAL('triggered()'), self.on_delete)

    def _tblContextMenuEvent(self, event): # event: QContextMenuEvent
        if self._popupMenu:
            self._popupMenu.exec_(event.globalPos())
            event.accept()
        else:
            event.ignore()

    def on_popupMenu_aboutToShow(self):
        self.tblNomenlcatureExpense.emit(SIGNAL('popupMenuAboutToShow()'))

    def on_delete(self):
        rows = [i.row() for i in self.tblNomenlcatureExpense.selectedIndexes()]
        for row in set(rows):
            self.modelNomenclatureExpense.removeRow(row)

    def load(self, data, notEditableData):
        self.modelNomenclatureExpense.load(data, notEditableData)

    @property
    def nomenclatureExpenseResult(self):
        return self._result

    def saveData(self):
        result = []
        zeroTime = QtCore.QTime(0, 0)
        times = {}
        for row, item in enumerate(self.tblNomenlcatureExpense.model().items()):
            if not item.editable:
                result.append({'time': item.time, 'dosage': item.dosage, 'record': item.record, 'idx': row})
                continue
            time = item.time
            if not time.isValid():
                time = zeroTime

            if time != zeroTime:
                pyTime = time.toPyTime()
                if pyTime in times:
                    exitsRow = times[pyTime]
                    return self.checkValueMessage(u'Данное время уже указано.', False, self.tblNomenlcatureExpense, exitsRow, TIME_INDEX)

                times[pyTime] = row

            dosage = item.dosage
            if not dosage:
                return self.checkInputMessage(u'дозировку.', False, self.tblNomenlcatureExpense, row, DOSES_INDEX)

            result.append({'time': time, 'dosage': dosage, 'record': None, 'idx': row})

        self._result = result
        return True
