# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
import json

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import SIGNAL, pyqtSignature


from library.DialogBase import CDialogBase

from Events.NomenclatureExpense.Ui_NomenclatureExpenseDialog import Ui_NomenclatureExpenseDialog

from Events.NomenclatureExpense.NomenclatureExpenseModel import CNomenclatureExpenseModel

from Events.NomenclatureExpense.QueriesStatements import getNomenclatureActionTypesIds

from library.Utils import forceString


def _getDefaultValues():
    result = {}
    data = forceString(
        QtGui.qApp.preferences.appPrefs.get('NomenclatureExpenseDialogDefaultValues', '')
    )
    data = json.loads(data) if data else {}

    result['actionTypeId'] = data.get('actionTypeId', None)
    result['nomenclatureId'] = data.get('nomenclatureId', None)
    result['begDate'] = QtCore.QDate.fromString(data.get('begDate', QtCore.QDate.currentDate().toString('yyyy.MM.dd')), 'yyyy.MM.dd')
    result['endDate'] = QtCore.QDate.fromString(data.get('endDate', QtCore.QDate.currentDate().toString('yyyy.MM.dd')), 'yyyy.MM.dd')
    result['year'] = data.get('year', QtCore.QDate.currentDate().year())
    result['month'] = data.get('month', QtCore.QDate.currentDate().month() - 1)
    result['period'] = data.get('period', False)
    result['actual'] = data.get('actual', True)
    result['currentEvent'] = data.get('currentEvent', True)

    return result


class CNomenclatureExpenseDialog(CDialogBase, Ui_NomenclatureExpenseDialog):
    def __init__(self, parent=None, eventEditor=None, groups=None, fromEventEditor=True):
        CDialogBase.__init__(self, parent)

        self.modelNomenclatureExpense = None
        self.selectionModelNomenclatureExpense = None

        self.addModels('NomenclatureExpense', CNomenclatureExpenseModel(self))
        selectionModelName = 'selectionModellNomenclatureExpenseDays'
        self.selectionModelNomenclatureExpenseDays = QtGui.QItemSelectionModel(self.modelNomenclatureExpense, self)
        self.selectionModelNomenclatureExpenseDays.setObjectName(selectionModelName)

        self.modelNomenclatureExpense.setGroups(groups)

        self._eventEditor = eventEditor
        self._fromEventEditor = fromEventEditor

        self.setupUi(self)

        self.setWindowTitle(u'Назначение ЛС')

        self.cmbActionType.setClass(None)
        self.cmbActionType.setClassesVisible(True)
        self.cmbNomenclature.setOnlyExists(True)

        self.setCmbActionTypeFilter()

        self.setModels(self.tblNomenclatureExpense, self.modelNomenclatureExpense, self.selectionModelNomenclatureExpense)
        self.setModels(self.tblNomenclatureExpenseDays, self.modelNomenclatureExpense, self.selectionModelNomenclatureExpenseDays)
        self.tblNomenclatureExpenseDays.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
        self.tblNomenclatureExpenseDays.addEditDay()
        self.tblNomenclatureExpenseDays.addCopyDay()
        self.tblNomenclatureExpenseDays.addPasteDay()
        self.tblNomenclatureExpenseDays.addDeleteDays()

        self._hideDaysColumns(self.tblNomenclatureExpense)
        self._hideMainColumns(self.tblNomenclatureExpenseDays)

        self._cmbActionTypeWidgetsDependets = [
            self.cmbNomenclature, self.chkPeriod, self.chkActual, self.chkCurrentEvent, self.edtYear, self.cmbMonth,
            self.lblNomenclature, self.lblYear, self.lblMonth
        ]

        self.chkCurrentEvent.setVisible(False)

        self.connect(self.cmbActionType, SIGNAL('currentIndexChanged(int)'), self.on_cmbActionTypeIndexChanged)
        self.connect(self.cmbNomenclature, SIGNAL('currentIndexChanged(int)'), self.on_cmbNomenclatureIndexChanged)
        self.connect(self.cmbMonth, SIGNAL('currentIndexChanged(int)'), self.on_cmbMonthIndexChanged)
        self.connect(self.edtYear, SIGNAL('valueChanged(int)'), self.on_edtYearValueChanged)
        self.connect(self.chkActual, SIGNAL('clicked(bool)'), self.on_chkActualClicked)
        # self.connect(self.edtBegDate, SIGNAL('dateChanged(QDate)'), self.on_edtBegDateChanged)
        # self.connect(self.edtEndDate, SIGNAL('dateChanged(QDate)'), self.on_edtEndDateChanged)
        self.connect(self.chkPeriod, SIGNAL('clicked(bool)'), self.on_chkPeriodClicked)

        self.connect(self.selectionModelNomenclatureExpense, SIGNAL('currentChanged(QModelIndex, QModelIndex)'),
                     self.on_nomenclatureExpenseSelectionChanged)

        self.connect(self.selectionModelNomenclatureExpenseDays, SIGNAL('currentChanged(QModelIndex, QModelIndex)'),
                     self.on_nomenclatureExpenseDaysSelectionChanged)

        defaults = _getDefaultValues()

        self.cmbActionType.setValue(defaults['actionTypeId'])
        self.cmbNomenclature.setValue(defaults['nomenclatureId'])
        self.edtBegDate.setDate(defaults['begDate'])
        self.edtEndDate.setDate(defaults['endDate'])
        self.edtYear.setValue(defaults['year'])
        self.cmbMonth.setCurrentIndex(defaults['month'])
        self.chkPeriod.setChecked(defaults['period'])
        self.chkActual.setChecked(defaults['actual'])
        self.chkCurrentEvent.setChecked(defaults['currentEvent'])

        self.on_cmbActionTypeIndexChanged()
        self.on_cmbNomenclatureIndexChanged()
        self.on_cmbMonthIndexChanged(self.cmbMonth.currentIndex())
        self.on_edtYearValueChanged(self.edtYear.value())
        self.on_chkActualClicked(self.chkActual.isChecked())
        self.on_chkPeriodClicked()
        self.on_edtBegDate_dateChanged(self.edtBegDate.date())
        self.on_edtEndDate_dateChanged(self.edtEndDate.date())

    def on_chkPeriodClicked(self, v=None):
        self.modelNomenclatureExpense.considerPeriod(self.chkPeriod.isChecked())

    @pyqtSignature('QDate')
    def on_edtBegDate_dateChanged(self, date):
        self.modelNomenclatureExpense.setBegDate(date)

    @pyqtSignature('QDate')
    def on_edtEndDate_dateChanged(self, date):
        self.modelNomenclatureExpense.setEndDate(date)

    def on_nomenclatureExpenseSelectionChanged(self, i1, i2):
        self.selectionModelNomenclatureExpenseDays.clearSelection()

    def on_nomenclatureExpenseDaysSelectionChanged(self, i1, i2):
        self.selectionModelNomenclatureExpense.clearSelection()

    def _hideDaysColumns(self, tbl):
        header = tbl.horizontalHeader()
        start = tbl.model().lastMainIndex
        for i in range(header.count()):
            if i > start:
                header.setSectionHidden(i, True)

    def _hideMainColumns(self, tbl):
        header = tbl.horizontalHeader()
        stop = tbl.model().lastMainIndex
        for i in range(header.count()):
            if i > stop:
                break
            header.setSectionHidden(i, True)

    @property
    def eventEditor(self):
        return self._eventEditor

    def setEventEditor(self, eventEditor):
        self._eventEditor = eventEditor
        self.modelNomenclatureExpense.setEventEditor(eventEditor)


    def on_chkActualClicked(self, value):
        self.modelNomenclatureExpense.setOnlyActual(value)

    def on_cmbMonthIndexChanged(self, monthIndex):
        month = monthIndex + 1
        year = self.edtYear.value()
        self._updateModelDate(year, month)
        self._hideDaysColumns(self.tblNomenclatureExpense)

    def on_edtYearValueChanged(self, year):
        month = self.cmbMonth.currentIndex()+1
        self._updateModelDate(year, month)
        self._hideDaysColumns(self.tblNomenclatureExpense)

    def _updateModelDate(self, year, month):
        date = QtCore.QDate(year, month, 1)
        date = QtCore.QDate(year, month, date.daysInMonth())
        self.modelNomenclatureExpense.setDate(date)

    def setCmbActionTypeFilter(self):
        db = QtGui.qApp.db
        table = db.table('ActionType')
        idList = getNomenclatureActionTypesIds()
        descendants = []
        for id in idList:
            descendants.extend(db.getDescendants(table, 'group_id', id))

        idList = db.getTheseAndParents(table, 'group_id', descendants)

        self.cmbActionType.setEnabledActionTypeIdList(idList)

    def exec_(self):
        return CDialogBase.exec_(self)

    def on_cmbNomenclatureIndexChanged(self, index=None):
        nomenclatureId = self.cmbNomenclature.value()
        self.modelNomenclatureExpense.setNomenclatureId(nomenclatureId)

    def on_cmbActionTypeIndexChanged(self, index=None):
        actionTypeId = self.cmbActionType.value()

        for wgt in self._cmbActionTypeWidgetsDependets:
            wgt.setEnabled(bool(actionTypeId))

        self.modelNomenclatureExpense.setActionTypeId(actionTypeId)

        self.chkPeriod.emit(SIGNAL('clicked(bool)'), self.chkPeriod.isChecked() and bool(actionTypeId))

    def done(self, result):
        defaults = {
            'actionTypeId': self.cmbActionType.value(),
            'nomenclatureId': self.cmbNomenclature.value(),
            'begDate': str(self.edtBegDate.date().toString('yyyy.MM.dd')),
            'endDate': str(self.edtEndDate.date().toString('yyyy.MM.dd')),
            'year': self.edtYear.value(),
            'month': self.cmbMonth.currentIndex(),
            'period': self.chkPeriod.isChecked(),
            'actual': self.chkActual.isChecked(),
            'currentEvent': self.chkCurrentEvent.isChecked()
        }

        defaults = json.dumps(defaults)

        QtGui.qApp.preferences.appPrefs['NomenclatureExpenseDialogDefaultValues'] = defaults

        return CDialogBase.done(self, result)

    def saveData(self):
        if self._fromEventEditor:
            self._prepareGoupsToSave()
            self._addNewGroups()
            return True

    def discardData(self):
        if self._fromEventEditor:
            self.modelNomenclatureExpense.items().discardChanges()

    def _prepareGoupsToSave(self):
        for group in self.modelNomenclatureExpense.items():
            group.prepareToSave()

    def _addNewGroups(self):
        tabs = self._eventEditor.getActionsTabsList()
        differClasses = len(tabs) > 1
        for group in self.modelNomenclatureExpense.items().groupsToAdd():
            group.initNomenclatureReservation(self._eventEditor.clientId)
            if differClasses:
                tabs[group.class_].modelAPActions.addRow(presetAction=group.headItem[1])
            else:
                tabs[0].modelAPActions.addRow(presetAction=group.headItem[1])
        for tab in tabs:
            tab.onActionCurrentChanged()
