# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################


from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import SIGNAL

from library.Utils import forceInt, getPref, setPref

from library.PreferencesMixin import CPreferencesMixin

from Events.NomenclatureExpense.NomenclatureExpenseDayDialog import CNomenclatureExpenseDayDialog
from Events.NomenclatureExpense.NomenclatureExpenseItemDelegate import CLocItemDelegate


class CNomenclatureExpenseView(QtGui.QTableView, CPreferencesMixin):
    __pyqtSignals__ = ('popupMenuAboutToShow()',
                       )

    def __init__(self, parent):
        QtGui.QTableView.__init__(self, parent)
        # h = self.fontMetrics().height()
        # self.verticalHeader().setDefaultSectionSize(3*h/2)
        self.verticalHeader().hide()
        self.setItemDelegate(CLocItemDelegate(self))
        self._popupMenu = None
        self._actEditDayNomenclatureExpense = None
        self._copiedDayData = None

    def contextMenuEvent(self, event): # event: QContextMenuEvent
        if self._popupMenu:
            self._popupMenu.exec_(event.globalPos())
            event.accept()
        else:
            event.ignore()

    def createPopupMenu(self):
        menu = QtGui.QMenu(self)
        menu.setObjectName('popupMenu')
        self.setPopupMenu(menu)
        return self._popupMenu

    def setPopupMenu(self, menu):
        if self._popupMenu:
            self.disconnect(self._popupMenu, SIGNAL('aboutToShow()'), self.on_popupMenu_aboutToShow)
        self._popupMenu = menu
        if menu:
            self.connect(menu, SIGNAL('aboutToShow()'), self.on_popupMenu_aboutToShow)

    def on_popupMenu_aboutToShow(self):
        index = self.currentIndex()
        model = self.model()

        exists = model.existsByIndex(index)
        isDone = model.existsDoneByIndex(index)

        editable = exists and not isDone

        self._actDeleteDayNomenclatureExpense.setEnabled(editable)
        self._actEditDayNomenclatureExpense.setEnabled(editable)
        self._actPasteDayNomenclatureExpense.setEnabled(editable)

        self.emit(SIGNAL('popupMenuAboutToShow()'))

    def addDeleteDays(self):
        if self._popupMenu is None:
            self.createPopupMenu()

        self._actDeleteDayNomenclatureExpense = QtGui.QAction(u'Удалить', self)
        self._popupMenu.addAction(self._actDeleteDayNomenclatureExpense)
        self.connect(self._actDeleteDayNomenclatureExpense, SIGNAL('triggered()'), self.on_deleteDays)

    def on_deleteDays(self):
        rowsColumns = {}
        for index in self.selectedIndexes():
            rowsColumns.setdefault(index.row(), []).append(index.column())

        for row, columns in rowsColumns.items():
            self.model().deleteDays(row, columns)
        self.model().emitAllChanged()


    def addEditDay(self):
        if self._popupMenu is None:
            self.createPopupMenu()

        self._actEditDayNomenclatureExpense = QtGui.QAction(u'Редактировать', self)
        self._popupMenu.addAction(self._actEditDayNomenclatureExpense)
        self.connect(self._actEditDayNomenclatureExpense, SIGNAL('triggered()'), self.on_editDayNomenclatureExpense)

    def addCopyDay(self):
        if self._popupMenu is None:
            self.createPopupMenu()

        self._actCopyDayNomenclatureExpense = QtGui.QAction(u'Копировать', self)
        self._popupMenu.addAction(self._actCopyDayNomenclatureExpense)
        self.connect(self._actCopyDayNomenclatureExpense, SIGNAL('triggered()'), self.on_copyDayNomenclatureExpense)

    def addPasteDay(self):
        if self._popupMenu is None:
            self.createPopupMenu()

        self._actPasteDayNomenclatureExpense = QtGui.QAction(u'Вставить', self)
        self._popupMenu.addAction(self._actPasteDayNomenclatureExpense)
        self.connect(self._actPasteDayNomenclatureExpense, SIGNAL('triggered()'), self.on_pasteDayNomenclatureExpense)

    def on_copyDayNomenclatureExpense(self):
        index = self.currentIndex()
        if not index.isValid():
            return

        row = index.row()
        column = index.column()
        data, notEditableData = self.model().getGroupDayDataToEdit(row, column)
        self._copiedDayData = {row: ((notEditableData or []) + (data or []))}

    def on_pasteDayNomenclatureExpense(self):
        if not self._copiedDayData:
            return

        rowsColumns = {}
        for index in self.selectedIndexes():
            if index.row() not in self._copiedDayData:
                continue
            rowsColumns.setdefault(index.row(), []).append(index.column())

        model = self.model()

        for row, data in self._copiedDayData.items():
            for column in rowsColumns[row]:
                model.setGroupDayNomenclatureExpenseData(row, column, data)
        self.model().emitAllChanged()

    def on_editDayNomenclatureExpense(self):
        index = self.currentIndex()
        if not index.isValid():
            return
        row = index.row()
        column = index.column()
        result = self.model().getGroupDayDataToEdit(row, column)
        if not result:
            return
        data, notEditableData = result
        date = self.model().getDateByColumn(column)
        dosageUnitName = self.model().getDosageUnitName(row)
        dialog = CNomenclatureExpenseDayDialog(self, date, dosageUnitName=dosageUnitName)
        dialog.load(data, notEditableData)
        dialog.exec_()
        if dialog.nomenclatureExpenseResult:
            self.model().setGroupDayNomenclatureExpenseData(row, column, dialog.nomenclatureExpenseResult)


    def loadPreferences(self, preferences):
        model = self.model()
        self.horizontalHeader().setStretchLastSection(True)
        charWidth = self.fontMetrics().width('A0')/2
        cols = model.cols()
        i = 0
        for col in cols:
            width = forceInt(getPref(preferences, col.key, col.width*charWidth))
            if width:
                self.setColumnWidth(i, width)
            i += 1

    def savePreferences(self):
        preferences = {}
        model = self.model()
        cols = model.cols()
        i = 0
        for col in cols:
            width = self.columnWidth(i)
            setPref(preferences, col.key, QtCore.QVariant(width))
            i += 1
        return preferences