# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import sip
from PyQt4 import QtGui
from PyQt4.QtCore import Qt, pyqtSignature, SIGNAL, QDate, QString, QVariant

from library.DialogBase import CConstructHelperMixin
from library.database   import CTableRecordCache
from library.InDocTable import CInDocTableModel, CBoolInDocTableCol, CRBInDocTableCol, CDateInDocTableCol, CEnumInDocTableCol, CInDocTableCol, CTextInDocTableCol
from library.Utils      import forceBool, forceDate, forceInt, forceRef, forceString, forceStringEx, formatSex, pyDate, toVariant
from Registry.Utils     import getRightEditTempInvalid, getClientMiniInfo
from Events.TempInvalidEditDialog import CTempInvalidCreateDialog, CTempInvalidEditDialog, titleList, getTempInvalidIdOpen
from Events.TempInvalidInfo import CTempInvalidInfo, CTempInvalidReasonInfo, CTempInvalidExtraReasonInfo, CTempInvalidDocTypeInfo, CTempInvalidPeriodInfoList

from Events.Ui_TempInvalid import Ui_grpTempInvalid


class CTempInvalid(QtGui.QGroupBox, CConstructHelperMixin, Ui_grpTempInvalid):
    def __init__(self, parent=None):
        QtGui.QGroupBox.__init__(self, parent)
        self.eventEditor = None
        self.addObject('modelTempInvalidPrivate', CTempInvalidPrivateModel(self))
        self.addObject('modelTempInvalidPatronage', CTempInvalidPatronageModel(self))
        self.setupUi(self)
        self.tblTempInvalidPrivate.setModel(self.modelTempInvalidPrivate)
        self.tblTempInvalidPatronage.setModel(self.modelTempInvalidPatronage)
        self.addPopupEditTempInvalidPrivate()
        self.addPopupEditTempInvalidPatronage()
        self.connect(self.tblTempInvalidPrivate._popupMenu, SIGNAL('aboutToShow()'), self.on_popupMenuTempInvalidPrivate_aboutToShow)
        self.connect(self.tblTempInvalidPatronage._popupMenu, SIGNAL('aboutToShow()'), self.on_popupMenuTempInvalidPatronage_aboutToShow)


    def on_popupMenuTempInvalidPrivate_aboutToShow(self):
        table = self.tblTempInvalidPrivate
        model = self.modelTempInvalidPrivate
        self.actEditTempInvalidPrivate.setEnabled(bool(self.popupMenuTempInvalidId(table, model)))


    def on_popupMenuTempInvalidPatronage_aboutToShow(self):
        table = self.tblTempInvalidPatronage
        model = self.modelTempInvalidPatronage
        self.actEditTempInvalidPatronage.setEnabled(bool(self.popupMenuTempInvalidId(table, model)))


    def popupMenuTempInvalidId(self, table, model):
        table.on_popupMenu_aboutToShow()
        row = table.currentIndex().row()
        rowCount = model.realRowCount() if hasattr(model, 'realRowCount') else model.rowCount()
        record = model._items[row] if 0 <= row < len(model._items) else None
        return forceRef(record.value('id')) if record else None


    def addPopupEditTempInvalidPrivate(self):
        if self.tblTempInvalidPrivate._popupMenu is None:
            self.tblTempInvalidPrivate.createPopupMenu()
        self.actEditTempInvalidPrivate = QtGui.QAction(u'Редактировать (F4)', self)
        self.actEditTempInvalidPrivate.setObjectName('actEditTempInvalidPrivate')
        self.tblTempInvalidPrivate._popupMenu.addAction(self.actEditTempInvalidPrivate)
        self.connect(self.actEditTempInvalidPrivate, SIGNAL('triggered()'), self.on_actEditTempInvalidPrivate_triggered)
        self.addObject('qshcEditTempInvalidPrivate', QtGui.QShortcut('F4', self.tblTempInvalidPrivate, self.on_actEditTempInvalidPrivate_triggered))
        self.qshcEditTempInvalidPrivate.setContext(Qt.WidgetShortcut)


    def addPopupEditTempInvalidPatronage(self):
        if self.tblTempInvalidPatronage._popupMenu is None:
            self.tblTempInvalidPatronage.createPopupMenu()
        self.actEditTempInvalidPatronage = QtGui.QAction(u'Редактировать (F4)', self)
        self.actEditTempInvalidPatronage.setObjectName('actEditTempInvalidPatronage')
        self.tblTempInvalidPatronage._popupMenu.addAction(self.actEditTempInvalidPatronage)
        self.connect(self.actEditTempInvalidPatronage, SIGNAL('triggered()'), self.on_actEditTempInvalidPatronage_triggered)
        self.addObject('qshcEditTempInvalidPatronage', QtGui.QShortcut('F4', self.tblTempInvalidPatronage, self.on_actEditTempInvalidPatronage_triggered))
        self.qshcEditTempInvalidPatronage.setContext(Qt.WidgetShortcut)


    @pyqtSignature('')
    def on_actEditTempInvalidPrivate_triggered(self):
        currentItem = self.tblTempInvalidPrivate.currentItem()
        tempInvalidId = forceRef(currentItem.value('id')) if currentItem else None
        self.openTempInvalid(tempInvalidId)


    @pyqtSignature('')
    def on_actEditTempInvalidPatronage_triggered(self):
        currentItem = self.tblTempInvalidPatronage.currentItem()
        tempInvalidId = forceRef(currentItem.value('id')) if currentItem else None
        self.openTempInvalid(tempInvalidId)


    def protectFromEdit(self, isProtected):
        widgets = [self.tblTempInvalidPrivate.model(),
                   self.tblTempInvalidPatronage.model()
                  ]
        for widget in widgets:
            widget.setReadOnly(isProtected)


    def destroy(self):
        self.tblTempInvalidPrivate.setModel(None)
        del self.modelTempInvalidPrivate
        self.tblTempInvalidPatronage.setModel(None)
        del self.modelTempInvalidPatronage


    def setEventEditor(self, eventEditor):
        self.eventEditor = eventEditor
        self.modelTempInvalidPrivate.setEventEditor(eventEditor)
        self.modelTempInvalidPatronage.setEventEditor(eventEditor)


    def setType(self, type_, docCode=None):
        self.type_ = type_
        self.docCode = docCode
        self.modelTempInvalidPrivate.setType(self.type_, self.docCode)
        self.modelTempInvalidPatronage.setType(self.type_, self.docCode)


    def pickupTempInvalid(self):
        self.modelTempInvalidPrivate.loadItems(self.eventEditor.clientId)
        self.modelTempInvalidPatronage.loadItems(self.eventEditor.clientId)



    @pyqtSignature('QModelIndex')
    def on_tblTempInvalidPrivate_doubleClicked(self, index):
        self.editTempInvalid(self.tblTempInvalidPrivate, 0)


    @pyqtSignature('QModelIndex')
    def on_tblTempInvalidPatronage_doubleClicked(self, index):
        self.editTempInvalid(self.tblTempInvalidPatronage, 1)


    def openTempInvalid(self, tempInvalidId):
        if tempInvalidId and getRightEditTempInvalid(tempInvalidId):
            db = QtGui.qApp.db
            clientId = self.eventEditor.clientId
            clientCache = CTableRecordCache(db, db.forceTable('Client'), u'*', capacity=None)
            dialog = CTempInvalidEditDialog(self, clientCache)
            clientInfo = getClientMiniInfo(clientId)
            dialog.setWindowTitle(titleList[self.type_] + u': ' + clientInfo)
            dialog.setType(self.type_, self.docCode)
            dialog.load(tempInvalidId)
            try:
                if dialog.exec_():
                    tempInvalidNewId = dialog.itemId()
                    self.modelTempInvalidPrivate.loadItems(clientId)
                    self.modelTempInvalidPatronage.loadItems(clientId)
            finally:
                sip.delete(dialog)
                del dialog


    def editTempInvalid(self, table, type = 0):
        currentItem = table.currentItem()
        tempInvalidId = forceRef(currentItem.value('id')) if currentItem else None
        self.getEditorTempInvalid(tempInvalidId, type)


    def getEditorTempInvalid(self, tempInvalidId, type = 0):
        if tempInvalidId:
            self.openTempInvalid(tempInvalidId)
        else:
            self.createTempInvalid(type)


    def newTempInvalid(self):
        tempInvalidId = getTempInvalidIdOpen(self.eventEditor.clientId, self.type_, self.docCode)
        self.getEditorTempInvalid(tempInvalidId)


    def createTempInvalid(self, type = 0):
        clientId = self.eventEditor.clientId
        if clientId:
            MKBList = self.eventEditor.getFinalDiagnosisMKB() if hasattr(self.eventEditor, 'getFinalDiagnosisMKB') else u''
            MKB = MKBList[0] if len(MKBList) > 1 else MKBList
            db = QtGui.qApp.db
            clientCache = CTableRecordCache(db, db.forceTable('Client'), u'*', capacity=None)
            dialog = CTempInvalidCreateDialog(self, clientId, clientCache)
            dialog.setType(self.type_, self.docCode)
            clientInfo = getClientMiniInfo(clientId)
            dialog.setWindowTitle(titleList[self.type_] + u': ' + clientInfo)
            dialog.createTempInvalidDocument(MKB, type = type)
            try:
                if dialog.exec_():
                   self.modelTempInvalidPrivate.loadItems(clientId)
                   self.modelTempInvalidPatronage.loadItems(clientId)
            finally:
                sip.delete(dialog)
                del dialog


    def getTempInvalidInfo(self, context):
        result = context.getInstance(CTempInvalidInfo, None)
        if result._ok:
            return result
#        if self.isChecked():
#            result._doctype = context.getInstance(CTempInvalidDocTypeInfo,  self.cmbTempInvalidDoctype.value())
#            result._reason  = context.getInstance(CTempInvalidReasonInfo,  self.cmbTempInvalidReason.value())
#            result._extraReason  = context.getInstance(CTempInvalidExtraReasonInfo, forceRef(self.cmbExtraReason.value()))
#            result._busyness = forceInt(self.cmbBusyness.currentIndex())
#            result._placeWork = forceString(self.edtPlaceWork.text())
#            result._serial  = forceStringEx(self.edtTempInvalidSerial.text())
#            result._number  = forceStringEx(self.edtTempInvalidNumber.text())
#            result._sex     = formatSex(self.cmbTempInvalidOtherSex.currentIndex())
#            result._age     = self.edtTempInvalidOtherAge.value()
#            result._receiver= context.getInstance(CClientInfo, self.cmbReceiver.value())
#            result._duration, result._externalDuration = self.modelTempInvalidPeriods.calcLengths()
#            result._begDate = CDateInfo(self.modelTempInvalidPeriods.begDate())
#            result._endDate = CDateInfo(self.modelTempInvalidPeriods.endDate())
#            result._issueDate = CDateInfo(forceDate(self.edtIssueDate.date()))
#            result._accountPregnancyTo12Weeks = forceInt(self.cmbAccountPregnancyTo12Weeks.currentIndex())
#            MKB, MKBEx = self.eventEditor.getFinalDiagnosisMKB()
#            result._MKB = context.getInstance(CMKBInfo, MKB)
#            result._MKBEx = context.getInstance(CMKBInfo, MKBEx)
#            closed = self.modelTempInvalidPeriods.getTempInvalidClosedStatus()
#            result._closed = closed
#            result._periods = self.modelTempInvalidPeriods.getPeriodsInfo(context)
#            if self.prevTempInvalidId:
#                result._prev = context.getInstance(CTempInvalidInfo, self.prevTempInvalidId)
#            else:
#                result._prev = None
#            result._ok = True
#        else:
#            result._ok = False
        result._loaded = True
        return result


class CTempInvalidModel(CInDocTableModel):
    class CLocMKBColumn(CInDocTableCol):
        def __init__(self, title, fieldName, designationChain, width, **params):
            CInDocTableCol.__init__(self, title, fieldName, width, **params)
            tableName, fieldName = designationChain
            self.MKBCache = CTableRecordCache(QtGui.qApp.db, tableName, fieldName)

        def toString(self, val, record):
            diagnosisId  = forceRef(val)
            if diagnosisId and self.MKBCache:
                MKBRecord = self.MKBCache.get(diagnosisId) if diagnosisId else None
                if MKBRecord:
                    return toVariant(forceString(MKBRecord.value('MKB')))
            return QVariant()

        def invalidateRecordsCache(self):
            self.MKBCache.invalidate()


    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'TempInvalid', 'id', 'client_id', parent)
        self.addCol(CBoolInDocTableCol(u'СтСт',                       'insuranceOfficeMark',                                                                 6)).setReadOnly(True)
        self.addCol(CRBInDocTableCol(  u'Тип',                        'doctype_id',                        15, 'rbTempInvalidDocument'                        )).setReadOnly(True)
        self.addCol(CRBInDocTableCol(  u'Причина нетрудоспособности', 'tempInvalidReason_id',              15, 'rbTempInvalidReason'                          )).setReadOnly(True)
        self.addCol(CDateInDocTableCol(u'Дата начала ВУТ',            'caseBegDate',                                                                        10)).setReadOnly(True)
        self.addCol(CDateInDocTableCol(u'Начало',                     'begDate',                                                                            10)).setReadOnly(True)
        self.addCol(CDateInDocTableCol(u'Окончание',                  'endDate',                                                                            10)).setReadOnly(True)
        self.addCol(CRBInDocTableCol(  u'Врач',                       'person_id',                         15, 'vrbPersonWithSpeciality'                      )).setReadOnly(True)
        self.addCol(CTempInvalidModel.CLocMKBColumn(u'МКБ',           'diagnosis_id',                          ('Diagnosis', 'MKB'),                         6)).setReadOnly(True)
        self.addCol(CEnumInDocTableCol(u'Состояние',                  'closed',                             4, [u'открыт', u'закрыт', u'продлён', u'передан'] )).setReadOnly(True)
        self.addCol(CInDocTableCol(u'Длительность',                   'duration',                                                                            5)).setReadOnly(True)
        self.addCol(CDateInDocTableCol(u'В стационаре "с"',           'begDateStationary',                                                                  12)).setReadOnly(True)
        self.addCol(CDateInDocTableCol(u'В стационаре "по"',          'endDateStationary',                                                                  12)).setReadOnly(True)
        self.addCol(CRBInDocTableCol(u'Нарушение режима',             'break_id',                          15, 'rbTempInvalidBreak'                           )).setReadOnly(True)
        self.addCol(CDateInDocTableCol(u'Дата нарушения режима',      'breakDate',                                                                          12)).setReadOnly(True)
        self.addCol(CRBInDocTableCol(u'Результат',                    'result_id',                         15, 'rbTempInvalidResult'                          )).setReadOnly(True)
        self.addCol(CDateInDocTableCol(u'Дата результата - Приступить к работе', 'resultDate',                                                              12)).setReadOnly(True)
        self.addCol(CDateInDocTableCol(u'Дата результата - Иное',                'resultOtherwiseDate',                                                     12)).setReadOnly(True)
        self.addCol(CInDocTableCol(u'Номер путевки',                             'numberPermit',                                                            12)).setReadOnly(True)
        self.addCol(CDateInDocTableCol(u'Дата начала путевки',                   'begDatePermit',                                                           12)).setReadOnly(True)
        self.addCol(CDateInDocTableCol(u'Дата окончания путевки',                'endDatePermit',                                                           12)).setReadOnly(True)
        self.addCol(CRBInDocTableCol(u'Инвалидность',                            'disability_id',          15, 'rbTempInvalidRegime'                          )).setReadOnly(True)
        self.eventEditor = None
        self.type_ = None
        self.docCode = None
        self.docId = None
        self.filterLoc = ''
        self.filierDoc = ''
        self.readOnly = False


    def setReadOnly(self, value):
        self.readOnly = value


    def flags(self, index):
        if self.readOnly:
           return Qt.ItemIsSelectable | Qt.ItemIsEnabled
        return CInDocTableModel.flags(self, index)


    def setType(self, type_, docCode=None):
        self.type_ = type_
        self.docCode = docCode
        self.docId = forceRef(QtGui.qApp.db.translate('rbTempInvalidDocument', 'code', self.docCode, 'id')) if self.docCode else None
        self.filterLoc = u'type=%d'%self.type_
        self.filierDoc = (self.filterLoc+u' AND code=\'%s\''%self.docCode) if self.docCode else self.filterLoc


    def setEventEditor(self, eventEditor):
        self.eventEditor = eventEditor


class CTempInvalidPrivateModel(CTempInvalidModel):
    def loadItems(self, clientId):
        items = []
        self.clientId = clientId
        db = QtGui.qApp.db
        tableTempInvalid = db.table('TempInvalid')
        tableRBTempInvalidResult = db.table('rbTempInvalidResult')
        cond = [tableTempInvalid['deleted'].eq(0),
                tableTempInvalid['client_id'].eq(self.clientId),
                tableTempInvalid['closed'].eq(0),
                tableRBTempInvalidResult['closed'].eq(0),
                tableTempInvalid['type'].eq(self.type_),
                tableRBTempInvalidResult['able'].ne(1)
                ]
        if self.docCode:
            tableRBTempInvalidDocument = db.table('rbTempInvalidDocument')
            cond.append(tableRBTempInvalidDocument['code'].eq(self.docCode))
            table = tableTempInvalid.leftJoin(tableRBTempInvalidDocument, tableTempInvalid['doctype_id'].eq(tableRBTempInvalidDocument['id']))
        else:
            table = tableTempInvalid
        table = table.leftJoin(tableRBTempInvalidResult, tableRBTempInvalidResult['id'].eq(tableTempInvalid['result_id']))
        record = db.getRecordEx(table, 'TempInvalid.*', cond, 'TempInvalid.begDate DESC')
        if record:
            items.append(record)
            tempInvalidId = forceRef(record.value('prev_id'))
            if tempInvalidId:
                id = tempInvalidId
                while id:
                    recordChild = db.getRecordEx(tableTempInvalid, 'TempInvalid.*', [tableTempInvalid['id'].eq(id), tableTempInvalid['deleted'].eq(0)], tableTempInvalid['begDate'].name())
                    if recordChild:
                        id = forceRef(recordChild.value('prev_id'))
                        items.append(recordChild)
                    else:
                        id = None
        self.setItems(items)
        self.reset()


class CTempInvalidPatronageModel(CTempInvalidModel):
    def loadItems(self, clientId):
        items = []
        self.clientId = clientId
        db = QtGui.qApp.db
        tableTempInvalid = db.table('TempInvalid')
        tableRBTempInvalidResult = db.table('rbTempInvalidResult')
        tableTempInvalidDocument = db.table('TempInvalidDocument')
        cond = [tableTempInvalid['deleted'].eq(0),
                tableTempInvalid['client_id'].ne(self.clientId),
                tableTempInvalid['closed'].eq(0),
                tableRBTempInvalidResult['closed'].eq(0),
                tableTempInvalid['type'].eq(self.type_),
                tableRBTempInvalidResult['able'].ne(1),
                db.joinOr([tableTempInvalidDocument['clientPrimum_id'].eq(self.clientId), tableTempInvalidDocument['clientSecond_id'].eq(self.clientId)])
                ]
        if self.docCode:
            tableRBTempInvalidDocument = db.table('rbTempInvalidDocument')
            cond.append(tableRBTempInvalidDocument['code'].eq(self.docCode))
            table = tableTempInvalid.leftJoin(tableRBTempInvalidDocument, tableTempInvalid['doctype_id'].eq(tableRBTempInvalidDocument['id']))
        else:
            table = tableTempInvalid
        table = table.leftJoin(tableRBTempInvalidResult, tableRBTempInvalidResult['id'].eq(tableTempInvalid['result_id']))
        table = table.leftJoin(tableTempInvalidDocument, db.joinAnd([tableTempInvalidDocument['master_id'].eq(tableTempInvalid['id']), tableTempInvalidDocument['deleted'].eq(0)]))
        record = db.getRecordEx(table, 'TempInvalid.*', cond, 'TempInvalid.begDate DESC')
        if record:
            items.append(record)
            tempInvalidId = forceRef(record.value('prev_id'))
            if tempInvalidId:
                id = tempInvalidId
                while id:
                    recordChild = db.getRecordEx(tableTempInvalid, 'TempInvalid.*', [tableTempInvalid['id'].eq(id), tableTempInvalid['deleted'].eq(0)], tableTempInvalid['begDate'].name())
                    if recordChild:
                        id = forceRef(recordChild.value('prev_id'))
                        items.append(recordChild)
                    else:
                        id = None
        self.setItems(items)
        self.reset()

