# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import json, sip

from PyQt4 import QtGui, QtSql
from PyQt4.QtCore import Qt, SIGNAL, QString, QVariant, QSize, QDate, QDateTime, QEvent

from library.Utils                         import copyFields, forceBool, forceRef, forceStringEx, toVariant, forceString, forceDateTime, forceDate
from library.InDocTable                    import CInDocTableView, CRecordListModel, CLocItemDelegate
from library.ROComboBox                    import CROEditableComboBox
from Registry.ClientRelationSimpleComboBox import CClientRelationSimpleComboBox
from Registry.Utils                        import getRightEditTempInvalid, deleteTempInvalidDocument
from Events.Action                         import CAction, CActionTypeCache
from Events.Utils                          import getActionTypeIdListByFlatCode, checkTissueJournalStatusByActions
from Events.ActionTypeDialog               import CActionTypeDialogTableModel
from Events.ActionCreateDialog             import CActionCreateDialog
from Users.Rights                          import urRegTabEditExpertMC


class CClientRelationDelegate(QtGui.QItemDelegate):
    def __init__(self, parent=None):
        QtGui.QItemDelegate.__init__(self, parent)


    def commit(self):
        editor = self.sender()
        self.emit(SIGNAL('commitData(QWidget *)'), editor)


    def commitAndCloseEditor(self):
        editor = self.sender()
        self.emit(SIGNAL('commitData(QWidget *)'), editor)
        self.emit(SIGNAL('closeEditor(QWidget *,QAbstractItemDelegate::EndEditHint)'), editor, QtGui.QAbstractItemDelegate.NoHint)


    def createEditor(self, parent, option, index):
        model = index.model()
        clientId = model.getTempInvalidClientId()
        editor = CClientRelationSimpleComboBox(parent, clientId)
        self.connect(editor, SIGNAL('commit()'), self.commit)
        self.connect(editor, SIGNAL('editingFinished()'), self.commitAndCloseEditor)
        return editor


    def alignment(self):
        return QVariant(Qt.AlignLeft + Qt.AlignTop)


    def setEditorData(self, editor, index):
        model = index.model()
        value = model.data(index, Qt.EditRole)
        editor.setValue(forceRef(value))


    def setModelData(self, editor, model, index):
        model = index.model()
        value = editor.value()
        model.setData(index, toVariant(value))


class CPlaceWorkItemDelegate(CLocItemDelegate):
    def __init__(self, parent):
        CLocItemDelegate.__init__(self, parent)


    def eventFilter(self, object, event):
        def editorIsEmpty():
            if isinstance(self.editor, QtGui.QLineEdit):
                return self.editor.text() == ''
            if  isinstance(self.editor, QtGui.QComboBox):
                return self.editor.currentIndex() == 0
            if  isinstance(self.editor, CDateEdit):
                return not self.editor.dateIsChanged()
            if  isinstance(self.editor, QtGui.QDateEdit):
                return not self.editor.date().isValid()
            return False
        if event.type() == QEvent.KeyPress:
            if event.key() == Qt.Key_Tab or event.key() == Qt.Key_Backtab:
                if self.editor is not None:
                    self.editor.keyPressEvent(event)
                self.parent().keyPressEvent(event)
                return True
            elif event.key() == Qt.Key_Return:
                return True
        return QtGui.QItemDelegate.eventFilter(self, object, event)


class CTempInvalidDocumentsInDocTableView(CInDocTableView):

    Col_IssueDate      = 0
    Col_Serial         = 1
    Col_Number         = 2
    Col_duplicate      = 3
    Col_duplicateReason= 4
    Col_Electronic     = 5
    Col_Busyness       = 6
    Col_PlaceWork      = 7
    Col_PrevNumber     = 8
    Col_PrevId         = 9
    Col_IssuePersonId  = 10
    Col_ChairPersonId  = 11
    Col_ClientPrimumId = 12
    Col_ClientSecondId = 13
    Col_LastId         = 14
    Col_Note           = 15


    def __init__(self, parent):
        CInDocTableView.__init__(self, parent)
        self.clientRelationDelegate = CClientRelationDelegate(self)
        self.setItemDelegateForColumn(CTempInvalidDocumentsInDocTableView.Col_ClientPrimumId, self.clientRelationDelegate)
        self.setItemDelegateForColumn(CTempInvalidDocumentsInDocTableView.Col_ClientSecondId, self.clientRelationDelegate)
        self.placeWorkItemDelegate = CPlaceWorkItemDelegate(self)
        self.setItemDelegateForColumn(CTempInvalidDocumentsInDocTableView.Col_PlaceWork, self.placeWorkItemDelegate)
        self.resizeRowsToContents()
        self._CInDocTableView__actDirectionMC = None


    def enableColHide(self, column):
        header = self.horizontalHeader()
        header.hideSection(column)


    def showColHide(self, column):
        header = self.horizontalHeader()
        if header.isSectionHidden(column):
            header.showSection(column)


    def addPopupDuplicateCurrentRow(self):
        if self._popupMenu is None:
            self.createPopupMenu()
        self.__actDuplicateCurrentRow = QtGui.QAction(u'Создать дубликат документа', self)
        self.__actDuplicateCurrentRow.setObjectName('actDuplicateCurrentRow')
        self._popupMenu.addAction(self.__actDuplicateCurrentRow)
        self.connect(self.__actDuplicateCurrentRow, SIGNAL('triggered()'), self.on_duplicateCurrentRow)


    def addPopupDirectionMC(self):
        if self._popupMenu is None:
            self.createPopupMenu()
        self._CInDocTableView__actDirectionMC = QtGui.QAction(u'Создать направление на ВК', self)
        self._CInDocTableView__actDirectionMC.setObjectName('actDirectionMC')
        self._popupMenu.addAction(self._CInDocTableView__actDirectionMC)
        self.connect(self._CInDocTableView__actDirectionMC, SIGNAL('triggered()'), self.on_directionMC)


    def on_directionMC(self):
        eventEditor = self.model().eventEditor
        if eventEditor and eventEditor.preCreateDirectionMC():
            actions = []
            db = QtGui.qApp.db
            clientId = self.model().getTempInvalidClientId()
            items = self.model().items()
            selectRowList = self.getSelectedRows()
            if selectRowList:
                selectRowList.sort()
                actionTypeIdList = getActionTypeIdListByFlatCode(u'inspection_disability%')
                if actionTypeIdList:
                    actionTypeId = None
                    if len(actionTypeIdList) > 1:
                        try:
                            dialog = CActionTypeDialogTableModel(self, actionTypeIdList)
                            if dialog.exec_():
                                actionTypeId = dialog.currentItemId()
                        finally:
                            dialog.destroy()
                            sip.delete(dialog)
                            del dialog
                    else:
                        actionTypeId = actionTypeIdList[0]
                    if actionTypeId and selectRowList:
                        tableAction = db.table('Action')
                        action = None
                        dialog = CActionCreateDialog(self)
                        try:
                            actionType = CActionTypeCache.getById(actionTypeId)
                            defaultStatus = actionType.defaultStatus
                            defaultOrgId = actionType.defaultOrgId
                            newRecord = tableAction.newRecord()
                            newRecord.setValue('createDatetime',  toVariant(QDateTime.currentDateTime()))
                            newRecord.setValue('createPerson_id', toVariant(QtGui.qApp.userId))
                            newRecord.setValue('modifyDatetime',  toVariant(QDateTime.currentDateTime()))
                            newRecord.setValue('modifyPerson_id', toVariant(QtGui.qApp.userId))
                            newRecord.setValue('actionType_id',   toVariant(actionTypeId))
                            newRecord.setValue('status',          toVariant(defaultStatus))
                            newRecord.setValue('begDate',         toVariant(QDateTime.currentDateTime()))
                            newRecord.setValue('directionDate',   toVariant(QDateTime.currentDateTime()))
                            newRecord.setValue('org_id',          toVariant(defaultOrgId if defaultOrgId else QtGui.qApp.currentOrgId()))
                            newRecord.setValue('setPerson_id',    toVariant(QtGui.qApp.userId))
                            begDateLastPeriod = eventEditor.begDateLastPeriod()
                            newRecord.setValue('plannedEndDate',   toVariant(begDateLastPeriod if (begDateLastPeriod and begDateLastPeriod > QDate.currentDate()) else QDateTime.currentDateTime()))
                            newAction = CAction(record=newRecord)
                            record = items[selectRowList[0]]
                            newAction = self.setNumberVUT(newAction, record)
                            if not newAction:
                                return
                            dialog.load(newAction.getRecord(), newAction, clientId)
                            dialog.btnPrint.setEnabled(False)
                            if dialog.exec_() and dialog.checkDataEntered(secondTry=True):
                                action = dialog.getAction()
                                dialog.btnPrint.setEnabled(True)
                        finally:
                            dialog.destroy()
                            sip.delete(dialog)
                            del dialog
                        if action:
                            actions.append(action)
                            selectRowList.pop(0)
                            for row in selectRowList:
                                record = items[row]
                                newRecord = tableAction.newRecord()
                                copyFields(newRecord, action.getRecord())
                                newRecord.setValue('id', toVariant(None))
                                newRecord.setValue('idx', toVariant(0))
                                newAction = CAction(record=newRecord)
                                newAction.updateByAction(action)
                                newAction = self.setNumberVUT(newAction, record)
                                if not newAction:
                                    return
                                actions.append(newAction)
                            if actions:
                                tableEvent = db.table('Event')
                                tableEventType = db.table('EventType')
                                tempInvalidId = self.model().getTempInvalidId()
                                if tempInvalidId:
                                    queryTable = tableEvent.innerJoin(tableEventType, tableEventType['id'].eq(tableEvent['eventType_id']))
                                    cond = [tableEvent['tempInvalid_id'].eq(tempInvalidId),
                                            tableEventType['context'].like(u'inspection%'),
                                            tableEvent['execDate'].isNull(),
                                            tableEvent['client_id'].eq(clientId),
                                            tableEvent['deleted'].eq(0),
                                            tableEventType['deleted'].eq(0),
                                            ]
                                    recordEvent = db.getRecordEx(queryTable, 'Event.*', cond, u'Event.id DESC')
                                    eventId = forceRef(recordEvent.value('id')) if recordEvent else None
                                    if eventId:
                                        actionIdList = self.saveMedicalCommissionActions(actions, recordEvent, eventId)
                                    else:
                                        recordEventType = db.getRecordEx(tableEventType, [tableEventType['id']], [tableEventType['context'].like(u'inspection%'), tableEventType['deleted'].eq(0)], u'EventType.id')
                                        eventTypeId = forceRef(recordEventType.value('id')) if recordEventType else None
                                        if eventTypeId:
                                            recordEvent = tableEvent.newRecord()
                                            recordEvent.setValue('createDatetime', toVariant(QDateTime.currentDateTime()))
                                            recordEvent.setValue('createPerson_id',toVariant(QtGui.qApp.userId))
                                            recordEvent.setValue('modifyDatetime', toVariant(QDateTime.currentDateTime()))
                                            recordEvent.setValue('modifyPerson_id',toVariant(QtGui.qApp.userId))
                                            recordEvent.setValue('setDate',        toVariant(QDateTime.currentDateTime()))
                                            recordEvent.setValue('eventType_id',   toVariant(eventTypeId))
                                            recordEvent.setValue('client_id',      toVariant(clientId))
                                            recordEvent.setValue('relegatePerson_id', toVariant(QtGui.qApp.userId))
                                            recordEvent.setValue('relegateOrg_id', toVariant(QtGui.qApp.currentOrgId()))
                                            recordEvent.setValue('tempInvalid_id', toVariant(tempInvalidId))
                                            eventId = db.insertRecord(tableEvent, recordEvent)
                                            if eventId:
                                                recordEvent.setValue('id', toVariant(eventId))
                                                actionIdList = self.saveMedicalCommissionActions(actions, recordEvent, eventId)
            eventEditor.medicalCommissionLoadItems()


    def setNumberVUT(self, action, record):
        actionType = action.getType()
        number = forceString(record.value('number'))
        if number:
            if u'Номер ЛН' in actionType._propertiesByName:
                action[u'Номер ЛН'] = number
            else:
                QtGui.QMessageBox.question(self,
                    u'Создание Действия ВК!', u'Отсутствует Свойство "Номер ЛН"! Регистрация направления на ВК невозможна!',
                    QtGui.QMessageBox.Ok,
                    QtGui.QMessageBox.Ok)
                return None
        else:
            QtGui.QMessageBox.question(self,
                u'Создание Действия ВК!', u'Отсутствует номер документа ВУТ! Регистрация направления на ВК невозможна!',
                QtGui.QMessageBox.Ok,
                QtGui.QMessageBox.Ok)
            return None
        return action


    def saveMedicalCommissionActions(self, actions, recordEvent = None, eventId = None):
        actionIdList = []
        try:
            try:
                db = QtGui.qApp.db
                db.transaction()
                for idx, action in enumerate(actions):
                    id = action.save(eventId, idx, checkModifyDate = False)
                    if id:
                        actionIdList.append(id)
                        action.getRecord().setValue('id', toVariant(id))
                        checkTissueJournalStatusByActions([(action.getRecord(), action)])
                        if action.getType().closeEvent and recordEvent:
                            eventExecDate = forceDate(recordEvent.value('execDate'))
                            actionEndDate = forceDateTime(action.getRecord().value('endDate'))
                            if not eventExecDate and actionEndDate:
                                recordEvent.setValue('execDate', QVariant(actionEndDate))
                                recordEvent.setValue('isClosed', QVariant(1))
                                db.updateRecord('Event', recordEvent)
                db.commit()
            except:
                db.rollback()
                raise
        except Exception, e:
            QtGui.qApp.logCurrentException()
            QtGui.QMessageBox.critical( self,
                                        u'',
                                        unicode(e),
                                        QtGui.QMessageBox.Close)
        return actionIdList


    def addPopupDelRow(self):
        if self._popupMenu is None:
            self.createPopupMenu()
        self._CInDocTableView__actDeleteRows = QtGui.QAction(u'Удалить строку', self)
        self._CInDocTableView__actDeleteRows.setObjectName('actDeleteRows')
        self._popupMenu.addAction(self._CInDocTableView__actDeleteRows)
        self.connect(self._CInDocTableView__actDeleteRows, SIGNAL('triggered()'), self.on_deleteCurrentRow)


    def on_deleteCurrentRow(self):
        if QtGui.QMessageBox.question(self,
                u'Удаление документа', u'Вы действительно хотите удалить документ?',
                QtGui.QMessageBox.Yes|QtGui.QMessageBox.No,
                QtGui.QMessageBox.No) == QtGui.QMessageBox.Yes:
            currentRow = self.currentIndex().row()
            items = self.model().items()
            if currentRow < len(items):
                item = items[currentRow]
                tempInvalidId = self.model().getTempInvalidId()
                clientId = self.model().getTempInvalidClientId()
                tempInvalidDocumentId = forceRef(item.value('id'))
                if not tempInvalidId:
                    tempInvalidId = forceRef(item.value('master_id'))
                if tempInvalidId and getRightEditTempInvalid(tempInvalidId):
                    if tempInvalidDocumentId:
                        res1, res2 = QtGui.qApp.call(self, deleteTempInvalidDocument, (self, clientId, tempInvalidId, tempInvalidDocumentId,))
                        if res1 and res2:
                            self.model().removeRow(currentRow)
                    else:
                        self.model().removeRow(currentRow)
                elif not tempInvalidId and not tempInvalidDocumentId:
                    self.model().removeRow(currentRow)


    def on_duplicateCurrentRow(self):
        currentRow = self.currentIndex().row()
        items = self.model().items()
        if currentRow < len(items):
            item = items[currentRow]
            newRecord = self.model().getEmptyRecord()
            copyFields(newRecord, item)
            newRecord.setValue('id', toVariant(None))
            newRecord.setValue('issueDate', toVariant(QDate.currentDate()))
            newRecord.setValue('serial', toVariant(None))
            newRecord.setValue('number', toVariant(None))
            newRecord.setValue('duplicate', toVariant(1))
#            userId = QtGui.qApp.userId
#            self.chairUser = forceBool(QtGui.qApp.db.translate('Person', 'id', userId, 'chairPerson')) if userId else False
#            if self.chairUser:
#                newRecord.setValue('chairPerson_id', toVariant(userId))
            newRecord.setValue('person_id', toVariant(QtGui.qApp.userId))
            newRecord.setValue('execPerson_id', toVariant(QtGui.qApp.userId))
            newRecord.setValue('electronic', toVariant(0))
            newRecord.setValue('prevDuplicate_id', toVariant(forceRef(item.value('id'))))
            newRecord.setValue('note', toVariant(None))
            newRecord.setValue(self.model()._idFieldName, toVariant(None))
            self.model().insertRecord(currentRow+1, newRecord)


    def on_popupMenu_aboutToShow(self):
        CInDocTableView.on_popupMenu_aboutToShow(self)
        model = self.model()
        row = self.currentIndex().row()
        rowCount = model.realRowCount() if hasattr(model, 'realRowCount') else model.rowCount()
        if self._CInDocTableView__actDeleteRows:
            self._CInDocTableView__actDeleteRows.setText(u'Удалить текущую строку')
            self._CInDocTableView__actDeleteRows.setEnabled(0<=row<rowCount)
        if self._CInDocTableView__actDuplicateCurrentRow:
            isEnabled = False
            if 0<=row<rowCount:
                items = model.items()
                if not forceBool(items[row].value('duplicate')):
                   isEnabled = True
            self._CInDocTableView__actDuplicateCurrentRow.setEnabled(isEnabled)
        if self._CInDocTableView__actDirectionMC:
            self._CInDocTableView__actDirectionMC.setEnabled(0<=row<rowCount and QtGui.qApp.userHasRight(urRegTabEditExpertMC))

