# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
import sip

from PyQt4 import QtGui, QtSql
from PyQt4.QtCore import Qt, QDate, QString, QVariant, pyqtSignature, SIGNAL, SLOT


from library.DialogBase         import CDialogBase
from library.RecordLock         import CRecordLockMixin
from library.database           import CTableRecordCache
from library.crbcombobox        import CRBComboBox
from library.ICDUtils           import MKBwithoutSubclassification
from library.interchange        import getCheckBoxValue, getComboBoxValue, getLineEditValue, getRBComboBoxValue, getSpinBoxValue, setDateEditValue, getDateEditValue, setCheckBoxValue, setComboBoxValue, setLineEditValue, setRBComboBoxValue, setSpinBoxValue
from library.ItemsListDialog    import CwithoutPreferenceItemEditorBaseDialog
from library.PrintInfo          import CInfoContext, CDateInfo
from library.PrintTemplates     import getPrintButton, applyTemplate
from library.ICDInDocTableCol   import CICDExInDocTableCol
from library.Utils              import copyFields, forceBool, forceDate, forceInt, forceRef, forceString, forceStringEx, formatDate, formatSex, pyDate, toVariant, formatName, trim
from library.InDocTable         import CBoolInDocTableCol, CDateInDocTableCol, CDateTimeInDocTableCol, CEnumInDocTableCol, CInDocTableCol, CSelectStrInDocTableCol, CInDocTableModel, CIntInDocTableCol, CRBInDocTableCol
from library.ROComboBox         import CROEditableComboBox
from Blank.BlankComboBox        import CBlankComboBox
from Blank.BlankComboBoxPopup   import codeToTextForBlank
from Events.Action              import CAction
from Events.ActionStatus        import CActionStatus
from Events.EventEditDialog     import CEventEditDialog
from Events.MKBInfo             import CMKBInfo
from Events.TempInvalidInfo     import CTempInvalidInfo, CTempInvalidReasonInfo, CTempInvalidExtraReasonInfo, CTempInvalidDocTypeInfo, CTempInvalidPeriodInfoList, CTempInvalidDocumentItemInfoList, CTempInvalidResultInfo, CTempInvalidBreakInfo
from Events.EventInfo           import CEventInfoList
from Events.ActionTypeComboBox  import CActionTypeTableCol
from Events.Utils               import getAvailableCharacterIdByMKB, getDiagnosisId2, specifyDiagnosis, getActionTypeIdListByFlatCode
from Registry.Utils             import CClientInfo, getClientWork, getClientMiniInfo
from Registry.ClientEditDialog  import CClientEditDialog
from Registry.ClientRelationsEditDialog import CClientRelationsEditDialog
from Orgs.Utils                 import getOrganisationShortName
from Orgs.Orgs                  import selectOrganisation
from Users.Rights               import urAdmin, urRefEditMedTempInvalidExpertKAK, urRegWriteInsurOfficeMark, urRegTabWriteRegistry, urRegTabReadRegistry

from Events.Ui_TempInvalidEditDialog   import Ui_TempInvalidEditDialog

titleList = [u'ВУТ', u'Инвалидность', u'Ограничения жизнедеятельности']

class CTempInvalidEditDialog(CwithoutPreferenceItemEditorBaseDialog, Ui_TempInvalidEditDialog):

    TempInvalid      = 0
    Disability       = 1
    VitalRestriction = 2
    MSI              = 3

    MedicalCertificate = '1'

    def __init__(self,  parent, clientCache):
        CwithoutPreferenceItemEditorBaseDialog.__init__(self, parent, 'TempInvalid')
        self.addObject('modelPeriods', CTempInvalidPeriodModel(self))
        self.addObject('modelDocuments', CTempInvalidDocumentsModel(self, clientCache))
        self.addObject('modelMedicalCommission', CMedicalCommissionModel(self))
        self.addObject('modelMedicalCommissionMSI', CMedicalCommissionMSIModel(self))
        self.addObject('btnPrint', getPrintButton(self, 'tempInvalid', u'Печать'))
        self.setupUi(self)
        self.setWindowFlags(self.windowFlags() | Qt.WindowMaximizeButtonHint)
        self.setWindowTitleEx(u'Документ временной нетрудоспособности')
        self.setWindowState(Qt.WindowMaximized)
        self.grpTempInvalidPeriods.setStyleSheet("QGroupBox {font-weight: bold;}")
        self.grpTempInvalidDocuments.setStyleSheet("QGroupBox {font-weight: bold;}")
        self.grpMedicalCommission.setStyleSheet("QGroupBox {font-weight: bold;}")
        self.grpMedicalCommissionMSI.setStyleSheet("QGroupBox {font-weight: bold;}")
        self.tblPeriods.setModel(self.modelPeriods)
        self.tblDocuments.setModel(self.modelDocuments)
        self.tblPeriods.addPopupDelRow()
        self.tblDocuments.addPopupDirectionMC()
        self.tblDocuments.addPopupDuplicateCurrentRow()
        self.tblDocuments.addPopupDelRow()
        self.tblMedicalCommission.setModel(self.modelMedicalCommission)
        self.tblMedicalCommission.addPopupEditDirectionMC()
        self.tblMedicalCommission.addPopupDelRow()
        self.tblMedicalCommissionMSI.setModel(self.modelMedicalCommissionMSI)
        self.tblMedicalCommissionMSI.addPopupEditDirectionMC()
        self.tblMedicalCommissionMSI.addPopupDelRow()
        self.clientCache = clientCache
        self.modelPeriods.setEventEditor(self)
        self.modelDocuments.setEventEditor(self)
        self.modelMedicalCommission.setEventEditor(self)
        self.modelMedicalCommissionMSI.setEventEditor(self)
        self.buttonBox.addButton(self.btnPrint, QtGui.QDialogButtonBox.ActionRole)
        self.clientId = None
        self.clientSex = None
        self.clientAge = None
        self.diagnosisId = None
        self.prevId = None
        self.prevClosed = None
        self.orgId = QtGui.qApp.currentOrgId()
        self.personId = None
        self.caseBegDate = None
        self.docCode = None
        self.docId = None
        self.prolonging = False
        self.saveProlonging = False
        self.isUpdatePlaceWork = False
        self.closed = 0
        self.setupDirtyCather()
        self.lblDiagnosis.setVisible(False)
        self.edtDiagnosis.setVisible(False)
        self.cmbDiseaseCharacter.setVisible(False)
        self.edtCaseBegDate.setDate(self.caseBegDate)
        self.edtCaseBegDate.setEnabled(False)
        self.chkInsuranceOfficeMark.setEnabled(QtGui.qApp.userHasRight(urRegWriteInsurOfficeMark))
        self.modifiableDiagnosisesMap = {}
        self.mapSpecialityIdToDiagFilter = {}
        self.blankParams = {}
        self.defaultBlankMovingId = None
        self.cmbDiseaseCharacter.setTable('rbDiseaseCharacter', order='code')
        self.edtNumberPermit.setText(u'')
        self.edtBegDatePermit.setDate(QDate())
        self.edtEndDatePermit.setDate(QDate())
        self.cmbBreak.setTable('rbTempInvalidBreak')
        self.edtBreakDate.setDate(QDate())
        self.edtBegDateStationary.setDate(QDate())
        self.edtEndDateStationary.setDate(QDate())
        self.cmbDisability.setTable('rbTempInvalidRegime', filter = u'type = 1')
        self.cmbResult.setTable('rbTempInvalidResult')
        self.edtResultDate.setDate(QDate())
        self.edtResultOtherwiseDate.setDate(QDate())
        self.placeRegistry = False
        self.isReasonPrimary = False
        self.showMaximized()


    def setRecord(self, record):
        self.isReasonPrimary = True
        CwithoutPreferenceItemEditorBaseDialog.setRecord(self, record)
        setCheckBoxValue(self.chkInsuranceOfficeMark, record, 'insuranceOfficeMark')
        self.closed = forceInt(record.value('closed'))
        self.clientId = forceRef(record.value('client_id'))
        self.cmbReceiver.setClientId(self.clientId)
        self.diagnosisId = forceRef(record.value('diagnosis_id'))
        MKB, MKBEx, characterId = self.getMKBs()
        self.edtDiagnosis.setText(MKB)
        self.cmbDiseaseCharacter.setValue(characterId)
        self.setType(forceInt(record.value('type')))
        setRBComboBoxValue(self.cmbDoctype, record, 'doctype_id')
        setRBComboBoxValue(self.cmbReason,  record, 'tempInvalidReason_id')
        if requiredDiagnosis(self.cmbReason.value()):
            self.lblDiagnosis.setVisible(True)
            self.edtDiagnosis.setVisible(True)
            self.cmbDiseaseCharacter.setVisible(True)
        setRBComboBoxValue(self.cmbExtraReason,  record, 'tempInvalidExtraReason_id')
        setLineEditValue(self.edtNumberPermit, record, 'numberPermit')
        setDateEditValue(self.edtBegDatePermit, record, 'begDatePermit')
        setDateEditValue(self.edtEndDatePermit, record, 'endDatePermit')
        setRBComboBoxValue(self.cmbBreak,  record, 'break_id')
        setDateEditValue(self.edtBreakDate, record, 'breakDate')
        setDateEditValue(self.edtBegDateStationary, record, 'begDateStationary')
        setDateEditValue(self.edtEndDateStationary, record, 'endDateStationary')
        setRBComboBoxValue(self.cmbDisability,  record, 'disability_id')
        setRBComboBoxValue(self.cmbResult,  record, 'result_id')
        setDateEditValue(self.edtResultDate, record, 'resultDate')
        setDateEditValue(self.edtResultOtherwiseDate, record, 'resultOtherwiseDate')
        setLineEditValue(self.edtOGRN, record, 'OGRN')
        self.on_edtOGRN_textEdited(self.edtOGRN.text())
        self.clientSex, self.clientAge = self.getClientSexAge(self.clientId)
        setComboBoxValue(self.cmbOtherSex,  record, 'sex')
        setSpinBoxValue(self.edtOtherAge,   record, 'age')
        self.cmbReceiver.setValue(forceRef(record.value('client_id')))
        self.caseBegDate = forceDate(record.value('caseBegDate'))
        self.edtCaseBegDate.setDate(self.caseBegDate)
        self.cmbAccountPregnancyTo12Weeks.setCurrentIndex(forceInt(record.value('accountPregnancyTo12Weeks')))
        self.setEnabledWidget(self.chkInsuranceOfficeMark.isChecked(), [self.cmbDoctype, self.cmbReason, self.cmbExtraReason, self.edtDiagnosis, self.cmbDiseaseCharacter, self.chkInsuranceOfficeMark, self.tblPeriods])
        db = QtGui.qApp.db
        table = db.table('TempInvalid')
        condDeleted = table['deleted'].eq(0)
        condClient = table['client_id'].eq(self.clientId)
        prevCond = [condDeleted, condClient, table['closed'].eq(2), table['endDate'].eq(forceDate(record.value('begDate')).addDays(-1))]
        prevRecord = db.getRecordEx(table, '*', prevCond, 'endDate DESC')
        self.setPrev(prevRecord)
        self.modelPeriods.loadItems(self.itemId())
        self.modelDocuments.loadItems(self.itemId(), forceRef(record.value('client_id')), self.prevId)
        self.modelMedicalCommission.loadItems(self.itemId())
        self.medicalCommissionMSILoadItems()
        self.updateLength()
        fullLength, externalLength = self.modelPeriods.calcLengths()
        self.btnTempInvalidProlong.setEnabled(self.closed == 0 and fullLength)
        self.prolonging = False
        self.defaultBlankMovingId = None
        self.isReasonPrimary = False


    def medicalCommissionLoadItems(self):
        self.modelMedicalCommission.loadItems(self.itemId())


    def medicalCommissionMSILoadItems(self):
        self.modelMedicalCommissionMSI.setTempInvalidId(self.itemId())
        self.modelMedicalCommissionMSI.loadItems(self.modelMedicalCommission.getActionIdList())


    def setEnabledWidget(self, checked, listWidget = []):
        enabled = QtGui.qApp.userHasRight(urRegWriteInsurOfficeMark)
        otherPersonEnabled = requiredOtherPerson(self.cmbReason.value()) if self.type_ != CTempInvalidEditDialog.Disability else True
        receiverId = self.cmbReceiver.value()
        if not otherPersonEnabled or self.placeRegistry:
            self.cmbReceiver.setValue(self.clientId)
        elif receiverId == self.clientId and not self.isReasonPrimary:
            self.cmbReceiver.setValue(None)
            self.cmbReceiver.setClientId(None)
            for item in self.modelDocuments.items():
                item.setValue('clientPrimum_id', toVariant(self.clientId))
            self.modelDocuments.reset()
        pregnancyAndBirthEnabled = requiredPregnancyAndBirth(self.cmbReason.value())
        diagnosisEnabled = requiredDiagnosis(self.cmbReason.value())
        self.lblDiagnosis.setVisible(diagnosisEnabled)
        self.edtDiagnosis.setVisible(diagnosisEnabled)
        self.cmbDiseaseCharacter.setVisible(diagnosisEnabled)
        if checked:
            for widget in listWidget:
                widget.setEnabled(enabled)
            self.cmbReceiver.setEnabled(enabled)
            self.cmbOtherSex.setEnabled(otherPersonEnabled and enabled and not self.cmbReceiver.value())
            self.edtOtherAge.setEnabled(otherPersonEnabled and enabled and not self.cmbReceiver.value())
            self.cmbAccountPregnancyTo12Weeks.setEnabled(pregnancyAndBirthEnabled and enabled)
            if self.btnTempInvalidProlong.isEnabled():
                fullLength, externalLength = self.modelPeriods.calcLengths()
                closed = self.getTempInvalidClosedStatus()
                self.btnTempInvalidProlong.setEnabled(enabled and closed == 0 and fullLength)
            self.modelDocuments.setEnabledPatient(otherPersonEnabled, enabled, checked, self.isReasonPrimary)
        else:
            self.cmbOtherSex.setEnabled(otherPersonEnabled and not self.cmbReceiver.value())
            self.edtOtherAge.setEnabled(otherPersonEnabled and not self.cmbReceiver.value())
            self.chkInsuranceOfficeMark.setEnabled(enabled)
            self.cmbAccountPregnancyTo12Weeks.setEnabled(pregnancyAndBirthEnabled)
            self.modelDocuments.setEnabledPatient(otherPersonEnabled, enabled, False, self.isReasonPrimary)
        self.modelPeriods.setReason(self.cmbReason.value())


    @pyqtSignature('QString')
    def on_cmbReceiver_textChanged(self, index):
        if self.cmbReceiver.isVisible():
            otherPersonEnabled = requiredOtherPerson(self.cmbReason.value()) if self.type_ != CTempInvalidEditDialog.Disability else True
            receiverId = self.cmbReceiver.value()
            sex, ageClient = self.getClientSexAge(receiverId)
            self.cmbOtherSex.setCurrentIndex(sex)
            self.edtOtherAge.setValue(ageClient)
            if self.chkInsuranceOfficeMark.isChecked():
                enabled = QtGui.qApp.userHasRight(urRegWriteInsurOfficeMark)
                self.cmbOtherSex.setEnabled(otherPersonEnabled and enabled and not receiverId)
                self.edtOtherAge.setEnabled(otherPersonEnabled and enabled and not receiverId)
            else:
                self.cmbOtherSex.setEnabled(otherPersonEnabled and not receiverId)
                self.edtOtherAge.setEnabled(otherPersonEnabled and not receiverId)
            self.modelDocuments.setTempInvalidClientId(receiverId)
            if otherPersonEnabled:
                self.isUpdatePlaceWork = True
                if not receiverId:
                    self.cmbReceiver.setClientId(None)
                self.updatePlaceWork(receiverId)
            elif self.isUpdatePlaceWork:
                self.isUpdatePlaceWork = False
                self.updatePlaceWork(self.clientId)


    def updatePlaceWork(self, receiverId):
        self.getPlaceWorkValues(receiverId)
        values = self.modelDocuments.cols()[CTempInvalidDocumentsModel.Col_PlaceWork].values
        if values and len(values) > 0:
            work = values[0]
        else:
            work = formatWorkTempInvalid(getClientWork(receiverId)) if receiverId else u''
        work = trim(work) if work else None
        for item in self.modelDocuments.items():
            item.setValue('placeWork', toVariant(work if work else None))
        self.modelDocuments.reset()


    def getClientSexAge(self, clientId):
        sex = 0
        ageClient = 0
        if clientId:
            db = QtGui.qApp.db
            table = db.table('Client')
            begDate = self.modelPeriods.begDate()
            record = db.getRecordEx(table, [table['sex'], u'age(Client.birthDate, %s) AS ageClient'%(db.formatDate(begDate if begDate else QDate.currentDate()))], [table['id'].eq(clientId), table['deleted'].eq(0)])
            if record:
                sex = forceInt(record.value('sex'))
                ageClient = forceInt(record.value('ageClient'))
        return sex, ageClient


    def setType(self, type_, docCode=None, isNotEvent=False):
        self.type_ = type_
        self.docCode = docCode
        self.docId = forceRef(QtGui.qApp.db.translate('rbTempInvalidDocument', 'code', self.docCode, 'id')) if self.docCode else None
        filter = 'type=%d'%self.type_
        filierDoc = (filter+' AND code=\'%s\''%self.docCode) if (self.docCode and not isNotEvent) else filter
        self.cmbDoctype.setTable('rbTempInvalidDocument', False, filierDoc)
        self.clientSex, self.clientAge = self.getClientSexAge(self.clientId)
        self.cmbReason.setTable('rbTempInvalidReason', False, filter + (u''' AND code NOT IN ('05', '020')''' if self.clientSex == 1 else u''))
        self.cmbExtraReason.setTable('rbTempInvalidExtraReason', True, filter)
        self.cmbResult.setTable('rbTempInvalidResult', True, filter)
        self.modelPeriods.setType(self.type_)
        self.modelDocuments.setType(self.type_)
        for widget in (self.lblOtherSex, self.cmbOtherSex,
                       self.lblOtherAge, self.edtOtherAge,
                       self.lblAccountPregnancyTo12Weeks,
                       self.cmbAccountPregnancyTo12Weeks,):
            widget.setVisible(self.type_==0)
        if self.type_ == 0:
            self.cmbDoctype.setCode(QtGui.qApp.tempInvalidDoctype())
            self.cmbReason.setCode(QtGui.qApp.tempInvalidReason())
            if self.docCode:
                self.cmbDoctype.setEnabled(isNotEvent)
                self.cmbDoctype.setValue(self.docId)
        if self.type_ == CTempInvalidEditDialog.Disability:
            self.lblCaseBegDate.setText(u'Дата начала Инв.')
            self.lblPermit.setText(u'Установлена')
            self.lblResultDate.setText(u'Дата снятия')
            self.lblResultOtherwiseDate.setText(u'Дата ОС')
            self.edtNumberPermit.setVisible(False)
            self.lblAccountPregnancyTo12Weeks.setVisible(False)
            self.cmbAccountPregnancyTo12Weeks.setVisible(False)
            self.lblOGRN.setVisible(False)
            self.edtOGRN.setVisible(False)
            self.btnSelectHeadOrganisation.setVisible(False)
            self.lblOrganisation.setVisible(False)
            self.lblOrganisationOGRN.setVisible(False)
        self.getPlaceWorkValues(self.clientId)


    def setPrev(self, prevRecord):
        if prevRecord:
            self.prevId = forceRef(prevRecord.value('id'))
            self.prevClosed = forceInt(prevRecord.value('close'))
            self.caseBegDate = forceDate(prevRecord.value('caseBegDate'))
        else:
            self.prevId = None
            self.prevClosed = None
        self.edtCaseBegDate.setDate(self.caseBegDate)
        self.edtCaseBegDate.setEnabled(False)


    def begDateLastPeriod(self):
        return self.modelPeriods.begDateLast()


    def getRecord(self):
        record = CwithoutPreferenceItemEditorBaseDialog.getRecord(self)
        db = QtGui.qApp.db
        table = db.table('TempInvalid')
        itemId = self.itemId()
        record.setValue('type', toVariant(self.type_))
        getRBComboBoxValue(self.cmbDoctype, record, 'doctype_id')
        getRBComboBoxValue(self.cmbReason,  record, 'tempInvalidReason_id')
        getRBComboBoxValue(self.cmbExtraReason,  record, 'tempInvalidExtraReason_id')
        getLineEditValue(self.edtNumberPermit, record, 'numberPermit')
        getDateEditValue(self.edtBegDatePermit, record, 'begDatePermit')
        getDateEditValue(self.edtEndDatePermit, record, 'endDatePermit')
        getRBComboBoxValue(self.cmbBreak,  record, 'break_id')
        getDateEditValue(self.edtBreakDate, record, 'breakDate')
        getDateEditValue(self.edtBegDateStationary, record, 'begDateStationary')
        getDateEditValue(self.edtEndDateStationary, record, 'endDateStationary')
        getRBComboBoxValue(self.cmbDisability,  record, 'disability_id')
        getRBComboBoxValue(self.cmbResult,  record, 'result_id')
        getDateEditValue(self.edtResultDate, record, 'resultDate')
        getDateEditValue(self.edtResultOtherwiseDate, record, 'resultOtherwiseDate')
        record.setValue('client_id', toVariant(self.cmbReceiver.value()))
        getComboBoxValue(self.cmbOtherSex,  record, 'sex')
        getSpinBoxValue(self.edtOtherAge,   record, 'age')
        getCheckBoxValue(self.chkInsuranceOfficeMark,  record, 'insuranceOfficeMark')
        getLineEditValue(self.edtOGRN, record, 'OGRN')
        fullLength, externalLength = self.modelPeriods.calcLengths()
        record.setValue('begDate',  toVariant(self.modelPeriods.begDate()))
        record.setValue('endDate',  toVariant(self.modelPeriods.endDate()))
        record.setValue('duration', toVariant(fullLength))
        date = self.modelPeriods.endDate()
        if not date:
            date = self.modelPeriods.begDate()
        diagnosisTypeId = 1 #diagnosisType = закл
        diagnosis = getDiagnosisId2(date, self.modelPeriods.lastPerson(), self.clientId, diagnosisTypeId, unicode(self.edtDiagnosis.text()), u'', self.cmbDiseaseCharacter.value(), None, None)
        self.diagnosisId = diagnosis[0]
        record.setValue('diagnosis_id', toVariant(self.diagnosisId))
        if self.prolonging:
            closed = 2
        else:
            closed = self.getTempInvalidClosedStatus()
        record.setValue('closed',   toVariant(closed))
        if requiredDiagnosis(self.cmbReason.value()):
            if self.diagnosisId:
                record.setValue('diagnosis_id', toVariant(self.diagnosisId))
        else:
            record.setValue('diagnosis_id', QVariant())
        record.setValue('person_id', toVariant(self.modelPeriods.lastPerson()))
        record.setValue('prev_id', toVariant(self.prevId))
        if not self.caseBegDate:
            self.caseBegDate = self.modelPeriods.begDate()
        record.setValue('caseBegDate', toVariant(self.caseBegDate))
        record.setValue('accountPregnancyTo12Weeks', toVariant(self.cmbAccountPregnancyTo12Weeks.currentIndex()))
        self.saveProlonging = False
        return record


    def exec_(self):
        result = CwithoutPreferenceItemEditorBaseDialog.exec_(self)
        if not result:
            if self.saveProlonging and self.prevId and self.prevClosed is not None:
                self.prolonging = False
                self.saveProlonging = False
                try:
                    db = QtGui.qApp.db
                    db.transaction()
                    table = db.table('TempInvalid')
                    tableTD = db.table('TempInvalidDocument')
                    db.updateRecords(table, table['closed'].eq(self.prevClosed), [table['deleted'].eq(0), table['id'].eq(self.prevId)])
                    records = db.getRecordList(tableTD, u'*', [tableTD['deleted'].eq(0), tableTD['master_id'].eq(self.prevId), tableTD['modifyDatetime'].dateEq(QDate.currentDate())])
                    for row, record in enumerate(records):
                        record.setValue('execPerson_id', toVariant(None))
                        db.updateRecord(tableTD, record)
                    db.commit()
                except:
                    db.rollback()
                    raise
        return result


    def checkDataEntered(self):
        result = True
        reasonId = self.cmbReason.value()
        result = result and (reasonId or self.checkInputMessage(u'причину', False, self.cmbReason))
        result = result and (self.cmbReceiver.value() or self.checkInputMessage(u'получателя', False, self.cmbReceiver))
        result = result and (self.cmbResult.value() or self.checkInputMessage(u'результат', False, self.cmbResult))
        if self.getTempInvalidAbleStatus() and not self.getTempInvalidException():
            result = result and (self.edtResultDate.date() or self.checkInputMessage(u'дату "Приступить к работе"', False, self.edtResultDate))
            endDateStationary = self.edtEndDateStationary.date()
            result = result and ((not endDateStationary or endDateStationary < self.edtResultDate.date()) or self.checkValueMessage(u'Дата "Приступить к работе" должна быть позднее даты окончания нахождения в стационаре.', False, self.edtResultDate))
        elif self.getTempInvalidException():
            result = result and (self.edtResultOtherwiseDate.date() or self.checkInputMessage(u'дату "Иное"', True, self.edtResultOtherwiseDate))
        if requiredOtherPerson(reasonId) and self.type_ != CTempInvalidEditDialog.Disability:
            result = result and (self.cmbOtherSex.currentIndex() or self.checkInputMessage(u'пол', False, self.cmbOtherSex))
            result = result and (self.edtOtherAge.value() or self.checkInputMessage(u'возраст', False, self.edtOtherAge))
        if requiredPregnancyAndBirth(reasonId):
            result = result and (self.cmbAccountPregnancyTo12Weeks.currentIndex() or self.checkInputMessage(u'учёт по беременности до 12 нед.', True, self.cmbAccountPregnancyTo12Weeks))
        items = self.modelPeriods.items()
        result = result and (len(items) or self.checkInputMessage(u'период', False, self.tblPeriods, 0, 0))
        if not result:
            return
        date = None
        for row, record in enumerate(items):
            date, recordOk = self.checkPeriodDataEntered(date, row, record)
            if not recordOk:
                return False
        result = result and (self.modelPeriods.lastPerson() or self.checkInputMessage(u'Врача последнего периода', True, self.tblPeriods, len(items)-1 if items else None, items[-1].indexOf('endPerson_id') if items else None))
        # проверка отсутствия пересекающихся периодов нетрудоспособности
        begDate = self.modelPeriods.begDate()
        endDate = self.modelPeriods.endDate()
        db = QtGui.qApp.db
        table = db.table('TempInvalid')
        cond = [table['begDate'].between(begDate, endDate),
                table['endDate'].between(begDate, endDate),
                db.joinAnd([table['begDate'].le(begDate), table['endDate'].ge(begDate)]),
                db.joinAnd([table['begDate'].le(endDate), table['endDate'].ge(endDate)]), # а на бумажке рисую - вроде-бы и не нужно...
               ]
        cond = [db.joinOr(cond)]
        cond.append(table['id'].ne(self.itemId()))
        cond.append(table['client_id'].eq(self.clientId))
        cond.append(table['deleted'].eq(0))
        cond.append(table['doctype_id'].eq(self.cmbDoctype.value()))

        stmt = 'SELECT '+db.existsStmt(table, cond)+' AS X';
        query = db.query(stmt)
        if query.first():
            present = forceBool(query.record().value(0))
            if present:
                result = self.checkValueMessage(u'Период нетрудоспособности пересекается с существующим.', False, self.tblPeriods, 0, 0)
                return False
        #result = result and self.checkTempInvalidPeriodsDateKAKEntered(reasonId)
        result = result and (len(self.modelDocuments.items()) or self.checkInputMessage(u'документ', False, self.tblDocuments, 0, 0))
        result = result and self.checkNumberTempInvalidDocument()
        result = result and self.checkSerialNumberTempInvalidDocument()
        result = result and self.checkReason()
        result = result and self.checkBreak()
        result = result and self.checkTempInvalidDocumentIssueDate()
        result = result and self.checkTempInvalidDisabilityDate()
        result = result and self.checkDataOther()
        result = result and self.checkTempInvalidDocumentClients()
        result = result and self.checkActualMKB()
        result = result and self.checkDocumentsSNILS()
        result = result and self.checkAbleStatusDocuments()
        return result


    def checkAbleStatusDocuments(self):
        if self.cmbResult.code() not in [u'31'] or self.getTempInvalidAbleStatus():
            items = self.modelDocuments.items()
            for row, item in enumerate(items):
                if not forceRef(item.value('execPerson_id')):
                    number = trim(forceString(item.value('number')))
                    if not self.checkExecPersonValueMessage(u'Необходимо указать врача закрывшего документ номер %s! Заполнить его текущим пользователем?'%(number), self.tblDocuments, row, item.indexOf('execPerson_id')):
                        return False
        return True


    def checkExecPersonValueMessage(self, message, widget, row=None, column=None, detailWdiget=None, setFocus=True):
        buttons = QtGui.QMessageBox.Yes | QtGui.QMessageBox.No
        res = QtGui.QMessageBox.warning( self,
                                         u'Внимание!',
                                         message,
                                         buttons,
                                         QtGui.QMessageBox.Yes)
        if res == QtGui.QMessageBox.Yes:
            self.modelDocuments.items()[row].setValue('execPerson_id', toVariant(QtGui.qApp.userId))
            return True
        elif res == QtGui.QMessageBox.No:
            if setFocus:
                self.setFocusToWidget(widget, row, column)
                if isinstance(detailWdiget, QtGui.QWidget):
                    self.setFocusToWidget(detailWdiget, row, column)
            return False
        return True


    def checkReason(self):
        code = self.cmbReason.code()
        if code in [u'05', u'08', u'017', u'018', u'019'] and not self.edtBegDatePermit.date():
            self.checkInputMessage(u'дату начала путёвки', False, self.edtBegDatePermit)
            return False
        if code in [u'08', u'017', u'018', u'019'] and not self.edtEndDatePermit.date():
            self.checkInputMessage(u'дату окончания путёвки', False, self.edtEndDatePermit)
            return False
        if code in [u'08', u'017', u'018', u'019'] and not trim(self.edtNumberPermit.text()):
            self.checkInputMessage(u'номер путёвки', False, self.edtNumberPermit)
            return False
        begDatePermit = self.edtBegDatePermit.date()
        endDatePermit = self.edtEndDatePermit.date()
        if begDatePermit and endDatePermit and begDatePermit > endDatePermit:
            self.checkValueMessage(u'Дата начала путёвки не может быть позже даты окончания.', False, self.edtEndDatePermit)
            return False
        if not begDatePermit and endDatePermit:
            self.checkValueMessage(u'Дата окончания путёвки введена должна быть и дата начала путёвки.', False, self.edtBegDatePermit)
            return False
        if code in [u'08', u'017', u'018', u'019'] and not trim(self.edtOGRN.text()):
            self.checkInputMessage(u'ОГРН', False, self.edtOGRN)
            return False
        if self.cmbExtraReason.code() in [u'018'] and code not in [u'02', u'04']:
            self.checkValueMessage(u'Несовместимые значения причины и доп.причины нетрудоспособности.', False, self.cmbReason)
            return False
        if trim(self.edtNumberPermit.text()):
            if not trim(self.edtOGRN.text()):
                self.checkInputMessage(u'ОГРН', False, self.edtOGRN)
                return False
            if not self.edtBegDatePermit.date():
                self.checkInputMessage(u'дату начала путёвки', False, self.edtBegDatePermit)
                return False
            if not self.edtEndDatePermit.date():
                self.checkInputMessage(u'дату окончания путёвки', False, self.edtEndDatePermit)
                return False
        if trim(self.edtOGRN.text()):
            if not trim(self.edtNumberPermit.text()):
                self.checkInputMessage(u'номер путёвки', False, self.edtNumberPermit)
                return False
            if not self.edtBegDatePermit.date():
                self.checkInputMessage(u'дату начала путёвки', False, self.edtBegDatePermit)
                return False
            if not self.edtEndDatePermit.date():
                self.checkInputMessage(u'дату окончания путёвки', False, self.edtEndDatePermit)
                return False
        return True


    def checkBreak(self):
        breakId = self.cmbBreak.value()
        if breakId:
            if not self.edtBreakDate.date():
                self.checkInputMessage(u'дату нарушения режима', False, self.edtBreakDate)
                return False
            if not (self.edtBreakDate.date() >= self.modelPeriods.begDate() and self.edtBreakDate.date() <= self.modelPeriods.endDate()):
                self.checkValueMessage(u'Дата нарушения режима не попадает в период временной нетрудоспособности.', False, self.edtBreakDate)
                return False
        if self.cmbBreak.code() in [u'23', u'24', u'25'] and self.cmbResult.code() not in [u'36']:
            self.checkValueMessage(u'Несовместимые значения Нарушения режима и Результата.', False, self.cmbResult)
            return False
        if self.edtBreakDate.date() and not self.cmbBreak.value():
            self.checkInputMessage(u'значение Нарушения режима', False, self.cmbBreak)
            return False
        return True


    def checkDataOther(self):
        begDateStationary = self.edtBegDateStationary.date()
        endDateStationary = self.edtEndDateStationary.date()
        if begDateStationary and endDateStationary and begDateStationary > endDateStationary:
            self.checkValueMessage(u'Дата начала нахождения в стационаре не может быть позже даты окончания.', False, self.edtEndDateStationary)
            return False
        if not begDateStationary and endDateStationary:
            self.checkValueMessage(u'Дата окончания нахождения в стационаре введена должна быть и дата начала нахождения в стационаре.', False, self.edtBegDateStationary)
            return False
        return True


    def checkTempInvalidDocumentIssueDate(self):
        if self.type_ != CTempInvalidEditDialog.Disability:
            items = self.modelDocuments.items()
            begDatePermit = self.edtBegDatePermit.date()
            resultOtherwiseDate = self.edtResultOtherwiseDate.date()
            for row, item in enumerate(items):
                issueDate = forceDate(item.value('issueDate'))
                duplicate = forceBool(item.value('duplicate'))
                if not duplicate and issueDate and begDatePermit and issueDate > begDatePermit:
                    self.checkValueMessage(u'Дата выдачи ЛН не должна быть позже Даты начала путёвки.', False, self.tblDocuments, row, item.indexOf('issueDate'))
                    return False
                if not duplicate and issueDate and resultOtherwiseDate and issueDate > resultOtherwiseDate:
                    self.checkValueMessage(u'Дата выдачи ЛН не должна быть позже Даты Иное.', False, self.tblDocuments, row, item.indexOf('issueDate'))
                    return False
        return True


    def checkTempInvalidDisabilityDate(self):
        if self.type_ == CTempInvalidEditDialog.Disability:
            begDatePermit = self.edtBegDatePermit.date()
            endDatePermit = self.edtEndDatePermit.date()
            endDate = self.modelPeriods.endDate()
            if begDatePermit > endDatePermit:
                self.checkValueMessage(u'Дата Начала Установления инвалидности не должна быть позже Дата Окончания Установления инвалидности.', False, self.edtBegDatePermit)
                return False
            if endDate:
                if begDatePermit > endDate:
                    self.checkValueMessage(u'Дата Начала Установления инвалидности не должна быть позже последней дате последнего периода.', False, self.edtBegDatePermit)
                    return False
                if endDatePermit > endDate:
                    self.checkValueMessage(u'Дата Окончания Установления инвалидности не должна быть позже последней дате последнего периода.', False, self.edtEndDatePermit)
                    return False
        return True


    def checkNumberTempInvalidDocument(self):
        items = self.modelDocuments.items()
        for row, item in enumerate(items):
            number = trim(forceString(item.value('number')))
            if not number:
                self.checkInputMessage(u'Номер в ЛВН', False, self.tblDocuments, row, item.indexOf('number'))
                return False
            elif self.type_ != CTempInvalidEditDialog.Disability and len(number) < 12:
                self.checkValueMessage(u'Номер в ЛВН %s содержит менее 12 символов. Введите корректный номер.'%(number), False, self.tblDocuments, row, item.indexOf('number'))
                return False
            busyness = forceInt(item.value('busyness'))
            if not busyness:
                self.checkInputMessage(u'Занятость в ЛВН', False, self.tblDocuments, row, item.indexOf('busyness'))
                return False
            placeWork = trim(forceString(item.value('placeWork')))
            if not placeWork:
                self.checkInputMessage(u'Место работы в ЛВН', False, self.tblDocuments, row, item.indexOf('placeWork'))
                return False
        return True


    def checkTempInvalidDocumentClients(self):
        if self.type_ != CTempInvalidEditDialog.Disability:
            otherPersonEnabled = requiredOtherPerson(self.cmbReason.value())
            if not otherPersonEnabled:
                return True
            items = self.modelDocuments.items()
            for row, item in enumerate(items):
                clientPrimumId = forceRef(item.value('clientPrimum_id'))
                clientSecondId = forceRef(item.value('clientSecond_id'))
                if clientPrimumId == clientSecondId:
                    self.checkValueMessage(u'Поля "Пациент 1" и "Пациент 2" таблицы "Документы" не могут иметь одинаковое значение.', False, self.tblDocuments, row, item.indexOf('clientSecond_id'))
                    return False
        return True


    def checkSerialNumberTempInvalidDocument(self):
        items = self.modelDocuments.items()
        for row, item in enumerate(items):
            serial = forceString(item.value('serial'))
            number = forceString(item.value('number'))
            duplicate = forceBool(item.value('duplicate'))
            if not duplicate:
                for row2, item2 in enumerate(items):
                    if row != row2:
                        serial2 = forceString(item2.value('serial'))
                        number2 = forceString(item2.value('number'))
                        duplicate2 = forceBool(item2.value('duplicate'))
                        if serial == serial2 and number == number2 and not duplicate2:
                            self.checkValueMessage(u'ЛВН с указанными серией %s и номером %s уже существуют. Укажите корректные данные.'%(serial2, number2), False, self.tblDocuments, row2, item2.indexOf('serial'))
                            return False
        db = QtGui.qApp.db
        table = db.table('TempInvalidDocument')
        for row, item in enumerate(items):
            documentId = forceRef(item.value('id'))
            serial = forceString(item.value('serial'))
            number = forceString(item.value('number'))
            duplicate = forceBool(item.value('duplicate'))
            if not duplicate:
                cond = [table['deleted'].eq(0),
                        table['serial'].like(serial),
                        table['number'].like(number),
                        table['duplicate'].eq(0)
                        ]
                if documentId:
                    cond.append(table['id'].ne(documentId))
                record = db.getRecordEx(table, [table['id']], cond)
                if record:
                    self.checkValueMessage(u'ЛВН с указанными серией %s и номером %s уже существуют. Укажите корректные данные.'%(serial, number), False, self.tblDocuments, row, item.indexOf('serial'))
                    return False
        return True


    def checkValueMessageUpdateSNILS(self, clientId, widget, row=None, column=None):
        result = False
        messageBox = QtGui.QMessageBox()
        messageBox.setWindowTitle(u'Внимание!')
        messageBox.setText(u'Для оформления электронного больничного листа необходим СНИЛС получателя.')
        messageBox.addButton(QtGui.QPushButton(u'ОК'), QtGui.QMessageBox.ActionRole)
        messageBox.addButton(QtGui.QPushButton(u'Добавить СНИЛС'), QtGui.QMessageBox.ActionRole)
        messageBox.addButton(QtGui.QPushButton(u'Пропустить'), QtGui.QMessageBox.ActionRole)
        res = messageBox.exec_()
        if res == 0:
            self.setFocusToWidget(widget, row, column)
            result = False
        elif res == 1:
            QtGui.qApp.callWithWaitCursor(self, self.editClient, clientId)
            self.setFocusToWidget(widget, row, column)
            result = False
        elif res == 2:
            result = True
        return result


    def editClient(self, clientId):
        if QtGui.qApp.userHasAnyRight([urAdmin, urRegTabWriteRegistry, urRegTabReadRegistry]):
            dialog = CClientEditDialog(self)
            try:
                dialog.load(clientId)
                QtGui.qApp.restoreOverrideCursor()
                dialog.setFocusToWidget(dialog.edtSNILS)
                if dialog.exec_():
                    pass
            finally:
                dialog.destroy()
                sip.delete(dialog)
                del dialog


    def checkDocumentsSNILS(self):
        items = self.modelDocuments.items()
        clientId = self.cmbReceiver.value()
        for row, item in enumerate(items):
            if forceBool(item.value('electronic')):
                db = QtGui.qApp.db
                table = db.table('Client')
                record = db.getRecordEx(table, [table['SNILS']], [table['deleted'].eq(0), table['id'].eq(clientId)])
                SNILS = forceString(record.value('SNILS')) if record else None
                if SNILS:
                    return True
                else:
                    return self.checkValueMessageUpdateSNILS(clientId, self.tblDocuments, row, item.indexOf('electronic'))
        return True


    def checkActualMKB(self):
        result = True
        reasonTypeId = self.cmbReason.value()
        if reasonTypeId:
            db = QtGui.qApp.db
            table = db.table('rbTempInvalidReason')
            if forceBool(db.translate(table, 'id', reasonTypeId, 'requiredDiagnosis')):
                MKB = unicode(self.edtDiagnosis.text())
                items = self.modelPeriods.items()
                result = result and (MKB or self.checkInputMessage(u'диагноз', False, self.edtDiagnosis))
                if len(items) > 0 and MKB:
                    begDate = forceDate(items[0].value('begDate'))
                    db = QtGui.qApp.db
                    tableMKB = db.table('MKB')
                    cond = [tableMKB['DiagID'].eq(MKBwithoutSubclassification(MKB))]
                    cond.append(db.joinOr([tableMKB['endDate'].isNull(), tableMKB['endDate'].dateGe(begDate)] ))
                    recordMKB = db.getRecordEx(tableMKB, [tableMKB['DiagID']], cond)
                    result = result and (forceString(recordMKB.value('DiagID')) == MKBwithoutSubclassification(MKB) if recordMKB else False) or self.checkValueMessage(u'Диагноз %s не доступен для применения'%MKB, False, self.edtDiagnosis)
        return result


    def checkTempInvalidPeriodsDateKAKEntered(self, tempInvalidReasonId):
        db = QtGui.qApp.db
        table = db.table('rbTempInvalidReason')
        tableTempInvalid = db.table('TempInvalid')
        tableTempInvalidPeriod = db.table('TempInvalid_Period')
        recordReason = db.getRecordEx(table, [table['restriction']], [table['id'].eq(tempInvalidReasonId)])
        restriction = forceInt(recordReason.value('restriction')) if recordReason else 0
        if restriction:
            itemsPrev = []
            if self.prevId:
                prevId = self.prevId
                tempInvalidIdList = []
                while prevId:
                    cond = [tableTempInvalid['prev_id'].isNotNull(),
                            tableTempInvalid['id'].eq(prevId),
                            tableTempInvalid['deleted'].eq(0),
                            tableTempInvalid['doctype_id'].eq(self.cmbDoctype.value())
                            ]
                    if self.clientId:
                        cond.append(tableTempInvalid['client_id'].eq(self.clientId))
                    cond.append(u'TempInvalid.closed = 2 OR TempInvalid.closed = 0')
                    stmt = db.selectDistinctStmt(tableTempInvalid, [tableTempInvalid['prev_id']], cond, '', None)
                    query = db.query(stmt)
                    prevId = None
                    while query.next():
                        prevId = query.value(0).toInt()[0]
                        if prevId and prevId not in tempInvalidIdList:
                            tempInvalidIdList.append(prevId)
                if self.prevId not in tempInvalidIdList:
                    tempInvalidIdList.append(self.prevId)
                if tempInvalidIdList:
                    prevRecords = db.getRecordListGroupBy(tableTempInvalidPeriod, u'TempInvalid_Period.*', [tableTempInvalidPeriod['master_id'].inlist(tempInvalidIdList)], u'TempInvalid_Period.id', u'TempInvalid_Period.begDate')
                    for recordPrev in prevRecords:
                        itemsPrev.append(recordPrev)
            items = self.modelPeriods.items()
            if itemsPrev:
                for item in items:
                    itemsPrev.append(item)
            else:
                itemsPrev = items
            durationSum = 0
            begDate = None
            for row, record in enumerate(itemsPrev):
                begDateDur = forceDate(record.value('begDate'))
                endDateDur = forceDate(record.value('endDate'))
                duration = begDateDur.daysTo(endDateDur)+1 if begDateDur and endDateDur and begDateDur <= endDateDur else 0
                directDateOnKAK = forceDate(record.value('directDateOnKAK'))
                isExternal = forceBool(record.value('isExternal'))
                directDateOnKAKIndex = record.indexOf('directDateOnKAK')
                resultId = forceRef(record.value('result_id'))
                able = forceBool(db.translate('rbTempInvalidResult', 'id', resultId, 'able'))
                if able or directDateOnKAK or isExternal:
                    durationSum = 0
                    begDate = None
                else:
                    durationSum += duration
                    if not begDate:
                        begDate = forceDate(record.value('begDate'))
            if durationSum and restriction < durationSum:
                if begDate:
                    newDirectDateOnKAK = begDate.addDays(restriction - 1)
                result = CEventEditDialog(self).checkValueMessage(u'Необходимо направить на КЭК%s'%((u', предположительная дата напрвления на КЭК %s'%(newDirectDateOnKAK.toString('dd.MM.yyyy'))) if newDirectDateOnKAK else u''), False, self.tblPeriods, len(items)-1, directDateOnKAKIndex)
                if newDirectDateOnKAK:
                    record.setValue('directDateOnKAK', toVariant(newDirectDateOnKAK))
                return result
        return True


    def checkPeriodDataEntered(self, prevPerionEndDate, row, record):
        begDateIndex = record.indexOf('begDate')
        endDateIndex = record.indexOf('endDate')
        resultIndex  = record.indexOf('result_id')
        begDate = forceDate(record.value(begDateIndex))
        endDate = forceDate(record.value(endDateIndex))
        resultId = forceRef(record.value(resultIndex))
        result = True
        if result and not begDate:
            result = self.checkValueMessage(u'Не заполнена дата начала периода', False, self.tblPeriods, row, begDateIndex)
        if result and (prevPerionEndDate and prevPerionEndDate.daysTo(begDate) != 1):
            result = self.checkValueMessage(u'Недопустимая дата начала периода', False, self.tblPeriods, row, begDateIndex)
        if result and not endDate:
            result = self.checkValueMessage(u'Не заполнена дата окончания периода', False, self.tblPeriods, row, endDateIndex)
        if result and (begDate.daysTo(endDate)<0):
            result = self.checkValueMessage(u'Недопустимая дата окончания периода', False, self.tblPeriods, row, endDateIndex)
        return endDate, result


    def getTempInvalidClosedStatus(self):
        resultId = self.cmbResult.value()
        if resultId:
            return forceInt(QtGui.qApp.db.translate('rbTempInvalidResult', 'id', resultId, 'closed'))
        return 0


    def getTempInvalidAbleStatus(self):
        resultId = self.cmbResult.value()
        if resultId:
            return forceInt(QtGui.qApp.db.translate('rbTempInvalidResult', 'id', resultId, 'able'))
        return 0


    def getTempInvalidException(self):
        resultId = self.cmbResult.value()
        if resultId:
            db = QtGui.qApp.db
            table = db.table('rbTempInvalidResult')
            record = db.getRecordEx(table, [table['id']], [table['id'].eq(resultId), table['code'].inlist([32, 33, 34, 36])])
            return forceBool(forceRef(record.value('id')) if record else None)
        return False


    def updateLength(self):
        fullLength, externalLength = self.modelPeriods.calcLengths()
        self.lblLengthValue.setText(str(fullLength))
        self.lblExternalLengthValue.setText(str(externalLength))
        if self.caseBegDate:
            caseLength = self.caseBegDate.daysTo(self.modelPeriods.begDate())+fullLength
        else:
            caseLength = fullLength
        self.lblCaseLengthValue.setText(str(caseLength))
        closed = self.getTempInvalidClosedStatus()
        self.btnTempInvalidProlong.setEnabled(closed == 0 and self.closed != 2 and caseLength)


    def newTempInvalid(self, begDate):
        self.tempInvalidId = None
        self.insuranceOfficeMark = None
        fullLength, externalLength = self.modelPeriods.calcLengths()
        self.btnTempInvalidProlong.setEnabled((not self.prolonging) and fullLength)
        self.prolonging = False
        self.modelPeriods.clearItems()
        self.modelPeriods.addStart(begDate)
        self.chkInsuranceOfficeMark.setChecked(False)
        self.updateLength()
        self.setItemId(None)
        self.modelMedicalCommission.clearItems()
        self.modelMedicalCommissionMSI.clearItems()


    def getMKBs(self):
        if self.diagnosisId:
            db = QtGui.qApp.db
            record = db.getRecord('Diagnosis', '*', self.diagnosisId)
            if record:
               return forceString(record.value('MKB')), forceString(record.value('MKBEx')), forceRef(record.value('character_id'))
        return '', '', None


    def getDiagFilter(self):
        db =QtGui.qApp.db
        specialityId = forceRef(db.translate('Person', 'id', self.modelPeriods.lastPerson(), 'speciality_id'))
        result = self.mapSpecialityIdToDiagFilter.get(specialityId, None)
        if result is None:
            result = db.translate('rbSpeciality', 'id', specialityId, 'mkbFilter')
            if result is None:
                result = ''
            else:
                result = forceString(result)
            self.mapSpecialityIdToDiagFilter[specialityId] = forceString(result)
        return result


    def specifyDiagnosis(self, MKB):
        diagFilter = self.getDiagFilter()
        date = self.modelPeriods.begDate()
        if not date:
            date = QDate.currentDate()
        acceptable, specifiedMKB, specifiedMKBEx, specifiedCharacterId, specifiedTraumaTypeId, modifiableDiagnosisId, specifiedDispanserId, specifiedRequiresFillingDispanser, specifiedProlongMKB = specifyDiagnosis(self, MKB, diagFilter, self.clientId, self.clientSex, self.clientAge, date)
        self.modifiableDiagnosisesMap[specifiedMKB] = modifiableDiagnosisId
        return acceptable, specifiedMKB, specifiedMKBEx, specifiedCharacterId, specifiedTraumaTypeId, specifiedDispanserId, specifiedRequiresFillingDispanser, specifiedProlongMKB


    def setEditorDataTI(self):
        db = QtGui.qApp.db
        MKB  = unicode(self.edtDiagnosis.text())
        codeIdList = getAvailableCharacterIdByMKB(MKB)
        table = db.table('rbDiseaseCharacter')
        self.cmbDiseaseCharacter.setTable(table.name(), not bool(codeIdList), filter=table['id'].inlist(codeIdList))


    def updateCharacterByMKB(self, MKB, specifiedCharacterId):
        characterIdList = getAvailableCharacterIdByMKB(MKB)
        if specifiedCharacterId in characterIdList:
            characterId = specifiedCharacterId
        else:
            characterId = forceRef(self.cmbDiseaseCharacter.value())
            if (characterId in characterIdList) or (characterId is None and not characterIdList):
                return
            if characterIdList:
                characterId = characterIdList[0]
            else:
                characterId = None
        self.cmbDiseaseCharacter.setValue(characterId)


    def getTempInvalidInfo(self, context):
        result = context.getInstance(CTempInvalidInfo, None)
        result._doctype = context.getInstance(CTempInvalidDocTypeInfo,  self.cmbDoctype.value())
        result._reason  = context.getInstance(CTempInvalidReasonInfo,  self.cmbReason.value())
        result._extraReason  = context.getInstance(CTempInvalidExtraReasonInfo, forceRef(self.cmbExtraReason.value()))
        result._sex     = formatSex(self.cmbOtherSex.currentIndex())
        result._age     = self.edtOtherAge.value()
        result._client= context.getInstance(CClientInfo, self.cmbReceiver.value())
        result._duration, result._externalDuration = self.modelPeriods.calcLengths()
        result._begDate = CDateInfo(self.modelPeriods.begDate())
        result._endDate = CDateInfo(self.modelPeriods.endDate())
        result._accountPregnancyTo12Weeks = forceInt(self.cmbAccountPregnancyTo12Weeks.currentIndex())
        MKB, MKBEx, characterId = self.getMKBs()
        result._MKB = context.getInstance(CMKBInfo, MKB)
        result._MKBEx = context.getInstance(CMKBInfo, MKBEx)
        closed = self.getTempInvalidClosedStatus()
        result._closed = closed
        result._periods = self.modelPeriods.getPeriodsInfo(context)
        result._result = context.getInstance(CTempInvalidResultInfo, forceRef(self.cmbResult.value()))

        result._caseBegDate = CDateInfo(self.edtCaseBegDate.date())
        result._begDateStationary = CDateInfo(forceDate(self.edtBegDateStationary.date()))
        result._endDateStationary = CDateInfo(forceDate(self.edtEndDateStationary.date()))
        result._break = context.getInstance(CTempInvalidBreakInfo, self.cmbBreak.value())
        result._breakDate = CDateInfo(forceDate(self.edtBreakDate.date()))
        result._resultDate = CDateInfo(forceDate(self.edtResultDate.date()))
        result._resultOtherwiseDate = CDateInfo(forceDate(self.edtResultOtherwiseDate.date()))
        result._OGRN = self.edtOGRN.text()
        result._numberPermit = self.edtNumberPermit.text()
        result._begDatePermit =  CDateInfo(self.edtBegDatePermit.date())
        result._endDatePermit =  CDateInfo(self.edtEndDatePermit.date())
        result._receiver= context.getInstance(CClientInfo, self.cmbReceiver.value())

        result._items = self.modelDocuments.getDocumentsInfo(context)
        if self.prevId:
            result._prev = context.getInstance(CTempInvalidInfo, self.prevId)
        else:
            result._prev = None
        result._ok = True
        return result


    @pyqtSignature('')
    def on_btnClientRelations_clicked(self):
        dialog = CClientRelationsEditDialog(self)
        clientInfo = getClientMiniInfo(self.clientId)
        dialog.setWindowTitle(u'Связи: ' + clientInfo)
        dialog.load(self.clientId)
        try:
            if dialog.exec_():
                newClientId = dialog.itemId()
        finally:
            sip.delete(dialog)
            del dialog


    @pyqtSignature('')
    def on_btnSelectHeadOrganisation_clicked(self):
        OGRN = u''
        shortName = u''
        orgId = selectOrganisation(self, None, False)
        if orgId:
            db = QtGui.qApp.db
            table = db.table('Organisation')
            record = db.getRecordEx(table, [table['OGRN'], table['shortName']], [table['id'].eq(orgId), table['deleted'].eq(0)], 'shortName, INN, OGRN')
            if record:
                OGRN = forceString(record.value('OGRN'))
                shortName = forceString(record.value('shortName'))
        self.edtOGRN.setText(OGRN)
        charCount = 32
        if len(shortName) > charCount:
            shortNameLeft = QString(shortName).left(charCount) + u'...'
        else:
            shortNameLeft = shortName
        self.lblOrganisationOGRN.setText(shortNameLeft)
        self.lblOrganisationOGRN.setToolTip(shortName)


    @pyqtSignature('QString')
    def on_edtOGRN_textEdited(self, OGRN):
        shortName = u''
        if OGRN:
            db = QtGui.qApp.db
            table = db.table('Organisation')
            record = db.getRecordEx(table, [table['shortName']], [table['OGRN'].eq(OGRN), table['deleted'].eq(0)], 'shortName, INN, OGRN')
            if record:
                shortName = forceString(record.value('shortName'))
        charCount = 32
        if len(shortName) > charCount:
            shortNameLeft = QString(shortName).left(charCount) + u'...'
        else:
            shortNameLeft = shortName
        self.lblOrganisationOGRN.setText(shortNameLeft)
        self.lblOrganisationOGRN.setToolTip(shortName)


    @pyqtSignature('int')
    def on_cmbResult_currentIndexChanged(self, index):
        resultId = self.cmbResult.value()
        if resultId:
            if self.getTempInvalidAbleStatus() and not self.getTempInvalidException():
                self.edtResultDate.setEnabled(True)
                endDate = self.modelPeriods.endDate()
                self.edtResultDate.setDate(endDate.addDays(1) if endDate else QDate())
                self.edtResultOtherwiseDate.setEnabled(False)
                self.edtResultOtherwiseDate.setDate(QDate())
            else:
                self.edtResultDate.setEnabled(False)
                self.edtResultDate.setDate(QDate())
                self.edtResultOtherwiseDate.setEnabled(True)
            self.ableStatusDocuments()
        else:
            self.edtResultDate.setEnabled(False)
            self.edtResultDate.setDate(QDate())
            self.edtResultOtherwiseDate.setEnabled(False)
            self.edtResultOtherwiseDate.setDate(QDate())
        closed = self.getTempInvalidClosedStatus()
        fullLength, externalLength = self.modelPeriods.calcLengths()
        self.btnTempInvalidProlong.setEnabled(closed == 0 and self.closed != 2 and fullLength)


    def ableStatusDocuments(self):
        if self.getTempInvalidAbleStatus():
            items = self.modelDocuments.items()
            for row, item in enumerate(items):
                if not forceRef(item.value('execPerson_id')):
                    self.modelDocuments.items()[row].setValue('execPerson_id', toVariant(QtGui.qApp.userId))
            self.modelDocuments.reset()


    @pyqtSignature('int')
    def on_cmbDoctype_currentIndexChanged(self, index):
        self.modelDocuments.setDocCode(self.cmbDoctype.code())


    @pyqtSignature('int')
    def on_cmbReason_currentIndexChanged(self, index):
        if not self.isReasonPrimary:
            self.setEnabledWidget(self.chkInsuranceOfficeMark.isChecked(), [self.cmbDoctype, self.cmbReason, self.cmbExtraReason, self.edtDiagnosis, self.cmbDiseaseCharacter, self.chkInsuranceOfficeMark, self.tblPeriods])
        if self.type_ != CTempInvalidEditDialog.Disability:
            self.btnClientRelations.setEnabled(requiredOtherPerson(self.cmbReason.value()))

    def getDuplicateAndParent(self):
        items = []


    def selectTempInvalidDocument(self, items):
        newItems = []
        for item in items:
            newRecord = self.modelDocuments.getEmptyRecord()
            copyFields(newRecord, item)
            newRecord.setValue('id', toVariant(None))
            newRecord.setValue('issueDate', toVariant(QDate.currentDate()))
            newRecord.setValue('serial', toVariant(None))
            newRecord.setValue('number', toVariant(None))
            newRecord.setValue('duplicate', toVariant(0))
            newRecord.setValue('duplicateReason_id', toVariant(None))
            newRecord.setValue('prevDuplicate_id', toVariant(None))
            newRecord.setValue('prevNumber', toVariant(forceString(item.value('number'))))
            newRecord.setValue('prev_id', toVariant(forceRef(item.value('id'))))
            newRecord.setValue('last_id', toVariant(None))
            newRecord.setValue('person_id', toVariant(QtGui.qApp.userId))
            newRecord.setValue('execPerson_id', toVariant(None))
            newRecord.setValue('note', toVariant(None))
            newItems.append(newRecord)
        return newItems


    def save(self):
        tempInvalidId = CwithoutPreferenceItemEditorBaseDialog.save(self)
        if tempInvalidId:
            self.modelPeriods.saveItems(tempInvalidId)
            self.modelDocuments.saveItems(tempInvalidId)
        return tempInvalidId


    @pyqtSignature('')
    def on_btnTempInvalidProlong_clicked(self):
        execPersonitems = self.modelDocuments.items()
        for row, execPersonitem in enumerate(execPersonitems):
            if forceBool(execPersonitem.value('duplicate')):
                personId = forceRef(execPersonitem.value('person_id'))
                if personId:
                    self.modelDocuments.items()[row].setValue('execPerson_id', toVariant(personId))
                else:
                    self.modelDocuments.items()[row].setValue('person_id', toVariant(QtGui.qApp.userId))
                    self.modelDocuments.items()[row].setValue('execPerson_id', toVariant(QtGui.qApp.userId))
            else:
                self.modelDocuments.items()[row].setValue('execPerson_id', toVariant(QtGui.qApp.userId))
        if not self.checkDataEntered():
            return
        self.prolonging = True
        if not self.save():
            self.prolonging = False
            self.saveProlonging = False
            return
        items = self.modelDocuments.items()
        newItems = []
        isDuplicate = False
        for item in items:
            if forceBool(item.value('duplicate')):
                isDuplicate = True
                break
        if isDuplicate:
            resItems = []
            dialog = CTempInvalidDocumentProlongDialog(self, self.clientCache, items)
            try:
                if dialog.exec_():
                    resItems = dialog.getItems()
            finally:
                sip.delete(dialog)
                del dialog
            includeItems = []
            for includeItem in resItems:
                if forceBool(includeItem.value('include')):
                    includeItems.append(includeItem)
            if not includeItems:
                if self.itemId():
                    try:
                        db = QtGui.qApp.db
                        db.transaction()
                        table = db.table('TempInvalid')
                        close = self.prevClosed if (self.prevClosed is not None) else 0
                        db.updateRecords(table, table['closed'].eq(close), [table['deleted'].eq(0), table['id'].eq(self.itemId())])
                        db.commit()
                    except:
                        db.rollback()
                        raise
                return
            newItems = self.selectTempInvalidDocument(includeItems)
        else:
            newItems = self.selectTempInvalidDocument(items)
        self.saveProlonging = True
        itemId = self.itemId()
        if itemId:
            db = QtGui.qApp.db
            table = db.table('TempInvalid')
            prevRecord = db.getRecordEx(table, '*', [table['id'].eq(itemId), table['deleted'].eq(0), table['client_id'].eq(self.clientId), table['type'].eq(self.type_), table['closed'].eq(2)], 'endDate DESC')
            self.setPrev(prevRecord)
        self.newTempInvalid(forceDate(self.modelPeriods.endDate().addDays(1)))
        self.modelDocuments.clearItems()
        for row, newItem in enumerate(newItems):
            self.modelDocuments.insertRecord(row, newItem)
        self.modelDocuments.reset()
        self.edtNumberPermit.setText(u'')
        self.edtBegDatePermit.setDate(QDate())
        self.edtEndDatePermit.setDate(QDate())
        self.cmbBreak.setValue(None)
        self.edtBreakDate.setDate(QDate())
        self.edtBegDateStationary.setDate(QDate())
        self.edtEndDateStationary.setDate(QDate())
        self.cmbDisability.setValue(None)
        self.cmbResult.setValue(None)
        self.edtResultDate.setDate(QDate())
        self.edtResultOtherwiseDate.setDate(QDate())
        self.tblPeriods.setFocus(Qt.OtherFocusReason)
        self.tblPeriods.setCurrentIndex(self.modelPeriods.index(0, 1))


    def preCreateDirectionMC(self):
        itemId = self.itemId()
        if not itemId:
            if not self.checkDataEntered():
                return False
            tempInvalidId = self.save()
            if tempInvalidId:
                self.setItemId(tempInvalidId)
                db = QtGui.qApp.db
                table = db.table('TempInvalid')
                prevRecord = db.getRecordEx(table, '*', [table['id'].eq(itemId), table['deleted'].eq(0), table['client_id'].eq(self.clientId), table['type'].eq(self.type_), table['closed'].eq(2)], 'endDate DESC')
                self.setPrev(prevRecord)
                self.modelDocuments.setClientId(self.clientId)
                self.modelDocuments.setTempInvalidId(tempInvalidId)
                self.modelDocuments.setTempInvalidPrevId(self.prevId)
                return True
            return False
        else:
            return True


    @pyqtSignature('int')
    def on_btnPrint_printByTemplate(self, templateId):
        context = CInfoContext()
        tempInvalidInfo = self.getTempInvalidInfo(context)
        eventInfo = None
        data = { 'event' : eventInfo,
                 'client': context.getInstance(CClientInfo, self.clientId, QDate.currentDate()),
                 'tempInvalid': tempInvalidInfo,
                 'getEventList': lambda begDate, endDate: getEventListByDates(context, self.clientId, begDate, endDate)
               }
        applyTemplate(self, templateId, data)


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_modelPeriods_dataChanged(self, topLeft, bottomRight):
        self.updateLength()
        db = QtGui.qApp.db
        closed = self.getTempInvalidClosedStatus()
        fullLength, externalLength = self.modelPeriods.calcLengths()
        self.btnTempInvalidProlong.setEnabled(closed == 0 and self.closed != 2 and fullLength)
        items = self.modelPeriods.items()
        if len(items) > 0:
            begDate = forceDate(items[0].value('begDate'))
            receiverId = self.cmbReceiver.value()
            if receiverId:
                clientSex, ageClient = self.getClientSexAge(receiverId)
                self.edtOtherAge.setValue(ageClient)
                self.cmbOtherSex.setCurrentIndex(clientSex)
        self.edtDiagnosis.setFilter('''IF(MKB.endDate IS NOT NULL, MKB.endDate >= %s, 1)'''%(db.formatDate(begDate)))
        if not self.prevId:
            self.caseBegDate = self.modelPeriods.begDate()
            self.edtCaseBegDate.setDate(self.caseBegDate)


    def getPlaceWorkValues(self, clientId):
        placeWorkValues = []
        if clientId:
            db = QtGui.qApp.db
            table = db.table('TempInvalid')
            tableTD = db.table('TempInvalidDocument')
            cond = [table['client_id'].eq(clientId),
                    table['deleted'].eq(0),
                    tableTD['deleted'].eq(0),
                    tableTD['busyness'].ne(0)
                   ]
            group = u'TempInvalidDocument.busyness, TempInvalidDocument.placeWork'
            tableQuery = table.innerJoin(tableTD, tableTD['master_id'].eq(table['id']))
            records = db.getRecordListGroupBy(tableQuery, u'DISTINCT TempInvalidDocument.busyness, TempInvalidDocument.placeWork', cond, group, u'TempInvalidDocument.issueDate DESC')
            for record in records:
                busyness = forceInt(record.value('busyness'))
                if busyness and busyness != 3:
                    placeWork = trim(forceString(record.value('placeWork')))
                    if placeWork and placeWork not in placeWorkValues:
                        placeWorkValues.append(placeWork)
            work = trim(formatWorkTempInvalid(getClientWork(clientId)))
            if work and work not in placeWorkValues:
                placeWorkValues.append(work)
        self.modelDocuments.setPlaceWorkValues(placeWorkValues)


    def getLastPlaceWork(self, clientId):
        placeWorkValues = []
        if clientId:
            db = QtGui.qApp.db
            table = db.table('TempInvalid')
            tableTD = db.table('TempInvalidDocument')
            tableQuery = table.innerJoin(tableTD, tableTD['master_id'].eq(table['id']))
            cond = [table['client_id'].eq(clientId),
                    table['deleted'].eq(0),
                    tableTD['deleted'].eq(0),
                    tableTD['issueDate'].dateLe(QDate.currentDate())
                   ]
            record = db.getRecordEx(tableQuery, [table['id']], cond, 'TempInvalidDocument.issueDate DESC')
            tempInvalidId = forceRef(record.value('id')) if record else None
            if tempInvalidId:
                cols = [tableTD['busyness'],
                        tableTD['placeWork']
                        ]
                cond = [tableTD['master_id'].eq(tempInvalidId),
                        tableTD['deleted'].eq(0)
                        ]
                records = db.getRecordList(tableQuery, cols, cond, u'TempInvalidDocument.issueDate DESC')
                for record in records:
                    busyness = forceInt(record.value('busyness'))
                    if busyness and busyness != 3:
                        placeWork = trim(forceString(record.value('placeWork')))
                        if placeWork and placeWork not in placeWorkValues:
                            placeWorkValues.append((busyness, placeWork))
        return placeWorkValues


class CTempInvalidCreateDialog(CTempInvalidEditDialog):
    def __init__(self,  parent, clientId = None, clientCache = None, MKB = u''):
        CTempInvalidEditDialog.__init__(self, parent, clientCache)
        self.lblDiagnosis.setVisible(True)
        self.edtDiagnosis.setVisible(True)
        self.cmbDiseaseCharacter.setVisible(True)
        self.clientId = clientId
        self.MKB = MKB
        self.clientSex = None
        self.clientAge = None
        self.modifiableDiagnosisesMap = {}
        self.mapSpecialityIdToDiagFilter = {}
        self.cmbDiseaseCharacter.setTable('rbDiseaseCharacter', order='code')
        self.cmbReceiver.setClientId(self.clientId)
        self.placeRegistry = False


    def createTempInvalidDocument(self, MKB = u'', placeRegistry = False, type = 0):
        self.placeRegistry = placeRegistry
        self.modelDocuments.setClientId(self.clientId)
        otherPersonEnabled = requiredOtherPerson(self.cmbReason.value()) if self.type_ != CTempInvalidEditDialog.Disability else True
        self.getPlaceWorkValues(self.clientId)
        placeWorkValues = self.getLastPlaceWork(self.clientId)
        if not placeWorkValues:
            newRecord = self.modelDocuments.getEmptyRecord()
            if not otherPersonEnabled or self.placeRegistry:
                self.cmbReceiver.setValue(self.clientId)
            else:
                newRecord.setValue('clientPrimum_id', toVariant(self.clientId))
            newRecord.setValue('issueDate', toVariant(QDate.currentDate()))
            newRecord.setValue('busyness', toVariant(1))
            newRecord.setValue('placeWork', toVariant(trim(formatWorkTempInvalid(getClientWork(self.clientId)))))
            self.modelDocuments.addRecord(newRecord)
        else:
            for (busyness, placeWork) in placeWorkValues:
                newRecord = self.modelDocuments.getEmptyRecord()
                if not otherPersonEnabled or self.placeRegistry:
                    self.cmbReceiver.setValue(self.clientId)
                else:
                    newRecord.setValue('clientPrimum_id', toVariant(self.clientId))
                newRecord.setValue('issueDate', toVariant(QDate.currentDate()))
                newRecord.setValue('busyness', toVariant(busyness))
                newRecord.setValue('placeWork', toVariant(placeWork))
                self.modelDocuments.addRecord(newRecord)
        newRecordPeriods = self.modelPeriods.getEmptyRecord()
        self.modelPeriods.addRecord(newRecordPeriods)
        if MKB:
            acceptable, specifiedMKB, specifiedMKBEx, specifiedCharacterId, specifiedTraumaTypeId, specifiedDispanserId, specifiedRequiresFillingDispanser, specifiedProlongMKB = self.specifyDiagnosis(MKB)
            self.MKB = specifiedMKB
            self.edtDiagnosis.setText(self.MKB)
            self.updateCharacterByMKB(specifiedMKB, specifiedCharacterId)
        itemsPeriods = self.modelPeriods.items()
        if len(itemsPeriods) == 1:
            record = itemsPeriods[0]
            record.setValue('id', toVariant(None))
            record.setValue('begDate', toVariant(QDate.currentDate()))
            self.modelPeriods.items()[0] = record
        if not self.caseBegDate:
            self.caseBegDate = self.modelPeriods.begDate()
            self.edtCaseBegDate.setDate(self.caseBegDate)
        if not placeRegistry:
            db = QtGui.qApp.db
            tableResult = db.table('rbTempInvalidResult')
            record = db.getRecordEx(tableResult, [tableResult['id']], [tableResult['code'].like(u'31')], tableResult['id'].name())
            resultId = forceRef(record.value('id')) if record else None
            self.cmbResult.setValue(resultId)
            if type:
                tableReason = db.table('rbTempInvalidReason')
                record = db.getRecordEx(tableReason, [tableReason['id']], [tableReason['code'].like(u'09'), tableReason['grouping'].eq(1)], tableReason['id'].name())
                reasonId = forceRef(record.value('id')) if record else None
                self.cmbReason.setValue(reasonId)


    def getRecord(self):
        record = CwithoutPreferenceItemEditorBaseDialog.getRecord(self)
        record.setValue('type', toVariant(self.type_))
        getRBComboBoxValue(self.cmbDoctype, record, 'doctype_id')
        getRBComboBoxValue(self.cmbReason,  record, 'tempInvalidReason_id')
        getRBComboBoxValue(self.cmbExtraReason,  record, 'tempInvalidExtraReason_id')
        getLineEditValue(self.edtNumberPermit, record, 'numberPermit')
        getDateEditValue(self.edtBegDatePermit, record, 'begDatePermit')
        getDateEditValue(self.edtEndDatePermit, record, 'endDatePermit')
        getRBComboBoxValue(self.cmbBreak,  record, 'break_id')
        getDateEditValue(self.edtBreakDate, record, 'breakDate')
        getDateEditValue(self.edtBegDateStationary, record, 'begDateStationary')
        getDateEditValue(self.edtEndDateStationary, record, 'endDateStationary')
        getRBComboBoxValue(self.cmbDisability,  record, 'disability_id')
        getRBComboBoxValue(self.cmbResult,  record, 'result_id')
        getDateEditValue(self.edtResultDate, record, 'resultDate')
        getDateEditValue(self.edtResultOtherwiseDate, record, 'resultOtherwiseDate')
        getComboBoxValue(self.cmbOtherSex,  record, 'sex')
        getSpinBoxValue(self.edtOtherAge,   record, 'age')
        getCheckBoxValue(self.chkInsuranceOfficeMark,  record, 'insuranceOfficeMark')
        getLineEditValue(self.edtOGRN, record, 'OGRN')
        fullLength, externalLength = self.modelPeriods.calcLengths()
        record.setValue('begDate',  toVariant(self.modelPeriods.begDate()))
        record.setValue('endDate',  toVariant(self.modelPeriods.endDate()))
        record.setValue('duration', toVariant(fullLength))
        record.setValue('client_id', toVariant(self.cmbReceiver.value()))
        date = self.modelPeriods.endDate()
        if not date:
            date = self.modelPeriods.begDate()
        diagnosisTypeId = 1 #diagnosisType = закл
        diagnosis = getDiagnosisId2(date, self.modelPeriods.lastPerson(), self.clientId, diagnosisTypeId, unicode(self.edtDiagnosis.text()), u'', self.cmbDiseaseCharacter.value(), None, None)
        record.setValue('diagnosis_id', toVariant(diagnosis[0]))
        if self.prolonging:
            closed = 2
        else:
            closed = self.getTempInvalidClosedStatus()
        record.setValue('closed',  toVariant(closed))
        record.setValue('prev_id', toVariant(self.prevId))
        record.setValue('person_id', toVariant(self.modelPeriods.lastPerson()))
        if not self.caseBegDate:
            self.caseBegDate = self.modelPeriods.begDate()
        record.setValue('caseBegDate', toVariant(self.caseBegDate))
        record.setValue('accountPregnancyTo12Weeks', toVariant(self.cmbAccountPregnancyTo12Weeks.currentIndex()))
        db = QtGui.qApp.db
        table = db.table('TempInvalid')
        self.saveProlonging = False
        return record


    def getMKBs(self):
        return unicode(self.edtDiagnosis.text()), '', self.cmbDiseaseCharacter.value()


def getEventListByDates(context, clientId, begDate, endDate):
    if not (begDate and endDate):
        return context.getInstance(CEventInfoList, [])
    if type(begDate) == CDateInfo:
        begDate = begDate.date
    if type(endDate) == CDateInfo:
        endDate = endDate.date
    db = QtGui.qApp.db
    table = db.table("Event")
    cond = [table['setDate'].dateLe(endDate),  table['setDate'].dateGe(begDate), table['client_id'].eq(clientId)]
    recordList = db.getRecordList(table, [table['id'].name()],  cond)
    idList = []
    for record in recordList:
        idList.append(forceRef(record.value(0)))
    return context.getInstance(CEventInfoList, idList)


class CTempInvalidDocumentsModel(CInDocTableModel):
    class CLocDocumentColumn(CInDocTableCol):
        def __init__(self, title, fieldName, width, **params):
            CInDocTableCol.__init__(self, title, fieldName, width, **params)
            self.recordCache = params.get('documentCaches', [])

        def toString(self, val, record):
            documentId  = forceRef(val)
            if documentId and self.recordCache:
                documentRecord = self.recordCache.get(documentId) if documentId else None
                if documentRecord:
                    if not forceInt(documentRecord.value('deleted')):
                        issueDate = forceString(documentRecord.value('issueDate'))
                        name = forceString(documentRecord.value('serial')) + u'-' + forceString(documentRecord.value('number')) + u', ' + forceString(documentRecord.value('placeWork')) + u', ' + issueDate
                        return toVariant(name)
            return QVariant()

    class CLocClientColumn(CInDocTableCol):
        def __init__(self, title, fieldName, width, **params):
            CInDocTableCol.__init__(self, title, fieldName, width, **params)
            self.clientCaches = params.get('clientCaches', [])

        def toString(self, val, record):
            clientId  = forceRef(val)
            if clientId and self.clientCaches:
                clientRecord = self.clientCaches.get(clientId) if clientId else None
                if clientRecord:
                    birthDate = forceString(clientRecord.value('birthDate'))
                    name = formatName(forceString(clientRecord.value('lastName')),
                           forceString(clientRecord.value('firstName')),
                           forceString(clientRecord.value('patrName'))) + u', ' + birthDate
                    return toVariant(name)
            return QVariant()

        def setEditorData(self, editor, value, record):
            editor.setValue(forceRef(value))

        def getEditorData(self, editor):
            return toVariant(editor.value())

        def alignment(self):
            return QVariant(Qt.AlignLeft + Qt.AlignTop)


    class CPlaceWorkInDocTableCol(CSelectStrInDocTableCol):
        def __init__(self, title, fieldName, width, values, **params):
            CSelectStrInDocTableCol.__init__(self, title, fieldName, width, values, **params)
            self.values = values

        def toString(self, val, record):
            str = trim(forceString(val).lower())
            for item in self.values:
                if trim(item.lower()) == str:
                    return toVariant(item)
            if str:
                self.values.append(forceString(val))
            return toVariant(val)

        def createEditor(self, parent):
            editor = CROEditableComboBox(parent)
            for val in self.values:
                editor.addItem(val)
            return editor

        def setValues(self, values):
            self.values = values

    Col_IssueDate      = 0
    Col_Serial         = 1
    Col_Number         = 2
    Col_duplicate      = 3
    Col_duplicateReason= 4
    Col_Electronic     = 5
    Col_Busyness       = 6
    Col_PlaceWork      = 7
    Col_PrevNumber     = 8
    Col_PrevId         = 9
    Col_IssuePersonId  = 10
    Col_ExecPersonId   = 11
    Col_ChairPersonId  = 12
    Col_ClientPrimumId = 13
    Col_ClientSecondId = 14
    Col_LastId         = 15
    Col_Note           = 16

    def __init__(self, parent, clientCache):
        CInDocTableModel.__init__(self, 'TempInvalidDocument', 'id', 'master_id', parent)
        self.issuePersonCol = CActionPerson(u'Выдал',                               'person_id',       20, 'vrbPersonWithSpeciality')
        self.execPersonCol  = CActionPerson(u'Закрыл',                              'execPerson_id',   20, 'vrbPersonWithSpeciality')
        db = QtGui.qApp.db
        documentCaches = CTableRecordCache(db, db.forceTable('TempInvalidDocument'), u'*', capacity=None)
        self.chairPersonCol = CActionPerson(u'Председатель ВК',                     'chairPerson_id',  20, 'vrbPersonWithSpeciality', filter=u'chairPerson = 1')
        self.addCol(CDateInDocTableCol(     u'Дата выдачи',                         'issueDate',       10                                                 ))
        self.addCol(CInDocTableCol(         u'Серия',                               'serial',          22                                                 ))
        self.addCol(CInDocTableCol(         u'Номер',                               'number',          22,  maxLength=12, inputMask='999999999999;'       ))
        self.addCol(CBoolInDocTableCol(     u'Д',                                   'duplicate',       3                                                  ).setToolTip(u'Дубликат')).setReadOnly(True)
        self.addCol(CRBInDocTableCol(       u'Причина выдачи дубликата',            'duplicateReason_id',  10, 'rbTempInvalidDuplicateReason'             ))
        self.addCol(CBoolInDocTableCol(     u'Э',                                   'electronic',      3                                                  ).setToolTip(u'Электронный'))
        self.addCol(CEnumInDocTableCol(     u'Занятость',                           'busyness',        4,  [u'', u'основное', u'совместитель', u'на учете']))
        self.addCol(CTempInvalidDocumentsModel.CPlaceWorkInDocTableCol(u'Место работы','placeWork',    10, []                                             ))
        self.addCol(CInDocTableCol(         u'ПДН',                                 'prevNumber',      10                                                 ).setToolTip(u'Продолжение документа с номером'))
        self.addCol(CTempInvalidDocumentsModel.CLocDocumentColumn(u'ИПД',           'prev_id',         10, documentCaches=documentCaches).setToolTip(u'Идентификатор продолжения документа')).setReadOnly(True)
        self.addCol(self.issuePersonCol)
        self.addCol(self.execPersonCol)
        self.addCol(self.chairPersonCol)
        self.addCol(CTempInvalidDocumentsModel.CLocClientColumn(u'Пациент 1',       'clientPrimum_id', 30, clientCaches=clientCache                       ))
        self.addCol(CTempInvalidDocumentsModel.CLocClientColumn(u'Пациент 2',       'clientSecond_id', 30, clientCaches=clientCache                       ))
        self.addCol(CTempInvalidDocumentsModel.CLocDocumentColumn(u'ИВД',           'last_id', 10, documentCaches=documentCaches).setToolTip(u'Идентификатор выданного документа')).setReadOnly(True)
        self.addCol(CInDocTableCol(         u'Примечание',                          'note',            10                                                 ))
        self.eventEditor = None
        self.readOnly = False
        self.tempInvalidId = None
        self.clientId = None
        self.tempInvalidPrevId = None
        self.blankIdList = []
        self.numberBlankList = {}
        self.isEnabledPatient = True
        self.chairPersonCol.setFilter(filter=u'chairPerson = 1')
        userId = QtGui.qApp.userId
        self.chairUser = forceBool(db.translate('Person', 'id', userId, 'chairPerson')) if userId else False
        self.type = None
        self.docCode = None


    def getEmptyRecord(self):
        result = CInDocTableModel.getEmptyRecord(self)
        if QtGui.qApp.userSpecialityId:
            result.setValue('person_id', toVariant(QtGui.qApp.userId))
            if forceBool(result.value('duplicate')):
                result.setValue('execPerson_id', toVariant(QtGui.qApp.userId))
        return result


    def setType(self, type_):
        self.type = type_
        if self.type == CTempInvalidEditDialog.Disability:
            self.cols()[CTempInvalidDocumentsModel.Col_Number].setInputMask('9'*64)
            self.cols()[CTempInvalidDocumentsModel.Col_Number].setMaxLength(64)
            self.cols()[CTempInvalidDocumentsModel.Col_ClientPrimumId].setTitle(u'Патрон')
            self.eventEditor.tblDocuments.enableColHide(CTempInvalidDocumentsModel.Col_ClientSecondId)


    def setPlaceWorkValues(self, values):
        self.cols()[CTempInvalidDocumentsModel.Col_PlaceWork].setValues(values)


    def setDocCode(self, docCode):
        self.docCode = docCode
        if self.type == CTempInvalidEditDialog.TempInvalid and self.docCode == CTempInvalidEditDialog.MedicalCertificate:
            self.eventEditor.tblDocuments.enableColHide(CTempInvalidDocumentsModel.Col_Serial)
        else:
            self.eventEditor.tblDocuments.showColHide(CTempInvalidDocumentsModel.Col_Serial)


    def setEnabledPatient(self, otherPersonEnabled, rightRegWriteInsurOfficeMark, checkedInsuranceOfficeMark, isReasonPrimary):
        enable = rightRegWriteInsurOfficeMark if checkedInsuranceOfficeMark else True
        self.isEnabledPatient = bool(otherPersonEnabled and enable)
        if not otherPersonEnabled and enable and not isReasonPrimary:
            for record in self._items:
                record.setValue('clientPrimum_id', toVariant(None))
                record.setValue('clientSecond_id', toVariant(None))


    def setReadOnly(self, value):
        self.readOnly = value


    def cellReadOnly(self, index):
        column = index.column()
        if not self.isEnabledPatient and column in [CTempInvalidDocumentsModel.Col_ClientPrimumId,
                                                    CTempInvalidDocumentsModel.Col_ClientSecondId]:
            return True
        row = index.row()
        if 0 <= row < len(self._items):
            record = self._items[row]
            if record:
                if forceBool(self._items[row].value('electronic')) and column in [CTempInvalidDocumentsModel.Col_Serial,
                                                                                  CTempInvalidDocumentsModel.Col_Number]:
                    return True
                if forceBool(self._items[row].value('duplicate')) and column in [CTempInvalidDocumentsModel.Col_duplicate,
                                                                                 CTempInvalidDocumentsModel.Col_Electronic,
                                                                                 CTempInvalidDocumentsModel.Col_Busyness,
                                                                                 CTempInvalidDocumentsModel.Col_PlaceWork,
                                                                                 CTempInvalidDocumentsModel.Col_PrevNumber,
                                                                                 CTempInvalidDocumentsModel.Col_PrevId,
                                                                                 CTempInvalidDocumentsModel.Col_ClientPrimumId,
                                                                                 CTempInvalidDocumentsModel.Col_ClientSecondId,
                                                                                 CTempInvalidDocumentsModel.Col_LastId,
                                                                                 CTempInvalidDocumentsModel.Col_Note]:
                    return True
                if column == CTempInvalidDocumentsModel.Col_ChairPersonId:
                    if self.chairUser and forceBool(self._items[row].value('duplicate')):
                        return False
                    return True
                if column == CTempInvalidDocumentsModel.Col_PlaceWork and forceInt(self._items[row].value('busyness')) == 3:
                    return True
            elif column == CTempInvalidDocumentsModel.Col_ChairPersonId:
                return True
        elif column == CTempInvalidDocumentsModel.Col_ChairPersonId:
            return True
        return False


    def flags(self, index):
        if self.readOnly:
           return Qt.ItemIsSelectable | Qt.ItemIsEnabled
        return CInDocTableModel.flags(self, index)


    def setClientId(self, clientId):
        self.clientId = clientId


    def setTempInvalidId(self, tempInvalidId):
        self.tempInvalidId = tempInvalidId


    def getTempInvalidId(self):
        return self.tempInvalidId


    def setTempInvalidPrevId(self, tempInvalidPrevId):
        self.tempInvalidPrevId = tempInvalidPrevId


    def setData(self, index, value, role=Qt.EditRole):
        row = index.row()
        if self.type == CTempInvalidEditDialog.Disability:
            if row >= 1:
                return False
        result = CInDocTableModel.setData(self, index, value, role)
        if result and role == Qt.EditRole:
            column = index.column()
            if column == CTempInvalidDocumentsModel.Col_Busyness: # Занятость
                busyness = forceInt(value)
                placeWork = trim(forceString(self.value(row, 'placeWork')))
                if self.clientId and not placeWork and (busyness == 0 or busyness == 1):
                    work = formatWorkTempInvalid(getClientWork(self.clientId))
                    if work:
                        self.setValue(row, 'placeWork', toVariant(trim(work)))
                        self.updatePlaceWork(row, toVariant(work))
                        self.setValue(row, 'busyness', toVariant(1))
                    else:
                        self.setValue(row, 'busyness', value)
                else:
                    self.setValue(row, 'busyness', value)
            elif column == CTempInvalidDocumentsModel.Col_PlaceWork: # Место работы
                val = trim(forceString(value))
                self.setValue(row, 'placeWork', toVariant(val))
                self.updatePlaceWork(row, val)
        return result


    def updatePlaceWork(self, row, value):
        newRow = row + 1
        duplicate = True
        while duplicate:
            if newRow >= 0 and newRow < len(self._items):
                duplicate = forceBool(self._items[newRow].value('duplicate'))
                if duplicate:
                    self._items[newRow].setValue('placeWork', QVariant(value))
            else:
                duplicate = False
            newRow += 1


    def loadItems(self, tempInvalidId, clientId, tempInvalidPrevId):
        self.tempInvalidId = tempInvalidId
        self.clientId = clientId
        self.tempInvalidPrevId = tempInvalidPrevId
        CInDocTableModel.loadItems(self, tempInvalidId)


    def saveItems(self, masterId):
        if self._items is not None:
            CInDocTableModel.saveItems(self, masterId)
            db = QtGui.qApp.db
            table = self._table
            for idx, record in enumerate(self._items):
                lastId = forceRef(record.value('id'))
                prevId = forceRef(record.value('prev_id'))
                duplicate = forceBool(record.value('duplicate'))
                if prevId and lastId: #and not duplicate:
                    id = db.updateRecords(table, table['last_id'].eq(lastId), [table['id'].eq(prevId), table['deleted'].eq(0)])
                if duplicate:
                    id = db.updateRecords(table, table['prevDuplicate_id'].eq(toVariant(forceRef(self._items[idx-1].value('id')))), [table['id'].eq(lastId), table['deleted'].eq(0)])


    def setEventEditor(self, eventEditor):
        self.issuePersonCol.setEventEditor(eventEditor)
        self.eventEditor = eventEditor


    def setTempInvalidClientId(self, clientId):
        self.clientId = clientId


    def getTempInvalidClientId(self):
        return self.clientId


    def getDocumentsInfo(self, context):
        result = context.getInstance(CTempInvalidDocumentItemInfoList, None)
        for i, item in enumerate(self.items()):
            id = forceRef(item.value('id'))
            result.addItem(id or -i-1, item)
        return result


class CTempInvalidPeriodModel(CInDocTableModel):

    Col_BegDate       = 0
    Col_EndDate       = 1
    Col_ChairPersonId = 4

    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'TempInvalid_Period', 'id', 'master_id', parent)
        self.endPersonCol = CActionPerson(              u'Врач',             'endPerson_id',    20, 'vrbPersonWithSpeciality')
        self.chairPersonCol = CActionPerson(            u'Председатель ВК',  'chairPerson_id',  20, 'vrbPersonWithSpeciality', filter=u'chairPerson = 1')
        self.addCol(CDateInDocTableCol(                 u'Начало',           'begDate',         10                                          ))
        self.addCol(CDateInDocTableCol(                 u'Окончание',        'endDate',         10                                          ))
        self.addExtCol(CIntInDocTableCol(               u'Дни',              'duration',        10, high=999), QVariant.Int).setReadOnly(not QtGui.qApp.userSpecialityId)
        self.addCol(self.endPersonCol)
        self.addCol(self.chairPersonCol)
        self.colRegime = self.addCol(CRBInDocTableCol(  u'Режим',            'regime_id',       10, 'rbTempInvalidRegime', preferredWidth=150))
        self.addCol(CInDocTableCol(                     u'Примечание',       'note',            10                                           ))
        self.addCol(CBoolInDocTableCol(                 u'Внешний',          'isExternal',      3                                            ))
        self.eventEditor = None
        self.readOnly = False
        self.isExternal = False
        self.primary = 0
        self.prolongate = 0
        self.restriction = 0
        self.currentRow = -1
        self.currentDegDate = None
        self.currentEndDate = None
        self.chairPersonCol.setFilter(filter=u'chairPerson = 1')
        userId = QtGui.qApp.userId
        self.chairUser = forceBool(QtGui.qApp.db.translate('Person', 'id', userId, 'chairPerson')) if userId else False
        self.type = None


    def cellReadOnly(self, index):
        column = index.column()
        row = index.row()
        if 0 <= row < len(self._items):
            record = self._items[row]
            if record:
                if column == CTempInvalidPeriodModel.Col_ChairPersonId:
                    if self.chairUser:
                        return False
                    return True
            elif column == CTempInvalidPeriodModel.Col_ChairPersonId:
                return True
        elif column == CTempInvalidPeriodModel.Col_ChairPersonId:
            return True
        return False


    def setReadOnly(self, value):
        self.readOnly = value


    def flags(self, index):
        if self.readOnly:
           return Qt.ItemIsSelectable | Qt.ItemIsEnabled
        row = index.row()
        if self.type != CTempInvalidEditDialog.Disability:
            if self.isExternal:
                if row >= 4:
                    return Qt.ItemIsSelectable | Qt.ItemIsEnabled
            elif row >= 3:
                return Qt.ItemIsSelectable | Qt.ItemIsEnabled
        elif row >= 1 and not forceBool(self._items[row-1].value('isExternal')):
            return Qt.ItemIsSelectable | Qt.ItemIsEnabled
        return CInDocTableModel.flags(self, index)


    def setData(self, index, value, role=Qt.EditRole):
        row = index.row()
        column = index.column()
        if self.type != CTempInvalidEditDialog.Disability:
            if self.isExternal:
                if row >= 4:
                    return False
            elif row >= 3:
                return False
        else:
            if row >= 1 and not forceBool(self._items[row-1].value('isExternal')):
                return False
        result = CInDocTableModel.setData(self, index, value, role)
        if result and role == Qt.EditRole:
            if column == 0 or column == 1: # начало или окончание
                begDate = forceDate(self.value(row, 'begDate'))
                endDate = forceDate(self.value(row, 'endDate'))
                duration = self._calcDuration(begDate, endDate, row, column)
                self.setValue(row, 'duration', duration)
            elif column == 2: # длительность
                duration = forceInt(self.value(row, 'duration'))
                if duration > 0:
                    begDate = forceDate(self.value(row, 'begDate'))
                    endDate = forceDate(self.value(row, 'endDate'))
                    if begDate:
                        self.setValue(row, 'endDate',  begDate.addDays(duration-1))
                    elif endDate:
                        self.setValue(row, 'begDate',  endDate.addDays(1-duration))
        if role == Qt.CheckStateRole:
            if column == 6: # Внешний = isExternal
                self.setValue(row, 'isExternal', value)
                if forceBool(value):
                    self.setValue(row, 'endPerson_id', toVariant(None))
                self.getIsExternal()
        return result


    def getIsExternal(self):
        self.isExternal = False
        for item in self._items:
            self.isExternal = forceBool(item.value('isExternal'))
            if self.isExternal:
                break


    def loadItems(self, masterId):
        CInDocTableModel.loadItems(self, masterId)
        for item in self.items():
            begDate = forceDate(item.value('begDate'))
            endDate = forceDate(item.value('endDate'))
            duration = self._calcDuration(begDate, endDate)
            item.setValue('duration', toVariant(duration))
        self.getIsExternal()


    def setEventEditor(self, eventEditor):
        self.endPersonCol.setEventEditor(eventEditor)
        self.chairPersonCol.setEventEditor(eventEditor)
        self.eventEditor = eventEditor


    def setReason(self, reasonId):
        if reasonId:
            db = QtGui.qApp.db
            table = db.table('rbTempInvalidReason')
            record = db.getRecordEx(table, [table['primary'], table['prolongate'], table['restriction']], [table['id'].eq(reasonId)])
            if record:
                self.primary = forceInt(record.value('primary'))
                self.prolongate = forceInt(record.value('prolongate'))
                self.restriction = forceInt(record.value('restriction'))


    def setType(self, type_):
        self.type = type_
        filter = 'type=%d' % (type_ if type_ != CTempInvalidEditDialog.Disability else 2)
        self.colRegime.filter = filter


    def getEmptyRecord(self):
        result = CInDocTableModel.getEmptyRecord(self)
        if QtGui.qApp.userSpecialityId:
            result.setValue('endPerson_id', toVariant(QtGui.qApp.userId))
            #result.setValue('chairPerson_id', toVariant(QtGui.qApp.userId))
        return result


    def _calcDuration(self, begDate, endDate, row = -1, column = -1):
        duration = begDate.daysTo(endDate)+1 if begDate and endDate and begDate <= endDate else 0
        if duration > 0 and row >= 0 and column >= 0:
            items = self.items()
            primary = True
            durationItems = 0
            if self.currentRow != row or self.currentDegDate != begDate or self.currentEndDate != endDate:
                self.currentRow = row
                self.currentDegDate = begDate
                self.currentEndDate = endDate
                for i, item in enumerate(items):
                    if not forceBool(item.value('isExternal')):
                        begDateItem = forceDate(self.value(row, 'begDate'))
                        endDateItem = forceDate(self.value(row, 'endDate'))
                        durationItems += begDateItem.daysTo(endDateItem)+1 if begDateItem and endDateItem and begDateItem <= endDateItem else 0
                        if row == i:
                            if primary:
                                primary = False
                                if self.primary and duration > self.primary:
                                    self.eventEditor.checkValueMessage(u'Превышена максимальная длительность первичного периода ВУТ в днях (%s).'%(self.primary), True, self.eventEditor.tblPeriods, row, column)
                            else:
                                if self.prolongate and duration > self.prolongate:
                                    self.eventEditor.checkValueMessage(u'Превышена максимальная длительность продления ВУТ в днях (%s).'%(self.prolongate), True, self.eventEditor.tblPeriods, row, column)
                        else:
                            primary = False
                        if self.restriction and durationItems > self.restriction:
                            self.eventEditor.checkValueMessage(u'Превышена максимальная длительность ВУТ, после которого требуется Направление на ВК (%s).'%(self.restriction), True, self.eventEditor.tblPeriods, i, column)
        return duration


    def calcLengths(self):
        internalLength = 0
        externalLength = 0
        for record in self.items():
            isExternal = forceBool(record.value('isExternal'))
            begDate = forceDate(record.value('begDate'))
            endDate = forceDate(record.value('endDate'))
            length = self._calcDuration(begDate, endDate)
            if isExternal:
                externalLength += length
            else:
                internalLength += length
        return internalLength+externalLength, externalLength


    def begDate(self):
        items = self.items()
        if items:
            firstRecord = items[0]
            return forceDate(firstRecord.value('begDate'))
        else:
            return None


    def endDate(self):
        items = self.items()
        if items:
            lastRecord = items[-1]
            return forceDate(lastRecord.value('endDate'))
        else:
            return None


    def lastPerson(self):
        result = None
        items = self.items()
        if items:
            lastRecord = items[-1]
            result = forceRef(lastRecord.value('endPerson_id'))
        return result


    def begDateLast(self):
        items = self.items()
        if items:
            lastRecord = items[-1]
            if not forceBool(lastRecord.value('isExternal')):
                return forceDate(lastRecord.value('begDate'))
        else:
            return None


    def addStart(self, begDate):
        item = self.getEmptyRecord()
        item.setValue('begDate', toVariant(begDate))
        self.items().append(item)
        self.reset()


    def getPeriodsInfo(self, context):
        result = context.getInstance(CTempInvalidPeriodInfoList, None)
        for i, item in enumerate(self.items()):
            id = forceRef(item.value('id'))
            result.addItem(id or -i-1, item)
        return result


class CActionPerson(CRBInDocTableCol):
    def __init__(self, title, fieldName, width, tableName, **params):
        CRBInDocTableCol.__init__(self, title, fieldName, width, tableName, **params)
        self.eventEditor = None
        self.filterLoc = u''


    def setFilter(self, filter):
        self.filterLoc = filter


    def setEventEditor(self, eventEditor):
        self.eventEditor = eventEditor


    def setEditorData(self, editor, value, record):
        filterRetired = u''
        if self.eventEditor is not None:
            orgId = self.eventEditor.orgId
            begDate = forceDate(record.value('begDate'))
            endDate = forceDate(record.value('endDate'))
            filterDate = None
            if endDate:
                filterDate = u'DATE(retireDate) > DATE(%s)'%QtGui.qApp.db.formatDate(endDate)
            elif begDate:
                filterDate = u'DATE(retireDate) > DATE(%s)'%QtGui.qApp.db.formatDate(begDate)
            filterRetired = u' AND (retireDate IS NULL %s)'%((u' OR (%s)'%filterDate) if filterDate else u'')
        else:
            orgId = QtGui.qApp.currentOrgId()
        filter = 'speciality_id IS NOT NULL AND org_id=\'%s\' %s'%(orgId, filterRetired)
        if self.filterLoc:
            filter += u' AND ' + self.filterLoc
        editor.setTable(self.tableName, self.addNone, filter,'name')
        editor.setShowFields(self.showFields)
        editor.setValue(forceInt(value))


def requiredOtherPerson(tempInvalidReasonId):
    grouping = forceInt(QtGui.qApp.db.translate('rbTempInvalidReason', 'id', tempInvalidReasonId, 'grouping'))
    return grouping == 1


def requiredPregnancyAndBirth(tempInvalidReasonId):
    grouping = forceInt(QtGui.qApp.db.translate('rbTempInvalidReason', 'id', tempInvalidReasonId, 'grouping'))
    return grouping == 2


def requiredDiagnosis(tempInvalidReasonId):
    return forceBool(QtGui.qApp.db.translate('rbTempInvalidReason', 'id', tempInvalidReasonId, 'requiredDiagnosis'))


def formatWorkTempInvalid(workRecord):
    if workRecord:
        orgId = forceRef(workRecord.value('org_id'))
        if orgId:
            orgShortName = getOrganisationShortName(orgId)
        else:
            orgShortName = forceString(workRecord.value('freeInput'))
    else:
        orgShortName = ''
    return orgShortName


def updateUsed(tempInvalidSerial, tempInvalidNumber, blankMovingId, defaultBlankMovingId):
    db = QtGui.qApp.db
    tableBlankTempInvalidMoving = db.table('BlankTempInvalid_Moving')
    tableNumb = tableBlankTempInvalidMoving
    tableBlankTempInvalidParty = db.table('BlankTempInvalid_Party')
    cond = []
    if not defaultBlankMovingId and tempInvalidSerial and blankMovingId:
        cond = [ tableBlankTempInvalidMoving['deleted'].eq(0),
                 tableBlankTempInvalidMoving['id'].eq(blankMovingId)]
    elif defaultBlankMovingId:
        blankMovingId = defaultBlankMovingId
        cond = [ tableBlankTempInvalidMoving['deleted'].eq(0),
                 tableBlankTempInvalidMoving['id'].eq(blankMovingId)]
    elif not tempInvalidSerial and blankMovingId:
        cond = [ tableBlankTempInvalidMoving['deleted'].eq(0),
                 tableBlankTempInvalidMoving['id'].eq(blankMovingId)]
    elif tempInvalidSerial and tempInvalidNumber:
        cond = [ tableBlankTempInvalidMoving['deleted'].eq(0),
                 tableBlankTempInvalidParty['deleted'].eq(0),
                 tableBlankTempInvalidParty['serial'].like(str(tempInvalidSerial)),
                 tableBlankTempInvalidMoving['numberFrom'].le(tempInvalidNumber),
                 tableBlankTempInvalidMoving['numberTo'].ge(tempInvalidNumber)]
        tableNumb = tableNumb.innerJoin(tableBlankTempInvalidParty, tableBlankTempInvalidParty['id'].eq(tableBlankTempInvalidMoving['blankParty_id']))
    elif not tempInvalidSerial and tempInvalidNumber:
        cond = [ tableBlankTempInvalidMoving['deleted'].eq(0),
                 tableBlankTempInvalidParty['deleted'].eq(0),
                 tableBlankTempInvalidParty['serial'].isNull(),
                 tableBlankTempInvalidMoving['numberFrom'].le(tempInvalidNumber),
                 tableBlankTempInvalidMoving['numberTo'].ge(tempInvalidNumber)]
        tableNumb = tableNumb.innerJoin(tableBlankTempInvalidParty, tableBlankTempInvalidParty['id'].eq(tableBlankTempInvalidMoving['blankParty_id']))
    recordMoving = db.getRecordEx(tableNumb, u'BlankTempInvalid_Moving.*', cond) if cond else None
    if recordMoving:
        used = forceInt(recordMoving.value('used'))
        blankPartyId = forceRef(recordMoving.value('blankParty_id'))
        recordMoving.setValue('used', toVariant(used + 1))
        db.updateRecord(tableBlankTempInvalidMoving, recordMoving)
        if blankPartyId:
            recordParty = db.getRecordEx(tableBlankTempInvalidParty, u'*', [tableBlankTempInvalidParty['id'].eq(blankPartyId), tableBlankTempInvalidParty['deleted'].eq(0)])
            if recordParty:
                used = forceInt(recordParty.value('used'))
                recordParty.setValue('used', toVariant(used + 1))
                db.updateRecord(tableBlankTempInvalidParty, recordParty)


class CTempInvalidDocumentsProlongModel(CTempInvalidDocumentsModel):

    Col_Include        = 0
    Col_IssueDate      = 1
    Col_Serial         = 2
    Col_Number         = 3
    Col_duplicate      = 4
    Col_duplicateReason= 5
    Col_Electronic     = 6
    Col_Busyness       = 7
    Col_PlaceWork      = 8
    Col_PrevNumber     = 9
    Col_PrevId         = 10
    Col_IssuePersonId  = 11
    Col_ExecPersonId   = 12
    Col_ChairPersonId  = 13
    Col_ClientPrimumId = 14
    Col_ClientSecondId = 15
    Col_LastId         = 16
    Col_Note           = 17

    def __init__(self, parent, clientCache):
        CTempInvalidDocumentsModel.__init__(self, parent, clientCache)
        self.addExtCol(CBoolInDocTableCol(u'Включить', 'include', 10), QVariant.Int, idx=0)


    def cellReadOnly(self, index):
        if index.column() == CTempInvalidDocumentsProlongModel.Col_Include:
            return False
        return True


    def flags(self, index):
        if self.readOnly:
           return Qt.ItemIsSelectable | Qt.ItemIsEnabled
        if index.column() == CTempInvalidDocumentsProlongModel.Col_Include:
            return Qt.ItemIsSelectable | Qt.ItemIsEnabled | Qt.ItemIsUserCheckable
        return Qt.ItemIsSelectable | Qt.ItemIsEnabled


    def getEmptyRecord(self):
        result = CTempInvalidDocumentsModel.getEmptyRecord(self)
        result.append(QtSql.QSqlField('include', QVariant.Int))
        return result


    def setData(self, index, value, role=Qt.EditRole):
        result = CTempInvalidDocumentsModel.setData(self, index, value, role)
        row = index.row()
        column = index.column()
        if role == Qt.CheckStateRole:
            if column == self.getColIndex('include'):
                if row >= 0 and row < len(self.items()):
                    self.setValue(row, 'include', QVariant(forceBool(value)))
                    self.emitCellChanged(row, column)
                    placeWork = trim(forceString(self.items()[row].value('placeWork')))
                    self.updatePlaceWork(row, column, forceBool(self.value(row, 'include')), placeWork)
                    return True
        return result


    def updatePlaceWork(self, row, column, value, placeWork):
        for i, item in enumerate(self._items):
            if placeWork == trim(forceString(item.value('placeWork'))):
                if i != row:
                   self.setValue(i, 'include', QVariant(0))
                   self.emitCellChanged(i, column)


    def setIncludeItems(self, items):
        includeItems = []
        for item in items:
            newRecord = self.getEmptyRecord()
            copyFields(newRecord, item)
            if not forceBool(newRecord.value('duplicate')):
                newRecord.setValue('include', QVariant(1))
            includeItems.append(newRecord)
        rows = len(includeItems)
        for row, item in enumerate(includeItems):
            if not forceBool(item.value('duplicate')):
                placeWork = trim(forceString(item.value('placeWork')))
                clientPrimumId = forceRef(item.value('clientPrimum_id'))
                clientSecondId = forceRef(item.value('clientSecond_id'))
                for i in range(row+1, rows):
                    if placeWork == trim(forceString(includeItems[i].value('placeWork'))):
                        if clientPrimumId == forceRef(includeItems[i].value('clientPrimum_id')) and clientSecondId == forceRef(includeItems[i].value('clientSecond_id')):
                            includeItems[i].setValue('include', QVariant(0))
        self.setItems(includeItems)
        self.reset()


from Ui_TempInvalidDocumentProlongDialog import Ui_TempInvalidDocumentProlongDialog


class CTempInvalidDocumentProlongDialog(CDialogBase, CRecordLockMixin, Ui_TempInvalidDocumentProlongDialog):
    def __init__(self,  parent, clientCache, items):
        CDialogBase.__init__(self, parent)
        self.setWindowFlags(self.windowFlags() | Qt.WindowMaximizeButtonHint)
        CRecordLockMixin.__init__(self)
        self.addObject('modelDocuments', CTempInvalidDocumentsProlongModel(self, clientCache))
        self.setupUi(self)
        self.setWindowTitleEx(u'Продлить документы временной нетрудоспособности')
        self.setWindowState(Qt.WindowMaximized)
        self.tblDocuments.setModel(self.modelDocuments)
        self.clientCache = clientCache
        self.modelDocuments.setEventEditor(self)
        self.setupDirtyCather()
        self.modelDocuments.setIncludeItems(items)
        self.tblDocuments.enableColHide(CTempInvalidDocumentsProlongModel.Col_Serial)


    def getItems(self):
        return self.modelDocuments.items()


def getTempInvalidIdOpen(clientId, type, docCode = None):
    if clientId:
        db = QtGui.qApp.db
        tableTempInvalid = db.table('TempInvalid')
        tableRBTempInvalidResult = db.table('rbTempInvalidResult')
        cond = [tableTempInvalid['deleted'].eq(0),
                tableTempInvalid['client_id'].eq(clientId),
                tableTempInvalid['closed'].eq(0),
                tableRBTempInvalidResult['closed'].eq(0),
                tableTempInvalid['type'].eq(type),
                tableRBTempInvalidResult['able'].ne(1)
                ]
        if docCode:
            tableRBTempInvalidDocument = db.table('rbTempInvalidDocument')
            cond.append(tableRBTempInvalidDocument['code'].eq(docCode))
            table = tableTempInvalid.leftJoin(tableRBTempInvalidDocument, tableTempInvalid['doctype_id'].eq(tableRBTempInvalidDocument['id']))
        else:
            table = tableTempInvalid
        table = table.leftJoin(tableRBTempInvalidResult, tableRBTempInvalidResult['id'].eq(tableTempInvalid['result_id']))
        record = db.getRecordEx(table, 'TempInvalid.*', cond, 'TempInvalid.begDate DESC')
        tempInvalidId = forceRef(record.value('id')) if record else None
        return tempInvalidId
    return None


class CMedicalCommissionModel(CInDocTableModel):
    class CLocActionPropertiColumn(CInDocTableCol):
        def __init__(self, title, fieldName, propertiName, width, **params):
            CInDocTableCol.__init__(self, title, fieldName, width, **params)
            self.actionCaches = {}
            self.propertiName = propertiName

        def updateActionCaches(self, id):
            if id in self.actionCaches.keys():
                self.actionCaches.pop(id, None)

        def clearActionCaches(self):
            self.actionCaches.clear()

        def toString(self, val, record):
            actionId = forceRef(val)
            if actionId:
                action = self.actionCaches.get(actionId, None)
                if not action:
                    action = CAction(record=record)
                    if action:
                        self.actionCaches[actionId] = action
                if action and self.propertiName in action._actionType._propertiesByName:
                    return QVariant(action[self.propertiName])
            return val

    Col_Number = 1
    Col_NumberMC = 2
    Col_Decision = 11

    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'Action', 'id', 'event_id', parent)
        self.setPersonCol = CActionPerson(u'Назначил',  'setPerson_id',   20, 'vrbPersonWithSpeciality')
        self.endPersonCol = CActionPerson(u'Выполнил',  'person_id',      20, 'vrbPersonWithSpeciality')
        self.addCol(CActionTypeTableCol(u'Тип',          'actionType_id',  15, None, classesVisible=True, showFields=CRBComboBox.showName))
        self.addCol(CMedicalCommissionModel.CLocActionPropertiColumn(u'Номер',    'id', u'Номер',    15))
        self.addCol(CMedicalCommissionModel.CLocActionPropertiColumn(u'Номер ЛН', 'id', u'Номер ЛН', 15))
        self.addCol(CEnumInDocTableCol(    u'Состояние', 'status',         10, CActionStatus.names))
        self.addCol(CDateTimeInDocTableCol(u'Назначено', 'directionDate',  10))
        self.addCol(CDateTimeInDocTableCol(u'План',      'plannedEndDate', 10))
        self.addCol(CDateTimeInDocTableCol(u'Начато',    'begDate',        10))
        self.addCol(CDateTimeInDocTableCol(u'Окончено',  'endDate',        10))
        self.addCol(CICDExInDocTableCol(   u'МКБ',       'MKB',            10))
        self.addCol(self.setPersonCol)
        self.addCol(self.endPersonCol)
        self.addCol(CMedicalCommissionModel.CLocActionPropertiColumn(u'Решение', 'id', u'Решение', 50))
        self.addCol(CInDocTableCol(        u'Примечание','note',           10))
        self.eventEditor = None
        self.readOnly = False
        self.tempInvalidId = None


    def setReadOnly(self, value):
        self.readOnly = value


    def flags(self, index):
        return Qt.ItemIsSelectable | Qt.ItemIsEnabled


    def setTempInvalidId(self, tempInvalidId):
        self.tempInvalidId = tempInvalidId


    def getTempInvalidId(self):
        return self.tempInvalidId


    def updateItems(self):
        self.loadItems(self.tempInvalidId)


    def data(self, index, role=Qt.DisplayRole):
        result = CInDocTableModel.data(self, index, role)
        row = index.row()
        if 0 <= row < len(self._items):
            if role == Qt.ToolTipRole:
                column = index.column()
                if column == CMedicalCommissionModel.Col_Decision:
                    col = self._cols[column]
                    record = self._items[row]
                    return col.toString(record.value(col.fieldName()), record)
        return result


    def loadItems(self, masterId):
        self._items = []
        self.clearColumnsCaches()
        self.setTempInvalidId(masterId)
        if masterId:
            actionTypeIdList = getActionTypeIdListByFlatCode(u'inspection_disability%')
            if actionTypeIdList:
                db = QtGui.qApp.db
                tableEvent = db.table('Event')
                eventIdList = db.getDistinctIdList(tableEvent, [tableEvent['id']], [tableEvent['tempInvalid_id'].eq(masterId), tableEvent['deleted'].eq(0)])
                if eventIdList:
                    cols = []
                    for col in self._cols:
                        if not col.external():
                            cols.append(col.fieldName())
                    cols.append(self._idFieldName)
                    cols.append(self._masterIdFieldName)
                    if self._idxFieldName:
                        cols.append(self._idxFieldName)
                    for col in self._hiddenCols:
                        cols.append(col)
                    table = self._table
                    filter = [table[self._masterIdFieldName].inlist(eventIdList)]
                    if self._filter:
                        filter.append(self._filter)
                    filter.append(table['deleted'].eq(0))
                    filter.append(table['actionType_id'].inlist(actionTypeIdList))
                    order = [u'Action.directionDate DESC']
                    self._items = db.getRecordList(table, cols, filter, order)
        self.reset()


    def getActionIdList(self):
        actionIdList = []
        for item in self._items:
            actionId = forceRef(item.value('id'))
            if actionId and actionId not in actionIdList:
                actionIdList.append(actionId)
        return actionIdList


    def setEventEditor(self, eventEditor):
        self.setPersonCol.setEventEditor(eventEditor)
        self.endPersonCol.setEventEditor(eventEditor)
        self.eventEditor = eventEditor


    def updateColumnsCaches(self, id):
        self._cols[CMedicalCommissionModel.Col_Number].updateActionCaches(id)
        self._cols[CMedicalCommissionModel.Col_NumberMC].updateActionCaches(id)
        self._cols[CMedicalCommissionModel.Col_Decision].updateActionCaches(id)


    def clearColumnsCaches(self):
        self._cols[CMedicalCommissionModel.Col_Number].clearActionCaches()
        self._cols[CMedicalCommissionModel.Col_NumberMC].clearActionCaches()
        self._cols[CMedicalCommissionModel.Col_Decision].clearActionCaches()


class CMedicalCommissionMSIModel(CMedicalCommissionModel):
    class CLocDateActionPropertiColumn(CDateInDocTableCol):
        def __init__(self, title, fieldName, propertiName, width, **params):
            CDateInDocTableCol.__init__(self, title, fieldName, width, **params)
            self.actionCaches = {}
            self.propertiName = propertiName

        def updateActionCaches(self, id):
            if id in self.actionCaches.keys():
                self.actionCaches.pop(id, None)

        def clearActionCaches(self):
            self.actionCaches.clear()

        def toString(self, val, record):
            actionId = forceRef(val)
            if actionId:
                action = self.actionCaches.get(actionId, None)
                if not action:
                    action = CAction(record=record)
                    if action:
                        self.actionCaches[actionId] = action
                if action and self.propertiName in action._actionType._propertiesByName:
                    properti = action[self.propertiName]
                    if properti:
                        return QVariant(formatDate(properti))
            return QVariant()

    Col_Decision = 13
    Col_DisabilityGroup = 14

    def __init__(self, parent):
        CMedicalCommissionModel.__init__(self, parent)
        self._cols = []
        self.setPersonCol = CActionPerson(u'Назначил',  'setPerson_id',   20, 'vrbPersonWithSpeciality')
        self.endPersonCol = CActionPerson(u'Выполнил',  'person_id',      20, 'vrbPersonWithSpeciality')
        self.addCol(CActionTypeTableCol(u'Тип',          'actionType_id',  15, None, classesVisible=True, showFields=CRBComboBox.showName))
        self.addCol(CMedicalCommissionModel.CLocActionPropertiColumn(u'Номер',    'id', u'Номер',    15))
        self.addCol(CMedicalCommissionModel.CLocActionPropertiColumn(u'Номер ЛН', 'id', u'Номер ЛН', 15))
        self.addCol(CEnumInDocTableCol(    u'Состояние', 'status',         10, CActionStatus.names))
        self.addCol(CDateTimeInDocTableCol(u'Назначено', 'directionDate',  10))
        self.addCol(CDateTimeInDocTableCol(u'План',      'plannedEndDate', 10))
        self.addCol(CDateTimeInDocTableCol(u'Начато',    'begDate',        10))
        self.addCol(CDateTimeInDocTableCol(u'Окончено',  'endDate',        10))
        self.addCol(CICDExInDocTableCol(   u'МКБ',       'MKB',            10))
        self.addCol(self.setPersonCol)
        self.addCol(self.endPersonCol)
        self.addCol(CMedicalCommissionMSIModel.CLocDateActionPropertiColumn(u'Рег-ция', 'id', u'Дата регистрации МСЭ',         50).setToolTip(u'Дата регистрации документов в бюро МСЭ'))
        self.addCol(CMedicalCommissionMSIModel.CLocDateActionPropertiColumn(u'МСЭ',     'id', u'Дата освидетельствования МСЭ', 50).setToolTip(u'Дата освидетельствования в бюро МСЭ'))
        self.addCol(CMedicalCommissionModel.CLocActionPropertiColumn(u'Результат', 'id', u'Результат', 50))
        self.addCol(CMedicalCommissionModel.CLocActionPropertiColumn(u'Группа',    'id', u'Группа инвалидности', 50))
        self.addCol(CInDocTableCol(        u'Примечание','note',           10))


    def data(self, index, role=Qt.DisplayRole):
        result = CInDocTableModel.data(self, index, role)
        row = index.row()
        if 0 <= row < len(self._items):
            if role == Qt.ToolTipRole:
                column = index.column()
                if column in [CMedicalCommissionMSIModel.Col_Decision, CMedicalCommissionMSIModel.Col_DisabilityGroup]:
                    col = self._cols[column]
                    record = self._items[row]
                    return col.toString(record.value(col.fieldName()), record)
        return result


    def updateItems(self):
        if self.tempInvalidId:
            actionTypeIdList = getActionTypeIdListByFlatCode(u'inspection_disability%')
            if actionTypeIdList:
                db = QtGui.qApp.db
                tableAction = db.table('Action')
                tableEvent = db.table('Event')
                eventIdList = db.getDistinctIdList(tableEvent, [tableEvent['id']], [tableEvent['tempInvalid_id'].eq(self.tempInvalidId), tableEvent['deleted'].eq(0)])
                if eventIdList:
                    prevActionIdList = db.getDistinctIdList(tableAction, [tableAction['id']], [tableAction['event_id'].inlist(eventIdList), tableAction['actionType_id'].inlist(actionTypeIdList), tableAction['deleted'].eq(0)])
                    self.loadItems(prevActionIdList)


    def loadItems(self, prevActionIdList):
        self._items = []
        self.clearColumnsCaches()
        if prevActionIdList:
            actionTypeIdList = getActionTypeIdListByFlatCode(u'inspection_mse%')
            if actionTypeIdList:
                db = QtGui.qApp.db
                table = self._table
                cols = [self._idFieldName,
                        self._masterIdFieldName,
                        table['prevAction_id']]
                for col in self._cols:
                    if not col.external():
                        cols.append(col.fieldName())
                if self._idxFieldName:
                    cols.append(self._idxFieldName)
                for col in self._hiddenCols:
                    cols.append(col)
                filter = [table['prevAction_id'].inlist(prevActionIdList)]
                if self._filter:
                    filter.append(self._filter)
                filter.append(table['deleted'].eq(0))
                filter.append(table['actionType_id'].inlist(actionTypeIdList))
                order = [u'Action.directionDate DESC']
                self._items = db.getRecordList(table, cols, filter, order)
        self.reset()

