# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##hospitalBedProfileId
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import os
import os.path
import re
import shutil

from PyQt4 import QtGui
from PyQt4.QtCore import QDate, QDir, QVariant, pyqtSignature, SIGNAL

from library.dbfpy.dbf  import Dbf
from library.ICDUtils   import MKBwithoutSubclassification
from library.Identification import getIdentification

from library.Utils      import ( forceString,
                                 forceInt,
                                 forceDouble,
#                                 calcAgeInDays,
                                 calcAgeInMonths,
                                 calcAgeInYears,
                                 forceRef,
                                 getInfisCodes,
                                 forceBool,
#                                 firstYearDay,
                                 trim,
                                 nameCase,
                                 pyDate,
                                 calcAgeTuple,
                                 forceDate,
                                 forceStringEx,
                                 forceDateTime,
                                 formatSNILS,
                                 formatSex,
                                 toVariant,
                               )
from Registry.Utils     import ( clientIsWorking,
                                 getClientAddress,
                                 getAddress,
                                 clientIsStudent,
                                 getClientWork,
                                 formatAddress,
                               )

from Accounting.AccountBuilder import ( CCircumstance,
                                        getEventVisitsCircumstances,
                                        getEventActionsCircumstances,
                                        getInappropriateCircumstances,
                                        filterUniqueCircumstances,
                                      )

from Accounting.ServiceDetailCache import CServiceDetailCache
from Accounting.Tariff  import CTariff
from Events.Action      import CAction #, CActionType
from Events.ActionServiceType import CActionServiceType

from Exchange.Utils     import getClientRepresentativeInfo

from Exchange.Ui_ExportEISOMSPage1 import Ui_ExportEISOMSPage1
from Exchange.Ui_ExportEISOMSPage2 import Ui_ExportEISOMSPage2


def getAccountInfo(accountId):
    db = QtGui.qApp.db
    accountRecord = db.getRecord('Account', 'date, number', accountId)
#            settleDate = forceDate(accountRecord.value('settleDate'))
    if accountRecord:
        date = forceDate(accountRecord.value('date'))
        number = forceString(accountRecord.value('number'))
    else:
        date = None
        number = ''
    return date, number, adviseDBFFileName(date, number)


def adviseDBFFileName(date, number):
    tmp = trim(''.join([c if c.isalnum() or '-_'.find(c)>=0 else ' ' for c in number]))
    return tmp.replace(' ','_')


def exportEISOMS(widget, accountId, accountItemIdList):
    wizard = CExportEISOMSWizard(widget)
    wizard.setAccountId(accountId)
    wizard.setAccountItemsIdList(accountItemIdList)
    wizard.exec_()


class CExportEISOMSWizard(QtGui.QWizard):
    def __init__(self, parent=None):
        QtGui.QWizard.__init__(self, parent)
        self.setWizardStyle(QtGui.QWizard.ModernStyle)
        self.page1 = CExportEISOMSPage1(self)
        self.page2 = CExportEISOMSPage2(self)
        self.addPage(self.page1)
        self.addPage(self.page2)
        self.setWindowTitle(u'Мастер экспорта в ЕИС.ОМС.ВМУ.АПУ')
        self.tmpDir = ''
        self.dbfFileName = ''


    def setAccountId(self, accountId):
        self.accountId = accountId
        date, number, fileName = getAccountInfo(accountId)
        self.dbfFileName = fileName
        strNumber = number if trim(number) else u'б/н'
        strDate = forceString(date) if date else u'б/д'
        self.page1.setTitle(u'Экспорт данных реестра по счёту №%s от %s' %(strNumber, strDate))
        self.page2.setTitle(u'Укажите директорий для сохранения обменного файла "%s.dbf"' %(self.dbfFileName))


    def setAccountExposeDate(self):
        db = QtGui.qApp.db
        accountRecord = db.table('Account').newRecord(['id', 'exposeDate'])
        accountRecord.setValue('id', toVariant(self.accountId))
        accountRecord.setValue('exposeDate', toVariant(QDate.currentDate()))
        db.updateRecord('Account', accountRecord)


    def getTmpDir(self):
        if not self.tmpDir:
            self.tmpDir = QtGui.qApp.getTmpDir('eisoms')
        return self.tmpDir


    def getFullDbfFileName(self, suffix=''):
        return os.path.join(self.getTmpDir(), self.dbfFileName + suffix + '.dbf')


    def getFullClientsDbfFileName(self):
        return os.path.join(self.getTmpDir(), 'pat_' + self.dbfFileName + '.dbf')


    def setAccountItemsIdList(self, accountItemIdList):
        self.page1.setAccountItemsIdList(accountItemIdList)


    def cleanup(self):
        if self.tmpDir:
            QtGui.qApp.removeTmpDir(self.tmpDir)
            self.tmpDir = ''


    def exec_(self):
        QtGui.QWizard.exec_(self)
        self.cleanup()


class CExportEISOMSPage1(QtGui.QWizardPage, Ui_ExportEISOMSPage1):
    mapDocTypeToEIS = { '1' : '1', # ПАСПОРТ РФ
                        '2' :'10', # ЗГПАСПОРТ
                        '3' : '3', # СВИД О РОЖД
#                        '2' : '6',
                        '5' : '7', # СПРАВКА ОБ ОСВ
                        '6' : '8', # ПАСПОРТ МОРФЛТ
                        '9' : '6', # ИНПАСПОРТ
                        '14': '2', # ПАСПОРТ СССР
                        '16': '8', # ПАСПОРТ МОРЯКА
                        '24':'24', # СВИД О РОЖД НЕ РФ
                      }

    childrenDocTypes = set(
                           (  3, # СВИД О РОЖД
                              6, # ИНПАСПОРТ
                             10, # ЗГПАСПОРТ
                             24, # СВИД О РОЖД НЕ РФ
                           )
                          )

    mapEventOrderToEIS    = {1: u'п', 2: u'э',               6: u'э', None:u'п'}
    mapEventOrderToForPom = {1: 3,    2:1,    3:3, 4:3, 5:3, 6:2,     None:3}
    mapEventOrderToTransf = {         2:2,    3:1,      5:4, 6:2,     None:1}

    streetTypeDict = { u'ул':     1, # улица
                       u'пр-кт':  2, # проспект
                       u'ш':      3, # шоссе
                       u'аллея':  4, # аллея
                       u'б-р':    5, # бульвар
                       u'кв-л':   6, # квартал
                       u'пер':    7, # переулок
                       u'пл':     8, # площадь
                       u'проезд': 9, # проезд
                       u'туп':   10, # тупик
                       u'наб':   11, # набережная
                       u'линия': 12, # линия
                       u'мкр':   13, # микрорайон
                       u'':      14, # другое
                     }
    # это, вероятно, лучше брать из таблицы. Но у меня нет ясного способа получать эти данные в виде таблицы :-(
    # IDOKATOREG.dbf не годится, так как требуется искать название региона в строке с произвольным форматированием
    # Есть предложение использовать RF_REGION@DBMU.GDB, но лучшее что можно сделать - избежать выгрузки поля IDOKATOREG
    mapKladrCodeToIdRerion = { 41: 93, # Камчатская обл.(вошла в состав Камчатского края)
                               59: 92, # Пермская обл.(вошла в состав Пермского края)
                               75: 94, # Читинская обл.(вошла в состав Забайкальского края)
                               80: 94, # Агинский Бурятский авт.окр.(вошел в состав Забайкальского края)
                               81: 92, # Коми-Пермяцкий авт.окр. (вошел в состав Пермского края)
                               82: 93, # Корякский авт.окр.(вошел в состав Камчатского края)
                               84: 24, # Таймырский авт.окр.(вошел в состав Красноярского края)
                               85: 38, # Усть-Ордынский Бурятский авт.окр.(вошел в состав Иркутской области)
                               88: 24, # Эвенкийский авт.окр.(вошел в состав Красноярского края)
                               90: 52, # Арзамас -16 (г.Саров) (вошел в состав Нижегородской области)
                             }

    # по данным SPRAV_D_TYPE_GROUP
    mapDirectionTypeToGroup = { 1: 2, # Направлен на консультацию в медицинскую организацию по месту прикрепления
                                2: 2, # Направлен на консультацию в иную медицинскую организацию
                                3: 2, # Направлен на обследование
                                4: 2, # Направлен в дневной стационар
                                5: 2, # Направлен на госпитализацию
                                6: 2, # Направлен в реабилитационное отделение
                                7: 1, # Талон ВМП
                                8: 3, # Диспансерное наблюдение
                                9: 2, # Направление не выдано
                               10: 4, # Направление на ЭКО
                               11: 5, # Направление на диализ
                               12: 6, # Отказ пациента от прохождения 2 этапа диспансеризации
                               14: 6, # Согласие пациента на прохождение 2 этапа диспансеризации
                               19:11, # Направление на лечение (вх)
                               20:12, # Направление на исследование
                               21:13, # Направление на 2 этап диспансеризации (вх)
                              }


    def __init__(self, parent):
        QtGui.QWizardPage.__init__(self, parent)
        self.setupUi(self)
        self.progressBar.setMinimum(0)
        self.progressBar.setMaximum(1)
        self.progressBar.setText('')

        self.setTitle(u'Экспорт данных реестра по счёту')
        self.setSubTitle(u'для выполнения шага нажмите кнопку "экспорт"')
        self.chkExportClients.setChecked(forceBool(QtGui.qApp.preferences.appPrefs.get('EISOMSExportClients', False)))
        self.chkGroupByProfile.setChecked(forceBool(QtGui.qApp.preferences.appPrefs.get('EISOMSGroupByProfile', False)))
        self.eisLpuId = forceString(QtGui.qApp.preferences.appPrefs.get('EISOMSLpuId', ''))
        self.isHospital = forceInt(QtGui.qApp.db.translate('Organisation', 'id', QtGui.qApp.currentOrgId(), 'isMedical')) == 2
        self.edtEisLpuId.setText(self.eisLpuId)
        self.setExportMode(False)
        self.aborted = False
        self.done = False
        self.idList = []
        self.mkbCache = {}
        self.serviceDetailCache = CServiceDetailCache()
        self.profileCache = {}
        self.profileSpecCache = {}
        self.kindCache = {}
        self.typeCache = {}
        self.mapMesIdToGroupCode = {}
        self.mapOrgIdToEis = {}
        self.mapOrgIdToOffsideLpuId = {}
        self.mapOrgAndOrgStructureIdToEisOrgStructureCode = {}
        self.mapServiceIdToSpravGroupNmlkId = {}
        self.mapPersonIdToEisOrgStructureCode = {}
        self.mapPersonIdToEisPersonId = {}
        self.payers = {}
        self.currentOrgId = QtGui.qApp.currentOrgId()
        self.receivingDepartmentIdentifier = self.getReceivingDepartmentIdentifier()
        self.connect(parent, SIGNAL('rejected()'), self.abort)


    def setExportMode(self, flag):
        self.btnCancel.setEnabled(flag)
        self.btnExport.setEnabled(not flag)


    def setAccountItemsIdList(self, accountItemIdList):
        self.idList = accountItemIdList


    def prepareToExport(self, ignoreConfirmation, exportClients, includeEvents, includeVisits, includeActions):
        self.done = False
        self.aborted = False
        self.emit(SIGNAL('completeChanged()'))
        self.setExportMode(True)
        dbf = self.createDbf()
        self.extServDbf = self.createExtServDbf()
        extCaseDbf = self.createExtCaseDbf()
        self.directionDbf = self.createDirectionDbf()
        self.clientsDbf = self.createClientsDbf() if exportClients else None
        query = self.createQuery(ignoreConfirmation, includeEvents, includeVisits, includeActions)
        self.progressBar.setMaximum(max(query.size(), 1))
        self.progressBar.reset()
        self.progressBar.setValue(0)
        return dbf, self.extServDbf, extCaseDbf, self.directionDbf, self.clientsDbf, query


    def export(self,
               ignoreConfirmation,
               exportClients,
               includeEvents,
               includeVisits,
               includeActions,
               groupByProfile,
               eisLpuId):
        QtGui.qApp.call(self,
                        self.exportInt,
                        (ignoreConfirmation,
                         exportClients,
                         includeEvents,
                         includeVisits,
                         includeActions,
                         groupByProfile,
                         eisLpuId))
        self.setExportMode(False)
        if self.aborted:
            self.progressBar.setText(u'прервано')
        else:
            self.progressBar.setText(u'готово')
            self.done = True
            self.emit(SIGNAL('completeChanged()'))


    def exportInt(self,
                  ignoreConfirmation,
                  exportClients,
                  includeEvents,
                  includeVisits,
                  includeActions,
                  groupByProfile,
                  eisLpuId):
        self.groupByProfile = groupByProfile
        self.eisLpuId = eisLpuId
        (self.dbf,
         self.extServDbf,
         self.extCaseDbf,
         self.directionDbf,
         self.clientsDbf,
         query)  = self.prepareToExport(ignoreConfirmation,
                                        exportClients,
                                        includeEvents,
                                        includeVisits,
                                        includeActions)
        self.knownClientIdSet = set()
        self.mapClientIdToNewbornId = {}
        self.caseRegistry = {}
        self.mapEventIdToCount = {}
        self.mapServiceToDBFRecord = {}
        self.cachedLeaveHospitalBedProfileForEventId = None
        prevEventId = None
        if self.idList:
            while query.next():
                QtGui.qApp.processEvents()
                if self.aborted:
                    break
                self.progressBar.step()
                record = query.record()
                eventId = forceRef(record.value('eventId'))
                if prevEventId != eventId:
                    self.mapServiceToDBFRecord = {}
                    prevEventId = eventId
                self.process(record)
        else:
            self.progressBar.step()

        # было обнаружено, что ЕИС не может принять данные, если в выгрузке существует
        # такой case, что не существует записи с ID_PRVS == ID_PRVS_C
        # (это может быть, если исcледование проведено - и выставляется -
        # после выставления других позиций). Принято решение искать такие данные
        # и исправлять ID_PRVS_C на ID_PRVS из первой записи.
        caseFixup = {}
        for eventId, caseInfo in self.caseRegistry.iteritems():
            eventSpecialityCode, itemSpecialityCodeList = caseInfo
            if itemSpecialityCodeList and eventSpecialityCode not in itemSpecialityCodeList:
                caseFixup[eventId] = itemSpecialityCodeList[0]

        for i in xrange(len(self.dbf)):
            dbfRecord = self.dbf[i]
            eventId = dbfRecord['SERV_ID']
            if eventId in caseFixup and dbfRecord['ID_PRVS_C'] != caseFixup[eventId]:
                dbfRecord['ID_PRVS_C'] = caseFixup[eventId]
                dbfRecord.store()

        self.dbf.close()
        self.extServDbf.close()
        self.extCaseDbf.close()
        self.directionDbf.close()
        if self.clientsDbf is not None:
            self.clientsDbf.close()
        if len(self.extServDbf) == 0 and len(self.extCaseDbf) == 0:
            os.unlink(self.extServDbf.name)
            os.unlink(self.extCaseDbf.name)
        if len(self.directionDbf) == 0:
            os.unlink(self.directionDbf.name)


    def createDbf(self):
        dbf = Dbf(self.wizard().getFullDbfFileName(), new=True, encoding='cp866')
        dbf.addField(
            ('SURNAME',    'C', 30),    # Фамилия пациента
            ('NAME1',      'C', 30),    # Имя пациента
            ('NAME2',      'C', 30),    # Отчество пациента
            ('BIRTHDAY',   'D'),        # Дата рождения
            ('SEX',        'C',  1),    # Пол (М/Ж)
            ('ORDER',      'C',  1),    # Признак экстренности случая лечения
                                        # (если случай экстренный - принимает значение "э" или "Э")
            ('POLIS_S',    'C', 10),    # Серия полиса
            ('POLIS_N',    'C', 20),    # Номер полиса
            ('POLIS_W',    'C',  5),    # Код СМО, выдавшей полис (удалить)
            ('PAYER',      'C',  5),    # Код СМО, выдавшей полис?
            ('STREET',     'C', 25),    # Адрес пациента: код улицы
            ('STREETYPE',  'C',  2),    # Адрес пациента: тип улицы
            ('AREA',       'C',  3),    # Адрес пациента: код район
            ('HOUSE',      'C',  7),    # Адрес пациента: номер дома
            ('KORP',       'C',  2),    # Адрес пациента: корпус
            ('FLAT',       'C', 15),    # Адрес пациента: номер квартиры
            ('PROFILE',    'C', 30),    # Код профиля лечения
            ('PROFILENET', 'C',  1),    # Тип сети профиля (в - взрослая, д - детская)
            ('DATEIN',     'D'),        # Дата начала услуги
            ('DATEOUT',    'D'),        # Дата окончания услуги
            ('AMOUNT',     'N', 15, 5), # Объем лечения (кратность услуги)
            ('AMOUNT_D',   'N',  3, 0), # кол-во дней (для случая МЭС ДСТАЦ при ЛПУ)
            ('DIAGNOSIS',  'C',  6),    # Код диагноза (6 - со звездой, нужно 5, - без звезды?)
            ('DIAG_PREF',  'C',  6),    # Код сопутствующего диагноза
            ('SEND',       'L'),        # Флаг обработки записи
            ('ERROR',      'C',100),    # Описание ошибки
            ('TYPEDOC',    'C',  1),    # Тип документа
            ('SER1',       'C', 10),    # Серия документа, левая часть
            ('SER2',       'C', 10),    # Серия документа, левая часть
            ('NPASP',      'C', 10),    # Номер документа
            ('SERV_ID',    'N', 11, 0), # Идентификатор случая
            ('ID_IN_CASE', 'N',  3, 0), # Порядковый номер записи в случае (в пределах одного SERV_ID)
            ('ID_PRVS',    'N', 10),    # Региональный код специальности врача
            ('IDPRVSTYPE', 'N',  2, 0), # код типа указанной услуги для КСГ
            ('PRVS_PR_G',  'N',  2, 0), # номер группы из справочника профилей ЗСКСГ
            ('ID_EXITUS',  'N',  2),    # исход лечения
            ('ILLHISTORY', 'C', 20),    # номер история болезни
            ('CASE_CAST',  'N',  2, 0), # тип случая лечения
            ('ID_GOSP',    'N',  2, 0), # тип госпитализации

            ('ID_PRMP',    'N',  3, 0), # Код профиля по Классификатору профиля
            ('ID_PRMP_C',  'N',  3, 0), # Код профиля по Классификатору профиля для случая лечения
            ('DIAG_C',     'C',  5),    # Код диагноза для случая лечения
            ('DIAG_S_C',   'C',  5),    # Код сопутствующего диагноза для случая лечения
            ('DIAG_P_C',   'C',  5),    # Код предварительного диагноза для случая лечения
            ('QRESULT',    'N',  3, 0), # Результат обращения за медицинской помощью
            ('ID_PRVS_C',  'N', 10, 0), # ID врачебной специальности для случая лечения
            ('ID_SP_PAY',  'N',  2, 0), # ID способа оплаты медицинской помощи
            ('ID_ED_PAY',  'N',  5, 2), # Количество единиц оплаты медицинской помощи
            ('ID_VMP',     'N',  2, 0), # ID вида медицинской помощи

            ('ID_DOC',     'C', 20),    # Идентификатор врача из справочника SPRAV_DOC.DBF (для услуги)
            ('ID_DEPT',    'C', 20),    # Идентификатор отделения МО из справочника SPRAV_DEPTS.DBF (для услуги)
            ('ID_DOC_C',   'C', 20),    # Идентификатор врача из справочника SPRAV_DOC.DBF (для случая)
            ('ID_DEPT_C',  'C', 20),    # Идентификатор отделения МО из справочника SPRAV_DEPTS.DBF (для случая)
            ('ID_LPU_D',   'N', 10, 0), # Идентификатор ЛПУ, направившего на лечение (из справочника SPRAV_LPU.DBF)
            ('ID_PRVS_D',  'N', 10),    # Идентификатор специальности направившего врача
            ('ID_GOAL_D',  'N', 5),     # ID цели обращения при направлении
            ('IS_CRIM',    'L'),        # криминал F - нет , T - есть
            ('LONGADDR',   'C',120),    # Длинный адрес
            ('ACC_ID',     'N', 8),     # Account.id
            ('ACCITEM_ID', 'C', 255),   # Account_Item.id
            ('CLIENT_ID',  'N', 8),     # Client.id
            ('PRIM',       'C', 255),   # примечание - возврат от страховщиков

#            ('Q_RES',      'N',  2),    # результат события/осмотра (не нужно)
#            ('PRVD',       'N',  4),    # Региональный код должности врача (не нужно)
#            ('ID_PRMP',    'N',  5),    # профиль по приказу 97

            ('IDSERVDATA', 'N', 10),    # Идентификатор пункта карты диспансеризации (SPRAV_PRVS_PROFILE.DBF)
            ('IDSERVMADE', 'N',  1),    # Идентификатор порядка выполнения пункта карты диспансеризации (Кодификатор в руководстве пользователя)
            ('IDSERVLPU',  'N', 10),    # Идентификатор ЛПУ, выполнившего пункт карты диспансеризации (из справочника SPRAV_LPU.DBF)
            ('ID_GOAL',    'N',  5),    # ID цели обращения для услуги (SPRAV_GOAL.DBF)
            ('ID_GOAL_C',  'N',  5),    # ID цели обращения для случая (SPRAV_GOAL.DBF)
            ('ID_PAT_CAT', 'N', 10),    # статус пациента (0 - без изм. 7 - не работает, 8 - работает)
            ('IDVIDVME',   'N',  2, 0), # Идентификатор вида мед. вмешательства
            ('IDFORPOM',   'N',  2, 0), # Идентификатор формы оказания помощи
            ('IDVIDHMP',   'N',  2, 0), # Идентификатор вида высокотехнологичной мед помощи
            ('IDMETHMP',   'N',  2, 0), # Идентификатор метода высокотехнологичной мед помощи
#            ('CASE_ADD_ID','N', 10, 0), # Признак группировки строк для полей IDCADDOBJ и CASEADDVAL
#            ('IDCADDOBJ',  'N',  2, 0), # Идентификатор объекта учета
#            ('CASEADDVAL', 'C', 10),    # Значение
            ('ID_LPU',     'N', 11,  0),# Идентификатор БД ЛПУ, в которую загружаются данные
            ('N_BORN',     'N',  1,  0),# Порядковый номер новорожденного
            ('IS_STAGE',   'L'),        # Признак "Этапное лечение"
            ('ID_FINT',    'N', 11,  0),# тип финансирования
            ('ID_CASE',    'C', 20),    # Идентификатор случая в БД ЕИС (проставляется после принятия)
            ('ID_SERV',    'C', 20),    # Идентификатор услуги в БД ЕИС (проставляется после принятия)
            ('SNILS',      'C', 14),    # СНИЛС пациента (для обновления) - необязательное поле
            ('ID_TRANSF',  'N', 6, 0),  # Признак перевода (для случая)
            ('ID_INCOMPL', 'N', 6, 0),  # Признак "Неполный объём" (для услуги)
            ('ID_MAIN',    'N', 3, 0),  # Идентификатор (поле ID_IN_CASE) главной услуги (для сопутствующих услуг)
            ('ID_LPU_RF',  'N',11, 0),  # Идентификатор иногородней МО
            ('ID_LPU_P',   'N',11, 0),  # Идентификатор подразделения МО (для услуги)   Обязательное    SPRAV_LPU   ID_LPU  1.2.3.146
            ('ID_LPU_P_C', 'N',11, 0),  # Идентификатор подразделения МО (для случая)   Обязательное    SPRAV_LPU   ID_LPU  1.2.3.146
            ('ID_B_PROF',  'N',11, 0),  # Идентификатор профиля койки
        )
        return dbf


    def createExtServDbf(self):
        dbf = Dbf(self.wizard().getFullDbfFileName('_V'), new=True, encoding='cp866')
        dbf.addField(
            ('SERV_ID',    'N', 11, 0), # Идентификатор случая лечения в основном файле импорта услуг
            ('ID_IN_CASE', 'N',  3, 0), # Порядковый номер услуги (записи) в рамках одного SERV_ID
            ('DATE_BEGIN', 'D'),        # Дата начала
            ('DATE_END',   'D'),        # Дата окончания
            ('ID_NMKL',    'N', 11, 0), # ID номенклатуры
            ('V_MULTI',    'N',  6, 0), # Кратность
            ('V_LONG_IVL', 'N',  6, 0), # Кол-во часов ИВЛ
            ('V_LONG_MON', 'N',  6, 0), # Кол-во часов мониторинга
            ('ERROR',      'C',200),    # Описание ошибки
        )
        return dbf


    def createExtCaseDbf(self):
        dbf = Dbf(self.wizard().getFullDbfFileName('_add'), new=True, encoding='cp866')
        dbf.addField(
            ('SERV_ID',    'N', 11, 0), # Идентификатор случая лечения в основном файле импорта услуг
            ('ID_OBJECT',  'N',  4, 0), # Идентификатор объекта учета
            ('OBJ_VALUE',  'C', 10),    # Значение объекта учета
            ('ERROR',      'C',200),    # Описание ошибки
        )
        return dbf


    def createDirectionDbf(self):
        dbf = Dbf(self.wizard().getFullDbfFileName('_D'), new=True, encoding='cp866')
        dbf.addField(
            ('SERV_ID',    'N', 11, 0), # Идентификатор случая лечения в основном файле импорта услуг
            ('D_NUMBER',   'C', 20),    # Номер направления (талона)
            ('DATE_ISSUE', 'D'),        # Дата выдачи направления (талона)
            ('DATE_PLANG', 'D'),        # Дата планируемой госпитализации
            ('ID_LPU_F',   'N', 11, 0), # Идентификатор МО (откуда)	Пусто	SPRAV_LPU	ID_LPU
            ('ID_LPU_T',   'N', 11, 0), # Идентификатор МО (куда)	Пусто	SPRAV_LPU	ID_LPU
            ('ID_D_TYPE',  'N', 11, 0), # Тип направления (назначения)
            ('ID_D_GROUP', 'N', 11, 0), # Группа направления (назначения)
            ('ID_PRVS',    'N', 11, 0), # Идентификатор специальности врача
            ('ID_OB_TYPE', 'N', 11, 0), # Идентификатор вида обследования
            ('ID_PRMP',    'N', 11, 0), # Идентификатор профиля медицинской помощи
            ('ID_B_PROF',  'N', 11, 0), # Идентификатор профиля койки
            ('ID_DN',      'N', 11, 0), # Идентификатор диспансерного наблюдения	Пусто	SPRAV_DN_ACTION	ID_DN	1.2.3.157
            ('ID_GOAL',    'N', 11, 0), # Идентификатор цели обращения	Пусто	SPRAV_GOAL	ID_GOAL	1.2.3.157
            ('ID_DIAGNOS', 'N', 11, 0), # Идентификатор диагноза	Пусто	SPRAV_DIAG	ID_DIAGNOS	1.2.3.157
            ('ID_LPU_RF',  'N', 11, 0), # Идентификатор иногородней МО	Пусто	SPRAV_LPU_RF	ID_LPU_RF	1.2.3.157
            ('ERROR',      'C',200),    # Описание ошибки
        )
        return dbf

    def createClientsDbf(self):
        dbf = Dbf(self.wizard().getFullClientsDbfFileName(), new=True, encoding='cp866')
        dbf.addField(
            ('SURNAME',    'C', 30),    # Фамилия пациента
            ('NAME',       'C', 30),    # Имя пациента
            ('S_NAME',     'C', 30),    # Отчество пациента
            ('BIRTHDAY',   'D'),        # Дата рождения
            ('SEX',        'C',  1),    # Пол (М/Ж)
            ('ID_PAT_CAT', 'N',  2, 0), # Статус
            ('DOC_TYPE',   'N',  2, 0), # Тип документа
            ('SER_L',      'C', 6),     # Серия документа, левая часть
            ('SER_R',      'C', 6),     # Серия документа, левая часть
            ('DOC_NUMBER', 'C', 12),    # Номер документа
            ('SNILS',      'C', 14),    # СНИЛС
            ('C_OKSM',     'C',  3),    # Гражданство
            ('IS_SMP',     'L'),        # Признак "пациент СМП"
            ('POLIS_TYPE', 'C', 1),     # Тип полиса
            ('POLIS_S',    'C', 20),    # Серия полиса
            ('POLIS_N',    'C', 20),    # Номер полиса
            ('ID_SMO',     'N',  3, 0), # СМО
            ('POLIS_BD',   'D'),        # Дата начала действия полиса
            ('POLIS_ED',   'D'),        # Дата окончания действия полиса
            ('ID_SMO_REG', 'N',  4, 0), # Региональная СМО (для иногородних)
            ('ADDR_TYPE',  'C',  1),    # Тип адреса
            ('IDOKATOREG', 'N',  3, 0), # Регион
            ('IDOBLTOWN',  'N',  4, 0), # - не используется
            ('ID_PREFIX',  'N',  4, 0), # - не используется
            ('ID_HOUSE',   'N',  8, 0), # Идентификатор дома (для адресов СПб, тип "г")
            ('HOUSE',      'C', 10),    # Номер дома (для типа "р")
            ('KORPUS',     'C',  5),    # Корпус (для типа "р")
            ('FLAT',       'C',  5),    # Квартира
            ('U_ADDRESS',  'C',200),    # Неструктурированный адрес (для типа "п")
            ('KLADR_CODE', 'C', 13),    # Код КЛАДР (для типа "р")
            ('STREET',     'C',150),    # Название улицы (для типа "р")
            ('IDSTRTYPE',  'N',  2, 0), # Тип улицы (для типа "р")
            ('ADDRTYPE_L', 'C',  1),    # Тип адреса
            ('OKATOREG_L', 'N',  3, 0), # Регион
            ('OBLTOWN_L',  'N',  4, 0), # - не используется
            ('PREFIX_L',   'N',  4, 0), # - не используется
            ('ID_HOUSE_L', 'N',  8, 0), # Идентификатор дома (для адресов СПб, тип "г")
            ('HOUSE_L',    'C', 10),    # Номер дома (для типа "р")
            ('KORPUS_L',   'C',  5),    # Корпус (для типа "р")
            ('FLAT_L',     'C',  5),    # Квартира
            ('U_ADDR_L',   'C',200),    # Неструктурированный адрес (для типа "п")
            ('KLADR_L',    'C', 13),    # Код КЛАДР (для типа "р")
            ('STREET_L',   'C',150),    # Название улицы (для типа "р")
            ('STRTYPE_L',  'N',  2, 0), # Тип улицы (для типа "р")
            ('PLACE_WORK', 'C',254),    # Место работы
            ('ADDR_WORK',  'C',254),    # Адрес места работы
            ('ADDR_PLACE', 'C',254),    # Место взятия
            ('REMARK',     'C',254),    # Примечение
            ('B_PLACE',    'C',100),    # Место рождения
            ('VNOV_D',     'N',  4, 0), # Вес при рождении
            ('ID_G_TYPE',  'N',  2, 0), # Тип представителя
            ('G_SURNAME',  'C', 30),    # Фамилия
            ('G_NAME',     'C', 25),    # Имя
            ('G_S_NAME',   'C', 25),    # Отчество
            ('G_BIRTHDAY', 'D'),        # Дата рождения
            ('G_SEX',      'C',  1),    # Пол
            ('G_DOC_TYPE', 'N',  2, 0), # Тип документа
            ('G_SERIA_L',  'C',  6),    # Левая часть серии документа
            ('G_SERIA_R',  'C',  2),    # Правая часть серии документа
            ('G_DOC_NUM',  'C', 12),    # Номер документа
            ('G_B_PLACE',  'C',100),    # Место рождения
            ('N_BORN',     'N',  1, 0), # Порядковый номер новорожденного
            ('SEND',       'L'),        # Признак принят (true/false), default - false
            ('ERROR',      'C',200),    # Описание ошибки
            ('ID_MIS',     'C', 20),    # Идентификатор пациента из внешних данных
            ('ID_PATIENT', 'C', 20),    # Идентификатор записи пациента в БД ЕИС (проставляется после принятия)
            ('LGOTS',      'C', 20),    # Льготы пациента (список ID через запятую)
        )
        return dbf


    def createQuery(self, ignoreConfirmation, includeEvents, includeVisits, includeActions):
        db = QtGui.qApp.db

        if includeEvents and includeVisits and includeActions:
            includeCond = '1'
        else:
            includeCondList = []
            if includeEvents:
                includeCondList.append('Account_Item.visit_id IS NULL AND Account_Item.action_id IS NULL')
            if includeVisits:
                includeCondList.append('Account_Item.visit_id IS NOT NULL AND Account_Item.action_id IS NULL')
            if includeActions:
                includeCondList.append('Account_Item.visit_id IS NULL AND Account_Item.action_id IS NOT NULL')
            if includeCondList:
                includeCond = db.joinOr(includeCondList)
            else:
                includeCond = '0'

        db = QtGui.qApp.db
        tableAccountItem = db.table('Account_Item')
        stmt = """
            SELECT
              Account_Item.id        AS accountItem_id,
              Account_Item.master_id AS account_id,
              Account_Item.event_id  AS event_id,
              Event.prevEvent_id     AS prevEvent_id,
              Account_Item.visit_id  AS visit_id,
              Account_Item.action_id AS action_id,
              Event.client_id        AS client_id,
              Event.order            AS eventOrder,
              Event.externalId       AS externalId,
              Event.MES_id           AS MES_id,
              rbMesSpecification.level = 2 AS mesIsDone,
              Event.setDate          AS setDate,
              Event.execDate         AS execDate,
              rbEventTypePurpose.code AS eventTypePurposeCode,
              EventType.serviceReason AS serviceReason,
              EventType.form         AS eventForm,
              rbMedicalAidKind.regionalCode  AS medicalAidKindCode,
              rbMedicalAidType.code  AS medicalAidTypeCode,
              Client.lastName        AS lastName,
              Client.firstName       AS firstName,
              Client.patrName        AS patrName,
              Client.birthDate       AS birthDate,
              Client.sex             AS sex,
              Client.SNILS           AS SNILS,
              ClientPolicy.serial    AS policySerial,
              ClientPolicy.number    AS policyNumber,
              ClientPolicy.begDate   AS policyBegDate,
              ClientPolicy.endDate   AS policyEndDate,
#              (SELECT COUNT(DISTINCT CP.client_id)
#               FROM ClientPolicy AS CP
#               WHERE CP.deleted = 0
#                 AND CP.client_id != Client.id
#                 AND CP.serial = ClientPolicy.serial
#                 AND CP.number = ClientPolicy.number
##                 AND CP.begDate = ClientPolicy.begDate
#              ) != 0 AS policyIsVicarious,
              EXISTS(SELECT NULL
                     FROM ClientPolicy AS CP
                     WHERE CP.deleted = 0
                       AND CP.serial = ClientPolicy.serial
                       AND CP.number = ClientPolicy.number
#                      AND CP.begDate = ClientPolicy.begDate
                       AND (   EXISTS(SELECT NULL FROM ClientRelation
                                      INNER JOIN rbRelationType ON rbRelationType.id = ClientRelation.relativeType_id
                                      WHERE ClientRelation.client_id = Client.id
                                        AND ClientRelation.relative_id = CP.client_id
                                        AND ClientRelation.deleted = 0
                                        AND rbRelationType.isBackwardRepresentative
                                     )
                            OR EXISTS(SELECT NULL FROM ClientRelation
                                      INNER JOIN rbRelationType ON rbRelationType.id = ClientRelation.relativeType_id
                                      WHERE ClientRelation.client_id = CP.client_id
                                        AND ClientRelation.relative_id = Client.id
                                        AND ClientRelation.deleted = 0
                                        AND rbRelationType.isDirectRepresentative
                                     )
                     )
              ) AS policyIsVicarious,
              rbPolicyKind.regionalCode AS policyKindRegionalCode,
              Insurer.infisCode      AS policyInsurer,
              Insurer.tfomsExtCode   AS tfomsExtCode,
              Insurer.area           AS insurerArea,
              ClientDocument.serial  AS documentSerial,
              ClientDocument.number  AS documentNumber,
              getClientCitizenship(Client.id, Event.setDate) as citizenship,
              rbDocumentType.code    AS documentType,
              rbDocumentType.regionalCode AS documentRegionalCode,
              IFNULL(rbItemService.id, rbVisitService.id) AS service_id,
              VisitDiagnosis.MKB     AS visitDiag,
              Action.MKB             AS actionDiag,
              EventDiagnosis.MKB     AS eventDiag,
              CASE ActionType.exposeDateSelector
                  WHEN 0 THEN Action.begDate
                  WHEN 1 THEN Action.begDate
                  WHEN 2 THEN Action.directionDate
                  WHEN 3 THEN TakenTissueJournal.datetimeTaken
                  ELSE Action.begDate
               END                   AS actionBegDate,
              Account_Item.serviceDate AS date,
              Account_Item.amount    AS amount,
              rbEventDiagnosticResult.regionalCode AS eventDiagnosticResultCode,
              rbVisitDiagnosticResult.regionalCode AS visitDiagnosticResultCode,
              EventResult.regionalCode AS eventResultCode,
              Event.execPerson_id AS eventPerson_id,
              EventPerson.speciality_id AS eventSpeciality_id,
              rbEventSpeciality.regionalCode AS eventSpecialityCode,
              rbActionSetPersonSpeciality.regionalCode AS actionSetPersonSpecialityCode,
              Contract_Tariff.tariffType AS tariffType,
              rbMedicalAidUnit.regionalCode AS unitCode,
              IF(Account_Item.visit_id IS NOT NULL, VisitPerson.id,
                 IF( Account_Item.action_id IS NOT NULL, ActionPerson.id, EventPerson.id)
                ) AS person_id,
              rbSpeciality.id        AS speciality_id,
              rbSpeciality.regionalCode AS specialityCode,
              rbVisitType.code       AS visitTypeCode,
              Event.relegateOrg_id   AS eventRelegateOrg_id,
              rbRelegateSpeciality.regionalCode AS relegateSpecialityCode,
              Event.srcNumber,
              Event.srcDate,
              EXISTS(
                select NULL from Action as A
                    LEFT JOIN ActionType as AT on AT.id = A.actionType_id
                    WHERE
                        A.event_id = Event.id
                        AND AT.flatCode = 'moving'
                        AND A.deleted = 0
                    LIMIT 1
                ) as 'haveMoving',
            Contract.financeSubtypeCode
            FROM Account_Item
            LEFT JOIN Account ON Account.id = Account_Item.master_id
            LEFT JOIN Contract ON Contract.id = Account.contract_id
            LEFT JOIN rbFinance ON rbFinance.id = Contract.finance_id
            LEFT JOIN Action ON Action.id = Account_Item.action_id
            LEFT JOIN Visit  ON Visit.id  = Account_Item.visit_id
            LEFT JOIN rbVisitType ON rbVisitType.id = Visit.visitType_id
            LEFT JOIN Event  ON Event.id  = Account_Item.event_id
            LEFT JOIN EventType ON EventType.id  = Event.eventType_id
            LEFT JOIN rbEventTypePurpose ON rbEventTypePurpose.id = EventType.purpose_id
            LEFT JOIN rbMedicalAidKind ON rbMedicalAidKind.id = EventType.medicalAidKind_id
            LEFT JOIN rbMedicalAidType ON rbMedicalAidType.id = EventType.medicalAidType_id
            LEFT JOIN Client ON Client.id = Event.client_id
            LEFT JOIN ClientPolicy ON ClientPolicy.id = getClientPolicyIdForDate(Client.id, rbFinance.code != '3', Account_Item.serviceDate)
            LEFT JOIN rbPolicyKind ON rbPolicyKind.id = ClientPolicy.policyKind_id
            LEFT JOIN Organisation AS Insurer ON Insurer.id = ClientPolicy.insurer_id
            LEFT JOIN ClientDocument ON
                ClientDocument.client_id = Client.id AND
                ClientDocument.id = (
                    SELECT MAX(CD.id)
                    FROM   ClientDocument AS CD
                    LEFT JOIN rbDocumentType AS rbDT ON rbDT.id = CD.documentType_id
                    LEFT JOIN rbDocumentTypeGroup AS rbDTG ON rbDTG.id = rbDT.group_id
                    WHERE  rbDTG.code = '1' AND CD.deleted=0 AND CD.client_id = Client.id)
            LEFT JOIN rbDocumentType ON rbDocumentType.id = ClientDocument.documentType_id
            LEFT JOIN Person AS VisitPerson     ON VisitPerson.id  = Visit.person_id
            LEFT JOIN Person AS ActionPerson    ON ActionPerson.id = Action.person_id
            LEFT JOIN Person AS ActionSetPerson ON ActionSetPerson.id = Action.setPerson_id
            LEFT JOIN Person AS EventPerson  ON EventPerson.id  = Event.execPerson_id
            LEFT JOIN rbSpeciality ON rbSpeciality.id = IF(Account_Item.visit_id IS NOT NULL, VisitPerson.speciality_id,
                                               IF( Account_Item.action_id IS NOT NULL, ActionPerson.speciality_id, EventPerson.speciality_id))
            LEFT JOIN rbSpeciality AS rbEventSpeciality ON rbEventSpeciality.id = EventPerson.speciality_id
            LEFT JOIN rbSpeciality AS rbActionSetPersonSpeciality ON rbActionSetPersonSpeciality.id = ActionSetPerson.speciality_id
            LEFT JOIN Diagnostic         AS VisitDiagnostic         ON VisitDiagnostic.id = getEventPersonDiagnostic(Account_Item.event_id, Visit.person_id)
            LEFT JOIN Diagnosis          AS VisitDiagnosis          ON VisitDiagnosis.id = VisitDiagnostic.diagnosis_id
            LEFT JOIN rbDiagnosticResult AS rbVisitDiagnosticResult ON rbVisitDiagnosticResult.id=VisitDiagnostic.result_id
            LEFT JOIN Diagnostic         AS EventDiagnostic         ON EventDiagnostic.id = getEventDiagnostic(Account_Item.event_id)
            LEFT JOIN Diagnosis          AS EventDiagnosis          ON EventDiagnosis.id = EventDiagnostic.diagnosis_id
            LEFT JOIN rbDiagnosticResult AS rbEventDiagnosticResult ON rbEventDiagnosticResult.id=EventDiagnostic.result_id
            LEFT JOIN rbResult           AS EventResult             ON EventResult.id=Event.result_id
            LEFT JOIN rbService          AS rbVisitService          ON rbVisitService.id = Visit.service_id
            LEFT JOIN ActionType ON ActionType.id = Action.actionType_id
            LEFT JOIN rbService AS rbItemService ON rbItemService.id = Account_Item.service_id
            LEFT JOIN rbPayRefuseType ON rbPayRefuseType.id = Account_Item.refuseType_id
            LEFT JOIN Contract_Tariff ON Contract_Tariff.id = Account_Item.tariff_id
            LEFT JOIN rbMedicalAidUnit ON rbMedicalAidUnit.id = Contract_Tariff.unit_id
            LEFT JOIN rbMesSpecification ON rbMesSpecification.id = Event.mesSpecification_id
            LEFT JOIN TakenTissueJournal ON TakenTissueJournal.id = Action.takenTissueJournal_id
            LEFT JOIN Person AS RelegatePerson ON RelegatePerson.id = Event.relegatePerson_id
            LEFT JOIN rbSpeciality AS rbRelegateSpeciality ON rbRelegateSpeciality.id = RelegatePerson.speciality_id
            WHERE
                (Account_Item.visit_id IS NOT NULL
                 OR rbItemService.eisLegacy
                 OR (Account_Item.event_id IS NOT NULL AND Event.MES_id IS NOT NULL)
                )
            AND Account_Item.reexposeItem_id IS NULL
            AND ( %d
                 OR Account_Item.date IS NULL
                 OR (Account_Item.date IS NOT NULL AND rbPayRefuseType.rerun != 0)
                )
            AND %s
            AND %s
            ORDER BY Client.id, Event.id, Account_Item.serviceDate
        """ % ((1 if ignoreConfirmation else 0), includeCond, tableAccountItem['id'].inlist(self.idList))
        query = db.query(stmt)
        return query


    def getAddrRecord(self, clientId, addrType):
        db = QtGui.qApp.db
        stmt = '''
            SELECT
                kladr.STREET.NAME AS streetName, kladr.STREET.SOCR AS streetType,
                AddressHouse.number AS number, AddressHouse.corpus AS corpus, AddressHouse.KLADRCode, AddressHouse.KLADRStreetCode,
                Address.flat AS flat, Address.id AS addressId, ClientAddress.freeInput AS freeInput
            FROM ClientAddress
            LEFT JOIN Address ON Address.id = ClientAddress.address_id
            LEFT JOIN AddressHouse ON AddressHouse.id = Address.house_id
            LEFT JOIN kladr.STREET ON kladr.STREET.CODE = AddressHouse.KLADRStreetCode
            WHERE
                ClientAddress.client_id = %d AND
                ClientAddress.id = (
                    SELECT MAX(CA.id)
                    FROM ClientAddress AS CA
                    WHERE  CA.type = %d AND CA.client_id = %d)
        ''' % (clientId, addrType, clientId)
        query = db.query(stmt)
        if query.next():
            return query.record()
        else:
            return None


    def getAreaAndRegion(self, clientId, addrType):
        clientAddressRecord = getClientAddress(clientId, addrType)
        if clientAddressRecord:
            address = getAddress(clientAddressRecord.value('address_id'))
            area, region, npunkt, street, streetType = getInfisCodes(
                address.KLADRCode, address.KLADRStreetCode,
                address.number, address.corpus)
            return area, region
        return '', ''


    def process(self, record):
        accountId = forceInt(record.value('account_id'))
        accountItemId = forceInt(record.value('accountItem_id'))
        eventId   = forceRef(record.value('event_id'))
        actionId  = forceRef(record.value('action_id'))
        eventExternalId = forceString(record.value('externalId'))
        eventSpecialityCode = forceInt(record.value('eventSpecialityCode'))
        if not eventId in self.caseRegistry:
            self.caseRegistry[eventId] = (eventSpecialityCode, [])
        eventTypePurposeCode = forceString(record.value('eventTypePurposeCode'))
        eventTypeAidKindCode = forceString(record.value('medicalAidKindCode'))
        eventTypeAidTypeCode = forceString(record.value('medicalAidTypeCode'))
        actionSetPersonSpecialityCode = forceInt(record.value('actionSetPersonSpecialityCode'))
        prevEventId = forceRef(record.value('prevEvent_id'))
        clientId  = forceInt(record.value('client_id'))
        lastName  = nameCase(forceString(record.value('lastName')))
        firstName = nameCase(forceString(record.value('firstName')))
        patrName  = nameCase(forceString(record.value('patrName')))
        birthDate = forceDate(record.value('birthDate'))
        date = forceDate(record.value('date'))
        dateIn = forceDate(record.value('setDate'))
#        ageInDays = calcAgeInDays(birthDate, date)
        ageInMonths = calcAgeInMonths(birthDate, date)
        age = calcAgeInYears(birthDate, dateIn)
        sex = forceInt(record.value('sex'))
        documentType = self.getDocumentType(forceInt(record.value('documentRegionalCode')),
                                            forceString(record.value('documentType'))
                                           )
        documentSerial=forceStringEx(record.value('documentSerial'))
        documentSeries=self.parseDocumentSerial(documentSerial)
        documentNumber = forceString(record.value('documentNumber')) or '-'
        citizenship = filter(lambda c: c.isdigit(),
                             forceString(record.value('citizenship'))) or '643'

        eventOrder = forceInt(record.value('eventOrder'))
        order = self.mapEventOrderToEIS.get(eventOrder) or self.mapEventOrderToEIS.get(None)
        forPom = self.mapEventOrderToForPom.get(eventOrder) or self.mapEventOrderToForPom.get(None)
        transf = self.mapEventOrderToTransf.get(eventOrder) or self.mapEventOrderToTransf.get(None)
        policySerial = forceString(record.value('policySerial'))
        policyNumber = forceString(record.value('policyNumber'))
        policyBegDate = forceDate(record.value('policyBegDate'))
        if not policyBegDate:
#            policyBegDate = max(QDate(1900, 1, 1), firstYearDay(date))
            policyBegDate = QDate(1900, 1, 1)
        policyEndDate = forceDate(record.value('policyEndDate'))
        if not policyEndDate:
            policyEndDate = QDate(2200, 1, 1)
        policyIsVicarious = forceBool(record.value('policyIsVicarious'))
        policyInsurer = forceString(record.value('policyInsurer'))
        insurerArea = forceString(record.value('insurerArea'))
        tfomsExtCode = forceInt(record.value('tfomsExtCode'))
        snils = formatSNILS(forceString(record.value('SNILS')))
        eventHaveMoving = forceBool(record.value('haveMoving'))
        finType = forceInt(record.value('financeSubtypeCode'))
        street = ''
        streetType = ''
        area = ''
        house = ''
        corpus = ''
        flat = ''
        longAddress = ''
        addrRecord = self.getAddrRecord(clientId, 0)
        addrType = 0
        if not addrRecord:
            addrRecord = self.getAddrRecord(clientId, 1)
            addrType = 1
        if addrRecord:
            KLADRCode=forceString(addrRecord.value('KLADRCode'))
            area, region = self.getAreaAndRegion(clientId, addrType)
            if KLADRCode.startswith('78'):
                street = forceString(addrRecord.value('streetName')) # Адрес пациента: код улицы
                streetType = forceString(addrRecord.value('streetType')) # Адрес пациента: тип улицы
                house = forceString(addrRecord.value('number')) # Адрес пациента: номер дома
                corpus  = forceString(addrRecord.value('corpus')) # Адрес пациента: корпус
                flat = forceString(addrRecord.value('flat'))   # Адрес пациента: номер квартиры
            else:
                street = '*'
            addressId =forceInt(addrRecord.value('addressId'))
            longAddress = formatAddress(addressId)

        if not finType:
            if eventTypeAidTypeCode in ('4','5'):
                if not insurerArea or insurerArea.startswith('78'):
                    finType = 4
                else:
                    finType = 1
            else:
                finType = 1

#        if citizenship == '643' and age<1 and documentType not in self.childrenDocTypes: # мелкий и документ - не детский, предположительно - родительский
#            # сбрасываем документ
#            documentType = 5
#            documentNumber = '-'
#            documentSeries = ['-', '-']
#            # устанавливаем отметку "новорожденный"
#            newbornId = self.getNewbornId(clientId)
#        else:
#            newbornId = 0

#        newbornId = self.mapClientIdToNewbornId.get(clientId)
#        if newbornId is not None:
#            pass
#        elif ageInMonths>=3 or (policyNumber and birthDate < policyBegDate <= date):
        if ageInMonths>=4 or (policyNumber and birthDate < policyBegDate and not policyIsVicarious):
            newbornId = 0
        else:
            newbornId = self.getNewbornId(clientId)

        patCat = 0
#        if age<1 and documentType!=3: # мелкий и документ - не свидетельство о рождении
        if newbornId:
            patCat = 9
        elif age<7:
            patCat = 10
        elif age<14:
            patCat = 11
        elif age<18:
#            isStudent = clientIsStudent(clientId)
#            if isStudent:
#                patCat = 12
            patCat = 12
        else:
            isStudent = clientIsStudent(clientId)
            if isStudent:
                patCat = 12
            else:
                isPens = (age>=60 and sex==1) or (age>=55 and sex !=1)
                patCat = (4 if clientIsWorking(clientId) else 6) + (1 if isPens else 0)
        serviceId = forceRef(record.value('service_id'))
        serviceDetail = self.serviceDetailCache.get(serviceId)
        profileNet = u'в' if age>=18 else u'д'
        amount = forceDouble(record.value('amount'))
        eventDiag = forceString(record.value('eventDiag'))
        actionDiag = forceString(record.value('actionDiag'))
        visitDiag = forceString(record.value('visitDiag'))
#        resultCode = forceInt(record.value('resultCode'))
        specialityId = forceRef(record.value('speciality_id'))
        specialityCode = forceInt(record.value('specialityCode'))
        personId = forceRef(record.value('person_id'))
        eventPersonId = forceRef(record.value('eventPerson_id'))
        eventSpecialityId = forceRef(record.value('eventSpeciality_id'))
        mesId = forceRef(record.value('MES_id'))
#        mesIsDone = forceBool(record.value('mesIsDone'))
        tariffType = forceInt(record.value('tariffType'))
        visitTypeCode = forceString(record.value('visitTypeCode'))
        mesGroupCode = self.getMesGroupCode(mesId)
        circumstances = []
        if not record.value('visit_id').isNull() or not record.value('action_id').isNull():
            # выставление визита или действия
            circumstances = [CCircumstance(date, personId, specialityId, specialityCode, 0, 0, True, 0, None, 1, 5, self.currentOrgId)]
        elif eventId and mesId:
            # выставление события
            if tariffType == CTariff.ttVisitsByMES:
                availabilityCounters = {}
                circumstances = getEventVisitsCircumstances(availabilityCounters, eventId, mesId)
                onlyVisitLike = mesGroupCode not in (u'ДиспанС', u'МедОсм')
                circumstances += getEventActionsCircumstances(availabilityCounters, eventId, mesId, onlyVisitLike)
                if mesGroupCode in (u'ДиспанС', u'МедОсм'):
                    circumstances += getInappropriateCircumstances(availabilityCounters, eventId, mesId)
                circumstances = filterUniqueCircumstances(circumstances,
                                                          serviceDetail.infisCode.startswith((u'И',u'и'))
                                                         )
            elif tariffType == CTariff.ttEventByMES or tariffType == CTariff.ttEventByMESLen:
                # событие по МЭС и событие по МЭС с учётом длительности
                circumstances = [CCircumstance(date, personId, specialityId, specialityCode, 0, 1, True, 0, None, 1, 5, self.currentOrgId)]
            else:
                circumstances = [CCircumstance(date, personId, specialityId, specialityCode, 0, 0, True, 0, None, 1, 5, self.currentOrgId)]
            amount = 1
        else:
            circumstances = [CCircumstance(date, personId, specialityId, specialityCode, 0, 0, True, 0, None, 1, 5, self.currentOrgId)]

        eventAidProfileCode, eventAidKindCode, eventAidTypeCode, eventAidProfileId = self.getAidCodesAndProfileId(serviceDetail, dateIn, eventSpecialityId, birthDate, sex, eventDiag)
        eventRelegateOrgId      = forceRef(record.value('eventRelegateOrg_id'))
        eventRelegateOrOwnOrgId = eventRelegateOrgId or self.currentOrgId

        for date, personId, specialityId, specialityCode, prvsTypeId, prvsGroup, important, additionalServiceCode, originatedFromAction, servMade, incompl, orgId in circumstances:
            serviceAidProfileCode, serviceAidKindCode, serviceAidTypeCode = self.getAidCodes(serviceDetail, dateIn, specialityId, birthDate, sex, actionDiag or eventDiag)
            aidProfileCode = serviceAidProfileCode or eventAidProfileCode
            aidKindCode = serviceAidKindCode or eventAidKindCode or eventTypeAidKindCode
            aidTypeCode = serviceAidTypeCode or eventAidTypeCode or eventTypeAidTypeCode
            aidKindCodeAsInt = forceInt(QVariant(aidKindCode))

            if  (   tariffType == CTariff.ttEventByMES
                 or tariffType == CTariff.ttEventByMESLen
                 or (    eventTypeAidTypeCode == '7'  # Дневной стационар
                     and tariffType in ( CTariff.ttEventAsCoupleVisits,
                                         CTariff.ttHospitalBedDay,
                                         CTariff.ttVisitsByMES,
                                         CTariff.ttCoupleVisits
                                       )
                    )
                ):
                amountDays = forceInt(record.value('amount'))
                amount = 1
                dateIn = forceDate(record.value('setDate'))
            elif(    (   tariffType == CTariff.ttActionAmount
                      or tariffType == CTariff.ttActionUET
                     )
                 and serviceDetail.infisCode.startswith('43')
                ):
                amountDays = forceInt(record.value('amount'))
                amount = 1
                dateIn = forceDate(record.value('actionBegDate')) or date
            else:
                amountDays = 0
                dateIn = date

            spPay = forceInt(record.value('unitCode'))

            caseCast = forceInt(record.value('serviceReason'))
            if not caseCast:
                if mesId and mesGroupCode in(u'ДиспанС', u'МедОсм'):
                    if prevEventId:
                        caseCast = (12 if age>=18 else 16)
                    else:
                        caseCast = (10 if age>=18 else 14)
                elif mesId and mesGroupCode == u'ПрофК':
                    caseCast = (19 if age>=18 else 20)
                elif eventTypeAidTypeCode in ('1', '2', '3'):
                    caseCast = 6 # Стационар
                    if not amountDays:
                        amountDays = 1
                elif eventTypeAidTypeCode in ('4', '5'):
                    if mesId:
                        caseCast = 18 # СМП c МЭС
                    else:
                        caseCast = 8 # СМП
                elif visitTypeCode == u'ПД': #
                    caseCast = (19 if age>=18 else 20)
                else:
                    caseCast = (2 if aidTypeCode == '7' else 0) + (1 if mesId else 0) + 1

            modifyPref = QtGui.qApp.getGlobalPreference('24') # Желчь брыжжет фонтаном...
            if caseCast == 2 and actionId and modifyPref==u'да': #0006955: Добавить в описание услуги атрибут для определения CASE_CAST
                caseCast = 1

            # 0008576: ТФОМС СПб. Изменение профиля оплаты при экспорте
            serviceCode = serviceDetail.infisCode
            if (     tariffType == CTariff.ttVisit
                 and serviceCode in (u'аТерУч', u'аВОПУч', u'аПедУч')
               ):
                if order == u'э':
                    serviceCode += u'Н'
                elif self.serviceRepetionCount(eventId, serviceCode) > 1:
                    serviceCode += u'О'


            # 0008580: ТФОМС СПб. Изменение CASE_CAST при экспорте
            if (     tariffType == CTariff.ttVisit
#                 and visitSceneCode in ('2', '3')
                 and serviceCode in (u'кТерап', u'кПедиа', u'кОбщПр')
                 and caseCast in (34, 35, 36, 37)
               ):
                caseCast = 1
            elif (   tariffType == CTariff.ttVisit
                 and serviceCode in (u'аТерУчО', u'аВОПУчО', u'аПедУчО')
                 and caseCast == 35
               ):
                caseCast = 36
                spPay = 30
            # 8580, ~0019704:
            if caseCast == 39 and (policyInsurer == u'кТФ3' or not insurerArea.startswith('78')):
                caseCast = 2


            ##################################################### beg of ID_GOAL
            # 0008570: ТФОМС СПб. Вычисление ID_GOAL
            # 1. Если значение поля CASE_CAST соответствует 1, 2, 34б 37 или 39,
            #    а также порядок события соответствует значению "неотложная помощь",
            #    то заполняется 3.
            if ( caseCast in (1, 2, 34, 37, 39)
                 and eventOrder in (2, 6)
               ):
                    goalCode = 3 # Посещение в связи с оказанием неотложной помощи
            # 2. Если значение поля CASE_CAST соответствует 1, 2, 34б 35 или 39,
            #    а также параметр "назначение" типа события соответствует значению "прочее",
            #    заполняется 4.
            elif ( caseCast in (1, 2, 34, 35, 39)
                   and eventTypePurposeCode == '4'
                 ):
                goalCode = 4 # Посещение с иными целями
            # 3. Если значение поля CASE_CAST соответствует 1, 2, 34 или 39,
            #    а также тарифицируется визит с кодом типа="ПД",
            #    либо тарифицируется МЭС с группой из списка ("МедОсм", "ДиспанС", "ПрофК")
            #    заполняется 2.
            elif ( caseCast in (1, 2, 34, 39)
                   and (   visitTypeCode == u'ПД'
                        or mesGroupCode in (u'МедОсм', u'ДиспанС', u'ПрофК')
                       )
                 ):
                goalCode = 2 # Посещение с профилактической целью
            # 4. Если значение поля CASE_CAST соответствует 1  или 34,
            #    а также есть визит с кодом типа="ЗубПр" (возможно, лучше будет назвать как-то по-другому этот Тип визита "Подготовка к зубопротезированию", но я не придумала),
            #    заполняется 6.
            elif ( caseCast in (1, 34)
                   and visitTypeCode == u'ЗубПр'
                 ):
                goalCode = 6 # зубопротезирование
            # 5. Если значение поля CASE_CAST соответствует 1, 2, 34, 36 или 39,
            #    а также заключительный диагноз не начинается на на буквы Z и O и
            #    дата начала события не соответствует дате окончания события,
            #    заполняется 5.
            elif ( caseCast in (1, 2, 34, 36, 39)
                   and not eventDiag.startswith(('O', 'Z'))
                   and forceDate(record.value('setDate')) != forceDate(record.value('execDate'))
                 ):
                goalCode = 5 # Обращение по поводу заболевания
            # 6. Если значение поля CASE_CAST соответствует 1, 2, 34, 35 или 39,
            #    а также заключительный диагноз не начинается на на буквы Z и O
            #    и дата начала события соответствует дате окончания события,
            #    заполняется 7.
            elif ( caseCast in (1, 2, 34, 35, 39)
                   and not eventDiag.startswith(('O', 'Z'))
                   and forceDate(record.value('setDate')) == forceDate(record.value('execDate'))
                 ):
                goalCode = 7 # Разовое обращение по поводу заболевания
            # 7. Если значение поля CASE_CAST соответствует 1, 2, 34, 35 или 39,
            #    а также случай не попал в предыдущие категории,
            #    заполняется 4.
            elif ( caseCast in (1, 2, 34, 35, 39)
                 ):
                goalCode = 4 # Посещение с иными целями
            # 8. Если значение поля CASE_CAST соответствует 10, 11, 12, 13, 14, 15, 16 или 17,
            #    заполняется 2.
            elif ( caseCast in (10, 11, 12, 13, 14, 15, 16, 17)
                 ):
                goalCode = 2 # Посещение с профилактической целью
            # 9. Если значение поля CASE_CAST соответствует 19, 20, 21, 22, 23, 24, 28, 29, 30 или 32,
            # заполняется 2.
            elif ( caseCast in (19, 20, 21, 22, 23, 24, 28, 29, 30, 32)
                 ):
                goalCode = 2 # Посещение с профилактической целью
            elif ( caseCast == 44
                   and mesGroupCode == u'ДиспанС'
                 ):
                goalCode = 13 # 0009766: Диспансеризация 1 раз в 2 года (ID_GOAL = 13)
            # 10. При всех прочих значениях поля CASE_CAST, заполняется 1.
            else:
                goalCode = 1 # без уточнения
            ##################################################### end of ID_GOAL

            hospType = 5 # нет госпитализации
            if caseCast == 6:
                if not eventHaveMoving:
                    hospType = 2 # приемное отделение
                elif dateIn == date or forceDateTime(record.value('setDate')).secsTo(forceDateTime(record.value('execDate'))) < 86400:
                    hospType = 3 # Досуточная госпитализация
                else:
                    hospType = 1 # Стационар
            elif caseCast == 7:
                if self.isHospital:
                    hospType = 4 # Дневной стационар с МЭС

            if aidKindCodeAsInt == 2: # СМП
                callCardNumber = forceString(QtGui.qApp.db.translate('EmergencyCall', 'event_id', eventId, 'numberCardCall'))
                if callCardNumber:
                    illHistoryId = u'%s s'% (callCardNumber, )
                else:
                    illHistoryId = u'%d i' % (eventId, )

                preliminaryDiag  = ''
#            elif eventExternalId and eventTypeAidTypeCode in ('1', '2', '3', '7'):
            else:
                if eventExternalId:
                    illHistoryId = u'%s' % (eventExternalId, )
                else:
                    illHistoryId = u'%d i' % (eventId, )
                preliminaryDiag = self.formatDiagCode(eventDiag)


            if (      self.groupByProfile
                  and tariffType == CTariff.ttActionAmount
                  and serviceDetail.infisCode in (u'иБХим', u'иКлин', u'иГорм',  u'иИмму')
               ):
                key = (eventId, pyDate(dateIn), serviceDetail.infisCode, )
                if key in self.mapServiceToDBFRecord:
                    dbfRecord = self.mapServiceToDBFRecord[key]
                else:
                    dbfRecord = self.dbf.newRecord()
                    self.mapServiceToDBFRecord[key] = dbfRecord
            else:
                dbfRecord = self.dbf.newRecord()

            serviceCode = serviceDetail.infisCode
            if (     tariffType == CTariff.ttVisit
                 and re.match(u'^а(Пед|Тер|ВОП)Уч$', serviceCode)
               ):
                if order == u'э':
                    serviceCode += u'Н'
                elif self.serviceRepetionCount(eventId, serviceCode) > 1:
                    serviceCode += u'О'

            dbfRecord['SURNAME']    = lastName  # Фамилия пациента
            dbfRecord['NAME1']      = firstName # Имя пациента
            dbfRecord['NAME2']      = patrName  # Отчество пациента
            dbfRecord['BIRTHDAY']   = pyDate(birthDate)  # дата рождения
            dbfRecord['SEX']        = formatSex(sex)     # Пол (М/Ж)
            dbfRecord['POLIS_S']    = policySerial  # Серия полиса
            dbfRecord['POLIS_N']    = policyNumber  # Номер полиса
            dbfRecord['POLIS_W']    = policyInsurer # Код СМО, выдавшей полис
            dbfRecord['PAYER']      = '' # Код СМО, выдавшей полис?
            dbfRecord['TYPEDOC']    = str(documentType)  # Тип документа
            dbfRecord['SER1']       = documentSeries[0] # Серия документа, левая часть
            dbfRecord['SER2']       = documentSeries[1] # Серия документа, левая часть
            dbfRecord['NPASP']      = documentNumber# Номер документа
            dbfRecord['SNILS']      = snils
            dbfRecord['STREET']     = street       # Адрес пациента: код улицы
            dbfRecord['STREETYPE']  = streetType   # Адрес пациента: тип улицы
            dbfRecord['AREA']       = area         # Адрес пациента: код район
            dbfRecord['HOUSE']      = house        # Адрес пациента: номер дома
            dbfRecord['KORP']       = corpus       # Адрес пациента: корпус
            dbfRecord['FLAT']       = flat         # Адрес пациента: номер квартиры
            dbfRecord['LONGADDR']   = longAddress  # дополнительно для поиска пациента
            dbfRecord['ORDER']      = order        # Признак экстренности случая лечения (если случай экстренный - принимает значение "э" или "Э")
            dbfRecord['SERV_ID']    = eventId      # идентификатор случая
            dbfRecord['ID_IN_CASE'] = self.mapEventIdToCount[eventId] = self.mapEventIdToCount.get(eventId, 0)+1 # Порядковый номер записи в случае (в пределах одного SERV_ID)
            dbfRecord['PROFILE']    = serviceCode  # Код профиля лечения
            dbfRecord['PROFILENET'] = profileNet   # Тип сети профиля (в - взрослая, д - детская)

            dbfRecord['DATEIN']     = pyDate(dateIn) # Дата начала услуги
            dbfRecord['DATEOUT']    = pyDate(date) # Дата окончания услуги
            dbfRecord['AMOUNT']     += amount       # Объем (кратность?) лечения
            dbfRecord['AMOUNT_D']   += amountDays   # Длительность лечения в днях
            dbfRecord['DIAGNOSIS']  = self.formatDiagCode(actionDiag or visitDiag or eventDiag) # Код диагноза
            dbfRecord['SEND']       = False        # Флаг обработки записи
            dbfRecord['ERROR']      = ''           # Описание ошибки
            dbfRecord['ID_PRVS']    = specialityCode
            dbfRecord['IDPRVSTYPE'] = prvsTypeId or 1 # говорят, что значение 0 "табу"
            dbfRecord['PRVS_PR_G']  = prvsGroup  or 1 # говорят, что значение 0 "табу"
            dbfRecord['ID_EXITUS']  = forceInt(record.value('visitDiagnosticResultCode')) or forceInt(record.value('eventDiagnosticResultCode'))
            dbfRecord['ILLHISTORY'] = illHistoryId
            dbfRecord['CASE_CAST']  = caseCast
            dbfRecord['ID_GOSP']    = hospType
            dbfRecord['ID_PRMP']    = aidProfileCode # Код профиля по Классификатору профиля
            dbfRecord['ID_PRMP_C']  = eventAidProfileCode # Код профиля по Классификатору профиля для случая лечения
            dbfRecord['DIAG_C']     = self.formatDiagCode(eventDiag)                # Код диагноза для случая лечения
            dbfRecord['DIAG_S_C']   = ''                                            # Код сопутствующего диагноза для случая лечения
            dbfRecord['DIAG_P_C']   = preliminaryDiag                               # Код предварительного диагноза для случая лечения
            dbfRecord['QRESULT']    = forceInt(record.value('eventResultCode'))     # Результат обращения за медицинской помощью
            dbfRecord['ID_PRVS_C']  = eventSpecialityCode                           # Код врачебной специальности для случая лечения
            dbfRecord['ID_SP_PAY']  = spPay                                         # ID способа оплаты медицинской помощи
            dbfRecord['ID_ED_PAY']  = forceDouble(record.value('amount'))           # Количество единиц оплаты медицинской помощи
            dbfRecord['ID_VMP']     = aidKindCodeAsInt                              # ID вида медицинской помощи
            dbfRecord['ID_DOC']     = self.getPersonIdentifier(personId)  # Идентификатор врача из справочника SPRAV_DOC.DBF (для услуги)
            dbfRecord['ID_DOC_C']   = self.getPersonIdentifier(eventPersonId) # Идентификатор врача из справочника SPRAV_DOC.DBF (для случая)
            if hospType == 2:
                dbfRecord['ID_DEPT']    = self.receivingDepartmentIdentifier or self.getOrgStructureIdentifier(personId)      # Идентификатор отделения МО из справочника SPRAV_DEPTS.DBF (для услуги)
                dbfRecord['ID_DEPT_C']  = self.receivingDepartmentIdentifier or self.getOrgStructureIdentifier(eventPersonId) # Идентификатор отделения МО из справочника SPRAV_DEPTS.DBF (для случая)
            else:
                dbfRecord['ID_DEPT']    = self.getOrgStructureIdentifier(personId) # Идентификатор отделения МО из справочника SPRAV_DEPTS.DBF (для услуги)
                dbfRecord['ID_DEPT_C']  = self.getOrgStructureIdentifier(eventPersonId) # Идентификатор отделения МО из справочника SPRAV_DEPTS.DBF (для случая)

            if tariffType == CTariff.ttActionAmount and actionSetPersonSpecialityCode:
                directionLpuId  = self.getOrgIdentifier(eventRelegateOrOwnOrgId, True)
#                directionSpecialityCode = actionSetPersonSpecialityCode
#                directionGoalId = goalCode
            elif eventRelegateOrOwnOrgId and eventSpecialityCode:
                directionLpuId  = self.getOrgIdentifier(eventRelegateOrOwnOrgId, True)
#                directionSpecialityCode = eventSpecialityCode
#                directionGoalId = goalCode
            else:
                directionLpuId  = 0
#                directionSpecialityCode = 0
#                directionGoalId = 0

            offsideLpuId = self.getOffsideLpuId(eventRelegateOrOwnOrgId)
            if offsideLpuId is None:
                directionLpuIdTuple = ( directionLpuId,   # Идентификатор ЛПУ, направившего на лечение (из справочника SPRAV_LPU.DBF)
                                        0
                                      )
            else:
                directionLpuIdTuple = ( 20578,  # что соответствует по справочнику LPU в ЕИС Иногородней МО
                                        forceInt(toVariant(offsideLpuId))
                                      )

            dbfRecord['IS_CRIM']    = False # криминал F - нет , T - есть

#            dbfRecord['Q_RES']      = resultCode
            dbfRecord['IDSERVDATA'] = additionalServiceCode                         # Идентификатор пункта карты диспансеризации

            orgStructureIdentifier = self.getOrgStructureIdentifier2(personId)
            if not orgStructureIdentifier:
                orgStructureIdentifier = self.getOrgIdentifier(orgId, True)
            personOrgStructureIdentifier = orgStructureIdentifier

            eventOrgStructureIdentifier = self.getOrgStructureIdentifier2(eventPersonId)
            if not eventOrgStructureIdentifier:
                eventOrgStructureIdentifier = self.getOrgIdentifier(orgId, True)

            if servMade == 1:
                if personOrgStructureIdentifier != eventOrgStructureIdentifier:
                    servMade = 2
                orgStructureIdentifier = personOrgStructureIdentifier
            elif servMade == 3:
                personOrgStructureIdentifier = eventOrgStructureIdentifier
            elif servMade == 4:
                orgStructureIdentifier = personOrgStructureIdentifier = eventOrgStructureIdentifier
            elif servMade == 5:
                personOrgStructureIdentifier = eventOrgStructureIdentifier
            else:
                personOrgStructureIdentifier = eventOrgStructureIdentifier

#            if orgId == self.currentOrgId:
#                orgStructureIdentifier = self.getOrgStructureIdentifier2(personId)
#                if orgStructureIdentifier and servMade == 1:
#                    servMade = 2
#            else:
#                orgStructureIdentifier = 0
#            if not orgStructureIdentifier:
#                orgStructureIdentifier = self.getOrgIdentifier(orgId, True)

            dbfRecord['IDSERVMADE'] = servMade
            dbfRecord['IDSERVLPU']  = orgStructureIdentifier
            dbfRecord['ID_GOAL_C']  = goalCode
            dbfRecord['ID_GOAL']    = goalCode
            dbfRecord['ID_PAT_CAT'] = patCat         # статус пациента
            dbfRecord['IDVIDVME']   = 1              # Идентификатор вида мед. вмешательства
            dbfRecord['IDFORPOM']   = forPom         # Идентификатор формы оказания помощи
            dbfRecord['IDVIDHMP']   = 0              # Идентификатор вида высокотехнологичной мед помощи
            dbfRecord['IDMETHMP']   = 0              # Идентификатор метода высокотехнологичной мед помощи
            dbfRecord['ID_LPU']     = self.eisLpuId  # Идентификатор БД ЛПУ, в которую загружаются данные
            dbfRecord['N_BORN']     = newbornId      # Порядковый номер новорожденного
            dbfRecord['ACC_ID']     = accountId

            if dbfRecord['ACCITEM_ID'] == '':
                dbfRecord['ACCITEM_ID'] = str(accountItemId)
            else:
                dbfRecord['ACCITEM_ID'] += ' ' + str(accountItemId)
            dbfRecord['CLIENT_ID']  = clientId
            dbfRecord['ID_FINT']    = finType
            if eventTypeAidTypeCode in ('1', '2', '3', '7'): # 0008916 -- заполнять ID_TRANSF не во всех случаях
                dbfRecord['ID_TRANSF']  = transf
            dbfRecord['ID_INCOMPL'] = incompl
            dbfRecord['ID_LPU_P']   = personOrgStructureIdentifier # Идентификатор подразделения МО (для услуги)
            dbfRecord['ID_LPU_P_C'] = eventOrgStructureIdentifier  # Идентификатор подразделения МО (для случая)


            if eventTypeAidTypeCode in ('1', '2', '3', '7'):
                dbfRecord['ID_B_PROF']  = self.getHospitalBedProfile(eventId,
                                                                     eventAidProfileId if eventTypeAidTypeCode == '7' else None,  # Дневной стационар
                                                                    )
            dbfRecord.store()

            self.caseRegistry[eventId][1].append(specialityCode)
            if self.isReanimation(serviceId):
                self.addExtService(dbfRecord['SERV_ID'],
                                   dbfRecord['ID_IN_CASE'],
                                   eventId,
                                   actionId
                                  )
            if self.mapEventIdToCount[eventId] == 1: # первая запись для этого eventId
                eventFormIs001 = forceString(record.value('eventForm')) == '001'
                relegateSpecialityCode = forceInt(record.value('relegateSpecialityCode'))

                if (    ( eventRelegateOrgId and not eventFormIs001 ) # есть направление
                     or eventTypeAidTypeCode == '7'  # дневной стационар
                     or caseCast in (12,13,44,45) # Второй этап ДД
                   ) :
                    if caseCast in (12,13,44,45): # Второй этап ДД
                        directionType = 21  # (Направление на ДД2)
                    else:
                        directionType = 19  # (Направление на лечение)
                    self.addDirection(eventId,
                                      directionType = directionType,
                                      number = forceString(record.value('srcNumber')),
                                      date   = forceDate(record.value('srcDate')) or forceDate(record.value('setDate')),
                                      hospitalBedProfileId = None,
                                      examinationType = None,
                                      specialityCode = relegateSpecialityCode or eventSpecialityCode,
                                      idLpuTuple = directionLpuIdTuple
                                     )

                if caseCast in (1,10,11,13,15,19,20,24,34,35,37,45):
                    directionGroup = None
                    if self.eventHasResearch(eventId):
                        specialityCode = relegateSpecialityCode or eventSpecialityCode
                        if eventFormIs001:
                            lpuTuple = directionLpuIdTuple
                        else:
#                            specialityCode = eventSpecialityCode
                            lpuTuple = (self.getOrgIdentifier(self.currentOrgId, True), 0)
                        if caseCast in (11,13,15,20,24,34,45):
                            directionGroup = 14
                        else:
                            directionGroup = 12

                        self.addDirection(eventId,
                                          directionType = 20,
                                          directionGroup = directionGroup,
                                          goalCode = goalCode,
                                          specialityCode = specialityCode,
                                          idLpuTuple = lpuTuple
                                         )
                    if caseCast in (11,13,15,20,24,34,45) and directionGroup != 14:
                        specialityCode = relegateSpecialityCode or eventSpecialityCode
                        self.addDirection(eventId,
                                          directionType = 19, # лечение
                                          directionGroup = 14,
                                          number = forceString(record.value('srcNumber')),
                                          date = forceDate(record.value('srcDate')) or forceDate(record.value('setDate')),
                                          goalCode = goalCode,
                                          specialityCode = specialityCode,
                                          idLpuTuple = directionLpuIdTuple
                                         )


                if eventTypeAidTypeCode in ('1', '2', '3'):
                    self.addExtCase(eventId)
                    self.addDirections(True, False, False, eventId)
                else:
                    self.addDirections(False, mesGroupCode in (u'ДиспанС', u'МедОсм'), forceInt(record.value('eventResultCode')) in (89, 90, 91, 94, 95),  eventId)

        if self.clientsDbf is not None and clientId not in self.knownClientIdSet:
            self.knownClientIdSet.add(clientId)
            #if self.clientDoesNotHaveEISId(clientId):
            if True:
                smoReg = 0
                if citizenship != '643': # указано гражданство - не Россия
                    tfomsExtCode = self.getPayer(u'иКом')
                else:
                    if record.isNull('insurerArea'):
                        # будем это считать признаком, что нет полиса
                        tfomsExtCode = self.getPayer(u'кТФ1') # нет полиса
                    else:
                        if not insurerArea.startswith('78'):
                            smoReg = tfomsExtCode
                            tfomsExtCode = self.getPayer(u'кТФ3') # не питер
                dbfRecord = self.clientsDbf.newRecord()
                dbfRecord['SURNAME']   = lastName          # Фамилия пациента
                dbfRecord['NAME']      = firstName         # Имя пациента
                dbfRecord['S_NAME']    = patrName          # Отчество пациента
                dbfRecord['BIRTHDAY']  = pyDate(birthDate) # Дата рождения
                dbfRecord['SEX']       = formatSex(sex)    # Пол (М/Ж)
                dbfRecord['ID_PAT_CAT']= patCat            # Статус (2 - неопределено), ?!

                dbfRecord['DOC_TYPE']  = documentType      # Тип документа
                dbfRecord['SER_L']     = documentSeries[0] # Серия документа, левая часть
                dbfRecord['SER_R']     = documentSeries[1] # Серия документа, левая часть
                dbfRecord['DOC_NUMBER']= documentNumber    # Номер документа
                dbfRecord['SNILS']     = snils             # СНИЛС
                dbfRecord['C_OKSM']    = citizenship       # Гражданство ('643' - РФ, '000' - Не определено) ?!
                dbfRecord['IS_SMP']    = False             # Признак "пациент СМП"
                if policyInsurer == u'нКом' and (tfomsExtCode == 389 or smoReg == 389):
                    policyKind = '0'
                elif policyInsurer == u'иКом' and (tfomsExtCode == 41 or smoReg == 41):
                    policyKind = '0'
                else:
                    policyKind = forceString(record.value('policyKindRegionalCode'))
                dbfRecord['POLIS_TYPE']= policyKind        # Тип полиса
                dbfRecord['POLIS_S']   = policySerial      # Серия полиса
                dbfRecord['POLIS_N']   = policyNumber      # Номер полиса
                dbfRecord['ID_SMO']    = tfomsExtCode      # СМО
                dbfRecord['POLIS_BD']  = pyDate(policyBegDate) # Дата начала действия полиса
                dbfRecord['POLIS_ED']  = pyDate(policyEndDate) # Дата окончания действия полиса
                dbfRecord['ID_SMO_REG'] = smoReg # Региональная СМО (для иногородних)

                addrType, regOkatoId, houseId, house, corpus, flat, freeInput, KLADRCode, street, streetTypeId  = self.getClientAddressParts(clientId, 0)

                dbfRecord['ADDR_TYPE'] = addrType   # Тип адреса
                dbfRecord['IDOKATOREG']= regOkatoId # Регион
                dbfRecord['ID_HOUSE']  = houseId    # Идентификатор дома (для адресов СПб, тип "г")
                dbfRecord['HOUSE']     = house      # Номер дома (для типа "р")
                dbfRecord['KORPUS']    = corpus     # Корпус (для типа "р")
                dbfRecord['FLAT']      = flat       # Квартира
                dbfRecord['U_ADDRESS'] = freeInput  # Неструктурированный адрес (для типа "п")
                dbfRecord['KLADR_CODE']= KLADRCode  # Код КЛАДР (для типа "р")
                dbfRecord['STREET']    = street     # Название улицы (для типа "р")
                dbfRecord['IDSTRTYPE'] = streetTypeId # Тип улицы (для типа "р")
                addrType, regOkatoId, houseId, house, corpus, flat, freeInput, KLADRCode, street, streetTypeId = self.getClientAddressParts(clientId, 1)
                dbfRecord['ADDRTYPE_L']= addrType   # Тип адреса
                dbfRecord['OKATOREG_L']= regOkatoId # Регион
                dbfRecord['ID_HOUSE_L']= houseId    # Идентификатор дома (для адресов СПб, тип "г")
                dbfRecord['HOUSE_L']   = house      # Номер дома (для типа "р")
                dbfRecord['KORPUS_L']  = corpus     # Корпус (для типа "р")
                dbfRecord['FLAT_L']    = flat       # Квартира
                dbfRecord['U_ADDR_L']  = freeInput  # Неструктурированный адрес (для типа "п")
                dbfRecord['KLADR_L']   = KLADRCode  # Код КЛАДР (для типа "р")
                dbfRecord['STREET_L']  = street     # Название улицы (для типа "р")
                dbfRecord['STRTYPE_L'] = streetTypeId # Тип улицы (для типа "р")

                #dbfRecord['PLACE_WORK'] =   # Место работы
                #dbfRecord['ADDR_WORK'] =   # Адрес места работы
                #dbfRecord['ADDR_PLACE'] =  # Место взятия
                #dbfRecord['REMARK'] =   # Примечение
                #dbfRecord['B_PLACE'] =  # Место рождения

                dbfRecord['N_BORN'] = newbornId # Порядковый номер новорожденного
                dbfRecord['SEND']   = False    # Признак принят (true/false), default - false
                dbfRecord['ID_MIS'] = clientId # Идентификатор пациента из внешних данных

                if newbornId:
                    representativeInfo = getClientRepresentativeInfo(clientId)
                    if representativeInfo:
                        try:
                            relationType = int(representativeInfo['regionalRelationTypeCode'])
                        except:
                            relationType = 0
                        dbfRecord['ID_G_TYPE'] = relationType # Тип представителя
                        dbfRecord['G_SURNAME'] = representativeInfo['lastName']  # Фамилия
                        dbfRecord['G_NAME']    = representativeInfo['firstName'] # Имя
                        dbfRecord['G_S_NAME']  = representativeInfo['patrName']  # Отчество
                        dbfRecord['G_BIRTHDAY']= pyDate(representativeInfo['birthDate']) # Дата рождения
                        dbfRecord['G_SEX']     = formatSex(representativeInfo['sex'])   # Пол
                        dbfRecord['G_DOC_TYPE']= self.getDocumentType( # Тип документа
                                                    representativeInfo['documentTypeRegionalCode'],
                                                    representativeInfo['documentTypeCode'])
                        dbfRecord['G_SERIA_L'], dbfRecord['G_SERIA_R'] = self.parseDocumentSerial(representativeInfo['serial'])
                        representativeInfo['serial']
                        dbfRecord['G_DOC_NUM'] = representativeInfo['number'] or '-' # Номер документа
                        #dbfRecord['G_B_PLACE'] =    # Место рождения
                dbfRecord.store()


    def getDocumentType(self, regionalCode, code):
        if regionalCode and isinstance(regionalCode, (int, long)):
            return regionalCode
        result = regionalCode or self.mapDocTypeToEIS.get(code, '')
        if result and result.isdigit():
            return int(regionalCode)
        return 5


    def parseDocumentSerial(self, documentSerial):
        if len(documentSerial)==4 and documentSerial.isdigit():
            return [documentSerial[:2], documentSerial[2:]]
        result = trim(documentSerial.replace('-', ' ')).split(None, 1)
        return [ result[0] if len(result)>0 and result[0] else '-',
                 result[1] if len(result)>1 and result[1] else '-'
               ]


    def getPayer(self, code):
        if not code in self.payers:
            db = QtGui.qApp.db
            self.payers[code] = forceInt(db.translate('Organisation', 'infisCode', code, 'tfomsExtCode'))
        return self.payers[code]


    def formatDiagCode(self, mkb):
        result = self.mkbCache.get(mkb, None)
        if result is None:
            prim    = forceString(QtGui.qApp.db.translate('MKB', 'DiagID', MKBwithoutSubclassification(mkb), 'Prim'))
            # новая вводная: в случае субклассификации без пятого знака
            # субклассификация должна помещаться на пятый знак
            result = MKBwithoutSubclassification(mkb.replace(' ', '')) + prim
            self.mkbCache[mkb] = result
        return result


    def getMedicalAidProfileCodes(self, profileId):
        if profileId:
            item = self.profileCache.get(profileId, None)
            if item is None:
                record = QtGui.qApp.db.getRecord('rbMedicalAidProfile', ['regionalCode', 'federalCode'], profileId)
                if record:
                    item = ( forceInt(record.value(0)),
                             forceInt(record.value(1))
                           )
                    self.profileCache[profileId] = item
            if item:
                return item
        return None


    def getAidCodesAndProfileId(self, serviceDetail, date, specialityId, birthDate, sex, diagnosis):
        age = calcAgeTuple(birthDate, date)
        profileId, kindId, typeId = serviceDetail.getMedicalAidIds(specialityId, sex, age, diagnosis)
        profileCodesTuple = self.getMedicalAidProfileCodes(profileId)
        if profileCodesTuple:
            profileRegionalCode, profileFederalCode = profileCodesTuple
        else:
            profileId, profileRegionalCode = self.profileSpecCache.get(specialityId,  (None, None))
            if profileRegionalCode is None:
#               profileRegionalCode = profileFederalCode = 0 # это ноль
                profileRegionalCode = 0 # это ноль
                db = QtGui.qApp.db
                tableSpeciality = db.table('rbSpeciality')
                tableProfile = db.table('rbMedicalAidProfile')
                table = tableSpeciality.leftJoin(tableProfile, tableSpeciality['OKSOName'].eq(tableProfile['name']))
                record = db.getRecordEx(table,
                                        tableProfile['id'],
                                        tableSpeciality['id'].eq(specialityId))
                if record:
                    profileId = forceInt(record.value(0))
                    profileCodesTuple = self.getMedicalAidProfileCodes(profileId)
                    if profileCodesTuple:
                        profileRegionalCode, profileFederalCode = profileCodesTuple
                self.profileSpecCache[specialityId] = profileId, profileRegionalCode
        if kindId:
            kindCode = self.kindCache.get(kindId, None)
            if kindCode is None:
                kindCode = forceString(QtGui.qApp.db.translate('rbMedicalAidKind', 'id', kindId, 'regionalCode'))
                self.kindCache[kindId] = kindCode
        else:
            kindCode = ''
        if typeId:
            typeCode = self.typeCache.get(typeId, None)
            if typeCode is None:
                typeCode = forceString(QtGui.qApp.db.translate('rbMedicalAidType', 'id', typeId, 'code'))
                self.typeCache[typeId] = typeCode
        else:
            typeCode = ''
        return profileRegionalCode, kindCode, typeCode, profileId


    def getAidCodes(self, serviceDetail, date, specialityId, birthDate, sex, diagnosis):
        profileRegionalCode, kindCode, typeCode, profileId = self.getAidCodesAndProfileId(serviceDetail, date, specialityId, birthDate, sex, diagnosis)
        return profileRegionalCode, kindCode, typeCode


    def getMesGroupCode(self, mesId):
        result = self.mapMesIdToGroupCode.get(mesId, None)
        if result is None:
            db = QtGui.qApp.db
            groupId = forceRef(db.translate('mes.MES', 'id', mesId, 'group_id'))
            result  = forceString(db.translate('mes.mrbMESGroup', 'id', groupId, 'code'))
            self.mapMesIdToGroupCode[mesId] = result
        return result


    def getPersonIdentifier(self, personId):
        result = self.mapPersonIdToEisPersonId.get(personId, None)
        if result is None:
            if personId is None:
                result = '0'
            else:
                db = QtGui.qApp.db
                result = forceString(db.translate('Person', 'id', personId, 'regionalCode'))
                if result:
                    if result.find('.') < 0:
                        result = str(self.eisLpuId) + '.' + result
                else:
                    orgId = forceInt(db.translate('Person', 'id', personId, 'org_id'))
                    if orgId != self.currentOrgId:
                        result = '0'
                    else:
                        result = '%s.%s' % (self.getOrgIdentifier(self.currentOrgId, True), personId)
            self.mapPersonIdToEisPersonId[personId] = result
        return result


    def getOrgIdentifier(self, orgId, substOwnCode):
        result = self.mapOrgIdToEis.get(orgId, None)
        if result is None:
            if orgId is None:
                result = 0
            else:
                result = forceInt(QtGui.qApp.db.translate('Organisation', 'id', orgId, 'tfomsExtCode'))
            self.mapOrgIdToEis[orgId] = result

        if substOwnCode and orgId == self.currentOrgId:
            return self.eisLpuId or result
        else:
            return result or self.eisLpuId


    def getOffsideLpuId(self, orgId):
        # для ЛПУ вне СПб - отдельный справочник.
        result = self.mapOrgIdToOffsideLpuId.get(orgId, False)
        if result is False:
            result = getIdentification('Organisation',
                                       orgId,
                                       'urn:tfoms78:SPRAV_LPU_RF',
                                       False
                                      )
            self.mapOrgIdToOffsideLpuId[orgId] = result
        return result


    def getReceivingDepartmentIdentifier(self):
        result = 0
        db = QtGui.qApp.db
        tableOrgStructure = db.table('OrgStructure')
        cond = [ tableOrgStructure['deleted'].eq(0),
                 tableOrgStructure['type'].eq(4), # приёмное отделение
                 tableOrgStructure['organisation_id'].eq(self.currentOrgId),
               ]
        record = db.getRecordEx(tableOrgStructure, 'infisCode', cond, 'id')
        if record:
            result = forceInt(record.value(0))
        return result


    def getOrgStructureIdentifier(self, personId):
        result = 1
        db = QtGui.qApp.db
        tablePerson = db.table('Person')
        tableOrgStructure = db.table('OrgStructure')
        table = tablePerson.leftJoin(tableOrgStructure, tablePerson['orgStructure_id'].eq(tableOrgStructure['id']))
        cond = [tablePerson['id'].eq(personId)]
        record = db.getRecordEx(table, 'OrgStructure.infisCode',  cond)
        if record:
            result = forceInt(record.value(0)) or 1
        return result


    # насрать и розами засыпать.
    def getOrgStructureIdentifier2(self, personId):
        result = self.mapPersonIdToEisOrgStructureCode.get(personId, None)
        if result is None:
            db = QtGui.qApp.db
            record = db.getRecord('Person', ['org_id','orgStructure_id'], personId)
            if record:
                orgId = forceRef(record.value('org_id'))
                orgStructureId = forceRef(record.value('orgStructure_id'))
                result = self.getOrgStructureIdentifier2Int(orgId, orgStructureId)
            else:
                result = 0
            self.mapPersonIdToEisOrgStructureCode[personId] = result
        return result


    def getOrgStructureIdentifier2Int(self, orgId, orgStructureId):
        result = self.mapOrgAndOrgStructureIdToEisOrgStructureCode.get((orgId, orgStructureId), None)
        if result is None:
            db = QtGui.qApp.db
            record = db.getRecord(db.table('OrgStructure'),
                                  ['tfomsCode', 'parent_id', 'organisation_id'],
                                  orgStructureId)
            if record:
                tfomsCode = forceInt(record.value('tfomsCode'))
                parentId  = forceRef(record.value('parent_id'))
                if not orgId:
                    orgId = forceRef(record.value('organisation_id'))
                if tfomsCode:
                    result = tfomsCode
                elif parentId:
                    result = self.getOrgStructureIdentifier2Int(orgId, parentId)
            if result is None:
                result = forceInt(db.translate('Organisation', 'id', orgId, 'tfomsExtCode'))
            self.mapOrgAndOrgStructureIdToEisOrgStructureCode[(orgId, orgStructureId)] = result
        return result



#    def clientDoesNotHaveEISId(self, clientId):
#        asEIS = '2'
#        asEISMU = '3'
#
#        db = QtGui.qApp.db
#        tableClientIdentification = db.table('ClientIdentification')
#        tableAccountingSystem = db.table('rbAccountingSystem')
#        table = tableClientIdentification.leftJoin(tableAccountingSystem, tableAccountingSystem['id'].eq(tableClientIdentification['accountingSystem_id']))
#        cnt = db.getCount(table, where=db.joinAnd([tableClientIdentification['client_id'].eq(clientId),
#                                                   tableClientIdentification['deleted'].eq(0),
#                                                   tableAccountingSystem['code'].inlist((asEIS, asEISMU))
#                                                  ]
#                                                 )
#                         )
#        return cnt == 0


    def getClientAddressParts(self, clientId, addrType):
        addrRecord = self.getAddrRecord(clientId, addrType)
        if not addrRecord and addrType == 0:
            addrRecord = self.getAddrRecord(clientId, 1)

        eisAddrType = u''
        regOkatoId = 0
        houseId = 0
        house = ''
        corpus = ''
        flat = ''
        freeInput = ''
        KLADRCode = ''
        street = ''
        streetTypeId = 0

        if addrRecord:
            KLADRCode=forceString(addrRecord.value('KLADRCode'))
            freeInput = forceString(addrRecord.value('freeInput'))
            street = forceString(addrRecord.value('streetName')) or '-'
            streetType = forceString(addrRecord.value('streetType'))
            streetTypeId = self.streetTypeDict.get(streetType, self.streetTypeDict[''])
            house = forceString(addrRecord.value('number'))
            corpus = forceString(addrRecord.value('corpus'))
            flat = forceString(addrRecord.value('flat'))
            if KLADRCode:
                regOkatoId = int(KLADRCode[:2])
                if KLADRCode.startswith('78'):
                    eisAddrType = u'г'
                    houseId = self.getEisHouseId(addrRecord.value('KLADRCode'), addrRecord.value('KLADRStreetCode'), addrRecord.value('number'), addrRecord.value('corpus'))
                else:
                    eisAddrType = u'р'
                    regOkatoId = self.mapKladrCodeToIdRerion.get(regOkatoId, regOkatoId)
            else:
                eisAddrType = u'п'
        return eisAddrType, regOkatoId, houseId, house, corpus, flat, freeInput, KLADRCode, street, streetTypeId


    def getEisHouseId(self, KLADRCode, KLADRStreetCode, number, corpus):
        db = QtGui.qApp.db
        if corpus == '-':
            corpus = ''
        stmt = 'SELECT kladr.getEisHouseId(%s, %s, %s, %s) AS X' % ( db.formatValueEx(QVariant.String, KLADRCode),
                                                                     db.formatValueEx(QVariant.String, KLADRStreetCode),
                                                                     db.formatValueEx(QVariant.String, number),
                                                                     db.formatValueEx(QVariant.String, corpus)
                                                                   )

        query = db.query(stmt)
        if query.first():
            record=query.record()
            return forceInt(record.value(0))
        else:
            return -1


    def isClientWorking(self, clientId):
        getClientWork(clientId)

# номер новорожденного можно получать как-то так -
    def getNewbornId(self, clientId):
        result = self.mapClientIdToNewbornId.get(clientId)
        if result is None:
            stmt = '''SELECT SQL_NO_CACHE
CA2.client_id FROM
(SELECT CA.type, CA.address_id, CA.freeInput, Client.birthDate
 FROM ClientAddress AS CA
 INNER JOIN Client ON Client.id = CA.client_id
 LEFT JOIN ClientAddress AS CAF ON (CAF.deleted, CAF.type, CAF.client_id) = (0, CA.type, CA.client_id)
                                AND CAF.id>CA.id
 WHERE CA.deleted = 0
   AND CA.type = 1
   AND CA.client_id = %(clientId)d
   AND CAF.id IS NULL
) AS CAS
INNER JOIN ClientAddress AS CA2 ON CA2.type = CAS.type
                                AND CA2.deleted=0
                                AND (CA2.address_id = CAS.address_id OR CA2.freeInput = CAS.freeInput AND CA2.address_id IS NULL AND CAS.address_id IS NULL)
INNER JOIN Client AS Client2 ON Client2.id = CA2.client_id AND Client2.birthDate = CAS.birthDate
LEFT JOIN ClientAddress AS CA2F ON (CA2F.deleted, CA2F.type, CA2F.client_id) = (0, CA2.type, CA2.client_id)
                                AND CA2F.id>CA2.id
WHERE CA2F.id IS NULL
  AND Client2.deleted = 0
ORDER BY CA2.client_id''' % {'clientId':clientId }
            query = QtGui.qApp.db.query(stmt)
            newbornId = 0
            while query.next():
                id = forceInt(query.value(0))
                self.mapClientIdToNewbornId[id] = newbornId = newbornId+1
            result = self.mapClientIdToNewbornId.get(clientId)
            if result is None:
                self.mapClientIdToNewbornId[clientId] = result = 1
        return result


    def isReanimation(self, serviceId):
        result = self.mapServiceIdToSpravGroupNmlkId.get(serviceId, False)
        if result is False:
            result = getIdentification('rbService',
                                       serviceId,
                                       'urn:tfoms78:SPRAV_GROUP_NMKL',
                                       False
                                       )
            self.mapServiceIdToSpravGroupNmlkId[serviceId] = result
        return result


    def addExtService(self,
                      servId,
                      idInCase,
                      eventId,
                      actionId
                     ):
        mapPropNameToField = { u'ИВЛ':       'V_LONG_IVL',
                               u'Кратность': 'V_MULTI',
                               u'Мониторинг':'V_LONG_MON',
                             }

        db = QtGui.qApp.db
        record = db.getRecord('Action', ['begDate', 'endDate'], actionId)
        groupBegDateTime = forceDateTime(record.value('begDate'))
        groupEndDateTime = forceDateTime(record.value('endDate'))
        if not groupBegDateTime:
            groupBegDateTime = QDate(1, 1, 1)
        if not groupEndDateTime:
            groupEndDateTime = QDate(9999, 12, 31)

        tableAction = db.table('Action')
        tableActionType = db.table('ActionType')
        tableActionProperty = db.table('ActionProperty')
        tableActionPropertyType = db.table('ActionPropertyType')
        tableActionPropertyInteger = db.table('ActionProperty_Integer')
        tableService = db.table('rbService')
        tableServiceIdentification = db.table('rbService_Identification')
        tableAccountingSystem = db.table('rbAccountingSystem')

        table = tableAction
        table = table.innerJoin(tableActionType,         tableActionType['id'].eq(tableAction['actionType_id']))
        table = table.innerJoin(tableService,            tableService['id'].eq(tableActionType['nomenclativeService_id']))
        table = table.innerJoin(tableServiceIdentification, tableServiceIdentification['master_id'].eq(tableService['id']))
        table = table.innerJoin(tableAccountingSystem,   tableAccountingSystem['id'].eq(tableServiceIdentification['system_id']))
        table = table.leftJoin(tableActionPropertyType,  [ tableActionPropertyType['actionType_id'].eq(tableAction['actionType_id']),
                                                           tableActionPropertyType['typeName'].eq('Integer'),
                                                           tableActionPropertyType['name'].inlist(mapPropNameToField.keys()),
                                                           tableActionPropertyType['deleted'].eq(0),
                                                         ]
                              )
        table = table.leftJoin(tableActionProperty,      [ tableActionProperty['type_id'].eq(tableActionPropertyType['id']),
                                                           tableActionProperty['action_id'].eq(tableAction['id']),
                                                           tableActionProperty['deleted'].eq(0),
                                                         ]
                              )
        table = table.leftJoin(tableActionPropertyInteger, tableActionPropertyInteger['id'].eq(tableActionProperty['id']))

        cond = [ tableAction['deleted'].eq(0),
                 tableAction['event_id'].eq(eventId),
                 tableAction['endDate'].between(groupBegDateTime, groupEndDateTime),
                 tableServiceIdentification['deleted'].eq(0),
                 tableAccountingSystem['urn'].eq('urn:tfoms78:SPRAV_NMKL'),
               ]

        records = db.getRecordList(table,
                                   [ tableAction['id'],
                                     tableServiceIdentification['value'].alias('nmklId'),
                                     tableAction['begDate'],
                                     tableAction['endDate'],
                                     tableActionPropertyType['name'],
                                     tableActionPropertyInteger['value'],
                                   ],
                                   cond
                                  )
        data = {}
        for record in records:
            id = forceRef(record.value('id'))
            nkmlId  = forceInt(record.value('nmklId'))
            begDate = forceDate(record.value('begDate'))
            endDate = forceDate(record.value('endDate'))
            name = forceString(record.value('name'))
            value = forceInt(record.value('value'))
            key = (id, nkmlId)
            item = data.setdefault(key, (begDate, endDate, {}))
            item[2][name] = value

        for key, item in data.iteritems():
            id, nkmlId = key
            begDate, endDate, values = item
            dbfRecord = self.extServDbf.newRecord()
            dbfRecord['SERV_ID']    = servId
            dbfRecord['ID_IN_CASE'] = idInCase
            dbfRecord['DATE_BEGIN'] = pyDate(begDate)
            dbfRecord['DATE_END']   = pyDate(endDate)
            dbfRecord['ID_NMKL']    = nkmlId
            for propName, fieldName in mapPropNameToField.iteritems():
                value = values.get(propName, None)
                if value:
                    dbfRecord[fieldName] = value
            dbfRecord.store()


    def addExtCase(self, eventId):
        mapPropNameToObjectId = { 'ASA'  : 17,
                                  'SOFA1': 1801,
                                  'SOFA2': 1802,
                                  'SOFA3': 1803,
                                  'SOFA4': 1804,
                                  'SOFA5': 1805,
                                  'SOFA6': 1806,
                                }

        db = QtGui.qApp.db
        tableAction = db.table('Action')
        tableActionType = db.table('ActionType')
        tableActionProperty = db.table('ActionProperty')
        tableActionPropertyType = db.table('ActionPropertyType')
        tableActionPropertyString = db.table('ActionProperty_String')

        table = tableActionPropertyString
        table = table.innerJoin(tableActionProperty,     tableActionProperty['id'].eq(tableActionPropertyString['id']))
        table = table.innerJoin(tableActionPropertyType, tableActionPropertyType['id'].eq(tableActionProperty['type_id']))
        table = table.innerJoin(tableAction,             tableAction['id'].eq(tableActionProperty['action_id']))
        table = table.innerJoin(tableActionType,         tableActionType['id'].eq(tableAction['actionType_id']))

        cond = [ tableActionPropertyType['shortName'].inlist(mapPropNameToObjectId.keys()),
                 tableActionPropertyType['deleted'].eq(0),
                 tableActionProperty['deleted'].eq(0),
                 tableAction['deleted'].eq(0),
                 tableActionType['serviceType'].inlist((CActionServiceType.operation,
                                                        CActionServiceType.reanimation,
                                                       )
                                                      ),
                 tableActionPropertyString['value'].ne(''),
                 tableAction['event_id'].eq(eventId)
               ]
        records = db.getRecordList(table,
                                   [ tableAction['id'].alias('action_id'),
                                     tableActionPropertyType['shortName'],
                                     tableActionPropertyString['value'],
                                   ],
                                   cond,
                                   tableAction['endDate'].name()+' desc'
                                  )
        prevActionId = None
        for record in records:
            actionId = forceRef(record.value('action_id'))
            if prevActionId and prevActionId != actionId:
                break
            prevActionId = actionId
            name  = forceString(record.value('shortName'))
            value = forceString(record.value('value'))
            if name == 'ASA':
                value = value[:1]
            objectId = mapPropNameToObjectId.get(name, None)
            if objectId:
                dbfRecord = self.extCaseDbf.newRecord()
                dbfRecord['SERV_ID']    = eventId
                dbfRecord['ID_OBJECT']  = objectId
                dbfRecord['OBJ_VALUE']  = value
                dbfRecord.store()


    def eventHasResearch(self, eventId):
        db = QtGui.qApp.db
        tableAction = db.table('Action')
        tableActionType = db.table('ActionType')
        table = tableAction.innerJoin(tableActionType,  tableActionType['id'].eq(tableAction['actionType_id']))
        cnt = db.getCount(table,
                          where=db.joinAnd([ tableAction['deleted'].eq(0),
                                             tableAction['event_id'].eq(eventId),
                                             tableActionType['serviceType'].inlist([CActionServiceType.research,
                                                                                    CActionServiceType.labResearch,
                                                                                   ])
                                           ]
                                          )
                         )
        return cnt != 0


    def addDirections(self,
                      hospitalCase,
                      forceAddDirection,
                      checkForRefusalFromDD2,
                      eventId
                     ):
        db = QtGui.qApp.db
        cnt = len(self.directionDbf)
        refusalDD2 = None
        if hospitalCase:
            table = db.table('vActionLeaved')
            record = db.getRecordEx(table,
                                    '*',
                                    db.joinAnd(['deleted=0',
                                                table['event_id'].eq(eventId)
                                               ]
                                              )
                                   )
            if record:
                outcome = forceString(record.value('propOutcome')).lower()
                hospitalBedProfileId = record.value('propHospitalBedProfle_id')
                if outcome.endswith(u'в дневной стационар'):
                    self.addDirection(eventId,
                                      directionType = 4,
                                      hospitalBedProfileId = hospitalBedProfileId
                                     )
                elif outcome == u'переведен в другой стационар':
                    self.addDirection(eventId,
                                      directionType = 5,
                                      hospitalBedProfileId = hospitalBedProfileId
                                     )
                elif outcome.endswith((u'в санаторий', u'восстановит лечение')):
                    self.addDirection(eventId,
                                      directionType = 6,
                                      hospitalBedProfileId = hospitalBedProfileId
                                     )
        else:
            tableAction = db.table('Action')
            tableActionType = db.table('ActionType')
            table = tableAction.innerJoin(tableActionType,
                                          tableActionType['id'].eq(tableAction['actionType_id']))
            for (flatCode, directionType, examinationType) in (
                                        ('consultationDirection', 2, None),
                                        ('hospitalDirection',     5, None),
                                        ('inspectionDirection',   3, 2),
                                        ('recoveryDirection',     6, None),
                                        ('researchDirection',     3, 3)
                                        ):
                records = db.getRecordList(table,
                                           'Action.*',
                                            db.joinAnd([ tableAction['deleted'].eq(0),
                                                         tableAction['event_id'].eq(eventId),
                                                         tableActionType['flatCode'].eq(flatCode)
                                                       ]
                                                      )
                                          )
                for record in records:
                    action = CAction(record=record)
                    self.addDirection(eventId,
                                      directionType        = directionType,
                                      number               = action.get(u'Номер'),
                                      date                 = forceDate(record.value('begDate')),
                                      hospitalBedProfileId = action.get(u'Профиль'),
                                      examinationType      = examinationType
                                     )
            if forceAddDirection and checkForRefusalFromDD2:
                refusalDD2 = db.getCount(table,
                                         '1',
                                         db.joinAnd([ tableAction['deleted'].eq(0),
                                                      tableAction['event_id'].eq(eventId),
                                                      tableActionType['flatCode'].eq('refusal')
                                                    ]
                                                   )
                                         )

        if forceAddDirection and cnt == len(self.directionDbf):
            self.addDirection(eventId,
                              directionType = 9 # без направления
                             )

        if refusalDD2 is not None:
            if refusalDD2:
                self.addDirection(eventId,
                                  directionType = 12 # отказ от ДД2
                                 )
            else:
                self.addDirection(eventId,
                                  directionType = 14 # согласие на ДД2
                                 )


    def addDirection(self,
                     eventId,
                     directionType = 0,
                     directionGroup = None,
                     number = None,
                     date = None,
                     hospitalBedProfileId = None,
                     examinationType = None,
                     idLpuTuple = None,
                     specialityCode = None,
                     goalCode = None
                    ):
        db = QtGui.qApp.db
        if hospitalBedProfileId:
            medicalAidProfileId = db.translate('rbHospitalBedProfile', 'id', hospitalBedProfileId, 'medicalAidProfile_id')
            profileCode = forceRef(db.translate('rbMedicalAidProfile', 'id',  medicalAidProfileId, 'regionalCode'))
        else:
            profileCode = None
        dbfRecord = self.directionDbf.newRecord()
        dbfRecord['SERV_ID']    = eventId
        dbfRecord['ID_D_TYPE']  = directionType
        dbfRecord['ID_D_GROUP'] = directionGroup or self.mapDirectionTypeToGroup.get(directionType, 0)

        if number and isinstance(number, basestring):
            dbfRecord['D_NUMBER'] = number
        if date and isinstance(date, QDate):
            dbfRecord['DATE_ISSUE'] = pyDate(date)
        if examinationType:
            dbfRecord['ID_OB_TYPE'] = examinationType
        if profileCode:
            dbfRecord['ID_PRMP'] = profileCode
        if idLpuTuple:
            assert isinstance(idLpuTuple, (tuple, list)) and len(idLpuTuple)==2
            dbfRecord['ID_LPU_F'], dbfRecord['ID_LPU_RF'] = idLpuTuple
        if specialityCode:
            dbfRecord['ID_PRVS'] = specialityCode
        if goalCode:
            dbfRecord['ID_GOAL'] = goalCode


#            ('SERV_ID',    'N', 11, 0), # Идентификатор случая лечения в основном файле импорта услуг
#            ('D_NUMBER',   'C', 20),    # Номер направления (талона)
#            ('DATE_ISSUE', 'D'),        # Дата выдачи направления (талона)
#            ('DATE_PLANG', 'D'),        # Дата планируемой госпитализации
#            ('ID_LPU_F',   'N', 11, 0), # Идентификатор МО (откуда)	Пусто	SPRAV_LPU	ID_LPU
#            ('ID_LPU_T',   'N', 11, 0), # Идентификатор МО (куда)	Пусто	SPRAV_LPU	ID_LPU
#            ('ID_D_TYPE',  'N', 11, 0), # Тип направления (назначения)
#            ('ID_D_GROUP', 'N', 11, 0), # Группа направления (назначения)
#            ('ID_PRVS',    'N', 11, 0), # Идентификатор специальности врача
#            ('ID_OB_TYPE', 'N', 11, 0), # Идентификатор вида обследования
#            ('ID_PRMP',    'N', 11, 0), # Идентификатор профиля медицинской помощи
#            ('ID_B_PROF',  'N', 11, 0), # Идентификатор профиля койки
#            ('ID_DN',      'N', 11, 0), # Идентификатор диспансерного наблюдения	Пусто	SPRAV_DN_ACTION	ID_DN	1.2.3.157
#            ('ID_GOAL',    'N', 11, 0), # Идентификатор цели обращения	Пусто	SPRAV_GOAL	ID_GOAL	1.2.3.157
#            ('ID_DIAGNOS', 'N', 11, 0), # Идентификатор диагноза	Пусто	SPRAV_DIAG	ID_DIAGNOS	1.2.3.157
#            ('ID_LPU_RF',  'N', 11, 0), # Идентификатор иногородней МО	Пусто	SPRAV_LPU_RF	ID_LPU_RF	1.2.3.157
#            ('ERROR',      'C',200),    # Описание ошибки
        dbfRecord.store()


    def serviceRepetionCount(self, eventId, serviceCode):
        db = QtGui.qApp.db
        tableAccountItem = db.table('Account_Item')
        tableService     = db.table('rbService')
        table = tableAccountItem
        table = table.innerJoin(tableService, tableService['id'].eq(tableAccountItem['service_id']))
        cond = [ tableAccountItem['id'].inlist(self.idList),
                 tableAccountItem['event_id'].eq(eventId),
                 tableService['infis'].eq(serviceCode),
               ]
        return db.getCount(table, '1', cond)


    def getHospitalBedProfile(self, eventId, medicalAidProfileId):
        cachedEventId, cachedHospitalBedProfileCode = self.cachedLeaveHospitalBedProfileForEventId or (None,  None)
        if eventId == cachedEventId:
            return cachedHospitalBedProfileCode

        db = QtGui.qApp.db
        hospitalBedProfileId = forceRef(db.translate('vActionLeaved', 'event_id', eventId, 'propHospitalBedProfle_id'))
        if not hospitalBedProfileId:
            tableActionMoving = db.table('vActionMoving')
            hospitalBedProfileIdList = db.getIdList(tableActionMoving,
                                                    idCol='propHospitalBedProfle_id',
                                                    where=db.joinAnd([tableActionMoving['deleted'].eq(0),
                                                                      tableActionMoving['event_id'].eq(eventId),
                                                                     ]),
                                                    order='id desc',
                                                    limit=1)
            if hospitalBedProfileIdList:
                hospitalBedProfileId = hospitalBedProfileIdList[0]
        if not hospitalBedProfileId and medicalAidProfileId:
            hospitalBedProfileId = forceRef(db.translate('rbHospitalBedProfile', 'medicalAidProfile_id', medicalAidProfileId, 'id'))
        if hospitalBedProfileId:
            hospitalBedProfileCodeAsStr = getIdentification('rbHospitalBedProfile',
                                                            hospitalBedProfileId,
                                                            'urn:tfoms78:SPRAV_BED_PROFILE',
                                                            True
                                                           )
            try:
                hospitalBedProfileCode = int(hospitalBedProfileCodeAsStr)
            except:
                hospitalBedProfileCode = 0
        else:
            hospitalBedProfileCode = 0
        self.cachedLeaveHospitalBedProfileForEventId = (eventId, hospitalBedProfileCode)
        return hospitalBedProfileCode


    def isComplete(self):
        return self.done


    def abort(self):
        self.aborted = True


    def updateBtnExport(self):
        exportNotEmpty = ( self.chkIncludeEvents.isChecked()
                           or self.chkIncludeVisits.isChecked()
                           or self.chkIncludeActions.isChecked() )
        eisLpuIdPresent = self.edtEisLpuId.text() != ''
        self.btnExport.setEnabled(exportNotEmpty and eisLpuIdPresent)


    @pyqtSignature('bool')
    def on_chkIncludeEvents_toggled(self, value):
        self.updateBtnExport()


    @pyqtSignature('bool')
    def on_chkIncludeVisits_toggled(self, value):
        self.updateBtnExport()


    @pyqtSignature('bool')
    def on_chkIncludeActions_toggled(self, value):
        self.updateBtnExport()


    @pyqtSignature('const QString&')
    def on_edtEisLpuId_textChanged(self, value):
        self.updateBtnExport()


    @pyqtSignature('')
    def on_btnExport_clicked(self):
        exportClients = self.chkExportClients.isChecked()
        includeEvents = self.chkIncludeEvents.isChecked()
        includeVisits = self.chkIncludeVisits.isChecked()
        includeActions = self.chkIncludeActions.isChecked()
        self.groupByProfile = self.chkGroupByProfile.isChecked()
        self.eisLpuId = self.edtEisLpuId.text().toInt()[0]
        if (includeEvents or includeVisits or includeActions) and self.eisLpuId:
            QtGui.qApp.preferences.appPrefs['EISOMSLpuId'] = toVariant(self.eisLpuId)
            QtGui.qApp.preferences.appPrefs['EISOMSGroupByProfile'] = toVariant(self.groupByProfile)
            QtGui.qApp.preferences.appPrefs['EISOMSExportClients'] = toVariant(exportClients)
            self.export(self.chkIgnoreConfirmation.isChecked(), exportClients, includeEvents, includeVisits, includeActions, self.groupByProfile, self.eisLpuId)


    @pyqtSignature('')
    def on_btnCancel_clicked(self):
        self.abort()


class CExportEISOMSPage2(QtGui.QWizardPage, Ui_ExportEISOMSPage2):
    def __init__(self, parent):
        QtGui.QWizardPage.__init__(self, parent)
        self.setupUi(self)
        self.setTitle(u'Экспорт данных реестра по счёту')
        self.setSubTitle(u'для выполнения шага нажмите кнопку "финиш"')

        self.pathIsValid = True
        homePath = QDir.toNativeSeparators(QDir.homePath())
        exportDir = forceString(QtGui.qApp.preferences.appPrefs.get('EISOMSExportDir', homePath))
        self.edtDir.setText(exportDir)


    def isComplete(self):
        return self.pathIsValid


    def validatePage(self):
        success = True
        destDir = forceStringEx(self.edtDir.text())
        for suffix in ('', '_V', '_add', '_D'):
            if success:
                src = self.wizard().getFullDbfFileName(suffix)
                if os.path.isfile(src):
                    dst = os.path.join(destDir, os.path.basename(src))
                    success, result = QtGui.qApp.call(self, shutil.move, (src, dst))
        if success:
            if forceBool(QtGui.qApp.preferences.appPrefs['EISOMSExportClients']):
                src = self.wizard().getFullClientsDbfFileName()
                dst = os.path.join(destDir, os.path.basename(src))
                success, result = QtGui.qApp.call(self, shutil.move, (src, dst))
        if success:
            QtGui.qApp.preferences.appPrefs['EISOMSExportDir'] = toVariant(self.edtDir.text())
            self.wizard().setAccountExposeDate()
        return success


    @pyqtSignature('QString')
    def on_edtDir_textChanged(self):
        dir = forceStringEx(self.edtDir.text())
        pathIsValid = os.path.isdir(dir)
        if self.pathIsValid != pathIsValid:
            self.pathIsValid = pathIsValid
            self.emit(SIGNAL('completeChanged()'))


    @pyqtSignature('')
    def on_btnSelectDir_clicked(self):
        dir = QtGui.QFileDialog.getExistingDirectory(self,
                u'Выберите директорий для сохранения файла выгрузки в ЕИС-ОМС',
                 forceStringEx(self.edtDir.text()),
                 QtGui.QFileDialog.ShowDirsOnly)
        if forceString(dir):
            self.edtDir.setText(QDir.toNativeSeparators(dir))
