# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

u"""Экспорт реестра  в формате XML. Республика Калмыкия. ДД"""

from PyQt4 import QtGui
from PyQt4.QtCore import Qt

from library.Utils import (forceString, forceRef, toVariant, forceInt,
                           forceDate, forceBool)
from Exchange.Order79Export import COrder79ExportWizard
from Exchange.Export import ExtendedRecord
from Exchange.ExportR08Hospital import (CR08PersonalDataWriter as
                                        PersonalDataWriter)
from Exchange.ExportR47D3 import (XmlStreamWriter,
                                  CExportPage1, CExportPage2, getEventTypeCode)

def exportR08NativeHealthExamination(widget, accountId, accountItemIdList, _):
    u"""Создает диалог экспорта реестра счета"""

    wizard = CR08ExportWizard(widget)
    wizard.setAccountId(accountId)
    wizard.setAccountItemsIdList(accountItemIdList)
    wizard.exec_()

# ******************************************************************************

def formatAccNumber(params):
    u"""две последние цифры Organisation.miacCode организации в умолчаниях)+
        (две последние цифры года из Account.settleDate)
        +(две цифры месяца из Account.settleDate)+(Account.id)"""
    settleDate = params['settleDate']
    return u'{miacCode}{settleYear:02d}{settleMonth:02d}{accId:d}'.format(
        miacCode=params['lpuMiacCode'][-2:],
        settleYear=settleDate.year() % 100,
        settleMonth=settleDate.month(),
        accId=params['accId']
    )


# ******************************************************************************

class CR08ExportWizard(COrder79ExportWizard):
    u"""Мастер экспорта для Калмыкии"""

    mapPrefixToEventTypeCode = {u'ДВ1': 'DP', u'ДВ2': 'DV', u'ОПВ': 'DO',
                                u'ДС1': 'DS', u'ДС2': 'DU', u'ОН1': 'DF',
                                u'ОН2': 'DD', u'ОН3': 'DR'}

    def __init__(self, parent=None):
        title = u'Мастер экспорта реестра услуг для Калмыкии'
        prefix = 'R08D3'
        COrder79ExportWizard.__init__(self, title, prefix, CR08ExportPage1,
                                      CExportPage2, parent)

        self.page1.setXmlWriter((R08XmlStreamWriter(self.page1),
                                 PersonalDataWriter(self.page1)))

        self.__xmlBaseFileName = None


    def prefixX(self, accountId):
        u"""Вовзращает префикс имени файла"""
        code = getEventTypeCode(self.db, accountId)
        return self.mapPrefixToEventTypeCode.get(code)


    def _getXmlBaseFileName(self, addPostfix=True):
        u"""Возвращает имя файла для записи данных"""

        if self.__xmlBaseFileName:
            result = self.__xmlBaseFileName
        else:
            tblOrganisation = self.db.table('Organisation')
            info = self.info

            payer = self.db.getRecord(tblOrganisation, 'infisCode, isInsurer',
                                      info['payerId'])

            if payer and not forceBool(payer.value(1)):
                payerPrefix = 'T'
                payerCode = '08'
            else:
                payerPrefix = 'S'
                payerCode = forceString(payer.value(0)) if payer else ''

            recipientCode = forceString(self.db.translate(tblOrganisation, 'id',
                                                          QtGui.qApp.
                                                          currentOrgId(),
                                                          'infisCode'))

            year = info['settleDate'].year() %100
            month = info['settleDate'].month()
            packetNumber = self.page1.edtPacketNumber.value()

            postfix = u'%02d%02d%d' % (year, month,
                                       packetNumber) if addPostfix else u''
            result = u'%2sM%s%s%s_%s.xml' % (self.prefixX(info['accId']),
                                             recipientCode, payerPrefix,
                                             payerCode, postfix)
            self.__xmlBaseFileName = result

        return result


    def getXmlFileName(self, addPostfix=True):
        u"""Возвращает имя файла для записи данных."""
        return self._getXmlBaseFileName(addPostfix)


    def getPersonalDataXmlFileName(self, addPostfix=True):
        u"""Возвращает имя файла для записи личных данных."""
        return u'L{0}'.format(self._getXmlBaseFileName(addPostfix)[1:])


    def getZipFileName(self):
        u"""Возвращает имя архива:"""
        return u'{0}.zip'.format(self._getXmlBaseFileName(True)[:-4])

# ******************************************************************************

class CR08ExportPage1(CExportPage1):
    u"""Первая страница мастера экспорта"""
    mapDiagResultToNazV = {'505': 1, '506': 2, '507':3}

    def __init__(self, parent, prefix):
        CExportPage1.__init__(self, parent, prefix)
        self._orgStructInfoCache = {}
        self.edtPacketNumber.setVisible(True)
        self.lblPacketNumber.setVisible(True)


    def getOrgStructureInfo(self, _id):
        u"""Возвращает parent_id и tfomsCode для подразделения"""
        result = self._orgStructInfoCache.get(_id, -1)

        if result == -1:
            result = (None, None)
            record = self._db.getRecord('OrgStructure',
                                        ['parent_id', 'infisCode'], _id)

            if record:
                result = (forceRef(record.value(0)),
                          forceString(record.value(1)))

            self._orgStructInfoCache[_id] = result

        return result


#2)Тег <ST_OKATO> в блоке <PACIENT>: нужно выгружать только в случае если
# Client.id-> ClientPolicy.PolicyKind_id -> rbPolicyKind.regionalCode=1
#
#4)Тег <LPU_1> в блоке <SLUCH> и в блоке <USL> заполняется по прежнему правилу,
# только используется код OrgStructure.infisCode:
#   Определяем по ответственному за событие врачу
#Account_Item. event_id -> Event.execPerson_id ->Person. orgStructure_id ->
# OrgStructure.infisCode
#Если значение не задано смотрим на вышестоящее подразделение
# OrgStructure. parent_id-> OrgStructure.id-> OrgStructure.infisCode
#Если и у него не заполнено – смотрим еще выше и т.д.
#Если у всех элементов в ветке дерева не будет задано значение – не заполняем
#
#5)Тег <PRVS> в блоке <USL> заполняется по старому правилу, только используется
# код rbSpeciality.regionalCode (вместо rbSpeciality. usishCode)

    def prepareStmt(self, params):
        (select, tables, where, orderBy) = CExportPage1.prepareStmt(
            self, params)
        select = u"""Account_Item.event_id AS eventId,
            Account_Item.service_id AS serviceId,
            Event.client_id AS clientId,
            Action.id AS actionId,
            Visit.id AS visitId,
            Event.execDate,

            Account_Item.event_id AS SLUCH_IDCASE,
            rbMedicalAidKind.regionalCode AS SLUCH_VIDPOM,
            PersonOrganisation.infisCode AS SLUCH_LPU,
            PersonOrgStructure.infisCode AS SLUCH_LPU_1,
            IF(rbScene.code = '4', 1, 0) AS SLUCH_VBR,
            Event.id AS SLUCH_NHISTORY,
            IF(rbDiagnosticResult.code = '502', 1, 0) AS SLUCH_P_OTK,
            Event.setDate AS SLUCH_DATE_1,
            Event.execDate AS SLUCH_DATE_2,
            Diagnosis.MKB AS SLUCH_DS1,
            Diagnosis.setDate BETWEEN Event.setDate AND Event.execDate AS SLUCH_DS1_PR,
            EventResult.regionalCode AS SLUCH_RSLT_D,
            IF(rbDispanser.code IN ('1', '2', '6'), 1, 0) AS SLUCH_PR_D_N,
            rbMedicalAidUnit.federalCode AS SLUCH_IDSP,
            TotalAmount.amount AS SLUCH_ED_COL,

            PersonOrganisation.infisCode AS USL_LPU,
            ActionOrgSruct.infisCode AS USL_LPU_1,
            IFNULL(Visit.date,
                IFNULL(Action.begDate, Action.endDate)) AS USL_DATE_IN,
            IFNULL(Visit.date, Action.endDate) AS USL_DATE_OUT,
            0 AS USL_P_OTK,
            rbService.infis AS USL_CODE_USL,
            Account_Item.price AS USL_TARIF,
            Account_Item.`sum` AS USL_SUMV_USL,
            ActionPersonSpeciality.regionalCode AS USL_PRVS,
            ActionPerson.regionalCode AS USL_CODE_MD,

            Event.client_id AS PACIENT_ID_PAC,
            rbPolicyKind.regionalCode AS PACIENT_VPOLIS,
            ClientPolicy.serial AS PACIENT_SPOLIS,
            ClientPolicy.number AS PACIENT_NPOLIS,
            IF(rbPolicyKind.regionalCode= '1', Insurer.OKATO,
                '') AS PACIENT_ST_OKATO,
            Insurer.smoCode AS PACIENT_SMO,
            IF(Insurer.smoCode IS NULL OR Insurer.smoCode = '',
                Insurer.OGRN,'') AS PACIENT_SMO_OGRN,
            IF(Insurer.smoCode IS NULL OR Insurer.smoCode = '',
                Insurer.OKATO, '') AS PACIENT_SMO_OK,
            IF((Insurer.smoCode IS NULL OR Insurer.smoCode = '')
                AND (Insurer.OGRN IS NULL OR Insurer.OGRN = ''),
                Insurer.shortName,'') AS PACIENT_SMO_NAM,

            Event.client_id AS PERS_ID_PAC,
            UPPER(Client.firstName) AS PERS_IM,
            UPPER(Client.patrName) AS PERS_OT,
            UPPER(Client.lastName) AS PERS_FAM,
            Client.sex AS PERS_W,
            Client.birthDate AS PERS_DR,
            UPPER(Client.birthPlace) AS PERS_MR,
            rbDocumentType.federalCode AS PERS_DOCTYPE,
            IF(rbDocumentType.regionalCode = '3',
                REPLACE(TRIM(ClientDocument.serial),' ', '-'),
                REPLACE(TRIM(ClientDocument.serial),'-', ' ')) AS PERS_DOCSER,
            ClientDocument.number AS PERS_DOCNUM,
            Client.SNILS AS clientSnils,

            EXISTS(
                SELECT NULL FROM ClientDocument AS CD
                LEFT JOIN rbDocumentType ON CD.documentType_id = rbDocumentType.id
                WHERE
                    CD.client_id= Client.id
                    AND rbDocumentType.code = '3'
                    AND CD.deleted = 0
                LIMIT 1
            ) as 'isAnyBirthDoc',
            PersonOrgStructure.parent_id AS personParentOrgStructureId,
            ActionOrgSruct.parent_id AS actionParentOrgStructureId"""
        tables = """Account_Item
            LEFT JOIN Event ON Event.id  = Account_Item.event_id
            LEFT JOIN Client ON Client.id = Event.client_id
            LEFT JOIN Diagnostic ON Diagnostic.id = (
                SELECT MAX(dc.id)
                    FROM Diagnostic dc
                    WHERE dc.event_id = Account_Item.event_id
                    AND dc.diagnosisType_id IN (SELECT id FROM rbDiagnosisType WHERE code = '1')
                    AND dc.deleted = 0
            )
            LEFT JOIN Diagnosis ON Diagnosis.id = Diagnostic.diagnosis_id AND Diagnosis.deleted = 0
            LEFT JOIN rbResult AS EventResult ON EventResult.id=Event.result_id
            LEFT JOIN rbDiagnosticResult ON rbDiagnosticResult.id = Diagnostic.result_id
            LEFT JOIN rbPayRefuseType ON rbPayRefuseType.id = Account_Item.refuseType_id
            LEFT JOIN rbService ON Account_Item.service_id = rbService.id
            LEFT JOIN Person ON Person.id = Event.execPerson_id
            LEFT JOIN rbSpeciality ON Person.speciality_id = rbSpeciality.id
            LEFT JOIN ClientPolicy ON ClientPolicy.id = getClientPolicyIdForDate(Client.`id`, 1, Event.execDate)
            LEFT JOIN rbPolicyKind ON rbPolicyKind.id = ClientPolicy.policyKind_id
            LEFT JOIN Organisation AS Insurer ON Insurer.id = ClientPolicy.insurer_id
            LEFT JOIN Contract_Tariff ON Account_Item.tariff_id = Contract_Tariff.id
            LEFT JOIN rbMedicalAidUnit ON rbMedicalAidUnit.id = Contract_Tariff.unit_id
            LEFT JOIN Organisation AS PersonOrganisation ON PersonOrganisation.id = Person.org_id
            LEFT JOIN OrgStructure AS PersonOrgStructure ON PersonOrgStructure.id = Person.orgStructure_id
            LEFT JOIN ClientDocument ON ClientDocument.client_id = Client.id AND
                      ClientDocument.id = (SELECT MAX(CD.id)
                                         FROM   ClientDocument AS CD
                                         LEFT JOIN rbDocumentType AS rbDT ON rbDT.id = CD.documentType_id
                                         LEFT JOIN rbDocumentTypeGroup AS rbDTG ON rbDTG.id = rbDT.group_id
                                         WHERE  rbDTG.code = '1' AND CD.client_id = Client.id AND CD.deleted=0)
            LEFT JOIN rbDocumentType ON rbDocumentType.id = ClientDocument.documentType_id
            LEFT JOIN Visit ON Account_Item.visit_id = Visit.id
            LEFT JOIN Action ON Action.id = Account_Item.action_id
            LEFT JOIN Person AS ActionPerson ON ActionPerson.id = IFNULL(Action.person_id, Visit.person_id)
            LEFT JOIN Organisation AS ActionOrg ON ActionOrg.id = ActionPerson.org_id
            LEFT JOIN OrgStructure AS ActionOrgSruct ON ActionPerson.orgStructure_id = ActionOrgSruct.id
            LEFT JOIN rbSpeciality AS ActionPersonSpeciality ON ActionPersonSpeciality.id = ActionPerson.speciality_id
            LEFT JOIN rbMedicalAidProfile AS ServiceMedicalAidProfile
                ON rbService.medicalAidProfile_id = ServiceMedicalAidProfile.id
            LEFT JOIN rbScene ON rbScene.id = Visit.scene_id
            LEFT JOIN rbDispanser ON rbDispanser.id = Diagnostic.dispanser_id
            LEFT JOIN (
                SELECT A.event_id, SUM(A.amount) AS amount
                FROM Account_Item A
                WHERE A.deleted = 0
                GROUP BY A.event_id
            ) AS TotalAmount ON TotalAmount.event_id = Account_Item.event_id
            LEFT JOIN EventType ON EventType.id = Event.eventType_id
            LEFT JOIN rbMedicalAidKind ON
                rbMedicalAidKind.id = EventType.medicalAidKind_id"""

        return (select, tables, where, orderBy)


    def getAccMkbList(self, eventId, mkb):
        stmt = u"""SELECT ds.MKB,
                ds.setDate BETWEEN Event.setDate AND Event.execDate
                FROM Diagnosis ds
                INNER JOIN Diagnostic dc ON (ds.id = dc.diagnosis_id
                    AND ds.deleted = 0
                    AND dc.deleted = 0)
                LEFT JOIN Event ON dc.event_id = Event.id
                WHERE dc.diagnosisType_id IN (
                    SELECT id
                    FROM rbDiagnosisType
                    WHERE code = '2')
                AND ds.MKB != 'Z00.0'
                AND ds.MKB != '%s'
                AND dc.event_id = %d""" % (mkb, eventId)

        query = self.db.query(stmt)
        result = []

        while query.next():
            record = query.record()
            result.append((forceString(record.value(0)),
                           forceString(record.value(1))))

        return result


    def getDiagResultList(self, eventId, serviceId):
        u'Возвращает словарь с множествами кодов NAZ*'
        stmt = u"""SELECT rbDiagnosticResult.code AS SLUCH_NAZ_V,
        IF(rbHealthGroup.code IN ('1','2'), NULL,
            rbDiagnosticResult.regionalCode) AS SLUCH_NAZR,
        IF(rbHealthGroup.code NOT IN ('1','2') AND
            rbDiagnosticResult.regionalCode IN ('1','2'),
                rbSpeciality.usishCode,'') AS SLUCH_NAZ_SP,
        IF(rbHealthGroup.code NOT IN ('1','2') AND
            rbDiagnosticResult.regionalCode IN ('4','5'),
            rbMedicalAidProfile.regionalCode, NULL) AS SLUCH_NAZ_PMP,
        IF(rbHealthGroup.code NOT IN ('1','2') AND
            rbDiagnosticResult.regionalCode = '6',
            rbMedicalAidProfile.regionalCode, NULL) AS SLUCH_NAZ_PK
        FROM Diagnostic dc
        LEFT JOIN rbDiagnosticResult ON dc.result_id = rbDiagnosticResult.id
        LEFT JOIN rbHealthGroup ON rbHealthGroup.id = dc.healthGroup_id
        LEFT JOIN rbSpeciality ON rbSpeciality.id = dc.speciality_id
        LEFT JOIN Person ON Person.id = dc.person_id
        LEFT JOIN rbService_Profile ON rbService_Profile.master_id = {serviceId}
            AND rbService_Profile.speciality_id = Person.speciality_id
        LEFT JOIN rbMedicalAidProfile ON
            rbService_Profile.medicalAidProfile_id = rbMedicalAidProfile.id
        WHERE dc.event_id = {eventId}""".format(
            eventId=eventId, serviceId=serviceId)
        query = self.db.query(stmt)
        result = {
            'SLUCH_NAZ_V': set(),
            'SLUCH_NAZR': set(),
            'SLUCH_NAZ_SP': set(),
            'SLUCH_NAZ_PMP': set(),
            'SLUCH_NAZ_PK': set()
        }

        while query.next():
            record = query.record()

            for key in result.keys():
                val = forceString(record.value(key))

                if val and key == 'SLUCH_NAZ_V':
                    val = self.mapDiagResultToNazV.get(val)

                if val:
                    result[key].add(val)

        return result

# ******************************************************************************

class R08XmlStreamWriter(XmlStreamWriter):
    u"""Осуществляет запись данных экспорта в XML"""

    eventFields1 = XmlStreamWriter.eventFields1 + ('DS1_PR', )
    ds2nFields = XmlStreamWriter.ds2nFields + ('DS2_PR', )

    def __init__(self, parent):
        XmlStreamWriter.__init__(self, parent)

    def writeHeader(self, params):
        u"""Запись заголовка xml файла"""
        settleDate = params['settleDate']
        self.writeStartElement('ZL_LIST')

        self.writeStartElement('ZGLV')
        self.writeTextElement('VERSION', '2.1')
        self.writeTextElement('DATA', settleDate.toString(Qt.ISODate))
        self.writeTextElement('FILENAME', params['shortFileName'][:-4])
        self.writeTextElement('SD_Z', self._parent.getEventCount())
        self.writeEndElement() # ZGLV

        self.writeStartElement('SCHET')
        self.writeTextElement('CODE', '%d' % params['accId'])
        self.writeTextElement('CODE_MO', params['lpuCode'])
        self.writeTextElement('YEAR', forceString(settleDate.year()))
        self.writeTextElement('MONTH', forceString(settleDate.month()))
        self.writeTextElement('NSCHET', formatAccNumber(params))
        self.writeTextElement('DSCHET',
                              params['accDate'].toString(Qt.ISODate))
                              
        if params['payerCode']:
            self.writeTextElement('PLAT', params['payerCode'])
            
        self.writeTextElement('SUMMAV', forceString(params['accSum']))
        self.writeTextElement('DISP', self.dispanserType(params))
        self.writeEndElement() # SCHET


    def writeEventInfo(self, record, params):
        u"""Пишет информацию о событии"""
        eventId = forceRef(record.value('eventId'))
        patrName = forceString(record.value('PERS_OT'))
        lpu1Code = forceString(record.value('SLUCH_LPU_1'))
        serviceLpu1Code = forceString(record.value('USL_LPU_1'))
        noPatrName = not patrName or patrName.upper() == u'НЕТ'

        if not lpu1Code:
            parentId = forceRef(record.value('personParentOrgStructureId'))
            lpu1Code = self.getLpuCode(parentId)

            if lpu1Code:
                record.setValue('SLUCH_LPU_1', toVariant(lpu1Code))


        if not serviceLpu1Code:
            parentId = forceRef(record.value('actionParentOrgStructureId'))
            serviceLpu1Code = self.getLpuCode(parentId)

            if serviceLpu1Code:
                record.setValue('USL_LPU_1', toVariant(serviceLpu1Code))

        eventSum = params['mapEventIdToSum'].get(eventId, '0')

        local_params = {
            'SLUCH_OS_SLUCH': '2' if (noPatrName or (params['isJustBorn'] and
                                                     not params['isAnyBirthDoc']
                                                    )) else '',
            'SLUCH_VERS_SPEC': 'V015',
            'SLUCH_KODLPU': params['lpuCode'],
            'SLUCH_EXTR': self.mapEventOrderToExtr.get(
                forceInt(record.value('eventOrder')), ''),
            'SLUCH_SUMV': eventSum,
            'SLUCH_TARIF': eventSum,
        }

        local_params.update(self._parent.getDiagResultList(
            eventId, forceRef(record.value('serviceId'))))
        local_params.update(params)

        if eventId != params.setdefault('lastEventId'):
            params['USL_IDSERV'] = 0

            if params['lastEventId']:
                self.exportActions(eventId, params)
                self.writeEndElement() # SLUCH

            mkb = forceString(record.value('SLUCH_DS1'))
            accMkbList = self._parent.getAccMkbList(eventId, mkb)
            record.setValue('SLUCH_DS1', toVariant(mkb[:5]))
            _record = ExtendedRecord(record, local_params)
            self.writeGroup('SLUCH', self.eventFields1, _record,
                            closeGroup=False, dateFields=self.eventDateFields)


            for mkb, isPrimary in accMkbList:
                local_params['DS2_N_DS2'] = mkb[:5]
                local_params['DS2_N_DS2_PR'] = isPrimary
                _record = ExtendedRecord(record, local_params)
                self.writeGroup('DS2_N', self.ds2nFields, _record)

            self.writeGroup('SLUCH', self.eventFields2, _record,
                            closeGroup=False, openGroup=False)
            params['lastEventId'] = eventId


    def exportActions(self, eventId, params):
        u"""Выгружает  все действия по указанному событию"""

        if eventId not in params.setdefault('exportedEvents', set()):
            params['exportedEvents'].add(eventId)
            stmt = (u"""SELECT PersonOrganisation.infisCode AS USL_LPU,
                ActionOrgSruct.infisCode AS USL_LPU_1,
                IFNULL(Action.begDate, Action.endDate) AS USL_DATE_IN,
                Action.endDate AS USL_DATE_OUT,
                0 AS USL_P_OTK,
                ActionType.code AS USL_CODE_USL,
                0 AS USL_TARIF,
                0 AS USL_SUMV_USL,
                rbSpeciality.regionalCode AS USL_PRVS,
                Person.regionalCode AS USL_CODE_MD,
                Action.id AS actionId
            FROM Action
            LEFT JOIN Event ON Action.event_id = Event.id
            LEFT JOIN Person ON Person.id = Action.person_id
            LEFT JOIN Organisation ON Organisation.id = IFNULL(Action.org_id, Person.org_id)
            LEFT JOIN rbSpeciality ON Person.speciality_id = rbSpeciality.id
            LEFT JOIN ActionType ON Action.actionType_id = ActionType.id
            LEFT JOIN OrgStructure AS ActionOrgSruct ON Person.orgStructure_id = ActionOrgSruct.id
            LEFT JOIN Organisation AS PersonOrganisation ON PersonOrganisation.id = Person.org_id
            LEFT JOIN rbService ON ActionType.nomenclativeService_id = rbService.id
            WHERE Action.event_id = {evendId}
              AND ActionType.nomenclativeService_id IS NOT NULL
              AND Action.id NOT IN ({exportedActions})
              AND Action.deleted = 0""".format(
                  evendId=eventId, exportedActions=','.join(
                      params.setdefault('exportedActions', set()))))

            query = self._parent.db.query(stmt)

            while query.next():
                params['USL_IDSERV'] += 1
                record = query.record()
                record = ExtendedRecord(record, params)
                self.writeGroup('USL', self.serviceFields, record,
                                dateFields=self.serviceDateFields)


    def writeService(self, record, params):
        params.setdefault('exportedActions', set()).add(
            forceString(record.value('actionId')))
        XmlStreamWriter.writeService(self, record, params)


    def writeFooter(self, params):
        u"""Закрывает теги в конце файла"""
        if params.get('lastEventId'):
            self.exportActions(params['lastEventId'], params)

        XmlStreamWriter.writeFooter(self, params)


    def writeRecord(self, record, params):
        u"""Пишет sql запись в файл"""
        clientId = forceRef(record.value('clientId'))

        if clientId != params.setdefault('lastClientId'):
            if params['lastClientId']:
                self.exportActions(params['lastEventId'], params)
                params['SLUCH_IDCASE'] += 1
                self.writeEndElement() # SLUCH
                self.writeEndElement() # ZAP

            self.writeStartElement('ZAP')
            self.writeTextElement('N_ZAP', forceString(clientId))
            self.writeTextElement('PR_NOV', '0')

            params['birthDate'] = forceDate(record.value('clientBirthDate'))
            execDate = forceDate(record.value('execDate'))
            params['isJustBorn'] = params['birthDate'].daysTo(execDate) < 28
            params['isAnyBirthDoc'] = forceString(record.value('isAnyBirthDoc'))

            self.writeClientInfo(record, params)

            params['rowNumber'] += 1
            params['lastClientId'] = clientId
            params['lastEventId'] = None

        self.writeEventInfo(record, params)
        self.writeService(record, params)


if __name__ == '__main__':
    #pylint: disable=wrong-import-position
    from Exchange.Utils import testAccountExport
    #pylint: enable=wrong-import-position
    testAccountExport(exportR08NativeHealthExamination, 141)
