# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import os.path
import shutil
import datetime
from zipfile import ZIP_DEFLATED, ZipFile
# таварищи, а round(10.005, 2)-> 10.01 не годится?
from decimal import Decimal, getcontext, ROUND_HALF_UP

from PyQt4 import QtGui
from PyQt4.QtCore import Qt,QDate, pyqtSignature

from library.AmountToWords import amountToWords
from library.Calendar import monthName
from library.dbfpy.dbf import Dbf
from library.Utils     import calcAgeInYears, forceBool, forceDate, forceDouble, forceInt, forceRef, forceString, forceStringEx, formatSex, nameCase, pyDate, toVariant, trim


from Events.Action import CAction
from Exchange.Export import  CExportHelperMixin, CAbstractExportPage1, CAbstractExportPage2, CAbstractExportWizard
from Reports.ReportBase import CReportBase, createTable

from Exchange.Ui_ExportR23NativePage1 import Ui_ExportR23NativePage1


def exportR23Native(widget, accountId, accountItemIdList,accountIdList=[]):
    wizard = CExportWizard(widget)
    wizard.setAccountId(accountId)
    wizard.setAccountItemsIdList(accountItemIdList)
    wizard.setSelectedAccounts(accountIdList)
    wizard.exec_()

# ******************************************************************************

class CExportWizard(CAbstractExportWizard):
    def __init__(self, parent = None):
        title = u'Мастер экспорта в ОМС Краснодарского края'
        CAbstractExportWizard.__init__(self, parent, title)
        self.page1 = CExportPage1(self)
        self.page2 = CExportPage2(self)

        if parent.tabWorkType.currentIndex() == 0: # закладка Расчет
            self.orgStructId = parent.cmbCalcOrgStructure.value()
        else:
            self.orgStructId = parent.cmbAnalysisOrgStructure.value()

        self.addPage(self.page1)
        self.addPage(self.page2)


    def getTmpDir(self):
        return CAbstractExportWizard.getTmpDirEx(self, 'R23NATIVE')


    def setSelectedAccounts(self,selectedAccountIds):
        self.page1.selectedAccountIds = selectedAccountIds


    def setAccountExposeDate(self):
        accountRecord = self.db.table('Account').newRecord(['id', 'exposeDate', 'number','note'])
        accountRecord.setValue('id', toVariant(self.accountId))
        accountRecord.setValue('exposeDate', toVariant(QDate.currentDate()))
        accountRecord.setValue('number', toVariant(self.page1.edtRegistryNumber.value()))
        accountRecord.setValue('note', toVariant(self.info.get('note', self.info.get('oldNumber'))))
        self.db.updateRecord('Account', accountRecord)


    def setAccountId(self, accountId):
        CAbstractExportWizard.setAccountId(self, accountId)
        try:
            self.page1.edtRegistryNumber.setValue(forceInt(self.info.get('accNumber')))
        except:
            pass

# *****************************************************************************************

class CExportPage1(CAbstractExportPage1, Ui_ExportR23NativePage1, CExportHelperMixin):
    mapAccountType = {0:'2', 1:'3', 2:'4', 3:'5', 4:'6', 5:'7', 6:'8', 7:'9', 8:'a', 9:'b',
        10:'c', 11:'d', 12:'e', 13:'f', 14:'g', 15:'h', 16:'i', 17:'j', 18:'k', 19:'l'}

    exportTypeP10 = 0 # формат экспорта: положение 10
    exportTypeP11 = 1 # формат экспорта: положение 11
    exportTypeFLK = 2 # формат экспорта: данные для ФЛК
    exportTypeP12 = 3 # формат экпорта: положение 12
    exportTypeP13 = 4 # формат экпорта: положение 13

    def __init__(self, parent):
        CAbstractExportPage1.__init__(self, parent)
        CExportHelperMixin.__init__(self)

        self.selectedAccountIds =[]
        self.tariffCache = {}
        self.prefix = 'ExportR23Native'

        self.cmbPayMethod.setTable('rbMedicalAidUnit')
        self.ignoreErrors = forceBool(QtGui.qApp.preferences.appPrefs.get('%sIgnoreErrors' % self.prefix, False))
        self.chkVerboseLog.setChecked(forceBool(QtGui.qApp.preferences.appPrefs.get('%sVerboseLog' % self.prefix, False)))
        self.cmbExportType.setCurrentIndex(
            forceInt(QtGui.qApp.preferences.appPrefs.get('%sExportType' % self.prefix, 0)))
        self.chkIgnoreErrors.setChecked(self.ignoreErrors)

        self.exportedTempInvalidList = []
        self.tempInvalidCache = {}
        self.exportedClients = set()
        self.modernClients = set()
        self.lastInsurerId = None
        self.exportType = self.cmbExportType.currentIndex()
        self.serviceNum = 0
        self.dbfFileName = {}


    def setExportMode(self, flag):
        self.btnCancel.setEnabled(flag)
        self.btnExport.setEnabled(not flag)
        self.chkIgnoreErrors.setEnabled(not flag)
        self.chkVerboseLog.setEnabled(not flag)
        self.btnExport.setEnabled(not flag)
        self.cmbAccountType.setEnabled(not flag)
        self.cmbPayMethod.setEnabled(not flag and (self.cmbExportType.currentIndex()  != self.exportTypeFLK ))

        self.cmbExportType.setEnabled(not flag)

        self.edtRegistryNumber.setEnabled(self.cmbExportType.currentIndex()  != self.exportTypeFLK and not flag)
        self.chkMakeInvoice.setEnabled(not flag)


    def validatePage(self):
        QtGui.qApp.preferences.appPrefs['%sIgnoreErrors' % self.prefix] = toVariant(self.chkIgnoreErrors.isChecked())
        QtGui.qApp.preferences.appPrefs['%sVerboseLog' % self.prefix] = toVariant(self.chkVerboseLog.isChecked())
        QtGui.qApp.preferences.appPrefs['%sExportType' % self.prefix] = toVariant(self.cmbExportType.currentIndex())
        return CAbstractExportPage1.validatePage(self)

# *****************************************************************************************

    def getDbfBaseName(self, isModern):
        result = self.dbfFileName.get(isModern)

        if not result:
            if self.exportType in (self.exportTypeFLK, self.exportTypeP13) and self._parent.orgStructId:
                lpuCode = forceString(self.db.translate(
                    'OrgStructure', 'id', self._parent.orgStructId, 'infisCode'))
            else:
                lpuCode = forceString(self.db.translate(
                    'Organisation', 'id', QtGui.qApp.currentOrgId() , 'infisCode'))

            postfix= 'M' if isModern else ''
            result = u'%s%s.DBF' % (lpuCode, postfix)
            self.dbfFileName[isModern] = result

        return result


    def getZipFileName(self, modern, isFLK):
        if self._parent.orgStructId:
            lpuCode = forceString(self.db.translate(
                'OrgStructure', 'id', self._parent.orgStructId, 'infisCode'))
        else:
            lpuCode = forceString(self.db.translate(
                'Organisation', 'id', QtGui.qApp.currentOrgId() , 'infisCode'))

        if isFLK:
            #  Файл ФЛК должен иметь вид ГГММККККК.zip , где ГГММ – отчетный год и месяц ,
            # ККККК – код структурного подразделения (код омс), маска файла PKKKKK – где
            # ККККК – код структурного подразделения, P – буква латинского алфавита.
            # должен быть код омс подразделения в котором была оказана услуга.

            exdate = pyDate(QDate.currentDate())
            if exdate.day < 8: exdate = exdate-datetime.timedelta(days=8)
            result = u'%s%s.ZIP' % (exdate.strftime('%y%m'),  lpuCode)
        else:
            info = self.accountInfo()
            payerId = forceRef(self.db.translate('Contract', 'id', info['contractId'], 'payer_id'))
            exposeDiscipline = forceInt(self.db.translate('Contract', 'id', info['contractId'], 'exposeDiscipline'))
            payerCode = forceString(self.db.translate('Organisation', 'id', payerId, 'infisCode')) if payerId else '000'
            postfix= 'M' if modern else ''
            result = u'%s%s%03d%s.ZIP' % (payerCode,  lpuCode,  self.edtRegistryNumber.value(), postfix)

            if (exposeDiscipline & 0x60) and self.lastInsurerId: # 5 и 6 бит - выставление по страховой
                headId = forceRef(self.db.translate('Organisation', 'id',
                                                          self.lastInsurerId, 'head_id'))
                insurerInfis = forceString(self.db.translate('Organisation', 'id',
                                                                   headId if headId else self.lastInsurerId, 'infisCode'))

                result = u'%s%s' % (insurerInfis[:4], result[4:])

        return result

# *****************************************************************************************

    def exportInt(self):
        params = {}

        self.ignoreErrors = self.chkIgnoreErrors.isChecked()
        self.exportType = self.cmbExportType.currentIndex()
        self.mkInvoice = (self.exportType in (self.exportTypeP11, self.exportTypeP12, self.exportTypeP13)) and self.chkMakeInvoice.isChecked()
        self.serviceNum = 0
        self.exportedTempInvalidList = []
        self.exportedClients = set()

        self.setExportMode(True)
        self.progressBar.reset()
        self.progressBar.setMaximum(1)
        self.progressBar.setValue(0)
        self.progressBar.setText(u'Запрос в БД...')
        self.log(u'Запрос в БД...')
        QtGui.qApp.processEvents()
        self.clientInfo = QtGui.qApp.callWithWaitCursor(self, self.getClientInfo)

        if self._parent.orgStructId:
            self.log(u'В фильтре выбрано подразделение, испольуем его код',  True)
            params['codeLpu'] = forceString(self.db.translate(
                'OrgStructure', 'id', self._parent.orgStructId, 'bookkeeperCode'))
            self.log(u'Подразделение код: "%s".' % params['codeLpu'], True)
        else:
            lpuId = QtGui.qApp.currentOrgId()
            params['codeLpu'] = forceString(self.db.translate(
                'Organisation', 'id', lpuId , 'infisCode'))
            self.log(u'ЛПУ: код инфис: "%s".' % params['codeLpu'], True)

        if not params['codeLpu']:
            self.log(u'<b><font color=red>ОШИБКА<\font><\b>:'
                     u'Для текущего ЛПУ/подразделения не задан код инфис/для бухгалтерии', True)
            if not self.ignoreErrors:
                self.aborted = True
                return

        if self.exportType in (self.exportTypeP11, self.exportTypeP12):
            summKMultiplierId = forceRef(self.db.translate('rbContractCoefficientType', 'code', '230058', 'id'))

            if not summKMultiplierId:
                self.log(u'<b><font color=red>ОШИБКА:</b>'
                 u' Не найден тип тарифного коэффициента'
                 u' (rbContractCoefficientType) с кодом "230058" (Коэффициент справочника SPR58)', True)
                if not self.ignoreErrors:
                    self.aborted = True
                    return

            summUcMultiplierId = forceRef(self.db.translate('rbContractCoefficientType', 'code', '230066', 'id'))

            if not summUcMultiplierId:
                self.log(u'<b><font color=red>ОШИБКА:</b>'
                 u' Не найден тип тарифного коэффициента'
                 u' (rbContractCoefficientType) с кодом "230066" (Коэффициент справочника SPR66)', True)
                if not self.ignoreErrors:
                    self.aborted = True
                    return

        params.update(self.accountInfo())
        params['expenseCache'] = self.createExpenseCache(params['contractId'])
        params['iAccNumber'] = 0

        try:
            params['iAccNumber'] = forceInt(params['accNumber'])
        except:
            self.log(u'Невозможно привести номер счёта' \
                       u' "%s" к числовому виду' % params['accNumber'])

        params['payMethod'] = forceString(self.db.translate('rbMedicalAidUnit', 'id',  self.cmbPayMethod.value(), 'regionalCode'))
        params['accType'] = self.mapAccountType.get(self.cmbAccountType.currentIndex())
        params['summKMultiplier'] = 0
        params['summUCMultiplier'] = 0

        params['payerId'] = forceRef(self.db.translate(
            'Contract', 'id', params['contractId'], 'payer_id'))
        params['payerOGRN'] = forceString(self.db.translate(
            'Organisation', 'id', params['payerId'], 'OGRN')) if params['payerId'] else ''
        self.log(u'ОГРН плательщика `%s`.' % params['payerOGRN'])

        if self.idList:
            if self.exportType in (self.exportTypeP11, self.exportTypeP12):
                params['summKMultiplier'] = self.getTariffCoefficient(summKMultiplierId, params['contractId'], params['exposeDate'])
                params['summUCMultiplier'] = self.getTariffCoefficient(summUcMultiplierId, params['contractId'], params['exposeDate'])

        self.setProcessFuncList([self.process, self.processPerson])
        self.setProcessParams(params)
        CAbstractExportPage1.exportInt(self)

        # Формируем фактуру в zip архив если стоит галочка
        if self.mkInvoice:
            self.log(u'Выгрузка счет-фактуры...')
            isOutArea = forceInt(self.db.translate(
                'Contract', 'id', params['contractId'], 'exposeDiscipline'))
            isOutArea = ((isOutArea & 0b1100000) >> 5) == 0
            self.makeInvoice(params['accNumber'], params['accDate'], params['settleDate'],
                             params['payerId'], isOutArea, params['accType'])

# *****************************************************************************************
    def getInvoiceData(self):

        #Определяем, откуда брать данные фактуры, из модернизации или нет (что не пусто оттуда и берем)
        if self.exportType not in (self.exportTypeP12, self.exportTypeP13):
            dbfName = os.path.join(self.getTmpDir(), 'P'+self.getDbfBaseName(False))
            dbfNamem = os.path.join(self.getTmpDir(), 'P'+self.getDbfBaseName(True))
            dbf_p  = Dbf(dbfName, True, encoding='cp866')
            dbf_pm = Dbf(dbfNamem, True, encoding='cp866')
            modern = dbf_pm.recordCount>0
            dbf_p.close()
            dbf_pm.close()
        else:
            modern = False
        #-------------------------
        dbfNamep = os.path.join(self.getTmpDir(), 'P'+self.getDbfBaseName(modern))
        dbfNameu = os.path.join(self.getTmpDir(), 'U'+self.getDbfBaseName(modern))
        dbf_p  = Dbf(dbfNamep, True, encoding='cp866')
        dbf_u  = Dbf(dbfNameu, True, encoding='cp866')
        #Берем из базы типы оказания медпомощи
        db = QtGui.qApp.db
        vps = db.getRecordListGroupBy('rbMedicalAidType','name,regionalCode',group='regionalCode')
        report =[]

        self.progressBar.reset()
        self.progressBar.setMaximum(max(len(vps), 1))
        self.progressBar.setValue(0)
        #Для оптимизации скорости, загружаем данные о пациентах в словарь
        pdict=dict()
        #Для определения плательщика
        pogrn=dict()
         #Для определения обращений
        pobr=dict()
        obr_srt = u"Обращение"
        stmt = """SELECT rbService.infis
            FROM rbService
            WHERE substr(rbService.name,1,9)  = '%s';""" % obr_srt
        query = self.db.query(stmt)
        while query.next():
            record=query.record()
            pobr[forceString(record.value('infis'))]=1

        #Грузим признаки работающих и неработающих в словарь, чтоб быстрее работало
        for rec_p in dbf_p:
            pdict[rec_p['SN']]=[rec_p['KAT'],rec_p['IMA'],rec_p['OTCH'],rec_p['DATR'],rec_p['POL']]
            pogrn[rec_p['PL_OGRN']]=1
        for vp in vps:
            fl_w = set() #Множество работающих
            fl_f = set() #Множество неработающих
            sn_w = set();
            sn_f = set()
            somp_w=0; somp_f=0; #Количество СОМП для работающих и неработающих
            kd_w=0;kd_f=0; #Кол-во койко-дней
            kolu_w=0;kolu_f=0; #Количество услуг
            summ_w=0.0; summ_f=0.0;summ_t=0.0;summ_b=0.0;summ_d=0.0;summ_dm=0.0;summ_k=0.0;summ_uc=0.0; #Суммы по всем разделам тарифа
            cp_w=0;cp_f=0;cp_t=0; #Кол-во посещений
            pd_w=0;pd_f=0;pd_t=0; #Кол-во пациенто-дней
            uet_w=0;uet_f=0;uet_t=0;#Кол-во УЕТ
            obr_w=0; obr_f=0; #Кол-ко обращений
            self.progressBar.step()
            #Проходим по файлу услуг, разбиваем данные по виду помощи и помещаем в список ввиде словарей
            for rec_u in dbf_u:
                if rec_u['VP'] == vp.value('regionalCode'):
                    if pdict.has_key(rec_u['SN']):
                            if pdict[rec_u['SN']][0]== '1': #Для работающих
                                fl_w.add(tuple(pdict[rec_u['SN']])) #Кол-во физлиц
                                sn_w.add(rec_u['SN']) #Кол-во перссчетов
                                if rec_u['KUSL'][:1] in ('A',  'B'):
                                    if pobr.has_key(rec_u['KUSL']):
                                        obr_w+=1
                                    elif self.isPos(rec_u['KUSL'], rec_u['VP']) == 1:
                                        cp_w += 1
                                    else:
                                        kolu_w+=rec_u['KOLU'] #Количество услуг
                                if (rec_u['KUSL'][:1]==u'S') and (rec_u['KUSL']==rec_u['KSTAND']) or rec_u['KUSL'][:1]==u'G': somp_w+=1 #Количество сомп
                                if rec_u['KUSL'][:1] in ('K',  'S',  'G'): #Количество койко-дней / пациенто-дней (для ДС)
                                    if rec_u['VP'] in ('11','12'):
                                        kd_w+=rec_u['KD']
                                    elif rec_u['VP'] in ('41','42','43','51','52','71','72','90'):
                                        pd_w+=rec_u['KD']
                                summ_w+=rec_u['SUMM'] #Сумма
                                uet_w+=rec_u['UET']
                            else: #Для неработающих
                                fl_f.add(tuple(pdict[rec_u['SN']])) #Кол-вл физлиц
                                sn_f.add(rec_u['SN']) #Кол-во перссчетов
                                if rec_u['KUSL'][:1] in ('A',  'B'):
                                    if pobr.has_key(rec_u['KUSL']):
                                        obr_f+=1
                                    elif self.isPos(rec_u['KUSL'], rec_u['VP']) == 1:
                                        cp_f += 1
                                    else:
                                        kolu_f+=rec_u['KOLU'] #Количество услуг
                                if (rec_u['KUSL'][:1]==u'S') and (rec_u['KUSL']==rec_u['KSTAND']) or rec_u['KUSL'][:1]==u'G': somp_f+=1
                                if rec_u['KUSL'][:1] in ('K',  'S',  'G'):
                                    if rec_u['VP'] in ('11','12'):
                                        kd_f+=rec_u['KD']
                                    elif rec_u['VP'] in ('41','42','43','51','52','71','72','90'):
                                        pd_f+=rec_u['KD']
                                uet_f+=rec_u['UET']
                                summ_f+=rec_u['SUMM'] #Сумма
                            summ_b+=rec_u['SUMM_B'] #Сумма по базовому тарифу
                            summ_d+=rec_u['SUMM_D'] #Сумма по доп расходам
                            summ_dm+=rec_u['SUMM_DM'] #Доплата по СОМП/АПМП
                            summ_k+=rec_u['SUMM_K'] #Сумма по комуналке
                            summ_uc+=rec_u['SUMM_UC'] #Сумма по стимул. выплатам
                            uet_t+=rec_u['UET']

            if len(fl_w) or len(fl_f): report.append({'name'   : forceString(vp.value('name')),
                                                      'wcnt'   : len(fl_w),
                                                      'fcnt'   : len(fl_f),
                                                      'tcnt'   : len(fl_w)+len(fl_f),
                                                      'wsn'    : len(sn_w),
                                                      'fsn'    : len(sn_f),
                                                      'tsn'    : len(sn_w)+len(sn_f),
                                                      'wsomp'  : somp_w,
                                                      'fsomp'  : somp_f,
                                                      'tsomp'  : somp_w+somp_f,
                                                      'wkd'    : kd_w,
                                                      'fkd'    : kd_f,
                                                      'tkd'    : kd_w+kd_f,
                                                      'wkolu'  : kolu_w,
                                                      'fkolu'  : kolu_f,
                                                      'tkolu'  : kolu_w+kolu_f,
                                                      'wsumm'  : summ_w,
                                                      'fsumm'  : summ_f,
                                                      'tsumm'  : summ_w+summ_f,
                                                      'summ_b' : summ_b,
                                                      'summ_d' : summ_d,
                                                      'summ_dm': summ_dm,
                                                      'summ_k' : summ_k,
                                                      'summ_uc': summ_uc,
                                                      'wcp':cp_w,
                                                      'fcp':cp_f,
                                                      'tcp':cp_w+cp_f,
                                                      'wpd':pd_w,
                                                      'fpd':pd_f,
                                                      'tpd':pd_w+pd_f,
                                                      'vp' : vp.value('regionalCode'),
                                                      'wuet' : uet_w,
                                                      'fuet' : uet_f,
                                                      'tuet' : uet_t,
                                                      'wobr' : obr_w,
                                                      'fobr' : obr_f,
                                                      'tobr' : obr_w+obr_f
            })
        sn_set = set()
        summ = 0.0
        for rec_u in dbf_u:
            sn_set.add(rec_u['SN'])
            summ += rec_u['SUMM']
        dbf_u.close()
        dbf_p.close()
        #Формируем итого по всем видам помощи
        somp_w=0; somp_f=0;somp_t=0;kd_w=0;kd_f=0;kd_t=0;kolu_w=0;kolu_f=0;kolu_t=0;summ_w=0.0; summ_f=0.0;summ_t=0.0;summ_b=0.0;
        summ_d=0.0;summ_dm=0.0;summ_k=0.0;summ_uc=0.0; wcnt=0;fcnt=0;tcnt=0;wsn=0;fsn=0;tsn=0;
        uet_w=0;uet_f=0;uet_t=0 #Посещения
        cp_w=0;cp_f=0;cp_t=0 #Посещения
        pd_w=0;pd_f=0;pd_t=0 #Пациенто-дни
        obr_t=0;
        for row in report:
            wcnt+=row['wcnt']
            fcnt+=row['fcnt']
            tcnt+=row['tcnt']
            wsn+=row['wsn']
            fsn+=row['fsn']
            tsn+=row['tsn']
            somp_w+=row['wsomp']
            somp_f+=row['fsomp']
            somp_t+=row['tsomp']
            kd_w+=row['wkd']
            kd_f+=row['fkd']
            kd_t+=row['tkd']
            kolu_w+=row['wkolu']
            kolu_f+=row['fkolu']
            kolu_t+=row['tkolu']
            summ_w+=row['wsumm']
            summ_f+=row['fsumm']
            summ_t+=row['tsumm']
            summ_b+=row['summ_b']
            summ_d+=row['summ_d']
            summ_dm+=row['summ_dm']
            summ_k+=row['summ_k']
            summ_uc+=row['summ_uc']
            cp_w+=row['wcp']
            cp_f+=row['fcp']
            cp_t+=row['tcp']
            pd_w+=row['wpd']
            pd_f+=row['fpd']
            pd_t+=row['tpd']
            uet_w+=row['wuet']
            uet_f+=row['fuet']
            uet_t+=row['tuet']
            obr_t+=row['tobr'];
        report.append({'name'   : u'ИТОГО',
                       'wcnt'   : wcnt,
                       'fcnt'   : fcnt,
                       'tcnt'   : tcnt,
                       'wsn'    : wsn,
                       'fsn'    : fsn,
                       'tsn'    : tsn,
                       'wsomp'  : somp_w,
                       'fsomp'  : somp_f,
                       'tsomp'  : somp_w+somp_f,
                       'wkd'    : kd_w,
                       'fkd'    : kd_f,
                       'tkd'    : kd_w+kd_f,
                       'wkolu'  : kolu_w,
                       'fkolu'  : kolu_f,
                       'tkolu'  : kolu_w+kolu_f,
                       'wsumm'  : summ_w,
                       'fsumm'  : summ_f,
                       'tsumm'  : summ_w+summ_f,
                       'summ_b' : summ_b,
                       'summ_d' : summ_d,
                       'summ_dm': summ_dm,
                       'summ_k' : summ_k,
                       'summ_uc': summ_uc,
                       'wcp'    :cp_w,
                       'fcp'    :cp_f,
                       'tcp'    :cp_t,
                       'wpd'    :pd_w,
                       'fpd'    :pd_f,
                       'tpd'    :pd_t,
                       'wuet' : uet_w,
                       'fuet' : uet_f,
                       'tuet' : uet_t,
                       'tobr' : obr_t,
                       'vp'     :'-1'
        })

        return len(sn_set),summ, report,pogrn


    def makeInvoice(self,accNumber,accDate, periodDate, payerId,isOutArea,accType):
        mapAccountName = {'2':u'основной по ОМС','3':u'возвратный по ОМС','4':u'дополнительный по ОМС',
                        '5':u'повторный по ОМС','6':u'основной с модернизацией',
                        '7':u'возвратный с модернизацией','8':u'дополнительные с модернизацией','9':u'повторный с модернизацией',
                        'a':u'основной по доп.дисп.(осмотрам) взр. населения',
                        'b':u'возвратный по доп.дисп.(осмотрам) взр. населения',
                        'c':u'дополнительный по доп.дисп.(осмотрам) взр. населен',
                        'd':u'повторный по доп.дисп.(осмотрам) взр. населения',
                        'e':u'основной по доп.дисп.(осмотрам) несов., сирот',
                        'f':u'возвратный по доп.дисп.(осмотрам) несов., сирот',
                        'g':u'дополнительный по доп.дисп.(осмотрам) несов, сирот',
                        'h':u'повторный по доп.дисп.(осмотрам) несов., сирот',
                        'i':u'основной по ВМП (выс.техн.МП)',
                        'j':u'возвратный по ВМП (выс.техн.МП)',
                        'k':u'дополнительный по ВМП (выс.техн.МП)',
                        'l':u'повторный по ВМП (выс.техн.МП)'}

        accName =  mapAccountName.get(accType)
        casecount,summ,reps,pogrn =  self.getInvoiceData()
        invoice  = QtGui.QTextDocument()
        #Рисуем фактуру
        fnt = invoice.defaultFont()
        fnt.setPointSize(9)
        invoice.setDefaultFont(fnt)
        invoice_cursor = QtGui.QTextCursor(invoice)
        fmtdiv = QtGui.QTextBlockFormat()
        invoice_cursor.insertBlock(fmtdiv)
        invoice_cursor.insertHtml("@@@@=@@@@")

        fmt =QtGui.QTextBlockFormat()

        fmt.setAlignment(Qt.AlignCenter)
        invoice_cursor.insertBlock(fmt)
        invoice_num  = forceString(accNumber)
        invoice_date = forceString(accDate)
        invoice_cursor.insertHtml(u"<b>СЧЕТ-ФАКТУРА %s от %s</b><br>" % ('_'*3,invoice_date.rjust(10,'_')))
        invoice_cursor.insertHtml(u"<b>ИСПРАВЛЕНИЕ  %s от %s</b><br>" % ('_'*3,'_'*10))
        invoice_cursor.insertHtml(u"к реестру счетов № %s от %s за %s %s %s<br>" % (invoice_num,invoice_date,str(periodDate.year()), monthName[periodDate.month()], accName))
        fmt.setAlignment(Qt.AlignLeft)
        invoice_cursor.insertBlock(fmt)
        table = createTable (invoice_cursor, [ ('30%', [], CReportBase.AlignLeft), ('70%', [], CReportBase.AlignLeft) ],
                             headerRowCount=10, border=0, cellPadding=2, cellSpacing=0)
        fmt = QtGui.QTextCharFormat()
        fmt.setFontUnderline(True)
        db = QtGui.qApp.db
        #Формируем запросы
        tableOrganisation = db.table('Organisation')
        tableAccount = db.table('Organisation_Account')
        tableBank = db.table('Bank')
        tablef = tableOrganisation.leftJoin(tableAccount,[tableAccount['organisation_id'].eq(tableOrganisation['id'])])
        tablef = tablef.leftJoin(tableBank,[tableAccount['bank_id'].eq(tableBank['id'])])
        #Определяем плательщика - тфомс или конкретная СМК
        if len(pogrn)==1 and not isOutArea:
            ogrn = pogrn.keys()[0]
            record = db.getRecordEx(tableOrganisation,[tableOrganisation['id'].name()],
                                [tableOrganisation['OGRN'].eq(ogrn),
                                 tableOrganisation['isInsurer'].eq(1),
                                 tableOrganisation['deleted'].eq(0),
                                 tableOrganisation['head_id'].isNull()])
            if record:
                payerId = forceInt(record.value('id'))

        #Выгружаем данные поставщика и плательщика
        record = db.getRecordEx(tablef,
                                [tableOrganisation['fullName'].name(),
                                 tableOrganisation['INN'].name(),
                                 tableOrganisation['KPP'].name(),
                                 tableOrganisation['Address'].name(),
                                 tableOrganisation['chief'].name(),
                                 tableOrganisation['accountant'].name(),
                                 tableBank['name'].alias('BankName'),
                                 tableBank['BIK'].name(),
                                 tableAccount['name'].alias('schet'),
                                 tableAccount['personalAccount']
                                ],
                                tableOrganisation['id'].eq(QtGui.qApp.currentOrgId())
        )
        record_pay = db.getRecordEx(tablef,
                                    [tableOrganisation['fullName'].name(),
                                     tableOrganisation['INN'].name(),
                                     tableOrganisation['KPP'].name(),
                                     tableOrganisation['Address'].name(),
                                     tableBank['name'].alias('BankName'),
                                     tableBank['BIK'].name(),
                                     tableAccount['name'].alias('schet')
                                    ],
                                    tableOrganisation['id'].eq(payerId)
        )
        table.setText(0, 0, u'Продавец:')
        table.setText(0, 1, forceString(record.value('fullName')),fmt)
        table.setText(1, 0, u'Адрес:')
        table.setText(1, 1, forceString(record.value('Address')),fmt)
        table.setText(2, 0, u'ИНН/КПП продавца:')
        table.setText(2, 1, forceString(record.value('INN'))+'/'+forceString(record.value('KPP')),fmt)
        table.setText(3, 0, u'Грузоотправитель и его адрес:')
        table.setText(3, 1, forceString(record.value('Address')),fmt)
        table.setText(4, 0, u'Грузополучатель и его адрес:')
        table.setText(4, 1, forceString(record_pay.value('Address')),fmt)
        table.setText(5, 0, u'К платежному-расчетному документу №:')
        table.setText(5, 1, '',fmt)
        table.setText(6, 0, u'Покупатель:')
        table.setText(6, 1, forceString(record_pay.value('fullName')),fmt)
        table.setText(7, 0, u'Адрес:')
        table.setText(7, 1, forceString(record_pay.value('Address')),fmt)
        table.setText(8, 0, u'ИНН/КПП покупателя:')
        table.setText(8, 1, forceString(record_pay.value('INN'))+'/'+forceString(record_pay.value('KPP')),fmt)
        table.setText(9, 0, u'Валюта: наименование, код:')
        table.setText(9, 1, u'Российский рубль, 383',fmt)
        tableColumns = [
            ('48%',  [ u'Наименование товара (описание выполненных работ, оказанных услуг), имущественного права' ],                 CReportBase.AlignLeft ),
            ('5%',  [ u'Единица измерения' ], CReportBase.AlignLeft ),
            ('5%',  [ u'' ], CReportBase.AlignCenter ),
            ('5%',  [ u'Количество (объем)'], CReportBase.AlignRight ),
            ('5%',  [ u'Цена (тариф) за единицу измерения'], CReportBase.AlignCenter ),
            ('5%',  [ u'Стоимость товаров (работ, услуг), имущественных прав без налога - всего'], CReportBase.AlignCenter ),
            ('5%',  [ u'В том числе сумма акциза'], CReportBase.AlignCenter ),
            ('5%',  [ u'Налоговая ставка'], CReportBase.AlignCenter ),
            ('5%',  [ u'Сумма налога, предъявляемая покупателю'], CReportBase.AlignCenter ),
            ('5%',  [ u'Стоимость товаров (работ, услуг), имущественных прав с налогом - всего'], CReportBase.AlignCenter ),
            ]
        invoice_cursor.movePosition(QtGui.QTextCursor.End)
        invoice_cursor.insertBlock()
        table = createTable (invoice_cursor,tableColumns, headerRowCount=5, border=1, cellPadding=1, cellSpacing=0)
        table.mergeCells(0,1,1,2)
        table.mergeCells(0,0,2,1)
        for i in range(2,10): table.mergeCells(0,i,2,1)
        fmt =QtGui.QTextBlockFormat()
        fmt.setAlignment(Qt.AlignCenter)
        chr =QtGui.QTextCharFormat()
        chr.setFontWeight(QtGui.QFont.Bold)
        table.cellAt(1,1).setFormat(chr)
        table.cellAt(1,2).setFormat(chr)
        table.setText(1,1,u'код',blockFormat=fmt)
        table.setText(1,2,u'условное обозна чение (нацио нальное)',blockFormat=fmt)
        table.setText(2,0,u'1',blockFormat=fmt)
        table.setText(2,1,u'2',blockFormat=fmt)
        table.setText(2,2,u'2a',blockFormat=fmt)
        table.setText(2,3,u'3',blockFormat=fmt)
        table.setText(2,4,u'4',blockFormat=fmt)
        table.setText(2,5,u'5',blockFormat=fmt)
        table.setText(2,6,u'6',blockFormat=fmt)
        table.setText(2,7,u'7',blockFormat=fmt)
        table.setText(2,8,u'8',blockFormat=fmt)
        table.setText(2,9,u'9',blockFormat=fmt)
        table.setHtml(3,0,u'Сводный счет за пролеченных больных:<br>- случаев лечения')
        table.setText(3,1,u'-',blockFormat=fmt)
        table.setText(3,2,u'-',blockFormat=fmt)
        table.setText(3,3,str(casecount),blockFormat=fmt)
        table.setText(3,4,u'-',blockFormat=fmt)
        table.setText(3,5,u'%.2f'%summ,blockFormat=fmt)
        table.setText(3,9,u'%.2f'%summ,blockFormat=fmt)
        table.setText(3,6,u'без акциза',blockFormat=fmt)
        table.setText(3,7,u'без НДС',blockFormat=fmt)
        table.setText(3,8,u'без НДС',blockFormat=fmt)
        table.mergeCells(4,0,4,5)
        table.setHtml(4,0,u'<b><i>Всего к оплате:</i></b>')
        table.cellAt(4,5).setFormat(chr)
        table.setText(4,5,u'%.2f'%summ,blockFormat=fmt)
        table.cellAt(4,9).setFormat(chr)
        table.setText(4,9,u'%.2f'%summ,blockFormat=fmt)
        invoice_cursor.movePosition(QtGui.QTextCursor.End)
        invoice_cursor.insertHtml(u"<br>"+u"&nbsp;"*50+u"Главный врач __________ %30s."%forceString(record.value('chief')) +
                                          u"&nbsp;"*50+u"Гл. бухгалтер __________ %30s."%forceString(record.value('accountant')))
        #Отчет о пролеченных пациентах
        invoice_cursor.insertBlock(fmtdiv)
        invoice_cursor.insertHtml("$$$$=$$$$@@@@=@@@@")

        fmt =QtGui.QTextBlockFormat()
        fmt.setAlignment(Qt.AlignCenter)
        invoice_cursor.insertBlock(fmt)
        invoice_cursor.insertHtml(u"<br><br><br><b>Отчет о пролеченных больных, застрахованных на территории Краснодарского края</b><br>")
        invoice_cursor.insertHtml(u"<br><br>к реестру счетов № %s от %s за %s %s %s<br>" % (invoice_num,invoice_date,str(periodDate.year()), monthName[periodDate.month()], accName))
        table = createTable (invoice_cursor, [ ('30%', [], CReportBase.AlignLeft), ('70%', [], CReportBase.AlignLeft) ], headerRowCount=2, border=0, cellPadding=2, cellSpacing=0)
        table.setText(0, 0, u'Медицинская организация:')
        table.setText(0, 1, forceString(record.value('fullName')))
        table.setText(1, 0, u'Плательщик:')
        table.setText(1, 1, forceString(record_pay.value('fullName')))
        tableColumns = [
            ('38%',  [ u'Вид медицинской помощи' ],CReportBase.AlignCenter ),
            ('5%',  [ u'Количество физ. лиц' ], CReportBase.AlignCenter ),
            ('5%',  [ u'Количество персональных счетов' ], CReportBase.AlignCenter ),
            ('5%',  [ u'Количество СОМП'], CReportBase.AlignCenter ),
            ('5%',  [ u'Количество койко-дней'], CReportBase.AlignCenter ),
            ('5%',  [ u'Количество посещений'], CReportBase.AlignCenter ),
            ('5%',  [ u'Количество УЕТ'], CReportBase.AlignCenter ),
            ('5%',  [ u'Количество пациенто-дней'], CReportBase.AlignCenter ),
            ('5%',  [ u'Количество простых, слож-ных и ком-плексных услуг'], CReportBase.AlignCenter ),
            ('10%',  [ u'Сумма, руб.'], CReportBase.AlignCenter ),
            ]
        invoice_cursor.movePosition(QtGui.QTextCursor.End)
        invoice_cursor.insertBlock()
        table = createTable (invoice_cursor,tableColumns, headerRowCount=len(reps)*3+1, border=1, cellPadding=1, cellSpacing=0)
        fmtc =QtGui.QTextBlockFormat()
        fmtc.setAlignment(Qt.AlignCenter)
        fmtl=QtGui.QTextBlockFormat()
        fmtl.setAlignment(Qt.AlignLeft)
        rownum=1
        for i in reversed(range(len(reps))):
            table.setText(rownum,0,reps[i]['name'],blockFormat=fmtl)
            table.setText(rownum,1,reps[i]['tcnt'],blockFormat=fmtc)
            table.setText(rownum,2,reps[i]['tsn'],blockFormat=fmtc)
            table.setText(rownum,3,reps[i]['tsomp'],blockFormat=fmtc)
            table.setText(rownum,4,reps[i]['tkd'],blockFormat=fmtc)
            table.setText(rownum,5,reps[i]['tcp'],blockFormat=fmtc)
            table.setText(rownum,6,reps[i]['tuet'],blockFormat=fmtc)
            table.setText(rownum,7,reps[i]['tpd'],blockFormat=fmtc)
            table.setText(rownum,8,reps[i]['tkolu'],blockFormat=fmtc)
            table.setText(rownum,9,'%.2f'%reps[i]['tsumm'],blockFormat=fmtc);rownum+=1
            table.setText(rownum,0,u'---По работающим',blockFormat=fmtl)
            table.setText(rownum,1,reps[i]['wcnt'],blockFormat=fmtc)
            table.setText(rownum,2,reps[i]['wsn'],blockFormat=fmtc)
            table.setText(rownum,3,reps[i]['wsomp'],blockFormat=fmtc)
            table.setText(rownum,4,reps[i]['wkd'],blockFormat=fmtc)
            table.setText(rownum,5,reps[i]['wcp'],blockFormat=fmtc)
            table.setText(rownum,6,reps[i]['wuet'],blockFormat=fmtc)
            table.setText(rownum,7,reps[i]['wpd'],blockFormat=fmtc)
            table.setText(rownum,8,reps[i]['wkolu'],blockFormat=fmtc)
            table.setText(rownum,9,'%.2f'%reps[i]['wsumm'],blockFormat=fmtc);rownum+=1
            table.setText(rownum,0,u'---По неработающим',blockFormat=fmtl)
            table.setText(rownum,1,reps[i]['fcnt'],blockFormat=fmtc)
            table.setText(rownum,2,reps[i]['fsn'],blockFormat=fmtc)
            table.setText(rownum,3,reps[i]['fsomp'],blockFormat=fmtc)
            table.setText(rownum,4,reps[i]['fkd'],blockFormat=fmtc)
            table.setText(rownum,5,reps[i]['fcp'],blockFormat=fmtc)
            table.setText(rownum,6,reps[i]['fuet'],blockFormat=fmtc)
            table.setText(rownum,7,reps[i]['fpd'],blockFormat=fmtc)
            table.setText(rownum,8,reps[i]['fkolu'],blockFormat=fmtc)
            table.setText(rownum,9,'%.2f'%reps[i]['fsumm'],blockFormat=fmtc);rownum+=1

        invoice_cursor.movePosition(QtGui.QTextCursor.End)
        invoice_cursor.insertHtml(u"<br> От плательщика:<br><br>"+u'_'*30+u'&nbsp;'*4+u'_'*30+u'&nbsp;'*4+u'_'*30+u'&nbsp;'*4+u"'___'________201_г<br>")
        invoice_cursor.insertHtml(u'&nbsp;'*15+u'должность'+u'&nbsp;'*55+u'подпись'+u'&nbsp;'*55+u'(Ф.И.О.)<br>')
        invoice_cursor.insertHtml(u"<br> Копию акта получил:<br><br>"+u'_'*30+u'&nbsp;'*4+u'_'*30+u'&nbsp;'*4+u'_'*30+u'&nbsp;'*4+u"'___'________201_г<br>")
        invoice_cursor.insertHtml(u'&nbsp;'*15+u'должность'+u'&nbsp;'*55+u'подпись'+u'&nbsp;'*55+u'(Ф.И.О.)<br><br><br>')
        invoice_cursor.movePosition(QtGui.QTextCursor.End)
        #Счет
        invoice_cursor.insertBlock(fmtdiv)
        invoice_cursor.insertHtml("$$$$=$$$$####=####")

        invoice_cursor.insertBlock()
        table = createTable (invoice_cursor, [ ('15%', [], CReportBase.AlignLeft),
                                               ('30%', [], CReportBase.AlignLeft),
                                               ('15%', [], CReportBase.AlignLeft),
                                               ('30%', [], CReportBase.AlignLeft)
        ], headerRowCount=3, border=0, cellPadding=0, cellSpacing=0)
        chr.setFontWeight(QtGui.QFont.Bold)
        table.cellAt(0,0).setFormat(chr)
        table.cellAt(0,2).setFormat(chr)
        chr.setFontUnderline(True)
        chr.setFontWeight(QtGui.QFont.Normal)
        table.cellAt(0,1).setFormat(chr)
        table.cellAt(0,3).setFormat(chr)
        table.setText(0, 0, u'Поставщик:')
        table.setText(0, 1, forceString(record.value('fullName')))
        table.setText(0, 2, u'Плательщик:')
        table.setText(0, 3, forceString(record_pay.value('fullName')))
        chr.setFontWeight(QtGui.QFont.Bold)
        table.cellAt(1,0).setFormat(chr)
        table.cellAt(1,2).setFormat(chr)
        chr.setFontUnderline(True)
        chr.setFontWeight(QtGui.QFont.Normal)
        table.cellAt(1,1).setFormat(chr)
        table.cellAt(1,3).setFormat(chr)
        table.setText(1, 0, u'Расчетный:')
        pacc_str = u""
        pacc_val = forceString(record.value('personalAccount'))
        if pacc_val:
            pacc_str = u", л/с %s" % pacc_val
        table.setText(1, 1, u'ИНН %s, КПП %s, %s, БИК: %s, р/с %s%s' % (forceString(record.value('INN')),
                                                              forceString(record.value('KPP')),
                                                              forceString(record.value('BankName')),
                                                              forceString(record.value('BIK')),
                                                              forceString(record.value('schet')),
                                                              pacc_str
        ))
        table.setText(1, 2, u'Расчетный:')
        table.setText(1, 3, u'ИНН %s, КПП %s, %s, БИК: %s, р/с %s' %  (forceString(record_pay.value('INN')),
                                                                       forceString(record_pay.value('KPP')),
                                                                       forceString(record_pay.value('BankName')),
                                                                       forceString(record_pay.value('BIK')),
                                                                       forceString(record_pay.value('schet'))
        ))

        chr.setFontWeight(QtGui.QFont.Bold)
        table.cellAt(2,0).setFormat(chr)
        table.cellAt(2,2).setFormat(chr)
        chr.setFontUnderline(True)
        chr.setFontWeight(QtGui.QFont.Normal)
        table.cellAt(2,1).setFormat(chr)
        table.cellAt(2,3).setFormat(chr)
        table.setText(2, 0, u'Адрес:')
        table.setText(2, 1, forceString(record.value('Address')))
        table.setText(2, 2, u'Адрес:')
        table.setText(2, 3, forceString(record_pay.value('Address')))
        invoice_cursor.movePosition(QtGui.QTextCursor.End)


        fmt =QtGui.QTextBlockFormat()
        fmt.setAlignment(Qt.AlignCenter)
        invoice_cursor.insertBlock(fmt)
        invoice_cursor.insertHtml(u"<br><b>СЧЕТ № %s от %s</b><br>" % ('_'*3,invoice_date.rjust(10,'_')))
        invoice_cursor.insertHtml(u"к реестру счетов № %s от %s за %s %s %s<br>" % (invoice_num,invoice_date,str(periodDate.year()), monthName[periodDate.month()], accName))
        tableColumns = [
            ('15%', [ u'Предмет счета' ],CReportBase.AlignCenter ),
            ('45%',  [ u'Наименование' ], CReportBase.AlignCenter ),
            ('15%',  [ u'Един. изм.' ], CReportBase.AlignCenter ),
            ('10%',  [ u'Количество'], CReportBase.AlignCenter ),
            ('15%',  [ u'Сумма, руб.'], CReportBase.AlignCenter )
            ]
        invoice_cursor.movePosition(QtGui.QTextCursor.End)
        invoice_cursor.insertBlock()

        table = createTable (invoice_cursor,tableColumns, 33, border=1, cellPadding=0, cellSpacing=0)

        for i in range(0,5):
            table.setText(1,i,str(i+1),blockFormat=fmtc)
        table.mergeCells(2,0,28,1)

        table.setText(2,0,u'Медицинская помощь, оказанная застрахованным гражданам',blockFormat=fmtc)

        table.setText(2,1,u'Стационар',blockFormat=fmtl)
        table.mergeCells(2,1,2,1)
        table.setText(4,1,u'Поликлиника',blockFormat=fmtl)
        table.mergeCells(4,1,2,1)
        table.setText(6,1,u'Стомотология',blockFormat=fmtl)
        table.mergeCells(6,1,3,1)
        table.setText(9,1,u'Дневной стационар',blockFormat=fmtl)
        table.mergeCells(9,1,2,1)
        table.setText(11,1,u'Стационар дневного пребывания',blockFormat=fmtl)
        table.mergeCells(11,1,2,1)
        table.setText(13,1,u'Женская консультация',blockFormat=fmtl)
        table.mergeCells(13,1,2,1)
        table.setText(15,1,u'Стационар на дому',blockFormat=fmtl)
        table.mergeCells(15,1,2,1)
        table.setText(17,1,u'Центр здоровья',blockFormat=fmtl)
        table.setText(18,1,u'Приемное отделение',blockFormat=fmtl)
        table.setText(19,1,u'Диагностические исследования',blockFormat=fmtl)
        table.setText(20,1,u'Фельдшерско-акушерский пункт',blockFormat=fmtl)
        table.setText(21,1,u'Скорая медицинская помощь',blockFormat=fmtl)
        table.setText(22,1,u'Диспансеризация детей-сирот',blockFormat=fmtl)
        table.setText(23,1,u'Диспансеризация детей остав-ся без попечения родителей',blockFormat=fmtl)
        table.setText(24,1,u'Диспансеризация взрослого населения',blockFormat=fmtl)
        table.setText(25,1,u'Неотложная помощь',blockFormat=fmtl)
        table.setText(26,1,u'Медицинские осмотры взрослых',blockFormat=fmtl)
        table.setText(27,1,u'Медицинские осмотры несовершеннолетних',blockFormat=fmtl)
        table.setText(28,1,u'Высокотехнологичная медицинская помощь',blockFormat=fmtl)
        table.mergeCells(28,1,2,1)

        table.setText(2,2,u'Случаи',blockFormat=fmtc)
        table.setText(2,4,u'X',blockFormat=fmtc)
        table.setText(3,2,u'койко-день',blockFormat=fmtc)
        table.setText(4,2,u'посещение',blockFormat=fmtc)
        table.setText(5,2,u'обращение',blockFormat=fmtc)
        table.setText(5,4,u'X',blockFormat=fmtc)
        table.setText(6,2,u'УЕТ',blockFormat=fmtc)
        table.setText(7,2,u'посещение',blockFormat=fmtc)
        table.setText(8,2,u'обращение',blockFormat=fmtc)
        table.setText(8,4,u'X',blockFormat=fmtc)
        table.setText(9,2,u'Случаи',blockFormat=fmtc)
        table.setText(9,4,u'X',blockFormat=fmtc)
        table.setText(10,2,u'пациенто-день',blockFormat=fmtc)
        table.setText(11,2,u'Случаи',blockFormat=fmtc)
        table.setText(11,4,u'X',blockFormat=fmtc)
        table.setText(12,2,u'пациенто-день',blockFormat=fmtc)
        table.setText(13,2,u'посещение',blockFormat=fmtc)
        table.setText(14,2,u'обращение',blockFormat=fmtc)
        table.setText(14,4,u'X',blockFormat=fmtc)
        table.setText(15,2,u'Случаи',blockFormat=fmtc)
        table.setText(15,4,u'X',blockFormat=fmtc)
        table.setText(16,2,u'пациенто-день',blockFormat=fmtc)
        table.setText(17,2,u'посещение',blockFormat=fmtc)
        table.setText(18,2,u'посещение',blockFormat=fmtc)
        table.setText(19,2,u'услуга',blockFormat=fmtc)
        table.setText(20,2,u'посещение',blockFormat=fmtc)
        table.setText(21,2,u'вызов',blockFormat=fmtc)
        table.setText(22,2,u'посещение',blockFormat=fmtc)
        table.setText(23,2,u'посещение',blockFormat=fmtc)
        table.setText(24,2,u'посещение',blockFormat=fmtc)
        table.setText(25,2,u'посещение',blockFormat=fmtc)
        table.setText(26,2,u'посещение',blockFormat=fmtc)
        table.setText(27,2,u'посещение',blockFormat=fmtc)
        table.setText(28,2,u'Случаи',blockFormat=fmtc)
        table.setText(28,4,u'X',blockFormat=fmtc)
        table.setText(29,2,u'койко-день',blockFormat=fmtc)

        summ = 0.0;
        #Стационар
        kolsn = 0
        kolkd = 0
        vpsumm = 0.0
        for rep in reps:
            if rep['vp'] in ('11', '12', '301', '302'):
                kolsn += rep['tsn']
                kolkd += rep['tkd']
                vpsumm += rep['tsumm']
        table.setText(2,3,kolsn,blockFormat=fmtc)
        table.setText(3,3,kolkd,blockFormat=fmtc)
        table.setText(3,4,'%.2f' % vpsumm, blockFormat=fmtc)
        summ += vpsumm

        #Поликлиника
        vpsumm = 0.0
        kolpos = 0
        obr = 0
        for rep in reps:
            if rep['vp'] in ('21', '22', '222'):
                vpsumm += rep['tsumm']
                kolpos += rep['tcp']
                obr += rep['tobr']
        table.setText(4,3,kolpos,blockFormat=fmtc)
        table.setText(4,4,'%.2f' % vpsumm,blockFormat=fmtc)
        table.setText(5,3,obr,blockFormat=fmtc)
        summ += vpsumm

        #Стоматология
        kolpos = 0
        vpsumm = 0.0
        uet = 0.0
        obr = 0
        for rep in reps:
            if rep['vp'] in ('31', '32'):
                kolpos += rep['tcp']
                vpsumm += rep['tsumm']
                uet += rep['tuet']
                obr += rep['tobr']
        table.setText(6,3,'%.2f' % uet,blockFormat=fmtc)
        table.setText(6,4,'%.2f' % vpsumm,blockFormat=fmtc)
        table.setText(7,3, kolpos,blockFormat=fmtc)
        table.setText(7,4, 0,blockFormat=fmtc)
        table.setText(8,3,obr,blockFormat=fmtc)
        summ += vpsumm

        #Дневной стационар
        kolsn = 0
        kolkd = 0
        vpsumm = 0.0
        for rep in reps:
            if rep['vp'] in ('41', '42', '43'):
                kolsn += rep['tsn']
                kolkd += rep['tpd']
                vpsumm += rep['tsumm']
        table.setText(9,3,kolsn,blockFormat=fmtc)
        table.setText(10,3,kolkd,blockFormat=fmtc)
        table.setText(10,4,'%.2f' % vpsumm,blockFormat=fmtc)
        summ += vpsumm

        #Стационар дневного пребывания
        kolsn = 0
        kolkd = 0
        vpsumm = 0.0
        for rep in reps:
            if rep['vp'] in ('51', '52'):
                kolsn += rep['tsn']
                kolkd += rep['tpd']
                vpsumm += rep['tsumm']
        table.setText(11,3, kolsn,blockFormat=fmtc)
        table.setText(12,3,kolkd,blockFormat=fmtc)
        table.setText(12,4,'%.2f' % vpsumm,blockFormat=fmtc)
        summ += vpsumm

        #Женская консультация
        vpsumm = 0.0
        kolpos = 0
        obr = 0
        for rep in reps:
            if rep['vp'] == '60':
                vpsumm += rep['tsumm']
                kolpos += rep['tcp']
                obr += rep['tobr']
        table.setText(13,3, kolpos,blockFormat=fmtc)
        table.setText(13,4,'%.2f' % vpsumm,blockFormat=fmtc)
        table.setText(14,3,kolkd,blockFormat=fmtc)
        summ += vpsumm

        #Стационар на дому
        kolsn = 0
        kolkd = 0
        vpsumm = 0.0
        for rep in reps:
            if rep['vp'] in ('71', '72'):
                kolsn += rep['tsn']
                kolkd += rep['tpd']
                vpsumm += rep['tsumm']
        table.setText(15,3, kolsn,blockFormat=fmtc)
        table.setText(16,3, kolkd,blockFormat=fmtc)
        table.setText(16,4,'%.2f' % vpsumm,blockFormat=fmtc)
        summ += vpsumm

        #ЦЗ
        vpsumm = 0.0
        kolpos = 0
        for rep in reps:
            if rep['vp'] in ('01', '02'):
                vpsumm += rep['tsumm']
                kolpos += rep['tcp']
        table.setText(17,3,kolpos,blockFormat=fmtc)
        table.setText(17,4,'%.2f' % vpsumm,blockFormat=fmtc)
        summ += vpsumm

        #Приемное отделение
        vpsumm = 0.0
        kolpos = 0
        for rep in reps:
            if rep['vp'] in ('111', '112'):
                vpsumm += rep['tsumm']
                kolpos += rep['tcp']
        table.setText(18,3,kolpos,blockFormat=fmtc)
        table.setText(18,4,'%.2f' % vpsumm,blockFormat=fmtc)
        summ += vpsumm

        #Диагностические исследования
        vpsumm = 0.0
        kolpos = 0
        for rep in reps:
            if rep['vp'] == '80':
                vpsumm += rep['tsumm']
                kolpos += rep['tkolu']
        table.setText(19,3,kolpos,blockFormat=fmtc)
        table.setText(19,4,'%.2f' % vpsumm,blockFormat=fmtc)
        summ += vpsumm

        #ФАП
        vpsumm = 0.0
        kolpos = 0
        for rep in reps:
            if rep['vp'] in ('201', '202'):
                vpsumm += rep['tsumm']
                kolpos += rep['tcp']
        table.setText(20,3,kolpos,blockFormat=fmtc)
        table.setText(20,4,'%.2f' % vpsumm,blockFormat=fmtc)
        summ += vpsumm

        #Скорая
        vpsumm = 0.0
        kolpos = 0
        for rep in reps:
            if rep['vp'] in ('801', '802'):
                vpsumm += rep['tsumm']
                kolpos += rep['tcp']
        table.setText(21,3,kolpos,blockFormat=fmtc)
        table.setText(21,4,'%.2f' % vpsumm,blockFormat=fmtc)
        summ += vpsumm

        #Дисп. детей-сирот
        vpsumm = 0.0
        kolpos = 0
        for rep in reps:
            if rep['vp'] == '232':
                vpsumm += rep['tsumm']
                kolpos += rep['tcp']
        table.setText(22,3,kolpos,blockFormat=fmtc)
        table.setText(22,4,'%.2f' % vpsumm,blockFormat=fmtc)
        summ += vpsumm

        #Дисп. детей без попечения
        vpsumm = 0.0
        kolpos = 0
        for rep in reps:
            if rep['vp'] == '252':
                vpsumm += rep['tsumm']
                kolpos += rep['tcp']
        table.setText(23,3,kolpos,blockFormat=fmtc)
        table.setText(23,4,'%.2f' % vpsumm,blockFormat=fmtc)
        summ += vpsumm

        #Дисп. взрослых
        vpsumm = 0.0
        kolpos = 0
        for rep in reps:
            if rep['vp'] == '211':
                vpsumm += rep['tsumm']
                kolpos += rep['tcp']
        table.setText(24,3,kolpos,blockFormat=fmtc)
        table.setText(24,4,'%.2f' % vpsumm,blockFormat=fmtc)
        summ += vpsumm

        #неотложная помощь
        vpsumm = 0.0
        kolpos = 0
        for rep in reps:
            if rep['vp'] in ('241', '242'):
                vpsumm += rep['tsumm']
                kolpos += rep['tcp']
        table.setText(25,3,kolpos,blockFormat=fmtc)
        table.setText(25,4,'%.2f' % vpsumm,blockFormat=fmtc)
        summ += vpsumm

        #Медосм. взрослых
        vpsumm = 0.0
        kolpos = 0
        for rep in reps:
            if rep['vp'] == '261':
                vpsumm += rep['tsumm']
                kolpos += rep['tcp']
        table.setText(26,3,kolpos,blockFormat=fmtc)
        table.setText(26,4,'%.2f' % vpsumm,blockFormat=fmtc)
        summ += vpsumm

        #Медосм. несовершеннолетних
        vpsumm = 0.0
        kolpos = 0
        for rep in reps:
            if rep['vp'] == '262':
                vpsumm += rep['tsumm']
                kolpos += rep['tcp']
        table.setText(27,3,kolpos,blockFormat=fmtc)
        table.setText(27,4,'%.2f' % vpsumm,blockFormat=fmtc)
        summ += vpsumm

        #ВМП
        kolsn = 0
        kolkd = 0
        vpsumm = 0.0
        for rep in reps:
            if rep['vp'] in ('401', '402'):
                kolsn += rep['tsn']
                kolkd += rep['tkd']
                vpsumm += rep['tsumm']
        table.setText(28,3,kolsn,blockFormat=fmtc)
        table.setText(29,3,kolkd,blockFormat=fmtc)
        table.setText(29,4,'%.2f' % vpsumm,blockFormat=fmtc)
        summ += vpsumm

        rownum = 30
        table.mergeCells(rownum,0,3,3)
        chr.setFontWeight(QtGui.QFont.Bold)
        chr.setFontUnderline(False)
        table.cellAt(rownum,3).setFormat(chr)
        table.setText(rownum,3,u'Итого',blockFormat=fmtl);
        table.setText(rownum,4,u'%.2f'%summ,blockFormat=fmtc);
        rownum+=1
        table.cellAt(rownum,3).setFormat(chr)
        table.setText(rownum,3,u'НДС',blockFormat=fmtl);
        table.setText(rownum,4,u'-',blockFormat=fmtc);
        rownum+=1
        table.cellAt(rownum,3).setFormat(chr)
        table.setText(rownum,3,u'Всего без НДС',blockFormat=fmtl);
        table.setText(rownum,4,u'%.2f'%summ,blockFormat=fmtc);
        rownum+=1
        invoice_cursor.movePosition(QtGui.QTextCursor.End)
        fmt =QtGui.QTextBlockFormat()
        fmt.setAlignment(Qt.AlignLeft)
        invoice_cursor.insertBlock(fmt)
        invoice_cursor.insertHtml(u"<br><b>Итого:</b> <u><i>%s</i></u>" % amountToWords(summ))
        invoice_cursor.movePosition(QtGui.QTextCursor.End)
        invoice_cursor.insertHtml(u"<br><br>"+u"&nbsp;"*50+u"Главный врач __________ %30s."%forceString(record.value('chief')) +
                                  u"&nbsp;"*50+u"Гл. бухгалтер __________ %30s."%forceString(record.value('accountant')))
        invoice_cursor.insertBlock(fmtdiv)
        invoice_cursor.insertHtml("$$$$=$$$$")
        invoice_writer = QtGui.QTextDocumentWriter()
        filePath = os.path.join(forceStringEx(self.getTmpDir()),os.path.basename("schfakt.html"))
        invoice_writer.setFileName(filePath)
        invoice_writer.setFormat("HTML")
        invoice_writer.write(invoice)
        #Разрывы стрниц
        f = open(filePath)
        inv = f.read()
        inv = inv.replace('$$$$=$$$$',r'</div>')
        inv = inv.replace('@@@@=@@@@',r'<div style="page-break-after:always;">')
        inv = inv.replace('####=####',r'<div">')
        f.close()

        f = open(filePath,'w')
        f.seek(0)
        f.write(inv)
        f.close()

# *****************************************************************************************

    def createDbf(self):
        if self.exportType == self.exportTypeP13:
            return (self.createPDbf(False), self.createUDbf(False),
                        self.createDDbf(False), self.createSDbf(False),
                        self.createNDbf(False))
        elif self.exportType == self.exportTypeP12:
            return (self.createPDbf(False), self.createUDbf(False),
                        self.createDDbf(False), self.createLDbf(False),
                        self.createNDbf(False))
        else:
            return (self.createPDbf(False), self.createUDbf(False),
                        self.createBDbf(False), self.createDDbf(False),
                        self.createSDbf(False), self.createKDbf(False),
                        self.createLDbf(False),
                        # модернизация
                        self.createPDbf(True), self.createUDbf(True),
                        self.createBDbf(True), self.createDDbf(True),
                        self.createSDbf(True), self.createKDbf(True),
                        self.createLDbf(True))


    def createPDbf(self, modern):
        u"""Создает структуру для паспортной части реестра счетов."""

        dbfName = os.path.join(self.getTmpDir(), 'P'+self.getDbfBaseName(modern))
        dbf = Dbf(dbfName, new=True, encoding='cp866')

        nsSize = 5 if self.exportType in (self.exportTypeP12, self.exportTypeP13, self.exportTypeFLK) else 4

        dbf.addField(
            ('NS', 'N', nsSize, 0),  # номер реестра счетов (п. 1 примечаний) обязательное
            ('VS', 'C', 1),      # тип реестра счетов (п. 2;4 примечаний) обязательное SPR21
            ('DATS', 'D'),      # дата формирования реестра счетов (п. 3 примечаний) обязательное
            ('SN', 'N', 10, 0), # номер персонального счета обязательное
            ('DATPS', 'D'),     #дата формирования персонального счета обязательное
            ('CODE_MO', 'C', 5),    #код медицинской организации в системе ОМС,
                                            #предоставившей медицинскую помощь обязательное SPR01
        )
        if self.exportType != self.exportTypeFLK:
            dbf.addField(
                                            #предоставившей медицинскую помощь обязательное SPR01
            ('PL_OGRN', 'C', 15),     # ОГРН плательщика (п. 4 примечаний) обязательное SPR02
            )
        if self.exportType == self.exportTypeP10:
            dbf.addField(
                ('KSO', 'C', 1),         # код метода оплаты (п. 4 примечаний) обязательное SPR17
                ('FIO', 'C', 40),      # фамилия (п. 5 примечаний) обязательное
                ('IMA', 'C', 40),       # имя (п. 5 примечаний) обязательное
                ('OTCH', 'C', 40),      # отчество (п. 5 примечаний)
            )
        else:
            dbf.addField(
                ('FIO', 'C', 30),      # фамилия (п. 5 примечаний) обязательное
                ('IMA', 'C', 20),       # имя (п. 5 примечаний) обязательное
                ('OTCH', 'C', 30),      # отчество (п. 5 примечаний)
            )

        dbf.addField(
            ('POL', 'C', 1),         # символьное (1)   пол (М/Ж) (п. 6 примечаний) обязательное
            ('DATR', 'D'),           # дата рождения (п. 7 примечаний) обязательное
        )

        if self.exportType not in (self.exportTypeP12, self.exportTypeP13, self.exportTypeFLK):
            dbf.addField(
                ('C_OKATO2', 'C', 5),   # код ОКАТО территории постоянной регистрации пациента обязательное для инокраевых SPR39
            )

        if self.exportType != self.exportTypeFLK:
            dbf.addField(
                         ('KAT', 'C', 1),         # категория граждан обязательное SPR09
            )

        dbf.addField(
            ('SNILS', 'C', 14),    # СНИЛС
        )

        if self.exportType == self.exportTypeP10:
            dbf.addField(
                ('STAT_Z', 'N', 2, 0),    # статус пациента обязательное SPR40
            )

        dbf.addField(
            ('OKATO_OMS', 'C', 5),    # код ОКАТО территории страхования по ОМС обязательное для инокраевых SPR39
        )

        if self.exportType== self.exportTypeP11:
            dbf.addField(
                ('SMOCODE', 'C', 5),      #реестровый номер инокраевой СМО, выдавшей полис ОМС
            )

        if self.exportType not in (self.exportTypeP12, self.exportTypeP13, self.exportTypeFLK):
            dbf.addField(
                ('Q_OGRN', 'C', 15),    #   ОГРН СМО, выдавшей полис ОМС обязательное для инокраевых СМО SPR33
            )

        if self.exportType in (self.exportTypeP11,self.exportTypeFLK, self.exportTypeP12, self.exportTypeP13):
            dbf.addField(
                ('SPV', 'N', 1), #тип документа, подтверждающего факт страхования по ОМС (п. 14 примечаний)
            )

        spsSize = 10 if self.exportType in (self.exportTypeP11,self.exportTypeFLK, self.exportTypeP12, self.exportTypeP13) else 12

        dbf.addField(
            ('SPS', 'C', spsSize),  # серия полиса ОМС обязательное
            ('SPN', 'C', 20),  # номер полиса ОМС обязательное
        )

        if self.exportType == self.exportTypeP10:
            dbf.addField(
                ('DATNSP', 'D'),  # дата начала действия полиса ОМС (п. 8 примечаний) обязательное для инокраевых
                ('DATOSP', 'D'),  # дата окончания действия полиса ОМС (п. 8 примечаний) обязательное для инокраевых
            )

        dbf.addField(
            ('STAT_P', 'C', 1), #   статус представителя пациента  обязательное для инокраевых SPR41
            ('Q_G', 'C', 7), # признак "Особый случай" при регистрации обращения
                                    #за медицинской помощью (п. 9 примечаний) SPR42
        )

        if self.exportType in (self.exportTypeP11,self.exportTypeFLK, self.exportTypeP12, self.exportTypeP13):
            dbf.addField(
                ('NOVOR', 'C', 9), # признак новорожденного (п. 17 примечаний)
            )

        (fampSize,  impSize,  otpSize) = (30, 20, 30) if self.exportType in (self.exportTypeP11,self.exportTypeFLK, self.exportTypeP13) else (40, 40, 40)

        dbf.addField(
            ('FAMP', 'C', fampSize), # фамилия родителя (представителя) пациента (п. 5 примечаний) обязательное для инокраевых
            ('IMP', 'C', impSize), # имя родителя (представителя) пациента (п. 5 примечаний) обязательное для инокраевых
            ('OTP', 'C', otpSize), # отчество родителя (представителя) пациента (п. 5 примечаний)
        )

        if self.exportType in (self.exportTypeP12, self.exportTypeP13, self.exportTypeFLK):
            dbf.addField(
                ('POLP', 'C', 1),         # пол представителя пациента (М/Ж)
                ('DATRP', 'D'),           # дата рождения представителя пациента
            )


        dbf.addField(
            ('C_DOC', 'N', 2, 0),    # код типа документа, удостоверяющего личность пациента (представителя)
                                        #(п. 10 примечаний) SPR43
            ('S_DOC', 'C', 10), # серия документа, удостоверяющего личность пациента (представителя) (п. 10 примечаний)
            ('N_DOC', 'C', 15), # номер документа, удостоверяющего личность пациента (представителя) (п. 10 примечаний)
        )

        if self.exportType not in (self.exportTypeFLK, self.exportTypeP12, self.exportTypeP13):
            dbf.addField(
             ('R_NAME', 'C', 50 if self.exportType in (self.exportTypeP11,self.exportTypeFLK) else 150), # наименование района по месту регистрации пациента (п. 11 примечаний)
                                            # обязательное, в случае отсутствия района “-”
            ('Q_NP', 'N', 2, 0), #    код вида населенного пункта (п. 12 примечаний) обязательное SPR44
            ('NP_NAME', 'C', 50 if self.exportType in (self.exportTypeP11,self.exportTypeFLK) else 150), # наименование населенного пункта по месту  регистрации (п. 11 примечаний)    обязательное
            ('Q_UL', 'N', 2, 0), # код типа наименования улицы (п. 12 примечаний)     SPR45
            ('UL_NAME', 'C', 50 if self.exportType in (self.exportTypeP11,self.exportTypeFLK) else 150), # наименование улицы (п. 11 примечаний)
            ('DOM', 'C', 7), # дом (п. 13 примечаний) обязательное
            ('KOR', 'C', 5), # корпус/строение (п. 13 примечаний)
            ('KV', 'C', 5), # квартира/комната (п. 13 примечаний)
            )

        dbf.addField(
               ('KSMO', 'C', 4), # код страховщика (Краснодарского края) обязательное для краевых счетов SPR03
        )


        if self.exportType == self.exportTypeP10:
            dbf.addField(
                ('STRANA', 'C', 3), # код страны проживания (гражданство) обязательное SPR05
                ('MRAB', 'C', 150), # место работы/учебы пациента (п. 14 примечаний)
                                                #обязательное в соответсвии со справочником SPR40
            )


        if self.exportType != self.exportTypeFLK:
            dbf.addField(
                ('NAPR_MO', 'C', 5), # код направившей медицинской организации в системе
                                            #ОМС обязательное для внешних услуг    SPR01
            )

        if self.exportType in (self.exportTypeP12, self.exportTypeP13):
            dbf.addField(
                ('NAPR_N', 'C', 13), # номер направления (п.15 примечаний)
            )

        if self.exportType != self.exportTypeFLK:
            dbf.addField(
                ('ISTI', 'C', 10),     # номер амбулаторной карты или истории болезни обязательное кроме внешних  услуг
            )

        if self.exportType in (self.exportTypeP12, self.exportTypeP13):
            dbf.addField(
                ('DATN', 'D'), # дата начала лечения
                ('DATO', 'D'), # дата окончания лечения
            )

        if self.exportType != self.exportTypeFLK:
            ishSize =  3  if self.exportType  in (self.exportTypeP11, self.exportTypeP12, self.exportTypeP13) else 2
            dbf.addField(
                ('ISHL', 'C', ishSize),       # код исхода заболевания обязательное SPR11
                ('ISHOB', 'C', ishSize),     # код исхода обращения обязательное SPR12
                ('MP', 'C', 1),          # код вида обращения обязательное SPR14
            )

        if self.exportType not in (self.exportTypeP12, self.exportTypeP13):
            dbf.addField(
                ('DATN', 'D'),          # дата начала лечения обязательное
                ('DATO', 'D'),          # дата окончания лечения обязательное
            )

        if self.exportType == self.exportTypeP10:
            dbf.addField(
                ('IS_VNESH', 'N', 1, 0), # признак: счет за внешнюю медицинскую услугу (п. 15 примечаний) обязательное
                ('NAPR_N', 'C', 8),   # номер направления (п. 16 примечаний) обязательное для внешних услуг
                ('NAPR_D', 'D'),   # дата направления обязательное для внешних услуг
            )

        if self.exportType != self.exportTypeFLK:
            dbf.addField(
                         ('SUMMA_I', 'N', 14,2), # сумма к оплате по случаю заболевания пациента обязательное
                         )

        if self.exportType == self.exportTypeP10:
            dbf.addField(
                ('SUMMA_M', 'N', 14,2),
            )

        if self.exportType != self.exportTypeFLK:
            dbf.addField(
                         ('PV', 'C', 40), # коды причин возврата (п. 17 примечаний) обязательное для возвратных счетов SPR15
                         ('DVOZVRAT', 'D'), # дата возврата обязательное для возвратных счетов
                         )

        if self.exportType == self.exportTypeP10:
            dbf.addField(
                ('SPV', 'N', 1),  # вид полиса
            )

        return dbf


    def createUDbf(self, modern):
        u"""Создает структуру dbf для медицинских услуг."""

        dbfName = os.path.join(self.getTmpDir(), 'U'+self.getDbfBaseName(modern))
        dbf = Dbf(dbfName, new=True, encoding='cp866')

        if self.exportType in (self.exportTypeP11, self.exportTypeP12, self.exportTypeP13):
            specFieldLen = 9
        else:
            specFieldLen = 3

        if self.exportType in (self.exportTypeP11,  self.exportTypeP12, self.exportTypeP13):
            dbf.addField(
                ('UID', 'N', 14),  # уникальный номер записи об оказанной медицинской услуге в пределах реестра (п. 1 примечаний)
            )

        nsSize = 5 if self.exportType in (self.exportTypeP12, self.exportTypeP13, self.exportTypeFLK) else 4

        dbf.addField(
            ('CODE_MO', 'C', 5),  # код медицинской организации в системе ОМС,
                                        # предоставившей медицинскую помощь обязательное SPR01
            ('NS', 'N', nsSize, 0),   # номер реестра счетов (п. 1 примечаний) обязательное
            ('SN', 'N', 10, 0), # номер персонального счета обязательное
            ('KOTD', 'C', 4),   # символьное (4) код отделения (п. 2 примечаний) обязательное SPR07
            ('KPK', 'C', 2),     # код профиля койки (п. 2 примечаний) обязательное для стационаров SPR08
            ('MKBX', 'C', 5),  # код диагноза основного заболевания по МКБ–Х (п. 2 примечаний)
                                        # обязательное для всех услуг, кроме диагностических SPR20
            ('MKBXS', 'C', 5), # код диагноза сопутствующего заболевания по МКБ–Х (п. 2 примечаний) SPR20
            ('KSTAND', 'C', 14), # код стандарта оказания медицинской помощи обязательное
                                            # для нозологий, у которых разработан СОМП SPR38
            )

        dbf.addField(
            ('VP', 'C', 3), # код условия оказания медицинской помощи (п. 2, 3 примечаний) обязательное SPR13
            ('KUSL', 'C', 14), # код медицинской услуги обязательное SPR18
            ('KOLU', 'N', 3, 0), # количество услуг обязательное
            ('KD', 'N', 3, 0),  # количество койко-дней обязательное  для стационаров
            ('DATN', 'D'), # дата начала выполнения услуги обязательное
            ('DATO', 'D'), # дата окончания выполнения услуги обязательное
        )

        if self.exportType in (self.exportTypeP11, self.exportTypeP12, self.exportTypeP13):
            dbf.addField(
                ('TARU', 'N', 10, 2), # Тариф (см.п.6 примечаний)   обязательное    SPR22
                ('TARU_B', 'N', 10, 2), # Базовая часть тарифа услуги по ОМС из справочника spr 22 поле TARU_B
                ('TARU_DM', 'N', 10, 2), # в т.ч. по выполнению СОМП и повышению доступности АПМП
                ('TARU_D', 'N', 10, 2), #тариф на оплату дополнительных статей расходов из справочника spr 22 поле TARIF_D
                ('TARU_UC', 'N', 10, 2), #тариф стимулирующих выплат медицинскому персоналу участковой службы по ОМС
            )
        else:
            dbf.addField(
                ('TARU', 'N', 12, 2), # Тариф (см.п.6 примечаний)   обязательное    SPR22
                ('TARU_B', 'N', 12, 2), # Базовая часть тарифа услуги по ОМС из справочника spr 22 поле TARU_B
                ('TARU_D', 'N', 12, 2), #тариф на оплату дополнительных статей расходов из справочника spr 22 поле TARIF_D
                ('TARU_M', 'N', 12, 2),
            )

        dbf.addField(
            ('SUMM', 'N', 14, 2), # сумма к оплате  обязательное
            ('SUMM_B', 'N', 14, 2), #сумма по базовой части тарифа к оплате по ОМС поле SUMM _B
        )

        if self.exportType in (self.exportTypeP11, self.exportTypeP12, self.exportTypeP13):
            dbf.addField(
                ('SUMM_DM', 'N', 14, 2), #в т.ч. по выполнению СОМП и повышению доступности АПМП
            )

        dbf.addField(
            ('SUMM_D', 'N', 14, 2), # сумма дополнительных статей расходов к оплате по ОМС
            ('SUMM_K', 'N', 14, 2), # сумма расходов на оплату коммунальных услуг (п. 9 примечаний)
        )

        if self.exportType == self.exportTypeP10:
            dbf.addField(
                ('SUMM_M', 'N', 14, 2),
            )

        if self.exportType in (self.exportTypeP11, self.exportTypeP12, self.exportTypeP13):
            dbf.addField(
                ('SUMM_UC', 'N', 14, 2), #сумма расходов на оплату стимулирующих выплат
                                                    #медицинскому персоналу участковой службы по ОМС
            )

        dbf.addField(
            ('IS_OUT', 'N', 1, 0), # признак: услуга оказана в другой медицинской организации (п. 4 примечаний) обязательное
            ('OUT_MO', 'C', 5), # код медицинской организации в системе ОМС, оказавшей услугу (п. 5 примечаний) SPR01
            ('DOC_TABN', 'C', 10), #табельный номер специалиста, оказавшего услугу
            ('SPEC', 'C', specFieldLen) # код специальности специалиста, оказавшего услугу
        )

        if self.exportType in (self.exportTypeP11, self.exportTypeP12, self.exportTypeP13):
            vmpSize = 2 if self.exportType in (self.exportTypeP12, self.exportTypeP13) else 1
            dbf.addField(
                ('PROFIL', 'C', 3), # профиль оказанной медицинской помощи
                ('VMP', 'C', vmpSize), # вид медицинской помощи
                ('KSO', 'C', 2), # способ оплаты медицинской помощи
            )
        dbf.addField(
            ('UET', 'N', 12, 2),
        )
        return dbf


    def createBDbf(self, modern):
        u"""Создает структуру dbf для листка временной нетрудоспособности."""

        dbfName = os.path.join(self.getTmpDir(), 'B'+self.getDbfBaseName(modern))
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('CODE_MO', 'C', 5),  # код медицинской организации в системе ОМС,
                                            # предоставившей медицинскую помощь обязательное SPR01
            ('NS', 'N', 4, 0), # номер реестра счетов обязательное
            ('SN', 'N', 10, 0), # номер персонального счета обязательное
            ('BLS', 'C', 10), # серия листка нетрудоспособности обязательное
            ('BLN', 'N', 7, 0), # номер листка нетрудоспособности обязательное
            ('KPBL', 'N', 2, 0), # причина нетрудоспособности обязательное SPR27
            ('DATN', 'D'), # дата выписки б.листа обязательное
            ('DATO', 'D'), # дата закрытия б.листа обязательное
            ('MKBX', 'C', 5), # код диагноза по МКБ–Х SPR20
        )

        return dbf


    def createDDbf(self, modern):
        u"""Специалисты структурного подразделения"""
        dbfName = os.path.join(self.getTmpDir(), 'D'+self.getDbfBaseName(modern))
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        (fioSize,  imaSize,  otchSize) = (30, 20, 30) if self.exportType in (self.exportTypeP11, self.exportTypeP12, self.exportTypeP13) else (40, 40, 40)

        dbf.addField(
            ('CODE_MO', 'C', 5), # код медицинской организации в системе ОМС, предоставившей медицинскую помощь
            ('DOC_TABN', 'C', 10), #табельный номер специалиста (врача)
            ('FIO', 'C', fioSize), #фамилия специалиста (врача)
            ('IMA', 'C', imaSize), #имя специалиста (врача)
            ('OTCH', 'C', otchSize), # отчество специалиста (врача)
            ('POL', 'C', 1), # пол (М/Ж)
            ('DATR', 'D'),  # дата рождения
            ('DATN', 'D'),  # дата устройства на работу
            ('DATO', 'D'),  # дата увольнения
        )
        return dbf


    def createSDbf(self, modern):
        dbfName = os.path.join(self.getTmpDir(), 'S'+self.getDbfBaseName(modern))
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('CODE_MO', 'C', 5), #код медицинской организации в системе ОМС, предоставившей мед. помощь
        )

        if self.exportType == self.exportTypeP13:
            dbf.addField(
                ('NS', 'N', 5), #номер реестра счетов
                ('SOFT', 'C', 2),  #код используемого программного обеспечения
            )
        else:
            dbf.addField(
                ('DOC_TABN', 'C', 10), #табельный номер врача
                ('SPEC', 'C', 5), #специальность врача
                ('KOTD', 'C', 4),  #код отделения
                ('DOLG', 'C', 4),  #занимаемая должность
                ('DATN', 'D'),  #дата начала работы в  должности
                ('DATO', 'D'),  #дата окончания работы в должности
            )
        return dbf


    def createLDbf(self, modern):
        u"""Назначенные лекарственные препараты, имя файла LXXXXX.dbf"""
        dbfName = os.path.join(self.getTmpDir(), 'L'+self.getDbfBaseName(modern))
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        nsSize = 5 if self.exportType == self.exportTypeP12 else 4

        dbf.addField(
            ('CODE_MO', 'C', 5), #код медицинской организации
            ('NS', 'N', nsSize), #номер реестра счетов
            ('SN', 'N', 10), #номер персонального счета
            ('KSTAND', 'C', 14), # код СОМП, при оказании которого использовался препарат   Да  SPR38
            ('CODETRN', 'N', 10),   #код торгового наименования Да  SPR53
            ('CENAUP', 'N', 10, 2),  #стоимость упаковки, по которой МО закупила препарат   Да
            ('SUMDOZA', 'N', 10, 2), #стоимость примененной курсовой дозы   Да
        )
        return dbf


    def createKDbf(self, modern):
        u"""Койко-дни, проведенные в сопутствующих отделениях, имя файла KXXXXX.dbf"""
        dbfName = os.path.join(self.getTmpDir(), 'K'+self.getDbfBaseName(modern))
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('CODE_MO', 'C', 5), #код медицинской организации
            ('NS', 'N', 4), #номер реестра счетов
            ('SN', 'N', 10), #номер персонального счета
            ('KSTAND', 'C', 14), # код СОМП, при оказании которого пациенту оказывалась МП
            ('KOTD', 'C', 3), # код отделения
            ('KPK', 'C', 2), # код профиля койки
            ('KUSL', 'C', 14), # код койко-дня
            ('DATN', 'D'), # дата начала
            ('DATO', 'D'), # дата окончания
            ('KOLU', 'N', 10), #количество услуг
            ('KD', 'N', 10), #количество койко-дней
        )
        return dbf


    def createNDbf(self, modern):
        u"""Направления на плановую госпитализацию, имя файла NXXXXX.dbf"""
        dbfName = os.path.join(self.getTmpDir(), 'N'+self.getDbfBaseName(modern))
        dbf = Dbf(dbfName, new=True, encoding='cp866')

        dbf.addField(
            ('CODE_MO', 'C', 5), # код медицинской организации
            ('NAPR_N', 'C', 13), # номер направления, (ККККК_ХХХХХХХ)
            ('NAPR_MO', 'C', 5), # код МО, в которое направлен пациент
            ('NAPR_D', 'D'), # дата направления
            ('DOC_TABN', 'C', 10), # табельный номер сотрудника, выдавшего направление
        )
        return dbf


# ******************************************************************************

    def createQuery(self):
        return (self.createAccountQuery(), self.createPersonQuery())

# ******************************************************************************
    def isPos(self,  kusl,  vp):
        res = 0

        #стоматология
        if  vp in ('31',  '32') and (kusl[:7] in ('B01.063', 'B01.064', 'B01.065', 'B01.066', 'B01.067', 'B01.068',
            'B04.063', 'B04.064', 'B04.065', 'B04.066', 'B04.067', 'B04.068') or kusl in ('B02.065.001', 'B02.065.002'))  and kusl not in ('B01.063.004',
            'B01.063.005', 'B01.063.006','B01.063.007','B01.065.004','B01.065.005'):
            res = 1
        #приемное отделение
        if vp in ('111',  '112') and kusl == 'B01.069.003':
            res = 1
        #поликлиника, диспансеризация и медосмотры, диагн. исслед., женск. конс., неотложная помощь, ЦЗ
        if vp in ('21', '22', '232', '211', '222', '252', '261', '262', '80', '60', '241', '242', '01', '02') and kusl[:3] in ('B01',  'B04') and kusl != 'B01.031.015':
            res = 1
        #ФАПы
        if vp in ('201', '202') and (kusl[:10] in ('B02.031.00',  'B02.047.00')  or  kusl[:3] in ('B01',  'B04')):
            res = 1
        return res

 # ******************************************************************************
    def createAccountQuery(self):
        tableAccountItem = self.db.table('Account_Item')
        cond = tableAccountItem['master_id'].inlist(self.selectedAccountIds) \
            if self.exportType == self.exportTypeFLK \
            else tableAccountItem['id'].inlist(self.idList)

        stmt = """SELECT    Account_Item.id AS accountItem_id,
          Account_Item.master_id AS accountId,
          Account_Item.event_id  AS event_id,
          Account_Item.action_id AS action_id,
          Account_Item.visit_id AS visit_id,
          Account_Item.uet AS UET,
          Event.client_id        AS client_id,
          Event.`order`          AS `order`,
          Event.setDate          AS begDate,
          Event.execDate         AS endDate,
          Event.MES_id AS mesId,
          Client.lastName        AS lastName,
          Client.firstName       AS firstName,
          Client.patrName        AS patrName,
          Client.birthDate       AS birthDate,
          Client.sex             AS sex,
          Client.SNILS           AS SNILS,
          ClientPolicy.serial    AS policySerial,
          ClientPolicy.number    AS policyNumber,
          ClientPolicy.note AS policyNote,
          ClientPolicy.begDate AS policyBegDate,
          ClientPolicy.endDate AS policyEndDate,
          Insurer.infisCode      AS policyInsurer,
          Insurer.INN               AS insurerINN,
          Insurer.shortName          AS insurerName,
          Insurer.OKATO AS insurerOKATO,
          Insurer.OGRN AS insurerOGRN,
          Insurer.area AS insurerArea,
          Insurer.id AS insurerId,
          rbPolicyType.code      AS policyType,
          rbPolicyKind.regionalCode AS policyKindCode,
          ClientDocument.serial  AS documentSerial,
          ClientDocument.number  AS documentNumber,
          rbDocumentType.code    AS documentType,
          rbDocumentType.regionalCode AS documentRegionalCode,
          RegSOCR.infisCode AS placeCode,
          RegKLADR.NAME AS cityName,
          RegSTREETSOCR.infisCode AS streetType,
          RegKLADRStreet.NAME AS streetName,
          RegAddressHouse.number AS regNumber,
          RegAddressHouse.corpus AS regCorpus,
          RegAddress.flat AS regFlat,
          RegAddressHouse.KLADRCode AS regKLADRCode,
          RegRegionKLADR.OCATD AS regionOKATO,
          RegRegionKLADR.NAME AS regionName,
          IF(work.title IS NOT NULL,
              work.title, ClientWork.freeInput) AS `workName`,
          IF(Account_Item.service_id IS NOT NULL,
            rbItemService.infis,
            IF(Account_Item.visit_id IS NOT NULL, rbVisitService.infis, rbEventService.infis)
            ) AS service,
          IF(Account_Item.service_id IS NOT NULL,
            Account_Item.service_id,
            IF(Account_Item.visit_id IS NOT NULL, rbVisitService.id, rbEventService.id)
            ) AS serviceId,
          IF(Account_Item.service_id IS NOT NULL,
            Account_Item.service_id,
            IF(Account_Item.visit_id IS NOT NULL, Account_Item.visit_id,
                IF(Account_Item.action_id IS NOT NULL, Account_Item.action_id, Event.id)
            )) AS cardId,
          IF(Account_Item.visit_id IS NOT NULL, Visit.person_id,
                IF(Account_Item.action_id IS NOT NULL,
                        IF(Action.person_id IS NOT NULL, Action.person_id, Action.setPerson_id), Event.execPerson_id)
            ) AS execPersonId,
          Visit.date AS visitDate,
          Action.begDate AS actionDate,
          Diagnosis.MKB          AS MKB,
          AssociatedDiagnosis.MKB AS AssociatedMKB,
          Account_Item.price   AS price,
          Account_Item.amount    AS amount,
          Account_Item.`sum`     AS `sum`,
          Contract_Tariff.federalPrice AS federalPrice,
          Contract_Tariff.id AS tariffId,
          LEAST(
            IF(Contract_Tariff.federalLimitation = 0,Account_Item.amount,
                LEAST(Contract_Tariff.federalLimitation, Account_Item.amount)) * Contract_Tariff.federalPrice,
            Account_Item.sum) AS federalSum,
          Person.code AS personCode,
          Person.regionalCode AS personRegionalCode,
          rbSpeciality.regionalCode AS specialityCode,
          EventType.code AS eventTypeCode,
          DiagnosticResult.regionalCode AS diagnosticResultCode,
          EventResult.regionalCode AS eventResultCode,
          OrgStructure.infisCode AS orgStructCode,
          RelegateOrg.infisCode AS RelegateOrgCode,
          Action.note AS actNote,
          Action.directionDate AS actSetDate,
          Citizenship.regionalCode AS citizenshipCode,
          SocStatus.regionalCode AS socStatusCode,
          HospitalAction.id AS hospitalActionId,
          HospitalDirectionAction.id AS hospitalDirectionActionId,
          rbMedicalAidType.code AS medicalAidTypeCode,
          rbMedicalAidType.regionalCode AS medicalAidTypeRegionalCode,
          BaseExpense.percent AS baseExpensePercent,
          AdditionalExpense.percent AS additionalExpensePercent,
          Contract.regionalTariffRegulationFactor AS regulator,
          IFNULL(ItemMedicalAidKind.code, rbMedicalAidKind.code) AS medicalAidKindCode,
          IF(Account_Item.service_id IS NOT NULL,
              ItemMedicalAidProfile.code,
              IF(Account_Item.visit_id IS NOT NULL, VisitMedicalAidProfile.code, EventMedicalAidProfile.code)
              ) AS medicalAidProfileCode,
          IF(rbMesSpecification.code = '2',1,0) AS mesNotUsedFlag,
          rbMedicalAidUnit.regionalCode AS medicalAidUnitCode,
          Account.number AS accountNumber,
          DATEDIFF(Event.execDate, Event.setDate) as dayCount,
          IF(substr(rbMedicalAidType.name, 1, 3) = '(I ', '5',
            IF(substr(rbMedicalAidType.name, 1, 3) = '(II', '6', '')) as Q_G,
            IFNULL(ActionOrg.infisCode, '') as outOrgCode
        FROM Account_Item
        LEFT JOIN Action ON Action.id = Account_Item.action_id
        LEFT JOIN Visit  ON Visit.id  = Account_Item.visit_id
        LEFT JOIN Event  ON Event.id  = Account_Item.event_id
        LEFT JOIN Client ON Client.id = Event.client_id
        LEFT JOIN ClientPolicy ON ClientPolicy.client_id = Client.id AND
                  ClientPolicy.id = (SELECT MAX(CP.id)
                                     FROM   ClientPolicy AS CP
                                     LEFT JOIN rbPolicyType AS CPT ON CPT.id = CP.policyType_id
                                     WHERE  CP.client_id = Client.id AND CP.deleted=0 AND CPT.code IN ('1','2'))
        LEFT JOIN Organisation AS Insurer ON Insurer.id = ClientPolicy.insurer_id
        LEFT JOIN rbPolicyType ON rbPolicyType.id = ClientPolicy.policyType_id
        LEFT JOIN rbPolicyKind ON rbPolicyKind.id = ClientPolicy.policyKind_id
        LEFT JOIN ClientDocument ON ClientDocument.client_id = Client.id AND
                  ClientDocument.id = (SELECT MAX(CD.id)
                                     FROM   ClientDocument AS CD
                                     LEFT JOIN rbDocumentType AS rbDT ON rbDT.id = CD.documentType_id
                                     LEFT JOIN rbDocumentTypeGroup AS rbDTG ON rbDTG.id = rbDT.group_id
                                     WHERE  rbDTG.code = '1' AND CD.client_id = Client.id AND CD.deleted=0)
        LEFT JOIN rbDocumentType ON rbDocumentType.id = ClientDocument.documentType_id
        LEFT JOIN ClientAddress ClientRegAddress ON ClientRegAddress.client_id = Client.id AND
                  ClientRegAddress.id = (SELECT MAX(CRA.id)
                                     FROM   ClientAddress AS CRA
                                     WHERE  CRA.type = 0 AND CRA.client_id = Client.id AND CRA.deleted=0)
        LEFT JOIN Address   RegAddress ON RegAddress.id = ClientRegAddress.address_id
        LEFT JOIN AddressHouse RegAddressHouse ON RegAddressHouse.id = RegAddress.house_id
        LEFT JOIN kladr.KLADR AS RegKLADR ON RegKLADR.CODE = RegAddressHouse.KLADRCode
        LEFT JOIN kladr.STREET AS RegKLADRStreet ON RegKLADRStreet.CODE = RegAddressHouse.KLADRStreetCode
        LEFT JOIN kladr.SOCRBASE AS RegSTREETSOCR ON RegSTREETSOCR.KOD_T_ST = (SELECT KOD_T_ST
                    FROM kladr.SOCRBASE AS SB
                    WHERE SB.SCNAME = RegKLADRStreet.SOCR
                    LIMIT 1)
        LEFT JOIN kladr.SOCRBASE AS RegSOCR ON RegSOCR.KOD_T_ST = (SELECT KOD_T_ST
                    FROM kladr.SOCRBASE AS SB
                    WHERE SB.SCNAME = RegKLADR.SOCR
                    LIMIT 1)
        LEFT JOIN kladr.KLADR RegRegionKLADR ON RegRegionKLADR.CODE = (
                SELECT RPAD(LEFT(RegAddressHouse.KLADRCode,2),13,'0') LIMIT 1)
        LEFT JOIN Person       ON Person.id = Event.execPerson_id
        LEFT JOIN Person AS setEventPerson ON setEventPerson.id = Event.setPerson_id
        LEFT JOIN Person AS setActionPerson ON setActionPerson.id = Action.setPerson_id
        LEFT JOIN Diagnostic ON Diagnostic.event_id = Account_Item.event_id AND Diagnostic.id = (
            SELECT MAX(d.id) FROM Diagnostic d
            WHERE d.event_id = Account_Item.event_id AND
                        d.diagnosisType_id IN (
                            SELECT id FROM rbDiagnosisType WHERE code IN ('1', '2')) AND d.deleted = 0)
        LEFT JOIN Diagnostic AS AssociatedDiagnostic ON (
            AssociatedDiagnostic.event_id = Account_Item.event_id
            AND AssociatedDiagnostic.diagnosisType_id IN (SELECT id FROM rbDiagnosisType WHERE code='9')
            AND AssociatedDiagnostic.deleted = 0)
            AND AssociatedDiagnostic.id = (
                SELECT id FROM Diagnostic
                WHERE Diagnostic.event_id = Account_Item.event_id
                    AND Diagnostic.diagnosisType_id IN (
                        SELECT id FROM rbDiagnosisType WHERE code='9'
                    )
                    AND Diagnostic.deleted = 0 LIMIT 1
            )
        LEFT JOIN ClientContact  ON ClientContact.client_id = Client.id AND
                  ClientContact.id = (SELECT MAX(CC.id)
                                     FROM   ClientContact AS CC
                                     LEFT JOIN rbContactType ON CC.contactType_id= rbContactType.id
                                     WHERE  rbContactType.code IN (1,2,3) AND CC.client_id = Client.id AND CC.deleted=0)
        LEFT JOIN ClientWork ON ClientWork.client_id=Event.client_id AND
                    ClientWork.id = (SELECT max(CW.id)
                                     FROM ClientWork AS CW
                                     WHERE CW.client_id=Client.id AND CW.deleted=0)
        LEFT JOIN Organisation AS work ON work.id=ClientWork.org_id AND work.deleted = 0
        LEFT JOIN Diagnosis ON Diagnosis.id = Diagnostic.diagnosis_id AND Diagnosis.deleted = 0
        LEFT JOIN Diagnosis AS AssociatedDiagnosis ON AssociatedDiagnosis.id = AssociatedDiagnostic.diagnosis_id
                        AND AssociatedDiagnosis.deleted = 0
        LEFT JOIN rbDiagnosticResult AS DiagnosticResult ON DiagnosticResult.id=Diagnostic.result_id
        LEFT JOIN rbResult AS EventResult ON EventResult.id=Event.result_id
        LEFT JOIN EventType ON EventType.id = Event.eventType_id
        LEFT JOIN rbService AS rbEventService ON rbEventService.id = EventType.service_id
        LEFT JOIN rbService AS rbVisitService ON rbVisitService.id = Visit.service_id
        LEFT JOIN ActionType ON ActionType.id = Action.actionType_id
        LEFT JOIN rbPayRefuseType ON rbPayRefuseType.id = Account_Item.refuseType_id
        LEFT JOIN OrgStructure ON OrgStructure.id = Person.orgStructure_id
        LEFT JOIN rbService AS rbItemService ON rbItemService.id = Account_Item.service_id
        LEFT JOIN rbSpeciality ON Person.speciality_id = rbSpeciality.id
        LEFT JOIN Organisation AS RelegateOrg ON Event.org_id = RelegateOrg.id
        LEFT JOIN Organisation AS ActionOrg ON Action.org_id = ActionOrg.id
        LEFT JOIN rbMedicalAidType ON EventType.medicalAidType_id =rbMedicalAidType.id
        LEFT JOIN rbMedicalAidKind ON EventType.medicalAidKind_id =rbMedicalAidKind.id
        LEFT JOIN rbMedicalAidKind AS ItemMedicalAidKind ON rbItemService.medicalAidKind_id = ItemMedicalAidKind.id
        LEFT JOIN rbMedicalAidProfile AS EventMedicalAidProfile ON
            EventMedicalAidProfile.id = rbEventService.medicalAidProfile_id
        LEFT JOIN rbMedicalAidProfile AS VisitMedicalAidProfile ON
            VisitMedicalAidProfile.id = rbVisitService.medicalAidProfile_id
        left join rbService_Profile on rbService_Profile.id = (select max(id) from rbService_Profile rs where rs.master_id = rbItemService.id and rs.speciality_id = Person.speciality_id)
        LEFT JOIN rbMedicalAidProfile AS ItemMedicalAidProfile ON
            ItemMedicalAidProfile.id = IFNULL(rbService_Profile.medicalAidProfile_id, rbItemService.medicalAidProfile_id)
        LEFT JOIN Action AS HospitalAction ON
            HospitalAction.id = (
                SELECT MAX(A.id)
                FROM Action A
                WHERE A.event_id = Event.id AND
                          A.deleted = 0 AND
                          A.actionType_id IN (
                                SELECT AT.id
                                FROM ActionType AT
                                WHERE AT.flatCode ='moving'
                                    AND AT.deleted = 0
                          )
            )
        LEFT JOIN Action AS HospitalDirectionAction ON
            HospitalDirectionAction.id = (
                SELECT MAX(A1.id)
                FROM Action A1
                WHERE A1.event_id = Event.id AND
                          A1.deleted = 0 AND
                          A1.actionType_id IN (
                                SELECT AT1.id
                                FROM ActionType AT1
                                WHERE AT1.flatCode ='hospitalDirection'
                                    AND AT1.deleted = 0
                          )
            )
        LEFT JOIN ClientSocStatus AS ClientCitizenshipStatus ON ClientCitizenshipStatus.id= (
            SELECT MAX(CSS.id)
            FROM ClientSocStatus AS CSS
            WHERE CSS.client_id = `Client`.id AND
            CSS.deleted = 0 AND CSS.socStatusClass_id = (
                SELECT rbSSC.id
                FROM rbSocStatusClass AS rbSSC
                WHERE rbSSC.code = '8' AND rbSSC.group_id IS NULL
                LIMIT 0,1
            )
        )
        LEFT JOIN rbSocStatusType AS Citizenship ON
            ClientCitizenshipStatus.socStatusType_id = Citizenship.id
        LEFT JOIN ClientSocStatus ON ClientSocStatus.id= (
            SELECT MAX(CSS2.id)
            FROM ClientSocStatus AS CSS2
            WHERE CSS2.client_id = `Client`.id AND
            CSS2.deleted = 0 AND CSS2.socStatusClass_id = (
                SELECT rbSSC2.id
                FROM rbSocStatusClass AS rbSSC2
                WHERE rbSSC2.code = '9' AND rbSSC2.group_id IS NULL
                LIMIT 0,1
            )
        )
        LEFT JOIN Contract_CompositionExpense AS BaseExpense ON  BaseExpense.id = (
            SELECT MAX(CCE.id)
            FROM Contract_CompositionExpense AS CCE
            LEFT JOIN rbExpenseServiceItem ON CCE.rbTable_id =rbExpenseServiceItem.id
            WHERE rbExpenseServiceItem.code = '1'
        )
        LEFT JOIN Contract_CompositionExpense AS AdditionalExpense ON  AdditionalExpense.id = (
            SELECT MAX(CCE2.id)
            FROM Contract_CompositionExpense AS CCE2
            LEFT JOIN rbExpenseServiceItem ON CCE2.rbTable_id =rbExpenseServiceItem.id
            WHERE rbExpenseServiceItem.code = '2'
        )
        LEFT JOIN rbSocStatusType AS SocStatus ON
            ClientSocStatus.socStatusType_id = SocStatus.id
        LEFT JOIN Contract_Tariff ON Account_Item.tariff_id = Contract_Tariff.id
        LEFT JOIN rbMedicalAidUnit ON Contract_Tariff.unit_id = rbMedicalAidUnit.id
        LEFT JOIN Account ON Account_Item.master_id = Account.id
        LEFT JOIN Contract ON Account.contract_id = Contract.id
        LEFT JOIN rbMesSpecification ON rbMesSpecification.id = Event.mesSpecification_id
        WHERE
            Account_Item.reexposeItem_id IS NULL
        AND (Account_Item.date IS NULL
             OR (Account_Item.date IS NOT NULL AND rbPayRefuseType.rerun != 0)
            )
        AND %s
        ORDER BY insurerName""" % cond

        return self.db.query(stmt)

# *****************************************************************************************

    def createPersonQuery(self):
        table = self.db.table('Person')
        orgStructId = QtGui.qApp.currentOrgStructureId()
        cond = table['orgStructure_id'].eq(orgStructId) if orgStructId\
            else table['org_id'].eq(QtGui.qApp.currentOrgId())

        stmt = """SELECT
            lastName, firstName, patrName, Person.sex, birthDate,
            Person.code AS tabNum,
            OrgStructure.infisCode AS orgStructCode,
            rbPost.regionalCode AS postCode,
            rbSpeciality.regionalCode AS specialityCode
        FROM Person
        LEFT JOIN OrgStructure ON Person.orgStructure_id = OrgStructure.id
        LEFT JOIN rbSpeciality ON Person.speciality_id = rbSpeciality.id
        LEFT JOIN rbPost ON Person.post_id = rbPost.id
        WHERE %s /*AND LEFT(rbPost.code,1) IN ('1','2','3')*/
        """ % cond
        return self.db.query(stmt)

# *****************************************************************************************

    def getClientInfo(self):
        u"""возвращает общую стоимость услуг за пациента"""

        QtGui.qApp.processEvents()
        tableAccountItem = self.db.table('Account_Item')

        cond = tableAccountItem['master_id'].inlist(self.selectedAccountIds) \
            if self.exportType == self.exportTypeFLK \
            else tableAccountItem['id'].inlist(self.idList)

        stmt = """SELECT Event.client_id, SUM(sum) AS totalSum,
                MIN(Event.setDate) AS begDate,
                MAX(Event.execDate) AS endDate,
                SUM(LEAST(IF(Contract_Tariff.federalLimitation = 0,
                    Account_Item.amount,
                    LEAST(Contract_Tariff.federalLimitation, Account_Item.amount)) * Contract_Tariff.federalPrice,
                    Account_Item.sum)) AS federalSum,
                Account_Item.master_id AS accountId
            FROM Account_Item
            LEFT JOIN rbPayRefuseType ON rbPayRefuseType.id = Account_Item.refuseType_id
            LEFT JOIN Event ON Account_Item.event_id = Event.id
            LEFT JOIN Contract_Tariff ON Account_Item.tariff_id = Contract_Tariff.id
            WHERE Account_Item.reexposeItem_id IS NULL
                AND (Account_Item.date IS NULL
                     OR (Account_Item.date IS NOT NULL AND rbPayRefuseType.rerun != 0)
                    )
                AND %s
            GROUP BY client_id;""" % cond

        query = self.db.query(stmt)
        result = {}

        while query.next():
            record  = query.record()
            accountId = forceRef(record.value('accountId'))
            clientId = forceRef(record.value(0))
            key = (accountId, clientId)
            result[key] = (
                forceDouble(record.value(1)),
                forceDate(record.value(2)),
                forceDate(record.value(3)),
                forceDouble(record.value(4))
            )

        return result

# *****************************************************************************************

    def createExpenseCache(self, contractId):
        expenseCache = {}
        self.expenseTARU = forceRef(self.db.translate('rbExpenseServiceItem', 'code','230001', 'id'))
        self.expenseTARU_B= forceRef(self.db.translate('rbExpenseServiceItem', 'code','230002', 'id'))
        self.expenseTARU_D = forceRef(self.db.translate('rbExpenseServiceItem', 'code','230003', 'id'))
        self.expenseTARU_DM = forceRef(self.db.translate('rbExpenseServiceItem', 'code','230004', 'id'))
        self.expenseTARU_UC = forceRef(self.db.translate('rbExpenseServiceItem', 'code','230005', 'id'))

        tableTariff = self.db.table('Contract_Tariff')
        tariffIdList = self.db.getIdList(tableTariff, where=tableTariff['master_id'].eq(contractId))

        table = self.db.table('Contract_CompositionExpense')
        cond = []
        cond.append(table['master_id'].inlist(tariffIdList))
        cond.append(table['rbTable_id'].inlist([self.expenseTARU, self.expenseTARU_B,
                                                self.expenseTARU_D, self.expenseTARU_D,
                                                self.expenseTARU_DM, self.expenseTARU_UC]))
        recordList = self.db.getRecordList(table, where=cond, order='master_id')

        for record in recordList:
            tariffId = forceRef(record.value('master_id'))

            if not expenseCache .has_key(tariffId):
                expenseCache[tariffId] = {}

            expenseId = forceRef(record.value('rbTable_id'))
            expenseCache[tariffId][expenseId] = forceDouble(record.value('sum'))

        return expenseCache

# *****************************************************************************************

    def getTariffCoefficient(self, coefficientId, contractId, date):
        table = self.db.table('Contract_Coefficient')
        cond = [  table['coefficientType_id'].eq(coefficientId),
                  table['master_id'].eq(contractId),
                  table['begDate'].le(date),
                  table['deleted'].eq(0),
               ]
        record = self.db.getRecordEx(table, 'value', where=cond, order='begDate DESC')
        result = forceDouble(record.value(0)) if record else 0.0
        return result

# *****************************************************************************************

    def process(self, dbf, record, params):
        birthDate = forceDate(record.value('birthDate'))
        begDate = forceDate(record.value('begDate'))
        endDate = forceDate(record.value('endDate'))
        # Номер стат.талона
        clientId = forceRef(record.value('client_id'))
        eventId = forceRef(record.value('event_id'))
        actionId = forceRef(record.value('action_id'))
        visitId = forceRef(record.value('visit_id'))
        accountItemId = forceRef(record.value('accountItem_id'))
        number = forceString(record.value('regNumber'))
        corpus = forceString(record.value('regCorpus'))
        flat = forceString(record.value('regFlat'))
        hospitalActionId =  forceRef(record.value('hospitalActionId'))
        hospitalDirectionActionId = forceRef(record.value('hospitalDirectionActionId'))
        hospitalFlag = hospitalActionId is not None
        medicalAidTypeCode = forceString(record.value('medicalAidTypeCode'))
        setOrgCode = forceString(record.value('outOrgCode'))
        isExternalService = ((setOrgCode != '') and (setOrgCode != params['codeLpu']))
        insurerArea = forceString(record.value('insurerArea'))
        isAlien = insurerArea[:2] != QtGui.qApp.defaultKLADR()[:2]
        federalPrice = forceDouble(record.value('federalPrice'))
        isModernClient = ((federalPrice != 0.0) and (self.exportType == self.exportTypeP10))
        tariffId = forceRef(record.value('tariffId'))
        accountId = forceRef(record.value('accountId'))

        if isModernClient:
            (dbfPM, dbfUM, dbfBM,  dbfDM, dbfSM, dbfKM, dbfLM, dbfP, dbfU, dbfB,  dbfD, dbfS, dbfK, dbfL) = dbf
        else:
            if self.exportType == self.exportTypeP13:
                (dbfP, dbfU, dbfD, dbfS, dbfN) = dbf
            elif self.exportType == self.exportTypeP12:
                 (dbfP, dbfU, dbfD, dbfL, dbfN) = dbf
            else:
                (dbfP, dbfU, dbfB,  dbfD, dbfS, dbfK, dbfL, dbfPM, dbfUM, dbfBM,  dbfDM, dbfSM, dbfKM, dbfLM) = dbf

        if (clientId, isModernClient) not in self.exportedClients:
            dbfRecord = dbfP.newRecord()
            if self.exportType != self.exportTypeFLK:
                # номер реестра счетов (п. 1 примечаний) обязательное
                dbfRecord['NS'] = self.edtRegistryNumber.value()
            else:# Для ФЛК берем номера из счетов, если их возможно преобразовать в число, если невозможно то берем от основного
                try:
                    dbfRecord['NS'] = forceInt(record.value('accountNumber'))
                except:
                     dbfRecord['NS'] = self.edtRegistryNumber.value()
                     self.log(u'Невозможно привести номер счёта  "%s" к числовому виду' % forceString(record.value('accountNumber')))
            # тип реестра счетов (п. 2;4 примечаний) обязательное SPR21
            dbfRecord['VS'] = params['accType']
            # дата формирования реестра счетов (п. 3 примечаний) обязательное
            if self.exportType == self.exportTypeP13:
                dbfRecord['DATS'] = pyDate(params.get('exposeDate', QDate.currentDate()))
            else:
                personalDate = params.get('personalDate', QDate())
                dbfRecord['DATS'] = pyDate(personalDate if personalDate else QDate.currentDate())
            # номер персонального счета обязательное
            dbfRecord['SN'] = clientId

            # код медицинской организации в системе ОМС,
            #предоставившей медицинскую помощь обязательное SPR01
            dbfRecord['CODE_MO'] = params['codeLpu']
            # ОГРН плательщика (п. 4 примечаний) обязательное SPR02
            if self.exportType != self.exportTypeFLK:
                dbfRecord['PL_OGRN'] = params['payerOGRN'] if isAlien else forceString(record.value('insurerOGRN')) # 'C', 15),

            if self.exportType == self.exportTypeP10:
                # код метода оплаты (п. 4 примечаний) обязательное SPR17
                dbfRecord['KSO'] = params['payMethod']

            #  фамилия (п. 5 примечаний) обязательное
            dbfRecord['FIO'] = nameCase(forceString(record.value('lastName')))
            # имя (п. 5 примечаний) обязательное
            dbfRecord['IMA'] = nameCase(forceString(record.value('firstName')))
            # отчество (п. 5 примечаний)
            patrName = forceString(record.value('patrName'))
            dbfRecord['OTCH'] = nameCase(patrName)
            # пол (М/Ж) (п. 6 примечаний) обязательное
            dbfRecord['POL']=formatSex(record.value('sex')).upper()
            # дата рождения (п. 7 примечаний) обязательное
            dbfRecord['DATR'] = pyDate(birthDate)
            # код ОКАТО территории постоянной регистрации пациента обязательное для инокраевых SPR39
            if self.exportType not in (self.exportTypeP12, self.exportTypeP13, self.exportTypeFLK):
                dbfRecord['C_OKATO2'] = forceString(record.value('regionOKATO')) # if isAlien else '' # 'C', 5),
            # категория граждан обязательное SPR09
            # 1 работающий\ 0 не работающий
            workOrgName = forceString(record.value('workName'))
            # СНИЛС
            dbfRecord['SNILS'] = forceString(record.value('SNILS'))
            # статус пациента обязательное SPR40
            # студент/учащийся ?
            age = calcAgeInYears(birthDate, endDate)
            socStatusCode = forceInt(record.value('socStatusCode')) % 100

            if self.exportType != self.exportTypeFLK:
                if workOrgName:
                    if self.exportType == self.exportTypeP10:
                        dbfRecord['STAT_Z'] = 5 # 'N', 2, 0),
                    dbfRecord['KAT'] =  '1'
                else:
                    dbfRecord['KAT'] = '0'
                    if self.exportType == self.exportTypeP10:
                        dbfRecord['STAT_Z'] = 5 # 'N', 2, 0),

            if self.exportType == self.exportTypeP10 and socStatusCode:
                dbfRecord['STAT_Z'] = socStatusCode # 'N', 2, 0),
                #dbfRecord['KAT'] = '0'

            # код ОКАТО территории страхования по ОМС обязательное для инокраевых SPR39
            dbfRecord['OKATO_OMS'] = forceString(record.value('insurerOKATO'))

            if dbfRecord['OKATO_OMS'] == '':
                insurerName = forceString(record.value('insurerName'))
                self.log(u'<b><font color=orange>Внимание<\font><\b>:'\
                    u' ОКАТО для ОМС "%s" не задан, пытаюсь определить по области страхования!' % insurerName)

                dbfRecord['OKATO_OMS'] = forceString(QtGui.qApp.db.translate('kladr.KLADR', 'CODE', insurerArea, 'OCATD'))

                if dbfRecord['OKATO_OMS'] == '':
                    self.log(u'<b><font color=red>Внимание<\font><\b>:'\
                        u' ОКАТО для ОМС "%s" не задан!' % forceString(record.value('insurerName')))

            if isAlien and (self.exportType == self.exportTypeP11):
                dbfRecord['SMOCODE'] = forceString(record.value('policyInsurer'))

            # ОГРН СМО, выдавшей полис ОМС обязательное для инокраевых СМО SPR33
            if self.exportType not in (self.exportTypeP12, self.exportTypeP13, self.exportTypeFLK):
                dbfRecord['Q_OGRN'] = forceString(record.value('insurerOGRN'))
            # серия полиса ОМС обязательное
            dbfRecord['SPS'] = forceString(record.value('policySerial'))
            # номер полиса ОМС обязательное
            dbfRecord['SPN'] = forceString(record.value('policyNumber'))

            if self.exportType == self.exportTypeP10:
                # дата начала действия полиса ОМС (п. 8 примечаний) обязательное для инокраевых
                dbfRecord['DATNSP'] =pyDate(forceDate(record.value('policyBegDate')))
                # дата окончания действия полиса ОМС (п. 8 примечаний) обязательное для инокраевых
                dbfRecord['DATOSP'] = pyDate(forceDate(record.value('policyEndDate')))

            if  self.exportType in (self.exportTypeP11,self.exportTypeFLK, self.exportTypeP12, self.exportTypeP13):
                dbfRecord['NOVOR'] =  '' if age > 0 else  '%s%s0' % (forceInt(record.value('sex')),\
                                                               birthDate.toString('ddMMyy'))

            # признак "Особый случай" при регистрации обращения
            # за медицинской помощью (п. 9 примечаний) SPR42
            flags = ''
            if dbfRecord['SPS'] == '' and dbfRecord['SPN'] == '':
                flags += '1'
            if age < 1:
                flags += '2'
            if patrName == '':
                flags += '4'
            dbfRecord['Q_G'] = trim(flags) + forceString(record.value('Q_G'))

            # код типа документа, удостоверяющего личность пациента (представителя)
            # (п. 10 примечаний) SPR43
            dbfRecord['C_DOC'] = forceInt(record.value('documentRegionalCode')) % 100
            # серия документа, удостоверяющего личность пациента (представителя) (п. 10 примечаний)
            dbfRecord['S_DOC'] = forceString(record.value('documentSerial'))
            # номер документа, удостоверяющего личность пациента (представителя) (п. 10 примечаний)
            dbfRecord['N_DOC'] = forceString(record.value('documentNumber'))

            representativeInfo = self.getClientRepresentativeInfo(clientId)

            dbfRecord['STAT_P'] = '0' if isAlien else ''

            if representativeInfo != {}:
                # код типа документа, удостоверяющего личность пациента (представителя)
                # (п. 10 примечаний) SPR43
                dbfRecord['C_DOC'] = representativeInfo.get('documentTypeRegionalCode', 18) % 100
                # серия документа, удостоверяющего личность пациента (представителя) (п. 10 примечаний)
                dbfRecord['S_DOC'] = representativeInfo.get('serial', '')
                # номер документа, удостоверяющего личность пациента (представителя) (п. 10 примечаний)
                dbfRecord['N_DOC'] = representativeInfo.get('number', '')

                if self.exportType in (self.exportTypeP12, self.exportTypeP13, self.exportTypeFLK):
                    dbfRecord['POLP'] = formatSex(representativeInfo.get('sex', '')).upper()
                    dbfRecord['DATRP'] = pyDate(representativeInfo.get('birthDate', QDate()))

            # наименование района по месту регистрации пациента (п. 11 примечаний)
            # обязательное, в случае отсутствия района “-”
            if self.exportType not in (self.exportTypeFLK, self.exportTypeP12, self.exportTypeP13):
                KLADRCode = forceString(record.value('regKLADRCode'))
                dbfRecord['R_NAME'] = self.getRegionName(KLADRCode) if KLADRCode[2:5] != '000' else '-'
                # код вида населенного пункта (п. 12 примечаний) обязательное SPR44
                try:
                    dbfRecord['Q_NP'] = forceInt(record.value('placeCode'))
                except:
                    self.log(u'Невозможно привести инфис код населенного пункта для' \
                           u' "%s" к числовому виду, clientId=%d' %\
                           (forceString(record.value('placeCode')), clientId))

                # наименование населенного пункта по месту  регистрации (п. 11 примечаний) обязательное
                dbfRecord['NP_NAME'] = forceString(record.value('cityName'))
                # код типа наименования улицы (п. 12 примечаний)     SPR45
                try:
                    dbfRecord['Q_UL'] = forceInt(record.value('streetType'))
                except:
                    self.log(u'Невозможно привести инфис код типа наименования'\
                            u' улицы "%s" к числовому виду, clientId=%d' % \
                            (forceString(record.value('streetType')), clientId))
                # наименование улицы (п. 11 примечаний)
                dbfRecord['UL_NAME'] = forceString(record.value('streetName'))
                # дом (п. 13 примечаний) обязательное
                dbfRecord['DOM'] = number
                # корпус/строение (п. 13 примечаний)
                dbfRecord['KOR'] = corpus
                # квартира/комната (п. 13 примечаний)
                dbfRecord['KV'] = flat
                # код страховщика (Краснодарского края) обязательное для краевых счетов SPR03

            self.lastInsurerId =  forceRef(record.value('insurerId'))
            dbfRecord['KSMO'] = forceString(record.value('policyInsurer'))

            if self.exportType == self.exportTypeP10:
                citizenshipCode = forceString(record.value('citizenshipCode'))
                # код страны проживания (гражданство) обязательное SPR05
                dbfRecord['STRANA'] = citizenshipCode if citizenshipCode else '643'
                # место работы/учебы пациента (п. 14 примечаний)
                # обязательное в соответсвии со справочником SPR40
                dbfRecord['MRAB'] = workOrgName

            # код направившей медицинской организации в системе
            # ОМС обязательное для внешних услуг    SPR01
            if isExternalService  and (self.exportType != self.exportTypeFLK):
                dbfRecord['NAPR_MO'] = setOrgCode

            if self.exportType in (self.exportTypeP12, self.exportTypeP13) and hospitalDirectionActionId:
                directionAction = CAction.getActionById(hospitalDirectionActionId)
                dbfRecord['NAPR_N'] = '%s_%s' % (params['codeLpu'],  directionAction ['Номер'])

            # номер амбулаторной карты или истории болезни обязательное кроме внешних  услуг
            if (not isExternalService) and (self.exportType != self.exportTypeFLK):
                dbfRecord['ISTI'] = forceString(clientId) # 'C', 10),
            (clientSum, clientBegDate, clientEndDate, clientFederalSum) = self.clientInfo[(accountId, clientId)]
            # дата начала лечения обязательное
            dbfRecord['DATN'] = pyDate(clientBegDate)
            # дата окончания лечения обязательное
            dbfRecord['DATO'] = pyDate(clientEndDate)
            #дата формирования персонального счета обязательное
            dbfRecord['DATPS'] = pyDate(clientEndDate)
            # код исхода заболевания обязательное SPR11
            if (self.exportType != self.exportTypeFLK):
                dbfRecord['ISHL'] = forceString(record.value('diagnosticResultCode'))
                # код исхода обращения обязательное SPR12
                dbfRecord['ISHOB'] = forceString(record.value('eventResultCode'))
                # код вида обращения обязательное SPR14
                dbfRecord['MP'] = '2' if forceInt(record.value('order')) == 2 else '1'

            if self.exportType == self.exportTypeP10:
                # признак: счет за внешнюю медицинскую услугу (п. 15 примечаний) обязательное
                dbfRecord['IS_VNESH'] = 1 if isExternalService else 0

                if isExternalService:
                    # номер направления (п. 16 примечаний) обязательное для внешних услуг
                    dbfRecord['NAPR_N'] = forceString(record.value('actNote'))
                    # дата направления обязательное для внешних услуг
                    actSetDate = forceDate(record.value('actSetDate'))
                    if actSetDate:
                        dbfRecord['NAPR_D'] = pyDate(actSetDate)
                    else:
                        self.log(u'Для направления %s не задана дата.' % dbfRecord['NAPR_N'])
                        if not self.ignoreErrors:
                            return

            if (self.exportType != self.exportTypeFLK):
                # сумма к оплате по случаю заболевания пациента обязательное
                dbfRecord['SUMMA_I'] = clientSum

            if self.exportType == self.exportTypeP10:
                dbfRecord['SUMMA_M'] = clientFederalSum

            #dbfRecord['PV', 'C', 40), # коды причин возврата (п. 17 примечаний) обязательное для возвратных счетов SPR15
            #dbfRecord['DVOZVRAT', 'D'), # дата возврата обязательное для возвратных счетов
            dbfRecord['SPV'] = forceInt(record.value('policyKindCode')) % 10
            dbfRecord.store()
            self.exportedClients.add((clientId, isModernClient))

        if  self.exportType not in (self.exportTypeFLK,  self.exportTypeP12, self.exportTypeP13):
            tempInvalidList = self.getClientTempInvalidList(clientId, begDate,
                                        endDate, forceRef(record.value('person_id')))

            for (tiBegDate, tiEndDate, tiSerial, tiNumber, tiId, tiReasonCode, tiMKB) in tempInvalidList:
                if tiId not in self.exportedTempInvalidList:
                    dbfRecord = dbfB.newRecord()
                    # код медицинской организации в системе ОМС,
                    # предоставившей медицинскую помощь обязательное SPR01
                    dbfRecord['CODE_MO'] = params['codeLpu']
                    # номер реестра счетов обязательное
                    dbfRecord['NS'] = self.edtRegistryNumber.value()
                    # номер персонального счета обязательное
                    dbfRecord['SN'] = accountItemId
                    # серия листка нетрудоспособности обязательное
                    dbfRecord['BLS'] = tiSerial #, 'C', 10),
                    # номер листка нетрудоспособности обязательное
                    dbfRecord['BLN'] = tiNumber #'N', 7, 0),
                    # причина нетрудоспособности обязательное SPR27
                    dbfRecord['KPBL'] = tiReasonCode #'N', 2, 0),
                    # дата выписки б.листа обязательное
                    dbfRecord['DATN'] = pyDate(tiBegDate)
                    # дата закрытия б.листа обязательное
                    dbfRecord['DATO'] = pyDate(tiEndDate)
                    # код диагноза по МКБ–Х SPR20
                    dbfRecord['MKBX'] = tiMKB
                    dbfRecord.store()
                    self.exportedTempInvalidList.append(tiId)

        if self.exportType != self.exportTypeFLK:
            serviceCode = forceString(record.value('service'))
            mesNotUsed = forceBool(record.value('mesNotUsedFlag'))

            if (self.exportType in (self.exportTypeP11, self.exportTypeP12, self.exportTypeP13) and
                    mesNotUsed and
                    serviceCode[:1] =='S' and
                    not actionId and
                    not visitId):
                # для событий у которых в "Особенностях выполнения МЭС"
                # указано "МЭС не применен" не нужно выгружать строку с МЭСом
                return

            dbfRecord = dbfU.newRecord()

            if self.exportType in (self.exportTypeP11,  self.exportTypeP12, self.exportTypeP13):
                dbfRecord['UID'] = accountItemId

            # код медицинской организации в системе ОМС,
            # предоставившей медицинскую помощь обязательное SPR01
            dbfRecord['CODE_MO'] = params['codeLpu']
            # номер реестра счетов (п. 1 примечаний) обязательное
            dbfRecord['NS'] = self.edtRegistryNumber.value()
            # номер персонального счета обязательное
            dbfRecord['SN'] = clientId
            # код отделения (п. 2 примечаний) обязательное SPR07
            dbfRecord['KOTD'] = forceString(record.value('orgStructCode'))
            # код профиля койки (п. 2 примечаний) обязательное для стационаров SPR08
            if self.exportType in (self.exportTypeP11, self.exportTypeP12, self.exportTypeP13):
                # поле KPK = региональному коду справочника "профили коек",
                # койки указанной в действии "движение". если в движении
                # койка не указана = пусто. если нет действия движение = пусто.
                if hospitalActionId:
                    action = CAction.getActionById(hospitalActionId)
                    bedCode = self.getHospitalBedProfileRegionalCode(self.getHospitalBedProfileId(forceRef(action[u'койка'])))

                    if bedCode:
                        dbfRecord['KPK'] = bedCode

            else:
                if hospitalFlag:
                    dbfRecord['KPK'] = '00'

                if medicalAidTypeCode == '7':
                    dbfRecord['KPK'] = '40'

            # код диагноза основного заболевания по МКБ–Х (п. 2 примечаний)
            # обязательное для всех услуг, кроме диагностических SPR20
            dbfRecord['MKBX'] = forceString(record.value('MKB'))
            # код диагноза сопутствующего заболевания по МКБ–Х (п. 2 примечаний) SPR20
            dbfRecord['MKBXS'] = forceString(record.value('AssociatedMKB'))
            # код стандарта оказания медицинской помощи обязательное
            # для нозологий, у которых разработан СОМП SPR38
            mesId = forceRef(record.value('mesId'))
            if mesId and not (serviceCode[:1] in ('A', 'B')) and self.exportType != self.exportTypeP13:
                dbfRecord['KSTAND'] = forceString(QtGui.qApp.db.translate('mes.MES', 'id', mesId, 'code'))
            # код условия оказания медицинской помощи (п. 2, 3 примечаний) обязательное SPR13
            medicalAidTypeId = self.getServiceMedicalAidTypeId(forceRef(record.value('serviceId')))
            serviceMedicalAidTypeCode = self.getMedicalAidTypeRegionalCode(medicalAidTypeId)

            dbfRecord['VP'] = serviceMedicalAidTypeCode if serviceMedicalAidTypeCode\
                                else forceString(record.value('medicalAidTypeRegionalCode'))
            # код медицинской услуги обязательное SPR18
            dbfRecord['KUSL'] = serviceCode
            # количество услуг обязательное
            amount = forceInt(record.value('amount'))
            dbfRecord['KOLU'] = amount

            # для услуг инфис код которых начинается с S (МЭСы) поле KOLU = 1
            if self.exportType == self.exportTypeP11 and\
                    serviceCode[:1] == 'S':
                dbfRecord['KOLU'] = 1

            if self.exportType == self.exportTypeP13:
                if serviceCode[:1] not in ('A', 'B'):
                    # для услуг инфис код которых начинается на A, B (латинские)
                    # поле KD должно быть пусто
                    dayCount = forceInt(record.value('dayCount'))
                    dbfRecord['KD'] = dayCount if dayCount > 1 else 1
            elif self.exportType in (self.exportTypeP11, self.exportTypeP12):
                if  serviceCode[:1] in ('A', 'B'):
                    # для услуг инфис код которых начинается на
                    # A, B (латинские) поле KD должно быть пусто
                    pass
                elif serviceCode[:1] in ('K','S'):
                    dbfRecord['KD'] = amount
            else:
                # количество койко-дней обязательное  для стационаров
                if hospitalFlag or medicalAidTypeCode == '7':
                    dbfRecord['KD'] = amount

            # дата начала выполнения услуги обязательное
            servDate = forceDate(record.value('actionDate'))

            if not servDate:
                servDate = forceDate(record.value('visitDate'))

                if not servDate:
                    servDate = begDate

            if not servDate:
                self.log(u'Не задана дата услуги: accountItemId=%d,'\
                    u' eventId=%d.' % (accountItemId, eventId))

            dbfRecord['DATN'] = pyDate(servDate)
            # дата окончания выполнения услуги обязательное
            # В случае МЭСА и его успешного выполнения дата окончаня для мэса будет Дата+Длительность мэса,
            # в случае если полученная дата меньше даты события целиком
            if (self.exportType in (self.exportTypeP11, self.exportTypeP12, self.exportTypeP13)) and (not mesNotUsed) and (serviceCode[:1] =='S'):
                mesDuration = forceInt(QtGui.qApp.db.translate('mes.MES', 'id', mesId, 'maxDuration'))
                tmpDate = servDate
                tmpDate = tmpDate.addDays(mesDuration)

                if tmpDate <= endDate:
                    endDate=tmpDate

            if  serviceCode[:1] in ('A', 'B'):
                endDate = servDate
            dbfRecord['DATO'] = pyDate(endDate)
            # Тариф (см.п.6 примечаний) обязательное    SPR22
            price = forceDouble(record.value('price'))
            dbfRecord['TARU'] = price #'N', 12, 2),
            # сумма к оплате обязательное
            dbfRecord['SUMM'] = forceDouble(record.value('sum'))
            dbfRecord['UET'] = forceDouble(record.value('UET'))
            baseExpensePercent = forceDouble(record.value('baseExpensePercent'))
            additionalExpensePercent = forceDouble(record.value('additionalExpensePercent'))
            regulator = forceDouble(record.value('regulator'))

            if self.exportType != self.exportTypeP13:
                dbfRecord['TARU_B'] = round(price * baseExpensePercent/100, 2)
                dbfRecord['TARU_D'] = round(price * additionalExpensePercent/100, 2)
                dbfRecord['SUMM_B'] = round(price * baseExpensePercent/100,2)*amount
                dbfRecord['SUMM_D'] = round(price * additionalExpensePercent/100,2)*amount
                dbfRecord['SUMM_K'] = round(price * baseExpensePercent/100 *regulator,2)*amount

            if self.exportType in (self.exportTypeP11, self.exportTypeP12):
                kolu=dbfRecord['KOLU']
                expenseCache = params['expenseCache']
                dbfRecord['TARU']   = expenseCache.get(tariffId, {}).get(self.expenseTARU, 0.0)
                dbfRecord['TARU_B']  = expenseCache.get(tariffId, {}).get(self.expenseTARU_B, 0.0)
                dbfRecord['TARU_D']  = expenseCache.get(tariffId, {}).get(self.expenseTARU_D, 0.0)
                dbfRecord['TARU_DM'] = expenseCache.get(tariffId, {}).get(self.expenseTARU_DM, 0.0)
                dbfRecord['TARU_UC'] = expenseCache.get(tariffId, {}).get(self.expenseTARU_UC, 0.0)

                getcontext().prec=6
                dbfRecord['SUMM'] = dbfRecord['TARU']*kolu
                dbfRecord['SUMM_B']  = dbfRecord['TARU_B']*kolu
                dbfRecord['SUMM_D']  = dbfRecord['TARU_D']*kolu
                dbfRecord['SUMM_DM'] = dbfRecord['TARU_DM']*kolu
                dbfRecord['SUMM_UC'] = Decimal(str(dbfRecord['TARU_B'] *params['summUCMultiplier'])).quantize(Decimal('0.01'),rounding=ROUND_HALF_UP) *dbfRecord['KOLU']
                dbfRecord['SUMM_K']  = Decimal(str(dbfRecord['TARU_B'] *params['summKMultiplier'])).quantize(Decimal('0.01'),rounding=ROUND_HALF_UP) *dbfRecord['KOLU']

            if self.exportType == self.exportTypeP10:
                dbfRecord['TARU_M'] = forceDouble(record.value('federalPrice')) #'N', 12, 2),
                # сумма к оплате обязательное
                dbfRecord['SUMM_M'] = forceDouble(record.value('federalSum'))

            # признак: услуга оказана в другой медицинской организации (п. 4 примечаний) обязательное
            dbfRecord['IS_OUT'] = 1 if isExternalService else 0
            # код медицинской организации в системе ОМС, оказавшей услугу (п. 5 примечаний) SPR01
            if isExternalService:
                dbfRecord['OUT_MO'] = params['codeLpu']

            dbfRecord['DOC_TABN'] = forceString(record.value('personCode'))
            dbfRecord['SPEC'] = forceString(record.value('specialityCode')).rjust(3, '0')

            if self.exportType in (self.exportTypeP11, self.exportTypeP12, self.exportTypeP13):
                #С (3) профиль оказанной медицинской помощи
                dbfRecord['PROFIL'] = forceString(record.value('medicalAidProfileCode'))
                #С (1)  вид медицинской помощи
                dbfRecord['VMP'] =  forceString(record.value('medicalAidKindCode'))
                #С (2)  способ оплаты медицинской помощи
                medicalAidUnitCode = forceString(record.value('medicalAidUnitCode'))
                if self.exportType == self.exportTypeP13:
                    dbfRecord['KSO'] =  medicalAidUnitCode if medicalAidUnitCode else params['payMethod']
                else:
                    dbfRecord['KSO'] =  medicalAidUnitCode if medicalAidUnitCode and not mesNotUsed else params['payMethod']

            dbfRecord.store()

            if hospitalDirectionActionId and self.exportType in (self.exportTypeP12, self.exportTypeP13):
                directionAction = CAction.getActionById(hospitalDirectionActionId)
                directionOrgCode = self.getOrgInfisCode(forceRef(directionAction['ЛПУ']))

                dbfRecord = dbfN.newRecord()
                # код медицинской организации
                dbfRecord['CODE_MO']  = params['codeLpu']
                # номер направления, (ККККК_ХХХХХХХ)
                dbfRecord['NAPR_N'] = '%s_%s' % (params['codeLpu'],  forceString(directionAction['Номер']))
                # код МО, в которое направлен пациент
                dbfRecord['NAPR_MO'] = directionOrgCode
                # дата направления
                dbfRecord['NAPR_D'] = pyDate(endDate)
                # табельный номер сотрудника, выдавшего направление
                dbfRecord['DOC_TABN'] = forceString(record.value('personCode'))

                dbfRecord.store()

            if self.exportType == self.exportTypeP13 and not params.get('isVersionInfoExported', False):
                params['isVersionInfoExported'] = True
                row = dbfS.newRecord()
                row['CODE_MO'] = params['codeLpu']
                try:
                    row['NS'] = forceInt(record.value('accountNumber'))
                except:
                    row['NS'] = self.edtRegistryNumber.value()
                    self.log(u'Невозможно привести номер счёта  "%s" к числовому виду' % forceString(record.value('accountNumber')))
                row['SOFT'] = '4'
                row.store()

# *****************************************************************************************

    def processPerson(self, dbf, record, params):
        personCode = forceString(record.value('tabNum'))

        if self.exportType == self.exportTypeP13:
            (dbfP, dbfU, dbfD, dbfS, dbfN) = dbf
            items = [(dbfD, None)]
        elif self.exportType == self.exportTypeP12:
            (dbfP, dbfU, dbfD, dbfL, dbfN) = dbf
            items = [(dbfD, None)]
        else:
            (dbfP, dbfU, dbfB,  dbfD, dbfS,  dbfK, dbfL,
                dbfPM, dbfUM, dbfBM,  dbfDM, dbfSM, dbfKM, dbfLM) = dbf
            items = [(dbfD, dbfS)]

            if len(dbfDM) > 0:
                items.append((dbfDM, dbfSM))

        for (d, s) in items:
            row = d.newRecord()
            row['CODE_MO'] = params['codeLpu']
            row['DOC_TABN'] = personCode
            row['FIO'] = forceString(record.value('lastName'))
            row['IMA'] = forceString(record.value('firstName'))
            row['OTCH'] = forceString(record.value('patrName'))
            row['POL'] = self.sexMap.get(forceInt(record.value('sex')))
            row['DATR'] = pyDate(forceDate(record.value('birthDate')))
            row.store()

            if s:
                row = s.newRecord()
                row['CODE_MO'] = params['codeLpu']
                row['DOC_TABN'] = personCode
                row['SPEC'] = forceString(record.value('specialityCode')).rjust(3, '0')
                row['KOTD'] = forceString(record.value('orgStructCode'))
                row['DOLG'] = forceString(record.value('postCode'))
                row.store()

# *****************************************************************************************

    def getClientTempInvalidList(self, clientId, begDate, endDate, personId):
        key = (clientId, begDate, endDate, personId)
        list = self.tempInvalidCache.get(key, [])

        if list == []:
            db = QtGui.qApp.db
            table = db.table('TempInvalid')
            stmt = """
                SELECT  tr.regionalCode AS reasonCode,
                            TempInvalid.begDate,
                            TempInvalid.endDate,
                            TempInvalid.serial,
                            TempInvalid.number,
                            TempInvalid.id,
                            Diagnosis.MKB
                FROM TempInvalid
                LEFT JOIN rbTempInvalidDocument td ON doctype_id = td.id
                LEFT JOIN rbTempInvalidReason tr ON tempInvalidReason_id = tr.id
                LEFT JOIN Diagnosis ON diagnosis_id = Diagnosis.id
                WHERE %s AND td.code = '1'
            """ % db.joinAnd([ table['client_id'].eq(clientId),
                table['deleted'].eq(0),
                table['closed'].eq(1),
                table['person_id'].eq(personId),
                db.joinOr([table['begDate'].isNull(), table['begDate'].ge(begDate)]),
                db.joinOr([table['endDate'].isNull(), table['endDate'].le(endDate)])])

            query = db.query(stmt)
            while query.next():
                record=query.record()
                list.append((forceDate(record.value('begDate')),
                    forceDate(record.value('endDate')),
                    forceString(record.value('serial')),
                    forceInt(record.value('number')),
                    forceRef(record.value('id')),
                    forceInt(record.value('reasonCode')),
                    forceString(record.value('MKB'))
                    ))

            self.tempInvalidCache[key] = list

        return list

# ******************************************************************************

    @pyqtSignature('int')
    def on_cmbExportType_currentIndexChanged(self, index):
        self.chkMakeInvoice.setEnabled(index in (self.exportTypeP11, self.exportTypeP12, self.exportTypeP13))
        self.cmbPayMethod.setEnabled(index in  (self.exportTypeP10, self.exportTypeP11, self.exportTypeP12, self.exportTypeP13))
        self.edtRegistryNumber.setEnabled(index != self.exportTypeFLK)

# ******************************************************************************

class CExportPage2(CAbstractExportPage2):
    def __init__(self, parent):
        CAbstractExportPage2.__init__(self, parent,  'R23NATIVE')


    def saveExportResults(self):
        modern = (self._parent.page1.cmbAccountType.currentIndex() > 3 and
                  self._parent.page1.exportType not in (CExportPage1.exportTypeP12,   CExportPage1.exportTypeP13))
        baseName = self._parent.page1.getDbfBaseName(modern)
        zipFileName = self._parent.page1.getZipFileName(modern,
                self._parent.page1.exportType == CExportPage1.exportTypeFLK)
        zipFilePath = os.path.join(forceStringEx(self._parent.getTmpDir()),
                                                zipFileName)
        zf = ZipFile(zipFilePath, 'w', allowZip64=True)
        exportType = self._parent.page1.exportType

        if exportType != CExportPage1.exportTypeFLK:
            if exportType == CExportPage1.exportTypeP13:
                prefixes = ('P', 'U', 'D', 'S', 'N')
            elif exportType == CExportPage1.exportTypeP12:
                prefixes = ('P', 'U', 'D', 'L', 'N')
            else:
                prefixes = ('P', 'U', 'B', 'S', 'D', 'K', 'L')

            for src in prefixes:
                filePath = os.path.join(forceStringEx(self.getTmpDir()),
                                        os.path.basename(src+baseName))
                zf.write(filePath, src+os.path.basename(self._parent.page1.getDbfBaseName(False)), ZIP_DEFLATED)

            #Добавляем фактуру
            if self._parent.page1.mkInvoice:
                filePath = os.path.join(forceStringEx(self.getTmpDir()),
                                                                os.path.basename("schfakt.html"))
                zf.write(filePath, "schfakt.html", ZIP_DEFLATED)
        else:
            filePath = os.path.join(forceStringEx(self.getTmpDir()),os.path.basename('P'+baseName))
            zf.write(filePath, 'P'+os.path.basename(self._parent.page1.getDbfBaseName(False)), ZIP_DEFLATED)

        zf.close()
        dst = os.path.join(forceStringEx(self.edtDir.text()), zipFileName)
        success, result = QtGui.qApp.call(self, shutil.move, (zipFilePath, dst))
        return success
