# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import os.path
import shutil

from PyQt4 import QtGui
from PyQt4.QtCore import QDate

from library.dbfpy.dbf import Dbf
from library.Utils import forceBool, forceDate, forceInt, forceRef, forceString, forceStringEx, pyDate, toVariant

from Exchange.Export import (CAbstractExportWizard, CAbstractExportPage1, CAbstractExportPage2, CExportHelperMixin)
from Registry.Utils import formatAddress

from Exchange.Ui_ExportR45NativePage1 import Ui_ExportPage1

# ******************************************************************************

def exportR45Native(widget, accountId, accountItemIdList):
    wizard = CExportR45NativeWizard(widget)
    wizard.setAccountId(accountId)
    wizard.setAccountItemsIdList(accountItemIdList)
    wizard.exec_()

# ******************************************************************************

class CExportR45NativeWizard(CAbstractExportWizard):
    def __init__(self, parent = None):
        title = u'Мастер экспорта для Курганской области'
        CAbstractExportWizard.__init__(self, parent, title)
        self.page1 = CExportPage1(self)
        self.page2 = CExportPage2(self)

        self.addPage(self.page1)
        self.addPage(self.page2)


    def getTmpDir(self):
        return CAbstractExportWizard.getTmpDirEx(self, 'R45NATIVE')


# ******************************************************************************

class CExportPage1(CAbstractExportPage1, Ui_ExportPage1, CExportHelperMixin):
    def __init__(self, parent):
        CAbstractExportPage1.__init__(self, parent)
        CExportHelperMixin.__init__(self)

        self.ignoreErrors = forceBool(QtGui.qApp.preferences.appPrefs.get(
            'ExportR45NativeIgnoreErrors', False))
        self.chkVerboseLog.setChecked(forceBool(QtGui.qApp.preferences.appPrefs.get(
            'ExportR45NativeVerboseLog', False)))
        self.chkIgnoreErrors.setChecked(self.ignoreErrors)


    def validatePage(self):
        QtGui.qApp.preferences.appPrefs['ExportR45NativeIgnoreErrors'] = toVariant(self.chkIgnoreErrors.isChecked())
        QtGui.qApp.preferences.appPrefs['ExportR45NativeVerboseLog'] = toVariant(self.chkVerboseLog.isChecked())
        return CAbstractExportPage1.validatePage(self)


    def setExportMode(self, flag):
        self.btnCancel.setEnabled(flag)
        self.btnExport.setEnabled(not flag)
        self.chkIgnoreErrors.setEnabled(not flag)
        self.chkVerboseLog.setEnabled(not flag)


    def getDbfBaseName(self):
        lpuCode = forceString(QtGui.qApp.db.translate(
            'Organisation', 'id', QtGui.qApp.currentOrgId() , 'infisCode'))
        info = self.accountInfo()
        accDate = info.get('accDate', QDate())
        return forceString(lpuCode + (accDate.toString('MMyy') if accDate else u'0000') + u'.DBF')

# ******************************************************************************

    def exportInt(self):
        self.ignoreErrors = self.chkIgnoreErrors.isChecked()
        self.exportedEvents = set()
        params = {}

        lpuId = QtGui.qApp.currentOrgId()
        params['lpuId'] = lpuId
        params['lpuCode'] = forceString(QtGui.qApp.db.translate(
            'Organisation', 'id', lpuId , 'infisCode'))
        self.log(u'ЛПУ: код инфис: "%s".' % params['lpuCode'])

        if not params['lpuCode']:
            self.log(u'<b><font color=red>ОШИБКА<\font><\b>:'
                     u'Для текущего ЛПУ не задан код инфис', True)
            if not self.ignoreErrors:
                return

        params.update(self.accountInfo())
        self.setProcessParams(params)

        CAbstractExportPage1.exportInt(self)


    def createDbf(self):
        dbfName = os.path.join(self.getTmpDir(), self.getDbfBaseName())
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            #Служебная информация:
            ('MDATE', 'D'), #1. Дата заполнения (дата последнего изменения события)
            ('MPERSON', 'C', 40),      #2. Ф.И.О. заполнителя (автор последнего изменения события)
            #Пациент:
            ('PSERIAL', 'C', 16),       # 1. Серия полиса
            ('PNUMBER', 'C', 16),   # 2. № полиса
            ('SNILS', 'C', 20),             # 3. СНИЛС
            ('CLIENTID', 'N',  10),     #4. ID пациента
            ('LASTNAME', 'C', 30),  # 5. Фамилия
            ('FIRSTNAME', 'C', 30),  # 6. Имя
            ('PATRNAME', 'C', 30),  # 7. Отчество
            ('SEX', 'N', 1), # 8. Пол
            ('BIRTHDAY',  'D'),  # 9. Дата рождения
            ('REGADDR',  'C',  200),  # 10. Адрес  регистрации (по KLADR: регион, нас.пункт, улица, дом, корпус, квартира)
            ('KLADR', 'C', 13),  #
            ('KLADRST', 'C', 17),  #
            ('HOUSENUM', 'C', 8),  #
            ('HOUSECORP', 'C', 8),  #
            ('FLAT', 'C', 6),  #
            ('CITY', 'N', 1),  # 11. Житель села/города
            ('BENEFIT', 'C',  8),  # 12. Льгота (выгружаем региональный код типа соц статуса с кодом класса 1-льгота)
            ('SOCSTATUS', 'C',  8),  # Социальный статус (выгружаем региональный код типа соц статуса с кодом класса 9-социальный статус)
            ('DISABLED',  'C',  8),  # Инвалидность (выгружаем региональный код типа соц статуса с кодом класса 10-инвалидность)
            ('INSREGION',  'C',  8),  #15. Территория страхования («регион» страхования СМО из справочника организаций)
            ('INSURER',  'C',  64),  #16. Страховая компания (краткое наименование страховой компании)
            #Событие:
            ('BEGDATE', 'D'),  # 1. Дата начала
            ('PERSON', 'C', 35),  # 2. Ф.И.О. врача
            ('PLACE', 'C', 8),  # 3. Место обслуживания (справочник место выполнения визита, выгружаем код)
            ('PURPOSE', 'C', 8),  # 4. Цель обращения (справочник типов событий, региональный код)
            ('MKB', 'C', 8),  # 5. Основной диагноз (выгружаем заключительный диагноз, если нет заключительного выгружаем основной)
            ('MES', 'C', 64), # Код  МЭС
            ('CHARACTER', 'C', 8),  # Характер заболевания
            ('TRAUMA', 'C', 8),  # Тип травмы
            ('ACCMKB', 'C', 8),  # Сопутствующий диагноз (выгружаем сопутствующий диагноз ответственного врача, если несколько выгружаем последний)
            ('ACCMES', 'C', 64), # Код  МЭС
            ('ACCCHAR', 'C', 8),  # Характер заболевания
            ('RESULT', 'C', 8),  # Результат обращения
            # Больничный лист
            ('TIBEGDATE', 'D'), # Дата открытия
            ('TIENDDATE', 'D'), # Дата закрытия
            ('TIPRIMARY', 'N', 1), # Первичный/продление
            ('TIREASON', 'C', 8),  # Причина выдачи
            # Услуги:
            ('SMKB',  'C', 8), # Ссылка на диагноз (выгружаем диагноз действия)
            ('SDATE', 'D'), # Дата оказания
            ('SPERSON',  'C',  35),  # Ф.И.О. врача
            ('SSPEC',  'C',  8),  # Специальность врача (региональный код)
            ('SASSIST', 'C', 35),  # Ф.И.О. ассистента
            ('SCODE', 'C', 8), # Код услуги
            ('SAMOUNT', 'N', 10) # Количество
            )
        return dbf


    def createQuery(self):
        tableAccountItem = self.db.table('Account_Item')

        stmt = """SELECT    modifyPerson.name AS modifyPersonName,
            Event.modifyDatetime,
            Client.SNILS,
            ClientPolicy.serial AS policySerial,
            ClientPolicy.number AS policyNumber,
            Event.client_id,
            Client.lastName,
            Client.firstName,
            Client.patrName,
            Client.birthDate,
            Client.sex,
            ClientRegAddress.address_id AS addressId,
            RegAddressHouse.KLADRCode,
            RegAddressHouse.KLADRStreetCode,
            RegAddressHouse.number,
            RegAddressHouse.corpus,
            RegAddress.flat,
            Benefit.regionalCode AS benefitCode,
            SocStatus.regionalCode AS socStatusCode,
            Disabled.regionalCode AS disabledCode,
            Insurer.area AS insurerArea,
            Insurer.shortName AS insurerName,
            Event.setDate AS begDate,
            rbResult.federalCode AS resultCode,
            execPerson.name AS execPersonName,
            rbScene.code AS sceneCode,
            EventType.regionalCode AS eventTypeCode,
            Diagnosis.MKB,
            MainMES.code AS mesCode,
            rbTraumaType.code AS traumaTypeCode,
            rbDiseaseCharacter.code AS characterCode,
            AccDiagnosis.MKB AS accMKB,
            AccCharacter.code AS accCharacterCode,
            TempInvalid.begDate AS tempInvalidBegDate,
            TempInvalid.endDate AS tempInvalidEndDate,
            TempInvalid.closed AS tempInvalidClosed,
            rbTempInvalidReason.code AS tempInvalidReason,
            IF(Account_Item.service_id IS NOT NULL,
                Account_Item.service_id,
                IF(Account_Item.visit_id IS NOT NULL, rbVisitService.id, rbEventService.id)
                ) AS serviceId,
            IF(Account_Item.visit_id IS NOT NULL, Visit.person_id,
                IF(Account_Item.action_id IS NOT NULL,
                        IF(Action.person_id IS NOT NULL, Action.person_id, Action.setPerson_id), Event.execPerson_id)
                ) AS servicePersonId,
            Action.assistant_id AS assistantPersonId,
            Action.MKB AS actionMKB,
            Account_Item.serviceDate,
            Account_Item.event_id,
            Account_Item.amount
        FROM Account_Item
            LEFT JOIN Event  ON Event.id  = Account_Item.event_id
            LEFT JOIN vrbPerson AS modifyPerson ON modifyPerson.id = Event.modifyPerson_id
            LEFT JOIN Client ON Client.id = Event.client_id
            LEFT JOIN ClientPolicy ON ClientPolicy.client_id = Client.id AND
                      ClientPolicy.id = (SELECT MAX(CP.id)
                                         FROM   ClientPolicy AS CP
                                         LEFT JOIN rbPolicyType AS CPT ON CPT.id = CP.policyType_id
                                         WHERE  CP.client_id = Client.id AND CP.deleted=0 AND CPT.code IN ('1','2'))
            LEFT JOIN ClientAddress ClientRegAddress ON ClientRegAddress.client_id = Client.id AND
                      ClientRegAddress.id = (SELECT MAX(CRA.id)
                                         FROM   ClientAddress AS CRA
                                         WHERE  CRA.type = 0 AND CRA.client_id = Client.id AND CRA.deleted=0)
            LEFT JOIN Address   RegAddress ON RegAddress.id = ClientRegAddress.address_id
            LEFT JOIN AddressHouse RegAddressHouse ON RegAddressHouse.id = RegAddress.house_id
            LEFT JOIN ClientSocStatus AS ClientBenefitStatus ON
                ClientBenefitStatus.client_id = Client.id AND
                ClientBenefitStatus.deleted = 0 AND
                ClientBenefitStatus.socStatusClass_id = (
                    SELECT rbSSC.id
                    FROM rbSocStatusClass AS rbSSC
                    WHERE rbSSC.code = '1' AND rbSSC.group_id IS NULL
                    LIMIT 0,1
                )
            LEFT JOIN rbSocStatusType AS Benefit ON
                ClientBenefitStatus.socStatusType_id = Benefit.id
            LEFT JOIN ClientSocStatus ON
                ClientSocStatus .client_id = Client.id AND
                ClientSocStatus .deleted = 0 AND
                ClientSocStatus .socStatusClass_id = (
                    SELECT rbSSC2.id
                    FROM rbSocStatusClass AS rbSSC2
                    WHERE rbSSC2.code = '9' AND rbSSC2.group_id IS NULL
                    LIMIT 0,1
                )
            LEFT JOIN rbSocStatusType AS SocStatus ON
                ClientSocStatus.socStatusType_id = SocStatus.id
            LEFT JOIN ClientSocStatus AS ClientDisabledStatus ON
                ClientDisabledStatus.client_id = Client.id AND
                ClientDisabledStatus.deleted = 0 AND
                ClientDisabledStatus.socStatusClass_id = (
                    SELECT rbSSC3.id
                    FROM rbSocStatusClass AS rbSSC3
                    WHERE rbSSC3.code = '10' AND rbSSC3.group_id IS NULL
                    LIMIT 0,1
                )
            LEFT JOIN rbSocStatusType AS Disabled ON
                ClientDisabledStatus.socStatusType_id = Disabled.id
            LEFT JOIN Organisation AS Insurer ON Insurer.id = ClientPolicy.insurer_id
            LEFT JOIN vrbPerson AS execPerson ON execPerson.id = Event.execPerson_id
            LEFT JOIN Visit ON Visit.id = Account_Item.visit_id
            LEFT JOIN rbScene ON Visit.scene_id = rbScene.id
            LEFT JOIN EventType ON EventType.id = Event.eventType_id
            LEFT JOIN Diagnostic ON ( Diagnostic.event_id = Account_Item.event_id
                                      AND Diagnostic.diagnosisType_id IN (SELECT id FROM rbDiagnosisType WHERE code= '1')
                                      AND Diagnostic.deleted = 0)
            LEFT JOIN Diagnosis ON Diagnosis.id = Diagnostic.diagnosis_id AND Diagnosis.deleted = 0
            LEFT JOIN rbTraumaType ON Diagnosis.traumaType_id = rbTraumaType.id
            LEFT JOIN rbDiseaseCharacter ON rbDiseaseCharacter.id = Diagnosis.character_id
            LEFT JOIN Diagnostic AS AccDiagnostic ON AccDiagnostic.id = (
                SELECT id FROM Diagnostic AS AD
                WHERE AD.event_id = Account_Item.event_id AND
                    AD.diagnosisType_id IN (SELECT id FROM rbDiagnosisType WHERE code='9') AND
                    AD.person_id = Event.execPerson_id AND
                    AD.deleted = 0 LIMIT 1)
            LEFT JOIN Diagnosis AS AccDiagnosis ON
                AccDiagnosis.id = AccDiagnostic.diagnosis_id AND
                AccDiagnosis.deleted = 0
            LEFT JOIN rbDiseaseCharacter AccCharacter ON AccCharacter.id = AccDiagnosis.character_id
            LEFT JOIN mes.MES AS MainMES ON MainMES.id = Event.MES_id
            LEFT JOIN TempInvalid ON
                TempInvalid.id = (
                    SELECT  TI.id
                    FROM    TempInvalid TI
                    WHERE   TI.client_id = Client.id AND
                                TI.deleted = 0 AND
                                (TI.begDate IS NULL OR TI.begDate >= Event.setDate) AND
                                (TI.endDate IS NULL OR TI.endDate <= Event.execDate)
                    LIMIT 1
                )
            LEFT JOIN rbTempInvalidReason ON TempInvalid.tempInvalidReason_id = rbTempInvalidReason.id
            LEFT JOIN rbService AS rbEventService ON rbEventService.id = EventType.service_id
            LEFT JOIN rbService AS rbVisitService ON rbVisitService.id = Visit.service_id
            LEFT JOIN rbService AS rbItemService ON rbItemService.id = Account_Item.service_id
            LEFT JOIN Action ON Account_Item.action_id = Action.id
            LEFT JOIN rbPayRefuseType ON rbPayRefuseType.id = Account_Item.refuseType_id
            LEFT JOIN rbResult ON rbResult.id = Event.result_id
            WHERE
                Account_Item.reexposeItem_id IS NULL
            AND (Account_Item.date IS NULL
                 OR (Account_Item.date IS NOT NULL AND rbPayRefuseType.rerun != 0)
                )
            AND %s
        """ % tableAccountItem['id'].inlist(self.idList)

        return self.db.query(stmt)


# *****************************************************************************************

    def process(self, dbf, record, params):
        birthDate = forceDate(record.value('birthDate'))
        begDate = forceDate(record.value('begDate'))
        eventId = forceRef(record.value('event_id'))
        clientId = forceRef(record.value('client_id'))

        row = dbf.newRecord()
        #1. Дата заполнения (дата последнего изменения события)
        row['MDATE'] = pyDate(forceDate(record.value('modifyDatetime')))
       #2. Ф.И.О. заполнителя (автор последнего изменения события)
        row['MPERSON'] = forceString(record.value('modifyPersonName'))
        # Пациент:
        row['PSERIAL'] = forceString(record.value('policySerial'))  # 1. Серия полиса
        row['PNUMBER'] = forceString(record.value('policyNumber')) # 2. № полиса
        row['SNILS'] = forceString(record.value('SNILS'))             # 3. СНИЛС
        row['CLIENTID'] = clientId   #4. ID пациента
        row['LASTNAME'] = forceString(record.value('lastName'))  # 5. Фамилия
        row['FIRSTNAME'] = forceString(record.value('firstName'))  # 6. Имя
        row['PATRNAME'] = forceString(record.value('patrName'))  # 7. Отчество
        row['SEX'] = forceInt(record.value('sex')) % 10 # 8. Пол
        row['BIRTHDAY'] =pyDate(birthDate) # 9. Дата рождения
        # 10. Адрес  регистрации (по KLADR: регион, нас.пункт, улица, дом, корпус, квартира)
        kladrCode = forceString(record.value('KLADRCode'))
        row['REGADDR'] = formatAddress(forceRef(record.value('addressId')))
        row['KLADR'] = kladrCode
        row['KLADRST'] = forceString(record.value('KLADRStreetCode'))
        row['HOUSENUM'] = forceString(record.value('number'))
        row['HOUSECORP'] = forceString(record.value('corpus'))
        row['FLAT'] = forceString(record.value('flat'))
        # 11. Житель села/города
        row['CITY'] = 1 if str(kladrCode)[8:11] == '000'  else 0
        # 12. Льгота (выгружаем региональный код типа соц статуса с кодом класса 1-льгота)
        row['BENEFIT'] = forceString(record.value('benefitCode'))
        # Социальный статус (выгружаем региональный код типа соц статуса с кодом класса 9-социальный статус)
        row['SOCSTATUS'] = forceString(record.value('socStatusCode'))
        # Инвалидность (выгружаем региональный код типа соц статуса с кодом класса 10-инвалидность)
        row['DISABLED'] = forceString(record.value('disabledCode'))
        #15. Территория страхования («регион» страхования СМО из справочника организаций)
        row['INSREGION'] = forceString(record.value('insurerArea'))
        #16. Страховая компания (краткое наименование страховой компании)
        row['INSURER'] = forceString(record.value('insurerName'))
        #Событие:
        row['BEGDATE'] = pyDate(begDate)  # 1. Дата начала
        # 2. Ф.И.О. врача
        row['PERSON'] = forceString(record.value('execPersonName'))
        # 3. Место обслуживания (справочник место выполнения визита, выгружаем код)
        row['PLACE'] = forceString(record.value('sceneCode'))
        # 4. Цель обращения (справочник типов событий, региональный код)
        row['PURPOSE'] = forceString(record.value('eventTypeCode'))
        # 5. Основной диагноз (выгружаем заключительный диагноз, если нет заключительного выгружаем основной)
        row['MKB'] = forceString(record.value('MKB'))
        row['MES'] = forceString(record.value('mesCode'))
        # Характер заболевания
        row['CHARACTER'] = forceString(record.value('characterCode'))
        row['TRAUMA'] = forceString(record.value('traumaTypeCode'))  # Тип травмы
        # Сопутствующий диагноз (выгружаем сопутствующий диагноз
        # ответственного врача, если несколько выгружаем последний)
        row['ACCMKB'] = forceString(record.value('accMKB'))
        row['ACCMES'] = row['MES']
        # Характер заболевания
        row['ACCCHAR'] = forceString(record.value('accCharacterCode'))
        row['RESULT'] = forceString(record.value('resultCode'))
        # Больничный лист
        # Дата открытия
        row['TIBEGDATE'] = pyDate(forceDate(record.value('tempInvalidBegDate')))
        # Дата закрытия
        row['TIENDDATE'] = pyDate(forceDate(record.value('tempInvalidEndDate')))
        # Первичный/продление
        row['TIPRIMARY'] = forceInt(record.value('tempInvalidClosed')) % 10
         # Причина выдачи
        row['TIREASON'] = forceString(record.value('tempInvalidReason'))
        # Услуги:
        serviceId = forceString(record.value('serviceId'))
        # Ссылка на диагноз (выгружаем диагноз действия)
        row['SMKB'] =  forceString(record.value('actionMKB'))
        # Дата оказания
        row['SDATE'] = pyDate(forceDate(record.value('serviceDate')))
        servicePersonId = forceRef(record.value('servicePersonId'))
        # Ф.И.О. врача
        row['SPERSON'] = self.getPersonName(servicePersonId)
         # Специальность врача (региональный код)
        row['SSPEC'] = self.getPersonSpecialityRegionalCode(servicePersonId)
        # Ф.И.О. ассистента
        row['SASSIST'] = self.getPersonName(forceRef(record.value('assistantPersonId')))
        row['SCODE'] = self.getServiceCode(serviceId) # Код услуги
        row['SAMOUNT'] = forceInt(record.value('amount')) # Количество
        row.store()

# *****************************************************************************************

class CExportPage2(CAbstractExportPage2):
    def __init__(self, parent):
        CAbstractExportPage2.__init__(self, parent,  'R45NATIVE')


    def saveExportResults(self):
        src = self._parent.page1.getDbfBaseName()
        srcFullName = os.path.join(forceStringEx(self.getTmpDir()), src)
        dst = os.path.join(forceStringEx(self.edtDir.text()), src)
        success, result = QtGui.qApp.call(self, shutil.move, (srcFullName, dst))
        return success
