# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

u"""Экспорт реестра стационарной и стационарзамещающей медицинской помощи.
    Мурманск"""

import os.path

from PyQt4 import QtGui
from PyQt4.QtCore import QDate

from library.dbfpy.dbf import Dbf
from library.Utils     import (forceBool, forceInt, forceRef, forceString,
    pyDate, toVariant, forceDate, firstMonthDay, formatSNILS, forceDouble)
from Exchange.Export import (CExportHelperMixin, CAbstractExportWizard,
                                    CAbstractExportPage1, CAbstractExportPage2)
from Exchange.ExportR51OMS import getIdsp
from Events.Action import CAction
from Events.ActionServiceType import CActionServiceType

from Exchange.Ui_ExportR51HospitalPage1 import Ui_ExportPage1


def exportR51Hospital(widget, accountId, accountItemIdList, _=None):
    u"""Создает диалог экспорта реестра счета"""

    wizard = CExportWizard(widget)
    wizard.setAccountId(accountId)
    wizard.setAccountItemsIdList(accountItemIdList)
    wizard.exec_()

# ******************************************************************************

class CExportWizard(CAbstractExportWizard):
    u"""Мастер экспорта в ОМС Мурманской области (стационар)"""

    def __init__(self, parent=None):
        title = u'Мастер экспорта в ОМС Мурманской области (стационар)'
        CAbstractExportWizard.__init__(self, parent, title)

        self.page1 = CExportPage1(self)
        self.page2 = CExportPage2(self)
        self.addPage(self.page1)
        self.addPage(self.page2)


    def getTmpDir(self):
        return CAbstractExportWizard.getTmpDirEx(self, 'R51HOSP')

# ******************************************************************************

class CExportPage1(CAbstractExportPage1, Ui_ExportPage1, CExportHelperMixin):
    u"""Первая страница мастера экспорта"""

    exportTypeSMO = 0 # индексы из cmbExportType
    exportTypeTFOMS = 1
    exportTypeDializ = 2
    exportTypeDS = 3

    exportFormat2017 = 0 # cmbExportFormat
    exportFormat2018 = 1

    mapEventOrderToHosp = {2: '1', 1: '0'}

    def __init__(self, parent):
        CAbstractExportPage1.__init__(self, parent)
        CExportHelperMixin.__init__(self)

        self.ignoreErrors = False

        prefs = QtGui.qApp.preferences.appPrefs
        self.cmbExportFormat.setCurrentIndex(
            forceInt(prefs.get('ExportR51HospitalExportFormat', 0)))
        self.cmbExportType.setCurrentIndex(
            forceInt(prefs.get('ExportR51HospitalExportType', 0)))
        self.chkVerboseLog.setChecked(
            forceBool(prefs.get('ExportR51HospitalVerboseLog', False)))
        self.chkIgnoreErrors.setChecked(
            forceBool(prefs.get('ExportR51HospitalIgnoreErrors', False)))

        self.actionTypeMovement = None
        self.actionTypeArrival = None
        self.actionTypeIllegalActions = None
        self.actionTypeMultipleBirth = None
        self.actionTypeLeaved = None

        self.hasSenderActionPropertyType = False
        self.hasIllegalActionPropertyType = False

        self.exportFormat = 0
        self.mapEventIdtoIdCase = {}


    def setExportMode(self, flag):
        self.btnCancel.setEnabled(flag)
        self.btnExport.setEnabled(not flag)
        self.chkIgnoreErrors.setEnabled(not flag)
        self.chkVerboseLog.setEnabled(not flag)
        self.cmbExportType.setEnabled(not flag)
        self.cmbExportFormat.setEnabled(not flag)


    def validatePage(self):
        QtGui.qApp.preferences.appPrefs['ExportR51HospitalIgnoreErrors'] =\
            toVariant(self.chkIgnoreErrors.isChecked())
        QtGui.qApp.preferences.appPrefs['ExportR51HospitalVerboseLog'] =\
            toVariant(self.chkVerboseLog.isChecked())
        QtGui.qApp.preferences.appPrefs['ExportR51HospitalExportType'] = \
            toVariant(self.cmbExportType.currentIndex())
        QtGui.qApp.preferences.appPrefs['ExportR51HospitalExportFormat'] = \
            toVariant(self.cmbExportFormat.currentIndex())
        return CAbstractExportPage1.validatePage(self)

# ******************************************************************************

    def exportInt(self):
        tableOrganisation = self.db.table('Organisation')
        tableActionType = self.db.table('ActionType')
        params = {}

        self.ignoreErrors = self.chkIgnoreErrors.isChecked()
        self.exportFormat = self.cmbExportFormat.currentIndex()
        self.mapEventIdtoIdCase = {}
        params['exportType'] = self.cmbExportType.currentIndex()
        self.log(u'Выгружаем счет в формате "%d".' % params['exportType'])

        params['operOperFieldName'] = 'service' if (
            params['exportType'] in (self.exportTypeTFOMS, self.exportTypeSMO)
            ) else 'serviceCode'

        lpuId = QtGui.qApp.currentOrgId()
        params['lpuOKPO'] = forceString(self.db.translate(
            tableOrganisation, 'id', lpuId, 'OKPO'))
        params['lpuCode'] = forceString(self.db.translate(
            tableOrganisation, 'id', lpuId, 'infisCode'))
        self.log(u'ЛПУ: ОКПО "%s", код инфис: "%s".' % (
            params['lpuOKPO'], params['lpuCode']))

        if not params['lpuCode']:
            self.log(u'<b><font color=red>ОШИБКА<\font><\b>:'
                     u'Для текущего ЛПУ не задан код инфис', True)
            if not self.ignoreErrors:
                self.abort()
                return

        actionTypes = (
            ('actionTypeArrival', 'received', u'Поступление'),
            ('actionTypeIllegalActions', 'IllegalActions',
                u'Противоправные действия'),
            ('actionTypeMovement', 'moving', u'Движение'),
            ('actionTypeMultipleBirth', 'multipleBirth', u'Многоплодные роды'),
            ('actionTypeLeaved', 'leaved', u'Выписка')
        )

        for (attr, flatCode, desc) in actionTypes:
            _id = forceRef(self.db.translate(tableActionType,
                'flatCode', flatCode, 'id'))

            if not _id:
                self.log(u'<b><font color=red>ОШИБКА<\font><\b>:'
                    u'Не найден тип действия "%s" (плоский код: "%s")' % (
                    desc, flatCode), True)
                if not self.ignoreErrors:
                    self.abort()
                    return

            setattr(self, attr, _id)

        actionPropertyTypes = (
            ('actionTypeArrival', u'Поступление', u'Кем направлен',
                'hasSenderActionPropertyType'),
            ('actionTypeArrival', u'Поступление', u'Направлен в отделение',
                'hasSentToActionPropertyType'),
            ('actionTypeArrival', u'Поступление', u'Вес',
                'hasWeightActionPropertyType'),
            ('actionTypeArrival', u'Поступление', u'Квота',
                'hasArrivalQuotaActionPropertyType'),
            ('actionTypeIllegalActions', u'Противоправные действия',
                u'Признак', 'hasIllegalActionPropertyType'),
            ('actionTypeLeaved', u'Выписка', u'Квота',
                'hasLeavedQuotaActionPropertyType'),
            ('actionTypeMovement', u'Движение', u'Квота',
                'hasMovementQuotaActionPropertyType'),
        )

        for (actionType, desc, _property, attr) in actionPropertyTypes:
            actionTypeId = getattr(self, actionType, None)
            flag = self.getActionPropertyTypeIdByName(actionTypeId,
                                                      _property) is not None

            if not flag:
                self.log(u'<b><font color=red>ОШИБКА<\font><\b>:'
                         u'Не найдено свойство "%s" у типа действия "%s")' % (
                         _property, desc), True)
                if not self.ignoreErrors:
                    self.abort()
                    return

            setattr(self, attr, flag)

        params.update(self.accountInfo())

        orgStructId = params['accOrgStructureId']
        record = self.db.getRecord('OrgStructure',
            'infisCode, tfomsCode', orgStructId)
        if record:
            params['accOrgStructInfisCode'] = forceString(
                record.value('infisCode'))
            params['accOrgStructTfomsCode'] = forceString(
                record.value('tfomsCode'))

        params['payerInfis'] = forceString(self.db.translate(
            tableOrganisation, 'id', params['payerId'], 'infisCode'))
        params['rowNumber'] = 0
        self.setProcessParams(params)

        CAbstractExportPage1.exportInt(self)


    def createDbf(self):
        return (self._createReestrDbf(),
                self._createDepReeDbf(),
                self._createAddInfDbf(),
                self._createOperDbf(),
                self._createDepAddDbf(),
                    )


    def _createReestrDbf(self):
        """ Создает структуру dbf для REESTR.DBF """

        dbfName = os.path.join(self.getTmpDir(), 'REESTR.DBF')
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('NIB', 'C', 10), # Номер истории болезни
                                    # Не повторяется в отчетном году
            ('INS', 'C', 2), # Код СМО (по справочнику фонда)
            ('DATA1', 'D'), # Начальная дата интервала,
                                        # за который представляется реестр
            ('DATA2', 'D'), # Конечная дата интервала,
                                        # за который представляется реестр
            ('FAM', 'C', 40), # Фамилия пациента
            ('IMM', 'C', 40), # Имя пациента
            ('OTC', 'C', 40), # Отчество пациента
            ('SER_PASP', 'C', 8), # Серия документа, удостоверяющего личность
            ('NUM_PASP', 'C', 8), # Номер документа, удостоверяющего личность
            ('TYP_DOC', 'N', 2), # Тип документа, удостоверяющего личность
            ('SS', 'C', 14), # Страховой номер индивидуального лицевого счета
            ('DROJD', 'D'), # Дата рождения пациента
            ('POL', 'C', 1), # Пол пациента   («М», «Ж»)
            ('RAB', 'C', 1), # Признак работающий- “1”/неработающий – “0”
            ('SPOLIS', 'C', 10), # Серия полиса
            ('NPOLIS', 'C', 20), # Номер полиса
            ('TAUN', 'C', 3), # Код населенного пункта проживания пациента по
                                        #справочнику фонда TAUNS
            ('HOSP', 'C', 1), # Признак поступления экстренное –“1”
                                    #/плановое –“0   У нас дневной стационар =0
            ('DATAP', 'D'), # Дата поступления в ЛПУ
            ('DATAV', 'D'), # Дата выписки из ЛПУ
            ('STOIM', 'N', 10, 2), # Суммарная стоимость пребывания в ЛПУ
            ('KRIM', 'C', 1), # Признак противоправных действий 0 – обычное СБО,
                                        # 1- противоправное действие.
            ('DIAG', 'C', 6), # Заключительный диагноз
            ('DIRECT_LPU', 'C', 3), # Код ЛПУ, направившего на госпитализацию
            ('DIR_SUBLPU', 'C', 2),
            ('DIR_TOWN', 'C', 3),
            ('MASTER', 'C', 9), # Коды ЛПУ приписки по справочнику фонда
                                            #(поликлиника + стоматология +ЖК)
                                            #В строке через 3 пробела.
                                            # Пример 103   160   121
            ('LPU', 'C', 3), # Код ЛПУ пребывания по справочнику фонда
                                    # 141- это стационар на дому пол-ки 3,
                                   # 951- реабилитация, 605- дневной стационар
            ('SUB_HOSP', 'C', 2),
            ('TOWN_HOSP', 'C', 3),
            ('COUNT', 'C', 10), # Номер счета представляемого в фонд
            ('STOIM_S', 'N', 10, 2), # Сумма из средств Федеральных субвенций
            ('TPOLIS', 'N', 1), # Тип полиса:   1 – старого образца,
                            #2 – временное свидетельство, 3 – нового образца
            ('VNOV_D', 'N', 4), # Вес при рождении в граммах
            ('P_CODE', 'C', 7), #Личный код врача, закрывшего историю болезни
            ('NPR_FDATE', 'D'), #Дата направления на лечение
        )

        return dbf


    def _createDepReeDbf(self):
        """ Создает структуру dbf для DEP_REE.DBF """

        dbfName = os.path.join(self.getTmpDir(), 'DEP_REE.DBF')
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('NIB', 'C', 10), # Номер истории болезни
            ('ID_CASE', 'N', 10), # Порядковый номер случая в реестре
            ('OTD', 'C', 3), # Код отделения по справочнику фонда
            ('DIRECT_OTD', 'C', 3), # Код отделения, откуда переведен пациент
                        # (при переводе из одного отделения в другое отделение)
            ('HOSP', 'C', 1), # Признак поступления экстренное – “1”
                        # плановое – “0”/ перевод из отделения в отделение – “2”
            ('DATAP', 'D'), # Дата поступления  в отделение
            ('DATAV', 'D'), # Дата выписки из отделения
            ('DOCTOR', 'C', 8), # Код врача лечившего пациента
            ('LEVEL', 'C', 1), # Уровень оказанной помощи
                                    # (“1”- дневные стационары,
                                # “0” – стационары круглосуточного пребывания)
            ('TALON', 'C', 6), # № направления на госпитализацию
            ('DIAG', 'C', 6), # Код МКБ Х
            ('DS0', 'C', 6), # Диагноз первичный
            ('DS_S', 'C', 6), # Код диагноза сопутствующего заболевания
                                        #(состояния) по МКБ-10
            ('BED_PROF', 'C', 3), # Код профиля койки    bed_prof
            ('STOIM_S', 'N', 10, 2), # Сумма из средств Федеральных субвенций
        )

        if self.exportFormat >= self.exportFormat2017:
            dbf.addField(
                ('STOIM_D', 'N', 10, 2), #Стоимость услуг по гемодиализу
            )

        dbf.addField(
            ('RSLT', 'N', 3), # Результат обращения/ госпитализации
            ('ISHOD', 'N', 3), # Исход заболевания
            ('VID_HMP', 'C', 12), # Вид высокотехнологичной медицинской
                                                # помощи (ВМП)
            ('METOD_HMP', 'N', 3), # Метод высокотехнологичной
                                                     # медицинской помощи
            ('PROFIL', 'N', 3), # Профиль оказанной медицинской помощи
            ('PRVS', 'N', 3), # Код специальности медицинского работника,
                                        # оказавшего услугу
            ('KSG', 'N', 5, 1), # Код клинико-статистической группы
            ('DS_P', 'C', 6, 0),
            ('DS3', 'C', 6, 0),
            ('VIDPOM', 'N', 4),#
            ('VID_FIN', 'N', 1), # Источник финансирования

            ('MOTHER', 'N', 1),#
            ('P_MODE', 'N', 1),#
        )

        if self.exportFormat >= self.exportFormat2017:
            dbf.addField(
                ('P_PER', 'N', 1), #Признак поступления / перевода
                ('NPL', 'N', 1), #Неполный объём
                ('PR_OS', 'N', 2), #Признак “Особый случай»
            )

        if self.exportFormat >= self.exportFormat2018:
            dbf.addField(
                ('IDSP', 'N', 2), #Код способа оплаты медицинской помощи
                ('ADD_CR_KSG', 'C', 5), # Дополнительный классификационный
                                        # критерий для выбора КСГ
                ('STAGE', 'N', 1), #Этап проведения ЭКО
            )

        return dbf


    def _createAddInfDbf(self):
        """ Создает структуру dbf для ADD_INF.DBF """

        dbfName = os.path.join(self.getTmpDir(), 'ADD_INF.DBF')
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('NIB', 'C', 10), # Номер истории болезни
            ('MR', 'C', 100), # Место рождения пациента или представителя
            ('OKATOG', 'C', 11), # Код места жительства по ОКАТО
            ('OKATOP', 'C', 11), # Код места пребывания по ОКАТО
            ('OKATO_OMS', 'C', 5),  # Код ОКАТО территории страхования по ОМС
                                                        #(по справочнику фонда)
            ('FAMP', 'C', 40), # Фамилия родителя (представителя) пациента
            ('IMP', 'C', 40), # Имя родителя (представителя) пациента
            ('OTP', 'C', 40), # Отчество родителя (представителя) пациента
            ('DRP', 'D'), # Дата рождения родителя (представителя) пациента
            ('WP', 'C', 1), # Пол родителя (представителя) пациента
            ('C_DOC', 'N', 2), # Код типа документа, удостоверяющего личность
                            # пациента (представителя) (по  справочнику фонда)
            ('S_DOC', 'C', 9), # Серия документа, удостоверяющего личность
                            # пациента (представителя) (по справочнику фонда)
            ('N_DOC', 'C', 8), # Номер документа, удостоверяющего личность
                            # пациента (представителя) (справочник фонда)
            ('NOVOR', 'C', 9), # Признак новорожденного
            ('Q_G', 'C', 7), # Признак «Особый случай» при регистрации
                                    # обращения  за медицинской помощью
                                    #«1» –  отсутствие у пациента полиса
                                    # обязательного медицинского страхования;
                                    #«2»  - медицинская помощь оказана
                                    # новорожденному;
                                    #«3» – при оказании медицинской помощи
                                    # ребенку до 14 лет был предъявлен документ,
                                    # удостоверяющий личность, и полис ОМС
                                    #одного из его родителей или представителей;
                                    #«4» – отсутствие отчества в документе,
                                    # удостоверяющем личность пациента
                                    # (представителя пациента);
                                    #«5» -  медицинская помощь, оказанная
                                    # новорожденному при  многоплодных родах.
        )

        if self.exportFormat >= self.exportFormat2018:
            dbf.addField(
                ('MSE', 'N', 1), # Направление на МСЭ
            )

        return dbf

    def _createOperDbf(self):
        """ Создает структуру dbf для OPER.DBF """

        dbfName = os.path.join(self.getTmpDir(), 'OPER.DBF')
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('NIB', 'C', 10), # Номер истории болезни
            ('ID_CASE', 'N', 10), # Порядковый номер случая в реестре
            ('OTD_O', 'C', 3), # Код отделения, где проведена операция
            ('DATA_O', 'D'), # Дата, когда проведена операция
            ('OPER', 'C', 15), # Код операции
            ('KOL_OPER', 'N', 1), # Количество операций
            ('DOCTOR', 'C', 8), # Код врача лечившего пациента
            ('PROFIL', 'N', 3), # Профиль оказанной медицинской помощи
            ('PRVS', 'N', 4), # Код специальности медицинского работника,
                                        # оказавшего услугу
        )

        if self.exportFormat >= self.exportFormat2017:
            dbf.addField(
                ('PR_OPER', 'N', 1), #Дополнительный признак операции
            )
        return dbf

    def _createDepAddDbf(self):
        u"""Создает структуру DBF для DEP_ADD.DBF"""

        dbfName = os.path.join(self.getTmpDir(), 'DEP_ADD.DBF')
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('NIB', 'C', 10), # Номер истории болезни
            ('ID_CASE', 'N', 10), # Порядковый номер случая в реестре
            ('DATE_1', 'D'), # Дата поступления пациента в отделение
                                        #интенсивной терапии и  реанимации
            ('DATE_2', 'D'), # Дата выбытия пациента из отделения
                                        #интенсивной терапии и  реанимации
            ('DS', 'C', 6, 0), # Диагноз основной
            ('DOCTOR', 'C', 8), # Код врача лечившего пациента
            ('PROFIL', 'N', 3), # Профиль оказанной медицинской помощи
            ('PRVS', 'N', 4), # Код специальности медицинского работника,
                                        #оказавшего услугу
        )
        return dbf

# ******************************************************************************

    def createQuery(self):
        tableAccountItem = self.db.table('Account_Item')

        stmt = u"""SELECT Event.client_id,
                Account_Item.event_id,
                ClientPolicy.serial AS policySerial,
                ClientPolicy.number AS policyNumber,
                rbPolicyKind.regionalCode AS policyKind,
                IF(work.title IS NOT NULL,
                    work.title,
                    ClientWork.freeInput) AS workName,
                Event.setDate AS begDate,
                Event.execDate AS endDate,
                Diagnosis.MKB,
                IF(rbDiagnosticResult.regionalCode IS NULL,
                    EventResult.regionalCode,
                    rbDiagnosticResult.regionalCode) AS resultCode,
                EventResult.regionalCode AS eventResultRegionalCode,
                Client.lastName,
                Client.firstName,
                Client.patrName,
                ClientDocument.serial AS documentSerial,
                ClientDocument.number AS documentNumber,
                rbDocumentType.regionalCode AS documentRegionalCode,
                Client.SNILS,
                Client.birthDate,
                Client.birthPlace,
                Client.sex,
                RegAddressHouse.KLADRCode,
                RegKLADR.infis AS placeCode,
                LocRegionKLADR.OCATD AS locRegionOKATO,
                RegRegionKLADR.OCATD AS regRegionOKATO,
                AccDiagnosis.MKB AS accMKB,
                IF(Account_Item.visit_id IS NOT NULL, VisitPerson.code,
                    IF(Account_Item.action_id IS NOT NULL,
                        IF(Action.person_id IS NOT NULL, ActionPerson.code,
                            ActionSetPerson.code), Person.code)
                ) AS personCode,
                IF(Account_Item.visit_id IS NOT NULL, VisitSpeciality.regionalCode,
                    IF(Account_Item.action_id IS NOT NULL,
                        IF(Action.person_id IS NOT NULL, ActionSpeciality.regionalCode,
                            ActionSetSpeciality.regionalCode), rbSpeciality.regionalCode)
                )  AS specialityCode,
                IF(Account_Item.visit_id IS NOT NULL, VisitSpeciality.federalCode,
                    IF(Account_Item.action_id IS NOT NULL,
                        IF(Action.person_id IS NOT NULL, ActionSpeciality.federalCode,
                            ActionSetSpeciality.federalCode), rbSpeciality.federalCode)
                )  AS specialityFederalCode,
                IF(Account_Item.service_id IS NOT NULL,
                    rbItemService.infis,
                    IF(Account_Item.visit_id IS NOT NULL, rbVisitService.infis, rbEventService.infis)
                    ) AS service,
                Account_Item.amount,
                Action.endDate AS actionDate,
                Account_Item.`sum` AS `sum`,
                Insurer.OKATO AS insurerOKATO,
                RelegateOrg.smoCode AS relegateOrgSmoCode,
                RelegateOrg.tfomsExtCode AS relegateOrgTfomsExtCode,
                AttachOrg.infisCode AS attachCodeString,
                AttachOrg.smoCode AS attachOrgSmoCode,
                AttachOrg.tfomsExtCode AS attachOrgTfomsExtCode,
                Event.externalId,
                rbMedicalAidType.regionalCode AS medicalAidTypeCode,
                HospitalAction.id AS hospitalActionId,
                ActionType.code AS serviceCode,
                mes.MES_ksg.code AS ksgCode,
                Event.order AS eventOrder,
                Hospital.infisCode AS hospitalInfisCode,
                rbMedicalAidKind.regionalCode AS medicalAidKindCode,
                Contract_Tariff.price,
                Action.status AS actionStatus,
                ActionType.serviceType AS serviceType,
                CriterionActionType.code AS criterionCode,
                rbMedicalAidUnit.regionalCode AS medicalAidUnitCode,
                NOT ISNULL(CurrentOrgAttach.id) AS isAttached,
                rbScene.name LIKE "%%на дому%%" AS isHomeVisit,
                Event.srcDate,
                Person.regionalCode AS execPersonRegionalCode
            FROM Account_Item
            LEFT JOIN Action ON Action.id = Account_Item.action_id
            LEFT JOIN Visit  ON Visit.id  = Account_Item.visit_id
            LEFT JOIN Event  ON Event.id  = Account_Item.event_id
            LEFT JOIN Client ON Client.id = Event.client_id
            LEFT JOIN ClientPolicy ON ClientPolicy.client_id = Client.id AND
                      ClientPolicy.id = (SELECT MAX(CP.id)
                                         FROM   ClientPolicy AS CP
                                         LEFT JOIN rbPolicyType AS CPT ON CPT.id = CP.policyType_id
                                         WHERE  CP.client_id = Client.id AND CP.deleted=0 AND CPT.code IN ('1','2'))
            LEFT JOIN Organisation AS Insurer ON Insurer.id = ClientPolicy.insurer_id
            LEFT JOIN rbPolicyKind ON rbPolicyKind.id = ClientPolicy.policyKind_id
            LEFT JOIN ClientDocument ON ClientDocument.client_id = Client.id AND
                      ClientDocument.id = (SELECT MAX(CD.id)
                                         FROM   ClientDocument AS CD
                                         LEFT JOIN rbDocumentType AS rbDT ON rbDT.id = CD.documentType_id
                                         LEFT JOIN rbDocumentTypeGroup AS rbDTG ON rbDTG.id = rbDT.group_id
                                         WHERE  rbDTG.code = '1' AND CD.client_id = Client.id AND CD.deleted=0)
            LEFT JOIN rbDocumentType ON rbDocumentType.id = ClientDocument.documentType_id
            LEFT JOIN ClientAddress ClientRegAddress ON ClientRegAddress.client_id = Client.id AND
                      ClientRegAddress.id = (SELECT MAX(CRA.id)
                                         FROM   ClientAddress AS CRA
                                         WHERE  CRA.type = 0 AND CRA.client_id = Client.id AND CRA.deleted=0)
            LEFT JOIN ClientAddress ClientLocAddress ON ClientLocAddress.client_id = Client.id AND
                      ClientLocAddress.id = (SELECT MAX(CLA.id)
                                         FROM   ClientAddress AS CLA
                                         WHERE  CLA.type = 1 AND CLA.client_id = Client.id AND CLA.deleted=0)
            LEFT JOIN Address   RegAddress ON RegAddress.id = ClientRegAddress.address_id
            LEFT JOIN AddressHouse RegAddressHouse ON RegAddressHouse.id = RegAddress.house_id
            LEFT JOIN Address   LocAddress ON LocAddress.id = ClientLocAddress.address_id
            LEFT JOIN AddressHouse LocAddressHouse ON LocAddressHouse.id = LocAddress.house_id
            LEFT JOIN kladr.KLADR RegKLADR ON RegKLADR.CODE = RegAddressHouse.KLADRCode
            LEFT JOIN (
                SELECT * FROM kladr.SOCRBASE AS SB
                GROUP BY SB.SCNAME
            ) AS  RegSOCR ON RegSOCR.SCNAME = RegKLADR.SOCR
            LEFT JOIN kladr.KLADR RegRegionKLADR ON RegRegionKLADR.CODE = (
                SELECT RPAD(LEFT(RegAddressHouse.KLADRCode,2),13,'0'))
            LEFT JOIN kladr.KLADR LocRegionKLADR ON LocRegionKLADR.CODE = (
                SELECT RPAD(LEFT(LocAddressHouse.KLADRCode,2),13,'0'))
            LEFT JOIN Person ON Person.id = Event.execPerson_id
                AND Person.id IS NOT NULL
            LEFT JOIN Person AS VisitPerson ON VisitPerson.id = Visit.person_id
                AND VisitPerson.id IS NOT NULL
            LEFT JOIN Person AS ActionPerson ON ActionPerson.id = Action.person_id
                AND ActionPerson.id IS NOT NULL
            LEFT JOIN Person AS ActionSetPerson ON ActionSetPerson.id = Action.setPerson_id
                AND ActionSetPerson.id IS NOT NULL
            LEFT JOIN rbSpeciality ON Person.speciality_id = rbSpeciality.id
                AND rbSpeciality.id IS NOT NULL
            LEFT JOIN rbSpeciality AS VisitSpeciality ON VisitPerson.speciality_id = VisitSpeciality.id
                AND VisitSpeciality.id IS NOT NULL
            LEFT JOIN rbSpeciality AS ActionSpeciality ON ActionPerson.speciality_id = ActionSpeciality.id
                AND ActionSpeciality.id IS NOT NULL
            LEFT JOIN rbSpeciality AS ActionSetSpeciality ON ActionSetPerson.speciality_id = ActionSetSpeciality.id
                AND ActionSetSpeciality.id IS NOT NULL
            LEFT JOIN Diagnostic ON ( Diagnostic.event_id = Account_Item.event_id
                                      AND Diagnostic.diagnosisType_id IN (SELECT id FROM rbDiagnosisType WHERE code IN ('1', '2'))
                                      AND Diagnostic.person_id = Event.execPerson_id AND Diagnostic.deleted = 0)
            LEFT JOIN Diagnostic AS AccDiagnostic ON AccDiagnostic.id = (
             SELECT id FROM Diagnostic AS AD
             WHERE AD.event_id = Account_Item.event_id AND
                AD.diagnosisType_id IN (SELECT id FROM rbDiagnosisType WHERE code='9') AND
                AD.person_id = Event.execPerson_id AND
                AD.deleted = 0 LIMIT 1)
            LEFT JOIN Diagnosis AS AccDiagnosis ON
                AccDiagnosis.id = AccDiagnostic.diagnosis_id AND
                AccDiagnosis.deleted = 0
            LEFT JOIN ClientWork ON ClientWork.client_id=Event.client_id AND
                        ClientWork.id = (SELECT max(CW.id)
                                         FROM ClientWork AS CW
                                         WHERE CW.client_id=Client.id AND CW.deleted=0)
            LEFT JOIN Organisation AS work ON work.id=ClientWork.org_id AND work.deleted = 0
            LEFT JOIN Diagnosis ON Diagnosis.id = Diagnostic.diagnosis_id AND Diagnosis.deleted = 0
            LEFT JOIN rbDiagnosticResult ON rbDiagnosticResult.id=Diagnostic.result_id
            LEFT JOIN rbResult AS EventResult ON EventResult.id=Event.result_id
            LEFT JOIN ActionType ON ActionType.id = Action.actionType_id
            LEFT JOIN EventType ON Event.eventType_id = EventType.id
            LEFT JOIN rbService AS rbEventService ON rbEventService.id = EventType.service_id
            LEFT JOIN rbService AS rbVisitService ON rbVisitService.id = Visit.service_id
            LEFT JOIN rbService AS rbItemService ON rbItemService.id = Account_Item.service_id
            LEFT JOIN rbPayRefuseType
                ON rbPayRefuseType.id = Account_Item.refuseType_id
            LEFT JOIN Organisation AS RelegateOrg
                ON Event.relegateOrg_id = RelegateOrg.id
            LEFT JOIN ClientAttach ON ClientAttach.client_id=Event.client_id AND
                        ClientAttach.id = (SELECT max(CA.id)
                                         FROM ClientAttach AS CA
                                         WHERE CA.client_id=Client.id AND CA.deleted=0)
            LEFT JOIN Organisation AS AttachOrg
                ON ClientAttach.LPU_id = AttachOrg.id
            LEFT JOIN mes.MES ON Event.MES_id = mes.MES.id
                AND mes.MES.deleted = 0
            LEFT JOIN mes.MES_ksg ON mes.MES_ksg.id = mes.MES.ksg_id
                AND mes.MES_ksg.deleted = 0
            LEFT JOIN Action AS HospitalAction ON HospitalAction.id = (
                SELECT MAX(A.id)
                FROM Action A
                WHERE A.event_id = Event.id AND
                          A.deleted = 0 AND
                          A.actionType_id IN (
                                SELECT AT.id
                                FROM ActionType AT
                                WHERE AT.flatCode ='moving'
                          )
            )
            LEFT JOIN rbMedicalAidType ON EventType.medicalAidType_id =rbMedicalAidType.id
            LEFT JOIN rbMedicalAidKind ON EventType.medicalAidKind_id = rbMedicalAidKind.id
            LEFT JOIN Organisation AS Hospital ON Hospital.id = Person.org_id
            LEFT JOIN Contract_Tariff ON Contract_Tariff.id = Account_Item.tariff_id
            LEFT JOIN ActionType AS CriterionActionType ON CriterionActionType.id = (
                SELECT MAX(AT.id)
                FROM Action A
                LEFT JOIN ActionType AT ON AT.id = A.actionType_id
                    AND AT.flatCode = 'AddCriterion'
                WHERE A.event_id = Event.id AND A.deleted = 0
            )
            LEFT JOIN rbScene ON rbScene.id = Visit.scene_id
            LEFT JOIN rbMedicalAidUnit ON rbMedicalAidUnit.id = Account_Item.unit_id
            LEFT JOIN ClientAttach AS CurrentOrgAttach ON CurrentOrgAttach.id = (
                SELECT MAX(COA.id)
                FROM ClientAttach COA
                WHERE COA.LPU_id = %d AND COA.client_id = Client.id
                    AND (COA.begDate IS NULL OR COA.begDate <= Event.execDate)
                    AND (COA.endDate IS NULL OR COA.endDate >= Event.execDate)
            )
            WHERE
                Account_Item.reexposeItem_id IS NULL
            AND (Account_Item.date IS NULL
                 OR (Account_Item.date IS NOT NULL AND rbPayRefuseType.rerun != 0)
                )
            AND %s
        """ % (QtGui.qApp.currentOrgId(),
               tableAccountItem['id'].inlist(self.idList))
        query = self.db.query(stmt)
        return query


    def _getQuota(self, eventId):
        u"""Возвращает квоту по идентификатору события"""

        try:
            quota = self.getActionPropertyText(eventId,
                                           self.actionTypeLeaved, u'Квота')
        except KeyError:
            quota = None

        if not quota:
            try:
                quota = self.getActionPropertyText(eventId,
                                       self.actionTypeMovement, u'Квота')
            except KeyError:
                quota = None

        if not quota:
            try:
                quota = self.getActionPropertyText(eventId,
                                       self.actionTypeArrival, u'Квота')
            except KeyError:
                quota = None

        if quota:
            code = quota.split("|")
            quota = forceRef(self.db.translate('QuotaType', 'code',
                                                 code[0], 'regionalCode'))

        return quota

# ******************************************************************************

    def process(self, dbf, record, params):
        local_params = {
            'begDate': forceDate(record.value('begDate')),
            'endDate': forceDate(record.value('endDate')),
            'eventId': forceRef(record.value('event_id')),
            'eventIdStr': forceString(record.value('event_id')),
            'clientId': forceRef(record.value('client_id')),
            'KLADRCode': forceString(record.value('KLADRCode')),
        }

        if params['lpuCode'] == '037': # для МСЧ 118
            externalId = forceString(record.value('externalId'))

            if externalId:
                local_params['eventIdStr'] = externalId

        local_params['orgStructId'] = self.getActionPropertyValue(
              local_params['eventId'], self.actionTypeArrival,
              u'Направлен в отделение')
        local_params['infisInternalCode'] = (
            self.getOrgStructureInfisInternalCode(local_params['orgStructId']))
        local_params['orgStructInfis'] = self.getOrgStructureInfisCode(
                                                    local_params['orgStructId'])
        local_params['orgStructInfis'] = (0 if not local_params['orgStructInfis'
            ] else forceInt(local_params['orgStructInfis']))
        local_params['orgStructTfoms'] = self.getOrgStructureTfomsCode(
                                                    local_params['orgStructId'])
        local_params['orgStructTfoms'] = (0 if not local_params['orgStructTfoms'
            ] else forceInt(local_params['orgStructTfoms']))

        local_params['policySerial'] = forceString(record.value('policySerial'))
        local_params['policyNumber'] = forceString(record.value('policyNumber'))

        hospitalActionId = forceRef(record.value('hospitalActionId'))
        local_params['bedProfileId'] = None
        local_params['isIntensive'] = False

        if hospitalActionId:
            action = CAction.getActionById(hospitalActionId)
            bedId = forceRef(action[u'койка'])

            if bedId:
                local_params['bedProfileId'] = (
                                    self.getHospitalBedProfileId(bedId))

                bedProfileCode = (self.getHospitalBedProfileCode(
                     local_params['bedProfileId'])
                     if local_params['bedProfileId'] else None)

                local_params['isIntensive'] = bedProfileCode in ('2', '3')

        local_params.update(params)

        if not local_params['KLADRCode']:
            self.log(u'Отсутвует КЛАДР адрес проживания и '
                u'регистрации для клиента clientId=%d' %
                local_params['clientId'])

        (dbfReestr, dbfDepRee, dbfAddInf, dbfOper, dbfDepAdd) = dbf

        serviceCode = forceString(record.value('serviceCode'))
        serviceType = forceInt(record.value('serviceType'))

        if ((serviceCode[:3] == u'А16'
                and params['exportType'] != self.exportTypeDS)
            or (serviceType == CActionServiceType.operation
                and params['exportType'] == self.exportTypeDS)):
            self._processOper(dbfOper, record, local_params) # операция
        else: # операции не должны попадать в реестр
            self._processReestr(dbfReestr, record, local_params)

        price = forceDouble(record.value('price'))
        if not price == 0.0:
            self._processDepRee(dbfDepRee, record, local_params)

        if params['exportType'] == self.exportTypeTFOMS:
            self._processAddInf(dbfAddInf, record, local_params)

        params['rowNumber'] += 1


    def _processReestr(self, dbf, record, params):
        u"""Эспорт данных в REESTR.DBF"""

        row = dbf.newRecord()
        # Номер истории болезни   Не повторяется в отчетном году
        row['NIB'] = params['eventIdStr']
        # Код СМО (по справочнику фонда)
        row['INS'] = params.get('payerInfis', '')
        # Начальная дата интервала, за который представляется реестр
        row['DATA1'] = pyDate(firstMonthDay(params['endDate']))
         # Конечная дата интервала, за который представляется реестр
        exposeDate = params.get('exposeDate', QDate())
        row['DATA2'] = pyDate(exposeDate if exposeDate else QDate.currentDate())
         # Фамилия пациента
        row['FAM'] = forceString(record.value('lastName'))
         # Имя пациента
        row['IMM'] = forceString(record.value('firstName'))
        # Отчество пациента
        row['OTC'] = forceString(record.value('patrName'))
        # Серия документа, удостоверяющего личность
        row['SER_PASP'] = forceString(record.value('documentSerial'))
        # Номер документа, удостоверяющего личность
        row['NUM_PASP'] = forceString(record.value('documentNumber'))
        #, 'N', 2, Тип документа, удостоверяющего личность
        row['TYP_DOC'] = forceInt(record.value('documentRegionalCode'))
        # Страховой номер индивидуального лицевого счета (СНИЛС)
        row['SS'] = formatSNILS(forceString(record.value('SNILS')))
        # Дата рождения пациента
        row['DROJD'] = pyDate(forceDate(record.value('birthDate')))
        # Пол пациента   («М», «Ж»)
        row['POL'] = u'М' if forceInt(record.value('sex')) == 1 else u'Ж'
        # Признак работающий- “1”/неработающий – “0”
        row['RAB'] = '1' if forceString(record.value('workName')) else '0'
        row['SPOLIS'] = params['policySerial'] #, 'C', 10), # Серия полиса
        row['NPOLIS'] = params['policyNumber'] #, 'C', 10), # Номер полиса
        # Код населенного пункта проживания пациента по справочнику фонда
        row['TAUN'] = forceString(record.value('placeCode'))
        # Признак поступления экстренное –“1” /плановое –“0
        # У нас дневной стационар = 0
        row['HOSP'] = self.mapEventOrderToHosp.get(forceInt(
                                        record.value('eventOrder')), '')
        row['DATAP'] = pyDate(params['begDate']) # Дата поступления в ЛПУ
        row['DATAV'] = pyDate(params['endDate']) # Дата выписки из ЛПУ
        # Суммарная стоимость пребывания в ЛПУ
        row['STOIM'] = forceDouble(record.value('sum'))
        row['STOIM_S'] = row['STOIM']
        row['TPOLIS'] = forceInt(record.value('policyKind'))

        # Признак противоправных действий   0 – обычное СБО,
        # 1- противоправное действие.
        criminalSign = self.getActionPropertyText(params['eventId'],
                                self.actionTypeIllegalActions, u'Признак') \
                                if self.hasIllegalActionPropertyType else False

        if criminalSign:
            row['KRIM'] = criminalSign

        row['DIAG'] = forceString(record.value('MKB')) # Заключительный диагноз
        # Код ЛПУ, направившего на госпитализацию
        row['DIRECT_LPU'] = params.get('lpuCode', '')
        # Коды ЛПУ приписки по справочнику фонда
        # (поликлиника + стоматология +ЖК)
        # В строке через 3 пробела. Пример 103   160   121
        row['MASTER'] = forceString(record.value('attachCodeString'))
        # Код ЛПУ пребывания по справочнику фонда
        row['LPU'] = forceString(record.value('hospitalInfisCode'))
        # Номер счета представляемого в фонд
        row['COUNT'] = params.get('accNumber')

        if self.hasWeightActionPropertyType:
            try:
                row['VNOV_D'] = forceInt(self.getActionPropertyValue(
                    params['eventId'], self.actionTypeArrival, u'Вес')) % 10000
            except:
                pass

        relegateOrgTfomsExtCode = forceString(record.value(
            'relegateOrgTfomsExtCode'))
        relegateOrgSmoCode = forceString(record.value('relegateOrgSmoCode'))
        row['DIR_SUBLPU'] = (relegateOrgSmoCode if relegateOrgSmoCode
                         else forceString(record.value('attachOrgSmoCode')))
        row['DIR_TOWN'] = (relegateOrgTfomsExtCode if
            relegateOrgTfomsExtCode else
            forceString(record.value('attachOrgTfomsExtCode')))
        row['SUB_HOSP'] = params.get('accOrgStructInfisCode', '')
        row['TOWN_HOSP'] = params.get('accOrgStructTfomsCode', '')
        row['P_CODE'] = forceString(record.value('execPersonRegionalCode'))
        row['NPR_FDATE'] = pyDate(forceDate(record.value('srcDate')))
        row.store()


    def _processDepRee(self, dbf, record, params):
        u"""Экспорт информации в DEP_REE.DBF"""

        eventOrder = forceInt(record.value('eventOrder'))

        row = dbf.newRecord()
        row['NIB'] = params['eventIdStr'] # Номер истории болезни
        # Код отделения по справочнику фонда
        row['OTD'] = self.getOrgStructureInfisDepTypeCode(params['orgStructId'])
        medicalAidTypeCode = forceString(record.value('medicalAidTypeCode'))
        # Код отделения, откуда переведен пациент
        # (при переводе из одного отделения в другое отделение)
        row['DIRECT_OTD'] = self.getActionPropertyText(params['eventId'],
                        self.actionTypeMovement, u'Переведен из отделения')
        # Признак поступления экстренное – “1” /плановое – “0”
        # / перевод из отделения в отделение – “2”
        row['HOSP'] = '2' if row['DIRECT_OTD'] else '0'

        if row['HOSP'] == '0':
            row['HOSP'] = self.mapEventOrderToHosp.get(eventOrder, '0')

        row['DATAP'] = pyDate(params['begDate']) # Дата поступления  в отделение
        row['DATAV'] = pyDate(params['endDate']) # Дата выписки из отделения

        #'C', 8, Код врача лечившего пациента
        row['DOCTOR'] = forceString(record.value('personCode'))
        # Уровень оказанной помощи (“1”- дневные стационары,
        # “0” – стационары круглосуточного пребывания)
        row['LEVEL'] = '1' if (medicalAidTypeCode == '7'
                or params['exportType'] == self.exportTypeDializ) else '0'
        # 'C',6), № направления на госпитализацию
        row['TALON'] = self.getActionPropertyText(params['eventId'],
              self.actionTypeArrival, u'№ направления на госпитализацию')
        row['VID_HMP'] = self._getQuota(params['eventId'])
        # Код МКБ Х
        row['DIAG'] = forceString(record.value('MKB'))
        # Код диагноза сопутствующего заболевания (состояния) по МКБ-10
        row['DS_S'] = forceString(record.value('accMKB'))
        # Код профиля койки
        row['BED_PROF'] = self.getHospitalBedProfileRegionalCode(
                                                         params['bedProfileId'])
        row['STOIM_S'] = forceDouble(record.value('sum'))
        row['ID_CASE'] = params['rowNumber'] % 10000000000
        self.mapEventIdtoIdCase[params['eventId']] = row['ID_CASE']
        # Результат обращения/ госпитализации
        row['RSLT'] = forceInt(record.value(
            'eventResultRegionalCode')) % 1000
        # рофиль оказанной медицинской помощи
        row['ISHOD'] = forceInt(record.value('resultCode')) %1000
        # Профиль оказанной медицинской помощи
        row['PROFIL'] = (forceInt(params['infisInternalCode']
                            if params['infisInternalCode'] else 0)) % 1000
        # Код специальности медицинского работника, оказавшего услугу
        if self.exportFormat < self.exportFormat2017:
            row['PRVS'] = forceInt(record.value(
                                'specialityFederalCode')) % 1000
        else:
            row['PRVS'] = params['orgStructTfoms'] % 1000

        # Код клинико-статистической группы
        row['KSG'] = forceDouble(record.value('ksgCode'))

        if row['RSLT'] in (102, 103, 104, 202, 203, 204):
            row['DS_P'] = forceString(record.value('accMKB'))


        row['VIDPOM'] = forceInt(record.value(
            'medicalAidKindCode')) % 10000
        row['VID_FIN'] = 1
        row['MOTHER'] = 0 # FIXME
        pMode = 1

        if params['exportType'] == self.exportTypeDS:
            pMode = 6

        row['P_MODE'] = pMode

        if self.exportFormat >= self.exportFormat2017:
            row['P_PER'] = 1
            row['PR_OS'] = 0
            try:
                row['NPL'] = forceInt(self.getActionPropertyValue(
                    params['eventId'], self.actionTypeLeaved, 'NPL')) % 10
            except:
                pass

        if self.exportFormat >= self.exportFormat2018:
            row['ADD_CR_KSG'] = forceString(record.value('criterionCode'))
            row['IDSP'] = getIdsp(forceInt(record.value('medicalAidUnitCode')),
                                  forceBool(record.value('isAttached')),
                                  forceBool(record.value('isHomeVisit')))
        row.store()


    def _processAddInf(self, dbf, record, params):
        u"""Экспорт информации в ADD_INF.DBF"""

        row = dbf.newRecord()

        row['NIB'] = params['eventIdStr'] # Номер истории болезни
        # Место рождения пациента или представителя
        row['MR'] = forceString(record.value('birthPlace'))

        # Код места жительства по ОКАТО
        row['OKATOG'] = forceString(record.value('regRegionOKATO'))
        # Код места пребывания по ОКАТО
        row['OKATOP'] = forceString(record.value('locRegionOKATO'))
        # Код ОКАТО территории страхования по ОМС (по справочнику фонда)
        row['OKATO_OMS'] = forceString(record.value('insurerOKATO'))
        # Статус представителя пациента
        info = self.getClientRepresentativeInfo(params['clientId'])

        # Фамилия родителя (представителя) пациента (п.2 Примечаний)
        if info != {}:
            row['FAMP'] = info.get('lastName', '')
            # Имя родителя (представителя) пациента (п.2 Примечаний)
            row['IMP'] = info.get('firstName', '')
            # Отчество родителя (представителя) пациента (п.2 Примечаний)
            row['OTP'] = info.get('patrName', '')
            # Дата рождения родителя (представителя) пациента
            row['DRP'] = pyDate(info.get('birthDate', QDate()))
            # Пол родителя (представителя) пациента
            row['WP'] = u'М' if info.get('sex', 1) == 1 else u'Ж'

        # Код типа документа, удостоверяющего личность пациента
        # (представителя) (по  справочнику фонда)
        row['C_DOC'] = forceInt(record.value('documentTypeRegionalCode'))
        #, 'C', 9), # Серия документа, удостоверяющего личность пациента
        # (представителя) (по  справочнику фонда)
        row['S_DOC'] = forceString(record.value('documentSerial'))
        # Номер документа, удостоверяющего личность пациента
        # (представителя) (по  справочнику фонда)
        row['N_DOC'] = forceString(record.value('documentNumber'))
        row['NOVOR'] = '' # Признак новорожденного
        specialCase = []
        row['Q_G'] = ' '.join(x for x in specialCase)

        if self.exportFormat >= self.exportFormat2018:
            row['MSE'] = 0

        row.store()


    def _processOper(self, dbf, record, params):
        u"""Экспорт информации в OPER.DBF"""

        row = dbf.newRecord()
        # Номер истории болезни
        row['NIB'] = params['eventIdStr']
        # Порядковый номер случая в реестре
        row['ID_CASE'] = self.mapEventIdtoIdCase.get(params['eventId'], 0)
        # Код отделения, где проведена операция
        row['OTD_O'] = self.getOrgStructureInfisDepTypeCode(
                                                    params['orgStructId'])
        # Дата, когда проведена операция
        row['DATA_O'] = pyDate(params['endDate'])
        # Код операции
        row['OPER'] = forceString(record.value(params['operOperFieldName']))
        # Количество операций
        row['KOL_OPER'] = forceInt(record.value('amount'))
        # Код врача лечившего пациента
        row['DOCTOR'] = forceString(record.value('personCode'))
        # Профиль оказанной медицинской помощи
        row['PROFIL'] = (forceInt(params['infisInternalCode']
                         if params['infisInternalCode'] else 0))% 1000
        # Код специальности медицинского работника, оказавшего услугу
        row['PRVS'] = forceInt(record.value('specialityFederalCode')) % 10000

        if (self.exportFormat >= self.exportFormat2017
            and params['lpuCode'] == '037'): # для МСЧ 118
            forceString(record.value('service'))
            row['PR_OPER'] = (1 if forceString(
                record.value('service')) == u'A16.20.037' else 0)

        row.store()


    def _processDepAdd(self, dbf, record, params):
        u"""Экспорт информации в DEP_ADD.DBF"""

        row = dbf.newRecord()
        # Номер истории болезни
        row['NIB'] = forceString(params['eventId'])
        # Порядковый номер случая в реестре
        row['ID_CASE'] = params['rowNumber'] % 10000000000
        # Дата поступления пациента в отделение
        # интенсивной терапии и  реанимации
        row['DATE_1'] = pyDate(params['begDate'])
        # Дата выбытия пациента из отделения
        # интенсивной терапии и  реанимации
        row['DATE_2'] = pyDate(params['endDate'])
        # Диагноз основной
        row['DS'] = forceString(record.value('MKB'))
        # Код врача лечившего пациента
        row['DOCTOR'] = forceString(record.value('personCode'))
        # Профиль оказанной медицинской помощи
        row['PROFIL'] = params['orgStructInfis'] % 1000
        # Код специальности медицинского работника,
        # оказавшего услугу
        if self.exportFormat < self.exportFormat2017:
            row['PRVS'] = forceInt(record.value('specialityCode')) % 10000
        else:
            row['PRVS'] = params['orgStructTfoms'] % 10000

        row.store()

# ******************************************************************************

class CExportPage2(CAbstractExportPage2):
    u"""Вторая страница мастера экспорта"""

    def __init__(self, parent):
        CAbstractExportPage2.__init__(self, parent, 'ExportR51Hospital')


    def saveExportResults(self):
        fileList = ['REESTR.DBF', 'DEP_REE.DBF', 'DEP_ADD.DBF', 'ADD_INF.DBF']
        exportType = self._parent.page1.cmbExportType.currentIndex()
        exportFormat = self._parent.page1.cmbExportFormat.currentIndex()

        if (exportType != CExportPage1.exportTypeDS or
            exportFormat >= CExportPage1.exportFormat2017):
            fileList.append('OPER.DBF')

        return self.moveFiles(fileList)

if __name__ == '__main__':
    #pylint: disable=wrong-import-position,ungrouped-imports
    from Exchange.Utils import testAccountExport
    #pylint: enable=wrong-import-position,ungrouped-imports
    testAccountExport(exportR51Hospital, 14200)
