# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

u"""Экспорт реестра  в формате XML. Ростовская область"""

from operator import itemgetter

from PyQt4 import QtGui

from Accounting.Tariff import CTariff
from Exchange.Order79Export import (COrder79ExportWizard, COrder79ExportPage1,
        COrder79ExportPage2, COrder79PersonalDataWriter,
        COrder79XmlStreamWriter, ExtendedRecord)

from Exchange.Ui_ExportR08NativePage1 import Ui_ExportPage1

from library.Utils import (forceInt, forceString, forceDate, forceDouble,
    forceRef, forceBool, formatSNILS, toVariant)

def exportR61Native(widget, accountId, accountItemIdList):
    u"""Создает диалог экспорта реестра счета"""

    wizard = CExportWizard(widget)
    wizard.setAccountId(accountId)
    wizard.setAccountItemsIdList(accountItemIdList)
    wizard.exec_()

# выводит в консоль имена невыгруженных полей
DEBUG = False

# ******************************************************************************

class CExportWizard(COrder79ExportWizard):
    u"""Мастер экспорта Ростовская область"""

    def __init__(self, parent=None):
        prefix = 'R61Native'
        title = u'Мастер экспорта реестра услуг для Ростовской области'
        COrder79ExportWizard.__init__(self, title, prefix, CExportPage1,
                                      CExportPage2, parent)

# ******************************************************************************

class CExportPage1(COrder79ExportPage1, Ui_ExportPage1):
    u"""Первая страница мастера экспорта"""
    def __init__(self, parent, prefix):
        xmlWriter = (CExportXmlStreamWriter(self),
                     COrder79PersonalDataWriter(self))
        COrder79ExportPage1.__init__(self, parent, prefix, xmlWriter)

        self.query_fields = {
            # для отладки
            'aiId': 'Account_Item.id',
            'eventId': 'Account_Item.event_id',
            # PACIENT_*
            'PACIENT_ID_PAC': 'Event.client_id',
            'PACIENT_VPOLIS': 'rbPolicyKind.regionalCode',
            'PACIENT_SPOLIS': """CASE
                WHEN (rbPolicyKind.regionalCode LIKE '1') THEN ClientPolicy.serial
                ELSE ''
              END""",
            'PACIENT_NPOLIS': 'ClientPolicy.number',
            'PACIENT_SMO': 'Insurer.miacCode',
            'PACIENT_SMO_OGRN': """CASE
                WHEN (Insurer.miacCode IS NULL OR Insurer.miacCode LIKE '') THEN Insurer.OGRN
                ELSE ''
              END""",
            'PACIENT_SMO_OK': 'Insurer.OKATO',
            'PACIENT_SMO_NAM': """CASE
                WHEN ((Insurer.miacCode IS NULL OR Insurer.miacCode LIKE '') AND (Insurer.OGRN IS NULL OR Insurer.OGRN LIKE '')) THEN Insurer.shortName
                ELSE ''
              END""",
            'PACIENT_STAT_Z_raw': 'rbSocStatusType.code',
            # SLUCH_*
            'SLUCH_IDCASE': 'Account_Item.event_id',
            'SLUCH_USL_OK': 'rbEventTypePurpose.regionalCode',
            'SLUCH_VIDPOM': """CASE
                WHEN (rbService_Profile.medicalAidKind_id IS NOT NULL) THEN ServiceProfileMedicalAidKind.regionalCode
                WHEN (rbService.medicalAidKind_id IS NOT NULL) THEN ServiceMedicalAidKind.regionalCode
                ELSE rbMedicalAidKind.regionalCode
              END""",
            'SLUCH_FOR_POM_raw': 'Event.order',
            'SLUCH_VID_HMP_raw': 'rbCureType.code',
            'SLUCH_METOD_HMP_raw': 'rbCureMethod.code',
            'SLUCH_HMODP': 'rbPatientModel.code',
            'SLUCH_TAL_D': None,
            'SLUCH_TAL_P': None,
            'SLUCH_NPR_MO': 'RelegateOrgId.value',
            'SLUCH_EXTR': None,
            'SLUCH_PODR': None,
            'SLUCH_LPU': 'PersonOrganisation.miacCode',
            'SLUCH_PROFIL': None,
            'SLUCH_DET': """CASE
                WHEN (age(Client.birthDate,Event.setDate) < 18 AND rbSpeciality.code LIKE '22') THEN 1
                ELSE 0
              END""",
            'SLUCH_NHISTORY': None,
            'SLUCH_P_PER': 1,
            'SLUCH_DATE_1': """CAST(CASE
                 WHEN (EventType.form LIKE '110') THEN Event.execDate
                 ELSE Event.setDate
               END AS DATE)""",
            'SLUCH_DATE_2': 'CAST(Event.execDate AS DATE)',
            'SLUCH_DS1': None,
            'SLUCH_DS2': 'DS9.MKB',
            'SLUCH_RSLT': u"""CASE
                WHEN (mes.mrbMESGroup.code LIKE 'ДиспаснС') THEN ''
                ELSE EventResult.federalCode
              END""",
            'SLUCH_ISHOD': u"""CASE
                WHEN (mes.mrbMESGroup.code LIKE 'ДиспаснС') THEN ''
                ELSE rbDiagnosticResult.federalCode
              END""",
            'SLUCH_PRVS': 'rbSpeciality.code',
            'SLUCH_IDDOKT_raw': 'IFNULL(ActionPerson.SNILS, Person.SNILS)',
            'SLUCH_IDSP': 'rbMedicalAidUnit.code',
            'SLUCH_PRZAB': 'IFNULL(rbDiseaseCharacter.code, 0)',
            'SLUCH_SHDZOS': 'DS3.MKB',
            'SLUCH_NAPUCH': 'RelegateOrg.infisCode',
            'SLUCH_NOM_NAP': 'Event.srcNumber',
            'SLUCH_NAPDAT': 'Event.srcDate',
            # USL_*
            'USL_PODR': None,
            'USL_PROFIL': None,
            'USL_PROFIL_KOIKI': None,
            'USL_DET': """CASE
                WHEN (age(Client.birthDate,Event.setDate) < 18 AND rbSpeciality.code LIKE '22') THEN 1
                ELSE 0
              END""",
            'USL_DATE_IN': None,
            'USL_DATE_OUT': None,
            'USL_DS': None,
            'USL_CODE_USL': """CASE
                WHEN (ActionType.serviceType = 4 OR (ActionType.code LIKE 'A%%' OR ActionType.code LIKE 'B%%')) THEN '0'
                ELSE rbService.code
              END """,
            'USL_KOL_USL': u"""CASE
                WHEN (ActionType.serviceType = 4 OR (ActionType.code LIKE 'A%%' OR ActionType.code LIKE 'B%%')) THEN '0'
                WHEN (mes.mrbMESGroup.code LIKE 'ДиспаснС') THEN ''
                ELSE Account_Item.amount
              END """,
            'USL_TARIF': None,
            'USL_SUMV_USL': """CASE
                WHEN (ActionType.serviceType = 4 OR (ActionType.code LIKE 'A%%' OR ActionType.code LIKE 'B%%')) THEN 0
                ELSE Contract_Tariff.price
              END""",
            'USL_SK_KOEF': 0,
            'USL_PRVS': 'rbSpeciality.code',
            'USL_CODE_MD_raw': 'IFNULL(ActionPerson.SNILS, Person.SNILS)',
            'USL_CODE_OPER': None,
            # PERS_*
            'PERS_ID_PAC': 'Event.client_id',
            'PERS_FAM': 'UPPER(Client.lastName)',
            'PERS_IM': 'UPPER(Client.firstName)',
            'PERS_OT': 'UPPER(Client.patrName)',
            'PERS_W': 'Client.sex',
            'PERS_DR': 'Client.birthDate',
            'PERS_MR': 'UPPER(Client.birthPlace)',
            'PERS_DOCTYPE': 'rbDocumentType.regionalCode',
            'PERS_DOCSER': 'ClientDocument.serial',
            'PERS_DOCNUM': 'ClientDocument.number',
            'PERS_SNILS_raw': 'Client.SNILS',
            # разное
            'eventPurposeCode': 'rbEventTypePurpose.regionalCode', # для SLUCH_NSVOD
            'tariffType': 'Contract_Tariff.tariffType',
            'mesGroupCode': 'mes.mrbMESGroup.code',
            'haveMoving': None,
            'serviceCode': None,
            'persAge': None,
            'minboldn': None,
            'isActualPolicy': """IFNULL(ClientPolicy.endDate >= CAST(Event.execDate as date), TRUE)
                AND (ClientPolicy.begDate <= CAST(Event.execDate as date))""",
            'actionEventCsgId': None,
            'eventCsgMKB': None
        }

    def prepare_query_field(self):
        count = len(self.query_fields)
        i = 0
        result_string = ''

        for field, value in self.query_fields.iteritems():
            if value is None:
                result_string += 'NULL'
            elif type(value) is bool:
                result_string += 'TRUE' if value else 'FALSE'
            elif type(value) is int:
                result_string += str(value)
            else:
                result_string += value

            result_string += ' AS ' + field
            i += 1
            if i < count:
                result_string += ', \n'

        return result_string

    def createAmbQuery(self, idList):
        self.query_fields['SLUCH_EXTR'] = "''" # а точно надо ставить "пустышку"?
        self.query_fields['SLUCH_PODR'] = 'SUBSTR(rbService.code, 1, 4)'
        self.query_fields['SLUCH_PROFIL'] = """CASE
            WHEN (rbService_Profile.medicalAidProfile_id IS NOT NULL) THEN ServiceProfileMedicalAidProfile.code
            ELSE ServiceMedicalAidProfile.code
          END"""
        self.query_fields['SLUCH_NHISTORY'] = 'Event.id'
        self.query_fields['SLUCH_DS1'] = 'Diagnosis.MKB'
        self.query_fields['USL_PODR'] = 'SUBSTR(rbService.code, 1, 4)'
        self.query_fields['USL_PROFIL'] = """CASE
            WHEN (rbService_Profile.medicalAidProfile_id IS NOT NULL) THEN ServiceProfileMedicalAidProfile.code
            ELSE ServiceMedicalAidProfile.code
          END"""
        self.query_fields['USL_DATE_IN'] = 'Event.setDate'
        self.query_fields['USL_DATE_OUT'] = 'Event.execDate'
        self.query_fields['USL_DS'] = 'Diagnosis.MKB'
        self.query_fields['USL_TARIF'] = 'IFNULL(Account_Item.sum, -1)'
        self.query_fields['haveMoving'] = False

        return u"""/* -- АМБУЛАТОРИЯ -- */
SELECT %s
FROM Account_Item
  INNER JOIN Event                   ON (Event.id = Account_Item.event_id)
  -- PACIENT_*
  INNER JOIN Client                  ON (Client.id = Event.client_id)
  LEFT  JOIN ClientPolicy            ON (ClientPolicy.id = getClientPolicyIdForDate(Client.`id`, 1, Event.execDate))
  LEFT  JOIN rbPolicyKind            ON (rbPolicyKind.id = ClientPolicy.policyKind_id)
  LEFT  JOIN Organisation AS Insurer ON (Insurer.id = ClientPolicy.insurer_id)
  LEFT  JOIN ClientSocStatus         ON ClientSocStatus.id = (
    SELECT MAX(CSS.id)
    FROM ClientSocStatus CSS
    WHERE CSS.client_id = Client.id  AND CSS.deleted = 0
      AND CSS.socStatusClass_id = (SELECT rbSSC.id FROM rbSocStatusClass AS rbSSC WHERE rbSSC.code = '2' AND rbSSC.group_id IS NULL LIMIT 0,1)
  )
  LEFT  JOIN rbSocStatusType         ON (rbSocStatusType.id = ClientSocStatus.socStatusType_id)
  -- SLUCH_*
  LEFT  JOIN EventType               ON (EventType.id = Event.eventType_id)
  LEFT  JOIN rbEventTypePurpose      ON (rbEventTypePurpose.id = EventType.purpose_id)
  LEFT  JOIN Organisation AS RelegateOrg ON (RelegateOrg.id = Event.relegateOrg_id)
  LEFT  JOIN Organisation_Identification AS RelegateOrgId ON (
    RelegateOrg.id = RelegateOrgId.master_id AND
    RelegateOrgId.deleted = 0 AND
    RelegateOrgId.system_id = (SELECT id FROM rbAccountingSystem WHERE rbAccountingSystem.code = 'f003' LIMIT 0, 1)
  )
  LEFT  JOIN Person                  ON (Person.id = Event.execPerson_id)
  LEFT  JOIN Organisation AS PersonOrganisation ON (PersonOrganisation.id = Person.org_id)
  -- SLUCH_VIDPOM
  LEFT  JOIN rbService               ON (rbService.id = Account_Item.service_id)
  LEFT  JOIN rbService_Profile       ON (rbService_Profile.id = (SELECT MAX(id) FROM rbService_Profile rs WHERE rs.master_id = rbService.id AND rs.speciality_id = Person.speciality_id))
  LEFT  JOIN rbMedicalAidKind AS ServiceProfileMedicalAidKind ON (ServiceProfileMedicalAidKind.id = rbService_Profile.medicalAidKind_id)
  LEFT  JOIN rbMedicalAidKind AS ServiceMedicalAidKind ON (ServiceMedicalAidKind.id = rbService.medicalAidKind_id)
  LEFT  JOIN rbMedicalAidKind        ON (rbMedicalAidKind.id = EventType.medicalAidKind_id)
  --
  LEFT  JOIN rbMedicalAidProfile AS ServiceProfileMedicalAidProfile ON (ServiceProfileMedicalAidProfile.id = rbService_Profile.medicalAidProfile_id)
  LEFT  JOIN rbMedicalAidProfile AS ServiceMedicalAidProfile ON (ServiceMedicalAidProfile.id = rbService.medicalAidProfile_id)
  LEFT  JOIN rbSpeciality            ON (rbSpeciality.id = Person.speciality_id)
  -- Диагнозы
  LEFT  JOIN Diagnostic              ON (Diagnostic.id = (
    SELECT MAX(dc.id)
    FROM Diagnostic dc
    WHERE dc.event_id = Account_Item.event_id AND dc.deleted = 0
      AND dc.diagnosisType_id IN (SELECT id FROM rbDiagnosisType WHERE code IN ('1', '2')) -- заключительный, основной
  ))
  LEFT  JOIN Diagnosis               ON (Diagnosis.id = Diagnostic.diagnosis_id AND Diagnosis.deleted = 0)
  LEFT  JOIN Diagnosis AS DS9        ON (DS9.id = (
    SELECT MAX(ds.id)
    FROM Diagnosis ds
      INNER JOIN Diagnostic dc ON (ds.id = dc.diagnosis_id AND ds.deleted = 0 AND dc.deleted = 0)
    WHERE dc.diagnosisType_id IN (SELECT id FROM rbDiagnosisType WHERE code LIKE '9') -- сопутствующий
      AND dc.event_id = Account_Item.event_id
  ))
  LEFT JOIN Diagnosis AS DS3         ON (DS3.id = (
    SELECT MAX(ds.id)
    FROM Diagnosis ds
      INNER JOIN Diagnostic dc ON (ds.id = dc.diagnosis_id AND ds.deleted = 0 AND dc.deleted = 0)
    WHERE dc.diagnosisType_id IN (SELECT id FROM rbDiagnosisType WHERE code LIKE '3') -- осложнение основного
      AND dc.event_id = Account_Item.event_id
  ))
  --
  LEFT  JOIN mes.MES                 ON (mes.MES.id = Event.MES_id)
  LEFT  JOIN mes.mrbMESGroup         ON (mes.mrbMESGroup.id = mes.MES.group_id)
  LEFT  JOIN rbResult AS EventResult ON (EventResult.id = Event.result_id)
  LEFT  JOIN rbDiagnosticResult      ON (rbDiagnosticResult.id = Diagnostic.result_id)
  LEFT  JOIN rbMedicalAidUnit        ON (rbMedicalAidUnit.id = IFNULL(Account_Item.unit_id, (
    SELECT AI.unit_id FROM Account_Item AI WHERE AI.event_id = Event.id AND AI.unit_id IS NOT NULL LIMIT 0,1
  )))
  LEFT  JOIN rbDiseaseCharacter      ON (rbDiseaseCharacter.id = Diagnosis.character_id)
  LEFT  JOIN Action                  ON (Action.id = Account_Item.action_id)
  LEFT  JOIN ActionType              ON (ActionType.id = Action.actionType_id)
  LEFT  JOIN Contract_Tariff         ON (Contract_Tariff.id = Account_Item.tariff_id)
  LEFT  JOIN rbCureType              ON (rbCureType.id = Event.cureType_id)
  LEFT  JOIN rbCureMethod            ON (rbCureMethod.id = Event.cureMethod_id)
  LEFT  JOIN rbPatientModel          ON (rbPatientModel.id = Event.patientModel_id)
  LEFT  JOIN Person AS ActionPerson  ON (ActionPerson.id = Action.person_id)
  -- PERS_*
  LEFT  JOIN ClientDocument          ON (ClientDocument.client_id = Client.id AND ClientDocument.id = (
    SELECT MAX(CD.id)
    FROM ClientDocument AS CD
      LEFT JOIN rbDocumentType AS rbDT ON rbDT.id = CD.documentType_id
      LEFT JOIN rbDocumentTypeGroup AS rbDTG ON rbDTG.id = rbDT.group_id
    WHERE rbDTG.code = '1' AND CD.client_id = Client.id AND CD.deleted=0))
  LEFT  JOIN rbDocumentType          ON (rbDocumentType.id = ClientDocument.documentType_id)
  -- разное
  LEFT JOIN rbPayRefuseType          ON (rbPayRefuseType.id = Account_Item.refuseType_id)
WHERE rbEventTypePurpose.regionalCode = '3' -- Амбулатория
AND Account_Item.reexposeItem_id IS NULL
AND (Account_Item.date IS NULL OR (Account_Item.date IS NOT NULL AND rbPayRefuseType.rerun != 0))
AND %s\n""" % (self.prepare_query_field(), idList)

    def createStacQuery(self, idList):
        self.query_fields['SLUCH_EXTR'] = """CASE
            WHEN (Event.`order` IN (2,6)) THEN '2'
            ELSE '1'
          END """
        self.query_fields['SLUCH_PODR'] = 'sluchPP.podr'
        self.query_fields['SLUCH_PROFIL'] = 'IFNULL(sluchProfil2.value,sluchPP.profil)'
        self.query_fields['SLUCH_NHISTORY'] = 'Event.externalId'
        self.query_fields['SLUCH_DS1'] = """(SELECT ds.MKB
          FROM Diagnosis ds
            INNER JOIN Diagnostic dc ON (ds.id = dc.diagnosis_id AND ds.deleted = 0 AND dc.deleted = 0)
            INNER JOIN rbDiagnosisType dt on (dc.diagnosisType_id=dt.id)
          WHERE (dt.code = '1' OR dt.code = '2')
            AND dc.event_id = Account_Item.event_id order by -dt.code desc limit 1)"""
        self.query_fields['USL_PODR'] = 'uslPP.podr'
        self.query_fields['USL_PROFIL'] = 'IFNULL(sluchProfil2.value,uslPP.profil)' # странная просьба...
        self.query_fields['USL_PROFIL_KOIKI'] = 'uslPP.tfomsCode'
        self.query_fields['USL_DATE_IN'] = 'CAST(IFNULL(Action.begDate, CsgAction.begDate) AS DATE)'
        self.query_fields['USL_DATE_OUT'] = 'CAST(IFNULL(Action.endDate, CsgAction.endDate) AS DATE)'
        self.query_fields['USL_DS'] = """CASE
            WHEN (ActionType.serviceType = 4 OR (ActionType.code LIKE 'A%%' OR ActionType.code LIKE 'B%%')) THEN
              CASE
                WHEN (uslPP.MKB LIKE '') THEN Diagnosis.MKB
                ELSE uslPP.MKB
              END
            ELSE IFNULL(
              (CASE
                WHEN (uslPP.MKB LIKE '') THEN Diagnosis.MKB
                ELSE uslPP.MKB
              END), CsgAction.MKB)
          END"""
        self.query_fields['USL_TARIF'] = """CASE
            WHEN (ActionType.serviceType = 4 OR (ActionType.code LIKE 'A%%' OR ActionType.code LIKE 'B%%')) THEN 0
            ELSE Contract_Tariff.price
          END"""
        self.query_fields['USL_CODE_OPER'] = """CASE
            WHEN (ActionType.serviceType = 4 OR (ActionType.code LIKE 'A%%' OR ActionType.code LIKE 'B%%')) THEN rbService.code
            ELSE NULL
          END"""
        self.query_fields['haveMoving'] = """EXISTS(
            SELECT NULL FROM Action AS A LEFT JOIN ActionType as AT on AT.id = A.actionType_id
            WHERE A.event_id = Event.id AND AT.flatCode = 'moving' AND A.deleted = 0 LIMIT 1
          )"""
        self.query_fields['serviceCode'] = 'rbService.code'
        self.query_fields['persAge'] = """TIMESTAMPDIFF(year,
            Client.birthDate,
            CASE
              WHEN (EventType.form = '110') THEN Event.execDate
              ELSE Event.setDate
            END)"""
        self.query_fields['minboldn'] = 'cce.sum'
        self.query_fields['actionEventCsgId'] = 'Action.eventCSG_id'
        self.query_fields['eventCsgMKB'] = 'Event_CSG.MKB'

        return u"""/* -- СТАЦИОНАР (без ВМП) -- */
SELECT %s
FROM Account_Item
  INNER JOIN Event                   ON (Event.id = Account_Item.event_id)
  -- PACIENT_*
  INNER JOIN Client                  ON (Client.id = Event.client_id)
  LEFT  JOIN ClientPolicy            ON (ClientPolicy.id = getClientPolicyIdForDate(Client.`id`, 1, Event.execDate))
  LEFT  JOIN rbPolicyKind            ON (rbPolicyKind.id = ClientPolicy.policyKind_id)
  LEFT  JOIN Organisation AS Insurer ON (Insurer.id = ClientPolicy.insurer_id)
  LEFT  JOIN ClientSocStatus         ON ClientSocStatus.id = (
    SELECT MAX(CSS.id)
    FROM ClientSocStatus CSS
    WHERE CSS.client_id = Client.id  AND CSS.deleted = 0
      AND CSS.socStatusClass_id = (SELECT rbSSC.id FROM rbSocStatusClass AS rbSSC WHERE rbSSC.code = '2' AND rbSSC.group_id IS NULL LIMIT 0,1)
  )
  LEFT  JOIN rbSocStatusType         ON (rbSocStatusType.id = ClientSocStatus.socStatusType_id)
  -- SLUCH_*
  LEFT  JOIN EventType               ON (EventType.id = Event.eventType_id)
  LEFT  JOIN rbEventTypePurpose      ON (rbEventTypePurpose.id = EventType.purpose_id AND rbEventTypePurpose.regionalCode = '1')
  LEFT  JOIN Organisation AS RelegateOrg ON (RelegateOrg.id = Event.relegateOrg_id)
  LEFT  JOIN Organisation_Identification AS RelegateOrgId ON (
    RelegateOrg.id = RelegateOrgId.master_id AND
    RelegateOrgId.deleted = 0 AND
    RelegateOrgId.system_id = (SELECT id FROM rbAccountingSystem WHERE rbAccountingSystem.code = 'f003' LIMIT 0, 1)
  )
  LEFT  JOIN Person                  ON (Person.id = Event.execPerson_id)
  LEFT  JOIN Organisation AS PersonOrganisation ON (PersonOrganisation.id = Person.org_id)
  -- SLUCH_VIDPOM
  LEFT  JOIN rbService               ON (rbService.id = Account_Item.service_id)
  LEFT  JOIN rbService_Profile       ON (rbService_Profile.id = (SELECT MAX(id) FROM rbService_Profile rs WHERE rs.master_id = rbService.id AND rs.speciality_id = Person.speciality_id))
  LEFT  JOIN rbMedicalAidKind AS ServiceProfileMedicalAidKind ON (ServiceProfileMedicalAidKind.id = rbService_Profile.medicalAidKind_id)
  LEFT  JOIN rbMedicalAidKind AS ServiceMedicalAidKind ON (ServiceMedicalAidKind.id = rbService.medicalAidKind_id)
  LEFT  JOIN rbMedicalAidKind        ON (rbMedicalAidKind.id = EventType.medicalAidKind_id)
  --
  LEFT  JOIN rbMedicalAidProfile AS ServiceProfileMedicalAidProfile ON (ServiceProfileMedicalAidProfile.id = rbService_Profile.medicalAidProfile_id)
  LEFT  JOIN rbMedicalAidProfile AS ServiceMedicalAidProfile ON (ServiceMedicalAidProfile.id = rbService.medicalAidProfile_id)
  LEFT  JOIN rbSpeciality            ON (rbSpeciality.id = Person.speciality_id)
  -- Диагнозы
  LEFT  JOIN Diagnostic              ON (Diagnostic.id = (
    SELECT MAX(dc.id)
    FROM Diagnostic dc
    WHERE dc.event_id = Account_Item.event_id AND dc.deleted = 0
      AND dc.diagnosisType_id IN (SELECT id FROM rbDiagnosisType WHERE code IN ('1', '2')) -- заключительный, основной
  ))
  LEFT  JOIN Diagnosis               ON (Diagnosis.id = Diagnostic.diagnosis_id AND Diagnosis.deleted = 0)
  LEFT  JOIN Diagnosis AS DS9        ON (DS9.id = (
    SELECT MAX(ds.id)
    FROM Diagnosis ds
      INNER JOIN Diagnostic dc ON (ds.id = dc.diagnosis_id AND ds.deleted = 0 AND dc.deleted = 0)
    WHERE dc.diagnosisType_id IN (SELECT id FROM rbDiagnosisType WHERE code LIKE '9') -- сопутствующий
      AND dc.event_id = Account_Item.event_id
  ))
  LEFT JOIN Diagnosis AS DS3         ON (DS3.id = (
    SELECT MAX(ds.id)
    FROM Diagnosis ds
      INNER JOIN Diagnostic dc ON (ds.id = dc.diagnosis_id AND ds.deleted = 0 AND dc.deleted = 0)
    WHERE dc.diagnosisType_id IN (SELECT id FROM rbDiagnosisType WHERE code LIKE '3') -- осложнение основного
      AND dc.event_id = Account_Item.event_id
  ))
  --
  LEFT  JOIN mes.MES                 ON (mes.MES.id = Event.MES_id)
  LEFT  JOIN mes.mrbMESGroup         ON (mes.mrbMESGroup.id = mes.MES.group_id)
  LEFT  JOIN rbResult AS EventResult ON (EventResult.id = Event.result_id)
  LEFT  JOIN rbDiagnosticResult      ON (rbDiagnosticResult.id = Diagnostic.result_id)
  LEFT  JOIN rbMedicalAidUnit        ON (rbMedicalAidUnit.id = IFNULL(Account_Item.unit_id, (
    SELECT AI.unit_id FROM Account_Item AI WHERE AI.event_id = Event.id AND AI.unit_id IS NOT NULL LIMIT 0,1
  )))
  LEFT  JOIN rbDiseaseCharacter      ON (rbDiseaseCharacter.id = Diagnosis.character_id)
  LEFT  JOIN Action                  ON (Action.id = Account_Item.action_id)
  LEFT  JOIN ActionType              ON (ActionType.id = Action.actionType_id)
  LEFT  JOIN Contract_Tariff         ON (Contract_Tariff.id = Account_Item.tariff_id)
  LEFT  JOIN rbCureType              ON (rbCureType.id = Event.cureType_id)
  LEFT  JOIN rbCureMethod            ON (rbCureMethod.id = Event.cureMethod_id)
  LEFT  JOIN rbPatientModel          ON (rbPatientModel.id = Event.patientModel_id)
  LEFT  JOIN Person AS ActionPerson  ON (ActionPerson.id = Action.person_id)
  -- Подразделение/Профиль для SLUCH
  LEFT  JOIN (SELECT al.event_id, CONCAT(osi1.value, osi2.value) AS 'podr', map.code as 'profil', al.propOrgStructure_id AS 'alOrgStrId'
              FROM vActionLeaved al
              -- PROFIL
              INNER JOIN rbHospitalBedProfile hbp ON (al.propHospitalBedProfle_id=hbp.id)
              INNER JOIN rbMedicalAidProfile map ON (hbp.medicalAidProfile_id=map.id)
              -- PODR
              INNER JOIN OrgStructure_Identification osi1 ON (al.propOrgStructure_id = osi1.master_id)
              INNER JOIN rbAccountingSystem ast1 ON (osi1.system_id=ast1.id AND ast1.code like 'PODR')
              -- N_OTD
              INNER JOIN OrgStructure_Identification osi2 ON (al.propOrgStructure_id = osi2.master_id)
              INNER JOIN rbAccountingSystem ast2 ON (osi2.system_id=ast2.id AND ast2.code like 'N_OTD')
  ) AS sluchPP                       ON (sluchPP.event_id=Account_Item.event_id)
  -- для исключительных значений PROFIL
  LEFT  JOIN (SELECT osi.master_id AS osiMasterId, osi.value AS value
              FROM OrgStructure_Identification osi
              INNER JOIN rbAccountingSystem ast ON (ast.id=osi.system_id AND ast.code LIKE 'PROFIL')
  ) AS sluchProfil2                  ON (sluchProfil2.osiMasterId=sluchPP.alOrgStrId)
  -- USL_*
  LEFT JOIN Action AS CsgAction      ON (CsgAction.eventCSG_id = Account_Item.eventCSG_id)
  -- Подразделение/Профиль для USL
  LEFT  JOIN (SELECT am.event_id, CONCAT(osi1.value, osi2.value) AS 'podr', map.code as 'profil', am.MKB, am.begDate, am.endDate,
                     hbp.tfomsCode
              FROM vActionMoving am
              -- PROFIL
              INNER JOIN rbHospitalBedProfile hbp ON (am.propHospitalBedProfle_id=hbp.id)
              INNER JOIN rbMedicalAidProfile map ON (hbp.medicalAidProfile_id=map.id)
              -- PODR
              INNER JOIN OrgStructure_Identification osi1 ON (am.propOrgStructure_id = osi1.master_id)
              INNER JOIN rbAccountingSystem ast1 ON (osi1.system_id=ast1.id AND ast1.code like 'PODR')
              -- N_OTD
              INNER JOIN OrgStructure_Identification osi2 ON (am.propOrgStructure_id = osi2.master_id)
              INNER JOIN rbAccountingSystem ast2 ON (osi2.system_id=ast2.id AND ast2.code like 'N_OTD')
  ) AS uslPP                         ON (uslPP.event_id = Account_Item.event_id
                                         AND (DATE_FORMAT(IFNULL(Action.begDate, CsgAction.begDate), '%%Y-%%m-%%d %%H:%%i') >= DATE_FORMAT(uslPP.begDate, '%%Y-%%m-%%d %%H:%%i')
                                              AND DATE_FORMAT(IFNULL(IFNULL(Action.endDate, CsgAction.endDate),Event.execDate), '%%Y-%%m-%%d %%H:%%i') <= DATE_FORMAT(uslPP.endDate, '%%Y-%%m-%%d %%H:%%i')))
  -- PERS_*
  LEFT  JOIN ClientDocument          ON (ClientDocument.client_id = Client.id AND ClientDocument.id = (
    SELECT MAX(CD.id)
    FROM ClientDocument AS CD
      LEFT JOIN rbDocumentType AS rbDT ON rbDT.id = CD.documentType_id
      LEFT JOIN rbDocumentTypeGroup AS rbDTG ON rbDTG.id = rbDT.group_id
    WHERE rbDTG.code = '1' AND CD.client_id = Client.id AND CD.deleted=0))
  LEFT  JOIN rbDocumentType          ON (rbDocumentType.id = ClientDocument.documentType_id)
  -- разное
  LEFT  JOIN rbPayRefuseType          ON (rbPayRefuseType.id = Account_Item.refuseType_id)
  LEFT  JOIN ActionType AS CsgActionType ON (CsgActionType.id = CsgAction.actionType_id)
  LEFT  JOIN Contract_CompositionExpense cce ON (cce.master_id = Contract_Tariff.id AND cce.rbTable_id = 10)
  LEFT  JOIN Event_CSG                ON (Event_CSG.id = Action.eventCSG_id)
WHERE -- Стационар
Account_Item.reexposeItem_id IS NULL
AND (Account_Item.date IS NULL OR (Account_Item.date IS NOT NULL AND rbPayRefuseType.rerun != 0))
AND ((Account_Item.eventCSG_id IS NOT NULL AND CsgActionType.flatCode LIKE 'moving')
     OR (Account_Item.eventCSG_id IS NULL AND ActionType.flatCode NOT LIKE 'moving'))
AND %s
HAVING SLUCH_VIDPOM NOT LIKE '32' -- не ВМП
  AND (CASE 
         WHEN actionEventCsgId IS NOT NULL THEN (eventCsgMKB = USL_DS)
         ELSE TRUE
       END)\n""" % (self.prepare_query_field(), idList)

    def createStacVmpQuery(self, idList):
        self.query_fields['SLUCH_TAL_D'] = "DATE_FORMAT(talonVMP_D.value, '%Y-%m-%d')"
        self.query_fields['SLUCH_TAL_P'] = "DATE_FORMAT(talonVMP_P.value, '%Y-%m-%d')"
        self.query_fields['USL_DATE_IN'] = 'CAST(IFNULL(Action.begDate,Event.setDate) AS DATE)'
        self.query_fields['USL_DATE_OUT'] = 'CAST(IFNULL(Action.endDate,Event.execDate) AS DATE)'
        self.query_fields['USL_DS'] = 'Diagnosis.MKB'
        self.query_fields['USL_TARIF'] = 'Contract_Tariff.price'
        self.query_fields['actionEventCsgId'] = None
        self.query_fields['eventCsgMKB'] = None

        return u"""/* -- СТАЦИОНАР (ВМП) -- */
SELECT DISTINCT %s
FROM Account_Item
  INNER JOIN Event                   ON (Event.id = Account_Item.event_id)
  -- PACIENT_*
  INNER JOIN Client                  ON (Client.id = Event.client_id)
  LEFT  JOIN ClientPolicy            ON (ClientPolicy.id = getClientPolicyIdForDate(Client.`id`, 1, Event.execDate))
  LEFT  JOIN rbPolicyKind            ON (rbPolicyKind.id = ClientPolicy.policyKind_id)
  LEFT  JOIN Organisation AS Insurer ON (Insurer.id = ClientPolicy.insurer_id)
  LEFT  JOIN ClientSocStatus         ON ClientSocStatus.id = (
    SELECT MAX(CSS.id)
    FROM ClientSocStatus CSS
    WHERE CSS.client_id = Client.id  AND CSS.deleted = 0
      AND CSS.socStatusClass_id = (SELECT rbSSC.id FROM rbSocStatusClass AS rbSSC WHERE rbSSC.code = '2' AND rbSSC.group_id IS NULL LIMIT 0,1)
  )
  LEFT  JOIN rbSocStatusType         ON (rbSocStatusType.id = ClientSocStatus.socStatusType_id)
  -- SLUCH_*
  LEFT  JOIN EventType               ON (EventType.id = Event.eventType_id)
  LEFT  JOIN rbEventTypePurpose      ON (rbEventTypePurpose.id = EventType.purpose_id AND rbEventTypePurpose.regionalCode = '1')
  LEFT  JOIN Organisation AS RelegateOrg ON (RelegateOrg.id = Event.relegateOrg_id)
  LEFT  JOIN Organisation_Identification AS RelegateOrgId ON (
    RelegateOrg.id = RelegateOrgId.master_id AND
    RelegateOrgId.deleted = 0 AND
    RelegateOrgId.system_id = (SELECT id FROM rbAccountingSystem WHERE rbAccountingSystem.code = 'f003' LIMIT 0, 1)
  )
  LEFT  JOIN Person                  ON (Person.id = Event.execPerson_id)
  LEFT  JOIN Organisation AS PersonOrganisation ON (PersonOrganisation.id = Person.org_id)
  -- SLUCH_VIDPOM
  LEFT  JOIN rbService               ON (rbService.id = Account_Item.service_id)
  LEFT  JOIN rbService_Profile       ON (rbService_Profile.id = (SELECT MAX(id) FROM rbService_Profile rs WHERE rs.master_id = rbService.id AND rs.speciality_id = Person.speciality_id))
  LEFT  JOIN rbMedicalAidKind AS ServiceProfileMedicalAidKind ON (ServiceProfileMedicalAidKind.id = rbService_Profile.medicalAidKind_id)
  LEFT  JOIN rbMedicalAidKind AS ServiceMedicalAidKind ON (ServiceMedicalAidKind.id = rbService.medicalAidKind_id)
  LEFT  JOIN rbMedicalAidKind        ON (rbMedicalAidKind.id = EventType.medicalAidKind_id)
  --
  LEFT  JOIN rbMedicalAidProfile AS ServiceProfileMedicalAidProfile ON (ServiceProfileMedicalAidProfile.id = rbService_Profile.medicalAidProfile_id)
  LEFT  JOIN rbMedicalAidProfile AS ServiceMedicalAidProfile ON (ServiceMedicalAidProfile.id = rbService.medicalAidProfile_id)
  LEFT  JOIN rbSpeciality            ON (rbSpeciality.id = Person.speciality_id)
  -- Диагнозы
  LEFT  JOIN Diagnostic              ON (Diagnostic.id = (
    SELECT MAX(dc.id)
    FROM Diagnostic dc
    WHERE dc.event_id = Account_Item.event_id AND dc.deleted = 0
      AND dc.diagnosisType_id IN (SELECT id FROM rbDiagnosisType WHERE code IN ('1', '2')) -- заключительный, основной
  ))
  LEFT  JOIN Diagnosis               ON (Diagnosis.id = Diagnostic.diagnosis_id AND Diagnosis.deleted = 0)
  LEFT  JOIN Diagnosis AS DS9        ON (DS9.id = (
    SELECT MAX(ds.id)
    FROM Diagnosis ds
      INNER JOIN Diagnostic dc ON (ds.id = dc.diagnosis_id AND ds.deleted = 0 AND dc.deleted = 0)
    WHERE dc.diagnosisType_id IN (SELECT id FROM rbDiagnosisType WHERE code LIKE '9') -- сопутствующий
      AND dc.event_id = Account_Item.event_id
  ))
  LEFT JOIN Diagnosis AS DS3         ON (DS3.id = (
    SELECT MAX(ds.id)
    FROM Diagnosis ds
      INNER JOIN Diagnostic dc ON (ds.id = dc.diagnosis_id AND ds.deleted = 0 AND dc.deleted = 0)
    WHERE dc.diagnosisType_id IN (SELECT id FROM rbDiagnosisType WHERE code LIKE '3') -- осложнение основного
      AND dc.event_id = Account_Item.event_id
  ))
  --
  LEFT  JOIN mes.MES                 ON (mes.MES.id = Event.MES_id)
  LEFT  JOIN mes.mrbMESGroup         ON (mes.mrbMESGroup.id = mes.MES.group_id)
  LEFT  JOIN rbResult AS EventResult ON (EventResult.id = Event.result_id)
  LEFT  JOIN rbDiagnosticResult      ON (rbDiagnosticResult.id = Diagnostic.result_id)
  LEFT  JOIN rbMedicalAidUnit        ON (rbMedicalAidUnit.id = IFNULL(Account_Item.unit_id, (
    SELECT AI.unit_id FROM Account_Item AI WHERE AI.event_id = Event.id AND AI.unit_id IS NOT NULL LIMIT 0,1
  )))
  LEFT  JOIN rbDiseaseCharacter      ON (rbDiseaseCharacter.id = Diagnosis.character_id)
  LEFT  JOIN Action                  ON (Action.id = Account_Item.action_id)
  LEFT  JOIN ActionType              ON (ActionType.id = Action.actionType_id)
  LEFT  JOIN Contract_Tariff         ON (Contract_Tariff.id = Account_Item.tariff_id)
  LEFT  JOIN rbCureType              ON (rbCureType.id = Event.cureType_id)
  LEFT  JOIN rbCureMethod            ON (rbCureMethod.id = Event.cureMethod_id)
  LEFT  JOIN rbPatientModel          ON (rbPatientModel.id = Event.patientModel_id)
  LEFT  JOIN Person AS ActionPerson  ON (ActionPerson.id = Action.person_id)
  -- Подразделение/Профиль для SLUCH
  LEFT  JOIN (SELECT al.event_id, CONCAT(osi1.value, osi2.value) AS 'podr', map.code as 'profil', al.propOrgStructure_id AS 'alOrgStrId'
              FROM vActionLeaved al
              -- PROFIL
              INNER JOIN rbHospitalBedProfile hbp ON (al.propHospitalBedProfle_id=hbp.id)
              INNER JOIN rbMedicalAidProfile map ON (hbp.medicalAidProfile_id=map.id)
              -- PODR
              INNER JOIN OrgStructure_Identification osi1 ON (al.propOrgStructure_id = osi1.master_id)
              INNER JOIN rbAccountingSystem ast1 ON (osi1.system_id=ast1.id AND ast1.code like 'PODR')
              -- N_OTD
              INNER JOIN OrgStructure_Identification osi2 ON (al.propOrgStructure_id = osi2.master_id)
              INNER JOIN rbAccountingSystem ast2 ON (osi2.system_id=ast2.id AND ast2.code like 'N_OTD')
  ) AS sluchPP                       ON (sluchPP.event_id=Account_Item.event_id)
  -- для исключительных значений PROFIL
  LEFT  JOIN (SELECT osi.master_id AS osiMasterId, osi.value AS value
              FROM OrgStructure_Identification osi
              INNER JOIN rbAccountingSystem ast ON (ast.id=osi.system_id AND ast.code LIKE 'PROFIL')
  ) AS sluchProfil2                  ON (sluchProfil2.osiMasterId=sluchPP.alOrgStrId)
  -- TAL_D
  LEFT JOIN (SELECT * FROM (SELECT Action.event_id, ActionProperty_Date.value FROM Action
               INNER JOIN ActionType ON (ActionType.id=Action.actionType_id AND ActionType.code LIKE 'talonVMP')
               INNER JOIN ActionPropertyType ON (ActionPropertyType.actionType_id=ActionType.id AND ActionPropertyType.name LIKE 'Дата выдачи талона на ВМП')
               INNER JOIN ActionProperty ON (ActionProperty.type_id=ActionPropertyType.id AND ActionProperty.action_id=Action.id)
               INNER JOIN ActionProperty_Date ON (ActionProperty_Date.id=ActionProperty.id)
             WHERE Action.deleted=0
             ORDER BY Action.event_id, ActionProperty_Date.value DESC) AS tabTalonD GROUP BY tabTalonD.event_id
  ) AS talonVMP_D                    ON (talonVMP_D.event_id=Account_Item.event_id)
  -- TAL_P
  LEFT JOIN (SELECT * FROM (SELECT Action.event_id, ActionProperty_Date.value
             FROM Action
               INNER JOIN ActionType ON (ActionType.id=Action.actionType_id AND ActionType.code LIKE 'talonVMP')
               INNER JOIN ActionPropertyType ON (ActionPropertyType.actionType_id=ActionType.id AND ActionPropertyType.name LIKE 'Дата планируемой госпитализации')
               INNER JOIN ActionProperty ON (ActionProperty.type_id=ActionPropertyType.id AND ActionProperty.action_id=Action.id)
               INNER JOIN ActionProperty_Date ON (ActionProperty_Date.id=ActionProperty.id)
             WHERE Action.deleted=0
             ORDER BY Action.event_id, ActionProperty_Date.value DESC) AS tabTalonP GROUP BY tabTalonP.event_id
  ) AS talonVMP_P                    ON (talonVMP_P.event_id=Account_Item.event_id)
  -- USL_*
  LEFT JOIN Action AS CsgAction      ON (CsgAction.eventCSG_id = Account_Item.eventCSG_id)
  -- Подразделение/Профиль для USL
  LEFT  JOIN (SELECT am.event_id, CONCAT(osi1.value, osi2.value) AS 'podr', map.code as 'profil', am.MKB, am.begDate, am.endDate,
                     hbp.tfomsCode
              FROM vActionMoving am
              -- PROFIL
              INNER JOIN rbHospitalBedProfile hbp ON (am.propHospitalBedProfle_id=hbp.id)
              INNER JOIN rbMedicalAidProfile map ON (hbp.medicalAidProfile_id=map.id)
              -- PODR
              INNER JOIN OrgStructure_Identification osi1 ON (am.propOrgStructure_id = osi1.master_id)
              INNER JOIN rbAccountingSystem ast1 ON (osi1.system_id=ast1.id AND ast1.code like 'PODR')
              -- N_OTD
              INNER JOIN OrgStructure_Identification osi2 ON (am.propOrgStructure_id = osi2.master_id)
              INNER JOIN rbAccountingSystem ast2 ON (osi2.system_id=ast2.id AND ast2.code like 'N_OTD')
  ) AS uslPP                         ON (uslPP.event_id = Account_Item.event_id
                                         AND (CAST(Event.setDate as DATE) >= CAST(uslPP.begDate as DATE)
                                              AND CAST(Event.execDate as DATE) <= CAST(uslPP.endDate as DATE)))
  -- PERS_*
  LEFT  JOIN ClientDocument          ON (ClientDocument.client_id = Client.id AND ClientDocument.id = (
    SELECT MAX(CD.id)
    FROM ClientDocument AS CD
      LEFT JOIN rbDocumentType AS rbDT ON rbDT.id = CD.documentType_id
      LEFT JOIN rbDocumentTypeGroup AS rbDTG ON rbDTG.id = rbDT.group_id
    WHERE rbDTG.code = '1' AND CD.client_id = Client.id AND CD.deleted=0))
  LEFT  JOIN rbDocumentType          ON (rbDocumentType.id = ClientDocument.documentType_id)
  -- разное
  LEFT  JOIN rbPayRefuseType          ON (rbPayRefuseType.id = Account_Item.refuseType_id)
  LEFT  JOIN ActionType AS CsgActionType ON (CsgActionType.id = CsgAction.actionType_id)
  LEFT  JOIN Contract_CompositionExpense cce ON (cce.master_id = Contract_Tariff.id AND cce.rbTable_id = 10)
WHERE -- Стационар
Account_Item.reexposeItem_id IS NULL
AND (Account_Item.date IS NULL OR (Account_Item.date IS NOT NULL AND rbPayRefuseType.rerun != 0))
AND %s
HAVING SLUCH_VIDPOM LIKE '32' -- ВМП\n""" % (self.prepare_query_field(), idList)

    def createQuery(self):
        idList   = self.tableAccountItem['id'].inlist(self.idList)

        sqlAmb = self.createAmbQuery(idList)
        sqlStac = self.createStacQuery(idList)
        sqlStacVmp = self.createStacVmpQuery(idList)

        stmt = u"""SELECT * FROM ( %s UNION ALL %s UNION ALL %s ) AS GigaTable
ORDER BY PACIENT_ID_PAC, eventId, SLUCH_USL_OK, serviceCode""" % (sqlAmb,sqlStac,sqlStacVmp)

        return self.db.query(stmt)


    def preprocessQuery(self, query, params):
        # Некоторые услуги (блоки USL) необходимо объединять между собой по следующим признакам:
        #1. один и тот же МКБ или класс МКБ (первая буква);
        #2. один и тот же КСГ (указан в теге CODE_USL).
        result = {}

        while query.next():
            record = query.record()
            csgActionId = forceRef(record.value('csgActionId'))

            if not csgActionId:
                continue

            mkb = forceString(record.value('USL_DS'))[:1]
            ksg = forceString(record.value('USL_CODE_USL'))
            eventId = forceRef(record.value('SLUCH_IDCASE'))
            dateIn = forceDate(record.value('USL_DATE_IN'))
            key = (eventId, mkb, ksg)
            val = (csgActionId, dateIn)
            if result.has_key(key):
                result[key].append(val)
            else:
                result[key] = [val]

        skipSet = set()
        replaceBegDate = {}
        for val in result.values():
            if len(val) > 1:
                val.sort(key=itemgetter(1))
                (id, begDate) = val[0]
                skipSet.add(id)
                (id, _) = val[-1]
                replaceBegDate[id] = begDate

        query.exec_()
        params['skipSet'] = skipSet
        params['replaceBegDate'] = replaceBegDate

# ******************************************************************************

class CExportPage2(COrder79ExportPage2):
    u"""Вторая страница мастера экспорта"""
    def __init__(self, parent, prefix):
        COrder79ExportPage2.__init__(self, parent, prefix)

# ******************************************************************************

class CExportXmlStreamWriter(COrder79XmlStreamWriter):
    u"""Осуществляет запись данных экспорта в XML"""

    # копированое переменных из родителя во избежании ломания других обменных форматов
    eventDateFields1 = ('DATE_1', 'DATE_2')
    eventDateFields2 = ('NAPDAT',)
    eventFields = ('IDCASE', 'USL_OK', 'VIDPOM', 'FOR_POM', 'VID_HMP',
                   'METOD_HMP', 'HMODP', 'TAL_D', 'TAL_P', 'NPR_MO', 'EXTR', 'PODR', 'LPU',
                   'PROFIL', 'DET', 'NHISTORY', 'P_PER') +  eventDateFields1 + ('DS1',
                   'DS2', 'RSLT', 'ISHOD', 'PRVS', 'VERS_SPEC',
                   'IDDOKT', 'OS_SLUCH', 'IDSP', 'SUMV', 'OPLATA',
                   'NSVOD', 'KODLPU', 'PRNES', 'BOLDN', 'PCHAST', 'PRZAB',
                   'SHDZOS', 'NAPUCH', 'NOM_NAP') + eventDateFields2 + (
                   'CODE_FKSG',  )

    serviceDateFields = ('DATE_IN', 'DATE_OUT')
    serviceFields = (('IDSERV', 'IDMASTER', 'LPU', 'PODR', 'PROFIL', 'PROFIL_KOIKI', 'DET')  +
            serviceDateFields + ('DS', 'CODE_USL', 'KOL_USL', 'TARIF',
            'SUMV_USL', 'SL_KOEF', 'SK_KOEF', 'PRVS', 'CODE_MD', 'KODLPU', 'KSGA', 'KTLPU', 'PKUR',
            'CODE_OPER'))

    def __init__(self, parent):
        COrder79XmlStreamWriter.__init__(self, parent)
        self.sluchDate1 = None
        self.sluchDate1_uslCheck = False
        self.sluchDate2 = None
        self.sluchDate2_uslCheck = False
        self.lastEventId = None
        self.lastUslOk = None
        self.eventGroup = CExportR61EventGroup()

    def checkerClientInfo(self, record):
        clientId = forceInt(record.value('PACIENT_ID_PAC'))
        self.checkElement(clientId, 'VPOLIS', forceString(record.value('PACIENT_VPOLIS')), 1, optional=True, isClientInfo=True)
        # если задействован не действующий полис (кроме временного)...
        if (forceInt(record.value('isActualPolicy')) == 0 and forceInt(record.value('PACIENT_VPOLIS')) != 2):
            self._parent.logWarning(u'Пациент %d: использован недействительный полис на момент обращения' % (clientId))
            # данные заполняются нулями
            record.setValue('PACIENT_SPOLIS', '0000000000')
            record.setValue('PACIENT_NPOLIS', '00000000000000000000')
        else:
            self.checkElement(clientId, 'SPOLIS', forceString(record.value('PACIENT_SPOLIS')), 10, optional=True, isClientInfo=True)
            self.checkElement(clientId, 'NPOLIS', forceString(record.value('PACIENT_NPOLIS')), 20, optional=True, isClientInfo=True)
        self.checkElement(clientId, 'SMO', forceString(record.value('PACIENT_SMO')), 5, optional=True, isClientInfo=True)
        self.checkElement(clientId, 'SMO_OGRN', forceString(record.value('PACIENT_SMO_OGRN')), 15, optional=True, isClientInfo=True)
        # Вопреки спецификации, ТФОМС РО позволяет вносить в этот тег до 11 символов. Поясняли устно ссылаясь на спеку.
        # В спеке ничего подобного не найдено.
        self.checkElement(clientId, 'SMO_OK', forceString(record.value('PACIENT_SMO_OK')), 11, optional=True, isClientInfo=True)
        self.checkElement(clientId, 'SMO_NAM', forceString(record.value('PACIENT_SMO_NAM')), 100, optional=True, isClientInfo=True)
        self.checkElement(clientId, 'NOVOR', forceString(record.value('PACIENT_NOVOR')), 9, isClientInfo=True)
        self.checkElement(clientId, 'STAT_Z', forceString(record.value('PACIENT_STAT_Z')), 1, isClientInfo=True)


    def checkerEventInfo(self, record):
        if self.lastEventId:
            if not self.sluchDate1_uslCheck:
                self._parent.logError(u'Обращение %d%s: дата начала случая не соответствует минимальной дате услуг'
                                      % (self.lastEventId, (u'(С)' if self.lastUslOk == 1 else u'(П)')))
            if not self.sluchDate2_uslCheck:
                self._parent.logError(u'Обращение %d%s: дата окончания случая не соответствует максимальной дате услуг'
                                      % (self.lastEventId, (u'(С)' if self.lastUslOk == 1 else u'(П)')))

        eventId = forceInt(record.value('eventId'))
        self.lastEventId = eventId
        uslOk = forceInt(record.value('SLUCH_USL_OK'))
        self.lastUslOk = uslOk
        suffix = (u'(С)' if uslOk == 1 else u'(П)')
        vidpom = forceString(record.value('SLUCH_VIDPOM'))
        self.checkElement(eventId, 'VIDPOM', vidpom, 4, suffix=suffix)
        if not self.checkElement(eventId, 'FOR_POM', forceString(record.value('SLUCH_FOR_POM')), 1, suffix=suffix):
            record.setValue('SLUCH_FOR_POM', '0') # по просьбе РОКБ: если тег пустой(что является ошибкой), то заполнять его нулем
        if uslOk == 1 and vidpom == '32':
            self.checkElement(eventId, 'VID_HMP', forceString(record.value('SLUCH_VID_HMP')), 12, suffix=suffix)  # должно быть 9, но ТФОМС принимает значение из 12 символов
            self.checkElement(eventId, 'METOD_HMP', forceString(record.value('SLUCH_METOD_HMP')), 3, suffix=suffix)
            self.checkFill (eventId, 'TAL_D', forceString(record.value('SLUCH_TAL_D')), suffix=suffix)
            self.checkFill (eventId, 'TAL_P', forceString(record.value('SLUCH_TAL_P')), suffix=suffix)
        # Тег "NPR_MO" для МО "Министерство здравоохранения Ростовской области" (5610099) не заполняется - это исключение.
        napuch = forceString(record.value('SLUCH_NAPUCH'))
        if (self.checkElement(eventId, 'NAPUCH', napuch, 7, suffix=suffix)):
            if napuch != '5610099':
                self.checkElement(eventId, 'NPR_MO', forceString(record.value('SLUCH_NPR_MO')), 6, suffix=suffix)
        if uslOk == 1:
            self.checkElement(eventId, 'EXTR', forceString(record.value('SLUCH_EXTR')), 2, suffix=suffix)
        if not self.checkElement(eventId, 'PODR', forceString(record.value('SLUCH_PODR')), 8, suffix=suffix):
            record.setValue('SLUCH_PODR', '0')
        if not self.checkElement(eventId, 'LPU', forceString(record.value('SLUCH_LPU')), 6, suffix=suffix):
            record.setValue('SLUCH_LPU', '0')
        self.checkElement(eventId, 'PROFIL', forceString(record.value('SLUCH_PROFIL')), 3, suffix=suffix)
        self.checkElement(eventId, 'DET', forceString(record.value('SLUCH_DET')), 1, suffix=suffix)
        self.checkElement(eventId, 'NHISTORY', forceString(record.value('SLUCH_NHISTORY')), 50, suffix=suffix)
        self.checkElement(eventId, 'P_PER', forceString(record.value('SLUCH_P_PER')), 1, optional=True, suffix=suffix)
        self.checkFill(eventId, 'DATE_1', forceString(record.value('SLUCH_DATE_1')), suffix=suffix)
        self.sluchDate1 = forceDate(record.value('SLUCH_DATE_1')).toPyDate()
        self.sluchDate1_uslCheck = False
        self.checkFill(eventId, 'DATE_2', forceString(record.value('SLUCH_DATE_2')), suffix=suffix)
        self.sluchDate2 = forceDate(record.value('SLUCH_DATE_2')).toPyDate()
        self.sluchDate2_uslCheck = False
        self.checkElement(eventId, 'DS1', forceString(record.value('SLUCH_DS1')), 10, suffix=suffix)
        self.checkElement(eventId, 'DS2', forceString(record.value('SLUCH_DS2')), 10, optional=True, suffix=suffix)
        self.checkElement(eventId, 'RSLT', forceString(record.value('SLUCH_RSLT')), 3, suffix=suffix)
        self.checkElement(eventId, 'ISHOD', forceString(record.value('SLUCH_ISHOD')), 3, suffix=suffix)
        self.checkElement(eventId, 'PRVS', forceString(record.value('SLUCH_PRVS')), 9, suffix=suffix)
        self.checkElement(eventId, 'IDDOKT', forceString(record.value('SLUCH_IDDOKT')), 25, suffix=suffix)
        self.checkElement(eventId, 'OS_SLUCH', forceString(record.value('SLUCH_OS_SLUCH')), 1, optional=True, suffix=suffix)
        self.checkElement(eventId, 'IDSP', forceString(record.value('SLUCH_IDSP')), 2, suffix=suffix)
        self.checkFill(eventId, 'SUMV', forceString(record.value('SLUCH_SUMV')), suffix=suffix)
        self.checkElement(eventId, 'NSVOD', forceString(record.value('SLUCH_NSVOD')), 3, suffix=suffix)
        self.checkElement(eventId, 'KODLPU', forceString(record.value('SLUCH_KODLPU')), 7, suffix=suffix)
        self.checkElement(eventId, 'PRNES', forceString(record.value('SLUCH_PRNES')), 1, optional=True, suffix=suffix)
        if uslOk == 1:
            self.checkElement(eventId, 'BOLDN', forceString(record.value('SLUCH_BOLDN')), 3, suffix=suffix)
        self.checkElement(eventId, 'PCHAST', forceString(record.value('SLUCH_PCHAST')), 1, suffix=suffix)
        self.checkElement(eventId, 'PCHAST', forceString(record.value('SLUCH_PCHAST')), 1, suffix=suffix)
        self.checkElement(eventId, 'PRZAB', forceString(record.value('SLUCH_PRZAB')), 1, suffix=suffix)
        self.checkElement(eventId, 'SHDZOS', forceString(record.value('SLUCH_SHDZOS')), 10, optional=True, suffix=suffix)
        self.checkElement(eventId, 'NOM_NAP', forceString(record.value('SLUCH_NOM_NAP')), 16, suffix=suffix)
        self.checkFill(eventId, 'NAPDAT', forceString(record.value('SLUCH_NAPDAT')), suffix=suffix)
        self.checkElement(eventId, 'CODE_FKSG', forceString(record.value('SLUCH_CODE_FKSG')), 40, optional=True, suffix=suffix)

        self.checkFill(eventId, 'SUMV', forceString(record.value('SLUCH_SUMV')), suffix=suffix)


    def checkerService(self, record):
        uslOk = forceInt(record.value('SLUCH_USL_OK'))
        suffix = (u'(С)' if uslOk == 1 else u'(П)')
        eventId = forceInt(record.value('eventId'))
        idServ = forceInt(record.value('USL_IDSERV'))
        # self.checkElement(eventId, 'IDMASTER', forceString(record.value('USL_IDMASTER')), 36, idServ)
        self.checkElement(eventId, 'LPU', forceString(record.value('USL_LPU')), 6, idServ, suffix=suffix)
        if not self.checkElement(eventId, 'PODR', forceString(record.value('USL_PODR')), 8, idServ, suffix=suffix):
            record.setValue('USL_PODR', '0') # по просьбе РОКБ: если тег пустой(что является ошибкой), то заполнять его нулем
        if not self.checkElement(eventId, 'PROFIL', forceString(record.value('USL_PROFIL')), 3, idServ, suffix=suffix):
            record.setValue('USL_PROFIL', '0')  # (тоже самое, что и выше)
        self.checkElement(eventId, 'DET', forceString(record.value('USL_DET')), 1, idServ, suffix=suffix)
        if self.checkFill(eventId, 'DATE_IN', forceString(record.value('USL_DATE_IN')), idServ, suffix=suffix):
            uslDate = forceDate(record.value('USL_DATE_IN')).toPyDate()
            if uslDate < self.sluchDate1 :
                self._parent.logError(
                    u'Обращение %d%s, услуга %d: дата начала услуги меньше даты начала случая (%s &lt; %s)' % (
                    eventId, suffix, idServ, uslDate, self.sluchDate1))
            elif uslDate > self.sluchDate2:
                self._parent.logError(
                    u'Обращение %d%s, услуга %d: дата начала услуги больше даты окончания случая (%s &gt; %s)' % (
                    eventId, suffix, idServ, uslDate, self.sluchDate2))
            elif not self.sluchDate1_uslCheck and uslDate == self.sluchDate1:
                self.sluchDate1_uslCheck = True
        if self.checkFill(eventId, 'DATE_OUT', forceString(record.value('USL_DATE_OUT')), idServ, suffix=suffix):
            uslDate = forceDate(record.value('USL_DATE_OUT')).toPyDate()
            if uslDate < self.sluchDate1:
                self._parent.logError(
                    u'Обращение %d%s, услуга %d: дата окончания услуги меньше даты начала случая (%s &lt; %s)' % (
                    eventId, suffix, idServ, uslDate, self.sluchDate1))
            elif uslDate > self.sluchDate2:
                self._parent.logError(
                    u'Обращение %d%s, услуга %d: дата окончания услуги больше даты окончания случая (%s &gt; %s)' % (
                    eventId, suffix, idServ, uslDate, self.sluchDate2))
            elif not self.sluchDate2_uslCheck and uslDate == self.sluchDate2:
                self.sluchDate2_uslCheck = True
        self.checkElement(eventId, 'DS', forceString(record.value('USL_DS')), 10, idServ, suffix=suffix)
        self.checkElement(eventId, 'CODE_USL', forceString(record.value('USL_CODE_USL')), 20, idServ, suffix=suffix)
        self.checkElement(eventId, 'KOL_USL', forceString(record.value('USL_KOL_USL')), 6, idServ, suffix=suffix)
        self.checkFill(eventId, 'TARIF', forceString(record.value('USL_TARIF')), idServ, suffix=suffix)
        self.checkFill(eventId, 'SUMV_USL', forceString(record.value('USL_SUMV_USL')), idServ, suffix=suffix)
        self.checkElement(eventId, 'PRVS', forceString(record.value('USL_PRVS')), 9, idServ, suffix=suffix)
        self.checkElement(eventId, 'CODE_MD', forceString(record.value('USL_CODE_MD')), 25, idServ, suffix=suffix)
        self.checkElement(eventId, 'KODLPU', forceString(record.value('USL_KODLPU')), 7, idServ, suffix=suffix)
        self.checkElement(eventId, 'KSGA', forceString(record.value('USL_KODLPU')), 7, idServ, optional=True, suffix=suffix)
        self.checkElement(eventId, 'KTLPU', forceString(record.value('USL_KTLPU')), 2, idServ, optional=True, suffix=suffix)
        self.checkElement(eventId, 'PKUR', forceString(record.value('USL_PKUR')), 1, idServ, suffix=suffix)
        self.checkElement(eventId, 'CODE_OPER', forceString(record.value('USL_CODE_OPER')), 16, idServ, optional=True, suffix=suffix)


    def writeRecord(self, record, params):
        u"""Пишет sql запись в файл"""
        clientId = forceRef(record.value('PACIENT_ID_PAC'))

        # если дата окончания случая за пределами отчетного периода (начала текущего месяца)
        # то данный случай является "дополнительным"
        date2 = forceDate(record.value('SLUCH_DATE_2'))
        if hasattr(self, 'startDate') and date2 < self.startDate:
            prNov = '1'
        else:
            prNov = self.getExposetAccountItem(forceString(record.value('aiId')))

        changedPrNov = (prNov != params.setdefault('lastPrNov'))

        if (clientId != params.setdefault('lastClientId')) or changedPrNov:
            if params['lastClientId'] or params['lastPrNov']:
                self.eventGroup.clear(self)
                self.writeEndElement() # SLUCH
                self.writeEndElement() # ZAP

            self.writeStartElement('ZAP')
            self.writeTextElement('N_ZAP', forceString(params['N_ZAP']))
            self.writeTextElement('PR_NOV', prNov)

            self.getAddr(forceInt(record.value('PACIENT_ID_PAC')), params)
            isAddressError = params['isAddressError']

            if isAddressError:
                self._parent.logError(
                    u'У пациента %s %s %s (%d) не заполнен адрес' %
                    (forceString(record.value('PERS_FAM')),
                     forceString(record.value('PERS_IM')),
                     forceString(record.value('PERS_OT')),
                     clientId))
                self.abort()

            self.writeClientInfo(record, params)

            params['N_ZAP'] += 1
            params['lastClientId'] = clientId
            params['lastPrNov'] = prNov
            params['lastEventId'] = None

        self.writeEventInfo(record, params)
        self.writeService(record, params)


    def getAddr(self, clientId, params):
        stmt = u"""SELECT 
  CASE SUBSTRING(RegKLADR.OCATD, 1, 2) 
    WHEN '60' THEN RegKLADR.OCATD
    WHEN '' THEN ''
    ELSE CONCAT(SUBSTRING(RegKLADR.OCATD, 1, 2), '000000000')
  END AS PERS_OKATOG,
  CASE SUBSTRING(LocKLADR.OCATD, 1, 2) 
    WHEN '60' THEN LocKLADR.OCATD
    WHEN '' THEN ''
    ELSE CONCAT(SUBSTRING(LocKLADR.OCATD, 1, 2), '000000000')
  END AS PERS_OKATOP,
  CASE
    WHEN ((RegAddressHouse.KLADRCode IS NULL OR RegAddressHouse.KLADRCode = '') AND
                (ClientRegAddress.freeInput IS NULL OR ClientRegAddress.freeInput = '')) THEN 1
    ELSE 0
  END AS isAddressError, -- для детектирования отсутствия адреса
  CASE
    WHEN (RegAddressHouse.KLADRStreetCode IS NULL OR RegAddressHouse.KLADRStreetCode LIKE '') THEN ClientRegAddress.freeInput
    ELSE formatClientAddress(ClientRegAddress.id)
  END  AS PERS_ADRES,
  RPAD(
    (CASE
      WHEN (RegAddressHouse.KLADRStreetCode IS NULL OR RegAddressHouse.KLADRStreetCode LIKE '') THEN RegAddressHouse.KLADRCode
      ELSE RegAddressHouse.KLADRStreetCode
    END), 17, '0') AS PERS_KLADR,
  CASE
    WHEN (RegAddressHouse.id) THEN RegAddressHouse.number
    ELSE LocAddressHouse.number
  END AS PERS_DOM,
  CASE
    WHEN (RegAddressHouse.id) THEN RegAddress.flat
    ELSE LocAddress.flat
  END AS PERS_KVART,
  CASE
    WHEN (RegAddressHouse.id) THEN RegAddressHouse.corpus
    ELSE LocAddressHouse.corpus
  END AS PERS_KORP
FROM Client
  LEFT JOIN ClientAddress AS ClientRegAddress ON (ClientRegAddress.id  = getClientRegAddressId(Client.id))
  LEFT JOIN Address AS RegAddress ON (RegAddress.id = ClientRegAddress.address_id)
  LEFT JOIN AddressHouse RegAddressHouse ON (RegAddressHouse.id = RegAddress.house_id)
  LEFT JOIN kladr.KLADR AS RegKLADR ON (RegKLADR.CODE = RegAddressHouse.KLADRCode)

  LEFT JOIN ClientAddress AS ClientLocAddress ON (ClientLocAddress.id  = getClientLocAddressId(Client.id))
  LEFT JOIN Address AS LocAddress ON (LocAddress.id = ClientLocAddress.address_id)
  LEFT JOIN AddressHouse LocAddressHouse ON (LocAddressHouse.id = LocAddress.house_id)
  LEFT JOIN kladr.KLADR AS LocKLADR ON (LocKLADR.CODE = LocAddressHouse.KLADRCode)
WHERE Client.id = %d;""" % (clientId)
        query = QtGui.qApp.db.query(stmt)

        if query.next():
            record = query.record()
            params['PERS_OKATOG'] = forceString(record.value('PERS_OKATOG'));
            params['PERS_OKATOP'] = forceString(record.value('PERS_OKATOP'));
            params['isAddressError'] = forceInt(record.value('isAddressError'));
            params['PERS_ADRES'] = forceString(record.value('PERS_ADRES'));
            params['PERS_KLADR'] = forceString(record.value('PERS_KLADR'));
            params['PERS_DOM'] = forceString(record.value('PERS_DOM'));
            params['PERS_KVART'] = forceString(record.value('PERS_KVART'));
            params['PERS_KORP'] = forceString(record.value('PERS_KORP'));


    def getExposetAccountItem(self, account_id):
        stmt = """SELECT
  CASE
    WHEN (ExposedAccountItem.id IS NULL AND rbPayRefuseType.id IS NULL) THEN 0
    ELSE 1
  END  AS isExposedItem
FROM Account_Item
  LEFT  JOIN Account_Item AS ExposedAccountItem ON (ExposedAccountItem.reexposeItem_id = Account_Item.id)
  LEFT  JOIN rbPayRefuseType ON (rbPayRefuseType.id = Account_Item.refuseType_id)
WHERE Account_Item.id = %s;
""" % (account_id)
        query = QtGui.qApp.db.query(stmt)

        if query.next():
            record = query.record()
            return forceString(record.value('isExposedItem'))
        else:
            return '0'


    def writeEventInfo(self, record, params):
        u"""Пишет информацию о событии"""
        eventId = forceRef(record.value('eventId'))
        execDate = forceDate(record.value('SLUCH_DATE_2'))
        isDispanser = forceString(record.value('mesGroupCode')) == u'ДиспаснС'
        eventPurposeCode = forceString(record.value(
                'eventPurposeCode'))
        patrName = forceString(record.value('PERS_OT'))
        haveMoving = forceBool(record.value('haveMoving'))
        uslOk = forceInt(record.value('SLUCH_USL_OK'))
        cureTypeCode = forceString(record.value('SLUCH_VID_HMP_raw'))
        vidpom = forceString(record.value('SLUCH_VIDPOM'))

        local_params = {
            'SLUCH_OS_SLUCH': '2' if (not patrName or
                                      patrName.upper() == u'НЕТ') else '',
            'SLUCH_VERS_SPEC': 'V015',
            'SLUCH_OPLATA': '0',
            'SLUCH_NSVOD': '%s%02d' % (eventPurposeCode[:1],
                    execDate.month()),
            'SLUCH_KODLPU': params['lpuCode'],
            'SLUCH_PRNES': '0' if haveMoving else None,
            'SLUCH_PCHAST': '0',
            'SLUCH_IDDOKT': formatSNILS(forceString(record.value('SLUCH_IDDOKT_raw')))
        }

        if not isDispanser:
            local_params['SLUCH_METOD_HMP'] = forceString(
                                              record.value('SLUCH_METOD_HMP_raw'))
            local_params['SLUCH_VID_HMP'] = cureTypeCode
            local_params['SLUCH_FOR_POM'] = self.mapEventOrderToForPom.get(
                                  forceString(record.value('SLUCH_FOR_POM_raw')), '')

        # если тег PRZAB пустой, выводить 0
        if not forceString(record.value('SLUCH_PRZAB')):
            record.setValue('SLUCH_PRZAB', '0')

        if uslOk == 1: # если это стационар
            if vidpom != '32':
                local_params['SLUCH_CODE_FKSG'] = params[
                        'mapEventIdToFksgCode'].get(eventId, '')
            else:
                record.setValue('serviceCode', '')
            local_params['SLUCH_DOPL'] = ''
            # дабы уменьшить размеры SQL запроса, SLUCH_BOLDN расчитывается здесь
            delta = forceDate(record.value('SLUCH_DATE_2')).toPyDate() - forceDate(record.value('SLUCH_DATE_1')).toPyDate()
            params['SLUCH_BOLDN'] = (delta.days if delta.days > 0 else 1)
        elif uslOk == 3: # амбулатория
            params['SLUCH_BOLDN'] = ''
            record.setValue('USL_KSGA', toVariant(''))

        local_params.update(params)
        _record = ExtendedRecord(record, local_params, DEBUG)

        if eventId != params.setdefault('lastEventId'):
            params['USL_IDSERV'] = 0
            _record.setValue('USL_IDSERV', 0)
            params['countMove'] = 0

            if params['lastEventId']:
                self.eventGroup.clear(self)
                self.writeEndElement() # SLUCH

            self.eventGroup.addRecord(_record)
            params['lastEventId'] = eventId


    def getCountActionMove(self, eventId, params):
        if params.setdefault('countMove', 0) == 0:
            stmt = u"SELECT COUNT(id) as _count FROM vActionMoving WHERE event_id = %d;" % (eventId)
            query = self._parent.db.query(stmt)
            query.next()
            params['countMove'] = forceInt(query.record().value('_count'))
        return params['countMove']


    def writeService(self, record, params):
        u"""Выгружает информацию об услугах"""
        eventId = forceRef(record.value('eventId'))
        tariffType = forceInt(record.value('tariffType'))
        uslOk = forceInt(record.value('SLUCH_USL_OK'))
        vidpom = forceString(record.value('SLUCH_VIDPOM'))

        if (tariffType == CTariff.ttCoupleVisits
                and eventId not in params['visitExportedEventList']):
            params['visitExportedEventList'].add(eventId)
            params['toggleFlag'] = 1
            self.exportVisits(eventId, params)
            params['toggleFlag'] = 2


        local_params = {
            'USL_CODE_MD': formatSNILS(forceString(record.value('USL_CODE_MD_raw'))),
            'USL_PKUR': '0',
            'SL_KOEF_IDSL': [0, 0],
            'SL_KOEF_Z_SL': [0, 0]
        }

        if uslOk == 1:
            record.setValue('USL_DOPL', '')
            local_params['USL_KTLPU'] = u'ВС'

            # костыль, потому что не знаю как проверить на NULL =(
            tarif = forceInt(record.value('USL_TARIF'))
            if (tarif == -1):
                record.setValue('USL_TARIF', 0)
                record.setValue('USL_SUMV_USL', 0)
            elif (vidpom != '32' and forceInt(record.value('USL_SUMV_USL')) > 0):
                # Коэфициент сложности лечения пациента (КСЛП)
                kslp1 = 1
                kslp2 = 1
                sumvUsl = forceDouble(record.value('USL_SUMV_USL'))
                # до 1 года - 1.1
                # от 1 до 4 года - 1.05
                # более 75 лет - 1.05
                #TODO: Кроме КСГ, относящихся к профилю "неонатология"
                persAge = forceInt(record.value('persAge'))
                if persAge == 0:
                    kslp1 = 1.1
                    local_params['SL_KOEF_IDSL'][0] = 1
                    local_params['SL_KOEF_Z_SL'][0] = kslp1
                elif (1 <= persAge < 4) or (persAge >= 75):
                    if (persAge == 75 and forceDate(record.value('PERS_DR')) == forceDate(record.value('SLUCH_DATE_1'))):
                        pass
                    else:
                        kslp1 = 1.05
                        local_params['SL_KOEF_Z_SL'][0] = kslp1
                        if (1 <= persAge < 4):
                            local_params['SL_KOEF_IDSL'][0] = 2
                        elif (persAge >= 75):
                            local_params['SL_KOEF_IDSL'][0] = 4
                # кроме того, необходимо учитывать к\дни
                if self.getCountActionMove(eventId, params) > 1:
                    delta = forceDate(record.value('USL_DATE_OUT')).toPyDate() - forceDate(record.value('USL_DATE_IN')).toPyDate()
                    bolDn = (delta.days if delta.days > 0 else 1)
                else:
                    bolDn = params['SLUCH_BOLDN']
                # TODO: Необходимо учесть КСГ, для которых нормативное к\дней =45 (см. Приложение 2.2, Таб.5)
                maxDuration = 30.0
                # FIXME временный костыль с захардкоренными в код КСГ
                ksg45 = ['45','46','108','109','161','162','233','279','280','298']
                # если случай является сверхкоротким, то берем готовый коэф.
                if bolDn <= 3:
                    kslp2 = forceDouble(record.value('minboldn'))
                    kslp2 = forceDouble(format(kslp2, '.2f'))
                    record.setValue('USL_SK_KOEF', format(kslp2, '.2f'))
                # если пациент пролежал более 30 к\дней, то КСЛП
                # расчитывается по формуле: 1 + ( ([к\дней]-30) / 30 ) * 0.25
                elif bolDn > 30:
                    needKspl = True
                    if any(forceString(record.value('USL_CODE_USL')) in s for s in ksg45):
                        if bolDn > 45:
                            maxDuration = 45.0
                        else:
                            needKspl = False

                    if needKspl:
                        kslp2 = 1.0 + ((bolDn-maxDuration)/maxDuration) * 0.25
                        kslp2 = forceDouble(round(kslp2+0.0005,2))
                        local_params['SL_KOEF_IDSL'][1] = 10
                        local_params['SL_KOEF_Z_SL'][1] = kslp2
                if (kslp1 < 1 or kslp2 < 1):
                    newVal = forceDouble(format(sumvUsl * kslp1, '.2f')) * kslp2
                else:
                    newVal = sumvUsl * forceDouble(format((kslp1 + (kslp2 - 1)), '.2f'))
                record.setValue('USL_SUMV_USL', format(newVal, '.2f'))
            else:
                record.setValue('USL_SUMV_USL', format(forceDouble(record.value('USL_SUMV_USL')), '.2f'))

        csgActionId = forceRef(record.value('csgActionId'))

        if csgActionId in params['replaceBegDate'].keys():
            record.setValue('USL_DATE_IN',
                toVariant(params['replaceBegDate'][csgActionId]))

        # не выводить тег, если коэф. не применены
        if (forceDouble(record.value('USL_SK_KOEF')) == 0.0):
                record.setValue('USL_SK_KOEF', '')
        if (local_params['SL_KOEF_IDSL'][0] == 0 and local_params['SL_KOEF_IDSL'][1] == 0):
            local_params['SL_KOEF_IDSL'] = []
            local_params['SL_KOEF_Z_SL'] = []
        elif (local_params['SL_KOEF_IDSL'][0] == 0):
            local_params['SL_KOEF_IDSL'] = local_params['SL_KOEF_IDSL'][1:2]
            local_params['SL_KOEF_Z_SL'] = local_params['SL_KOEF_Z_SL'][1:2]
        elif (local_params['SL_KOEF_IDSL'][1] == 0):
            local_params['SL_KOEF_IDSL'] = local_params['SL_KOEF_IDSL'][0:1]
            local_params['SL_KOEF_Z_SL'] = local_params['SL_KOEF_Z_SL'][0:1]

        if csgActionId not in params['skipSet'] and eventId not in params['doneVis']:
            params['USL_IDSERV'] += 1
            params['USL_IDMASTER'] = 0
            local_params.update(params)
            _record = ExtendedRecord(record, local_params, DEBUG)
            self.eventGroup.addRecord(_record)

        if params['toggleFlag'] == 2 and eventId not in params['doneVis']:
            params['doneVis'].add(eventId)
            params['toggleFlag'] = 0

    def exportVisits(self, eventId, params):
        u"""Экспортирует данные для визитов с 0 стоимостью,
            при тарификации по посещениям"""
        stmt = u"""SELECT
            Visit.event_id AS eventId,
            Diagnosis.MKB AS USL_DS,
            rbSpeciality.code AS USL_PRVS,
            CAST(Visit.date AS DATE) AS USL_DATE_IN,
            CAST(Visit.date AS DATE) AS USL_DATE_OUT,
            SUBSTR(rbService.code, 1, 4) AS USL_PODR,
             IF(mes.mrbMESGroup.code = 'ДиспаснС', '',
                IF(rbService_Profile.medicalAidProfile_id IS NOT NULL, ServiceProfileMedicalAidProfile.code,
                 ServiceMedicalAidProfile.code)) AS USL_PROFIL,
            IF(age(Client.birthDate,Event.setDate) < 18 AND rbSpeciality.code = '22', 1, 0) AS USL_DET,
            CAST(IF(EventType.form = '110', Event.execDate, Event.setDate) AS DATE) AS USL_DATE_1,
            RPAD(SUBSTR(rbService.code,1, 4), 11, '0') AS USL_CODE_USL,
            IF(mes.mrbMESGroup.code = 'ДиспаснС', '', 1) AS USL_KOL_USL,
            IF(rbService.infis = '', '0', rbService.infis) AS USL_KSGA,
            IF(rbService.code LIKE 'A16%%', rbService.code, '') AS USL_CODE_OPER,
            0 AS USL_SK_KOEF,
            0 AS USL_SUMV_USL,
            0 AS USL_TARIF,
            VisitPerson.SNILS AS USL_CODE_MD_raw
        FROM Visit
        LEFT JOIN Event ON Event.id = Visit.event_id
        LEFT JOIN Diagnostic ON ( Diagnostic.event_id = Event.id
                                  AND Diagnostic.diagnosisType_id IN (SELECT id FROM rbDiagnosisType WHERE code in ('1', '2'))
                                  AND Diagnostic.deleted = 0 )
        LEFT JOIN Diagnosis ON Diagnosis.id = Diagnostic.diagnosis_id AND Diagnosis.deleted = 0
        LEFT JOIN Person AS VisitPerson ON VisitPerson.id = Visit.person_id
        LEFT JOIN Person ON Person.id = Event.execPerson_id
        LEFT JOIN rbSpeciality ON Person.speciality_id = rbSpeciality.id
        LEFT JOIN rbService ON Visit.service_id = rbService.id
        LEFT JOIN Client ON Client.id = Event.client_id
        LEFT JOIN mes.MES ON Event.MES_id = mes.MES.id
        LEFT JOIN mes.mrbMESGroup ON mes.MES.group_id = mes.mrbMESGroup.id
        LEFT JOIN rbService_Profile ON rbService_Profile.id = (SELECT MAX(id) FROM rbService_Profile rs
            WHERE rs.master_id = rbService.id AND rs.speciality_id = Person.speciality_id)
        LEFT JOIN rbMedicalAidProfile AS ServiceProfileMedicalAidProfile ON ServiceProfileMedicalAidProfile.id = rbService_Profile.medicalAidProfile_id
        LEFT JOIN rbMedicalAidProfile AS ServiceMedicalAidProfile ON ServiceMedicalAidProfile.id = rbService.medicalAidProfile_id
        LEFT JOIN EventType ON EventType.id = Event.eventType_id
        WHERE Visit.event_id = %d
        AND Visit.deleted = '0'""" % eventId

        query = self._parent.db.query(stmt)

        while query.next():
            record = query.record()

            self.writeService(record, params)


    def writeClientInfo(self, record, params):
        u"""Пишет информацию о пациенте"""

        sex = forceString(record.value('PERS_W'))
        birthDate = forceDate(record.value('PERS_DR'))
        socStatus = forceString(record.value('PACIENT_STAT_Z_raw'))
        execDate = forceDate(record.value('execDate'))
        isJustBorn = birthDate.daysTo(execDate) < 28
        isAnyBirthDoc = forceString(record.value('PERS_DOCSER')) or forceString(
                                    record.value('PERS_DOCNUM'))
        statZ = self.mapSocStatusCode.get(socStatus, '')

        local_params = {
            'PACIENT_NOVOR':  ('%s%s1' % (sex[:1], birthDate.toString('ddMMyy')
                             )) if isJustBorn and not isAnyBirthDoc else '0',
            'PACIENT_STAT_Z': statZ,
        }

        local_params.update(params)
        _record = ExtendedRecord(record, local_params, DEBUG)
        self.checkerClientInfo(_record)
        self.writeGroup('PACIENT', self.clientFields, _record)


    def writeFooter(self, params):
        u"""Закрывает теги в конце файла"""
        if params.get('lastEventId'):
            self.eventGroup.clear(self)
            self.writeEndElement() # SLUCH
            self.writeEndElement() # ZAP

        self.writeEndElement() # ZL_LIST


    def checkElement(self, eventId, key, string, maxLen, idServ = None, optional = False, isClientInfo = False, suffix=''):
        if not self.checkFill(eventId, key, string, idServ, optional, isClientInfo, suffix):
            return False
        if not self.checkLength(eventId, key, string, maxLen, idServ, isClientInfo, suffix):
            return False
        return True


    def checkLength(self, eventId, key, string, maxLen, idServ = None, isClientInfo = False, suffix=''):
        if len(string) > maxLen:
            if idServ:
                message = u'Обращение %d%s, услуга %d: тег %s превышает допустимую длинну (%d > %d)!' % (
                            eventId, suffix, idServ, key, len(string), maxLen)
            elif isClientInfo:
                message = u'Пациент %d: тег %s превышает допустимую длинну (%d > %d)!' % (
                            eventId, key, len(string), maxLen)
            else:
                message = u'Обращение %d%s: тег %s превышает допустимую длинну (%d > %d)!' % (
                            eventId, suffix, key, len(string), maxLen)
            self._parent.logError(message)
            return False
        return True


    def checkFill(self, eventId, key, string, idServ = None, optional = False, isClientInfo = False, suffix=''):
        if not string and not optional:
            if idServ:
                message = u'Обращение %d%s, услуга %d: тег %s - пустой!' % (eventId, suffix, idServ, key)
            elif isClientInfo:
                message = u'Пациент %d: тег %s - пустой!' % (eventId, key)
            else:
                message = u'Обращение %d%s: тег %s - пустой!' % (eventId, suffix, key)
            self._parent.logError(message)
            return False
        return True

# ******************************************************************************

class CExportR61EventGroup:
    def __init__(self):
        self.records = []
        self.idmasterList = []

    def getIdMaster(self, record):
        uslOk = forceInt(record.value('SLUCH_USL_OK'))
        if uslOk == 3 or uslOk == 0:
            if forceDouble(record.value('USL_SUMV_USL')) == 0.0:
                self.idmasterList.append(forceInt(record.value('aiId')))
            else:
                idServ = forceInt(record.value('USL_IDSERV'))
                for i in range(1,len(self.idmasterList)+1):
                    self.records[i].setValue('USL_IDMASTER', idServ)
                record.setValue('USL_IDMASTER', len(self.idmasterList)+1)
            pass
        else:
            dateIn = forceDate(record.value('USL_DATE_IN')).toPyDate()
            dateOut = forceDate(record.value('USL_DATE_OUT')).toPyDate()
            for item in self.idmasterList:
                iDateIn = item['DATE_IN']
                iDateOut = item['DATE_OUT']
                if (iDateIn <= dateIn) and (iDateOut >= dateOut):
                    record.setValue('USL_IDMASTER', item['IDSERV'])
                    return

            idServ = forceInt(record.value('USL_IDSERV'))
            self.idmasterList.append({
                'IDSERV': idServ,
                'DATE_IN': dateIn,
                'DATE_OUT': dateOut
            })
            record.setValue('USL_IDMASTER', idServ)

    def addRecord(self, record):
        if forceInt(record.value('USL_IDSERV')) > 0:
            self.getIdMaster(record)
        self.records.append(record)

    def clear(self, writer):
        self.calc()
        self.write(writer)
        del self.records[:]
        del self.idmasterList[:]

    def calc(self):
        sumv = 0.0
        for rec in self.records[1:]:
            sumv = forceDouble(format(sumv + forceDouble(rec.value('USL_SUMV_USL')), ".2f"))
        self.records[0].setValue('SLUCH_SUMV', sumv)

    def write(self, writer):
        writer.checkerEventInfo(self.records[0])
        writer.writeGroup('SLUCH', writer.eventFields, self.records[0],
                          closeGroup=False, dateFields=writer.eventDateFields)
        for rec in self.records[1:]:
            writer.checkerService(rec)
            if (len(rec.value('SL_KOEF_IDSL')) > 0):
                writer.writeGroup('USL', writer.serviceFields, rec,
                        dateFields=writer.serviceDateFields,
                        subGroup=writer.serviceSubGroup)
            else:
                writer.writeGroup('USL', writer.serviceFields, rec,
                                  dateFields=writer.serviceDateFields)

# ******************************************************************************
