#!/usr/bin/env python
# -*- coding: utf-8 -*-

#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

u"""
Экспорт тарифа для ЕИС Краснодара
"""

import os.path
import shutil

from PyQt4 import QtGui
from PyQt4.QtCore import Qt, QDir, pyqtSignature, SIGNAL

from library.dbfpy.dbf import Dbf
from library.DialogBase import CConstructHelperMixin
from library.Utils import anyToUnicode, forceBool, forceDate, forceDouble, forceRef, forceString, forceStringEx, pyDate, toVariant


from Exchange.Export import CExportHelperMixin
from Exchange.Utils import compressFileInRar

from Exchange.Ui_ExportTariffR23_Wizard_1 import Ui_ExportTariff_Wizard_1
from Exchange.Ui_ExportTariffR23_Wizard_2 import Ui_ExportTariff_Wizard_2


def ExportTariffsR23(widget, tariffRecordList, expensesDict):
    appPrefs = QtGui.qApp.preferences.appPrefs
    fileName = forceString(appPrefs.get('ExportTariffsR23FileName', ''))
    exportAll = forceBool(appPrefs.get('ExportTariffsR23ExportAll', True))
    compressRAR = forceBool(appPrefs.get('ExportTariffsR23CompressRAR', False))
    dlg = CExportTariffR23(fileName, exportAll, compressRAR, tariffRecordList, expensesDict, widget)
    dlg.exec_()
    appPrefs['ExportTariffsR23FileName'] = toVariant(dlg.fileName)
    appPrefs['ExportTariffsR23ExportAll'] = toVariant(dlg.exportAll)
    appPrefs['ExportTariffsR23CompressRAR'] = toVariant(dlg.compressRAR)

class CExportTariffWizardPage1(QtGui.QWizardPage, Ui_ExportTariff_Wizard_1, CConstructHelperMixin):
    def __init__(self, parent):
        QtGui.QWizardPage.__init__(self, parent)
        self.parent = parent #wtf? скрывать Qt-шный parent - это плохая затея
        self.preSetupUi()
        self.setupUi(self)
        self.postSetupUi()
        self.setTitle(u'Параметры экспорта')
        self.setSubTitle(u'Выбор записей для экспорта')


    def preSetupUi(self):
        from Orgs.Contracts import CTariffModel
        self.addModels('Table', CTariffModel(self))
        self.modelTable.cellReadOnly = lambda index: True
        self.modelTable.setEnableAppendLine(False)


    def postSetupUi(self):
        self.setModels(self.tblItems, self.modelTable, self.selectionModelTable)
        self.tblItems.setSelectionMode(QtGui.QAbstractItemView.MultiSelection)
        self.tblItems.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
        self.modelTable.setItems(self.parent.tariffRecordList)
        self.selectionModelTable.clearSelection()
        self.tblItems.setFocus(Qt.OtherFocusReason)
        self.tblItems.setEnabled(not self.parent.exportAll)
        self.btnSelectAll.setEnabled(not self.parent.exportAll)
        self.btnClearSelection.setEnabled(not self.parent.exportAll)
        self.chkExportAll.setChecked(self.parent.exportAll)
        if self.parent.exportAll:
            self.parent.selectedItems = range(self.modelTable.rowCount())


    def isComplete(self):
        # проверим пустой ли у нас список выбранных элементов
        if self.parent.exportAll:
            return True
        else:
            return self.parent.selectedItems != []


    def selectedItemList(self):
        rows = [index.row() for index in self.selectionModelTable.selectedRows()]
        rows.sort()
        return rows


    @pyqtSignature('QModelIndex')
    def on_tblItems_clicked(self, index):
        self.parent.selectedItems = self.selectedItemList()
        self.emit(SIGNAL('completeChanged()'))


    @pyqtSignature('')
    def on_btnSelectAll_clicked(self):
        selectedItems = range(self.modelTable.rowCount())
        self.parent.selectedItems = selectedItems
        for row in selectedItems:
            index = self.modelTable.index(row, 0)
            self.selectionModelTable.select(index, QtGui.QItemSelectionModel.Select|QtGui.QItemSelectionModel.Rows)
        self.emit(SIGNAL('completeChanged()'))


    @pyqtSignature('')
    def on_btnClearSelection_clicked(self):
        self.selectionModelTable.clearSelection()
        self.parent.selectedItems = []
        self.emit(SIGNAL('completeChanged()'))


    @pyqtSignature('')
    def on_chkExportAll_clicked(self):
        self.parent.exportAll = self.chkExportAll.isChecked()
        self.tblItems.setEnabled(not self.parent.exportAll)
        self.btnSelectAll.setEnabled(not self.parent.exportAll)
        self.btnClearSelection.setEnabled(not self.parent.exportAll)
        if self.chkExportAll.isChecked():
            self.parent.selectedItems = range(self.modelTable.rowCount())
        else:
            self.parent.selectedItems = self.selectedItemList()
        self.emit(SIGNAL('completeChanged()'))


class CExportTariffWizardPage2(QtGui.QWizardPage, Ui_ExportTariff_Wizard_2, CExportHelperMixin):
    def __init__(self, parent):
        self.done = False
        QtGui.QWizardPage.__init__(self, parent)
        CExportHelperMixin.__init__(self)
        self.setupUi(self)
        self.parent=parent #wtf? скрывать Qt-шный parent - это плохая затея
        self.setTitle(u'Параметры сохранения')
        self.setSubTitle(u'Выбор места для сохранения')
        self.progressBar.setFormat('%p%')
        self.progressBar.setValue(0)
        self.edtFileName.setText(self.parent.fileName)
        self.btnExport.setEnabled(self.parent.fileName != '')
        self.checkRAR.setChecked(self.parent.compressRAR)
        self.expenseBase = None
        self.expenseAdditional  = None
        self.lpuCode = ''


    def initializePage(self):
        self.btnExport.setEnabled(self.parent.fileName != '')
        self.done = False


    def isComplete(self):
        return self.done


    def createDbf(self):
        dbfName = self.parent.getFullDbfFileName()
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('CODE_MO', 'C', 5),
            ('KUSL', 'C', 14),
            ('DATN', 'D'),
            ('TARIF_B', 'N', 10, 2),
            ('TARIF_D', 'N', 10, 2),
            ('TARIF', 'N', 10, 2),
            ('TARIF_M', 'N', 10, 2),
            ('CENA_KD', 'N', 10, 2),
        )

        return dbf


    def writeDBF(self, outFile, tariffRecordList, selectedItems, expensesDict):
        try:
            self.progressBar.setMaximum(max(len(selectedItems), 1))
            self.progressBar.reset()
            self.progressBar.setValue(0)

            for i in selectedItems:
                self.writeRecord(outFile, tariffRecordList[i], expensesDict[i])
                QtGui.qApp.processEvents()
                self.progressBar.step()

        except IOError, e:
            QtGui.qApp.logCurrentException()
            msg=''
            if hasattr(e, 'filename'):
                msg = u'%s:\n[Errno %s] %s' % (e.filename, e.errno, anyToUnicode(e.strerror))
            else:
                msg = u'[Errno %s] %s' % (e.errno, anyToUnicode(e.strerror))
            QtGui.QMessageBox.critical (self.parent,
                u'Произошла ошибка ввода-вывода', msg, QtGui.QMessageBox.Close)
            return False
        except Exception, e:
            QtGui.qApp.logCurrentException()
            QtGui.QMessageBox.critical (self.parent,
                u'Произошла ошибка', unicode(e), QtGui.QMessageBox.Close)
            return False

        return True


    def writeRecord(self, outFile, record, expenses):
        baseExpence = 0
        additionalExpence = 0
        price = forceDouble(record.value('price'))

        for ex in expenses:
            val = forceDouble(ex.value('percent'))*price/100
            if forceRef(ex.value('rbTable_id')) == self.expenseBase:
                baseExpence = val
            elif forceRef(ex.value('rbTable_id')) == self.expenseAdditional:
                additionalExpence = val

        row = outFile.newRecord()
        row['CODE_MO'] = self.lpuCode
        row['KUSL'] = self.getServiceInfis(forceRef(record.value('service_id')))
        row['DATN'] = pyDate(forceDate(record.value('begDate')))
        row['TARIF_B'] = baseExpence
        row['TARIF_D'] = additionalExpence
        row['TARIF'] = price
        row['TARIF_M'] = forceDouble(record.value('federalPrice'))
        row.store()


    @pyqtSignature('')
    def on_btnSelectFile_clicked(self):
        dir = QtGui.QFileDialog.getExistingDirectory(self,
                u'Выберите директорий для сохранения файла выгрузки',
                 forceStringEx(self.edtFileName.text()),
                 QtGui.QFileDialog.ShowDirsOnly)
        if forceString(dir):
            self.edtFileName.setText(QDir.toNativeSeparators(dir))


    @pyqtSignature('')
    def on_btnExport_clicked(self):
        dstDir = self.edtFileName.text()
        if dstDir.isEmpty():
            return

        if not self.parent.selectedItems:
            QtGui.QMessageBox.warning(self,
                                      u'Экспорт тарифов договора',
                                      u'Не выбрано ни одного элемента для выгрузки')
            self.parent.back() # вернемся на пред. страницу. пусть выбирают
            return

        self.expenseBase = forceRef(QtGui.qApp.db.translate('rbExpenseServiceItem', 'code','1', 'id'))
        self.expenseAdditional = forceRef(QtGui.qApp.db.translate('rbExpenseServiceItem', 'code','2', 'id'))
        self.lpuCode = forceString(QtGui.qApp.db.translate('Organisation', 'id', QtGui.qApp.currentOrgId(), 'infisCode'))

        if not self.expenseBase:
            QtGui.QMessageBox.critical(self,
             u'Не найдена статья затрат услуг',
             u'(rbExpenseServiceItem) с кодом "01" (базовая)')
            return

        if not self.expenseAdditional:
            QtGui.QMessageBox.critical(self,
             u'Не найдена статья затрат услуг',
             u'(rbExpenseServiceItem) с кодом "02" (дополнительная)')
            return

        outFile = self.createDbf()

        if (self.writeDBF(outFile, self.parent.tariffRecordList, self.parent.selectedItems, self.parent.expensesDict)):
            self.progressBar.setText(u'Готово')
        else:
            self.progressBar.setText(u'Прервано')
        outFile.close()

        fileName = self.parent.getFullDbfFileName()

        if self.checkRAR.isChecked():
            self.progressBar.setText(u'Сжатие')
            compressFileInRar(fileName, fileName+'.rar')
            self.progressBar.setText(u'Сжато в "%s"' % (fileName+'.rar'))
            fileName += '.rar'

        dstDir = forceStringEx(dstDir)

        if not os.path.exists(dstDir):
            os.makedirs(dstDir)

        dstDbf = os.path.join(dstDir, os.path.basename(fileName))
        success, result = QtGui.qApp.call(self, shutil.move, (fileName, dstDbf))

        self.done = success
        self.emit(SIGNAL('completeChanged()'))


    @pyqtSignature('')
    def on_checkRAR_clicked(self):
        self.parent.compressRAR = self.checkRAR.isChecked()


    @pyqtSignature('QString')
    def on_edtFileName_textChanged(self):
        self.parent.fileName = self.edtFileName.text()
        self.btnExport.setEnabled(self.parent.fileName != '')
        self.emit(SIGNAL('completeChanged()'))


class CExportTariffR23(QtGui.QWizard):
    def __init__(self, fileName, exportAll, compressRAR, tariffRecordList, expensesDict, parent=None):
        QtGui.QWizard.__init__(self, parent, Qt.Dialog)
        self.setWizardStyle(QtGui.QWizard.ModernStyle)
        self.setWindowTitle(u'Экспорт тарифов договора для ЕИС Краснодара')
        self.selectedItems = []
        self.fileName= fileName
        self.exportAll = exportAll
        self.compressRAR = compressRAR
        self.tariffRecordList = tariffRecordList
        self.expensesDict =  expensesDict
        self.addPage(CExportTariffWizardPage1(self))
        self.addPage(CExportTariffWizardPage2(self))
        self.tmpDir = ''


    def getTmpDir(self):
        if not self.tmpDir:
            self.tmpDir = QtGui.qApp.getTmpDir('TariffR23')
        return self.tmpDir


    def getFullDbfFileName(self):
        return os.path.join(self.getTmpDir(), u'SPR22.DBF')


    def cleanup(self):
        if self.tmpDir:
            QtGui.qApp.removeTmpDir(self.tmpDir)
            self.tmpDir = ''


    def exec_(self):
        QtGui.QWizard.exec_(self)
        self.cleanup()
