﻿# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui
from PyQt4.QtCore import QDate, QDir, pyqtSignature

from library.dbfpy.dbf import Dbf
from library.Utils import forceBool, forceDate, forceRef, forceString, forceStringEx, nameCase, toVariant

from Accounting.Utils import updateAccounts, updateDocsPayStatus
from Events.Utils import CPayStatus, getPayStatusMask
from Exchange.Cimport import CDBFimport
from Exchange.Utils import dbfCheckNames, tbl
from Registry.Utils import getClientPolicyEx

from Exchange.Ui_ImportPayRefuseR23 import Ui_Dialog



def ImportPayRefuseR23Native(widget, accountId, accountItemIdList):
    dlg = CImportPayRefuseR23Native(widget, accountId, accountItemIdList)
    dlg.edtFileName.setText(forceString(QtGui.qApp.preferences.appPrefs.get('ImportPayRefuseR23FileName', '')))
    dlg.chkRefreshPolicyInfo.setChecked(forceBool(QtGui.qApp.preferences.appPrefs.get('ImportPayRefuseR23RefreshPolicy', False)))
    dlg.chkDeleteAccount.setChecked(forceBool(QtGui.qApp.preferences.appPrefs.get('ImportPayRefuseR23DeleteAccount', False)))
    dlg.exec_()
    QtGui.qApp.preferences.appPrefs['ImportPayRefuseR23FileName'] = toVariant(dlg.edtFileName.text())
    QtGui.qApp.preferences.appPrefs['ImportPayRefuseR23RefreshPolicy'] = toVariant(dlg.chkRefreshPolicyInfo.isChecked())
    QtGui.qApp.preferences.appPrefs['ImportPayRefuseR23DeleteAccount'] = toVariant(dlg.chkDeleteAccount.isChecked())


class CImportPayRefuseR23Native(QtGui.QDialog, Ui_Dialog, CDBFimport):
    sexMap = {u'Ж':2, u'ж':2, u'М':1, u'м':1}

    def __init__(self, parent, accountId, accountItemIdList):
        QtGui.QDialog.__init__(self)
        self.setupUi(self)
        self.parent = parent #wtf? скрывать Qt-шный parent - это плохая затея
        CDBFimport.__init__(self, self.log)
        self.accountId=accountId
        self.accountItemIdList=accountItemIdList
        self.checkName()
        self.errorPrefix = ''

        self.nProcessed = 0
        self.nPayed = 0
        self.nRefused = 0
        self.nNotFound = 0

        self.tableAccountItem = tbl('Account_Item')
        self.tableEvent = tbl('Event')
        self.tableAccount = tbl('Account')
        self.tableAcc=self.db.join(self.tableAccountItem, self.tableAccount,
            'Account_Item.master_id=Account.id').join(self.tableEvent,
            'Account_Item.event_id=Event.id')

        self.prevContractId = None
        self.financeTypeOMS = forceRef(self.db.translate('rbFinance',
            'code', '2', 'id'))

        self.orgCache = {}
        self.policyKindCache = {}

        self.deleteAccount = False
        self.hasFieldPV = True
        self.hasFieldDvozvrat = True
        self.importType2013 = False
        self.defaultPolicyKindId = None


    @pyqtSignature('')
    def on_btnSelectFile_clicked(self):
        fileName = QtGui.QFileDialog.getOpenFileName(
            self, u'Укажите файл с данными', self.edtFileName.text(), u'Файлы DBF (*.dbf)')
        if fileName != '':
            self.edtFileName.setText(QDir.toNativeSeparators(fileName))
            self.btnImport.setEnabled(True)
            dbfFileName = unicode(forceStringEx(self.edtFileName.text()))
            dbfR23 = Dbf(dbfFileName, readOnly=True, encoding='cp866')
            self.labelNum.setText(u'всего записей в источнике: '+str(len(dbfR23)))


    def err2log(self, e):
        self.log.append(self.errorPrefix+e)


    def startImport(self):
        self.deleteAccount = False
        self.progressBar.setFormat('%p%')
        self.currentAccountOnly = self.chkOnlyCurrentAccount.isChecked()
        self.updateClientPolicy = self.chkRefreshPolicyInfo.isChecked()
#        self.importPayed =self.chkImportPayed.isChecked()
#        self.importRefused = self.chkImportRefused.isChecked()
        self.confirmation = self.edtConfirmation.text()
        importPolicyInfo = self.tabImportType.currentIndex() == 1
        if not self.confirmation:
            self.log.append(u'нет подтверждения')
            return

        self.prevContractId = None
        self.accountIdSet = set()

        self.nProcessed = 0
        self.nPayed = 0
        self.nRefused = 0
        self.nNotFound = 0

        dbfFileName = unicode(self.edtFileName.text())
        dbfR23 = Dbf(dbfFileName, readOnly=True, encoding='cp866')
        self.labelNum.setText(u'всего записей в источнике: '+str(len(dbfR23)))
        requiredFields = ['FIO', 'IMA', 'OTCH', 'POL', 'DATR', 'VS',  'SN']
        self.hasFieldPV = 'PV' in dbfR23.header.fields
        self.hasFieldDvozvrat = 'DVOZVRAT' in dbfR23.header.fields

        if importPolicyInfo:
            requiredFields += ['KSMOF', 'SPSF', 'SPNF', 'CS']
            self.policyTypeId = forceRef(self.db.translate('rbPolicyType', 'code', '1', 'id'))
            self.defaultPolicyKindId = forceRef(self.db.translate('rbPolicyKind', 'code', '1', 'id'))
            self.log.append(u'Режим предварительной проверки счетов.')
            self.accountNumber = forceString(self.db.translate('Account', 'id', self.accountId, 'number'))
            self.importType2013 = dbfCheckNames(dbfR23, ['SPVF', 'FIOF', 'IMAF',
                'OTCHF', 'DATRF', 'POLF', 'OKATO_OMSF'])

        if not dbfCheckNames(dbfR23, requiredFields):
            self.log.append(u'в файле импорта отсутствуют необходимые поля.')
            return

        self.progressBar.setMaximum(len(dbfR23)-1)
        process = self.processPolicyRow if importPolicyInfo else self.processRow

        for row in dbfR23:
            QtGui.qApp.processEvents()
            if self.abort:
                break
            self.progressBar.step()
            self.stat.setText(
                u'обработано: %d; оплаченых: %d; отказаных: %d; не найдено: %d' % \
                (self.nProcessed, self.nPayed, self.nRefused, self.nNotFound))
            process(row)


        self.stat.setText(
            u'обработано: %d; оплаченых: %d; отказаных: %d; не найдено: %d' % \
            (self.nProcessed, self.nPayed, self.nRefused, self.nNotFound))

        if importPolicyInfo:
            if self.deleteAccount and self.chkDeleteAccount.isChecked():
                self.log.append(u'Территория страхования одного из пациентов изменилась, текущий счет будет удалён.')
                self.parent.tblAccounts.removeSelectedRows()
                self.parent.updateAccountsPanel(self.parent.modelAccounts.idList())
        else:
            updateAccounts(self.accountIdSet)


    def processRow(self,  row):
        lastName = forceString(row['FIO'])
        firstName = forceString(row['IMA'])
        patrName = forceString(row['OTCH'])
        sex = self.sexMap.get(forceString(row['POL']))
        birthDate = QDate(row['DATR']) if row['DATR'] else QDate()

        if self.hasFieldDvozvrat:
            refuseDate = QDate(row['DVOZVRAT']) if row['DVOZVRAT'] else QDate()
        else:
            refuseDate = QDate.currentDate()

        refuseReasonCodeList =  forceString(row['PV']).split(' ') if self.hasFieldPV else ['Отказано']
        clientId = forceRef(row['SN'])
        payStatusMask = 0
        accDate = QDate(row['DATS']) if row['DATS'] else QDate()
        accNumber = forceString(row['NS'])

        self.errorPrefix = u'Строка %d (%s %s %s): ' % (self.progressBar.value(), lastName,  firstName,  patrName)

        accountType = forceString(row['VS'])
        if accountType != '3' and accountType != 'f' and accountType != 'b' and accountType != 'j':
            self.err2log(u'тип счёта не возвратный, код "%s"' % accountType)
            return

        if refuseDate and refuseReasonCodeList == []:
            self.err2log(u'нет кода отказа')
            return

        if not refuseDate and refuseReasonCodeList != []:
            self.err2log(u'нет даты отказа')
            return

        if not clientId:
            self.err2log(u'отсутствует идентификатор пациента')

        cond=[]
        cond.append(self.tableEvent['client_id'].eq(toVariant(clientId)))

        if accDate:
            cond.append(self.tableAccount['exposeDate'].eq(toVariant(accDate)))

        if accNumber:
            cond.append(self.tableAccount['number'].eq(toVariant(accNumber)))

        if self.currentAccountOnly:
            cond.append(self.tableAccount['id'].eq(toVariant(self.accountId)))

        fields = 'Account_Item.id, Account.date as Account_date, Account_Item.date as Account_Item_date, Account_Item.master_id as Account_id, Account.contract_id as contract_id'
        recordList = self.db.getRecordList(self.tableAcc, fields, where=cond)

        if recordList != []:
            for record in recordList:
                self.accountIdSet.add(forceRef(record.value('Account_id')))
                contractId = forceRef(record.value('contract_id'))

                if self.prevContractId != contractId:
                    self.prevContractId = contractId
                    financeId = forceRef(self.db.translate('Contract', 'id', contractId, 'finance_id'))
                    payStatusMask = getPayStatusMask(financeId)

                accDate = forceDate(record.value('Account_date'))
                accItemDate = forceDate(record.value('Account_Item_date'))

                if accItemDate:
                    self.err2log(u'счёт уже отказан')
                    return

                if accDate > refuseDate:
                    self.err2log(u'счёт уже отказан')
                    return

                self.nProcessed += 1
                accountItemId = forceRef(record.value('id'))
                accItem = self.db.getRecord(self.tableAccountItem, '*', accountItemId)
                accItem.setValue('date', toVariant(refuseDate))
                refuseTypeId = None

                if refuseDate:
                    self.nRefused += 1
                    refuseTypeId=self.getRefuseTypeId(refuseReasonCodeList[0])

                if not refuseTypeId:
                    refuseTypeId = self.addRefuseTypeId(refuseReasonCodeList[0], financeId=self.financeTypeOMS)

                accItem.setValue('refuseType_id', toVariant(refuseTypeId))
                updateDocsPayStatus(accItem, payStatusMask, CPayStatus.refusedBits)

                accItem.setValue('number', toVariant(self.confirmation))
                self.db.updateRecord(self.tableAccountItem, accItem)
        else:
            self.nNotFound += 1
            self.err2log(u'счёт не найден')


    def processPolicyRow(self, row):
        lastName = nameCase(forceString(row['FIO']))
        firstName = nameCase(forceString(row['IMA']))
        patrName = nameCase(forceString(row['OTCH']))

        self.errorPrefix = u'Строка %d (%s %s %s): ' % (self.progressBar.value(), lastName,  firstName,  patrName)

        clientId = forceRef(row['SN'])
        serial = forceString(row['SPSF'])
        number = forceString(row['SPNF'])
        insurerInfis = forceString(row['KSMOF'])
        (insurerId, insurerArea) = self.findOrgByInfis(insurerInfis)

#        if self.importType2013 and insurerId:
#            newInsurerOkato = forceString(row['OKATO_OMSF'])
#
#
#            if self.updateInsurer(insurerId, newInsurerOkato):
#                 self.err2log(u'<b><font color=blue>Обновлены</font></b> данные СМО'
#                                u' %s.' % (insurerInfis))

        if not clientId:
            self.err2log(u'отсутствует идентификатор пациента')
        elif clientId and self.importType2013:
            newLastName = nameCase(forceString(row['FIOF']))
            newFirstName = nameCase(forceString(row['IMAF']))
            newPatrName = nameCase(forceString(row['OTCHF']))
            newSex = self.sexMap.get(forceString(row['POLF']), 0)
            newBirthDate = QDate(row['DATRF']) if row['DATRF'] else QDate()

            if self.updateClient(clientId, newLastName, newFirstName, newPatrName, newSex, newBirthDate):
                 self.err2log(u'<b><font color=blue>Обновлены</font></b> личные данные клиента'
                                u' %d.' % (clientId))

        if not insurerId and insurerInfis:
            self.err2log(u'Страховая компания с инфис кодом `%s` не найдена.' % insurerInfis)

        if clientId and number:
            record = getClientPolicyEx(clientId, True)

            if record:
                oldSerial = forceString(record.value('serial'))
                oldNumber = forceString(record.value('number'))
                oldInsurerId = forceRef(record.value('insurer_id'))
                policyKindId = self.defaultPolicyKindId
                policyKindChanged = False

                if self.importType2013:
                    oldPolicyKindId = forceRef(record.value('policyKind_id'))
                    newPolicyKindId = self.getPolicyKindByRegionalCode(forceString(row['SPVF']))

                    if newPolicyKindId:
                        policyKindId = newPolicyKindId
                        policyKindChanged = newPolicyKindId != oldPolicyKindId

                if oldSerial != serial or oldNumber != number \
                    or oldInsurerId != insurerId or policyKindChanged:

                    if self.updateClientPolicy:
                        if insurerId and insurerId != oldInsurerId:
                            self.err2log(u'<b><font color=green>Добавляем</font></b>'
                                u' новый полис %sN%s (%d) -> %sN%s (%d).' %
                                 (oldSerial,  oldNumber, oldInsurerId if oldInsurerId else 0, serial, number, insurerId))
                            self.addClientPolicy(clientId, insurerId, serial, number, policyKindId)
                        else:
                            self.err2log(u'<b><font color=blue>Обновляем</font></b>'
                                u' полис %sN%s -> %sN%s.' %
                                 (oldSerial,  oldNumber, serial, number))

                            record.setValue('serial', toVariant(serial))
                            record.setValue('number',  toVariant(number))
                            record.setValue('policyKind_id', toVariant(policyKindId))
                            record.remove(record.indexOf('compulsoryServiceStop'))
                            record.remove(record.indexOf('voluntaryServiceStop'))
                            self.db.updateRecord(self.tableClientPolicy, record)

                    oldInsurerArea = forceString(self.db.translate('Organisation', 'id', oldInsurerId, 'area'))

                    if insurerArea and oldInsurerArea != insurerArea:
                        self.err2log(u'<b><font color=ref>Изменилась территория страхования пациента: `%s` -> `%s` для счета `%s`</font>'\
                                     % (oldInsurerArea, insurerArea,forceString(row['NS'])))
                        if self.accountNumber == forceString(row['NS']):
                            self.err2log(u'Территория изменилась для текущего счета')
                            self.deleteAccount = True

        else:
            self.err2log(u'<b><font color=sliver>Без изменений:</font></b> `%d`' % clientId)

        self.nProcessed += 1


    def findOrgByInfis(self, infis):
        u"""Возвращает id и область страховой"""
        if not infis:
            return (None,  None)

        result = self.orgCache.get(infis,  -1)

        if result == -1:
            result = (None,  None)
            table = QtGui.qApp.db.table('Organisation')
            record = QtGui.qApp.db.getRecordEx(table, 'id, area',  [table['deleted'].eq(0),
                                                          table['infisCode'].eq(infis)], 'id')

            if record:
                result = (forceRef(record.value(0)), forceString(record.value(1)))
                self.orgCache[infis] = result

        return result


    def getPolicyKindByRegionalCode(self, regionalCode):
        result = self.policyKindCache.get(regionalCode,  -1)

        if result == -1:
            result  = forceRef(self.db.translate('rbPolicyKind', 'regionalCode', regionalCode,  'id'))
            self.policyKindCache[regionalCode] = result

        return result


    def addClientPolicy(self, clientId, insurerId, serial, number, policyKindId=None):
        record = self.tableClientPolicy.newRecord()
        record.setValue('client_id', toVariant(clientId))
        record.setValue('insurer_id', toVariant(insurerId))
        record.setValue('policyType_id', toVariant(self.policyTypeId))
        record.setValue('policyKind_id', toVariant(policyKindId))
        record.setValue('serial', toVariant(serial))
        record.setValue('number', toVariant(number))
        self.db.insertRecord(self.tableClientPolicy, record)


    def updateClient(self, clientId, lastName, firstName, patrName, sex, birthDate):
        record = self.db.getRecord(self.tableClient, '*', clientId)
        dirty = False

        if lastName:
            record.setValue('lastName', toVariant(lastName))
            dirty = True

        if firstName:
            record.setValue('firstName', toVariant(firstName))
            dirty = True

        if patrName:
            record.setValue('patrName', toVariant(patrName))
            dirty = True

        if birthDate:
            record.setValue('birthDate', toVariant(birthDate))
            dirty = True

        if sex:
            record.setValue('sex', toVariant(sex))
            dirty = True

        if dirty:
            self.db.updateRecord(self.tableClient, record)

        return dirty


#    def updateInsurer(self, orgId, okato, ogrn):
#        record = self.db.getRecord(self.tableOrganisation, '*', orgId)
#        dirty = False
#
#        if okato:
#            record.setValue('OKATO', toVariant(okato))
#            dirty = True
#
#        if ogrn:
#            record.setValue('OGRN', toVariant(ogrn))
#            dirty = True
#
#        if dirty:
#            self.db.updateRecord(self.tableOrganisation, record)
#
#        return dirty
