# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
u"""
Импорт тарифа в R23
"""
from PyQt4 import QtGui
from PyQt4.QtCore import QDate, QDir, pyqtSignature

from library.dbfpy.dbf import Dbf
from library.Utils import forceDate, forceDouble, forceInt, forceRef, forceString, forceStringEx, toVariant

from Accounting.Tariff import CTariff
from Exchange.Cimport import CDBFimport, Cimport, CImportTariffsMixin

from Exchange.ImportTariffsXML import getTariffDifference, copyTariff
#from Exchange.Utils import *

from Exchange.Ui_ImportTariffsR23 import Ui_Dialog


def ImportTariffsR23(widget, contractId, begDate, endDate, tariffList,
                     tariffExpenseItems, tariffCoefficientItems):
    dlg = CImportTariffs(widget, contractId, begDate, endDate, tariffList,
                       tariffExpenseItems, tariffCoefficientItems)

    dlg.edtSpr18.setText(forceString(QtGui.qApp.preferences.appPrefs.get('ImportTariffR23FileName18', '')))
    dlg.edtSpr22.setText(forceString(QtGui.qApp.preferences.appPrefs.get('ImportTariffR23FileName22', '')))
    dlg.edtSpr58.setText(forceString(QtGui.qApp.preferences.appPrefs.get('ImportTariffR23FileName58', '')))
    dlg.edtSpr66.setText(forceString(QtGui.qApp.preferences.appPrefs.get('ImportTariffR23FileName66', '')))
    dlg.exec_()
    QtGui.qApp.preferences.appPrefs['ImportTariffR23FileName18'] = \
        toVariant(dlg.edtSpr18.text())
    QtGui.qApp.preferences.appPrefs['ImportTariffR23FileName22'] = \
        toVariant(dlg.edtSpr22.text())
    QtGui.qApp.preferences.appPrefs['ImportTariffR23FileName58'] = \
        toVariant(dlg.edtSpr58.text())
    QtGui.qApp.preferences.appPrefs['ImportTariffR23FileName66'] = \
        toVariant(dlg.edtSpr66.text())

    return ((not dlg.abort), dlg.tariffList, dlg.tariffExpenseItems,
        dlg.tariffCoefficientItems)


class CImportTariffs(QtGui.QDialog, Ui_Dialog, CDBFimport, CImportTariffsMixin):
    def __init__(self, parent, contractId, begDate, endDate, tariffList,
                 tariffExpenseItems, tariffCoefficientItems):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)
        Cimport.__init__(self, self.log)
        CImportTariffsMixin.__init__(self, self.log, tariffList,
                                     tariffExpenseItems)

        self.contractId = contractId
        self.tariffCoefficientItems = list(tariffCoefficientItems)

        self.tariffType = None
        self.loadMature = False
        self.loadChildren = False
        self.begDate = begDate
        self.endDate = endDate
        self.dupAskUser = False
        self.dupUpdate = False
        self.dupSkip = False
        self.amountUetCache = {}

        self.nSkipped = 0
        self.nAdded = 0
        self.nUpdated = 0
        self.nProcessed = 0

        self.unitVisitId = None
        self.unitServiceId = None
        self.eventType37 = None
        self.expenseBase = None
        self.expenseAdditional = None
        self.amount = 0
        self.federalPriceFieldName = 'TARIF_M'


    def startImport(self):
        contractId = self.contractId
        if not contractId:
            return

        self.nSkipped = 0
        self.nAdded = 0
        self.nUpdated = 0
        self.nProcessed = 0
        self.amountUetCache = {}

        self.unitVisitId = forceRef(self.db.translate('rbMedicalAidUnit', 'code', '01', 'id'))
        self.unitServiceId = forceRef(self.db.translate('rbMedicalAidUnit', 'code', '18', 'id'))
        self.eventType37 = forceRef(self.db.translate('EventType', 'code', '37', 'id'))
        self.expenseBase = forceRef(self.db.translate('rbExpenseServiceItem', 'code', '1', 'id'))
        self.expenseAdditional = forceRef(self.db.translate('rbExpenseServiceItem', 'code', '2', 'id'))
        self.expenseTARU = forceRef(self.db.translate('rbExpenseServiceItem', 'code', '230001', 'id'))
        self.expenseTARU_B = forceRef(self.db.translate('rbExpenseServiceItem', 'code', '230002', 'id'))
        self.expenseTARU_D = forceRef(self.db.translate('rbExpenseServiceItem', 'code', '230003', 'id'))
        self.expenseTARU_DM = forceRef(self.db.translate('rbExpenseServiceItem', 'code', '230004', 'id'))
        self.expenseTARU_UC = forceRef(self.db.translate('rbExpenseServiceItem', 'code', '230005', 'id'))
        self.summKMultiplierId = forceRef(self.db.translate('rbContractCoefficientType', 'code', '230058', 'id'))
        self.summUcMultiplierId = forceRef(self.db.translate('rbContractCoefficientType', 'code', '230066', 'id'))

        if not self.unitVisitId:
            self.log.append(u'<b><font color=red>ОШИБКА:</b>'
             u' Не найдена единица учета медицинской помощи'
             u' (rbMedicalAidUnit) с кодом "01" (Посещение)')
            return

        if not self.unitServiceId:
            self.log.append(u'<b><font color=red>ОШИБКА:</b>'
             u' Не найдена единица учета медицинской помощи'
             u' (rbMedicalAidUnit) с кодом "18" (МЭС)')
            return

        if not self.eventType37:
            self.log.append(u'<b><font color=red>ОШИБКА:</b>'
             u' Не найден тип события'
             u' (EventType) с кодом "37" (Стационарное лечение)')
            return

        if not self.expenseBase:
            self.log.append(u'<b><font color=red>ОШИБКА:</b>'
             u' Не найдена статья затрат услуг'
             u' (rbExpenseServiceItem) с кодом "1" (базовая)')
            return

        if not self.expenseAdditional:
            self.log.append(u'<b><font color=red>ОШИБКА:</b>'
             u' Не найдена статья затрат услуг'
             u' (rbExpenseServiceItem) с кодом "2" (дополнительная)')
            return

        if not self.expenseTARU:
            self.log.append(u'<b><font color=red>ОШИБКА:</b>'
             u' Не найдена статья затрат услуг'
             u' (rbExpenseServiceItem) с кодом "230001" (TARU)')
            return

        if not self.expenseTARU_B:
            self.log.append(u'<b><font color=red>ОШИБКА:</b>'
             u' Не найдена статья затрат услуг'
             u' (rbExpenseServiceItem) с кодом "230002" (TARU_B)')
            return

        if not self.expenseTARU_D:
            self.log.append(u'<b><font color=red>ОШИБКА:</b>'
             u' Не найдена статья затрат услуг'
             u' (rbExpenseServiceItem) с кодом "230003" (TARU_D)')
            return

        if not self.expenseTARU_DM:
            self.log.append(u'<b><font color=red>ОШИБКА:</b>'
             u' Не найдена статья затрат услуг'
             u' (rbExpenseServiceItem) с кодом "230004" (TARU_DM)')
            return

        if not self.expenseTARU_UC:
            self.log.append(u'<b><font color=red>ОШИБКА:</b>'
             u' Не найдена статья затрат услуг'
             u' (rbExpenseServiceItem) с кодом "230005" (TARU_UC)')
            return

        if not self.summKMultiplierId:
            self.log.append(u'<b><font color=red>ОШИБКА:</b>'
             u' Не найден тип тарифного коэффициента'
             u' (rbContractCoefficientType) с кодом "230058" (Коэффициент справочника SPR58)')
            return

        if not self.summUcMultiplierId:
            self.log.append(u'<b><font color=red>ОШИБКА:</b>'
             u' Не найден тип тарифного коэффициента'
             u' (rbContractCoefficientType) с кодом "230066" (Коэффициент справочника SPR66)')
            return

        self.dupAskUser = self.chkAskUser.isChecked()
        self.dupUpdate = self.chkUpdate.isChecked()
        self.dupSkip = self.chkSkip.isChecked()

        dbfFileName = forceStringEx(self.edtSpr18.text())
        dbfSpr18 = Dbf(dbfFileName, readOnly=True, encoding='cp866')
        dbfFileName = forceStringEx(self.edtSpr22.text())
        dbfSpr22 = Dbf(dbfFileName, readOnly=True, encoding='cp866')
        dbfFileName = forceStringEx(self.edtSpr58.text())
        dbfSpr58 = Dbf(dbfFileName, readOnly=True, encoding='cp866')
        dbfFileName = forceStringEx(self.edtSpr66.text())
        dbfSpr66 = Dbf(dbfFileName, readOnly=True, encoding='cp866')
        self.progressBar.setValue(0)
        self.progressBar.setFormat('%v')
        rowCount = max(len(dbfSpr18)+len(dbfSpr22)+len(dbfSpr58)+len(dbfSpr66)-1, 1)
        self.progressBar.setMaximum(rowCount)
        self.labelNum.setText(u'всего записей в источнике: %i' % (rowCount))
        self.federalPriceFieldName = 'TARIF_M' if 'TARIF_M' in dbfSpr22.header.fields else 'TARIF_DM'

        for (dbfFile, proc) in (
                            (dbfSpr58, self.processSpr58),
                            (dbfSpr66, self.processSpr66),
                            (dbfSpr18, self.processSpr18),
                            (dbfSpr22, self.processSpr22)
                        ):
            self.process(dbfFile, proc)
            dbfFile.close()

            if self.abort:
                break

        self.log.append(u'добавлено: %d; изменено: %d' % (self.nAdded, self.nUpdated))
        self.log.append(u'пропущено: %d; обработано: %d' % (self.nSkipped, self.nProcessed))
        self.log.append(u'готово')


    def process(self, dbf, step):
        self.log.append(u'Обработка `%s`....' % dbf.name)

        for row in dbf:
            QtGui.qApp.processEvents()
            if self.abort:
                self.reject()
                return
            self.progressBar.setValue(self.progressBar.value()+1)
            try:
                step(row)
            except:
                QtGui.qApp.logCurrentException()
                raise


    def processSpr18(self, row):
        amount = forceDouble(row['KOLICH'])
        uet = forceDouble(row['UET'])
        var = forceDouble(row['VAR'])
        key = forceString(row['CODE'].strip())

        self.amountUetCache[key] = (amount, uet, var)


    def processSpr22(self, row):
        tariffDate = QDate(row['DATN'])
        serviceCode = forceString(row['KUSL'].strip())
        serviceId = self.findServiceByInfisCode(serviceCode)

        if not serviceId:
            self.log.append(u'<b><font color=orange>ОШИБКА:</b>'
                u' Услуга, код ИНФИС: "%s", не найдена' % serviceCode)
            self.nSkipped += 1
            return

        (amount, uet, var) = self.amountUetCache.get(serviceCode, (0, 0, 0))
        newRecord = self.tblContract_Tariff.newRecord()
        newRecord.setValue('price', toVariant(row['TARIF']))
        newRecord.setValue('federalPrice', toVariant(row[self.federalPriceFieldName]))
        newRecord.setValue('amount', toVariant(amount))
        newRecord.setValue('uet', toVariant(uet))
        newRecord.setValue('service_id', serviceId)
        newRecord.setValue('master_id', toVariant(self.contractId))
        newRecord.setValue('begDate', toVariant(tariffDate))

        if serviceCode[:1] in ('A', 'B', 'K'):
            newRecord.setValue('tariffType', toVariant(CTariff.ttActionAmount))
        elif serviceCode[:1] == 'S':
            newRecord.setValue('tariffType', toVariant(CTariff.ttEventByMESLen))
            newRecord.setValue('eventType_id', toVariant(self.eventType37))

            if amount != 1:
                newRecord.setValue('frag1Start', toVariant(1))
                newRecord.setValue('frag2Start', toVariant(amount - var))
                newRecord.setValue('frag2Sum', toVariant(row['TARIF']))

            newRecord.setValue('amount', toVariant(amount + var))
        else:
            self.log.append(u'<b><font color=orange>ОШИБКА:</b>'
                u' Услуга, код ИНФИС: "%s", нет правила для добавления тарифа. Сообщите разработчику.' % serviceCode)
            self.nSkipped += 1
            return

        baseExpense = self.tblContract_CompositionExpense.newRecord()
        baseExpense.setValue('percent', toVariant(100*row['TARIF_B']/row['TARIF'] if row['TARIF'] != 0 else row['TARIF_B']))
        baseExpense.setValue('rbTable_id', toVariant(self.expenseBase))

        additionalExpense = self.tblContract_CompositionExpense.newRecord()
        additionalExpense.setValue('percent', toVariant(100*row['TARIF_D']/row['TARIF'] if row['TARIF'] != 0 else row['TARIF_D']))
        additionalExpense.setValue('rbTable_id', toVariant(self.expenseAdditional))

        expenseTARU = self.tblContract_CompositionExpense.newRecord()
        expenseTARU.setValue('sum', toVariant(row['TARIF']))
        expenseTARU.setValue('rbTable_id', toVariant(self.expenseTARU))

        expenseTARU_B = self.tblContract_CompositionExpense.newRecord()
        expenseTARU_B.setValue('sum', toVariant(row['TARIF_B']))
        expenseTARU_B.setValue('rbTable_id', toVariant(self.expenseTARU_B))

        expenseTARU_D = self.tblContract_CompositionExpense.newRecord()
        expenseTARU_D.setValue('sum', toVariant(row['TARIF_D']))
        expenseTARU_D.setValue('rbTable_id', toVariant(self.expenseTARU_D))

        expenseTARU_DM = self.tblContract_CompositionExpense.newRecord()
        expenseTARU_DM.setValue('sum', toVariant(row['TARIF_DM']))
        expenseTARU_DM.setValue('rbTable_id', toVariant(self.expenseTARU_DM))

        expenseTARU_UC = self.tblContract_CompositionExpense.newRecord()
        expenseTARU_UC.setValue('sum', toVariant(row['TARIF_UC']))
        expenseTARU_UC.setValue('rbTable_id', toVariant(self.expenseTARU_UC))

        expenses = [baseExpense, additionalExpense,
                                expenseTARU, expenseTARU_B, expenseTARU_D,
                                expenseTARU_DM, expenseTARU_UC]

        self.log.append(u'Тариф: `%s`, цена %.2f, количество %.2f, дата %s'
                        % (serviceCode, forceDouble(newRecord.value('price')),
                                                    forceDouble(newRecord.value('amount')),
                                                    forceDate(newRecord.value('begDate')).toString('dd.MM.yyyy')))
        self.log.append(u'Затраты: базовые %.2f, дополнительные %.2f' % (
                                                                         forceDouble(baseExpense.value('percent')),
                                                                         forceDouble(additionalExpense.value('percent'))))

        self.addOrUpdateTariff(serviceCode, newRecord, expenses)


    def addOrUpdateTariff(self, serviceCode, newTariff, newExpenses):
        tariffType = forceInt(newTariff.value('tariffType'))
        serviceId = forceRef(newTariff.value('service_id'))
        sex = forceInt(newTariff.value('sex'))
        age = forceString(newTariff.value('age'))
        newBegDate = forceDate(newTariff.value('begDate'))
        category = forceRef(newTariff.value('tariffCategory_id'))

        key = (tariffType, serviceId, sex, age, category)
        tariffIndexList = self.tariffDict.get(key, None)
        if tariffIndexList:
            self.log.append(u'Найден совпадающий тариф.')

            if self.dupSkip:
                self.log.append(u'Пропускаем.')
                self.nSkipped += len(tariffIndexList)
                return

            for i in tariffIndexList:
                tariff = self.tariffList[i]
                diffStr = getTariffDifference(tariff,
                    self.tariffExpenseItems[self.tariffList.index(tariff)],
                    newTariff, newExpenses)
                # Если есть полное совпадение, выходим
                if not diffStr:
                    self.log.append(u'Количество, цены и ограничения совпадают, пропускаем.')
                    self.nSkipped += 1
                    return

                oldBegDate = forceDate(tariff.value('begDate'))

                if oldBegDate != newBegDate:
                    continue

                if self.dupUpdate:
                    self.log.append(u'Обновляем.')
                    self.tariffExpenseItems[self.tariffList.index(tariff)] = newExpenses
                    copyTariff(tariff, newTariff)
                    self.nUpdated += 1
                else:
                    self.log.append(u'Запрос действий у пользователя.')
                    answer = QtGui.QMessageBox.question(self, u'Совпадающий тариф',
                                        u'Услуга "%s"\nРазличия: %s\n'
                                        u'Обновить?' %(serviceCode, diffStr),
                                        QtGui.QMessageBox.No|QtGui.QMessageBox.Yes,
                                        QtGui.QMessageBox.No)
                    self.log.append(u'Выбор пользователя %s' % \
                    (u'обновить' if answer == QtGui.QMessageBox.Yes else u'пропустить'))
                    if answer == QtGui.QMessageBox.Yes:
                        self.tariffExpenseItems[self.tariffList.index(tariff)] = newExpenses
                        copyTariff(tariff, newTariff)
                        self.nUpdated += 1
                    else:
                        self.nSkipped += 1
                 # изменения в тариф внесены, выходим
                return

            #  с новым тарифом не совпали даты. Добавляем
            self.log.append(u'Добавляем тариф с %s' % newBegDate.toString('dd.MM.yyyy'))
            self.appendTariff(newTariff, newExpenses, tariffIndexList)
        else:
            self.log.append(u'Добавляем тариф для услуги %s' % serviceCode)
            self.nAdded += 1
            self.addTariff(newTariff)
            self.tariffExpenseItems.append(newExpenses)


    def appendTariff(self, newTariff, newExpenses, tariffIndexList):
        newBegDate = forceDate(newTariff.value('begDate'))
        closestBegDate = QDate()
        isModified = False

        for i in tariffIndexList:
            tariff = self.tariffList[i]

            begDate = forceDate(tariff.value('begDate'))
            endDate = forceDate(tariff.value('endDate'))

            # новый тариф попал внутрь ищеющегося
            if begDate and endDate and begDate <= newBegDate <= endDate:
                tariff.setValue('endDate', toVariant(newBegDate.addDays(-1)))
                newTariff.setValue('endDate', toVariant(endDate))
                isModified = True
                break

            # тариф без окончания срока действия
            if begDate and not endDate:
                if begDate <= newBegDate:
                    tariff.setValue('endDate',
                                    toVariant(newBegDate.addDays(-1)))
                if begDate > newBegDate:
                    newTariff.setValue('endDate',
                                       begDate.addDays(-1))

                isModified = True
                break

            if begDate and endDate:
                if begDate > newBegDate:
                    if closestBegDate and (closestBegDate.toJulianDay() - begDate.toJulianDay()) > 0:
                        closestBegDate = begDate

        if not isModified:
            newTariff.setValue('endDate', closestBegDate.addDays(-1))


        self.nAdded += 1
        self.addTariff(newTariff)
        self.tariffExpenseItems.append(newExpenses)

# ******************************************************************************

    def processSpr58(self, row):
        date = QDate(row['DATN'])
        value = forceDouble(row['VALUE2'])
        self.setContractCoefficient(self.summKMultiplierId, date, value)

# ******************************************************************************

    def processSpr66(self, row):
        date = QDate(row['DATN'])
        value = forceDouble(row['TARIF'])
        self.setContractCoefficient(self.summUcMultiplierId, date, value)

# ******************************************************************************

    def setContractCoefficient(self, typeId, date, value):
        isUpdated = False

        for record in self.tariffCoefficientItems:
            oldTypeId = forceRef(record.value('coefficientType_id'))

            if typeId == oldTypeId:
                oldBegDate = forceDate(record.value('begDate'))

                if oldBegDate == date:
                    record.setValue('value', toVariant(value))
                    isUpdated = True
                    break

        if not isUpdated:
            record = self.tblContract_Coefficient.newRecord()
            record.setValue('coefficientType_id', toVariant(typeId))
            record.setValue('begDate', toVariant(date))
            record.setValue('value', toVariant(value))
            self.tariffCoefficientItems.append(record)

# ******************************************************************************

    def checkFileName(self):
        flag = not self.edtSpr18.text().isEmpty() and \
                    not self.edtSpr22.text().isEmpty() and \
                    not self.edtSpr58.text().isEmpty() and \
                    not self.edtSpr66.text().isEmpty()

        self.btnImport.setEnabled(flag)

# ******************************************************************************

    @pyqtSignature('')
    def on_btnSelect18_clicked(self):
        fileName = QtGui.QFileDialog.getOpenFileName(
            self, u'Укажите файл с данными', self.edtSpr18.text(), u'Файлы DBF (*.dbf)')
        if fileName != '':
            self.edtSpr18.setText(QDir.toNativeSeparators(fileName))
            self.checkFileName()


    @pyqtSignature('')
    def on_btnSelect22_clicked(self):
        fileName = QtGui.QFileDialog.getOpenFileName(
            self, u'Укажите файл с данными', self.edtSpr22.text(), u'Файлы DBF (*.dbf)')
        if fileName != '':
            self.edtSpr22.setText(QDir.toNativeSeparators(fileName))
            self.checkFileName()


    @pyqtSignature('')
    def on_btnSelect58_clicked(self):
        fileName = QtGui.QFileDialog.getOpenFileName(
            self, u'Укажите путь к файлу SPR58.DBF', self.edtSpr58.text(),
            u'Файлы DBF (*.dbf)')
        if fileName != '':
            self.edtSpr58.setText(QDir.toNativeSeparators(fileName))
            self.checkFileName()


    @pyqtSignature('')
    def on_btnSelect66_clicked(self):
        fileName = QtGui.QFileDialog.getOpenFileName(
            self, u'Укажите путь к файлу SPR66.DBF', self.edtSpr66.text(),
            u'Файлы DBF (*.dbf)')
        if fileName != '':
            self.edtSpr66.setText(QDir.toNativeSeparators(fileName))
            self.checkFileName()
