# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
u"""Импорт тарифа из XML для Архангельска"""
from PyQt4 import QtGui
from PyQt4.QtCore import Qt, QDate, QFile, QString, pyqtSignature

from library.crbcombobox import CRBComboBox
from library.Utils import forceBool, forceDouble, forceString, toVariant

from Accounting.Tariff import CTariff
from Exchange.Cimport import CXMLimport, CImportTariffsMixin

from Exchange.Ui_ImportTariffsR29 import Ui_ImportTariffsR29
from Exchange.Ui_ImportTariffsR29Filter import Ui_FilterDialog


def ImportTariffsR29(widget, contractId, begDate, endDate, tariffList, tariffExpenseItems, tariffCoefficientItems):
    appPrefs = QtGui.qApp.preferences.appPrefs

    fileName = forceString(appPrefs.get('ImportTariffsR29FileName', ''))
    fullLog = forceBool(appPrefs.get('ImportTariffsR29FullLog', ''))
    updateTariff = forceBool(appPrefs.get('ImportTariffsR29UpdatePrices', ''))
    onlyCodes = forceBool(appPrefs.get('ImportTariffsR29OnlyCodes', ''))

    dlg = CImportTariffs(widget, fileName, contractId, begDate, endDate,
                         tariffList, tariffExpenseItems, tariffCoefficientItems)

    dlg.chkFullLog.setChecked(fullLog)
    dlg.chkUpdateTariff.setChecked(updateTariff)
    dlg.chkOnlyCodes.setChecked(onlyCodes)

    dlg.exec_()

    appPrefs['ImportTariffsR29FileName'] = toVariant(dlg.fileName)
    appPrefs['ImportTariffsR29FullLog'] = toVariant(dlg.chkFullLog.isChecked())
    appPrefs['ImportTariffsR29UpdatePrices'] = toVariant(dlg.chkUpdateTariff.isChecked())
    appPrefs['ImportTariffsR29OnlyCodes'] = toVariant(dlg.chkOnlyCodes.isChecked())
    return (not dlg.abort), dlg.tariffList,  dlg.tariffExpenseItems, dlg.tariffCoefficientItems



class CFilterDialog(QtGui.QDialog, Ui_FilterDialog):
    def __init__(self, parent, departmentList, subDepartmentList, serviceList,
                 serviceCategoryList, stomatologyCodeList, stomatologyNameList):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)

        self._departments = departmentList
        self._subDeparments = subDepartmentList
        self._services = serviceList
        self._categories = serviceCategoryList

        self.edtInfis.setText(forceString(QtGui.qApp.db.translate('Organisation',
            'id', QtGui.qApp.currentOrgId(), 'infisCode')))
        self.cmbTariffType.addItems(CTariff.tariffTypeNames)
        self.cmbEventType.setShowFields(CRBComboBox.showCodeAndName)
        self.cmbEventType.setTable('EventType')
        self.cmbDepartment.addItem('-')
        self.cmbDepartment.addItems(departmentList)

        if '-' not in serviceCategoryList:
            self.cmbServiceCategory.addItem('-')

        self.cmbServiceCategory.addItems(serviceCategoryList)
        self.cmbStomatologyCode.addItem('-')
        self.cmbStomatologyCode.addItems(stomatologyCodeList)
        self.cmbStomatologyName.addItem('-')
        self.cmbStomatologyName.addItems(stomatologyNameList)

    @pyqtSignature('int')
    def on_cmbDepartment_currentIndexChanged(self, index):
        self.cmbSubDepartment.clear()
        self.cmbSubDepartment.addItem('-')

        for (dep, name) in self._subDeparments:
            if index == 0 or dep == forceString(self.cmbDepartment.currentText()):
                self.cmbSubDepartment.addItem(name, toVariant(dep))


    @pyqtSignature('int')
    def on_cmbSubDepartment_currentIndexChanged(self, index):
        self.cmbServiceCode.clear()
        self.cmbServiceName.clear()
        self.cmbServiceCode.addItem('-')
        self.cmbServiceName.addItem('-')

        currentDep= forceString(self.cmbDepartment.currentText())
        currentSubDep = forceString(self.cmbSubDepartment.currentText())
        depIndex = self.cmbDepartment.currentIndex()

        for ((dep, subDep), (serviceCode,  serviceName)) in self._services:
            if ((index == 0 or subDep == currentSubDep) and
                    (depIndex == 0 or dep == currentDep)):
                self.cmbServiceCode.addItem(serviceCode)
                self.cmbServiceName.addItem(serviceName)


class CImportTariffs(QtGui.QDialog, Ui_ImportTariffsR29, CXMLimport, CImportTariffsMixin):
    headerFields = ('VERSION', 'EDITION', 'DATE')
    departmentFields = ('CODE', 'RAZD_NAME', 'USL_OK')
    uetFields = ('CODE', 'USL_NAME', 'DATE_BEG', 'DATE_END', 'USL_UET_V', 'USL_UET_D')
    subDepartmentFields = ('CODE', 'PODRAZD_NAME', 'USL_KAT', 'IDSP')
    serviceFields = ('CODE', 'USL_NAME', 'DATE_BEG', 'DATE_END', 'PROFIL')
    tariffFields = ('TGROUP', 'TARIF_V', 'TARIF_D', 'DATE_BEG', 'DATE_END',  'IDMSP', 'IDPOL')
    additionalServiceFields = ('CODE', 'TYPE_DOP', 'OBZ')
    moFields = ('MCOD', 'TGROUP', 'DATE_BEG', 'DATE_END', 'LPU_1', 'NAM_MOP')

    tariffGroup = {'TARIF':{'fields': tariffFields}}
    tariffListGroup = {'TARIF_LIST': {'subGroup': tariffGroup}}
    additionalServiceGroup = {'DOP_USL': {'fields':additionalServiceFields}}
    serviceGroup = {'USL': {'fields': serviceFields,  'subGroup':tariffListGroup}}
    serviceListGroup = {
        'USL_LIST': {'subGroup': serviceGroup},
        'DOP_USL_LIST': {'subGroup': additionalServiceGroup}
    }

    subDepartmentGroup = {'PODRAZD': {'fields': subDepartmentFields, 'subGroup': serviceListGroup}}
    subDepartmentListGroup = {'PODRAZD_LIST': {'subGroup': subDepartmentGroup}}
    departmentListGroup = {'RAZD': { 'fields': departmentFields, 'subGroup': subDepartmentListGroup}}
    uetListGroup = {'UET': {'fields': uetFields}}
    moGroup = {'MO':{'fields': moFields}}

    tariffsGroupName = 'TARIFFS'
    tariffsGroup = {
        'ZGLV':  {'fields': headerFields},
        'RAZD_LIST': {'subGroup': departmentListGroup},
        'MO_LIST': {'subGroup': moGroup},
        'UET_LIST': {'subGroup': uetListGroup},
    }


    def __init__(self, parent, fileName, contractId, begDate, endDate, tariffList, tariffExpenseItems, tariffCoefficientItems):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)
        CXMLimport.__init__(self)
        CImportTariffsMixin.__init__(self,  self.log, tariffList, tariffExpenseItems)

        self.fileName = fileName
        self.tariffCoefficientItems = list(tariffCoefficientItems)
        self.contractId = contractId

        if fileName:
            self.edtFileName.setText(fileName)
            self.btnImport.setEnabled(True)

        self._unitIdCache = {}
        self.findUnitId = lambda code: self.findIdByCode(code, 'rbMedicalAidUnit', self._unitIdCache)

        self._serviceCache = {}
        self.findService = lambda code, name: self.lookupIdByCodeName(code, name, self.tblService, self._serviceCache)


    def setImportMode(self, flag):
        self.chkFullLog.setEnabled(not flag)
        self.chkOnlyCodes.setEnabled(not flag)
        self.chkUpdateTariff.setEnabled(not flag)
        self.btnSelectFile.setEnabled(not flag)
        self.edtFileName.setEnabled(not flag)


    def startImport(self):
        fileName = self.edtFileName.text()
        self.setImportMode(True)

        if not fileName:
            return

        inFile = QFile(fileName)
        if not inFile.open(QFile.ReadOnly | QFile.Text):
            QtGui.QMessageBox.warning(self, u'Импорт тарифов для договора',
                                      QString(u'Не могу открыть файл для чтения %1:\n%2.')
                                      .arg(fileName)
                                      .arg(inFile.errorString()))
            return

        self.progressBar.setFormat(u'%v байт')
        self.progressBar.setMaximum(max(inFile.size(), 1))
        self.lblStatus.setText('')

        self.setDevice(inFile)

        while (not self.atEnd()):
            self.readNext()

            if self.isStartElement():
                if self.name() == self.tariffsGroupName:
                    xmlData = self.readGroupEx(self.tariffsGroupName, tuple(), False, self.tariffsGroup)
                else:
                    self.raiseError(u'Неверный формат данных.')

            if self.hasError():
                break

        if not self.hasError():
            filter = self.showFilterDialog(xmlData)

            if filter != {}:
                self.processData(xmlData, filter)
                self.progressBar.setText(u'Готово')
            else:
                self.abort = True
                self.err2log(u'! Прервано пользователем.')
        else:
            self.progressBar.setText(u'Прервано')
            if self.abort:
                self.err2log(u'! Прервано пользователем.')
            else:
                self.err2log(u'! Ошибка: файл %s, %s' % (fileName,self.errorString()))

        inFile.close()
        self.setImportMode(False)


    def showFilterDialog(self, data):
        depData = data.get('RAZD_LIST',{}).get('RAZD',[])
        departmentList =  []
        subDepartmentList = []
        serviceList = []
        serviceCategoryList = set()

        for d in depData:
            depName ='%s|%s' % (d.get('CODE', '-'), d.get('RAZD_NAME', ''))
            departmentList.append(depName)
            subDepList = d.get('PODRAZD_LIST', {}).get('PODRAZD', [])

            if not isinstance(subDepList, list):
                subDepList = [subDepList]

            for subDep in subDepList:
                subDepName = '%s|%s' % (subDep.get('CODE', '-'), subDep.get('PODRAZD_NAME', ''))
                subDepartmentList.append((depName, subDepName))
                serviceCategoryList.add(subDep.get('USL_KAT', '-'))
                srvList = subDep.get('USL_LIST', {}).get('USL', [])

                if not isinstance(srvList, list):
                    subDep['USL_LIST']['USL'] = [srvList]
                    srvList = subDep['USL_LIST']['USL']

                for service in srvList:
                    serviceCode = service.get('CODE', '-')
                    serviceName = service.get('USL_NAME', '')
                    serviceList.append(((depName, subDepName), (serviceCode, serviceName)))

        stomatologyCodes = set()
        stomatologyNames = set()

        uetList = data.get('UET_LIST', {}).get('UET', [])

        if not isinstance(uetList, list):
            data.get['UET_LIST']['UET'] = [uetList]
            uetList = data.get['UET_LIST']['UET']

        for uet in uetList:
            stomatologyCodes.add(uet.get('CODE', ''))
            stomatologyNames.add(uet.get('USL_NAME', ''))

        dlg = CFilterDialog(self, departmentList, subDepartmentList,
            serviceList, list(serviceCategoryList), list(stomatologyCodes),
            list(stomatologyNames))

        ok = dlg.exec_()
        result = {}

        if ok:
            result['infisCode'] = forceString(dlg.edtInfis.text())
            result['tariffType'] = dlg.cmbTariffType.currentIndex()
            result['eventType'] = dlg.cmbEventType.value()
            result['department'] = forceString(dlg.cmbDepartment.currentText())
            result['subDepartment'] = forceString(dlg.cmbSubDepartment.currentText())
            result['serviceCode'] = forceString(dlg.cmbServiceCode.currentText())
            result['serviceName'] = forceString(dlg.cmbServiceName.currentText())
            result['serviceCategory'] = forceString(dlg.cmbServiceCategory.currentText())
            result['stomatologyCode'] = forceString(dlg.cmbStomatologyCode.currentText())
            result['stomatologyName'] = forceString(dlg.cmbStomatologyName.currentText())

            for (key, val) in result.items():
                if val == '-':
                    result[key] = None

        return result


    def processData(self, data, filterDict):
        self.progressBar.setText(u'Обработка данных...')
        self.progressBar.setMaximum(0)

        infisCode = filterDict.get('infisCode') #'290004'
        tariffGroupList = set()

        if infisCode:
            orgList = data.get('MO_LIST', {}).get('MO', [])

            if not isinstance(orgList, list):
                orgList = [orgList]

            filteredList = filter(lambda x: x.get('MCOD') == infisCode, orgList)
            tariffGroupList = [x.get('TGROUP') for x in filteredList]

        depFilter = filterDict.get('department')
        subDepFilter = filterDict.get('subDepartment')
        serviceCodeFilter = filterDict.get('serviceCode')
        serviceNameFilter = filterDict.get('serviceName')
        serviceCategoryFilter = filterDict.get('serviceCategory')
        stomatologyCodeFilter = filterDict.get('stomatologyCode')
        stomatologyNameFilter = filterDict.get('stomatologyName')

        # фильтр по имени раздела
        depList = filter(lambda d: '%s|%s' % (d.get('CODE', '-'),
                                d.get('RAZD_NAME', '')) == depFilter,
                                data.get('RAZD_LIST',{}).get('RAZD',[])) \
                            if depFilter else data.get('RAZD_LIST',{}).get('RAZD',{})

        for dep in depList:
            QtGui.qApp.processEvents()
            # фильтр по имени подраздела
            subDepList = filter(lambda d: '%s|%s' % (d.get('CODE', '-'),
                                d.get('PODRAZD_NAME', '')) == subDepFilter,
                                dep.get('PODRAZD_LIST',{}).get('PODRAZD',[])) \
                            if subDepFilter else dep.get('PODRAZD_LIST', {}).get('PODRAZD', [])

            #фильтр по категории
            subDepList = filter(lambda x: x.get('USL_KAT') == serviceCategoryFilter,
                                subDepList) if serviceCategoryFilter else subDepList

            for subDep in subDepList:
                QtGui.qApp.processEvents()
                # фильтр по частичному совпадению кода
                serviceList = filter(lambda x: x.get('CODE', '')[:len(serviceCodeFilter)] == serviceCodeFilter,
                        subDep.get('USL_LIST', {}).get('USL', [])) \
                            if serviceCodeFilter else subDep.get('USL_LIST', {}).get('USL', [])

                # фильр по частичному совпадению имени
                serviceList = filter(lambda x: x.get('USL_NAME', '')[:len(
                                    serviceNameFilter)] == serviceNameFilter, serviceList) \
                                        if serviceNameFilter else serviceList

                unitId = self.findUnitId(subDep.get('IDSP'))

                if not unitId and subDep.has_key('IDSP'):
                    self.err2log(u'Единица учета медицинской помощи не найдена: `%s`' % subDep['IDSP'])

                for service in serviceList:
                    QtGui.qApp.processEvents()
                    if not isinstance(service.get('TARIF_LIST', {}).get('TARIF', []), list):
                        service['TARIF_LIST']['TARIF'] = [service['TARIF_LIST']['TARIF']]

                    serviceCode = service.get('CODE', '')
                    serviceName = service.get('USL_NAME', '')
                    serviceId = self.findService(serviceCode, serviceName)

                    if not serviceId:
                        self.err2log(u'Услуга `%s` `%s` не найдена, пропускаем' % (serviceCode, serviceName))
                        continue

                    tariffList = filter(lambda t: t.get('TGROUP') in tariffGroupList,
                                service.get('TARIF_LIST', {}).get('TARIF', []))
#                    tariffList = service.get('TARIF_LIST', {}).get('TARIF', [])
                    map(lambda x: self.processTariff(x, serviceCode, serviceId, unitId, filterDict), tariffList)

        uetList = data.get('UET_LIST', {}).get('UET', [])

        uetList = filter(lambda x: x.get('CODE', '')[:len(stomatologyCodeFilter
                                )] == stomatologyCodeFilter, uetList) \
                                    if stomatologyCodeFilter else uetList

        uetList = filter(lambda x: x.get('USL_NAME', '')[:len(stomatologyNameFilter
                                )] == stomatologyNameFilter, uetList) \
                                    if stomatologyNameFilter else uetList

        map(lambda x: self.processStomatology(x, filterDict), uetList)
        self.progressBar.setMaximum(100)
        self.progressBar.setValue(100)


    def processTariff(self, tariff, serviceCode, serviceId, unitId, data):
        begDate = QDate.fromString(tariff.get('DATE_BEG', ''), Qt.ISODate)
        endDate = QDate.fromString(tariff.get('DATE_END', ''), Qt.ISODate)
        maturePrice = forceDouble(tariff.get('TARIF_V', 0.0))
        childPrice = forceDouble(tariff.get('TARIF_D', 0.0))
        tariffType = data.get('tariffType', CTariff.ttVisit)

        for (price, age) in ((maturePrice, u'18г-'), (childPrice, u'-17г')):
            if price > 0.0:
                tariff = self.tblContract_Tariff.newRecord()
                tariff.setValue('price', price)
                tariff.setValue('service_id',  toVariant(serviceId))
                tariff.setValue('master_id',  toVariant(self.contractId))
                tariff.setValue('begDate', toVariant(begDate))
                tariff.setValue('endDate', toVariant(endDate))
                tariff.setValue('tariffType',  toVariant(tariffType))
                tariff.setValue('unit_id', toVariant(unitId))
                tariff.setValue('age', toVariant(age))
                self.addOrUpdateTariff(serviceCode, tariff, [])

        QtGui.qApp.processEvents()
        self.progressBar.step()


    def processStomatology(self, tariff, data):
        serviceCode = forceString(tariff.get('CODE'))
        serviceName = forceString(tariff.get('USL_NAME'))
        serviceId = self.findService(serviceCode, serviceName)

        if not serviceId:
            self.err2log(u'Услуга `%s` `%s` не найдена, пропускаем' % (serviceCode, serviceName))
            return

        begDate = QDate.fromString(tariff.get('DATE_BEG', ''), Qt.ISODate)
        endDate = QDate.fromString(tariff.get('DATE_END', ''), Qt.ISODate)
        maturePrice = forceDouble(tariff.get('USL_UET_V', 0.0))
        childPrice = forceDouble(tariff.get('USL_UET_D', 0.0))
        tariffType = data.get('tariffType', CTariff.ttVisit)

        for (uet, age) in ((maturePrice, u'18г-'), (childPrice, u'-17г')):
            if uet > 0.0:
                tariff = self.tblContract_Tariff.newRecord()
                tariff.setValue('uet', uet)
                tariff.setValue('service_id',  toVariant(serviceId))
                tariff.setValue('master_id',  toVariant(self.contractId))
                tariff.setValue('begDate', toVariant(begDate))
                tariff.setValue('endDate', toVariant(endDate))
                tariff.setValue('tariffType',  toVariant(tariffType))
                tariff.setValue('age', toVariant(age))
                self.addOrUpdateTariff(serviceCode, tariff, [])

        QtGui.qApp.processEvents()
        self.progressBar.step()
