# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
u"""Импорт тарифов для Ростовской области"""

from PyQt4 import QtGui
from PyQt4.QtCore import QDate, QDir, pyqtSignature

from library.dbfpy.dbf import Dbf
from library.Utils import forceString, toVariant, forceRef, forceStringEx

from Accounting.Tariff import CTariff
from Exchange.Cimport import CDBFimport, CImportTariffsMixin

from Exchange.Ui_ImportTariffsR61 import Ui_Dialog


def importTariffsR61(widget, contractId, begDate, endDate, tariffList,
                     tariffExpenseItems, _):
    u"""Запускает мастер импорта тарифов для Ростова"""

    dlg = CImportTariffs(widget, contractId, begDate, endDate, tariffList,
                       tariffExpenseItems)
    dlg.edtFileName.setText(forceString(QtGui.qApp.preferences.appPrefs.get('ImportTariffR61FileName', '')))
    dlg.exec_()
    QtGui.qApp.preferences.appPrefs['ImportTariffR61FileName'] = \
        toVariant(dlg.edtFileName.text())
    return not dlg.abort, dlg.tariffList, dlg.tariffExpenseItems, []


class CImportTariffs(QtGui.QDialog, Ui_Dialog, CDBFimport, CImportTariffsMixin):
    u"""Мастер импорта тарифов для Ростова"""

    def __init__(self, parent, contractId, begDate, endDate, tariffList,
                 tariffExpenseItems):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)
        CDBFimport.__init__(self, self.log)
        CImportTariffsMixin.__init__(self, self.log, tariffList,
                                     tariffExpenseItems)
        self.progressBar.setFormat('%v')
        self.contractId = contractId

        self.tariffType = None
        self.begDate = begDate
        self.endDate = endDate
        self.dupAskUser = False
        self.dupUpdate = False
        self.dupSkip = False

        self.nProcessed = 0

        self.tariffCategory = {}
        self._serviceCodeCache = {}

        self.unitId = None

        self.ageStr = None
        self.matureTariff = False
        self.expenseFields = None
        self.amount = 0
        self.infisCode = None
        self.priceFieldName = None
        self.zplExpenseId = None

        self.findServiceByCode = lambda code: self.findIdByCode(
                            code, 'rbService', self._serviceCodeCache, 'code')


    def startImport(self):
        if not self.contractId:
            return

        self.progressBar.setFormat('%v')

        dbfFileName = forceStringEx(self.edtFileName.text())
        self.tariffType = CTariff.ttActionAmount
        self.unitId = None
        self.nSkipped = 0
        self.nAdded = 0
        self.nUpdated = 0
        self.nProcessed = 0
        self.ageStr = u'18г-'
        self.dupAskUser = self.chkAskUser.isChecked()
        self.dupUpdate = self.chkUpdate.isChecked()
        self.dupSkip = self.chkSkip.isChecked()
        self.amount = 0

        if not self.setupExpenses():
            return

        dbfTariff = Dbf(dbfFileName, readOnly=True, encoding='cp866')
        self.progressBar.setMaximum(len(dbfTariff)-1)
        self.process(dbfTariff, self.processTariff)
        dbfTariff.close()

        self.log.append(u'добавлено: %d; изменено: %d' % (
                                                  self.nAdded, self.nUpdated))
        self.log.append(u'пропущено: %d; обработано: %d' % (
                                                self.nSkipped, self.nProcessed))
        self.log.append(u'готово')


    def setupExpenses(self):
        result = True
        self.zplExpenseId = forceRef(self.db.translate(
                'rbExpenseServiceItem', 'code', '1', 'id'))

        if not self.zplExpenseId:
            self.log.append(u'<b><font color=red>ОШИБКА:</b>'
             u' Не найдена статья затрат'
             u' (rbExpenseServiceItem) с кодом 1')
            result = False

        return result


    def process(self, dbf, step):
        for row in dbf:
            QtGui.qApp.processEvents()
            if self.abort:
                self.reject()
                return
            self.progressBar.setValue(self.progressBar.value()+1)
            try:
                step(row)
            except:
                QtGui.qApp.logCurrentException()
                raise


    def processTariff(self, row):
        tariffBegDate = QDate(row['DATAN'])
        tariffEndDate = QDate(row['DATAK'])
        zplExpense = row['ZPL']
        serviceCode = forceString(row['KSPEC'])
        price = row['STOIM']
        strTariffDate = forceString(tariffBegDate.toString('dd.MM.yyyy'))

        serviceId = self.findServiceByCode(serviceCode)

        if not serviceId:
            self.log.append(u'<b><font color=orange>ОШИБКА:</b>'
                u' Услуга, код: "%s", не найдена' % serviceCode)
            self.nSkipped += 1
        else:
            tariff = self.tblContract_Tariff.newRecord()
            tariff.setValue('price', price)
            tariff.setValue('amount', toVariant(self.amount))
            tariff.setValue('service_id', toVariant(serviceId))
            tariff.setValue('master_id', toVariant(self.contractId))
            tariff.setValue('begDate', toVariant(tariffBegDate))
            tariff.setValue('endDate', toVariant(tariffEndDate))
            tariff.setValue('tariffType', toVariant(self.tariffType))
            tariff.setValue('unit_id', toVariant(self.unitId))
            tariff.setValue('age', toVariant(self.ageStr))

            expense = self.tblContract_CompositionExpense.newRecord()
            expense.setValue('percent', toVariant(
                100*zplExpense/price if price != 0 else zplExpense))
            expense.setValue('rbTable_id', toVariant(self.zplExpenseId))

            self.log.append(u'Услуга: "%s". Дата "%s". Возраст "%s": <b>'
                u'<font color=green>"%.2f"</b>' % (serviceCode, strTariffDate,
                                                   self.ageStr, price))
            self.addOrUpdateTariff(serviceCode, tariff, [expense])
            self.nProcessed += 1


    @pyqtSignature('')
    def on_btnSelectFile_clicked(self):
        fileName = QtGui.QFileDialog.getOpenFileName(self,
            u'Укажите файл с данными', self.edtFileName.text(),
            u'Файлы DBF (*.dbf)')
        if fileName != '':
            self.edtFileName.setText(QDir.toNativeSeparators(fileName))
            self.btnImport.setEnabled(True)
            dbfFileName = forceStringEx(self.edtFileName.text())
            dbfTariff = Dbf(dbfFileName, readOnly=True, encoding='cp866')
            self.labelNum.setText(
                u'всего записей в источнике: '+str(len(dbfTariff)))
