# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2015-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

# поддержка получения полиса пациента по унифицированному протоколу

from PyQt4                  import QtCore, QtGui
from library.JsonRpc.client import CJsonRpcClent
from library.Utils          import forceString, formatSex, formatSNILS, smartDict

__all__ = ['CTFUnifiedIdentService',
          ]


class CTFUnifiedIdentService:
    def __init__(self, url, login=None, password=None):
        self.url = url
        self.login = login
        self.password = password


    def getPolicyAndAttach(self, firstName, lastName, patrName, sex, birthDate, snils, policySerial, policyNumber, docSerial, docNumber):

        def strToQDate(s):
            try:
                result = QtCore.QDate.fromString(s, QtCore.Qt.ISODate)
                if not result.isValid():
                    result = None
                return result
            except:
                return None


        dbServerName = QtGui.qApp.preferences.dbServerName
        # это "нечестный" вариант, но на первое время хватит
        #url = self.url.replace('%dbServerName%', dbServerName).replace('${dbServerName}', dbServerName)
        url = self.url.replace('${dbServerName}', dbServerName)
        client = CJsonRpcClent(url, self.login, self.password)
        resp = client.call('searchClientPolicyAndAttach',
                           dict(lastName     = lastName,
                                firstName    = firstName,
                                patrName     = patrName,
                                sex          = formatSex(sex),
                                birthDate    = unicode(birthDate.toString(QtCore.Qt.ISODate)) if birthDate else None,
                                snils        = formatSNILS(snils),
                                policySerial = policySerial,
                                policyNumber = policyNumber,
                                docSerial    = docSerial,
                                docNumber    = docNumber,
                                personId     = QtGui.qApp.userId,
                                workstation  = QtGui.qApp.hostName,
                               )
                          )
        if resp:
            policy = resp.get('policy')
            if policy:
                p = smartDict()
                p.hicId = policy.get('hicId', None)
                p.hicName = policy.get('hicName', '?')
                p.typeCode = policy.get('type', None)
                p.kindCode = policy.get('kind', None)
                p.policySerial = forceString(policy.get('serial', ''))
                p.policyNumber = forceString(policy.get('number', ''))
                p.begDate = strToQDate(policy.get('begDate', ''))
                p.endDate = strToQDate(policy.get('endDate', ''))
            else:
                p = None

            return p, resp.get('attach'), resp.get('message')
        else:
            return None, None, u'Ответ от сервера не получен'
