# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import os
import re
import datetime
from binascii import hexlify
from zipfile import ZIP_DEFLATED, ZipFile
from PyQt4 import QtGui, QtSql
from PyQt4.QtCore import Qt, QDate, QRegExp, QVariant

from library.database import connectDataBase
from library.dbfpy.dbf import Dbf
from library.exception import CException
from library.Utils import forceDate, forceInt, forceRef, forceString, forceStringEx, nameCase, toVariant, variantEq

# это скверная затея - Utils это что-то не спеифичное, общепотребительное.
# получается, что getTariffDifference и copyTariff ещё более общие?
# тогда почему они не в Utils ?
#from Exchange.ImportTariffsXML import getTariffDifference, copyTariff
from Registry.Utils import getClientAddress


def getId(table, fields, fields2=[]):
    db = QtGui.qApp.db
#    cond=[table[name].eq(toVariant(val)) for (name, val) in fields]
    cond=[]
    for (name, val) in fields:
        if val==None:
            cond.append(table[name].isNull())
        else:
            cond.append(table[name].eq(toVariant(val)))
    record=db.getRecordEx(table, '*', where=cond)
    if record:
        updateRecord = False
        for (name, val) in fields2:
            recVal=record.value(name)
            if (recVal.isNull() or forceString(recVal)=='') and isNotNull(val):
                record.setValue(name, toVariant(val))
                updateRecord = True
        if updateRecord:
            db.updateRecord(table, record)
        return forceInt(record.value('id'))
    else:
        record = table.newRecord()
        for (name, val) in fields+fields2:
            record.setValue(name, toVariant(val))
        return db.insertRecord(table, record)


def findAndUpdateOrCreateRecord(table, filterFields, updateFields=[], id=None):
    # вариант getId, обновляющий запись не только если поля пусты
    db = QtGui.qApp.db
    cond=[ table[name].isNull() if val is None else table[name].eq(toVariant(val))
           for (name, val) in filterFields
         ]
    if id:
        cond.append(table['id'].eq(id))
    record=db.getRecordEx(table, '*', where=cond)
    if record:
        updateRecord = False
        for (name, val) in updateFields:
            recVal=record.value(name)
            newVal=toVariant(val)
            if not variantEq(recVal, newVal):
                record.setValue(name, newVal)
                updateRecord = True
        if updateRecord:
            db.updateRecord(table, record)
        return forceInt(record.value('id'))
    else:
        record = table.newRecord()
        for (name, val) in filterFields:
            record.setValue(name, toVariant(val))
        for (name, val) in updateFields:
            record.setValue(name, toVariant(val))
        return db.insertRecord(table, record)


def checkPatientData(self, lastNameField, firstNameField, patrNameField, sexField, birthDateField):
    bad=False
    lastName=nameCase(self.row[lastNameField])
    firstName=nameCase(self.row[firstNameField])
    patrName=nameCase(self.row[patrNameField])
    if not (lastName and firstName and patrName):
        bad=True
        self.err2log(u'нет полного ФИО')
    fio=lastName+firstName+patrName
    if not check_rus_lat(fio):
        bad=True
        self.err2log(u'недопустимое ФИО')
    sex=self.row[sexField].upper()
    if sex == u'М':
        sex = 1
    elif sex == u'Ж':
        sex = 2
    else:
        self.err2log(u'не указан пол')

    birthDate=self.row[birthDateField]
    if not birthDate:
        bad=True
        self.err2log(u'не указан день рождения')
    if bad:
        return None
    else:
        clientFields=[
            ('lastName', lastName), ('firstName', firstName), ('patrName', patrName),
            ('sex', sex), ('birthDate', birthDate)]
        return checkData(clientFields, self.tableClient)

def checkData(fields, table):
    db = QtGui.qApp.db
    cond=[]
    for (name, val) in fields:
        if val==None:
            cond.append(table[name].isNull())
        else:
            cond.append(table[name].eq(toVariant(val)))
    record=db.getRecordEx(table, '*', where=cond)
    if record:
        return True
    return False


def isNull(s):
    return s in (None, '', 'Null', 'null', 'NULL')


def isNotNull(s):
    return s not in (None, '', 'Null', 'null', 'NULL')


def dbfCheckNames(d, names):
    for name in names:
        if name not in d.header.fields:
            return False
    return True


def getOKVED(okved):
    OKVED=''
    for c in okved:
        if c==' ': pass
        elif c=='-': break
        else: OKVED=OKVED+c
    if len(OKVED) and OKVED[0:1].isdigit():
        OKVED=QtGui.qApp.db.translate('rbOKVED', 'OKVED', OKVED, 'code')
    return OKVED

currentYear=QDate.currentDate().year()
currentYearBeg=datetime.date(currentYear, 1, 1)
currentYearEnd=datetime.date(currentYear, 12, 31)

def tbl(tn):
    t=QtGui.qApp.db.table(tn)
    assert t.fields
    return t

rus_re=re.compile(u'[абвгдеёжзийклмнопрстуфхцчшщъыьэюяАБВГДЕЁЖЗИЙКЛМНОПРСТУФХЦЧШЩЪЫЬЭЮЯ]')
lat_re=re.compile('[a-zA-Z]')

def check_rus_lat(s):
    rus=rus_re.search(s)!=None
    lat=lat_re.search(s)!=None
    return rus!=lat

def check_rus(s):
    return rus_re.search(s)

#WFT? это, товарищи, говнище: эта функция очень специфична, что она делает в Utils?
def get_specs(POL):
    if POL in (1, u'м', u'М'):
        POL_spec1, POL_spec2 = [], [('U', '84', 3)]
    else:
        POL_spec1, POL_spec2 = [('A', '02', 3)], []
    return [('T', '78', 5)]+POL_spec1+[('N', '40', 3), ('H', '89', 3)]+POL_spec2+[('O', '49', 3), ('E', '91', 3)]


def getWorkHurt(clientWorkId):
    hurt   = ''
    stage  = 0
    factors = ''
    if clientWorkId:
        db = QtGui.qApp.db
        stmt = u'''
SELECT
    rbHurtType.code AS hurt,
    ClientWork_Hurt.stage AS stage,
    GROUP_CONCAT(DISTINCT rbHurtFactorType.code ORDER BY rbHurtFactorType.code) AS factors
FROM
    ClientWork_Hurt
    LEFT JOIN rbHurtType ON rbHurtType.id = ClientWork_Hurt.hurtType_id
    LEFT JOIN ClientWork_Hurt_Factor ON ClientWork_Hurt_Factor.master_id = ClientWork_Hurt.id
    LEFT JOIN rbHurtFactorType ON rbHurtFactorType.id = ClientWork_Hurt_Factor.factorType_id
WHERE
    ClientWork_Hurt.master_id = %d AND
    ClientWork_Hurt.stage = (
        SELECT MAX(CWH.stage) FROM ClientWork_Hurt AS CWH WHERE CWH.master_id=%d)
GROUP BY ClientWork_Hurt.id
            ''' % (clientWorkId, clientWorkId)
        query = db.query(stmt)
        if query.next():
            record = query.record()
            hurt    = forceString(record.value(0))
            stage   = forceInt(record.value(1))
            factors = forceString(record.value(2))
    return hurt, stage, factors

def get_Account_Item(eventId):
    if not eventId: return None
    stmt='''
select
Account_Item.date, rbPayRefuseType.code, rbPayRefuseType.name
from
Account_Item
left join rbPayRefuseType on rbPayRefuseType.id=Account_Item.refuseType_id
where
Account_Item.event_id='''+str(eventId)+''' and Account_Item.date=(
    select max(a.date) from Account_Item as a where a.master_id=Account_Item.master_id)
        '''
    query=QtGui.qApp.db.query(stmt)
    if query.next():
        return query.record()
    return None

def getClientAddressEx(clientId):
    freeInput = ''
    KLADRCode = ''
    KLADRStreetCode = ''
    house = ''
    corpus = ''
    flat = ''

    db = QtGui.qApp.db
    regAddressRecord=getClientAddress(clientId, 0)
    if regAddressRecord:
        addressId=forceRef(regAddressRecord.value('address_id'))
        freeInput=forceString(regAddressRecord.value('freeInput'))
        if addressId:
            recAddress = db.getRecord('Address', 'house_id, flat', addressId)
            if recAddress:
                flat = forceString(recAddress.value('flat'))
                houseId = forceRef(recAddress.value('house_id'))
                if houseId:
                    recHouse = db.getRecord(
                        'AddressHouse', 'KLADRCode, KLADRStreetCode, number, corpus',  houseId)
                    KLADRCode=forceString(recHouse.value('KLADRCode'))
                    KLADRStreetCode=forceString(recHouse.value('KLADRStreetCode'))
                    house = forceString(recHouse.value('number'))
                    corpus = forceString(recHouse.value('corpus'))
    return freeInput, KLADRCode, KLADRStreetCode, house, corpus, flat



def getDiags(event_id, speciality):
    stmt='''
select
    Person.lastName, Person.firstName, Person.patrName, Person.code,
    Diagnostic.sanatorium, Diagnostic.endDate, Diagnostic.diagnosisType_id,
    Diagnosis.MKB, Diagnostic.character_id, Diagnostic.stage_id, Diagnostic.healthGroup_id,
    rbDispanser.code AS dispanser_code
from
    Diagnostic
    join Person on Diagnostic.person_id=Person.id
    join Diagnosis on Diagnosis.id=Diagnostic.diagnosis_id
    join rbSpeciality on rbSpeciality.id=Person.speciality_id
    left join rbDispanser on rbDispanser.id = Diagnostic.dispanser_id
where
    Diagnostic.event_id='''+str(event_id)+''' and rbSpeciality.code=\''''+str(speciality)+'''\'
    AND Diagnostic.deleted = 0 AND Diagnosis.deleted = 0
order
    by Diagnostic.diagnosisType_id
        '''
    query=QtGui.qApp.db.query(stmt)
    Diags=[]
    while query.next():
        Diags.append(query.record())
    return Diags

def getOtherDiags(event_id, specs):
    sp=' and rbSpeciality.code not in ('+', '.join([s[1] for s in specs])+')'
    stmt='''
select
    Person.lastName, Person.firstName, Person.patrName, Person.code,
    Diagnostic.sanatorium, Diagnostic.endDate, Diagnostic.diagnosisType_id,
    Diagnosis.MKB, Diagnostic.character_id, Diagnostic.stage_id, Diagnostic.healthGroup_id,
    rbSpeciality.code as spec_code, rbSpeciality.OKSOCode, rbSpeciality.name as spec_name
from
    Diagnostic
    join Person on Diagnostic.person_id=Person.id
    join Diagnosis on Diagnosis.id=Diagnostic.diagnosis_id
    join rbSpeciality on rbSpeciality.id=Person.speciality_id
where
    Diagnostic.event_id='''+str(event_id)+sp+'''
    AND Diagnostic.deleted = 0 AND Diagnosis.deleted = 0
order
    by Diagnostic.diagnosisType_id
        '''
    query=QtGui.qApp.db.query(stmt)
    Diags=[]
    while query.next():
        Diags.append(query.record())
    return Diags

def get_dom_korp_kv(freeInput):
    DOM, KOR, KV = None, None, None
    dom_pos=freeInput.rfind(u' д.')
    if dom_pos==-1:
        dom_pos=freeInput.rfind(u',д.')
    if dom_pos==-1:
        dom_pos=freeInput.rfind(u' Д ')
    if dom_pos==-1:
        dom_pos=freeInput.rfind(u' Д.')
    if dom_pos==-1:
        dom_pos=freeInput.rfind(u',Д.')
    if dom_pos>=0:
        f=freeInput[dom_pos+3:]
        d2=f.find(',')
        if d2>=0:
            f=f[:d2]
        DOM=f.strip()
    k_pos=freeInput.find(u' к.')
    if k_pos==-1:
        k_pos=freeInput.find(u',к.')
    if k_pos==-1:
        k_pos=freeInput.find(u',КОРП.')
    if k_pos==-1:
        k_pos=freeInput.find(u',КОР.')
    if k_pos==-1:
        k_pos=freeInput.find(u',К.')
    if k_pos>=0:
        f=freeInput[k_pos+3:]
        k2=f.find(',')
        if k2>=0:
            f=f[:k2]
        KOR=f.strip()
    kv_pos=freeInput.find(u' кв.')
    if kv_pos==-1:
        kv_pos=freeInput.find(u',кв.')
    if kv_pos==-1:
        kv_pos=freeInput.find(u' КВ ')
    if kv_pos==-1:
        kv_pos=freeInput.find(u' КВ.')
    if kv_pos==-1:
        kv_pos=freeInput.find(u',КВ.')
    if kv_pos>=0:
        f=freeInput[kv_pos+4:]
        kv2=f.find(',')
        if kv2>=0:
            f=f[:kv2]
        KV=f.strip()
    return DOM, KOR, KV

def get_dop(dop0):
    dop=[]
    for (dop_name, dop_code, dop_group_id) in dop0:
        r=QtGui.qApp.db.getRecordEx(
            'ActionType', 'id', 'code=\'%s\' and group_id=%d' % (dop_code, dop_group_id))
        if r:
            dop_id=forceInt(r.value(0))
            if dop_id:
                dop.append((dop_name, dop_id))
    return dop

def setEIS_db():
    EIS_db=QtGui.qApp.EIS_db
    if not EIS_db:
        pref=QtGui.qApp.preferences
        props=pref.appPrefs
        EIS_dbDriverName=forceStringEx(props.get('EIS_driverName', QVariant()))
        EIS_dbServerName=forceStringEx(props.get('EIS_serverName', QVariant()))
        EIS_dbServerPort=forceInt(props.get('EIS_serverPort', QVariant()))
        EIS_dbDatabaseName=forceStringEx(props.get('EIS_databaseName', QVariant()))
        EIS_dbUserName=forceStringEx(props.get('EIS_userName', QVariant()))
        EIS_dbPassword=forceStringEx(props.get('EIS_password', QVariant()))
        EIS_db = connectDataBase(
            EIS_dbDriverName, EIS_dbServerName, EIS_dbServerPort,
            EIS_dbDatabaseName, EIS_dbUserName, EIS_dbPassword, 'EIS')
        QtGui.qApp.EIS_db=EIS_db

def EIS_close():
    if QtGui.qApp.EIS_db:
        QtGui.qApp.EIS_db.close()
        QtGui.qApp.EIS_db=None
    QtGui.QMessageBox.information(
        None, u'нет связи', u'не удалось установить соединение с базой ЕИС',
        QtGui.QMessageBox.Ok, QtGui.QMessageBox.Ok)


def binaryToInt(str):
    str = str.encode('cp866')
    return int('0' + hexlify(str), 16)


def insertRecordFromDbf(db, tablename, record, fields = None, mode=0):
    u"""
    Конвертирует запись DBF в запись в любой БД
    db - БД
    tablename - имя таблицы БД, куда вставлять запись
    record - запись DBF
    fields - номера полей, которые надо конвертировать (None - все поля). Если номер равен -1, нужно вставить значение по умолчанию.
    mode: 0 - IGNORE,
      1 - REPLACE
    """
    db_record = db.record(tablename)
    values = record.asList()
    valuesLen = len(values)
    strvalues = []
    strfieldNames = []
    if fields is None:
        fields = xrange(len(values))
    for i in xrange(len(fields)):
        field = db_record.field(i)
        if fields[i] != -1:
            if fields[i] < valuesLen:
                field.setValue(QVariant(values[fields[i]]))
            strvalues.append(unicode(db.db.driver().formatValue(field, False)))
            strfieldNames.append('`%s`'%forceString(field.name()))
    stmt = '%s INTO %s ( %s ) VALUES ( %s )'%(('INSERT IGNORE', 'REPLACE')[mode], tablename, ', '.join(strfieldNames), ', '.join(strvalues))
    querier =  QtSql.QSqlQuery(db.db)
    if not querier.exec_(stmt):
        return querier.lastError()
    return None

def insertTableDataFromDbf(db, tablename, filename, encoding, binaries=[], fields=None, mode=0, errors=0):
    u"""
    Выкачивает данные из DBF и добавляет в таблицу БД
    tablename - имя таблицы БД
    filename - имя файла DBF
    encoding - кодировка, в которой хранится DBF
    binaries - список номеров полей, которые нужно перекодировать из бинарных в целые
    fields - номера полей, которые надо конвертировать (None - все поля). Если номер равен -1, нужно вставить значение по умолчанию.
    mode: 0 - IGNORE,
      1 - REPLACE
    errors: 0 - игнорировать
        1 - завершать работу и возвращать ошибку
    """
    dbftable = Dbf(filename, readOnly=True, encoding=encoding, enableFieldNameDups=True)
#    for i in binaries:
#        dbf.fieldDefs[i].encoding = 'latin-1'
    for record in dbftable:
        for i in binaries:
            record[i] = binaryToInt(record[i])
        error = insertRecordFromDbf(db, tablename, record, fields,  mode=mode)
        if errors == 1 and error:
            return error
        QtGui.qApp.processEvents()
    dbftable.close()
    return None


def logMessage(log, str):
    strings = str.split('\n')
    for s in strings:
        log.append(s)


def logQueryResult(log, query):
    u"""Записывает результат выполнения запроса query в лог по строкам"""
    while query.next():
        rec = query.record()
        str = query.value(0).toString()
        for i in xrange(1,  rec.count()):
            str = str + '\t' + query.value(i).toString()
        log.append(str)

def setSqlVariable(db, name, value):
    u"""Устанавливает значение переменной в запросе на SQL"""
    if value:
        if str(type(value)) == "<type 'string'>" or str(type(value)) == "<type 'unicode'>":
            strvalue = '\'' + value + '\''
        else:
            strvalue = str(value)
    else:
        strvalue = 'NULL'
    querier =  QtSql.QSqlQuery(db.db)
    if not querier.exec_('SELECT @' + name + ' := ' + strvalue):
        return querier.lastError()
    return None


def execQuery(db, s, log):
    if QRegExp('[ \n\r\t]*').exactMatch(s): # пустой запрос
        return None
    #print "QUERY: " + s
    querier =  QtSql.QSqlQuery(db.db)
    if not querier.exec_(s):
        return querier.lastError()
    if log:
        logQueryResult(log, querier) # записываем результат выполнения
    return None

def runScript(db, log, instream, dict = {}):
    u"""Выполняет последовательность запросов к базе данных db, считанную из входного потока instream.
    instream - это произвольный контейнер со строками
    Записывает результаты в окно log
    dict - набор пар (имя, значение) для установки переменных в SQL
    """
    # устанавливаем значения переменных:
    for (key,  value) in dict.items():
        setSqlVariable(db, key,  value)

    # анализируем запросы:
    fullstr = '' # текущий запрос
    delimiter = ';' # разделитель запросов
    for str in instream:
        delimexp = QRegExp('delimiter', Qt.CaseInsensitive)
        if delimexp.indexIn(str.strip()) == 0: # в этой строке меняется разделитель
            delimiter = str[delimexp.matchedLength():].strip()
        else:
            pos = QRegExp('--|\'|\"|%s'%QRegExp.escape(delimiter)).indexIn(str, 0) # позиция, с которой начинается комментарий или открывается строка
            while pos != -1:
                if str[pos] == '\'':
                    pos = str.find('\'', pos+1)
                    pos += (1 if pos != -1 else 0)
                elif str[pos] == '\"':
                    pos = str.find('\"', pos+1)
                    pos += (1 if pos != -1 else 0)
                elif str[pos:pos+len(delimiter)] == delimiter: # здесь запрос закончился
                    fullstr = fullstr + ' ' + str[:pos]
                    result = execQuery(db, fullstr, log) # выполняем его
                    if result:
                        return result
                    fullstr = '' # и ждем нового
                    str = str[pos+len(delimiter):]
                    pos = 0
                else: # найден комментарий
                    str = str[:pos]
                    break
                pos = QRegExp('--|\'|\"|%s'%QRegExp.escape(delimiter)).indexIn(str, pos)
            fullstr = fullstr + ' ' + str
    result = execQuery(db, fullstr, log) # выполняем всё, что осталось в конце
    return result

def checkEmail(email):
    reg = '^.+\\@(\\[?)[a-zA-Z0-9\\-\\.]+\\.([a-zA-Z]{2,3}|[0-9]{1,3})(\\]?)$'
    if len(email) > 7:
        if re.match(reg, email):
            return True
    return False

def compressFileInRar(filePath='', rarFilePath=''):
        if not filePath:
            raise CException(u'При сжатии получен пустой адрес сохранения файла')
        if not rarFilePath:
            rarFilePath = filePath+'.rar'
        prg=QtGui.qApp.execProgram(u'rar mf -ep -m5 -o+ -y -- "%s" "%s"'% (rarFilePath, filePath))
        if not prg[0]:
            raise CException(u'не удалось запустить rar')
        if prg[2]:
            raise CException(u'ошибка при выполнении rar')
        return True


def compressFileInArj(filePath, arjFilePath=''):
    if filePath:
        if not arjFilePath:
            arjFilePath = filePath+'.arj'
        prg=QtGui.qApp.execProgram(u'arj m -m4 -y -e "%s" "%s"'% (arjFilePath, filePath))
        if not prg[0]:
            raise CException(u'не удалось запустить arj')
        if prg[2]:
            raise CException(u'ошибка при выполнении arj')
        return True

    return False


def compressFileInZip(filePath, zipFilePath=''):
    if not zipFilePath:
        zipFilePath = filePath+'.zip'

    try:
        zf = ZipFile(zipFilePath, 'w', allowZip64=True)
        if (isinstance(filePath, list) or isinstance(filePath, tuple) or
                    isinstance(filePath, set)):
            for file in filePath:
                zf.write(file, os.path.basename(file), ZIP_DEFLATED)
        else:
            zf.write(filePath, os.path.basename(filePath), ZIP_DEFLATED)
        zf.close()
    except:
        return False

    return True


def compressFileIn7Zip(filePath, zipFilePath=''):
    if filePath:
        if not zipFilePath:
            zipFilePath = filePath+'.7z'
        prg=QtGui.qApp.execProgram(u'7za a -y "%s" "%s"'% (zipFilePath, filePath))
        if not prg[0]:
            raise CException(u'не удалось запустить 7za')
        if prg[2]:
            raise CException(u'ошибка при выполнении 7za')
        return True

    return False


def getClientRepresentativeInfo(clientId):
    db = QtGui.qApp.db
    stmt = """
SELECT
    Client.firstName,
    Client.lastName,
    Client.patrName,
    Client.sex,
    Client.birthDate,
    Client.birthPlace,
    ClientDocument.number,
    ClientDocument.serial,
    Client.SNILS,
    rbDocumentType.code AS documentTypeCode,
    rbDocumentType.regionalCode AS documentTypeRegionalCode,
    rbDocumentType.federalCode AS documentTypeFederalCode,
    T.regionalRelationTypeCode AS regionalRelationTypeCode
FROM
    (SELECT rbRelationType.code, ClientRelation.client_id, rbRelationType.regionalReverseCode AS regionalRelationTypeCode
    FROM ClientRelation
    LEFT JOIN rbRelationType ON ClientRelation.relativeType_id = rbRelationType.id
    WHERE ClientRelation.deleted=0
      AND ClientRelation.relative_id = %d
      AND rbRelationType.isDirectRepresentative
    UNION
    SELECT rbRelationType.code, ClientRelation.relative_id AS client_id, rbRelationType.regionalCode AS regionalRelationTypeCode
    FROM ClientRelation
    LEFT JOIN rbRelationType ON ClientRelation.relativeType_id=rbRelationType.id
    WHERE ClientRelation.deleted=0
      AND ClientRelation.client_id = %d
      AND rbRelationType.isBackwardRepresentative
    ) AS T
    LEFT JOIN Client ON T.client_id = Client.id
    LEFT JOIN ClientDocument ON ClientDocument.client_id = Client.id AND
        ClientDocument.id = (SELECT MAX(CD.id)
                        FROM   ClientDocument AS CD
                        LEFT JOIN rbDocumentType AS rbDT ON rbDT.id = CD.documentType_id
                        LEFT JOIN rbDocumentTypeGroup AS rbDTG ON rbDTG.id = rbDT.group_id
                        WHERE  rbDTG.code = '1' AND CD.client_id = Client.id AND CD.deleted=0)
    LEFT JOIN rbDocumentType ON rbDocumentType.id = ClientDocument.documentType_id
WHERE
    Client.deleted = 0
ORDER BY T.code
LIMIT 1
    """ % (clientId, clientId)
    query = db.query(stmt)
    result = {}

    if query.first():
        record=query.record()
        if record:
            result['firstName'] = forceString(record.value('firstName'))
            result['lastName'] = forceString(record.value('lastName'))
            result['patrName'] = forceString(record.value('patrName'))
            result['serial'] = forceString(record.value('serial'))
            result['number'] = forceString(record.value('number'))
            result['sex'] = forceInt(record.value('sex'))
            result['birthDate'] = forceDate(record.value('birthDate'))
            result['birthPlace'] = forceString(record.value('birthPlace'))
            result['documentTypeCode'] = forceString(record.value('documentTypeCode'))
            result['documentTypeRegionalCode'] = forceInt(record.value('documentTypeRegionalCode'))
            result['documentTypeFederalCode'] = forceString(record.value('documentTypeFederalCode'))
            result['regionalRelationTypeCode'] = forceString(record.value('regionalRelationTypeCode'))
            result['SNILS'] = forceString(record.value('SNILS'))
    return result


def testAccountExport(exportFunc, accountId):
    u'Запускает приложение в демо режиме, вызывает exportFunc для accountId'

    import sys
    from PyQt4 import QtGui
    from s11main import CS11mainApp, CS11MainWindow

    result = False

    try:
        app = CS11mainApp(sys.argv, True, 'S11App.ini', True, 'mark')
        QtGui.qApp = app
        app.applyDecorPreferences()
        mainWindow = CS11MainWindow()
        app.mainWindow = mainWindow
        mainWindow.show()
        mainWindow.actLogin.activate(QtGui.QAction.Trigger)

        if app.db:
            mainWindow.statusBar.showMessage(app.db.connectionId)
            mainWindow.menuBar.setVisible(False)
            tbl = app.db.table('Account_Item')
            accountItemIdList = app.db.getIdList(
                tbl, where=tbl['master_id'].eq(accountId))
            exportFunc(mainWindow, accountId, accountItemIdList, None)
            app.clearUserId(False)
            app.closeDatabase()
            result = True

        app.doneTrace()

    except:
        pass

    QtGui.qApp = None
    print('Test {0}.'.format('ok' if result else 'failed'))
    return result
