# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
##
## Форма 106: свидетельство о смерти
##
#############################################################################

import sip
from PyQt4 import QtGui, QtSql
from PyQt4.QtCore import Qt, QDate, QDateTime, QModelIndex, QObject, QTime, QVariant, pyqtSignature, SIGNAL

from library.Attach.AttachAction import getAttachAction
from library.crbcombobox         import CRBComboBox
from library.ICDInDocTableCol    import CICDExInDocTableCol
from library.ICDMorphologyInDocTableCol import CMKBMorphologyCol
from library.InDocTable          import CMKBListInDocTableModel, CBoolInDocTableCol, CInDocTableCol, CRBInDocTableCol

from library.interchange         import getDatetimeEditValue, getLineEditValue, setComboBoxValue, getRBComboBoxValue, getTextEditValue, setDatetimeEditValue, setLineEditValue, setRBComboBoxValue, setTextEditValue
from library.TNMS.TNMSComboBox   import CTNMSCol
from library.Utils               import copyFields, forceBool, forceDate, forceInt, forceRef, forceString, toVariant, variantEq
from library.PrintTemplates      import customizePrintButton, getPrintButton, applyTemplate
from library.PrintInfo           import CInfoContext

from Events.ActionInfo           import CActionInfoProxyList
from Events.EventEditDialog      import CEventEditDialog, CDiseaseCharacter, CDiseaseStage, CDiseasePhases
from Events.EventInfo            import CDiagnosticInfoProxyList
from Events.Utils                import checkDiagnosis, checkIsHandleDiagnosisIsChecked, getAvailableCharacterIdByMKB, getDiagnosisId2, getEventShowTime, recordAcceptable, setAskedClassValueForDiagnosisManualSwitch
from F106.F106DiagnosisSelectionDialog import CDiagnosisSelectionDialog
from F106.PreF106Dialog          import CPreF106Dialog, CPreF106DagnosticAndActionPresets
from Registry.Utils              import fixClientDeath
from Users.Rights                import urAccessF025planner, urAdmin, urEditEndDateEvent, urRegTabWriteRegistry, urRegTabReadRegistry

from F106.Ui_F106                import Ui_Dialog


class CF106Dialog(CEventEditDialog, Ui_Dialog):

    dfAccomp = 2 # Сопутствующий

    def __init__(self, parent):
# ctor
        CEventEditDialog.__init__(self, parent)
        self.mapSpecialityIdToDiagFilter = {}

# create models
        self.addModels('PreliminaryDiagnostics', CF106DiagnosticsModel(self, '8', '10'))
        self.addModels('FinalDiagnostics',       CF106DiagnosticsModel(self, '4', '9'))

# ui
        self.addObject('actEditClient', QtGui.QAction(u'Открыть регистрационную карточку', self))
        self.addObject('actShowAttachedToClientFiles', getAttachAction('Client_FileAttach',  self))
        self.createSaveAndCreateAccountButton()
        self.setupUi(self)
        self.setupSaveAndCreateAccountButton()
        self.addObject('btnPrint', getPrintButton(self, ''))
        self.addObject('btnPrintMedicalDiagnosis', getPrintButton(self, '', u'Врачебный диагноз'))

        self.setWindowFlags(self.windowFlags() | Qt.WindowMaximizeButtonHint)
        self.setWindowTitleEx(u'Ф.106/у')

        self.setMedicalDiagnosisContext()
        self.tabToken.setFocusProxy(self.tblPreliminaryDiagnostics)

        self.tabStatus.setEventEditor(self)
        self.tabMedicalDiagnosis.setEventEditor(self)
        self.tabMisc.setEventEditor(self)
        self.tabCash.setEventEditor(self)
        self.tabStatus.setActionTypeClass(0)
        self.tabMisc.setActionTypeClass(3)
        self.buttonBox.addButton(self.btnPrint, QtGui.QDialogButtonBox.ActionRole)
        self.connect(self.btnPrint, SIGNAL('printByTemplate(int)'), self.on_btnPrint_printByTemplate)

# tables to rb and combo-boxes
        # мы полагаем, что cmbPerson - свой врач,
        # мы полагаем, что cmbPerson2 - любой паталогоанатом (код специальности - 51)
        self.cmbPerson2.setOrgId(None)
        self.cmbPerson2.setSpecialityId(forceRef(QtGui.qApp.db.translate('rbSpeciality', 'code', '51', 'id'))) # magic!
        # мы полагаем, что cmbStatusPerson - свой врач,
        # мы полагаем, что cmbMiscPerson - свой врач,

# assign models
        self.tblPreliminaryDiagnostics.setModel(self.modelPreliminaryDiagnostics)
        self.tblFinalDiagnostics.setModel(self.modelFinalDiagnostics)
        self.tabCash.addActionModel(self.tabStatus.modelAPActions)
        self.tabCash.addActionModel(self.tabMisc.modelAPActions)

# popup menus
        self.txtClientInfoBrowser.actions.append(self.actEditClient)
        self.txtClientInfoBrowser.actions.append(self.actShowAttachedToClientFiles)
        self.actEditClient.setEnabled(QtGui.qApp.userHasAnyRight([urAdmin, urRegTabWriteRegistry, urRegTabReadRegistry]))
        self.tblPreliminaryDiagnostics.addCopyDiagnosisToFinal(self)

# default values
        self.buttonBox.addButton(self.btnPrintMedicalDiagnosis, QtGui.QDialogButtonBox.ActionRole)
        self.setupDirtyCather()
        self.setIsDirty(False)
        self.blankMovingIdList = []
        self.tabNotes.setEventEditor(self)
        self.setupVisitsIsExposedPopupMenu()

        self.postSetupUi()
        if hasattr(self, 'edtEndDate'):
            self.edtEndDate.setEnabled(QtGui.qApp.userHasRight(urEditEndDateEvent))
        if hasattr(self, 'edtEndTime'):
            self.edtEndTime.setEnabled(QtGui.qApp.userHasRight(urEditEndDateEvent))
        self.btnPrintMedicalDiagnosis.setVisible(False)
# done


    def destroy(self):
        self.tblPreliminaryDiagnostics.setModel(None)
        self.tblFinalDiagnostics.setModel(None)
        del self.modelPreliminaryDiagnostics
        del self.modelFinalDiagnostics

        self.tabStatus.destroy()
        self.tabMisc.destroy()
        self.tabCash.destroy()


    def setExternalId(self, externalId):
        pass


    def getServiceActionCode(self):
        return None


    def setMesInfo(self, mesCode):
        pass


    def _prepare(self, clientId, eventTypeId, orgId, personId, eventSetDatetime, eventDatetime, weekProfile, numDays, presetDiagnostics,
                 presetActions, disabledActions, externalId, assistantId, curatorId, actionTypeIdValue = None, valueProperties = [],
                 relegateOrgId = None, relegatePersonId=None, diagnos = None, financeId = None, protocolQuoteId = None, actionByNewEvent = [],
                 order = 1, relegateInfo=[], plannedEndDate = None):
        self.eventSetDateTime = eventSetDatetime if isinstance(eventSetDatetime, QDateTime) else QDateTime(eventSetDatetime)
        self.eventDate = eventDatetime.date() if isinstance(eventDatetime, QDateTime) else eventDatetime
        self.setOrgId(orgId if orgId else QtGui.qApp.currentOrgId())
        self.cmbOrg.setValue(self.orgId)
        self.setEventTypeId(eventTypeId)
        self.setClientId(clientId)
        self.tabNotes.setNotesEx(externalId, assistantId, curatorId, relegateOrgId, relegatePersonId, clientId, relegateInfo)
        self.setPersonId(personId)
        self.cmbPerson.setValue(personId)
        self.cmbPerson2.setValue(None)
        self.edtBegDate.setDate(self.eventDate)
        self.edtEndDate.setDate(self.eventSetDateTime.date())
        self.edtBegTime.setTime(eventDatetime.time() if isinstance(eventDatetime, QDateTime) else QTime())
        self.edtEndTime.setTime(eventSetDatetime.time() if isinstance(eventSetDatetime, QDateTime) else QTime())
        self.cmbContract.setCurrentIndex(0)
        self.initFocus()

        self.prepareActions(presetActions, disabledActions, actionTypeIdValue, valueProperties, diagnos, financeId, protocolQuoteId, actionByNewEvent, plannedEndDate)
        self.setIsDirty(False)
        self.tabNotes.setEventEditor(self)
        self.setFilterResult(self.eventSetDateTime.date())
        return True and self.checkDeposit()


    def prepare(self, clientId, eventTypeId, orgId, personId, eventSetDatetime, eventDatetime, weekProfile,
                numDays, externalId, assistantId, curatorId, flagHospitalization = False,
                actionTypeIdValue = None, valueProperties = [], tissueTypeId=None, selectPreviousActions=False,
                relegateOrgId = None, relegatePersonId=None, diagnos = None, financeId = None,
                protocolQuoteId = None, actionByNewEvent = [], order = 1,
                actionListToNewEvent = [], typeQueue = -1, docNum=None, relegateInfo=[], plannedEndDate = None,
                mapJournalInfoTransfer = []):
        self.setPersonId(personId)
        eventDate = eventDatetime.date() if isinstance(eventDatetime, QDateTime) else eventDatetime
        if not eventDate and eventSetDatetime:
            eventDate = eventSetDatetime.date() if isinstance(eventSetDatetime, QDateTime) else eventSetDatetime
        else:
            eventDate = QDate.currentDate()
        result = True
        if QtGui.qApp.userHasRight(urAccessF025planner):
            dlg = CPreF106Dialog(self, self.contractTariffCache)
            try:
                dlg.setBegDateEvent(eventSetDatetime.date() if isinstance(eventSetDatetime, QDateTime) else eventSetDatetime)
                dlg.prepare(clientId, eventTypeId, eventDate, self.personId, self.personSpecialityId, self.personTariffCategoryId, flagHospitalization, actionTypeIdValue, tissueTypeId)
                if dlg.diagnosticsTableIsNotEmpty() or dlg.actionsTableIsNotEmpty():
                    if not dlg.exec_():
                        return False
                result = self._prepare(clientId, eventTypeId, orgId, personId, eventSetDatetime, eventDatetime, weekProfile, numDays,
                                     dlg.diagnostics(), dlg.actions(), dlg.disabledActionTypeIdList,
                                     externalId, assistantId, curatorId, actionTypeIdValue, valueProperties,
                                     relegateOrgId, relegatePersonId, diagnos, financeId, protocolQuoteId,
                                     actionByNewEvent, order, relegateInfo, plannedEndDate)
            finally:
                dlg.destroy()
                sip.delete(dlg)
                del dlg
        else:
            presets = CPreF106DagnosticAndActionPresets(clientId, eventTypeId, eventDate, self.personSpecialityId, flagHospitalization, actionTypeIdValue)
            presets.setBegDateEvent(eventSetDatetime.date() if isinstance(eventSetDatetime, QDateTime) else eventSetDatetime)
            result = self._prepare(clientId, eventTypeId, orgId, personId, eventSetDatetime, eventDatetime, weekProfile, numDays,
                                 presets.unconditionalDiagnosticList, presets.unconditionalActionList, presets.disabledActionTypeIdList,
                                 externalId, assistantId, curatorId, None, [], relegateOrgId, relegatePersonId, diagnos,
                                 financeId, protocolQuoteId, actionByNewEvent, order, relegateInfo, plannedEndDate)

        if result:
            diagSelectDlg = CDiagnosisSelectionDialog(self)
            if diagSelectDlg.findDiagnosis(clientId):
                if diagSelectDlg.exec_():
                    mkbrecord = diagSelectDlg.getDiagnosis()
                    record = self.modelFinalDiagnostics.getEmptyRecord()
                    record.setValue('MKB', mkbrecord.value('MKB'))
                    record.setValue('MKBEx', mkbrecord.value('MKBEx'))
                    record.setValue('traumaType_id', mkbrecord.value('traumaType_id'))
                    record.setValue('character_id', mkbrecord.value('character_id'))
                    record.setValue('phase_id', mkbrecord.value('phase_id'))
                    record.setValue('stage_id', mkbrecord.value('stage_id'))
                    self.modelFinalDiagnostics.addRecord(record)
                    record = self.modelPreliminaryDiagnostics.getEmptyRecord()
                    record.setValue('MKB', mkbrecord.value('MKB'))
                    record.setValue('MKBEx', mkbrecord.value('MKBEx'))
                    record.setValue('traumaType_id', mkbrecord.value('traumaType_id'))
                    record.setValue('character_id', mkbrecord.value('character_id'))
                    record.setValue('phase_id', mkbrecord.value('phase_id'))
                    record.setValue('stage_id', mkbrecord.value('stage_id'))
                    self.modelPreliminaryDiagnostics.addRecord(record)
                    db = QtGui.qApp.db
                    tableResult = db.table('rbResult')
                    cond = [tableResult['eventPurpose_id'].eq(self.eventPurposeId),
                            tableResult['code'].eq('0')]
                    resultRecord = db.getRecordEx(tableResult, 'id', cond)
                    if resultRecord:
                        resultId = forceInt(resultRecord.value(0))
                        self.cmbResult.setValue(resultId)
        return result

#    def prepareActions(self):
#        def recordAcceptable(specialityId, sex, age):
#            if specialityId:
#                if specialityId != self.personSpecialityId:
#                    return False
#            if sex:
#                if sex != self.clientSex:
#                    return False
#            if age:
#                if self.clientAge is None:
#                    return False
#                ageSelector = parseAgeSelector(age)
#                return checkAgeSelector(ageSelector, self.clientAge)
#            return True
#
#        db = QtGui.qApp.db
#        table = db.table('EventType_Action')
#        cond = [table['eventType_id'].eq(self.eventTypeId)]
#        for record in db.getRecordList(table, 'actionType_id, speciality_id, sex, age, selectionGroup', cond):
#            actionTypeId = forceRef(record.value('actionType_id'))
#            specialityId = forceRef(record.value('speciality_id'))
#            sex          = forceInt(record.value('sex'))
#            age          = forceString(record.value('age'))
#            selectionGroup = forceString(record.value('selectionGroup'))
#            if recordAcceptable(specialityId, sex, age) and selectionGroup>0:
#                for model in (self.tabStatus.modelAPActions,
#                              self.tabMisc.modelAPActions):
#                    if actionTypeId in model.actionTypeIdList:
#                        model.addRow(actionTypeId)
#                        break


    def prepareActions(self, presetActions, disabledActions, actionTypeIdValue, valueProperties, diagnos, financeId, protocolQuoteId, actionByNewEvent, plannedEndDate=None):
        def addActionType(actionTypeId, amount, idListActionType, idListActionTypeIPH, actionFinance, idListActionTypeMoving, plannedEndDate):
            db = QtGui.qApp.db
            tableOrgStructure = db.table('OrgStructure')
            for model in (self.tabStatus.modelAPActions,
                          self.tabMisc.modelAPActions):
                if actionTypeId in model.actionTypeIdList:
                    if actionTypeId in idListActionType and not actionByNewEvent:
                        model.addRow(actionTypeId, amount)
                        record, action = model.items()[-1]
                        if plannedEndDate:
                            record.setValue('directionDate', QVariant(plannedEndDate))
                        if u'Приемное отделение' in action._actionType._propertiesByName:
                            curOrgStructureId = QtGui.qApp.currentOrgStructureId()
                            if curOrgStructureId:
                                recOS = db.getRecordEx(tableOrgStructure,
                                               [tableOrgStructure['id']],
                                               [tableOrgStructure['deleted'].eq(0),
                                                tableOrgStructure['id'].eq(curOrgStructureId),
                                                tableOrgStructure['type'].eq(4)])
                                if recOS:
                                    curOSId = forceRef(recOS.value('id'))
                                    action[u'Приемное отделение'] = curOSId if curOSId else None
                        if valueProperties and len(valueProperties) > 0 and valueProperties[0]:
                            action[u'Направлен в отделение'] = valueProperties[0]
                        if protocolQuoteId:
                            action[u'Квота'] = protocolQuoteId
                        if actionFinance == 0:
                            record.setValue('finance_id', toVariant(financeId))
                    elif actionTypeId in idListActionTypeIPH:
                        model.addRow(actionTypeId, amount)
                        record, action = model.items()[-1]
                        if diagnos:
                            record, action = model.items()[-1]
                            action[u'Диагноз'] = diagnos
                    elif actionByNewEvent and actionTypeId in idListActionTypeMoving:
                        model.addRow(actionTypeId, amount)
                        record, action = model.items()[-1]
                        if actionByNewEvent[0] == 0:
                            record.setValue('finance_id', toVariant(actionByNewEvent[1]))
                        action[u'Отделение пребывания'] = actionByNewEvent[2]
                        if actionByNewEvent[3]:
                            action[u'Переведен из отделения'] = actionByNewEvent[3]
                        if actionByNewEvent[4]:
                            record.setValue('begDate', toVariant(actionByNewEvent[4]))
                        else:
                            record.setValue('begDate', toVariant(QDateTime.currentDateTime()))
                        if u'Квота' in action._actionType._propertiesByName and actionByNewEvent[5]:
                            action[u'Квота'] = actionByNewEvent[5]
                        if actionByNewEvent[6]:
                            record.setValue('person_id', toVariant(actionByNewEvent[6]))
                    elif (actionByNewEvent and actionTypeId not in idListActionType) or not actionByNewEvent:
                        model.addRow(actionTypeId, amount)
                        record, action = model.items()[-1]

        def disableActionType(actionTypeId):
            for model in (self.tabStatus.modelAPActions,
                          self.tabMisc.modelAPActions):
                if actionTypeId in model.actionTypeIdList:
                    model.disableActionType(actionTypeId)
                    break
        if disabledActions:
            for actionTypeId in disabledActions:
                disableActionType(actionTypeId)
        if presetActions:
            db = QtGui.qApp.db
            tableActionType = db.table('ActionType')
            tableEventType = db.table('EventType')
            idListActionType = db.getIdList(tableActionType, [tableActionType['id']], [tableActionType['flatCode'].like(u'received%'), tableActionType['deleted'].eq(0)])
            idListActionTypeIPH = db.getIdList(tableActionType, [tableActionType['id']], [tableActionType['flatCode'].like(u'inspectPigeonHole%'), tableActionType['deleted'].eq(0)])
            idListActionTypeMoving = db.getIdList(tableActionType, [tableActionType['id']], [tableActionType['flatCode'].like(u'moving%'), tableActionType['deleted'].eq(0)])
            eventTypeId = self.getEventTypeId()
            actionFinance = None
            if eventTypeId:
                recordET = db.getRecordEx(tableEventType, [tableEventType['actionFinance']], [tableEventType['deleted'].eq(0), tableEventType['id'].eq(eventTypeId)])
                actionFinance = forceInt(recordET.value('actionFinance')) if recordET else None
            if actionByNewEvent:
                actionTypeMoving = False
                for actionTypeId, amount, cash in presetActions:
                    if actionTypeId in idListActionTypeMoving:
                        actionTypeMoving = True
                        break
                if not actionTypeMoving and idListActionTypeMoving:
                    presetActions.append((idListActionTypeMoving[0], 1.0, False))
            for actionTypeId, amount, cash in presetActions:
                addActionType(actionTypeId, amount, idListActionType, idListActionTypeIPH, actionFinance, idListActionTypeMoving, plannedEndDate)


    def setLeavedAction(self, actionTypeIdValue, params = {}):
        pass


    def initFocus(self):
        self.chkAutopsy.setFocus(Qt.OtherFocusReason)
#        self.tblPreliminaryDiagnostics.setFocus(Qt.OtherFocusReason)


    def newDiagnosticRecord(self, template):
        result = self.tblPreliminaryDiagnostics.model().getEmptyRecord()
        return result


    def setRecord(self, record):
        CEventEditDialog.setRecord(self, record)
        setLineEditValue(self.edtNumber,        record, 'externalId')
        setDatetimeEditValue(self.edtEndDate, self.edtEndTime, record, 'setDate')
        setRBComboBoxValue(self.cmbPerson,      record, 'execPerson_id')
        setDatetimeEditValue(self.edtBegDate, self.edtBegTime, record, 'execDate')
        setRBComboBoxValue(self.cmbPerson2,     record, 'setPerson_id')
        self.chkAutopsy.setChecked(forceInt(record.value('isPrimary'))==1)
        setComboBoxValue(self.cmbAutopsyType,   record, 'isAutopsyType')
        self.setPersonId(self.cmbPerson.value())
        setRBComboBoxValue(self.cmbResult,      record, 'result_id')
        setRBComboBoxValue(self.cmbContract,    record, 'contract_id')
        setTextEditValue(self.edtNote,          record, 'note')
        self.tabNotes.setNotes(record)
        self.tabNotes.setEventEditor(self)
        self.loadDiagnostics(self.modelPreliminaryDiagnostics)
        self.loadDiagnostics(self.modelFinalDiagnostics)
        self.tabMedicalDiagnosis.load(self.itemId())
        self.loadActions()
        self.tabCash.load(self.itemId())
        self.initFocus()
        self.setIsDirty(False)
        self.blankMovingIdList = []
        if self.tabNotes.isEventClosed() or self.isHBDialog:
            self.protectClosedEvent()


    def loadDiagnostics(self, modelDiagnostics):
        db = QtGui.qApp.db
        table = db.table('Diagnostic')
#        tablePerson = db.table('Person')
        isDiagnosisManualSwitch = modelDiagnostics.manualSwitchDiagnosis()
        rawItems = db.getRecordList(table, '*', [table['deleted'].eq(0), table['event_id'].eq(self.itemId()), modelDiagnostics.filter], 'id')
        items = []
        for record in rawItems:
#            specialityId = forceRef(record.value('speciality_id'))
            diagnosisId     = record.value('diagnosis_id')
            MKB             = db.translate('Diagnosis', 'id', forceRef(diagnosisId), 'MKB')
            MKBEx           = db.translate('Diagnosis', 'id', forceRef(diagnosisId), 'MKBEx')
            morphologyMKB   = db.translate('Diagnosis', 'id', forceRef(diagnosisId), 'morphologyMKB')
            setDate         = forceDate(record.value('setDate'))
            newRecord =  modelDiagnostics.getEmptyRecord()
            copyFields(newRecord, record)
            newRecord.setValue('MKB',           MKB)
            newRecord.setValue('MKBEx',         MKBEx)
            newRecord.setValue('morphologyMKB', morphologyMKB)

            if isDiagnosisManualSwitch:
                isCheckedHandleDiagnosis = checkIsHandleDiagnosisIsChecked(setDate,
                                                                           self.clientId,
                                                                           diagnosisId)
                newRecord.setValue('handleDiagnosis', QVariant(isCheckedHandleDiagnosis))

            items.append(newRecord)
        modelDiagnostics.setItems(items)


    def loadActions(self):
        eventId = self.itemId()
        self.tabStatus.loadActions(eventId)
        self.tabMisc.loadActions(eventId)
        self.tabCash.modelAccActions.regenerate()


    def getRecord(self):
        record = CEventEditDialog.getRecord(self)
        showTime = getEventShowTime(self.eventTypeId)
        self.getShowButtonAccount()
#перенести в exec_ в случае успеха или в accept?
#        CF106Dialog.defaultEventResultId = self.modelDiagnostics.resultId()

        getRBComboBoxValue(self.cmbContract,    record, 'contract_id')
#        getDateEditValue(self.edtPrevDate,      record, 'prevEventDate')
        getLineEditValue(self.edtNumber,        record, 'externalId')
        getDatetimeEditValue(self.edtEndDate, self.edtEndTime, record, 'setDate', showTime)
        getRBComboBoxValue(self.cmbPerson2,     record, 'setPerson_id')
        getDatetimeEditValue(self.edtBegDate, self.edtBegTime, record, 'execDate', showTime)
        getRBComboBoxValue(self.cmbPerson,      record, 'execPerson_id')
        getRBComboBoxValue(self.cmbResult,      record, 'result_id')
        record.setValue('order', toVariant(1))
        record.setValue('isPrimary', toVariant(1 if self.chkAutopsy.isChecked() else 2))
        record.setValue('isAutopsyType', toVariant(self.cmbAutopsyType.currentIndex() if self.chkAutopsy.isChecked() else None))
        getTextEditValue(self.edtNote,          record, 'note')
###  payStatus
        self.tabNotes.getNotes(record, self.eventTypeId)
        return record


    def saveInternals(self, eventId):
        self.saveDiagnostics(self.modelPreliminaryDiagnostics, eventId)
        self.saveDiagnostics(self.modelFinalDiagnostics, eventId)
        self.tabMedicalDiagnosis.save(eventId)
        setAskedClassValueForDiagnosisManualSwitch(None)
        self.saveActions(eventId)
        self.tabCash.save(eventId)
        fixClientDeath(self.clientId, self.edtBegDate.date(), self.edtEndDate.date(), self.orgId)
        self.saveBlankUsers(self.blankMovingIdList)
        self.tabNotes.saveAttachedFiles(eventId)


    def saveDiagnostics(self, modelDiagnostics, eventId):
        items = modelDiagnostics.items()
        isDiagnosisManualSwitch = modelDiagnostics.manualSwitchDiagnosis()
        isFirst = True
        date = self.edtBegDate.date()
        dateVariant = toVariant(date)
        personIdVariant = toVariant(self.personId)
        specialityIdVariant = QtGui.qApp.db.translate('Person', 'id', personIdVariant, 'speciality_id')
        MKBDiagnosisIdPairList = []
        prevId=0
        for item in items:
            MKB   = forceString(item.value('MKB'))
            MKBEx = forceString(item.value('MKBEx'))
            TNMS  = forceString(item.value('TNMS'))
            morphologyMKB = forceString(item.value('morphologyMKB'))
            diagnosisTypeId = modelDiagnostics.getDiagnosisTypeId(isFirst)
            item.setValue('diagnosisType_id', toVariant(diagnosisTypeId))
            item.setValue('speciality_id', specialityIdVariant)
            item.setValue('person_id', toVariant(self.personId) )
            item.setValue('setDate', dateVariant )
            item.setValue('endDate', dateVariant )
            diagnosisId = forceRef(item.value('diagnosis_id'))
            characterId = forceRef(item.value('character_id'))
            diagnosisId, characterId = getDiagnosisId2(
                date,
                self.personId,
                self.clientId,
                diagnosisTypeId,
                MKB,
                MKBEx,
                forceRef(item.value('character_id')),
                forceRef(item.value('dispanser_id')),
                forceRef(item.value('traumaType_id')),
                diagnosisId,
                forceRef(item.value('id')),
                isDiagnosisManualSwitch,
                forceBool(item.value('handleDiagnosis')),
                TNMS=TNMS,
                morphologyMKB=morphologyMKB)
            item.setValue('diagnosis_id', toVariant(diagnosisId))
            item.setValue('TNMS', toVariant(TNMS))
            item.setValue('character_id', toVariant(characterId))
            itemId = forceInt(item.value('id'))
            if prevId>itemId:
                item.setValue('id', QVariant())
                prevId=0
            else:
               prevId=itemId
            isFirst = False
            MKBDiagnosisIdPairList.append((MKB, diagnosisId))
        modelDiagnostics.saveItems(eventId)
        self.modifyDiagnosises(MKBDiagnosisIdPairList)


    def saveActions(self, eventId):
        self.tabStatus.saveActions(eventId)
        self.tabMisc.saveActions(eventId)


    def setOrgId(self, orgId):
        self.orgId = orgId
        self.cmbContract.setOrgId(orgId)
        self.tabStatus.setOrgId(orgId)
        self.tabMisc.setOrgId(orgId)
        self.cmbOrg.setValue(self.orgId)


    def setEventTypeId(self, eventTypeId):
        CEventEditDialog.setEventTypeId(self, eventTypeId, u'Ф.106')
        self.tabCash.windowTitle = self.windowTitle()
        showTime = getEventShowTime(eventTypeId)
        self.getShowButtonAccount()
        self.edtBegTime.setVisible(showTime)
        self.edtEndTime.setVisible(showTime)
        self.cmbResult.setTable('rbResult', True, 'eventPurpose_id=\'%d\'' % self.eventPurposeId)
        self.cmbContract.setEventTypeId(eventTypeId)
        customizePrintButton(self.btnPrint, self.eventContext if self.eventContext else 'F106')


    def resetActionTemplateCache(self):
        self.tabStatus.actionTemplateCache.reset()
        self.tabMisc.actionTemplateCache.reset()


    def checkDataEntered(self):
        result = CEventEditDialog.checkDataEntered(self)
        self.blankMovingIdList = []
#        result = result and (self.orgId != QtGui.qApp.currentOrgId() or self.cmbContract.value() or self.checkInputMessage(u'договор', False, self.cmbContract))
        showTime = getEventShowTime(self.eventTypeId)
        if showTime:
            begDate = QDateTime(self.edtBegDate.date(), self.edtBegTime.time())
            endDate = QDateTime(self.edtEndDate.date(), self.edtEndTime.time())
        else:
            begDate = self.edtBegDate.date()
            endDate = self.edtEndDate.date()
        result = result and (begDate or self.checkInputMessage(u'дату смерти', False, self.edtBegDate))
        result = result and (endDate or self.checkInputMessage(u'дату констатации', False, self.edtEndDate))
        result = result and (self.cmbResult.value()   or self.checkInputMessage(u'результат',    False, self.cmbResult))
        result = result and (self.cmbPerson.value()   or self.checkInputMessage(u'врач',         False, self.cmbPerson))
#        if not endDate:
#            maxEndDate = self.getMaxEndDateByVisits()
#            if maxEndDate:
#                if QtGui.QMessageBox.question(self,
#                                    u'Внимание!',
#                                    u'Дата выполнения обращения не указана.\nУстановить дату завершения по максимальной дате посещений',
#                                    QtGui.QMessageBox.No|QtGui.QMessageBox.Yes,
#                                    QtGui.QMessageBox.No) == QtGui.QMessageBox.Yes:
#                    self.edtEndDate.setDate(maxEndDate)
#                    endDate = maxEndDate
        if begDate and endDate:
            result = result and self.checkActionDataEntered(begDate, QDateTime(), endDate, self.tabToken, self.edtBegDate, None, self.edtEndDate)
            result = result and self.checkEventDate(begDate, endDate, None, self.tabToken, None, self.edtEndDate, True)
        result = result and (len(self.modelPreliminaryDiagnostics.items())>0 or self.checkInputMessage(u'предварительный диагноз', False, self.tblPreliminaryDiagnostics))
        if self.edtEndDate.date():
            result = result and self.checkDiagnosticsDataEntered(self.tblPreliminaryDiagnostics)
            result = result and self.checkDiagnosticsDataEntered(self.tblFinalDiagnostics)
            result = result and (len(self.modelFinalDiagnostics.items())>0 or self.checkInputMessage(u'заключительный диагноз', False, self.tblFinalDiagnostics))
            result = result and self.checkDiagnosticsAndResultDivergence()
        result = result and self.checkActionsDateEnteredActuality(begDate, endDate, [self.tabStatus, self.tabMisc])
        result = result and self.checkActionsDataEntered(begDate, endDate)
        result = result and self.checkDeposit(True)
        result = result and self.tabCash.checkDataLocalContract()
        result = result and self.checkSerialNumberEntered()
        result = result and self.checkTabNotesEventExternalId()
        if self.edtEndDate.date():
            result = result and self.checkAndUpdateExpertise(self.edtEndDate.date(), self.cmbPerson.value())
        return result


    def checkDiagnosticsAndResultDivergence(self):
        itemsPre = self.tblPreliminaryDiagnostics.model().items()
        itemsFin = self.tblFinalDiagnostics.model().items()
        divergence = False
        lenPre = len(itemsPre)
        lenFin = len(itemsFin)
        if lenPre != lenFin:
            divergence = True
        else:
            for i in range(0, lenPre):
                if forceString(itemsPre[i].value('MKB'))[:3] != forceString(itemsFin[i].value('MKB'))[:3]:
                    divergence = True

        result = True
        if self.cmbResult.currentIndex() <= 1 and divergence:
            result = result and self.checkValueMessage(u'Расхождение диагнозов не установлено, но диагнозы отличаются.', True, self.cmbResult)
        elif self.cmbResult.currentIndex() > 1 and not divergence:
            result = result and self.checkValueMessage(u'Установлено расхождение диагнозов, но диагнозы не отличаются.', True, self.cmbResult)
        return result


    def checkDiagnosticsDataEntered(self, tbl):
        for row, record in enumerate(tbl.model().items()):
            if not self.checkDiagnosticDataEntered(tbl, row, record):
                return False
        return True


    def checkDiagnosticDataEntered(self, tbl, row, record):
###     self.checkValueMessage(self, message, canSkip, widget, row=None, column=None):
        result = True
        if result:
            MKB = forceString(record.value('MKB'))
            MKBEx = forceString(record.value('MKBEx'))
            result = MKB or self.checkInputMessage(u'диагноз', False, tbl, row, record.indexOf('MKB'))
            result = result and self.checkActualMKB(tbl, self.edtBegDate.date(), MKB, record, row)
            if result:
                char = MKB[:1]
                traumaTypeId = forceRef(record.value('traumaType_id'))
                if char in 'ST':
                    if not traumaTypeId:
                        result = self.checkValueMessage(u'Необходимо указать тип травмы', False, tbl, row, record.indexOf('traumaType_id'))
                    if result:
                        result = MKBEx or self.checkInputMessage(u'Дополнительный диагноз', True if QtGui.qApp.controlMKBExForTraumaType()==0 else False, tbl, row, record.indexOf('MKBEx'))
                        if result:
                            charEx = MKBEx[:1]
                            if charEx not in 'VWXY':
                                result = self.checkValueMessage(u'Доп.МКБ не соотвествует Доп.МКБ при травме', True, tbl, row, record.indexOf('MKBEx'))
                if char not in 'ST' and traumaTypeId:
                    result = self.checkValueMessage(u'Необходимо удалить тип травмы', False, tbl, row, record.indexOf('traumaType_id'))
                result = self.checkRequiresFillingDispanser(result, tbl, record, row, MKB)
#        if result and row == 0:
#            resultId = forceRef(record.value('result_id'))
#            result = resultId or self.checkInputMessage(u'результат', False, tbl, row, record.indexOf('result_id'))
            result = result and self.checkPeriodResultHealthGroup(record, row, tbl)
        return result


    def checkRowEndDate(self, begDate, endDate, row, record, widget):
        result = True
        column = record.indexOf('endDate')
        rowEndDate = forceDate(record.value('endDate'))
        if rowEndDate:
            if rowEndDate>endDate:
                result = result and self.checkValueMessage(u'Дата выполнения должна быть не позже %s' % forceString(endDate), False, widget, row, column)
#            if rowEndDate < lowDate:
#                result = result and self.checkValueMessage(u'Дата выполнения должна быть не раньше %s' % forceString(lowDate), False, widget, row, column)
        return result


    def recordAcceptable(self, record):
        return recordAcceptable(self.clientSex, self.clientAge, record)


    def getDiagFilter(self):
        specialityId = self.personSpecialityId
        result = self.mapSpecialityIdToDiagFilter.get(specialityId, None)
        if result is None:
            result = QtGui.qApp.db.translate('rbSpeciality', 'id', specialityId, 'mkbFilter')
            if result is None:
                result = ''
            else:
                result = forceString(result)
            self.mapSpecialityIdToDiagFilter[specialityId] = forceString(result)
        return result


    def checkDiagnosis(self, MKB):
        diagFilter = self.getDiagFilter()
        return checkDiagnosis(self, MKB, diagFilter, self.clientId, self.clientSex, self.clientAge, self.edtBegDate.date())


    def getDiagnosisTypeId(self, dt):
        return forceRef(QtGui.qApp.db.translate('rbDiagnosisType', 'code', '2' if dt else '9', 'id'))


    def getEventInfo(self, context):
        result = CEventEditDialog.getEventInfo(self, context)
        # ручная инициализация свойств
        result._isPrimary = False
        # ручная инициализация таблиц
        result._actions = CActionInfoProxyList(context, [self.tabStatus.modelAPActions, self.tabMisc.modelAPActions], result)
        result._diagnosises = CDiagnosticInfoProxyList(context, [self.modelPreliminaryDiagnostics, self.modelFinalDiagnostics])
        return result


    def getTempInvalidInfo(self, context):
        return None


    def setContractId(self, contractId):
        if self.contractId != contractId:
            CEventEditDialog.setContractId(self, contractId)
            self.tabCash.modelAccActions.setContractId(contractId)
            self.tabCash.updatePaymentsSum()

# # #
    @pyqtSignature('int')
    def on_tabWidget_currentChanged(self, index):
        widget = self.tabWidget.widget(index)
        if widget is not None:
            focusProxy = widget.focusProxy()
            if focusProxy:
                focusProxy.setFocus(Qt.OtherFocusReason)
        self.btnPrintMedicalDiagnosisEnabled(index)


    @pyqtSignature('QDate')
    def on_edtBegDate_dateChanged(self, date):
        self.eventDate = QDate(date)
        self.setFilterResult(date)
#        contractId = self.cmbContract.value()
        self.cmbContract.setDate(self.getDateForContract())
#        self.cmbContract.setValue(contractId)
        self.cmbPerson.setBegDate(date)
        self.cmbPerson2.setBegDate(date)
        self.setPersonDate(date)


    @pyqtSignature('QDate')
    def on_edtEndDate_dateChanged(self, date):
        self.eventSetDateTime.setDate(date)
        self.cmbContract.setDate(self.getDateForContract())
        self.cmbPerson.setEndDate(self.eventSetDateTime.date())
        self.cmbPerson2.setEndDate(self.eventSetDateTime.date())
        self.tabStatus.setEndDate(self.eventSetDateTime.date())
        self.tabMisc.setEndDate(self.eventSetDateTime.date())
        self.setEnabledChkCloseEvent(self.eventSetDateTime.date())
        if getEventShowTime(self.eventTypeId):
            time = QTime.currentTime() if date else QTime()
            self.edtEndTime.setTime(time)


    @pyqtSignature('QTime')
    def on_edtEndTime_timeChanged(self, time):
        self.eventSetDateTime.setTime(time)


    @pyqtSignature('int')
    def on_cmbOrg_currentIndexChanged(self):
        self.setOrgId(self.cmbOrg.value())


    @pyqtSignature('int')
    def on_cmbPerson_currentIndexChanged(self):
        oldPersonId = self.personId
        self.setPersonId(self.cmbPerson.value())
# что-то сомнительным показалось - ну поменяли отв. врача,
# всё равно менять врачей в действии вроде неправильно. или правильно?
        self.tabStatus.updatePersonId(oldPersonId, self.personId)
        self.tabMisc.updatePersonId(oldPersonId, self.personId)


    @pyqtSignature('int')
    def on_btnPrint_printByTemplate(self, templateId):
        context = CInfoContext()
        eventInfo = self.getEventInfo(context)

        data = { 'event' : eventInfo,
                 'client': eventInfo.client
               }
        applyTemplate(self, templateId, data)


# # #


    @pyqtSignature('')
    def on_actDiagnosticsAddAccomp_triggered(self):
        tbl = self.tblPreliminaryDiagnostics
        model = tbl.model()
        currentRow = tbl.currentIndex().row()
        if currentRow>=0:
            currentRecord = model.items()[currentRow]
            newRecord = model.getEmptyRecord()
            newRecord.setValue('diagnosisType', QVariant(CF106Dialog.dfAccomp))
            newRecord.setValue('speciality_id', currentRecord.value('speciality_id'))
            newRecord.setValue('healthGroup_id', currentRecord.value('healthGroup_id'))
            model.insertRecord(currentRow+1, newRecord)
            tbl.setCurrentIndex(model.index(currentRow+1, newRecord.indexOf('MKB')))



# # # Actions # # #

    def getPrevActionId(self, action, type):
        return None


#
# #####################################################################################33
#


class CF106DiagnosticsModel(CMKBListInDocTableModel):
    MKB_allowed_morphology = ['C', 'D']
    deathDiagnosisTypes = True

    def __init__(self, parent, mainDiagnosisTypeCode, accompDiagnosisTypeCode):
        CMKBListInDocTableModel.__init__(self, 'Diagnostic', 'id', 'event_id', parent)
        self.isManualSwitchDiagnosis = QtGui.qApp.defaultIsManualSwitchDiagnosis()
        self.isMKBMorphology = QtGui.qApp.defaultMorphologyMKBIsVisible()
        self.characterIdForHandleDiagnosis = None
        self.columnHandleDiagnosis = None
        self.addExtCol(CICDExInDocTableCol(u'МКБ',         'MKB',   7), QVariant.String)
        self.addExtCol(CICDExInDocTableCol(u'Доп.МКБ',     'MKBEx', 7), QVariant.String)
        if QtGui.qApp.isTNMSVisible():
            self.addCol(CTNMSCol(u'TNM-Ст', 'TNMS',  10))
        if self.isMKBMorphology:
            self.addExtCol(CMKBMorphologyCol(u'Морф.', 'morphologyMKB', 10, 'MKB_Morphology', filter='`group` IS NOT NULL'), QVariant.String)
        self.addCol(CDiseaseCharacter(     u'Хар',         'character_id',   7, showFields=CRBComboBox.showCode, preferredWidth=150)).setToolTip(u'Характер')
        if self.isManualSwitchDiagnosis:
            self.addExtCol(CBoolInDocTableCol( u'П',   'handleDiagnosis', 10), QVariant.Int)
            self.characterIdForHandleDiagnosis = forceRef(QtGui.qApp.db.translate('rbDiseaseCharacter', 'code', '1', 'id'))
            self.columnHandleDiagnosis = self._mapFieldNameToCol.get('handleDiagnosis')
        self.addCol(CDiseasePhases(        u'Фаза',        'phase_id',       7, showFields=CRBComboBox.showCode, preferredWidth=150)).setToolTip(u'Фаза')
        self.addCol(CDiseaseStage(         u'Ст',          'stage_id',       7, showFields=CRBComboBox.showCode, preferredWidth=150)).setToolTip(u'Стадия')
        self.addCol(CRBInDocTableCol(    u'Травма',        'traumaType_id', 10, 'rbTraumaType', addNone=True, showFields=CRBComboBox.showName, preferredWidth=150))
        self.addCol(CInDocTableCol(u'Описание',     'freeInput', 15))
        db = QtGui.qApp.db
        self.mainDiagnosisTypeId   = forceRef(db.translate('rbDiagnosisType', 'code', mainDiagnosisTypeCode, 'id'))
        self.accompDiagnosisTypeId = forceRef(db.translate('rbDiagnosisType', 'code', accompDiagnosisTypeCode, 'id'))
        self.setFilter(self.table['diagnosisType_id'].inlist([self.mainDiagnosisTypeId, self.accompDiagnosisTypeId]))
        self.readOnly = False
        self.getDispanserIdLists()


    def getDispanserIdLists(self):
        db = QtGui.qApp.db
        self.observedDispanserIdList = db.getDistinctIdList('rbDispanser', 'id', ['observed = 1'])
        recordIdList = db.getDistinctIdList('rbDispanser', 'id', ['code = 2'])
        self.takenDispanserId = recordIdList[0] if len(recordIdList) > 0 else None
        self.takenDispanserIdList = db.getDistinctIdList('rbDispanser', 'id', ['code = 2 OR code = 6'])
        consistDispanserIdList = db.getDistinctIdList('rbDispanser', 'id', ['code = 1'])
        self.consistDispanserId = consistDispanserIdList[0] if len(consistDispanserIdList) > 0 else None


    def setReadOnly(self, value):
        self.readOnly = value


    def manualSwitchDiagnosis(self):
        return self.isManualSwitchDiagnosis

    def getCloseOrMainDiagnosisTypeIdList(self):
        return []


    def flags(self, index=QModelIndex()):
        if self.readOnly:
            return Qt.ItemIsSelectable | Qt.ItemIsEnabled
        result = CMKBListInDocTableModel.flags(self, index)
        row = index.row()
        if row < len(self._items):
            column = index.column()
            if self.isManualSwitchDiagnosis and index.isValid():
                if column == self.columnHandleDiagnosis:
                    characterId = forceRef(self.items()[row].value('character_id'))
                    if characterId != self.characterIdForHandleDiagnosis:
                        result = (result & ~Qt.ItemIsUserCheckable)
#                        return result
            if self.isMKBMorphology and index.isValid():
                if column == self._mapFieldNameToCol.get('morphologyMKB'):
                    mkb = forceString(self.items()[row].value('MKB'))
                    if not (bool(mkb) and mkb[0] in CF106DiagnosticsModel.MKB_allowed_morphology):
                        result = (result & ~Qt.ItemIsEditable)
        return result


    def getDiagnosisTypeId(self, first):
        if first:
            return self.mainDiagnosisTypeId
        else:
            return self.accompDiagnosisTypeId


    def getEmptyRecord(self):
        isFirst = not bool(self.items())
        result = CMKBListInDocTableModel.getEmptyRecord(self)
        result.append(QtSql.QSqlField('diagnosis_id',     QVariant.Int))
        result.append(QtSql.QSqlField('diagnosisType_id', QVariant.Int))
        result.append(QtSql.QSqlField('speciality_id',    QVariant.Int))
        result.append(QtSql.QSqlField('person_id',        QVariant.Int))
        result.append(QtSql.QSqlField('dispanser_id',     QVariant.Int))
        result.append(QtSql.QSqlField('hospital',         QVariant.Int))
        result.append(QtSql.QSqlField('healthGroup_id',   QVariant.Int))
        result.append(QtSql.QSqlField('result_id',        QVariant.Int))
        result.append(QtSql.QSqlField('setDate',          QVariant.DateTime))
        result.append(QtSql.QSqlField('endDate',          QVariant.DateTime))
        result.append(QtSql.QSqlField('payStatus',        QVariant.Int))
        result.setValue('diagnosisType_id', QVariant(self.getDiagnosisTypeId(isFirst)))
        return result


    def setData(self, index, value, role=Qt.EditRole):
        column = index.column()
        row = index.row()
        if not variantEq(self.data(index, role), value):
            if column == 0: # код МКБ
                newMKB = forceString(value)
                if not newMKB:
                    specifiedMKB = ''
                    specifiedMKBEx = ''
                    specifiedCharacterId = None
                    specifiedTraumaTypeId = None
                    specifiedDispanserId = None
                    specifiedRequiresFillingDispanser = 0
                    specifiedProlongMKB = False
                else:
                    acceptable, specifiedMKB, specifiedMKBEx, specifiedCharacterId, specifiedTraumaTypeId, specifiedDispanserId, specifiedRequiresFillingDispanser, specifiedProlongMKB = QObject.parent(self).specifyDiagnosis(newMKB)
                    if not acceptable:
                        return False
                value = toVariant(specifiedMKB)
                result = CMKBListInDocTableModel.setData(self, index, value, role)
                if result:
                    if specifiedRequiresFillingDispanser == 2:
                        self.updateDispanserByMKB(row, specifiedDispanserId, specifiedProlongMKB)
                    self.updateCharacterByMKB(row, specifiedMKB, specifiedCharacterId)
                    self.updateTraumaType(row, specifiedMKB, specifiedTraumaTypeId)
                return result
            if column == 1: # доп. код МКБ
                newMKB = forceString(value)
                if not newMKB:
                    pass
                else:
                    acceptable = QObject.parent(self).checkDiagnosis(newMKB)
                    if not acceptable:
                        return False
                value = toVariant(newMKB)
                result = CMKBListInDocTableModel.setData(self, index, value, role)
#                if result:
#                    self.updateCharacterByMKB(row, specifiedMKB)
                return result
            return CMKBListInDocTableModel.setData(self, index, value, role)
        else:
            return True


    def payStatus(self, row):
        if 0 <= row < len(self.items()):
            return forceInt(self.items()[row].value('payStatus'))
        else:
            return 0


    def removeRowEx(self, row):
        self.removeRows(row, 1)


    def updateDispanserByMKB(self, row, specifiedDispanserId, specifiedProlongMKB):
        item = self.items()[row]
        if specifiedProlongMKB and specifiedDispanserId and specifiedDispanserId in self.observedDispanserIdList:
            dispanserId = specifiedDispanserId if specifiedDispanserId not in self.takenDispanserIdList else self.consistDispanserId
            item.setValue('dispanser_id', toVariant(dispanserId))
            self.emitCellChanged(row, item.indexOf('dispanser_id'))
        elif not specifiedProlongMKB:
            item.setValue('dispanser_id', toVariant(self.takenDispanserId))
            self.emitCellChanged(row, item.indexOf('dispanser_id'))


    def updateCharacterByMKB(self, row, MKB, specifiedCharacterId):
        characterIdList = getAvailableCharacterIdByMKB(MKB)
        item = self.items()[row]
        if specifiedCharacterId in characterIdList:
            characterId = specifiedCharacterId
        else:
            characterId = forceRef(item.value('character_id'))
            if (characterId in characterIdList) or (characterId is None and not characterIdList):
                return
            if characterIdList:
                characterId = characterIdList[0]
            else:
                characterId = None
        item.setValue('character_id', toVariant(characterId))
        self.emitCellChanged(row, item.indexOf('character_id'))


    def updateTraumaType(self, row, MKB, specifiedTraumaTypeId):
        item = self.items()[row]
        prevTraumaTypeId = forceRef(item.value('traumaType_id'))
        if specifiedTraumaTypeId:
            traumaTypeId = specifiedTraumaTypeId
        else:
            traumaTypeId = prevTraumaTypeId
        if traumaTypeId != prevTraumaTypeId:
            item.setValue('traumaType_id', toVariant(traumaTypeId))
            self.emitCellChanged(row, item.indexOf('traumaType_id'))
