# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
u"""Журнал оповещений"""

from PyQt4 import QtGui
from PyQt4.QtCore import QDate, Qt

from library.ItemsListDialog import CItemsListDialog, CItemEditorBaseDialog
from library.TableModel import CTextCol, CIntCol, CRefBookCol, CDateTimeCol
from library.Utils import (forceString, forceDateTime, forceStringEx, forceDate,
     forceRef)

from RefBooks.Tables import rbNotificationKind
from Notifications.NotificationManager import strSuccess
from Notifications.Ui_NotificationLogEditorDialog import Ui_NotificationLogEditorDialog
from Notifications.Ui_NotificationLogFilterDialog import Ui_ItemFilterDialog

tblNotificationLog = 'Notification_Log'
strError = u'Ошибка'

class CNotificationLogList(CItemsListDialog):
    u"""Список видов оповещений"""
    def __init__(self, parent, clientId=None):
        CItemsListDialog.__init__(self, parent, [
            CIntCol(u'Правило', ['rule_id'], 20),
            CIntCol(u'Инициатор', ['createPerson_id'], 20),
            CTextCol(u'Адресат', ['recipient'], 20),
            CTextCol(u'Адрес', ['addr'], 20),
            CRefBookCol(u'Вид', ['kind_id'], rbNotificationKind, 20),
            CIntCol(u'Попытки', ['attempt'], 20),
            CDateTimeCol(u'Отправка', ['sendDatetime'], 20),
            CDateTimeCol(u'Подтверждение', ['confirmationDatetime'], 20),
            CTextCol(u'Текст', ['text'], 40),
            CTextCol(u'Статус',  ['lastErrorText'], 40)
            ], tblNotificationLog, ['id'], filterClass=CNotificationLogFilterDialog)
        self.setWindowTitleEx(u'Журнал оповещений')
        self.recipient = clientId


    def select(self, props={}):
        db = QtGui.qApp.db
        table = self.model.table()
        cond = self.generateFilterByProps(props)

        if self.recipient:
            cond.append(table['recipient'].eq(self.recipient))

        addr = props.get('addrTemp')
        if addr:
            cond.append(table['addr'].like(addr))

        recipient = props.get('recipientTemp')
        if recipient:
            cond.append(table['recipient'].eq(recipient))

        sendBegDate = props.get('sendBegDate', QDate())
        if sendBegDate:
            cond.append(table['sendDatetime'].ge(
                forceString(sendBegDate.toString(Qt.ISODate))))

        sendEndDate = props.get('sendEndDate', QDate())
        if sendEndDate:
            cond.append(table['sendDatetime'].le(
                forceString(sendEndDate.toString(Qt.ISODate))))

        confirmBegDate = props.get('confirmBegDate', QDate())
        if confirmBegDate:
            cond.append(table['confirmationDatetime'].ge(
                forceString(confirmBegDate.toString(Qt.ISODate))))

        confirmEndDate = props.get('confirmEndDate', QDate())
        if confirmEndDate:
            cond.append(table['confirmationDatetime'].le(
                forceString(confirmEndDate.toString(Qt.ISODate))))

        createPerson = props.get('createPerson')
        if createPerson:
            cond.append(table['createPerson_id'].eq(createPerson))

        kind = props.get('kind')
        if kind:
            cond.append(table['kind_id'].eq(kind))

        status = props.get('status')
        if status == strError:
            cond.append(table['lastErrorText'].ne(strSuccess))
        elif status:
            cond.append(table['lastErrorText'].eq(strSuccess))

        return db.getIdList(table, self.idFieldName, cond, self.order)


    def getItemEditor(self):
        u"""Возвращает диалог редактировния вида оповещения"""
        return CNotificationLogEditor(self)


class CNotificationLogEditor(CItemEditorBaseDialog, Ui_NotificationLogEditorDialog):
    u"""Диалог редактирования события журнала оповещений"""
    def __init__(self, parent):
        CItemEditorBaseDialog.__init__(self, parent, tblNotificationLog)
        self.setupUi(self)
        self.buttonBox.setEnabled(False)


    def setRecord(self, record):
        CItemEditorBaseDialog.setRecord(self, record)
        self.edtNotificationRule.setText(forceString(record.value('rule_id')))
        self.edtCreateDatetime.setDateTime(forceDateTime(record.value('createDatetime')))
        self.edtRecipient.setText(forceString(record.value('recipient')))
        self.edtText.setText(forceString(record.value('text')))
        self.edtLastErrorText.setText(forceString(record.value('lastErrorText')))
        self.edtAddress.setText(forceString(record.value('addr')))


class CNotificationLogFilterDialog(QtGui.QDialog, Ui_ItemFilterDialog):
    def __init__(self, parent):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)
        self.setWindowTitle(u'Фильтр журнала оповещений')
        self.edtSendBegDate.canBeEmpty(True)
        self.edtSendEndDate.canBeEmpty(True)
        self.edtConfirmBegDate.canBeEmpty(True)
        self.edtConfirmEndDate.canBeEmpty(True)
        self.cmbKind.setTable(rbNotificationKind)
        self.cmbStatus.addItem('')
        self.cmbStatus.addItem(strSuccess)
        self.cmbStatus.addItem(strError)


    def setProps(self, props):
        self.edtAddressTemplate.setText(props.get('addrTemp', ''))
        self.edtRecipientTemplate.setText(props.get('recipientTemp', ''))
        self.edtSendBegDate.setDate(props.get('sendBegDate', QDate()))
        self.edtSendEndDate.setDate(props.get('sendEndDate', QDate()))
        self.edtConfirmBegDate.setDate(props.get('confirmBegDate', QDate()))
        self.edtConfirmEndDate.setDate(props.get('confirmEndDate', QDate()))
        self.cmbCreatePerson.setValue(props.get('createPerson', 0))
        self.cmbKind.setValue(props.get('kind', 0))
        self.cmbStatus.setCurrentIndex(
            self.cmbStatus.findData(props.get('status', ''), Qt.DisplayRole))


    def props(self):
        result = {
         'addrTemp': forceStringEx(self.edtAddressTemplate.text()),
         'recipientTemp': forceStringEx(self.edtRecipientTemplate.text()),
         'sendBegDate': forceDate(self.edtSendBegDate.date()),
         'sendEndDate': forceDate(self.edtSendEndDate.date()),
         'confirmBegDate': forceDate(self.edtConfirmBegDate.date()),
         'confirmEndDate': forceDate(self.edtConfirmEndDate.date()),
         'createPerson': forceRef(self.cmbCreatePerson.value()),
         'kind': forceRef(self.cmbKind.value()),
         'status': forceString(self.cmbStatus.currentText()),
        }
        return result
