# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
u"""Работа с правилами оповещений"""

from PyQt4 import QtGui
from PyQt4.QtCore import pyqtSignature

from library.ItemsListDialog import CItemsListDialog, CItemEditorBaseDialog
from library.TableModel import CIntCol, CEnumCol, CTextCol
from library.InDocTable import CInDocTableModel, CRBInDocTableCol
from library.Utils import (toVariant, forceBool, forceInt, forceStringEx,
    forceString, forceRef)

from RefBooks.Tables import rbNotificationKind

from Notifications.Ui_NotificationRuleEditorDialog import (
    Ui_NotificationRuleEditorDialog)

tblNotificationRule = 'Notification_Rule'
tblNotificationKind = 'Notification_Kind'


class CNotificationRule(object):
    u"""Класс для работы с правилами оповещений"""
    ncInternal = 0
    ncExternal = 1
    ntSchedule = 0
    ntRegistry = 1
    nrIsRestricted = 1
    ncdConfirmation = 0
    ncdNotification = 1
    ncdCancel = 2


    def __init__(self, record):
        self.id = forceRef(record.value('id'))
        self.template = forceString(record.value('template'))
        self.name = forceString(record.value('name'))
        self.condition = forceInt(record.value('condition'))
        self.notificationKindList = self.__getNotificationKindList(self.id)
        self.multiplicity = forceInt(record.value('multiplicity'))
        self.term = forceInt(record.value('term'))
        self.modifyPersonId = forceRef(record.value('modifyPerson_id'))
        self.createPersonId = forceRef(record.value('createPerson_id'))
        self.ignoreToday = forceBool(record.value('ignoreToday'))


    def __getNotificationKindList(self, _id):
        u"""Получает виды оповещения по идентификатору правила,
        возвращает список кортежей (код оповещения типа,
        идентификатор типа оповещения, идентификатор типа контакта)"""
        result = []

        stmt = """SELECT rbNotificationKind.code,
            rbNotificationKind.contactType_id,
            rbNotificationKind.id
        FROM Notification_Kind
        LEFT JOIN rbNotificationKind ON
            Notification_Kind.kind_id = rbNotificationKind.id
        WHERE Notification_Kind.rule_id = %d""" % _id

        query = QtGui.qApp.db.query(stmt)

        while query.next():
            record = query.record()

            if record:
                code = forceString(record.value(0))
                contactTypeId = forceRef(record.value(1))
                kindId = forceRef(record.value(2))
                result.append((code, kindId, contactTypeId))

        return result


class CNotificationRuleList(CItemsListDialog):
    u"""Список правил оповещений"""
    def __init__(self, parent):
        CItemsListDialog.__init__(self, parent, [
                CIntCol(u'Идентификатор', ['id'], 20),
                CTextCol(u'Наименование', ['name'], 40),
                CEnumCol(u'Класс', ['class'], (u'Внутренний', u'Внешний'), 20),
                CEnumCol(u'Тип', ['type'], (u'Расписание', u'Картотека'), 20),
                CEnumCol(u'Условие', ['condition'], (u'Подтверждение записи',
                    u'Напоминание', u'Отмена записи'), 20),
            ], tblNotificationRule, ['id', 'class', 'type'])
        self.setWindowTitleEx(u'Правила оповещений')


    def getItemEditor(self):
        u"""Возвращает диалог редактировния вида оповещения"""
        return CNotificationRuleEditor(self)


class CNotificationRuleEditor(CItemEditorBaseDialog,
        Ui_NotificationRuleEditorDialog):
    u"""Диалог редактирования правил оповещения"""
    def __init__(self, parent):
        CItemEditorBaseDialog.__init__(self, parent, tblNotificationRule)
        self.addModels('NotificationKind', CNotificationKindModel(self))

        self.setupUi(self)
        self.setModels(self.tblNotificationKind, self.modelNotificationKind,
            self.selectionModelNotificationKind)
        self.tblNotificationKind.addPopupDelRow()


    def setRecord(self, record):
        CItemEditorBaseDialog.setRecord(self, record)
        _id = self.itemId()

        self.cmbClass.setCurrentIndex(forceInt(record.value('class')))
        self.cmbType.setCurrentIndex(forceInt(record.value('type')))
        self.cmbCondition.setCurrentIndex(forceInt(record.value('condition')))
        self.chkIsRestricted.setChecked(forceBool(record.value('restricted')))
        self.edtTemplate.setText(forceStringEx(record.value('template')))
        self.edtName.setText(forceStringEx(record.value('name')))
        self.edtMultiplicity.setValue(forceInt(record.value('multiplicity')))
        self.edtTerm.setValue(forceInt(record.value('term')))
        self.chkIgnoreToday.setChecked(forceBool(record.value('ignoreToday')))


        self.modelNotificationKind.loadItems(_id)


    def getRecord(self):
        record = CItemEditorBaseDialog.getRecord(self)
        record.setValue('class', toVariant(
            forceInt(self.cmbClass.currentIndex())))
        record.setValue('type', toVariant(
            forceInt(self.cmbType.currentIndex())))
        record.setValue('condition', toVariant(
            forceInt(self.cmbCondition.currentIndex())))
        record.setValue('restricted', toVariant(
            forceBool(self.chkIsRestricted.isChecked())))
        record.setValue('template', toVariant(
            forceStringEx(self.edtTemplate.toPlainText())))
        record.setValue('name', toVariant(
            forceStringEx(self.edtName.text())))
        record.setValue('multiplicity', toVariant(
            forceInt(self.edtMultiplicity.value())))
        record.setValue('term', toVariant(
            forceInt(self.edtTerm.value())))
        record.setValue('ignoreToday', toVariant(
            forceBool(self.chkIgnoreToday.isChecked())))
        return record


    def saveInternals(self, _id):
        self.modelNotificationKind.saveItems(_id)


    def checkDataEntered(self):
        result = True
        return result


    def destroy(self):
        u"""Удаляет созданные модели"""
        self.tblNotificationKind.setModel(None)

        del self.modelNotificationKind

    @pyqtSignature('int')
    def on_cmbCondition_currentIndexChanged(self, index):
        flag = index == CNotificationRule.ncdNotification
        self.edtMultiplicity.setEnabled(flag)
        self.edtTerm.setEnabled(flag)


class CNotificationKindModel(CInDocTableModel):
    u"""Модель видов оповещений"""
    def __init__(self, parent):
        CInDocTableModel.__init__(self,
            tblNotificationKind, 'id', 'rule_id', parent)
        self.addCol(CRBInDocTableCol(
            u'Тип', 'kind_id', 30, rbNotificationKind))
