# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
u"""Диалог оповещения пациента"""

from PyQt4 import QtGui

from PyQt4.QtCore import pyqtSignature
from PyQt4.QtGui import QDialogButtonBox

from library.DialogBase import CDialogBase
from library.Utils      import forceString

from Notifications.NotificationManager import CNotificationManager,  parseNotificationTemplate
from Notifications.NotificationRule    import CNotificationRule, tblNotificationRule

from Notifications.Ui_NotifyDialog     import Ui_NotifyDialog


class CNotifyDialog(CDialogBase, Ui_NotifyDialog):
    u"""Диалог редактирования оповещения"""
    def __init__(self, clientInfoList, type=None, schedule=None):
        CDialogBase.__init__(self, None)
        self.clientInfoList = clientInfoList
        self.schedule = schedule
        self.setupUi(self)
        filter = 'class=%d' % CNotificationRule.ncInternal

        if type is not None:
            filter += ' AND type=%d' % type

        self.cmbNotificationRule.setFilter(filter)
        self.updateButtonBox()


    @pyqtSignature('QAbstractButton*')
    def on_bbxNotify_clicked(self, button):
        u"""Обработчик нажатия кнопок диалога"""
        buttonCode = self.bbxNotify.standardButton(button)
        manager = CNotificationManager()

        if self.clientInfoList and not isinstance(self.clientInfoList[0], tuple):
            self.clientInfoList = zip(
                self.clientInfoList, (None,)*len(self.clientInfoList))

        if buttonCode == QDialogButtonBox.Ok:
            msg = forceString(self.edtMessage.toPlainText())
            manager.notifyClient(self.clientInfoList,
                self.cmbNotificationRule.value(), self.schedule, msg)


    def updateExample(self):
        ruleId = self.cmbNotificationRule.value()
        info = self.clientInfoList[0] if len(self.clientInfoList) > 0 else None
        clientId, scheduleItem = (info if info and isinstance(info, tuple)
                                    else (info, None))

        if ruleId and clientId:
            record = QtGui.qApp.db.getRecord(tblNotificationRule, '*', ruleId)
            rule = CNotificationRule(record)
            text = parseNotificationTemplate(rule.template, clientId,
                                (scheduleItem, self.schedule) if scheduleItem else None,
                                msg=self.edtMessage.toPlainText())
            if not text:
                text = u'Синтаксическая ошибка в шаблоне'
        else:
            text = u'Выберите правило из списка'

        self.edtMsgExample.setPlainText(text)


    def updateButtonBox(self):
        self.bbxNotify.button(QDialogButtonBox.Ok).setEnabled(
                              self.cmbNotificationRule.value() is not None)


    @pyqtSignature('int')
    def on_cmbNotificationRule_currentIndexChanged(self):
        self.updateExample()
        self.updateButtonBox()


    @pyqtSignature('')
    def on_edtMessage_textChanged(self):
        self.updateExample()

