# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui
from PyQt4.QtCore import Qt, SIGNAL, QEvent, QSize, QVariant

from library.Utils         import forceString, toVariant
from library.InDocTable    import CInDocTableCol, CInDocTableView

from Events.ActionProperty import CActionPropertyValueTypeRegistry


class CActionTypeBaseDelegate(QtGui.QItemDelegate):
    def __init__(self, lineHeight, parent=None):
        QtGui.QItemDelegate.__init__(self, parent)
        self.lineHeight = lineHeight


    def eventFilter(self, editor, event):
        if event.type() == QEvent.KeyPress:
            if event.key() in (Qt.Key_Return, Qt.Key_Enter) and isinstance(editor, QtGui.QTextEdit):
                return False
        return QtGui.QItemDelegate.eventFilter(self, editor, event)


    def commit(self):
        editor = self.sender()
        self.emit(SIGNAL('commitData(QWidget *)'), editor)


    def commitAndCloseEditor(self):
        editor = self.sender()
        self.emit(SIGNAL('commitData(QWidget *)'), editor)
        self.emit(SIGNAL('closeEditor(QWidget *,QAbstractItemDelegate::EndEditHint)'), editor, QtGui.QAbstractItemDelegate.NoHint)


class CActionTypeDelegate(CActionTypeBaseDelegate):
    def __init__(self, lineHeight, parent, isValueDomain=False):
        CActionTypeBaseDelegate.__init__(self, lineHeight, parent)
        self.isValueDomain = isValueDomain


    def createEditor(self, parent, option, index):
        model = index.model()
        row = index.row()
        editor = CInDocTableCol( u'Настройка', 'valueDomain', 25).createEditor(parent)
        if row >= 0 and row < len(model._items):
            record = model._items[row]
            typeName = forceString(record.value('typeName'))
            if typeName == 'Constructor':
                valueType = CActionPropertyValueTypeRegistry.get(typeName, None)
                editorClass = valueType.getEditorClass()
                if editorClass:
                    editor = editorClass(None, None, parent, None, None, self.isValueDomain) # action, domain, parent, clientId, eventTypeId, isValueDomain
                    editor.setStatusTip(forceString(model.data(index, Qt.StatusTipRole)))
                    self.connect(editor, SIGNAL('commit()'), self.commit)
                    self.connect(editor, SIGNAL('editingFinished()'), self.commitAndCloseEditor)
        return editor


    def setEditorData(self, editor, index):
        model = index.model()
        row = index.row()
        value = model.data(index, Qt.EditRole)
        if row >= 0 and row < len(model._items):
            record = model._items[row]
            typeName = forceString(record.value('typeName'))
            if typeName == 'Constructor':
                editor.setValue(value)
            else:
                editor.setText(forceString(value))
        else:
            editor.setText(forceString(value))


    def setModelData(self, editor, model, index):
        model = index.model()
        row = index.row()
        if row >= 0 and row < len(model._items):
            record = model._items[row]
            typeName = forceString(record.value('typeName'))
            if typeName == 'Constructor':
                value = editor.value()
            else:
                value = editor.text()
        else:
            value = editor.text()
        model.setData(index, toVariant(value))


    def sizeHint(self, option, index):
        model = index.model()
        row = index.row()
        result = QSize(10, self.lineHeight)
        if row >= 0 and row < len(model._items):
            record = model._items[row]
            typeName = forceString(record.value('typeName'))
            if typeName == 'Constructor':
                propertyType = CActionPropertyValueTypeRegistry.get(typeName, None)
                preferredHeightUnit = propertyType.preferredHeightUnit
                preferredHeight = propertyType.preferredHeight * 2
                result = QSize(10, self.lineHeight*preferredHeight if preferredHeightUnit == 1 else preferredHeight)
        return result


    def updateEditorGeometry(self, editor, option, index):
        QtGui.QItemDelegate.updateEditorGeometry(self, editor, option, index)
        index.model().afterUpdateEditorGeometry(editor, index)


class CActionTypeInDocTableView(CInDocTableView):
    ciSetting = 6

    def __init__(self, parent):
        CInDocTableView.__init__(self, parent)
        self.constructorDelegate = CActionTypeDelegate(self.fontMetrics().height(), self, isValueDomain=True)
        self.setItemDelegateForColumn(self.ciSetting, self.constructorDelegate)


    def sizeHintForRow(self, row):
        model = self.model()
        if model:
            index = model.index(row, self.ciSetting)
            return self.constructorDelegate.sizeHint(None, index).height()*1+1
        return -1

