# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2016-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui
#from PyQt4.QtCore import QRegExp


from library.DbEntityCache import CDbEntityCache
from library.LineEditWithRegExpValidator import prepareRegExp
from library.Utils         import forceBool, forceRef, forceString

__all__ = [ 'getDocumentTypeDescr',
            'CDocumentTypeDescription',
          ]


def getDocumentTypeDescr(documentTypeId):
    return CDocumentTypeDescription.getDescription(documentTypeId)


class CDocumentTypeDescription(CDbEntityCache):
    mapIdToDescription = {}

    @classmethod
    def purge(cls):
        cls.mapIdToDescription.clear()


    @classmethod
    def register(cls, record):
        description = cls(record)
        cls.mapIdToDescription[id] = description
        return description


    @classmethod
    def getDescription(cls, id):
        if id in cls.mapIdToDescription:
            return cls.mapIdToDescription[id]
        else:
            cls.connect()
            db = QtGui.qApp.db
            table = db.table('rbDocumentType')
            description = cls.register(db.getRecord(table, '*', id))
            return description


    def __init__(self, record):
        if record:
            self.id = forceRef(record.value('id'))
            self.code = forceString(record.value('code'))
            self.name = forceString(record.value('name'))
            self.leftPartRegExp = forceString(record.value('leftPartRegExp')) or '\\S*'
            self.hasRightPart = forceBool(record.value('hasRightPart'))
            self.rightPartRegExp = forceString(record.value('rightPartRegExp')) or '\\S*'
            self.partSeparator = forceString(record.value('partSeparator'))[1:] or ' '
            self.numberRegExp = forceString(record.value('numberRegExp')) or '\\S*'
        else:
            self.id = None
            self.code = None
            self.name = None
            self.leftPartRegExp  = '\\S*'
            self.hasRightPart    = False
            self.rightPartRegExp = None
            self.partSeparator   = None
            self.numberRegExp    = '\\S*'


    def splitDocSerial(self, serial):
        if self.hasRightPart:
            for regExp in ( '(%s)\s*%s\s*(%s)' % (self.leftPartRegExp, self.partSeparator, self.rightPartRegExp),
                            '(%s)\s*%s\s*(.*)' % (self.leftPartRegExp, self.partSeparator),
                            '(%s)\s*(%s)' % (self.leftPartRegExp, self.rightPartRegExp),
                            '(%s)\s*(.*)' % (self.leftPartRegExp, )
                          ):
                rx = prepareRegExp(regExp)
                if rx.exactMatch(serial):
                    capturedTexts = rx.capturedTexts()
                    return unicode(capturedTexts[1]), unicode(capturedTexts[2])
        return serial, ''


    def compoundDocSerial(self, leftPart, rightPart):
        if self.hasRightPart:
            return leftPart + self.partSeparator + rightPart
        else:
            return leftPart


    def matchLeftPart(self, part):
        rx = prepareRegExp(self.leftPartRegExp)
        return rx.exactMatch(part)


    def matchRightPart(self, part):
        if self.hasRightPart:
            rx = prepareRegExp(self.rightPartRegExp)
            return rx.exactMatch(part)
        else:
            return not part


    def matchNumber(self, number):
        rx = prepareRegExp(self.numberRegExp)
        return rx.exactMatch(number)
