# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import hashlib
from PyQt4 import QtGui
from PyQt4.QtCore import Qt, QByteArray, QDate, QObject, pyqtSignature,SIGNAL

from library.IdentificationModel import CIdentificationModel, checkIdentification
from library.InDocTable        import CInDocTableModel, CDateInDocTableCol, CEnumInDocTableCol, CInDocTableCol, CRBInDocTableCol
from library.interchange       import getCheckBoxValue, getComboBoxValue, getDateEditValue, getLineEditValue, getRBComboBoxValue, getSpinBoxValue, setCheckBoxValue, setComboBoxValue, setDateEditValue, setLineEditValue, setRBComboBoxValue, setSpinBoxValue
from library.ItemsListDialog   import CItemsListDialog, CItemEditorBaseDialog
from library.TableModel        import CDesignationCol, CRefBookCol, CTextCol
from library.Utils             import addDotsEx, agreeNumberAndWord, forceDate, forceInt, forceRef, forceString, forceStringEx, nameCase, toVariant, trim

from Orgs.Utils                import getOrgStructureDescendants
from Registry.ClientEditDialog import checkSNILSEntered
from Registry.Utils            import getAddressId,  getAddress
from Reports.ReportBase        import CReportBase, createTable
from Reports.ReportView        import CReportViewDialog
from Timeline.PersonTimeTable  import CPersonTimeTableModel

from RefBooks.Ui_PersonsListDialog import Ui_PersonsListDialog
from RefBooks.Ui_PersonEditor      import Ui_ItemEditorDialog
from Users.Rights                  import urEditLoginPasswordProfileUser


class CPersonList(Ui_PersonsListDialog, CItemsListDialog):

    def __init__(self, parent):
        CItemsListDialog.__init__(self, parent, [
            CTextCol(u'Код', ['code'], 6),
            CTextCol(u'СНИЛС', ['SNILS'], 20),
            CTextCol(u'Фамилия', ['lastName'], 20),
            CTextCol(u'Имя', ['firstName'], 20),
            CTextCol(u'Отчество', ['patrName'], 20),
            CTextCol(u'Регистрационное имя', ['login'], 20),
            CTextCol(u'Федеральный код', ['federalCode'], 10),
            CDesignationCol(u'ЛПУ', ['org_id'], ('Organisation', 'infisCode'), 5),
            CDesignationCol(u'Подразделение', ['orgStructure_id'], ('OrgStructure', 'name'), 5),
            CRefBookCol(u'Должность', ['post_id'], 'rbPost', 20),
            CRefBookCol(u'Специальность', ['speciality_id'], 'rbSpeciality', 10),
###            CTextCol(u'ИНФИС код', ['infis'], 20),
            ], 'Person',
#            ['code', 'lastName', 'firstName', 'patrName'])
            # 1. кодов может не быть; 2. лучше видно повторы
            ['lastName', 'firstName', 'patrName'])
        self.setWindowTitleEx(u'Сотрудники')
        self.cmbUserRightsProfile.setTable('rbUserProfile')
        self.cmbUserRightsProfile.setAddNone(True, u'все,не задано')
        self.cmbUserRightsProfile.setValue(None)
        stmt="select distinct speciality_id, name from Person join rbSpeciality on rbSpeciality.id=Person.speciality_id order by name"
        self.specialityList=[]
        query = QtGui.qApp.db.query(stmt)
        self.boxSpec.addItem(u'Без специальности')
        self.boxSpec.addItem(u'Любая специальность')
        self.specialityList.append(None)
        self.specialityList.append(None)
        while query.next():
            record=query.record()
            speciality_id=forceInt(record.value('speciality_id'))
            name=forceString(record.value('name'))
            self.boxSpec.addItem(name)
            self.specialityList.append(speciality_id)
        stmt="select distinct org_id, infisCode from Person left join Organisation on Organisation.id=Person.org_id order by infisCode"
        self.LPUList=[]
        query = QtGui.qApp.db.query(stmt)
        while query.next():
            record=query.record()
            org_id=forceInt(record.value('org_id'))
            infisCode=forceString(record.value('infisCode')) if org_id else u'не задано'
            self.boxLPU.addItem(infisCode)
            self.LPUList.append(org_id)
        stmt='select distinct id, CONCAT(code," | ", name) AS name from rbActivity'
        self.activityList=[]
        query = QtGui.qApp.db.query(stmt)
        self.cmbActivity.addItem(u'Без вида деятельности')
        self.cmbActivity.addItem(u'Любой вид деятельности')
        self.activityList.append(None)
        self.activityList.append(None)
        while query.next():
            record=query.record()
            id = forceRef(record.value('id'))
            name = forceString(record.value('name'))
            self.cmbActivity.addItem(name)
            self.activityList.append(id)

        QObject.connect(
            self.tblItems.horizontalHeader(), SIGNAL('sectionClicked(int)'), self.setSort)


    def getItemEditor(self):
        return CRBPersonEditor(self)


    def select(self, props):
        table = self.model.table()
        cond  = []
        onlyOwn = self.chkOnlyOwn.isChecked()
        onlyWorking = self.chkOnlyWorking.isChecked()
        if self.chkChairPerson.isChecked():
            cond.append(table['chairPerson'].eq(1))
        if onlyOwn:
            cond.append(table['org_id'].eq(QtGui.qApp.currentOrgId()))
        if onlyWorking:
            cond.append(table['retireDate'].isNull())
        if self.chkStrPodr.isChecked():
            orgStructure_id=self.boxStrPodr.value()
            orgStructureIdList=getOrgStructureDescendants(orgStructure_id)
            cond.append(table['orgStructure_id'].inlist(orgStructureIdList))
        if self.chkSpec.isChecked():
            index = self.boxSpec.currentIndex()
            if index == 0:
                cond.append(table['speciality_id'].isNull())
            elif index == 1:
                cond.append(table['speciality_id'].isNotNull())
            else:
                speciality_id=self.specialityList[index]
                cond.append(table['speciality_id'].eq(toVariant(speciality_id)))
        if self.chkActivity.isChecked():
            index = self.cmbActivity.currentIndex()
            c = '%s (SELECT `id` FROM `Person_Activity` WHERE `master_id` = Person.`id`  %s AND `deleted` = 0)'
            if index == 0:
                cond.append(c % ('NOT EXISTS', ''))
            elif index == 1:
                cond.append(c % ('EXISTS', 'AND `activity_id` IS NOT NULL'))
            else:
                activityId = self.activityList[index]
                cond.append(c % ('EXISTS', 'AND `activity_id` ='+str(activityId)))
        if self.chkLPU.isChecked():
            org_id=self.LPUList[self.boxLPU.currentIndex()]
            cond.append(table['org_id'].eq(toVariant(org_id)))
        if self.chkUserRightsProfile.isChecked():
            userProfileId = self.cmbUserRightsProfile.value()
            if userProfileId:
                if userProfileId > 0:
                    cond.append(table['userProfile_id'].eq(toVariant(userProfileId)))
                elif userProfileId == -1:
                    cond.append(table['userProfile_id'].isNotNull())
        if self.chkUserLogin.isChecked():
            userLogin = trim(self.edtUserLogin.text())
            cond.append(table['login'].like(addDotsEx(userLogin)))
        return QtGui.qApp.db.getIdList(table, 'id', where=cond, order=self.order)


    @pyqtSignature('int')
    def on_chkOnlyOwn_stateChanged(self, text):
        self.renewListAndSetTo(self.currentItemId())


    @pyqtSignature('int')
    def on_chkChairPerson_stateChanged(self, text):
        self.renewListAndSetTo(self.currentItemId())


    @pyqtSignature('int')
    def on_chkOnlyWorking_stateChanged(self, text):
        self.renewListAndSetTo(self.currentItemId())


    @pyqtSignature('int')
    def on_chkStrPodr_stateChanged(self, text):
        self.renewListAndSetTo(self.currentItemId())


    @pyqtSignature('int')
    def on_boxStrPodr_currentIndexChanged(self, text):
        if self.chkStrPodr.isChecked():
            self.renewListAndSetTo(self.currentItemId())


    @pyqtSignature('int')
    def on_cmbUserRightsProfile_currentIndexChanged(self, text):
        if self.chkUserRightsProfile.isChecked():
            self.renewListAndSetTo(self.currentItemId())


    @pyqtSignature('int')
    def on_chkUserRightsProfile_stateChanged(self, text):
        self.renewListAndSetTo(self.currentItemId())

    @pyqtSignature('int')
    def on_chkUserLogin_stateChanged(self, text):
        self.renewListAndSetTo(self.currentItemId())

    @pyqtSignature('int')
    def on_chkSpec_stateChanged(self, text):
        self.renewListAndSetTo(self.currentItemId())


    @pyqtSignature('int')
    def on_boxSpec_currentIndexChanged(self, text):
        if self.chkSpec.isChecked():
            self.renewListAndSetTo(self.currentItemId())

    @pyqtSignature('int')
    def on_chkActivity_stateChanged(self, text):
        self.renewListAndSetTo(self.currentItemId())


    @pyqtSignature('int')
    def on_cmbActivity_currentIndexChanged(self, text):
        if self.chkActivity.isChecked():
            self.renewListAndSetTo(self.currentItemId())


    @pyqtSignature('int')
    def on_chkLPU_stateChanged(self, text):
        self.renewListAndSetTo(self.currentItemId())


    @pyqtSignature('int')
    def on_boxLPU_currentIndexChanged(self, text):
        if self.chkLPU.isChecked():
            self.renewListAndSetTo(self.currentItemId())


    @pyqtSignature('')
    def on_btnPrint_clicked(self):
        tbl=self.tblItems
        model = tbl.model()
        doc = QtGui.QTextDocument()
        cursor = QtGui.QTextCursor(doc)

        cursor.setCharFormat(CReportBase.ReportTitle)
        cursor.insertText(u'Сотрудники\n')
        cursor.insertBlock()

        cursor.setCharFormat(CReportBase.ReportBody)
        if self.chkOnlyOwn.isChecked():
            cursor.insertText(u'Только свои\n')
        if self.chkOnlyWorking.isChecked():
            cursor.insertText(u'Только работающие\n')
        if self.chkStrPodr.isChecked():
            cursor.insertText(u'Структурное подразделение: %s\n' % self.boxStrPodr.currentText())
        if self.chkSpec.isChecked():
            cursor.insertText(u'Специальность: %s\n' % self.boxSpec.currentText())
        if self.chkActivity.isChecked():
            cursor.insertText(u'Вид деятельности: %s\n' % self.cmbActivity.currentText())
        if self.chkLPU.isChecked():
            cursor.insertText(u'Внешнее ЛПУ: %s\n' % self.boxLPU.currentText())
        if self.chkUserRightsProfile.isChecked():
            cursor.insertText(u'Профиль прав: %s\n' % self.cmbUserRightsProfile.currentText())
        if self.chkUserLogin.isChecked():
            cursor.insertText(u'Регистрационное имя: %s\n' % self.edtUserLogin.text())
        cursor.insertBlock()

        cols = model.cols()
        colWidths  = [ tbl.columnWidth(i) for i in xrange(len(cols)) ]
        colWidths.insert(0,10)
        totalWidth = sum(colWidths)
        tableColumns = []
        for iCol, colWidth in enumerate(colWidths):
            widthInPercents = str(max(1, colWidth*90/totalWidth))+'%'
            if iCol == 0:
                tableColumns.append((widthInPercents, [u'№'], CReportBase.AlignRight))
            else:
                col = cols[iCol-1]
                colAlingment = Qt.AlignHorizontal_Mask & forceInt(col.alignment())
                format = QtGui.QTextBlockFormat()
                format.setAlignment(Qt.AlignmentFlag(colAlingment))
                tableColumns.append((widthInPercents, [forceString(col.title())], format))

        table = createTable(cursor, tableColumns)
        for iModelRow in xrange(model.rowCount()):
            iTableRow = table.addRow()
            table.setText(iTableRow, 0, iModelRow+1)
            for iModelCol in xrange(len(cols)):
                index = model.createIndex(iModelRow, iModelCol)
                text = forceString(model.data(index))
                table.setText(iTableRow, iModelCol+1, text)

        html = doc.toHtml(QByteArray('utf-8'))
        view = CReportViewDialog(self)
        view.setText(html)
        view.exec_()


    def setSort(self, col):
        name=self.model.cols()[col].fields()[0]
        self.order = name
        header=self.tblItems.horizontalHeader()
        header.setSortIndicatorShown(True)
        header.setSortIndicator(col, Qt.AscendingOrder)
        self.renewListAndSetTo(self.currentItemId())


#
# ##########################################################################
#

class CRBPersonEditor(CItemEditorBaseDialog, Ui_ItemEditorDialog):
    prevAddress = None

    def __init__(self,  parent):
        CItemEditorBaseDialog.__init__(self, parent, 'Person')

        self.__regAddressRecord = None
        self.__regAddress = None
        self.__locAddressRecord = None
        self.__locAddress = None
        self.__documentRecord = None

        self.addModels('EducationDocs',  CEducationDocsModel(self))
        self.addModels('OrderDocs',      COrderDocsModel(self))
        self.addModels('PersonActivity', CPersonActivityModel(self))
        self.addModels('PersonJobType',  CPersonJobTypeModel(self))
        self.addModels('TimeTable',      CPersonTimeTableModel(self))
        self.addModels('PersonContacts', CPersonContactsModel(self))
        self.addModels('Identification', CIdentificationModel(self, 'Person_Identification', 'Person'))

        self.setupUi(self)

        self.setWindowTitleEx(u'Сотрудник')
        self.edtBirthDate.setHighlightRedDate(False)
        self.cmbPost.setTable('rbPost')
        self.cmbSpeciality.setTable('rbSpeciality')
        self.cmbTariffCategory.setTable('rbTariffCategory')
        self.cmbFinance.setTable('rbFinance')
        self.cmbOrg.setValue(QtGui.qApp.currentOrgId())
        self.edtRetireDate.setDate(QDate())
        self.edtLastAccessibleTimelineDate.setDate(QDate())
        self.chkChangePassword.setChecked(False)
        self.edtPassword.setEnabled(False)
        self.cmbUserRightsProfile.setTable('rbUserProfile')
        self.btnCopyPrevAddress.setEnabled(bool(CRBPersonEditor.prevAddress))
        self.cmbDocType.setTable('rbDocumentType', True, 'group_id IN (SELECT id FROM rbDocumentTypeGroup WHERE code=\'1\')')

        self.setModels(self.tblEducationDocs,  self.modelEducationDocs,  self.selectionModelEducationDocs)
        self.setModels(self.tblOrderDocs,      self.modelOrderDocs,      self.selectionModelOrderDocs)
        self.setModels(self.tblPersonActivity, self.modelPersonActivity, self.selectionModelPersonActivity)
        self.setModels(self.tblPersonJobType,  self.modelPersonJobType,  self.selectionModelPersonJobType)
        self.setModels(self.tblTimeTable,      self.modelTimeTable,      self.selectionModelTimeTable)
        self.setModels(self.tblPersonContacts, self.modelPersonContacts, self.selectionModelPersonContacts)
        self.setModels(self.tblIdentification, self.modelIdentification, self.selectionModelIdentification)

        self.tblPersonJobType.addPopupDelRow()
        self.tblPersonContacts.addPopupDelRow()
        self.tblIdentification.addPopupDelRow()

        self.modelTimeTable.setPeriod(0, 1) # для новой записи
        self.setupDirtyCather()
        self.setLoginPasswordProfileEnable()


    def setLoginPasswordProfileEnable(self):
        isEnable = QtGui.qApp.userHasAnyRight([urEditLoginPasswordProfileUser])
        self.edtLogin.setEnabled(isEnable)
        self.chkChangePassword.setEnabled(isEnable)
        self.edtPassword.setEnabled(isEnable)
        self.cmbUserRightsProfile.setEnabled(isEnable)


    def destroy(self):
        self.tblEducationDocs.setModel(None)
        self.tblOrderDocs.setModel(None)
        self.tblPersonActivity.setModel(None)
        self.tblPersonJobType.setModel(None)
        self.tblTimeTable.setModel(None)
        self.tblPersonContacts.setModel(None)

        del self.modelEducationDocs
        del self.modelOrderDocs
        del self.modelPersonActivity
        del self.modelPersonJobType
        del self.modelTimeTable
        del self.modelPersonContacts

    # WTF? свой save - так неожиданно...
    def save(self):
        try:
            db = QtGui.qApp.db
            db.transaction()
            try:
                record = self.getRecord()
                id = db.insertOrUpdate('Person', record)
                regAddressRecord, regAddress, regAddressRecordChanged = self.getAddressRecord(id, 0)
                if regAddressRecordChanged and regAddressRecord is not None:
                    db.insertOrUpdate('Person_Address', regAddressRecord)
                locAddressRecord, locAddress, locAddressRecordChanged = self.getAddressRecord(id, 1)
                if locAddressRecordChanged and locAddressRecord is not None:
                    db.insertOrUpdate('Person_Address', locAddressRecord)
                documentRecord, documentRecordChanged = self.getDocumentRecord(id)
                if documentRecordChanged and documentRecord is not None:
                    db.insertOrUpdate('Person_Document', documentRecord)
                self.modelEducationDocs.saveItems(id)
                self.modelOrderDocs.saveItems(id)
                self.modelPersonActivity.saveItems(id)
                self.modelPersonJobType.saveItems(id)
                self.modelTimeTable.saveItems(id)
                self.modelPersonContacts.saveItems(id)
                self.modelIdentification.saveItems(id)
                db.commit()
            except:
                db.rollback()
                QtGui.qApp.logCurrentException()
                raise
            self.setItemId(id)
            self.__regAddressRecord = regAddressRecord
            self.__regAddress = regAddress
            self.__locAddressRecord = locAddressRecord
            self.__locAddress = locAddress
            self.__documentRecord = documentRecord
            self.setIsDirty(False)
            CRBPersonEditor.prevAddress = (regAddress, forceStringEx(regAddressRecord.value('freeInput')), locAddress)
            return id

        except Exception, e:
            QtGui.qApp.logCurrentException()
            QtGui.QMessageBox.critical( self,
                                        u'',
                                        unicode(e),
                                        QtGui.QMessageBox.Close)
            return None


    def setRecord(self, record):
        CItemEditorBaseDialog.setRecord(self, record)
        id = self.itemId()
        setLineEditValue(self.edtLastName,     record, 'lastName')
        setLineEditValue(self.edtFirstName,    record, 'firstName')
        setLineEditValue(self.edtPatrName,     record, 'patrName')
        setLineEditValue(self.edtCode,         record, 'code')
        setLineEditValue(self.edtFederalCode,  record, 'federalCode')
        setLineEditValue(self.edtRegionalCode, record, 'regionalCode')
        setRBComboBoxValue(self.cmbOrg,        record, 'org_id')
        setRBComboBoxValue(self.cmbOrgStructure,  record, 'orgStructure_id')
        setRBComboBoxValue(self.cmbPost,       record, 'post_id')
        setRBComboBoxValue(self.cmbSpeciality, record, 'speciality_id')
        setLineEditValue(self.edtAcademicDegree, record, 'academicDegree')
        setLineEditValue(self.edtOffice,       record, 'office')
        setLineEditValue(self.edtOffice2,      record, 'office2')
        setRBComboBoxValue(self.cmbTariffCategory,    record, 'tariffCategory_id')
        setRBComboBoxValue(self.cmbFinance,    record, 'finance_id')
        setLineEditValue(self.edtLogin,        record, 'login')
        setRBComboBoxValue(self.cmbUserRightsProfile,  record,  'userProfile_id')
        setDateEditValue(self.edtRetireDate,   record, 'retireDate')
        setCheckBoxValue(self.chkRetired,      record, 'retired')
        setCheckBoxValue(self.chkChairPerson,  record, 'chairPerson')
        setCheckBoxValue(self.chkAvailableForExternal, record, 'availableForExternal')
        setDateEditValue(self.edtLastAccessibleTimelineDate, record, 'lastAccessibleTimelineDate')
        setSpinBoxValue(self.edtTimelineAccessibilityDays, record, 'timelineAccessibleDays')
        setSpinBoxValue(self.edtAmbPlan,       record, 'ambPlan')
        setSpinBoxValue(self.edtHomePlan,      record, 'homPlan')
        setSpinBoxValue(self.edtAmbPlan2,      record, 'ambPlan2')
        setSpinBoxValue(self.edtHomePlan2,     record, 'homPlan2')
        setSpinBoxValue(self.edtExpPlan,       record, 'expPlan')
        setSpinBoxValue(self.edtPrimaryQuota,  record, 'primaryQuota')
        setSpinBoxValue(self.edtOwnQuota,      record, 'ownQuota')
        setSpinBoxValue(self.edtConsultancyQuota,  record, 'consultancyQuota')
        setSpinBoxValue(self.edtExternalQuota,  record, 'externalQuota')

        self.edtBirthDate.setDate(forceDate(record.value('birthDate')))

        setLineEditValue(self.edtBirthPlace, record, 'birthPlace')
        setComboBoxValue(self.cmbSex, record, 'sex')
        setLineEditValue(self.edtSNILS, record, 'SNILS')
        setLineEditValue(self.edtINN, record, 'INN')

        setComboBoxValue(self.cmbTimelinePeriod,       record, 'timelinePeriod')
        setSpinBoxValue( self.edtTimelineCustomLength, record, 'timelineCustomLength')
        setCheckBoxValue(self.chkFillRedDays,    record, 'fillRedDays')

        self.setRegAddressRecord(getPersonAddress(id, 0))
        self.setLocAddressRecord(getPersonAddress(id, 1))
        self.setDocumentRecord(getPersonDocument(id))
        self.modelEducationDocs.loadItems(id)
        self.modelOrderDocs.loadItems(id)
        self.modelPersonJobType.loadItems(id)
        self.modelTimeTable.loadItems(id, self.cmbTimelinePeriod.currentIndex(), self.edtTimelineCustomLength.value())
        self.modelPersonActivity.loadItems(id)
        self.modelPersonContacts.loadItems(id)
        self.modelIdentification.loadItems(self.itemId())
        self.setIsDirty(False)


    def getRecord(self):
        record = CItemEditorBaseDialog.getRecord(self)
#        getLineEditValue(self.edtLastName,     record, 'lastName')
#        getLineEditValue(self.edtFirstName,    record, 'firstName')
#        getLineEditValue(self.edtPatrName,     record, 'patrName')
        record.setValue('lastName',  toVariant(nameCase(forceStringEx(self.edtLastName.text()))))
        record.setValue('firstName', toVariant(nameCase(forceStringEx(self.edtFirstName.text()))))
        record.setValue('patrName',  toVariant(nameCase(forceStringEx(self.edtPatrName.text()))))
        getLineEditValue(self.edtCode,             record, 'code')
        getLineEditValue(self.edtFederalCode,      record, 'federalCode')
        getLineEditValue(self.edtRegionalCode,     record, 'regionalCode')
        getRBComboBoxValue(self.cmbOrg,            record, 'org_id')
        getRBComboBoxValue(self.cmbOrgStructure,   record, 'orgStructure_id')
        getRBComboBoxValue(self.cmbPost,           record, 'post_id')
        getRBComboBoxValue(self.cmbSpeciality,     record, 'speciality_id')
        getLineEditValue(self.edtOffice,           record, 'office')
        getLineEditValue(self.edtOffice2,          record, 'office2')
        getLineEditValue(self.edtAcademicDegree,   record, 'academicDegree')
        getRBComboBoxValue(self.cmbTariffCategory, record, 'tariffCategory_id')
        getRBComboBoxValue(self.cmbFinance,        record, 'finance_id')
        if QtGui.qApp.userHasAnyRight([urEditLoginPasswordProfileUser]):
            getLineEditValue(self.edtLogin,            record, 'login')
            if self.chkChangePassword.isChecked():
                encodedPassword = unicode(self.edtPassword.text()).encode('utf8')
                hashedPassword = hashlib.md5(encodedPassword).hexdigest()
                record.setValue('password', toVariant(hashedPassword))
            elif not self.itemId():
                hashedPassword = hashlib.md5('').hexdigest()
                record.setValue('password', toVariant(hashedPassword))
            else:
                record.remove(record.indexOf('password'))
            getRBComboBoxValue(self.cmbUserRightsProfile,  record,  'userProfile_id')
        getDateEditValue(self.edtRetireDate,   record, 'retireDate')
        getCheckBoxValue(self.chkRetired,      record, 'retired')
        getCheckBoxValue(self.chkChairPerson,  record, 'chairPerson')
        getCheckBoxValue(self.chkAvailableForExternal, record, 'availableForExternal')
        getDateEditValue(self.edtLastAccessibleTimelineDate, record, 'lastAccessibleTimelineDate')
        getSpinBoxValue(self.edtTimelineAccessibilityDays, record, 'timelineAccessibleDays')
        getSpinBoxValue(self.edtAmbPlan,       record, 'ambPlan')
        getSpinBoxValue(self.edtHomePlan,      record, 'homPlan')
        getSpinBoxValue(self.edtAmbPlan2,      record, 'ambPlan2')
        getSpinBoxValue(self.edtHomePlan2,     record, 'homPlan2')
        getSpinBoxValue(self.edtExpPlan,       record, 'expPlan')
        getSpinBoxValue(self.edtPrimaryQuota,  record, 'primaryQuota')
        getSpinBoxValue(self.edtOwnQuota,      record, 'ownQuota')
        getSpinBoxValue(self.edtConsultancyQuota,  record, 'consultancyQuota')
        getSpinBoxValue(self.edtExternalQuota,   record, 'externalQuota')
        getDateEditValue(self.edtBirthDate,      record, 'birthDate')
        getComboBoxValue(self.cmbTimelinePeriod, record, 'timelinePeriod')
        getSpinBoxValue( self.edtTimelineCustomLength, record, 'timelineCustomLength')
        getCheckBoxValue(self.chkFillRedDays,    record, 'fillRedDays')
        getLineEditValue(self.edtBirthPlace,     record, 'birthPlace')
        record.setValue('sex',       toVariant(self.cmbSex.currentIndex()))
        record.setValue('SNILS',     toVariant(forceStringEx(self.edtSNILS.text()).replace('-','').replace(' ','')))
        getLineEditValue(self.edtINN, record, 'INN')
        return record


    def getAddress(self, addressType):
        if addressType == 0:
            return { 'useKLADR'         : self.chkRegKLADR.isChecked(),
                     'KLADRCode'        : self.cmbRegCity.code(),
                     'KLADRStreetCode'  : self.cmbRegStreet.code(),
                     'number'           : forceStringEx(self.edtRegHouse.text()),
                     'corpus'           : forceStringEx(self.edtRegCorpus.text()),
                     'flat'             : forceStringEx(self.edtRegFlat.text()),
                     'freeInput'        : forceStringEx(self.edtRegFreeInput.text())}
        else:
            return { 'useKLADR'         : True,
                     'KLADRCode'        : self.cmbLocCity.code(),
                     'KLADRStreetCode'  : self.cmbLocStreet.code(),
                     'number'           : forceStringEx(self.edtLocHouse.text()),
                     'corpus'           : forceStringEx(self.edtLocCorpus.text()),
                     'flat'             : forceStringEx(self.edtLocFlat.text()),
                     'freeInput'        : ''}



    def getAddressRecord(self,  personId, addressType):
        address = self.getAddress(addressType)
        if address['useKLADR']:
            addressId = getAddressId(address)
        else:
            addressId = None
        oldAddressRecord = self.__regAddressRecord if addressType == 0 else self.__locAddressRecord

        if oldAddressRecord:
            recordChanged = addressId != forceRef(oldAddressRecord.value('address_id')) or address['freeInput'] != forceString(oldAddressRecord.value('freeInput'))
        else:
            recordChanged = True

        if recordChanged:
            record = QtGui.qApp.db.record('Person_Address')
            record.setValue('master_id',  toVariant(personId))
            record.setValue('type',       toVariant(addressType))
            record.setValue('address_id', toVariant(addressId))
            record.setValue('freeInput',  toVariant(address['freeInput']))
        else:
            record = oldAddressRecord

        return record, address, recordChanged


    def setRegAddressRecord(self, record):
        self.__regAddressRecord = record
        if record:
            addressId = forceRef(record.value('address_id'))
#            self.chkRegKLADR.setChecked(addressId is not None)
            if addressId:
                self.__regAddress = getAddress(addressId)
            else:
                self.__regAddress = None
            self.setRegAddress(self.__regAddress, forceString(record.value('freeInput')))
        else:
            self.chkRegKLADR.setChecked(False)
            self.__regAddress = None
            self.setRegAddress(self.__regAddress, '')


    def setRegAddress(self, regAddress, freeInput):
        self.chkRegKLADR.setChecked(regAddress!=None)
        self.edtRegFreeInput.setText(freeInput)
        if regAddress:
            self.cmbRegCity.setCode(regAddress.KLADRCode)
            self.cmbRegStreet.setCity(regAddress.KLADRCode)
            self.cmbRegStreet.setCode(regAddress.KLADRStreetCode)
            self.edtRegHouse.setText(regAddress.number)
            self.edtRegCorpus.setText(regAddress.corpus)
            self.edtRegFlat.setText(regAddress.flat)
        else:
            self.cmbRegCity.setCode(QtGui.qApp.defaultKLADR())
            self.cmbRegStreet.setCity(QtGui.qApp.defaultKLADR())
            self.cmbRegStreet.setCode('')
            self.edtRegHouse.setText('')
            self.edtRegCorpus.setText('')
            self.edtRegFlat.setText('')


    def setLocAddressRecord(self, record):
        self.__locAddressRecord = record
        if record:
            addressId = forceRef(record.value('address_id'))
            self.__locAddress = getAddress(addressId)
        else:
            self.__locAddress = None
        self.setLocAddress(self.__locAddress)


    def setLocAddress(self, locAddress):
        if locAddress:
            self.cmbLocCity.setCode(locAddress.KLADRCode)
            self.cmbLocStreet.setCity(locAddress.KLADRCode)
            self.cmbLocStreet.setCode(locAddress.KLADRStreetCode)
            self.edtLocHouse.setText(locAddress.number)
            self.edtLocCorpus.setText(locAddress.corpus)
            self.edtLocFlat.setText(locAddress.flat)
        else:
            self.cmbLocCity.setCode(QtGui.qApp.defaultKLADR())
            self.cmbLocStreet.setCity(QtGui.qApp.defaultKLADR())
            self.cmbLocStreet.setCode('')
            self.edtLocHouse.setText('')
            self.edtLocCorpus.setText('')
            self.edtLocFlat.setText('')


    def getDocumentRecord(self, personId):
        docTypeId = self.cmbDocType.value()
        serialLeft = forceStringEx(self.edtDocSerialLeft.text())
        serialRight = forceStringEx(self.edtDocSerialRight.text())
        serial = serialLeft + ' ' + serialRight
        number = forceStringEx(self.edtDocNumber.text())
        origin = forceStringEx(self.edtOrigin.text())
        date = forceDate(self.edtDocDate.date())
        if docTypeId in (0, None):
            record = None
            recordChanged = self.__documentRecord is not None
        else:
            if self.__documentRecord is not None:
                recordChanged = (
                    docTypeId != forceRef(self.__documentRecord.value('documentType_id')) or
                    serial != forceString(self.__documentRecord.value('serial')) or
                    number != forceString(self.__documentRecord.value('number')) or
                    origin != forceString(self.__documentRecord.value('origin')) or
                    date != forceDate(self.__documentRecord.value('date'))
                    )
            else:
                recordChanged=True

            if recordChanged:
                record = QtGui.qApp.db.record('Person_Document')
                record.setValue('master_id',       toVariant(personId))
                record.setValue('documentType_id', toVariant(docTypeId))
                record.setValue('serial',          toVariant(serial))
                record.setValue('number',          toVariant(number))
                record.setValue('origin',          toVariant(origin))
                record.setValue('date',             toVariant(date))
            else:
                record = self.__documentRecord

        return record, recordChanged


    def setDocumentRecord(self, record):
        self.__documentRecord = record
        if record:
            self.cmbDocType.setValue(forceInt(record.value('documentType_id')))
            serial = forceString(record.value('serial'))
            for c in '-=/_|':
                serial = serial.replace('c',' ')
            serial = trim(serial).split()
            serialLeft = serial[0] if len(serial)>=1 else ''
            serialRight = serial[1] if len(serial)>=2 else ''
            self.edtDocSerialLeft.setText(serialLeft)
            self.edtDocSerialRight.setText(serialRight)
            self.edtDocNumber.setText(forceString(record.value('number')))
            self.edtOrigin.setText(forceString(record.value('origin')))
            self.edtDocDate.setDate(forceDate(record.value('date')))
        else:
            self.cmbDocType.setValue(None)
            self.edtDocSerialLeft.setText('')
            self.edtDocSerialRight.setText('')
            self.edtDocNumber.setText('')


    def checkDataEntered(self):
        result = True
        code = forceStringEx(self.edtCode.text())
        name = forceStringEx(self.edtLastName.text())
        login = forceStringEx(self.edtLogin.text())
        result = result and (code or self.checkInputMessage(u'код', False, self.edtCode))
        result = result and (name or self.checkInputMessage(u'Фамилия', False, self.edtLastName))
        result = result and (login or  self.checkInputMessage(u'регистрационное имя', True, self.edtLogin))
        if result and login:
            db = QtGui.qApp.db
            table = db.table('Person')
            idList = db.getIdList(table,  'id',  table['login'].eq(login))
            if idList and idList != [self.itemId()]:
                QtGui.QMessageBox.warning(   self,
                                             u'Внимание!',
                                             u'Регистрационное имя "%s" уже занято' % login,
                                             QtGui.QMessageBox.Ok,
                                             QtGui.QMessageBox.Ok)
                self.edtLogin.setFocus(Qt.OtherFocusReason)
                result = False
            result = result and (self.cmbUserRightsProfile.value() or self.checkInputMessage(u'Профиль', False, self.cmbUserRightsProfile))

        if self.cmbSpeciality.value():
            sumQuota = self.edtPrimaryQuota.value()+self.edtOwnQuota.value()+self.edtConsultancyQuota.value()
            if self.chkAvailableForExternal.isChecked():
                sumQuota += self.edtExternalQuota.value()
            result = result and (sumQuota>=100 or self.checkValueMessage(u'Сумма квот должна быть не менее 100%', True, self.edtPrimaryQuota))
        result = result and checkSNILSEntered(self)
        result = result and checkIdentification(self, self.tblIdentification)
        return result


    def updateSchedulePeriod(self):
        period       = self.cmbTimelinePeriod.currentIndex()
        customLength = self.edtTimelineCustomLength.value()
        self.modelTimeTable.setPeriod(period, customLength)


    @pyqtSignature('int')
    def on_cmbOrg_currentIndexChanged(self, index):
        orgId = self.cmbOrg.value()
        self.cmbOrgStructure.setEnabled(bool(orgId))
        self.cmbOrgStructure.setOrgId(orgId)


    @pyqtSignature('int')
    def on_cmbTimelinePeriod_currentIndexChanged(self, index):
        enableCustomLength = index == 2
        self.lblTimelineCustomLength.setEnabled(enableCustomLength)
        self.edtTimelineCustomLength.setEnabled(enableCustomLength)
        self.updateSchedulePeriod()


    @pyqtSignature('int')
    def on_edtTimelineCustomLength_valueChanged(self, i):
        self.updateSchedulePeriod()


    @pyqtSignature('int')
    def on_cmbSpeciality_currentIndexChanged(self, index):
        isDoctor = bool(self.cmbSpeciality.value())
        isAvailableDoctor = isDoctor and self.chkAvailableForExternal.isChecked()
        self.chkAvailableForExternal.setEnabled(isDoctor)
        self.edtLastAccessibleTimelineDate.setEnabled(isDoctor)
        self.edtTimelineAccessibilityDays.setEnabled(isDoctor)
        self.edtAmbPlan.setEnabled(isDoctor)
        self.edtHomePlan.setEnabled(isDoctor)
        self.edtAmbPlan2.setEnabled(isDoctor)
        self.edtHomePlan2.setEnabled(isDoctor)
        self.edtExpPlan.setEnabled(isDoctor)
        self.edtPrimaryQuota.setEnabled(isDoctor)
        self.edtOwnQuota.setEnabled(isDoctor)
        self.edtConsultancyQuota.setEnabled(isDoctor)
        self.edtExternalQuota.setEnabled(isAvailableDoctor)


    @pyqtSignature('int')
    def on_chkChangePassword_stateChanged(self, state):
        self.setIsDirty(True)


    @pyqtSignature('bool')
    def on_chkAvailableForExternal_toggled(self, checked):
        isDoctor = bool(self.cmbSpeciality.value())
        isAvailableDoctorDoctor = isDoctor and checked
        self.edtLastAccessibleTimelineDate.setEnabled(isDoctor)
        self.edtTimelineAccessibilityDays.setEnabled(isDoctor)
        self.edtExternalQuota.setEnabled(isAvailableDoctorDoctor)


    @pyqtSignature('int')
    def on_edtTimelineAccessibilityDays_valueChanged(self, value):
        self.lblTimelineAccessibilityDaysSuffix.setText(agreeNumberAndWord(value, (u'день', u'дня', u'дней')))


    @pyqtSignature('')
    def on_btnCopyPrevAddress_clicked(self):
        if CRBPersonEditor.prevAddress:
            reg, freeInput, loc = CRBPersonEditor.prevAddress
            self.setRegAddress(reg, freeInput)
            self.setLocAddress(loc)


    @pyqtSignature('bool')
    def on_chkRegKLADR_toggled(self, checked):
        self.edtRegFreeInput.setEnabled(not checked)
        self.cmbRegCity.setEnabled(checked)
        self.cmbRegStreet.setEnabled(checked)
        self.edtRegHouse.setEnabled(checked)
        self.edtRegCorpus.setEnabled(checked)
        self.edtRegFlat.setEnabled(checked)


    @pyqtSignature('int')
    def on_cmbRegCity_currentIndexChanged(self,  val):
        code = self.cmbRegCity.code()
        self.cmbRegStreet.setCity(code)


    @pyqtSignature('')
    def on_btnCopy_clicked(self):
        code = self.cmbRegCity.code()
        self.cmbLocCity.setCode(code)
        self.cmbLocStreet.setCity(code)
        self.cmbLocStreet.setCode(self.cmbRegStreet.code())
        self.edtLocHouse.setText(self.edtRegHouse.text())
        self.edtLocCorpus.setText(self.edtRegCorpus.text())
        self.edtLocFlat.setText(self.edtRegFlat.text())


    @pyqtSignature('int')
    def on_cmbLocCity_currentIndexChanged(self,  index):
        code = self.cmbLocCity.code()
        self.cmbLocStreet.setCity(code)


    @pyqtSignature('int')
    def on_cmbDocType_currentIndexChanged(self,  index):
        docTypeId = self.cmbDocType.value()
        serialFormat = forceInt(QtGui.qApp.db.translate('rbDocumentType', 'id', docTypeId, 'serial_format'))
        self.edtDocSerialLeft.setFormat(serialFormat)
        self.edtDocSerialRight.setFormat(serialFormat)



class CEducationDocsModel(CInDocTableModel):
    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'Person_Education', 'id', 'master_id', parent)
        self.setFilter('documentType_id IN (SELECT rbDocumentType.id FROM rbDocumentType LEFT JOIN rbDocumentTypeGroup ON rbDocumentTypeGroup.id = rbDocumentType.group_id WHERE rbDocumentTypeGroup.code=\'3\')')
        self.addCol(CDateInDocTableCol(u'Дата',  'date', 15, canBeEmpty=True))
        self.addCol(CRBInDocTableCol(u'Специальность', 'speciality_id', 30, 'rbSpeciality'))
        self.addCol(CInDocTableCol(u'Статус', 'status', 30))
        self.addCol(CRBInDocTableCol(u'Тип документа', 'documentType_id', 30, 'rbDocumentType', filter='group_id IN (SELECT id FROM rbDocumentTypeGroup WHERE code=\'3\')'))
        self.addCol(CInDocTableCol(u'Серия', 'serial', 8))
        self.addCol(CInDocTableCol(u'Номер', 'number', 16))
        self.addCol(CInDocTableCol(u'Выдан', 'origin', 30))
        self.addCol(CDateInDocTableCol(u'Действителен с',  'validFromDate', 15, canBeEmpty=True))
        self.addCol(CDateInDocTableCol(u'Действителен по',  'validToDate', 15, canBeEmpty=True))


class COrderDocsModel(CInDocTableModel):
    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'Person_Order', 'id', 'master_id', parent)
        self.setFilter('documentType_id IN (SELECT rbDocumentType.id FROM rbDocumentType LEFT JOIN rbDocumentTypeGroup ON rbDocumentTypeGroup.id = rbDocumentType.group_id WHERE rbDocumentTypeGroup.code=\'4\')')
        self.addCol(CDateInDocTableCol(u'Дата',  'date', 15))
        self.addCol(CEnumInDocTableCol(u'Тип перемещения',  'type',  15,
            [u'Приём на работу', u'Увольнение', u'Назначение на должность',
             u'Отпуск', u'Учёба', u'Командировка'])).setSortable(True)
        self.addCol(CDateInDocTableCol(u'Дата',  'documentDate', 15))
        self.addCol(CInDocTableCol(u'Номер', 'documentNumber', 8))
        self.addCol(CRBInDocTableCol(u'Тип документа', 'documentType_id', 30, 'rbDocumentType', filter='group_id IN (SELECT id FROM rbDocumentTypeGroup WHERE code=\'4\')'))
        self.addCol(CDateInDocTableCol(u'Действителен с',  'validFromDate', 15, canBeEmpty=True))
        self.addCol(CDateInDocTableCol(u'Действителен по',  'validToDate', 15, canBeEmpty=True))
        self.addCol(CRBInDocTableCol(u'Должность', 'post_id', 30, 'rbPost'))
        self.addCol(COrgStructureInDocTableCol(u'Подразделение',  'orgStructure_id',  15,  'OrgStructure'))
        self.addCol(CInDocTableCol(u'Ставка', 'salary', 15))


class CPersonActivityModel(CInDocTableModel):
    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'Person_Activity', 'id', 'master_id', parent)
        self.addCol(CRBInDocTableCol(u'Вид деятельности', 'activity_id', 30, 'rbActivity'))


class CPersonJobTypeModel(CInDocTableModel):
    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'Person_JobType', 'id', 'master_id', parent)
        self.addCol(CRBInDocTableCol(u'Вид деятельности', 'jobType_id', 30, 'rbJobType'))


#FIXME: modeldatacache - отсутствует.
class COrgStructureInDocTableCol(CInDocTableCol):
    def __init__(self, title, fieldName, width, tableName, **params):
        CInDocTableCol.__init__(self, title, fieldName, width, **params)
        self.tableName  = tableName
        self.filter     = params.get('filter', '')
        self.preferredWidth = params.get('preferredWidth', None)


    def toString(self, val, record):
        from Orgs.Utils import getOrgStructureName
        return toVariant(getOrgStructureName(val))


    def toStatusTip(self, val, record):
        from Orgs.Utils import getOrgStructureFullName
        return toVariant(getOrgStructureFullName(val))


    def createEditor(self, parent):
        from Orgs.OrgStructComboBoxes import COrgStructureComboBox
        editor = COrgStructureComboBox(parent)
#        editor.setPreferredWidth(self.preferredWidth)
        return editor


    def setEditorData(self, editor, value, record):
        editor.setValue(forceInt(value))


    def getEditorData(self, editor):
        return toVariant(editor.value())


class CPersonContactsModel(CInDocTableModel):
    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'Person_Contact', 'id', 'master_id', parent)
        self.addCol(CRBInDocTableCol(u'Тип', 'contactType_id', 30, 'rbContactType', addNone=False))
        self.addCol(CInDocTableCol(u'Номер', 'contact', 30))
        self.addCol(CInDocTableCol(u'Примечание', 'notes', 30))


# #########################3


def getPersonAddress(id, addrType):
    return selectLatestRecord('Person_Address', id, 'type=\'%d\'' % addrType)


def getPersonDocument(id):
    filter = '''Tmp.documentType_id IN (SELECT rbDocumentType.id FROM rbDocumentType LEFT JOIN rbDocumentTypeGroup ON rbDocumentTypeGroup.id=rbDocumentType.group_id WHERE rbDocumentTypeGroup.code = '1')'''
    return selectLatestRecord('Person_Document', id, filter)


def selectLatestRecordStmt(tableName, personId, filter='' ):
    if type(tableName) == tuple:
        tableName1, tableName2 = tableName
    else:
        tableName1 = tableName
        tableName2 = tableName
    pos = tableName2.find('AS Tmp')
    if pos < 0:
        tableName2 = tableName2 + ' AS Tmp'

    if filter:
        filter = ' AND ('+filter+')'
    return u'SELECT * FROM %s AS Main WHERE Main.master_id = \'%d\' AND Main.id = (SELECT MAX(Tmp.id) FROM %s WHERE Tmp.master_id =\'%d\' %s)' % (tableName1, personId, tableName2, personId, filter)


def selectLatestRecord(tableName, personId, filter=''):
    stmt = selectLatestRecordStmt(tableName, personId, filter)
    query = QtGui.qApp.db.query(stmt)
    if query.next():
        return query.record()
    else:
        return None
