# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui
from PyQt4.QtCore import SIGNAL

from library.AgeSelector     import parseAgeSelector, composeAgeSelector
from library.interchange     import ( setCheckBoxValue,
                                      getComboBoxValue,
                                      setComboBoxValue,
                                      getCheckBoxValue,
                                      setRBComboBoxValue,
                                      getRBComboBoxValue,
                                    )
from library.ItemsListDialog import CItemsListDialog, CItemEditorDialog
from library.TableModel      import CTextCol
from library.Utils           import forceString, forceStringEx, toVariant
from library.PrintInfo import CRBInfo

from RefBooks.Tables         import rbAppointmentPurpose, rbCode, rbName


from RefBooks.Ui_RBAppointmentPurposeEditor import Ui_RBAppointmentPurposeEditorDialog

#

class CRBAppointmentPurposeList(CItemsListDialog):
    def __init__(self, parent):
        CItemsListDialog.__init__(self, parent, [
            CTextCol(u'Код',          [rbCode], 20),
            CTextCol(u'Наименование', [rbName], 40),
            ], rbAppointmentPurpose, [rbCode, rbName])
        self.setWindowTitleEx(u'Назначения приёма')
        self.actDuplicate = QtGui.QAction(u'Дублировать', self)
        self.actDuplicate.setObjectName('actDuplicate')
        self.connect(self.actDuplicate, SIGNAL('triggered()'), self.duplicateCurrentRow)
        self.tblItems.createPopupMenu([self.actDuplicate])


    def getItemEditor(self):
        return CRBAppointmentPurposeEditor(self)


#    def duplicateCurrentRow(self):
#        def duplicateCurrentInternal():
#            currentItemId = self.currentItemId()
#            if currentItemId:
#                db = QtGui.qApp.db
#                table = db.table('rbAppointmentPurpose')
#                db.transaction()
#                try:
#                    record = db.getRecord(table, '*', currentItemId)
#                    record.setValue('code', toVariant(forceString(record.value('code'))+'_1'))
#                    record.setValue('name', toVariant(forceString(record.value('name'))+'_1'))
#                    record.setNull('id')
#                    newItemId = db.insertRecord(table, record)
#                    db.commit()
#                except:
#                    db.rollback()
#                    QtGui.qApp.logCurrentException()
#                    raise
#                self.renewListAndSetTo(newItemId)
#        QtGui.qApp.call(self, duplicateCurrentInternal)


class CRBAppointmentPurposeEditor(Ui_RBAppointmentPurposeEditorDialog, CItemEditorDialog):
    def __init__(self,  parent):
        CItemEditorDialog.__init__(self, parent, rbAppointmentPurpose)
        self.setWindowTitleEx(u'Назначение приёма')
        self.cmbMedicalAidProfile.setTable('rbMedicalAidProfile', True)
        self.cmbMedicalAidProfile.setCurrentIndex(0)
        self.setupDirtyCather()


    def setRecord(self, record):
        CItemEditorDialog.setRecord(self, record)
        setComboBoxValue(   self.cmbSex,            record, 'sex')
        (begUnit, begCount, endUnit, endCount) = parseAgeSelector(forceString(record.value('age')))
        self.cmbBegAgeUnit.setCurrentIndex(begUnit)
        self.edtBegAgeCount.setText(str(begCount))
        self.cmbEndAgeUnit.setCurrentIndex(endUnit)
        self.edtEndAgeCount.setText(str(endCount))
        setCheckBoxValue(self.chkEnablePrimaryRecord,       record, 'enablePrimaryRecord')
        setCheckBoxValue(self.chkEnableOwnRecord,           record, 'enableOwnRecord')
        setCheckBoxValue(self.chkEnableConsultancyRecord,   record, 'enableConsultancyRecord')
        setCheckBoxValue(self.chkEnableRecordViaInfomat,    record, 'enableRecordViaInfomat')
        setCheckBoxValue(self.chkEnableRecordViaCallCenter, record, 'enableRecordViaCallCenter')
        setCheckBoxValue(self.chkEnableRecordViaInternet,   record, 'enableRecordViaInternet')
        setCheckBoxValue(self.chkRequireReferral,           record, 'requireReferral')
        setCheckBoxValue(self.chkForExternalSchedule,           record, 'forExternalSchedule')
        setRBComboBoxValue(self.cmbMedicalAidProfile,       record, 'medicalAidProfile_id')


    def getRecord(self):
        record = CItemEditorDialog.getRecord(self)
        getComboBoxValue(   self.cmbSex,            record, 'sex')
        record.setValue('age',        toVariant(composeAgeSelector(
                    self.cmbBegAgeUnit.currentIndex(),  forceStringEx(self.edtBegAgeCount.text()),
                    self.cmbEndAgeUnit.currentIndex(),  forceStringEx(self.edtEndAgeCount.text())
                        )))
        getCheckBoxValue(self.chkEnablePrimaryRecord,       record, 'enablePrimaryRecord')
        getCheckBoxValue(self.chkEnableOwnRecord,           record, 'enableOwnRecord')
        getCheckBoxValue(self.chkEnableConsultancyRecord,   record, 'enableConsultancyRecord')
        getCheckBoxValue(self.chkEnableRecordViaInfomat,    record, 'enableRecordViaInfomat')
        getCheckBoxValue(self.chkEnableRecordViaCallCenter, record, 'enableRecordViaCallCenter')
        getCheckBoxValue(self.chkEnableRecordViaInternet,   record, 'enableRecordViaInternet')
        getCheckBoxValue(self.chkRequireReferral,           record, 'requireReferral')
        getCheckBoxValue(self.chkForExternalSchedule,           record, 'forExternalSchedule')
        getRBComboBoxValue(self.cmbMedicalAidProfile,       record, 'medicalAidProfile_id')
        return record


class CAppointmentPurposeInfo(CRBInfo):
    tableName = 'rbAppointmentPurpose'
