# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui
from PyQt4.QtCore import Qt, QVariant, SIGNAL

from library.AgeSelector     import parseAgeSelector, composeAgeSelector
from library.InDocTable      import CInDocTableModel, CEnumInDocTableCol, CRBInDocTableCol
from library.interchange     import (
    setLineEditValue, setComboBoxValue, setSpinBoxValue,
    getLineEditValue, getComboBoxValue, getSpinBoxValue
)
from library.ItemsListDialog import CItemsListDialog, CItemEditorBaseDialog
from library.TableModel      import CTextCol
from library.Utils           import forceBool, forceInt, forceString, forceStringEx, toVariant


from Orgs.OrgStructureCol    import COrgStructureInDocTableCol
from RefBooks.Tables         import rbCode, rbName, rbJobPurpose

from RefBooks.Ui_RBJobPurposeEditor import Ui_RBJobPurposeEditorDialog


class CRBJobPurposeList(CItemsListDialog):
    def __init__(self, parent):
        CItemsListDialog.__init__(self, parent, [
            CTextCol(u'Код',          [rbCode], 20),
            CTextCol(u'Наименование', [rbName], 40),
            ], rbJobPurpose, [rbCode, rbName])
        self.setWindowTitleEx(u'Назначения работы')
        self.actDuplicate = QtGui.QAction(u'Дублировать', self)
        self.actDuplicate.setObjectName('actDuplicate')
        self.connect(self.actDuplicate, SIGNAL('triggered()'), self.duplicateCurrentRow)
        self.tblItems.createPopupMenu([self.actDuplicate])

    # для дублирования
    def copyInternals(self, newItemId, oldItemId):
        db = QtGui.qApp.db
        tablePractice = db.table('rbJobPurpose_Practice')
        records = db.getRecordList(
            tablePractice, '*',
            tablePractice['master_id'].eq(oldItemId))
        for record in records:
            record.setNull('id')
            record.setValue('master_id', toVariant(newItemId))
            db.insertRecord(tablePractice, record)


    def getItemEditor(self):
        return CRBJobPurposeEditor(self)



class CRBJobPurposeEditor(Ui_RBJobPurposeEditorDialog, CItemEditorBaseDialog):
    def __init__(self,  parent):
        CItemEditorBaseDialog.__init__(self, parent, rbJobPurpose)
        self.addModels('Practices',  CPracticesModel(self))
        self.setupUi(self)
        self.setWindowTitleEx(u'Назначение работы')
        self.setModels(self.tblPractices, self.modelPractices, self.selectionModelPractices)

        self.setupDirtyCather()
        self.tblPractices.addMoveRow()
        self.tblPractices.popupMenu().addSeparator()
        self.tblPractices.addPopupDelRow()


    def setRecord(self, record):
        CItemEditorBaseDialog.setRecord(self, record)
        setLineEditValue(   self.edtCode,           record, 'code')
        setLineEditValue(   self.edtName,           record, 'name')
        setComboBoxValue(   self.cmbSex,            record, 'sex')
        setSpinBoxValue(self.edtPrematurelyClosingThreshold, record, 'prematurelyClosingThreshold')
        (begUnit, begCount, endUnit, endCount) = parseAgeSelector(forceString(record.value('age')))
        self.cmbBegAgeUnit.setCurrentIndex(begUnit)
        self.edtBegAgeCount.setText(str(begCount))
        self.cmbEndAgeUnit.setCurrentIndex(endUnit)
        self.edtEndAgeCount.setText(str(endCount))
        self.modelPractices.loadItems(self.itemId())
#        self.setIsDirty(False)


    def getRecord(self):
        record = CItemEditorBaseDialog.getRecord(self)
        getLineEditValue(   self.edtCode,           record, 'code')
        getLineEditValue(   self.edtName,           record, 'name')
        getComboBoxValue(   self.cmbSex,            record, 'sex')
        getSpinBoxValue(self.edtPrematurelyClosingThreshold, record, 'prematurelyClosingThreshold')
        record.setValue('age',        toVariant(composeAgeSelector(
                    self.cmbBegAgeUnit.currentIndex(),  forceStringEx(self.edtBegAgeCount.text()),
                    self.cmbEndAgeUnit.currentIndex(),  forceStringEx(self.edtEndAgeCount.text())
                        )))
        return record


    def saveInternals(self, id):
        self.modelPractices.saveItems(id)



class CPracticesModel(CInDocTableModel):
    # В этой модели есть такое "украшение".
    # Записи rbJobPurpose_Practice используются
    # "по ИЛИ" если они находятся в разных группах
    # и по "И" если они находятся в одной группе.
    # Найдена возможность реализовать более привычную запись - с И и ИЛИ
    # этому посвящены поля junction и "волшебство" в flags/data/loadItems/...
    #
    # логика группировки скопирована в RefBooks/RBContractCoefficientType.py
    # будет использовата третий раз - нужно будет выносить в отдельный класс (миксин?)

    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'rbJobPurpose_Practice', 'id', 'master_id', parent)
        self.addExtCol(CEnumInDocTableCol(u'Связка', 'junction', 5, (u'или', u'и')),
                       QVariant.Int)
        self.addCol(CEnumInDocTableCol(u'Условие', 'excludeOrgStructure', 10,
                                       (u'∈', u'∉')
                                      ))
        self.addCol(COrgStructureInDocTableCol(u'Подразделение',  'orgStructure_id',  15))
        self.addCol(CEnumInDocTableCol(u'Условие', 'excludeEventType', 10,
                                       (u'=', u'≠')
                                      ))
        self.addCol(CRBInDocTableCol(u'Тип события', 'eventType_id', 20, 'EventType'))
        self.addCol(CEnumInDocTableCol(u'Доступность', 'avail', 35,
                                       (u'недоступно', u'доступно для ручного выбора', u'доступно для автоматического выбора')
                                      ))
        self.addHiddenCol('grouping')



    def _cellIsDisabled(self, index):
        row = index.row()
        column = index.column()
        if column == 0:
            if row == 0:
                return True
        elif column == 5: # avail
            if row+1 < len(self._items) and \
               forceBool(self._items[row+1].value('junction')): # True == «И»
                return True
        return False


    def flags(self, index):
        if self._cellIsDisabled(index):
            return Qt.ItemIsSelectable
        return CInDocTableModel.flags(self, index)


    def data(self, index, role=Qt.DisplayRole):
        if role == Qt.DisplayRole:
            if self._cellIsDisabled(index):
                return QVariant()
        return CInDocTableModel.data(self, index, role)


    def setData(self, index, value, role=Qt.EditRole):
        result = CInDocTableModel.setData(self, index, value, role)
        if result:
            column = index.column()
            if column == 0: # junction
                self.emitColumnChanged(5) # avail
            elif column == 5: # avail
                row = index.row()
                while row>0:
                    if self._items[row].value('junction'): # True == «И»
                        self._items[row-1].setValue('avail', value)
                    row -= 1
        return result


    def loadItems(self, masterId):
        CInDocTableModel.loadItems(self, masterId)
        prevGrouping = -1
        for item in self._items:
            grouping = forceInt(item.value('grouping'))
            item.setValue('junction', grouping == prevGrouping) # True == «И»
            prevGrouping = grouping


    def saveItems(self, masterId):
        if self._items:
            grouping = -1
            for item in self._items:
                junction = forceBool(item.value('junction')) # True == «И»
                if grouping == -1 or not junction:
                    grouping = grouping+1
                item.setValue('grouping', grouping)

            grouping = -1
            for item in reversed(self._items):
                currGrouping = forceInt(item.value('grouping'))
                if grouping != currGrouping:
                    avail = forceInt(item.value('avail'))
                    grouping = currGrouping
                else:
                    item.setValue('avail', avail)

        CInDocTableModel.saveItems(self, masterId)
