# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import sip

from PyQt4 import QtGui
from PyQt4.QtCore import Qt, QDate, QLocale, QVariant, pyqtSignature, SIGNAL

from library.DialogBase              import CDialogBase
from library.InDocTable              import CInDocTableModel, CRBInDocTableCol
from library.interchange             import getDateEditValue, getDbComboBoxTextValue, getDoubleBoxValue, getLineEditValue, getRBComboBoxValue, setDateEditValue, setDbComboBoxTextValue, setDoubleBoxValue, setLineEditValue, setRBComboBoxValue
from library.ItemsListDialog         import CItemEditorBaseDialog
from library.RBCheckTable            import CRBCheckTableModel
from library.TableModel              import CDateCol, CDesignationCol, CNumCol, CRefBookCol, CTableModel, CTextCol
from library.Utils                   import forceDouble, forceInt, forceRef, forceString, toVariant, trim
from library.PrintTemplates       import applyTemplate, getPrintButton
from library.PrintInfo           import CInfoContext

from RefBooks.RBVaccine              import CRBVaccineEditor
from Registry.Utils                  import getClientBanner, CClientInfo
from Users.Rights                    import urCanEditClientVaccination

from Ui_ClientVaccinationCardDialog  import Ui_ClientVaccinationCardDialog
from Ui_ClientVaccinationEditor      import Ui_ClientVaccinationEditor
from Ui_ClientMedicalExemptionEditor import Ui_ClientMedicalExemptionEditor

from ClientVaccinationInfo import CVaccinationInfoList, CMedicalExemptionInfoList


def openClientVaccinationCard(parent, clientId):
    result = None
    dialog = CClientVaccinationCard(parent, clientId)
    try:
        dialog.setReadOnly(not QtGui.qApp.userHasRight(urCanEditClientVaccination))
        result = dialog.exec_()
    finally:
        sip.delete(dialog)
        del dialog
    return result


# ###########################################################################


def openClientVaccination(parent, clientId, itemId):
    resultId = None
    dialog = CClientVaccinationEditor(parent, clientId)
    if itemId:
        try:
            dialog.load(itemId)
            dialog.setReadOnly(not QtGui.qApp.userHasRight(urCanEditClientVaccination))
            if dialog.exec_():
                resultId = dialog.itemId()
        finally:
            sip.delete(dialog)
            del dialog
    return resultId


def newClientVaccination(parent, clientId, params={}):
    resultId = None
    dialog = CClientVaccinationEditor(parent, clientId)
    try:
        dialog.setParams(params)
        if dialog.exec_():
            resultId = dialog.itemId()
    finally:
        sip.delete(dialog)
        del dialog
    return resultId


# ###########################################################################


def openClientMedicalExemption(parent, clientId, itemId):
    resultId = None
    if itemId:
        dialog = CClientMedicalExemptionEditor(parent, clientId)
        try:
            dialog.load(itemId)
            dialog.setReadOnly(not QtGui.qApp.userHasRight(urCanEditClientVaccination))
            if dialog.exec_():
                resultId = dialog.itemId()
        finally:
            sip.delete(dialog)
            del dialog
    return resultId


def newClientMedicalExemption(parent, clientId, params={}):
    resultId = None
    dialog = CClientMedicalExemptionEditor(parent, clientId)
    try:
        dialog.setParams(params)
        if dialog.exec_():
            resultId = dialog.itemId()
    finally:
        sip.delete(dialog)
        del dialog
    return resultId


# ###########################################################################


def openVaccineSchema(parent, vaccineId):
    if vaccineId:
        dialog = CRBVaccineEditor(parent)
        try:
            dialog.load(vaccineId)
            dialog.setReadOnly(True)
            dialog.exec_()
        finally:
            sip.delete(dialog)


# ###########################################################################


class CClientVaccinationCard(CDialogBase, Ui_ClientVaccinationCardDialog):
    def __init__(self, parent=None, clientId=None):
        CDialogBase.__init__(self, parent)

        # MODELS
        self.addModels('FilterInfection',            CRBCheckTableModel(self, 'rbInfection', u'Инфекции'))
        self.addModels('FilterVaccine',              CRBCheckTableModel(self, 'rbVaccine', u'Вакцины'))
        self.addModels('FilterProbe',                CRBCheckTableModel(self, 'rbVaccinationProbe', u'Пробы'))
        self.addModels('Vaccination',                CVaccinationTableModel(self))
        self.addModels('VaccineInfections',          CVaccineInfectionsTableModel(self))
        self.addModels('MedicalExemption',           CMedicalExemptionModel(self))
        self.addModels('MedicalExemptionInfections', CMedicalExemptionInfectionsModel(self))

        # MENUS
        # vaccination table menu
        self.addObject('actAddVaccination',    QtGui.QAction(u'Добавить', self))
        self.addObject('actEditVaccination',   QtGui.QAction(u'Редактировать', self))
        self.addObject('actDeleteVaccination', QtGui.QAction(u'Удалить', self))
        self.addObject('actSchema',            QtGui.QAction(u'Схема', self))

        self.actAddVaccination.setShortcut(Qt.Key_F9)
        self.addAction(self.actAddVaccination)

        self.actEditVaccination.setShortcut(Qt.Key_F4)
        self.addAction(self.actEditVaccination)

        # filter vaccine table menu
        self.addObject('actFilterVaccineSchema', QtGui.QAction(u'Схема', self))

        # medical exemption table menu
        self.addObject('actAddMedicalExemption',    QtGui.QAction(u'Добавить', self))
        self.addObject('actEditMedicalExemption',   QtGui.QAction(u'Редактировать', self))
        self.addObject('actDeleteMedicalExemption', QtGui.QAction(u'Удалить', self))

        self.setupUi(self)
        self.setWindowTitle(u'Прививочная карта пациента')
        self.setWindowFlags(self.windowFlags() | Qt.WindowMaximizeButtonHint)
        self.setWindowState(Qt.WindowMaximized)

        # TABLES
        self.setModels(self.tblFilterInfection,           self.modelFilterInfection,   self.selectionModelFilterInfection)
        self.setModels(self.tblFilterVaccine,             self.modelFilterVaccine,     self.selectionModelFilterVaccine)
        self.setModels(self.tblFilterProbe,               self.modelFilterProbe,       self.selectionModelFilterProbe)
        self.setModels(self.tblVaccination,               self.modelVaccination,       self.selectionModelVaccination)
        self.setModels(self.tblVaccinationLeftOverSchema, self.modelVaccineInfections, self.selectionModelVaccineInfections) #временно
        self.setModels(self.tblMedicalExemption, self.modelMedicalExemption, self.selectionModelMedicalExemption)
        self.setModels(self.tblMedicalExemptionInfections,
                       self.modelMedicalExemptionInfections,
                       self.selectionModelMedicalExemptionInfections)

        # MENUS
        self.tblVaccination.createPopupMenu([self.actAddVaccination, self.actDeleteVaccination,
                                             self.actSchema, self.actEditVaccination])

        self.tblFilterVaccine.addPopupAction(self.actFilterVaccineSchema)

        # medical exemption table menu
        self.tblMedicalExemption.createPopupMenu([self.actAddMedicalExemption, self.actEditMedicalExemption, self.actDeleteMedicalExemption])

        # view popup show
        self.connect(self.tblVaccination.popupMenu(), SIGNAL('aboutToShow()'), self.on_tblVaccinationPopup_aboutToShow)

        if clientId:
            self.setClientId(clientId)
        self.__setVisibleProbeAndVaccine(False, True)
        self.cmbFilterVaccinationCalendar.setTable('rbVaccinationCalendar')

        self.updateMedicalExemptionList()
        self.updateVaccinationList()
        self.addObject('btnPrint', getPrintButton(self, 'vaccinationCard'))
        self.buttonBox.addButton(self.btnPrint, QtGui.QDialogButtonBox.ActionRole)
        self.connect(self.btnPrint, SIGNAL('printByTemplate(int)'), self.on_btnPrint_printByTemplate)


    def setReadOnly(self, value=True):
        CDialogBase.setReadOnly(self, value)
        canEdit = not self.isReadOnly()
        self.actAddVaccination.setEnabled(canEdit)
        self.actAddMedicalExemption.setEnabled(canEdit)
        self.actEditMedicalExemption.setEnabled(canEdit)
        self.actDeleteMedicalExemption.setEnabled(canEdit)
        buttons = QtGui.QDialogButtonBox.Ok | QtGui.QDialogButtonBox.Cancel if canEdit else QtGui.QDialogButtonBox.Close
        self.buttonBox.setStandardButtons(buttons)


    def __setVisibleProbeAndVaccine(self, probe=False, vaccine=True):
        self.tblFilterVaccine.setVisible(vaccine)
        self.tblFilterProbe.setVisible(probe)


    def setClientId(self, clientId):
        self._clientId = clientId
        if clientId:
            self.txtClientInfoBrowser.setHtml(getClientBanner(clientId))
        else:
            self.txtClientInfoBrowser.setText('')


    def updateVaccinationList(self, id=None):
        db = QtGui.qApp.db

        table = db.table('ClientVaccination')

        cond = [table['client_id'].eq(self._clientId)]

        vaccineIdList = self.modelFilterVaccine.getCheckedIdList()
        if vaccineIdList:
            cond.append(table['vaccine_id'].inlist(vaccineIdList))
        else:
            vaccineIdList = self.modelFilterVaccine.idList()
            if vaccineIdList:
                cond.append(table['vaccine_id'].inlist(vaccineIdList))

        idList = db.getIdList(table, 'id', cond, 'datetime, id')

        self.tblVaccination.setIdList(idList, id)

        self.updateVaccinationInfectionList(idList)


    def updateVaccinationInfectionList(self, vaccinationIdList):
        db = QtGui.qApp.db
        tableClientVaccination = db.table('ClientVaccination')
        vaccineIdList = db.getDistinctIdList(tableClientVaccination, 'vaccine_id',
                                             tableClientVaccination['id'].inlist(vaccinationIdList))
        tableInfectionVaccine = db.table('rbInfection_rbVaccine')
        idList = QtGui.qApp.db.getDistinctIdList(tableInfectionVaccine, 'infection_id',
                                                 tableInfectionVaccine['vaccine_id'].inlist(vaccineIdList))
        self.modelVaccineInfections.setIdList(idList)


    def updateMedicalExemptionList(self, id=None):
        db = QtGui.qApp.db
        table = db.table('ClientMedicalExemption')
        cond = [table['client_id'].eq(self._clientId)]
        idList = db.getIdList(table, 'id', cond, 'date')
        self.tblMedicalExemption.setIdList(idList, id)


    def updateMedicalExemptionInfectionsList(self, medicalExemptionId):
        db = QtGui.qApp.db
        table = db.table('ClientMedicalExemption_Infection')
        cond = [table['master_id'].eq(medicalExemptionId)]
        idList = db.getIdList(table, 'id', cond)
        self.tblMedicalExemptionInfections.setIdList(idList)


    def _params(self):
        params = {}
        vaccinationCalendarId = self.cmbFilterVaccinationCalendar.value()
        if vaccinationCalendarId:
            params['vaccinationCalendarId'] = vaccinationCalendarId
        infectionIdList = self.modelFilterInfection.getCheckedIdList()
        if infectionIdList:
            params['infectionIdList'] = infectionIdList
        vaccineIdList = self.modelFilterVaccine.getCheckedIdList()
        if vaccineIdList:
            params['vaccineIdList'] = vaccineIdList
        else:
            params['vaccineIdList'] = self.modelFilterVaccine.idList()
        personId = QtGui.qApp.userId
        if personId:
            params['personId'] = personId
        return params

    @pyqtSignature('int')
    def on_btnPrint_printByTemplate(self, templateId):
        context = CInfoContext()
        vaccineList = context.getInstance(CVaccinationInfoList, self._clientId)
        medicalExemptionList = context.getInstance(CMedicalExemptionInfoList, self._clientId)
        clientInfo = context.getInstance(CClientInfo, self._clientId)
        data = {'vaccineList' : vaccineList,
                    'medicalExemptionList' : medicalExemptionList,
                    'client' : clientInfo
               }
        applyTemplate(self, templateId, data)

    @pyqtSignature('int')
    def on_tabWidget_currentChanged(self, index):
        if index >= 0:
            if index == 0:
                self.__setVisibleProbeAndVaccine(False, True)
            elif index == 1:
                self.__setVisibleProbeAndVaccine(True, False)


    @pyqtSignature('int')
    def on_cmbFilterVaccinationCalendar_currentIndexChanged(self, index):
        vaccinationCalendarId = self.cmbFilterVaccinationCalendar.value()
        if vaccinationCalendarId:
            db = QtGui.qApp.db
            idList = db.getIdList('rbVaccinationCalendar_Infection', 'infection_id', 'master_id=%d'%vaccinationCalendarId)
            tableInfection = db.table('rbInfection')
            self.modelFilterInfection.setFilter(tableInfection['id'].inlist(idList))
        else:
            self.modelFilterInfection.setFilter('')


    @pyqtSignature('QModelIndex')
    def on_tblVaccination_doubleClicked(self, index):
        self.on_actEditVaccination_triggered()



    @pyqtSignature('')
    def on_modelFilterInfection_itemCheckingChanged(self):
        infectionIdList = self.modelFilterInfection.getCheckedIdList()
        if infectionIdList:
            db = QtGui.qApp.db
            tableInfectionVaccine = db.table('rbInfection_rbVaccine')
            tableVaccine = db.table('rbVaccine')
            cond = tableInfectionVaccine['infection_id'].inlist(infectionIdList)
            idList = db.getIdList(tableInfectionVaccine, 'vaccine_id', cond)
            self.modelFilterVaccine.setFilter(tableVaccine['id'].inlist(idList))
        else:
            self.modelFilterVaccine.setFilter('')

        self.updateVaccinationList()


    def on_modelFilterVaccine_itemCheckingChanged(self):
        self.updateVaccinationList()


    @pyqtSignature('')
    def on_tblVaccinationPopup_aboutToShow(self):
        isVaccinationIndexValid = self.tblVaccination.currentIndex().isValid()
        canEdit = not self.isReadOnly()
        self.actEditVaccination.setEnabled(isVaccinationIndexValid and canEdit)
        self.actDeleteVaccination.setEnabled(isVaccinationIndexValid and canEdit)
        self.actSchema.setEnabled(isVaccinationIndexValid)


    @pyqtSignature('')
    def on_actAddVaccination_triggered(self):
        vaccinationId = newClientVaccination(self, self._clientId, self._params())
        if vaccinationId:
            self.updateVaccinationList(vaccinationId)


    @pyqtSignature('')
    def on_actEditVaccination_triggered(self):
        vaccinationId = self.tblVaccination.currentItemId()
        if vaccinationId:
            vaccinationId = openClientVaccination(self, self._clientId, vaccinationId)
            self.updateVaccinationList(vaccinationId)


    @pyqtSignature('')
    def on_actDeleteVaccination_triggered(self):
        index = self.tblVaccination.currentIndex()
        row = index.row()
        if index.isValid() and self.modelVaccination.confirmRemoveRow(self.tblVaccination, row):
            self.modelVaccination.removeRow(row)


    @pyqtSignature('')
    def on_actSchema_triggered(self):
        vaccinationItem = self.tblVaccination.currentItem()
        openVaccineSchema(self, forceRef(vaccinationItem.value('vaccine_id')))


    @pyqtSignature('')
    def on_actFilterVaccineSchema_triggered(self):
        vaccineId = self.tblFilterVaccine.currentItemId()
        openVaccineSchema(self, vaccineId)


    @pyqtSignature('')
    def on_actAddMedicalExemption_triggered(self):
        medicalExemptionId = newClientMedicalExemption(self, self._clientId)
        if medicalExemptionId:
            self.updateMedicalExemptionList(medicalExemptionId)


    @pyqtSignature('')
    def on_actEditMedicalExemption_triggered(self):
        itemId = self.tblMedicalExemption.currentItemId()
        medicalExemptionId = openClientMedicalExemption(self, self._clientId, itemId)
        if medicalExemptionId:
            self.updateMedicalExemptionList(medicalExemptionId)


    @pyqtSignature('')
    def on_actDeleteMedicalExemption_triggered(self):
        index = self.tblMedicalExemption.currentIndex()
        row = index.row()
        if index.isValid() and self.modelMedicalExemption.confirmRemoveRow(self.tblMedicalExemption, row):
            self.modelMedicalExemption.removeRow(row)


    @pyqtSignature('int, int')
    def on_tabVaccinationSplitter_splitterMoved(self, pos, index):
        self.tabProbeSplitter.blockSignals(True)
        self.tabMedicalExemptionSplitter.blockSignals(True)
        self.tabProbeSplitter.moveSplitter(pos, index)
        self.tabMedicalExemptionSplitter.moveSplitter(pos, index)
        self.tabProbeSplitter.blockSignals(False)
        self.tabMedicalExemptionSplitter.blockSignals(False)


    @pyqtSignature('int, int')
    def on_tabProbeSplitter_splitterMoved(self, pos, index):
        self.tabVaccinationSplitter.blockSignals(True)
        self.tabMedicalExemptionSplitter.blockSignals(True)
        self.tabVaccinationSplitter.moveSplitter(pos, index)
        self.tabMedicalExemptionSplitter.moveSplitter(pos, index)
        self.tabVaccinationSplitter.blockSignals(False)
        self.tabMedicalExemptionSplitter.blockSignals(False)


    @pyqtSignature('int, int')
    def on_tabMedicalExemptionSplitter_splitterMoved(self, pos, index):
        self.tabVaccinationSplitter.blockSignals(True)
        self.tabProbeSplitter.blockSignals(True)
        self.tabProbeSplitter.moveSplitter(pos, index)
        self.tabVaccinationSplitter.moveSplitter(pos, index)
        self.tabVaccinationSplitter.blockSignals(False)
        self.tabProbeSplitter.blockSignals(False)


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelVaccination_currentChanged(self, current, previous):
        item = self.tblVaccination.currentItem()
        if item:
            vaccineId = forceInt(item.value('vaccine_id'))
            idList = QtGui.qApp.db.getIdList('rbInfection_rbVaccine', 'infection_id', 'vaccine_id=%d'%vaccineId)
            self.modelVaccineInfections.setCurrntInfectionIdList(idList)
        else:
            self.modelVaccineInfections.setCurrntInfectionIdList([])


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelMedicalExemption_currentChanged(self, current, previous):
        itemId = self.tblMedicalExemption.currentItemId()
        self.updateMedicalExemptionInfectionsList(itemId)


# ###########################################################################


class CClientVaccinationEditor(CItemEditorBaseDialog, Ui_ClientVaccinationEditor):
    def __init__(self, parent, clientId):
        CItemEditorBaseDialog.__init__(self, parent, 'ClientVaccination')

        self.setupUi(self)

        self.cmbVaccine.setTable('rbVaccine', needCache=False)
        self.cmbReaction.setTable('rbReaction')
        self.cmbTransitionType.setTable('rbVaccine_SchemaTransitionType')
        self._clientId = clientId

        self.setWindowTitleEx(u'Прививка')


    def setParams(self, params):
#        vaccinationCalendarId = params.get('vaccinationCalendarId', None)
#        infectionIdList       = params.get('infectionIdList', None)
        vaccineIdList         = params.get('vaccineIdList', None)
#        personId              = params.get('personId', None)

        if vaccineIdList:
            self.cmbVaccine.setFilter(QtGui.qApp.db.table('rbVaccine')['id'].inlist(vaccineIdList))
            self.cmbVaccine.setValue(vaccineIdList[0])

#        if personId:
#            self.cmbPerson.setValue(personId)



    def setRecord(self, record):
        CItemEditorBaseDialog.setRecord(self, record)
        self._clientId = forceRef(record.value('client_id'))
        setRBComboBoxValue(    self.cmbVaccine,         record, 'vaccine_id')
        setDbComboBoxTextValue(self.cmbVaccinationType, record, 'vaccinationType')
        setDateEditValue(      self.edtDate,            record, 'datetime')
        setDoubleBoxValue(     self.edtDose,            record, 'dose')
        setLineEditValue(      self.edtSeria,           record, 'seria')
        setRBComboBoxValue(    self.cmbPerson,          record, 'person_id')
        setRBComboBoxValue(    self.cmbReaction,        record, 'reaction_id')
        setRBComboBoxValue(    self.cmbTransitionType,  record, 'transitionType_id')
        setRBComboBoxValue(    self.cmbRelegateOrg,     record, 'relegateOrg_id')


    def getRecord(self):
        record = CItemEditorBaseDialog.getRecord(self)
        record.setValue('client_id', QVariant(self._clientId))
        getRBComboBoxValue(    self.cmbVaccine,         record, 'vaccine_id')
        getDbComboBoxTextValue(self.cmbVaccinationType, record, 'vaccinationType')
        getDateEditValue(      self.edtDate,            record, 'datetime')
        getDoubleBoxValue(     self.edtDose,            record, 'dose')
        getLineEditValue(      self.edtSeria,           record, 'seria')
        getRBComboBoxValue(    self.cmbPerson,          record, 'person_id')
        getRBComboBoxValue(    self.cmbReaction,        record, 'reaction_id')
        getRBComboBoxValue(    self.cmbTransitionType,  record, 'transitionType_id')
        getRBComboBoxValue(    self.cmbRelegateOrg,     record, 'relegateOrg_id')
        return record


    def checkDataEntered(self):
        result = True

        vaccineId = self.cmbVaccine.value()
        result = result and (vaccineId or self.checkInputMessage(u'вакцина', False, self.cmbVaccine))

        vaccinationType = trim(self.cmbVaccinationType.text())
        result = result and (vaccinationType or self.checkInputMessage(u'тип прививки', False, self.cmbVaccinationType))

        date = self.edtDate.date()
        result = result and (date or self.checkInputMessage(u'дата', False, self.edtDate))

        dose = self.edtDose.value()
        result = result and (dose or self.checkInputMessage(u'доза', True, self.edtDose))

        seria = trim(self.edtSeria.text())
        result = result and (seria or self.checkInputMessage(u'серия', False, self.edtSeria))

        personId = self.cmbPerson.value()
        result = result and (personId or self.checkInputMessage(u'врач', False, self.cmbPerson))

        reactionId = self.cmbReaction.value()
        result = result and (reactionId or self.checkInputMessage(u'реакция', True, self.cmbReaction))

        transitionTypeId = self.cmbTransitionType.value()
        result = result and (transitionTypeId or self.checkInputMessage(u'дополнительно', True, self.cmbTransitionType))

        relegateOrgId = self.cmbRelegateOrg.value()
        result = result and (relegateOrgId or self.checkInputMessage(u'направитель', True, self.cmbRelegateOrg))

        return result


    def setDependentsByVaccineId(self, vaccineId):
        db = QtGui.qApp.db

        tableClienVaccination = db.table('ClientVaccination')
        tableCV = db.table('ClientVaccination').alias('CV')

        subCond = [tableCV['vaccine_id'].eq(vaccineId)]
        subStmt = db.selectStmt(tableCV, 'MAX('+tableCV['id'].name()+')', subCond)

        cond = [tableClienVaccination['id'].name()+'=('+subStmt+')']

        record = db.getRecordEx(tableClienVaccination,
                                [tableClienVaccination['seria'],
                                tableClienVaccination['person_id']],
                                cond)

        if record:
            seria = forceString(record.value('seria'))
            personId = forceRef(record.value('person_id'))

            self.edtSeria.setText(seria)
            self.cmbPerson.setValue(QtGui.qApp.userId if QtGui.qApp.userSpecialityId else personId)

        else:
            self.cmbPerson.setValue(QtGui.qApp.userId if QtGui.qApp.userSpecialityId else None)
            self.edtSeria.clear()


    @pyqtSignature('int')
    def on_cmbVaccine_currentIndexChanged(self, index):
        vaccineId = self.cmbVaccine.value()

        self.cmbVaccinationType.setVaccineId(vaccineId)
        self.cmbVaccinationType.setEnabled(bool(vaccineId))

        self.edtDose.setValue(forceDouble(QtGui.qApp.db.translate('rbVaccine',
                                                                  'id',
                                                                  vaccineId,
                                                                  'dose')) if vaccineId else 0)

        self.setDependentsByVaccineId(vaccineId)


# ###########################################################################


class CClientMedicalExemptionEditor(CItemEditorBaseDialog, Ui_ClientMedicalExemptionEditor):
    def __init__(self, parent, clientId):
        CItemEditorBaseDialog.__init__(self, parent, 'ClientMedicalExemption')
        self.addModels(u'MedicalExemptionInfections', CMedicalExemptionInfectionsInDocTableModel(self))
        self.setupUi(self)
        self._clientId = clientId
        self.cmbMedicalExemptionType.setTable('rbMedicalExemptionType', addNone=True)
        self.edtEndDate.setDate(QDate())
        self.setModels(self.tblMedicalExemptionInfections,
                       self.modelMedicalExemptionInfections,
                       self.selectionModelMedicalExemptionInfections)
        self.setWindowTitleEx(u'Медотвод')


    def setParams(self, params):
        pass


    def setRecord(self, record):
        CItemEditorBaseDialog.setRecord(self, record)
        self._clientId = forceRef(record.value('client_id'))
        setDateEditValue(  self.edtDate,                record, 'date')
        self.cmbMKB.setText(forceString(record.value('MKB')))
        setRBComboBoxValue(self.cmbPerson,               record, 'person_id')
        setDateEditValue(  self.edtEndDate,              record, 'endDate')
        setRBComboBoxValue(self.cmbMedicalExemptionType, record, 'medicalExemptionType_id')
        self.modelMedicalExemptionInfections.loadItems(self.itemId())


    def getRecord(self):
        record = CItemEditorBaseDialog.getRecord(self)
        record.setValue('client_id', QVariant(self._clientId))
        getDateEditValue(  self.edtDate,                record, 'date')
        record.setValue('MKB', QVariant(self.cmbMKB.text()))
        getRBComboBoxValue(self.cmbPerson,               record, 'person_id')
        getDateEditValue(  self.edtEndDate,              record, 'endDate')
        getRBComboBoxValue(self.cmbMedicalExemptionType, record, 'medicalExemptionType_id')
        return record


    def saveInternals(self, id):
        self.modelMedicalExemptionInfections.saveItems(id)


    def checkDataEntered(self):
        result = True
        date = self.edtDate.date()
        result = result and (date or self.checkInputMessage(u'дата', False, self.edtDate))
        result = result and (self.cmbPerson.value() or self.checkInputMessage(u'врач', True, self.cmbPerson))
        result = result and self.checkMedicalExemptionInfections()
        return result


    def checkMedicalExemptionInfections(self):
        for row, item in enumerate(self.modelMedicalExemptionInfections.items()):
            infectionId = forceRef(item.value('infection_id'))
            if not infectionId:
                return self.checkInputMessage(u'инфекцию', False, self.tblMedicalExemptionInfections, row, 0)
        return True


# ###########################################################################


class CVaccineInfectionsTableModel(CTableModel):
    def __init__(self, parent):
        cols = [CRefBookCol(u'Инфекция', ['id'], 'rbInfection', 12, showFields=2)]
        CTableModel.__init__(self, parent, cols=cols, tableName='rbInfection')

        self._currentInfectionIdList = []

        boldFont = QtGui.QFont()
        boldFont.setWeight(QtGui.QFont.Bold)
        self._qBoldFont = QVariant(boldFont)


    def setCurrntInfectionIdList(self, currentInfectionIdList):
        if self._currentInfectionIdList != currentInfectionIdList:
            self._currentInfectionIdList = currentInfectionIdList
            self.emitDataChanged()


    def data(self, index, role=Qt.DisplayRole):
        if role == Qt.FontRole:
            column = index.column()
            row = index.row()
            (col, values) = self.getRecordValues(column, row)
            if forceRef(values[0]) in self._currentInfectionIdList:
                return self._qBoldFont
        else:
            return CTableModel.data(self, index, role)



# ###########################################################################


class CVaccinationTableModel(CTableModel):
    class CLocDoseCol(CNumCol):

        locale = QLocale()

        def format(self, values):
            sum = forceDouble(values[0])
            return toVariant(self.locale.toString(sum, 'f', 3))


    def __init__(self, parent):

        cols = [CRefBookCol(u'Вакцина', ['vaccine_id'], 'rbVaccine', 12, showFields=2),
                CTextCol(u'Тип прививки', ['vaccinationType'], 10),
                CDateCol(u'Дата', ['datetime'], 8),
                CVaccinationTableModel.CLocDoseCol(u'Доза', ['dose'], 8),
                CTextCol(u'Серия', ['seria'], 10),
                CRefBookCol(u'Врач', ['person_id'], 'vrbPersonWithSpeciality', 10),
                CRefBookCol(u'Реакция', ['reaction_id'], 'rbReaction', 10),
                CRefBookCol(u'Дополнительно', ['transitionType_id'], 'rbVaccine_SchemaTransitionType', 10),
                CDesignationCol(u'Направитель', ['relegateOrg_id'], ('Organisation', 'shortName'), 10)]

        CTableModel.__init__(self, parent, cols=cols, tableName='ClientVaccination')


# ###########################################################################


class CMedicalExemptionModel(CTableModel):
    def __init__(self, parent):

        cols = [CDateCol(u'Дата', ['date'], 8),
                CTextCol(u'МКБ', ['MKB'], 8),
                CRefBookCol(u'Врач', ['person_id'], 'vrbPersonWithSpeciality', 10),
                CDateCol(u'Дата окончания', ['endDate'], 8),
                CRefBookCol(u'Тип медотвода', ['medicalExemptionType_id'], 'rbMedicalExemptionType', 10)
               ]

        CTableModel.__init__(self, parent, cols=cols, tableName='ClientMedicalExemption')


# ###########################################################################


class CMedicalExemptionInfectionsModel(CTableModel):
    def __init__(self, parent):

        cols = [CRefBookCol(u'Инфекция', ['infection_id'], 'rbInfection', 10, showFields=2)
               ]

        CTableModel.__init__(self, parent, cols=cols, tableName='ClientMedicalExemption_Infection')


# ###########################################################################


class CMedicalExemptionInfectionsInDocTableModel(CInDocTableModel):
    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'ClientMedicalExemption_Infection', 'id', 'master_id',   parent)
        self.addCol(CRBInDocTableCol(  u'Инфекция', 'infection_id', 20, 'rbInfection'))
