# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################


from PyQt4 import QtGui
from library.PrintInfo import CInfo, CInfoList, CDateTimeInfo, CRBInfo, CDateInfo
from library.Utils     import forceDate, forceDouble, forceRef, forceString
from Orgs.Utils        import COrgInfo
from Orgs.PersonInfo   import CPersonInfo

class CVaccinationInfoList(CInfoList):
    def __init__(self, context, clientId):
        CInfoList.__init__(self, context)
        self.clientId = clientId
        
    def _load(self):
        db = QtGui.qApp.db
        table = db.table('ClientVaccination')
        idList = db.getIdList(table, 'id', table['client_id'].eq(self.clientId), 'id')
        self._items = [ self.getInstance(CVaccinationInfo, id) for id in idList ]
        return True
        

class CVaccinationInfo(CInfo):
    def __init__(self, context, id):
        CInfo.__init__(self, context)
        self.id = id
        
    def _load(self):
        db = QtGui.qApp.db
        record = db.getRecord('ClientVaccination', '*', self.id)
        if record:
            self._vaccine = self.getInstance(CRBVaccineInfo, forceRef(record.value('vaccine_id')))
            self._vaccinationType = forceString(record.value('vaccinationType'))
            self._datetime = CDateTimeInfo(forceDate(record.value('datetime')))
            self._dose = forceDouble(record.value('dose'))
            self._serial = forceString(record.value('seria'))
            self._person = self.getInstance(CPersonInfo, forceRef(record.value('person_id')))
            self._reaction = self.getInstance(CRBReactionInfo, forceRef(record.value('reaction_id')))
            self._transitionType = self.getInstance(CRBTransitionTypeInfo, forceRef(record.value('transitionType_id')))
            self._relegateOrg = self.getInstance(COrgInfo, forceRef(record.value('relegateOrg_id')))
        else:
            self._vaccine = self.getInstance(CRBVaccineInfo, None)
            self._vaccinationType = ''
            self._datetime = CDateTimeInfo()
            self._dose = 0
            self._serial = ''
            self._person = self.getInstance(CPersonInfo, None)
            self._reaction = self.getInstance(CRBReactionInfo, None)
            self._transitionType = self.getInstance(CRBTransitionTypeInfo, None)
            self._relegateOrg = self.getInstance(COrgInfo, None)
    
    vaccine = property(lambda self: self.load()._vaccine)
    vaccinationType = property(lambda self: self.load()._vaccinationType)
    datetime = property(lambda self: self.load()._datetime)
    dose = property(lambda self: self.load()._dose)
    serial = property(lambda self: self.load()._serial)
    person = property(lambda self: self.load()._person)
    reaction = property(lambda self: self.load()._reaction)
    transitionType = property(lambda self: self.load()._transitionType)
    relegateOrg = property(lambda self: self.load()._relegateOrg)

class CMedicalExemptionInfoList(CInfoList):
    def __init__(self, context, clientId):
        CInfoList.__init__(self, context)
        self.clientId = clientId
        
    def _load(self):
        db = QtGui.qApp.db
        table = db.table('ClientMedicalExemption')
        idList = db.getIdList(table, 'id', table['client_id'].eq(self.clientId), 'id')
        self._items = [ self.getInstance(CMedicalExemptionInfo, id) for id in idList ]
        return True
        
class CMedicalExemptionInfo(CInfo):
    def __init__(self, context, id):
        CInfo.__init__(self, context)
        self.id = id
        
    def _load(self):
        db = QtGui.qApp.db
        record = db.getRecord('ClientMedicalExemption', '*', self.id)
        if record:
            self._date = CDateInfo(forceDate(record.value('date')))
            self._MKB = forceString(record.value('MKB'))
            self._person = self.getInstance(CPersonInfo, forceRef(record.value('person_id')))
            self._endDate = CDateInfo(forceDate(record.value('endDate')))
            self._medicalExemptionType = self.getInstance(CRBMedicalExemptionTypeInfo, forceRef(record.value('medicalExemptionType_id')))
            self._infections = self.getInstance(CInfectionInfoList, forceRef(record.value('id')))
        else:
            self._date = CDateInfo()
            self._MKB = ''
            self._person = self.getInstance(CPersonInfo, None)
            self._endDate = CDateInfo()
            self._medicalExemptionType = self.getInstance(CRBMedicalExemptionTypeInfo, None)
            self._infections = self.getInstance(CInfectionInfoList, None)
    
    date = property(lambda self: self.load()._date)
    MKB = property(lambda self: self.load()._MKB)
    person = property(lambda self: self.load()._person)
    endDate = property(lambda self: self.load()._endDate)
    medicalExemptionType = property(lambda self: self.load()._medicalExemptionType)
    infections = property(lambda self: self.load()._infections)

class CRBVaccineInfo(CRBInfo):
    tableName = 'rbVaccine'
    
class CRBReactionInfo(CRBInfo):
    tableName = 'rbReaction'
    
class CRBTransitionTypeInfo(CRBInfo):
    tableName = 'rbVaccine_SchemaTransitionType'
    
class CRBMedicalExemptionTypeInfo(CRBInfo):
    tableName = 'rbMedicalExemptionType'

class CRBInfectionInfo(CRBInfo):
    tableName = 'rbInfection'
    
    
class CInfectionInfoList(CInfoList):
    def __init__(self, context, masterId):
        CInfoList.__init__(self, context)
        self.masterId = masterId

    def _load(self):
        db = QtGui.qApp.db
        table = db.table('ClientMedicalExemption_Infection')
        idList = db.getIdList(table, 'id', table['master_id'].eq(self.masterId), 'id')
        self._items = [ self.getInstance(CInfectionInfo, id) for id in idList ]
        return True
    
class CInfectionInfo(CInfo):
    def __init__(self, context, id):
        CInfo.__init__(self, context)
        self.id = id

    def _load(self):
        db = QtGui.qApp.db
        record = db.getRecord('ClientMedicalExemption_Infection', '*', self.id)
        if record:
            self._infection = self.getInstance(CRBInfectionInfo, forceRef(record.value('id')))
        else:
            self._infection = self.getInstance(CRBInfectionInfo, None)
    
    infection = property(lambda self: self.load()._infection)
