# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import sip
from PyQt4 import QtGui
from PyQt4.QtCore import Qt, pyqtSignature, QAbstractTableModel, QDate, QDateTime, QVariant

from library.DialogBase        import CDialogBase
from library.Utils             import forceRef, forceString, toVariant, formatShortName
from Registry.AmbCardMixin     import CAmbCardMixin


from Ui_RelationsClientListDialog import Ui_RelationsClientListDialog


class CRelationsClientListDialog(CDialogBase, CAmbCardMixin, Ui_RelationsClientListDialog):
    def __init__(self, parent, clientId):
        CDialogBase.__init__(self, parent)
        self.addModels('RelationsClientList', CRelationsClientListModel(self))
        self.addObject('mnuRelationsClientList', QtGui.QMenu(self))
        self.addObject('actEditClient', QtGui.QAction(u'Открыть регистрационную карточку', self))
        self.addObject('actMoveRegistry', QtGui.QAction(u'Перейти в картотеку', self))
        self.mnuRelationsClientList.addAction(self.actEditClient)
        self.mnuRelationsClientList.addAction(self.actMoveRegistry)
        self.setupUi(self)
        self.setWindowFlags(self.windowFlags() | Qt.WindowMaximizeButtonHint)
        self.setModels(self.tblRelationsClientList,  self.modelRelationsClientList, self.selectionModelRelationsClientList)
        self.tblRelationsClientList.setPopupMenu(self.mnuRelationsClientList)
        self.clientId = clientId
        self.modelRelationsClientList.loadData(self.clientId)


    @pyqtSignature('')
    def on_mnuRelationsClientList_aboutToShow(self):
        currentRow = -1
        index = self.tblRelationsClientList.currentIndex()
        if index and index.isValid():
            currentRow = index.row()
        itemPresent = currentRow>=0
        self.actEditClient.setEnabled(itemPresent)
        self.actMoveRegistry.setEnabled(itemPresent)


    @pyqtSignature('')
    def on_actEditClient_triggered(self):
        currentIndex = self.tblRelationsClientList.currentIndex()
        self.getEditClient(currentIndex)


    @pyqtSignature('')
    def on_actMoveRegistry_triggered(self):
        currentIndex = self.tblRelationsClientList.currentIndex()
        if currentIndex and currentIndex.isValid():
            row = currentIndex.row()
            relationId = self.modelRelationsClientList.getRelationId(row)
            if relationId:
                self.close()
                QtGui.qApp.findClient(relationId)


    @pyqtSignature('')
    def on_btnClose_clicked(self):
        self.close()


    @pyqtSignature('QModelIndex')
    def on_tblRelationsClientList_doubleClicked(self, index):
        self.getEditClient(index)


    def getEditClient(self, currentIndex):
        if currentIndex and currentIndex.isValid():
            row = currentIndex.row()
            relationId = self.modelRelationsClientList.getRelationId(row)
            if relationId:
                QtGui.qApp.callWithWaitCursor(self, self.editClient, relationId)


    def editClient(self, clientId):
        from Registry.ClientEditDialog  import CClientEditDialog
        dialog = CClientEditDialog(self)
        try:
            dialog.load(clientId)
            QtGui.qApp.restoreOverrideCursor()
            if dialog.exec_():
                QtGui.qApp.callWithWaitCursor(self, self.modelRelationsClientList.loadData, self.clientId)
        finally:
            dialog.destroy()
            sip.delete(dialog)
            del dialog


class CRelationsClientListModel(QAbstractTableModel):
    column = [u'Связан с пациентом']
    def __init__(self, parent):
        QAbstractTableModel.__init__(self, parent)
        self.items = []
        self._cols = []


    def cols(self):
        self._cols = [CCol(u'Связан с пациентом', ['relativeType_id'], 20, 'l')]
        return self._cols


    def columnCount(self, index = None):
        return 1


    def rowCount(self, index = None):
        return len(self.items)


    def headerData(self, section, orientation, role = Qt.DisplayRole):
        if orientation == Qt.Horizontal:
            if role == Qt.DisplayRole:
                return QVariant(self.column[section])
        return QVariant()


    def data(self, index, role=Qt.DisplayRole):
        column = index.column()
        row = index.row()
        if role == Qt.DisplayRole:
            item = self.items[row]
            return toVariant(item[column])
        return QVariant()


    def getRelationId(self, row):
        relationId = None
        if 0 <= row < self.rowCount():
            relationId = self.items[row][1]
        return relationId


    def loadData(self, clientId):
        self.items = []
        db = QtGui.qApp.db
        tableC  = db.table('Client')
        tableCR = db.table('ClientRelation')
        tableRT = db.table('rbRelationType')
        queryTable = tableCR
        queryTable = queryTable.leftJoin(tableRT, tableRT['id'].eq(tableCR['relativeType_id']))
        queryTable = queryTable.leftJoin(tableC, tableC['id'].eq(tableCR['client_id']))
        cond = [tableCR['relative_id'].eq(clientId),
                tableC['deleted'].eq(0),
                tableCR['deleted'].eq(0)]
        fields = ['CONCAT_WS(\' | \', rbRelationType.`code`, CONCAT_WS(\'->\', rbRelationType.`rightName`, rbRelationType.`leftName`)) AS relationType',
                  tableC['lastName'].alias(), tableC['firstName'].name(),
                  tableC['patrName'].name(), tableC['birthDate'].name(),
                  tableC['id'].alias('relationId')]
        records = db.getRecordList(queryTable, fields, cond)
        for record in records:
            relationId = forceRef(record.value('relationId'))
            name = formatShortName(record.value('lastName'),
                                   record.value('firstName'),
                                   record.value('patrName'))
            relationName = ', '.join([name,
                                forceString(record.value('birthDate')),
                                forceString(record.value('relationType'))])
            item = [relationName,
                    relationId
                    ]
            self.items.append(item)
        queryTable = tableCR
        queryTable = queryTable.leftJoin(tableRT, tableRT['id'].eq(tableCR['relativeType_id']))
        queryTable = queryTable.leftJoin(tableC, tableC['id'].eq(tableCR['relative_id']))
        cond = [tableCR['client_id'].eq(clientId),
                tableC['deleted'].eq(0),
                tableCR['deleted'].eq(0)]
        fields = ['CONCAT_WS(\' | \', rbRelationType.`code`, CONCAT_WS(\'->\', rbRelationType.`leftName`, rbRelationType.`rightName`)) AS relationType',
                  tableC['lastName'].alias(), tableC['firstName'].name(),
                  tableC['patrName'].name(), tableC['birthDate'].name(),
                  tableC['id'].alias('relationId')]
        records = db.getRecordList(queryTable, fields, cond)
        for record in records:
            relationId = forceRef(record.value('relationId'))
            name = formatShortName(record.value('lastName'),
                                   record.value('firstName'),
                                   record.value('patrName'))
            relationName = ', '.join([name,
                                forceString(record.value('birthDate')),
                                forceString(record.value('relationType'))])
            item = [relationName,
                    relationId
                    ]
            self.items.append(item)
        self.reset()

