# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
##
## Работа с отложенной записью
##
#############################################################################

from PyQt4 import QtGui
from PyQt4.QtCore import Qt, QDate, QDateTime, QMetaObject, QVariant, pyqtSignature

from library.database                  import CTableRecordCache
from library.DialogBase                import CDialogBase, CConstructHelperMixin
from library.interchange               import setTextEditValue, getComboBoxValue, getTextEditValue, getCheckBoxValue
from library.PreferencesMixin          import CDialogPreferencesMixin
from library.PrintInfo                 import CInfoContext
from library.PrintTemplates            import applyTemplate, CPrintAction, getPrintTemplates
from library.RecordLock                import CRecordLockMixin
from library.SimpleProgressDialog      import CSimpleProgressDialog
from library.TableModel                import CBoolCol, CEnumCol, CDateCol, CDesignationCol, CTextCol, CDateTimeCol, CCol, CRefBookCol, CTableModel
from library.Utils                     import formatName, toVariant, forceString, withWaitCursor, forceRef, forceTime, forceBool, formatRecordsCount, forceDate, forceStringEx, formatSex
from Registry.SuspendedAppointmentInfo import CSuspendedAppointmentInfo, CSuspendedAppointmentInfoList
from Orgs.Utils                        import getOrgStructureDescendants
from Registry.RegistryWindow           import CSchedulesModel, CVisitsBySchedulesModel, convertFilterToTextItem
from Registry.Utils                    import getClientBanner


#from Users.Rights             import *

from Registry.Ui_SuspendedAppointment import Ui_SuspenedAppointmentWindow
from Registry.Ui_SuspendedAppointmentMarksDialog import Ui_SuspendedAppointmentMarksDialog


class CSuspenedAppointmentWindow(QtGui.QScrollArea, Ui_SuspenedAppointmentWindow, CDialogPreferencesMixin, CConstructHelperMixin, CRecordLockMixin):
    def __init__(self, parent):
        QtGui.QScrollArea.__init__(self, parent)
        CRecordLockMixin.__init__(self)
        self.addModels('SuspendedAppointments', CSuspendedAppointmentModel(self))
        self.addModels('Schedules', CSchedulesModel(self))
        self.addModels('VisitsBySchedules',    CVisitsBySchedulesModel(self))
        self.addObject('actFilterAppointment', QtGui.QAction(u'Подобрать номерок', self))
        self.addObject('actSetProcessed',      QtGui.QAction(u'Отработать', self))
        self.addObject('actUnsetProcessed',    QtGui.QAction(u'Отменить', self))

        self.setObjectName('SuspenedAppointmentWindow')
        self.internal = QtGui.QWidget(self)
        self.setWidget(self.internal)
        self.setupUi(self.internal)

        self.setWindowTitle(self.internal.windowTitle())
        self.setWidgetResizable(True)
        QMetaObject.connectSlotsByName(self) # т.к. в setupUi параметр не self
#        self.setWindowFlags(self.windowFlags() | Qt.WindowMaximizeButtonHint)
        suspenedTemplates = getPrintTemplates(['suspendedAppointmentList'])
        if not suspenedTemplates:
            self.btnPrint.setId(-1)
        else:
            for template in suspenedTemplates:
                action = CPrintAction(template[0], template[1], self.btnPrint, self.btnPrint)
                self.btnPrint.addAction(action)
            self.btnPrint.menu().addSeparator()
            self.btnPrint.addAction(CPrintAction(u'Напечатать список визитов', -1, self.btnPrint, self.btnPrint))
        self.setModels(self.tblSuspendedAppointments, self.modelSuspendedAppointments, self.selectionModelSuspendedAppointments)

        self.setModels(self.tblSchedules, self.modelSchedules, self.selectionModelSchedules)
        self.setModels(self.tblVisitsBySchedules, self.modelVisitsBySchedules, self.selectionModelVisitsBySchedules)
        self.tblSuspendedAppointments.createPopupMenu([self.actFilterAppointment,
                                                       self.actSetProcessed,
                                                       self.actUnsetProcessed
                                                      ]
                                                     )

#        self.setFocusProxy(self.tabFilter)
#        self.tabFilter.setFocusProxy(self.tblSuspendedAppointments)

        self.edtFilterBegBirthDay.canBeEmpty(True)
        self.edtFilterEndBirthDay.canBeEmpty(True)
        self.edtFilterBegDate.canBeEmpty(True)
        self.edtFilterEndDate.canBeEmpty(True)
        self.cmbFilterSpeciality.setTable('rbSpeciality', True)

#        self.connect(QtGui.qApp, SIGNAL('currentClientInfoJLWChanged(int)'), self.updateDataTable)
#        self.connect(QtGui.qApp, SIGNAL('currentClientInfoJLWDeleted(int)'), self.deletedScheduleItemId)

        self.filter = {}
        self.on_buttonBoxFilter_reset()
        self.focusSuspendedAppointmentsList()


    @pyqtSignature('int')
    def on_btnPrint_printByTemplate(self, templateId):
        if templateId == -1:
            self.tblSuspendedAppointments.setReportHeader(u'Журнал отложенной записи')
            self.tblSuspendedAppointments.setReportDescription(self.getSuspenedAppointmentFilterAsText())
            self.tblSuspendedAppointments.printContent()
            self.tblSuspendedAppointments.setFocus(Qt.TabFocusReason)
        else:
            idList = self.tblSuspendedAppointments.model().idList()
            context = CInfoContext()
            suspenedAppointmentInfo = context.getInstance(CSuspendedAppointmentInfo, self.tblSuspendedAppointments.currentItemId())
            suspenedAppointmentInfoList = context.getInstance(CSuspendedAppointmentInfoList, tuple(idList))
            data = { 'suspendedAppointmentList': suspenedAppointmentInfoList,
                     'suspendedAppointment'    : suspenedAppointmentInfo }
            QtGui.qApp.call(self, applyTemplate, (self, templateId, data))


    def getSuspenedAppointmentFilterAsText(self):
        db = QtGui.qApp.db
        filter  = self.filter
        resList = []
        tmpList = [
            ('lastName',  u'Фамилия', forceString),
            ('firstName', u'Имя', forceString),
            ('patrName',  u'Отчество', forceString),
            ('birthDate', u'Дата рождения', forceString),
            ('begBirthDay', u'Дата рождения с', forceString),
            ('endBirthDay', u'Дата рождения по', forceString),
            ('sex',       u'Пол',           formatSex),
            ('conract',   u'Контакт', forceString),
            ('begVisitDate', u'Планируемый период визита с', forceString),
            ('endVisitDate', u'Планируемый период визита по', forceString),
            ('orgStructureId', u'Подразделение',
                lambda id: forceString(db.translate('OrgStructure', 'id', id, 'name'))),
            ('specialityId', u'Специальность',
                lambda id: forceString(db.translate('rbSpeciality', 'id', id, 'name'))),
            ('personId', u'Врач',
                lambda id: forceString(db.translate('vrbPersonWithSpeciality', 'id', id, 'name'))),
            ('externalUserRole',   u'Роль внешнего пользователя', forceString),
            ('externalUserName', u'Внешний пользователь', forceString),
            ('processedName', u'Отображать', forceString),
            ('notifiedName', u'Оповещение', forceString),
            ('scheduledName', u'Запись', forceString),
            ('note', u'Примечание', forceString),
            ('createPersonId', u'Автор',
                lambda id: forceString(db.translate('vrbPersonWithSpeciality', 'id', id, 'name'))),
            ('begCreateDate', u'Дата создания с', forceString),
            ('endCreateDate', u'Дата создания по', forceString),
            ('modifyPersonId', u'Автор последнего изменения',
                lambda id: forceString(db.translate('vrbPersonWithSpeciality', 'id', id, 'name'))),
            ('begModifyDate', u'Дата последнего изменения с', forceString),
            ('endModifyDate', u'Дата последнего изменения по', forceString),
            ]

        for (x, y, z) in tmpList:
            convertFilterToTextItem(resList, filter, x, y, z)
        return '\n'.join([ ': '.join(item) for item in resList])


    @withWaitCursor
    def updateSuspendedAppointmentsList(self, posToId=None):
        db = QtGui.qApp.db

        table = db.table('SuspendedAppointment')
        tableClient = db.table('Client')
        queryTable = table.innerJoin(tableClient, tableClient['id'].eq(table['client_id']))

        cond = [table['deleted'].eq(0),
                tableClient['deleted'].eq(0),
               ]

        lastName = self.filter.get('lastName')
        if lastName is not None:
            cond.append(tableClient['lastName'].like(lastName))

        firstName = self.filter.get('firstName')
        if firstName is not None:
            cond.append(tableClient['firstName'].like(firstName))

        patrName = self.filter.get('patrName')
        if patrName is not None:
            cond.append(tableClient['patrName'].like(patrName))

        begBirthDay = self.filter.get('begBirthDay', None)
        endBirthDay = self.filter.get('endBirthDay', None)
        if begBirthDay != endBirthDay:
            if begBirthDay:
                cond.append(tableClient['birthDate'].ge(begBirthDay))
            if endBirthDay:
                cond.append(tableClient['birthDate'].le(endBirthDay))
        elif begBirthDay:
            cond.append(tableClient['birthDate'].eq(begBirthDay))

        sex = self.filter.get('sex', None)
        if sex is not None:
            cond.append(tableClient['sex'].eq(sex))

        contact = self.filter.get('contact')
        if contact is not None:
            cond.append(table['contact'].like(contact))

        dataRange = self.filter.get('dateRange')
        if dataRange:
            begDate, endDate = dataRange
            if begDate:
                cond.append(table['endDate'].ge(begDate))
            if endDate:
                cond.append(table['begDate'].le(endDate))

        orgStructureId = self.filter.get('orgStructureId')
        if orgStructureId:
            cond.append(table['orgStructure_id'].inlist(getOrgStructureDescendants(orgStructureId)))

        specialityId = self.filter.get('specialityId')
        if specialityId:
            cond.append(table['speciality_id'].eq(specialityId))

        personId = self.filter.get('personId')
        if personId:
            cond.append(table['person_id'].eq(personId))

        externalUserRole = self.filter.get('externalUserRole')
        if externalUserRole:
            cond.append(table['externalUserRole'].like(externalUserRole))

        externalUserName = self.filter.get('externalUserName')
        if externalUserName:
            cond.append(table['externalUserName'].like(externalUserName))

        processed = self.filter.get('processed')
        if processed is not None:
            cond.append(table['processed'].eq(processed))

        notified = self.filter.get('notified')
        if notified is not None:
            if notified:
                cond.append(table['notified'].ne(0))
            else:
                cond.append(table['notified'].eq(0))

        scheduled = self.filter.get('scheduled')
        if scheduled is not None:
            if scheduled:
                cond.append(table['appointment_id'].isNotNull())
            else:
                cond.append(table['appointment_id'].isNull())

        note = self.filter.get('note')
        if note is not None:
            cond.append(table['note'].like(note))

        createPersonId = self.filter.get('createPersonId')
        if createPersonId:
            cond.append(table['createPerson_id'].eq(createPersonId))

        createDateRange = self.filter.get('createDateRange')
        if createDateRange:
            begDate, endDate = createDateRange
            if begDate:
                cond.append(table['createDatetime'].ge(begDate))
            if endDate:
                cond.append(table['createDatetime'].lt(endDate.addDays(1)))

        modifyPersonId = self.filter.get('modifyPersonId')
        if modifyPersonId:
            cond.append(table['modifyPerson_id'].eq(modifyPersonId))

        modifyDateRange = self.filter.get('modifyDateRange')
        if modifyDateRange:
            begDate, endDate = modifyDateRange
            if begDate:
                cond.append(table['modifyDatetime'].ge(begDate))
            if endDate:
                cond.append(table['modifyDatetime'].lt(endDate.addDays(1)))


        order = table['id'].name()
        idList = db.getDistinctIdList(queryTable,
                                      table['id'].name(),
                                      cond,
                                      order
                                     )
        self.tblSuspendedAppointments.setIdList(idList, posToId)
        self.lblRecordCount.setText(formatRecordsCount(len(idList)))


    def focusSuspendedAppointmentsList(self):
        self.tblSuspendedAppointments.setFocus(Qt.TabFocusReason)


    def updateClientInfo(self):
        clientId = self.getCurrentClientId()
        if clientId:
            self.txtClientInfoBrowser.setHtml(getClientBanner(clientId))
        else:
            self.txtClientInfoBrowser.setText('')
        QtGui.qApp.setCurrentClientId(clientId)
        self.loadSchedules(clientId)


    def loadSchedules(self, clientId):
        self.modelSchedules.loadData(clientId)
        self.modelVisitsBySchedules.loadData(clientId)


    def getCurrentClientId(self):
        record = self.tblSuspendedAppointments.currentItem()
        if record:
            return forceRef(record.value('client_id'))
        return None


    def findAppointment(self, record):
        db = QtGui.qApp.db
        clientId = forceRef(record.value('client_id'))
        personId = forceRef(record.value('person_id'))
        specialityId = forceRef(record.value('speciality_id'))
        orgStructureId = forceRef(record.value('orgStructure_id'))
        begDate = forceDate(record.value('begDate'))
#           endDate = forceDate(record.value('endDate'))

        tableScheduleItem = db.table('Schedule_Item')
        tableSchedule = db.table('Schedule')
        table = tableScheduleItem.innerJoin(tableSchedule, tableSchedule['id'].eq(tableScheduleItem['master_id']))
        cond = [ tableScheduleItem['deleted'].eq(0),
                 tableSchedule['deleted'].eq(0),
                 tableSchedule['date'].ge(begDate),
#                 tableSchedule['date'].le(endDate),
                 tableScheduleItem['client_id'].eq(clientId),
               ]
        if personId:
            cond.append(tableSchedule['person_id'].eq(personId))
        else:
            if specialityId or orgStructureId:
                tablePerson = db.table('Person')
                table = table.innerJoin(tablePerson, tablePerson['id'].eq(tableSchedule['person_id']))
            if specialityId:
                cond.append(tablePerson['speciality_id'].eq(specialityId))
            if orgStructureId:
                cond.append(tablePerson['orgStructure_id'].inlist(getOrgStructureDescendants(orgStructureId)))

        ids = db.getIdList(table,
                           tableScheduleItem['id'],
                           cond,
                           tableScheduleItem['time'].name(),
                           1)
        return ids[0] if ids else None


    def editMarks(self, onProcess):
        db = QtGui.qApp.db
        table = db.table('SuspendedAppointment')
        record = self.tblSuspendedAppointments.currentItem()
        if record:
            id = forceRef(record.value('id'))
            appointmentId = forceRef(record.value('appointment_id'))
            if self.lock('SuspendedAppointment', id):
                try:
                    dlg = CSuspendedAppointmentMarksDialog(self)
                    dlg.setMarksFromRecord(record, onProcess)
                    if dlg.exec_():
                        tmpRecord = table.newRecord(['id', 'processed', 'notified', 'note', 'appointment_id'])
                        tmpRecord.setValue('id', id)
                        tmpRecord.setValue('appointment_id', appointmentId)
                        dlg.storeMarksToRecord(tmpRecord)
                        if onProcess:
                            if not appointmentId:
                                tmpRecord.setValue('appointment_id', self.findAppointment(record))
                        else:
                            tmpRecord.setNull('appointment_id')
                        db.updateRecord(table, tmpRecord)
                        self.updateSuspendedAppointmentsList(id)
                finally:
                    self.releaseLock()


    def findAppointments(self):
        def stepIterator(progressDialog):
            db = QtGui.qApp.db
            table = db.table('SuspendedAppointment')

            for idx in xrange(self.modelSuspendedAppointments.rowCount()):
                record = self.modelSuspendedAppointments.getRecordByRow(idx)
                if not forceBool(record.value('processed')):
                    id = forceRef(record.value('id'))
                    if self.lock('SuspendedAppointment', id):
                        try:
                            appointmentId = self.findAppointment(record)
                            if appointmentId:
                                tmpRecord = table.newRecord(['id', 'processed', 'notified', 'note', 'appointment_id'])
                                tmpRecord.setValue('id', id)
                                tmpRecord.setValue('processed', True)
                                tmpRecord.setValue('notified', 0)
                                tmpRecord.setValue('note', u'Номерок подобран при обработке журнала')
                                tmpRecord.setValue('appointment_id', appointmentId)
                                db.updateRecord(table, tmpRecord)
#                                self.updateSuspendedAppointmentsList(id)
                        finally:
                            self.releaseLock()
                yield 1

        pd = CSimpleProgressDialog(self)
        pd.setWindowTitle(u'Подбор талонов')
        pd.setStepCount(self.modelSuspendedAppointments.rowCount())
        pd.setAutoStart(True)
        pd.setAutoClose(False)
        pd.setStepIterator(stepIterator)
        pd.exec_()
        self.modelSuspendedAppointments.invalidateRecordsCache()
        self.tblSuspendedAppointments.update()


    def on_buttonBoxFilter_apply(self):
        def cmbToTristate(val):
            return val-1 if val>0 else None

        self.filter = {}
        if self.chkFilterLastName.isChecked():
            self.filter['lastName'] = forceStringEx(self.edtFilterLastName.text())
        if self.chkFilterFirstName.isChecked():
            self.filter['firstName'] = forceStringEx(self.edtFilterFirstName.text())
        if self.chkFilterPatrName.isChecked():
            self.filter['patrName'] = forceStringEx(self.edtFilterPatrName.text())
        if self.chkFilterBirthDay.isChecked():
            if self.chkFilterEndBirthDay.isChecked():
                self.filter['begBirthDay'] = self.edtFilterBegBirthDay.date()
                self.filter['endBirthDay'] = self.edtFilterEndBirthDay.date()
            else:
                self.filter['begBirthDay'] = self.filter['endBirthDay'] = self.edtFilterBegBirthDay.date()
        if self.chkFilterSex.isChecked():
            self.filter['sex'] = self.cmbFilterSex.currentIndex()
        if self.chkFilterContact.isChecked():
            self.filter['contact'] = forceStringEx(self.edtFilterContact.text())
        if self.chkFilterDateRange.isChecked():
            self.filter['dateRange'] = self.edtFilterBegDate.date(), self.edtFilterEndDate.date()
            self.filter['begVisitDate'] = self.edtFilterBegDate.date()
            self.filter['endVisitDate'] = self.edtFilterEndDate.date()
        if self.chkFilterOrgStructure.isChecked():
            self.filter['orgStructureId'] = self.cmbFilterOrgStructure.value()
        if self.chkFilterSpeciality.isChecked():
            self.filter['specialityId'] = self.cmbFilterSpeciality.value()
        if self.chkFilterPerson.isChecked():
            self.filter['personId'] = self.cmbFilterPerson.value()
        if self.chkFilterExternalUserRole.isChecked():
            self.filter['externalUserRole'] = forceStringEx(self.edtFilterExternalUserRole.text())
        if self.chkFilterExternalUserName.isChecked():
            self.filter['externalUserName'] = forceStringEx(self.edtFilterExternalUserName.text())
        self.filter['processed'] = cmbToTristate(self.cmbFilterProcessed.currentIndex())
        self.filter['processedName'] = self.cmbFilterProcessed.currentText()
        self.filter['notified']  = cmbToTristate(self.cmbFilterNotification.currentIndex())
        self.filter['notifiedName']  = self.cmbFilterNotification.currentText()
        self.filter['scheduled'] = cmbToTristate(self.cmbFilterScheduled.currentIndex())
        self.filter['scheduledName'] = self.cmbFilterScheduled.currentText()
        if self.chkFilterNote.isChecked():
            self.filter['note'] = forceStringEx(self.edtFilterNote.text())
        if self.chkFilterExCreatePerson.isChecked():
            self.filter['createPersonId'] = self.cmbFilterExCreatePerson.value()
        if self.chkFilterExCreateDate.isChecked():
            self.filter['createDateRange'] = self.edtFilterExBegCreateDate.date(), self.edtFilterExEndCreateDate.date()
            self.filter['begCreateDate'] = self.edtFilterExBegCreateDate.date()
            self.filter['endCreateDate'] = self.edtFilterExEndCreateDate.date()
        if self.chkFilterExModifyPerson.isChecked():
            self.filter['modifyPersonId'] = self.cmbFilterExModifyPerson.value()
        if self.chkFilterExModifyDate.isChecked():
            self.filter['modifyDateRange'] = self.edtFilterExBegModifyDate.date(), self.edtFilterExEndModifyDate.date()
            self.filter['begModifyDate'] = self.edtFilterExBegModifyDate.date()
            self.filter['endModifyDate'] = self.edtFilterExEndModifyDate.date()
        self.updateSuspendedAppointmentsList()


    def on_buttonBoxFilter_reset(self):
        self.chkFilterLastName.setChecked(False)
        self.edtFilterLastName.setText('')
        self.chkFilterFirstName.setChecked(False)
        self.edtFilterFirstName.setText('')
        self.chkFilterPatrName.setChecked(False)
        self.edtFilterPatrName.setText('')
        self.chkFilterBirthDay.setChecked(False)
        self.edtFilterBegBirthDay.setDate(QDate())
        self.chkFilterEndBirthDay.setChecked(False)
        self.edtFilterEndBirthDay.setDate(QDate())
        self.chkFilterSex.setChecked(False)
        self.cmbFilterSex.setCurrentIndex(0)
        self.chkFilterContact.setChecked(False)
        self.edtFilterContact.setText('')
        self.chkFilterDateRange.setChecked(False)
        self.edtFilterBegDate.setDate(QDate.currentDate())
        self.edtFilterEndDate.setDate(QDate.currentDate().addMonths(1))
        self.chkFilterOrgStructure.setChecked(False)
        self.cmbFilterOrgStructure.setValue(None)
        self.chkFilterSpeciality.setChecked(False)
        self.cmbFilterSpeciality.setValue(None)
        self.chkFilterPerson.setChecked(False)
        self.cmbFilterPerson.setValue(None)
        self.chkFilterExternalUserRole.setChecked(False)
        self.edtFilterExternalUserRole.setText('')
        self.chkFilterExternalUserName.setChecked(False)
        self.edtFilterExternalUserName.setText('')
        self.cmbFilterProcessed.setCurrentIndex(1)
        self.cmbFilterNotification.setCurrentIndex(0)
        self.chkFilterNote.setChecked(False)
        self.edtFilterNote.setText('')
        self.cmbFilterScheduled.setCurrentIndex(0)
        self.chkFilterExCreatePerson.setChecked(False)
        self.cmbFilterExCreatePerson.setValue(None)
        self.chkFilterExCreateDate.setChecked(False)
        self.edtFilterExBegCreateDate.setDate(QDate())
        self.edtFilterExEndCreateDate.setDate(QDate())
        self.chkFilterExModifyPerson.setChecked(False)
        self.cmbFilterExModifyPerson.setValue(None)
        self.chkFilterExModifyDate.setChecked(False)
        self.edtFilterExBegModifyDate.setDate(QDate())
        self.edtFilterExEndModifyDate.setDate(QDate())

        self.on_buttonBoxFilter_apply()


##############################################
## SLOTS #####################################

    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelSuspendedAppointments_currentRowChanged(self, current, previous):
        self.updateClientInfo()


    @pyqtSignature('')
    def on_tblSuspendedAppointments_popupMenuAboutToShow(self):
        dataPresent = self.tblSuspendedAppointments.currentItem() is not None
        self.actFilterAppointment.setEnabled(dataPresent)
        self.actSetProcessed.setEnabled(dataPresent)
        self.actUnsetProcessed.setEnabled(dataPresent)


    @pyqtSignature('')
    def on_actFilterAppointment_triggered(self):
        record = self.tblSuspendedAppointments.currentItem()
        if record:
            orgStructureId = forceRef(record.value('orgStructure_id'))
            specialityId   = forceRef(record.value('speciality_id'))
            personId       = forceRef(record.value('person_id'))
            begDate        = forceDate(record.value('begDate'))
            QtGui.qApp.setupDocFreeQueue(orgStructureId, specialityId, personId, begDate)


    @pyqtSignature('')
    def on_actSetProcessed_triggered(self):
        self.editMarks(True)


    @pyqtSignature('')
    def on_actUnsetProcessed_triggered(self):
        self.editMarks(False)


    @pyqtSignature('bool')
    def on_chkFilterBirthDay_toggled(self, value):
        self.edtFilterEndBirthDay.setEnabled(value and self.chkFilterEndBirthDay.isChecked())


    @pyqtSignature('QAbstractButton*')
    def on_buttonBoxFilter_clicked(self, button):
        buttonCode = self.buttonBoxFilter.standardButton(button)
        if buttonCode == QtGui.QDialogButtonBox.Apply:
            self.on_buttonBoxFilter_apply()
        elif buttonCode == QtGui.QDialogButtonBox.Reset:
            self.on_buttonBoxFilter_reset()
        self.focusSuspendedAppointmentsList()
        self.updateClientInfo()


    @pyqtSignature('')
    def on_btnFindAppointments_clicked(self):
        self.findAppointments()



class CSuspendedAppointmentModel(CTableModel):

    class CLocClientColumn(CCol):
        def __init__(self, title, fields, defaultWidth, clientCache):
            CCol.__init__(self, title, fields, defaultWidth, 'l')
            self.clientCache = clientCache

        def format(self, values):
            val = values[0]
            clientId  = forceRef(val)
            clientRecord = self.clientCache.get(clientId)
            if clientRecord:
                name  = formatName(clientRecord.value('lastName'),
                                   clientRecord.value('firstName'),
                                   clientRecord.value('patrName'))
                return toVariant(name)
            return CCol.invalid


    class CLocClientBirthDateColumn(CCol):
        def __init__(self, title, fields, defaultWidth, clientCache):
            CCol.__init__(self, title, fields, defaultWidth, 'l')
            self.clientCache = clientCache

        def format(self, values):
            val = values[0]
            clientId  = forceRef(val)
            clientRecord = self.clientCache.get(clientId)
            if clientRecord:
                return toVariant(forceString(clientRecord.value('birthDate')))
            return CCol.invalid


    class CLocClientSexColumn(CCol):
        def __init__(self, title, fields, defaultWidth, clientCache):
            CCol.__init__(self, title, fields, defaultWidth, 'l')
            self.clientCache = clientCache

        def format(self, values):
            val = values[0]
            clientId  = forceRef(val)
            clientRecord = self.clientCache.get(clientId)
            if clientRecord:
                return toVariant(formatSex(clientRecord.value('sex')))
            return CCol.invalid


    class CScheduleItemCol(CCol):
        def __init__(self, title, fields, defaultWidth):
            CCol.__init__(self, title, fields, defaultWidth, 'l')

            db = QtGui.qApp.db
            tableScheduleItem = db.table('Schedule_Item')
            tableSchedule = db.table('Schedule')
            tablePerson = db.table('vrbPersonWithSpeciality')
            tableOrgStructure = db.table('OrgStructure')
            cols = ['Schedule_Item.client_id',
                    'Schedule_Item.time',
                    'Schedule.date',
                    'OrgStructure.name AS osName',
                    'vrbPersonWithSpeciality.name AS personName',
                    '(Schedule_Item.deleted OR Schedule.deleted) AS deleted'
                   ]
            table = tableScheduleItem.innerJoin(tableSchedule, tableSchedule['id'].eq(tableScheduleItem['master_id']))
            table = table.innerJoin(tablePerson, tablePerson['id'].eq(tableSchedule['person_id']))
            table = table.innerJoin(tableOrgStructure, tableOrgStructure['id'].eq(tablePerson['orgStructure_id']))

            self.recordCache = CTableRecordCache(db, table, cols)


        def invalidateRecordsCache(self):
            self.recordCache.invalidate()


        def format(self, values):
            clientId = forceRef(values[0])
            record = self.recordCache.get(values[1])
            if record and not forceBool(record.value('deleted')) and clientId == forceRef(record.value('client_id')):
                date = forceDate(record.value('date'))
                time = forceTime(record.value('time'))
                personName = forceString(record.value('personName'))
#                osName = forceString(record.value('osName'))
                return toVariant(forceString(QDateTime(date, time))+' '+personName)
            return QVariant()


    def __init__(self, parent):
        self.clientCache = CTableRecordCache(QtGui.qApp.db, 'Client', ('id', 'lastName', 'firstName', 'patrName', 'birthDate', 'sex'), 300)
        CTableModel.__init__(self, parent)
        self.addColumn(self.CLocClientColumn( u'Ф.И.О.', ('client_id',), 60, self.clientCache))
        self.addColumn(self.CLocClientBirthDateColumn(u'Дата рожд.', ('client_id',), 20, self.clientCache))
        self.addColumn(self.CLocClientSexColumn(u'Пол', ('client_id',), 5, self.clientCache))
        self.addColumn(CTextCol(u'Телефон',    ('contact', ),  10))
        self.addColumn(CDateCol(u'С',   ('begDate',), 10))
        self.addColumn(CDateCol(u'По',  ('endDate',), 10))
        self.addColumn(CDesignationCol(u'Подразделение', ('orgStructure_id',), ('OrgStructure', 'name'), 10))
        self.addColumn(CRefBookCol(u'Специальность',     ('speciality_id',),    'rbSpeciality', 30))
        self.addColumn(CRefBookCol(u'Врач',              ('person_id',),        'vrbPerson', 30))
        self.addColumn(CBoolCol(u'Отработан',            ('processed',),        7))
        self.addColumn(self.CScheduleItemCol(u'Талон на приём к врачу', ('client_id', 'appointment_id', ), 50))
        self.addColumn(CEnumCol(u'Оповещён',      ('notified', ), (u'', u'телефон', u'СМС', u'эл.почта'),  10))
        self.addColumn(CTextCol(u'Примечание',    ('note', ),  10))
        self.addColumn(CTextCol(u'Роль',          ('externalUserRole', ),  10))
        self.addColumn(CTextCol(u'Внешний пользователь',  ('externalUserName', ),  10))
        self.addColumn(CTextCol(u'Причина',       ('reason', ),  10))
        self.addColumn(CDateTimeCol(u'Создано',   ('createDatetime',), 20))
        self.addColumn(CRefBookCol(u'Создал',    ('createPerson_id',), 'vrbPersonWithSpeciality', 30))
        self.addColumn(CDateTimeCol(u'Изменено',  ('modifyDatetime',), 20))
        self.addColumn(CRefBookCol(u'Изменил',    ('modifyPerson_id',), 'vrbPersonWithSpeciality', 30))
        self.setTable('SuspendedAppointment')


    def invalidateRecordsCache(self):
        self.clientCache.invalidate()
        CTableModel.invalidateRecordsCache(self)


class CSuspendedAppointmentMarksDialog(CDialogBase, Ui_SuspendedAppointmentMarksDialog):
    def __init__(self, parent):
        CDialogBase.__init__(self, parent)
        self.setupUi(self)
        self.setupDirtyCather()


    def setMarksFromRecord(self, record, onProcess):
        if onProcess:
            self.chkProcessed.setChecked(True)
            self.cmbNotified.setCurrentIndex(1) # телефон
            self.setWindowTitle(u'Отработать')
        else:
            self.chkProcessed.setChecked(False)
            self.cmbNotified.setCurrentIndex(0) # нет
            self.setWindowTitle(u'Отменить отработку')
        setTextEditValue(self.edtNote,      record, 'note')
        self.setIsDirty(False)


    def storeMarksToRecord(self, record):
        getCheckBoxValue(self.chkProcessed, record, 'processed')
        getComboBoxValue(self.cmbNotified,  record, 'notified')
        getTextEditValue(self.edtNote,      record, 'note')

