# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui
from PyQt4.QtCore import Qt, QDate, QVariant, SIGNAL

from library.database      import decorateString
from library.DbComboBox    import CDbModel
from library.DbEntityCache import CDbEntityCache
from library.PrintInfo     import CInfoContext, CInfo, CInfoList, CDateInfo, CDateTimeInfo, CRBInfo,  CTimeInfo
from library.Utils         import calcAge, calcAgeTuple, forceBool, forceDate, forceDateTime, forceDouble, forceInt, forceRef, forceString, forceStringEx, forceTime, formatAgeTuple, formatDate, formatName, formatNameInt, formatSex, formatShortName, formatShortNameInt, formatSNILS, smartDict, toVariant, trim
from library.ROComboBox    import CROComboBox

from Events.MKBInfo        import CMKBInfo
from KLADR.KLADRModel      import getCityName, getStreetName, getKladrTreeModel, getOKATO, getDistrictName, getMainRegionName
from Orgs.PersonInfo       import CPersonInfo
from Orgs.Utils            import CNet, COrgInfo, COrgStructureInfo, CSexAgeConstraint, getOrganisationShortName, getOrgStructureDescendants, getOrgStructureNetId
from RefBooks.QuotaType    import getQuotaTypeClassNameList
from Users.Rights          import urRegWriteInsurOfficeMark, urRegEditTempInvalidWithInsurance, urRegEditTempInvalidNoInsurance


expertiseClass = [(u'Нетрудоспособность', u'inspection_disability%'),
                 ];


def replaceMask(val, repFrom, repTo):
    return unicode('REPLACE(' + val + ',' + repFrom + ',' + repTo + ')')


# wtf? это точно имеет отношение к списку пациентов и не может быть перенесено в другое место?
def deleteTempInvalid(widget, tempInvalidId, onlyPrepare=False):
    db = QtGui.qApp.db
    table = db.table('TempInvalid')
    tablePeriod = db.table('TempInvalid_Period')
    tableResult = db.table('rbTempInvalidResult')
    tableDocument = db.table('TempInvalidDocument')
#            record = db.getRecord(table, 'id, deleted', tempInvalidId)
#            if record:
#                record.setValue('deleted',  QVariant(True))
#                db.updateRecord(table, record)
#            insuranceOfficeMark = forceInt(db.translate(table, 'id', tempInvalidId, 'insuranceOfficeMark'))
    record = db.getRecordEx(table, [table['insuranceOfficeMark'], table['prev_id'], table['client_id'], table['begDate'], table['endDate']], [table['id'].eq(tempInvalidId), table['deleted'].eq(0)])
    insuranceOfficeMark = 0
    prevId = None
    clientId = None
    begDate = None
    endDate = None
    if record:
        insuranceOfficeMark = forceInt(record.value('insuranceOfficeMark'))
        prevId = forceRef(record.value('prev_id'))
        clientId = forceRef(record.value('client_id'))
        begDate = forceDate(record.value('begDate'))
        endDate = forceDate(record.value('endDate'))
    recordPrev = db.getRecordEx(table, [table['id'], table['begDate'], table['endDate']], [table['prev_id'].eq(tempInvalidId), table['deleted'].eq(0)])
    prevTempInvalidId = None
    begDateLast = None
    endDateLast = None
    if recordPrev:
        prevTempInvalidId = forceRef(recordPrev.value('id'))
        begDateLast = forceDate(recordPrev.value('begDate'))
        endDateLast = forceDate(recordPrev.value('endDate'))
    closedResult = 0
    if prevId:
        queryTable = tablePeriod.innerJoin(table, tablePeriod['master_id'].eq(table['id']))
        queryTable = queryTable.innerJoin(tableResult, tableResult['id'].eq(tablePeriod['result_id']))
        recordClosed = db.getRecordEx(queryTable, [tableResult['closed'].alias('closedResult')], [table['id'].eq(prevId), table['deleted'].eq(0)], 'TempInvalid_Period.endDate DESC')
        if recordClosed:
            closedResult = forceInt(recordClosed.value('closedResult'))
    if (not prevTempInvalidId) and (not insuranceOfficeMark or (insuranceOfficeMark and QtGui.qApp.userHasRight(urRegWriteInsurOfficeMark))):
        documentLastId, number, issueDate = getLastTempInvalidDocument(tempInvalidId)
        if not documentLastId:
            if prevId:
                db.updateRecords(table.name(), table['closed'].eq(closedResult), [table['id'].eq(prevId), table['deleted'].eq(0)])
            if not onlyPrepare:
                db.markRecordsDeleted(table, table['id'].eq(tempInvalidId))
                tempInvalidDocumentIdList = getTempInvalidDocumentIdList(tempInvalidId)
                if tempInvalidDocumentIdList:
                    db.markRecordsDeleted(tableDocument, tableDocument['id'].inlist(tempInvalidDocumentIdList))
                return True
        else:
            clientName = getClientName(clientId)
            documentId, numberLast, issueDateLast, clientIdLast = getTempInvalidDocumentInfo(documentLastId)
            QtGui.QMessageBox.warning(widget,
                            u'Внимание',
                            u' Нельзя удалять документ/эпизод (%s, %s, %s),\n так как в системе существует связанный с ним документ/эпизод (%s, %s, %s)!'%(number, forceString(issueDate), clientName, numberLast, forceString(issueDateLast), clientIdLast),
                            QtGui.QMessageBox.Ok,
                            QtGui.QMessageBox.Ok)
    elif prevTempInvalidId:
        QtGui.QMessageBox.warning(widget,
                        u'Внимание',
                        u' Нельзя удалять эпизод (%s - %s),\n так как в системе существует связанный с ним эпизод (%s - %s)!'%(forceString(begDate), forceString(endDate), forceString(begDateLast), forceString(endDateLast)),
                        QtGui.QMessageBox.Ok,
                        QtGui.QMessageBox.Ok)
    return False


def getClientName(clientId):
    clientName = u''
    if clientId:
        db = QtGui.qApp.db
        tableClient = db.table('Client')
        recordClient = db.getRecordEx(tableClient, [tableClient['lastName'], tableClient['firstName'], tableClient['patrName'], tableClient['id']], [tableClient['id'].eq(clientId), tableClient['deleted'].eq(0)])
        if recordClient:
            clientId = forceString(recordClient.value('id'))
            lastName = forceString(recordClient.value('lastName'))
            firstName = forceString(recordClient.value('firstName'))
            patrName = forceString(recordClient.value('patrName'))
            clientName = formatNameInt(lastName, firstName, patrName) + u', ' + clientId
    return clientName


def getTempInvalidDocumentIdList(tempInvalidId):
    if tempInvalidId:
        db = QtGui.qApp.db
        table = db.table('TempInvalidDocument')
        cond = [table['deleted'].eq(0),
                table['master_id'].eq(tempInvalidId)
                ]
        return db.getDistinctIdList(table, [table['id']], cond, [table['issueDate'].name(), table['idx'].name()])
    return None


def getLastTempInvalidDocument(tempInvalidId):
    documentLastId = None
    number = u''
    issueDate = u''
    if tempInvalidId:
        db = QtGui.qApp.db
        table = db.table('TempInvalidDocument')
        tableDocumentLast = db.table('TempInvalidDocument').alias('TempInvalidDocumentLast')
        cols = [table['last_id'],
                table['number'],
                table['issueDate']
                ]
        cond = [table['deleted'].eq(0),
                table['master_id'].eq(tempInvalidId),
                table['last_id'].isNotNull(),
                tableDocumentLast['deleted'].eq(0)
                ]
        queryTable = table.innerJoin(tableDocumentLast, tableDocumentLast['id'].eq(table['last_id']))
        record = db.getRecordEx(queryTable, cols, cond, [table['issueDate'].name(), table['idx'].name()])
        if record:
            documentLastId = forceRef(record.value('last_id'))
            number = forceString(record.value('number'))
            issueDate = forceDate(record.value('issueDate'))
    return documentLastId, number, issueDate


def getTempInvalidDocumentInfo(tempInvalidDocumentId, lastBool = False):
    documentLastId = None
    number = u''
    issueDate = u''
    clientId = None
    if tempInvalidDocumentId:
        db = QtGui.qApp.db
        table = db.table('TempInvalid')
        tableDocument = db.table('TempInvalidDocument')
        tableDocumentLast = db.table('TempInvalidDocument').alias('TempInvalidDocumentLast')
        cols = [tableDocument['last_id'],
                tableDocument['number'],
                tableDocument['issueDate'],
                table['client_id']
                ]
        cond = [tableDocument['deleted'].eq(0),
                table['deleted'].eq(0),
                tableDocument['id'].eq(tempInvalidDocumentId)
                ]
        queryTable = table.innerJoin(tableDocument, tableDocument['master_id'].eq(table['id']))
        if lastBool:
            queryTable = queryTable.innerJoin(tableDocumentLast, db.joinAnd([tableDocumentLast['id'].eq(tableDocument['last_id']), tableDocumentLast['deleted'].eq(0)]))
        else:
            queryTable = queryTable.leftJoin(tableDocumentLast, db.joinAnd([tableDocumentLast['id'].eq(tableDocument['last_id']), tableDocumentLast['deleted'].eq(0)]))
        record = db.getRecordEx(queryTable, cols, cond, [tableDocument['issueDate'].name(), tableDocument['idx'].name()])
        if record:
            documentLastId = forceRef(record.value('last_id'))
            number = forceString(record.value('number'))
            issueDate = forceDate(record.value('issueDate'))
            clientId = forceRef(record.value('client_id'))
    return documentLastId, number, issueDate, clientId


def deleteTempInvalidDocument(widget, tempInvalidClientId, tempInvalidId, tempInvalidDocumentId):
    db = QtGui.qApp.db
    table = db.table('TempInvalid')
    if tempInvalidId:
        record = db.getRecordEx(table, [table['insuranceOfficeMark']], [table['id'].eq(tempInvalidId), table['deleted'].eq(0)])
    insuranceOfficeMark = 0
    if record:
        insuranceOfficeMark = forceInt(record.value('insuranceOfficeMark'))
    if not insuranceOfficeMark or (insuranceOfficeMark and QtGui.qApp.userHasRight(urRegWriteInsurOfficeMark)):
        documentLastId, number, issueDate, clientId = getTempInvalidDocumentInfo(tempInvalidDocumentId, lastBool = True)
        if not documentLastId:
            return True
        else:
            clientName = getClientName(tempInvalidClientId if tempInvalidClientId else clientId)
            documentId, numberLast, issueDateLast, clientIdLast = getTempInvalidDocumentInfo(documentLastId)
            clientNameLast = getClientName(clientIdLast)
            QtGui.QMessageBox.warning(widget,
                            u'Внимание',
                            u' Нельзя удалять документ (%s, %s, %s),\n так как в системе существует связанный с ним документ (%s, %s, %s)!'%(number, forceString(issueDate), clientName, numberLast, forceString(issueDateLast), clientNameLast),
                            QtGui.QMessageBox.Ok,
                            QtGui.QMessageBox.Ok)
    return False


def selectLatestRecordStmt(tableName, clientId, filter='' ):
    if type(tableName) == tuple:
        tableName1, tableName2 = tableName
    else:
        tableName1 = tableName
        tableName2 = tableName
    pos = tableName2.find('AS Tmp')
    if pos < 0:
        tableName2 = tableName2 + ' AS Tmp'

    if filter:
        filter = ' AND ('+filter+')'
    return u'SELECT * FROM %s AS Main WHERE Main.client_id = \'%s\' AND Main.id = (SELECT MAX(Tmp.id) FROM %s WHERE Tmp.client_id=%s AND Tmp.deleted=0 %s)' % (tableName1, clientId, tableName2, clientId, filter)


def selectLatestRecord(tableName, clientId, filter=''):
    db = QtGui.qApp.db
    stmt = selectLatestRecordStmt(tableName, clientId, filter)
    query = db.query(stmt)
    if query.next():
        return query.record()
    else:
        return None


def getClientAddress(clientId, addrType):
    return selectLatestRecord('ClientAddress', clientId,  'type=\'%d\'' % addrType)


def getClientDistrict(KLADRCode, KLADRStreetCode, number):
    return getDistrictName(KLADRCode, KLADRStreetCode, number)
# WFT?
#    if districtId is None:
#        return None
#    else:
#        return forceString(QtGui.qApp.db.translate('District', 'id', districtId, 'name'))


def getClientAddressEx(clientId):
    addr = getClientAddress(clientId, 1)
    if not addr:
        addr = getClientAddress(clientId, 0)
    return addr


def getAddress(addressId, freeInput=None):
    result = smartDict()
    result.KLADRCode = ''
    result.KLADRStreetCode = ''
    result.number = ''
    result.corpus = ''
    result.flat = ''
    result.freeInput = forceString(freeInput) if freeInput else ''

    db = QtGui.qApp.db
    houseId = None
    if addressId:
        recAddress = db.getRecord('Address', 'house_id, flat', addressId)
        if recAddress:
            houseId = forceRef(recAddress.value('house_id'))
            result.flat = forceString(recAddress.value('flat'))
    if houseId:
        houseRecord = db.getRecord('AddressHouse', 'KLADRCode, KLADRStreetCode, number, corpus', houseId)
        result.KLADRCode = forceString(houseRecord.value('KLADRCode'))
        result.KLADRStreetCode = forceString(houseRecord.value('KLADRStreetCode'))
        result.number = forceString(houseRecord.value('number'))
        result.corpus = forceString(houseRecord.value('corpus'))
    return result


def getInfisForKLADRCode(KLADRCode):
    if KLADRCode:
        return forceString(QtGui.qApp.db.translate('kladr.KLADR', 'CODE', KLADRCode, 'infis'))
    return ''


def getInfisForStreetKLADRCode(KLADRStreetCode):
    if KLADRStreetCode:
        return forceString(QtGui.qApp.db.translate('kladr.STREET', 'CODE', KLADRStreetCode, 'infis'))
    return ''


def formatAddressInt(address, type=None):
    if address.KLADRCode:
        parts = []
        parts.append(getCityName(address.KLADRCode))
        if address.KLADRStreetCode:
            parts.append(getStreetName(address.KLADRStreetCode))
        if address.number:
            parts.append(u'д. '+address.number)
        if address.corpus:
            parts.append(u'к. '+address.corpus)
        if address.flat:
            parts.append(u'кв. '+address.flat)
        return ', '.join(parts)
    if address.freeInput and type in [None, 0]:     # в соотвествии с задачей №8822 и №8950.
            return '[' + address.freeInput + ']'
    return ''


def formatAddress(addressId, freeInput = None):
    address = getAddress(addressId, freeInput)
    return formatAddressInt(address)


def findHouseId(address):
    # Необходимость обрезания number обусловлена тем, что если подать
    # слишком большую строку, то сравнение будет неудачно и на след. шаге будет
    # добавлена лишняя запись в AddressHouse.
    # Я не могу получать длины полей из БД. Сейчас (Qt 4.8.6) у меня метод
    # QSqlField.length() для MySQL VARCHAR(m) возвращает m*3.
    # Кто знает, что будет возвращаться в другой версии Qt или на другой платформе?
    truncate = lambda val, len: unicode(val)[:len] if val is not None else ''
    db = QtGui.qApp.db
    tableHouse = db.table('AddressHouse')
    filter = [ tableHouse['KLADRCode'].eq(address['KLADRCode']),
               tableHouse['KLADRStreetCode'].eq(address['KLADRStreetCode']),
               tableHouse['number'].eq(truncate(address['number'], 8)),
               tableHouse['corpus'].eq(truncate(address['corpus'], 8)),
               tableHouse['deleted'].eq(0),
             ]
    houseIdList = db.getIdList(tableHouse, 'id', filter, 'id', 1)
    return houseIdList[0] if houseIdList else None


def getHouseId(address):
    houseId = findHouseId(address)
    if not houseId:
        db = QtGui.qApp.db
        tableHouse = db.table('AddressHouse')
        houseRecord = db.record('AddressHouse')
        houseRecord.setValue('KLADRCode',  toVariant(address['KLADRCode']))
        houseRecord.setValue('KLADRStreetCode',  toVariant(address['KLADRStreetCode']))
        houseRecord.setValue('number',  toVariant(address['number']))
        houseRecord.setValue('corpus',  toVariant(address['corpus']))
        houseId = db.insertRecord(tableHouse, houseRecord)
    return houseId


def findAddressIdEx(address):
    db = QtGui.qApp.db
    houseId = findHouseId(address)
    if houseId:
        tableAddress = db.table('Address')
        filter = [tableAddress['house_id'].eq(houseId),
                  tableAddress['flat'].eq(address['flat']),
                  tableAddress['deleted'].eq(0)]
        addressIdList = db.getIdList(tableAddress, 'id', filter)
        addressId = addressIdList[0] if addressIdList else None
        return houseId, addressId
    else:
        return None, None


def findAddressId(address):
    return findAddressIdEx(address)[1]


def getAddressId(address):
    houseId, addressId = findAddressIdEx(address)
    if not addressId:
        db = QtGui.qApp.db
        houseId = houseId if houseId else getHouseId(address)
        addressRecord = db.record('Address')
        addressRecord.setValue('house_id',  toVariant(houseId))
        addressRecord.setValue('flat',      toVariant(address['flat']))
        tableAddress = db.table('Address')
        addressId = db.insertRecord(tableAddress, addressRecord)
    return addressId


def getClientDocument(clientId):
    filter = '''Tmp.documentType_id IN (SELECT rbDocumentType.id FROM rbDocumentType LEFT JOIN rbDocumentTypeGroup ON rbDocumentTypeGroup.id=rbDocumentType.group_id WHERE rbDocumentTypeGroup.code = '1')'''
    return selectLatestRecord('ClientDocument', clientId, filter)


def formatDocument(documentTypeId, serial, number):
    db = QtGui.qApp.db
    result = u'%s %s %s' % ( forceString(db.translate('rbDocumentType', 'id', documentTypeId, 'name')),
                             forceString(serial),
                             forceString(number)
                           )
    return result.strip()


def getClientPolicyEx(clientId, isCompulsory, date=None):
    db = QtGui.qApp.db
    if date:
        funcCall = 'getClientPolicyIdForDate(%d,%d,%s)' % (clientId, 1 if isCompulsory else 0, db.formatDate(date))
    else:
        funcCall = 'getClientPolicyId(%d,%d)' % (clientId, 1 if isCompulsory else 0)

    stmt = 'SELECT ClientPolicy.*, Organisation.compulsoryServiceStop, Organisation.voluntaryServiceStop, Organisation.area ' \
           ' FROM ClientPolicy'                                                                            \
           ' LEFT JOIN Organisation ON Organisation.id = ClientPolicy.insurer_id'                          \
           ' WHERE ClientPolicy.id = (select %s)' % funcCall
    query = db.query(stmt)
    if query.next():
        return query.record()
    else:
        return None


def getClientCompulsoryPolicy(clientId, date=None):
    return getClientPolicyEx(clientId, True, date)


def getClientVoluntaryPolicy(clientId, date=None):
    return getClientPolicyEx(clientId, False, date)


getClientPolicy = getClientCompulsoryPolicy


def formatPolicy(insurerId, serial, number, begDate=None, endDate=None, name=None,  note=None, policyKindId=None):
    policyKindId = forceRef(policyKindId)
    result = (('[' + CPolicyKindCache.getCode(policyKindId) + ']') if policyKindId  else '')+ u': '
    result += forceString(serial) + ' ' + forceString(number)

    insurerId = forceRef(insurerId)
    nameParts = []
    if insurerId:
        nameParts.append(getOrganisationShortName(insurerId))
    name  = forceStringEx(name)
    if name:
        nameParts.append(name)
    note = forceStringEx(note)
    if note:
        nameParts.append('('+note+')')
    insurerName = ' '.join(nameParts)
    if insurerName:
        result = result +u' выдан '+insurerName

    dateRange = ''
    if begDate:
        dateRange = u'с ' + forceString(begDate)
    if endDate:
        dateRange += u' по ' + forceString(endDate)
    if dateRange:
        result += u' действителен '+dateRange.strip()
    return result


def getClientWork(clientId):
    return selectLatestRecord('ClientWork', clientId)


def getAttachRecord(clientId, temporary):
    clientAttach = selectLatestRecord(('ClientAttach', 'ClientAttach AS Tmp JOIN rbAttachType ON Tmp.attachType_id=rbAttachType.id'), clientId, 'rbAttachType.temporary %s \'0\'' % ('!=' if temporary else '='))
    if clientAttach:
        attachTypeId = forceRef(clientAttach.value('attachType_id'))
        attachType = QtGui.qApp.db.getRecord('rbAttachType', '*', attachTypeId)
        result = {
                  'attachTypeId'    : attachTypeId,
                  'LPU_id'          : forceRef(clientAttach.value('LPU_id')),
                  'orgStructure_id' : forceRef(clientAttach.value('orgStructure_id')),
                  'begDate'         : forceDate(clientAttach.value('begDate')),
                  'endDate'         : forceDate(clientAttach.value('endDate')),
                  'code'            : forceString(attachType.value('code')),
                  'name'            : forceString(attachType.value('name')),
                  'temporary'       : forceBool(attachType.value('temporary')),
                  'outcome'         : forceBool(attachType.value('outcome')),
                  'finance_id'      : forceRef(attachType.value('finance_id')),
                  'document_id'     : forceRef(clientAttach.value('document_id')),
                  'isStatement'     : forceInt(clientAttach.value('isStatement'))
                 }
        if result['outcome']:
            result['date'] = result['endDate']
        else:
            result['date'] = result['begDate']
        return result
    else:
        return None


def getClientAttachEx(clientId, isTemporary, date=None):
    db = QtGui.qApp.db
    if date:
        funcCall = 'getClientAttachIdForDate(%d,%d,%s)' % (clientId, 1 if isTemporary else 0, db.formatDate(date))
    else:
        funcCall = 'getClientAttachId(%d,%d)' % (clientId, 1 if isTemporary else 0)

    stmt = 'SELECT ClientAttach.attachType_id,'                                 \
           ' ClientAttach.LPU_id, ClientAttach.orgStructure_id,'                \
           ' ClientAttach.begDate, ClientAttach.endDate,'                       \
           ' ClientAttach.document_id,'                                         \
           ' rbAttachType.code, rbAttachType.name, rbAttachType.temporary,'     \
           ' rbAttachType.outcome, rbAttachType.finance_id'                     \
           ' FROM ClientAttach'                                                 \
           ' LEFT JOIN rbAttachType ON rbAttachType.id=ClientAttach.attachType_id'\
           ' WHERE ClientAttach.id = (select %s)' % funcCall
    query = db.query(stmt)
    if query.next():
        record = query.record()
        result = {
                  'attachTypeId'    : forceRef(record.value('attachType_id')),
                  'LPU_id'          : forceRef(record.value('LPU_id')),
                  'orgStructure_id' : forceRef(record.value('orgStructure_id')),
                  'begDate'         : forceDate(record.value('begDate')),
                  'endDate'         : forceDate(record.value('endDate')),
                  'code'            : forceString(record.value('code')),
                  'name'            : forceString(record.value('name')),
                  'temporary'       : forceBool(record.value('temporary')),
                  'outcome'         : forceBool(record.value('outcome')),
                  'finance_id'      : forceRef(record.value('finance_id')),
                  'document_id'     : forceRef(record.value('document_id'))
                 }
        result['date'] = result['endDate' if result['outcome'] else 'begDate']
        return result
    else:
        return None


def getClientAttaches(clientId, date=None):
    temporary = getClientAttachEx(clientId, True,  date)
    permanent = getClientAttachEx(clientId, False, date)
    result = []
    if temporary:
        result.append(temporary)
    if permanent:
        result.append(permanent)
    result.sort(key=lambda record: record['date'])
    return result


def getClientSocStatuses(clientId):
    db = QtGui.qApp.db
    table = db.table('ClientSocStatus')
    cond  = [ table['client_id'].eq(clientId),
              table['deleted'].eq(0),
              db.joinOr([table['begDate'].isNull(), table['begDate'].le(QDate.currentDate())]),
              db.joinOr([table['endDate'].isNull(), table['endDate'].ge(QDate.currentDate())]),
            ]
    return db.getDistinctIdList(table, idCol=['socStatusType_id', 'begDate'], where=cond, order='begDate')


def getClientSocStatusIds(clientId):
    db = QtGui.qApp.db
    table = db.table('ClientSocStatus')
    cond  = [ table['client_id'].eq(clientId),
              table['deleted'].eq(0),
              db.joinOr([table['begDate'].isNull(), table['begDate'].le(QDate.currentDate())]),
              db.joinOr([table['endDate'].isNull(), table['endDate'].ge(QDate.currentDate())]),
            ]
    return db.getIdList(table, idCol=['id', 'begDate'], where=cond, order='begDate')


def getClientPhones(clientId):
    db = QtGui.qApp.db
    table = db.table('ClientContact')
    cond  = []
    cond.append(table['client_id'].eq(clientId))
    cond.append(table['deleted'].eq(0))
    tableContactTypes = db.table('rbContactType')
    queryTable = table.leftJoin(tableContactTypes, tableContactTypes['id'].eq(table['contactType_id']))
    records = db.getRecordList(queryTable, [tableContactTypes['name'], table['contact'], table['notes']], cond, [tableContactTypes['code'].name(), table['id'].name()])
    result = []
    for record in records:
        typeName = forceString(record.value(0))
        contact  = forceString(record.value(1))
        notes    = forceString(record.value(2))
        result.append((typeName, contact, notes))
    return result


def formatClientPhones(phones):
    return ', '.join([(phone[0]+': '+phone[1]+' ('+phone[2]+')') if phone[2] else (phone[0]+': '+phone[1]) for phone in phones])


def getClientPhonesEx(clientId):
    return formatClientPhones(getClientPhones(clientId))

def getClientDocumentLocation(clientId):
    db = QtGui.qApp.db
    table = db.table('Client_DocumentTracking')
    cond  = []
    cond.append(table['client_id'].eq(clientId))
    cond.append(table['deleted'].eq(0))
    tableClientDocumentTrackingItem = db.table('Client_DocumentTrackingItem')
    tableRBDocumentTypeForTracking = db.table('rbDocumentTypeForTracking')
    tableRBDocumentTypeLocation = db.table('rbDocumentTypeLocation')
    queryTable = table.leftJoin(tableRBDocumentTypeForTracking, tableRBDocumentTypeForTracking['id'].eq(table['documentTypeForTracking_id']))
    queryTable = queryTable.leftJoin(tableClientDocumentTrackingItem, tableClientDocumentTrackingItem['master_id'].eq(table['id']))
    queryTable = queryTable.leftJoin(tableRBDocumentTypeLocation, tableRBDocumentTypeLocation['id'].eq(tableClientDocumentTrackingItem['documentLocation_id']))
    cond.append(tableRBDocumentTypeForTracking['showInClientInfo'].eq(1))
    record = db.getRecordEx(queryTable, [tableRBDocumentTypeForTracking['name'], tableRBDocumentTypeLocation['name'], tableRBDocumentTypeLocation['color']], cond, 'concat_ws(\' \', Client_DocumentTrackingItem.documentLocationDate, Client_DocumentTrackingItem.documentLocationTime) DESC')
    result = []
    if record:
        documentTypeName = forceString(record.value(0))
        documentLocationName  = forceString(record.value(1))
        color = forceString(record.value(2))
        result.extend((documentTypeName, documentLocationName, color))
    return result



def getActualClientQuoting(clientId):
    date = QDate.currentDate()
    return getClientQuoting(clientId, date)


def getClientQuoting(clientId, date):
    db = QtGui.qApp.db
    table = db.table('Client_Quoting')
    tableQuotaType = db.table('QuotaType')
    queryTable = table.innerJoin(tableQuotaType, tableQuotaType['id'].eq(table['quotaType_id']))
    cond = [table['master_id'].eq(clientId),
            table['deleted'].eq(0)]
    if date:
        cond.extend([table['dateRegistration'].dateLe(date),
                     db.joinOr([
                                table['dateEnd'].dateGe(date),
                                table['dateEnd'].isNull()
                               ])
                    ])
    fields = [tableQuotaType['class'].name(),
              tableQuotaType['code'].name(),
              table['quotaTicket'].name(),
              table['status'].name()]
    recordList = db.getRecordList(queryTable,fields, cond)
    result = []
    for record in recordList:
        class_ = getQuotaTypeClassNameList()[forceInt(record.value('class'))]
        code   = forceString(record.value('code'))
        quotaTicket = forceString(record.value('quotaTicket'))
        status = [u'Отменено',
                  u'Ожидание',
                  u'Активный талон',
                  u'Талон для заполнения',
                  u'Заблокированный талон',
                  u'Отказано',
                  u'Необходимо согласовать дату обслуживания',
                  u'Дата обслуживания на согласовании',
                  u'Дата обслуживания согласована',
                  u'Пролечен',
                  u'Обслуживание отложено',
                  u'Отказ пациента',
                  u'Импортировано из ВТМП'][forceInt(record.value('status'))]
        result.append((class_, code, quotaTicket, status))
    return result


def formatClientQuoting(clientQuotingList):
    result = []
    for clientQuoting in clientQuotingList:
        values = {'class'       : clientQuoting[0],
                  'code'        : clientQuoting[1],
                  'quotaTicket' : clientQuoting[2],
                  'status'      : clientQuoting[3]}
        result.append(u'класс-%(class)s, код-%(code)s, талон-%(quotaTicket)s, состояние-%(status)s'% values)
    return u' | '.join(result)


def formatWork(workRecord):
    orgId = forceRef(workRecord.value('org_id'))
    if orgId:
        orgShortName = getOrganisationShortName(orgId)
    else:
        orgShortName = forceString(workRecord.value('freeInput'))
    post = forceString(workRecord.value('post'))
    OKVED = forceString(workRecord.value('OKVED'))
    result = []
    if orgShortName:
        result.append(orgShortName)
    if post:
        result.append(post)
    if OKVED:
        result.append(u'ОКВЭД: '+OKVED)
    return trim(', '.join(result))


def formatWorkPlace(workRecord):
    orgId = forceRef(workRecord.value('org_id'))
    if orgId:
        return getOrganisationShortName(orgId)
    else:
        return forceString(workRecord.value('freeInput'))


def workRecordIsNotEmpty(workRecord):
    return forceRef(workRecord.value('org_id')) or bool(forceString(workRecord.value('freeInput')))


def workRecordDescribesStudent(workRecord):
    post = forceString(workRecord.value('post')).lower()
    return post.startswith(u'учащ') or post.startswith(u'студ')


def clientIsWorking(clientId):
    workRecord = getClientWork(clientId)
    return workRecordIsNotEmpty(workRecord) if workRecord else False


def clientIsStudent(clientId):
    workRecord = getClientWork(clientId)
    return workRecordDescribesStudent(workRecord) if workRecord else False


def getSocStatusTypeClasses(id):
    db = QtGui.qApp.db
    table = db.table('rbSocStatusClassTypeAssoc')
    cond  = table['type_id'].eq(id)
    return db.getIdList(table, idCol='class_id', where=cond, order='class_id')


def formatClientIdentification(clientIdentification):
    return '. '.join(['%s: <B>%s</B>' % (key,  val[0]) for key,  val in clientIdentification.items()])


def getClientObservationStatus(clientId):
    if id:
        db = QtGui.qApp.db
        table = db.table('Client_StatusObservation')
        rbTable = db.table('rbStatusObservationClientType')
        record = db.getRecordEx(table.innerJoin(rbTable, table['statusObservationType_id'].eq(rbTable['id'])), [rbTable['code'], rbTable['name'], rbTable['color']], [table['deleted'].eq(0), table['master_id'].eq(clientId)], 'Client_StatusObservation.id DESC')
        if record:
#            code = forceString(record.value('code')) + '-' + forceString(record.value('name'))
            code = forceString(record.value('name'))
            color = forceString(record.value('color'))
            return [code, color]
    return None


def getVaccineInfectionIdList(vaccineId):
    db = QtGui.qApp.db
    table = db.table('rbInfection_rbVaccine')
    return db.getDistinctIdList(table, 'infection_id', where=table['vaccine_id'].eq(vaccineId))


def getClientVaccinationIdList(clientId):
    db = QtGui.qApp.db
    table = db.table('ClientVaccination')
    return db.getIdList(table, where=table['client_id'].eq(clientId))


def getClientConsentsEx(clientId, date):
    result = {}
    maxList = {}
    clientConsentRecordList = getClientConsents(clientId, date)
    for clientConsentRecord in clientConsentRecordList:
        consentTypeCode = forceString(clientConsentRecord.value('consentTypeCode'))
        clientConsentDate = forceDate(clientConsentRecord.value('date'))
        currMax = maxList.get(consentTypeCode, QDate())
        if clientConsentDate > currMax:
            maxList[consentTypeCode] = clientConsentDate
            result[consentTypeCode] = clientConsentRecord
    return result.values()


def getClientConsents(clientId, date):
    if clientId:
        db = QtGui.qApp.db
        tableCC = db.table('ClientConsent')
        tableCCT = db.table('rbClientConsentType')
        queryTable = tableCC.leftJoin(tableCCT, tableCCT['id'].eq(tableCC['clientConsentType_id']))
        cond = [tableCC['client_id'].eq(clientId), tableCC['deleted'].eq(0), tableCCT['inClientInfoBrowser'].eq(1)]
        if date:
            cond.append(tableCC['date'].dateLe(date))
            cond.append(db.joinOr([tableCC['endDate'].dateGe(date), tableCC['endDate'].isNull()]))
        fields = [tableCCT['code'].alias('consentTypeCode'),
                  tableCCT['name'].alias('consentTypeName'),
                  tableCC['value'].name(),
                  tableCC['endDate'].name(),
                  tableCC['date'].name(),
                  tableCC['id'].alias('clientConsentId'),
                  tableCC['representerClient_id'].alias('representerClientId')]
        return db.getRecordList(queryTable, fields, cond)
    return []


def formatClientConsents(clientConsentRecordList):
    result = []
    currentDate = QDate.currentDate()
    for clientConsentRecord in clientConsentRecordList:
        code = forceString(clientConsentRecord.value('consentTypeCode'))
#        name = forceString(clientConsentRecord.value('consentTypeName'))
        value = forceInt(clientConsentRecord.value('value'))
        endDate = forceDate(clientConsentRecord.value('endDate'))
        date = forceString(clientConsentRecord.value('date'))

        value = u'Да' if value else u'Нет'

        if not endDate or (endDate and endDate>= currentDate):
            result.append(u'%s - <B><font color=green>%s</font></B>(%s)'%(code, value, date))
        else:
            result.append(u'%s - <B><font color=red>%s</font></B>(%s)'%(code, value, date))
    return ' | '.join(result)


def getClientContingentTypeId(clientId):
    # Пол, Возраст и соц.статус имеют приоритет перед событиями и МЭС
    result = None
    query = QtGui.qApp.db.query('SELECT getClientContingentTypeId(%d)'%clientId)
    if query.next():
        result = forceRef(query.value(0))
    return result


def getContingentHasEventTypeCond(contingentTypeId):
    return forceBool(QtGui.qApp.db.translate('rbContingentType_EventType', 'master_id', contingentTypeId, 'id'))


def formatClientContingentEventType(clientId, contingentTypeId):
    contingentHasEventTypeCond = getContingentHasEventTypeCond(contingentTypeId)
    if contingentHasEventTypeCond:
        query = QtGui.qApp.db.query('SELECT checkClientContingentTypeETM(%d, %d)'%(clientId, contingentTypeId))
        if query.next():
            eventTypeCondValue = forceInt(query.value(0))
        else:
            eventTypeCondValue = 0
    else:
        eventTypeCondValue = 0
    return eventTypeCondValue, contingentHasEventTypeCond


def getContingentHasActionTypeCond(contingentTypeId):
    return forceBool(QtGui.qApp.db.translate('rbContingentType_ActionType', 'master_id', contingentTypeId, 'id'))


def formatClientContingentActionType(clientId, contingentTypeId):
    contingentHasActionTypeCond = getContingentHasActionTypeCond(contingentTypeId)
    if contingentHasActionTypeCond:
        query = QtGui.qApp.db.query('SELECT checkClientContingentTypeEAT(%d, %d)'%(clientId, contingentTypeId))
        if query.next():
            actionTypeCondValue = forceInt(query.value(0))
        else:
            actionTypeCondValue = 0
    else:
        actionTypeCondValue = 0
    return actionTypeCondValue, contingentHasActionTypeCond


def formatClientContingentType(clientId, contingentTypeId):
    if not contingentTypeId:
        return u' ', u''
    eventTypeCondValue, contingentHasEventTypeCond = formatClientContingentEventType(clientId, contingentTypeId)
    actionTypeCondValue, contingentHasActionTypeCond = formatClientContingentActionType(clientId, contingentTypeId)
    typeCondValue = 0
    contingentHasTypeCond = False
    if contingentHasEventTypeCond and contingentHasActionTypeCond:
        if eventTypeCondValue <= actionTypeCondValue:
            typeCondValue = eventTypeCondValue
            contingentHasTypeCond = contingentHasEventTypeCond
        elif eventTypeCondValue > actionTypeCondValue:
            typeCondValue = actionTypeCondValue
            contingentHasTypeCond = contingentHasActionTypeCond
    elif contingentHasEventTypeCond:
        typeCondValue = eventTypeCondValue
        contingentHasTypeCond = contingentHasEventTypeCond
    elif contingentHasActionTypeCond:
        typeCondValue = actionTypeCondValue
        contingentHasTypeCond = contingentHasActionTypeCond
#    синий - попадает в контингент в котором нет требований для Событий
#    красный - попадает в контингент и не имеет требуемых Событий
#    желтый - попадает в контингент, имеет Открытое Событие
#    зеленый - попадает в контингент, имеет Закрытое Событие
    color = {0 : u'#0000CD', 1: u'#FF0000', 2: u'#CC9900', 3: u'#00FF00'}.get(typeCondValue+contingentHasTypeCond, u'#000000')
    contingentTypeCode = forceString(QtGui.qApp.db.translate('rbContingentType', 'id', contingentTypeId, 'code'))
    return contingentTypeCode, color


def getClientInfo(clientId, hasRegAddress=True, hasLocAddress=True, date=None):
    db = QtGui.qApp.db
    table  = db.table('Client')
    record = db.getRecord(table, '*', clientId)
    result = smartDict()
    result.id          = forceRef(clientId)
    result.lastName    = forceString(record.value('lastName'))
    result.firstName   = forceString(record.value('firstName'))
    result.patrName    = forceString(record.value('patrName'))
    result.sexCode     = forceInt(record.value('sex'))
    result.birthDate   = forceDate(record.value('birthDate'))
    result.deathDate   = forceDate(record.value('deathDate'))
    result.birthPlace  = forceString(record.value('birthPlace'))
    result.SNILS       = forceString(record.value('SNILS'))
    result.notes       = forceString(record.value('notes'))
    result.attaches    = getClientAttaches(clientId)
    result.socStatuses = getClientSocStatuses(clientId)
    result.quoting     = getActualClientQuoting(clientId)
    result.identification = formatClientIdentification(getClientIdentification(clientId))
    result.clientDocumentLocation = getClientDocumentLocation(clientId)
    result.clientObservationStatus = getClientObservationStatus(clientId)
    result.clientConsents = getClientConsentsEx(clientId, date)
    result.clientContingentTypeId = getClientContingentTypeId(clientId)
    result.contacts = getClientPhones(clientId)

    documentRecord = getClientDocument(clientId)
    result.documentRecord = documentRecord
    if documentRecord:
        result.document  = formatDocument(documentRecord.value('documentType_id'),
                                             documentRecord.value('serial'),
                                             documentRecord.value('number'))
    else:
        result.document = None

    policyRecord = getClientCompulsoryPolicy(clientId, date)
    result.compulsoryPolicyRecord = policyRecord
    result.compulsoryPolicy = u''
    result.voluntaryPolicy = u''
    colorOMC = u'green'
    colorDMC = u'blue'
    currentDate = forceDate(date) if date else QDate.currentDate()
    if policyRecord:
        result.policy = formatPolicy( policyRecord.value('insurer_id'),
                                      policyRecord.value('serial'),
                                      policyRecord.value('number'),
                                      policyRecord.value('begDate'),
                                      policyRecord.value('endDate'),
                                      policyRecord.value('name'),
                                      policyRecord.value('note'),
                                      policyRecord.value('policyKind_id')
                                    )
        compulsoryPolicyServiceOMC = forceBool(policyRecord.value('compulsoryServiceStop'))
        colorOMC = u'green' if not compulsoryPolicyServiceOMC else u'red'
        endDateOMC = forceDate(policyRecord.value('endDate'))
        if endDateOMC and endDateOMC < currentDate:
            colorOMC = u'red'
        result.compulsoryPolicy = result.policy
    policyRecord = getClientVoluntaryPolicy(clientId, date)
    result.voluntaryPolicyRecord = policyRecord
    if policyRecord:
        result.voluntaryPolicy = formatPolicy(policyRecord.value('insurer_id'),
                                              policyRecord.value('serial'),
                                              policyRecord.value('number'),
                                              policyRecord.value('begDate'),
                                              policyRecord.value('endDate'),
                                              policyRecord.value('name'),
                                              policyRecord.value('note'),
                                              policyRecord.value('policyKind_id')
                                             )
        voluntaryPolicyServiceDMC = forceBool(policyRecord.value('voluntaryServiceStop'))
        colorDMC = u'blue' if not voluntaryPolicyServiceDMC else u'red'
        endDateDMC = forceDate(policyRecord.value('endDate'))
        if endDateDMC and endDateDMC < currentDate:
            colorDMC = u'red'
    result.compulsoryvoluntaryPolicy = u'''<B><font color=%s>%s</font></B>%s<B><font color=%s>%s</font></B>'''%(colorOMC, result.compulsoryPolicy, (u', полис ДМС' if result.voluntaryPolicy else ''), colorDMC, result.voluntaryPolicy)

    if hasRegAddress:
        regAddressRecord = getClientAddress(clientId, 0)
        if regAddressRecord:
            result.regAddressInfo = getAddress(regAddressRecord.value('address_id'), regAddressRecord.value('freeInput'))
            result.regAddress     = formatAddressInt(result['regAddressInfo'], 0)
            # WTF district?
            # district = getClientDistrict(forceRef(regAddressRecord.value('district')))
#            if district:
#                result['regAddress'] += u' %s окр.' % district
            district = getClientDistrict(result.regAddressInfo.KLADRCode, result.regAddressInfo.KLADRStreetCode, result.regAddressInfo.number)
            if district:
                result['regAddress'] += u' %s район.' % district

    if hasLocAddress:
        locAddressRecord = getClientAddress(clientId, 1)
        if locAddressRecord:
            result.locAddressInfo = getAddress(locAddressRecord.value('address_id'), locAddressRecord.value('freeInput'))
            result.locAddress     = formatAddressInt(result.locAddressInfo, 1)
            # WTF district?
            district = getClientDistrict(result.locAddressInfo.KLADRCode, result.locAddressInfo.KLADRStreetCode, result.locAddressInfo.number)
            if district:
                result['locAddress'] += u' %s район.' % district

    workRecord = getClientWork(clientId)
    if workRecord:
        result.work = formatWork(workRecord)
    return result


def getClientInfoEx(clientId, date=None):
  #TODO: сделать в CClientInfo
    clientInfo = getClientInfo(clientId)
    clientInfo.fullName = formatNameInt(clientInfo.lastName, clientInfo.firstName, clientInfo.patrName)
    clientInfo.shortName = formatShortNameInt(clientInfo.lastName, clientInfo.firstName, clientInfo.patrName)

    birthDate = clientInfo.birthDate
    clientInfo.birthDateStr = formatDate(birthDate)
    clientInfo.age = calcAge(clientInfo.birthDate, date)
    clientInfo.sex = formatSex(clientInfo.sexCode)
    clientInfo.SNILS = formatSNILS(clientInfo.SNILS)

    clientInfo.setdefault('document', u'нет')
    clientInfo.setdefault('compulsoryPolicy', u'нет')
    clientInfo.setdefault('voluntaryPolicy', u'нет')
    clientInfo.policy = clientInfo.compulsoryPolicy
    clientInfo.socStatus = formatSocStatuses(clientInfo.get('socStatuses', []))
    clientInfo.setdefault('regAddress', u'не указан')
    clientInfo.setdefault('locAddress', u'не указан')
    clientInfo.setdefault('work', u'не указано')
#    clientInfo['id'] = clientId
    clientInfo.phones = formatClientPhones(clientInfo.contacts)
    return clientInfo


def formatAttachAsHTML(attach,  atDate):
    td = forceDate(atDate) if atDate else QDate.currentDate()
    bold = False
    txt = attach['name']
    orgStructureCode = u' (' + forceString(QtGui.qApp.db.translate('OrgStructure', 'id', attach['orgStructure_id'], 'code')) + u')'
    if attach['outcome']:
        color = 'red'
        txt = txt + ' ' + forceString(attach['begDate'])
    elif attach['temporary']:
        color = 'blue'
        bold = True
        txt = txt + ' ' + getOrganisationShortName(attach['LPU_id']) + orgStructureCode
        if attach['begDate'].isValid():
            txt = txt + u' с ' + forceString(attach['begDate'])
            color = 'blue' if attach['begDate'] <= td else 'red'
        if attach['endDate'].isValid():
            txt = txt + u' по ' + forceString(attach['endDate'])
            color = 'blue' if attach['endDate'] >= td else 'red'
    else:
        txt = txt + ' ' + getOrganisationShortName(attach['LPU_id']) + orgStructureCode
        if attach['endDate'].isValid():
            color = 'red' if attach['endDate'] <= td else 'green'
        else:
            color= 'green'
    return u'<font color="%s">%s%s%s</font>' % (color,  '<B>' if bold else '',  txt,  '</B>' if bold else '')


def formatAttachesAsHTML(attaches,  atDate):
    return ', '.join([formatAttachAsHTML(x,  atDate) for x in attaches])


def formatAttach(attach,  atDate):
    td = forceDate(atDate) if atDate else QDate.currentDate()
    txt = attach['name']
    orgStructureCode = u' (' + forceString(QtGui.qApp.db.translate('OrgStructure', 'id', attach['orgStructure_id'], 'code')) + u')'
    if attach['outcome']:
        txt = txt + ' ' + forceString(attach['begDate'])
    elif attach['temporary']:
        txt = txt + ' ' + getOrganisationShortName(attach['LPU_id']) + orgStructureCode
        if attach['begDate'].isValid():
            txt = txt + u' с ' + forceString(attach['begDate'])
        if attach['endDate'].isValid():
            txt = txt + u' по ' + forceString(attach['endDate'])
    else:
        txt = txt + ' ' + getOrganisationShortName(attach['LPU_id']) + orgStructureCode
    return txt


def formatAttaches(attaches,  atDate):
    return ', '.join([formatAttach(x,  atDate) for x in attaches])


def formatSocStatuses(socStatuses, asHtml=False):
    db = QtGui.qApp.db
    tableSSC = db.table('rbSocStatusClass')
    tableSSCT = db.table('rbSocStatusClassTypeAssoc')
    table = tableSSC.innerJoin(tableSSCT, tableSSCT['class_id'].eq(tableSSC['id']))
    sstIdList = db.getDistinctIdList(table, [tableSSCT['type_id']], [tableSSC['code'].eq('8')])
    if socStatuses:
        if QtGui.qApp.showingInInfoBlockSocStatus() == 0:
            if asHtml:
                lines = [(('<B>' + CSocStatusTypeCache.getCode(socStatusTypeId) + u'</B>') if socStatusTypeId not in sstIdList
                                                            else CSocStatusTypeCache.getCode(socStatusTypeId)) for socStatusTypeId in socStatuses]
            else:
                lines = [CSocStatusTypeCache.getCode(socStatusTypeId) for socStatusTypeId in socStatuses]
        elif QtGui.qApp.showingInInfoBlockSocStatus() == 1:
            if asHtml:
                lines = [((u'<B>' + CSocStatusTypeCache.getName(socStatusTypeId) + u'</B>') if (socStatusTypeId in sstIdList and CSocStatusTypeCache.getCode(socStatusTypeId) != u'м643')
                                                            else CSocStatusTypeCache.getName(socStatusTypeId)) for socStatusTypeId in socStatuses]
            else:
                lines = [CSocStatusTypeCache.getName(socStatusTypeId) for socStatusTypeId in socStatuses]
        else:
            if asHtml:
                lines = [((('<B>' + CSocStatusTypeCache.getCode(socStatusTypeId) + u'</B>-') if socStatusTypeId not in sstIdList
                                                            else (CSocStatusTypeCache.getCode(socStatusTypeId) + '-'))
                                + ((u'<B>' + CSocStatusTypeCache.getName(socStatusTypeId) + u'</B>') if (socStatusTypeId in sstIdList and CSocStatusTypeCache.getCode(socStatusTypeId) != u'м643')
                                                            else CSocStatusTypeCache.getName(socStatusTypeId))) for socStatusTypeId in socStatuses]
            else:
                lines = [CSocStatusTypeCache.getCode(socStatusTypeId) + '-' + CSocStatusTypeCache.getName(socStatusTypeId) for socStatusTypeId in socStatuses]
        lines.sort()
        return ', '.join(lines)
    else:
        return u'нет'


def getClientMiniInfo(clientId, atDate=None):
    db = QtGui.qApp.db
    record = db.getRecord('Client', ['lastName', 'firstName', 'patrName', 'birthDate', 'sex'], clientId)
    if record:
        name  = formatName(record.value('lastName'), record.value('firstName'), record.value('patrName'))
        birthDate = forceDate(record.value('birthDate'))
        birthDateStr = formatDate(birthDate)
        age       = calcAge(birthDate, atDate)
        sex       = formatSex(forceInt(record.value('sex')))
        return u'%s, %s (%s) %s' % (name, birthDateStr, age, sex)
    else:
        return ''


def getClientBanner(clientId, atDate=None):
    info = getClientInfo(clientId, date=atDate)
    return formatClientBanner(info, atDate)


def getClientString(clientId, atDate=None):
    info = getClientInfo(clientId)
    return formatClientString(info, atDate)


def clientIdToText(clientId):
    text = ''
    if clientId:
        db = QtGui.qApp.db
        tableClient = db.table('Client')
        tableDocument = db.table('ClientDocument')
        tableAddress = db.table('ClientAddress')
        tableDocumentType = db.table('rbDocumentType')
        cols = [tableClient['id'],
                tableClient['lastName'],
                tableClient['firstName'],
                tableClient['patrName'],
                tableClient['birthDate'],
                tableClient['sex'],
                tableDocument['serial'],
                tableDocument['number'],
                tableDocument['date'],
                tableDocument['origin'],
                tableDocumentType['name']
                ]
        cols.append(u'''IF(ClientAddress.type = 0, concat(_utf8'Адрес регистрации: ', ClientAddress.freeInput), _utf8'') AS regAddress, IF(ClientAddress.type = 1, concat(_utf8'Адрес проживания: ', ClientAddress.freeInput), _utf8'') AS logAddress''')
        queryTable = tableClient.leftJoin(tableAddress, tableClient['id'].eq(tableAddress['client_id']))
        queryTable = queryTable.leftJoin(tableDocument, tableClient['id'].eq(tableDocument['client_id']))
        queryTable = queryTable.leftJoin(tableDocumentType, tableDocument['documentType_id'].eq(tableDocumentType['id']))
        cond = [tableClient['id'].eq(clientId),
                tableClient['deleted'].eq(0)
               ]
        cond.append(db.joinOr([tableDocument['id'].isNull(), tableDocument['deleted'].eq(0)]))
        cond.append(db.joinOr([tableAddress['id'].isNull(), tableAddress['deleted'].eq(0)]))
        record = db.getRecordEx(queryTable, cols, cond)
        if record:
            clientId = forceString(record.value('id'))
            lastName = forceString(record.value('lastName'))
            firstName = forceString(record.value('firstName'))
            patrName = forceString(record.value('patrName'))
            birthDate = forceString(record.value('birthDate'))
            sex = formatSex(forceInt(record.value('sex')))
            regAddress = forceString(record.value('regAddress'))
            logAddress = forceString(record.value('logAddress'))
            docName = forceString(record.value('name'))
            serialDoc = forceString(record.value('serial'))
            numberDoc = forceString(record.value('number'))
            originDoc = forceString(record.value('origin'))
            dateDoc = forceString(record.value('date'))
            FIO = ' '.join([lastName, firstName, patrName])
            sexStr = ': '.join([u'пол', sex])
            docSerialNumb = ' '.join([u' серия', serialDoc])
            docSerialNumb += ' '.join([u' номер', numberDoc])
            infoDoc = ': '.join([docName, docSerialNumb])
            infoDocOrg = ': '.join([u' выдан', originDoc])
            infoDocDate = ' '.join([dateDoc])
            infoDoc += ' '.join([infoDocOrg, infoDocDate])
            text = ', '.join([field for field in (FIO, clientId, birthDate, sexStr, regAddress, logAddress, infoDoc) if field])
    else:
        text = ''
    return text


#def getClientHospitalBeds(clientId = None):
#    result = [u'', 0]
#    if clientId:
#        currentDate = QDate.currentDate()
#        db = QtGui.qApp.db
#        tableAPHB = db.table('ActionProperty_HospitalBed')
#        tableAPT = db.table('ActionPropertyType')
#        tableAP = db.table('ActionProperty')
#        tableActionType = db.table('ActionType')
#        tableAction = db.table('Action')
#        tableEvent = db.table('Event')
#        tableOSHB = db.table('OrgStructure_HospitalBed')
#        tableOS = db.table('OrgStructure')
#        tableHBP = db.table('rbHospitalBedProfile')
#        cols = [tableAction['id'],
#                tableEvent['id'].alias('eventId'),
#                tableEvent['client_id'],
#                tableActionType['flatCode'],
#                tableAction['begDate'],
#                tableAction['endDate'],
#                tableOS['name'].alias('nameOS'),
#                tableOSHB['code'].alias('codeBed'),
#                tableOSHB['name'].alias('nameBed'),
#                tableHBP['code'].alias('codeBedProfile'),
#                tableHBP['name'].alias('nameBedProfile'),
#                ]
#        queryTable = tableActionType.innerJoin(tableAction, tableActionType['id'].eq(tableAction['actionType_id']))
#        queryTable = queryTable.innerJoin(tableEvent, tableAction['event_id'].eq(tableEvent['id']))
#        queryTable = queryTable.innerJoin(tableAPT, tableAPT['actionType_id'].eq(tableActionType['id']))
#        queryTable = queryTable.innerJoin(tableAP, tableAP['type_id'].eq(tableAPT['id']))
#        queryTable = queryTable.innerJoin(tableAPHB, tableAPHB['id'].eq(tableAP['id']))
#        queryTable = queryTable.innerJoin(tableOSHB, tableOSHB['id'].eq(tableAPHB['value']))
#        queryTable = queryTable.innerJoin(tableHBP, tableHBP['id'].eq(tableOSHB['profile_id']))
#        queryTable = queryTable.innerJoin(tableOS, tableOS['id'].eq(tableOSHB['master_id']))
#        cond = [ tableEvent['deleted'].eq(0),
#                 tableAction['deleted'].eq(0),
#                 tableAP['deleted'].eq(0),
#                 tableActionType['deleted'].eq(0),
#                 tableOS['deleted'].eq(0),
#                 tableEvent['client_id'].eq(clientId),
#                 tableEvent['setDate'].le(currentDate),
#                 tableAction['begDate'].le(currentDate),
#                 tableAPT['typeName'].like('HospitalBed'),
#                 tableAP['action_id'].eq(tableAction['id'])
#               ]
#        cond.append(db.joinOr([tableEvent['execDate'].isNull(), tableEvent['execDate'].ge(currentDate)]))
#        cond.append(db.joinOr([tableAction['endDate'].isNull(), tableAction['endDate'].ge(currentDate)]))
#        cond.append(db.joinOr([tableActionType['flatCode'].like('moving'), tableActionType['flatCode'].like('planning')]))
#        records =  db.getRecordList(queryTable, cols, cond)
#        for record in records:
#            nameOS = forceString(record.value('nameOS'))
#            codeBed = forceString(record.value('codeBed'))
#            nameBed = forceString(record.value('nameBed'))
#            flatCode = forceString(record.value('flatCode'))
#            codeBedProfile = forceString(record.value('codeBedProfile'))
#            nameBedProfile = forceString(record.value('nameBedProfile'))
#            resultStr = u'подразделение: ' + nameOS + u', ' + codeBed + u'-' + nameBed + u'(' + codeBedProfile + u'-' + nameBedProfile + u')'
#            if u'moving' in flatCode.lower():
#               resultFlatCode = 1
#            elif u'planning' in flatCode.lower():
#               resultFlatCode = 0
#            result = [resultStr, resultFlatCode]
#        return result


def getClientHospitalOrgStructureAndBedRecords(clientId):
    result = [None, None]
    if clientId:
        db = QtGui.qApp.db
        #tableAPHB = db.table('ActionProperty_HospitalBed')
        tableAPT = db.table('ActionPropertyType')
        tableAP = db.table('ActionProperty')
        tableActionType = db.table('ActionType')
        tableAction = db.table('Action')
        tableEvent = db.table('Event')
        tableOSHB = db.table('OrgStructure_HospitalBed')
        tableOS = db.table('OrgStructure')
        tableHBP = db.table('rbHospitalBedProfile')
        #tableAPOS = db.table('ActionProperty_OrgStructure')
        currentDate = QDate.currentDate()
        cols = [tableAction['id'],
                tableEvent['id'].alias('eventId'),
                tableEvent['client_id'],
                tableActionType['flatCode'],
                tableAction['begDate'],
                tableAction['endDate']
                ]
        condBeds = [tableAP['deleted'].eq(0),
                    tableActionType['deleted'].eq(0),
                    tableAPT['deleted'].eq(0),
                    tableAPT['typeName'].like('HospitalBed')
                   ]
        condBeds.append('A.deleted = 0')
        condBeds.append('A.id = Action.id')
        condBeds.append('ActionProperty.action_id=A.id')
        condOS = [tableAP['deleted'].eq(0),
                  tableActionType['deleted'].eq(0),
                  tableAPT['deleted'].eq(0)
                 ]
        condOS.append('A.id = Action.id')
        condOS.append('A.deleted = 0')
        condOS.append('ActionProperty.action_id=A.id')
        condOS.append('ActionProperty.type_id=ActionPropertyType.id')
        condOS.append('''(ActionProperty.id IN (SELECT DISTINCT APHB.id FROM ActionProperty_HospitalBed AS APHB) AND NOT(SELECT DISTINCT APHB.value !=0 FROM ActionProperty_HospitalBed AS APHB WHERE APHB.id = ActionProperty.id LIMIT 1)) OR (ActionProperty.id NOT IN (SELECT DISTINCT APHB.id FROM ActionProperty_HospitalBed AS APHB))''')
        condOS.append(db.joinOr([tableAPT['name'].like(u'Отделение пребывания'), tableAPT['typeName'].like('HospitalBed')]))
        cols.append('''(SELECT ActionProperty_HospitalBed.value
        FROM ActionType
        INNER JOIN Action AS A ON ActionType.id=A.actionType_id
        INNER JOIN ActionPropertyType ON ActionPropertyType.actionType_id=ActionType.id
        INNER JOIN ActionProperty ON ActionProperty.type_id=ActionPropertyType.id
        INNER JOIN ActionProperty_HospitalBed ON ActionProperty_HospitalBed.id=ActionProperty.id
        WHERE %s
        LIMIT 1) AS bedId''' % (db.joinAnd(condBeds)))
        cols.append('''(SELECT ActionProperty_OrgStructure.value
        FROM ActionType
        INNER JOIN Action AS A ON ActionType.id=A.actionType_id
        INNER JOIN ActionPropertyType ON ActionPropertyType.actionType_id=ActionType.id
        INNER JOIN ActionProperty ON ActionProperty.type_id=ActionPropertyType.id
        INNER JOIN ActionProperty_OrgStructure ON ActionProperty_OrgStructure.id=ActionProperty.id
        WHERE %s
        LIMIT 1) AS orgStructureId ''' % (db.joinAnd(condOS)))

        queryTable = tableActionType.innerJoin(tableAction, tableActionType['id'].eq(tableAction['actionType_id']))
        queryTable = queryTable.innerJoin(tableEvent, tableAction['event_id'].eq(tableEvent['id']))
        queryTable = queryTable.innerJoin(tableAPT, tableAPT['actionType_id'].eq(tableActionType['id']))
        queryTable = queryTable.innerJoin(tableAP, tableAP['type_id'].eq(tableAPT['id']))
        cond = [ tableEvent['deleted'].eq(0),
             tableAction['deleted'].eq(0),
             tableAP['deleted'].eq(0),
             tableActionType['deleted'].eq(0),
             tableAPT['deleted'].eq(0),
             tableEvent['client_id'].eq(clientId),
             tableEvent['setDate'].le(currentDate),
             tableAction['begDate'].le(currentDate),
             tableAP['action_id'].eq(tableAction['id'])
            ]
        cond.append(db.joinOr([tableEvent['execDate'].isNull(), tableEvent['execDate'].ge(currentDate)]))
        cond.append(db.joinOr([tableAction['endDate'].isNull(), tableAction['endDate'].ge(currentDate)]))
        cond.append(db.joinOr([tableActionType['flatCode'].like('moving'), tableActionType['flatCode'].like('planning')]))
        stmt = db.selectStmtGroupBy(queryTable, cols, cond, u'Action.id, eventId, Event.client_id, ActionType.flatcode, Action.begdate, Action.enddate')
        stmt = 'select query.*, OrgStructure.name as orgStructureName from (%s) as query left join OrgStructure on OrgStructure.id = query.orgStructureId'%stmt
        #recordsMoving = db.getRecordListGroupBy(queryTable, cols, cond, u'Action.id')
        query = db.query(stmt)
        while query.next():
        #for record in recordsMoving:
            record = query.record()
            result = [record, None]
            bedId = forceRef(record.value('bedId'))
            queryTableBed = tableOSHB.innerJoin(tableHBP, tableHBP['id'].eq(tableOSHB['profile_id']))
            queryTableBed = queryTableBed.innerJoin(tableOS, tableOS['id'].eq(tableOSHB['master_id']))
            colsBed = [ tableOS['name'].alias('nameOS'),
                    tableOS['code'].alias('codeOS'),
                    tableOSHB['code'].alias('codeBed'),
                    tableOSHB['name'].alias('nameBed'),
                    tableHBP['code'].alias('codeBedProfile'),
                    tableHBP['name'].alias('nameBedProfile')
                    ]
            recordsBed = db.getRecordListGroupBy(queryTableBed, colsBed, [tableOSHB['id'].eq(bedId)], u'OrgStructure_HospitalBed.id, nameOS, codeOS, codeBedProfile, nameBedProfile')
            for recordBed in recordsBed:
                result = [record, recordBed]
    return result


def getClientHospitalOrgStructureAndBeds(clientId):
    result = [u'', 0]
    resultStr = ''
    flatCode = ''
    resultFlatCode = -1
    [recordMoving, recordBed] = getClientHospitalOrgStructureAndBedRecords(clientId)
    if recordMoving:
#        bedId = forceRef(recordMoving.value('bedId'))
        orgStructureName = forceString(recordMoving.value('orgStructureName'))
        flatCode = forceString(recordMoving.value('flatCode'))
        resultStr = (u' отделение : %s;' % orgStructureName) if orgStructureName else u''
    if recordBed:
#        nameOS = forceString(recordBed.value('nameOS'))
        codeOS = forceString(recordBed.value('codeOS'))
        codeBed = forceString(recordBed.value('codeBed'))
#        nameBed = forceString(recordBed.value('nameBed'))
        codeBedProfile = forceString(recordBed.value('codeBedProfile'))
        nameBedProfile = forceString(recordBed.value('nameBedProfile'))
        resultStr += u' койка: %s, %s (%s-%s)' % ( codeOS, codeBed, codeBedProfile, nameBedProfile)
    if u'moving' in flatCode.lower():
        resultFlatCode = 1
    elif u'planning' in flatCode.lower():
        resultFlatCode = 0
    result = [resultStr, resultFlatCode]
    return result


def formatClientBanner(info, atDate=None):
    id        = info.id
    name      = formatName(info.lastName, info.firstName, info.patrName)
    birthDate = formatDate(info.birthDate)
    deathDate = ''
    if info.deathDate:
        deathDate = u' <font color=red>Умер %s</font>'%formatDate(info.deathDate)
    birthPlace= info.birthPlace
    age       = calcAge(info.birthDate, info.deathDate if info.deathDate else atDate)
    sex       = formatSex(info.sexCode)
    SNILS     = formatSNILS(info.SNILS)
    attaches  = info.get('attaches', [])
    socStatuses = info.get('socStatuses', [])

    document                  = info.get('document', u'нет')
#    compulsoryPolicy          = info.get('compulsoryPolicy', u'нет')
#    voluntaryPolicy           = info.get('voluntaryPolicy', u'')
    compulsoryvoluntaryPolicy = info.get('compulsoryvoluntaryPolicy', u'нет')
    regAddress                = info.get('regAddress', u'не указан')
    locAddress                = info.get('locAddress', u'не указан')
    work                      = info.get('work', u'не указано')
    notes                     = info.get('notes', u'')
    identification            = info.get('identification',  u'')
    quoting                   = formatClientQuoting(info.get('quoting', []))
#    contacts                  = getClientPhones(id)
    phones                    = getClientPhonesEx(id)
    documentLocation = getClientDocumentLocation(id)
    clientObservationStatus   = info.get('clientObservationStatus', u'')
    hospitalBed, busy         = getClientHospitalOrgStructureAndBeds(id)
    clientConsents            = formatClientConsents(info.get('clientConsents', []))
    clientContingentTypeCode, clientContingentTypeColor = formatClientContingentType(id, info.clientContingentTypeId)
    clientContingent = u' <B><font color="%s">[%s]</font></B>' % (clientContingentTypeColor, clientContingentTypeCode)
    if hospitalBed:
        bed = u' госпитализация: <B><font color=%s>%s</font></B>' % ( 'green' if busy else 'blue',  hospitalBed)
    else:
        bed = ''
    bannerDocumentLocation = ''
    bannerStatusObservation = ''
    if documentLocation:
        bannerDocumentLocation = u''' [%s: <B><font color=%s>%s</font></B>]'''%(documentLocation[0], documentLocation[2], documentLocation[1])
    if clientObservationStatus and len(clientObservationStatus) == 2:
        bannerStatusObservation = u''' [Статус: <B><font color=%s>%s</font></B>] '''%(clientObservationStatus[1], clientObservationStatus[0])

    bannerHTML=u'''
             <B>%s</B>, дата рождения: <B>%s%s</B> (%s) пол: <B>%s</B> код: <B><font color=blue>%s</font></B>%s%s%s статус: %s<BR>
             %s%s%s%sСНИЛС: <B>%s</B>, прикрепление:%s<BR>
             Документ:<B>%s</B>, полис ОМС %s<BR>
             Адрес регистрации:<B>%s</B>
             Адрес проживания:<B>%s</B><BR>
             Занятость:<B>%s</B><BR>
             ''' % (name, birthDate, deathDate, age, sex, id,
                    bannerDocumentLocation,
                    bannerStatusObservation,
                    bed,
                    formatSocStatuses(socStatuses, True),
                    u'Согласия: <B>%s</B><BR>' % clientConsents if clientConsents else u'',
                    ((u'%s<BR>' % identification) if identification else ''),
                    u'<I>Квоты:</I> %s<BR>' % quoting if quoting else '', clientContingent,
                    SNILS, formatAttachesAsHTML(attaches, atDate),
                    document, compulsoryvoluntaryPolicy,
                    regAddress,
                    locAddress,
                    work)
    if phones:
        bannerHTML+=u'Телефоны:<B>%s</B><BR>' % phones
    if birthPlace:
        bannerHTML+=u'Место рождения: <B>%s</B><BR>' % birthPlace
    if notes:
        bannerHTML+=u'Примечания:<B>%s</B><BR>' % notes
    try:
        TFAccountingSystemId = QtGui.qApp.TFAccountingSystemId()
        if TFAccountingSystemId:
            tableClientIdentification=QtGui.qApp.db.table('ClientIdentification')
            identDate=QtGui.qApp.db.getRecordEx(
                tableClientIdentification,
                'max(checkDate)',
                'client_id=%d and accountingSystem_id=%d' % (id, TFAccountingSystemId))
            if identDate:
                checkDate=forceDate(identDate.value(0))
                if checkDate:
                    bannerHTML+=u'Дата подтверждения ТФОМС: <B>%s</B><BR>' % forceString(checkDate)
    except:
        QtGui.qApp.logCurrentException()

    clientBanner = u'<HTML><BODY>%s</BODY></HTML>' % bannerHTML
    return clientBanner


def formatClientString(info, atDate=None):
    id        = info.id
    name      = formatName(info.lastName, info.firstName, info.patrName)
    birthDate = formatDate(info.birthDate)
    birthPlace= info.birthPlace
    age       = calcAge(info.birthDate, atDate)
    sex       = formatSex(info.sexCode)
    SNILS     = formatSNILS(info.SNILS)
    attaches  = info.get('attaches', [])
    socStatuses = info.get('socStatuses', [])

    document  = info.get('document', u'нет')
    compulsoryPolicy = info.get('compulsoryPolicy', u'нет')
    voluntaryPolicy  = info.get('voluntaryPolicy', u'')
    regAddress = info.get('regAddress', u'не указан')
    locAddress = info.get('locAddress', u'не указан')
    work       = info.get('work', u'не указано')
    notes      = info.get('notes', u'')
    identification = info.get('identification',  u'')
    phones     = getClientPhonesEx(id)
    hospitalBed, busy = getClientHospitalOrgStructureAndBeds(id)
    if hospitalBed:
        bed = u' госпитализация: %s %s' % (busy, hospitalBed)
    else:
        bed = ''

    bannerHTML=u'''%s, дата рождения: %s (%s) пол: %s код: %s %s статус: %s %s СНИЛС: %s, прикрепление: %s Документ: %s, полис ОМС: %s %s %s Адрес регистрации: %s Адрес проживания: %s Занятость: %s''' % (name, birthDate, age, sex, id, bed, formatSocStatuses(socStatuses),
                    ((u' %s' % identification) if identification else ''),
                    SNILS, formatAttaches(attaches,  atDate),
                    document, compulsoryPolicy, (u', полис ДМС: ' if voluntaryPolicy else ''), voluntaryPolicy,
                    regAddress,
                    locAddress,
                    work)

    if phones:
        bannerHTML+=u'Телефоны: %s' % phones
    if birthPlace:
        bannerHTML+=u'Место рождения: %s' % birthPlace
    if notes:
        bannerHTML+=u'Примечания: %s' % notes

    try:
        TFAccountingSystemId = QtGui.qApp.TFAccountingSystemId()
        if TFAccountingSystemId:
            tableClientIdentification=QtGui.qApp.db.table('ClientIdentification')
            identDate=QtGui.qApp.db.getRecordEx(
                tableClientIdentification,
                'max(checkDate)',
                'client_id= %d and accountingSystem_id = %d' % (id, TFAccountingSystemId))
            if identDate:
                checkDate=forceDate(identDate.value(0))
                if checkDate:
                    bannerHTML+=u'Дата подтверждения ТФОМС: %s' % forceString(checkDate)
    except:
        QtGui.qApp.logCurrentException()

    clientBanner = u'%s' % bannerHTML
    return clientBanner


def getClientContextData(clientId):
    from Events.TempInvalidInfo import CTempInvalidInfoList
    clientInfo = getClientInfo2(clientId)
    getTempInvalidList = lambda begDate=None, endDate=None, types=None: CTempInvalidInfoList._get(clientInfo.context, clientId, begDate, endDate, types)
    data = {'client':clientInfo,
            'getTempInvalidList': getTempInvalidList,
            'tempInvalids':getTempInvalidList()}
    return data

# проверка правильности ввода "номера единого полиса" (см. приложение к задаче №0004219)
def unitedPolicyIsVaid(number):
    return (len(number) == 16
            and number.isdigit()
            and str(-sum(int(d) for d in number[-3::-2]+str(int(number[-2::-2])*2)) % 10) == number[-1])

######################################################
# mixin для проверки применимости врача/подразделения
# к данному пациенту
######################################################

class CCheckNetMixin:
    def __init__(self):
        self.reset()
        self.connect(QtGui.qApp, SIGNAL('dbConnectionChanged(bool)'), self.onConnectionChanged)


    def reset(self):
        self.mapSpecialityIdToSpecialityConstraint = {}
        self.mapPersonIdToSpecialityId = {}
        self.mapPersonIdToOrgId = {}
        self.mapPersonIdToOrgStructureId = {}
        self.mapOrgStructureIdToNetId = {}
        self.mapNetIdToNet = {}


    def onConnectionChanged(self, state):
        if not state:
            self.reset()


    def _getPersonIds(self, personId):
        db = QtGui.qApp.db
        record = db.getRecord('Person', 'speciality_id, org_id, orgStructure_id', personId)
        if record:
            specialityId = forceRef(record.value(0))
            orgId = forceRef(record.value(1))
            orgStructureId = forceRef(record.value(2))
        else:
            specialityId = None
            orgId = None
            orgStructureId = None
        self.mapPersonIdToSpecialityId[personId] = specialityId
        self.mapPersonIdToOrgId[personId] = orgId
        self.mapPersonIdToOrgStructureId[personId] = orgStructureId


    def getPersonSpecialityId(self, personId):
        if personId not in self.mapPersonIdToOrgStructureId:
            self._getPersonIds(personId)
        return self.mapPersonIdToSpecialityId[personId]


    def getSpecialityConstraint(self, specialityId):
        if specialityId in self.mapSpecialityIdToSpecialityConstraint:
            return self.mapSpecialityIdToSpecialityConstraint[specialityId]
        else:
            specialityConstraint = CSpecialityConstraint(specialityId)
            self.mapSpecialityIdToSpecialityConstraint[specialityId] = specialityConstraint
            return specialityConstraint


    def getPersonOrgId(self, personId):
        if personId not in self.mapPersonIdToOrgId:
            self._getPersonIds(personId)
        return self.mapPersonIdToOrgId[personId]


    def getPersonOrgStructureId(self, personId):
        if personId not in self.mapPersonIdToOrgStructureId:
            self._getPersonIds(personId)
        return self.mapPersonIdToOrgStructureId[personId]


    def getOrgStructureNetId(self, orgStructureId):
        if orgStructureId in self.mapOrgStructureIdToNetId:
            return self.mapOrgStructureIdToNetId[orgStructureId]
        else:
            netId = getOrgStructureNetId(orgStructureId)
            self.mapOrgStructureIdToNetId[orgStructureId] = netId
            return netId


    def getNet(self, netId):
        if netId in self.mapNetIdToNet:
            return self.mapNetIdToNet[netId]
        else:
            net = CNet(netId)
            self.mapNetIdToNet[netId] = net
            return net


    def getOrgStructureNet(self, orgStructureId):
        return self.getNet(self.getOrgStructureNetId(orgStructureId))


    def getPersonSpecialityConstraint(self, personId):
        return self.getSpecialityConstraint(self.getPersonSpecialityId(personId))


    def getPersonNet(self, personId):
        return self.getNet(self.getOrgStructureNetId(self.getPersonOrgStructureId(personId)))


    def getClientSexAndAge(self, clientId):
        db = QtGui.qApp.db
        table  = db.table('Client')
        record = db.getRecord(table, 'sex, birthDate', clientId)
        if record:
            clientSex = forceInt(record.value('sex'))
            clientBirthDate = forceDate(record.value('birthDate'))
            clientAge       = calcAgeTuple(clientBirthDate, QDate.currentDate())
            return clientSex, clientAge
        else:
            return None, None


#    def checkNetApplicable(self, net, clientId):
#        if net.sex or net.age:
#            clientSex, clientAge = self.getClientSexAndAge(clientId)
#            return not clientSex or net.applicable(clientSex, clientAge)
#        return True


#    def checkOrgStructureNetApplicable(self, orgStructureId, clientId):
#        return self.checkNetApplicable(self.getOrgStructureNet(orgStructureId), clientId)


    def checkClientAttendance(self, personId, clientId):
        net = self.getPersonNet(personId)
        specialityConstraint = self.getPersonSpecialityConstraint(personId)
        if net.constrain() or specialityConstraint.constrain():
            clientSex, clientAge = self.getClientSexAndAge(clientId)
            return specialityConstraint.applicable(clientSex, clientAge) and net.applicable(clientSex, clientAge)
        return True


    def checkClientAttendanceEx(self, personId, clientSex, clientAge):
        net = self.getPersonNet(personId)
        specialityConstraint = self.getPersonSpecialityConstraint(personId)
        if net.constrain() or specialityConstraint.constrain():
            return specialityConstraint.applicable(clientSex, clientAge) and net.applicable(clientSex, clientAge)
        return True


    def confirmClientAttendance(self, widget, personId, clientId):
        message = u'Пациент не относится к лицам, обслуживаемым указанным врачом.\nВсё равно продолжить?'
        return QtGui.QMessageBox.critical(widget,
                                    u'Внимание!',
                                    message,
                                    QtGui.QMessageBox.Yes|QtGui.QMessageBox.No,
                                    QtGui.QMessageBox.No) == QtGui.QMessageBox.Yes


    def checkClientAttach(self, personId, clientId, date, boolCreateOrder = False):
        def _isAttachActive(attach):
            if not attach:
                return False
            return not(attach['outcome']
                       or attach['endDate']         and (forceDate(date) if date else QDate.currentDate()) > attach['endDate']
                       or attach['orgStructure_id'] and self.getPersonOrgStructureId(personId) not in getOrgStructureDescendants(attach['orgStructure_id'])
                       or attach['LPU_id']          and self.getPersonOrgId(personId) != attach['LPU_id']
                      )

        if QtGui.qApp.isStrictAttachCheckOnEventCreation() != 2:
            isStrict = QtGui.qApp.isStrictAttachCheckOnEventCreation()
            temporary = getAttachRecord(clientId, True)
            if isStrict and not (_isAttachActive(temporary) or _isAttachActive(getAttachRecord(clientId, False))):
                QtGui.QMessageBox.critical(self,
                                            u'Внимание!',
                                            u'У этого пациента нет подходящего прикрепления',
                                            QtGui.QMessageBox.Ok,
                                            QtGui.QMessageBox.Ok)
                return False

            if temporary:
                attachTypeId = temporary.get('attachTypeId', None)
                code = temporary.get('code', u'')
                name = temporary.get('name', u'')
                if attachTypeId:
                    db = QtGui.qApp.db
                    tablePerson = db.table('Person')
                    #tableOrgStructure = db.table('OrgStructure')
                    tableOSDA = db.table('OrgStructure_DisabledAttendance')
                    table = tablePerson.innerJoin(tableOSDA, tableOSDA['master_id'].eq(tablePerson['orgStructure_id']))
                    record = db.getRecordEx(table, [tableOSDA['disabledType']], [tablePerson['deleted'].eq(0), tablePerson['id'].eq(personId), tableOSDA['attachType_id'].eq(attachTypeId)])
                    if record:
                        disabledType = forceInt(record.value('disabledType'))
                        if not isStrict and (disabledType == 0 or (disabledType == 1 and not boolCreateOrder)):
                            buttons = QtGui.QMessageBox.Yes | QtGui.QMessageBox.No
                            message = u'Запрещено обслуживание врачом по прикреплению: %s.\nВсё равно продолжить?'%(code + u'-' + name)
                            buttonsFocus = QtGui.QMessageBox.No
                            buttonsResult = QtGui.QMessageBox.Yes
                        else:
                            buttons = QtGui.QMessageBox.Ok
                            message = u'Запрещено обслуживание врачом по прикреплению: %s.\n'%(code + u'-' + name)
                            buttonsFocus = QtGui.QMessageBox.Ok
                            buttonsResult = None
                        return QtGui.QMessageBox.critical(self,
                                            u'Внимание!',
                                            message,
                                            buttons,
                                            buttonsFocus) == buttonsResult
        return True


    def confirmClientPolicyConstraint(self, widget, clientId):
        if QtGui.qApp.isStrictPolicyCheckOnEventCreation() != 2:
            compulsoryServiceStop = 0
            voluntaryServiceStop = 0
            recordCompulsoryPolicy = getClientCompulsoryPolicy(clientId)
            if recordCompulsoryPolicy:
                compulsoryServiceStop = forceBool(recordCompulsoryPolicy.value('compulsoryServiceStop'))
            recordVoluntaryPolicy = getClientVoluntaryPolicy(clientId)
            if recordVoluntaryPolicy:
                voluntaryServiceStop = forceBool(recordVoluntaryPolicy.value('voluntaryServiceStop'))
            if compulsoryServiceStop or voluntaryServiceStop:
                if compulsoryServiceStop:
                    messageFinance = u'ОМС'
                elif voluntaryServiceStop:
                    messageFinance =  u'ДМС'
                message = u'По данной СМО приостановлено обслуживание %s полисов.\nЭто может привести к затруднениям оплаты обращения.\nВсё равно продолжить?'%(messageFinance)
                return QtGui.QMessageBox.critical(widget,
                                            u'Внимание!',
                                            message,
                                            QtGui.QMessageBox.Yes|QtGui.QMessageBox.No,
                                            QtGui.QMessageBox.No) == QtGui.QMessageBox.Yes
        return True


class CSpecialityConstraint(CSexAgeConstraint):
    def __init__(self, id):
        CSexAgeConstraint.__init__(self)
        db = QtGui.qApp.db
        table = db.table('rbSpeciality')
        record = db.getRecord(table, 'sex, age', id)
        if record:
            self.initByRecord(record)
        else:
            self.code = None
            self.name = None

##################################################
# ClientInfo как объект: требуется для
# печати форм и т.п.
##################################################


class CClientInfo(CInfo):
    def __init__(self, context, id, date = None):
        CInfo.__init__(self, context)
        self._id = id
        self._date = date
        self._lastName  = ''
        self._firstName = ''
        self._patrName  = ''

        self._sexCode   = -1
        self._sex       = ''
        self._birthDate = CDateInfo(None)
        self._birthTime = CTimeInfo(None)
        self._deathDate = CDateTimeInfo(None)
        self._birthPlace= ''
        self._ageTuple  = None
        self._age       = ''
        self._SNILS     = ''
        self._notes     = ''
        self._permanentAttach = self.getInstance(CClientAttachInfo, None, False)
        self._temporaryAttach = self.getInstance(CClientAttachInfo, None, True)
        self._socStatuses = self.getInstance(CClientSocStatusInfoList, None)
        self._document  = self.getInstance(CClientDocumentInfo, clientId=None)
        self._compulsoryPolicy = None
        self._voluntaryPolicy  = None
        self._regAddress = None
        self._locAddress = None
        self._work       = None
        self._phones     = ''
        self._contacts   = ''
        self._bloodType  = ''
        self._bloodDate  = CDateInfo(None)
        self._bloodNotes = ''
        self._intolerances = self.getInstance(CClientIntoleranceMedicamentInfoList, None)
        self._allergies    = self.getInstance(CClientAllergyInfoList, None)
        self._identification = self.getInstance(CClientIdentificationInfo, None)
        self._relations    = self.getInstance(CClientRelationInfoList, None, date)
        self._quotas       = self.getInstance(CClientQuotaInfoList, None)
        self._consents     = self.getInstance(CClientConsentInfoList, None, date)
        self._vaccinations = self.getInstance(CClientVaccinationInfoList, None)
        self._birthHeight     = 0
        self._birthWeight     = 0
        self._birthGestationalAge  = 0
        self._statusObservation = self.getInstance(CClientStatusObservationInfo, None)

        self._createDatetime = CDateTimeInfo(None)
        self._createPerson = self.getInstance(CPersonInfo, None)
        self._modifyDatetime = CDateTimeInfo(None)
        self._modifyPerson = self.getInstance(CPersonInfo, None)

    def _load(self):
        db = QtGui.qApp.db
        table  = db.table('Client')
        if self._id:
            record = db.getRecord(table, '*', self._id)
        else:
            record = None
        if record:
            #self.id        = forceString(record.value('id'))
            self._lastName  = forceString(record.value('lastName'))
            self._firstName = forceString(record.value('firstName'))
            self._patrName  = forceString(record.value('patrName'))

            self._sexCode   = forceInt(record.value('sex'))
            self._sex       = formatSex(self._sexCode)
            self._birthDate = CDateInfo(record.value('birthDate'))
            self._birthTime = CTimeInfo(forceTime(record.value('birthTime')))
            self._deathDate = CDateTimeInfo(record.value('deathDate'))
            self._birthPlace= forceString(record.value('birthPlace'))
            self._ageTuple  = calcAgeTuple(self._birthDate.date, self._date)
            self._age       = formatAgeTuple(self._ageTuple, self._birthDate.date, self._date)
            self._SNILS     = formatSNILS(forceString(record.value('SNILS')))
            self._notes     = forceString(record.value('notes'))
            self._permanentAttach = self.getInstance(CClientAttachInfo, self._id, False)
            self._temporaryAttach = self.getInstance(CClientAttachInfo, self._id, True)
            self._socStatuses = self.getInstance(CClientSocStatusInfoList, self._id)
            self._document  = self.getInstance(CClientDocumentInfo, clientId=self._id)
            self._compulsoryPolicy = self.getInstance(CClientPolicyInfo, self._id, True)
            self._voluntaryPolicy  = self.getInstance(CClientPolicyInfo, self._id, False)
            self._regAddress = self.getInstance(CClientAddressInfo, self._id, 0)
            self._locAddress = self.getInstance(CClientAddressInfo, self._id, 1)
            self._work       = self.getInstance(CClientWorkInfo, self._id)
            self._contacts   = getClientPhones(self._id)
            self._phones     = getClientPhonesEx(self._id)
            self._bloodType  = forceString(db.translate('rbBloodType', 'id', forceRef(record.value('bloodType_id')), 'name'))
            self._bloodDate  = CDateInfo(record.value('bloodDate'))
            self._bloodNotes     = forceString(record.value('bloodNotes'))
            self._intolerances = self.getInstance(CClientIntoleranceMedicamentInfoList, self._id)
            self._allergies    = self.getInstance(CClientAllergyInfoList, self._id)
            self._identification = self.getInstance(CClientIdentificationInfo, self._id)
            self._relations    = self.getInstance(CClientRelationInfoList, self._id, self._date)
            self._quotas       = self.getInstance(CClientQuotaInfoList, self._id)
            self._consents     = self.getInstance(CClientConsentInfoList, self._id, self._date)
            self._vaccinations = self.getInstance(CClientVaccinationInfoList, self._id)
            self._birthHeight     = forceInt(record.value('birthHeight'))
            self._birthWeight     = forceInt(record.value('birthWeight'))
            self._birthGestationalAge     = forceInt(record.value('birthGestationalAge'))
            self._statusObservation = self.getInstance(CClientStatusObservationInfo, self._id)
            self._createDatetime = CDateTimeInfo(record.value('createDatetime'))
            self._createPerson = self.getInstance(CPersonInfo, forceRef(record.value('createPerson_id')))
            self._modifyDatetime = CDateTimeInfo(record.value('modifyDatetime'))
            self._modifyPerson = self.getInstance(CPersonInfo, forceRef(record.value('modifyPerson_id')))
            return True
        else:
            return False


    def __str__(self):
        self.load()
        return formatShortNameInt(self.lastName, self.firstName, self.patrName)

    def ageOnDate(self, date):
        if type(date) == CDateInfo:
            date = date.date
        ageTuple = calcAgeTuple(self.load()._birthDate.date, date)
        return formatAgeTuple(ageTuple, self.load()._birthDate.date, date)

    id           = property(lambda self: self.load()._id)
    lastName     = property(lambda self: self.load()._lastName)
    firstName    = property(lambda self: self.load()._firstName)
    patrName     = property(lambda self: self.load()._patrName)
    fullName     = property(lambda self: formatNameInt(self.lastName, self.firstName,self.patrName))
    shortName    = property(lambda self: formatShortNameInt(self.lastName, self.firstName,self.patrName))
    sexCode      = property(lambda self: self.load()._sexCode)
    sex          = property(lambda self: self.load()._sex)
    birthDate    = property(lambda self: self.load()._birthDate)
    birthTime    = property(lambda self: self.load()._birthTime)
    deathDate    = property(lambda self: self.load()._deathDate)
    birthPlace   = property(lambda self: self.load()._birthPlace)
    ageTuple     = property(lambda self: self.load()._ageTuple)
    age          = property(lambda self: self.load()._age)
    SNILS        = property(lambda self: self.load()._SNILS)
    notes        = property(lambda self: self.load()._notes)
    permanentAttach = property(lambda self: self.load()._permanentAttach)
    temporaryAttach = property(lambda self: self.load()._temporaryAttach)
    socStatuses     = property(lambda self: self.load()._socStatuses)
    document        = property(lambda self: self.load()._document)
    compulsoryPolicy= property(lambda self: self.load()._compulsoryPolicy)
    voluntaryPolicy = property(lambda self: self.load()._voluntaryPolicy)
    policy          = compulsoryPolicy
    policyDMS       = voluntaryPolicy
    regAddress      = property(lambda self: self.load()._regAddress)
    locAddress      = property(lambda self: self.load()._locAddress)
    work            = property(lambda self: self.load()._work)
    phones          = property(lambda self: self.load()._phones)
    contacts        = property(lambda self: self.load()._contacts)
    bloodType       = property(lambda self: self.load()._bloodType)
    bloodDate       = property(lambda self: self.load()._bloodDate)
    bloodNotes       = property(lambda self: self.load()._bloodNotes)
    intolerances    = property(lambda self: self.load()._intolerances)
    allergies       = property(lambda self: self.load()._allergies)
    identification  = property(lambda self: self.load()._identification)
    relations       = property(lambda self: self.load()._relations)
    quotas          = property(lambda self: self.load()._quotas)
    consents        = property(lambda self: self.load()._consents)
    vaccinations    = property(lambda self: self.load()._vaccinations)
    birthHeight    = property(lambda self: self.load()._birthHeight)
    birthWeight    = property(lambda self: self.load()._birthWeight)
    birthGestationalAge    = property(lambda self: self.load()._birthGestationalAge)
    statusObservation = property(lambda self: self.load()._statusObservation)
    createDatetime = property(lambda self: self.load()._createDatetime)
    createPerson = property(lambda self: self.load()._createPerson)
    modifyDatetime = property(lambda self: self.load()._modifyDatetime)
    modifyPerson = property(lambda self: self.load()._modifyPerson)


class CClientInfoListEx(CInfoList):
    def __init__(self, context, clientIdList):
        CInfoList.__init__(self, context)
        self.idList = clientIdList


    def _load(self):
        self._items = [ self.getInstance(CClientInfo, id) for id in self.idList ]
        return True


class CRBVaccineSchemaTransitionTypeInfo(CRBInfo):
    tableName = 'rbVaccine_SchemaTransitionType'


class CRBReactionInfo(CRBInfo):
    tableName = 'rbReaction'


class CRBVaccineInfo(CRBInfo):
    tableName = 'rbVaccine'

    def _initByRecord(self, record):
        self._regionalCode = forceString(record.value('regionalCode'))
        self._dose         = forceDouble(record.value('dose'))

    def _initByNull(self):
        self._regionalCode = u''
        self._dose         = 0.0

    regionalCode = property(lambda self: self.load()._regionalCode)
    dose         = property(lambda self: self.load()._dose)


class CRBInfectionInfo(CRBInfo):
    tableName = 'rbInfection'


    def _initByRecord(self, record):
        self._regionalCode = forceString(record.value('regionalCode'))
        self._minimumTerm  = forceString(record.value('minimumTerm'))

    def _initByNull(self):
        self._regionalCode = u''
        self._minimumTerm  = u''


    regionalCode = property(lambda self: self.load()._regionalCode)
    minimumTerm  = property(lambda self: self.load()._minimumTerm)



class CRBInfectionInfoList(CInfoList):
    def __init__(self, context, vaccineId):
        CInfoList.__init__(self, context)
        self._vaccineId = vaccineId
        self._idList = []


    def _load(self):
        vaccineInfectionIdList = getVaccineInfectionIdList(self._vaccineId)
        self._items = [self.getInstance(CRBInfectionInfo, id) for id in vaccineInfectionIdList]
        return True


    def __str__(self):
        self.load()
        if self._items:
            return u', '.join([unicode(item) for item in self._items])
        return u''


class CClientVaccinationInfo(CInfo):
    def __init__(self, context, id):
        CInfo.__init__(self, context)
        self._id = id

    def _load(self):
        table = QtGui.qApp.db.table('ClientVaccination')
        if self._id:
            record = QtGui.qApp.db.getRecord(table, '*', self._id)
            result = True
        else:
            record = table.newRecord()
            result = False

        self._vaccine          = self.getInstance(CRBVaccineInfo, forceRef(record.value('vaccine_id')))
        self._vaccineationType = forceString(record.value('vaccinationType'))
        self._datetime         = CDateTimeInfo(forceDateTime(record.value('datetime')))
        self._dose             = forceDouble(record.value('dose'))
        self._seria            = forceString(record.value('seria'))
        self._person           = self.getInstance(CPersonInfo, forceRef(record.value('person_id')))
        self._reaction         = self.getInstance(CRBReactionInfo, forceRef(record.value('reaction_id')))
        self._transitionType   = self.getInstance(CRBVaccineSchemaTransitionTypeInfo,
                                                  forceRef(record.value('transitionType_id')))
        self._relegateOrg      = self.getInstance(COrgInfo, forceRef(record.value('relegateOrg_id')))
        self._infections       = self.getInstance(CRBInfectionInfoList, forceRef(record.value('vaccine_id')))

        return result

    id               = property(lambda self: self.load()._id)
    vaccine          = property(lambda self: self.load()._vaccine)
    vaccineationType = property(lambda self: self.load()._vaccineationType)
    datetime         = property(lambda self: self.load()._datetime)
    dose             = property(lambda self: self.load()._dose)
    seria            = property(lambda self: self.load()._seria)
    person           = property(lambda self: self.load()._person)
    reaction         = property(lambda self: self.load()._reaction)
    transitionType   = property(lambda self: self.load()._transitionType)
    relegateOrg      = property(lambda self: self.load()._relegateOrg)
    infections       = property(lambda self: self.load()._infections)

    def __str__(self):
        self.load()
        return self._vaccine.__str__()


class CClientVaccinationInfoList(CInfoList):
    def __init__(self, context, clientId):
        CInfoList.__init__(self, context)
        self._clientId = clientId
        self._idList = []

    def _load(self):
        clientVaccinationIdList = getClientVaccinationIdList(self._clientId)
        self._items = [self.getInstance(CClientVaccinationInfo, id) for id in clientVaccinationIdList]
        return True


class CClientConsentInfo(CInfo):
    def __init__(self, context, record):
        CInfo.__init__(self, context)
        self._record = record

    def _load(self):
        if self._record:
            self._id = forceRef(self._record.value('clientConsentId'))
            self._code = forceString(self._record.value('consentTypeCode'))
            self._name = forceString(self._record.value('consentTypeName'))
            self._value = forceInt(self._record.value('value'))
            self._strValue = u'Да' if forceInt(self._record.value('value')) else u'Нет'
            self._date = forceDate(self._record.value('date'))
            self._endDate = forceDate(self._record.value('endDate'))

            self._representerClientId = forceRef(self._record.value('representerClientId'))

            representerRecord = QtGui.qApp.db.getRecord('Client',
                                                        'lastName, firstName, patrName',
                                                        self._representerClientId)
            if representerRecord:
                self._representer = formatShortNameInt(
                                                       forceString(representerRecord.value('lastName')),
                                                       forceString(representerRecord.value('firstName')),
                                                       forceString(representerRecord.value('patrName'))
                                                      )
            else:
                 self._representer = u''
        else:
            self._id = None
            self._code = u''
            self._name = u''
            self._value = None
            self._strValue = u''
            self._date = CDateInfo(None)
            self._endDate = CDateInfo(None)
            self._representerClientId = None
            self._representer = u''

    def __str__(self):
        self.load()
        if self._id:
            return u': '.join([self._code, self._strValue])+forceString(self._date).join(['(', ')'])
        return u''

    id           = property(lambda self: self.load()._id)
    code = property(lambda self: self.load()._code)
    name = property(lambda self: self.load()._name)
    value = property(lambda self: self.load()._value)
    strValue = property(lambda self: self.load()._strValue)
    date = property(lambda self: self.load()._date)
    endDate = property(lambda self: self.load()._endDate)
    representerClientId = property(lambda self: self.load()._representerClientId)
    representer = property(lambda self: self.load()._representer)

class CClientConsentInfoList(CInfoList):
    def __init__(self, context, clientId, date = None):
        CInfoList.__init__(self, context)
        self._clientId = clientId
        self._date = date
        self._idList = []

    def _load(self):
        clientConsentRecordList = getClientConsentsEx(self._clientId, self._date)
        self._items = [self.getInstance(CClientConsentInfo, record) for record in clientConsentRecordList]
        return True

class CClientStatusObservationInfo(CInfo):
    def __init__(self, context, clientId=None):
        CInfo.__init__(self, context)
        self._clientId = clientId
        self._color = u''
        self._name = u''
        self._code = u''

    def _load(self):
        if self._clientId:
            record = self.getClientStatusObservation(self._clientId)
        if record:
            self._code = forceString(record.value('code'))
            self._name = forceString(record.value('name'))
            self._color = forceString(record.value('color'))
            return True
        else:
            return False

    def getClientStatusObservation(self, clientId):
        db = QtGui.qApp.db
        cond=[]
        tableRbStatusObservationClientType = db.table('rbStatusObservationClientType')
        queryTable = db.table('Client_StatusObservation')
        queryTable = queryTable.innerJoin(tableRbStatusObservationClientType, queryTable['statusObservationType_id'].eq(tableRbStatusObservationClientType['id']))
        cond.append(queryTable['deleted'].eq(0))
        cond.append(queryTable['master_id'].eq(clientId))
        orderBy = u'Client_StatusObservation.id DESC'
        record = db.getRecordEx(queryTable, [tableRbStatusObservationClientType['code'], tableRbStatusObservationClientType['name'], tableRbStatusObservationClientType['color']], cond, orderBy)
        return record

    color = property(lambda self: self.load()._color)
    name = property(lambda self: self.load()._name)
    code = property(lambda self: self.load()._code)

class CClientDocumentInfo(CInfo):
    def __init__(self, context, clientId=None, documentId=None):
        CInfo.__init__(self, context)
        self._clientId = clientId
        self._documentId = documentId
        self._documentType = u'-'
        self._documentTypeCode = u'-'
        self._serial = ''
        self._number = ''
        self._date = CDateInfo()
        self._origin = ''

    def _load(self):
        if self._documentId:
            record = QtGui.qApp.db.getRecord('ClientDocument', '*', self._documentId)
        elif self._clientId:
            record = getClientDocument(self._clientId)
        else:
            record = None
        if record:
            documentTypeId = forceRef(record.value('documentType_id'))
            self._documentType = forceString(QtGui.qApp.db.translate('rbDocumentType', 'id', documentTypeId, 'name'))
            self._documentTypeCode = forceString(QtGui.qApp.db.translate('rbDocumentType', 'id', documentTypeId, 'code'))
            self._serial = forceString(record.value('serial'))
            self._number = forceString(record.value('number'))
            self._date = CDateInfo(forceDate(record.value('date')))
            self._origin = forceString(record.value('origin'))
            return True
        else:
            return False


#    def __unicode__(self):
    def __str__(self):
        self.load()
        return (' '.join([self._documentType, self._serial, self._number])).strip()


    documentType     = property(lambda self: self.load()._documentType)
    documentTypeName = property(lambda self: self.load()._documentType)
    documentTypeCode = property(lambda self: self.load()._documentTypeCode)
    type             = property(lambda self: self.load()._documentType)
    serial           = property(lambda self: self.load()._serial)
    number           = property(lambda self: self.load()._number)
    date             = property(lambda self: self.load()._date)
    origin           = property(lambda self: self.load()._origin)


class CRBPolicyTypeInfo(CRBInfo):
    tableName = 'rbPolicyType'


class CRBPolicyKindInfo(CRBInfo):
    tableName = 'rbPolicyKind'


class CClientPolicyInfo(CInfo):
    def __init__(self, context, clientId, isCompulsory=True):
        CInfo.__init__(self, context)
        self.clientId = clientId
        self.isCompulsory = isCompulsory


    def _load(self):
        record = getClientPolicyEx(self.clientId, self.isCompulsory)
        if record:
            self._policyType = self.getInstance(CRBPolicyTypeInfo, forceRef(record.value('policyType_id')))
            self._policyKind = self.getInstance(CRBPolicyKindInfo, forceRef(record.value('policyKind_id')))
            self._insurer = self.getInstance(COrgInfo, forceRef(record.value('insurer_id')))
            self._serial  = forceString(record.value('serial'))
            self._number  = forceString(record.value('number'))
            self._name    = forceString(record.value('name'))
            self._note    = forceString(record.value('note'))
            self._begDate = CDateInfo(record.value('begDate'))
            self._endDate = CDateInfo(record.value('endDate'))
            return True
        else:
            self._policyType = self.getInstance(CRBPolicyTypeInfo, None)
            self._policyKind = self.getInstance(CRBPolicyKindInfo, None)
            self._insurer = self.getInstance(COrgInfo, None)
            self._serial  = ''
            self._number  = ''
            self._name    = ''
            self._note    = ''
            self._begDate = CDateInfo()
            self._endDate = CDateInfo()
            return False


#    def __unicode__(self):
    def __str__(self):
        self.load()
        return (' '.join([self._policyType.name, unicode(self._insurer), self._serial, self._number])).strip()

    kind  = property(lambda self: self.load()._policyKind)
    type        = property(lambda self: self.load()._policyType)
    insurer     = property(lambda self: self.load()._insurer)
    serial      = property(lambda self: self.load()._serial)
    number      = property(lambda self: self.load()._number)
    name        = property(lambda self: self.load()._name)
    note        = property(lambda self: self.load()._note)
#    notes       = note
    begDate     = property(lambda self: self.load()._begDate)
    endDate     = property(lambda self: self.load()._endDate)

class COKATOInfo(CInfo):
    def __init__(self, context, code = None):
        CInfo.__init__(self, context)
        self._code = code
        self._name = None

    def _load(self):
        if self._code:
            db = QtGui.qApp.db
            stmt = 'SELECT `kladr`.`getOKATOName`(%s)' % (decorateString(self._code))
            query = db.query(stmt)
            if query.next():
                self._name = forceString(query.record().value(0))
                return True
        return False

    def __str__(self):
        return self.load()._code

    code = property(lambda self: self.load()._code)
    name = property(lambda self: self.load()._name)


class CAddressInfo(CInfo):
    def __init__(self, context, addressId):
        CInfo.__init__(self, context)
        self._addressId = addressId
        self._KLADRCode = ''
        self._KLADRStreetCode = ''
        self._city = ''
        self._street = ''
        self._number = ''
        self._corpus = ''
        self._flat = ''
        self._text = ''
        self._OKATO = None
        self._district = None
        self._district23 = ''
        self._mainRegion = None


    def _load(self):
        parts = []
        address = getAddress(self._addressId)
        self._KLADRCode = address.KLADRCode
        self._KLADRStreetCode = address.KLADRStreetCode
        if self._KLADRCode:
            self._city = getCityName(self._KLADRCode)
            parts.append(self._city)
        else:
            self._city = ''
        if self._KLADRStreetCode:
            self._street = getStreetName(self._KLADRStreetCode)
            parts.append(self._street)
        else:
            self._street = ''
        self._number = address.number
        self._corpus = address.corpus
        self._flat = address.flat
        if self._number:
            parts.append(u'д.'+self._number)
        if self._corpus:
            parts.append(u'к.'+self._corpus)
        if self._flat:
            parts.append(u'кв.'+self._flat)
        if self._district23:
            parts.append(u'%s район' % self._district23)
        self._text = (', '.join(parts)).strip()
        return bool(self._text)


    def getOKATO(self):
        if self._OKATO is None:
            self.load()
            number = [self._number]
            if self._corpus:
                number.append(self._corpus)
            self._OKATO = self.getInstance(COKATOInfo, getOKATO(self._KLADRCode, self._KLADRStreetCode, u'к'.join(number)))
        return self._OKATO


    def getDistrict(self):
        if self._district is None:
            self.load()
            self._district = getDistrictName(self._KLADRCode, self._KLADRStreetCode, self._number)
        return self._district


    def getMainRegion(self):
        if self._mainRegion is None:
            self.load()
            self._mainRegion = getMainRegionName(self._KLADRCode)
        return self._mainRegion


    KLADRCode       = property(lambda self: self.load()._KLADRCode)
    KLADRStreetCode = property(lambda self: self.load()._KLADRStreetCode)
    city            = property(lambda self: self.load()._city)
    town            = city
    street          = property(lambda self: self.load()._street)
    number          = property(lambda self: self.load()._number)
    corpus          = property(lambda self: self.load()._corpus)
    flat            = property(lambda self: self.load()._flat)
    OKATO           = property(getOKATO)
    district        = property(getDistrict)
    mainRegion         = property(getMainRegion)

#    def __unicode__(self):
    def __str__(self):
        self.load()
        return self._text



class CClientAddressInfo(CAddressInfo):
    def __init__(self, context, clientId, addrType):
        CAddressInfo.__init__(self, context, None)
        self._clientId = clientId
        self._addrType = addrType
        self._freeInput = ''

    def _load(self):
        record = getClientAddress(self._clientId, self._addrType)
        if record:
            self._addressId = record.value('address_id')
            self._freeInput = forceString(record.value('freeInput'))
            # WTF district23?
            #self._district23 = getClientDistrict(forceRef(record.value('district')))
            result = getAddress(self._addressId, self._freeInput)
            self._district23 = getClientDistrict(result.KLADRCode, result.KLADRStreetCode, result.number)
            if self._addressId:
                return CAddressInfo._load(self)
            else:
                return True
        else:
            self._addressId = None
            self._freeInput = ''
            return False

    freeInput  = property(lambda self: self.load()._freeInput)


#    def __unicode__(self):
    def __str__(self):
        self.load()
        if self._addressId and len(self._text):
            return CAddressInfo.__str__(self)
        else:
            return self.freeInput



class CClientWorkInfo(COrgInfo):
    def __init__(self, context, clientId):
        COrgInfo.__init__(self, context, None)
        self.clientId = clientId
        self._post = ''
        self._OKVED = ''
        self._hurts = []


    def _load(self):
        workRecord = getClientWork(self.clientId)
        if workRecord:
#            self.orgId = forceRef(workRecord.value('org_id'))
            self.id = forceRef(workRecord.value('org_id'))
            if self.id:
                COrgInfo._load(self)
            else:
                self._shortName = forceString(workRecord.value('freeInput'))
            self._post = forceString(workRecord.value('post'))
            self._OKVED = forceString(workRecord.value('OKVED'))
            self._hurts = self.getInstance(CClientWorkHurtInfoList, forceRef(workRecord.value('id')))
            return True
        else:
            return False


#    def __unicode__(self):
    def __str__(self):
        self.load()
        parts = []
        if self._shortName:
            parts.append(self._shortName)
        if self._post:
            parts.append(self._post)
        if self._OKVED:
            parts.append(u'ОКВЭД: '+self._OKVED)
        return ', '.join(parts)

    shortName  = property(lambda self: self.load()._shortName)
    post       = property(lambda self: self.load()._post)
    OKVED      = property(lambda self: self.load()._OKVED)
    hurts      = property(lambda self: self.load()._hurts)


class CClientWorkHurtInfoList(CInfoList):
    def __init__(self, context, workId):
        CInfoList.__init__(self, context)
        self.workId = workId

    def _load(self):
        db = QtGui.qApp.db
        table = db.table('ClientWork_Hurt')
        idList = db.getIdList(table, 'id', table['master_id'].eq(self.workId), 'id')
        self._items = [ self.getInstance(CClientWorkHurtInfo, id) for id in idList ]
        return True


class CClientWorkHurtInfo(CInfo):
    def __init__(self, context, clientWorkHurtId):
        CInfo.__init__(self, context)
        self.clientWorkHurtId = clientWorkHurtId

    def _load(self):
        db = QtGui.qApp.db
        table = db.table('ClientWork_Hurt')
        tableHurtType = db.table('rbHurtType')

        record = db.getRecordEx(table.leftJoin(tableHurtType, tableHurtType['id'].eq(table['hurtType_id'])),
                                [table['id'], tableHurtType['code'], tableHurtType['name'], table['stage']],
                                table['id'].eq(self.clientWorkHurtId))
        if record:
            self._code = forceString(record.value('code'))
            self._name = forceString(record.value('name'))
            self._stage = forceInt(record.value('stage'))
            self._factors = self.getInstance(CClientWorkHurtFactorInfoList, forceRef(record.value('id')))
            return True
        else:
            self._code = ''
            self._name = ''
            self._stage = 0
            self._factors = []
            return False

    code  = property(lambda self: self.load()._code)
    name  = property(lambda self: self.load()._name)
    stage = property(lambda self: self.load()._stage)
    factors = property(lambda self: self.load()._factors)

#    def __unicode__(self):
    def __str__(self):
        return self.load()._name


class CClientWorkHurtFactorInfoList(CInfoList):
    def __init__(self, context, hurtId):
        CInfoList.__init__(self, context)
        self.hurtId = hurtId

    def _load(self):
        db = QtGui.qApp.db
        table = db.table('ClientWork_Hurt_Factor')
        idList = db.getIdList(table, 'id', table['master_id'].eq(self.hurtId), 'id')
        self._items = [ self.getInstance(CClientWorkHurtFactorInfo, id) for id in idList ]


class CClientWorkHurtFactorInfo(CInfo):
    def __init__(self, context, clientWorkHurtFactorId):
        CInfo.__init__(self, context)
        self.clientWorkHurtFactorId = clientWorkHurtFactorId
        self._code = ''
        self._name = ''

    def _load(self):
        db = QtGui.qApp.db
        table = db.table('ClientWork_Hurt_Factor')
        tableHurtFactorType = db.table('rbHurtFactorType')
        record = db.getRecordEx(table.leftJoin(tableHurtFactorType, tableHurtFactorType['id'].eq(table['factorType_id'])),
                                [tableHurtFactorType['name'], tableHurtFactorType['code']],
                                table['id'].eq(self.clientWorkHurtFactorId))
        if record:
            self._code = forceString(record.value('code'))
            self._name = forceString(record.value('name'))
            return True
        return False

#    def __unicode__(self):
    def __str__(self):
        return self.load()._name

    code  = property(lambda self: self.load()._code)
    name  = property(lambda self: self.load()._name)


class CClientAttachInfo(CInfo):
    def __init__(self, context, clientId, temporary):
        CInfo.__init__(self, context)
        self.clientId = clientId
        self.temporary = temporary
        self._document = None


    def _load(self):
        attach = getAttachRecord(self.clientId, self.temporary)
        if attach:
            self._code = attach['code']
            self._name = attach['name']
            self._outcome = attach['outcome']
            self._org = self.getInstance(COrgInfo, attach['LPU_id'])
            self._orgStructure = self.getInstance(COrgStructureInfo, attach['orgStructure_id'])
            self._begDate = CDateInfo(attach['begDate'])
            self._endDate = CDateInfo(attach['endDate'])
            self._document = self.getInstance(CClientDocumentInfo, documentId=attach['document_id'])
            self._isStatement = attach['isStatement']
            return True
        else:
            self._code = ''
            self._name = ''
            self._outcome = ''
            self._org = self.getInstance(COrgInfo, None)
            self._orgStructure = self.getInstance(COrgStructureInfo, None)
            self._begDate = CDateInfo()
            self._endDate = CDateInfo()
            self._document = None
            self._isStatement = 0
            return False


    def __str__(self):
        self.load()
        if self._ok:
            result = self._name
            if self._outcome:
                result += ' '+ unicode(self._endDate)
            elif self.temporary:
                result += ' ' + self._org.shortName
                if self._begDate:
                    result += u' c ' + unicode(self._begDate)
                if self.endDate:
                    result += u' по ' + unicode(self._endDate)
            else:
                result += ' ' + self._org.shortName
        else:
            result = ''
        return result

    code    = property(lambda self: self.load()._code)
    name    = property(lambda self: self.load()._name)
    outcome = property(lambda self: self.load()._outcome)
    begDate = property(lambda self: self.load()._begDate)
    endDate = property(lambda self: self.load()._endDate)
    org     = property(lambda self: self.load()._org)
    orgStructure = property(lambda self: self.load()._orgStructure)
    document = property(lambda self: self.load()._document)
    isStatement = property(lambda self: self.load()._isStatement)


class CClientSocStatusInfoList(CInfoList):
    def __init__(self, context, clientId):
        CInfoList.__init__(self, context)
        self.clientId = clientId

    def _load(self):
        idList = getClientSocStatusIds(self.clientId)
        self._items = [ self.getInstance(CClientSocStatusInfo, id) for id in idList ]



class CClientSocStatusInfo(CInfo):
    def __init__(self, context, socStatusId):
        CInfo.__init__(self, context)
        self.socStatusId = socStatusId
        self._code = ''
        self._name = ''
        self._document = None
        self._begDate  = None
        self._endDate  = None
        self._classes  = []


    def _load(self):
        db = QtGui.qApp.db
        tableClientSocStatus = db.table('ClientSocStatus')
        tableSocStatusType = db.table('rbSocStatusType')
        record = QtGui.qApp.db.getRecord(tableClientSocStatus.leftJoin(tableSocStatusType,
                                                                       tableSocStatusType['id'].eq(tableClientSocStatus['socStatusType_id'])),
                                         'code, name, document_id, begDate, endDate, socStatusType_id',
                                         self.socStatusId)
#        record = QtGui.qApp.db.getRecord('ClientSocStatus LEFT JOIN rbSocStatusType ON rbSocStatusType.id = ClientSocStatus.socStatusType_id', 'code, name, document_id', self.socStatusId)
        if record:
            self._code = forceString(record.value('code'))
            self._name = forceString(record.value('name'))
            self._document = self.getInstance(CClientDocumentInfo, documentId=forceRef(record.value('document_id')))
            self._begDate = CDateInfo(forceDate(record.value('begDate')))
            self._endDate = CDateInfo(forceDate(record.value('endDate')))
            self._classes = self.getInstance(CSocStatusClassInfoList, forceRef(record.value('socStatusType_id')))
            return True
        return False

    code    = property(lambda self: self.load()._code)
    name    = property(lambda self: self.load()._name)
    document= property(lambda self: self.load()._document)
    begDate = property(lambda self: self.load()._begDate)
    endDate = property(lambda self: self.load()._endDate)
    classes = property(lambda self: self.load()._classes)

#    def __unicode__(self):
    def __str__(self):
        return self.load()._name


class CSocStatusClassInfoList(CInfoList):
    def __init__(self, context, socStatusTypeId):
        CInfoList.__init__(self, context)
        self.socStatusTypeId = socStatusTypeId


    def _load(self):
        idList = getSocStatusTypeClasses(self.socStatusTypeId)
        self._items = [ self.getInstance(CSocStatusClassInfo, id) for id in idList ]


class CSocStatusClassInfo(CInfo):
    def __init__(self, context, socStatusClassId):
        CInfo.__init__(self, context)
        self.socStatusClassId = socStatusClassId
        self._code = ''
        self._name = ''
        self._group = None

    def _load(self):
        record = QtGui.qApp.db.getRecord('rbSocStatusClass', 'code, name, group_id', self.socStatusClassId)
        if record:
            self._code = forceString(record.value('code'))
            self._name = forceString(record.value('name'))
            groupId = forceRef(record.value('group_id'))
            self._group = self.getInstance(CSocStatusClassInfo, groupId) if groupId else None
            return True
        return False

    code    = property(lambda self: self.load()._code)
    name    = property(lambda self: self.load()._name)
    group   = property(lambda self: self.load()._group)

#    def __unicode__(self):
    def __str__(self):
        return self.load()._name


    def isPartOf(self, name):
        return self._isPartOf(name.lower(), set([]))


    def _isPartOf(self, name, seen):
        self.load()
        if self._name.lower() == name:
            return True
        if self.socStatusClassId in seen:
            return None
        elif self._group:
            seen.add(self.socStatusClassId)
            return self._group._isPartOf(name, seen)
        else:
            return False


class CClientIntoleranceMedicamentInfoList(CInfoList):
    def __init__(self, context, clientId):
        CInfoList.__init__(self, context)
        self.clientId = clientId

    def _load(self):
        db = QtGui.qApp.db
        table = db.table('ClientIntoleranceMedicament')
        idList = db.getIdList(table, 'id', table['client_id'].eq(self.clientId), 'id')
        self._items = [ self.getInstance(CClientIntoleranceMedicamentInfo, id) for id in idList ]
        return True


class CClientIntoleranceMedicamentInfo(CInfo):
    def __init__(self, context, itemId):
        CInfo.__init__(self, context)
        self.itemId = itemId

    def _load(self):
        db = QtGui.qApp.db
        table = db.table('ClientIntoleranceMedicament')
        record = db.getRecord(table, ['nameMedicament', 'power', 'createDate', 'notes'], self.itemId)
        if record:
            self._name  = forceString(record.value('nameMedicament'))
            self._power = forceInt(record.value('power'))
            self._date  = CDateInfo(record.value('createDate'))
            self._notes = forceString(record.value('notes'))
            return True
        else:
            self._name  = ''
            self._power = 0
            self._date  = CDateInfo(None)
            self._notes = ''
            return False

    name  = property(lambda self: self.load()._name)
    power = property(lambda self: self.load()._power)
    date  = property(lambda self: self.load()._date)
    notes = property(lambda self: self.load()._notes)

#    def __unicode__(self):
    def __str__(self):
        return self.load()._name


class CClientAllergyInfoList(CInfoList):
    def __init__(self, context, clientId):
        CInfoList.__init__(self, context)
        self.clientId = clientId

    def _load(self):
        db = QtGui.qApp.db
        table = db.table('ClientAllergy')
        idList = db.getIdList(table, 'id', [table['client_id'].eq(self.clientId),  table['deleted'].eq(0)], 'id')
        self._items = [ self.getInstance(CClientAllergyInfo, id) for id in idList ]
        return True


class CClientAllergyInfo(CInfo):
    def __init__(self, context, itemId):
        CInfo.__init__(self, context)
        self.itemId = itemId

    def _load(self):
        db = QtGui.qApp.db
        table = db.table('ClientAllergy')
        record = db.getRecord(table, ['nameSubstance', 'power', 'createDate', 'notes'], self.itemId)
        if record:
            self._name  = forceString(record.value('nameSubstance'))
            self._power = forceInt(record.value('power'))
            self._date  = CDateInfo(record.value('createDate'))
            self._notes = forceString(record.value('notes'))
            return True
        else:
            self._name  = ''
            self._power = 0
            self._date  = CDateInfo(None)
            self._notes = ''
            return False

    name   = property(lambda self: self.load()._name)
    power  = property(lambda self: self.load()._power)
    date   = property(lambda self: self.load()._date)
    notes  = property(lambda self: self.load()._notes)

#    def __unicode__(self):
    def __str__(self):
        return self.load()._name


class CClientIdentificationInfo(CInfo):
    def __init__(self, context, clientId):
        CInfo.__init__(self, context)
        self._clientId = clientId
        self._byCode = {}
#        self._byName = {}
        self._nameDict = {}
        self._checkDateDict = {}


    def _load(self):
        db = QtGui.qApp.db
        tableCI = db.table('ClientIdentification')
        tableAS = db.table('rbAccountingSystem')
        stmt = db.selectStmt(tableCI.leftJoin(tableAS, tableAS['id'].eq(tableCI['accountingSystem_id'])),
                             ['code', 'name', 'identifier', 'checkDate'],
                             db.joinAnd([tableCI['client_id'].eq(self._clientId),
                                         tableCI['deleted'].eq(0),
                                        ])
                            )
        query = db.query(stmt)
        while query.next():
            record = query.record()
            code = forceString(record.value('code'))
            name = forceString(record.value('name'))
            identifier = forceString(record.value('identifier'))
            checkDate = forceDate(record.value('checkDate'))
            self._byCode[code] = identifier
#            self._byName[name] = identifier
            self._nameDict[code] = name
            self._checkDateDict[code] = checkDate
        return True


    def has_key(self, key):
        return key in self._byCode


    def get(self, key, default=None):
        return self._byCode.get(key, default)


    def iteritems(self):
        return self._byCode.iteritems()


    def iterkeys(self):
        return self._byCode.iterkeys()


    def itervalues(self):
        return self._byCode.itervalues()


    def items(self):
        return self._byCode.items()


    def keys(self):
        return self._byCode.keys()


    def values(self):
        return self._byCode.values()


    def __nonzero__(self):
        return bool(self._byCode)


    def __len__(self):
        return len(self._byCode)


    def __contains__(self, key):
        return key in self._byCode


    def __getitem__(self, key):
        return self._byCode.get(key, '')


    def __iter__(self):
        return self._byCode.iterkeys()


    def __str__(self):
        self.load()
        l = [ u'%s (%s): %s' % (self._nameDict[code], code, identifier)
              for code, identifier in self._byCode.iteritems()
            ]
        l.sort()
        return ', '.join(l)

    byCode = property(lambda self: self.load()._byCode)
#    byName = property(lambda self: self.load()._byName)
    nameDict = property(lambda self: self.load()._nameDict)
    checkDateDict = property(lambda self: self.load()._checkDateDict)


class CClientRelationInfoList(CInfoList):
    def __init__(self, context, clientId, date):
        CInfoList.__init__(self, context)
        self.clientId = clientId
        self.date = date

    def _load(self):
        db = QtGui.qApp.db
        table = db.table('ClientRelation')
        directIdList = db.getIdList(table,
                              'id',
                              db.joinAnd([table['deleted'].eq(0),
                                          table['relativeType_id'].isNotNull(),
                                          table['client_id'].eq(self.clientId)
                                         ]),
                              'id')
        reversedIdList = db.getIdList(table,
                              'id',
                              db.joinAnd([table['deleted'].eq(0),
                                          table['relativeType_id'].isNotNull(),
                                          table['relative_id'].eq(self.clientId)
                                         ]),
                              'id')

        self._items = ([ self.getInstance(CClientRelationInfo, id, self.date, True) for id in directIdList ] +
                       [ self.getInstance(CClientRelationInfo, id, self.date, False) for id in reversedIdList ])
        return True


class CClientRelationInfo(CInfo):
    def __init__(self, context, itemId, date, isDirect):
        CInfo.__init__(self, context)
        self._itemId = itemId
        self._date = date
        self._isDirect = isDirect

    def _load(self):
        db = QtGui.qApp.db
        tableCR = db.table('ClientRelation')
        tableRT = db.table('rbRelationType')
        record = db.getRecord(tableCR.leftJoin(tableRT, tableRT['id'].eq(tableCR['relativeType_id'])),
                              ['client_id', 'relative_id',
                               'leftName', 'rightName',
                               'code',
                               'isDirectGenetic', 'isBackwardGenetic',
                               'isDirectRepresentative',  'isBackwardRepresentative',
                               'isDirectEpidemic', 'isBackwardEpidemic',
                               'isDirectDonation',  'isBackwardDonation',
                               'regionalCode', 'regionalReverseCode'],
                              self._itemId)
        if record:
            leftName  = forceString(record.value('leftName'))
            rightName = forceString(record.value('rightName'))
            code = forceString(record.value('code'))

            isDirectGenetic = forceBool(record.value('isDirectGenetic'))
            isBackwardGenetic = forceBool(record.value('isBackwardGenetic'))
            isDirectRepresentative = forceBool(record.value('isDirectRepresentative'))
            isBackwardRepresentative = forceBool(record.value('isBackwardRepresentative'))
            isDirectEpidemic = forceBool(record.value('isDirectEpidemic'))
            isBackwardEpidemic = forceBool(record.value('isBackwardEpidemic'))
            isDirectDonation = forceBool(record.value('isDirectDonation'))
            isBackwardDonation = forceBool(record.value('isBackwardDonation'))

            if self._isDirect:
                clientId = forceRef(record.value('relative_id'))
                role, otherRole = leftName, rightName
                regionalCode = forceString(record.value('regionalCode'))
            else:
                clientId = forceRef(record.value('client_id'))
                role, otherRole = rightName, leftName
                regionalCode = forceString(record.value('regionalReverseCode'))
                isDirectGenetic, isBackwardGenetic = isBackwardGenetic, isDirectGenetic
                isDirectRepresentative, isBackwardRepresentative = isBackwardRepresentative, isDirectRepresentative
                isDirectEpidemic, isBackwardEpidemic = isBackwardEpidemic, isDirectEpidemic
                isDirectDonation, isBackwardDonation = isBackwardDonation, isDirectDonation

            self._role = role
            self._otherRole = otherRole
            self._other = self.getInstance(CClientInfo, clientId, self._date)
            self._name = role + ' -> ' + otherRole
            self._code = code
            self._regionalCode = regionalCode
            self._isDirectGenetic = isDirectGenetic
            self._isBackwardGenetic = isBackwardGenetic
            self._isDirectRepresentative = isDirectRepresentative
            self._isBackwardRepresentative = isBackwardRepresentative
            self._isDirectEpidemic = isDirectEpidemic
            self._isBackwardEpidemic = isBackwardEpidemic
            self._isDirectDonation = isDirectDonation
            self._isBackwardDonation = isBackwardDonation
            return True
        else:
            self._role = ''
            self._otherRole = ''
            self._other = None
            self._name = ''
            self._code = ''
            self._regionalCode = ''
            self._isDirectGenetic = False
            self._isBackwardGenetic = False
            self._isDirectRepresentative = False
            self._isBackwardRepresentative = False
            self._isDirectEpidemic = False
            self._isBackwardEpidemic = False
            self._isDirectDonation = False
            self._isBackwardDonation = False
            return False


    role      = property(lambda self: self.load()._role)
    otherRole = property(lambda self: self.load()._otherRole)
    other     = property(lambda self: self.load()._other)
    name      = property(lambda self: self.load()._name)
    code      = property(lambda self: self.load()._code)
    regionalCode = property(lambda self: self.load()._regionalCode)
    isDirectGenetic          = property(lambda self: self.load()._isDirectGenetic)
    isBackwardGenetic        = property(lambda self: self.load()._isBackwardGenetic)
    isDirectRepresentative   = property(lambda self: self.load()._isDirectRepresentative)
    isBackwardRepresentative = property(lambda self: self.load()._isBackwardRepresentative)
    isDirectEpidemic         = property(lambda self: self.load()._isDirectEpidemic)
    isBackwardEpidemic       = property(lambda self: self.load()._isBackwardEpidemic)
    isDirectDonation         = property(lambda self: self.load()._isDirectDonation)
    isBackwardDonation       = property(lambda self: self.load()._isBackwardDonation)

    def __str__(self):
        return self.name + ' ' + self.other


class CClientQuotaInfoList(CInfoList):
    def __init__(self, context, clientId):
        CInfoList.__init__(self, context)
        self.clientId = clientId


    def _load(self):
        db = QtGui.qApp.db
        table = db.table('Client_Quoting')
        idList = db.getIdList(table,
                              'id',
                              db.joinAnd([table['deleted'].eq(0),
                                          table['master_id'].eq(self.clientId)
                                         ]),
                              'directionDate, id')
        self._items = ([ self.getInstance(CClientQuotaInfo, id) for id in idList ])
        return True


class CQuotaTypeInfo(CRBInfo):
    tableName = 'QuotaType'


    def _initByRecord(self, record):
        self._class = forceInt(record.value('class'))
        self._groupCode = forceString(record.value('group_code'))


    def _initByNull(self):
        self._class = 0
        self._group = ''


    class_ = property(lambda self: self.load()._class)
    group = property(lambda self: self.load()._group)


class CClientQuotaInfo(CInfo):
    def __init__(self, context, itemId):
        CInfo.__init__(self, context)
        self._itemId = itemId

    def _load(self):
        db = QtGui.qApp.db
        table = db.table('Client_Quoting')
        record = db.getRecord(table, '*', self._itemId)
        if record:
            result = True
        else:
            record = table.newRecord()
            result = False

        self._identifier    = forceString(record.value('identifier'))
        self._ticket        = forceString(record.value('quotaTicket'))
        self._type          = self.getInstance(CQuotaTypeInfo, forceRef(record.value('quotaType_id')))
        self._stage         = forceInt(record.value('stage'))
        self._directionDate = CDateInfo(forceDate(record.value('directionDate')))
        self._freeInput     = forceString(record.value('freeInput'))
        self._org           = self.getInstance(COrgInfo, forceRef(record.value('org_id')))
        self._amount        = forceInt(record.value('amount'))
        self._MKB           = self.getInstance(CMKBInfo, forceString(record.value('MKB')))
        self._status        = forceInt(record.value('status'))
        self._request       = forceInt(record.value('request'))
        self._statement     = forceString(record.value('statment'))
        self._registrationDate = CDateInfo(forceDate(record.value('dateRegistration')))
        self._endDate       = CDateInfo(forceDate(record.value('dateEnd')))
        self._orgStructure  = self.getInstance(COrgStructureInfo, forceRef(record.value('orgStructure_id')))
        self._regionCode    = forceString(record.value('regionCode'))
        self._discussion    = self.getInstance(CQuotaDiscussionInfoList, self._itemId)
        return result

    identifier       = property(lambda self: self.load()._identifier)
    ticket           = property(lambda self: self.load()._ticket)
    type             = property(lambda self: self.load()._type)
    stage            = property(lambda self: self.load()._stage)
    directionDate    = property(lambda self: self.load()._directionDate)
    freeInput        = property(lambda self: self.load()._freeInput)
    org              = property(lambda self: self.load()._org)
    amount           = property(lambda self: self.load()._amount)
    MKB              = property(lambda self: self.load()._MKB)
    status           = property(lambda self: self.load()._status)
    request          = property(lambda self: self.load()._request)
    statement        = property(lambda self: self.load()._statement)
    registrationDate = property(lambda self: self.load()._registrationDate)
    endDate          = property(lambda self: self.load()._endDate)
    orgStructure     = property(lambda self: self.load()._orgStructure)
    regionCode       = property(lambda self: self.load()._regionCode)
    discussion       = property(lambda self: self.load().discussion)

    def __str__(self):
        return self.identifier + ' ' + self.ticket


class CQuotaDiscussionInfoList(CInfoList):
    def __init__(self, context, quotaId):
        CInfoList.__init__(self, context)
        self.quotaId = quotaId

    def _load(self):
        db = QtGui.qApp.db
        table = db.table('Client_QuotingDiscussion')
        idList = db.getIdList(table, 'id', table['master_id'].eq(self.quotaId), 'id')
        self._items = ([ self.getInstance(CQuotaDiscussionInfo, id) for id in idList ])
        return True


class CQuotaAgreementTypeInfo(CRBInfo):
    tableName = 'rbAgreementType'


class CQuotaDiscussionInfo(CInfo):
    def __init__(self, context, itemId):
        CInfo.__init__(self, context)
        self._itemId = itemId

    def _load(self):

        db = QtGui.qApp.db
        table = db.table('Client_QuotingDiscussion')
        record = db.getRecord(table, '*', self._itemId)
        if record:
            result = True
        else:
            record = table.newRecord()
            result = False
        return result


        self._date = CDateInfo(forceDate(record.value('dateMessage')))
        self._agreementType   = self.getInstance(CQuotaAgreementTypeInfo, forceRef(record.value('agreementType_id')))
        self._person          = self.getInstance(CPersonInfo, forceRef(record.value('responsiblePerson_id')))
        self._cosignatory     = forceString(record.value('cosignatory'))
        self._cosignatoryPost = forceString(record.value('cosignatoryPost'))
        self._cosignatoryName = forceString(record.value('cosignatoryName'))
        self._remark          = forceString(record.value('remark'))


    date             = property(lambda self: self.load()._date)
    agreementType    = property(lambda self: self.load()._agreementType)
    person           = property(lambda self: self.load()._person)
    cosignatory      = property(lambda self: self.load()._cosignatory)
    cosignatoryPost  = property(lambda self: self.load()._cosignatoryPost)
    cosignatoryName  = property(lambda self: self.load()._cosignatoryName)
    remark           = property(lambda self: self.load()._remark)

    def __str__(self):
        return self.date + ' ' + self.remark + ' ' + self.person



def getClientInfo2(id, date=None):
    context = CInfoContext()
    return CClientInfo(context, id, date)


def fixClientDeath(clientId, deathDate, fixDeathDate, orgId):
    db = QtGui.qApp.db
    tableClient = db.table('Client')
    record = db.getRecord(tableClient, 'id, deathDate', clientId)
    if record:
        record.setValue('deathDate', toVariant(fixDeathDate))
        db.updateRecord(tableClient, record)


def getClientIdentifier(accountingSystemId, clientId):
    record = selectLatestRecord('ClientIdentification', clientId, filter='accountingSystem_id=%d' % accountingSystemId)
    if record:
        return forceString(record.value('identifier'))
    return ''


def getClientIdentification(clientId):
    result = {}
    stmt = """
    SELECT r.name, ci.identifier, ci.checkDate
    FROM ClientIdentification ci
    LEFT JOIN rbAccountingSystem r ON r.id = ci.accountingSystem_id
    WHERE ci.deleted = 0 AND ci.client_id = %s
        AND r.showInClientInfo = 1
    """ % clientId
    db = QtGui.qApp.db
    query = db.query(stmt)
    while query.next():
        r = query.record()
        if r:
            result[forceString(r.value(0))] = (forceString(r.value(1)), forceDate(r.value(2)))
    return result


def changeClientIdentification(clientId, name, value):
    if not clientId:
        return
    db = QtGui.qApp.db
    accountingSystemId = forceRef(db.translate('rbAccountingSystem', 'code', name, 'id'))
    record = db.getRecordEx('ClientIdentification', '*', 'deleted=0 AND accountingSystem_id=%d AND client_id=%d'%(accountingSystemId, clientId))
    if not record:
        record = db.table('ClientIdentification').newRecord()
        record.setValue('accountingSystem_id', QVariant(accountingSystemId))
        record.setValue('client_id', QVariant(clientId))
    record.setValue('identifier', QVariant(value))
    record.setValue('checkDate', QVariant(QDate.currentDate()))
    db.insertOrUpdate('ClientIdentification', record)


def findKLADRRegionRecordsInQuoting(code, quotingRecord):
    def getValueFromRecords(records):
        res = []
        for rec in records:
            res.append(forceString(rec.value('region_code')))
        return res
    def appendCode(list, code, codesList, recordsList):
        if code in codesList:
            x = codesList.index(code)
            list.append(recordsList[x])
    db = QtGui.qApp.db
    model = getKladrTreeModel()
    item = model.getRootItem().findCode(code)
    quotingId = forceInt(quotingRecord.value('id'))
    cond = 'Quoting_Region.`master_id`=%d AND Quoting_Region.`deleted`=0'%quotingId
    quotingRegionRecords = db.getRecordList('Quoting_Region', '*', cond)
    codesList = getValueFromRecords(quotingRegionRecords)
    resumeList = []
    appendCode(resumeList, code, codesList, quotingRegionRecords)
    item = item.parent()
    while item:
        code = item._code
        appendCode(resumeList, code, codesList, quotingRegionRecords)
        item = item.parent()
    return resumeList


def findDistrictRecordsInQuoting(codes, quotingRecord):
    def getValueFromRecords(records):
        res = []
        for rec in records:
            res.append((forceString(rec.value('region_code')), forceString(rec.value('district_code'))))
        return res
    def appendCode(list, codes, codesList, recordsList):
        if codes in codesList:
            x = codesList.index(codes)
            list.append(recordsList[x])
    db = QtGui.qApp.db
    quotingId = forceInt(quotingRecord.value('id'))
    cond = 'Quoting_District.`master_id`=%d AND Quoting_District.`deleted`=0'%quotingId
    quotingRegionRecords = db.getRecordList('Quoting_District', '*', cond)
    codesList = getValueFromRecords(quotingRegionRecords)
    resumeList = []
    appendCode(resumeList, codes, codesList, quotingRegionRecords)
    return resumeList

# ############################################################################

def uniqueIdentifierCheckingIsPassed(currentItemId, accountingSystemId, newIdentifier):
    db = QtGui.qApp.db
    table = db.table('ClientIdentification')
    cond = [table['accountingSystem_id'].eq(accountingSystemId),
            table['identifier'].eq(newIdentifier)]
    if currentItemId:
        cond.append(table['id'].ne(currentItemId))
    record = db.getRecordEx(table, 'id', cond)
    if record:
        QtGui.QMessageBox.warning(QtGui.qApp.mainWindow, u'Изменение идентификатора',
            u'В выбранной учётной системе требуется ввод уникального удентификатора.\n\'%s\' не уникален'%newIdentifier,
            QtGui.QMessageBox.Close)
        return False
    return True


# ######################################################################################

class CPolicyKindCache(CDbEntityCache):
    mapIdToCode = {}

    @classmethod
    def purge(cls):
        cls.mapIdToCode.clear()


    @classmethod
    def getCode(cls, id):
        result = cls.mapIdToCode.get(id, False)
        if result == False:
            cls.connect()
            result = forceString(QtGui.qApp.db.translate('rbPolicyKind', 'id', id, 'code'))
            cls.mapIdToCode[id] = result
        return result


class CAppointmentPurposeCache(CDbEntityCache):
    mapIdToItem = {}

    class CAppointmentPurpose(CSexAgeConstraint):
        def __init__(self, record):
            CSexAgeConstraint.__init__(self)
            CSexAgeConstraint.initByRecord(self, record)
            self.code = forceString(record.value('code'))
            self.name = forceString(record.value('name'))
            self.enablePrimaryRecord     = forceBool(record.value('enablePrimaryRecord'))
            self.enableOwnRecord         = forceBool(record.value('enableOwnRecord'))
            self.enableConsultancyRecord = forceBool(record.value('enableConsultancyRecord'))
            self.enableRecordViaInfomat  = forceBool(record.value('enableRecordViaInfomat'))
            self.enableRecordViaCallCenter=forceBool(record.value('enableRecordViaCallCenter'))
            self.enableRecordViaInternet = forceBool(record.value('enableRecordViaInternet'))
            self.requireReferral         = forceBool(record.value('requireReferral'))


    @classmethod
    def purge(cls):
        cls.mapIdToItem.clear()


    @classmethod
    def getItem(cls, id):
        result = cls.mapIdToItem.get(id, False) if id is not None else None
        if result == False:
            cls.connect()
            record = QtGui.qApp.db.getRecord('rbAppointmentPurpose', '*', id)
            result = CAppointmentPurposeCache.CAppointmentPurpose(record) if record is not None else None
            cls.mapIdToItem[id] = result
        return result


    @classmethod
    def getName(cls, id):
        item = cls.getItem(id)
        return item.name if item else ''



class CSocStatusTypeCache(CDbEntityCache):
    mapIdToCode = {}
    mapIdToName = {}


    @classmethod
    def purge(cls):
        cls.mapIdToCode.clear()
        cls.mapIdToName.clear()


    @classmethod
    def register(cls, id):
        cls.connect()
        record = QtGui.qApp.db.getRecord('rbSocStatusType', ['code', 'name'], id)
        if record:
            code = forceString(record.value(0))
            name = forceString(record.value(1))
        else:
            code = name = u'Соц.статус {%r}' % id
        cls.mapIdToCode[id] = code
        cls.mapIdToName[id] = name
        return code, name


    @classmethod
    def getCode(cls, id):
        result = cls.mapIdToCode.get(id, False)
        if result == False:
            result, name = cls.register(id)
        return result


    @classmethod
    def getName(cls, id):
        result = cls.mapIdToName.get(id, False)
        if result == False:
            code, result = cls.register(id)
        return result

# #####################################################

def canRemoveEventWithTissue():
    widget = QtGui.qApp.mainWindow
    hasRight = QtGui.qApp.userHasRight('deleteEventWithTissue')
    if hasRight:
        mbResult = QtGui.QMessageBox.question(widget,
                                            u'Внимание!',
                                            u'В событии присутствуют действия связанные с забором биометериалов.\nУдалить?',
                                            QtGui.QMessageBox.Yes | QtGui.QMessageBox.No,
                                            QtGui.QMessageBox.No)
        result = mbResult == QtGui.QMessageBox.Yes
    else:
        result = False
        QtGui.QMessageBox.warning(widget,
                                  u'Внимание!',
                                  u'В событии присутствуют действия связанные с забором биометериалов.\nУ вас нет прав на удаление',
                                  QtGui.QMessageBox.Ok)
    return result


def canRemoveEventWithJobTicket():
    widget = QtGui.qApp.mainWindow
    hasRight = QtGui.qApp.userHasRight('deleteEventWithJobTicket')
    if hasRight:
        mbResult = QtGui.QMessageBox.question(widget,
                                            u'Внимание!',
                                            u'В событии присутствуют действия связанные с номерком на Работу.\nУдалить?',
                                            QtGui.QMessageBox.Yes | QtGui.QMessageBox.No,
                                            QtGui.QMessageBox.No)
        result = mbResult == QtGui.QMessageBox.Yes
    else:
        result = False
        QtGui.QMessageBox.warning(widget,
                                  u'Внимание!',
                                  u'В событии присутствуют действия связанные с номерком на Работу.\nУ вас нет прав на удаление',
                                  QtGui.QMessageBox.Ok)
    return result

# ########################################################################################


class CClientRelationComboBoxForConsents(QtGui.QComboBox):
    def __init__(self, parent=None):
        QtGui.QComboBox.__init__(self, parent)
        self._clientId = None


    def setClientId(self, clientId):
        self._clientId = clientId
        db = QtGui.qApp.db
        tableCR = db.table('ClientRelation')
        tableRT = db.table('rbRelationType')
        tableC  = db.table('Client')

        clientRecord = db.getRecord(tableC, 'lastName, firstName, patrName, birthDate', clientId)
        if clientRecord:
            name = formatShortName(clientRecord.value('lastName'),
                                   clientRecord.value('firstName'),
                                   clientRecord.value('patrName'))
            itemText = u', '.join([name, forceString(clientRecord.value('birthDate'))])
            self.addItem(itemText, QVariant(clientId))
        else:
            self.addItem(u'Неправильные данные!', QVariant())

        queryTable = tableCR
        queryTable = queryTable.leftJoin(tableRT, tableRT['id'].eq(tableCR['relativeType_id']))
        queryTable = queryTable.leftJoin(tableC, tableC['id'].eq(tableCR['relative_id']))

        cond = [tableCR['client_id'].eq(clientId), tableCR['deleted'].eq(0)]
        fields = ['CONCAT_WS(\' | \', rbRelationType.`code`, CONCAT_WS(\'->\', rbRelationType.`leftName`, rbRelationType.`rightName`)) AS relationType',
                  tableC['lastName'].alias(), tableC['firstName'].name(),
                  tableC['patrName'].name(), tableC['birthDate'].name(),
                  tableC['id'].alias('relativeId')]
        records = db.getRecordList(queryTable, fields, cond)
        for record in records:
            name = formatShortName(record.value('lastName'),
                                   record.value('firstName'),
                                   record.value('patrName'))
            itemText = u', '.join([name,
                                  forceString(record.value('birthDate')),
                                  forceString(record.value('relationType'))])
            relativeId = forceRef(record.value('relativeId'))
            self.addItem(itemText, QVariant(relativeId))

        queryTable = tableCR
        queryTable = queryTable.leftJoin(tableRT, tableRT['id'].eq(tableCR['relativeType_id']))
        queryTable = queryTable.leftJoin(tableC, tableC['id'].eq(tableCR['client_id']))

        cond = [tableCR['relative_id'].eq(clientId), tableCR['deleted'].eq(0)]
        fields = ['CONCAT_WS(\' | \', rbRelationType.`code`, CONCAT_WS(\'->\', rbRelationType.`rightName`, rbRelationType.`leftName`)) AS relationType',
                  tableC['lastName'].alias(), tableC['firstName'].name(),
                  tableC['patrName'].name(), tableC['birthDate'].name(),
                  tableC['id'].alias('relativeId')]
        records = db.getRecordList(queryTable, fields, cond)
        for record in records:
            name = formatShortName(record.value('lastName'),
                                   record.value('firstName'),
                                   record.value('patrName'))
            itemText = u', '.join([name,
                                  forceString(record.value('birthDate')),
                                  forceString(record.value('relationType'))])
            relativeId = forceRef(record.value('relativeId'))
            self.addItem(itemText, QVariant(relativeId))


    def value(self):
        currIndex = self.currentIndex()
        representerClientId = forceRef(self.itemData(currIndex))
        if representerClientId:
            return representerClientId
        return self._clientId


    def setValue(self, clientId):
        index = self.findData(QVariant(clientId))
        self.setCurrentIndex(index)


class CClientRelationComboBoxPatron(CROComboBox):
    def __init__(self, parent=None):
        CROComboBox.__init__(self, parent)
        self._clientId = None


    def setClientId(self, clientId):
        self._clientId = clientId
        self.addItem(u'не задано', QVariant(None))
        db = QtGui.qApp.db
        tableCR = db.table('ClientRelation')
        self.addPatronItem(db, tableCR, 'relative_id', 'relativeId', [tableCR['client_id'].eq(clientId), tableCR['deleted'].eq(0)])
        self.addPatronItem(db, tableCR, 'client_id', 'clientId', [tableCR['relative_id'].eq(clientId), tableCR['deleted'].eq(0)])


    def addPatronItem(self, db, tableCR, colRelative, colFields, cond):
        tableRT = db.table('rbRelationType')
        tableC  = db.table('Client')
        queryTable = tableCR
        queryTable = queryTable.leftJoin(tableRT, tableRT['id'].eq(tableCR['relativeType_id']))
        queryTable = queryTable.leftJoin(tableC, tableC['id'].eq(tableCR[colRelative]))

        fields = ['CONCAT_WS(\' | \', rbRelationType.code, CONCAT_WS(\'->\', rbRelationType.leftName, rbRelationType.rightName)) AS relationType',
                  tableC['lastName'].alias(), tableC['firstName'].name(),
                  tableC['patrName'].name(), tableC['birthDate'].name(),
                  tableC['id'].alias(colFields)]
        clientRelationRecordList = db.getRecordList(queryTable, fields, cond)
        for relationRecord in clientRelationRecordList:
            name = formatShortName(relationRecord.value('lastName'),
                                   relationRecord.value('firstName'),
                                   relationRecord.value('patrName'))
            itemText = u', '.join([name,
                                  forceString(relationRecord.value('birthDate')),
                                  forceString(relationRecord.value('relationType'))])
            relativeId = forceRef(relationRecord.value(colFields))
            self.addItem(itemText, QVariant(relativeId))


    def value(self):
        currIndex = self.currentIndex()
        representerClientId = forceRef(self.itemData(currIndex))
        if representerClientId:
            return representerClientId
        return None


    def setValue(self, clientId):
        index = self.findData(QVariant(clientId))
        self.setCurrentIndex(index)


def getHousesList(records):
    numberList = {}
    for record in records:
        name = forceString(record.value('NAME'))
        korp = forceString(record.value('KORP'))
        #code = forceString(record.value('CODE'))
        flatHouseList = forceString(record.value('flatHouseList'))
        korpList = []
        nameList = name.split(',')
        notNameCorp = True
        if len(nameList) == 1 and korp:
            nameList = name.split('-')
            if len(nameList) == 1:
                notNameCorp = False
                if korp:
                    korpList = korp.split(',')
                korpListOld = numberList.get(unicode(nameList[0]), [])
                for corp in korpList:
                    if corp and corp not in korpListOld:
                        korpListOld.append(corp)
                if not korpListOld:
                    korpListOld.append(u'')
                korpListOld.sort()
                numberList[unicode(nameList[0])] = korpListOld
        if notNameCorp:
            nameList = flatHouseList.split(',')
            for number in nameList:
                i = 0
                house = []
                korp = []
                target = house
                while i < len( number ):
                    if number[i] >= u'А' and number[i] <= u'Я' and i > 0:
                        korp.append(number[i])
                    elif number[i] == u'к':
                        target=korp
                    elif number[i] == u'л' and number[i:i+5] == u'литер':
                        target = korp
                        i += 4
                    else:
                        target.append(number[i])
                    i += 1
                houseStr = u''.join(house)
                korpStr = u''.join(korp)
                korpListOld = numberList.get(houseStr, [])
                if korpStr not in korpListOld:
                    korpListOld.append(korpStr)
                korpListOld.sort()
                numberList[houseStr] = korpListOld
    return numberList


# ######################################################


class CDbSearchWidget(QtGui.QWidget):
    class CLineEdit(QtGui.QLineEdit):
        def __init__(self, parent = None):
            QtGui.QLineEdit.__init__(self, parent)

        def keyPressEvent(self, event):
            key = event.key()
            if key == Qt.Key_Down:
                self.emit(SIGNAL('downKeyPressed()'))
            elif key == Qt.Key_Up:
                self.emit(SIGNAL('upKeyPressed()'))
            elif key == Qt.Key_Enter or key == Qt.Key_Return:
                self.emit(SIGNAL('enterKeyPressed()'))
            else:
                QtGui.QLineEdit.keyPressEvent(self, event)


    class CListView(QtGui.QListView):
        def __init__(self, parent = None):
            QtGui.QListView.__init__(self, parent)

        def keyPressEvent(self, event):
            key = event.key()
            if key == Qt.Key_Enter or key == Qt.Key_Return:
                self.emit(SIGNAL('enterKeyPressed()'))
            else:
                QtGui.QListView.keyPressEvent(self, event)

    def __init__(self, parent = None):
        QtGui.QWidget.__init__(self, parent)
        self.setMinimumHeight(200)
        self.lineEdit = CDbSearchWidget.CLineEdit()
        self.connect(self.lineEdit, SIGNAL('downKeyPressed()'), self.on_downKeyPress)
        self.connect(self.lineEdit, SIGNAL('upKeyPressed()'), self.on_upKeyPressed)
        self.connect(self.lineEdit, SIGNAL('enterKeyPressed()'), self.on_enterKeyPressed)
        self.connect(self.lineEdit, SIGNAL('textEdited(QString)'), self.on_textEdited)

        self.view = CDbSearchWidget.CListView(self)
        self.connect(self.view, SIGNAL('enterKeyPressed()'), self.on_enterKeyPressed)
        self.connect(self.view, SIGNAL('activated(QModelIndex)'), self.on_activated)

        lv = QtGui.QVBoxLayout(self)
        lv.setSpacing(0)
        lv.setContentsMargins(0, 0, 0, 0)
        lv.addWidget(self.lineEdit)
        lv.addWidget(self.view)

        self.sourceModel = None
        self.proxy = QtGui.QSortFilterProxyModel()
        self.proxy.setFilterCaseSensitivity(0)
        self.setFocusProxy(self.lineEdit)


    def setModel(self, model):
        self.sourceModel = model
        self.proxy.setSourceModel(self.sourceModel)
        self.view.setModel(self.proxy)
        index = self.proxy.index(0, 0)
        self.view.setCurrentIndex(index)


    def setTable(self, tableName, addNone=True, filter=None):
        model = CDbModel(self)
        model.setAddNone(addNone)
        model.setFilter(filter)
        model.setTable(tableName)
        self.setModel(model)


    def on_textEdited(self, text):
        self.proxy.setFilterFixedString(text)
        index = self.proxy.index(0, 0)
        self.view.setCurrentIndex(index)
        if self.view.isHidden():
            self.view.show()


    def on_activated(self, index):
        self.lineEdit.setText(self.proxy.data(index).toString())
        self.close()


    def on_upKeyPressed(self):
        index = self.view.currentIndex()
        new_index = index.sibling(index.row()-1, 0)
        if new_index.isValid():
            self.view.setCurrentIndex(new_index)


    def on_downKeyPress(self):
        if self.view.isHidden():
            self.view.show()
            return
        index = self.view.currentIndex()
        new_index = index.sibling(index.row()+1, 0)
        if new_index.isValid():
            self.view.setCurrentIndex(new_index)


    def on_enterKeyPressed(self):
        index = self.view.currentIndex()
        self.lineEdit.setText(self.proxy.data(index).toString())
        self.close()


    def setValue(self, itemId):
        self.itemId = itemId
        val = self.sourceModel.getNameById(itemId).toString()
        self.lineEdit.setText(val)


    def value(self):
        itemId = self.sourceModel.getIdByName(QVariant(self.lineEdit.text()))
        return itemId


def getRightEditTempInvalid(tempInvalidId):
    app = QtGui.qApp
    db = QtGui.qApp.db
    tableTempInvalid = db.table('TempInvalid')
    if app.userHasRight(urRegEditTempInvalidWithInsurance) and app.userHasRight(urRegEditTempInvalidNoInsurance):
        return True
    elif app.userHasRight(urRegEditTempInvalidWithInsurance):
        currentOrgStructureId = QtGui.qApp.currentOrgStructureId()
        if currentOrgStructureId:
            table = db.table('OrgStructure')
            tablePerson = db.table('Person')
            record = db.getRecordEx(table, [table['net_id']], [table['id'].eq(currentOrgStructureId), table['deleted'].eq(0)])
            currentNetId = forceRef(record.value('net_id')) if record else None
            if tempInvalidId:
                cond = [tableTempInvalid['id'].eq(tempInvalidId),
                        tableTempInvalid['deleted'].eq(0),
                        table['deleted'].eq(0),
                        tablePerson['deleted'].eq(0),
                        ]
                queryTable = tablePerson.innerJoin(tableTempInvalid, tableTempInvalid['person_id'].eq(tablePerson['id']))
                queryTable = queryTable.innerJoin(table, table['id'].eq(tablePerson['orgStructure_id']))
                record = db.getRecordEx(queryTable, [table['net_id'], tableTempInvalid['insuranceOfficeMark']], cond)
                createNetId = forceRef(record.value('net_id')) if record else None
                if currentNetId == createNetId and forceBool(record.value('insuranceOfficeMark')):
                    return True
    elif app.userHasRight(urRegEditTempInvalidNoInsurance):
        if tempInvalidId:
            cond = [tableTempInvalid['id'].eq(tempInvalidId),
                    tableTempInvalid['deleted'].eq(0)
                    ]
            record = db.getRecordEx(tableTempInvalid, [tableTempInvalid['insuranceOfficeMark']], cond)
            if not forceBool(record.value('insuranceOfficeMark')):
                return True
    return False

