# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui

from library.Utils      import forceInt, forceString, formatName, formatSex, formatSNILS
from Reports.ReportBase import CReportBase, createTable



def selectData(accountItemIdList):
    stmt="""
        SELECT DISTINCT
            Client.id,
            Client.lastName,
            Client.firstName,
            Client.patrName,
            Client.birthDate,
            Client.sex,
            Client.SNILS,
            ClientPolicy.serial    AS policySerial,
            ClientPolicy.number    AS policyNumber,
            Insurer.infisCode      AS policyInsurer,

            ClientDocument.serial  AS documentSerial,
            ClientDocument.number  AS documentNumber,
            rbDocumentType.code    AS documentType,
            formatClientAddress(ClientAddress.id) AS address
        FROM
            Account_Item
            LEFT JOIN Event         ON  Event.id  = Account_Item.event_id
            LEFT JOIN Client        ON  Client.id = Event.client_id
            LEFT JOIN ClientPolicy  ON ClientPolicy.id = getClientPolicyId(Event.client_id, 1)
            LEFT JOIN Organisation AS Insurer ON Insurer.id = ClientPolicy.insurer_id
            LEFT JOIN ClientDocument ON ClientDocument.client_id = Client.id AND
                      ClientDocument.id = (SELECT MAX(CD.id)
                                         FROM   ClientDocument AS CD
                                         LEFT JOIN rbDocumentType AS rbDT ON rbDT.id = CD.documentType_id
                                         LEFT JOIN rbDocumentTypeGroup AS rbDTG ON rbDTG.id = rbDT.group_id
                                         WHERE  rbDTG.code = '1' AND CD.client_id = Client.id)
            LEFT JOIN rbDocumentType ON rbDocumentType.id = ClientDocument.documentType_id

            LEFT JOIN ClientAddress ON  ClientAddress.client_id = Client.id
                                    AND ClientAddress.id = (SELECT MAX(id) FROM ClientAddress AS CA WHERE CA.Type=0 and CA.client_id = Client.id)
            WHERE
                %s
            ORDER BY
                Client.lastName,
                Client.firstName,
                Client.patrName,
                Client.birthDate,
                Client.sex
    """
    db = QtGui.qApp.db
    tableAccountItem = db.table('Account_Item')
    return db.query(stmt % tableAccountItem['id'].inlist(accountItemIdList))


class CClientsListByRegistry(CReportBase):
    def __init__(self, parent):
        CReportBase.__init__(self, parent)
        self.setTitle(u'Список пациентов по реестру')


    def build(self, description, params):
#        accountId = params.get('accountId', None)
        accountItemIdList = params.get('accountItemIdList', None)
        query = selectData(accountItemIdList)

        # now text
        doc = QtGui.QTextDocument()
        cursor = QtGui.QTextCursor(doc)

        cursor.setCharFormat(CReportBase.ReportTitle)
        cursor.insertText(self.title())
        cursor.insertBlock()
        cursor.setCharFormat(CReportBase.ReportBody)
        cursor.insertText(description)
        cursor.insertBlock()

        tableColumns = [
                          ('3%',  [ u'№'             ], CReportBase.AlignRight ),
                          ('10%', [ u'СНИЛС'         ], CReportBase.AlignLeft ),
                          ('20%', [ u'ФИО'           ], CReportBase.AlignLeft ),
                          ('10%', [ u'Дата рождения' ], CReportBase.AlignLeft ),
                          ('3%',  [ u'Пол'           ], CReportBase.AlignCenter ),
                          ('15%', [ u'Полис'         ], CReportBase.AlignCenter ),
                          ('15%', [ u'Документ'      ], CReportBase.AlignCenter ),
                          ('20%', [ u'Адрес'         ], CReportBase.AlignLeft ),
                       ]
        table = createTable(cursor, tableColumns)
        n = 0
        while query.next():
            n += 1
            record = query.record()
            name = formatName(record.value('lastName'),
                              record.value('firstName'),
                              record.value('patrName'))
            birthDate = forceString(record.value('birthDate'))
            sex = formatSex(forceInt(record.value('sex')))
            SNILS   = formatSNILS(record.value('SNILS'))
            policy  = ' '.join([forceString(record.value('policySerial')), forceString(record.value('policyNumber')), forceString(record.value('policyInsurer'))])
            document= ' '.join([forceString(record.value('documentSerial')), forceString(record.value('documentNumber'))])
            address = forceString(record.value('address'))
            i = table.addRow()
            table.setText(i, 0, n)
            table.setText(i, 1, SNILS)
            table.setText(i, 2, name)
            table.setText(i, 3, birthDate)
            table.setText(i, 4, sex)
            table.setText(i, 5, policy)
            table.setText(i, 6, document)
            table.setText(i, 7, address)
        return doc
