# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui
from PyQt4.QtCore import pyqtSignature, QDate, QDateTime, QLocale

from library.Utils            import firstMonthDay, forceBool, forceDate, forceDouble, forceInt, forceRef, forceString, forceStringEx, formatName, lastMonthDay
from Events.ActionServiceType import CActionServiceType
from Events.Utils             import getWorkEventTypeFilter
from Orgs.Utils               import getOrgStructureDescendants
from Reports.Report           import CReport
from Reports.ReportBase       import CReportBase, createTable
from Events.EventTypeListEditorDialog import CEventTypeListEditorDialog

from Ui_FinanceSummarySetupDialog import Ui_FinanceSummarySetupDialog


def getCond(params):
    db = QtGui.qApp.db
    tableAccount = db.table('Account')
    tableContract = db.table('Contract')
    tableAccountItem = db.table('Account_Item')
    tablePerson = db.table('Person')
    tableClientWork   = db.table('ClientWork')
    tableAction = db.table('Action')
    tableVisit = db.table('Visit')
    tableEvent = db.table('Event')
    tableClient = db.table('Client')

    cond = [tableAccountItem['reexposeItem_id'].isNull(),
            tableAccountItem['deleted'].eq(0),
            tableAccount['deleted'].eq(0),
           ]
    if params.get('accountItemIdList',  None) is not None:
        cond.append(tableAccountItem['id'].inlist(params['accountItemIdList']))
    else:
        if params.get('accountIdList', None)is not None:
            cond.append(tableAccountItem['master_id'].inlist(params['accountIdList']))
        elif params.get('accountId', None):
            cond.append(tableAccountItem['master_id'].eq(params['accountId']))
        begDate = params.get('begDate', None)
        endDate = params.get('endDate', None)
        if not params.get('isPeriodOnService', False):
            if begDate:
                cond.append(tableAccount['date'].ge(begDate))
            if endDate:
                cond.append(tableAccount['date'].lt(endDate.addDays(1)))
        if params.get('orgInsurerId', None):
            print '''params['orgInsurerId'] = %s'''%params['orgInsurerId']
            cond.append(db.joinOr([db.joinAnd([tableContract['id'].isNotNull(),
                                               tableContract['deleted'].eq(0),
                                               tableContract['payer_id'].eq(params['orgInsurerId'])
                                               ]),
                                   db.joinAnd([tableAccount['id'].isNotNull(),
                                               tableAccount['deleted'].eq(0),
                                               tableAccount['payer_id'].eq(params['orgInsurerId']) ])
                                   ]))
    if params.get('eventTypeId', None):
        cond.append(tableEvent['eventType_id'].eq(params['eventTypeId']))
    eventTypeList = params.get('eventTypeList', [])
    if eventTypeList:
        cond.append(tableEvent['eventType_id'].inlist(eventTypeList))
    if params.get('personId', None):
        cond.append(tablePerson['id'].eq(params['personId']))
    else:
        if params.get('orgStructureId', None):
            cond.append(tablePerson['orgStructure_id'].inlist(getOrgStructureDescendants(params['orgStructureId'])))
        else:
            cond.append(db.joinOr([tablePerson['org_id'].eq(QtGui.qApp.currentOrgId()),
                                   tablePerson['org_id'].isNull()]))
        if params.get('specialityId', None):
            cond.append(tablePerson['speciality_id'].eq(params['specialityId']))

    if params.get('confirmation', None):
        if params.get('confirmationType', 0) == 1:
            cond.append(tableAccountItem['date'].isNotNull())
            cond.append(tableAccountItem['number'].ne(''))
            cond.append(tableAccountItem['refuseType_id'].isNull())
        elif params.get('confirmationType', 0) == 2:
            cond.append(tableAccountItem['date'].isNotNull())
            cond.append(tableAccountItem['number'].ne(''))
            cond.append(tableAccountItem['refuseType_id'].isNotNull())
        else:
            cond.append(tableAccountItem['number'].eq(''))
        condAIDate = []
        if params.get('confirmationBegDate', None):
            condAIDate.append(tableAccountItem['date'].ge(params['confirmationBegDate']))
        if params.get('confirmationEndDate', None):
            condAIDate.append(tableAccountItem['date'].lt(params['confirmationEndDate'].addDays(1)))
        if params.get('confirmationType', 0) in [1, 2]:
            if condAIDate:
                cond.append(db.joinAnd(condAIDate))
        else:
            if condAIDate:
                cond.append(db.joinOr([tableAccountItem['date'].isNull(), db.joinAnd(condAIDate)]))
        if params.get('refuseType', None):
            cond.append(tableAccountItem['refuseType_id'].eq(params['refuseType']))
    detailClient = params.get('detailClient', False)
    filterSex    = params.get('filterSex', 0)
    filterBegAge = params.get('filterBegAge', 0)
    filterEndAge = params.get('filterEndAge', 150)
    if filterSex:
        cond.append(tableClient['sex'].eq(filterSex))
    if filterEndAge and filterBegAge <= filterEndAge:
        cond.append('''age(Client.birthDate, IF(Event.execDate IS NOT NULL, DATE(Event.execDate), DATE(%s))) >= %d
                           AND age(Client.birthDate, IF(Event.execDate IS NOT NULL, DATE(Event.execDate), DATE(%s))) < %d''' % (db.formatDate(QDate.currentDate()), filterBegAge, db.formatDate(QDate.currentDate()), filterEndAge+1))
    clientOrganisationId = params.get('clientOrganisationId', None)
    freeInputWork = params.get('freeInputWork', False)
    freeInputWorkValue = params.get('freeInputWorkValue', '')
    if clientOrganisationId or (freeInputWork and freeInputWorkValue):
        workSubCond = []
        if clientOrganisationId:
            workSubCond.append(tableClientWork['org_id'].eq(clientOrganisationId))
        if (freeInputWork and freeInputWorkValue):
            workSubCond.append(tableClientWork['freeInput'].like(freeInputWorkValue))
        workCond = [db.joinOr(workSubCond),
                    'ClientWork.id = getClientWorkId(Event.client_id)'
                   ]
        cond.append(
                    '''EXISTS(
                    SELECT 1
                    FROM ClientWork
                    WHERE %s
                    )'''%db.joinAnd(workCond)
                   )

    return db.joinAnd(cond)


def selectDataByEvents(params):
    db = QtGui.qApp.db
    isPeriodOnService         = params.get('isPeriodOnService', False)
    detailClient              = params.get('detailClient', False)
    detailClientSex           = params.get('detailClientSex', False)
    detailClientAge           = params.get('detailClientAge', False)
    isSharingOfEventsByVisits = params.get('isSharingOfEventsByVisits', False)
    if isSharingOfEventsByVisits:
        AIEventsByVisits = u'''Account_Item.amount/IF((SELECT COUNT(1) FROM Visit WHERE Visit.event_id = Event.id AND Visit.deleted=0) > 0, (SELECT COUNT(1) FROM Visit WHERE Visit.event_id = Event.id AND Visit.deleted=0), 1) AS amount,
                               Account_Item.sum/IF((SELECT COUNT(1) FROM Visit WHERE Visit.event_id = Event.id AND Visit.deleted=0) > 0, (SELECT COUNT(1) FROM Visit WHERE Visit.event_id = Event.id AND Visit.deleted=0), 1) AS sum,
                               Account_Item.uet/IF((SELECT COUNT(1) FROM Visit WHERE Visit.event_id = Event.id AND Visit.deleted=0) > 0, (SELECT COUNT(1) FROM Visit WHERE Visit.event_id = Event.id AND Visit.deleted=0), 1) AS uet,'''
        queryEventsByVisits = u'''LEFT JOIN Visit ON Visit.event_id = Event.id
                                  LEFT JOIN Person ON Person.id = IF(Visit.id IS NULL, Event.execPerson_id, Visit.person_id)'''
    else:
        AIEventsByVisits = u'''Account_Item.amount AS amount,
                               Account_Item.sum AS sum,
                               Account_Item.uet AS uet,'''
        queryEventsByVisits = u'''LEFT JOIN Person ON Person.id = Event.execPerson_id'''
    cond = getCond(params)
    if isPeriodOnService:
        condDate = []
        begDate = params.get('begDate', None)
        endDate = params.get('endDate', None)
        tableEvent = db.table('Event')
        if begDate:
            condDate.append(tableEvent['execDate'].ge(begDate))
        if endDate:
            condDate.append(tableEvent['execDate'].lt(endDate.addDays(1)))
        cond += u' AND ' + db.joinAnd(condDate) if condDate else ''

    stmt="""
SELECT
    rbSpeciality.name as specialityName,
    Account_Item.visit_id,
    Account_Item.action_id,
    Account_Item.serviceDate,
    Event.MES_id,
    %s
    mes.MES.code AS mesCode,
    mes.MES.name AS mesName,
    rbMedicalAidType.code AS matCode,
    rbMedicalAidType.name AS matName,
    Person.speciality_id as speciality_id,
    Person.lastName, Person.firstName, Person.patrName, Person.code, Person.id AS personId,
    rbService.id AS serviceId, rbService.name AS serviceName, rbService.code AS serviceCode,
    %s
    (Account_Item.date IS NOT NULL AND Account_Item.number != '' AND Account_Item.refuseType_id IS NULL) AS exposed,
    (Account_Item.date IS NOT NULL AND Account_Item.number != '' AND Account_Item.refuseType_id IS NOT NULL) AS refused,
    Contract_Tariff.federalPrice,
    Contract_Tariff.federalLimitation,
    rbFinance.id AS financeId,
    rbFinance.code AS financeCode,
    rbFinance.name AS financeName,
    Contract.payer_id,
    Organisation.shortName

FROM Account_Item
LEFT JOIN Contract_Tariff ON Account_Item.tariff_id = Contract_Tariff.id
LEFT JOIN Account         ON Account.id = Account_Item.master_id
LEFT JOIN Contract        ON Contract.id = Account.contract_id
LEFT JOIN Organisation    ON Organisation.id = Contract.payer_id
LEFT JOIN rbFinance       ON (rbFinance.id = Contract.finance_id AND Contract.deleted = 0)
LEFT JOIN rbService       ON rbService.id = Account_Item.service_id
LEFT JOIN Event           ON Event.id = Account_Item.event_id
%s
LEFT JOIN rbSpeciality    ON rbSpeciality.id = Person.speciality_id
LEFT JOIN mes.MES         ON mes.MES.id = Event.MES_id
LEFT JOIN EventType       ON EventType.id = Event.eventType_id
LEFT JOIN rbMedicalAidType ON rbMedicalAidType.id = EventType.medicalAidType_id
INNER JOIN Client         ON Client.id = Event.client_id

WHERE
    Account_Item.visit_id IS NULL
    AND Account_Item.action_id IS NULL
    AND (EventType.id IS NULL OR EventType.deleted = 0)
    AND %s
    """
    db = QtGui.qApp.db
    return db.query(stmt % ('''Event.client_id,
    Client.lastName AS clientLastName,
    Client.firstName AS clientFirstName,
    Client.patrName AS clientPatrName,
    Client.id AS clientId,''' if detailClient else '', AIEventsByVisits, queryEventsByVisits, cond))


def selectDataByVisits(params):
    db = QtGui.qApp.db
    isPeriodOnService = params.get('isPeriodOnService', False)
    cond = getCond(params)
    if isPeriodOnService:
        condDate = []
        begDate = params.get('begDate', None)
        endDate = params.get('endDate', None)
        tableVisit = db.table('Visit')
        if begDate:
            condDate.append(tableVisit['date'].ge(begDate))
        if endDate:
            condDate.append(tableVisit['date'].lt(endDate.addDays(1)))
        cond += u' AND ' + db.joinAnd(condDate) if condDate else ''
    detailClient    = params.get('detailClient', False)
    detailClientSex = params.get('detailClientSex', False)
    detailClientAge = params.get('detailClientAge', False)
    stmt="""
SELECT
    rbSpeciality.name as specialityName,
    Account_Item.visit_id,
    Account_Item.action_id,
    Account_Item.serviceDate,
    Event.MES_id,
    %s
    mes.MES.code AS mesCode,
    mes.MES.name AS mesName,
    rbMedicalAidType.code AS matCode,
    rbMedicalAidType.name AS matName,
    Person.speciality_id as speciality_id,
    Person.lastName, Person.firstName, Person.patrName, Person.code, Person.id AS personId,
    rbService.id AS serviceId, rbService.name AS serviceName, rbService.code AS serviceCode,
    Account_Item.amount AS amount,
    Account_Item.sum AS sum,
    Account_Item.uet AS uet,
    (Account_Item.date IS NOT NULL AND Account_Item.number != '' AND Account_Item.refuseType_id IS NULL) AS exposed,
    (Account_Item.date IS NOT NULL AND Account_Item.number != '' AND Account_Item.refuseType_id IS NOT NULL) as refused,
    Contract_Tariff.federalPrice,
    Contract_Tariff.federalLimitation,
    rbFinance.id AS financeId,
    rbFinance.code AS financeCode,
    rbFinance.name AS financeName,
    Contract.payer_id,
    Organisation.shortName

FROM Account_Item
LEFT JOIN Contract_Tariff ON Account_Item.tariff_id = Contract_Tariff.id
LEFT JOIN Account         ON Account.id = Account_Item.master_id
LEFT JOIN Contract        ON Contract.id = Account.contract_id
LEFT JOIN Organisation    ON Organisation.id = Contract.payer_id
LEFT JOIN rbFinance       ON (rbFinance.id = Contract.finance_id AND Contract.deleted = 0)
LEFT JOIN rbService       ON rbService.id = Account_Item.service_id
LEFT JOIN Event           ON Event.id = Account_Item.event_id
LEFT JOIN Visit           ON Visit.id = Account_Item.visit_id
LEFT JOIN Person          ON Person.id = Visit.person_id
LEFT JOIN rbSpeciality    ON rbSpeciality.id = Person.speciality_id
LEFT JOIN mes.MES         ON mes.MES.id = Event.MES_id
LEFT JOIN EventType       ON EventType.id = Event.eventType_id
LEFT JOIN rbMedicalAidType ON rbMedicalAidType.id = EventType.medicalAidType_id
INNER JOIN Client         ON Client.id = Event.client_id
WHERE
    Account_Item.visit_id IS NOT NULL
    AND Account_Item.action_id IS NULL
    AND (EventType.id IS NULL OR EventType.deleted = 0)
    AND %s
    """
    db = QtGui.qApp.db
    return db.query(stmt % ('''Event.client_id,
    Client.lastName AS clientLastName,
    Client.firstName AS clientFirstName,
    Client.patrName AS clientPatrName,
    Client.id AS clientId,''' if detailClient else '', cond))


def selectDataByActions(params):
    db = QtGui.qApp.db
    isPeriodOnService = params.get('isPeriodOnService', False)
    cond = getCond(params)
    if isPeriodOnService:
        condDate = []
        begDate = params.get('begDate', None)
        endDate = params.get('endDate', None)
        tableAction = db.table('Action')
        if begDate:
            condDate.append(tableAction['endDate'].ge(begDate))
        if endDate:
            condDate.append(tableAction['endDate'].lt(endDate.addDays(1)))
        cond += u' AND ' + db.joinAnd(condDate) if condDate else ''
    detailClient    = params.get('detailClient', False)
    detailClientSex = params.get('detailClientSex', False)
    detailClientAge = params.get('detailClientAge', False)
    stmt="""
SELECT
    rbSpeciality.name as specialityName,
    Account_Item.visit_id,
    Account_Item.action_id,
    Account_Item.serviceDate,
    Event.MES_id,
    %s
    mes.MES.code AS mesCode,
    mes.MES.name AS mesName,
    rbMedicalAidType.code AS matCode,
    rbMedicalAidType.name AS matName,
    ActionType.serviceType,
    Person.speciality_id as speciality_id,
    Person.lastName, Person.firstName, Person.patrName, Person.code, Person.id AS personId,
    rbService.id AS serviceId, rbService.name AS serviceName, rbService.code AS serviceCode,
    Account_Item.amount AS amount,
    Account_Item.sum AS sum,
    Account_Item.uet AS uet,
    (Account_Item.date IS NOT NULL AND Account_Item.number != '' AND Account_Item.refuseType_id IS NULL) AS exposed,
    (Account_Item.date IS NOT NULL AND Account_Item.number != '' AND Account_Item.refuseType_id IS NOT NULL) as refused,
    Contract_Tariff.federalPrice,
    Contract_Tariff.federalLimitation,
    rbFinance.id AS financeId,
    rbFinance.code AS financeCode,
    rbFinance.name AS financeName,
    Contract.payer_id,
    Organisation.shortName

FROM Account_Item
LEFT JOIN Contract_Tariff ON Account_Item.tariff_id = Contract_Tariff.id
LEFT JOIN Account         ON Account.id = Account_Item.master_id
LEFT JOIN Contract        ON Contract.id = Account.contract_id
LEFT JOIN Organisation    ON Organisation.id = Contract.payer_id
LEFT JOIN rbFinance       ON (rbFinance.id = Contract.finance_id AND Contract.deleted = 0)
LEFT JOIN rbService       ON rbService.id = Account_Item.service_id
LEFT JOIN Event           ON Event.id = Account_Item.event_id
LEFT JOIN Action          ON Action.id = Account_Item.action_id
LEFT JOIN Person          ON Person.id = Action.person_id
LEFT JOIN rbSpeciality    ON rbSpeciality.id = Person.speciality_id
LEFT JOIN ActionType      ON ActionType.id = Action.actionType_id
LEFT JOIN mes.MES         ON mes.MES.id = Event.MES_id
LEFT JOIN EventType       ON EventType.id = Event.eventType_id
LEFT JOIN rbMedicalAidType ON rbMedicalAidType.id = EventType.medicalAidType_id
INNER JOIN Client         ON Client.id = Event.client_id
WHERE
    Account_Item.visit_id IS NULL
    AND Account_Item.action_id IS NOT NULL
    AND (ActionType.id IS NULL OR ActionType.deleted = 0)
    AND (EventType.id IS NULL OR EventType.deleted = 0)
    AND %s
    """
    db = QtGui.qApp.db
    return db.query(stmt % ('''Event.client_id,
    Client.lastName AS clientLastName,
    Client.firstName AS clientFirstName,
    Client.patrName AS clientPatrName,
    Client.id AS clientId,''' if detailClient else '', cond))


class CFinanceSummaryByDoctors(CReport):
    def __init__(self, parent):
        CReport.__init__(self, parent)
        self.setTitle(u'Финансовая сводка по врачам')


    def build(self, description, params):
        reportFinanceData = {}
        reportPayerData = {}
        reportData = {}
        detailService   = params.get('detailService', False)
        detailFedTariff = params.get('detailFedTariff', False)
        detailClient    = params.get('detailClient', False)
        detailFinance   = params.get('detailFinance', False)
        detailDate      = params.get('detailDate', False)
        detalilPayer    = params.get('detailPayer', False)
        isSharingOfEventsByVisits = params.get('isSharingOfEventsByVisits', False)
        dataLineStep = 4 if detailFedTariff else 3
        reportRowSize = 12 if detailFedTariff else 9

        def processQuery(query, reportData):
            while query.next():
                record = query.record()
                specialityName = forceString(record.value('specialityName'))
                serviceDate = forceDate(record.value('serviceDate'))
                specialityId = forceInt(record.value('speciality_id'))
                financeId = forceRef(record.value('financeId'))
                financeCode = forceString(record.value('financeCode'))
                financeName = forceString(record.value('financeName'))
                visitId = forceRef(record.value('visit_id'))
                actionId = forceRef(record.value('action_id'))
                lastName = forceString(record.value('lastName'))
                firstName = forceString(record.value('firstName'))
                patrName = forceString(record.value('patrName'))
                personId = forceRef(record.value('personId'))
                code = forceString(record.value('code'))
                serviceId = forceRef(record.value('serviceId'))
                serviceName = forceString(record.value('serviceName'))
                serviceCode = forceString(record.value('serviceCode'))
                mesCodeName = forceString(record.value('mesCode')) + u'-' + forceString(record.value('mesName'))
                matCodeName = forceString(record.value('matCode')) + u'-' + forceString(record.value('matName'))
                serviceType = forceInt(record.value('serviceType'))
                clientId = forceRef(record.value('clientId'))
                clientInfo = u' '.join(clName for clName in [forceString(record.value('clientLastName')),
                                                             forceString(record.value('clientFirstName')),
                                                             forceString(record.value('clientPatrName')),
                                                             forceString(record.value('clientId'))])
                amount = forceDouble(record.value('amount'))
                if amount == int(amount):
                   amount = int(amount)
                sum = forceDouble(record.value('sum'))
                uet = forceDouble(record.value('uet'))
                exposed = forceBool(record.value('exposed'))
                refused = forceBool(record.value('refused'))
                federalPrice = forceDouble(record.value('federalPrice'))
                federalLimitation = forceInt(record.value('federalLimitation'))
                if detailDate:
                    serviceKey = (serviceDate, serviceId, serviceCode, serviceName if serviceId else u'Услуга не указана')
                else:
                    serviceKey = (serviceId, serviceCode, serviceName if serviceId else u'Услуга не указана') if detailService else None
                name = formatName(lastName, firstName, patrName)
                if not actionId and not visitId:
                    serviceTypeName = u'МЭС'
                elif actionId:
                    serviceTypeName = CActionServiceType.text(serviceType)
                else:
                    serviceTypeName = u''
                key = (specialityName if specialityName else u'Без указания специальности',
                       name if name else u'Без указания врача',
                       code,
                       serviceKey,
                       serviceTypeName,
                       matCodeName,
                       clientInfo,
                       personId)
                if detalilPayer:
                    payerId = forceRef(record.value('payer_id'))
                    payerName = forceString(record.value('shortName'))
                    if not detailFinance:
                        reportPayerData = reportData.setdefault((payerId, payerName), {})
                        reportLine = reportPayerData.setdefault(key, [0]*reportRowSize)
                    else:
                        reportPayerData = reportData.setdefault((payerId, payerName), {})
                        reportFinanceData = reportPayerData.setdefault((financeId, financeCode, financeName), {})
                        reportLine = reportFinanceData.setdefault(key, [0]*reportRowSize)
                elif detailFinance:
                    reportFinanceData = reportData.setdefault((financeId, financeCode, financeName), {})
                    reportLine = reportFinanceData.setdefault(key, [0]*reportRowSize)
                else:
                    reportLine = reportData.setdefault(key, [0]*reportRowSize)
                if detailFedTariff:
                    actualAmount = federalLimitation if amount > federalLimitation else amount
                    resultFederalPrice = actualAmount*federalPrice
                    reportLine[0] += amount
                    reportLine[1] += uet
                    reportLine[2] += sum
                    reportLine[3] += resultFederalPrice
                    if exposed:
                        reportLine[4] += amount
                        reportLine[5] += uet
                        reportLine[6] += sum
                        reportLine[7] += resultFederalPrice
                    if refused:
                        reportLine[8]  += amount
                        reportLine[9]  += uet
                        reportLine[10] += sum
                        reportLine[11] += resultFederalPrice
                else:
                    reportLine[0] += amount
                    reportLine[1] += uet
                    reportLine[2] += sum
                    if exposed:
                        reportLine[3] += amount
                        reportLine[4] += uet
                        reportLine[5] += sum
                    if refused:
                        reportLine[6] += amount
                        reportLine[7] += uet
                        reportLine[8] += sum
                if detalilPayer:
                    if not detailFinance:
                        reportPayerData[key] = reportLine
                        reportData[(payerId, payerName)] = reportPayerData
                    else:
                        reportFinanceData[key] = reportLine
                        reportPayerData[(financeId, financeCode, financeName)] = reportFinanceData
                        reportData[(payerId, payerName)] = reportPayerData
                elif detailFinance:
                    reportFinanceData[key] = reportLine
                    reportData[(financeId, financeCode, financeName)] = reportFinanceData
                else:
                    reportData[key] = reportLine
            return reportData

        query = selectDataByEvents(params)
        reportData = processQuery(query, reportData)
        query = selectDataByVisits(params)
        reportData = processQuery(query, reportData)
        query = selectDataByActions(params)
        reportData = processQuery(query, reportData)

        # now text
        doc = QtGui.QTextDocument()
        cursor = QtGui.QTextCursor(doc)

        cursor.setCharFormat(CReportBase.ReportTitle)
        cursor.insertText(self.title())
        cursor.insertBlock()
        cursor.setCharFormat(CReportBase.ReportBody)
        descriptionIns = (u'Делить стоимость События по визитам.' if isSharingOfEventsByVisits else u'Не делить стоимость События по визитам.') + u'\n' + description
        cursor.insertText(descriptionIns)
        cursor.insertBlock()

        tableColumns = [
                          ('8%',  [ u'Врач',     u'ФИО'       ], CReportBase.AlignLeft ),
                          ('6%',  [ u'',         u'код'       ], CReportBase.AlignLeft ),
                          ('6%',  [ u'',         u'вид услуги/МЭС'], CReportBase.AlignLeft ),
                          ('6%',  [ u'',         u'тип медицинской помощи'], CReportBase.AlignLeft ),
                          ('6%',  [ u'Всего',    u'кол-во'    ], CReportBase.AlignRight ),
                          ('6%',  [ u'',         u'УЕТ'       ], CReportBase.AlignRight ),
                          ('6%',  [ u'',         u'руб'       ], CReportBase.AlignRight ),
                          ('6%',  [ u'Оплачено', u'кол-во'    ], CReportBase.AlignRight ),
                          ('6%',  [ u'',         u'УЕТ'       ], CReportBase.AlignRight ),
                          ('6%',  [ u'',         u'руб'       ], CReportBase.AlignRight ),
                          ('6%',  [ u'Отказано', u'кол-во'    ], CReportBase.AlignRight ),
                          ('6%',  [ u'',         u'УЕТ'       ], CReportBase.AlignRight ),
                          ('6%',  [ u'',         u'руб'       ], CReportBase.AlignRight ),
                       ]

        if detailFedTariff:
            tableColumns.insert(5,  ('6%', [ u'', u'фед.тариф'], CReportBase.AlignRight ))
            tableColumns.insert(9,  ('6%', [ u'', u'фед.тариф'], CReportBase.AlignRight ))
            tableColumns.insert(13, ('6%', [ u'', u'фед.тариф'], CReportBase.AlignRight ))
        if detailDate:
            tableColumns.insert(2,  ('6%', [ u'', u'Дата'], CReportBase.AlignLeft ))
        self.tableColumnsLen = len(tableColumns)
        table = createTable(cursor, tableColumns)
        table.mergeCells(0, 0,  1, 5 if detailDate else 4)
        table.mergeCells(0, 5 if detailDate else 4,  1, dataLineStep)
        if detailDate:
            table.mergeCells(0, 9  if detailFedTariff else 8,  1, dataLineStep)
            table.mergeCells(0, 13 if detailFedTariff else 11,  1, dataLineStep)
        else:
            table.mergeCells(0, 8  if detailFedTariff else 7,  1, dataLineStep)
            table.mergeCells(0, 12 if detailFedTariff else 10,  1, dataLineStep)
        prevSpecialityName = None
        prevDoctor = None
        totalBySpeciality = [0]*reportRowSize
        totalByReport     = [0]*reportRowSize
        totalByDoctor     = [0]*reportRowSize
        locale = QLocale()
        font = CReportBase.TableTotal if detailService else None
        if detalilPayer:
            payerKeys = reportData.keys()
            payerKeys.sort()
            prevPayerId = None
            prevPayerRow = 2
            for payerId, payerName in payerKeys:
                if detailFinance:
                    reportPayerData = reportData.get((payerId, payerName), {})
                    if not reportPayerData:
                        continue
                else:
                    reportPayerData = reportData.get((payerId, payerName), {})
                    if not reportPayerData:
                        continue
                if prevPayerId != payerId:
                    i = table.addRow()
                    table.setText(i, 0, payerName, CReportBase.TableTotal, blockFormat=CReportBase.AlignCenter)
                    prevPayerId = payerId
                    prevPayerRow = i
                    table.mergeCells(prevPayerRow, 0,  1, self.tableColumnsLen)
                if detailFinance:
                    financeKeys = reportPayerData.keys()
                    financeKeys.sort()
                    prevFinanceId = None
                    prevFinanceRow = 2
                    for financeId, financeCode, financeName in financeKeys:
                        firstFinance = True
                        totalByFinance = [0]*reportRowSize
                        reportFinanceData = reportPayerData.get((financeId, financeCode, financeName), {})
                        if not reportFinanceData:
                            continue
                        if prevFinanceId != financeId:
                            i = table.addRow()
                            table.setText(i, 0, financeName, CReportBase.TableTotal, blockFormat=CReportBase.AlignCenter)
                            prevFinanceId = financeId
                            prevFinanceRow = i
                            table.mergeCells(prevFinanceRow, 0,  1, self.tableColumnsLen)
                        keys = reportFinanceData.keys()
                        keys.sort()
                        doctorNamePrev = None
                        clientInfoPrev = u''
                        doctorNameRow = 3
                        clientNameRow = 4
                        for key in keys:
                            specialityName = key[0]
                            doctorName  = key[1]
                            doctorCode  = key[2]
                            serviceTypeName = key[4]
                            matCodeName = key[5]
                            clientInfo  = key[6]
                            personId    = key[7]
                            if prevDoctor and prevDoctor != personId:
                                if not firstFinance:
                                    self.addTotal(table, 4*' '+u'По врачу', totalByDoctor, locale, dataLineStep, detailClient, detailDate)
                                totalByDoctor = [0]*reportRowSize
                            if prevSpecialityName != specialityName:
                                if prevSpecialityName is not None:
                                    if not firstFinance:
                                        self.addTotal(table, u'всего по специальности', totalBySpeciality, locale, dataLineStep, detailClient, detailDate)
                                    totalBySpeciality = [0]*reportRowSize
                                self.addSpecialityHeader(table, specialityName)
                                prevSpecialityName = specialityName
                            firstFinance = False
                            if not detailService or prevDoctor != personId:
                                i = table.addRow()
                                nameShift = 4*' ' if detailService else ''
                                if doctorNamePrev != personId:
                                    table.setText(i, 0, nameShift+doctorName, font)
                                    table.setText(i, 1, doctorCode, font)
                                    if not detailService:
                                        doctorNamePrev = personId
                                        doctorNameRow = i
                                if not detailService:
                                    table.setText(i, 2, serviceTypeName, font)
                                    table.setText(i, 3, matCodeName, font)
                                if detailService:
                                    table.mergeCells(i, 2, 1, self.tableColumnsLen-2)
                                    prevDoctor = personId
                                if not detailClient:
                                    table.mergeCells(doctorNameRow, 0,  i-doctorNameRow+1, 1)
                                    table.mergeCells(doctorNameRow, 1,  i-doctorNameRow+1, 1)
                            if detailClient:
                                if clientInfoPrev != clientInfo:
                                    i = table.addRow()
                                    table.setText(i, 1 if detailDate else 0, clientInfo)
                                    clientInfoPrev = clientInfo
                                    clientNameRow = i
                                    table.mergeCells(clientNameRow, 0,  1, self.tableColumnsLen)
                            if detailService:
                                i = table.addRow()
                                if detailDate:
                                    serviceDate, serviceId, serviceCode, serviceName = key[3]
                                    table.setText(i, 2, forceString(serviceDate))
                                else:
                                    serviceId, serviceCode, serviceName = key[3]
                                table.setText(i, 0, 8*' '+serviceName)
                                table.setText(i, 1, serviceCode)
                                table.setText(i, 3 if detailDate else 2, serviceTypeName)
                                table.setText(i, 4 if detailDate else 3, matCodeName)
                            if detailClient and not detailService:
                                i = table.addRow()
                                table.mergeCells(clientNameRow+1, 0, i-clientNameRow, 4)
                            reportLine = reportFinanceData[key]
                            for j in xrange(0, reportRowSize, dataLineStep):
                                table.setText(i, j+(5 if detailDate else 4), locale.toString(float(reportLine[j]), 'f', 2))
                            for j in xrange(1, reportRowSize, dataLineStep):
                                table.setText(i, j+(5 if detailDate else 4), locale.toString(float(reportLine[j]), 'f', 2))
                            for j in xrange(2, reportRowSize, dataLineStep):
                                table.setText(i, j+(5 if detailDate else 4), locale.toString(float(reportLine[j]), 'f', 2))
                            if detailFedTariff:
                                for j in xrange(3, reportRowSize, dataLineStep):
                                    table.setText(i, j+(5 if detailDate else 4), locale.toString(float(reportLine[j]), 'f', 2))
                            for j in xrange(reportRowSize):
                                totalBySpeciality[j] += reportLine[j]
                                totalByReport[j] += reportLine[j]
                                totalByDoctor[j] += reportLine[j]
                                totalByFinance[j] += reportLine[j]
                        if detailService:
                            self.addTotal(table, 4*' '+u'По врачу', totalByDoctor, locale, dataLineStep, detailClient, detailDate)
                        self.addTotal(table, u'всего по специальности', totalBySpeciality, locale, dataLineStep, detailClient, detailDate)
                        self.addTotal(table, u'Всего по типу финансирования %s'%(financeName), totalByFinance, locale, dataLineStep, detailClient, detailDate)
                    self.addTotal(table, u'Всего', totalByReport, locale, dataLineStep, detailClient, detailDate)
                    totalByReport = [0]*reportRowSize
                else:
                    keys = reportPayerData.keys()
                    keys.sort()
                    doctorNamePrev = None
                    clientInfoPrev = u''
                    doctorNameRow = 2
                    clientNameRow = 3
                    for key in keys:
                        specialityName = key[0]
                        doctorName  = key[1]
                        doctorCode  = key[2]
                        serviceTypeName = key[4]
                        matCodeName = key[5]
                        clientInfo  = key[6]
                        personId    = key[7]
                        if prevDoctor and prevDoctor != personId:
                            self.addTotal(table, 4*' '+u'По врачу', totalByDoctor, locale, dataLineStep, detailClient, detailDate)
                            totalByDoctor = [0]*reportRowSize
                        if prevSpecialityName != specialityName:
                            if prevSpecialityName is not None:
                                self.addTotal(table, u'всего по специальности', totalBySpeciality, locale, dataLineStep, detailClient, detailDate)
                            totalBySpeciality = [0]*reportRowSize
                            self.addSpecialityHeader(table, specialityName)
                            prevSpecialityName = specialityName
                        if not detailService or prevDoctor != personId:
                            i = table.addRow()
                            nameShift = 4*' ' if detailService else ''
                            if doctorNamePrev != personId:
                                table.setText(i, 0, nameShift+doctorName, font)
                                table.setText(i, 1, doctorCode, font)
                                if not detailService:
                                    doctorNamePrev = personId
                                    doctorNameRow = i
                            if not detailService:
                                table.setText(i, 3 if detailDate else 2, serviceTypeName, font)
                                table.setText(i, 4 if detailDate else 3, matCodeName, font)
                            if detailService:
                                table.mergeCells(i, 2, 1, self.tableColumnsLen-2)
                                prevDoctor = personId
                            if not detailClient:
                                table.mergeCells(doctorNameRow, 0,  i-doctorNameRow+1, 1)
                                table.mergeCells(doctorNameRow, 1,  i-doctorNameRow+1, 1)
                        if detailClient:
                            if clientInfoPrev != clientInfo:
                                i = table.addRow()
                                table.setText(i, 2 if detailDate else 1 if detailDate else 0, clientInfo)
                                clientInfoPrev = clientInfo
                                clientNameRow = i
                                table.mergeCells(clientNameRow, 0,  1, self.tableColumnsLen)
                        if detailService:
                            i = table.addRow()
                            if detailDate:
                                serviceDate, serviceId, serviceCode, serviceName = key[3]
                                table.setText(i, 2, forceString(serviceDate))
                            else:
                                serviceId, serviceCode, serviceName = key[3]
                            table.setText(i, 0, 8*' '+serviceName)
                            table.setText(i, 1, serviceCode)
                            table.setText(i, 3 if detailDate else 2, serviceTypeName)
                            table.setText(i, 4 if detailDate else 3, matCodeName)
                        if detailClient and not detailService:
                            i = table.addRow()
                            table.mergeCells(clientNameRow+1, 0, i-clientNameRow, 4)
                        reportLine = reportPayerData[key]
                        for j in xrange(0, reportRowSize, dataLineStep):
                            table.setText(i, j+(5 if detailDate else 4), locale.toString(float(reportLine[j]), 'f', 2))
                        for j in xrange(1, reportRowSize, dataLineStep):
                            table.setText(i, j+(5 if detailDate else 4), locale.toString(float(reportLine[j]), 'f', 2))
                        for j in xrange(2, reportRowSize, dataLineStep):
                            table.setText(i, j+(5 if detailDate else 4), locale.toString(float(reportLine[j]), 'f', 2))
                        if detailFedTariff:
                            for j in xrange(3, reportRowSize, dataLineStep):
                                table.setText(i, j+(5 if detailDate else 4), locale.toString(float(reportLine[j]), 'f', 2))
                        for j in xrange(reportRowSize):
                            totalBySpeciality[j] += reportLine[j]
                            totalByReport[j] += reportLine[j]
                            totalByDoctor[j] += reportLine[j]
                    if detailService:
                        self.addTotal(table, 4*' '+u'По врачу', totalByDoctor, locale, dataLineStep, detailClient, detailDate)
                    self.addTotal(table, u'всего по специальности', totalBySpeciality, locale, dataLineStep, detailClient, detailDate)
                    self.addTotal(table, u'Всего', totalByReport, locale, dataLineStep, detailClient, detailDate)
                    prevSpecialityName = None
                    totalByReport = [0]*reportRowSize
        else:
            if detailFinance:
                financeKeys = reportData.keys()
                financeKeys.sort()
                prevFinanceId = None
                prevFinanceRow = 2
                for financeId, financeCode, financeName in financeKeys:
                    firstFinance = True
                    totalByFinance = [0]*reportRowSize
                    reportFinanceData = reportData.get((financeId, financeCode, financeName), {})
                    if not reportFinanceData:
                        continue
                    if prevFinanceId != financeId:
                        i = table.addRow()
                        table.setText(i, 0, financeName, CReportBase.TableTotal, blockFormat=CReportBase.AlignCenter)
                        prevFinanceId = financeId
                        prevFinanceRow = i
                        table.mergeCells(prevFinanceRow, 0,  1, self.tableColumnsLen)
                    keys = reportFinanceData.keys()
                    keys.sort()
                    doctorNamePrev = None
                    clientInfoPrev = u''
                    doctorNameRow = 3
                    clientNameRow = 4
                    for key in keys:
                        specialityName = key[0]
                        doctorName  = key[1]
                        doctorCode  = key[2]
                        serviceTypeName = key[4]
                        matCodeName = key[5]
                        clientInfo  = key[6]
                        personId    = key[7]
                        if prevDoctor and prevDoctor != personId:
                            if not firstFinance:
                                self.addTotal(table, 4*' '+u'По врачу', totalByDoctor, locale, dataLineStep, detailClient, detailDate)
                            totalByDoctor = [0]*reportRowSize
                        if prevSpecialityName != specialityName:
                            if prevSpecialityName is not None:
                                if not firstFinance:
                                    self.addTotal(table, u'всего по специальности', totalBySpeciality, locale, dataLineStep, detailClient, detailDate)
                                totalBySpeciality = [0]*reportRowSize
                            self.addSpecialityHeader(table, specialityName)
                            prevSpecialityName = specialityName
                        firstFinance = False
                        if not detailService or prevDoctor != personId:
                            i = table.addRow()
                            nameShift = 4*' ' if detailService else ''
                            if doctorNamePrev != personId:
                                table.setText(i, 0, nameShift+doctorName, font)
                                table.setText(i, 1, doctorCode, font)
                                if not detailService:
                                    doctorNamePrev = personId
                                    doctorNameRow = i
                            if not detailService:
                                table.setText(i, 2, serviceTypeName, font)
                                table.setText(i, 3, matCodeName, font)
                            if detailService:
                                table.mergeCells(i, 2, 1, self.tableColumnsLen-2)
                                prevDoctor = personId
                            if not detailClient:
                                table.mergeCells(doctorNameRow, 0,  i-doctorNameRow+1, 1)
                                table.mergeCells(doctorNameRow, 1,  i-doctorNameRow+1, 1)
                        if detailClient:
                            if clientInfoPrev != clientInfo:
                                i = table.addRow()
                                table.setText(i, 1 if detailDate else 0, clientInfo)
                                clientInfoPrev = clientInfo
                                clientNameRow = i
                                table.mergeCells(clientNameRow, 0,  1, self.tableColumnsLen)
                        if detailService:
                            i = table.addRow()
                            if detailDate:
                                serviceDate, serviceId, serviceCode, serviceName = key[3]
                                table.setText(i, 2, forceString(serviceDate))
                            else:
                                serviceId, serviceCode, serviceName = key[3]
                            table.setText(i, 0, 8*' '+serviceName)
                            table.setText(i, 1, serviceCode)
                            table.setText(i, 3 if detailDate else 2, serviceTypeName)
                            table.setText(i, 4 if detailDate else 3, matCodeName)
                        if detailClient and not detailService:
                            i = table.addRow()
                            table.mergeCells(clientNameRow+1, 0, i-clientNameRow, 4)
                        reportLine = reportFinanceData[key]
                        for j in xrange(0, reportRowSize, dataLineStep):
                            table.setText(i, j+(5 if detailDate else 4), locale.toString(float(reportLine[j]), 'f', 2))
                        for j in xrange(1, reportRowSize, dataLineStep):
                            table.setText(i, j+(5 if detailDate else 4), locale.toString(float(reportLine[j]), 'f', 2))
                        for j in xrange(2, reportRowSize, dataLineStep):
                            table.setText(i, j+(5 if detailDate else 4), locale.toString(float(reportLine[j]), 'f', 2))
                        if detailFedTariff:
                            for j in xrange(3, reportRowSize, dataLineStep):
                                table.setText(i, j+(5 if detailDate else 4), locale.toString(float(reportLine[j]), 'f', 2))
                        for j in xrange(reportRowSize):
                            totalBySpeciality[j] += reportLine[j]
                            totalByReport[j] += reportLine[j]
                            totalByDoctor[j] += reportLine[j]
                            totalByFinance[j] += reportLine[j]
                    if detailService:
                        self.addTotal(table, 4*' '+u'По врачу', totalByDoctor, locale, dataLineStep, detailClient, detailDate)
                    self.addTotal(table, u'всего по специальности', totalBySpeciality, locale, dataLineStep, detailClient, detailDate)
                    self.addTotal(table, u'Всего по типу финансирования %s'%(financeName), totalByFinance, locale, dataLineStep, detailClient, detailDate)
                self.addTotal(table, u'Всего', totalByReport, locale, dataLineStep, detailClient, detailDate)
            else:
                keys = reportData.keys()
                keys.sort()
                doctorNamePrev = None
                clientInfoPrev = u''
                doctorNameRow = 2
                clientNameRow = 3
                for key in keys:
                    specialityName = key[0]
                    doctorName  = key[1]
                    doctorCode  = key[2]
                    serviceTypeName = key[4]
                    matCodeName = key[5]
                    clientInfo  = key[6]
                    personId    = key[7]
                    if prevDoctor and prevDoctor != personId:
                        self.addTotal(table, 4*' '+u'По врачу', totalByDoctor, locale, dataLineStep, detailClient, detailDate)
                        totalByDoctor = [0]*reportRowSize
                    if prevSpecialityName != specialityName:
                        if prevSpecialityName is not None:
                            self.addTotal(table, u'всего по специальности', totalBySpeciality, locale, dataLineStep, detailClient, detailDate)
                            totalBySpeciality = [0]*reportRowSize
                        self.addSpecialityHeader(table, specialityName)
                        prevSpecialityName = specialityName
                    if not detailService or prevDoctor != personId:
                        i = table.addRow()
                        nameShift = 4*' ' if detailService else ''
                        if doctorNamePrev != personId:
                            table.setText(i, 0, nameShift+doctorName, font)
                            table.setText(i, 1, doctorCode, font)
                            if not detailService:
                                doctorNamePrev = personId
                                doctorNameRow = i
                        if not detailService:
                            table.setText(i, 3 if detailDate else 2, serviceTypeName, font)
                            table.setText(i, 4 if detailDate else 3, matCodeName, font)
                        if detailService:
                            table.mergeCells(i, 2, 1, self.tableColumnsLen-2)
                            prevDoctor = personId
                        if not detailClient:
                            table.mergeCells(doctorNameRow, 0,  i-doctorNameRow+1, 1)
                            table.mergeCells(doctorNameRow, 1,  i-doctorNameRow+1, 1)
                    if detailClient:
                        if clientInfoPrev != clientInfo:
                            i = table.addRow()
                            table.setText(i, 2 if detailDate else 1 if detailDate else 0, clientInfo)
                            clientInfoPrev = clientInfo
                            clientNameRow = i
                            table.mergeCells(clientNameRow, 0,  1, self.tableColumnsLen)
                    if detailService:
                        i = table.addRow()
                        if detailDate:
                            serviceDate, serviceId, serviceCode, serviceName = key[3]
                            table.setText(i, 2, forceString(serviceDate))
                        else:
                            serviceId, serviceCode, serviceName = key[3]
                        table.setText(i, 0, 8*' '+serviceName)
                        table.setText(i, 1, serviceCode)
                        table.setText(i, 3 if detailDate else 2, serviceTypeName)
                        table.setText(i, 4 if detailDate else 3, matCodeName)
                    if detailClient and not detailService:
                        i = table.addRow()
                        table.mergeCells(clientNameRow+1, 0, i-clientNameRow, 4)
                    reportLine = reportData[key]
                    for j in xrange(0, reportRowSize, dataLineStep):
                        table.setText(i, j+(5 if detailDate else 4), locale.toString(float(reportLine[j]), 'f', 2))
                    for j in xrange(1, reportRowSize, dataLineStep):
                        table.setText(i, j+(5 if detailDate else 4), locale.toString(float(reportLine[j]), 'f', 2))
                    for j in xrange(2, reportRowSize, dataLineStep):
                        table.setText(i, j+(5 if detailDate else 4), locale.toString(float(reportLine[j]), 'f', 2))
                    if detailFedTariff:
                        for j in xrange(3, reportRowSize, dataLineStep):
                            table.setText(i, j+(5 if detailDate else 4), locale.toString(float(reportLine[j]), 'f', 2))
                    for j in xrange(reportRowSize):
                        totalBySpeciality[j] += reportLine[j]
                        totalByReport[j] += reportLine[j]
                        totalByDoctor[j] += reportLine[j]
                if detailService:
                    self.addTotal(table, 4*' '+u'По врачу', totalByDoctor, locale, dataLineStep, detailClient, detailDate)
                self.addTotal(table, u'всего по специальности', totalBySpeciality, locale, dataLineStep, detailClient, detailDate)
                self.addTotal(table, u'Всего', totalByReport, locale, dataLineStep, detailClient, detailDate)
        return doc


    def addSpecialityHeader(self, table, specialityName):
        i = table.addRow()
        table.mergeCells(i, 0, 1, self.tableColumnsLen)
        table.setText(i, 0, specialityName, CReportBase.TableHeader)


    def addTotal(self, table, title, reportLine, locale, dataLineStep, detailClient, detailDate):
        reportRowSize = len(reportLine)
        i = table.addRow()
        table.mergeCells(i, 0, 1, 2)
        table.setText(i, 1 if detailDate else 0, title, CReportBase.TableTotal)
        for j in xrange(0, reportRowSize, dataLineStep):
            table.setText(i, j+(5 if detailDate else 4), locale.toString(float(reportLine[j]), 'f', 2))
        for j in xrange(1, reportRowSize, dataLineStep):
            table.setText(i, j+(5 if detailDate else 4), locale.toString(float(reportLine[j]), 'f', 2))
        for j in xrange(2, reportRowSize, dataLineStep):
            table.setText(i, j+(5 if detailDate else 4), locale.toString(float(reportLine[j]), 'f', 2))
        if dataLineStep == 4:
            for j in xrange(3, reportRowSize, dataLineStep):
                table.setText(i, j+(5 if detailDate else 4), locale.toString(float(reportLine[j]), 'f', 2))


class CFinanceSummaryByDoctorsEx(CFinanceSummaryByDoctors):
    def __init__(self, parent):
        CFinanceSummaryByDoctors.__init__(self, parent)
        self.setTitle(u'Финансовая сводка по врачам за период')


    def exec_(self, accountIdList=None):
        self.accountIdList = accountIdList
        CFinanceSummaryByDoctors.exec_(self)


    def getSetupDialog(self, parent):
        result = CFinanceSummarySetupDialog(parent)
        result.setVisibleDetailService(True)
        result.setVisibleClientOrganisation(True)
        result.setVisibleDetailFedTariff(True)
        result.setVisiblePeriodOnService(True)
        result.setVisibleDetailFinance(True)
        result.setVisibleDetailPayer(True)
        result.setVisibleDetailDate(True)
        result.setSharingOfEventsByVisits(True)
        result.setVisibleAgeFilter(True)
        result.setVisibleSexFilter(True)
        result.setEventTypeVisible(False)
        result.setEventTypeListVisible(True)
        result.setTitle(self.title())
        return result


    def getDescription(self, params):
        db = QtGui.qApp.db
        begDate              = params.get('begDate', None)
        endDate              = params.get('endDate', None)
        eventTypeId          = params.get('eventTypeId', None)
        orgStructureId       = params.get('orgStructureId', None)
        personId             = params.get('personId', None)
        specialityId         = params.get('specialityId', None)
        clientOrganisationId = params.get('clientOrganisationId', None)
        freeInputWork        = params.get('freeInputWork', False)
        freeInputWorkValue   = params.get('freeInputWorkValue', '')
        orgInsurerId         = params.get('orgInsurerId', None)
        confirmation         = params.get('confirmation', False)
        confirmationBegDate  = params.get('confirmationBegDate', None)
        confirmationEndDate  = params.get('confirmationEndDate', None)
        confirmationType     = params.get('confirmationType', 0)
        refuseType           = params.get('refuseType', None)
        filterBegAge         = params.get('filterBegAge', 0)
        filterEndAge         = params.get('filterEndAge', 150)
        isPeriodOnService    = params.get('isPeriodOnService', False)
        filterSex            = params.get('filterSex', 0)
        rows = []
        if isPeriodOnService:
            rows.append(u'учитывать период по услуге')
        if begDate:
            rows.append(u'Дата начала периода: %s'%forceString(begDate))
        if endDate:
            rows.append(u'Дата окончания периода: %s'%forceString(endDate))
        if eventTypeId:
            rows.append(u'Тип события: %s'%forceString(db.translate('EventType', 'id', eventTypeId, 'CONCAT_WS(\' | \', code,name)')))
        eventTypeList = params.get('eventTypeList', None)
        if eventTypeList:
            tableET = db.table('EventType')
            records = db.getRecordList(tableET, [tableET['name']], [tableET['deleted'].eq(0), tableET['id'].inlist(eventTypeList)])
            nameList = []
            for record in records:
                nameList.append(forceString(record.value('name')))
            rows.append(u'Тип события:  %s'%(u','.join(name for name in nameList if name)))
        if orgStructureId:
            rows.append(u'Подразделение исполнителя: %s'%forceString(db.translate('OrgStructure', 'id', orgStructureId, 'CONCAT_WS(\' | \', code,name)')))
        if personId:
            rows.append(u'Врач: %s'%forceString(db.translate('vrbPerson', 'id', personId, 'name')))
        if specialityId:
            rows.append(u'Специальность: %s'%forceString(db.translate('rbSpeciality', 'id', specialityId, 'CONCAT_WS(\' | \', code,name)')))
        if orgInsurerId:
            rows.append(u'Плательщик по договору: %s'%forceString(db.translate('Organisation', 'id', orgInsurerId, 'shortName')))
        if clientOrganisationId:
            rows.append(u'Место работы: %s'%forceString(db.translate('Organisation', 'id', clientOrganisationId, 'shortName')))
        if freeInputWork:
            rows.append(u'Место работы по названию: %s'%forceString(freeInputWorkValue))
        if confirmation:
            rows.append(u'Тип подтверждения: %s'%{0: u'без подтверждения',
                                                  1: u'оплаченные',
                                                  2: u'отказанные'}.get(confirmationType, u''))
            if confirmationBegDate:
                rows.append(u'Начало периода подтверждения: %s'%forceString(confirmationBegDate))
            if confirmationEndDate:
                rows.append(u'Окончание периода подтверждения: %s'%forceString(confirmationEndDate))
            if refuseType:
                rows.append(u'Причина отказа: %s'%forceString(db.translate('rbPayRefuseType', 'id', refuseType, 'CONCAT_WS(\' | \', code,name)')))
        if filterEndAge and filterBegAge <= filterEndAge:
            rows.append(u'Возраст с %s по %s'%(forceString(filterBegAge), forceString(filterEndAge)))
        if filterSex:
            rows.append(u'Пол %s'%([u'не задано', u'мужской', u'женский'][filterSex]))
        rows.append(u'Отчёт составлен: '+forceString(QDateTime.currentDateTime()))
        return rows


    def build(self, params):
        params['accountIdList'] = self.accountIdList
        return CFinanceSummaryByDoctors.build(self, '\n'.join(self.getDescription(params)), params)


class CFinanceSummarySetupDialog(QtGui.QDialog, Ui_FinanceSummarySetupDialog):
    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)
        self.cmbEventType.setTable('EventType', True, filter=getWorkEventTypeFilter())
        self.cmbSpeciality.setTable('rbSpeciality', True)
        self.cmbOrgStructure.setOrgId(QtGui.qApp.currentOrgId())
        self.cmbOrgStructure.setValue(QtGui.qApp.currentOrgStructureId())
        self.cmbInsurerDoctors.setAddNone(True)
        self.cmbRefuseType.setTable('rbPayRefuseType', True)
        self.cmbConfirmationType.addItem(u'без подтверждения')
        self.cmbConfirmationType.addItem(u'оплаченные')
        self.cmbConfirmationType.addItem(u'отказанные')
        self._visibleDetailService = False
        self.setVisibleDetailService(self._visibleDetailService)
        self._visibleClientOrganisation = False
        self.setVisibleClientOrganisation(self._visibleClientOrganisation)
        self._visibleDetailFedTariff = False
        self.setVisibleDetailFedTariff(self._visibleDetailFedTariff)
        self._visibleAgeFilter = False
        self.setVisibleAgeFilter(self._visibleAgeFilter)
        self._visibleSexFilter = False
        self.setVisibleSexFilter(self._visibleSexFilter)
        self.setVisiblePeriodOnService(False)
        self.cmbSocStatusType.setTable('rbSocStatusClass', True, filter='group_id IS NULL')
        self.setVisibleSocStatusParams(False)
        self.setVisibleDetailFinance(False)
        self.setVisibleDetailDate(False)
        self.setVisibleDetailPayer(False)
        self.setSharingOfEventsByVisits(False)
        self.setVisibleDetailVAT(False)
        self.setEventTypeVisible(True)
        self.setEventTypeListVisible(False)
        self.eventTypeList = []


    def setEventTypeVisible(self, value=True):
        self.eventTypeVisible = value
        self.cmbEventType.setVisible(value)
        self.lblEventType.setVisible(value)


    def setEventTypeListVisible(self, value=True):
        self.eventTypeListVisible = value
        self.btnEventTypeList.setVisible(value)
        self.lblEventTypeList.setVisible(value)


    def setSharingOfEventsByVisits(self, value):
        self._visibleEventsByVisits = value
        self.chkSharingOfEventsByVisits.setVisible(value)
        if not value:
            self.chkSharingOfEventsByVisits.setChecked(False)


    def setVisibleDetailVAT(self, value):
        self._visibleDetailVAT = value
        self.chkDetailVAT.setVisible(value)
        if not value:
            self.chkDetailVAT.setChecked(False)


    def setVisibleDetailDate(self, value):
        self._visibleDetailDate = value
        self.chkDetailDate.setVisible(value)
        if not value:
            self.chkDetailDate.setChecked(False)


    def setVisibleDetailFinance(self, value):
        self._visibleDetailFinance = value
        self.chkDetailFinance.setVisible(value)
        if not value:
            self.chkDetailFinance.setChecked(False)


    def setVisibleDetailPayer(self, value):
        self._visibleDetailPayer = value
        self.chkDetailPayer.setVisible(value)
        if not value:
            self.chkDetailPayer.setChecked(False)


    def setVisibleDetailFedTariff(self, value):
        self._visibleDetailFedTariff = value
        self.chkDetailFedTariff.setVisible(value)


    def setVisibleDetailService(self, value):
        self._visibleDetailService = value
        self.chkDetailService.setVisible(value)


    def setVisibleClientOrganisation(self, value):
        self._visibleClientOrganisation = value
        self.cmbClientOrganisation.setVisible(value)
        self.lblClientOrganisation.setVisible(value)


    def setVisibleSexFilter(self, value):
        self._visibleSexFilter = value
        self.lblFilterSexFrom.setVisible(value)
        self.cmbFilterSex.setVisible(value)


    def setVisibleAgeFilter(self, value):
        self._visibleAgeFilter = value
        self.lblFilterAgeFrom.setVisible(value)
        self.edtFilterBegAge.setVisible(value)
        self.lblFilterAgeTo.setVisible(value)
        self.edtFilterEndAge.setVisible(value)


    def setVisiblePeriodOnService(self, value):
        self._visiblePeriodOnService = value
        self.chkPeriodOnService.setVisible(value)
        self.chkPeriodOnService.setChecked(False)



    def setVisibleSocStatusParams(self, value):
        self._visibleSocStatus = value
        self.chkDetailedSocStatus.setVisible(value)
        self.chkDetailedSocStatus.setChecked(False)
        self.cmbSocStatusType.setVisible(value)
        self.chkGroupByMonths.setVisible(value)
        self.chkGroupByMonths.setChecked(False)


    def setTitle(self, title):
        self.setWindowTitle(title)


    def setParams(self, params):
        date = QDate.currentDate().addDays(-3)
        self.edtBegDate.setDate(params.get('begDate', firstMonthDay(date)))
        self.edtEndDate.setDate(params.get('endDate', lastMonthDay(date)))
        if self.eventTypeVisible:
            self.cmbEventType.setValue(params.get('eventTypeId', None))
        self.cmbOrgStructure.setValue(params.get('orgStructureId', None))
        self.cmbSpeciality.setValue(params.get('specialityId', None))
        self.cmbPerson.setValue(params.get('personId', None))
        self.cmbInsurerDoctors.setValue(params.get('orgInsurerId', None))
        self.chkConfirmation.setChecked(params.get('confirmation', False))
        self.edtConfirmationBegDate.setDate(params.get('confirmationBegDate', firstMonthDay(date)))
        self.edtConfirmationEndDate.setDate(params.get('confirmationEndDate', lastMonthDay(date)))
        self.cmbConfirmationType.setCurrentIndex(params.get('confirmationType', 0))
        self.cmbRefuseType.setValue(params.get('refuseType', None))
        self.chkDetailService.setChecked(params.get('detailService', False))
        self.chkDetailFinance.setChecked(params.get('detailFinance', False))
        self.chkDetailPayer.setChecked(params.get('detailPayer', False))
        self.chkDetailDate.setChecked(params.get('detailDate', False))
        self.cmbClientOrganisation.setValue(params.get('clientOrganisationId', None))
        self.chkFreeInputWork.setChecked(params.get('freeInputWork', False))
        self.edtFreeInputWork.setText(params.get('freeInputWorkValue', ''))
        self.chkDetailFedTariff.setChecked(params.get('detailFedTariff', False))
        self.chkDetailClient.setChecked(params.get('detailClient', False))
        if self._visibleSexFilter:
            self.cmbFilterSex.setCurrentIndex(params.get('filterSex', 0))
        if self._visibleAgeFilter:
            self.edtFilterBegAge.setValue(params.get('filterBegAge', 0))
            self.edtFilterEndAge.setValue(params.get('filterEndAge', 150))
        if self._visiblePeriodOnService:
            self.chkPeriodOnService.setChecked(params.get('isPeriodOnService', False))
        if self._visibleEventsByVisits:
            self.chkSharingOfEventsByVisits.setChecked(params.get('isSharingOfEventsByVisits', False))
        if self._visibleDetailVAT:
            self.chkDetailVAT.setChecked(params.get('datailVAT', False))
        if self._visibleSocStatus:
            self.chkDetailedSocStatus.setChecked(params.get('detailedSocStatus', False))
            self.cmbSocStatusType.setValue(params.get('socStatusType', None))
        self.chkGroupByMonths.setChecked(params.get('groupByMonths', False))
        if self.eventTypeListVisible:
            self.eventTypeList =  params.get('eventTypeList', [])
            if self.eventTypeList:
                db = QtGui.qApp.db
                tableET = db.table('EventType')
                records = db.getRecordList(tableET, [tableET['name']], [tableET['deleted'].eq(0), tableET['id'].inlist(self.eventTypeList)])
                nameList = []
                for record in records:
                    nameList.append(forceString(record.value('name')))
                self.lblEventTypeList.setText(u','.join(name for name in nameList if name))
            else:
                self.lblEventTypeList.setText(u'не задано')


    def params(self):
        result = {}
        result['begDate'] = self.edtBegDate.date()
        result['endDate'] = self.edtEndDate.date()
        if self.eventTypeVisible:
            result['eventTypeId'] = self.cmbEventType.value()
        result['orgStructureId'] = self.cmbOrgStructure.value()
        result['specialityId'] = self.cmbSpeciality.value()
        result['personId'] = self.cmbPerson.value()
        result['orgInsurerId'] = self.cmbInsurerDoctors.value()
        result['confirmation'] = self.chkConfirmation.isChecked()
        result['confirmationType'] = self.cmbConfirmationType.currentIndex()
        result['confirmationBegDate'] = self.edtConfirmationBegDate.date()
        result['confirmationEndDate'] = self.edtConfirmationEndDate.date()
        result['refuseType'] = self.cmbRefuseType.value()
        result['freeInputWork'] = self.chkFreeInputWork.isChecked()
        result['detailClient'] = self.chkDetailClient.isChecked()
        result['freeInputWorkValue'] = forceStringEx(self.edtFreeInputWork.text())
        if self._visiblePeriodOnService:
            result['isPeriodOnService'] = self.chkPeriodOnService.isChecked()

        if self._visibleDetailService:
            result['detailService'] = self.chkDetailService.isChecked()

        if self._visibleClientOrganisation:
            result['clientOrganisationId'] = self.cmbClientOrganisation.value()

        if self._visibleDetailFedTariff:
            result['detailFedTariff'] = self.chkDetailFedTariff.isChecked()

        result['detailClientSex'] = False
        if self._visibleSexFilter:
            result['detailClientSex'] = forceBool(self.cmbFilterSex.currentIndex())
            result['filterSex'] = self.cmbFilterSex.currentIndex()

        result['detailClientAge'] = False
        if self._visibleAgeFilter:
            result['filterBegAge'] = self.edtFilterBegAge.value()
            result['filterEndAge'] = self.edtFilterEndAge.value()
            if self.edtFilterEndAge.value():
                result['detailClientAge'] = True

        if self._visibleDetailFinance:
            result['detailFinance'] = self.chkDetailFinance.isChecked()

        if self._visibleDetailPayer:
            result['detailPayer'] = self.chkDetailPayer.isChecked()

        if self._visibleDetailDate:
            result['detailDate'] = self.chkDetailDate.isChecked()

        if self._visibleEventsByVisits:
            result['isSharingOfEventsByVisits'] = self.chkSharingOfEventsByVisits.isChecked()

        if self._visibleDetailVAT:
            result['datailVAT'] = self.chkDetailVAT.isChecked()

        if self._visibleSocStatus:
            result['detailedSocStatus'] = self.chkDetailedSocStatus.isChecked()
            result['socStatusType'] = self.cmbSocStatusType.value()

        result['groupByMonths'] = self.chkGroupByMonths.isChecked()

        if self.eventTypeListVisible:
            result['eventTypeList'] = self.eventTypeList

        return result


    def onStateChanged(self, state):
        self.lblConfirmationType.setEnabled(state)
        self.lblBegDateConfirmation.setEnabled(state)
        self.lblEndDateConfirmation.setEnabled(state)
        self.lblRefuseType.setEnabled(state)
        self.cmbConfirmationType.setEnabled(state)
        self.edtConfirmationBegDate.setEnabled(state)
        self.edtConfirmationEndDate.setEnabled(state)
        self.cmbRefuseType.setEnabled(state)


    @pyqtSignature('QDate')
    def on_edtBegDate_dateChanged(self, date):
        self.cmbPerson.setBegDate(date)


    @pyqtSignature('QDate')
    def on_edtEndDate_dateChanged(self, date):
        self.cmbPerson.setEndDate(date)


    @pyqtSignature('int')
    def on_cmbOrgStructure_currentIndexChanged(self, index):
        orgStructureId = self.cmbOrgStructure.value()
        self.cmbPerson.setOrgStructureId(orgStructureId)


    @pyqtSignature('int')
    def on_cmbSpeciality_currentIndexChanged(self, index):
        specialityId = self.cmbSpeciality.value()
        self.cmbPerson.setSpecialityId(specialityId)


    @pyqtSignature('bool')
    def on_chkDetailService_toggled(self, checked):
        self.chkDetailDate.setEnabled(checked and self.chkDetailClient.isChecked())


    @pyqtSignature('bool')
    def on_chkDetailClient_toggled(self, checked):
        self.chkDetailDate.setEnabled(checked and self.chkDetailService.isChecked())


    @pyqtSignature('')
    def on_btnEventTypeList_clicked(self):
        self.eventTypeList = []
        self.lblEventTypeList.setText(u'не задано')
        filter = getWorkEventTypeFilter()
        dialog = CEventTypeListEditorDialog(self, filter)
        if dialog.exec_():
            self.eventTypeList = dialog.values()
            if self.eventTypeList:
                db = QtGui.qApp.db
                tableET = db.table('EventType')
                records = db.getRecordList(tableET, [tableET['name']], [tableET['deleted'].eq(0), tableET['id'].inlist(self.eventTypeList)])
                nameList = []
                for record in records:
                    nameList.append(forceString(record.value('name')))
                self.lblEventTypeList.setText(u','.join(name for name in nameList if name))

