# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui
from PyQt4.QtCore import QDateTime, QLocale, QDate

from library.Utils      import forceBool, forceDouble, forceRef, forceString, forceDate
from Reports.Report     import CReport
from Reports.ReportBase import CReportBase, createTable
from Reports.FinanceSummaryByDoctors import getCond, CFinanceSummarySetupDialog


def selectDataByEvents(params):
    db = QtGui.qApp.db
    isPeriodOnService = params.get('isPeriodOnService', False)
    cond = getCond(params)
    if isPeriodOnService:
        condDate = []
        begDate = params.get('begDate', None)
        endDate = params.get('endDate', None)
        tableEvent = db.table('Event')
        if begDate:
            condDate.append(tableEvent['execDate'].ge(begDate))
        if endDate:
            condDate.append(tableEvent['execDate'].lt(endDate.addDays(1)))
        cond += u' AND ' + db.joinAnd(condDate) if condDate else ''
    detailClient = params.get('detailClient', False)
    datailVAT    = params.get('datailVAT', False)
    socStatusType = params.get('socStatusType', None)
    groupBySocStatus = params.get('groupBySocStatus', False)
    groupByMonths = params.get('groupByMonths', False)
    stmt="""
SELECT
    rbService.code,
    rbService.name,
    %s
    %s
    %s
    (Account_Item.amount) AS amount,
    (Account_Item.sum) AS sum,
    (Account_Item.uet) AS uet,
    (Account_Item.date IS NOT NULL AND Account_Item.number != '' AND Account_Item.refuseType_id IS NULL) AS exposed,
    (Account_Item.date IS NOT NULL AND Account_Item.number != '' AND Account_Item.refuseType_id IS NOT NULL) as refused,
    Account_Item.price,
    %s
    rbFinance.id AS financeId,
    rbFinance.code AS financeCode,
    rbFinance.name AS financeName,
    Contract.payer_id,
    Organisation.shortName

FROM Account_Item
LEFT JOIN Account         ON Account.id = Account_Item.master_id
LEFT JOIN Contract        ON Contract.id = Account.contract_id
LEFT JOIN Organisation    ON Organisation.id = Contract.payer_id
LEFT JOIN rbFinance       ON (rbFinance.id = Contract.finance_id AND Contract.deleted = 0)
LEFT JOIN Event           ON Event.id = Account_Item.event_id
LEFT JOIN EventType       ON EventType.id = Event.eventType_id
LEFT JOIN rbService       ON rbService.id = IF(Account_Item.service_id IS NULL, EventType.service_id, Account_Item.service_id)
LEFT JOIN Diagnostic      ON Diagnostic.event_id = Event.id
LEFT JOIN rbDiagnosisType ON rbDiagnosisType.id = Diagnostic.diagnosisType_id
LEFT JOIN Person          ON Person.id = Diagnostic.person_id
LEFT JOIN rbSpeciality    ON rbSpeciality.id = Person.speciality_id
INNER JOIN Client         ON Client.id = Event.client_id
%s
WHERE
    Account_Item.visit_id IS NULL
    AND Account_Item.action_id IS NULL
    AND Diagnostic.deleted = 0
    AND rbDiagnosisType.code in ('1','2')
    AND %s
    group by Account_Item.id
    """
    return db.query(stmt % ('''Event.client_id,
    Client.lastName AS clientLastName,
    Client.firstName AS clientFirstName,
    Client.patrName AS clientPatrName,
    Client.id AS clientId,''' if detailClient else '',
        '''group_concat(rbSocStatusType.name SEPARATOR ',') as socName,''' if socStatusType else '',
        '''DATE(Event.execDate) as execDate,''' if groupByMonths else '',
        '''Account_Item.VAT,''' if datailVAT else '',
        '''INNER JOIN ClientSocStatus ON ClientSocStatus.client_id=Client.id and ClientSocStatus.socStatusClass_id = %s
        LEFT JOIN rbSocStatusType on ClientSocStatus.socStatusType_id = rbSocStatusType.id''' %(socStatusType)  if socStatusType else '',
        cond))


def selectDataByVisits(params):
    db = QtGui.qApp.db
    isPeriodOnService = params.get('isPeriodOnService', False)
    cond = getCond(params)
    if isPeriodOnService:
        condDate = []
        begDate = params.get('begDate', None)
        endDate = params.get('endDate', None)
        tableVisit = db.table('Visit')
        if begDate:
            condDate.append(tableVisit['date'].ge(begDate))
        if endDate:
            condDate.append(tableVisit['date'].lt(endDate.addDays(1)))
        cond += u' AND ' + db.joinAnd(condDate) if condDate else ''
    detailClient = params.get('detailClient', False)
    datailVAT    = params.get('datailVAT', False)
    socStatusType = params.get('socStatusType', '')
    groupBySocStatus = params.get('groupBySocStatus', False)
    groupByMonths = params.get('groupByMonths', False)
    stmt="""
SELECT
    rbService.code,
    rbService.name,
    %s
    %s
    %s
    (Account_Item.amount) AS amount,
    (Account_Item.sum) AS sum,
    (Account_Item.uet) AS uet,
    (Account_Item.date IS NOT NULL AND Account_Item.number != '' AND Account_Item.refuseType_id IS NULL) AS exposed,
    (Account_Item.date IS NOT NULL AND Account_Item.number != '' AND Account_Item.refuseType_id IS NOT NULL) as refused,
    Account_Item.price,
    %s
    rbFinance.id AS financeId,
    rbFinance.code AS financeCode,
    rbFinance.name AS financeName,
    Contract.payer_id,
    Organisation.shortName

FROM Account_Item
LEFT JOIN Account      ON Account.id = Account_Item.master_id
LEFT JOIN Contract     ON Contract.id = Account.contract_id
LEFT JOIN Organisation ON Organisation.id = Contract.payer_id
LEFT JOIN rbFinance    ON (rbFinance.id = Contract.finance_id AND Contract.deleted = 0)
LEFT JOIN Event        ON Event.id = Account_Item.event_id
LEFT JOIN Visit        ON Visit.id = Account_Item.visit_id
LEFT JOIN rbService    ON rbService.id = IF(Account_Item.service_id IS NULL, Visit.service_id, Account_Item.service_id)
LEFT JOIN Person       ON Person.id = Visit.person_id
LEFT JOIN rbSpeciality ON rbSpeciality.id = Person.speciality_id
INNER JOIN Client      ON Client.id = Event.client_id
%s
WHERE
    Account_Item.visit_id IS NOT NULL
    AND Account_Item.action_id IS NULL
    AND %s
    group by Account_Item.id
    """
    return db.query(stmt % ('''Event.client_id,
    Client.lastName AS clientLastName,
    Client.firstName AS clientFirstName,
    Client.patrName AS clientPatrName,
    Client.id AS clientId,''' if detailClient else '',
        '''group_concat(rbSocStatusType.name SEPARATOR ',') as socName,''' if socStatusType else '',
        '''DATE(Event.execDate) as execDate,''' if groupByMonths else '',
        '''Account_Item.VAT,''' if datailVAT else '',
        '''INNER JOIN ClientSocStatus ON ClientSocStatus.client_id=Client.id and ClientSocStatus.socStatusClass_id = %s
        LEFT JOIN rbSocStatusType on ClientSocStatus.socStatusType_id = rbSocStatusType.id''' %(socStatusType)  if socStatusType else '',
    cond))


def selectDataByActions(params):
    db = QtGui.qApp.db
    isPeriodOnService = params.get('isPeriodOnService', False)
    cond = getCond(params)
    if isPeriodOnService:
        condDate = []
        begDate = params.get('begDate', None)
        endDate = params.get('endDate', None)
        tableAction = db.table('Action')
        if begDate:
            condDate.append(tableAction['endDate'].ge(begDate))
        if endDate:
            condDate.append(tableAction['endDate'].lt(endDate.addDays(1)))
        cond += u' AND ' + db.joinAnd(condDate) if condDate else ''
    detailClient = params.get('detailClient', False)
    datailVAT    = params.get('datailVAT', False)
    socStatusType = params.get('socStatusType', [])
    groupBySocStatus = params.get('groupBySocStatus', False)
    groupByMonths = params.get('groupByMonths', False)
    stmt="""
SELECT
    rbService.code,
    rbService.name,
    %s
    %s
    %s
    (Account_Item.amount) AS amount,
    (Account_Item.sum) AS sum,
    (Account_Item.uet) AS uet,
    (Account_Item.date IS NOT NULL AND Account_Item.number != '' AND Account_Item.refuseType_id IS NULL) AS exposed,
    (Account_Item.date IS NOT NULL AND Account_Item.number != '' AND Account_Item.refuseType_id IS NOT NULL) as refused,
    Account_Item.price,
    %s
    rbFinance.id AS financeId,
    rbFinance.code AS financeCode,
    rbFinance.name AS financeName,
    Contract.payer_id,
    Organisation.shortName

FROM Account_Item
LEFT JOIN Account      ON Account.id = Account_Item.master_id
LEFT JOIN Contract     ON Contract.id = Account.contract_id
LEFT JOIN Organisation ON Organisation.id = Contract.payer_id
LEFT JOIN rbFinance    ON (rbFinance.id = Contract.finance_id AND Contract.deleted = 0)
LEFT JOIN Event        ON Event.id = Account_Item.event_id
LEFT JOIN Action       ON Action.id = Account_Item.action_id
LEFT JOIN ActionType   ON ActionType.id = Action.actionType_id
LEFT JOIN rbService    ON rbService.id = Account_Item.service_id
LEFT JOIN Person       ON Person.id = Action.person_id
LEFT JOIN rbSpeciality ON rbSpeciality.id = Person.speciality_id
INNER JOIN Client      ON Client.id = Event.client_id
%s
WHERE
    Account_Item.visit_id IS NULL
    AND Account_Item.action_id IS NOT NULL
    AND %s
    group by Account_Item.id
    """
    return db.query(stmt % ('''Event.client_id,
    Client.lastName AS clientLastName,
    Client.firstName AS clientFirstName,
    Client.patrName AS clientPatrName,
    Client.id AS clientId,''' if detailClient else '',
        '''group_concat(rbSocStatusType.name SEPARATOR ',') as socName,''' if socStatusType else '',
        '''DATE(Event.execDate) as execDate,''' if groupByMonths else '',
        '''Account_Item.VAT,''' if datailVAT else '',
        '''INNER JOIN ClientSocStatus ON ClientSocStatus.client_id=Client.id and ClientSocStatus.socStatusClass_id = %s
        LEFT JOIN rbSocStatusType on ClientSocStatus.socStatusType_id = rbSocStatusType.id''' %(socStatusType)  if socStatusType else '',
        cond))


class CFinanceSummaryByServices(CReport):
    def __init__(self, parent):
        CReport.__init__(self, parent)
        self.setTitle(u'Финансовая сводка по услугам')
        self.datailVAT = False


    def build(self, description, params):
        reportFinanceData = {}
        reportPayerData = {}
        reportDataAll = {}
        detailClient   = params.get('detailClient', False)
        detailFinance  = params.get('detailFinance', False)
        detalilPayer   = params.get('detailPayer', False)
        self.datailVAT = params.get('datailVAT', False)
        self.socStatusType = params.get('socStatusType', [])
        self.groupByMonths   = params.get('groupByMonths', False)
        reportRowSize = 12 if self.datailVAT else 9
        def processQuery(query, reportDataGlobal):
            while query.next():
                record = query.record()
                code        = forceString(record.value('code'))
                name        = forceString(record.value('name'))
                amount      = forceDouble(record.value('amount'))
                price       = forceDouble(record.value('price'))
                VAT         = forceDouble(record.value('VAT'))
                financeId   = forceRef(record.value('financeId'))
                financeCode = forceString(record.value('financeCode'))
                financeName = forceString(record.value('financeName'))
                uet         = forceDouble(record.value('uet'))
                if amount == int(amount):
                   amount = int(amount)
                sum         = forceDouble(record.value('sum'))
                exposed     = forceBool(record.value('exposed'))
                refused     = forceBool(record.value('refused'))
                if detailClient:
                    clientId = forceRef(record.value('clientId'))
                    clientInfo = u' '.join(clName for clName in [forceString(record.value('clientLastName')),
                                                                 forceString(record.value('clientFirstName')),
                                                                 forceString(record.value('clientPatrName')),
                                                                 forceString(record.value('clientId'))])
                    key = (name if name else u'Без указания услуги',
                           code, price, clientInfo)
                else:
                    key = (name if name else u'Без указания услуги',
                           code, price)
                if self.groupByMonths:
                    month = forceString(forceDate(record.value('execDate')).toString('yyyy.MM'))
                    reportData = reportDataGlobal.setdefault(month, {})
                else:
                    reportData = reportDataGlobal
                if self.socStatusType:
                    socName = forceString(record.value('socName'))
                    socName = ', '.join(sorted(socName.split(',')))
                    reportSocData = reportData.setdefault(socName, {})
                    reportLine = reportSocData.setdefault(key, [0]*reportRowSize)
                elif detalilPayer:
                    payerId = forceRef(record.value('payer_id'))
                    payerName = forceString(record.value('shortName'))
                    if not detailFinance:
                        reportPayerData = reportData.setdefault((payerId, payerName), {})
                        reportLine = reportPayerData.setdefault(key, [0]*reportRowSize)
                    else:
                        reportPayerData = reportData.setdefault((payerId, payerName), {})
                        reportFinanceData = reportPayerData.setdefault((financeId, financeCode, financeName), {})
                        reportLine = reportFinanceData.setdefault(key, [0]*reportRowSize)
                elif detailFinance:
                    reportFinanceData = reportData.setdefault((financeId, financeCode, financeName), {})
                    reportLine = reportFinanceData.setdefault(key, [0]*reportRowSize)
                else:
                    reportLine = reportData.setdefault(key, [0]*reportRowSize)
                if self.datailVAT:
                    reportLine[0] += amount
                    reportLine[1] += uet
                    reportLine[2] += sum
                    reportLine[3] += ((price*VAT)/(100.0+VAT))*amount
                    if exposed:
                        reportLine[4] += amount
                        reportLine[5] += uet
                        reportLine[6] += sum
                        reportLine[7] += ((price*VAT)/(100.0+VAT))*amount
                    if refused:
                        reportLine[8] += amount
                        reportLine[9] += uet
                        reportLine[10] += sum
                        reportLine[11] += ((price*VAT)/(100.0+VAT))*amount
                else:
                    reportLine[0] += amount
                    reportLine[1] += uet
                    reportLine[2] += sum
                    if exposed:
                        reportLine[3] += amount
                        reportLine[4] += uet
                        reportLine[5] += sum
                    if refused:
                        reportLine[6] += amount
                        reportLine[7] += uet
                        reportLine[8] += sum
                if self.socStatusType:
                    reportSocData[key] = reportLine
                    reportData[socName] = reportSocData
                elif detalilPayer:
                    if not detailFinance:
                        reportPayerData[key] = reportLine
                        reportData[(payerId, payerName)] = reportPayerData
                    else:
                        reportFinanceData[key] = reportLine
                        reportPayerData[(financeId, financeCode, financeName)] = reportFinanceData
                        reportData[(payerId, payerName)] = reportPayerData
                elif detailFinance:
                    reportFinanceData[key] = reportLine
                    reportData[(financeId, financeCode, financeName)] = reportFinanceData
                else:
                    reportData[key] = reportLine
            return reportDataGlobal

        query = selectDataByEvents(params)
        reportDataAll = processQuery(query, reportDataAll)
        query = selectDataByVisits(params)
        reportDataAll = processQuery(query, reportDataAll)
        query = selectDataByActions(params)
        reportDataAll = processQuery(query, reportDataAll)

        # now text
        doc = QtGui.QTextDocument()
        cursor = QtGui.QTextCursor(doc)

        cursor.setCharFormat(CReportBase.ReportTitle)
        cursor.insertText(self.title())
        cursor.insertBlock()
        cursor.setCharFormat(CReportBase.ReportBody)
        cursor.insertText(description)
        cursor.insertBlock()
        if self.datailVAT:
            procentCode = '10%'
            procentVal  = '5%'
        else:
            procentCode = '5%'
            procentVal  = '7%'
        tableColumns = [
                          ('25%',      [ u'Услуга',   u'наименование'], CReportBase.AlignLeft ),
                          (procentCode,[ u'',         u'код'         ], CReportBase.AlignLeft ),
                          (procentVal, [ u'Тариф',    u''            ], CReportBase.AlignRight ),
                          (procentVal, [ u'Всего',    u'кол-во'      ], CReportBase.AlignRight ),
                          (procentVal, [ u'',         u'УЕТ'         ], CReportBase.AlignRight ),
                          (procentVal, [ u'',         u'руб'         ], CReportBase.AlignRight ),
                          (procentVal, [ u'Оплачено', u'кол-во'      ], CReportBase.AlignRight ),
                          (procentVal, [ u'',         u'УЕТ'         ], CReportBase.AlignRight ),
                          (procentVal, [ u'',         u'руб'         ], CReportBase.AlignRight ),
                          (procentVal, [ u'Отказано', u'кол-во'      ], CReportBase.AlignRight ),
                          (procentVal, [ u'',         u'УЕТ'         ], CReportBase.AlignRight ),
                          (procentVal, [ u'',         u'руб'         ], CReportBase.AlignRight ),
                       ]
        if self.datailVAT:
            tableColumns.insert(6, (procentVal,  [u'', u'НДС'], CReportBase.AlignRight))
            tableColumns.insert(10, (procentVal, [u'', u'НДС'], CReportBase.AlignRight))
            tableColumns.insert(14, (procentVal, [u'', u'НДС'], CReportBase.AlignRight))
        elif self.socStatusType:
            tableColumns.insert(0, (procentVal,  [u'', u'Соц. статус'], CReportBase.AlignRight))
        self.tableColumnsLen = len(tableColumns)
        table = createTable(cursor, tableColumns)
        if self.datailVAT:
            table.mergeCells(0, 0, 1, 2)
            table.mergeCells(0, 2, 2, 1)
            table.mergeCells(0, 3, 1, 4)
            table.mergeCells(0, 7, 1, 4)
            table.mergeCells(0, 11, 1, 4)
        elif self.socStatusType:
            table.mergeCells(0, 1, 1, 2)
            table.mergeCells(0, 3, 2, 1)
            table.mergeCells(0, 4, 1, 3)
            table.mergeCells(0, 7, 1, 3)
            table.mergeCells(0, 10, 1, 3)
        else:
            table.mergeCells(0, 0, 1, 2)
            table.mergeCells(0, 2, 2, 1)
            table.mergeCells(0, 3, 1, 3)
            table.mergeCells(0, 6, 1, 3)
            table.mergeCells(0, 9, 1, 3)

        locale = QLocale()
        self.groupRows = 2
        if self.groupByMonths:
            self.groupRows += 1

        def printTableData(table, reportData):
            totalByReport = [0]*reportRowSize
            totalReportPrice = 0
            if self.socStatusType:
                for status in sorted(reportData.keys()):
                    keys = reportData[status].keys()
                    keys.sort()
                    namePrev = u''
                    codePrev = u''
                    for key in keys:
                        name = key[0]
                        code = key[1]
                        price = key[2]
                        if detailClient:
                            if namePrev != name and codePrev != code:
                                i = table.addRow()
                                table.setText(i, 1, name)
                                table.setText(i, 2, code)
                                namePrev = name
                                codePrev = code
                                table.mergeCells(i, 3, 1, 10)
                            clientInfo  = key[3]
                            i = table.addRow()
                            table.setText(i, 1, clientInfo)
                            table.setText(i, 3, locale.toString(float(price), 'f', 2))
                        else:
                            i = table.addRow()
                            table.setText(i, 1, name)
                            table.setText(i, 2, code)
                            table.setText(i, 3, locale.toString(float(price), 'f', 2))
                        reportLine = reportData[status][key]
                        totalReportPrice += price
                        for j in xrange(0, reportRowSize):
                            if isinstance(reportLine[j], float):
                                table.setText(i, j+4, locale.toString(float(reportLine[j]), 'f', 2))
                            else:
                                table.setText(i, j+4, reportLine[j])
                        for j in xrange(reportRowSize):
                            totalByReport[j] += reportLine[j]
                    table.mergeCells(self.groupRows, 0, i,  1)
                    table.setText(i, 0, status)
                    self.addTotal(table, u'Всего', totalByReport, locale, totalReportPrice, 3)
                    self.groupRows = i + (3 if self.groupByMonths else 2)
                    totalByReport = [0]*reportRowSize
                    totalReportPrice = 0
            elif detalilPayer:
                payerKeys = reportData.keys()
                payerKeys.sort()
                prevPayerId = None
                prevPayerRow = 2
                for payerId, payerName in payerKeys:
                    if detailFinance:
                        reportPayerData = reportData.get((payerId, payerName), {})
                        if not reportPayerData:
                            continue
                    else:
                        reportPayerData = reportData.get((payerId, payerName), {})
                        if not reportPayerData:
                            continue
                    if prevPayerId != payerId:
                        i = table.addRow()
                        table.setText(i, 0, payerName, CReportBase.TableTotal, blockFormat=CReportBase.AlignCenter)
                        prevPayerId = payerId
                        prevPayerRow = i
                        table.mergeCells(prevPayerRow, 0,  1, self.tableColumnsLen)
                    if detailFinance:
                        financeKeys = reportPayerData.keys()
                        financeKeys.sort()
                        prevFinanceId = None
                        prevFinanceRow = 2
                        for financeId, financeCode, financeName in financeKeys:
                            firstFinance = True
                            totalByFinance = [0]*reportRowSize
                            reportFinanceData = reportPayerData.get((financeId, financeCode, financeName), {})
                            if not reportFinanceData:
                                continue
                            if prevFinanceId != financeId:
                                i = table.addRow()
                                table.setText(i, 0, financeName, CReportBase.TableTotal, blockFormat=CReportBase.AlignCenter)
                                prevFinanceId = financeId
                                prevFinanceRow = i
                                table.mergeCells(prevFinanceRow, 0,  1, self.tableColumnsLen)
                            keys = reportFinanceData.keys()
                            keys.sort()
                            namePrev = u''
                            codePrev = u''
                            for key in keys:
                                name = key[0]
                                code = key[1]
                                price = key[2]
                                if detailClient:
                                    if namePrev != name and codePrev != code:
                                        i = table.addRow()
                                        table.setText(i, 0, name)
                                        table.setText(i, 1, code)
                                        namePrev = name
                                        codePrev = code
                                        table.mergeCells(i, 2, 1, 10)
                                    clientInfo  = key[3]
                                    i = table.addRow()
                                    table.setText(i, 0, clientInfo)
                                    table.setText(i, 2, locale.toString(float(price), 'f', 2))
                                else:
                                    i = table.addRow()
                                    table.setText(i, 0, name)
                                    table.setText(i, 1, code)
                                    table.setText(i, 2, locale.toString(float(price), 'f', 2))
                                reportLine = reportFinanceData[key]
                                totalReportPrice += price
                                for j in xrange(0, reportRowSize):
                                    if isinstance(reportLine[j], float):
                                        table.setText(i, j+3, locale.toString(float(reportLine[j]), 'f', 2))
                                    else:
                                        table.setText(i, j+3, reportLine[j])
                                for j in xrange(reportRowSize):
                                    totalByReport[j] += reportLine[j]
                                    totalByFinance[j] += reportLine[j]
                            self.addTotal(table, u'Всего по типу финансирования %s'%(financeName), totalByFinance, locale, totalReportPrice)
                        self.addTotal(table, u'Всего', totalByReport, locale, totalReportPrice)
                        totalByReport = [0]*reportRowSize
                        totalReportPrice = 0
                    else:
                        keys = reportPayerData.keys()
                        keys.sort()
                        namePrev = u''
                        codePrev = u''
                        for key in keys:
                            name = key[0]
                            code = key[1]
                            price = key[2]
                            if detailClient:
                                if namePrev != name and codePrev != code:
                                    i = table.addRow()
                                    table.setText(i, 0, name)
                                    table.setText(i, 1, code)
                                    namePrev = name
                                    codePrev = code
                                    table.mergeCells(i, 2, 1, 10)
                                clientInfo  = key[3]
                                i = table.addRow()
                                table.setText(i, 0, clientInfo)
                                table.setText(i, 2, locale.toString(float(price), 'f', 2))
                            else:
                                i = table.addRow()
                                table.setText(i, 0, name)
                                table.setText(i, 1, code)
                                table.setText(i, 2, locale.toString(float(price), 'f', 2))
                            reportLine = reportPayerData[key]
                            totalReportPrice += price
                            for j in xrange(0, reportRowSize):
                                if isinstance(reportLine[j], float):
                                    table.setText(i, j+3, locale.toString(float(reportLine[j]), 'f', 2))
                                else:
                                    table.setText(i, j+3, reportLine[j])
                            for j in xrange(reportRowSize):
                                totalByReport[j] += reportLine[j]
                        self.addTotal(table, u'Всего', totalByReport, locale, totalReportPrice)
                        totalByReport = [0]*reportRowSize
                        totalReportPrice = 0
            else:
                if detailFinance:
                    financeKeys = reportData.keys()
                    financeKeys.sort()
                    prevFinanceId = None
                    prevFinanceRow = 2
                    for financeId, financeCode, financeName in financeKeys:
                        firstFinance = True
                        totalByFinance = [0]*reportRowSize
                        reportFinanceData = reportData.get((financeId, financeCode, financeName), {})
                        if not reportFinanceData:
                            continue
                        if prevFinanceId != financeId:
                            i = table.addRow()
                            table.setText(i, 0, financeName, CReportBase.TableTotal, blockFormat=CReportBase.AlignCenter)
                            prevFinanceId = financeId
                            prevFinanceRow = i
                            table.mergeCells(prevFinanceRow, 0,  1, self.tableColumnsLen)
                        keys = reportFinanceData.keys()
                        keys.sort()
                        namePrev = u''
                        codePrev = u''
                        for key in keys:
                            name = key[0]
                            code = key[1]
                            price = key[2]
                            if detailClient:
                                if namePrev != name and codePrev != code:
                                    i = table.addRow()
                                    table.setText(i, 0, name)
                                    table.setText(i, 1, code)
                                    namePrev = name
                                    codePrev = code
                                    table.mergeCells(i, 2, 1, 10)
                                clientInfo  = key[3]
                                i = table.addRow()
                                table.setText(i, 0, clientInfo)
                                table.setText(i, 2, locale.toString(float(price), 'f', 2))
                            else:
                                i = table.addRow()
                                table.setText(i, 0, name)
                                table.setText(i, 1, code)
                                table.setText(i, 2, locale.toString(float(price), 'f', 2))
                            reportLine = reportFinanceData[key]
                            totalReportPrice += price
                            for j in xrange(0, reportRowSize):
                                if isinstance(reportLine[j], float):
                                    table.setText(i, j+3, locale.toString(float(reportLine[j]), 'f', 2))
                                else:
                                    table.setText(i, j+3, reportLine[j])
                            for j in xrange(reportRowSize):
                                totalByReport[j] += reportLine[j]
                                totalByFinance[j] += reportLine[j]
                        self.addTotal(table, u'Всего по типу финансирования %s'%(financeName), totalByFinance, locale, totalReportPrice)
                    self.addTotal(table, u'Всего', totalByReport, locale, totalReportPrice)
                    totalByReport = [0]*reportRowSize
                    totalReportPrice = 0
                else:
                    keys = reportData.keys()
                    keys.sort()
                    namePrev = u''
                    codePrev = u''
                    for key in keys:
                        name = key[0]
                        code = key[1]
                        price = key[2]
                        if detailClient:
                            if namePrev != name and codePrev != code:
                                i = table.addRow()
                                table.setText(i, 0, name)
                                table.setText(i, 1, code)
                                namePrev = name
                                codePrev = code
                                table.mergeCells(i, 2, 1, 10)
                            clientInfo  = key[3]
                            i = table.addRow()
                            table.setText(i, 0, clientInfo)
                            table.setText(i, 2, locale.toString(float(price), 'f', 2))
                        else:
                            i = table.addRow()
                            table.setText(i, 0, name)
                            table.setText(i, 1, code)
                            table.setText(i, 2, locale.toString(float(price), 'f', 2))
                        reportLine = reportData[key]
                        totalReportPrice += price
                        for j in xrange(0, reportRowSize):
                            if isinstance(reportLine[j], float):
                                table.setText(i, j+3, locale.toString(float(reportLine[j]), 'f', 2))
                            else:
                                table.setText(i, j+3, reportLine[j])
                        for j in xrange(reportRowSize):
                            totalByReport[j] += reportLine[j]
                    self.addTotal(table, u'Всего', totalByReport, locale, totalReportPrice)
                    totalByReport = [0]*reportRowSize
                    totalReportPrice = 0

        if self.groupByMonths:
            for date in sorted(reportDataAll.keys()):
                i = table.addRow()
                month = parseDate(date)
                table.setText(i, 0, month, CReportBase.TableTotal, blockFormat=CReportBase.AlignCenter)
                table.mergeCells(i, 0,  1, self.tableColumnsLen)
                printTableData(table, reportDataAll[date])
        else:
            printTableData(table, reportDataAll)

        return doc


    def addTotal(self, table, title, reportLine, locale, totalPrice, width=2):
        i = table.addRow()
        table.mergeCells(i, 0, 1, width)
        table.setText(i, 0, title, CReportBase.TableTotal)
        table.setText(i, width, locale.toString(float(totalPrice), 'f', 2))
        endCol = 12 if self.datailVAT else 9
        for j in xrange(0, endCol):
            if isinstance(reportLine[j], float):
                table.setText(i, j+width+1, locale.toString(float(reportLine[j]), 'f', 2))
            else:
                table.setText(i, j+width+1, reportLine[j])


class CFinanceSummaryByServicesEx(CFinanceSummaryByServices):
    def exec_(self, accountIdList=None):
        self.accountIdList = accountIdList
        CFinanceSummaryByServices.exec_(self)


    def getDescription(self, params):
        db = QtGui.qApp.db
        begDate              = params.get('begDate', None)
        endDate              = params.get('endDate', None)
        eventTypeId          = params.get('eventTypeId', None)
        orgStructureId       = params.get('orgStructureId', None)
        personId             = params.get('personId', None)
        specialityId         = params.get('specialityId', None)
        freeInputWork        = params.get('freeInputWork', False)
        freeInputWorkValue   = params.get('freeInputWorkValue', '')
        orgInsurerId         = params.get('orgInsurerId', None)
        confirmation         = params.get('confirmation', False)
        confirmationBegDate  = params.get('confirmationBegDate', None)
        confirmationEndDate  = params.get('confirmationEndDate', None)
        confirmationType     = params.get('confirmationType', 0)
        refuseType           = params.get('refuseType', None)
        isPeriodOnService    = params.get('isPeriodOnService', False)
        clientOrganisationId = params.get('clientOrganisationId', None)

        rows = []
        if isPeriodOnService:
            rows.append(u'учитывать период по услуге')
        if begDate:
            rows.append(u'Дата начала периода: %s'%forceString(begDate))
        if endDate:
            rows.append(u'Дата окончания периода: %s'%forceString(endDate))
        if eventTypeId:
            rows.append(u'Тип события: %s'%forceString(db.translate('EventType', 'id', eventTypeId, 'CONCAT_WS(\' | \', code,name)')))
        eventTypeList = params.get('eventTypeList', None)
        if eventTypeList:
            tableET = db.table('EventType')
            records = db.getRecordList(tableET, [tableET['name']], [tableET['deleted'].eq(0), tableET['id'].inlist(eventTypeList)])
            nameList = []
            for record in records:
                nameList.append(forceString(record.value('name')))
            rows.append(u'Тип события:  %s'%(u','.join(name for name in nameList if name)))
        if orgStructureId:
            rows.append(u'Подразделение исполнителя: %s'%forceString(db.translate('OrgStructure', 'id', orgStructureId, 'CONCAT_WS(\' | \', code,name)')))
        if personId:
            rows.append(u'Врач: %s'%forceString(db.translate('vrbPerson', 'id', personId, 'name')))
        if specialityId:
            rows.append(u'Специальность: %s'%forceString(db.translate('rbSpeciality', 'id', specialityId, 'CONCAT_WS(\' | \', code,name)')))
        if orgInsurerId:
            rows.append(u'Плательщик по договору: %s'%forceString(db.translate('Organisation', 'id', orgInsurerId, 'shortName')))
        if clientOrganisationId:
            rows.append(u'Место работы: %s'%forceString(db.translate('Organisation', 'id', clientOrganisationId, 'shortName')))
        if freeInputWork:
            rows.append(u'Место работы по названию: %s'%forceString(freeInputWorkValue))
        if confirmation:
            rows.append(u'Тип подтверждения: %s'%{0: u'без подтверждения',
                                                  1: u'оплаченные',
                                                  2: u'отказанные'}.get(confirmationType, u''))
            if confirmationBegDate:
                rows.append(u'Начало периода подтверждения: %s'%forceString(confirmationBegDate))
            if confirmationEndDate:
                rows.append(u'Окончание периода подтверждения: %s'%forceString(confirmationEndDate))
            if refuseType:
                rows.append(u'Причина отказа: %s'%forceString(db.translate('rbPayRefuseType', 'id', refuseType, 'CONCAT_WS(\' | \', code,name)')))
        rows.append(u'Отчёт составлен: '+forceString(QDateTime.currentDateTime()))
        return rows


    def getSetupDialog(self, parent):
        result = CFinanceSummarySetupDialog(parent)
        result.setVisiblePeriodOnService(True)
        result.setVisibleClientOrganisation(True)
        result.setVisibleSocStatusParams(False)
        result.setVisibleDetailFinance(True)
        result.setVisibleDetailPayer(True)
        result.setVisibleDetailVAT(True)
        result.setEventTypeVisible(False)
        result.setEventTypeListVisible(True)
        result.setTitle(self.title())
        return result


    def build(self, params):
        params['accountIdList'] = self.accountIdList
        return CFinanceSummaryByServices.build(self, '\n'.join(self.getDescription(params)), params)



class CDetailedFinanceSummaryByServices(CFinanceSummaryByServicesEx):
    def exec_(self, accountIdList=None):
        self.accountIdList = accountIdList
        CFinanceSummaryByServices.exec_(self)

    def getSetupDialog(self, parent):
        result = CFinanceSummarySetupDialog(parent)
        result.setVisiblePeriodOnService(True)
        result.setVisibleSocStatusParams(True)
        result.setVisibleDetailFinance(True)
        result.setVisibleDetailPayer(True)
        result.setVisibleDetailVAT(True)
        result.setTitle(self.title())
        return result


def parseDate(date):
    return forceString(QDate.fromString(date, 'yyyy.MM').toString('MMMM yyyy'))
