# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
from decimal import Decimal

from PyQt4 import QtGui
from PyQt4 import QtCore

from Events.Action import CActionStatus

from library.Utils import forceRef, forceString, forceDouble, formatNameByRecord, forceDateTime

from Reports.Report     import CReport
from Reports.ReportBase import CReportBase, createTable

from Reports.Ui_NomenclatureReportSetup import Ui_NomenclatureReportDialog


class CNomenclatureReportDialog(QtGui.QDialog, Ui_NomenclatureReportDialog):
    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)
        self.setWindowTitle(u'Параметры отчёта')
        self.setDatesVisible(True)
        self.setMonthVisible(False)
        self.setMonthOptionVisible(False)
        self.setDatesOptionVisible(False)
        self.setClientIdVisible(False)
        self.setOrgStructureVisible(True)
        self.setNomenclatureVisible(True)
        self.setSignaVisible(True)
        self.setSupplierVisible(False)
        self.setSupplierOrgVisible(False)
        self.setReceiverVisible(False)
        self.setReceiverPersonVisible(False)
        self.setFinanceVisible(False)

    def setDatesVisible(self, value):
        self.datesVisible = value
        self.lblBegDate.setVisible(value)
        self.edtBegDate.setVisible(value)
        self.edtBegTime.setVisible(value)
        self.lblEndDate.setVisible(value)
        self.edtEndDate.setVisible(value)
        self.edtEndTime.setVisible(value)

    def setDatesOptionVisible(self, value):
        self.datesOptionVisible = value
        self.chkBegDate.setVisible(value)
        self.edtBegDateOption.setVisible(value)
        self.chkEndDate.setVisible(value)
        self.edtEndDateOption.setVisible(value)

    def setMonthVisible(self, value):
        self.monthVisible = value
        self.lblMonth.setVisible(value)
        self.edtMonth.setVisible(value)
        
    def setMonthOptionVisible(self, value):
        self.monthOptionVisible = value
        self.chkMonth.setVisible(value)
        self.edtMonthOption.setVisible(value)

    def setClientIdVisible(self, value):
        self.clientIdVisible = value
        self.lblClientId.setVisible(value)
        self.edtClientId.setVisible(value)

    def setOrgStructureVisible(self, value):
        self.orgStructureVisible = value
        self.lblOrgStructure.setVisible(value)
        self.cmbOrgStructure.setVisible(value)

    def setNomenclatureVisible(self, value):
        self.nomenclatureVisible = value
        self.lblNomenclature.setVisible(value)
        self.cmbNomenclature.setVisible(value)
    
    def setSignaVisible(self, value):
        self.signaVisible = value
        self.lblSigna.setVisible(value)
        self.edtSigna.setVisible(value)

    def setSupplierVisible(self, value):
        self.supplierVisible = value
        self.lblSupplier.setVisible(value)
        self.cmbSupplier.setVisible(value)

    def setSupplierOrgVisible(self, value):
        self.supplierOrgVisible = value
        self.lblSupplierOrg.setVisible(value)
        self.cmbSupplierOrg.setVisible(value)

    def setReceiverVisible(self, value):
        self.receiverVisible = value
        self.lblReceiver.setVisible(value)
        self.cmbReceiver.setVisible(value)

    def setReceiverPersonVisible(self, value):
        self.receiverPersonVisible = value
        self.lblReceiverPerson.setVisible(value)
        self.cmbReceiverPerson.setVisible(value)

    def setFinanceVisible(self, value):
        self.financeVisible = value
        self.lblFinance.setVisible(value)
        self.cmbFinance.setVisible(value)
        self.cmbFinance.setTable('rbFinance', True)

    def setOnlyExists(self, value):
        self.cmbNomenclature.setOnlyExists(value)

    def setParams(self, params):
        if self.monthVisible:
            self.edtMonth.setDate(params.get('month', QtCore.QDate.currentDate()))
        if self.datesVisible:
            self.edtBegDate.setDate(params.get('begDate', QtCore.QDate.currentDate()))
            self.edtBegTime.setTime(params.get('begTime', QtCore.QTime(0, 0)))
            self.edtEndDate.setDate(params.get('endDate', QtCore.QDate.currentDate()))
            self.edtEndTime.setTime(params.get('endTime', QtCore.QTime(23, 59)))
        if self.monthOptionVisible:
            self.edtMonthOption.setDate(params.get('monthOption', QtCore.QDate.currentDate()))
            self.chkMonth.setChecked(params.get('monthChecked', True))
        if self.datesOptionVisible:
            self.edtBegDateOption.setDate(params.get('begDateOption', QtCore.QDate.currentDate()))
            self.edtEndDateOption.setDate(params.get('endDateOption', QtCore.QDate.currentDate()))
            self.chkBegDate.setChecked(params.get('begDateChecked', False))
            self.chkEndDate.setChecked(params.get('endDateChecked', False))
        if self.clientIdVisible:
            self.edtClientId.setText(params.get('clientId', ''))
        if self.orgStructureVisible:
            self.cmbOrgStructure.setValue(params.get('orgStructureId', None))
        if self.nomenclatureVisible:
            self.cmbNomenclature.setValue(params.get('nomenclatureId', None))
        if self.signaVisible:
            self.edtSigna.setText(params.get('signa', u''))
        if self.supplierVisible:
            self.cmbSupplier.setValue(params.get('supplierId', None))
        if self.supplierOrgVisible:
            self.cmbSupplierOrg.setValue(params.get('supplierOrgId', None))
        if self.receiverVisible:
            self.cmbReceiver.setValue(params.get('receiverId', None))
        if self.receiverPersonVisible:
            self.cmbReceiverPerson.setValue(params.get('receiverPersonId', None))
        if self.financeVisible:
            self.cmbFinance.setValue(params.get('financeId', None))

    def params(self):
        result = {}
        if self.monthVisible:
            result['month'] = self.edtMonth.date()
        if self.datesVisible:
            result['begDate'] = self.edtBegDate.date()
            result['begTime'] = self.edtBegTime.time()
            result['endDate'] = self.edtEndDate.date()
            result['endTime'] = self.edtEndTime.time()
        if self.monthOptionVisible:
            result['monthOption'] = self.edtMonthOption.date()
            result['monthChecked'] = self.chkMonth.isChecked()
        if self.datesOptionVisible:
            result['begDateChecked'] = self.chkBegDate.isChecked()
            result['endDateChecked'] = self.chkEndDate.isChecked()
            result['begDateOption'] = self.edtBegDateOption.date()
            result['endDateOption'] = self.edtEndDateOption.date()
        if self.clientIdVisible:
            result['clientId'] = unicode(self.edtClientId.text())
        if self.orgStructureVisible:
            result['orgStructureId'] = self.cmbOrgStructure.value()
        if self.nomenclatureVisible:
            result['nomenclatureId'] = self.cmbNomenclature.value()
        if self.signaVisible:
            result['signa'] = unicode(self.edtSigna.text())
        if self.supplierVisible:
            result['supplierId'] = self.cmbSupplier.value()
        if self.supplierOrgVisible:
            result['supplierOrgId'] = self.cmbSupplierOrg.value()
        if self.receiverVisible:
            result['receiverId'] = self.cmbReceiver.value()
        if self.receiverPersonVisible:
            result['receiverPersonId'] = self.cmbReceiverPerson.value()
        if self.financeVisible:
            result['financeId'] = self.cmbFinance.value()
        return result


class _CTables(object):
    def __init__(self):
        db = QtGui.qApp.db

        self.E = db.table('Event')
        self.A = db.table('Action')
        self.AT = db.table('ActionType')
        self.Client = db.table('Client')

        self.APn = db.table('ActionProperty').alias('APn')
        self.APTn = db.table('ActionPropertyType').alias('APTn')
        self.APN = db.table('ActionProperty_rbNomenclature')
        self.N = db.table('rbNomenclature')

        self.APd = db.table('ActionProperty').alias('APd')
        self.APTd = db.table('ActionPropertyType').alias('APTd')
        self.APD = db.table('ActionProperty_Double')

        self.APs = db.table('ActionProperty').alias('APs')
        self.APTs = db.table('ActionPropertyType').alias('APTs')
        self.APS = db.table('ActionProperty_String')

        self.Ar = db.table('Action').alias('Ar')
        self.ATr = db.table('ActionType').alias('ATr')
        self.APor = db.table('ActionProperty').alias('APor')
        self.APTor = db.table('ActionPropertyType').alias('APTor')
        self.APOr = db.table('ActionProperty_OrgStructure').alias('APOr')

        self.Am = db.table('Action').alias('Am')
        self.ATm = db.table('ActionType').alias('ATPom')
        self.APom = db.table('ActionProperty').alias('APom')
        self.APTom = db.table('ActionPropertyType').alias('APTom')
        self.APOm = db.table('ActionProperty_OrgStructure').alias('APOm')


def _formatQueryTable(tables, signa):
    db = QtGui.qApp.db

    queryTable = tables.E
    queryTable = queryTable.innerJoin(tables.Client, tables.Client['id'].eq(tables.E['client_id']))
    queryTable = queryTable.innerJoin(tables.A, tables.A['event_id'].eq(tables.E['id']))
    queryTable = queryTable.innerJoin(tables.AT, tables.AT['id'].eq(tables.A['actionType_id']))
    queryTable = queryTable.innerJoin(tables.APTn, tables.APTn['actionType_id'].eq(tables.AT['id']))
    queryTable = queryTable.innerJoin(
        tables.APn,
        db.joinAnd(
            [tables.APn['action_id'].eq(tables.A['id']), tables.APn['type_id'].eq(tables.APTn['id'])]
        )
    )
    queryTable = queryTable.innerJoin(tables.APN, tables.APN['id'].eq(tables.APn['id']))
    queryTable = queryTable.innerJoin(tables.N, tables.N['id'].eq(tables.APN['value']))

    queryTable = queryTable.innerJoin(
        tables.APTd,
        # Доза это 2
        db.joinAnd([tables.APTd['actionType_id'].eq(tables.AT['id']), tables.APTd['inActionsSelectionTable'].eq(2)])
    )
    queryTable = queryTable.innerJoin(
        tables.APd,
        db.joinAnd(
            [tables.APd['action_id'].eq(tables.A['id']), tables.APd['type_id'].eq(tables.APTd['id'])]
        )
    )
    queryTable = queryTable.innerJoin(tables.APD, tables.APD['id'].eq(tables.APd['id']))

    if signa:
        queryTable = queryTable.innerJoin(
            tables.APTs,
            # Сигна это 3
            db.joinAnd([tables.APTs['actionType_id'].eq(tables.AT['id']), tables.APTs['inActionsSelectionTable'].eq(3)])
        )
        queryTable = queryTable.innerJoin(
            tables.APs,
            db.joinAnd(
                [tables.APs['action_id'].eq(tables.A['id']), tables.APs['type_id'].eq(tables.APTs['id'])]
            )
        )
        queryTable = queryTable.innerJoin(tables.APS, tables.APS['id'].eq(tables.APs['id']))

    queryTable = queryTable.innerJoin(tables.Ar, tables.Ar['event_id'].eq(tables.E['id']))
    queryTable = queryTable.innerJoin(tables.ATr, tables.Ar['actionType_id'].eq(tables.ATr['id']))
    queryTable = queryTable.innerJoin(tables.APTor, tables.APTor['actionType_id'].eq(tables.ATr['id']))
    queryTable = queryTable.innerJoin(
        tables.APor,
        db.joinAnd(
            [tables.APor['action_id'].eq(tables.Ar['id']), tables.APor['type_id'].eq(tables.APTor['id'])]
        )
    )
    queryTable = queryTable.innerJoin(tables.APOr, tables.APOr['id'].eq(tables.APor['id']))

    queryTable = queryTable.leftJoin(
        tables.Am, db.joinAnd(
            [tables.Am['event_id'].eq(tables.E['id']), tables.Am['endDate'].isNull()]
        )
    )
    queryTable = queryTable.leftJoin(
        tables.ATm, db.joinAnd(
            [
                tables.Am['actionType_id'].eq(tables.ATm['id']),
                tables.ATm['flatCode'].eq('moving')
            ]
        )
    )
    queryTable = queryTable.leftJoin(tables.APTom, tables.APTom['actionType_id'].eq(tables.ATm['id']))
    queryTable = queryTable.leftJoin(
        tables.APom,
        db.joinAnd(
            [tables.APom['action_id'].eq(tables.Am['id']), tables.APom['type_id'].eq(tables.APTom['id'])]
        )
    )

    return queryTable.leftJoin(tables.APOm, tables.APOm['id'].eq(tables.APom['id']))


def _getDateTimes(params):
    begDate = params.get('begDate')
    endDate = params.get('endDate')

    begTime = params.get('begTime')
    endTime = params.get('endTime')

    if begDate:
        begDateTime = QtCore.QDateTime(begDate)
        if begTime:
            begDateTime.setTime(begTime)
    else:
        begDateTime = None

    if endDate:
        endDateTime = QtCore.QDateTime(endDate)
        if begTime:
            endDateTime.setTime(endTime)
    else:
        endDateTime = None

    return begDateTime, endDateTime


def _getCond(tables, params):
    db = QtGui.qApp.db

    begDateTime, endDateTime = _getDateTimes(params)

    orgStructureId = params.get('orgStructureId')
    signa = params.get('signa')
    nomenclatureId = params.get('nomenclatureId')

    cond = [
        tables.ATr['flatCode'].eq('received'),
        tables.A['status'].eq(CActionStatus.started)
    ]

    if begDateTime:
        cond.append(tables.A['begDate'].gt(begDateTime))
    if endDateTime:
        cond.append(tables.A['begDate'].lt(endDateTime))

    if nomenclatureId:
        cond.append(tables.APN['value'].eq(nomenclatureId))

    if signa:
        cond.append(tables.APS['value'].eq(signa))

    if orgStructureId:
        cond.append(
            db.joinOr(
                [
                    db.joinAnd([tables.APOr['value'].eq(orgStructureId), tables.Ar['endDate'].isNull()]),
                    db.joinAnd([tables.APOm['value'].eq(orgStructureId), tables.Am['endDate'].isNull()]),
                ]
            )
        )
    else:
        cond.append(
            db.joinOr(
                [
                    db.joinAnd([tables.APOr['value'].isNotNull(), tables.Ar['endDate'].isNull()]),
                    db.joinAnd([tables.APOm['value'].isNotNull(), tables.Am['endDate'].isNull()]),
                ]
            )
        )

    return cond


def _getFields(tables):
    return [
        tables.N['id'].alias('nId'),
        tables.N['name'].alias('nName'),
        tables.N['code'].alias('ncode'),
        tables.N['producer'],
        tables.N['dosageValue'],
        tables.A['begDate'],
        tables.A['aliquoticity'],
        tables.Client['id'].alias('clientId'),
        tables.Client['lastName'],
        tables.Client['firstName'],
        tables.Client['patrName'],
        tables.APD['value'].alias('doses'),
        """(
        SELECT
            GROUP_CONCAT(execDate)
        FROM Action_ExecutionPlan
            WHERE DATE(execDate) = (
                SELECT
                    DATE(MIN(execDate))
                FROM
                    Action_ExecutionPlan
                WHERE
                    master_id = Action.id
                )
            AND master_id = Action.id
        ) AS execDates"""
    ]


def _getOrder(tables):
    return [
        tables.A['begDate'].name(),
        tables.N['name'].name(),
        tables.Client['lastName'].name(),
        tables.Client['firstName'].name(),
        tables.Client['patrName'].name(),
    ]


def _formatExecDates(endDateTime, begDate, execDatesAsStr):
    if not execDatesAsStr:
        return begDate.date().toPyDate(), []

    dt_format = 'yyyy-MM-dd HH:mm:ss'
    strList = execDatesAsStr.split(',')

    execDateList = sorted([QtCore.QDateTime.fromString(s, dt_format) for s in strList])

    date = execDateList[0].date()

    return (
        date.toPyDate(),
        [v.time().toPyTime() for v in execDateList
         if (date < endDateTime.date() or (date == endDateTime.date() and v.time() < endDateTime.time()))]
    )


def _selectData(params):
    db = QtGui.qApp.db

    _, endDateTime = _getDateTimes(params)

    tables = _CTables()

    queryTable = _formatQueryTable(tables, params.get('signa'))

    cond = _getCond(tables, params)

    fields = _getFields(tables)

    order = _getOrder(tables)

    stmt = db.selectStmt(queryTable, fields, cond, order=order)

    query = db.query(stmt)

    tmpResult = {}

    while query.next():
        record = query.record()

        clientId = forceRef(record.value('clientId'))
        clientName = formatNameByRecord(record)
        nId = forceRef(record.value('nId'))
        nName = forceString(record.value('nName'))
        nCode = forceString(record.value('nCode'))
        producer = forceString(record.value('producer'))
        doses =  Decimal(str(forceDouble(record.value('doses'))))
        dosageValue = Decimal(str(forceDouble(record.value('dosageValue'))))
        begDate = forceDateTime(record.value('begDate'))
        aliquoticity = Decimal(str(forceDouble(record.value('aliquoticity'))))
        execDate, times = _formatExecDates(endDateTime, begDate, forceString(record.value('execDates')))

        clientQnt = Decimal('1')
        if dosageValue:
            clientQnt = doses / dosageValue

        value = tmpResult \
            .setdefault(execDate, {}) \
            .setdefault((nId, nCode, nName, producer), {}) \
            .setdefault((clientId, clientName),
                {'qnt': Decimal('0'), 'times': [], 'aliquoticity': aliquoticity}
            )

        value['qnt'] += clientQnt
        value['times'].extend(times)

    result = []

    dateList = sorted(tmpResult.keys())
    prevDate = dateList[0] if dateList else None
    nQnt = Decimal('0')
    nName = None

    ONE = Decimal('1')

    for date in dateList:
        newDate = date != prevDate

        dateResult = tmpResult[date]
        prevNId = None

        for key in sorted(dateResult.keys(), key=lambda i: i[2]):
            nId, nCode, _, producer = key
            if prevNId and prevNId != nId or newDate:
                result.append(('NOMENCLATURE_RESULT', {
                    'nName': nName,
                    'qnt': nQnt,
                    'date': forceString(forceString(QtCore.QDate(prevDate)))
                }))
                nQnt = Decimal('0')

            nName = _

            prevNId = nId
            cData = dateResult[key]
            for cKey in sorted(cData.keys()):
                _, clientName = cKey
                timesCount = len(cData[cKey]['times'])

                qntPerOneAction = cData[cKey]['qnt']
                qnt =  qntPerOneAction * timesCount if timesCount else ONE

                times = sorted(list(set(cData[cKey]['times'])))
                times = [forceString(QtCore.QTime(t)) for t in times if t.hour != 0 or t.minute != 0]
                times = ', '.join(times)
                result.append(('ROW_RESULT',
                    {
                        'nomenclatureName': '%s | %s' % (nCode, nName),
                        'producer': producer,
                        'clientName': clientName,
                        'qnt': qnt,
                        'date': forceString(QtCore.QDate(date)),
                        'times': times,
                        'aliquoticity': cData[cKey]['aliquoticity'],
                        'qntPerOneAction': qntPerOneAction
                    }
                ))
                nQnt += qnt

        prevDate = date

    if nQnt and nName is not None:
        result.append(('NOMENCLATURE_RESULT', {
            'nName': nName,
            'qnt': nQnt,
            'date': forceString(forceString(QtCore.QDate(prevDate)))
        }))

    return result


class CPlannedClientInvoiceNomenclaturesReport(CReport):
    def __init__(self, parent = None):
        CReport.__init__(self, parent)
        self.setTitle(u'Отчет о выдаче ЛС на выбранный период')

    def getSetupDialog(self, parent):
        result = CNomenclatureReportDialog(parent)
        result.setOnlyExists(True)
        return result

    def build(self, params):
        tableColumns = [
            ('5%', [u'№'], CReportBase.AlignLeft),
            ('20%', [u'Наименование ЛСиИМН'], CReportBase.AlignLeft),
            ('15%', [u'Производитель'], CReportBase.AlignLeft),
            ('20%', [u'Пациент'], CReportBase.AlignLeft),
            ('5%', [u'Кол-во'], CReportBase.AlignRight),
            ('10%', [u'Дата'], CReportBase.AlignRight),
            ('10%', [u'Время'], CReportBase.AlignRight),
            ('5%', [u'Кратность'], CReportBase.AlignRight),
            ('5%', [u'Кол-во на 1 прием'], CReportBase.AlignRight)
        ]

        doc = QtGui.QTextDocument()
        cursor = QtGui.QTextCursor(doc)

        cursor.setCharFormat(CReportBase.ReportTitle)
        cursor.insertText(self.title())
        cursor.insertBlock()
        self.dumpParams(cursor, params)
        cursor.insertBlock()

        table = createTable(cursor, tableColumns)

        data = _selectData(params)

        rowNumber = 1
        for dataType, values in data:
            row = table.addRow()
            if dataType == 'NOMENCLATURE_RESULT':
                table.mergeCells(row, 0, 1, len(tableColumns))
                table.setText(row, 0, u"%s \"%s\": %s" % (values['date'], values['nName'], values['qnt']))
            else:
                table.setText(row, 0, rowNumber)
                table.setText(row, 1, values['nomenclatureName'])
                table.setText(row, 2, values['producer'])
                table.setText(row, 3, values['clientName'])
                table.setText(row, 4, values['qnt'])
                table.setText(row, 5, values['date'])
                table.setText(row, 6, values['times'])
                table.setText(row, 7, values['aliquoticity'])
                table.setText(row, 8, values['qntPerOneAction'])

                rowNumber += 1

        return doc
