# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui

from Reports.ReportBase import CReportBase, createTable, CReportTableBase
from library.Utils      import forceDouble, forceString, forceBool, forceInt



def selectData(accountItemIdList, orgInsurerId = None):
    db = QtGui.qApp.db
    tableAccountItem = db.table('Account_Item')
    tableEventType = db.table('EventType')
    cond = db.joinAnd([tableAccountItem['id'].inlist(accountItemIdList), tableEventType['medicalAidType_id'].inlist([6, 9])])
    stmt="""
SELECT 
    IF(Account_Item.visit_id,
        VisitSpeciality.name,
        ActionSpeciality.name) AS specialityName,
    rbService.code,
    rbService.name,
    SUM(Account_Item.amount) AS amount,
    IF(Account_Item.visit_id, 1, 0) AS visitFlag,
    COUNT(DISTINCT Event.id) AS eventAmount,
    SUM(Account_Item.sum) AS itemsSum,
    SUM((SELECT 
            COUNT(id)
        FROM
            Visit AS V
        WHERE
            V.event_id = Account_Item.event_id
                AND V.service_id = Visit.service_id)) AS sameVisitsInEvent
FROM
    Account_Item
        LEFT JOIN
    rbService ON rbService.id = Account_Item.service_id
        LEFT JOIN
    Event ON Event.id = Account_Item.event_id
        LEFT JOIN
    EventType ON EventType.id = Event.eventType_id
        LEFT JOIN
    Visit ON Visit.id = Account_Item.visit_id
        LEFT JOIN
    Person AS VisitPerson ON VisitPerson.id = Visit.person_id
        LEFT JOIN
    rbSpeciality AS VisitSpeciality ON VisitSpeciality.id = VisitPerson.speciality_id
        LEFT JOIN
    Action ON Action.id = Account_Item.action_id
        LEFT JOIN
    Person AS ActionPerson ON ActionPerson.id = Action.person_id
        LEFT JOIN
    rbSpeciality AS ActionSpeciality ON ActionSpeciality.id = ActionPerson.speciality_id
    WHERE
    %(cond)s"""    % {
        'cond' : cond
        }
    stmt += """
GROUP BY specialityName , rbService.code , rbService.name , visitFlag
ORDER BY specialityName , rbService.code , rbService.name , visitFlag
    """
    query = db.query(stmt)
    return query


class CPolyclinicAccountSummary(CReportBase):
    def __init__(self, parent):
        CReportBase.__init__(self, parent)
        self.setTitle(u'Финансовая сводка по амбулаторной помощи')


    def build(self, description, params):
        accountId = params.get('accountId', None)
        accountItemIdList = params.get('accountItemIdList', None)
        number, infis, payer, recipient = self.getContract(accountId)
        query = selectData(accountItemIdList)

        doc = QtGui.QTextDocument()
        cursor = QtGui.QTextCursor(doc)

        tableColumns1 = [
            ('50%', [u'Наименование ЛПУ: %s'%recipient, u'код %s'%infis], CReportBase.AlignLeft),
            ('50%', [u'Наименование плательщика: %s'%payer,u''], CReportBase.AlignLeft)
        ]

        tableColumns2 = [
            ('5%', [u'№ п/п'], CReportBase.AlignLeft),
            ('25%', [u'Наименование врачебной специальности'], CReportBase.AlignLeft),
            ('15%', [u'Код услуги'], CReportBase.AlignLeft),
            ('25%', [u'Наименование услуги'], CReportBase.AlignLeft),
            ('10%', [u'Кол-во обращ. и посещ.'], CReportBase.AlignRight),
            ('10%', [u'Кол-во услуг'], CReportBase.AlignRight),
            ('10%', [u'Стоимость'], CReportBase.AlignRight),
        ]

        cursor.setCharFormat(CReportBase.ReportBody)
        table = self.createTable(cursor, tableColumns1, border = 0, alignHeader = CReportBase.AlignLeft)
        table.mergeCells(0, 0, 1, 1) 
        table.mergeCells(0, 1, 1, 1) 
        table.mergeCells(0, 2, 1, 1) 
        table.mergeCells(0, 3, 1, 1) 
        cursor.movePosition(QtGui.QTextCursor.End)
        cursor.insertBlock()
        cursor.insertBlock()
        boldChars = QtGui.QTextCharFormat()
        boldChars.setFontWeight(QtGui.QFont.Bold)

        table = self.createTable(cursor, [(u'100%', u'Сводный отчет по  оказанным медицинским услугам в амбулаторно-поликлинических условиях по счету № %s '%number, CReportBase.AlignLeft), ], border = 0)
        table.mergeCells(0, 0, 1, 1) 
        cursor.insertBlock()
        cursor.insertBlock()
        
        table = createTable(cursor, tableColumns2)
        table.mergeCells(0, 0, 1, 1) 
        table.mergeCells(0, 1, 1, 1) 
        table.mergeCells(0, 2, 1, 1) 
        table.mergeCells(0, 3, 1, 1) 
        table.mergeCells(0, 4, 1, 1)
        table.mergeCells(0, 5, 1, 1)
        table.mergeCells(0, 6, 1, 1)
        
        i=0
        prevSpecialityName = u''
        subTotalVisits = 0
        subTotalActions = 0
        subTotalEvents = 0
        subTotalSum = 0
        totalVisits = 0
        totalActions = 0
        totalEvents = 0
        totalSum = 0
        while query.next():
            record = query.record()
            specialityName = forceString(record.value('specialityName'))
            code = forceString(record.value('code'))
            name = forceString(record.value('name'))
            amount = forceInt(record.value('amount'))
            visitFlag = forceBool(record.value('visitFlag'))
            eventAmount = forceInt(record.value('eventAmount'))
            itemsSum = forceDouble(record.value('itemsSum'))
            sameVisitsInEvent = forceInt(record.value('sameVisitsInEvent'))

            if prevSpecialityName!=specialityName and prevSpecialityName != u'':
                row = table.addRow()
                i+=1
                table.setText(row, 0, i)
                table.setText(row, 1, u'Итого по %s'%prevSpecialityName, charFormat=boldChars)
                table.setText(row, 2, u'')
                table.setText(row, 3, u'')
                table.setText(row, 4, u'%i / %i'%(subTotalEvents, subTotalVisits), charFormat=boldChars)
                table.setText(row, 5, subTotalActions, charFormat=boldChars)
                table.setText(row, 6, subTotalSum, charFormat=boldChars)
                subTotalVisits = 0
                subTotalActions = 0
                subTotalEvents = 0
                subTotalSum = 0
            
            row = table.addRow()
            i+=1
            table.setText(row, 0, i)
            table.setText(row, 1, specialityName if prevSpecialityName!=specialityName else u'')
            table.setText(row, 2, code)
            table.setText(row, 3, name)
            table.setText(row, 4, u'%i / %i'%(eventAmount, sameVisitsInEvent)  if visitFlag else u'')
            table.setText(row, 5, amount if not visitFlag else u'')
            table.setText(row, 6, itemsSum)
            prevSpecialityName = specialityName
            if visitFlag:
                totalVisits += sameVisitsInEvent
                totalEvents += eventAmount
                subTotalVisits += sameVisitsInEvent
                subTotalEvents += eventAmount
            else:
                subTotalActions += amount
                totalActions += amount
            totalSum += itemsSum
            subTotalSum += itemsSum

        row = table.addRow()
        i+=1
        table.setText(row, 0, i)
        table.setText(row, 1, u'Итого по %s'%prevSpecialityName, charFormat=boldChars)
        table.setText(row, 2, u'')
        table.setText(row, 3, u'')
        table.setText(row, 4, u'%i / %i'%(subTotalEvents, subTotalVisits), charFormat=boldChars)
        table.setText(row, 5, subTotalActions, charFormat=boldChars)
        table.setText(row, 6, subTotalSum, charFormat=boldChars)
        subTotalVisits = 0
        subTotalActions = 0
        subTotalEvents = 0
        subTotalSum = 0

        row = table.addRow()
        i+=1        
        table.setText(row, 0, i)
        table.setText(row, 1, u'Итого по ЛПУ', charFormat=boldChars)
        table.setText(row, 2, u'')
        table.setText(row, 3, u'')
        table.setText(row, 4, u'%i / %i'%(totalEvents, totalVisits), charFormat=boldChars)
        table.setText(row, 5, totalActions, charFormat=boldChars)
        table.setText(row, 6, totalSum, charFormat=boldChars)

        return doc


    def getContract(self, accountId):
        db = QtGui.qApp.db
        tableAccount = db.table('Account')
        tableContract = db.table('Contract')
        tableOrganisation = db.table('Organisation')
        tablePayer = tableOrganisation.alias('Payer')
        tableRecipient = tableOrganisation.alias('Recipient')
        queryTable = tableContract.leftJoin(tableAccount, tableContract['id'].eq(tableAccount['contract_id']))
        queryTable = queryTable.leftJoin(tablePayer, tableContract['payer_id'].eq(tablePayer['id']))
        queryTable = queryTable.leftJoin(tableRecipient, tableContract['recipient_id'].eq(tableRecipient['id']))
        record = db.getRecordEx(queryTable,
                                [tableContract['number'].name(), tableRecipient['infisCode'].alias('infisCode'),
                                 tablePayer['shortName'].alias('payer'), tableRecipient['shortName'].alias('recipient')
                                ],
                                tableAccount['id'].eq(accountId)
                               )
        if record:
            return (forceString(record.value('number')), forceString(record.value('infisCode')),
                    forceString(record.value('payer')),  forceString(record.value('recipient')))
        else:
            return ('', '', '', '')

    def createTable(self, testCursor, columnDescrs, headerRowCount=1, border=1, cellPadding=2, cellSpacing=0, alignHeader=None):
        def widthToTextLenght(width):
            widthSpec = QtGui.QTextLength.VariableLength
            widthVal  = 0
            try:
                if type(width) == str:
                    if len(width)>0:
                        if width[-1:] == '%':
                            widthVal  = float(width[:-1])
                            widthSpec = QtGui.QTextLength.PercentageLength
                        elif width[-1:] == '?':
                            widthVal  = float(width[:-1])
                        elif width[-1:] == '=':
                            widthVal  = float(width[:-1])
                            widthSpec = QtGui.QTextLength.FixedLength
                        else:
                            widthVal  = float(width)
                            widthSpec = QtGui.QTextLength.FixedLength
                else:
                    widthVal  = float(width)
                    widthSpec = QtGui.QTextLength.FixedLength
            except:
                pass
            return QtGui.QTextLength(widthSpec, widthVal)


        columnWidthConstraints = []
        for columnDescr in columnDescrs:
            assert type(columnDescr)==tuple
            width, headers, align = columnDescr
            columnWidthConstraints.append(widthToTextLenght(width))
            if type(headers) == list:
                headerRowCount = max(headerRowCount,  len(headers))

        tableFormat = QtGui.QTextTableFormat()
        tableFormat.setBorder(border)
        tableFormat.setCellPadding(cellPadding)
        tableFormat.setCellSpacing(cellSpacing)
        tableFormat.setColumnWidthConstraints(columnWidthConstraints)
        tableFormat.setHeaderRowCount(headerRowCount)
    #    tableFormat.setBackground(QtGui.QBrush(Qt.red))
        table = testCursor.insertTable(max(1, headerRowCount), max(1, len(columnDescrs)), tableFormat)

        column = 0
        aligns = []
        for columnDescr in columnDescrs:
            assert type(columnDescr)==tuple
            width, headers, align = columnDescr
            if type(headers) != list:
                headers = [ headers ]
            row = 0
            for header in headers:
                if header != '':
                    cellCursor = table.cellAt(row, column).firstCursorPosition()
                    if alignHeader:
                        cellCursor.setBlockFormat(alignHeader)
                    else:
                        cellCursor.setBlockFormat(CReportBase.AlignCenter)
                    cellCursor.setCharFormat(CReportBase.TableHeader)
                    cellCursor.insertText(header)
                row += 1
            aligns.append(align)
            column += 1

        return CReportTableBase(table, aligns)
