# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui
from PyQt4.QtCore import QDate

from library.Utils             import forceDate, forceInt, forceRef, forceString

from Reports.Report            import CReport
from Reports.ReportBase        import CReportBase, createTable
from Reports.ReportSetupDialog import CReportSetupDialog
from Orgs.Utils                import getOrgStructurePersonIdList


def selectData(params):
    begDate = params.get('begDate', QDate())
    endDate = params.get('endDate', QDate())
    orgStructureId = params.get('orgStructureId', None)
    if not endDate:
        return None
    orgStructure = u''
    if orgStructureId:
        personIdList = getOrgStructurePersonIdList(orgStructureId)
        if personIdList:
            orgStructure = u''' AND Event.execPerson_id IN (%s)'''%(u','.join(str(personId) for personId in personIdList if personId))
    db = QtGui.qApp.db
    stmt = u'''
SELECT
--    Event.id AS eventId,
--    Event.client_id AS clientId,
--    Event.execDate AS eventExecDate,
    Action.id AS actionId,
    Action.status,
    Action.endDate,
    Action.MKB AS actionMkb,
--    Action.actionType_id AS actionTypeId,
--    CONCAT_WS(' | ', ActionType.code, ActionType.name) AS actionTypeName,
--    rbService.id AS serviceId,
--    rbService.code AS serviceCode,
--    rbService.name AS serviceName,

--    mes.MES_visit.id AS mvId,
--    mes.MES_visit.serviceCode AS mvServiceCode,
    mes.MES_visit.additionalServiceCode AS numMesVisitCode,
    DATE(Action.endDate) BETWEEN DATE(Event.setDate) AND DATE(Event.execDate) AS actionExecNow,
    DATE(Action.endDate) < DATE(Event.setDate) AS actionExecPrev,

    EXISTS(SELECT 1
           FROM ActionProperty
           WHERE ActionProperty.action_id = Action.id
             AND ActionProperty.deleted = 0
             AND ActionProperty.evaluation IS NOT NULL
             AND ActionProperty.evaluation != 0
          ) AS propertyEvaluation

FROM Event
INNER JOIN mes.MES ON mes.MES.id=Event.MES_id
INNER JOIN mes.MES_visit ON mes.MES_visit.master_id=mes.MES.id
INNER JOIN mes.mrbMESGroup ON mes.mrbMESGroup.id=mes.MES.group_id
INNER JOIN Action ON Action.event_id=Event.id
INNER JOIN ActionType ON ActionType.id=Action.actionType_id
INNER JOIN rbService ON rbService.id=ActionType.nomenclativeService_id
INNER JOIN Person ON Person.id=Action.person_id
INNER JOIN rbSpeciality ON rbSpeciality.id=Person.speciality_id
INNER JOIN mes.mrbSpeciality ON mes.mrbSpeciality.id=mes.MES_visit.speciality_id

WHERE Event.deleted=0
  %(orgStructure)s
  AND Event.prevEvent_id IS NULL
  AND DATE(Event.execDate) BETWEEN %(begDate)s AND %(endDate)s
  AND mes.mrbMESGroup.code='ДиспанС'
  AND mes.MES_visit.deleted = 0
  AND Action.deleted=0
  AND ActionType.deleted=0
  AND (     mes.MES_visit.serviceCode = SUBSTR(rbService.code,1, CHAR_LENGTH(mes.MES_visit.serviceCode))
        AND SUBSTR(rbService.code, CHAR_LENGTH(mes.MES_visit.serviceCode)+1,1) IN ('','.','*')
      )
ORDER BY Action.id, mes.MES_visit.additionalServiceCode;
''' % { 'orgStructure':orgStructure,
        'begDate': db.formatDate(begDate),
        'endDate': db.formatDate(endDate)
      }
    return db.query(stmt)


class CReportForm131_o_2000_2015(CReport):
    def __init__(self, parent):
        CReport.__init__(self, parent)
        self.setTitle(u'Сведения о первом этапе диспансеризации определенных групп взрослого населения (2015)')

        self.mapNumMesVisitCodeToRow = {
                    1:  0,
                    3:  1,
                    2:  2,
                    4:  3,
                    5:  4,
                    6:  5,
                    86: 6,
                    15: 7,
                    18: 8,
                    14: 9,
                    19:10,
                    8: 11,
                    9: 12,
                    11:13,
                    10:14,
                    12:15,
                    13:16,
                    87:16,
                    89:16,
                    88:17,
                    7: 18,
                    17:19,
                    51:19,
                    }

        self.rowNames = [
                     u'Опрос(анкетирование), направленный на выявление хронических неинфекционных заболеваний, факторов риска их развития, потребления наркотических средств и психотропных веществ без назначения врача',
                     u'Антропометрия (измерение роста стоя, массы тела, окружности талии), расчет индекса массы тела',
                     u'Измерение артериального давления',
                     u'Определение уровня общего холестерина в крови',
                     u'Определение уровня глюкозы в крови экспресс-методом',
                     u'Определение относительного суммарного сердечно-сосудистого риска',
                     u'Определение абсолютного суммарного сердечно-сосудистого риска',
                     u'Электрокардиография (в покое)',
                     u'Осмотр фельдшером (акушеркой), включая взятие мазка (соскоба) с поверхности шейки матки (наружного маточного зева) и цервикального канала на цитологическое исследование',
                     u'Флюорография легких',
                     u'Маммография обеих молочных желез',
                     u'Клинический анализ крови',
                     u'Клинический анализ крови развернутый',
                     u'Анализ крови биохимический общетерапевтический',
                     u'Общий анализ мочи',
                     u'Исследование кала на скрытую кровь иммунохимическим методом',
                     u'Ультразвуковое исследование (УЗИ) на предмет исключения новообразований органов брюшной полости, малого таза и аневризмы брюшной аорты',
                     u'Ультразвуковое исследование (УЗИ) в целях исключения аневризмы брюшной аорты',
                     u'Измерение внутриглазного давления',
                     u'Прием (осмотр) врача-терапевта',
                    ]


    def getSetupDialog(self, parent):
        result = CReportSetupDialog(parent)
        result.setEventTypeVisible(False)
        result.setOnlyPermanentAttachVisible(False)
        result.setOrgStructureVisible(True)
        result.setTitle(self.title())
        return result


    def _getDefault(self):
        result = [ [0, 0, 0, 0] for row in self.rowNames
                 ]
        return result


    def getReportData(self, query):
        reportData = self._getDefault()
        reportDataTotal = [0, 0, 0, 0]
        uniqueSet = set()
        clientIdSetExecNow = set()
        clientIdSetExecPrev = set()
        clientIdSetExecRefusal = set()
        clientIdWithMkbSet = set()
        while query.next():
            record = query.record()
            actionId = forceRef(record.value('actionId'))
            actionMkb = forceString(record.value('actionMkb'))
            clientId = forceRef(record.value('clientId'))
            numMesVisitCode = forceInt(record.value('numMesVisitCode'))
            actionExecNow = forceInt(record.value('actionExecNow'))
            actionExecPrev = forceInt(record.value('actionExecPrev'))
            propertyEvaluation = forceInt(record.value('propertyEvaluation'))
            endDate = forceDate(record.value('endDate'))
            status = forceInt(record.value('status'))
            actionExecRefusal = (not endDate) and (status == 3)

            if numMesVisitCode not in self.mapNumMesVisitCodeToRow:
                continue
            if actionId in uniqueSet:
                continue
            uniqueSet.add(actionId)

            reportLine = reportData[self.mapNumMesVisitCodeToRow[numMesVisitCode]]
            if endDate and status != 3:
                if actionExecNow:
                    reportLine[0] += 1
                    reportDataTotal[0] += 1
                elif actionExecPrev:
                    reportLine[1] += 1
                    reportDataTotal[1] += 1
            if actionExecRefusal:
                reportLine[2] += 1
                reportDataTotal[2] += 1
            if (actionMkb and 'A00'<=actionMkb<'U') or propertyEvaluation:
                reportLine[3] += 1
                reportDataTotal[3] += 1
        return reportData, reportDataTotal


    def build(self, params):
        doc = QtGui.QTextDocument()
        cursor = QtGui.QTextCursor(doc)
        cursor.setCharFormat(CReportBase.ReportTitle)
        cursor.insertText(self.title())
        cursor.insertBlock()
        self.dumpParams(cursor, params)
        cursor.insertBlock()
        cursor.insertText(u'(2000)')
        cursor.insertBlock()
        tableColumns = [
            ( '45%', [u'', u'Осмотр, исследование, иное медицинское мероприятие первого этапа диспансеризации', u'1'], CReportBase.AlignLeft),
            ( '3%',  [u'', u'№ строки', u'2'], CReportBase.AlignRight),
            ( '15%', [u'Медицинское мероприятие', u'проведено', u'3'], CReportBase.AlignRight),
            ( '15%', [u'', u'учтено, выполненных ранее (в предшествующие 12 мес.)', u'4'], CReportBase.AlignRight),
            ( '15%', [u'', u'отказы', u'5'], CReportBase.AlignRight),
            ( '15%', [u'Выявлены патологические отклонения', u'', u'6'], CReportBase.AlignRight),
        ]
        table = createTable(cursor, tableColumns)
        table.mergeCells(0, 0, 2, 1)
        table.mergeCells(0, 1, 2, 1)
        table.mergeCells(0, 2, 1, 3)
        table.mergeCells(0, 5, 2, 1)
        query = selectData(params)
        if query is None:
            return doc
        reportData, total = self.getReportData(query)

        for row, name in enumerate(self.rowNames):
            reportLine = reportData[row]
            i = table.addRow()
            table.setText(i, 0, name)
            table.setText(i, 1, row+1)
            table.setText(i, 2, reportLine[0])
            table.setText(i, 3, reportLine[1])
            table.setText(i, 4, reportLine[2])
            table.setText(i, 5, reportLine[3])
        i = table.addRow()
        table.mergeCells(i, 0, 1, 2)
        table.setText(i, 1, u'Всего')
        table.setText(i, 2, total[0])
        table.setText(i, 3, total[1])
        table.setText(i, 4, total[2])
        table.setText(i, 5, total[3])
        return doc

