# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui
from PyQt4.QtCore import QDate

from library.Utils             import forceInt, forceString
from Reports.Report            import CReport
from Reports.ReportBase        import CReportBase, createTable
from Reports.ReportSetupDialog import CReportSetupDialog


def selectData(params):
    begDate = params.get('begDate', QDate())
    endDate = params.get('endDate', QDate())

    db = QtGui.qApp.db

    tableClientVaccination = db.table('ClientVaccination')
    tableVaccine = db.table('rbVaccine')
    tableInfectionVaccine = db.table('rbInfection_rbVaccine')
    tableInfection = db.table('rbInfection')
    tableClient = db.table('Client')

    queryTable = tableClientVaccination
    queryTable = queryTable.leftJoin(tableVaccine, tableVaccine['id'].eq(tableClientVaccination['vaccine_id']))
    queryTable = queryTable.leftJoin(tableInfectionVaccine, tableInfectionVaccine['vaccine_id'].eq(tableVaccine['id']))
    queryTable = queryTable.leftJoin(tableInfection, tableInfection['id'].eq(tableInfectionVaccine['infection_id']))
    queryTable = queryTable.leftJoin(tableClient, tableClient['id'].eq(tableClientVaccination['client_id']))

    cond = []
    if begDate:
        cond.append(tableClientVaccination['datetime'].dateGe(begDate))
    if endDate:
        cond.append(tableClientVaccination['datetime'].dateLe(endDate))

    fields = [tableClient['id'].alias('clientId'),
              'age(Client.`birthDate`, DATE(ClientVaccination.`datetime`)) AS clientAge',
              tableClientVaccination['vaccinationType'],
              tableInfection['code'].alias('infectionCode')]

    stmt = db.selectStmt(queryTable, fields, cond)
#    print stmt
    return db.query(stmt)


class CReportImunoprophylaxisForm5(CReport):
    def __init__(self, parent):
        CReport.__init__(self, parent)
        self.setTitle(u'Сведения о профилактических прививках')
        # key -> (rbInfection.code, isRevaccination), value -> reportRow
        self._mapInfectionCode2row = {(u'01', False) : 50,
                                      (u'01', True)  : 51,
                                      (u'02', False) : 29,
                                      (u'02', True)  : 30,
                                      (u'03', False) : 20,
                                      (u'04', False) : 44,
                                      (u'05', False) : 49,
                                      (u'06', False) : 25,
                                      (u'07', False) : 23,
                                      (u'08', False) : None,
                                      (u'09', False) : 35,
                                      (u'10', False) : 53,
                                      (u'11', False) : 2,
                                      (u'11', True)  : 4,
                                      (u'12', False) : 34,
                                      (u'13', False) : 37,
                                      (u'13', True)  : 39,
                                      (u'14', False) : 0,
                                      (u'14', True)  : 1,
                                      (u'15', False) : 12,
                                      (u'15', True)  : 14,
                                      (u'16', False) : 18,
                                      (u'16', True)  : 19,
                                      (u'17', False) : 52,
                                      (u'18', False) : 41,
                                      (u'19', False) : 42,
                                      (u'20', False) : None,
                                      (u'21', False) : 16,
                                      (u'21', True)  : 17,
                                      (u'22', False) : 45,
                                      (u'22', True)  : 47,
                                      (u'23', False) : 10,
                                      (u'23', True)  : 11,
                                      (u'24', False) : None,
                                      (u'25', False) : 31,
                                      (u'25', True)  : 32,
                                      (u'26', False) : 6,
                                      (u'26', True)  : 8,
                                      (u'27', False) : None,
                                      (u'28', False) : 21,
                                      (u'29', False) : 27,
                                      (u'29', True)  : 28,
                                      (u'30', False) : None,
                                      (u'31', False) : None,
                                      (u'32', False) : 33}
        self._rowsWithChildren = [2, 4, 6, 8, 12, 14, 23, 25, 35, 37, 39, 45, 47]

    def getSetupDialog(self, parent):
        result = CReportSetupDialog(parent)
        result.setEventTypeVisible(False)
        result.setOnlyPermanentAttachVisible(False)
        result.setTitle(self.title())
        return result


    def _applyDefaults(self):
        rows  = [[u'Вакцинация против коклюша'], # 0
                 [u'Ревакцинация против коклюша'], # 1
                 [u'Вакцинация против дифтерии - всего'], # 2
                 [u'в том числе детей'], # 3
                 [u'Ревакцинация против дифтерии - всего'], # 4
                 [u'в том числе детей'], # 5
                 [u'Вакцинация против столбняка - всего'], # 6
                 [u'в том числе детей'], # 7
                 [u'Ревакцинация против столбняка - всего'], # 8
                 [u'в том числе детей'], # 9
                 [u'Вакцинация против полиомиелита'], # 10
                 [u'Ревакцинация против полиомиелита'], # 11
                 [u'Вакцинация против кори - всего'], # 12
                 [u'в том числе детей'], # 13
                 [u'Ревакцинация против кори - всего'], # 14
                 [u'в том числе детей'], # 15
                 [u'Вакцинация против эпидемического паротита'], # 16
                 [u'Ревакцинация против эпидемического паротита'], # 17
                 [u'Вакцинация против краснухи'], # 18
                 [u'Ревакцинация против краснухи'], # 19
                 [u'Прививки против брюшного тифа'], # 20
                 [u'Прививки против туберкулеза - всего'], # 21
                 [u'в том числе новорожденным'], # 22
                 [u'Вакцинация против вирусного гепатита В - всего'], # 23
                 [u'в том числе детей'], # 24
                 [u'Прививки против вирусного гепатита А - всего'], # 25
                 [u'в том числе детей'], # 26
                 [u'Вакцинация против туляремии'], # 27
                 [u'Ревакцинация против туляремии'], # 28
                 [u'Вакцинация против бруцеллеза'], # 29
                 [u'Ревакцинация против бруцеллеза'], # 30
                 [u'Вакцинация против сибирской язвы'], # 31
                 [u'Ревакцинация против сибирской язвы'], # 32
                 [u'Прививки против чумы'], # 33
                 [u'Прививки против желтой лихорадки'], # 34
                 [u'Прививки против гриппа - всего'], # 35
                 [u'в том числе детям'], # 36
                 [u'Вакцинация против клещевого энцефалита - всего'], # 37
                 [u'в том числе детей'], # 38
                 [u'Ревакцинация против клещевого энцефалита - всего'], # 39
                 [u'в том числе детей'], # 40
                 [u'Прививки против лептоспироза'], # 41
                 [u'Прививки против менингококковой инфекции'], # 42
                 [u'Прививки против гемофильной инфекции'], # 43
                 [u'Прививки против ветряной оспы'], # 44
                 [u'Вакцинация против пневмококковой инфекции - всего'], # 45
                 [u'в том числе детей'], # 46
                 [u'Ревакцинация против пневмококковой инфекции - всего'], # 47
                 [u'в том числе детей'], # 48
                 [u'Прививки против вируса папилломы человека'], # 49
                 [u'Вакцинация против бешенства'], # 50
                 [u'Ревакцинация против бешенства'], # 51
                 [u'Прививки против лихорадки Ку'], # 52
                 [u'Прививки против дизентерии Зонне']] # 53
        for idx, row in enumerate(rows):
            row.append(str(idx+1).zfill(2))
            row.append(0)

        return rows


    def getReportData(self, query):

        reportData = self._applyDefaults()

        while query.next():

            record = query.record()

            vaccinationType = forceString(record.value('vaccinationType'))
            infectionCode = forceString(record.value('infectionCode'))

            row = self._mapInfectionCode2row.get((infectionCode, vaccinationType[:1]=='R'), None)
            if row is not None:
                reportData[row][2] += 1
                if row in self._rowsWithChildren:
                    if forceInt(record.value('clientAge')) < 18:
                        reportData[row+1][2] += 1

        return reportData


    def build(self, params):
        doc = QtGui.QTextDocument()
        cursor = QtGui.QTextCursor(doc)

        cursor.setCharFormat(CReportBase.ReportTitle)
        cursor.insertText(self.title())
        cursor.insertBlock()
        self.dumpParams(cursor, params)
        cursor.insertBlock()

        tableColumns = [( '30%', [u'Наименование'], CReportBase.AlignLeft),
                        ( '5%',  [u'№ строки'], CReportBase.AlignRight),
                        ( '15%', [u'Число привитых лиц'], CReportBase.AlignRight)]

        table = createTable(cursor, tableColumns)

        query = selectData(params)

        reportData = self.getReportData(query)

        for reportLine in reportData:
            i = table.addRow()

            for idx, value in enumerate(reportLine):
                table.setText(i, idx, value)

        return doc
