# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import datetime
import string

from PyQt4 import QtGui
from PyQt4.QtCore import Qt, pyqtSignature, QDate

from library.Utils      import forceDate, forceDouble, forceInt, forceString

from Events.Utils       import getWorkEventTypeFilter
from Orgs.Utils         import getOrgStructureDescendants
from Reports.Report     import CReport
from Reports.ReportBase import createTable, CReportBase


def calculateData(begDate, endDate, eventPurposeId, eventTypeId, orgStructureId, personId, sex, ageFrom, ageTo, type, detailBed, specialityId, financeId, includeHoliday):
    db = QtGui.qApp.db
    tableVisit  = db.table('Visit')
    tableEvent  = db.table('Event')
    tableAction = db.table('Action')
    tableEventType = db.table('EventType')
    tablePerson = db.table('Person')
    tableClient = db.table('Client')
    tableSpeciality = db.table('rbSpeciality')
    rbFinance = db.table('rbFinance')
    cond = []

#    addDateInRange(cond, tableVisit['date'], begDate, endDate)
#    cond.append(tableEvent['setDate'].ge(begDate))
#    cond.append(tableEvent['execDate'].lt(endDate))
    if eventTypeId:
        cond.append(tableEvent['eventType_id'].eq(eventTypeId))
    if specialityId:
        cond.append(tablePerson['speciality_id'].eq(specialityId))
    elif eventPurposeId:
        tableEvent = db.table('Event')
        cond.append(tableEventType['purpose_id'].eq(eventPurposeId))
    if personId:
        cond.append(tablePerson['id'].eq(personId))

    if orgStructureId:
        cond.append(tablePerson['orgStructure_id'].inlist(getOrgStructureDescendants(orgStructureId)))
    else:
        cond.append(tablePerson['org_id'].eq(QtGui.qApp.currentOrgId()))
    if sex:
        cond.append(tableClient['sex'].eq(sex))
    if ageFrom <= ageTo:
        cond.append('Event.setDate >= ADDDATE(Client.birthDate, INTERVAL %d YEAR)'%ageFrom)
        cond.append('Event.execDate < SUBDATE(ADDDATE(Client.birthDate, INTERVAL %d YEAR),1)'%(ageTo+1))
    if financeId:
        cond.append(rbFinance['code'].eq(financeId))
    data =[]
    dataBedProfile = []
    bedIds = []
    if detailBed:
        dataBedProfile, bedIds  = createQueryForAmbulance(db, u'Action', cond,
                                                 createCondition( 5, 1, 1, 3, 2, begDate, endDate, False, None, None, None, 1, None, detailBed),
                                                 'rbHospitalBedProfile.id as id, rbHospitalBedProfile.name',
                                                 u"""LEFT JOIN `ActionType` ON Action.actionType_id=`ActionType`.id
                                                     LEFT JOIN ActionProperty ON ActionProperty.action_id = Action.id
                                                    LEFT JOIN `ActionPropertyType` ON `ActionPropertyType`.id = ActionProperty.type_id
                                                    LEFT JOIN ActionProperty_HospitalBed ON ActionProperty_HospitalBed.id = ActionProperty.id
                                                    LEFT JOIN OrgStructure_HospitalBed ON OrgStructure_HospitalBed.id = ActionProperty_HospitalBed .value
                                                    LEFT JOIN rbHospitalBedProfile ON rbHospitalBedProfile.id = OrgStructure_HospitalBed.profile_id
                                                 """, detailBed)

    data = selectData(db, cond, type, orgStructureId, begDate, endDate, detailBed, bedIds, includeHoliday)
    if detailBed:
        return data, dataBedProfile
    else:
        return data

def selectData(db, cond, type, orgStructureId,begDate,endDate, detailBed, bedIds, includeHoliday):
    if type == 1: data =selectAmbulanceData(db, cond, begDate, endDate)
    elif type ==2: data =selectF62Data(db, cond, begDate, endDate)
    elif type ==3: data =selectTomographyData(db, cond, begDate, endDate)
    elif type ==4: data =selectDHospitalData(db, cond, begDate, endDate, orgStructureId, includeHoliday)
    elif type ==5: data =selectDialisData(db, cond, begDate, endDate)
    elif type ==6: data = selectHospitalData(db, cond, begDate, endDate, orgStructureId, detailBed, bedIds)
    elif type ==7: data = selectEmergencyData(db, cond, begDate, endDate)
    elif type ==8: data =selectPaidData(db, cond, begDate, endDate)
    else:
        data = None
    return data


def selectAmbulanceData(db, cond, begDate, endDate):
    data = []
    condition = dict({'eventType': u"'3%'",
              #   'financeId': 2,
                 'begDate': begDate,
                 'endDate':endDate,
                 'isDoctor': True})
    cond = changeTable(cond, 'Visit', 'Action')
    uetNumberToDoctors = createQueryForStomotology(db, 'Action',cond,
                                            createCondition(condition),
                                            """if(((YEAR(Event.setDate)-YEAR(Client.birthDate)) -
                                            (DATE_FORMAT(Event.setDate, '%m-%d')<RIGHT(Client.birthDate,5))) >17,
                                            rbService.adultUetDoctor*Action.amount, rbService.childUetDoctor*Action.amount)""")
    cond = changeTable(cond, 'Action', 'Visit')

    condition['eventType'] = "'3%'"
    condition['visitType'] = 8
    preventiveVisitNumberToDoctors = createQueryForAmbulance(db, 'Visit',
                                            cond, createCondition(condition),
                                            'COUNT(DISTINCT(Visit.id))')

    condition['visitType'] = 3
    consultVisitNumberToDoctors = createQueryForAmbulance(db, 'Visit',
                                            cond, createCondition(condition),
                                            'COUNT(DISTINCT(Visit.id))')
    condition['eventType'] = "'31%'"
    condition['visitType'] = 1
    dispanserisationVisitNumberToDoctors = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')

    condition['eventType'] = "'32%'"
    condition['visitType'] = 1
    complexVisitNumberToDoctors = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')

    condition['eventType'] = '305'
    condition['visitType'] = 1
    emergencyVisitNumberToDoctors = createQueryForAmbulance(db, 'Visit',
                                            cond, createCondition(condition),
                                            'COUNT(DISTINCT(Visit.id))')
    condition['eventType'] = "'301%'"
    condition['excludeEventType'] = '305'
    condition['visitType'] = 1
    treatmentEventNumberToDoctors = createQueryForAmbulance(db, 'Visit',
                                            cond, createCondition(condition),
                                            'COUNT(DISTINCT(`Event`.`id`))', '', None, 'Event')
    treatmentVisitNumberToDoctors = createQueryForAmbulance(db, 'Visit',
                                            cond, createCondition(condition),
                                            'COUNT(DISTINCT(Visit.id))')
    condition['eventType'] = "'30%'"
    del condition['visitType']
    del condition['excludeEventType']
    condition['isDoctor'] = False
    cond = changeTable(cond, 'Visit', 'Action')
    uetNumberToStaff = createQueryForStomotology(db, 'Action',cond,
                                            createCondition(condition),
                                            """if(((YEAR(Event.setDate)-YEAR(Client.birthDate)) -
                                            (DATE_FORMAT(Event.setDate, '%m-%d')<RIGHT(Client.birthDate,5))) >17,
                                            rbService.adultUetDoctor*Action.amount, rbService.childUetDoctor*Action.amount)""")
    cond = changeTable(cond, 'Action', 'Visit')
    condition['eventType'] = "'3%'"
    condition['visitType'] = 8
    preventiveVisitNumberToStaff = createQueryForAmbulance(db, 'Visit',
                                            cond, createCondition(condition),
                                            'COUNT(DISTINCT(Visit.id))')
    condition['visitType'] = 3
    consultVisitNumberToStaff = createQueryForAmbulance(db, 'Visit',
                                            cond, createCondition(condition),
                                            'COUNT(DISTINCT(Visit.id))')
    condition['eventType'] = "'31%'"
    condition['visitType'] = 1
    dispanserisationVisitNumberToStaff = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')

    condition['eventType'] = "'32%'"
    condition['visitType'] = 1
    complexVisitNumberToStaff = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')

    condition['eventType'] = '305'
    condition['visitType'] = 1
    emergencyVisitNumberToStaff = createQueryForAmbulance(db, 'Visit',
                                            cond, createCondition(condition),
                                            'COUNT(DISTINCT(Visit.id))')
    condition['eventType'] = "'301%'"
    condition['visitType'] = 1
    treatmentEventNumberToStaff = createQueryForAmbulance(db, 'Visit',
                                            cond, createCondition(condition),
                                            'COUNT(DISTINCT(`Event`.`id`))','', None, 'Event')
    treatmentVisitNumberToStaff = createQueryForAmbulance(db, 'Visit',
                                            cond, createCondition(condition),
                                            'COUNT(DISTINCT(Visit.id))')
    # --- общее
    data.append(uetNumberToStaff + uetNumberToDoctors)
    data.append(consultVisitNumberToStaff + preventiveVisitNumberToStaff+
                consultVisitNumberToDoctors + preventiveVisitNumberToDoctors +
                dispanserisationVisitNumberToStaff + dispanserisationVisitNumberToDoctors + complexVisitNumberToDoctors + complexVisitNumberToStaff)
    data.append(emergencyVisitNumberToDoctors + emergencyVisitNumberToStaff)
    data.append(treatmentEventNumberToStaff + treatmentEventNumberToDoctors)
    data.append(treatmentVisitNumberToStaff + treatmentVisitNumberToDoctors)
    data.append(consultVisitNumberToStaff + preventiveVisitNumberToStaff +
                consultVisitNumberToDoctors + preventiveVisitNumberToDoctors +
                treatmentVisitNumberToStaff + treatmentVisitNumberToDoctors +
                emergencyVisitNumberToStaff + emergencyVisitNumberToDoctors +
                dispanserisationVisitNumberToStaff + dispanserisationVisitNumberToDoctors + complexVisitNumberToDoctors + complexVisitNumberToStaff)
    # ---  к врачам
    data.append(uetNumberToDoctors)
    data.append(consultVisitNumberToDoctors + preventiveVisitNumberToDoctors + dispanserisationVisitNumberToDoctors + complexVisitNumberToDoctors)
    data.append(emergencyVisitNumberToDoctors)
    data.append(treatmentEventNumberToDoctors)
    data.append(treatmentVisitNumberToDoctors)
    data.append(consultVisitNumberToDoctors + preventiveVisitNumberToDoctors +
                treatmentVisitNumberToDoctors + emergencyVisitNumberToDoctors + dispanserisationVisitNumberToDoctors + complexVisitNumberToDoctors)
    # --- к ср. мед персоналу
    data.append(uetNumberToStaff)
    data.append(consultVisitNumberToStaff + preventiveVisitNumberToStaff + dispanserisationVisitNumberToStaff + complexVisitNumberToStaff)
    data.append(emergencyVisitNumberToStaff)
    data.append(treatmentEventNumberToStaff)
    data.append(treatmentVisitNumberToStaff)
    data.append(consultVisitNumberToStaff + preventiveVisitNumberToStaff +
                treatmentVisitNumberToStaff+emergencyVisitNumberToStaff + dispanserisationVisitNumberToStaff + complexVisitNumberToStaff)
    return data


def selectF62Data(db, cond, begDate, endDate):
    data = []

    condition = dict({'eventType': u"'30%'",
                # 'financeId': 2,
                 'begDate': begDate,
                 'endDate':endDate,
                 'visitType':8})
 #   numberUETToDoctors

    preventiveVisitNumber = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')
    del condition['visitType']
    condition['eventType'] = "'32%'"
    complexVisitNumber = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')
    condition['eventType'] = "'34%'"
    otherConditionVisitNumber = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')
    condition['eventType'] = "'3%'"
    condition['scene'] = '3'
    condition['visitType'] = 8
    patronageVisitNumber = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))', u'LEFT JOIN rbScene ON rbScene.id = Visit.scene_id')
    condition['eventType'] = "'3019%'"
    del condition['scene']
    stomatologyPreventiveVisitNumber = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')
    condition['eventType'] = "'31%'"
    condition['visitType'] = 1
   # del condition['visitType']
    dispanserisationVisitNumber = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')
    sumPreventiveVisitNumber = preventiveVisitNumber+complexVisitNumber+otherConditionVisitNumber+patronageVisitNumber + dispanserisationVisitNumber
    condition['eventType'] = "'3%'"
    condition['visitType'] = 3
    consultVisitNumber = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')
    condition['scene'] = '2'
    consultHomeVisitNumber = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))', u'LEFT JOIN rbScene ON rbScene.id = Visit.scene_id')
    condition['eventType'] = "'305'"
    condition['visitType'] = 1
    emergencyHomeVisitNumber = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))',u'LEFT JOIN rbScene ON rbScene.id = Visit.scene_id')
    del condition['scene']
    emergencyVisitNumber = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')
    condition['eventType'] = "'301%'"

    treatmentVisitNumber = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')
    treatmentEventNumber = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(`Event`.`id`))','', None, 'Event')
    visitNumber = treatmentVisitNumber + emergencyVisitNumber + consultVisitNumber + sumPreventiveVisitNumber  - patronageVisitNumber
    condition['eventType'] = "'30%'"
    condition['isDoctor'] = False
    condition['visitType'] = 8
    preventiveVisitNumberToStaff  = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')
    del condition['visitType']
    condition['eventType'] = "'32%'"
    complexVisitNumberToStaff  = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')
    condition['eventType'] = "'34%'"
    otherConditionVisitNumberToStaff  = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')
    condition['eventType'] = "'3%'"
    condition['scene'] = '3'
#    del condition['visitType']
    patronageVisitNumberToStaff  = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))', u'LEFT JOIN rbScene ON rbScene.id = Visit.scene_id')
    del condition['scene']
    condition['eventType'] = "'31%'"
    condition['visitType'] = 1
    dispanserisationVisitNumberToStaff  = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')
    condition['eventType'] = "'30%'"
    condition['excludeEventType'] = '305'
    treatmentVisitNumberToStaff = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')
    condition['eventType'] = "'305'"
    del condition['excludeEventType']
    emergencyVisitNumberToStaff = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')
    condition['eventType'] = "'3%'"
    condition['visitType'] = 3
    consultVisitNumberToStaff = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')
    visitToStaff = (consultVisitNumberToStaff + treatmentVisitNumberToStaff +
                    dispanserisationVisitNumberToStaff + patronageVisitNumberToStaff +
                    otherConditionVisitNumberToStaff + emergencyVisitNumberToStaff+
                    complexVisitNumberToStaff +preventiveVisitNumberToStaff)
    data.append(visitNumber)
    data.append(sumPreventiveVisitNumber - patronageVisitNumber)
    data.append(preventiveVisitNumber - patronageVisitNumber)
    data.append(dispanserisationVisitNumber)
    data.append(complexVisitNumber)
    data.append(patronageVisitNumber)
    data.append(otherConditionVisitNumber)
    data.append(stomatologyPreventiveVisitNumber)
    data.append(consultVisitNumber+emergencyVisitNumber)
    data.append(consultHomeVisitNumber+emergencyHomeVisitNumber)
    data.append(emergencyVisitNumber)
    data.append(emergencyHomeVisitNumber)
    data.append(treatmentVisitNumber)
    data.append(treatmentEventNumber)
    data.append(visitToStaff-patronageVisitNumberToStaff)
    return data


def selectTomographyData(db, cond, begDate, endDate):
    data = []
    cond = changeTable(cond, 'Visit', 'Action')
    condition = dict({ 'begDate': begDate,
                      'endDate':endDate,
                      'actionType': "'ComputerTomographyContrasting%'" #,
                     # 'financeId': 2
                     }
                     )

    computerTomographyContrastingNumber = createQueryForAmbulance(db, 'Action', cond, createCondition(condition), 'SUM(Action.amount)', u'LEFT JOIN `ActionType` ON Action.actionType_id=`ActionType`.id ')
    # del condition['eventType']
    condition['actionType'] = "'ComputerTomography%'"
    condition['attachType'] = 2
    computerTomographyForAttachedNumber = createQueryForAmbulance(db, 'Action', cond, createCondition(condition),
                                                                  'SUM(Action.amount)',
                                                                  u'''LEFT JOIN `ActionType` ON Action.actionType_id=`ActionType`.id
                                                                  LEFT JOIN ClientAttach ON (ClientAttach.client_id = Client.id AND
                                                                  (ClientAttach.endDate IS NULL or ClientAttach.endDate>Event.execDate
                                                                  or ClientAttach.endDate='0000-00-00')
                                                                  AND ClientAttach.deleted =0)
                                                                    LEFT JOIN rbAttachType ON rbAttachType.id = ClientAttach.attachType_id''')
    condition['attachType'] = 1
    computerTomographyForNonAttachedNumber = createQueryForAmbulance(db, 'Action', cond, createCondition(condition), 'SUM(Action.amount)', u'''LEFT JOIN `ActionType` ON Action.actionType_id=`ActionType`.id
                                                                  LEFT JOIN ClientAttach ON (ClientAttach.client_id = Client.id AND
                                                                  (ClientAttach.endDate IS NULL or ClientAttach.endDate>Event.execDate
                                                                  or ClientAttach.endDate='0000-00-00')
                                                                  AND ClientAttach.deleted =0)
                                                                    LEFT JOIN rbAttachType ON rbAttachType.id = ClientAttach.attachType_id''')

    condition['actionType'] = "'MRIContrasting%'"
    del condition['attachType']
    MRIContrastingNumber = createQueryForAmbulance(db, 'Action', cond, createCondition(condition), 'SUM(Action.amount)', u'LEFT JOIN `ActionType` ON Action.actionType_id=`ActionType`.id ')
    condition['actionType'] = "'MRI%'"
    condition['attachType'] = 2
    MRIForAttachedNumber = createQueryForAmbulance(db, 'Action', cond, createCondition(condition), 'SUM(Action.amount)', u'''LEFT JOIN `ActionType` ON Action.actionType_id=`ActionType`.id
                                                                  LEFT JOIN ClientAttach ON (ClientAttach.client_id = Client.id AND
                                                                  (ClientAttach.endDate IS NULL or ClientAttach.endDate>Event.execDate
                                                                  or ClientAttach.endDate='0000-00-00')
                                                                  AND ClientAttach.deleted =0)
                                                                    LEFT JOIN rbAttachType ON rbAttachType.id = ClientAttach.attachType_id''')
    condition['attachType'] = 1
    MRIForNonAttachedNumber = createQueryForAmbulance(db, 'Action', cond, createCondition(condition), 'SUM(Action.amount)', u'''LEFT JOIN `ActionType` ON Action.actionType_id=`ActionType`.id
                                                                  LEFT JOIN ClientAttach ON (ClientAttach.client_id = Client.id AND
                                                                  (ClientAttach.endDate IS NULL or ClientAttach.endDate>Event.execDate
                                                                  or ClientAttach.endDate='0000-00-00')
                                                                  AND ClientAttach.deleted =0)
                                                                    LEFT JOIN rbAttachType ON rbAttachType.id = ClientAttach.attachType_id''')

    data.append(computerTomographyForAttachedNumber + computerTomographyForNonAttachedNumber)
    data.append(computerTomographyContrastingNumber)
    data.append(computerTomographyForAttachedNumber)
    data.append(computerTomographyForNonAttachedNumber)

    data.append(MRIForAttachedNumber + MRIForNonAttachedNumber)
    data.append(MRIContrastingNumber)
    data.append(MRIForAttachedNumber)
    data.append(MRIForNonAttachedNumber)

    return data


def selectDHospitalData(db, cond,begDate,endDate,orgStructureId, includeHoliday):
    data =[]
    #количество коек
    cond = changeTable(cond, 'Visit', 'Action')
    condition = dict({'eventType': "'250%'",
                 'begDate': begDate,
                 'endDate':endDate,
                 'visitType':10 #,
                # 'financeId': 2
                }
                     )
    data.append(forceInt(createQueryForHospitalStructure(db, cond,
                                                         createConditionForHospitalStructure( 0, orgStructureId),
                                                         'COUNT(`OrgStructure_HospitalBed`.`id` )')))
    data.append(forceInt(createQueryForHospitalStructure(db, cond,
                                                         createConditionForHospitalStructure( 0, orgStructureId),
                                                         'SUM(`OrgStructure_HospitalBed`.relief)')))
    cond = changeTable(cond, 'Action', 'Visit')
    eventNumber = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Event.id))')
    if includeHoliday is True:
        visitNumber = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')
    else:
        visitNumber = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'SUM(if(DAYOFWEEK(Visit.date) != 7 and DAYOFWEEK(Visit.date) != 1, 1, 0))')
    data.append(eventNumber)
    data.append(visitNumber)
    return data


def selectDialisData(db, cond, begDate, endDate):
    data =[]
    condition = dict({'eventType': '281',
                 'begDate': begDate,
                 'endDate': endDate
                 #,
                 #'financeId': 2
                 }
                     )
    hemodialysisNumber = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')
    condition['eventType'] = "'282'"
    peritoneoclysisNumber = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')
    condition['eventType'] = "'283'"
    hemofiltrationNumber = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')

    data.append(hemodialysisNumber)
    data.append(peritoneoclysisNumber)
    data.append(hemofiltrationNumber)
    return data


def selectHospitalData(db, cond,begDate,endDate, orgStructureId, detailBed, bedIds = None):
    data =[]
#    dataBedProfile = []
    #количество коек
    cond = changeTable(cond, 'Visit', 'Action')
    data.append(forceInt(createQueryForHospitalStructure(db, cond,
                                                         createConditionForHospitalStructure( 1, orgStructureId),
                                                         'COUNT(`OrgStructure_HospitalBed`.`id` )')))
    condition = dict({'eventType': '110',
                 'begDate': begDate,
                 'endDate':endDate
                 #,
                 #'financeId': 2
                 }
                     )
    #число пролеченных больных
    patientNumber = createQueryForAmbulance( db, 'Action', cond,
                                                 createCondition(condition),
                                                 'COUNT(DISTINCT `event_id`)', '', None, 'Event')
    condition['isHospital'] = True
    daysAmount=createQueryForAmbulance(db, u'Action', cond,
                                                 createCondition(condition),
                                                 'Sum(`Action`.`amount`)',
                                                 u'LEFT JOIN `ActionType` ON Action.actionType_id=`ActionType`.id ', None, 'Event')
    data.append(patientNumber)
    data.append(0)
    data.append(0)
    data.append(daysAmount)
    data.append(0)
    data.append(0)
    newYearBegDate = forceDate(datetime.datetime.strptime(str(begDate.year())+'0101',"%Y%m%d"))
    condition['begDate'] = newYearBegDate
    del  condition['isHospital']
    condition['isDead'] = True
    numberDead = createQueryForAmbulance(db, u'Action', cond,
                                                  createCondition(condition),
                                                  'COUNT(DISTINCT `event_id`)',
                                                  u'LEFT JOIN rbResult ON Event.result_id = rbResult.id', None, 'Event')
    del condition['isDead']
    numberOut = createQueryForAmbulance(db, u'Action', cond,
                                                  createCondition( condition),
                                                  'COUNT(DISTINCT `event_id`)',  '', None, 'Event')
    del condition['endDate']
    numberCame= createQueryForAmbulance(db, u'Action',cond,
                                                  createCondition(condition),
                                                  'COUNT(DISTINCT `event_id`)',  '', None, 'Event')

    data.append((numberDead + numberCame + numberOut)/2)

    return data


def selectEmergencyData(db, cond, begDate, endDate):
    data = []
    cond = changeTable(cond, 'Visit', 'Action')
    condition = dict({'actionType': "'emergency'",
                 'begDate': begDate,
                 'endDate': endDate})
    data.append(createQueryForAmbulance( db, u'Action', cond,
                                                 createCondition(condition),'COUNT(DISTINCT Action.id)', u'LEFT JOIN rbResult ON Event.result_id = rbResult.id LEFT JOIN `ActionType` ON Action.actionType_id=`ActionType`.id'))
    condition['isFalseEmergencyCall'] = True
    data.append(createQueryForAmbulance( db, u'Action', cond,
                                                 createCondition(condition),'COUNT(DISTINCT Action.id)', u'LEFT JOIN rbResult ON Event.result_id = rbResult.id LEFT JOIN `ActionType` ON Action.actionType_id=`ActionType`.id'))

    del condition['isFalseEmergencyCall']
    condition['isTrueEmergencyCall'] = True
    condition['financeId'] = 2
    data.append(createQueryForAmbulance(db,u'Action', cond,
                                                 createCondition(condition),
                                                 'COUNT(DISTINCT(Action.id))', u'LEFT JOIN rbResult ON Event.result_id = rbResult.id LEFT JOIN `ActionType` ON Action.actionType_id=`ActionType`.id'))
    return data


def selectPaidData(db, cond, begDate, endDate):
    data = []
    condition = dict({'eventType': "'3%'",
                 'begDate': begDate,
                 'endDate':endDate,
                 'financeId': 3})
    visitsOFDMSInAmbulatory = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')
    condition['financeId'] = 4
    visitsOFPayInAmbulatory = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')
  #  condition['isArmy'] = True
    condition['financeId'] = 5
    visitsOF911InAmbulatory = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')
  #  del condition['isArmy']

    data.append(visitsOFDMSInAmbulatory + visitsOFPayInAmbulatory + visitsOF911InAmbulatory)
    data.append(visitsOFDMSInAmbulatory)
    data.append(visitsOFPayInAmbulatory)
    data.append(visitsOF911InAmbulatory)

    condition['eventType'] = "'3019%'"
    condition['financeId'] = 3
    visitsOFDMSInStomatAmbulatory = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')
    condition['financeId'] = 4
    visitsOFPayInStomatAmbulatory = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')
  #  condition['isArmy'] = True
    condition['financeId'] = 5
    visitsOF911InStomatAmbulatory = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')
  #  del condition['isArmy']

    data.append(visitsOFDMSInStomatAmbulatory + visitsOFPayInStomatAmbulatory + visitsOF911InStomatAmbulatory)
    data.append(visitsOFDMSInStomatAmbulatory)
    data.append(visitsOFPayInStomatAmbulatory)
    data.append(visitsOF911InStomatAmbulatory)

    condition['eventType'] = "'3%'"
    condition['financeId'] = 3
    condition['flatCode'] = 'diagnost'
    eventsOFDMSInAmbulatory = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Event.id))', u'LEFT JOIN `Action` ON Action.event_id = Event.id LEFT JOIN `ActionType` ON Action.actionType_id=`ActionType`.id')
    condition['financeId'] = 4
    eventsOFPayInAmbulatory = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Event.id))', u'LEFT JOIN `Action` ON Action.event_id = Event.id LEFT JOIN `ActionType` ON Action.actionType_id=`ActionType`.id')
 #   condition['isArmy'] = True
    condition['financeId'] = 5
    eventsOF911InAmbulatory = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Event.id))', u'LEFT JOIN `Action` ON Action.event_id = Event.id LEFT JOIN `ActionType` ON Action.actionType_id=`ActionType`.id')
  #  del condition['isArmy']
    del condition['flatCode']
    data.append(eventsOFDMSInAmbulatory + eventsOFPayInAmbulatory + eventsOF911InAmbulatory)
    data.append(eventsOFDMSInAmbulatory)
    data.append(eventsOFPayInAmbulatory)
    data.append(eventsOF911InAmbulatory)

    condition['eventType'] = "'2%'"
    condition['financeId'] = 3
    condition['visitType'] = 10

    eventsOFDMSInDS = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Event.id))')
    condition['financeId'] = 4
    eventsOFPayInDS = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Event.id))')
 #   condition['isArmy'] = True
    condition['financeId'] = 5
    eventsOF911InDS = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Event.id))')
  #  del condition['isArmy']

    data.append(eventsOFDMSInDS + eventsOFPayInDS + eventsOF911InDS)
    data.append(eventsOFDMSInDS)
    data.append(eventsOFPayInDS)
    data.append(eventsOF911InDS)

    condition['financeId'] = 3
    visitsOFDMSInDS = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')
    condition['financeId'] = 4
    visitsOFPayInDS = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')
  #  condition['isArmy'] = True
    condition['financeId'] = 5
    visitsOF911InDS = createQueryForAmbulance(db, 'Visit', cond, createCondition(condition), 'COUNT(DISTINCT(Visit.id))')
 #   del condition['isArmy']

    data.append(visitsOFDMSInDS + visitsOFPayInDS + visitsOF911InDS)
    data.append(visitsOFDMSInDS)
    data.append(visitsOFPayInDS)
    data.append(visitsOF911InDS)

    condition['eventType'] = "'1%'"
    condition['financeId'] = 3

    del condition['visitType']
    cond = changeTable(cond, 'Visit', 'Action')
    eventsOFDMSInHospital = createQueryForAmbulance(db, 'Action', cond, createCondition(condition), 'COUNT(DISTINCT(Event.id))')
    condition['financeId'] = 4
    eventsOFPayInHospital = createQueryForAmbulance(db, 'Action', cond, createCondition(condition), 'COUNT(DISTINCT(Event.id))')
 #   condition['isArmy'] = True
    condition['financeId'] = 5
    eventsOF911InHospital = createQueryForAmbulance(db, 'Action', cond, createCondition(condition), 'COUNT(DISTINCT(Event.id))')

    data.append(eventsOFDMSInHospital + eventsOFPayInHospital + eventsOF911InHospital)
    data.append(eventsOFDMSInHospital)
    data.append(eventsOFPayInHospital)
    data.append(eventsOF911InHospital)

 #   del condition['isArmy']
    condition['financeId'] = 3
    condition['isHospital'] = True
    actionsAmountOFDMSInHospital = createQueryForAmbulance(db, 'Action', cond, createCondition(condition), 'Sum(`Action`.`amount`)',
                                                 u'LEFT JOIN `ActionType` ON Action.actionType_id=`ActionType`.id ')
    condition['financeId'] = 4
    actionsAmountOFPayInHospital = createQueryForAmbulance(db, 'Action', cond, createCondition(condition), 'Sum(`Action`.`amount`)',
                                                 u'LEFT JOIN `ActionType` ON Action.actionType_id=`ActionType`.id ')
 #   condition['isArmy'] = True
    condition['financeId'] = 5
    actionsAmountOF911InHospital = createQueryForAmbulance(db, 'Action', cond, createCondition(condition), 'Sum(`Action`.`amount`)',
                                                 u'LEFT JOIN `ActionType` ON Action.actionType_id=`ActionType`.id ')

    data.append(actionsAmountOFDMSInHospital + actionsAmountOFPayInHospital + actionsAmountOF911InHospital)
    data.append(actionsAmountOFDMSInHospital)
    data.append(actionsAmountOFPayInHospital)
    data.append(actionsAmountOF911InHospital)

 #   del condition['isArmy']

    return data


def createCondition(cond):
    condition = []
    if cond.has_key('eventType'):
        condition.append(u"EventType.regionalCode LIKE %s"%cond['eventType'])
    if cond.has_key('excludeEventType'):
        condition.append(u'EventType.regionalCode NOT LIKE %s'%cond['excludeEventType'])
    if cond.has_key('financeId'):
        condition.append(u"rbFinance.code =%s"%cond['financeId'])
    if cond.has_key('isArmy'):
        if cond['isArmy'] == True:
            condition.append(u'''Event.Client_id IN ( SELECT  `client_id`  FROM  `ClientSocStatus`
                                LEFT JOIN rbSocStatusType ON ClientSocStatus.socStatusType_id = rbSocStatusType.id
                                WHERE  `rbSocStatusType`.`code` =  "с09" OR rbSocStatusType.`code`=  "008")''')
    if cond.has_key('isDoctor'):
        condition.append(u'rbPost.code RLIKE "^1|^2|^3"') if cond['isDoctor'] == True else condition.append(u'rbPost.code NOT RLIKE "^1|^2|^3"')
    if cond.has_key('visitType'):
        condition.append(u'(rbVisitType.regionalCode =%s)'%(cond['visitType']))
    if cond.has_key('isHospital'):
        condition.append(u"`ActionType`.`flatCode` =  'moving'")
    if cond.has_key('isDead'):
        condition.append(u"Event.`client_id` IN(SELECT DISTINCT Event.`client_id` FROM Event LEFT JOIN EventType ON Event.`eventType_id` = EventType.id WHERE EventType.code ='15')")
        condition.append(u"rbResult.code = 106 or rbResult.code = 105")
#    if cond.has_key('begDate') and cond.has_key('visitType') and  cond['visitType'] != 10:
#        condition.append(u"Visit.`date`>='%s'"%(cond['begDate']).toString(Qt.ISODate))
    if cond.has_key('begDate'):
        condition.append(u"Event.`execDate`>='%s'"%(cond['begDate']).toString(Qt.ISODate))
    if cond.has_key('endDate'):
        condition.append(u"Event.`execDate`<'%s'"%(cond['endDate']).toString(Qt.ISODate))
    if cond.has_key('detailBed'):
        condition.append(u"ActionPropertyType.typeName = 'HospitalBed'")
    if cond.has_key('scene'):
        condition.append(u'(rbScene.code =%s)'%(cond['scene']))
    if cond.has_key('attachType'):
        if cond['attachType'] == 1:
            condition.append(u'(rbAttachType.code !=2)' + 'or ClientAttach.id is NULL')
        else:
            condition.append(u'(rbAttachType.code =%s)'%(cond['attachType']))
    if cond.has_key('actionType'):
        condition.append(u'(ActionType.flatCode LIKE %s)'%(cond['actionType']))
    if cond.has_key('isFalseEmergencyCall'):
        condition.append(u"rbResult.code = '11'")
    if cond.has_key('isTrueEmergencyCall'):
        condition.append(u"rbResult.code != '11'")
    return condition


def createConditionForHospitalStructure(structureType,orgStructureId=None):
    condition =[]
    condition.append(u'OrgStructure.type =%s'%structureType)
    #===========================================================================
    # if not orgStructureId:orgStructureId = QtGui.qApp.currentOrgId()
    # condition.append(u'OrgStructure.id=%s'%orgStructureId)
    #===========================================================================
    return condition


def createQueryForAmbulance(db, table,  cond, condition, selection, additional='',  detailBed = None, tableEvent = None):
    if table == 'Visit':
        additional += ' LEFT JOIN rbVisitType ON rbVisitType.id = Visit.visitType_id'
    s = 'GROUP BY rbHospitalBedProfile.name ORDER BY rbHospitalBedProfile.name' if detailBed else ''
    stmt = """SELECT %s as Number
                FROM %s
                LEFT JOIN Event ON %s.Event_id=Event.id
                -- LEFT JOIN Person ON `Event`.`execPerson_id` = Person.id
                LEFT JOIN Person ON %s = Person.id
                LEFT JOIN rbPost ON Person.post_id=rbPost.id
                LEFT JOIN EventType ON Event.`eventType_id` = EventType.id
                LEFT JOIN rbMedicalAidKind ON EventType.medicalAidKind_id = rbMedicalAidKind.id
                LEFT JOIN rbMedicalAidType ON EventType.medicalAidType_id = rbMedicalAidType.id
                LEFT JOIN rbFinance ON %s.finance_id = rbFinance.id
                LEFT JOIN rbEventProfile ON EventType.eventProfile_id = rbEventProfile.id
                LEFT JOIN rbEventTypePurpose ON EventType.purpose_id = rbEventTypePurpose.id
                LEFT JOIN Client ON Event.client_id = Client.id
                %s
                WHERE %s AND %s
                AND %s.deleted=0
                AND Event.deleted=0 %s
                """%(selection,
                     table,
                     table,
                     "`Event`.`execPerson_id`" if tableEvent else '%s.person_id'%table,
                     table,
                     additional,
                     db.joinAnd(condition),
                     db.joinAnd(cond),
                     table,
                     s)
    query = db.query(stmt)
#    print(stmt)
    if detailBed:
        result = []
        resultId = []
        while query.next():
                result.append(forceString(query.record().value('Number'))),
                resultId.append(forceInt(query.record().value('id')))
        return result, resultId
    else:
        if query.next():
            return forceInt(query.record().value('Number'))
        else:
            return None


def changeTable(mass, tableBefore, tableAfter):
    s=''
    for x in mass:
        if string.find(x, tableBefore) >=0:
            mass.append(string.replace(x, tableBefore, tableAfter))
            mass.remove(x)
    return mass


def createQueryForStomotology(db,table, cond, condition, selection):
    stmt = """ SELECT SUM(%s) as Number
                FROM %s
                LEFT JOIN Event ON Action.Event_id=Event.id
                LEFT JOIN Person ON `Event`.`setPerson_id` = Person.id
                LEFT JOIN rbPost ON Person.post_id=rbPost.id
                LEFT JOIN rbVisitType ON Event.`eventType_id` = rbVisitType.id
                LEFT JOIN EventType ON Event.`eventType_id` = EventType.id
                LEFT JOIN rbMedicalAidKind ON EventType.medicalAidKind_id = rbMedicalAidKind.id
                LEFT JOIN rbMedicalAidType ON EventType.medicalAidType_id = rbMedicalAidType.id
                LEFT JOIN rbEventProfile ON EventType.eventProfile_id = rbEventProfile.id
         --       LEFT JOIN Visit ON  Visit.Event_id=Event.id
                LEFT JOIN rbFinance ON Action.finance_id = rbFinance.id
                LEFT JOIN Client ON Event.client_id = Client.id
                LEFT JOIN ActionType ON ActionType.id = Action.actionType_id
                LEFT JOIN rbService ON ActionType.nomenclativeService_id = rbService.id
                WHERE %s AND %s
                AND Action.deleted=0
                AND Event.deleted=0
                """ % (selection, table, db.joinAnd(condition), db.joinAnd(cond))
    query = db.query(stmt)
    if query.next():
        return forceDouble(query.record().value('Number'))
    else:
        return 0


def createQueryForHospitalStructure(db,cond,condition,selection):
    stmt ="""
    SELECT %s as Number
    FROM  `OrgStructure_HospitalBed`
    LEFT JOIN  `OrgStructure` ON  `OrgStructure_HospitalBed`.`master_id` =  `OrgStructure`.id
    WHERE  `isPermanent` =1
    AND  %s AND `OrgStructure`.hasHospitalBeds =1
    """%(selection,db.joinAnd(condition))
    query = db.query(stmt)
    if query.next():
        return query.record().value('Number')
    else:
        return None

def createQueryForHospital(db,cond,condition,selection):
    stmt ="""SELECT %s as Number
    LEFT JOIN Event ON Action.Event_id=Event.id
    LEFT JOIN Person ON `Event`.`setPerson_id` = Person.id
    LEFT JOIN rbPost ON Person.post_id=rbPost.id
    LEFT JOIN EventType ON Event.`eventType_id` = EventType.id
    LEFT JOIN rbMedicalAidKind ON EventType.medicalAidKind_id = rbMedicalAidKind.id
    LEFT JOIN rbMedicalAidType ON EventType.medicalAidType_id = rbMedicalAidType.id
    LEFT JOIN rbFinance ON EventType.finance_id = rbFinance.id
    LEFT JOIN rbEventProfile ON EventType.eventProfile_id = rbEventProfile.id
    WHERE %s
            AND %s
            AND Action.deleted=0
            AND Event.deleted=0
    """%(selection,db.joinAnd(cond),db.joinAnd(condition))
    query = db.query(stmt)
    print(stmt)
    if query.next():
        return query.record().value('Number')
    else:
        return None


class CReportPGG2014(CReport):
    def __init__(self, parent, type):
        CReport.__init__(self, parent)
        self.type = type
        self.setTitle(u'Форма ПГГ')


    def getSetupDialog(self, parent):
        result = CReportPGGSetupDialog(parent)
        result.setTitle(self.title())
        if self.type == 8 or self.type == 7:
            result.lblFinanceType.setVisible(False)
            result.cmbFinanceId.setVisible(False)
        return result


    def build(self, params):
        begDate = params.get('begDate', QDate())
        endDate = params.get('endDate', QDate())
        detailBed= params.get('detailBed', False)
        eventPurposeId = params.get('eventPurposeId', None)
        eventTypeId = params.get('eventTypeId', None)
        orgStructureId = params.get('orgStructureId', None)
        personId = params.get('personId', None)
        sex = params.get('sex', 0)
        ageFrom = params.get('ageFrom', 0)
        ageTo = params.get('ageTo', 150)
        specialityId = params.get('specialityId', None)
        includeHoliday = params.get('includeHoliday', None)
        financeId = params.get('financeId', None)
        if detailBed:
            data, dataBedprofile= calculateData(begDate, endDate.addDays(1), eventPurposeId, eventTypeId, orgStructureId, personId, sex, ageFrom, ageTo, self.type, detailBed, specialityId, financeId, includeHoliday)
        else:
            data = calculateData(begDate, endDate.addDays(1), eventPurposeId, eventTypeId, orgStructureId, personId, sex, ageFrom, ageTo, self.type, detailBed, specialityId, financeId, includeHoliday)
            dataBedprofile =[]
        # now text
        doc = QtGui.QTextDocument()
        cursor = QtGui.QTextCursor(doc)
        cursor.setCharFormat(CReportBase.ReportTitle)
        cursor.insertText(self.title())
        cursor.insertBlock()
        self.dumpParams(cursor, params)
        cursor.insertBlock()
        tableColumns = self.drawTable(dataBedprofile)
        table = createTable(cursor, tableColumns)

        self.mergeCells(table)
        i=3
#        if type
        for datum in data:
            table.setText(i, 5, datum)
            i=i+1
        return doc


    def mergeCells(self, table):
        #заголовок
        table.mergeCells(0, 0, 1, 7)
        table.mergeCells(1, 2, 1, 2)
        table.mergeCells(2, 2, 1, 2)
        if self.type ==1: self.mergeCellsForAmbulance(table)
        elif self.type ==2: self.mergeCellsFor62(table)
        elif self.type ==3: self.mergeCellsForTomography(table),
        elif self.type ==4: self.mergeCellsForDHospital(table),
        elif self.type ==5: self.mergeCellsForDialysis(table)
        elif self.type ==6: self.mergeCellsForHospital(table)
        elif self.type ==7: self.mergeCellsForEmergency(table)
        elif self.type ==8: self.mergeCellsForPayers(table)

    def mergeCellsForAmbulance(self, table):
        table.mergeCells(3, 0, 6, 1)
        table.mergeCells(9, 0, 6, 1)
        table.mergeCells(15, 0, 6, 1)

        table.mergeCells(3, 2, 6, 1)
        table.mergeCells(9, 2, 6, 1)
        table.mergeCells(15, 2, 6, 1)

    def mergeCellsFor62(self, table):
        table.mergeCells(3, 0, 15, 1)
        table.mergeCells(5, 2, 5, 1)

        table.mergeCells(3, 2, 1, 2)
        table.mergeCells(4, 2, 1, 2)
        table.mergeCells(10, 2, 1, 2)
        table.mergeCells(11, 2, 1, 2)
        table.mergeCells(12, 2, 1, 2)
        table.mergeCells(13, 2, 1, 2)
        table.mergeCells(14, 2, 1, 2)
        table.mergeCells(15, 2, 1, 2)
        table.mergeCells(16, 2, 1, 2)
        table.mergeCells(17, 2, 1, 2)

    def mergeCellsForDHospital(self, table):
        table.mergeCells(3, 0, 2, 1)

        table.mergeCells(3, 2, 1, 2)
        table.mergeCells(4, 2, 1, 2)

    def mergeCellsForDialysis(self, table):
        table.mergeCells(3, 0, 3, 1)
        table.mergeCells(3, 2, 1, 2)
        table.mergeCells(4, 2, 1, 2)
        table.mergeCells(5, 2, 1, 2)


    def mergeCellsForHospital(self, table):
        table.mergeCells(4, 0, 3, 1)
        table.mergeCells(7, 0, 3, 1)
        table.mergeCells(4, 2, 3, 1)
        table.mergeCells(7, 2, 3, 1)
        table.mergeCells(10, 2, 1, 2)

    def mergeCellsForEmergency(self, table):
        table.mergeCells(3, 0, 3, 1)
        table.mergeCells(3, 2, 3, 1)

    def mergeCellsForTomography(self, table):
        table.mergeCells(3, 0, 4, 1)
        table.mergeCells(7, 0, 4, 1)
        table.mergeCells(3, 2, 4, 1)
        table.mergeCells(7, 2, 4, 1)
        table.mergeCells(3, 3, 4, 1)
        table.mergeCells(7, 3, 4, 1)
        table.mergeCells(2, 2, 1, 3)
        table.mergeCells(1, 2, 1, 3)

    def mergeCellsForPayers(self, table):
        table.mergeCells(3, 0, 12, 1)
        table.mergeCells(15, 0, 8, 1)
        table.mergeCells(23, 0, 8, 1)
        table.mergeCells(3, 2, 12, 1)
        table.mergeCells(15, 2, 8, 1)
        table.mergeCells(23, 2, 8, 1)
        table.mergeCells(27, 3, 8, 1)
        table.mergeCells(3, 3, 4, 1)
        table.mergeCells(11, 3, 4, 1)
        table.mergeCells(19, 3, 4, 1)
        table.mergeCells(23, 3, 4, 1)
        table.mergeCells(7, 3, 4, 1)
        table.mergeCells(15, 3, 4, 1)
        table.mergeCells(23, 3, 4, 1)
        table.mergeCells(3, 2, 4, 1)
        table.mergeCells(2, 2, 1, 3)
        table.mergeCells(1, 2, 1, 3)

    def drawTable(self, dataBedprofile):
        tokens = [
                 self.drawTableForAmbulance(),
                 self.drawTableFor62(),
                 self.drawTableForTomography(),
                 self.drawTableForDHospital(),
                 self.drawTableForDialysis(),
                 self.drawTableForHospital(dataBedprofile),
                 self.drawTableForEmergency(),
                 self.drawTableForPayers()
                 ]
        tableColumns = tokens[self.type-1]
        return tableColumns

    def drawTableForAmbulance(self):
        tableColumns = [
                ( '5%', [u'1. Амбулаторно-поликлиническая помощь',u'',u'1',
                         u'1',u'',u'',u'',u'',u'',
                         u'2',u'',u'',u'',u'',u'',
                         u'3',u'',u'',u'',u'',u'',], CReportBase.AlignLeft),
                ( '5%', [u'',u'№',u'2',
                         u'1.1',u'1.2',u'1.3',u'1.4',u'1.5',u'1.6',
                         u'2.1',u'2.2',u'2.3',u'2.4',u'2.5',u'2.6',
                         u'3.1',u'3.2',u'3.3',u'3.4',u'3.5',u'3.6'], CReportBase.AlignLeft),
                ( '20%', [u'', u'Наименование показателя',u'',
                          u'Всего число посещений и обращений к врачам и среднему медперсоналу, ведущему самостоятельный прием(включая стоматологическую помощь)',u'',u'',u'',u'',u'',
                          u'Всего число посещений и обращений к врачам(включая стоматологическую помощь)',u'',u'',u'',u'',u'',
                          u'Всего число посещений и обращений к среднему медперсоналу, ведущему самостоятельный прием(включая стоматологическую помощь)',u'',u'',u'',u'',u''], CReportBase.AlignLeft),
                ( '30%', [u'', u'',u'3',
                          u'количество УЕТ', u'число посещений с профилактической и иными целями', u'число посещений по медицинской помощи в неотложной форме',
                          u'обращений с диагностической и/или лечебной целью в связи с заболеваниями из них',
                          u'Посещений, включенных в обращениях в связи с заболеваниями',u'всего посещений',
                          u'Количество УЕТ к врачам ', u'число посещений с профилактической и иными целями к врачам ', u'число посещений по медицинской помощи в неотложной форме к врачам',
                          u'обращений с диагностической и/или лечебной целью в связи с заболеваниями из них к врачам ',
                          u'посещений, включенных в обращениях в связи с заболеваниями к врачам ',u'всего посещений к врачам ',
                          u'количество УЕТ к среднему медперсоналу, ведущему самостоятельный прием',
                          u'число посещений с профилактической и иными целями к среднему медперсоналу, ведущему самостоятельный прием',
                          u'число посещений по медицинской помощи в неотложной форме к среднему медперсоналу, ведущему самостоятельный прием',
                          u'обращений с диагностической и/или лечебной целью в связи с заболеваниями из них к среднему медперсоналу, ведущему самостоятельный прием',
                          u'посещений, включенных в обращениях в связи с заболеваниями к среднему медперсоналу, ведущему самостоятельный прием',u'всего посещений к среднему медперсоналу, ведущему самостоятельный прием'
                          ], CReportBase.AlignLeft),
                ( '10%', [u'',u'План',u'4'], CReportBase.AlignRight),
                ( '10%', [u'',u'Факт',u'5'], CReportBase.AlignRight),
                ]
        return tableColumns

    def drawTableFor62(self):
        tableColumns = [
            ( '5%', [u'2. Форма №62', u'', u'1',
                     u'1', u'', u'', u'', u'', u'', u'', u'', u'', u'', u'', u'', u'', u'', u''
                     ], CReportBase.AlignLeft),
            ( '5%', [u'', u'№',u'2',
                     u'01', u'02', u'03', u'04', u'05', u'06',u'07',u'08',u'09',u'10',u'11',u'12',u'13',u'14',u'15'
                     ], CReportBase.AlignLeft),
            ( '20%', [u'', u'Наименование показателя',u'',
                      u'Посещений  - всего, в том числе',
                      u'Посещений с профилактической целью всего', u'в том числе', u'', u'', u'',u'',
                      u'Из стр.02: посещения с профилактической целью к стоматологам',
                      u'Разовые посещения по поводу заболеваний - всего', u'из них: на дому',
                      u'Из стр.  09 - при оказании медицинской помощи в неотложной форме', u'из них: на дому',
                      u'Посещений, включенных  в обращения в связи с заболеваниями',
                      u'Обращений по заболеванию',
                      u'Из стр. 01 - посещений медицинских работников, имеющих среднее медициснкое образование, ведущих самостоятельный прием'
                      ], CReportBase.AlignLeft),
            ( '30%', [u'', u'', u'3',
                      u'',u'',
                      u'медицинские осмотры',
                      u'диспансеризация',
                      u'комплексный медицинский осмотр',
                      u'патронаж',
                      u'другие обстоятельства',
                      u'',u'',u'',u'',u'',u'',u'',''], CReportBase.AlignLeft),
            ( '10%', [u'',u'План',u'4'], CReportBase.AlignRight),
            ( '10%', [u'',u'Факт',u'5'], CReportBase.AlignRight)
                   ]
        return tableColumns

    def drawTableForTomography(self):
        tableColumns = [
            ( '5%', [u'3. Проведение КТ, МРТ амбулаторным пациентам',u'',u'1',
                     u'1',u'',u'',u'',
                     u'2',u'',u'', u''
                     ], CReportBase.AlignLeft),
            ( '5%', [u'', u'№',u'2',
                     u'1.1',u'1.1.1',u'1.2',u'1.3',
                     u'2.1',u'2.1.1',u'2.2',u'2.3'
                     ], CReportBase.AlignLeft),
            ( '5%', [u'', u'Наименование показателя',u'',
                      u'КТ',u'',u'',u'',
                      u'МРТ',u'',u'',u''
                      ], CReportBase.AlignLeft),
            ( '15%', [u'', u'',u'3',
                      u'Количество исследований',u'',u'',u'',
                      u'Количество исследований',u'',u'',u''
                      ], CReportBase.AlignLeft),
            ( '30%', [u'', u'',u'',
                    u'ВСЕГО из них:', u'из них: с констрастированием',u'прикрепленному населению',u'неприкрепленному населению',
                     u'ВСЕГО из них:',u'из них: с констрастированием',u'прикрепленному населению',u'неприкрепленному населению'
                      ], CReportBase.AlignLeft),
            ( '10%', [u'',u'План',u'4'], CReportBase.AlignRight),
            ( '10%', [u'',u'Факт',u'5'], CReportBase.AlignRight)
                   ]
        return tableColumns

    def drawTableForDHospital(self):
        tableColumns = [
            ( '5%', [u'4. Деятельность дневного стационара',u'',u'1',
                     u'1',u'',
                     u'2',
                     u'3'
                     ], CReportBase.AlignLeft),
            ( '5%', [u'', u'№',u'2',
                     u'1.1',u'1.2',
                     u'2.1',
                     u'3.1'
                     ], CReportBase.AlignLeft),
            ( '20%', [u'', u'Наименование показателя',u'',
                      u'Количество коек',u'Количество мест(с учетом сменности)',
                      u'Число пролеченных больных(чел.)',
                      u'Проведено больными дней лечения'
                      ], CReportBase.AlignLeft),
            ( '30%', [u'', u'',u'3',
                      u'',u'',
                      u'всего',
                      u'всего'
                      ], CReportBase.AlignLeft),
            ( '10%', [u'',u'План',u'4'], CReportBase.AlignRight),
            ( '10%', [u'',u'Факт',u'5'], CReportBase.AlignRight)
                   ]
        return tableColumns

    def drawTableForDialysis(self):
        tableColumns = [
            ( '5%', [u'5. Диализная помощь',u'',u'1',
                     u'1',u'',
                     ], CReportBase.AlignLeft),
            ( '5%', [u'', u'№',u'2',
                     u'1.1',u'1.2',u'1.3',
                     ], CReportBase.AlignLeft),
            ( '20%', [u'', u'Наименование показателя',u'',
                      u'Гемодиализ (сеансы)',
                      u'Перитонеальный диализ(сеансы)',
                      u'Продленная гемофильтрация(сеансы)'
                      ], CReportBase.AlignLeft),
            ( '30%', [u'', u'',u'3',
                      u'',
                      u'',
                      u''
                      ], CReportBase.AlignLeft),
            ( '10%', [u'',u'План',u'4'], CReportBase.AlignRight),
            ( '10%', [u'',u'Факт',u'5'], CReportBase.AlignRight)
                   ]
        return tableColumns

    def drawTableForHospital(self, dataBedProfile):
        tableColumns = [
            ( '5%', [u'6. Деятельность стационара',u'',u'1',
                     u'1',
                     u'2',u'',u'',
                     u'3',u'',u'',
                     u'4'
                     ], CReportBase.AlignLeft),
            ( '5%', [u'', u'№',u'2',
                     u'1',
                     u'2.1',u'2.2',u'2.3',
                     u'3.1',u'3.2',u'3.3',
                     u'4'
                     ], CReportBase.AlignLeft),
            ( '20%', [u'', u'Наименование показателя',u'',
                      u'Количество коек',
                      u'Число законченных случаев госпитализации(чел.)',u'',u'',
                      u'Проведено больными койко-дней', u'', u'',
                      u'Число пользованных больных',
                      ], CReportBase.AlignLeft),
            ( '30%', [u'', u'',u'3',
                      u'',
                      u'всего, в том числе',u'с использованием ВМП, финансируемой за счет средств ОМС',
                      u'по медицинской реабилитации(только ГБУЗ АО "1ГКБ им Е.Е. Волосевич")',
                      u'всего, в том числе',u'с использованием ВМП, финансируемой за счет средств ОМС',
                      u'по медицинской реабилитации(только ГБУЗ АО "1ГКБ им Е.Е. Волосевич")',
                      u''], CReportBase.AlignLeft),
            ( '10%', [u'',u'План',u'4'], CReportBase.AlignRight),
            ( '10%', [u'',u'Факт',u'5'], CReportBase.AlignRight)]
        i =6
        for beds in dataBedProfile:
            tableColumns.insert(i,  ( '20%', [u'', u'', beds], CReportBase.AlignRight))
            i=i+1
        return tableColumns



    def drawTableForEmergency(self):
        tableColumns = [
            ( '5%', [u'7. Работа скорой медицинской помощи',u'',u'1',
                     u'1',u'',u'',
                     ], CReportBase.AlignLeft),
            ( '5%', [u'', u'№',u'2',
                     u'1.1',u'1.2',u'1.3',
                     ], CReportBase.AlignLeft),
            ( '20%', [u'', u'Наименование показателя',u'',
                      u'Вызовов',
                      u'',
                      ], CReportBase.AlignLeft),
            ( '30%', [u'', u'',u'3',
                      u'всего ',
                      u'безрезультативных',
                      u'к застрахованным(ОМС (без учета безрезультативных)',
                      ], CReportBase.AlignLeft),
            ( '10%', [u'',u'План',u'4'], CReportBase.AlignRight),
            ( '10%', [u'',u'Факт',u'5'], CReportBase.AlignRight)
                   ]
        return tableColumns

    def drawTableForPayers(self):
        tableColumns = [
                ( '5%', [u'8. Платная медицинская помощь',u'',u'1',
                         u'1',u'',u'',u'',u'',u'',u'',u'',u'',u'',u'',u'',
                         u'2',u'',u'',u'',u'',u'',u'',u'',
                         u'3',u'',u'',u'',u'',u'',u'',u'',], CReportBase.AlignLeft),
                ( '5%', [u'',u'№',u'2',
                         u'1.1',u'1.2',u'1.3',u'1.4',u'1.5',u'1.6',u'1.7',u'1.8',u'1.9',u'1.10',u'1.11',u'1.12',
                         u'2.1',u'2.2',u'2.3',u'2.4',u'2.5',u'2.6',u'2.7',u'2.8',
                         u'3.1',u'3.2',u'3.3',u'3.4',u'3.5',u'3.6',u'3.7',u'3.8'], CReportBase.AlignLeft),
                ( '15%', [u'', u'Наименование показателя',u'',
                          u'Амбулаторно-поликническая помощь(в том числе стоматологическая помощь с учетом ортодонтии)',u'',u'',u'',u'',u'',u'',u'',u'',u'',u'',u'',
                          u'Дневные стационары',u'',u'',u'',u'',u'',u'',u'',
                          u'Круглосуточный стационар',u'',u'',u'',u'',u'',u'',u''], CReportBase.AlignLeft),
                ( '15%', [u'', u'',u'3',
                          u'посещения', u'', u'', u'',
                          u'из них стоматологические посещения(ортодонтов)', u'', u'', u'',
                          u'количество человек, получивших медицинскую услугу', u'', u'', u'',
                          u'число пролеченных пациентов', u'', u'', u'',
                          u'число пациенто-дней', u'', u'', u'',
                          u'число пролеченных пациентов', u'', u'', u'',
                          u'число койко-дней', u'', u'', u''
                          ], CReportBase.AlignLeft),
                ( '20%', [u'', u'',u'',
                          u'ВСЕГО, из них', u'за счет средств ДМС', u'за счет личного хозяйства(личных средств граждан)',
                          u'прочие источники, в том числе по 911 постановлению',
                          u'ВСЕГО, из них', u'за счет средств ДМС', u'за счет личного хозяйства(личных средств граждан)',
                          u'прочие источники, в том числе по 911 постановлению',
                          u'ВСЕГО, из них', u'за счет средств ДМС', u'за счет личного хозяйства(личных средств граждан)',
                          u'прочие источники, в том числе по 911 постановлению',
                          u'ВСЕГО, из них', u'за счет средств ДМС', u'за счет личного хозяйства(личных средств граждан)',
                          u'прочие источники, в том числе по 911 постановлению',
                          u'ВСЕГО, из них', u'за счет средств ДМС', u'за счет личного хозяйства(личных средств граждан)',
                          u'прочие источники, в том числе по 911 постановлению',
                          u'ВСЕГО, из них', u'за счет средств ДМС', u'за счет личного хозяйства(личных средств граждан)',
                          u'прочие источники, в том числе по 911 постановлению',
                          u'ВСЕГО, из них', u'за счет средств ДМС', u'за счет личного хозяйства(личных средств граждан)',
                          u'прочие источники, в том числе по 911 постановлению'
                          ], CReportBase.AlignLeft),
                ( '10%', [u'',u'План',u'4'], CReportBase.AlignRight),
                ( '10%', [u'',u'Факт',u'5'], CReportBase.AlignRight),
                ]
        return tableColumns

from Ui_ReportPGGSetup import Ui_ReportPGGSetupDialog


class CReportPGGSetupDialog(QtGui.QDialog, Ui_ReportPGGSetupDialog):
    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)
        self.cmbEventPurpose.setTable('rbEventTypePurpose', True, filter='code != \'0\'')
        self.cmbEventType.setTable('EventType', True, filter=getWorkEventTypeFilter())
        self.cmbOrgStructure.setOrgId(QtGui.qApp.currentOrgId())
        self.cmbOrgStructure.setValue(QtGui.qApp.currentOrgStructureId())
        self.chbIncludeHoliday.setEnabled(True)
#        if self.type == 8:
#            self.cmbFinanceId.setEnabled(False)


    def setTitle(self, title):
        self.setWindowTitle(title)

    def setParams(self, params):
        self.edtBegDate.setDate(params.get('begDate', QDate.currentDate()))
        self.edtEndDate.setDate(params.get('endDate', QDate.currentDate()))
        self.cmbEventPurpose.setValue(params.get('eventPurposeId', None))
        self.cmbEventType.setValue(params.get('eventTypeId', None))
        self.cmbSpeciality.setTable('rbSpeciality', True)
        self.cmbFinanceId.setTable('rbFinance', True)
        self.cmbOrgStructure.setValue(params.get('orgStructureId', None))
        self.cmbPerson.setValue(params.get('personId', None))
        self.cmbSex.setCurrentIndex(params.get('sex', 0))
        self.edtAgeFrom.setValue(params.get('ageFrom', 0))
        self.edtAgeTo.setValue(params.get('ageTo', 150))
        self.chbIncludeHoliday.setChecked(bool(params.get('onlyFirstTime', False)))


    def params(self):
        result = {}
        result['begDate'] = self.edtBegDate.date()
        result['endDate'] = self.edtEndDate.date()
        result['eventPurposeId'] = self.cmbEventPurpose.value()
        result['eventTypeId'] = self.cmbEventType.value()
        result['orgStructureId'] = self.cmbOrgStructure.value()
        result['personId'] = self.cmbPerson.value()
        result['sex'] = self.cmbSex.currentIndex()
        result['ageFrom'] = self.edtAgeFrom.value()
        result['ageTo'] = self.edtAgeTo.value()
        result['specialityId'] = self.cmbSpeciality.value()
        result['includeHoliday'] = self.chbIncludeHoliday.isChecked()
        result['financeId'] = self.cmbFinanceId.value()
        return result


    @pyqtSignature('QDate')
    def on_edtBegDate_dateChanged(self, date):
        self.cmbPerson.setBegDate(date)


    @pyqtSignature('QDate')
    def on_edtEndDate_dateChanged(self, date):
        self.cmbPerson.setEndDate(date)


    @pyqtSignature('int')
    def on_cmbSpeciality_currentIndexChanged(self, index):
        specialityId = self.cmbSpeciality.value()
        self.cmbPerson.setSpecialityId(specialityId)


    @pyqtSignature('int')
    def on_cmbEventPurpose_currentIndexChanged(self, index):
        eventPurposeId = self.cmbEventPurpose.value()
        if eventPurposeId:
            filter = 'EventType.purpose_id =%d' % eventPurposeId
        else:
            filter = getWorkEventTypeFilter()
        self.cmbEventType.setFilter(filter)


    @pyqtSignature('int')
    def on_cmbOrgStructure_currentIndexChanged(self, index):
        orgStructureId = self.cmbOrgStructure.value()
        self.cmbPerson.setOrgStructureId(orgStructureId)
