# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui
from PyQt4.QtCore import *

from library.Utils      import forceString, forceRef, forceDate, forceDouble, forceInt, formatDate, forceDateTime, formatShortName

from Reports.Report     import createTable, CReportEx, getOrgStructureFullName
from Reports.ReportBase import CReportBase
from Reports.ReportMonthActions import CSetupReportMonthActions
from Reports.ReportSetupDialog  import CReportSetupDialog
from Reports.Utils              import dateRangeAsStr
from Orgs.Utils                 import getOrgStructurePersonIdList

class CStomatDayReport(CReportEx):
    def __init__(self, parent):
        CReportEx.__init__(self, parent)
        self._mapActionTypeId2ServiceIdList = {}
        self.setTitle(u'Листок ежедневного учета работы врача-стоматолога (форма 037/у)', u'Листок ежедневного учета работы врача-стоматолога (форма 037/у)')
        self.table_columns = [
            ('8%', [u'№ п/п', u'1'], CReportBase.AlignRight),
            ('8%', [u'Время приема пациента',  u'2'], CReportBase.AlignLeft),
            ('8%', [u'ФИО пациента', u'3'], CReportBase.AlignLeft),
            ('8%', [u'Год рождения', u'4'], CReportBase.AlignLeft),
            ('8%', [u'Адрес', u'5'], CReportBase.AlignLeft),
            ('8%', [u'Первично принятые', u'6'], CReportBase.AlignRight),
            ('8%', [u'В том числе дети', u'7'], CReportBase.AlignRight),
            ('8%', [u'Диагноз', u'8'], CReportBase.AlignLeft),
            ('12%',[u'Фактически выполненный объем работы', u'9'], CReportBase.AlignLeft),
            ('8%', [u'Санированные', u'10'], CReportBase.AlignRight),
            ('8%', [u'В т.ч. в плановом порядке', u'11'], CReportBase.AlignRight),
            ('8%', [u'УЕТ', u'12'], CReportBase.AlignRight),
            ]

    def getPeriodName(self, params):
        date = params.get('begDate', QDate.currentDate())
        params['endDate'] = date
        return date.toString(u"dd.MM.yyyy")

    def getSetupDialog(self, parent):
        result = CReportSetupDialog(parent)
        result.setTitle(self.title())
        result.setOrgStructureVisible(True)
        result.setPersonVisible(True)
        result.setFinanceVisible(False)
        result.setSpecialityVisible(False)
        result.setInsurerVisible(False)
        result.setStageVisible(False)
        result.setPayPeriodVisible(False)

        # заменяем подходящий комбобокс:
        result.setWorkTypeVisible(True)
        result.lblWorkType.setText(u'Адрес: ')
        result.cmbAddressType = result.cmbWorkType
        result.cmbAddressType.setMaxCount(0) # а как его еще очистить???
        result.cmbAddressType.setMaxCount(2)
        result.cmbAddressType.addItem(u'регистрации')
        result.cmbAddressType.addItem(u'проживания')
        result.params = lambda : dict(CReportSetupDialog.params(result).items() + [('addressType', result.cmbWorkType.currentIndex())])

        result.setOwnershipVisible(False)
        result.setWorkOrganisationVisible(False)
        result.setSexVisible(False)
        result.setAgeVisible(False)
        result.setActionTypeVisible(False)
        result.setMKBFilterVisible(False)
        result.setEventTypeVisible(False)
        result.lblBegDate.setText(u'Дата')
        result.lblEndDate.hide()
        result.edtEndDate.hide()
        result.lblEventType.hide()
        result.cmbEventType.hide()
        result.chkOnlyPermanentAttach.hide()
        result.chkDetailPerson.hide()
        return result

    def key(self, key):
        return key[1]

    def build(self, params):
        doc = QtGui.QTextDocument()
        boldChars = QtGui.QTextCharFormat()
        boldChars.setFontWeight(QtGui.QFont.Bold)
        blockFormatCenter =QtGui.QTextBlockFormat()
        blockFormatCenter.setAlignment(Qt.AlignCenter)
        cursor = QtGui.QTextCursor(doc)
        cursor.setCharFormat(CReportBase.ReportTitle)
        reportName = self.getReportName()
        if 'getPeriodName' in dir(self):
            reportName += u' за ' + self.getPeriodName(params)
        self.addParagraph(cursor, reportName)
        self.dumpParams(cursor, params)
        cursor.insertBlock()
        cursor.movePosition(QtGui.QTextCursor.End)

        data = self.getData(params)
        table = self.createTable(cursor)
        sumRow = [0]*12
        for i, key in enumerate(sorted(data.keys(), key=self.key)):
            dataRow = data[key]
            row = table.addRow()
            for j, val in enumerate(dataRow):
                if j == 11:
                    table.setText(row, j, round(val, 2))
                else:
                    table.setText(row, j, val)
                if j in [5, 6, 9, 10, 11]:
                    sumRow[j] += dataRow[j]
            table.setText(row, 0, i+1)
        tableRow = table.addRow()
        table.setText(tableRow, 0, u'Итого:')
        for col in [5, 6, 9, 10, 11]:
            if col == 11:
                table.setText(tableRow, col, round(sumRow[col],  2))
            else:
                table.setText(tableRow, col, sumRow[col])
        return doc

    def getData(self, params):
        db = QtGui.qApp.db
        data = {}
        date = params.get('begDate', QDate.currentDate())
        orgStructureId = params.get('orgStructureId', None)
        personId = params.get('personId', None)
        addressType = params.get('addressType', 0)
        tableAction = db.table('Action')
        tableEvent = db.table('Event')
        tableAT = db.table('ActionType')
        tableClient = db.table('Client')
        tableET = db.table('EventType')
        tableATS = db.table('ActionType_Service')
        tableCT = db.table('Contract_Tariff')
        tableMKB = db.table('Diagnosis')
        tableVA = db.table('vActionStomOther')

        table = tableAction.leftJoin(tableEvent, tableEvent['id'].eq(tableAction['event_id']))
        table = table.leftJoin(tableET, tableET['id'].eq(tableEvent['eventType_id']))
        table = table.leftJoin(tableClient, tableClient['id'].eq(tableEvent['client_id']))
        table = table.leftJoin(tableAT, tableAT['id'].eq(tableAction['actionType_id']))
        table = table.leftJoin(tableATS, db.joinAnd([tableATS['master_id'].eq(tableAT['id']),
                                                                    db.joinOr([tableATS['finance_id'].isNull(), tableATS['finance_id'].eq(tableAction['finance_id'])])]))
        table = table.leftJoin(tableCT, [tableCT['service_id'].eq(tableATS['service_id']), tableCT['master_id'].eq(tableEvent['contract_id']), 'isSexAndAgeSuitable(Client.sex, Client.birthDate, Contract_Tariff.sex, Contract_Tariff.age, Action.endDate)'])
        table = table.leftJoin(tableMKB, tableMKB['id'].eqEx('getEventDiagnosis(Event.id)'))
        table = table.leftJoin(tableVA, tableVA['action_id'].eq(tableAction['id']))
        cols = [tableClient['id'].alias('client_id'),
            tableEvent['execDate'],
            tableClient['firstName'],
            tableClient['lastName'],
            tableClient['patrName'],
            tableEvent['isPrimary'],
            tableClient['birthDate'],
            tableMKB['MKB'],
            'age(Client.birthDate, Action.endDate) as age',
            tableAT['code'],
            tableEvent['id'].alias('event_id'),
            tableAction['uet'].alias('actionUet'),
            tableCT['uet'].alias('tariffUet'),
            tableAction['amount'],
            tableVA['sanitation'],
            tableEvent['order']
            ]
        cols.append('getClientRegAddress(Client.id) as address' if addressType else 'getClientLocAddress(Client.id) as address')
        contractTariffDateCond = db.joinAnd([db.joinOr([tableCT['begDate'].dateLe(tableEvent['setDate']), 
                                            tableCT['begDate'].isNull()]), db.joinOr([tableCT['endDate'].dateGe(tableEvent['setDate']), 
                                            tableCT['endDate'].isNull()])])
        cond = [tableET['form'].eq('043'),
            tableAction['deleted'].eq(0),
            tableAction['endDate'].dateEq(date), 
            tableCT['deleted'].eq(0), 
            contractTariffDateCond]
        if personId:
            cond.append(tableAction['person_id'].eq(personId))
        elif orgStructureId:
            persons = getOrgStructurePersonIdList(orgStructureId)
            cond.append(tableAction['person_id'].inlist(persons))
        recordList = db.getRecordList(table, cols, cond)
        eventIdList = []
        for record in recordList:
            clientId = forceRef(record.value('client_id'))
            #clientAddress = context.getInstance(CClientAddressInfo, clientId, addressType)
            clientAddress = forceString(record.value('address'))
            endDate = forceDateTime(record.value('execDate'))
            isPrimary = forceInt(record.value('isPrimary'))
            age = forceInt(record.value('age'))
            actionCode = forceString(record.value('code'))
            eventId = forceRef(record.value('event_id'))
            actionUet = forceDouble(record.value('actionUet'))
            tariffUet = forceDouble(record.value('tariffUet'))
            uet = actionUet if actionUet else tariffUet
            actionAmount = forceInt(record.value('amount'))
            clientName = formatShortName(forceString(record.value('lastName')), forceString(record.value('firstName')), forceString(record.value('patrName')))
            sanitation = forceString(record.value('sanitation')).replace(' ', '').split(',')
            order = forceInt(record.value('order'))
            row = data.setdefault((clientId, clientName), [0, '', '', '', '', 0, 0, '', '', 0, 0, 0])
            row[1] = endDate.toString("hh:mm")
            row[2] = clientName
            row[3] = formatDate(forceDate(record.value('birthDate')))
            row[4] = unicode(clientAddress)
            if eventId not in eventIdList:
                row[5] += (1 if isPrimary==1 else 0)
                row[6] += (1 if (isPrimary==1 and age < 15) else 0)
                row[7] = forceString(record.value('MKB'))
                eventIdList.append(eventId)
            if uet and actionAmount:
                printCode = '%s: %i'%(actionCode, actionAmount)
                row[8] = '\n'.join([row[8], printCode]) if row[8] else printCode
            row[11] += actionUet
            if checkInList(sanitation, [u'проведена']):
                row[9] += 1
            if checkInList(sanitation, [u'проведена']) and order == 1:
                row[10] += 1
        return data


class CStomatReport(CReportEx):
    def __init__(self, parent):
        CReportEx.__init__(self, parent)
        self.setTitle(u'Отчет о работе стоматолога (форма 039-2)', u'Отчет о работе стоматолога (форма 039-2)')
        self.table_columns = self.getTableColumns()
        self.colCount = len(self.table_columns)

    def getTableColumns(self, keyName=False):
        return [
            ('3%', [u'Месяц' if keyName else u'Дата', '', '', '', u'1'], CReportBase.AlignLeft),
            ('3%', [u'Принято больных (Всего)', '', '', '', u'2'], CReportBase.AlignRight),
            ('5%', [u'Принято первичных больных', '', u'Всего', '', u'3'], CReportBase.AlignRight),
            ('5%', [u'', '', u'Из них детей', '', u'4'], CReportBase.AlignRight),
            ('3%', [u'Запломбировано зубов', u'Всего', '', '', u'5'], CReportBase.AlignRight),
            ('5%', [u'', u'В том числе, по поводу', u'кариеса', u'постоянных', u'6'], CReportBase.AlignRight),
            ('4%', [u'', '', '', u'молочных зубов', u'7'], CReportBase.AlignRight),
            ('4%', [u'', '', u'его осложнений', u'постоянных', u'8'], CReportBase.AlignRight),
            ('4%', [u'', '', '', u'молочных зубов', u'9'], CReportBase.AlignRight),
            ('4%', [u'Вылечено зубов в одно посещение по поводу осложн. кариеса', '', '', '', u'10'], CReportBase.AlignRight),
            ('3%', [u'Кол.-во пломб из амальгамы', '', '', '', u'11'], CReportBase.AlignRight),
            ('4%', [u'Проведен курс лечения', u'Заболевание пародонта', u'Количество зубов', '', u'12'], CReportBase.AlignRight),
            ('4%', [u'', u'', u'Количество пациентов', '', u'13'], CReportBase.AlignRight),
            ('4%', [u'', u'Слизистой оболочки рта', u'', '', u'14'], CReportBase.AlignRight),
            ('3%', [u'Удалено зубов', u'Молочного прикуса (ВСЕГО)', '', '', u'15'], CReportBase.AlignRight),
            ('3%', [u'', u'В том числе по поводу заболеваний пародонта постоянного прикуса', '', '', u'16'], CReportBase.AlignRight),
            ('4%', [u'', u'Постоянного прикуса (ВСЕГО)', '', '', u'17'], CReportBase.AlignRight),
            ('3%', [u'Произведено операций', '', '', '', u'18'], CReportBase.AlignRight),
            ('4%', [u'Всего санировано в порядке плановой санации и по обращению', '', '', '', u'19'], CReportBase.AlignRight),
            ('4%', [u'Профилактическая работа', u'Осмотрено в порядке плановой санации', '', '', u'20'], CReportBase.AlignRight),
            ('4%', [u'', u'Из числа осмотренных нуждалось в санации', '', '', u'21'], CReportBase.AlignRight),
            ('4%', [u'', u'Санировано из числа выявленных при плановой санации', '', '', u'22'], CReportBase.AlignRight),
            ('4%', [u'', u'Проведен курс профилактических мероприятий', u'Количество мероприятий', '', u'23'], CReportBase.AlignRight),
            ('4%', [u'', u'', u'Количество пациентов', '', u'24'], CReportBase.AlignRight),
            ('4%', [u'', u'', u'Количество событий', '', u'25'], CReportBase.AlignRight),
            ('4%', [u'Выработано условных единиц трудоемкости (УЕТ)', '', '', '', u'26'], CReportBase.AlignRight),
            ]

    def getSetupDialog(self, parent):
        result = CSetupReportMonthActions(parent)
        result.setTitle(self.title())
        result.setActionTypeClassVisible(False)
        result.setActionTypeGroupVisible(False)
        result.setStatusVisible(False)
        result.setAssistantVisible(False)
        result.setTissueTypeVisible(False)
        result.setPersonDetailVisible(True)
        result.setPersonMonthDetailVisible(True)
        result.setLocalityVisible(True)
        result.setUETVisible(True)
        return result


    def dumpParams(self, cursor, params):
        sexList = (u'не определено', u'мужской', u'женский')
        description = []
        begDate = params.get('begDate', QDate())
        endDate = params.get('endDate', QDate())
        if begDate and endDate:
            description.append(dateRangeAsStr(u'за период', begDate, endDate))
        orgStructureId = params.get('orgStructureId', None)
        sexIndex = params.get('sex', 0)
        ageFor = params.get('ageFrom', 0)
        ageTo = params.get('ageTo', 150)
        personExecId = params.get('personId', None)
        if orgStructureId:
            description.append(u'подразделение: ' + getOrgStructureFullName(orgStructureId))
        else:
            description.append(u'подразделение: ЛПУ')
        financeId = params.get('financeId', None)
        description.append(u'тип финансирования: %s'%(forceString(QtGui.qApp.db.translate('rbFinance', 'id', financeId, 'name')) if financeId else u'не задано'))
        if sexIndex:
            description.append(u'пол ' + sexList[sexIndex])
        if ageFor or ageTo:
            description.append(u'возраст' + u' с '+forceString(ageFor) + u' по '+forceString(ageTo))
        locality = params.get('locality', 0)
        if locality:
            description.append(u'местность: %s жители' % ((u'городские', u'сельские')[locality-1]))
        if personExecId:
            personExecName = forceString(QtGui.qApp.db.translate('vrbPersonWithSpeciality', 'id', personExecId, 'name'))
            description.append(u'исполнитель ' + personExecName)
        if params.get('chkPersonOSDetail', False):
            description.append(u'Детализировать по подразделениям')
        if params.get('chkPersonOverall', False):
            description.append(u'Общий по врачам')
        if params.get('chkPersonDetail', False):
            description.append(u'Детализировать по врачам')
        if params.get('chkMonthDetail', False):
            description.append(u'Детализировать по месяцам')
        if params.get('chkPersonOverall', False):
            description.append(u'Детализировать по датам')
        uetCond = params.get('uetCond', 0)
        if uetCond:
            description.append(u'Учитывать УЕТ по нормативу')
        else:
            description.append(u'Учитывать УЕТ по договору')
        description.append(u'отчёт составлен: '+forceString(QDateTime.currentDateTime()))
        columns = [ ('100%', [], CReportBase.AlignLeft) ]
        table = createTable(cursor, columns, headerRowCount=len(description), border=0, cellPadding=2, cellSpacing=0)
        for i, row in enumerate(description):
            table.setText(i, 0, row)
        cursor.movePosition(QtGui.QTextCursor.End)
        cursor.insertBlock()


    def build(self, params):
        doc = QtGui.QTextDocument()
        boldChars = QtGui.QTextCharFormat()
        boldChars.setFontWeight(QtGui.QFont.Bold)
        blockFormatCenter =QtGui.QTextBlockFormat()
        blockFormatCenter.setAlignment(Qt.AlignCenter)
        cursor = QtGui.QTextCursor(doc)
        cursor.setCharFormat(CReportBase.ReportTitle)
        reportName = self.getReportName()
        if 'getPeriodName' in dir(self):
            reportName += u' за ' + self.getPeriodName(params)
        self.addParagraph(cursor, reportName)
        self.dumpParams(cursor, params)
        cursor.insertBlock()
        cursor.movePosition(QtGui.QTextCursor.End)
        data = self.getData(params)
        chkPersonOSDetail = params.get('chkPersonOSDetail', False)
        chkPersonDetail = params.get('chkPersonDetail', False)
        chkPersonOverall = params.get('chkPersonOverall', False)
        chkPersonDateDetail = params.get('chkPersonDateDetail', False)
        chkMonthDetail = params.get('chkMonthDetail', False)
        resultRow = [0]*25
        if chkPersonDetail:
            firstPerson = True
            for osName, osData in data.iteritems():
                for personName, personData in osData.iteritems():
                    cursor.movePosition(QtGui.QTextCursor.End)
                    format = QtGui.QTextCharFormat()
                    font = QtGui.QFont()
                    font.setBold(True)
                    font.setPointSize(13)
                    format.setFont(font)
                    cursor.setCharFormat(format)
                    if firstPerson:
                        cursor.insertText(u'Врач: %s\n'%personName)
                        firstPerson = False
                    else:
                        cursor.insertText(u'\n\nВрач: %s\n'%personName)
                    table = self.getTable(cursor)
                    resultRow = [0]*25
                    for date in sorted(personData.keys(), key=dateKey):
                        dateRow = personData[date]
                        if chkPersonDateDetail:
                            row = table.addRow()
                            table.setText(row, 0, date)
                        for i, val in enumerate(dateRow):
                            if chkPersonDateDetail:
                                table.setText(row, i+1, val)
                            resultRow[i] += val
                    row = table.addRow()
                    table.setText(row, 0, u'Итого', charFormat=boldChars)
                    for i, val in enumerate(resultRow):
                        table.setText(row, i+1, val, charFormat=boldChars)
        elif chkPersonOverall:
            table = self.getTable(cursor)
            resultRow = [0]*25
            for osName, osData in data.iteritems():
                for personName, personData in osData.iteritems():
                    personRow = [0]*25
                    for date in sorted(personData.keys(), key=dateKey):
                        dateRow = personData[date]
                        for i, val in enumerate(dateRow):
                            personRow[i] += val
                            resultRow[i] += val
                    row = table.addRow()
                    table.setText(row, 0, personName)
                    for i, val in enumerate(personRow):
                        table.setText(row, i+1, val)
            row = table.addRow()
            table.setText(row, 0, u'Итого', charFormat=boldChars)
            for i, val in enumerate(resultRow):
                table.setText(row, i+1, val, charFormat=boldChars)
        elif chkPersonOSDetail:
            table = self.getTable(cursor)
            for osName, osData in data.iteritems():
                resultRow = [0]*25
                row = table.addRow()
                table.setText(row, 0, osName, charFormat=boldChars, blockFormat=blockFormatCenter)
                table.mergeCells(row, 0, 1, 25)
                dateResultRow = {}
                for personName, personData in osData.iteritems():
                    for date in sorted(personData.keys(), key=dateKey):
                        dateRow = personData[date]
                        if chkMonthDetail:
                            date = '.'.join(date.split('.')[1:])
                        resRow = dateResultRow.setdefault(date, [0]*25)
                        for i, val in enumerate(dateRow):
                            resRow[i] += val
                            resultRow[i] += val
                if chkPersonDateDetail or chkMonthDetail:
                    for date in sorted(dateResultRow.keys(), key=dateKey):
                        row = table.addRow()
                        table.setText(row, 0, date)
                        for i, val in enumerate(dateResultRow.get(date, [0]*25)):
                            table.setText(row, i+1, val)
                row = table.addRow()
                table.setText(row, 0, u'Итого', charFormat=boldChars)
                for i, val in enumerate(resultRow):
                    table.setText(row, i+1, val, charFormat=boldChars)
        else:
            table = self.getTable(cursor)
            dateResultRow = {}
            for osName, osData in data.iteritems():
                for personName, personData in osData.iteritems():
                    for date, dateRow in personData.iteritems():
                        if chkMonthDetail:
                            date = '.'.join(date.split('.')[1:])
                        resRow = dateResultRow.setdefault(date, [0]*25)
                        for i, val in enumerate(dateRow):
                            resRow[i] += val
            for date in sorted(dateResultRow.keys(), key=dateKey):
                row = table.addRow()
                table.setText(row, 0, date)
                for i, val in enumerate(dateResultRow.get(date, [0]*25)):
                    table.setText(row, i+1, val)
                    resultRow[i] += val
            row = table.addRow()
            table.setText(row, 0, u'Итого', charFormat=boldChars)
            for i, val in enumerate(resultRow):
                table.setText(row, i+1, val, charFormat=boldChars)
        return doc


    def getData(self, params):
        begDate = params.get('begDate', QDate.currentDate())
        endDate = params.get('endDate', QDate.currentDate())
        orgStructureId = params.get('orgStructureId', None)
        personId = params.get('personId', None)
        sex = params.get('sex', None)
        ageFrom = params.get('ageFrom', None)
        ageTo = params.get('ageTo', None)
        financeId = params.get('financeId', None)
        locality = params.get('locality', 0)
        uetCond = params.get('uetCond', 0)
        db = QtGui.qApp.db
        vActionUU = db.table('vActionStomUU')
        vActionLL = db.table('vActionStomLL')
        vActionSU = db.table('vActionStomSU')
        vActionSL = db.table('vActionStomSL')
        vActionCU = db.table('vActionStomCU')
        vActionCL = db.table('vActionStomCL')
        vActionOt = db.table('vActionStomOther')
        tableAction = db.table('Action')
        tableEvent = db.table('Event')
        tableAT = db.table('ActionType')
        tableClient = db.table('Client')
        tableATS = db.table('ActionType_Service')
        tableCT = db.table('Contract_Tariff')
        tableRBS = db.table('rbService')
        tablePerson = db.table('vrbPersonWithSpeciality')
        tableOS = db.table('OrgStructure')
        cond = [tableAction['endDate'].dateLe(endDate),
            tableAction['endDate'].dateGe(begDate)]
        if personId:
            cond.append(tableAction['person_id'].eq(personId))
        elif orgStructureId:
            persons = getOrgStructurePersonIdList(orgStructureId)
            cond.append(tableAction['person_id'].inlist(persons))
        if sex:
            cond.append(tableClient['sex'].eq(sex))
        if ageFrom:
            cond.append('age(Client.`birthDate`, Action.endDate) >= %d'%ageFrom)
        if ageTo is not None:
            cond.append('age(Client.`birthDate`, Action.endDate) <= %d'%ageTo)
        if financeId:
            cond.append('(Action.finance_id=%d)'%financeId)
        if locality:
            # 1: горожане, isClientVillager == 0 или NULL
            # 2: сельские жители, isClientVillager == 1
            cond.append('IFNULL(isClientVillager(Client.id), 0) = %d' % (locality-1))
        cols = [tableEvent['isPrimary'].alias('isPrimary'),
            tableAction['endDate'].alias('endDate'),
            tableAction['event_id'].alias('event_id'),
            tableAction['id'].alias('action_id'),
            'age(Client.birthDate, Action.endDate) as age',
            tableClient['id'].alias('client_id'),
            tableAT['serviceType'].alias('serviceType'),
            tablePerson['id'].alias('person_id'),
            tablePerson['name'].alias('person_name'),
            tableOS['name'].alias('orgStructure_name')
            ]
        table = tableAction.leftJoin(tableEvent, tableEvent['id'].eq(tableAction['event_id']))
        table = table.leftJoin(tableClient, tableClient['id'].eq(tableEvent['client_id']))
        table = table.leftJoin(tableAT, tableAT['id'].eq(tableAction['actionType_id']))
        table = table.leftJoin(tablePerson, tablePerson['id'].eq(tableAction['person_id']))
        table = table.leftJoin(tableOS, tableOS['id'].eq(tablePerson['orgStructure_id']))

        recordList = db.getRecordList(table, cols, cond)
        data = {}
        eventList = []
        actionIdList = []
        actionIdToEventId = {}
        actionIdToClientId = {}
        actionIdToServiceType = {}
        actionIdToPersonName = {}
        actionIdToOSName = {}
        actionIdToDate = {}
        for record in recordList:
            age = forceInt(record.value('age'))
            if ageFrom and age < ageFrom:
                continue
            if ageTo is not None and age > ageTo:
                continue
            date = formatDate(forceDate(record.value('endDate')))
            eventId = forceRef(record.value('event_id'))
            isPrimary = forceInt(record.value('isPrimary'))
            action_id = forceRef(record.value('action_id'))
            client_id = forceRef(record.value('client_id'))
            serviceType = forceInt(record.value('serviceType'))
            osName = forceString(record.value('orgStructure_name'))
            personName = forceString(record.value('person_name'))
            actionIdList.append(action_id)
            actionIdToClientId[action_id] = client_id
            actionIdToServiceType[action_id] = serviceType
            actionIdToPersonName[action_id] = personName
            actionIdToOSName[action_id] = osName
            actionIdToDate[action_id] = date
            age = forceInt(record.value('age'))
            osdata = data.setdefault(osName, {})
            persondata = osdata.setdefault(personName, self.getEmptyDict(begDate, endDate))
            row = persondata.setdefault(date, [0]*25)
            actionIdToEventId[action_id] = eventId
            if eventId not in eventList:
                row[0] += 1
                eventList.append(eventId)
                if isPrimary == 1:
                    row[1] += 1
                    if age < 15:
                        row[2] += 1


        toothData = {}
        otherParams = ['machine',
            'objectivity',
            'o_treatment',
            'bite',
            'note',
            'prostheses',
            'sanitation',
            'mucous']
        otherParamsEmptyString = ["", "", "", "", "", "", "", ""]
        SURecordList = db.getRecordList(vActionSU, '*', [vActionSU['action_id'].inlist(actionIdList)])
        SLRecordList = db.getRecordList(vActionSL, '*', [vActionSL['action_id'].inlist(actionIdList)])
        UURecordList = db.getRecordList(vActionUU, '*', [vActionUU['action_id'].inlist(actionIdList)])
        LLRecordList = db.getRecordList(vActionLL, '*', [vActionLL['action_id'].inlist(actionIdList)])
        CURecordList = db.getRecordList(vActionCU, '*', [vActionCU['action_id'].inlist(actionIdList)])
        CLRecordList = db.getRecordList(vActionCL, '*', [vActionCL['action_id'].inlist(actionIdList)])
        OtRecordList = db.getRecordList(vActionOt, '*', [vActionOt['action_id'].inlist(actionIdList)])
        for record in SURecordList:
            row = toothData.setdefault(forceRef(record.value('action_id')), {'status':[""]*32, 'condition': [""]*32, 'number': [""]*32,
                'other':dict(zip(otherParams, otherParamsEmptyString))})['status']
            for i in range(record.count()-1):
                row[i] = forceString(record.value(i+1))
        for record in SLRecordList:
            row = toothData.setdefault(forceRef(record.value('action_id')), {'status':[""]*32, 'condition': [""]*32, 'number': [""]*32,
                'other':dict(zip(otherParams, otherParamsEmptyString))})['status']
            for i in range(record.count()-1):
                row[i+16] = forceString(record.value(i+1))
        for record in UURecordList:
            row = toothData.setdefault(forceRef(record.value('action_id')), {'status':[""]*32, 'condition': [""]*32, 'number': [0]*32,
                'other':dict(zip(otherParams, otherParamsEmptyString))})['number']
            for i in range(record.count()-1):
                row[i] = forceInt(record.value(i+1))
        for record in LLRecordList:
            row = toothData.setdefault(forceRef(record.value('action_id')), {'status':[""]*32, 'condition': [""]*32, 'number': [0]*32,
                'other':dict(zip(otherParams, otherParamsEmptyString))})['number']
            for i in range(record.count()-1):
                row[i+16] = forceInt(record.value(i+1))
        for record in CURecordList:
            row = toothData.setdefault(forceRef(record.value('action_id')), {'status':[""]*32, 'condition': [""]*32, 'number': [""]*32,
                'other':dict(zip(otherParams, otherParamsEmptyString))})['condition']
            for i in range(record.count()-1):
                row[i] = forceString(record.value(i+1))
        for record in CLRecordList:
            row = toothData.setdefault(forceRef(record.value('action_id')), {'status':[""]*32, 'condition': [""]*32, 'number': [""]*32,
                'other':dict(zip(otherParams, otherParamsEmptyString))})['condition']
            for i in range(record.count()-1):
                row[i+16] = forceString(record.value(i+1))
        for record in OtRecordList:
            otherData = toothData.setdefault(forceRef(record.value('action_id')), {'status':[""]*32, 'condition': [""]*32, 'number': [""]*32,
                'other':dict(zip(otherParams, otherParamsEmptyString))})['other']
            for par in otherParams:
                otherData[par] = forceString(record.value(par))

        clientIdList = set()
        eventIdList = set()
        clientIdListForCols212223 = set()
        eventIdListForCols212223 = set()
        clientIdList13 = set()
        clientIdList14 = set()
        for (action_id, actionData) in toothData.iteritems():
            osdata = data.setdefault(actionIdToOSName[action_id], {})
            persondata = osdata.setdefault(actionIdToPersonName[action_id], {})
            row = persondata.setdefault(actionIdToDate[action_id], [0]*25)
            sanation = actionData['other']['sanitation'].replace(' ', '').split(',')
            for i in range(32):
                conditions = actionData['condition'][i].replace(' ', '').split(',')
                if actionData['status'][i] in ('5', '6', '7'):
                    row[3] += 1
                if actionData['status'][i] == '5' and checkInList(conditions, [u"П/С", u"C"]) and actionData['number'][i] <= 48 and actionData['number'][i] > 0:
                    row[4] += 1
                if actionData['status'][i] == '5' and checkInList(conditions, [u"П/С", u"C"]) and actionData['number'][i] >= 50:
                    row[5] += 1
                if actionData['status'][i] in ('6',  '7') and checkInList(conditions, [u"П/С", u"C", u"P", u"Pt", u"R"]) and actionData['number'][i] <= 48 and actionData['number'][i] > 0:
                    row[6] += 1
                if actionData['status'][i] in ('6',  '7') and checkInList(conditions, [u"П/С", u"C", u"P", u"Pt", u"R"]) and actionData['number'][i] >= 50:
                    row[7] += 1
                if actionData['status'][i] == '6' and checkInList(conditions, [u"П/С", u"C", u"P", u"Pt", u"R"]):
                    row[8] += 1
                if actionData['status'][i] in ('5', '6', '7') and checkInList(conditions, [u"Па"]):
                    row[9] += 1
                if actionData['status'][i] == '4' and checkInList(conditions, [u"A", u"G"]):
                    row[10] += 1
                if actionData['status'][i] == '4' and checkInList(conditions, [u"A", u"G"]) and actionIdToClientId[action_id] not in clientIdList13:
                    row[11] += 1
                    clientIdList13.add(actionIdToClientId[action_id])
                if actionData['status'][i] == '4' and not checkInList(conditions, [u"A", u"G"]) and actionIdToClientId[action_id] not in clientIdList14:
                    row[12] += 1
                    clientIdList14.add(actionIdToClientId[action_id])
                if actionData['status'][i] == '9' and actionData['number'][i] >= 50:
                    row[13] += 1
                if actionData['status'][i] == '9' and checkInList(conditions, [u"A"]) and actionData['number'][i] <= 48 and actionData['number'][i] > 0:
                    row[14] += 1
                if actionData['status'][i] == '9' and actionData['number'][i] <= 48 and actionData['number'][i] > 0:
                    row[15] += 1
            if actionIdToServiceType[action_id] == 4:
                row[16] += 1
            if checkInList(sanation, [u'проведена']):
                row[17] += 1
            if checkInList(sanation, [u'запланирована']):
                row[18] += 1
            if checkInList(sanation, [u'нуждается']):
                row[19] += 1
            if checkAllInList(sanation, [u'проведена', u'запланирована']):
                row[20] += 1
            clientIdList.add(actionIdToClientId[action_id])
            eventIdList.add(actionIdToEventId[action_id])
        for (action_id, actionData) in actionIdToServiceType.iteritems():
            osdata = data.setdefault(actionIdToOSName[action_id], {})
            persondata = osdata.setdefault(actionIdToPersonName[action_id], {})
            row = persondata.setdefault(actionIdToDate[action_id], [0]*25)
            if actionIdToServiceType[action_id] == 7:
                row[21] += 1
            if actionIdToServiceType[action_id] == 7 and actionIdToClientId[action_id] not in clientIdListForCols212223:
                row[22] += 1
                clientIdListForCols212223.add(actionIdToClientId[action_id])
            if actionIdToServiceType[action_id] == 7 and actionIdToEventId[action_id] not in eventIdListForCols212223:
                row[23] += 1
                eventIdListForCols212223.add(actionIdToEventId[action_id])


        table = tableAction.leftJoin(tableEvent, tableEvent['id'].eq(tableAction['event_id']))
        table = table.leftJoin(tableClient, tableClient['id'].eq(tableEvent['client_id']))
        table = table.leftJoin(tablePerson, tablePerson['id'].eq(tableAction['person_id']))
        table = table.leftJoin(tableAT, tableAT['id'].eq(tableAction['actionType_id']))
        table = table.leftJoin(tableOS, tableOS['id'].eq(tablePerson['orgStructure_id']))
        cols = [tableAction['id'],
                tableAction['endDate'],
                tableAction['uet'],
                tableAction['amount'],
                tablePerson['name'].alias('person_name'),
                tableOS['name'].alias('orgStructure_name')]

        #UET
        if uetCond == 0:
            table = table.leftJoin(tableATS, db.joinAnd([tableATS['master_id'].eq(tableAT['id']),
                                                                        db.joinOr([tableATS['finance_id'].isNull(), tableATS['finance_id'].eq(tableAction['finance_id'])])]))
            table = table.leftJoin(tableCT, [tableCT['service_id'].eq(tableATS['service_id']), tableCT['master_id'].eq(tableEvent['contract_id'])])
            cols.append(tableCT['uet'],)
            cond = ['isSexAndAgeSuitable(Client.sex, Client.birthDate, Contract_Tariff.sex, Contract_Tariff.age, Action.endDate)',
                tableCT['deleted'].eq(0),
                db.joinOr([tableCT['tariffCategory_id'].eq(tablePerson['tariffCategory_id']), tableCT['tariffCategory_id'].isNull()]),
                db.joinOr([tableCT['begDate'].dateLe(tableAction['endDate']), tableCT['begDate'].isNull()]),
                db.joinOr([tableCT['endDate'].dateGe(tableAction['begDate']), tableCT['begDate'].isNull()]), 
                tableAction['endDate'].dateLe(endDate),
                tableAction['endDate'].dateGe(begDate)]
        else:
            table = table.leftJoin(tableRBS, tableRBS['id'].eq(tableAT['nomenclativeService_id']))
            cols.append(u'IF(AGE(Client.birthDate, CURDATE()) > 18, rbService.`adultUetDoctor`, rbService.`childUetDoctor`) as `uet`')
            cond.append(db.joinOr([tableRBS['adultUetDoctor'].isNotNull(), tableRBS['childUetDoctor'].isNotNull()]))

        if len(eventList):
            cond.append(tableEvent['id'].inlist(eventList))

        recordList = db.getRecordList(table, cols, cond) # may be use group by for prevent multiple join of ActionType_Service?
        for record in recordList:
            action_id = forceRef(record.value(0))
            endDate = formatDate(forceDate(record.value(1)))
            auet = forceDouble(record.value(2))
            amount = forceInt(record.value(3))
            personName = forceString(record.value(4))
            osName = forceString(record.value(5))
            tuet = forceDouble(record.value(6))

            osdata = data.setdefault(osName, {})
            persondata = osdata.setdefault(personName, {})
            row = persondata.setdefault(endDate, [0]*25)
            if uetCond==0:
                row[24] += auet
            else:
                row[24] += tuet*amount

        return data

    def getEmptyDict(self, begDate, endDate):
        result = {}
        for i in range(begDate.daysTo(endDate)+1):
            date = formatDate(begDate.addDays(i))
            result[date] = [0]*25
        return result

    def getTable(self, cursor, keyName=False):
        cursor.movePosition(QtGui.QTextCursor.End)

        table = self.createTable(cursor)
        table.mergeCells(0, 0, 4, 1)
        if keyName:
            table.mergeCells(0, 1, 4, 1)

        table.mergeCells(0, 2 if keyName else 1, 4, 1)

        table.mergeCells(0, 3 if keyName else 2, 2, 2)
        table.mergeCells(2, 3 if keyName else 2, 2, 1)
        table.mergeCells(2, 4 if keyName else 3, 2, 1)

        table.mergeCells(0, 5 if keyName else 4, 1, 5)
        table.mergeCells(1, 5 if keyName else 4, 3, 1)
        table.mergeCells(1, 6 if keyName else 5, 1, 4)
        table.mergeCells(2, 6 if keyName else 5, 1, 2)
        table.mergeCells(2, 8 if keyName else 7, 1, 2)

        table.mergeCells(0, 10 if keyName else 9, 4, 1)

        table.mergeCells(0, 11 if keyName else 10, 4, 1)

        table.mergeCells(0, 12 if keyName else 11, 1, 3)
        table.mergeCells(1, 12 if keyName else 11, 1, 2)
        table.mergeCells(2, 12 if keyName else 11, 2, 1)
        table.mergeCells(2, 13 if keyName else 12, 2, 1)
        table.mergeCells(1, 14 if keyName else 13, 3, 1)

        table.mergeCells(0, 15 if keyName else 14, 1, 3)
        table.mergeCells(1, 15 if keyName else 14, 3, 1)
        table.mergeCells(1, 16 if keyName else 15, 3, 1)
        table.mergeCells(1, 17 if keyName else 16, 3, 1)

        table.mergeCells(0, 18 if keyName else 17, 4, 1)

        table.mergeCells(0, 19 if keyName else 18, 4, 1)

        table.mergeCells(0, 20 if keyName else 19, 1, 6)
        table.mergeCells(1, 20 if keyName else 19, 3, 1)
        table.mergeCells(1, 21 if keyName else 20, 3, 1)
        table.mergeCells(1, 22 if keyName else 21, 3, 1)
        table.mergeCells(1, 23 if keyName else 22, 1, 3)

        table.mergeCells(2, 23 if keyName else 22, 2, 1)
        table.mergeCells(2, 24 if keyName else 23, 2, 1)
        table.mergeCells(2, 25 if keyName else 24, 2, 1)
        table.mergeCells(0, 26 if keyName else 25, 4, 1)

        return table


def checkInList(list, vals):
    return any(val in list for val in vals)

def checkAllInList(list, vals):
    return all(val in list for val in vals)

def checkNotInList(list, vals):
    return all(val not in list for val in vals)

def dateKey(date):
    return '.'.join(reversed(date.split('.')))
