# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui
from PyQt4.QtCore import QDate

from library.database        import addDateInRange
from library.Utils           import calcAgeInYears, forceBool, forceDate, forceInt, forceRef, forceString

from Orgs.Utils              import getOrgStructureDescendants
from Reports.Report          import CReport
from Reports.ReportBase      import CReportBase, createTable
from Reports.TempInvalidList import CTempInvalidSetupDialog


def selectData(params):
    begDate = params.get('begDate', QDate())
    endDate = params.get('endDate', QDate())
    byPeriod = params.get('byPeriod', False)
    doctype = params.get('doctype', 0)
    tempInvalidReasonId = params.get('tempInvalidReason', None)
    onlyClosed = params.get('onlyClosed', True)
    orgStructureId = params.get('orgStructureId', None)
    personId = params.get('personId', None)
    insuranceOfficeMark = params.get('insuranceOfficeMark', 0)
    hasRegAddress = params.get('hasRegAddress', True)
    hasLocAddress = params.get('hasLocAddress', True)
    isTempInvalidReceiver = params.get('isTempInvalidReceiver', False)
    isTempInvalidDuplicate = params.get('isTempInvalidDuplicate', False)
    isTempInvalidDuplicateWork = params.get('isTempInvalidDuplicateWork', False)
    dateSort = params.get('dateSort', 0)
    isClientNameSort = params.get('isClientNameSort', False)

    stmt="""
SELECT
   CONCAT_WS(_utf8' ', Client.lastName, Client.firstName, Client.patrName, CAST(Client.id AS CHAR)) AS clientName,
   Client.birthDate,
   (SELECT T.number FROM TempInvalid AS T WHERE T.deleted = 0 AND T.prev_id = TempInvalid.id) AS prevNumberTI,
   TempInvalid.id AS tempInvalidId,
   TempInvalid.number,
   TempInvalid.prev_id,
   TempInvalid.placeWork,
   TempInvalid.diagnosis_id,
   TempInvalid.person_id,
   vrbPersonWithSpeciality.name AS personName,
   IF(EXISTS(SELECT TempInvalid_Period.id
   FROM TempInvalid_Period
   WHERE TempInvalid_Period.master_id = TempInvalid.prev_id
   AND TempInvalid_Period.isExternal = 1), 1, 0) AS isExternalPrev,
   IF((SELECT TI.prev_id
   FROM TempInvalid AS TI
   WHERE TI.id = TempInvalid.prev_id AND TI.deleted = 0) IS NOT NULL, 1, 0) AS isPreviousPrevId,
   IF(EXISTS(SELECT TempInvalid_Period.id
   FROM TempInvalid_Period
   WHERE TempInvalid_Period.master_id = TempInvalid.id
   AND TempInvalid_Period.isExternal = 1
   LIMIT 1), 1, 0) AS isExternal,
   (SELECT TempInvalid_Period.begPerson_id
   FROM TempInvalid_Period
   WHERE TempInvalid_Period.master_id = TempInvalid.id
   ORDER BY TempInvalid_Period.begDate ASC
   LIMIT 1) AS begPersonId,
   (SELECT TempInvalid_Period.endPerson_id
   FROM TempInvalid_Period
   WHERE TempInvalid_Period.master_id = TempInvalid.id
   ORDER BY TempInvalid_Period.endDate DESC
   LIMIT 1) AS endPersonId,
   (SELECT CONCAT_WS(\' - \', D.MKB, D_MKB.DiagName)
   FROM TempInvalid_Period
   INNER JOIN Diagnosis AS D ON D.id = TempInvalid_Period.diagnosis_id
   INNER JOIN MKB AS D_MKB ON D_MKB.DiagID = D.MKB
   WHERE TempInvalid_Period.master_id = TempInvalid.id
   AND D.deleted = 0
   ORDER BY TempInvalid_Period.begDate ASC
   LIMIT 1) AS MKBFirstPeroid,
   IF(TempInvalid.closed = 3, 1, 0) AS closedExternal,
   TempInvalid.caseBegDate,
   TempInvalid.begDate,
   TempInvalid.endDate,
   TempInvalid.age AS tage,
   TempInvalid.closed,
   DATEDIFF(TempInvalid.endDate, TempInvalid.begDate)+1 AS duration,
   Diagnosis.MKB,
   MKB.DiagName,
   formatClientAddress(ClientAddress.id) AS address
   %s
   %s
   FROM TempInvalid
   LEFT JOIN TempInvalid AS NextTempInvalid ON (TempInvalid.id = NextTempInvalid.prev_id AND NextTempInvalid.deleted = 0)
   LEFT JOIN Diagnosis ON (Diagnosis.id = TempInvalid.diagnosis_id AND Diagnosis.deleted = 0)
   LEFT JOIN MKB AS MKB ON MKB.DiagID = Diagnosis.MKB
   LEFT JOIN vrbPersonWithSpeciality ON vrbPersonWithSpeciality.id = TempInvalid.person_id
   LEFT JOIN Client ON Client.id = TempInvalid.client_id
   LEFT JOIN ClientAddress ON
        ClientAddress.client_id = Client.id AND ClientAddress.deleted = 0 AND
        ClientAddress.id = (SELECT MAX(id) FROM ClientAddress AS CA WHERE CA.Type=%d AND CA.client_id = Client.id)
   %s
   %s
WHERE
   Client.deleted = 0 AND
   %s
ORDER BY %s
    """
    db = QtGui.qApp.db
    table = db.table('TempInvalid')
    cond = []
    if doctype:
        cond.append(table['doctype_id'].eq(doctype))
    else:
        cond.append(table['type'].eq(0))
    cond.append(table['deleted'].eq(0))
    if tempInvalidReasonId:
        cond.append(table['tempInvalidReason_id'].eq(tempInvalidReasonId))
    if byPeriod:
        cond.append(table['caseBegDate'].le(endDate))
        cond.append(table['endDate'].ge(begDate))
    else:
        addDateInRange(cond, table['endDate'], begDate, endDate)
    if onlyClosed:
        cond.append(table['closed'].eq(1))
    if orgStructureId:
        tablePerson = db.table('vrbPersonWithSpeciality')
        cond.append(tablePerson['orgStructure_id'].inlist(getOrgStructureDescendants(orgStructureId)))
    if personId:
        cond.append(table['person_id'].eq(personId))
    if insuranceOfficeMark in (1, 2):
        cond.append(table['insuranceOfficeMark'].eq(insuranceOfficeMark-1))
    colsReceiver = u''
    joinReceiver = u''
    if isTempInvalidReceiver:
        colsReceiver = u''', CONCAT_WS(_utf8' ', ClientReceiver.lastName, ClientReceiver.firstName, ClientReceiver.patrName, CAST(ClientReceiver.id AS CHAR)) AS clientReceiverName,
                           ClientReceiver.birthDate AS birthDateReceiver,
                           IF(ClientAddress.id != ClientReceiverAddress.id, formatClientAddress(ClientReceiverAddress.id), '') AS receiveraddress,
                           (SELECT CONCAT_WS(_utf8' - ', rbRelationType.leftName, rbRelationType.rightName)
                            FROM ClientRelation INNER JOIN rbRelationType ON ClientRelation.relativeType_id = rbRelationType.id
                            WHERE ClientRelation.deleted = 0 AND ClientRelation.client_id = ClientReceiver.id AND ClientRelation.relative_id = Client.id) AS relation'''
        joinReceiver = u'''LEFT JOIN Client AS ClientReceiver ON (TempInvalid.receiver_id != TempInvalid.client_id AND ClientReceiver.id = TempInvalid.receiver_id AND ClientReceiver.deleted = 0)
                           LEFT JOIN ClientAddress AS ClientReceiverAddress ON ClientReceiverAddress.client_id = ClientReceiver.id AND ClientReceiverAddress.deleted = 0
                           AND ClientReceiverAddress.id = (SELECT MAX(id) FROM ClientAddress AS CA WHERE CA.Type=%d AND CA.client_id = ClientReceiver.id)'''%(0 if hasRegAddress else (1 if hasLocAddress else 0))
    colsDuplicate = u''
    joinDuplicate = u''
    if isTempInvalidDuplicate and not isTempInvalidDuplicateWork:
        colsDuplicate = u''', TempInvalidDuplicate.id AS tempInvalidDuplicateId,
                            TempInvalidDuplicate.number AS duplicateNumber,
                            TempInvalidDuplicate.destination'''
        joinDuplicate = u'''LEFT JOIN TempInvalidDuplicate ON (TempInvalidDuplicate.tempInvalid_id = TempInvalid.id AND TempInvalidDuplicate.deleted = 0
                            AND TempInvalidDuplicate.reason_id IN (SELECT rbTempInvalidDuplicateReason.id FROM rbTempInvalidDuplicateReason WHERE rbTempInvalidDuplicateReason.code = '1'))'''
    elif isTempInvalidDuplicateWork and not isTempInvalidDuplicate:
        colsDuplicate = u''', TempInvalidDuplicate.id AS tempInvalidDuplicateId,
                            TempInvalidDuplicate.number AS duplicateNumber,
                            TempInvalidDuplicate.destination'''
        joinDuplicate = u'''LEFT JOIN TempInvalidDuplicate ON (TempInvalidDuplicate.tempInvalid_id = TempInvalid.id AND TempInvalidDuplicate.deleted = 0
                            AND TempInvalidDuplicate.reason_id IN (SELECT rbTempInvalidDuplicateReason.id FROM rbTempInvalidDuplicateReason WHERE rbTempInvalidDuplicateReason.code != '1'))'''
    elif isTempInvalidDuplicateWork and isTempInvalidDuplicate:
        colsDuplicate = u''', TempInvalidDuplicate.id AS tempInvalidDuplicateId,
                            TempInvalidDuplicate.number AS duplicateNumber,
                            TempInvalidDuplicate.destination'''
        joinDuplicate = u'''LEFT JOIN TempInvalidDuplicate ON (TempInvalidDuplicate.tempInvalid_id = TempInvalid.id AND TempInvalidDuplicate.deleted = 0)'''
    order = []
    order.append(u'TempInvalid.begDate ASC' if dateSort else u'TempInvalid.caseBegDate ASC')
    if isClientNameSort:
        order.append(u'Client.lastName, Client.firstName, Client.patrName ASC')
    return db.query(stmt % (colsReceiver,
                            colsDuplicate,
                            0 if hasRegAddress else (1 if hasLocAddress else 0),
                            joinReceiver,
                            joinDuplicate,
                            db.joinAnd(cond),
                            u','.join(i for i in order)))


class CTempInvalidBookF036(CReport):
    name = u'КНИГА РЕГИСТРАЦИИ ЛИСТКОВ НЕТРУДОСПОСОБНОСТИ (Ф.036)'

    def __init__(self, parent):
        CReport.__init__(self, parent)
        self.setTitle(self.name)


    def getSetupDialog(self, parent):
        result = CTempInvalidSetupDialog(parent)
        result.setTitle(self.title())
        result.setCntUserVisible(True)
        result.setTempInvalidReceiverVisible(True)
        result.setTempInvalidDuplicateVisible(True)
        result.setTempInvalidDuplicateWorkVisible(True)
        result.setClientNameSortVisible(True)
        result.setDateSortVisible(True)
        return result


    def build(self, params):
        doc = QtGui.QTextDocument()
        cursor = QtGui.QTextCursor(doc)
        cursor.setCharFormat(CReportBase.ReportTitle)
        cursor.insertText(self.name)
        cursor.insertBlock()
        self.dumpParams(cursor, params)
        isTempInvalidReceiver = params.get('isTempInvalidReceiver', False)
        isTempInvalidDuplicate = params.get('isTempInvalidDuplicate', False)
        isTempInvalidDuplicateWork = params.get('isTempInvalidDuplicateWork', False)
        cursor.insertBlock()
        tableColumns = [
            ('2%', [u'N N п/п', u'', u'1' ], CReportBase.AlignLeft),
            ('8%',[u'N листка нетрудоспособности, выданного данным леч. учреждением', u'первый', u'2' ], CReportBase.AlignLeft),
            ('8%',[u'', u'продолжение', u'3' ], CReportBase.AlignLeft),
            ('8%',[u'N листка нетрудоспособности, выданного другим учреждением', u'первый', u'4' ], CReportBase.AlignLeft),
            ('8%',[u'', u'продолжение', u'5' ], CReportBase.AlignCenter),
            ('9%', [u'Фамилия, имя, отчество больного', u'', u'6' ], CReportBase.AlignLeft),
            ('3%', [u'Возраст', u'', u'7' ], CReportBase.AlignRight),
            ('9%',[u'Адрес больного', u'', u'8' ], CReportBase.AlignLeft),
            ('5%', [u'Место работы и выполняемая работа', u'', u'9' ], CReportBase.AlignLeft),
            ('5%', [u'Диагноз', u'первичный', u'10' ], CReportBase.AlignLeft),
            ('5%', [u'', u'заключительный', u'11' ], CReportBase.AlignLeft),
            ('5%', [u'Фамилия врача', u'выдавшего листок нетрудоспособности', u'12'], CReportBase.AlignLeft),
            ('5%', [u'', u'закончившего листок нетрудоспособности', u'13'], CReportBase.AlignLeft),
            ('5%', [u'Освобожден от работы', u'с какого числа', u'14'], CReportBase.AlignLeft),
            ('5%', [u'', u'по какое число', u'15'], CReportBase.AlignLeft),
            ('5%', [u'Всего календарных дней освобождения от работы', u'', u'16'], CReportBase.AlignRight),
            ('5%', [u'Отметка о направлении больного в другие лечебные учреждения', u'', u'17'], CReportBase.AlignLeft)
            ]

        table = createTable(cursor, tableColumns)
        table.mergeCells(0, 0, 2, 1)
        table.mergeCells(0, 1, 1, 2)
        table.mergeCells(0, 3, 1, 2)
        table.mergeCells(0, 5, 2, 1)
        table.mergeCells(0, 6, 2, 1)
        table.mergeCells(0, 7, 2, 1)
        table.mergeCells(0, 8, 2, 1)
        table.mergeCells(0, 9, 1, 2)
        table.mergeCells(0, 11, 1, 2)
        table.mergeCells(0, 13, 1, 2)
        table.mergeCells(0, 15, 2, 1)
        table.mergeCells(0, 16, 2, 1)
        cnt = params.get('cntUser', 1)
        db = QtGui.qApp.db
        query = selectData(params)
        tempInvalidIdList = []
        tempInvalidDuplicateIdList = []
        while query.next():
            record   = query.record()
            tempInvalidId = forceRef(record.value('tempInvalidId'))
            if tempInvalidId and tempInvalidId not in tempInvalidIdList:
                tempInvalidIdList.append(tempInvalidId)
                duration = forceInt(record.value('duration'))
                age = calcAgeInYears(forceDate(record.value('birthDate')), forceDate(record.value('caseBegDate')))
                MKB = forceString(record.value('MKB'))
                DiagName = forceString(record.value('DiagName'))
                endDateTempInvalid = forceDate(record.value('endDate'))
                caseBegDate = forceDate(record.value('caseBegDate'))
                begDate = forceDate(record.value('begDate'))
                closedExternal = forceBool(record.value('closedExternal'))
                clientName = forceString(record.value('clientName'))
                isExternalPrev = forceBool(record.value('isExternalPrev'))
                isPreviousPrevId = forceBool(record.value('isPreviousPrevId'))
                isExternal = forceBool(record.value('isExternal'))
                number = forceString(record.value('number'))
                prevId = forceRef(record.value('prev_id'))
                prevNumberTI = forceString(record.value('prevNumberTI'))
                placeWork = forceString(record.value('placeWork'))
                begPersonId = forceRef(record.value('begPersonId'))
                begPersonName = forceString(db.translate('vrbPersonWithSpeciality', 'id', begPersonId, 'name')) if begPersonId else u''
                endPersonId = forceRef(record.value('endPersonId'))
                endPersonName = forceString(db.translate('vrbPersonWithSpeciality', 'id', endPersonId, 'name')) if endPersonId else u''
                address = forceString(record.value('address'))
                closed = forceBool(record.value('closed'))
                MKBFirstPeroid = forceString(record.value('MKBFirstPeroid'))
                if isTempInvalidReceiver:
                    birthDateReceiver = forceDate(record.value('birthDateReceiver'))
                    ageReceiver = forceString(calcAgeInYears(birthDateReceiver, forceDate(record.value('caseBegDate')))) if birthDateReceiver else ''
                    relation = forceString(record.value('relation'))
                    clientReceiverName = forceString(record.value('clientReceiverName')) + ((', ' + relation) if relation else u'')
                    receiveraddress = forceString(record.value('receiveraddress'))
                i = table.addRow()
                table.setText(i, 0, cnt)
                if not isExternal:
                    table.setText(i, 1, number)
                    if isExternalPrev:
                        if isPreviousPrevId:
                            table.setText(i, 4, prevNumberTI)
                        else:
                            table.setText(i, 3, prevNumberTI)
                    else:
                        table.setText(i, 2, prevNumberTI)
                else:
                    table.setText(i, 3, number)
                    if isExternalPrev:
                        table.setText(i, 4, prevNumberTI)
                    else:
                        table.setText(i, 2, prevNumberTI)
                table.setText(i, 5, (clientName + u', ' + clientReceiverName) if (isTempInvalidReceiver and clientReceiverName) else clientName)
                table.setText(i, 6, (forceString(age) + u', ' + ageReceiver) if (isTempInvalidReceiver and ageReceiver) else forceString(age))
                table.setText(i, 7, (address + u', ' + receiveraddress) if (isTempInvalidReceiver and receiveraddress) else address)
                table.setText(i, 8, placeWork)
                table.setText(i, 9, MKBFirstPeroid if MKBFirstPeroid else ((MKB + u' - ' + DiagName) if (MKB and DiagName) else u''))
                table.setText(i, 10, (MKB + u' - ' + DiagName) if (MKB and DiagName) else u'')
                table.setText(i, 11, begPersonName)
                table.setText(i, 12, endPersonName if closed else u'')
                table.setText(i, 13, begDate.toString('dd.MM.yyyy') if begDate else u'')
                table.setText(i, 14, (endDateTempInvalid.toString('dd.MM.yyyy') if endDateTempInvalid else u'') if closed else u'')
                table.setText(i, 15, duration)
                table.setText(i, 16, u'Направлен в другое лечебное учреждение' if closedExternal else u'')
                cnt += 1
            tempInvalidDuplicateId = forceRef(record.value('tempInvalidDuplicateId'))
            if (isTempInvalidDuplicate or isTempInvalidDuplicateWork) and tempInvalidDuplicateId and tempInvalidDuplicateId not in tempInvalidDuplicateIdList:
                tempInvalidDuplicateIdList.append(tempInvalidDuplicateId)
                duplicateNumber = forceString(record.value('duplicateNumber'))
                destination = forceString(record.value('destination'))
                i = table.addRow()
                table.setText(i, 0, cnt)
                if not isExternalPrev:
                    table.setText(i, 1, duplicateNumber)
                else:
                    table.setText(i, 3, duplicateNumber)
                if not isExternal:
                    table.setText(i, 2, duplicateNumber if prevId else u'')
                else:
                    table.setText(i, 4, duplicateNumber if prevId else u'')
                table.setText(i, 5, (clientName + u', ' + clientReceiverName) if (isTempInvalidReceiver and clientReceiverName) else clientName)
                table.setText(i, 6, (forceString(age) + u', ' + ageReceiver) if (isTempInvalidReceiver and ageReceiver) else forceString(age))
                table.setText(i, 7, (address + u', ' + receiveraddress) if (isTempInvalidReceiver and receiveraddress) else address)
                table.setText(i, 8, destination)
                cnt += 1
        return doc
