# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui
from PyQt4.QtCore import QDate

from library.Utils      import forceInt, forceString
from Reports.Report     import CReport
from Reports.ReportBase import CReportBase, createTable
from Reports.ReportSetupDialog import CReportSetupDialog

from Ui_ReportWorkloadSetup import Ui_ReportWorkloadSetupDialog


def selectData(params):
    begDate     = params.get('begDate', QDate())
    endDate     = params.get('endDate', QDate())
    eventTypeId = params.get('eventTypeId', None)
    orderType   = params.get('orderType', 0)
    db = QtGui.qApp.db
    tableEvent = db.table('Event')
    cond = [
        tableEvent['createDatetime'].ge(begDate),
        tableEvent['createDatetime'].lt(endDate.addDays(1)),
        ]
    if eventTypeId:
        cond.append(tableEvent['eventType_id'].eq(eventTypeId))
    if orderType:
        joinOS = u'LEFT JOIN OrgStructure ON (OrgStructure.id = Person.orgStructure_id AND OrgStructure.deleted = 0)'
        order = u'OrgStructure.name'
    else:
        joinOS = u''
        order = u'Person.lastName, Person.firstName, Person.patrName'
    condStr = db.joinAnd(cond)
    stmt=u"""
SELECT
    Person.login, Person.lastName, Person.firstName, Person.patrName,
    COUNT(*) AS cnt1, SUM(IF(Event.execDate is null,1,0)) AS cnt2
FROM
    Event
    LEFT JOIN Person ON Person.id=Event.createPerson_id
    %s
WHERE Event.deleted=0 AND %s
GROUP BY Event.createPerson_id
ORDER BY %s
    """ % (joinOS, condStr, order)
    return db.query(stmt)


class CWorkload(CReport):
    def __init__(self, parent):
        CReport.__init__(self, parent)
        self.setPayPeriodVisible(False)
        self.setTitle(u'Выработка')


    def getSetupDialog(self, parent):
        result = CReportWorkloadSetupDialog(parent)
        result.setTitle(self.title())
        return result


    def build(self, params):
        reportData = []
        query = selectData(params)
        while query.next():
            record = query.record()
            login = forceString(record.value('login'))
            lastName = forceString(record.value('lastName'))
            firstName = forceString(record.value('firstName'))
            patrName = forceString(record.value('patrName'))
            cnt1 = forceInt(record.value('cnt1'))
            cnt2 = forceInt(record.value('cnt2'))
            reportData.append([login, lastName, firstName, patrName, cnt1, cnt2])

        doc = QtGui.QTextDocument()
        cursor = QtGui.QTextCursor(doc)

        cursor.setCharFormat(CReportBase.ReportTitle)
        cursor.insertText(self.title())
        cursor.insertBlock()
        self.dumpParams(cursor, params)
        cursor.insertBlock()

        tableColumns = [
            ( '10%', [u'Регистрационное имя'], CReportBase.AlignLeft),
            ( '20%', [u'Фамилия'], CReportBase.AlignLeft),
            ( '20%', [u'Имя'], CReportBase.AlignLeft),
            ( '20%', [u'Отчество'], CReportBase.AlignLeft),
            ( '10%', [u'Всего'], CReportBase.AlignRight),
            ( '10%', [u'В т.ч. без даты выполнения'], CReportBase.AlignRight),
        ]

        table = createTable(cursor, tableColumns)
        totalCnt1 = 0
        totalCnt2 = 0

        for row in reportData:
            i = table.addRow()
            for j in xrange(len(row)):
                table.setText(i, j, row[j])
            totalCnt1 += row[4]
            totalCnt2 += row[5]

        i = table.addRow()
        table.mergeCells(i, 0, 1, 4)
        table.setText(i, 0, u'всего', CReportBase.TableTotal)
        table.setText(i, 4, totalCnt1, CReportBase.TableTotal)
        table.setText(i, 5, totalCnt2, CReportBase.TableTotal)
        return doc


class CReportWorkloadSetupDialog(QtGui.QDialog, Ui_ReportWorkloadSetupDialog):
    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)
        self.cmbEventType.setTable('EventType', True)


    def setTitle(self, title):
        self.setWindowTitle(title)


    def setParams(self, params):
        self.edtBegDate.setDate(params.get('begDate', QDate.currentDate()))
        self.edtEndDate.setDate(params.get('endDate', QDate.currentDate()))
        self.cmbEventType.setValue(params.get('eventTypeId', None))
        self.cmbOrderType.setCurrentIndex(params.get('orderType', 0))


    def params(self):
        result = {}
        result['begDate'] = self.edtBegDate.date()
        result['endDate'] = self.edtEndDate.date()
        result['eventTypeId'] = self.cmbEventType.value()
        result['orderType'] = self.cmbOrderType.currentIndex()
        return result

