# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
##
## Редактирование расписания работ
##
#############################################################################

import sip

from PyQt4 import QtGui
from PyQt4.QtCore import Qt, QDate, pyqtSignature

from library.DialogBase              import CDialogBase

from Orgs.OrgStructComboBoxes        import COrgStructureModel
from Resources.JobFlexTemplateDialog import CJobFlexTemplateDialog
from Resources.Jobs                  import CJobsModel
from Resources.JobTemplateDialog     import CJobTemplateDialog
from Resources.JobTicketsDialog      import CJobTicketsDialog

from Resources.Ui_JobPlannerDialog   import Ui_JobPlannerDialog


class CJobPlanner(CDialogBase, Ui_JobPlannerDialog):
    def __init__(self, parent, orgStructureId=None):
        CDialogBase.__init__(self, parent)
        self.addModels('OrgStructure', COrgStructureModel(self, QtGui.qApp.currentOrgId()))
        self.addModels('Jobs',         CJobsModel(self))


        # действия контекстного меню расписания
        self.addObject('actFillByTemplate',     QtGui.QAction(u'Заполнить по шаблону для текущего подразделения', self))
        self.addObject('actFillByFlexTemplate', QtGui.QAction(u'Заполнить по шаблону "скользящий график"', self))
        self.addObject('actEditJobTickets',     QtGui.QAction(u'Номерки', self))
#        self.addObject('actScheduleItemsHistory',QtGui.QAction(u'История', self))
        self.actFillByTemplate.setShortcut('F9')
        self.actFillByFlexTemplate.setShortcut('Alt+F9')

        # меню кнопки "заполнить"
        self.addObject('mnuFill', QtGui.QMenu(self))
        self.mnuFill.addAction(self.actFillByTemplate)
        self.mnuFill.addAction(self.actFillByFlexTemplate)

        self.setupUi(self)
#        self.btnPrint.setMenu(self.mnuPrint)
        self.setWindowFlags(self.windowFlags() | Qt.WindowMaximizeButtonHint)
        self.setModels(self.treeOrgStructure, self.modelOrgStructure, self.selectionModelOrgStructure)
        self.setModels(self.tblJobs, self.modelJobs, self.selectionModelJobs)

        self.btnFill.setMenu(self.mnuFill)
        self.tblJobs.addPopupActions([
                                       '-',
                                       self.actFillByTemplate,
                                       self.actFillByFlexTemplate,
                                       '-',
                                       self.actEditJobTickets,
#                                       self.actScheduleItemsHistory,
                                     ])
        orgStructureId = orgStructureId or QtGui.qApp.currentOrgStructureId()
        orgStructureIndex = self.modelOrgStructure.findItemId(orgStructureId)
        if orgStructureIndex and orgStructureIndex.isValid():
            self.treeOrgStructure.setCurrentIndex(orgStructureIndex)
            self.treeOrgStructure.setExpanded(orgStructureIndex, True)

        date = QDate.currentDate()
        self.calendar.setSelectedDate(date)


    def exec_(self):
        result = CDialogBase.exec_(self)
        return result

    def saveData(self):
        if not self.modelJobs.checkBegEndDate():
            return False
        QtGui.qApp.callWithWaitCursor(self, self.modelJobs.saveData)
        return True


    def getBegDate(self):
        return QDate(self.calendar.yearShown(), self.calendar.monthShown(), 1)


    def getEndDate(self):
        return QDate(self.calendar.yearShown(), self.calendar.monthShown(), 1).addMonths(1).addDays(-1)


    def getCurrentOrgStructureId(self):
       treeIndex = self.treeOrgStructure.currentIndex()
       treeItem = treeIndex.internalPointer() if treeIndex.isValid() else None
       return treeItem.id() if treeItem else None


    def updateJobs(self, scrollToDate):
        prevOrgStructureId = self.modelJobs.orgStructureId
        prevYear       = self.modelJobs.year
        prevMonth      = self.modelJobs.month
        orgStructureId = self.getCurrentOrgStructureId()
        date           = self.calendar.selectedDate()

        QtGui.qApp.callWithWaitCursor(self, self.modelJobs.setOrgStructureAndMonth, orgStructureId, date.year(), date.month())
        row = self.modelJobs.getRowForDate(date)
        currentIndex = self.tblJobs.currentIndex()
        currentColumn = currentIndex.column() if currentIndex.isValid() else 0
        newIndex = self.modelJobs.index(row, currentColumn)
        self.tblJobs.setCurrentIndex(newIndex)
        if scrollToDate:
            self.tblJobs.scrollTo(newIndex, QtGui.QAbstractItemView.EnsureVisible)
        self.tblJobs.setEnabled( bool(orgStructureId) )
#        self.btnFill.setEnabled( bool(orgStructureId) )
        if prevOrgStructureId != self.modelJobs.orgStructureId or prevYear != self.modelJobs.year or prevMonth != self.modelJobs.month:
               pass
#            self.updateStatisctics()


    def fillByTemplate(self):
        currentOrgStructureId = self.getCurrentOrgStructureId()
        begDate = self.getBegDate()
        endDate = self.getEndDate()
        dialog = CJobTemplateDialog(self)
        try:
            dialog.setOrgStructureId(currentOrgStructureId)
            dialog.setDateRange(begDate, endDate)
            if dialog.exec_():
                QtGui.qApp.callWithWaitCursor(self,
                                              self.modelJobs.setWorkPlan,
                                              dialog.getDateRange(),
                                              dialog.getPeriod(),
                                              dialog.getCustomLength(),
                                              dialog.getFillRedDays(),
                                              dialog.getTemplates(),
                                              dialog.removeExistingJobs()
                                             )
        finally:
            sip.delete(dialog)


    def fillByFlexTemplate(self):
        currentOrgStructureId = self.getCurrentOrgStructureId()
        begDate = self.getBegDate()
        endDate = self.getEndDate()
        dialog = CJobFlexTemplateDialog(self)
        try:
            dialog.setDateRange(begDate, endDate)
            dialog.setSelectedDate(self.calendar.selectedDate())
            if dialog.exec_():
                QtGui.qApp.callWithWaitCursor(self,
                                              self.modelJobs.setFlexWorkPlan,
                                              dialog.getSelectedDates(),
                                              dialog.getTemplates(),
                                              dialog.removeExistingJobs()
                                             )
        finally:
            sip.delete(dialog)



    # обработчики сигналов
    # календарь
    @pyqtSignature('int, int')
    def on_calendar_currentPageChanged(self, year, month):
        selectedDate = self.calendar.selectedDate()
        currYear = selectedDate.year()
        currMonth = selectedDate.month()
        newDate = selectedDate.addMonths((year-currYear)*12+(month-currMonth))
        self.calendar.setSelectedDate(newDate)


    @pyqtSignature('')
    def on_calendar_selectionChanged(self):
        self.updateJobs(True)


    # список подразделений
    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelOrgStructure_currentChanged(self, current, previous):
        self.updateJobs(False)


    @pyqtSignature('')
    def on_tblJobs_popupMenuAboutToShow(self):
        currentIndex = self.tblJobs.currentIndex()
        row = currentIndex.row()
        if 0<=row<self.modelJobs.rowCount():
           job = self.modelJobs.getItem(row)
#           isFreeToChange = job.isFreeToChange()
           isEmpty = job.isEmpty()
        else:
#           isFreeToChange = True
           isEmpty = True
        self.actEditJobTickets.setEnabled(not isEmpty)
#        self.actScheduleItemsHistory.setEnabled(True)


    @pyqtSignature('')
    def on_actFillByTemplate_triggered(self):
        self.fillByTemplate()


    @pyqtSignature('')
    def on_actFillByFlexTemplate_triggered(self):
        self.fillByFlexTemplate()


    @pyqtSignature('')
    def on_actEditJobTickets_triggered(self):
        currentIndex = self.tblJobs.currentIndex()
        row = currentIndex.row()
        if 0<=row<self.modelJobs.rowCount():
           job = self.modelJobs.getItem(row)
           job.reloadItems()
           isFreeToChange = job.isFreeToChange()
           isEmpty = job.isEmpty()

           if isFreeToChange and not isEmpty:
                job.calcTimePlanIfRequired()
                dialog = CJobTicketsDialog(self)
                try:
                    dialog.setJob(job)
                    if dialog.exec_():
                        pass
                finally:
                    sip.delete(dialog)


#
#
#    def getJobPlanNumbersDialog(self):
#        currentIndex = self.tblJobs.currentIndex()
#        row = currentIndex.row()
#        date = QDate(self.calendar.selectedDate())
#        orgStructureJobRecord = self.tblJobs.currentItem()
#        if orgStructureJobRecord:
#            orgStructureId = forceRef(orgStructureJobRecord.value('master_id'))
#            jobTypeId = forceRef(orgStructureJobRecord.value('jobType_id'))
#        else:
#            orgStructureId = None
#            jobTypeId = None
#        if orgStructureId and jobTypeId and date:
#            dialog = CJobPlanNumbersDialog(self, orgStructureId, jobTypeId, date, self.modelJobs.items[row])
#            try:
#                if dialog.exec_():
#                    pass
#            finally:
#                sip.delete(dialog)
#
#
#    def recountJobPlanNumbers(self):
#        currentIndex = self.tblJobs.currentIndex()
#        row = currentIndex.row()
#        planTime = self.modelJobs.items[row].timeRange
#        planCount = self.modelJobs.items[row].quantity
#        self.modelJobs.items[row].forceChange = planTime and planCount > 0
#
#
#
#
#    @pyqtSignature('')
#    def on_actPlanNumbers_triggered(self):
#        self.getJobPlanNumbersDialog()
#
#
#    @pyqtSignature('')
#    def on_actRecountNumbers_triggered(self):
#        self.recountJobPlanNumbers()
#
