# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import uuid

from PyQt4 import QtGui
from PyQt4.QtCore import QDateTime

from library.Utils import calcAgeInYears, forceBool, forceInt, forceRef, forceString, smartDict,  sorry



def getJobTypeAttributes(jobTicketId):
    db = QtGui.qApp.db
    tableJobTicket = db.table('Job_Ticket')
    tableJob = db.table('Job')
    tableJobType = db.table('rbJobType')
    queryTable = tableJobTicket.leftJoin(tableJob, tableJob['id'].eq(tableJobTicket['master_id']))
    queryTable = queryTable.leftJoin(tableJobType, tableJobType['id'].eq(tableJob['jobType_id']))
    record = db.getRecordEx(queryTable, [tableJobType['isInstant'], tableJobType['laboratory_id']], tableJobTicket['id'].eq(jobTicketId))
    if record:
        return smartDict(isInstant=forceBool(record.value('isInstant')),
                         laboratoryId=forceBool(record.value('laboratory_id'))
                        )
    else:
        return None


def isInstant(jobTicketId):
    d = getJobTypeAttributes(jobTicketId)
    return d.isInstant if d else False


def getLaboratoryId(jobTicketId):
    d = getJobTypeAttributes(jobTicketId)
    return d.laboratoryId if d else None


def getLaboratory(laboratoryId):
    db = QtGui.qApp.db
    record = db.getRecord('rbLaboratory', '*', laboratoryId)
    if record:
        return smartDict(
            code = forceString(record.value('code')),
            name = forceString(record.value('name')),
            protocol = forceInt(record.value('protocol')),
            address = forceString(record.value('address')),
            ownName = forceString(record.value('ownName')),
            labName = forceString(record.value('labName')))
    else:
        return None


def getTestCodes(laboratoryId, testIdList):
    db = QtGui.qApp.db
    tableLaboratoryTest = db.table('rbLaboratory_Test')
    tableTest = db.table('rbTest')
    queryTable = tableLaboratoryTest.leftJoin(tableTest, tableTest['id'].eq(tableLaboratoryTest['test_id']))
    records = db.getRecordList(queryTable,
                            [tableLaboratoryTest['test_id'],
                             tableLaboratoryTest['book'],
                             tableLaboratoryTest['code'],
                             tableTest['name']
                            ],
                            [tableLaboratoryTest['master_id'].eq(laboratoryId),
                             tableLaboratoryTest['test_id'].inlist(testIdList),
                            ])
    result = {}
    for record in records:
        testId = forceInt(record.value('test_id'))
        book   = forceString(record.value('book'))
        code   = forceString(record.value('code'))
        name   = forceString(record.value('name'))
        result[testId] = book, code, name
    return result


def sendTest(widget, clientInfo, jobTicketId, properties):
    def _sendTest():
        laboratoryId = getLaboratoryId(jobTicketId)
        laboratory = getLaboratory(laboratoryId)
        if laboratory:
            testIdList = [property.type().testId for property in properties]
            testCodes = getTestCodes(laboratoryId, testIdList)
            if laboratory.protocol == 0:
                sorry()
            elif laboratory.protocol == 1:
                sendTestASTM(widget, clientInfo, jobTicketId, laboratory, testCodes, properties)
    QtGui.qApp.call(widget, _sendTest)


def idToUuid(timestamp, id):
    node = uuid.getnode() # 48 bit, 6 bytes
    timestamp_low = timestamp & 0xFFFFFFFFL # timestamp: 32 bit, 4 bytes
    id_low  = id & 0xFFFFL # id: low 16 bit
    id_mid = (id>>16) & 0xFFL # id: mid 8 bit
    id_high = (id>>24) & 0xFFL # id: high 8 bit
    intValue = (timestamp_low << 96L) | (id_low << 80L) | (id_mid << 64L) | (id_high << 48L) | node
    return uuid.UUID(int=intValue, version=4)




# ====================================================================================

def sendTestASTM(widget, clientInfo, jobTicketId, laboratory, testCodes, properties):
        from Exchange.Lab.AstmE1381.FileInterface import CFileInterface
        from Exchange.Lab.AstmE1394.Message import CMessage
        import json

        opts = json.loads(laboratory.address)

        now = QDateTime.currentDateTime()
        message = CMessage()
        patient = message.newPatient()
        patient.patientId = clientInfo['id']
        patient.laboratoryPatientId = clientInfo['id']
        patient.lastName  = clientInfo['lastName']
        patient.firstName = clientInfo['firstName']
        patient.patrName  = clientInfo['patrName']
        patient.birthDate = clientInfo['birthDate']
        patient.sex = ['U', 'M', 'F'][clientInfo['sexCode']]
        patient.age       = calcAgeInYears(clientInfo['birthDate'], now.date())

        for property in properties:
            testId = property.type().testId
            if testId in testCodes:
                book, code, name = testCodes[testId]
                propertyRecordId = forceRef(property.getRecord().value('id'))
                order = patient.newOrder()
                order.specimenId = jobTicketId
                order.assayCode       =  code
                order.assayName       =  name
                order.requestDateTime =  now
                order.specimenCollectionDateTime = now
                order. propertyRecordId
                order.priority        = 'R'
                order.actionCode      = 'A'
                #order.specimenDescriptor =  'S'
                order.userField1      =  jobTicketId
                order.userField2      =  propertyRecordId
                order.reportTypes     =  'O'
                #order.specimenInstitution='LAB2'
            else:
                pass # 'testId' not found

        records = message.getRecords(encoding=opts.get('encoding', 'utf-8'))
        interface = CFileInterface(opts)
        interface.send(records)
        QtGui.QMessageBox.information(widget,
                                    u'Результат',
                                    u'Передача успешна',
                                    QtGui.QMessageBox.Close)
