# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################


from PyQt4 import QtGui
from PyQt4.QtCore import Qt, QByteArray, QDateTime, QVariant, pyqtSignature, SIGNAL, QDate

from library.crbcombobox            import CRBComboBox
from library.InDocTable             import CDateInDocTableCol, CFloatInDocTableCol, CInDocTableCol, CLocItemDelegate, CRBInDocTableCol
from library.interchange            import getDateEditValue, getDatetimeEditValue, getLineEditValue, getRBComboBoxValue, setDateEditValue, setDatetimeEditValue, setLineEditValue, setRBComboBoxValue
from library.ItemsListDialog        import CItemEditorBaseDialog
from library.PrintInfo              import CInfoContext
from library.PrintTemplates         import applyTemplate, CPrintAction, CPrintButton, getPrintTemplates
from library.Utils                  import forceDouble, forceRef, forceString, forceStringEx, toVariant, forceDate

from Events.Utils                   import syncNomenclature, getExistsNomenclature
from Registry.Utils                 import getClientInfo, formatClientBanner, getClientString
from Reports.ReportBase             import CReportBase, createTable
from Reports.ReportView             import CReportViewDialog
from Stock.InvoiceEditDialog        import CSummaryInfoModelMixin
from Stock.NomenclatureComboBox     import CNomenclatureInDocTableCol
from Stock.StockMotionBaseDialog    import CStockMotionBaseDialog, CNomenclatureItemsBaseModel
from Users.Rights                   import urAccessStockEditSupplier, urAccessStockEditSupplierPerson

from Stock.Utils import getStockMotionItemQuantityColumn, getExistsNomenclatureAmount

from Stock.Ui_ClientInvoice         import Ui_ClientInvoiceDialog
from Stock.Ui_ClientRefundInvoice   import Ui_ClientRefundInvoiceDialog


class CClientInvoiceEditDialog(CStockMotionBaseDialog, Ui_ClientInvoiceDialog):
    stockDocumentType = 4 # Списание на пациента

    def __init__(self,  parent, fromAction=False):
        CStockMotionBaseDialog.__init__(self, parent)
        self._fromAction = fromAction
        self.closeByOkButton = False
        self.addModels('Items', CItemsModel(self))
        self.addObject('btnPrint', CPrintButton(self, u'Печать'))
        self.btnPrint.setShortcut('F6')
        self.setupUi(self)
        if hasattr(self, 'cmbSupplierPerson'):
            self.cmbSupplierPerson.setSpecialityIndependents() 
        self.setWindowFlags(self.windowFlags() | Qt.WindowMaximizeButtonHint)
        self.setupDirtyCather()
        self.buttonBox.addButton(self.btnPrint, QtGui.QDialogButtonBox.ActionRole)
        templates = getPrintTemplates(self.getStockContext())
        if not templates:
            self.btnPrint.setId(-1)
        else:
            for template in templates:
                action = CPrintAction(template[0], template[1], self.btnPrint, self.btnPrint)
                self.btnPrint.addAction(action)
            self.btnPrint.menu().addSeparator()
            self.btnPrint.addAction(CPrintAction(u'Напечатать список', -1, self.btnPrint, self.btnPrint))
        self.tblItems.setItemDelegateForColumn(CItemsModel.batchColumnIndex, CBatchItemDelegate(self.tblItems))
        self.tblItems.setModel(self.modelItems)
        self.prepareItemsPopupMenu(self.tblItems)
        self.tblItems.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
        self.tblItems.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
        if fromAction:
#            self.tblItems.addPopupDelRow()
#             self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Close)
            self.edtDate.setReadOnly(True)
            self.edtTime.setReadOnly(True)
        else:
            self.setupDirtyCather()
#            self.prepareItemsPopupMenu(self.tblItems)
        self._clientId = None
        self._clientInfo = None
        self.getRightEditSupplier()
        self.connect(self.modelItems, SIGNAL('dataChanged(QModelIndex, QModelIndex)'), self.onModelItemsDataCahnged)

    def done(self, result):
        if self._fromAction:
            # Когда мы работаем с накладной на списание из редакторов действия
            # логика сохранения данных вынесена в другие объекты
            # по этом здесь имитируем закрытие кнопкой отмены,
            # но информацию о том как азкрывали диалог сохраняем
            self.closeByOkButton = result > 0
            CStockMotionBaseDialog.done(self, 0)
        else:
            CStockMotionBaseDialog.done(self, result)


    @pyqtSignature('int')
    def on_btnPrint_printByTemplate(self, templateId):
        self.actPrintMotions(templateId)


    def actPrintMotions(self, templateId):
        from Stock.StockMotionInfo import CStockMotionInfoList
        if templateId == -1:
            self.getNomenclaturePrint()
        else:
            idList = self.tblItems.model().itemIdList()
            context = CInfoContext()
            data = { 'InventoryList': CStockMotionInfoList(context, idList)}
            QtGui.qApp.call(self, applyTemplate, (self, templateId, data))


    def getStockContext(self):
        return ['InventoryList']


    def dumpParams(self, cursor):
        db = QtGui.qApp.db
        description = []
        number = self.edtNumber.text()
        if number:
            description.append(u'Номер %s'%forceString(number))
        date = QDateTime(self.edtDate.date(), self.edtTime.time())
        if date:
           description.append(u'Дата %s'%forceString(date))
        reason = self.edtReason.text()
        if reason:
            description.append(u'Основание %s'%forceString(reason))
        reasonDate = self.edtReasonDate.date()
        if reasonDate:
           description.append(u'Дата основания %s'%forceString(reasonDate))
        if hasattr(self, 'cmbSupplier'):
            supplier = self.cmbSupplier.value()
            if supplier:
                description.append(u'Получатель %s'%forceString(db.translate('OrgStructure', 'id', supplier, 'name')))
        if hasattr(self, 'cmbSupplierPerson'):
            supplierPerson = self.cmbSupplierPerson.value()
            if supplierPerson:
                description.append(u'Ответственный %s'%forceString(db.translate('vrbPersonWithSpeciality', 'id', supplierPerson, 'name')))
        if hasattr(self, 'cmbReceiver'):
            receiver = self.cmbReceiver.value()
            if receiver:
                description.append(u'Получатель %s'%forceString(db.translate('OrgStructure', 'id', receiver, 'name')))
        if hasattr(self, 'cmbReceiverPerson'):
            receiverPerson = self.cmbReceiverPerson.value()
            if receiverPerson:
                description.append(u'Ответственный %s'%forceString(db.translate('vrbPersonWithSpeciality', 'id', receiverPerson, 'name')))
        note = self.edtNote.text()
        if note:
            description.append(u'Примечания %s'%forceString(note))
        description.append(u'отчёт составлен: ' + forceString(QDateTime.currentDateTime()))
        columns = [ ('100%', [], CReportBase.AlignLeft) ]
        table = createTable(cursor, columns, headerRowCount=len(description), border=0, cellPadding=2, cellSpacing=0)
        for i, row in enumerate(description):
            table.setText(i, 0, row)
        cursor.movePosition(QtGui.QTextCursor.End)
        cursor.insertBlock()


    def getNomenclaturePrint(self):
        model = self.tblItems.model()
        doc = QtGui.QTextDocument()
        cursor = QtGui.QTextCursor(doc)
        cursor.setCharFormat(CReportBase.ReportTitle)
        cursor.insertText(self.windowTitle())
        self.dumpParams(cursor)
        cursor.insertBlock()
        cursor.setCharFormat(CReportBase.ReportBody)
        cursor.insertText(u'Пациент: %s' % (getClientString(self._clientId)))
        colWidths  = [ self.tblItems.columnWidth(i) for i in xrange(model.columnCount()-1) ]
        colWidths.insert(0,10)
        totalWidth = sum(colWidths)
        tableColumns = []
        iColNumber = False
        for iCol, colWidth in enumerate(colWidths):
            widthInPercents = str(max(1, colWidth*90/totalWidth))+'%'
            if iColNumber == False:
                tableColumns.append((widthInPercents, [u'№'], CReportBase.AlignRight))
                iColNumber = True
            tableColumns.append((widthInPercents, [forceString(model._cols[iCol].title())], CReportBase.AlignLeft))
        table = createTable(cursor, tableColumns)
        for iModelRow in xrange(model.rowCount()-1):
            iTableRow = table.addRow()
            table.setText(iTableRow, 0, iModelRow+1)
            for iModelCol in xrange(model.columnCount()):
                index = model.createIndex(iModelRow, iModelCol)
                text = forceString(model.data(index))
                table.setText(iTableRow, iModelCol+1, text)
        html = doc.toHtml(QByteArray('utf-8'))
        view = CReportViewDialog(self)
        view.setText(html)
        view.exec_()


    def getRightEditSupplier(self):
        app = QtGui.qApp
        isAdmin = app.isAdmin()
        self.cmbSupplier.setEnabled(app.userHasRight(urAccessStockEditSupplier) or isAdmin)
        self.cmbSupplierPerson.setEnabled(app.userHasRight(urAccessStockEditSupplierPerson) or isAdmin)



    def exec_(self):
        self.lblSummaryInfo.setText(self.modelItems.getSummaryInfo())
        return CStockMotionBaseDialog.exec_(self)


    def resetCounterNumber(self):
        result = not len(self.modelItems.items())
        if result:
            self.edtNumber.setText('')
        return result


    def onModelItemsDataCahnged(self,  topLeftIndex, bottomRightIndex):
        self.lblSummaryInfo.setText(self.modelItems.getSummaryInfo())


    def setRecord(self, record):
        CStockMotionBaseDialog.setRecord(self, record)
        self.setClientId(forceRef(record.value('client_id')))
        self.modelItems.loadItems(self.itemId())
        self.lblSummaryInfo.setText(self.modelItems.getSummaryInfo())
        self.setIsDirty(False)


    def getRecord(self):
        record = CStockMotionBaseDialog.getRecord(self)
        record.setValue('type', QVariant(self.stockDocumentType))
        record.setValue('client_id', QVariant(self._clientId))
        return record


    def setClientId(self, clientId):
        if self._clientId != clientId:
            self._clientId = clientId
            self.setClientInfo()


    def setFinanceId(self, financeId):
        self.modelItems.setFinanceId(financeId)


    def setClientInfo(self):
        if not self._clientInfo and self._clientId:
            self._clientInfo = getClientInfo(self._clientId, date=self.edtDate.date())
        if self._clientInfo:
            self.txtClientInfoBrowser.setHtml(formatClientBanner(self._clientInfo))


    def setStockMotionItems(self, items):
        for item in items:
            item.setValue('sum', toVariant(self.modelItems.calcSum(item)))
        self.modelItems.setItems(items)


    def setData(self, data):
        if data:
            self.setRecord(data.stockMotionRecord())
            self.setStockMotionItems(data.stockMotionItems())


    def saveInternals(self, id):
        self.modelItems.saveItems(id)


    def checkDataEntered(self):
        result = self._checkStockMotionItemsData(self.tblItems)
        result = result and self.checkItemsDataEntered()
        return result

    def checkItemsDataEntered(self):
        if hasattr(self, 'cmbSupplier'):
            supplierId = self.cmbSupplier.value()
            for row, item in enumerate(self.modelItems.items()):
                if supplierId and not self.checkNomenclatureExists(row, item, supplierId):
                    return False
            return True
        else:
            return True

    def checkNomenclatureExists(self, row, item, supplierId=None):
        supplierId = supplierId or self.cmbSupplier.value()
        financeId = forceRef(item.value('finance_id'))
        batch = forceString(item.value('batch'))
        nomenclature_id = forceRef(item.value('nomenclature_id'))
        unitId = forceRef(item.value('unit_id'))
        existsQnt = getExistsNomenclatureAmount(nomenclature_id, financeId, batch, supplierId, unitId)
        qnt = forceDouble(item.value('qnt'))
        if existsQnt < qnt:
            nomenclatureName = self.modelItems.getNomenclatureNameById(nomenclature_id)
            if existsQnt > 0:
                message = u'На складе {0} {1}, а списание на {2}'.format(existsQnt, nomenclatureName, qnt)
            else:
                message = u'На складе отсутствует %s' % nomenclatureName
            return self.checkValueMessage(message, False, self.tblItems, row, self.modelItems.qntColumnIndex)
        return True


class CClientRefundInvoiceEditDialog(Ui_ClientRefundInvoiceDialog, CClientInvoiceEditDialog):
    stockDocumentType = 5 # Возврат от пациента

    def getRightEditSupplier(self):
        pass


    def setDefaults(self):
        now = QDateTime.currentDateTime()
        self.edtDate.setDate(now.date())
        self.edtTime.setTime(now.time())
        self.cmbReceiver.setValue(QtGui.qApp.currentOrgStructureId())


    def setData(self, data):
        if data:
            self.setClientInvoiceRecord(data.stockMotionRecord())
            self.setStockMotionItems(data.stockMotionItems())


    def setClientInvoiceRecord(self, record):
        self.setClientId(forceRef(record.value('client_id')))
        setLineEditValue(   self.edtNumber,         record, 'number')
        # setDatetimeEditValue(self.edtDate, self.edtTime, record, 'date')
        setLineEditValue(   self.edtReason,    record, 'reason')
        setDateEditValue(   self.edtReasonDate,record, 'reasonDate')
        setRBComboBoxValue( self.cmbReceiver,       record, 'supplier_id')
        setRBComboBoxValue( self.cmbReceiverPerson, record, 'supplierPerson_id')
        setLineEditValue(   self.edtNote,           record, 'note')
        # дату и время нужно истановить текщие
        now = QDateTime.currentDateTime()
        self.edtDate.setDate(now.date())
        self.edtTime.setTime(now.time())


    def loadDataFromClientInvoice(self, clientInvoiceId):
        record = QtGui.qApp.db.getRecord('StockMotion', '*', clientInvoiceId)
        self.setClientInvoiceRecord(record)
        self.modelItems.loadItems(clientInvoiceId)
        self.modelItems.clearItemIds()
        self.lblSummaryInfo.setText(self.modelItems.getSummaryInfo())
        self.setIsDirty(False)


    def setRecord(self, record):
        CItemEditorBaseDialog.setRecord(self, record)
        self.setClientId(forceRef(record.value('client_id')))
        setLineEditValue(   self.edtNumber,         record, 'number')
        setDatetimeEditValue(self.edtDate, self.edtTime, record, 'date')
        setLineEditValue(   self.edtReason,    record, 'reason')
        setDateEditValue(   self.edtReasonDate,record, 'reasonDate')
        setRBComboBoxValue( self.cmbReceiver,       record, 'receiver_id')
        setRBComboBoxValue( self.cmbReceiverPerson, record, 'receiverPerson_id')
        setLineEditValue(   self.edtNote,           record, 'note')
        self.modelItems.loadItems(self.itemId())
        self.lblSummaryInfo.setText(self.modelItems.getSummaryInfo())
        self.setIsDirty(False)


    def getRecord(self):
        record = CItemEditorBaseDialog.getRecord(self)
        record.setValue('type', QVariant(self.stockDocumentType))
        record.setValue('client_id', QVariant(self._clientId))
        getLineEditValue(   self.edtNumber,         record, 'number')
        getDatetimeEditValue(self.edtDate, self.edtTime, record, 'date', True)
        getLineEditValue(   self.edtReason,    record, 'reason')
        getDateEditValue(   self.edtReasonDate,record, 'reasonDate')
        getRBComboBoxValue( self.cmbReceiver,       record, 'receiver_id')
        getRBComboBoxValue( self.cmbReceiverPerson, record, 'receiverPerson_id')
        getLineEditValue(   self.edtNote,           record, 'note')
        return record


class CClientInvoiceReservationEditDialog(CClientInvoiceEditDialog):
    stockDocumentType = 6  # Резервирвоание на пациента


# ########################################################################

class CBatchItemDelegate(CLocItemDelegate):
    def createEditor(self, parent, option, index):
        column = index.column()
        row = index.row()
        editor = index.model().createBatchEditor(row, parent)
        self.connect(editor, SIGNAL('commit()'), self.emitCommitData)
        self.connect(editor, SIGNAL('editingFinished()'), self.commitAndCloseEditor)
        self.editor   = editor
        self.row      = row
        self.rowcount = index.model().rowCount(None)
        self.column   = column
        return editor


    def setEditorData(self, editor, index):
        if editor is not None:
            row    = index.row()
            model  = index.model()
            if row < len(model.items()):
                record = model.items()[row]
            else:
                record = model.getEmptyRecord()
            model.setBatchEditorData(row, editor, model.data(index, Qt.EditRole), record)

    def setModelData(self, editor, model, index):
        if editor is not None:
            model.setData(index, index.model().getBatchEditorData(index.row(), editor))


class CItemsModel(CNomenclatureItemsBaseModel, CSummaryInfoModelMixin):
    nomenclatureColumnIndex = 0
    batchColumnIndex = 1
    shelfTimeColumnIndex = 2
    financeColumnIndex = 3
    qntColumnIndex = 4
    unitColumnIndex = 4
    sumColumnIndex = 6

    class CBatchCol(CInDocTableCol):
        def __init__(self):
            CInDocTableCol.__init__(self, u'Партия', 'batch', 16)


    class CExistsCol(CInDocTableCol):
        def __init__(self):
            CInDocTableCol.__init__(self, u'Остаток', 'existsColumn', 10)
            self._cache = {}

        def toString(self, val, record):
            return QVariant(
                self.getExistsNomenclatureAmount(
                    forceRef(record.value('nomenclature_id')), forceRef(record.value('finance_id')),
                    forceString(record.value('batch')), forceRef(record.value('unit_id'))
                )
            )

        def getExistsNomenclatureAmount(self, nomenclatureId, financeId=None, batch=None, unitId=None):
            key = (nomenclatureId, financeId, batch, unitId)
            if key  not in self._cache:
                self._cache[key] = getExistsNomenclatureAmount(nomenclatureId, financeId, batch, unitId=unitId)
            return self._cache[key]

    class CSumCol(CFloatInDocTableCol):
        def _toString(self, value):
            if value.isNull():
                return None
            return format(forceDouble(value), '.2f')

    class CLocNomenclatureCol(CNomenclatureInDocTableCol):
        def createEditor(self, parent):
            editor = CNomenclatureInDocTableCol.createEditor(self, parent)
            editor.setOrgStructureId(QtGui.qApp.currentOrgStructureId())
            editor.setOnlyExists()
            return editor

    def __init__(self, parent, showExists=False):
        self.priceCache = parent
        CNomenclatureItemsBaseModel.__init__(self, 'StockMotion_Item', 'id', 'master_id', parent)
        self._unitColumn = CRBInDocTableCol(u'Ед.Учета', 'unit_id', 12, 'rbUnit', addNone=False)
        self._nomenclatureColumn = CItemsModel.CLocNomenclatureCol(u'ЛСиИМН', 'nomenclature_id', 50, showFields = CRBComboBox.showName)
        self.addCol(self._nomenclatureColumn)
        self.batchCol = CItemsModel.CBatchCol()
        self.addCol(self.batchCol)
        self.addCol(CDateInDocTableCol( u'Годен до', 'shelfTime', 12, canBeEmpty=True))
        self.addCol(CRBInDocTableCol(    u'Тип финансирования', 'finance_id', 15, 'rbFinance'))
        self.addCol(getStockMotionItemQuantityColumn( u'Кол-во', 'qnt', 12))
        self.addCol(self._unitColumn)
        self.addCol(CItemsModel.CSumCol( u'Сумма', 'sum', 12))
        self.existsCol = CItemsModel.CExistsCol().setReadOnly()
        self.addExtCol(self.existsCol, QVariant.Double)
        self._batchCache = {}
        self._financeCache = {}
        self._cachRow2CommonSum = {}
        self.applyHelpers()
        self._financeId = None
    
    def setFinanceId(self, financeId):
        self._financeId = financeId

    def getNomenclatureNameById(self, nomenclatureId):
        return forceString(self._nomenclatureColumn.toString(nomenclatureId, None))


    def _getBatchEditorItems(self, nomenclatureId, financeId, batch):
        if nomenclatureId not in self._batchCache:
            result = [u'']
            nomenclatureAmount = self.existsCol.getExistsNomenclatureAmount(nomenclatureId, financeId, batch)
            if nomenclatureAmount <= 0:
                return result

            db = QtGui.qApp.db
            table = db.table('StockMotion_Item')

            stmt = 'SELECT DISTINCT `batch` FROM StockMotion_Item WHERE %s'%table['nomenclature_id'].eq(nomenclatureId)
            query = db.query(stmt)
            while query.next():
                batch = forceStringEx(query.record().value('batch'))
                if batch not in result:
                    result.append(batch)
            self._batchCache[nomenclatureId] = result
        return self._batchCache[nomenclatureId]


    def _getBatchEditorItemsByRow(self, row):
        if 0 <= row < len(self._items):
            item = self._items[row]
            nomenclatureId = forceRef(item.value('nomenclature_id'))
            financeId = forceRef(item.value('finance_id'))
            batch = forceString(item.value('batch'))
            return self._getBatchEditorItems(nomenclatureId, financeId, batch)
        return [u'']

    def _setSuitableBatchValue(self, item, nomenclatureId, oldNomenclatureId):
        if nomenclatureId != oldNomenclatureId:
            financeId = forceRef(item.value('finance_id'))
            batch = forceString(item.value('batch'))
            for batchValue in self._getBatchEditorItems(nomenclatureId, financeId, batch):
                if batchValue:
                    item.setValue('batch', QVariant(batchValue))
                    return True
            item.setValue('batch', QVariant(''))
            return True
        return False

    def _getNomenclatureFinanceIdList(self, nomenclatureId, financeId):
        if (nomenclatureId, financeId) not in self._financeCache:
            result = []
            db = QtGui.qApp.db
            table = db.table('StockMotion_Item')
            cond = [table['nomenclature_id'].eq(nomenclatureId), 
                        table['finance_id'].eq(financeId)]
            stmt = 'SELECT DISTINCT `finance_id` FROM StockMotion_Item WHERE %s'%cond
            query = db.query(stmt)
            while query.next():
                financeId = forceRef(query.record().value('finance_id'))
                if financeId and financeId not in result:
                    result.append(financeId)
            self._financeCache[nomenclatureId, financeId] = result
        return self._financeCache[nomenclatureId, financeId]


    def _setSuitableFinanceValue(self, item, nomenclatureId, oldNomenclatureId, financeId):
        if nomenclatureId != oldNomenclatureId:
            for financeId in self._getNomenclatureFinanceIdList(nomenclatureId, financeId):
                if financeId:
                    item.setValue('finance_id', QVariant(financeId))
                    return True
            item.setValue('finance_id', QVariant(None))
            return True
        return False

    def createBatchEditor(self, row, parent):
        editor = QtGui.QComboBox(parent)
        editor.addItems(self._getBatchEditorItemsByRow(row))
        return editor

    def setBatchEditorData(self, row, editor, value, record):
        index = editor.findText(forceString(value), Qt.MatchFixedString)
        if index < 0:
            for locIndex in xrange(editor.count()):
                if forceStringEx(editor.itemText(locIndex)):
                    editor.setCurrentIndex(locIndex)
                    return
            editor.setCurrentIndex(0)
        editor.setCurrentIndex(index)

    def getBatchEditorData(self, row, editor):
        return toVariant(editor.currentText())


    def getEmptyRecord(self):
        record = CNomenclatureItemsBaseModel.getEmptyRecord(self)
        record.setValue('qnt', QVariant(1))
        return record


    def setData(self, index, value, role=Qt.EditRole):
        if not index.isValid():
            return False

        column = index.column()
        row = index.row()

        if role == Qt.EditRole:
            if column == CItemsModel.nomenclatureColumnIndex:
                if 0 <= row < len(self._items):
                    oldNomenclatureId = forceRef(self._items[row].value('nomenclature_id'))
                else:
                    oldNomenclatureId = None

                result = CNomenclatureItemsBaseModel.setData(self, index, value, role)
                if not result:
                    return False

                item = self._items[row]

                nomenclatureId = forceRef(self._items[row].value('nomenclature_id'))
                financeId = forceRef(item.value('finance_id'))
                batch = forceString(item.value('batch'))
                if len(self._getBatchEditorItems(nomenclatureId, financeId, batch)) == 2: # два варианта: ничего и еще что-то
                    self._setSuitableBatchValue(item, nomenclatureId, oldNomenclatureId)
                else:
                    if nomenclatureId != oldNomenclatureId:
                        item.setValue('batch', QVariant(''))

                if len(self._getNomenclatureFinanceIdList(nomenclatureId, self._financeId)) == 1:
                    self._setSuitableFinanceValue(item, nomenclatureId, oldNomenclatureId, self._financeId)
                else:
                    if nomenclatureId != oldNomenclatureId:
                        item.setValue('finance_id', QVariant(None))
                if oldNomenclatureId != nomenclatureId:
                    item.setValue('unit_id', toVariant(self.getDefaultClientUnitId(nomenclatureId)))

                self._countSum(row, item)

                self.emitRowChanged(row)

            elif column == self.getColIndex('unit_id'):
                if not (0 <= row < len(self._items)):
                    return False
                item = self._items[row]
                oldUnitId = forceRef(item.value('unit_id'))
                result = CNomenclatureItemsBaseModel.setData(self, index, value, role)
                if result:
                    newUnitId = forceRef(item.value('unit_id'))
                    self._applySumRatio(item, oldUnitId, newUnitId)
                    self.emitRowChanged(row)
            elif column == CItemsModel.qntColumnIndex:
                result = CNomenclatureItemsBaseModel.setData(self, index, value, role)
                if not result:
                    return False
                item = self._items[row]
                item.setValue('sum', toVariant(self.calcSum(item)))
                self._countSum(row, item)
                self.emitRowChanged(row)
            elif column == CItemsModel.batchColumnIndex:
                result = CNomenclatureItemsBaseModel.setData(self, index, value, role)
                if not result:
                    return False
                item = self._items[row]
                item.setValue('shelfTime', toVariant(self.getShelfTime(item)))
                self.emitRowChanged(row)

            elif column in (CItemsModel.batchColumnIndex, CItemsModel.financeColumnIndex, CItemsModel.qntColumnIndex):
                result = CNomenclatureItemsBaseModel.setData(self, index, value, role)
                if not result:
                    return False
                item = self._items[row]
                self._countSum(row, item)
                self.emitRowChanged(row)
            else:
                result = CNomenclatureItemsBaseModel.setData(self, index, value, role)
        else:
            result = CNomenclatureItemsBaseModel.setData(self, index, value, role)

        return result


    def _countSum(self, row, item):
        nomenclatureId = forceRef(item.value('nomenclature_id'))
        _filter = {
            'batch': forceString(item.value('batch')),
            'financeId': forceRef(item.value('finance_id'))
        }

        existsData = getExistsNomenclature([nomenclatureId], _filter)
        syncNomenclature(existsData, item, unset=True, translateExistsQnt=True)

    def getShelfTime(self, item):
        shelfTime = QDate()
        db = QtGui.qApp.db
        table = db.table('StockMotion_Item')
        cond = [table['nomenclature_id'].eq(forceRef(item.value('nomenclature_id'))), 
                    table['batch'].eq(forceString(item.value('batch'))), 
                    table['finance_id'].eq(forceRef(item.value('finance_id')))]

        stmt = 'SELECT DISTINCT `shelfTime` FROM StockMotion_Item WHERE %(cond)s'% {'cond': db.joinAnd(cond),}
        query = db.query(stmt)
        while query.next():
            shelfTime = forceDate(query.record().value('shelfTime'))

        return shelfTime

    def flags(self, index):
        column = index.column()
        row = index.row()
        if column == CItemsModel.batchColumnIndex:
            if 0 <= row < len(self._items):
                item = self._items[row]
                if not forceRef(item.value('nomenclature_id')):
                    return Qt.ItemIsSelectable | Qt.ItemIsEnabled
            else:
                return Qt.ItemIsSelectable | Qt.ItemIsEnabled
        return CNomenclatureItemsBaseModel.flags(self, index)

