# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui
from PyQt4.QtCore import Qt, QByteArray, QDateTime, pyqtSignature

from library.crbcombobox                   import CRBComboBox
from library.InDocTable                    import CDateInDocTableCol, CFloatInDocTableCol, CInDocTableCol, CRBInDocTableCol
from library.interchange                   import getRBComboBoxValue
from library.PrintInfo                     import CInfoContext
from library.PrintTemplates                import applyTemplate, CPrintAction, CPrintButton, getPrintTemplates
from library.Utils                         import forceDouble, forceString, forceRef, toVariant

from Reports.ReportBase                    import CReportBase, createTable
from Reports.ReportView                    import CReportViewDialog
from Stock.NomenclatureComboBox            import CNomenclatureInDocTableCol
from Stock.StockMotionBaseDialog           import CStockMotionBaseDialog, CStockMotionItemsCopyPasteMixin, CNomenclatureItemsBaseModel

from Stock.Utils import getStockMotionItemQuantityColumn

from Stock.Ui_Inventory                    import Ui_InventoryDialog


class CInventoryEditDialog(CStockMotionBaseDialog, CStockMotionItemsCopyPasteMixin, Ui_InventoryDialog):
    stockDocumentType = 1 #Инвентаризация
    
    def __init__(self,  parent):
        CStockMotionBaseDialog.__init__(self, parent)
        CStockMotionItemsCopyPasteMixin.__init__(self, parent)
        self.addModels('Items', CItemsModel(self))
        self.addObject('actDuplicate', QtGui.QAction(u'Дублировать', self))
        self.addObject('btnFill', QtGui.QPushButton(u'Заполнить', self))
        self.btnFill.setShortcut('F9')
        self.addObject('btnPrint', CPrintButton(self, u'Печать'))
        self.btnPrint.setShortcut('F6')
        self.setupUi(self)
        self.cmbSupplierPerson.setSpecialityIndependents()
        self.setWindowFlags(self.windowFlags() | Qt.WindowMaximizeButtonHint)
        self.setupDirtyCather()
        self.buttonBox.addButton(self.btnPrint, QtGui.QDialogButtonBox.ActionRole)
        templates = getPrintTemplates(self.getStockContext())
        if not templates:
            self.btnPrint.setId(-1)
        else:
            for template in templates:
                action = CPrintAction(template[0], template[1], self.btnPrint, self.btnPrint)
                self.btnPrint.addAction(action)
            self.btnPrint.menu().addSeparator()
            self.btnPrint.addAction(CPrintAction(u'Напечатать список', -1, self.btnPrint, self.btnPrint))
        self.tblItems.setModel(self.modelItems)
        self.prepareItemsPopupMenu(self.tblItems)
        self.tblItems.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
        self.tblItems.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
        self.buttonBox.addButton(self.btnFill, QtGui.QDialogButtonBox.ActionRole)
        self._initView()


    @pyqtSignature('int')
    def on_btnPrint_printByTemplate(self, templateId):
        self.actPrintMotions(templateId)


    def actPrintMotions(self, templateId):
        from Stock.StockMotionInfo import CStockMotionInfoList
        if templateId == -1:
            self.getNomenclaturePrint()
        else:
            idList = self.tblItems.model().itemIdList()
            context = CInfoContext()
            data = { 'InventoryList': CStockMotionInfoList(context, idList)}
            QtGui.qApp.call(self, applyTemplate, (self, templateId, data))


    def getStockContext(self):
        return ['InventoryList']


    def dumpParams(self, cursor):
        db = QtGui.qApp.db
        description = []
        number = self.edtNumber.text()
        if number:
            description.append(u'Номер %s'%forceString(number))
        date = QDateTime(self.edtDate.date(), self.edtTime.time())
        if date:
           description.append(u'Дата %s'%forceString(date))
        reason = self.edtReason.text()
        if reason:
            description.append(u'Основание %s'%forceString(reason))
        reasonDate = self.edtReasonDate.date()
        if reasonDate:
           description.append(u'Дата основания %s'%forceString(reasonDate))
        supplier = self.cmbSupplier.value()
        if supplier:
            description.append(u'Подразделение %s'%forceString(db.translate('OrgStructure', 'id', supplier, 'name')))
        supplierPerson = self.cmbSupplierPerson.value()
        if supplierPerson:
            description.append(u'Ответственный %s'%forceString(db.translate('vrbPersonWithSpeciality', 'id', supplierPerson, 'name')))
        note = self.edtNote.text()
        if note:
            description.append(u'Примечания %s'%forceString(note))
        description.append(u'отчёт составлен: ' + forceString(QDateTime.currentDateTime()))
        columns = [ ('100%', [], CReportBase.AlignLeft) ]
        table = createTable(cursor, columns, headerRowCount=len(description), border=0, cellPadding=2, cellSpacing=0)
        for i, row in enumerate(description):
            table.setText(i, 0, row)
        cursor.movePosition(QtGui.QTextCursor.End)
        cursor.insertBlock()


    def getNomenclaturePrint(self):
#        title = self.windowTitle()
        model = self.tblItems.model()
        doc = QtGui.QTextDocument()
        cursor = QtGui.QTextCursor(doc)
        cursor.setCharFormat(CReportBase.ReportTitle)
        cursor.insertText(self.windowTitle())
        self.dumpParams(cursor)
        cursor.insertBlock()
        cursor.setCharFormat(CReportBase.ReportBody)
        colWidths  = [ self.tblItems.columnWidth(i) for i in xrange(model.columnCount()-1) ]
        colWidths.insert(0,10)
        totalWidth = sum(colWidths)
        tableColumns = []
        iColNumber = False
        for iCol, colWidth in enumerate(colWidths):
            widthInPercents = str(max(1, colWidth*90/totalWidth))+'%'
            if iColNumber == False:
                tableColumns.append((widthInPercents, [u'№'], CReportBase.AlignRight))
                iColNumber = True
            tableColumns.append((widthInPercents, [forceString(model._cols[iCol].title())], CReportBase.AlignLeft))
        table = createTable(cursor, tableColumns)
        for iModelRow in xrange(model.rowCount()-1):
            iTableRow = table.addRow()
            table.setText(iTableRow, 0, iModelRow+1)
            for iModelCol in xrange(model.columnCount()):
                index = model.createIndex(iModelRow, iModelCol)
                text = forceString(model.data(index))
                table.setText(iTableRow, iModelCol+1, text)
        html = doc.toHtml(QByteArray('utf-8'))
        view = CReportViewDialog(self)
        view.setText(html)
        view.exec_()


    def setDefaults(self):
        CStockMotionBaseDialog.setDefaults(self)


    def setRecord(self, record):
        CStockMotionBaseDialog.setRecord(self, record)
        self.modelItems.loadItems(self.itemId())
        self.setIsDirty(False)


    def getRecord(self):
        record = CStockMotionBaseDialog.getRecord(self)
        getRBComboBoxValue( self.cmbSupplier, record, 'receiver_id')
        getRBComboBoxValue( self.cmbSupplierPerson, record, 'receiverPerson_id')
        record.setValue('type', 1)
        return record


    def saveInternals(self, id):
        self.modelItems.saveItems(id)


    def checkDataEntered(self):
        result = self._checkStockMotionItemsData(self.tblItems)
#        result = result and (self.cmbPurpose.value() or self.checkInputMessage(u'назначение', False, self.cmbPurpose))
#        result = result and self.checkItemsDataEntered()
        return result


#    def checkItemsDataEntered(self):
#        for row, item in enumerate(self.modelItems.items()):
#            if not self.checkItemDataEntered(row, item):
#                return False
#        return True
#
#
#    def checkItemDataEntered(self, row, item):
#        nomenclatureId = forceString(item.value('nomenclature_id'))
#        qnt            = forceDouble(item.value('qnt'))
#        result = nomenclatureId or self.checkInputMessage(u'лекарственное средство или изделие медицинского назначения', False, self.tblProperties, row, 0)
#        result = result and (qnt or self.checkInputMessage(u'количество', False, self.tblProperties, row, 2))
#        return result

    @pyqtSignature('')
    def on_cmbSupplier_valueChanged(self):
        self.btnFill.setEnabled(self.cmbSupplier.value() is not None)


    @pyqtSignature('')
    def on_btnFill_clicked(self):
        orgStructureId = self.cmbSupplier.value()
        if orgStructureId:
            self.modelItems.fill(orgStructureId)


class CItemsModel(CNomenclatureItemsBaseModel):
    class CLocFloatCol(CFloatInDocTableCol):
        def _toString(self, value):
            if value.isNull():
                return None
            return format(forceDouble(value), '.2f')

    def __init__(self, parent):
        CNomenclatureItemsBaseModel.__init__(self, 'StockMotion_Item', 'id', 'master_id', parent)
        self.priceCache = parent
        self._unitColumn = CRBInDocTableCol(u'Ед.Учета', 'unit_id', 12, 'rbUnit', addNone=False)
        self.addCol(
            CNomenclatureInDocTableCol(
                u'ЛСиИМН', 'nomenclature_id', 50, showFields=CRBComboBox.showName, showLfForm=True
            )
        )
        self.addCol(CInDocTableCol( u'Партия', 'batch', 16))
        self.addCol(CDateInDocTableCol( u'Годен до', 'shelfTime', 12, canBeEmpty=True))
        self.addCol(CRBInDocTableCol(    u'Тип финансирования', 'finance_id', 15, 'rbFinance'))
        self.addCol(getStockMotionItemQuantityColumn( u'Кол-во по документам', 'oldQnt', 12))
        self.addCol(CItemsModel.CLocFloatCol( u'Сумма по документам', 'oldSum', 12))
        self.addCol(self._unitColumn)
        self.addCol(getStockMotionItemQuantityColumn( u'Фактическое кол-во', 'qnt', 12))
        self.addCol(CItemsModel.CLocFloatCol( u'Фактическая сумма', 'sum', 12))


    def setData(self, index, value, role=Qt.EditRole):
        if not index.isValid():
            return False

        if role == Qt.EditRole:

            col = index.column()
            row = index.row()

            if col == self.getColIndex('nomenclature_id'):
                if 0 <= row < len(self._items):
                    oldNomenclatureId = forceRef(self._items[row].value('nomenclature_id'))
                else:
                    oldNomenclatureId = None
                result = CNomenclatureItemsBaseModel.setData(self, index, value, role)
                if result:
                    item = self._items[row]
                    nomenclatureId = forceRef(item.value('nomenclature_id'))
                    if oldNomenclatureId != nomenclatureId:
                        unitId = self.getDefaultStockUnitId(nomenclatureId)
                        item.setValue('unit_id', toVariant(unitId))
                        self.emitRowChanged(row)
                return result
            elif col == self.getColIndex('unit_id'):
                if not (0 <= row < len(self._items)):
                    return False

                item = self._items[row]
                oldUnitId = forceRef(item.value('unit_id'))
                result = CNomenclatureItemsBaseModel.setData(self, index, value, role)
                if result:
                    newUnitId = forceRef(item.value('unit_id'))
                    self._applySumRatio(item, oldUnitId, newUnitId)
                    self.emitRowChanged(row)
            elif col == self.getColIndex('qnt'):
                result = CNomenclatureItemsBaseModel.setData(self, index, value, role)
                if result:
                    item = self._items[row]
                    item.setValue('sum', toVariant(self.calcSum(item, qntCol='qnt')))
            elif col == self.getColIndex('oldQnt'):
                result = CNomenclatureItemsBaseModel.setData(self, index, value, role)
                if result:
                    item = self._items[row]
                    item.setValue('oldSum', toVariant(self.calcSum(item, qntCol='oldQnt')))
            else:
                return CNomenclatureItemsBaseModel.setData(self, index, value, role)
        else:
            return CNomenclatureItemsBaseModel.setData(self, index, value, role)


    def _applySumRatio(self, item, oldUnitId, newUnitId, sumCol='sum', qntCol='qnt'):
        CNomenclatureItemsBaseModel._applySumRatio(self, item, oldUnitId, newUnitId, sumCol='sum', qntCol='qnt')
        CNomenclatureItemsBaseModel._applySumRatio(self, item, oldUnitId, newUnitId, sumCol='oldSum', qntCol='oldQnt')


    def fill(self, orgStructureId):
        stmt = u'''
SELECT nomenclature_id,
       finance_id,
       batch,
       shelfTime,
       sum(qnt) AS `qnt`,
       sum(`sum`) AS `sum`
FROM
    (
    SELECT debNomenclature_id AS nomenclature_id,
           debFinance_id      AS finance_id,
           batch,
           shelfTime,
           sum(qnt)           AS `qnt`,
           sum(`sum`)         AS `sum`
    FROM StockTrans
    WHERE debOrgStructure_id = %d
    GROUP BY debOrgStructure_id, debNomenclature_id, debFinance_id, batch
    UNION
    SELECT creNomenclature_id AS nomenclature_id,
           creFinance_id      AS finance_id,
           batch,
           shelfTime,
           -sum(qnt)          AS `qnt`,
           -sum(`sum`)        AS `sum`
    FROM StockTrans
    WHERE creOrgStructure_id = %d
    GROUP BY creOrgStructure_id, creNomenclature_id, creFinance_id, batch, shelfTime
    ) AS T
    LEFT JOIN rbNomenclature ON rbNomenclature.id = nomenclature_id
GROUP BY nomenclature_id, finance_id, batch, shelfTime
HAVING qnt != 0 OR `sum` != 0
ORDER BY rbNomenclature.name''' % (orgStructureId, orgStructureId)
        query = QtGui.qApp.db.query(stmt)
        while query.next():
            record = query.record()
            nomenclatureId = forceRef(record.value('nomenclature_id'))
            unitId = self.getDefaultStockUnitId(nomenclatureId)
            myItem = self.getEmptyRecord()
            myItem.setValue('nomenclature_id', toVariant(nomenclatureId))
            myItem.setValue('unit_id',         toVariant(unitId))
            myItem.setValue('finance_id',      record.value('finance_id'))
            myItem.setValue('batch',           record.value('batch'))
            myItem.setValue('shelfTime',       record.value('shelfTime'))
            myItem.setValue('qnt',             record.value('qnt'))
            myItem.setValue('sum',             record.value('sum'))
            myItem.setValue('oldQnt',          record.value('qnt'))
            myItem.setValue('oldSum',          record.value('sum'))
            self.items().append(myItem)
        self.reset()
