# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import sip

from PyQt4 import QtGui, QtSql
from PyQt4.QtCore import Qt, QByteArray, QDateTime, QModelIndex, QVariant, pyqtSignature, SIGNAL, QDate

from library.crbcombobox                   import CRBComboBox
from library.InDocTable                    import CDateInDocTableCol, CFloatInDocTableCol, CInDocTableCol, CRBInDocTableCol
from library.interchange                   import getRBComboBoxValue, setRBComboBoxValue, getLineEditValue, setLineEditValue
from library.PrintInfo                     import CInfoContext
from library.PrintTemplates                import applyTemplate, CPrintAction, CPrintButton, getPrintTemplates
from library.Utils                         import forceDouble, forceRef, forceString, toVariant, forceInt, forceDate

from RefBooks.RBNomenclature               import CRBNomenclatureList
from Reports.ReportBase                    import CReportBase, createTable
from Reports.ReportView                    import CReportViewDialog
from Stock.NomenclatureComboBox            import CNomenclatureInDocTableCol, CToolButtonInDocTableCol
from Stock.StockMotionBaseDialog           import CStockMotionBaseDialog, CStockMotionItemsCopyPasteMixin, CNomenclatureItemsBaseModel

from Stock.Utils import getStockMotionItemQuantityColumn, getExistsNomenclatureAmount
from Orgs.Utils                       import getOrgStructureDescendants

from Stock.Ui_Invoice                      import Ui_InvoiceDialog


class CInvoiceEditDialog(CStockMotionBaseDialog, CStockMotionItemsCopyPasteMixin, Ui_InvoiceDialog):
    stockDocumentType = 0 #Накладная
    
    def __init__(self,  parent):
        CStockMotionBaseDialog.__init__(self, parent)
        CStockMotionItemsCopyPasteMixin.__init__(self, parent)
        self.addModels('Items', CItemsModel(self))
        self.addObject('btnPrint', CPrintButton(self, u'Печать'))
        self.btnPrint.setShortcut('F6')
        self.setupUi(self)
        self.cmbReceiverPerson.setSpecialityIndependents()
        self.cmbSupplierPerson.setSpecialityIndependents()
        self.cmbSupplierOrg.setFilter('isSupplier = 1')
        self.edtSupplierOrgPerson.setEnabled(False)
        self.setWindowFlags(self.windowFlags() | Qt.WindowMaximizeButtonHint)
        self.setupDirtyCather()
        self.buttonBox.addButton(self.btnPrint, QtGui.QDialogButtonBox.ActionRole)
        templates = getPrintTemplates(self.getStockContext())
        if not templates:
            self.btnPrint.setId(-1)
        else:
            for template in templates:
                action = CPrintAction(template[0], template[1], self.btnPrint, self.btnPrint)
                self.btnPrint.addAction(action)
            self.btnPrint.menu().addSeparator()
            self.btnPrint.addAction(CPrintAction(u'Напечатать список', -1, self.btnPrint, self.btnPrint))
        self.tblItems.setModel(self.modelItems)
        self.prepareItemsPopupMenu(self.tblItems)
        self.tblItems.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
        self.tblItems.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
        self.requisitionIdList = None
        self.connect(self.modelItems, SIGNAL('dataChanged(QModelIndex, QModelIndex)'), self.onModelItemsDataCahnged)
        self._initView()


    def getStockContext(self):
        return ['InvoiceCreate']


    @pyqtSignature('int')
    def on_btnPrint_printByTemplate(self, templateId):
        self.actPrintMotions(templateId)


    def actPrintMotions(self, templateId):
        from Stock.StockMotionInfo import CStockMotionInfoList
        if templateId == -1:
            self.getNomenclaturePrint()
        else:
            idList = self.tblItems.model().itemIdList()
            context = CInfoContext()
            data = { 'invoicesList': CStockMotionInfoList(context, idList)}
            QtGui.qApp.call(self, applyTemplate, (self, templateId, data))


    def dumpParams(self, cursor):
        db = QtGui.qApp.db
        description = []
        number = self.edtNumber.text()
        if number:
            description.append(u'Номер %s'%forceString(number))
        date = QDateTime(self.edtDate.date(), self.edtTime.time())
        if date:
           description.append(u'Дата %s'%forceString(date))
        reason = self.edtReason.text()
        if reason:
            description.append(u'Основание %s'%forceString(reason))
        reasonDate = self.edtReasonDate.date()
        if reasonDate:
           description.append(u'Дата основания %s'%forceString(reasonDate))
        supplier = self.cmbSupplier.value()
        if supplier:
            description.append(u'Поставщик %s'%forceString(db.translate('OrgStructure', 'id', supplier, 'name')))
        supplierPerson = self.cmbSupplierPerson.value()
        if supplierPerson:
            description.append(u'Ответственный %s'%forceString(db.translate('vrbPersonWithSpeciality', 'id', supplierPerson, 'name')))
        receiver = self.cmbReceiver.value()
        if receiver:
            description.append(u'Получатель %s'%forceString(db.translate('OrgStructure', 'id', receiver, 'name')))
        receiverPerson = self.cmbReceiverPerson.value()
        if receiverPerson:
            description.append(u'Ответственный %s'%forceString(db.translate('vrbPersonWithSpeciality', 'id', receiverPerson, 'name')))
        note = self.edtNote.text()
        if note:
            description.append(u'Примечания %s'%forceString(note))
        description.append(u'отчёт составлен: ' + forceString(QDateTime.currentDateTime()))
        columns = [ ('100%', [], CReportBase.AlignLeft) ]
        table = createTable(cursor, columns, headerRowCount=len(description), border=0, cellPadding=2, cellSpacing=0)
        for i, row in enumerate(description):
            table.setText(i, 0, row)
        cursor.movePosition(QtGui.QTextCursor.End)
        cursor.insertBlock()


    def getNomenclaturePrint(self):
        model = self.tblItems.model()
        doc = QtGui.QTextDocument()
        cursor = QtGui.QTextCursor(doc)
        cursor.setCharFormat(CReportBase.ReportTitle)
        cursor.insertText(self.windowTitle())
        self.dumpParams(cursor)
        cursor.insertBlock()
        cursor.setCharFormat(CReportBase.ReportBody)
        colWidths  = [ self.tblItems.columnWidth(i) for i in xrange(model.columnCount()-1) ]
        colWidths.insert(0,10)
        totalWidth = sum(colWidths)
        tableColumns = []
        iColNumber = False
        for iCol, colWidth in enumerate(colWidths):
            widthInPercents = str(max(1, colWidth*90/totalWidth))+'%'
            if iColNumber == False:
                tableColumns.append((widthInPercents, [u'№'], CReportBase.AlignRight))
                iColNumber = True
            tableColumns.append((widthInPercents, [forceString(model._cols[iCol].title())], CReportBase.AlignLeft))
        table = createTable(cursor, tableColumns)
        for iModelRow in xrange(model.rowCount()-1):
            iTableRow = table.addRow()
            table.setText(iTableRow, 0, iModelRow+1)
            for iModelCol in xrange(model.columnCount()):
                index = model.createIndex(iModelRow, iModelCol)
                text = forceString(model.data(index))
                table.setText(iTableRow, iModelCol+1, text)
        html = doc.toHtml(QByteArray('utf-8'))
        view = CReportViewDialog(self)
        view.setText(html)
        view.exec_()


    def onModelItemsDataCahnged(self,  topLeftIndex, bottomRightIndex):
        self.lblSummaryInfo.setText(self.modelItems.getSummaryInfo())


    def setDefaults(self):
        now = QDateTime.currentDateTime()
        self.edtDate.setDate(now.date())
        self.edtTime.setTime(now.time())
        self.cmbReceiver.setValue(QtGui.qApp.currentOrgStructureId())


    def setRequsitions(self, requisitionIdList):
        if requisitionIdList:
            requisitionId = requisitionIdList[0]
            self.cmbReceiver.setValue(forceRef(QtGui.qApp.db.translate('StockRequisition', 'id', requisitionId, 'recipient_id')))
            number = forceString(QtGui.qApp.db.translate('StockRequisition', 'id', requisitionId, 'number'))
            stockMotionsCount = forceInt(QtGui.qApp.db.translate('StockRequisition', 'id', requisitionId, 'stockMotionsCount'))
            idx = forceString(stockMotionsCount + 1)
            self.edtNumber.setText(number + '/' + idx)
            self.cmbSupplier.setValue(QtGui.qApp.currentOrgStructureId())
            self.modelItems.setRequsitions(requisitionIdList)
            self.requisitionIdList = requisitionIdList


    def setRecord(self, record):
        CStockMotionBaseDialog.setRecord(self, record)
        setRBComboBoxValue( self.cmbReceiver, record, 'receiver_id')
        setRBComboBoxValue( self.cmbReceiverPerson, record, 'receiverPerson_id')
        setRBComboBoxValue(self.cmbSupplierOrg, record, 'supplierOrg_id')
        setLineEditValue(self.edtSupplierOrgPerson, record, 'supplierOrgPerson')
        self.modelItems.loadItems(self.itemId())
        self.lblSummaryInfo.setText(self.modelItems.getSummaryInfo())
        self.setIsDirty(False)


    def getRecord(self):
        record = CStockMotionBaseDialog.getRecord(self)
        getRBComboBoxValue( self.cmbReceiver, record, 'receiver_id')
        getRBComboBoxValue( self.cmbReceiverPerson, record, 'receiverPerson_id')
        getRBComboBoxValue(self.cmbSupplierOrg, record, 'supplierOrg_id')
        getLineEditValue(self.edtSupplierOrgPerson, record, 'supplierOrgPerson')
        record.setValue('type', 0)
        return record


    def saveInternals(self, id):
        self.modelItems.saveItems(id)
        if self.requisitionIdList:
            self.updateRequisition()


    def updateRequisition(self):
        db = QtGui.qApp.db
        tableSR = db.table('StockRequisition')
        tableSRI = db.table('StockRequisition_Item')
        table = tableSRI.leftJoin(tableSR, tableSR['id'].eq(tableSRI['master_id']))
        requisitionItems = db.getRecordList(table,
                           'StockRequisition_Item.*',
                           [tableSRI['master_id'].inlist(self.requisitionIdList),
                            tableSR['recipient_id'].eq(self.cmbReceiver.value())
                           ],
                           'master_id, idx')

        fndict = self.modelItems.getDataAsFNDict()
        preciseGroupDict = {}
        for item in requisitionItems:
            masterId = forceInt(item.value('master_id'))
            nomenclatureId = forceRef(item.value('nomenclature_id'))
            financeId = forceRef(item.value('finance_id'))
            #qnt = forceDouble(item.value('qnt'))
            satisfiedQnt = forceDouble(item.value('satisfiedQnt'))
            oldSatisfiedQnt = satisfiedQnt
            ndict = fndict.get(financeId, None)
            if ndict:
                if nomenclatureId in ndict:
                    nomenclatureIdList = [nomenclatureId]
                else:
                    nomenclatureIdList = preciseGroupDict.get(nomenclatureId, None)
                    if nomenclatureIdList is None:
                        nomenclatureIdList = getNomenclatureAnalogies(nomenclatureId)
                        preciseGroupDict[nomenclatureId] = nomenclatureIdList
                for nomenclatureId in nomenclatureIdList:
                    if nomenclatureId in ndict:
                        satisfiedQnt = ndict[nomenclatureId]
#                        outQnt = ndict[nomenclatureId]
#                        delta = min(max(0, qnt-satisfiedQnt), outQnt)
#                        if delta>0:
#                            ndict[nomenclatureId] -= delta
#                            satisfiedQnt += delta
            if oldSatisfiedQnt != satisfiedQnt:
                sumQnt = oldSatisfiedQnt + satisfiedQnt
                item.setValue('satisfiedQnt', toVariant(sumQnt))
                db.updateRecord('StockRequisition_Item', item)
        if masterId:
            requisition = db.getRecordEx(table,
                   'StockRequisition.*',
                   [tableSR['id'].eq(masterId)
                   ])
            if requisition:
                stockMotionsCount = forceInt(requisition.value('stockMotionsCount'))
                stockMotionsCount += 1
                requisition.setValue('stockMotionsCount', toVariant(stockMotionsCount))
                db.updateRecord('StockRequisition', requisition)


    def checkDataEntered(self):
        result = self._checkStockMotionItemsData(self.tblItems)
#        result = result and (self.cmbPurpose.value() or self.checkInputMessage(u'назначение', False, self.cmbPurpose))
        result = result and self.checkItemsDataEntered()
        return result


    def checkItemsDataEntered(self):
        supplierId = self.cmbSupplier.value()
        for row, item in enumerate(self.modelItems.items()):
            if supplierId and not self.checkNomenclatureExists(row, item, supplierId):
                return False
        return True


    def checkNomenclatureExists(self, row, item, supplierId=None):
        supplierId = supplierId or self.cmbSupplier.value()
        financeId = forceRef(item.value('finance_id'))
        batch = forceString(item.value('batch'))
        nomenclature_id = forceRef(item.value('nomenclature_id'))
        unitId = forceRef(item.value('unit_id'))
        existsQnt = getExistsNomenclatureAmount(nomenclature_id, financeId, batch, supplierId, unitId)
        qnt = forceDouble(item.value('qnt'))
        if existsQnt < qnt:
            nomenclatureName = self.modelItems.getNomenclatureNameById(nomenclature_id)
            if existsQnt > 0:
                message = u'На складе {0} {1}, а списание на {2}'.format(existsQnt, nomenclatureName, qnt)
            else:
                message = u'На складе отсутствует %s' % nomenclatureName
            return self.checkValueMessage(message, False, self.tblItems, row, self.modelItems.qntColumnIndex)
        return True


    @pyqtSignature('int')
    def on_cmbReceiver_currentIndexChanged(self, val):
        orgStructureId = self.cmbReceiver.value()
        self.cmbReceiverPerson.setOrgStructureId(orgStructureId)

    @pyqtSignature('int')
    def on_cmbSupplierOrg_currentIndexChanged(self, val):
        supplierOrgExists = bool(self.cmbSupplierOrg.value())

        if supplierOrgExists:
            self.modelItems.updateBatchShelfTimeFinance = False
        if self.cmbSupplier.isEnabled():
            self.cmbSupplier.setEnabled(not supplierOrgExists)
        if self.cmbSupplierPerson.isEnabled():
            self.cmbSupplierPerson.setEnabled(not supplierOrgExists)
        if self.edtSupplierOrgPerson.isEnabled():
            self.edtSupplierOrgPerson.setEnabled(supplierOrgExists)


    @pyqtSignature('QModelIndex')
    def on_tblItems_doubleClicked(self, index):
        if index and index.isValid():
            col = index.column()
            if col == 1:
                self.getRBNomenclatureId(index)
            else:
                self.emit(SIGNAL('doubleClicked(QModelIndex)'), index)


    @pyqtSignature('QModelIndex')
    def on_tblItems_clicked(self, index):
        if index and index.isValid():
            col = index.column()
            if col == 1:
                self.getRBNomenclatureId(index)
            else:
                self.emit(SIGNAL('clicked(QModelIndex)'), index)


    def getRBNomenclatureId(self, index):
        row = index.row()
        if row > -1:
            try:
                dialog = CRBNomenclatureList(self, True)
                if dialog.exec_():
                    rbNomenclatureId = dialog.currentItemId()
                    if rbNomenclatureId:
                        nomenclatureIndex = self.modelItems.index(row, self.modelItems.getColIndex('nomenclature_id'))
                        self.modelItems.setData(nomenclatureIndex, toVariant(rbNomenclatureId), role=Qt.EditRole)
            finally:
                sip.delete(dialog)


class CSummaryInfoModelMixin():
    def applyHelpers(self):
        self.columnListForSummary = (self.getColIndex('sum'), self.getColIndex('qnt'))

    def updateSummaryInfo(self, topLeftIndexColumn, bottomRightIndex):
        return self.getSummaryInfo()

    def getSummaryInfo(self):
        sumResult = 0.0
        qntResult = 0.0
        for item in self._items:
            sumResult += forceDouble(item.value('sum'))
            qntResult += forceDouble(item.value('qnt'))
        return u'Сумма: {sumResult:.2f}, Количество: {qntResult:.2f}'.format(sumResult=sumResult, qntResult=qntResult)


class CItemsModel(CNomenclatureItemsBaseModel, CSummaryInfoModelMixin):
    class CExistsCol(CInDocTableCol):
        def __init__(self):
            CInDocTableCol.__init__(self, u'Остаток', 'existsColumn', 10)
            self._cache = {}

        def toString(self, val, record):
            nomenclatureId = forceRef(record.value('nomenclature_id'))
            financeId = forceRef(record.value('finance_id'))
            batch = forceString(record.value('batch'))
            unitId = forceRef(record.value('unit_id'))
            key = (nomenclatureId, financeId, batch, unitId)
            if key not in self._cache:
                self._cache[key] = getExistsNomenclatureAmount(nomenclatureId, financeId, batch, unitId=unitId)
            return QVariant(self._cache[key])

    class CPriceCol(CFloatInDocTableCol):
        def toString(self, value, record):
            qnt = forceDouble(record.value('qnt'))
            if qnt:
                sum = forceDouble(record.value('sum'))
                return toVariant(self._toString(toVariant(sum/qnt)))
            return QVariant()

        def setEditorData(self, editor, value, record):
            qnt = forceDouble(record.value('qnt'))
            if qnt:
                sum = forceDouble(record.value('sum'))
                s = self._toString(toVariant(sum/qnt))
            else:
                s = ''
            editor.setText('' if s is None else s)
            editor.selectAll()

    class CSumCol(CFloatInDocTableCol):
        def _toString(self, value):
            if value.isNull():
                return None
            return format(forceDouble(value), '.2f')

    def __init__(self, parent, showExists=False):
        CNomenclatureItemsBaseModel.__init__(self, 'StockMotion_Item', 'id', 'master_id', parent)
        self._nomenclatureColumn = CNomenclatureInDocTableCol(u'ЛСиИМН', 'nomenclature_id', 50, showFields = CRBComboBox.showName)
        self._unitColumn = CRBInDocTableCol(u'Ед.Учета', 'unit_id', 12, 'rbUnit', addNone=False)
        self.addCol(self._nomenclatureColumn)
        self.addExtCol(CToolButtonInDocTableCol( u'', 'include', 1), QVariant.String)
        self.addCol(CInDocTableCol( u'Партия', 'batch', 16))
        self.addCol(CDateInDocTableCol( u'Годен до', 'shelfTime', 12, canBeEmpty=True))
        self.addCol(CRBInDocTableCol(    u'Тип финансирования', 'finance_id', 15, 'rbFinance'))
        self.addCol(getStockMotionItemQuantityColumn(u'Кол-во', 'qnt', 12))
        self.addCol(self._unitColumn)
        sumCol = CItemsModel.CSumCol( u'Сумма', 'sum', 12)
        self.addCol(sumCol)
        self.addCol(CItemsModel.CPriceCol(u'Цена', 'id', 12, precision=2))
        self.priceColIndex = len(self.cols())-1
        existsCol = CItemsModel.CExistsCol()
        self.addExtCol(existsCol.setReadOnly(), QVariant.Double)
        self.priceCache = parent
        self.applyHelpers()
        self.qntColumnIndex = self.getColIndex('qnt')

        self._recountSumByQntItems = []
        self.requisitionItems = None
        self.updateBatchShelfTimeFinance = True


    def getNomenclatureNameById(self, nomenclatureId):
        return forceString(self._nomenclatureColumn.toString(nomenclatureId, None))


    def getEmptyRecord(self):
        record = CNomenclatureItemsBaseModel.getEmptyRecord(self)
        record.append(QtSql.QSqlField('include', QVariant.Bool))
        record.setValue('include', QVariant(u'...'))
        return record


    def setData(self, index, value, role=Qt.EditRole):
        if not index.isValid():
            return False
        col = index.column()
        row = index.row()


        if self.requisitionItems or self.updateBatchShelfTimeFinance:
            if col == self.getColIndex('shelfTime'):
                item = self._items[row]
                batch, shelfTime, financeId = self.getBatchShelfTimeFinance(forceRef(item.value('nomenclature_id')), 
                                                                        None, 
                                                                        forceDate(value), 
                                                                        None
                                                                        )
                result = CNomenclatureItemsBaseModel.setData(self, index, value, role)
                if result:
                    item.setValue('finance_id', toVariant(financeId))
                    item.setValue('batch', toVariant(batch))
            elif col == self.getColIndex('finance_id'):
                item = self._items[row]
                batch, shelfTime, financeId = self.getBatchShelfTimeFinance(forceRef(item.value('nomenclature_id')), 
                                                                        forceInt(value), 
                                                                        None, 
                                                                        None
                                                                        )
                result = CNomenclatureItemsBaseModel.setData(self, index, value, role)
                if result:
                    item.setValue('shelfTime', toVariant(shelfTime))
                    item.setValue('batch', toVariant(batch))
            elif col == self.getColIndex('batch'):
                item = self._items[row]
                batch, shelfTime, financeId = self.getBatchShelfTimeFinance(forceRef(item.value('nomenclature_id')), 
                                                                        None, 
                                                                        None, 
                                                                        forceString(value)
                                                                        )
                result = CNomenclatureItemsBaseModel.setData(self, index, value, role)
                item.setValue('finance_id', toVariant(financeId))
                item.setValue('shelfTime', toVariant(shelfTime))

        if col == 1:
            if not (0 <= row < len(self._items)):
                return False

            item = self._items[row]

            item.setValue('include', QVariant(u'...'))
            self.emitRowChanged(row)
            return True
        elif col == self.priceColIndex:
            if not (0 <= row < len(self._items)):
                return False

            item = self._items[row]
            qnt = forceDouble(item.value('qnt'))
            item.setValue('sum', QVariant(qnt*forceDouble(value)))
            self.emitRowChanged(row)
            return True
        elif col == self.getColIndex('qnt'):
            result = CNomenclatureItemsBaseModel.setData(self, index, value, role)
            if result:
                item = self._items[row]
                if item in self._recountSumByQntItems:
                    item.setValue('sum', toVariant(self.calcSum(item)))
        elif col == self.getColIndex('nomenclature_id'):
            if 0 <= row < len(self._items):
                oldNomenclatureId = forceRef(self._items[row].value('nomenclature_id'))
            else:
                oldNomenclatureId = None
            result = CNomenclatureItemsBaseModel.setData(self, index, value, role)
            if result:
                item = self._items[row]
                nomenclatureId = forceRef(item.value('nomenclature_id'))
                if oldNomenclatureId != nomenclatureId:
                    unitId = self.getDefaultStockUnitId(nomenclatureId)
                    item.setValue('unit_id', toVariant(unitId))
                    if self.updateBatchShelfTimeFinance:
                        batch, shelfTime, financeId = self.getBatchShelfTimeFinance(forceRef(value))
                        result = CNomenclatureItemsBaseModel.setData(self, index, value, role)
                        item.setValue('batch', toVariant(batch))
                        item.setValue('finance_id', toVariant(financeId))
                        item.setValue('shelfTime', toVariant(shelfTime))
                    self.emitRowChanged(row)
            return result
        elif col == self.getColIndex('unit_id'):
            if not (0 <= row < len(self._items)):
                return False
            item = self._items[row]
            oldUnitId = forceRef(item.value('unit_id'))
            result = CNomenclatureItemsBaseModel.setData(self, index, value, role)
            if result:
                newUnitId = forceRef(item.value('unit_id'))
                self._applySumRatio(item, oldUnitId, newUnitId)
                self.emitRowChanged(row)
        else:
            return CNomenclatureItemsBaseModel.setData(self, index, value, role)


    def setRequsitions(self, requisitionIdList):
        db = QtGui.qApp.db
        table = db.table('StockRequisition_Item')
        self.requisitionItems = db.getRecordList(table,
                           '*',
                           table['master_id'].inlist(requisitionIdList),
                           'master_id, idx')
        for item in self.requisitionItems:
            qnt = forceDouble(item.value('qnt'))-forceDouble(item.value('satisfiedQnt'))
            if qnt > 0:
                myItem = self.getEmptyRecord()
                nomenclatureId = forceRef(item.value('nomenclature_id'))
                unitId = forceRef(item.value('unit_id'))
                financeId = forceInt(item.value('finance_id'))
                myItem.setValue('nomenclature_id', toVariant(nomenclatureId))
                batch, shelfTime, finance = self.getBatchShelfTimeFinance(nomenclatureId, financeId)
                myItem.setValue('batch',         toVariant(batch))
                myItem.setValue('shelfTime',         toVariant(shelfTime))
                if finance:
                    myItem.setValue('finance_id',      toVariant(finance))
                myItem.setValue('unit_id',         toVariant(unitId))
                myItem.setValue('qnt',             toVariant(qnt))
                myItem.setValue('sum',             toVariant(self.calcSum(myItem)))

                self._recountSumByQntItems.append(myItem)

                self.items().append(myItem)
        self.reset()


    def getBatchShelfTimeFinance(self, nomenclatureId, financeId=None, shelfTime=None, batch=None):
        order = u''
        recordBatch = u''
        financeCond = u'1'
        if financeId:
            financeCond = u'SMI.finance_id = %s' %financeId
        recordShelfTime =  QDate()
        recordFinance = 0
        tableStockMotionItem = QtGui.qApp.db.table('StockMotion_Item')
        tableStockMotion = QtGui.qApp.db.table('StockMotion')
        cols = [tableStockMotionItem['batch'], 
                    tableStockMotionItem['shelfTime'], 
                    tableStockMotionItem['finance_id'], 
                    '''SUM(StockMotion_Item.qnt) as qnt''', 
                    '''(SELECT 
                        SUM(qnt)
                    FROM
                        StockMotion_Item AS SMI
                            LEFT JOIN
                        StockMotion ON StockMotion.id = SMI.master_id
                    WHERE
                        SMI.nomenclature_id = %(nomenclature)s
                            AND supplier_id = %(orgStructure)s
                            AND %(financeCond)s
                            AND SMI.batch = StockMotion_Item.batch
                            AND SMI.shelfTime = StockMotion_Item.shelfTime
                            AND SMI.unit_id is not NULL
                            AND SMI.id NOT IN (StockMotion_Item.id)) AS transQnt'''
                                %{'nomenclature':nomenclatureId, 
                                'orgStructure':QtGui.qApp.currentOrgStructureId(), 
                                'financeCond' : financeCond
                                }]
        queryTable = tableStockMotionItem
        queryTable = queryTable.leftJoin(tableStockMotion, tableStockMotion['id'].eq(tableStockMotionItem['master_id']))
        cond = [tableStockMotionItem['nomenclature_id'].eq(nomenclatureId), 
                    tableStockMotion['receiver_id'].eq(QtGui.qApp.currentOrgStructureId()), 
                    tableStockMotion['type'].inlist((0, 1)), 
                    tableStockMotionItem['shelfTime'].dateGe(QDate.currentDate())]
        if shelfTime:
            cond.append(tableStockMotionItem['shelfTime'].eq(shelfTime))
            if order==u'':
                order = 'shelfTime'
            else:
                order += ',shelfTime'
        if batch:
            cond.append(tableStockMotionItem['batch'].eq(batch))
            if order==u'':
                order = 'batch'
            else:
                order += ',batch'
        if financeId:
            cond.append(tableStockMotionItem['finance_id'].eq(financeId))
        if order==u'':
            order = 'shelfTime'
            
        recordList = QtGui.qApp.db.getRecordListGroupBy(queryTable,
                           cols,
                           cond,
                           'batch, shelfTime, finance_id ORDER BY %s'%order)
        for record in recordList:
            if record and (forceDouble(record.value('qnt'))>forceDouble(record.value('transQnt'))):
                recordBatch = forceString(record.value('batch'))
                recordShelfTime = forceDate(record.value('shelfTime'))
                recordFinance = forceInt(record.value('finance_id'))
                return recordBatch, recordShelfTime, recordFinance
        return recordBatch, recordShelfTime, recordFinance


    def getDataAsFNDict(self):
        result = {}
        for item in self.items():
            financeId = forceRef(item.value('finance_id'))
            nomenclatureId = forceRef(item.value('nomenclature_id'))
            if nomenclatureId:
                ndict = result.setdefault(financeId, {})
                ndict[nomenclatureId]=ndict.get(nomenclatureId, 0) + forceDouble(item.value('qnt'))
        return result

def getNomenclatureAnalogies(nomenclatureId):
    db = QtGui.qApp.db
    tableNomenclature = db.table('rbNomenclature')
    tableNomenclature2 = db.table('rbNomenclature').alias('RBN2')
    table = tableNomenclature.leftJoin(tableNomenclature2,
                                       tableNomenclature2['analog_id'].eq(tableNomenclature['analog_id']))
    return db.getIdList(table,
                        tableNomenclature['id'],
                        db.joinAnd([
                                    tableNomenclature2['id'].eq(nomenclatureId),
                                    tableNomenclature['analog_id'].isNotNull(),
                                   ]
                                  )
                       )



