# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui, QtSql
from PyQt4.QtCore import Qt, QDate, QVariant, pyqtSignature, SIGNAL

from library.adjustPopup         import adjustPopupToWidget
from library.crbcombobox         import CRBComboBox
from library.InDocTable          import CLocItemDelegate, CRecordListModel, CInDocTableCol, CRBInDocTableCol, CSelectStrInDocTableCol
from library.SortedDict          import CSortedDict
from library.TableModel          import CTableModel, CTextCol, CRefBookCol
from library.Utils               import forceRef, forceString, forceStringEx, toVariant, forceDouble

from Stock.Utils import getExistsNomenclatureIdList, getExistsNomenclatureAmount, applyNomenclatureUnitRatio

from Stock.Ui_NomenclatureComboBoxPopup import Ui_NomenclatureComboBoxPopup


class CNomenclatureComboBox(CRBComboBox):
    def __init__(self, parent):
        CRBComboBox.__init__(self, parent)
        self.setTable('rbNomenclature')
        self.defaultClassId = None
        self.defaultKindId = None
        self.defaultTypeId = None
        self.defaultFeatures = []
        self._popup = None
        self._stockOrgStructureId = None
        self._onlyExists = False
        self._useClientUnitId = False
        self._financeId = None

    def setUseClientUnitId(self, value=True):
        self._useClientUnitId = value

    def setOrgStructureId(self, orgStructureId):
        self._stockOrgStructureId = orgStructureId

    def setOnlyExists(self, value=True):
        self._onlyExists = value


    def setFinanceId(self, value=None):
        self._financeId = value


    def setDefaultIds(self, classId, kindId, typeId):
        self.defaultClassId = classId
        self.defaultKindId = kindId
        self.defaultTypeId = typeId


    def setDefaultClassId(self, classId):
        self.defaultClassId = classId


    def setDefaultKindId(self, kindId):
        self.defaultKindId = kindId


    def setDefaultTypeId(self, typeId):
        self.defaultTypeId = typeId


    def setDefaultFeatures(self, features):
        self.defaultFeatures = features


    def setQValue(self, var):
        self.setValue(forceRef(var))


    def showPopup(self):
#        self.__searchString = ''
        if not self._popup:
            self._popup = CNomenclatureComboBoxPopup(self, self._useClientUnitId, self._financeId)
            self.connect(self._popup, SIGNAL('itemSelected(QVariant)'), self.setQValue)
        self._popup.setDefaultIds(self.defaultClassId, self.defaultKindId, self.defaultTypeId)
        self._popup.setOnlyExists(self._onlyExists)
        if QtGui.qApp.controlSMFinance() == 1:
            self._popup.setFinanceId(self._financeId)
            self._popup.setFinanceEnabled(True)
        elif QtGui.qApp.controlSMFinance() == 2:
            self._popup.setFinanceId(self._financeId)
            self._popup.setFinanceEnabled(False)
        elif QtGui.qApp.controlSMFinance() == 0:
            self._popup.setFinanceVisible(False)
            self._popup.setFinanceEnabled(False)
        self._popup.setDefaultFeatures(self.defaultFeatures)
        self._popup.setStockOrgStructureId(self._stockOrgStructureId)
        adjustPopupToWidget(self, self._popup, True, preferredWidth=self.preferredWidth)
        self._popup.show()


class CNomenclatureInDocTableCol(CRBInDocTableCol):
    def __init__(self, title, fieldName, width, **params):
        CRBInDocTableCol.__init__(self, title, fieldName, width, 'rbNomenclature', **params)
        self.defaultClassId = None
        self.defaultKindId = None
        self.defaultTypeId = None
        self._showLfForm = params.get('showLfForm', False)
        self._stringCache = {}
        self._tableNomenclature = QtGui.qApp.db.table(self.tableName)
        self._tableLfForm = QtGui.qApp.db.table('rbLfForm')

    def toString(self, val, record):
        if not self._showLfForm:
            return CRBInDocTableCol.toString(self, val, record)

        nomenclatureId = forceRef(val)
        if nomenclatureId not in self._stringCache:
            text = forceString(CRBInDocTableCol.toString(self, val, record))
            lfForm = self._getLfForm(nomenclatureId)
            lfForm = u'/{0}'.format(lfForm) if lfForm else u''
            self._stringCache[nomenclatureId] = u'{0}{1}'.format(text, lfForm).strip()
        return toVariant(self._stringCache[nomenclatureId])

    def _getLfForm(self, nomenclatureId):
        queryTable = self._tableNomenclature.innerJoin(
            self._tableLfForm, self._tableLfForm['id'].eq(self._tableNomenclature['lfForm_id'])
        )

        fields = [self._tableLfForm['code'].name(), self._tableLfForm['name'].name()]

        cond = self._tableNomenclature['id'].eq(nomenclatureId)

        record = QtGui.qApp.db.getRecordEx(queryTable, fields, cond)
        if not record:
            return ''

        return u'{0}-{1}'.format(
            forceString(record.value('code')),
            forceString(record.value('name'))
        )

    def setDefaultClassId(self, classId):
        self.defaultClassId = classId


    def setDefaultKindId(self, kindId):
        self.defaultKindId = kindId


    def setDefaultTypeId(self, typeId):
        self.defaultTypeId = typeId


    def createEditor(self, parent):
        editor = CNomenclatureComboBox(parent)
        editor.setShowFields(self.showFields)
        editor.setPreferredWidth(self.preferredWidth)
        editor.setDefaultIds(self.defaultClassId, self.defaultKindId, self.defaultTypeId)
        return editor


class CToolButtonInDocTableCol(CInDocTableCol):
    def __init__(self, title, fieldName, width, **params):
        CInDocTableCol.__init__(self, title, fieldName, width, **params)


    def toString(self, val, record):
        return toVariant(val)


    def createEditor(self, parent):
        editor = QtGui.QToolButton(parent)
        editor.setText(u'...')
        return editor


    def setEditorData(self, editor, value, record):
        editor.setText(forceString(value))


    def getEditorData(self, editor):
        return toVariant(editor.text())


class CNomenclatureComboBoxPopup(QtGui.QFrame, Ui_NomenclatureComboBoxPopup):
    __pyqtSignals__ = ('itemSelected(int)',
                      )

    def __init__(self, parent=None, useClientUnitId=False, defaultFinanceId=None):
        QtGui.QFrame.__init__(self, parent, Qt.Popup)
        self._parent = parent
        self.model  = CNomenclatureModel(self, useClientUnitId, defaultFinanceId)
        self.modelFeatures = CFeaturesModel(self)

        self.setupUi(self)
        self.tblNomenclature.setModel(self.model)
        self.tblNomenclature.setSelectionMode(QtGui.QAbstractItemView.SingleSelection)

        self.cmbClass.setTable('rbNomenclatureClass', True)
        self.cmbKind.setTable('rbNomenclatureKind', True)
        self.cmbType.setTable('rbNomenclatureType', True)
        self.cmbLfForm.setTable('rbLfForm', True)
        self.cmbFinance.setTable('rbFinance', True)

        self.tblFeatures.setItemDelegate(CFeatureItemDelegate(self))
        self.tblFeatures.setModel(self.modelFeatures)
        self.tblFeatures.addPopupSelectAllRow()
        self.tblFeatures.addPopupClearSelectionRow()
        self.tblFeatures.addPopupSeparator()
        self.tblFeatures.addPopupDelRow()

        self._defaultClassId = None
        self._defaultKindId = None
        self._defaultTypeId = None
        self._defaultOnluExists = False
        self._defaultFinanceId = None
        self._onlyExists = False
        self._stockOrgStructureId = None
        self._defaultFeatures = []
        # я не хочу загружать features на каждое изменение class/kind/type, поэтому
        # ввёл флаг: features в modelFeatures достоверен или нет
        self._featuresIsValid = True
        self.initSearch()
#        self.cmbClass.installEventFilter(self)

    def setStockOrgStructureId(self, orgStructureId):
        self._stockOrgStructureId = orgStructureId

    def setDefaultIds(self, classId, kindId, typeId):
        self._defaultClassId = classId
        self._defaultKindId = kindId
        self._defaultTypeId = typeId


    def setDefaultFeatures(self, features):
        self._defaultFeatures = features


    def setOnlyExists(self, value=True):
        self.chkOnlyExists.setChecked(value)
        self._defaultOnluExists = value

    def setFinanceId(self, value=None):
        self.cmbFinance.setValue(value)
        self._defaultFinanceId = value
        self.model.setFinanceId(value)

    def setFinanceEnabled(self, value=True):
        self.cmbFinance.setEnabled(value)

    def setFinanceVisible(self, value=True):
        self.lblFinance.setVisible(value)
        self.cmbFinance.setVisible(value)

    def show(self):
        self.resetSearch()
        self.applySearch()
        self.tblNomenclature.setFocus()
        id = self._parent.value()
        if id:
            self.tblNomenclature.setCurrentItemId(self._parent.value())
        else:
            self.tblNomenclature.setCurrentRow(0)
        self.tabSearchAttributes.setCurrentIndex(0)
        QtGui.QFrame.show(self)


    def selectCurrentItem(self):
        id = self.tblNomenclature.currentItemId()
        self.emit(SIGNAL('itemSelected(QVariant)'), toVariant(id))
        self.hide()


    def initSearch(self):
        self._classId = self._defaultClassId
        self._kindId = self._defaultKindId
        self._typeId = self._defaultTypeId
        self._lfFormId = None
        self._financeId = self._defaultFinanceId
        self._code = ''
        self._name = ''
        self._producer = ''
        self._ATC = ''
        self._includeAnalogies = False
        self._features = self._defaultFeatures
        self._onlyExists = self._defaultOnluExists
        self._mnnFilter = ''


    def resetSearch(self):
        self.initSearch()
        self.cmbClass.setValue(self._classId)
        self.cmbKind.setValue(self._kindId)
        self.cmbType.setValue(self._typeId)
        self.cmbLfForm.setValue(self._lfFormId)
        self.cmbFinance.setValue(self._financeId)
        self.edtCode.setText(self._code)
        self.edtName.setText(self._name)
        self.edtProducer.setText(self._producer)
        self.edtATC.setText(self._ATC)
        self.chkIncludeAnalogies.setChecked(self._includeAnalogies)
        self.chkOnlyExists.setChecked(self._onlyExists)
        self.edtMnnFilter.setText(self._mnnFilter)
        self._featuresIsValid = False

#        self.modelFeatures.setValuableFeatures(self._defaultFeatures)


    def applySearch(self):
        self._classId = self.cmbClass.value()
        self._kindId = self.cmbKind.value()
        self._typeId = self.cmbType.value()
        self._lfFormId = self.cmbLfForm.value()
        self._financeId = self.cmbFinance.value()
        self._code = forceStringEx(self.edtCode.text())
        self._name = forceStringEx(self.edtName.text())
        self._producer = forceStringEx(self.edtProducer.text())
        self._ATC = forceStringEx(self.edtATC.text())
        self._includeAnalogies = self.chkIncludeAnalogies.isChecked()
        self._mnnFilter = forceStringEx(self.edtMnnFilter.text())

        if self._features or self._defaultFeatures:
            if not self._featuresIsValid:
                self.modelFeatures.prepareList(self._classId, self._kindId, self._typeId)
                self.modelFeatures.setValuableFeatures(self._features)
                self._featuresIsValid = True
            self._features = self.modelFeatures.getValuableFeatures()
        else:
            self._features = []
        self._onlyExists = self.chkOnlyExists.isChecked()

        db = QtGui.qApp.db
        cond = []
        tableNomenclature = db.table('rbNomenclature')
        table = tableNomenclature

        if self._typeId:
            cond.append(tableNomenclature['type_id'].eq(self._typeId))
        elif self._kindId or self._classId:
            tableType = db.table('rbNomenclatureType')
            table = table.innerJoin(tableType, tableType['id'].eq(tableNomenclature['type_id']))
            if self._kindId:
                cond.append(tableType['kind_id'].eq(self._kindId))
            else:
                tableKind = db.table('rbNomenclatureKind')
                table = table.innerJoin(tableKind, tableKind['id'].eq(tableType['kind_id']))
                cond.append(tableKind['class_id'].eq(self._classId))
        if self._lfFormId:
            cond.append(tableNomenclature['lfForm_id'].eq(self._lfFormId))
        if self._code:
            cond.append(tableNomenclature['code'].like(self._code))
        if self._name:
            cond.append(tableNomenclature['name'].like(self._name))
        if self._producer:
            cond.append(tableNomenclature['producer'].like(self._producer))
        if self._ATC:
            cond.append(tableNomenclature['ATC'].like(self._ATC))
        if self._mnnFilter:
            cond.append(
                db.joinOr([
                    tableNomenclature['mnnLatin'].like(self._mnnFilter),
                    tableNomenclature['internationalNonproprietaryName'].like(self._mnnFilter),
                ])
            )

        if self._includeAnalogies and cond:
            tableTarget = db.table('rbNomenclature').alias('A')
            table = table.innerJoin(tableTarget,
                                   db.joinOr([
                                       db.joinAnd( [tableTarget['analog_id'].isNotNull(),
                                                    tableTarget['analog_id'].eq(tableNomenclature['analog_id'])] ),
                                       db.joinAnd( [tableTarget['analog_id'].isNull(),
                                                    tableTarget['id'].eq(tableNomenclature['id'])] )
                                              ]))
        else:
            tableTarget = tableNomenclature

        if self._features:
            cnt = 0
            for name, value in self._features:
                tableFeature = db.table('rbNomenclature_Feature').alias('F%d' % cnt)
                table = table.innerJoin(tableFeature, tableFeature['master_id'].eq(tableTarget['id']))
                cond.append(tableFeature['name'].eq(name))
                cond.append(tableFeature['value'].eq(value))
                cnt += 1
        if self._onlyExists:
            existsIdList = getExistsNomenclatureIdList(self._stockOrgStructureId, self._financeId)
            cond.append(tableNomenclature['id'].inlist(existsIdList))
        idList = db.getDistinctIdList(table, tableTarget['id'].name(), cond, [tableTarget['name'].name(), tableTarget['code'].name()] )
        self.tblNomenclature.setIdList(idList)
        if self._financeId:
            self.model.setFinanceId(self._financeId)


    @pyqtSignature('QModelIndex')
    def on_tblNomenclature_clicked(self, index):
        self.selectCurrentItem()


    @pyqtSignature('int')
    def on_tabSearchAttributes_currentChanged(self, index):
        if index == 1:
            self.modelFeatures.prepareList(self.cmbClass.value(), self.cmbKind.value(), self.cmbType.value())
            self.modelFeatures.setValuableFeatures(self._features)
            self._featuresIsValid = True # пользователь видел, значит всё ок


    @pyqtSignature('int')
    def on_cmbClass_currentIndexChanged(self, index):
        classId = self.cmbClass.value()
        if classId:
            table = QtGui.qApp.db.table('rbNomenclatureKind')
            self.cmbKind.setFilter(table['class_id'].eq(classId))
        else:
            self.cmbKind.setFilter('')
        self._featuresIsValid = False # пользователь не видел, значит нужно будет перезагрузить


    @pyqtSignature('int')
    def on_cmbKind_currentIndexChanged(self, index):
        kindId = self.cmbKind.value()
        if kindId:
            table = QtGui.qApp.db.table('rbNomenclatureType')
            self.cmbType.setFilter(table['kind_id'].eq(kindId))
        else:
            self.cmbType.setFilter('')
        self._featuresIsValid = False


    @pyqtSignature('QAbstractButton*')
    def on_buttonBox_clicked(self, button):
        buttonCode = self.buttonBox.standardButton(button)
        if buttonCode == QtGui.QDialogButtonBox.Apply:
            self.applySearch()
            self.tabWidget.setCurrentIndex(0)
        elif buttonCode == QtGui.QDialogButtonBox.Reset:
            self.resetSearch()
        self._featuresIsValid = False

#    def eventFilter(self, obj, event):
#        if event.type() == QEvent.KeyPress:
#            key = event.key()
#            if key in (Qt.Key_C, Qt.Key_G):
#                if key == Qt.Key_C:
#                    obj.keyPressEvent(event)
#                self.keyPressEvent(event)
#                return False
#            if  key == Qt.Key_Tab:
#                self.focusNextPrevChild(True)
#                return True
#        return False


    def keyPressEvent(self, event):
        if event.key() in (Qt.Key_Return, Qt.Key_Enter):
            if self.tabWidget.currentIndex():
                self.applySearch()
                self.tabWidget.setCurrentIndex(0)
            else:
                if self.tblNomenclature.currentItemId():
                    self.selectCurrentItem()
#        if (event.modifiers() == Qt.ControlModifier and event.key() == Qt.Key_C):
#            self.tabWidget.setCurrentIndex(0)
#        if (event.modifiers() == Qt.ControlModifier and event.key() == Qt.Key_G):
#            self.tabWidget.setCurrentIndex(1)
        QtGui.QFrame.keyPressEvent(self, event)


def _getExistsNomenclatureAmount(nomenclatureId, unitId=None, financeId = None):
    db = QtGui.qApp.db
    table = db.table('StockTrans')

    date = QDate.currentDate()

    debCond = [table['date'].dateLe(date),
               db.joinOr([table['shelfTime'].isNull(), table['shelfTime'].dateGe(date)])]
    creCond = [table['date'].dateLe(date),
               db.joinOr([table['shelfTime'].isNull(), table['shelfTime'].dateGe(date)])]

    orgStructureId = QtGui.qApp.currentOrgStructureId()

    if orgStructureId:
        debCond.append(table['debOrgStructure_id'].eq(orgStructureId))
        creCond.append(table['creOrgStructure_id'].eq(orgStructureId))
    else:
        debCond.append(table['debOrgStructure_id'].isNotNull())
        creCond.append(table['creOrgStructure_id'].isNotNull())

    if financeId:
        debCond.append(table['debFinance_id'].eq(financeId))
        creCond.append(table['creFinance_id'].eq(financeId))

    debCond.append(table['debNomenclature_id'].eq(nomenclatureId))
    creCond.append(table['creNomenclature_id'].eq(nomenclatureId))

    stmt = u'''
    SELECT
       sum(T.qnt) AS `qnt`
    FROM
    (
    SELECT
           sum(qnt)           AS `qnt`,
            debFinance_id AS `finance_id`
    FROM StockTrans
    WHERE %(debCond)s
    UNION
    SELECT
           -sum(qnt)          AS `qnt`,
           creFinance_id AS `finance_id`
    FROM StockTrans
    WHERE %(creCond)s
    ) AS T

    ''' % {
        'debCond': db.joinAnd(debCond),
        'creCond': db.joinAnd(creCond)
    }

    query = db.query(stmt)
    if query.next():
        qnt = forceDouble(query.record().value('qnt'))
        if unitId is not None:
            return applyNomenclatureUnitRatio(qnt, nomenclatureId, unitId)
        return qnt
    return 0


class CNomenclatureModel(CTableModel):
    class CQntCol(CTextCol):
        def __init__(self, useClientUnitId, financeId):
            CTextCol.__init__(self, u'Остаток', ['id'], 10)
            self.useClientUnitId = useClientUnitId
            self.financeId = financeId
            self._cacheClientUnitId = {}
            self._cache = {}
        
        def setFinanceId(self, value):
            self.financeId = value

        def format(self, values):
            nomenclatureId = forceRef(values[0])
            if (nomenclatureId, self.financeId) not in self._cache:
                unitId = self._getUnitId(nomenclatureId)
                self._cache[nomenclatureId, self.financeId] = _getExistsNomenclatureAmount(nomenclatureId, unitId=unitId, financeId = self.financeId)
            return QVariant(self._cache[nomenclatureId, self.financeId])

        def _getUnitId(self, nomenclatureId):
            if not self.useClientUnitId:
                return None

            if nomenclatureId not in self._cacheClientUnitId:

                unitId = forceRef(
                    QtGui.qApp.db.translate('rbNomenclature', 'id', nomenclatureId, 'defaultClientUnit_id')
                )
                self._cacheClientUnitId[nomenclatureId] = unitId

            return self._cacheClientUnitId[nomenclatureId]


    def __init__(self, parent, useClientUnitId=False, financeId=None):
        self.qntCol = CNomenclatureModel.CQntCol(useClientUnitId, financeId)
        CTableModel.__init__(self, parent, [
            CTextCol(     u'Код',             ['code'],      10),
            CTextCol(     u'Наименование',    ['name'],      60),
            CTextCol(     u'МНН на латыни',   ['mnnLatin'],  20),
            CTextCol(     u'Дозировка',       ['dosageValue'], 20),
            CRefBookCol(  u'Форма выпуска',   ['lfForm_id'], 'rbLfForm', 15, CRBComboBox.showName),
            CTextCol(     u'Дата включения',  ['inDate'],   20),
            CTextCol(     u'Дата исключения', ['exDate'],   20),
            CTextCol(     u'Производитель',   ['producer'],  20),
            self.qntCol,
            ], 'rbNomenclature' )
        
    def setFinanceId(self, value):
        self.qntCol.setFinanceId(value)
        self.emitDataChanged()


class CFeatureItemDelegate(CLocItemDelegate):
    def createEditor(self, parent, option, index):
        column = index.column()
        editor = index.model().createEditor(column, index.row(), parent)
        self.connect(editor, SIGNAL('commit()'), self.emitCommitData)
        self.connect(editor, SIGNAL('editingFinished()'), self.commitAndCloseEditor)

        self.editor   = editor
        self.row      = index.row()
        self.rowcount = index.model().rowCount(None)
        self.column   = column
        return editor


class CFeatureEditor(CSelectStrInDocTableCol):
    def toString(self, val, record):
        return val


class CFeaturesModel(CRecordListModel):
    def __init__(self, parent):
        CRecordListModel.__init__(self, parent)
        self.colName  = self.addExtCol(CFeatureEditor(u'Наименование',  'name', 20, []), QVariant.String)
        self.colValue = self.addExtCol(CFeatureEditor(u'Значение',  'value', 20, []), QVariant.String)
        self.setEnableAppendLine(True)
        self.actualValues = {}


    def getEmptyRecord(self):
        record = CRecordListModel.getEmptyRecord(self)
        record.append(QtSql.QSqlField('name', QVariant.String))
        record.append(QtSql.QSqlField('value', QVariant.String))
        return record


    def prepareList(self, classId, kindId, typeId):
        self.actualValues = getFeaturesAndValues(classId, kindId, typeId)


    def createEditor(self, column, row, parent):
        if column == 0:
            names = self.actualValues.keys()
            names.sort()
            self.colName.values = names
            return self.colName.createEditor(parent)
        else:
            name = forceString(self.value(row, 'name') or '')
            self.colValue.values = self.actualValues.get(name, [])
            return self.colValue.createEditor(parent)


    def setEmptyFeatures(self):
        items = []
        names = self.actualValues.keys()
        names.sort()
        for name in names:
            if name in self.actualValues:
                item = self.getEmptyRecord()
                item.setValue('name', name)
                item.setValue('value', '')
                items.append(item)
        self.setItems(items)


    def setValuableFeatures(self, features):
        items = []
        for name, value in features:
            if name in self.actualValues:
                item = self.getEmptyRecord()
                item.setValue('name', name)
                item.setValue('value', value)
                items.append(item)
        self.setItems(items)


    def getValuableFeatures(self):
        result = []
        for item in self.items():
            name = forceString(item.value('name'))
            value = forceString(item.value('value'))
            if value:
                result.append((name, value))
        return result


def getFeaturesAndValues(classId = 0, kindId = 0, typeId = 0, nomenclatureId = 0):
    db = QtGui.qApp.db
    cond = []
    tableNomenclature = db.table('rbNomenclature')
    table = tableNomenclature

    if nomenclatureId:
        cond.append(tableNomenclature['id'].eq(nomenclatureId))
    elif typeId:
        cond.append(tableNomenclature['type_id'].eq(typeId))
    elif kindId or classId:
        tableType = db.table('rbNomenclatureType')
        table = table.innerJoin(tableType, tableType['id'].eq(tableNomenclature['type_id']))
        if kindId:
            cond.append(tableType['kind_id'].eq(kindId))
        else:
            tableKind = db.table('rbNomenclatureKind')
            table = table.innerJoin(tableKind, tableKind['id'].eq(tableType['kind_id']))
            cond.append(tableKind['class_id'].eq(classId))

    tableFeature = db.table('rbNomenclature_Feature')
    table = table.innerJoin(tableFeature, tableFeature['master_id'].eq(tableNomenclature['id']))

    stmt = db.selectDistinctStmt(table, [tableFeature['name'], tableFeature['value']], cond, 'idx')
    query = db.query(stmt)
    result = CSortedDict()
    while query.next():
        name  = forceString(query.value(0))
        value = forceString(query.value(1))
        result.setdefault(name, []).append(value)
    for name, value in result.iteritems():
        value.sort()
    return result
        #print d
