# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import sip

from PyQt4 import QtGui
from PyQt4.QtCore import Qt, QByteArray, QDateTime, pyqtSignature

from library.crbcombobox                   import CRBComboBox
from library.DialogBase                    import CDialogBase
from library.InDocTable                    import CInDocTableModel, CDateInDocTableCol, CFloatInDocTableCol, CInDocTableCol, CRBInDocTableCol
from library.interchange                   import getRBComboBoxValue
from library.PrintInfo                     import CInfoContext
from library.PrintTemplates                import applyTemplate, CPrintAction, CPrintButton, getPrintTemplates
from library.Utils                         import forceBool, forceDouble, forceString, toVariant

from Reports.ReportBase                    import CReportBase, createTable
from Reports.ReportView                    import CReportViewDialog
from Stock.NomenclatureComboBox            import CNomenclatureInDocTableCol
from Stock.StockMotionBaseDialog           import CStockMotionBaseDialog

from Stock.Utils import getStockMotionItemQuantityColumn

from Stock.Ui_Production                   import Ui_ProductionDialog
from Stock.Ui_SelectRecipeDialog           import Ui_Dialog as Ui_SelectRecipeDialog


class CProductionEditDialog(CStockMotionBaseDialog, Ui_ProductionDialog):
    stockDocumentType = 3
    
    def __init__(self,  parent):
        CStockMotionBaseDialog.__init__(self, parent)
        self.addObject('btnAddRecipe', QtGui.QPushButton(u'По рецепту', self))
        self.addModels('InItems', CItemsModel(self, False))
        self.addModels('OutItems', CItemsModel(self, True))
        self.addObject('btnPrint', CPrintButton(self, u'Печать'))
        self.btnPrint.setShortcut('F6')
        self.setupUi(self)
        self.cmbSupplierPerson.setSpecialityIndependents()
        self.buttonBox.addButton(self.btnAddRecipe, QtGui.QDialogButtonBox.ActionRole)
        self.setWindowFlags(self.windowFlags() | Qt.WindowMaximizeButtonHint)
        self.setupDirtyCather()
        self.buttonBox.addButton(self.btnPrint, QtGui.QDialogButtonBox.ActionRole)
        templates = getPrintTemplates(self.getStockContext())
        if not templates:
            self.btnPrint.setId(-1)
        else:
            for template in templates:
                action = CPrintAction(template[0], template[1], self.btnPrint, self.btnPrint)
                self.btnPrint.addAction(action)
            self.btnPrint.menu().addSeparator()
            self.btnPrint.addAction(CPrintAction(u'Напечатать список', -1, self.btnPrint, self.btnPrint))
        self.tblInItems.setModel(self.modelInItems)
        self.prepareItemsPopupMenu(self.tblInItems)
        self.tblInItems.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
        self.tblInItems.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)

        self.tblOutItems.setModel(self.modelOutItems)
        self.prepareItemsPopupMenu(self.tblOutItems)
        self.tblOutItems.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
        self.tblOutItems.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)


    def setDefaults(self):
        CStockMotionBaseDialog.setDefaults(self)


    @pyqtSignature('int')
    def on_btnPrint_printByTemplate(self, templateId):
        self.actPrintMotions(templateId)


    def actPrintMotions(self, templateId):
        from Stock.StockMotionInfo import CStockMotionInfoList
        if templateId == -1:
            self.getNomenclaturePrint()
        else:
            idList = self.tblItems.model().itemIdList()
            context = CInfoContext()
            data = { 'InventoryList': CStockMotionInfoList(context, idList)}
            QtGui.qApp.call(self, applyTemplate, (self, templateId, data))


    def getStockContext(self):
        return ['InventoryList']


    def dumpParams(self, cursor):
        db = QtGui.qApp.db
        description = []
        number = self.edtNumber.text()
        if number:
            description.append(u'Номер %s'%forceString(number))
        date = QDateTime(self.edtDate.date(), self.edtTime.time())
        if date:
           description.append(u'Дата %s'%forceString(date))
        reason = self.edtReason.text()
        if reason:
            description.append(u'Основание %s'%forceString(reason))
        reasonDate = self.edtReasonDate.date()
        if reasonDate:
           description.append(u'Дата основания %s'%forceString(reasonDate))
        supplier = self.cmbSupplier.value()
        if supplier:
            description.append(u'Подразделение %s'%forceString(db.translate('OrgStructure', 'id', supplier, 'name')))
        supplierPerson = self.cmbSupplierPerson.value()
        if supplierPerson:
            description.append(u'Ответственный %s'%forceString(db.translate('vrbPersonWithSpeciality', 'id', supplierPerson, 'name')))
        note = self.edtNote.text()
        if note:
            description.append(u'Примечания %s'%forceString(note))
        description.append(u'отчёт составлен: ' + forceString(QDateTime.currentDateTime()))
        columns = [ ('100%', [], CReportBase.AlignLeft) ]
        table = createTable(cursor, columns, headerRowCount=len(description), border=0, cellPadding=2, cellSpacing=0)
        for i, row in enumerate(description):
            table.setText(i, 0, row)
        cursor.movePosition(QtGui.QTextCursor.End)
        cursor.insertBlock()


    def getNomenclaturePrint(self):
#        title = self.windowTitle()
        doc = QtGui.QTextDocument()
        cursor = QtGui.QTextCursor(doc)
        cursor.setCharFormat(CReportBase.ReportTitle)
        cursor.insertText(self.windowTitle())
        self.dumpParams(cursor)
        cursor.insertBlock()
        cursor.setCharFormat(CReportBase.ReportBody)
        cursor.insertText(self.groupBoxIn.title())
        self.modelPrint(self.tblInItems, self.tblInItems.model(), cursor)
        cursor.movePosition(QtGui.QTextCursor.End)
        cursor.insertBlock()
        cursor.insertText(self.groupBoxOut.title())
        self.modelPrint(self.tblOutItems, self.tblOutItems.model(), cursor)
        html = doc.toHtml(QByteArray('utf-8'))
        view = CReportViewDialog(self)
        view.setText(html)
        view.exec_()


    def modelPrint(self, tblItems, model, cursor):
        colWidths  = [ tblItems.columnWidth(i) for i in xrange(model.columnCount()-1) ]
        colWidths.insert(0,10)
        totalWidth = sum(colWidths)
        tableColumns = []
        iColNumber = False
        for iCol, colWidth in enumerate(colWidths):
            widthInPercents = str(max(1, colWidth*90/totalWidth))+'%'
            if iColNumber == False:
                tableColumns.append((widthInPercents, [u'№'], CReportBase.AlignRight))
                iColNumber = True
            tableColumns.append((widthInPercents, [forceString(model._cols[iCol].title())], CReportBase.AlignLeft))
        table = createTable(cursor, tableColumns)
        for iModelRow in xrange(model.rowCount()-1):
            iTableRow = table.addRow()
            table.setText(iTableRow, 0, iModelRow+1)
            for iModelCol in xrange(model.columnCount()):
                index = model.createIndex(iModelRow, iModelCol)
                text = forceString(model.data(index))
                table.setText(iTableRow, iModelCol+1, text)


    def setRecord(self, record):
        CStockMotionBaseDialog.setRecord(self, record)
        self.modelInItems.loadItems(self.itemId())
        self.modelOutItems.loadItems(self.itemId())
        self.setIsDirty(False)


    def getRecord(self):
        record = CStockMotionBaseDialog.getRecord(self)
        getRBComboBoxValue( self.cmbSupplier, record, 'receiver_id')
        getRBComboBoxValue( self.cmbSupplierPerson, record, 'receiverPerson_id')
        record.setValue('type', 3)
        return record


    def saveInternals(self, id):
        self.modelInItems.saveItems(id)
        self.modelOutItems.saveItems(id)


    def checkDataEntered(self):
        result = self._checkStockMotionItemsData(self.tblInItems)
        result = result and self._checkStockMotionItemsData(self.tblOutItems)
#        result = result and (self.cmbPurpose.value() or self.checkInputMessage(u'назначение', False, self.cmbPurpose))
#        result = result and self.checkItemsDataEntered()
        return result


#    def checkItemsDataEntered(self):
#        for row, item in enumerate(self.modelItems.items()):
#            if not self.checkItemDataEntered(row, item):
#                return False
#        return True
#
#
#    def checkItemDataEntered(self, row, item):
#        nomenclatureId = forceString(item.value('nomenclature_id'))
#        qnt            = forceDouble(item.value('qnt'))
#        result = nomenclatureId or self.checkInputMessage(u'лекарственное средство или изделие медицинского назначения', False, self.tblProperties, row, 0)
#        result = result and (qnt or self.checkInputMessage(u'количество', False, self.tblProperties, row, 2))
#        return result

    def addRecipe(self, recipeId, rate, financeId):
        db = QtGui.qApp.db
        table = db.table('rbStockRecipe_Item')
        records = db.getRecordList(table, '*', table['master_id'].eq(recipeId), 'idx')
        models = [self.modelInItems, self.modelOutItems]
        for record in records:
            model = models[forceBool(record.value('isOut'))]
            newRecord = model.getEmptyRecord()
            newRecord.setValue('nomenclature_id', record.value('nomenclature_id'))
            newRecord.setValue('finance_id',      toVariant(financeId))
            newRecord.setValue('qnt',             toVariant(rate*forceDouble(record.value('qnt'))))
            model.addRecord(newRecord)


    @pyqtSignature('')
    def on_btnAddRecipe_clicked(self):
        dialog = CSelectRecipe(self)
        try:
            if dialog.exec_():
                recipeId, rate, financeId = dialog.getResult()
                if recipeId and rate:
                    self.addRecipe(recipeId, rate, financeId)
        finally:
            sip.delete(dialog)


class CItemsModel(CInDocTableModel):
    def __init__(self, parent, isOut):
        CInDocTableModel.__init__(self, 'StockMotion_Item', 'id', 'master_id', parent)
        self.isOut = isOut
        self.addCol(CNomenclatureInDocTableCol(u'ЛСиИМН', 'nomenclature_id', 50, showFields = CRBComboBox.showName))
        self.addCol(CInDocTableCol( u'Партия', 'batch', 16))
        self.addCol(CDateInDocTableCol( u'Годен до', 'shelfTime', 12, canBeEmpty=True))
        self.addCol(CRBInDocTableCol(    u'Тип финансирования', 'finance_id', 15, 'rbFinance'))
        self.addCol(getStockMotionItemQuantityColumn( u'Кол-во', 'qnt', 12))
        self.addCol(CFloatInDocTableCol( u'Сумма', 'sum', 12))
        self.addHiddenCol('isOut')
        self.setFilter('isOut!=0' if isOut else 'isOut=0')


    def getEmptyRecord(self):
        record = CInDocTableModel.getEmptyRecord(self)
        record.setValue('isOut', toVariant(self.isOut))
        return record


class CSelectRecipe(CDialogBase, Ui_SelectRecipeDialog):
    def __init__(self, parent):
        CDialogBase.__init__(self, parent)
        self.setupUi(self)
        self.cmbRecipe.setTable('rbStockRecipe', order='name')
        self.cmbFinance.setTable('rbFinance', True)


    def getResult(self):
        return self.cmbRecipe.value(), self.edtRate.value(), self.cmbFinance.value()
