# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui

from library.PrintInfo                     import CInfo, CInfoList, CDateInfo, CDateTimeInfo, CRBInfo
from library.Utils                         import forceDate, forceDateTime, forceDouble, forceInt, forceRef, forceString

from Orgs.Utils                            import COrgStructureInfo, COrgInfo
from Orgs.PersonInfo import CPersonInfo 
from Registry.Utils                        import CClientInfo
from Stock.NomenclatureComboBox            import getFeaturesAndValues
from Stock.StockMotion                     import stockMotionType


class CUnitInfo(CRBInfo):
    tableName = 'rbUnit'


class CStockMotionItemReasonInfo(CRBInfo):
    tableName = 'rbStockMotionItemReason'

# WFT?
class CLocFinanceInfo(CRBInfo):
    tableName = 'rbFinance'


class CStockMotionItemInfo(CInfo):
    def __init__(self, context, id):
        CInfo.__init__(self, context)
        self.id = id


    def setRecord(self, record):
        self._idx            = forceInt(record.value('idx'))
        self._nomenclature   =  self.getInstance(CLSiIMNInfo, forceRef(record.value('nomenclature_id')))
        self._batch          = forceString(record.value('batch'))
        self._shelfTime      = CDateInfo(forceDate(record.value('shelfTime')))
        self._finance        = self.getInstance(CLocFinanceInfo, forceRef(record.value('finance_id')))
        self._qnt            = forceDouble(record.value('qnt'))
        self._sum            = forceDouble(record.value('sum'))
        self._isOut          = forceInt(record.value('isOut'))
        self._isOutString    = (u'затрата', u'получение')[self._isOut] if self._isOut in (0, 1) else u''
        self._note           = forceString(record.value('note'))
        self._unit            = self.getInstance(CUnitInfo, forceRef(record.value('unit_id')))
        self._reason      = self.getInstance(CStockMotionItemReasonInfo, forceRef(record.value('reason_id')))
        return self


    def setOkLoaded(self):
        CInfo.setOkLoaded(self)
        return self


    def _load(self):
        db = QtGui.qApp.db
        record = db.getRecord('StockMotion_Item', '*', self.id) if self.id else None
        if record:
            self.setRecord(record)
            return True
        else:
            self._idx            = None
            self._nomenclature   =  self.getInstance(CLSiIMNInfo, None)
            self._batch          = u''
            self._shelfTime      = CDateInfo()
            self._finance        = self.getInstance(CLocFinanceInfo, None)
            self._qnt            = 0
            self._sum            = 0
            self._isOut          = None
            self._isOutString    = u''
            self._note           = u''
            self._unit           = self.getInstance(CUnitInfo, None)
            self._reason           = self.getInstance(CStockMotionItemReasonInfo, None)
            return False

    sum = property(lambda self: self.load()._sum)
    qnt = property(lambda self: self.load()._qnt)
    nomenclature = property(lambda self: self.load()._nomenclature)
    batch = property(lambda self: self.load()._batch)
    isOut = property(lambda self: self.load()._isOutString)
    note = property(lambda self: self.load()._note)
    shelfTime = property(lambda self: self.load()._shelfTime)
    finance = property(lambda self: self.load()._finance)
    unit = property(lambda self: self.load()._unit)
    reason = property(lambda self: self.load()._reason)


class CStockMotionItemInfoList(CInfoList):
    def __init__(self,  context, parentId):
        CInfoList.__init__(self, context)
        self._idList = []
        self._parentId = parentId


    def _load(self):
        self._idList = QtGui.qApp.db.getDistinctIdList('StockMotion_Item', where='master_id=%i'%self._parentId)
        self._items = [ self.getInstance(CStockMotionItemInfo, id) for id in self._idList ]
        return True



class CStockMotionInfo(CInfo):
    def __init__(self, context, id):
        CInfo.__init__(self, context)
        self.id = id
        self._type           = 0
        self._number         = ''
        self._date           = CDateTimeInfo()
        self._reason         = ''
        self._reasonDate     = CDateInfo()
        self._items          = []


    def setRecord(self, record):
        if record:
            self.id          = forceInt(record.value('id'))
            self._type       = stockMotionType[forceInt(record.value('type'))][0]
            self._number     = forceString(record.value('number'))
            self._date       = CDateTimeInfo(forceDateTime(record.value('date')))
            self._reason     = forceString(record.value('reason'))
            self._reasonDate = CDateInfo(forceDate(record.value('reasonDate')))
            self._receiver   = self.getInstance(COrgStructureInfo, forceInt(record.value('receiver_id')))
            self._supplier   = self.getInstance(COrgStructureInfo, forceInt(record.value('supplier_id')))
            self._note       = forceString(record.value('note'))
            self._client     = self.getInstance(CClientInfo, forceInt(record.value('client_id')))
            self._supplierPerson   =  self.getInstance(CPersonInfo, forceInt(record.value('supplierPerson_id')))
            self._receiverPerson   =  self.getInstance(CPersonInfo, forceInt(record.value('receiverPerson_id')))
            self._supplierOrgPerson = forceString(record.value('supplierOrgPerson'))
            self._supplierOrg   =  self.getInstance(COrgInfo, forceInt(record.value('supplierOrg_id')))
            self._items      = self.getInstance(CStockMotionItemInfoList, self.id)
            self.setOkLoaded()


    def _load(self):
        db = QtGui.qApp.db
        record = db.getRecord('StockMotion', '*', self.id) if self.id else None
        if record:
            self.setRecord(record)
            return True
        else:
            return False


    def getNomenclatureItem(self, nomecnlatureId):
        for item in self._items:
            if item._nomenclature.id == nomecnlatureId:
                return item
        return None


 #  def __str__(self):
 #     return self.load()._name

    type = property(lambda self: self.load()._type)
    number = property(lambda self: self.load()._number)
    date = property(lambda self: self.load()._date)
    reason = property(lambda self: self.load()._reason)
    reasonDate = property(lambda self: self.load()._reasonDate)
    receiver = property(lambda self: self.load()._receiver)
    supplier = property(lambda self: self.load()._supplier)
    note = property(lambda self: self.load()._note)
    client = property(lambda self: self.load()._client)
    supplierPerson = property(lambda self: self.load()._supplierPerson)
    receiverPerson = property(lambda self: self.load()._receiverPerson)
    supplierOrgPerson = property(lambda self: self.load()._supplierOrgPerson)
    supplierOrg = property(lambda self: self.load()._supplierOrg)
    items = property(lambda self: self.load()._items)


class CStockMotionInfoList(CInfoList):
    def __init__(self,  context, idList):
        CInfoList.__init__(self, context)
        self._class = CStockMotionInfo
        self._idList = idList

    def _load(self):
        self._items = [ self.getInstance(self._class, id) for id in self._idList ]
        return True


class CStockRemainingsInfo(CInfo):
    def __init__(self, context, record):
        CInfo.__init__(self, context)
        self.record = record


    def setRecord(self, record):
        if record:
            db = QtGui.qApp.db
            self._orgStructure       = self.getInstance(COrgStructureInfo, forceInt(record.value('orgStructure_id')))
            self._nomenclature    = self.getInstance(CLSiIMNInfo, forceRef(record.value('nomenclature_id')))
            self._batch                  = forceString(record.value('batch'))
            self._qnt                  = forceDouble(record.value('qnt'))
            self._shelfTime         = CDateInfo(forceDate(record.value('shelfTime')))
            self._finance     = self.getInstance(CLocFinanceInfo, forceRef(record.value('finance_id')))
            self._unit          = self.getInstance(CUnitInfo, forceRef(db.translate(u'rbNomenclature', u'id', record.value('nomenclature_id'), u'unit_id')))
            self._sum          = forceDouble(record.value('sum'))
            self.setOkLoaded()


    def _load(self):
        if self.record:
            self.setRecord(self.record)
            return True
        else:
            return False


    orgStructure = property(lambda self: self.load()._orgStructure)
    nomenclature = property(lambda self: self.load()._nomenclature)
    batch = property(lambda self: self.load()._batch)
    qnt = property(lambda self: self.load()._qnt)
    shelfTime = property(lambda self: self.load()._shelfTime)
    finance = property(lambda self: self.load()._finance)
    unit = property(lambda self: self.load()._unit)
    sum = property(lambda self: self.load()._sum)
    
    
class CStockRemainingsInfoList(CInfoList):
    def __init__(self,  context, records):
        CInfoList.__init__(self, context)
        self._class = CStockRemainingsInfo
        self._records = records

    def _load(self):
        self._items = [ self.getInstance(self._class, record) for record in self._records ]
        return True



class CStockRequisitionsItemInfo(CInfo):
    def __init__(self, context, id):
        CInfo.__init__(self, context)
        self.id = id


    def setRecord(self, record):
        self._nomenclature   =  self.getInstance(CLSiIMNInfo, forceRef(record.value('nomenclature_id')))
        self._finance        = self.getInstance(CLocFinanceInfo, forceRef(record.value('finance_id')))
        self._qnt            = forceDouble(record.value('qnt'))
        self._satisfiedQnt            = forceDouble(record.value('satisfiedQnt'))
        self._unit          = self.getInstance(CUnitInfo, forceRef(record.value('unit_id')))
        return self


    def setOkLoaded(self):
        CInfo.setOkLoaded(self)
        return self


    def _load(self):
        db = QtGui.qApp.db
        record = db.getRecord('StockRequisition_Item', '*', self.id) if self.id else None
        if record:
            self.setRecord(record)
            return True
        else:
            return False

    nomenclature = property(lambda self: self.load()._nomenclature)
    finance = property(lambda self: self.load()._finance)
    qnt = property(lambda self: self.load()._qnt)
    satisfiedQnt = property(lambda self: self.load()._satisfiedQnt)
    unit = property(lambda self: self.load()._unit)


class CStockRequisitionsItemInfoList(CInfoList):
    def __init__(self,  context, parentId):
        CInfoList.__init__(self, context)
        self._idList = []
        self._parentId = parentId


    def _load(self):
        self._idList = QtGui.qApp.db.getDistinctIdList('StockRequisition_Item', where='master_id=%i'%self._parentId)
        self._items = [ self.getInstance(CStockRequisitionsItemInfo, id) for id in self._idList ]
        return True


class CStockRequisitionsInfo(CInfo):
    def __init__(self, context, id):
        CInfo.__init__(self, context)
        self.id = id
        self._items = []


    def setRecord(self, record):
        if record:
            self._number     = forceString(record.value('number'))
            self._date       = CDateTimeInfo(forceDateTime(record.value('date')))
            self._deadline       = CDateTimeInfo(forceDateTime(record.value('date')))
            self._receiver   = self.getInstance(COrgStructureInfo, forceInt(record.value('recipient_id')))
            self._supplier   = self.getInstance(COrgStructureInfo, forceInt(record.value('supplier_id')))            
            self._note       = forceString(record.value('note'))
            self._items      = self.getInstance(CStockRequisitionsItemInfoList, self.id)
            self.setOkLoaded()


    def _load(self):
        db = QtGui.qApp.db
        record = db.getRecord('StockRequisition', '*', self.id) if self.id else None
        if record:
            self.setRecord(record)
            return True
        else:
            self._number = u''
            self._date = CDateTimeInfo()
            self._deadline = CDateTimeInfo()
            self._receiver   = self.getInstance(COrgStructureInfo, None)
            self._supplier   = self.getInstance(COrgStructureInfo, None)            
            self._note       = u''
            self._items      = []
            return False


    number = property(lambda self: self.load()._number)
    date = property(lambda self: self.load()._date)
    deadline = property(lambda self: self.load()._deadline)
    receiver = property(lambda self: self.load()._receiver)
    supplier = property(lambda self: self.load()._supplier)
    note = property(lambda self: self.load()._note)
    items = property(lambda self: self.load()._items)


class CStockRequisitionsInfoList(CInfoList):
    def __init__(self,  context, idList):
        CInfoList.__init__(self, context)
        self._idList = idList

    def _load(self):
        self._items = [ self.getInstance(CStockRequisitionsInfo, id) for id in self._idList ]
        return True


class CLSiIMNTypeInfo(CRBInfo):
    tableName = 'rbNomenclatureType'


class CLFFormInfo(CRBInfo):
    tableName = 'rbLfForm'


class CLSiIMNInfo(CInfo): # wtf
    def __init__(self, context, id):
        CInfo.__init__(self, context)
        self.id = id
        self._code                             = ''
        self._name                             = ''
        self._RegionalCode                     = ''
        self._type                             = None
        self._internationalNonproprietaryName  = ''
        self._producer                         = ''
        self._ATC                              = ''
        self._mnnCode                          = ''
        self._mnnLatin                           = ''
        self._originName                        = ''
        self._trnCode                          = ''
        self._packSize                         = 0
        self._dosageValue                      = ''
        self._dosageUnit                       = ''
        self._lfForm                           = ''
        self._inDate                           = CDateInfo()
        self._exDate                           = CDateInfo()
        self._completeness                     = ''
        self._features                         = dict()


    def _load(self):
        db = QtGui.qApp.db
        record = db.getRecord('rbNomenclature', '*', self.id) if self.id else None
        if record:
            recordUnit = db.getRecord('rbUnit', '*', forceInt(record.value('unit_id')))
            #recordLfForm = db.getRecord('rbLfForm', '*', forceInt(record.value('lfForm_id')))
            self._code                             = forceString(record.value('code'))
            self._name                             = forceString(record.value('name'))
            self._RegionalCode                     = forceString(record.value('regionalCode'))
            self._type                             = self.getInstance(CLSiIMNTypeInfo, forceRef(record.value('type_id')))
            self._internationalNonproprietaryName  = forceString(record.value('internationalNonproprietaryName'))
            self._producer                         = forceString(record.value('producer'))
            self._ATC                              = forceString(record.value('ATC'))
            self._packSize                         = forceInt(record.value('packSize'))
            self._mnnCode                          = forceString(record.value('mnnCode'))
            self._mnnLatin                          = forceString(record.value('mnnLatin'))
            self._originName                          = forceString(record.value('originName'))
            self._trnCode                          = forceString(record.value('trnCode'))
            self._dosageValue                      = forceString(record.value('dosageValue'))
            self._dosageUnit                       = forceString(recordUnit.value('name')) if recordUnit else ''
            self._lfForm                           = self.getInstance(CLFFormInfo, forceRef(record.value('lfForm_id')))
            self._inDate                           = CDateInfo(forceDate(record.value('inDate')))
            self._exDate                           = CDateInfo(forceDate(record.value('exDate')))
            self._completeness                     = forceString(record.value('completeness'))
            self._features                         = getFeaturesAndValues(nomenclatureId = record.value('id'))
            return True
        else:
            return False


    def __str__(self):
        return self.load()._name


    code = property(lambda self: self.load()._code)
    name = property(lambda self: self.load()._name)
    RegionalCode = property(lambda self: self.load()._RegionalCode)
    type = property(lambda self: self.load()._type)
    internationalNonproprietaryName = property(lambda self: self.load()._internationalNonproprietaryName)
    producer = property(lambda self: self.load()._producer)
    ATC = property(lambda self: self.load()._ATC)
    packSize = property(lambda self: self.load()._packSize)
    mnnCode = property(lambda self: self.load()._mnnCode)
    mnnLatin = property(lambda self: self.load()._mnnLatin)
    originName = property(lambda self: self.load()._originName)
    trnCode = property(lambda self: self.load()._trnCode)
    dosageValue = property(lambda self: self.load()._dosageValue)
    dosageUnit = property(lambda self: self.load()._dosageUnit)
    lfForm = property(lambda self: self.load()._lfForm)
    inDate = property(lambda self: self.load()._inDate)
    exDate = property(lambda self: self.load()._exDate)
    completeness = property(lambda self: self.load()._completeness)
    features = property(lambda self: self.load()._features)

