# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################



from PyQt4 import QtGui
from PyQt4.QtCore import Qt, QVariant, SIGNAL

from library.crbcombobox            import CRBComboBox
from library.InDocTable             import CDateInDocTableCol, CInDocTableCol, CRBInDocTableCol, CLocItemDelegate
from library.interchange            import getDatetimeEditValue, getLineEditValue, getRBComboBoxValue, setDatetimeEditValue, setLineEditValue, setRBComboBoxValue
from library.ItemsListDialog        import CItemEditorBaseDialog
from library.PrintTemplates         import CPrintButton
from library.Utils                  import forceDouble, forceRef, forceString, forceStringEx, toVariant

from Stock.NomenclatureComboBox     import CNomenclatureInDocTableCol
from Stock.StockMotionBaseDialog    import CStockMotionBaseDialog, CNomenclatureItemsBaseModel

from Stock.Utils import getStockMotionItemQuantityColumn, getExistsNomenclatureAmount, calcNomenclatureSum
from Stock.Service import CStockService
from Stock.StockModel import CStockMotionType

from Stock.Ui_StockUtilizationDialog import Ui_StockUtilizationDialog


class CStockUtilizationEditDialog(CStockMotionBaseDialog, Ui_StockUtilizationDialog):
    stockDocumentType = CStockMotionType.utilization

    def __init__(self,  parent):
        CStockMotionBaseDialog.__init__(self, parent)

        self.addModels('Items', CItemsModel(self))
        self.addObject('btnPrint', CPrintButton(self, u'Печать'))
        self.btnPrint.setShortcut('F6')
        self.setupUi(self)
        self.cmbSupplierPerson.setSpecialityIndependents()
        self.setWindowFlags(self.windowFlags() | Qt.WindowMaximizeButtonHint)
        self.setupDirtyCather()
        self.buttonBox.addButton(self.btnPrint, QtGui.QDialogButtonBox.ActionRole)

        self.tblItems.setModel(self.modelItems)
        self.prepareItemsPopupMenu(self.tblItems)
        self.tblItems.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
        self.tblItems.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)

        self.tblItems.setItemDelegateForColumn(CItemsModel.batchColumnIndex, CBatchItemDelegate(self.tblItems))

        self.setWindowTitleEx(u'Утилизация')

    def checkDataEntered(self):
        result = self._checkStockMotionItemsData(self.tblItems)
        result = result and self.checkItemsDataEntered()
        return result

    def checkItemsDataEntered(self):
        supplierId = self.cmbSupplier.value()
        for row, item in enumerate(self.modelItems.items()):
            if supplierId and not self.checkNomenclatureExists(row, item, supplierId):
                return False
        return True


    def checkNomenclatureExists(self, row, item, supplierId=None):
        supplierId = supplierId or self.cmbSupplier.value()
        financeId = forceRef(item.value('finance_id'))
        batch = forceString(item.value('batch'))
        nomenclature_id = forceRef(item.value('nomenclature_id'))
        unitId = forceRef(item.value('unit_id'))
        existsQnt = getExistsNomenclatureAmount(nomenclature_id, financeId, batch, supplierId, unitId)
        qnt = forceDouble(item.value('qnt'))
        if existsQnt < qnt:
            nomenclatureName = self.modelItems.getNomenclatureNameById(nomenclature_id)
            if existsQnt > 0:
                message = u'На складе {0} {1}, а списание на {2}'.format(existsQnt, nomenclatureName, qnt)
            else:
                message = u'На складе отсутствует %s' % nomenclatureName
            return self.checkValueMessage(message, False, self.tblItems, row, self.modelItems.qntColumnIndex)
        return True

    def setRecord(self, record):
        CItemEditorBaseDialog.setRecord(self, record)
        setLineEditValue(    self.edtNumber,          record, 'number')
        setDatetimeEditValue(self.edtDate, self.edtTime, record, 'date')
        setRBComboBoxValue(  self.cmbSupplier,        record, 'supplier_id')
        setRBComboBoxValue(  self.cmbSupplierPerson,  record, 'supplierPerson_id')
        setLineEditValue(    self.edtNote,            record, 'note')
        self.modelItems.loadItems(self.itemId())
        self.modelItems.setStockMotion(CStockService.getStockMotionByRecord(record))
        self.setIsDirty(False)


    def getRecord(self):
        record = CItemEditorBaseDialog.getRecord(self)
        getLineEditValue(   self.edtNumber,         record, 'number')
        getDatetimeEditValue(self.edtDate, self.edtTime, record, 'date', True)
        getRBComboBoxValue( self.cmbSupplier,       record, 'supplier_id')
        getRBComboBoxValue( self.cmbSupplierPerson, record, 'supplierPerson_id')
        getLineEditValue(   self.edtNote,           record, 'note')
        record.setValue('type', self.stockDocumentType)
        return record

    def saveInternals(self, id):
        self.modelItems.saveItems(id)

    def _on_cmbSupplierChanged(self):
        if not self._record:
            self._record = self.getRecord()
            self.modelItems.setStockMotion(CStockService.getStockMotionByRecord(self._record))

        self._record.setValue('supplier_id', self.cmbSupplier.value())

# #####################################################################################################################

class CStockInternalConsumptionEditDialog(CStockUtilizationEditDialog, Ui_StockUtilizationDialog):
    stockDocumentType = CStockMotionType.internalConsumption
    
    def __init__(self,  parent):
        #CStockUtilizationEditDialog.__init__(self, parent)
       
        
        CStockMotionBaseDialog.__init__(self, parent)
        self.addModels('Items', CInternalConsumptionItemsModel(self))
        
        self.addObject('btnPrint', CPrintButton(self, u'Печать'))
        self.btnPrint.setShortcut('F6')
        self.setupUi(self)
        self.cmbSupplierPerson.setSpecialityIndependents()
        self.setWindowFlags(self.windowFlags() | Qt.WindowMaximizeButtonHint)
        self.setupDirtyCather()
        self.buttonBox.addButton(self.btnPrint, QtGui.QDialogButtonBox.ActionRole)

        self.tblItems.setModel(self.modelItems)
        self.prepareItemsPopupMenu(self.tblItems)
        self.tblItems.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
        self.tblItems.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
        
        self.tblItems.setItemDelegateForColumn(CInternalConsumptionItemsModel.batchColumnIndex, CBatchItemDelegate(self.tblItems))
        self.setWindowTitleEx(u'Внутреннее потребление')


class CBatchItemDelegate(CLocItemDelegate):
    def createEditor(self, parent, option, index):
        column = index.column()
        row = index.row()
        editor = index.model().createBatchEditor(row, parent)
        self.connect(editor, SIGNAL('commit()'), self.emitCommitData)
        self.connect(editor, SIGNAL('editingFinished()'), self.commitAndCloseEditor)
        self.editor   = editor
        self.row      = row
        self.rowcount = index.model().rowCount(None)
        self.column   = column
        return editor


    def setEditorData(self, editor, index):
        if editor is not None:
            row    = index.row()
            model  = index.model()
            if row < len(model.items()):
                record = model.items()[row]
            else:
                record = model.getEmptyRecord()
            model.setBatchEditorData(row, editor, model.data(index, Qt.EditRole), record)

    def setModelData(self, editor, model, index):
        if editor is not None:
            model.setData(index, index.model().getBatchEditorData(index.row(), editor))


class CLocNomenclatureCol(CNomenclatureInDocTableCol):
    def createEditor(self, parent):
        editor = CNomenclatureInDocTableCol.createEditor(self, parent)
        editor.setOrgStructureId(QtGui.qApp.currentOrgStructureId())
        editor.setOnlyExists()
        return editor


class CItemsModel(CNomenclatureItemsBaseModel):
    nomenclatureColumnIndex = 0
    batchColumnIndex = 1
    shelfTimeColumnIndex = 2
    financeColumnIndex = 3
    qntColumnIndex = 4
    unitColumnIndex = 5
    reasonColumnIndex = 6

    def __init__(self, parent, showExists=False):
        self.priceCache = parent
        CNomenclatureItemsBaseModel.__init__(self, 'StockMotion_Item', 'id', 'master_id', parent)
        self._unitColumn = CRBInDocTableCol(u'Ед.Учета', 'unit_id', 12, 'rbUnit', addNone=False)
        self._nomenclatureColumn = CLocNomenclatureCol(u'ЛСиИМН', 'nomenclature_id', 50, showFields = CRBComboBox.showName)
        self._batchCol = CInDocTableCol(u'Партия', 'batch', 16)
        self._unitColumn = CRBInDocTableCol(u'Ед.Учета', 'unit_id', 12, 'rbUnit', addNone=False)

        self.addCol(self._nomenclatureColumn)
        self.addCol(self._batchCol)
        self.addCol(CDateInDocTableCol( u'Годен до', 'shelfTime', 12, canBeEmpty=True))
        self.addCol(CRBInDocTableCol(    u'Тип финансирования', 'finance_id', 15, 'rbFinance'))
        self.addCol(getStockMotionItemQuantityColumn( u'Количество бракованных ЛСиИМН', 'qnt', 12))
        self.addCol(self._unitColumn)
        self.addCol(
            CRBInDocTableCol(
                u'Причина утилизации', 'reason_id', 12, 'rbStockMotionItemReason',
                filter='stockMotionType=%s' % CStockMotionType.utilization
            )
        )
        self.addHiddenCol('sum')

        self._batchCache = {}
        self._financeCache = {}
        self._shelfTimeCache = {}

        self._setterHandlers = {
            self.nomenclatureColumnIndex: self._handleNomenclatureSet,
            self.batchColumnIndex: self._handleBatchSet,
            self.shelfTimeColumnIndex: self._handleShelfTimeSet,
            self.financeColumnIndex: self._handleFinanceIdSet,
            self.unitColumnIndex: self._handleUnitIdSet, 
            self.qntColumnIndex: self._handleQntSet
        }

        self._stockMotion = None

    def setStockMotion(self, stockMotion):
        self._stockMotion = stockMotion

    def _handleNomenclatureSet(self, stockMotionItem, value):
        previousValue = stockMotionItem.nomenclature_id
        stockMotionItem.nomenclature_id = value
        if previousValue != stockMotionItem.nomenclature_id:
            CStockService.setFinanceBatchShelfTime(stockMotionItem)
            stockMotionItem.unit_id = self.getDefaultClientUnitId(stockMotionItem.nomenclature_id)
            stockMotionItem.qnt = 1.0
            stockMotionItem.setSum(calcNomenclatureSum(stockMotionItem))
            return True
        return False

    def _handleUnitIdSet(self, stockMotionItem, value):
        previousValue = stockMotionItem.unit_id
        stockMotionItem.unit_id = value
        if previousValue != stockMotionItem.unit_id:
            ratio = self.getRatio(stockMotionItem.nomenclature_id, previousValue, stockMotionItem.unit_id)
            stockMotionItem.qnt = stockMotionItem.qnt * ratio
            stockMotionItem.setSum(calcNomenclatureSum(stockMotionItem))
            return True
        return False

    def _handleBatchSet(self, stockMotionItem, value):
        result = stockMotionItem.setBatch(value)
        if result:
            CStockService.setFinanceAndShelfTimeByBatch(stockMotionItem)
            stockMotionItem.setSum(calcNomenclatureSum(stockMotionItem))
        return result

    def _handleQntSet(self, stockMotionItem, value):
        result = stockMotionItem.setQnt(value)
        if result:
            stockMotionItem.setSum(calcNomenclatureSum(stockMotionItem))
        return True

    def _handleShelfTimeSet(self, stockMotionItem, value):
        result =  stockMotionItem.setShelfTime(value)
        if result:
            CStockService.setFinanceAndBatchByShelfTime(stockMotionItem)
            stockMotionItem.setSum(calcNomenclatureSum(stockMotionItem))
        return result

    def _handleFinanceIdSet(self, stockMotionItem, value):
        result = stockMotionItem.setFinanceId(value)
        if result:
            CStockService.setBatchAndShelfTimeByFinance(stockMotionItem)
            stockMotionItem.setSum(calcNomenclatureSum(stockMotionItem))
        return result

    def setData(self, index, value, role=Qt.EditRole):
        if role != Qt.EditRole:
            return CNomenclatureItemsBaseModel.setData(self, index, value, role)

        columnIndex = index.column()

        if columnIndex in self._setterHandlers.keys():
            row = index.row()
            if row == len(self._items):
                if value.isNull():
                    return False
                self._addEmptyItem()

            stockMotionItem = CStockService(self._stockMotion).getStockMotionItemByRecord(self._items[index.row()])

            if self._setterHandlers[columnIndex](stockMotionItem, value):
                self.emitRowChanged(index.row())
                return True
            return False

        else:
            CNomenclatureItemsBaseModel.setData(self, index, value, role)

    def getNomenclatureNameById(self, nomenclatureId):
        return forceString(self._nomenclatureColumn.toString(nomenclatureId, None))

    def _getBatchEditorItemsByRow(self, row):
        if 0 <= row < len(self._items):
            stockMotionItem = CStockService(self._stockMotion).getStockMotionItemByRecord(self._items[row])
            if stockMotionItem.id not in self._batchCache:
                batchList = [u''] + CStockService.getBatchListDependOnFinanceAndShelfTime(stockMotionItem)
                self._batchCache[stockMotionItem.id] = batchList
            return self._batchCache[stockMotionItem.id]
        return [u'']

    def _getNomenclatureFinanceIdList(self, record, nomenclatureId):
        stockMotionItem = CStockService(self._stockMotion).getStockMotionItemByRecord(record)
        if stockMotionItem.id not in self._financeCache:
            financeIdList = CStockService.getFinanceIdListDependOnBatchAndShelfTime(stockMotionItem)
            self._financeCache[stockMotionItem.id] = financeIdList
        return self._financeCache[stockMotionItem.id]

    def _setSuitableFinanceValue(self, item, nomenclatureId, oldNomenclatureId):
        if nomenclatureId != oldNomenclatureId:
            for financeId in self._getNomenclatureFinanceIdList(item, nomenclatureId):
                if financeId:
                    item.setValue('finance_id', QVariant(financeId))
                    return True
            item.setValue('finance_id', QVariant(None))
            return True
        return False

    def createBatchEditor(self, row, parent):
        editor = QtGui.QComboBox(parent)
        editor.addItems(self._getBatchEditorItemsByRow(row))
        return editor

    def setBatchEditorData(self, row, editor, value, record):
        index = editor.findText(forceString(value), Qt.MatchFixedString)
        if index < 0:
            for locIndex in xrange(editor.count()):
                if forceStringEx(editor.itemText(locIndex)):
                    editor.setCurrentIndex(locIndex)
                    return
            editor.setCurrentIndex(0)
        editor.setCurrentIndex(index)

    def getBatchEditorData(self, row, editor):
        return toVariant(editor.currentText())

    def getEmptyRecord(self):
        record = CNomenclatureItemsBaseModel.getEmptyRecord(self)
        record.setValue('qnt', QVariant(1))
        return record

    def flags(self, index):
        column = index.column()
        if column in (
                self.batchColumnIndex, self.shelfTimeColumnIndex, self.financeColumnIndex, self.unitColumnIndex
        ):
            row = index.row()
            if 0 <= row < len(self._items):
                item = self._items[row]
                if not forceRef(item.value('nomenclature_id')):
                    return Qt.ItemIsSelectable | Qt.ItemIsEnabled
            else:
                return Qt.ItemIsSelectable | Qt.ItemIsEnabled
        return CNomenclatureItemsBaseModel.flags(self, index)


class CInternalConsumptionItemsModel(CItemsModel):
    nomenclatureColumnIndex = 0
    batchColumnIndex = 1
    shelfTimeColumnIndex = 2
    financeColumnIndex = 3
    qntColumnIndex = 4
    unitColumnIndex = 5
    
    
    def __init__(self, parent, showExists=False):
        #CItemsModel.__init__(self, parent)
        self.priceCache = parent
        CNomenclatureItemsBaseModel.__init__(self, 'StockMotion_Item', 'id', 'master_id', parent)
        self._unitColumn = CRBInDocTableCol(u'Ед.Учета', 'unit_id', 12, 'rbUnit', addNone=False)
        self._nomenclatureColumn = CLocNomenclatureCol(u'ЛСиИМН', 'nomenclature_id', 50, showFields = CRBComboBox.showName)
        self._batchCol = CInDocTableCol(u'Партия', 'batch', 16)
        self._unitColumn = CRBInDocTableCol(u'Ед.Учета', 'unit_id', 12, 'rbUnit', addNone=False)

        self.addCol(self._nomenclatureColumn)
        self.addCol(self._batchCol)
        self.addCol(CDateInDocTableCol( u'Годен до', 'shelfTime', 12, canBeEmpty=True))
        self.addCol(CRBInDocTableCol(    u'Тип финансирования', 'finance_id', 15, 'rbFinance'))
        self.addCol(getStockMotionItemQuantityColumn( u'Количество ЛСиИМН', 'qnt', 12))
        self.addCol(self._unitColumn)
        self.addHiddenCol('sum')
        
        self._batchCache = {}
        self._financeCache = {}
        self._shelfTimeCache = {}

        self._setterHandlers = {
            self.nomenclatureColumnIndex: self._handleNomenclatureSet,
            self.batchColumnIndex: self._handleBatchSet,
            self.shelfTimeColumnIndex: self._handleShelfTimeSet,
            self.financeColumnIndex: self._handleFinanceIdSet,
            self.unitColumnIndex: self._handleUnitIdSet, 
            self.qntColumnIndex: self._handleQntSet
        }

        self._stockMotion = None

