# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import sip
from PyQt4 import QtGui
from PyQt4.QtCore import (Qt,
                          pyqtSignature,
                          SIGNAL,
                          QByteArray,
                          QChar,
                          QDate,
                          QDateTime,
                          QEvent,
                          QObject,
                          QString,
                          QTime,
                          QVariant,
                         )
from library.TableModel import (CTableModel,
                                CBoolCol,
                                CCol,
                                CDateCol,
                                CDesignationCol,
                                CEnumCol,
                                CIntCol,
                                CNameCol,
                                CNumCol,
                                CRefBookCol,
                                CTextCol,
                                CDoubleCol,
                               )
from library.TableView  import CTableView
from library.DialogBase import CDialogBase
from library.Utils      import (calcAge,
                                copyFields,
                                forceBool,
                                forceDate,
                                forceDateTime,
                                forceInt,
                                forceRef,
                                forceString,
                                forceStringEx,
                                forceTime,
                                formatDate,
                                formatName,
                                formatRecordsCount,
                                toVariant,
                                trim,
                               )
from Events.EditDispatcher import getEventFormClass
from Events.AmbCardDialog  import CAmbCardDialog
from Events.EventInfo      import CDiagnosticInfoIdList
from Registry.AmbCardMixin import CAmbCardMixin
from Registry.Utils        import (CCheckNetMixin,
                                   formatClientString,
                                   getClientInfo,
                                   getClientInfoEx,
                                   getClientPhonesEx,
                                   CClientInfo,
                                   CClientInfoListEx,
                                   formatAddressInt,
                                   formatDocument,
                                   formatPolicy,
                                   getClientBanner,
                                   getClientContextData,
                                   getClientMiniInfo,
                                   formatSocStatuses,
                                  )
from Registry.RegistryTable     import CClientEvalCol, CSNILSCol
from Registry.ClientEditDialog  import CClientEditDialog
from Reports.ReportBase         import CReportBase, createTable
from Reports.ReportView         import CReportViewDialog
from Reports.Utils              import dateRangeAsStr
from library.PrintInfo          import CInfoContext
from Orgs.Utils                 import getOrgStructureAddressIdList, getOrgStructureDescendants, getOrganisationShortName, getOrgStructureFullName, getPersonInfo
from KLADR.Utils                import getLikeMaskForRegion
from library.PrintTemplates     import getPrintButton, addButtonActions, applyTemplate, CPrintAction, getPrintTemplates
from Users.Rights               import urSurReadEvent, urSurReadClientInfo, urSurEditEvent, urSurEditClientInfo, urRegTabReadAmbCard, urRegTabWriteAmbCard
from HospitalBeds.HospitalizationEventDialog import CFindClientInfoDialog

from Ui_SurveillanceDialog import Ui_SurveillanceDialog


class CSurveillanceDialog(CDialogBase, CAmbCardMixin, CCheckNetMixin, Ui_SurveillanceDialog):
    def __init__(self, parent):
        CDialogBase.__init__(self, parent)
        CCheckNetMixin.__init__(self)
        self.addModels('ConsistsClients',    CConsistsClientsModel(self))
        self.addModels('ConsistsMonitoring', CConsistsMonitoringModel(self))
        self.addModels('TakenClients',       CTakenClientsModel(self))
        self.addModels('TakenMonitoring',    CTakenMonitoringModel(self))
        self.addModels('RemoveClients',      CRemoveClientsModel(self))
        self.addModels('RemoveMonitoring',   CRemoveMonitoringModel(self))
        self.addObject('btnPrint', getPrintButton(self, ''))
        self.setupClientsMenu()
        self.setupMonitoringMenu()
        self.setupUi(self)
        self.setWindowFlags(self.windowFlags() | Qt.WindowMaximizeButtonHint)
        self.setModels(self.tblConsistsClients,  self.modelConsistsClients, self.selectionModelConsistsClients)
        self.setModels(self.tblConsistsMonitoring,  self.modelConsistsMonitoring, self.selectionModelConsistsMonitoring)
        self.setModels(self.tblTakenClients,  self.modelTakenClients, self.selectionModelTakenClients)
        self.setModels(self.tblTakenMonitoring,  self.modelTakenMonitoring, self.selectionModelTakenMonitoring)
        self.setModels(self.tblRemoveClients,  self.modelRemoveClients, self.selectionModelRemoveClients)
        self.setModels(self.tblRemoveMonitoring,  self.modelRemoveMonitoring, self.selectionModelRemoveMonitoring)
        self.buttonBox.addButton(self.btnPrint, QtGui.QDialogButtonBox.ActionRole)
        self.tblConsistsClients.setPopupMenu(self.mnuClients)
        self.tblConsistsMonitoring.setPopupMenu(self.mnuMonitoring)
        self.tblTakenClients.setPopupMenu(self.mnuClients)
        self.tblTakenMonitoring.setPopupMenu(self.mnuMonitoring)
        self.tblRemoveClients.setPopupMenu(self.mnuClients)
        self.tblRemoveMonitoring.setPopupMenu(self.mnuMonitoring)
        self.tblConsistsClients.enableColsMove()
        self.tblConsistsMonitoring.enableColsMove()
        self.tblTakenClients.enableColsMove()
        self.tblTakenMonitoring.enableColsMove()
        self.tblRemoveClients.enableColsMove()
        self.tblRemoveMonitoring.enableColsMove()
        self.txtConsistsClientInfoBrowser.actions.append(self.actEditClientInfoBeds)
        self.txtTakenClientInfoBrowser.actions.append(self.actEditClientInfoBeds)
        self.txtRemoveClientInfoBrowser.actions.append(self.actEditClientInfoBeds)
        self.cmbFilterEventSpeciality.setTable(u'rbSpeciality')
        self.cmbSocStatusesType.setTable('rbSocStatusType', True)
        self.cmbFilterAccountingSystem.setTable('rbAccountingSystem',  True)
        self.cmbFilterAccountingSystem.setValue( forceRef(QtGui.qApp.preferences.appPrefs.get('FilterAccountingSystem', 0)))
        self.filter = {}
        self.resetFilter()
        self.fillingSpeciality()
        self.on_tabDispensaireClients_currentChanged(0)
        surveillance = getPrintTemplates(getSurveillanceContext())
        if not surveillance:
            self.btnPrint.setId(-1)
        else:
            for template in surveillance:
                action = CPrintAction(template[0], template[1], self.btnPrint, self.btnPrint)
                self.btnPrint.addAction(action)
            self.btnPrint.menu().addSeparator()
            self.btnPrint.addAction(CPrintAction(u'Список Диспансерное наблюдение', -1, self.btnPrint, self.btnPrint))
        self.btnPrint.setEnabled(True)


    def setupClientsMenu(self):
        self.addObject('mnuClients', QtGui.QMenu(self))
        self.addObject('actEditClientInfoBeds', QtGui.QAction(u'Открыть регистрационную карточку', self))
        self.addObject('actAmbCardShow',    QtGui.QAction(u'Открыть медицинскую карту', self))
        self.actEditClientInfoBeds.setShortcut('Shift+F4')
        self.mnuClients.addAction(self.actEditClientInfoBeds)
        self.mnuClients.addAction(self.actAmbCardShow)


    def setupMonitoringMenu(self):
        self.addObject('mnuMonitoring', QtGui.QMenu(self))
        self.addObject('actOpenEvent', QtGui.QAction(u'Открыть обращение', self))
        self.actOpenEvent.setShortcut(Qt.Key_F4)
        self.mnuMonitoring.addAction(self.actOpenEvent)


    def dumpParams(self, cursor, params):
        description = self.getDescription(params)
        columns = [ ('100%', [], CReportBase.AlignLeft) ]
        table = createTable(cursor, columns, headerRowCount=len(description), border=0, cellPadding=2, cellSpacing=0)
        for i, row in enumerate(description):
            table.setText(i, 0, row)
        cursor.movePosition(QtGui.QTextCursor.End)
        cursor.insertBlock()


    def getDescription(self, params):
        db = QtGui.qApp.db
        rows = []
        begDate = params.get('begDate', QDate())
        endDate = params.get('endDate', QDate())
        if begDate or endDate:
            rows.append(dateRangeAsStr(u'За период', begDate, endDate))
        orgStructureId = params.get('orgStructureId', None)
        if orgStructureId:
            rows.append(u'\n' + u'Подразделение врача: ' + forceString(QtGui.qApp.db.translate('OrgStructure', 'id', orgStructureId, 'name')))
        else:
            rows.append(u'\n' + u'ЛПУ')
        specialityType = params.get('specialityType', 0)
        specialityId = params.get('specialityId', None)
        if specialityId:
            rows.append(u'Cпециальность: ' + [u'Отбор по "ИЛИ": ', u'Отбор по "И": '][specialityType] + forceString(db.translate('rbSpeciality', 'id', specialityId, 'name')))
        personId = params.get('personId', None)
        if personId:
            personInfo = getPersonInfo(personId)
            rows.append(u'Врач: ' + personInfo['shortName']+', '+personInfo['specialityName'])
        MKBFilter = params.get('MKBFilter', 0)
        MKBFrom = params.get('MKBFrom', '')
        MKBTo = params.get('MKBTo', '')
        if MKBFilter == 1:
            rows.append(u'Код МКБ: с "%s" по "%s"' % (MKBFrom, MKBTo))
        accountingSystemId = params.get('accountingSystemId', None)
        if accountingSystemId:
            accountingSystemName = forceString(QtGui.qApp.db.translate('rbAccountingSystem', 'id', accountingSystemId, 'name'))
            titleDescription += u'\n' + u'Внешняя учётная система: ' + accountingSystemName
            rows.append(u'Внешняя учётная система: ' + accountingSystemName)
        filterClientId = params.get('filterClientId', None)
        if filterClientId:
            titleDescription += u'\n' + u'Идентификатор пациента: %s'%(str(filterClientId))
            rows.append(u'Идентификатор пациента: %s'%(str(filterClientId)))
        sex = params.get('sex', 0)
        ageFrom = params.get('ageFrom', None)
        ageTo = params.get('ageTo', None)
        if sex:
            rows.append(u'Пол: ' + formatSex(sex))
        if ageFrom is not None and ageTo is not None and ageFrom <= ageTo:
            rows.append(u'Возраст: c %d по %d %s' % (ageFrom, ageTo, agreeNumberAndWord(ageTo, (u'год', u'года', u'лет'))))
        death = params.get('death', 0)
        if death == 1:
            rows.append(u'Летальность: только живые')
        elif death == 2:
            deathBegDate = params.get('deathBegDate', None)
            deathEndDate = params.get('deathEndDate', None)
            rows.append(u'Летальность: только умершие в период с %s по %s'%(deathBegDate.toString('dd.MM.yyyy') if deathBegDate else u'', deathEndDate.toString('dd.MM.yyyy') if deathEndDate else u''))
        attachOrganisationId = params.get('attachOrganisationId', None)
        if attachOrganisationId:
            isNotAttachOrganisation = params.get('isNotAttachOrganisation', False)
            lpu = forceString(db.translate('Organisation', 'id', attachOrganisationId, 'shortName'))
            rows.append((u'Не имеет прикрепления к выбранному ЛПУ: ' if isNotAttachOrganisation else u'Прикрепление к ЛПУ: ') + lpu)
        if params.get('isFilterAddressOrgStructure', False):
            addressOrgStructureTypeId = params.get('addressOrgStructureTypeId', None)
            addressOrgStructureId = params.get('addressOrgStructureId', None)
            if addressOrgStructureId:
                addressOrgStructureType = [u'Регистрация', u'Проживание', u'Регистрация или проживание', u'Прикрепление', u'Регистрация или прикрепление', u'Проживание или прикрепление', u'Регистрация, проживание или прикрепление']
                rows.append(u'\n' + u'По участку ' + addressOrgStructureType[addressOrgStructureTypeId] + u': ' + forceString(QtGui.qApp.db.translate('OrgStructure', 'id', addressOrgStructureId, 'name')))
        if params.get('isFilterAddress', False):
            addressTypeId = params.get('addressTypeId', None)
            typeAdress = [u'регистрации', u'проживания']
            filterAddressOkato = params.get('addressOkato', 0)
            filterAddressCity = params.get('addressCity', None)
            filterAddressStreet = params.get('addressStreet', None)
            if filterAddressOkato or filterAddressCity or filterAddressStreet:
                filterAddressHouse = params.get('addressHouse', u'')
                filterAddressCorpus = params.get('addressCorpus', u'')
                filterAddressFlat = params.get('addressFlat', u'')
                rows.append(u'Адрес: ' + forceString(typeAdress[addressTypeId]) + u':')
                rows.append((u'город ' + forceString(db.translate(u'kladr.KLADR', u'CODE', filterAddressCity, u'NAME')) if filterAddressCity else u'')
                                    + ((u'район ' + forceString(db.translate(u'kladr.OKATO', u'CODE', filterAddressOkato, u'NAME'))) if filterAddressOkato else u'')
                                    + (u' улица ' + forceString(db.translate(u'kladr.STREET', u'CODE', filterAddressStreet, u'NAME')) if filterAddressStreet else u'')
                                    + (u' дом ' + forceString(filterAddressHouse) if filterAddressHouse else u'')
                                    + (u' корпус ' + forceString(filterAddressCorpus) if filterAddressCorpus else u'')
                                    + (u' квартира ' + forceString(filterAddressFlat) if filterAddressFlat else u''))
        if params.get('isSocStatuses', False):
            socStatusesClass = params.get('socStatusesClass', None)
            socStatusTypeId  = params.get('socStatusesType', None)
            if socStatusTypeId:
                rows.append(u'Тип соц.статуса: ' + forceString(db.translate('vrbSocStatusType', 'id', socStatusTypeId, 'name')))
            if socStatusesClass:
                rows.append(u'Класс соц.статуса: ' + forceString(db.translate('rbSocStatusClass', 'id', socStatusesClass, 'name')))
            if socStatusTypeId or socStatusesClass:
                socStatusesBegDate = params.get('socStatusesBegDate', QDate())
                socStatusesEndDate = params.get('socStatusesEndDate', QDate())
                rows.append(dateRangeAsStr(u'За период', socStatusesBegDate, socStatusesEndDate))
        if params.get('isFilterEvent', False):
            eventVisitType = params.get('eventVisitType', None)
            if eventVisitType is not None:
                eventVisitTypeList = [u'обращались', u'не обращались']
                chkEventVisitDiagnosis = params.get('chkEventVisitDiagnosis', 0)
                rows.append(u'Обращения: ' + (u'по Диспансерному наблюдению ') if chkEventVisitDiagnosis else u'') + eventVisitTypeList[eventVisitType]
                eventBegDate = params.get('eventBegDate', QDate())
                eventEndDate = params.get('eventEndDate', QDate())
                rows.append(dateRangeAsStr(u'За период', eventBegDate, eventEndDate))
        rows.append(u'отчёт составлен: '+forceString(QDateTime.currentDateTime()))
        return rows


    @pyqtSignature('int')
    def on_cmbFilterAttachOrganisation_currentIndexChanged(self, index):
        orgId = self.cmbFilterAttachOrganisation.value()
        self.chkFilterNotAttachOrganisation.setEnabled(bool(orgId))


    @pyqtSignature('int')
    def on_cmbFilterDeath_currentIndexChanged(self, index):
        value = self.cmbFilterDeath.currentIndex()
        self.edtFilterDeathBegDate.setEnabled(bool(value == 2))
        self.edtFilterDeathEndDate.setEnabled(bool(value == 2))


    @pyqtSignature('int')
    def on_btnPrint_printByTemplate(self, templateId):
        widgetIndex = self.tabDispensaireClients.currentIndex()
        if widgetIndex == 0:
            model = self.modelConsistsClients
            tableClients = self.tblConsistsClients
            messag = u'Состоят'
            modelDiagnostic = self.modelConsistsMonitoring
        elif widgetIndex == 1:
            model = self.modelTakenClients
            tableClients = self.tblTakenClients
            messag = u'Взяты'
            modelDiagnostic = self.modelTakenMonitoring
        elif widgetIndex == 2:
            model = self.modelRemoveClients
            tableClients = self.tblRemoveClients
            messag = u'Сняты'
            modelDiagnostic = self.modelRemoveMonitoring
        if templateId == -1:
            doc = QtGui.QTextDocument()
            cursor = QtGui.QTextCursor(doc)
            cursor.setCharFormat(CReportBase.ReportTitle)
            cursor.insertText(u'Диспансерное наблюдение %s'%(messag))
            cursor.setCharFormat(CReportBase.ReportBody)
            cursor.insertBlock()
            self.dumpParams(cursor, self.filter)
            cursor.insertBlock()
            colWidths  = [ tableClients.columnWidth(i) for i in xrange(model.columnCount()) ]
            totalWidth = sum(colWidths)
            tableColumns = []
            for iCol, colWidth in enumerate(colWidths):
                widthInPercents = str(max(1, colWidth*90/totalWidth))+'%'
                tableColumns.append((widthInPercents, [forceString(model._cols[iCol].title())], CReportBase.AlignLeft))
            table = createTable(cursor, tableColumns)
            for iModelRow in xrange(model.rowCount()):
                iTableRow = table.addRow()
                for iModelCol in xrange(model.columnCount()):
                    index = model.createIndex(iModelRow, iModelCol)
                    text = forceString(model.data(index))
                    table.setText(iTableRow, iModelCol, text)
            html = doc.toHtml(QByteArray('utf-8'))
            view = CReportViewDialog(self)
            view.setText(html)
            view.exec_()
        else:
            context = CInfoContext()
            clientInfoList = context.getInstance(CClientInfoListEx, tuple(model.idList()))
            diagnosticsInfo = context.getInstance(CDiagnosticInfoIdList, tuple(modelDiagnostic.idList()))
            data = { 'diagnostics': diagnosticsInfo,
                     'clientsInfo': clientInfoList }
            QtGui.qApp.call(self, applyTemplate, (self, templateId, data))


    @pyqtSignature('QAbstractButton*')
    def on_buttonBoxFilter_clicked(self, button):
        buttonCode = self.buttonBoxFilter.standardButton(button)
        if buttonCode == QtGui.QDialogButtonBox.Apply:
            self.setFilter()
            self.on_tabDispensaireClients_currentChanged(self.tabDispensaireClients.currentIndex())
        elif buttonCode == QtGui.QDialogButtonBox.Reset:
            self.resetFilter()


    def setFilter(self):
        self.filter = {}
        self.filter['personId'] = self.cmbFilterEventPerson.value()
        self.filter['begDate'] = self.edtFilterEventBegDate.date()
        self.filter['endDate'] = self.edtFilterEventEndDate.date()
        self.filter['orgStructureId'] = self.cmbFilterEventOrgStructure.value()
        self.filter['specialityType'] = self.cmbFilterEventSpecialityType.currentIndex()
        self.filter['specialityId'] = self.cmbFilterEventSpeciality.value()
        self.filter['MKBFilter'] = self.cmbMKBFilter.currentIndex()
        self.filter['MKBFrom']   = unicode(self.edtMKBFrom.text())
        self.filter['MKBTo']     = unicode(self.edtMKBTo.text())
        self.filter['sex'] = self.cmbSex.currentIndex()
        self.filter['ageFor'] = self.spbAgeFor.value()
        self.filter['ageTo'] = self.spbAgeTo.value()
        death = self.cmbFilterDeath.currentIndex()
        self.filter['death'] = death
        if death == 2:
            self.filter['deathBegDate'] = self.edtFilterDeathBegDate.date()
            self.filter['deathEndDate'] = self.edtFilterDeathEndDate.date()
        self.filter['attachOrganisationId'] = self.cmbFilterAttachOrganisation.value()
        self.filter['isNotAttachOrganisation'] = self.chkFilterNotAttachOrganisation.isChecked()
        self.filter['isFilterAddressOrgStructure'] = self.chkFilterAddressOrgStructure.isChecked()
        if self.chkFilterAddressOrgStructure.isChecked():
            self.filter['addressOrgStructureTypeId'] = self.cmbFilterAddressOrgStructureType.currentIndex()
            self.filter['addressOrgStructureId'] = self.cmbFilterAddressOrgStructure.value()
        self.filter['isFilterAddress'] = self.chkFilterAddress.isChecked()
        if self.chkFilterAddress.isChecked():
            self.filter['addressTypeId'] = self.cmbFilterAddressType.currentIndex()
            self.filter['addressCity'] = self.cmbFilterAddressCity.code()
            self.filter['addressOkato'] = self.cmbFilterAddressOkato.value()
            self.filter['KLADRStreetCodeList'] = self.cmbFilterAddressStreet.codeList()
            self.filter['addressStreet'] = self.cmbFilterAddressStreet.code()
            self.filter['addressHouse'] = self.edtFilterAddressHouse.text()
            self.filter['addressCorpus'] = self.edtFilterAddressCorpus.text()
            self.filter['addressFlat'] = self.edtFilterAddressFlat.text()
        self.filter['isSocStatuses'] = self.chkSocStatuses.isChecked()
        if self.chkSocStatuses.isChecked():
            self.filter['socStatusesBegDate'] = self.edtFilterSocStatusesBegDate.date()
            self.filter['socStatusesEndDate'] = self.edtFilterSocStatusesEndDate.date()
            self.filter['socStatusesClass'] = self.cmbSocStatusesClass.value()
            self.filter['socStatusesType'] = self.cmbSocStatusesType.value()
        self.filter['isFilterEvent'] = self.chkFilterEvent.isChecked()
        if self.chkFilterEvent.isChecked():
            self.filter['chkEventVisitDiagnosis'] = self.chkEventVisitDiagnosis.isChecked()
            self.filter['eventVisitType'] = self.cmbFilterEventVisitType.currentIndex()
            self.filter['eventBegDate'] = self.edtFilterEventVisitBegDate.date()
            self.filter['eventEndDate'] = self.edtFilterEventVisitEndDate.date()
        self.filter['accountingSystemId'] = self.cmbFilterAccountingSystem.value()
        self.filter['filterClientId'] = forceStringEx(self.edtFilterClientId.text())


    def resetFilter(self):
        personId = None
        if QtGui.qApp.isGetPersonStationary():
            personId = QtGui.qApp.userId if QtGui.qApp.userSpecialityId else None
        self.cmbFilterEventPerson.setValue(personId)
        self.edtFilterEventBegDate.setDate(QDate.currentDate())
        self.edtFilterEventEndDate.setDate(QDate())
        self.cmbFilterEventOrgStructure.setValue(None)
        self.cmbFilterEventSpecialityType.setCurrentIndex(0)
        self.cmbFilterEventSpeciality.clearValue()
        self.cmbFilterEventSpeciality.setValue(None)
        self.cmbFilterEventSpeciality.setText(u'')
        self.cmbMKBFilter.setCurrentIndex(0)
        self.edtMKBFrom.setText('A__. _')
        self.edtMKBTo.setText('Z99.9_')
        self.on_cmbMKBFilter_currentIndexChanged(0)
        self.cmbSex.setCurrentIndex(0)
        self.spbAgeFor.setValue(0)
        self.spbAgeTo.setValue(self.spbAgeTo.maximum())
        self.cmbFilterDeath.setCurrentIndex(0)
        self.edtFilterDeathBegDate.setDate(QDate())
        self.edtFilterDeathEndDate.setDate(QDate())
        self.cmbFilterAttachOrganisation.setValue(None)
        self.chkFilterNotAttachOrganisation.setChecked(False)
        self.cmbFilterAddressOrgStructureType.setCurrentIndex(0)
        self.cmbFilterAddressOrgStructure.setValue(None)
        self.cmbFilterAddressType.setCurrentIndex(0)
        self.cmbFilterAddressCity.setCode(u'')
        self.cmbFilterAddressOkato.setValue(None)
        self.cmbFilterAddressStreet.setCode(u'')
        self.edtFilterAddressHouse.setText(u'')
        self.edtFilterAddressCorpus.setText(u'')
        self.edtFilterAddressFlat.setText(u'')
        self.edtFilterSocStatusesBegDate.setDate(QDate())
        self.edtFilterSocStatusesEndDate.setDate(QDate())
        self.cmbSocStatusesClass.setValue(None)
        self.cmbSocStatusesType.setValue(None)
        self.chkEventVisitDiagnosis.setChecked(False)
        self.cmbFilterEventVisitType.setCurrentIndex(0)
        self.edtFilterEventVisitBegDate.setDate(QDate(QDate.currentDate().year(),1,1))
        self.edtFilterEventVisitEndDate.setDate(QDate(QDate.currentDate().year(),12,31))
        self.cmbFilterAccountingSystem.setValue(None)
        self.edtFilterClientId.setText('')
        self.chkFilterAddressOrgStructure.setChecked(False)
        self.chkFilterAddress.setChecked(False)
        self.chkSocStatuses.setChecked(False)
        self.chkFilterEvent.setChecked(False)
        self.setFilter()


    def fillingSpeciality(self):
        userId = QtGui.qApp.userId
        userSpecialityId = QtGui.qApp.userSpecialityId
        specialityId = None
        if userId and userSpecialityId:
            db = QtGui.qApp.db
            table = db.table('Person')
            tableRBPost = db.table('rbPost')
            cond = [table['id'].eq(userId),
                    table['deleted'].eq(0),
                    u'''LEFT(rbPost.code, 1) IN ('1','2','3')'''
                    ]
            queryTable = table.innerJoin(tableRBPost, tableRBPost['id'].eq(table['post_id']))
            record = db.getRecordEx(queryTable, ['speciality_id'], cond)
            if record:
                specialityId = forceRef(record.value('speciality_id'))
        self.cmbFilterEventSpecialityType.setCurrentIndex(0)
        self.cmbFilterEventSpeciality.clearValue()
        self.cmbFilterEventSpeciality.setValue((u'%s'%(forceString(specialityId))) if specialityId else None)
        if not specialityId:
            self.cmbFilterEventSpeciality.setText(u'')


    @pyqtSignature('int')
    def on_cmbMKBFilter_currentIndexChanged(self, index):
        mode = bool(index)
        for widget in (self.edtMKBFrom, self.edtMKBTo):
            widget.setEnabled(mode)


    @pyqtSignature('')
    def on_btnFindClientInfo_clicked(self):
        self.cmbFilterAccountingSystem.setValue(None)
        self.edtFilterClientId.setText('')
        clientIdList = self.getCurrentClietsTable().model().idList()
        dialog = CSurveillanceFindClientInfoDialog(self, clientIdList)
        if dialog:
            dialog.setWindowTitle(u'''Поиск пациента''')
            dialog.exec_()
            self.edtFilterClientId.setText(forceString(dialog.filterClientId))


    @pyqtSignature('int')
    def on_cmbFilterAddressCity_currentIndexChanged(self, index):
        code = self.cmbFilterAddressCity.code()
        self.cmbFilterAddressOkato.setEnabled(True)
        self.cmbFilterAddressOkato.setKladrCode(code)
        self.cmbFilterAddressStreet.setCity(code)


    @pyqtSignature('int')
    def on_cmbFilterAddressOkato_currentIndexChanged(self, index):
        okato = self.cmbFilterAddressOkato.value()
        self.cmbFilterAddressStreet.setOkato(okato)


    @pyqtSignature('')
    def on_mnuClients_aboutToShow(self):
        isEnabled = False
        widgetIndex = self.tabDispensaireClients.currentIndex()
        if widgetIndex == 0:
            self.tblConsistsClients.setFocus(Qt.OtherFocusReason)
            currentIndex = self.tblConsistsClients.currentIndex()
            isEnabled = currentIndex.row() >= 0
        elif widgetIndex == 1:
            self.tblTakenClients.setFocus(Qt.TabFocusReason)
            currentIndex = self.tblTakenClients.currentIndex()
            isEnabled = currentIndex.row() >= 0
        elif widgetIndex == 2:
            self.tblRemoveClients.setFocus(Qt.OtherFocusReason)
            currentIndex = self.tblRemoveClients.currentIndex()
            isEnabled = currentIndex.row() >= 0
        self.actEditClientInfoBeds.setEnabled(isEnabled)
        self.actAmbCardShow.setEnabled(isEnabled)


    @pyqtSignature('')
    def on_mnuMonitoring_aboutToShow(self):
        isEnabled = False
        widgetIndex = self.tabDispensaireClients.currentIndex()
        if widgetIndex == 0:
            self.tblConsistsMonitoring.setFocus(Qt.OtherFocusReason)
            currentIndex = self.tblConsistsMonitoring.currentIndex()
            isEnabled = currentIndex.row() >= 0
        elif widgetIndex == 1:
            self.tblTakenMonitoring.setFocus(Qt.TabFocusReason)
            currentIndex = self.tblTakenMonitoring.currentIndex()
            isEnabled = currentIndex.row() >= 0
        elif widgetIndex == 2:
            self.tblRemoveMonitoring.setFocus(Qt.OtherFocusReason)
            currentIndex = self.tblRemoveMonitoring.currentIndex()
            isEnabled = currentIndex.row() >= 0
        self.actOpenEvent.setEnabled(isEnabled)


    @pyqtSignature('')
    def on_actAmbCardShow_triggered(self):
        self.ambCardShow()


    def ambCardShow(self):
        isRightAdmin =  QtGui.qApp.isAdmin()
        isRegTabReadAmbCard = QtGui.qApp.userHasRight(urRegTabReadAmbCard) or isRightAdmin
        isRegTabWriteAmbCard = QtGui.qApp.userHasRight(urRegTabWriteAmbCard) or isRightAdmin
        if isRegTabReadAmbCard or isRegTabWriteAmbCard:
            currentTableIndex = self.tabDispensaireClients.currentIndex()
            currentTable = self.getCurrentClietsTable()
            currentRow = currentTable.currentRow()
            clientId = currentTable.currentItemId()
            if clientId:
                try:
                    dialog = CAmbCardDialog(self, clientId)
                    if dialog.exec_():
                        currentTable.setCurrentRow(currentRow)
                        self.on_tabDispensaireClients_currentChanged(self.tabDispensaireClients.currentIndex())
                finally:
                    dialog.destroy()
                    sip.delete(dialog)
                    del dialog
        else:
            QtGui.QMessageBox.warning( self,
                                     u'Внимание!',
                                     u'Нет права на чтение и редактирование Мед.карты!',
                                     QtGui.QMessageBox.Ok,
                                     QtGui.QMessageBox.Ok)


    @pyqtSignature('')
    def on_actEditClientInfoBeds_triggered(self):
        self.editClient()


    def editClient(self):
        isRightAdmin =  QtGui.qApp.isAdmin()
        isSurReadClientInfo = QtGui.qApp.userHasRight(urSurReadClientInfo) or isRightAdmin
        isSurEditClientInfo = QtGui.qApp.userHasRight(urSurEditClientInfo) or isRightAdmin
        if isSurReadClientInfo or isSurEditClientInfo:
            table = self.getCurrentClietsTable()
            if table:
                tableIndex = table.currentIndex()
                row = tableIndex.row()
                if row >= 0:
                    currentRow = table.currentRow()
                    clientId = table.currentItemId()
                    if clientId:
                        dialog = CSurveillanceClientEditDialog(self)
                        try:
                            if clientId:
                                dialog.load(clientId)
                            if dialog.exec_():
                                table.setCurrentRow(currentRow)
                                self.on_tabDispensaireClients_currentChanged(self.tabDispensaireClients.currentIndex())
                        finally:
                            dialog.destroy()
                            sip.delete(dialog)
                            del dialog
        else:
            res = QtGui.QMessageBox.warning( self,
                         u'Внимание!',
                         u'Нет права на чтение и редактирование карты пациента!',
                         QtGui.QMessageBox.Ok,
                         QtGui.QMessageBox.Ok)


    @pyqtSignature('')
    def on_actOpenEvent_triggered(self):
        QtGui.qApp.callWithWaitCursor(self, self.openEvent)


    def openEvent(self):
        isRightAdmin =  QtGui.qApp.isAdmin()
        isSurReadEvent = QtGui.qApp.userHasRight(urSurReadEvent) or isRightAdmin
        isSurEditEvent = QtGui.qApp.userHasRight(urSurEditEvent) or isRightAdmin
        if isSurReadEvent or isSurEditEvent:
            currentTable = self.getCurrentMonitoringTable()
            currentRow = currentTable.currentRow()
            record = currentTable.model().getRecordByRow(currentRow)
            eventId = forceRef(record.value('event_id')) if record else None
            if eventId:
                try:
                    formClass = getEventFormClass(eventId)
                    dialog = formClass(self)
                    dialog.load(eventId)
                    QtGui.qApp.restoreOverrideCursor()
                    dialog.setReadOnly(isSurReadEvent and not isSurEditEvent)
                    if dialog.exec_():
                        currentTable.setCurrentRow(currentRow)
                        self.getFillingMonitoring(self.tabDispensaireClients.currentIndex(), currentTable.currentItemId())
                finally:
                    dialog.destroy()
                    sip.delete(dialog)
                    del dialog
        else:
            QtGui.QMessageBox.warning( self,
                                     u'Внимание!',
                                     u'Нет права на чтение и редактирование события!',
                                     QtGui.QMessageBox.Ok,
                                     QtGui.QMessageBox.Ok)


    def getCurrentClietsTable(self):
        index = self.tabDispensaireClients.currentIndex()
        return [self.tblConsistsClients, self.tblTakenClients, self.tblRemoveClients][index]


    def getCurrentMonitoringTable(self):
        index = self.tabDispensaireClients.currentIndex()
        return [self.tblConsistsMonitoring, self.tblTakenMonitoring, self.tblRemoveMonitoring][index]


    def getCurrentInfoBrowserTable(self):
        index = self.tabDispensaireClients.currentIndex()
        return [self.txtConsistsClientInfoBrowser, self.txtTakenClientInfoBrowser, self.txtRemoveClientInfoBrowser][index]


    @pyqtSignature('int')
    def on_tabDispensaireClients_currentChanged(self, index):
        if index == 0:
            QtGui.qApp.callWithWaitCursor(self, self.getFillingClients, self.tblConsistsClients, self.txtConsistsClientInfoBrowser)
            self.periodChanged(u'На дату', False)
        elif index == 1:
            QtGui.qApp.callWithWaitCursor(self, self.getFillingClients, self.tblTakenClients, self.txtTakenClientInfoBrowser)
            self.periodChanged(u'Начало периода', True)
        elif index == 2:
            QtGui.qApp.callWithWaitCursor(self, self.getFillingClients, self.tblRemoveClients, self.txtRemoveClientInfoBrowser)
            self.periodChanged(u'Начало периода', True)


    def periodChanged(self, text, isVisible):
        self.lblBegDate.setText(text)
        self.lblEndDate.setVisible(isVisible)
        self.edtFilterEventEndDate.setVisible(isVisible)


    def getFillingClients(self, table, clientInfoBrowser):
        self.setFilter()
        currentRow = table.currentRow()
        model = table.model()
        model.loadData(self.filter)
        if model.rowCount() >= 0:
            if currentRow is None:
                currentRow = 0
            table.setCurrentRow(currentRow)
            clientId = table.currentItemId()
            self.getClientInfoBrowser(clientId, clientInfoBrowser)
        else:
            clientInfoBrowser.setText('')
        tableMonitoring = self.getCurrentMonitoringTable()
        tableMonitoring.model().loadData(table.currentItemId(), self.filter)
        QtGui.qApp.restoreOverrideCursor()
        self.lblCountRecordList.setText(formatRecordsCount(self.getCurrentClietsTable().model().rowCount()))


    def getFillingMonitoring(self, indexTab, clientId):
        table = self.getCurrentMonitoringTable()
        currentRow = table.currentRow()
        model = table.model()
        model.loadData(clientId, self.filter)
        if model.rowCount() >= 0:
            if currentRow is None:
                currentRow = 0
            table.setCurrentRow(currentRow)


    def getClientInfoBrowser(self, clientId, clientInfoBrowser):
        if clientId:
            clientInfoBrowser.setHtml(getClientBanner(clientId))
        else:
            clientInfoBrowser.setText('')


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelConsistsClients_currentRowChanged(self, current, previous):
        clientId = self.tblConsistsClients.currentItemId()
        self.getClientInfoBrowser(clientId, self.txtConsistsClientInfoBrowser)
        self.modelConsistsMonitoring.loadData(clientId, self.filter)


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelTakenClients_currentRowChanged(self, current, previous):
        clientId = self.tblTakenClients.currentItemId()
        self.getClientInfoBrowser(clientId, self.txtTakenClientInfoBrowser)
        self.modelTakenMonitoring.loadData(clientId, self.filter)


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelRemoveClients_currentRowChanged(self, current, previous):
        clientId = self.tblRemoveClients.currentItemId()
        self.getClientInfoBrowser(clientId, self.txtRemoveClientInfoBrowser)
        self.modelRemoveMonitoring.loadData(clientId, self.filter)


class CSurveillanceClientsModel(CTableModel):
    class CLocClientNameColumn(CTextCol):
        def __init__(self, title, fields, defaultWidth, alignment='l'):
            CTextCol.__init__(self, title, fields, defaultWidth, alignment)

        def format(self, values):
            val = formatName(unicode(values[0].toString()), unicode(values[1].toString()), unicode(values[2].toString()))
            return QVariant(val)


    def __init__(self, parent):
        CTableModel.__init__(self, parent)
        self.addColumn(CTextCol(u'Код пациента',            ['id'],                                                            15))
        self.addColumn(CSurveillanceClientsModel.CLocClientNameColumn(u'ФИО пациента ', ['lastName', 'firstName', 'patrName'], 30, 'l'),)
        self.addColumn(CEnumCol(u'Пол',                     ['sex'], [u'-', u'М', u'Ж'],                                       4, 'c'))
        self.addColumn(CDateCol(u'Дата рожд.',              ['birthDate'],                                                     12, highlightRedDate=False))
        self.addColumn(CSNILSCol(u'СНИЛС',                  ['SNILS'],                                                         4))
        self.setTable('Client')


    def getClientIdList(self, records, filter, observedType):
        clientIdList = []
        specialityType = filter.get('specialityType', None)
        specialityList = filter.get('specialityId', None)
        clientSpecIdList = {}
        if specialityList and specialityType:
            specialityListId = []
            specialityLine = specialityList.split(u',')
            for speciality in specialityLine:
                specialityId = int(trim(speciality))
                if specialityId and specialityId not in specialityListId:
                    specialityListId.append(specialityId)
            for record in records:
                clientId = forceRef(record.value('id'))
                specialityId  = forceRef(record.value('speciality_id'))
                if specialityId and specialityId in specialityListId:
                    observed = forceInt(record.value('observed'))
                    if observed == observedType:
                        clientSpecIdLine = clientSpecIdList.get(clientId, [])
                        clientSpecIdLine.append(specialityId)
                        clientSpecIdList[clientId] = clientSpecIdLine
            for clientId, clientSpecIdLine in clientSpecIdList.items():
                if not set(clientSpecIdLine) ^ set(specialityListId):
                    if clientId and clientId not in clientIdList:
                        clientIdList.append(clientId)
        else:
            for record in records:
                observed = forceInt(record.value('observed'))
                if observed == observedType:
                    clientIdList.append(forceRef(record.value('id')))
        return clientIdList


    def getColsGroupBySpeciality(self, cols, groupBy):
        db = QtGui.qApp.db
        tablePerson = db.table('Person')
        cols.append(tablePerson['speciality_id'])
        groupBy += u', Person.speciality_id '
        return cols, groupBy


class CConsistsClientsModel(CSurveillanceClientsModel):
    def __init__(self, parent):
        CSurveillanceClientsModel.__init__(self, parent)


    def loadData(self, filter):
        self.filter = filter
        clientIdList = []
        date = self.filter.get('begDate', QDate.currentDate())
        specialityType = self.filter.get('specialityType', None)
        specialityList = self.filter.get('specialityId', None)
        if not date:
            date = QDate.currentDate()
        db = QtGui.qApp.db
        tableClient = db.table('Client')
        tableDiagnosis = db.table('Diagnosis')
        tableDiagnostic = db.table('Diagnostic')
        tableRBDispanser = db.table('rbDispanser')
        cond = [tableClient['deleted'].eq(0),
                tableDiagnosis['deleted'].eq(0),
                tableDiagnostic['deleted'].eq(0),
                tableDiagnostic['dispanser_id'].isNotNull(),
                tableDiagnostic['endDate'].le(date),
                ]
        queryTable = tableClient.innerJoin(tableDiagnosis, tableDiagnosis['client_id'].eq(tableClient['id']))
        queryTable = queryTable.innerJoin(tableDiagnostic, tableDiagnostic['diagnosis_id'].eq(tableDiagnosis['id']))
        queryTable = queryTable.innerJoin(tableRBDispanser, tableRBDispanser['id'].eq(tableDiagnostic['dispanser_id']))
        cond, queryTable = diagnosticCondAdd(db, queryTable, filter, cond, tableDiagnosis, tableDiagnostic)
        cond, queryTable = clientsCondAdd(db, queryTable, filter, cond, tableClient)
        cond.append(u'''NOT EXISTS(SELECT DC.id
                               FROM Diagnostic AS DC
                               INNER JOIN rbDispanser AS rbDP ON rbDP.id = DC.dispanser_id
                               WHERE DC.diagnosis_id = Diagnosis.id AND DC.endDate <= %s AND DC.deleted = 0 AND rbDP.name LIKE '%s')'''%(db.formatDate(date), u'%снят%'))
        cols = [u'DISTINCT MAX(Diagnostic.endDate)', u'Client.id', u'rbDispanser.observed', u'Client.lastName', u'Client.firstName', u'Client.patrName']
        group='Client.id'
        if specialityList and specialityType:
            cols, group = self.getColsGroupBySpeciality(cols, group)
        records = db.getRecordListGroupBy(queryTable,
                                          cols,
                                          where=cond, group=group,
                                          order='Client.lastName ASC, Client.firstName ASC, Client.patrName ASC, Diagnostic.endDate DESC')
        clientIdList = self.getClientIdList(records, self.filter, observedType = 1)
        self.setIdList(clientIdList)


class CTakenClientsModel(CSurveillanceClientsModel):
    def __init__(self, parent):
        CSurveillanceClientsModel.__init__(self, parent)


    def loadData(self, filter):
        self.filter = filter
        clientIdList = []
        begDate = self.filter.get('begDate', QDate.currentDate())
        endDate = self.filter.get('endDate', QDate())
        specialityType = self.filter.get('specialityType', None)
        specialityList = self.filter.get('specialityId', None)
        if not begDate:
            begDate = QDate.currentDate()
        db = QtGui.qApp.db
        tableClient = db.table('Client')
        tableDiagnosis = db.table('Diagnosis')
        tableDiagnostic = db.table('Diagnostic')
        tableRBDispanser = db.table('rbDispanser')
        cond = [tableClient['deleted'].eq(0),
                tableDiagnosis['deleted'].eq(0),
                tableDiagnostic['deleted'].eq(0),
                tableDiagnostic['dispanser_id'].isNotNull(),
                tableDiagnostic['endDate'].ge(begDate),
                tableRBDispanser['observed'].eq(1),
                tableRBDispanser['name'].like(u'%взят%')
                ]
        if endDate:
            cond.append(tableDiagnostic['endDate'].le(endDate))
        queryTable = tableClient.innerJoin(tableDiagnosis, tableDiagnosis['client_id'].eq(tableClient['id']))
        queryTable = queryTable.innerJoin(tableDiagnostic, tableDiagnostic['diagnosis_id'].eq(tableDiagnosis['id']))
        queryTable = queryTable.innerJoin(tableRBDispanser, tableRBDispanser['id'].eq(tableDiagnostic['dispanser_id']))
        cond, queryTable = diagnosticCondAdd(db, queryTable, filter, cond, tableDiagnosis, tableDiagnostic)
        cond, queryTable = clientsCondAdd(db, queryTable, filter, cond, tableClient)
        cols = [u'DISTINCT MAX(Diagnostic.endDate)', u'Client.id', u'rbDispanser.observed', u'Client.lastName', u'Client.firstName', u'Client.patrName']
        group='Client.id'
        if specialityList and specialityType:
            cols, group = self.getColsGroupBySpeciality(cols, group)
        records = db.getRecordListGroupBy(queryTable,
                                          cols,
                                          where=cond, group=group,
                                          order='Client.lastName ASC, Client.firstName ASC, Client.patrName ASC, Diagnostic.endDate DESC')
        clientIdList = self.getClientIdList(records, self.filter, observedType = 1)
        self.setIdList(clientIdList)


class CRemoveClientsModel(CSurveillanceClientsModel):
    def __init__(self, parent):
        CSurveillanceClientsModel.__init__(self, parent)


    def loadData(self, filter):
        self.filter = filter
        clientIdList = []
        begDate = self.filter.get('begDate', QDate.currentDate())
        endDate = self.filter.get('endDate', QDate())
        specialityType = self.filter.get('specialityType', None)
        specialityList = self.filter.get('specialityId', None)
        if not begDate:
            begDate = QDate.currentDate()
        db = QtGui.qApp.db
        tableClient = db.table('Client')
        tableDiagnosis = db.table('Diagnosis')
        tableDiagnostic = db.table('Diagnostic')
        tableRBDispanser = db.table('rbDispanser')
        cond = [tableClient['deleted'].eq(0),
                tableDiagnosis['deleted'].eq(0),
                tableDiagnostic['deleted'].eq(0),
                tableDiagnostic['dispanser_id'].isNotNull(),
                tableDiagnostic['endDate'].ge(begDate),
                tableRBDispanser['observed'].eq(0),
                tableRBDispanser['name'].like(u'%снят%'),
                ]
        if endDate:
            cond.append(tableDiagnostic['endDate'].le(endDate))
        queryTable = tableClient.innerJoin(tableDiagnosis, tableDiagnosis['client_id'].eq(tableClient['id']))
        queryTable = queryTable.innerJoin(tableDiagnostic, tableDiagnostic['diagnosis_id'].eq(tableDiagnosis['id']))
        queryTable = queryTable.innerJoin(tableRBDispanser, tableRBDispanser['id'].eq(tableDiagnostic['dispanser_id']))
        cond, queryTable = diagnosticCondAdd(db, queryTable, filter, cond, tableDiagnosis, tableDiagnostic)
        cond, queryTable = clientsCondAdd(db, queryTable, filter, cond, tableClient)
        cols = [u'DISTINCT MAX(Diagnostic.endDate)', u'Client.id', u'rbDispanser.observed', u'Client.lastName', u'Client.firstName', u'Client.patrName']
        group='Client.id'
        if specialityList and specialityType:
            cols, group = self.getColsGroupBySpeciality(cols, group)
        records = db.getRecordListGroupBy(queryTable,
                                          cols,
                                          where=cond, group=group,
                                          order='Client.lastName ASC, Client.firstName ASC, Client.patrName ASC, Diagnostic.endDate DESC')
        clientIdList = self.getClientIdList(records, self.filter, observedType = 0)
        self.setIdList(clientIdList)


class CSurveillanceMonitoringModel(CTableModel):
    def __init__(self, parent):
        CTableModel.__init__(self, parent)
        self.addColumn(CDesignationCol(u'МКБ',                             ['diagnosis_id'], ('Diagnosis', 'MKB'),      6))
        self.addColumn(CDateCol(       u'Дата записи',                     ['endDate'],                                 12))
        self.addColumn(CRefBookCol(    u'Статус ДН',                       ['dispanser_id'], 'rbDispanser',             15))
        self.addColumn(CRefBookCol(    u'Врач',                            ['person_id'],    'vrbPersonWithSpeciality', 6))
        self.addColumn(CTextCol(       u'Описание диагноза',               ['freeInput'],                               6))
        self.loadField('event_id')
        self.loadField('diagnosis_id')
        self.setTable('Diagnostic')


class CConsistsMonitoringModel(CSurveillanceMonitoringModel):
    def __init__(self, parent):
        CSurveillanceMonitoringModel.__init__(self, parent)


    def loadData(self, masterId, filter):
        diagnosticIdList = []
        if masterId:
            self.filter = filter
            date = self.filter.get('begDate', QDate.currentDate())
            if not date:
                date = QDate.currentDate()
            db = QtGui.qApp.db
            tableDiagnosis = db.table('Diagnosis')
            tableDiagnostic = db.table('Diagnostic')
            tableRBDispanser = db.table('rbDispanser')
            cond = [tableDiagnosis['client_id'].eq(masterId),
                    tableDiagnosis['deleted'].eq(0),
                    tableDiagnostic['deleted'].eq(0),
                    tableDiagnostic['dispanser_id'].isNotNull(),
                    tableDiagnostic['endDate'].le(date),
                    ]
            queryTable = tableDiagnosis.innerJoin(tableDiagnostic, tableDiagnostic['diagnosis_id'].eq(tableDiagnosis['id']))
            queryTable = queryTable.innerJoin(tableRBDispanser, tableRBDispanser['id'].eq(tableDiagnostic['dispanser_id']))
            cond, queryTable = diagnosticCondAdd(db, queryTable, filter, cond, tableDiagnosis, tableDiagnostic)
            cond.append(u'''NOT EXISTS(SELECT DC.id
                                   FROM Diagnostic AS DC
                                   INNER JOIN rbDispanser AS rbDP ON rbDP.id = DC.dispanser_id
                                   WHERE DC.diagnosis_id = Diagnosis.id AND DC.endDate <= %s AND DC.deleted = 0 AND rbDP.name LIKE '%s')'''%(db.formatDate(date), u'%снят%'))
            diagnosticIdList = db.getDistinctIdList(queryTable, [u'Diagnostic.id'], where=cond, order='Diagnostic.endDate DESC')
        self.setIdList(diagnosticIdList)


class CTakenMonitoringModel(CSurveillanceMonitoringModel):
    def __init__(self, parent):
        CSurveillanceMonitoringModel.__init__(self, parent)


    def loadData(self, masterId, filter):
        diagnosticIdList = []
        if masterId:
            self.filter = filter
            begDate = self.filter.get('begDate', QDate.currentDate())
            endDate = self.filter.get('endDate', QDate())
            if not begDate:
                begDate = QDate.currentDate()
            db = QtGui.qApp.db
            tableDiagnosis = db.table('Diagnosis')
            tableDiagnostic = db.table('Diagnostic')
            tableRBDispanser = db.table('rbDispanser')
            cond = [tableDiagnosis['client_id'].eq(masterId),
                    tableDiagnosis['deleted'].eq(0),
                    tableDiagnostic['deleted'].eq(0),
                    tableDiagnostic['dispanser_id'].isNotNull(),
                    tableDiagnostic['endDate'].ge(begDate),
                    tableRBDispanser['observed'].eq(1),
                    tableRBDispanser['name'].like(u'%взят%'),
                    ]
            if endDate:
                cond.append(tableDiagnostic['endDate'].le(endDate))
            queryTable = tableDiagnosis.innerJoin(tableDiagnostic, tableDiagnostic['diagnosis_id'].eq(tableDiagnosis['id']))
            queryTable = queryTable.innerJoin(tableRBDispanser, tableRBDispanser['id'].eq(tableDiagnostic['dispanser_id']))
            cond, queryTable = diagnosticCondAdd(db, queryTable, filter, cond, tableDiagnosis, tableDiagnostic)
            diagnosticIdList = db.getDistinctIdList(queryTable, [u'Diagnostic.id'], where=cond, order='Diagnostic.endDate DESC')
        self.setIdList(diagnosticIdList)


class CRemoveMonitoringModel(CSurveillanceMonitoringModel):
    def __init__(self, parent):
        CSurveillanceMonitoringModel.__init__(self, parent)


    def loadData(self, masterId, filter):
        diagnosticIdList = []
        if masterId:
            self.filter = filter
            begDate = self.filter.get('begDate', QDate.currentDate())
            endDate = self.filter.get('endDate', QDate())
            if not begDate:
                begDate = QDate.currentDate()
            db = QtGui.qApp.db
            tableDiagnosis = db.table('Diagnosis')
            tableDiagnostic = db.table('Diagnostic')
            tableRBDispanser = db.table('rbDispanser')
            cond = [tableDiagnosis['client_id'].eq(masterId),
                    tableDiagnosis['deleted'].eq(0),
                    tableDiagnostic['deleted'].eq(0),
                    tableDiagnostic['dispanser_id'].isNotNull(),
                    tableDiagnostic['endDate'].ge(begDate),
                    tableRBDispanser['observed'].eq(0),
                    tableRBDispanser['name'].like(u'%снят%'),
                    ]
            if endDate:
                cond.append(tableDiagnostic['endDate'].le(endDate))
            queryTable = tableDiagnosis.innerJoin(tableDiagnostic, tableDiagnostic['diagnosis_id'].eq(tableDiagnosis['id']))
            queryTable = queryTable.innerJoin(tableRBDispanser, tableRBDispanser['id'].eq(tableDiagnostic['dispanser_id']))
            cond, queryTable = diagnosticCondAdd(db, queryTable, filter, cond, tableDiagnosis, tableDiagnostic)
            diagnosticIdList = db.getDistinctIdList(queryTable, [u'Diagnostic.id'], where=cond, order='Diagnostic.endDate DESC')
        self.setIdList(diagnosticIdList)


class CSurveillanceClientEditDialog(CClientEditDialog):
    def __init__(self, parent):
        CClientEditDialog.__init__(self, parent)


    def saveData(self):
        isRightAdmin =  QtGui.qApp.isAdmin()
        isSurReadClientInfo = QtGui.qApp.userHasRight(urSurReadClientInfo) or isRightAdmin
        isSurEditClientInfo = QtGui.qApp.userHasRight(urSurEditClientInfo) or isRightAdmin
        if isSurReadClientInfo and not isSurEditClientInfo:
            QtGui.QMessageBox.warning(self,
                            u'Внимание',
                            u'Право только на чтение!',
                            QtGui.QMessageBox.Ok,
                            QtGui.QMessageBox.Ok)
            return False
        if isSurEditClientInfo:
            return self.checkDataEntered() and self.save()
        return False


def diagnosticCondAdd(db, queryTable, filter, cond, tableDiagnosis, tableDiagnostic):
    MKBFilter = filter.get('MKBFilter', 0)
    if MKBFilter:
        MKBFrom = filter.get('MKBFrom', None)
        MKBTo   = filter.get('MKBTo', None)
        cond.append(isSurveillanceMKB(MKBFrom, MKBTo))
    personId = filter.get('personId', None)
    if personId:
        cond.append(tableDiagnostic['person_id'].eq(personId))
    orgStructureId = filter.get('orgStructureId', None)
    speciality = filter.get('specialityId', None)
    if orgStructureId or speciality:
        tablePerson = db.table('Person')
        queryTable = queryTable.innerJoin(tablePerson, tablePerson['id'].eq(tableDiagnostic['person_id']))
        cond.append(tablePerson['deleted'].eq(0))
    if orgStructureId:
        orgStructureIdList = db.getDescendants('OrgStructure', 'parent_id', orgStructureId)
        if orgStructureIdList:
            cond.append(tablePerson['orgStructure_id'].inlist(orgStructureIdList))
    if speciality:
        specialityType = filter.get('specialityType', None)
        cond.append(u'''Person.speciality_id IN (%s)'''%(speciality))
    return cond, queryTable


def clientsCondAdd(db, queryTable, filter, cond, tableClient):
    def addAddressCond(cond3, addrType, addrIdList):
        tableClientAddress = db.table('ClientAddress')
        subcond = [tableClientAddress['client_id'].eqEx('`Client`.`id`'),
                   tableClientAddress['id'].eqEx(u'getClientLocAddressId(Client.id)' if addrType else u'getClientRegAddressId(Client.id)')
                  ]
        if addrIdList is None:
            subcond.append(tableClientAddress['address_id'].isNull())
            subcondNoCAId = u'(SELECT %s IS NULL)'%(u'getClientLocAddressId(Client.id)' if addrType else u'getClientRegAddressId(Client.id)')
        else:
            subcond.append(tableClientAddress['address_id'].inlist(addrIdList))
        if addrIdList is None:
            cond3.append(db.joinOr([db.existsStmt(tableClientAddress, subcond), subcondNoCAId]))
        else:
            cond3.append(db.existsStmt(tableClientAddress, subcond))
    sex = filter.get('sex', 0)
    if sex > 0:
        cond.append(tableClient['sex'].eq(sex))
    ageFor = filter.get('ageFor', 0)
    ageTo = filter.get('ageTo', 0)
    if ageFor <= ageTo:
        currentDate = db.formatDate(QDate.currentDate())
        cond.append('''(%s >= ADDDATE(Client.birthDate, INTERVAL %d YEAR)) AND (%s < ADDDATE(Client.birthDate, INTERVAL %d YEAR))''' % (currentDate, ageFor, currentDate, ageTo+1))
    death = filter.get('death', 0)
    if death == 1:
        cond.append(tableClient['deathDate'].isNull())
    elif death == 2:
        cond.append(tableClient['deathDate'].isNotNull())
        deathBegDate = filter.get('deathBegDate', None)
        deathEndDate = filter.get('deathEndDate', None)
        if deathBegDate:
            cond.append(tableClient['deathDate'].dateGe(deathBegDate))
        if deathEndDate:
            cond.append(tableClient['deathDate'].dateLe(deathEndDate))
    accountingSystemId = filter.get('accountingSystemId', None)
    filterClientId = filter.get('filterClientId', None)
    if accountingSystemId and filterClientId:
        tableIdentification = db.table('ClientIdentification')
        queryTable = queryTable.innerJoin(tableIdentification, tableIdentification['client_id'].eq(tableClient['id']))
        cond.append(tableIdentification['accountingSystem_id'].eq(accountingSystemId))
        cond.append(tableIdentification['identifier'].eq(filterClientId))
        cond.append(tableIdentification['deleted'].eq(0))
    elif filterClientId:
        cond.append(tableClient['id'].eq(filterClientId))
    if filter.get('isFilterEvent', False):
        eventVisitType = filter.get('eventVisitType', None)
        if eventVisitType is not None:
            eventBegDate = filter.get('eventBegDate', QDate())
            eventEndDate = filter.get('eventEndDate', QDate())
            tableEvent = db.table('Event').alias(u'E')
            condDate = []
            if eventBegDate:
                condDate.append(db.joinOr([tableEvent['execDate'].isNull(), tableEvent['execDate'].ge(eventBegDate)]))
            if eventEndDate:
                condDate.append(tableEvent['setDate'].le(eventEndDate))
            chkEventVisitDiagnosis = filter.get('chkEventVisitDiagnosis', 0)
            if chkEventVisitDiagnosis:
                if eventVisitType == 0:
                    cond.append(u'''EXISTS(SELECT E.id
                                           FROM Event AS E
                                           INNER JOIN Diagnostic AS DC ON DC.event_id = E.id
                                           INNER JOIN Diagnosis AS DS ON DS.id = DC.diagnosis_id
                                           WHERE E.client_id = Client.id
                                           AND DC.deleted = 0
                                           AND DS.deleted = 0
                                           AND DS.MKB = Diagnosis.MKB
                                           AND E.deleted = 0 %s)'''%(('AND %s'%db.joinAnd(condDate)) if condDate else u''))
                if eventVisitType == 1:
                    cond.append(u'''NOT EXISTS(SELECT E.id
                                           FROM Event AS E
                                           INNER JOIN Diagnostic AS DC ON DC.event_id = E.id
                                           INNER JOIN Diagnosis AS DS ON DS.id = DC.diagnosis_id
                                           WHERE E.client_id = Client.id
                                           AND DC.deleted = 0
                                           AND DS.deleted = 0
                                           AND DS.MKB = Diagnosis.MKB
                                           AND E.deleted = 0 %s)'''%(('AND %s'%db.joinAnd(condDate)) if condDate else u''))
            else:
                if eventVisitType == 0:
                    cond.append(u'''EXISTS(SELECT E.id FROM Event AS E WHERE E.client_id = Client.id AND E.deleted = 0 %s)'''%(('AND %s'%db.joinAnd(condDate)) if condDate else u''))
                if eventVisitType == 1:
                    cond.append(u'''NOT EXISTS(SELECT E.id FROM Event AS E WHERE E.client_id = Client.id AND E.deleted = 0 %s)'''%(('AND %s'%db.joinAnd(condDate)) if condDate else u''))
    attachOrgId = filter.get('attachOrganisationId', None)
    if attachOrgId:
        isNotAttachOrganisation = filter.get('isNotAttachOrganisation', False)
        stmt = '''%s EXISTS (SELECT ClientAttach.id
           FROM ClientAttach
           LEFT JOIN rbAttachType ON rbAttachType.id = ClientAttach.attachType_id
           WHERE ClientAttach.deleted=0
           AND ClientAttach.client_id = Client.id AND LPU_id=%s
           AND ClientAttach.id = (SELECT MAX(CA2.id)
                       FROM ClientAttach AS CA2
                       LEFT JOIN rbAttachType AS rbAttachType2 ON rbAttachType2.id = CA2.attachType_id
                       WHERE CA2.deleted=0 AND CA2.client_id = Client.id))'''
        cond.append(stmt % ((u'NOT' if isNotAttachOrganisation else u''), attachOrgId))
    if filter.get('isFilterAddressOrgStructure', False):
        addrType = filter.get('addressOrgStructureTypeId', 0)
        areaOrgStructureId = filter.get('addressOrgStructureId', None)
        if areaOrgStructureId:
            addrIdList = None
            cond2 = []
            if (addrType+1) & 1:
                addrIdList = getOrgStructureAddressIdList(areaOrgStructureId)
                addAddressCond(cond2, 0, addrIdList)
            if (addrType+1) & 2:
                if addrIdList is None:
                    addrIdList = getOrgStructureAddressIdList(areaOrgStructureId)
                addAddressCond(cond2, 1, addrIdList)
            if ((addrType+1) & 4):
                if areaOrgStructureId:
                    orgStructureIdList = getOrgStructureDescendants(areaOrgStructureId)
                    outerCond = ['ClientAttach.client_id = Client.id']
                    innerCond = ['CA2.client_id = Client.id']
                    outerCond.append('orgStructure_id IN (%s)'%(u','.join(forceString(areaOrgStructureId) for areaOrgStructureId in orgStructureIdList)))
                    innerCond.append('rbAttachType2.temporary=0')
                    stmt = '''EXISTS (SELECT ClientAttach.id
                       FROM ClientAttach
                       LEFT JOIN rbAttachType ON rbAttachType.id = ClientAttach.attachType_id
                       WHERE ClientAttach.deleted=0
                       AND %s
                       AND ClientAttach.id = (SELECT MAX(CA2.id)
                                   FROM ClientAttach AS CA2
                                   LEFT JOIN rbAttachType AS rbAttachType2 ON rbAttachType2.id = CA2.attachType_id
                                   WHERE CA2.deleted=0 AND %s))'''
                    cond2.append(stmt % (db.joinAnd(outerCond), db.joinAnd(innerCond)))
                else:
                    outerCond = ['ClientAttach.client_id = Client.id']
                    innerCond = ['CA2.client_id = Client.id']
                    outerCond.append('LPU_id=%d'%QtGui.qApp.currentOrgId())
                    innerCond.append('rbAttachType2.temporary=0')
                    stmt = '''EXISTS (SELECT ClientAttach.id
                       FROM ClientAttach
                       LEFT JOIN rbAttachType ON rbAttachType.id = ClientAttach.attachType_id
                       WHERE ClientAttach.deleted=0
                       AND %s
                       AND ClientAttach.id = (SELECT MAX(CA2.id)
                                   FROM ClientAttach AS CA2
                                   LEFT JOIN rbAttachType AS rbAttachType2 ON rbAttachType2.id = CA2.attachType_id
                                   WHERE CA2.deleted=0 AND %s))'''
                    cond2.append(stmt % (db.joinAnd(outerCond), db.joinAnd(innerCond)))
            if cond2:
                cond.append(db.joinOr(cond2))
    if filter.get('isFilterAddress', False):
        addrType = filter.get('addressTypeId', 0)
        KLADRCode = filter.get('addressCity', None)
        Okato = filter.get('addressOkato', None)
        KLADRStreetCode = filter.get('addressStreet', None)
        KLADRStreetCodeList = filter.get('KLADRStreetCodeList', [])
        house = filter.get('addressHouse', u'')
        corpus = filter.get('addressCorpus', u'')
        flat = filter.get('addressFlat', u'')
        if KLADRCode or Okato or KLADRStreetCode:
            tableClientAddress = db.table('ClientAddress')
            tableAddressHouse = db.table('AddressHouse')
            tableAddress = db.table('Address')
            queryTable = queryTable.leftJoin(tableClientAddress, tableClientAddress['client_id'].eq(tableClient['id']))
            queryTable = queryTable.leftJoin(tableAddress, tableAddress['id'].eq(tableClientAddress['address_id']))
            queryTable = queryTable.leftJoin(tableAddressHouse, tableAddressHouse['id'].eq(tableAddress['house_id']))
            cond.append('''ClientAddress.id IN (SELECT MAX(CA.id) FROM ClientAddress AS CA WHERE CA.type=%d
            AND CA.deleted=0 AND CA.client_id=Client.id)''' % addrType)
            if KLADRStreetCode:
                cond.append(tableAddressHouse['KLADRStreetCode'].eq(KLADRStreetCode))
                if Okato and house:
                    okatoCond = (u'''kladr.DOMA.OCATD LIKE \'%s%%\'''' % Okato) if Okato else '''1'''
                    cond.append(u'''(SELECT DISTINCT kladr.STREET.CODE
                    FROM kladr.STREET INNER JOIN kladr.DOMA ON LEFT(kladr.DOMA.CODE,15) = LEFT(kladr.STREET.CODE,15)
                    WHERE %sRIGHT(kladr.STREET.CODE,2)=\'00\'
                    AND kladr.STREET.CODE = %s
                    AND %s
                    AND kladr.STREET.CODE = AddressHouse.KLADRStreetCode
                    AND (IF(TRIM(AddressHouse.number) != '', FIND_IN_SET(AddressHouse.number,
                    kladr.DOMA.flatHouseList), 1))
                    ORDER BY kladr.STREET.NAME, kladr.STREET.SOCR, kladr.STREET.CODE)'''%((u'kladr.DOMA.CODE LIKE \'%s%%\' AND '%(KLADRCode[0:-2])) if KLADRCode else u'', KLADRStreetCode, okatoCond))
                else:
                    cond.append(tableAddressHouse['KLADRStreetCode'].eq(KLADRStreetCode))
            elif Okato:
                cond.append(tableAddressHouse['KLADRStreetCode'].inlist(KLADRStreetCodeList))
            elif KLADRCode:
                mask = getLikeMaskForRegion(KLADRCode)
                if mask == KLADRCode:
                    cond.append(tableAddressHouse['KLADRCode'].eq(KLADRCode))
                else:
                    cond.append(tableAddressHouse['KLADRCode'].like(mask))
            if house:
                cond.append(tableAddressHouse['number'].eq(house))
            if corpus:
                cond.append('''TRIM(AddressHouse.corpus) = TRIM('%s')'''%(corpus))
            if flat:
                cond.append(tableAddress['flat'].eq(flat))
    if filter.get('isSocStatuses', False):
        socStatusesClass = filter.get('socStatusesClass', None)
        socStatusesType = filter.get('socStatusesType', None)
        socStatusesBegDate = filter.get('socStatusesBegDate', QDate())
        socStatusesEndDate = filter.get('socStatusesEndDate', QDate())
        if socStatusesClass or socStatusesType:
            tableClientSocStatus = db.table('ClientSocStatus')
            cond.append(tableClientSocStatus['deleted'].eq(0))
            if socStatusesClass:
                socStatusClassIdList = db.getDescendants('rbSocStatusClass', 'group_id', socStatusesClass)
                if socStatusClassIdList:
                    stmtStatusTypeId = u'''SELECT DISTINCT rbSocStatusClassTypeAssoc.type_id
            FROM  rbSocStatusClassTypeAssoc
            WHERE rbSocStatusClassTypeAssoc.class_id IN (%s)'''%(u','.join(str(socStatusClassId) for socStatusClassId in socStatusClassIdList))
                    queryStatusTypeId = db.query(stmtStatusTypeId)
                    resultStatusTypeIdList = []
                    while queryStatusTypeId.next():
                        resultStatusTypeIdList.append(queryStatusTypeId.value(0).toInt()[0])
                    if resultStatusTypeIdList:
                        cond.append(tableClientSocStatus['socStatusType_id'].inlist(resultStatusTypeIdList))
                    else:
                        parentSocStatusClassIdList = db.getTheseAndParents('rbSocStatusClass', 'group_id', [socStatusesClass])
                        cond.append(tableClientSocStatus['socStatusClass_id'].inlist(parentSocStatusClassIdList))
                        cond.append(tableClientSocStatus['socStatusType_id'].isNull())
                else:
                    cond.append(tableClientSocStatus['socStatusClass_id'].eq(socStatusesClass))
            if socStatusesType:
                cond.append(tableClientSocStatus['socStatusType_id'].eq(socStatusesType))
            if socStatusesBegDate:
                cond.append(db.joinOr([ tableClientSocStatus['endDate'].dateGe(socStatusesBegDate),
                                        tableClientSocStatus['endDate'].isNull()
                                      ]
                                     )
                           )
            if socStatusesEndDate:
                cond.append(db.joinOr([ tableClientSocStatus['begDate'].dateLe(socStatusesEndDate),
                                        tableClientSocStatus['begDate'].isNull()
                                      ]
                                     )
                           )
            queryTable = queryTable.leftJoin(tableClientSocStatus, tableClient['id'].eq(tableClientSocStatus['client_id']))
    return cond, queryTable


def getAgeSurveillanceRangeCond(ageFor, ageTo):
    return '''(DATE(Diagnostic.endDate) >= ADDDATE(Client.birthDate, INTERVAL %d YEAR))
           AND (DATE(Diagnostic.endDate) < ADDDATE(Client.birthDate, INTERVAL %d YEAR))''' % (ageFor, ageTo+1)


def isSurveillanceMKB(MKBFrom, MKBTo):
    return '''Diagnosis.MKB >= '%s' AND Diagnosis.MKB <= '%s' '''%(MKBFrom, MKBTo)


def getSurveillanceContext():
    return ['surveillance']


class CSurveillanceFindClientInfoDialog(CFindClientInfoDialog):
    def __init__(self, parent, clientIdList = []):
        CFindClientInfoDialog.__init__(self, parent, clientIdList)


    def setupGetClientIdMenu(self):
        self.addObject('mnuGetClientId', QtGui.QMenu(self))
        self.addObject('actGetClientId', QtGui.QAction(u'''Добавить в фильтр "Диспансерного наблюдения"''', self))
        self.mnuGetClientId.addAction(self.actGetClientId)

