# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2016-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import re
from collections import namedtuple

from PyQt4 import QtGui
from PyQt4.QtCore import QDateTime

#from library.Identification import CIdentificationException
from library.Utils          import forceDateTime, forceRef, forceString

from Events.Action                     import CAction
from Events.ActionStatus               import CActionStatus
#from Exchange.FHIRClient.server        import FHIRErrorException
from Registry.Utils                    import getClientInfo

from TissueJournal.LabInterfaceFHIR    import sendLocalResultsOverFHIR as sendLocalResultsOverFHIR050
from TissueJournal.LabInterfaceFHIR102 import sendLocalResultsOverFHIR as sendLocalResultsOverFHIR102
from TissueJournal.Utils               import getEquipmentInterface


def exportLocalLabResultsToUsish(days=7):
    db = QtGui.qApp.db
    # подготовительный этап
    destEquipmentIdList = getDestEquipmentIdList()
    srcEquipmentIdList  = getSrcEquipmentIdList()

    systemId = getExternalSystemId(u'N3.РЕГИСЗ.ОДЛИ:доп', u'Нетрика, сервис «ОДЛИ», исследования проведённые в локальной лаборатории')
    if destEquipmentIdList:
        destEquipmentId = destEquipmentIdList[0]
        mapTestIdToCode = getMapTestIdToTestCode(destEquipmentId)
        equipmentInterface = getEquipmentInterface(destEquipmentId)

        # выбрать подходящие действия, без группировки
        actionIdList = selectActionIdList(srcEquipmentIdList, systemId, days)
        # в цикле передавать действия и отмечать как переданные
        for actionId in actionIdList:
            try:
                testResults, eventId, serviceId, modifyDatetime = extractResultsEtc(mapTestIdToCode, actionId)
                if testResults and serviceId:
                    clientId   = forceRef(db.translate('Event', 'id', eventId, 'client_id'))
                    clientInfo = getClientInfo(clientId)
                    if equipmentInterface.protocol == 3:
                        sendLocalResultsOverFHIR050(equipmentInterface, clientInfo, serviceId, testResults)
                    elif equipmentInterface.protocol == 4:
                        sendLocalResultsOverFHIR102(equipmentInterface, clientInfo, serviceId, testResults)
                    else:
                        assert False, 'Invalid equipmentInterface.protocol'
                markActionAsExported(systemId, actionId, modifyDatetime)
#            except CIdentificationException:
#                QtGui.qApp.logCurrentException()
#            except FHIRErrorException:
            except Exception:
                QtGui.qApp.logCurrentException()


def getDestEquipmentIdList():
    db = QtGui.qApp.db
    table = db.table('rbEquipment')
    return db.getIdList(table,
                        'id',
                        [ table['status'].eq(1),
                          table['protocol'].inlist([3, 4]),
                          table['roleInIntegration'].eq(2),
                        ]
                       )


def getSrcEquipmentIdList():
    db = QtGui.qApp.db
    table = db.table('rbEquipment')
    return db.getIdList(table,
                        'id',
                        [ table['status'].eq(1),
                          table['roleInIntegration'].eq(1),
                        ]
                       )


def getExternalSystemId(code, name):
    db = QtGui.qApp.db
    table = db.table('rbExternalSystem')
    result = db.translate(table, 'code', code, 'id')
    if not result:
        record = table.newRecord()
        record.setValue('code', code)
        record.setValue('name', name)
        candidate = db.insertRecord(table, record)
        if db.translate(table, 'code', code, 'id') == candidate:
            result = candidate
        else:
            db.deleteRecord(table, table['id'].eq(candidate))
            raise Exception(u'Невозможно добавить запись с кодом «%s» в rbExternalSystem' % code)
    return result


def getMapTestIdToTestCode(eqipmentId):
    db = QtGui.qApp.db
    table = db.table('rbEquipment_Test')
    records = db.getRecordList(table,
                               ['test_id', 'hardwareTestCode'],
                               db.joinAnd([table['equipment_id'].eq(eqipmentId),
                                           table['type'].inlist((1, 2)),
                                           table['test_id'].isNotNull()
                                          ])
                              )
    result = {}
    for record in records:
        code   = forceString(record.value('hardwareTestCode'))
        testId = forceRef(record.value('test_id'))
        result[testId] = code
    return result


def selectActionIdList(equipmentIdList, systemId, days):
    db = QtGui.qApp.db
    tableAction = db.table('Action')
    tableActionType = db.table('ActionType')
    tableActionPropertyType = db.table('ActionPropertyType')
    tableEquipmentTest = db.table('rbEquipment_Test')
    tableEvent = db.table('Event')
    tableExport = db.table('Action_Export')
    tableProbe = db.table('Probe')

    table = tableAction
    table = table.innerJoin( tableActionType, tableActionType['id'].eq( tableAction['actionType_id'] ))
    table = table.innerJoin( tableActionPropertyType, db.joinAnd([
                                                                    tableActionPropertyType['actionType_id'].eq(tableAction['actionType_id']),
                                                                    tableActionPropertyType['deleted'].eq(0)
                                                                 ])
                           )
    table = table.innerJoin( tableEquipmentTest,      tableEquipmentTest['test_id'].eq( tableActionPropertyType['test_id'] ))
    table = table.innerJoin( tableEvent,              tableEvent['id'].eq( tableAction['event_id'] ))
    table = table.leftJoin (tableProbe,               tableProbe['takenTissueJournal_id'].eq(tableAction['takenTissueJournal_id']))
    table = table.leftJoin(tableExport, db.joinAnd([
                                                    tableExport['master_id'].eq(tableAction['id']),
                                                    tableExport['system_id'].eq(systemId)
                                                   ])
                          )
    return db.getDistinctIdList( table,
                                 tableAction['id'],
                                 db.joinAnd([
                                             tableEquipmentTest['equipment_id'].inlist(equipmentIdList),
                                             tableAction['status'].eq(CActionStatus.finished),
                                             tableAction['deleted'].eq(0),
                                             tableAction['endDate'].ge(QDateTime.currentDateTime().addDays(-days)), # совсем старые не берём
                                             tableActionType['class'].eq(1), # диагностика
                                             tableActionType['serviceType'].eq(10), # лаб.исследование
                                             tableEvent['deleted'].eq(0),
                                             'IFNULL(Action.org_id, Event.org_id) = %d' % QtGui.qApp.currentOrgId(),
                                             db.joinOr([ tableProbe['id'].isNull(),
                                                         tableProbe['equipment_id'].inlist(equipmentIdList)
                                                       ]
                                                      ),
                                             tableExport['id'].isNull()
                                            ]),
                                 tableAction['id'].name()
                               )


def extractResultsEtc(mapTestIdToCode, actionId):
    testResults = []
    testCodesSet = set()

    action = CAction.getActionById(actionId)
    record = action.getRecord()
    personId = forceRef(record.value('person_id'))
    if not personId:
        personId = forceRef(record.value('setPerson_id'))
    eventId = action.getEventId()
    nomenclativeServiceId = action.getType().nomenclativeServiceId
    modifyDatetime = record.value('modifyDatetime')

    for prop in action.getProperties():
        propType = prop.type()
        if propType.testId:
            testCode = mapTestIdToCode.get(propType.testId)
            if testCode and testCode not in testCodesSet:
                testResults.append(CResultDto(datetime = forceDateTime(record.value('endDate')),
                                              personId = personId,
                                              testCode = testCode,
                                              value    = prop.getValue(),
                                              unitId   = prop.getUnitId(),
                                              norm     = parseNorm(prop.getNorm()),
                                              evaluation = prop.getEvaluation(),
                                              actionId = actionId,
                                              propName = propType.name,
                                             )
                                  )
                testCodesSet.add(testCode)

    return testResults, eventId, nomenclativeServiceId, modifyDatetime


def parseNorm(norm):
# регулярное выражение: пара чисел с плав.точкой через минус
# впрочем, одно из чисел может отсутствовать
    f = r'([-+]?(?:\d+\.?\d*|\.\d+)(?:[eE][-+]?\d+)?|)'
    s = r'^\s*' + f + r'\s*-\s*' + f + r'\s*$'
    x = re.match(s, norm)
    if x:
        groups = x.groups()
        return ( float(groups[0]) if groups[0] else None,
                 float(groups[1]) if groups[1] else None
               )
    return (None, None)


def markActionAsExported(systemId, actionId, modifyDatetime):
    db = QtGui.qApp.db
    table = db.table('Action_Export')
    record = table.newRecord()
    record.setValue('master_id',      actionId)
    record.setValue('masterDatetime', modifyDatetime)
    record.setValue('system_id',      systemId)
    record.setValue('success',        True)
    record.setValue('dateTime',       QDateTime.currentDateTime())
    db.insertRecord(table, record)


CResultDto = namedtuple('CResultDto',
                          ('datetime',
                           'personId',
                           'testCode',
                           'value',
                           'unitId',
                           'norm',
                           'evaluation',
                           'actionId',
                           'propName',
                          )
                        )
