#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import codecs
from PyQt4 import QtCore
from PyQt4 import QtGui

from library.database import *
from library.Preferences import CPreferences
from TissueJournal.LabInterface import selectTasks
from TissueJournal.ProbeWorkListPage import CProbeSaver, probeSent2LIS
from TissueJournal.Utils                   import getEquipmentInterface
from TissueJournal.LabInterface               import sendRequestOverASTM
try:
    from TissueJournal.LabInterfaceFHIR        import sendRequestOverFHIR
    FHIRisOk = True
except:
    FHIRisOk = False

from Registry.Utils import getClientInfo
from library.Utils import *

globalProcessedItemList = []

def openDatabase(preferences):
    return connectDataBase(preferences.dbDriverName,
                                 preferences.dbServerName,
                                 preferences.dbServerPort,
                                 preferences.dbDatabaseName,
                                 preferences.dbUserName,
                                 preferences.dbPassword,
                                 compressData = preferences.dbCompressData)

def sendTests(probeIdList, probeSaver=None):
    global globalProcessedItemList
    mapEquipmentIdToInterface = {}
    mapClientIdToInfo = {}
    def getEquipmentInterfaceCache(equipmentId):
        if equipmentId in mapEquipmentIdToInterface:
            return mapEquipmentIdToInterface[equipmentId]

        interface = getEquipmentInterface(equipmentId)
        mapEquipmentIdToInterface[equipmentId] = interface
        return interface


    def getClientInfoCache(clientId):
        if clientId in mapClientIdToInfo:
            return mapClientIdToInfo[clientId]
        clientInfo = getClientInfo(clientId)
        mapClientIdToInfo[clientId] = clientInfo
        return clientInfo

    try:
        tasks = selectTasks(probeIdList)
        for task in tasks:
            ((equipmentId, clientId, dateTime, externalId), testList) = task
            equipmentInterface = getEquipmentInterfaceCache(equipmentId)
            clientInfo = getClientInfoCache(clientId)
            try:
                if equipmentInterface:
                    count = 0
                    if equipmentInterface.protocol == 1:
                        count = sendRequestOverASTM(None,
                                            equipmentId,
                                            equipmentInterface,
                                            clientInfo,
                                            QDateTime(dateTime),
                                            externalId,
                                            testList,
                                            probeSaver)
                    elif  equipmentInterface.protocol == 3:
                        if FHIRisOk:
                            count = sendRequestOverFHIR(None,
                                                equipmentId,
                                                equipmentInterface,
                                                clientInfo,
                                                QDateTime(dateTime),
                                                externalId,
                                                testList,
                                                probeSaver)
                        else:
                            raise Exception('FHIR support is disabled')
                    if count:
                        log('ok', 'probe (%i,%s) successfully sended'%(testList[0][0], externalId), 3)
                        globalProcessedItemList.append(testList[0][0])
            except Exception as e:
                log('error', 'sending probe (%i,%s) failed: %s'%(testList[0][0], externalId, unicode(e)), 1)
                globalProcessedItemList.append(testList[0][0])
    except Exception as e:
        log('error', 'preparing data failed: %s'%unicode(e), 1)

def save(item, status=None):
    global globalProcessedItemList
    item.setValue('status', QVariant(status))
    db = QtGui.qApp.db
    id = db.updateRecord('Probe', item)
    globalProcessedItemList.append(id)


def log(title, message, level):
    app = QtGui.qApp
    if app.logLevel >= level:
        try:
            if not os.path.exists(app.logDir):
                os.makedirs(app.logDir)
            logFile = os.path.join(app.logDir, 'error.log')
            timeString = unicode(QtCore.QDateTime.currentDateTime().toString(QtCore.Qt.SystemLocaleDate))
            logString = u'%s %s: %s\n' % (timeString, title, message)
            file = codecs.open(logFile, mode='a', encoding=locale.getpreferredencoding())
            file.write(logString)
            file.close()
        except:
            pass

def main():
    try:
        global globalProcessedItemList
        app = QtCore.QCoreApplication(sys.argv)
        preferences = CPreferences('autoLabExchange.ini')
        preferences.load()
        db = openDatabase(preferences)
        app.db = db
        app.userId = 1
        app.logLevel = preferences.appPrefs.get('logLevel', 3)
        logDir = preferences.appPrefs.get('logDir', None)
        if not logDir:
            logDir = os.path.join(unicode(QDir.toNativeSeparators(QDir.homePath())), '.labExchange')
        app.logDir = logDir
        app.currentOrgId = lambda: preferences.appPrefs.get('currentOrgId', None)
        QtGui.qApp = app
        tableProbe = db.table('Probe')
        tableEquip = db.table('rbEquipment')
        table = tableProbe.leftJoin(tableEquip, tableProbe['equipment_id'].eq(tableEquip['id']))
        cond = [tableProbe['status'].eq(1), tableEquip['samplePreparationMode'].eq(1)]
        #cond.append(tableProbe['id'].lt())
        recordList = db.getRecordList(table, 'Probe.*', where=cond, order='Probe.id')
        idList = []
        externalIdList = {}
        for record in recordList:
            id = forceInt(record.value('id'))
            idList.append(id)
            externalIdList[id] = forceString(record.value('externalId'))
        probeSaver = CProbeSaver({'status':probeSent2LIS},
                                     recordList,
                                     save)
        sendTests(idList, probeSaver=probeSaver)
        for id in idList:
            if id not in globalProcessedItemList:
                log('warning', 'probe (%i,%s) was not processed'%(id, externalIdList[id]), 2)
    except Exception as e:
        log('error', '%s'%unicode(e), 1)
    app.quit()


if __name__ == '__main__':
    main()
