#!/usr/bin/env python
# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################


import sys
from sys import *
import codecs
import traceback

from PyQt4 import QtCore, QtGui, QtSql
from PyQt4.QtCore import *
from PyQt4.QtSql import *

from library.Utils          import *
from library.database       import *
from library.Preferences    import CPreferences
from preferences.connection import CConnectionDialog
from library.crbcombobox    import CRBComboBox
from library.Calendar       import CCalendarInfo
from Reports.MesDescription import showMesDescription
from Reports.ReportView     import CReportViewDialog

from appendix.mes.MesInfo    import *
from appendix.mes.Ui_MesList import Ui_MainWindow
from appendix.mes.MesEdit    import MesEditor
from appendix.mes.CSG        import CCSGTableModel, CCSGMKBModel, CCSGServicesModel, CCSGEditor

from appendix.mes.RefBooksLocal.RBVisitType import CRBVisitTypeList
from appendix.mes.RefBooksLocal.RBBloodPreparation import CRBBloodPreparationList
from appendix.mes.RefBooksLocal.RBBloodPreparationType import CRBBloodPreparationTypeList
from appendix.mes.RefBooksLocal.RBEquipment import CRBEquipmentList
from appendix.mes.RefBooksLocal.RBEquipmentGroup import CRBEquipmentGroupList
from appendix.mes.RefBooksLocal.RBMedicament import CRBMedicamentList
from appendix.mes.RefBooksLocal.RBMedicamentDosageForm import CRBMedicamentDosageFormList
from appendix.mes.RefBooksLocal.RBMedicamentGroup import CRBMedicamentGroupList
from appendix.mes.RefBooksLocal.RBMesGroup import CRBMesGroupList
from appendix.mes.RefBooksLocal.RBNutrient import CRBNutrientList
from appendix.mes.RefBooksLocal.RBNutrientGroup import CRBNutrientGroupList
from appendix.mes.RefBooksLocal.RBService import CRBServiceList
from appendix.mes.RefBooksLocal.RBServiceGroup import CRBServiceGroupList
from appendix.mes.RefBooksLocal.RBSpeciality import CRBSpecialityList
from appendix.mes.RefBooksLocal.MesKSG import CMesKSGList

from appendix.mes.Exchange.ImportXML import ImportXML
from appendix.mes.Exchange.ExportXML import ExportXML

title = u'САМСОН'
subtitle = u'Редактор МЭС'
about = u'Комплекс Программных Средств \n' \
        u'"Система автоматизации медико-страхового обслуживания населения"\n' \
        u'КПС «%s»\n'   \
        u'Версия 2.5 (ревизия %s от %s)\n' \
        u'Утилита "Редактор МЭС"\n' \
        u'Copyright © 2014 ООО "САМСОН Групп"\n' \
        u'распространяется под лицензией GNU GPL v.3 или выше\n' \
        u'Основано на КПС «САМСОН-ВИСТА» версии 2.0\n' \
        u'телефон тех.поддержки (812) 418-39-70'


class mainWin(QtGui.QMainWindow, Ui_MainWindow):
    # Номера колонок в mesView
    colId = 0
    colActive = 1
    colGroupName = 2
    colKSGName = 3
    colCode = 4
    colName = 5
    colAvgDuration = 6
    colPatientModel =  7
    colTariff = 8
    colDescr = 9
    colMinDuration = 10
    colMaxDuration = 11
    colKSGNorm = 12
    colGroupId = 13
    colInternal = 14
    colPeriodicity = 15
    colKSGId = 16
    colEndDate = 17
    colPolyTrauma = 18

    # Скрытые колонки в mesView
    hiddenColumns = (colMinDuration, colMaxDuration,
                         colKSGNorm, colGroupId, colId, colKSGId)

    def __init__(self, app):
        QtGui.QMainWindow.__init__(self)
        self.qApp = app
        QtGui.qApp = app

        self.openDbSetup()
        self.tableModelCSGList = CCSGTableModel(self)
        self.tableSelectionModelCSGList = QtGui.QItemSelectionModel(self.tableModelCSGList, self)
        self.tableSelectionModelCSGList.setObjectName('tableSelectionModelCSGList')

        self.modelCSGDiagnosis = CCSGMKBModel(self)
        self.selectionModelCSGDiagnosis = QtGui.QItemSelectionModel(self.modelCSGDiagnosis, self)
        self.selectionModelCSGDiagnosis.setObjectName('selectionModelCSGDiagnosis')

        self.modelCSGService = CCSGServicesModel(self)
        self.selectionModelCSGService = QtGui.QItemSelectionModel(self.modelCSGService, self)
        self.selectionModelCSGService.setObjectName('selectionModelCSGService')

        self.setupUi(self)
        self.setWindowFlags(self.windowFlags() | Qt.WindowMaximizeButtonHint)


        self.mes = CMesInfo()
        self.initModels()
        self.initView()
        self.initFilter()
        self.openDbPostSetup()
#        self.openDb()
        self.setTableParam()

        if self.qApp.db:
            self.cmbGroup.setTable('mrbMESGroup', filter='deleted=0')

        self.tblCSGList.setModel(self.tableModelCSGList)
        self.tblCSGList.setSelectionModel(self.tableSelectionModelCSGList)
        self.tblCSGList.installEventFilter(self)

        self.tblCSGDiagnosis.setModel(self.modelCSGDiagnosis)
        self.tblCSGDiagnosis.setSelectionModel(self.selectionModelCSGDiagnosis)

        self.tblCSGService.setModel(self.modelCSGService)
        self.tblCSGService.setSelectionModel(self.selectionModelCSGService)

        self.showMaximized()

        QObject.connect(self.action_connect, QtCore.SIGNAL('triggered()'), self.openDb)
        QObject.connect(self.action_disconnect, QtCore.SIGNAL('triggered()'), self.closeDb)
        QObject.connect(self.action_add, QtCore.SIGNAL('triggered()'), self.add)
        QObject.connect(self.action_edit, QtCore.SIGNAL('triggered()'), self.edit)
        QObject.connect(self.action_print, QtCore.SIGNAL('triggered()'), self.print_)
        QObject.connect(self.action_find, QtCore.SIGNAL('triggered()'), self.find)
        QObject.connect(self.action_connection, QtCore.SIGNAL('triggered()'), self.on_actConnection_triggered)
        QObject.connect(self.action_about, QtCore.SIGNAL('triggered()'), self.on_actAbout_triggered)
        QObject.connect(self.action_Qt, QtCore.SIGNAL('triggered()'), self.on_actAboutQt_triggered)

        for action in dir(self):
            if action[:9] == 'action_rb':
                QObject.connect(getattr(self, action), QtCore.SIGNAL('triggered()'), getattr(self, 'on_act' + action[7:] + '_triggered'))

        QObject.connect(self.btnAdd, QtCore.SIGNAL('clicked()'), self.add)
        QObject.connect(self.btnEdit, QtCore.SIGNAL('clicked()'), self.edit)
        QObject.connect(self.btnFind, QtCore.SIGNAL('clicked()'), self.find)
        QObject.connect(self.mesView, QtCore.SIGNAL('doubleClicked(QModelIndex)'), self.edit)
        QObject.connect(self.btnPrint, QtCore.SIGNAL('clicked()'), self.print_)
        QObject.connect(self.btnClear, QtCore.SIGNAL('clicked()'), self.clearFilter)
        QObject.connect(self.btnUpdate, QtCore.SIGNAL('clicked()'), self.updateFilter)

        self.tblCSGList.model().setIdList(self.setCSGList())
        self.tblCSGList.setCurrentRow(0)
        self.txtCSGBrowser.setHtml(self.getCSGBrowserInfo())
        self.lblCountCSG.setText(u'В списке %d записей'%self.tableModelCSGList.rowCount())
        self.tblCSGList.addPopupDublicateRow()
        self.tblCSGList.addPopupDelRow()


    def getCSGFilter(self):
        db = QtGui.qApp.db
        table = db.table('CSG')
        cond = []
        if self.checkCode.isChecked():
            cond.append(table['code'].like(forceString(self.edtCode.text()) + u'%'))
        if self.checkName.isChecked():
            cond.append(table['name'].like(forceString(self.edtName.text()) + u'%'))
        if self.checkMKB.isChecked():
            cond.append('''EXISTS(SELECT *
                                  FROM CSG_Diagnosis
                                  WHERE CSG_Diagnosis.master_id = CSG.id AND CSG_Diagnosis.mkb LIKE '%s%%')'''%forceString(self.edtMKB.text()))
        if self.rbActive.isChecked():
            cond.append(table['active'].eq(1))
        elif self.rbNotActive.isChecked():
            cond.append(table['active'].eq(0))
        if self.rbInternal.isChecked():
            cond.append(table['internal'].eq(1))
        elif self.rbExternal.isChecked():
            cond.append(table['internal'].eq(0))
        return cond


    def setCSGList(self):
        db = QtGui.qApp.db
        if not db:
            return []
        table = db.table('CSG')
        cond = self.getCSGFilter()
        return db.getDistinctIdList(table, table['id'].name(),
                              where=cond,
                              order=u'CSG.code ASC, CSG.name ASC')


    @pyqtSignature('QModelIndex')
    def on_tblCSGList_doubleClicked(self, index):
        self.edit()


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_tableSelectionModelCSGList_currentChanged(self, current, previous):
        CSGId = self.tblCSGList.itemId(current)
        if CSGId:
            db = QtGui.qApp.db
            table = db.table('CSG_Diagnosis')
            idList = db.getIdList(table, 'id', [table['master_id'].eq(CSGId)],  ['id'])
            self.tblCSGDiagnosis.model().setIdList(idList)
            table = db.table('CSG_Service')
            idList = db.getIdList(table, 'id', [table['master_id'].eq(CSGId)],  ['id'])
            self.tblCSGService.model().setIdList(idList)
        self.txtCSGBrowser.setHtml(self.getCSGBrowserInfo())


    @pyqtSignature('int')
    def on_tabMESCSGWidget_currentChanged(self, index):
        enable = not index
        self.checkGroup.setEnabled(enable)
        self.cmbGroup.setEnabled(enable)


    def initModels(self):
        self.mesModel = CMesModel(self)
        self.mkbModel = CQueryModel(self)
        self.servicesModel = CQueryModel(self)
        self.visitsModel = CQueryModel(self)
        self.equipmentModel = CQueryModel(self)
        self.medicamentModel = CQueryModel(self)
        self.bloodModel = CQueryModel(self)
        self.nutrientModel = CQueryModel(self)
        self.limitedBySexAgeModel = CQueryModel(self)


    def initView(self):
        self.mesView.setModel(self.mesModel)
        self.mesView.addPopupDublicateRow()
        self.mesView.addPopupDelRow()
        QObject.connect(self.mesView.selectionModel(), QtCore.SIGNAL('currentRowChanged(QModelIndex, QModelIndex)'), self.update)
        self.tableMKB.setModel(self.mkbModel)
        self.tableServices.setModel(self.servicesModel)
        self.tableVisits.setModel(self.visitsModel)
        self.tableEquipment.setModel(self.equipmentModel)
        self.tableMedicament.setModel(self.medicamentModel)
        self.tableBlood.setModel(self.bloodModel)
        self.tableNutrient.setModel(self.nutrientModel)
        self.tableLimitedBySexAge.setModel(self.limitedBySexAgeModel)


    def initFilter(self):
        self.filter = '1'


    def clearFilter(self):
        self.checkGroup.setChecked(False)
        self.checkCode.setChecked(False)
        self.checkName.setChecked(False)
        self.checkMKB.setChecked(False)
        self.checkMKBEx.setChecked(False)
        self.rbActive.setChecked(True)
        self.rbInternalAll.setChecked(True)
        self.updateFilter()


    def updateFilter(self):
        if self.tabMESCSGWidget.currentIndex():
            self.updateCSGList()
        else:
            self.filter = '1'
            if self.checkGroup.isChecked():
                group_id = self.cmbGroup.value()
                if group_id:
                    self.filter += """ AND MES.group_id = %d"""%group_id
            if self.checkCode.isChecked():
                self.filter += """ AND MES.code like '%s%%'"""%self.edtCode.text()
            if self.checkName.isChecked():
                self.filter += """ AND MES.name like '%%%s%%'"""%self.edtName.text()
            if self.checkMKB.isChecked():
                self.filter += """ AND exists
                                                (select * from MES_mkb
                                                where MES_mkb.master_id = MES.id
                                                and MES_mkb.mkb like '%s%%')"""%self.edtMKB.text()
            if self.checkMKBEx.isChecked():
                self.filter += """ AND exists
                                                (select * from MES_mkb
                                                where MES_mkb.master_id = MES.id
                                                and MES_mkb.mkbEx like '%s%%')"""%self.edtMKBEx.text()
            if self.rbActive.isChecked():
                self.filter += """ AND MES.active = 1"""
            elif self.rbNotActive.isChecked():
                self.filter += """ AND MES.active = 0"""
            if self.rbInternal.isChecked():
                self.filter += """ AND MES.internal = 1"""
            elif self.rbExternal.isChecked():
                self.filter += """ AND MES.internal = 0"""

            self.updateMain()


    def getCellStr(self, row, column):
        return self.mesModel.data(self.mesModel.index(row, column)).toString()


    def getCellInteger(self, row, column):
        return self.mesModel.data(self.mesModel.index(row, column)).toInt()[0]


    def getCellFloat(self, row, column):
        return self.mesModel.data(self.mesModel.index(row, column)).toDouble()[0]


    def getCellDate(self, row, column):
        return self.mesModel.data(self.mesModel.index(row, column)).toDate()


    def getCellBool(self, row, column):
        return self.mesModel.data(self.mesModel.index(row, column)).toBool()


    def getBrowserCode(self):
        result = ''
        result += '<b>' + self.mes.getName() + '</b>' + '<br>'
        result += u'Минимальная длительность: ' + '<i>' + str(self.mes.getMinDuration()) + '</i>' + '<br>'
        result += u'Максимальная длительность: ' + '<i>' + str(self.mes.getMaxDuration()) + '</i>' + '<br>'
        result += u'Средняя длительность: ' + '<i>' + str(self.mes.getAvgDuration()) + '</i>' + '<br>'
        result += u'Норматив визитов: ' + '<i>' + str(self.mes.getKSGNorm()) + '</i>' + '<br>'
        result += u'Дата окончания: ' + '<i>' + forceString(self.mes.getEndDate()) + '</i>' + '<br>'
        result += u'Политравма: ' + '<i>' + [u'нет', u'да'][forceInt(self.mes.getPolyTrauma())] + '</i>' + '<br>'
        patient_model = self.mes.getPatientModelDict()
        formatted_model = ["%s: <i>%s</i>"%(key, patient_model[key]) for key in patient_model]
        result += '<br>'.join(formatted_model)
        return result


    def getCSGBrowserInfo(self):
        CSGId = self.tblCSGList.currentItemId()
        result = ''
        if CSGId:
            db = QtGui.qApp.db
            table = db.table('CSG')
            record = db.getRecordEx(table, '*', [table['id'].eq(CSGId)])
            if record:
                result += '<b>' + forceString(record.value('name')) + '</b>' + '<br>'
                result += u'Код: ' + '<i>' + forceString(record.value('code')) + '</i>' + '<br>'
                result += u'Пол: ' + '<i>' + [u'не определено', u'мужской', u'женский'][forceInt(record.value('sex'))] + '</i>' + '<br>'
                result += u'Возраст: ' + '<i>' + forceString(record.value('age')) + '</i>' + '<br>'
                result += u'Некое доп. условие: ' + '<i>' + forceString(record.value('note')) + '</i>' + '<br>'
                result += u'Активно: ' + '<i>' + [u'нет', u'да'][forceInt(record.value('active'))] + '</i>' + '<br>'
                result += u'Является внутренним: ' + '<i>' + [u'нет', u'да'][forceInt(record.value('internal'))] + '</i>' + '<br>'
                result += u'Политравма: ' + [u'нет', u'да'][forceInt(record.value('isPolyTrauma'))] + '\n'
        return result


    def getCSGInfo(self):
        CSGId = self.tblCSGList.currentItemId()
        result = ''
        if CSGId:
            db = QtGui.qApp.db
            table = db.table('CSG')
            record = db.getRecordEx(table, '*', [table['id'].eq(CSGId)])
            if record:
                result += u'Название: ' + forceString(record.value('name')) + '\n'
                result += u'Код: ' + forceString(record.value('code')) + '\n'
                result += u'Пол: ' + [u'не определено', u'мужской', u'женский'][forceInt(record.value('sex'))] + '\n'
                result += u'Возраст: ' + forceString(record.value('age')) + '\n'
                result += u'Некое доп. условие: ' + forceString(record.value('note')) + '\n'
                result += u'Активно: ' + [u'нет', u'да'][forceInt(record.value('active'))] + '\n'
                result += u'Является внутренним: ' + [u'нет', u'да'][forceInt(record.value('internal'))] + '\n'
                result += u'Политравма: ' + [u'нет', u'да'][forceInt(record.value('isPolyTrauma'))] + '\n'
        return result


    def openDb(self):
        settings = QSettings(QSettings.IniFormat, QSettings.UserScope, 'samson-vista', 'mesEditor')
        server = settings.value('server', QtCore.QVariant('localhost')).toString()
        port = settings.value('port', QtCore.QVariant(0)).toInt()[0]
        database = settings.value('database', QtCore.QVariant('mes')).toString()
        user = settings.value('user', QtCore.QVariant('dbuser')).toString()
        password = settings.value('password', QtCore.QVariant('dbpassword')).toString()
        connectionName = settings.value('database', QtCore.QVariant('mes')).toString()
        try:
            self.qApp.db = CMySqlDatabase(server, port, database, user, password, connectionName)
            self.emit(SIGNAL('dbConnectionChanged(bool)'), True)
            #QtGui.qApp.calendarInfo.load()
            self.updateMain()
            self.tableModelCSGList = CCSGTableModel(self)
            self.tableSelectionModelCSGList = QtGui.QItemSelectionModel(self.tableModelCSGList, self)
            self.tableSelectionModelCSGList.setObjectName('tableSelectionModelCSGList')
            self.modelCSGDiagnosis = CCSGMKBModel(self)
            self.selectionModelCSGDiagnosis = QtGui.QItemSelectionModel(self.modelCSGDiagnosis, self)
            self.selectionModelCSGDiagnosis.setObjectName('selectionModelCSGDiagnosis')
            self.modelCSGService = CCSGServicesModel(self)
            self.selectionModelCSGService = QtGui.QItemSelectionModel(self.modelCSGService, self)
            self.selectionModelCSGService.setObjectName('selectionModelCSGService')
            self.tblCSGList.setModel(self.tableModelCSGList)
            self.tblCSGList.setSelectionModel(self.tableSelectionModelCSGList)
            self.tblCSGList.installEventFilter(self)
            self.tblCSGDiagnosis.setModel(self.modelCSGDiagnosis)
            self.tblCSGDiagnosis.setSelectionModel(self.selectionModelCSGDiagnosis)
            self.tblCSGService.setModel(self.modelCSGService)
            self.tblCSGService.setSelectionModel(self.selectionModelCSGService)
            self.connect(self.tableSelectionModelCSGList, SIGNAL('currentChanged(QModelIndex,QModelIndex)'),
                         self.on_tableSelectionModelCSGList_currentChanged)
            self.tblCSGList.addPopupDublicateRow()
            self.tblCSGList.addPopupDelRow()
            self.updateCSGList()
        except Exception, e:
            self.qApp.db = None
            QtGui.QMessageBox.critical(self,
                                            u'Произошла ошибка',
                                            unicode(e),

                       QtGui.QMessageBox.Close)
            return


    def openDbSetup(self):
        settings = QSettings(QSettings.IniFormat, QSettings.UserScope, 'samson-vista', 'mesEditor')
        server = settings.value('server', QtCore.QVariant('localhost')).toString()
        port = settings.value('port', QtCore.QVariant(0)).toInt()[0]
        database = settings.value('database', QtCore.QVariant('mes')).toString()
        user = settings.value('user', QtCore.QVariant('dbuser')).toString()
        password = settings.value('password', QtCore.QVariant('dbpassword')).toString()
        connectionName = settings.value('database', QtCore.QVariant('mes')).toString()
        try:
            self.qApp.db = CMySqlDatabase(server, port, database, user, password, connectionName)
            self.emit(SIGNAL('dbConnectionChanged(bool)'), True)
            #QtGui.qApp.calendarInfo.load()
        except Exception, e:
            self.qApp.db = None
            QtGui.QMessageBox.critical(self,
                                            u'Произошла ошибка',
                                            unicode(e),

                       QtGui.QMessageBox.Close)
            return


    def openDbPostSetup(self):
        try:
            self.updateMain()
        except Exception, e:
            self.qApp.db = None
            QtGui.QMessageBox.critical(self,
                                            u'Произошла ошибка',
                                            unicode(e),
                       QtGui.QMessageBox.Close)
            return


    def closeDb(self):
        if self.qApp.db:
            connectionName = self.qApp.db.db.databaseName()
            self.tblCSGList.setModel(None)
            self.tblCSGDiagnosis.setModel(None)
            self.tblCSGService.setModel(None)
            del self.tableModelCSGList
            del self.modelCSGDiagnosis
            del self.modelCSGService
            self.txtCSGBrowser.setHtml('')
            self.lblCountCSG.setText(u'')
            self.qApp.db.db.close()
            self.qApp.db.close()
            self.mesModel.clear()
            self.mesModel.reset()
            self.textInfo.setHtml('')
            self.lblCount.setText(u'')
            QtSql.QSqlDatabase.removeDatabase(connectionName)
            self.qApp.db.db = None
            self.qApp.db = None
            self.emit(SIGNAL('dbConnectionChanged(bool)'), False)
            #QtGui.qApp.calendarInfo.clear()


    def updateMain(self):
        row = self.mesView.currentIndex().row()
        self.mesModel.setParam(self.qApp.db, self.filter)
        self.lblCount.setText(u'В списке %d записей'%self.mesModel.rowCount())
        if (not self.mesView.currentRow()) and self.mesModel.rowCount(): # такая фигня всегда при обновлении запроса к модели
            self.mesView.setCurrentRow(row if row > 0 else 0)


    def updateCSGList(self):
        row = self.tblCSGList.currentRow()
        self.tblCSGList.model().setIdList(self.setCSGList())
        self.lblCountCSG.setText(u'В списке %d записей'%self.tableModelCSGList.rowCount())
        if (not self.tblCSGList.currentRow()) and self.tableModelCSGList.rowCount():
            self.tblCSGList.setCurrentRow(row if row and row > 0 else 0)
        self.txtCSGBrowser.setHtml(self.getCSGBrowserInfo())


    def update(self, a=None, b=None):
        row = self.mesView.currentIndex().row()
        self.mes = CMesInfo(self.getCellInteger(row, self.colId))  # id
        self.mes.init(self.getCellInteger(row, self.colActive), # active
                      self.getCellInteger(row, self.colInternal), # internal
                 self.getCellInteger(row, self.colGroupId),  # group_id
                 self.getCellStr(row, self.colCode), # code
                 self.getCellStr(row, self.colName),   # name
                 self.getCellInteger(row, self.colMinDuration), # minDuration
                 self.getCellInteger(row, self.colMaxDuration), # maxDuration
                 self.getCellFloat(row, self.colAvgDuration), # avgDuration
                 self.getCellStr(row, self.colPatientModel),     # patientModel
                 self.getCellFloat(row, self.colTariff),     # tariff
                 self.getCellStr(row, self.colDescr),   # descr
                 self.getCellInteger(row, self.colKSGNorm),
                 self.getCellInteger(row, self.colPeriodicity), # periodicity
                 self.getCellInteger(row, self.colKSGId),
                 self.getCellDate(row, self.colEndDate),
                 self.getCellInteger(row, self.colPolyTrauma)
                 )
        self.textInfo.setHtml(self.getBrowserCode())
        self.mkbModel.setQuery(self.mes.getMKBQuery(), self.qApp.db.db)
        self.servicesModel.setQuery(self.mes.getServicesQuery(), self.qApp.db.db)
        self.visitsModel.setQuery(self.mes.getVisitsQuery(), self.qApp.db.db)
        self.equipmentModel.setQuery(self.mes.getEquipmentQuery(), self.qApp.db.db)
        self.medicamentModel.setQuery(self.mes.getMedicamentQuery(), self.qApp.db.db)
        self.bloodModel.setQuery(self.mes.getBloodQuery(), self.qApp.db.db)
        self.nutrientModel.setQuery(self.mes.getNutrientQuery(), self.qApp.db.db)
        self.limitedBySexAgeModel.setQuery(self.mes.getLimitedBySexAgeQuery(), self.qApp.db.db)


    def cloneCurrentRow(self):
        row = self.mesView.currentIndex().row()
        self.mesModel.cloneRow(row)


    def on_actConnection_triggered(self):
        #print 'connection!'
        dlg   = CConnectionDialog(self)
        settings = QSettings(QSettings.IniFormat, QSettings.UserScope, 'samson-vista', 'mesEditor')
        #dlg.setDriverName(preferences.dbDriverName)
        dlg.setServerName(settings.value('server', QtCore.QVariant('localhost')).toString())
        dlg.setServerPort(settings.value('port', QtCore.QVariant(0)).toInt()[0])
        dlg.setDatabaseName(settings.value('database', QtCore.QVariant('mes')).toString())
        dlg.setUserName(settings.value('user', QtCore.QVariant('dbuser')).toString())
        dlg.setPassword(settings.value('password', QtCore.QVariant('dbpassword')).toString())
        if dlg.exec_():
            #preferences.dbDriverName = dlg.driverName()
            settings.setValue('server', QtCore.QVariant(dlg.serverName()))
            settings.setValue('port', QtCore.QVariant(dlg.serverPort()))
            settings.setValue('database', QtCore.QVariant(dlg.databaseName()))
            settings.setValue('user', QtCore.QVariant(dlg.userName()))
            settings.setValue('password', QtCore.QVariant(dlg.password()))


    def add(self):
        if self.tabMESCSGWidget.currentIndex():
            editor = CCSGEditor(self)
            if editor.exec_() == QtGui.QDialog.Accepted:
                self.updateCSGList()
        else:
            mes = CMesInfo(0)
            editor = MesEditor(self, mes)
            if editor.exec_() == QtGui.QDialog.Accepted:
                self.updateMain()


    def edit(self):
        if self.tabMESCSGWidget.currentIndex():
            CSGId = self.tblCSGList.currentItemId()
            if CSGId:
                editor = CCSGEditor(self)
                editor.load(CSGId)
                if editor.exec_() == QtGui.QDialog.Accepted:
                    self.updateCSGList()
        else:
            editor = MesEditor(self, self.mes)
            if editor.exec_() == QtGui.QDialog.Accepted:
                self.updateMain()


    def print_(self):
        if self.tabMESCSGWidget.currentIndex():
            CSGId = self.tblCSGList.currentItemId()
            if CSGId:
                self.getCSGFullDescription(CSGId)
        else:
            view = CReportViewDialog(self)
            view.setWindowTitle(u'МЭС')
            view.setText(self.mes.getFullDescription())
            view.exec_()


    def getCSGFullDescription(self, CSGId):
        doc = QtGui.QTextDocument()
        cursor = QtGui.QTextCursor(doc)
        cursor.setCharFormat(CReportBase.ReportTitle)
        cursor.insertText(u'КСГ')
        cursor.insertBlock()
        cursor.setCharFormat(CReportBase.ReportBody)
        cursor.insertText(u'%s'%(self.getCSGInfo()))
        cursor.insertBlock()

        model = self.tblCSGDiagnosis.model()
        colWidths  = [ self.tblCSGDiagnosis.columnWidth(i) for i in xrange(model.columnCount()-1) ]
        colWidths.insert(0,10)
        totalWidth = sum(colWidths)
        tableColumns = []
        iColNumber = False
        for iCol, colWidth in enumerate(colWidths):
            if iColNumber == False:
                tableColumns.append(('10%', [u'№'], CReportBase.AlignRight))
                iColNumber = True
            tableColumns.append(('90%', [forceString(model._cols[iCol].title())], CReportBase.AlignLeft))
        table = createTable(cursor, tableColumns)
        for iModelRow in xrange(model.rowCount()):
            iTableRow = table.addRow()
            table.setText(iTableRow, 0, iModelRow+1)
            for iModelCol in xrange(model.columnCount()):
                index = model.createIndex(iModelRow, iModelCol)
                text = forceString(model.data(index))
                table.setText(iTableRow, iModelCol+1, text)

        cursor.movePosition(QtGui.QTextCursor.End)
        cursor.insertBlock()
        cursor.insertBlock()

        model = self.tblCSGService.model()
        colWidths  = [ self.tblCSGService.columnWidth(i) for i in xrange(model.columnCount()-1) ]
        colWidths.insert(0,10)
        totalWidth = sum(colWidths)
        tableColumns = []
        iColNumber = False
        for iCol, colWidth in enumerate(colWidths):
            widthInPercents = str(max(1, colWidth*90/totalWidth))+'%'
            if iColNumber == False:
                tableColumns.append((widthInPercents, [u'№'], CReportBase.AlignRight))
                iColNumber = True
            tableColumns.append((widthInPercents, [forceString(model._cols[iCol].title())], CReportBase.AlignLeft))
        table = createTable(cursor, tableColumns)
        for iModelRow in xrange(model.rowCount()):
            iTableRow = table.addRow()
            table.setText(iTableRow, 0, iModelRow+1)
            for iModelCol in xrange(model.columnCount()):
                index = model.createIndex(iModelRow, iModelCol)
                text = forceString(model.data(index))
                table.setText(iTableRow, iModelCol+1, text)

        html = doc.toHtml(QByteArray('utf-8'))
        view = CReportViewDialog(self)
        view.setText(html)
        view.exec_()


    def find(self):
        if self.checkGroup.isChecked():
            self.cmbGroup.setFocus()
        elif self.checkCode.isChecked():
            self.edtCode.setFocus()
        elif self.checkName.isChecked():
            self.edtName.setFocus()
        elif self.checkMKB.isChecked():
            self.edtMKB.setFocus()
        elif self.checkMKBEx.isChecked():
            self.edtMKBEx.setFocus()
        else:
            self.checkGroup.setFocus()


    def keyPressEvent(self, e):
        if e.key() in (Qt.Key_Enter, Qt.Key_Return):
            self.btnUpdate.click()
        QtGui.QMainWindow.keyPressEvent(self, e)


    def on_actAbout_triggered(self):
        global lastChangedRev
        global lastChangedDate
        try:
            from buildInfo import lastChangedRev, lastChangedDate
        except:
            lastChangedRev  = 'unknown'
            lastChangedDate = 'unknown'
        QtGui.QMessageBox.about(
            self, u'О программе "%s"'%subtitle, about % (title, lastChangedRev, lastChangedDate)
            )


    def on_actAboutQt_triggered(self):
        QtGui.QMessageBox.aboutQt(self, u'О Qt')


    def on_actrbVisitType_triggered(self):
        CRBVisitTypeList(self).exec_()


    def on_actrbBloodPreparation_triggered(self):
        CRBBloodPreparationList(self).exec_()


    def on_actrbBloodPreparationType_triggered(self):
        CRBBloodPreparationTypeList(self).exec_()


    def on_actrbEquipment_triggered(self):
        CRBEquipmentList(self).exec_()


    def on_actrbEquipmentGroup_triggered(self):
        CRBEquipmentGroupList(self).exec_()


    def on_actrbMedicament_triggered(self):
        CRBMedicamentList(self).exec_()


    def on_actrbMedicamentDosageForm_triggered(self):
        CRBMedicamentDosageFormList(self).exec_()


    def on_actrbMedicamentGroup_triggered(self):
        CRBMedicamentGroupList(self).exec_()


    def on_actrbMesGroup_triggered(self):
        CRBMesGroupList(self).exec_()


    def on_actrbNutrient_triggered(self):
        CRBNutrientList(self).exec_()


    def on_actrbNutrientGroup_triggered(self):
        CRBNutrientGroupList(self).exec_()


    def on_actrbService_triggered(self):
        CRBServiceList(self).exec_()


    def on_actrbServiceGroup_triggered(self):
        CRBServiceGroupList(self).exec_()


    def on_actrbSpeciality_triggered(self):
        CRBSpecialityList(self).exec_()


    @QtCore.pyqtSignature('')
    def on_actMesKSG_triggered(self):
        CMesKSGList(self).exec_()


    @QtCore.pyqtSignature('')
    def on_actImportXML_triggered(self):
        ImportXML(self)


    @QtCore.pyqtSignature('')
    def on_actExportXML_triggered(self):
        ExportXML(self)


    def setTableParam(self):
        map(self.mesView.hideColumn, self.hiddenColumns)

        self.mesView.setColumnWidth(self.colActive, 20)
        self.mesView.setColumnWidth(self.colGroupName, 150)
        self.mesView.setColumnWidth(self.colCode, 50)
        self.mesView.setColumnWidth(self.colName , 300)
        self.mesView.setColumnWidth(self.colAvgDuration, 90)
        self.mesView.setColumnWidth(self.colPatientModel, 140)
        self.mesView.setColumnWidth(self.colTariff, 50)
        self.mesView.setColumnWidth(self.colDescr, 300)
        self.mesView.setCurrentIndex(self.mesModel.index(0, 0))
        self.mesView.setFocus(Qt.OtherFocusReason)

        self.tableMKB.setColumnWidth(0, 100)
        self.tableMKB.setColumnWidth(1, 500)

        self.tableServices.setColumnWidth(0, 250)
        self.tableServices.setColumnWidth(1, 100)
        self.tableServices.setColumnWidth(2, 250)
        self.tableServices.setColumnWidth(3, 50)
        self.tableServices.setColumnWidth(4, 50)
        self.tableServices.setColumnWidth(5, 50)
        self.tableServices.setColumnWidth(6, 50)
        self.tableServices.setColumnWidth(7, 50)

        self.tableVisits.setColumnWidth(0, 200)
        self.tableVisits.setColumnWidth(1, 200)
        self.tableVisits.setColumnWidth(2, 100)
        self.tableVisits.setColumnWidth(3, 70)
        self.tableVisits.setColumnWidth(4, 50)
        self.tableVisits.setColumnWidth(5, 100)

        self.tableEquipment.setColumnWidth(0, 100)
        self.tableEquipment.setColumnWidth(1, 500)
        self.tableEquipment.setColumnWidth(2, 50)
        self.tableEquipment.setColumnWidth(3, 50)

        self.tableMedicament.setColumnWidth(0, 80)
        self.tableMedicament.setColumnWidth(1, 100)
        self.tableMedicament.setColumnWidth(2, 150)
        self.tableMedicament.setColumnWidth(3, 100)
        self.tableMedicament.setColumnWidth(4, 100)
        self.tableMedicament.setColumnWidth(5, 100)
        self.tableMedicament.setColumnWidth(6, 100)
        self.tableMedicament.setColumnWidth(7, 100)
        self.tableMedicament.setColumnWidth(8, 150)
        self.tableMedicament.setColumnWidth(9, 50)
        self.tableMedicament.setColumnWidth(10, 50)

        self.tableBlood.setColumnWidth(0, 100)
        self.tableBlood.setColumnWidth(2, 100)
        self.tableBlood.setColumnWidth(3, 100)
        self.tableBlood.setColumnWidth(4, 100)
        self.tableBlood.setColumnWidth(5, 100)

        self.tableNutrient.setColumnWidth(0, 100)
        self.tableNutrient.setColumnWidth(1, 250)
        self.tableNutrient.setColumnWidth(2, 100)
        self.tableNutrient.setColumnWidth(3, 100)
        self.tableNutrient.setColumnWidth(4, 100)
        self.tableNutrient.setColumnWidth(5, 100)

        self.tableLimitedBySexAge.resizeColumnToContents(0)
        self.tableLimitedBySexAge.resizeColumnToContents(1)
        self.tableLimitedBySexAge.resizeColumnToContents(2)


    def closeEvent(self, event):
        if QtGui.qApp.db:
            QtGui.qApp.db.close()
        QtGui.QMainWindow.closeEvent(self, event)


class MesEditorApp(QtGui.QApplication):
    __pyqtSignals__ = ('dbConnectionChanged(bool)'
                      )
    def __init__(self, args):
        QtGui.QApplication.__init__(self, args)
        self.preferences = CPreferences('MesEditor.ini')
        self.preferences.load()
        self.appPreferences = CPreferences('S11App.ini')
        self.disableLock = True
        self._highlightRedDate = None
        self._highlightInvalidDate = None
        self.calendarInfo = CCalendarInfo(self)
        self.mainWindow = mainWin(self)
        self.logDir = os.path.join(unicode(QDir.toNativeSeparators(QDir.homePath())), '.samson-vista')

    def userHasRight(self, action):
        u'Мы ничего страшного не делаем, поэтому пользователь может ВСЁ!'
        return True

    def documentEditor(self):
        return ''

    def startProgressBar(self, steps, format='%v/%m'):
        pass

    def stepProgressBar(self, step=1):
        pass

    def stopProgressBar(self):
        pass

    def enableFastPrint(self):
#        return True
        return forceBool(self.appPreferences.appPrefs.get('enableFastPrint', False))


    def setWaitCursor(self):
        self.setOverrideCursor(QtGui.QCursor(Qt.WaitCursor))


    def highlightRedDate(self):
        if self._highlightRedDate is None:
            self._highlightRedDate = forceBool(QtGui.qApp.preferences.appPrefs.get('highlightRedDate', False))
        return self._highlightRedDate


    def highlightInvalidDate(self):
        if self._highlightInvalidDate is None:
            self._highlightInvalidDate = forceBool(QtGui.qApp.preferences.appPrefs.get('highlightInvalidDate', False))
        return self._highlightInvalidDate


    def call(self, widget, func, params = ()):
        try:
            return True, func(*params)
        except IOError, e:
            msg=''
            if hasattr(e, 'filename'):
                msg = u'%s:\n[Errno %s] %s' % (e.filename, e.errno, anyToUnicode(e.strerror))
            else:
                msg = u'[Errno %s] %s' % (e.errno, anyToUnicode(e.strerror))
            QtGui.QMessageBox.critical(widget,
                u'Произошла ошибка ввода-вывода', msg, QtGui.QMessageBox.Close)
        except Exception, e:
            #self.logCurrentException()
            widget = widget or self.activeModalWidget() or self.mainWindow
            QtGui.QMessageBox.critical( widget,
                                        u'Произошла ошибка',
                                        unicode(e),
                                        QtGui.QMessageBox.Close)
        return False, None


    def log(self, title, message, stack=None):
        try:
            if not os.path.exists(self.logDir):
                os.makedirs(self.logDir)
            logFile = os.path.join(self.logDir, 'error.log')
            timeString = unicode(QDateTime.currentDateTime().toString(Qt.SystemLocaleDate))
            logString = u'%s\n%s: %s(%s)\n' % ('='*72, timeString, title, message)
            if stack:
                try:
                    logString += ''.join(traceback.format_list(stack)).decode('utf-8') + '\n'
                except:
                    logString += 'stack lost\n'
            file = codecs.open(logFile, mode='a', encoding=locale.getpreferredencoding())
            file.write(logString)
            file.close()
        except:
            pass


    def logException(self, exceptionType, exceptionValue, exceptionTraceback):
        title = repr(exceptionType)
        message = anyToUnicode(exceptionValue)
        self.log(title, message, traceback.extract_tb(exceptionTraceback))
        sys.__excepthook__(exceptionType, exceptionValue, exceptionTraceback)


    def logCurrentException(self):
        self.logException(*sys.exc_info())


if __name__ == "__main__":
    QtGui.qApp = MesEditorApp(sys.argv)
    try:
        #mw=mainWin()
        #mw.show()
        #QtGui.qApp.mainWindow.show()
        QtGui.qApp.exec_()
        QtGui.qApp.preferences.save()
    except Exception, e:
        QtGui.QMessageBox.critical( None,
                                    'error',
                                    unicode(e),
                                    QtGui.QMessageBox.Close)
    mw = None
    QtGui.qApp = None
