<?php

require_once 'config/config.php';
require_once 'library/setup.php';
require_once 'library/trace-log.php';
require_once 'library/database.php';
require_once 'library/formatSNILS.php';

#var_dump( $_SERVER );

function findClients($aSex,
                     $aBirthDate,
                     $aLastName,
                     $aFirstName,
                     $aPatrName,
                     $aSNILS,
                     $aPolicySerial,
                     $aPolicyNumber,
#                     $aDocumentSerial,
#                     $aDocumentNumber,
                     $aLimit
                    )
{
    $db = GetDB();

    $vSearchByPolicy = !is_null($aPolicySerial) || !is_null($aPolicyNumber);
    if ( $vSearchByPolicy )
    {
        $vPPCond = $db->CondAnd('ClientPolicy.deleted = 0',
                                '(rbPolicyType.isCompulsory OR rbPolicyType.id IS NULL)',
                                is_null($aPolicySerial) ? '' : $db->CondEqual('ClientPolicy`.`serial', $aPolicySerial),
                                is_null($aPolicyNumber) ? '' : $db->CondEqual('ClientPolicy`.`number', $aPolicyNumber)
                               );
    }
    else
        $vPPCond = '';

#    $vSearchByDocument = $aDocumentSerial || $aDocumentNumber;
#    if ( $vSearchByDocument )
#    {
#        $vPDCond = $db->CondAnd('ClientDocument.deleted = 0',
#                                $aDocumentSerial ? $db->CondEqual('ClientDocument`.`serial', $aDocumentSerial) : '',
#                                $aDocumentNumber ? $db->CondEqual('ClientDocument`.`number', $aDocumentNumber) : ''
#                               );
#    }
#    else
#        $vPDCond = '';

    $vCond = $db->CondAnd($aSex                ? $db->CondEqual('Client`.`sex', $aSex)  : '',
                          is_null($aBirthDate) ? '' : $db->CondEqual('birthDate', $aBirthDate),
                          is_null($aLastName)  ? '' : $db->CondLikeBase('lastName',  ConvertDotsToMask($aLastName)),
                          is_null($aFirstName) ? '' : $db->CondLikeBase('firstName', ConvertDotsToMask($aFirstName)),
                          is_null($aPatrName)  ? '' : $db->CondLikeBase('patrName',  ConvertDotsToMask($aPatrName)),
                          is_null($aSNILS)     ? '' : $db->CondEqual('Client`.`SNILS', $aSNILS),
                          $vPPCond,
                          '`Client`.`deleted`=0'
                         );
    $vTable = 'Client';
    if ( $vSearchByPolicy )
       $vTable .= ' LEFT JOIN ClientPolicy ON ClientPolicy.client_id = Client.id'
               .  ' LEFT JOIN rbPolicyType ON rbPolicyType.id = ClientPolicy.policyType_id';
    return $db->SelectIDsEx($vTable, 'DISTINCT Client.id', $vCond, 'lastName, firstName, patrName, birthDate, Client.id', $aLimit);
}



function checkPatient()
{
    // заголовки запроса
    if ( @$_SERVER['REQUEST_METHOD'] != 'POST' )
        throw new Exception('Only HTTP method POST is accepted.');
    if ( @$_SERVER['CONTENT_TYPE'] != 'application/json' )
        throw new Exception('Only Content-Type "application/json" is accepted');

    if ( array_key_exists('HTTP_ACCEPT', $_SERVER) && $_SERVER['HTTP_ACCEPT'] != 'application/json' )
        throw new Exception('Only "application/json" is accepted as header "Accept"');

    if ( array_key_exists('HTTP_ACCEPT_CHARSET', $_SERVER) && !preg_match('/^[uU][tT][fF]-?8$/', $_SERVER['HTTP_ACCEPT_CHARSET']) )
    {
        throw new Exception('Only "UTF-8" is accepted as header "Accept-Charset"');
    }

    // собственно запрос
    $vInput = file_get_contents('php://input');
    // я предпочёл разбирать словари как объекты, так проще отличить.
    // сериализованные словари от списков
    $vRequest = json_decode($vInput, FALSE);
    if ( json_last_error() != JSON_ERROR_NONE )
        throw new Exception('JSON decode error');
    if ( !is_object($vRequest) )
        throw new Exception('JSON data must be an object');

    // разбор запроса
    if ( !property_exists( $vRequest, 'PolisN') )
        throw new Exception('PolisN is a mandatory property');
    $vPolisN = $vRequest->PolisN;
    if ( !is_string($vPolisN) )
        throw new Exception('PolisN must be a string');

    $vPolisS = @$vRequest->PolisS;
    if ( !is_null($vPolisS) && !is_string($vPolisS) )
        throw new Exception('PolisS must be a string');

    $vSNILS = @$vRequest->SNILS;
    if ( !is_null($vSNILS) && !is_string($vSNILS) )
        throw new Exception('SNILS must be a string');

    if ( $vSNILS !== NULL )
    {
        if( !preg_match('/^\d{3}-\d{3}-\d{3} \d{2}$/',$vSNILS) && !preg_match('/^\d{11}$/', $vSNILS) )
            throw new Exception('Format of SNILS must be "XXX-XXX-XXX YY" or "XXXXXXXXXYY"');
        $vSNILS = unformatSNILS($vSNILS);
    }

    $vFirstName = @$vRequest->FirstName;
    if ( !is_null($vFirstName) && !is_string($vFirstName) )
        throw new Exception('FirstName must be a string');

    $vLastName = @$vRequest->LastName;
    if ( !is_null($vLastName) && !is_string($vLastName) )
        throw new Exception('LastName must be a string');

    $vMiddleName = @$vRequest->MiddleName;
    if ( !is_null($vMiddleName) && !is_string($vMiddleName) )
        throw new Exception('MiddleName must be a string');

    $vBirthDate = @$vRequest->BirthDate;
    if ( !is_null($vBirthDate) && !is_string($vBirthDate) )
        throw new Exception('BirthDate must be a string');

    if ( $vBirthDate !== NULL )
    {
        $vDate = NULL;
        if ( preg_match('/^\d{4}-\d{2}-\d{2}$/', $vBirthDate) )
        {
            $vDate = @DateTime::createFromFormat('Y-m-d', $vBirthDate);
        }
        else if( preg_match('/^\d{1,2}\.\d{1,2}\.\d{4}$/', $vBirthDate) )
        {
            $vDate = @DateTime::createFromFormat('d.m.Y', $vBirthDate);
        }
        if ( is_object($vDate) )
        {
            $vDateErr = DateTime::getLastErrors();
            if ( $vDateErr['warning_count'] || $vDateErr['error_count'] )
                $vDate = NULL;
        }
        if ( is_null($vDate) )
            throw new Exception('BirthDate does not have an appropriate value');
        $vBirthDate = $vDate->format('Y-m-d');
    }

    $vSex = @$vRequest->Sex;
    switch ( $vSex )
    {
        case NULL:
        case '':
                   $vSex = 0;
                   break;

        case 'm':
        case 'M':
                   $vSex = 1;
                   break;

        case 'f':
        case 'F':
                   $vSex = 2;
                   break;

        default:   throw new Exception('Sex must be either "F" or "M"');
    }


    $vClientIdList = findClients($vSex, $vBirthDate, $vLastName, $vFirstName, $vMiddleName, $vSNILS, $vPolisS, $vPolisN, 2);
    if ( $vClientIdList )
        return json_decode(gOrganisations);
    else
        return array();
}


try
{
    $vOrganisations = checkPatient();
    $vStatus = 0;
    $vMessage = NULL;
}
catch( Exception $e )
{
    $vOrganisations = NULL;
    $vStatus = 1;
    $vMessage = $e->getMessage();;
}


$vResp = array('Organisations' => $vOrganisations,
               'Status'        => $vStatus,
               'Message'       => $vMessage);

header('Content-Type: application/json');
echo json_encode($vResp);
exit(0);

?>


