#!/usr/bin/env python
# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
u"""Импорт населения"""

import gc
import sys
from optparse import OptionParser

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import (pyqtSignature, QDir, QVariant)


from preferences.connection import CConnectionDialog
from library             import database
from library.Utils       import getPref, getPrefString, setPref

from Progress              import CProgressDialog
from Users.Login          import CLoginDialog

from Ui_importRegistry import Ui_MainWindow
from appendix.regional.r60.logicalControl.logicalControl import (CMyApp,
    parseGCDebug, parseGCThreshold, gAbout, gTitle, gVersion)


class CMainWindow(QtGui.QMainWindow, Ui_MainWindow):

    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)

        self.loadPreferences()


    def loadPreferences(self):
        preferences = getPref(QtGui.qApp.preferences.windowPrefs, self.objectName(), {})
        geometry = getPref(preferences, 'geometry', None)
        if type(geometry) == QVariant and geometry.type() == QVariant.ByteArray and not geometry.isNull():
            self.restoreGeometry(geometry.toByteArray())
        state = getPref(preferences, 'state', None)
        if type(state) == QVariant and state.type() == QVariant.ByteArray and not geometry.isNull():
            self.restoreState(state.toByteArray())

        appPreferences = QtGui.qApp.preferences.appPrefs
        self.edtFileName.setText(
                getPrefString(appPreferences, 'fileName', ''))
        self.checkFileNames()


    def savePreferences(self):
        preferences = {}
        setPref(preferences,'geometry',QVariant(self.saveGeometry()))
        setPref(preferences,'state', QVariant(self.saveState()))
        setPref(QtGui.qApp.preferences.windowPrefs, self.objectName(),
                preferences)

        appPreferences = QtGui.qApp.preferences.appPrefs
        setPref(appPreferences, 'fileName',
            QVariant(self.edtFileName.text()))


    def setUserName(self, userName):
        title = u'Импорт населения для Республики Калмыкия'
        if userName:
            self.setWindowTitle(title + u': '+ userName)
        else:
            self.setWindowTitle(title)


    def updateStatus(self):
        pass


    @pyqtSignature('')
    def on_btnImport_clicked(self):
        process = CProgressDialog(self)
        QtGui.qApp.call(self, process.exec_)


    @pyqtSignature('')
    def on_btnSelectFileName_clicked(self):
        self.getOpenFileName(self.edtFileName, u'Файлы XML (*.xml)')


    def checkFileNames(self):
        self.btnImport.setEnabled(not self.edtFileName.text().isEmpty())


    def getOpenFileName(self, editor, mask):
        fileName = QtGui.QFileDialog.getOpenFileName(
            self, u'Укажите файл с данными', editor.text(), mask)
        if fileName != '':
            editor.setText(QDir.toNativeSeparators(fileName))
            self.checkFileNames()


    def enableControls(self, enable=True):
        self.checkFileNames()


    @pyqtSignature('QString')
    def on_edtFileName_textChanged(self, text):
        self.checkFileNames()


    @pyqtSignature('')
    def on_actLogin_triggered(self):
        try:
            QtGui.qApp.openDatabase()
            if QtGui.qApp.demoModePosible() and QtGui.qApp.demoModeRequested:
                QtGui.qApp.setUserId(None, True)
                self.setUserName(QtGui.qApp.userName())
                self.enableControls()
            else:
                dialog = CLoginDialog(self)
                dialog.setLoginName(QtGui.qApp.preferences.appUserName)
                if dialog.exec_():
                    QtGui.qApp.setUserId(dialog.userId(), dialog.demoMode())
                    QtGui.qApp.preferences.appUserName = dialog.loginName()
                    self.setUserName(QtGui.qApp.preferences.appUserName)
                    self.enableControls()
                else:
                    self.enableControls(False)
            return
        except database.CDatabaseException, e:
            QtGui.QMessageBox.critical(
                self,
                u'Ошибка открытия базы данных',
                unicode(e),
                QtGui.QMessageBox.Close)
        except Exception, e:
            QtGui.QMessageBox.critical(
                self,
                u'Ошибка открытия базы данных',
                u'Невозможно установить соедиенение с базой данных\n' + unicode(e),
                QtGui.QMessageBox.Close)
            self.logCurrentException()
        self.enableControls(False)
        QtGui.qApp.closeDatabase()


    @pyqtSignature('')
    def on_actLogout_triggered(self):
        QtGui.qApp.setUserId(None, False)
        QtGui.qApp.closeDatabase()
        self.setUserName('')
        self.enableControls(False)


    @pyqtSignature('')
    def on_actQuit_triggered(self):
        self.close()


    @QtCore.pyqtSignature('')
    def on_actConnection_triggered(self):
        dlg = CConnectionDialog(self)
        preferences = QtGui.qApp.preferences
        dlg.setDriverName(preferences.dbDriverName)
        dlg.setServerName(preferences.dbServerName)
        dlg.setServerPort(preferences.dbServerPort)
        dlg.setDatabaseName(preferences.dbDatabaseName)
        dlg.setUserName(preferences.dbUserName)
        dlg.setPassword(preferences.dbPassword)
        if dlg.exec_():
            preferences.dbDriverName = dlg.driverName()
            preferences.dbServerName = dlg.serverName()
            preferences.dbServerPort = dlg.serverPort()
            preferences.dbDatabaseName = dlg.databaseName()
            preferences.dbUserName = dlg.userName()
            preferences.dbPassword = dlg.password()
            preferences.save()


    @QtCore.pyqtSignature('')
    def on_actAbout_triggered(self):
        global lastChangedRev
        global lastChangedDate
        try:
            from buildInfo import lastChangedRev, lastChangedDate
        except:
            lastChangedRev  = 'unknown'
            lastChangedDate = 'unknown'

        QtGui.QMessageBox.about(
            self, u'О программе', gAbout % ( gTitle, gVersion, lastChangedRev,
                                            lastChangedDate))


    @QtCore.pyqtSignature('')
    def on_actAboutQt_triggered(self):
        QtGui.QMessageBox.aboutQt(self, u'О Qt')


if __name__ == '__main__':
    parser = OptionParser(usage = "usage: %prog [options]")
    parser.add_option('-d', '--demo',
                      dest='demo',
                      help='Login as demo',
                      action='store_true',
                      default=False)
    parser.add_option('--gc-debug',
                      dest='gcDebug',
                      help='gc debug flags, chars S(stat), C(collectable), U(uncollectable), I(instances), O(objects), for details see gc.set_debug',
                      metavar='[S][C][U][I][O]',
                      default=''
                      )
    parser.add_option('--gc-threshold',
                      dest='gcThreshold',
                      help='gc thresholds list, see gc.set_threshold ',
                      metavar='threshold0[,threshold1[,threshold2]]',
                      default=''
                     )
    parser.add_option('--version',
                      dest='version',
                      help='print version and exit',
                      action='store_true',
                      default=False)
    (options, args) = parser.parse_args()

    if options.version:
        print '%s v.0.1' % (sys.argv[0] if sys.argv else '')
    else:
        gcDebug = parseGCDebug(options.gcDebug)
        gcThreshold = parseGCThreshold(options.gcThreshold)
        if gcDebug:
            gc.set_debug(gcDebug)
            print 'debug set to ', gcDebug
        if gcThreshold:
            gc.set_threshold(*gcThreshold)
            print 'threshold set to ', gcThreshold

        app = CMyApp(sys.argv, options.demo)
        stdTtranslator = QtCore.QTranslator()
        stdTtranslator.load('i18n/std_ru.qm')
        app.installTranslator(stdTtranslator)

        QtGui.qApp = app
        app.applyDecorPreferences() # надеюсь, что это поможет немного сэкономить при создании гл.окна
        mainWindow = CMainWindow()
        app.mainWindow = mainWindow
        app.applyDecorPreferences() # применение максимизации/полноэкранного режима к главному окну

        if app.preferences.dbAutoLogin:
            mainWindow.actLogin.activate(QtGui.QAction.Trigger)

        mainWindow.show()
        res = app.exec_()
        mainWindow.savePreferences()
        app.preferences.save()
        app.doneTrace()
        QtGui.qApp = None
