#!/usr/bin/env python
# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
u"""ФЛК"""

import gc
import sys
from optparse import OptionParser

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import (pyqtSignature, QDir, QVariant)


from preferences.connection import CConnectionDialog
from library             import database
from library.Utils       import (getPref, getPrefString, getPrefBool,
    getPrefInt, setPref)

from Control              import CControlDialog
from Users.Login          import CLoginDialog

from Ui_logicalControl     import Ui_MainWindow
from appendix.regional.r60.logicalControl.logicalControl import (CMyApp,
    parseGCDebug, parseGCThreshold, gAbout, gTitle, gVersion)


class CMainWindow(QtGui.QMainWindow, Ui_MainWindow):

    maskCsv = u'Файлы CSV (*.csv)'

    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)

        self.loadPreferences()


    def loadPreferences(self):
        preferences = getPref(QtGui.qApp.preferences.windowPrefs, self.objectName(), {})
        geometry = getPref(preferences, 'geometry', None)
        if type(geometry) == QVariant and geometry.type() == QVariant.ByteArray and not geometry.isNull():
            self.restoreGeometry(geometry.toByteArray())
        state = getPref(preferences, 'state', None)
        if type(state) == QVariant and state.type() == QVariant.ByteArray and not geometry.isNull():
            self.restoreState(state.toByteArray())

        appPreferences = QtGui.qApp.preferences.appPrefs
        self.edtFileName.setText(
                getPrefString(appPreferences, 'fileName', ''))
        self.chkEventTariff.setChecked(
                getPrefBool(appPreferences, 'eventTariff', False))
        self.chkMedicalAidType.setChecked(
                getPrefBool(appPreferences, 'medicalAidType', False))
        self.chkRequiredTags.setChecked(
                getPrefBool(appPreferences, 'requiredTags', False))
        self.chkResults.setChecked(
                getPrefBool(appPreferences, 'results', False))
        self.chkVisitCount.setChecked(
                getPrefBool(appPreferences, 'visitCount', False))
        self.edtVisitCountRbFileName.setText(
                getPrefString(appPreferences, 'visitCountRbFileName', ''))
        self.edtMedicalAidTypeRbFileName.setText(
                getPrefString(appPreferences, 'medicalAidTypeRbFileName', ''))
        self.edtServiceSpecialityRbFileName.setText(
            getPrefString(appPreferences, 'serviceSpecialityRbFileName', ''))
        self.edtHealthExaminationResultRbFileName.setText(getPrefString(
            appPreferences, 'healthExaminationResultRbFileName', ''))
        self.chkHealthExaminationRequiredTags.setChecked(
            getPrefBool(appPreferences, 'healthExaminationRequiredTags', False))
        self.chkHealthExaminationServiceSpeciality.setChecked(getPrefBool(
            appPreferences, 'healthExaminationServiceSpeciality', False))
        self.chkHealthExaminationResult.setChecked(getPrefBool(appPreferences,
            'healthExaminationResult', False))
        self.tabControlType.setCurrentIndex(getPrefInt(
            appPreferences, 'controlTypeIndex', 0))
        self.chkMkbDots.setChecked(getPrefBool(
            appPreferences, 'mkbDots', False))
        self.chkMkbZ.setChecked(getPrefBool(appPreferences, 'mkbZ', False))
        self.chkMkbFilter.setChecked(getPrefBool(
            appPreferences, 'mkbFilter', False))
        self.edtMkbFilterRbFileName.setText(
            getPrefString(appPreferences, 'mkbFilterRbFileName', ''))
        self.chkHolidays.setChecked(getPrefBool(
            appPreferences, 'holidays', False))
        self.chkAmbulanceHospitalIntersect.setChecked(getPrefBool(
            appPreferences, 'ambulanceHospitalIntersect', False))
        self.chkHospitalIntersect.setChecked(getPrefBool(
            appPreferences, 'hospitalIntersect', False))
        self.chkInsurer.setChecked(getPrefBool(
            appPreferences, 'insurer', False))
        self.chkPeriod.setChecked(getPrefBool(appPreferences, 'period', False))
        self.chkFormat.setChecked(getPrefBool(appPreferences, 'format', False))
        self.chkVisit.setChecked(getPrefBool(appPreferences, 'visit', False))
        self.chkDifferentVisitType.setChecked(getPrefBool(appPreferences,
                                                          'diffVisitType',
                                                          False))
        self.chkTreatment.setChecked(getPrefBool(appPreferences, 'treatment',
                                                 False))
        self.chkMes.setChecked(getPrefBool(appPreferences, 'mes', False))
        self.chkVisitPeriod.setChecked(getPrefBool(appPreferences,
                                                   'visitPeriod', False))
        self.checkFileNames()


    def savePreferences(self):
        preferences = {}
        setPref(preferences,'geometry',QVariant(self.saveGeometry()))
        setPref(preferences,'state', QVariant(self.saveState()))
        setPref(QtGui.qApp.preferences.windowPrefs, self.objectName(),
                preferences)

        appPreferences = QtGui.qApp.preferences.appPrefs
        setPref(appPreferences, 'fileName',
            QVariant(self.edtFileName.text()))
        setPref(appPreferences, 'eventTariff',
            QVariant(self.chkEventTariff.isChecked()))
        setPref(appPreferences, 'medicalAidType',
            QVariant(self.chkMedicalAidType.isChecked()))
        setPref(appPreferences, 'requiredTags',
            QVariant(self.chkRequiredTags.isChecked()))
        setPref(appPreferences, 'results',
            QVariant(self.chkResults.isChecked()))
        setPref(appPreferences, 'visitCount',
            QVariant(self.chkVisitCount.isChecked()))
        setPref(appPreferences, 'visitCountRbFileName',
            QVariant(self.edtVisitCountRbFileName.text()))
        setPref(appPreferences, 'medicalAidTypeRbFileName',
            QVariant(self.edtMedicalAidTypeRbFileName.text()))
        setPref(appPreferences, 'serviceSpecialityRbFileName',
            QVariant(self.edtServiceSpecialityRbFileName.text()))
        setPref(appPreferences, 'healthExaminationResultRbFileName',
            QVariant(self.edtHealthExaminationResultRbFileName.text()))
        setPref(appPreferences, 'healthExaminationRequiredTags',
            QVariant(self.chkHealthExaminationRequiredTags.isChecked()))
        setPref(appPreferences, 'healthExaminationServiceSpeciality',
            QVariant(self.chkHealthExaminationServiceSpeciality.isChecked()))
        setPref(appPreferences, 'healthExaminationResult',
            QVariant(self.chkHealthExaminationResult.isChecked()))
        setPref(appPreferences, 'controlTypeIndex',
            QVariant(self.tabControlType.currentIndex()))
        setPref(appPreferences, 'mkbDots',
            QVariant(self.chkMkbDots.isChecked()))
        setPref(appPreferences, 'mkbZ', QVariant(self.chkMkbZ.isChecked()))
        setPref(appPreferences, 'mkbFilter',
            QVariant(self.chkMkbFilter.isChecked()))
        setPref(appPreferences, 'mkbFilterRbFileName',
            QVariant(self.edtMkbFilterRbFileName.text()))
        setPref(appPreferences, 'holidays',
                QVariant(self.chkHolidays.isChecked()))
        setPref(appPreferences, 'ambulanceHospitalIntersect',
                QVariant(self.chkAmbulanceHospitalIntersect.isChecked()))
        setPref(appPreferences, 'hospitalIntersect',
                QVariant(self.chkHospitalIntersect.isChecked()))
        setPref(appPreferences, 'insurer',
                QVariant(self.chkInsurer.isChecked()))
        setPref(appPreferences, 'period', QVariant(self.chkPeriod.isChecked()))
        setPref(appPreferences, 'format', QVariant(self.chkFormat.isChecked()))
        setPref(appPreferences, 'visit', QVariant(self.chkVisit.isChecked()))
        setPref(appPreferences, 'treatment',
                QVariant(self.chkTreatment.isChecked()))
        setPref(appPreferences, 'diffVisitType',
                QVariant(self.chkDifferentVisitType.isChecked()))
        setPref(appPreferences, 'mes', QVariant(self.chkMes.isChecked()))
        setPref(appPreferences, 'visitPeriod',
                QVariant(self.chkVisitPeriod.isChecked()))


    def setUserName(self, userName):
        title = u'Форматно-логический контроль для Республики Калмыкия'
        if userName:
            self.setWindowTitle(title + u': '+ userName)
        else:
            self.setWindowTitle(title)


    def updateStatus(self):
        pass


    @pyqtSignature('')
    def on_btnControl_clicked(self):
        process = CControlDialog(self)
        process.checkRequiredTags = self.chkRequiredTags.isChecked()
        process.checkEventTariff = self.chkEventTariff.isChecked()
        process.checkMedicalAidType = (self.chkMedicalAidType.isChecked()
            and self.chkMedicalAidType.isEnabled())
        process.checkResults = self.chkResults.isChecked()
        process.checkVisitCount = (self.chkVisitCount.isChecked()
            and self.chkVisitCount.isEnabled())
        process.checkHealthExaminationRequiredTags = (
            self.chkHealthExaminationRequiredTags.isChecked())
        process.checkHealthExaminationServiceSpeciality = (
            self.chkHealthExaminationServiceSpeciality.isChecked())
        process.checkHealthExaminationResult = (
            self.chkHealthExaminationResult.isChecked())
        process.checkMkbDots = self.chkMkbDots.isChecked()
        process.checkMkbZ = self.chkMkbZ.isChecked()
        process.checkMkbByFilter = self.chkMkbFilter.isChecked()
        process.checkHolidays = self.chkHolidays.isChecked()
        process.checkAmbulanceHospitalIntersect = (
            self.chkAmbulanceHospitalIntersect.isChecked())
        process.checkHospitalIntersect = (
            self.chkHospitalIntersect.isChecked())
        process.checkInsurer = self.chkInsurer.isChecked()
        process.checkPeriod = self.chkPeriod.isChecked()
        process.checkFormat = self.chkFormat.isChecked()
        process.checkVisit = self.chkVisit.isChecked() and self.chkVisit.isEnabled()
        process.checkTreatment = self.chkTreatment.isChecked() and self.chkTreatment.isEnabled()
        process.checkDiffVisitType = (self.chkDifferentVisitType.isChecked()
                                    and self.chkDifferentVisitType.isEnabled())
        process.checkMes = self.chkMes.isChecked() and self.chkMes.isEnabled()
        process.checkVisitPeriod = (self.chkVisitPeriod.isChecked()
                                    and self.chkVisitPeriod.isEnabled())
        mode = (process.modeNormal if self.tabControlType.currentIndex() == 0
            else process.modeHealthExamination)
        process.setCheckMode(mode)
        QtGui.qApp.call(self, process.exec_)


    def checkFileNames(self):
        self.btnControl.setEnabled(
            ((self.tabControlType.currentIndex() == 0 and
                (self.chkEventTariff.isChecked() or
                self.chkMedicalAidType.isChecked() or
                self.chkRequiredTags.isChecked() or
                self.chkResults.isChecked() or
                self.chkVisitCount.isChecked() or
                self.chkMkbDots.isChecked() or
                self.chkMkbZ.isChecked() or
                self.chkMkbFilter.isChecked()) or
                self.chkHolidays.isChecked() or
                self.chkAmbulanceHospitalIntersect.isChecked() or
                self.chkHospitalIntersect.isChecked() or
                self.chkInsurer.isChecked() or
                self.chkPeriod.isChecked() or
                self.chkFormat.isChecked() or
                (self.chkVisit.isChecked() and self.chkVisit.isEnabled()) or
                (self.chkTreatment.isChecked() and self.chkTreatment.isEnabled()) or
                (self.chkDifferentVisitType.isChecked() and self.chkDifferentVisitType.isEnabled()) or
                (self.chkMes.isChecked() and self.chkMes.isEnabled()) or
                (self.chkVisitPeriod.isChecked() and self.chkVisitPeriod.isEnabled())
                )
            or (self.tabControlType.currentIndex() == 1 and
                (self.chkHealthExaminationRequiredTags.isChecked() or
                self.chkHealthExaminationResult.isChecked() or
                self.chkHealthExaminationServiceSpeciality.isChecked())))
            and not self.edtFileName.text().isEmpty())

        self.chkMedicalAidType.setEnabled(
            not self.edtMedicalAidTypeRbFileName.text().isEmpty())
        self.chkVisitCount.setEnabled(
            not self.edtVisitCountRbFileName.text().isEmpty())
        self.chkHealthExaminationResult.setEnabled(
            not self.edtHealthExaminationResultRbFileName.text().isEmpty())
        self.chkHealthExaminationServiceSpeciality.setEnabled(
            not self.edtServiceSpecialityRbFileName.text().isEmpty())
        self.chkMkbFilter.setEnabled(
            not self.edtMkbFilterRbFileName.text().isEmpty())


    def getOpenFileName(self, editor, mask):
        fileName = QtGui.QFileDialog.getOpenFileName(
            self, u'Укажите файл с данными', editor.text(), mask)
        if fileName != '':
            editor.setText(QDir.toNativeSeparators(fileName))
            self.checkFileNames()


    def enableControls(self, enable=True):
        self.chkDifferentVisitType.setEnabled(enable)
        self.chkVisit.setEnabled(enable)
        self.chkTreatment.setEnabled(enable)
        self.chkMes.setEnabled(enable)
        self.chkVisitPeriod.setEnabled(enable)
        self.actLogin.setEnabled(not enable)
        self.actLogout.setEnabled(enable)
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkEventTariff_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkMedicalAidType_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkRequiredTags_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkResults_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkVisitCount_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkMkbDots_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkMkbZ_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkHealthExaminationRequiredTags_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkHealthExaminationResult_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkHealthExaminationServiceSpeciality_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkMkbFilter_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkHolidays_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkAmbulanceHospitalIntersect_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkHospitalIntersect_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkInsurer_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkPeriod_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkFormat_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkVisit_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkTreatment_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkDifferentVisitType_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkVisitPeriod_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('bool')
    def on_chkMes_toggled(self, checked):
        self.checkFileNames()


    @pyqtSignature('QString')
    def on_edtFileName_textChanged(self, text):
        self.checkFileNames()


    @pyqtSignature('QString')
    def on_edtVisitCountRbFileName_textChanged(self, text):
        self.checkFileNames()


    @pyqtSignature('QString')
    def on_edtMedicalAidTypeRbFileName_textChanged(self, text):
        self.checkFileNames()


    @pyqtSignature('QString')
    def on_edtHealthExaminationResultRbFileName_textChanged(self, text):
        self.checkFileNames()


    @pyqtSignature('QString')
    def on_edtServiceSpecialityRbFileName_textChanged(self, text):
        self.checkFileNames()


    @pyqtSignature('QString')
    def on_edtMkbFilterRbFileName_textChanged(self, text):
        self.checkFileNames()


    @pyqtSignature('')
    def on_btnSelectFileName_clicked(self):
        self.getOpenFileName(self.edtFileName, u'Файлы XML (*.xml)')


    @pyqtSignature('')
    def on_btnSelectVisitCountRbFileName_clicked(self):
        self.getOpenFileName(self.edtVisitCountRbFileName, self.maskCsv)


    @pyqtSignature('')
    def on_btnSelectMedicalAidTypeRbFileName_clicked(self):
        self.getOpenFileName(self.edtMedicalAidTypeRbFileName, self.maskCsv)


    @pyqtSignature('')
    def on_btnSelectServiceSpecialityRbFileName_clicked(self):
        self.getOpenFileName(self.edtServiceSpecialityRbFileName, self.maskCsv)


    @pyqtSignature('')
    def on_btnSelectHealthExaminationResultRbFileName_clicked(self):
        self.getOpenFileName(self.edtHealthExaminationResultRbFileName,
            self.maskCsv)


    @pyqtSignature('')
    def on_btnSelectMkbFilterRbFileName_clicked(self):
        self.getOpenFileName(self.edtMkbFilterRbFileName, self.maskCsv)


    @pyqtSignature('int')
    def on_tabControlType_currentChanged(self, index):
        self.checkFileNames()

    @pyqtSignature('')
    def on_actLogin_triggered(self):
        try:
            QtGui.qApp.openDatabase()
            if QtGui.qApp.demoModePosible() and QtGui.qApp.demoModeRequested:
                QtGui.qApp.setUserId(None, True)
                self.setUserName(QtGui.qApp.userName())
                self.enableControls()
            else:
                dialog = CLoginDialog(self)
                dialog.setLoginName(QtGui.qApp.preferences.appUserName)
                if dialog.exec_():
                    QtGui.qApp.setUserId(dialog.userId(), dialog.demoMode())
                    QtGui.qApp.preferences.appUserName = dialog.loginName()
                    self.setUserName(QtGui.qApp.preferences.appUserName)
                    self.enableControls()
                else:
                    self.enableControls(False)
            return
        except database.CDatabaseException, e:
            QtGui.QMessageBox.critical(
                self,
                u'Ошибка открытия базы данных',
                unicode(e),
                QtGui.QMessageBox.Close)
        except Exception, e:
            QtGui.QMessageBox.critical(
                self,
                u'Ошибка открытия базы данных',
                u'Невозможно установить соедиенение с базой данных\n' + unicode(e),
                QtGui.QMessageBox.Close)
            self.logCurrentException()
        self.enableControls(False)
        QtGui.qApp.closeDatabase()


    @pyqtSignature('')
    def on_actLogout_triggered(self):
        QtGui.qApp.setUserId(None, False)
        QtGui.qApp.closeDatabase()
        self.setUserName('')
        self.enableControls(False)


    @pyqtSignature('')
    def on_actQuit_triggered(self):
        self.close()


    @QtCore.pyqtSignature('')
    def on_actConnection_triggered(self):
        dlg = CConnectionDialog(self)
        preferences = QtGui.qApp.preferences
        dlg.setDriverName(preferences.dbDriverName)
        dlg.setServerName(preferences.dbServerName)
        dlg.setServerPort(preferences.dbServerPort)
        dlg.setDatabaseName(preferences.dbDatabaseName)
        dlg.setUserName(preferences.dbUserName)
        dlg.setPassword(preferences.dbPassword)
        if dlg.exec_():
            preferences.dbDriverName = dlg.driverName()
            preferences.dbServerName = dlg.serverName()
            preferences.dbServerPort = dlg.serverPort()
            preferences.dbDatabaseName = dlg.databaseName()
            preferences.dbUserName = dlg.userName()
            preferences.dbPassword = dlg.password()
            preferences.save()


    @QtCore.pyqtSignature('')
    def on_actAbout_triggered(self):
        global lastChangedRev
        global lastChangedDate
        try:
            from buildInfo import lastChangedRev, lastChangedDate
        except:
            lastChangedRev  = 'unknown'
            lastChangedDate = 'unknown'

        QtGui.QMessageBox.about(
            self, u'О программе', gAbout % ( gTitle, gVersion, lastChangedRev,
                                            lastChangedDate))


    @QtCore.pyqtSignature('')
    def on_actAboutQt_triggered(self):
        QtGui.QMessageBox.aboutQt(self, u'О Qt')


if __name__ == '__main__':
    parser = OptionParser(usage = "usage: %prog [options]")
    parser.add_option('-d', '--demo',
                      dest='demo',
                      help='Login as demo',
                      action='store_true',
                      default=False)
    parser.add_option('--gc-debug',
                      dest='gcDebug',
                      help='gc debug flags, chars S(stat), C(collectable), U(uncollectable), I(instances), O(objects), for details see gc.set_debug',
                      metavar='[S][C][U][I][O]',
                      default=''
                      )
    parser.add_option('--gc-threshold',
                      dest='gcThreshold',
                      help='gc thresholds list, see gc.set_threshold ',
                      metavar='threshold0[,threshold1[,threshold2]]',
                      default=''
                     )
    parser.add_option('--version',
                      dest='version',
                      help='print version and exit',
                      action='store_true',
                      default=False)
    (options, args) = parser.parse_args()

    if options.version:
        print '%s v.0.1' % (sys.argv[0] if sys.argv else '')
    else:
        gcDebug = parseGCDebug(options.gcDebug)
        gcThreshold = parseGCThreshold(options.gcThreshold)
        if gcDebug:
            gc.set_debug(gcDebug)
            print 'debug set to ', gcDebug
        if gcThreshold:
            gc.set_threshold(*gcThreshold)
            print 'threshold set to ', gcThreshold

        app = CMyApp(sys.argv, options.demo)
        stdTtranslator = QtCore.QTranslator()
        stdTtranslator.load('i18n/std_ru.qm')
        app.installTranslator(stdTtranslator)

        QtGui.qApp = app
        app.applyDecorPreferences() # надеюсь, что это поможет немного сэкономить при создании гл.окна
        mainWindow = CMainWindow()
        app.mainWindow = mainWindow
        app.applyDecorPreferences() # применение максимизации/полноэкранного режима к главному окну

        if app.preferences.dbAutoLogin:
            mainWindow.actLogin.activate(QtGui.QAction.Trigger)

        mainWindow.show()
        res = app.exec_()
        mainWindow.savePreferences()
        app.preferences.save()
        app.doneTrace()
        QtGui.qApp = None
