#!/usr/bin/env python
# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################


import codecs
import gc
import locale
import os
import os.path
import shutil
import sys
import tempfile
import traceback
import StringIO
from optparse import OptionParser

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import *
from PyQt4.QtGui  import *

#import library.patches
from Users.Login         import CLoginDialog
from Users.RightList     import CUserRightListDialog,  CUserRightProfileListDialog
from Users.UserInfo      import CUserInfo, CDemoUserInfo
from Users.Rights        import *
from Users.Tables        import *
from library             import database
from library.Preferences import CPreferences
from library.DialogBase  import CDialogBase
from library.Utils       import *

from preferences.connection import CConnectionDialog
from preferences.decor import CDecorDialog

from Progress              import CProgressDialog
from Ui_importRegistry     import Ui_MainWindow


class CMyApp(QtGui.QApplication):
    __pyqtSignals__ = ('dbConnectionChanged(bool)',
                       'currentUserIdChanged()',
                      )

    def __init__(self, args, demoModeRequest):
        QtGui.QApplication.__init__(self, args)
        self.logDir = os.path.join(unicode(QDir.toNativeSeparators(QDir.homePath())), '.samson-vista')
        self.oldMsgHandler = qInstallMsgHandler(self.msgHandler)
        self.oldEexceptHook = sys.excepthook
        sys.excepthook = self.logException
        self.traceActive = True
        self.homeDir= None
        self.saveDir= None
        self.userId = None
        self.userSpecialityId = None
        self.userOrgStructureId = None
        self.db     = None
        self.mainWindow = None
        self.preferences = CPreferences('r23ImportReestr.ini')
        self.preferences.load()
        self.preferencesDir = self.preferences.getDir()
        self.registerDocumentTables()
        self._currentClientId = None
        self.userInfo = None
        self.demoModeRequested = True


    def doneTrace(self):
        self.traceActive = False
        qInstallMsgHandler(self.oldMsgHandler)
        sys.excepthook = self.oldEexceptHook


    def applyDecorPreferences(self):
        self.setStyle(self.preferences.decorStyle)
        if self.preferences.decorStandardPalette:
            self.setPalette(self.style().standardPalette())
        if self.mainWindow:
            state = Qt.WindowNoState
            if self.preferences.decorMaximizeMainWindow:
                state |= Qt.WindowMaximized
            if self.preferences.decorFullScreenMainWindow:
                state |= Qt.WindowFullScreen
            self.mainWindow.setWindowState(state)


    def registerDocumentTables(self):
        database.registerDocumentTable('Account')
        database.registerDocumentTable('Action')
        database.registerDocumentTable('ActionProperty')
        database.registerDocumentTable('ActionPropertyTemplate')
        database.registerDocumentTable('ActionTemplate')
        database.registerDocumentTable('ActionType')
        database.registerDocumentTable('Address')
#        database.registerDocumentTable('AddressAreaItem')
        database.registerDocumentTable('AddressHouse')
        database.registerDocumentTable('Bank')
        database.registerDocumentTable('CalendarExceptions')
        database.registerDocumentTable('Client')
        database.registerDocumentTable('ClientAddress')
        database.registerDocumentTable('ClientAllergy')
        database.registerDocumentTable('ClientAttach')
        database.registerDocumentTable('ClientContact')
        database.registerDocumentTable('ClientIdentification')
        database.registerDocumentTable('ClientIntoleranceMedicament')
        database.registerDocumentTable('ClientDocument')
        database.registerDocumentTable('ClientPolicy')
        database.registerDocumentTable('ClientSocStatus')
        database.registerDocumentTable('ClientWork')
        database.registerDocumentTable('Contract')
        database.registerDocumentTable('Diagnosis')
        database.registerDocumentTable('Diagnostic')
        database.registerDocumentTable('Event')
        database.registerDocumentTable('EventType')
        database.registerDocumentTable('InformerMessage')
        database.registerDocumentTable('Licence')
        database.registerDocumentTable('Organisation')
        database.registerDocumentTable('OrgStructure')
        database.registerDocumentTable('Person')
        database.registerDocumentTable('TempInvalid')
        database.registerDocumentTable('Visit')


    def getHomeDir(self):
        if not self.homeDir:
            homeDir = os.path.expanduser('~')
            if homeDir == '~':
                homeDir = unicode(QDir.homePath())
            if isinstance(homeDir, str):
                homeDir = unicode(homeDir, locale.getpreferredencoding())
            self.homeDir = homeDir
        return self.homeDir


    def getSaveDir(self):
        self.saveDir = forceString(self.preferences.appPrefs.get('saveDir', QVariant()))
        if not self.saveDir:
            self.saveDir = self.getHomeDir()
        return self.saveDir


    def setSaveDir(self, path):
#        mode = os.stat(path)[ST_MODE]
#        if if S_ISDIR(mode):
        saveDir = os.path.dirname(unicode(path))
        if saveDir and self.saveDir != saveDir:
            self.preferences.appPrefs['saveDir'] = QVariant(saveDir)


    def getTemplateDir(self):
        result = forceString(self.preferences.appPrefs.get('templateDir', None))
        if not result:
            result = os.path.join(self.logDir, 'templates')
        return result


    def getTmpDir(self, suffix=''):
        return unicode(tempfile.mkdtemp('','samson_%s_'%suffix), locale.getpreferredencoding())


    def removeTmpDir(self, dir):
        try:
            shutil.rmtree(dir, False)
        except:
            self.logCurrentException()


#    def setTemplateDir(self, path):
#        self.preferences.appPrefs.setdefault('templateDir', QVariant(path))


    def log(self, title, message, stack=None):
        try:
            if not os.path.exists(self.logDir):
                os.makedirs(self.logDir)
            logFile = os.path.join(self.logDir, 'error.log')
            timeString = unicode(QDateTime.currentDateTime().toString(Qt.SystemLocaleDate))
            logString = u'%s\n%s: %s(%s)\n' % ('='*72, timeString, title, message)
            if stack:
                try:
                    logString += ''.join(traceback.format_list(stack)).decode('utf-8') + '\n'
                except:
                    logString += 'stack lost\n'
            file = codecs.open(logFile, mode='a', encoding=locale.getpreferredencoding())
            file.write(logString)
            file.close()
        except:
            pass


    def logException(self, exceptionType, exceptionValue, exceptionTraceback):
        title = repr(exceptionType)
        message = unicode(exceptionValue)
        self.log(title, message, traceback.extract_tb(exceptionTraceback))
        sys.__excepthook__(exceptionType, exceptionValue, exceptionTraceback)


    def logCurrentException(self):
        self.logException(*sys.exc_info())


    def msgHandler(self, type, msg):
        if type == 0: # QtMsgType.QtDebugMsg:
            typeName = 'QtDebugMsg'
        elif type == 1: # QtMsgType.QtWarningMsg:
            typeName = 'QtWarningMsg'
        elif type == 2: # QtMsgType.QtCriticalMsg:
            typeName = 'QtCriticalMsg'
        elif type == 3: # QtFatalMsg
            typeName = 'QtFatalMsg'
        else:
            typeName = 'QtUnknownMsg'

        self.log( typeName, msg, traceback.extract_stack()[:-1])
#        print typeName, msg


    def openDatabase(self):
        self.db = None
        self.db = database.connectDataBase(self.preferences.dbDriverName,
                                     self.preferences.dbServerName,
                                     self.preferences.dbServerPort,
                                     self.preferences.dbDatabaseName,
                                     self.preferences.dbUserName,
                                     self.preferences.dbPassword)
        self.emit(QtCore.SIGNAL('dbConnectionChanged(bool)'), True)


    def closeDatabase(self):
        if self.db:
            self.db.close()
            self.db = None
            self.emit(QtCore.SIGNAL('dbConnectionChanged(bool)'), False)


    def notify(self, receiver, event):
        try:
#            report = ["notify:: receiver.type=",unicode(type(receiver))," receiver.name=", unicode(receiver.objectName())," event.type=",unicode(type(event)), "event.type()=", unicode(event.type())]
#            print >>sys.stderr, ("".join(report)).encode('cp866')
            return QtGui.QApplication.notify(self, receiver, event)
        except Exception, e:
            if self.traceActive:
                self.logCurrentException()
                widget = self.activeModalWidget()
                if widget is None:
                    widget = self.mainWindow
                QtGui.QMessageBox.critical( widget,
                                            u'Произошла ошибка',
                                            unicode(e),
                                            QtGui.QMessageBox.Close)
            return False
        except:
            return False


    def call(self, widget, func, params = ()):
        try:
            return True, func(*params)
        except IOError, e:
            msg=''
            if hasattr(e, 'filename'):
                msg = u'%s:\n[Errno %s] %s' % (e.filename, e.errno, anyToUnicode(e.strerror))
            else:
                msg = u'[Errno %s] %s' % (e.errno, anyToUnicode(e.strerror))
            QtGui.QMessageBox.critical(widget,
                u'Произошла ошибка ввода-вывода', msg, QtGui.QMessageBox.Close)
        except Exception, e:
            self.logCurrentException()
            widget = widget or self.activeModalWidget() or self.mainWindow
            QtGui.QMessageBox.critical( widget,
                                        u'Произошла ошибка',
                                        unicode(e),
                                        QtGui.QMessageBox.Close)
        return False, None


    def callWithWaitCursor(self, widget, func, *params, **kwparams):
        self.setOverrideCursor(QtGui.QCursor(Qt.WaitCursor))
        try:
            return func(*params, **kwparams)
        finally:
            self.restoreOverrideCursor()


    def demoModePosible(self):
        try:
            tableUser = self.db.table(tblUser)
            record = self.db.getRecordEx(tableUser, [usrPassword, usrRetired], tableUser[usrLogin].eq(demoUserName))
            return not record or (not forceString(record.value(usrPassword)) and not forceBool(record.value(usrRetired)))
        except:
            pass
        return False


    def setUserId(self, userId, demoMode = False):
        self.userId = userId
        record = self.db.getRecord('Person', ['speciality_id', 'orgStructure_id'], userId) if userId else None
        if record:
            self.userSpecialityId = forceRef(record.value('speciality_id'))
            self.userOrgStructureId = forceRef(record.value('orgStructure_id'))
        else:
            self.userSpecialityId = None
            self.userOrgStructureId = None
        self.demoMode = demoMode
        if demoMode:
            self.userInfo = CDemoUserInfo(userId)
        else:
            if userId:
                self.userInfo = CUserInfo(userId)
            else:
                self.userInfo = None
        self.emit(QtCore.SIGNAL('currentUserIdChanged()'))


    def userName(self):
        if self.userInfo:
#            orgId = self.currentOrgId()
#            orgInfo = getOrganisationInfo(orgId)
#            if not orgInfo:
#                QtGui.qApp.preferences.appPrefs['orgId'] = QVariant()
#            shortName = orgInfo.get('shortName', u'ЛПУ в настройках не указано!')
            return u'%s (%s)' % (self.userInfo.name(), self.userInfo.login())
        else:
            return ''


    def userHasRight(self, right):
        return self.userInfo is not None and self.userInfo.hasRight(right)


    def userHasAnyRight(self, rights):
        return self.userInfo is not None and self.userInfo.hasAnyRight(rights)



    def defaultKLADR(self):
        # возвращает код по умолчанию
        result = forceString(self.preferences.appPrefs.get('defaultKLADR', ''))
        if not result:
            result = '2300000000000'
        return result



# #############################################################


class CMainWindow(QtGui.QMainWindow, Ui_MainWindow):
    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)
#        self.registryFileName = 'reestr.dbf'

#        self.accountingSystemId  = None
#        self.policyTypeId        = None
#        self.childDocumentTypeId = None
#        self.adultDocumentTypeId = None
#        self.contactTypeId       = None

        self.loadPreferences()


    def loadPreferences(self):
        preferences = getPref(QtGui.qApp.preferences.windowPrefs, self.objectName(), {})
        geometry = getPref(preferences, 'geometry', None)
        if type(geometry) == QVariant and geometry.type() == QVariant.ByteArray and not geometry.isNull():
            self.restoreGeometry(geometry.toByteArray())
        state = getPref(preferences, 'state', None)
        if type(state) == QVariant and state.type() == QVariant.ByteArray and not geometry.isNull():
            self.restoreState(state.toByteArray())

        appPreferences = QtGui.qApp.preferences.appPrefs
        self.edtRBDirName.setText(getPrefString(appPreferences, 'rbDirName', ''))
        self.edtMainDataFileName.setText(getPrefString(appPreferences, 'mainDataFileName', ''))
        self.edtServiceFileName.setText(getPrefString(appPreferences, 'serviceFileName', ''))
        self.chkCopyToActionType.setChecked(getPrefBool(appPreferences, 'copyToActionType', False))
        self.chkImportClients.setChecked(getPrefBool(appPreferences, 'importClients', True))
        self.edtMainDataFileName.setEnabled(self.chkImportClients.isChecked())
        self.checkName()
        self.checkServiceName()


    def savePreferences(self):
        preferences = {}
        setPref(preferences,'geometry',QVariant(self.saveGeometry()))
        setPref(preferences,'state', QVariant(self.saveState()))
        setPref(QtGui.qApp.preferences.windowPrefs, self.objectName(), preferences)

        appPreferences = QtGui.qApp.preferences.appPrefs
        setPref(appPreferences, 'rbDirName', QVariant(self.edtRBDirName.text()))
        setPref(appPreferences, 'mainDataFileName', QVariant(self.edtMainDataFileName.text()))
        setPref(appPreferences, 'serviceFileName', QVariant(self.edtServiceFileName.text()))
        setPref(appPreferences, 'copyToActionType', QVariant(self.chkCopyToActionType.isChecked()))
        setPref(appPreferences, 'importClients', QVariant(self.chkImportClients.isChecked()))
  #      setPref(appPreferences, 'fillSpecialityRegionalCode', QVariant(self.chkFillSpecialityRegionalCode.isChecked()))


    def setUserName(self, userName):
        title = u'Импорт реестра застрахованного населения для Краснодарского края'
        if userName:
            self.setWindowTitle(title + u': '+ userName)
        else:
            self.setWindowTitle(title)



    def updateStatus(self):
#        self.btnImport.setEnabled(self.registryOk and self.orgsOk and self.insurersOk)
        pass


    def checkName(self):
        self.btnImport.setEnabled(
            self.edtRBDirName.text()!='' and
            (self.edtMainDataFileName.text()!='' or not self.chkImportClients.isChecked()))


    def checkServiceName(self):
        self.btnImportService.setEnabled(self.edtServiceFileName.text() != '')


    def getServiceStandartFileName(self):
        return unicode(self.edtRBDirName.text()) + '/SPR19.DBF'


    def getMKBStandartFileName(self):
        return unicode(self.edtRBDirName.text()) + '/SPR38.DBF'


    def getMedicamentFileName(self):
        return unicode(self.edtRBDirName.text()) + '/SPR53.DBF'


    def getMedicamentStandartFileName(self):
        return unicode(self.edtRBDirName.text()) + '/SPR54.DBF'



    @QtCore.pyqtSignature('int')
    def on_chkImportClients_stateChanged(self, state):
        self.checkName()


    @pyqtSignature('QString')
    def on_edtRBDirName_textChanged(self, text):
        self.checkName()


    @pyqtSignature('QString')
    def on_edtMainDataFileName_textChanged(self, text):
        self.checkName()


    @pyqtSignature('QString')
    def on_edtServiceFileName_textChanged(self, text):
        self.checkServiceName()


    @pyqtSignature('')
    def on_btnSelectRBDir_clicked(self):
        dirName = forceString(QtGui.QFileDialog.getExistingDirectory(
            self, u'Укажите каталог', self.edtRBDirName.text()))
        if dirName != '':
            self.edtRBDirName.setText(QDir.toNativeSeparators(dirName))
            self.checkName()


    @pyqtSignature('')
    def on_btnSelectServiceFile_clicked(self):
        fileName = QtGui.QFileDialog.getOpenFileName(
            self, u'Укажите файл с данными', self.edtServiceFileName.text(), u'Файлы DBF (*.dbf)')
        if fileName != '':
            self.edtServiceFileName.setText(QDir.toNativeSeparators(fileName))
            self.checkServiceName()



    @pyqtSignature('')
    def on_btnSelectMainDataFile_clicked(self):
        fileName = QtGui.QFileDialog.getOpenFileName(
            self, u'Укажите файл с данными', self.edtMainDataFileName.text(), u'Файлы DBF (*.dbf)')
        if fileName != '':
            self.edtMainDataFileName.setText(QDir.toNativeSeparators(fileName))
            self.checkName()


    @pyqtSignature('')
    def on_btnImportService_clicked(self):
        def checkConstrain(cond, message):
            if not cond:
                QtGui.QMessageBox.critical(
                    self,
                    u'Импорт невозможен',
                    message,
                    QtGui.QMessageBox.Close)
            return cond

        process = CProgressDialog(self)

        parentGroup = None
        if self.chkCopyToActionType.isChecked():
            db = QtGui.qApp.db
            record = db.getRecordEx('ActionType', 'id', 'code="r23" AND class="3"', 'id')
            if record:
                parentGroup = forceRef(record.value(0))

            if not (checkConstrain(parentGroup,
                u'В справочнике типов действий (ActionType)'
                u' отсутствует запись с кодом r23, класс "прочее"'
                u' (Группа для мед. услуг Краснодарского края)')):
                return

        process.setOptions(parentGroup, self.chkCopyToMes.isChecked(), self.chkRecognizeNomenclature.isChecked())

        QtGui.qApp.call(self, process.exec_, (True, ))


    @pyqtSignature('')
    def on_btnImport_clicked(self):
        def checkConstrain(cond, message):
            if not cond:
                QtGui.QMessageBox.critical(
                    self,
                    u'Импорт невозможен',
                    message,
                    QtGui.QMessageBox.Close)
            return cond


        process = CProgressDialog(self)
        db = QtGui.qApp.db
        process.policyTypeId = forceRef(db.translate('rbPolicyType', 'code', '1', 'id'))
        process.importClients = self.chkImportClients.isChecked()
        process.matureNetId = forceRef(db.translate('rbNet', 'code', '1', 'id'))

        if not (checkConstrain(process.policyTypeId, u'В справочнике типов полиса (rbPolicyType) отсутствует запись с кодом 1') or
            checkConstrain(process.matureNetId, u'В справочнике Сеть (rbNet) отсутствует запись с кодом 1')):
            return

        QtGui.qApp.call(self, process.exec_)


    @pyqtSignature('')
    def on_actLogin_triggered(self):
        try:
            QtGui.qApp.openDatabase()
            if QtGui.qApp.demoModePosible() and QtGui.qApp.demoModeRequested:
                QtGui.qApp.setUserId(None, True)
                self.setUserName(QtGui.qApp.userName())
#                self.updateActionsState()
            else:
                dialog = CLoginDialog(self)
                dialog.setLoginName(QtGui.qApp.preferences.appUserName)
                if dialog.exec_():
                    QtGui.qApp.setUserId(dialog.userId(), dialog.demoMode())
                    QtGui.qApp.preferences.appUserName = dialog.loginName()
                    self.setUserName(QtGui.qApp.preferences.appUserName)
#                    self.setUserName(QtGui.qApp.userName())
#                    self.updateActionsState()
            return
        except database.CDatabaseException, e:
            QtGui.QMessageBox.critical(
                self,
                u'Ошибка открытия базы данных',
                unicode(e),
                QtGui.QMessageBox.Close)
        except Exception, e:
            QtGui.QMessageBox.critical(
                self,
                u'Ошибка открытия базы данных',
                u'Невозможно установить соедиенение с базой данных\n' + unicode(e),
                QtGui.QMessageBox.Close)
            self.logCurrentException()
        QtGui.qApp.closeDatabase()


    @pyqtSignature('')
    def on_actLogout_triggered(self):
        QtGui.qApp.setUserId(None, False)
        QtGui.qApp.closeDatabase()
#        self.updateActionsState()
        self.setUserName('')


    @pyqtSignature('')
    def on_actQuit_triggered(self):
        self.close()


    @QtCore.pyqtSignature('')
    def on_actConnection_triggered(self):
        dlg   = CConnectionDialog(self)
        preferences = QtGui.qApp.preferences
        dlg.setDriverName(preferences.dbDriverName)
        dlg.setServerName(preferences.dbServerName)
        dlg.setServerPort(preferences.dbServerPort)
        dlg.setDatabaseName(preferences.dbDatabaseName)
        dlg.setUserName(preferences.dbUserName)
        dlg.setPassword(preferences.dbPassword)
        if dlg.exec_():
            preferences.dbDriverName = dlg.driverName()
            preferences.dbServerName = dlg.serverName()
            preferences.dbServerPort = dlg.serverPort()
            preferences.dbDatabaseName = dlg.databaseName()
            preferences.dbUserName = dlg.userName()
            preferences.dbPassword = dlg.password()
            preferences.save()


#    @QtCore.pyqtSignature('bool')
#    def on_chkCopyToActionType_toggled(self, checked):
#        self.chkFillSpecialityRegionalCode.setEnabled(not checked)


    @QtCore.pyqtSignature('')
    def on_actDecor_triggered(self):
        dlg = CDecorDialog(self)
        preferences = QtGui.qApp.preferences
        dlg.setStyle(preferences.decorStyle)
        dlg.setStandardPalette(preferences.decorStandardPalette)
        dlg.setMaximizeMainWindow(preferences.decorMaximizeMainWindow)
        dlg.setFullScreenMainWindow(preferences.decorFullScreenMainWindow)
        if dlg.exec_():
            preferences.decorStyle = dlg.style()
            preferences.decorStandardPalette = dlg.standardPalette()
            preferences.decorMaximizeMainWindow = dlg.maximizeMainWindow()
            preferences.decorFullScreenMainWindow = dlg.fullScreenMainWindow()
            preferences.save()
            QtGui.qApp.applyDecorPreferences()


#    @QtCore.pyqtSignature('')
#    def on_actAbout_triggered(self):
#        global lastChangedRev
#        global lastChangedDate
#        try:
#            from buildInfo import lastChangedRev, lastChangedDate
#        except:
#            lastChangedRev  = 'unknown'
#            lastChangedDate = 'unknown'
#
#        QtGui.QMessageBox.about(
#            self, u'О программе', about % ( title, lastChangedRev, lastChangedDate)
#            )


    @QtCore.pyqtSignature('')
    def on_actAboutQt_triggered(self):
        QtGui.QMessageBox.aboutQt(self, u'О Qt')


def parseGCDebug(val):
    result = 0
    for char in val.upper():
        if char == 'S':
            result |= gc.DEBUG_STATS
        elif char == 'C':
            result |= gc.DEBUG_COLLECTABLE
        elif char == 'U':
            result |= gc.DEBUG_UNCOLLECTABLE
        elif char == 'I':
            result |= gc.DEBUG_INSTANCES
        elif char == 'O':
            result |= gc.DEBUG_OBJECTS
    return result

def parseGCThreshold(val):
    result = []
    if val:
        for s in val.split(','):
            try:
                v = int(s)
            except:
                v = 0
            result.append(v)
        l = len(result)
        if l > 3:
            result = result[0:3]
    return result


if __name__ == '__main__':
##    gc.set_threshold(*[10*k for k in gc.get_threshold()])
    parser = OptionParser(usage = "usage: %prog [options]")
    parser.add_option('-d', '--demo',
                      dest='demo',
                      help='Login as demo',
                      action='store_true',
                      default=False)
    parser.add_option('--gc-debug',
                      dest='gcDebug',
                      help='gc debug flags, chars S(stat), C(collectable), U(uncollectable), I(instances), O(objects), for details see gc.set_debug',
                      metavar='[S][C][U][I][O]',
                      default=''
                      )
    parser.add_option('--gc-threshold',
                      dest='gcThreshold',
                      help='gc thresholds list, see gc.set_threshold ',
                      metavar='threshold0[,threshold1[,threshold2]]',
                      default=''
                     )
    parser.add_option('--version',
                      dest='version',
                      help='print version and exit',
                      action='store_true',
                      default=False)
#    parser.add_option("-q", "--quiet",   action="store_false", dest="verbose", default=True,  help="don't print status messages to stdout")
    (options, args) = parser.parse_args()

    if options.version:
        print '%s v.0.1' % (sys.argv[0] if sys.argv else '')
    else:
        gcDebug = parseGCDebug(options.gcDebug)
        gcThreshold = parseGCThreshold(options.gcThreshold)
        if gcDebug:
            gc.set_debug(gcDebug)
            print 'debug set to ', gcDebug
        if gcThreshold:
            gc.set_threshold(*gcThreshold)
            print 'threshold set to ', gcThreshold

        app = CMyApp(sys.argv, options.demo)
        stdTtranslator = QtCore.QTranslator()
        stdTtranslator.load('i18n/std_ru.qm')
        app.installTranslator(stdTtranslator)

        QtGui.qApp = app
        app.applyDecorPreferences() # надеюсь, что это поможет немного сэкономить при создании гл.окна
        mainWindow = CMainWindow()
        app.mainWindow = mainWindow
        app.applyDecorPreferences() # применение максимизации/полноэкранного режима к главному окну

        if app.preferences.dbAutoLogin:
            mainWindow.actLogin.activate(QtGui.QAction.Trigger)
        mainWindow.show()
        res = app.exec_()
        mainWindow.savePreferences()
        app.preferences.save()
        app.closeDatabase()
        app.doneTrace()
        QtGui.qApp = None

#    sys.exit(res)
