# -*- coding: utf-8 -*-
from PyQt4 import QtGui
from PyQt4.QtCore import *
from library.Utils import *
from appendix.regional.r29.exportAttached.Common.Describers import PolicyDescriber

def safelyExecute(db, stmt):
    """
    безопасно выполнеяет sql запрос, производя роллбек в случае ошибки
    """
    if not db:
        raise
    try:
        db.query(stmt)
        db.commit()
    except Exception, e:
        QtGui.qApp.logCurrentException()
        db.rollback()


def closeClientsAttaches(db, clientId, endDate):
    """
    Проставляет дату закрытия в ClientAttach
    """
    stmt = """UPDATE ClientAttach
              SET endDate = %s
              WHERE client_id = %d
              AND endDate IS NULL""" % (db.formatDate(endDate), clientId)
    safelyExecute(db, stmt)


def insertNewClientAttach(db, clientId, begDate, infis):
    """
    Вставляет новую сущность в clientAttach
    """
    table = db.table('ClientAttach')
    table.setIdFieldName('id')
    record = table.newRecord()
    record.setValue('client_id', clientId)
    record.setValue('begDate', begDate)
    record.setValue('createDatetime', QDateTime.currentDateTime())
    record.setValue('modifyDatetime', QDateTime.currentDateTime())
    record.setValue('deleted', 0)
    record.setValue('attachType_id', getAttachTypeIdByCode(db))
    record.setValue('LPU_id', getOrganisationIdByInfis(db, infis))
    return db.insertRecord(table, record)


def getAttachTypeIdByCode(db, code=1):
    return db.translate('rbAttachType', 'code', code, 'id')


def getClientDbId(db, client):
    """
    По ФИО и Др и полу пациента возвращает ClientAttachId или None, если ничего не нашли
    """
    stmt = """SELECT `Client`.`id` AS `clientId`,
                     `serial`,
                     `number`
              FROM `Client`
              LEFT JOIN `ClientPolicy` ON `ClientPolicy`.`client_Id` = `Client`.`id`
              WHERE lastName = '%s' AND firstName = '%s' AND patrName = '%s'
              AND birthDate = %s
              """ % (client.lastName, client.firstName, client.patrName,
                     db.formatDate(client.birthDate))
    q = db.query(stmt)
    if q.next():
        record = q.record()
        serial = forceString(record.value('serial'))
        clientId = forceInt(record.value('clientId'))
        number = None
        unifiedNumber = None
        if serial:
            number = forceString(record.value('number'))
        else:
            unifiedNumber = forceString(record.value('number'))
        return clientId, serial, number, unifiedNumber
    return None

#TODO: уточнить логику!
def getClientAttachIds(db, clientId):
    stmt = """SELECT id
              FROM ClientAttach
              WHERE client_id = %d
              AND endDate IS NULL""" % clientId
    q = db.query(stmt)
    while q.next():
        yield forceInt(q.record().value(0))


def getClientLastAttachType(db, clientId):
    stmt = """SELECT ClientAttach.Code
              FROM Client
              LEFT JOIN ClientAttach ON Client.id = ClientAttach.client_id
              WHERE
              Client.id = %d AND
              ORDER BY ClientAttach.begDAte DESC
              LIMIT 1""" % clientId
    q = db.query(stmt)
    if q.next():
        return forceInt(q.record().value(0))
    else:
        return None

def checkAttachmentOrganisation(db, clientAttachId, codeMo):
    moOrgId = forceInt(getOrganisationIdByInfis(db, codeMo))
    originalOrgId = forceInt(db.translate('ClientAttach', 'id', clientAttachId, 'LPU_id'))
    return originalOrgId == moOrgId


def getHeaderCreationDateTime(db, headerId):
    return db.translate('ExportAttachHeader', 'id', headerId, 'createDateTime')


def getCurrentInfis(db):
    orgId = QtGui.qApp.currentOrgId()
    infis = db.translate('Organisation', 'id', orgId, 'infisCode')
    return infis


def getOrganisationIdByInfis(db, smo):
    return db.translate('Organisation', 'infisCode', smo, 'id')


def checkClientAttachment(db, externalId, headerId, smo):
    """
    Метод проверяет, что организация, указанная в тэге SMO совпадает с данными, хранящимися в реестре
    """
    record = getExportItemRecord(db, externalId, headerId)
    stmt = """SELECT orgStructure_id
              FROM ClientAttach
              WHERE id = %d""" % forceInt(record.val('clientAttachId'))
    q = db.query(stmt)
    if q.next():
        return q.record.val(0) == getOrganisationIdByInfis(db, smo)
    else:
        return False


def updateExportItem(db, externalId, headerId, column, value):
    """
    Метод общего назначения для обновления записи
    """
    setStmt = ""
    if type(value) is int:
        setStmt = "SET %s = %s" % (column, value)
    else:
        value = re.escape(unicode(value))
        setStmt = "SET %s = '%s'" % (column, value)

    stmt = """UPDATE ExportAttachItem
           %s
           WHERE externalId = %s AND headerId = %s""" % (setStmt, externalId, headerId)
    safelyExecute(db, stmt)


def updateExportItemCondition(db, headerId, setst, wherest):
    """
    То же, что и выше, но более гибкий
    """
    stmt = """UPDATE ExportAttachItem
           SET %s
           WHERE headerId = %d AND %s""" % (setst, headerId, wherest)
    safelyExecute(db, stmt)


def getExportItemRecord(db, externalId, headerId):
    stmt = """SELECT *
              FROM ExportAttachItem
              WHERE externalId = %s AND headerId = %s""" % (externalId, headerId)
    q = db.query(stmt)
    if q.next():
        return q.record()
    return None


def getClientRecordByExternalId(db, externalId, headerId):
    stmt = """SELECT
              lastName as lastName,
              firstName as firstName,
              patrName as patrName,
              sex as sex,
              birthDate as birthDate,
              serial as serial,
              policyKind_id, -- ??? TODO!
              number as number,
              clientId as clientId,
              FROM
              ExportAttachItem
              LEFT JOIN Client On Client.id = ExportAttachItem.clientId
              LEFT JOIN ClientPolicy On ClientPolicy.id = Client.clientPolicy_Id
              WHERE externalId = %s AND headerId = %s
              """ % (externalId, headerId)
    q = db.query(stmt)
    if q.next():
        return q.record()
    return None


def itemExistsInRegistry(db, externalId, headerId):
        stmt = """SELECT EXISTS(
                  SELECT 1 FROM ExportAttachItem
                  WHERE externalId = %d AND headerId = %d) """ % (externalId, headerId)
        q = db.query(stmt)
        if q.next():
            n = forceInt(q.record().value(0))
            if n > 0:
                return True
        return False


def itemExistsInDb(db, tableName, column, value):
        stmt = """SELECT EXISTS(SELECT 1 FROM %s WHERE %s = %s) """ % (tableName, column, value)
        q = db.query(stmt)
        if q.next():
            n = forceInt(q.record().value(0))
            if n > 0:
                return True
        return False


def removeHeader(db, headerId):
    stmt = """DELETE FROM ExportAttachHeader WHERE id = %d""" % headerId
    safelyExecute(db, stmt)


def removeItems(db, headerId):
    stmt = """DELETE FROM ExportAttachItem WHERE headerId = %d""" % headerId
    safelyExecute(db, stmt)


def setErrorState(db, headerId, externalId, code, message):
    updateExportItem(db, externalId, headerId, 'code', code)
    updateExportItem(db, externalId, headerId, 'message', u"""%s""" % message)


