# -*- coding: utf-8 -*-
import os.path
import shutil

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import *

from library.database  import *
from library.Utils     import *
from datetime import *
from Ui_ExportAttachPage1 import Ui_ExportPage1
from Ui_ExportAttachPage2 import Ui_ExportPage2

from XmlWriter import CXmlWriter
from zipfile import *


class CExportWizard(QtGui.QWizard):
    def __init__(self, parent, onlyCurrentLPU, headerId):
        QtGui.QWizard.__init__(self, parent)
        self.setWizardStyle(QtGui.QWizard.ModernStyle)
        self.page1 = CExportPage1(self, headerId)
        self.page2 = CExportPage2(self)
        self.addPage(self.page1)
        self.addPage(self.page2)
        self.setWindowTitle(u'Мастер экспорта прикрепленного населения')
        self.page1.setTitle(u'Экспорт')
        self.page2.setTitle(u'Укажите директорию для сохранения обменных файлов "*.xml"')
        self.xmlFileName = ''
        self.fullXmlFileName = ''
        self.tmpDir = ''
        self.onlyCurrentLPU = onlyCurrentLPU
        self.headerId = headerId

    def getTmpDir(self):
        if not self.tmpDir:
            self.tmpDir = QtGui.qApp.getTmpDir()
        return self.tmpDir

    def cleanup(self):
        if self.tmpDir:
            QtGui.qApp.removeTmpDir(self.tmpDir)
            self.tmpDir = ''


    def exec_(self):
        QtGui.QWizard.exec_(self)
        self.cleanup()


# *****************************************************************************************


class CExportPage1(QtGui.QWizardPage, Ui_ExportPage1):
    def __init__(self, parent, headerId):
        QtGui.QWizardPage.__init__(self, parent)
        self.setupUi(self)
        self.progressBar.setMinimum(0)
        self.progressBar.setMaximum(1)
        self.progressBar.setText('')
        appPreferences = QtGui.qApp.preferences.appPrefs
        self.infis = forceString(QtGui.qApp.db.translate('Organisation', 'id',
                                                       QtGui.qApp.currentOrgId() , 'infisCode'))

        self.headerId = headerId
        (self.settleDate, self.regNum) = self.getHeaderInfo()
        self.setSubTitle(u'для выполнения шага нажмите кнопку "вперед"')
        self.xmlFileName = ""
        self.setExportMode(False)
        self.aborted = False
        self.done = False
        self.parent = parent #wtf? скрывать Qt-шный parent - это плохая затея
        self.connect(parent, QtCore.SIGNAL('rejected()'), self.abort)


    def validatePage(self):
        return self.done


    def setExportMode(self, flag):
        self.btnCancel.setEnabled(flag)


    def log(self, str, forceLog = False):
        if self.chkVerboseLog.isChecked() or forceLog:
            self.logBrowser.append(str)
            self.logBrowser.update()


    def getHeaderInfo(self):
        db = QtGui.qApp.db
        qry = db.query("""SELECT `settleDate`, `number`
                          FROM ExportAttachHeader
                          WHERE id = %d""" % self.headerId)
        if qry.next():
            record = qry.record()
            return (forceDate(record.value(0)), forceInt(record.value(1)))
        else:
            return None, None


    def getFullXmlFileName(self):
        if not self.xmlFileName:
            tprefix = u'NI'
            currendDate = QDate.currentDate()
            year        = currendDate.year()
            month       = currendDate.month()
            registryNn  = str(self.regNum)
            self.xmlFileName = u'%s_%s_%02d%02d%s.xml' % (tprefix, self.infis, year%100, month, self.regNum)
            self.parent.xmlFileName = self.xmlFileName
            self.parent.fullXmlFileName = os.path.join(self.getTmpDir(), self.xmlFileName)
        return self.parent.fullXmlFileName


    def getTmpDir(self):
        return self.parent.getTmpDir()


    def createXML(self):
        outFile = QtCore.QFile(self.getFullXmlFileName())
        outFile.open(QIODevice.WriteOnly | QIODevice.Text)
        return outFile

    def prepareToExport(self):
        self.done = False
        self.aborted = False
        self.emit(QtCore.SIGNAL('completeChanged()'))
        self.setExportMode(True)

        output = self.createXML()

        self.progressBar.reset()
        self.progressBar.setMaximum(1)
        self.progressBar.setValue(0)
        self.progressBar.setText(u'Запрос в БД...')
        QtGui.qApp.processEvents()
        query = self.createQuery()
        self.progressBar.setMaximum(max(query.size(), 1))
        self.progressBar.reset()
        self.progressBar.setValue(0)
        return query, output


    def export(self):
        (result, rc) = QtGui.qApp.call(self, self.exportInt)
        self.setExportMode(False)

        if self.aborted or not result:
            self.progressBar.setText(u'прервано')
        else:
            self.progressBar.setText(u'готово')
            self.done = True
            self.emit(QtCore.SIGNAL('completeChanged()'))

# *****************************************************************************************

    def exportInt(self):
        query, outFile = self.prepareToExport()
        out = CXmlWriter(self, outFile)
        (root, ext) = os.path.splitext(self.xmlFileName)
        out.writeHeader('1.0', self.infis, self.settleDate, root)
        while query.next():
            QtGui.qApp.processEvents()
            if self.aborted:
                break
            self.progressBar.step()
            out.writeRecord(query.record())

        out.writeFooter()
        outFile.close()
        self.progressBar.setValue(0)

    def createQuery(self):
        db = QtGui.qApp.db
        return db.query("""SELECT `id`,
                            `clientId`,
                            `clientAttachId`,
                            `lastName`,
                            `firstName`,
                            `patrName`,
                            `birthDate`,
                            `sex`,
                            `policyKind`,
                            `serial`,
                            `number`,
                            `dateAttach`,
                            `headerId`,
                            `externalId`
                            FROM ExportAttachItem WHERE headerId = %d """ %self.headerId)


    def isComplete(self):
        return self.done


    def abort(self):
        self.aborted = True


    @QtCore.pyqtSignature('')
    def on_btnExport_clicked(self):
        self.export()


    @QtCore.pyqtSignature('')
    def on_btnCancel_clicked(self):
        self.abort()

# *****************************************************************************************

class CExportPage2(QtGui.QWizardPage, Ui_ExportPage2):
    def __init__(self, parent):
        QtGui.QWizardPage.__init__(self, parent)
        self.parent = parent #wtf? скрывать Qt-шный parent - это плохая затея
        self.setupUi(self)
        self.setSubTitle(u'для выполнения шага нажмите кнопку "Финиш"')
        self.pathIsValid = True
        homePath = QtCore.QDir.toNativeSeparators(QtCore.QDir.homePath())
        exportDir = forceString(QtGui.qApp.preferences.appPrefs.get('AttachExportDir', homePath))
        self.edtDir.setText(exportDir)


    def isComplete(self):
        return self.pathIsValid

    def validatePage(self):
        success = False
        baseName = os.path.basename(self.parent.fullXmlFileName)
        (root, ext) = os.path.splitext(baseName)
        zipFileName = '%s.zip' % root
        zipFilePath = os.path.join(forceStringEx(self.parent.getTmpDir()),
                                                zipFileName)
        zf = ZipFile(zipFilePath, 'w', allowZip64=True)

        src = self.parent.fullXmlFileName
        zf.write(src, os.path.basename(src), ZIP_DEFLATED)

        zf.close()

        dst = os.path.join(forceStringEx(self.edtDir.text()), zipFileName)
        success, result = QtGui.qApp.call(self, shutil.move, (zipFilePath, dst))

        return success


    @QtCore.pyqtSignature('QString')
    def on_edtDir_textChanged(self):
        dir = forceStringEx(self.edtDir.text())
        pathIsValid = os.path.isdir(dir)
        if self.pathIsValid != pathIsValid:
            self.pathIsValid = pathIsValid
            self.emit(QtCore.SIGNAL('completeChanged()'))


    @QtCore.pyqtSignature('')
    def on_btnSelectDir_clicked(self):
        dir = QtGui.QFileDialog.getExistingDirectory(self,
                u'Выберите директорию для сохранения файла выгрузки',
                 forceStringEx(self.edtDir.text()),
                 QtGui.QFileDialog.ShowDirsOnly)
        if forceString(dir):
            self.edtDir.setText(QtCore.QDir.toNativeSeparators(dir))
