# -*- coding: utf-8 -*-

from PyQt4 import QtGui
from PyQt4.QtCore import *
from library.Utils import *
from appendix.regional.r29.exportAttached.Common.DbMethods import *


class CFormExportData(object):
    def __init__(self):
        self.db = QtGui.qApp.db

    def getReady(self, settleDate, attachTypeFilter):
        if attachTypeFilter.count == 0:
            return False

        (fromDate, lastSettleDate, number, lastFilter) = self.getLastHeader()
        if lastSettleDate is not None and settleDate == lastSettleDate:
            if attachTypeFilter == [] or lastFilter == []: #already selected every attach type
                return False
            if not self.checkFiltersCorresponding(lastFilter, attachTypeFilter):
                return False
            else:
                lastSettleDate = fromDate

        if lastSettleDate:
            self.fromDate = lastSettleDate
        else:
            self.fromDate = QDate(0, 0, 0)

        self.number = number + 1
        self.settleDate = settleDate
        self.filterSimpleStr = ','.join(str(x) for x in attachTypeFilter)
        self.attachTypeFilter = attachTypeFilter
        self.attachTypeFilterSql = 'AND `attachType_id` IN (%s)' % \
                                   ', '.join(str(x) for x in attachTypeFilter) \
            if attachTypeFilter else ''
        return True

    def start(self, hComment, progressBar):
        headerId = None
        result = None
        try: #TODO: FIX THIS, return touple
            headerId = self.insertHeader(hComment)
            result = self.insertItems(headerId, progressBar)
        except Exception, e:
            QtGui.qApp.logCurrentException()
            if headerId:
                removeItems(self.db, headerId)
                removeHeader(self.db, headerId)
        return result

    #a little bit magic
    def _parseFilter(self, strFilter):
        if strFilter == '':
            return []
        return map(int, strFilter.split(','))

    #a little bit MOAR magic
    def checkFiltersCorresponding(self, filterOld, filterNew):
        return not bool(set(filterOld) & set(filterNew))

    def getLastHeader(self):
        stmt = """SELECT fromDate, settleDate, number, filter 
                  FROM ExportAttachHeader
                  ORDER BY number DESC
                  LIMIT 1
                  """
        qry = self.db.query(stmt)
        if qry.next():
            return forceDate(qry.record().value(0)), \
                   forceDate(qry.record().value(1)), \
                   forceInt(qry.record().value(2)), \
                   self._parseFilter(forceString(qry.record().value(3)))
        else:
            return None, None, 0, None

    def _safelyExecute(self, db, stmt):
        try:
            self.db.query(stmt)
            self.db.commit()
        except Exception, e:
            QtGui.qApp.logCurrentException()
            self.db.rollback()

    def isNew(self, clientId):
        stmt = """SELECT EXISTS(SELECT 1 FROM ExportAttachItem WHERE clientId = %d) """ % clientId
        q = self.db.query(stmt)
        if q.next():
            n = forceInt(q.record().value(0))
            if n > 0:
                return False
        return True

    def isChanged(self, clientAttachId, attachDate, clientId):
        if self.number == 1:
            return True

        stmtSelect = """SELECT
                      `ExportAttachItem`.`clientAttachId` AS `clientAttachId`,
                      `ExportAttachItem`.`dateAttach` AS `dateAttach`
                      FROM `ExportAttachItem`
                      WHERE `ExportAttachItem`.`clientId` = %d
                      ORDER BY HeaderId DESC
                      LIMIT 1
        """ % (clientId)

        q = self.db.query(stmtSelect)
        while q.next():
            record = q.record()
            oldClientAttachId = forceInt(record.value('clientAttachId'))
            oldAttachDate = forceDate(record.value('dateAttach'))
            if oldClientAttachId != clientAttachId or attachDate != oldAttachDate:
                msg = "clientId = %d, oldClAttId = %d, newClAttId = %d dateold = %s dateNew = %s" % (clientId,
                                                                                                     oldClientAttachId,
                                                                                                     clientAttachId,
                                                                                                     str(oldAttachDate),
                                                                                                     str(attachDate))
                QtGui.qApp.log(u"отладочная информация: ", u"произошли изменения: \r\n %s" % msg)
                return True
        return False

    def getFromDateForAttachType(self, type):
        stmt = """SELECT fromDate 
                  FROM ExportAttachHeader
                  WHERE filter LIKE '%%%d%%'
                  ORDER BY SettleDate DESC
                  LIMIT 1
        """ % type
        q = self.db.query(stmt)
        if q.next():
            return forceDate(q.record().value(0))

    def generateWhenThenFilter(self):
        result = ''
        for type in self.attachTypeFilter:
            fromDate = self.getFromDateForAttachType(type)
            settleDate = self.settleDate
            result += """WHEN %d THEN \r\n
                                (ClientAttach.`modifyDatetime`>%s) AND (ClientAttach.`modifyDatetime`<=%s) \r\n""" \
                     % (type, self.db.formatDate(fromDate), self.db.formatDate(settleDate))
        return result

    def insertItems(self, headerId, progressBar):
        if not headerId:
            return (0, 0)
        wt = self.generateWhenThenFilter()

        stmtSelect = """
SELECT DISTINCT
`Client`.`id` AS `clientId`,
`ClientAttach`.`id` AS `clientAttachId`,
`lastName` AS `lastName`,
`firstName` AS `firstName`, 
`patrName` AS `patrName`,
`birthDate` AS `birthDate`,
`Client`.`sex` as `sex`,
`ClientPolicy`.`policyKind_id` as `policyKind`,
`ClientPolicy`.`serial` as `serial`,
`ClientPolicy`.`number` as `number`,
`ClientAttach`.`begDate` AS `dateAttach`,
`ClientAttach`.`endDate` as `dateAttachEnd`,
`ClientAttach`.`modifyDatetime` as `modDate`
FROM `ClientAttach`
LEFT JOIN `Client` ON Client.id = ClientAttach.`client_id`
LEFT JOIN `ClientPolicy` ON `ClientPolicy`.`client_id` = ClientAttach.`client_id`
WHERE (`number` <> '' AND `number` IS NOT NULL) 

%s -- attach type filter

AND
CASE (`ClientAttach`.`endDate` IS NULL)
    WHEN 1
        THEN 1 = 1
    ELSE
       `ClientAttach`.`endDate` > %s
    END
AND
CASE attachType_id
    %s    
END
GROUP BY ClientAttach.`client_id`
HAVING  MAX(`dateAttach`)
ORDER BY `Client`.`id`
        """ % (self.attachTypeFilterSql,
               (self.db.formatDate(self.settleDate)),
               wt)
        QtGui.qApp.log(u"Отладочная информация, запрос", stmtSelect)

        qry = self.db.query(stmtSelect)

        if qry.size() == 0:
            removeHeader(self.db, headerId)
            return (0, 0) #TODO: dummy way, fix it!

        progressBar.reset()
        progressBar.setMinimum(0)
        progressBar.setValue(0)
        count = qry.size()
        i = 0
        progressBar.setMaximum(max(count, 1))

        while qry.next():
            QtGui.qApp.processEvents()
            selrecord = qry.record()
            dateAttach = forceDate(selrecord.value('dateAttach'))
            clientAttachId = forceInt(selrecord.value('clientAttachId'))
            clientId = forceInt(selrecord.value('clientId'))
            progressBar.setValue(progressBar.value() + 1)
            if not (self.isChanged(clientAttachId, dateAttach, clientId) or self.isNew(clientId)):
                continue
            i += 1
            tableItem = self.db.table('ExportAttachItem')
            tableItem.setIdFieldName('id')
            record = tableItem.newRecord()
            record.setValue('clientId', clientId)
            record.setValue('clientAttachId', forceInt(selrecord.value('clientAttachId')))
            record.setValue('lastName', forceString(selrecord.value('lastName')))
            record.setValue('firstName', forceString(selrecord.value('firstName')))
            record.setValue('patrName', forceString(selrecord.value('patrName')))
            record.setValue('birthDate', forceDate(selrecord.value('birthDate')))
            record.setValue('sex', forceInt(selrecord.value('sex')))
            record.setValue('policyKind', forceInt(selrecord.value('policyKind')))
            record.setValue('serial', forceString(selrecord.value('serial')))
            record.setValue('number', forceString(selrecord.value('number')))
            record.setValue('dateAttach', dateAttach)
            record.setValue('headerId', headerId)
            record.setValue('externalId', i)
            self.db.insertRecord(tableItem, record)

        if i > 0:
            stmt = """UPDATE ExportAttachHeader SET count = %d WHERE id = %d""" % (i, headerId)
            safelyExecute(self.db, stmt)
            return headerId, i
        else:
            removeHeader(self.db, headerId)
            return headerId, 0

    def insertHeader(self, hComment):
        tableHeader = self.db.table('ExportAttachHeader')
        tableHeader.setIdFieldName('id')
        record = tableHeader.newRecord()
        record.setValue('settleDate', self.settleDate)
        record.setValue('fromDate', self.fromDate)
        record.setValue('createDatetime', QDateTime.currentDateTime())
        record.setValue('number', self.number)
        record.setValue('comment', hComment)
        record.setValue('filter', self.filterSimpleStr)
        return self.db.insertRecord(tableHeader, record)
