# -*- coding: utf-8 -*-
#############################################################################
from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import *
import codecs
import locale
import os
import shutil
import sys
import tempfile
import traceback
from Users.UserInfo import CUserInfo, CDemoUserInfo
from Users.Tables import *
from library import database
from library.Preferences import CPreferences
from library.Utils import *
from MainWindow import CMainWindow

# #############################################################


class CMyApp(QtGui.QApplication):
    __pyqtSignals__ = (
                       'dbConnectionChanged(bool)',
                       'currentUserIdChanged()',
                      )

    def __init__(self, args, demoModeRequest):
        QtGui.QApplication.__init__(self, args)
        self.logDir = os.path.join(unicode(QDir.toNativeSeparators(QDir.homePath())), '.samson-vista')
        self.oldMsgHandler = qInstallMsgHandler(self.msgHandler)
        self.oldEexceptHook = sys.excepthook
        sys.excepthook = self.logException
        self.traceActive = True
        self.homeDir= None
        self.saveDir= None
        self.userId = None
        self.userSpecialityId = None
        self.userOrgStructureId = None
        self.db     = None
        self.mainWindow = None
        self.preferences = CPreferences('r29ExportAttached.ini')
        self.preferences.load()
        self.preferencesDir = self.preferences.getDir()
        self._currentClientId = None
        self.userInfo = None
        self.demoModeRequested = False
        self._highlightInvalidDate = None

    def currentClientId(self):
        if self.mainWindow and self.mainWindow.tblItems:
            return self.mainWindow.tblItems.currentClientId
        else:
            return None

    def identServiceEnabled(self):
        return True

#################################
    def highlightRedDate(self):
        return False


    def highlightInvalidDate(self):
        return False
#################################

    def doneTrace(self):
        self.traceActive = False
        qInstallMsgHandler(self.oldMsgHandler)
        sys.excepthook = self.oldEexceptHook


    def applyDecorPreferences(self):
        self.setStyle(self.preferences.decorStyle)
        if self.preferences.decorStandardPalette:
            self.setPalette(self.style().standardPalette())
        if self.mainWindow:
            state = Qt.WindowNoState
            if self.preferences.decorMaximizeMainWindow:
                state |= Qt.WindowMaximized
            if self.preferences.decorFullScreenMainWindow:
                state |= Qt.WindowFullScreen
            self.mainWindow.setWindowState(state)


    def currentOrgId(self):
        return forceRef(QtGui.qApp.preferences.appPrefs.get('orgId', QVariant()))


    def getHomeDir(self):
        if not self.homeDir:
            homeDir = os.path.expanduser('~')
            if homeDir == '~':
                homeDir = unicode(QDir.homePath())
            if isinstance(homeDir, str):
                homeDir = unicode(homeDir, locale.getpreferredencoding())
            self.homeDir = homeDir
        return self.homeDir


    def getSaveDir(self):
        self.saveDir = forceString(self.preferences.appPrefs.get('saveDir', QVariant()))
        if not self.saveDir:
            self.saveDir = self.getHomeDir()
        return self.saveDir


    def setSaveDir(self, path):
        saveDir = os.path.dirname(unicode(path))
        if saveDir and self.saveDir != saveDir:
            self.preferences.appPrefs['saveDir'] = QVariant(saveDir)


    def getTemplateDir(self):
        result = forceString(self.preferences.appPrefs.get('templateDir', None))
        if not result:
            result = os.path.join(self.logDir, 'templates')
        return result


    def getTmpDir(self, suffix=''):
        return unicode(tempfile.mkdtemp('', 'samson_%s_' % suffix), locale.getpreferredencoding())


    def removeTmpDir(self, dir):
        try:
            shutil.rmtree(dir, False)
        except:
            self.logCurrentException()


    def log(self, title, message, stack=None):
        try:
            if not os.path.exists(self.logDir):
                os.makedirs(self.logDir)
            logFile = os.path.join(self.logDir, 'error.log')
            timeString = unicode(QDateTime.currentDateTime().toString(Qt.SystemLocaleDate))
            logString = u'%s\n%s: %s(%s)\n' % ('='*72, timeString, title, message)
            if stack:
                try:
                    logString += ''.join(traceback.format_list(stack)).decode('utf-8') + '\n'
                except:
                    logString += 'stack lost\n'
            file = codecs.open(logFile, mode='a', encoding=locale.getpreferredencoding())
            file.write(logString)
            file.close()
        except:
            pass


    def logException(self, exceptionType, exceptionValue, exceptionTraceback):
        title = repr(exceptionType)
        message = unicode(exceptionValue)
        self.log(title, message, traceback.extract_tb(exceptionTraceback))
        sys.__excepthook__(exceptionType, exceptionValue, exceptionTraceback)


    def logCurrentException(self):
        self.logException(*sys.exc_info())


    def msgHandler(self, type, msg):
        if type == 0: # QtMsgType.QtDebugMsg:
            typeName = 'QtDebugMsg'
        elif type == 1: # QtMsgType.QtWarningMsg:
            typeName = 'QtWarningMsg'
        elif type == 2: # QtMsgType.QtCriticalMsg:
            typeName = 'QtCriticalMsg'
        elif type == 3: # QtFatalMsg
            typeName = 'QtFatalMsg'
        else:
            typeName = 'QtUnknownMsg'

        self.log( typeName, msg, traceback.extract_stack()[:-1])
#        print typeName, msg


    def openDatabase(self):
        self.db = None
        self.db = database.connectDataBase(self.preferences.dbDriverName,
                                     self.preferences.dbServerName,
                                     self.preferences.dbServerPort,
                                     self.preferences.dbDatabaseName,
                                     self.preferences.dbUserName,
                                     self.preferences.dbPassword)
        self.emit(QtCore.SIGNAL('dbConnectionChanged(bool)'), True)


    def closeDatabase(self):
        if self.db:
            self.db.close()
            self.db = None
            self.emit(QtCore.SIGNAL('dbConnectionChanged(bool)'), False)

    def notify(self, receiver, event):
        try:
            return QtGui.QApplication.notify(self, receiver, event)
        except Exception, e:
            if self.traceActive:
                self.logCurrentException()
                widget = self.activeModalWidget()
                if widget is None:
                    widget = self.mainWindow
                QtGui.QMessageBox.critical(widget,
                                            u'Произошла ошибка',
                                            unicode(e),
                                            QtGui.QMessageBox.Close)
            return False
        except:
            return False


    def call(self, widget, func, params = ()):
        try:
            return True, func(*params)
        except IOError, e:
            msg=''
            if hasattr(e, 'filename'):
                msg = u'%s:\n[Errno %s] %s' % (e.filename, e.errno, anyToUnicode(e.strerror))
            else:
                msg = u'[Errno %s] %s' % (e.errno, anyToUnicode(e.strerror))
            QtGui.QMessageBox.critical(widget,u'Произошла ошибка ввода-вывода', msg, QtGui.QMessageBox.Close)
        except Exception, e:
            self.logCurrentException()
            widget = widget or self.activeModalWidget() or self.mainWindow
            QtGui.QMessageBox.critical( widget,
                                        u'Произошла ошибка',
                                        unicode(e),
                                        QtGui.QMessageBox.Close)
        return False, None


    def callWithWaitCursor(self, widget, func, *params, **kwparams):
        self.setOverrideCursor(QtGui.QCursor(Qt.WaitCursor))
        try:
            return func(*params, **kwparams)
        finally:
            self.restoreOverrideCursor()


    def demoModePosible(self):
        try:
            tableUser = self.db.table(tblUser)
            record = self.db.getRecordEx(tableUser, [usrPassword, usrRetired], tableUser[usrLogin].eq(demoUserName))
            return not record or (not forceString(record.value(usrPassword)) and not forceBool(record.value(usrRetired)))
        except:
            pass
        return False


    def setUserId(self, userId, demoMode = False):
        self.userId = userId
        record = self.db.getRecord('Person', ['speciality_id', 'orgStructure_id'], userId) if userId else None
        if record:
            self.userSpecialityId = forceRef(record.value('speciality_id'))
            self.userOrgStructureId = forceRef(record.value('orgStructure_id'))
        else:
            self.userSpecialityId = None
            self.userOrgStructureId = None
        self.demoMode = demoMode
        if demoMode:
            self.userInfo = CDemoUserInfo(userId)
        else:
            if userId:
                self.userInfo = CUserInfo(userId)
            else:
                self.userInfo = None
        self.emit(QtCore.SIGNAL('currentUserIdChanged()'))


    def userName(self):
        if self.userInfo:
            return u'%s (%s)' % (self.userInfo.name(), self.userInfo.login())
        else:
            return ''


    def userHasRight(self, right):
        return self.userInfo is not None and self.userInfo.hasRight(right)

    def userHasAnyRight(self, rights):
        return self.userInfo is not None and self.userInfo.hasAnyRight(rights)

    def defaultKLADR(self):
        result = forceString(self.preferences.appPrefs.get('defaultKLADR', ''))
        if not result:
            result = '2900000000000'
        return result


if __name__ == '__main__':
    app = CMyApp(sys.argv, False)
    stdTtranslator = QtCore.QTranslator()
    stdTtranslator.load('i18n/std_ru.qm')
    app.installTranslator(stdTtranslator)

    QtGui.qApp = app
    app.applyDecorPreferences()
    mainWindow = CMainWindow()
    app.mainWindow = mainWindow
    app.applyDecorPreferences()

    if app.preferences.dbAutoLogin:
        mainWindow.actLogin.activate(QtGui.QAction.Trigger)
        mainWindow.setDb(app.db)

    mainWindow.show()
    mainWindow.addPopupDelRow(mainWindow.tblHeaders)
    mainWindow.addPopupDelItemRow(mainWindow.tblItems)
    mainWindow.addPopupUpgradeRow(mainWindow.tblHeaders)
    mainWindow.addPopupImportNf(mainWindow.tblHeaders)
    mainWindow.addPopupImportNp(mainWindow.tblHeaders)
    mainWindow.addPopupOpenClient(mainWindow.tblItems)
    res = app.exec_()
    mainWindow.savePreferences()
    app.preferences.save()
    app.closeDatabase()
    app.doneTrace()
    QtGui.qApp = None
