--
-- Создание БД со справочниками ЛОФОМС, необходимыми для экпорта счетов.
-- Справочники взяты из программы 
--   "Учет пациентов и формирование статистики" v3.91.00A, Декабрь 2008 ЛОФОМС
-- где они хранятся в формате TPS - Clarion 6. Для получения текстовых файлов 
-- необходимо запустить утилиту TopSpeed Database Scanner из поставки Clarion 6,
-- открыть в ней соответствующий справочник, меню Column->Show\Hide, в диалоге 
-- нажимаем кнопку Select All, затем кнопку Ok, далее меню File->Export.
-- В диалоге экспорта убираем галки с "Create Titles", "Quote strings"
-- Выгружаем все справочники, кладем их в директорию к этому скрипту, 
-- запускаем скрипт.
--
DROP SCHEMA IF EXISTS `LOFOMS`;
CREATE SCHEMA `LOFOMS`;

CREATE TABLE `LOFOMS`.`S_POL` (
`id` INT NOT NULL AUTO_INCREMENT PRIMARY KEY ,
`N` VARCHAR(10) NOT NULL COMMENT 'Код',
`PRIM` VARCHAR(40) COMMENT 'Название ЛПУ',
`TMO`  INT,
`COATO` INT COMMENT 'ОКАТО?',
`IIN` VARCHAR(15) COMMENT 'ИНН?',
`BANK1` VARCHAR(39) COMMENT 'Банк строка 1',
`BANK2` VARCHAR(39) COMMENT 'Банк строка 2',
`ADDRES_U1` VARCHAR(39) COMMENT 'Адрес строка 1',
`ADDRES_U2` VARCHAR(39) COMMENT 'Адрес строка 2',
`ADDRES_F` VARCHAR(78) COMMENT 'Адрес',
`MFO` INT,
`SCHETR` VARCHAR(20) COMMENT 'Счет расчетный',
`SCHETK` VARCHAR(20)  COMMENT 'Счет корреспондирующий',
`FAM` VARCHAR(18),
`IM` VARCHAR(18),
`OTCH` VARCHAR(18),
`PHONE` VARCHAR(15),
`KOMM` VARCHAR(78),
`TIP` INT,
`MUN` INT,
`OKPO` VARCHAR(10),
`DELIVERY` INT,
`FAM_DOCTOR` INT,
`TOUN_COUNTRY_DOCTOR` VARCHAR(2),
`TOUN_COUNTRY` VARCHAR(2),
`AMB_POLIKL` INT,
`HAVE_AMB_STAC` VARCHAR(2),
`DATE_INSERT_OMS` DATE,
`DATE_DELETE_OMS` DATE,
`TMU_CODE_STAC_NEW` INT,
`TMU_CODE_AMB_NEW` INT,
`CODE_FEO` INT,
`LICENCE_NUMBER` VARCHAR(20),
`LICENCE_DATE_START` DATE,
`LICENCE_DATE_FINISH` DATE
) ENGINE = InnoDB DEFAULT CHARSET=utf8 COMMENT = 'Справочник ЛПУ' ;

CREATE TABLE `LOFOMS`.`CAT_PACS` (
`id` INT NOT NULL AUTO_INCREMENT PRIMARY KEY ,
`N` VARCHAR(10) NOT NULL COMMENT 'Код',
`DESCRIPTION` VARCHAR(40) COMMENT 'Описание'
) ENGINE = InnoDB DEFAULT CHARSET=utf8 COMMENT = 'Категории пациентов' ;

CREATE TABLE `LOFOMS`.`CL_SERVS` (
`id` INT NOT NULL AUTO_INCREMENT PRIMARY KEY ,
`CODE` VARCHAR(10) NOT NULL COMMENT 'Код',
`DESCRIPTION` VARCHAR(40)  COMMENT 'Описание',
`TYPE` INT,
`CS_TO_PLAN_VOL_CODE` INT,
`DATE_ANNUL` DATE
) ENGINE = InnoDB DEFAULT CHARSET=utf8 COMMENT = 'Услуги клиники' ;

CREATE TABLE `LOFOMS`.`E_COUSES` (
`id` INT NOT NULL AUTO_INCREMENT PRIMARY KEY ,
`CODE` VARCHAR(10) NOT NULL COMMENT 'Код',
`DESCRIPTION` VARCHAR(40) COMMENT 'Описание'
) ENGINE = InnoDB DEFAULT CHARSET=utf8 COMMENT = 'Причины' ;

CREATE TABLE `LOFOMS`.`MAN` (
`id` INT NOT NULL AUTO_INCREMENT PRIMARY KEY ,
`N` VARCHAR(10) NOT NULL COMMENT 'Код',
`NAME` VARCHAR(59) COMMENT 'Наименование'
) ENGINE = InnoDB DEFAULT CHARSET=utf8 COMMENT = 'Манипуляции' ;

CREATE TABLE `LOFOMS`.`S_COMP` (
`id` INT NOT NULL AUTO_INCREMENT PRIMARY KEY ,
`N` VARCHAR(10) NOT NULL COMMENT 'Код',
`PRIM` VARCHAR(40) COMMENT 'Наименование',
`IIN` VARCHAR(15) COMMENT 'ИНН?',
`N_REG` VARCHAR(6),
`DATA_REG` DATE,
`N_USTAV` VARCHAR(10),
`DATA_U` DATE,
`BANK1` VARCHAR(39) COMMENT 'Банк строка 1',
`BANK2` VARCHAR(39) COMMENT 'Банк строка 2',
`ADDRES_U1` VARCHAR(39),
`ADDRES_U2` VARCHAR(39),
`MFO` INT,
`SCHETR` VARCHAR(20),
`SCHETK` VARCHAR(20),
`N_DOG` VARCHAR(10),
`DATA_Z`DATE,
`N_LIC` VARCHAR(6),
`SROK_LIC` DATE,
`FIOD` VARCHAR(18) COMMENT 'Фамилия директора',
`IM` VARCHAR(18) COMMENT 'Имя директора',
`OTCH` VARCHAR(18) COMMENT 'Отчество директора',
`PHONE` VARCHAR(15) COMMENT 'Номер телефона',
`KOMM` VARCHAR(78) COMMENT 'Примечание',
`NUMBER_FEO_LO` INT,
`NUMBER_FEO_SPB` INT
) ENGINE = InnoDB DEFAULT CHARSET=utf8 COMMENT = 'Страховые компании' ;

CREATE TABLE `LOFOMS`.`S_FIL` (
`id` INT NOT NULL AUTO_INCREMENT PRIMARY KEY ,
`N` VARCHAR(10) NOT NULL COMMENT 'Код',
`PRIM` VARCHAR(40) COMMENT 'Наименование'
) ENGINE = InnoDB DEFAULT CHARSET=utf8 COMMENT = 'Справочник наименований районов' ;

CREATE TABLE `LOFOMS`.`S_FIL_L` (
`id` INT NOT NULL AUTO_INCREMENT PRIMARY KEY ,
`N` VARCHAR(10) NOT NULL COMMENT 'Код',
`PRIM` VARCHAR(40) COMMENT 'Наименование',
`IIN` VARCHAR(15) COMMENT 'ИНН',
`RN` VARCHAR(25),
`BANK1` VARCHAR(39) COMMENT 'Банк строка 1',
`BANK2` VARCHAR(39) COMMENT 'Банк строка 2',
`ADDRES_F1` VARCHAR(39) COMMENT 'Адрес строка 1',
`ADDRES_F2` VARCHAR(39) COMMENT 'Адрес строка 2',
`MFO` INT,
`SCHETR` VARCHAR(20) COMMENT 'Счет расчетный',
`SCHETK` VARCHAR(20)  COMMENT 'Счет корреспондирующий',
`FAM` VARCHAR(18) COMMENT 'Фамилия',
`IM` VARCHAR(18) COMMENT 'Имя',
`OTCH` VARCHAR(18) COMMENT 'Отчество',
`KOMM` VARCHAR(78) COMMENT 'Примечание',
`COMPANI` INT
) ENGINE = InnoDB DEFAULT CHARSET=utf8 COMMENT = 'Справочник подразделений фонда' ;

CREATE TABLE `LOFOMS`.`S_GRUP` (
`id` INT NOT NULL AUTO_INCREMENT PRIMARY KEY ,
`N` VARCHAR(10) NOT NULL COMMENT 'Код',
`PRIM` VARCHAR(40) COMMENT 'Наименование'
) ENGINE = InnoDB DEFAULT CHARSET=utf8 COMMENT = 'Справочник социальных групп' ;

CREATE TABLE `LOFOMS`.`S_MUN` (
`id` INT NOT NULL AUTO_INCREMENT PRIMARY KEY ,
`N` VARCHAR(10) NOT NULL COMMENT 'Код',
`PRIM` VARCHAR(40) COMMENT 'Наименование',
`ADDRES_F1` VARCHAR(39) COMMENT 'Адрес строка 1',
`ADDRES_F2` VARCHAR(39) COMMENT 'Адрес строка 2',
`PHONE` VARCHAR(15) COMMENT 'Номер телефона',
`COMPANI` INT,
`data` date,
`OLD_COMPANI` INT,
`CODE_FEO` INT,
`FIL_CODE` INT
) ENGINE = InnoDB DEFAULT CHARSET=utf8 COMMENT = 'Справочник муниципальных образований' ;

CREATE TABLE `LOFOMS`.`S_TERR` (
`id` INT NOT NULL AUTO_INCREMENT PRIMARY KEY ,
`N` VARCHAR(10) NOT NULL COMMENT 'Код',
`PRIM` VARCHAR(40) COMMENT 'Наименование',
`KOD_REGION` INT,
`KOD_TERR` INT,
`PRIM_TERR` VARCHAR(40),
`N_DOG` VARCHAR(6),
`IIN` VARCHAR(15) COMMENT 'ИНН',
`OKONH` VARCHAR(15)  COMMENT 'Код по ОКОНХ',
`OKPO` VARCHAR(15) COMMENT 'Код по ОКПО',
`BANK1` VARCHAR(39) COMMENT 'Банк строка 1',
`BANK2` VARCHAR(39) COMMENT 'Банк строка 2',
`ADDRES_U1` VARCHAR(39) COMMENT 'Адрес строка 1',
`ADDRES_U2` VARCHAR(39) COMMENT 'Адрес строка 2',
`MFO` INT,
`SCHETR` VARCHAR(20) COMMENT 'Счет расчетный',
`SCHETK` VARCHAR(20)  COMMENT 'Счет корреспондирующий',
`FIOD` VARCHAR(18) COMMENT 'Фамилия директора',
`IM` VARCHAR(18) COMMENT 'Имя директора',
`OTCH` VARCHAR(18) COMMENT 'Отчество директора',
`PHONE` VARCHAR(15) COMMENT 'Номер телефона',
`KOMM` VARCHAR(78) COMMENT 'Примечание',
`PRIM_RP` VARCHAR(26),
`PRIM_DP` VARCHAR(23)
) ENGINE = InnoDB DEFAULT CHARSET=utf8 COMMENT = 'Справочник территориальных фондов' ;

CREATE TABLE `LOFOMS`.`TR_TYPES` (
`id` INT NOT NULL AUTO_INCREMENT PRIMARY KEY ,
`N` VARCHAR(10) NOT NULL COMMENT 'Код',
`DESCRIPTION` VARCHAR(40) COMMENT 'Описание'
) ENGINE = InnoDB DEFAULT CHARSET=utf8 COMMENT = 'Виды травм' ;

CREATE TABLE `LOFOMS`.`NPUNKT` (
`id` INT NOT NULL AUTO_INCREMENT PRIMARY KEY ,
`N_FILIAL` INT,
`N_PUNKT` INT,
`PRIM` VARCHAR(46),
`G_D` INT,
`N_POLIKL` INT,
`MUNICIPAL` INT,
`N_FILIAL_L` INT
) ENGINE = InnoDB DEFAULT CHARSET=utf8 COMMENT = 'Справочник населенных пунктов' ;

CREATE TABLE `LOFOMS`.`OPER` (
`id` INT NOT NULL AUTO_INCREMENT PRIMARY KEY ,
`CODE` VARCHAR(10) NOT NULL COMMENT 'Код',
`DESCRIPT` VARCHAR(40)  COMMENT 'Описание',
`CATEGORY` INT,
`CODE_MVK` VARCHAR(10),
`AMB_STAC` INT
) ENGINE = InnoDB DEFAULT CHARSET=utf8 COMMENT = 'Справочник операций' ;

LOAD DATA LOCAL INFILE 's_pol.txt'
    INTO TABLE `LOFOMS`.`S_POL`
    CHARACTER SET cp1251
    LINES TERMINATED BY '\r\n'
    (`id`, @`n_val`, `PRIM`, `TMO`, `COATO`, `IIN`, `BANK1`, `BANK2`, `ADDRES_U1`, `ADDRES_U2`, 
    `ADDRES_F`, `MFO`, `SCHETR`, `SCHETK`, `FAM`, `IM`, `OTCH`, `PHONE`, `KOMM`, `TIP`, 
    `MUN`, `OKPO`, `DELIVERY`, `FAM_DOCTOR`, `TOUN_COUNTRY_DOCTOR`, `TOUN_COUNTRY`, 
    `AMB_POLIKL`, `HAVE_AMB_STAC`, @`oms_beg`, @`oms_end`, 
    `TMU_CODE_STAC_NEW`, `TMU_CODE_AMB_NEW`, `CODE_FEO`, `LICENCE_NUMBER`, 
    @`l_beg`, @`l_end`)
    SET `DATE_INSERT_OMS`=IF(@`oms_beg`>0, date_add('1800-12-28', INTERVAL REPLACE(@`oms_beg`,',','') DAY), NULL),
        `DATE_DELETE_OMS`=IF(@`oms_end`>0, date_add('1800-12-28', INTERVAL REPLACE(@`oms_end`,',','') DAY), NULL),
        `LICENCE_DATE_START`=IF(@`l_beg`>0, date_add('1800-12-28', INTERVAL REPLACE(@`l_beg`,',','') DAY), NULL),
        `LICENCE_DATE_FINISH`=IF(@`l_end`>0, date_add('1800-12-28', INTERVAL REPLACE(@`l_end`,',','') DAY), NULL),
        `N` = TRIM(REPLACE(@`n_val`,',',''));

LOAD DATA LOCAL INFILE 'cat_pacs.txt'
    INTO TABLE `LOFOMS`.`CAT_PACS`
    CHARACTER SET cp1251
    LINES TERMINATED BY '\r\n';
    
LOAD DATA LOCAL INFILE 'cl_servs.txt'
    INTO TABLE `LOFOMS`.`CL_SERVS`
    CHARACTER SET cp1251
    LINES TERMINATED BY '\r\n'
    (`id`, @`code_val`, `DESCRIPTION`, `TYPE`, `CS_TO_PLAN_VOL_CODE`, 
    @`term_date`)
    SET `DATE_ANNUL`=IF(@`term_date`>0, date_add('1800-12-28', INTERVAL REPLACE(@`term_date`,',','') DAY), NULL),
        `CODE`= TRIM(REPLACE(@`code_val`,',',''));
    
LOAD DATA LOCAL INFILE 'e_couses.txt'
    INTO TABLE `LOFOMS`.`E_COUSES`
    CHARACTER SET cp1251
    LINES TERMINATED BY '\r\n';

LOAD DATA LOCAL INFILE 'man.txt'
    INTO TABLE `LOFOMS`.`MAN`
    CHARACTER SET cp1251
    LINES TERMINATED BY '\r\n'
    (`id`, @`n_val`, `NAME`)
    SET `N` = TRIM(REPLACE(@`n_val`,',',''));

LOAD DATA LOCAL INFILE 's_comp.txt'
    INTO TABLE `LOFOMS`.`S_COMP`
    CHARACTER SET cp1251
    LINES TERMINATED BY '\r\n'
    (`id`, @`n_val`, `PRIM`, `IIN`, `N_REG`, @`days_reg`, `N_USTAV`, @`days_u`, `BANK1`, 
     `BANK2`, `ADDRES_U1`, `ADDRES_U2`, `MFO`, `SCHETR`, `SCHETK`, `N_DOG`, 
     @`days_z`, `N_LIC`, @`l_duration`, `FIOD`, `IM`, `OTCH`, `PHONE`, `KOMM`, 
     `NUMBER_FEO_LO`, `NUMBER_FEO_SPB`)
    SET `DATA_REG`=IF(@`days_reg`>0, date_add('1800-12-28', INTERVAL REPLACE(@`days_reg`,',','') DAY), NULL),
        `DATA_U`=IF(@`days_u`>0, date_add('1800-12-28', INTERVAL REPLACE(@`days_u`,',','') DAY), NULL),
        `DATA_Z`=IF(@`days_z`>0, date_add('1800-12-28', INTERVAL REPLACE(@`days_z`,',','') DAY), NULL),
        `SROK_LIC`=IF(@`l_duration`>0, date_add('1800-12-28', INTERVAL REPLACE(@`l_duration`,',','') DAY), NULL),
        `N` = TRIM(REPLACE(@`n_val`,',',''));
    
LOAD DATA LOCAL INFILE 's_fil.txt'
    INTO TABLE `LOFOMS`.`S_FIL`
    CHARACTER SET cp1251
    LINES TERMINATED BY '\r\n';

LOAD DATA LOCAL INFILE 's_fil_l.txt'
    INTO TABLE `LOFOMS`.`S_FIL_L`
    CHARACTER SET cp1251
    LINES TERMINATED BY '\r\n';

LOAD DATA LOCAL INFILE 's_grup.txt'
    INTO TABLE `LOFOMS`.`S_GRUP`
    CHARACTER SET cp1251
    LINES TERMINATED BY '\r\n';

LOAD DATA LOCAL INFILE 's_mun.txt'
    INTO TABLE `LOFOMS`.`S_MUN`
    CHARACTER SET cp1251
    LINES TERMINATED BY '\r\n'
    (`id`,`N`, `PRIM`, `ADDRES_F1`, `ADDRES_F2`, `PHONE`, `COMPANI`, 
    @`dt`, `OLD_COMPANI`, `CODE_FEO`, `FIL_CODE`)
    SET `data`=IF(@`dt`>0, date_add('1800-12-28', INTERVAL REPLACE(@`dt`,',','') DAY), NULL);

LOAD DATA LOCAL INFILE 's_terr.txt'
    INTO TABLE `LOFOMS`.`S_TERR`
    CHARACTER SET cp1251
    LINES TERMINATED BY '\r\n';

LOAD DATA LOCAL INFILE 'tr_types.txt'
    INTO TABLE `LOFOMS`.`TR_TYPES`
    CHARACTER SET cp1251
    LINES TERMINATED BY '\r\n';

LOAD DATA LOCAL INFILE 'npunkt.txt'
    INTO TABLE `LOFOMS`.`NPUNKT`
    CHARACTER SET cp1251
    LINES TERMINATED BY '\r\n';

-- этот справочник импортируется из dbf->txt исчточника, столбец id пропускаем
LOAD DATA LOCAL INFILE 'oper.txt'
    INTO TABLE `LOFOMS`.`OPER`
    CHARACTER SET cp1251
    LINES TERMINATED BY '\r\n'
    IGNORE 1 LINES
    (`CODE`, `DESCRIPT`, `CATEGORY`, `CODE_MVK`, `AMB_STAC`);
