# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
u"""Импорт организаций из XML"""

import csv
import os

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import Qt, pyqtSignature, SIGNAL, QString, QDate

from library.Utils import forceRef, forceString, toVariant, forceInt
from Registry.Utils import selectLatestRecord

from XmlReader import (COrgXmlStreamReader, CServiceXmlStreamReader,
    CFederalOrgXmlStreamReader)
from Ui_Progress import Ui_ProgressDialog


class CProgressDialog(QtGui.QDialog, Ui_ProgressDialog):
    __pyqtSignals__ = ('startWork()', )

    importOrgs = 0
    importClients = 1
    importServices = 2
    importFederalOrgs = 3

    #СНИЛСВРАЧА,ФАМИЛИЯ,ИМЯ,ОТЧЕСТВО,ДР,ПОЛИС,АДРЕСРЕГ,АР_ОКАТО,АР_ОБЛ,АР_РАЙОН,АР_НП1,АР_НП2,АР_УЛИЦА,АР_ДОМ,АР_К,АР_КВ,АДРЕСФАКТ,АФ_ОКАТО,АФ_ОБЛ,АФ_РАЙОН,АФ_НП1,АФ_НП2,АФ_УЛИЦА,АФ_ДОМ,АФ_К,АФ_КВ
    csvFields = ('SNILS', 'lastName', 'firstName', 'patrName', 'birthDate',
        'policyNumber', 'regAddress', 'regOkato', 'regArea', 'regRegion',
        'regCity1', 'regCity2', 'regStreet', 'regHouse', 'regFlat', 'locAddress')

    decodeFields = ('lastName', 'firstName', 'patrName', 'birthDate',
        'policyNumber', 'regAddress', 'locAddress')

    clientFields = ('lastName', 'firstName', 'patrName', 'birthDate')
    mapMpToIsMedical = {
        1: 1, 11: 1, 12: 1, 13: 1,
        3: 2, 31: 2, 32: 2,
        2: 4, 21: 4, 22: 4,
        4: 3
    }


    def __init__(self, parent, importType=importOrgs):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)
        self.setPlacesCount(0)
        self.setClientsCount(0)
        self._setCount(self.prbOrgs, 0)
        self._setCount(self.prbServices, 0)

        self.start()
        self.connect(self, SIGNAL('startWork()'), self.startWork,
                Qt.QueuedConnection)

        self.OKFSCache = {}
        self._okpfCache = {}
        self._areaCache = {}
        self._parent = parent
        self._importType = importType

        self.nAdded = 0
        self.nProcessed = 0
        self.nUpdated = 0

        self._db = QtGui.qApp.db
        self.tableClient = self._db.table('Client')
        self.tableAttach = self._db.table('ClientAttach')
        self.tableAddress = self._db.table('ClientAddress')

        self.attachOrgId = None
        self.attachTypeId = None

        self.medicalAidKindCache = {}
        self.medicalAidProfileCache = {}


    def _setCount(self, progressBar, cnt):
        if cnt:
            progressBar.setValue(0)
            progressBar.setMaximum(cnt)
            progressBar.setEnabled(True)
            progressBar.setTextVisible(True)
        else:
            progressBar.setValue(0)
            progressBar.setMaximum(1)
            progressBar.setEnabled(False)
            progressBar.setTextVisible(False)


    def setPlacesCount(self, cnt):
        self._setCount(self.prbPlaces, cnt)


    def setClientsCount(self, cnt):
        self._setCount(self.prbClients, cnt)


    def start(self):
        self.processDone = False
        self.aborted = False
        self.btnCancel.setText(u'Отмена')


    def stop(self):
        self.processDone = True
        self.btnCancel.setText(u'Закрыть')


    def done(self, r):
        self.stop()
        QtGui.QDialog.done(self, r)

    def exec_(self):
        self.emit(SIGNAL('startWork()'))

        return QtGui.QDialog.exec_(self)


    @pyqtSignature('')
    def on_btnCancel_clicked(self):
        if self.processDone:
            self.accept()
        else:
            self.aborted = True


    def startWork(self):
        self.start()
        QtGui.qApp.call(self, self.work)
        self.stop()


    def work(self):
        if self._importType == self.importOrgs:
            self.processXml(self.prbOrgs)
        elif self._importType == self.importClients:
            self.processCsv(self.prbClients)
        elif self._importType == self.importServices:
            self.processServices(self.prbServices)
        elif self._importType == self.importFederalOrgs:
            self.processXml(self.prbOrgs, True)


    def process(self, query, step, progressBar):
        while query.next():
            QtGui.qApp.processEvents()
            if self.aborted:
                self.reject()
                return
            progressBar.setValue(progressBar.value()+1)
            QtGui.qApp.db.transaction()
            try:
                step(query.record())
                QtGui.qApp.db.commit()
            except:
                QtGui.qApp.db.rollback()
                raise

# ******************************************************************************

    def findOrgId(self, inn, kpp):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecordEx(table, 'id', [table['deleted'].eq(0),
                                                          table['INN'].eq(inn),
                                                          table['KPP'].eq(kpp)
                                                         ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None


    def addOrg(self, shortName, fullName, INN, KPP, OKFS, OKVED, OGRN,
               address, chief, phone, fax=None, email=None, smoCode=None,
               okpfId=None, isInsurer=False, area=None, okato=None,
               compulsoryServiceStop=False, isActive=True, www=None,
               isMedical=-1):
        table = QtGui.qApp.db.table('Organisation')
        record = table.newRecord()

        record.setValue('fullName', toVariant(fullName))
        record.setValue('shortName', toVariant(shortName))
        record.setValue('title', toVariant(shortName))
        record.setValue('OKVED', toVariant(OKVED))
        record.setValue('INN', toVariant(INN))
        record.setValue('KPP', toVariant(KPP))
        record.setValue('OGRN', toVariant(OGRN))
        record.setValue('OKFS_id', toVariant(self.getOKFSId(OKFS)))
        record.setValue('address', toVariant(address))
        record.setValue('chief', toVariant(chief))
        record.setValue('phone', toVariant(phone))

        notes = u''

        if fax:
            notes += u'факс: %s' % fax

        if email:
            notes += u' email: %s' % email

        if www:
            notes += u' url: %s' % www

        if notes != '':
            record.setValue('notes', toVariant(notes))

        if smoCode:
            record.setValue('smoCode', toVariant(smoCode))
            record.setValue('infisCode', toVariant(smoCode))

        if okpfId:
            record.setValue('OKPF_id', toVariant(okpfId))

        if isInsurer:
            record.setValue('isInsurer', toVariant(isInsurer))

        if area:
            record.setValue('area', toVariant(area))

        if okato:
            record.setValue('OKATO', toVariant(okato))

        if compulsoryServiceStop:
            record.setValue('compulsoryServiceStop', toVariant(True))

        if isActive:
            record.setValue('isActive', toVariant(True))

        if isMedical != -1:
            record.setValue('isMedical', toVariant(isMedical))

        orgId = QtGui.qApp.db.insertRecord(table, record)
        return orgId


    def updateOrg(self, orgId, shortName, fullName, OKFS, OKVED, OGRN,
                  address, chief, phone, fax=None, email=None, smoCode=None,
                  okpfId=None, isInsurer=False, area=None, okato=None,
                  compulsoryServiceStop=False, isActive=True, www=None,
                  isMedical=-1):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecord(table, '*', orgId)
        record.setValue('fullName', toVariant(fullName))
        record.setValue('shortName', toVariant(shortName))
        record.setValue('title', toVariant(shortName))
        record.setValue('OKVED', toVariant(OKVED))
        record.setValue('OGRN', toVariant(OGRN))
        record.setValue('OKFS_id', toVariant(self.getOKFSId(OKFS)))
        record.setValue('address', toVariant(address))
        record.setValue('chief', toVariant(chief))
        record.setValue('phone', toVariant(phone))
        notes = u''

        if fax:
            notes += u'факс: %s' % fax

        if email:
            notes += u' email: %s' % email

        if www:
            notes += u' url: %s' % www

        if notes != '':
            record.setValue('notes', toVariant(notes))

        if smoCode:
            record.setValue('smoCode', toVariant(smoCode))
            record.setValue('infisCode', toVariant(smoCode))

        if okpfId:
            record.setValue('OKPF_id', toVariant(okpfId))

        record.setValue('isInsurer', toVariant(isInsurer))

        if area:
            record.setValue('area', toVariant(area))

        if okato:
            record.setValue('OKATO', toVariant(okato))

        record.setValue('compulsoryServiceStop',
            toVariant(compulsoryServiceStop))
        record.setValue('isActive', toVariant(isActive))

        if isMedical != -1:
            record.setValue('isMedical', toVariant(isMedical))

        QtGui.qApp.db.updateRecord(table, record)


    def _getOKFSId(self, OKFS):
        return forceRef(QtGui.qApp.db.translate('rbOKFS', 'code', OKFS, 'id'))


    def getOKFSId(self, OKFS):
        if OKFS in self.OKFSCache:
            return self.OKFSCache[OKFS]
        else:
            result = self._getOKFSId(OKFS)
            self.OKFSCache[OKFS] = result
            return result

# ******************************************************************************

    def processXml(self, progressBar, isFederalImport=False):
        u"""Разбирает указанный XML файл, отправляет данные в БД"""
        fileName = forceString(self._parent.edtXmlFileName.text())

        if not fileName:
            return False

        progressBar.setFormat(u'%v байт')
        inFile = QtCore.QFile(fileName)

        if not inFile.open(QtCore.QFile.ReadOnly | QtCore.QFile.Text):
            QtGui.QMessageBox.warning(self, u'Импорт страховых компаний',
                  QString(u'Не могу открыть файл для чтения %1:\n%2.')
                  .arg(fileName)
                  .arg(inFile.errorString()))
            return False

        progressBar.setMaximum(max(inFile.size(), 1))
        readerType = CFederalOrgXmlStreamReader if isFederalImport else COrgXmlStreamReader
        reader = readerType(self, self.logBrowser, True)
        reader.setDevice(inFile)
        params = {
            'currentDate': QDate.currentDate(),
            'isFederalImport': isFederalImport,
            'shortName': 'NAM_MOK' if isFederalImport else 'NAM_SMOK',
            'fullName': 'NAM_MOP' if isFederalImport else 'NAM_SMOP',
            'infisCode': 'MCOD' if isFederalImport else 'SMOCOD',
            'address': 'ADDR_J' if isFederalImport else 'ADDR_F',
        }

        if reader.readHeader():
            for org in reader.readData():
                progressBar.setValue(inFile.pos())
                QtGui.qApp.processEvents()

                if org:
                    self.processXmlOrg(org, params)

                if self.aborted:
                    break

        self.log(u'импорт элементов: %d добавлено; %d обновлено;'
            u' %d обработано' % (
            self.nAdded, self.nUpdated, self.nProcessed))

        if not (reader.hasError() or self.aborted):
            self.log(u'Готово')
            return True
        else:
            self.log(u'Прервано')

            if self.aborted:
                self.log(u'! Прервано пользователем.')
            else:
                self.log(u'! Ошибка: файл %s, %s' % (fileName,
                    reader.errorString()))

            return False


    def log(self, msg):
        self.logBrowser.append(msg)
        self.logBrowser.update()


    def processXmlOrg(self, org, params):
        self.nProcessed += 1
        inn = org.get('INN', '')
        kpp = org.get('KPP', '')
        _id = self.findOrgId(inn, kpp)

        shortName = org.get(params['shortName'], '')
        fullName = org.get(params['fullName'], '')
        address = org.get(params['address'], '')
        chief = u' '.join([org.get('FAM_RUK', ''), org.get('IM_RUK', ''),
            org.get('OT_RUK', '')])
        phone = org.get('PHONE', '')
        fax = org.get('FAX', '')
        email = org.get('E_MAIL', '')
        www = org.get('WWW', '')
        infisCode = org.get(params['infisCode'], '')
        ogrn = org.get('OGRN', '')
        okpfId = self.getOkpfId(org.get('OKOPF', ''))
        okato = org.get('TF_OKATO', '')
        area = self.getArea(org.get('TF_OKATO', '').ljust(11, '0'))
        compulsoryServiceStop = forceInt(org.get('IS_DELETED', 0)) == 1
        endDate = QDate.fromString(org.get('ENDDATE', ''), QtCore.Qt.ISODate)
        isActive = not (compulsoryServiceStop or
                (endDate and (endDate < params['currentDate'])))

        isMedical = (self.mapMpToIsMedical.get(forceInt(org.get('MP', 0)))
            if params['isFederalImport'] else None)
        isInsurer = not params['isFederalImport']

        if _id:
            self.log(u'* Обновляем: %s (id=%d)' % (
               org.get(params['shortName'], '-'), _id))
            self.updateOrg(_id, shortName, fullName, '', '', ogrn,
                address, chief, phone, fax, email,
                infisCode, okpfId, isInsurer, area, okato, compulsoryServiceStop,
                isActive, www, isMedical)
            self.nUpdated += 1
        elif isActive:
            self.log(u'+ Добавляем: %s' % (
               org.get(params['shortName'], '-')))
            self.addOrg(shortName, fullName, inn, kpp, '', '', ogrn,
                address, chief, phone, fax, email,
                infisCode, okpfId, isInsurer, area, okato, compulsoryServiceStop,
                isActive, www, isMedical)
            self.nAdded += 1


    def getOkpfId(self, code):
        result = self._okpfCache.get(code, -1)

        if result == -1:
            result = forceRef(QtGui.qApp.db.translate(
                    'rbOKPF', 'code', code, 'id'))
            self._okpfCache[code] = result

        return result


    def getArea(self, okato):
        result = self._areaCache.get(okato, -1)

        if result == -1:
            result = forceString(QtGui.qApp.db.translate(
                    'kladr.KLADR', 'OCATD', okato, 'CODE'))
            self._areaCache[okato] = result

        return result

# ******************************************************************************

    def processCsv(self, progressBar):
        u"""Разбирает указанный CSV файл, отправляет данные в БД"""
        fileName = forceString(self._parent.edtCsvFileName.text())
        progressBar.setMaximum(os.path.getsize(fileName))
        progressBar.setFormat('%p%')
        progressBar.setTextVisible(True)
        progressBar.setEnabled(True)

        self.attachOrgId = self._parent.cmbAttachOrg.value()
        self.attachTypeId = self._parent.cmbAttachType.value()
        self.log(u'Импорт данных пациентов из `%s`' % fileName)

        with open(fileName) as csvfile:
            reader = csv.DictReader(csvfile, self.csvFields, 'unused')
            reader.next() # пропускаем заголовок

            for row in reader:
                progressBar.setValue(csvfile.tell())
                self.processClient(row)
                QtGui.qApp.processEvents()

        self.log(u'Импорт завершен')
        progressBar.setMaximum(1)
        progressBar.setValue(1)


    def processClient(self, row):
        u"""Обрабатывает данные пациента"""
        for i in self.decodeFields:
            row[i] = row[i].decode('utf-8').strip()

        row['birthDate'] = QDate.fromString(row['birthDate'], 'dd.MM.yyyy')
        clientId = self.findClient(row)

        if not clientId:
            clientId = self.addClient(row)

        self.ensureClientAttach(clientId, self.attachTypeId, self.attachOrgId)
        self.ensureClientAddress(clientId, 0, row['regAddress'])
        self.ensureClientAddress(clientId, 1, row['locAddress'])


    def findClient(self, row):
        u"""Получает идентификатор пациента.
            Список полей для поиска - clientFields"""
        cond = [self.tableClient['deleted'].eq(0), ]

        for i in self.clientFields:
            cond.append(self.tableClient[i].eq(row[i]))

        record = self._db.getRecordEx(self.tableClient, 'id', cond, 'id')
        result = None

        if record:
            result = forceRef(record.value(0))

        return result


    def addClient(self, row):
        u"""Добавляет пациента в БД"""
        record = self.tableClient.newRecord()

        for i in self.clientFields:
            record.setValue(i,  toVariant(row[i]))

        clientId = self._db.insertRecord(self.tableClient, record)
        return clientId


    def ensureClientAttach(self, clientId, attachTypeId, orgId):
        u"""Проверяет наличие прикрепления пациента и при
            отсутствии создает новое"""
        if not orgId or not attachTypeId:
            return

        record = selectLatestRecord('ClientAttach', clientId,
            '(Tmp.`attachType_id`=%d) AND (Tmp.`LPU_id`=%d)' % (
                                                        attachTypeId, orgId))

        if not record:
            record = self.tableAttach.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('attachType_id',  toVariant(attachTypeId))
            record.setValue('LPU_id',toVariant(orgId))
            self._db.insertRecord(self.tableAttach, record)


    def ensureClientAddress(self, clientId, addressType, freeInput):
        u"""Проверяет адрес пациента и при отсутсвии создает новый"""
        table = self.tableAddress
        record = self._db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                  table['client_id'].eq(clientId),
                                                  table['type'].eq(addressType),
                                                  table['freeInput'].eq(freeInput),
                                                 ], 'id')
        if not record:
            record = table.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('type', toVariant(addressType))
            record.setValue('freeInput', toVariant(freeInput))
            self._db.insertRecord(table, record)

# ******************************************************************************

    def processServices(self, progressBar):
        u"""Разбирает указанный XML файл, отправляет данные в БД"""
        fileName = forceString(self._parent.edtServiceXmlFileName.text())

        if not fileName:
            return False

        progressBar.setFormat(u'%v байт')
        inFile = QtCore.QFile(fileName)

        if not inFile.open(QtCore.QFile.ReadOnly | QtCore.QFile.Text):
            QtGui.QMessageBox.warning(self, u'Импорт услуг',
                  QString(u'Не могу открыть файл для чтения %1:\n%2.')
                  .arg(fileName)
                  .arg(inFile.errorString()))
            return False

        progressBar.setMaximum(max(inFile.size(), 1))
        reader = CServiceXmlStreamReader(self, self.logBrowser, True)
        reader.setDevice(inFile)
        params = {
            'groupId': self._parent.cmbServiceGroup.value(),
            'orgLevel': forceString(self._parent.edtOrgLevel.value()),
            'orgInfis': forceString(self._parent.edtOrgInfis.value())
        }

        if reader.readHeader():
            for row in reader.readData():
                progressBar.setValue(inFile.pos())
                QtGui.qApp.processEvents()

                if row:
                    self.processService(row, params)

                if self.aborted:
                    break

        self.log(u'импорт элементов: %d добавлено; %d обновлено;'
            u' %d обработано' % (
            self.nAdded, self.nUpdated, self.nProcessed))

        if not (reader.hasError() or self.aborted):
            self.log(u'Готово')
            return True
        else:
            self.log(u'Прервано')

            if self.aborted:
                self.log(u'! Прервано пользователем.')
            else:
                self.log(u'! Ошибка: файл %s, %s' % (fileName,
                    reader.errorString()))

            return False


    def processService(self, row, params):
        self.nProcessed += 1

        if (params['orgLevel'] != row['LPU_LEVEL']
            or params['orgInfis'] != row['AMB_CODE']):
            return

        code = row.get('TARIF_CODE')
        _id = self.findServiceByCode(code)
        name = row.get('TITLE')
        begDate = row.get('STARTDATE', QDate())
        endDate = row.get('ENDDATE', QDate())
        medicalAidProfileId = self.findMedicalAidProfileByRegionalCode(
            forceInt(code[4:7]))
        medicalAidKindId = self.findMedicalAidKindByRegionalCode(
            self.medicalAidKindRegionalCode(code))

        if _id:
            self.log(u'* Обновляем: %s (id=%d)' % (
               name, _id))
            self.updateService(_id, name, params['groupId'], begDate, endDate,
                medicalAidProfileId, medicalAidKindId)
            self.nUpdated += 1
        else:
            self.log(u'+ Добавляем: %s' % name)
            self.addService(code, name, params['groupId'], begDate, endDate,
                medicalAidProfileId, medicalAidKindId)
            self.nAdded += 1


    def findServiceByCode(self, code):
        table = self._db.table('rbService')
        record = self._db.getRecordEx(table, 'id',
            [table['code'].eq(code)], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None


    def findMedicalAidKindByRegionalCode(self, code):
        result = self.medicalAidKindCache.get(code, -1)

        if result == -1:
            result = forceRef(self._db.translate('rbMedicalAidKind',
                'regionalCode', code, 'id'))
            self.medicalAidKindCache[code] = result

        return result


    def findMedicalAidProfileByRegionalCode(self, code):
        result = self.medicalAidProfileCache.get(code, -1)

        if result == -1:
            result = forceRef(self._db.translate('rbMedicalAidProfile',
                'regionalCode', code, 'id'))
            self.medicalAidProfileCache[code] = result

        return result


    def medicalAidKindRegionalCode(self, code):
        result = 13

        if code[2:4] == '19':
            result = 21
        elif code[2:4] in ('01', '03', '06', '08', '15', '21'):
            result = 31
        elif code[4:7] in ('068', '097', '057'):
            result = 12
        elif code[4:7] in ('042', '003', '085', '058', '087'):
            result = 11

        return result


    def addService(self, code, name, groupId, begDate, endDate,
                medicalAidProfileId, medicalAidKindId):
        table = self._db.table('rbService')
        record = table.newRecord()
        record.setValue('code', toVariant(code))
        record.setValue('name', toVariant(name))
        record.setValue('begDate', toVariant(begDate))
        record.setValue('endDate', toVariant(endDate))
        record.setValue('group_id', toVariant(groupId))
        record.setValue('medicalAidProfile_id', toVariant(medicalAidProfileId))
        record.setValue('medicalAidKind_id', toVariant(medicalAidKindId))
        return QtGui.qApp.db.insertRecord(table, record)


    def updateService(self, serviceId, name, groupId, begDate, endDate,
                medicalAidProfileId, medicalAidKindId):
        table = self._db.table('rbService')
        record = self._db.getRecord(table, '*', serviceId)
        record.setValue('name', toVariant(name))
        record.setValue('begDate', toVariant(begDate))
        record.setValue('endDate', toVariant(endDate))
        record.setValue('group_id', toVariant(groupId))
        record.setValue('medicalAidProfile_id', toVariant(medicalAidProfileId))
        record.setValue('medicalAidKind_id', toVariant(medicalAidKindId))
        return QtGui.qApp.db.updateRecord(table, record)
