# -*- coding: utf-8 -*-

#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################


from PyQt4 import QtGui
from PyQt4.QtCore import QXmlStreamReader

from library.Utils import forceString, anyToUnicode

# ******************************************************************************

class CCommonXmlStreamReader(QXmlStreamReader):
    def __init__(self, parent, logBrowser, showLog=False):
        QXmlStreamReader.__init__(self)
        self._parent = parent
        self.db = QtGui.qApp.db
        self.logBrowser = logBrowser
        self.showLog = showLog

    def raiseError(self, msg):
        QXmlStreamReader.raiseError(self, u'[%d:%d] %s' % (
            self.lineNumber(), self.columnNumber(), msg))


    def log(self, msg, forceLog=False):
        if self.showLog or forceLog:
            self.logBrowser.append(msg)
            self.logBrowser.update()


    def readHeader(self):
        u''' Разбирает заголовок'''

        try:
            while not self.atEnd():
                self.readNext()

                if self.isStartElement():
                    if self.name() == 'DATA':
                            return True
                    else:
                        self.raiseError(u'Неверный формат экспорта данных.')

        except IOError, e:
            QtGui.qApp.logCurrentException()
            msg = ''
            if hasattr(e, 'filename'):
                msg = u'%s:\n[Errno %s] %s' % (e.filename, e.errno,
                                               anyToUnicode(e.strerror))
            else:
                msg = u'[Errno %s] %s' % (e.errno, anyToUnicode(e.strerror))
            QtGui.QMessageBox.critical(self._parent,
                u'Произошла ошибка ввода-вывода', msg, QtGui.QMessageBox.Close)
            self.log(u'! Ошибка ввода-вывода: %s' % msg, True)

        except Exception, e:
            QtGui.qApp.logCurrentException()
            QtGui.QMessageBox.critical(self._parent,
                u'Произошла ошибка', unicode(e), QtGui.QMessageBox.Close)
            self.log(u'! Ошибка: %s' % unicode(e), True)

        return False


    def readData(self):
        assert self.isStartElement() and self.name() == 'DATA'

        # очищаем список событий перед заполнением

        while not self.atEnd():
            QtGui.qApp.processEvents()
            self.readNext()

            if self.isEndElement():
                break

            if self.isStartElement():
                if self.name() == 'ROW':
                    yield self.readRow()
                else:
                    self.readUnknownElement()

            if self.hasError() or self._parent.aborted:
                break


    def readValue(self):
        assert self.isStartElement()
        result = None

        if self._parent.aborted:
            return None

        while not self.atEnd():
            self.readNext()

            if self.isEndElement():
                break

            if self.isStartElement():
                name = forceString(self.name())
                if name == 'VALUE':
                    result = forceString(self.readElementText())
                else:
                    self.readUnknownElement()

        return result


    def readUnknownElement(self, report=True):
        """ Читает неизвестный элемент, и сообщает об этом,
            если report ==True """

        assert self.isStartElement()

        if report:
            self.log(u'Неизвестный элемент: '+self.name().toString())

        while not self.atEnd():
            self.readNext()

            if self.isEndElement():
                break

            if self.isStartElement():
                self.readUnknownElement(report)

            if self.hasError() or self._parent.aborted:
                break

# ******************************************************************************

class COrgXmlStreamReader(CCommonXmlStreamReader):
    insurerFields = ('TF_OKATO', 'SMOCOD', 'NAM_SMOP', 'NAM_SMOK', 'INN',
        'OGRN', 'KPP', 'INDEX_J', 'ADDR_J', 'INDEX_F', 'ADDR_F', 'OKOPF',
        'FAM_RUK', 'IM_RUK', 'OT_RUK', 'PHONE', 'FAX', 'E_MAIL', 'IS_DELETED' )
    dateFields = ('ENDDATE', )

    ignoredFields = ('STARTDATE', 'WWW', 'LICENZIY', 'N_DOC',
        'D_START', 'DATA_E', 'ORG', 'D_BEGIN', 'D_END', 'NAME_E', 'NAL_P',
        'DUVED', 'KOL_ZL', 'D_EDIT', 'YEAR_WORK')

    def __init__(self, parent, logBrowser, showLog=False):
        CCommonXmlStreamReader.__init__(self, parent, logBrowser, showLog)


    def readRow(self):
        assert self.isStartElement() and self.name() == 'ROW'

        result = {}

        if self._parent.aborted:
            return None

        while not self.atEnd():
            self.readNext()

            if self.isEndElement():
                break

            if self.isStartElement():
                name = forceString(self.name())
                if name in self.insurerFields:
                    result[name] = forceString(self.readValue())
                elif name in self.dateFields:
                    result[name] = forceString(self.readElementText())
                elif name in self.ignoredFields:
                    # игноруемые параметры,
                    # не сообщаем в журнал об ошибках
                    self.readUnknownElement(False)
                else:
                    self.readUnknownElement()

        return result


# ******************************************************************************

class CServiceXmlStreamReader(CCommonXmlStreamReader):
    serviceFields = ('TITLE', 'TARIF_CODE', 'LPU_LEVEL', 'AMB_CODE')
    dateFields = ('STARTDATE', 'ENDDATE')

    ignoredFields = ('SUMM', 'AVG_CASE_PERIOD', 'MIN_CASE_PERIOD', 'PRVS_CODE',
        'IDSP_CODE', 'USL_MP', 'UCH_SL')

    def __init__(self, parent, logBrowser, showLog=False):
        CCommonXmlStreamReader.__init__(self, parent, logBrowser, showLog)


    def readRow(self):
        assert self.isStartElement() and self.name() == 'ROW'

        result = {}

        if self._parent.aborted:
            return None

        while not self.atEnd():
            self.readNext()

            if self.isEndElement():
                break

            if self.isStartElement():
                name = forceString(self.name())
                if name in self.serviceFields:
                    result[name] = forceString(self.readValue())
                elif name in self.dateFields:
                    result[name] = forceString(self.readElementText())
                elif name in self.ignoredFields:
                    # игноруемые параметры,
                    # не сообщаем в журнал об ошибках
                    self.readUnknownElement(False)
                else:
                    self.readUnknownElement()

        return result

# ******************************************************************************

class CFederalOrgXmlStreamReader(COrgXmlStreamReader):
    insurerFields = ('TF_OKATO', 'MCOD', 'NAM_MOP', 'NAM_MOK', 'INN',
        'OGRN', 'KPP', 'ADDR_J', 'OKOPF', 'FAM_RUK', 'IM_RUK', 'OT_RUK',
        'PHONE', 'FAX', 'E_MAIL', 'WWW', 'MP', 'IS_DELETED' )
    dateFields = ('ENDDATE', )

    ignoredFields = ('STARTDATE', 'VEDPRI', 'N_DOC', 'INDEX_J', 'DOC',
        'D_START', 'DATA_E', 'ORG', 'D_BEGIN', 'D_END', 'NAME_E', 'UCH_SL',
        'DUVED', 'D_TERM', 'D_EDIT', 'YEAR_WORK')

    def __init__(self, parent, logBrowser, showLog=False):
        COrgXmlStreamReader.__init__(self, parent, logBrowser, showLog)
