# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import os.path
import shutil

from PyQt4 import QtCore, QtGui


from library.dbfpy.dbf import Dbf
from library.Utils     import (forceDate, forceInt, forceString, pyDate,
                               formatSex, toVariant, forceStringEx)

from Ui_ExportAttachPage1 import Ui_ExportPage1
from Ui_ExportAttachPage2 import Ui_ExportPage2


def exportAttach(widget, onlyCurrentLPU):
    wizard = CExportWizard(widget, onlyCurrentLPU)
    wizard.exec_()


class CExportWizard(QtGui.QWizard):
    def __init__(self, parent, onlyCurrentLPU):
        QtGui.QWizard.__init__(self, parent)
        self.setWizardStyle(QtGui.QWizard.ModernStyle)
        self.page1 = CExportPage1(self)
        self.page2 = CExportPage2(self)
        self.addPage(self.page1)
        self.addPage(self.page2)
        self.setWindowTitle(u'Мастер экспорта прикреплений для Мурманской области')
        self.page1.setTitle(u'Экспорт прикреплений')
        self.page2.setTitle(u'Укажите директорий для сохранения обменных файлов "*.dbf"')
        self.dbfFileName = ''
        self.tmpDir = ''
        self.onlyCurrentLPU = onlyCurrentLPU


    def getTmpDir(self):
        if not self.tmpDir:
            self.tmpDir = QtGui.qApp.getTmpDir('R51Attach')
        return self.tmpDir


    def getFullDbfFileName(self):
        return os.path.join(self.getTmpDir(), self.dbfFileName + '.dbf')


    def setAccountItemsIdList(self, accountItemIdList):
        self.page1.setAccountItemsIdList(accountItemIdList)


    def cleanup(self):
        if self.tmpDir:
            QtGui.qApp.removeTmpDir(self.tmpDir)
            self.tmpDir = ''


    def exec_(self):
        QtGui.QWizard.exec_(self)
        self.cleanup()


# *****************************************************************************************


class CExportPage1(QtGui.QWizardPage, Ui_ExportPage1):
    def __init__(self, parent):
        QtGui.QWizardPage.__init__(self, parent)
        self.setupUi(self)
        self.progressBar.setMinimum(0)
        self.progressBar.setMaximum(1)
        self.progressBar.setText('')

        self.setSubTitle(u'для выполнения шага нажмите кнопку "вперед"')

        self.setExportMode(False)
        self.aborted = False
        self.done = False
        self._parent = parent
        self.idList = None
        self.num = 0
        self.db = QtGui.qApp.db

        self.connect(parent, QtCore.SIGNAL('rejected()'), self.abort)


    def validatePage(self):
        return self.done


    def setExportMode(self, flag):
        self.btnCancel.setEnabled(flag)


    def log(self, _str, forceLog=False):
        if self.chkVerboseLog.isChecked() or forceLog:
            self.logBrowser.append(_str)
            self.logBrowser.update()


    def setAccountItemsIdList(self, accountItemIdList):
        self.idList = accountItemIdList


    def prepareToExport(self):
        self.done = False
        self.aborted = False
        self.emit(QtCore.SIGNAL('completeChanged()'))
        self.setExportMode(True)
        dbf = self.createDbf()
        self.progressBar.reset()
        self.progressBar.setMaximum(1)
        self.progressBar.setValue(0)
        self.progressBar.setText(u'Запрос в БД...')
        QtGui.qApp.processEvents()
        query = self.createQuery()
        self.progressBar.setMaximum(max(query.size(), 1))
        self.progressBar.reset()
        self.progressBar.setValue(0)
        return dbf, query


    def export(self):
        (result, _) = QtGui.qApp.call(self, self.exportInt)
        self.setExportMode(False)

        if self.aborted or not result:
            self.progressBar.setText(u'прервано')
        else:
            self.progressBar.setText(u'готово')
            self.done = True
            self.emit(QtCore.SIGNAL('completeChanged()'))


    def getDbfBaseName(self):
        lpuCode = forceString(self.db.translate(
            'Organisation', 'id', QtGui.qApp.currentOrgId(), 'infisCode'))
        return forceString(u'FM%s.DBF' % lpuCode)


# *****************************************************************************************

    def exportInt(self):
        dbf, query = self.prepareToExport()
        self.num = 0

        while query.next():
            QtGui.qApp.processEvents()
            if self.aborted:
                break
            self.progressBar.step()
            self.process(dbf, query.record())

        dbf.close()


    def createDbf(self):
        dbfName = os.path.join(self._parent.getTmpDir(), self.getDbfBaseName())
        dbf = Dbf(dbfName, new=True, encoding='cp866')
        dbf.addField(
            ('CLIENTID', 'N', 11),  # client_id
            ('CODE_HOSP', 'C', 3),   # Код МО
            ('NREC', 'I'),  # Номер записи в файле
            ('VPOLIS', 'C', 1),   # Тип полиса ОМС
            ('SPOLIS', 'C', 10),   # Серия полиса ОМС
            ('NPOLIS', 'C', 20),   #  Номер полиса ОМС
            ('DOCTYPE', 'N', 2),   # Тип документа, удостоверяющего личность
            ('DOCSER', 'C', 10),   #  Серия документа, удостоверяющего личность
            ('DOCNUM', 'C', 20),   # Номер документа, удостоверяющего личность
            ('SNILS', 'C', 11),   #  СНИЛС ПФ РФ без символов-разделителей
            ('FAM', 'C', 40),   # Фамилия
            ('IM', 'C', 40),   # Имя
            ('OT', 'C', 40),   # Отчество
            ('DR', 'D'),  # Дата рождения
            ('SEX', 'C', 1),   # Пол М – муж, Ж - жен
            ('V_PR', 'C', 1),   # Вид прикрепления
            ('DPR_P', 'D'), # Дата заявления застрахованного лица о прикреплении к поликлинике
            ('DPR_K', 'D'),  # Дата заявления застрахованного лица о прикреплении к женской консультации
            ('DPR_S', 'D'), # Дата заявления застрахованного лица о прикреплении к стоматологической поликлинике
            ('KNPN', 'C', 3),   # Код населенного пункта адреса проживания застрахованного лица
            ('GOROD', 'C', 40),   # Название населенного пункта адреса проживания застрахованного лица
            ('KUL', 'C', 4),   # Код улицы адреса проживания застрахованного лица
            ('ULICA', 'C', 40),   # Название улицы адреса проживания застрахованного лица
            ('DOM', 'C', 10),   # Номер дома адреса проживания застрахованного лица*
            ('KV', 'C', 5),   # Номер квартиры адреса проживания застрахованного лица
            )
        return dbf


    def createQuery(self):
        lpuFilter = ('AND ClientAttach.LPU_id = %d' % (QtGui.qApp.currentOrgId())) if self._parent.onlyCurrentLPU else ''

        stmt = """SELECT Client.lastName AS lastName,
            Client.firstName AS firstName,
            Client.patrName AS patrName,
            Client.birthDate AS birthDate,
            Client.sex AS sex,
            Client.SNILS AS SNILS,
            Client.id AS clientId,
            ClientDocument.serial  AS documentSerial,
            ClientDocument.number  AS documentNumber,
            ClientPolicy.serial AS policySerial,
            ClientPolicy.number AS policyNumber,
            rbPolicyKind.regionalCode AS policyKindCode,
            rbDocumentType.regionalCode AS documentRegionalCode,
            RegAddressHouse.number AS regNumber,
            RegAddress.flat AS regFlat,
            RegKLADR.infis AS regPlaceCode,
            RegKLADR.NAME AS regPlaceName,
            RegStreet.infis AS regStreetCode,
            RegStreet.NAME AS regStreetName,
            LocAddressHouse.KLADRCode AS locKLADRCode,
            LocKLADR.infis AS locPlaceCode,
            LocKLADR.NAME AS locPlaceName,
            LocStreet.infis AS locStreetCode,
            LocStreet.NAME AS locStreetName,
            LocAddressHouse.number AS locNumber,
            LocAddress.flat AS locFlat,
            LPU.infisCode AS lpuInfis,
            ClientAttach.begDate,
            rbAttachType.regionalCode AS attachTypeCode
            FROM ClientAttach
            LEFT JOIN Client ON Client.id = ClientAttach.client_id
            LEFT JOIN Organisation AS LPU ON LPU.id = ClientAttach.LPU_id
            LEFT JOIN ClientPolicy ON ClientPolicy.client_id = Client.id AND
                      ClientPolicy.id = (SELECT MAX(CP.id)
                                         FROM   ClientPolicy AS CP
                                         LEFT JOIN rbPolicyType AS CPT ON CPT.id = CP.policyType_id
                                         WHERE  CP.client_id = Client.id AND CP.deleted=0 AND CPT.code IN ('1','2'))
            LEFT JOIN rbPolicyKind ON ClientPolicy.policyKind_id = rbPolicyKind.id
            LEFT JOIN ClientDocument ON ClientDocument.client_id = Client.id AND
                      ClientDocument.id = (SELECT MAX(CD.id)
                                         FROM   ClientDocument AS CD
                                         LEFT JOIN rbDocumentType AS rbDT ON rbDT.id = CD.documentType_id
                                         LEFT JOIN rbDocumentTypeGroup AS rbDTG ON rbDTG.id = rbDT.group_id
                                         WHERE  rbDTG.code = '1' AND CD.client_id = Client.id AND CD.deleted=0)
            LEFT JOIN rbDocumentType ON rbDocumentType.id = ClientDocument.documentType_id
            LEFT JOIN ClientAddress ClientRegAddress ON ClientRegAddress.client_id = Client.id AND
                      ClientRegAddress.id = (SELECT MAX(CRA.id)
                                         FROM   ClientAddress AS CRA
                                         WHERE  CRA.type = 0 AND CRA.client_id = Client.id AND CRA.deleted=0)
            LEFT JOIN ClientAddress ClientLocAddress ON ClientLocAddress.client_id = Client.id AND
                      ClientLocAddress.id = (SELECT MAX(CLA.id)
                                         FROM   ClientAddress AS CLA
                                         WHERE  CLA.type = 1 AND CLA.client_id = Client.id AND CLA.deleted=0)
            LEFT JOIN Address   RegAddress ON RegAddress.id = ClientRegAddress.address_id
            LEFT JOIN AddressHouse RegAddressHouse ON RegAddressHouse.id = RegAddress.house_id
            LEFT JOIN Address   LocAddress ON LocAddress.id = ClientLocAddress.address_id
            LEFT JOIN AddressHouse LocAddressHouse ON LocAddressHouse.id = LocAddress.house_id
            LEFT JOIN kladr.KLADR RegKLADR ON RegKLADR.CODE = RegAddressHouse.KLADRCode
            LEFT JOIN kladr.STREET RegStreet ON RegStreet.CODE = RegAddressHouse.KLADRStreetCode
            LEFT JOIN kladr.KLADR LocKLADR ON LocKLADR.CODE = LocAddressHouse.KLADRCode
            LEFT JOIN kladr.STREET LocStreet ON LocStreet.CODE = LocAddressHouse.KLADRStreetCode

            LEFT JOIN rbAttachType ON rbAttachType.id = ClientAttach.attachType_id
            WHERE rbAttachType.temporary = '0'
                AND ClientAttach.deleted = '0' %s""" % lpuFilter
        return self.db.query(stmt)

# *****************************************************************************************

    def process(self, dbf, record):
        row = dbf.newRecord()
        row['CLIENTID'] = forceInt(record.value('clientId'))
        row['CODE_HOSP'] = forceString(record.value('lpuInfis'))
        row['NREC'] = self.num
        row['VPOLIS'] = forceString(record.value('policyKindCode'))   # Тип полиса ОМС
        row['SPOLIS'] = forceString(record.value('policySerial')) # Серия полиса ОМС
        row['NPOLIS'] = forceString(record.value('policyNumber')) #  Номер полиса ОМС
        # Тип документа, удостоверяющего личность
        row['DOCTYPE'] = forceInt(record.value('documentRegionalCode')) % 100
        #  Серия документа, удостоверяющего личность
        row['DOCSER'] = forceString(record.value('documentSerial'))
        # Номер документа, удостоверяющего личность
        row['DOCNUM'] = forceString(record.value('documentNumber'))
        #  СНИЛС ПФ РФ без символов-разделителей
        row['SNILS'] = forceString(record.value('SNILS'))
        row['FAM'] = forceString(record.value('lastName')) # Фамилия
        row['IM'] = forceString(record.value('firstName')) # Имя
        row['OT'] = forceString(record.value('patrName')) # Отчество
        row['DR'] = pyDate(forceDate(record.value('birthDate'))) # Дата рождения
        row['SEX'] = formatSex(forceInt(record.value('sex'))) # Пол М – муж, Ж - жен
        row['V_PR'] = forceString(record.value('attachTypeCode')) # Вид прикрепления
        # Дата заявления застрахованного лица о прикреплении к поликлинике
        row['DPR_P'] = pyDate(forceDate(record.value('begDate')))
        #row['DPR_K'] # Дата заявления застрахованного лица о прикреплении к женской консультации
        #row['DPR_S'] # Дата заявления застрахованного лица о прикреплении к стоматологической поликлинике

        useLocAddr = forceString(record.value('locKLADRCode')) != ''

        # Код населенного пункта адреса проживания застрахованного лица
        row['KNPN'] = forceString(record.value('locPlaceCode'  if useLocAddr else 'regPlaceCode'))
        # Название населенного пункта адреса проживания застрахованного лица
        row['GOROD'] = forceString(record.value('locPlaceName'  if useLocAddr else 'regPlaceName'))
        # Код улицы адреса проживания застрахованного лица
        row['KUL'] = forceString(record.value('locStreetCode'  if useLocAddr else 'regStreetCode'))
        # Название улицы адреса проживания застрахованного лица
        row['ULICA'] = forceString(record.value('locStreetName' if useLocAddr else 'regStreetName'))
        # Номер дома адреса проживания застрахованного лица*
        row['DOM'] = forceString(record.value('locNumber' if useLocAddr else 'regNumber'))
        # Номер квартиры адреса проживания застрахованного лица
        row['KV'] = forceString(record.value('locFlat' if useLocAddr else 'regFlat'))
        row.store()
        self.num += 1

# *****************************************************************************************

    def isComplete(self):
        return self.done


    def abort(self):
        self.aborted = True


    @QtCore.pyqtSignature('')
    def on_btnExport_clicked(self):
        self.export()


    @QtCore.pyqtSignature('')
    def on_btnCancel_clicked(self):
        self.abort()

# *****************************************************************************************

class CExportPage2(QtGui.QWizardPage, Ui_ExportPage2):
    def __init__(self, parent):
        QtGui.QWizardPage.__init__(self, parent)
        self._parent = parent
        self.setupUi(self)
        self.setSubTitle(u'для выполнения шага нажмите кнопку "финиш"')

        self.pathIsValid = True
        homePath = QtCore.QDir.toNativeSeparators(QtCore.QDir.homePath())
        exportDir = forceString(QtGui.qApp.preferences.appPrefs.get('AttachExportDir', homePath))
        self.edtDir.setText(exportDir)


    def isComplete(self):
        return self.pathIsValid


    def validatePage(self):
        baseName = os.path.basename(self._parent.page1.getDbfBaseName())
        srcFullName = os.path.join(forceStringEx(self._parent.getTmpDir()),
                                                baseName)
        dst = os.path.join(forceStringEx(self.edtDir.text()), baseName)
        success, _ = QtGui.qApp.call(self, shutil.move, (srcFullName, dst))

        if success:
            QtGui.qApp.preferences.appPrefs['AttachExportDir'] = toVariant(self.edtDir.text())
        return success


    @QtCore.pyqtSignature('QString')
    def on_edtDir_textChanged(self):
        dir_ = forceStringEx(self.edtDir.text())
        pathIsValid = os.path.isdir(dir_)
        if self.pathIsValid != pathIsValid:
            self.pathIsValid = pathIsValid
            self.emit(QtCore.SIGNAL('completeChanged()'))


    @QtCore.pyqtSignature('')
    def on_btnSelectDir_clicked(self):
        dir_ = QtGui.QFileDialog.getExistingDirectory(self,
                u'Выберите директорий для сохранения файла выгрузки в ОМС Мурманской области',
                 forceStringEx(self.edtDir.text()),
                 QtGui.QFileDialog.ShowDirsOnly)
        if forceString(dir_):
            self.edtDir.setText(QtCore.QDir.toNativeSeparators(dir_))
