# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import re
import os.path

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import *

from library.Utils import *
from library.roman import itor
from library import dbfpy
from library.dbfpy.dbf import Dbf

from Registry.Utils import selectLatestRecord

from Ui_Progress import Ui_ProgressDialog


class CProgressDialog(QtGui.QDialog, Ui_ProgressDialog):
    __pyqtSignals__ = ('startWork(int)', )
    importTypeNormal = 0
    importTypeServices= 1
    importTypeMES = 2
    importTypeAttach = 3
    importTypeRb = 4

    rbFormatNaspunkt = 0
    rbFormatHospital = 1
    rbFormatUlica = 2
    rbFormatSMO = 3
    rbFormatStomatServ = 4

    fieldNamesDict = {
        rbFormatHospital : ('INN', 'KPP', 'FAMS','IMS','OTC','FAMB','IMB',
            'OTB','CODE_HOSP','MASTER','SHORT_NAME','NAME','OKPO','OKVED',
            'OGRN','ADRES','TELEFON','FAX','EMAIL'),
        rbFormatUlica : ('CODUL', 'ULIC'),
        rbFormatNaspunkt : ('CODE', 'DSCRB'),
        rbFormatSMO: ('SMOCOD','TF_OKATO','INN','KPP','NAM_SMOP','NAM_SMOK',
            'OGRN','ADDR_J','FAM_RUK','IM_RUK','OT_RUK','PHONE','FAX','E_MAIL'),
        rbFormatStomatServ: ('ID_SERV',  'CODE',  'DSCRB',  'UNIT',  'D_FROM',
            'NOT_MED',  'PRIM',  'D_TO')
    }

    mapKSGType = {
        u'Тер.' :  1,
        u'Комб.': 2,
        u'Хир.': 3,
        u'Проч.': 4
    }

    def __init__(self, parent):
        QtGui.QDialog.__init__(self, parent)
        self.parent = parent #wtf? скрывать Qt-шный parent - это плохая затея
        self.setupUi(self)
        self.setPlacesCount(0)
        self.setOrgsCount(0)
        self.setClientsCount(0)
        self.setServicesCount(0)

        self.start()
        self.connect(self, SIGNAL('startWork(int)'), self.startWork, Qt.QueuedConnection)

        self.policyTypeId       = None
        self.passportRFId       = None
        self.passportUSSRId     = None
        self.birthCertificateId = None
        self.temporaryCertificateId = None
        self.contactTypeId      = None
        self.clientIdentificationId = None
        self.accountingSystemId = None
        self.attachTypeId = None
        self.matureNetId =  None
        self.insurerM = None
        self.insurerG = None
        self.actionTypeParentGroupId = None
        self.attachTypeTemp = None
        self.attachTypeCode8 = None
        self.chkDeleteExistAttach = None
        self.refreshPolicy = False

        self.serviceFilter = None
        self.copyToActionType = False
        self.fillSpecialityRegionalCode = False
        self.serviceLevelMap = {1:u'М', 2:u'О',  3: u'N'}
        self.sexMap = {u'Ж':2, u'ж':2, u'М':1, u'м':1}
        self.mapSpecialityCodeNameToId = {}
        self.mapActionTypeCodeNameToId = {}

        self.mapAreaToKLADR = {}
        self.mapAreaAndPlaceToKLADR = {}
        self.streetMap = {}
        self.cityMap = {}
        self.hospitalMap = {}
        self.mapEventProfile = {}
        self.mapMESGroup = {}
        self.mapOrgIdToInfis = {}
        self.mapMKBtoKSG = {}
        self.mapKSGTypeAge = {}
        self.mapKSGTypeSex = {}
        self.mapKSGTypeTr = {}
        self.mapKsgManagementFactor = {}

        self.insurerCache = {}
        self.attachTypeCache = {}
        self.documentTypeCache = {}
        self.policyKindCache = {}
        self.OKFSCache = {}
        self.ksgCache = {}
        self.showLog = True
        self.nAdded = 0
        self.nSkipped = 0
        self.nUpdated = 0
        self.nProcessed = 0


    def log(self, str,  forceLog = False):
        if self.showLog or forceLog:
            self.logBrowser.append(str)
            self.logBrowser.update()


    def _setCount(self, progressBar, cnt):
        if cnt:
            progressBar.setValue(0)
            progressBar.setMaximum(cnt)
            progressBar.setEnabled(True)
            progressBar.setTextVisible(True)
        else:
            progressBar.setValue(0)
            progressBar.setMaximum(1)
            progressBar.setEnabled(False)
            progressBar.setTextVisible(False)


    def setPlacesCount(self, cnt):
        self._setCount(self.prbPlaces, cnt)


    def setClientsCount(self, cnt):
        self._setCount(self.prbClients, cnt)


    def setOrgsCount(self,  cnt):
        self._setCount(self.prbOrgs,  cnt)


    def setServicesCount(self,  cnt):
        self._setCount(self.prbServices,  cnt)


    def start(self):
        self.processDone = False
        self.aborted = False
        self.btnCancel.setText(u'Отмена')


    def stop(self):
        self.processDone = True
        self.btnCancel.setText(u'Закрыть')


    def done(self, r):
        self.stop()
        QtGui.QDialog.done(self, r)


    def exec_(self, importType=importTypeNormal):
        self.emit(SIGNAL('startWork(int)'), importType)

        return QtGui.QDialog.exec_(self)


    @pyqtSignature('')
    def on_btnCancel_clicked(self):
        if self.processDone:
            self.accept()
        else:
            self.aborted = True


    def startWork(self,  importType):
        self.start()

        proc = self.work

        if importType == self.importTypeServices:
            proc = self.importServices
        elif importType == self.importTypeMES:
            proc = self.importMES
        elif importType == self.importTypeAttach:
            proc = self.importAttach
        elif importType == self.importTypeRb:
            proc = self.importRb

        QtGui.qApp.call(self, proc)
        self.stop()


    def importRb(self):
        abortImport = None
        rbDir = forceString(self.parent.edtRBDirName.text())

        dbfNaspunkt = Dbf(os.path.join(rbDir,'naspunkt.dbf'), readOnly=True, encoding='cp866')
        dbfUlica = Dbf(os.path.join(rbDir,'ulica.dbf'), readOnly=True, encoding='cp866')
        dbfHospital = Dbf(os.path.join(rbDir,'hospital.dbf'), readOnly=True, encoding='cp866')
        dbfSMO = Dbf(os.path.join(rbDir,'r_smo.dbf'), readOnly=True, encoding='cp866')

        rbList = (
                  ('NASPUNKT.DBF',  dbfNaspunkt,  self.rbFormatNaspunkt,  self.processArea,  self.prbPlaces),
                  ('ULICA.DBF', dbfUlica,  self.rbFormatUlica, self.processStreet,  self.prbPlaces),
                  ('HOSPITAL.DBF', dbfHospital, self.rbFormatHospital, self.processOrg, self.prbOrgs),
                  ('SMO.DBF', dbfSMO, self.rbFormatSMO, self.processInsurer, self.prbOrgs),
        )

        self.setPlacesCount(len(dbfNaspunkt)+len(dbfUlica))
        self.setOrgsCount(len(dbfHospital)+len(dbfSMO))

        for (rbName, dbfName, dbfFormat, processFunc, progressBar) in rbList:
            (formatOk,  abortImport) = self.checkRbFormat(dbfName, dbfFormat)

            if formatOk:
                self.process(dbfName, processFunc, progressBar)
            else:
                self.log(u'Справочник %s не обработан: отсутствуют необходимые поля.' % rbName)

            dbfName.close()

            if abortImport:
                break

        return not abortImport


    def work(self):
        if not self.importRb():
            return

        dbfRegistry = Dbf(forceString(self.parent.edtMainDataFileName.text()), readOnly=True, encoding='cp866')
        self.setClientsCount(len(dbfRegistry))
        self.process(dbfRegistry, self.processClient, self.prbClients)
        dbfRegistry.close()


    def importServices(self):
        self.serviceFilter = self.serviceLevelMap.get(self.parent.cmbFilterLevel.currentIndex())
        self.copyToActionType = self.parent.chkCopyToActionType.isChecked()
        self.fillSpecialityRegionalCode = self.parent.chkFillSpecialityRegionalCode.isChecked()

        dbfServices = Dbf(forceString(self.parent.edtServiceFileName.text()), readOnly=True, encoding='cp866')

        self.setServicesCount(len(dbfServices))

        (formatOk,  msg) = self.checkRbFormat(dbfServices, self.rbFormatStomatServ)

        proc = self.processService

        if formatOk:
            self.log(u'Импорт стоматологических услуг. Фильтр по уровню и заполненине региональных кодов специальностей не поддерживается.')
            proc = self.processServiceTooth

        self.process(dbfServices, proc, self.prbServices)

        dbfServices.close()


    def importMES(self):
        rbDir = forceString(self.parent.edtMESDirName.text())
        dbfTalonProfiles = Dbf(os.path.join(rbDir,'TAL_PROF.DBF'), readOnly=True, encoding='cp866')
        dbfMES = Dbf(os.path.join(rbDir,'R_MES.DBF'), readOnly=True, encoding='cp866')
        dbfKSG = Dbf(os.path.join(rbDir,'R_KSG.DBF'), readOnly=True, encoding='cp866')
        dbfKSGtoMKB = Dbf(os.path.join(rbDir,'R_KSG_MKB.DBF'), readOnly=True, encoding='cp866')
        dbfKsgManagementFactor = Dbf(os.path.join(rbDir,'KSG_K_UPR.DBF'), readOnly=True, encoding='cp866')
        dbfKsgToMkbAge = Dbf(os.path.join(rbDir,'R_KSG_MKB_AGE.DBF'), readOnly=True, encoding='cp866')
        dbfKsgToMkbSex = Dbf(os.path.join(rbDir,'R_KSG_MKB_SEX.DBF'), readOnly=True, encoding='cp866')
        dbfKsgToMkbTr = Dbf(os.path.join(rbDir,'R_KSG_MKB_TR.DBF'), readOnly=True, encoding='cp866')

        self.setServicesCount(len(dbfMES)+len(dbfTalonProfiles)+len(dbfKSG)
                              +len(dbfKSGtoMKB)+len(dbfKsgToMkbAge)+len(dbfKsgToMkbSex)
                              +len(dbfKsgToMkbTr)+len(dbfKsgManagementFactor))

        self.process(dbfKsgToMkbAge, self.processKsgToMkbAge,  self.prbServices)
        self.process(dbfKsgToMkbSex, self.processKsgToMkbSex,  self.prbServices)
        self.process(dbfKsgToMkbTr, self.processKsgToMkbTr,  self.prbServices)
        self.process(dbfKSGtoMKB, self.processKSGtoMKB,  self.prbServices)
        self.process(dbfKsgManagementFactor, self.processKsgManagementFactor,  self.prbServices)
        self.process(dbfKSG, self.processKSG,  self.prbServices)
        self.process(dbfTalonProfiles, self.processTalonProfiles, self.prbServices)
        self.process(dbfMES, self.processMES, self.prbServices)

        for x in (dbfTalonProfiles,  dbfMES, dbfKSG, dbfKSGtoMKB,
                  dbfKsgToMkbAge, dbfKsgToMkbSex, dbfKsgToMkbTr,
                  dbfKsgManagementFactor):
            x.close()

# *****************************************************************************************

    def checkRbFormat(self, dbf, dbfType):
        if not self.fieldNamesDict.get(dbfType):
            return False

        for fieldName in self.fieldNamesDict[dbfType]:
            if fieldName not in dbf.fieldNames:
                messageStr = u'В справочнике `%s` отсуствует необходимое' \
                    u' для импорта поле `%s`.' % (dbf.name, fieldName)

                self.log(messageStr)
                result = QtGui.QMessageBox.critical(self, u'Импорт невозможен',
                                           messageStr, QtGui.QMessageBox.Abort|QtGui.QMessageBox.Ignore, QtGui.QMessageBox.Abort)
                return (False,  result)

        return (True,  None)

# *****************************************************************************************

    def importAttach(self):
        dbfAttach= Dbf(forceString(self.parent.edtAttachImportFileName.text()), readOnly=True, encoding='cp866')
        self.nAdded = 0
        self.nSkipped = 0
        self.nUpdated = 0
        self.nProcessed = 0

        if self.chkDeleteExistAttach:
            self.deleteExistAttach()

        if not self.importRb():
            dbfAttach.close()
            return

        self.setClientsCount(len(dbfAttach))
        self.process(dbfAttach, self.processAttach,  self.prbClients)
        dbfAttach.close()
        self.log(u'Обработано %d записей: добавлено %d, обновлено %d, пропущено %d' % (
            self.nProcessed,  self.nAdded,  self.nUpdated,  self.nSkipped))

# *****************************************************************************************

    def process(self, dbf, step, progressBar):
        for record in dbf:
            QtGui.qApp.processEvents()
            if self.aborted:
                self.reject()
                return
            progressBar.setValue(progressBar.value()+1)
            QtGui.qApp.db.transaction()
            try:
                step(record)
                QtGui.qApp.db.commit()
                self.nProcessed += 1
            except:
                QtGui.qApp.db.rollback()
                raise

# *****************************************************************************************

    def processTalonProfiles(self, row):
        code = forceString(row['CODE'].strip())
        name = forceString(row['DSCRB'].strip())

        if not (code and name):
            self.log(u'Пропуск профиля талона `%s`(%s).' % (name, code))
            return

        eventProfileId = self.findEventProfile(code, name)

        if not eventProfileId:
            self.log(u'Добавляем профиль события `%s`(%s)' % (name, code))
            eventProfileId = self.addEventProfile(code, name)

        mesGroupId = self.findMESGroup(code, name)

        if not mesGroupId:
            self.log(u'Добавляем группу МЭС `%s`(%s)' % (name, code))
            mesGroupId = self.addMESGroup(code, name)

        self.mapMESGroup[code] = (mesGroupId, name)
        self.mapEventProfile[code] = eventProfileId


    def processMES(self, row):
        codeMKB = forceString(row['KOD_MKB'].strip())
        code = forceString(row['KOD_MES'].strip())
        name = forceString(row['NAME'].strip())
        group = forceString(row['TAL_PROF'].strip())
        KSGNorm = forceString(row['SROK'])

        (groupId, groupName) = self.mapMESGroup.get(group, (None, ''))

        if not groupId:
            self.log(u'Не найдена группа МЭС с кодом `%s`.' % group)

        if groupName:
            name = groupName + ': ' + name

#        patientModel = "Н.Н.**.00.5.1.*.5.0.К"

        id = self.findMES(code, name)

        if codeMKB:
            codeMKB = codeMKB[:3]+'.'+codeMKB[3:]

        ksgId = self.ksgCache.get(self.mapMKBtoKSG.get(codeMKB)) if codeMKB else None

        if not id:
            self.log(u'Добавляем МЭС `%s`(%s).' % (code, name))

            id = self.addMES(code, name, groupId, KSGNorm, ksgId)

            if codeMKB:
                self.addMESMKB(id, codeMKB)
        else:
            message = (u'Обновлен МЭС `%s` (%s).' if self.updateMES(id, ksgId) else
                u'Пропускаем МЭС `%s`(%s): уже есть в базе.')
            self.log(message % (code, name))

# *****************************************************************************************

    def processKSG(self, row):
        code = forceString(row['KSG'])
        name = forceString(row['KSG_NAME'].strip())

        id = self.findKSG(code, name)

        vk = forceDouble(row['VK'])
        prof = forceInt(row['PROF'])
        typeStr = forceString(row['TYPE'].strip())
        type = self.mapKSGType.get(typeStr, 0)
        managementFactor = self.mapKsgManagementFactor.get(code, 0.0)

        if type == 0:
            self.log(u'<b><font color=yellow>ВНИМАНИЕ</b> Неизвестный тип КСГ `%s`, сообщите разработчику.' % typeStr)

        if not id:
            self.log(u'Добавляем КСГ `%s`(%s).' % (code, name))
            id = self.addKsg(code, name, vk, prof, type, managementFactor)
            self.ksgCache[code] = id
        else:
            self.log(u'Обновляем КСГ `%s`(%s).' % (code, name))
            self.updateKsg(id, vk, prof, type, managementFactor)


    def processKSGtoMKB(self, row):
        mkbCode = forceString(row['DS'].strip())
        ksgCode = forceString(row['KSG'])
        sex = forceBool(row['SEX'])
        age = forceBool(row['AGE'])
        tr = forceBool(row['TR'])

        if ksgCode == '0':
            if sex:
                ksgCode = self.mapKSGTypeSex.get(mkbCode)
            elif age:
                ksgCode = self.mapKSGTypeAge.get(mkbCode)
            elif tr:
                ksgCode = self.mapKSGTypeTr.get(mkbCode)

        self.mapMKBtoKSG[mkbCode] = ksgCode


    def processKsgToMkbAge(self, row):
        code = forceString(row['DS'].strip())
        val =forceDouble(row['KSG'])
        self.mapKSGTypeAge[code] = val


    def processKsgToMkbSex(self, row):
        code = forceString(row['DS'].strip())
        val =forceDouble(row['KSG'])
        self.mapKSGTypeSex[code] = val


    def processKsgToMkbTr(self, row):
        code = forceString(row['DS'].strip())
        val =forceDouble(row['KSG'])
        self.mapKSGTypeTr[code] = val


    def processKsgManagementFactor(self, row):
        code = forceString(row['KSG_CODE'])
        val =forceDouble(row['K_UPR'])
        self.mapKsgManagementFactor[code] = val

# *****************************************************************************************

    def deleteExistAttach(self):
        stmt = """UPDATE `ClientAttach`
            SET `deleted` = '1'
            WHERE `LPU_id` = '%d'
                AND `attachType_id` NOT IN (%s, %d)""" % (
            QtGui.qApp.currentOrgId(), ','.join ('%d' % x for x in self.attachTypeTemp),
            self.attachTypeCode8)

        db = QtGui.qApp.db
        db.transaction()
        try:
            db.query(stmt)
            db.commit()
        except:
            db.rollback()
            raise


# *****************************************************************************************

    def processAttach(self,  row):
        validAddress  = True
        kladrAddress = True
        freeInputAddress = u''
        codeLPU = forceString(row['CODE_HOSP'])

        lastName = nameCase(forceString(row['FAM']))
        firstName = nameCase(forceString(row['IM']))
        patrName = nameCase(forceString(row['OT']))
        birthDate = forceDate(row['DR'])
        sex = self.sexMap.get(forceString(row['SEX']), 0)
        SNILS = forceString(row['SNILS']).replace('-', '').replace(' ', '').strip()
        policySerial = forceString(row['SPOLIS'])
        policyNumber = forceString(row['NPOLIS'])
        policyKind = forceString(row['VPOLIS'])
        policyKindId = self.findPolicyKind(policyKind)
        insurerInfis = forceString(row['SMO'])
        policyInsurer = self.findOrgByInfis(insurerInfis)

        docSerial = forceString(row['DOCSER']).strip()
        docNumber  = forceString(row['DOCNUM'])
        docType = forceString(row['DOCTYPE'])
        docTypeId = self.findDocumentType(docType)

        clientId = self.findClientByNameSexAndBirthDate(lastName, firstName, patrName, sex, birthDate)
        attachTypeId = self.findAttachTypeByRegionalCode(forceString(row['V_PR']))
        orgId = self.findOrgByInfis(codeLPU)
        begDate = forceDate(row['DPR_P'])

        cityCode = forceInt(row['KNPN'].strip()) if row['KNPN'].strip() else None
        placeCode = self.mapAreaToKLADR.get(cityCode,  None) if cityCode else None
        streetCode    = forceInt(row['KUL'].strip()) if row['KUL'].strip() else None
        street = forceString(row['ULICA'])
        city = forceString(row['GOROD'])

        ncList = forceString(row['DOM']).strip().split('/')
        number = ncList[0] if len(ncList) > 0 else ''
        corpus = ncList[1] if len(ncList) > 1 else ''
        flat      = forceString(row['KV']).strip()

        if not begDate:
            begDate = forceDate(row['DPR_K'])

        if not begDate:
            begDate = forceDate(row['DPR_S'])

        if not begDate:
            begDate = firstMonthDay(QDate.currentDate())

        if not attachTypeId:
            self.log(u'<b><font color=red>ОШИБКА</b>: Тип прикрепления с региональным кодом `%s` не найден' % (
                            forceString(row['V_PR'])))
            self.log(u'Информация о прикреплении %s %s %s %s не загружена' % (
                                        lastName,  firstName,  patrName, birthDate.toString('dd.MM.yyyy')))
            self.nSkipped += 1
            return

        if not orgId:
            self.log(u'<b><font color=red>ОШИБКА</b>: Организация с инфис кодом `%s` не найдена' % (
                            codeLPU))
            self.log(u'Информация о прикреплении %s %s %s %s не загружена' % (
                                        lastName,  firstName,  patrName, birthDate.toString('dd.MM.yyyy')))
            self.nSkipped += 1
            return

        if not docTypeId and docType:
            self.log(u'<b><font color=red>ОШИБКА</b>: Тип документа с региональным кодом `%s` не найден' % (
                            docType))
            self.log(u'Информация о прикреплении %s %s %s %s не загружена' % (
                                        lastName,  firstName,  patrName, birthDate.toString('dd.MM.yyyy')))
            self.nSkipped += 1
            return

        if self.refreshPolicy and not policyKindId and policyKind:
            self.log(u'<b><font color=red>ОШИБКА</b>: Тип полиса с федеральным кодом `%s` не найден' % (
                            policyKind))
            self.log(u'Информация о полисе %s %s %s %s не загружена' % (
                                        lastName,  firstName,  patrName, birthDate.toString('dd.MM.yyyy')))

        if self.refreshPolicy and not policyInsurer and insurerInfis:
            self.log(u'<b><font color=red>ОШИБКА</b>: СМО с инфис кодом `%s` не найдена' % (
                            insurerInfis))
            self.log(u'Информация о СМО для полиса %s %s %s %s не загружена' % (
                                        lastName,  firstName,  patrName, birthDate.toString('dd.MM.yyyy')))

        if not clientId and SNILS:
            clientId = self.findClientBySNILS(SNILS)

        if not clientId and (docSerial and docNumber and docTypeId):
            clientId = self.findClientByDoc(docSerial, docNumber,  docTypeId)

        kladrAddress = not (((street == '') and streetCode) or (not placeCode and cityCode))

        if not kladrAddress:
            if not placeCode and cityCode:
                self.log(u'! Населенный пункт: (%d)"%s".' % (cityCode,  city))

            if (street == '') and streetCode:
                self.log(u'<b><font color=red>ВНИМАНИЕ</b>:'
                    u'Адрес для клиента "%s %s %s (%s)" не импортирован. '
                    u'Отсутствуют данные об улице (код "%d")'
                    u' в справочниках МОФОМС.' %\
                    (lastName, firstName, patrName,  birthDate.toString('dd.MM.yyyy'), streetCode),  True)
                street = forceString(streetCode)

            self.log(u'! Улица: (%s) "%s".' % \
                     (forceString(streetCode),  street))

            if cityCode:
                freeInputAddress = city
            if street:
                freeInputAddress += u', %s' % street
            if number:
                freeInputAddress += u', д. %s' % number
            if corpus:
                freeInputAddress += u', корп. %s' % corpus
            if flat:
                freeInputAddress += u', кв. %s' % flat

            if freeInputAddress == '':
                self.log(u'<b><font color=red>ВНИМАНИЕ</b>:'
                    u'Адрес для клиента "%s %s %s (%s)" не импортирован. '
                    u'Отсутствуют данные в справочниках МОФОМС.' %\
                    (lastName, firstName, patrName,  birthDate.toString('dd.MM.yyyy')),  True)
                validAddress  = False
            else:
                self.log(u'<b><font color=red>ВНИМАНИЕ</b>:'
                    u'Адрес для клиента "%s %s %s (%s)" импортирован в формате'
                    u' "свободного ввода": "%s".\n'
                    u'Отсутствуют данные в справочниках КЛАДР.' %\
                    (lastName, firstName, patrName, birthDate.toString('dd.MM.yyyy'),  freeInputAddress),  True)


        self.log(u'<b><font color=%s</b>: %s %s %s %s' % (
                                        u'blue>Обновляем' if clientId else  u'green>Добавляем',
                                        lastName,  firstName,  patrName,
                                        birthDate.toString('dd.MM.yyyy')))

        if clientId:
            self.updateClient(clientId, lastName, firstName, patrName, birthDate, sex, SNILS)

            if docTypeId:
                self.updateClientDocument(clientId, docTypeId, docSerial, docNumber)

            if self.refreshPolicy and policyKindId:
                self.updateClientPolicy(clientId, policyInsurer, policySerial, policyNumber, None, None,  None, policyKindId)

            self.updateClientAttach(clientId, orgId, attachTypeId, begDate)

            if validAddress and kladrAddress:
                self.updateClientAddress(clientId, placeCode, street, number, corpus, flat, '',  False)

            self.nUpdated += 1
        else:
            clientId = self.addClient(None,  lastName, firstName, patrName, birthDate, sex, SNILS)

            if docTypeId:
                self.addClientDocument(clientId, docTypeId, docSerial, docNumber)

            if self.refreshPolicy and policyKindId:
                self.addClientPolicy(clientId, policyInsurer, policySerial, policyNumber, None, None,  None, policyKindId)

            self.addClientAttach(clientId,  orgId, attachTypeId, begDate)

            if validAddress:
                self.addClientAddress(clientId, placeCode, street, number, corpus, flat, freeInputAddress)

            self.nAdded+= 1

# *****************************************************************************************

    def findEventProfile(self, code, name):
        table = QtGui.qApp.db.table('rbEventProfile')
        record = QtGui.qApp.db.getRecordEx(table, 'id',  [table['code'].eq(code),
                                                          table['name'].eq(name),
                                                         ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None


    def addEventProfile(self, code, name):
        table = QtGui.qApp.db.table('rbEventProfile')
        record = table.newRecord()
        record.setValue('code', toVariant(code))
        record.setValue('name', toVariant(name))
        return QtGui.qApp.db.insertRecord(table, record)


    def findMESGroup(self, code, name):
        table = QtGui.qApp.db.table('mes.mrbMESGroup')
        record = QtGui.qApp.db.getRecordEx(table, 'id',  [table['code'].eq(code),
                                                          table['name'].eq(name),
                                                          table['deleted'].eq(0)
                                                         ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None


    def addMESGroup(self, code, name):
        table = QtGui.qApp.db.table('mes.mrbMESGroup')
        record = table.newRecord()
        record.setValue('code', toVariant(code))
        record.setValue('name', toVariant(name))
        record.setValue('deleted', toVariant(0))
        return QtGui.qApp.db.insertRecord(table, record)


    def findMES(self, code, name):
        table = QtGui.qApp.db.table('mes.MES')
        record = QtGui.qApp.db.getRecordEx(table, 'id',  [table['code'].eq(code),
                                                          table['name'].eq(name),
                                                          table['deleted'].eq(0)
                                                         ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None


    def addMES(self, code, name, groupId, KSGNorm, ksgId):
        table = QtGui.qApp.db.table('mes.MES')
        record = table.newRecord()
        record.setValue('code', toVariant(code))
        record.setValue('name', toVariant(name))
        record.setValue('deleted', toVariant(0))
        record.setValue('group_id', toVariant(groupId))
        record.setValue('maxDuration', toVariant(KSGNorm))
        record.setValue('ksg_id', toVariant(ksgId))
        return QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def updateMES(self, id, ksgId):
        result = None

        if ksgId:
            dirty = False
            table = QtGui.qApp.db.table('mes.MES')
            record = QtGui.qApp.db.getRecord(table, '*', id)
            oldKsgId = forceRef(record.value('ksg_id'))

            if oldKsgId != ksgId:
                record.setValue('ksg_id', toVariant(ksgId))
                dirty = True

            if dirty:
                result = QtGui.qApp.db.updateRecord(table, record)

        return result


    def addMESMKB(self, mesId, MKB):
        table = QtGui.qApp.db.table('mes.MES_mkb')
        record = table.newRecord()
        record.setValue('deleted', toVariant(0))
        record.setValue('master_id', toVariant(mesId))
        record.setValue('mkb', toVariant(MKB))
        return QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def findKSG(self, code, name):
        table = QtGui.qApp.db.table('mes.MES_ksg')
        record = QtGui.qApp.db.getRecordEx(table, 'id',  [table['code'].eq(code),
                                                          table['name'].eq(name),
                                                          table['deleted'].eq(0)
                                                         ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None


    def addKsg(self, code, name, vk, prof, type, managementFactor):
        table = QtGui.qApp.db.table('mes.MES_ksg')
        record = table.newRecord()
        record.setValue('code', toVariant(code))
        record.setValue('name', toVariant(name))
        record.setValue('deleted', toVariant(0))
        record.setValue('vk', toVariant(vk))
        record.setValue('prof', toVariant(prof))
        record.setValue('type', toVariant(type))
        record.setValue('managementFactor', toVariant(managementFactor))
        return QtGui.qApp.db.insertRecord(table, record)


    def updateKsg(self, id, vk, prof, type, managementFactor):
        table = QtGui.qApp.db.table('mes.MES_ksg')
        record = QtGui.qApp.db.getRecord(table, '*', id)
        record.setValue('vk', toVariant(vk))
        record.setValue('prof', toVariant(prof))
        record.setValue('type', toVariant(type))
        record.setValue('managementFactor', toVariant(managementFactor))
        return QtGui.qApp.db.updateRecord(table, record)

# *****************************************************************************************

    def processService(self, row):
        if self.serviceFilter and self.serviceFilter != row['LEVEL'][0].upper():
            return

        rowDict = row.asDict()

        if rowDict.has_key('CLASS') and (row['CLASS'].strip() != row['SPEC'].strip()):
            return

        code = forceString(row['CODE'].strip())
        name = forceString(row['DSCRB'].strip())
        endDate = QDate(row['NOTVALDATE'])
        adultUetDoctor = forceDouble(row['AUNIT0'])
        adultUetAverageMedWorker = forceDouble(row['AUNIT1'])
        childUetDoctor = forceDouble(row['CUNIT0'])
        childUetAverageMedWorker = forceDouble(row['CUNIT1'])

        serviceId = self.findServiceByCodeAndName(code, name)

        if serviceId:
            self.updateService(serviceId, code, None, endDate, adultUetDoctor,
                                        adultUetAverageMedWorker, childUetDoctor,
                                        childUetAverageMedWorker)
        else:
            serviceId = self.addService(code,  name,  code,  None,  endDate,
                                        adultUetDoctor, adultUetAverageMedWorker,
                                        childUetDoctor, childUetAverageMedWorker)

        specialityCode = row['SPEC'].strip()
        specialityName = row['SPEC_NAME'].strip()

        specialityId = self.findSpecialityByName(specialityName)

        if specialityId and self.fillSpecialityRegionalCode:
            self.ensureSpecialityRegionalCode(specialityId, specialityCode)

        if self.copyToActionType:
            actionTypeId = self.findActionTypeByCodeAndName(code,  name)
            groupCode = row['GRP_CODE'].strip() if rowDict.has_key('GRP_CODE') else ''
            groupName = row['GRP_NAME'].strip() if rowDict.has_key('GRP_NAME') else ''
            groupId = self.actionTypeParentGroupId

            if groupName != '':
                groupId = self.findActionTypeByCodeAndName(groupCode,  groupName)

                if not groupId:
                    groupId = self.addActionType(groupCode,  groupName, self.actionTypeParentGroupId, None)

            if actionTypeId:
                self.updateActionType(actionTypeId,  groupId,  serviceId)
            else:
                self.addActionType(code, name,  groupId,  serviceId)

# *****************************************************************************************

    def processServiceTooth(self, row):
        code = forceString(row['CODE'].strip())
        name = forceString(row['DSCRB'].strip())
        begDate = QDate(row['D_FROM'])
        endDate = QDate(row['D_TO'])
#        uet = forceDouble(row['UNIT'])

        serviceId = self.findServiceByCodeAndName(code, name)

        if serviceId:
            self.updateService(serviceId, code, begDate, endDate, 0, 0, 0, 0)
        else:
            serviceId = self.addService(code,  name,  code,  begDate,  endDate, 0, 0, 0, 0)

        if self.copyToActionType:
            actionTypeId = self.findActionTypeByCodeAndName(code,  name)
            groupCode = row['GRP_CODE'].strip() if row.asDict.has_key('GRP_CODE') else ''
            groupName = row['GRP_NAME'].strip() if row.asDict.has_key('GRP_NAME') else ''
            groupId = self.actionTypeParentGroupId

            if groupName != '':
                groupId = self.findActionTypeByCodeAndName(groupCode,  groupName)

                if not groupId:
                    groupId = self.addActionType(groupCode,  groupName, self.actionTypeParentGroupId, None)

            if actionTypeId:
                self.updateActionType(actionTypeId,  groupId,  serviceId)
            else:
                self.addActionType(code, name,  groupId,  serviceId)

# *****************************************************************************************

    def processArea(self, record):
        code = forceInt(record['CODE'])
        name = forceString(record['DSCRB'])
        table = QtGui.qApp.db.table('kladr.KLADR')
        record = QtGui.qApp.db.getRecordEx(table, 'CODE',
            [table['NAME'].eq(name), table['CODE'].like('510%'),
            ], 'CODE')

        KLADRCode = forceString(record.value(0)) if record else None
        if KLADRCode:
            self.mapAreaToKLADR[code] = KLADRCode
            self.updateKLADRInfisCode(KLADRCode,  code)
        else:
            self.cityMap[code] = name
            self.log(u'! Населённый пункт "%s" (код "%s")'
                        u' не найден в КЛАДРе.' % (name,  code),  True)

# *****************************************************************************************

    def processStreet(self, record):
        codeEx = record['CODUL']
        code = forceInt(codeEx) if codeEx else None
        name = forceString(record['ULIC'])
        self.streetMap[code] = name

# *****************************************************************************************

    def processOrg(self,  row):
        INN = forceString(row['INN']).strip()
        KPP = forceString(row['KPP']).strip()
        chief = nameCase(trim(row['FAMS']+ ' ' + row['IMS']+ ' '+row['OTC']))
        accountant = nameCase(trim(row['FAMB']+' ' + row['IMB']+ ' '+row['OTB']))
        code = forceString(row['CODE_HOSP']).strip()
        master = forceString(row['MASTER'])
        masterId = None

        orgId = self.findOrgByInfis(code)

        if not orgId:
            orgId =  self.findOrgByName(row['NAME'])

        if master and master != code:
            masterId = self.hospitalMap.get(master)

        if orgId:
            self.updateOrg(orgId, row['SHORT_NAME'], row['NAME'],
                row['OKPO'], row['OKVED'], row['OGRN'],
                row['ADRES'], chief, accountant,  row['TELEFON'],  row['FAX'],  row['EMAIL'],  masterId,  code)
        else:
            orgId = self.addOrg(row['SHORT_NAME'], row['NAME'], INN, KPP,
                row['OKPO'], row['OKVED'], row['OGRN'],
                row['ADRES'], chief, accountant,  row['TELEFON'],  row['FAX'],  row['EMAIL'],  masterId,  code)

        self.hospitalMap[code] = orgId

# *****************************************************************************************

    def processInsurer(self, row):
        orgId = None
        code = forceString(row['SMOCOD']).strip()
        iOKATO = forceString(row['TF_OKATO']).strip()
        INN = forceString(row['INN']).strip()
        KPP = forceString(row['KPP']).strip()
        fullName = forceString(row['NAM_SMOP']).strip()
        shortName = forceString(row['NAM_SMOK']).strip()
        OGRN = forceString(row['OGRN']).strip()
        address = forceString(row['ADDR_J']).strip()
        chief = nameCase(trim(row['FAM_RUK']+ ' ' + row['IM_RUK']+ ' '+row['OT_RUK']))
        phone = forceString(row['PHONE'])
        fax = forceString(row['FAX'])
        email = forceString(row['E_MAIL'])

        if INN and KPP :
            orgId = self.findOrgId(INN, KPP)

        if not orgId:
            orgId = self.findOrgByInfis(code)

        if not orgId:
            orgId = self.findOrgByName(fullName)

        if orgId:
            self.updateOrg(orgId, shortName, fullName, '', '', OGRN, address,
                           chief, '',  phone, fax,  email, None,  code, isInsurer=True, OKATO=iOKATO)
        else:
            orgId = self.addOrg(shortName, fullName, INN, KPP,
                '', '', OGRN, address, chief, '',  phone,  fax, email,  None,  code,  isInsurer=True, OKATO=iOKATO)

        self.insurerCache[code] = orgId

# *****************************************************************************************

    def updateKLADRInfisCode(self,  KLADRCode,  infisCode):
        table = QtGui.qApp.db.table('kladr.KLADR')
        table.setIdFieldName('CODE')
        record = QtGui.qApp.db.getRecordEx(table, 'CODE, infis',
            [table['CODE'].eq(KLADRCode),], 'CODE')
        record.setValue('infis', toVariant(infisCode))
        QtGui.qApp.db.updateRecord(table, record)

# *****************************************************************************************

    def findClientByNPP(self, npp):
        table = QtGui.qApp.db.table('ClientIdentification')
        record = QtGui.qApp.db.getRecordEx(table, 'client_id', [table['deleted'].eq(0),
                                                          table['accountingSystem_id'].eq(self.accountingSystemId),
                                                          table['identifier'].eq(npp)
                                                         ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None

# *****************************************************************************************

    def findClientByNameSexAndBirthDate(self, lastName, firstName, patrName, sex, birthDate):
        table = QtGui.qApp.db.table('Client')
        record = QtGui.qApp.db.getRecordEx(table, 'id',  [table['deleted'].eq(0),
                                                          table['lastName'].eq(lastName),
                                                          table['firstName'].eq(firstName),
                                                          table['patrName'].eq(patrName),
                                                          table['sex'].eq(sex),
                                                          table['birthDate'].eq(birthDate)
                                                         ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None

# *****************************************************************************************

    def findClientByDoc(self, docSerial,  docNumber, docTypeId):
        table = QtGui.qApp.db.table('ClientDocument')
        record = QtGui.qApp.db.getRecordEx(table, 'client_id',  [table['deleted'].eq(0),
                                                          table['serial'].eq(docSerial),
                                                          table['number'].eq(docNumber),
                                                          table['documentType_id'].eq(docTypeId),
                                                         ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None


# *****************************************************************************************

    def findServiceByCodeAndName(self, code,  name):
        table = QtGui.qApp.db.table('rbService')
        record = QtGui.qApp.db.getRecordEx(table, 'id',  [table['code'].eq(code),
                                                          table['name'].eq(name),
                                                         ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None

# *****************************************************************************************

    def findSpecialityByName(self, name):
        key = (name)
        result = self.mapSpecialityCodeNameToId.get(key)

        if not result:
            table = QtGui.qApp.db.table('rbSpeciality')
            record = QtGui.qApp.db.getRecordEx(table, 'id',  [
                                                        table['name'].eq(name),
                                                         ], 'id')
            if record:
                result = forceRef(record.value(0))
                self.mapSpecialityCodeNameToId[key] = result

        return result

# *****************************************************************************************

    def findActionTypeByCodeAndName(self, code,  name):
        key = (code,  name)
        result = self.mapActionTypeCodeNameToId.get(key)

        if not result:
            table = QtGui.qApp.db.table('ActionType')
            record = QtGui.qApp.db.getRecordEx(table, 'id',  [table['deleted'].eq(0),
                                                        table['code'].eq(code),
                                                        table['name'].eq(name),
                                                        table['class'].eq('3'),
                                                         ], 'id')
            if record:
                result = forceRef(record.value(0))
                self.mapActionTypeCodeNameToId[key] = result

        return result


# *****************************************************************************************

    def findDocumentType(self, code):
        result = self.documentTypeCache.get(code,  -1)
        if result == -1:
            result = forceRef(QtGui.qApp.db.translate('rbDocumentType', 'regionalCode', code, 'id'))
            self.documentTypeCache[code] = result
        return result

# *****************************************************************************************

    def findPolicyKind(self, code):
        result = self.policyKindCache.get(code,  -1)
        if result == -1:
            result = forceRef(QtGui.qApp.db.translate('rbPolicyKind', 'federalCode', code, 'id'))
            self.policyKindCache[code] = result
        return result

# *****************************************************************************************

    def findClientBySNILS(self, SNILS):
        table = QtGui.qApp.db.table('Client')
        record = QtGui.qApp.db.getRecordEx(table, 'id',  [table['deleted'].eq(0),
                                                          table['SNILS'].eq(SNILS),
                                                         ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None

# *****************************************************************************************

    def addClient(self, npp, lastName, firstName, patrName, birthDate, sex,  SNILS):
        table = QtGui.qApp.db.table('Client')
        record = table.newRecord()
        record.setValue('lastName',  toVariant(lastName))
        record.setValue('firstName', toVariant(firstName))
        record.setValue('patrName',  toVariant(patrName))
        record.setValue('birthDate', toVariant(birthDate))
        record.setValue('sex',       toVariant(sex))
        record.setValue('SNILS',     toVariant(SNILS))
        clientId = QtGui.qApp.db.insertRecord(table, record)
        self.addClientIdentification(clientId, npp)
        return clientId

# *****************************************************************************************

    def updateClient(self, clientId, lastName, firstName, patrName, birthDate, sex,  SNILS):
        table = QtGui.qApp.db.table('Client')
        record = QtGui.qApp.db.getRecord(table, '*', clientId)
        record.setValue('lastName', toVariant(lastName))
        record.setValue('firstName', toVariant(firstName))
        record.setValue('patrName', toVariant(patrName))
        record.setValue('birthDate', toVariant(birthDate))
        record.setValue('sex', toVariant(sex))
        record.setValue('SNILS', toVariant(SNILS))
        QtGui.qApp.db.updateRecord(table, record)

# *****************************************************************************************

    def addClientDocument(self, clientId, documentTypeId, serial, number):
        if documentTypeId:
            table = QtGui.qApp.db.table('ClientDocument')
            record = table.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('documentType_id', toVariant(documentTypeId))
            record.setValue('serial', toVariant(serial))
            record.setValue('number', toVariant(number))
            QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def updateClientDocument(self, clientId, documentTypeId, serial, number):
        if documentTypeId:
            table = QtGui.qApp.db.table('ClientDocument')
            record = QtGui.qApp.db.getRecordEx(table, '*', [table['client_id'].eq(clientId),
                                                      table['deleted'].eq(0),
                                                      table['serial'].eq(serial),
                                                      table['number'].eq(number)], 'id')
            if not record:
                self.addClientDocument(clientId, documentTypeId, serial, number)

# *****************************************************************************************

    def addClientPolicy(self, clientId, insurerId, serial, number, begDate, endDate,  note, policyKindId = None):
        table = QtGui.qApp.db.table('ClientPolicy')
        record = table.newRecord()
        record.setValue('client_id', toVariant(clientId))
        record.setValue('insurer_id', toVariant(insurerId))
        record.setValue('policyType_id', toVariant(self.policyTypeId))
        record.setValue('serial', toVariant(serial))
        record.setValue('number', toVariant(number))
        record.setValue('begDate', toVariant(begDate))
        record.setValue('endDate', toVariant(endDate))
        record.setValue('note',  toVariant(note))
        record.setValue('policyKind_id', toVariant(policyKindId))
        QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def updateClientPolicy(self, clientId, insurerId, serial, number, begDate, endDate,  note, policyKindId = None):
        table = QtGui.qApp.db.table('ClientPolicy')
        record = selectLatestRecord('ClientPolicy', clientId, '')

        if record and forceString(record.value('serial')) == serial \
                and forceString(record.value('number')) == number:
            if (begDate and (forceDate(record.value('begDate')) != begDate)) or \
                (endDate and (forceDate(record.value('endDate')) != endDate)):
                record.setValue('begDate', toVariant(begDate))
                record.setValue('endDate', toVariant(endDate))
                QtGui.qApp.db.updateRecord(table, record)
            if (note and forceString(record.value('note')) != note):
                record.setValue('note', toVariant(note))
                QtGui.qApp.db.updateRecord(table, record)
            if not forceRef(record.value('insurer_id')) and insurerId:
                record.setValue('insurer_id', toVariant(insurerId))
                QtGui.qApp.db.updateRecord(table, record)
            if policyKindId and policyKindId != forceRef(record.value('policyKind_id')):
                record.setValue('policyKind_id', toVariant(policyKindId))
                QtGui.qApp.db.updateRecord(table, record)
        else:
            self.addClientPolicy(clientId, insurerId, serial, number, begDate, endDate,  note, policyKindId)

# *****************************************************************************************

    def getKLADRCodeList(self, areaCode, placeCode):
        key = (areaCode, placeCode)
        return self.mapAreaAndPlaceToKLADR.get(key, [])

    affixes = [u'б-р', u'бульвар', u'ул', u'улица', u'пер', u'пр', u'проспект', u'пр-т', u'наб', u'ш', u'шоссе']

# *****************************************************************************************

    def getKLADRStreetCodeList(self, KLADRCodeList, street):
        result = []

        if KLADRCodeList:
            if street == '' or street == '-':
                return [(KLADRCode, '') for KLADRCode in KLADRCodeList]
            if street.count(','): # нас. пункт, улица, д, кв.
                street = trim(street.split(',')[1])
            street = re.sub(ur'(,?д\.?[ 0-9]+.*$)', '', street) # удаляем приписанные номера домов в виде д.12
            street = re.sub(r'[0-9-/ ]*$', '', street) # удаляем приписанные номера домов в виде 12/3-1
            loCaseStreet = street.lower()
            for affix in self.affixes:
                if loCaseStreet.startswith(affix+'.'):
                    street = trim(street[len(affix)+1:])
                    break
                if loCaseStreet.startswith(affix+' '):
                    street = trim(street[len(affix)+1:])
                    break
                elif loCaseStreet.endswith(' '+affix):
                    street = trim(street[:-len(affix)])
                    break
                elif loCaseStreet.endswith(' '+affix+'.'):
                    street = trim(street[:-len(affix)-1])
                    break
            for KLADRCode in KLADRCodeList:
                table = QtGui.qApp.db.table('kladr.STREET')
                records = QtGui.qApp.db.getRecordList(table, 'CODE', [table['NAME'].eq(street),
                                                                      table['CODE'].like(KLADRCode[:-2]+'%'),
                                                                     ], 'CODE')
                for record in records:
                    result.append((KLADRCode, forceString(record.value('CODE'))))

        return result

# *****************************************************************************************

    def getAddressHouseId(self, placeCode, street, number, corpus):
        if not placeCode or not street:
            return None

        KLADRCodeList = [placeCode]
        KLADRStreetCodeList = self.getKLADRStreetCodeList(KLADRCodeList, street)
        if len(KLADRStreetCodeList) == 1:
            KLADRCode, KLADRStreetCode = KLADRStreetCodeList[0]
            table = QtGui.qApp.db.table('AddressHouse')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                            table['KLADRCode'].eq(KLADRCode),
                                                            table['KLADRStreetCode'].eq(KLADRStreetCode),
                                                            table['number'].eq(number),
                                                            table['corpus'].eq(corpus)], 'id')
            if record:
                return forceRef(record.value(0))
            else:
                record = table.newRecord()
                record.setValue('KLADRCode', toVariant(KLADRCode))
                record.setValue('KLADRStreetCode', toVariant(KLADRStreetCode))
                record.setValue('number', toVariant(number))
                record.setValue('corpus', toVariant(corpus))
                return QtGui.qApp.db.insertRecord(table, record)
        else:
            return None


# *****************************************************************************************

    def getAddressId(self, placeCode, street, number, corpus, flat, address):
        houseId = self.getAddressHouseId(placeCode, street, number, corpus)
        if houseId:
            table = QtGui.qApp.db.table('Address')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                            table['house_id'].eq(houseId),
                                                            table['flat'].eq(flat)], 'id')
            if record:
                return forceRef(record.value(0))
            else:
                record = table.newRecord()
                record.setValue('house_id', toVariant(houseId))
                record.setValue('flat', toVariant(flat))
                return QtGui.qApp.db.insertRecord(table, record)
        else:
            return None

# *****************************************************************************************

    def ensureClientAddress(self, clientId, addressType, addressId, freeInput):
        table = QtGui.qApp.db.table('ClientAddress')
        record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                  table['client_id'].eq(clientId),
                                                  table['type'].eq(addressType),
                                                  table['address_id'].eq(addressId),
                                                  table['freeInput'].eq(freeInput),
                                                 ], 'id')
        if not record:
            record = table.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('type',      toVariant(addressType))
            record.setValue('address_id',toVariant(addressId))
            record.setValue('freeInput', toVariant(freeInput))
            QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def addClientAddress(self, clientId, placeCode, street, number, corpus, flat, freeInput):
        addressId = self.getAddressId(placeCode, street, number, corpus, flat, freeInput)
        self.ensureClientAddress(clientId, 0, addressId, freeInput)
        self.ensureClientAddress(clientId, 1, addressId, freeInput)

# *****************************************************************************************

    def updateClientAddress(self, clientId, placeCode, street, number, corpus, flat, freeInput, updateLocalAddress = True):
        addressId = self.getAddressId(placeCode, street, number, corpus, flat, freeInput)
        self.ensureClientAddress(clientId, 0, addressId, freeInput)

        if updateLocalAddress:
            self.ensureClientAddress(clientId, 1, addressId, freeInput)

# *****************************************************************************************

    def findAttachTypeByRegionalCode(self,  code):
        result = self.attachTypeCache.get(code,  -1)

        if result == -1:
            table = QtGui.qApp.db.table('rbAttachType')
            record = QtGui.qApp.db.getRecordEx(table, 'id', [
                                                          table['regionalCode'].eq(code),
                                                         ], 'id')
            result = forceRef(record.value(0)) if record else None
            self.attachTypeCache[code] = result

        return result

# *****************************************************************************************

    def ensureClientAttach(self, clientId, attachTypeId, orgId, begDate, endDate):
        if not orgId or not attachTypeId:
            return

        checkBegDate = begDate if begDate else QDate.currentDate()
        checkEndDate = endDate if endDate else QDate.currentDate()

        record = selectLatestRecord('ClientAttach', clientId,
            '(Tmp.`attachType_id`=%d) AND \
         (Tmp.`LPU_id`=%d) AND \
         (Tmp.`begDate`=\'%s\') AND \
         (Tmp.`endDate`=\'%s\')' % \
            (attachTypeId, orgId, checkBegDate.toString(QtCore.Qt.ISODate),
             checkEndDate.toString(QtCore.Qt.ISODate)))

        if not record:
            table = QtGui.qApp.db.table('ClientAttach')
            record = table.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('attachType_id',  toVariant(attachTypeId))
            record.setValue('LPU_id',toVariant(orgId))
            if begDate:
                record.setValue('begDate',toVariant(begDate))
            if endDate:
                record.setValue('endDate', toVariant(endDate))
            QtGui.qApp.db.insertRecord(table, record)


# *****************************************************************************************

    def addClientAttach(self,  clientId,  orgId, attachTypeId, begDate=None, endDate=None):
        self.ensureClientAttach(clientId,  attachTypeId,  orgId, begDate, endDate)

# *****************************************************************************************

    def updateClientAttach(self,  clientId,  orgId, attachTypeId, begDate=None, endDate=None):
        self.ensureClientAttach(clientId,  attachTypeId,  orgId, begDate, endDate)

# *****************************************************************************************

    def addClientIdentification(self, clientId, npp):
        if npp:
            table = QtGui.qApp.db.table('ClientIdentification')
            record = table.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('accountingSystem_id', toVariant(self.accountingSystemId))
            record.setValue('identifier', toVariant(npp))
            record.setValue('checkDate', toVariant(QDate.currentDate()))
            clientId = QtGui.qApp.db.insertRecord(table, record)
#        self.addChange('ClientIdentification', 1)

# *****************************************************************************************

    def updateClientIdentification(self, clientId, npp):
        table = QtGui.qApp.db.table('ClientIdentification')
        record = QtGui.qApp.db.getRecordEx(table, '*',   [table['deleted'].eq(0),
                                                          table['accountingSystem_id'].eq(self.accountingSystemId),
                                                          table['client_id'].eq(clientId),
                                                         ], 'id')
        if record:
            record.setValue('identifier', toVariant(npp))
            record.setValue('checkDate', toVariant(QDate.currentDate()))
            QtGui.qApp.db.updateRecord(table, record)
#            self.addChange('ClientIdentification', 2)
        else:
            self.addClientIdentification(clientId, npp)

# *****************************************************************************************

    def getOrganisationId(self, INN, KPP):
        if INN and KPP:
            table = QtGui.qApp.db.table('Organisation')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                      table['INN'].eq(INN),
                                                      table['KPP'].eq(KPP)], 'id')
            if record:
                return forceRef(record.value(0))
            elif self.addDummyOrganisations:
                record = table.newRecord()
                name = u'ИНН: %s; КПП: %s' % (INN, KPP)
                record.setValue('fullName', toVariant(name))
                record.setValue('shortName', toVariant(name))
                record.setValue('INN', toVariant(INN))
                record.setValue('KPP', toVariant(KPP))
                return QtGui.qApp.db.insertRecord(table, record)
        return None

# *****************************************************************************************

    mapPolicySerialToInsurerId = {}

    def adviseInsurerId(self, serial):
        u""" подобрать с.к. по серии полиса """
        result = self.mapPolicySerialToInsurerId.get(serial)

        if not result:
            db = QtGui.qApp.db
            table = db.table('Organisation_PolicySerial')
            record = db.getRecordEx(table, 'organisation_id',
                                    table['serial'].eq(serial))
            if record:
                result = forceRef(record.value(0))
                self.mapPolicySerialToInsurerId[serial] = result

        return result

# *****************************************************************************************

    def processClient(self, record):
        SNILS = forceString(forceString(record['SNILS'])).strip().replace('-', '').replace(' ', '')
        lastName  = nameCase(forceString(record['FAMN']))
        firstName = nameCase(forceString(record['IMN']))
        patrName  = nameCase(forceString(record['OTN']))
        sex       = self.sexMap.get(forceString(record['POLN']), 0)
        birthDate = QDate(record['DRN'])
        npp = record['INNUM']

        docSerial = forceString(record['SPAN']).strip()
        docNumber  = forceInt(record['NPAN'])

        policySerial = forceString(record['SPON1']).strip()
        policyNumber = forceInt(record['NPON1']) if record['NPON1'] else None
        policyNote = forceString(record['RNAMN1']).strip()
        policyInsurer = self.adviseInsurerId(policySerial)

        if not policyInsurer:
            self.log(u'<b><font color=orange>ВНИМАНИЕ</b>:'
                    u'Определить СМО для клиента "%s %s %s (%s)" по таблице серий полисов невозможно. '
                    u'Серия полиса `%s`.' %\
                    (lastName, firstName, patrName,  birthDate.toString('dd.MM.yyyy'), policySerial),  True)
            if policySerial[0] == u'М':
                policyInsurer = self.insurerM
            elif policySerial[0] == u'Г':
                policyInsurer = self.insurerG
            else:
                self.log(u'<b><font color=orange>ВНИМАНИЕ</b>:'
                        u'Определить СМО для клиента "%s %s %s (%s)" невозможно. '
                        u'Неизвестная серия полиса %s %d' %\
                        (lastName, firstName, patrName, birthDate.toString('dd.MM.yyyy'),  policySerial,  policyNumber),  True)

        cityCode = forceInt(record['KNPN'].strip()) if record['KNPN'].strip() else None
        placeCode = self.mapAreaToKLADR.get(cityCode,  None) if cityCode else None
        streetCode    = forceInt(record['KULN'].strip()) if record['KULN'].strip() else None
        street = self.streetMap.get(streetCode,  '')
        number     = forceString(record['DN']).strip()
        corpus    = forceString(record['KORPN']).strip()
        flat      = forceString(record['KN']).strip()
        attachCode = forceString(record['KPN']).strip()
        attachId = self.hospitalMap.get(attachCode)
        freeInputAddress = ''

        if not attachId and attachCode:
            self.log(u'<b><font color=orange>ВНИМАНИЕ</b>:'
                    u'Прикрепление для клиента "%s %s %s (%s)" невозможно. '
                    u'Организация с кодом "%s" не импортирована.' %\
                    (lastName, firstName, patrName,  birthDate.toString('dd.MM.yyyy'),  attachCode),  True)

        if not npp:
            self.log(u'<b><font color=blue>ВНИМАНИЕ</b>:'
                    u'У клиента "%s %s %s (%s)" отсутствует идентификатор ТФОМС. ' %\
                    (lastName, firstName, patrName,  birthDate.toString('dd.MM.yyyy')),  True)

        if ((street == '') and streetCode) or (not placeCode and cityCode):
            if not placeCode and cityCode:
                self.log(u'! Населенный пункт: (%d)"%s".' % \
                     (cityCode,  self.cityMap.get(cityCode,
                        u'отсутсвует в справочнике МОФОМС')))

            if (street == '') and streetCode:
                self.log(u'<b><font color=red>ВНИМАНИЕ</b>:'
                    u'Адрес для клиента "%s %s %s (%s)" не импортирован. '
                    u'Отсутствуют данные об улице (код "%d")'
                    u' в справочниках МОФОМС.' %\
                    (lastName, firstName, patrName,  birthDate.toString('dd.MM.yyyy'), streetCode),  True)
                street = forceString(streetCode)

            self.log(u'! Улица: (%s) "%s".' % \
                     (forceString(streetCode),  street))

            if cityCode:
                freeInputAddress = '%s' % \
                    self.cityMap.get(cityCode,  forceString(cityCode))
            if street:
                freeInputAddress += u', %s' % street
            if number:
                freeInputAddress += u', д. %s' % number
            if corpus:
                freeInputAddress += u', корп. %s' % corpus
            if flat:
                freeInputAddress += u', кв. %s' % flat

            if freeInputAddress == '':
                self.log(u'<b><font color=red>ВНИМАНИЕ</b>:'
                    u'Адрес для клиента "%s %s %s (%s)" не импортирован. '
                    u'Отсутствуют данные в справочниках МОФОМС.' %\
                    (lastName, firstName, patrName,  birthDate.toString('dd.MM.yyyy')),  True)
            else:
                self.log(u'<b><font color=red>ВНИМАНИЕ</b>:'
                    u'Адрес для клиента "%s %s %s (%s)" импортирован в формате'
                    u' "свободного ввода": "%s".\n'
                    u'Отсутствуют данные в справочниках КЛАДР.' %\
                    (lastName, firstName, patrName, birthDate.toString('dd.MM.yyyy'),  freeInputAddress),  True)

        clientId = self.findClientByNPP(npp)
        if not clientId:
            clientId = self.findClientByNameSexAndBirthDate(lastName, firstName, patrName, sex, birthDate)
        if not clientId and SNILS:
            clientId = self.findClientBySNILS(SNILS)
        if clientId:
            self.updateClientIdentification(clientId, npp)
            self.updateClient(clientId, lastName, firstName, patrName, birthDate, sex, SNILS)
            self.updateClientDocument(clientId, self.passportRFId, docSerial, docNumber)
            self.updateClientPolicy(clientId, policyInsurer, policySerial, policyNumber, None, None,  policyNote)
            self.updateClientAddress(clientId, placeCode, street, number, corpus, flat, freeInputAddress)
            self.updateClientAttach(clientId, attachId, self.attachTypeId)
        else:
            clientId = self.addClient(npp,  lastName, firstName, patrName, birthDate, sex, SNILS)
            self.addClientDocument(clientId, self.passportRFId, docSerial, docNumber)
            self.addClientPolicy(clientId, policyInsurer, policySerial, policyNumber, None, None,  policyNote)
            self.addClientAddress(clientId, placeCode, street, number, corpus, flat, freeInputAddress)
            self.addClientAttach(clientId,  attachId,  self.attachTypeId)

# *****************************************************************************************

    def findOrgId(self, INN, KPP):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecordEx(table, 'id',  [table['deleted'].eq(0),
                                                          table['INN'].eq(INN),
                                                          table['KPP'].eq(KPP)
                                                         ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None

# *****************************************************************************************

    def findOrgByName(self, fullName):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecordEx(table, 'id',  [table['deleted'].eq(0),
                                                          table['fullName'].eq(fullName)], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None

# *****************************************************************************************

    def findOrgByInfis(self, infis):
        result = self.mapOrgIdToInfis.get(infis,  -1)

        if result == -1:
            table = QtGui.qApp.db.table('Organisation')
            record = QtGui.qApp.db.getRecordEx(table, 'id',  [table['deleted'].eq(0),
                                                              table['infisCode'].eq(infis)], 'id')
            result = forceRef(record.value(0)) if record else None
            self.mapOrgIdToInfis[infis] = result

        return result

# *****************************************************************************************

    def addOrg(self, shortName, fullName, INN, KPP, OKPO, OKVED, OGRN,
                        address, chief, accountant,  phone,  fax,  email,  masterId,
                        infisCode, isInsurer = False, OKATO = None):
        table = QtGui.qApp.db.table('Organisation')
        record = table.newRecord()
        record.setValue('fullName', toVariant(fullName))
        record.setValue('shortName', toVariant(shortName))
        record.setValue('title', toVariant(shortName))
        record.setValue('OKPO', toVariant(OKPO))
        record.setValue('OKVED', toVariant(OKVED))
        record.setValue('INN', toVariant(INN))
        record.setValue('KPP', toVariant(KPP))
        record.setValue('OGRN', toVariant(OGRN))
        record.setValue('address', toVariant(address))
        record.setValue('chief', toVariant(chief))
        record.setValue('accountant', toVariant(accountant))
        record.setValue('phone', toVariant(phone))

        if not isInsurer:
            record.setValue('net_id',  toVariant(self.matureNetId))

        notes = u''

        if fax:
            notes += u'факс: %s' % fax

        if email:
            notes += u' email: %s' % email

        if notes != '':
            record.setValue('notes', toVariant(notes))

        if masterId:
            record.setValue('head_id',  toVariant(masterId))

        record.setValue('infisCode',  toVariant(infisCode))

        if isInsurer:
            record.setValue('isInsurer', toVariant(1))

        if OKATO:
            record.setValue('OKATO', toVariant(OKATO))

        return QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def updateOrg(self, orgId, shortName, fullName, OKPO, OKVED, OGRN,
                            address, chief, accountant,  phone,  fax,  email, masterId,
                            infisCode, isInsurer = False, OKATO = None):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecord(table, '*', orgId)
        record.setValue('fullName', toVariant(fullName))
        record.setValue('shortName', toVariant(shortName))
        record.setValue('title', toVariant(shortName))
        record.setValue('OKPO', toVariant(OKPO))
        record.setValue('OKVED', toVariant(OKVED))
        record.setValue('OGRN', toVariant(OGRN))
        record.setValue('address', toVariant(address))
        record.setValue('chief', toVariant(chief))
        record.setValue('accountant', toVariant(accountant))
        record.setValue('phone', toVariant(phone))

        if not isInsurer:
            record.setValue('net_id',  toVariant(self.matureNetId))

        notes = u''

        if fax:
            notes += u'факс: %s' % fax

        if email:
            notes += u' email: %s' % email

        if notes != '':
            record.setValue('notes', toVariant(notes))

        if masterId:
            record.setValue('head_id',  toVariant(masterId))

        record.setValue('infisCode',  toVariant(infisCode))

        if isInsurer:
            record.setValue('isInsurer', toVariant(1))

        if OKATO:
            record.setValue('OKATO', toVariant(OKATO))

        return QtGui.qApp.db.updateRecord(table, record)

# *****************************************************************************************

    def addService(self, code,  name, infisCode, begDate, endDate, adultUetDoctor,
                                        adultUetAverageMedWorker, childUetDoctor,
                                        childUetAverageMedWorker):
        table = QtGui.qApp.db.table('rbService')
        record = table.newRecord()
        record.setValue('code', toVariant(code))
        record.setValue('name', toVariant(name))
        record.setValue('adultUetDoctor', toVariant(adultUetDoctor))
        record.setValue('adultUetAverageMedWorker', toVariant(adultUetAverageMedWorker))
        record.setValue('childUetDoctor', toVariant(childUetDoctor))
        record.setValue('childUetAverageMedWorker', toVariant(childUetAverageMedWorker))

        if infisCode:
            record.setValue('infis', toVariant(infisCode))
            record.setValue('eisLegacy', toVariant(True))

        record.setValue('begDate',  toVariant(begDate))
        record.setValue('endDate',  toVariant(endDate))
        return QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def updateService(self, serviceId, infisCode, begDate, endDate, adultUetDoctor,
                                        adultUetAverageMedWorker, childUetDoctor,
                                        childUetAverageMedWorker):
        table = QtGui.qApp.db.table('rbService')
        record = QtGui.qApp.db.getRecord(table, '*', serviceId)

        if infisCode:
            record.setValue('infis', toVariant(infisCode))
            record.setValue('eisLegacy', toVariant(True))

        record.setValue('adultUetDoctor', toVariant(adultUetDoctor))
        record.setValue('adultUetAverageMedWorker', toVariant(adultUetAverageMedWorker))
        record.setValue('childUetDoctor', toVariant(childUetDoctor))
        record.setValue('childUetAverageMedWorker', toVariant(childUetAverageMedWorker))
        record.setValue('begDate',  toVariant(begDate))
        record.setValue('endDate',  toVariant(endDate))
        return QtGui.qApp.db.updateRecord(table, record)

# *****************************************************************************************

    def ensureSpecialityRegionalCode(self, specialityId, regionalCode):
        table = QtGui.qApp.db.table('rbSpeciality')
        record = QtGui.qApp.db.getRecord(table, '*', specialityId)
        if forceString(record.value('regionalCode')) == '':
            record.setValue('regionalCode',  toVariant(regionalCode))
            return QtGui.qApp.db.updateRecord(table, record)

# *****************************************************************************************

    def addActionType(self, code,  name, groupId,  serviceId):
        table = QtGui.qApp.db.table('ActionType')
        record = table.newRecord()
        record.setValue('code', toVariant(code))
        record.setValue('name', toVariant(name))
        record.setValue('class', toVariant(3)) # прочее

        if groupId:
            record.setValue('group_id', toVariant(groupId))

        if serviceId:
            record.setValue('service_id',  toVariant(serviceId))

        return QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def updateActionType(self, id, groupId,  serviceId):
        table = QtGui.qApp.db.table('ActionType')
        record = QtGui.qApp.db.getRecord(table, '*', id)
        if groupId:
            record.setValue('group_id', toVariant(groupId))

        if serviceId:
            record.setValue('service_id',  toVariant(serviceId))
        return QtGui.qApp.db.updateRecord(table, record)
