# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import re
import os.path

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import *

from library.Utils import *
from library.roman import itor
from library.dbfpy.dbf import Dbf
from Registry.Utils import selectLatestRecord

from Ui_Progress import Ui_ProgressDialog


class CProgressDialog(QtGui.QDialog, Ui_ProgressDialog):
    __pyqtSignals__ = ('startWork()', )
    importTypeDMS = 0
    importTypeServices = 1
    importTypeOMS = 2

    policyReasonOfInvalidity = {
        '1':    u'Увольнение (для работающих)',
        '2': u'Выезд за пределы региона страхования (для неработающих)',
        '3': u'Утрачен',
        '4': u'Смерть',
        '5': u'Смена фамилии, имени, отчества или даты рождения',
        '6': u'Окончание действия договора',
        '7': u'Окончание действия, документа удостоверяющего личность',
        '8': u'Смена места регистрации',
        '9': u'Переход в другую страховую',
        'L': u'Утрачен'
    }

    def __init__(self, parent):
        QtGui.QDialog.__init__(self, parent)
        self.parent = parent #wtf? скрывать Qt-шный parent - это плохая затея
        self.setupUi(self)
        self.setInsuranceTypeCount(0)
        self.setOrgsCount(0)
        self.setClientsCount(0)
        self.setServicesCount(0)

        self.start()
        self.connect(self, SIGNAL('startWork()'), self.startWork, Qt.QueuedConnection)

        self.policyTypeId       = None
        self.currentOrgId = None
        self.socClass9Id = None
        self.socStatusEmployedId = None
        self.policyTypeOMSTerritorialId= None
        self.policyTypeOMSIndustrialId= None


        self.checkUniqueNPP = False
        self.strictNPP = False
        self.convertAddressToKLADR = False

        self.nAdded = 0
        self.nSkipped = 0
        self.nUpdated = 0

        self.serviceLevelMap = {1:u'М', 2:u'О',  3: u'N'}
        self.sexMap = {u'Ж':2, u'ж':2, u'М':1, u'м':1}

        self.mapActionTypeCodeNameToId = {}
        self.skipClientSet = set()

        self.insurerCache = {}
        self.attachCache = {}
        self.showLog = True
        self.canCloseClientAttach = False
        self.logBrowser.document().setMaximumBlockCount(1024)


    def log(self, str,  forceLog = False):
        if self.showLog or forceLog:
            self.logBrowser.append(str)
            self.logBrowser.update()


    def _setCount(self, progressBar, cnt):
        if cnt:
            progressBar.setValue(0)
            progressBar.setMaximum(cnt)
            progressBar.setEnabled(True)
            progressBar.setTextVisible(True)
        else:
            progressBar.setValue(0)
            progressBar.setMaximum(1)
            progressBar.setEnabled(False)
            progressBar.setTextVisible(False)


    def setClientsCount(self, cnt):
        self._setCount(self.prbClients, cnt)


    def setInsuranceTypeCount(self,  cnt):
        self._setCount(self.prbInsuranceTypes,  cnt)


    def setOrgsCount(self,  cnt):
        self._setCount(self.prbOrgs,  cnt)


    def setServicesCount(self,  cnt):
        self._setCount(self.prbServices,  cnt)


    def start(self):
        self.processDone = False
        self.aborted = False
        self.nAdded = 0
        self.nSkipped = 0
        self.nUpdated = 0
        self.btnCancel.setText(u'Отмена')


    def stop(self):
        self.processDone = True
        self.log(u'Клиенты: добавлено "%d", обновлено "%d",'\
            u' пропущено "%d"' % (self.nAdded,  self.nUpdated,  self.nSkipped))
        self.btnCancel.setText(u'Закрыть')


    def done(self, r):
        self.stop()
        QtGui.QDialog.done(self, r)


    def exec_(self):
        self.emit(SIGNAL('startWork()'))
        return QtGui.QDialog.exec_(self)


    @pyqtSignature('')
    def on_btnCancel_clicked(self):
        if self.processDone:
            self.accept()
        else:
            self.aborted = True


    def startWork(self):
        self.start()
        QtGui.qApp.call(self, self.work)
        self.stop()


    def work(self):
        dbfRegistry = Dbf(forceString(self.parent.edtMainDataFileName.text()), readOnly=True, encoding='cp866')
        self.setClientsCount(len(dbfRegistry))
        self.skipClientSet = set()
        self.process(dbfRegistry, self.processClient, self.prbClients)
        dbfRegistry.close()


    def process(self, dbf, step, progressBar):
        #for i in range(1700000,  len(dbf)):
            #record = dbf[i]
        for record in dbf:
            QtGui.qApp.processEvents()
            if self.aborted:
                self.reject()
                return
            progressBar.setValue(progressBar.value()+1)
            QtGui.qApp.db.transaction()
            try:
                step(record)
                QtGui.qApp.db.commit()
            except:
                QtGui.qApp.db.rollback()
                raise


    def addInsurer(self, npp, name, dir, OKPO):
        table = QtGui.qApp.db.table('Organisation')
        record = table.newRecord()
# OKONH, ?
        record.setValue('fullName', toVariant(name))
        record.setValue('shortName', toVariant(name))
        record.setValue('title', toVariant(name))
        record.setValue('infisCode', toVariant(dir))
        record.setValue('OKPO', toVariant(OKPO))
        record.setValue('isInsurer', toVariant(1))
        record.setValue('notes', toVariant(self.getInsurerNotes(npp)))
        orgId = QtGui.qApp.db.insertRecord(table, record)
        return orgId


    def _getInsurerId(self, insurerCode):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecordEx(table, 'id', [table['deleted'].eq(0),
             QtGui.qApp.db.joinOr([table['notes'].eq(self.getInsurerNotes(insurerCode)),
                                           table['infisCode'].eq(insurerCode)]),
                                            table['isInsurer'].eq(1)
                                                        ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return 0


    def getInsurerId(self, insurerCode):
        if insurerCode in self.insurerCache:
            return self.insurerCache[insurerCode]
        else:
            result = self._getInsurerId(insurerCode)
            self.insurerCache[insurerCode] = result
            return result


    def getInsurerNotes(self, npp):
        return u'СМО # %s' % npp


    def updateInsurer(self, orgId, name, dir, OKPO):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecord(table, '*', orgId)
        record.setValue('fullName', toVariant(name))
        record.setValue('shortName', toVariant(name))
        record.setValue('title', toVariant(name))
        record.setValue('infisCode', toVariant(dir))
        record.setValue('OKPO', toVariant(OKPO))
        QtGui.qApp.db.updateRecord(table, record)


    def _getOKFSId(self, OKFS):
        return forceRef(QtGui.qApp.db.translate('rbOKFS', 'code', OKFS, 'id'))


    def getOKFSId(self, OKFS):
        if OKFS in self.OKFSCache:
            return self.OKFSCache[OKFS]
        else:
            result = self._getOKFSId(OKFS)
            self.OKFSCache[OKFS] = result
            return result

# *****************************************************************************************

    def processInsuranceType(self,  row):
        code = u'п'+ forceString(row['KGOS'])
        name = row['NGOS'].strip()

        attachTypeId = self.findAttachTypeByCode(code)

        if attachTypeId:
            self.updateAttachType(attachTypeId,  name)
            self.log(u'<b><font color=green>Обновлен</b>:'
                    u'тип прикрепления "%s". ' % name,  True)
            self.attachCache[row['KGOS']] = attachTypeId
        else:
            self.attachCache[row['KGOS']] = self.addAttachType(code,  name)
            self.log(u'<b><font color=blue>Добавлен</b>:'
                    u'тип прикрепления ("%s") "%s". ' % (code,  name),  True)


    def findAttachTypeByCode(self,  code):
        table = QtGui.qApp.db.table('rbAttachType')
        record = QtGui.qApp.db.getRecordEx(table, 'id', [
                                                          table['code'].eq(code),
                                                         ], 'id')

        return forceRef(record.value(0)) if record else None


    def updateAttachType(self,  id,  name):
        table = QtGui.qApp.db.table('rbAttachType')
        record = QtGui.qApp.db.getRecord(table, '*', id)
        record.setValue('name', toVariant(name))
        record.setValue('temporary', toVariant(1))
        record.setValue('outcome', toVariant(0))
        record.setValue('finance_id', toVariant(self.financeId))
        QtGui.qApp.db.updateRecord(table, record)


    def addAttachType(self, code, name):
        table = QtGui.qApp.db.table('rbAttachType')
        record = table.newRecord()
        record.setValue('code', toVariant(code))
        record.setValue('name', toVariant(name))
        record.setValue('temporary', toVariant(1))
        record.setValue('outcome', toVariant(0))
        record.setValue('finance_id', toVariant(self.financeId))
        return QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def updateKLADRInfisCode(self,  KLADRCode,  infisCode):
        table = QtGui.qApp.db.table('kladr.KLADR')
        table.setIdFieldName('CODE')
        record = QtGui.qApp.db.getRecordEx(table, 'CODE, infis',
            [table['CODE'].eq(KLADRCode),], 'CODE')
        record.setValue('infis', toVariant(infisCode))
        QtGui.qApp.db.updateRecord(table, record)

# *****************************************************************************************

    def findClientByNPP(self, npp):
        table = QtGui.qApp.db.table('ClientIdentification')
        record = QtGui.qApp.db.getRecordEx(table, 'client_id', [table['deleted'].eq(0),
                                                          table['accountingSystem_id'].eq(self.accountingSystemId),
                                                          table['identifier'].eq(npp)
                                                         ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None

# *****************************************************************************************

    def findClientByNameSexAndBirthDate(self, lastName, firstName, patrName, sex, birthDate):
        table = QtGui.qApp.db.table('Client')
        filter = [table['deleted'].eq(0),
                    table['lastName'].eq(lastName),
                    table['firstName'].eq(firstName),
                    table['patrName'].eq(patrName),
                    table['birthDate'].eq(birthDate)]
        if sex != 0:
            filter.append(table['sex'].eq(sex))

        record = QtGui.qApp.db.getRecordEx(table, 'id',  filter, 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None

# *****************************************************************************************

    def findServiceByCodeAndName(self, code,  name):
        table = QtGui.qApp.db.table('rbService')
        record = QtGui.qApp.db.getRecordEx(table, 'id',  [table['code'].eq(code),
                                                          table['name'].eq(name),
                                                         ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None

# *****************************************************************************************

    def findSpecialityByName(self, name):
        key = (name)
        result = self.mapSpecialityCodeNameToId.get(key)

        if not result:
            table = QtGui.qApp.db.table('rbSpeciality')
            record = QtGui.qApp.db.getRecordEx(table, 'id',  [
                                                        table['name'].eq(name),
                                                         ], 'id')
            if record:
                result = forceRef(record.value(0))
                self.mapSpecialityCodeNameToId[key] = result

        return result

# *****************************************************************************************

    def findActionTypeByCodeAndName(self, code,  name):
        key = (code,  name)
        result = self.mapActionTypeCodeNameToId.get(key)

        if not result:
            table = QtGui.qApp.db.table('ActionType')
            record = QtGui.qApp.db.getRecordEx(table, 'id',  [table['deleted'].eq(0),
                                                        table['code'].eq(code),
                                                        table['name'].eq(name),
                                                        table['class'].eq('3'),
                                                         ], 'id')
            if record:
                result = forceRef(record.value(0))
                self.mapActionTypeCodeNameToId[key] = result

        return result

# *****************************************************************************************

    def findClientBySNILS(self, SNILS):
        if SNILS:
            table = QtGui.qApp.db.table('Client')
            record = QtGui.qApp.db.getRecordEx(table, 'id',  [table['deleted'].eq(0),
                                                              table['SNILS'].eq(SNILS),
                                                             ], 'id')
            if record:
                return forceRef(record.value(0))

        return None

# *****************************************************************************************

    def addClient(self, npp, lastName, firstName, patrName, birthDate, sex,  SNILS):
        table = QtGui.qApp.db.table('Client')
        record = table.newRecord()
        record.setValue('lastName',  toVariant(lastName))
        record.setValue('firstName', toVariant(firstName))
        record.setValue('patrName',  toVariant(patrName))
        record.setValue('birthDate', toVariant(birthDate))
        record.setValue('sex',       toVariant(sex))
        record.setValue('SNILS',     toVariant(SNILS))
        clientId = QtGui.qApp.db.insertRecord(table, record)
        self.addClientIdentification(clientId, npp)
        return clientId

# *****************************************************************************************

    def updateClient(self, clientId, lastName, firstName, patrName, birthDate, sex,  SNILS):
        table = QtGui.qApp.db.table('Client')
        record = QtGui.qApp.db.getRecord(table, '*', clientId)
        if lastName:
            record.setValue('lastName', toVariant(lastName))
        if firstName:
            record.setValue('firstName', toVariant(firstName))
        if patrName:
            record.setValue('patrName', toVariant(patrName))
        if birthDate:
            record.setValue('birthDate', toVariant(birthDate))
        if sex:
            record.setValue('sex', toVariant(sex))
        if SNILS:
            record.setValue('SNILS', toVariant(SNILS))
        QtGui.qApp.db.updateRecord(table, record)

# *****************************************************************************************

    def addClientDocument(self, clientId, documentTypeId, serial, number):
        if documentTypeId:
            table = QtGui.qApp.db.table('ClientDocument')
            record = table.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('documentType_id', toVariant(documentTypeId))
            record.setValue('serial', toVariant(serial))
            record.setValue('number', toVariant(number))
            QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def updateClientDocument(self, clientId, documentTypeId, serial, number):
        if documentTypeId:
            #table = QtGui.qApp.db.table('ClientDocument')
#            record = QtGui.qApp.db.getRecordEx(table, '*', [table['client_id'].eq(clientId),
#                                                      table['serial'].eq(serial),
#                                                      table['number'].eq(number)], 'id')
            record = selectLatestRecord('ClientDocument', clientId)
            oldSerial = oldNumber = oldDocTypeId = None

            if record:
                oldSerial = forceString(record.value('serial'))
                oldNumber = forceString(record.value('number'))
                oldDocTypeId = forceRef(record.value('documentType_id'))

            if not record or (oldSerial != serial or oldNumber != number or oldDocTypeId != documentTypeId):
                self.addClientDocument(clientId, documentTypeId, serial, number)

# *****************************************************************************************

    def addClientPolicy(self, clientId, insurerId, serial, number, begDate, endDate,  note,  policyTypeId):
        table = QtGui.qApp.db.table('ClientPolicy')
        record = table.newRecord()
        record.setValue('client_id', toVariant(clientId))
        record.setValue('insurer_id', toVariant(insurerId))
        record.setValue('policyType_id', toVariant(policyTypeId))
        record.setValue('serial', toVariant(serial))
        record.setValue('number', toVariant(number))
        record.setValue('begDate', toVariant(begDate))
        record.setValue('endDate', toVariant(endDate))
        record.setValue('note',  toVariant(note))
        QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def updateClientPolicy(self, clientId, insurerId, serial, number, begDate, endDate,  note,  policyTypeId):
        table = QtGui.qApp.db.table('ClientPolicy')
        record = selectLatestRecord('ClientPolicy', clientId,
            '(Tmp.`policyType_id`=%d)' % policyTypeId)

        if record and (
            (forceRef(record.value('insurer_id')) == insurerId) and
            (forceString(record.value('serial')) == serial) and
            (forceInt(record.value('number')) == number)
            ):
            if forceDate(record.value('begDate')) != begDate or forceDate(record.value('endDate')) != endDate:
                record.setValue('begDate', toVariant(begDate))
                record.setValue('endDate', toVariant(endDate))
                QtGui.qApp.db.updateRecord(table, record)
            if forceString(record.value('note')) != note:
                record.setValue('note', toVariant(note))
                QtGui.qApp.db.updateRecord(table, record)
        else:
            self.addClientPolicy(clientId, insurerId, serial, number, begDate, endDate,  note,  policyTypeId)

# *****************************************************************************************
    regionNameToKLADRCodeCache = {}

    def getKLADRRegionCode(self, region):
        code = self.regionNameToKLADRCodeCache.get(region)

        if not code:
            code = forceString(QtGui.qApp.db.translate('kladr.KLADR', 'NAME',  region,  'CODE'))
            self.regionNameToKLADRCodeCache[region] = code

        return code

# *****************************************************************************************
    cityNameToKLADRCodeCache = {}

    def getKLADRCityCode(self, city, KLADRCode):
        key = (city,  KLADRCode)
        code = self.cityNameToKLADRCodeCache.get(key)

        if not code:
            table = QtGui.qApp.db.table('kladr.KLADR')
            filter = [table['NAME'].eq(city)]

            if KLADRCode:
                filter.append(table['CODE'].like(KLADRCode[:2]+'%'))

            record = QtGui.qApp.db.getRecordEx(table, 'CODE', filter,  'CODE')

            if record:
                code = forceString(record.value(0))
                self.cityNameToKLADRCodeCache[key] = code

        return code


    affixes = [u'б-р', u'бульвар', u'ул', u'улица', u'пер', u'пр', u'проспект', u'пр-т', u'наб', u'ш', u'шоссе']

# *****************************************************************************************

    streetNameToKLADRStreetCodeCache = {}

    def getKLADRStreetCode(self, KLADRCode, street):
        key = (KLADRCode,  street)
        result = self.streetNameToKLADRStreetCodeCache.get(key)

        if not result:
            srcStreet = street

            if KLADRCode:
                if street == '' or street == '-':
                    result = KLADRCode
                else:
                    if street.count(','): # нас. пункт, улица, д, кв.
                        street = trim(street.split(',')[1])
                    street = re.sub(ur'(,?д\.?[ 0-9]+.*$)', '', street) # удаляем приписанные номера домов в виде д.12
                    street = re.sub(r'[0-9-/ ]*$', '', street) # удаляем приписанные номера домов в виде 12/3-1
                    loCaseStreet = street.lower()
                    for affix in self.affixes:
                        if loCaseStreet.startswith(affix+'.'):
                            street = trim(street[len(affix)+1:])
                            break
                        if loCaseStreet.startswith(affix+' '):
                            street = trim(street[len(affix)+1:])
                            break
                        elif loCaseStreet.endswith(' '+affix):
                            street = trim(street[:-len(affix)])
                            break
                        elif loCaseStreet.endswith(' '+affix+'.'):
                            street = trim(street[:-len(affix)-1])
                            break

                    table = QtGui.qApp.db.table('kladr.STREET')
                    record = QtGui.qApp.db.getRecordEx(table, 'CODE', [table['NAME'].eq(street),
                                                                         table['CODE'].like(KLADRCode[:-2]+'%'),
                                                                     ], 'CODE')
                    if record:
                        result = forceString(record.value('CODE'))

                self.streetNameToKLADRStreetCodeCache[key] = result

        return result

# *****************************************************************************************

    def getAddressHouseId(self, placeCode, street, number, corpus):
        if not placeCode or not street:
            return None

        KLADRCode = placeCode
        KLADRStreetCode = street
        table = QtGui.qApp.db.table('AddressHouse')
        record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                        table['KLADRCode'].eq(KLADRCode),
                                                        table['KLADRStreetCode'].eq(KLADRStreetCode),
                                                        table['number'].eq(number),
                                                        table['corpus'].eq(corpus)], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            record = table.newRecord()
            record.setValue('KLADRCode', toVariant(KLADRCode))
            record.setValue('KLADRStreetCode', toVariant(KLADRStreetCode))
            record.setValue('number', toVariant(number))
            record.setValue('corpus', toVariant(corpus))
            return QtGui.qApp.db.insertRecord(table, record)


# *****************************************************************************************

    def getAddressId(self, placeCode, street, number, corpus, flat, address):
        houseId = self.getAddressHouseId(placeCode, street, number, corpus)
        if houseId:
            table = QtGui.qApp.db.table('Address')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                            table['house_id'].eq(houseId),
                                                            table['flat'].eq(flat)], 'id')
            if record:
                return forceRef(record.value(0))
            else:
                record = table.newRecord()
                record.setValue('house_id', toVariant(houseId))
                record.setValue('flat', toVariant(flat))
                return QtGui.qApp.db.insertRecord(table, record)
        else:
            return None

# *****************************************************************************************

    def ensureClientAddress(self, clientId, addressType, addressId, freeInput):
        table = QtGui.qApp.db.table('ClientAddress')
        record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                  table['client_id'].eq(clientId),
                                                  table['type'].eq(addressType),
                                                  table['address_id'].eq(addressId),
                                                  table['freeInput'].eq(freeInput),
                                                 ], 'id')
        if not record:
            record = table.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('type',      toVariant(addressType))
            record.setValue('address_id',toVariant(addressId))
            record.setValue('freeInput', toVariant(freeInput))
            QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def addClientAddress(self, clientId, placeCode, street, number, corpus, flat, freeInput):
        addressId = self.getAddressId(placeCode, street, number, corpus, flat, freeInput)
        self.ensureClientAddress(clientId, 0, addressId, freeInput)
        self.ensureClientAddress(clientId, 1, addressId, freeInput)

# *****************************************************************************************

    def updateClientAddress(self, clientId, placeCode, street, number, corpus, flat, freeInput):
        addressId = self.getAddressId(placeCode, street, number, corpus, flat, freeInput)
        self.ensureClientAddress(clientId, 0, addressId, freeInput)
        self.ensureClientAddress(clientId, 1, addressId, freeInput)

# *****************************************************************************************

    def ensureClientAttach(self, clientId, attachTypeId, orgId,  begDate,  endDate):
        if not orgId or not attachTypeId:
            return

        checkBegDate = begDate if begDate else QDate.currentDate()
        checkEndDate = endDate if endDate else QDate.currentDate()

        record = selectLatestRecord('ClientAttach', clientId,
            '(Tmp.`attachType_id`=%d) AND \
         (Tmp.`LPU_id`=%d) AND \
         (Tmp.`begDate`=\'%s\') AND \
         (Tmp.`endDate`=\'%s\')' % \
            (attachTypeId, orgId, checkBegDate.toString(QtCore.Qt.ISODate),
             checkEndDate.toString(QtCore.Qt.ISODate)))

        if not record:
            table = QtGui.qApp.db.table('ClientAttach')
            record = table.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('attachType_id',  toVariant(attachTypeId))
            record.setValue('LPU_id',toVariant(orgId))
            if begDate:
                record.setValue('begDate',toVariant(begDate))
            if endDate:
                record.setValue('endDate', toVariant(endDate))
            QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def deleteClientClosedAttach(self, clientId, orgId,  begDate,  endDate):
        if not orgId:

            checkBegDate = begDate if begDate else QDate.currentDate()
            checkEndDate = endDate if endDate else QDate.currentDate()

            table = QtGui.qApp.db.table('ClientAttach')
            record = QtGui.qApp.db.getRecordEx(table, '*',[table['deleted'].eq(0),
                                                      table['client_id'].eq(clientId),
                                                      table['attachType_id'].eq(self.attachClose),
                                                      table['LPU_id'].eq(orgId),
                QtGui.qApp.db.joinOr([table['begDate'].isNull(), table['begDate'].le(checkBegDate)]),
                QtGui.qApp.db.joinOr([table['endDate'].isNull(), table['endDate'].ge(checkEndDate)]),
                                                     ], 'id')
            if record:
                record.setValue('deleted', toVariant(1))
                QtGui.qApp.db.updateRecord(table, record)
                return True

        return False

# *****************************************************************************************

    def addClientAttach(self,  clientId,  orgId,  attachTypeId,  begDate=None,  endDate=None):
        self.ensureClientAttach(clientId,  attachTypeId,  orgId,  begDate,  endDate)

# *****************************************************************************************

    def updateClientAttach(self,  clientId,  orgId,  attachTypeId,  begDate=None,  endDate=None):
        self.ensureClientAttach(clientId,  attachTypeId,  orgId,  begDate,  endDate)

# *****************************************************************************************

    def addClientIdentification(self, clientId, npp):
        table = QtGui.qApp.db.table('ClientIdentification')
        record = table.newRecord()
        record.setValue('client_id', toVariant(clientId))
        record.setValue('accountingSystem_id', toVariant(self.accountingSystemId))
        record.setValue('identifier', toVariant(npp))
        record.setValue('checkDate', toVariant(QDate.currentDate()))
        clientId = QtGui.qApp.db.insertRecord(table, record)
#        self.addChange('ClientIdentification', 1)

# *****************************************************************************************

    def updateClientIdentification(self, clientId, npp):
        table = QtGui.qApp.db.table('ClientIdentification')
        record = QtGui.qApp.db.getRecordEx(table, '*',   [table['deleted'].eq(0),
                                                          table['accountingSystem_id'].eq(self.accountingSystemId),
                                                          table['client_id'].eq(clientId),
                                                         ], 'id')
        if record:
            record.setValue('identifier', toVariant(npp))
            record.setValue('checkDate', toVariant(QDate.currentDate()))
            QtGui.qApp.db.updateRecord(table, record)
#            self.addChange('ClientIdentification', 2)
        else:
            self.addClientIdentification(clientId, npp)

# *****************************************************************************************

    def getClientIdentification(self, clientId):
        table = QtGui.qApp.db.table('ClientIdentification')
        record = QtGui.qApp.db.getRecordEx(table, 'identifier',   [table['deleted'].eq(0),
                                                          table['accountingSystem_id'].eq(self.accountingSystemId),
                                                          table['client_id'].eq(clientId),
                                                         ], 'id')

        return forceString(record.value(0)) if record else None

# *****************************************************************************************

    def getOrganisationId(self, INN, KPP):
        if INN and KPP:
            table = QtGui.qApp.db.table('Organisation')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                      table['INN'].eq(INN),
                                                      table['KPP'].eq(KPP)], 'id')
            if record:
                return forceRef(record.value(0))
            elif self.addDummyOrganisations:
                record = table.newRecord()
                name = u'ИНН: %s; КПП: %s' % (INN, KPP)
                record.setValue('fullName', toVariant(name))
                record.setValue('shortName', toVariant(name))
                record.setValue('INN', toVariant(INN))
                record.setValue('KPP', toVariant(KPP))
                return QtGui.qApp.db.insertRecord(table, record)
        return None

# *****************************************************************************************

    def addClientWork(self, clientId, organisationId,  organisationName):
        if clientId and (organisationId or organisationName):
            table = QtGui.qApp.db.table('ClientWork')
            record = table.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('org_id', toVariant(organisationId))
            if not organisationId:
                record.setValue('freeInput', toVariant(organisationName))
            QtGui.qApp.db.insertRecord(table, record)


    def updateClientWork(self, clientId, organisationId,  organisationName):
        if organisationId:
#            table = QtGui.qApp.db.table('ClientWork')
#            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
#                                                      table['client_id'].eq(clientId),
#                                                      table['org_id'].eq(organisationId)], 'id')
            record = selectLatestRecord('ClientWork', clientId,
                '(Tmp.`org_id`=%d)' % \
                (organisationId))
            if not record:
                self.addClientWork(clientId, organisationId,  organisationName)
                return

        if organisationName:
            record = selectLatestRecord('ClientWork', clientId,
                '(Tmp.`freeInput`=%s)' % \
                (organisationName))

            if not record:
                self.addClientWork(clientId, organisationId,  organisationName)


# *****************************************************************************************

    def addClientPhone(self, clientId, phone):
        if phone and phone != '0':
            table = QtGui.qApp.db.table('ClientContact')
            record = table.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('contactType_id', toVariant(self.contactTypeId))
            record.setValue('contact', toVariant(phone))
            QtGui.qApp.db.insertRecord(table, record)


    def updateClientPhone(self, clientId, phone):
        if phone and phone != '0':
            table = QtGui.qApp.db.table('ClientContact')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                      table['client_id'].eq(clientId),
                                                      table['contactType_id'].eq(self.contactTypeId),
                                                      table['contact'].eq(phone)], 'id')
            if not record:
                self.addClientPhone(clientId, phone)


    def checkBirthDate(self, birthDate, lastName, firstName, patrName):
        if birthDate:
            if birthDate>QDate.currentDate():
                self.log(u'<b><font color=red>ОШИБКА</b>:'
                    u'Дата рождения клиента "%s %s %s" находится в будущем (`%s`). Пропускаем.' %\
                    (lastName, firstName, patrName,  birthDate.toString('dd.MM.yyyy')),  True)
                return False

            if QDate.currentDate().year() - birthDate.year() > 130:
                self.log(u'<b><font color=red>ОШИБКА</b>:'
                    u'Возраст клиента "%s %s %s" больше 130 лет!(д.р. `%s`). Пропускаем.' %\
                    (lastName, firstName, patrName,  birthDate.toString('dd.MM.yyyy')),  True)
                return False

        return True

# *****************************************************************************************

    documentTypeCache = {}

    def getDocumentTypeId(self, regionalCode):
        result = self.documentTypeCache.get(regionalCode, -1)
        if result == -1:
            result = forceRef(QtGui.qApp.db.translate('rbDocumentType', 'regionalCode', code, 'id'))
            self.documentTypeCache[regionalCode] = result
        return result


# *****************************************************************************************

    def updateClientSocStatus(self,  clientId,  classId, statusTypeId,  overwriteSocStatus=False):
        if classId and statusTypeId:
#            table = QtGui.qApp.db.table('ClientSocStatus')
#            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
#                                                    table['socStatusClass_id'].eq(classId),
#                                                    table['socStatusType_id'].eq(statusTypeId)], 'id')
            record = selectLatestRecord('ClientSocStatus', clientId,
                '(Tmp.`socStatusClass_id`=%d)' % \
                (classId))

            oldStatusTypeId = forceRef(record.value('socStatusType_id')) if record else None

            if not record or oldStatusTypeId != statusTypeId:
                if record and overwriteSocStatus:
                    record.setValue('socStatusType_id', toVariant(statusTypeId))
                    QtGui.qApp.db.updateRecord('ClientSocStatus', record)
                else:
                    self.addClientSocStatus(clientId, classId, statusTypeId)



    def addClientSocStatus(self,  clientId,  classId,  statusTypeId):
        if classId and statusTypeId:
            table = QtGui.qApp.db.table('ClientSocStatus')
            record = table.newRecord()
            record.setValue('socStatusType_id',  toVariant(statusTypeId))
            record.setValue('socStatusClass_id',  toVariant(classId))
            record.setValue('client_id',  toVariant(clientId))
            QtGui.qApp.db.insertRecord(table,  record)

# *****************************************************************************************

    socStatusTypeCache = {}

    def getSocStatusType(self, regionalCode):
        result = self.socStatusTypeCache.get(regionalCode)

        if result is None:
            table = QtGui.qApp.db.table('rbSocStatusType')
            record = QtGui.qApp.db.getRecordEx(table, 'id', table['regionalCode'].eq(regionalCode), 'id')

            if record:
                result = forceRef(record.value(0))
            else:
                self.log(u'<b><font color=red>ОШИБКА</b>:'\
                    u' Соц. статус с региональным кодом `%d` не найден в БД' % regionalCode,  True)
                result = 0

            self.socStatusTypeCache[regionalCode] = result

        return result

# *****************************************************************************************

    def processClient(self, record):
        lastName  = nameCase(forceString(record['FAM'].strip()))
        firstName = nameCase(forceString(record['IM'].strip()))
        patrName  = nameCase(forceString(record['OT'].strip()))
        sex       = forceInt(record['SEX']) + 1
        birthDate = QDate(record['BIRTHDAY']) if record['BIRTHDAY'] else None
        SNILS = forceString(forceString(record['SS'])).strip().replace('-', '').replace(' ', '')

        if not self.checkBirthDate(birthDate, lastName, firstName, patrName):
            self.nSkipped +=1
            return

        documentTypeId = self.getDocumentTypeId(forceString(record['C_P']).strip())
        documentSerial = forceString(record['SER_DOC']).strip()
        documentNumber = forceString(record['NUM_DOC']).strip()

        KLADRStreetCode = forceString(record['C_ADS']).strip()
        freeInputAddress = ''

        cityName = forceString(record['TOWN']).strip()
        streetName = forceString(record['STREET']).strip()
        house = forceString(record['HOUSE']).strip()
        corpus = forceString(record['BUILDING']).strip()
        flat = forceString(record['FLAT']).strip()
        freeInput = []

        for x in (cityName,  streetName):
            if x:
                freeInput.append(x)

        if house:
            freeInput.append(u'д. %s' % house)
        if corpus:
            freeInput.append(u'корп. %s' % corpus)
        if flat:
            freeInput.append(u'кв. %s' % flat)

        freeInputAddress = u','.join([x for x in freeInput])

        policySerial = forceString(record['SER_SP']).strip().replace(' ', '')
        policyNumber = forceString(record['NUM_SP']).strip()
        policyNote = u'Дата выдачи: ' + forceString(record['DATE_SP']).strip()
        policyInsurer = self.getInsurerId(forceString(record['COMPANY']).strip())
        policyBegDate = QDate(record['DATE_BEGIN']) if record['DATE_BEGIN'] else None
        policyEndDate = QDate(record['DATE_END']) if record['DATE_END'] else None
        policyInvalidCode = forceString(record['TYP']).strip()
        policyContractNumber = forceString(record['ORG']).strip()

        if policyInvalidCode and policyInvalidCode != '0':
            policyNote += u', %s' % self.policyReasonOfInvalidity.get(policyInvalidCode,  '')
        if policyContractNumber:
            policyNote += u', договор №: %s' % policyContractNumber

        clientWorkId = None
        clientWork = ''
        clientSocStatusId = None
        socStatusCode = forceInt(record['WRK'])

        if socStatusCode == 0:
            clientWorkINN = forceString(record['PS_INN']).strip()
            clientWorkKPP = forceString(record['PS_KPP']).strip()
            clientWorkId = self.findOrgId(clientWorkINN,  clientWorkKPP)

            if not clientWorkId:
                clientWork = u'ИНН: %s, КПП: %s' % (clientWorkINN,  clientWorkKPP)

            clientSocStatusId = self.socStatusEmployedId
            policyTypeId = self.policyTypeOMSIndustrialId
        else:
            # безработный
            clientSocStatusId = self.getSocStatusType(socStatusCode)
            policyTypeId = self.policyTypeOMSTerritorialId

        clientId = self.findClientBySNILS(SNILS)
        if not clientId:
            clientId = self.findClientByNameSexAndBirthDate(lastName, firstName, patrName, sex, birthDate)

        if clientId:
#            self.log(u'<b><font color=brown>Обновляем</b>:'
#                u' %s %s %s' % (lastName, firstName, patrName))
            overwriteSocStatus = False

            if clientId in self.skipClientSet:
                self.log(u'<b><font color=blue>Пропуск</b>:' \
                u' клиента %s %s %s, т.к. была запись с датой 31.12.2999' % (lastName, firstName, patrName))
                return

            if policyEndDate == QDate(2999, 12, 31):
                overwriteSocStatus = True
                self.skipClientSet.add(clientId)

            self.updateClient(clientId, lastName, firstName, patrName, birthDate, sex, SNILS)
            self.updateClientPolicy(clientId, policyInsurer, policySerial, policyNumber,
                                    policyBegDate, policyEndDate,  policyNote,  policyTypeId)
            self.updateClientAddress(clientId, KLADRStreetCode[:13], KLADRStreetCode, house, corpus, flat, freeInputAddress)
            self.updateClientWork(clientId, clientWorkId,  clientWork)
            self.updateClientSocStatus(clientId, self.socClass9Id,  clientSocStatusId,  overwriteSocStatus)
            self.updateClientDocument(clientId,  documentTypeId,  documentSerial,  documentNumber)
            self.nUpdated += 1
        else:
#            self.log(u'<b><font color=green>Добавляем</b>:'
#                u' %s %s %s' % (lastName, firstName, patrName))
            clientId = self.addClient(None,  lastName, firstName, patrName, birthDate, sex, SNILS)
            self.addClientPolicy(clientId, policyInsurer, policySerial, policyNumber,
                                 policyBegDate, policyEndDate,  policyNote, policyTypeId)
            self.addClientAddress(clientId, KLADRStreetCode[:13], KLADRStreetCode, house, corpus, flat, freeInputAddress)
            self.addClientWork(clientId, clientWorkId, clientWork)
            self.addClientSocStatus(clientId, self.socClass9Id, clientSocStatusId)
            self.addClientDocument(clientId, documentTypeId,  documentSerial,  documentNumber)
            self.nAdded += 1

# *****************************************************************************************

    def findOrgId(self, INN, KPP):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecordEx(table, 'id',  [table['deleted'].eq(0),
                                                          table['INN'].eq(INN),
                                                          table['KPP'].eq(KPP)
                                                         ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None

# *****************************************************************************************

    def findOrgByName(self, fullName):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecordEx(table, 'id',  [table['deleted'].eq(0),
                                                          table['fullName'].eq(fullName)], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None

# *****************************************************************************************

    def findOrgByShortName(self, shortName):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecordEx(table, 'id',  [table['deleted'].eq(0),
                                                          table['shortName'].eq(shortName)], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None

# *****************************************************************************************

    def addOrg(self, shortName, fullName, INN, KPP, OKPO, OKVED, OGRN,
                        address, chief, accountant,  phone,  fax,  email,  masterId,
                        infisCode):
        table = QtGui.qApp.db.table('Organisation')
        record = table.newRecord()
        record.setValue('fullName', toVariant(fullName))
        record.setValue('shortName', toVariant(shortName))
        record.setValue('title', toVariant(shortName))
        record.setValue('OKPO', toVariant(OKPO))
        record.setValue('OKVED', toVariant(OKVED))
        record.setValue('INN', toVariant(INN))
        record.setValue('KPP', toVariant(KPP))
        record.setValue('OGRN', toVariant(OGRN))
        record.setValue('address', toVariant(address))
        record.setValue('chief', toVariant(chief))
        record.setValue('accountant', toVariant(accountant))
        record.setValue('phone', toVariant(phone))
        record.setValue('net_id',  toVariant(self.matureNetId))

        notes = u''

        if fax:
            notes += u'факс: %s' % fax

        if email:
            notes += u' email: %s' % email

        if notes != '':
            record.setValue('notes', toVariant(notes))

        if masterId:
            record.setValue('head_id',  toVariant(masterId))

        record.setValue('infisCode',  toVariant(infisCode))
        return QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def updateOrg(self, orgId, shortName, fullName, OKPO, OKVED, OGRN,
                            address, chief, accountant,  phone,  fax,  email, masterId,
                            infisCode):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecord(table, '*', orgId)
        record.setValue('fullName', toVariant(fullName))
        record.setValue('shortName', toVariant(shortName))
        record.setValue('title', toVariant(shortName))
        record.setValue('OKPO', toVariant(OKPO))
        record.setValue('OKVED', toVariant(OKVED))
        record.setValue('OGRN', toVariant(OGRN))
        record.setValue('address', toVariant(address))
        record.setValue('chief', toVariant(chief))
        record.setValue('accountant', toVariant(accountant))
        record.setValue('phone', toVariant(phone))
        record.setValue('net_id',  toVariant(self.matureNetId))
        notes = u''

        if fax:
            notes += u'факс: %s' % fax

        if email:
            notes += u' email: %s' % email

        if notes != '':
            record.setValue('notes', toVariant(notes))

        if masterId:
            record.setValue('head_id',  toVariant(masterId))

        record.setValue('infisCode',  toVariant(infisCode))
        return QtGui.qApp.db.updateRecord(table, record)

# *****************************************************************************************

    def addService(self, code,  name, infisCode, begDate, endDate):
        table = QtGui.qApp.db.table('rbService')
        record = table.newRecord()
        record.setValue('code', toVariant(code))
        record.setValue('name', toVariant(name))

        if infisCode:
            record.setValue('infis', toVariant(infisCode))
            record.setValue('eisLegacy', toVariant(True))

        record.setValue('begDate',  toVariant(begDate))
        record.setValue('endDate',  toVariant(endDate))
        return QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def updateService(self, serviceId, infisCode, begDate, endDate):
        table = QtGui.qApp.db.table('rbService')
        record = QtGui.qApp.db.getRecord(table, '*', serviceId)

        if infisCode:
            record.setValue('infis', toVariant(infisCode))
            record.setValue('eisLegacy', toVariant(True))

        record.setValue('begDate',  toVariant(begDate))
        record.setValue('endDate',  toVariant(endDate))
        return QtGui.qApp.db.updateRecord(table, record)

# *****************************************************************************************

    def ensureSpecialityRegionalCode(self, specialityId, regionalCode):
        table = QtGui.qApp.db.table('rbSpeciality')
        record = QtGui.qApp.db.getRecord(table, '*', specialityId)
        if forceString(record.value('regionalCode')) == '':
            record.setValue('regionalCode',  toVariant(regionalCode))
            return QtGui.qApp.db.updateRecord(table, record)

# *****************************************************************************************

    def addActionType(self, code,  name, groupId,  serviceId):
        table = QtGui.qApp.db.table('ActionType')
        record = table.newRecord()
        record.setValue('code', toVariant(code))
        record.setValue('name', toVariant(name))
        record.setValue('class', toVariant(3)) # прочее

        if groupId:
            record.setValue('group_id', toVariant(groupId))

        if serviceId:
            record.setValue('service_id',  toVariant(serviceId))

        return QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def updateActionType(self, id, groupId,  serviceId):
        table = QtGui.qApp.db.table('ActionType')
        record = QtGui.qApp.db.getRecord(table, '*', id)
        if groupId:
            record.setValue('group_id', toVariant(groupId))

        if serviceId:
            record.setValue('service_id',  toVariant(serviceId))
        return QtGui.qApp.db.updateRecord(table, record)
