# -*- coding: utf-8 -*-

#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################


from PyQt4 import QtGui
from PyQt4.QtCore import QXmlStreamReader

from library.Utils import forceString, anyToUnicode

# ******************************************************************************

class CXmlStreamReader(QXmlStreamReader):

    groupNames = ('ZAP', 'PACIENT', 'SLUCH', 'USL', 'SCHET', 'ZGLV', 'ZL_LIST')

    def __init__(self, parent, logBrowser, showLog=False):
        QXmlStreamReader.__init__(self)
        self._parent = parent
        self.db = QtGui.qApp.db
        self.logBrowser = logBrowser
        self.showLog = showLog

    def raiseError(self, msg):
        QXmlStreamReader.raiseError(self, u'[%d:%d] %s' % (
            self.lineNumber(), self.columnNumber(), msg))


    def log(self, msg, forceLog=False):
        if self.showLog or forceLog:
            self.logBrowser.append(msg)
            self.logBrowser.update()


    def readHeader(self):
        u''' Разбирает заголовок'''

        try:
            while not self.atEnd():
                self.readNext()

                if self.isStartElement():
                    if self.name() == 'ZL_LIST':
                            return True
                    else:
                        self.raiseError(u'Неверный формат экспорта данных.')

        except IOError, e:
            QtGui.qApp.logCurrentException()
            msg = ''
            if hasattr(e, 'filename'):
                msg = u'%s:\n[Errno %s] %s' % (e.filename, e.errno,
                                               anyToUnicode(e.strerror))
            else:
                msg = u'[Errno %s] %s' % (e.errno, anyToUnicode(e.strerror))
            QtGui.QMessageBox.critical(self._parent,
                u'Произошла ошибка ввода-вывода', msg, QtGui.QMessageBox.Close)
            self.log(u'! Ошибка ввода-вывода: %s' % msg, True)

        except Exception, e:
            QtGui.qApp.logCurrentException()
            QtGui.QMessageBox.critical(self._parent,
                u'Произошла ошибка', unicode(e), QtGui.QMessageBox.Close)
            self.log(u'! Ошибка: %s' % unicode(e), True)

        return False


    def readData(self):
        assert self.isStartElement() and self.name() in self.groupNames

        # очищаем список событий перед заполнением

        while not self.atEnd():
            QtGui.qApp.processEvents()
            self.readNext()

            if self.isEndElement():
                break

            if self.isStartElement():
                if self.name() in self.groupNames:
                    yield self.readRow()
                else:
                    self.readUnknownElement()

            if self.hasError() or self._parent.aborted:
                break


    def readValue(self):
        assert self.isStartElement()
        result = None

        if self._parent.aborted:
            return None

        while not self.atEnd():
            self.readNext()

            if self.isEndElement():
                break

            if self.isStartElement():
                name = forceString(self.name())
                if name == 'VALUE':
                    result = forceString(self.readElementText())
                else:
                    self.readUnknownElement()

        return result


    def readUnknownElement(self, report=True):
        """ Читает неизвестный элемент, и сообщает об этом,
            если report ==True """

        assert self.isStartElement()

        if report:
            self.log(u'Неизвестный элемент: '+self.name().toString())

        while not self.atEnd():
            self.readNext()

            if self.isEndElement():
                break

            if self.isStartElement():
                self.readUnknownElement(report)

            if self.hasError() or self._parent.aborted:
                break


    def readRow(self):
        assert self.isStartElement() and self.name() in self.groupNames

        groupName = forceString(self.name())
        result = {}

        if self._parent.aborted:
            return None

        while not self.atEnd():
            self.readNext()

            if self.isEndElement():
                break

            if self.isStartElement():
                name = forceString(self.name())

                if name in self.groupNames:
                    _, row = self.readRow()

                    if result.has_key(name):
                        if not isinstance(result[name], list):
                            result[name] = [result[name]]
                        result[name].append(row)
                    else:
                        result[name] = row
                else:
                    result[name] = forceString(self.readElementText())

        return groupName, result
