# -*- coding: utf-8 -*-

#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################


import os.path

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import *

from library.Utils import *
from library.roman import itor
from library.dbfpy.dbf import Dbf
from Registry.Utils import selectLatestRecord


from Ui_Progress import Ui_ProgressDialog


class CProgressDialog(QtGui.QDialog, Ui_ProgressDialog):
    __pyqtSignals__ = ('startWork()', )

    def __init__(self, parent):
        QtGui.QDialog.__init__(self, parent)
        self.parent = parent #wtf? скрывать Qt-шный parent - это плохая затея
        self.setupUi(self)
        self.setPlacesCount(0)
        self.setOrgsCount(0)
        self.setClientsCount(0)

        self.start()
        self.connect(self, SIGNAL('startWork()'), self.startWork, Qt.QueuedConnection)

        self.policyTypeId       = None
        self.passportRFId       = None
        self.passportUSSRId     = None
        self.birthCertificateId = None
        self.temporaryCertificateId = None
        self.contactTypeId      = None
        self.clientIdentificationId = None
        self.accountingSystemId = None
        self.attachTypeId = None
        self.matureNetId =  None
        self.socClassBenefitId = None
        self.socClass9Id = None
        self.actionTypeParentGroupId = None

        self.sexMap = {u'Ж':2, u'ж':2, u'М':1, u'м':1}
        self.policyStatusMap = {'1':u'выдан первоначально',
                                '3':u'выдан взамен утерянного',
                                '4':u'заменен по изменению реквизита',
                                '5':u'выдан взамен погашенного',
                                u'А': u'уточнены реквизиты'}

        self.mapAreaToKLADR = {}
        self.mapAreaAndPlaceToKLADR = {}
        self.areaMap = {}
        self.regionMap = {}
        self.orgMap = {}
        self.unemployedSet = set()

        self.insurerCache = {}
        self.OKFSCache = {}
        self.showLog = True
        self.skipOrgs = False

        self.db = QtGui.qApp.db
        self.tableClient = self.db.table('Client')


    def log(self, str, forceLog = False):
        if self.showLog or forceLog:
            self.logBrowser.append(str)
            self.logBrowser.update()


    def _setCount(self, progressBar, cnt):
        if cnt:
            progressBar.setValue(0)
            progressBar.setMaximum(cnt)
            progressBar.setEnabled(True)
            progressBar.setTextVisible(True)
        else:
            progressBar.setValue(0)
            progressBar.setMaximum(1)
            progressBar.setEnabled(False)
            progressBar.setTextVisible(False)


    def setPlacesCount(self, cnt):
        self._setCount(self.prbPlaces, cnt)


    def setClientsCount(self, cnt):
        self._setCount(self.prbClients, cnt)


    def setOrgsCount(self, cnt):
        self._setCount(self.prbOrgs, cnt)


    def start(self):
        self.processDone = False
        self.aborted = False
        self.btnCancel.setText(u'Отмена')


    def stop(self):
        self.processDone = True
        self.btnCancel.setText(u'Закрыть')


    def done(self, r):
        self.stop()
        QtGui.QDialog.done(self, r)

    def exec_(self):
        self.emit(SIGNAL('startWork()'))

        return QtGui.QDialog.exec_(self)


    @pyqtSignature('')
    def on_btnCancel_clicked(self):
        if self.processDone:
            self.accept()
        else:
            self.aborted = True


    def startWork(self):
        self.start()
        QtGui.qApp.call(self, self.work)
        self.stop()


    def work(self):
        rbDir = forceString(self.parent.edtRBDirName.text())

        dbfCounty = Dbf(os.path.join(rbDir, u'Приложение 34 Административное деление РФ.DBF'), readOnly=True, encoding='cp866')
        dbfRegion = Dbf(os.path.join(rbDir, u'Приложение 7 Справочник адмистративного деления Смоленской области.DBF'), readOnly=True, encoding='cp866')
        dbfOrgs = Dbf(forceString(self.parent.edtOrgDataFileName.text()), readOnly=True, encoding='cp866')
        dbfRegistry = Dbf(forceString(self.parent.edtMainDataFileName.text()), readOnly=True, encoding='cp866')

        self.setPlacesCount(len(dbfCounty)+len(dbfRegion))
        self.process(dbfCounty, self.processArea, self.prbPlaces)
        self.process(dbfRegion, self.processRegion, self.prbPlaces)
        self.setOrgsCount(len(dbfOrgs))
        self.process(dbfOrgs, self.processOrg, self.prbOrgs)

        self.setClientsCount(len(dbfRegistry))
        self.process(dbfRegistry, self.processClient, self.prbClients)

        for d in (dbfRegistry, dbfCounty, dbfRegion, dbfOrgs):
            d.close()


    def process(self, dbf, step, progressBar):
        self.lblElapsed.setText(u'Текущая операция: ??? зап/с, окончание в ??:??:??')
        time = QTime()
        time.start()
        startPos = 0
        oldSpeed = 0

        for record in dbf:
            QtGui.qApp.processEvents()
            if self.aborted:
                self.reject()
                return
            progressBar.setValue(progressBar.value()+1)
            QtGui.qApp.db.transaction()
            try:
                step(record)
                QtGui.qApp.db.commit()
            except:
                QtGui.qApp.db.rollback()
                raise

            elapsed =time.elapsed()

            if elapsed != 0 and (progressBar.value()-startPos>100):
                startPos = progressBar.value()
                oldSpeed = 100*1000/elapsed
                newSpeed = (oldSpeed+100*1000/elapsed)/2
                if newSpeed != 0:
                    finishTime = QTime.currentTime().addSecs(progressBar.maximum()/newSpeed)
                    self.lblElapsed.setText(u'Текущая операция: %03.f зап/с, окончание в %s' % (newSpeed, finishTime.toString('hh:mm:ss')))
                    time.restart()

# *****************************************************************************************

    def processArea(self, record):
        code = forceString(record['CODE'])
        name = forceString(record['NAME'])
        self.areaMap[code] = name

# *****************************************************************************************

    def processRegion(self, record):
        code = forceString(record['CODE'])
        name = forceString(record['NAME'])
        self.regionMap[code] = name


# *****************************************************************************************

    def processOrg(self, row):
        code = forceString(row['NDOG']).strip()

        if forceBool(row['UNEMP']):
            self.unemployedSet.add(code)
        else:
            INN = forceString(row['INN']).strip()
            KPP = forceString(row['KPP']).strip()

            orgId = self.findOrgId(INN, KPP) if INN and KPP else self.findOrgByName(row['NAME'])
            chief = nameCase(trim(row['BOSS']))


            if orgId:
                if self.skipOrgs and self.checkOrg(orgId):
                    self.log(u'Не обновляем органищацию `%s` - установлена сеть и флаг страховой.'% INN)
                else:
                    self.updateOrg(orgId, row['NAME'], row['NAME'],
                        None, None, None,
                        row['ADRES'], chief, None, row['TEL'], None, None, None, code)
            else:
                orgId = self.addOrg(row['NAME'], row['NAME'], INN, KPP,
                    None, None, None,
                    row['ADRES'], chief, None, row['TEL'], None, None, None, code)

            self.orgMap[code] = orgId

# *****************************************************************************************

    def checkOrg(self, orgId):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecord(table, 'net_id, isInsurer', orgId)

        if record:
            return (not record.isNull(0)) or forceBool(record.value(1))

        return False

# *****************************************************************************************

    def processInsurer(self, row):
        npp = row['NPP']
        orgId = self.getInsurerId(npp)
        if orgId:
            self.updateInsurer(orgId, row['NAME'], row['DIR'], row['COD_OKPO'])
        else:
            orgId = self.addInsurer(npp, row['NAME'], row['DIR'], row['COD_OKPO'])
        self.insurerCache[npp] = orgId


    def addInsurer(self, npp, name, dir, OKPO):
        table = QtGui.qApp.db.table('Organisation')
        record = table.newRecord()
# OKONH, ?
        record.setValue('fullName', toVariant(name))
        record.setValue('shortName', toVariant(name))
        record.setValue('title', toVariant(name))
        record.setValue('infisCode', toVariant(dir))
        record.setValue('OKPO', toVariant(OKPO))
        record.setValue('isInsurer', toVariant(1))
        record.setValue('notes', toVariant(self.getInsurerNotes(npp)))
        orgId = QtGui.qApp.db.insertRecord(table, record)
        return orgId


    def _getInsurerId(self, insurerCode):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecordEx(table, 'id', [table['deleted'].eq(0),
            QtGui.qApp.db.joinOr([
                                            table['notes'].eq(self.getInsurerNotes(insurerCode)),
                                            table['infisCode'].eq(insurerCode)
                                        ]),
                                                         table['isInsurer'].eq(1)
                                                        ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return 0


    def getInsurerId(self, insurerCode):
        if insurerCode in self.insurerCache:
            return self.insurerCache[insurerCode]
        else:
            result = self._getInsurerId(insurerCode)
            self.insurerCache[insurerCode] = result
            return result


    def getInsurerNotes(self, npp):
        return u'СМО # %s' % npp


    def updateInsurer(self, orgId, name, dir, OKPO):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecord(table, '*', orgId)
        record.setValue('fullName', toVariant(name))
        record.setValue('shortName', toVariant(name))
        record.setValue('title', toVariant(name))
        record.setValue('infisCode', toVariant(dir))
        record.setValue('OKPO', toVariant(OKPO))
        QtGui.qApp.db.updateRecord(table, record)


    def _getOKFSId(self, OKFS):
        return forceRef(QtGui.qApp.db.translate('rbOKFS', 'code', OKFS, 'id'))


    def getOKFSId(self, OKFS):
        if OKFS in self.OKFSCache:
            return self.OKFSCache[OKFS]
        else:
            result = self._getOKFSId(OKFS)
            self.OKFSCache[OKFS] = result
            return result

# *****************************************************************************************

    def getOrgIdByInfisCode(self, infisCode):
        result = self.orgMap.get(infisCode)

        if not result:
            table = QtGui.qApp.db.table('Organisation')
            record = QtGui.qApp.db.getRecordEx(table, 'id', [table['deleted'].eq(0),
                                            table['infisCode'].eq(infisCode)], 'id')
            if record:
                result = forceRef(record.value(0))
                self.orgMap[infisCode] = result

        return result

# *****************************************************************************************

    def updateKLADRInfisCode(self, KLADRCode, infisCode):
        table = QtGui.qApp.db.table('kladr.KLADR')
        table.setIdFieldName('CODE')
        record = QtGui.qApp.db.getRecordEx(table, 'CODE, infis',
            [table['CODE'].eq(KLADRCode),], 'CODE')
        record.setValue('infis', toVariant(infisCode))
        QtGui.qApp.db.updateRecord(table, record)

# *****************************************************************************************

    def findClientByNPP(self, npp):
        table = QtGui.qApp.db.table('ClientIdentification')
        record = QtGui.qApp.db.getRecordEx(table, 'client_id', [table['deleted'].eq(0),
                                                          table['accountingSystem_id'].eq(self.accountingSystemId),
                                                          table['identifier'].eq(npp)
                                                         ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None

# *****************************************************************************************

    def findClientByNameSexAndBirthDate(self, lastName, firstName, patrName, sex, birthDate):
        table = QtGui.qApp.db.table('Client')
        record = QtGui.qApp.db.getRecordEx(table, 'id', [table['deleted'].eq(0),
                                                         table['lastName'].eq(lastName),
                                                         table['firstName'].eq(firstName),
                                                         table['patrName'].eq(patrName),
                                                         table['sex'].eq(sex),
                                                         table['birthDate'].eq(birthDate)
                                                        ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None


# *****************************************************************************************

    def findClientBySNILS(self, SNILS):
        table = QtGui.qApp.db.table('Client')
        record = QtGui.qApp.db.getRecordEx(table, 'id', [table['deleted'].eq(0),
                                                         table['SNILS'].eq(SNILS),
                                                        ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None

# *****************************************************************************************

    def addClient(self, npp, lastName, firstName, patrName, birthDate, sex, SNILS, notes):
        table = QtGui.qApp.db.table('Client')
        record = table.newRecord()
        record.setValue('lastName',  toVariant(lastName))
        record.setValue('firstName', toVariant(firstName))
        record.setValue('patrName',  toVariant(patrName))
        record.setValue('birthDate', toVariant(birthDate))
        record.setValue('sex',       toVariant(sex))
        record.setValue('SNILS',     toVariant(SNILS))
        record.setValue('notes',     toVariant(notes))
        clientId = QtGui.qApp.db.insertRecord(table, record)
        if npp:
            self.addClientIdentification(clientId, npp)
        return clientId

# *****************************************************************************************

    def updateClient(self, clientId, lastName, firstName, patrName, birthDate, sex, SNILS, notes):
        record = QtGui.qApp.db.getRecord(self.tableClient, '*', clientId)
        isDirty = False

        if forceString(record.value('lastName')) != lastName:
            record.setValue('lastName', toVariant(lastName))
            isDirty = True

        if forceString(record.value('firstName')) != firstName:
            record.setValue('firstName', toVariant(firstName))
            isDirty = True

        if forceString(record.value('patrName')) != patrName:
            record.setValue('patrName', toVariant(patrName))
            isDirty = True

        if forceDate(record.value('birthDate')) != birthDate:
            record.setValue('birthDate', toVariant(birthDate))
            isDirty = True

        if forceInt(record.value('sex')) != sex:
            record.setValue('sex', toVariant(sex))
            isDirty = True

        if forceString(record.value('SNILS')) != SNILS:
            record.setValue('SNILS', toVariant(SNILS))
            isDirty = True

#        if forceString(record.value('notes')) != notes:
#            notes = notes + forceString(record.value('notes'))
#            record.setValue('notes', toVariant(notes))
#            isDirty = True

        if isDirty:
            self.db.updateRecord(self.tableClient, record)

# *****************************************************************************************

    def addClientDocument(self, clientId, documentTypeId, serial, number):
        if documentTypeId:
            table = QtGui.qApp.db.table('ClientDocument')
            record = table.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('documentType_id', toVariant(documentTypeId))
            record.setValue('serial', toVariant(serial))
            record.setValue('number', toVariant(number))
            QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def updateClientDocument(self, clientId, documentTypeId, serial, number):
        if documentTypeId:
            table = QtGui.qApp.db.table('ClientDocument')
            record = QtGui.qApp.db.getRecordEx(table, '*', [table['client_id'].eq(clientId),
                                                      table['serial'].eq(serial),
                                                      table['number'].eq(number)], 'id')
            if not record:
                self.addClientDocument(clientId, documentTypeId, serial, number)

# *****************************************************************************************

    def addClientPolicy(self, clientId, insurerId, serial, number, begDate, endDate, note):
        table = QtGui.qApp.db.table('ClientPolicy')
        record = table.newRecord()
        record.setValue('client_id', toVariant(clientId))
        record.setValue('insurer_id', toVariant(insurerId))
        record.setValue('policyType_id', toVariant(self.policyTypeId))
        record.setValue('serial', toVariant(serial))
        record.setValue('number', toVariant(number))
        record.setValue('begDate', toVariant(begDate))
        record.setValue('endDate', toVariant(endDate))
        record.setValue('note', toVariant(note))
        QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def updateClientPolicy(self, clientId, insurerId, serial, number, begDate, endDate, note):
        u"""Возвращаем True, если данные импортируемого полиса актуальны, иначе False"""

        record = selectLatestRecord('ClientPolicy', clientId,
            '(`Tmp`.`policyType_id`=''%d'')' % self.policyTypeId)

        if record:
            if (forceDate(record.value('begDate')) != begDate or \
                forceDate(record.value('endDate')) != endDate)  and \
                (forceDate(record.value('begDate')) <= begDate):
                    record.setValue('begDate', toVariant(begDate))
                    record.setValue('endDate', toVariant(endDate))

                    if insurerId and forceRef(record.value('insurer_id')) != insurerId:
                        record.setValue('insurer_id', toVariant(insurerId))

                    if forceString(record.value('serial')) != serial:
                        record.setValue('serial', toVariant(serial))

                    if forceString(record.value('number')) != number:
                        record.setValue('number', toVariant(number))

                    self.db.updateRecord('ClientPolicy', record)
            else:
                return False

            if forceString(record.value('note')) != note:
                record.setValue('note', toVariant(note))
                self.db.updateRecord('ClientPolicy', record)
        else:
            self.addClientPolicy(clientId, insurerId, serial, number, begDate, endDate, note)

        return True


    def addClientWork(self, clientId, organisationId):
        table = QtGui.qApp.db.table('ClientWork')
        record = table.newRecord()
        record.setValue('client_id', toVariant(clientId))
        record.setValue('org_id', toVariant(organisationId))
        QtGui.qApp.db.insertRecord(table, record)
#        self.addChange('ClientWork', 1)


    def updateClientWork(self, clientId, organisationId):
        if organisationId:
            table = QtGui.qApp.db.table('ClientWork')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                      table['client_id'].eq(clientId),
                                                      table['org_id'].eq(organisationId)], 'id')
            if not record:
                self.addClientWork(clientId, organisationId)


    def addClientPhone(self, clientId, phone):
        if phone and phone != '0':
            table = QtGui.qApp.db.table('ClientContact')
            record = table.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('contactType_id', toVariant(self.contactTypeId))
            record.setValue('contact', toVariant(phone))
            QtGui.qApp.db.insertRecord(table, record)
#            self.addChange('ClientContact', 1)


    def updateClientPhone(self, clientId, phone):
        if phone and phone != '0':
            table = QtGui.qApp.db.table('ClientContact')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                      table['client_id'].eq(clientId),
                                                      table['contactType_id'].eq(self.contactTypeId),
                                                      table['contact'].eq(phone)], 'id')
            if not record:
                self.addClientPhone(clientId, phone)


# *****************************************************************************************

    regionInfisCodeToKLADRCodeCache = {}

    def getKLADRRegionCodeByInfisCode(self, region):
        code = self.regionInfisCodeToKLADRCodeCache.get(region)

        if not code:
            code = forceString(QtGui.qApp.db.translate('kladr.KLADR', 'infis', region, 'CODE'))
            self.regionInfisCodeToKLADRCodeCache[region] = code

        return code

# *****************************************************************************************

    streetInfisCodeToKLADRCodeCache = {}

    def getKLADRStreetCodeByInfisCode(self, KLADRCode, streetCode):
        key = (streetCode, KLADRCode)
        code = self.streetInfisCodeToKLADRCodeCache.get(key)

        if not code:
            table = QtGui.qApp.db.table('kladr.STREET')
            filter = [table['infis'].eq(streetCode)]

            if KLADRCode:
                filter.append(table['CODE'].like(KLADRCode[:2]+'%'))

            record = QtGui.qApp.db.getRecordEx(table, 'CODE', filter, 'CODE')

            if record:
                code = forceString(record.value(0))
                self.streetInfisCodeToKLADRCodeCache[key] = code

        return code

# *****************************************************************************************

    addressHouseIdCache = {}

    def getAddressHouseId(self, placeCode, street, number, corpus):
        if not placeCode or not street:
            return None

        key = (placeCode, street, number, corpus)
        result = self.addressHouseIdCache.get(key)

        if not result:
            table = QtGui.qApp.db.table('AddressHouse')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                            table['KLADRCode'].eq(placeCode),
                                                            table['KLADRStreetCode'].eq(street),
                                                            table['number'].eq(number),
                                                            table['corpus'].eq(corpus)], 'id')
            if record:
                result = forceRef(record.value(0))
            else:
                record = table.newRecord()
                record.setValue('KLADRCode', toVariant(placeCode))
                record.setValue('KLADRStreetCode', toVariant(street))
                record.setValue('number', toVariant(number))
                record.setValue('corpus', toVariant(corpus))
                result = QtGui.qApp.db.insertRecord(table, record)

            self.addressHouseIdCache[key] = result

        return result


# *****************************************************************************************

    def getAddressId(self, placeCode, street, number, corpus, flat, address):
        houseId = self.getAddressHouseId(placeCode, street, number, corpus)
        if houseId:
            table = QtGui.qApp.db.table('Address')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                            table['house_id'].eq(houseId),
                                                            table['flat'].eq(flat)], 'id')
            if record:
                return forceRef(record.value(0))
            else:
                record = table.newRecord()
                record.setValue('house_id', toVariant(houseId))
                record.setValue('flat', toVariant(flat))
                return QtGui.qApp.db.insertRecord(table, record)
        else:
            return None

# *****************************************************************************************

    def ensureClientAddress(self, clientId, addressType, addressId, freeInput):
        table = QtGui.qApp.db.table('ClientAddress')
        record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                  table['client_id'].eq(clientId),
                                                  table['type'].eq(addressType),
                                                  table['address_id'].eq(addressId),
                                                  table['freeInput'].eq(freeInput),
                                                 ], 'id')
        if not record:
            record = table.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('type',      toVariant(addressType))
            record.setValue('address_id',toVariant(addressId))
            record.setValue('freeInput', toVariant(freeInput))
            QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def addClientAddress(self, clientId, placeCode, street, number, corpus, flat, freeInput):
        addressId = self.getAddressId(placeCode, street, number, corpus, flat, freeInput)
        self.ensureClientAddress(clientId, 0, addressId, freeInput)
        self.ensureClientAddress(clientId, 1, addressId, freeInput)

# *****************************************************************************************

    def updateClientAddress(self, clientId, placeCode, street, number, corpus, flat, freeInput):
        addressId = self.getAddressId(placeCode, street, number, corpus, flat, freeInput)
        self.ensureClientAddress(clientId, 0, addressId, freeInput)
        self.ensureClientAddress(clientId, 1, addressId, freeInput)

# *****************************************************************************************

    def ensureClientAttach(self, clientId, attachTypeId, orgId):
        if not orgId:
            return

        table = QtGui.qApp.db.table('ClientAttach')
        record = QtGui.qApp.db.getRecordEx(table, '*',[table['deleted'].eq(0),
                                                  table['client_id'].eq(clientId),
                                                  table['attachType_id'].eq(attachTypeId),
                                                  table['LPU_id'].eq(orgId),
                                                 ], 'id')
        if not record:
            record = table.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('attachType_id', toVariant(attachTypeId))
            record.setValue('LPU_id',toVariant(orgId))
            record.setValue('begDate', toVariant(QDate.currentDate()))
            QtGui.qApp.db.insertRecord(table, record)
        elif record and (forceRef(record.value('LPU_id')) != orgId):
            record.setValue('LPU_id',toVariant(orgId))
            record.setValue('begDate', toVariant(QDate.currentDate()))
            QtGui.qApp.db.updateRecord(table, record)

# *****************************************************************************************

    def addClientAttach(self, clientId, orgId):
        self.ensureClientAttach(clientId, self.attachTypeId, orgId)

# *****************************************************************************************

    def updateClientAttach(self, clientId, orgId):
        self.ensureClientAttach(clientId, self.attachTypeId, orgId)

# *****************************************************************************************

    def addClientIdentification(self, clientId, npp):
        table = QtGui.qApp.db.table('ClientIdentification')
        record = table.newRecord()
        record.setValue('client_id', toVariant(clientId))
        record.setValue('accountingSystem_id', toVariant(self.accountingSystemId))
        record.setValue('identifier', toVariant(npp))
        record.setValue('checkDate', toVariant(QDate.currentDate()))
        clientId = QtGui.qApp.db.insertRecord(table, record)
#        self.addChange('ClientIdentification', 1)

# *****************************************************************************************

    def updateClientIdentification(self, clientId, npp):
        table = QtGui.qApp.db.table('ClientIdentification')
        record = QtGui.qApp.db.getRecordEx(table, '*', [table['deleted'].eq(0),
                                                        table['accountingSystem_id'].eq(self.accountingSystemId),
                                                        table['client_id'].eq(clientId),
                                                       ], 'id')
        if record:
            record.setValue('identifier', toVariant(npp))
            record.setValue('checkDate', toVariant(QDate.currentDate()))
            QtGui.qApp.db.updateRecord(table, record)
#            self.addChange('ClientIdentification', 2)
        else:
            self.addClientIdentification(clientId, npp)

# *****************************************************************************************

    def getOrganisationId(self, INN, KPP):
        if INN and KPP:
            table = QtGui.qApp.db.table('Organisation')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                      table['INN'].eq(INN),
                                                      table['KPP'].eq(KPP)], 'id')
            if record:
                return forceRef(record.value(0))
            elif self.addDummyOrganisations:
                record = table.newRecord()
                name = u'ИНН: %s; КПП: %s' % (INN, KPP)
                record.setValue('fullName', toVariant(name))
                record.setValue('shortName', toVariant(name))
                record.setValue('INN', toVariant(INN))
                record.setValue('KPP', toVariant(KPP))
                return QtGui.qApp.db.insertRecord(table, record)
        return None

# *****************************************************************************************

    mapPolicySerialToInsurerId = {}

    def adviseInsurerId(self, serial):
        u""" подобрать с.к. по серии полиса """
        result = self.mapPolicySerialToInsurerId.get(serial)

        if not result:
            db = QtGui.qApp.db
            table = db.table('Organisation_PolicySerial')
            record = db.getRecordEx(table, 'organisation_id',
                                    table['serial'].eq(serial))
            if record:
                result = forceRef(record.value(0))
                self.mapPolicySerialToInsurerId[serial] = result

        return result

# *****************************************************************************************

    def updateClientSocStatus(self, clientId, classId, statusTypeId):
        if classId and statusTypeId:
            table = QtGui.qApp.db.table('ClientSocStatus')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                    table['client_id'].eq(clientId),
                                                    table['socStatusClass_id'].eq(classId),
                                                    table['socStatusType_id'].eq(statusTypeId)], 'id')
            if not record:
                self.addClientSocStatus(clientId, classId, statusTypeId)


    def addClientSocStatus(self, clientId, classId, statusTypeId):
        if classId and statusTypeId:
            table = QtGui.qApp.db.table('ClientSocStatus')
            record = table.newRecord()
            record.setValue('socStatusType_id', toVariant(statusTypeId))
            record.setValue('socStatusClass_id', toVariant(classId))
            record.setValue('client_id', toVariant(clientId))
            QtGui.qApp.db.insertRecord(table, record)



# *****************************************************************************************
    socStatusTypeIdCache = {}

    def getSocStatusTypeId(self, socClassId, regionalCode):
        if regionalCode:
            key = (socClassId, regionalCode)
            result = self.socStatusTypeIdCache.get(key)

            if not result:
                stmt = """
                    SELECT rbSocStatusType.id
                    FROM rbSocStatusType
                    LEFT JOIN rbSocStatusClassTypeAssoc ON type_id = rbSocStatusType.id
                    WHERE class_id = %d AND regionalCode = %s
                """ % (socClassId, regionalCode)

                query = QtGui.qApp.db.query(stmt)
                while query.next():
                    record = query.record()

                    if record:
                        result = forceRef(record.value(0))
                        self.socStatusTypeIdCache[key] = result

            return result

# *****************************************************************************************

    def addClientTempInvalid(self, clientId, tempInvalidDocId):
        if clientId and tempInvalidDocId:
            table = QtGui.qApp.db.table('TempInvalid')
            record = table.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('doctype_id', toVariant(tempInvalidDocId))
            record.setValue('type', toVariant(1))
            QtGui.qApp.db.insertRecord(table, record)


    def updateClientTempInvalid(self, clientId, tempInvalidDocId):
        if clientId and tempInvalidDocId:
            table = QtGui.qApp.db.table('TempInvalid')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                    table['client_id'].eq(clientId),
                                                    table['doctype_id'].eq(tempInvalidDocId),
                                                    table['type'].eq(1)], 'id')
            if not record:
                self.addClientTempInvalid(clientId, tempInvalidDocId)

# *****************************************************************************************

    tempInvalidCodeCache = {}

    def getTempInvalidDocId(self, tempInvalidCode):
        result = self.tempInvalidCodeCache.get(tempInvalidCode)

        if not result:
            table = QtGui.qApp.db.table('rbTempInvalidDocument')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[
                                                    table['type'].eq(1),
                                                    table['code'].eq(tempInvalidCode)], 'id')
            if record:
                result = forceRef(record.value(0))
                self.tempInvalidCodeCache[tempInvalidCode] = result

        return result

# *****************************************************************************************

    def checkBirthDate(self, birthDate, lastName, firstName, patrName):
        if not birthDate:
            self.log(u'<b><font color=red>ОШИБКА</b>:'
                u'Дата рождения клиента "%s %s %s" отсутствует. Пропускаем.' %\
                (lastName, firstName, patrName), True)
            return False

        if birthDate>QDate.currentDate():
            self.log(u'<b><font color=red>ОШИБКА</b>:'
                u'Дата рождения клиента "%s %s %s" находится в будущем (`%s`). Пропускаем.' %\
                (lastName, firstName, patrName, birthDate.toString('dd.MM.yyyy')), True)
            return False

        if QDate.currentDate().year() - birthDate.year() > 130:
            self.log(u'<b><font color=red>ОШИБКА</b>:'
                u'Возраст клиента "%s %s %s" больше 130 лет!(д.р. `%s`). Пропускаем.' %\
                (lastName, firstName, patrName, birthDate.toString('dd.MM.yyyy')), True)
            return False

        return True

# *****************************************************************************************

    def processClient(self, record):
        try:
            SNILS = forceString(forceString(record['SS'])).strip().replace('-', '').replace(' ', '')
        except KeyError:
            SNILS = ''

        lastName  = nameCase(forceString(record['FAM']))
        firstName = nameCase(forceString(record['IM']))
        patrName  = nameCase(forceString(record['OT']))
        sex       = forceInt(record['W']) if record['W'] else 0
        birthDate = QDate(record['DR']) if record['DR'] else QDate()

        if not self.checkBirthDate(birthDate, lastName, firstName, patrName):
            return

        policySerial = forceString(record['S_POL']).strip()
        policyNumber = forceString(record['N_POL']).strip()
        policyInsurer = self.getInsurerId(forceString(record['Q']))
        policyBegDate = QDate(record['DP']) if record['DP'] else QDate()
        policyEndDate = QDate(record['DENDP']) if record['DENDP'] else QDate()
        policyNote = forceString(record['NDOG']).strip()

        if self.skipOutdatedData and policyEndDate and policyEndDate < QDate.currentDate():
            self.log(u'<b><font color=orange>ВНИМАНИЕ</b>:'
                        u'Данные для клиента "%s %s %s" устарели. '
                        u'Пропускаем.' %\
                        (lastName, firstName, patrName), True)
            return

        docSN = forceString(record['SN_PASP']).strip()
        lastSpacePos = docSN.rfind(' ')

        if lastSpacePos != -1:
            docSerial = docSN[:lastSpacePos]
            docNumber = docSN[lastSpacePos+1:]
        else:
            docSerial = docSN
            docNumber = ''

        if not policyInsurer:
            policyInsurer = self.adviseInsurerId(policySerial)

        if not policyInsurer:
            self.log(u'<b><font color=orange>ВНИМАНИЕ</b>:'
                    u'Определить СМО для клиента "%s %s %s" по таблице серий полисов невозможно. '
                    u'Серия полиса `%s`.' %\
                    (lastName, firstName, patrName, policySerial), True)

        policyRegionCode = forceString(record['REGP']).strip()
        regionCode = forceString(record['REGS']).strip()
        regionName = self.areaMap.get(regionCode)

        if not regionName:
            regionName = self.regionMap.get(regionCode)

            if not regionName:
                self.log(u'<b><font color=red>ОШИБКА</b>:'
                        u': Код региона `%s` не найден в справочниках' % regionCode, True)

        KLADRCode = '' if regionName else QtGui.qApp.defaultKLADR()
        KLADRStreetCode = KLADRCode if KLADRCode else None

        if record.asDict().has_key('CODEKLADR'):
            KLADRCode = record['CODEKLADR']

        if record.asDict().has_key('CODESTREET'):
            KLADRStreetCode = record['CODESTREET']

        streetName = forceString(record['UL']).strip()
        number     = forceString(record['DOM']).strip()
        corpus    = forceString(record['KOR']).strip()

        if not corpus:
            corpus = forceString(record['STR']).strip()

        flat = forceString(record['KV']).strip()
        phone = forceString(record['TEL']).strip()

        freeInput = []
        if regionName:
            freeInput.append(regionName)
        if streetName:
            freeInput.append(streetName)
        if number:
            freeInput.append(u'д. %s' % number)
        if corpus:
            freeInput.append(u'корп. %s' % corpus)
        if flat:
            freeInput.append(u'кв. %s' % flat)

        freeInputAddress = u', '.join(x for x in freeInput)

        clientWorkId = self.getOrgIdByInfisCode(policyNote) if policyRegionCode != regionCode else None

        attachCode = forceString(record['MCOD']).strip()
        attachId = self.getOrgIdByInfisCode(attachCode)

        if not attachId and attachCode:
            orgName = u'СМО с кодом ' + attachCode
            attachId = self.addOrg(orgName, orgName, None, None, None, None,
                                None, None, None, None, None, None, None, None, attachCode)

        if not attachId and attachCode:
            self.log(u'<b><font color=orange>ВНИМАНИЕ</b>:'
                    u'Прикрепление для клиента "%s %s %s" невозможно. '
                    u'Организация с кодом "%s" не импортирована.' %\
                    (lastName, firstName, patrName, attachCode), True)

        socialStatus = self.getSocStatusTypeId(self.socClass9Id, forceString(record['SP']))
        benefitCode = self.getSocStatusTypeId(self.socClassBenefitId, forceString(record['KT'])) if forceString(record['KT']) != '0' else None
        tempInvalidCode = self.getTempInvalidDocId(forceString(record['GR']))

        if (forceString(record['SP']) == '2') or (policyNote in self.unemployedSet):
            clientWorkId = None


        notes = u'REGS:%s;' % regionCode
        clientId = self.findClientByNameSexAndBirthDate(lastName, firstName, patrName, sex, birthDate)

        if not clientId and SNILS:
            clientId = self.findClientBySNILS(SNILS)
        if clientId:
            if self.updateClientPolicy(clientId, policyInsurer, policySerial, policyNumber, policyBegDate, policyEndDate, policyNote):
                # полис актуален, можно обновлять информацию
                self.updateClient(clientId, lastName, firstName, patrName, birthDate, sex, SNILS, notes)
                self.updateClientWork(clientId, clientWorkId)
                self.updateClientDocument(clientId, self.passportRFId, docSerial, docNumber)
                self.updateClientAddress(clientId, KLADRCode, KLADRStreetCode, number, corpus, flat, freeInputAddress)
                self.updateClientAttach(clientId, attachId)
                self.updateClientPhone(clientId, phone)
                self.updateClientSocStatus(clientId, self.socClass9Id, socialStatus)
                self.updateClientSocStatus(clientId, self.socClassBenefitId, benefitCode)
                self.updateClientTempInvalid(clientId, tempInvalidCode)
        else:
            clientId = self.addClient(None, lastName, firstName, patrName, birthDate, sex, SNILS, notes)
            self.addClientDocument(clientId, self.passportRFId, docSerial, docNumber)
            self.addClientWork(clientId, clientWorkId)
            self.addClientPolicy(clientId, policyInsurer, policySerial, policyNumber, policyBegDate, policyEndDate, policyNote)
            self.addClientAddress(clientId, KLADRCode, KLADRStreetCode, number, corpus, flat, freeInputAddress)
            self.addClientAttach(clientId, attachId)
            self.addClientPhone(clientId, phone)
            self.addClientSocStatus(clientId, self.socClass9Id, socialStatus)
            self.addClientSocStatus(clientId, self.socClassBenefitId, benefitCode)
            self.addClientTempInvalid(clientId, tempInvalidCode)

# *****************************************************************************************


    def findOrgId(self, INN, KPP):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecordEx(table, 'id', [table['deleted'].eq(0),
                                                         table['INN'].eq(INN),
                                                         table['KPP'].eq(KPP)
                                                        ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None

# *****************************************************************************************

    def findOrgByName(self, fullName):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecordEx(table, 'id', [table['deleted'].eq(0),
                                                         table['fullName'].eq(fullName)
                                                        ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None

# *****************************************************************************************

    def addOrg(self, shortName, fullName, INN, KPP, OKPO, OKVED, OGRN,
                        address, chief, accountant, phone, fax, email, masterId,
                        infisCode):
        table = QtGui.qApp.db.table('Organisation')
        record = table.newRecord()
        record.setValue('fullName', toVariant(fullName))
        record.setValue('shortName', toVariant(shortName))
        record.setValue('title', toVariant(shortName))
        record.setValue('OKPO', toVariant(OKPO))
        record.setValue('OKVED', toVariant(OKVED))
        record.setValue('INN', toVariant(INN))
        record.setValue('KPP', toVariant(KPP))
        record.setValue('OGRN', toVariant(OGRN))
        record.setValue('address', toVariant(address))
        record.setValue('chief', toVariant(chief))
        record.setValue('accountant', toVariant(accountant))
        record.setValue('phone', toVariant(phone))
#        record.setValue('net_id', toVariant(self.matureNetId))

        notes = u''

        if fax:
            notes += u'факс: %s' % fax

        if email:
            notes += u' email: %s' % email

        if notes != '':
            record.setValue('notes', toVariant(notes))

        if masterId:
            record.setValue('head_id', toVariant(masterId))

        record.setValue('infisCode', toVariant(infisCode))
        return QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def updateOrg(self, orgId, shortName, fullName, OKPO, OKVED, OGRN,
                            address, chief, accountant, phone, fax, email, masterId,
                            infisCode):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecord(table, '*', orgId)
        record.setValue('fullName', toVariant(fullName))
        record.setValue('shortName', toVariant(shortName))
        record.setValue('title', toVariant(shortName))
        record.setValue('OKPO', toVariant(OKPO))
        record.setValue('OKVED', toVariant(OKVED))
        record.setValue('OGRN', toVariant(OGRN))
        record.setValue('address', toVariant(address))
        record.setValue('chief', toVariant(chief))
        record.setValue('accountant', toVariant(accountant))
        record.setValue('phone', toVariant(phone))
#        record.setValue('net_id', toVariant(self.matureNetId))
        notes = u''

        if fax:
            notes += u'факс: %s' % fax

        if email:
            notes += u' email: %s' % email

        if notes != '':
            record.setValue('notes', toVariant(notes))

        if masterId:
            record.setValue('head_id', toVariant(masterId))

        record.setValue('infisCode', toVariant(infisCode))
        return QtGui.qApp.db.updateRecord(table, record)
