# -*- coding: utf-8 -*-

#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################


import os.path

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import *

from library.Utils import *
from library.dbfpy.dbf  import Dbf
from Exchange.Utils import dbfCheckNames
from Registry.Utils import selectLatestRecord
from importSMO import CMyXmlStreamReader

import re


from Ui_Progress import Ui_ProgressDialog


class CProgressDialog(QtGui.QDialog, Ui_ProgressDialog):
    __pyqtSignals__ = ('startWork()', )

    def __init__(self, parent):
        QtGui.QDialog.__init__(self, parent)
        self._parent = parent
        self.setupUi(self)
        self.setDocsCount(0)
        self.setPolisesCount(0)
        self.setClientsCount(0)

        self.importSMO = None
        self.importPeople = None

        self.start()
        self.connect(self, SIGNAL('startWork()'), self.startWork, Qt.QueuedConnection)

        self.policyTypeId       = None
        self.passportRFId       = None
        self.passportUSSRId     = None
        self.birthCertificateId = None
        self.temporaryCertificateId = None
        self.contactTypeId      = None
        self.clientIdentificationId = None
        self.accountingSystemId = None
        self.attachTypeId = None
        self.matureNetId =  None
        self.socClassBenefitId = None
        self.socClass9Id = None
        self.actionTypeParentGroupId = None

        self.sexMap = {u'Ж':2, u'ж':2, u'женский':2, u'М':1, u'м':1, u'мужской':1}
        self.policyStatusMap = {'1':u'выдан первоначально',
                                '3':u'выдан взамен утерянного',
                                '4':u'заменен по изменению реквизита',
                                '5':u'выдан взамен погашенного',
                                u'А': u'уточнены реквизиты'}

        self.mapAreaToKLADR = {}
        self.mapAreaAndPlaceToKLADR = {}
        self.areaMap = {}
        self.regionMap = {}
        self.orgMap = {}
        self.unemployedSet = set()

        self.insurerCache = {}
        self.OKFSCache = {}
        self.countryCache = {}
        self.showLog = True
        self.skipOrgs = False

        self.db = QtGui.qApp.db
        self.tableClient = self.db.table('Client')
        self.mapCodePatToClientId = {}
        self.smoCodeRe = re.compile('smocode (\d+)')


    def log(self, str, forceLog = False):
        if self.showLog or forceLog:
            self.logBrowser.append(str)
            self.logBrowser.update()


    def _setCount(self, progressBar, cnt):
        if cnt:
            progressBar.setValue(0)
            progressBar.setMaximum(cnt)
            progressBar.setEnabled(True)
            progressBar.setTextVisible(True)
        else:
            progressBar.setValue(0)
            progressBar.setMaximum(1)
            progressBar.setEnabled(False)
            progressBar.setTextVisible(False)


    def setDocsCount(self, cnt):
        self._setCount(self.prbDocs, cnt)


    def setClientsCount(self, cnt):
        self._setCount(self.prbClients, cnt)


    def setPolisesCount(self, cnt):
        self._setCount(self.prbPolises, cnt)


    def start(self):
        self.processDone = False
        self.aborted = False
        self.btnCancel.setText(u'Отмена')


    def stop(self):
        self.processDone = True
        self.btnCancel.setText(u'Закрыть')


    def done(self, r):
        self.stop()
        QtGui.QDialog.done(self, r)

    def exec_(self):
        if self.importPeople:
            self.lblClients.setEnabled(True)
            self.lblPolises.setEnabled(True)
            self.lblDocs.setEnabled(True)
            self.prbClients.setEnabled(True)
            self.prbPolises.setEnabled(True)
            self.prbDocs.setEnabled(True)
        if self.importSMO:
            self.lblSMO.setEnabled(True)
            self.prbSMO.setEnabled(True)
        if self.importPeople or self.importSMO:
            self.emit(SIGNAL('startWork()'))

        return QtGui.QDialog.exec_(self)


    @pyqtSignature('')
    def on_btnCancel_clicked(self):
        if self.processDone:
            self.accept()
        else:
            self.aborted = True


    def openDbf(self, fileName, fieldsList):
        dbf = None

        result = os.path.isfile(fileName)
        if result:
            dbf = Dbf(fileName, readOnly=True, encoding='cp866')
            if not dbfCheckNames(dbf, fieldsList):
                self.log(u'файл %s\nне содержит одного из полей:\n%s' % (fileName, ', '.join(fieldsList)))
                return None
        return dbf

    def openRegistryDbf(self, dir):
        fieldList = ['EDCOD', 'FAM', 'NAME', 'OTCH', 'DATEROZ', 'POL',  'SNILS',
                            'GOS1', 'REG1', 'RAI1', 'GOR1', 'PUN1', 'STR1', 'DOM1', 'IDOM1', 'KVA1',
                            'GOS2', 'REG2', 'RAI2', 'GOR2', 'PUN2', 'STR2', 'DOM2', 'IDOM2', 'KVA2']
        return self.openDbf(os.path.join(dir, u'2FIOS.DBF'), fieldList)

    def openPolisDbf(self, dir):
        fieldList = ['EDCOD', 'NAMESMO', 'TYPEPOL', 'SERIAPOL', 'NUMPOL', 'DATEBEG', 'DATEEND', 'NOTE']
        return self.openDbf(os.path.join(dir, u'2FIO_PO.DBF'), fieldList)

    def openDocsDbf(self, dir):
        fieldList = ['EDCOD', 'ID_UD', 'SERIA', 'NUMBER', 'KEM_VIDAN', 'DATE_START']
        return self.openDbf(os.path.join(dir, u'UD_PASP.DBF'), fieldList)


    def startWork(self):
        self.start()
        result = True
        if self.importSMO:
            result = QtGui.qApp.call(self, self.workSMO)
        if self.importPeople and result:
            QtGui.qApp.call(self, self.work)
        self.stop()


    def work(self):
        rbDir = forceString(self._parent.edtRBDirName.text())

        dbfRegistry = self.openRegistryDbf(rbDir)
        dbfPolis = self.openPolisDbf(rbDir)
        dbfDocuments = self.openDocsDbf(rbDir)

        if not dbfRegistry or not dbfPolis or not dbfDocuments:
            return

        try:
            QtGui.qApp.db.transaction()
            self.process(dbfRegistry, self.loadTmpClient, self.prbClients)
            self.process(dbfPolis, self.loadTmpClientPolis, self.prbPolises)
            self.process(dbfDocuments, self.loadTmpClientDocs, self.prbDocs)
            QtGui.qApp.db.commit()
        except:
            QtGui.qApp.db.rollback()
            self.log(u'Произошла непредвиденная ошибка :(')
        finally:
            for d in (dbfRegistry, dbfPolis, dbfDocuments):
                d.close()

    def workSMO(self):
        fileName = forceString(self._parent.edtSMO.text())
        if not fileName:
            return False
        self.prbSMO.reset()
        self.prbSMO.setValue(0)
        self.prbSMO.setFormat(u'%v байт')
        inFile = QtCore.QFile(fileName)
        if not inFile.open(QtCore.QFile.ReadOnly | QtCore.QFile.Text):
            QtGui.QMessageBox.warning(self, u'Импорт страховых компаний',
                                      QString(u'Не могу открыть файл для чтения %1:\n%2.')
                                      .arg(fileName)
                                      .arg(inFile.errorString()))
            return False

        self.prbSMO.setMaximum(max(inFile.size(), 1))
        myXmlStreamReader = CMyXmlStreamReader(self, self.prbSMO)

        if (myXmlStreamReader.readFile(inFile)):
            self.log(u'Готово')
            return True
        else:
            self.log(u'Прервано')
            if self.aborted:
                self.log(u'! Прервано пользователем.')
            else:
                self.log(u'! Ошибка: файл %s, %s' % (fileName, myXmlStreamReader.errorString()))
            return False


    def loadTmpClient(self, record):
        codePat = forceLong(record['EDCOD'])
        lastName  = nameCase(forceString(record['FAM']).strip())
        firstName = nameCase(forceString(record['NAME']).strip())
        patrName  = nameCase(forceString(record['OTCH']).strip())
        sex       = self.sexMap[forceString(record['POL']).strip()]
        birthDate = QDate.fromString(record['DATEROZ'], 'dd.MM.yyyy')
        SNILS = forceString(record['SNILS']).strip()

        address1 = [
            forceString(record['GOS1']),
            forceString(record['REG1']),
            forceString(record['RAI1']),
            forceString(record['GOR1']),
            forceString(record['PUN1']),
            forceString(record['STR1']),
            forceString(record['DOM1']),
            forceString(record['IDOM1']),
            forceString(record['KVA1'])]

        address2 = [
            forceString(record['GOS2']),
            forceString(record['REG2']),
            forceString(record['RAI2']),
            forceString(record['GOR2']),
            forceString(record['PUN2']),
            forceString(record['STR2']),
            forceString(record['DOM2']),
            forceString(record['IDOM2']),
            forceString(record['KVA2'])]

        regAddress = self.parseAddress(address1)
        locAddress = self.parseAddress(address2)

        clientId = self.findClientByNameSexAndBirthDate(lastName, firstName, patrName, sex, birthDate)
        if not clientId:
            clientId = self.addClient(None, lastName, firstName, patrName, birthDate, sex, SNILS, u'Импортирован')

        self.mapCodePatToClientId[codePat] = clientId

        self.ensureClientAddress(clientId, 0, regAddress, ', '.join(filter(None, address1)))
        self.ensureClientAddress(clientId, 1, locAddress, ', '.join(filter(None, address2)))
        if not regAddress and address1[0] == u'Россия':
            self.log(u'Клиент %s %s %s, код %i'%(lastName, firstName, patrName, clientId))
            self.log(u'Адрес регистрации не определен: %s'%(', '.join(filter(None, address1))))
            self.log('\n')
#        if not locAddress and address2[0] == u'Россия':
#            self.log(u'Клиент %s %s %s, код %i'%(lastName, firstName, patrName, clientId))
#            self.log(u'Адрес проживания не определен: %s'%(', '.join(filter(None, address2))))
#            self.log('\n')

        if self.accountingSystemId:
            self.updateClientIdentification(clientId, codePat)

        if address1[0] != u'Россия' and self.foreignerClassId:
            countryId = self.getCountryId(address1[0])
            if countryId:
                self.updateClientSocStatus(clientId, self.foreignerClassId, countryId)


    def getCountryId(self, countryName):
        id = self.countryCache.get(countryName, None)
        if not id:
            id = forceRef(self.db.translate('rbSocStatusType', 'name', countryName, 'id'))
            self.countryCache[countryName] = id
        return id

    def loadTmpClientPolis(self, record):
        codePat = forceLong(record['EDCOD'])
        name = forceString(record['NAMESMO']).strip()
        type = forceString(record['TYPEPOL']).strip()
        serial = forceString(record['SERIAPOL']).strip()
        number = forceString(record['NUMPOL']).strip()
        begDate = QDate.fromString(record['DATEBEG'], 'dd.MM.yyyy')
        endDate = QDate.fromString(record['DATEEND'], 'dd.MM.yyyy')
        note = forceString(record['NOTE'])
        smocode = ''

        m = self.smoCodeRe.search(note)
        if m:
            smocode = m.group(1)

        policyTypeId = forceInt(self.db.translate('rbPolicyKind', 'code', type, 'id'))
        clientId = self.mapCodePatToClientId.get(codePat)

        if clientId:
            if not policyTypeId:
                self.log(u'Клиент код %i'%(clientId))
                self.log(u'%i - тип полиса не известен, пропускаем\n'%policyTypeId)
                return

            insurerId = forceRef(self.db.translate('Organisation', 'smoCode', smocode, 'id'))
            if insurerId:
                name = ''
            self.updateClientPolicy(clientId, insurerId, policyTypeId, serial, number, begDate, endDate, '', name)
        else:
            self.log(u'Клиент с кодом %i не найден' % codePat)


    def loadTmpClientDocs(self, record):
        codePat = forceLong(record['EDCOD'])
        type = forceString(record['ID_UD'])
        serial = forceString(record['SERIA']).strip()
        number = forceString(record['NUMBER']).strip()
        begDate = forceDate(record['DATE_START'])
#        endDate = QDate.fromString(record['DATE_END'], 'dd.MM.yy')

        mapType = {'1':self.passportRFId, '3':self.birthCertificateId, '9':self.passportForeigner, '11':self.passportZG}
        documentTypeId = mapType.get(type, None)

        clientId = self.mapCodePatToClientId.get(codePat)

        if clientId:
            if not documentTypeId:
                self.log(u'Клиент код %i'%(clientId))
                self.log(u'%s - тип документа не известен, пропускаем\n'%type)
                return
            if documentTypeId == self.passportRFId:
                serial = serial.replace(' ', '')
                serial = '%s %s'%(serial[:2], serial[2:])
            elif documentTypeId == self.birthCertificateId:
                serial = serial.replace(' ', '').replace('-', '')
                serial = '%s %s'%(serial[:-2], serial[-2:])
            self.updateClientDocument(clientId, documentTypeId, serial, number, begDate)
        else:
            self.log(u'Клиент с кодом %i не найден' % codePat)



#        dbfRegion = Dbf(os.path.join(rbDir, u'Приложение 7 Справочник адмистративного деления Смоленской области.DBF'), readOnly=True, encoding='cp866')
#        dbfOrgs = Dbf(forceString(self.parent.edtOrgDataFileName.text()), readOnly=True, encoding='cp866')
#        dbfRegistry = Dbf(forceString(self.parent.edtMainDataFileName.text()), readOnly=True, encoding='cp866')
#
#        self.setPlacesCount(len(dbfCounty)+len(dbfRegion))
#        self.process(dbfCounty, self.processArea, self.prbPlaces)
#        self.process(dbfRegion, self.processRegion, self.prbPlaces)
#        self.setOrgsCount(len(dbfOrgs))
#        self.process(dbfOrgs, self.processOrg, self.prbOrgs)
#
#        self.setClientsCount(len(dbfRegistry))
#        self.process(dbfRegistry, self.processClient, self.prbClients)
#



    def process(self, dbf, step, progressBar):
        self.lblElapsed.setText(u'Текущая операция: ??? зап/с, окончание в ??:??:??')
        time = QTime()
        time.start()
        startPos = 0
        oldSpeed = 0

        self._setCount(progressBar, len(dbf))
        for record in dbf:
            QtGui.qApp.processEvents()
            if self.aborted:
                self.reject()
                return
            progressBar.setValue(progressBar.value()+1)

            try:
                step(record.asDict())
            except:
                raise

            elapsed =time.elapsed()

            if elapsed != 0 and (progressBar.value()-startPos>100):
                startPos = progressBar.value()
                oldSpeed = 100*1000/elapsed
                newSpeed = (oldSpeed+100*1000/elapsed)/2
                if newSpeed != 0:
                    finishTime = QTime.currentTime().addSecs(progressBar.maximum()/newSpeed)
                    self.lblElapsed.setText(u'Текущая операция: %03.f зап/с, окончание в %s' % (newSpeed, finishTime.toString('hh:mm:ss')))
                    time.restart()

    def getKladrCode(self, name, socr = None, code = None):
        table = self.db.table('kladr.KLADR')
        cols = [table['CODE'].name()]
        cond = [table['NAME'].like(name), table['CODE'].like('%%00')]
        if socr:
            socrCond = [table['SOCR'].like('%s%%'%socr)]
            if socr == u'рп' or socr == u'дп':
                socrCond.append(table['SOCR'].like(u'пгт'))
            if socr == u'пгт':
                socrCond.append(table['SOCR'].like(u'рп'))
                socrCond.append(table['SOCR'].like(u'дп'))
            if socr == u'п':
                socrCond.append(table['SOCR'].like(u'с'))
            if socr == u'x':
                socrCond.append(table['SOCR'].like(u'п'))
            cond.append(self.db.joinOr(socrCond))
        if code:
            cond.append(table['CODE'].like('%s%%'%code))
        return self.db.getRecordList(table, cols, cond)

    def parseCity(self, name, socr, areaName, areaSocr, regionName, regionSocr):
        kladrCode = ''
        if not areaName:
            areaName = regionName
            areaSocr = regionSocr
        recordList = self.getKladrCode(name, socr)
        if len(recordList) > 1:
            recordList = self.getKladrCode(areaName, areaSocr)
            if len(recordList) > 1:
                recordList = self.getKladrCode(regionName, regionSocr)
                if len(recordList) != 1:
                    return kladrCode
                areaCode = forceString(recordList[0].value(0))
                recordList = self.getKladrCode(areaName, areaSocr, areaCode[:2])
            if len(recordList) == 1:
                areaCode = forceString(recordList[0].value(0))
                recordList = self.getKladrCode(name, socr, areaCode[:5])
                if recordList:
                    kladrCode = forceString(recordList[0].value(0))
        elif len(recordList) == 1:
                kladrCode = forceString(recordList[0].value(0))
        return kladrCode

    def parseStreet(self, kladrCode, name, socr):
        #mapSocr = {u'пр-т':u'пр-кт', u'пр-к':u'пр-кт', u'пр':u'пр-кт'}
        kladrStreetCode = ''
        table = self.db.table('kladr.STREET')
        cols = [table['CODE'].name()]
        cond = [table['CODE'].like('%s%%'%kladrCode), table['NAME'].like(name), table['SOCR'].like(u'%s%%'%socr[:3])]
        record = self.db.getRecordEx(table, cols, cond)
        if record:
            kladrStreetCode = forceString(record.value(0))
        return kladrStreetCode

    def parseAddress(self, address):
        addressId = None
        kladrCode = ''
        kladrStreet = ''
        cityName = None
        citySocr = None
        rawCountry = address[0]
        if rawCountry != u'Россия':
            return None
        rawCity = address[3].split(' ')
        rawArea = address[4].split(' ')
        if len(rawCity) > 1:
            citySocr = rawCity.pop().replace('.', '')
            cityName = ' '.join(rawCity)
        elif len(rawArea) > 1:
            citySocr = rawArea.pop().replace('.', '')
            cityName = ' '.join(rawArea)
        if cityName and citySocr:
            rawArea = address[2].split(' ')
            rawRegion = address[1].split(' ')
            areaSocr = rawArea.pop().replace('.', '')
            areaName = ' '.join(rawArea)
            if areaName == u'Волгодонский':
                areaName = u'Волгодонской'
            if cityName[:2] == u'Б-':
                cityName = u'Большая %s'%cityName[2:]
            regionSocr = rawRegion.pop().replace('.', '')
            regionName = ' '.join(rawRegion)
            kladrCode = self.parseCity(cityName, citySocr, areaName, areaSocr, regionName, regionSocr)
            rawStreet = address[5].split(' ')
            if len(rawStreet)>1:
                streetSocr = rawStreet.pop().replace('.', '')
                streetName = ' '.join(rawStreet)
                kladrStreet = self.parseStreet(kladrCode[:-2], streetName, streetSocr)
            addressId = self.getAddressId(kladrCode, kladrStreet, address[6].strip(), address[7].strip(), address[8].strip(), '')
        return addressId



# *****************************************************************************************

    def findClientByNameSexAndBirthDate(self, lastName, firstName, patrName, sex, birthDate):
        table = QtGui.qApp.db.table('Client')
        record = QtGui.qApp.db.getRecordEx(table, 'id', [table['deleted'].eq(0),
                                                         table['lastName'].eq(lastName),
                                                         table['firstName'].eq(firstName),
                                                         table['patrName'].eq(patrName),
                                                         table['sex'].eq(sex),
                                                         table['birthDate'].eq(birthDate)
                                                        ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None


# *****************************************************************************************

    def findClientBySNILS(self, SNILS):
        table = QtGui.qApp.db.table('Client')
        record = QtGui.qApp.db.getRecordEx(table, 'id', [table['deleted'].eq(0),
                                                         table['SNILS'].eq(SNILS),
                                                        ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None

# *****************************************************************************************

    def addClient(self, npp, lastName, firstName, patrName, birthDate, sex, SNILS, notes):
        table = QtGui.qApp.db.table('Client')
        record = table.newRecord()
        record.setValue('lastName',  toVariant(lastName))
        record.setValue('firstName', toVariant(firstName))
        record.setValue('patrName',  toVariant(patrName))
        record.setValue('birthDate', toVariant(birthDate))
        record.setValue('sex',       toVariant(sex))
        record.setValue('SNILS',     toVariant(SNILS))
        record.setValue('notes',     toVariant(notes))
        clientId = QtGui.qApp.db.insertRecord(table, record)
        if npp:
            self.addClientIdentification(clientId, npp)
        return clientId

# *****************************************************************************************

    def updateClient(self, clientId, lastName, firstName, patrName, birthDate, sex, SNILS, notes):
        record = QtGui.qApp.db.getRecord(self.tableClient, '*', clientId)
        isDirty = False

        if forceString(record.value('lastName')) != lastName:
            record.setValue('lastName', toVariant(lastName))
            isDirty = True

        if forceString(record.value('firstName')) != firstName:
            record.setValue('firstName', toVariant(firstName))
            isDirty = True

        if forceString(record.value('patrName')) != patrName:
            record.setValue('patrName', toVariant(patrName))
            isDirty = True

        if forceDate(record.value('birthDate')) != birthDate:
            record.setValue('birthDate', toVariant(birthDate))
            isDirty = True

        if forceInt(record.value('sex')) != sex:
            record.setValue('sex', toVariant(sex))
            isDirty = True

        if forceString(record.value('SNILS')) != SNILS:
            record.setValue('SNILS', toVariant(SNILS))
            isDirty = True

#        if forceString(record.value('notes')) != notes:
#            notes = notes + forceString(record.value('notes'))
#            record.setValue('notes', toVariant(notes))
#            isDirty = True

        if isDirty:
            self.db.updateRecord(self.tableClient, record)

# *****************************************************************************************

    def addClientDocument(self, clientId, documentTypeId, serial, number, date):
        if documentTypeId:
            table = QtGui.qApp.db.table('ClientDocument')
            record = table.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('documentType_id', toVariant(documentTypeId))
            record.setValue('serial', toVariant(serial))
            record.setValue('number', toVariant(number))
            record.setValue('date', toVariant(date))
            QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def updateClientDocument(self, clientId, documentTypeId, serial, number, date):
        if documentTypeId:
            table = QtGui.qApp.db.table('ClientDocument')
            record = QtGui.qApp.db.getRecordEx(table, '*', [table['client_id'].eq(clientId),
                                                      table['serial'].eq(serial),
                                                      table['number'].eq(number)], 'id')
            if not record:
                self.addClientDocument(clientId, documentTypeId, serial, number, date)

# *****************************************************************************************

    def addClientPolicy(self, clientId, insurerId, policyKindId, serial, number, begDate, endDate, note, name = ''):
        table = QtGui.qApp.db.table('ClientPolicy')
        record = table.newRecord()
        record.setValue('client_id', toVariant(clientId))
        record.setValue('insurer_id', toVariant(insurerId))
        record.setValue('policyType_id', toVariant(self.policyTypeId))
        record.setValue('policyKind_id', toVariant(policyKindId))
        record.setValue('serial', toVariant(serial))
        record.setValue('number', toVariant(number))
        record.setValue('begDate', toVariant(begDate))
        record.setValue('endDate', toVariant(endDate))
        record.setValue('note', toVariant(note))
        record.setValue('name', toVariant(name))
        QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def updateClientPolicy(self, clientId, insurerId, policyKindId, serial, number, begDate, endDate, note, name):
        u"""Возвращаем True, если данные импортируемого полиса актуальны, иначе False"""

        record = selectLatestRecord('ClientPolicy', clientId,
            '(`Tmp`.`policyType_id`=''%d'')' % self.policyTypeId)

        if record:
            if (forceDate(record.value('begDate')) != begDate or \
                forceDate(record.value('endDate')) != endDate)  and \
                (forceDate(record.value('begDate')) <= begDate):
                    record.setValue('begDate', toVariant(begDate))
                    record.setValue('endDate', toVariant(endDate))

                    if insurerId and forceRef(record.value('insurer_id')) != insurerId:
                        record.setValue('insurer_id', toVariant(insurerId))

                    if forceString(record.value('serial')) != serial:
                        record.setValue('serial', toVariant(serial))

                    if forceString(record.value('number')) != number:
                        record.setValue('number', toVariant(number))

                    self.db.updateRecord('ClientPolicy', record)
            else:
                return False

            if forceString(record.value('note')) != note:
                record.setValue('note', toVariant(note))
                self.db.updateRecord('ClientPolicy', record)
        else:
            self.addClientPolicy(clientId, insurerId, policyKindId, serial, number, begDate, endDate, note, name)

        return True


    def addClientWork(self, clientId, organisationId):
        table = QtGui.qApp.db.table('ClientWork')
        record = table.newRecord()
        record.setValue('client_id', toVariant(clientId))
        record.setValue('org_id', toVariant(organisationId))
        QtGui.qApp.db.insertRecord(table, record)
#        self.addChange('ClientWork', 1)


    def updateClientWork(self, clientId, organisationId):
        if organisationId:
            table = QtGui.qApp.db.table('ClientWork')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                      table['client_id'].eq(clientId),
                                                      table['org_id'].eq(organisationId)], 'id')
            if not record:
                self.addClientWork(clientId, organisationId)


    def addClientPhone(self, clientId, phone):
        if phone and phone != '0':
            table = QtGui.qApp.db.table('ClientContact')
            record = table.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('contactType_id', toVariant(self.contactTypeId))
            record.setValue('contact', toVariant(phone))
            QtGui.qApp.db.insertRecord(table, record)
#            self.addChange('ClientContact', 1)


    def updateClientPhone(self, clientId, phone):
        if phone and phone != '0':
            table = QtGui.qApp.db.table('ClientContact')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                      table['client_id'].eq(clientId),
                                                      table['contactType_id'].eq(self.contactTypeId),
                                                      table['contact'].eq(phone)], 'id')
            if not record:
                self.addClientPhone(clientId, phone)


# *****************************************************************************************

    regionInfisCodeToKLADRCodeCache = {}

    def getKLADRRegionCodeByInfisCode(self, region):
        code = self.regionInfisCodeToKLADRCodeCache.get(region)

        if not code:
            code = forceString(QtGui.qApp.db.translate('kladr.KLADR', 'infis', region, 'CODE'))
            self.regionInfisCodeToKLADRCodeCache[region] = code

        return code

# *****************************************************************************************

    streetInfisCodeToKLADRCodeCache = {}

    def getKLADRStreetCodeByInfisCode(self, KLADRCode, streetCode):
        key = (streetCode, KLADRCode)
        code = self.streetInfisCodeToKLADRCodeCache.get(key)

        if not code:
            table = QtGui.qApp.db.table('kladr.STREET')
            filter = [table['infis'].eq(streetCode)]

            if KLADRCode:
                filter.append(table['CODE'].like(KLADRCode[:2]+'%'))

            record = QtGui.qApp.db.getRecordEx(table, 'CODE', filter, 'CODE')

            if record:
                code = forceString(record.value(0))
                self.streetInfisCodeToKLADRCodeCache[key] = code

        return code

# *****************************************************************************************

    addressHouseIdCache = {}

    def getAddressHouseId(self, placeCode, street, number, corpus):
        if not placeCode:
            return None

        key = (placeCode, street, number, corpus)
        result = self.addressHouseIdCache.get(key)

        if not result:
            table = QtGui.qApp.db.table('AddressHouse')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                            table['KLADRCode'].eq(placeCode),
                                                            table['KLADRStreetCode'].eq(street),
                                                            table['number'].eq(number),
                                                            table['corpus'].eq(corpus)], 'id')
            if record:
                result = forceRef(record.value(0))
            else:
                record = table.newRecord()
                record.setValue('KLADRCode', toVariant(placeCode))
                record.setValue('KLADRStreetCode', toVariant(street))
                record.setValue('number', toVariant(number))
                record.setValue('corpus', toVariant(corpus))
                result = QtGui.qApp.db.insertRecord(table, record)

            self.addressHouseIdCache[key] = result

        return result


# *****************************************************************************************

    def getAddressId(self, placeCode, street, number, corpus, flat, address):
        houseId = self.getAddressHouseId(placeCode, street, number, corpus)
        if houseId:
            table = QtGui.qApp.db.table('Address')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                            table['house_id'].eq(houseId),
                                                            table['flat'].eq(flat)], 'id')
            if record:
                return forceRef(record.value(0))
            else:
                record = table.newRecord()
                record.setValue('house_id', toVariant(houseId))
                record.setValue('flat', toVariant(flat))
                return QtGui.qApp.db.insertRecord(table, record)
        else:
            return None

# *****************************************************************************************

    def ensureClientAddress(self, clientId, addressType, addressId, freeInput):
        table = QtGui.qApp.db.table('ClientAddress')
        cond = [table['deleted'].eq(0),
                    table['client_id'].eq(clientId),
                    table['type'].eq(addressType),
                    table['address_id'].eq(addressId)
                    ]
        if not addressId:
            cond.append(table['freeInput'].eq(freeInput))
        record = QtGui.qApp.db.getRecordEx(table, 'id', cond, 'id')
        if not record:
            record = table.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('type',      toVariant(addressType))
            record.setValue('address_id',toVariant(addressId))
            record.setValue('freeInput', toVariant(freeInput))
            QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def addClientAddress(self, clientId, placeCode, street, number, corpus, flat, freeInput):
        addressId = self.getAddressId(placeCode, street, number, corpus, flat, freeInput)
        self.ensureClientAddress(clientId, 0, addressId, freeInput)
        self.ensureClientAddress(clientId, 1, addressId, freeInput)

# *****************************************************************************************

    def updateClientAddress(self, clientId, placeCode, street, number, corpus, flat, freeInput):
        addressId = self.getAddressId(placeCode, street, number, corpus, flat, freeInput)
        self.ensureClientAddress(clientId, 0, addressId, freeInput)
        self.ensureClientAddress(clientId, 1, addressId, freeInput)

# *****************************************************************************************

    def ensureClientAttach(self, clientId, attachTypeId, orgId):
        if not orgId:
            return

        table = QtGui.qApp.db.table('ClientAttach')
        record = QtGui.qApp.db.getRecordEx(table, '*',[table['deleted'].eq(0),
                                                  table['client_id'].eq(clientId),
                                                  table['attachType_id'].eq(attachTypeId),
                                                  table['LPU_id'].eq(orgId),
                                                 ], 'id')
        if not record:
            record = table.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('attachType_id', toVariant(attachTypeId))
            record.setValue('LPU_id',toVariant(orgId))
            record.setValue('begDate', toVariant(QDate.currentDate()))
            QtGui.qApp.db.insertRecord(table, record)
        elif record and (forceRef(record.value('LPU_id')) != orgId):
            record.setValue('LPU_id',toVariant(orgId))
            record.setValue('begDate', toVariant(QDate.currentDate()))
            QtGui.qApp.db.updateRecord(table, record)

# *****************************************************************************************

    def addClientAttach(self, clientId, orgId):
        self.ensureClientAttach(clientId, self.attachTypeId, orgId)

# *****************************************************************************************

    def updateClientAttach(self, clientId, orgId):
        self.ensureClientAttach(clientId, self.attachTypeId, orgId)

# *****************************************************************************************

    def addClientIdentification(self, clientId, npp):
        table = QtGui.qApp.db.table('ClientIdentification')
        record = table.newRecord()
        record.setValue('client_id', toVariant(clientId))
        record.setValue('accountingSystem_id', toVariant(self.accountingSystemId))
        record.setValue('identifier', toVariant(npp))
        record.setValue('checkDate', toVariant(QDate.currentDate()))
        clientId = QtGui.qApp.db.insertRecord(table, record)
#        self.addChange('ClientIdentification', 1)

# *****************************************************************************************

    def updateClientIdentification(self, clientId, npp):
        table = QtGui.qApp.db.table('ClientIdentification')
        record = QtGui.qApp.db.getRecordEx(table, '*', [table['deleted'].eq(0),
                                                        table['accountingSystem_id'].eq(self.accountingSystemId),
                                                        table['client_id'].eq(clientId),
                                                       ], 'id')
        if record:
            record.setValue('identifier', toVariant(npp))
            record.setValue('checkDate', toVariant(QDate.currentDate()))
            QtGui.qApp.db.updateRecord(table, record)
#            self.addChange('ClientIdentification', 2)
        else:
            self.addClientIdentification(clientId, npp)

# *****************************************************************************************

    def getOrganisationId(self, INN, KPP):
        if INN and KPP:
            table = QtGui.qApp.db.table('Organisation')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                      table['INN'].eq(INN),
                                                      table['KPP'].eq(KPP)], 'id')
            if record:
                return forceRef(record.value(0))
            elif self.addDummyOrganisations:
                record = table.newRecord()
                name = u'ИНН: %s; КПП: %s' % (INN, KPP)
                record.setValue('fullName', toVariant(name))
                record.setValue('shortName', toVariant(name))
                record.setValue('INN', toVariant(INN))
                record.setValue('KPP', toVariant(KPP))
                return QtGui.qApp.db.insertRecord(table, record)
        return None

# *****************************************************************************************

    mapPolicySerialToInsurerId = {}

    def adviseInsurerId(self, serial):
        u""" подобрать с.к. по серии полиса """
        result = self.mapPolicySerialToInsurerId.get(serial)

        if not result:
            db = QtGui.qApp.db
            table = db.table('Organisation_PolicySerial')
            record = db.getRecordEx(table, 'organisation_id',
                                    table['serial'].eq(serial))
            if record:
                result = forceRef(record.value(0))
                self.mapPolicySerialToInsurerId[serial] = result

        return result

# *****************************************************************************************

    def updateClientSocStatus(self, clientId, classId, statusTypeId):
        if classId and statusTypeId:
            table = QtGui.qApp.db.table('ClientSocStatus')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                    table['client_id'].eq(clientId),
                                                    table['socStatusClass_id'].eq(classId),
                                                    table['socStatusType_id'].eq(statusTypeId)], 'id')
            if not record:
                self.addClientSocStatus(clientId, classId, statusTypeId)


    def addClientSocStatus(self, clientId, classId, statusTypeId):
        if classId and statusTypeId:
            table = QtGui.qApp.db.table('ClientSocStatus')
            record = table.newRecord()
            record.setValue('socStatusType_id', toVariant(statusTypeId))
            record.setValue('socStatusClass_id', toVariant(classId))
            record.setValue('client_id', toVariant(clientId))
            QtGui.qApp.db.insertRecord(table, record)



# *****************************************************************************************
    socStatusTypeIdCache = {}

    def getSocStatusTypeId(self, socClassId, regionalCode):
        if regionalCode:
            key = (socClassId, regionalCode)
            result = self.socStatusTypeIdCache.get(key)

            if not result:
                stmt = """
                    SELECT rbSocStatusType.id
                    FROM rbSocStatusType
                    LEFT JOIN rbSocStatusClassTypeAssoc ON type_id = rbSocStatusType.id
                    WHERE class_id = %d AND regionalCode = %s
                """ % (socClassId, regionalCode)

                query = QtGui.qApp.db.query(stmt)
                while query.next():
                    record = query.record()

                    if record:
                        result = forceRef(record.value(0))
                        self.socStatusTypeIdCache[key] = result

            return result

# *****************************************************************************************

    def addClientTempInvalid(self, clientId, tempInvalidDocId):
        if clientId and tempInvalidDocId:
            table = QtGui.qApp.db.table('TempInvalid')
            record = table.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('doctype_id', toVariant(tempInvalidDocId))
            record.setValue('type', toVariant(1))
            QtGui.qApp.db.insertRecord(table, record)


    def updateClientTempInvalid(self, clientId, tempInvalidDocId):
        if clientId and tempInvalidDocId:
            table = QtGui.qApp.db.table('TempInvalid')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                    table['client_id'].eq(clientId),
                                                    table['doctype_id'].eq(tempInvalidDocId),
                                                    table['type'].eq(1)], 'id')
            if not record:
                self.addClientTempInvalid(clientId, tempInvalidDocId)

# *****************************************************************************************

    tempInvalidCodeCache = {}

    def getTempInvalidDocId(self, tempInvalidCode):
        result = self.tempInvalidCodeCache.get(tempInvalidCode)

        if not result:
            table = QtGui.qApp.db.table('rbTempInvalidDocument')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[
                                                    table['type'].eq(1),
                                                    table['code'].eq(tempInvalidCode)], 'id')
            if record:
                result = forceRef(record.value(0))
                self.tempInvalidCodeCache[tempInvalidCode] = result

        return result

# *****************************************************************************************

    def checkBirthDate(self, birthDate, lastName, firstName, patrName):
        if not birthDate:
            self.log(u'<b><font color=red>ОШИБКА</b>:'
                u'Дата рождения клиента "%s %s %s" отсутствует. Пропускаем.' %\
                (lastName, firstName, patrName), True)
            return False

        if birthDate>QDate.currentDate():
            self.log(u'<b><font color=red>ОШИБКА</b>:'
                u'Дата рождения клиента "%s %s %s" находится в будущем (`%s`). Пропускаем.' %\
                (lastName, firstName, patrName, birthDate.toString('dd.MM.yyyy')), True)
            return False

        if QDate.currentDate().year() - birthDate.year() > 130:
            self.log(u'<b><font color=red>ОШИБКА</b>:'
                u'Возраст клиента "%s %s %s" больше 130 лет!(д.р. `%s`). Пропускаем.' %\
                (lastName, firstName, patrName, birthDate.toString('dd.MM.yyyy')), True)
            return False

        return True

# *****************************************************************************************

    def findOrgId(self, INN, KPP):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecordEx(table, 'id', [table['deleted'].eq(0),
                                                         table['INN'].eq(INN),
                                                         table['KPP'].eq(KPP)
                                                        ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None

# *****************************************************************************************

    def findOrgByName(self, fullName):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecordEx(table, 'id', [table['deleted'].eq(0),
                                                         table['fullName'].eq(fullName)
                                                        ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None

# *****************************************************************************************

    def addOrg(self, shortName, fullName, INN, KPP, OKPO, OKVED, OGRN,
                        address, chief, accountant, phone, fax, email, masterId,
                        infisCode):
        table = QtGui.qApp.db.table('Organisation')
        record = table.newRecord()
        record.setValue('fullName', toVariant(fullName))
        record.setValue('shortName', toVariant(shortName))
        record.setValue('title', toVariant(shortName))
        record.setValue('OKPO', toVariant(OKPO))
        record.setValue('OKVED', toVariant(OKVED))
        record.setValue('INN', toVariant(INN))
        record.setValue('KPP', toVariant(KPP))
        record.setValue('OGRN', toVariant(OGRN))
        record.setValue('address', toVariant(address))
        record.setValue('chief', toVariant(chief))
        record.setValue('accountant', toVariant(accountant))
        record.setValue('phone', toVariant(phone))
#        record.setValue('net_id', toVariant(self.matureNetId))

        notes = u''

        if fax:
            notes += u'факс: %s' % fax

        if email:
            notes += u' email: %s' % email

        if notes != '':
            record.setValue('notes', toVariant(notes))

        if masterId:
            record.setValue('head_id', toVariant(masterId))

        record.setValue('infisCode', toVariant(infisCode))
        return QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def updateOrg(self, orgId, shortName, fullName, OKPO, OKVED, OGRN,
                            address, chief, accountant, phone, fax, email, masterId,
                            infisCode):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecord(table, '*', orgId)
        record.setValue('fullName', toVariant(fullName))
        record.setValue('shortName', toVariant(shortName))
        record.setValue('title', toVariant(shortName))
        record.setValue('OKPO', toVariant(OKPO))
        record.setValue('OKVED', toVariant(OKVED))
        record.setValue('OGRN', toVariant(OGRN))
        record.setValue('address', toVariant(address))
        record.setValue('chief', toVariant(chief))
        record.setValue('accountant', toVariant(accountant))
        record.setValue('phone', toVariant(phone))
#        record.setValue('net_id', toVariant(self.matureNetId))
        notes = u''

        if fax:
            notes += u'факс: %s' % fax

        if email:
            notes += u' email: %s' % email

        if notes != '':
            record.setValue('notes', toVariant(notes))

        if masterId:
            record.setValue('head_id', toVariant(masterId))

        record.setValue('infisCode', toVariant(infisCode))
        return QtGui.qApp.db.updateRecord(table, record)
