# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

u"""Импорт тарифа из XML"""
import sys
from PyQt4 import QtCore, QtGui
#from PyQt4.QtCore import *
from PyQt4.QtCore import QXmlStreamReader

from library.Utils import *

smoKeyFields = ['INN', 'KPP', 'smoCode']
smoCheckFields = ['fullName', 'shortName', 'OGRN']

class CMyXmlStreamReader(QXmlStreamReader):
    def __init__(self, parent, progressBar, showLog = True, updateSMO = True,  onlyCodes = None):
        QXmlStreamReader.__init__(self)
        self._parent = parent
        self.progressBar = progressBar
        self.table = QtGui.qApp.db.table('Organisation')

        self.showLog = showLog
        self.updateSMO = updateSMO
        self.onlyCodes = onlyCodes
        self.exportVersion = '1.0.1'

        self.skip = False
        self.nadded = 0
        self.ncoincident = 0
        self.nprocessed = 0
        self.nupdated = 0
        self.nskipped = 0
        
        self.mapSMOKeyToRecordList = {}
        cols = ['id']
        cols.extend(smoKeyFields)
        cols.extend(smoCheckFields)
        records = QtGui.qApp.db.getRecordList(self.table, '%s'%', '.join(cols))
        for record in records:
            self.addSMOToMap(record)

    def getSMOKey(self, record):
        return tuple([ forceStringEx(record.value(fieldName)) if not record.value(fieldName).isNull() else None for fieldName in smoKeyFields ])

    def addSMOToMap(self, record):
        key = self.getSMOKey(record)
        self.mapSMOKeyToRecordList.setdefault(key, []).append(record)

    def raiseError(self, str):
        QXmlStreamReader.raiseError(self, u'[%d:%d] %s' % (self.lineNumber(), self.columnNumber(), str))

    def readNext(self):
        QXmlStreamReader.readNext(self)
        self.progressBar.setValue(self.device().pos())

    def log(self, str, forceLog = False,  red = None):
        if self.showLog or forceLog:
            if red:
                color = self._parent.logBrowser.textColor()
                self._parent.logBrowser.setTextColor( Qt.red )
                self._parent.logBrowser.append(str)
                self._parent.logBrowser.setTextColor( color )
            else:
                self._parent.logBrowser.append(str)
            self._parent.logBrowser.update()

    def readFile(self, device):
        self.setDevice(device)
        try:
            while (not self.atEnd()):
                self.readNext()

                if self.isStartElement():
                    if self.name() == 'packet':
                        if self.attributes().value('version') == self.exportVersion:
                            self.readData()
                        else:
                            self.raiseError(u'Версия формата "%s" не поддерживается. Должна быть "%s"' \
                                % (self.attributes().value('version').toString(), self.exportVersion))
                    else:
                        self.raiseError(u'Неверный формат экспорта данных.')

                if self.hasError() or self._parent.aborted:
                    return False

        except IOError, e:
            QtGui.qApp.logCurrentException()
            msg=''
            if hasattr(e, 'filename'):
                msg = u'%s:\n[Errno %s] %s' % (e.filename, e.errno, anyToUnicode(e.strerror))
            else:
                msg = u'[Errno %s] %s' % (e.errno, anyToUnicode(e.strerror))
            QtGui.QMessageBox.critical (self._parent,
                u'Произошла ошибка ввода-вывода', msg, QtGui.QMessageBox.Close)
            self.log(u'! Ошибка ввода-вывода: %s' % msg, True)
            return False
        except Exception, e:
            QtGui.qApp.logCurrentException()
            QtGui.QMessageBox.critical (self._parent,
                u'Произошла ошибка', unicode(e), QtGui.QMessageBox.Close)
            self.log(u'! Ошибка: %s' % unicode(e), True)
            return False
        return not (self.hasError() or self._parent.aborted)


    def readData(self):
        assert self.isStartElement() and self.name() == 'packet'
        while (not self.atEnd()):
            self.readNext()
            if self.isEndElement():
                break
            if self.isStartElement():
                if (self.name() == 'insCompany'):
                    self.readSMOData()
                else:
                    self.readUnknownElement()
            if self.hasError() or self._parent.aborted:
                break


    def readSMOData(self):
        assert self.isStartElement() and self.name() == 'insCompany'
        newSMORecord = self.table.newRecord()
        pstAddress = None
        jurAddress = None
        notes = []
        phone = []
        fam_ruk = ''
        im_ruk = ''
        ot_ruk = ''
        kpp = None
        smocode = None

        while not self.atEnd():
            self.readNext()
            QtGui.qApp.processEvents()
            if self.isEndElement() and self.name() == 'insCompany':
                break
            if self.isStartElement():
                name = forceString(self.name())
                self.readNext()
                if self.isStartElement():
                    if name == 'pstAddress':
                        pstAddress = self.readAddress(name)
                    elif name == 'jurAddress':
                        jurAddress = self.readAddress(name)
                    else:
                        self.gotoNextElement(name)
                else:
                    if name == 'tf_okato':
                        newSMORecord.setValue('OKATO', toVariant(forceString(self.text())))
                    elif name == 'smocod':
                        smocode = forceString(self.text())
                        newSMORecord.setValue('smoCode', toVariant(smocode))
                    elif name == 'nam_smop':
                        newSMORecord.setValue('fullName', toVariant(forceString(self.text())))
                    elif name == 'nam_smok':
                        newSMORecord.setValue('shortName', toVariant(forceString(self.text())))
                        newSMORecord.setValue('title', toVariant(forceString(self.text())))
                    elif name == 'inn':
                        newSMORecord.setValue('INN', toVariant(forceString(self.text())))
                    elif name == 'Ogrn':
                        newSMORecord.setValue('OGRN', toVariant(forceString(self.text())))
                    elif name == 'KPP':
                        kpp = forceString(self.text())
                        newSMORecord.setValue('KPP', toVariant(kpp))
                    elif name == 'fam_ruk':
                        fam_ruk = forceString(self.text())
                    elif name == 'im_ruk':
                        im_ruk = forceString(self.text())
                    elif name == 'ot_ruk':
                        ot_ruk = forceString(self.text())
                    elif name == 'phone':
                        phone.append(u'телефон: %s'%forceString(self.text()))
                    elif name == 'fax':
                        phone.append(u'факс: %s'%forceString(self.text()))
                    elif name == 'www':
                        notes.append(u'сайт: %s'%forceString(self.text()))
                    elif name == 'e_mail':
                        notes.append(u'эл. почта: %s'%forceString(self.text()))
                        
        if pstAddress:
            newSMORecord.setValue('Address', pstAddress)
        elif jurAddress:
            newSMORecord.setValue('Address', jurAddress)
        
        newSMORecord.setValue('chief', toVariant(u' '.join([fam_ruk, im_ruk, ot_ruk])))
        newSMORecord.setValue('notes', toVariant(u'\n'.join(notes)))
        newSMORecord.setValue('phone', toVariant(u'; '.join(phone)))
        newSMORecord.setValue('isInsurer', toVariant(1))
        
        if kpp:
            region = kpp[:2]
            if smocode:
                if region != smocode[:2]:
                    region = None
            if region and len(region) == 2:
                newSMORecord.setValue('area', toVariant('%s00000000000'%region))

        self.log(u'Организация: %s' % forceString(newSMORecord.value('fullName')))

#        if self.skip:
#            self.log(u'Информация по тарифу не полная. Пропускаем.',  True,  True)
#            self.nskipped += 1
#            self.skip = False
#        else:

        if self.hasError() or self._parent.aborted:
            return None

        key = self.getSMOKey(newSMORecord)
        recordList = self.mapSMOKeyToRecordList.get(key, None)
        if recordList:
            if self.updateSMO:
                for record in recordList:
                    diff = self.getSMODifference(record, newSMORecord)
                    if diff:
                        self.log(u'* Существующий организация "%s" отличается и будет обновлена' % forceString(record.value('shortName')),  True)
                        self.log(u': Различия: %s' % diff)
                        newSMORecord.setValue('id', record.value('id'))
                        QtGui.qApp.db.updateRecord(self.table, newSMORecord)
                        self.nupdated += 1
                    else:
                        self.log(u'* Изменений не обнаружено')
            else:
                self.log(u'%% Найдена совпадающая запись, пропускаем')
                self.ncoincident += 1
        else:
            self.log(u'% Сходные записи не обнаружены, добавляем')
            QtGui.qApp.db.insertRecord(self.table, newSMORecord)
            self.addSMOToMap(newSMORecord)
            self.nadded += 1

        self.nprocessed += 1
#        self._parent.lblStatus.setText(
#                u'импорт организаций: %d добавлено; %d обновлено; %d совпадений; %d неполных; %d обработано;' % (self.nadded,
#                self.nupdated, self.ncoincident, self.nskipped, self.nprocessed))

    def readAddress(self, addressName):
        address = ''
        while not ( self.isEndElement() and self.name() == addressName):
            if self.isStartElement():
                name = forceString(self.name())
                self.readNext()
                if name == 'addr_j' or name == 'addr_f':
                    address = forceString(self.text())
            else:
                self.readNext()
        return address

    def readUnknownElement(self):
        self.log(u'Неизвестный элемент: '+self.name().toString())
        self.gotoNextElement(forceString(self.name()))
    
    def gotoNextElement(self, elemName):
        while not (( self.isEndElement() and forceString(self.name()) == elemName) or self.atEnd()):
                self.readNext()
                
    def getSMODifference(self, oldRecord, newRecord):
        parts = []

        for fieldName in smoCheckFields:
            if fieldName not in smoKeyFields:
                oldValue = oldRecord.value(fieldName)
                newValue = newRecord.value(fieldName)
                if not variantEq(oldValue, newValue):
                    parts.append(u'%s было %s стало %s' % (fieldName, forceString(oldValue), forceString(newValue)))
        return ', '.join(parts)
