# -*- coding: utf-8 -*-

#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################


from PyQt4 import QtCore, QtGui, QtSql
from PyQt4.QtCore import *

from library.Utils import *
from library.roman import itor

from Ui_Progress import Ui_ProgressDialog


class CProgressDialog(QtGui.QDialog, Ui_ProgressDialog):
    __pyqtSignals__ = ('startWork()', )

    def __init__(self, parent):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)
        self.setInsurersCount(0)
        self.setClientsCount(0)
        self.setOrgsCount(0)

        self.setKLADRAdrCount(0)
        self.setKLADRStreetCount(0)
        self.setKLADRDomaCount(0)

        self.start()
        self.connect(self, SIGNAL('startWork()'), self.startWork, Qt.QueuedConnection)

        self.accountingSystemId  = None
        self.policyTypeId        = None

        self.passportRFId        = None
        self.passportUSSRId      = None
        self.birthCertificateId  = None
        self.contactTypeId       = None

        self.dbfInsurers = None
        self.dbfRegistry = None
        self.dbfOrgs     = None

        self.dbfKLADAdr       = None
        self.dbfKLADRStreet  = None
        self.dbfKLADRDoma   = None

        self.addNonexistentDeleted = True
        self.addDummyOrganisations  = True
        self.addAllOrganisations    = True
        self.birthDateOldMan = QDate(1978, 1, 1)

        self.sexMap = {0:2, 1:1}
        self.insurerCache = {}
        self.OKFSCache = {}


    def _setCount(self, progressBar, cnt):
        if cnt:
            progressBar.setValue(0)
            progressBar.setMaximum(cnt)
            progressBar.setEnabled(True)
            progressBar.setTextVisible(True)
        else:
            progressBar.setValue(0)
            progressBar.setMaximum(1)
            progressBar.setEnabled(False)
            progressBar.setTextVisible(False)


    def setInsurersCount(self, cnt):
        self._setCount(self.prbInsurers, cnt)


    def setClientsCount(self, cnt):
        self._setCount(self.prbClients, cnt)


    def setOrgsCount(self, cnt):
        self._setCount(self.prbOrgs, cnt)

    def setKLADRAdrCount(self, cnt):
        self._setCount(self.prbKLADRAdr, cnt)

    def setKLADRStreetCount(self, cnt):
        self._setCount(self.prbKLADRStreet, cnt)

    def setKLADRDomaCount(self, cnt):
        self._setCount(self.prbKLADRDoma, cnt)


    def start(self):
        self.processDone = False
        self.aborted = False
        self.btnCancel.setText(u'Отмена')
        self.edtLog.setPlainText('')


    def stop(self):
        self.processDone = True
        self.btnCancel.setText(u'Закрыть')


    def done(self, r):
        self.stop()
        QtGui.QDialog.done(self, r)

    def exec_(self):
        self.emit(SIGNAL('startWork()'))

        return QtGui.QDialog.exec_(self)


    @pyqtSignature('')
    def on_btnCancel_clicked(self):
        if self.processDone:
            self.accept()
        else:
            self.aborted = True


    @pyqtSignature('')
    def on_actStartWork_triggered(self):
        self.startWork()


    def startWork(self):
        self.setKLADRAdrCount(len(self.dbfKLADRAdr) if self.dbfKLADRAdr else 0)
        self.setKLADRStreetCount(len(self.dbfKLADRStreet) if self.dbfKLADRStreet else 0)
        self.setKLADRDomaCount(len(self.dbfKLADRDoma) if self.dbfKLADRDoma else 0)

        self.setInsurersCount(len(self.dbfInsurers) if self.dbfInsurers else 0)
        self.setClientsCount(len(self.dbfRegistry) if self.dbfRegistry else 0)
        self.setOrgsCount(len(self.dbfOrgs) if self.dbfOrgs else 0)
        self.start()

        self.processKladr(self.dbfKLADRAdr, self.prbKLADRAdr, 'kladr', u'КЛАДР: адреса')
        self.processKladr(self.dbfKLADRStreet, self.prbKLADRStreet, 'street', u'КЛАДР: улицы')
        self.processKladr(self.dbfKLADRDoma, self.prbKLADRDoma, 'doma', u'КЛАДР: дома')

        self.process(self.dbfInsurers, self.processInsurer, self.prbInsurers, u'СМО')
        self.process(self.dbfRegistry, self.processClient,  self.prbClients, u'Население')
        self.process(self.dbfOrgs,     self.processOrg,     self.prbOrgs, u'Организации')
        self.stop()


    def process(self, dbf, step, progressBar, logTitle):
        if dbf:
            self.edtLog.append(u'%s [%s записей]'%(logTitle, len(dbf)))
            self.edtLog.append(u'Файл: %s' % dbf.name)
            self.changes = {}
            for row in dbf:
                QtGui.qApp.processEvents()
                if self.aborted:
                    self.reject()
                    return
                progressBar.setValue(progressBar.value() + 1)
                QtGui.qApp.db.transaction()
                try:
                    step(row)
                    QtGui.qApp.db.commit()
                except:
                    QtGui.qApp.db.rollback()
                    raise
            progressBar.setValue(progressBar.maximum())
            logActions = self.changes.keys();
            logActions.sort()
            mapLogAction = {1: u'добавлено',  2: u'обновлено',  0: u'удалено'}
            for logAction in logActions:
                self.edtLog.append(u'    Таблица "%s": %s %s записей' % (logAction[0], mapLogAction[logAction[1]], str(self.changes[logAction])))


    def addChange(self,  table,  op,  cnt=1):
        s = (table, op)
        if not self.changes.has_key(s):
            self.changes[s] = 0
        self.changes[s] += cnt

# *****************************************************************************************

    def processInsurer(self, row):
        npp = row['NPP']
        orgId = self.getInsurerId(npp)
        if orgId:
            self.updateInsurer(orgId, row['NAME'], row['DIR'], row['COD_OKPO'])
        else:
            orgId = self.addInsurer(npp, row['NAME'], row['DIR'], row['COD_OKPO'])
        self.insurerCache[npp] = orgId



    def addInsurer(self, npp, name, dir, OKPO):
        table = QtGui.qApp.db.table('Organisation')
        record = table.newRecord()
# OKONH, ?
        record.setValue('fullName', toVariant(name))
        record.setValue('shortName', toVariant(name))
        record.setValue('title', toVariant(name))
        record.setValue('infisCode', toVariant(dir))
        record.setValue('OKPO', toVariant(OKPO))
        record.setValue('isInsurer', toVariant(1))
        record.setValue('notes', toVariant(self.getInsurerNotes(npp)))
        orgId = QtGui.qApp.db.insertRecord(table, record)
        self.addChange('Organisation', 1)
        return orgId

    def _getInsurerId(self, insurerCode):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecordEx(table, 'id', [table['deleted'].eq(0),
                                                         table['notes'].eq(self.getInsurerNotes(insurerCode)),
                                                         table['isInsurer'].eq(1)
                                                        ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return 0


    def getInsurerId(self, insurerCode):
        if insurerCode in self.insurerCache:
            return self.insurerCache[insurerCode]
        else:
            result = self._getInsurerId(insurerCode)
            self.insurerCache[insurerCode] = result
            return result


    def getInsurerNotes(self, npp):
        return u'СМО # %s' % npp


    def updateInsurer(self, orgId, name, dir, OKPO):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecord(table, '*', orgId)
        record.setValue('fullName', toVariant(name))
        record.setValue('shortName', toVariant(name))
        record.setValue('title', toVariant(name))
        record.setValue('infisCode', toVariant(dir))
        record.setValue('OKPO', toVariant(OKPO))
        QtGui.qApp.db.updateRecord(table, record)
        self.addChange('Organisation', 2)


    def _getOKFSId(self, OKFS):
        return forceRef(QtGui.qApp.db.translate('rbOKFS', 'code', OKFS, 'id'))


    def getOKFSId(self, OKFS):
        if OKFS in self.OKFSCache:
            return self.OKFSCache[OKFS]
        else:
            result = self._getOKFSId(OKFS)
            self.OKFSCache[OKFS] = result
            return result

# *****************************************************************************************

    def findClientByNPP(self, npp):
        table = QtGui.qApp.db.table('ClientIdentification')
        record = QtGui.qApp.db.getRecordEx(table, 'client_id', [table['deleted'].eq(0),
                                                          table['accountingSystem_id'].eq(self.accountingSystemId),
                                                          table['identifier'].eq(npp)
                                                         ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None

    def findClientByNameSexAndBirthDate(self, lastName, firstName, patrName, sex, birthDate):
        table = QtGui.qApp.db.table('Client')
        record = QtGui.qApp.db.getRecordEx(table, 'id',  [table['deleted'].eq(0),
                                                          table['lastName'].eq(lastName),
                                                          table['firstName'].eq(firstName),
                                                          table['patrName'].eq(patrName),
                                                          table['sex'].eq(sex),
                                                          table['birthDate'].eq(birthDate)
                                                         ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None


    def findClientBySNILS(self, SNILS):
        table = QtGui.qApp.db.table('Client')
        record = QtGui.qApp.db.getRecordEx(table, 'id',  [table['deleted'].eq(0),
                                                          table['SNILS'].eq(SNILS),
                                                         ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None


    def addClientIdentification(self, clientId, npp):
        table = QtGui.qApp.db.table('ClientIdentification')
        record = table.newRecord()
        record.setValue('client_id', toVariant(clientId))
        record.setValue('accountingSystem_id', toVariant(self.accountingSystemId))
        record.setValue('identifier', toVariant(npp))
        record.setValue('checkDate', toVariant(QDate.currentDate()))
        clientId = QtGui.qApp.db.insertRecord(table, record)
        self.addChange('ClientIdentification', 1)


    def updateClientIdentification(self, clientId, npp):
        table = QtGui.qApp.db.table('ClientIdentification')
        record = QtGui.qApp.db.getRecordEx(table, '*',   [table['deleted'].eq(0),
                                                          table['accountingSystem_id'].eq(self.accountingSystemId),
                                                          table['client_id'].eq(clientId),
                                                         ], 'id')
        if record:
            record.setValue('identifier', toVariant(npp))
            record.setValue('checkDate', toVariant(QDate.currentDate()))
            QtGui.qApp.db.updateRecord(table, record)
            self.addChange('ClientIdentification', 2)
        else:
            self.addClientIdentification(self, clientId, npp)



    def addClient(self, npp, lastName, firstName, patrName, birthDate, sex, SNILS):
        table = QtGui.qApp.db.table('Client')
        record = table.newRecord()
        record.setValue('lastName',  toVariant(lastName))
        record.setValue('firstName', toVariant(firstName))
        record.setValue('patrName',  toVariant(patrName))
        record.setValue('birthDate', toVariant(birthDate))
        record.setValue('sex',       toVariant(sex))
        record.setValue('SNILS',     toVariant(SNILS))
        clientId = QtGui.qApp.db.insertRecord(table, record)
        self.addChange('Client', 1)
        self.addClientIdentification(clientId, npp)
        return clientId


    def updateClient(self, clientId, lastName, firstName, patrName, birthDate, sex, SNILS):
        table = QtGui.qApp.db.table('Client')
        record = QtGui.qApp.db.getRecord(table, '*', clientId)
        record.setValue('lastName', toVariant(lastName))
        record.setValue('firstName', toVariant(firstName))
        record.setValue('patrName', toVariant(patrName))
        record.setValue('birthDate', toVariant(birthDate))
        record.setValue('sex', toVariant(sex))
        record.setValue('SNILS', toVariant(SNILS))
        QtGui.qApp.db.updateRecord(table, record)
        self.addChange('Client', 2)


    def addClientPolicy(self, clientId, insurerCode, serial, number, deleted, dateOfDeletion):
        table = QtGui.qApp.db.table('ClientPolicy')
        record = table.newRecord()
        record.setValue('client_id', toVariant(clientId))
        record.setValue('insurer_id', toVariant(self.getInsurerId(insurerCode)))
        record.setValue('policyType_id', toVariant(self.policyTypeId))
        record.setValue('serial', toVariant(serial))
        record.setValue('number', toVariant(number))
        if deleted:
            record.setValue('endDate', toVariant(dateOfDeletion))
        QtGui.qApp.db.insertRecord(table, record)
        self.addChange('ClientPolicy', 1)


    def updateClientPolicy(self, clientId, insurerCode, serial, number, deleted, dateOfDeletion):
        table = QtGui.qApp.db.table('ClientPolicy')
        record = QtGui.qApp.db.getRecordEx(table, '*', [table['client_id'].eq(clientId),
                                                  table['insurer_id'].eq(self.getInsurerId(insurerCode)),
                                                  table['serial'].eq(serial),
                                                  table['number'].eq(number)], 'id')
        if record:
            if forceDate(record.value('endDate')) != dateOfDeletion:
                if deleted:
                    record.setValue('endDate', toVariant(dateOfDeletion))
                else:
                    record.setValue('endDate', toVariant(None))
                QtGui.qApp.db.updateRecord(table, record)
                self.addChange('ClientPolicy', 2)
        else:
            self.addClientPolicy(clientId, insurerCode, serial, number, deleted, dateOfDeletion)


    def suggestDocumentType(self, birthDate, ns, s, num):
        if ns and s and num:
            canHavePassportUSSR = birthDate < self.birthDateOldMan
            sIsNumeric = s.isdigit()
            sIsLat     = s.isdigit()
            if sIsNumeric:
                return self.passportRFId, '%02d %s' % (ns, s), '%06d' % num
            else:
                if canHavePassportUSSR:
                    return self.passportUSSRId, '%s %s' % (itor(ns), s), '%06d' % num
                else:
                    return self.birthCertificateId, '%s %s' % (itor(ns), s), '%06d' % num
        else:
            return None, '', ''


    def addClientDocument(self, clientId, documentTypeId, serial, number):
        if documentTypeId:
            table = QtGui.qApp.db.table('ClientDocument')
            record = table.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('documentType_id', toVariant(documentTypeId))
            record.setValue('serial', toVariant(serial))
            record.setValue('number', toVariant(number))
            QtGui.qApp.db.insertRecord(table, record)
            self.addChange('ClientDocument', 1)


    def updateClientDocument(self, clientId, documentTypeId, serial, number):
        if documentTypeId:
            table = QtGui.qApp.db.table('ClientDocument')
            record = QtGui.qApp.db.getRecordEx(table, '*', [table['client_id'].eq(clientId),
                                                      table['serial'].eq(serial),
                                                      table['number'].eq(number)], 'id')
            if not record:
                self.addClientDocument(clientId, documentTypeId, serial, number)


    def isKLADRCodeExists(self, code):
        return forceString(QtGui.qApp.db.translate('kladr.KLADR', 'CODE', code, 'CODE')) == code


    def isKLADRStreetCodeExists(self, code):
        return forceString(QtGui.qApp.db.translate('kladr.STREET', 'CODE', code, 'CODE')) == code

#    def findAlteredCode(self, code):

    def getActualKLADRCode(self, shortedCode):
        code = shortedCode+'00'
        if self.isKLADRCodeExists(code):
            return code
        alteredCode = forceString(QtGui.qApp.db.translate('kladr.ALTNAMES', 'OLDCODE', code, 'NEWCODE'))
        if alteredCode:
            return alteredCode
        return code


    def getActualKLADRStreetCode(self, shortedCode):
        code = shortedCode+'00'
        if self.isKLADRStreetCodeExists(code):
            return code
        alteredCode = forceString(QtGui.qApp.db.translate('kladr.ALTNAMES', 'OLDCODE', code, 'NEWCODE'))
        if alteredCode:
            return alteredCode
        KLADRpart, streetPart = code[:11], code[11:]
        KLADRpart = self.getActualKLADRCode(KLADRpart)[:-2]
        return KLADRpart+streetPart


    def getAddressHouseId(self, streetCode, house, houseInd):
        if len(streetCode) == 11:
            KLADRCode = self.getActualKLADRCode(streetCode)
            KLADRStreetCode = ''
        elif len(streetCode) == 15:
            KLADRStreetCode = self.getActualKLADRStreetCode(streetCode)
            KLADRCode = KLADRStreetCode[:11]+'00'
        else:
            return None

        if not self.isKLADRCodeExists(KLADRCode):
            pass
        if not self.isKLADRStreetCodeExists(KLADRStreetCode):
            pass

        number = str(house) if house else ''
        table = QtGui.qApp.db.table('AddressHouse')
        record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                  table['KLADRCode'].eq(KLADRCode),
                                                  table['KLADRStreetCode'].eq(KLADRStreetCode),
                                                  table['number'].eq(number),
                                                  table['corpus'].eq(houseInd)], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            record = table.newRecord()
            record.setValue('KLADRCode', toVariant(KLADRCode))
            record.setValue('KLADRStreetCode', toVariant(KLADRStreetCode))
            record.setValue('number', toVariant(number))
            record.setValue('corpus', toVariant(houseInd))
            self.addChange('AddressHouse', 1)
            return QtGui.qApp.db.insertRecord(table, record)


    def isKLADRStreetCodeExists(self, code):
        return forceString(QtGui.qApp.db.translate('kladr.STREET', 'CODE', code, 'CODE')) == code


        number = str(house) if house else ''
        table = QtGui.qApp.db.table('AddressHouse')
        record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                  table['KLADRCode'].eq(KLADRCode),
                                                  table['KLADRStreetCode'].eq(KLADRStreetCode),
                                                  table['number'].eq(number),
                                                  table['corpus'].eq(houseInd)], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            record = table.newRecord()
            record.setValue('KLADRCode', toVariant(KLADRCode))
            record.setValue('KLADRStreetCode', toVariant(KLADRStreetCode))
            record.setValue('number', toVariant(number))
            record.setValue('corpus', toVariant(houseInd))
            self.addChange('AddressHouse', 1)
            return QtGui.qApp.db.insertRecord(table, record)


    def getAddressId(self, streetCode, house, houseInd, room, roomInd):
        houseId = self.getAddressHouseId(streetCode, house, houseInd)
        if houseId:
            if room:
                flat = str(room)
                if roomInd:
                    flat = flat + ' '+roomInd
            else:
                flat = roomInd
            table = QtGui.qApp.db.table('Address')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                      table['house_id'].eq(houseId),
                                                      table['flat'].eq(flat)], 'id')
            if record:
                return forceRef(record.value(0))
            else:
                record = table.newRecord()
                record.setValue('house_id', toVariant(houseId))
                record.setValue('flat', toVariant(flat))
                self.addChange('Address', 1)
                return QtGui.qApp.db.insertRecord(table, record)
        else:
            return None


    def ensureClientAddress(self, clientId, addressType, addressId, asString):
        table = QtGui.qApp.db.table('ClientAddress')
        record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                  table['client_id'].eq(clientId),
                                                  table['type'].eq(addressType),
                                                  table['address_id'].eq(addressId),
                                                  table['freeInput'].eq(asString),
                                                 ], 'id')
        if not record:
            record = table.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('type',      toVariant(addressType))
            record.setValue('address_id',toVariant(addressId))
            record.setValue('freeInput', toVariant(asString))
            QtGui.qApp.db.insertRecord(table, record)
            self.addChange('ClientAddress', 1)


    def addClientAddress(self, clientId, asString, streetCode, house, houseInd, room, roomInd):
        addressId = self.getAddressId(streetCode, house, houseInd, room, roomInd)
        self.ensureClientAddress(clientId, 0, addressId, asString)
        self.ensureClientAddress(clientId, 1, addressId, asString)


    def updateClientAddress(self, clientId, asString, streetCode, house, houseInd, room, roomInd):
        addressId = self.getAddressId(streetCode, house, houseInd, room, roomInd)
        self.ensureClientAddress(clientId, 0, addressId, asString)
        self.ensureClientAddress(clientId, 1, addressId, asString)


    def getOrganisationId(self, INN, KPP):
        if INN and KPP:
            table = QtGui.qApp.db.table('Organisation')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                      table['INN'].eq(INN),
                                                      table['KPP'].eq(KPP)], 'id')
            if record:
                return forceRef(record.value(0))
            elif self.addDummyOrganisations:
                record = table.newRecord()
                name = u'ИНН: %s; КПП: %s' % (INN, KPP)
                record.setValue('fullName', toVariant(name))
                record.setValue('shortName', toVariant(name))
                record.setValue('INN', toVariant(INN))
                record.setValue('KPP', toVariant(KPP))
                self.addChange('Organisation', 1)
                return QtGui.qApp.db.insertRecord(table, record)
        return None


    def _addClientWork(self, clientId, organisationId):
        table = QtGui.qApp.db.table('ClientWork')
        record = table.newRecord()
        record.setValue('client_id', toVariant(clientId))
        record.setValue('org_id', toVariant(organisationId))
        QtGui.qApp.db.insertRecord(table, record)
        self.addChange('ClientWork', 1)


    def addClientWork(self, clientId, INN, KPP):
        organisationId = self.getOrganisationId(INN, KPP)
        if organisationId:
            self._addClientWork(clientId, organisationId)


    def updateClientWork(self, clientId, INN, KPP):
        organisationId = self.getOrganisationId(INN, KPP)
        if organisationId:
            table = QtGui.qApp.db.table('ClientWork')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                      table['client_id'].eq(clientId),
                                                      table['org_id'].eq(organisationId)], 'id')
            if not record:
                self._addClientWork(clientId, organisationId)


    def addClientPhone(self, clientId, phone):
        if phone and phone != '0':
            table = QtGui.qApp.db.table('ClientContact')
            record = table.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('contactType_id', toVariant(self.contactTypeId))
            record.setValue('contact', toVariant(phone))
            QtGui.qApp.db.insertRecord(table, record)
            self.addChange('ClientContact', 1)


    def updateClientPhone(self, clientId, phone):
        if phone and phone != '0':
            table = QtGui.qApp.db.table('ClientContact')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                      table['client_id'].eq(clientId),
                                                      table['contactType_id'].eq(self.contactTypeId),
                                                      table['contact'].eq(phone)], 'id')
            if not record:
                self.addClientPhone(clientId, phone)


    def processClient(self, row):
        def toQDate(d):
            if d is None:
                return QDate()
            else:
                return QDate(d)

        npp = row['NPP']
        lastName  = nameCase(row['FAMILY'])
        firstName = nameCase(row['NAME'])
        patrName  = nameCase(row['FATHER'])
        sex       = self.sexMap.get(row['SEX'], 0)
        birthDate = toQDate(row['DATE'])
        SNILS     = row['PERS_NUM']

        clientId = self.findClientByNPP(npp)
        if not clientId:
            clientId = self.findClientByNameSexAndBirthDate(lastName, firstName, patrName, sex, birthDate)
        if not clientId and SNILS:
            clientId = self.findClientBySNILS(SNILS)
        if clientId:
            docTypeId, docSerial, docNumber = self.suggestDocumentType(birthDate, row['NSDOC'], row['SDOC'], row['NDOC'])
            self.updateClientIdentification(clientId, npp)
            self.updateClient(clientId, lastName, firstName, patrName, toQDate(row['DATE']), sex, SNILS)
            self.updateClientPolicy(clientId, row['COD_SMO'], row['SERIES'], row['NUMBER'], row['DELETED'], toQDate(row['DATEDEL']))
            self.updateClientDocument(clientId, docTypeId, docSerial, docNumber)
            self.updateClientAddress(clientId, row['ADDRESS'], row['CITYSTREET'], row['HOUSE'], row['IND_HOUSE'], row['ROOM'], row['IND_ROOM'])
            self.updateClientWork(clientId, row['OFFICE_INN'], row['OFFICE_KPP'])
            self.updateClientPhone(clientId, row['TELEPHONE'])
        else:
            if row['DELETED'] and not self.addNonexistentDeleted:
                return
            docTypeId, docSerial, docNumber = self.suggestDocumentType(birthDate, row['NSDOC'], row['SDOC'], row['NDOC'])
            clientId = self.addClient(npp, lastName, firstName, patrName, toQDate(row['DATE']), sex, SNILS)
            self.addClientPolicy(clientId, row['COD_SMO'], row['SERIES'], row['NUMBER'], row['DELETED'], toQDate(row['DATEDEL']))
            self.addClientDocument(clientId, docTypeId, docSerial, docNumber)
            self.addClientAddress(clientId, row['ADDRESS'], row['CITYSTREET'], row['HOUSE'], row['IND_HOUSE'], row['ROOM'], row['IND_ROOM'])
            self.addClientWork(clientId, row['OFFICE_INN'], row['OFFICE_KPP'])
            self.addClientPhone(clientId, row['TELEPHONE'])

# *****************************************************************************************

    def processOrg(self, row):
        INN = row['INN']
        KPP = row['KPP']
        orgId = self.findOrgId(INN, KPP)
        if orgId:
            self.updateOrg(orgId, row['SHORTNAME'], row['FULLNAME'], row['KONX'], row['KFS'], row['OKVED'], row['OGRN'], row['ADRES'], row['DIRECTOR'], row['PHONE'])
        elif self.addAllOrganisations:
            self.addOrg(row['SHORTNAME'], row['FULLNAME'], INN, KPP, row['KONX'], row['KFS'], row['OKVED'], row['OGRN'], row['ADRES'], row['DIRECTOR'], row['PHONE'])


    def findOrgId(self, INN, KPP):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecordEx(table, 'id',  [table['deleted'].eq(0),
                                                          table['INN'].eq(INN),
                                                          table['KPP'].eq(KPP)
                                                         ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None


    def addOrg(self, shortName, fullName, INN, KPP, OKONH, OKFS, OKVED, OGRN, address, chief, phone):
        table = QtGui.qApp.db.table('Organisation')
        record = table.newRecord()
# OKONH, ?
        record.setValue('fullName', toVariant(fullName))
        record.setValue('shortName', toVariant(shortName))
        record.setValue('title', toVariant(shortName))
        record.setValue('OKVED', toVariant(OKVED))
        record.setValue('INN', toVariant(INN))
        record.setValue('KPP', toVariant(KPP))
        record.setValue('OGRN', toVariant(OGRN))
        record.setValue('OKFS_id', toVariant(self.getOKFSId(OKFS)))
        record.setValue('address', toVariant(address))
        record.setValue('chief', toVariant(chief))
        record.setValue('phone', toVariant(phone))
        orgId = QtGui.qApp.db.insertRecord(table, record)
        self.addChange('Organisation', 1)
        return orgId


    def updateOrg(self, orgId, shortName, fullName, OKONH, OKFS, OKVED, OGRN, address, chief, phone):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecord(table, '*', orgId)
        record.setValue('fullName', toVariant(fullName))
        record.setValue('shortName', toVariant(shortName))
        record.setValue('title', toVariant(shortName))
        record.setValue('OKVED', toVariant(OKVED))
        record.setValue('OGRN', toVariant(OGRN))
        record.setValue('OKFS_id', toVariant(self.getOKFSId(OKFS)))
        record.setValue('address', toVariant(address))
        record.setValue('chief', toVariant(chief))
        record.setValue('phone', toVariant(phone))
        QtGui.qApp.db.updateRecord(table, record)
        self.addChange('Organisation', 2)


    def _getOKFSId(self, OKFS):
        return forceRef(QtGui.qApp.db.translate('rbOKFS', 'code', OKFS, 'id'))


    def getOKFSId(self, OKFS):
        if OKFS in self.OKFSCache:
            return self.OKFSCache[OKFS]
        else:
            result = self._getOKFSId(OKFS)
            self.OKFSCache[OKFS] = result
            return result

# *****************************************************************************************

    def processKladr(self, dbf, progressBar, tableName, logTitle):
        if dbf:
            self.btnCancel.setEnabled(False)
            try:
                self.edtLog.append(u'%s [%s записей]'%(logTitle, len(dbf)))
                self.edtLog.append(u'Файл: %s' % dbf.name)
                self.changes = {}

                sqlInsert = 'insert into `kladr`.`%s`(`code`, `name`, `socr`, `index` %s) values (:code, :name, :socr, :index %s)' % (tableName, ', `parent`' if tableName == 'kladr' else '', ', :parent' if tableName == 'kladr' else '')
                sqlDelete = 'delete from `kladr`.`%s` where `code` like "74%%" or `code` like "02%%"' % tableName
                qIns = QtSql.QSqlQuery(QtGui.qApp.db.db)
                qIns.prepare(sqlInsert)
                qDel = QtSql.QSqlQuery(QtGui.qApp.db.db)
                QtGui.qApp.db.transaction()
                try:
                    if not qDel.exec_(sqlDelete):
                        raise Exception, qDel.lastError().databaseText()
                    insCount = 0
                    for row in dbf:
                        if (row.index % 100) == 0:
                            QtGui.qApp.processEvents()
                            progressBar.setValue(progressBar.value() + 100)
                        if row['CODE'][:2] in ('02', '74'):
                            qIns.bindValue(':code', toVariant(row['CODE']))
                            qIns.bindValue(':name', toVariant(row['NAME']))
                            qIns.bindValue(':socr', toVariant(row['SOCR']))
                            qIns.bindValue(':index', toVariant(row['INDEX']))
                            if tableName == 'kladr':
                                qIns.bindValue(':parent', toVariant(self.getKLADRAdrParent(row['CODE'])))
                            if not qIns.exec_():
                                raise Exception, qDel.lastError().databaseText()
                            insCount += 1
                    progressBar.setValue(progressBar.maximum())
                    QtGui.qApp.db.commit()
                    self.edtLog.append(u'    Таблица "%s": удалено %d записей' % ('kladr.' + tableName, qDel.numRowsAffected()))
                    self.edtLog.append(u'    Таблица "%s": добавлено %d записей' % ('kladr.' + tableName, insCount))
                except: #Exception, e:
                    QtGui.qApp.db.rollback()
                    self.edtLog.append(u'    !!! ОШИБКА !!!')
                    #err = forceString(e)
                    #err = QString(err)
                    #self.edtLog.append(err)
                    #raise
            finally:
                self.btnCancel.setEnabled(True)
                QtGui.qApp.processEvents()

# *****************************************************************************************

    def processKLADRAdr(self, row):
        if row['CODE'][:2] not in ('02', '74'):
            return
        if self.findKladrAdrCode(row['CODE']):
            self.updateKladrAdr(row['CODE'], row['NAME'], row['SOCR'], row['INDEX'])
        else:
            self.addKladrAdr(row['CODE'], row['NAME'], row['SOCR'], row['INDEX'])


    def findKladrAdrCode(self, code):
        table = QtGui.qApp.db.table('kladr.kladr')
        table.setIdFieldName('CODE')
        record = QtGui.qApp.db.getRecordEx(table, 'CODE', [table['CODE'].eq(code)])
        if record:
            return forceRef(record.value(0))
        else:
            return None

    def getKLADRAdrParent(self, code):
        if code[8:11] != '000':
            if code[5:8] != '000':
                return code[:8]
            elif code[2:5] != '000':
                return code[:5]
            else:
                return code[:3]
        elif code[5:8] != '000':
            if code[2:5] != '000':
                return code[:5]
            else:
                return code[:3]
        elif code[2:5] != '000':
            return code[:3]
        else:
            return ''


    def addKladrAdr(self, code, name, socr, index):
        table = QtGui.qApp.db.table('kladr.kladr')
        record = table.newRecord()
        record.setValue('code', toVariant(code))
        record.setValue('name', toVariant(name))
        record.setValue('socr', toVariant(socr))
        record.setValue('index', toVariant(index))
        record.setValue('parent', toVariant(self.getKLADRAdrParent(code)))
        kladrCode = QtGui.qApp.db.insertRecord(table, record)
        self.addChange('kladr.kladr', 1)
        return kladrCode


    def updateKladrAdr(self, code, name, socr, index):
        table = QtGui.qApp.db.table('kladr.kladr')
        record = QtGui.qApp.db.getRecord(table, '*', code)
        record.setValue('name', toVariant(name))
        record.setValue('socr', toVariant(socr))
        record.setValue('index', toVariant(index))
        record.setValue('parent', toVariant(self.getKLADRAdrParent(code)))
        QtGui.qApp.db.updateRecord(table, record)
        self.addChange('kladr.kladr', 2)


# *****************************************************************************************

    def processKLADRStreet(self, row):
        if row['CODE'][:2] not in ('02', '74'):
            return
        if self.findKladrStreetCode(row['CODE']):
            self.updateKladrStreet(row['CODE'], row['NAME'], row['SOCR'], row['INDEX'])
        else:
            self.addKladrStreet(row['CODE'], row['NAME'], row['SOCR'], row['INDEX'])


    def findKladrStreetCode(self, code):
        table = QtGui.qApp.db.table('kladr.street')
        table.setIdFieldName('CODE')
        record = QtGui.qApp.db.getRecordEx(table, 'CODE', [table['CODE'].eq(code)])
        if record:
            return forceRef(record.value(0))
        else:
            return None


    def addKladrStreet(self, code, name, socr, index):
        table = QtGui.qApp.db.table('kladr.street')
        record = table.newRecord()
        record.setValue('code', toVariant(code))
        record.setValue('name', toVariant(name))
        record.setValue('socr', toVariant(socr))
        record.setValue('index', toVariant(index))
        kladrCode = QtGui.qApp.db.insertRecord(table, record)
        self.addChange('kladr.street', 1)


    def updateKladrStreet(self, code, name, socr, index):
        table = QtGui.qApp.db.table('kladr.street')
        record = QtGui.qApp.db.getRecord(table, '*', code)
        record.setValue('name', toVariant(name))
        record.setValue('socr', toVariant(socr))
        record.setValue('index', toVariant(index))
        QtGui.qApp.db.updateRecord(table, record)
        self.addChange('kladr.street', 2)


# *****************************************************************************************

    def processKLADRDoma(self, row):
        if row['CODE'][:2] not in ('02', '74'):
            return
        self.addKladrDoma(row['CODE'], row['NAME'], row['SOCR'], row['INDEX'])


    def addKladrDoma(self, code, name, socr, index):
        table = QtGui.qApp.db.table('kladr.doma')
        table.setIdFieldName('CODE')
        record = table.newRecord()
        record.setValue('code', toVariant(code))
        record.setValue('name', toVariant(name))
        record.setValue('socr', toVariant(socr))
        record.setValue('index', toVariant(index))
        kladrCode = QtGui.qApp.db.insertRecord(table, record)
        self.addChange('kladr.doma', 1)
