# -*- coding: utf-8 -*-

#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################


import re
import os.path

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import *

from library.Utils import *
from library.roman import itor
from library.dbfpy.dbf import Dbf
from library.PrintTemplates import  applyTemplate
from Registry.Utils import selectLatestRecord, getClientWork

from Ui_Progress import Ui_ProgressDialog


class CProgressDialog(QtGui.QDialog, Ui_ProgressDialog):
    __pyqtSignals__ = ('startWork()', )
    importTypeDMS = 0
    importTypeServices = 1
    importTypeOMS = 2
    importTypeAttach = 3
    importTypeDetach = 4

    def __init__(self, parent):
        QtGui.QDialog.__init__(self, parent)
        self._parent = parent
        self.setupUi(self)
        self.setInsuranceTypeCount(0)
        self.setOrgsCount(0)
        self.setClientsCount(0)
        self.setServicesCount(0)

        self.start()
        self.connect(self, SIGNAL('startWork(int)'), self.startWork, Qt.QueuedConnection)

        self.policyTypeId       = None
        self.policyTypeOMSId       = None
        self.passportRFId       = None
        self.passportUSSRId     = None
        self.birthCertificateId = None
        self.temporaryCertificateId = None
        self.contactTypeId      = None
        self.clientIdentificationId = None
        self.accountingSystemId = None
        self.attachTypeId = None
        self.matureNetId =  None
        self.financeId = None
        self.currentOrgId = None
        self.attachClose = None
        self.actionTypeParentGroupId =None
        self.personalInsurerId = None
        self.addDummyOrganisations = False
        self.serviceFinanceId = None
        self.recoverDeletedActionTypes = False

        self.checkUniqueNPP = False
        self.strictNPP = False
        self.convertAddressToKLADR = False

        self.nAdded = 0
        self.nSkipped = 0
        self.nUpdated = 0

        self.serviceLevelMap = {1:u'М', 2:u'О',  3: u'N'}
        self.sexMap = {u'Ж':2, u'ж':2, u'М':1, u'м':1}

        self.mapActionTypeCodeNameToId = {}

        self.insurerCache = {}
        self.OKFSCache = {}
        self.attachCache = {}
        self.showLog = True
        self.canCloseClientAttach = False
        self.addAttachClosed = False

        # данные для отчета
        self.statAttached ={}
        self.statProlongated = {}
        self.statDetached = {}
        self.statSkipped = []


    def log(self, str,  forceLog = False):
        if self.showLog or forceLog:
            self.logBrowser.append(str)
            self.logBrowser.update()


    def _setCount(self, progressBar, cnt):
        if cnt:
            progressBar.setValue(0)
            progressBar.setMaximum(cnt)
            progressBar.setEnabled(True)
            progressBar.setTextVisible(True)
        else:
            progressBar.setValue(0)
            progressBar.setMaximum(1)
            progressBar.setEnabled(False)
            progressBar.setTextVisible(False)


    def setClientsCount(self, cnt):
        self._setCount(self.prbClients, cnt)


    def setInsuranceTypeCount(self,  cnt):
        self._setCount(self.prbInsuranceTypes,  cnt)


    def setOrgsCount(self,  cnt):
        self._setCount(self.prbOrgs,  cnt)


    def setServicesCount(self,  cnt):
        self._setCount(self.prbServices,  cnt)


    def start(self):
        self.processDone = False
        self.aborted = False
        self.nAdded = 0
        self.nSkipped = 0
        self.nUpdated = 0
        self.btnCancel.setText(u'Отмена')


    def stop(self):
        self.processDone = True
        self.btnCancel.setText(u'Закрыть')


    def done(self, r):
        self.stop()
        QtGui.QDialog.done(self, r)


    def exec_(self, importType):
        self.emit(SIGNAL('startWork(int)'), importType)
        return QtGui.QDialog.exec_(self)


    @pyqtSignature('')
    def on_btnCancel_clicked(self):
        if self.processDone:
            self.accept()
        else:
            self.aborted = True


    def startWork(self,  importType):
        self.start()

        global lastChangedRev
        global lastChangedDate
        try:
            from buildInfo import lastChangedRev, lastChangedDate
        except:
            lastChangedRev  = 'unknown'
            lastChangedDate = 'unknown'

        self.log(u'Версия 2.5 (ревизия %s от %s)' % (lastChangedRev,  lastChangedDate))

        if importType == self.importTypeDMS:
            QtGui.qApp.call(self, self.work)
        elif importType == self.importTypeServices:
            QtGui.qApp.call(self, self.importServices)
        elif importType == self.importTypeAttach:
            QtGui.qApp.call(self, self.processAttach)
        elif importType == self.importTypeDetach:
            QtGui.qApp.call(self, self.processDetach)
        else:
            QtGui.qApp.call(self, self.importOMS)

        if importType != self.importTypeServices:
            self.log(u'Клиенты: добавлено "%d", обновлено "%d",'\
                u' пропущено "%d"' % (self.nAdded,  self.nUpdated,  self.nSkipped))

        self.stop()


    def work(self):
        rbDir = forceString(self._parent.edtRBDirName.text())

        dbfInsurers = Dbf(os.path.join(rbDir,'compan.dbf'), readOnly=True, encoding='cp866')
        dbfInsuranceTypes = Dbf(os.path.join(rbDir,'gos.dbf'), readOnly=True, encoding='cp866')
        dbfRegistry = Dbf(forceString(self._parent.edtMainDataFileName.text()), readOnly=True, encoding='cp866')

        self.setInsuranceTypeCount(len(dbfInsuranceTypes))
        self.process(dbfInsuranceTypes,  self.processInsuranceType,  self.prbInsuranceTypes)

        self.setOrgsCount(len(dbfInsurers))
        self.process(dbfInsurers,  self.processInsurer,  self.prbOrgs)

        self.setClientsCount(len(dbfRegistry))
        format = self.detectFormat(dbfRegistry)

        if format:
            self.log(u'<b><font color=green>Импорт ДМС формат версии %d</b>' % format)

        if format == 1:
            self.process(dbfRegistry, self.processClient, self.prbClients)
        elif format == 2:
            self.process(dbfRegistry, self.processDMSClientV2, self.prbClients)
        else:
            self.log(u'<b><font color=red>ОШИБКА</b>:'
                        u'Неизвестный формат файла %s' % \
                        forceString(self._parent.edtMainDataFileName.text()))

        for d in (dbfInsurers,  dbfRegistry,  dbfInsuranceTypes):
            d.close()


    def importOMS(self):
        rbDir = forceString(self._parent.edtRBDirName.text())

        dbfInsurers = Dbf(os.path.join(rbDir,'compan.dbf'), readOnly=True, encoding='cp866')
        dbfRegistry = Dbf(forceString(self._parent.edtOMSDataFileName.text()), readOnly=True, encoding='cp866')

        self.setOrgsCount(len(dbfInsurers))
        self.process(dbfInsurers,  self.processInsurer,  self.prbOrgs)

        self.setClientsCount(len(dbfRegistry))
        self.process(dbfRegistry, self.processOMSClient, self.prbClients)

        for d in (dbfInsurers,  dbfRegistry):
            d.close()


    def importServices(self):
        self.copyToActionType = self._parent.chkCopyToActionType.isChecked()

        dbfServices = Dbf(forceString(self._parent.edtServiceFileName.text()), readOnly=True, encoding='cp866')

        self.setServicesCount(len(dbfServices))
        self.actionTypeParentGroupIdList = self.searchActionTypeSubgorups(self.actionTypeParentGroupId)
        self.process(dbfServices, self.processService, self.prbServices)

        dbfServices.close()


    def processAttach(self):
        self.statAttached ={}
        self.statProlongated = {}
        self.statDetached = {}

        fileName = forceString(self._parent.edtAttachFileName.text())

        rbDir = forceString(self._parent.edtRBDirName.text())
        dbfAttach = Dbf(fileName, readOnly=True, encoding='cp866')
        dbfInsurers = Dbf(os.path.join(rbDir,'compan.dbf'), readOnly=True, encoding='cp866')
        dbfInsuranceTypes = Dbf(os.path.join(rbDir,'gos.dbf'), readOnly=True, encoding='cp866')

        if not self.checkFormat(dbfAttach, self.attachFieldNameType):
            return

        self.setInsuranceTypeCount(len(dbfInsuranceTypes))
        self.setOrgsCount(len(dbfInsurers))
        self.setClientsCount(len(dbfAttach))
        self.process(dbfInsuranceTypes,  self.processInsuranceType,  self.prbInsuranceTypes)
        self.process(dbfInsurers,  self.processInsurer,  self.prbOrgs)
        self.process(dbfAttach, self.processClientAttach, self.prbClients)

        dbfAttach.close()
        dbfInsurers.close()
        dbfInsuranceTypes.close()
        self.createReport(fileName)


    def processDetach(self):
        self.statAttached ={}
        self.statProlongated = {}
        self.statDetached = {}
        fileName = forceString(self._parent.edtDetachFileName.text())

        dbfDetach = Dbf(fileName, readOnly=True, encoding='cp866')

        if not self.checkFormat(dbfDetach, self.detachFieldNameType):
            return

        rbDir = forceString(self._parent.edtRBDirName.text())
        dbfInsurers = Dbf(oPr&ogress.pys.path.join(rbDir,'compan.dbf'), readOnly=True, encoding='cp866')

        self.setClientsCount(len(dbfDetach))
        self.setOrgsCount(len(dbfInsurers))

        self.process(dbfInsurers,  self.processInsurer,  self.prbOrgs)
        self.process(dbfDetach, self.processClientDetach, self.prbClients)
        dbfInsurers.close()
        dbfDetach.close()
        self.createReport(fileName)


    def process(self, dbf, step, progressBar):
# left for fast debug
#        for i in range(144, 145):
#            record = dbf[i]
        for record in dbf:
            QtGui.qApp.processEvents()
            if self.aborted:
                self.reject()
                return
            progressBar.setValue(progressBar.value()+1)
            QtGui.qApp.db.transaction()
            try:
                step(record)
                QtGui.qApp.db.commit()
            except:
                QtGui.qApp.db.rollback()
                raise

# *****************************************************************************************

    def createReport(self,  fileName):

        def reportSort(x, y):
            return -1 if x[2] < y[2] else 1 if x[2] > y[2] else 0

        tablesList = []
        prevData = None

        for key in set(self.statAttached.keys()+self.statProlongated.keys()+self.statDetached.keys()):
            (insurerId,  attachId, work, begDate, endDate) = key
            sInsurer = forceString(QtGui.qApp.db.translate('Organisation', 'id', insurerId, 'shortName'))
            sProgram = forceString(QtGui.qApp.db.translate('rbAttachType', 'id', attachId, 'name'))
            sBegDate = QDate.fromJulianDay(begDate) if begDate else QDate()
            sEndDate = QDate.fromJulianDay(endDate) if endDate else QDate()

            for title, data in ((u'Продление', self.statProlongated.get(key, [])),
                                (u'Прикрепление', self.statAttached.get(key, [])),
                                (u'Открепление', self.statDetached.get(key, []))):
                if data != []:
                    stat = {}
                    data.sort(cmp=reportSort)
                    stat['insurer'] = sInsurer
                    stat['program'] = sProgram
                    stat['work'] = work
                    stat['begDate'] = sBegDate
                    stat['endDate'] = sEndDate
                    stat['pagebreak'] = False
                    stat['title'] = title
                    stat['dataList'] = data
                    tablesList.append(stat)

            if len(tablesList) > 0:
                if prevData:
                    (prevInsurerId,  prevAttachId, prevWork, prevBegDate, prevEndDate) = prevData
                    stat['pagebreak']  =  not ((prevInsurerId == insurerId) and (prevAttachId == attachId) and (prevWork == work))
                else:
                    tablesList[-1]['pagebreak'] = True

                prevData = key

        if self.statSkipped != []:
            stat = {}
            stat['title'] = u'Пропущены'
            stat['insurer'] = '-'
            stat['program'] = '-'
            stat['work'] = '-'
            stat['begDate'] = QDate()
            stat['endDate'] = QDate()
            stat['pagebreak'] = True
            stat['dataList'] = self.statSkipped
            tablesList.append(stat)

        params = {}
        params['fileName'] = fileName
        params['stat'] = tablesList

        templateId = forceRef(QtGui.qApp.db.translate('rbPrintTemplate', 'code', 'r77i1','id'))

        if not templateId:
            QtGui.QMessageBox.critical( self,
                                       u'Ошибка формирования отчета',
                                       u'В справочнике шаблонов отчетов не найден шаблон с кодом `r77i1`.',
                                       QtGui.QMessageBox.Close)
            return

        applyTemplate(self, templateId, params)
        return

# *****************************************************************************************

    attachFieldNameType = (('DROSCH',  'DROSCH     D   8   0'),
                                      ('DNACSTR', 'DNACSTR    D   8   0'),
                                      ('DKONSTR', 'DKONSTR    D   8   0'))

    detachFieldNameType = (('DROSCH',  'DROSCH     D   8   0'),
                                        ('DATVV',  'DATVV      D   8   0'),
                                        ('DKONSTR', 'DKONSTR    D   8   0'))

    def checkFormat(self,  dbf,  fields):

        for (name,  type) in fields:
                if not (name in dbf.fieldNames and str(dbf.header.fields[dbf.header.mapNameToIndex[name]]) == type):
                    QtGui.QMessageBox.critical(
                        self, u'Импорт невозможен',
                        u'Неправильный формат данных:\n' \
                        u'поле `%s`\nимеет тип `%s`,\nожидается `%s`' % (name,
                            str(dbf.header.fields[dbf.header.mapNameToIndex[name]]),  type),
                        QtGui.QMessageBox.Close)
                    return False

        return True

# *****************************************************************************************

    def detectFormat(self,  dbf):
        v1FieldNames = ('FAM','IMJA','OTSCH','POL','DROSCH','NN','KGOS','SPOLIS',
                                'NPOLIS','PROGR','NCOM','DNACSTR','DKONSTR','ADRESS',
                                'MESTRAB')
        v2FieldNames = ('SPOLIS', 'NPOLIS', 'FAM', 'IMJA', 'OTSCH', 'POL', 'DROSCH',
                                'N8', 'N9', 'KGOS', 'DNACSTR', 'DKONSTR', 'MESTRAB', 'NCOM',
                                'ADRFULL')
        result = None

        for (list, format) in ((v1FieldNames,  1), (v2FieldNames,  2)):
            for name in list:
                if name in dbf.fieldNames:
                    result = format
                else:
                    result = None
                    break

            if result:
                break

        return result

# *****************************************************************************************

    def processService(self, row):
        useOnlyCode = self._parent.chkUseOnlyCodeForServiceSearch.isChecked()
        code = forceString(row['SCODE'])
        name = forceString(row['NAME'].strip())

        serviceId = self.findServiceByCode(code) if useOnlyCode else self.findServiceByCodeAndName(code, name)

        if serviceId:
            if self._parent.chkUpdateServiceName.isChecked():
                self.updateService(serviceId, code, name,  None,  None)
            else:
                self.log(u'Пропуск существующей услуги, код `%s`' % code)
        else:
            serviceId = self.addService(code,  name,  code,  None,  None)

        if self.copyToActionType:
            actionTypeId = self.findActionTypeByCodeAndName(code,
                                                            None if useOnlyCode else name,
                                                            self.actionTypeParentGroupIdList,
                                                            not self.recoverDeletedActionTypes)

            if actionTypeId:
                self.updateActionType(actionTypeId,  self.actionTypeParentGroupId,  serviceId)
            else:
                self.addActionType(code, name,  self.actionTypeParentGroupId,  serviceId)

# *****************************************************************************************

    def processArea(self, record):
        code = forceInt(record['KGOR'])
        name = forceString(record['GOROD'])
        table = QtGui.qApp.db.table('kladr.KLADR')
        record = QtGui.qApp.db.getRecordEx(table, 'CODE',
            [table['NAME'].eq(name), table['CODE'].like('510%'),
            ], 'CODE')

        KLADRCode = forceString(record.value(0)) if record else None
        if KLADRCode:
            self.mapAreaToKLADR[code] = KLADRCode
            self.updateKLADRInfisCode(KLADRCode,  code)
        else:
            self.cityMap[code] = name
            self.log(u'! Населённый пункт "%s" (код "%s")'
                        u' не найден в КЛАДРе.' % (name,  code),  True)

# *****************************************************************************************

    def processStreet(self, record):
        codeEx = record['CODUL']
        code = forceInt(codeEx) if codeEx else None
        name = forceString(record['ULIC'])
        self.streetMap[code] = name

# *****************************************************************************************

    def processInsurer(self, row):
        npp = row['NN']
        orgId = self.getInsurerId(npp)
        if orgId:
            self.updateInsurer(orgId, row['NAZV'].strip(), row['KRT'].strip(), None)
        else:
            orgId = self.addInsurer(npp, row['NAZV'].strip(), row['KRT'].strip(), None)
        self.insurerCache[npp] = orgId


    def addInsurer(self, npp, name, dir, OKPO):
        table = QtGui.qApp.db.table('Organisation')
        record = table.newRecord()
# OKONH, ?
        record.setValue('fullName', toVariant(name))
        record.setValue('shortName', toVariant(name))
        record.setValue('title', toVariant(name))
        record.setValue('infisCode', toVariant(dir))
        record.setValue('OKPO', toVariant(OKPO))
        record.setValue('isInsurer', toVariant(1))
        record.setValue('notes', toVariant(self.getInsurerNotes(npp)))
        orgId = QtGui.qApp.db.insertRecord(table, record)
        return orgId


    def _getInsurerId(self, insurerCode):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecordEx(table, 'id', [table['deleted'].eq(0),
             QtGui.qApp.db.joinOr([table['notes'].eq(self.getInsurerNotes(insurerCode)),
                                           table['infisCode'].eq(insurerCode)]),
                                            table['isInsurer'].eq(1)
                                                        ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return 0


    def getInsurerId(self, insurerCode):
        if insurerCode in self.insurerCache:
            return self.insurerCache[insurerCode]
        else:
            result = self._getInsurerId(insurerCode)
            self.insurerCache[insurerCode] = result
            return result


    def getInsurerNotes(self, npp):
        return u'СМО # %s' % npp


    def updateInsurer(self, orgId, name, dir, OKPO):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecord(table, '*', orgId)
        record.setValue('fullName', toVariant(name))
        record.setValue('shortName', toVariant(name))
        record.setValue('title', toVariant(name))
        record.setValue('infisCode', toVariant(dir))
        record.setValue('OKPO', toVariant(OKPO))
        QtGui.qApp.db.updateRecord(table, record)


    def _getOKFSId(self, OKFS):
        return forceRef(QtGui.qApp.db.translate('rbOKFS', 'code', OKFS, 'id'))


    def getOKFSId(self, OKFS):
        if OKFS in self.OKFSCache:
            return self.OKFSCache[OKFS]
        else:
            result = self._getOKFSId(OKFS)
            self.OKFSCache[OKFS] = result
            return result

# *****************************************************************************************

    def processInsuranceType(self,  row):
        code = u'п'+ forceString(row['KGOS'])
        name = row['NGOS'].strip()

        attachTypeId = self.findAttachTypeByCode(code)

        if attachTypeId:
            self.updateAttachType(attachTypeId,  name)
            self.log(u'<b><font color=green>Обновлен</b>:'
                    u'тип прикрепления "%s". ' % name,  True)
            self.attachCache[row['KGOS']] = attachTypeId
        else:
            self.attachCache[row['KGOS']] = self.addAttachType(code,  name)
            self.log(u'<b><font color=blue>Добавлен</b>:'
                    u'тип прикрепления ("%s") "%s". ' % (code,  name),  True)


    def findAttachTypeByCode(self,  code):
        table = QtGui.qApp.db.table('rbAttachType')
        record = QtGui.qApp.db.getRecordEx(table, 'id', [
                                                          table['code'].eq(code),
                                                         ], 'id')

        return forceRef(record.value(0)) if record else None


    def updateAttachType(self,  id,  name):
        table = QtGui.qApp.db.table('rbAttachType')
        record = QtGui.qApp.db.getRecord(table, '*', id)
        record.setValue('name', toVariant(name))
        record.setValue('temporary', toVariant(1))
        record.setValue('outcome', toVariant(0))
        record.setValue('finance_id', toVariant(self.financeId))
        QtGui.qApp.db.updateRecord(table, record)


    def addAttachType(self, code, name):
        table = QtGui.qApp.db.table('rbAttachType')
        record = table.newRecord()
        record.setValue('code', toVariant(code))
        record.setValue('name', toVariant(name))
        record.setValue('temporary', toVariant(1))
        record.setValue('outcome', toVariant(0))
        record.setValue('finance_id', toVariant(self.financeId))
        return QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def updateKLADRInfisCode(self,  KLADRCode,  infisCode):
        table = QtGui.qApp.db.table('kladr.KLADR')
        table.setIdFieldName('CODE')
        record = QtGui.qApp.db.getRecordEx(table, 'CODE, infis',
            [table['CODE'].eq(KLADRCode),], 'CODE')
        record.setValue('infis', toVariant(infisCode))
        QtGui.qApp.db.updateRecord(table, record)

# *****************************************************************************************

    def findClientByNPP(self, npp):
        table = QtGui.qApp.db.table('ClientIdentification')
        record = QtGui.qApp.db.getRecordEx(table, 'client_id', [table['deleted'].eq(0),
                                                          table['accountingSystem_id'].eq(self.accountingSystemId),
                                                          table['identifier'].eq(npp)
                                                         ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None

# *****************************************************************************************

    def findClientByNameSexAndBirthDate(self, lastName, firstName, patrName, sex, birthDate):
        table = QtGui.qApp.db.table('Client')
        filter = [table['deleted'].eq(0),
                    table['lastName'].eq(lastName),
                    table['firstName'].eq(firstName),
                    table['patrName'].eq(patrName),
                    table['birthDate'].eq(birthDate)]
        if sex != 0:
            filter.append(table['sex'].eq(sex))

        record = QtGui.qApp.db.getRecordEx(table, 'id',  filter, 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None
# *****************************************************************************************

    def findClientByNameSexBirthDateAndInsurerId(self, lastName, firstName, patrName, sex, birthDate, insurerId):
        table = QtGui.qApp.db.table('Client')
        tablePolicy = QtGui.qApp.db.table('ClientPolicy')
        queryTable = table
        queryTable = queryTable.leftJoin(tablePolicy, table['id'].eq(tablePolicy['client_id']))
        filter = [table['deleted'].eq(0),
                    table['lastName'].eq(lastName),
                    table['firstName'].eq(firstName),
                    table['patrName'].eq(patrName),
                    table['birthDate'].eq(birthDate),
                    tablePolicy['insurer_id'].eq(insurerId),
                    tablePolicy['deleted'].eq(0)]
        if sex != 0:
            filter.append(table['sex'].eq(sex))

        return QtGui.qApp.db.getIdList(queryTable, table['id'],  filter, 'id')

# *****************************************************************************************

    def findServiceByCodeAndName(self, code,  name):
        table = QtGui.qApp.db.table('rbService')
        record = QtGui.qApp.db.getRecordEx(table, 'id',  [table['code'].eq(code),
                                                          table['name'].eq(name),
                                                         ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None


    def findServiceByCode(self, code):
        table = QtGui.qApp.db.table('rbService')
        record = QtGui.qApp.db.getRecordEx(table, 'id',  [table['code'].eq(code)], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None

# *****************************************************************************************

    def findSpecialityByName(self, name):
        key = (name)
        result = self.mapSpecialityCodeNameToId.get(key)

        if not result:
            table = QtGui.qApp.db.table('rbSpeciality')
            record = QtGui.qApp.db.getRecordEx(table, 'id',  [
                                                        table['name'].eq(name),
                                                         ], 'id')
            if record:
                result = forceRef(record.value(0))
                self.mapSpecialityCodeNameToId[key] = result

        return result

# *****************************************************************************************

    def findActionTypeByCodeAndName(self, code, name, groupIdList, ignoreDeleted):
        key = (code,  name,  groupIdList[0] if groupIdList != [] else None)
        result = self.mapActionTypeCodeNameToId.get(key)

        if not result:
            table = QtGui.qApp.db.table('ActionType')

            filter = [table['code'].eq(code),
                        table['class'].eq('3')]

            if name:
                filter.append(table['name'].eq(name))

            if groupIdList and groupIdList != []:
                filter.append(table['group_id'].inlist(groupIdList))

            if ignoreDeleted:
                filter.append(table['deleted'].eq(0))

            record = QtGui.qApp.db.getRecordEx(table, 'id',  filter, 'id')
            if record:
                result = forceRef(record.value(0))
                self.mapActionTypeCodeNameToId[key] = result

        return result

# *****************************************************************************************

    def findClientBySNILS(self, SNILS):
        table = QtGui.qApp.db.table('Client')
        record = QtGui.qApp.db.getRecordEx(table, 'id',  [table['deleted'].eq(0),
                                                          table['SNILS'].eq(SNILS),
                                                         ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None

# *****************************************************************************************

    def addClient(self, npp, lastName, firstName, patrName, birthDate, sex,  SNILS):
        table = QtGui.qApp.db.table('Client')
        record = table.newRecord()
        record.setValue('lastName',  toVariant(lastName))
        record.setValue('firstName', toVariant(firstName))
        record.setValue('patrName',  toVariant(patrName))
        record.setValue('birthDate', toVariant(birthDate))
        record.setValue('sex',       toVariant(sex))
        record.setValue('SNILS',     toVariant(SNILS))
        clientId = QtGui.qApp.db.insertRecord(table, record)
        self.addClientIdentification(clientId, npp)
        return clientId

# *****************************************************************************************

    def updateClient(self, clientId, lastName, firstName, patrName, birthDate, sex,  SNILS):
        table = QtGui.qApp.db.table('Client')
        record = QtGui.qApp.db.getRecord(table, '*', clientId)
        if lastName:
            record.setValue('lastName', toVariant(lastName))
        if firstName:
            record.setValue('firstName', toVariant(firstName))
        if patrName:
            record.setValue('patrName', toVariant(patrName))
        if birthDate:
            record.setValue('birthDate', toVariant(birthDate))
        if sex:
            record.setValue('sex', toVariant(sex))
        if SNILS:
            record.setValue('SNILS', toVariant(SNILS))
        QtGui.qApp.db.updateRecord(table, record)

# *****************************************************************************************

    def addClientDocument(self, clientId, documentTypeId, serial, number):
        if documentTypeId:
            table = QtGui.qApp.db.table('ClientDocument')
            record = table.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('documentType_id', toVariant(documentTypeId))
            record.setValue('serial', toVariant(serial))
            record.setValue('number', toVariant(number))
            QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def updateClientDocument(self, clientId, documentTypeId, serial, number):
        if documentTypeId:
            table = QtGui.qApp.db.table('ClientDocument')
            record = QtGui.qApp.db.getRecordEx(table, '*', [table['client_id'].eq(clientId),
                                                      table['serial'].eq(serial),
                                                      table['number'].eq(number)], 'id')
            if not record:
                self.addClientDocument(clientId, documentTypeId, serial, number)

# *****************************************************************************************

    def addClientPolicy(self, clientId, insurerId, serial, number, begDate, endDate,  note,  policyTypeId):
        table = QtGui.qApp.db.table('ClientPolicy')
        record = table.newRecord()
        record.setValue('client_id', toVariant(clientId))
        record.setValue('insurer_id', toVariant(insurerId))
        record.setValue('policyType_id', toVariant(policyTypeId))
        record.setValue('serial', toVariant(serial))
        record.setValue('number', toVariant(number))
        record.setValue('begDate', toVariant(begDate))
        record.setValue('endDate', toVariant(endDate))
        record.setValue('note',  toVariant(note))
        QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def updateClientPolicy(self, clientId, insurerId, serial, number, begDate, endDate,  note,  policyTypeId):
        table = QtGui.qApp.db.table('ClientPolicy')
        record = selectLatestRecord('ClientPolicy', clientId,
            '(Tmp.`policyType_id`=%d)' % policyTypeId)

        if record and (
            (forceRef(record.value('insurer_id')) == insurerId) and
            (forceString(record.value('serial')) == serial) and
            (forceInt(record.value('number')) == number)
            ):
            if forceDate(record.value('begDate')) != begDate or forceDate(record.value('endDate')) != endDate:
                record.setValue('begDate', toVariant(begDate))
                record.setValue('endDate', toVariant(endDate))
                QtGui.qApp.db.updateRecord(table, record)
            if forceString(record.value('note')) != note:
                record.setValue('note', toVariant(note))
                QtGui.qApp.db.updateRecord(table, record)
        else:
            self.addClientPolicy(clientId, insurerId, serial, number, begDate, endDate,  note,  policyTypeId)

# *****************************************************************************************
    regionNameToKLADRCodeCache = {}

    def getKLADRRegionCode(self, region):
        code = self.regionNameToKLADRCodeCache.get(region)

        if not code:
            code = forceString(QtGui.qApp.db.translate('kladr.KLADR', 'NAME',  region,  'CODE'))
            self.regionNameToKLADRCodeCache[region] = code

        return code

# *****************************************************************************************
    cityNameToKLADRCodeCache = {}

    def getKLADRCityCode(self, city, KLADRCode):
        key = (city,  KLADRCode)
        code = self.cityNameToKLADRCodeCache.get(key)

        if not code:
            table = QtGui.qApp.db.table('kladr.KLADR')
            filter = [table['NAME'].eq(city)]

            if KLADRCode:
                filter.append(table['CODE'].like(KLADRCode[:2]+'%'))

            record = QtGui.qApp.db.getRecordEx(table, 'CODE', filter,  'CODE')

            if record:
                code = forceString(record.value(0))
                self.cityNameToKLADRCodeCache[key] = code

        return code


    affixes = [u'б-р', u'бульвар', u'ул', u'улица', u'пер', u'пр', u'проспект', u'пр-т', u'наб', u'ш', u'шоссе']

# *****************************************************************************************

    streetNameToKLADRStreetCodeCache = {}

    def getKLADRStreetCode(self, KLADRCode, street):
        key = (KLADRCode,  street)
        result = self.streetNameToKLADRStreetCodeCache.get(key)

        if not result:
            srcStreet = street

            if KLADRCode:
                if street == '' or street == '-':
                    result = KLADRCode
                else:
                    if street.count(','): # нас. пункт, улица, д, кв.
                        street = trim(street.split(',')[1])
                    street = re.sub(ur'(,?д\.?[ 0-9]+.*$)', '', street) # удаляем приписанные номера домов в виде д.12
                    street = re.sub(r'[0-9-/ ]*$', '', street) # удаляем приписанные номера домов в виде 12/3-1
                    loCaseStreet = street.lower()
                    for affix in self.affixes:
                        if loCaseStreet.startswith(affix+'.'):
                            street = trim(street[len(affix)+1:])
                            break
                        if loCaseStreet.startswith(affix+' '):
                            street = trim(street[len(affix)+1:])
                            break
                        elif loCaseStreet.endswith(' '+affix):
                            street = trim(street[:-len(affix)])
                            break
                        elif loCaseStreet.endswith(' '+affix+'.'):
                            street = trim(street[:-len(affix)-1])
                            break

                    table = QtGui.qApp.db.table('kladr.STREET')
                    record = QtGui.qApp.db.getRecordEx(table, 'CODE', [table['NAME'].eq(street),
                                                                         table['CODE'].like(KLADRCode[:-2]+'%'),
                                                                     ], 'CODE')
                    if record:
                        result = forceString(record.value('CODE'))

                self.streetNameToKLADRStreetCodeCache[key] = result

        return result

# *****************************************************************************************

    def getAddressHouseId(self, placeCode, street, number, corpus):
        if not placeCode or not street:
            return None

        KLADRCode = placeCode
        KLADRStreetCode = self.getKLADRStreetCode(KLADRCode, street)
        table = QtGui.qApp.db.table('AddressHouse')
        record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                        table['KLADRCode'].eq(KLADRCode),
                                                        table['KLADRStreetCode'].eq(KLADRStreetCode),
                                                        table['number'].eq(number),
                                                        table['corpus'].eq(corpus)], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            record = table.newRecord()
            record.setValue('KLADRCode', toVariant(KLADRCode))
            record.setValue('KLADRStreetCode', toVariant(KLADRStreetCode))
            record.setValue('number', toVariant(number))
            record.setValue('corpus', toVariant(corpus))
            return QtGui.qApp.db.insertRecord(table, record)


# *****************************************************************************************

    def getAddressId(self, placeCode, street, number, corpus, flat, address):
        houseId = self.getAddressHouseId(placeCode, street, number, corpus)
        if houseId:
            table = QtGui.qApp.db.table('Address')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                            table['house_id'].eq(houseId),
                                                            table['flat'].eq(flat)], 'id')
            if record:
                return forceRef(record.value(0))
            else:
                record = table.newRecord()
                record.setValue('house_id', toVariant(houseId))
                record.setValue('flat', toVariant(flat))
                return QtGui.qApp.db.insertRecord(table, record)
        else:
            return None

# *****************************************************************************************

    def ensureClientAddress(self, clientId, addressType, addressId, freeInput):
        table = QtGui.qApp.db.table('ClientAddress')
        record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                  table['client_id'].eq(clientId),
                                                  table['type'].eq(addressType),
                                                  table['address_id'].eq(addressId),
                                                  table['freeInput'].eq(freeInput),
                                                 ], 'id')
        if not record:
            record = table.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('type',      toVariant(addressType))
            record.setValue('address_id',toVariant(addressId))
            record.setValue('freeInput', toVariant(freeInput))
            QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def addClientAddress(self, clientId, placeCode, street, number, corpus, flat, freeInput):
        addressId = self.getAddressId(placeCode, street, number, corpus, flat, freeInput)
        self.ensureClientAddress(clientId, 0, addressId, freeInput)
        self.ensureClientAddress(clientId, 1, addressId, freeInput)

# *****************************************************************************************

    def updateClientAddress(self, clientId, placeCode, street, number, corpus, flat, freeInput):
        addressId = self.getAddressId(placeCode, street, number, corpus, flat, freeInput)
        self.ensureClientAddress(clientId, 0, addressId, freeInput)
        self.ensureClientAddress(clientId, 1, addressId, freeInput)

# *****************************************************************************************

    def ensureClientAttach(self, clientId, attachTypeId, orgId,  begDate,  endDate):
        if not orgId or not attachTypeId:
            return

        checkBegDate = begDate if begDate else QDate.currentDate()
        checkEndDate = endDate if endDate else QDate.currentDate()

        record = selectLatestRecord('ClientAttach', clientId,
            '(Tmp.`attachType_id`=%d) AND \
         (Tmp.`LPU_id`=%d) AND \
         (Tmp.`begDate`=\'%s\') AND \
         (Tmp.`endDate`=\'%s\')' % \
            (attachTypeId, orgId, checkBegDate.toString(QtCore.Qt.ISODate),
             checkEndDate.toString(QtCore.Qt.ISODate)))

        if not record:
            table = QtGui.qApp.db.table('ClientAttach')
            record = table.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('attachType_id',  toVariant(attachTypeId))
            record.setValue('LPU_id',toVariant(orgId))
            if begDate:
                record.setValue('begDate',toVariant(begDate))
            if endDate:
                record.setValue('endDate', toVariant(endDate))
            QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def closeLastAttach(self, clientId, attachTypeId, orgId,  endDate):
        if not orgId or not attachTypeId:
            return

        record = selectLatestRecord('ClientAttach', clientId,
            '(Tmp.`attachType_id`=%d) AND \
         (Tmp.`LPU_id`=%d)' % (attachTypeId, orgId))

        if record:
            table = QtGui.qApp.db.table('ClientAttach')
            record.setValue('endDate', toVariant(endDate))
            QtGui.qApp.db.updateRecord(table, record)

# *****************************************************************************************

    def deleteClientClosedAttach(self, clientId, orgId,  begDate,  endDate):
        if not orgId:

            checkBegDate = begDate if begDate else QDate.currentDate()
            checkEndDate = endDate if endDate else QDate.currentDate()

            table = QtGui.qApp.db.table('ClientAttach')
            record = QtGui.qApp.db.getRecordEx(table, '*',[table['deleted'].eq(0),
                                                      table['client_id'].eq(clientId),
                                                      table['attachType_id'].eq(self.attachClose),
                                                      table['LPU_id'].eq(orgId),
                QtGui.qApp.db.joinOr([table['begDate'].isNull(), table['begDate'].le(checkBegDate)]),
                QtGui.qApp.db.joinOr([table['endDate'].isNull(), table['endDate'].ge(checkEndDate)]),
                                                     ], 'id')
            if record:
                record.setValue('deleted', toVariant(1))
                QtGui.qApp.db.updateRecord(table, record)
                return True

        return False

# *****************************************************************************************

    def addClientAttach(self,  clientId,  orgId,  attachTypeId,  begDate=None,  endDate=None):
        self.ensureClientAttach(clientId,  attachTypeId,  orgId,  begDate,  endDate)

# *****************************************************************************************

    def updateClientAttach(self,  clientId,  orgId,  attachTypeId,  begDate=None,  endDate=None):
        self.ensureClientAttach(clientId,  attachTypeId,  orgId,  begDate,  endDate)

# *****************************************************************************************
    lastFreeClientIdentificator = 1000001

    def generateClientIdentificator(self):
        while self.findClientByNPP(forceString(self.lastFreeClientIdentificator)):
            self.lastFreeClientIdentificator += 1

        self.log(u'Сгенерирован новый идентификатор пациента %d' % self.lastFreeClientIdentificator)
        return self.lastFreeClientIdentificator

# *****************************************************************************************

    def addClientIdentification(self, clientId, npp):
        table = QtGui.qApp.db.table('ClientIdentification')
        record = table.newRecord()
        record.setValue('client_id', toVariant(clientId))
        record.setValue('accountingSystem_id', toVariant(self.accountingSystemId))
        record.setValue('identifier', toVariant(npp))
        record.setValue('checkDate', toVariant(QDate.currentDate()))
        clientId = QtGui.qApp.db.insertRecord(table, record)
#        self.addChange('ClientIdentification', 1)

# *****************************************************************************************

    def updateClientIdentification(self, clientId, npp):
        table = QtGui.qApp.db.table('ClientIdentification')
        record = QtGui.qApp.db.getRecordEx(table, '*',   [table['deleted'].eq(0),
                                                          table['accountingSystem_id'].eq(self.accountingSystemId),
                                                          table['client_id'].eq(clientId),
                                                         ], 'id')
        if record:
            record.setValue('identifier', toVariant(npp))
            record.setValue('checkDate', toVariant(QDate.currentDate()))
            QtGui.qApp.db.updateRecord(table, record)
#            self.addChange('ClientIdentification', 2)
        else:
            self.addClientIdentification(clientId, npp)

# *****************************************************************************************

    def getClientIdentification(self, clientId):
        table = QtGui.qApp.db.table('ClientIdentification')
        record = QtGui.qApp.db.getRecordEx(table, 'identifier',   [table['deleted'].eq(0),
                                                          table['accountingSystem_id'].eq(self.accountingSystemId),
                                                          table['client_id'].eq(clientId),
                                                         ], 'id')

        return forceString(record.value(0)) if record else None

# *****************************************************************************************

    def getOrganisationId(self, INN, KPP):
        if INN and KPP:
            table = QtGui.qApp.db.table('Organisation')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                      table['INN'].eq(INN),
                                                      table['KPP'].eq(KPP)], 'id')
            if record:
                return forceRef(record.value(0))
            elif self.addDummyOrganisations:
                record = table.newRecord()
                name = u'ИНН: %s; КПП: %s' % (INN, KPP)
                record.setValue('fullName', toVariant(name))
                record.setValue('shortName', toVariant(name))
                record.setValue('INN', toVariant(INN))
                record.setValue('KPP', toVariant(KPP))
                return QtGui.qApp.db.insertRecord(table, record)
        return None

# *****************************************************************************************

    def addClientWork(self, clientId, organisationId,  organisationName):
        if clientId and (organisationId or organisationName):
            table = QtGui.qApp.db.table('ClientWork')
            record = table.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('org_id', toVariant(organisationId))
            if not organisationId:
                record.setValue('freeInput', toVariant(organisationName))
            QtGui.qApp.db.insertRecord(table, record)


    def updateClientWork(self, clientId, organisationId,  organisationName):
        record = selectLatestRecord('ClientWork', clientId)

        if not record:
            self.addClientWork(clientId, organisationId,  organisationName)
        else:
            if organisationId:
                oldOrgId = forceRef(record.value('org_id'))

                if oldOrgId != organisationId:
                    self.addClientWork(clientId, organisationId, organisationName)
            elif organisationName:
                oldOrgName = forceString(record.value('freeInput'))

                if oldOrgName != organisationName:
                    self.addClientWork(clientId, organisationId, organisationName)

# *****************************************************************************************

    def addClientPhone(self, clientId, phone):
        if phone and phone != '0':
            table = QtGui.qApp.db.table('ClientContact')
            record = table.newRecord()
            record.setValue('client_id', toVariant(clientId))
            record.setValue('contactType_id', toVariant(self.contactTypeId))
            record.setValue('contact', toVariant(phone))
            QtGui.qApp.db.insertRecord(table, record)


    def updateClientPhone(self, clientId, phone):
        if phone and phone != '0':
            table = QtGui.qApp.db.table('ClientContact')
            record = QtGui.qApp.db.getRecordEx(table, 'id',[table['deleted'].eq(0),
                                                      table['client_id'].eq(clientId),
                                                      table['contactType_id'].eq(self.contactTypeId),
                                                      table['contact'].eq(phone)], 'id')
            if not record:
                self.addClientPhone(clientId, phone)

# *****************************************************************************************

    def evalSexByName(self, tableName, name):
        return forceInt(QtGui.qApp.db.translate(tableName, 'name', name, 'sex'))


    def checkBirthDate(self, birthDate, lastName, firstName, patrName):
        if birthDate:
            if birthDate>QDate.currentDate():
                self.log(u'<b><font color=red>ОШИБКА</b>:'
                    u'Дата рождения клиента "%s %s %s" находится в будущем (`%s`). Пропускаем.' %\
                    (lastName, firstName, patrName,  birthDate.toString('dd.MM.yyyy')),  True)
                return False

            if QDate.currentDate().year() - birthDate.year() > 130:
                self.log(u'<b><font color=red>ОШИБКА</b>:'
                    u'Возраст клиента "%s %s %s" больше 130 лет!(д.р. `%s`). Пропускаем.' %\
                    (lastName, firstName, patrName,  birthDate.toString('dd.MM.yyyy')),  True)
                return False

        return True


# *****************************************************************************************

    def processOMSClient(self, record):
        lastName  = nameCase(forceString(record['FAM'].strip()))
        firstName = nameCase(forceString(record['IM'].strip()))
        patrName  = nameCase(forceString(record['OT'].strip()))
        birthDate = QDate(record['BERTDAY']) if record['BERTDAY'] else None

        if not self.checkBirthDate(birthDate, lastName, firstName, patrName):
            self.nSkipped +=1
            return

        npp = forceString(record['KARTA'])
        policySerial = forceString(record['SERIYA_POL']).strip().replace(' ', '')
        policyNumber = forceString(record['NOMER_POLI']) if record['NOMER_POLI'] else None
        insurerName = forceString(record['STRAH.COMP']).strip()
        policyNote = insurerName

        if insurerName.upper().strip() in (u'МЕДИЦИНСКАЯ КАРТА',  u'ПАСПОРТ'):
            policyInsurer = self.personalInsurerId
        else:
            policyInsurer = self.insurerCache.get(insurerName)

        region = forceString(record['RAION'])
        city = forceString(record['CITY'])
        street = forceString(record['STREET'])
        house = forceString(record['DOM'])
        corpus = forceString(record['KOR'])
        building = forceString(record['STR'])
        flat = forceString(record['KV'])
        sex = self.evalSexByName('rdFirstName',  firstName)

        if self.convertAddressToKLADR:
            KLADRCode = self.getKLADRRegionCode(region)
            KLADRCode = self.getKLADRCityCode(city,  KLADRCode)

            if not KLADRCode:
                self.log(u'<b><font color=red>ВНИМАНИЕ</b>:'
                    u'КЛАДР код для клиента "%s %s %s" не определен (`%s`, `%s`).' %\
                    (lastName, firstName, patrName,  region,  city),  True)

            KLADRStreetCode = self.getKLADRStreetCode(KLADRCode,  street)
        else:
            KLADRCode = None
            KLADRStreetCode = None

        freeInput = []
        for x in (region,  city,  street):
            if x:
                freeInput.append(x)

        if house:
            freeInput.append(u'д. ' + house)
        if corpus:
            freeInput.append(u'корп. ' + corpus)
        if building:
            freeInput.append(u'стр. '+ building)
        if flat:
            freeInput.append(u'кв. ' + flat)

        freeInputAddress = u' ,'.join([et for et in freeInput])
        corpus = building if building else corpus

        if sex == 0:
            sex = self.evalSexByName('rdPatrName',  patrName)

            if sex == 0:
                self.log(u'<b><font color=orange>ВНИМАНИЕ</b>:'\
                            u'Клиент "%s %s %s", невозможно определить пол.' % \
                            (lastName, firstName, patrName),  True)

        if not policyInsurer:
            self.log(u'<b><font color=red>ВНИМАНИЕ</b>:'
                    u'СМО `%s` для клиента "%s %s %s" не импортирован.' %\
                    (forceString(record['STRAH.COMP']),  lastName, firstName, patrName),  True)

        clientId = self.findClientByNPP(npp)
        if not clientId:
            clientId = self.findClientByNameSexAndBirthDate(lastName, firstName, patrName, sex, birthDate)

        if self.strictNPP and clientId:
            anotherNPP = self.getClientIdentification(clientId)

            if anotherNPP and anotherNPP != npp:
                self.log(u'<b><font color=orange>ВНИМАНИЕ</b>:'\
                         u'Клиент "%s %s %s", код "%s" - уже есть в базе.' \
                         u' Добавляем двойника с кодом "%s".' % \
                         (lastName, firstName, patrName, anotherNPP,  npp),  True)
                clientId = None
        elif clientId and self.checkUniqueNPP:
            anotherClientId = self.findClientByNameSexAndBirthDate(lastName, firstName, patrName, sex, birthDate)
            if anotherClientId and (anotherClientId != clientId):
                msgText = u'<b><font color=red>ВНИМАНИЕ</b>:'\
                    u'Клиент "%s %s %s", код "%s" - не уникальный.<br>'\
                    u'Идентификатор поиска по коду: "%d"<br>'\
                    u'Идентификатор поиска по ФИО, полу и дате рождения: "%d"' % \
                    (lastName, firstName, patrName,  npp,  clientId,  anotherClientId)
                self.log(msgText,  True)
                # спросим пользователя, следует ли заменять информацию о клиенте
                answer = QtGui.QMessageBox.question(self._parent,
                            u'Проверка на уникальность',
                            u'%s<br><br>Варианты действий:<br>'\
                            u'Да - обновить данные записи, найденной по коду клиента<br>'\
                            u'Нет - обновить данные записи,'\
                            u' найденной по ФИО, полу, дате рождения<br>'\
                            u'Прервать - не обрабатывать текущую запись' % msgText ,
                            QtGui.QMessageBox.No|QtGui.QMessageBox.Yes|QtGui.QMessageBox.Abort,
                            QtGui.QMessageBox.Abort)
                if answer == QtGui.QMessageBox.Abort:
                    self.nSkipped += 1
                    return
                elif answer == QtGui.QMessageBox.No:
                    clientId = anotherClientId

        if clientId:
            self.log(u'<b><font color=brown>Обновляем</b>:'
                u' %s %s %s, код "%s"' % (lastName, firstName, patrName,  npp))
            self.updateClientIdentification(clientId, npp)
            self.updateClient(clientId, lastName, firstName, patrName, birthDate, sex, None)
            self.updateClientPolicy(clientId, policyInsurer, policySerial, policyNumber,
                                    None, None,  policyNote,  self.policyTypeOMSId)
            self.updateClientAddress(clientId, KLADRCode, KLADRStreetCode, house, corpus, flat, freeInputAddress)
            self.updateClientAttach(clientId, self.currentOrgId, self.attachTypeId, None,  None)
            self.nUpdated += 1
        else:
            self.log(u'<b><font color=green>Добавляем</b>:'
                u' %s %s %s, код "%s"' % (lastName, firstName, patrName,  npp))
            clientId = self.addClient(npp,  lastName, firstName, patrName, birthDate, sex, None)
            self.addClientPolicy(clientId, policyInsurer, policySerial, policyNumber,
                                None, None,  policyNote,  self.policyTypeOMSId)
            self.addClientAddress(clientId, KLADRCode, KLADRStreetCode, house, corpus, flat, freeInputAddress)
            self.addClientAttach(clientId,  self.currentOrgId,  self.attachTypeId, None,  None)
            self.nAdded += 1


# *****************************************************************************************

    def processClient(self, record):
        lastName  = nameCase(forceString(record['FAM'].strip()))
        firstName = nameCase(forceString(record['IMJA'].strip()))
        patrName  = nameCase(forceString(record['OTSCH'].strip()))
        sex       = self.sexMap.get(forceString(record['POL'][0] if record['POL']!= '' else 'X'), 0)
        birthDate = QDate(record['DROSCH']) if record['DROSCH'] else None

        if not self.checkBirthDate(birthDate, lastName, firstName, patrName):
            self.nSkipped +=1
            return

        npp = record['NN']
        attachType = self.attachCache.get(record['KGOS'])
        clientIsClosed = record['NY'].strip() == '**'

        policySerial = forceString(record['SPOLIS']).strip().replace(' ', '')
        policyNumber = forceString(record['NPOLIS']).strip() if record['NPOLIS'] else ''
        policyNote = forceString(record['PROGR']).strip()
        policyInsurer = self.insurerCache.get(record['NCOM'])
        policyBegDate = QDate(record['DNACSTR']) if record['DNACSTR'] else None
        policyEndDate = QDate(record['DKONSTR']) if record['DKONSTR'] else None

        if not policyInsurer:
            self.log(u'<b><font color=red>ВНИМАНИЕ</b>:'
                    u'СМО для клиента "%s %s %s" не импортирован. ' %\
                    (lastName, firstName, patrName),  True)
            self.nSkipped += 1
            return

        if not attachType:
            self.log(u'<b><font color=red>ВНИМАНИЕ</b>:'
                    u'Программа страхования, код "%d" не найдена в кэше. Ищем в БД.' % record['KGOS'])
            attachType = self.findAttachTypeByCode(u'п%d' % record['KGOS'])

            if not attachType:
                self.log(u'<b><font color=red>ВНИМАНИЕ</b>:'
                        u'У клиента "%s %s %s" неизвестная программа страхования (код "%d").' %\
                        (lastName, firstName, patrName,  record['KGOS']),  True)
            else:
                self.attachCache[record['KGOS']] = attachType

        freeInputAddress = record['ADRESS'].strip()

        clientWork = forceString(record['MESTRAB']).strip()
        clientWorkId = None

        if clientWork == u'ФИЗ. ЛИЦО':
            clientWork = None

        if clientWork:
            clientWorkId = self.findOrgByShortName(clientWork)

            if not clientWorkId:
                clientWorkId = self.findOrgByName(clientWork)

        clientId = self.findClientByNPP(npp)
        if not clientId:
            clientId = self.findClientByNameSexAndBirthDate(lastName, firstName, patrName, sex, birthDate)

        if self.strictNPP and clientId:
            anotherNPP = self.getClientIdentification(clientId)

            if anotherNPP and anotherNPP != forceString(npp):
                self.log(u'<b><font color=orange>ВНИМАНИЕ</b>:'\
                         u'Клиент "%s %s %s", код "%s" - уже есть в базе.' \
                         u' Добавляем двойника с кодом "%d".' % \
                         (lastName, firstName, patrName, anotherNPP,  npp),  True)
                clientId = None
        elif clientId and self.checkUniqueNPP:
            anotherClientId = self.findClientByNameSexAndBirthDate(lastName, firstName, patrName, sex, birthDate)
            if anotherClientId and (anotherClientId != clientId):
                msgText = u'<b><font color=red>ВНИМАНИЕ</b>:'\
                    u'Клиент "%s %s %s", код "%d" - не уникальный.<br>'\
                    u'Идентификатор поиска по коду: "%d"<br>'\
                    u'Идентификатор поиска по ФИО, полу и дате рождения: "%d"' % \
                    (lastName, firstName, patrName,  npp,  clientId,  anotherClientId)
                self.log(msgText,  True)
                # спросим пользователя, следует ли заменять информацию о клиенте
                answer = QtGui.QMessageBox.question(self._parent,
                            u'Проверка на уникальность',
                            u'%s<br><br>Варианты действий:<br>'\
                            u'Да - обновить данные записи, найденной по коду клиента<br>'\
                            u'Нет - обновить данные записи,'\
                            u' найденной по ФИО, полу, дате рождения<br>'\
                            u'Прервать - не обрабатывать текущую запись' % msgText ,
                            QtGui.QMessageBox.No|QtGui.QMessageBox.Yes|QtGui.QMessageBox.Abort,
                            QtGui.QMessageBox.Abort)
                if answer == QtGui.QMessageBox.Abort:
                    self.nSkipped += 1
                    return
                elif answer == QtGui.QMessageBox.No:
                    clientId = anotherClientId


        if clientId:
            self.log(u'<b><font color=brown>Обновляем</b>:'
                u' %s %s %s, код "%d"' % (lastName, firstName, patrName,  npp))
            self.updateClientIdentification(clientId, npp)
            self.updateClient(clientId, lastName, firstName, patrName, birthDate, sex, None)
            clientId = self.advancedAttachLogic(clientId, lastName, firstName, patrName, birthDate, sex,
                                        policyInsurer, policySerial, policyNumber,
                                        policyBegDate, policyEndDate,  policyNote,  self.policyTypeId,  attachType, False)
            self.updateClientAddress(clientId, None, None, None, None, None, freeInputAddress)
            self.updateClientWork(clientId, clientWorkId,  clientWork)

            if clientIsClosed:
                if self.canCloseClientAttach:
                    self.closeLastAttach(clientId, self.currentOrgId, attachType, QDate.currentDate())

                    if self.addAttachClosed:
                        self.log(u'<b><font color=blue>Добавляем прикрепление типа "Выбыл"</b>:'
                            u'Клиент "%s %s %s".' % (lastName, firstName, patrName),  True)
                        self.updateClientAttach(clientId, self.currentOrgId, self.attachClose, QDate.currentDate())

                    self.log(u'<b><font color=blue>Прикрепление закрыто с текущей даты</b>:'
                        u'Клиент "%s %s %s".' % (lastName, firstName, patrName),  True)
                else:
                    self.log(u'<b><font color=red>Прикрепление закрыто в источнике, но запрещено'
                        u' пользователем. Информация о закрытии прикрепления не импортирована</b>:'
                        u'Клиент "%s %s %s".' % (lastName, firstName, patrName),  True)
            else:
                if self.deleteClientClosedAttach(clientId, self.currentOrgId,  QDate.currentDate(),  None):
                    self.log(u'<b><font color=green>Прикрепление открыто с текущей даты</b>:'
                    u'Клиент "%s %s %s".' % (lastName, firstName, patrName),  True)

            self.nUpdated += 1
        else:
            self.log(u'<b><font color=green>Добавляем</b>:'
                u' %s %s %s, код "%d"' % (lastName, firstName, patrName,  npp))
            clientId = self.addClient(npp,  lastName, firstName, patrName, birthDate, sex, None)
            self.addClientPolicy(clientId, policyInsurer, policySerial, policyNumber,
                                 policyBegDate, policyEndDate,  policyNote, self.policyTypeId)
            self.addClientAddress(clientId, None, None, None, None, None, freeInputAddress)
            self.addClientWork(clientId, clientWorkId, clientWork)
            self.addClientAttach(clientId,  self.currentOrgId,  attachType, policyBegDate,  policyEndDate)

            if clientIsClosed:
                if self.canCloseClientAttach:
                    self.closeLastAttach(clientId, self.currentOrgId, attachType, QDate.currentDate())

                    if self.addAttachClosed:
                        self.log(u'<b><font color=blue>Добавляем прикрепление типа "Выбыл"</b>:'
                            u'Клиент "%s %s %s".' % (lastName, firstName, patrName),  True)
                        self.addClientAttach(clientId, self.currentOrgId, self.attachClose,  QDate.currentDate())

                    self.log(u'<b><font color=blue>Прикрепление закрыто с текущей даты</b>:'
                        u'Клиент "%s %s %s".' % (lastName, firstName, patrName),  True)
                else:
                    self.log(u'<b><font color=red>Прикрепление закрыто в источнике, но запрещено'
                        u' пользователем. Информация о закрытии прикрепления не импортирована</b>:'
                        u'Клиент "%s %s %s".' % (lastName, firstName, patrName),  True)
            else:
                if self.deleteClientClosedAttach(clientId, self.currentOrgId,  QDate.currentDate(),  None):
                    self.log(u'<b><font color=green>Прикрепление открыто с текущей даты</b>:'
                    u'Клиент "%s %s %s".' % (lastName, firstName, patrName),  True)
                self.nAdded += 1

# *****************************************************************************************

    def processDMSClientV2(self, record):
        lastName  = nameCase(forceString(record['FAM'].strip()))
        firstName = nameCase(forceString(record['IMJA'].strip()))
        patrName  = nameCase(forceString(record['OTSCH'].strip()))
        sex       = self.sexMap.get(forceString(record['POL'][0] if record['POL']!= '' else 'X'), 0)
        birthDate = QDate(record['DROSCH']) if record['DROSCH'] else None
        attachType = self.attachCache.get(record['KGOS'])

        if not self.checkBirthDate(birthDate, lastName, firstName, patrName):
            self.nSkipped +=1
            return

        policySerial = forceString(record['SPOLIS']).strip().replace(' ', '')
        policyNumber = forceString(record['NPOLIS']) if record['NPOLIS'] else ''
        policyInsurer = self.insurerCache.get(record['NCOM'])
        policyBegDate = QDate(record['DNACSTR']) if record['DNACSTR'] else None
        policyEndDate = QDate(record['DKONSTR']) if record['DKONSTR'] else None

        if not policyInsurer:
            self.log(u'<b><font color=red>ВНИМАНИЕ</b>:'
                    u'СМО для клиента "%s %s %s" не импортирован. ' %\
                    (lastName, firstName, patrName),  True)
            self.nSkipped += 1
            return

        if not attachType:
            self.log(u'<b><font color=red>ВНИМАНИЕ</b>:'
                    u'Программа страхования, код "%d" не найдена в кэше. Ищем в БД.' % record['KGOS'])
            attachType = self.findAttachTypeByCode(u'п%d' % record['KGOS'])

            if not attachType:
                self.log(u'<b><font color=red>ВНИМАНИЕ</b>:'
                        u'У клиента "%s %s %s" неизвестная программа страхования (код "%d").' %\
                        (lastName, firstName, patrName,  record['KGOS']),  True)
            else:
                self.attachCache[record['KGOS']] = attachType

        freeInputAddress = forceString(record['N8']).strip()
        phone = forceString(record['N9']).strip()

        clientWork = forceString(record['MESTRAB']).strip()
        clientWorkId = None

        if clientWork == u'ФИЗ. ЛИЦО':
            clientWork = None

        if clientWork:
            clientWorkId = self.findOrgByShortName(clientWork)

            if not clientWorkId:
                clientWorkId = self.findOrgByName(clientWork)

        clientId = self.findClientByNameSexAndBirthDate(lastName, firstName, patrName, sex, birthDate)

        if clientId:
            self.log(u'<b><font color=brown>Обновляем</b>:'
                u' %s %s %s' % (lastName, firstName, patrName))
            self.updateClient(clientId, lastName, firstName, patrName, birthDate, sex, None)
            clientId = self.advancedAttachLogic(clientId, lastName, firstName, patrName, birthDate, sex,
                                        policyInsurer, policySerial, policyNumber,
                                        policyBegDate, policyEndDate,  None,  self.policyTypeId,  attachType)

            #self.updateClientPolicy(clientId, policyInsurer, policySerial, policyNumber,
            #                        policyBegDate, policyEndDate,  None,  self.policyTypeId)
            self.updateClientAddress(clientId, None, None, None, None, None, freeInputAddress)
            self.updateClientWork(clientId, clientWorkId, clientWork)
            #self.updateClientAttach(clientId, self.currentOrgId, attachType, policyBegDate,  policyEndDate)
            self.updateClientPhone(clientId, phone)
            self.nUpdated += 1
        else:
            self.log(u'<b><font color=green>Добавляем</b>:'
                u' %s %s %s' % (lastName, firstName, patrName))
            clientId = self.addClient(None,  lastName, firstName, patrName, birthDate, sex, None)
            self.addClientPolicy(clientId, policyInsurer, policySerial, policyNumber,
                                 policyBegDate, policyEndDate,  None, self.policyTypeId)
            self.addClientAddress(clientId, None, None, None, None, None, freeInputAddress)
            self.addClientWork(clientId, clientWorkId,  clientWork)
            self.addClientAttach(clientId,  self.currentOrgId,  attachType, policyBegDate,  policyEndDate)
            self.addClientPhone(clientId, phone)
            self.nAdded += 1

# *****************************************************************************************

    def processClientAttach(self, row):
        lastName  = nameCase(forceString(row['FAM'].strip()))
        firstName = nameCase(forceString(row['IMJA'].strip()))
        patrName  = nameCase(forceString(row['OTSCH'].strip()))
        sex       = self.sexMap.get(forceString(row['POL'][:1] if row.asDict().has_key('POL') else 'X'), 0)
        birthDate = QDate(row['DROSCH']) if row['DROSCH'] else None
        attachType = self.attachCache.get(row['KGOS']) if row.asDict().has_key('KGOS') else None

        policyBegDate = QDate(row['DNACSTR']) if row['DNACSTR'] else None
        policyEndDate = QDate(row['DKONSTR']) if row['DKONSTR'] else None
        policyInsurer = self.insurerCache.get(row.asDict().get('NCOM'))
        policySerial = forceString(row['SPOLIS']).strip().replace(' ', '')
        policyNumber = forceString(row['NPOLIS']) if row.asDict().has_key('NPOLIS') else ''
        policyNote = forceString(row['PROGR']).strip()
        phone = forceString(row['TELH']) if row.asDict().has_key('TELH') else ''

        if not attachType:
            self.log(u'<b><font color=red>ВНИМАНИЕ</b>: '
                    u'Программа страхования, код "%d" не найдена в кэше. Ищем в БД.' % row['KGOS'])
            attachType = self.findAttachTypeByCode(u'п%d' % row['KGOS'])

            if not attachType:
                self.log(u'<b><font color=red>ВНИМАНИЕ</b>: '
                        u'У клиента "%s %s %s" неизвестная программа страхования (код "%d").' %\
                        (lastName, firstName, patrName,  row['KGOS']),  True)
                self.statSkipped.append([u'неизвестная программа страхования', ('%s %s' % (policySerial, policyNumber)).strip(), lastName, firstName, patrName])
                self.nSkipped +=1
                return
            else:
                self.attachCache[row['KGOS']] = attachType

        if not birthDate:
            self.log(u'<b><font color=red>ВНИМАНИЕ</b>: ' \
                        u'У пациента `%s %s %s` не заполнена дата рождения, пропускаем.' %\
                        (lastName, firstName, patrName),  True)
            self.statSkipped.append([u'не заполнена дата рождения', ('%s %s' % (policySerial, policyNumber)).strip(), lastName, firstName, patrName])
            self.nSkipped += 1
            return

        if not self.checkBirthDate(birthDate, lastName, firstName, patrName):
            self.statSkipped.append([u'неверная дата рождения', ('%s %s' % (policySerial, policyNumber)).strip(), lastName, firstName, patrName])
            self.nSkipped +=1
            return

        if calcAgeInYears(birthDate,  QDate.currentDate()) < 18:
            self.log(u'<b><font color=red>ВНИМАНИЕ</b>: ' \
                        u'Возраст пациента `%s %s %s` менее 18 лет, прикрепление невозможно.' %\
                        (lastName, firstName, patrName),  True)
            self.statSkipped.append([u'менее 18 лет', ('%s %s' % (policySerial, policyNumber)).strip(), lastName, firstName, patrName])
            self.nSkipped += 1
            return

        if not policyInsurer:
            self.log(u'<b><font color=red>ВНИМАНИЕ</b>: ' \
                        u'Пациент `%s %s %s`, неизвестная страховая компания `%s`.' %\
                        (lastName, firstName, patrName,  forceString(row.asDict().get('NCOM', '-'))),  True)
            self.statSkipped.append([u'неизвестная страховая компания', ('%s %s' % (policySerial, policyNumber)).strip(), lastName, firstName, patrName])
            self.nSkipped += 1
            return

        if not policyBegDate:
            self.log(u'<b><font color=red>ВНИМАНИЕ</b>: ' \
                        u'Пациент `%s %s %s`, нет даты начала действия полиса.' %\
                        (lastName, firstName, patrName),  True)
            self.statSkipped.append([u'нет даты начала действия полиса', ('%s %s' % (policySerial, policyNumber)).strip(), lastName, firstName, patrName])
            self.nSkipped += 1
            return

        clientWork = forceString(row['MESTRAB']).strip()
        clientWorkId = None

        if clientWork == u'ФИЗ. ЛИЦО':
            clientWork = None

        if clientWork:
            clientWorkId = self.findOrgByShortName(clientWork)

            if not clientWorkId:
                clientWorkId = self.findOrgByName(clientWork)

        freeInputAddress = row['ADRESS'].strip()

        clientIdList = self.findClientByNameSexBirthDateAndInsurerId(lastName, firstName, patrName, sex, birthDate, policyInsurer)
        id = self.findClientByNameSexAndBirthDate(lastName, firstName, patrName, sex, birthDate)

        if clientIdList == [] and id:
            clientIdList.append(id)

        if clientIdList != []:
            clientId = clientIdList[0]
            self.log(u'<b><font color=brown>Обновляем</b>:'
                u' %s %s %s' % (lastName, firstName, patrName))
            self.updateClient(clientId, lastName, firstName, patrName, birthDate, sex, None)
            clientId = self.advancedAttachLogic(clientId, lastName, firstName, patrName, birthDate, sex,
                                    policyInsurer, policySerial, policyNumber,
                                    policyBegDate, policyEndDate,  policyNote,  self.policyTypeId,  attachType)
            str = [self.getClientIdentification(clientId), ('%s %s' % (policySerial, policyNumber)).strip(), lastName, firstName, patrName]

#            self.updateClientAttach(clientId, self.currentOrgId, attachType, policyBegDate,  policyEndDate)
#            self.updateClientPolicy(clientId, policyInsurer, policySerial, policyNumber,
#                                    policyBegDate, policyEndDate,  policyNote,  self.policyTypeId)
            self.updateClientWork(clientId, clientWorkId, clientWork)
            self.updateClientPhone(clientId, phone)
            self.updateClientAddress(clientId, None, None, None, None, None, freeInputAddress)
            key =  (policyInsurer, attachType, clientWork,
                    policyBegDate.toJulianDay(), policyEndDate.toJulianDay())

            if self.statProlongated.get(key):
                if str not in self.statProlongated[key]:
                    self.statProlongated[key].append(str)
            else:
                self.statProlongated[key] = [str]

            self.nUpdated += 1
        else:
            self.log(u'<b><font color=green>Добавляем</b>:'
                u' %s %s %s' % (lastName, firstName, patrName))
            clientId = self.addClient(self.generateClientIdentificator(),  lastName, firstName, patrName, birthDate, sex, None)
            self.addClientPolicy(clientId, policyInsurer, policySerial, policyNumber,
                                 policyBegDate, policyEndDate,  policyNote, self.policyTypeId)
            self.addClientAttach(clientId,  self.currentOrgId,  attachType, policyBegDate,  policyEndDate)
            self.addClientWork(clientId, clientWorkId, clientWork)
            self.addClientAddress(clientId, None, None, None, None, None, freeInputAddress)
            self.addClientPhone(clientId, phone)
            str = [self.getClientIdentification(clientId), ('%s %s' % (policySerial, policyNumber)).strip(), lastName, firstName, patrName]
            key =  (policyInsurer, attachType, clientWork,
                        policyBegDate.toJulianDay(), policyEndDate.toJulianDay())
            if self.statAttached.get(key):
                if str not in self.statAttached[key]:
                    self.statAttached[key].append(str)
            else:
                self.statAttached[key] = [str]

            self.nAdded += 1


    def processClientDetach(self, row):
        lastName  = nameCase(forceString(row['FAM'].strip()))
        firstName = nameCase(forceString(row['IMJA'].strip()))
        patrName  = nameCase(forceString(row['OTSCH'].strip()))
        sex  = self.sexMap.get(forceString(row['POL'][0] if row.asDict().has_key('POL') else 'X'), 0)
        birthDate = QDate(row['DROSCH']) if row['DROSCH'] else None
        endDate = QDate(row['DKONSTR']) if row['DKONSTR'] else QDate.currentDate()
        policyInsurer = self.insurerCache.get(row.asDict().get('NCOM'))

        if not self.checkBirthDate(birthDate, lastName, firstName, patrName):
            self.statSkipped.append([u'неверная дата рождения', '',  lastName, firstName, patrName])
            self.nSkipped +=1
            return

        if not policyInsurer:
            self.log(u'<b><font color=red>ВНИМАНИЕ</b>: ' \
                        u'Пациент `%s %s %s`, неизвестная страховая компания `%s`.' %\
                        (lastName, firstName, patrName,  forceString(row.asDict().get('NCOM', '-'))),  True)
            self.nSkipped += 1
            self.statSkipped.append([u'неизвестная страховая компания', '', lastName, firstName, patrName])
            return

        clientIdList = self.findClientByNameSexBirthDateAndInsurerId(lastName, firstName, patrName, sex, birthDate, policyInsurer)

        if clientIdList != []:
            for clientId in clientIdList:
                self.log(u'<b><font color=brown>Открепляем</b>:'
                    u' %s %s %s' % (lastName, firstName, patrName))
                self.updateClient(clientId, lastName, firstName, patrName, birthDate, sex, None)
                if self.detachClient(clientId, self.currentOrgId, endDate):
                    self.nUpdated += 1

                    record = selectLatestRecord('ClientPolicy', clientId,
                        '(Tmp.`insurer_id`=%d)' %  policyInsurer)
                    policySN = ('%s %s'% (
                        forceString(record.value('serial')),
                        forceString(record.value('number')))) if record else ''

                    str = [self.getClientIdentification(clientId), policySN, lastName, firstName, patrName]

                    record = selectLatestRecord('ClientAttach', clientId,
                        '(Tmp.`LPU_id`=%d)' %  self.currentOrgId)
                    attachId = forceRef(record.value('attachType_id')) if record else None

                    key = (policyInsurer, attachId, self.getClientWorkName(clientId), None, endDate.toJulianDay())

                    if self.statDetached.get(key):
                        if str not in self.statDetached[key]:
                            self.statDetached[key].append(str)
                    else:
                        self.statDetached[key] = [str]
                else:
                    self.statSkipped.append([u'', '', lastName, firstName, patrName])
                    self.nSkipped += 1
        else:
            self.log(u'<b><font color=red>Пациент не найден</b>:'
                u' %s %s %s' % (lastName, firstName, patrName))
            self.statSkipped.append([u'Пациент не найден', '', lastName, firstName, patrName])
            self.nSkipped += 1

# *****************************************************************************************

    def getClientWorkName(self, clientId):
        record = getClientWork(clientId)
        result = ''

        if record:
            workId = forceRef(record.value('org_id'))
            result = forceString(QtGui.qApp.db.translate('Organisation', 'id', workId,
                            'shortName')) if workId else forceString(record.value('freeInput'))

        return result

# *****************************************************************************************

    def findOrgId(self, INN, KPP):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecordEx(table, 'id',  [table['deleted'].eq(0),
                                                          table['INN'].eq(INN),
                                                          table['KPP'].eq(KPP)
                                                         ], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None

# *****************************************************************************************

    def findOrgByName(self, fullName):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecordEx(table, 'id',  [table['deleted'].eq(0),
                                                          table['fullName'].eq(fullName)], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None

# *****************************************************************************************

    def findOrgByShortName(self, shortName):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecordEx(table, 'id',  [table['deleted'].eq(0),
                                                          table['shortName'].eq(shortName)], 'id')
        if record:
            return forceRef(record.value(0))
        else:
            return None

# *****************************************************************************************

    def addOrg(self, shortName, fullName, INN, KPP, OKPO, OKVED, OGRN,
                        address, chief, accountant,  phone,  fax,  email,  masterId,
                        infisCode):
        table = QtGui.qApp.db.table('Organisation')
        record = table.newRecord()
        record.setValue('fullName', toVariant(fullName))
        record.setValue('shortName', toVariant(shortName))
        record.setValue('title', toVariant(shortName))
        record.setValue('OKPO', toVariant(OKPO))
        record.setValue('OKVED', toVariant(OKVED))
        record.setValue('INN', toVariant(INN))
        record.setValue('KPP', toVariant(KPP))
        record.setValue('OGRN', toVariant(OGRN))
        record.setValue('address', toVariant(address))
        record.setValue('chief', toVariant(chief))
        record.setValue('accountant', toVariant(accountant))
        record.setValue('phone', toVariant(phone))
        record.setValue('net_id',  toVariant(self.matureNetId))

        notes = u''

        if fax:
            notes += u'факс: %s' % fax

        if email:
            notes += u' email: %s' % email

        if notes != '':
            record.setValue('notes', toVariant(notes))

        if masterId:
            record.setValue('head_id',  toVariant(masterId))

        record.setValue('infisCode',  toVariant(infisCode))
        return QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def updateOrg(self, orgId, shortName, fullName, OKPO, OKVED, OGRN,
                            address, chief, accountant,  phone,  fax,  email, masterId,
                            infisCode):
        table = QtGui.qApp.db.table('Organisation')
        record = QtGui.qApp.db.getRecord(table, '*', orgId)
        record.setValue('fullName', toVariant(fullName))
        record.setValue('shortName', toVariant(shortName))
        record.setValue('title', toVariant(shortName))
        record.setValue('OKPO', toVariant(OKPO))
        record.setValue('OKVED', toVariant(OKVED))
        record.setValue('OGRN', toVariant(OGRN))
        record.setValue('address', toVariant(address))
        record.setValue('chief', toVariant(chief))
        record.setValue('accountant', toVariant(accountant))
        record.setValue('phone', toVariant(phone))
        record.setValue('net_id',  toVariant(self.matureNetId))
        notes = u''

        if fax:
            notes += u'факс: %s' % fax

        if email:
            notes += u' email: %s' % email

        if notes != '':
            record.setValue('notes', toVariant(notes))

        if masterId:
            record.setValue('head_id',  toVariant(masterId))

        record.setValue('infisCode',  toVariant(infisCode))
        return QtGui.qApp.db.updateRecord(table, record)

# *****************************************************************************************

    def addService(self, code,  name, infisCode, begDate, endDate):
        table = QtGui.qApp.db.table('rbService')
        record = table.newRecord()
        record.setValue('code', toVariant(code))
        record.setValue('name', toVariant(name))

        if infisCode:
            record.setValue('infis', toVariant(infisCode))
            record.setValue('eisLegacy', toVariant(True))

        record.setValue('begDate',  toVariant(begDate))
        record.setValue('endDate',  toVariant(endDate))
        return QtGui.qApp.db.insertRecord(table, record)

# *****************************************************************************************

    def updateService(self, serviceId, infisCode, name, begDate, endDate):
        table = QtGui.qApp.db.table('rbService')
        record = QtGui.qApp.db.getRecord(table, '*', serviceId)

        if name:
            record.setValue('name',  toVariant(name))

        if infisCode:
            record.setValue('infis', toVariant(infisCode))
            record.setValue('eisLegacy', toVariant(True))

        record.setValue('begDate',  toVariant(begDate))
        record.setValue('endDate',  toVariant(endDate))
        return QtGui.qApp.db.updateRecord(table, record)

# *****************************************************************************************

    def ensureSpecialityRegionalCode(self, specialityId, regionalCode):
        table = QtGui.qApp.db.table('rbSpeciality')
        record = QtGui.qApp.db.getRecord(table, '*', specialityId)
        if forceString(record.value('regionalCode')) == '':
            record.setValue('regionalCode',  toVariant(regionalCode))
            return QtGui.qApp.db.updateRecord(table, record)

# *****************************************************************************************

    def addActionType(self, code,  name, groupId,  serviceId):
        table = QtGui.qApp.db.table('ActionType')
        record = table.newRecord()
        record.setValue('code', toVariant(code))
        record.setValue('name', toVariant(name))
        record.setValue('class', toVariant(3)) # прочее
        record.setValue('deleted',  toVariant(0))

        if groupId:
            record.setValue('group_id', toVariant(groupId))

        if serviceId:
            record.setValue('service_id',  toVariant(serviceId))
        id = QtGui.qApp.db.insertRecord(table, record)
        self.ensureActionTypeService(id, serviceId, True)
        return id

# *****************************************************************************************

    def updateActionType(self, id, groupId,  serviceId):
        table = QtGui.qApp.db.table('ActionType')
        record = QtGui.qApp.db.getRecord(table, '*', id)
        if groupId:
            record.setValue('group_id', toVariant(groupId))

        if serviceId:
            record.setValue('service_id',  toVariant(serviceId))

        record.setValue('deleted',  toVariant(0))
        self.ensureActionTypeService(id, serviceId)
        return QtGui.qApp.db.updateRecord(table, record)

# *****************************************************************************************

    def ensureActionTypeService(self, actionTypeId,  serviceId,  new=False):
        table = QtGui.qApp.db.table('ActionType_Service')
        record = None if new else QtGui.qApp.db.getRecordEx(table, '*', [
                        table['master_id'].eq(actionTypeId),
                        table['service_id'].eq(serviceId)])
        if not record:
            record = table.newRecord()
            record.setValue('master_id', toVariant(actionTypeId))
            record.setValue('service_id', toVariant(serviceId))
            record.setValue('finance_id', toVariant(self.serviceFinanceId))
            return QtGui.qApp.db.insertRecord(table, record)
        else:
            record.setValue('finance_id', toVariant(self.serviceFinanceId))
            QtGui.qApp.db.updateRecord(table, record)

        return forceRef(record.value('id'))

# *****************************************************************************************

    def detachClient(self, clientId, orgId, endDate):
        record = selectLatestRecord('ClientAttach', clientId,
            '(Tmp.`LPU_id`=%d)' %  orgId)

        if record:
            record.setValue('endDate',  toVariant(endDate))
            QtGui.qApp.db.updateRecord('ClientAttach', record)
            self.log(u'Открепляем с %s (%d)' % (endDate.toString('dd.MM.yyyy'), clientId))
            return True
        else:
            self.log(u'<b><font color=red>ВНИМАНИЕ</b>:' \
                        u'Последнее действующее прикрепление не найдено.')

        return False

# *****************************************************************************************

    def advancedAttachLogic(self, clientId, lastName, firstName, patrName,
            birthDate, sex, policyInsurer, policySerial, policyNumber,
            policyBegDate, policyEndDate,  policyNote,  policyTypeId, attachType, generateNPP=True):

        record = selectLatestRecord('ClientPolicy', clientId,
            '(Tmp.`insurer_id` = %d) AND (Tmp.`policyType_id`=%d)' % (policyInsurer,  policyTypeId))

        # Если «NCOM» тот же = проверяем, совпадают ли по «DNACSTR», «DKONSTR» и «SPOLIS»
        # при совпадении выдача сообщения «прикрепление уже существует».

        if record:
            begDate = forceDate(record.value('begDate'))
            endDate = forceDate(record.value('endDate'))
            serial = forceString(record.value('serial'))
            number = forceString(record.value('number'))
            oldSN = (u'%s%s' % (serial.strip(), number.strip())).strip()
            newSN = (u'%s%s' % (policySerial.strip(), policyNumber.strip())).strip()

            if begDate == policyBegDate and endDate == policyEndDate and \
                    oldSN == newSN:
                self.log(u'Прикрепление уже существует')

                latestRecord = selectLatestRecord('ClientPolicy', clientId,
                '(Tmp.`policyType_id`=%d)' % policyTypeId)

                if forceRef(latestRecord.value('id')) != forceRef(record.value('id')):
                    self.log(u'Полис не является актуальным, исправляем.')
                    record.setValue('deleted',  toVariant(2))
                    QtGui.qApp.db.updateRecord('ClientPolicy',  record)
                else:
                    # устраняем различия в случае, если часть цифр полиса
                    # была перемещена из серии в номер или наоборот
                    if serial != policySerial or number != policyNumber:
                        self.log(u'Устраняем отличия в серии и номере полиса.')
                        record.setValue('serial', toVariant(policySerial))
                        record.setValue('number', toVariant(policyNumber))
                        QtGui.qApp.db.updateRecord('ClientPolicy',  record)

                    self.touchClientAttach(clientId, attachType, policyBegDate, policyEndDate)
                    return clientId
            else:
                self.log(u'Cтраховая не изменилась, полис изменился.')
        else:
            record = selectLatestRecord('ClientPolicy', clientId,
                '(Tmp.`policyType_id`=%d)' % policyTypeId)

            self.log(u'Cтраховая изменилась.')

            if record:
                # Если «NCOM» не совпадает, то смотрим:
                insurerId = forceRef(record.value('insurer_id'))
                assert insurerId != policyInsurer
                endDate = forceDate(record.value('endDate'))
                self.log(u'Изменилась страховая `%d` -> `%d`, дата снятия `%s`' % (insurerId,
                            policyInsurer,  endDate.toString('dd.MM.yyyy')))

                if (generateNPP and (not endDate or endDate.month() == policyEndDate.month()
                        or policyEndDate> endDate)):
                    # Если снят в этом месяце или не снят, то делается прикрепление с новым номером.
                    self.log(u'Клонируем пациента')
                    clientId = self.addClient(self.generateClientIdentificator(),  lastName, firstName, patrName, birthDate, sex, None)

                # Если пациент снят в прошлом месяце или ранее, то делаем прикрепление со старым номером И/Б.

        self.log(u'Добавляем новый полис и прикрепление.')
        self.addClientPolicy(clientId, policyInsurer, policySerial, policyNumber,
                         policyBegDate, policyEndDate,  policyNote, policyTypeId)
        self.addClientAttach(clientId,  self.currentOrgId,  attachType, policyBegDate,  policyEndDate)

        if record:
            endDate = forceDate(record.value('endDate'))

            if endDate > policyBegDate:
                self.log(u'Дата окончания предыдущего полиса изменена с %s на %s' % (
                         endDate.toString('dd.MM.yyyy'),  policyBegDate.addDays(-1).toString('dd.MM.yyyy')))
                record.setValue('endDate', toVariant(policyBegDate.addDays(-1)))
                newNote = u'%s; %s' % (endDate.toString('dd.MM.yyyy'),  forceString(record.value('note')))
                record.setValue('note', toVariant(newNote))
                QtGui.qApp.db.updateRecord('ClientPolicy', record)

        return clientId

# *****************************************************************************************

    def searchActionTypeSubgorups(self, parentId):
        result = []

        if parentId:
            table = QtGui.qApp.db.table('ActionType')
            idList = QtGui.qApp.db.getIdList(table, where = [
                                                             table['group_id'].eq(parentId),
                                                             table['id'].ne(parentId)
                                                             ])

            if idList != []:
                result.append(parentId)

            for id in idList:
                result.extend(self.searchActionTypeSubgorups(id))

        return result

# *****************************************************************************************

    def touchClientAttach(self, clientId, attachTypeId, begDate, endDate):
        u"""Обновляет дату изменения и пользователя у последнего прикрепления"""
        checkBegDate = begDate if begDate else QDate.currentDate()
        checkEndDate = endDate if endDate else QDate.currentDate()

        record = selectLatestRecord('ClientAttach', clientId,
            '(Tmp.`attachType_id`=%d) AND \
         (Tmp.`LPU_id`=%d) AND \
         (Tmp.`begDate`=\'%s\') AND \
         (Tmp.`endDate`=\'%s\')' % \
            (attachTypeId,self.currentOrgId, checkBegDate.toString(QtCore.Qt.ISODate),
             checkEndDate.toString(QtCore.Qt.ISODate)))

        if record:
            QtGui.qApp.db.updateRecord('ClientAttach', record)
        else: # изменился тип прикрепления
            self.addClientAttach(clientId,  self.currentOrgId,  attachTypeId, checkBegDate,  checkEndDate)

