#!/usr/bin/env python
# -*- coding: utf-8 -*-

#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtCore, QtGui, QtSql
from PyQt4.QtCore import *
from PyQt4.QtGui  import *

from library.crbcombobox import CRBPopupView
from library.Utils       import *

from Ui_ProfilePopup import Ui_ProfilePopup

class CProfilePopup(QtGui.QFrame, Ui_ProfilePopup):
    def __init__(self, parent):
        QtGui.QFrame.__init__(self, parent, Qt.Popup)
        self.setupUi(self)
        self.parent = parent #wtf? скрывать Qt-шный parent - это плохая затея
        self.connect(self.tblProfiles, QtCore.SIGNAL('doubleClicked(QModelIndex)'), self.emitDoubleClicked)

    def emitDoubleClicked(self, index):
        row = index.row()
#        self.emit(SIGNAL('profileClicked(int'), row)
        self.parent.setValueByRow(row)

    def setModel(self, model):
        self.selectionModel = QtGui.QItemSelectionModel(model)
        self.tblProfiles.setSelectionModel(self.selectionModel)
        self.tblProfiles.setModel(model)

    def setCurrent(self, id, column):
        model = self.tblProfiles.model()
        row = model.getRow(id)
        index = model.index(row, column)
        self.tblProfiles.setCurrentIndex(index)


class CProfileModel(QAbstractTableModel):
    def __init__(self, parent):
        QAbstractTableModel.__init__(self, parent)
        self.dataByRow = []
        self.mapDataToCode = {}
        self.mapDataToName = {}
        self.mapDataToId = {}
        self.mapIdToRow = {}
        self._loaded = False
        self.charLength = 0

    def loaded(self):
        return self._loaded

    def loadData(self):
        stmt = 'SELECT ID_PROFILE, PROFILE_INFIS_CODE, PROFILE_NAME FROM VMU_PROFILE ORDER BY PROFILE_NAME, PROFILE_INFIS_CODE'
        query = QtGui.qApp.db.query(stmt)
        maxName = 0
        maxCode = 0
        self.mapDataToCode.clear()
        self.mapDataToName.clear()
        self.mapDataToId.clear()
        self.mapIdToRow.clear()
        self.addItem(0, u'-', u'не задано')
        while query.next():
            record = query.record()
            id        = forceRef(record.value('ID_PROFILE'))
            infisCode = record.value('PROFILE_INFIS_CODE')
            name      = record.value('PROFILE_NAME')
            maxCode, maxName = self.addItem(id, infisCode, name, maxCode, maxName)
        self._loaded = True
        self.charLength = maxCode+maxName
        self.reset()

    def addItem(self, id, infisCode, name, maxCode=0, maxName=0):
        data = (id, infisCode, name)
        codeStr = forceString(infisCode)
        nameStr = forceString(name)
        maxCode = len(codeStr) if len(codeStr) > maxCode else maxCode
        maxName = len(nameStr) if len(nameStr) > maxName else maxName

        self.dataByRow.append(data)
        self.mapDataToCode[codeStr] = data
        self.mapDataToName[nameStr] = data
        self.mapDataToId[id] = data
        self.mapIdToRow[id]  = len(self.dataByRow)-1

        return maxCode, maxName

    def getData(self, id):
        return self.mapDataToId[id]

    def getName(self, id):
        data = self.getData(id)
        return forceString(data[2])

    def getCode(self, id):
        data = self.getData(id)
        return forceString(data[1])

    def rowCount(self, index=None):
        return len(self.dataByRow)

    def getRow(self, id):
        return self.mapIdToRow[id]

    def columnCount(self, index=None):
        return 2

    def getValue(self, row, column):
        data = self.dataByRow[row]
        return data[1+column]

    def getId(self, rowIndex):
        data = self.dataByRow[rowIndex]
        return forceRef(data[0])

    def getIdByCodeOrName(self, txt, column):
        if column == 0:
            return self.getIdByCode(txt)
        elif column == 1:
            return self.getIdByName(txt)
        return 0

    def getIdByCode(self, code):
        data = self.mapDataToCode.get(code, None)
        if data:
            return forceRef(data[0])
        return 0

    def getIdByName(self, name):
        data = self.mapDataToName.get(name, None)
        if data:
            return forceRef(data[0])
        return 0

    def searchId(self, id):
        return self.mapIdToRow[id]

    def find(self, txt):
        charCount = len(txt)
        for code in self.mapDataToCode.keys():
            if unicode(txt).upper() == unicode(code[:charCount]).upper():
                return code, 0
        for name in self.mapDataToName.keys():
            if unicode(txt).upper() == unicode(name[:charCount]).upper():
                return name, 1
        return txt, 1


    def headerData(self, section, orientation, role):
        if orientation == Qt.Horizontal:
            if role == Qt.DisplayRole:
                if section == 0:
                    return toVariant(u'Код профиля инфис')
                elif section == 1:
                    return toVariant(u'Наименование профиля')
        return QVariant()

    def data(self, index, role):
        if not index.isValid():
            return QVariant()
        elif role == Qt.DisplayRole or role == Qt.EditRole:
            row = index.row()
            if row < self.rowCount():
                value = self.getValue(row, index.column())
                return value
        return QVariant()

class CProfileLineEdit(QtGui.QLineEdit):
    def __init__(self):
        QtGui.QLineEdit.__init__(self)

    def setFound(self, probobly, txt):
        startSelection = len(txt)
        lngthSelection = len(probobly)-startSelection
        self.setText(probobly)
        self.setSelection(startSelection, lngthSelection)
        self.setCursorPosition(startSelection)

    def emitTextChangedEx(self, txt):
        self.emit(SIGNAL('textChangedEx(str)'), str(txt))

class CProfileComboBox(QtGui.QComboBox):
    showCode = 0
    showName = 1
    def __init__(self, parent):
        QtGui.QComboBox.__init__(self, parent)
        self.popupView = CProfilePopup(self)
        self._model = CProfileModel(self)
        self.popupView.setModel(self._model)
        self.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToMinimumContentsLength)
        self.preferredWidth = None
        self.setModel(self._model)
        self.setModelColumn(CProfileComboBox.showName)
        self.lineEdit = CProfileLineEdit()
        self.setLineEdit(self.lineEdit)
        self.connect(self.popupView, QtCore.SIGNAL('profileClicked(int)'), self.setValueByRow)
        self.connect(self.lineEdit, QtCore.SIGNAL('textEdited(QString)'), self.textEdited)

    def loadData(self):
        self._model.loadData()



    def showPopup(self):
        if not self._model.loaded():
            self._model.loadData()
        pos = self.rect().bottomLeft()
        pos2 = self.rect().topLeft()
        pos = self.mapToGlobal(pos)
        pos2 = self.mapToGlobal(pos2)
        size = self.popupView.sizeHint()
        width= max(size.width(), self.width())
        size.setWidth(width+self._model.charLength*2)
        screen = QtGui.QApplication.desktop().availableGeometry(pos)
        pos.setX( max(min(pos.x(), screen.right()-size.width()), screen.left()) )
        pos.setY( max(min(pos.y(), screen.bottom()-size.height()), screen.top()) )
        popupFrame = self.popupView
        popupFrame.move(pos)
        popupFrame.resize(size)
        popupFrame.show()

    def setValueByRow(self, row):
        self.setModelColumn(CProfileComboBox.showName)
        id = self._model.getId(row)
        self.setValue(id)
        self.popupView.hide()

    def value(self):
        u"""id записи"""
        rowIndex = self._currentRow
        return self._model.getId(rowIndex)

    def stringValue(self):
        id = self.value()
        code = self._model.getCode(id)
        name = self._model.getName(id)
        return ' | '.join([code, name])

    def setValue(self, itemId):
        rowIndex = self._model.searchId(itemId)
        self.setCurrentIndex(rowIndex)
        self.setToolTip(self.currentText())

    def textEdited(self, txt):
        if bool(txt):
            probobly, column = self._model.find(txt)
            proboblyId = self._model.getIdByCodeOrName(probobly, column)
            self._currentRow = self._model.getRow(proboblyId)
            self.popupView.setCurrent(proboblyId, column)
            self.setModelColumn(column)
#            if unicode(probobly) != unicode(txt):
            self.lineEdit.setFound(probobly, txt)
        else:
            self.lineEdit.setText('')
            self.popupView.setCurrent(0, 1)
            self._currentRow = 0
