# -*- coding: utf-8 -*-

#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################


import re
import time

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import *

from library.Utils import *

from Ui_Progress import Ui_ProgressDialog


def decodeString(v):
    return unicode(v.toString().simplified(),'cp1251')


class CProgressDialog(QtGui.QDialog, Ui_ProgressDialog):
    __pyqtSignals__ = ('startWork()', )

    def __init__(self, parent):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)
        self.KESDb = None
        self.EISDb = None

        self.mapMedicamentDosageFormCodeToId = {}
        self.mapMedicamentCodeToId = {}
        self.mapEquipmentGroupCodeToId = {}
        self.mapEquipmentCodeToId = {}
        self.mapNutrientGroupCodeToId = {}
        self.mapNutrientCodeToId = {}
        self.mapBloodPreparationTypeCodeToId = {}
        self.mapBloodPreparationCodeToId = {}
        self.mapServiceGroupCodeToId = {}
        self.mapServiceCodeToId = {}
        self.mapMesCodeToId = {}
        self.mapEISSpecialityIdToId = {}
        self.mapEISVisitTypeIdToId = {}
        self.mapEISMesGroupIdToId = {}
        self.mapMesEISIdToId = {}

        self.setMedicamentDosageFormsCount(0)
        self.setMedicamentGroupsCount(0)
        self.setMedicamentsCount(0)
        self.setEquipmentGroupsCount(0)
        self.setEquipmentsCount(0)
        self.setNutrientGroupsCount(0)
        self.setNutrientsCount(0)
        self.setBloodPreparationTypesCount(0)
        self.setBloodPreparationsCount(0)
        self.setServicesCount(0)
        self.setServicesContentsCount(0)
        self.setSpecialitiesCount(0)
        self.setVisitTypesCount(0)
        self.setKSGGroupsCount(0)
        self.setKSGProfilesCount(0)
        self.setKESCount(0)
        self.setKESMKBCount(0)
        self.setKESServicesCount(0)
        self.setKESMedicamentsCount(0)
        self.setKESBloodPreparationsCount(0)
        self.setKESEquipmentsCount(0)
        self.setKESNutrientsCount(0)
        self.setKESVisitsCount(0)

        self.start()
        self.connect(self, SIGNAL('startWork()'), self.startWork, Qt.QueuedConnection)


    def _setCount(self, progressBar, cnt):
        if cnt:
            progressBar.setValue(0)
            progressBar.setMaximum(cnt)
            progressBar.setEnabled(True)
            progressBar.setTextVisible(True)
        else:
            progressBar.setValue(0)
            progressBar.setMaximum(1)
            progressBar.setEnabled(False)
            progressBar.setTextVisible(False)


    def setMedicamentDosageFormsCount(self, cnt):
        self._setCount(self.prbMedicamentDosageForms, cnt)


    def setMedicamentGroupsCount(self, cnt):
        self._setCount(self.prbMedicamentGroups, cnt)


    def setMedicamentsCount(self, cnt):
        self._setCount(self.prbMedicaments, cnt)


    def setEquipmentGroupsCount(self, cnt):
        self._setCount(self.prbEquipmentGroups, cnt)


    def setEquipmentsCount(self, cnt):
        self._setCount(self.prbEquipments, cnt)


    def setNutrientGroupsCount(self, cnt):
        self._setCount(self.prbNutrientGroups, cnt)


    def setNutrientsCount(self, cnt):
        self._setCount(self.prbNutrients, cnt)


    def setBloodPreparationTypesCount(self, cnt):
        self._setCount(self.prbBloodPreparationTypes, cnt)


    def setBloodPreparationsCount(self, cnt):
        self._setCount(self.prbBloodPreparations, cnt)


    def setServicesCount(self, cnt):
        self._setCount(self.prbServices, cnt)


    def setServicesContentsCount(self, cnt):
        self._setCount(self.prbServicesContents, cnt)


    def setSpecialitiesCount(self, cnt):
        self._setCount(self.prbSpecialities, cnt)


    def setVisitTypesCount(self, cnt):
        self._setCount(self.prbVisitTypes, cnt)


    def setKSGGroupsCount(self, cnt):
        self._setCount(self.prbKSGGroups, cnt)


    def setKSGProfilesCount(self, cnt):
        self._setCount(self.prbKSGProfiles, cnt)


    def setKESCount(self, cnt):
        self._setCount(self.prbKES, cnt)


    def setKESMKBCount(self, cnt):
        self._setCount(self.prbKESMKB, cnt)


    def setKESServicesCount(self, cnt):
        self._setCount(self.prbKESServices, cnt)


    def setKESMedicamentsCount(self, cnt):
        self._setCount(self.prbKESMedicaments, cnt)


    def setKESEquipmentsCount(self, cnt):
        self._setCount(self.prbKESEquipments, cnt)


    def setKESNutrientsCount(self, cnt):
        self._setCount(self.prbKESNutrients, cnt)


    def setKESBloodPreparationsCount(self, cnt):
        self._setCount(self.prbKESBloodPreparations, cnt)


    def setKESVisitsCount(self, cnt):
        self._setCount(self.prbKESVisits, cnt)


    def start(self):
        self.processDone = False
        self.aborted = False
        self.btnCancel.setText(u'Отмена')


    def stop(self):
        self.processDone = True
        self.btnCancel.setText(u'Закрыть')


    def done(self, r):
        self.stop()
        QtGui.QDialog.done(self, r)


    def exec_(self):
        self.emit(SIGNAL('startWork()'))
        return QtGui.QDialog.exec_(self)


    @pyqtSignature('')
    def on_btnCancel_clicked(self):
        if self.processDone:
            self.accept()
        else:
            self.aborted = True


    def startWork(self):
        self.start()
        if self.KESDb:
            QtGui.qApp.call(self, self.KESWork)
        if self.EISDb:
            QtGui.qApp.call(self, self.EISWork)
        self.stop()


    def KESWork(self):
        def getMedicamentDosageFormsCount():
            return self.KESDb.getCount('Drug_form')
        def getMedicamentGroupsCount():
            return self.KESDb.getCount('Drug_farm_group')
        def getMedicamentsCount():
            return self.KESDb.getCount('Drug_reestr')
        def getEquipmentGroupsCount():
            return self.KESDb.getCount('Med_pr_grop')
        def getEquipmentsCount():
            return self.KESDb.getCount('Med_pr_spr')
        def getNutrientGroupsCount():
            return self.KESDb.getCount('Nutrient_grop')
        def getNutrientsCount():
            return self.KESDb.getCount('Nutrient_spr')
        def getBloodPreparationTypesCount():
            return self.KESDb.getCount('Blood_group')+self.KESDb.getCount('Blood_spr')
        def getBloodPreparationsCount():
            return self.KESDb.getCount('Blood_doz')
        def getServicesCount():
            return self.KESDb.getCount('uslugi')
        def getServicesContentsCount():
            return self.KESDb.getCount('uslugi_sl')
        def getKESCount():
            return self.KESDb.getCount('Std_stan')
        def getKESMKBCount():
            return self.KESDb.getCount('Std_ds')
        def getKESServicesCount():
            return self.KESDb.getCount('Std_obsl')
        def getKESMedicamentsCount():
            return self.KESDb.getCount('Std_lec')
        def getKESEquipmentsCount():
            return self.KESDb.getCount('Std_predm')
        def getKESNutrientsCount():
            return self.KESDb.getCount('Std_nutr')
        def getKESBloodPreparationsCount():
            return self.KESDb.getCount('Std_blood')
        self.setMedicamentDosageFormsCount(QtGui.qApp.callWithWaitCursor(self, getMedicamentDosageFormsCount))
        self.setMedicamentGroupsCount(QtGui.qApp.callWithWaitCursor(self, getMedicamentGroupsCount))
        self.setMedicamentsCount(QtGui.qApp.callWithWaitCursor(self, getMedicamentsCount))
        self.setEquipmentGroupsCount(QtGui.qApp.callWithWaitCursor(self, getEquipmentGroupsCount))
        self.setEquipmentsCount(QtGui.qApp.callWithWaitCursor(self, getEquipmentsCount))
        self.setNutrientGroupsCount(QtGui.qApp.callWithWaitCursor(self, getNutrientGroupsCount))
        self.setNutrientsCount(QtGui.qApp.callWithWaitCursor(self, getNutrientsCount))
        self.setBloodPreparationTypesCount(QtGui.qApp.callWithWaitCursor(self, getBloodPreparationTypesCount))
        self.setBloodPreparationsCount(QtGui.qApp.callWithWaitCursor(self, getBloodPreparationsCount))
        self.setServicesCount(QtGui.qApp.callWithWaitCursor(self, getServicesCount))
        self.setServicesContentsCount(QtGui.qApp.callWithWaitCursor(self, getServicesContentsCount))
        self.setKESCount(QtGui.qApp.callWithWaitCursor(self, getKESCount))
        self.setKESMKBCount(QtGui.qApp.callWithWaitCursor(self, getKESMKBCount))
        self.setKESServicesCount(QtGui.qApp.callWithWaitCursor(self, getKESServicesCount))
        self.setKESMedicamentsCount(QtGui.qApp.callWithWaitCursor(self, getKESMedicamentsCount))
        self.setKESEquipmentsCount(QtGui.qApp.callWithWaitCursor(self, getKESEquipmentsCount))
        self.setKESNutrientsCount(QtGui.qApp.callWithWaitCursor(self, getKESNutrientsCount))
        self.setKESBloodPreparationsCount(QtGui.qApp.callWithWaitCursor(self, getKESBloodPreparationsCount))
        self.step(self.medicamentsStep)
        self.step(self.equipmentsStep)
        self.step(self.nutrientsStep)
        self.step(self.bloodPreparationsStep)
        self.step(self.servicesStep)
        self.step(self.kesStep)


    def EISWork(self):
        def getSpecialitiesCount():
            return self.EISDb.getCount('VMU_PRVS', '*', 'IS_ACTIVE!=0')

        def getVisitTypesCount():
            return self.EISDb.getCount('VMU_PRVS_TYPE')

        def getKSGGroupsCount():
            return self.EISDb.getCount('VMU_KSG_GROUP', '*', 'IS_ACTIVE!=0')

        def getKSGProfilesCount():
            return self.EISDb.getCount('VMU_PROFILE', '*', 'ID_PROFILE_TYPE=7')

        def getKESVisitsCount():
            return self.EISDb.getCount('VMU_PRVS_PROFILE', '*', 'IS_ACTIVE!=0')

        self.setSpecialitiesCount(QtGui.qApp.callWithWaitCursor(self, getSpecialitiesCount))
        self.setVisitTypesCount(QtGui.qApp.callWithWaitCursor(self, getVisitTypesCount))
        self.setKSGGroupsCount(QtGui.qApp.callWithWaitCursor(self, getKSGGroupsCount))
        self.setKSGProfilesCount(QtGui.qApp.callWithWaitCursor(self, getKSGProfilesCount))
        self.setKESVisitsCount(QtGui.qApp.callWithWaitCursor(self, getKESVisitsCount))
        self.step(self.specialitiesStep)
        self.step(self.visitTypesStep)
        self.step(self.KSGGroupsStep)
        self.step(self.KSGProfilesStep)
        self.step(self.KESVisitsStep)


    def step(self, procedure):
        db = QtGui.qApp.db
        db.transaction()
        try:
            procedure()
            if self.aborted:
                db.rollback()
                return
            db.commit()
        except:
            db.rollback()
            raise


    def process(self, query, procedure, progressBar):
        n = progressBar.value()
        t0 = time.clock()
        while query.next():
            QtGui.qApp.processEvents()
            if self.aborted:
                self.reject()
                return
            n += 1
            t1 = time.clock()
            if t1-t0>0.25:
                progressBar.setValue(n)
                t0 = t1
            procedure(query.record())
        progressBar.setValue(n)


# *****************************************************************************************

    def medicamentsStep(self):
        db = QtGui.qApp.db
        db.query('DELETE FROM `mrbMedicamentDosageForm`');
        db.query('DELETE FROM `mrbMedicamentGroup`');
        db.query('DELETE FROM `mrbMedicament`');
        self.process(self.KESDb.query('SELECT * FROM Drug_form'),       self.addMedicamentDosageForm, self.prbMedicamentDosageForms)
        self.process(self.KESDb.query('SELECT * FROM Drug_farm_group'), self.addMedicamentGroup,      self.prbMedicamentGroups)
        self.process(self.KESDb.query('SELECT * FROM Drug_reestr'),     self.addMedicament,           self.prbMedicaments)


    def addMedicamentDosageForm(self, record):
        code = forceString(record.value('CODE_PV'))
        name = record.value('NAME')

        db = QtGui.qApp.db
        table = db.table('mrbMedicamentDosageForm')
        record = table.newRecord()
        record.setValue('code', QVariant(code))
        record.setValue('name', name)
        id = db.insertRecord(table, record)
        self.registerMedicamentDosageForm(code, id)


    def registerMedicamentDosageForm(self, code, id):
        self.mapMedicamentDosageFormCodeToId[code] = id


    def getMedicamentDosageFormId(self, code):
        result = self.mapMedicamentDosageFormCodeToId.get(code, None)
        return result


    def addMedicamentGroup(self, record):
        code1 = forceString(record.value('Farm_tip'))
        code2 = forceString(record.value('CODE_FARM'))
        code = code2 if code2 else code1
        name = record.value('Name2')

        db = QtGui.qApp.db
        table = db.table('mrbMedicamentGroup')
        record = table.newRecord()
        record.setValue('code', QVariant(code))
        record.setValue('name', name)
        id = db.insertRecord(table, record)


    def addMedicament(self, record):
        code = forceString(record.value('CODE_FARM'))
        name = record.value('MNN_Name')
        tradeName = record.value('TORG_Name')
        dosageFormCode = forceString(record.value('CODE_PV'))
        dosageFormId = self.getMedicamentDosageFormId(dosageFormCode)
        dosage = record.value('DOZA')
        form = record.value('FORMA')
        packSize = record.value('NUM_FORM')
        packPrice = record.value('SUM_RUB')
        unitPrice = record.value('SUM_FORM_RUB')

        db = QtGui.qApp.db
        table = db.table('mrbMedicament')
        record = table.newRecord()
        record.setValue('code', QVariant(code))
        record.setValue('name', name)
        record.setValue('tradeName', tradeName)
        record.setValue('dosage', dosage)
        record.setValue('form', form)
        record.setValue('dosageForm_id', toVariant(dosageFormId))
        record.setValue('packSize', packSize)
        record.setValue('packPrice', packPrice)
        record.setValue('unitPrice', unitPrice)
        id = db.insertRecord(table, record)
        self.registerMedicament(code, id)


    def registerMedicament(self, code, id):
        self.mapMedicamentCodeToId[code] = id


    def getMedicamentId(self, code):
        result = self.mapMedicamentCodeToId.get(code, None)
        return result


# *****************************************************************************************

    def equipmentsStep(self):
        db = QtGui.qApp.db
        db.query('DELETE FROM `mrbEquipmentGroup`');
        db.query('DELETE FROM `mrbEquipment`');
        self.process(self.KESDb.query('SELECT * FROM Med_pr_grop'),  self.addEquipmentGroup, self.prbEquipmentGroups)
        self.process(self.KESDb.query('SELECT * FROM Med_pr_spr'),   self.addEquipment, self.prbEquipments)


    def addEquipmentGroup(self, record):
        shortCode = forceString(record.value('TIP'))
        longCode  = forceString(record.value('CODE_USLUG'))
        code = longCode if longCode else shortCode
        name = record.value('NAME')

        db = QtGui.qApp.db
        table = db.table('mrbEquipmentGroup')
        record = table.newRecord()
        record.setValue('code', QVariant(code))
        record.setValue('name', name)
        id = db.insertRecord(table, record)
        self.registerEquipmentGroup(code, id)


    def registerEquipmentGroup(self, code, id):
        self.mapEquipmentGroupCodeToId[code] = id


    def getEquipmentGroupId(self, code):
        result = self.mapEquipmentGroupCodeToId.get(code, None)
        return result


    def addEquipment(self, record):
        code = forceString(record.value('CODE_USLUG'))
        groupCode = code.rsplit('.',1)[0]
        name = record.value('NAME')
        unit = record.value('EDINICA')
        dosage = record.value('DOZA')
        tariff = record.value('TARIFF')

        db = QtGui.qApp.db
        table = db.table('mrbEquipment')
        record = table.newRecord()
        record.setValue('group_id', toVariant(self.getEquipmentGroupId(groupCode)))
        record.setValue('code', QVariant(code))
        record.setValue('name', name)

        record.setValue('unit', unit)
        record.setValue('dosage', dosage)
        record.setValue('tariff', tariff)
        id = db.insertRecord(table, record)
        self.registerEquipment(code, id)


    def registerEquipment(self, code, id):
        self.mapEquipmentCodeToId[code] = id


    def getEquipmentId(self, code):
        result = self.mapEquipmentCodeToId.get(code, None)
        return result


# *****************************************************************************************

    def nutrientsStep(self):
        db = QtGui.qApp.db
        db.query('DELETE FROM `mrbNutrientGroup`');
        db.query('DELETE FROM `mrbNutrient`');
        self.process(self.KESDb.query('SELECT * FROM Nutrient_grop'),  self.addNutrientGroup, self.prbNutrientGroups)
        self.process(self.KESDb.query('SELECT * FROM Nutrient_spr'),   self.addNutrient, self.prbNutrients)


    def addNutrientGroup(self, record):
        shortCode = forceString(record.value('TIP'))
        longCode  = forceString(record.value('CODE_USLUG'))
        code = longCode if longCode else shortCode
        name = record.value('NAME')

        db = QtGui.qApp.db
        table = db.table('mrbNutrientGroup')
        record = table.newRecord()
        record.setValue('code', QVariant(code))
        record.setValue('name', name)
        id = db.insertRecord(table, record)
        self.registerNutrientGroup(code, id)


    def registerNutrientGroup(self, code, id):
        self.mapNutrientGroupCodeToId[code] = id


    def getNutrientGroupId(self, code):
        result = self.mapNutrientGroupCodeToId.get(code, None)
        return result


    def addNutrient(self, record):
        code = forceString(record.value('CODE_USLUG'))
        groupCode = code.rsplit('.',1)[0]
        name = record.value('NAME')
        unit = record.value('EDINICA')
        dosage = record.value('DOZA')
        tariff = record.value('TARIFF')

        db = QtGui.qApp.db
        table = db.table('mrbNutrient')
        record = table.newRecord()
        record.setValue('group_id', toVariant(self.getNutrientGroupId(groupCode)))
        record.setValue('code', QVariant(code))
        record.setValue('name', name)

        record.setValue('unit', unit)
        record.setValue('dosage', dosage)
        record.setValue('tariff', tariff)
        id = db.insertRecord(table, record)
        self.registerNutrient(code, id)


    def registerNutrient(self, code, id):
        self.mapNutrientCodeToId[code] = id


    def getNutrientId(self, code):
        result = self.mapNutrientCodeToId.get(code, None)
        return result


# *****************************************************************************************

    def bloodPreparationsStep(self):
        db = QtGui.qApp.db
        db.query('DELETE FROM `mrbBloodPreparationType`');
        db.query('DELETE FROM `mrbBloodPreparation`');
        self.process(self.KESDb.query('SELECT * FROM Blood_group'),     self.addBloodPreparationType1, self.prbBloodPreparationTypes)
        self.process(self.KESDb.query('SELECT * FROM Blood_spr'),       self.addBloodPreparationType2, self.prbBloodPreparationTypes)
        self.process(self.KESDb.query('SELECT * FROM Blood_doz'),       self.addBloodPreparation,      self.prbBloodPreparations)


    def addBloodPreparationType1(self, record):
        shortCode = forceString(record.value('TIP'))
        longCode  = forceString(record.value('CODE_USLUG'))
        code = longCode if longCode else shortCode
        name = record.value('NAME')

        db = QtGui.qApp.db
        table = db.table('mrbBloodPreparationType')
        record = table.newRecord()
        record.setValue('code', QVariant(code))
        record.setValue('name', name)
        id = db.insertRecord(table, record)
        self.registerBloodPreparationType(code, id)


    def addBloodPreparationType2(self, record):
        code  = forceString(record.value('CODE_USLUG'))
        name = record.value('NAME')

        db = QtGui.qApp.db
        table = db.table('mrbBloodPreparationType')
        record = table.newRecord()
        record.setValue('code', QVariant(code))
        record.setValue('name', name)
        id = db.insertRecord(table, record)
        self.registerBloodPreparationType(code, id)


    def registerBloodPreparationType(self, code, id):
        self.mapBloodPreparationTypeCodeToId[code] = id


    def getBloodPreparationTypeId(self, code):
        result = self.mapBloodPreparationTypeCodeToId.get(code, None)
        return result


    def addBloodPreparation(self, record):
        typeCode = forceString(record.value('CODE_USLUG'))
        code = forceString(record.value('CODE_USLUG1'))
        dosage = record.value('DOZA')
        tariff = record.value('TARIFF')

        db = QtGui.qApp.db
        table = db.table('mrbBloodPreparation')
        record = table.newRecord()
        record.setValue('code', QVariant(code))
        record.setValue('type_id', toVariant(self.getBloodPreparationTypeId(typeCode)))
        record.setValue('dosage', dosage)
        record.setValue('tariff', tariff)
        id = db.insertRecord(table, record)
        self.registerBloodPreparation(code, id)


    def registerBloodPreparation(self, code, id):
        self.mapBloodPreparationCodeToId[code] = id


    def getBloodPreparationId(self, code):
        result = self.mapBloodPreparationCodeToId.get(code, None)
        return result


# *****************************************************************************************

    def servicesStep(self):
        db = QtGui.qApp.db
        db.query('DELETE FROM `mrbService`');
        db.query('DELETE FROM `mrbService_Contents`');
        self.process(self.KESDb.query('SELECT * FROM uslugi'),        self.addService, self.prbServices)
        self.process(self.KESDb.query('SELECT * FROM uslugi_sl'),     self.addServiceContents, self.prbServicesContents)


    def addService(self, record):
        shortCode = forceString(record.value('CODE_USLUG'))
        code = forceString(record.value('RAZDEL'))+shortCode
        name = record.value('NAME')
        groupId = self.getServiceGroup(forceString(record.value('TIP_USLUGI')), code)
        doctorWTU = record.value('UET_VR')
        paramedicalWTU = record.value('UET_MS')

        db = QtGui.qApp.db
        table = db.table('mrbService')
        record = table.newRecord()
        record.setValue('code', QVariant(code))
        record.setValue('group_id', QVariant(groupId))
        record.setValue('name', name)
        record.setValue('doctorWTU', doctorWTU)
        record.setValue('paramedicalWTU', paramedicalWTU)
        id = db.insertRecord(table, record)
        self.registerServiceId(shortCode, id)


    def getServiceGroup(self, code, hint):
        result = self.mapServiceGroupCodeToId.get(code, None)
        if not result:
            db = QtGui.qApp.db
            table = db.table('mrbServiceGroup')
            record = db.getRecordEx(table, 'id', [table['code'].eq(code)], 'id')
            if record:
                result = forceInt(record.value('id'))
            else:
                record = table.newRecord()
                record.setValue('code', QVariant(code))
                record.setValue('name', QVariant(u'Что-то типа '+hint))
                result = db.insertRecord(table, record)
            self.mapServiceGroupCodeToId[code] = result
        return result


    def registerServiceId(self, shortServiceCode, id):
        self.mapServiceCodeToId[shortServiceCode] = id


    def getServiceId(self, shortServiceCode):
        result = self.mapServiceCodeToId.get(shortServiceCode, None)
        return result


    def addServiceContents(self, record):
        shortCodeMaster = forceString(record.value('CODE_USLUG_B'))
        masterServiceId = self.getServiceId(shortCodeMaster)
        shortCodeDepend = forceString(record.value('CODE_USLUG_A'))
        dependServiceId = self.getServiceId(shortCodeDepend)
        if masterServiceId and dependServiceId:
            averageQnt = record.value('KRATN')
            necessity = record.value('POTREBN')
            db = QtGui.qApp.db
            table = db.table('mrbService_Contents')
            record = table.newRecord()
            record.setValue('master_id', QVariant(masterServiceId))
            record.setValue('service_id', QVariant(dependServiceId))
            record.setValue('averageQnt', QVariant(averageQnt))
            record.setValue('necessity', QVariant(necessity))
            db.insertRecord(table, record)
        else:
            print 'service not found!', shortCodeMaster, masterServiceId, shortCodeDepend, dependServiceId


# *****************************************************************************************

    def kesStep(self):
        db = QtGui.qApp.db
        db.query('DELETE FROM `MES`');
        db.query('DELETE FROM `MES_mkb`');
        db.query('DELETE FROM `MES_service`');
        db.query('DELETE FROM `MES_medicament`');
        db.query('DELETE FROM `MES_bloodPreparation`');
        db.query('DELETE FROM `MES_visit`');

        self.process(self.KESDb.query('SELECT * FROM Std_stan'),  self.addKES,    self.prbKES)
        self.process(self.KESDb.query('SELECT * FROM Std_ds'),    self.addKESMKB, self.prbKESMKB)
        self.process(self.KESDb.query('SELECT * FROM Std_obsl'),  self.addKESService, self.prbKESServices)
        self.process(self.KESDb.query('SELECT * FROM Std_lec'),   self.addKESMedicament, self.prbKESMedicaments)
        self.process(self.KESDb.query('SELECT * FROM Std_predm'), self.addKESEquipment,  self.prbKESEquipments)
        self.process(self.KESDb.query('SELECT * FROM Std_nutr'),  self.addKESNutrient,   self.prbKESNutrients)
        self.process(self.KESDb.query('SELECT * FROM Std_blood'), self.addKESBloodPreparation, self.prbKESBloodPreparations)


    def addKES(self, record):
        code = record.value('MES')
        name = record.value('NAME')
        patientModel = record.value('MODEL')
        minDuration = record.value('K_DN_MIN')
        maxDuration = record.value('K_DN_MAX')
        avgDuration = record.value('K_DN')
        tariff = record.value('TARIFF')

        db = QtGui.qApp.db
        table = QtGui.qApp.db.table('MES')
        record = table.newRecord()
        record.setValue('code', code)
        record.setValue('name', name)
        record.setValue('patientModel', patientModel)
        record.setValue('minDuration', minDuration)
        record.setValue('maxDuration', maxDuration)
        record.setValue('avgDuration', avgDuration)
        record.setValue('tariff', tariff)
        id = QtGui.qApp.db.insertOrUpdate(table, record)
        self.registerMes(forceString(code), id)


    def registerMes(self, code, id):
        self.mapMesCodeToId[code] = id


    def getMesId(self, code):
        result = self.mapMesCodeToId.get(code, None)
        return result


    def addKESMKB(self, record):
        mesCode = forceString(record.value('MES'))
        mesId = self.getMesId(mesCode)
        if mesId:
            mkb = record.value('CODE_USLUG')
            db = QtGui.qApp.db
            table = db.table('MES_mkb')
            record = table.newRecord()
            record.setValue('master_id', QVariant(mesId))
            record.setValue('mkb', mkb)
            db.insertRecord(table, record)
        else:
            print 'mes not found!', mesCode, mesId


    def addKESService(self, record):
        mesCode = forceString(record.value('MES'))
        mesId = self.getMesId(mesCode)
        serviceCode = forceString(record.value('CODE_USLUG'))
        serviceId = self.getServiceId(serviceCode)
        if mesId and serviceId:
            averageQnt = record.value('KRATN')
            necessity = record.value('POTREBN')
            db = QtGui.qApp.db
            table = db.table('MES_service')
            record = table.newRecord()
            record.setValue('master_id', QVariant(mesId))
            record.setValue('service_id', QVariant(serviceId))
            record.setValue('averageQnt', QVariant(averageQnt))
            record.setValue('necessity', QVariant(necessity))
            id = db.insertRecord(table, record)
        else:
            print 'mes or service not found!', mesCode, mesId, serviceCode, serviceId


    def addKESMedicament(self, record):
        mesCode = forceString(record.value('MES'))
        mesId = self.getMesId(mesCode)
        medicamentCode = forceString(record.value('CODE_USLUG'))
        if mesId:
            averageQnt = record.value('KRATN')
            necessity = record.value('POTREBN')
            dosageFormCode = forceString(record.value('CODE_PV'))
            dosageFormId = self.getMedicamentDosageFormId(dosageFormCode)
            dosage = record.value('DOZA')

            db = QtGui.qApp.db
            table = db.table('MES_medicament')
            record = table.newRecord()
            record.setValue('master_id', QVariant(mesId))
            record.setValue('medicamentCode', QVariant(medicamentCode))
            record.setValue('dosageForm_id',  QVariant(dosageFormId))
            record.setValue('dosage',         QVariant(dosage))
            record.setValue('averageQnt', QVariant(averageQnt))
            record.setValue('necessity', QVariant(necessity))
            db.insertRecord(table, record)
        else:
            print 'mes not found!', mesCode, mesId


    def addKESEquipment(self, record):
        mesCode = forceString(record.value('MES'))
        mesId = self.getMesId(mesCode)
        equipmentCode = forceString(record.value('CODE_USLUG'))
        equipmentId = self.getEquipmentId(equipmentCode)
        if mesId and equipmentId:
            averageQnt = record.value('KRATN')
            necessity = record.value('POTREBN')

            db = QtGui.qApp.db
            table = db.table('MES_equipment')
            record = table.newRecord()
            record.setValue('master_id', QVariant(mesId))
            record.setValue('equipment_id', QVariant(equipmentId))
            record.setValue('averageQnt', QVariant(averageQnt))
            record.setValue('necessity', QVariant(necessity))
            db.insertRecord(table, record)
        else:
            print 'mes or equipment not found!', mesCode, mesId, equipmentCode, equipmentId


    def addKESNutrient(self, record):
        mesCode = forceString(record.value('MES'))
        mesId = self.getMesId(mesCode)
        nutrientCode = forceString(record.value('CODE_USLUG'))
        nutrientId = self.getNutrientId(nutrientCode)
        if mesId and nutrientId:
            averageQnt = record.value('KRATN')
            necessity = record.value('POTREBN')

            db = QtGui.qApp.db
            table = db.table('MES_nutrient')
            record = table.newRecord()
            record.setValue('master_id', QVariant(mesId))
            record.setValue('nutrient_id', QVariant(nutrientId))
            record.setValue('averageQnt', QVariant(averageQnt))
            record.setValue('necessity', QVariant(necessity))
            db.insertRecord(table, record)
        else:
            print 'mes or nutrient not found!', mesCode, mesId, nutrientCode, nutrientId


    def addKESBloodPreparation(self, record):
        mesCode = forceString(record.value('MES'))
        mesId = self.getMesId(mesCode)
        bloodPreparationCode = forceString(record.value('CODE_USLUG'))
        bloodPreparationId = self.getBloodPreparationId(bloodPreparationCode)
        if mesId and bloodPreparationId:
            averageQnt = record.value('KRATN')
            necessity = record.value('POTREBN')

            db = QtGui.qApp.db
            table = db.table('MES_bloodPreparation')
            record = table.newRecord()
            record.setValue('master_id', QVariant(mesId))
            record.setValue('preparation_id', QVariant(bloodPreparationId))
            record.setValue('averageQnt', QVariant(averageQnt))
            record.setValue('necessity', QVariant(necessity))
            db.insertRecord(table, record)
        else:
            print 'mes or bloodPreparation not found!', mesCode, mesId, bloodPreparationCode, bloodPreparationId

# *****************************************************************************************


    def specialitiesStep(self):
        db = QtGui.qApp.db
        db.query('DELETE FROM `mrbSpeciality`');
        self.process(self.EISDb.query('SELECT * FROM VMU_PRVS WHERE IS_ACTIVE!=0'),  self.addSpeciality, self.prbSpecialities)


    def addSpeciality(self, record):
        EISId = forceInt(record.value('ID_PRVS'))
        code = forceString(record.value('PRVS_CODE'))
        name = record.value('PRVS_NAME')
        db = QtGui.qApp.db

        if code and code[:1] != '0':
            code = '0'+code

        table = db.table('mrbSpeciality')
        record = table.newRecord()
        record.setValue('code', toVariant(code))
        record.setValue('name', name)
        record.setValue('regionalCode', toVariant(str(EISId)))
        id = db.insertRecord(table, record)
        self.mapEISSpecialityIdToId[EISId] = id


    def visitTypesStep(self):
        db = QtGui.qApp.db
        db.query('DELETE FROM `mrbVisitType`');
        self.process(self.EISDb.query('SELECT * FROM VMU_PRVS_TYPE'),  self.addVisitType, self.prbVisitTypes)


    def addVisitType(self, record):
        EISId = forceInt(record.value('ID_PRVS_TYPE'))
        code = record.value('CODE')
        name = record.value('NAME')
        db = QtGui.qApp.db

        table = db.table('mrbVisitType')
        record = table.newRecord()
        record.setValue('code', code)
        record.setValue('name', name)
        id = db.insertRecord(table, record)
        self.mapEISVisitTypeIdToId[EISId] = id


    def KSGGroupsStep(self):
        db = QtGui.qApp.db
        db.query('UPDATE `MES` SET `group_id`=NULL');
        db.query('DELETE FROM `mrbMESGroup`');
        self.process(self.EISDb.query('SELECT * FROM VMU_KSG_GROUP WHERE IS_ACTIVE!=0'),  self.addMESGroup, self.prbKSGGroups)


    def addMESGroup(self, record):
        EISId = forceInt(record.value('ID_KSG_GROUP'))
        code = record.value('KSG_GROUP_CODE')
        name = record.value('KSG_GROUP_NAME')
        db = QtGui.qApp.db

        table = db.table('mrbMESGroup')
        record = table.newRecord()
        record.setValue('code', code)
        record.setValue('name', name)
        id = db.insertRecord(table, record)
        self.mapEISMesGroupIdToId[EISId] = id


    def KSGProfilesStep(self):
        db = QtGui.qApp.db
        db.query('UPDATE `MES` SET `KSGNorm`=0');
        self.process(self.EISDb.query('SELECT * FROM VMU_PROFILE WHERE ID_PROFILE_TYPE=7'),  self.addEISMES, self.prbKSGProfiles)


    def addEISMES(self, record):
        oldId = forceInt(record.value('ID_PROFILE'))
        code = forceString(record.value('PROFILE_INFIS_CODE'))
        name = record.value('PROFILE_NAME')
        groupId = self.mapEISMesGroupIdToId.get(forceRef(record.value('ID_KSG_GROUP')), None)
        norm = record.value('KSG_NORM')

        db = QtGui.qApp.db

        table = db.table('MES')
        record = db.getRecordEx(table, '*', table['code'].eq(code))
        if record:
            record.setValue('group_id', toVariant(groupId))
            record.setValue('KSGNorm',  norm)
            db.updateRecord(table, record)
            self.mapMesEISIdToId[oldId] = forceInt(record.value('id'))



# *****************************************************************************************


    def KESVisitsStep(self):
        db = QtGui.qApp.db
        db.query('DELETE FROM `MES_visit`');
        self.process(self.EISDb.query('SELECT * FROM VMU_PRVS_PROFILE WHERE IS_ACTIVE!=0'),  self.addKESVisit, self.prbKESVisits)


    def addKESVisit(self, record):
        mesId = self.mapMesEISIdToId.get(forceInt(record.value('ID_PROFILE')), None)
        specialityId = self.mapEISSpecialityIdToId.get(forceInt(record.value('ID_PRVS')), None)
        visitTypeId  = self.mapEISVisitTypeIdToId.get(forceInt(record.value('ID_PRVS_TYPE')), None)
        if mesId and specialityId and visitTypeId:
            groupCode   = forceInt(record.value('PRVS_PROFILE_GROUP'))
            averageQnt  = forceInt(record.value('PRVS_PROFILE_SK'))
            serviceCode = forceString(record.value('CODE_SERVICE'))

            db = QtGui.qApp.db
            table = db.table('MES_visit')
            record = table.newRecord()
            record.setValue('master_id',     toVariant(mesId))
            record.setValue('visitType_id',  toVariant(visitTypeId))
            record.setValue('speciality_id', toVariant(specialityId))
            record.setValue('serviceCode',   toVariant(serviceCode))
            record.setValue('groupCode',     toVariant(groupCode))
            record.setValue('averageQnt',    toVariant(averageQnt))
            id = db.insertRecord(table, record)