#!/usr/bin/env python
# -*- coding: utf-8 -*-

#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4         import QtGui, QtCore, QtSql
from PyQt4.QtGui   import *
from PyQt4.QtCore  import *

from library.TableModel  import *
from library.database    import CInterbaseDatabase, CTable, CField, CRecordCache
from library.database    import CTableRecordCache
from library             import database
from library.Utils       import *

from Utils               import *

# ####Columns################
class CMyDateCol(CCol):
    def __init__(self, title, fields, defaultWidth, alignment='l', highlightRedDate=True):
        CCol.__init__(self, title, fields, defaultWidth, alignment)
        self.highlightRedDate = False #highlightRedDate and QtGui.qApp.highlightRedDate()

    def format(self, values):
        val = values[0]
        if val.type() == QVariant.Date or val.type() == QVariant.DateTime:
            val = val.toDate()
            return QVariant(val.toString(Qt.LocaleDate))
        return CCol.invalid


    def getForegroundColor(self, values):
        val = values[0]
        date = forceDate(val)
        return QVariant(QtGui.QColor(0, 0, 0))

class CMyDesignationCol(CCol):

    def __init__(self, title, fields, designationChain, defaultWidth, alignment='l'):
        CCol.__init__(self, title, fields, defaultWidth, alignment)
        db = QtGui.qApp.db
        if type(designationChain) != list:
            designationChain = [designationChain]
        self._caches = []
        for tableName, fieldName in designationChain:
            self.DBColumnName = fieldName
            self._caches.append(CMyTableRecordCache(db, db.table(tableName), fieldName))


    def format(self, values):
        val = values[0]
        for cache in self._caches:
            id  = val.toInt()[0]
            record = cache.get(id)
            if not record:
                return CCol.invalid
            else:
                val = record.value(0)
        return val


    def invalidateRecordsCache(self):
        for cache in self._caches:
            cache.invalidate()


class CMyDoubleDesignationCol(CCol):
    def __init__(self, title, fields, designationChain, defaultWidth, alignment='l'):
        CCol.__init__(self, title, fields, defaultWidth, alignment)
        db = QtGui.qApp.db
        if type(designationChain) != list:
            designationChain = [designationChain]
        self._caches = []
        for tableName, fieldName, tableNameSecond, fieldNameSecond in designationChain:
            self.DBColumnName = fieldNameSecond
            self._caches.append((CMyTableRecordCache(db, db.table(tableName), fieldName),
                                CMyTableRecordCache(db, db.table(tableNameSecond), fieldNameSecond)))



    def format(self, values):
        val = values[0]
        for cache in self._caches:
            if val.type() == QtCore.QVariant.String:
                id = val.toString()
            elif val.type() == QtCore.QVariant.Int:
                id  = val.toInt()[0]
            record = cache[0].get(id)
            if not record:
                return CCol.invalid
            else:
                val = record.value(0)
                if val.type() == QtCore.QVariant.String:
                    id = val.toString()
                elif val.type() == QtCore.QVariant.Int:
                    id  = val.toInt()[0]
                record = cache[1].get(id)
                if not record:
                    return CCol.invalid
                else:
                    val = record.value(0)
        return val


    def invalidateRecordsCache(self):
        for cache in self._caches:
            cache[0].invalidate()
            cache[1].invalidate()
# ################MODELS############################
class CMyTableModel(CTableModel):
    def __init__(self, parent=None, cols=[], tableName=''):
        CTableModel.__init__(self, parent, cols, tableName)
        self.where = None
    def setTable(self, tableName):
        db = QtGui.qApp.db
        self._table = db.table(tableName)
        id = self.getIdName(tableName)
        self._table.fieldsDict['id'] =  self._table.fieldsDict[id]

        loadFields = [id] + self._loadFields
        for col in self._cols:
            loadFields = loadFields + col.fields()
        loadFields = set(loadFields)
        if '*' in loadFields:
            loadFields = '*'
        else:
            loadFields = ', '.join([self._table[fieldName].name() for fieldName in loadFields])
        self._recordsCache = CMyTableRecordCache(db, self._table, loadFields)
    def getIdName(self, tableName):
        if tableName == 'SMO':
            return 'ID_SMO'
        elif tableName == 'CASE_SERVICES_ACCOUNTS':
            return 'ID_ACCOUNT'

    def clear(self):
        pass

    def getRecordByRow(self, row):
        id = self._idList[row]
#        self._recordsCache.weakFetch(id, self._idList[max(0, row-self.fetchSize):(row+self.fetchSize)])
        return self._recordsCache.get(id)

    def getRecordValues(self, column, row):
        col = self._cols[column]
        if self._prevColumn != column or self._prevRow != row or self._prevData is None:
            id = self._idList[row]
#            self._recordsCache.weakFetch(id, self._idList[max(0, row-self.fetchSize):(row+self.fetchSize)])
            record = self._recordsCache.get(id)
            self._prevData   = col.extractValuesFromRecord(record)
            self._prevColumn = column
            self._prevRow    = row
        return (col, self._prevData)


class CPatientsModel(CMyTableModel):
    def __init__(self, parent=None):
        CMyTableModel.__init__(self, parent)
        self.addColumn(CTextCol(u'Фамилия',            ['SURNAME'],  16))
        self.addColumn(CTextCol(u'Имя',                ['NAME'],  16))
        self.addColumn(CTextCol(u'Отчество',           ['SECOND_NAME'],  16))
        self.addColumn(CDateCol(u'Дата рождения',      ['BIRTHDAY'],  8))
        self.addColumn(CTextCol(u'Пол',                ['SEX'],  3))
        self.addColumn(CTextCol(u'Документ: Серия(л)', ['SERIA_LEFT'],  16))
        self.addColumn(CTextCol(u'Документ: Серия(п)', ['SERIA_RIGHT'],  16))
        self.addColumn(CTextCol(u'Документ: Номер',    ['DOC_NUMBER'],  16))
        self.setTable('PATIENT_DATA')

    def getIdName(self, tableName='PATIENT_DATA'):
        return 'ID_PATIENT'

    def clear(self):
        pass

    def whoIAm(self):
        return 'patients'


class CCaseServicesAccountsTableModel(CMyTableModel):
    class CSummCol(CSumCol):

        def format(self, values):
            sum = forceDouble(values[0])
            sum = '%.2f' % sum
            return toVariant(sum)

    class CRefuseCol(CCol):
        def __init__(self, title, fields, defaultWidth, alignment='r'):
            CCol.__init__(self, title, fields, defaultWidth, alignment)
        def format(self, values):
#            self.setToolTip('REFUSES.TOTAL_SUMM')
            self.setWhatsThis('REFUSES.TOTAL_SUMM')
            val = values[0]
            db = QtGui.qApp.db
            stmt = 'SELECT TOTAL_SUMM FROM REFUSES WHERE REFUSES.ID_ACCOUNT = \'%s\'' % val.toString()
            query = db.query(stmt)
            if query.first():
                val = '%.2f' % query.value(0).toDouble()[0]
                return QVariant(val)
            else:
                return CCol.invalid
    class CRefuseActCol(CCol):
        def __init__(self, title, fields, defaultWidth, model, alignment='l'):
            CCol.__init__(self, title, fields, defaultWidth, alignment)
            self.model = model
        def format(self, values):
            self.setWhatsThis('REFUSES.ACT_NUMBER')
            val = values[0]
            check = self.checkExecDate(values[1].value('ID_ACCOUNT').toString())
            if check:
                db = QtGui.qApp.db
                stmt = 'SELECT ACT_NUMBER FROM REFUSES WHERE REFUSES.ID_ACCOUNT = \'%s\'' % val.toString()
                query = db.query(stmt)
                if query.first():
                    return query.value(0)
                else:
                    return CCol.invalid
            else:
                return CCol.invalid
        def checkExecDate(self, val):
            model = self.model
            row = model.idList().index(val)
            if model.data(model.index(row, 10)).toString() != '0.00':
                return True
            else:
                return False
    class CRefuseDateCol(CCol):
        def __init__(self, title, fields, defaultWidth, model, alignment='l'):
            CCol.__init__(self, title, fields, defaultWidth, alignment)
            self.model = model
        def format(self, values):
            self.setWhatsThis('REFUSES.ACT_NUMBER')
            val = values[0]
            check = self.checkExecDate(values[1].value('ID_ACCOUNT').toString())
            if check:
                db = QtGui.qApp.db
                stmt = 'SELECT ACT_DATE FROM REFUSES WHERE REFUSES.ID_ACCOUNT = \'%s\'' % val.toString()
                query = db.query(stmt)
                if query.first():
                    return query.value(0)
                else:
                    return CCol.invalid
            else:
                return CCol.invalid
        def checkExecDate(self, val):
            model = self.model
            row = model.idList().index(val)
            if model.data(model.index(row, 10)).toString() != '0.00':
                return True
            else:
                return False
    class CStatusCol(CCol):
        def __init__(self, title, fields, designationChain, defaultWidth, model, alignment='l'):
            CCol.__init__(self, title, fields, defaultWidth, alignment)
            db = QtGui.qApp.db
            if type(designationChain) != list:
                designationChain = [designationChain]
            self._caches = []
            self.cache = CRecordCache()
            for tableName, fieldName in designationChain:
                self.DBColumnName = fieldName
                self._caches.append(CMyTableRecordCache(db, db.table(tableName), fieldName))
            self.model = model

        def format(self, values):
#            self.setToolTip('STATUS_NAME')
            self.setWhatsThis('STATUS_NAME')
            val = values[0]
            for cache in self._caches:
                id  = val.toInt()[0]
                record = cache.get(id)
                check = self.checkExecDate(values[1].value('ID_ACCOUNT').toString())
                if check:
                    return QVariant(u'Оплачено')
                else:
                    if not record:
                        return CCol.invalid
                    else:
                        val = record.value(0)
            return val

        def invalidateRecordsCache(self):
            for cache in self._caches:
                cache.invalidate()
        def checkExecDate(self, val):
            model = self.model
            row = model.idList().index(val)
            if model.data(model.index(row, 9)).toString() != '':
                return True
            else:
                return False

    def __init__(self, parent=None):
        CMyTableModel.__init__(self, parent, [
#            CTextCol(u'Id Счета', ['ID_ACCOUNT'], 20),
            CTextCol(u'Номер счета', ['ACCOUNT_NUMBER'],  16),
            CMyDateCol(u'Дата создания', ['ACCOUNT_DATE'], 15),
            CMyDesignationCol(u'Код СМО', ['ID_PAYER'], ('SMO', 'CODE'), 10),
            CMyDesignationCol(u'Короткое имя СМО', ['ID_PAYER'], ('SMO', 'SMO_SHORT_NAME'), 10),
            self.CStatusCol(u'Статус', ['ID_STATUS'], ('VMU_ACC_STATUS', 'STATUS_NAME'), 200, self),
            CMyDateCol(u'Тарифный месяц', ['TARIFF_MONTH'], 15),
            self.CSummCol(u'Выставленая сумма', ['TOTAL_SUMM'], 15),
            CMyDoubleDesignationCol(u'Номер ПП', ['ID_ACCOUNT'],
                                    ('PAY_SUMMS', 'ID_PAY_CALC', 'PAY_SHEETS', 'PAY_NUMBER'), 15),
            CMyDoubleDesignationCol(u'Дата ПП', ['ID_ACCOUNT'],
                                    ('PAY_SUMMS', 'ID_PAY_CALC', 'PAY_SHEETS', 'PAY_DATE'), 15),
            CMyDoubleDesignationCol(u'Дата проводки', ['ID_ACCOUNT'],
                                    ('PAY_SUMMS', 'ID_PAY_CALC', 'PAY_SHEETS', 'EXECUTE_DATE'), 15),

            self.CRefuseCol(u'Отказы', ['ID_ACCOUNT'], 15.2),
            self.CRefuseActCol(u'№ Реестра отказов', ['ID_ACCOUNT'], 20, self),
            self.CRefuseDateCol(u'Дата реестра отказов', ['ID_ACCOUNT'], 15, self)

            ], 'CASE_SERVICES_ACCOUNTS')
        for col in self.cols():
            if isinstance(col, (CMyDesignationCol, CMyDoubleDesignationCol)):
                tt = col.DBColumnName
            else:
                tt = self.table().name()+'.'+col.fields()[0]
#            col.setToolTip(tt)
            col.setWhatsThis(tt)

    def getIdName(self, tableName='CASE_SERVICES_ACCOUNTS'):
        return 'ID_ACCOUNT'

    def clear(self):
        pass
    def whoIAm(self):
        return 'bill'


class CVisitsTableModel(CMyTableModel):
    def __init__(self, parent=None):
        CMyTableModel.__init__(self, parent)

#        self.addColumn(CTextCol(u'Id', ['ID_VISIT'], 20))
        self.addColumn(CTextCol(u'Код СМО', ['CODE'], 10))
        self.addColumn(CTextCol(u'Короткое имя СМО', ['SMO_SHORT_NAME'], 10))
        self.addColumn(CSumCol(u'Стоимость услуги', ['SERVICE_COST'], 15.2))
        self.addColumn(CMyDateCol(u'Дата начала', ['DATE_BEGIN'], 15))
        self.addColumn(CMyDateCol(u'Дата окончания', ['DATE_END'], 15))
#        self.addColumn(CNumCol(u'Количество ', ['VISIT_COUNT'], 3))

        self.setTable('VIEW_VISITS')

        for col in self.cols():
            if isinstance(col, (CMyDesignationCol, CMyDoubleDesignationCol)):
                tt = col.DBColumnName
            else:
                tt = col.fields()[0]
            col.setWhatsThis(tt)

    def getIdName(self, tableName='VIEW_VISITS'):
        return 'ID_VISIT'

    def clear(self):
        pass
    def whoIAm(self):
        return 'visits'



class CALRTableModel(CMyTableModel):

    def __init__(self, parent=None):
        CMyTableModel.__init__(self, parent)
        self.addColumn(CTextCol(u'Id Услуги', ['ID_SERVICE'], 20))
        self.addColumn(CTextCol(u'Id Счета', ['ID_ACCOUNT'], 20))
        self.addColumn(CNumCol(u'Стоимость услуги', ['SERVICE_COST'], 15.2))
        self.addColumn(CMyDoubleDesignationCol(u'Фамилия пациента', ['ID_CASE'],
                        ('CASES','ID_PATIENT','PATIENT_DATA', 'SURNAME'), 30))
        self.addColumn(CMyDoubleDesignationCol(u'Имя пациента', ['ID_CASE'],
                        ('CASES','ID_PATIENT', 'PATIENT_DATA', 'NAME'), 25))
        self.addColumn(CMyDoubleDesignationCol(u'Отчетсво пациента', ['ID_CASE'],
                        ('CASES', 'ID_PATIENT','PATIENT_DATA', 'SECOND_NAME'), 25))
        self.addColumn(CMyDesignationCol(u'Профиль', ['ID_PROFILE'], ('VMU_PROFILE', 'PROFILE_NAME'), 200))
        self.addColumn(CMyDesignationCol(u'Код профиля INFIS', ['ID_PROFILE'],('VMU_PROFILE', 'PROFILE_INFIS_CODE'),  10))
        self.addColumn(CMyDesignationCol(u'Диагноз', ['ID_DIAGNOSIS'],('VMU_DIAGNOSIS', 'DIAGNOSIS_NAME'),  255))
        self.addColumn(CMyDesignationCol(u'Код МКБ', ['ID_DIAGNOSIS'],('VMU_DIAGNOSIS', 'DIAGNOSIS_CODE'),  10))

        self.setTable('CASE_SERVICES')

    def getIdName(self, tableName='CASE_SERVICES'):
        return 'ID_SERVICE'

    def whoIAm(self):
        return 'VIEW_ALR'

class CStrictALRTableModel(CMyTableModel):
    class CTotalRefuseCol(CMyDesignationCol):

        def __init__(self, title, fields, designationChain, defaultWidth, alignment='r'):
            CMyDesignationCol.__init__(self, title, fields, designationChain, defaultWidth, alignment)

        def format(self, values):
            val = values[0]
            for cache in self._caches:
                id  = val.toInt()[0]
                record = cache.get(id)
                if not record:
                    return CCol.invalid
                else:
                    val = record.value(0)
                val = '%.2f' % val.toDouble()[0]
            return QVariant(forceDouble(val))

    class CSumCol(CCol):
        def __init__(self, title, fields, defaultWidth, alignment='r'):
            CCol.__init__(self, title, fields, defaultWidth, alignment)
        def format(self, values):
            sum = forceDouble('%10.2f'%values[0].toDouble()[0])
            return QVariant(sum)
    def __init__(self, parent=None):
        CMyTableModel.__init__(self, parent)

        self.addColumn(CTextCol(u'Id Услуги', ['ID_SERVICE'], 20))
        self.addColumn(CMyDesignationCol(u'Номер акта', ['ID_REFUSE'],('REFUSES', 'ACT_NUMBER'),  20))
        self.addColumn(CMyDesignationCol(u'Дата акта', ['ID_REFUSE'],('REFUSES', 'ACT_DATE'),  15))
        self.addColumn(self.CTotalRefuseCol(u'Общая сумма отказа по счету', ['ID_REFUSE'],('REFUSES', 'TOTAL_SUMM'),  15))
        self.addColumn(self.CSumCol(u'Сумма отказа', ['SUMM_REFUSE'], 15))
        self.addColumn(CTextCol(u'Примечание', ['REMARK'], 200))

        self.setTable('REFUSES_OBJ')

    def getIdName(self, tableName):
        return 'ID_SUMM'

    def whoIAm(self):
        return 'refusesObj'

class CPrintExpandedServices(CMyTableModel):
    class CPaySumCol(CCol):
        def __init__(self, title, fields, defaultWidth, model, alignment='l'):
            CCol.__init__(self, title, fields, defaultWidth, alignment)
            self.model = model
        def format(self, values):
            row = self.getRow(values[1])
            model = self.model
            return QtCore.QVariant(forceDouble(model.data(model.index(row, 2)))-forceDouble(model.data(model.index(row, 3))))
        def getRow(self, record):
            return self.model.idList().index(record.value(5).toString())
    class CRefuseCol(CCol):
        def __init__(self, title, fields, defaultWidth, alignment='l'):
            CCol.__init__(self, title, fields, defaultWidth, alignment)
        def format(self, values):
            val = values[0]
            db = QtGui.qApp.db
            stmt = 'SELECT SUMM_REFUSE FROM REFUSES_OBJ WHERE REFUSES_OBJ.ID_SERVICE = \'%s\'' % val.toString()
            query = db.query(stmt)
            if query.first():
                return query.value(0)
            else:
                return CCol.invalid

    def __init__(self, parent=None):
        CMyTableModel.__init__(self, parent)
        self.addColumn(CTextCol(u'Id Услуги', ['ID_SERVICE'], 20))
        self.addColumn(CTextCol(u'Id Счета', ['ID_ACCOUNT'], 20))
        self.addColumn(CNumCol(u'Стоимость услуги', ['SERVICE_COST'], 15.2))
        self.addColumn(self.CRefuseCol(u'Сумма отказа', ['ID_SERVICE'], 15.2))
        self.addColumn(self.CPaySumCol(u'Оплачено', ['ID_SERVICE'], 15.2, self))
        self.addColumn(CMyDoubleDesignationCol(u'Фамилия пациента', ['ID_CASE'],
                        ('CASES','ID_PATIENT','PATIENT_DATA', 'SURNAME'), 30))
        self.addColumn(CMyDoubleDesignationCol(u'Имя пациента', ['ID_CASE'],
                        ('CASES','ID_PATIENT', 'PATIENT_DATA', 'NAME'), 25))
        self.addColumn(CMyDoubleDesignationCol(u'Отчетсво пациента', ['ID_CASE'],
                        ('CASES', 'ID_PATIENT','PATIENT_DATA', 'SECOND_NAME'), 25))
        self.addColumn(CMyDesignationCol(u'Профиль', ['ID_PROFILE'], ('VMU_PROFILE', 'PROFILE_NAME'), 200))
        self.addColumn(CMyDesignationCol(u'Код профиля INFIS', ['ID_PROFILE'],('VMU_PROFILE', 'PROFILE_INFIS_CODE'),  10))
        self.addColumn(CMyDesignationCol(u'Диагноз', ['ID_DIAGNOSIS'],('VMU_DIAGNOSIS', 'DIAGNOSIS_NAME'),  255))

        self.setTable('CASE_SERVICES')

    def getIdName(self, tableName='CASE_SERVICES'):
        return 'ID_SERVICE'

    def whoIAm(self):
        return 'VIEW_ALR'

# #####################################################################

recordCapacity = 300

class CProfileTableModel(CMyTableModel):
    class CNameProfile(CCol):
        def __init__(self, title, fields, defaultWidth, alignment='l'):
            CCol.__init__(self, title, fields, defaultWidth, alignment)

        def format(self, row):
            ksgGroups = QtGui.qApp.ksgGroups
            if ksgGroups.keys()[row] == u'Центре здоровья':
                return QVariant(u'Центр здоровья')
            return QVariant(ksgGroups.keys()[row])

    class CCountCol(CCol):
        def __init__(self, title, fields, where, defaultWidth, dDateCol=None, model=None, alignment='r'):
            CCol.__init__(self, title, fields, defaultWidth, alignment)
            self.calcTable = 'CASE_SERVICES'
            self.condField = 'ID_PROFILE'
            self.cache = CRecordCache(recordCapacity)
            self.model = model
            self.dDateCol = dDateCol
            if where:
                self.where = where
            else:
                self.where = ''

        def format(self, row):
            dDateCol=''
            if self.dDateCol:
                dDateCol = 'CASE_SERVICES_ACCOUNTS.'+self.dDateCol
            ksgGroups = QtGui.qApp.ksgGroups
            record = self.cache.get(row)
            if not record:
                dFrom = QtGui.qApp.mainWindow.edtDateFrom.date().toString('dd.MM.yyyy')
                dTo   = QtGui.qApp.mainWindow.edtDateTo.date().toString('dd.MM.yyyy')
                where2 = 'AND %s >= \'%s\' AND %s <= \'%s\' '%(dDateCol, dFrom, dDateCol, dTo)
                resume = 0
                for idProfile in ksgGroups[ksgGroups.keys()[row]]:
                    self.condField = 'ID_PROFILE'
                    db = QtGui.qApp.db
                    if len(self.where.split('WHERE')) > 2:
                            where = ' AND '+self.where.split('WHERE')[1]+' WHERE '+self.where.split('WHERE')[2]
#                            where = where % idProfile
                    else:
                        where = ' AND '+self.where.split('WHERE')[1]
#                        where = where % idProfile
                    if not self.dDateCol:
                        where2 = where
                    stmt = 'SELECT COUNT(*) FROM %s JOIN CASE_SERVICES_ACCOUNTS ON CASE_SERVICES_ACCOUNTS.ID_ACCOUNT = CASE_SERVICES.ID_ACCOUNT WHERE ID_PROFILE=%d %s' % (self.calcTable, idProfile,  where2)
                    query = db.query(stmt)
                    resume += firstQueryValueInt(query)
                self.cache.put(row, QVariant(resume))
                return QVariant(resume)
            else:
                return record




    class CSumServiceCost(CCol):
        def __init__(self, title, fields, where, defaultWidth, dDateCol=None, alignment='r', parent=None):
            CCol.__init__(self, title, fields, defaultWidth, alignment)
            self.cache = CRecordCache(recordCapacity)
            self.where = where
            self.dDateCol = dDateCol
        def format(self, row):
            dDateCol=''
            if self.dDateCol:
                dDateCol = 'CASE_SERVICES_ACCOUNTS.'+self.dDateCol
            ksgGroups = QtGui.qApp.ksgGroups
            record = self.cache.get(row)
            if not record:
                dFrom = QtGui.qApp.mainWindow.edtDateFrom.date().toString('dd.MM.yyyy')
                dTo   = QtGui.qApp.mainWindow.edtDateTo.date().toString('dd.MM.yyyy')
                where2 = 'AND %s >= \'%s\' AND %s <= \'%s\' '%(dDateCol, dFrom, dDateCol, dTo)
                resume = 0
                for idProfile in ksgGroups[ksgGroups.keys()[row]]:
                    db = QtGui.qApp.db
                    if len(self.where.split('WHERE')) > 2:
                        where = ' AND '+self.where.split('WHERE')[1]+' WHERE '+self.where.split('WHERE')[2]
                    else:
                        where = ' AND '+self.where.split('WHERE')[1]
                    if not self.dDateCol:
                        where2 = where
                    stmt = 'SELECT SUM(SERVICE_COST) FROM CASE_SERVICES JOIN CASE_SERVICES_ACCOUNTS ON CASE_SERVICES_ACCOUNTS.ID_ACCOUNT = CASE_SERVICES.ID_ACCOUNT WHERE ID_PROFILE=%d %s' % (idProfile, where2)
#                    print stmt
                    query = db.query(stmt)
                    resume += firstQueryValueDouble(query)
                val = '%.2f' % resume
                self.cache.put(row, QVariant(val))
                return QVariant(val)
            else:
                return record


    class CSumRefuse(CCol):
        def __init__(self, title, fields, where, defaultWidth, dDateCol=None, alignment='r', parent=None):
            CCol.__init__(self, title, fields, defaultWidth, alignment)
            self.cache = CRecordCache(recordCapacity)
            self.where = where
            self.dDateCol = dDateCol
        def format(self, row):
            dDateCol=''
            if self.dDateCol:
                dDateCol = 'CASE_SERVICES_ACCOUNTS.'+self.dDateCol
            dRefDateCol = 'REFUSES_OBJ.REFUSE_DATE'
            ksgGroups = QtGui.qApp.ksgGroups
            record = self.cache.get(row)
            if not record:
                dFrom = QtGui.qApp.mainWindow.edtDateFrom.date().toString('dd.MM.yyyy')
                dTo   = QtGui.qApp.mainWindow.edtDateTo.date().toString('dd.MM.yyyy')
                date = '01.01.'+dFrom.split('.')[2]
                where2 = 'AND %s >= \'%s\' AND %s <= \'%s\' '%(dDateCol, dFrom, dDateCol, dTo)
#                where3 = ' AND %s >= \'%s\' ' %(dRefDateCol, date)
#                where2 += where3
                resume = 0
                for idProfile in ksgGroups[ksgGroups.keys()[row]]:
                    db = QtGui.qApp.db
                    if len(self.where.split('WHERE')) > 2:
                        where = ' AND '+self.where.split('WHERE')[1]+' WHERE '+self.where.split('WHERE')[2]
                    else:
                        where = ' AND '+self.where.split('WHERE')[1]
                    if not self.dDateCol:
                        where2 = where
                    stmt = 'SELECT SUM(REFUSES_OBJ.SUMM_REFUSE) FROM REFUSES_OBJ JOIN CASE_SERVICES ON REFUSES_OBJ.ID_SERVICE = CASE_SERVICES.ID_SERVICE JOIN CASE_SERVICES_ACCOUNTS ON CASE_SERVICES_ACCOUNTS.ID_ACCOUNT=CASE_SERVICES.ID_ACCOUNT WHERE ID_PROFILE=%d %s' % (idProfile, where2)
#                    print stmt
                    query = db.query(stmt)
                    resume += firstQueryValueDouble(query)
                val = '%.2f' % resume
                self.cache.put(row, QVariant(val))
                return QVariant(val)
            else:
                return record

    class CSumServiceCostClosed2(CCol):
        def __init__(self, title, fields,  defaultWidth, model):
            CCol.__init__(self, title, fields, defaultWidth, 'l')
            self.model = model

        def format(self, row):
            ts = self.model.data(self.model.index(row, 2)).toDouble()[0]
            rf = self.model.data(self.model.index(row, 3)).toDouble()[0]
            val = '%.2f' % (ts-rf)
            return QVariant(val)

    class CSumServiceCostClosed(CCol):
        def __init__(self, title, fields, where, defaultWidth, dDateCol=None, model=None, alignment='r', parent=None):
            CCol.__init__(self, title, fields, defaultWidth, alignment)
            self.cache = CRecordCache(recordCapacity)
            self.where = where
            self.model = model
            self.dDateCol = dDateCol

        def format(self, row):
            dDateCol=''
            if self.dDateCol:
                dDateCol = 'CASE_SERVICES_ACCOUNTS.'+self.dDateCol
            dPPDateCol = 'PAY_SHEETS.EXECUTE_DATE'
            ksgGroups = QtGui.qApp.ksgGroups
            record = self.cache.get(row)
            if not record:
                dFrom = QtGui.qApp.mainWindow.edtDateFrom.date().toString('dd.MM.yyyy')
                dTo   = QtGui.qApp.mainWindow.edtDateTo.date().toString('dd.MM.yyyy')
                where2 = ' AND %s >= \'%s\' AND %s <= \'%s\' '%(dDateCol, dFrom, dDateCol, dTo)
#                where3 = ' WHERE %s >= \'%s\' AND %s <= \'%s\' '%(dPPDateCol, dFrom, dPPDateCol, dTo)
                where3 = ' WHERE %s IS NOT NULL ' % dPPDateCol
#                where2 += where3
                resume = 0
                for idProfile in ksgGroups[ksgGroups.keys()[row]]:
                    db = QtGui.qApp.db
                    if len(self.where.split('WHERE')) > 2:
                        where = ' AND '+self.where.split('WHERE')[1]+' WHERE '+self.where.split('WHERE')[2]
                    else:
                        where = ' AND '+self.where.split('WHERE')[1]
                    if not self.dDateCol:
                        where2 = where
                    stmt = 'SELECT SUM(CASE_SERVICES.SERVICE_COST) FROM CASE_SERVICES_ACCOUNTS JOIN CASE_SERVICES ON CASE_SERVICES.ID_ACCOUNT = CASE_SERVICES_ACCOUNTS.ID_ACCOUNT  WHERE ID_PROFILE=%d  %s AND CASE_SERVICES_ACCOUNTS.ID_ACCOUNT IN (SELECT DISTINCT ID_ACCOUNT FROM PAY_SUMMS JOIN PAY_SHEETS ON PAY_SHEETS.ID_PAY_CALC=PAY_SUMMS.ID_PAY_CALC %s )' % (idProfile, where2, where3)
#                    print stmt
                    query = db.query(stmt)
                    resume += firstQueryValueDouble(query)
                resume -= self.model.data(self.model.index(row, 3)).toDouble()[0]
                val = '%.2f' % resume
                self.cache.put(row, QVariant(val))
                return QVariant(val)

            else:
                return record

    def __init__(self, parent=None):
        CMyTableModel.__init__(self, parent)
        self.addColumn(CProfileTableModel.CNameProfile(u'Профиль', [], 15))
        self.numKsgHeader = []
        self.dDateCol = None
        self.setTable('VMU_PROFILE')
    def setDDateCol(self, dDateCol):
        self.dDateCol = dDateCol
    def setWhere(self, where):
        self.where = where
        self.addColumn(CProfileTableModel.CCountCol(u'Количество случаев', [], self.where, 8, self.dDateCol, self))
        self.addColumn(CProfileTableModel.CSumServiceCost(u'Выставленная сумма', [], self.where, 15, self.dDateCol))
        self.addColumn(CProfileTableModel.CSumRefuse(u'Сумма отказов', [], self.where, 15, self.dDateCol))
#        self.addColumn(CProfileTableModel.CSumServiceCostClosed(u'Оплаченная сумма', [], self.where, 15, self.dDateCol, self))
        self.addColumn(CProfileTableModel.CSumServiceCostClosed2(u'Сумма отказов', ['ID_PROFILE'], 8, self))
    def getIdName(self, tableName):
        return 'ID_PROFILE'

    def whoIAm(self):
        return 'profile'

    def data(self, index, role=Qt.DisplayRole):
        if not index.isValid():
            return QVariant()
        column = index.column()
        row    = index.row()
        if role == Qt.DisplayRole: ### or role == Qt.EditRole:
            col = self._cols[column]
            return col.format(row)
        elif role == Qt.TextAlignmentRole:
            col = self._cols[column]
            return col.alignment()
        elif role == Qt.CheckStateRole:
            col = self._cols[column]
            return col.invalid
        return QVariant()


class CKSGGroupsTableModel(CMyTableModel):
    class CDaysCountColVisit_Count(CCol):
        field = 'VISIT_COUNT'
        def __init__(self, title, fields, calcTable, calcField, where, defaultWidth, dDateCol, model, alignment='r'):
            CCol.__init__(self, title, fields, defaultWidth, alignment)
            self.calcTable = calcTable
            self.calcField = calcField
            self.condField = fields[0]
            self.dDateCol = dDateCol
            self.model = model
            self.cache = CRecordCache(recordCapacity)
            if where:
                self.where = where
            else:
                self.where = ''

        def format(self, values):
            dDateCol = ''
            if self.dDateCol:
                dDateCol = 'CASE_SERVICES_ACCOUNTS.'+self.dDateCol
            idProfile = values[0].toInt()[0]
            record = self.cache.get(idProfile)
            if not record:
                dFrom = QtGui.qApp.mainWindow.edtDateFrom.date().toString('dd.MM.yyyy')
                dTo   = QtGui.qApp.mainWindow.edtDateTo.date().toString('dd.MM.yyyy')
                where2 = 'AND %s >= \'%s\' AND %s <= \'%s\' '%(dDateCol, dFrom, dDateCol, dTo)
                db = QtGui.qApp.db
                if len(self.where.split('WHERE')) > 2:
                        where = ' AND '+self.where.split('WHERE')[1]+' WHERE '+self.where.split('WHERE')[2]
                else:
                    where = ' AND '+self.where.split('WHERE')[1]
                if not self.dDateCol:
                    where2 = where
                stmt = 'SELECT SUM(CASE_SERVICES.'+self.field+') FROM %s JOIN CASE_SERVICES_ACCOUNTS ON CASE_SERVICES_ACCOUNTS.ID_ACCOUNT = CASE_SERVICES.ID_ACCOUNT WHERE ID_PROFILE=%d %s' % (self.calcTable, idProfile, where2)
                query = db.query(stmt)
                if query.first():
                    if query.value(0).isNull():
                        row = self.getRow(values[1])
                        model = self.model
                        val = model.data(model.index(row, 2))
                    else:
                        val = query.value(0)
                    self.cache.put(idProfile, val)
                    return val
                else:
                    self.cache.put(idProfile, self.invalid)
                    return self.invalid
            else:
                return record

        def getRow(self, record):
            return self.model.idList().index(record.value(0).toInt()[0])

    class CDaysCountColService_Count(CDaysCountColVisit_Count):
        field = 'SERVICE_COUNT'

    class CDaysCountSecondCol(CCol):
        def __init__(self, title, fields, calcTable, calcField, where, defaultWidth, alignment='r'):
            CCol.__init__(self, title, fields, defaultWidth, alignment)
            self.calcTable = calcTable
            self.calcField = calcField
            self.condField = fields[0]
            self.cache = CRecordCache(recordCapacity)
            if where:
                self.where = where
            else:
                self.where = ''

        def format(self, values):
            idProfile = values[0].toInt()[0]
            record = self.cache.get(idProfiel)
            if not record:
                db = QtGui.qApp.db
                if len(self.where.split('WHERE')) > 2:
                        where = 'WHERE ID_PROFILE=%d AND '+self.where.split('WHERE')[1]+' WHERE '+self.where.split('WHERE')[2]
                        where = where % idProfile
                else:
                    where = 'WHERE ID_PROFILE=%d AND '+self.where.split('WHERE')[1]
                    where = where % idProfile
                stmt = 'SELECT COUNT(*) FROM %s  %s' % (self.calcTable, where)
                stmtWithParents = 'SELECT ID_PARENT_SERVICES FROM %s %s AND ID_PARENT_SERVICES IS NOT NULL' % (self.calcTable, where)
                idListParents = makeListFromQuery(db.query(stmtWithParents))
                parentCount = self.getParentsCount(self.calcTable, where, idListParents)
                query = db.query(stmt)
                if query.first():
                    count = QVariant(parentCount+query.value(0).toInt()[0])
                    self.cache.put(idProfile, count)
                    return count
                else:
                    self.cache.put(idProfile, self.invalid)
                    return self.invalid
            else:
                return record

        def getParentsCount(self, calcTable, where, list):
            count = len(list)
            if count:
                db = QtGui.qApp.db
                for i in list:
                    stmt = 'SELECT ID_PARENT_SERVICES FROM %s %s AND ID_PARENT_SERVICES IS NOT NULL AND ID_SERVICE=\'%s\'' % (calcTable, where, i)
                    idListParents = makeListFromQuery(db.query(stmt))
                    count += self.getParentsCount(calcTable, where, idListParents)
            return count


    class CCountCol(CCol):
        def __init__(self, title, fields, calcTable, calcField, where, defaultWidth, dDateCol, model=None, alignment='r'):
            CCol.__init__(self, title, fields, defaultWidth, alignment)
            self.calcTable = calcTable
            self.calcField = calcField
            self.condField = fields[0]
            self.dDateCol = dDateCol
            self.cache = CRecordCache(recordCapacity)
            self.model = model
            if where:
                self.where = where
            else:
                self.where = ''

        def format(self, values):
            dDateCol = ''
            if self.dDateCol:
                dDateCol = 'CASE_SERVICES_ACCOUNTS.'+self.dDateCol
            idProfile = values[0].toInt()[0]
            record = self.cache.get(idProfile)
            if not record:
                dFrom = QtGui.qApp.mainWindow.edtDateFrom.date().toString('dd.MM.yyyy')
                dTo   = QtGui.qApp.mainWindow.edtDateTo.date().toString('dd.MM.yyyy')
                where2 = 'AND %s >= \'%s\' AND %s <= \'%s\' '%(dDateCol, dFrom, dDateCol, dTo)
                db = QtGui.qApp.db
                if len(self.where.split('WHERE')) > 2:
                        where = ' AND '+self.where.split('WHERE')[1]+' WHERE '+self.where.split('WHERE')[2]
                else:
                    where = ' AND '+self.where.split('WHERE')[1]
                if not self.dDateCol:
                    where2 = where
                stmt = 'SELECT COUNT(*) FROM %s JOIN CASE_SERVICES_ACCOUNTS ON CASE_SERVICES_ACCOUNTS.ID_ACCOUNT = CASE_SERVICES.ID_ACCOUNT WHERE ID_PROFILE=%d %s' % (self.calcTable, idProfile, where2)
                query = db.query(stmt)
                if query.first():
                    self.cache.put(idProfile, query.value(0))
                    return query.value(0)
                else:
                    self.cache.put(idProfile, self.invalid)
                    return self.invalid
            else:
                return record



    class CSumRefuse(CCol):
        def __init__(self, title, fields, where, defaultWidth, dDateCol, alignment='r', parent=None):
            CCol.__init__(self, title, fields, defaultWidth, alignment)
            self.cache = CRecordCache(recordCapacity)
            self.where = where
            self.dDateCol = dDateCol
        def format(self, values):
            dDateCol = ''
            if self.dDateCol:
                dDateCol = 'CASE_SERVICES_ACCOUNTS.'+self.dDateCol
            dRefDateCol = 'REFUSES_OBJ.REFUSE_DATE'
            idProfile = values[0].toInt()[0]
            record = self.cache.get(idProfile)
            if not record:
                dFrom = QtGui.qApp.mainWindow.edtDateFrom.date().toString('dd.MM.yyyy')
                dTo   = QtGui.qApp.mainWindow.edtDateTo.date().toString('dd.MM.yyyy')
                date = '01.01.'+dFrom.split('.')[2]
                where2 = 'AND %s >= \'%s\' AND %s <= \'%s\' '%(dDateCol, dFrom, dDateCol, dTo)
#                where3 = ' AND %s >= \'%s\' ' %(dRefDateCol, date)
#                where2 += where3
                db = QtGui.qApp.db
                if len(self.where.split('WHERE')) > 2:
                    where = ' AND '+self.where.split('WHERE')[1]+' WHERE '+self.where.split('WHERE')[2]
                else:
                    where = ' AND '+self.where.split('WHERE')[1]
                if not self.dDateCol:
                    where2 = where
                stmt = 'SELECT SUM(REFUSES_OBJ.SUMM_REFUSE) FROM REFUSES_OBJ JOIN CASE_SERVICES ON REFUSES_OBJ.ID_SERVICE = CASE_SERVICES.ID_SERVICE JOIN CASE_SERVICES_ACCOUNTS ON CASE_SERVICES_ACCOUNTS.ID_ACCOUNT=CASE_SERVICES.ID_ACCOUNT WHERE ID_PROFILE=%d %s' % (idProfile, where2)
                query = db.query(stmt)
                if query.first():
                    val = '%.2f' % query.value(0).toDouble()[0]
                    val = QVariant(val)
                    self.cache.put(idProfile, val)
                    return val
                else:
                    self.cache.put(idProfile, self.invalid)
                    return self.invalid
            else:
                return record

    class CSumServiceCost(CCol):
        def __init__(self, title, fields, where, defaultWidth, dDateCol, alignment='r', parent=None):
            CCol.__init__(self, title, fields, defaultWidth, alignment)
            self.cache = CRecordCache(recordCapacity)
            self.where = where
            self.dDateCol = dDateCol
        def format(self, values):
            idProfile = values[0].toInt()[0]
            record = self.cache.get(idProfile)
            dDateCol = ''
            if self.dDateCol:
                dDateCol = 'CASE_SERVICES_ACCOUNTS.'+self.dDateCol
            if not record:
                dFrom = QtGui.qApp.mainWindow.edtDateFrom.date().toString('dd.MM.yyyy')
                dTo   = QtGui.qApp.mainWindow.edtDateTo.date().toString('dd.MM.yyyy')
                where2 = 'AND %s >= \'%s\' AND %s <= \'%s\' '%(dDateCol, dFrom, dDateCol, dTo)
                db = QtGui.qApp.db
                if len(self.where.split('WHERE')) > 2:
                    where = ' AND '+self.where.split('WHERE')[1]+' WHERE '+self.where.split('WHERE')[2]
                else:
                    where = ' AND '+self.where.split('WHERE')[1]
                if not self.dDateCol:
                    where2 = where
                stmt = 'SELECT SUM(SERVICE_COST) FROM CASE_SERVICES JOIN CASE_SERVICES_ACCOUNTS ON CASE_SERVICES_ACCOUNTS.ID_ACCOUNT = CASE_SERVICES.ID_ACCOUNT WHERE ID_PROFILE=%d %s' % (idProfile, where2)
                query = db.query(stmt)
                if query.first():
                    val = '%.2f' % query.value(0).toDouble()[0]
                    val = QVariant(val)
                    self.cache.put(idProfile, val)
                    return val
                else:
                    self.cache.put(idProfile, self.invalid)
                    return self.invalid
            else:
                return record

    class CSumServiceCostClosed2(CCol):
        def __init__(self, title, fields,  defaultWidth, model):
            CCol.__init__(self, title, fields, defaultWidth, 'l')
            self.model = model

        def format(self, values):
            row = self.getRow(values[1])
            ts = self.model.data(self.model.index(row, 4)).toDouble()[0]
            rf = self.model.data(self.model.index(row, 5)).toDouble()[0]
            val = '%.2f' % (ts-rf)
            return QVariant(val)

        def getRow(self, record):
            return self.model.idList().index(record.value(0).toInt()[0])

    class CSumServiceCostClosed(CCol):
        def __init__(self, title, fields, where, defaultWidth, dDateCol, model, alignment='r', parent=None):
            CCol.__init__(self, title, fields, defaultWidth, alignment)
            self.cache = CRecordCache(recordCapacity)
            self.where = where
            self.model = model
            self.dDateCol = dDateCol
        def format(self, values):
            dDateCol = ''
            if self.dDateCol:
                dDateCol = 'CASE_SERVICES_ACCOUNTS.'+self.dDateCol
            dPPDateCol = 'PAY_SHEETS.EXECUTE_DATE'
            idProfile = values[0].toInt()[0]
            record = self.cache.get(idProfile)
            if not record:
                dFrom = QtGui.qApp.mainWindow.edtDateFrom.date().toString('dd.MM.yyyy')
                dTo   = QtGui.qApp.mainWindow.edtDateTo.date().toString('dd.MM.yyyy')
                where2 = 'AND %s >= \'%s\' AND %s <= \'%s\' '%(dDateCol, dFrom, dDateCol, dTo)
#                where3 = ' WHERE %s >= \'%s\' AND %s <= \'%s\' '%(dPPDateCol, dFrom, dPPDateCol, dTo)
                where3 = ' WHERE %s IS NOT NULL ' % dPPDateCol
                db = QtGui.qApp.db
                if len(self.where.split('WHERE')) > 2:
                    where = ' AND '+self.where.split('WHERE')[1]+' WHERE '+self.where.split('WHERE')[2]
                else:
                    where = ' AND '+self.where.split('WHERE')[1]
                if not self.dDateCol:
                    where2 = where
                stmt = 'SELECT SUM(CASE_SERVICES.SERVICE_COST) FROM CASE_SERVICES_ACCOUNTS JOIN CASE_SERVICES ON CASE_SERVICES.ID_ACCOUNT = CASE_SERVICES_ACCOUNTS.ID_ACCOUNT  WHERE ID_PROFILE=%d  %s AND CASE_SERVICES_ACCOUNTS.ID_ACCOUNT IN (SELECT DISTINCT ID_ACCOUNT FROM PAY_SUMMS JOIN PAY_SHEETS ON PAY_SHEETS.ID_PAY_CALC=PAY_SUMMS.ID_PAY_CALC %s )' % (idProfile, where2, where3)
                row = self.getRow(values[1])
#                print stmt
                query = db.query(stmt)
                if query.first():
                    val = '%.2f' % (query.value(0).toDouble()[0])
                    val = '%.2f' % (forceDouble(val)-self.model.data(self.model.index(row, 5)).toDouble()[0])
                    val = QVariant(val)
                    self.cache.put(idProfile, val)
                    return val
                else:
                    self.cache.put(idProfile, self.invalid)
                    return self.invalid
            else:
                return record
        def getRow(self, record):
            return self.model.idList().index(record.value(0).toInt()[0])


    def __init__(self, parent=None):
        CMyTableModel.__init__(self, parent)
        self.groupCount = 0
        self.headerCache = CRecordCache(recordCapacity)
        self.dDateCol = None
        self.addColumn(CTextCol(u'Код КСГ', ['PROFILE_INFIS_CODE'], 10))
        self.addColumn(CTextCol(u'Название КСГ', ['PROFILE_NAME'], 200))
        self.setTable('VMU_PROFILE')

    def setDDateCol(self, dDateCol):
        self.dDateCol = dDateCol

    def setWhere(self, where):
        self.where = where
        dDateCol = self.dDateCol
        self.addColumn(CKSGGroupsTableModel.CCountCol(u'Количество случаев', ['ID_PROFILE'], 'CASE_SERVICES', 'ID_SERVICE', self.where, 8, dDateCol, self))
        self.addColumn(CKSGGroupsTableModel.CDaysCountColVisit_Count(u'Количество визитов', ['ID_PROFILE'], 'CASE_SERVICES', 'ID_SERVICE', self.where, 12, dDateCol, self))
#        self.addColumn(CKSGGroupsTableModel.CDaysCountColService_Count(u'Количество услуг', ['ID_PROFILE'], 'CASE_SERVICES', 'ID_SERVICE', self.where, 12, dDateCol, self))
        self.addColumn(CKSGGroupsTableModel.CSumServiceCost(u'Общая сумма', ['ID_PROFILE'], self.where, 15.2, dDateCol))
        self.addColumn(CKSGGroupsTableModel.CSumRefuse(u'Сумма отказов', ['ID_PROFILE'], self.where, 15.2, dDateCol))
#        self.addColumn(CKSGGroupsTableModel.CSumServiceCostClosed(u'Оплаченная сумма', ['ID_PROFILE'], self.where, 15.2, dDateCol, self))
        self.addColumn(CKSGGroupsTableModel.CSumServiceCostClosed2(u'Сумма отказов', ['ID_PROFILE'], 8, self))


    def getIdName(self, tableName='VMU_PROFILE'):
        return 'ID_PROFILE'
    def whoIAm(self):
        return 'ksg'


    def setIdList(self, idList, realItemCount=None):
        self._idList = idList
        self._realItemCount = realItemCount
        self._prevColumn  = None
        self._prevRow     = None
        self._prevData    = None
        self.invalidateRecordsCache()
        self.reset()
        self.emitItemsCountChanged()
        self.ksgGroups     = QtGui.qApp.ksgGroups
        self.groups        = [i for i in self.ksgGroups.keys()]
        self.groupsLength  = self.getGroupLength()
        self.groupCount    = len(self.groups)
        self.numKsgHeader  = self.getNumKsgHeader()
        self.currentGroup  = 0
        self.previousGroup = None
        self.previousGroupsRows = 0

    def getGroupLength(self):
        group = {}
        for i in self.ksgGroups.keys():
            group[i] = len(self.ksgGroups[i])
        return group

    def getNumKsgHeader(self):
        resume = [0]
        rows = 0
        for i in self.ksgGroups.values():
            rows += len(i)+1
            resume += [rows]
        return resume
    def toKnowNumKsgGroup(self, row):
        num = -1
        for i in self.numKsgHeader:
            if row >= i:
                num += 1
        return num

    def getHeaderValue(self, row, column):
        value = self.headerCache.get(row)
        if not value:
            value = [None, None, None, None, None, None, None]
            val = 0
            for i in range(row+1, self.groupsLength[self.groups[self.currentGroup]]+row+1):
                v = self.data(self.index(i, column))
                if v:
                    if column in (2, 3):
                        val += v.toInt()[0]
                    elif column in (4, 5, 6):
                        val += v.toDouble()[0]
                    elif column == 0:
                        val = u'Позиций: '+forceString(self.groupsLength[self.groups[self.currentGroup]])
                        break
            value[column] = val
            self.headerCache.put(row, value)
        else:
            if not value[column]:
                val = 0
                for i in range(row+1, self.groupsLength[self.groups[self.currentGroup]]+row+1):
                    v = self.data(self.index(i, column))
                    if v:
                        if column in (2, 3):
                            val += v.toInt()[0]
                        elif column in (4, 5, 6):
                            val += v.toDouble()[0]
                        elif column == 0:
                            val = u'Позиций: '+forceString(self.groupsLength[self.groups[self.currentGroup]])
                            break
                value[column] = val
                self.headerCache.put(row, value)
        return value[column]

    def data(self, index, role=Qt.DisplayRole):
        if not index.isValid():
            return QVariant()
        column = index.column()
        row    = index.row()
        if not self.previousGroup:
            self.previousGroupsRows = row
            self.previousGroup = self.currentGroup
        self.currentGroup = self.toKnowNumKsgGroup(row)
        if row in self.numKsgHeader:
            if column == 1 and role == Qt.DisplayRole:
                return QVariant(self.groups[self.currentGroup])
            elif column in (0, 2, 3, 4, 5, 6) and role == Qt.DisplayRole:
                return QVariant(self.getHeaderValue(row, column))
            else:
                return QVariant()
        else:
            self.previousGroup = None
            if role == Qt.DisplayRole: ### or role == Qt.EditRole:
                (col, values) = self.getRecordValues(column, row)
                return col.format(values)
            elif role == Qt.TextAlignmentRole:
               col = self._cols[column]
               return col.alignment()
            elif role == Qt.CheckStateRole:
                (col, values) = self.getRecordValues(column, row)
                return col.checked(values)
            return QVariant()

    def getRecordValues(self, column, row):
        col = self._cols[column]
        if self._prevColumn != column or self._prevRow != row or self._prevData is None:
            ksgList = self.ksgGroups[self.groups[self.currentGroup]]
            ksg = ksgList[row-(self.numKsgHeader[self.currentGroup]+1)]
            record = self._recordsCache.get(ksg)
            self._prevData   = col.extractValuesFromRecord(record)
            self._prevColumn = column
            self._prevRow    = row
        return (col, self._prevData)


class CRefusesTableModel(CMyTableModel):
    class CCountCol(CCol):
        def __init__(self, title, fields, defaultWidth, where, alignment='r', parent=None):
            CCol.__init__(self, title, fields, defaultWidth, alignment)
            self.cache = CRecordCache()
            self.where = where

        def format(self, values):
            idRefReason = values[0].toInt()[0]
            record = self.cache.get(idRefReason)
            where = self.where.split('WHERE')[1]
            if not record:
                resume = 0
                db = QtGui.qApp.db
                stmt = 'SELECT COUNT(CASE_SERVICES.ID_SERVICE) FROM CASE_SERVICES JOIN REFUSES_OBJ ON REFUSES_OBJ.ID_SERVICE=CASE_SERVICES.ID_SERVICE JOIN CASE_SERVICES_ACCOUNTS ON CASE_SERVICES_ACCOUNTS.ID_ACCOUNT=CASE_SERVICES.ID_ACCOUNT JOIN SMO ON CASE_SERVICES_ACCOUNTS.ID_PAYER = SMO.ID_SMO WHERE REFUSES_OBJ.ID_REF_REASON=%d AND %s' % (idRefReason, where)
                query = db.query(stmt)
                val = firstQueryValue(query)
                self.cache.put(idRefReason, val)
                return val

            else:
                return record

    class CSumRefuseCol(CCol):
        def __init__(self, title, fields, defaultWidth, where, alignment='r', parent=None):
            CCol.__init__(self, title, fields, defaultWidth, alignment)
            self.cache = CRecordCache()
            self.where = where

        def format(self, values):
            idRefReason = values[0].toInt()[0]
            record = self.cache.get(idRefReason)
            where = self.where.split('WHERE')[1]
            if not record:
                resume = 0
                db = QtGui.qApp.db
                stmt = 'SELECT SUM(SUMM_REFUSE) FROM CASE_SERVICES JOIN REFUSES_OBJ ON REFUSES_OBJ.ID_SERVICE=CASE_SERVICES.ID_SERVICE JOIN CASE_SERVICES_ACCOUNTS ON CASE_SERVICES_ACCOUNTS.ID_ACCOUNT=CASE_SERVICES.ID_ACCOUNT JOIN SMO ON CASE_SERVICES_ACCOUNTS.ID_PAYER = SMO.ID_SMO WHERE REFUSES_OBJ.ID_REF_REASON=%d AND %s' % (idRefReason, where)
                query = db.query(stmt)
                val = firstQueryValue(query)
                val = '%.2f' % val.toDouble()[0]
                val = QVariant(val)
                self.cache.put(idRefReason, val)
                return val

            else:
                return record


    def __init__(self, parent=None):
        CMyTableModel.__init__(self, parent)
        self.addColumn(CTextCol(u'Причина отказа', ['REFUSE_NAME'], 70))
        self.setTable('VMU_REFUSE')
        self._whoIAm = 'refuses'
    def setWhere(self, where):
        self.addColumn(self.CCountCol(u'Количество', ['ID_REF_REASON'], 10, where))
        self.addColumn(self.CSumRefuseCol(u'Сумма по отказам', ['ID_REF_REASON'], 15, where))
    def setWhoIAm(self, val):
        self._whoIAm = val
    def getIdName(self, tableName):
        return 'ID_REF_REASON'

    def whoIAm(self):
        return self._whoIAm

# ##########

class CMySqlTableModel(QtSql.QSqlQueryModel):
    def __init__(self, parent=None):
        QtSql.QSqlQueryModel.__init__(self)
        self.listId = []
    def setIdList(self, listId):
        self.listId = listId
    def idList(self):
        return self.listId



class CMyTableRecordCache(CRecordCache):
    def __init__(self, database, table, cols='*', capacity=200):
        CRecordCache.__init__(self, capacity)
        self.database = database
        self.table = table
        self.cols = cols if not isinstance(cols, (list,  tuple)) else ', '.join(cols)


    def get(self, id):
        if type(id) == QVariant:
            if id.type() == QVariant.String:
                id = id.toString()
            elif id.type() == QVariant.Int:
                id = id.toInt()[0]
        res = CRecordCache.get(self, id)
        if not res:
#            try:
            res = self.database.getRecord(self.table, self.cols, id)
#            except:
#                res = self.database.getRecord(table)
            self.put(id, res)
        return res


DocumentTables = []
def registerDocumentTable(tableName):
    DocumentTables.append(tableName)


CMyField = CField

class CMyTable(CTable):

    def idField(self):
        if not self._idField:
            if self.tableName == 'SMO':
                self.setIdFieldName('ID_SMO')
            elif self.tableName == 'VMU_ACC_STATUS':
                self.setIdFieldName('ID_STATUS')
            elif self.tableName == 'PAY_SUMMS':
                self.setIdFieldName('ID_ACCOUNT')
            elif self.tableName == 'PAY_SHEETS':
                self.setIdFieldName('ID_PAY_CALC')
            elif self.tableName == 'VMU_DIAGNOSIS':
                self.setIdFieldName('ID_DIAGNOSIS')
            elif self.tableName == 'VMU_PROFILE':
                self.setIdFieldName('ID_PROFILE')
            elif self.tableName == 'CASES':
                self.setIdFieldName('ID_CASE')
            elif self.tableName == 'PATIENT_DATA':
                self.setIdFieldName('ID_PATIENT')
            elif self.tableName == 'REFUSES_OBJ':
                self.setIdFieldName('ID_SUMM')
            elif self.tableName == 'REFUSES':
                self.setIdFieldName('ID_REFUSE')
            else:
                self.setIdFieldName('id')
##            raise CDatabaseException(CDatabase.errNoIdField % (self.tableName))
        return self._idField


class CMyInterbaseDatabase(CInterbaseDatabase):
    def __init__(self, serverName, serverPort, databaseName, userName, password, connectionName=None):
        CInterbaseDatabase.__init__(self, serverName, serverPort, databaseName, userName, password, connectionName=None)
    def table(self, tableName):
        self.checkdb()
        if self.tables.has_key(tableName):
            return self.tables[tableName]
        else:
            if tableName in DocumentTables:
                table = CDocumentTable(tableName, self)
            else:
                table = CMyTable(tableName, self)
            self.tables[tableName] = table
            return table



    def getSelectStmtWithJoin(self, fieldList, mainTable, dictJoins, d=True):
        if d:
            return 'SELECT DISTINCT %s %s'+self.getFieldsFromList(fieldList)+self.getFromWithJoin(mainTable, dictJoins)
        else:
            return 'SELECT %s %s'+self.getFieldsFromList(fieldList)+self.getFromWithJoin(mainTable, dictJoins)

    def getFieldsFromList(self, fieldList):
        return ', '.join(fieldList)

    def getFromWithJoin(self, mainTable, dictJoins):
        begining = self.getFrom(mainTable)
        end = ''
        for table in dictJoins.keys():
            end += self.tableJoin(table, dictJoins[table])
        return begining+end

    def tableJoin(self, table, values):
        return values[2]+' JOIN '+table+' ON '+values[0]+'='+values[1]+' '

    def getFrom(self, table):
        return ' FROM '+table+' '


def connectDataBase(driverName, serverName, serverPort, databaseName, userName, password, connectionName=None):
    driverName = unicode(driverName).upper()
    if driverName == 'MYSQL':
        return CMySqlDatabase(serverName, serverPort, databaseName, userName, password, connectionName)
    elif driverName in ('INTERBASE', 'FIREBIRD'):
        return CMyInterbaseDatabase(serverName, serverPort, databaseName, userName, password, connectionName)
    elif driverName == 'ODBC':
        return CODBCDatabase(serverName, serverPort, databaseName, userName, password, connectionName)
    else:
        raise CDatabaseException(CDataBase.errUndefinedDriver % driverName)
