#!/usr/bin/env python
# -*- coding: utf-8 -*-

#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtCore, QtGui, QtSql
from PyQt4.QtCore import *
from PyQt4.QtGui  import *

from library.Utils      import *
from library.DialogBase import CDialogBase
#from TableModel         import CPatientsModel
from Reports.ReportBase import CReportBase, createTable

import re
from copy import copy



def fromVariant(v):
    if v.type() == QVariant.String:
        v = forceString(v)
    elif v.type() == QVariant.Int:
        v = forceInt(v)
    elif v.type() == QVariant.Double:
        v = forceDouble(v)
    elif v.type() == QVariant.Date:
        v = forceDate(v)
    elif v.type() == QVariant.DateTime:
        v = forceDateTime(v)
    return v

def makeListFromQuery(query):
    go = query.first()
    if go:
        l = []
        while go:
            l.append(fromVariant(query.value(0)))
            go = query.next()
        return l
    else:
        return []

def firstQueryValue(q):
    q.first()
    if q:
        return q.value(0)
    else:
        return None

def firstQueryValueInt(q):
    val = firstQueryValue(q)
    if val:
        return forceInt(val)
    else:
        return 0

def firstQueryValueDouble(q):
    val = firstQueryValue(q)
    if val:
        return forceDouble(val)
    else:
        return 0

def makeViewStmt(v):
    stmt = None
    if v == 'VIEW_REFUSE':
        stmt = '''CREATE VIEW VIEW_REFUSE AS
                SELECT
                    REFUSES.ID_ACCOUNT,
                    ACT_NUMBER, ACT_DATE, TOTAL_SUMM, REFUSES.OPER,
                    SUMM_REFUSE, REMARK
                FROM
                    REFUSES, REFUSES_OBJ
                WHERE
                    REFUSES.ID_REFUSE = REFUSES_OBJ.ID_REFUSE
            '''
    elif v == 'VIEW_VISITS':
        stmt = '''CREATE VIEW VIEW_VISITS AS
            SELECT
                CASE_SERVICES_VISIT.ID_VISIT,
                SMO.CODE, SMO.SMO_SHORT_NAME,
                CASE_SERVICES.SERVICE_COST,
                CASE_SERVICES_VISIT.DATE_BEGIN,
                CASE_SERVICES_VISIT.DATE_END,
                CASE_SERVICES.VISIT_COUNT
            FROM
                CASE_SERVICES_VISIT
                JOIN CASE_SERVICES ON
                    CASE_SERVICES.ID_SERVICE = CASE_SERVICES_VISIT.ID_SERVICE
                JOIN CASES ON
                    CASE_SERVICES.ID_CASE = CASES.ID_CASE
                JOIN SMO ON
                    CASES.ID_PAYER = SMO.ID_SMO
            '''
    if stmt:
        db = QtGui.qApp.db
        db.query(stmt)

def checkIndexes():
    db = QtGui.qApp.db
    stmt = 'CREATE INDEX date_begin_end ON CASE_SERVICES (DATE_BEGIN, DATE_END)'
    stmt1 = 'CREATE INDEX confirm_date ON CASE_SERVICES_ACCOUNTS (CONFIRM_DATE)'
    stmt2 = 'CREATE INDEX account_date ON CASE_SERVICES_ACCOUNTS (ACCOUNT_DATE)'
    list = [stmt, stmt1, stmt2]
    for stmt in list:
        try:
            db.query(stmt)
        except:
            pass

def checkViews(viewsDict):
    db = QtGui.qApp.db
    for view in viewsDict:
        stmt = 'SELECT RDB$VIEW_NAME FROM RDB$VIEW_RELATIONS WHERE RDB$VIEW_NAME = \'%s\'' % view
        query = db.query(stmt)
        if query.first():
            pass
        else:
            makeViewStmt(view)

def getAllIdProfileKSG():
    db = QtGui.qApp.db
    stmt = 'SELECT ID_PROFILE, PROFILE_INFIS_CODE FROM VMU_PROFILE WHERE ID_KSG_GROUP IS NOT NULL'
    query = db.query(stmt)
    go = query.first()
    profileIdList = []
    while go:
        profileIdList.append(query.value(0).toString())
        go = query.next()
    return profileIdList

def getAllIdProfile():
    db = QtGui.qApp.db
    stmt = 'SELECT ID_PROFILE, PROFILE_INFIS_CODE FROM VMU_PROFILE WHERE ID_KSG_GROUP IS NULL'
    query = db.query(stmt)
    go = query.first()
    profileIdList = []
    while go:
        profileIdList.append(query.value(0).toString())
        go = query.next()
    return profileIdList

def makeKsgGroups(ksgGroups=None):
    if not ksgGroups:
        db = QtGui.qApp.db
        groups = []
        query = db.query('SELECT DISTINCT KSG_GROUP_NAME FROM VMU_KSG_GROUP')
        go = query.first()
        while go:
            groups.append(query.value(0).toString())
            go = query.next()
        ksgGroups = {}
        for group in groups:
            query = db.query('SELECT PROFILE_INFIS_CODE FROM VMU_PROFILE JOIN VMU_KSG_GROUP ON VMU_KSG_GROUP.ID_KSG_GROUP = VMU_PROFILE.ID_KSG_GROUP WHERE KSG_GROUP_NAME = \'%s\'' % group)
            go = query.first()
            ksgGroups[group] = []
            while go:
                ksgGroups[group] = ksgGroups[group]+[query.value(0).toString()]
                go = query.next()
    return ksgGroups

def makeKsgGroupsByIdProfile(ksgGroups=None):
    if not ksgGroups:
        db = QtGui.qApp.db
        groups = []
        query = db.query('SELECT DISTINCT KSG_GROUP_NAME FROM VMU_KSG_GROUP')
        go = query.first()
        while go:
            groups.append(query.value(0).toString())
            go = query.next()
        ksgGroups = {}
        for group in groups:
            query = db.query('SELECT ID_PROFILE FROM VMU_PROFILE JOIN VMU_KSG_GROUP ON VMU_KSG_GROUP.ID_KSG_GROUP = VMU_PROFILE.ID_KSG_GROUP WHERE KSG_GROUP_NAME = \'%s\'' % group)
            go = query.first()
            ksgGroups[group] = []
            while go:
                ksgGroups[group] = ksgGroups[group]+[query.value(0).toInt()[0]]
                go = query.next()
    return ksgGroups

def makeExpandedKsgGroups(v, expanded):
    if v == u'ID_PROFILE':
        prepareKsgGroups = makeKsgGroupsByIdProfile()
    elif v == u'PROFILE_INFIS_CODE':
        prepareKsgGroups = makeKsgGroups()
    prepareKsgGroups = addGroups(prepareKsgGroups)
    db = QtGui.qApp.db
    for group in prepareKsgGroups.keys():
        stmt = u'SELECT %s, PROFILE_INFIS_CODE FROM VMU_PROFILE WHERE PROFILE_NAME LIKE \'%s\'' %(v, '%'+group+'%')
#        list = makeListFromQuery(db.query(stmt))
        list = []
        query = db.query(stmt)
        go = query.first()
        if go:
            while go:
                if not expanded:
#                    if not re.match('^\d+$', forceString(query.value(1))):
                    if forceString(query.value(1)).isdigit():
                        go = query.next()
                        continue
                list.append(fromVariant(query.value(0)))
                go = query.next()
        prepareKsgGroups[group] += list
    return prepareKsgGroups

def addGroups(ksgGroups):
    orgNet = QtGui.qApp.orgNet
    list = None
    if orgNet == u'взрослая':
        list = [u'Врач общей практики',
                u'Ревматология',
                u'Терапия',
                u'Травматология-ортопедия',
                u'Урология',
                u'Педиатрия',
                u'Хирургия',
                u'Центре здоровья']
    elif orgNet == u'детская':
        list = [u'Акушерство-гинекология',
                u'Восстановительное лечение',
                u'Педиатрия',
                u'Проктология',
                u'Хирургия',
                u'Травматология-ортопедия',
                u'Центре здоровья']
    if list:
        for i in list:
            if i not in ksgGroups.keys():
                ksgGroups[i] = []
    else:
        widget = QtGui.qApp.focusWidget() or QtGui.qApp.activeWindow() or QtGui.qApp.mainWindow
        QtGui.QMessageBox.critical( widget,
                                    u'Ошибка',
                                    u'Не задан тип сети(детская/взрослая)',
                                    QtGui.QMessageBox.Close)
    return ksgGroups

def makeActualKsgGroups(dFrom, dTo, expanded=True, dField='CASE_SERVICES_ACCOUNTS.ACCOUNT_DATE'):
#    if expanded:
    ksgGroups = makeExpandedKsgGroups('ID_PROFILE', expanded)
#    else:
#        ksgGroups = makeKsgGroupsByIdProfile()
    db = QtGui.qApp.db
    stmt = 'SELECT DISTINCT CASE_SERVICES.ID_PROFILE FROM CASE_SERVICES JOIN CASE_SERVICES_ACCOUNTS ON CASE_SERVICES_ACCOUNTS.ID_ACCOUNT=CASE_SERVICES.ID_ACCOUNT LEFT OUTER JOIN VMU_PROFILE ON VMU_PROFILE.ID_PROFILE=CASE_SERVICES.ID_PROFILE WHERE %s >=\'%s\' AND %s <=\'%s\'' %(dField, dFrom, dField, dTo)
    actual = makeListFromQuery(db.query(stmt))
    emptyGroups = []
    for group in ksgGroups.keys():
        tmp = copy(ksgGroups[group])
        if group == u'Центре здоровья':
            del ksgGroups[group]
            group = u'Центр здоровья'
            ksgGroups[group] = copy(tmp)
        for i in tmp:
            if i not in actual:
                ksgGroups[group].pop(ksgGroups[group].index(i))
        if len(ksgGroups[group]) == 0:
            emptyGroups.append(group)
    for group in emptyGroups:
        del ksgGroups[group]
    return checkDoublesKsgGroups(ksgGroups)

def checkDoublesKsgGroups(ksgGroups):
    for k in ksgGroups.keys():
        for v in ksgGroups[k]:
            if ksgGroups[k].count(v) > 1:
                for i in range(ksgGroups[k].count(v)-1):
                    ksgGroups[k].pop(ksgGroups[k].index(v))
    return ksgGroups




def makeReportBillServices(modelServices, modelBill, cursor, doc):
    tableColumns = [('2%',
                    [u'№'],
                    CReportBase.AlignLeft),
                    ('7%',
                    [u'Номер счета'], # 0
                    CReportBase.AlignLeft),
                    ('7%',
                    [u'Дата создания'], # 1
                    CReportBase.AlignLeft),
                    ('7%',
                    [u'Статус'], # 2
                    CReportBase.AlignLeft),
                    ('7%',
                    [u'Номер ПП'], # 3
                    CReportBase.AlignLeft),
                    ('7%',
                    [u'Дата ПП'], # 4
                    CReportBase.AlignLeft),
                    ('7%',
                    [u'Стоимость услуги'], # 5
                    CReportBase.AlignLeft),
                    ('7%',
                    [u'Сумма отказа'], # 6
                    CReportBase.AlignLeft),
                    ('7%',
                    [u'Оплачено'], # 7
                    CReportBase.AlignLeft),
                    ('7%',
                    [u'Фамилия пациента'], # 8
                    CReportBase.AlignLeft),
                    ('7%',
                    [u'Имя пациента'], # 9
                    CReportBase.AlignLeft),
                    ('7%',
                    [u'Отчество пациента'], # 10
                    CReportBase.AlignLeft),
                    ('7%',
                    [u'Профиль'], # 11
                    CReportBase.AlignLeft),
                    ('7%',
                    [u'Код профиля ИНФИС'], # 12
                    CReportBase.AlignLeft),
                    ('7%',
                    [u'Диагноз'], # 13
                    CReportBase.AlignLeft),
                    ]

    values = []
    billPassedColumns = [2, 3, 5, 6, 9, 10, 11, 12]
    for serviceRow in range(modelServices.rowCount()):
        row = []
        idBill = fromVariant(modelServices.data(modelServices.index(serviceRow, 1)))
        billRow = modelBill.idList().index(idBill)
        for billColumn in range(modelBill.columnCount()):
            if billColumn not in billPassedColumns:
                row.append(forceString(modelBill.data(modelBill.index(billRow, billColumn))))
        for serviceColumn in range(modelServices.columnCount())[2:]: # первые две это id услуги и id счета
            row.append(forceString(modelServices.data(modelServices.index(serviceRow, serviceColumn))))
        assert len(row) == 14, 'not valid columns count'
        values.append(row)
    table = createTable(cursor, tableColumns)
    for valueRow in values:
        i = table.addRow()
        table.setText(i, 0, i)
        for column, value in enumerate(valueRow):
            table.setText(i, column+1, value)
