#!/usr/bin/env python
# -*- coding: utf-8 -*-

#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui, QtCore, QtSql

from library.DialogBase import CConstructHelperMixin

from library.Utils         import *
from Utils import *

from TableModel import *
from Ui_accountListRegisterWidget import Ui_Form

class CTabAccountListRegister(QtGui.QWidget, CConstructHelperMixin, Ui_Form):

    def __init__(self, where=None):
        QtGui.QWidget.__init__(self)
        self.setupUi(self)
        self.accountId = None
        self.profileId = None
        self.patientId = None
        self.accountNumber = None
        self.currentIdService = None
        self.where = where
        self.printExpandedALRModel = CPrintExpandedServices(self)
        self.addModels('StrictALR', CStrictALRTableModel(self))
        self.setModels(self.tblStrictListRegister, self.modelStrictALR, self.selectionModelStrictALR)
        self.addModels('ALR', CALRTableModel(self))
        self.setModels(self.tblListRegister, self.modelALR, self.selectionModelALR)
        self.tblListRegister.setModel(self.modelALR)
        self.contextIsSet = False
        self.actNumber = None
        QtCore.QObject.connect(
            self.selectionModelALR,
            QtCore.SIGNAL('currentRowChanged(QModelIndex, QModelIndex)'),
            self.ALRRowChanged)

    def setActNumber(self, actNumber):
        self.actNumber = actNumber

    def modelServices(self):
        return self.modelALR

    def fillCommonEdits(self):
#        self.setStrictALRContext()
        refuse   = 0
        paySum   = 0
        totalSum = 0
#        print len(self.modelStrictALR.idList())
        for i in range(len(self.modelStrictALR.idList())):
            refuse += forceDouble(self.modelStrictALR.data(self.modelStrictALR.index(i, 4)))
        for i in range(len(self.modelALR.idList())):
            totalSum += self.modelALR.data(self.modelALR.index(i, 2)).toDouble()[0]
        self.edtRefuse.setText('%10.2f'%refuse)
        self.edtCommonTotalSumm.setText('%10.2f'%totalSum)
        self.edtCommonPaySumm.setText('%10.2f'%(totalSum-refuse))
#        self.setStrictALRContext(-1)

    def getTableViewALR(self):
        return self.tblListRegister
    def getTableViewALRStrict(self):
        return self.tblStrictListRegister

    def getCurrentIndex(self):
        return self.selectionModelALR.selectedRows()

    @QtCore.pyqtSignature('int')
    def on_chkShowAllRefuses_stateChanged(self, state):
        if state == QtCore.Qt.Checked:
            self.setStrictALRContext()
        elif state == QtCore.Qt.Unchecked:
            currentIndex = self.selectionModelALR.selectedRows()
            if len(currentIndex) > 0:
                self.ALRRowChanged(self.selectionModelALR.selectedRows()[0])
            else:
                self.setStrictALRContext(-1)

    def tblListRegisterActivated(self, index):
        pass

    def preparePopupMenu(self):
        self.showAllRefuseAction = QtGui.QAction(u'Все отказы', self)
        self.connect(self.showAllRefuseAction,
                     QtCore.SIGNAL('triggered()'),
                     self.setStrictALRContext)
        self.tblStrictListRegister.createPopupMenu([self.showAllRefuseAction])

    def setAccountId(self, id):
        if type(id) == QtCore.QVariant:
            if id.type() == QtCore.QVariant.String:
                self.accountId = id.toString()
            if id.type() == QtCore.QVariant.Int:
                self.accountId = id.toInt()[0]

    def ALRRowChanged(self, current, previous=None):
        id = self.getId(current)
        index = current
        model = self.modelALR
        self.currentIdService = model.data(model.index(index.row(), 0))
        if type(id) == QtCore.QVariant:
            if id.type() == QtCore.QVariant.String:
                id = forceString(id)
        if not self.chkShowAllRefuses.isChecked():
            self.setStrictALRContext(id)

    def setALRContext(self, patientId=None, profileId=None):
        self.patientId = patientId
        self.profileId = profileId
        self.contextIsSet = True
        self.modelALR.setIdList(self.setModelIdList())
        self.fillCommonEdits()

    def setModelIdList(self):
        db = QtGui.qApp.db
        if self.where:
            where =  ' AND '+ self.where.split('WHERE')[1]
        else:
            where = ''
#        stmt = '''
#                SELECT
#                    ID_SERVICE,
#                    ID_ACCOUNT,
#                    SERVICE_COST,
#                    SURNAME, NAME, SECOND_NAME,
#                    PROFILE_NAME,
#                    DIAGNOSIS_NAME
#                FROM
#                    CASE_SERVICES
#                    JOIN CASES ON
#                        CASES.ID_CASE = CASE_SERVICES.ID_CASE
#                    JOIN PATIENT_DATA ON
#                        CASES.ID_PATIENT = PATIENT_DATA.ID_PATIENT
#                    JOIN VMU_PROFILE ON
#                        CASE_SERVICES.ID_PROFILE = VMU_PROFILE.ID_PROFILE
#                    JOIN VMU_DIAGNOSIS ON
#                        CASE_SERVICES.ID_DIAGNOSIS = VMU_DIAGNOSIS.ID_DIAGNOSIS
#                    WHERE
#                        ID_ACCOUNT = \'%s\' %s
#        ''' % (self.accountId, '')#, where)
        patientCond = 'AND CASES.ID_PATIENT=\'%s\'' % self.patientId if bool(self.patientId) else ''
        profileCond = 'AND CASE_SERVICES.ID_PROFILE=%d' % self.profileId if bool(self.profileId) else ''
        stmt = 'SELECT DISTINCT ID_SERVICE  FROM CASE_SERVICES LEFT OUTER JOIN CASES ON CASES.ID_CASE=CASE_SERVICES.ID_CASE WHERE ID_ACCOUNT = \'%s\' %s %s' % (str(self.accountId), patientCond, profileCond)
        query = db.query(stmt)
#        print query.lastQuery()
        result = []
        while query.next():
            result.append(query.value(0))
        self.accountNumber = self.getActNumber(self.accountId)
        self.printExpandedALRModel.setIdList(result)
        return result

    def getActNumber(self, id):
        db = QtGui.qApp.db
        stmt = 'SELECT ACCOUNT_NUMBER FROM CASE_SERVICES_ACCOUNTS WHERE ID_ACCOUNT = \'%s\'' % id
        accountNumber = firstQueryValue(db.query(stmt))
        return forceString(accountNumber)
    def getPrintExpandedALRModel(self):
        return self.printExpandedALRModel

    def setStrictALRContext(self, id=None):
        db = QtGui.qApp.db
        if self.actNumber:
            actNumber = ' AND ACT_NUMBER=\'%s\''%self.actNumber
        else:
            actNumber = ''
        patientCond = 'AND CASES.ID_PATIENT=\'%s\'' % self.patientId if bool(self.patientId) else ''
        profileCond = 'AND CASE_SERVICES.ID_PROFILE=%d' % self.profileId if bool(self.profileId) else ''
        if not id:
            # Оно мне надо было?
            #REFUSES.ACT_NUMBER, REFUSES.ACT_DATE, REFUSES.TOTAL_SUMM, REFUSES.OPER,
            #        REFUSES_OBJ.SUMM_REFUSE, REFUSES_OBJ.REMARK
            stmt = '''
                SELECT DISTINCT
                    REFUSES_OBJ.ID_SUMM
                FROM
                    REFUSES
                    JOIN REFUSES_OBJ ON
                        REFUSES_OBJ.ID_REFUSE = REFUSES.ID_REFUSE
                    LEFT JOIN CASE_SERVICES ON
                        CASE_SERVICES.ID_ACCOUNT = REFUSES.ID_ACCOUNT
                    LEFT JOIN CASES ON
                        CASES.ID_CASE = CASE_SERVICES.ID_CASE
                WHERE
                    REFUSES.ID_ACCOUNT = \'%s\'
                %s %s %s

                    ''' % (self.accountId, actNumber, patientCond, profileCond)
        else:
            stmt = '''
                SELECT DISTINCT
                    REFUSES_OBJ.ID_SUMM
                FROM
                    REFUSES
                    JOIN REFUSES_OBJ ON
                        REFUSES_OBJ.ID_REFUSE = REFUSES.ID_REFUSE
                    LEFT JOIN CASE_SERVICES ON
                        CASE_SERVICES.ID_SERVICE = REFUSES_OBJ.ID_SERVICE
                    LEFT JOIN CASES ON
                        CASES.ID_CASE = CASE_SERVICES.ID_CASE
                WHERE
                    REFUSES.ID_ACCOUNT = \'%s\'
                AND
                    REFUSES_OBJ.ID_SERVICE = \'%s\'
                %s %s %s
                    ''' % (self.accountId, id, actNumber, patientCond, profileCond)
        query = db.query(stmt)
#        print query.lastQuery()
        result = []
        while query.next():
            result.append(query.value(0))
        self.modelStrictALR.setIdList(result)
        self.fillCommonEdits()


    def getId(self, index):
        return self.tblListRegister.itemId(index)

    def setIsContext(self, bool):
        self.contextIsSet = False

    def resetALRContext(self):
        self.model.reset()
