#!/usr/bin/env python
# -*- coding: utf-8 -*-

#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import codecs
import gc
import locale
import os
import os.path
import shutil
import sys
import tempfile
import traceback
import StringIO
from optparse import OptionParser
from copy import copy

from PyQt4 import QtCore, QtGui, QtSql
from PyQt4.QtCore import *
from PyQt4.QtGui  import *

#import library.patches

from Users.Login           import CLoginDialog
from Users.RightList       import CUserRightListDialog,  CUserRightProfileListDialog
from Users.UserInfo        import CUserInfo, CDemoUserInfo
from Users.Rights          import *
from Users.Tables          import *
from library               import database
from library.Preferences   import CPreferences
from library.DialogBase    import CDialogBase
from library.Utils         import *
#from library.dbfpy.dbf     import Dbf
from library.DbfViewDialog import viewDbf, CDbfViewDialog
from library.DialogBase    import CConstructHelperMixin
from library.TableView     import CTableView
from library.Calendar      import CCalendarInfo
from Reports.ReportView    import CReportViewDialog
from Reports.ReportBase    import CReportBase, createTable

from preferences.connection import CConnectionDialog
from preferences.decor      import CDecorDialog


from EISConnection             import CEISConnectionDialog
from queryResult               import CResultProgressDialog
from exportSettingsDialog      import CExportSettingsDialog
from accountListRegisterWidget import CTabAccountListRegister
from profileDistinct           import CTabProfileDistinct
from autorizationDialog        import CAutorizationDialog
from TableModel                import *
from Utils                     import *

from Ui_browserEIS             import Ui_MainWindow
from Ui_PatientsListDialog     import Ui_PatientsListDialog

title = u'САМСОН'
titleLat = 'SAMSON'
about = u'Утилита для анализа данных ЕИС.ОМС.ВМУ \n' \
        u'Комплекс Программных Средств \n' \
        u'"Система автоматизации медико-страхового обслуживания населения"\n' \
        u'КПС «%s»\n'   \
        u'Версия 2.0 (ревизия %s от %s)\n' \
        u'Copyright © 2015 ООО "САМСОН Групп"\n' \
        u'распространяется под лицензией GNU GPL v.3 или выше\n' \
        u'Основано на аналогичном элементе КПС «САМСОН-ВИСТА» версии 2.0\n' \
        u'телефон тех.поддержки (812) 418-39-70'


class CMyApp(QtGui.QApplication):
    __pyqtSignals__ = ('dbConnectionChanged(bool)',
                       'currentUserIdChanged()',
                      )

    def __init__(self, args, demoModeRequest):
        QtGui.QApplication.__init__(self, args)
        self.logDir = os.path.join(unicode(QDir.toNativeSeparators(QDir.homePath())), '.samson-vista')
        self.oldMsgHandler = qInstallMsgHandler(self.msgHandler)
        self.oldEexceptHook = sys.excepthook
        sys.excepthook = self.logException
        self.traceActive = True
        self.homeDir= None
        self.saveDir= None
        self.userId = None
        self.userSpecialityId   = None
        self.userOrgStructureId = None
        self.organisation       = None
        self.orgNet             = None
        self.db     = None
        self.s11db  = None
        self.mainWindow = None
        self.ksgGroups  = None
        self.calendarInfo = CCalendarInfo(self)
        self.preferences = CPreferences('r78browserEIS.ini')
        self.preferences.load()
        self.preferencesDir = self.preferences.getDir()
        self.registerDocumentTables()
        self._currentClientId = None
        self.userInfo = None
        self.demoModeRequested = True


    def highlightRedDate(self):
        return False

    def highlightInvalidDate(self):
        return False

    def doneTrace(self):
        self.traceActive = False
        qInstallMsgHandler(self.oldMsgHandler)
        sys.excepthook = self.oldEexceptHook

    def applyDecorPreferences(self):
        self.setStyle(self.preferences.decorStyle)
        if self.preferences.decorStandardPalette:
            self.setPalette(self.style().standardPalette())
        if self.mainWindow:
            state = Qt.WindowNoState
            if self.preferences.decorMaximizeMainWindow:
                state |= Qt.WindowMaximized
            if self.preferences.decorFullScreenMainWindow:
                state |= Qt.WindowFullScreen
            self.mainWindow.setWindowState(state)

    def registerDocumentTables(self):
        pass

    def userHasRight(self, right):
        return True

    def documentEditor(self):
        return forceString(QtGui.qApp.preferences.appPrefs.get('documentEditor', ''))

    def getHomeDir(self):
        if not self.homeDir:
            homeDir = os.path.expanduser('~')
            if homeDir == '~':
                homeDir = unicode(QDir.homePath())
            if isinstance(homeDir, str):
                homeDir = unicode(homeDir, locale.getpreferredencoding())
            self.homeDir = homeDir
        return self.homeDir

    def getSaveDir(self):
        self.saveDir = forceString(self.preferences.appPrefs.get('saveDir', QVariant()))
        if not self.saveDir:
            self.saveDir = self.getHomeDir()
        return self.saveDir

    def setSaveDir(self, path):
        saveDir = os.path.dirname(unicode(path))
        if saveDir and self.saveDir != saveDir:
            self.preferences.appPrefs['saveDir'] = QVariant(saveDir)

    def getTemplateDir(self):
        result = forceString(self.preferences.appPrefs.get('templateDir', None))
        if not result:
            result = os.path.join(self.logDir, 'templates')
        return result

    def getTmpDir(self, suffix=''):
        return unicode(tempfile.mkdtemp('','samson_%s_'%suffix), locale.getpreferredencoding())

    def removeTmpDir(self, dir):
        try:
            shutil.rmtree(dir, False)
        except:
            self.logCurrentException()

#    def setTemplateDir(self, path):
#        self.preferences.appPrefs.setdefault('templateDir', QVariant(path))

    def log(self, title, message, stack=None):
        try:
            if not os.path.exists(self.logDir):
                os.makedirs(self.logDir)
            logFile = os.path.join(self.logDir, 'error.log')
            timeString = unicode(QDateTime.currentDateTime().toString(Qt.SystemLocaleDate))
            logString = u'%s\n%s: %s(%s)\n' % ('='*72, timeString, title, message)
            if stack:
                try:
                    logString += ''.join(traceback.format_list(stack)).decode('utf-8') + '\n'
                except:
                    logString += 'stack lost\n'
            file = codecs.open(logFile, mode='a', encoding=locale.getpreferredencoding())
            file.write(logString)
            file.close()
        except:
            pass

    def logException(self, exceptionType, exceptionValue, exceptionTraceback):
        title = repr(exceptionType)
        message = unicode(exceptionValue)
        self.log(title, message, traceback.extract_tb(exceptionTraceback))
        sys.__excepthook__(exceptionType, exceptionValue, exceptionTraceback)

    def logCurrentException(self):
        self.logException(*sys.exc_info())

    def msgHandler(self, type, msg):
        if type == 0: # QtMsgType.QtDebugMsg:
            typeName = 'QtDebugMsg'
        elif type == 1: # QtMsgType.QtWarningMsg:
            typeName = 'QtWarningMsg'
        elif type == 2: # QtMsgType.QtCriticalMsg:
            typeName = 'QtCriticalMsg'
        elif type == 3: # QtFatalMsg
            typeName = 'QtFatalMsg'
        else:
            typeName = 'QtUnknownMsg'

        self.log( typeName, msg, traceback.extract_stack()[:-1])
#        print typeName, msg

    def openDatabase(self, preferences):
        self.s11db = None
        self.s11db = database.connectDataBase(preferences.dbDriverName,
                                     preferences.dbServerName,
                                     preferences.dbServerPort,
                                     preferences.dbDatabaseName,
                                     preferences.dbUserName,
                                     preferences.dbPassword)
        self.emit(QtCore.SIGNAL('dbConnectionChanged(bool)'), True)

    def organizationInfo(self):
        preferences = CPreferences('S11App.ini')
        preferences.load()
        settings = preferences.getSettings()
        settings.beginGroup('appPrefs')
        orgId = forceInt(settings.value('orgId', QVariant(None)))
        orgStructureId = forceInt(settings.value('orgStructureId', QVariant(None)))
        settings.endGroup()
        self.openDatabase(preferences)
        stmt = 'SELECT Organisation.`shortName`, rbNet.`name` FROM Organisation JOIN rbNet ON rbNet.`id`=Organisation.`net_id` WHERE Organisation.id=%d' % orgId
        query = self.s11db.query(stmt)
        if query.first():
            self.organisation = forceString(query.value(0))
            self.orgNet       = forceString(query.value(1))
        self.closeDatabase()



    def closeDatabase(self):
        if self.s11db:
            self.s11db.close()
            self.s11db = None
            self.emit(QtCore.SIGNAL('dbConnectionChanged(bool)'), False)

    def notify(self, receiver, event):
        try:
#            report = ["notify:: receiver.type=",unicode(type(receiver))," receiver.name=", unicode(receiver.objectName())," event.type=",unicode(type(event)), "event.type()=", unicode(event.type())]
#            print >>sys.stderr, ("".join(report)).encode('cp866')
            return QtGui.QApplication.notify(self, receiver, event)
        except Exception, e:
            if self.traceActive:
                self.logCurrentException()
                widget = self.activeModalWidget()
                if widget is None:
                    widget = self.mainWindow
                QtGui.QMessageBox.critical( widget,
                                            u'Произошла ошибка',
                                            unicode(e),
                                            QtGui.QMessageBox.Close)
            return False
        except:
            return False

    def call(self, widget, func, params = ()):
        try:
            return True, func(*params)
        except IOError, e:
            msg=''
            if hasattr(e, 'filename'):
                msg = u'%s:\n[Errno %s] %s' % (e.filename, e.errno, anyToUnicode(e.strerror))
            else:
                msg = u'[Errno %s] %s' % (e.errno, anyToUnicode(e.strerror))
            QtGui.QMessageBox.critical(widget,
                u'Произошла ошибка ввода-вывода', msg, QtGui.QMessageBox.Close)
        except Exception, e:
            self.logCurrentException()
            widget = widget or self.activeModalWidget() or self.mainWindow
            QtGui.QMessageBox.critical( widget,
                                        u'Произошла ошибка',
                                        unicode(e),
                                        QtGui.QMessageBox.Close)
        return False, None

    def startProgressBar(self, steps, format=u'%v из %m'):
        progressBar = self.mainWindow.getProgressBar()
        progressBar.setFormat(format)
        progressBar.setMinimum(0)
        progressBar.setMaximum(steps)
        progressBar.setValue(0)

    def stepProgressBar(self, step=1):
        progressBar = self.mainWindow.getProgressBar()
        progressBar.setValue(progressBar.value()+step)


    def stopProgressBar(self):
        self.mainWindow.hideProgressBar()


    def callWithWaitCursor(self, widget, func, *params, **kwparams):
        self.setOverrideCursor(QtGui.QCursor(Qt.WaitCursor))
        try:
            return func(*params, **kwparams)
        finally:
            self.restoreOverrideCursor()


    def demoModePosible(self):
        try:
            tableUser = self.db.table(tblUser)
            record = self.db.getRecordEx(tableUser, [usrPassword, usrRetired], tableUser[usrLogin].eq(demoUserName))
            return not record or (not forceString(record.value(usrPassword)) and not forceBool(record.value(usrRetired)))
        except:
            pass
        return False


    def getEISPreferences(self):
        appPreferences = QtGui.qApp.preferences.appPrefs
        eisPreferences = appPreferences.get('eis', {})
        result = { 'server'  : forceString(eisPreferences.get('server', 'localhost')),
                   'port'    : forceInt(eisPreferences.get('port', 0)),
                   'database': forceString(eisPreferences.get('database', 'C:\\EISOMS\\DBMU.GDB')),
                   'user'    : forceString(eisPreferences.get('user', 'SYSDBA')),
                   'password': forceString(eisPreferences.get('password', 'masterkey')),
                 }
        return result

    def setEISPreferences(self, eisPreferences):
        appPreferences = QtGui.qApp.preferences.appPrefs
        appPreferences['eis'] = eisPreferences
        QtGui.qApp.preferences.save()

    def endWorkWithEISOMS(self, viewsDict):
        for view in viewsDict:
            try:
                self.db.query('DROP VIEW %s' % view)
                if self.checkViews(view):
                    self.endWorkWithEISOMS([view])
            except Exception, e:
                pass
#                print 'End work'

    def checkViews(self, view):
        stmt = 'SELECT RDB$VIEW_NAME FROM RDB$VIEW_RELATIONS WHERE RDB$VIEW_NAME = \'%s\'' % view
        query = self.db.query(stmt)
        if query.first():
            return True
        return False


class CMainWindow(QtGui.QMainWindow, Ui_MainWindow, CConstructHelperMixin):
    viewsDict = [
                 'VIEW_VISITS'
                 ]
    filtersValue = {
                    'noPP'             : '',
                    'payDate'          : '',
                    'datePP'           : '',
                    'status'           : '',
                    'smoShort'         : '',
                    'smoLong'          : '',
                    'dateFrom'         : '',
                    'dateTo'           : '',
                    'chkNoPP'          : False,
                    'chkDatePP'        : False,
                    'chkPayDate'       : False,
                    'patientSurname'   : '',
                    'patientName'      : '',
                    'patientSecondName': '',
                    'patientBirthday'  : '',
                    'patientSex'       : ''
                    }
    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)
        self.tmp                 = None
        self.tabALR              = None
        self.tabProfileDistinct  = None
        self.execDone            = None
        self.cuurrentActive      = None
        self.LPU                 = None
        self.LPUNet              = None
        self.mnuALRPrint         = None
        self.patientId           = None
        self.idList              = []
        self.tabsList            = []
        self.order               = 'ACCOUNT_DATE'
        self.orderForExport      = ''
        self.headerSortIndicator = Qt.AscendingOrder
        self.reconnect           = False
        self.reportHeader        = ''
        self.fillTreeQuery()
        self.loadPreferences()
        self.btnExport.setEnabled(False)
        self.cmbSMO.setEnabled(False)
        self.cmbStatus.setEnabled(False)
        self.setDatesDefaultValue()
        self.setDatesEnabled(False)
        self.setChkClientFilterEnabled(False)
        self.setChkProfileEnabled(False)
        self.prepareStatusBar()
        self.btnPrint.setEnabled(False)
        self.tabView.setTabsClosable(True)
        self.tabView.setTabText(0, u'')
        self.edtDateFrom.setDisplayFormat('dd.MM.yyyy')
        self.edtDateTo.setDisplayFormat('dd.MM.yyyy')
        self.edtDatePP.setDisplayFormat('dd.MM.yyyy')
        self.edtDatePay.setDisplayFormat('dd.MM.yyyy')
        self.edtClientBirthday.lineEdit.clear()
        self.createPopupActions()
        self.tableView.setSortingEnabled(True)
        self.createPrintButtonALRMenu()
        QtCore.QObject.connect(
            self.tableView.horizontalHeader(),
            QtCore.SIGNAL('sectionClicked(int)'),
            self.setSort)

    def createPrintButtonALRMenu(self):
        self.mnuALRPrint = QtGui.QMenu(self)
        self.actPrint   = QtGui.QAction(u'Печать', self)
        self.actPrintRefuses = QtGui.QAction(u'Реестр отказов', self)
        self.actPrintRefuses.setEnabled(False)
        self.actPrintExpandedServices = QtGui.QAction(u'Реестр услуг', self)
        self.actPrintExpandedServices.setEnabled(False)
        self.actPrintProfileDistinct = QtGui.QAction(u'Счета', self)
        self.actPrintProfileDistinct.setEnabled(False)
        self.actPrintBillServices = QtGui.QAction(u'Список услуг счета', self)
        self.actPrintBillServices.setEnabled(False)
        self.mnuALRPrint.addAction(self.actPrint)
        self.mnuALRPrint.addAction(self.actPrintExpandedServices)
        self.mnuALRPrint.addAction(self.actPrintRefuses)
        self.mnuALRPrint.addAction(self.actPrintProfileDistinct)
        self.mnuALRPrint.addAction(self.actPrintBillServices)

        self.connect(self.actPrint, QtCore.SIGNAL('triggered()'), self.on_btnPrint_clicked)
        self.connect(self.actPrintExpandedServices, QtCore.SIGNAL('triggered()'), self.printExpandedServices)
        self.connect(self.actPrintRefuses, QtCore.SIGNAL('triggered()'), self.printRefuses)
        self.connect(self.actPrintBillServices, QtCore.SIGNAL('triggered()'), self.prinBillServices)

        self.setPrintButtonALRMenu()


    def printExpandedServices(self):
        html = self.expandedServicesContentToHTML()
        view = CReportViewDialog(self)
        view.setText(self.expandedServicesResume(html))
        view.exec_()

    def expandedServicesResume(self, html):
        widget = self.tabView.currentWidget()
        cursor = QtGui.QTextCursor(html)
        cursor.movePosition(cursor.End)
        cursor.setCharFormat(CReportBase.ReportBody)
        resume = u'Итого\n'
        resume += u'Сумма выставленная: ' + widget.edtCommonTotalSumm.text()+'\n'
        resume += u'Сумма отказанная: ' + widget.edtRefuse.text()+'\n'
        resume += u'Всего: ' + widget.edtCommonPaySumm.text()+'\n\n'
        resume += forceString(QDateTime.currentDateTime())
        if Qt.mightBeRichText(resume):
            cursor.insertHtml(resume)
        else:
            cursor.insertText(resume)
        cursor.insertBlock()
        return html

    def expandedServicesContentToHTML(self):
        widget = self.tabView.currentWidget()
        reportHeader = u'Реестр счета \''+widget.accountNumber+'\''
        tableView = CTableView(self)
        tableView.setModel(widget.getPrintExpandedALRModel())
        tableView.setReportHeader(reportHeader)
        reportDescription = self.getReportDescription('expandedServices')
        tableView.setReportDescription(reportDescription)
        return tableView.contentToHTML()

    def printRefuses(self):
        doc = QtGui.QTextDocument()
        cursor = QtGui.QTextCursor(doc)
        cursor.setCharFormat(CReportBase.ReportTitle)
        widget = self.tabView.currentWidget()
        chk = widget.chkShowAllRefuses.isChecked()
        if chk:
            header = u'Отказы по счету \''+widget.accountNumber+'\''
        else:
            header = u'Выборочные отказы по услугам счета \''+widget.accountNumber+'\''
        if Qt.mightBeRichText(header):
            cursor.insertHtml(header)
        else:
            cursor.insertText(header)
        cursor.insertBlock()
        if chk:
            html = self.refusesContentToHTML()
            cursor.insertFragment(QtGui.QTextDocumentFragment(html))
        else:
            cursor.setCharFormat(CReportBase.ReportBody)
            fragment1 = QtGui.QTextDocumentFragment(self.getRefusesReportDescription(widget))
            cursor.insertFragment(fragment1)
            fragment2 = QtGui.QTextDocumentFragment(self.refusesContentToHTML())
            cursor.insertFragment(fragment2)
        view = CReportViewDialog(self)
        view.setText(doc)
        view.exec_()


    def getReportBillServicesTitle(self):
        main = u'Список услуг с %s по %s'
        main = main % (forceString(self.edtDateFrom.date()), forceString(self.edtDateTo.date()))
        return main


    def getReportBillServiceSubTitle(self):
        smo = unicode(self.cmbSMO.currentText()) # use .value() Luke!
        if smo.strip() == '-':
            smo = u''
        else:
            smo = u'СМО: %s\n' % smo
        status = unicode(self.cmbStatus.currentText())
        if status == u'-':
            status = u''
        else:
            status = u'Статус: %s\n' % status
        if self.chkDatePP.isChecked():
            datePP = forceString(self.edtDatePP.date())
            datePP = u'Дата ПП: %s\n' % datePP
        else:
            datePP = u''
        if self.chkNoPP.isChecked():
            noPP = forceString(self.edtNoPP.value())
            noPP = u'Номер ПП: %s\n' % noPP
        else:
            noPP = u''
        if self.chkDatePay.isChecked():
            datePay = forceString(self.edtDatePay.date())
            datePay = u'Дата проводки: %s\n' % datePay
        else:
            datePay = u''
        if self.chkClientFilter.isChecked():
            surname      = self.edtClientSurname.text()
            name         = self.edtClientName.text()
            secondName   = self.edtClientSecondName.text()
            patientName  = formatName(surname, name, secondName)
            if bool(patientName):
                patientName = u'ФИО клиента: %s\n' % patientName
            birthday     = self.edtClientBirthday.date()
            if birthday.isValid():
                birthday = forceString(birthday)
                birthday = u'Дата рождения клиента: %s\n' % birthday
            else:
                birthday = u''
            sex          = self.cmbClientSex.currentText()
            if bool(sex):
                sex = u'Пол клиента: %s\n' % sex
        else:
            patientName = u''
            birthday    = u''
            sex         = u''
        if self.chkProfile.isChecked():
            if bool(self.cmbProfile.value()):
                profile = unicode(self.cmbProfile.stringValue())
                profile = u'Профиль: %s\n' % profile
            else:
                profile = u''
        else:
            profile = u''
#        return u''.join([smo, status, datePP, noPP, datePay, patientName, birthday, sex, profile])
        return smo+status+datePP+noPP+datePay+patientName+birthday+sex+profile


    def prinBillServices(self):
        self.setCursor(QtCore.Qt.WaitCursor)
        if not (bool(self.tabALR) and hasattr(self, 'modelBill')):
            return
        modelServices = self.tabALR.getPrintExpandedALRModel()
        modelBill     = self.modelBill
        doc = QtGui.QTextDocument()
        cursor = QtGui.QTextCursor(doc)
        cursor.setCharFormat(CReportBase.ReportTitle)
        cursor.insertText(self.getReportBillServicesTitle())
        cursor.insertBlock()
        cursor.setCharFormat(CReportBase.ReportSubTitle)
        cursor.insertText(self.getReportBillServiceSubTitle())
        cursor.insertBlock()
        cursor.setCharFormat(CReportBase.ReportBody)
        makeReportBillServices(modelServices, modelBill, cursor, doc)
        viewDialog = CReportViewDialog(self)
        viewDialog.setWindowTitle(u'Печать')
        viewDialog.setText(doc)
        self.setCursor(QtCore.Qt.ArrowCursor)
        viewDialog.exec_()


    def refusesContentToHTML(self):
        widget = self.tabView.currentWidget()
        tableView = widget.getTableViewALRStrict()
        if len(tableView.model().idList()) == 0:
            doc = QtGui.QTextDocument()
            cursor = QtGui.QTextCursor(doc)
            cursor.setCharFormat(CReportBase.ReportTitle)
            cursor.insertText(u'Отказов нет')
            return doc
        reportDescription = u''
        tableView.setReportHeader('')
        tableView.setReportDescription(u'Отказы')
        return tableView.contentToHTML()


    def getRefusesReportDescription(self, widget):
        index = widget.getCurrentIndex()
        tableView = CTableView(self)
        model = CALRTableModel(self)
        model.setIdList([widget.currentIdService])
        tableView.setModel(model)
        tableView.setReportHeader('')
        tableView.setReportDescription(u'Услуги')
        return tableView.contentToHTML()

    def startAutorization(self):
        try:
            QtGui.qApp.organizationInfo()
        except Exception, e:
            QtGui.QMessageBox.critical(
                self,
                u'Ошибка',
                u'Ошибка при получении информации об активной организации \n'+unicode(e),
                QtGui.QMessageBox.Close)
            self.EISDb = None
            self.setDatesEnabled(False)
            self.setChkClientFilterEnabled(False)
            self.setChkProfileEnabled(False)
            self.btnExport.setEnabled(False)
            self.cmbSMO.setEnabled(False)
            self.cmbStatus.setEnabled(False)

    def autoAutorization(self):
        self.startAutorization()
        self.loadEisBase()

    def autorization(self):
        organisation = QtGui.qApp.organisation
        orgNet       = QtGui.qApp.orgNet
        dlg = CAutorizationDialog(self)
        dlg.setLPU(organisation)
        dlg.setLPUNet(orgNet)
        if dlg.exec_():
            QtGui.qApp.organisation = dlg.edtLPU.text()
            QtGui.qApp.orgNet       = dlg.cmbTypeLPU.currentText()

    def loadFiltersSettings(self):
        preferences = QtGui.qApp.preferences
        settings = preferences.getSettings()
        settings.beginGroup('commonSettings')
        chkFilters = settings.value('saveFilters', QtCore.QVariant(False)).toBool()
        settings.endGroup()
        if chkFilters:
            settings.beginGroup('filtersSettings')
            self.filtersValue['noPP']     = settings.value('noPP', QVariant(self.filtersValue['noPP'])).toString()
            self.filtersValue['payDate']  = settings.value('payDate', QVariant(self.filtersValue['payDate'])).toString()
            self.filtersValue['datePP']   = settings.value('datePP', QVariant(self.filtersValue['datePP'])).toString()
            self.filtersValue['status']   = settings.value('status', QVariant(self.filtersValue['status'])).toString()
            self.filtersValue['smoShort'] = settings.value('smoShort', QVariant(self.filtersValue['smoShort'])).toString()
            self.filtersValue['smoLong']  = settings.value('smoLong', QVariant(self.filtersValue['smoLong'])).toString()
            self.filtersValue['dateFrom'] = settings.value('dateFrom', QVariant(self.filtersValue['dateFrom'])).toString()
            self.filtersValue['dateTo']   = settings.value('dateTo', QVariant(self.filtersValue['dateTo'])).toString()
            self.filtersValue['chkNoPP']  = settings.value('chkNoPP', QVariant(False)).toBool()
            self.filtersValue['chkDatePP']  = settings.value('chkDatePP', QVariant(False)).toBool()
            self.filtersValue['chkPayDate']  = settings.value('chkPayDate', QVariant(False)).toBool()
            self.filtersValue['patientSurname']   = settings.value('patientSurname', QVariant(self.filtersValue['patientSurname'])).toString()
            self.filtersValue['patientName']   = settings.value('patientName', QVariant(self.filtersValue['patientName'])).toString()
            self.filtersValue['patientSecondName']   = settings.value('patientSecondName', QVariant(self.filtersValue['patientSecondName'])).toString()
            self.filtersValue['patientBirthday']   = settings.value('patientBirthday', QVariant(self.filtersValue['patientBirthday'])).toString()
            self.filtersValue['patientSex']   = settings.value('patientSex', QVariant(self.filtersValue['patientSex'])).toString()
            settings.endGroup()
            self.loadWidgetsFiltersSettings()
        else:
            self.setDatesDefaultValue()


    def loadWidgetsFiltersSettings(self):
        if self.filtersValue['noPP'] != '':
            self.edtNoPP.setValue(forceInt(self.filtersValue['noPP']))
            self.chkNoPP.setChecked(self.filtersValue['chkNoPP'])
        if self.filtersValue['payDate'] != '':
            self.edtDatePay.setDate(QDate.fromString(self.filtersValue['payDate'], 'dd.MM.yyyy'))
            self.chkDatePay.setChecked(self.filtersValue['chkPayDate'])
        if self.filtersValue['datePP'] != '':
            self.edtDatePP.setDate(QDate.fromString(self.filtersValue['datePP'], 'dd.MM.yyyy'))
            self.chkDatePP.setChecked(self.filtersValue['chkDatePP'])
        if self.filtersValue['dateFrom'] != '':
            self.edtDateFrom.setDate(QDate.fromString(self.filtersValue['dateFrom'], 'dd.MM.yyyy'))
        if self.filtersValue['dateTo'] != '':
            self.edtDateTo.setDate(QDate.fromString(self.filtersValue['dateTo'], 'dd.MM.yyyy'))
        if self.filtersValue['status'] != '':
            self.cmbStatus.setCurrentIndex(self.cmbStatus.findText(self.filtersValue['status']))
        if self.filtersValue['smoShort'] != '':
            self.cmbSMO.setCurrentIndex(self.cmbSMO.findText(self.filtersValue['smoShort']))
        if self.filtersValue['patientSurname'] != '':
            self.edtClientSurname.setText(self.filtersValue['patientSurname'])
        if self.filtersValue['patientName'] != '':
            self.edtClientName.setText(self.filtersValue['patientName'])
        if self.filtersValue['patientSecondName'] != '':
            self.edtClientSecondName.setText(self.filtersValue['patientSecondName'])
        if self.filtersValue['patientBirthday'] != '':
            self.edtClientBirthday.setDate(QDate.fromString(self.filtersValue['patientBirthday'], 'dd.MM.yyyy'))
        if self.filtersValue['patientSex'] != '':
            self.cmbClientSex.setCurrentIndex(self.cmbClientSex.findText(self.filtersValue['patientSex']))

    def createPopupActions(self):
        self.sayHelloAction = QtGui.QAction(u'Привет', self)
        self.openLRAction  = QtGui.QAction(u'Открыть \'Реестр\'', self)
        self.connect(self.sayHelloAction,
                     QtCore.SIGNAL('triggered()'),
                     self.sayHello)
        self.connect(self.openLRAction,
                     QtCore.SIGNAL('triggered()'),
                     self.prepareMakeTab)
        self.tableView.createPopupMenu([self.openLRAction])
    def sayHello(self):
        msg = QtGui.QMessageBox.information(
                                            self,
                                            u'Окошко \'Привет\'',
                                            u'Привет, друг',
                                            QMessageBox.Ok)

    def prepareMakeTab(self):
        model = self.tableView.model()
        if isinstance(model, CCaseServicesAccountsTableModel):
            self.makeALRTab()
            if self.execDone:
                index = self.tableView.currentIndex()
                self.modelRowChanged(index)
                self.tabView.setCurrentIndex(1)
        if isinstance(model, (CProfileTableModel, CKSGGroupsTableModel)):
            self.makeProfileDistinctTab()
            if self.execDone:
                index = self.tableView.currentIndex()
                self.modelRowChanged(index)
                self.tabView.setCurrentIndex(1)

    def connectSelection(self):
        selectionModel = self.tableView.selectionModel()
        QtCore.QObject.connect(
            selectionModel,
            QtCore.SIGNAL('currentRowChanged(QModelIndex, QModelIndex)'),
            self.modelRowChanged)

    def modelRowChanged(self, current, previous=None):
        model = self.tableView.model()
        id = self.currentId(current)
        if isinstance(model, CCaseServicesAccountsTableModel):
            if self.tabALR:
                self.tabALR.setAccountId(id)
                self.tabALR.setIsContext(False)
        elif isinstance(model, (CProfileTableModel, CKSGGroupsTableModel)):
            if self.tabProfileDistinct:
                profileId = self.currentId(current)
                self.tabProfileDistinct.setProfileId(id)
                self.tabProfileDistinct.setIsContext(False)

    def setWindowTitleDB(self):
        if self.EISDb:
            eisPreferences = QtGui.qApp.getEISPreferences()
            txt = u'Утилита для анализа данных ЕИС.ОМС.ВМУ :: ' + eisPreferences['database']
        else:
            txt = u'Утилита для анализа данных ЕИС.ОМС.ВМУ :: Нет подключения!'
        self.setWindowTitle(txt)

    def prepareStatusBar(self):
        self.progressBar = QProgressBar()
        self.progressBar.setMaximumWidth(200)
        self.progressBar.setMaximumHeight(self.statusbar.height()/2)
        self.progressBarVisible = False

    def hideProgressBar(self):
        self.statusbar.removeWidget(self.progressBar)
        self.progressBarVisible = False

    def setPrintButtonALRMenu(self):
        self.btnPrint.setMenu(self.mnuALRPrint)


    @pyqtSignature('int')
    def on_tabView_tabCloseRequested(self, index):
        if index:
            self.closeTabHelper(index)

    def closeTabHelper(self, index):
        if index < self.tabView.count():
            self.tabView.removeTab(index)
            self.closeTabHelper(index)
        else:
            self.tabView.removeTab(index)

    @pyqtSignature('int')
    def on_tabView_currentChanged(self,  index):
        self.setCursor(QtCore.Qt.WaitCursor)
        model = self.tableView.model()
        if self.tabALR and self.tabALR.accountId and index == 1 and (not self.tabALR.contextIsSet):
            self.tabALR.setALRContext(self.patientId, self.profileId)
            self.tmp = copy(self.btnPrint)
            self.setPrintButtonALRMenu()
        if self.tabProfileDistinct and self.tabProfileDistinct.profileId and index == 1 and (not self.tabProfileDistinct.contextIsSet):
            row   = self.tableView.currentIndex().row()
            if row in model.numKsgHeader:
                pass
            else:
                self.tabProfileDistinct.setProfileDistinctContext()
        widget = self.tabView.widget(index)
        if isinstance(widget, CTabAccountListRegister):
            self.actPrint.setEnabled(False)
            self.actPrintRefuses.setEnabled(True)
            self.actPrintBillServices.setEnabled(True)
            self.actPrintExpandedServices.setEnabled(True)
            self.actPrintProfileDistinct.setEnabled(False)
        elif isinstance(widget, CTabProfileDistinct):
            self.actPrint.setEnabled(False)
            self.actPrintRefuses.setEnabled(False)
            self.actPrintBillServices.setEnabled(False)
            self.actPrintExpandedServices.setEnabled(False)
            self.actPrintProfileDistinct.setEnabled(True)
        else:
            self.actPrint.setEnabled(True)
            self.actPrintRefuses.setEnabled(False)
            self.actPrintBillServices.setEnabled(False)
            self.actPrintExpandedServices.setEnabled(False)
            self.actPrintProfileDistinct.setEnabled(False)
        self.setCursor(QtCore.Qt.ArrowCursor)

    @pyqtSignature('QModelIndex')
    def on_tableView_doubleClicked(self, index):
        self.prepareMakeTab()


    @pyqtSignature('int')
    def on_chkDatePP_stateChanged(self, state):
        if state == QtCore.Qt.Checked and self.chkDatePP.isEnabled():
            self.edtDatePP.setEnabled(True)
        elif state == QtCore.Qt.Unchecked and self.chkDatePP.isEnabled():
            self.edtDatePP.setEnabled(False)

    @pyqtSignature('int')
    def on_chkDatePay_stateChanged(self, state):
        if state == QtCore.Qt.Checked and self.chkDatePay.isEnabled():
            self.edtDatePay.setEnabled(True)
        elif state == QtCore.Qt.Unchecked and self.chkDatePay.isEnabled():
            self.edtDatePay.setEnabled(False)

    @pyqtSignature('int')
    def on_chkNoPP_stateChanged(self, state):
        if state == QtCore.Qt.Checked and self.chkNoPP.isEnabled():
            self.edtNoPP.setEnabled(True)
        elif state == QtCore.Qt.Unchecked and self.chkNoPP.isEnabled():
            self.edtNoPP.setEnabled(False)


    @pyqtSignature('QDate')
    def on_edtDateFrom_dateChanged(self, date):
        self.edtDateTo.setMinimumDate(date)
    @pyqtSignature('QDate')
    def on_edtDateTo_dateChanged(self, date):
        self.edtDateFrom.setMaximumDate(date)

    @pyqtSignature('')
    def on_treeQuery_itemSelectionChanged(self):
        value = self.treeQuery.selectedItems()[0]
        if value.parent():
            parentName = value.parent().text(0)
            currValue = value.text(0)
            if self.EISDb:
                if parentName == u'Счет':
                    self.setDatesEnabled(True)
                    self.cmbSMO.setEnabled(True)
                    self.cmbStatus.setEnabled(True)
                    self.setEnabledFiltersPP(True)
                    if currValue == u'За период':
                        self.setChkClientFilterEnabled(True)
                        self.setChkProfileEnabled(True)
                        self.order = 'ACCOUNT_DATE'
                        self.openLRAction.setEnabled(True)

                    elif currValue == u'Визиты':
                        self.setChkClientFilterEnabled(False)
                        self.setChkProfileEnabled(False)
                        self.order = 'CASE_SERVICES_VISIT.DATE_BEGIN'
                        self.openLRAction.setEnabled(False)
                elif (parentName == u'Отчеты' or parentName == u'По профилям' or parentName == u'По КСГ' or parentName == u'По отказам' or parentName == u'По видам медецинской помощи') and currValue != u'По отказам':
                    if parentName == u'По профилям' or currValue == u'По профилям':
                        self.order = 'PROFILE_INFIS_CODE'
                    if parentName == u'По видам медецинской помощи' or currValue == u'По видам медецинской помощи':
                        self.order = 'PROFILE_INFIS_CODE'
                    if parentName == u'По КСГ' or currValue == u'По КСГ':
                        self.order = 'PROFILE_INFIS_CODE'
                    self.setDatesEnabled(True)
                    if parentName == u'По отказам':
                        self.cmbSMO.setEnabled(True)
                    else:
                        self.cmbSMO.setEnabled(False)
#                    self.cmbSMO.setCurrentIndex(0)
                    self.cmbStatus.setEnabled(False)
#                    self.cmbStatus.setCurrentIndex(0)
                    self.openLRAction.setEnabled(True)
                    self.setEnabledFiltersPP(False)
                    self.setChkProfileEnabled(False)
                else:
                    self.setEnabledAll(False)
        else:
            self.setEnabledAll(False)

    def setEnabledAll(self, bool):
        self.setDatesEnabled(bool)
        self.cmbSMO.setEnabled(bool)
        self.cmbStatus.setEnabled(bool)
        self.openLRAction.setEnabled(bool)
        self.setEnabledFiltersPP(bool)
        self.setChkClientFilterEnabled(bool)
        self.setChkProfileEnabled(bool)

    def setEnabledFiltersPP(self, bool):
        self.chkDatePay.setEnabled(bool)
        self.chkDatePP.setEnabled(bool)
        self.chkNoPP.setEnabled(bool)
        if self.chkDatePay.isChecked():
            self.edtDatePay.setEnabled(bool)
        if self.chkDatePP.isChecked():
            self.edtDatePP.setEnabled(bool)
        if self.chkNoPP.isChecked():
            self.edtNoPP.setEnabled(bool)

    @pyqtSignature('')
    def on_btnExit_clicked(self):
        self.close()

    @pyqtSignature('')
    def on_btnExport_clicked(self):
        model = self.tableView.model()
        where = self.getCondition(model.whoIAm())
        if model:
            if isinstance(model, (CProfileTableModel, CKSGGroupsTableModel, CRefusesTableModel)):
                stmt = None
            else:
                stmt = self.stmt % ('', '', where, self.orderForExport)
            preferences = QtGui.qApp.preferences
            settings = preferences.getSettings()
            settings.beginGroup('exportSettings')
            saveDir = settings.value('savePath', QtCore.QVariant('')).toString()
            settings.endGroup()
            dlg = CResultProgressDialog(self.tableView, saveDir, stmt)
            dlg.linesNumber = forceInt(self.edtEntriesAmount.text())
            dlg.exec_()

    @pyqtSignature('')
    def on_btnPrint_clicked(self):
        html = self.contentToHTML()
        view = CReportViewDialog(self)
        html = self.setReportResume(html)
        view.setText(html)
        view.exec_()


    def setReportResume(self, html, option=None):
        model = self.tableView.model()
        ksgGroups = QtGui.qApp.ksgGroups
        cursor = QtGui.QTextCursor(html)
        cursor.movePosition(cursor.End)
        cursor.setCharFormat(CReportBase.ReportBody)
        if self.cmbSMO.currentText() != '-' and self.cmbSMO.currentText() !='-    ':
            payer = self.cmbSMO.itemData(self.cmbSMO.currentIndex()).toString()
            resume = u'Итого по '+payer
        else:
            resume = u'Итого'
        if not isinstance(model, (CProfileTableModel, CKSGGroupsTableModel, CRefusesTableModel)):
            resume += '\n'+u'Счетов: '+ self.edtEntriesAmount.text()+'\n'
        elif isinstance(model, CRefusesTableModel):
            resume += '\n'+u'Отказов: '+ self.edtEntriesAmount.text()+'\n'
        else:
            resume += '\n'+u'Позиций: %d'+'\n'
            if isinstance(model, (CProfileTableModel, )):
                resume = resume % len(model.idList())
            else:
                resume = resume % (len(model.idList())-len(ksgGroups))
        if isinstance(model, CRefusesTableModel):
            resume += u'Сумма отказов: ' + self.edtRefuse.text()+'\n'
        else:
            resume += u'Сумма выставленная: ' + self.edtCommonTotalSumm.text()+'\n'
            resume += u'Сумма отказанная: ' + self.edtRefuse.text()+'\n'
            resume += u'Всего: ' + self.edtCommonPaySumm.text()+'\n\n'
        resume += forceString(QDateTime.currentDateTime())

        if Qt.mightBeRichText(resume):
            cursor.insertHtml(resume)
        else:
            cursor.insertText(resume)
        cursor.insertBlock()
        return html

    def setReportHeader(self, v):
        self.reportHeader = ''
        while v.parent():
            self.reportHeader = v.text(0).toLower()+' '+self.reportHeader
            v = v.parent()
        self.reportHeader = v.text(0)+' '+self.reportHeader
    @pyqtSignature('')
    def on_btnExecQuery_clicked(self):
        self.tabView.setCurrentIndex(0)
        self.lblEntriesAmount.setText(u'Кол-во записей')
        self.orderForExport = self.order
        model = self.tableView.model()
        self.makeFiltersValue()
        if model:
            model.reset()
            model.clear()
            self.closeTabHelper(1)
        value = self.treeQuery.selectedItems()[0]
        self.setReportHeader(value)
        self.tabView.setTabText(0, self.reportHeader)
        if value.parent().text(0) == u'Счет':
            if unicode(value.text(0)) == u'За период':
                if self.chkClientFilter.isChecked():
                    self.patientId = self.getClientId()
                    if not bool(self.patientId):
                        return
                else:
                    self.patientId = None
                    self.profileId = None
            else:
                self.patientId = None
                self.profileId = None
            self.setCursor(QtCore.Qt.WaitCursor)
            self.execQueryBill(value)
        elif value.parent().text(0) == u'Отчеты' or value.parent().text(0) == u'По профилям' or value.parent().text(0) == u'По КСГ' or value.parent().text(0) == u'По отказам' or value.parent().text(0) == u'По видам медецинской помощи':
            self.patientId = None
            self.profileId = None
            self.setCursor(QtCore.Qt.WaitCursor)
            self.execQueryReport(value)
        self.setCursor(QtCore.Qt.ArrowCursor)


    def execQueryBill(self, v):
        value = v.text(0)
        self.actPrintProfileDistinct.setVisible(False)
        if self.EISDb and value == u'За период':
            self.setVisibleFourEdits(True)
            self.currentActive = 'bill'
            self.makeDifferentQueries('bill')
            self.addModels('Bill', CCaseServicesAccountsTableModel(self))
            self.modelBill.setIdList(self.checkUniqueIdInList(self.getIdList('bill')))
            self.execDone = True
            self.setModels(self.tableView, self.modelBill, self.selectionModelBill)
            self.connectSelection()
            self.setFourEdits()
            self.makeALRTab()
            self.btnPrint.setEnabled(True)
        elif self.EISDb and value == u'Визиты':
            self.setVisibleFourEdits(False)
            self.cmbStatus.setEnabled(False)
            self.currentActive = 'visits'
            self.makeDifferentQueries('visits')
            self.addModels('Visits', CVisitsTableModel(self))
            self.modelVisits.setIdList(self.checkUniqueIdInList(self.getIdList('visits')))
            self.setModels(self.tableView, self.modelVisits, self.selectionModelVisits)
            self.execDone = True
            self.setFourEdits()
            self.btnPrint.setEnabled(True)
            self.removeTabALR()
        else:
            self.removeTabALR()
            self.removeModelContext(self.tableView.model())
            self.tabView.setTabText(0, u'')


    def execQueryReport(self, v):
        self.actPrintProfileDistinct.setVisible(True)
        dFrom = self.edtDateFrom.date().toString('dd.MM.yyyy')
        dTo   = self.edtDateTo.date().toString('dd.MM.yyyy')
        value = v.text(0)
        self.setVisibleFourEdits(True)
        if v.parent().text(0) == u'По профилям' or v.parent().text(0) == u'По КСГ' or v.parent().text(0) == u'По видам медецинской помощи':
            dField = None
            if value == u'По дате формирования счета':
                dDateCol = 'ACCOUNT_DATE'
            elif value == u'По дате тарифного месяца':
                dDateCol = 'TARIFF_MONTH'
            elif value == u'По дате подтверждения':
                dDateCol = 'CONFIRM_DATE'
            else:
                dDateCol = None
            checkIndexes()
            self.currentActive = 'profiles'
            self.makeDifferentQueries('profiles')
            if u'по профилям' in self.reportHeader:
                QtGui.qApp.ksgGroups = makeActualKsgGroups(dFrom, dTo, True, 'CASE_SERVICES_ACCOUNTS.'+dDateCol)
                model = CKSGGroupsTableModel
            elif u'по видам медецинской помощи' in self.reportHeader:
                QtGui.qApp.ksgGroups = makeActualKsgGroups(dFrom, dTo, True, 'CASE_SERVICES_ACCOUNTS.'+dDateCol)
                model = CProfileTableModel
            elif u'по ксг' in self.reportHeader:
                QtGui.qApp.ksgGroups = makeActualKsgGroups(dFrom, dTo, False, 'CASE_SERVICES_ACCOUNTS.'+dDateCol)
                model = CKSGGroupsTableModel
            self.addModels('Profiles', model(self))

            self.modelProfiles.setDDateCol(dDateCol)
            if v.text(0) == u'По профилям' or v.parent().text(0) == u'По профилям':
                self.modelProfiles.setIdList(self.getKsgList())
            elif v.text(0) == u'По видам медецинской помощи' or v.parent().text(0) == u'По видам медецинской помощи':
                self.modelProfiles.setIdList(self.getProfileList())
            elif v.text(0) == u'По КСГ' or v.parent().text(0) == u'По КСГ':
                self.modelProfiles.setIdList(self.getKsgList())
            where = self.getCondition('profile')
            self.modelProfiles.setWhere(where)
            self.execDone = True
            self.removeTabALR()
            self.makeProfileDistinctTab(where)
            self.btnExport.setEnabled(True)
            self.btnPrint.setEnabled(True)
            self.setModels(self.tableView, self.modelProfiles, self.selectionModelProfiles)
            self.connectSelection()
            self.setFourEdits()
            self.tabProfileDistinct.setCondDates(dFrom, dTo)
#            print datetime.datetime.time(datetime.datetime.now())
        if v.parent().text(0) == u'По отказам':
            QtGui.qApp.ksgGroups = makeActualKsgGroups(dFrom, dTo)
            self.addModels('Refuses', CRefusesTableModel(self))
            if value == u'В периоде выставления счетов':
                self.modelRefuses.setWhoIAm('refuses')
            elif value == u'В периоде получения отказа':
                self.modelRefuses.setWhoIAm('refusesDate')
            self.setModels(self.tableView, self.modelRefuses, self.selectionModelRefuses)
            where = self.getCondition(self.modelRefuses.whoIAm())
            self.modelRefuses.setIdList(self.getRefusesList())
            self.modelRefuses.setWhere(where)
            self.execDone = True
            self.btnExport.setEnabled(True)
            self.btnPrint.setEnabled(True)
            self.lblEntriesAmount.setText(u'Кол-во отказов')
            self.setVisibleFourEdits(False, 'refuses')
            self.setFourEdits()

    def setVisibleFourEdits(self,bool, who=None):
        if who == 'refuses':
            self.edtCommonTotalSumm.setVisible(bool)
            self.lblCommonTotalSumm.setVisible(bool)
            self.edtCommonPaySumm.setVisible(bool)
            self.lblCommonPaySumm.setVisible(bool)
        else:
            self.lblCommonPaySumm.setVisible(bool)
            self.lblCommonTotalSumm.setVisible(bool)
            self.edtCommonPaySumm.setVisible(bool)
            self.edtCommonTotalSumm.setVisible(bool)

    def makeALRTab(self):
        if self.tabALR:
            self.removeTabALR()
        if u'Реестр счета' not in self.tabsList:
            self.tabALR = CTabAccountListRegister()
            self.setActNumber() # Акт об отказах, иначе какие-то левые к счету приписываются
            self.tabView.addTab(self.tabALR, u'Реестр счета')
            self.tabsList.append(u'Реестр счета')
    def makeProfileDistinctTab(self, where =''):
        if self.tabProfileDistinct:
            self.removeTabProfileDistinct()
            self.closeTabHelper(1)
        if u'Реестр профиля' not in self.tabsList:
            where = self.getCondition('profile')
            self.tabProfileDistinct = CTabProfileDistinct(self.tabView)
            self.tabProfileDistinct.setWhere(where)
            self.tabView.addTab(self.tabProfileDistinct, u'Реестр профиля')
            self.tabsList.append(u'Реестр профиля')

    def setActNumber(self):
        index = self.tableView.currentIndex()
        if index:
            row = index.row()
            model = self.tableView.model()
            actNumber = forceString(model.data(model.index(row, 11)))
            self.tabALR.setActNumber(actNumber)

    def getRefusesList(self):
#        print datetime.datetime.time(datetime.datetime.now())
        db = QtGui.qApp.db
        model = self.tableView.model()
        where = self.getCondition(model.whoIAm())
        stmt = 'SELECT DISTINCT ID_REF_REASON FROM REFUSES_OBJ JOIN CASE_SERVICES ON CASE_SERVICES.ID_SERVICE=REFUSES_OBJ.ID_SERVICE JOIN CASE_SERVICES_ACCOUNTS ON CASE_SERVICES_ACCOUNTS.ID_ACCOUNT=CASE_SERVICES.ID_ACCOUNT JOIN SMO ON CASE_SERVICES_ACCOUNTS.ID_PAYER = SMO.ID_SMO %s' % where
        result = makeListFromQuery(db.query(stmt))
#        print datetime.datetime.time(datetime.datetime.now())
        return result
    def getProfileList(self):
#        print datetime.datetime.time(datetime.datetime.now())
        ksgGroups = QtGui.qApp.ksgGroups
        return ksgGroups.keys()
#        print datetime.datetime.time(datetime.datetime.now())
    def getKsgList(self):
#        print datetime.datetime.time(datetime.datetime.now())
        ksgGroups = QtGui.qApp.ksgGroups
        list = []
        if ksgGroups:
            for i in ksgGroups.keys():
                list += [i]+ksgGroups[i]
#        print datetime.datetime.time(datetime.datetime.now())
        return list

    def checkUniqueIdInList(self, l):
#        for v in list:
#            if list.count(v) > 1:
#                for i in range(list.count(v)-1):
#                    list.pop(list.index(v))
#        return list
        result = list(set(l)&set(l))
        return result

    def getIdList(self, tableName, value=None):
        db = QtGui.qApp.db
        stmt = self.stmt
        if self.headerSortIndicator == Qt.DescendingOrder:
            orderSort = ' DESC'
        else:
            orderSort = ''
        if tableName == 'bill':
            where =  self.getCondition('bill')
            dictJoins = {'VMU_ACC_STATUS':['CASE_SERVICES_ACCOUNTS.ID_STATUS',
                                           'VMU_ACC_STATUS.ID_STATUS', ''],
                         'PAY_SUMMS':['PAY_SUMMS.ID_ACCOUNT',
                                      'CASE_SERVICES_ACCOUNTS.ID_ACCOUNT', 'LEFT OUTER'],
                         'PAY_SHEETS':['PAY_SUMMS.ID_PAY_CALC',
                                       'PAY_SHEETS.ID_PAY_CALC', 'LEFT OUTER'],
                         'SMO':['CASE_SERVICES_ACCOUNTS.ID_PAYER',
                                'SMO.ID_SMO', ''],
                         'REFUSES':['REFUSES.ID_ACCOUNT',
                                    'CASE_SERVICES_ACCOUNTS.ID_ACCOUNT', 'LEFT OUTER']
                         }
            dictJoins.update({'CASE_SERVICES':['CASE_SERVICES.ID_ACCOUNT',
                                    'CASE_SERVICES_ACCOUNTS.ID_ACCOUNT', 'LEFT OUTER']})
            if bool(self.patientId):
                dictJoins.update({
                                 'CASES':['CASES.ID_CASE', 'CASE_SERVICES.ID_CASE', 'LEFT OUTER']})
                where = where + ' AND CASES.ID_PATIENT=\'%s\'' % self.patientId
            if self.chkProfile.isChecked():
                self.profileId = self.cmbProfile.value()
                if self.profileId:
                    where = where + ' AND CASE_SERVICES.ID_PROFILE=%d' % self.profileId
            else:
                self.profileId = 0
            stmt2 = db.getSelectStmtWithJoin(
                            [''],
                             'CASE_SERVICES_ACCOUNTS',
                             dictJoins) + '%s ORDER BY %s ' + orderSort
            if self.order == 'ID_ACCOUNT':
                stmt = stmt % ('CASE_SERVICES_ACCOUNTS.ID_ACCOUNT, ',self.order+' AS ID ', where, self.order)
            else:
                if '.' in self.order:
                    order = self.order[:-1].split('.')[1]
                else:
                    order = self.order[:-1]
                stmt = stmt2 % ('CASE_SERVICES_ACCOUNTS.ID_ACCOUNT, ',(self.order+' AS '+order+' '), where, self.order)


        elif tableName == 'visits':
            where = self.getCondition(tableName)
            if self.order == 'ID_VISIT':
                stmt = stmt % ('ID_VISIT, ', self.order+' AS ID, ', where, self.order+orderSort)
            else:
                if '.' in self.order:
                    order = self.order[:-1].split('.')[1]
                else:
                    order = self.order[:-1]
                stmt = stmt % ('ID_VISIT, ', self.order+ ' AS '+order[:-1]+', ', where, self.order+orderSort)
#        print datetime.datetime.time(datetime.datetime.now())
        query = db.query(stmt)
        result = []
#        print datetime.datetime.time(datetime.datetime.now())
        while query.next():
            result.append(query.value(0))
#        print datetime.datetime.time(datetime.datetime.now())
        return result


    def setFourEdits(self):
        db = QtGui.qApp.db
        v = self.treeQuery.selectedItems()[0]
        value = v.text(0)
        model = self.tableView.model()
        if value == u'За период':
            dictJoins = {'VMU_ACC_STATUS':['CASE_SERVICES_ACCOUNTS.ID_STATUS',
                                       'VMU_ACC_STATUS.ID_STATUS', ''],
                     'PAY_SUMMS':['PAY_SUMMS.ID_ACCOUNT',
                                  'CASE_SERVICES_ACCOUNTS.ID_ACCOUNT', 'LEFT OUTER'],
                     'PAY_SHEETS':['PAY_SUMMS.ID_PAY_CALC',
                                   'PAY_SHEETS.ID_PAY_CALC', 'LEFT OUTER'],
                     'SMO':['CASE_SERVICES_ACCOUNTS.ID_PAYER',
                            'SMO.ID_SMO', ''],
                     'REFUSES':['REFUSES.ID_ACCOUNT',
                                'CASE_SERVICES_ACCOUNTS.ID_ACCOUNT', 'LEFT OUTER']}
            dictJoins.update({'CASE_SERVICES':
                            ['CASE_SERVICES.ID_ACCOUNT', 'CASE_SERVICES_ACCOUNTS.ID_ACCOUNT', 'LEFT OUTER']})
            where = self.getCondition(model.whoIAm())
            if bool(self.patientId):
                dictJoins.update({
                                 'CASES':['CASES.ID_CASE', 'CASE_SERVICES.ID_CASE', 'LEFT OUTER']})
                where = where + ' AND CASES.ID_PATIENT=\'%s\'' % self.patientId
            if bool(self.profileId):
                where = where + ' AND CASE_SERVICES.ID_PROFILE=%d' % self.profileId
            stmtTmp = db.getSelectStmtWithJoin([''],
                                            'CASE_SERVICES_ACCOUNTS',
                                            dictJoins, False) +' %s'
            stmt  = stmtTmp % ('COUNT(DISTINCT CASE_SERVICES_ACCOUNTS.ID_ACCOUNT), SUM(DISTINCT CASE_SERVICES_ACCOUNTS.TOTAL_SUMM)', '' , where)
            stmtR = stmtTmp % ('CASE_SERVICES_ACCOUNTS.ID_ACCOUNT', '', where)
            stmtRefuse = '''
                SELECT
                    SUM(SUMM_REFUSE)
                FROM
                    REFUSES
                    JOIN REFUSES_OBJ ON
                    REFUSES.ID_REFUSE = REFUSES_OBJ.ID_REFUSE
                WHERE
                    ACT_DATE >= \'%s\'
                AND
                    REFUSES.ID_ACCOUNT IN
                        (%s)
            ''' % (self.edtDateFrom.text(), stmtR)
            query = self.EISDb.query(stmt)
            query.first()
            linesNumber   = len(model.idList())
            sumTOTAL_SUMM = 0
#            sumTOTAL_SUMM = query.value(1).toDouble()[0]
            for i in range(linesNumber):
                sumTOTAL_SUMM += model.data(model.index(i, 6) ).toDouble()[0]
            sumPAY_SUMM   = query.value(2).toDouble()[0]
            query = self.EISDb.query(stmtRefuse)
            query.first()
            sumSUMM_REFUSE = query.value(0).toDouble()[0]
            ts = '%.2f' % (sumTOTAL_SUMM)
            ps = '%.2f' % (sumTOTAL_SUMM-sumSUMM_REFUSE)
            sr = '%.2f' % (sumSUMM_REFUSE)
            self.edtCommonTotalSumm.setText(ts)
            self.edtRefuse.setText(sr)
            self.edtCommonPaySumm.setText(ps)
            if linesNumber:
                self.btnExport.setEnabled(True)
        elif value == u'Визиты':
            linesNumber = len(model.idList())
            self.edtCommonTotalSumm.setText('')
            self.edtRefuse.setText('')
            if linesNumber:
                self.btnExport.setEnabled(True)
            where = self.getCondition(model.whoIAm())
            query = db.query('SELECT SUM(SERVICE_COST) FROM CASE_SERVICES_VISIT JOIN CASE_SERVICES ON CASE_SERVICES.ID_SERVICE=CASE_SERVICES_VISIT.ID_SERVICE %s' % where)
            query.first()
            ts = query.value(0).toDouble()[0]
            self.edtCommonPaySumm.setText('%10.2f'%ts)
        elif value == u'По профилям' or value == u'По КСГ' or v.parent().text(0) == u'По профилям' or v.parent().text(0) == u'По КСГ' or v.parent().text(0) == u'По видам медецинской помощи':
            linesNumber = len(model.idList())
            ts  = 0.0
            ref = 0.0
            ps  = 0.0
            for i in range(linesNumber):
                if isinstance(model, (CProfileTableModel, )):
                    ts  += model.data(model.index(i, 2) ).toDouble()[0]
                    ref += model.data(model.index(i, 3) ).toDouble()[0]
                    ps  += model.data(model.index(i, 4) ).toDouble()[0]
                    model.data(model.index(i, 1))
                if isinstance(model, CKSGGroupsTableModel):
                    if i not in model.numKsgHeader:
                        ts  += model.data(model.index(i, 4) ).toDouble()[0]
                        ref += model.data(model.index(i, 5) ).toDouble()[0]
                        ps  += model.data(model.index(i, 6) ).toDouble()[0]
                        model.data(model.index(i, 2))
                        model.data(model.index(i, 3))

            self.edtCommonPaySumm.setText('%10.2f'%ps)
            self.edtRefuse.setText('%10.2f'%ref)
            self.edtCommonTotalSumm.setText('%10.2f'%ts)
        elif v.parent().text(0) == u'По отказам':
            count = 0
            ref   = 0
            for i in range(len(model.idList())):
                count += model.data(model.index(i, 1)).toInt()[0]
                ref   += model.data(model.index(i, 2)).toDouble()[0]
            linesNumber = count
            self.edtRefuse.setText('%10.2f'%ref)
#            print datetime.datetime.time(datetime.datetime.now())

        else:
            self.btnExport.setEnabled(False)
        self.edtEntriesAmount.setText(str(linesNumber))

    def removeModelContext(self, model):
        if model:
            model.reset()
            model.clear()
            m = CMySqlTableModel(self)
            self.tableView.setModel(m)
            self.edtEntriesAmount.setText('')
            self.edtCommonTotalSumm.setText('')
            self.edtCommonPaySumm.setText('')
            self.edtRefuse.setText('')
            self.btnExport.setEnabled(False)
            self.btnPrint.setEnabled(False)

    def removeTabALR(self):
        self.tabView.removeTab(self.tabView.indexOf(self.tabALR))
        if u'Реестр счета' in self.tabsList:
            self.tabsList.remove(u'Реестр счета')
        self.tabALR = None
    def removeTabProfileDistinct(self):
        self.tabView.removeTab(self.tabView.indexOf(self.tabProfileDistinct))
        if u'Реестр профиля' in self.tabsList:
            self.tabsList.remove(u'Реестр профиля')
        self.tabProfileDistinct = None

    def currentId(self, index):
        return self.tableView.itemId(index)

    def getProgressBar(self):
        self.statusbar.addWidget(self.progressBar)
        self.progressBar.show()
        return self.progressBar

    def makeFiltersValue(self):
        self.filtersValue['dateFrom'] = self.edtDateFrom.date().toString('dd.MM.yyyy')
        self.filtersValue['dateTo']   = self.edtDateTo.date().toString('dd.MM.yyyy')
        if self.chkNoPP.isChecked():
            self.filtersValue['noPP'] = forceString(self.edtNoPP.value())
        else:
            self.filtersValue['noPP'] = ''
        if self.chkDatePP.isChecked():
            self.filtersValue['datePP'] = self.edtDatePP.date().toString('dd.MM.yyyy')
        else:
            self.filtersValue['datePP'] = ''
        if self.chkDatePay.isChecked():
            self.filtersValue['payDate'] = self.edtDatePay.date().toString('dd.MM.yyyy')
        else:
            self.filtersValue['payDate'] = ''
        if self.cmbSMO.isEnabled() and self.cmbSMO.currentText() != '-' and self.cmbSMO.currentText() != '-    ':
            self.filtersValue['smoShort'] = self.cmbSMO.currentText()
            self.filtersValue['smoLong']  = self.cmbSMO.itemData(self.cmbSMO.currentIndex()).toString()
        else:
            self.filtersValue['smoShort'] = ''
            self.filtersValue['smoLong']  = ''
        if self.cmbStatus.isEnabled() and self.cmbStatus.currentText() != '-':
           self.filtersValue['status'] =  self.cmbStatus.currentText()
        else:
            self.filtersValue['status'] = ''
        self.filtersValue['patientSurname'] = forceStringEx(self.edtClientSurname.text())
        self.filtersValue['patientName'] = forceStringEx(self.edtClientName.text())
        self.filtersValue['patientSecondName'] = forceStringEx(self.edtClientSecondName.text())
        self.filtersValue['patientBirthday'] = forceString(self.edtClientBirthday.date())
        self.filtersValue['patientSex'] = forceString(self.cmbClientSex.currentText())


    def contentToHTML(self):
        reportHeader = self.getReportHeader()
        self.tableView.setReportHeader(reportHeader)
        reportDescription = self.getReportDescription()
        self.tableView.setReportDescription(reportDescription)
        return self.tableView.contentToHTML()

    def getReportHeader(self):
        model  = self.tableView.model()
        LPU    = QtGui.qApp.organisation
        LPUNet = QtGui.qApp.orgNet
        beg = u'<p align=center><h3>'
        end = u'</h3></p>'
        if isinstance(model, CCaseServicesAccountsTableModel):
            txt = u'Сведения о составе платежных документов'
        elif isinstance(model, (CProfileTableModel, )):
            txt = u'Сведения об объеме медицинской помощи в разрезе профилей'
        elif isinstance(model, CKSGGroupsTableModel):
            txt = u'Сведения об объеме медицинской помощи по клинико-статистическим группам'
        else:
            return self.reportHeader
        if LPU:
            txt += '<p>'+LPU+'</p>'
        if LPUNet:
            txt += '<p>'+LPUNet+'</p>'
        return beg+txt+end

    def getReportDescription(self, option=None):
        model = self.tableView.model()
        rd =u'<p align=left>\n'
        if not isinstance(model, (CProfileTableModel, CKSGGroupsTableModel)):
            rd += u'<p align=left>СМО: '+self.filtersValue['smoShort']+'</p>\n'
            payer = self.filtersValue['smoLong']
        if option != 'expandedServices':
            rd += u'<p>Даты платежных документов от '+self.filtersValue['dateFrom']+QString(u' до ')+self.filtersValue['dateTo']+'</p>\n'
        if not isinstance(model, (CProfileTableModel, CKSGGroupsTableModel, CRefusesTableModel)):
            rd += u'<p>Статус счета: '+self.filtersValue['status']+'</p>\n'
        rd += u'<p>Дата формирования счета: '+forceString(QDateTime.currentDateTime())+'</p>\n\n'
        if not isinstance(model, (CProfileTableModel, CKSGGroupsTableModel)):
            rd += u'<p align=center><b>Плательщик '+payer+'</b></p>\n'
        rd += '</p>\n'
        if option != 'expandedServices':
            if not isinstance(model, (CProfileTableModel, CKSGGroupsTableModel)):
                rd += self.getFiltersResume()
        return rd

    def getFiltersResume(self):
        resume =''
        if self.filtersValue['noPP'] != '':
            resume +=  u'№ ПП=%s   ' % self.filtersValue['noPP']
        if self.filtersValue['datePP'] != '':
            resume += u'Дата: %s   ' % self.filtersValue['datePP']
        if self.filtersValue['payDate'] != '':
            resume += u'Проводка: %s   ' % self.filtersValue['payDate']
        resume += u'Сумма=%s' % self.edtCommonPaySumm.text()
        return resume


    def getClientId(self):
        surname    = forceStringEx(self.edtClientSurname.text())
        name       = forceStringEx(self.edtClientName.text())
        secondName = forceStringEx(self.edtClientSecondName.text())
        birthday   = self.edtClientBirthday.date()
        sex        = unicode(self.cmbClientSex.currentText())


        cond = []
        if bool(surname):
            cond.append('PATIENT_DATA.SURNAME=\'%s\''%surname)
        if bool(name):
            cond.append('PATIENT_DATA.NAME=\'%s\''%name)
        if bool(secondName):
            cond.append('PATIENT_DATA.SECOND_NAME=\'%s\''%secondName)
        if birthday.isValid():
            birthdayStr = forceString(birthday)
            cond.append('PATIENT_DATA.BIRTHDAY=\'%s\''%birthdayStr)
        if bool(sex):
            cond.append('PATIENT_DATA.SEX=\'%s\''%sex)
        if bool(cond):
            patientSelectorDialog = CPatientSelectorDialog(self, cond)
            patientSelectorDialog.exec_()
            patientId = patientSelectorDialog.patientId()
            if patientId:
                return patientId
        return None

    def getCondition(self, c=None):
        datePayCond   = ''
        datePayPPCond = ''
        noPPCond      = ''
        smoCond       = ''
        statusCond    = ''
        clientFilter  = ''
        if not c:
            c = self.tableView.model.whoIAm()
        if c == 'bill':
            dFrom = 'ACCOUNT_DATE'
            dTo   = 'ACCOUNT_DATE'
            if self.chkDatePay.isChecked():
                datePayCond = ' AND EXECUTE_DATE=\'%s\'' % self.filtersValue['payDate']
            if self.chkDatePP.isChecked():
                datePayPPCond = ' AND PAY_DATE=\'%s\'' % self.filtersValue['datePP']
            if self.chkNoPP.isChecked():
                noPPCond = ' AND PAY_NUMBER=\'%s\'' % self.filtersValue['noPP']
        if c == 'visits':
            dFrom = 'CASE_SERVICES_VISIT.DATE_BEGIN'
            dTo   = 'CASE_SERVICES_VISIT.DATE_END'

        if c == 'profile' or  c == 'ksg':
            dFrom = 'CASE_SERVICES.DATE_BEGIN'
            dTo   = 'CASE_SERVICES.DATE_END'
        if c == 'refuses':
            dFrom = 'CASE_SERVICES_ACCOUNTS.ACCOUNT_DATE'
            dTo   = 'CASE_SERVICES_ACCOUNTS.ACCOUNT_DATE'
        if c == 'refusesDate':
            dFrom = 'REFUSES_OBJ.REFUSE_DATE'
            dTo   = 'REFUSES_OBJ.REFUSE_DATE'
        cond = 'WHERE %s <= \'%s\' AND %s >= \'%s\' ' % (dTo, self.filtersValue['dateTo'],
                                                 dFrom, self.filtersValue['dateFrom'])
        if self.filtersValue['smoShort'] != '' and (c == 'bill' or c == 'visits' or c == 'refuses'):
            smoCond = 'AND CODE = \'%s\'' % self.filtersValue['smoShort']
        else:
            smoComd = ''
        if self.filtersValue['status'] != '' and c == 'bill':
            if self.filtersValue['status'] == u'Оплачено':
                statusCond = u' AND STATUS_NAME = \'Включен в ПП\' AND EXECUTE_DATE IS NOT NULL '
            else:
                statusCond = ' AND STATUS_NAME = \'%s\' ' % self.filtersValue['status']
        else:
            statusCond = ''
        cond = cond + smoCond + statusCond + datePayCond + datePayPPCond + noPPCond
        return cond

    def loadEisBase(self):
        self.execDone = False
        self.setCursor(QtCore.Qt.WaitCursor)
        if self.reconnect:
            QtGui.qApp.endWorkWithEISOMS(self.viewsDict)
        try:
            eisPreferences = QtGui.qApp.getEISPreferences()
            self.EISDb   = connectDataBase('FIREBIRD', #driverName
                                                    eisPreferences['server'],   #serverName
                                                    eisPreferences['port'],     #serverPort
                                                    eisPreferences['database'], # databaseName
                                                    eisPreferences['user'],     # userName
                                                    eisPreferences['password'], # password
                                                    'EISDb')    # connectionName

            self.reconnect = True
            QtGui.qApp.db = self.EISDb
            self.fillCmb(self.cmbSMO,'SELECT DISTINCT CODE, SMO_SHORT_NAME FROM SMO')
            self.fillCmb(self.cmbStatus, 'SELECT DISTINCT STATUS_NAME FROM VMU_ACC_STATUS')
            self.loadFiltersSettings()
            checkViews(self.viewsDict)
            self.cmbProfile.loadData()

        except database.CDatabaseException, e:
            QtGui.QMessageBox.critical(
                self,
                u'Ошибка открытия базы данных',
                unicode(e),
                QtGui.QMessageBox.Close)
            self.EISDb = None
            self.setDatesEnabled(False)
            self.setChkClientFilterEnabled(False)
            self.setChkProfileEnabled(False)
            self.btnExport.setEnabled(False)
            self.cmbSMO.setEnabled(False)
            self.cmbStatus.setEnabled(False)
        except Exception, e:
            QtGui.QMessageBox.critical(
                self,
                u'Ошибка открытия базы данных',
                u'Невозможно установить соедиенение с базой данных\n' + unicode(e),
                QtGui.QMessageBox.Close)
            self.EISDb = None
            self.setDatesEnabled(False)
            self.setChkClientFilterEnabled(False)
            self.setChkProfileEnabled(False)
            self.btnExport.setEnabled(False)
            self.cmbSMO.setEnabled(False)
            self.cmbStatus.setEnabled(False)
        self.setCursor(QtCore.Qt.ArrowCursor)

    def setDatesDefaultValue(self):
        currentDate = QtCore.QDate.currentDate()
        curMonth = currentDate.month()
        curYear = currentDate.year()
        startDate = QtCore.QDate(curYear, curMonth, 1)

        self.edtDateTo.setDate(currentDate)
        self.edtDateFrom.setDate(startDate)
        self.edtDateTo.setCalendarPopup(True)
        self.edtDateFrom.setCalendarPopup(True)
        self.edtDatePay.setDate(currentDate)
        self.edtDatePP.setDate(currentDate)

    def setChkClientFilterEnabled(self, bool):
       self.chkClientFilter.setEnabled(bool)

    def setChkProfileEnabled(self, bool):
        self.chkProfile.setEnabled(bool)

    def setDatesEnabled(self, bool):
        self.edtDateFrom.setEnabled(bool)
        self.edtDateTo.setEnabled(bool)
        self.btnExecQuery.setEnabled(bool)


    def setSort(self, col):
        model = self.tableView.model()
        name=model.headerData(col, QtCore.Qt.Horizontal, QtCore.Qt.WhatsThisRole).toString()
        tmp = forceString(name).upper().split(' ')
        if len(tmp) > 1:
            name = ''
            for i in tmp:
                name = name+ i + '_'
            name = name[:-1]
        else:
            name = tmp[0]
        self.order = name
        header=self.tableView.horizontalHeader()
        header.setSortIndicatorShown(True)
        if self.headerSortIndicator == Qt.DescendingOrder:
            self.headerSortIndicator = Qt.AscendingOrder
        else:
            self.headerSortIndicator = Qt.DescendingOrder
        header.setSortIndicator(col, self.headerSortIndicator)
        self.makeDifferentQueries(model.whoIAm())
        self.on_btnExecQuery_clicked()

#        self.tableView.sortByColumn(col, QtCore.Qt.AscendingOrder)

    def fillCmb(self,cmb, stmt):
        db = QtGui.qApp.db
        cmb.clear()
        query = db.query(stmt)
        cmb.addItem('-')
        go = query.first()
        while go:
            cmb.addItem(query.value(0).toString())
            cmb.setItemData(cmb.count()-1, query.value(1))
            go = query.next()
        if query.record().fieldName(0) == u'STATUS_NAME':
            cmb.addItem(u'Оплачено')

    def fillTreeQuery(self):
        treeItems = []
        self.billItem                  = QtGui.QTreeWidgetItem()
        treeItems.append(self.billItem)
        self.billItem.setText(0, u'Счет')
        self.reportItem                = QtGui.QTreeWidgetItem()
        treeItems.append(self.reportItem)
        self.reportItem.setText(0, u'Отчеты')
        self.billItemPeriod            = QtGui.QTreeWidgetItem()
        treeItems.append(self.billItemPeriod)
        self.billItemPeriod.setText(0, u'За период')
        self.billItem.addChild(self.billItemPeriod)
        self.billItemVisit             = QtGui.QTreeWidgetItem()
        treeItems.append(self.billItemVisit)
        self.billItemVisit.setText(0, u'Визиты')
        self.billItem.addChild(self.billItemVisit)

        self.reportItemMedecineTypeHelp = QtGui.QTreeWidgetItem()
        treeItems.append(self.reportItemMedecineTypeHelp)
        self.reportItemMedecineTypeHelp.setText(0, u'По видам медецинской помощи')
        self.reportItem.addChild(self.reportItemMedecineTypeHelp)
        self.reportItemMedecineTypeHelpAccountDate = QtGui.QTreeWidgetItem()
        treeItems.append(self.reportItemMedecineTypeHelpAccountDate)
        self.reportItemMedecineTypeHelpAccountDate.setText(0, u'По дате формирования счета')
        self.reportItemMedecineTypeHelp.addChild(self.reportItemMedecineTypeHelpAccountDate)
        self.reportItemMedecineTypeHelpTariffMonth = QtGui.QTreeWidgetItem()
        treeItems.append(self.reportItemMedecineTypeHelpTariffMonth)
        self.reportItemMedecineTypeHelpTariffMonth.setText(0, u'По дате тарифного месяца')
        self.reportItemMedecineTypeHelp.addChild(self.reportItemMedecineTypeHelpTariffMonth)
        self.reportItemMedecineTypeHelpConfirmDate = QtGui.QTreeWidgetItem()
        treeItems.append(self.reportItemMedecineTypeHelpConfirmDate)
        self.reportItemMedecineTypeHelpConfirmDate.setText(0, u'По дате подтверждения')
        self.reportItemMedecineTypeHelp.addChild(self.reportItemMedecineTypeHelpConfirmDate)

        self.reportItemKSG             = QtGui.QTreeWidgetItem()
        treeItems.append(self.reportItemKSG)
        self.reportItemKSG.setText(0, u'По КСГ')
        self.reportItem.addChild(self.reportItemKSG)
        self.reportItemKsgAccountDate = QtGui.QTreeWidgetItem()
        treeItems.append(self.reportItemKsgAccountDate)
        self.reportItemKsgAccountDate.setText(0, u'По дате формирования счета')
        self.reportItemKSG.addChild(self.reportItemKsgAccountDate)
        self.reportItemKsgTariffMonth = QtGui.QTreeWidgetItem()
        treeItems.append(self.reportItemKsgTariffMonth)
        self.reportItemKsgTariffMonth.setText(0, u'По дате тарифного месяца')
        self.reportItemKSG.addChild(self.reportItemKsgTariffMonth)
        self.reportItemKsgConfirmDate = QtGui.QTreeWidgetItem()
        treeItems.append(self.reportItemKsgConfirmDate)
        self.reportItemKsgConfirmDate.setText(0, u'По дате подтверждения')
        self.reportItemKSG.addChild(self.reportItemKsgConfirmDate)

        self.reportItemProfile         = QtGui.QTreeWidgetItem()
        treeItems.append(self.reportItemProfile)
        self.reportItemProfile.setText(0, u'По профилям')
        self.reportItem.addChild(self.reportItemProfile)
        self.reportItemProfileAccountDate = QtGui.QTreeWidgetItem()
        treeItems.append(self.reportItemProfileAccountDate)
        self.reportItemProfileAccountDate.setText(0, u'По дате формирования счета')
        self.reportItemProfile.addChild(self.reportItemProfileAccountDate)
        self.reportItemProfileTariffMonth = QtGui.QTreeWidgetItem()
        treeItems.append(self.reportItemProfileTariffMonth)
        self.reportItemProfileTariffMonth.setText(0, u'По дате тарифного месяца')
        self.reportItemProfile.addChild(self.reportItemProfileTariffMonth)
        self.reportItemProfileConfirmDate = QtGui.QTreeWidgetItem()
        treeItems.append(self.reportItemProfileConfirmDate)
        self.reportItemProfileConfirmDate.setText(0, u'По дате подтверждения')
        self.reportItemProfile.addChild(self.reportItemProfileConfirmDate)

        self.reportItemRefuse          = QtGui.QTreeWidgetItem()
        treeItems.append(self.reportItemRefuse)
        self.reportItemRefuse.setText(0, u'По отказам')
        self.reportItemRefuseAccountDate = QtGui.QTreeWidgetItem()
        treeItems.append(self.reportItemRefuseAccountDate)
        self.reportItemRefuseAccountDate.setText(0, u'В периоде выставления счетов')
        self.reportItemRefuse.addChild(self.reportItemRefuseAccountDate)
        self.reportItemRefuseDate = QtGui.QTreeWidgetItem()
        treeItems.append(self.reportItemRefuseDate)
        self.reportItemRefuseDate.setText(0, u'В периоде получения отказа')
        self.reportItemRefuse.addChild(self.reportItemRefuseDate)
        self.reportItem.addChild(self.reportItemRefuse)
        self.treeQuery.addTopLevelItem(self.billItem)
        self.treeQuery.addTopLevelItem(self.reportItem)

        for item in treeItems:
            self.treeQuery.setItemExpanded(item, True)


    def loadPreferences(self):
        preferences = getPref(QtGui.qApp.preferences.windowPrefs, self.objectName(), {})
        geometry = getPref(preferences, 'geometry', None)
        if type(geometry) == QVariant and geometry.type() == QVariant.ByteArray and not geometry.isNull():
            self.restoreGeometry(geometry.toByteArray())
        state = getPref(preferences, 'state', None)
        if type(state) == QVariant and state.type() == QVariant.ByteArray and not geometry.isNull():
            self.restoreState(state.toByteArray())
        splitter = getPref(preferences, 'splitter', None)
        if type(splitter) == QVariant and splitter.type() ==QVariant.ByteArray and not splitter.isNull():
            self.splitter.restoreState(splitter.toByteArray())

        appPreferences = QtGui.qApp.preferences.appPrefs

    def savePreferences(self):
        preferences = {}
        setPref(preferences, 'geometry', QVariant(self.saveGeometry()))
        setPref(preferences, 'state',    QVariant(self.saveState()))
        setPref(preferences, 'splitter', QVariant(self.splitter.saveState()))
        setPref(QtGui.qApp.preferences.windowPrefs, self.objectName(), preferences)
        appPreferences = QtGui.qApp.preferences.appPrefs

        preferences = QtGui.qApp.preferences
        settings = preferences.getSettings()
        settings.beginGroup('commonSettings')
        chkFilters = settings.value('saveFilters', QtCore.QVariant(False)).toBool()
        settings.endGroup()
        if chkFilters:
            self.makeFiltersValue()
            settings.beginGroup('filtersSettings')
            settings.setValue('noPP', QVariant(self.filtersValue['noPP']))
            settings.setValue('payDate', QVariant(self.filtersValue['payDate']))
            settings.setValue('datePP', QVariant(self.filtersValue['datePP']))
            settings.setValue('status', QVariant(self.filtersValue['status']))
            settings.setValue('smoShort', QVariant(self.filtersValue['smoShort']))
            settings.setValue('smoLong', QVariant(self.filtersValue['smoLong']))
            settings.setValue('dateFrom', QVariant(self.filtersValue['dateFrom']))
            settings.setValue('dateTo', QVariant(self.filtersValue['dateTo']))
            settings.setValue('chkNoPP', QVariant(self.chkNoPP.isChecked()))
            settings.setValue('chkDatePP', QVariant(self.chkDatePP.isChecked()))
            settings.setValue('chkPayDate', QVariant(self.chkDatePay.isChecked()))
            settings.setValue('patientSurname', QVariant(self.filtersValue['patientSurname']))
            settings.setValue('patientName', QVariant(self.filtersValue['patientName']))
            settings.setValue('patientSecondName', QVariant(self.filtersValue['patientSecondName']))
            settings.setValue('patientBirthday', QVariant(self.filtersValue['patientBirthday']))
            settings.setValue('patientSex', QVariant(self.filtersValue['patientSex']))
            settings.endGroup()

    @pyqtSignature('')
    def on_actOpenDb_triggered(self):
        self.loadEisBase()
        self.setWindowTitleDB()
        if self.treeQuery.selectedItems():
            self.on_treeQuery_itemSelectionChanged()

    @pyqtSignature('')
    def on_actLogin_triggered(self):
        try:
            QtGui.qApp.openDatabase()
            return
        except database.CDatabaseException, e:
            QtGui.QMessageBox.critical(
                self,
                u'Ошибка открытия базы данных',
                unicode(e),
                QtGui.QMessageBox.Close)
        except Exception, e:
            QtGui.QMessageBox.critical(
                self,
                u'Ошибка открытия базы данных',
                u'Невозможно установить соедиенение с базой данных\n' + unicode(e),
                QtGui.QMessageBox.Close)
            self.logCurrentException()
        QtGui.qApp.closeDatabase()

    @pyqtSignature('')
    def on_actLogout_triggered(self):
        QtGui.qApp.closeDatabase()
        self.setUserName('')

    @pyqtSignature('')
    def on_actQuit_triggered(self):
        self.close()

    @pyqtSignature('')
    def on_actAutorization_triggered(self):
        self.autorization()

    @pyqtSignature('')
    def on_actAutoAutorization_triggered(self):
        self.autoAutorization()

    @QtCore.pyqtSignature('')
    def on_actConnection_triggered(self):
        dlg   = CConnectionDialog(self)
        preferences = QtGui.qApp.preferences
        dlg.setDriverName(preferences.dbDriverName)
        dlg.setServerName(preferences.dbServerName)
        dlg.setServerPort(preferences.dbServerPort)
        dlg.setDatabaseName(preferences.dbDatabaseName)
        dlg.setUserName(preferences.dbUserName)
        dlg.setPassword(preferences.dbPassword)
        if dlg.exec_():
            preferences.dbDriverName = dlg.driverName()
            preferences.dbServerName = dlg.serverName()
            preferences.dbServerPort = dlg.serverPort()
            preferences.dbDatabaseName = dlg.databaseName()
            preferences.dbUserName = dlg.userName()
            preferences.dbPassword = dlg.password()
            preferences.save()

    @pyqtSignature('')
    def on_actExportSettings_triggered(self):
        dlg = CExportSettingsDialog()
        preferences = QtGui.qApp.preferences
        settings = preferences.getSettings()
        settings.beginGroup('exportSettings')
        saveDir = settings.value('savePath', QtCore.QVariant('')).toString()
        settings.endGroup()
        settings.beginGroup('commonSettings')
        chkFilters = settings.value('saveFilters', QtCore.QVariant(False)).toBool()
        settings.endGroup()

        dlg.setSavePath(saveDir)
        if chkFilters:
            dlg.chkSaveFiltersSettings.setChecked(chkFilters)

        if dlg.exec_():
            settings.beginGroup('exportSettings')
            settings.setValue('savePath', QtCore.QVariant(dlg.getSavePath() ) )
            settings.endGroup()
            settings.beginGroup('commonSettings')
            settings.setValue('saveFilters', QtCore.QVariant(dlg.chkSaveFiltersSettings.isChecked()))
            settings.endGroup()




    @QtCore.pyqtSignature('')
    def on_actEISConnection_triggered(self):
        eisPreferences = QtGui.qApp.getEISPreferences()
        dlg = CEISConnectionDialog(self)
        dlg.setServerName(eisPreferences['server'])
        dlg.setServerPort(eisPreferences['port'])
        dlg.setDatabaseName(eisPreferences['database'])
        dlg.setUserName(eisPreferences['user'])
        dlg.setPassword(eisPreferences['password'])
        if dlg.exec_():
            eisPreferences['server']   = dlg.serverName()
            eisPreferences['port']     = dlg.serverPort()
            eisPreferences['database'] = dlg.databaseName()
            eisPreferences['user']     = dlg.userName()
            eisPreferences['password'] = dlg.password()
            QtGui.qApp.setEISPreferences(eisPreferences)

    @QtCore.pyqtSignature('')
    def on_actDecor_triggered(self):
        dlg = CDecorDialog(self)
        preferences = QtGui.qApp.preferences
        dlg.setStyle(preferences.decorStyle)
        dlg.setStandardPalette(preferences.decorStandardPalette)
        dlg.setMaximizeMainWindow(preferences.decorMaximizeMainWindow)
        dlg.setFullScreenMainWindow(preferences.decorFullScreenMainWindow)
        if dlg.exec_():
            preferences.decorStyle = dlg.style()
            preferences.decorStandardPalette = dlg.standardPalette()
            preferences.decorMaximizeMainWindow = dlg.maximizeMainWindow()
            preferences.decorFullScreenMainWindow = dlg.fullScreenMainWindow()
            preferences.save()
            QtGui.qApp.applyDecorPreferences()

    @pyqtSignature('')
    def on_actDbfView_triggered(self):
        preferences = QtGui.qApp.preferences
        settings = preferences.getSettings()
        settings.beginGroup('exportSettings')
        saveDir = settings.value('savePath', QtCore.QVariant('')).toString()
        settings.endGroup()
        fileName = forceString(QtGui.QFileDialog.getOpenFileName(
                self, u'Укажите DBF файл', saveDir, u'Файлы DBF (*.dbf)'))
        if fileName != '':
            gFileName = fileName
            CDbfViewDialog(self, fileName=fileName, encoding='cp866').exec_()

    @QtCore.pyqtSignature('')
    def on_actAboutQt_triggered(self):
        QtGui.QMessageBox.aboutQt(self, u'О Qt')

    @QtCore.pyqtSignature('')
    def on_actAbout_triggered(self):
        global lastChangedRev
        global lastChangedDate
        try:
            from buildInfo import lastChangedRev, lastChangedDate
        except:
            lastChangedRev  = 'unknown'
            lastChangedDate = 'unknown'

        QtGui.QMessageBox.about(
            self, u'О программе', about % ( title, lastChangedRev, lastChangedDate)
            )


    def makeDifferentQueries(self, modelName=None):
        db = QtGui.qApp.db
        if not modelName:
            modelName = self.tableView.model().whoIAm()
        if self.headerSortIndicator == Qt.DescendingOrder:
            orderSort = ' DESC'
        else:
            orderSort = ''
        where =  self.getCondition('bill')
        dictJoins = {'VMU_ACC_STATUS':['CASE_SERVICES_ACCOUNTS.ID_STATUS',
                                       'VMU_ACC_STATUS.ID_STATUS', ''],
                     'PAY_SUMMS':['PAY_SUMMS.ID_ACCOUNT',
                                  'CASE_SERVICES_ACCOUNTS.ID_ACCOUNT', 'LEFT OUTER'],
                     'PAY_SHEETS':['PAY_SUMMS.ID_PAY_CALC',
                                   'PAY_SHEETS.ID_PAY_CALC', 'LEFT OUTER'],
                     'SMO':['CASE_SERVICES_ACCOUNTS.ID_PAYER',
                            'SMO.ID_SMO', ''],
                     'REFUSES':['REFUSES.ID_ACCOUNT',
                                'CASE_SERVICES_ACCOUNTS.ID_ACCOUNT', 'LEFT OUTER']}

        stmtWithCond = db.getSelectStmtWithJoin(
                        ['CASE_SERVICES_ACCOUNTS.ID_ACCOUNT',
                         'CODE', 'SMO_SHORT_NAME',
                         'PAY_NUMBER',
                         'ACCOUNT_NUMBER', 'TARIFF_MONTH', 'ACCOUNT_DATE',
                         'CASE_SERVICES_ACCOUNTS.TOTAL_SUMM',
                         'STATUS_NAME', 'PAY_DATE', 'EXECUTE_DATE',
                         'REFUSES.TOTAL_SUMM AS REF_SUMM', 'ACT_NUMBER', 'ACT_DATE'],
                         'CASE_SERVICES_ACCOUNTS',
                         dictJoins) + '%s ORDER BY %s ' + orderSort

        where = self.getCondition('visits')
#        dictJoins = {
#                     'CASE_SERVICES':['CASE_SERVICES.ID_SERVICE',
#                                      'CASE_SERVICES_VISIT.ID_SERVICE', ''],
#                     'CASES':['CASE_SERVICES.ID_CASE',
#                              'CASES.ID_CASE', ''],
#                     'SMO':['CASES.ID_PAYER',
#                            'SMO.ID_SMO', '']
#                    }
#        stmtVisit = db.getSelectStmtWithJoin(
#                        ['CASE_SERVICES_VISIT.ID_VISIT',
#                         'SMO.CODE', 'SMO.SMO_SHORT_NAME',
#                         'CASE_SERVICES.SERVICE_COST',
#                         'CASE_SERVICES_VISIT.DATE_BEGIN',
#                         'CASE_SERVICES_VISIT.DATE_END',
#                         'CASE_SERVICES.VISIT_COUNT'],
#                         'CASE_SERVICES_VISIT',
#                         dictJoins) + '%s ORDER BY %s' + orderSort
#        stmtVisit = 'SELECT DISTINCT %s %s ID_VISIT, CODE, SMO_SHORT_NAME, SERVICE_COST, DATE_BEGIN, DATE_END, VISIT_COUNT FROM VIEW_VISITS %s ORDER BY %s '
        stmtVisit = 'SELECT DISTINCT %s %s CASE_SERVICES_VISIT.ID_VISIT, SMO.CODE, SMO.SMO_SHORT_NAME, CASE_SERVICES.SERVICE_COST, CASE_SERVICES_VISIT.DATE_BEGIN, CASE_SERVICES_VISIT.DATE_END, CASE_SERVICES.VISIT_COUNT FROM CASE_SERVICES_VISIT JOIN CASE_SERVICES ON CASE_SERVICES.ID_SERVICE=CASE_SERVICES_VISIT.ID_SERVICE JOIN CASES ON CASE_SERVICES.ID_CASE=CASES.ID_CASE JOIN SMO ON CASES.ID_PAYER=SMO.ID_SMO %s ORDER BY %s '
        where =  self.getCondition('profile')
        stmtProfile = '''
            SELECT
                VMU_PROFILE.ID_PROFILE
            FROM
                VMU_PROFILE, CASE_SERVICES
            %s
            AND
                VMU_PROFILE.ID_PROFILE = CASE_SERVICES.ID_PROFILE
        ''' % where
        if modelName == 'bill':
            self.stmt = stmtWithCond
        elif modelName == 'visits':
            self.stmt = stmtVisit
        elif modelName == 'profiles':
            self.stmt = stmtProfile

    def execQueryForMe(self, stmt):
        db = QtGui.qApp.db
        query = db.query(stmt)
        go = query.first()
        while go:
            result = ''
            for i in range(query.record().count()):
                result += (query.value(i).toString()+', ')
            print result
            go = query.next()


class CPatientSelectorDialog(CDialogBase, Ui_PatientsListDialog):
    def __init__(self, parent, cond):
        CDialogBase.__init__(self, parent)
        self.setupUi(self)
        self.addModels('Patients', CPatientsModel(self))
        self.setModels(self.tblPatients, self.modelPatients, self.selectionModelPatients)
        self.cond = cond
        self._patientId = None
        self.loadPatients(cond)
        self.setWindowTitleEx(u'Список найденных пациентов')

    def patientId(self):
        return forceString(self._patientId)

    def exec_(self):
        if self.modelPatients.rowCount() == 1:
            self._patientId = self.modelPatients.idList()[0]
            return True
        result = CDialogBase.exec_(self)
        if not bool(self._patientId):
            self._patientId = self.tblPatients.currentItemId()
        if not bool(self._patientId) and bool(self.modelPatients.idList()):
            res = QtGui.QMessageBox.warning(self, u'Внимание!',
                                            u'Вы не выбрали пациента!',
                                            QMessageBox.Ok|QMessageBox.Ignore,
                                            QMessageBox.Ok)
            if res == QMessageBox.Ok:
                self.exec_()
        return result

    def loadPatients(self, cond):
        db = QtGui.qApp.db
        where = db.joinAnd(cond)
        stmt = 'SELECT DISTINCT ID_PATIENT FROM PATIENT_DATA WHERE %s' % where
        query = db.query(stmt)
        idList = []
        while query.next():
            idList.append(query.value(0))
        self.modelPatients.setIdList(idList)


    @QtCore.pyqtSignature('QModelIndex')
    def on_tblPatients_doubleClicked(self, index):
        if index.isValid():
            self._patientId = self.tblPatients.currentItemId()
            self.accept()

def parseGCDebug(val):
    result = 0
    for char in val.upper():
        if char == 'S':
            result |= gc.DEBUG_STATS
        elif char == 'C':
            result |= gc.DEBUG_COLLECTABLE
        elif char == 'U':
            result |= gc.DEBUG_UNCOLLECTABLE
        elif char == 'I':
            result |= gc.DEBUG_INSTANCES
        elif char == 'O':
            result |= gc.DEBUG_OBJECTS
    return result

def parseGCThreshold(val):
    result = []
    if val:
        for s in val.split(','):
            try:
                v = int(s)
            except:
                v = 0
            result.append(v)
        l = len(result)
        if l > 3:
            result = result[0:3]
    return result


if __name__ == '__main__':

##    gc.set_threshold(*[10*k for k in gc.get_threshold()])
    parser = OptionParser(usage = "usage: %prog [options]")
    parser.add_option('-d', '--demo',
                      dest='demo',
                      help='Login as demo',
                      action='store_true',
                      default=False)
    parser.add_option('--gc-debug',
                      dest='gcDebug',
                      help='gc debug flags, chars S(stat), C(collectable), U(uncollectable), I(instances), O(objects), for details see gc.set_debug',
                      metavar='[S][C][U][I][O]',
                      default=''
                      )
    parser.add_option('--gc-threshold',
                      dest='gcThreshold',
                      help='gc thresholds list, see gc.set_threshold ',
                      metavar='threshold0[,threshold1[,threshold2]]',
                      default=''
                     )
    parser.add_option('--version',
                      dest='version',
                      help='print version and exit',
                      action='store_true',
                      default=False)
#    parser.add_option("-q", "--quiet",   action="store_false", dest="verbose", default=True,  help="don't print status messages to stdout")
    (options, args) = parser.parse_args()

    if options.version:
        print '%s v.0.1' % (sys.argv[0] if sys.argv else '')
    else:
        gcDebug = parseGCDebug(options.gcDebug)
        gcThreshold = parseGCThreshold(options.gcThreshold)
        if gcDebug:
            gc.set_debug(gcDebug)
            print 'debug set to ', gcDebug
        if gcThreshold:
            gc.set_threshold(*gcThreshold)
            print 'threshold set to ', gcThreshold

        app = CMyApp(sys.argv, options.demo)
        stdTtranslator = QtCore.QTranslator()
        stdTtranslator.load('i18n/std_ru.qm')
        app.installTranslator(stdTtranslator)

        QtGui.qApp = app
#        app.organizationInfo()
        app.applyDecorPreferences() # надеюсь, что это поможет немного сэкономить при создании гл.окна
        mainWindow = CMainWindow()
        app.mainWindow = mainWindow
        app.applyDecorPreferences() # применение максимизации/полноэкранного режима к главному окну

#        if app.preferences.dbAutoLogin:
#            mainWindow.actLogin.activate(QtGui.QAction.Trigger)
        mainWindow.show()
        mainWindow.startAutorization()
        mainWindow.loadEisBase()
        mainWindow.setWindowTitleDB()
        res = app.exec_()
        mainWindow.savePreferences()
        app.preferences.save()
        app.endWorkWithEISOMS(mainWindow.viewsDict)
        app.closeDatabase()
        app.doneTrace()
        QtGui.qApp = None

#    sys.exit(res)
