#!/usr/bin/env python
# -*- coding: utf-8 -*-

#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui, QtCore, QtSql

from library.Utils         import *

from library.DialogBase import CConstructHelperMixin

from Ui_profileDistinct import Ui_Form

from TableModel                import *
from accountListRegisterWidget import CTabAccountListRegister



class CTabProfileDistinct(QtGui.QWidget, CConstructHelperMixin, Ui_Form):
    def __init__(self, generalTab):
        QtGui.QWidget.__init__(self)
        self.setupUi(self)
        self.profileId = None
        self.tabALR    = None
        self.execDone  = None
        self.dFrom     = None
        self.dTo       = None
        self.tabsList  = []
        self.contextIsSet = False
        self.tabView = generalTab
        self.addModels('Accounts', CCaseServicesAccountsTableModel(self))
        self.setModels(self.tblProfileDistinct, self.modelAccounts, self.selectionModelAccounts)
        self.where = ''
        self.createPopupActions()
        QtCore.QObject.connect(
            self.selectionModelAccounts, 
            QtCore.SIGNAL('currentRowChanged(QModelIndex, QModelIndex)'), 
            self.modelRowChanged)
            
    def createPopupActions(self):
        self.openLRAction  = QtGui.QAction(u'Открыть \'Реестр\'', self)
        self.connect(self.openLRAction, 
                     QtCore.SIGNAL('triggered()'), 
                     self.prepareMakeTab)
        self.tblProfileDistinct.createPopupMenu([self.openLRAction])
        
    def prepareMakeTab(self):
        model = self.tblProfileDistinct.model()
        if isinstance(model, CCaseServicesAccountsTableModel):
            self.makeALRTab()
            index = self.tblProfileDistinct.currentIndex()
            self.modelRowChanged(index)
            self.tabView.setCurrentIndex(1)
            
    def modelRowChanged(self, current, previous=None):
        self.setCursor(QtCore.Qt.WaitCursor)
        model = self.tblProfileDistinct.model()
        id = self.currentId(current)
        if isinstance(model, CCaseServicesAccountsTableModel):
            if self.tabALR:
                self.tabALR.setAccountId(id)
                self.tabALR.setALRContext()
                self.tabALR.setIsContext(False)
            else:
                self.makeALRTab()
                self.modelRowChanged(current)
        self.setCursor(QtCore.Qt.ArrowCursor)
                
    def makeALRTab(self):
        if self.tabALR:
            self.removeTabALR()
        if u'Реестр счета' not in self.tabsList:
            self.tabALR = CTabAccountListRegister(self.where)
            self.setActNumber()
            self.tabView.addTab(self.tabALR, u'Реестр счета')
            self.tabsList.append(u'Реестр счета')
            
    def setActNumber(self):
        index = self.tblProfileDistinct.currentIndex()
        if index:
            row = index.row()
            model = self.tblProfileDistinct.model()
            actNumber = forceString(model.data(model.index(row, 11)))
            self.tabALR.setActNumber(actNumber)
        
        
    def removeTabALR(self):
        self.tabView.removeTab(self.tabView.indexOf(self.tabALR))
        if u'Реестр счета' in self.tabsList:
            self.tabsList.remove(u'Реестр счета')
        self.tabALR = None
        
    def currentId(self, index):
        return self.tblProfileDistinct.itemId(index)
        
    def setWhere(self, where):
        self.where = where
        
    def setCondDates(self, dFrom, dTo):
        self.dFrom = dFrom
        self.dTo   = dTo
        
    def setProfileDistinctContext(self):
        id = self.getId()
        listId = self.getListId()
        self.modelAccounts.setIdList(listId)
        self.contextIsSet = True
        self.setFourEdits(listId)
        
    def setFourEdits(self, listId):
        lines = len(listId)
        self.edtEntriesAmount.setText(forceString(lines))
        model = self.modelAccounts
        ts = 0
        rs = 0
        for i in range(lines):
            ts += model.data(model.index(i, 6)).toDouble()[0]
            rs += model.data(model.index(i, 10)).toDouble()[0]
        self.edtCommonTotalSumm.setText('%.2f'%ts)
        self.edtCommonPaySumm.setText('%.2f'%(ts-rs))
        self.edtRefuse.setText('%.2f'%rs)
    def getListId(self):
        val = self.getId()
        if isinstance(val, int):
            return self.getIdListFromKsg(val)
        elif isinstance(val, (QtCore.QString, str, unicode)):
            return self.getIdListFromProfile(val)
        return []
        
        
    def getIdListFromProfile(self, ksgGroup):
        ksgGroups = QtGui.qApp.ksgGroups
        result = []
        for id in ksgGroups[ksgGroup]:
            tmp = self.getIdListFromKsg(id)
            result += tmp
        return result
        
        
    def getIdListFromKsg(self, id):
        db = QtGui.qApp.db
        if self.where != '':
            where =  ' AND '+ self.where.split('WHERE')[1]
        else:
            where = ''
        stmt = '''
            SELECT DISTINCT
                ID_ACCOUNT
            FROM
                CASE_SERVICES
            WHERE
                ID_PROFILE = %d %s
        ''' % (id, where)
        result = []
        query = db.query(stmt)
#        print datetime.datetime.time(datetime.datetime.now())
        while query.next():
            result.append(query.value(0))
#        print datetime.datetime.time(datetime.datetime.now())
        return result 
    
    def setProfileId(self, id):
        if type(id) == QtCore.QVariant:
            if id.type() == QtCore.QVariant.String:
                self.profileId = id.toString()
            if id.type() == QtCore.QVariant.Int:
                self.profileId = id.toInt()[0]
            return
        self.profileId = id
                
    def setIsContext(self, bool):
        self.contextIsSet = False
        
    def getId(self):
        return self.profileId
        