#!/usr/bin/env python
# -*- coding: utf-8 -*-

#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
import os

from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import *

from library.dbfpy.dbf import Dbf
from library.Utils     import *
from TableModel import *

from library.DbfViewDialog import CDbfViewDialog

from library.ProgressBar import CProgressBar

class CResultProgressDialog(QtGui.QDialog):
    def __init__(self, tableView, saveDir, stmt):
        QtGui.QDialog.__init__(self)
        self.model        = tableView.model()
        self.tableView    = tableView
        self.stmt         = stmt if stmt else None
        self.first        = 0
        self.linesNumber  = 0
        self.dbfFile      = ''
        self.aborted      = False
        self.dbfFilePath  = saveDir
        self.dbfFileName  = self.dbfFilePath + '/myBase.dbf'

        self.btnClose     = QtGui.QPushButton(u'Закрыть')

        self.btnSaveAsDbf = QtGui.QPushButton(u'Начать')

        self.btnViewDbf   = QtGui.QPushButton(u'Просмотреть результат')
        self.btnViewDbf.setEnabled(False)

        self.progressBar = CProgressBar()
        self.progressBar.setMinimum(0)
        self.progressBar.setMaximum(1)
        self.progressBar.setText('')

        self.btnOpen = QtGui.QPushButton(u'...')
        self.edtFilePath = QtGui.QLineEdit(self.dbfFileName)
        self.lblFilePath = QtGui.QLabel(u'Полный путь сохраняемого файла')

        self.hSpacer = QtGui.QSpacerItem(268, 21, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)

        self.myHLayout = QtGui.QHBoxLayout()
        self.myHLayout.addItem(self.hSpacer)
        self.myHLayout.addWidget(self.btnViewDbf)
        self.myHLayout.addWidget(self.btnSaveAsDbf)
        self.myHLayout.addWidget(self.btnClose)

        self.savePathLayout = QtGui.QHBoxLayout()
        self.savePathLayout.addWidget(self.lblFilePath)
        self.savePathLayout.addWidget(self.edtFilePath)
        self.savePathLayout.addWidget(self.btnOpen)
        self.myVLayout = QtGui.QVBoxLayout()

        self.myVLayout.addWidget(self.progressBar)
        self.myVLayout.addLayout(self.savePathLayout)
        self.myVLayout.addLayout(self.myHLayout)

        self.setLayout(self.myVLayout)

        self.setWindowTitle(u'Экспортирование данных')

        self.connect(self.btnClose,
                     QtCore.SIGNAL('clicked()'),
                     self.btnCloseClicked)

        self.connect(self.btnSaveAsDbf,
                     QtCore.SIGNAL('clicked()'),
                     self.export)

        self.connect(self.btnOpen,
                     QtCore.SIGNAL('clicked()'),
                     self.selectDir)

        self.connect(self.btnViewDbf,
                     QtCore.SIGNAL('clicked()'),
                     self.viewDbf)

    def viewDbf(self):
        CDbfViewDialog(self, fileName=unicode(self.dbfFile), encoding='cp866').exec_()

    def btnCloseClicked(self):
        if not self.aborted and self.btnClose.text() == u'Прервать':
            self.aborted = True
        else:
            self.accept()

    def selectDir(self):
        filter = QString('')
        file = QtGui.QFileDialog.getExistingDirectory(self,
                                           u'Выбирете путь для сохранения',
                                           self.dbfFilePath)
        if file:
            self.edtFilePath.setText(file)
            self.dbfFilePath = file

    def export(self):
        self.dbfFile = self.edtFilePath.text()
        self.aborted = False
        self.edtFilePath.setEnabled(False)
        self.btnOpen.setEnabled(False)
        QtGui.qApp.call(self, self.exportInt)
        self.edtFilePath.setEnabled(True)
        self.btnOpen.setEnabled(True)
        self.emit(QtCore.SIGNAL('completeChanged()'))

    def prepareToExport(self):
        self.emit(QtCore.SIGNAL('completeChanged()'))
        dbf = self.createDbf()
        query = self.createQuery()
        if isinstance(self.model, CRefusesTableModel):
            self.progressBar.setMaximum(len(self.model.idList()))
        else:
            self.progressBar.setMaximum(self.linesNumber)
        self.progressBar.reset()
        self.progressBar.setValue(0)
        return dbf, query

    def createDbf(self):
        dbf = Dbf(unicode(self.edtFilePath.text()), new=True, encoding='cp866')
        if isinstance(self.model, CCaseServicesAccountsTableModel):
            dbf.addField(
#                ('ID_ACCOUNT',     'C', 20),
                ('ACC_NUMBER',     'C', 12),
                ('ACC_DATE',       'D'),
                ('CODE',           'C', 7),
                ('SHORT_NAME',     'C', 12),
                ('STAT_NAME',      'C', 200),
                ('TAR_MONTH',      'D'),
                ('TOTAL_SUM',      'N', 15, 2),
                ('PAY_NUMBER',     'C', 16),
                ('PAY_DATE',       'D'),
                ('EXEC_DATE',      'D'),
                ('REFUSE_SUM',     'N', 15, 2),
                ('REFUSE_ACT',     'C', 22),
                ('ACT_DATE',       'D')
                )
        elif isinstance(self.model, CVisitsTableModel):
            dbf.addField(
                ('ID_VISIT',   'C', 20),
                ('CODE',       'C', 5),
                ('SHORT_NAME', 'C', 10),
                ('SERV_COST',  'N', 15, 2),
                ('DATE_BEGIN', 'D'),
                ('DATE_END',   'D'),
                ('VIS_COUNT',  'N', 15, 2)
                )
        elif isinstance(self.model, CProfileTableModel):
            dbf.addField(
#                ('ID_PROFILE',  'N', 15),
                ('PROFILE',     'C', 200),
                ('SERV_COUNT',  'N', 15),
                ('TOTAL_SUMM',  'N', 15, 2),
                ('REFUSE',      'N', 15, 2),
                ('PAY_SUMM',    'N', 15, 2)
                         )
        elif isinstance(self.model, CKSGGroupsTableModel):
            dbf.addField(
                ('KSG_CODE',    'C', 15),
                ('PROFILE',     'C', 200),
                ('SERV_COUNT',  'N', 15),
                ('DAYS_COUNT',  'N', 15),
                ('TOTAL_SUMM',  'N', 15, 2),
                ('REFUSE',      'N', 15, 2),
                ('PAY_SUMM',    'N', 15, 2)
                         )
        elif isinstance(self.model, CRefusesTableModel):
            dbf.addField(
                ('REF_NAME',  'C', 70),
                ('REF_COUNT', 'N', 7),
                ('REF_SUMM',  'N', 15, 2)
                )
        return dbf

    def createQuery(self):
        db = QtGui.qApp.db
        if isinstance(self.model, (CCaseServicesAccountsTableModel, CVisitsTableModel)):
            query = db.query(self.stmt)
            return query
        if isinstance(self.model, (CProfileTableModel, CKSGGroupsTableModel, CRefusesTableModel)):
            return None
        return self.model.query()

    def exportInt(self):
        dbf, query = self.prepareToExport()
        if query:
            go = query.first()
        else:
            go = query
#        go = query.numRowsAffected()
        self.btnClose.setText(u'Прервать')
        if isinstance(self.model, CCaseServicesAccountsTableModel):
            process = self.processAccounts
        elif isinstance(self.model, CVisitsTableModel):
            process = self.processVisits
        elif isinstance(self.model, CProfileTableModel):
            process = self.processProfiles
        elif isinstance(self.model, CKSGGroupsTableModel):
            process = self.processKSGGroups
        elif isinstance(self.model, CRefusesTableModel):
            process = self.processRefuses
        if isinstance(self.model, (CProfileTableModel, CKSGGroupsTableModel, CRefusesTableModel)):
            for i in range(len(self.model.idList())):
                QtGui.qApp.processEvents()
                if self.aborted:
                    break
                self.progressBar.step()
                dbfRecord = process(dbf, i)
        while go:
            QtGui.qApp.processEvents()
            if self.aborted:
                break
            self.progressBar.step()
            dbfRecord = process(dbf, query.record())
            go = query.next()
        self.btnClose.setText(u'Закрыть')
        self.btnViewDbf.setEnabled(True)
        self.aborted =True
        dbf.close()

    def processRefuses(self, dbf, row):
        refName = forceString(self.model.data(self.model.index(row, 0)))
        refCount  = forceInt(self.model.data(self.model.index(row, 1)))
        refSumm   = forceDouble(self.model.data(self.model.index(row, 2)))

        dbfRecord = dbf.newRecord()

        dbfRecord['REF_NAME']  = refName
        dbfRecord['REF_COUNT'] = refCount
        dbfRecord['REF_SUMM']  = refSumm

        dbfRecord.store()
        return dbfRecord

    def processProfiles(self, dbf, row):
#        idProfile     = forceInt(self.model.data(self.model.index(row, 0)))
        profile       = forceString(self.model.data(self.model.index(row, 0)))
        servicesCount = forceInt(self.model.data(self.model.index(row, 1)))
        totalSum      = forceDouble(self.model.data(self.model.index(row, 2)))
        refuse        = forceDouble(self.model.data(self.model.index(row, 3)))
        paySum        = forceDouble(self.model.data(self.model.index(row, 4)))

        dbfRecord = dbf.newRecord()

#        dbfRecord['ID_PROFILE'] = idProfile
        dbfRecord['PROFILE']    = profile
        dbfRecord['SERV_COUNT'] = servicesCount
        dbfRecord['TOTAL_SUMM'] = totalSum
        dbfRecord['REFUSE']     = refuse
        dbfRecord['PAY_SUMM']   = paySum

        dbfRecord.store()
        return dbfRecord

    def processKSGGroups(self, dbf, row):
        ksgCode       = forceString(self.model.data(self.model.index(row, 0)))
        profile       = forceString(self.model.data(self.model.index(row, 1)))
        servicesCount = forceInt(self.model.data(self.model.index(row, 2)))
        daysCount     = forceInt(self.model.data(self.model.index(row, 3)))
        totalSum      = forceDouble(self.model.data(self.model.index(row, 4)))
        refuse        = forceDouble(self.model.data(self.model.index(row, 5)))
        paySum        = forceDouble(self.model.data(self.model.index(row, 6)))

        dbfRecord = dbf.newRecord()

        dbfRecord['KSG_CODE']   = ksgCode
        dbfRecord['PROFILE']    = profile
        dbfRecord['SERV_COUNT'] = servicesCount
        dbfRecord['DAYS_COUNT'] = daysCount
        dbfRecord['TOTAL_SUMM'] = totalSum
        dbfRecord['REFUSE']     = refuse
        dbfRecord['PAY_SUMM']   = paySum

        dbfRecord.store()
        return dbfRecord
    def processVisits(self, dbf, record, row=None):
        idVisit       = forceString(record.value('ID_VISIT'))
        code          = forceString(record.value('CODE'))
        smoShortName  = forceString(record.value('SMO_SHORT_NAME'))
        serviceCost   = forceDouble(record.value('SERVICE_COST'))
        dateBegin     = self.checkDate(record.value('DATE_BEGIN'))
        dateEnd       = self.checkDate(record.value('DATE_END'))
        visitCount    = forceInt(record.value('VISIT_COUNT'))

        dbfRecord = dbf.newRecord()

        dbfRecord['ID_VISIT']   = idVisit
        dbfRecord['CODE']       = code
        dbfRecord['SHORT_NAME'] = smoShortName
        dbfRecord['SERV_COST']  = serviceCost
        dbfRecord['DATE_BEGIN'] = dateBegin
        dbfRecord['DATE_END']   = dateEnd
        dbfRecord['VIS_COUNT']  = visitCount

        dbfRecord.store()
        return dbfRecord

    def checkDate(self, value):
        if value.toString() == '':
            return None
        else:
            return forceDate(value).toPyDate()

    def processAccounts(self, dbf, record, row=None):
#        idAccount     = forceString(record.value('ID_ACCOUNT'))
        code          = forceString(record.value('CODE'))
        smoShortName  = forceString(record.value('SMO_SHORT_NAME'))
        accountNumber = forceString(record.value('ACCOUNT_NUMBER'))
        tariffMonth   = self.checkDate(record.value('TARIFF_MONTH'))
        accountDate   = self.checkDate(record.value('ACCOUNT_DATE'))
        totalSumm     = forceDouble(record.value('TOTAL_SUMM'))
        payDate       = self.checkDate(record.value('PAY_DATE'))
        executeDate   = self.checkDate(record.value('EXECUTE_DATE'))
        refuseSumm    = forceDouble(record.value('REF_SUMM'))
        payNumber     = forceString(record.value('PAY_NUMBER'))
        if executeDate:
            statusName = u'Оплачено'
        else:
            statusName    = forceString(record.value('STATUS_NAME'))
        if refuseSumm == 0:
            refuseAct = ''
            actDate   = ''
        else:
            refuseAct     = forceString(record.value('ACT_NUMBER'))
            actDate       = self.checkDate(record.value('ACT_DATE'))

        dbfRecord = dbf.newRecord()

#        dbfRecord['ID_ACCOUNT'] = idAccount
        dbfRecord['ACC_NUMBER'] = accountNumber
        dbfRecord['ACC_DATE']   = accountDate
        dbfRecord['CODE']       = code
        dbfRecord['SHORT_NAME'] = smoShortName
        dbfRecord['STAT_NAME']  = statusName
        dbfRecord['TAR_MONTH']  = tariffMonth
        dbfRecord['PAY_NUMBER'] = payNumber
        dbfRecord['PAY_DATE']   = payDate
        dbfRecord['EXEC_DATE']  = executeDate
        dbfRecord['TOTAL_SUM']  = totalSumm
        dbfRecord['REFUSE_SUM'] = refuseSumm
        dbfRecord['REFUSE_ACT'] = refuseAct
        dbfRecord['ACT_DATE']   = actDate

        dbfRecord.store()
        return dbfRecord
