#!/usr/bin/env python
# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtCore

from library.Utils import *
from library.database import *

def export(db, log, exportPath, orgId, ffomsCode, eisLpuId):
    v1File = QtCore.QFile(exportPath+u'/в'+ffomsCode+'_1.csv')
    if not v1File.open(QtCore.QFile.WriteOnly | QtCore.QFile.Text):
        log.append(u'Не могу открыть файл для записи %s:\n%s.'%(fileName, v1File.errorString()))
        return
    
    v2File = QtCore.QFile(exportPath+u'/в'+ffomsCode+'_2.csv')
    if not v2File.open(QtCore.QFile.WriteOnly | QtCore.QFile.Text):
        log.append(u'Не могу открыть файл для записи %s:\n%s.'%(fileName, v2File.errorString()))
        return
    
    d1File = QtCore.QFile(exportPath+u'/д'+ffomsCode+'_1.csv')
    if not d1File.open(QtCore.QFile.WriteOnly | QtCore.QFile.Text):
        log.append(u'Не могу открыть файл для записи %s:\n%s.'%(fileName, d1File.errorString()))
        return
    
    d2File = QtCore.QFile(exportPath+u'/д'+ffomsCode+'_2.csv')
    if not d2File.open(QtCore.QFile.WriteOnly | QtCore.QFile.Text):
        log.append(u'Не могу открыть файл для записи %s:\n%s.'%(fileName, d2File.errorString()))
        return
    
    log.append(u'Файлы созданы в директории %s'%exportPath)
    
    tPerson = db.table('Person')
    tOS = db.table('OrgStructure')
    tOSAddr = db.table('OrgStructure_Address')
    tNet = db.table('rbNet')
    tAddr = db.table('AddressHouse')
    tStreet = db.table('kladr.STREET')
    
    table = tPerson.leftJoin(tOS, tOS['id'].eq(tPerson['orgStructure_id']))
    table = table.leftJoin(tNet, tNet['id'].eq(tOS['net_id']))
    
    cond = [tOS['areaType'].eq(1), 
        tPerson['retireDate'].isNull(), 
        tPerson['deleted'].eq(0)
        ]
    
    cols = [
        tPerson['id'], 
        tPerson['regionalCode'], 
        tPerson['federalCode'], 
        tPerson['firstName'], 
        tPerson['lastName'], 
        tPerson['patrName'], 
        tPerson['SNILS'], 
        tPerson['sex'], 
        tOS['name'], 
        tNet['code']]
    
    dCount = 0
    vCount = 0
    recordList = db.getRecordList(table, cols, cond)
    for record in recordList:
        line = []
        line.append(forceString(record.value('federalCode')))
        line.append(forceString(record.value('lastName')))
        line.append(forceString(record.value('firstName')))
        line.append(forceString(record.value('patrName')))
        line.append(forceString(record.value('SNILS')))
        sex = ''
        if forceInt(record.value('sex')) == 1:
            sex = u'м'
        elif forceInt(record.value('sex')) == 2:
            sex = u'ж'
        line.append(sex)
        line.append(forceString(record.value('name')).replace(u'участок', '').strip())
        code = forceString(record.value('code'))
        
        personId = forceInt(record.value('id'))
        line.append(getPersonIdentifier(personId, eisLpuId))
        
        line = ';'.join(line)
        
        if code == '1':
            v1File.write((line+'\n').encode('cp1251'))
            #v1File.write((line+'\n').encode('utf8'))
            vCount += 1
        elif code == '2':
            d1File.write((line+'\n').encode('cp1251'))
            #d1File.write((line+'\n').encode('utf8'))
            dCount += 1
    
    log.append(u'Выгружено врачей взрослой сети: %i, детской сети: %i'%(vCount, dCount))
    
    table = tOS.leftJoin(tOSAddr, tOSAddr['master_id'].eq(tOS['id']))
    table = table.leftJoin(tAddr, tAddr['id'].eq(tOSAddr['house_id']))
    table = table.leftJoin(tStreet, tStreet['CODE'].eq(tAddr['KLADRStreetCode']))
    table = table.leftJoin(tNet, tNet['id'].eq(tOS['net_id']))
    
    cols = [
        tOS['name'].alias('orgName'), 
        tStreet['CODE'], 
        tStreet['SOCR'], 
        tStreet['NAME'].alias('streetName'), 
        tAddr['number'], 
        tAddr['corpus'], 
        tOSAddr['firstFlat'], 
        tOSAddr['lastFlat'], 
        tNet['code'].alias('netCode'), 
        tAddr['KLADRStreetCode'], 
        'kladr.getEisHouseId(%s, %s, %s, %s) as idHouse'%(tAddr['KLADRCode'], tAddr['KLADRStreetCode'], tAddr['number'], tAddr['corpus'])]
    
    cond = [
        tOS['areaType'].eq(1), 
        tOS['deleted'].eq(0)]
    
    dCount = 0
    vCount = 0
    recordList = db.getRecordList(table, cols, cond)
    for record in recordList:
        line = []
        line.append(forceString(record.value('idHouse')))
        address = u'%s %s, д%s'%(forceString(record.value('SOCR')), 
            forceString(record.value('streetName')), 
            forceString(record.value('number')))
        corpus = forceString(record.value('corpus'))
        if corpus:
            address = u'%s к%s'%(address, corpus)
        line.append(address)
        firstFlat = forceInt(record.value('firstFlat'))
        lastFlat = forceInt(record.value('lastFlat'))
        line.append(unicode(firstFlat) if firstFlat else '')
        line.append(unicode(lastFlat) if lastFlat else '')
        line.append(forceString(record.value('orgName')).replace(u'участок', '').strip())
        line.append(forceString(record.value('KLADRStreetCode')))
        
        code = forceString(record.value('netCode'))
        line = ';'.join(line)
        if code == '1':
            v2File.write((line+'\n').encode('cp1251'))
            #v2File.write((line+'\n').encode('utf8'))
            vCount += 1
        elif code == '2':
            d2File.write((line+'\n').encode('cp1251'))
            #d2File.write((line+'\n').encode('utf8'))
            dCount += 1
    
    log.append(u'Выгружено адресов взрослой сети: %i, детской сети: %i'%(vCount, dCount))
    
    v1File.close()
    v2File.close()
    d1File.close()
    d2File.close()
    

mapPersonIdToEisPersonId = {}
mapOrgIdToEis = {}
def getPersonIdentifier(personId, eisLpuId):
    result = mapPersonIdToEisPersonId.get(personId, None)
    if result is None:
        if personId is None:
            result = '0'
        else:
            db = QtGui.qApp.db
            result = forceString(db.translate('Person', 'id', personId, 'regionalCode'))
            if not result:
                orgId = forceInt(db.translate('Person', 'id', personId, 'org_id'))
                result = '%s.%s' % (eisLpuId, personId)
            else:
                if result.find('.') < 0:
                    result = str(eisLpuId) + '.' + result
        mapPersonIdToEisPersonId[personId] = result
    return result
