#!/usr/bin/env python
# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui, QtCore

import sys

from s11main import CS11mainApp
from library.Utils import *
from library.database import CDatabaseException

from export import export

from Ui_page1 import Ui_ExportPage1
from Ui_page2 import Ui_ExportPage2
from Ui_page3 import Ui_ExportPage3

class CExport(QtGui.QWizard):
    def __init__(self, parent = None):
        QtGui.QWizard.__init__(self, parent)
        self.setWizardStyle(QtGui.QWizard.ModernStyle)
        self.addPage(CExport_page1(self))
        self.addPage(CExport_page2(self))
        self.addPage(CExport_page3(self))

class CExport_page1(QtGui.QWizardPage, Ui_ExportPage1):
    def __init__(self, parent = None):
        QtGui.QWizardPage.__init__(self, parent)
        self.setupUi(self)
        self.setSubTitle(u'для выполнения шага заполните поля и нажмите кнопку "Подключиться"')
        prefs = QtGui.qApp.preferences
        self.edtAddress.setText(prefs.dbServerName)
        self.edtPort.setValue(prefs.dbServerPort)
        self.edtName.setText(prefs.dbDatabaseName)
        self.edtUser.setText(prefs.dbUserName)
        self.edtPassword.setText(prefs.dbPassword)
        self.readyNext = False

    def isComplete(self):
        return self.readyNext

    @QtCore.pyqtSignature('')
    def on_btnConnect_clicked(self):
        prefs = QtGui.qApp.preferences
        prefs.dbServerName = forceString(self.edtAddress.text())
        prefs.dbServerPort = forceInt(self.edtPort.value())
        prefs.dbDatabaseName = forceString(self.edtName.text())
        prefs.dbUserName = forceString(self.edtUser.text())
        prefs.dbPassword = forceString(self.edtPassword.text())
        prefs.save()

        if QtGui.qApp.db:
            QtGui.qApp.closeDatabase()
        try:
            QtGui.qApp.openDatabase()
        except CDatabaseException as error:
            QtGui.QMessageBox.critical(self, u'Ошибка!', u'%s'%exceptionToUnicode(error),
                QtGui.QMessageBox.Ok, QtGui.QMessageBox.Ok)
        except:
            QtGui.QMessageBox.critical(self, u'Ошибка!', u'Отсутствует подключение к базе',
                QtGui.QMessageBox.Ok, QtGui.QMessageBox.Ok)

        if QtGui.qApp.db:
            self.btnConnect.setText(u'Подключено')
            self.btnConnect.setEnabled(False)
            self.readyNext = True
            self.emit(QtCore.SIGNAL('completeChanged()'))

class CExport_page2(QtGui.QWizardPage, Ui_ExportPage2):
    def __init__(self, parent = None):
        QtGui.QWizardPage.__init__(self, parent)
        self.setupUi(self)
        self.setSubTitle(u'для выполнения шага заполните поля')
        self.ffomsCode = None
        self.readyNext = False

    def initializePage(self):
        props = QtGui.qApp.preferences.appPrefs
        self.cmbOrganisation.setValue(forceRef(props.get('orgId', QVariant())))
        self.edtExportPath.setText(forceString(props.get('exportPath', QVariant())))
        self.edtEISOMSLpuId.setText(forceString(props.get('EISLpuId', QVariant())))
        self.edtSmoCode.setText(forceString(props.get('ffomsCode', QVariant())))
        self.checkValues()

    def isComplete(self):
        return self.readyNext

    def checkValues(self):
        self.readyNext = True if self.edtEISOMSLpuId.text()\
            and self.edtExportPath.text() \
            and self.edtSmoCode.text() \
            and self.cmbOrganisation.value() \
                else False
        self.emit(QtCore.SIGNAL('completeChanged()'))

    def validatePage(self):
        QtGui.qApp.preferences.appPrefs['exportPath'] = self.edtExportPath.text()
        QtGui.qApp.preferences.appPrefs['EISLpuId'] = self.edtEISOMSLpuId.text()
        QtGui.qApp.preferences.appPrefs['ffomsCode'] = self.edtSmoCode.text()
        QtGui.qApp.preferences.appPrefs['orgId'] = toVariant(self.cmbOrganisation.value())
        return True

    @QtCore.pyqtSignature('')
    def on_btnSelectDirectory_clicked(self):
        path = QtGui.QFileDialog.getExistingDirectory(
            self, u'Выберите директорию', self.edtExportPath.text())
        self.edtExportPath.setText(path)
        self.checkValues()

    @QtCore.pyqtSignature('QString')
    def on_edtEISOMSLpuId_textEdited(self, text):
        self.checkValues()

    @QtCore.pyqtSignature('QString')
    def on_edtSmoCode_textEdited(self, text):
        self.checkValues()

    @QtCore.pyqtSignature('int')
    def on_cmbOrganisation_currentIndexChanged(self, index):
        orgId = self.cmbOrganisation.value()
        db = QtGui.qApp.db
        table = db.table('Organisation')
        ffomsCode = forceString(db.translate(table, 'id', orgId, 'smoCode'))
        if ffomsCode:
            self.edtSmoCode.setText(ffomsCode)
        self.checkValues()


class CExport_page3(QtGui.QWizardPage, Ui_ExportPage3):
    def __init__(self, parent = None):
        QtGui.QWizardPage.__init__(self, parent)
        self.setupUi(self)
        self.readyNext = False
        self.setFinalPage(True)

    def isComplete(self):
        return self.readyNext

    def validatePage(self):
        QtGui.qApp.preferences.save()
        return True

    @QtCore.pyqtSignature('')
    def on_btnExport_clicked(self):
        self.btnExport.setEnabled(False)
        exportPath = QtGui.qApp.preferences.appPrefs['exportPath']
        eisLpuId = QtGui.qApp.preferences.appPrefs['EISLpuId']
        orgId = forceRef(QtGui.qApp.preferences.appPrefs['orgId'])
        ffomsCode = forceString(QtGui.qApp.preferences.appPrefs['ffomsCode'])
        export(QtGui.qApp.db, self.log, exportPath, orgId, ffomsCode, eisLpuId)
        self.readyNext = True
        self.emit(QtCore.SIGNAL('completeChanged()'))


if __name__ == '__main__':
    app = CS11mainApp(sys.argv, False, 'serviceZonesExport.ini', None, 'Service zones export')
    stdTtranslator = QtCore.QTranslator()
    stdTtranslator.load(os.path.join(os.getcwd(), 'i18n', 'std_ru.qm'))
    app.installTranslator(stdTtranslator)
    stdTtranslatorLib = QtCore.QTranslator()
    stdTtranslatorLib.load(os.path.join(os.getcwd(), 'i18n', 'qt_ru_lib.qm'))
    app.installTranslator(stdTtranslatorLib)
    QtGui.qApp = app
    window = CExport()
    window.show()
    res = app.exec_()
    if app.db:
        app.closeDatabase()
