# -*- coding: utf-8 -*-

#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################


import re
import time

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import *

from Exchange.ExportHL7 import getPersonHireDate, getPersonFireDate
from Orgs.Utils import getOrgStructureDescendants
from library.Utils import *

from Ui_Progress import Ui_ProgressDialog


def decodeString(v):
    return unicode(v.toString().simplified(),'cp1251')


class CProgressDialog(QtGui.QDialog, Ui_ProgressDialog):
    __pyqtSignals__ = ('startWork()', )

    def __init__(self, parent):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)
        self.EISDb = None
        self.orgId = None
        self.orgStructureId = None
        self.onlyWithHistory = True
        self.doctorCodeSource = 1
        self.muIdent = None
        self.mapSpecialityIdToRegionalCode = {}

        self.setRecordCount(0)
        self.start()
        self.connect(self, SIGNAL('startWork()'), self.startWork, Qt.QueuedConnection)


    def _setCount(self, progressBar, cnt):
        if cnt:
            progressBar.setValue(0)
            progressBar.setMaximum(cnt)
            progressBar.setEnabled(True)
            progressBar.setTextVisible(True)
        else:
            progressBar.setValue(0)
            progressBar.setMaximum(1)
            progressBar.setEnabled(False)
            progressBar.setTextVisible(False)


    def setRecordCount(self, cnt):
        self._setCount(self.prbRecords, cnt)


    def start(self):
        self.processDone = False
        self.aborted = False
        self.btnCancel.setText(u'Отмена')


    def stop(self):
        self.processDone = True
        self.btnCancel.setText(u'Закрыть')


    def done(self, r):
        self.stop()
        QtGui.QDialog.done(self, r)


    def exec_(self):
        self.emit(SIGNAL('startWork()'))
        return QtGui.QDialog.exec_(self)


    @pyqtSignature('')
    def on_btnCancel_clicked(self):
        if self.processDone:
            self.accept()
        else:
            self.aborted = True


    def step(self, procedure):
        db = QtGui.qApp.db
        db.transaction()
        try:
            procedure()
            if self.aborted:
                db.rollback()
                return
            db.commit()
        except:
            db.rollback()
            raise


    def process(self, query, procedure, progressBar):
        n = progressBar.value()
        t0 = time.clock()
        while query.next():
            QtGui.qApp.processEvents()
            if self.aborted:
                self.reject()
                return
            n += 1
            t1 = time.clock()
            if t1-t0>0.25:
                progressBar.setValue(n)
                t0 = t1
            procedure(query.record())
        progressBar.setValue(n)


    def startWork(self):
        self.start()
        QtGui.qApp.call(self, self.work)
        self.stop()


    def work(self):
        def getPersonCount():
            db = QtGui.qApp.db
            tablePerson = db.table('Person')
            tableSpeciality = db.table('rbSpeciality')
            table = tablePerson.leftJoin(tableSpeciality, tableSpeciality['id'].eq(tablePerson['speciality_id']))
            cond = self.getPersonCond()
            return db.getCount(table, '1', cond)
        self.setRecordCount(QtGui.qApp.callWithWaitCursor(self, getPersonCount))
        self.setupMuIdent()
        self.step(self.personStep)


    def getPersonCond(self):
        db = QtGui.qApp.db
        tablePerson = db.table('Person')
        tableSpeciality = db.table('rbSpeciality')
        cond = [ tablePerson['deleted'].eq(0),
                 tablePerson['org_id'].eq(self.orgId),
                 tablePerson['lastName'].ne(''),
                 tablePerson['firstName'].ne(''),
                 tablePerson['speciality_id'].isNotNull(),
                 tableSpeciality['regionalCode'].ne(''),
               ]
        if self.orgStructureId:
            orgStructureIdList = getOrgStructureDescendants(self.orgStructureId)
            cond.append( tablePerson['orgStructure_id'].inlist(orgStructureIdList) )
        return cond

# *****************************************************************************************

    def personStep(self):
        db = QtGui.qApp.db
        tablePerson = db.table('Person')
        tableSpeciality = db.table('rbSpeciality')
        table = tablePerson.leftJoin(tableSpeciality, tableSpeciality['id'].eq(tablePerson['speciality_id']))

        cond = self.getPersonCond()
        stmt = db.selectStmt(table, 'Person.*', cond)
        self.process(db.query(stmt), self.addPersonOrUpdate, self.prbRecords)


    def setupMuIdent(self):
        record = self.EISDb.getRecordEx('MU_IDENT', 'ID_LPU')
        if record:
            self.muIdent = forceInt(record.value('ID_LPU'))
        else:
            self.muIdent = 0


    def getPRVS(self, specialityId):
        result = self.mapSpecialityIdToRegionalCode.get(specialityId, None)
        if result is None:
            result = forceInt(QtGui.qApp.db.translate('rbSpeciality', 'id', specialityId, 'regionalCode'))
            self.mapSpecialityIdToRegionalCode[specialityId] = result
        return result


    def getDoctorCode(self, record):
        fieldName = {1:'code', 2:'regionalCode', 3:'federalCode'}.get(self.doctorCodeSource, None)
        if fieldName:
            return forceString(record.value(fieldName))
        else:
            return ''


    def addPersonOrUpdate(self, record):
        db = QtGui.qApp.db
        personId = forceRef(record.value('id'))
        endDate = getPersonFireDate(personId) or QDate(2200, 1, 1)
        regionalCode = forceString(record.value('regionalCode'))
        if regionalCode and regionalCode.find('.') < 0:
            regionalCode = str(self.muIdent) + '.' + regionalCode
        outId = regionalCode if regionalCode else str(self.muIdent) + '.' + forceString(record.value('id'))
#        outId = (str(self.muIdent) + '.'+ regionalCode) if regionalCode else str(self.muIdent) + '.' + forceString(record.value('id'))
        table = self.EISDb.table('VMU_DOCTORS')
        table.setIdFieldName('ID_DOCTOR')
        currRecord = self.EISDb.getRecordEx(table, 'ID_DOCTOR, DOCTOR_END_DATE, DOCTOR_CODE, CHANGE_DATE, SEND', table['ID_DOCTOR'].eq(outId))
        if currRecord:
            code    = self.getDoctorCode(record)
            if (    forceDate(currRecord.value('DOCTOR_END_DATE')) != endDate
                 or forceString(currRecord.value('DOCTOR_CODE')) != code ):
                currRecord.setValue('DOCTOR_END_DATE', toVariant(endDate))
                currRecord.setValue('DOCTOR_CODE', toVariant(code))
                currRecord.setValue('CHANGE_DATE', toVariant(QDate.currentDate()))
                currRecord.setValue('SEND', toVariant(0))
                self.EISDb.updateRecord(table, currRecord)
        else:
            begDate = getPersonHireDate(personId)
            if self.onlyWithHistory <= (begDate is not None):
                prvsId = self.getPRVS(forceRef(record.value('speciality_id')))
                firstName = forceStringEx(record.value('firstName'))
                lastName = forceStringEx(record.value('lastName'))
                if prvsId and firstName and lastName:
                    outRecord = table.newRecord()
                    outRecord.setValue('ID_DOCTOR',   toVariant(outId))
                    outRecord.setValue('ID_LPU',      toVariant(self.muIdent))
                    outRecord.setValue('SURNAME',     record.value('lastName'))
                    outRecord.setValue('NAME',        record.value('firstName'))
                    outRecord.setValue('SECOND_NAME', record.value('patrName'))
                    outRecord.setValue('DOCTOR_CODE', toVariant(self.getDoctorCode(record)))
                    outRecord.setValue('ID_PRVS',     toVariant(prvsId))
                    outRecord.setValue('DOCTOR_BEGIN_DATE', toVariant(begDate or QDate(2000, 1, 1)))
                    outRecord.setValue('DOCTOR_END_DATE', toVariant(endDate))
                    outRecord.setValue('CHANGE_DATE', toVariant(QDate.currentDate()))
                    outRecord.setValue('SEND', toVariant(0))
                    self.EISDb.insertRecord(table, outRecord)
