#!/usr/bin/env python
# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import *

from library.Utils import *

def getId(table, fields, fields2=[]):
    db = QtGui.qApp.db
    cond = []
    for name, val in fields:
        if val is None:
            cond.append(table[name].isNull())
        else:
            if isinstance(val, QDate):
                cond.append(table[name].dateEq(toVariant(val)))
            else:
                cond.append(table[name].eq(toVariant(val)))
    record=db.getRecordEx(table, '*', where=cond)
    if record:
        updateRecord = False
        for (name, val) in fields2:
            recVal=record.value(name)
            if (recVal.isNull() or forceString(recVal)=='') and bool(val):
                record.setValue(name, toVariant(val))
                updateRecord = True
        if updateRecord:
            db.updateRecord(table, record)
        return forceRef(record.value('id'))
    else:
        record = table.newRecord()
        for (name, val) in fields+fields2:
            record.setValue(name, toVariant(val))
        return db.insertRecord(table, record)






class CImportLogTextEdit(QtGui.QTextEdit):
    u"""Лог для записей при импорте; поддерживает синхронную запись в файл"""
    def __init__(self, parent):
        QtGui.QTextEdit.__init__(self, parent)
        self.logfile = None
        self.logstream = None
        self.setReadOnly(True)

    def setLogFile(self, filename):
        if filename:
            self.logfile = QtCore.QFile(filename)
            self.logfile.open(QtCore.QFile.Text | QtCore.QFile.WriteOnly)
            self.logstream = QtCore.QTextStream(self.logfile)
        else:
            if self.logfile:
                self.logfile.close()
            self.logfile = None
            self.logstream = None

    def appendLogFile(self, str):
        QtGui.qApp.processEvents()
        if self.logstream:
            self.logstream << QtCore.QDateTime.currentDateTime().toString("yyyy-MM-dd hh:mm:ss\t");
            self.logstream << str << '\n'
            self.logstream.flush()

    def append(self, str):
        QtGui.QTextEdit.append(self, str)
        self.appendLogFile(str)

    def close(self):
        if self.logfile:
            self.logfile.close()
        QtGui.QWidget.close(self)
