#!/usr/bin/env python
# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import sys
import os
import traceback

from PyQt4 import QtCore, QtGui, QtSql
from PyQt4.QtCore import *
from PyQt4.QtSql import *

from library               import database
from library.Utils         import *
from library.Preferences   import CPreferences
from library.DbfViewDialog import CDbfViewDialog

from preferences.connection import CConnectionDialog
from preferences.decor      import CDecorDialog

from Utils import *

from library.dbfpy.dbf import Dbf

from Ui_main import Ui_MainWindow

BTN_STOP_STOP       = u'Остановить'
BTN_STOP_PAUSE      = u'Пауза'
BTN_IMPORT_IMPORT   = u'Импорт'
BTN_IMPORT_CONTINUE = u'Продолжить'


class CImportINFISApp(QtGui.QApplication):
    __pyqtSignals__ = ('dbConnectionChanged(bool)',)
    def __init__(self, args):
        QtGui.QApplication.__init__(self, args)
        self.logDir = os.path.join(unicode(QDir.toNativeSeparators(QDir.homePath())), '.samson-vista')
        self.preferences = CPreferences('importINFIS.ini')
        self.preferences.load()
        self.db = None
        self.userId = None
        self.mainWindow = None
        self.registerDocumentTables()

    def registerDocumentTables(self):
        database.registerDocumentTable('Account')
        database.registerDocumentTable('Action')
        database.registerDocumentTable('ActionProperty')
        database.registerDocumentTable('ActionPropertyTemplate')
        database.registerDocumentTable('ActionTemplate')
        database.registerDocumentTable('ActionType')
        database.registerDocumentTable('Address')
#        database.registerDocumentTable('AddressAreaItem')
        database.registerDocumentTable('AddressHouse')
        database.registerDocumentTable('Bank')
        database.registerDocumentTable('CalendarExceptions')
        database.registerDocumentTable('Client')
        database.registerDocumentTable('ClientAddress')
        database.registerDocumentTable('ClientAllergy')
        database.registerDocumentTable('ClientAttach')
        database.registerDocumentTable('ClientContact')
        database.registerDocumentTable('ClientIdentification')
        database.registerDocumentTable('ClientIntoleranceMedicament')
        database.registerDocumentTable('ClientDocument')
        database.registerDocumentTable('ClientPolicy')
        database.registerDocumentTable('ClientSocStatus')
        database.registerDocumentTable('ClientWork')
        database.registerDocumentTable('Contract')
        database.registerDocumentTable('Diagnosis')
        database.registerDocumentTable('Diagnostic')
        database.registerDocumentTable('Event')
        database.registerDocumentTable('EventType')
        database.registerDocumentTable('InformerMessage')
        database.registerDocumentTable('Licence')
        database.registerDocumentTable('Organisation')
        database.registerDocumentTable('OrgStructure')
        database.registerDocumentTable('Person')
        database.registerDocumentTable('TempInvalid')
        database.registerDocumentTable('Visit')


    def notify(self, receiver, event):
        try:
#            report = ["notify:: receiver.type=",unicode(type(receiver))," receiver.name=", unicode(receiver.objectName())," event.type=",unicode(type(event)), "event.type()=", unicode(event.type())]
#            print >>sys.stderr, ("".join(report)).encode('cp866')
            return QtGui.QApplication.notify(self, receiver, event)
        except Exception, e:
            if self.traceActive:
                self.logCurrentException()
                widget = self.activeModalWidget()
                if widget is None:
                    widget = self.mainWindow
                QtGui.QMessageBox.critical( widget,
                                            u'Произошла ошибка',
                                            unicode(e),
                                            QtGui.QMessageBox.Close)
            return False
        except:
            return False

    def log(self, title, message, stack=None):
        try:
            if not os.path.exists(self.logDir):
                os.makedirs(self.logDir)
            logFile = os.path.join(self.logDir, 'error.log')
            timeString = unicode(QDateTime.currentDateTime().toString(Qt.SystemLocaleDate))
            logString = u'%s\n%s: %s(%s)\n' % ('='*72, timeString, title, message)
            if stack:
                try:
                    logString += ''.join(traceback.format_list(stack)).decode('utf-8') + '\n'
                except:
                    logString += 'stack lost\n'
            file = codecs.open(logFile, mode='a', encoding=locale.getpreferredencoding())
            file.write(logString)
            file.close()
        except:
            pass


    def logException(self, exceptionType, exceptionValue, exceptionTraceback):
        title = repr(exceptionType)
        message = unicode(exceptionValue)
        self.log(title, message, traceback.extract_tb(exceptionTraceback))
        sys.__excepthook__(exceptionType, exceptionValue, exceptionTraceback)


    def logCurrentException(self):
        self.logException(*sys.exc_info())


    def msgHandler(self, type, msg):
        if type == 0: # QtMsgType.QtDebugMsg:
            typeName = 'QtDebugMsg'
        elif type == 1: # QtMsgType.QtWarningMsg:
            typeName = 'QtWarningMsg'
        elif type == 2: # QtMsgType.QtCriticalMsg:
            typeName = 'QtCriticalMsg'
        elif type == 3: # QtFatalMsg
            typeName = 'QtFatalMsg'
        else:
            typeName = 'QtUnknownMsg'

        self.log( typeName, msg, traceback.extract_stack()[:-1])
#        print typeName, msg


    def openDatabase(self):
        self.db = None
        self.db = database.connectDataBase(self.preferences.dbDriverName,
                                     self.preferences.dbServerName,
                                     self.preferences.dbServerPort,
                                     self.preferences.dbDatabaseName,
                                     self.preferences.dbUserName,
                                     self.preferences.dbPassword,
                                     compressData = self.preferences.dbCompressData)
        self.emit(QtCore.SIGNAL('dbConnectionChanged(bool)'), True)


    def closeDatabase(self):
        if self.db:
            self.db.close()
            self.db = None
            self.emit(QtCore.SIGNAL('dbConnectionChanged(bool)'), False)


    def applyDecorPreferences(self):
        self.setStyle(self.preferences.decorStyle)
        if self.preferences.decorStandardPalette:
            self.setPalette(self.style().standardPalette())
        if self.mainWindow:
            state = Qt.WindowNoState
            if self.preferences.decorMaximizeMainWindow:
                state |= Qt.WindowMaximized
            if self.preferences.decorFullScreenMainWindow:
                state |= Qt.WindowFullScreen
            self.mainWindow.setWindowState(state)

# ###############################3
badInsurerCodeList = []


class CMainWindow(QtGui.QMainWindow, Ui_MainWindow):
    def __init__(self):
        QtGui.QMainWindow.__init__(self)
        self.setupUi(self)
        self.dbfFile                       = None
        self.manufacturingOMSPolicyType    = None
        self.notManufacturingOMSPolicyType = None
        self.started                       = False
        self.reportType                    = None
        self.currentRow                    = None
        self.workTypes                     = []
        self.passportTypeId                = None
        self.preSetFileName()
        self.on_actLogin_triggered()
        self.setWindowTitle(u'Иморт из ИНФИС')
        self.progressBar.setFormat('%v')
        self.progressBar.reset()
        self.isChkDoubles = False
        self.aborted = True
        self.logList.setLogFile('importINFIS.log')
        self.widgetDuringImport = [self.actLogin, self.actLogout, self.actQuit, self.actDecor, self.actConnection,
                                   self.btnOpen, self.btnView]
        self.chkDoubles.setVisible(False)




    def setEnabledByConnection(self, value):
        widgets = [self.grpSource, self.grpSettings, self.logList, self.btnCleanLog]
        for widget in widgets:
            widget.setEnabled(value)
        if self.dbfFile and value:
            self.btnImport.setEnabled(value)
        else:
            self.btnImport.setEnabled(False)



    def preSetFileName(self):
        fileName = forceString(QtGui.qApp.preferences.appPrefs.get('openFIleDbf', QVariant()))
        if bool(fileName):
            if self.openDbfFile(fileName):
                self.edtFileName.setText(fileName)

    def openDbfFile(self, fileName):
        try:
            self.dbfFile = Dbf(fileName, readOnly=True, encoding='cp866')
            rowsCount = len(self.dbfFile)
            self.lblCount.setText(u'всего записей в источнике: %d' % rowsCount)
            if QtGui.qApp.db is not None:
                self.btnImport.setEnabled(True)
            return True
        except:
            self.dbfFile = None
            self.lblCount.setText(u'')
            self.btnImport.setEnabled(False)
            return False

    def startImport(self):
        db = QtGui.qApp.db
        if not (bool(db) and bool(self.dbfFile)):
            return
        self.started = True
        self.aborted = False
        if not self.currentRow:
            self.progressBar.reset()
            self.progressBar.setValue(1)
        self.progressBar.setMaximum(len(self.dbfFile))
        self.progressBar.setFormat('%v')
        self.tableClient         = db.table('Client')
        self.tableClientPolicy   = db.table('ClientPolicy')
        self.tableClientDocument = db.table('ClientDocument')
        self.tableAddressHouse   = db.table('AddressHouse')
        self.tableAddress        = db.table('Address')
        self.tableClientAddress  = db.table('ClientAddress')
        self.isChkDoubles = self.chkDoubles.isChecked()
        if self.chkReport.isChecked():
            self.reportType = 1 if self.rdbFullReport.isChecked() else 2
        else:
            self.reportType = None
        if self.currentRow == 0:
            self.logList.clear()
        self.process(self.stepImport)
        if not self.aborted:
            self.btnImport.setText(BTN_IMPORT_IMPORT)
            self.btnStop.setText(BTN_STOP_STOP)
            self.btnImport.setEnabled(True)
            self.btnStop.setEnabled(False)
            self.started = False
            self.currentRow = None
            self.setEnabledWidgetDuringImport(True)
            self.btnCleanLog.setEnabled(True)



    def process(self, stepFunc):
        try:
            if self.currentRow is None:
                self.currentRow = 0
            count = len(self.dbfFile)
            while self.currentRow < count:
                rowData = self.dbfFile[self.currentRow]
                QtGui.qApp.processEvents()
                if self.aborted:
                    return
                self.progressBar.setValue(self.progressBar.value()+1)
                stepFunc(rowData)
                self.currentRow += 1

        except ValueError:
            self.logList.append(u'<b><font color=red>ОШИБКА:</b>'
                    u' неверное значение строки "%d" в dbf.' % \
                    (self.progressBar.value()+1))
            self.on_btnStop_clicked()
            pass


    def stepImport(self, data):
        lastName         = trim(data['SURNAME'])
        firstName        = trim(data['NAME1'])
        patrName         = trim(data['NAME2'])
        birthDate        = self.getBirthDate(data['BIRTHDAY'])
        sex              = self.getSamsonSexValue(unicode(data['SEX']))
        policySerial     = trim(data['POLIS_S'])
        policyNumber     = trim(data['POLIS_N'])
        insurerInfisCode = trim(data['POLIS_W'])
        policyTypeId     = self.getSamsonPolicyType(unicode(data['TYPEINS']))
        KLADRStreetCode  = self.getSamsonKladrStreetCode(unicode(data['STREET']))
        KLADRCode        = self.getSamsonKladrCode(unicode(data['AREA']), KLADRStreetCode)
        house            = trim(data['HOUSE'])
        korp             = trim(data['KORP'])
        flat             = trim(data['FLAT'])
        dopInf           = self.getDopInfoDict(data['DOPINF'])
        clientId         = self.getClient(lastName, firstName, patrName, birthDate, sex)
        clientPolicyId   = self.getClientPolicy(insurerInfisCode, policyTypeId, policySerial, policyNumber, clientId)
        if bool(clientId):
            clientDocumentId = self.getClientDocumentId(dopInf, clientId)
            clientAddressId  = self.getClientAddress(clientId, KLADRStreetCode, KLADRCode, house, korp, flat)


    def getClientAddress(self, clientId, KLADRStreetCode, KLADRCode, house, korp, flat):
        addressHouseId = self.getAddressHouseId(KLADRCode, KLADRStreetCode, house, korp)
        if addressHouseId:
            addressId = self.getAddressId(addressHouseId, flat)
            if addressId:
                clientAddressFields = [
                                      ('client_id', clientId), ('type', 0),
                                      ('address_id', addressId)
                                      ]
                clientAddressId = getId(self.tableClientAddress, clientAddressFields)
                if clientAddressId:
                    self.setImportInfo(u'Адрес записан')

    def getAddressHouseId(self, KLADRCode, KLADRStreetCode, house, korp):
        if bool(KLADRCode) and (bool(KLADRStreetCode) or KLADRStreetCode == '*'):
            if KLADRStreetCode == '*':
                KLADRStreetCode = ''
            addressHouseFields = [
                                  ('KLADRCode', KLADRCode), ('KLADRStreetCode', KLADRStreetCode),
                                  ('number', house), ('corpus', korp)
                                  ]
            return getId(self.tableAddressHouse, addressHouseFields)
        self.setImportErrorInfo(u'Недостаточно данных об адресе пациента', forceAdd=False)
        return None

    def getAddressId(self, addressHouseId, flat):
        addressFields = [
                         ('house_id', addressHouseId), ('flat', flat)
                         ]
        return getId(self.tableAddress, addressFields)

    def getQDate(self, date):
        try:
            date.replace(',', '.').replace('-', '.').replace(':', '.')
            if len(date) == 8:
                parts = date.split('.')
                cDate = QDate.currentDate()
                if int(unicode(cDate.year())[2:]) >= int(parts[2]):
                    year = int('20'+parts[2])
                else:
                    year = int('19'+parts[2])
                month = int(parts[1])
                day = int(parts[0])
                return QDate(year, month, day)
            elif len(date) == 10:
                return QDate.fromString(date, 'dd.MM.yyyy')
            return None
        except:
            return None

    def getClientDocumentId(self, dopInf, clientId):
        docType = dopInf.get('Ser_type', None)
        docDate = dopInf.get('Ser_date', None)
        docOf   = dopInf.get('Ser_of', None)
        serial  = dopInf.get('Pasport_s', None)
        number  = dopInf.get('Pasport_n', None)
        if not (bool(serial) and bool(number) and bool(docType)):
            return None
        if u'паспорт' in unicode(docType).lower():
            docDate = docDate.replace('\r', '') if docDate else docDate
            if not docDate:
                return
            docDate = self.getQDate(docDate)
            if self.passportTypeId is None:
                self.passportTypeId = forceRef(QtGui.qApp.db.translate('rbDocumentType', 'code', '1', 'id'))
            if not self.passportTypeId:
                return
            serial = serial.replace('"', '')
            number = number.replace('"', '')
#            clientDocumentList  = [
#                                  ('client_id', clientId), ('documentType_id', self.passportTypeId)
#                                  ]
#            clientDocumentList2 = [
#                                  ('serial', serial), ('number', number), ('date', docDate), ('origin', docOf)
#                                  ]
#            return getId(self.tableClientDocument, clientDocumentList, clientDocumentList2)

            u'''Что-то ссыкотно мне обнавлять паспортные данные'''


            clientDocumentList = [
                                  ('client_id', clientId), ('documentType_id', self.passportTypeId),
                                  ('serial', serial), ('number', number), ('date', docDate), ('origin', docOf)
                                  ]
            return getId(self.tableClientDocument, clientDocumentList)



    def getClient(self, lastName, firstName, patrName, birthDate, sex):
        lastName  = nameCase(lastName)
        firstName = nameCase(firstName)
        patrName  = nameCase(patrName)
        if not (lastName and firstName and patrName):
            self.setImportErrorInfo(u'Недопустимое значение имен пациента', forceAdd=False)
            return None
        if not sex:
            self.setImportErrorInfo(u'Не удалось определить пол', forceAdd=False)
            return None
        if not birthDate:
            self.setImportErrorInfo(u'Отсутствует дата рождения', forceAdd=False)
            return None
        clientFields = [
                ('lastName', lastName), ('firstName', firstName), ('patrName', patrName),
                ('sex', sex), ('birthDate', birthDate)]
        id = getId(self.tableClient, clientFields)
        self.setImportInfo(u'%s %s %s %s id%d'%(lastName, firstName, patrName, forceString(birthDate), id))
        return id

    def getClientPolicy(self, insurerInfisCode, policyTypeId, policySerial, policyNumber, clientId):
        if not clientId:
            return
        if not (bool(policySerial) and bool(policyNumber) and bool(policyTypeId) and bool(insurerInfisCode)):
            self.setImportErrorInfo(u'Недостаточно данных о полисе пациента', forceAdd=False)
            return None
        insurerId = forceRef(QtGui.qApp.db.translate('Organisation', 'infisCode', insurerInfisCode, 'id'))
        if insurerId:
            clientPolicyFields = [
                    ('client_id', clientId), ('insurer_id', insurerId), ('policyType_id', policyTypeId),
                    ('serial', policySerial), ('number', policyNumber)]
            id = getId(self.tableClientPolicy, clientPolicyFields)
            return id
        else:
            self.setImportErrorInfo(u'Не удалось найти страховую компанию по коду "%s"'%insurerInfisCode, forceAdd=False, insurerCode=True)
            return None


    def setImportInfo(self, message):
        add = False
        if self.reportType:
            self.logList.append(message)
        else:
            self.logList.appendLogFile(message)

    def setImportErrorInfo(self, message, forceAdd=False, insurerCode=''):
        add = False
        if forceAdd:
            add = True
        else:
            if self.reportType:
                if self.reportType == 1:
                   add = True
                else:
                    if bool(insurerCode):
                        global badInsurerCodeList
                        if insurerCode not in badInsurerCodeList:
                            add = True
                            badInsurerCodeList.append(insurerCode)
        if add:
            self.logList.append(u'<b><font color=red>ОШИБКА: </b>'+message)
        else:
            self.logList.appendLogFile(u'ОШИБКА: '+message)

    def getBirthDate(self, val):
        if val:
            try:
                return QDate(val)
            except:
                return None
        return None

    def getDopInfoDict(self, txt):
        u'''
        Worktype  - работает ли('НеРаб', 'Работ', 'ВСлуж')
        Ser_type  - тип документа('нет паспорта', 'РФ паспорт', 'паспорт', 'др.док-т удост.личн.')
        Ser_date  - дата выдачи документа
        Ser_of    - кем выдан документ
        Pasport_s - серия документа
        Pasport_n - номер документа
        '''
#        global docValues
        d = {}
        try:
            pairs = txt.split('[Dopcard]\r\n')[1].split('\n')
            for pair in pairs:
                if '=' in pair:
                    parts  = pair.split('=')
                    key    = parts[0]
                    value  = parts[1]
                    if value:
                        value.replace('\r', '').replace('"', '')
                    d[key] = value
#                    if key == 'Ser_type':
#                        if value not in docValues:
#                            print value
#                            docValues.append(value)
        except:
            return {}
        return d

    def getKladrCodeByOkato(self, fullCode):
        kladr = None
        if fullCode:
            kladr = QtGui.qApp.db.getRecordEx('kladr.KLADR', 'CODE', 'infis LIKE "%s"'%(fullCode+'%'))
        if kladr:
            return forceStringEx(kladr.value('CODE'))
        else:
            return '7800000000000'

    def getSamsonKladrCode(self, infisCode, infisStreet):
        db = QtGui.qApp.db
        table = 'kladr.KLADR'
        if infisStreet == '*':
            return forceString(db.translate(table, 'infis', infisCode, 'CODE'))
        if len(trim(infisCode)) == 2:
            if infisCode == u'ЛО':
                return u'4700000000000'
            okatoCode = forceStringEx(db.translate('kladr.OKATO', 'infis', infisCode, 'CODE'))
            if okatoCode:
                return self.getKladrCodeByOkato(okatoCode)
        if len(trim(infisCode)) == 3:
            return forceString(db.translate(table, 'infis', infisCode, 'CODE'))

    def getSamsonKladrStreetCode(self, infisCode):
        if '*' in infisCode:
            return '*'
        table = 'kladr.STREET'
        result = forceString(QtGui.qApp.db.translate(table, 'infis', infisCode, 'CODE'))
        return result

    def getSamsonPolicyType(self, infisPolicyType):
        if infisPolicyType.upper() == u'П':
            return self.manufacturingOMSPolicyType
        else:
            return self.notManufacturingOMSPolicyType

    def getSamsonSexValue(self, infisSex):
        infisSex = infisSex.upper()
        if infisSex == u'М':
            return 1
        if infisSex == u'Ж':
            return 2
        return None

    def setEnabledWidgetDuringImport(self, value):
        for widget in self.widgetDuringImport:
            widget.setEnabled(value)

    @QtCore.pyqtSignature('')
    def on_btnCleanLog_clicked(self):
        self.logList.clear()

    @QtCore.pyqtSignature('')
    def on_btnImport_clicked(self):
        self.setEnabledWidgetDuringImport(False)
        self.btnStop.setEnabled(True)
        self.btnCleanLog.setEnabled(False)
        if self.currentRow is None:
            self.logList.clear()
        if self.started or self.currentRow is None:
            self.btnStop.setText(BTN_STOP_PAUSE)
#        else:
#            self.btnStop.setText(BTN_STOP_STOP)
        self.btnImport.setEnabled(False)
        self.startImport()


    @QtCore.pyqtSignature('')
    def on_btnStop_clicked(self):
        if self.aborted:
            self.currentRow = 0
            self.started = False
            self.btnImport.setText(BTN_IMPORT_IMPORT)
            self.btnStop.setEnabled(False)
            self.currentRow = None
            self.setEnabledWidgetDuringImport(True)
        else:
            self.btnImport.setText(BTN_IMPORT_CONTINUE)
            self.btnStop.setText(BTN_STOP_STOP)
        self.aborted = True
        self.btnImport.setEnabled(True)
        self.btnCleanLog.setEnabled(True)

    @QtCore.pyqtSignature('')
    def on_btnOpen_clicked(self):
        fileName = QtGui.QFileDialog.getOpenFileName(
            self, u'Укажите файл с данными', self.edtFileName.text(), u'Файлы DBF (*.dbf)')
        if fileName != '':
            self.edtFileName.setText(QtCore.QDir.toNativeSeparators(fileName))
            dbfFileName = forceStringEx(self.edtFileName.text())
            QtGui.qApp.preferences.appPrefs['openFIleDbf'] = QVariant(dbfFileName)
            self.openDbfFile(dbfFileName)


    @QtCore.pyqtSignature('')
    def on_btnView_clicked(self):
        fName=unicode(forceStringEx(self.edtFileName.text()))
        if fName:
            CDbfViewDialog(self, fileName=fName).exec_()

    @pyqtSignature('')
    def on_actLogin_triggered(self):
        try:
            QtGui.qApp.openDatabase()
            self.setEnabledByConnection(True)
            self.actLogout.setEnabled(True)
            self.actLogin.setEnabled(False)
            if bool(QtGui.qApp.db):
                self.manufacturingOMSPolicyType = forceRef(QtGui.qApp.db.translate('rbPolicyType', 'code', '2', 'id'))
                self.notManufacturingOMSPolicyType = forceRef(QtGui.qApp.db.translate('rbPolicyType', 'code', '1', 'id'))
            return
        except database.CDatabaseException, e:
            QtGui.QMessageBox.critical(
                self,
                u'Ошибка открытия базы данных',
                unicode(e),
                QtGui.QMessageBox.Close)
        except Exception, e:
            QtGui.QMessageBox.critical(
                self,
                u'Ошибка открытия базы данных',
                u'Невозможно установить соедиенение с базой данных\n' + unicode(e),
                QtGui.QMessageBox.Close)
            QtGui.qApp.logCurrentException()
        self.manufacturingOMSPolicyType = None
        self.notManufacturingOMSPolicyType = None
        self.setEnabledByConnection(False)
        self.actLogin.setEnabled(True)
        self.actLogout.setEnabled(False)
        QtGui.qApp.closeDatabase()

    @pyqtSignature('')
    def on_actLogout_triggered(self):
        QtGui.qApp.closeDatabase()
        self.manufacturingOMSPolicyType = None
        self.notManufacturingOMSPolicyType = None
        self.actLogout.setEnabled(False)
        self.actLogin.setEnabled(True)
        self.setEnabledByConnection(False)


    @pyqtSignature('')
    def on_actQuit_triggered(self):
        self.close()

    @QtCore.pyqtSignature('')
    def on_actConnection_triggered(self):
        dlg   = CConnectionDialog(self)
        preferences = QtGui.qApp.preferences
        dlg.setDriverName(preferences.dbDriverName)
        dlg.setServerName(preferences.dbServerName)
        dlg.setServerPort(preferences.dbServerPort)
        dlg.setDatabaseName(preferences.dbDatabaseName)
        dlg.setUserName(preferences.dbUserName)
        dlg.setPassword(preferences.dbPassword)
        if dlg.exec_():
            preferences.dbDriverName = dlg.driverName()
            preferences.dbServerName = dlg.serverName()
            preferences.dbServerPort = dlg.serverPort()
            preferences.dbDatabaseName = dlg.databaseName()
            preferences.dbUserName = dlg.userName()
            preferences.dbPassword = dlg.password()
            preferences.save()


    @QtCore.pyqtSignature('')
    def on_actDecor_triggered(self):
        dlg = CDecorDialog(self)
        preferences = QtGui.qApp.preferences
        dlg.setStyle(preferences.decorStyle)
        dlg.setStandardPalette(preferences.decorStandardPalette)
        dlg.setMaximizeMainWindow(preferences.decorMaximizeMainWindow)
        dlg.setFullScreenMainWindow(preferences.decorFullScreenMainWindow)
        if dlg.exec_():
            preferences.decorStyle = dlg.style()
            preferences.decorStandardPalette = dlg.standardPalette()
            preferences.decorMaximizeMainWindow = dlg.maximizeMainWindow()
            preferences.decorFullScreenMainWindow = dlg.fullScreenMainWindow()
            preferences.save()
            QtGui.qApp.applyDecorPreferences()




if __name__ == "__main__":
    app = CImportINFISApp(sys.argv)
    QtGui.qApp = app
    app.applyDecorPreferences()
    mainWindow = CMainWindow()
    app.mainWindow = mainWindow
    app.applyDecorPreferences()
    mainWindow.show()
    app.exec_()
    app.preferences.save()
    app.closeDatabase()
    mainWindow  = None
    app = None
