# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
import re
import sys

from PyQt4 import QtCore, QtGui
from PyQt4.QtCore import *
from PyQt4.QtGui import QDialog

from library.Utils import *
#from library.database import CDatabaseException

from library.dbfpy.dbf  import Dbf

from Ui_ImportTariffsEIS import Ui_ImportTariffsEIS
from Ui_ImportTariffsEIS_TariffSettings import Ui_TariffSettingsDialog
from Ui_DBSettings import Ui_DBSettingsDialog




from Exchange.Utils import dbfCheckNames

tariffKeyFields    = ('service_id')

from Utils import *

class CDBSettings(QDialog, Ui_DBSettingsDialog):
    def __init__(self, parent = None):
        QDialog.__init__(self, parent)
        self.setupUi(self)
        preferences = QtGui.qApp.preferences
        self.edtAddress.setText(preferences.dbServerName)
        self.edtPort.setText(str(preferences.dbServerPort))
        self.edtName.setText(preferences.dbDatabaseName)
        self.edtLogin.setText(preferences.dbUserName)
        self.edtPassword.setText(preferences.dbPassword)

    def setPrefs(self):
        prefs = QtGui.qApp.preferences
        prefs.dbServerName      = forceString(self.edtAddress.text())
        prefs.dbServerPort         = forceInt(self.edtPort.text()) if self.edtPort.text() else 0
        prefs.dbDatabaseName = forceString(self.edtName.text())
        prefs.dbUserName         = forceString(self.edtLogin.text())
        prefs.dbPassword           = forceString(self.edtPassword.text())



class CComboBoxDelegate(QtGui.QItemDelegate):
    def __init__(self, parent, items = []):
        QtGui.QItemDelegate.__init__(self, parent)
        self.items = items


    def createEditor(self, parent, option, index):
        editor = QtGui.QComboBox(parent)
        editor.addItems(self.items)
        return editor


    def setEditorData(self, editor, index):
        model = index.model()
        text = forceString(model.data(index, Qt.EditRole))
        pos = editor.findText(text)
        editor.setCurrentIndex(pos)


    def setModelData(self, editor, model, index):
        model.setData(index, editor.currentText())


class TariffSettings(QtGui.QDialog, Ui_TariffSettingsDialog):
    def __init__(self, parent = None, rules = [], caseCasts = {}):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)

        self.caseCasts = caseCasts
        self.priceTypes = {u'Простая цена':0, u'Цена по формуле МЭС':1}
        self.tariffTypes = {u'Посещение': 0, u'Услуга': 2, u'Событие по МЭС и дительности':10}
        self.units = {}
        unitRecordList = QtGui.qApp.db.getRecordList('rbMedicalAidUnit', 'id, name')
        for record in unitRecordList:
            self.units[forceString(record.value('name'))] = forceInt(record.value('id'))
        self.table.setItemDelegateForColumn(0, CComboBoxDelegate(self, self.priceTypes.keys()))
        self.table.setItemDelegateForColumn(1, CComboBoxDelegate(self, self.caseCasts.keys()))
        self.table.setItemDelegateForColumn(3, CComboBoxDelegate(self, self.tariffTypes.keys()))
        self.table.setItemDelegateForColumn(4, CComboBoxDelegate(self, self.units.keys()))

        if rules == []:
            rules.append([0, 0, '^6', 2])
            rules.append([0, 0, u'^о', 2])
            rules.append([0, 0, '', 2])
            rules.append([1, 0, '', 10])
        for row, rule in enumerate(rules):
            self.table.insertRow(row)
            for col, ruleText in enumerate(rule):
                if col == 0:
                    for key, value in self.priceTypes.items():
                        if value == ruleText:
                            str = QtGui.QTableWidgetItem(unicode(key))
                            break
                elif col == 1:
                    for key, value in self.caseCasts.items():
                        if value == ruleText:
                            str = QtGui.QTableWidgetItem(unicode(key))
                            break
                elif col == 3:
                    for key, value in self.tariffTypes.items():
                        if value == ruleText:
                            str = QtGui.QTableWidgetItem(unicode(key))
                            break
                elif col == 4:
                    for key, value in self.units.items():
                        if value == ruleText:
                            str = QtGui.QTableWidgetItem(unicode(key))
                            break
                else:
                    str = QtGui.QTableWidgetItem(unicode(ruleText))
                self.table.setItem(row, col, str)

    def getRules(self):
        rules = []
        for row in xrange(self.table.rowCount()):
            rule = []
            ptItem = self.table.item(row, 0)
            ccItem = self.table.item(row, 1)
            reItem = self.table.item(row, 2)
            ttItem = self.table.item(row, 3)
            uItem = self.table.item(row, 4)
            if ptItem and ccItem and ttItem and uItem:
                rule.append(self.priceTypes[forceString(ptItem.text())])
                rule.append(self.caseCasts[forceString(ccItem.text())])
                rule.append(forceString(reItem.text())) if reItem else ''
                rule.append(self.tariffTypes[forceString(ttItem.text())])
                rule.append(self.units[forceString(uItem.text())])
                rules.append(rule)
        return rules

    def done(self, result):
        if result > 0:
            for row in xrange(self.table.rowCount()):
                ptItem = self.table.item(row, 0)
                ccItem = self.table.item(row, 1)
                ttItem = self.table.item(row, 3)
                uItem = self.table.item(row, 4)
                if not ptItem or not ccItem or not ttItem or not uItem:
                    QtGui.QMessageBox.critical(self, u'Ошибка', u'Неверно заполнены правила')
                    return
        QtGui.QDialog.done(self, result)


class CImportTariffsEIS(QtGui.QDialog, Ui_ImportTariffsEIS):
    def __init__(self, parent=None, fileName = '', profilesFileName = ''):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)
        self.progressBar.reset()
        self.progressBar.setValue(0)

        self.aborted = False
        self.actionTypeSettings = None
        self.connect(self, QtCore.SIGNAL('import()'), self.import_, Qt.QueuedConnection)
        self.connect(self, QtCore.SIGNAL('rejected()'), self.abort)
        if fileName:
            self.edtFileName.setText(fileName)
        if profilesFileName:
            self.edtProfilesFileName.setText(profilesFileName)

        self.mapTariffKeyToRecordList = {}
        self.caseCasts = {u'Любой': 0}

        appPrefs = QtGui.qApp.preferences.appPrefs

        self.rules = []
        rules = appPrefs.get('rules',  QVariant())
        if rules.type() == 9:
            rows = rules.toList()
            for row in rows:
                if row.type() == 9:
                    rulesRow = []
                    elements = row.toList()
                    for element in elements:
                        rulesRow.append(forceInt(element) if element.type() == 2 else forceString(element))
                    self.rules.append(rulesRow)
        self.fileName = forceString(appPrefs.get('fileName',  ''))
        self.profilesFileName = forceString(appPrefs.get('profilesFileName',  ''))
        self.caseCastFileName = forceString(appPrefs.get('caseCastFileName',  ''))

        if self.fileName:
            self.edtFileName.setText(self.fileName)
        if self.profilesFileName:
            self.edtProfilesFileName.setText(self.profilesFileName)
        if self.caseCastFileName:
            self.edtCaseCastFileName.setText(self.caseCastFileName)

        if fileName and profilesFileName and bool(rules):
            self.btnImport.setEnabled(True)

    def addTariffToMap(self, record):
        key = self.getTariffKey(record)
        type = forceInt(record.value('tariffType'))
        self.mapTariffKeyToRecordList.setdefault(key, []).append(record)


    def getTariffKey(self, record):
        #return tuple([ forceStringEx(record.value(fieldName)) if not record.value(fieldName).isNull() else None for fieldName in tariffKeyFields ])
        age = forceString(record.value('age'))
        agei = 0
        if age == u'18г-':
            agei = 1
        elif age == u'-17г':
            agei = 2
        return (forceInt(record.value('service_id')) if not record.value('service_id').isNull() else 0, agei)


    def abort(self):
        self.aborted = True


    def import_(self):
        prefs = QtGui.qApp.preferences.appPrefs
        prefs['rules'] = self.rules
        prefs['fileName'] = forceString(self.fileName)
        prefs['profilesFileName'] = forceString(self.profilesFileName)
        prefs['caseCastFileName'] = forceString(self.caseCastFileName)
        QtGui.qApp.preferences.save()

        self.rulesRE = {}
        for rule in self.rules:
            self.rulesRE[rule[2]] = re.compile(rule[2])
        contractIdList = self.cmbContract.getIdList()
        if len(contractIdList) != 1:
            QtGui.QMessageBox.critical(self, u'Ошибка', u'Выбрано более одного договора')
            return
        self.contractId = contractIdList[0]
        self.contractEventType = 0
        recordList = QtGui.qApp.db.getRecordList('Contract_Specification', '*', 'master_id = %i and deleted = 0'%self.contractId)
        if len(recordList) == 1:
            self.contractEventType = forceInt(recordList[0].value('eventType_id'))

        tariffRecordList = QtGui.qApp.db.getRecordList('Contract_Tariff', '*', 'master_id = %i and deleted = 0'%self.contractId)
        for record in list(tariffRecordList):
            self.addTariffToMap(record)


        self.aborted = False
        self.btnAbort.setEnabled(True)
        self.onlyCodes = self.chkOnlyCodes.isChecked()
        self.addMissing = self.chkAddServicesAndAT.isChecked()
        self.updateTariff = self.chkUpdateTariff.isChecked()
        self.updateNames = self.chkUpdateNames.isChecked()
        self.btnImport.setEnabled(False)
        success, result = QtGui.qApp.call(self, self.doImport)
        self.btnImport.setEnabled(True)
        if self.aborted or not success:
            self.progressBar.setFormat(u'Прервано')
            self.progressBar.update()
        else:
            self.progressBar.setFormat(u'Готово')
            self.progressBar.update()
        self.btnAbort.setEnabled(False)


    @QtCore.pyqtSignature('')
    def on_btnSelectCaseCastFile_clicked(self):
        self.caseCastFileName = forceString(QtGui.QFileDialog.getOpenFileName(
            self, u'Укажите файл со справочником case_cast', self.edtFileName.text(), u'Файлы DBF (*.dbf)'))
        if self.caseCastFileName:
            self.edtCaseCastFileName.setText(self.caseCastFileName)

    @QtCore.pyqtSignature('')
    def on_btnSelectProfilesFile_clicked(self):
        self.profilesFileName = forceString(QtGui.QFileDialog.getOpenFileName(
            self, u'Укажите файл с профилями', self.edtFileName.text(), u'Файлы DBF (*.dbf)'))
        if self.profilesFileName:
            self.edtProfilesFileName.setText(self.profilesFileName)


    @QtCore.pyqtSignature('')
    def on_btnSelectFile_clicked(self):
        self.fileName = forceString(QtGui.QFileDialog.getOpenFileName(
            self, u'Укажите файл с тарифами', self.edtFileName.text(), u'Файлы DBF (*.dbf)'))
        if self.fileName:
            self.edtFileName.setText(self.fileName)
        if self.fileName and self.profilesFileName:
            self.btnImport.setEnabled(True)


    @QtCore.pyqtSignature('')
    def on_btnTariffSettings_clicked(self):
        try:
            self.caseCasts = {u'Любой': 0}
            dbf = Dbf(forceString(self.caseCastFileName), readOnly=True, encoding='cp866')
            for record in dbf:
                self.caseCasts[record['CC_NAME']] = record['CASE_CAST']
        except:
            QtGui.QMessageBox.critical(self, u'Ошибка', u'Невозможно прочитать справочник case_cast')
            return
        dlg = TariffSettings(rules = self.rules, caseCasts = self.caseCasts)
        if dlg.exec_():
            self.rules = dlg.getRules()
        self.btnImport.setEnabled(self.profilesFileName != '' and self.fileName != '' and bool(self.rules))


    @QtCore.pyqtSignature('')
    def on_btnClose_clicked(self):
        self.accept()


    @QtCore.pyqtSignature('')
    def on_btnAbort_clicked(self):
        self.abort()


    @QtCore.pyqtSignature('')
    def on_btnImport_clicked(self):
        self.emit(QtCore.SIGNAL('import()'))


    @QtCore.pyqtSignature('')
    def on_btnSettings_clicked(self):
        dlg = CGroupActionTypeEditor()
        if self.actionTypeSettings:
            dlg.loadSettings(self.actionTypeSettings)
        if dlg.exec_():
            self.actionTypeSettings = dlg.getSettings()


    @QtCore.pyqtSignature('')
    def on_btnDBSettings_clicked(self):
        dlg = CDBSettings()
        if dlg.exec_():
            dlg.setPrefs()
            try:
                QtGui.qApp.openDatabase()
                self.lblStatus.setText(u'Подключено к базе')
                self.cmbContract.setEnabled(True)
                self.cmbContract.init()
            except:
                self.lblStatus.setText(u'Подключение к базе отсутствует')
                self.cmbContract.setEnabled(False)



    @QtCore.pyqtSignature('QString')
    def on_edtFileName_textChanged(self):
        self.fileName = forceString(self.edtFileName.text())
        self.btnImport.setEnabled(self.profilesFileName != '' and self.fileName != '' and bool(self.rules))


    @QtCore.pyqtSignature('QString')
    def on_edtProfilesFileName_textChanged(self):
        self.profilesFileName = forceString(self.edtProfilesFileName.text())
        self.btnImport.setEnabled(self.profilesFileName != '' and self.fileName != '' and bool(self.rules))


    @QtCore.pyqtSignature('QString')
    def on_edtCaseCastFileName_textChanged(self):
        self.caseCastFileName = forceString(self.edtCaseCastFileName.text())
        self.btnTariffSettings.setEnabled(self.caseCastFileName != '')

    def log(self, str, forceLog = False,  color = None):
        if self.chkFullLog.isChecked() or forceLog:
            if color:
                oldColor = self.logBrowser.textColor()
                self.logBrowser.setTextColor( color )
                self.logBrowser.append(str)
                self.logBrowser.setTextColor( oldColor )
            else:
                self.logBrowser.append(str)
            self.logBrowser.update()

    def getId(self, table, code, name):
        id = None
        cond = [table['code'].eq(code)]
#        if not self.onlyCodes:
#            cond.append(table['name'].eq(name))
        cols = ['id', 'name']
        if table.tableName == 'ActionType':
            cols.append('title')
        record = QtGui.qApp.db.getRecordEx(table, cols, cond)
        if record:
            id = forceInt(record.value('id'))
            dbName = forceString(record.value('name'))
            if dbName != name:
                self.log(u'Наименование услуги отличается,\nв базе "%s",\nв ЕИС  "%s"'%(dbName, name))
                if self.updateNames:
                    record.setValue('name', name)
                    record.setValue('title', name)
                    QtGui.qApp.db.updateRecord(table, record)
                if not self.onlyCodes:
                    id = None
        return id

    def mainReadLoop(self):
        db = QtGui.qApp.db
        tableTariffs = db.table('tmpTariffs')
        tableProfiles = db.table('tmpProfiles')
        tableService = db.table('rbService')
        tableAT = db.table('ActionType')
        tableATS = db.table('ActionType_Service')
        table = tableTariffs.leftJoin(tableProfiles, tableProfiles['ID_PROFILE'].eq(tableTariffs['ID_PROFILE']))
        count = db.getCount(tableTariffs)
        self.progressBar.setValue(0)
        cond = [tableTariffs['DATE_END'].signEx('>', 'CURRENT_DATE()'),
            tableTariffs['DATE_BEGIN'].signEx('<=', 'CURRENT_DATE()')
            ]
        stmt = db.selectStmt(table, where=cond, order=[tableTariffs['ID_PROFILE'].name(), tableTariffs['ZONE_TYPE'].name(), tableTariffs['AMOUNT'].name()])
        limit = 0
        currentTariff = {}
        currentProfileId = 0
        currentZone = ''
        currentCaseCast = None
        iteration = 0
        while limit < count:
            tariffList = {}
            query = db.query(stmt+' limit %i, 1000'%limit)
            while query.next():
                self.progressBar.setValue((iteration * 100)/count)
                iteration += 1
                QtGui.qApp.processEvents()
                if self.aborted:
                    return False
                record = query.record()
                amount = forceInt(record.value('AMOUNT'))
                if not amount:
                    continue
                price = forceDouble(record.value('TARIFF'))
                ageType = 2 if forceString(record.value('ZONE_TYPE')) == u'д' else 1
                case_cast = forceInt(record.value('CASE_CAST'))
                profileName = forceString(record.value('PROF_NAME'))
                profileCode = forceString(record.value('PROF_CODE'))
                profileGlobalCode = forceString(record.value('CODE'))
                profileId = forceInt(record.value('ID_PROFILE'))
                if profileId == currentProfileId:
                    serviceId = currentTariff['serviceId']
                    actionTypeId = currentTariff['actionTypeId']
                else:
                    serviceId = self.getId(tableService, profileCode, profileName)
                    if not serviceId:
                        if self.addMissing:
                            service = tableService.newRecord()
                            service.setValue('code', toVariant(profileCode))
                            service.setValue('name', toVariant(profileName))
                            service.setValue('eisLegacy', toVariant(1))
                            serviceId = db.insertRecord(tableService, service)
                            self.log(u'Добавлен сервис с кодом %s'%profileCode)
                        else:
                            self.log(u'Сервис с кодом %s не найден, пропускаю\n'%profileCode, True)
                            continue

                    actionTypeId = self.getId(tableAT, profileCode, profileName)
                    if not actionTypeId:
                        if self.addMissing:
                            aType = tableAT.newRecord()
                            aType.setValue('code', toVariant(profileCode))
                            aType.setValue('name', toVariant(profileName))
                            aType.setValue('title', toVariant(profileName))
                            aType.setValue('class', toVariant(3))
                            aType.setValue('showInForm', toVariant(1))
                            aType.setValue('group_id', self.mainGroupId)
                            if self.actionTypeSettings:
                                for key in self.actionTypeSettings.keys():
                                    aType.setValue(key, toVariant(self.actionTypeSettings[key]))
                            actionTypeId = db.insertRecord(tableAT, aType)
                            aTypeS = tableATS.newRecord()
                            aTypeS.setValue('master_id', actionTypeId)
                            aTypeS.setValue('service_id', serviceId)
                            db.insertRecord(tableATS, aTypeS)
                            self.log(u'Добавлен тип действия с кодом %s (%s)'%(profileCode, profileName))
                        else:
                            self.log(u'Тип действия с кодом %s не найден\n'%profileCode)

                if profileId != currentProfileId or currentZone != ageType or currentCaseCast != case_cast:
                    if currentTariff:
                        tariffList[(currentTariff['serviceId'], currentZone, currentCaseCast)] = currentTariff
                    currentProfileId = profileId
                    currentZone = ageType
                    currentCaseCast = case_cast
                    currentTariff = {}

                if currentTariff:
                    prices = currentTariff.setdefault('prices', [])
                    if len(prices) == 2:
                        price2 = prices[1]['sum']
                        if price != price2:
                            prices.append({'from':amount-1, 'sum':price2, 'price': round(price-price2, 2)})
                    elif len(prices) == 1:
                        if round(price/amount, 2) != prices[0]['price']:
                            prices.append({'from':amount, 'sum': price, 'price':0})
                else:
                    currentTariff['serviceId'] = serviceId
                    currentTariff['actionTypeId'] = actionTypeId
                    currentTariff['prices'] = [{'from':0,  'sum':0, 'price':round(price/amount, 2)}]
                    currentTariff['profileId'] = profileId
                    currentTariff['profileName'] = profileName
                    currentTariff['profileCode'] = profileCode
                    currentTariff['ageType'] = ageType
                    currentTariff['globalCode'] = profileGlobalCode
                    currentTariff['case_cast'] = case_cast

            for key, tariff in tariffList.items():
                self.addOrUpdateTariff(tariff)

            limit += 1000
        self.addOrUpdateTariff(currentTariff)
        return True


    def addOrUpdateTariff(self, tariff):
        tableCT = QtGui.qApp.db.table('Contract_Tariff')
        serviceId = tariff['serviceId']
        age = tariff['ageType']
        existingTariffList = self.mapTariffKeyToRecordList.get((serviceId, age), [])
        if not existingTariffList:
            unit_id = None
            tariffType = None
            tariffRecord = tableCT.newRecord()
            pricesLen = len(tariff['prices'])
            priceType = 0 if pricesLen < 2 else 1
            for rule in self.rules:
                rec = self.rulesRE.get(rule[2], None)
                if rule[0] == priceType and (rule[1] == tariff['case_cast'] if rule[1] else True) and (rec.search(tariff['profileCode']) if rec else True):
                    unit_id = rule[4]
                    tariffType = rule[3]
                    break
            if tariffType is None:
                self.log(u'Не удалось определить тип тарификации для тарифа %s:%s, case_cast %i, тип цены "%s"'%(tariff['profileCode'],
                    tariff['profileName'], tariff['case_cast'], u'Простая цена'if priceType == 0 else u'Цена по формуле МЭС'), True)
                return
            if not unit_id:
                self.log(u'Не удалось определить единицы учета для тарифа %s:%s, case_cast %i, тип цены "%s"'%(tariff['profileCode'],
                    tariff['profileName'], tariff['case_cast'], u'Простая цена'if priceType == 0 else u'Цена по формуле МЭС'), True)
                return
            tariffRecord.setValue('service_id', toVariant(serviceId))
            tariffRecord.setValue('price', toVariant(tariff['prices'][0]['price']))
            age = u'-17г' if age == 2 else u'18г-'
            tariffRecord.setValue('age', toVariant(age))

            if pricesLen > 1:
                tariffRecord.setValue('frag1start', toVariant(tariff['prices'][1]['from']))
                tariffRecord.setValue('frag1price', toVariant(tariff['prices'][1]['price']))
                tariffRecord.setValue('frag1sum'    , toVariant(tariff['prices'][1]['sum']))
            if pricesLen > 2:
                tariffRecord.setValue('frag2start', toVariant(tariff['prices'][2]['from']))
                tariffRecord.setValue('frag2price', toVariant(tariff['prices'][2]['price']))
                tariffRecord.setValue('frag2sum'    , toVariant(tariff['prices'][2]['sum']))
            tariffRecord.setValue('unit_id', toVariant(unit_id))
            tariffRecord.setValue('tariffType', toVariant(tariffType))
            tariffRecord.setValue('master_id', toVariant(self.contractId))
            if self.contractEventType:
                tariffRecord.setValue('eventType_id', toVariant(self.contractEventType))
            QtGui.qApp.db.insertRecord(tableCT, tariffRecord)
            self.log(u'Добавлен новый тариф "%s"\n'%tariff['profileName'])
        else:
            if self.updateTariff:
                for existingTariff in existingTariffList:
                    existingTariff.setValue('price', toVariant(tariff['prices'][0]['price']))
                    if len(tariff['prices']) > 1:
                        existingTariff.setValue('frag1start', toVariant(tariff['prices'][1]['from']))
                        existingTariff.setValue('frag1price', toVariant(tariff['prices'][1]['price']))
                        existingTariff.setValue('frag1sum'    , toVariant(tariff['prices'][1]['sum']))
                    if len(tariff['prices']) > 2:
                        existingTariff.setValue('frag2start', toVariant(tariff['prices'][2]['from']))
                        existingTariff.setValue('frag2price', toVariant(tariff['prices'][2]['price']))
                        existingTariff.setValue('frag2sum'    , toVariant(tariff['prices'][2]['sum']))
                    QtGui.qApp.db.updateRecord(tableCT, existingTariff)
                    self.log(u'Обновлен тариф "%s"\n\n'%tariff['profileName'])
            else:
                self.log(u'Найден существующий тариф "%s", пропускаем\n\n'%tariff['profileName'])


    def doImport(self):
        db = QtGui.qApp.db
        tableAT = db.table('ActionType')
        cond = [tableAT['name'].eq(u'Услуги'),
                    tableAT['class'].eq(3)]
        record = db.getRecordEx(tableAT, 'id', cond)
        if record:
            self.mainGroupId = forceInt(record.value(0))
        else:
            QtGui.QMessageBox.warning(self, u'Импорт тарифов для договора',
                                      u'не найдена группа "Услуги" в классе "Прочие мероприятия"')
            return
        if not self.fileName or not self.profilesFileName:
            self.log(u'Не указаны файлы для импорта', True)
            return

        fileName = forceStringEx(self.fileName)
        try:
            tariffsDBF = Dbf(fileName, readOnly=True, encoding='cp866')
        except:
            raise CException(u'Ошибка при открытии файла' % fileName)
        tariffsCols = ['ID_PROFILE', 'ZONE_TYPE', 'DATE_BEGIN', 'DATE_END', 'AMOUNT', 'TARIFF', 'CASE_CAST']
        if not dbfCheckNames(tariffsDBF, tariffsCols):
            raise CException(u'файл %s\nне содержит одного из полей:\n%s' % (fileName, ', '.join(tariffsCols)))

        profilesFileName = forceStringEx(self.profilesFileName)
        try:
            profilesDBF = Dbf(profilesFileName, readOnly=True, encoding='cp866')
        except:
            raise CException(u'Ошибка при открытии файла' % profilesFileName)
        profilesCols = ['ID_PROFILE', 'PROF_NAME', 'PROF_CODE']
        if not dbfCheckNames(profilesDBF, profilesCols):
            raise CException(u'файл %s\nне содержит одного из полей:\n%s' % (profilesFileName, ', '.join(profilesCols)))
        self.createTmpDB(tariffsDBF, 'tmpTariffs', tariffsCols)
        self.createTmpDB(profilesDBF, 'tmpProfiles', profilesCols, ['ID_PROFILE'])

        self.log(u'Загрузка файла тарифов.. (может занять длительное время)', True)
        tableTariffs = db.table('tmpTariffs')
        i = 0
        for dbfRecord in tariffsDBF:
            sqlRecord = tableTariffs.newRecord(tariffsCols)
            for colName in tariffsCols:
                sqlRecord.setValue(colName, toVariant(dbfRecord[colName]))
            db.insertRecord(tableTariffs, sqlRecord)
            QtGui.qApp.processEvents()
            if self.aborted:
                return False
            i += 1
        self.log(u'Загружено %i строк'%i)

        self.log(u'Загрузка файла профилей..', True)
        tableProfiles = db.table('tmpProfiles')
        i = 0
        for dbfRecord in profilesDBF:
            sqlRecord = tableProfiles.newRecord(profilesCols)
            for colName in profilesCols:
                sqlRecord.setValue(colName, toVariant(dbfRecord[colName]))
            insertIgnoreRecord(db, tableProfiles, sqlRecord)
            QtGui.qApp.processEvents()
            if self.aborted:
                return False
            i += 1
        self.log(u'Загружено %i строк'%i)

        self.log(u'Обработка...', True)

        return self.mainReadLoop()


    def createTmpDB(self, dbf, tblName, askedCols = None, indexes = None):
        db = QtGui.qApp.db
        cols = []
        for fieldDef in dbf.fieldDefs:
            fieldType = None
            if type(askedCols) == list and fieldDef.name not in askedCols:
                continue
            if fieldDef.typeCode == 'N':
                if fieldDef.decimalCount != 0:
                    fieldType = 'DOUBLE(%i, %i)'%(fieldDef.length, fieldDef.decimalCount)
                else:
                    fieldType = 'INT(%i)'%fieldDef.length
            elif fieldDef.typeCode == 'C':
                fieldType = 'VARCHAR(%i)'%fieldDef.length
            elif fieldDef.typeCode == 'D':
                fieldType = 'DATE'
            if fieldType:
                cols.append('`%s` %s'%(fieldDef.name, fieldType))
        if cols:
            indexList = []
            indexString = ''
            if type(indexes) == list:
                for index in indexes:
                    indexList.append(u'UNIQUE KEY `index_%s` (`%s`)'%(index, index))
            if indexList:
                indexString = u', '.join(indexList) + ' ,'
            stmt='''
            CREATE TEMPORARY TABLE IF NOT EXISTS `%s` (
      `id` int(11) NOT NULL AUTO_INCREMENT,
      %s %s
      PRIMARY KEY (`id`)
    ) ENGINE=MyISAM AUTO_INCREMENT=1 DEFAULT CHARSET=utf8;'''%(tblName, ','.join(cols)+',', indexString)
            query = db.query(stmt)
            if query.first:
                return True
        return False


if __name__ == '__main__':
    from s11main import CS11mainApp
    app = CS11mainApp(sys.argv, None, 'importTariffsEIS.ini', None, None)
    stdTtranslator = QtCore.QTranslator()
    stdTtranslator.load(os.path.join(os.path.dirname(os.path.realpath(sys.argv[0])), 'i18n', 'std_ru.qm'))
    app.installTranslator(stdTtranslator)
    #Перевод стандартных диалогов Qt
    stdTtranslatorLib = QtCore.QTranslator()
    stdTtranslatorLib.load(os.path.join(os.path.dirname(os.path.realpath(sys.argv[0])), 'i18n', 'qt_ru_lib.qm'))
    app.installTranslator(stdTtranslatorLib)
    QtGui.qApp = app
    dlg = CImportTariffsEIS()
    dlg.show()
    try:
        QtGui.qApp.openDatabase()
        dlg.lblStatus.setText(u'Подключено к базе')
        dlg.cmbContract.setEnabled(True)
        dlg.cmbContract.init()
    except:
        dlg.lblStatus.setText(u'Подключение к базе отсутствует')
        dlg.cmbContract.setEnabled(False)
    QtGui.qApp.exec_()
