# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import os.path
from PyQt4.QtCore import *
from PyQt4.QtGui  import *
from Utils import *
#from Ui_settings import Ui_SettingsDialog

class CPreferences(object):
    def __init__(self, iniFileName='settings.ini'):

        self.iniFileName = iniFileName

        self.decorStyle = 'Plastique'
        self.decorStandardPalette = True
        self.decorMaximizeMainWindow = True
        self.decorFullScreenMainWindow = False
        self.useCustomFont = False
        self.font = qApp.font()
        
        self.address = ''
        self.port = 0
        self.updateService = ''

        self.windowPrefs = {}
        self.reportPrefs = {}
        self.appPrefs    = {}
        self.settings    = None


    def getSettings(self):
        if self.settings is None:
            self.settings = QSettings(self.iniFileName, QSettings.IniFormat)
        return self.settings


    def getDir(self):
       settings = self.getSettings()
       fileName = settings.fileName()
       return os.path.dirname(unicode(QDir.toNativeSeparators(fileName)))


    def load(self):
        try:
            settings = self.getSettings()


            settings.beginGroup('windowPrefs')
            self.windowPrefs = {}
            windows = settings.childGroups()
            for windowName in windows:
                setPref(self.windowPrefs, windowName, self.loadProp(settings, windowName))
            settings.endGroup()


            settings.beginGroup('appPrefs')
            self.appPrefs = {}
            for prop in settings.childKeys():
                self.appPrefs[unicode(prop)] = settings.value(prop, QVariant())
            for group in settings.childGroups():
                setPref(self.appPrefs, unicode(group), self.loadProp(settings, group))
            settings.endGroup()
        except:
            pass


    def loadProp(self, settings, propName):
        settings.beginGroup(propName)
        result = {}
        props = settings.childKeys()
        for prop in props:
            setPref(result, prop, settings.value(prop, QVariant()))
        groups = settings.childGroups()
        for group in groups:
            setPref(result, group, self.loadProp(settings, group))
        settings.endGroup()
        return result


    def save(self):
        try:
            settings = self.getSettings()


            settings.beginGroup('windowPrefs')
            for windowName in self.windowPrefs:
                self.saveProp(settings, windowName, self.windowPrefs[windowName])
            settings.endGroup()

            self.saveProp(settings, 'appPrefs', self.appPrefs)

            settings.sync()
        except:
            pass


    def saveProp(self, settings, propName, propVal):
        if type(propVal) == dict:
            settings.beginGroup(propName)
            for subPropName in propVal:
                self.saveProp(settings, subPropName, propVal[subPropName])
            settings.endGroup()
        else:
            settings.setValue(propName, toVariant(propVal))


#class CPreferencesDialog(QDialog, Ui_SettingsDialog):
#    def __init__(self, parent = None):
#        QDialog.__init__(self, parent)
#        self.setupUi(self)
#        prefs = QtGui.qApp.preferences.appPrefs
#        #self.edtAddress.setText(forceString(prefs.get('address', '')))
#        self.edtPort.setValue(forceInt(prefs.get('port', 0)))
#        self.edtUploadService.setText(forceString(prefs.get('uploadservice', '')))
#    
#    def save(self):
#        prefs = QtGui.qApp.preferences.appPrefs
#        #prefs['address'] = self.edtAddress.text()
#        prefs['port'] = self.edtPort.value()
#        prefs['uploadservice'] = self.edtUploadService.text()
