# -*- coding: utf-8 -*-

import os
import locale
import datetime
from PyQt4.QtCore import *

_eng_chars = u"~!@#$%^&qwertyuiop[]asdfghjkl;'zxcvbnm,./QWERTYUIOP{}ASDFGHJKL:\"|ZXCVBNM<>?"
_rus_chars = u"ё!\"№;%:?йцукенгшщзхъфывапролджэячсмитьбю.ЙЦУКЕНГШЩЗХЪФЫВАПРОЛДЖЭ/ЯЧСМИТЬБЮ,"
_trans_table = dict(zip(_rus_chars, _eng_chars))
 
def fix_layout(s):
    return u''.join([_trans_table.get(c, c) for c in s])

def setPref(dict, key, value):
    dict[unicode(key).lower()] = value

def forceBool(val):
    if isinstance(val, QVariant):
        return val.toBool()
    return bool(val)


def forceDate(val):
    if isinstance(val, QVariant):
        return val.toDate()
    if isinstance(val, QDate):
        return val
    if isinstance(val, QDateTime):
        return val.date()
    if val is None:
        return QDate()
    return QDate(val)


def forceTime(val):
    if isinstance(val, QVariant):
        return val.toTime()
    if isinstance(val, QTime):
        return val
    if isinstance(val, QDateTime):
        return val.time()
    if val is None:
        return QTime()
    return QTime(val)


def forceDateTime(val):
    if isinstance(val, QVariant):
        return val.toDateTime()
    if isinstance(val, QDateTime):
        return val
    if isinstance(val, QDate):
        return QDateTime(val)
    if isinstance(val, QTime):
        return QDateTime(QDate(), val)
    if val is None:
        return QDateTime()
    return QDateTime(val)


def forceInt(val):
    if isinstance(val, QVariant):
        return val.toInt()[0]
    elif val is None:
        return 0
    return int(val)


def forceLong(val):
    if isinstance(val, QVariant):
        return val.toLongLong()[0]
    elif val is None:
        return 0L
    return long(val)


def forceRef(val):
    if isinstance(val, QVariant):
        if val.isNull():
            val = None
        else:
            val = val.toInt()[0]
            if val == 0:
                val = None
    return val


def forceString(val):
    if isinstance(val, QVariant):
        valType = val.type()
        if  valType == QVariant.Date:
            return formatDate(val.toDate())
        elif valType == QVariant.DateTime:
            return formatDateTime(val.toDateTime())
        elif valType == QVariant.Time:
            return formatTime(val.toTime())
        else:
            val = val.toString()
    if isinstance(val, QDate):
        return formatDate(val)
    if isinstance(val, QDateTime):
        return formatDateTime(val)
    if isinstance(val, QTime):
        return formatTime(val)
    if val is None:
        return u''
    return unicode(val)


def forceStringEx(val):
    return trim(forceString(val))


def forceDouble(val):
    if isinstance(val, QVariant):
        return val.toDouble()[0]
    else:
        return float(val)


def formatBool(val):
    if forceBool(val):
        return u'да'
    else:
        return u'нет'

def toVariant(v):
    if v is None:
        return QVariant()
    if isinstance(v, QVariant):
        return v
    if isinstance(v, datetime.date):
        return QVariant(QDate(v))
    if isinstance(v, datetime.datetime):
        return QVariant(QDateTime(v))
    if isinstance(v, datetime.time):
        return QVariant(QTime(v))
    return QVariant(v)
        

def anyToUnicode(s, encodingHint=None): # это для обработки ошибок.
    # в python2 сообщения об ошибках, особенно когда источником сообщения об ошибке
    # выступает OS (tcp/ip, serial etc.) могут быть в разных кодировках.
    # я не имею ни возможности ни желания проверить все варианты, поэтому делаю так
    if isinstance(s, unicode):
        return s
    if os.name == 'nt':
        encodingList = ( encodingHint,
                         locale.getpreferredencoding(), # по уставу
                         'utf8', # as fallback
                         'cp1251',
                         'cp866',
                       )
    else:
        encodingList = ( encodingHint,
                         locale.getpreferredencoding(), # по уставу
                         'utf8', # as fallback
                       )

    for encoding in encodingList:
        try:
            if encoding:
                return s.decode(encoding)
        except:
            pass
    return unicode(repr(s))

def trim(s):
    return u' '.join(unicode(s).split())

def nameCase(s):
    r = u''
    up = True
    for c in s:
        if c.isalpha():
            if up:
                r += c.upper()
                up = False
            else:
                r += c.lower()
        else:
            up = True
            r += c
    return r

def formatShortNameInt(lastName, firstName, patrName):
    return trim(lastName + ' ' +((firstName[:1]+'.') if firstName else '') + ((patrName[:1]+'.') if patrName else ''))


def formatShortName(lastName, firstName, patrName):
    lastName  = nameCase(forceStringEx(lastName))
    firstName = nameCase(forceStringEx(firstName))
    patrName  = nameCase(forceStringEx(patrName))
    return formatShortNameInt(lastName, firstName, patrName)


def formatSex(sex):
    sex = forceInt(sex)
    if sex == 1:
        return u'М'
    elif sex == 2:
        return u'Ж'
    else:
        return u''
