# -*- coding: utf-8 -*-

import SendKeys
import serial
import codecs
import time


import MySQLdb

import re

import sys
from PyQt4 import QtGui, QtCore
from SerialSettings import CSerialSettings
from Utils import *
from Preferences import CPreferences

from datetime import datetime

from Ui_InputDialog import Ui_InputDialog

class CMainApp(QtGui.QApplication):
    def __init__(self, args):
        QtGui.QApplication.__init__(self, args)
        self.preferences = CPreferences('settings.ini')
        self.preferences.load()
        self.logLevel = self.preferences.appPrefs.get('logLevel', 3)
        self.logDir = self.preferences.appPrefs.get('logDir', None)
        if not self.logDir:
            self.logDir = "."

class CInputDialog(QtGui.QDialog, Ui_InputDialog):
    def __init__(self, parent = None):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)
        self.lblStatus.setText(u'Работает')
        self.scannerTimer = None
        self.reconnectTimer = None
        self.reallyExit = False
        self.tray = QtGui.QSystemTrayIcon(self)
        self.tray.setIcon(QtGui.QIcon('icon/scanner.ico'))
        self.tray.activated.connect(self.activate)
        self.trayMenu = QtGui.QMenu(parent)
        self.actExit = self.trayMenu.addAction("Exit")
        self.actExit.triggered.connect(self.on_actExit)
        self.tray.setContextMenu(self.trayMenu)
        self.tray.show()
        self.tryOpenScanner()
        self.outString = forceString(QtGui.qApp.preferences.appPrefs.get('outString', ''))
        self.jobModify = forceString(QtGui.qApp.preferences.appPrefs.get('jobModify', ''))
        self.actionModify = forceString(QtGui.qApp.preferences.appPrefs.get('actionModify', ''))
        self.dbPrefs = {}
        self.db = None
        prefs = QtGui.qApp.preferences.appPrefs.get('db', {})
        self.dbPrefs['host'] = forceString(prefs.get('host', '127.0.0.1'))
        self.dbPrefs['port'] = forceInt(prefs.get('port', 3306))
        self.dbPrefs['user'] = forceString(prefs.get('user', 'dbuser'))
        self.dbPrefs['passwd'] = forceString(prefs.get('passwd', 'dbpassword'))
        self.dbPrefs['db'] = forceString(prefs.get('db', 's11'))
        self.dbPrefs['sql']= forceString(prefs.get('sql', '')).replace('\n', '')
        QtGui.qApp.preferences.appPrefs['db'] = self.dbPrefs
        self.reconnectTimer = QtCore.QTimer(self)
        self.connect(self.reconnectTimer, QtCore.SIGNAL('timeout()'), self.opendb)
        self.birthDateRe = re.compile('({birthDate([%dmY.\s-]*)})')
        if not self.opendb():
            self.reconnectTimer.start(10000)
        
    
    def opendb(self):
        result = True
        try:
            db = MySQLdb.connect(host=self.dbPrefs['host'], user=self.dbPrefs['user'], passwd=self.dbPrefs['passwd'], db=self.dbPrefs['db'], charset='utf8')
            db.autocommit(False)
            self.db = db.cursor()
            if self.reconnectTimer:
                self.reconnectTimer.stop()
                self.reconnectTimer = None
        except Exception as e:
            self.lblStatus.setText(u'Нет подключения к базе данных. %s'%str(e))
            #self.show()
            self.tray.showMessage(u'Нет подключения к базе данных.', '%s'%str(e))
            result = False
        self.work('274069')
        return result

    
    def tryOpenScanner(self):
        self.scanner = None
        try:
            scannerPrefs = QtGui.qApp.preferences.appPrefs.get('scanner', None)
            settings = CSerialSettings(scannerPrefs)
            QtGui.qApp.preferences.appPrefs['scanner'] = settings.asDict()
            self.scanner = serial.Serial(settings.port,
                                         baudrate=settings.baudrate,
                                         bytesize=8,
                                         parity=settings.parity,
                                         stopbits=settings.stopbits,
                                         xonxoff=settings.xonxoff,
                                         rtscts=settings.rtscts,
                                         timeout=0.1)
#            except IOError, e:
        except UnicodeError, e:
            self.lblStatus.setText(u'Ошибка подключения к сканеру. %s'%anyToUnicode(e.object))
            self.show()
            time.sleep(10)
            self.tryOpenScanner()
            return
        except Exception, e:
            self.lblStatus.setText(u'Ошибка подключения к сканеру. %s'%anyToUnicode(str(e)))
            self.show()
            time.sleep(10)
            self.tryOpenScanner()
            return
        if self.scannerTimer is None:
            self.scannerTimer = QtCore.QTimer(self)
            self.connect(self.scannerTimer, QtCore.SIGNAL('timeout()'), self.checkScanner)
        self.scannerTimer.start(250)
    
    def checkScanner(self):
        if self.scanner is not None:
            data = None
            try:
                if self.scanner.inWaiting():
                    data = self.scanner.readall()
            except:
                return
            if data is not None:
                self.work(data)
    
    def dbRequest(self, input):
        try:
            inputId = int(input)
        except:
            return
        #sql = """
        #    select action.id, action.*, client.*, jobticket.* from ActionProperty_Job_Ticket as apjt
        #        left join Job_Ticket as jobticket on apjt.value = jobticket.id
        #        left join ActionProperty as ap on ap.id = apjt.id
        #        left join Action as action on action.id = ap.action_id
        #        left join Event as event on event.id = action.event_id
        #        left join Client as client on client.id = event.client_id
        #    where
        #        jobticket.id = %i
        #"""%(inputId)
        sql = self.dbPrefs['sql']%inputId
        self.db.execute(sql)
        return self.db.fetchone()
        
    
    def work(self, data):
        try:
            row = self.dbRequest(data.strip())
            if len(row):
                if self.actionModify == 'setClosed':
                    for action in actList:
                        action.endDate = datetime.today()
                        action.status = 2
                    try:
                        id = int(row[0])
                        sql = "update Action set status=2, endDate=now() where Action.id = %i"%(id)
                        self.db.execute(sql)
                        row = row[1:]
                    except Exception, e:
                        self.log(anyToUnicode(str(e)))
                        return
                if self.jobModify == 'setRunning':
                    try:
                        id = int(row[0])
                        sql = "update JobTicket set status=1 where JobTicket.id = %i"%(id)
                        self.db.execute(sql)
                        row = row[1:]
                    except Exception, e:
                        self.log(anyToUnicode(str(e)))
                        return
                elif self.jobModify == 'setClosed':
                    try:
                        id = int(row[0])
                        sql = "update JobTicket set status=2, endDateTime=now() where JobTicket.id = %i"%(id)
                        self.db.execute(sql)
                        row = row[1:]
                    except Exception, e:
                        self.log(anyToUnicode(str(e)))
                        return
                if self.outString:
                    outStr = self.outString[:]
                    outStr%tuple(row)
                    SendKeys.SendKeys(outStr, pause=0.02, with_spaces=True)
                
            else:
                self.tray.showMessage(u'Ошибка', u'Пациент не найден')
        except Exception as e:
            log('error', '%s'%unicode(e), 1)
            self.tray.showMessage(u'Ошибка', u'Обработать не получилось')

    
    def activate(self, reason):
        if reason==2:
            self.show()
            
    
    def on_actExit(self):
        #QtGui.qApp.preferences.save()
        self.reallyExit = True
        self.close()
        exit()
    
    def closeEvent(self, event):
        if self.reallyExit:
            event.accept()
        else:
            self.edtInput.setFocus()
            self.hide()
            event.ignore()
    
    def accept(self):
        self.hide()

def log(title, message, level):
    app = QtGui.qApp
    if app.logLevel >= level:
        try:
            if not os.path.exists(app.logDir):
                os.makedirs(app.logDir)
            logFile = os.path.join(app.logDir, 'error.log')
            timeString = unicode(QtCore.QDateTime.currentDateTime().toString(QtCore.Qt.SystemLocaleDate))
            logString = u'%s %s: %s\n' % (timeString, title, message)
            file = codecs.open(logFile, mode='a', encoding=locale.getpreferredencoding())
            file.write(logString)
            file.close()
        except:
            pass

if __name__ == '__main__':
    QtGui.qApp = CMainApp(sys.argv)
    dialog = CInputDialog()
    QtGui.qApp.mainWindow = dialog
    QtGui.qApp.exec_()
    exit()
