# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2016-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
##
## Кнопка, скрывающая под собой список прикреплённых файлов и её popup
##
#############################################################################

from PyQt4 import QtGui
from PyQt4.QtCore import Qt, SIGNAL, QSize

from .AttachedFile         import CAttachedFilesModel
from .AttachFilesPopup     import CAttachFilesPopup
from .AttachFilesTable     import CAttachFilesTable
from .AttachFilesTableFlag import CAttachFilesTableFlag

from Users.Rights import urCanAttachFile


class CAttachButton(QtGui.QPushButton):
    u'Кнопка, скрывающая под собой список прикреплённых файлов'

    def __init__(self, parent, text=None):
        QtGui.QPushButton.__init__(self, parent)
        if text:
            self.setText(text)
        self.modelFiles = CAttachedFilesModel(self)
        self.forceShowDown = False
        self.connect(self, SIGNAL('pressed()'), self.onPress)
        self._isEnabled = True     # разрешён с точки зрения Qt
        self._isAccessible = True  # разрешён (есть право добавлять или список файлов не пуст)

        self.modelFiles.setInterface(QtGui.qApp.webDAVInterface)


    def setTable(self, tableName):
        self.modelFiles.setTable(tableName)


    def loadItems(self, masterId):
        self.modelFiles.loadItems(masterId)
        if self.modelFiles.rowCount() == 0 and not QtGui.qApp.userHasRight(urCanAttachFile):
            self._isAccessible = False
        else:
            self._isAccessible = True
        self.__setEnabled()


    def saveItems(self, masterId):
        self.modelFiles.saveItems(masterId)


    def setEnabled(self, val):
        self._isEnabled = val
        self.__setEnabled()


    def __setEnabled(self):
        interface = QtGui.qApp.webDAVInterface
        QtGui.QPushButton.setEnabled(self, self._isEnabled and self._isAccessible and bool(interface) )


    def sizeHint(self):
        self.ensurePolished()
        style = self.style()
        styleOptions = QtGui.QStyleOptionButton()
        styleOptions.initFrom(self)
        styleOptions.text = self.text()
        styleOptions.icon = self.icon()
        styleOptions.features |= styleOptions.HasMenu

        w = h = 0
        icon = self.icon()
        if icon and not icon.isNull():
            iconSize = self.iconSize()
            w += iconSize.width()
            h = max(h, iconSize.height())
        text = self.text() or 'XXXX'
        fontMetrics = self.fontMetrics()
        textSize = fontMetrics.size(Qt.TextShowMnemonic, text)
        w += textSize.width()
        h = max(h, textSize.height())
        # место для индикатора меню
        w += style.pixelMetric(QtGui.QStyle.PM_MenuButtonIndicator, styleOptions, self)
        sizeHint = style.sizeFromContents(QtGui.QStyle.CT_PushButton,  # ContentsType type,
                                          styleOptions,                # const QStyleOption * option,
                                          QSize(w, h),                 # const QSize & contentsSize,
                                          self                         # const QWidget * widget = 0
                                         )
        return sizeHint.expandedTo(QtGui.qApp.globalStrut())


    def paintEvent(self, event):
        style = self.style()
        styleOptions = QtGui.QStyleOptionButton()
        styleOptions.initFrom(self)
        if self.forceShowDown:
            styleOptions.state = style.State_Sunken
        if self.modelFiles.isNotEmpty():
            styleOptions.features |= styleOptions.HasMenu
        styleOptions.text = self.text()
        styleOptions.icon = self.icon()

        painter = QtGui.QPainter(self)
        style.drawControl(QtGui.QStyle.CE_PushButton, # ControlElement element
                          styleOptions,               # const QStyleOption * option
                          painter,                    # QPainter * painter
                          self                        # const QWidget * widget = 0
                         )


    def setAttachedFileItemList(self, attachedFileItemList):
        self.modelFiles.setAttachedFileItemList(attachedFileItemList)


    def showPopup(self):
        popup = CAttachFilesPopup(self)
        popup.setFlags(CAttachFilesTableFlag.canAnything)
        popup.setModel(self.modelFiles)
        popup.exec_()


    def onPress(self):
        self.forceShowDown = True
        try:
            if self.modelFiles.isNotEmpty():
                self.showPopup()
            else:
                if QtGui.qApp.userHasRight(urCanAttachFile):
                    self.selectAndUploadFiles()
        finally:
            self.forceShowDown = False
            self.update()


    def selectAndUploadFiles(self):
        ofr = CAttachFilesTable.selectFiles(self)
        if ofr:
            self.modelFiles.uploadFiles( [ unicode(fn) for fn in ofr] )

