# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2016-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
##
## Список прикреплённых файлов
##
#############################################################################

import os.path
import sip
from PyQt4 import QtGui
from PyQt4.QtCore import Qt, pyqtSignature, SIGNAL, QMetaObject, QSize, QUrl

from Users.Rights import ( urCanAttachFile,
                           urCanOpenAnyAttachedFile,
                           urCanOpenOwnAttachedFile,
                           urCanRenameAnyAttachedFile,
                           urCanRenameOwnAttachedFile,
                           urCanDeleteAnyAttachedFile,
                           urCanDeleteOwnAttachedFile,
                         )


from .AttachFilesTableFlag import CAttachFilesTableFlag


class CAttachFilesTable(QtGui.QTableView):
    u'Представление списка прикреплённых файлов'
    __pyqtSignals__ = ('iteractionDone()',
                      )


    def __init__(self, parent):
        QtGui.QTableView.__init__(self)
        self._flags = CAttachFilesTableFlag.canAnything
        self.setSelectionBehavior(self.SelectRows)
        self.setSelectionMode(self.SingleSelection)
        self.setSortingEnabled(True)
        self.resetSortIndicator()

        h = self.fontMetrics().height()*3//2
        self.verticalHeader().setDefaultSectionSize(h)
        self.verticalHeader().hide()
        self.horizontalHeader().setResizeMode(QtGui.QHeaderView.ResizeToContents)
        self.horizontalHeader().setStretchLastSection(True)

        self.actOpen = QtGui.QAction(u'&Открыть', self)
        self.actOpen.setObjectName('actOpen')

        self.actSave = QtGui.QAction(u'&Сохранить', self)
        self.actSave.setObjectName('actSave')

        self.actAdd = QtGui.QAction(u'&Добавить',  self)
        self.actAdd.setObjectName('actAdd')
        self.actRename = QtGui.QAction(u'&Переименовать', self)
        self.actRename.setObjectName('actRename')
        self.actDelete = QtGui.QAction(u'&Удалить',  self)
        self.actDelete.setObjectName('actDelete')

#        self.setContextMenuPolicy(Qt.DefaultContextMenu)

        self.mnuPopup = QtGui.QMenu(self)
        self.mnuPopup.setObjectName('mnuPopup')

        self.mnuPopup.addAction(self.actOpen)
        self.mnuPopup.addAction(self.actSave)
        self.mnuPopup.addSeparator()
        self.mnuPopup.addAction(self.actAdd)
        self.mnuPopup.addAction(self.actRename)
        self.mnuPopup.addAction(self.actDelete)
        self.mnuPopup.setDefaultAction(self.actOpen)

        QMetaObject.connectSlotsByName(self)


    def setFlags(self, flags):
        self._flags = flags


    def resetSortIndicator(self):
        self.sortByColumn(-1, Qt.AscendingOrder)


    def sizeHint(self):
        model = self.model()
        if not model:
            return QtGui.QTableView.sizeHint()
        else:
            self.resizeColumnsToContents()
            self.resizeRowsToContents()
            w = self.verticalHeader().width() + self.frameWidth()*2 + self.style().pixelMetric(QtGui.QStyle.PM_ScrollBarExtent)
            for i in xrange(model.columnCount()):
                self.resizeColumnToContents(0)
                w += self.columnWidth(i)
            h = self.horizontalHeader().height() + self.frameWidth()*2
            for i in xrange(model.rowCount()):
                h += self.rowHeight(i)
            return QSize(w, h)

#            hh           = self.horizontalHeader()
#            headerHeight = hh.height()
#            rowHeight    = self.verticalHeader().defaultSectionSize()
#            frameWidth   = self.frameWidth()
#            lineWidth    = self.lineWidth()
#            iconMetric   = self.style().pixelMetric(QtGui.QStyle.PM_SmallIconSize)
#            dataWidth    = sum( max(self.sizeHintForColumn(column) + iconMetric,
#                                    hh.sectionSizeHint(column)
#                                   )
#                                for column in xrange(model.columnCount())
#                              )
#            return QSize(  dataWidth
#                         + lineWidth*model.columnCount() + 2*frameWidth + 16
#                         + self.style().pixelMetric(QtGui.QStyle.PM_ScrollBarExtent),
#                           headerHeight
#                         + (max(iconMetric, rowHeight) + lineWidth)*model.rowCount()
#                         + 2*frameWidth
#                         + self.style().pixelMetric(QtGui.QStyle.PM_ScrollBarExtent),
#                        )

    def widgetForFileDialog(self):
        window = self.window()
        if window and window.windowType() & Qt.Popup != 0:
            return window.parentWidget()
        else:
            return self


    def getCurrentFileItem(self):
        modelIndex = self.currentIndex()
        if modelIndex.isValid():
            return self.model().items[modelIndex.row()]
        else:
            return None


    def userHasRights(self, fileItem, anyRight, ownRight):
        app = QtGui.qApp
        if app.userHasRight(anyRight):
            return True
        ownFile = bool(fileItem) and not fileItem.isLost and fileItem.authorId == app.userId
        return ownFile and app.userHasRight(ownRight)


    def canAdd(self):
        return bool(self._flags & CAttachFilesTableFlag.canAdd) and QtGui.qApp.userHasRight(urCanAttachFile)


    def canOpen(self, fileItem):
        return (     bool(self._flags & CAttachFilesTableFlag.canOpen)
                 and self.userHasRights(fileItem, urCanOpenAnyAttachedFile, urCanOpenOwnAttachedFile)
               )


    def canSave(self):
        return (     bool(self._flags & CAttachFilesTableFlag.canSave)
               )


    def canRename(self, fileItem):
        return (     bool(self._flags & CAttachFilesTableFlag.canRename)
                and self.userHasRights(fileItem, urCanRenameAnyAttachedFile, urCanRenameOwnAttachedFile)
               )


    def canDelete(self, fileItem):
        return (     bool(self._flags & CAttachFilesTableFlag.canDelete)
                 and self.userHasRights(fileItem, urCanDeleteAnyAttachedFile, urCanDeleteOwnAttachedFile)
               )


    def openAttachedFile(self, attachedFileItem):
        interface = self.model().interface
        url = interface.getUrl(attachedFileItem)
        QtGui.QDesktopServices.openUrl(QUrl(url))


    def saveAttachedFile(self, attachedFileItem):
        # в Windows при использовании Qt4.6 при открытии QFileDialog popup закрывается :(
        model = self.model()
        documentsDir = unicode(QtGui.QDesktopServices.storageLocation(QtGui.QDesktopServices.DocumentsLocation))
        localFile = QtGui.QFileDialog.getSaveFileName(self.widgetForFileDialog(),
                                                      u'Сохранить файл',
                                                      os.path.join(documentsDir, attachedFileItem.newName),
                                                      u'Любые файлы(*)')
        if localFile:
            interface = model.interface
            interface.downloadFile(attachedFileItem, unicode(localFile))


    @staticmethod
    def selectFiles(widget):
        documentsDir = QtGui.QDesktopServices.storageLocation(QtGui.QDesktopServices.DocumentsLocation)
        ofr = QtGui.QFileDialog.getOpenFileNames(widget,
                                 u'Выберите файлы',
                                 documentsDir or QtGui.qApp.getHomeDir(),
                                 ';;'.join((u'Документы(*.pdf *.odt *.ods *.doc *.xls *.docx *.xlsx)',
                                            u'Изображения(*.png *.tiff *.jpg *.jpeg *.gif *.bmp *.xpm)',
                                            u'Текстовые файлы(*.txt)',
                                            u'Архивы(*.zip *.7z *.Z *.gz *.bz *.xz *.arj *.rar)',
                                            u'Любые файлы(*)')))
        return ofr


    def selectAndUploadFiles(self):
        # в Windows при использовании Qt4.6 при открытии QFileDialog popup закрывается :(
        model = self.model()
        ofr = self.selectFiles(self.widgetForFileDialog())
        if ofr:
            model.uploadFiles( [ unicode(fn) for fn in ofr] )
            if not sip.isdeleted(self):
                self.resetSortIndicator()


    def renameFile(self, fileItem):
        newName, ok = QtGui.QInputDialog.getText(self, # QWidget * parent,
                                                 u'Переименовать ' + fileItem.newName, # const QString & title,
                                                 '',                                   # const QString & label,
                                                 QtGui.QLineEdit.Normal,               # QLineEdit::EchoMode mode = QLineEdit::Normal,
                                                 fileItem.newName                      #  const QString & text = QString()
                                                )
        if ok:
            newName = unicode(newName)
            if all(c not in newName for c in '*?:/\\'):
                row = self.model().indexOfItem(fileItem)
                self.model().renameFile(row, unicode(newName))
                self.resetSortIndicator()
            else:
                pass #!!!


    def deleteFile(self, fileItem):
        row = self.model().indexOfItem(fileItem)
        self.model().removeRow(row)


    def emitIteractionDone(self):
        QtGui.qApp.emit(SIGNAL('iteractionDone()'))


    def mouseDoubleClickEvent(self, event): # event: QMouseEvent
        if event.button() == Qt.LeftButton and event.modifiers() == Qt.NoModifier:
            fileItem = self.getCurrentFileItem()
            fileOk = bool(fileItem) and not fileItem.isLost
            if fileOk:
                if self.canOpen(fileItem):
                    self.openAttachedFile(fileItem)
                    self.emitIteractionDone()
                elif self.canSave():
                    self.saveAttachedFile(fileItem)
            event.accept()
        else:
            QtGui.QTableView.mouseDoubleClickEvent(self, event)


    def contextMenuEvent(self, event): # event: QContextMenuEvent
#        app = QtGui.qApp
        fileItem = self.getCurrentFileItem()
        fileOk = bool(fileItem) and not fileItem.isLost
        self.actAdd.setEnabled(self.canAdd())
        self.actOpen.setEnabled(fileOk and self.canOpen(fileItem))
        self.actSave.setEnabled(fileOk and self.canSave())
        self.actRename.setEnabled(fileOk and self.canRename(fileItem))
        self.actDelete.setEnabled(bool(fileItem) and self.canDelete(fileItem))
        self.mnuPopup.exec_(event.globalPos())
        event.accept()


    @pyqtSignature('')
    def on_actOpen_triggered(self):
        fileItem = self.getCurrentFileItem()
        fileOk = bool(fileItem) and not fileItem.isLost
        if fileOk and self.canOpen(fileItem):
            self.openAttachedFile(fileItem)
            self.emitIteractionDone()


    @pyqtSignature('')
    def on_actSave_triggered(self):
        fileItem = self.getCurrentFileItem()
        fileOk = bool(fileItem) and not fileItem.isLost
        if fileOk and self.canSave():
            self.saveAttachedFile(fileItem)


    @pyqtSignature('')
    def on_actAdd_triggered(self):
        if self.canAdd():
            self.selectAndUploadFiles()


    @pyqtSignature('')
    def on_actRename_triggered(self):
        fileItem = self.getCurrentFileItem()
        fileOk = bool(fileItem) and not fileItem.isLost
        if fileOk and self.canRename(fileItem):
           self.renameFile(fileItem)


    @pyqtSignature('')
    def on_actDelete_triggered(self):
        fileItem = self.getCurrentFileItem()
        if fileItem and self.canDelete(fileItem):
           self.deleteFile(fileItem)
