# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2016-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################


from PyQt4 import QtGui

from library.Utils     import forceString
from library.exception import CException


class CIdentificationException(CException):
    pass


def getIdentification(tableName, id, urn, raiseIfNonFound=True):
    u'получить идентификатор записи таблицы tableName c заданным id в справочнике с заданным urn'
    value, version = getIdentificationEx(tableName, id, urn, raiseIfNonFound)
    return value


def getIdentificationEx(tableName, id, urn, raiseIfNonFound=True):
    u'получить идентификатор и версию записи таблицы tableName c заданным id в справочнике с заданным urn'
    db = QtGui.qApp.db
    tableIdentification = db.table(tableName + '_Identification')
    tableAccountingSystem = db.table('rbAccountingSystem')
    table = tableIdentification.innerJoin(tableAccountingSystem,
                                          tableAccountingSystem['id'].eq(tableIdentification['system_id'])
                                         )
    recordList = db.getRecordList(table,
                                  [ tableIdentification['value'],
                                    tableAccountingSystem['version']
                                  ],
                                  [ tableIdentification['master_id'].eq(id),
                                    tableIdentification['deleted'].eq(0),
                                    tableAccountingSystem['urn'].eq(urn)

                                  ],
                                  [ tableIdentification['id'].name() + ' DESC' ],
                                  1
                                 )
    if recordList:
        record = recordList[0]
        value   = forceString(record.value('value'))
        version = forceString(record.value('version'))
        return value, version
    if raiseIfNonFound:
        raise CIdentificationException(u'Для %s.id=%s не задан код в системе %s' % (tableName, id, urn))
    return None, None


def findByIdentification(tableName, urn, value, raiseIfNonFound=True):
    u'получить id записи таблицы tableName c заданным кодом в справочнике с заданным urn'
    db = QtGui.qApp.db
    tableIdentification = db.table(tableName + '_Identification')
    tableAccountingSystem = db.table('rbAccountingSystem')
    table = tableIdentification.innerJoin(tableAccountingSystem,
                                          tableAccountingSystem['id'].eq(tableIdentification['system_id'])
                                         )
    idList = db.getIdList(table,
                          tableIdentification['master_id'].name(),
                          [ tableIdentification['value'].eq(value),
                            tableIdentification['deleted'].eq(0),
                            tableAccountingSystem['urn'].eq(urn)
                          ],
                          tableIdentification['id'].name() + ' DESC',
                          1
                         )
    if idList:
        return idList[0]

    if raiseIfNonFound:
        raise CIdentificationException(u'Для %s в системе %s не найден код %s' % (tableName, urn, value))

    return None


def addIdentification(tableName, id, urn, value):
    db = QtGui.qApp.db
    tableIdentification = db.table(tableName + '_Identification')
    systemId = db.translate('rbAccountingSystem', 'urn', urn, 'id')
    if not systemId:
        raise Exception('rbAccountingSystem with urn=%r not found' % urn)

    record = tableIdentification.newRecord()
    record.setValue('master_id', id)
    record.setValue('system_id', systemId)
    record.setValue('value', value)
    db.insertRecord(tableIdentification, record)
