# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2016-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from library.InDocTable import CInDocTableModel, CDateInDocTableCol, CInDocTableCol, CRBInDocTableCol
from library.database   import decorateString
from library.Utils      import forceRef, forceStringEx

__all__ = ( 'CIdentificationModel',
            'checkIdentification',
          )


class CIdentificationModel(CInDocTableModel):
    def __init__(self, parent, tableName, key):
        CInDocTableModel.__init__(self, tableName, 'id', 'master_id', parent)

        filter = u'''FIND_IN_SET(%s, REPLACE(domain, ' ', ''))>0 OR domain='' ''' % decorateString(key)
        self.addCol(CRBInDocTableCol(  u'Справочник',
                                       'system_id',
                                       40,
                                       'rbAccountingSystem',
                                       addNone=False,
                                       filter=filter
                                    )
                   )
        self.addCol(CInDocTableCol(    u'Идентификатор', 'value',       30))
        self.addCol(CDateInDocTableCol(u'Дата подтверждения', 'checkDate', 20))


def checkIdentification(dialog, tableWidget):
    model = tableWidget.model()
    for row, item in enumerate(model.items()):
        systemId = forceRef(item.value('system_id'))
        value    = forceStringEx(item.value('value'))
        if not (systemId or dialog.checkInputMessage(u'справочник',  False, tableWidget, row, 0)):
            return False
        if not (value or dialog.checkInputMessage(u'идентификатор', False, tableWidget, row, 1)):
            return False
    return True
