# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2016-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui
from PyQt4.QtCore import Qt, QRegExp


__all__ = [ 'CLineEditWithRegExpValidator',
            'prepareRegExp',
            'checkRegExp'
          ]

def prepareRegExp(regExp):
    return QRegExp(regExp, Qt.CaseSensitive, QRegExp.RegExp2)


def checkRegExp(regExp):
    return prepareRegExp(regExp).isValid()


class CLineEditWithRegExpValidator(QtGui.QLineEdit):

    def __init__(self, parent = None):
        QtGui.QLineEdit.__init__(self, parent)
        self.validator = None


    def setRegExp(self, regExp):
        if not regExp:
            self.validator = None
        else:
            rx = prepareRegExp(regExp)
            if not rx.isValid():
                rx = prepareRegExp('')
            self.validator = QtGui.QRegExpValidator(rx, self)
        self.setValidator(self.validator)
        self.update()


    def paintEvent(self, event):
        palette = QtGui.QPalette()
        if self.isEnabled() and not self.hasAcceptableInput():
            palette.setColor(palette.Base,      QtGui.QColor(255, 192, 192))
            palette.setColor(palette.Highlight, Qt.darkRed)
        self.setPalette(palette)
        QtGui.QLineEdit.paintEvent(self, event)

