# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui
from PyQt4.QtCore import SIGNAL

from library.crbcombobox import CRBComboBox
from library.MES.MESComboBoxPopup import CMESComboBoxPopup

__all__ = ('CMESComboBox',
          )


class CMESComboBox(CRBComboBox):
    __pyqtSignals__ = ('textChanged(QString)',
                       'textEdited(QString)'
                      )

    def __init__(self, parent = None):
        CRBComboBox.__init__(self, parent)
        self._popup=None
        self.mesId=None
        self.clientSex = 0
        self.baseDate = None
        self.clientBirthDate = None
        self.clientAge = None
        self.clientAgePrevYearEnd = None
        self.clientAgeCurrYearEnd = None
        self.eventProfileId = None
        self.eventBegDate = None
        self.mesCodeTemplate = ''
        self.mesServiceTemplate = ''
        self.mesNameTemplate = ''
        self.specialityId = None
        self.MKB = ''
        self.MKBEx = ''
        self._tableName = 'mes.MES'
        self._addNone = True
        self._customFilter = None
        CRBComboBox.setTable(self, self._tableName, self._addNone, self.compileFilter(), self._order)
        self.setShowFields(CRBComboBox.showCodeAndName)


    def setEventBegDate(self, date):
        self.eventBegDate = date


    def setClientSex(self, clientSex):
        self.clientSex = clientSex


    def setClientAge(self, baseDate, clientBirthDate, clientAge, clientAgePrevYearEnd, clientAgeCurrYearEnd):
        self.baseDate = baseDate
        self.clientBirthDate = clientBirthDate
        self.clientAge = clientAge
        self.clientAgePrevYearEnd = clientAgePrevYearEnd
        self.clientAgeCurrYearEnd = clientAgeCurrYearEnd


    def getClientAgeSex(self):
        return (self.baseDate, self.clientBirthDate, self.clientAge, self.clientAgePrevYearEnd, self.clientAgeCurrYearEnd, self.clientSex)


    def setEventProfile(self, eventProfileId):
        self.eventProfileId = eventProfileId


    def setMESCodeTemplate(self, mesCodeTemplate):
        self.mesCodeTemplate = mesCodeTemplate


    def setMESNameTemplate(self, mesNameTemplate):
        self.mesNameTemplate = mesNameTemplate


    def setMESServiceTemplate(self, mesServiceTemplate):
        self.mesServiceTemplate = mesServiceTemplate


    def setSpeciality(self, specialityId):
        self.specialityId = specialityId


    def setMKB(self, MKB):
        self.MKB = MKB


    def setMKBEx(self, MKBEx):
        self.MKBEx = MKBEx


    def showPopup(self):
        if not self._popup:
            self._popup = CMESComboBoxPopup(self)
            self.connect(self._popup, SIGNAL('MESSelected(int)'), self.setValue)
        pos = self.mapToGlobal(self.rect().bottomLeft())
        size = self._popup.sizeHint()
        width= max(size.width(), self.width())
        size.setWidth(width)
        screen = QtGui.QApplication.desktop().availableGeometry(pos)
        pos.setX( max(min(pos.x(), screen.right()-size.width()), screen.left()) )
        pos.setY( max(min(pos.y(), screen.bottom()-size.height()), screen.top()) )
        self._popup.move(pos)
        self._popup.resize(size)
        self._popup.show()
        self._popup.setup(self.baseDate, self.clientSex, self.clientBirthDate, self.clientAge,
                          self.clientAgePrevYearEnd, self.clientAgeCurrYearEnd, self.eventProfileId,
                          self.mesCodeTemplate, self.mesNameTemplate, self.mesServiceTemplate,
                          self.specialityId, self.MKB, self.MKBEx, self.value(), self.eventBegDate)


#    def setValue(self, mesId):
#        self.mesId = mesId


#    def value(self):
#        return self.mesId


#    @staticmethod
#    def getText(mesId):
#        if mesId:
#            db = QtGui.qApp.db
#            table = db.table('mes.MES')
#            record = db.getRecordEx(table, ['code','name'], [table['id'].eq(mesId), table['deleted'].eq(0)])
#            if record:
#                code = forceString(record.value('code'))
#                name = forceString(record.value('name'))
#                text = code + ' | ' + name
#            else:
#                text = '{%s}' % mesId
#        else:
#            text = ''
#        return text


#    def updateText(self):
#        self.setEditText(self.getText(self.mesId))


    def compileFilter(self):
        db = QtGui.qApp.db
        tableMES = db.table('mes.MES')
        cond  = [tableMES['deleted'].eq(0)]
        return db.joinAnd(cond)


    def updateFilter(self):
        v = self.value()
        CRBComboBox.setTable(self, self._tableName, self._addNone, self.compileFilter(), self._order)
        self.setValue(v)


    def lookup(self):
        i, self._searchString = self._model.searchCodeEx(self._searchString)
        if i>=0 and i!=self.currentIndex():
            self.setCurrentIndex(i)
            rowIndex = self.currentIndex()
            self.mesId = self._model.getId(rowIndex)

