# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui
from PyQt4.QtCore import Qt, pyqtSignature, SIGNAL, QDate, QEvent

from library.TableModel import CRefBookCol, CTableModel, CTextCol
from library.Utils import calcAgeTuple, getPref, setPref

from Ui_MESComboBoxPopup import Ui_MESComboBoxPopup


__all__ = ('CMESComboBoxPopup',
          )


class CMESComboBoxPopup(QtGui.QFrame, Ui_MESComboBoxPopup):
    __pyqtSignals__ = ('MESSelected(int)',
                      )

    def __init__(self, parent = None):
        QtGui.QFrame.__init__(self, parent, Qt.Popup)
        self.setFrameShape(QtGui.QFrame.StyledPanel)
        self.setAttribute(Qt.WA_WindowPropagation)
        self.tableModel = CMESTableModel(self)
        self.tableSelectionModel = QtGui.QItemSelectionModel(self.tableModel, self)
        self.tableSelectionModel.setObjectName('tableSelectionModel')
        self.setupUi(self)
        self.tblMES.setModel(self.tableModel)
        self.tblMES.setSelectionModel(self.tableSelectionModel)
        self.buttonBox.button(QtGui.QDialogButtonBox.Apply).setDefault(True)
#       к сожалению в данном случае setDefault обеспечивает рамочку вокруг кнопочки
#       но enter не работает...
        self.buttonBox.button(QtGui.QDialogButtonBox.Apply).setShortcut(Qt.Key_Return)
        self.eventBegDate = None
        self.clientSex = 0
        self.baseDate = None
        self.clientBirthDate = None
        self.clientAge = None
        self.clientAgePrevYearEnd = None
        self.clientAgeCurrYearEnd = None
        self.eventProfileId = None
        self.specialityId = None
        self.MKB = ''
        self.MKBEx = ''
        self.mesId = None
        self.tblMES.installEventFilter(self)
        preferences = getPref(QtGui.qApp.preferences.windowPrefs, 'CMESComboBoxPopup', {})
        self.tblMES.loadPreferences(preferences)
        self.on_buttonBox_reset()


    def mousePressEvent(self, event):
        parent = self.parentWidget()
        if parent!=None:
            opt=QtGui.QStyleOptionComboBox()
            opt.init(parent)
            arrowRect = parent.style().subControlRect(
                QtGui.QStyle.CC_ComboBox, opt, QtGui.QStyle.SC_ComboBoxArrow, parent)
            arrowRect.moveTo(parent.mapToGlobal(arrowRect.topLeft()))
            if (arrowRect.contains(event.globalPos()) or self.rect().contains(event.pos())):
                self.setAttribute(Qt.WA_NoMouseReplay)
        QtGui.QFrame.mousePressEvent(self, event)


    def closeEvent(self, event):
        preferences = self.tblMES.savePreferences()
        setPref(QtGui.qApp.preferences.windowPrefs, 'CMESComboBoxPopup', preferences)
        QtGui.QFrame.closeEvent(self, event)


    def eventFilter(self, watched, event):
        if watched == self.tblMES:
            if event.type() == QEvent.KeyPress and event.key() in (Qt.Key_Return, Qt.Key_Enter, Qt.Key_Select):
                event.accept()
                index = self.tblMES.currentIndex()
                self.tblMES.emit(SIGNAL('doubleClicked(QModelIndex)'), index)
                return True
        return QtGui.QFrame.eventFilter(self, watched, event)


    @pyqtSignature('QAbstractButton*')
    def on_buttonBox_clicked(self, button):
        buttonCode = self.buttonBox.standardButton(button)
        if buttonCode == QtGui.QDialogButtonBox.Apply:
            self.on_buttonBox_apply()
        elif buttonCode == QtGui.QDialogButtonBox.Reset:
            self.on_buttonBox_reset()


    def on_buttonBox_reset(self):
        self.chkSex.setChecked(True)
        self.chkAge.setChecked(True)
        self.chkEventProfile.setChecked(True)
        self.chkMesCodeTemplate.setChecked(True)
        self.chkMesNameTemplate.setChecked(True)
        self.chkSpeciality.setChecked(True)
        self.chkActive.setChecked(True)
        self.chkMesServices.setChecked(True)
        self.cmbMKB.setCurrentIndex(2)
        self.cmbMKBEx.setCurrentIndex(0)


    def on_buttonBox_apply(self):
        useSex        = self.chkSex.isChecked()
        useAge        = self.chkAge.isChecked()
        useProfile    = self.chkEventProfile.isChecked()
        useMesCode  = self.chkMesCodeTemplate.isChecked()
        useMesName = self.chkMesNameTemplate.isChecked()
        useSpeciality   = self.chkSpeciality.isChecked()
        useMesServices = self.chkMesServices.isChecked()
        useActivity   = self.chkActive.isChecked()
        mkbCond    = self.cmbMKB.currentIndex()
        mkbCondEx  = self.cmbMKBEx.currentIndex()
        idList = self.getMesIdList(useSex, useAge, useProfile, useMesCode, useMesName, useMesServices, useSpeciality, useActivity, mkbCond, mkbCondEx)
        self.setMESIdList(idList)


    def setMESIdList(self, idList):
        if idList:
            self.tblMES.setIdList(idList, self.mesId)
            self.tabWidget.setCurrentIndex(0)
            self.tabWidget.setTabEnabled(0, True)
            self.tblMES.setFocus(Qt.OtherFocusReason)
        else:
            self.tabWidget.setCurrentIndex(1)
            self.tabWidget.setTabEnabled(0, False)
            self.chkSex.setFocus(Qt.OtherFocusReason)


    def getMesIdList(self, useSex, useAge, useProfile, useMesCode, useMesName, useMesServices, useSpeciality, useActivity, mkbCond, mkbCondEx):
        db = QtGui.qApp.db
        tableMES = db.table('mes.MES')
        tableMESService = db.table('mes.MES_service')
        tableMESRBService = db.table('mes.mrbService')
        queryTable = tableMES
        cond  = []
        if self.eventProfileId and useProfile:
            tableMESGroup = db.table('mes.mrbMESGroup')
            tableEventProfile = db.table('rbEventProfile')
            queryTable = queryTable.leftJoin(tableMESGroup, tableMESGroup['id'].eq(tableMES['group_id']))
            queryTable = queryTable.leftJoin(tableEventProfile, tableEventProfile['regionalCode'].eq(tableMESGroup['code']))
            cond.append(db.joinOr([tableEventProfile['id'].eq(self.eventProfileId), tableMESGroup['id'].isNull(), '''TRIM(mes.mrbMESGroup.code) = '' ''']))
        if self.mesCodeTemplate and useMesCode:
            cond.append(tableMES['code'].regexp(self.mesCodeTemplate))
        if self.mesNameTemplate and useMesName:
            cond.append(tableMES['name'].regexp(self.mesNameTemplate))
        if self.eventBegDate:
            cond.append(db.joinOr([tableMES['endDate'].isNull(), tableMES['endDate'].ge(self.eventBegDate)]))
        if (self.clientSex and useSex) or (self.clientAge and useAge):
            tableMESLimitedBySexAge = db.table('mes.MES_limitedBySexAge')
            queryTable = queryTable.leftJoin(tableMESLimitedBySexAge, tableMESLimitedBySexAge['master_id'].eq(tableMES['id']))
            sexStr = ''
            ageStr = ''
            if self.clientSex and useSex:
                sexStr = u''' AND mes.MES_limitedBySexAge.sex = %d'''%(self.clientSex)
            if self.clientAge and useAge:
                clientAge = self.clientAge[3]
                clientAgeMonths = self.clientAge[2]
                if clientAge == 0:
                    clientAge = round(clientAgeMonths / 12.0, 2)
                if self.clientAgePrevYearEnd is None:
                    self.clientAgePrevYearEnd = self.clientAgeCurrYearEnd
                    if self.baseDate and self.clientBirthDate:
                        self.clientAgeCurrYearEnd = calcAgeTuple(self.clientBirthDate, QDate(self.baseDate.year(), 12, 31))
                clientAgeCurrYearEnd = self.clientAgeCurrYearEnd[3]
                clientAgeMonthsCurrYear = self.clientAgeCurrYearEnd[2]
                if clientAgeCurrYearEnd == 0:
                    clientAgeCurrYearEnd = round(clientAgeMonthsCurrYear / 12.0, 2)
                clientAgePrevYearEnd = self.clientAgePrevYearEnd[3]
                clientAgeMonthsPrevYear = self.clientAgePrevYearEnd[2]
                if clientAgePrevYearEnd == 0:
                    clientAgePrevYearEnd = round(clientAgeMonthsPrevYear / 12.0, 2)
                ageStr = u''' AND mes.MES_limitedBySexAge.minimumAge <= (
                IF(mes.MES_limitedBySexAge.begAgeUnit = 1,
                IF(mes.MES_limitedBySexAge.controlPeriod = 1, %s, IF(mes.MES_limitedBySexAge.controlPeriod = 2, %s, %s)),
                IF(mes.MES_limitedBySexAge.begAgeUnit = 2,
                IF(mes.MES_limitedBySexAge.controlPeriod = 1, %s, IF(mes.MES_limitedBySexAge.controlPeriod = 2, %s, %s)),
                IF(mes.MES_limitedBySexAge.begAgeUnit = 3,
                IF(mes.MES_limitedBySexAge.controlPeriod = 1, %s, IF(mes.MES_limitedBySexAge.controlPeriod = 2, %s, %s)),
                IF(mes.MES_limitedBySexAge.begAgeUnit = 4,
                IF(mes.MES_limitedBySexAge.controlPeriod = 1, %s, IF(mes.MES_limitedBySexAge.controlPeriod = 2, %s, %s)),
                IF(mes.MES_limitedBySexAge.controlPeriod = 1, %s, IF(mes.MES_limitedBySexAge.controlPeriod = 2, %s, %s)))))))
                AND mes.MES_limitedBySexAge.maximumAge >= (
                IF(mes.MES_limitedBySexAge.endAgeUnit = 1,
                IF(mes.MES_limitedBySexAge.controlPeriod = 1, %s, IF(mes.MES_limitedBySexAge.controlPeriod = 2, %s, %s)),
                IF(mes.MES_limitedBySexAge.endAgeUnit = 2,
                IF(mes.MES_limitedBySexAge.controlPeriod = 1, %s, IF(mes.MES_limitedBySexAge.controlPeriod = 2, %s, %s)),
                IF(mes.MES_limitedBySexAge.endAgeUnit = 3,
                IF(mes.MES_limitedBySexAge.controlPeriod = 1, %s, IF(mes.MES_limitedBySexAge.controlPeriod = 2, %s, %s)),
                IF(mes.MES_limitedBySexAge.endAgeUnit = 4,
                IF(mes.MES_limitedBySexAge.controlPeriod = 1, %s, IF(mes.MES_limitedBySexAge.controlPeriod = 2, %s, %s)),
                IF(mes.MES_limitedBySexAge.controlPeriod = 1, %s, IF(mes.MES_limitedBySexAge.controlPeriod = 2, %s, %s))
                )))))
                '''%(u'%.2f'%(self.clientAgeCurrYearEnd[0]), u'%.2f'%(self.clientAgePrevYearEnd[0]), u'%.2f'%(self.clientAge[0]),
                u'%.2f'%(self.clientAgeCurrYearEnd[1]), u'%.2f'%(self.clientAgePrevYearEnd[1]), u'%.2f'%(self.clientAge[1]),
                u'%.2f'%(self.clientAgeCurrYearEnd[2]),u'%.2f'%(self.clientAgePrevYearEnd[2]),u'%.2f'%(self.clientAge[2]),
                u'%.2f'%(self.clientAgeCurrYearEnd[3]), u'%.2f'%(self.clientAgePrevYearEnd[3]), u'%.2f'%(self.clientAge[3]),
                u'%.2f'%(clientAgeCurrYearEnd), u'%.2f'%(clientAgePrevYearEnd), u'%.2f'%(clientAge),
                u'%.2f'%(self.clientAgeCurrYearEnd[0]), u'%.2f'%(self.clientAgePrevYearEnd[0]), u'%.2f'%(self.clientAge[0]),
                u'%.2f'%(self.clientAgeCurrYearEnd[1]), u'%.2f'%(self.clientAgePrevYearEnd[1]), u'%.2f'%(self.clientAge[1]),
                u'%.2f'%(self.clientAgeCurrYearEnd[2]),u'%.2f'%(self.clientAgePrevYearEnd[2]),u'%.2f'%(self.clientAge[2]),
                u'%.2f'%(self.clientAgeCurrYearEnd[3]), u'%.2f'%(self.clientAgePrevYearEnd[3]), u'%.2f'%(self.clientAge[3]),
                u'%.2f'%(clientAgeCurrYearEnd), u'%.2f'%(clientAgePrevYearEnd), u'%.2f'%(clientAge)
                )
            sexAgeStr = u'''NOT EXISTS(SELECT MES_limitedBySexAge.id FROM mes.MES_limitedBySexAge AS MES_limitedBySexAge
                  WHERE mes.MES.`id` = MES_limitedBySexAge.`master_id` AND MES_limitedBySexAge.`master_id` IS NOT NULL AND MES_limitedBySexAge.deleted = 0
                  LIMIT 1) OR (mes.MES_limitedBySexAge.deleted = 0%s%s)'''%(sexStr, ageStr)
            cond.append(sexAgeStr)
        if self.specialityId and useSpeciality:
            regionalCode = db.translate('rbSpeciality', 'id', self.specialityId, 'regionalCode')
            tableMESVisit      = db.table('mes.MES_visit')
            tableMESSpeciality = db.table('mes.mrbSpeciality')
            tableMESVisitType  = db.table('mes.mrbVisitType')
            subTable = tableMESVisit.leftJoin(tableMESVisitType,  tableMESVisitType['id'].eq(tableMESVisit['visitType_id']))
            subTable = subTable.leftJoin(tableMESSpeciality, tableMESSpeciality['id'].eq(tableMESVisit['speciality_id']))
            tableMESVisitType
            subCond  = [tableMESVisit['master_id'].eq(tableMES['id']),
                        tableMESVisit['deleted'].eq(0),
#                        tableMESVisitType['code'].eq(u'Л'),
                        tableMESSpeciality['regionalCode'].eq(regionalCode)
                       ]
            cond.append(db.joinOr([db.existsStmt(subTable, subCond),
                                   'NOT '+db.existsStmt(tableMESVisit, tableMESVisit['master_id'].eq(tableMES['id']))
                                  ]))
        if self.mesServiceTemplate and useMesServices:
            queryTable = queryTable.innerJoin(tableMESService, tableMESService['master_id'].eq(tableMES['id']))
            queryTable = queryTable.innerJoin(tableMESRBService, tableMESRBService['id'].eq(tableMESService['service_id']))
            joinOr = []
            for mesService in self.mesServiceTemplate:
                joinOr.append(u'''mes.mrbService.code LIKE '%s' '''%(mesService))
            cond.append(db.joinOr(joinOr))
            cond.append(tableMESService['deleted'].eq(0))
            cond.append(tableMESRBService['deleted'].eq(0))
        if self.MKB and mkbCond:
            tableMESMkb  = db.table('mes.MES_mkb')
            if mkbCond == 2: # строгое соответствие
                subCond = tableMESMkb['mkb'].inlist([#self.MKB[:3],
                                                     #self.MKB[:5],
#                                                     MKBwithoutSubclassification(self.MKB),
                                                     self.MKB,
                                                    ]
                                                   )
            else: # по рубрике
                subCond  = tableMESMkb['mkb'].like(self.MKB[:3]+'%')
            subCond  = [ tableMESMkb['master_id'].eq(tableMES['id']),
                         tableMESMkb['deleted'].eq(0),
                         subCond
                       ]
            cond.append(db.existsStmt(tableMESMkb, subCond))
        if self.MKBEx and mkbCondEx:
            tableMESMkbEx  = db.table('mes.MES_mkb').alias('MES_mkbEx')
            if mkbCondEx == 2: # строгое соответствие
                subCond = tableMESMkbEx['mkbEx'].inlist([self.MKBEx])
            else: # по рубрике
                subCond  = tableMESMkbEx['mkbEx'].like(self.MKBEx[:3]+'%')
            subCond  = [ tableMESMkbEx['master_id'].eq(tableMES['id']),
                         tableMESMkbEx['deleted'].eq(0),
                         subCond
                       ]
            cond.append(db.existsStmt(tableMESMkbEx, subCond))

        if useActivity:
            cond.append(tableMES['active'].eq(1))
        cond.append(tableMES['deleted'].eq(0))
        idList = db.getDistinctIdList(queryTable, tableMES['id'].name(),
                              where=cond,
                              order='mes.MES.code, mes.MES.id',
                              #limit=1000
                              )
        return idList


#    def mouseReleaseEvent(self, event):
##        self.emit(SIGNAL('resetButton()'))
#        self.parent().mouseReleaseEvent(event)
#        pass

#    def setDate(self, date):
#        self.tableModel.date = date


    def setup(self, baseDate, clientSex, clientBirthDate, clientAge, clientAgePrevYearEnd, clientAgeCurrYearEnd, eventProfileId, mesCodeTemplate, mesNameTemplate, mesServiceTemplate, specialityId, MKB, MKBEx, mesId, eventBegDate):
        self.eventBegDate = eventBegDate
        self.clientSex = clientSex
        self.baseDate = baseDate
        self.clientBirthDate = clientBirthDate
        self.clientAge = clientAge
        self.clientAgePrevYearEnd = clientAgePrevYearEnd
        self.clientAgeCurrYearEnd = clientAgeCurrYearEnd
        self.eventProfileId = eventProfileId
        self.mesCodeTemplate = mesCodeTemplate
        self.mesNameTemplate = mesNameTemplate
        self.mesServiceTemplate = mesServiceTemplate
        self.specialityId = specialityId
        self.MKB = MKB
        self.MKBEx = MKBEx
        self.mesId = mesId
        self.chkMesCodeTemplate.setChecked(bool(mesCodeTemplate))
        self.chkMesNameTemplate.setChecked(bool(mesNameTemplate))
        self.on_buttonBox_apply()


    @pyqtSignature('QModelIndex')
    def on_tblMES_doubleClicked(self, index):
        if index.isValid():
            if (Qt.ItemIsEnabled & self.tableModel.flags(index)):
                mesId = self.tblMES.currentItemId()
                self.mesId = mesId
                self.emit(SIGNAL('MESSelected(int)'), mesId)
                self.close()


class CMESTableModel(CTableModel):
    def __init__(self, parent):
        CTableModel.__init__(self, parent)
        self.addColumn(CRefBookCol(u'Группа',       ['group_id'],  'mes.mrbMESGroup', 10))
        self.addColumn(CTextCol(u'Код',             ['code'],  20))
        self.addColumn(CTextCol(u'Наименование',    ['name'],  40))
        self.addColumn(CTextCol(u'Описание',        ['descr'], 40))
        self.addColumn(CTextCol(u'Модель пациента', ['patientModel'], 20))
        self.setTable('mes.MES')
        self.date = QDate.currentDate()


    def flags(self, index):
        return Qt.ItemIsEnabled|Qt.ItemIsSelectable
#        row = index.row()
#        record = self.getRecordByRow(row)

#        enabled = not forceBool(record.value('disabledForPrescription'))
#        if enabled:
#            if self.date:
#                regDate = forceDate(record.value('regDate'))
#                annDate = forceDate(record.value('annDate'))
#                enabled = (not regDate or regDate<=self.date) and (not annDate or self.date<annDate)
#        if enabled:
#            return Qt.ItemIsEnabled|Qt.ItemIsSelectable
#        else:
#            return Qt.ItemIsSelectable
