# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4.QtCore           import QVariant

from library.CustomComboBoxLike import CCustomComboBoxLike
from library.TNMS.TNMSPopup import CTNMSPopup
from library.InDocTable     import CInDocTableCol
from library.Utils          import forceString, toVariant

__all__ = ( 'CTNMSComboBox',
            'CTNMSCol',
          )


class CTNMSComboBox(CCustomComboBoxLike):
    def __init__(self, parent = None):
        CCustomComboBoxLike.__init__(self, parent)


    def setValue(self, tnmsString):
        CCustomComboBoxLike.setValue(self, convertTNMSStringToDict(unicode(tnmsString)))


    def valueAsString(self, value):
        return convertTNMSDictToDigest(value)


    def getValue(self):
        return convertTNMSDictToString(CCustomComboBoxLike.getValue(self))


    def createPopup(self):
        return CTNMSPopup(self)


    def setValueToPopup(self):
        self._popup.setValue(CCustomComboBoxLike.getValue(self))


    def updateValueFromPopup(self):
        CCustomComboBoxLike.setValue(self, self._popup.getValue())


def convertTNMSStringToDict(s):
    result = {}
    mode = 0 # 0: initial, 1: prefix entered, 2: arg code entered
    prefix = ''
    param = ''
    value = ''

    for c in s:
        if mode == 0:
            if c in 'cp':
                prefix = c
                mode = 1
                continue
            elif c.isspace():
                mode = 0
            prefix = ''
            mode = 1
        if mode == 1:
            if c in 'TNMS':
                mode = 2
                param = c
                value = ''
                continue
            else:
                mode = 3
        if mode == 2:
            if c.isspace():
                result[prefix+param] = value
                mode = 0
            else:
                value += c

        if mode == 3:
           if c.isspace():
               mode = 0
               continue

    if mode == 2:
        result[prefix+param] = value
    return result


def convertTNMSDictToString(d):
    l = []
#    digest = {}

    for prefix in 'cp':
        for param in 'TNMS':
            key = prefix + param
            value = d.get(key,'x')
            l.append(key+value)
#            if value != 'x':
#                digest[param] = value
#    l.append('|')
#    for param in 'NNMS':
#        value = digest.get(param,'x')
#        l.append(param+value)

    return ' '.join(l)


def convertTNMSDictToDigest(d):
    parts = {}
    for param in 'TNMS':
        for prefix in 'cp':
            key = prefix + param
            value = d.get(key,'x')
            if value != 'x':
                parts.setdefault(prefix, []).append(param+value)
    return ' '.join('%s(%s)' % (param, ' '.join(parts[param]))
                    for param in sorted(parts.keys())
                   )


class CTNMSCol(CInDocTableCol):
    def __init__(self, title, fieldName, width, **params):
        CInDocTableCol.__init__(self, title, fieldName, width, **params)


    def toString(self, val, record):
        return QVariant(convertTNMSDictToDigest(convertTNMSStringToDict(forceString(val))))


    def createEditor(self, parent):
        editor = CTNMSComboBox(parent)
        return editor

    def setEditorData(self, editor, value, record):
        editor.setValue(forceString(value))


    def getEditorData(self, editor):
        return toVariant(editor.getValue())
