# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from PyQt4 import QtGui
from PyQt4.QtCore import Qt, SIGNAL, QEvent

from Ui_TNMSPopup import Ui_TNMSPopup

__all__ = ( 'CTNMSPopup',
          )

class CTNMSPopup(QtGui.QFrame, Ui_TNMSPopup):
    def __init__(self, parent = None):
        QtGui.QFrame.__init__(self, parent, Qt.Popup)
        self.combo = parent
#        self.setAttribute(Qt.WA_WindowPropagation)
#        self.setAttribute(Qt.WA_X11NetWmWindowTypeCombo)
        self.setupUi(self)
        opt = self.comboStyleOption()
        self.setFrameStyle(parent.style().styleHint(QtGui.QStyle.SH_ComboBox_PopupFrameStyle, opt, parent))
        self.setFocusProxy(self.cmbCTumor)

        for combo in self.cmbCTumor, self.cmbCNodus, self.cmbCMetastasis, self.cmbCStage, self.cmbPTumor, self.cmbPNodus, self.cmbPMetastasis, self.cmbPStage:
            combo.installEventFilter(self)
            self.connect(combo, SIGNAL('currentIndexChanged(int)'), self.onAnyCurrentIndexChanged)


    def comboStyleOption(self):
        opt = QtGui.QStyleOptionComboBox()
        opt.initFrom(self.combo)
        opt.subControls = QtGui.QStyle.SC_All
        opt.activeSubControls = QtGui.QStyle.SC_None
        opt.editable = False # self.combo.isEditable()
        return opt


    def eventFilter(self, obj, event):
        if event.type() == QEvent.KeyPress:
            key = event.key()
            if key in (Qt.Key_Enter, Qt.Key_Return, Qt.Key_Escape):
                self.combo.hidePopup()
                return True
#            elif key == Qt.Key_F4 and:
        return QtGui.QFrame.eventFilter(self, obj, event)


    def setValue(self, d):
        def setComboValue(combo, value):
            blocked = combo.blockSignals(True)
            try:
                for i in xrange(combo.count()):
                    if unicode(combo.itemText(i)) == value:
                        combo.setCurrentIndex(i)
                        return
                combo.setCurrentIndex(0)
            finally:
                combo.blockSignals(blocked)

        setComboValue(self.cmbCTumor, d.get('cT','x'))
        setComboValue(self.cmbCNodus, d.get('cN','x'))
        setComboValue(self.cmbCMetastasis, d.get('cM','x'))
        setComboValue(self.cmbCStage, d.get('cS','x'))

        setComboValue(self.cmbPTumor, d.get('pT','x'))
        setComboValue(self.cmbPNodus, d.get('pN','x'))
        setComboValue(self.cmbPMetastasis, d.get('pM','x'))
        setComboValue(self.cmbPStage, d.get('pS','x'))


    def getValue(self):
        return {
                'cT': unicode(self.cmbCTumor.currentText()),
                'cN': unicode(self.cmbCNodus.currentText()),
                'cM': unicode(self.cmbCMetastasis.currentText()),
                'cS': unicode(self.cmbCStage.currentText()),

                'pT': unicode(self.cmbPTumor.currentText()),
                'pN': unicode(self.cmbPNodus.currentText()),
                'pM': unicode(self.cmbPMetastasis.currentText()),
                'pS': unicode(self.cmbPStage.currentText()),
               }

#    @pyqtSignature('int')
    def onAnyCurrentIndexChanged(self, index):
        self.combo.updateValueFromPopup()
