# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

from urllib import unquote_plus
import urllib2

from PyQt4 import QtGui
from PyQt4.QtCore import QByteArray, QVariant

from library.pdf417 import pdf417image


class CResourceLoaderMixin:

    def __init__(self):
        self.images = {}


    def setCanvases(self, canvases):
        for key,  val in canvases.iteritems():
            self.images[key] = val.image


    def loadResource(self, type_, url):
        def decode(s):
            return unquote_plus(str(s))

        if type_ == 2:
            if url.scheme() == 'pdf417':
                params = dict([(decode(qi[0]), decode(qi[1])) for qi in url.encodedQueryItems()])
                return pdf417(**params)
            elif url.scheme() == 'canvas':
                canvasName = str(url.host())
                result = self.images.get(canvasName, None)
                if not result:
                    result = getEmptyImage()
                    self.images[canvasName] = result
                return QVariant(result)
            elif url.scheme() == 'data':
                # в старых версиях Qt нет обработки <img src="data:image/png;base64, ..." />
                # делаю это "вручную", это жрёт память и, возможно, тормозит
                encodedUrl = url.toEncoded()
                if encodedUrl.startsWith('data:image/png;base64,'):
                    encodedImage = encodedUrl.right( encodedUrl.length()-22 ) # 22 == len('data:image/png;base64,')
                    data = QByteArray.fromBase64(encodedImage)
                    image = QtGui.QImage()
                    if image.loadFromData(data):
                        return QVariant(image)
            elif url.scheme() == 'http':
                encodedUrl = unicode(url.toString())
                request = urllib2.Request(encodedUrl)
                resp = urllib2.urlopen(request)
                content = QByteArray(resp.read())
                img = QtGui.QImage()
                if img.loadFromData(content):
                    return QVariant(img)
        # fallback
        if hasattr(self, 'qtLoadResource'):
            return self.qtLoadResource(type_, url)
        return QVariant()



class CTextDocument(CResourceLoaderMixin, QtGui.QTextDocument):
    def __init__(self, parent=None):
        QtGui.QTextDocument.__init__(self, parent)
        CResourceLoaderMixin.__init__(self)

    def qtLoadResource(self, type_, url):
        return QtGui.QTextDocument.loadResource(self, type_, url)


def pdf417(data='', **params):
    try:
        image = pdf417image(data, **params)
        return QVariant(image)
    except:
        return QVariant()


def getEmptyImage():
    image = QtGui.QImage(16, 16, QtGui.QImage.Format_RGB32)
    painter = QtGui.QPainter(image)
    painter.fillRect(0, 0, 16, 16, QtGui.QBrush(QtGui.QColor(255, 255, 255)))
    painter.setPen(QtGui.QPen(QtGui.QColor(0, 255, 0)))
    painter.drawEllipse(0, 0, 15, 15)
    painter.setPen(QtGui.QPen(QtGui.QColor(255, 0, 0)))
    painter.drawLine(0,  0, 15, 15)
    painter.drawLine(0, 15, 15, 0)
    return image
