# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################


from PyQt4 import QtGui

from library.Utils import forceString


class CQuery(object):
    def __init__(self, model_class, queryTable, fields='*', where='', order='', limit=None):
        self._model_class = model_class
        self._queryTable = queryTable
        self._fields = fields
        self._where = where
        self._order = order
        self._limit = limit

    def orderBy(self, value):
        if not value:
            return self
        if not isinstance(self._order, (tuple, list)):
            order = [self._order] if self._order else []
        else:
            order = list(self._order)

        order.append(value)

        return CQuery(self._model_class, self._queryTable, self._fields, self._where, order, self._limit)

    def selectStmt(self):
        return QtGui.qApp.db.selectStmt(self._queryTable, self._fields, self._where, self._order, self._limit)

    def selectStmtGroupBy(self, group):
        return QtGui.qApp.db.selectStmtGroupBy(
            self._queryTable, self._fields, self._where, group, self._order, self._limit
        )

    def selectDistinctStmt(self):
        return QtGui.qApp.db.selectDistinctStmt(self._queryTable, self._fields, self._where, self._order, self._limit)

    def existsStmt(self):
        return QtGui.qApp.db.existsStmt(self._queryTable, self._where)

    def notExistsStmt(self, table, where):
        return QtGui.qApp.db.notExistsStmt(self._queryTable, self._where)

    def getSum(self):
        return QtGui.qApp.db.getSum(self._queryTable, self._fields, self._where)

    def getMax(self, maxCol='id'):
        return QtGui.qApp.db.getMax(self._queryTable, maxCol, self._where)

    def getMin(self, minCol='id'):
        return QtGui.qApp.db.getMin(self._queryTable, minCol, self._where)

    def getCount(self, countCol='1'):
        return QtGui.qApp.db.getCount(self._queryTable, countCol, self._where)

    def getDistinctCount(self, countCol='*'):
        return QtGui.qApp.db.getCount(self._queryTable, countCol, self._where)

    def getIdList(self, idCol='id'):
        return QtGui.qApp.db.getIdList(self._queryTable, idCol, self._where, self._order, self._limit)

    def getDistinctIdList(self, idCol=('id',)):
        return QtGui.qApp.db.getDistinctIdList(self._queryTable, idCol, self._where, self._order, self._limit)

    def getIdListGroupBy(self, idCol='id'):
        return QtGui.qApp.db.selectStmtGroupBy(self._queryTable, idCol, self._where, self._order, self._limit)

    def getRecordList(self):
        return QtGui.qApp.db.getRecordList(self._queryTable, self._fields, self._where, self._order, self._limit)

    def getRecordListGroupBy(self, group=''):
        return QtGui.qApp.db.getRecordListGroupBy(
            self._queryTable, self._fields, self._where, group, self._order, self._limit
        )

    def getList(self):
        return [self._model_class(r) for r in self.getRecordList()]

    def getDistinctList(self):
        return self.getDistinctFieldValues(self._fields, self._model_class)

    def getDistinctStringValues(self, field):
        def formatter(record):
            return forceString(record.value(0))
        return self.getDistinctFieldValues(field, formatter)

    def getDistinctFieldValues(self, field, formatter):
        stmt = QtGui.qApp.db.selectDistinctStmt(self._queryTable, field, self._where, self._order, self._limit)
        query = QtGui.qApp.db.query(stmt)
        result = []
        while query.next():
            result.append(formatter(query.record()))
        return result
