# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

# редактор разных умолчаний

import re
import serial

from PyQt4 import QtGui
from PyQt4.QtCore import Qt, pyqtSignature, QDate, QDir, QVariant

from library.database         import connectDataBase
from library.SendMailDialog   import DefaultSMTPPorts
from library.SerialSettings   import CSerialSettings
from library.TableModel       import CTableModel, CTextCol
from library.Utils            import exceptionToUnicode, forceBool, forceInt, forceRef, forceString, forceStringEx, toVariant

from Orgs.Orgs                import selectOrganisation
from Exchange.TFUnifiedIdent.Service    import CTFUnifiedIdentService
from KLADR.Utils              import getProvinceKLADRCode

from preferences.Ui_appPreferencesDialog  import Ui_appPreferencesDialog


#DefaultSMTPPorts = (25, 587, 465)
DefaultAverageDuration = 28

class CAppPreferencesDialog(QtGui.QDialog, Ui_appPreferencesDialog):
    def __init__(self, parent):
        QtGui.QDialog.__init__(self, parent)
        self.setupUi(self)
        self.tabWidget.removeTab(self.tabWidget.indexOf(self.tabRIS))
        self.cmbProvince.setAreaSelectable(True)
#        self.portValidator = QtGui.QIntValidator(1, 99999, self)
#        self.edtSMTPPort.setValidator(self.portValidator)
        self.cmbTFAccountingSystemId.setTable('rbAccountingSystem', True)
        self.cmbTempInvalidDoctype.setTable('rbTempInvalidDocument', True, 'type=0')
        self.cmbTempInvalidReason.setTable('rbTempInvalidReason', True, 'type=0')
        if 'QPrinterInfo' in QtGui.__dict__: # work-around for current version of pyqt in ubuntu 8.04
            printerNames = [ unicode(pi.printerName()) for pi in QtGui.QPrinterInfo.availablePrinters() ]
        else:
            printerNames = []
        printerNames.insert(0,'')
        self.cmbLabelPrinter.addItems(printerNames)

        self.modelGlobalPreferences = CGlobalPreferencesModel(self)
        self.selectionModelGlobalPreferences = QtGui.QItemSelectionModel(self.modelGlobalPreferences, self)
        self.tblGlobal.setModel(self.modelGlobalPreferences)
        self.tblGlobal.setSelectionModel(self.selectionModelGlobalPreferences)

    def setTabGlobalEnabled(self, value):
        self.tabWidget.setTabEnabled(self.tabWidget.indexOf(self.tabGlobal), value)
        if value:
            self.refreshGlobalPreferences()


    def refreshGlobalPreferences(self):
        self.modelGlobalPreferences.setIdList(QtGui.qApp.db.getIdList('GlobalPreferences'))

    def setProps(self, props):
#        global DefaultSMTPPorts

        self.cmbOrganisation.setValue(forceRef(props.get('orgId', QVariant())))
        self.cmbOrgStructure.setValue(forceRef(props.get('orgStructureId', QVariant())))
        self.chkFilterPaymentByOrgStructure.setChecked(forceBool(props.get('filterPaymentByOrgStructure', QVariant())))
        self.chkOrgStructurePriorityForAddActions.setChecked(forceBool(props.get('orgStructurePriorityForAddActions', QVariant())))
        kladrCode = forceString(props.get('defaultKLADR', '7800000000000'))
        self.cmbDefaultCity.setCode(kladrCode)
        provinceKLADR = forceString(props.get('provinceKLADR', ''))
        self.cmbProvince.setCode(provinceKLADR or getProvinceKLADRCode(kladrCode))
        self.edtSMTPServer.setText(forceString(props.get('SMTPServer', QVariant())))
        encryption = max(0, min(2, forceInt(props.get('SMTPEncryption', QVariant()))))
        self.cmbSMTPEncryption.setCurrentIndex(encryption)
        port = forceInt(props.get('SMTPPort', QVariant()))
        self.edtSMTPPort.setValue(port if port>0 else DefaultSMTPPorts[encryption])
        self.edtMailAddress.setText(forceString(props.get('mailAddress', QVariant())))
        self.chkSMTPAuthentification.setChecked(forceBool(props.get('SMTPAuthentification', QVariant())))
        self.edtSMTPLogin.setText(forceString(props.get('SMTPLogin', QVariant())))
        self.edtSMTPPassword.setText(forceString(props.get('SMTPPassword', QVariant())))

        self.cmbDriverName.setEditText(forceString(props.get('EIS_driverName', QVariant())))
        self.edtServerName.setText(forceString(props.get('EIS_serverName', QVariant())))
        self.edtServerPort.setValue(forceInt(props.get('EIS_serverPort', QVariant())))
        self.edtDatabaseName.setText(forceString(props.get('EIS_databaseName', QVariant())))
        self.edtUserName.setText(forceString(props.get('EIS_userName', QVariant())))
        self.edtPassword.setText(forceString(props.get('EIS_password', QVariant())))

        socCardSupport = forceBool(props.get('SocCard_SupportEnabled',  False))
        self.chkEnableSocCardSupport.setChecked(socCardSupport)
        self.chkCardReaderEmulation.setChecked(forceBool(props.get( 'SocCard_Emulation',  False)))
        self.chkDefaultClientSearchUsingSocCard.setChecked(forceBool(props.get( 'SocCard_SetAsDefaultSearch',  False)))
        self.sbPortNumber.setValue(forceInt(props.get('SocCard_PortNumber', 0)))
        self.sbDevId.setValue(forceInt(props.get( 'SocCard_DevId',  0)))
        self.sbIssBSC0.setValue(forceInt(props.get( 'SocCard_IssBSC0',  0)))
        self.sbIssBSC1.setValue(forceInt(props.get( 'SocCard_IssBSC1',  0)))
        self.sbIssBSC2.setValue(forceInt(props.get( 'SocCard_IssBSC2',  0)))
        self.sbIssBSC3.setValue(forceInt(props.get( 'SocCard_IssBSC3',  0)))
        self.sbIssBSC4.setValue(forceInt(props.get( 'SocCard_IssBSC4',  0)))
        self.sbIssBSC5.setValue(forceInt(props.get( 'SocCard_IssBSC5',  0)))
        self.edtOperatorId.setText(forceString(props.get( 'SocCard_OperatorId', '')))
        self.edtDocumentTypeRbFileName.setText(forceString(props.get('SocCard_rbDocumentTypeRbFileName',  '')))
        self.edtClientBenefitRbFileName.setText(forceString(props.get( 'SocCard_rbClientBenefitRbFileName', '')))

        self.chkScanEnable.setChecked(forceBool(props.get('ScannerEnable', QVariant())))
        self.setScannerPortSettings(forceString(props.get('ScannerPortSettings', '')))
        self.chkScanCorrectBackslashes.setChecked(forceBool(props.get('ScannerCorrectBackslashes', True)))
        self.chkScanConnectionReport.setChecked(forceBool(props.get('ScannerConnectionReport', True)))
        self.chkScanParseReport.setChecked(forceBool(props.get('ScannerParseReport', False)))

        self.chkTFCheckPolicy.setChecked(forceBool(props.get('TFCheckPolicy', False)))
        self.edtTFCPUrl.setText(forceString(props.get('TFCPUrl', '')))
#        self.edtTFCPLogin.setText(forceString(props.get('TFCPLogin', '')))
#        self.edtTFCPPassword.setText(forceString(props.get('TFCPPassword', '')))
        self.cmbTFAccountingSystemId.setValue(forceRef(props.get('TFAccountingSystemId', '1')))

        self.edtAverageDuration.setValue(forceInt(props.get('averageDuration', QVariant(DefaultAverageDuration))))
        self.cmbTempInvalidDoctype.setCode(forceString(props.get('tempInvalidDoctype', QVariant())))
        self.cmbTempInvalidReason.setCode(forceString(props.get('tempInvalidReason', QVariant())))
        self.edtTemplateDir.setText(forceString(props.get('templateDir', QVariant(QtGui.qApp.getTemplateDir()))))
        self.chkEnableFastPrint.setChecked(forceBool(props.get('enableFastPrint', QVariant())))
        iPrinter = self.cmbLabelPrinter.findText(forceString(props.get('labelPrinter', QVariant())))
        self.cmbLabelPrinter.setCurrentIndex(iPrinter)
        self.chkPrintBlackWhite.setChecked(forceBool(props.get('printBlackWhite', True)))
        self.cmbDoubleClickQueuePerson.setCurrentIndex(forceInt(props.get('doubleClickQueuePerson', QVariant())))
        self.cmbDoubleClickQueueClient.setCurrentIndex(forceInt(props.get('doubleClickQueueClient', QVariant())))
        self.cmbDoubleClickQueueFreeOrder.setCurrentIndex(forceInt(props.get('doubleClickQueueFreeOrder', QVariant())))
        self.chkAmbulanceUserCheckable.setChecked(forceBool(props.get('ambulanceUserCheckable', QVariant())))
        self.chkHighlightRedDate.setChecked(forceBool(props.get('highlightRedDate', QVariant())))
        self.chkHighlightInvalidDate.setChecked(forceBool(props.get('highlightInvalidDate', QVariant())))
        self.edtDocumentEditor.setText(forceString(props.get('documentEditor',  '')))
        self.edtExtGenRep.setText(forceString(props.get('extGenRep',  '')))
        self.edtCachBox.setText(forceString(props.get('cashBox',  '')))
        self.edtExaroEditor.setText(forceString(props.get('exaroEditor', '')))
        self.chkJobTicketEndDateAskingIsRequired.setChecked(forceBool(props.get('jobTicketEndDateAskingIsRequired', QVariant(True))))
        self.chkRISIntegration.setChecked(forceBool(props.get('RIS_enabled', QVariant(False))))
        self.edtServiceAddress.setText(forceString(props.get('RIS_serviceAddress',  '')))
        self.edtLPUCode.setText(forceString(props.get('RIS_athLPUCode',  '')))
        self.edtToken.setText(forceString(props.get('RIS_authToken',  '')))
        self.chkPersonnelByDblClick.setChecked(forceBool(props.get('openPersonnelByDblClick', False)))
        self.chkPersonFilterInCmb.setChecked(forceBool(props.get('personFilterInCmb', False)))
        self.chkOnlyDoctors.setChecked(forceBool(props.get('onlyDoctorsInPopup', False)))
        self.cmbPacsType.setCurrentIndex(forceInt(props.get('pacsType')))
        self.edtPacsParams.setText(forceString(props.get('pacsParams')))
        self.edtPacsWado.setText(forceString(props.get('pacsWado')))
        self.cmbShowingInInfoBlockSocStatus.setCurrentIndex(forceInt(props.get('showingInInfoBlockSocStatus', QVariant())))
        self.cmbDiagnosisTypeAfterFinalForm043.setCurrentIndex(forceInt(props.get('diagnosisTypeAfterFinalForm043', QVariant())))
        self.setTreesProps(props)
        pacsType = forceInt(props.get('pacsType'))
        self.edtPacsWado.setEnabled(pacsType == 1)
        self.lblPacsWado.setEnabled(pacsType == 1)

    def setTreesProps(self, props):
        treeOrgStrExpand = forceInt(props.get('treeOrgStructureExpand',  QVariant()))
        self.cmbTreeOrgStructureExpand.setCurrentIndex(treeOrgStrExpand)
        if treeOrgStrExpand == 2:
            self.edtTreeOrgStructureExpand.setVisible(True)
            self.edtTreeOrgStructureExpand.setValue(forceInt(props.get('treeOrgStructureExpandLevel')))
        else:
            self.edtTreeOrgStructureExpand.setVisible(False)

        treeActionTypeExpand = forceInt(props.get('actionTypeTreeExpand',  QVariant()))
        self.cmbActionTypeTreeExpand.setCurrentIndex(treeActionTypeExpand)
        if treeActionTypeExpand == 2:
            self.edtActionTypeTreeExpand.setVisible(True)
            self.edtActionTypeTreeExpand.setValue(forceInt(props.get('actionTypeTreeExpandLevel')))
        else:
            self.edtActionTypeTreeExpand.setVisible(False)

        treeProbesExpand = forceInt(props.get('probesTreeExpand',  QVariant()))
        self.cmbProbesTreeExpand.setCurrentIndex(treeProbesExpand)
        if treeProbesExpand == 2:
            self.edtProbesTreeExpand.setVisible(True)
            self.edtProbesTreeExpand.setValue(forceInt(props.get('probesTreeExpandLevel')))
        else:
            self.edtProbesTreeExpand.setVisible(False)


    def props(self):
#        global DefaultSMTPPorts

        result = {}
        result['orgId']           =toVariant(self.cmbOrganisation.value())
        result['orgStructureId']  =toVariant(self.cmbOrgStructure.value())
        result['filterPaymentByOrgStructure'] = toVariant(
            1 if self.chkFilterPaymentByOrgStructure.isChecked() else 0)
        result['orgStructurePriorityForAddActions'] = toVariant(
            1 if self.chkOrgStructurePriorityForAddActions.isChecked() else 0)
        result['defaultKLADR']    =toVariant(self.cmbDefaultCity.code())
        result['provinceKLADR']   =toVariant(self.cmbProvince.code())
        result['SMTPServer']      =toVariant(forceStringEx(self.edtSMTPServer.text()))
        encryption = self.cmbSMTPEncryption.currentIndex()
        port = self.edtSMTPPort.value()
        result['SMTPEncryption']  =toVariant(encryption)
        result['SMTPPort']        =toVariant(port if port>0 else DefaultSMTPPorts[encryption])
        result['mailAddress']     =toVariant(forceStringEx(self.edtMailAddress.text()))
        result['SMTPAuthentification'] =toVariant(self.chkSMTPAuthentification.isChecked())
        result['SMTPLogin']       =toVariant(forceStringEx(self.edtSMTPLogin.text()))
        result['SMTPPassword']    =toVariant(forceStringEx(self.edtSMTPPassword.text()))

        result['EIS_driverName']    =toVariant(forceStringEx(self.cmbDriverName.currentText()))
        result['EIS_serverName']    =toVariant(forceStringEx(self.edtServerName.text()))
        result['EIS_serverPort']    =toVariant(forceInt(self.edtServerPort.value()))
        result['EIS_databaseName']    =toVariant(forceStringEx(self.edtDatabaseName.text()))
        result['EIS_userName']    =toVariant(forceStringEx(self.edtUserName.text()))
        result['EIS_password']    =toVariant(forceStringEx(self.edtPassword.text()))

        result['SocCard_SupportEnabled'] = toVariant(self.chkEnableSocCardSupport.isChecked())
        result['SocCard_Emulation'] = toVariant(self.chkCardReaderEmulation.isChecked())
        result['SocCard_SetAsDefaultSearch'] = toVariant(self.chkDefaultClientSearchUsingSocCard.isChecked())
        result['SocCard_PortNumber'] = toVariant(self.sbPortNumber.value())
        result['SocCard_DevId'] = toVariant(self.sbDevId.value())
        result['SocCard_IssBSC0'] = toVariant(self.sbIssBSC0.value())
        result['SocCard_IssBSC1'] = toVariant(self.sbIssBSC1.value())
        result['SocCard_IssBSC2'] = toVariant(self.sbIssBSC2.value())
        result['SocCard_IssBSC3'] = toVariant(self.sbIssBSC3.value())
        result['SocCard_IssBSC4'] = toVariant(self.sbIssBSC4.value())
        result['SocCard_IssBSC5'] = toVariant(self.sbIssBSC5.value())
        result['SocCard_OperatorId'] = toVariant(self.edtOperatorId.text())
        result['SocCard_rbDocumentTypeRbFileName'] = toVariant(self.edtDocumentTypeRbFileName.text())
        result['SocCard_rbClientBenefitRbFileName'] = toVariant(self.edtClientBenefitRbFileName.text())

        result['ScannerEnable']             = toVariant(int(self.chkScanEnable.isChecked()))
        result['ScannerPortSettings']        = toVariant(self.getScannerPortSettings())
        result['ScannerCorrectBackslashes'] = toVariant(int(self.chkScanCorrectBackslashes.isChecked()))
        result['ScannerConnectionReport']   = toVariant(int(self.chkScanConnectionReport.isChecked()))
        result['ScannerParseReport']        = toVariant(int(self.chkScanParseReport.isChecked()))

        result['TFCheckPolicy'] = toVariant(int(self.chkTFCheckPolicy.isChecked()))
        result['TFCPUrl']       = toVariant(self.edtTFCPUrl.text())
#        result['TFCPLogin']     = toVariant(self.edtTFCPLogin.text())
#        result['TFCPPassword']  = toVariant(self.edtTFCPPassword.text())
        result['TFAccountingSystemId'] = toVariant(self.cmbTFAccountingSystemId.value())

        result['averageDuration']        = toVariant(self.edtAverageDuration.value())
        result['tempInvalidDoctype']     = toVariant(self.cmbTempInvalidDoctype.code())
        result['tempInvalidReason']      = toVariant(self.cmbTempInvalidReason.code())

        result['templateDir']            = toVariant(self.edtTemplateDir.text())
        result['enableFastPrint']        = toVariant(self.chkEnableFastPrint.isChecked())
        result['labelPrinter']           = toVariant(self.cmbLabelPrinter.currentText())
        result['printBlackWhite']        = toVariant(self.chkPrintBlackWhite.isChecked())

        result['doubleClickQueuePerson']    = toVariant(self.cmbDoubleClickQueuePerson.currentIndex())
        result['doubleClickQueueClient']    = toVariant(self.cmbDoubleClickQueueClient.currentIndex())
        result['doubleClickQueueFreeOrder'] = toVariant(self.cmbDoubleClickQueueFreeOrder.currentIndex())

        result['ambulanceUserCheckable'] = toVariant(int(self.chkAmbulanceUserCheckable.isChecked()))

        result['highlightRedDate']       = toVariant(int(self.chkHighlightRedDate.isChecked()))
        result['highlightInvalidDate']   = toVariant(int(self.chkHighlightInvalidDate.isChecked()))
        result['documentEditor']         = toVariant(self.edtDocumentEditor.text())
        result['extGenRep']              = toVariant(self.edtExtGenRep.text())
        result['cashBox']                = toVariant(self.edtCachBox.text())
        result['exaroEditor']            = toVariant(self.edtExaroEditor.text())

        result['jobTicketEndDateAskingIsRequired'] = toVariant(int(self.chkJobTicketEndDateAskingIsRequired.isChecked()))

        result['RIS_enabled'] = toVariant(self.chkRISIntegration.isChecked())
        result['RIS_serviceAddress'] = toVariant(self.edtServiceAddress.text())
        result['RIS_athLPUCode'] = toVariant(self.edtLPUCode.text())
        result['RIS_authToken'] = toVariant(self.edtToken.text())

        result['onlyDoctorsInPopup'] = toVariant(self.chkOnlyDoctors.isChecked())

        result['treeOrgStructureExpand'] = toVariant(self.cmbTreeOrgStructureExpand.currentIndex())
        result['treeOrgStructureExpandLevel'] = toVariant(self.edtTreeOrgStructureExpand.value())
        result['actionTypeTreeExpand'] = toVariant(self.cmbActionTypeTreeExpand.currentIndex())
        result['actionTypeTreeExpandLevel'] = toVariant(self.edtActionTypeTreeExpand.value())
        result['probesTreeExpand'] = toVariant(self.cmbProbesTreeExpand.currentIndex())
        result['probesTreeExpandLevel'] = toVariant(self.edtProbesTreeExpand.value())

        result['openPersonnelByDblClick'] = toVariant(self.chkPersonnelByDblClick.isChecked())
        result['personFilterInCmb'] = toVariant(self.chkPersonFilterInCmb.isChecked())

        result['pacsType'] = toVariant(self.cmbPacsType.currentIndex())
        result['pacsParams'] = toVariant(self.edtPacsParams.text())
        result['pacsWado'] = toVariant(self.edtPacsWado.text())
        
        result['showingInInfoBlockSocStatus'] = toVariant(self.cmbShowingInInfoBlockSocStatus.currentIndex())
        result['diagnosisTypeAfterFinalForm043'] = toVariant(self.cmbDiagnosisTypeAfterFinalForm043.currentIndex())
        return result


    def setScannerPortSettings(self, prop):
        settings = CSerialSettings(prop)
        self.edtScanPort.setText(settings.port)
        self.cmbScanBaudrate.setCurrentIndex(self.cmbScanBaudrate.findText(str(settings.baudrate), Qt.MatchFixedString))
        self.cmbScanParity.setCurrentIndex(self.cmbScanParity.findText(str(settings.parity), Qt.MatchStartsWith))
        self.cmbScanStopBits.setCurrentIndex(self.cmbScanStopBits.findText(str(settings.stopbits), Qt.MatchFixedString))
        self.cmbScanFlowControl.setCurrentIndex( 1 if settings.rtscts else 2 if settings.xonxoff else 0 )


    def getScannerPortSettingsAsObject(self):
        settings = CSerialSettings()
        settings.port = unicode(self.edtScanPort.text())
        settings.baudrate = int(self.cmbScanBaudrate.currentText())
        settings.parity = str(self.cmbScanParity.currentText()[0])
        settings.stopbits = float(self.cmbScanStopBits.currentText())

        flowControl = self.cmbScanFlowControl.currentIndex()
        if flowControl == 1:
            settings.xonxoff = False
            settings.rtscts = True
        elif flowControl == 2:
            settings.xonxoff = True
            settings.rtscts = False
        else:
            settings.xonxoff = settings.rtscts = False
        return settings


    def getScannerPortSettings(self):
        settings = self.getScannerPortSettingsAsObject()
        return settings.asString()

    
    @pyqtSignature('int')
    def on_cmbPacsType_currentIndexChanged(self, pacsType):
        self.edtPacsWado.setEnabled(pacsType == 1)
        self.lblPacsWado.setEnabled(pacsType == 1)

    @pyqtSignature('int')
    def on_cmbOrganisation_currentIndexChanged(self, index):
        orgId = self.cmbOrganisation.value()
        self.cmbOrgStructure.setOrgId(orgId)


    @pyqtSignature('')
    def on_btnSelectOrganisation_clicked(self):
        orgId = selectOrganisation(self, self.cmbOrganisation.value(), False)
        self.cmbOrganisation.updateModel()
        if orgId:
            self.cmbOrganisation.setValue(orgId)


    @pyqtSignature('int')
    def on_cmbDefaultCity_currentIndexChanged(self):
        cityKLADRCode = self.cmbDefaultCity.code()
        provinceKLADRCode = getProvinceKLADRCode(cityKLADRCode)
        self.cmbProvince.setCode(provinceKLADRCode)

    @pyqtSignature('int')
    def on_cmbProvince_currentIndexChanged(self):
        #Отображаем настройки РИС только для краснодарского края
        if self.cmbProvince.code()[:2] == '23':
            self.tabWidget.insertTab(4, self.tabRIS ,u"РИС")
        else:
            self.tabWidget.removeTab(self.tabWidget.indexOf(self.tabRIS))


    @pyqtSignature('int')
    def on_cmbSMTPEncryption_currentIndexChanged(self, index):
#        global DefaultSMTPPorts

        port = self.edtSMTPPort.value()
        if port == 0 or port in DefaultSMTPPorts:
            self.edtSMTPPort.setValue(DefaultSMTPPorts[index])

    @pyqtSignature('int')
    def on_cmbTreeOrgStructureExpand_currentIndexChanged(self, index):
        if index == 2:
            self.edtTreeOrgStructureExpand.setVisible(True)
        else:
            self.edtTreeOrgStructureExpand.setVisible(False)

    @pyqtSignature('int')
    def on_cmbActionTypeTreeExpand_currentIndexChanged(self, index):
        if index == 2:
            self.edtActionTypeTreeExpand.setVisible(True)
        else:
            self.edtActionTypeTreeExpand.setVisible(False)

    @pyqtSignature('int')
    def on_cmbProbesTreeExpand_currentIndexChanged(self, index):
        if index == 2:
            self.edtProbesTreeExpand.setVisible(True)
        else:
            self.edtProbesTreeExpand.setVisible(False)


    @pyqtSignature('bool')
    def on_chkSMTPAuthentification_clicked(self, checked):
        if checked and self.edtSMTPLogin.text().isEmpty():
            match = re.compile(r'''^(?:.*\s+)?([^\s]+)@''').match(forceString(self.edtMailAddress.text()))
            if match:
                self.edtSMTPLogin.setText(match.group(1))


    @pyqtSignature('')
    def on_btnEisTest_clicked(self):
        try:
            EIS_db=QtGui.qApp.EIS_db
            if not EIS_db:
                EIS_dbDriverName=forceStringEx(self.cmbDriverName.currentText())
                EIS_dbServerName=forceStringEx(self.edtServerName.text())
                EIS_dbServerPort=forceInt(self.edtServerPort.value())
                EIS_dbDatabaseName=forceStringEx(self.edtDatabaseName.text())
                EIS_dbUserName=forceStringEx(self.edtUserName.text())
                EIS_dbPassword=forceStringEx(self.edtPassword.text())
                EIS_db = connectDataBase(
                    EIS_dbDriverName, EIS_dbServerName, EIS_dbServerPort,
                    EIS_dbDatabaseName, EIS_dbUserName, EIS_dbPassword, 'EIS')
                QtGui.qApp.EIS_db=EIS_db
            TARIFF_MONTH=EIS_db.getRecordEx('TARIFF_MONTH', '*', '')
            ID_TARIFF_MONTH=forceString(TARIFF_MONTH.value('ID_TARIFF_MONTH'))
            TARIFF_MONTH_BEG=forceString(TARIFF_MONTH.value('TARIFF_MONTH_BEG').toDate())
            TARIFF_MONTH_END=forceString(TARIFF_MONTH.value('TARIFF_MONTH_END').toDate())
            MU_IDENT = EIS_db.getRecordEx('MU_IDENT', 'ID_LPU')
            ID_LPU = forceInt(MU_IDENT.value('ID_LPU')) if MU_IDENT else None
            message=u'Соединение успешно!\nтарифный месяц: %s (%s-%s)\nИдентификатор ЛПУ в ЕИС ОМС: %s' % (ID_TARIFF_MONTH, TARIFF_MONTH_BEG, TARIFF_MONTH_END, ID_LPU)
            QtGui.qApp.EIS_db.close()
            QtGui.qApp.EIS_db=None
        except:
            QtGui.qApp.logCurrentException()
            if QtGui.qApp.EIS_db:
                QtGui.qApp.EIS_db.close()
                QtGui.qApp.EIS_db=None
            message=u'Установить соединение не удалось'
        QtGui.QMessageBox.information(self, u'Проверка обращения к серверу ЕИС',
            message, QtGui.QMessageBox.Ok, QtGui.QMessageBox.Ok)


    @pyqtSignature('')
    def on_btnSelectTemplateDir_clicked(self):
        dir = QtGui.QFileDialog.getExistingDirectory(
            self, u'Укажите директорий с шаблонами документов', self.edtTemplateDir.text())
        if dir:
            self.edtTemplateDir.setText(QDir.toNativeSeparators(dir))


    @pyqtSignature('')
    def on_btnSelectDocumentEditor_clicked(self):
        fileName = forceString(QtGui.QFileDialog.getOpenFileName(
            self, u'Укажите исполняемый файл редактора документов',
            self.edtDocumentEditor.text(), u'Исполняемые файлы (*)'))
        if fileName:
            self.edtDocumentEditor.setText(QDir.toNativeSeparators(fileName))


    @pyqtSignature('')
    def on_btnSelectExtGenRep_clicked(self):
        fileName = forceString(QtGui.QFileDialog.getOpenFileName(
            self, u'Укажите исполняемый файл генератора отчетов',
            self.edtExtGenRep.text(), u'Исполняемые файлы (*)'))

        if fileName:
            self.edtExtGenRep.setText(QDir.toNativeSeparators(fileName))


    @pyqtSignature('')
    def on_btnSelectExaroEditor_clicked(self):
        fileName = forceString(QtGui.QFileDialog.getOpenFileName(
            self, u'Укажите исполняемый файл редактора отчетов Exaro',
            self.edtExaroEditor.text(), u'Исполняемые файлы (*)'))

        if fileName:
            self.edtExaroEditor.setText(QDir.toNativeSeparators(fileName))


    @pyqtSignature('')
    def on_btnSelectClientBenefitRbFileName_clicked(self):
        fileName = forceString(QtGui.QFileDialog.getOpenFileName(
            self, u'Укажите файл со справочниками типов льгот',
            self.edtClientBenefitRbFileName.text(), u'Файлы XML (*.xml)'))

        if fileName:
            self.edtClientBenefitRbFileName.setText(QDir.toNativeSeparators(fileName))


    @pyqtSignature('')
    def on_btnSelectDocumentTypeRbFileName_clicked(self):
        fileName = forceString(QtGui.QFileDialog.getOpenFileName(
            self, u'Укажите файл со справочниками типов документов',
            self.edtDocumentTypeRbFileName.text(), u'Файлы XML (*.xml)'))

        if fileName:
            self.edtDocumentTypeRbFileName.setText(QDir.toNativeSeparators(fileName))


    @pyqtSignature('')
    def on_btnSocCardTest_clicked(self):
        if self.chkCardReaderEmulation.isChecked():
            from Registry.CardReader import CCardReaderEmulator
            cardReader = CCardReaderEmulator()
        else:
            from Registry.CardReader import CCardReader
            cardReader = CCardReader()

        nPortNo = self.sbPortNumber.value()
        diDevId = self.sbDevId.value()
        listIssBSC = []

        for i in range(0, 6):
            listIssBSC.append(getattr(self, 'sbIssBSC%d' %i).value())

        szOperatorId = forceString(self.edtOperatorId.text())

        initOk = cardReader.init(nPortNo, diDevId, listIssBSC, szOperatorId)
        rc = cardReader.rc()
        cardIn = cardReader.checkCardType()
        from Registry.CardReader import loadDocumentTypeRb,  loadBenefitTypeRb
        docTypeRb = (loadDocumentTypeRb(self, forceString(self.edtDocumentTypeRbFileName.text())) != {})
        benefitTypeRb = (loadBenefitTypeRb(self, forceString(self.edtClientBenefitRbFileName.text())) != {})

        message = u'<h4>Результат тестирования: </h4>\n' \
                u'Инициализация: <b>%s</b><br>' \
                u'Наличие карты: <b>%s</b><br><br>' \
                u'Справочник типов документов: <b>%s</b><br>' \
                u'Справочник типов льгот: <b>%s</b><br><br>' \
                u'Параметры соединения:<table>' \
                u'<tr><td>Номер порта:</td><td><b>%d</b></td></tr>' \
                u'<tr><td>Идентификатор устройства:</td><td><b>%d</b></td></tr>'\
                u'<tr><td>Код банка:</td><td><b>%.2d%.2d%.2d%.2d%.2d%.2d</b></td></tr>' \
                u'<tr><td>Идентификатор оператора:</td><td><b>%s</b></td></tr>'\
                u'<tr><td>Ошибка:</td><td><b>%s (код %d)</b></td></tr></table>' % \
                (u'успешно' if initOk else u'ошибка',  u'есть' if cardIn else u'отсутствует',
                u'прочитан' if docTypeRb else u'ошибка',  u'прочитан' if benefitTypeRb else u'ошибка',
                nPortNo, diDevId, listIssBSC[0], listIssBSC[1], listIssBSC[2], listIssBSC[3],
                listIssBSC[4], listIssBSC[5], szOperatorId,  cardReader.errorText(rc),  rc)

        QtGui.QMessageBox.information(self, u'Проверка работы социальной карты',
            message, QtGui.QMessageBox.Ok, QtGui.QMessageBox.Ok)



    @pyqtSignature('')
    def on_btnScanTest_clicked(self):
        from ScanTestDialog import CScanTestDialog
        settings = self.getScannerPortSettingsAsObject()
        try:
            scanner = None
            try:
                scanner = serial.Serial(settings.port,
                                        baudrate=settings.baudrate,
                                        bytesize=8,
                                        parity=settings.parity,
                                        stopbits=settings.stopbits,
                                        xonxoff=settings.xonxoff,
                                        rtscts=settings.rtscts,
                                        timeout=0.1)
#            except IOError, e:
            except Exception, e:
                QtGui.qApp.logCurrentException()
                QtGui.QMessageBox.information(self,
                                              u'Ошибка подключения к сканеру',
                                              exceptionToUnicode(e),
                                              QtGui.QMessageBox.Close,
                                              QtGui.QMessageBox.Close)
                return
            testDialog = CScanTestDialog(self, scanner)
            testDialog.exec_()
        finally:
            if scanner is not None:
                scanner.close()


    @pyqtSignature('')
    def on_btnCheckPacs_clicked(self):
        addr = forceString(self.edtPacsAddress.text())
        port = forceString(self.edtPacsPort.value())
        authNeeded = self.grpPacsAutentification.isChecked()
        login = None
        password = None
        if authNeeded:
            login = forceString(self.edtPacsLogin.text())
            password = forceString(self.edtPacsPassword.text())
        message = u'Успешно' if checkPacsConnection('%s:%s'%(addr, port), login, password) else u'Ошибка'

        QtGui.QMessageBox.information(self, u'Проверка соединения',
                message, QtGui.QMessageBox.Ok, QtGui.QMessageBox.Ok)


    @pyqtSignature('')
    def on_btnTFCPTest_clicked(self):
        url = forceString(self.edtTFCPUrl.text())
        service = CTFUnifiedIdentService(url)
        try:
            QtGui.qApp.callWithWaitCursor(self, service.getPolicyAndAttach, u'Неуловимый', u'Джо', u'', 0,  QDate(1900, 1, 1), '', '', '', '', '')
            message = u'Соединение успешно'
        except Exception, e:
            QtGui.qApp.logCurrentException()
            message = u'Соединение не удалось: %s' % exceptionToUnicode(e)

        QtGui.QMessageBox.information(self, u'Проверка соединения',
            message, QtGui.QMessageBox.Ok, QtGui.QMessageBox.Ok)


    @pyqtSignature('QModelIndex')
    def on_tblGlobal_doubleClicked(self, index):
        if QtGui.qApp.userHasAnyRight(['adm', 'setupGlobalPreferencesEdit']):
            item = self.tblGlobal.currentItem()
            currentValue = forceString(item.value('value'))
            note = forceString(item.value('note'))
            newValue, ok = QtGui.QInputDialog.getText(self,
                                         u'Редактор',
                                         u'Значение\n'+note,
                                         QtGui.QLineEdit.Normal,
                                         currentValue)
            if ok and unicode(newValue) != unicode(currentValue):
                item.setValue('value', QVariant(newValue))
                QtGui.qApp.db.updateRecord('GlobalPreferences', item)
                self.refreshGlobalPreferences()


# ########################################################


class CGlobalPreferencesModel(CTableModel):
    def __init__(self, parent):
        CTableModel.__init__(self, parent)
        self.addColumn(CTextCol(u'Код', ['code'], 10))
        self.addColumn(CTextCol(u'Наименование', ['name'], 18))
        self.addColumn(CTextCol(u'Значение', ['value'], 10))
        self.loadField('note')
        self.setTable('GlobalPreferences')
