# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################

import datetime

from PyQt4 import QtGui, QtSql
from PyQt4.QtCore import Qt, QDate, QDateTime, QModelIndex, QObject, QTime, QVariant, pyqtSignature, SIGNAL, QString, QRegExp

from library.crbcombobox                      import CRBModelDataCache, CRBComboBox
from library.database                         import CTableRecordCache
from library.DbComboBox                       import CDbModel
from library.InDocTable                       import CInDocTableModel, CDateInDocTableCol, CEnumInDocTableCol, CFloatInDocTableCol, CInDocTableCol, CIntInDocTableCol, CRBInDocTableCol
from library.interchange                      import setComboBoxValue, setLineEditValue, setSpinBoxValue, setTextEditValue
from library.ItemsListDialog                  import CItemEditorBaseDialog
from library.LineEditWithRegExpValidator      import CLineEditWithRegExpValidator
from library.PrintTemplates                   import escape
from library.TableModel                       import CTableModel, CCol, CDateCol, CNameCol, CRefBookCol, CTextCol, CTimeCol
from library.Utils                            import calcAgeTuple, checkSNILS, fixSNILS, forceBool, forceDate, forceDateTime, forceDouble, forceInt, forceRef, forceString, forceStringEx, forceTime, formatSex, formatShortName, formatSNILS, isNameValid, nameCase, pyDate, smartDict, sorry, toVariant, trim, unformatSNILS

from Accounting.Utils                         import CTariff # WTF!
from ClientHousesList                         import CClientHousesList
from ClientQuotaDiscussionEditor              import CQuotingEditorDialog
from KLADR.KLADRModel                         import getDistrictName, getOKATO
from Orgs.ContractFindComboBox                import CContractTreeFindComboBox
from Orgs.OrgComboBox                         import CInsurerInDocTableCol, CPolyclinicComboBox
from Orgs.Orgs                                import selectOrganisation
from Orgs.OrgStructureCol                     import COrgStructureInDocTableCol
from Orgs.Utils                               import advisePolicyType, findOrgStructuresByHouseAndFlat, getOKVEDName, getOrganisationInfo
from Quoting.QuotaTypeComboBox                import CQuotaTypeComboBox
from Registry.ClientRelationComboBox          import CClientRelationComboBox
from Registry.HurtModels                      import CWorkHurtFactorsModel, CWorkHurtsModel
from Registry.NameMedicamentEdit              import CMedicamentInDocTableCol
from Registry.Utils                           import CClientRelationComboBoxForConsents, CDbSearchWidget, clientIdToText, findHouseId, findKLADRRegionRecordsInQuoting, findDistrictRecordsInQuoting, formatAddressInt, getAddress, getAddressId, getClientAddress, getClientDocument, getClientWork, uniqueIdentifierCheckingIsPassed, unitedPolicyIsVaid
from Users.Rights                             import urRegEditClientAttach, urRegTabReadRegistry, urRegTabWriteRegistry
from RefBooks.DocumentTypeDescription         import getDocumentTypeDescr
from Registry.Ui_ClientEditDialog             import Ui_Dialog


class CClientEditDialog(CItemEditorBaseDialog, Ui_Dialog):
    prevAddress = None
    prevWork    = None

#    defaultKLADRCode = '7800000000000'

    def __init__(self, parent):
# ctor
        CItemEditorBaseDialog.__init__(self, parent, 'Client')
        self.__id = None
        self.__regAddressRecord = None
        self.__quotaRecord = None
        self.__regAddress = None
        self.__locAddressRecord = None
        self.__locAddress = None
        self.__documentRecord = None
        self.__workRecord = None
# create models
        self.addModels('SocStatuses', CSocStatusesModel(self))
        self.addModels('Attaches',    CAttachesModel(self))
        self.addModels('DirectRelations', CDirectRelationsModel(self))
        self.addModels('BackwardRelations', CBackwardRelationsModel(self))
        self.addModels('WorkHurts',   CWorkHurtsModel(self))
        self.addModels('WorkHurtFactors', CWorkHurtFactorsModel(self))
        self.modelWorkHurts.setFactorsModel(self.modelWorkHurtFactors)
        self.addModels('IdentificationDocs',  CIdentificationDocsModel(self))
        self.addModels('Policies',  CPoliciesModel(self))
        self.addModels('SocStatusDocs',  CSocStatusDocsModel(self))
        self.addModels('Contacts',    CClientContactsModel(self))
        self.addModels('Allergy', CAllergyModel(self))
        self.addModels('IntoleranceMedicament', CIntoleranceMedicamentModel(self))
        self.addModels('ClientIdentification', CClientIdentificationModel(self))
        self.addModels('ClientQuoting', CClientQuotingModel(self))
        self.addModels('ClientQuotingDiscussion', CClientQuotingDiscussionModel(self))
        self.addModels('Deposit', CDepositModel(self))
        self.addModels('ClientConsents', CClientConsentsInDocModel(self))
        self.addModels('ClientAddressesReg', CAddressesTableModel(self, CAddressesTableModel.typeReg))
        self.addModels('ClientAddressesLoc', CAddressesTableModel(self, CAddressesTableModel.typeLoc))

# ui
        self.setupUi(self)
        self.setWindowFlags(self.windowFlags() | Qt.WindowMaximizeButtonHint)
        self.setObjectName('ClientEditDialog')
        self.edtBirthDate.setHighlightRedDate(False)
        self.edtWorkOrganisationINN.setBackgroundRole(QtGui.QPalette.Window)
        self.edtWorkOrganisationOGRN.setBackgroundRole(QtGui.QPalette.Window)
        self.edtOKVEDName.setBackgroundRole(QtGui.QPalette.Window)

        self.tabPassport.setFocusProxy(self.chkRegKLADR)
        self.tabSocStatus.setFocusProxy(self.tblSocStatuses)
        self.tabAttach.setFocusProxy(self.tblAttaches)
        self.tabAttach.setEnabled(QtGui.qApp.userHasRight(urRegEditClientAttach))
        self.tabRelations.setFocusProxy(self.tblDirectRelations)
        self.tabWork.setFocusProxy(self.btnSelectWorkOrganisation)
        self.tabPassport.setFocusProxy(self.tblContacts)
        self.tabFeature.setFocusProxy(self.tblAllergy)
        self.tabFeature.setFocusProxy(self.tblIntoleranceMedicament)
        self.tabIdentification.setFocusProxy(self.tblClientIdentification)
        self.btnSearchPolicy.setEnabled(QtGui.qApp.identServiceEnabled())
        self.tabDeposit.setFocusProxy(self.tblDeposit)

# tables to rb and combo-boxes
        self.cmbDocType.setTable('rbDocumentType', True, 'group_id IN (SELECT id FROM rbDocumentTypeGroup WHERE code=\'1\')')
        self.cmbCompulsoryPolisType.setTable('rbPolicyType', True, u'isCompulsory')
        self.cmbCompulsoryPolisKind.setTable('rbPolicyKind', True)
        self.cmbVoluntaryPolisType.setTable('rbPolicyType', True, u'NOT isCompulsory')
        self.cmbVoluntaryPolisKind.setTable('rbPolicyKind', True)
        self.cmbSocStatusDocType.setTable('rbDocumentType', True, 'group_id IN (SELECT id FROM rbDocumentTypeGroup WHERE code IN (\'2\', \'5\', \'7\'))')
        self.cmbBloodType.setTable('rbBloodType', True)
        self.edtBloodTypeDate.canBeEmpty(True)
        self.cmbWorkOKVED.setAddNone(True)
        self.btnAttachedFiles.setTable('Client_FileAttach')

# assign models
        self.setModels(self.tblSocStatuses, self.modelSocStatuses, self.selectionModelSocStatuses)
        self.setModels(self.tblAttaches, self.modelAttaches, self.selectionModelAttaches)
        self.setModels(self.tblDirectRelations, self.modelDirectRelations, self.selectionModelDirectRelations)
        self.setModels(self.tblBackwardRelations, self.modelBackwardRelations, self.selectionModelBackwardRelations)
        self.setModels(self.tblWorkHurts, self.modelWorkHurts, self.selectionModelWorkHurts)
        self.setModels(self.tblWorkHurtFactors, self.modelWorkHurtFactors, self.selectionModelWorkHurtFactors)
        self.setModels(self.tblIdentificationDocs, self.modelIdentificationDocs, self.selectionModelIdentificationDocs)
        self.setModels(self.tblPolicies, self.modelPolicies, self.selectionModelPolicies)
        self.setModels(self.tblSocStatusDocs, self.modelSocStatusDocs, self.selectionModelSocStatusDocs)
        self.setModels(self.tblContacts, self.modelContacts, self.selectionModelContacts)
        self.setModels(self.tblAllergy, self.modelAllergy, self.selectionModelAllergy)
        self.setModels(self.tblIntoleranceMedicament, self.modelIntoleranceMedicament, self.selectionModelIntoleranceMedicament)
        self.setModels(self.tblClientIdentification,  self.modelClientIdentification,  self.selectionModelClientIdentification)
        self.setModels(self.tblClientQuoting, self.modelClientQuoting, self.selectionModelClientQuoting)
        self.setModels(self.tblClientQuotingDiscussion, self.modelClientQuotingDiscussion, self.selectionModelClientQuotingDiscussion)
        self.setModels(self.tblDeposit, self.modelDeposit, self.selectionModelDeposit)
        self.setModels(self.tblClientConsents, self.modelClientConsents, self.selectionModelClientConsents)
        self.setModels(self.tblAddressesReg, self.modelClientAddressesReg, self.selectionModelClientAddressesReg)
        self.setModels(self.tblAddressesLoc, self.modelClientAddressesLoc, self.selectionModelClientAddressesLoc)

# popup menus
        self.tblSocStatuses.addPopupDelRow()
        self.tblSocStatuses.addPopupRecordProperies()
        self.tblAttaches.addPopupDelRow()
        self.tblAttaches.addPopupRecordProperies()
        self.tblDirectRelations.addPopupDelRow()
        self.tblDirectRelations.addPopupRecordProperies()
        self.tblDirectRelations.addRelativeClientEdit()
        self.tblBackwardRelations.addPopupDelRow()
        self.tblBackwardRelations.addPopupRecordProperies()
        self.tblBackwardRelations.addRelativeClientEdit()
        self.tblWorkHurts.addPopupDelRow()
        self.tblWorkHurtFactors.addPopupDelRow()
        self.tblIdentificationDocs.addPopupDelRow()
        self.tblIdentificationDocs.addPopupRecordProperies()
        self.tblPolicies.addPopupDelRow()
        self.tblPolicies.addPopupRecordProperies()
        self.tblSocStatusDocs.addPopupDelRow()
        self.tblSocStatusDocs.addPopupRecordProperies()
        self.tblContacts.addPopupDelRow()
        self.tblContacts.addPopupRecordProperies()
        self.tblAllergy.addPopupDelRow()
        self.tblAllergy.addPopupRecordProperies()
        self.tblDeposit.addPopupDelRow()
        self.tblDeposit.addPopupRecordProperies()
        self.tblIntoleranceMedicament.addPopupDelRow()
        self.tblIntoleranceMedicament.addPopupRecordProperies()
        self.tblClientIdentification.addPopupDelRow()
        self.tblClientIdentification.addPopupRecordProperies()
        self.tblClientQuoting.addPopupSelectAllRow()
        self.tblClientQuoting.addPopupClearSelectionRow()
        self.actDelClientQuotingRows = QtGui.QAction(u'Удалить выделенное', self.tblClientQuoting)
        self.tblClientQuoting.popupMenu().addAction(self.actDelClientQuotingRows)
        self.tblClientConsents.addPopupDelRow()
        self.connect(self.actDelClientQuotingRows, SIGNAL('triggered()'), self.deleteClientQuotingRows)
#        self.tblClientQuoting.addPopupDelRow()
        self.tblClientQuoting.addPopupRecordProperies()
        self.actNewMessage = QtGui.QAction(u'Добавить сообщение', self.tblClientQuotingDiscussion)
        self.connect(self.actNewMessage, SIGNAL('triggered()'), self.newQuotaDiscussionEditor)
        self.actEditMessage = QtGui.QAction(u'Редактировать сообщение', self.tblClientQuotingDiscussion)
        self.connect(self.actEditMessage, SIGNAL('triggered()'), self.editQuotaDiscussionEditor)
        self.actDelMessage = QtGui.QAction(u'Удалить сообщение', self.tblClientQuotingDiscussion)
        self.connect(self.actDelMessage, SIGNAL('triggered()'), self.deleteQuotaDiscussionMessage)
        self.connect(self.tblAddressesReg.horizontalHeader(), SIGNAL('sectionClicked(int)'), self._setAddressOrderByColumn)
        self.connect(self.tblAddressesLoc.horizontalHeader(), SIGNAL('sectionClicked(int)'), self._setAddressOrderByColumn)
        self.tblClientQuotingDiscussion.addPopupAction(self.actNewMessage)
        self.tblClientQuotingDiscussion.addPopupAction(self.actEditMessage)
        self.tblClientQuotingDiscussion.addPopupAction(self.actDelMessage)
        self.tblAddressesReg.addPopupDelRow()
        self.tblAddressesLoc.addPopupDelRow()

# connects
#        self.connect(self.tblWorkHurts.selectionModel(), SIGNAL('currentChanged(QModelIndex, QModelIndex)'), self.onHurtCurrentChanged)

# default values
        self.cmbRegCity.setCode(QtGui.qApp.defaultKLADR())
        self.cmbLocCity.setCode(QtGui.qApp.defaultKLADR())
        self.edtBirthDate.setDate(QDate())
# etc
        self.btnCopyPrevAddress.setEnabled(bool(CClientEditDialog.prevAddress))
        self.btnCopyPrevWork.setEnabled(bool(CClientEditDialog.prevWork))
        self.setupDirtyCather()
        self.setIsDirty(False)
        self.edtLastName.setFocus(Qt.OtherFocusReason)
        self.lblInfo.setWordWrap(True)
        self.lblMKBText.setWordWrap(True)
        self.edtDocDate.setDate(QDate())
        self.edtDocDate.setDateRange(QDate(1900, 1, 1), QDate(2200, 12, 31))

        self.edtCompulsoryPolisBegDate.setDate(QDate())
        self.edtCompulsoryPolisBegDate.setDateRange(QDate(1993, 1, 1), QDate(2200, 12, 31))
        self.edtCompulsoryPolisEndDate.setDate(QDate())
        self.edtCompulsoryPolisEndDate.setDateRange(QDate(1993, 1, 1), QDate(2200, 12, 31))
        self.edtVoluntaryPolisBegDate.setDate(QDate())
        self.edtVoluntaryPolisBegDate.setDateRange(QDate(1993, 1, 1), QDate(2200, 12, 31))
        self.edtVoluntaryPolisEndDate.setDate(QDate())
        self.edtVoluntaryPolisEndDate.setDateRange(QDate(1993, 1, 1), QDate(2200, 12, 31))
        self.tblAddressesReg.setOrder(0)
        self.tblAddressesLoc.setOrder(0)
        self.regAddressInfo = {}
        self.locAddressInfo = {}
        self.clientHousesListDialog = {}
        self.currentClientId = 0
        self.clientSex = 0
        self.clientBirthDate = QDate()
        self.clientWorkOrgId = None
        self.clientPolicyInfoList = []
        self.isHBDialog = False
        self.isHBEditClientInfo = False
        self.orgId = QtGui.qApp.currentOrgId()
        self.clientRegHousesList = CClientHousesList(parent)
        self.clientLocHousesList = CClientHousesList(parent)
        #Рисуем кнопку РИС
        #Если в настройках регион равен Краснодарский край и активирована галочка "Интеграция РИС"
        self.risActive=False;
        if forceString(QtGui.qApp.preferences.appPrefs.get('provinceKLADR','00'))[:2]=='23'and forceBool(QtGui.qApp.preferences.appPrefs.get('RIS_enabled',QVariant(False))):
            try:
                url =  forceString(QtGui.qApp.preferences.appPrefs.get('RIS_serviceAddress',QVariant('')))
                mocode = forceString(QtGui.qApp.preferences.appPrefs.get('RIS_athLPUCode',QVariant('')))
                token  = forceString(QtGui.qApp.preferences.appPrefs.get('RIS_authToken',QVariant('')))
                # зачем это, если можно просто написать
                # try:
                #     from Exchange.RIS.CRisExchange import CRisExchange
                #     ...
                # except:
                #     ...
                exmod=__import__('Exchange.RIS.CRisExchange',fromlist=['CRisExchange'])
                CRisExchange= getattr(exmod, 'CRisExchange')
                self.ex = CRisExchange(url,mocode,token,'')
                self.risActive=True
                shortcut = QtGui.QShortcut(self)
                shortcut.setKey("Ctrl+R")
                self.connect(shortcut, SIGNAL("activated()"), self.on_btnRIS_clicked)
                self.btnRIS.setEnabled(True)
            except: #В случае любой ошибки (отсутвие библиотеки suds отключаем интеграцию)
                QtGui.qApp.logCurrentException()
                self.btnRIS.hide()
        else:
            self.btnRIS.hide()


    def _setAddressOrderByColumn(self, column):
        type = self.tabWidgetAddresses.currentIndex()
        if type:
            table = self.tblAddressesLoc
        else:
            table = self.tblAddressesReg
        table.setOrder(column)
        order = table.order() if table.order() else ['ClientAddress.createDatetime ASC']
        db = QtGui.qApp.db
        tableCA = db.table('ClientAddress')
        tablePerson = db.table('vrbPerson')
        tableQuery = tableCA.leftJoin(tablePerson, tablePerson['id'].eq(tableCA['createPerson_id']))
        idList = db.getDistinctIdList(tableQuery, 'ClientAddress.id', [tableCA['deleted'].eq(0), tableCA['type'].eq(type), tableCA['client_id'].eq(self.itemId())], order)
        table.setIdList(idList)


# WFT?
    def setHBEditClientInfoRight(self, isHBEditClientInfo, isHBDialog):
        self.isHBDialog = isHBDialog
        self.isHBEditClientInfo = isHBEditClientInfo

# WFT? у нас есть редактирование события из карточки пациента?
    def getHBUpdateEventRight(self):
        if self.isHBDialog:
            if not self.isHBEditClientInfo:
                QtGui.QMessageBox.warning( self,
                                           u'Внимание!',
                                           u'Нет права на редактирование события!',
                                           QtGui.QMessageBox.Ok,
                                           QtGui.QMessageBox.Ok)
                return False
        return True


    #Возвращает последний полис СМК
    def getPolicy(self,policy):
        hicId = self.cmbCompulsoryPolisCompany.value()
        if hicId:
            db = QtGui.qApp.db
            stmt="""select infisCode,smocode from Organisation
                    where Organisation.id='%s'"""%hicId
            query=db.query(stmt)
            while query.next():
                policy.smo_localcode=forceString(query.record().value('infisCode'))
                policy.smo_globalcode=forceString(query.record().value('smocode'))
                policy.policseria = forceString(self.edtCompulsoryPolisSerial.text())
                policy.policnum=forceString(self.edtCompulsoryPolisNumber.text())
                stmt="""select federalCode from rbPolicyKind where rbPolicyKind.id='%s'"""%forceString(self.cmbCompulsoryPolisKind.value())
                query=db.query(stmt)
                while query.next():
                    policy.polictype=forceString(query.record().value('federalCode'))
                policy.begindate=forceString(self.edtCompulsoryPolisBegDate.date().toString('yyyy-MM-dd'))
                policy.enddate=forceString(self.edtCompulsoryPolisEndDate.date().toString('yyyy-MM-dd'))


    #Устанавливает полис пациенту
    def setPolicy(self,globalcode,regionalcode,policyTypeId,policySerial,policyNumber,begindate,enddate):
        hicId=None
        #Ищем страховую компанию по коду ИНФИС
        db = QtGui.qApp.db
        if regionalcode:
            stmt="""select id from Organisation
                    where Organisation.`isInsurer`=1 and Organisation.`deleted`=0
                        and Organisation.`infisCode`='%s'"""%regionalcode
            query=db.query(stmt)
            if query.size()==1:
                while query.next():
                    hicId=query.record().value('id')
                    break
        #Если не нашли, то пробуем искать по коду СМО
        if globalcode and not hicId:
            stmt="""select id from Organisation
                    where Organisation.`isInsurer`=1 and Organisation.`deleted`=0
                        and Organisation.`smocode`='%s'"""%globalcode
            query=db.query(stmt)
            if query.size()==1:
                while query.next():
                    hicId=query.record().value('id')
                    break
        if hicId:
            self.cmbCompulsoryPolisCompany.setValue(hicId)
            if policySerial:
                self.edtCompulsoryPolisSerial.setText(policySerial)
            else:
                self.edtCompulsoryPolisSerial.clear()
            if policyNumber:
                self.edtCompulsoryPolisNumber.setText(policyNumber)
            else:
                self.edtCompulsoryPolisNumber.clear()
            self.cmbCompulsoryPolisType.setValue(0)
            stmt="""select id from rbPolicyKind where rbPolicyKind.`federalCode`='%s'"""%policyTypeId
            query=db.query(stmt)
            kindId=0
            while query.next():
                kindId=query.record().value('id')
                break
            self.cmbCompulsoryPolisKind.setValue(forceInt(kindId))
            self.syncPolicy(True)
            row = self.modelPolicies.getCurrentCompulsoryPolicyRow(policySerial, policyNumber)
            self.modelPolicies.setValue(row, 'begDate', toVariant(begindate))
            self.modelPolicies.setValue(row, 'endDate', toVariant(enddate))
            if not begindate:
                begindate = QDate()
            if not enddate:
                enddate = QDate()
            self.edtCompulsoryPolisBegDate.setDate(QDate(begindate))
            self.edtCompulsoryPolisEndDate.setDate(QDate(enddate))


    #Устанавливливает документ УЛ пациенту
    def setDoc(self,code,seria,number,passdate,enddate,docorg,note):
        documentType_id=0
        isnew=0;
        db = QtGui.qApp.db
        stmt="""select rbDocumentType.id from rbDocumentType
                inner join rbDocumentTypeGroup on rbDocumentTypeGroup.id = rbDocumentType.group_id and rbDocumentTypeGroup.code='1'
                where rbDocumentType.regionalCode='%s'"""%code
        query=db.query(stmt)
        while query.next():
            documentType_id=forceInt(query.record().value('id'))
            break
        if documentType_id:
            for item in self.modelIdentificationDocs.items():
                if forceInt(item.value('documentType_id'))==documentType_id:
                    record=item
                    break
            else:
                record = self.modelIdentificationDocs.getEmptyRecord()
                isnew=1
            record.setValue('documentType_id', QVariant(documentType_id))
            if seria:
                record.setValue('serial', forceString(seria))
            else:
                record.setValue('serial', forceString(''))
            if number:
                record.setValue('number', forceString(number))
            else:
                record.setValue('number', forceString(''))
            if passdate:
                record.setValue('date', QVariant(QDate(passdate)))
            else:
                record.setValue('date', QVariant(QDate()))
            if docorg:
                record.setValue('origin', forceString(docorg))
            else:
                record.setValue('origin', forceString(''))
            if isnew:
                self.modelIdentificationDocs.items().append(record)
            self.modelIdentificationDocs.emit(SIGNAL("layoutChanged()"))
            self.setDocumentRecord(record)

    #Возвращает социальный статус
    def getSocStatus(self,classId):
        for item in self.modelSocStatuses.items():
            if (forceInt(item.value('socStatusClass_id'))==classId):
                db = QtGui.qApp.db
                stmt="""select socCode from rbSocStatusType  where id=%s"""%forceInt(item.value('socStatusType_id'))
                query=db.query(stmt)
                while query.next():
                    return query.record().value('socCode')
        return None

    #Устанавливает соц. статус
    def setSocStatus(self,classId,code,docname=None,docnum=None,docser=None,issuedate=None,enddate=None,note=None):
        typeId=0
        db = QtGui.qApp.db
        stmt="""select t.id from `rbSocStatusType` as t
                inner join `rbSocStatusClassTypeAssoc` as a on  a.`class_id`=%s and a.type_id=t.id
                where t.`socCode`='%s'"""%(classId,code)
        query=db.query(stmt)
        while query.next():
            typeId=query.record().value('id')
            break
        isnew=0;
        for item in self.modelSocStatuses.items():
            if (forceInt(item.value('socStatusClass_id'))==classId and forceInt(item.value('socStatusType_id'))==typeId):
                record=item
                break
        else:
            record = self.modelSocStatuses.getEmptyRecord()
            isnew=1
        if typeId:
            record.setValue('socStatusClass_id', QVariant(classId))
            record.setValue('socStatusType_id', QVariant(typeId))
            #Для льгот
            if classId==1:
                if issuedate:
                    record.setValue('begDate', QVariant(QDate(issuedate)))
                else:
                    record.setValue('begDate', QVariant(QDate()))
                if enddate:
                    record.setValue('endDate', QVariant(QDate(enddate)))
                else:
                    record.setValue('endDate', QVariant(QDate()))
                if docnum:
                    record.setValue('number', forceString(docnum))
                else:
                    record.setValue('number', forceString(''))
                if docser:
                    record.setValue('serial', forceString(docser))
                else:
                     record.setValue('serial', forceString(''))
                if note:
                    record.setValue('origin', forceString(note))
                else:
                    record.setValue('origin', forceString(''))
                if docnum or  docser or note:
                    record.setValue('documentType_id', QVariant(230))
                    if issuedate:
                        record.setValue('date', QVariant(QDate(issuedate)))
            if isnew:
                self.modelSocStatuses.items().append(record)
            self.modelSocStatuses.emit(SIGNAL("layoutChanged()"))

    #Установка пациенту идентификатора РИС
    def setClientIdentification(self):
        searchInfo={}
        searchInfo['lname'] = self.edtLastName.text()
        searchInfo['fname'] = self.edtFirstName.text()
        searchInfo['pname'] = self.edtPatrName.text()
        searchInfo['birthdate'] = self.edtBirthDate.date().toString('yyyy-MM-dd')
        searchInfo['snils'] = unformatSNILS(self.edtSNILS.text())
        if self.cmbCompulsoryPolisKind.getValue()==3:
            searchInfo['enp']=self.edtCompulsoryPolisNumber.text()
        else:
            searchInfo['enp']=''
        #Ищем только по документу УЛ заполненному в основной части
        document={}
        docTypeIsEmpty   = not self.cmbDocType.value()
        docNumberIsEmpty = not forceStringEx(self.edtDocNumber.text())
        if not (docTypeIsEmpty and docNumberIsEmpty):
            docTypeCode = forceString(QtGui.qApp.db.translate('rbDocumentType',
                                                               'id',
                                                               self.cmbDocType.value(),
                                                               'regionalCode'
                                                             ))
            if docTypeCode:
                document['code'] = docTypeCode
                document['seria'] = unicode(self.edtDocSerialLeft.text()+' '+self.edtDocSerialRight.text()).strip()
                document['number'] = self.edtDocNumber.text()
        searchInfo['document']=document
        risCode=self.ex.searchPatient(searchInfo)
        if risCode:
        #Добавляем уникальный идентификатор, если такового нет
            isnew=0;
            for item in self.modelClientIdentification.items():
                if (forceInt(item.value('accountingSystem_id'))==14):
                    record=item
                    break
            else:
                record = self.modelClientIdentification.getEmptyRecord()
                isnew=1
            record.setValue('createDatetime', QVariant(QDate.currentDate()))
            record.setValue('createPerson_id', QVariant(QtGui.qApp.userId))
            record.setValue('modifyDatetime', QVariant(QDate.currentDate()))
            record.setValue('modifyPerson_id', QVariant(QtGui.qApp.userId))
            record.setValue('client_id', QVariant(self.currentClientId))
            record.setValue('identifier', QVariant(risCode))
            record.setValue('accountingSystem_id', QVariant(14))
            record.setValue('checkDate', QVariant(QDate.currentDate()))
            if isnew:
                self.modelClientIdentification.items().append(record)
            self.modelClientIdentification.emit(SIGNAL("layoutChanged()"))
        return risCode


    @pyqtSignature('')
    def on_btnRIS_clicked(self):
        if self.risActive:
            risCode=self.setClientIdentification()
        if risCode:
            #Получаем информацию о пациенте из РИС
            #СНИЛС
            patient = self.ex.getPatient(risCode)
            if (hasattr(patient,'snils')):
                if patient.snils:
                    self.edtSNILS.setText(patient.snils)
            #Дата рождения
            if (hasattr(patient,'birthdate')):
                if patient.birthdate:
                    self.edtBirthDate.setDate(QDate(patient.birthdate))
            #Пол
            if (hasattr(patient,'sex')):
                if patient.sex in (1, 2):
                    self.cmbSex.setCurrentIndex(patient.sex)
            #Социальный статус
            if (hasattr(patient,'status_code')):
                if len(patient.status_code):
                    self.setSocStatus(34,patient.status_code)
            #Образование
            if (hasattr(patient,'training_code')):
                if len(patient.training_code):
                    self.setSocStatus(3,patient.training_code)
            #Страховая принадлежность (берем только одну активную)
            if (hasattr(patient,'insurances')):
                for insurance in patient.insurances:
                    if  (hasattr(insurance,'enddate')):
                        if insurance.enddate and (QDate(insurance.enddate)>=QDate.currentDate()):
                            globalcode=insurance.smo_globalcode if hasattr(insurance,'smo_globalcode') else None
                            regionalcode=insurance.smo_localcode if hasattr(insurance,'smo_localcode') else None
                            policyTypeId=insurance.polictype if hasattr(insurance,'polictype') else None
                            policySerial=insurance.policseria if hasattr(insurance,'policseria') else None
                            policyNumber=insurance.policnum if hasattr(insurance,'policnum') else None
                            begindate=insurance.begindate if hasattr(insurance,'begindate') else None
                            enddate=insurance.enddate if hasattr(insurance,'enddate') else None
                            self.setPolicy(globalcode,regionalcode,policyTypeId,policySerial,policyNumber,begindate,enddate)
                            break
                    else:
                            globalcode=insurance.smo_globalcode if hasattr(insurance,'smo_globalcode') else None
                            regionalcode=insurance.smo_localcode if hasattr(insurance,'smo_localcode') else None
                            policyTypeId=insurance.polictype if hasattr(insurance,'polictype') else None
                            policySerial=insurance.policseria if hasattr(insurance,'policseria') else None
                            policyNumber=insurance.policnum if hasattr(insurance,'policnum') else None
                            begindate=insurance.begindate if hasattr(insurance,'begindate') else None
                            enddate=insurance.enddate if hasattr(insurance,'enddate') else None
                            self.setPolicy(globalcode,regionalcode,policyTypeId,policySerial,policyNumber,begindate,enddate)
            #Группа крови
            if (hasattr(patient,'bloodgroup_code')):
                self.cmbBloodType.setValue(forceInt(patient.bloodgroup_code))
            #Адреса (забираем только адрес регистрации и адрес проживания addresstype_code=5,6)
            if (hasattr(patient,'addresses')):
                for addr in patient.addresses:
                    if (hasattr(addr,'addresstype_code') and hasattr(addr,'address')): #Учитывать только действительные адреса
                        if addr.addresstype_code in ('5','6'):
                            if hasattr(addr.address,'kladrcode'): #Пытаемся поставить адрес по КЛАДР
                                ad=smartDict()
                                ad.KLADRCode = addr.address.kladrcode[:11]+'00'
                                ad.KLADRStreetCode = addr.address.kladrcode+'00'
                                ad.number = '%s'%addr.address.house if  hasattr(addr.address,'house') and addr.address.house else ''
                                ad.corpus = '%s'%addr.address.unit if  hasattr(addr.address,'unit') and addr.address.unit  else ''
                                ad.flat ='%s'% addr.address.flat if  hasattr(addr.address,'flat') and addr.address.flat else ''
                                if hasattr(addr.address,'freeaddress'):
                                    ad.freeInput = addr.address.freeaddress if addr.address.freeaddress else ''
                                    ad.freeInput+=u' д. %s'%addr.address.house if  hasattr(addr.address,'house') and addr.address.house  else ''
                                    ad.freeInput+=u' кор. %s'%addr.address.unit if  hasattr(addr.address,'unit') and addr.address.unit else ''
                                    ad.freeInput+=u' кв. %s'%addr.address.flat if  hasattr(addr.address,'flat')  and addr.address.flat else ''
                                if addr.addresstype_code =='5':
                                    self.__regAddress = ad
                                    self.setRegAddress(self.__regAddress,ad.freeInput)
                                else:
                                    self.__locAddress = ad
                                    self.setLocAddress(ad)
                            else:
                                 if hasattr(addr.address,'freeaddress'):
                                    self.__regAddress = None
                                    freeaddr = addr.address.freeaddress
                                    freeaddr+=u' д. %s'%addr.address.house if  hasattr(addr.address,'house') and addr.address.house  else ''
                                    freeaddr+=u' кор. %s'%addr.address.unit if  hasattr(addr.address,'unit') and addr.address.unit else ''
                                    freeaddr+=u' кв. %s'%addr.address.flat if  hasattr(addr.address,'flat') and addr.address.flat else ''
                                    self.setRegAddress(self.__regAddress,freeaddr)
            #Льготы
            if (hasattr(patient,'benefits')):
                for ben in patient.benefits:
                    if (hasattr(ben,'benefit_code')):

                        self.setSocStatus(1,ben.benefit_code,
                                                 ben.docname   if  hasattr(ben,'docname')   else None,
                                                 ben.doc_num   if  hasattr(ben,'doc_num')   else None,
                                                 ben.docser    if  hasattr(ben,'docser')    else None,
                                                 ben.issuedate if  hasattr(ben,'issuedate') else None,
                                                 ben.enddate   if  hasattr(ben,'enddate')   else None,
                                                 ben.note      if  hasattr(ben,'note')      else None);

            #Контакты
            if (hasattr(patient,'contacts')):
                for con in patient.contacts:
                    if hasattr(con,'code') and  hasattr(con,'value'):
                        if con.code in ('1','2','6'):
                            typeContactId=None
                            db = QtGui.qApp.db
                            stmt="""select id from `rbContactType` where code='%s'"""%con.code
                            query=db.query(stmt)
                            while query.next():
                                typeContactId=query.record().value('id')
                                break
                            if typeContactId:
                                #Ищем контакт такого же типа
                                isnew=0;
                                for item in self.modelContacts.items():
                                    if (forceInt(item.value('contactType_id'))==typeContactId):
                                        record=item
                                        break
                                else:
                                    record = self.modelContacts.getEmptyRecord()
                                    isnew=1
                                record.setValue('contactType_id', QVariant(typeContactId))
                                record.setValue('contact', forceString(con.value))
                                if isnew:
                                    self.modelContacts.items().append(record)
                                self.modelContacts.emit(SIGNAL("layoutChanged()"))
            #Гражданства
            if (hasattr(patient,'countries')):
                for cnt in patient.countries:
                        self.setSocStatus(8,u'м'+cnt)
            #Инвалидности
            if (hasattr(patient,'disablements')):
                pass
            #Документы УЛ
            if (hasattr(patient,'documents')):
                for doc in patient.documents:
                    if hasattr(doc,'code') and hasattr(doc,'number'):
                        self.setDoc(doc.code,doc.seria if hasattr(doc,'seria') else None,doc.number,
                                    doc.passdate if hasattr(doc,'passdate') else None,
                                    doc.enddate if hasattr(doc,'enddate') else None ,
                                    doc.docorg if hasattr(doc,'docorg') else None ,
                                    doc.note if hasattr(doc,'note') else None);

            #Особые случаи (пока не грузим так как некуда)
            if (hasattr(patient,'extracases')):
                pass
            #Представители пациента, грузим только в случае, если у нас есть этот представитель в текущей БД
            if (hasattr(patient,'spokesmen')):
                pass
            #Прикрепление
            #Разделяем прикрепление на свой/чужой, для чужих указывает ЛПУ, дату прикрепления, дату открепления
            #Тип прикрепления и тип открепления (в одном поле)
            #Для своих также указваем участок (подтягиваем подразделение прикрепления)
            #В случае когда у пациента в РИС прикрепление к чужой ЛПУ а локально к текущей, выдаем предупреждение
            #Что у пациента сменилось прикрепление
            if (hasattr(patient,'attachments')):
                pass

    def destroy(self):
        self.tblSocStatuses.setModel(None)
        self.tblAttaches.setModel(None)
        self.tblDirectRelations.setModel(None)
        self.tblBackwardRelations.setModel(None)
        self.tblWorkHurts.setModel(None)
        self.tblWorkHurtFactors.setModel(None)
        self.tblContacts.setModel(None)
        self.tblAllergy.setModel(None)
        self.tblDeposit.setModel(None)
        self.tblClientConsents.setModel(None)
        self.tblIntoleranceMedicament.setModel(None)
        self.tblClientIdentification.setModel(None)
        self.tblClientQuoting.setModel(None)
        self.tblClientQuotingDiscussion.setModel(None)

        del self.modelSocStatuses
        del self.modelAttaches
        del self.modelDirectRelations
        del self.modelBackwardRelations
        del self.modelWorkHurts
        del self.modelWorkHurtFactors
        del self.modelIdentificationDocs
        del self.modelPolicies
        del self.modelSocStatusDocs
        del self.modelContacts
        del self.modelAllergy
        del self.modelDeposit
        del self.modelClientConsents
        del self.modelIntoleranceMedicament
        del self.modelClientIdentification
        del self.modelClientQuoting
        del self.modelClientQuotingDiscussion


    def saveData(self):
        if QtGui.qApp.userHasRight(urRegTabReadRegistry) and not QtGui.qApp.userHasRight(urRegTabWriteRegistry):
            QtGui.QMessageBox.warning(self,
                            u'Внимание',
                            u'Право только на чтение!',
                            QtGui.QMessageBox.Ok,
                            QtGui.QMessageBox.Ok)
            return False
        if QtGui.qApp.userHasRight(urRegTabWriteRegistry):
            return self.checkDataEntered() and self.save()
        return False


    def save(self):
        try:
            db = QtGui.qApp.db
            db.transaction()
            try:
                record = self.getRecord()
                id = db.insertOrUpdate('Client', record)
                regAddressRecord, regAddress, regAddressRecordChanged = self.getAddressRecord(id, 0)
                if regAddressRecordChanged and regAddressRecord is not None:
                    db.insertOrUpdate('ClientAddress', regAddressRecord)
                locAddressRecord, locAddress, locAddressRecordChanged = self.getAddressRecord(id, 1)
                if locAddressRecordChanged and locAddressRecord is not None:
                    db.insertOrUpdate('ClientAddress', locAddressRecord)

                self.modelAttaches.saveItems(id)
                self.modelDirectRelations.saveItems(id)
                self.modelBackwardRelations.saveItems(id)
                self.modelAllergy.saveItems(id)
                self.modelDeposit.saveItems(id)
                self.modelClientConsents.saveItems(id)
                self.modelIntoleranceMedicament.saveItems(id)
                workRecord, work, workRecordChanged = self.getWorkRecord(id)
                if workRecordChanged and workRecord is not None:
                    workRecordId = db.insertOrUpdate('ClientWork', workRecord)
                elif workRecord is not None:
                    workRecordId = forceRef(workRecord.value('id'))
                else:
                    workRecordId = None
                if workRecordId is not None:
                    self.modelWorkHurts.saveItems(workRecordId)
                self.modelIdentificationDocs.saveItems(id)
                self.syncPolicies()
                self.modelPolicies.cleanUpEmptyItems()
                self.modelPolicies.saveItems(id)
                self.modelSocStatusDocs.saveItems(id)
                self.modelContacts.saveItems(id)

                if regAddress['useKLADR']:
                    self.modelClientQuoting.setNewRegCityCode(regAddress['KLADRCode'])
                    self.modelClientQuoting.setNewRegDistrictCode(getOKATO(regAddress['KLADRCode'], regAddress['KLADRStreetCode'], regAddress['number']))
                self.modelClientQuoting.saveItems(id)
                 #------------------------------------------------------------------------------------------------------
                try:
                    if self.risActive:
                        pInfo = self.ex.getAddPatientInfo();
                        #Проверяем, если ли у пациента идентификатор в РИС
                        RisID=0
                        for item in self.modelClientIdentification.items():
                            if (forceInt(item.value('accountingSystem_id'))==14):
                                RisID=forceInt(item.value('identifier'))
                                break
                        else:
                            RisID=self.setClientIdentification() #Если идентификатора нету, пытаемся его получить из РИС
                        #Заполняем сведения о пациенте
                        pInfo.lname = self.edtLastName.text()
                        pInfo.fname = self.edtFirstName.text()
                        pInfo.patrname = self.edtPatrName.text()
                        pInfo.birthdate = self.edtBirthDate.date().toString('yyyy-MM-dd')
                        pInfo.sex = self.cmbSex.currentIndex()
                        pInfo.snils = unformatSNILS(self.edtSNILS.text())
                        pInfo.status_code = forceString(self.getSocStatus(34))
                        pInfo.training_code = forceString(self.getSocStatus(3))
                        insurance = self.ex.getInsuranceType()
                        self.getPolicy(insurance)
                        if insurance.smo_localcode or insurance.smo_globalcode:
                            pInfo.insurances=[insurance]
                        bloodTypeId=self.cmbBloodType.getValue()
                        if bloodTypeId:
                            pInfo.bloodgroup_code = forceInt(bloodTypeId)
                        regAddr = self.ex.getAddressType()
                        locAddr = self.ex.getAddressType()
                        addr=[]
                        if self.chkRegKLADR.isChecked() and self.cmbRegStreet.code():
                            regAddr.kladrcode=forceString(self.cmbRegStreet.code())[:-2]
                            regAddr.house = forceString(self.edtRegHouse.text())
                            regAddr.unit = forceString(self.edtRegCorpus.text())
                            regAddr.flat = forceString(self.edtRegFlat.text())
                        else:
                            if len(self.edtRegFreeInput.text()):
                                regAddr.freeaddress=forceString(self.edtRegFreeInput.text())
                        if regAddr.kladrcode or regAddr.freeaddress:
                            pa = self.ex.getPAddressType();
                            pa.addresstype_code='5'
                            pa.address = regAddr
                            pa.isvalid=1
                            addr.append(pa)
                        if self.cmbLocStreet.code():
                            locAddr.kladrcode=forceString(self.cmbLocStreet.code())[:-2]
                            locAddr.house = forceString(self.edtLocHouse.text())
                            locAddr.unit  = forceString(self.edtLocCorpus.text())
                            locAddr.flat = forceString(self.edtLocFlat.text())
                            pal = self.ex.getPAddressType();
                            pal.addresstype_code='6'
                            pal.isvalid=1
                            pal.address = locAddr
                            addr.append(pal)
                        if addr:
                            pInfo.addresses=addr
                        #Льготы, гражданства
                        benefits=[]
                        countries=[]
                        for item in self.modelSocStatuses.items():
                            if (forceInt(item.value('socStatusClass_id')) in (1,8)):
                                db = QtGui.qApp.db
                                stmt="""select socCode from rbSocStatusType  where id=%s"""%forceInt(item.value('socStatusType_id'))
                                query=db.query(stmt)
                                while query.next():
                                    if  (forceInt(item.value('socStatusClass_id'))==1):
                                        ben = self.ex.getBenefitType()
                                        ben.benefit_code=forceString(query.record().value('socCode'))
                                        ben.docname=u'не указан'
                                        ben.doc_num=forceString(item.value('number'))
                                        ben.docser=forceString(item.value('serial'))
                                        ben.issuedate=QDate(forceDate(item.value('begDate'))).toString('yyyy-MM-dd')
                                        ben.note=forceString(item.value('origin'))
                                        ben.enddate=QDate(forceDate(item.value('endDate'))).toString('yyyy-MM-dd')
                                        benefits.append(ben)
                                    if (forceInt(item.value('socStatusClass_id'))==8):
                                        countries.append(forceString(query.record().value('socCode')).replace(u'м',''))
                        if countries:
                            pInfo.countries=countries
                        if benefits:
                            pInfo.benefits=benefits
                        #Контакты
                        contacts=[]
                        for item in self.modelContacts.items():
                            db = QtGui.qApp.db
                            stmt="""select code from `rbContactType` where id='%s'"""%forceInt(item.value('contactType_id'))
                            query=db.query(stmt)
                            while query.next():
                                contact=self.ex.getContactType()
                                contact.code=forceString(query.record().value('code'))
                                contact.value=forceString(item.value('contact'))
                                contact.ismain=0
                                contacts.append(contact)
                        if contacts:
                            pInfo.contacts=contacts
                        #Документы УЛ
                        documents=[]
                        codes=[]
                        for item in reversed(self.modelIdentificationDocs.items()):
                            doc = self.ex.getDocumentType()
                            db = QtGui.qApp.db
                            stmt="""select regionalCode from rbDocumentType where id='%s'"""%forceInt(item.value('documentType_id'))
                            query=db.query(stmt)
                            while query.next():
                                doc.code=forceString(query.record().value('regionalCode'))
                                if doc.code not in codes:
                                    codes.append(doc.code)
                                    doc.seria=forceString(item.value('serial'))
                                    doc.number=forceString(item.value('number'))
                                    doc.passdate=QDate(forceDate(item.value('date'))).toString('yyyy-MM-dd')
                                    doc.docorg=forceString(item.value('origin'))
                                    documents.append(doc)
                        if documents:
                            pInfo.documents=documents
                        if RisID:   #Если получили сведения об идентификаторе - тогда изменяем пациента
                            pInfo.patientCode=RisID
                            self.ex.changePatient(pInfo)
                        else:       #Если сведений о идентификаторе в МИС нет, и РИС такой пациент не найден-добавляем его
                            RisID=self.ex.addPatient(pInfo)
                        if RisID: #Если пациент успешно добавился, устанавливаем идентификатор РИС в МИС
                            isnew=0;
                            for item in self.modelClientIdentification.items():
                                if (forceInt(item.value('accountingSystem_id'))==14):
                                    record=item
                                    break
                            else:
                                record = self.modelClientIdentification.getEmptyRecord()
                                isnew=1
                            record.setValue('createDatetime', QVariant(QDate.currentDate()))
                            record.setValue('createPerson_id', QVariant(QtGui.qApp.userId))
                            record.setValue('modifyDatetime', QVariant(QDate.currentDate()))
                            record.setValue('modifyPerson_id', QVariant(QtGui.qApp.userId))
                            record.setValue('client_id', QVariant(self.currentClientId))
                            record.setValue('identifier', QVariant(RisID))
                            record.setValue('accountingSystem_id', QVariant(14))
                            record.setValue('checkDate', QVariant(QDate.currentDate()))
                            if isnew:
                                self.modelClientIdentification.items().append(record)
                except  Exception, e:
                    raise
                    #if e.fault.faultcode!="19":
                    #    raise
                #------------------------------------------------------------------------------------------------------
                self.modelSocStatuses.saveItems(id)
                self.modelClientIdentification.saveItems(id)
                self.btnAttachedFiles.saveItems(id)
                db.commit()
            except:
                db.rollback()
                QtGui.qApp.logCurrentException()
                raise
            self.setItemId(id)
            self.__regAddressRecord = regAddressRecord
            self.__regAddress = regAddress
            self.__locAddressRecord = locAddressRecord
            self.__locAddress = locAddress

            self.setIsDirty(False)
            CClientEditDialog.prevAddress = (smartDict(**regAddress),
                                             forceStringEx(regAddressRecord.value('freeInput')),
                                             smartDict(**locAddress),
                                            )
            CClientEditDialog.prevWork    = work
            QtGui.qApp.emitCurrentClientInfoChanged()
            return id
        except Exception, e:
            QtGui.qApp.logCurrentException()
            QtGui.QMessageBox.critical( self,
                                        u'',
                                        unicode(e),
                                        QtGui.QMessageBox.Close)
            return None

    def checkDeadConfirmation(self, clientId):
        db = QtGui.qApp.db
        tableEvent = db.table('Event')
        tableEventType = db.table('EventType')
        tableEventTypePurpose = db.table('rbEventTypePurpose')
        table = tableEvent.leftJoin(tableEventType, tableEvent['eventType_id'].eq(tableEventType['id']))
        table = table.leftJoin(tableEventTypePurpose, tableEventTypePurpose['id'].eq(tableEventType['purpose_id']))
        condDeath = [tableEvent['deleted'].eq(0),
                              tableEventType['deleted'].eq(0),
                              tableEvent['client_id'].eq(clientId),
                              tableEventTypePurpose['code'].eq(5)
                              ]
        recordDeath = db.getRecordEx(table, ['NULL'], condDeath)
        return True if recordDeath else False


    def setRecord(self, record):
        CItemEditorBaseDialog.setRecord(self, record)
        id = self.itemId()
        self.setClientId(id)
        self.edtLastName.setText(forceString(record.value('lastName')))
        self.edtFirstName.setText(forceString(record.value('firstName')))
        self.edtPatrName.setText(forceString(record.value('patrName')))
        self.edtBirthPlace.setText(forceString(record.value('birthPlace')))
        self.edtBirthDate.setDate(forceDate(record.value('birthDate')))
        deathDateTime = forceDateTime(record.value('deathDate'))
        if deathDateTime:
            self.chkDeathDate.setChecked(True)
            if self.checkDeadConfirmation(id):
                self.chkDeathDate.setEnabled(False)
            else:
                self.edtDeathDate.setEnabled(True)
                self.edtDeathTime.setEnabled(True)
        self.edtDeathDate.setDate(deathDateTime.date())
        self.edtDeathTime.setTime(deathDateTime.time())

        self.edtBirthTime.setTime(forceTime(record.value('birthTime')))
        self.edtBirthHeight.setValue(forceInt(record.value('birthHeight')))
        self.edtBirthWeight.setValue(forceInt(record.value('birthWeight')))
        self.edtBirthGestationalAge.setValue(forceInt(record.value('birthGestationalAge')))
        self.edtMenarhe.setValue(forceInt(record.value('menarhe')))
        self.edtMenoPausa.setValue(forceInt(record.value('menoPausa')))
        self.cmbSex.setCurrentIndex(forceInt(record.value('sex')))
        self.edtSNILS.setText(forceString(record.value('SNILS')))
        self.edtNotes.setPlainText(forceString(record.value('notes')))

        self.setRegAddressRecord(getClientAddress(id, 0))
        self.setLocAddressRecord(getClientAddress(id, 1))
        self.setDocumentRecord(getClientDocument(id))
        workRecord = getClientWork(id)
        self.setWorkRecord(workRecord)
        if workRecord:
            self.modelWorkHurts.loadItems(forceRef(workRecord.value('id')))
        else:
            self.modelWorkHurts.clearItems()
        self.modelSocStatuses.loadItems(id)
        self.modelAttaches.loadItems(id)
        self.modelDirectRelations.loadItems(id)
        self.modelDirectRelations.setClientId(id)
        self.modelDirectRelations.setRegAddressInfo(self.getParamsRegAddress())
        self.modelDirectRelations.setLogAddressInfo(self.getParamsLocAddress())
        self.modelDirectRelations.setDirectRelationFilter(self.cmbSex.currentIndex())
        self.modelBackwardRelations.loadItems(id)
        self.modelBackwardRelations.setClientId(id)
        self.modelBackwardRelations.setRegAddressInfo(self.getParamsRegAddress())
        self.modelBackwardRelations.setLogAddressInfo(self.getParamsLocAddress())
        self.modelBackwardRelations.setBackwardRelationFilter(self.cmbSex.currentIndex())
        self.modelIdentificationDocs.loadItems(id)
        self.modelPolicies.loadItems(id)
#        self.modelPolicies.cleanUpEmptyItems()
        self.on_modelPolicies_policyChanged()
        self.modelSocStatusDocs.loadItems(id)
        self.modelContacts.loadItems(id)
        self.cmbBloodType.setValue(forceRef(record.value('bloodType_id')))
        self.edtBloodTypeDate.setDate(forceDate(record.value('bloodDate')))
        self.edtBloodTypeNotes.setText(forceString(record.value('bloodNotes')))
        self.modelAllergy.loadItems(id)
        self.modelDeposit.loadItems(id)
        self.modelClientConsents.setClientId(id)
        self.modelClientConsents.loadItems(id)
        self.modelIntoleranceMedicament.loadItems(id)
        self.modelClientIdentification.loadItems(id)
        self.modelClientQuoting.loadItems(id)
        if self.__regAddress:
            self.modelClientQuoting.setRegCityCode(self.__regAddress['KLADRCode'])
            self.modelClientQuoting.setDistrictCode(getOKATO(self.__regAddress['KLADRCode'], self.__regAddress['KLADRStreetCode'], self.__regAddress['number']))
        self.setSelectedQuota()
        self.setIsDirty(False)
        self.regAddressInfo = {}
        self.locAddressInfo = {}
        self.on_btnUpdateFactSum_clicked()
        self.btnAttachedFiles.loadItems(id)


    def setClientId(self, clientId):
        self.clientWorkOrgId = None
        self.currentClientId = clientId if clientId else 0
        if self.currentClientId:
            self.clientWorkOrgId = getClientWork(self.currentClientId)


    def getClientId(self):
        return self.currentClientId


    @pyqtSignature('int')
    def on_cmbSex_currentIndexChanged(self, sex):
        self.modelDirectRelations.setDirectRelationFilter(sex)
        self.modelBackwardRelations.setBackwardRelationFilter(sex)
        self.clientSex = sex


    @pyqtSignature('QDate')
    def on_edtBirthDate_dateChanged(self, date):
        self.clientBirthDate = date

    @pyqtSignature('')
    def on_btnUpdateFactSum_clicked(self):
        self.modelDeposit.factSum = []
        contractIdOfDateList = {}
        for row, record in enumerate(self.modelDeposit._items):
            contractId = forceRef(record.value('contract_id'))
            contractDate = forceDate(record.value('contractDate'))
            dateList = contractIdOfDateList.get(contractId, [])
            if contractDate and contractDate not in dateList:
                dateList.append(pyDate(contractDate))
                contractIdOfDateList[contractId] = dateList

        for row, record in enumerate(self.modelDeposit._items):
            contractId = forceRef(record.value('contract_id'))
            contractDate = forceDate(record.value('contractDate'))
            dateList = contractIdOfDateList.get(contractId, [])
            dateList.sort()
            lenDateList = len(dateList)
            if lenDateList > 1:
                begDateContract = QDate(dateList[0])
                endDateContract = QDate(dateList[lenDateList-1])
            else:
                begDateContract = contractDate
                endDateContract = 0

            sumAction = 0.0
            sumItem = 0.0
            currentDate = QDate.currentDate()
            clientAge = calcAgeTuple(self.clientBirthDate, currentDate)
            if contractId:
                db = QtGui.qApp.db
                tableAction = db.table('Action')
                tableEvent = db.table('Event')
                tableAccountItem = db.table('Account_Item')

            #  payStatus != 0
                cond = [tableAction['deleted'].eq(0),
                        tableEvent['deleted'].eq(0),
                        tableAccountItem['deleted'].eq(0),
                        tableAction['contract_id'].eq(contractId),
                        tableAction['payStatus'].ne(0),
                        tableAccountItem['refuseType_id'].isNull()
                        ]
                if begDateContract:
                    cond.append(tableAction['begDate'].dateGe(begDateContract))
                if endDateContract:
                    cond.append(tableAction['begDate'].dateLt(endDateContract))
                if self.currentClientId:
                    cond.append(tableEvent['client_id'].eq(self.currentClientId))
                table = tableEvent.innerJoin(tableAction, tableEvent['id'].eq(tableAction['event_id']))
                table = table.innerJoin(tableAccountItem, tableAction['id'].eq(tableAccountItem['action_id']))
                records = db.getRecordListGroupBy(table, u'SUM(Account_Item.sum) AS sumAction, Action.contract_id', cond, u'Action.contract_id')
                for newRecord in records:
                    contractId = forceRef(newRecord.value('contract_id'))
                    sumAction = forceDouble(newRecord.value('sumAction'))

            #  payStatus == 0
                tableActionType = db.table('ActionType')
                tableActionTypeService = db.table('ActionType_Service')
                tableRBService = db.table('rbService')
                tableContractTariff = db.table('Contract_Tariff')
                cols = u'''SUM(IF(Action.amount > Contract_Tariff.amount AND Contract_Tariff.amount != 0, Contract_Tariff.amount * Contract_Tariff.price,
                Action.amount * Contract_Tariff.price)) AS sumItem, Action.contract_id'''
                cond = [tableAction['deleted'].eq(0),
                        tableEvent['deleted'].eq(0),
                        tableActionType['deleted'].eq(0),
                        tableContractTariff['deleted'].eq(0),
                        tableAction['contract_id'].eq(contractId),
                        tableContractTariff['master_id'].eq(contractId),
                        tableAction['payStatus'].eq(0),
                        tableContractTariff['tariffType'].eq(CTariff.ttActionAmount),
                        tableActionType['id'].eq(tableActionTypeService['master_id'])
                        ]
                if begDateContract:
                    cond.append(tableAction['begDate'].dateGe(begDateContract))
                if endDateContract:
                    cond.append(tableAction['begDate'].dateLt(endDateContract))
                if clientAge:
                    cond.append(db.joinOr([tableContractTariff['age'].eq(''), tableContractTariff['age'].ge(clientAge[3])]))
                if self.currentClientId:
                    cond.append(tableEvent['client_id'].eq(self.currentClientId))
                cond.append(db.joinOr([tableContractTariff['sex'].eq(0), tableContractTariff['sex'].eq(self.clientSex)]))
                table = tableEvent.innerJoin(tableAction, tableEvent['id'].eq(tableAction['event_id']))
                table = table.innerJoin(tableActionType, tableAction['actionType_id'].eq(tableActionType['id']))
                table = table.innerJoin(tableActionTypeService, tableAction['finance_id'].eq(tableActionTypeService['finance_id']))
                table = table.innerJoin(tableRBService, tableActionTypeService['service_id'].eq(tableRBService['id']))
                table = table.innerJoin(tableContractTariff, tableRBService['id'].eq(tableContractTariff['service_id']))
                records = db.getRecordListGroupBy(table, cols, cond, u'Action.contract_id')
                for newRecord in records:
                    contractId = forceRef(newRecord.value('contract_id'))
                    sumItem = forceDouble(newRecord.value('sumItem'))
            self.modelDeposit.factSum.append(sumAction + sumItem)

        self.getFactSumList()
        self.modelDeposit.reset()


    def getFactSumList(self):
        lenItems = len(self.modelDeposit._items)
        lenFactSum = len(self.modelDeposit.factSum)
        diff = lenFactSum-lenItems
        if diff < 0:
            self.modelDeposit.factSum.extend([0]*(-diff))
        elif diff > 0:
            self.modelDeposit.factSum.extend([0]*diff)


    def setRegAddressRecord(self, record):
        self.__regAddressRecord = record
        if record:
            addressId = forceRef(record.value('address_id'))
#            self.chkRegKLADR.setChecked(addressId is not None)
            if addressId:
                self.__regAddress = getAddress(addressId)
            else:
                self.__regAddress = None
            self.setRegAddress(self.__regAddress, forceString(record.value('freeInput')))
        else:
            self.chkRegKLADR.setChecked(False)
            self.__regAddress = None
            self.setRegAddress(self.__regAddress, '')



    def setRegAddress(self, regAddress, freeInput):
        self.chkRegKLADR.setChecked(regAddress!=None)
        self.edtRegFreeInput.setText(freeInput)
        self.cmbCompulsoryPolisCompany._popup.setRegAdress(regAddress)
        if regAddress:
            self.cmbRegCity.setCode(regAddress.KLADRCode)
            self.cmbRegStreet.setCity(regAddress.KLADRCode)
            self.cmbRegStreet.setCode(regAddress.KLADRStreetCode)
            self.edtRegHouse.setText(regAddress.number)
            self.edtRegCorpus.setText(regAddress.corpus)
            self.edtRegFlat.setText(regAddress.flat)
        else:
            self.cmbRegCity.setCode(QtGui.qApp.defaultKLADR())
            self.cmbRegStreet.setCity(QtGui.qApp.defaultKLADR())
            self.cmbRegStreet.setCode('')
            self.edtRegHouse.setText('')
            self.edtRegCorpus.setText('')
            self.edtRegFlat.setText('')


    def setRegAddressRelation(self, regAddress):
        if regAddress:
            self.edtRegFreeInput.setText(regAddress.get('regFreeInput', u''))
            self.cmbRegCity.setCode(regAddress.get('regCity', u''))
            self.cmbRegStreet.setCode(regAddress.get('regStreet', u''))
            self.edtRegHouse.setText(regAddress.get('regHouse', u''))
            self.edtRegCorpus.setText(regAddress.get('regCorpus', u''))
            self.edtRegFlat.setText(regAddress.get('regFlat', u''))


    def setLocAddressRecord(self, record):
        self.__locAddressRecord = record
        if record:
            addressId = forceRef(record.value('address_id'))
            self.__locAddress = getAddress(addressId)
        else:
            self.__locAddress = None
        self.setLocAddress(self.__locAddress)


    def setLocAddress(self, locAddress):
        if locAddress:
            self.cmbLocCity.setCode(locAddress.KLADRCode)
            self.cmbLocStreet.setCity(locAddress.KLADRCode)
            self.cmbLocStreet.setCode(locAddress.KLADRStreetCode)
            self.edtLocHouse.setText(locAddress.number)
            self.edtLocCorpus.setText(locAddress.corpus)
            self.edtLocFlat.setText(locAddress.flat)
        else:
            self.cmbLocCity.setCode(QtGui.qApp.defaultKLADR())
            self.cmbLocStreet.setCity(QtGui.qApp.defaultKLADR())
            self.cmbLocStreet.setCode('')
            self.edtLocHouse.setText('')
            self.edtLocCorpus.setText('')
            self.edtLocFlat.setText('')


    def setLocAddressRelation(self, locAddress):
        if locAddress:
            self.cmbLocCity.setCode(locAddress.get('regCity', u''))
            self.cmbLocStreet.setCode(locAddress.get('regStreet', u''))
            self.edtLocHouse.setText(locAddress.get('regHouse', u''))
            self.edtLocCorpus.setText(locAddress.get('regCorpus', u''))
            self.edtLocFlat.setText(locAddress.get('regFlat', u''))


    def setDocumentRecord(self, record):
        self.__documentRecord = record
        if record:
            documentTypeId = forceRef(record.value('documentType_id'))
            serial = forceString(record.value('serial'))
            number = forceString(record.value('number'))
            date   = forceDate(record.value('date'))
            origin = forceString(record.value('origin'))
        else:
            documentTypeId = None
            serial = ''
            number = ''
            date   = QDate()
            origin = ''

        documentTypeDescr = getDocumentTypeDescr(documentTypeId)
        serialLeft, serialRight = documentTypeDescr.splitDocSerial(serial)

        bs = self.edtDocSerialLeft.blockSignals(True)
        try:
            self.edtDocSerialLeft.blockSignals(True)
            self.edtDocSerialRight.blockSignals(True)
            self.edtDocNumber.blockSignals(True)
            self.edtDocDate.blockSignals(True)
            self.edtDocOrigin.blockSignals(True)

            self.edtDocSerialLeft.setRegExp('')
            self.edtDocSerialRight.setRegExp('')
            self.edtDocSerialLeft.setText(serialLeft)
            self.edtDocSerialRight.setText(serialRight)
            self.edtDocNumber.setRegExp('')
            self.edtDocNumber.setText(number)
            self.edtDocDate.setDate(date)
            self.edtDocOrigin.setText(origin)
            self.cmbDocType.setValue(documentTypeId)
        finally:
            self.edtDocSerialLeft.blockSignals(bs)
            self.edtDocSerialRight.blockSignals(bs)
            self.edtDocNumber.blockSignals(bs)
            self.edtDocDate.blockSignals(bs)
            self.edtDocOrigin.blockSignals(bs)



    def setPolicyRecord(self, record, isCompulsory):
        if record:
            serial = forceString(record.value('serial'))
            number = forceString(record.value('number'))
            insurer = forceRef(record.value('insurer_id'))
            insurerArea = forceString(QtGui.qApp.db.translate('Organisation', 'id', insurer, 'area'))
            polisType = forceRef(record.value('policyType_id'))
            polisKind = forceRef(record.value('policyKind_id'))
            name = forceString(record.value('name'))
            note = forceString(record.value('note'))
            begDate = forceDate(record.value('begDate'))
            endDate = forceDate(record.value('endDate'))
        else:
            serial = ''
            number = ''
            insurer = None
            insurerArea = ''
            polisType = None
            polisKind = None
            name = ''
            note = ''
            begDate = QDate()
            endDate = QDate()
        if isCompulsory:
            self.edtCompulsoryPolisSerial.setText(serial)
            self.edtCompulsoryPolisNumber.setText(number)
            self.updateCompulsoryPolicyCompanyArea([insurerArea])
            self.cmbCompulsoryPolisCompany.setValue(insurer)
            self.cmbCompulsoryPolisType.setValue(polisType)
            self.cmbCompulsoryPolisKind.setValue(polisKind)
            self.edtCompulsoryPolisName.setText(name)
            self.edtCompulsoryPolisNote.setText(note)
            self.edtCompulsoryPolisBegDate.setDate(begDate)
            self.edtCompulsoryPolisEndDate.setDate(endDate)
        else:
            self.edtVoluntaryPolisSerial.setText(serial)
            self.edtVoluntaryPolisNumber.setText(number)
            self.updateVoluntaryPolicyCompanyArea([insurerArea])
            self.cmbVoluntaryPolisCompany.setValue(insurer)
            self.cmbVoluntaryPolisType.setValue(polisType)
            self.cmbVoluntaryPolisKind.setValue(polisKind)
            self.edtVoluntaryPolisName.setText(name)
            self.edtVoluntaryPolisNote.setText(note)
            self.edtVoluntaryPolisBegDate.setDate(begDate)
            self.edtVoluntaryPolisEndDate.setDate(endDate)


    def setWorkRecord(self, record):
        self.__workRecord = record
        if record:
            self.cmbWorkOrganisation.setValue(forceRef(record.value('org_id')))
            self.updateWorkOrganisationInfo()
            self.edtWorkOrganisationFreeInput.setText(forceString(record.value('freeInput')))
            self.edtWorkPost.setText(forceString(record.value('post')))
    #        self.edtWorkOKVED.setText(forceString(record.value('OKVED')))
            self.cmbWorkOKVED.setText(forceString(record.value('OKVED')))
            self.edtWorkStage.setValue(forceInt(record.value('stage')))
        else:
            self.cmbWorkOrganisation.setValue(None)
            self.updateWorkOrganisationInfo()
            self.edtWorkOrganisationFreeInput.setText('')
            self.edtWorkPost.setText('')
            self.cmbWorkOKVED.setText('')
            self.edtWorkStage.setValue(0)


    def setWork(self, work):
        if work:
            self.cmbWorkOrganisation.setValue(work['organisationId'])
            self.updateWorkOrganisationInfo()
            self.edtWorkOrganisationFreeInput.setText(work['freeInput'])
            self.edtWorkPost.setText(work['post'])
            self.cmbWorkOKVED.setText(work['OKVED'])
            self.edtWorkStage.setValue(work['stage'])
        else:
            self.cmbWorkOrganisation.setValue(None)
            self.updateWorkOrganisationInfo()
            self.edtWorkOrganisationFreeInput.setText('')
            self.edtWorkPost.setText('')
            self.cmbWorkOKVED.setText('')
            self.edtWorkStage.setValue(0)


    def getParamsRegAddress(self):
        self.regAddressInfo = {}
        self.regAddressInfo['regFreeInput'] = self.edtRegFreeInput.text()
        self.regAddressInfo['addressType'] = 0
        self.regAddressInfo['regCity'] = self.cmbRegCity.code()
        self.regAddressInfo['regStreet'] = self.cmbRegStreet.code()
        self.regAddressInfo['regHouse'] = self.edtRegHouse.text()
        self.regAddressInfo['regCorpus'] = self.edtRegCorpus.text()
        self.regAddressInfo['regFlat'] = self.edtRegFlat.text()
        return self.regAddressInfo


    def setParamsRegAddress(self, regAddressInfo):
        self.regAddressInfo = regAddressInfo


    def getParamsLocAddress(self):
        self.locAddressInfo = {}
        self.locAddressInfo['addressType'] = 1
        self.locAddressInfo['regCity'] = self.cmbLocCity.code()
        self.locAddressInfo['regStreet'] = self.cmbLocStreet.code()
        self.locAddressInfo['regHouse'] = self.edtLocHouse.text()
        self.locAddressInfo['regCorpus'] = self.edtLocCorpus.text()
        self.locAddressInfo['regFlat'] = self.edtLocFlat.text()
        return self.locAddressInfo


    def setParamsLocAddress(self, locAddressInfo):
        self.locAddressInfo = locAddressInfo


    def setClientDialogInfo(self, info):
        if info:
            self.edtLastName.setText(info.get('lastName', ''))
            self.edtFirstName.setText(info.get('firstName', ''))
            self.edtPatrName.setText(info.get('patrName', ''))
            birthDate = info.get('birthDate', None)
            if birthDate:
                self.edtBirthDate.setDate(birthDate)
            self.cmbSex.setCurrentIndex(info.get('sex', 0))
            self.edtSNILS.setText(info.get('SNILS', ''))
            docTypeId = info.get('docType', None)
            if docTypeId:
                self.cmbDocType.setValue(docTypeId)
                self.edtDocSerialLeft.setText(forceString(info.get('serialLeft', '')))
                self.edtDocSerialRight.setText(forceString(info.get('serialRight', '')))
                self.edtDocNumber.setText(forceString(info.get('docNumber', '')))
            addressType = info.get('addressType', None)
            if  addressType == 0:
                self.cmbRegCity.setCode(info.get('regCity', ''))
                self.cmbRegStreet.setCode(info.get('regStreet', ''))
                self.edtRegHouse.setText(info.get('regHouse', ''))
                self.edtRegCorpus.setText(info.get('regCorpus', ''))
                self.edtRegFlat.setText(info.get('regFlat', ''))
            elif addressType == 1:
                self.cmbLocCity.setCode(info.get('regCity', ''))
                self.cmbLocStreet.setCode(info.get('regStreet', ''))
                self.edtLocHouse.setText(info.get('regHouse', ''))
                self.edtLocCorpus.setText(info.get('regCorpus', ''))
                self.edtLocFlat.setText(info.get('regFlat', ''))
            extendedPolicyInformation = info.get('extendedPolicyInformation') or {}
            polisSerial = info.get('polisSerial', '')
            polisNumber = info.get('polisNumber', '') or extendedPolicyInformation.get('number', '')
            polisCompany = info.get('polisCompany', None) or extendedPolicyInformation.get('insurerId', '')
            polisType = info.get('polisType', None)
            polisKind = extendedPolicyInformation.get('polisKind', '')
            polisTypeName = forceString(info.get('polisTypeName', ''))
            if polisNumber:
                if u'ДМС' in polisTypeName:
                    self.edtVoluntaryPolisSerial.setText(polisSerial)
                    self.edtVoluntaryPolisNumber.setText(polisNumber)
                    self.cmbVoluntaryPolisCompany.setValue(polisCompany)
                    self.cmbVoluntaryPolisType.setValue(polisType)
                else:
                    self.edtCompulsoryPolisSerial.setText(polisSerial)
                    self.edtCompulsoryPolisNumber.setText(polisNumber)
                    self.cmbCompulsoryPolisCompany.setValue(polisCompany)
                    self.cmbCompulsoryPolisType.setValue(polisType)
                    self.cmbCompulsoryPolisKind.setValue(polisKind)
                    if extendedPolicyInformation.get('begDate'):
                        self.edtCompulsoryPolisBegDate.setDate(extendedPolicyInformation.get('begDate'))
                    if extendedPolicyInformation.get('endDate'):
                        self.edtCompulsoryPolisEndDate.setDate(extendedPolicyInformation.get('endDate'))

            contact = info.get('contact', '')
            typeContactId = forceRef(QtGui.qApp.db.translate('rbContactType', 'code', 1, 'id'))
            self.modelContacts.setDialogContact(typeContactId, contact)


    def getRecord(self):
        record = CItemEditorBaseDialog.getRecord(self)
        record.setValue('lastName',  toVariant(nameCase(forceStringEx(self.edtLastName.text()))))
        record.setValue('firstName', toVariant(nameCase(forceStringEx(self.edtFirstName.text()))))
        record.setValue('patrName',  toVariant(nameCase(forceStringEx(self.edtPatrName.text()))))
        record.setValue('birthPlace',  toVariant(nameCase(forceStringEx(self.edtBirthPlace.text()))))
        record.setValue('birthDate', toVariant(self.edtBirthDate.date()))
        record.setValue('birthTime', toVariant(self.edtBirthTime.time()))
        record.setValue('birthHeight',toVariant(self.edtBirthHeight.value()))
        record.setValue('birthWeight',toVariant(self.edtBirthWeight.value()))
        record.setValue('birthGestationalAge', toVariant(self.edtBirthGestationalAge.value()))
        record.setValue('menarhe', toVariant(self.edtMenarhe.value()))
        record.setValue('menoPausa', toVariant(self.edtMenoPausa.value()))
        record.setValue('sex',       toVariant(self.cmbSex.currentIndex()))
        record.setValue('SNILS',     toVariant(forceStringEx(self.edtSNILS.text()).replace('-','').replace(' ','')))
        record.setValue('notes',     toVariant(forceStringEx(self.edtNotes.toPlainText())))
        record.setValue('bloodType_id', toVariant(self.cmbBloodType.value()))
        record.setValue('bloodDate', toVariant(self.edtBloodTypeDate.date()))
        record.setValue('bloodNotes', toVariant(forceStringEx(self.edtBloodTypeNotes.text())))
        record.setValue('deathDate', toVariant(QDateTime(self.edtDeathDate.date(), self.edtDeathTime.time()) if self.chkDeathDate.isChecked() else None))
        return record


    def getAddress(self, addressType):
        if addressType == 0:
            return { 'useKLADR'         : self.chkRegKLADR.isChecked(),
                     'KLADRCode'        : self.cmbRegCity.code(),
                     'KLADRStreetCode'  : self.cmbRegStreet.code() if self.cmbRegStreet.code() else '', # без этого наростают адреса
                     'number'           : forceStringEx(self.edtRegHouse.text()),
                     'corpus'           : forceStringEx(self.edtRegCorpus.text()),
                     'flat'             : forceStringEx(self.edtRegFlat.text()),
                     'freeInput'        : forceStringEx(self.edtRegFreeInput.text())}
        else:
            return { 'useKLADR'         : True,
                     'KLADRCode'        : self.cmbLocCity.code(),
                     'KLADRStreetCode'  : self.cmbLocStreet.code() if self.cmbLocStreet.code() else '', # без этого наростают адреса
                     'number'           : forceStringEx(self.edtLocHouse.text()),
                     'corpus'           : forceStringEx(self.edtLocCorpus.text()),
                     'flat'             : forceStringEx(self.edtLocFlat.text()),
                     'freeInput'        : ''}


    def getAddressRecord(self,  clientId, addressType):
        address = self.getAddress(addressType)
        if address['useKLADR']:
            addressId = getAddressId(address)
        else:
            addressId = None
        oldAddressRecord = self.__regAddressRecord if addressType == 0 else self.__locAddressRecord

        if oldAddressRecord is not None:
            recordChanged = addressId != forceRef(oldAddressRecord.value('address_id')) or address['freeInput'] != forceString(oldAddressRecord.value('freeInput'))
        else:
            recordChanged = True

        if recordChanged:
            record = QtGui.qApp.db.record('ClientAddress')
            record.setValue('client_id',  toVariant(clientId))
            record.setValue('type',       toVariant(addressType))
            record.setValue('address_id', toVariant(addressId))
            record.setValue('freeInput',  toVariant(address['freeInput']))
        else:
            record = oldAddressRecord

        return record, address, recordChanged


    def getWorkRecord(self, clientId):
        organisationId = self.cmbWorkOrganisation.value()
        freeInput = u'' if organisationId else forceStringEx(self.edtWorkOrganisationFreeInput.text())
        post  = forceStringEx(self.edtWorkPost.text())
        OKVED = forceStringEx(self.cmbWorkOKVED.text())
        stage = self.edtWorkStage.value()

        work = {'organisationId': organisationId,
                'freeInput'     : freeInput,
                'post'          : post,
                'OKVED'         : OKVED,
                'stage'         : stage}

        if self.__workRecord is not None:
            recordChanged = (
                organisationId  != forceRef(self.__workRecord.value('org_id')) or
                freeInput       != forceString(self.__workRecord.value('freeInput')) or
                post            != forceString(self.__workRecord.value('post')) or
                OKVED           != forceString(self.__workRecord.value('OKVED')) or
                stage           != forceInt(self.__workRecord.value('stage'))
                )
        else:
            recordChanged=True

        if recordChanged:
            record = QtGui.qApp.db.record('ClientWork')
            record.setValue('client_id',    toVariant(clientId))
            record.setValue('org_id',       toVariant(organisationId))
            record.setValue('freeInput',    toVariant(freeInput))
            record.setValue('post',         toVariant(post))
            record.setValue('OKVED',        toVariant(OKVED))
            record.setValue('stage',        toVariant(stage))
        else:
            record = self.__workRecord

        return record, work, recordChanged


#    def replacePatient(self, patientId):
#        db = QtGui.qApp.db
#        patientRecord = db.getRecord(tblPatients, '*', patientId)
#        self.setRecord(patientRecord)


    def checkDataEntered(self):
        self.syncPolicies()
        birthDate = self.edtBirthDate.date()
        lastName = forceStringEx(self.edtLastName.text())
        isAnonymous = lastName.startswith('*')
        tinyChild = birthDate and birthDate.addMonths(2)>=QDate.currentDate()
        result = True
        result = result and (lastName or self.checkInputMessage(u'фамилию', False, self.edtLastName))
        if not isAnonymous:
            result = result and (isNameValid(lastName) or self.checkInputMessage(u'допустимую фамилию', False, self.edtLastName))
            if tinyChild:
                result = result and (isNameValid(self.edtFirstName.text()) or self.checkInputMessage(u'допустимое имя', True, self.edtFirstName))
                result = result and (isNameValid(self.edtPatrName.text()) or self.checkInputMessage(u'допустимое отчество', True, self.edtPatrName))
            else:
                result = result and (forceStringEx(self.edtFirstName.text()) or self.checkInputMessage(u'имя', False, self.edtFirstName))
                result = result and (isNameValid(self.edtFirstName.text()) or self.checkInputMessage(u'допустимое имя', False, self.edtFirstName))
                result = result and (forceStringEx(self.edtPatrName.text())  or self.checkInputMessage(u'отчество', True, self.edtPatrName))
                result = result and (isNameValid(self.edtPatrName.text()) or self.checkInputMessage(u'допустимое отчество', False, self.edtPatrName))
        result = result and self.checkSexEntered()
        result = result and (birthDate or self.checkInputMessage(u'дату рождения', False, self.edtBirthDate))
        result = result and self.checkBirthDateEntered()
#        result = result and (self.cmbSex.currentIndex() or self.checkInputMessage(u'пол', False, self.cmbSex))
        result = result and checkSNILSEntered(self)
        result = result and self.checkDocEntered()
        result = result and self.checkDocsEntered()
        result = result and self.checkDocNumber()

        result = result and self.checkPolicyEntered(True)
        result = result and self.checkPolicyEntered(False)
        result = result and self.checkContacts()
        result = result and self.checkSocStatuses()
        result = result and self.checkDup()
        #if QtGui.qApp.isCheckClientHouses():
        isControlAddress = QtGui.qApp.isStrictCheckControlAddress()
        if isControlAddress:
            result = result and self.checkRegAddress()
            result = result and self.checkLocAddress()
        result = result and self.checkHouses()
        result = result and self.checkAttachesDataEntered()
        result = result and self.checkClientRelations(self.cmbSex.currentIndex())
        result = result and self.checkClientDeposit()
        result = result and self.checkClientWorkHurts()
#        result = result and self.checkDeposit()
        return result


    def checkRegAddress(self):
        result = True
        if trim(self.edtRegFreeInput.text()):
            return True
        boolCorpus = (trim(self.edtRegFlat.text()) or trim(self.edtRegHouse.text()) or trim(self.edtRegCorpus.text()))
        result = result and (self.cmbRegCity.code() or self.checkStreetHouseMessage(u'Адресс регистрации. Введите населённый пункт.', not boolCorpus, self.cmbRegCity))
        if self.cmbRegStreet.count() > 0:
            result = result and (self.cmbRegStreet.code() or self.checkStreetHouseMessage(u'Адресс регистрации. Введите название улицы.', not boolCorpus, self.cmbRegStreet))
        result = result and (trim(self.edtRegFlat.text()) or self.checkStreetHouseMessage(u'Адресс регистрации. Введите номер квартиры.', True, self.edtRegFlat))
        result = result and (trim(self.edtRegHouse.text()) or self.checkStreetHouseMessage(u'Адресс регистрации. Введите номер дома.', not(trim(self.edtRegFlat.text()) or trim(self.edtRegCorpus.text()) or self.cmbRegStreet.code()), self.edtRegHouse))
        boolCorpus = (trim(self.edtRegFlat.text()) or trim(self.edtRegHouse.text()))
        if boolCorpus:
            result = result and (trim(self.edtRegCorpus.text()) or self.checkStreetHouseMessage(u'Адресс регистрации. Введите корпус.', True, self.edtRegCorpus))
        return result


    def checkLocAddress(self):
        result = True
        if trim(self.edtRegFreeInput.text()):
            return True
        boolCorpus = (trim(self.edtLocFlat.text()) or trim(self.edtLocHouse.text()) or trim(self.edtLocCorpus.text()))
        result = result and (self.cmbLocCity.code() or self.checkStreetHouseMessage(u'Адресс проживания. Введите населённый пункт.', not boolCorpus, self.cmbLocCity))
        if self.cmbLocStreet.count() > 0:
            result = result and (self.cmbLocStreet.code() or self.checkStreetHouseMessage(u'Адресс проживания. Введите название улицы.', not boolCorpus, self.cmbLocStreet))
        result = result and (trim(self.edtLocFlat.text()) or self.checkStreetHouseMessage(u'Адресс проживания. Введите номер квартиры.', True, self.edtLocFlat))
        result = result and (trim(self.edtLocHouse.text()) or self.checkStreetHouseMessage(u'Адресс проживания. Введите номер дома.', not(trim(self.edtLocFlat.text()) or trim(self.edtLocCorpus.text()) or self.cmbLocStreet.code()), self.edtLocHouse))
        boolCorpus = (trim(self.edtLocFlat.text()) or trim(self.edtLocHouse.text()))
        if boolCorpus:
            result = result and (trim(self.edtLocCorpus.text()) or self.checkStreetHouseMessage(u'Адресс проживания. Введите корпус.', True, self.edtLocCorpus))
        return result


    def checkClientWorkHurts(self):
        for row, item in enumerate(self.modelWorkHurts.items()):
            if not forceRef(item.value('hurtType_id')):
                return self.checkInputMessage(u'вредность', False, self.tblWorkHurts, row, 0)
        return True


    def checkHouses(self):
        result = True
        regHouse = forceString(self.edtRegHouse.text())
        regStreetCode = False
        if regHouse:
            regStreetCode = self.cmbRegStreet.code()
            if not regStreetCode:
                cityCode = self.cmbRegCity.code()
                regStreetCode = cityCode + u'000000'
            if regStreetCode:
                res = self.checkHouseList(regStreetCode, 0)
                if res:
                    base = self.clientRegHousesList.getCheckResult()
                    widgetHouse = self.edtRegHouse
                    result = result and self.checkStreetHouseMessage(u'Данные о номере дома по адресу %s %s, %s, %s отсутствуют в %s.'%(u'регистрации',  self.cmbRegCity.currentText(), self.cmbRegStreet.currentText(), regHouse, base), res == 1, widgetHouse)
        locHouse = forceString(self.edtLocHouse.text())
        if locHouse:
            locStreetCode = self.cmbLocStreet.code()
            if not locStreetCode:
                cityCode = self.cmbLocCity.code()
                locStreetCode = cityCode + u'000000'
            if locStreetCode:
                res = self.checkHouseList(locStreetCode, 1)
                if res:
                    base = self.clientLocHousesList.getCheckResult()
                    widgetHouse = self.edtLocHouse
                    result = result and self.checkStreetHouseMessage(u'Данные о номере дома по адресу %s %s, %s, %s отсутствуют в %s.'%(u'проживания',  self.cmbRegCity.currentText(), self.cmbRegStreet.currentText(), locHouse, base), res == 1, widgetHouse)
        return result


    def checkStreetHouseMessage(self, message, ignore, widget, row=None, column=None, detailWdiget=None, setFocus=True):
        messageBox = QtGui.QMessageBox.Ok
        if ignore:
            messageBox |= QtGui.QMessageBox.Ignore
        res = QtGui.QMessageBox.warning( self,
                                         u'Внимание!',
                                         message,
                                         messageBox,
                                         QtGui.QMessageBox.Ok)
        if res == QtGui.QMessageBox.Ok:
            if setFocus:
                self.setFocusToWidget(widget, row, column)
                if detailWdiget:
                    self.setFocusToWidget(detailWdiget, row, column)
            return False
        return True


    def checkClientDeposit(self):
        contractDateList = {}
        for row, record in enumerate(self.modelDeposit._items):
            contractIdNew = forceRef(record.value('contract_id'))
            contractDate = forceDate(record.value('contractDate'))
            if contractDate in contractDateList.keys():
                contractIdList = contractDateList.get(contractDate, None)
                if contractIdNew and contractIdList and contractIdNew in contractIdList:
                    return self.checkValueMessage(u'Не должно быть двух одинаковых договоров на одну дату в таблице', False, self.tblDeposit, row, record.indexOf('contract_id'))
                elif contractIdNew and contractIdList:
                   contractIdList.append(contractIdNew)
                   contractDateList[contractDate] = contractIdList
                elif contractIdNew and contractIdList:
                    contractDateList[contractDate] = [contractIdNew]
            else:
                contractDateList[contractDate] = [contractIdNew]
        return True


    def checkDeposit(self):
        from Events.ClientDepositDialog import CClientDepositDialog
        currentDate = QDate.currentDate()
        self.btnExit = False
        self.clientAge = calcAgeTuple(self.clientBirthDate, currentDate)
        sumAction = 0
        sumItem = 0
        contractIdList = {}
        serviceBool = False
        for row, record in enumerate(self.modelDeposit._items):
            contractId = forceRef(record.value('contract_id'))
            #contractDate = forceDate(record.value('contractDate'))
            sumAction = 0.0
            sumItem = 0.0
            currentDate = QDate.currentDate()
            if contractId:
                contractSum = forceDouble(record.value('contractSum'))
                #contractSumList = contractIdList.get((contractId, contractDate), 0)
                contractSumList = contractIdList.get(contractId, 0)
                contractSumList += contractSum
                #contractIdList[(contractId, contractDate)] = contractSumList
                contractIdList[contractId] = contractSumList

        db = QtGui.qApp.db
        tableAction = db.table('Action')
        tableEvent = db.table('Event')
        tableContract = db.table('Contract')
        tableAccountItem = db.table('Account_Item')
        tableActionType = db.table('ActionType')
        tableActionTypeService = db.table('ActionType_Service')
        tableRBService = db.table('rbService')
        tableContractTariff = db.table('Contract_Tariff')

        for contractKey, contractSum in contractIdList.items():
            #contractId, contractDate = contractKey
            contractId = contractKey
            if contractId:
                contractRecord = db.getRecordEx(tableContract,
                                           [tableContract['limitExceeding'], tableContract['limitOfFinancing']],
                                           [tableContract['deleted'].eq(0), tableContract['id'].eq(contractId)])
                if contractRecord:
                    limitExceeding = forceDouble(contractRecord.value('limitExceeding'))
                    limitOfFinancing = forceDouble(contractRecord.value('limitOfFinancing'))

            #  payStatus != 0
                cond = [tableAction['deleted'].eq(0),
                        tableEvent['deleted'].eq(0),
                        tableAccountItem['deleted'].eq(0),
                        tableAction['contract_id'].eq(contractId),
                        tableAction['payStatus'].ne(0),
                        tableAccountItem['refuseType_id'].isNull()
                        ]
#                if contractDate:
#                    cond.append(tableAction['begDate'].dateGe(contractDate))
#                    cond.append(tableAction['endDate'].dateLe(contractDate))
                if self.currentClientId:
                    cond.append(tableEvent['client_id'].eq(self.currentClientId))
                table = tableEvent.innerJoin(tableAction, tableEvent['id'].eq(tableAction['event_id']))
                table = table.innerJoin(tableAccountItem, tableAction['id'].eq(tableAccountItem['action_id']))
                records = db.getRecordListGroupBy(table, u'SUM(Account_Item.sum) AS sumAction', cond, u'Action.contract_id')
                if records:
                    newRecord = records[0]
                    sumAction = forceDouble(newRecord.value('sumAction'))

            #  payStatus == 0
                cols = u'''SUM(IF(Action.amount > Contract_Tariff.amount AND Contract_Tariff.amount != 0, Contract_Tariff.amount * Contract_Tariff.price,
                Action.amount * Contract_Tariff.price)) AS sumItem, Action.contract_id'''
                cond = [tableAction['deleted'].eq(0),
                        tableEvent['deleted'].eq(0),
                        tableActionType['deleted'].eq(0),
                        tableContractTariff['deleted'].eq(0),
                        tableAction['contract_id'].eq(contractId),
                        tableContractTariff['master_id'].eq(contractId),
                        tableAction['payStatus'].eq(0),
                        tableContractTariff['tariffType'].eq(CTariff.ttActionAmount),
                        tableActionType['id'].eq(tableActionTypeService['master_id'])
                        ]
#                if contractDate:
#                    cond.append(tableAction['begDate'].dateGe(contractDate))
#                    cond.append(tableAction['endDate'].dateLe(contractDate))
                if self.clientAge:
                    cond.append(db.joinOr([tableContractTariff['age'].eq(''), tableContractTariff['age'].ge(self.clientAge[3])]))
                if self.currentClientId:
                    cond.append(tableEvent['client_id'].eq(self.currentClientId))
                cond.append(db.joinOr([tableContractTariff['sex'].eq(0), tableContractTariff['sex'].eq(self.clientSex)]))
                table = tableEvent.innerJoin(tableAction, tableEvent['id'].eq(tableAction['event_id']))
                table = table.innerJoin(tableActionType, tableAction['actionType_id'].eq(tableActionType['id']))
                table = table.innerJoin(tableActionTypeService, tableAction['finance_id'].eq(tableActionTypeService['finance_id']))
                table = table.innerJoin(tableRBService, tableActionTypeService['service_id'].eq(tableRBService['id']))
                table = table.innerJoin(tableContractTariff, tableRBService['id'].eq(tableContractTariff['service_id']))
                records = db.getRecordListGroupBy(table, cols, cond, u'Action.contract_id')
                for newRecord in records:
                    contractId = forceRef(newRecord.value('contract_id'))
                    sumItem = forceDouble(newRecord.value('sumItem'))

                limitSum = contractSum
                limitSumContract = limitOfFinancing
                if limitExceeding >= limitOfFinancing:
                    limitSumContract = limitExceeding
                    if contractSum >= limitExceeding:
                       limitSum = limitSumContract
                else:
                    limitSumContract = limitOfFinancing
                    if contractSum >= limitOfFinancing:
                       limitSum = limitSumContract

                sumDeposit = (sumAction + sumItem)
                if sumDeposit > 0:
                    serviceBool = True
                buttonBoxIgnore = False
                if limitExceeding and sumDeposit > limitExceeding and sumDeposit < limitOfFinancing:
                    buttonBoxIgnore = True

                if limitExceeding or limitOfFinancing:
                    if ((sumAction + sumItem) > limitExceeding or limitSum < sumDeposit):
                        CClientDepositDialog(self, self.currentClientId, self.clientAge, self.clientSex, buttonBoxIgnore, title=u'Превышение общей стоимости услуг по договору').exec_()
                        if (sumAction + sumItem) > limitExceeding and self.btnExit:
                            if buttonBoxIgnore:
                                return True
                            else:
                                return False
                        else:
                            return False
                elif (sumAction + sumItem) > contractSum:
                    CClientDepositDialog(self, self.currentClientId, self.clientAge, self.clientSex, buttonBoxIgnore, title=u'Превышение общей стоимости услуг по договору').exec_()
                    if (sumAction + sumItem) > limitExceeding and self.btnExit:
                        return True
                    else:
                        return False
        if serviceBool:
            CClientDepositDialog(self, self.currentClientId, self.clientAge, self.clientSex, True, title=u'Предел стоимости услуг по договору').exec_()
            if self.btnExit:
                return True
            else:
                return False

        return True


    def checkDocNumber(self):
        for item in self.modelSocStatuses.items():
            if forceRef(item.value('documentType_id')):
                if not forceString(item.value('number')):
                    if QtGui.QMessageBox.question( self,
                                           u'Внимание!',
                                           u'Для типа соц.статуса не введен номер документа.',
                                           QtGui.QMessageBox.Ok|QtGui.QMessageBox.Ignore,
                                           QtGui.QMessageBox.Ok) == QtGui.QMessageBox.Ok:
                        self.edtSocStatusDocNumber.setFocus(Qt.ShortcutFocusReason)
                        return False
        return True


    def checkSexEntered(self):
        currentSex = self.cmbSex.currentIndex()
        firstName = forceStringEx(self.edtFirstName.text())
        firstNameSex = forceInt(QtGui.qApp.db.translate('rdFirstName', 'name', firstName, 'sex')) if firstName else 0
        patrName = forceStringEx(self.edtPatrName.text())
        patrNameSex = forceInt(QtGui.qApp.db.translate('rdPatrName', 'name', patrName, 'sex')) if patrName else 0
        if firstNameSex and patrNameSex:
            if firstNameSex == patrNameSex:
                detectedSex = firstNameSex
            else:
                res = QtGui.QMessageBox.question( self,
                                                  u'Внимание!',
                                                  u'Конфликт имени и отчества.\nХотите исправить?',
                                                  QtGui.QMessageBox.Yes|QtGui.QMessageBox.No,
                                                  QtGui.QMessageBox.Yes)
                if res == QtGui.QMessageBox.Yes:
                    if currentSex == firstNameSex:
                       self.edtPatrName.setFocus(Qt.ShortcutFocusReason)
                    else:
                       self.edtFirstName.setFocus(Qt.ShortcutFocusReason)
                    return False
                detectedSex = 0
        else:
            detectedSex = max(firstNameSex, patrNameSex)

        if currentSex:
            if detectedSex and currentSex != detectedSex:
                res = QtGui.QMessageBox.question( self,
                                                 u'Внимание!',
                                                 u'Конфликт при выборе пола.\nИсправить пол на %s?' % formatSex(detectedSex),
                                                 QtGui.QMessageBox.Yes|QtGui.QMessageBox.No,
                                                 QtGui.QMessageBox.Yes)
                if res == QtGui.QMessageBox.Yes:
                    self.cmbSex.setCurrentIndex(detectedSex)
        else:
            if detectedSex:
                self.cmbSex.setCurrentIndex(detectedSex)
            else:
                return self.checkInputMessage(u'пол', False, self.cmbSex)
        return True


    def checkBirthDateEntered(self):
        checkBirthDate = self.edtBirthDate.date()
        currentDate = QDate.currentDate()
        if checkBirthDate > currentDate:
            res = QtGui.QMessageBox.warning( self,
                                             u'Внимание!',
                                             u'Дата рождения не может быть больше текущей даты',
                                             QtGui.QMessageBox.Ok,
                                             QtGui.QMessageBox.Ok)
            if res == QtGui.QMessageBox.Ok:
                self.edtBirthDate.setFocus(Qt.ShortcutFocusReason)
                return False
        if currentDate.year() - checkBirthDate.year() > 135:
            res = QtGui.QMessageBox.question( self,
                                               u'Внимание!',
                                               u'Возраст превышает 135 лет.\nХотите исправить?',
                                               QtGui.QMessageBox.Yes|QtGui.QMessageBox.No,
                                               QtGui.QMessageBox.Yes)
            if res == QtGui.QMessageBox.Yes:
                self.edtBirthDate.setFocus(Qt.ShortcutFocusReason)
                return False
        return True


    def checkDocEntered(self):
        documentTypeId = self.cmbDocType.value()
        if not documentTypeId:
            result = self.checkInputMessage(u'тип документа', True, self.cmbDocType)
        else:
            result = True
        if self.edtDocSerialLeft.isEnabled() and not self.edtDocSerialLeft.hasAcceptableInput():
            result = self.checkInputMessage(u'серию документа', True, self.edtDocSerialLeft)
        if result and self.edtDocSerialRight.isEnabled() and not self.edtDocSerialRight.hasAcceptableInput():
            result = self.checkInputMessage(u'серию документа', True, self.edtDocSerialRight)
        if result and self.edtDocSerialRight.isEnabled() and not self.edtDocSerialRight.hasAcceptableInput():
            result = self.checkInputMessage(u'серию документа', True, self.edtDocSerialRight)
        if result and self.edtDocNumber.isEnabled() and not self.edtDocNumber.hasAcceptableInput():
            result = self.checkInputMessage(u'серию документа', True, self.edtDocNumber)
#        if result and self.edtDocDate.isEnabled() and not self.edtDocDate.date():
#            result = self.checkInputMessage(u'дату выдачи документа', True, self.edtDocDate)
#        if result and self.edtDocOrigin.isEnabled() and not forceStringEx(self.edtDocOrigin.text()):
#            result = self.checkInputMessage(u'кем выдан документ', True, self.edtDocOrigin)
        return result


    def checkDocsEntered(self):
        result = True
        for row in xrange(self.modelIdentificationDocs.rowCount()-1):
            result = result and self.checkDocsItemEntered(row)
        return result


    def checkDocsItemEntered(self, row):
        result = True
        items = self.modelIdentificationDocs.items()
        item = items[row]

        documentTypeId = forceRef(item.value('documentType_id'))
        if not documentTypeId:
            self.checkInputMessage(u'тип документа', False, self.tblIdentificationDocs, row, 0)
            return False

        serial = forceString(item.value('serial'))
        number = forceString(item.value('number'))
#        date   = forceDate(item.value('date'))
#        origin = forceStringEx(item.value('origin'))

        documentTypeDescr = getDocumentTypeDescr(documentTypeId)
        serialLeft, serialRight = documentTypeDescr.splitDocSerial(serial)
        if not documentTypeDescr.matchLeftPart(serialLeft):
            self.checkInputMessage(u'серию документа (левую часть)', False, self.tblIdentificationDocs, row, 1)
            return False
        if not documentTypeDescr.matchRightPart(serialRight):
            self.checkInputMessage(u'серию документа (правую часть)', False, self.tblIdentificationDocs, row, 1)
            return False
        if not documentTypeDescr.matchNumber(number):
            self.checkInputMessage(u'номер документа', False, self.tblIdentificationDocs, row, 2)
            return False
# старые данные никто не восполнит, новые данные должны быть указаны на титульной странице.
#        if result and not date:
#            result = self.checkInputMessage(u'дату выдачи документа', True, self.tblIdentificationDocs, row, 3)
#        if result and not origin:
#            result = self.checkInputMessage(u'кем выдан документ', True, self.tblIdentificationDocs, row, 4)
        return result


    def checkPolicyEntered(self, isCompulsory):
        if isCompulsory:
            edtSerial = self.edtCompulsoryPolisSerial
            edtNumber = self.edtCompulsoryPolisNumber
            cmbPolisCompany = self.cmbCompulsoryPolisCompany
#            cmbPolisType = self.cmbCompulsoryPolisType
            cmbPolisKind = self.cmbCompulsoryPolisKind
            edtPolisName = self.edtCompulsoryPolisName
            edtPolisBegDate = self.edtCompulsoryPolisBegDate
            edtPolisEndDate = self.edtCompulsoryPolisEndDate
        else:
            edtSerial = self.edtVoluntaryPolisSerial
            edtNumber = self.edtVoluntaryPolisNumber
            cmbPolisCompany = self.cmbVoluntaryPolisCompany
#            cmbPolisType = self.cmbVoluntaryPolisType
            cmbPolisKind = self.cmbVoluntaryPolisKind
            edtPolisName = self.edtVoluntaryPolisName
            edtPolisBegDate = self.edtVoluntaryPolisBegDate
            edtPolisEndDate = self.edtVoluntaryPolisEndDate

        serial = forceStringEx(edtSerial.text())
        number = forceStringEx(edtNumber.text())
        insurerId = cmbPolisCompany.value()
        polisKind = cmbPolisKind.value()
        polisCompanyIsEmpty = not (insurerId or forceStringEx(edtPolisName.text()))
        begDate = edtPolisBegDate.date()
        endDate = edtPolisEndDate.date()

        if not polisCompanyIsEmpty or serial or number or begDate or endDate:
            result = not polisCompanyIsEmpty or self.checkInputMessage(u'страховую компанию', False, cmbPolisCompany)
            if isCompulsory:
#                policyTypeId = cmbPolisType.value()
                policyKindId = cmbPolisKind.value()
                policyKindCode = cmbPolisKind.code()
                result = result and ( (polisKind==3) or  serial or self.checkInputMessage(u'серию полиса', True, edtSerial))
                result = result and (number or self.checkInputMessage(u'номер полиса', False, edtNumber))
                result = result and ( policyKindCode != '3'  or unitedPolicyIsVaid(number) or self.checkInputMessage(u'правильный номер единого полиса', False, edtNumber))
                result = result and ( policyKindId or self.checkInputMessage(u'вид полиса', False, cmbPolisKind))
                if QtGui.qApp.isStrictPolicyCheckOnEventCreation() != 2:
                    result = result and ( begDate or self.checkInputMessage(u'начало действия полиса ОМС', bool(endDate), edtPolisBegDate))
                    #result = result and ( (polisKind==3) or endDate or self.checkInputMessage(u'окончание действия полиса ОМС', True, edtPolisEndDate))
                result = result and self.checkDatePolicy(begDate, endDate, isCompulsory, serial, number)
            else:
                result = result and (serial or self.checkInputMessage(u'серию полиса', True, edtSerial))
                result = result and (number or self.checkInputMessage(u'номер полиса', False, edtNumber))
                if QtGui.qApp.isStrictPolicyCheckOnEventCreation() != 2:
                    result = result and ( begDate or self.checkInputMessage(u'начало действия полиса ДМС', False, edtPolisBegDate))
                    result = result and ( endDate or self.checkInputMessage(u'окончание действия полиса ДМС', False, edtPolisEndDate))
                result = result and self.checkDatePolicy(begDate, endDate, isCompulsory, serial, number)
            return result
        return True


    def checkDatePolicy(self, policyBegDate, policyEndDate, isCompulsory, policySerial, policyNumber):
        result = True
        policyTypeIdList = []
        for row, record in enumerate(self.modelPolicies.items()):
            policyTypeIdList.append(forceRef(record.value('policyType_id')))
        if policyTypeIdList:
            db = QtGui.qApp.db
            tableRBPolicyType = db.table('rbPolicyType')
            cond = [tableRBPolicyType['id'].inlist(policyTypeIdList),
                    tableRBPolicyType['isCompulsory'].eq(isCompulsory)]
            if not isCompulsory:
                cond.append(tableRBPolicyType['code'].eq(u'3'))
            idList = db.getDistinctIdList(tableRBPolicyType, [tableRBPolicyType['id']], cond)
            if idList:
                resultCorrectDatePolicy = False
                isCorrectDatePolicy = False
                for row, record in enumerate(self.modelPolicies.items()):
                    if isCorrectDatePolicy:
                        break
                    begDate = forceDate(record.value('begDate'))
                    endDate = forceDate(record.value('endDate'))
                    policyTypeId = forceRef(record.value('policyType_id'))
                    if policyTypeId and policyTypeId in idList:
                        serial = forceString(record.value('serial'))
                        number = forceString(record.value('number'))
                        periodBegDate = db.formatDate(begDate)
                        periodEndDate = db.formatDate(endDate)
                        result = result and (begDate or self.checkValueMessage(u'Укажите дату начала действия полиса серия %s номер %s период с %s по %s'%(serial, number, periodBegDate, periodEndDate), False, self.tblPolicies, row, record.indexOf('begDate')))
                        #result = result and (endDate or self.checkValueMessage(u'Укажите дату завершения действия полиса серия %s номер %s период с %s по %s'%(serial, number, periodBegDate, periodEndDate), True, self.tblPolicies, row, record.indexOf('endDate')))
                        for row2, record2 in enumerate(self.modelPolicies.items()):
                            if row != row2:
                                begDate2 = forceDate(record2.value('begDate'))
                                endDate2 = forceDate(record2.value('endDate'))
                                policyTypeId2 = forceRef(record2.value('policyType_id'))
                                if policyTypeId2 and policyTypeId2 in idList:
                                    if begDate:
                                        if ((not begDate2 or begDate2 <= begDate) and (not endDate2 or endDate2 >= begDate)):
                                            resultCorrectDatePolicy = (self.checkDatePolicyMessage(u'Пересечение действия полиса %s - %s с полисом %s - %s'%(forceString(begDate), forceString(endDate), forceString(begDate2), forceString(endDate2)), False, self.tblPolicies, row, record.indexOf('begDate'), idList=idList))
                                            result = result and resultCorrectDatePolicy
                                            isCorrectDatePolicy = True
                                            break
                                    if endDate:
                                        if ((not endDate2 or endDate2 >= endDate) and (not begDate2 or begDate2 <= endDate)):
                                            resultCorrectDatePolicy = (self.checkDatePolicyMessage(u'Пересечение действия полиса %s - %s с полисом %s - %s'%(forceString(begDate), forceString(endDate), forceString(begDate2), forceString(endDate2)), False, self.tblPolicies, row, record.indexOf('endDate'), idList=idList))
                                            result = result and resultCorrectDatePolicy
                                            isCorrectDatePolicy = True
                                            break
                result = result and self.checkIdenticalPolicy(idList, resultCorrectDatePolicy, isCompulsory)
        return result


    def checkDatePolicyMessage(self, message, skipable, widget, row=None, column=None, detailWdiget=None, setFocus=True, idList=[]):
        messageBox = QtGui.QMessageBox()
        buttons = QtGui.QMessageBox.Ok
        if skipable:
            buttons = buttons | QtGui.QMessageBox.Ignore
        messageBox = CCheckDatePolicyMessageBox(QtGui.QMessageBox.Warning,
                                                   u'Внимание!',
                                                   message,
                                                   buttons,
                                                   self
                                                )
        messageBox.setDefaultButton(QtGui.QMessageBox.Ok)
        messageBox.setEscapeButton(QtGui.QMessageBox.Ok)
        correctButton = QtGui.QPushButton(u'Исправить')
        messageBox.addButton(correctButton, QtGui.QMessageBox.AcceptRole)
        res = messageBox.exec_()
        clickedButton = messageBox.clickedButton()
        if res == QtGui.QMessageBox.Ok:
            if setFocus:
                self.setFocusToWidget(widget, row, column)
                if isinstance(detailWdiget, QtGui.QWidget):
                    self.setFocusToWidget(detailWdiget, row, column)
            return False
        elif clickedButton == correctButton:
            return QtGui.qApp.callWithWaitCursor(self, self.correctDatePolicy, idList)
        return False


    def correctDatePolicy(self, idList):
        db = QtGui.qApp.db
        items = []
        itemData = {}
        itemForDate = {}
        for row, record in enumerate(self.modelPolicies.items()):
            policyTypeId = forceRef(record.value('policyType_id'))
            if policyTypeId and policyTypeId in idList:
                serial = forceString(record.value('serial')).lower()
                number = forceString(record.value('number')).lower()
                insurerId = forceRef(record.value('insurer_id'))
                begDate = forceDate(record.value('begDate'))
                endDate = forceDate(record.value('endDate'))
                begDateDT = datetime.date(begDate.year(), begDate.month(), begDate.day())
                endDateDT = datetime.date(endDate.year(), endDate.month(), endDate.day()) if endDate else None
                newKey = (begDateDT, endDateDT, insurerId)
                itemForLine = itemForDate.get(newKey, {})
                itemForLine[(serial, number)] = (record, row, serial, number)
                itemForDate[newKey] = itemForLine
            else:
                items.append(record)
        for keyData, itemLine in itemForDate.items():
            if len(itemLine) > 1:
                QtGui.qApp.restoreOverrideCursor()
                return self.checkValueMessage(u'Обнаружены полисы с одинаковым периодом и типом, но с разными номерами. Автоматическое исправление не возможно!', False, self.tblPolicies, itemLine.values()[0][1], itemLine.values()[0][0].indexOf('begDate'))
        itemForDateKeys = itemForDate.keys()
        #itemForDateKeys.sort(key=lambda item:(item[0], item[1] if item[1] else datetime.date((QDate().currentDate().addYears(200)).year(), 12, 1)))
        itemForDateKeys.sort(key=lambda item:(item[0]))
        itemNewForDate = itemForDate.copy()
        rowFocus = None
        columnFocus = None
        for i, keyData in enumerate(itemForDateKeys):
            itemLine = itemForDate.get(keyData, {})
            if rowFocus is None and itemLine:
                rowFocus = 0
                columnFocus = itemLine.values()[0][0].indexOf('begDate')
            if i > 0:
                keyDataPrev = itemForDateKeys[i-1]
                itemLinePrev = itemForDate.get(keyDataPrev, {})
                if not keyDataPrev[1] or keyDataPrev[1] >= keyData[0]:
                    if (not keyDataPrev[1] and keyData[1]) or (keyDataPrev[1] and keyData[1] and keyDataPrev[1] >= keyData[1]):
                        if (keyDataPrev[2] != keyData[2]) or (itemLinePrev.values()[0][2] != itemLine.values()[0][2]) or (itemLinePrev.values()[0][3] != itemLine.values()[0][3]):
                            QtGui.qApp.restoreOverrideCursor()
                            return self.checkValueMessage(u'Обнаружены полисы: один поглощает период другого. Автоматическое исправление не возможно!', False, self.tblPolicies, itemLine.values()[0][1], itemLine.values()[0][0].indexOf('begDate'))
                    if (keyDataPrev[2] == keyData[2]) and (itemLinePrev.values()[0][2] == itemLine.values()[0][2]) and (itemLinePrev.values()[0][3] == itemLine.values()[0][3]):
                        endDatePrev = forceDate(keyDataPrev[1])
                        endDateNext = forceDate(keyData[1])
                        if not endDatePrev or (endDateNext and endDatePrev > endDateNext):
                            endDate = endDatePrev.addDays(-1) if endDatePrev else QDate()
                        elif not endDateNext or (endDatePrev and endDatePrev < endDateNext):
                            endDate = endDateNext
                        else:
                            endDate = forceDate(keyData[0]).addDays(-1)
                        record = itemLinePrev.values()[0][0]
                        begDateUpd = forceDate(record.value('begDate'))
                        if endDate and begDateUpd > endDate:
                            QtGui.qApp.restoreOverrideCursor()
                            return self.checkValueMessage(u'Обнаружен неразрешимый конфликт полисов. Автоматическое исправление не возможно!', False, self.tblPolicies, itemLine.values()[0][1], itemLine.values()[0][0].indexOf('begDate'))
                        if not endDatePrev or (endDateNext and endDatePrev > endDateNext):
                            itemNewForDate.pop(keyData, {})
                        elif not endDateNext or (endDatePrev and endDatePrev < endDateNext):
                            record.setValue('endDate', toVariant(endDate))
                            row = itemLinePrev.values()[0][1]
                            itemNewForDate.pop(keyDataPrev, {})
                            itemNewForDate.pop(keyData, {})
                            endDateNew = datetime.date(endDate.year(), endDate.month(), endDate.day()) if endDate else None
                            itemNewForDate[(keyDataPrev[0], endDateNew, keyDataPrev[2])] = {itemLinePrev.keys()[0]:(record, row)}
                        else:
                            record.setValue('endDate', toVariant(endDate))
                            row = itemLinePrev.values()[0][1]
                            itemNewForDate.pop(keyDataPrev, {})
                            endDateNew = datetime.date(endDate.year(), endDate.month(), endDate.day()) if endDate else None
                            itemNewForDate[(keyDataPrev[0], endDateNew, keyDataPrev[2])] = {itemLinePrev.keys()[0]:(record, row)}
                    else:
                        record = itemLinePrev.values()[0][0]
                        endDate = forceDate(keyData[0]).addDays(-1)
                        begDateUpd = forceDate(record.value('begDate'))
                        if begDateUpd > endDate:
                            QtGui.qApp.restoreOverrideCursor()
                            return self.checkValueMessage(u'Обнаружен неразрешимый конфликт полисов. Автоматическое исправление не возможно!', False, self.tblPolicies, itemLine.values()[0][1], itemLine.values()[0][0].indexOf('begDate'))
                        record.setValue('endDate', toVariant(endDate))
                        row = itemLinePrev.values()[0][1]
                        itemNewForDate.pop(keyDataPrev, {})
                        endDateNew = datetime.date(endDate.year(), endDate.month(), endDate.day()) if endDate else None
                        itemNewForDate[(keyDataPrev[0], endDateNew, keyDataPrev[2])] = {itemLinePrev.keys()[0]:(record, row)}
        for itemNewForDateLine in itemNewForDate.values():
            items.append(itemNewForDateLine.values()[0][0])
        items, policyTypeList, result = self.mergeIdenticalPolicy(items, idList)
        items.sort(key=lambda item: (forceDate(item.value('begDate')), policyTypeList.get(forceRef(item.value('policyType_id')), False)), reverse = False)
        self.modelPolicies.setItems(items)
        self.modelPolicies.reset()
        self.on_modelPolicies_policyChanged()
        self.syncPolicies()
        self.setFocusToWidget(self.tblPolicies, rowFocus, columnFocus)
        return True


    def checkIdenticalPolicy(self, idList, resultCorrectDatePolicy, isCompulsory):
        db = QtGui.qApp.db
        items = self.modelPolicies.items()
        items, policyTypeList, result = self.mergeIdenticalPolicy(items, idList)
        items.sort(key=lambda item: (forceDate(item.value('begDate')), policyTypeList.get(forceRef(item.value('policyType_id')), False)), reverse = False)
        self.modelPolicies.setItems(items)
        self.modelPolicies.reset()
        self.on_modelPolicies_policyChanged()
        self.syncPolicies()
        self.setFocusToWidget(self.tblPolicies, 0 if items else None, 0 if items else None)
        if result or resultCorrectDatePolicy:
            self.isCorrectDatePolicy(isCompulsory)
        return True


    def isCorrectDatePolicy(self, isCompulsory):
        res = QtGui.QMessageBox.warning( self,
                                         u'Внимание!',
                                         u'Произведено исправление пересечений полисов %s.'%(u'ОМС' if isCompulsory else u'ДМС'),
                                         QtGui.QMessageBox.Ok,
                                         QtGui.QMessageBox.Ok)
        return True


    def mergeIdenticalPolicy(self, items, idList):
        db = QtGui.qApp.db
        policyTypeList = {}
        result = False
        tableRBPolicyType = db.table('rbPolicyType')
        records = db.getRecordList(tableRBPolicyType, [tableRBPolicyType['id'], tableRBPolicyType['isCompulsory'], tableRBPolicyType['code']])
        for rec in records:
            id = forceRef(rec.value('id'))
            isCompulsory = forceBool(rec.value('isCompulsory'))
            policyTypeList[id] = isCompulsory
        items.sort(key=lambda item: (forceDate(item.value('begDate')), policyTypeList.get(forceRef(item.value('policyType_id')), False)), reverse = True)
        updateItems = []
        updateRows = []
        for row, item in enumerate(items):
            policyTypeId = forceRef(item.value('policyType_id'))
            if policyTypeId and policyTypeId in idList:
                updateItems.append(item)
                updateRows.append(row)
        deletedRows = []
        updateRow = None
        itemNew = []
        for row, item in enumerate(updateItems):
            if row > 0:
                itemPrev = updateItems[row-1]
                serial = forceString(item.value('serial')).lower()
                number = forceString(item.value('number')).lower()
                insurerId = forceRef(item.value('insurer_id'))
                serialPrev = forceString(itemPrev.value('serial')).lower()
                numberPrev = forceString(itemPrev.value('number')).lower()
                insurerIdPrev = forceRef(itemPrev.value('insurer_id'))
                if serial == serialPrev and number == numberPrev and insurerId == insurerIdPrev:
                    begDate = forceDate(item.value('begDate'))
                    endDate = forceDate(item.value('endDate'))
                    if not itemNew:
                        itemNew = itemPrev
                        updateRow = row-1
                    deletedRows.append(row)
                    itemNew.setValue('begDate', toVariant(begDate))
                    endDatePrev = forceDate(itemNew.value('endDate'))
                    if endDatePrev and (not endDate or endDate > endDatePrev):
                        itemNew.setValue('endDate', toVariant(endDate))
                    if itemNew and updateRow is not None:
                       updateItems[updateRow] = itemNew
                    result = True
                else:
                    itemNew = []
                    updateRow = None
        if updateItems:
            deletedRows.sort(reverse=True)
            for row in deletedRows:
                updateItems.pop(row)
            updateRows.sort(reverse=True)
            for row in updateRows:
                items.pop(row)
            items.extend(updateItems)
        items.sort(key=lambda item: (forceDate(item.value('begDate')), policyTypeList.get(forceRef(item.value('policyType_id')), False)), reverse = True)
        return items, policyTypeList, result


    def checkContacts(self):
        table = self.tblContacts
        for row, record in enumerate(self.modelContacts.items()):
            contactTypeId = forceRef(record.value('contactType_id'))
            contact = forceString(record.value('contact'))
            if contactTypeId is None:
                return self.checkValueMessage(u'Не указан тип контакта', False, table, row, 0)
            if not contact and not self.checkValueMessage(u'Не указан контакт', True, table, row, 1):
                return False
        return True


    def checkSocStatuses(self):
        table = self.tblSocStatuses
        for row, record in enumerate(self.modelSocStatuses.items()):
            socStatucTypeId = forceRef(record.value('socStatusType_id'))
            if socStatucTypeId is None:
                return self.checkValueMessage(u'Не указан соц.статус', False, table, row, 1)
        citizenshipGP = QtGui.qApp.isCitizenshipControl()
        if citizenshipGP:
            documentTypeId = self.cmbDocType.value()
            if documentTypeId:
                date = QDate.currentDate()
                db = QtGui.qApp.db
                tableType = db.table('rbSocStatusType')
                tableClass = db.table('rbSocStatusClass')
                isCitizenship = forceBool(db.translate('rbDocumentType', 'id', documentTypeId, 'isCitizenship'))
                isCitizenshipRF = False
                citizenshipClassId = forceRef(db.translate(tableClass,  'code', '8', 'id'))
                if citizenshipClassId:
                    for row, record in enumerate(self.modelSocStatuses.items()):
                        socStatusClassId = forceRef(record.value('socStatusClass_id'))
                        if socStatusClassId is None:
                            return self.checkValueMessage(u'Не указан соц.статус Класс', False, self.tblSocStatuses, row, 0)
                        else:
                            begDate = forceDate(record.value('begDate'))
                            endDate = forceDate(record.value('endDate'))
                            if socStatusClassId != citizenshipClassId:
                                recordClass = db.getRecordEx(tableClass, [tableClass['code']], [tableClass['id'].eq(socStatusClassId), tableClass['group_id'].eq(citizenshipClassId)])
                                if recordClass:
                                    socStatusClassCode = forceString(recordClass.value('code'))
                                    if isCitizenship:
                                        if socStatusClassCode == '1' and (not begDate or begDate <= date) and (not endDate or endDate >= date):
                                            isCitizenshipRF = True
                                    elif socStatusClassCode in ['0', '1', '2'] and (not begDate or begDate <= date) and (not endDate or endDate >= date):
                                        isCitizenshipRF = True
                            else:
                                socStatusTypeId = forceRef(record.value('socStatusType_id'))
                                if socStatusTypeId:
                                    socStatusTypeCode = forceString(db.translate(tableType,  'id', socStatusTypeId, 'code'))
                                    if isCitizenship:
                                        if socStatusTypeCode == u'м643' and (not begDate or begDate <= date) and (not endDate or endDate >= date):
                                            isCitizenshipRF = True
                                    elif socStatusTypeCode and (not begDate or begDate <= date) and (not endDate or endDate >= date):
                                        isCitizenshipRF = True
                    if not isCitizenshipRF:
                        return self.checkValueMessage(u'Необходимо указать Гражданство', True if citizenshipGP == 1 else False, self.tblSocStatuses, len(self.modelSocStatuses.items()), 0)
        return True


    def checkClientRelations(self, clientSex):
        def checkClientRelationsInt(table, isDirect, otherFieldName, relationTypeCache, sexFieldName, otherSexFieldName):
            db = QtGui.qApp.db
            model = table.model()
            for row, record in enumerate(model.items()):
                relationTypeId = forceRef(record.value('relativeType_id'))
                otherId        = forceRef(record.value(otherFieldName))
                if relationTypeId:
                    if otherId:
                        otherSex = forceInt(db.translate('Client', 'id', otherId, 'sex'))
                        relationTypeRecord = relationTypeCache.get(relationTypeId)
                        if relationTypeRecord:
                            requiredSex = forceInt(relationTypeRecord.value(sexFieldName))
                            requiredOtherSex = forceInt(relationTypeRecord.value(otherSexFieldName))
                            if (   (requiredSex and requiredSex != clientSex)
                                or (requiredOtherSex and requiredOtherSex != otherSex)
                               ):
                                return self.checkValueMessage(u'Несоответствие полов в связи', False, table, row, 0)
                    else:
                        return self.checkValueMessage(u'Не выбрана связь', False, table, row, 1)
                else:
                    return self.checkValueMessage(u'Не выбрана связь', False, table, row, 0)
            return True

        db = QtGui.qApp.db
        cache = CTableRecordCache(db, 'rbRelationType', ['leftSex', 'rightSex'])

        return (    checkClientRelationsInt(self.tblDirectRelations, True, 'relative_id', cache, 'leftSex', 'rightSex')
                and checkClientRelationsInt(self.tblBackwardRelations, False, 'client_id', cache, 'rightSex', 'leftSex')
               )


    def checkDup(self):
        dupCheckList = ((self.findDupByName,  u'по имени и дате рождения'),
                        (self.findDupBySNILS, u'по СНИЛС'),
                        (self.findDupByDoc,   u'по документу'),
                        (self.findDupByPolicy,u'по полису'),
                       )
        for method, title in dupCheckList:
            idlist = method()
            if idlist:
                res = QtGui.QMessageBox.question( self,
                                                 u'Внимание!',
                                                 u'Обнаружен "двойник" %s\nИгнорировать?' % title,
                                                 QtGui.QMessageBox.Yes|QtGui.QMessageBox.No|QtGui.QMessageBox.Open,
                                                 QtGui.QMessageBox.No)
                if res == QtGui.QMessageBox.No:
                    return False
                if res == QtGui.QMessageBox.Open:
                    self.load(idlist[0])
                    db = QtGui.qApp.db
                    record = db.getRecord(db.table('Client'), '*', self._id)
                    self.setRecord(record)
                    return False
        return True


    def findDup(self, cond):
        db = QtGui.qApp.db
        table = db.table('Client')
        id = self.itemId()
        if id:
            cond.append(table['id'].ne(id))
        return db.getIdList(table, where=cond, order='id')


    def findDupByName(self):
        db = QtGui.qApp.db
        table = db.table('Client')
        lastName  = forceStringEx(self.edtLastName.text())
        firstName = forceStringEx(self.edtFirstName.text())
        patrName  = forceStringEx(self.edtPatrName.text())
        birthDate = self.edtBirthDate.date()
        cond = [table['lastName'].eq(lastName),
                table['firstName'].eq(firstName),
                table['patrName'].eq(patrName),
                table['birthDate'].eq(birthDate),
                table['deleted'].eq(0)
               ]
        return self.findDup(cond)


    def findDupBySNILS(self):
        db = QtGui.qApp.db
        table = db.table('Client')
        SNILS = unformatSNILS(forceStringEx(self.edtSNILS.text()))
        if SNILS:
            cond = [table['SNILS'].eq(SNILS), table['deleted'].eq(0) ]
            return self.findDup(cond)
        else:
            return []


    def findDupByDoc(self):
        docTypeId = self.cmbDocType.value()
        if docTypeId:
            serialLeft = forceStringEx(self.edtDocSerialLeft.text())
            serialRight = forceStringEx(self.edtDocSerialRight.text())
            serial = serialLeft + ' ' + serialRight
            number = forceStringEx(self.edtDocNumber.text())
            db = QtGui.qApp.db
            table = db.table('ClientDocument')
            cond = [ table['documentType_id'].eq(docTypeId),
                     table['serial'].eq(serial),
                     table['number'].eq(number),
                     table['id'].eqEx('(SELECT MAX(CD1.id) FROM ClientDocument AS CD1 WHERE CD1.client_id = ClientDocument.client_id)')
                   ]
            id = self.itemId()
            if id:
                cond.append(table['client_id'].ne(id))
            return db.getIdList(table, 'client_id', cond, 'client_id')
        else:
            return []


    def findDupByPolicy(self):
        serial = forceStringEx(self.edtCompulsoryPolisSerial.text())
        number = forceStringEx(self.edtCompulsoryPolisNumber.text())
        if serial or number:
            db = QtGui.qApp.db
            table = db.table('ClientPolicy')
            cond = [ table['serial'].eq(serial),
                     table['number'].eq(number),
                     table['id'].eqEx('(SELECT MAX(CP1.id) FROM ClientPolicy AS CP1 WHERE CP1.client_id = ClientPolicy.client_id)')
                   ]
            id = self.itemId()
            if id:
                cond.append(table['client_id'].ne(id))
            return db.getIdList(table, 'client_id', cond, 'client_id')
        else:
            return []


    def checkAttachesDataEntered(self):
        result = True
        if QtGui.qApp.isStrictAttachCheckOnEventCreation() != 2:
            for i, item in enumerate(self.modelAttaches.items()):
                result = result and self.checkAttachDataEntered(i, item)
                if not result:
                    break
        return result


    def checkAttachDataEntered(self, row, item):
        result = ( forceRef(item.value('attachType_id')) or self.checkInputMessage(u'тип прикрепления',  False, self.tblAttaches, row, item.indexOf('attachType_id'))) \
             and ( forceRef(item.value('LPU_id'))        or self.checkInputMessage(u'ЛПУ',               False, self.tblAttaches, row, item.indexOf('LPU_id'))) \
             and ( forceDate(item.value('begDate'))      or self.checkInputMessage(u'дату прикрепления', False, self.tblAttaches, row, item.indexOf('begDate')))
        return result


    def syncIdentificationDocs(self):
        #  синхронизация полей описания документа
        # (cmbDocType, edtDocSerialLeft, edtDocSerialRight, edtDocNumber)
        # в modelIdentificationDocs
        documentTypeId = self.cmbDocType.value()
        if documentTypeId is None:
            serial = ''
            number = ''
            date = QDate()
            origin = ''
        else:
            documentTypeDescr = getDocumentTypeDescr(documentTypeId)
            serialLeft = forceStringEx(self.edtDocSerialLeft.text())
            serialRight = forceStringEx(self.edtDocSerialRight.text())
            serial = documentTypeDescr.compoundDocSerial(serialLeft, serialRight)
            number = forceStringEx(self.edtDocNumber.text())
            date = forceDate(self.edtDocDate.date())
            origin = forceStringEx(self.edtDocOrigin.text())

        if documentTypeId is not None or len(self.modelIdentificationDocs.items())!=0:
            row = self.modelIdentificationDocs.getCurrentDocRow(documentTypeId, serial, number)
            bs = self.modelIdentificationDocs.blockSignals(True)
            try:
                self.modelIdentificationDocs.setValue(row, 'documentType_id', documentTypeId)
                self.modelIdentificationDocs.setValue(row, 'serial', serial)
                self.modelIdentificationDocs.setValue(row, 'number', number)
                self.modelIdentificationDocs.setValue(row, 'date', date)
                self.modelIdentificationDocs.setValue(row, 'origin', origin)
            finally:
                self.modelIdentificationDocs.blockSignals(bs)


    def updateCompulsoryPolicyType(self):
        serial = forceStringEx(self.edtCompulsoryPolisSerial.text())
        insurerId = self.cmbCompulsoryPolisCompany.value()
        if serial and insurerId and not self.cmbCompulsoryPolisType.value():
            policyTypeId = advisePolicyType(insurerId, serial)
            self.cmbCompulsoryPolisType.setValue(policyTypeId)


    def updateCompulsoryPolicyKind(self):
        serial = forceStringEx(self.edtCompulsoryPolisSerial.text())
        number = forceStringEx(self.edtCompulsoryPolisNumber.text())
        if serial.upper() in (u'ЕП', '') and len(number) == 16 and number.isdigit():
            code = '3'
        elif serial == '' and 9<=len(number)<=12 and number.isdigit():
            code = '2'
        elif serial != '' and number != '':
            code = '1'
        else:
            code = None
        if code:
            self.cmbCompulsoryPolisKind.setCode(code)


    def updateVoluntaryPolicyType(self):
        serial = forceStringEx(self.edtVoluntaryPolisSerial.text())
        insurerId = self.cmbVoluntaryPolisCompany.value()
        if serial and insurerId:
            policyTypeId = advisePolicyType(insurerId, serial)
            self.cmbVoluntaryPolisType.setValue(policyTypeId)


    def syncPolicies(self):
        self.syncPolicy(True)
        self.syncPolicy(False)


    def syncPolicy(self, isCompulsory):
        # возможные изменения в свойствах полиса
        # (edtCompulsoryPolisSerial, edtCompulsoryPolisNumber, cmbCompulsoryPolisCompany, cmbCompulsoryPolisType, edtCompulsoryPolisName, edtCompulsoryPolisNote)
        # необходимо отобразить в modelPolicies
        if isCompulsory:
            getRow = self.modelPolicies.getCurrentCompulsoryPolicyRow
            edtSerial = self.edtCompulsoryPolisSerial
            edtNumber = self.edtCompulsoryPolisNumber
            cmbPolisCompany = self.cmbCompulsoryPolisCompany
            cmbPolisType = self.cmbCompulsoryPolisType
            cmbPolisKind = self.cmbCompulsoryPolisKind
            edtPolisName = self.edtCompulsoryPolisName
            edtPolisNote = self.edtCompulsoryPolisNote
            polisBegDate = self.edtCompulsoryPolisBegDate
            polisEndDate = self.edtCompulsoryPolisEndDate
        else:
            getRow = self.modelPolicies.getCurrentVoluntaryPolicyRow
            edtSerial = self.edtVoluntaryPolisSerial
            edtNumber = self.edtVoluntaryPolisNumber
            cmbPolisCompany = self.cmbVoluntaryPolisCompany
            cmbPolisType = self.cmbVoluntaryPolisType
            cmbPolisKind = self.cmbVoluntaryPolisKind
            edtPolisName = self.edtVoluntaryPolisName
            edtPolisNote = self.edtVoluntaryPolisNote
            polisBegDate = self.edtVoluntaryPolisBegDate
            polisEndDate = self.edtVoluntaryPolisEndDate

        serial = forceStringEx(edtSerial.text())
        number = forceStringEx(edtNumber.text())
        insurerId = cmbPolisCompany.value()
        policyTypeId = cmbPolisType.value()
        if policyTypeId is None:
            cmbPolisType.setCurrentIndex(1)
            policyTypeId = cmbPolisType.value()
        policyKindId = cmbPolisKind.value()
        name = forceStringEx(edtPolisName.text())
        note = forceStringEx(edtPolisNote.text())
        begDate = polisBegDate.date()
        endDate = polisEndDate.date()

        if (number and (insurerId or name) and policyTypeId):
            row = getRow(serial, number, insurerId)
            self.modelPolicies.setValue(row, 'serial', serial)
            self.modelPolicies.setValue(row, 'number', number)
            self.modelPolicies.setValue(row, 'insurer_id', insurerId)
            self.modelPolicies.setValue(row, 'policyType_id', policyTypeId)
            self.modelPolicies.setValue(row, 'policyKind_id', policyKindId)
            self.modelPolicies.setValue(row, 'name', name)
            self.modelPolicies.setValue(row, 'note', note)
            self.modelPolicies.setValue(row, 'begDate', begDate)
            self.modelPolicies.setValue(row, 'endDate', endDate)


    def evalSexByName(self, tableName, name):
        if not self.cmbSex.currentIndex():
            detectedSex = forceInt(QtGui.qApp.db.translate(tableName, 'name', name, 'sex'))
            self.cmbSex.setCurrentIndex(detectedSex)


    def updateSocStatus(self, fieldName, value):
        # изменено какое-то поле соц. статуса из числа доп.полей (сделанных отдельными widget-ами)
        row = self.tblSocStatuses.currentIndex().row()
        self.modelSocStatuses.setValue(row, fieldName, value)


    def updateSocStatuses(self, currStatuses):
        socStatusTypeIdSet = set()
        for socStatusTypeId, default in currStatuses:
            self.modelSocStatuses.establishStatus(socStatusTypeId)
            socStatusTypeIdSet.add(socStatusTypeId)
        self.modelSocStatuses.declineUnlisted(socStatusTypeIdSet)
        self.modelSocStatuses.reset()


    def updateWorkOrganisationInfo(self):
        id = self.cmbWorkOrganisation.value()
        self.clientWorkOrgId = id
        orgInfo = getOrganisationInfo(id)
        self.edtWorkOrganisationINN.setText(orgInfo.get('INN', ''))
        self.edtWorkOrganisationOGRN.setText(orgInfo.get('OGRN', ''))
        okved = orgInfo.get('OKVED', '')
        if self.cmbWorkOKVED.orgCode != okved:
            self.edtOKVEDName.clear()
        self.cmbWorkOKVED.setOrgCode(okved)
        self.edtWorkOrganisationFreeInput.setEnabled(not id)


    def updateCompulsoryPolicyCompanyArea(self, forceAreaList=[]):
        areaList = [QtGui.qApp.defaultKLADR(),
                    self.cmbRegCity.code() if self.cmbRegCity.isEnabled() else '',
                    self.cmbLocCity.code(),
                    forceString(QtGui.qApp.db.translate('Organisation', 'id', self.cmbCompulsoryPolisCompany.value(), 'area'))
                   ]
        self.cmbCompulsoryPolisCompany.setAreaFilter(areaList+forceAreaList)
        self.updateCompulsoryPolicyType()


    def updateVoluntaryPolicyCompanyArea(self, forceAreaList=[]):
        areaList = [QtGui.qApp.defaultKLADR(),
                    self.cmbRegCity.code() if self.cmbRegCity.isEnabled() else '',
                    self.cmbLocCity.code(),
                    forceString(QtGui.qApp.db.translate('Organisation', 'id', self.cmbVoluntaryPolisCompany.value(), 'area'))
                   ]
        self.cmbVoluntaryPolisCompany.setAreaFilter(areaList+forceAreaList)
        self.updateVoluntaryPolicyType()


    def updatePolicyCompaniesArea(self, forceAreaList=[]):
        self.updateCompulsoryPolicyCompanyArea(forceAreaList)
        self.updateVoluntaryPolicyCompanyArea(forceAreaList)

    def blockQuotaWidgets(self, val):
        blockSignalList = self.grpQuotingDetails.children()
        for blockObj in blockSignalList:
            blockObj.blockSignals(val)

    def setQuotaWidgets(self, val):
        previousVal = self.edtQuotaIdentifier.isEnabled()
        if val != previousVal:
            self.setEnabledQuotaWidgets(val)
        if val:
            self.blockQuotaWidgets(True)
            record = self.__quotaRecord
            kladrCodeIndex = self.cmbQuotaKladr.getIndexByCode(forceString(record.value('regionCode')))
            if kladrCodeIndex:
                self.cmbQuotaKladr.setCurrentIndex(kladrCodeIndex)
            else:
                self.cmbQuotaKladr.setCurrentIndex(0)
            self.cmbQuotaDistrictKladr.setKladrCode(self.cmbQuotaKladr.itemData(self.cmbQuotaKladr.currentIndex()))
            self.cmbQuotaDistrictKladr.setValue(forceString(record.value('districtCode')))
            setLineEditValue(self.edtQuotaIdentifier, record, 'identifier')
            setSpinBoxValue(self.edtQuotaStage, record, 'stage')
            setLineEditValue(self.edtQuotaTicket, record, 'quotaTicket')
            setSpinBoxValue(self.edtQuotaAmount, record, 'amount')
            self.cmbQuotaMKB.setText(forceString(record.value('MKB')))
            setComboBoxValue(self.cmbQuotaRequest, record, 'request')

            directionDate = forceDate(record.value('directionDate'))
            if directionDate:
                self.edtQuotaDirectionDate.setDate(directionDate)
            else:
                self.edtQuotaDirectionDate.setDate(QDate())

            self.cmbQuotaLPU.setValue(forceInt(record.value('org_id')))
            setLineEditValue(self.edtQuotaLPUFreeInput, record, 'freeInput')

            dateRegistration = forceDate(record.value('dateRegistration'))
            if dateRegistration:
                self.edtQuotaDateRegistration.setDate(dateRegistration)
            else:
                self.edtQuotaDateRegistration.setDate(QDate())

            dateEnd = forceDate(record.value('dateEnd'))
            if dateEnd:
                self.edtQuotaDateEnd.setDate(dateEnd)
            else:
                self.edtQuotaDateEnd.setDate(QDate())

            orgStructureId = forceRef(record.value('orgStructure_id'))
            self.cmbQuotaOrgStructure.setValue(orgStructureId)
            setComboBoxValue(self.cmbQuotaStatus, record, 'status')
            setTextEditValue(self.edtQuotaStatment, record, 'statment')
            self.blockQuotaWidgets(False)


    def setEnabledQuotaWidgets(self, val):
        self.edtQuotaIdentifier.setEnabled(val)
        self.edtQuotaStage.setEnabled(val)
        self.edtQuotaTicket.setEnabled(val)
        self.edtQuotaAmount.setEnabled(val)
        self.cmbQuotaMKB.setEnabled(val)
        self.cmbQuotaRequest.setEnabled(val)
        self.edtQuotaDirectionDate.setEnabled(val)
        self.cmbQuotaLPU.setEnabled(val)
        self.edtQuotaLPUFreeInput.setEnabled(val)
        self.edtQuotaDateRegistration.setEnabled(val)
        self.edtQuotaDateEnd.setEnabled(val)
        self.cmbQuotaOrgStructure.setEnabled(val)
        self.cmbQuotaStatus.setEnabled(val)
        self.edtQuotaStatment.setEnabled(val)
        self.tblClientQuotingDiscussion.setEnabled(val)
        self.cmbQuotaKladr.setEnabled(val)
        self.cmbQuotaDistrictKladr.setEnabled(val)


    def setQuotingInfo(self):
        info = self.modelClientQuoting.info()
        limit     = info.get('limit', 0)
        used      = info.get('used', 0)
        confirmed = info.get('confirmed', 0)
        inQueue   = info.get('inQueue', 0)
        quotaTypeName = info.get('quotaTypeName', '')
        txt = u''
        txt += quotaTypeName+'\n'
        if limit:
            txt += u'Предел: %d\n' % limit
        if used:
            txt += u'Использовано: %d\n' % used
        if confirmed:
            txt += u'Подтверждено: %d\n' % confirmed
        if inQueue:
            txt += u'В очереди: %d\n' % inQueue
        self.lblInfo.setText(txt)

    def selectClientQuotingDiscussion(self, quotaId):
        db = QtGui.qApp.db
        table = self.modelClientQuotingDiscussion.table()
        cond = [table['master_id'].eq(quotaId)]
        return db.getIdList(table, 'id', cond, 'dateMessage')

    def showQuotaDiscussionEditor(self, editor):
        if editor.exec_():
            master_id = forceInt(self.__quotaRecord.value('id'))
            itemId = editor.itemId()
            if self.checkIsLastMessage(master_id, itemId):
                agreementTypeId = editor.cmbAgreementType.value()
                if agreementTypeId:
                    statusModifier = forceInt(QtGui.qApp.db.translate('rbAgreementType', 'id',
                                                         agreementTypeId, 'quotaStatusModifier'))
                    status = forceInt(self.__quotaRecord.value('status'))
                    if statusModifier and status != statusModifier-1:
                        quotaRow = self.selectionModelClientQuoting.selectedIndexes()[0].row()
                        self.__quotaRecord.setValue('status', QVariant(statusModifier-1))
                        self.cmbQuotaStatus.setCurrentIndex(statusModifier-1)
                        self.tblClientQuoting.model().emitRowChanged(quotaRow)
                        self.setSelectedQuota(quotaRow)
            idList = self.selectClientQuotingDiscussion(master_id)
            self.modelClientQuotingDiscussion.setIdList(idList, itemId)

    def newQuotaDiscussionEditor(self):
        editor = CQuotingEditorDialog(self)
        master_id = forceInt(self.__quotaRecord.value('id'))
        if not master_id:
            self.__quotaRecord.setValue('master_id', QVariant(self._id))
            row = self.tblClientQuoting.currentIndex().row()
            master_id = QtGui.qApp.db.insertRecord('Client_Quoting', self.__quotaRecord)
            quotingRecords = self.modelClientQuoting.getActualQuotingRecords(self.__quotaRecord)
            code = self.cmbRegCity.code()
            clientOKATOCode = getOKATO(code, self.cmbRegStreet.code(), self.edtRegHouse.text())
            if quotingRecords:
                for quotingRecord in quotingRecords:
                    status = forceInt(self.__quotaRecord.value('status'))
                    amount = forceInt(self.__quotaRecord.value('amount'))
                    self.modelClientQuoting.basicData[row] = [amount, status]
                    self.modelClientQuoting.changeData(quotingRecord, status, amount, amount.__add__)
                    if code:
                        self.modelClientQuoting.changeRegionData(status, amount, amount.__add__,
                                                                 code, quotingRecord)
                    if code and clientOKATOCode:
                        self.modelClientQuoting.changeDistrictData(status, amount, amount.__add__,
                                                                 (code, clientOKATOCode), quotingRecord)
        editor.setMaster_id(master_id)
        editor.cmbResponsiblePerson.setValue(QtGui.qApp.userId)
        self.showQuotaDiscussionEditor(editor)

    def editQuotaDiscussionEditor(self):
        isSelected = self.tblClientQuotingDiscussion.selectedIndexes()
        if isSelected:
            itemId = self.tblClientQuotingDiscussion.currentItemId()
            editor = CQuotingEditorDialog(self)
            editor.load(itemId)
            if editor.canBeEdit():
                self.showQuotaDiscussionEditor(editor)
            else:
                QtGui.QMessageBox.critical(self, u'Внимание!',
                                    u'Нет прав на редактирование записи',
                                    QtGui.QMessageBox.Ok,
                                    QtGui.QMessageBox.Ok
                                    )
        else:
            self.newQuotaDiscussionEditor()

    def deleteQuotaDiscussionMessage(self):
        isSelected = self.tblClientQuotingDiscussion.selectedIndexes()
        if isSelected:
            itemId = self.tblClientQuotingDiscussion.currentItemId()
            table = self.modelClientQuotingDiscussion.table()
            responsiblePersonId = forceInt(QtGui.qApp.db.translate(table,
                                                          'id', itemId, 'responsiblePerson_id'))
            if responsiblePersonId == QtGui.qApp.userId:
                if QtGui.QMessageBox.question(self, u'Подтверждение удаления',
                                    u'Вы уверены что хотите удалить сообщение?',
                                    QtGui.QMessageBox.Yes|QtGui.QMessageBox.No,
                                    QtGui.QMessageBox.No
                                    ) == QtGui.QMessageBox.Yes:
                    QtGui.qApp.db.deleteRecord(table, table['id'].eq(itemId))
                    idList = self.selectClientQuotingDiscussion(forceInt(self.__quotaRecord.value('id')))
                    self.modelClientQuotingDiscussion.setIdList(idList)

            else:
                QtGui.QMessageBox.critical(self, u'Внимание!',
                                    u'Нет прав на удаление записи',
                                    QtGui.QMessageBox.Ok,
                                    QtGui.QMessageBox.Ok
                                    )

    def deleteClientQuotingRows(self):
        rows = self.tblClientQuoting.getSelectedRows()
        rows.sort(reverse=True)
        self.checkClientAttachesForDeleting(rows)
        for row in rows:
            self.tblClientQuoting.model().removeRow(row)

    def checkClientAttachesForDeleting(self, rows):
        db = QtGui.qApp.db
        attachTypeId = db.translate('rbAttachType', 'code', '9', 'id')
        clientAttachRecords = self.modelAttaches.items()
        quotaItems = self.tblClientQuoting.model().items()
        attachQuotes = {}
        clientAttachRowsForDeleting = []
        for attachRow, clientAttchRecord in enumerate(clientAttachRecords):
            if clientAttchRecord.value('attachType_id') == attachTypeId:
                quotaList = attachQuotes.get(attachRow, [])
                attachBegDate = forceDate(clientAttchRecord.value('begDate'))
                attachEndDate = forceDate(clientAttchRecord.value('endDate'))
                for quotaRow, quotaItem in enumerate(quotaItems):
                    quotaBegDate = forceDate(quotaItem.value('dateRegistration'))
                    quotaEndDate = forceDate(clientAttchRecord.value('dateEnd'))
                    if attachEndDate:
                        if quotaBegDate <= attachEndDate:
                            if quotaEndDate:
                                if quotaEndDate >= attachBegDate:
                                    quotaList.append(quotaRow)
                                    attachQuotes[attachRow] = quotaList
                            else:
                                quotaList.append(quotaRow)
                                attachQuotes[attachRow] = quotaList
                    else:
                        if quotaEndDate:
                            if quotaEndDate >= attachBegDate:
                                quotaList.append(quotaRow)
                                attachQuotes[attachRow] = quotaList
                        else:
                            quotaList.append(quotaRow)
                            attachQuotes[attachRow] = quotaList
        for attachRow, clientAttchRecord in enumerate(clientAttachRecords):
            if clientAttchRecord.value('attachType_id') == attachTypeId:
                quotaList = attachQuotes.get(attachRow, [])
                for row in rows:
                    if row in quotaList:
                        quotaList.pop(quotaList.index(row))
                if len(quotaList) == 0:
                    clientAttachRowsForDeleting.append(attachRow)
        if clientAttachRowsForDeleting:
            if QtGui.QMessageBox.question(self,
                                       u'Внимание!',
                                       u'Удалить соответствующие прикрепления пациента?',
                                       QtGui.QMessageBox.Yes|QtGui.QMessageBox.No,
                                       QtGui.QMessageBox.No) == QtGui.QMessageBox.Yes:
                clientAttachRowsForDeleting.sort(reverse=True)
                for attachRow in clientAttachRowsForDeleting:
                    self.tblAttaches.model().removeRow(attachRow)

    def checkIsLastMessage(self, quotingId, messageId):
        db = QtGui.qApp.db
        table = db.table('Client_QuotingDiscussion')
        cond = [ table['master_id'].eq(quotingId) ]
        stmt = db.selectStmt(table, 'MAX(id)', cond)
        query = db.query(stmt)
        if query.first():
            lastId = query.value(0).toInt()[0]
            if lastId == messageId:
                return True
        return False

    def setSelectedQuota(self, row=None):
        if row is None:
            items = self.modelClientQuoting.items()
            if items:
                row = 0
            else:
                return
        index = self.modelClientQuoting.index(row, 0)
        self.selectionModelClientQuoting.setCurrentIndex(index, QtGui.QItemSelectionModel.Rows)
        self.tblClientQuoting.setCurrentIndex(index)


    def setMKBTextInfo(self, text):
        if len(text)>1:
            if text[-1:] == '.':
                text = text[:-1]
            info = forceString(QtGui.qApp.db.translate('MKB', 'DiagID', text, 'DiagName'))
            self.lblMKBText.setText('<i>%s</i>'%info)
            if info:
                return True
            else:
                return False
        else:
            self.lblMKBText.clear()
            return False


    @pyqtSignature('int')
    def on_tabWidget_currentChanged(self, index):
        widget = self.tabWidget.widget(index)
        if widget == self.tabRelations:
            self.modelDirectRelations.setRegAddressInfo(self.getParamsRegAddress())
            self.modelDirectRelations.setLogAddressInfo(self.getParamsLocAddress())
            self.modelBackwardRelations.setRegAddressInfo(self.getParamsRegAddress())
            self.modelBackwardRelations.setLogAddressInfo(self.getParamsLocAddress())
        else:
            self.regAddressInfo = {}
            self.locAddressInfo = {}

            if widget == self.tabDocs:
#                self.syncIdentificationDocs()
                self.syncPolicies()

            elif widget == self.tabAddresses:
                self.modelClientAddressesReg.setClientId(self.itemId())
                self.modelClientAddressesLoc.setClientId(self.itemId())

        if widget:
            focusProxy = widget.focusProxy()
            if focusProxy:
                focusProxy.setFocus(Qt.OtherFocusReason)


    @pyqtSignature('')
    def on_edtFirstName_editingFinished(self):
        firstName = forceStringEx(self.edtFirstName.text())
        self.evalSexByName('rdFirstName', firstName)


    @pyqtSignature('')
    def on_edtPatrName_editingFinished(self):
        patrName = forceStringEx(self.edtPatrName.text())
        self.evalSexByName('rdPatrName', patrName)


    @pyqtSignature('')
    def on_btnCopyPrevAddress_clicked(self):
        if CClientEditDialog.prevAddress:
            reg, freeInput, loc = CClientEditDialog.prevAddress
            self.setRegAddress(reg, freeInput)
            self.setLocAddress(loc)
        elif self.regAddressInfo or self.locAddressInfo:
            self.setRegAddressRelation(self.regAddressInfo)
            self.setLocAddressRelation(self.locAddressInfo)


    @pyqtSignature('bool')
    def on_chkRegKLADR_toggled(self, checked):
        self.edtRegFreeInput.setEnabled(not checked)
        self.cmbRegCity.setEnabled(checked)
        self.cmbRegStreet.setEnabled(checked)
        self.edtRegHouse.setEnabled(checked)
        self.edtRegCorpus.setEnabled(checked)
        self.edtRegFlat.setEnabled(checked)


    @pyqtSignature('int')
    def on_cmbRegCity_currentIndexChanged(self,  val):
        code = self.cmbRegCity.code()
        self.cmbRegStreet.setCity(code)
        self.updatePolicyCompaniesArea()


    @pyqtSignature('')
    def on_btnCopy_clicked(self):
        code = self.cmbRegCity.code()
        self.cmbLocCity.setCode(code)
        self.cmbLocStreet.setCity(code)
        self.cmbLocStreet.setCode(self.cmbRegStreet.code())
        self.edtLocHouse.setText(self.edtRegHouse.text())
        self.edtLocCorpus.setText(self.edtRegCorpus.text())
        self.edtLocFlat.setText(self.edtRegFlat.text())


    @pyqtSignature('int')
    def on_cmbLocCity_currentIndexChanged(self, index):
        code = self.cmbLocCity.code()
        self.cmbLocStreet.setCity(code)
        self.updatePolicyCompaniesArea()


    @pyqtSignature('int')
    def on_cmbDocType_currentIndexChanged(self, index):
        documentTypeId = self.cmbDocType.value()
        if documentTypeId:
            documentTypeDescr = getDocumentTypeDescr(documentTypeId)
            self.lblDocSerial.setEnabled(True)
            self.edtDocSerialLeft.setEnabled(True)
            self.edtDocSerialLeft.setRegExp(documentTypeDescr.leftPartRegExp)
            self.edtDocSerialRight.setEnabled(documentTypeDescr.hasRightPart)
            self.edtDocSerialRight.setRegExp(documentTypeDescr.rightPartRegExp)
            self.lblDocNumber.setEnabled(True)
            self.edtDocNumber.setEnabled(True)
            self.edtDocNumber.setRegExp(documentTypeDescr.numberRegExp)
            self.lblDocDate.setEnabled(True)
            self.edtDocDate.setEnabled(True)
            self.lblDocOrigin.setEnabled(True)
            self.edtDocOrigin.setEnabled(True)
        else:
            self.lblDocSerial.setEnabled(False)
            self.edtDocSerialLeft.setEnabled(False)
            self.edtDocSerialRight.setEnabled(False)
            self.lblDocNumber.setEnabled(False)
            self.edtDocNumber.setEnabled(False)
            self.lblDocDate.setEnabled(False)
            self.edtDocDate.setEnabled(False)
            self.lblDocOrigin.setEnabled(False)
            self.edtDocOrigin.setEnabled(False)
        self.syncIdentificationDocs()


    @pyqtSignature('QString')
    def on_edtDocSerialLeft_textChanged(self, text):
        self.syncIdentificationDocs()


    @pyqtSignature('QString')
    def on_edtDocSerialRight_textChanged(self, text):
        self.syncIdentificationDocs()


    @pyqtSignature('QString')
    def on_edtDocNumber_textChanged(self, text):
        self.syncIdentificationDocs()


    @pyqtSignature('QDate')
    def on_edtDocDate_dateChanged(self, date):
        self.syncIdentificationDocs()


    @pyqtSignature('QString')
    def on_edtDocOrigin_textChanged(self, text):
        self.syncIdentificationDocs()


    @pyqtSignature('const QModelIndex&, const QModelIndex&')
    def on_modelIdentificationDocs_dataChanged(self, topLeft, bottomRight):
        items = self.modelIdentificationDocs.items()
        row = bottomRight.row()
        if row >=0 and row == len(items)-1:
            self.setDocumentRecord(items[-1])


    @pyqtSignature('const QModelIndex&, int, int')
    def on_modelIdentificationDocs_rowsRemoved(self, parent, start, end):
        items = self.modelIdentificationDocs.items()
        self.setDocumentRecord(items[-1] if items else None)


    @pyqtSignature('')
    def on_btnSearchPolicy_clicked(self):
        policy, attach, msg = QtGui.qApp.identService(forceStringEx(self.edtFirstName.text()),
                                                      forceStringEx(self.edtLastName.text()),
                                                      forceStringEx(self.edtPatrName.text()),
                                                      self.cmbSex.currentIndex(),
                                                      self.edtBirthDate.date(),
                                                      forceStringEx(self.edtSNILS.text()).replace('-','').replace(' ',''),
                                                      forceStringEx(self.edtCompulsoryPolisSerial.text()),
                                                      forceStringEx(self.edtCompulsoryPolisNumber.text()),
                                                      forceStringEx(self.edtDocSerialLeft.text()+' '+self.edtDocSerialRight.text()),
                                                      forceStringEx(self.edtDocNumber.text()))
        if policy:
            db = QtGui.qApp.db
            hicId = policy.hicId
            if hicId:
                hicName = forceString(db.translate('Organisation', 'id', hicId, 'shortName'))
            else:
                hicName = u'в справочнике не найдено: %s'%policy.hicName

            policyTypeCode = policy.typeCode
            policyTypeId = forceRef(db.translate('rbPolicyType', 'code', policyTypeCode, 'id'))
            if policyTypeId:
                policyTypeName = forceString(db.translate('rbPolicyType', 'id', policyTypeId, 'name'))
            else:
                policyTypeName = '-'

            policyKindCode = policy.kindCode
            policyKindId = forceRef(db.translate('rbPolicyKind', 'code', policyKindCode, 'id'))
            if policyKindId:
                policyKindName = forceString(db.translate('rbPolicyKind', 'id', policyKindId, 'name'))
            else:
                policyKindName = '-'

            if attach:
                attachHtml = u'<br/><b>Прикрепление</b>' \
                             u'<table>' \
                             + ''.join(u'<tr><td>%s:</td><td>%s</td></tr>' %
                                        ( escape(attachItem.get('netName')),
                                          escape(attachItem.get('orgName'))
                                        )
                                     for attachItem in attach
                                    )\
                             +u'</table><br/>'
            else:
                attachHtml = ''

            html = u'<html><body>'\
                   u'<b>Найден полис</b><br/>'\
                   u'<table>'\
                   u'<tr><td>СМО:</td><td>%s</td></tr>'\
                   u'<tr><td>серия:</td><td>%s</td></tr>'\
                   u'<tr><td>номер:</td><td>%s</td></tr>'\
                   u'<tr><td>тип:</td><td>%s</td></tr>'\
                   u'<tr><td>вид:</td><td>%s</td></tr>'\
                   u'<tr><td>действителен:</td><td>с %s по %s</td></tr>'\
                   u'</table><br/>'\
                   u'%s'\
                   u'<br/><b>Обновить сведения о полисе?</b>'\
                   u'</body></html>' % ( escape(hicName),
                            escape(policy.policySerial),
                            escape(policy.policyNumber),
                            escape(policyTypeName),
                            escape(policyKindName),
                            escape(forceString(policy.begDate) or '-'),
                            escape(forceString(policy.endDate) or '-'),
                            attachHtml
                          )


            replace = QtGui.QMessageBox.information(self,
                u'Поиск полиса и прикрепления',
                html,
                QtGui.QMessageBox.Ok|QtGui.QMessageBox.Cancel,
                QtGui.QMessageBox.Ok)
            if replace == QtGui.QMessageBox.Ok:
                self.cmbCompulsoryPolisCompany.setValue(hicId)
                self.edtCompulsoryPolisSerial.setText(policy.policySerial)
                self.edtCompulsoryPolisNumber.setText(policy.policyNumber)
                self.cmbCompulsoryPolisType.setValue(policyTypeId)
                self.cmbCompulsoryPolisKind.setValue(policyKindId)
                self.syncPolicy(True)
                row = self.modelPolicies.getCurrentCompulsoryPolicyRow(policy.policySerial, policy.policyNumber)
                self.modelPolicies.setValue(row, 'begDate', toVariant(policy.begDate))
                self.modelPolicies.setValue(row, 'endDate', toVariant(policy.endDate))
                begDate = policy.begDate or QDate()
                endDate = policy.endDate or QDate()
                self.edtCompulsoryPolisBegDate.setDate(begDate)
                self.edtCompulsoryPolisEndDate.setDate(endDate)
        else:
            QtGui.QMessageBox.information(self,
                u'Поиск полиса',
                u'Полис не найден' if not msg else u'Произошла ошибка\n%s' % msg,
                QtGui.QMessageBox.Ok, QtGui.QMessageBox.Ok)


    @pyqtSignature('QString')
    def on_edtCompulsoryPolisSerial_textEdited(self, text):
        self.cmbCompulsoryPolisCompany.setSerialFilter(text)
        self.updateCompulsoryPolicyType()
        self.updateCompulsoryPolicyKind()


    @pyqtSignature('QString')
    def on_edtCompulsoryPolisNumber_textEdited(self, text):
        if not self.cmbCompulsoryPolisType.value():
            self.cmbCompulsoryPolisType.setCurrentIndex(1)
        self.updateCompulsoryPolicyKind()


    @pyqtSignature('int')
    def on_cmbCompulsoryPolisCompany_currentIndexChanged(self, index):
        self.updateCompulsoryPolicyType()


    @pyqtSignature('QString')
    def on_edtVoluntaryPolisSerial_textEdited(self, text):
        self.cmbVoluntaryPolisCompany.setSerialFilter(text)
        self.updateVoluntaryPolicyType()


    @pyqtSignature('QString')
    def on_edtVoluntaryPolisNumber_textEdited(self, text):
        if not self.cmbVoluntaryPolisType.value():
            self.cmbVoluntaryPolisType.setCurrentIndex(1)


    @pyqtSignature('int')
    def on_cmbVoluntaryPolisCompany_currentIndexChanged(self, index):
        self.updateVoluntaryPolicyType()


    @pyqtSignature('')
    def on_modelPolicies_policyChanged(self):
        items = self.modelPolicies.items()
        self.clientPolicyInfoList = []
        for isCompulsory in (True, False):
            row = self.modelPolicies.getCurrentPolicyRowInt(isCompulsory)
            policyRecord = items[row] if 0<=row<len(items) else None
            self.setPolicyRecord(policyRecord, isCompulsory)
            if policyRecord:
                self.clientPolicyInfoList.append((forceRef(policyRecord.value('insurer_id')), forceRef(policyRecord.value('policyType_id'))))


    @pyqtSignature('')
    def on_btnSocStatusUpdate_clicked(self):
        sorry()
#        lastName = forceStringEx(self.edtLastName.text())
#        firstName= forceStringEx(self.edtFirstName.text())
#        patrName = forceStringEx(self.edtPatrName.text())
#        SNILS = forceStringEx(self.edtSNILS.text()).replace('-','').replace(' ','')
#        result, categories = getBenefitCategories(lastName, firstName, patrName, SNILS)
#        requestOk, isFound = checkBenefitCategoriesResult(result)
#        if result:
#            msg = u'Ответ сервера: ' + result
#        else:
#            msg = u'Ответ сервера не получен'
#        if requestOk:
#            QtGui.QMessageBox.information(self, u'Обновление соц.статуса', msg, QtGui.QMessageBox.Close)
#            if isFound:
#                l = []
#                db = QtGui.qApp.db
#                for code, default in categories:
#                    socStatusTypeId = forceRef(db.translate('rbSocStatusType', 'code', code, 'id'))
#                    if not socStatusTypeId:
#                        QtGui.QMessageBox.critical( self, u'Обновление соц.статуса', u'Получен неизвестный код льготы "%s"' % code, QtGui.QMessageBox.Close)
#                        return
#                    l.append( (socStatusTypeId, default) )
#                self.updateSocStatuses(l)
#        else:
#            QtGui.QMessageBox.critical( self, u'Обновление соц.статуса', msg, QtGui.QMessageBox.Close)


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelSocStatuses_currentChanged(self, current, previous):
        dirty = self.isDirty()
        row = current.row()
#        column = current.column()
        if 0<=row<len(self.modelSocStatuses.items()):
            docTypeId, serial, number, date, origin = self.modelSocStatuses.getDocInfo(row)
            self.frmSocStatusDocument.setEnabled(True)
            self.cmbSocStatusDocType.setEnabled(True)
            if not docTypeId:
                docTypeId, serial, number, date, origin = None, '', '', QDate(), ''
                item = self.modelSocStatuses.items()[row]
#                documentId = forceRef(item.value('document_id'))
                socStatusTypeId = forceRef(item.value('socStatusType_id'))
                if socStatusTypeId:
                    db = QtGui.qApp.db
                    tableRBSocStatusType = db.table('rbSocStatusType')
                    record = db.getRecordEx(tableRBSocStatusType, [tableRBSocStatusType['documentType_id']], [tableRBSocStatusType['id'].eq(socStatusTypeId)])
                    if record:
                        docTypeId = forceRef(record.value('documentType_id'))
                        self.edtSocStatusDocSerial.setFocus(Qt.ShortcutFocusReason)
        else:
            docTypeId, serial, number, date, origin = None, '', '', QDate(), ''
            self.cmbSocStatusDocType.setEnabled(False)
            self.frmSocStatusDocument.setEnabled(False)
        self.cmbSocStatusDocType.setValue(docTypeId)
        self.edtSocStatusDocSerial.setText(serial)
        self.edtSocStatusDocNumber.setText(number)
        self.edtSocStatusDocDate.setDate(date)
        self.edtSocStatusDocOrigin.setText(origin)
        self.setIsDirty(dirty)


    @pyqtSignature('int')
    def on_cmbSocStatusDocType_currentIndexChanged(self,  index):
        docTypeId = self.cmbSocStatusDocType.value()
        docTypeOk = bool(docTypeId)
        self.edtSocStatusDocSerial.setEnabled(docTypeOk)
        self.edtSocStatusDocSerial.setFocus(Qt.ShortcutFocusReason)
        self.edtSocStatusDocNumber.setEnabled(docTypeOk)
        self.edtSocStatusDocDate.setEnabled(docTypeOk)
        self.edtSocStatusDocOrigin.setEnabled(docTypeOk)
        self.updateSocStatus('documentType_id', docTypeId)


    @pyqtSignature('QString')
    def on_edtSocStatusDocSerial_textEdited(self, text):
        self.updateSocStatus('serial', text)


    @pyqtSignature('QString')
    def on_edtSocStatusDocNumber_textEdited(self, text):
        self.updateSocStatus('number', text)


    @pyqtSignature('QDate')
    def on_edtSocStatusDocDate_dateChanged(self, date):
        self.updateSocStatus('date', date)


    @pyqtSignature('QString')
    def on_edtSocStatusDocOrigin_textEdited(self, text):
        self.updateSocStatus('origin', text)


    @pyqtSignature('')
    def on_btnCopyPrevWork_clicked(self):
        if CClientEditDialog.prevWork:
            self.setWork(CClientEditDialog.prevWork)


    @pyqtSignature('int')
    def on_cmbWorkOrganisation_currentIndexChanged(self, index):
        self.updateWorkOrganisationInfo()


    @pyqtSignature('')
    def on_btnSelectWorkOrganisation_clicked(self):
        orgId = selectOrganisation(self, self.cmbWorkOrganisation.value(), False)
        self.cmbWorkOrganisation.updateModel()
        if orgId:
            self.setIsDirty()
            self.cmbWorkOrganisation.setValue(orgId)


    @pyqtSignature('')
    def on_btnRegHouseList_clicked(self):
        streetCode = forceString(self.cmbRegStreet.code())
        if not streetCode:
            cityCode = forceString(self.cmbRegCity.code())
            streetCode = cityCode + u'000000'
        self.getHouseList(streetCode, 0)


    @pyqtSignature('')
    def on_btnLocHouseList_clicked(self):
        streetCode = forceString(self.cmbLocStreet.code())
        if not streetCode:
            cityCode = forceString(self.cmbLocCity.code())
            streetCode = cityCode + u'000000'
        self.getHouseList(streetCode, 1)

    def checkHouseList(self, streetCode, typeAddress):
        result = True
        if streetCode:
            if not typeAddress:
                house = forceString(self.edtRegHouse.text())
                corp = forceString(self.edtRegCorpus.text())
                result = self.clientRegHousesList.checkHouse(house, corp)
            else:
                house = forceString(self.edtLocHouse.text())
                corp = forceString(self.edtLocCorpus.text())
                result = self.clientLocHousesList.checkHouse(house, corp)
        return result

    def getHouseListCheckResult(self, typeAddress):
        dialog = self.getHouseListDialog(typeAddress)
        return dialog.getHouseListCheckResult()


    def getHouseList(self, streetCode, typeAddress):
        if streetCode:
            #dialog = self.getHouseListDialog(typeAddress, streetCode)
            if not typeAddress:
                house = forceString(self.edtRegHouse.text())
                corp = forceString(self.edtRegCorpus.text())
                house, corp = self.clientRegHousesList.showHousesList(streetCode, house, corp)
                self.edtRegHouse.setText(house)
                self.edtRegCorpus.setText(corp)
            else:
                house = forceString(self.edtLocHouse.text())
                corp = forceString(self.edtLocCorpus.text())
                house, corp = self.clientLocHousesList.showHousesList(streetCode, house, corp)
                self.edtLocHouse.setText(house)
                self.edtLocCorpus.setText(corp)

    @pyqtSignature('int')
    def on_cmbWorkOKVED_currentIndexChanged(self,  index):
        self.edtOKVEDName.setText(getOKVEDName(self.cmbWorkOKVED.text()))
        self.edtOKVEDName.setCursorPosition(0)


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelWorkHurts_currentChanged(self, current, previous):
        dirty = self.isDirty()
        row = current.row()
        factors = self.modelWorkHurts.factors(row)
        self.modelWorkHurtFactors.setItems(factors)
        self.setIsDirty(dirty)


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelClientQuoting_currentChanged(self, current, previous):
        row = current.row()
        modelItems = self.modelClientQuoting.items()
        if row < len(modelItems):
            self.__quotaRecord = modelItems[row]
            self.modelClientQuotingDiscussion.setIdList(
                    self.selectClientQuotingDiscussion(
                    forceInt(self.__quotaRecord.value('id'))))
            self.modelClientQuoting.loadItemsInfo(row)
            self.setQuotaWidgets(True)
        else:
            self.modelClientQuotingDiscussion.setIdList([])
            self.setEnabledQuotaWidgets(False)
            self.__quotaRecord = None
            self.modelClientQuoting.loadItemsInfo(None)
        self.setQuotingInfo()

    @pyqtSignature('QString')
    def on_edtQuotaIdentifier_textEdited(self, text):
        self.__quotaRecord.setValue('identifier', QVariant(text))


    @pyqtSignature('int')
    def on_edtQuotaStage_valueChanged(self, val):
        self.__quotaRecord.setValue('stage', QVariant(val))
        index = self.tblClientQuoting.currentIndex()
        if index.isValid():
            self.modelClientQuoting.emitRowChanged(index.row())


    @pyqtSignature('QString')
    def on_edtQuotaTicket_textEdited(self, text):
        self.__quotaRecord.setValue('quotaTicket', QVariant(text))


    @pyqtSignature('int')
    def on_edtQuotaAmount_valueChanged(self, val):
        self.__quotaRecord.setValue('amount', QVariant(val))


    @pyqtSignature('QString')
    def on_cmbQuotaMKB_textChanged(self, text):
        self.__quotaRecord.setValue('MKB', QVariant(text))
        if self.setMKBTextInfo(text):
            index = self.tblClientQuoting.currentIndex()
            if index.isValid():
                self.modelClientQuoting.emitRowChanged(index.row())

    @pyqtSignature('int')
    def on_cmbQuotaRequest_currentIndexChanged(self, index):
        self.__quotaRecord.setValue('request', QVariant(index))


    @pyqtSignature('QDate')
    def on_edtQuotaDirectionDate_dateChanged(self, date):
        self.__quotaRecord.setValue('directionDate', QVariant(date))


    @pyqtSignature('int')
    def on_cmbQuotaLPU_currentIndexChanged(self, index):
        val = self.cmbQuotaLPU.value()
        self.__quotaRecord.setValue('org_id', QVariant(val))


    @pyqtSignature('QString')
    def on_edtQuotaLPUFreeInput_textEdited(self, text):
        self.__quotaRecord.setValue('freeInput', QVariant(text))


    @pyqtSignature('QDate')
    def on_edtQuotaDateRegistration_dateChanged(self, date):
        self.__quotaRecord.setValue('dateRegistration', QVariant(date))


    @pyqtSignature('QDate')
    def on_edtQuotaDateEnd_dateChanged(self, date):
        self.__quotaRecord.setValue('dateEnd', QVariant(date))
#        self.modelClientQuoting.setIsActiveByDate(date)


    @pyqtSignature('int')
    def on_cmbQuotaOrgStructure_currentIndexChanged(self, index):
        val = self.cmbQuotaOrgStructure.value()
        self.__quotaRecord.setValue('orgStructure_id', QVariant(val))


    @pyqtSignature('int')
    def on_cmbQuotaStatus_currentIndexChanged(self, index):
        self.__quotaRecord.setValue('status', QVariant(index))
        tblIndex = self.tblClientQuoting.currentIndex()
        if tblIndex.isValid():
            self.modelClientQuoting.emitRowChanged(tblIndex.row())

    @pyqtSignature('')
    def on_edtQuotaStatment_textChanged(self):
        text = self.edtQuotaStatment.toPlainText()
        self.__quotaRecord.setValue('statment', QVariant(text))


    @pyqtSignature('int')
    def on_cmbQuotaKladr_currentIndexChanged(self, index):
        data = self.cmbQuotaKladr.itemData(index)
        self.__quotaRecord.setValue('regionCode', data)
        self.cmbQuotaDistrictKladr.setKladrCode(data)


    @pyqtSignature('int')
    def on_cmbQuotaDistrictKladr_currentIndexChanged(self, index):
        okato = self.cmbQuotaDistrictKladr.value()
        self.__quotaRecord.setValue('districtCode', toVariant(okato))


    @pyqtSignature('bool')
    def on_chkDeathDate_clicked(self, state):
        self.edtDeathDate.setEnabled(state)
        self.edtDeathTime.setEnabled(state)
        if state:
            self.edtDeathDate.setDate(QDate.currentDate())
        else:
            self.edtDeathDate.setDate(None)
            self.edtDeathTime.setTime(QTime(0, 0, 0, 0))


def checkSNILSEntered(self):
    SNILS = unformatSNILS(forceStringEx(self.edtSNILS.text()))
    if SNILS:
        if len(SNILS) != 11:
            return self.checkInputMessage(u'CНИЛС', True, self.edtSNILS)
        elif not checkSNILS(SNILS):
            fixedSNILS = formatSNILS(fixSNILS(SNILS))
            res = QtGui.QMessageBox.question( self,
                                             u'Внимание!',
                                             u'СНИЛС указан с ошибкой.\nПравильный СНИЛС %s\nИсправить?' % fixedSNILS,
                                             QtGui.QMessageBox.Yes|QtGui.QMessageBox.No,
                                             QtGui.QMessageBox.Yes)
            if res == QtGui.QMessageBox.Yes:
                self.edtSNILS.setText(fixedSNILS)
            else:
                self.edtSNILS.setFocus(Qt.ShortcutFocusReason)
                return False
    return True

#    @pyqtSignature('')
#    def on_btnPrint_clicked(self):
#        if not self.checkDataEntered():
#            return
#        db = QtGui.qApp.db
#        record = self.getRecord()
#
#        printer = QtGui.QPrinter(QtGui.QPrinter.HighResolution)
#
#        printer.setFullPage(True)
##        printer.setOrientation(QtGui.QPrinter.Landscape)
#        printer.setOrientation(QtGui.QPrinter.Portrait)
#        printer.setPageSize(QtGui.QPrinter.A5)
##
#        if QtGui.qApp.keyboardModifiers() & Qt.ControlModifier:
#            dlg = QtGui.QPrintDialog(printer, self)
#            dlg.setWindowTitle(u'Печать информации о пациенте')
#            if dlg.exec_() != QtGui.QDialog.Accepted:
#                return
#
#        patientId = db.insertOrUpdate(tblPatients, record)
#        record.setValue(patId, toVariant(patientId))
#        self.setIsDirty(False)
#
#        textDocument = QtGui.QTextDocument()
#        textDocument.setHtml(getPatientInfoAsHTMLEx2(record))
##        textDocument.setPageSize(QSizeF(210.0*2, 148.0*2))
##        textDocument.setPageSize(QSizeF(printer.width(), printer.height()))
##        body = QRectF(0, 0, p.device()->width(), p.device()->height());
#
##        textDocument.setDefaultFont(font)
#        textDocument.print_(printer)
#
#    @pyqtSignature('')
#    def on_btnPrintTalon_clicked(self):
#        if not self.checkDataEntered():
#            return
#        db = QtGui.qApp.db
#        record = self.getRecord()
#
#        printer = QtGui.QPrinter(QtGui.QPrinter.HighResolution)
#
#        printer.setFullPage(True)
##        printer.setOrientation(QtGui.QPrinter.Landscape)
#        printer.setOrientation(QtGui.QPrinter.Portrait)
#        printer.setPageSize(QtGui.QPrinter.A5)
##
#        if QtGui.qApp.keyboardModifiers() & Qt.ControlModifier:
#            dlg = QtGui.QPrintDialog(printer, self)
#            dlg.setWindowTitle(u'Печать заготовки стат.талона')
#            if dlg.exec_() != QtGui.QDialog.Accepted:
#                return
#
#        patientId = db.insertOrUpdate(tblPatients, record)
#        record.setValue(patId, toVariant(patientId))
#        self.setIsDirty(False)
#
#        textDocument = QtGui.QTextDocument()
#        textDocument.setHtml(getPatientInfoAsHTMLEx3(record))
##        textDocument.setPageSize(QSizeF(210.0*2, 148.0*2))
##        textDocument.setPageSize(QSizeF(printer.width(), printer.height()))
##        body = QRectF(0, 0, p.device()->width(), p.device()->height());
#
##        textDocument.setDefaultFont(font)
#        textDocument.print_(printer)


#class CPatientDupsListDialog(CDialogBase, Ui_PatientDupsListDialog):
#    def __init__(self, parent, idList):
#        CDialogBase.__init__(self, parent)
#        self.setupUi(self)
#        self.model = CPatientsTableModel(self)
#        self.model.setIdList(idList)
#        self.tblPatients.setModel(self.model)
#        if len(idList) > 0:
#            self.tblPatients.selectRow(0)
#        self.__patientId = None
#
#    def patientId(self):
#        return self.__patientId
#
#    @pyqtSignature('QModelIndex')
#    def on_tblPatients_doubleClicked(self, index):
#        self.on_btnSelect_clicked()
#
#    @pyqtSignature('')
#    def on_btnOk_clicked(self):
#        self.reject()
#
#    @pyqtSignature('')
#    def on_btnIgnore_clicked(self):
#        self.accept()
#
#    @pyqtSignature('')
#    def on_btnSelect_clicked(self):
#        row = self.tblPatients.currentIndex().row()
#        if 0<=row and row<len(self.model.idList()):
#            self.__patientId = self.model.idList()[row]
#        self.accept()

#
###########################################################################
#

class CSocStatusTypeInDocTableCol(CRBInDocTableCol):
    def setEditorData(self, editor, value, record):
        socStatusClassId = forceRef(record.value('socStatusClass_id'))
        filter = ('class_id = %d' % socStatusClassId) if socStatusClassId else 'class_id is NULL'
        editor.setFilter(filter)
#        editor.setShowFields(self.showFields)
        editor.setValue(forceInt(value))


class CSocStatusesModel(CInDocTableModel):
    documentFields = 'documentType_id,serial,number,date,origin'

    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'ClientSocStatus', 'id', 'client_id', parent)
#        self.addCol(CRBTreeInDocTableCol(u'Класс', 'socStatusType_id', 50, 'rbSocStatusClass', showFields = CRBComboBox.showNameAndCode))
        self.addCol(CRBInDocTableCol(u'Класс', 'socStatusClass_id', 50, 'rbSocStatusClass', showFields = CRBComboBox.showNameAndCode, filter='group_id is NULL'))
        self.addCol(CSocStatusTypeInDocTableCol(u'Тип',   'socStatusType_id', 50, 'vrbSocStatusType', showFields = CRBComboBox.showNameAndCode))
        self.addCol(CDateInDocTableCol(u'Дата начала',     'begDate', 15, canBeEmpty=True))
        self.addCol(CDateInDocTableCol(u'Дата окончания',  'endDate', 15, canBeEmpty=True))
        self.addHiddenCol('document_id')


    def getEmptyRecord(self):
        result = CInDocTableModel.getEmptyRecord(self)
        self._addExCols(result)
        return result


    def loadItems(self, id):
        CInDocTableModel.loadItems(self, id)
        for item in self.items():
            self._addExCols(item)
            self._loadDocInfo(item)


    def saveItems(self, clientId):
        documentIdList = []
        for item in self.items():
            documentId = self._saveDocInfo(item, clientId)
            if documentId:
                documentIdList.append(documentId)
        items = self.items()
        savedItems = []
        for record in items:
            savedItems.append(QtSql.QSqlRecord(item))
            for n in self.documentFields.split(','):
                record.remove(record.indexOf(n))
        CInDocTableModel.saveItems(self, clientId)
        for record, savedRecord in zip(items, savedItems):
            for n in self.documentFields.split(','):
                record.append(savedRecord.field(n))

    def indexOfStatus(self, socStatusTypeId):
        for i, item in enumerate(self.items()):
            if forceRef(item.value('socStatusType_id')) == socStatusTypeId:
                return i
        return -1


    def establishStatus(self, socStatusTypeId):
        i = self.indexOfStatus(socStatusTypeId)
        if i>=0:
            item = self.items()[i]
            item.setValue('endDate', toVariant(None))
        else:
            item = self.getEmptyRecord()
            item.setValue('socStatusType_id', toVariant(socStatusTypeId))
            item.setValue('begDate', toVariant(QDate.currentDate()))
            item.setValue('endDate', toVariant(None))
            self.items().append(item)


    def declineUnlisted(self, socStatusTypeIdList):
        yesterday = QDate.currentDate().addDays(-1)
        for item in self.items():
            if forceRef(item.value('socStatusType_id')) not in socStatusTypeIdList:
                endDate = forceDate(item.value('endDate'))
                if not endDate or yesterday<endDate:
                    item.setValue('endDate', toVariant(yesterday))


# todo: clean unused docs
    def getDocInfo(self, row):
        record = self.items()[row]
        return [ forceRef(record.value('documentType_id')),
                 forceString(record.value('serial')),
                 forceString(record.value('number')),
                 forceDate(record.value('date')),
                 forceString(record.value('origin'))
               ]


    def _addExCols(self, record):
        record.append(QtSql.QSqlField('documentType_id', QVariant.Int))
        record.append(QtSql.QSqlField('serial', QVariant.String))
        record.append(QtSql.QSqlField('number', QVariant.String))
        record.append(QtSql.QSqlField('date',   QVariant.Date))
        record.append(QtSql.QSqlField('origin', QVariant.String))


    def _loadDocInfo(self, record):
        db = QtGui.qApp.db
        documentId = forceRef(record.value('document_id'))
        if documentId:
            documentRecord = db.getRecord('ClientDocument', self.documentFields, documentId)
            if documentRecord:
                for n in self.documentFields.split(','):
                    record.setValue(n, documentRecord.value(n))


    def _saveDocInfo(self, record, clientId):
        documentId = None
        if forceRef(record.value('documentType_id')):
            db = QtGui.qApp.db
            table = db.table('ClientDocument')
            documentId = forceRef(record.value('document_id'))
            if documentId:
                documentRecord = db.getRecord(table, '*', documentId)
            else:
                documentRecord = table.newRecord()
            for n in self.documentFields.split(','):
                documentRecord.setValue(n, record.value(n))
            documentRecord.setValue('client_id', toVariant(clientId))
            documentId = db.insertOrUpdate(table, documentRecord)
            record.setValue('document_id', toVariant(documentId))
        return documentId


class CPolyclinicInDocTableCol(CInDocTableCol):
    def __init__(self, title, fieldName, width):
        CInDocTableCol.__init__(self, title, fieldName, width)
        self.__model = None

    def initModel(self):
        self.__model = CDbModel(None)
        self.__model.setNameField(CPolyclinicComboBox.nameField)
        self.__model.setAddNone(True)
        self.__model.setFilter(CPolyclinicComboBox.filter)
        self.__model.setTable('Organisation')


    def toString(self, val, record):
        if not self.__model:
            self.initModel()
        text = self.__model.getNameById(forceRef(val))
        return toVariant(text)


    def createEditor(self, parent):
        if not self.__model:
            self.initModel()
        #editor = CDbComboBox(parent)
        #editor.setModel(self.__model)
        editor = CDbSearchWidget(parent)
        editor.setModel(self.__model)
        return editor


    def setEditorData(self, editor, value, record):
        editor.setValue(forceRef(value))


    def getEditorData(self, editor):
        return toVariant(editor.value())


class COrgStructureInDocTableColEx(COrgStructureInDocTableCol):

    def setEditorData(self, editor, value, record):
        editor.setOrgId(forceRef(record.value('LPU_id')))
        editor.setValue(forceInt(value))


class CClientRelationInDocTableCol(CInDocTableCol):
    def __init__(self, title, fieldName, width, tableName, **params):
        CInDocTableCol.__init__(self, title, fieldName, width, **params)
        self.tableName  = tableName
        self.filter     = params.get('filter', '')
        self.preferredWidth = params.get('preferredWidth', None)
        self.clientId = None
        self.regAddressInfo = None
        self.logAddressInfo = None


    def toString(self, val, record):
        return toVariant(clientIdToText(val))


    def createEditor(self, parent):
        editor = CClientRelationComboBox(parent, self.clientId, self.regAddressInfo, self.logAddressInfo)
        return editor


    def setEditorData(self, editor, value, record):
        editor.setValue(forceInt(value))


    def getEditorData(self, editor):
        return toVariant(editor.value())


    def setClientId(self, clientId):
        self.clientId = clientId


    def setRegAddressInfo(self, regAddressInfo):
        self.regAddressInfo = regAddressInfo


    def setLogAddressInfo(self, logAddressInfo):
        self.logAddressInfo = logAddressInfo


class CAttachesModel(CInDocTableModel):
    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'ClientAttach', 'id', 'client_id', parent)
        self.addCol(CRBInDocTableCol(            u'Тип',               'attachType_id',    30, 'rbAttachType'))
        self.addCol(CPolyclinicInDocTableCol(    u'ЛПУ',               'LPU_id',           15))
        self.addCol(COrgStructureInDocTableColEx(u'Подразделение',     'orgStructure_id',  15))
        self.addCol(CDateInDocTableCol(          u'Дата прикрепления', 'begDate',          15, canBeEmpty=True))
        self.addCol(CDateInDocTableCol(          u'Дата открепления',  'endDate',          15, canBeEmpty=True))
        self.addCol(CEnumInDocTableCol(          u'По заявлению',      'isStatement',      15, [u'нет', u'да']))
        self.clientEditDialog = parent


    def getEmptyRecord(self):
        orgId = QtGui.qApp.currentOrgId()
        address = self.clientEditDialog.getAddress(1)
        houseId = findHouseId(address)
        if houseId:
            clientSex = self.clientEditDialog.cmbSex.currentIndex()
            clientAge = calcAgeTuple(self.clientEditDialog.clientBirthDate, QDate.currentDate())
            orgStructureIdList = findOrgStructuresByHouseAndFlat(houseId, address['flat'], orgId, QtGui.qApp.currentOrgStructureId(), clientSex, clientAge)
            orgStructureId = orgStructureIdList[0] if orgStructureIdList else None
        else:
            orgStructureId = None
        result = CInDocTableModel.getEmptyRecord(self)
        result.setValue('LPU_id',          toVariant(orgId))
        result.setValue('orgStructure_id', toVariant(orgStructureId))
        result.setValue('begDate',         toVariant(QDate.currentDate()))
        return result

    def setData(self, index, value, role=Qt.EditRole):
        result = CInDocTableModel.setData(self, index, value, role)
        if result and index.column() == 0 and role == Qt.EditRole:
            db = QtGui.qApp.db
            outcome = forceInt(db.translate('rbAttachType', 'id', value, 'outcome'))
            if outcome and 0 <= index.row() < len(self._items):
                record = self._items[index.row()]
                record.setValue('endDate', record.value('begDate'))


class CDirectRelationsModel(CInDocTableModel):
    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'ClientRelation', 'id', 'client_id', parent)
        self.addCol(CRBInDocTableCol(u'Прямая связь', 'relativeType_id', 30, 'vrbDirectClientRelation'))
        self.addCol(CClientRelationInDocTableCol(u'Связан с пациентом', 'relative_id', 30, 'ClientRelation'))
        self.clientEditDialog = parent
        self.clientId = None


    def setClientId(self, clientId):
        self.clientId = clientId
        self.cols()[1].setClientId(clientId)


    def setRegAddressInfo(self, regAddressInfo):
        self.cols()[1].setRegAddressInfo(regAddressInfo)


    def setLogAddressInfo(self, logAddressInfo):
        self.cols()[1].setLogAddressInfo(logAddressInfo)


    def setDirectRelationFilter(self, sex):
        if sex:
            self.cols()[0].filter = 'id IN (SELECT rbRelationType.id FROM rbRelationType WHERE rbRelationType.leftSex = %d OR rbRelationType.leftSex = 0)'%(sex)
        else:
            self.cols()[0].filter = ''


class CBackwardRelationsModel(CInDocTableModel):
    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'ClientRelation', 'id', 'relative_id', parent)
        self.addCol(CRBInDocTableCol(u'Обратная связь', 'relativeType_id', 30, 'vrbBackwardClientRelation'))
        self.addCol(CClientRelationInDocTableCol(u'Связан с пациентом', 'client_id', 30, 'ClientRelation'))
        self.clientEditDialog = parent
        self.clientId = None


    def setClientId(self, clientId):
        self.clientId = clientId
        self.cols()[1].setClientId(clientId)


    def setRegAddressInfo(self, regAddressInfo):
        self.cols()[1].setRegAddressInfo(regAddressInfo)


    def setLogAddressInfo(self, logAddressInfo):
        self.cols()[1].setLogAddressInfo(logAddressInfo)


    def setBackwardRelationFilter(self, sex):
        if sex:
            self.cols()[0].filter = 'id IN (SELECT rbRelationType.id FROM rbRelationType WHERE rbRelationType.rightSex = %d OR rbRelationType.rightSex = 0)'%(sex)
        else:
            self.cols()[0].filter = ''


def valIsMatched(old, new):
    return old == new or not old or not new



class CRegExpedInDocTableCol(CInDocTableCol):
    def __init__(self, title, fieldName, width, **params):
        CInDocTableCol.__init__(self, title, fieldName, width, **params)
        self.regExp = None


    def createEditor(self, parent):
        editor = CLineEditWithRegExpValidator(parent)
        return editor



class CDocSerialInDocTableCol(CRegExpedInDocTableCol):
    def setEditorData(self, editor, value, record):
        documentTypeId = record.value('documentType_id')
        if documentTypeId:
            documentTypeDescr = getDocumentTypeDescr(documentTypeId)
            if documentTypeDescr.hasRightPart:
                regExp = ( documentTypeDescr.leftPartRegExp
                           + documentTypeDescr.partSeparator
                           + documentTypeDescr.rightPartRegExp )
            else:
                regExp = documentTypeDescr.leftPartRegExp
        else:
            regExp = ''

        editor.setRegExp(regExp)
        editor.setText(forceStringEx(value))


class CDocNumberInDocTableCol(CRegExpedInDocTableCol):
    def setEditorData(self, editor, value, record):
        documentTypeId = record.value('documentType_id')
        if documentTypeId:
            documentTypeDescr = getDocumentTypeDescr(documentTypeId)
            regExp = documentTypeDescr.numberRegExp
        else:
            regExp = ''
        editor.setRegExp(regExp)
        editor.setText(forceStringEx(value))



class CIdentificationDocsModel(CInDocTableModel):
    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'ClientDocument', 'id', 'client_id', parent)
        self.setFilter('documentType_id IN (SELECT rbDocumentType.id FROM rbDocumentType LEFT JOIN rbDocumentTypeGroup ON rbDocumentTypeGroup.id = rbDocumentType.group_id WHERE rbDocumentTypeGroup.code=\'1\')')
        self.addCol(CRBInDocTableCol(u'Тип', 'documentType_id', 30, 'rbDocumentType', filter='group_id IN (SELECT id FROM rbDocumentTypeGroup WHERE code=\'1\')'))
        self.addCol(CDocSerialInDocTableCol(u'Серия', 'serial', 8))
        self.addCol(CDocNumberInDocTableCol(u'Номер', 'number', 16))
        self.addCol(CDateInDocTableCol(u'Дата',  'date', 15, canBeEmpty=True))
        self.addCol(CInDocTableCol(u'Выдан', 'origin', 30))


    def setData(self, index, value, role=Qt.EditRole):
        result = CInDocTableModel.setData(self, index, value, role)


    def getCurrentDocRow(self, docTypeId, serial, number):
        items = self.items()
        if items:
            row = len(items)-1
            item = items[row]
            if ( item.isNull('id')
                 or ( valIsMatched(forceRef(item.value('documentType_id')), docTypeId)
                      and valIsMatched(forceStringEx(item.value('serial')), serial)
                      and valIsMatched(forceStringEx(item.value('number')), number)
                    )
               ):
                return row
        row = len(items)
        self.insertRecord(row, self.getEmptyRecord())
        return row



class CPoliciesModel(CInDocTableModel):
    __pyqtSignals__ = ('policyChanged()',
                      )
    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'ClientPolicy', 'id', 'client_id', parent)
        self.policyTypeColumn = CRBInDocTableCol(u'Тип', 'policyType_id', 30, 'rbPolicyType')
        self.policyKindColumn = CRBInDocTableCol(u'Вид', 'policyKind_id', 30, 'rbPolicyKind')
        self.addCol(self.policyTypeColumn)
        self.addCol(self.policyKindColumn)
        self.addCol(CInDocTableCol(u'Серия', 'serial', 30))
        self.addCol(CInDocTableCol(u'Номер', 'number', 30))
        self.addCol(CDateInDocTableCol(u'Дата начала',    'begDate', 15, canBeEmpty=True))
        self.addCol(CDateInDocTableCol(u'Дата окончания', 'endDate', 15, canBeEmpty=True))
##!        CInsurerComboBox
##        self.addCol(CRBInDocTableCol(u'СМО', 'insurer_id', 50, 'Organisation'))
        self.addCol(CInsurerInDocTableCol(u'СМО', 'insurer_id', 50))
        self.addCol(CInDocTableCol(u'Наименование', 'name', 50))
        self.addCol(CInDocTableCol(u'Примечание',   'note', 50))


    def setData(self, index, value, role=Qt.EditRole):
        result = CInDocTableModel.setData(self, index, value, role)
        if result and role == Qt.EditRole:
            self.emit(SIGNAL('policyChanged()'))


    def getCurrentPolicyRowInt(self, isCompulsory, serial='', number='', insurerId=None):
        items = self.items()
        for row in reversed(xrange(len(items))):
            item = items[row]
            policyTypeId = forceRef(item.value('policyType_id'))
            if (policyTypeId is None and isCompulsory) or self.policyTypeIsCompulsory(policyTypeId, isCompulsory):
                if insurerId:
                    if ( valIsMatched(forceStringEx(item.value('serial')), serial)
                        and valIsMatched(forceStringEx(item.value('number')), number)
                        and valIsMatched(forceRef(item.value('insurer_id')), insurerId)
                        ):
                        return row
                    elif isCompulsory:
                        return -1
                else:
                    if ( item.isNull('id')
                         or ( valIsMatched(forceStringEx(item.value('serial')), serial)
                        and valIsMatched(forceStringEx(item.value('number')), number))
                       ):
                        return row
                    elif isCompulsory:
                        return -1
        return -1


    def getCurrentPolicyRow(self, isCompulsory, serial, number, insurerId=None):
        row = self.getCurrentPolicyRowInt(isCompulsory, serial, number, insurerId)
        if row < 0:
            row = len(self.items())
#            self.insertRecord(row, self.getEmptyRecord())
            item = self.getEmptyRecord()
            self.insertRecord(row, item)
            item.setValue('policyType_id', toVariant(self.getFirstPolicyTypeId(isCompulsory)))
        return row


    def getCurrentCompulsoryPolicyRow(self, serial, number, insurerId=None):
        return self.getCurrentPolicyRow(True, serial, number, insurerId)


    def getCurrentVoluntaryPolicyRow(self, serial, number, insurerId=None):
        return self.getCurrentPolicyRow(False, serial, number, insurerId)


    def policyTypeIsCompulsory(self, policyTypeId, isCompulsory):
        if policyTypeId:
            cache = CRBModelDataCache.getData(self.policyTypeColumn.tableName, True)
            return self.policyTypeNameIsCompulsory(cache.getNameById(policyTypeId)) == isCompulsory
        return False


    def getFirstPolicyTypeId(self, isCompulsory):
        cache = CRBModelDataCache.getData(self.policyTypeColumn.tableName, True)
        for i in xrange(cache.getCount()):
            id = cache.getId(i)
            if id and self.policyTypeNameIsCompulsory(cache.getName(i)) == isCompulsory:
                return id
        return None


    def policyTypeNameIsCompulsory(self, name):
        return unicode(name)[:3].upper() == u'ОМС'


    def cleanUpEmptyItems(self):
        items = self.items()
        for i in reversed(xrange(len(items))):
            item = items[i]
            id = forceRef(item.value('id'))
            # serial = forceStringEx(item.value('serial'))
            number = forceStringEx(item.value('number'))
            insurerId = forceRef(item.value('insurer_id'))
            name = forceStringEx(item.value('name'))
            if not (id or number or insurerId or name):
                self.removeRows(i, 1)


    def loadItems(self, masterId):
        db = QtGui.qApp.db
        table = self._table
        tableRBPolicyType = db.table('rbPolicyType')
        cols = [table['policyType_id'],
                table['policyKind_id'],
                table['serial'],
                table['number'],
                table['begDate'],
                table['endDate'],
                table['insurer_id'],
                table['name'],
                table['note'],
                table['id'],
                table['client_id'],
                ]
        for col in self._hiddenCols:
            cols.append(col)
        filter = [table[self._masterIdFieldName].eq(masterId)]
        if self._filter:
            filter.append(self._filter)
        if table.hasField('deleted'):
            filter.append(table['deleted'].eq(0))
        qeryTable = table.leftJoin(tableRBPolicyType, tableRBPolicyType['id'].eq(table['policyType_id']))
        order = [u'ClientPolicy.begDate', u'rbPolicyType.isCompulsory']
        self._items = db.getRecordList(qeryTable, cols, filter, order)
        if self._extColsPresent:
            extSqlFields = []
            for col in self._cols:
                if col.external():
                    fieldName = col.fieldName()
                    if fieldName not in cols:
                        extSqlFields.append(QtSql.QSqlField(fieldName, col.valueType()))
            if extSqlFields:
                for item in self._items:
                    for field in extSqlFields:
                        item.append(field)
        self.reset()


class CSocStatusDocsModel(CInDocTableModel):
    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'ClientDocument', 'id', 'client_id', parent)
        self.setFilter('documentType_id IN (SELECT rbDocumentType.id FROM rbDocumentType LEFT JOIN rbDocumentTypeGroup ON rbDocumentTypeGroup.id = rbDocumentType.group_id WHERE rbDocumentTypeGroup.code IN (\'2\', \'5\', \'7\'))')
        self.addCol(CRBInDocTableCol(u'Тип', 'documentType_id', 30, 'rbDocumentType', filter='group_id IN (SELECT id FROM rbDocumentTypeGroup WHERE code IN (\'2\', \'5\', \'7\'))'))
        self.addCol(CInDocTableCol(u'Серия', 'serial', 8))
        self.addCol(CInDocTableCol(u'Номер', 'number', 16))
        self.addCol(CDateInDocTableCol(u'Дата',  'date', 15, canBeEmpty=True))
        self.addCol(CInDocTableCol(u'Выдан', 'origin', 30))


class CContactInDocTableCol(CInDocTableCol):
    def __init__(self, title, fieldName, width, **params):
        CInDocTableCol.__init__(self, title, fieldName, width, **params)
        db = QtGui.qApp.db
        table = db.table('rbContactType')
        self._phoneNumberIdList = db.getDistinctIdList('rbContactType', 'id', [table['code'].inlist([1, 2, 3])])
        self._emailIdList = db.getDistinctIdList('rbContactType', 'id', [table['code'].inlist([4])])

    def setEditorData(self, editor, value, record):
        editor.setText(forceStringEx(value))
        contactTypeId = forceRef(record.value('contactType_id'))
        if contactTypeId in self._phoneNumberIdList:
            editor.setInputMask('+9(999)999-99-99;_')
        elif contactTypeId in self._emailIdList:
            editor.setValidator(QtGui.QRegExpValidator(QRegExp(u"^([a-z0-9\\+_\\-]+)(\\.[a-z0-9\\+_\\-]+)*@([a-z0-9\\-]+\\.)+[a-z]+$"), None))


class CClientContactsModel(CInDocTableModel):
    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'ClientContact', 'id', 'client_id', parent)
        self.addCol(CRBInDocTableCol(u'Тип', 'contactType_id', 30, 'rbContactType', addNone=False))
        self.addCol(CContactInDocTableCol(u'Номер', 'contact', 30))
        self.addCol(CInDocTableCol(u'Примечание', 'notes', 30))


    def setDialogContact(self, typeContactId, contact):
        if typeContactId:
            row = len(self.items())
            item = CInDocTableModel.getEmptyRecord(self)
            self.insertRecord(row, item)
            item.setValue('contactType_id', toVariant(typeContactId))
            item.setValue('contact', toVariant(contact))


class CAllergyModel(CInDocTableModel):
    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'ClientAllergy', 'id', 'client_id', parent)
        self.addCol(CInDocTableCol(u'Наименование вещества', 'nameSubstance', 50))
        self.addCol(CEnumInDocTableCol(u'Степень', 'power', 15, [u'0 - не известно', u'1 - малая', u'2 - средняя', u'3 - высокая', u'4 - строгая']))
        self.addCol(CDateInDocTableCol(u'Дата установления', 'createDate', 15, canBeEmpty=True))
        self.addCol(CInDocTableCol(u'Примечание', 'notes', 30))


class CDepositModel(CInDocTableModel):
    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'ClientDeposit', 'id', 'client_id', parent)
        self.addCol(CDateInDocTableCol(u'Дата', 'contractDate', 15, canBeEmpty=True))
        self.addCol(CDepositModel.CContractInDocTableCol(self))
        self.addCol(CFloatInDocTableCol(u'Сумма депозита', 'contractSum', 8, precision=4))
        self.addExtCol(CFloatInDocTableCol( u'Сумма фактическая',   'factSum', 10), QVariant.Int).setReadOnly(True)
        self.columnFactSum = self.getColIndex('factSum')
        self.factSum = []


    def data(self, index, role=Qt.DisplayRole):
        column = index.column()
        if role == Qt.DisplayRole:
            if column == self.columnFactSum:
                row = index.row()
                if 0 <= row < len(self.items()) and len(self.factSum) <= len(self.items()):
                    return QVariant(self.factSum[row])
        return CInDocTableModel.data(self, index, role)


    class CContractInDocTableCol(CInDocTableCol):

        def __init__(self, model):
            CInDocTableCol.__init__(self, u'Договор', 'contract_id', 20)
            self.model = model


        def toString(self, val, record):
            contractId = forceRef(val)
            if contractId:
                names = ['number', 'date', 'resolution']
                record = QtGui.qApp.db.getRecord('Contract', names, contractId)
                str = ' '.join([forceString(record.value(name)) for name in names])
            else:
                str = u'не задано'
            return QVariant(str)


        def createEditor(self, parent):
            filter = {}
            editor = CContractTreeFindComboBox(parent, filter)
            return editor


        def getEditorData(self, editor):
            return QVariant(editor.value())


        def setEditorData(self, editor, value, record):
            date = forceDate(record.value('contractDate'))
            editor.setDate(date)


class CIntoleranceMedicamentModel(CInDocTableModel):
    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'ClientIntoleranceMedicament', 'id', 'client_id', parent)
        self.addCol(CMedicamentInDocTableCol(u'Название медикамента', 'nameMedicament', 50))
        self.addCol(CEnumInDocTableCol(u'Степень', 'power', 15, [u'0 - не известно', u'1 - малая', u'2 - средняя', u'3 - высокая', u'4 - строгая']))
        self.addCol(CDateInDocTableCol(u'Дата установления', 'createDate', 15, canBeEmpty=True))
        self.addCol(CInDocTableCol(u'Примечание', 'notes', 30))


class CClientIdentificationModel(CInDocTableModel):
    mapAccountingSystemIdToNeedUniqueValue = {}


    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'ClientIdentification', 'id', 'client_id', parent)
        self.__parent = parent
        self.addCol(CRBInDocTableCol(u'Внешняя учётная система', 'accountingSystem_id', 30, 'rbAccountingSystem',  addNone =False))
        self.addCol(CInDocTableCol(u'Идентификатор', 'identifier', 30))
        self.addCol(CDateInDocTableCol(u'Дата подтверждения',  'checkDate', 15, canBeEmpty=True))
        self.isEditable = {}
        flagsList = QtGui.qApp.db.getRecordList('rbAccountingSystem', 'id, isEditable')
        if flagsList:
            for x in flagsList:
                self.isEditable[forceRef(x.value(0))] = forceBool(x.value(1))


    def flags(self, index = QModelIndex()):
        column = index.column()
        if column == (0, 2): # accountingSystem_id, checkDate
            return Qt.ItemIsSelectable | Qt.ItemIsEnabled
        if column == 1 : #  identifier
            row = index.row()
            items = self.items()
            record = items[row] if 0<=row<len(items) else None
            accountingSystemId = forceRef(record.value('accountingSystem_id')) if record else None
            nullId = (forceString(record.value('identifier')) == '') if record else True
            if accountingSystemId and (self.isEditable.get(accountingSystemId,  False) or nullId):
                return Qt.ItemIsSelectable | Qt.ItemIsEnabled | Qt.ItemIsEditable

            return Qt.ItemIsSelectable | Qt.ItemIsEnabled
        return CInDocTableModel.flags(self, index)


    def getEmptyRecord(self):
        result = CInDocTableModel.getEmptyRecord(self)
        result.setValue('checkDate',         toVariant(QDate.currentDate()))
        return result


    def emitDataChanged(self, row, column):
        index = self.index(row, column)
        self.emit(SIGNAL('dataChanged(QModelIndex, QModelIndex)'), index, index)


    def setData(self, index, value, role=Qt.EditRole):
        items = self.items()
        col = index.column()
        if col == 0: # если меняем систему, проверим уникальность
            if not forceRef(value):
                return False

            for record in items:
                if forceRef(record.value('accountingSystem_id')) == forceRef(value):
#                    QtGui.QMessageBox.critical( self.__parent, u'Изменение типа учётной системы',
#                        u'Идентификатор в выбранной учётной системе уже задан.',
#                        QtGui.QMessageBox.Close)
                    #self.emit(SIGNAL('dataChanged(QModelIndex, QModelIndex)'), index, index)
                    return False

        if col == 1: # меняем идентификатор. Проверяем необходимость проверки на уникальность и если нужно проверяем.
            newIdentifier = forceString(value)
            if newIdentifier == '':
                return False
            item = items[index.row()]
            currentItemId = forceRef(item.value('id'))
            accountingSystemId = forceRef(item.value('accountingSystem_id'))
            if self.needUniqueValue(accountingSystemId):
                if not uniqueIdentifierCheckingIsPassed(currentItemId, accountingSystemId, newIdentifier):
                    return False
        result = CInDocTableModel.setData(self, index, value, role)
        if result and role == Qt.EditRole:
            row = index.row()
            record = items[row] if 0<=row<len(items) else None
            if record:
                record.setValue('checkDate',         toVariant(QDate.currentDate()))
                self.emitDataChanged(row, 2) # колонка checkDate
        return result

    @classmethod
    def needUniqueValue(cls, accountingSystemId):
        isUnique = cls.mapAccountingSystemIdToNeedUniqueValue.get(accountingSystemId, None)
        if isUnique is None:
            isUnique = forceBool(QtGui.qApp.db.translate('rbAccountingSystem',
                                                         'id',
                                                          accountingSystemId,
                                                         'isUnique'))
            cls.mapAccountingSystemIdToNeedUniqueValue[accountingSystemId] = isUnique
        return isUnique


# ####################################################################


class CRBQuotaTypeInDocTableCol(CRBInDocTableCol):
    def createEditor(self, parent):
        editor = CQuotaTypeComboBox(parent)
        editor.setTable(self.tableName, addNone=self.addNone, filter=self.filter)
        editor.setShowFields(self.showFields)
        editor.setPreferredWidth(self.preferredWidth)
        return editor


class CClientQuotingModel(CInDocTableModel):
    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'Client_Quoting', 'id', 'master_id', parent)
        self.addCol(CRBQuotaTypeInDocTableCol(u'Квота', 'quotaType_id', 255, 'QuotaType', showFields=0))
        self.addCol(CIntInDocTableCol(u'Этап', 'stage', 2))
        self.addCol(CInDocTableCol(u'МКБ', 'MKB', 8))
        self.addCol(CEnumInDocTableCol(u'Статус', 'status', 2, [u'Отменено',
                                                                u'Ожидание',
                                                                u'Активный талон',
                                                                u'Талон для заполнения',
                                                                u'Заблокированный талон',
                                                                u'Отказано',
                                                                u'Необходимо согласовать дату обслуживания',
                                                                u'Дата обслуживания на согласовании',
                                                                u'Дата обслуживания согласована',
                                                                u'Пролечен',
                                                                u'Обслуживание отложено',
                                                                u'Отказ пациента',
                                                                u'Импортировано из ВТМП']))
        self.addHiddenCol('identifier')
        self.addHiddenCol('quotaTicket')
        self.addHiddenCol('directionDate')
        self.addHiddenCol('freeInput')
        self.addHiddenCol('org_id')
        self.addHiddenCol('amount')
        self.addHiddenCol('request')
        self.addHiddenCol('statment')
        self.addHiddenCol('dateRegistration')
        self.addHiddenCol('dateEnd')
        self.addHiddenCol('orgStructure_id')
        self.addHiddenCol('regionCode')
        self.addHiddenCol('districtCode')
        self._info = {}
        self.basicData = []
        self.basicDataCount = 0
        self.isActiveQuota = None
        self.currentRow = None
        self.isChangeingRow = None
        self.recordsForDeleting = []
        self.clientAttachForDeleting = []
        self.regCityCode = None
        self.newRegCityCode = None
        self.regDistrictCode = None
        self.newRegDistrictCode = None

    def addClientAttachForDeleting(self, attachRecord):
        self.clientAttachForDeleting.append(attachRecord)

    def setRegCityCode(self, regCityCode):
        self.regCityCode = regCityCode

    def setDistrictCode(self, regDistrictCode):
        self.regDistrictCode = regDistrictCode

    def setNewRegDistrictCode(self, regDistrictCode):
        self.newRegDistrictCode = regDistrictCode

    def setNewRegCityCode(self, regCityCode):
        self.newRegCityCode = regCityCode

    def flags(self, index):
        column = index.column()
        row    = index.row()
        if column == 0 and row == len(self.items()):
            return CInDocTableModel.flags(self, index)
        return Qt.ItemIsSelectable | Qt.ItemIsEnabled

    def getEmptyRecord(self):
        result = CInDocTableModel.getEmptyRecord(self)
        result.setValue('status', toVariant(1))
        currentDate = QDate.currentDate()
        result.setValue('dateRegistration', QVariant(currentDate))
        result.setValue('amount', QVariant(1))
        return result

    def setData(self, index, value, role=Qt.EditRole):
        resume = CInDocTableModel.setData(self, index, value, role)
        if resume:
            self.basicData.append([None, None, '', ''])
        return resume

    def info(self):
        return self._info

    def loadItems(self, masterId):
        CInDocTableModel.loadItems(self, masterId)
        self.saveBasicData()

    def saveBasicData(self):
        items = self.items()
        self.basicDataCount = len(items)
        for item in items:
            data = []
            data.append(forceInt(item.value('amount')))
            data.append(forceInt(item.value('status')))
            data.append(forceString(item.value('regionCode')))
            data.append(forceString(item.value('districtCode')))
            self.basicData.append(data)

    def saveItems(self, masterId):
#        for clientAttachRecord in self.clientAttachForDeleting:
#            clientAttachRecord.setValue('deleted', QVariant(1))
#            QtGui.qApp.db.updateRecord('ClientAttach', clientAttachRecord)
        items = self.items()
        basicData = self.basicData
        endedQuotes = 0
        minDate = None
        maxDate = None
        for idx, record in enumerate(items):
            currAmount = forceInt(record.value('amount'))
            currStatus = forceInt(record.value('status'))
            currRegionCode = forceString(record.value('regionCode'))
            currDistrictCode = forceString(record.value('districtCode'))
            baseAmount = basicData[idx][0]
            baseStatus = basicData[idx][1]
            baseRegionCode = basicData[idx][2]
            baseDistrictCode = basicData[idx][3]
            if currStatus in (5, 9, 11):
                endedQuotes += 1
                date = forceDate(record.value('dateEnd'))
                if not maxDate:
                    maxDate = date
                else:
                    if date > maxDate:
                        maxDate = date
            if not minDate:
                minDate = forceDate(record.value('dateRegistration'))
            else:
                date = forceDate(record.value('dateRegistration'))
                if date < minDate:
                    minDate = date
            if baseRegionCode:
                regCityCode = baseRegionCode
            else:
                regCityCode = self.regCityCode
            if currRegionCode:
                newRegCityCode = currRegionCode
            else:
                newRegCityCode = self.newRegCityCode
            if baseDistrictCode:
                regDistrictCode = baseDistrictCode
            else:
                regDistrictCode = self.regDistrictCode
            if currDistrictCode:
                newRegDistrictCode = currDistrictCode
            else:
                newRegDistrictCode = self.newRegDistrictCode
            if currStatus != baseStatus:
                quotingRecords = self.getActualQuotingRecords(record)
                if quotingRecords:
                    for quotingRecord in quotingRecords:
                        if baseAmount:
                            self.changeData(quotingRecord, baseStatus, baseAmount, baseAmount.__sub__)
                        self.changeData(quotingRecord, currStatus, currAmount, currAmount.__add__)
                        if regCityCode:
                            if baseAmount:
                                self.changeRegionData(baseStatus, baseAmount, baseAmount.__sub__,
                                                      regCityCode, quotingRecord)
                                self.changeDistrictData(baseStatus, baseAmount, baseAmount.__sub__,
                                                           (regCityCode, regDistrictCode), quotingRecord)
                        if newRegCityCode:
                            if regCityCode != newRegCityCode:
                                self.changeRegionData(currStatus, currAmount, currAmount.__add__,
                                                          newRegCityCode, quotingRecord)
                                if regDistrictCode != newRegDistrictCode:
                                    self.changeDistrictData(currStatus, currAmount, currAmount.__add__,
                                                          (newRegCityCode, newRegDistrictCode), quotingRecord)
                                else:
                                    self.changeDistrictData(currStatus, currAmount, currAmount.__add__,
                                                           (newRegCityCode, regDistrictCode), quotingRecord)
                            else:
                                self.changeRegionData(currStatus, currAmount, currAmount.__add__,
                                                          regCityCode, quotingRecord)
                                self.changeDistrictData(currStatus, currAmount, currAmount.__add__,
                                                           (regCityCode, regDistrictCode), quotingRecord)
            else:
                if currAmount != baseAmount:
                    quotingRecords = self.getActualQuotingRecords(record)
                    if quotingRecords:
                        for quotingRecord in quotingRecords:
                            dAmount = currAmount-baseAmount
                            self.changeData(quotingRecord, currStatus, dAmount, dAmount.__add__)
                            if regCityCode != newRegCityCode:
                                if regCityCode:
                                    self.changeRegionData(baseStatus, baseAmount, baseAmount.__sub__,
                                                      regCityCode, quotingRecord)
                                    self.changeDistrictData(baseStatus, baseAmount, baseAmount.__sub__,
                                                           (regCityCode, regDistrictCode), quotingRecord)
                                if newRegCityCode:
                                    self.changeRegionData(currStatus, currAmount, currAmount.__add__,
                                                          newRegCityCode, quotingRecord)
                                    self.changeDistrictData(currStatus, currAmount, currAmount.__add__,
                                                           (newRegCityCode, newRegDistrictCode), quotingRecord)
                            else:
                                if newRegCityCode:
                                    self.changeRegionData(currStatus, dAmount, dAmount.__add__,
                                                          newRegCityCode, quotingRecord)
                                    self.changeDistrictData(currStatus, dAmount, dAmount.__add__,
                                                           (newRegCityCode, newRegDistrictCode), quotingRecord)

        if len(items) > 0:
            self.setQuotingAttach(masterId, endedQuotes==len(items), minDate, maxDate, self.basicDataCount<len(items))
        CInDocTableModel.saveItems(self, masterId)
        if self.recordsForDeleting:
            self.changeQuotingDataForDeletedRecordList(self.recordsForDeleting)


    def setQuotingAttach(self, clientId, allAreEnded, minDate, maxDate, newQuotaAdded):
        db = QtGui.qApp.db
        table = db.table('ClientAttach')
        attachTypeId = db.translate('rbAttachType', 'code', '9', 'id')
        parent = QObject.parent(self)
        assert isinstance(parent, CClientEditDialog)
        attachItems = parent.tblAttaches.model().items()
        quotaAttachListEnded = []
        quotaAttachListNotEnded = []
        recordList = []
        if attachItems:
            quotaAttachList = [attach for attach in attachItems if attach.value('attachType_id') == attachTypeId]
            if quotaAttachList:
                for attach in quotaAttachList:
                    if forceDate(attach.value('endDate')).isValid():
                        quotaAttachListEnded.append(attach)
                    else:
                        quotaAttachListNotEnded.append(attach)
        if (not attachItems) or (not quotaAttachListNotEnded and newQuotaAdded):
            record = table.newRecord()
            record.setValue('client_id', QVariant(clientId))
            record.setValue('attachType_id', attachTypeId)
            record.setValue('LPU_id', QVariant(QtGui.qApp.currentOrgId()))
            record.setValue('begDate', QVariant(minDate))
            if allAreEnded:
                record.setValue('endDate', QVariant(maxDate))
            recordList.append(record)
        else:
            if quotaAttachListNotEnded:
                for attach in quotaAttachListNotEnded:
                    attach.setValue('begDate', QVariant(minDate))
                    if allAreEnded:
                        attach.setValue('endDate', QVariant(maxDate))
                    recordList.append(attach)
        clientQuotaAttachIdList = []
        for record in recordList:
            clientQuotaAttachIdList.append(db.insertOrUpdate('ClientAttach', record))
        return clientQuotaAttachIdList


    def changeRegionData(self, status, amount, action, code, quotingRecord):
        regionRecords = findKLADRRegionRecordsInQuoting(code, quotingRecord)
        for regionRecord in regionRecords:
            self.changeData(regionRecord, status, amount, action, 'Quoting_Region')


    def changeDistrictData(self, status, amount, action, codes, quotingRecord):
        regionRecords = findDistrictRecordsInQuoting(codes, quotingRecord)
        for regionRecord in regionRecords:
            self.changeData(regionRecord, status, amount, action, 'Quoting_District')


    def changeData(self, quotingRecord, status, amount, action, table='Quoting'):
        columnName = self.translateStatusToColumn(status)
        if columnName:
            columnValue = forceInt(quotingRecord.value(columnName))
            resumeValue = abs(action(columnValue))
            quotingRecord.setValue(columnName, QVariant(resumeValue))
            QtGui.qApp.db.updateRecord(table, quotingRecord)

    def translateStatusToColumn(self, status):
        if status == 9:
            return 'used'
        if status == 8:
            return 'confirmed'
        if status in (1, 2, 3, 4, 6, 7, 10, 12):
            return 'inQueue'
#        if status in (0, 5, 11):
        return None

    def getActualQuotingRecords(self, record):
        db = QtGui.qApp.db
        dateRegistration = forceDate(record.value('dateRegistration'))
        dateEnd          = forceDate(record.value('dateEnd'))
        quotaType_id     = forceInt(record.value('quotaType_id'))
        tableQuoting = db.table('Quoting')
        if not dateEnd:
            condDate = [tableQuoting['endDate'].ge(dateRegistration)]
        else:
            condDate = [db.joinOr([db.joinAnd([tableQuoting['beginDate'].le(dateRegistration),
                                   tableQuoting['endDate'].ge(dateRegistration)]),
                        db.joinAnd([tableQuoting['beginDate'].le(dateEnd),
                                   tableQuoting['endDate'].ge(dateEnd)])])]
        condDate.append(tableQuoting['deleted'].eq(0))
        record = db.getRecordEx(tableQuoting,'*',[tableQuoting['quotaType_id'].eq(quotaType_id)]+condDate)
        list = []
        if record:
            list.append(record)
#        else:
        code = forceString(db.translate('QuotaType', 'id', quotaType_id, 'code'))
        while code.find('.') > -1:
            record, code = self.checkParent(tableQuoting, condDate, quotaType_id)
            quotaType_id = forceRef(db.translate('QuotaType', 'code', code, 'id'))
            if record:
                list.append(record)
        return list

    def setNullInfo(self):
        self._info['limit'] = 0
        self._info['used'] = 0
        self._info['confirmed'] = 0
        self._info['inQueue'] = 0

    def loadItemsInfo(self, row):
        if row is None:
            self.setNullInfo()
            self._info['quotaTypeName'] = ''
            return
        db = QtGui.qApp.db
        tableQuoting = db.table('Quoting')
        record = self.items()[row]
        quotaType_id = forceInt(record.value('quotaType_id'))
        quotaTypeName = forceString(
                        QtGui.qApp.db.translate('QuotaType', 'id', quotaType_id, 'name'))
        condDate = []
        dateRegistration = forceDateTime(record.value('dateRegistration'))
        dateEnd          = forceDateTime(record.value('dateEnd'))
        if dateRegistration:
            condDate.append(db.joinAnd([tableQuoting['beginDate'].le(dateRegistration),
                                       tableQuoting['endDate'].ge(dateRegistration)]))
        if dateEnd:
            condDate.append(db.joinAnd([tableQuoting['beginDate'].le(dateEnd),
                                       tableQuoting['endDate'].ge(dateEnd)]))
        cond = [db.joinOr(condDate)] if condDate else []
        recordQuoting, brothersLimitIfNeedElseZero = self.getQuotingRecord(tableQuoting, cond, quotaType_id)
        self._info['quotaTypeName']    = quotaTypeName
        if recordQuoting:
            self._info['limit'] = forceInt(recordQuoting.value('limitation')) - brothersLimitIfNeedElseZero
            self._info['used'] = forceInt(recordQuoting.value('used'))
            self._info['confirmed'] = forceInt(recordQuoting.value('confirmed'))
            self._info['inQueue'] = forceInt(recordQuoting.value('inQueue'))
        else:
            self.setNullInfo()

    def getQuotingRecord(self, tableQuoting, cond, quotaType_id):
        db = QtGui.qApp.db
        condTmp = list(cond)
        condTmp.append(tableQuoting['quotaType_id'].eq(quotaType_id))
        record = db.getRecordEx('Quoting', '*', condTmp)
        if record:
            return record, 0
        else:
            val = 0
            record, parentCode = self.checkParent(tableQuoting, cond, quotaType_id)
            if record:
                val = self.checkBrothersLimit(tableQuoting, cond, parentCode)
            return record, val


    def checkParent(self, tableQuoting, cond, quotaType_id):
        db = QtGui.qApp.db
        parentCode = forceString(db.translate('QuotaType', 'id', quotaType_id, 'group_code'))
        if parentCode:
            parentId = forceInt(db.translate('QuotaType', 'code', parentCode, 'id'))
            record, val = self.getQuotingRecord(tableQuoting, cond, parentId)
            return record, parentCode
        else:
            return None, ''

    def checkBrothersLimit(self, tableQuoting, cond, parentCode):
        db = QtGui.qApp.db
        idList = db.getIdList('QuotaType', 'id', 'group_code=%s'%parentCode)
        condTmp = list(cond)
        condTmp.append(tableQuoting['quotaType_id'].inlist(idList))
        stmt = db.selectStmt(tableQuoting, 'SUM(`limitation`)', condTmp)
        query = db.query(stmt)
        if query.first():
            return forceInt(query.value(0))
        return 0

    def removeRows(self, row, count, parentIndex = QModelIndex()):
        tmpItems = list(self.items())
        resume = CInDocTableModel.removeRows(self, row, count, parentIndex)
        if resume:
            itemsForDeleting = tmpItems[row:row+count]
            for item in itemsForDeleting:
                self.recordsForDeleting.append(item)
            decreaseBD = row in range(len(self.basicData))
            del self.basicData[row:row+count]
            if decreaseBD:
                self.basicDataCount = len(self.basicData)
        return resume

    def changeQuotingDataForDeletedRecordList(self, recordList):
        for record in recordList:
            if record.value('id').isValid():
                quotingRecords = self.getActualQuotingRecords(record)
                if quotingRecords:
                    for quotingRecord in quotingRecords:
                        status = forceInt(record.value('status'))
                        amount = forceInt(record.value('amount'))
                        self.changeData(quotingRecord, status, amount, amount.__sub__)
                        regCityCode = self.regCityCode
                        if not regCityCode:
                            parent = QObject.parent(self)
                            assert isinstance(parent, CClientEditDialog)
                            regCityCode = parent.cmbRegCity.code()
                        self.changeRegionData(status, amount, amount.__sub__,
                                              regCityCode, quotingRecord)
                        regDistrictCode = self.regDistrictCode
                        if not regDistrictCode:
                            parent = QObject.parent(self)
                            assert isinstance(parent, CClientEditDialog)
                            regDistrictCode = getOKATO(regCityCode, parent.cmbRegStreet.code(), parent.edtRegHouse.text())
                        self.changeDistrictData(status, amount, amount.__sub__,
                                               (regCityCode, regDistrictCode), quotingRecord)


class CClientQuotingDiscussionModel(CTableModel):
    def __init__(self, parent):
        CTableModel.__init__(self, parent,  [
            CDateCol(u'Дата', ['dateMessage'], 16),
            CTimeCol(u'Время', ['dateMessage'], 16),
            CRefBookCol(u'Тип согласования', ['agreementType_id'], 'rbAgreementType', 15),
            CRefBookCol(u'Ответственный ЛПУ', ['responsiblePerson_id'], 'vrbPersonWithSpeciality', 16, 1),
            CTextCol(u'Контрагент', ['cosignatory'], 25),
            CTextCol(u'Должность', ['cosignatoryPost'], 20),
            CNameCol(u'ФИО', ['cosignatoryName'], 50),
            CTextCol(u'Примечание', ['remark'], 128)
            ], 'Client_QuotingDiscussion')
# #############################################################################

class CClientConsentsInDocModel(CInDocTableModel):
    class CClientRelationInDocCol(CInDocTableCol):
        def __init__(self):
            CInDocTableCol.__init__(self, u'Подписавший', 'representerClient_id', 15)
            self._cache = {}
            self._clientId = None


        def setClientId(self, clientId):
            self._clientId = clientId


        def toString(self, val, record):
            representerClientId = forceRef(val)
            if representerClientId:
                result = self._cache.get(representerClientId, None)
                if result is not None:
                    return QVariant(result)

                db = QtGui.qApp.db
                tableC  = db.table('Client')
                if representerClientId == self._clientId:
                    clientRecord = db.getRecord(tableC, 'lastName, firstName, patrName, birthDate', representerClientId)
                    if clientRecord:
                        name = formatShortName(clientRecord.value('lastName'),
                                               clientRecord.value('firstName'),
                                               clientRecord.value('patrName'))
                        result = u', '.join([name, forceString(clientRecord.value('birthDate'))])
                    else:
                        result = u'Неправильные данные!'
                else:
                    tableCR = db.table('ClientRelation')
                    tableRT = db.table('rbRelationType')
                    queryTable = tableCR
                    queryTable = queryTable.leftJoin(tableRT, tableRT['id'].eq(tableCR['relativeType_id']))
                    queryTable = queryTable.leftJoin(tableC, tableC['id'].eq(tableCR['relative_id']))
                    cond = [tableCR['client_id'].eq(self._clientId),
                            tableCR['relative_id'].eq(representerClientId),
                            tableCR['deleted'].eq(0)]
                    fields = ['CONCAT_WS(\' | \', rbRelationType.`code`, CONCAT_WS(\'->\', rbRelationType.`leftName`, rbRelationType.`rightName`)) AS relationType',
                              tableC['lastName'].alias(), tableC['firstName'].name(),
                              tableC['patrName'].name(), tableC['birthDate'].name(),
                              tableC['id'].alias('relativeId')]
                    clientRelationRecord = db.getRecordEx(queryTable, fields, cond)
                    if clientRelationRecord:
                        name = formatShortName(clientRelationRecord.value('lastName'),
                                               clientRelationRecord.value('firstName'),
                                               clientRelationRecord.value('patrName'))
                        result = ', '.join([name,
                                            forceString(clientRelationRecord.value('birthDate')),
                                            forceString(clientRelationRecord.value('relationType'))])
                    else:
                        tableCR = db.table('ClientRelation')
                        tableRT = db.table('rbRelationType')
                        queryTable = tableCR
                        queryTable = queryTable.leftJoin(tableRT, tableRT['id'].eq(tableCR['relativeType_id']))
                        queryTable = queryTable.leftJoin(tableC, tableC['id'].eq(tableCR['client_id']))
                        cond = [tableCR['client_id'].eq(representerClientId),
                                tableCR['relative_id'].eq(self._clientId),
                                tableCR['deleted'].eq(0)]
                        fields = ['CONCAT_WS(\' | \', rbRelationType.`code`, CONCAT_WS(\'->\', rbRelationType.`rightName`, rbRelationType.`leftName`)) AS relationType',
                                  tableC['lastName'].alias(), tableC['firstName'].name(),
                                  tableC['patrName'].name(), tableC['birthDate'].name(),
                                  tableC['id'].alias('relativeId')]
                        clientRelationRecord = db.getRecordEx(queryTable, fields, cond)
                        if clientRelationRecord:
                            name = formatShortName(clientRelationRecord.value('lastName'),
                                                   clientRelationRecord.value('firstName'),
                                                   clientRelationRecord.value('patrName'))
                            result = ', '.join([name,
                                                forceString(clientRelationRecord.value('birthDate')),
                                                forceString(clientRelationRecord.value('relationType'))])
                        else:
                            result = u'Неправильные данные!'
                self._cache[representerClientId] = result
                return QVariant(result)
            return QVariant()


        def createEditor(self, parent):
            editor = CClientRelationComboBoxForConsents(parent)
            editor.setClientId(self._clientId)
            return editor


        def setEditorData(self, editor, value, record):
            editor.setValue(forceRef(value))


        def getEditorData(self, editor):
            return toVariant(editor.value())


    def __init__(self, parent):
        CInDocTableModel.__init__(self, 'ClientConsent', 'id', 'client_id', parent)
        self.addCol(CRBInDocTableCol(u'Тип согласия', 'clientConsentType_id', 15, 'rbClientConsentType'))
        self.addCol(CDateInDocTableCol(u'Дата регистрации согласия', 'date', 15))
        self.addCol(CEnumInDocTableCol( u'Значение', 'value', 10, [u'Нет', u'Да']))
        self.addCol(CDateInDocTableCol(u'Дата окончания согласия', 'endDate', 15))
        self.addCol(CInDocTableCol(u'Примечания', 'note', 15))
        self._colClientRelation = CClientConsentsInDocModel.CClientRelationInDocCol()
        self.addCol(self._colClientRelation)
        self._clientId = None

    def currentClientId(self):
        return self._clientId

    def setClientId(self, clientId):
        self._clientId = clientId
        self._colClientRelation.setClientId(clientId)


# #############################

class CAddressesTableModel(CTableModel):
    typeReg = 0
    typeLoc = 1
    class CClientAddressCol(CCol):
        def __init__(self, title, fields, defaultWidth):
            CCol.__init__(self, title, fields, defaultWidth, u'l')
            self._cache = {}

        def format(self, values):
            clientAddressId = forceRef(values[0])
            result = self._cache.get(clientAddressId, None)
            if result is None:
                addressId = forceRef(values[1].value('address_id'))
                if addressId:
                    address = getAddress(addressId)
                    result = ', '.join([part for part in [formatAddressInt(address),
                                                          getDistrictName(address.KLADRCode,
                                                                          address.KLADRStreetCode,
                                                                          address.number)
                                                         ] if part])
                else:
                    result = values[1].value('freeInput')
                self._cache[clientAddressId] = result
            return result


    def __init__(self, parent, _type):
        CTableModel.__init__(self, parent)
        self._type = _type
        self.addColumn(CDateCol(u'Дата', [u'createDatetime'], 8))
        self.addColumn(CRefBookCol(u'Создатель записи', [u'createPerson_id'], u'vrbPerson', 15))
        self.addColumn(CAddressesTableModel.CClientAddressCol(u'Адрес', [u'id'], 16))
        self.loadField(u'freeInput')
        self.loadField(u'address_id')
        self.setTable(u'ClientAddress')
        self._clientId = None
        self._mapColumn2Order = {u'createDatetime' :u'ClientAddress.createDatetime',
                                 u'createPerson_id':u'vrbPerson.name',
                                 u'id'             :u'IF(ClientAddress.address_id IS NOT NULL, formatAddress(ClientAddress.address_id), ClientAddress.freeInput)'
                                 }


    def getOrder(self, fieldName, column):
        if hasattr(self._cols[column], 'extraFields'):
            if len(self._cols[column].extraFields) > 0:
                fieldName = self._cols[column].extraFields[0]
        return self._mapColumn2Order[fieldName]


    def setClientId(self, clientId):
        if self._clientId != clientId:
            self.setIdList(QtGui.qApp.db.getIdList(self._table, where=[self._table['client_id'].eq(clientId), self._table['type'].eq(self._type), self._table['deleted'].eq(0)]))
            self._clientId = clientId


    def deleteRecord(self, table, itemId):
        QtGui.qApp.db.markRecordsDeleted(table, table[self.idFieldName].eq(itemId))


class CCheckDatePolicyMessageBox(QtGui.QMessageBox):
    def __init__(self, icon, title, message, buttons, parent):
        QtGui.QMessageBox.__init__(self, icon, title, message, buttons, parent)


    def on_correct(self):
        self.done(2)

