# -*- coding: utf-8 -*-
#############################################################################
##
## Copyright (C) 2006-2012 Chuk&Gek and Vista Software. All rights reserved.
## Copyright (C) 2012-2017 SAMSON Group. All rights reserved.
##
#############################################################################
##
## Это программа является свободным программным обеспечением.
## Вы можете использовать, распространять и/или модифицировать её согласно
## условиям GNU GPL версии 3 или любой более поздней версии.
##
#############################################################################
##
## Обслуживание пациентов: Реестр пациентов + спосок событий + ....
##
#############################################################################

import sip
from PyQt4         import QtGui
from PyQt4.QtCore  import Qt, QDate, QDateTime, QMetaObject, QObject, QRegExp, QString, QVariant, pyqtSignature, SIGNAL, QModelIndex

from library.crbcombobox                    import CRBModel, CRBModelDataCache
from library.database                       import addCondLike, undotLikeMask, CTableRecordCache
from library.DateEdit                       import CDateEdit
from library.DialogBase                     import CConstructHelperMixin
from library.ICDUtils                       import MKBwithoutSubclassification
from library.InDocTable                     import CRecordListModel, CBoolInDocTableCol, CDateTimeInDocTableCol, CEnumInDocTableCol, CInDocTableCol, CRBInDocTableCol
from library.Pacs.Explorer                  import CPacsExplorer
from library.PreferencesMixin               import CDialogPreferencesMixin
from library.PrintInfo                      import CDateInfo, CInfoContext, CTimeInfo
from library.PrintTemplates                 import addButtonActions, additionalCustomizePrintButton, applyTemplate, changeClientConsent, CPrintAction, directPrintTemplate, getFirstPrintTemplate, getPrintAction, getPrintTemplates
from library.RBCheckTable                   import CRBCheckTableModel
from library.RecordLock                     import CRecordLockMixin
from library.Utils                          import addDots, copyFields, formatDays, forceBool, forceDate, forceDateTime, forceInt, forceRef, forceString, forceStringEx, formatNum, formatRecordsCount, formatRecordsCount2, formatSex, formatSNILS, getPref, getPrefBool, quote, smartDict, toVariant, trim, agreeNumberAndWord

from Accounting.AccountingDialog            import CAccountingDialog
from DataCheck.RegistryControlDoubles       import CRegistryControlDoubles
from Events.Action                          import CActionTypeCache, CAction
from Events.ActionStatus                    import CActionStatus
from Events.ActionEditDialog                import CActionEditDialog
from Events.ActionInfo                      import CLocActionInfoList, CActionTypeInfo
from Events.ActionPropertiesTable           import CActionPropertiesTableModel
from Events.ActionTypeDialog                import CActionTypeDialogTableModel
from Events.ActionCreateDialog              import CActionCreateDialog
from Events.CreateEvent                     import editEvent, requestNewEvent
from Events.EventInfo                       import CEmergencyEventInfo, CEmergencyEventInfoList, CVisitInfoListEx, CVisitInfo, CEventTypeInfo, CFinanceInfo, CSceneInfo
from Events.Service                         import CServiceInfo
from Events.TempInvalidDuplicateEditDialog  import CTempInvalidDuplicateEditDialog
from Events.TempInvalidEditDialog           import CTempInvalidEditDialog, CTempInvalidCreateDialog, getEventListByDates, getTempInvalidIdOpen
from Events.TempInvalidInfo                 import CTempInvalidInfo, CTempInvalidAllInfoList
from Events.Utils                           import getActionTypeDescendants, getEventContext, getEventName, getActionTypeIdListByFlatCode, checkTissueJournalStatusByActions, getEventPrevEventTypeId, getEventPurposeId, getPayStatusMaskByCode, getPayStatusValueByCode, getPrevEventIdByEventTypeId, getWorkEventTypeFilter, payStatusText
from KLADR.Utils                            import getLikeMaskForRegion
from Notifications.NotificationRule import CNotificationRule
from Notifications.NotifyDialog import CNotifyDialog
from Notifications.NotificationLog import CNotificationLogList
from Orgs.Contracts                         import selectContract
from Orgs.Orgs                              import selectOrganisation
from Orgs.PersonInfo                        import CSpecialityInfo,  CPersonInfo
from Orgs.Utils                             import getOrganisationShortName, getOrgStructureAddressIdList, getOrgStructureDescendants, getOrgStructures, COrgStructureInfo
from RefBooks.RBContingentType              import CContingentTypeTranslator
from Registry.BatchRegistrationLocationCard import CGetParamsBatchRegistrationLocationCard, CSetParamsBatchRegistrationLocationCard
from Registry.BeforeRecordClient            import printOrderByScheduleItem
from Registry.ClientDocumentTracking        import CClientDocumentTrackingList
from Registry.ClientEditDialog              import CClientEditDialog
from Registry.RelationsClientListDialog     import CRelationsClientListDialog
from Registry.ClientVaccinationCard         import openClientVaccinationCard
from Registry.ComplaintsEditDialog          import CComplaintsEditDialog
from Registry.CPropertysComboBox            import CFilterPropertyOptionsItem
from Registry.policyBarCode                 import parsePolicyBarCode
from Registry.RegistryTable                 import (CActionsTableModel,
                                                    CMedicalCommissionActionsTableModel,
                                                    CProtocolMCActionsTableModel,
                                                    CMedicalSocialInspectionActionsTableModel,
                                                    CClientsTableModel,
                                                    CEventActionsTableModel,
                                                    CEventDiagnosticsTableModel,
                                                    CEventsTableModel,
                                                    CEventVisitsTableModel,
                                                    CExpertTempInvalidDocumentsTableModel,
                                                    CExpertTempInvalidPeriodsTableModel,
                                                    CExpertTempInvalidTableModel,
                                                    CVisitsTableModel
                                                    )
from Registry.ShowScheduleItemInfo          import showScheduleItemInfo
from Registry.SimplifiedClientSearch        import CSimplifiedClientSearch
from Registry.StatusObservationClientEditor import CStatusObservationClientEditor
from Registry.ShowContingentsClientDialog   import CShowContingentsClientDialog
from Registry.UpdateEventTypeByEvent        import CUpdateEventTypeByEvent
from Registry.Utils                         import CCheckNetMixin, CClientInfo, CClientInfoListEx, getRightEditTempInvalid, expertiseClass, replaceMask, deleteTempInvalid, formatAddressInt, formatDocument, formatPolicy, getClientBanner, getClientContextData, getClientInfo2, getClientMiniInfo
from Registry.VisitsBySchedules             import CVisitsBySchedulesDialog
from Reports.ReportClientServices           import CReportClientServices
from Reports.ReportView                     import CPageFormat
from Reports.Utils                          import getStringProperty, updateLIKE, existsPropertyValue
from Timeline.Schedule                      import CSchedule, confirmAndFreeScheduleItem
from Users.Rights                           import (urAdmin,
                                                    urRegTabEditExpertMC,
                                                    urBatchRegLocatCardProcess,
                                                    urCanEditClientVaccination,
                                                    urCanReadClientVaccination,
                                                    urEditAfterInvoicingEvent,
                                                    urRegTabReadLocationCard,
                                                    urEditLocationCard,
                                                    urEditStatusObservationClient,
                                                    urRegControlDoubles,
                                                    urRegDeleteTempInvalid,
                                                    urRegTabReadActions,
                                                    urRegTabReadAmbCard,
                                                    urRegTabReadEvents,
                                                    urRegTabReadExpert,
                                                    urRegTabReadRegistry,
                                                    urRegTabReadVisits,
                                                    urRegTabWriteActions,
                                                    urRegTabWriteEvents,
                                                    urRegTabWriteExpert,
                                                    urRegTabWriteRegistry,
                                                    urRegTabWriteVisits,
                                                    urUpdateEventTypeByEvent,
                                                    urSendInternalNotifications,
                                                    )
from Registry.Ui_Registry                   import Ui_RegistryWindow


class CRegistryWindow(QtGui.QScrollArea, Ui_RegistryWindow, CDialogPreferencesMixin, CCheckNetMixin, CConstructHelperMixin, CRecordLockMixin):
    u"""
        Окно для просмотра списка пациентов,
        обеспечивает возможность фильтрации, сортировки, редактирования
        описания пациента, перехода к связанным с данным пациентом event-ов
        и создания новых event-ов
    """
    def __init__(self, parent):
        QtGui.QScrollArea.__init__(self, parent)
        CCheckNetMixin.__init__(self)
        CRecordLockMixin.__init__(self)

        self.updateClientsListRequest = False

        self.addModels('Clients', CClientsTableModel(self))
        self.addModels('Schedules', CSchedulesModel(self))
        self.addModels('VisitsBySchedules', CVisitsBySchedulesModel(self))
        self.addModels('Events', CEventsTableModel(self, self.modelClients.recordCache()))
        self.addModels('EventDiagnostics', CEventDiagnosticsTableModel(self))
        self.addModels('EventActions', CEventActionsTableModel(self))
        self.addModels('EventVisits', CEventVisitsTableModel(self))
        self.addModels('ActionsStatus', CActionsTableModel(self, self.modelClients.recordCache(), self.modelEvents.recordCache()))
        self.addModels('ActionsStatusProperties', CActionPropertiesTableModel(self))
        self.addModels('ActionsDiagnostic', CActionsTableModel(self, self.modelClients.recordCache(), self.modelEvents.recordCache()))
        self.addModels('ActionsDiagnosticProperties', CActionPropertiesTableModel(self))
        self.addModels('ActionsCure', CActionsTableModel(self, self.modelClients.recordCache(), self.modelEvents.recordCache()))
        self.addModels('ActionsCureProperties', CActionPropertiesTableModel(self))
        self.addModels('ActionsMisc', CActionsTableModel(self, self.modelClients.recordCache(), self.modelEvents.recordCache()))
        self.addModels('ActionsMiscProperties', CActionPropertiesTableModel(self))
        self.addModels('ExpertTempInvalid', CExpertTempInvalidTableModel(self, self.modelClients.recordCache()))
        self.addModels('ExpertTempInvalidRelation', CExpertTempInvalidTableModel(self, self.modelClients.recordCache()))
        self.addModels('ExpertTempInvalidPeriods',    CExpertTempInvalidPeriodsTableModel(self))
        self.addModels('ExpertTempInvalidDocuments', CExpertTempInvalidDocumentsTableModel(self))
        self.addModels('ExpertTempInvalidDocumentsRelation', CExpertTempInvalidDocumentsTableModel(self))
        self.addModels('ExpertTempInvalidDocumentsEx', CExpertTempInvalidDocumentsTableModel(self))
        self.addModels('ExpertTempInvalidRelationDocumentsEx', CExpertTempInvalidDocumentsTableModel(self))
        self.addModels('ExpertTempInvalidPeriodsDocuments',    CExpertTempInvalidPeriodsTableModel(self))
        self.addModels('ExpertDisability', CExpertTempInvalidTableModel(self, self.modelClients.recordCache()))
        self.addModels('ExpertDisabilityRelation', CExpertTempInvalidTableModel(self, self.modelClients.recordCache()))
        self.addModels('ExpertDisabilityPeriods', CExpertTempInvalidPeriodsTableModel(self))
        self.addModels('ExpertDisabilityDocuments', CExpertTempInvalidDocumentsTableModel(self))
        self.addModels('ExpertDisabilityDocumentsRelation', CExpertTempInvalidDocumentsTableModel(self))
        self.addModels('ExpertDisabilityDocumentsEx', CExpertTempInvalidDocumentsTableModel(self))
        self.addModels('ExpertDisabilityRelationDocumentsEx', CExpertTempInvalidDocumentsTableModel(self))
        self.addModels('ExpertDisabilityPeriodsDocuments',    CExpertTempInvalidPeriodsTableModel(self))
        self.addModels('ExpertVitalRestriction', CExpertTempInvalidTableModel(self, self.modelClients.recordCache()))
        self.addModels('ExpertVitalRestrictionRelation', CExpertTempInvalidTableModel(self, self.modelClients.recordCache()))
        self.addModels('ExpertVitalRestrictionPeriods', CExpertTempInvalidPeriodsTableModel(self))
        self.addModels('ExpertVitalRestrictionDocuments', CExpertTempInvalidDocumentsTableModel(self))
        self.addModels('ExpertVitalRestrictionDocumentsRelation', CExpertTempInvalidDocumentsTableModel(self))
        self.addModels('ExpertVitalRestrictionDocumentsEx', CExpertTempInvalidDocumentsTableModel(self))
        self.addModels('ExpertVitalRestrictionRelationDocumentsEx', CExpertTempInvalidDocumentsTableModel(self))
        self.addModels('ExpertVitalRestrictionPeriodsDocuments',    CExpertTempInvalidPeriodsTableModel(self))
        self.addModels('ExpertDirectionsMC', CMedicalCommissionActionsTableModel(self, self.modelClients.recordCache(), self.modelEvents.recordCache()))
        self.addModels('ExpertProtocolsMC', CProtocolMCActionsTableModel(self, self.modelClients.recordCache(), self.modelEvents.recordCache()))
        self.addModels('ExpertMedicalSocialInspection', CMedicalSocialInspectionActionsTableModel(self, self.modelClients.recordCache(), self.modelEvents.recordCache()))
        self.addModels('ExpertDirectionsPropertiesMC', CActionPropertiesTableModel(self))
        self.addModels('ExpertProtocolsPropertiesMC', CActionPropertiesTableModel(self))
        self.addModels('ExpertMSIProperties', CActionPropertiesTableModel(self))
        self.addModels('FilterInfection', CRBCheckTableModel(self, 'rbInfection', u'Инфекции'))
        self.addModels('FilterVaccine', CRBCheckTableModel(self, 'rbVaccine', u'Вакцины'))
        self.addModels('Visits', CVisitsTableModel(self, self.modelClients.recordCache(), self.modelEvents.recordCache()))
        self.addObject('actEditClient', QtGui.QAction(u'Открыть регистрационную карточку', self))
        self.addObject('actRelationsClient', QtGui.QAction(u'Показать список связанных пациентов', self))
        self.addObject('actEventRelationsClient', QtGui.QAction(u'Показать список связанных пациентов', self))
        self.addObject('actAmbCardRelationsClient', QtGui.QAction(u'Показать список связанных пациентов', self))
        self.addObject('actActionsRelationsClient', QtGui.QAction(u'Показать список связанных пациентов', self))
        self.addObject('actExpertRelationsClient', QtGui.QAction(u'Показать список связанных пациентов', self))
        self.addObject('actVisitRelationsClient', QtGui.QAction(u'Показать список связанных пациентов', self))
        self.addObject('actOpenClientDocumentTrackingHistory', QtGui.QAction(u'Открыть журнал хранения учетных документов', self))
        self.addObject('actEditStatusObservationClient', QtGui.QAction(u'Изменить статус наблюдения пациента', self))
        self.addObject('actShowContingentsClient', QtGui.QAction(u'Показать все наблюдаемые контингенты', self))
        self.addObject('actPrintClient', getPrintAction(self, 'token', u'Напечатать талон'))
        self.addObject('actPrintClientLabel', QtGui.QAction(u'Напечатать визитку пациента', self))
        self.actPrintClientLabel.setShortcut('Shift+F6')
        self.clientLabelTemplate = getFirstPrintTemplate('clientLabel')
        self.addObject('actPrintClientList',   QtGui.QAction(u'Напечатать список пациентов', self))
        self.addObject('actEventPrint',  CPrintAction(u'Напечатать список обращений', None, self, self))
        self.addObject('actReportClientServices',  QtGui.QAction(u'Напечатать сводку об услугах на пациента', self))
        self.addObject('actEventEditClient',   QtGui.QAction(u'Открыть регистрационную карточку', self))
        self.addObject('actAmbCardEditClient', QtGui.QAction(u'Открыть регистрационную карточку', self))
        self.addObject('actActionEditClient',  QtGui.QAction(u'Открыть регистрационную карточку', self))
        self.addObject('actEditActionEvent',  QtGui.QAction(u'Редактировать обращение', self))
        self.addObject('actEditVisitEvent',  QtGui.QAction(u'Редактировать обращение', self))
        self.addObject('actExpertPrint',  CPrintAction(u'Напечатать список документов ВУТ', None, self, self))
        self.addObject('actExpertEditClient',  QtGui.QAction(u'Открыть регистрационную карточку', self))
        self.addObject('actExpertTempInvalidNext',       QtGui.QAction(u'Следующий эпизод', self))
        self.addObject('actExpertTempInvalidPrev',       QtGui.QAction(u'Предыдущий эпизод', self))
        self.addObject('actExpertTempInvalidDelete',     QtGui.QAction(u'Удалить эпизод', self))
        self.addObject('actExpertDisabilityNext',        QtGui.QAction(u'Следующий эпизод', self))
        self.addObject('actExpertDisabilityPrev',        QtGui.QAction(u'Предыдущий эпизод', self))
        self.addObject('actExpertDisabilityDelete',      QtGui.QAction(u'Удалить эпизод', self))
        self.addObject('actExpertVitalRestrictionNext',  QtGui.QAction(u'Следующий эпизод', self))
        self.addObject('actExpertVitalRestrictionPrev',  QtGui.QAction(u'Предыдущий эпизод', self))
        self.addObject('actExpertVitalRestrictionDelete',QtGui.QAction(u'Удалить эпизод', self))
        self.addObject('actExpertMedicalCommissionSelected', QtGui.QAction(u'Выделить группу', self))
        self.addObject('actExpertMedicalCommissionUpdateGroup', QtGui.QAction(u'Редактировать группу', self))
        self.addObject('actExpertMedicalCommissionClear',    QtGui.QAction(u'Снять выделение', self))
        self.addObject('actExpertMedicalCommissionDelete', QtGui.QAction(u'Удалить', self))
        self.addObject('actExpertMedicalCommissionUpdate', QtGui.QAction(u'Редактировать', self))
        self.addObject('actExpertMedicalCommissionMSI',    QtGui.QAction(u'Направить на МСЭ', self))
        self.addObject('actExpertMCUpdateTempInvalid',     QtGui.QAction(u'Редактор ВУТ', self))
        self.addObject('actListVisitsBySchedules',       QtGui.QAction(u'Протокол обращений пациента по предварительной записи', self))
        self.addObject('actBatchRegLocatCard',           QtGui.QAction(u'Изменить место нахождения карты пациентов', self))
        self.actBatchRegLocatCard.setShortcut('F5')
        self.addObject('actStatusObservationClient',     QtGui.QAction(u'Изменить статус наблюдения пациента', self))
        self.addObject('actControlDoublesRecordClient',  QtGui.QAction(u'Логический контроль двойников', self))
        self.addObject('actReservedOrderQueueClient',    QtGui.QAction(u'Использовать бронь в очереди', self))
        self.addObject('actOpenClientVaccinationCard',   QtGui.QAction(u'Открыть прививочную карту', self))
        self.addObject('actAmbCreateEvent',              QtGui.QAction(u'Новое обращение', self))
        self.addObject('actAmbDeleteOrder',              QtGui.QAction(u'Удалить из очереди', self))
        self.addObject('actAmbChangeNotes',              QtGui.QAction(u'Изменить жалобы', self))
        self.addObject('actAmbPrintOrder',               QtGui.QAction(u'Напечатать направление', self))
        self.addObject('actJumpQueuePosition',  QtGui.QAction(u'Перейти в график', self))
        self.addObject('actPrintBeforeRecords',  QtGui.QAction(u'Печать предварительной записи', self))
        self.addObject('actShowPreRecordInfo', QtGui.QAction(u'Свойства записи', self))
        self.addObject('actShowPreRecordInfoVisitBySchedule', QtGui.QAction(u'Свойства записи', self))
        self.addObject('actOpenAccountingByEvent',  QtGui.QAction(u'Перейти к счетам', self))
        self.addObject('actUpdateEventTypeByEvent',  QtGui.QAction(u'Изменить тип события', self))
        self.addObject('actOpenAccountingByAction', QtGui.QAction(u'Перейти к счетам', self))
        self.addObject('actOpenAccountingBySingleActionStatus', QtGui.QAction(u'Перейти к счетам', self))
        self.addObject('actOpenAccountingBySingleVisits', QtGui.QAction(u'Перейти к счетам', self))
        self.addObject('actOpenAccountingBySingleActionDiagnostic', QtGui.QAction(u'Перейти к счетам', self))
        self.addObject('actOpenAccountingBySingleActionCure', QtGui.QAction(u'Перейти к счетам', self))
        self.addObject('actOpenAccountingBySingleActionMisc', QtGui.QAction(u'Перейти к счетам', self))
        self.addObject('actOpenAccountingByVisit',  QtGui.QAction(u'Перейти к счетам', self))
        self.addObject('actConcatEvents',  QtGui.QAction(u'Объединить события', self))
        self.addObject('actMakePersonalAccount',  QtGui.QAction(u'Сформировать индивидуальный счет', self))
        self.addObject('actShowPacsImages',  QtGui.QAction(u'Показать все снимки', self))
        self.addObject('actNotify',  QtGui.QAction(u'Оповещение', self))
        self.addObject('actNotificationLog', QtGui.QAction(u'Журнал оповещений пациента', self))
        self.addObject('mnuPrint', QtGui.QMenu(self))
        self.mnuPrint.addAction(self.actPrintClient)
        self.mnuPrint.addAction(self.actPrintClientLabel)
        self.mnuPrint.addAction(self.actPrintClientList)
        self.addBarcodeScanAction('actScanBarcode')
        self.addObject('actSimplifiedClientSearch', QtGui.QAction(u'Упрощеный поиск пациента', self))
        self.actSimplifiedClientSearch.setShortcuts([QtGui.QKeySequence.Find,
                                                     'Alt+F7',
#                                                     QtGui.QKeySequence(Qt.ALT+Qt.Key_F7),
                                                    ]
                                                   )
        self.addObject('actCreateEvent', QtGui.QAction(u'Новое обращение', self))
        self.actCreateEvent.setShortcutContext(Qt.WidgetShortcut)
        self.actCreateEvent.setShortcut(Qt.Key_Space)

        self.setObjectName('RegistryWindow')
        self.internal = QtGui.QWidget(self)
        self.setWidget(self.internal)

        self.clientsSortingCol = 0
        self.clientsSortingDest = 'ASC'

        self.setupUi(self.internal)

        self.setWindowTitle(self.internal.windowTitle())
        self.setWidgetResizable(True)
        self.btnPrint.setMenu(self.mnuPrint)
        self.btnPrint.setShortcut('Alt+F6')
        if forceBool(QtGui.qApp.preferences.appPrefs.get('SocCard_SupportEnabled', False)):
            self.enableCardReaderSupport()
        self.btnEventPrint_actions = [{'action': self.actEventPrint, 'slot': self.on_actEventPrint_triggered},
                                      {'action': self.actReportClientServices, 'slot': self.on_actReportClientServices_triggered} ]
        eventTemplates = getPrintTemplates(getEventContextTemplate())
        if not eventTemplates:
            addButtonActions(self, self.btnEventPrint, self.btnEventPrint_actions, connect = False)
        else:
            addButtonActions(self, self.btnEventPrint, self.btnEventPrint_actions, connect = False)
            self.btnEventPrint.menu().addSeparator()
            subMenuDict={}
            for i, template in enumerate(eventTemplates):
                if template[2]==u'':
                    action = CPrintAction(template[0], template[1], self.btnEventPrint, self.btnEventPrint)
                    self.btnEventPrint.addAction(action)
                else:
                    subMenu = subMenuDict.get(template[2], QtGui.QMenu(template[2], self.parentWidget()))
                    action = CPrintAction(template[0], template[1], self.btnEventPrint, self.btnEventPrint)
                    subMenu.addAction(action)
                    subMenuDict[template[2]] = subMenu
            if len(subMenuDict):
                for subMenuKey in subMenuDict:
                    self.btnEventPrint.menu().addMenu(subMenuDict[subMenuKey])
            self.btnEventPrint.menu().addSeparator()
        self.btnExpertPrint_actions = [{'action': self.actExpertPrint, 'slot': self.on_actExpertPrint_triggered}, ]
        visitTemplates = getPrintTemplates(getVisitContext())
        if not visitTemplates:
            self.btnVisitPrint.setId(-1)
        else:
            subMenuDict={}
            for i, template in enumerate(visitTemplates):
                if template[2]==u'':
                    action = CPrintAction(template[0], template[1], self.btnVisitPrint, self.btnVisitPrint)
                    self.btnVisitPrint.addAction(action)
                else:
                    subMenu = subMenuDict.get(template[2], QtGui.QMenu(template[2], self.parentWidget()))
                    action = CPrintAction(template[0], template[1], self.btnVisitPrint, self.btnVisitPrint)
                    subMenu.addAction(action)
                    subMenuDict[template[2]] = subMenu
            if len(subMenuDict):
                for subMenuKey in subMenuDict:
                    self.btnVisitPrint.menu().addMenu(subMenuDict[subMenuKey])
            self.btnVisitPrint.menu().addSeparator()
            self.btnVisitPrint.addAction(CPrintAction(u'Напечатать список визитов', -1, self.btnVisitPrint, self.btnVisitPrint))
        addButtonActions(self, self.btnExpertPrint, self.btnExpertPrint_actions, connect = False)
        additionalCustomizePrintButton(self, self.btnExpertPrint, getKerContext(), self.btnExpertPrint_actions)
        actionTemplates = getPrintTemplates(getActionContext())
        if not actionTemplates:
            self.btnActionPrint.setId(-1)
        else:
            subMenuDict={}
            for i, template in enumerate(actionTemplates):
                if template[2]==u'':
                    action = CPrintAction(template[0], template[1], self.btnActionPrint, self.btnActionPrint)
                    self.btnActionPrint.addAction(action)
                else:
                    subMenu = subMenuDict.get(template[2], QtGui.QMenu(template[2], self.parentWidget()))
                    action = CPrintAction(template[0], template[1], self.btnActionPrint, self.btnActionPrint)
                    subMenu.addAction(action)
                    subMenuDict[template[2]] = subMenu
            if len(subMenuDict):
                for subMenuKey in subMenuDict:
                    self.btnActionPrint.menu().addMenu(subMenuDict[subMenuKey])
            self.btnActionPrint.menu().addSeparator()
            self.btnActionPrint.addAction(CPrintAction(u'Напечатать список', -1, self.btnActionPrint, self.btnActionPrint))
        clientTemplates = getPrintTemplates(getClientContext())
        if clientTemplates:
            self.btnPrint.menu().addSeparator()
            for template in clientTemplates:
                action = CPrintAction(template[0], template[1], self.btnPrint, self.btnPrint)
                self.btnPrint.addAction(action)
        self.setModels(self.tblFilterInfection,  self.modelFilterInfection,  self.selectionModelFilterInfection)
        self.setModels(self.tblFilterVaccine,    self.modelFilterVaccine,    self.selectionModelFilterVaccine)
        self.setModels(self.tblClients, self.modelClients, self.selectionModelClients)
        self.tblClients.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
        self.setModels(self.tblSchedules, self.modelSchedules, self.selectionModelSchedules)
        self.tblSchedules.horizontalHeader().moveSection(9, 2)
        self.setModels(self.tblVisitsBySchedules, self.modelVisitsBySchedules, self.selectionModelVisitsBySchedules)
        self.tblVisitsBySchedules.horizontalHeader().moveSection(8, 2)
        self.setModels(self.tblEvents, self.modelEvents, self.selectionModelEvents)
        self.tblEventDiagnostics.setModel(self.modelEventDiagnostics)
        self.tblEventActions.setModel(self.modelEventActions)
        self.tblEventVisits.setModel(self.modelEventVisits)
        self.setModels(self.tblActionsStatus, self.modelActionsStatus, self.selectionModelActionsStatus)
        self.setModels(self.tblActionsStatusProperties, self.modelActionsStatusProperties, self.selectionModelActionsStatusProperties)
        self.setModels(self.tblActionsDiagnostic, self.modelActionsDiagnostic, self.selectionModelActionsDiagnostic)
        self.setModels(self.tblActionsDiagnosticProperties, self.modelActionsDiagnosticProperties, self.selectionModelActionsDiagnosticProperties)
        self.setModels(self.tblActionsCure, self.modelActionsCure, self.selectionModelActionsCure)
        self.setModels(self.tblActionsCureProperties, self.modelActionsCureProperties, self.selectionModelActionsCureProperties)
        self.setModels(self.tblActionsMisc, self.modelActionsMisc, self.selectionModelActionsMisc)
        self.setModels(self.tblActionsMiscProperties, self.modelActionsMiscProperties, self.selectionModelActionsMiscProperties)
        self.setModels(self.tblExpertTempInvalid, self.modelExpertTempInvalid, self.selectionModelExpertTempInvalid)
        self.setModels(self.tblExpertTempInvalidRelation, self.modelExpertTempInvalidRelation, self.selectionModelExpertTempInvalidRelation)
        self.setModels(self.tblExpertTempInvalidPeriods, self.modelExpertTempInvalidPeriods, self.selectionModelExpertTempInvalidPeriods)
        self.setModels(self.tblExpertTempInvalidDocuments, self.modelExpertTempInvalidDocuments, self.selectionModelExpertTempInvalidDocuments)
        self.setModels(self.tblExpertTempInvalidDocumentsEx, self.modelExpertTempInvalidDocumentsEx, self.selectionModelExpertTempInvalidDocumentsEx)
        self.setModels(self.tblExpertTempInvalidRelationDocumentsEx, self.modelExpertTempInvalidRelationDocumentsEx, self.selectionModelExpertTempInvalidRelationDocumentsEx)
        self.setModels(self.tblExpertTempInvalidPeriodsDocuments, self.modelExpertTempInvalidPeriodsDocuments, self.selectionModelExpertTempInvalidPeriodsDocuments)
        self.setModels(self.tblExpertDisability, self.modelExpertDisability, self.selectionModelExpertDisability)
        self.setModels(self.tblExpertDisabilityRelation, self.modelExpertDisabilityRelation, self.selectionModelExpertDisabilityRelation)
        self.setModels(self.tblExpertDisabilityPeriods, self.modelExpertDisabilityPeriods, self.selectionModelExpertDisabilityPeriods)
        self.setModels(self.tblExpertDisabilityDocuments, self.modelExpertDisabilityDocuments, self.selectionModelExpertDisabilityDocuments)
        self.setModels(self.tblExpertDisabilityDocumentsEx, self.modelExpertDisabilityDocumentsEx, self.selectionModelExpertDisabilityDocumentsEx)
        self.setModels(self.tblExpertDisabilityRelationDocumentsEx, self.modelExpertDisabilityRelationDocumentsEx, self.selectionModelExpertDisabilityRelationDocumentsEx)
        self.setModels(self.tblExpertDisabilityPeriodsDocuments, self.modelExpertDisabilityPeriodsDocuments, self.selectionModelExpertDisabilityPeriodsDocuments)
        self.setModels(self.tblExpertVitalRestriction, self.modelExpertVitalRestriction, self.selectionModelExpertVitalRestriction)
        self.setModels(self.tblExpertVitalRestrictionRelation, self.modelExpertVitalRestrictionRelation, self.selectionModelExpertVitalRestrictionRelation)
        self.setModels(self.tblExpertVitalRestrictionPeriods, self.modelExpertVitalRestrictionPeriods, self.selectionModelExpertVitalRestrictionPeriods)
        self.setModels(self.tblExpertVitalRestrictionDocuments, self.modelExpertVitalRestrictionDocuments, self.selectionModelExpertVitalRestrictionDocuments)
        self.setModels(self.tblExpertVitalRestrictionDocumentsEx, self.modelExpertVitalRestrictionDocumentsEx, self.selectionModelExpertVitalRestrictionDocumentsEx)
        self.setModels(self.tblExpertVitalRestrictionRelationDocumentsEx, self.modelExpertVitalRestrictionRelationDocumentsEx, self.selectionModelExpertVitalRestrictionRelationDocumentsEx)
        self.setModels(self.tblExpertVitalRestrictionPeriodsDocuments, self.modelExpertVitalRestrictionPeriodsDocuments,     self.selectionModelExpertVitalRestrictionPeriodsDocuments)
        self.setModels(self.tblExpertDirectionsMC, self.modelExpertDirectionsMC, self.selectionModelExpertDirectionsMC)
        self.setModels(self.tblExpertProtocolsMC, self.modelExpertProtocolsMC, self.selectionModelExpertProtocolsMC)
        self.setModels(self.tblExpertMedicalSocialInspection, self.modelExpertMedicalSocialInspection, self.selectionModelExpertMedicalSocialInspection)
        self.setModels(self.tblExpertDirectionsPropertiesMC, self.modelExpertDirectionsPropertiesMC, self.selectionModelExpertDirectionsPropertiesMC)
        self.setModels(self.tblExpertProtocolsPropertiesMC, self.modelExpertProtocolsPropertiesMC, self.selectionModelExpertProtocolsPropertiesMC)
        self.setModels(self.tblExpertMSIProperties, self.modelExpertMSIProperties, self.selectionModelExpertMSIProperties)
        self.setModels(self.tblVisits, self.modelVisits, self.selectionModelVisits)
        self.cmbFilterDocumentType.setTable('rbDocumentType', True, 'group_id IN (SELECT id FROM rbDocumentTypeGroup WHERE code=\'1\')')
        self.edtFilterPolicyActualData.setDate(QDate.currentDate())
        self.cmbFilterPolicyType.setTable('rbPolicyType', True, specialValues=((-1, u'ОМС', u'ОМС'), ))
        self.cmbFilterPolicyKind.setTable('rbPolicyKind', True)
        self.cmbFilterAttachType.setTable('rbAttachType', True)
        self.cmbSocStatusesType.setTable('rbSocStatusType', True)
        self.cmbFilterVisitScene.setTable('rbScene', True)
        self.cmbFilterVisitType.setTable('rbVisitType', True)
        self.cmbFilterVisitProfile.setTable('rbService', True)
        self.cmbFilterServiceVisit.setTable('rbService', True)
        self.cmbFilterSceneVisit.setTable('rbScene', True)
        self.cmbFilterDocumentTypeForTracking.setTable('rbDocumentTypeForTracking', True)
        self.cmbFilterDocumentLocation.setTable('rbDocumentTypeLocation', True)
        self.cmbFilterStatusObservationType.setTable('rbStatusObservationClientType', True)
        self.cmbFilterVaccinationCalendar.setTable('rbVaccinationCalendar', addNone=True)
        self.cmbFilterMedicalExemptionType.setTable('rbMedicalExemptionType', addNone=True)
        self.cmbFilterClientConsentType.setTable('rbClientConsentType', addNone=True)
        self.cmbFilterContingentType.setTable('rbContingentType', addNone=True)
        self.edtFilterClientConsentBegDate.setDate(QDate())
        self.edtFilterClientConsentEndDate.setDate(QDate())
        self.idValidator = CIdValidator(self)
        self.edtFilterId.setValidator(self.idValidator)
        self.edtFilterBirthDay.setHighlightRedDate(False)
        self.cmbFilterAddressStreet.setAddNone(True)
        self.edtFilterSocStatusesBegDate.setDate(QDate())
        self.edtFilterSocStatusesEndDate.setDate(QDate())
        self.cmbFilterCreatePerson.setSpecialityPresent(False)
        self.cmbFilterModifyPerson.setSpecialityPresent(False)
        self.chkListOnClientsPage = [
            (self.chkFilterId,                    [self.edtFilterId,  self.cmbFilterAccountingSystem]),
            (self.chkFilterLastName,              [self.edtFilterLastName]),
            (self.chkFilterFirstName,             [self.edtFilterFirstName]),
            (self.chkFilterPatrName,              [self.edtFilterPatrName]),
            (self.chkFilterBirthDay,              [self.edtFilterBirthDay, self.chkFilterEndBirthDay]),
            (self.chkFilterEndBirthDay,           [self.edtFilterEndBirthDay]),
            (self.chkFilterSex,                   [self.cmbFilterSex]),
            (self.chkFilterContact,               [self.edtFilterContact]),
            (self.chkFilterSNILS,                 [self.edtFilterSNILS]),
            (self.chkFilterDocument,              [self.cmbFilterDocumentType,  self.edtFilterDocumentSerial,
                                                   self.edtFilterDocumentNumber]),
            (self.chkFilterPolicy,                [self.chkFilterPolicyOnlyActual, self.cmbFilterPolicyType,
                                                   self.cmbFilterPolicyKind, self.cmbFilterPolicyInsurer,
                                                   self.edtFilterPolicySerial, self.edtFilterPolicyNumber]),
            (self.chkFilterRegionSMO,             [self.cmbFilterRegionTypeSMO, self.cmbFilterRegionSMO]),
            (self.chkFilterPolicyOnlyActual,      [self.edtFilterPolicyActualData]),
            (self.chkFilterWorkOrganisation,      [self.cmbWorkOrganisation, self.btnSelectWorkOrganisation]),
            (self.chkSocStatuses,                 [self.edtFilterSocStatusesBegDate, self.edtFilterSocStatusesEndDate, self.cmbSocStatusesClass, self.cmbSocStatusesType]),
            (self.chkFilterDocumentLocation,      [self.cmbFilterDocumentLocation, self.edtBegDateFilterDocumentLocation, self.edtEndDateFilterDocumentLocation, self.cmbPersonDocumentLocation]),
            (self.chkFilterDocumentTypeForTracking,      [self.cmbFilterDocumentTypeForTracking]),
            (self.chkFilterStatusObservationType, [self.cmbFilterStatusObservationType]),
            (self.chkFilterContingentType,        [self.cmbFilterContingentType, self.cmbFilterContingentEventTypeStatus,
                                                   self.cmbFilterContingentActionType]),
            (self.chkFilterCreatePerson,          [self.cmbFilterCreatePerson]),
            (self.chkFilterCreateDate,            [self.edtFilterBegCreateDate, self.edtFilterEndCreateDate]),
            (self.chkFilterModifyPerson,          [self.cmbFilterModifyPerson]),
            (self.chkFilterModifyDate,            [self.edtFilterBegModifyDate, self.edtFilterEndModifyDate]),
            (self.chkFilterEvent,                 [self.chkFilterFirstEvent, self.edtFilterEventBegDate,
                                                   self.edtFilterEventEndDate]),
            (self.chkFilterAge,                   [self.edtFilterBegAge, self.cmbFilterBegAge,
                                                   self.edtFilterEndAge, self.cmbFilterEndAge]),
            (self.chkFilterAddress,               [self.cmbFilterAddressType,
                                                   self.cmbFilterAddressCity,
                                                   self.cmbFilterAddressOkato,
                                                   self.cmbFilterAddressStreet,
                                                   self.lblFilterAddressHouse, self.edtFilterAddressHouse,
                                                   self.chkFilterAddressCorpus, self.edtFilterAddressCorpus,
                                                   self.lblFilterAddressFlat, self.edtFilterAddressFlat]),
            (self.chkFilterAddressCorpus,         [self.edtFilterAddressCorpus]),
            (self.chkFilterAddressOrgStructure,   [self.cmbFilterAddressOrgStructureType,
                                                   self.cmbFilterAddressOrgStructure]),
            (self.chkFilterBeds,                  [self.cmbFilterStatusBeds, self.cmbFilterOrgStructureBeds]),
            (self.chkFilterRegAddressIsEmpty,     []),
            (self.chkFilterLocAddressIsEmpty,     []),
            (self.chkFilterAttachType,            [self.cmbFilterAttachCategory, self.edtFilterAttachBegDate, self.edtFilterAttachEndDate, self.cmbFilterAttachType, self.chkFilterToStatement]),
            (self.chkFilterToStatement,           [self.cmbFilterToStatement]),
            (self.chkFilterAttach,                [self.cmbFilterAttachOrganisation]),
            (self.chkFilterAttachNonBase,         []),
            (self.chkFilterTempInvalid,           [self.edtFilterBegTempInvalid, self.edtFilterEndTempInvalid]),
            (self.chkFilterTFUnconfirmed,         []),
            (self.chkFilterTFConfirmed,           [self.edtFilterBegTFConfirmed, self.edtFilterEndTFConfirmed]),
            (self.chkFilterClientConsent,         [self.cmbFilterClientConsentType, self.pnlFilterClientConsentDateRange, self.cmbFilterPersonConsent, self.cmbFilterClientConsentValue]),
            (self.chkFilterVaccinationPeriod,     [self.edtFilterVaccinationBegDate,
                                                   self.edtFilterVaccinationEndDate]),
            (self.chkFilterVaccinationCalendar,   [self.cmbFilterVaccinationCalendar]),
            (self.chkFilterVaccinationSeria,      [self.edtFilterVaccinationSeria]),
            (self.chkFilterVaccinationType,       [self.cmbFilterVaccinationType]),
            (self.chkFilterContingent,            [self.cmbFilterContingent]),
            (self.chkFilterMedicalExemption,      [self.cmbFilterMedicalExemption]),
            (self.chkFilterMedicalExemptionType,  [self.cmbFilterMedicalExemptionType]),
            (self.chkFilterExcludeLeaved, []),
            (self.chkFilterDead, []),
            (self.chkClientMKB, [self.edtClientMKBFrom, self.edtClientMKBTo])
            ]

        self.onChkFilterToggled(self.chkFilterClientConsent, False)
        self.edtFilterEventId.setValidator(self.idValidator)
        self.edtFilterJobTicketId.setValidator(self.idValidator)
        self.edtFilterActionJobTicketId.setValidator(self.idValidator)
        self.cmbFilterEventPurpose.setTable('rbEventTypePurpose', False, filter='code != \'0\'')
        self.cmbFilterEventType.setTable('EventType', False, filter=getWorkEventTypeFilter())
        self.cmbFilterEventSpeciality.setTable('rbSpeciality', False)
        self.cmbFilterEventPerson.setOrgId(None)
        self.cmbFilterEventPerson.setAddNone(False)
        self.cmbFilterEventPerson.addNotSetValue()
        self.cmbFilterEventCreatePerson.setSpecialityPresent(False)
        self.cmbFilterEventModifyPerson.setSpecialityPresent(False)
        self.cmbFilterAccountingSystem.setTable('rbAccountingSystem',  True)
        self.cmbFilterAccountingSystem.setValue(forceRef(QtGui.qApp.preferences.appPrefs.get('FilterAccountingSystem', 0)))
        self.chkListOnEventsPage = [
            (self.chkFilterEventPurpose, [self.cmbFilterEventPurpose]),
            (self.chkFilterEventType, [self.cmbFilterEventType]),
            (self.chkFilterEventSetDate, [self.edtFilterEventBegSetDate, self.edtFilterEventEndSetDate, self.edtFilterEventBegSetTime, self.edtFilterEventEndSetTime]),
            (self.chkFilterEventEmptyExecDate, []),
            (self.chkFilterEventExecDate, [self.edtFilterEventBegExecDate, self.edtFilterEventEndExecDate, self.edtFilterEventBegExecTime, self.edtFilterEventEndExecTime]),
            (self.chkFilterEventNextDate, [self.edtFilterEventBegNextDate, self.edtFilterEventEndNextDate]),
            (self.chkFilterEventOrgStructure, [self.cmbFilterEventOrgStructure]),
            (self.chkFilterEventSpeciality, [self.cmbFilterEventSpeciality]),
            (self.chkFilterEventPerson, [self.cmbFilterEventPerson]),
            (self.chkFilterEventIsPrimary, [self.cmbFilterEventIsPrimary]),
            (self.chkFilterEventOrder, [self.cmbFilterEventOrder]),
            (self.chkFilterEventDispanserObserved, []),
            (self.chkFilterEventLPU, [self.cmbFilterEventLPU]),
            (self.chkFilterEventNonBase, []),
            (self.chkRelegateOrg, [self.cmbRelegateOrg]),
            (self.chkFilterEventMes, [self.edtFilterEventMes]),
            (self.chkMKB, [self.chkPreliminary, self.chkConcomitant, self.edtMKBFrom, self.edtMKBTo]),
            (self.chkFilterEventResult, [self.cmbFilterEventResult]),
            (self.chkFilterEventCreatePerson, [self.cmbFilterEventCreatePerson]),
            (self.chkFilterEventCreateDate, [self.edtFilterEventBegCreateDate, self.edtFilterEventEndCreateDate]),
            (self.chkFilterEventModifyPerson, [self.cmbFilterEventModifyPerson]),
            (self.chkFilterEventModifyDate, [self.edtFilterEventBegModifyDate, self.edtFilterEventEndModifyDate]),
            (self.chkErrorInDiagnostic, []),
            (self.chkFilterEventPayStatus, [self.cmpFilterEventPayStatusCode, self.cmbFilterEventPayStatusFinance]),
            (self.chkFilterVisitPayStatus, [self.cmpFilterVisitPayStatusCode, self.cmbFilterVisitPayStatusFinance]),
            (self.chkFilterEventId, [self.edtFilterEventId]),
            (self.chkFilterJobTicketId, [self.edtFilterJobTicketId]),
            (self.chkFilterExternalId, [self.edtFilterExternalId]),
            (self.chkFilterAccountSumLimit, [self.cmbFilterAccountSumLimit, self.edtFilterSumLimitFrom, self.edtFilterSumLimitTo, self.edtFilterSumLimitDelta]),
            (self.chkFilterVisits, [self.cmbFilterVisitScene, self.cmbFilterVisitType, self.cmbFilterVisitProfile]),
            (self.chkFilterEventPayer, [self.cmbFilterEventPayer, self.btnFilterEventSelectPayer]),
            (self.chkFilterEventContract, [self.edtFilterEventContract, self.btnFilterEventContract]),
            (self.chkFilterEventExpertId, [self.cmbFilterEventExpertId]),
            (self.chkFilterEventExpertiseDate, [self.edtFilterEventBegExpertiseDate, self.edtFilterEventEndExpertiseDate]),
            ]
        self.cmbFilterActionType.setAllSelectable(True)
        self.cmbFilterActionType.setClass(0)
        self.cmbFilterActionType.setClassesPopup([0])
        self.cmbFilterActionSetSpeciality.setTable('rbSpeciality', False)
        self.cmbFilterExecSetSpeciality.setTable('rbSpeciality', False)
        self.cmbFilterActionSetPerson.setOrgId(None)
        self.cmbFilterActionSetPerson.setAddNone(False)
        self.cmbFilterExecSetPerson.setOrgId(None)
        self.cmbFilterExecSetPerson.setAddNone(True)
        self.cmbFilterExecSetPerson.addNotSetValue()
        self.cmbFilterAssistant.addNotSetValue()
        self.cmbFilterActionCreatePerson.setSpecialityPresent(False)
        self.cmbFilterActionModifyPerson.setSpecialityPresent(False)
        self.cmbFilterActionFinance.setTable('rbFinance', addNone=True)
        self.__filterActionContractId = None
        self.chkListOnActionsPage = [
            (self.chkFilterActionType, [self.cmbFilterActionType, self.chkListProperty]),
            (self.chkFilterActionStatus,     [self.cmbFilterActionStatus]),
            (self.chkFilterActionSetDate,    [self.edtFilterActionBegSetDate, self.edtFilterActionEndSetDate]),
            (self.chkFilterActionSetOrgStructure, [self.cmbFilterActionSetOrgStructure]),
            (self.chkFilterActionSetSpeciality, [self.cmbFilterActionSetSpeciality]),
            (self.chkFilterActionSetPerson, [self.cmbFilterActionSetPerson]),
            (self.chkFilterActionIsUrgent, []),
            (self.chkFilterActionPlannedEndDate, [self.edtFilterActionBegPlannedEndDate, self.edtFilterActionEndPlannedEndDate]),
            (self.chkFilterActionExecDate, [self.edtFilterActionBegExecDate, self.edtFilterActionBegExecTime, self.edtFilterActionEndExecDate, self.edtFilterActionEndExecTime]),
            (self.chkFilterExecSetPerson, [self.cmbFilterExecSetPerson]),
            (self.chkFilterExecSetOrgStructure, [self.cmbFilterExecSetOrgStructure]),
            (self.chkFilterExecSetSpeciality, [self.cmbFilterExecSetSpeciality]),
            (self.chkFilterAssistant, [self.cmbFilterAssistant]),
            (self.chkActionMKB, [self.edtActionMKBFrom, self.edtActionMKBTo]),
            (self.chkTakenTissueJournal, [self.cmbTakenTissueJournal]),
            (self.chkUncoordinated, []),
            (self.chkFilterActionJobTicketId, [self.edtFilterActionJobTicketId]),
            (self.chkFilterActionCreatePerson, [self.cmbFilterActionCreatePerson]),
            (self.chkFilterActionCreateDate, [self.edtFilterActionBegCreateDate, self.edtFilterActionEndCreateDate]),
            (self.chkFilterActionModifyPerson, [self.cmbFilterActionModifyPerson]),
            (self.chkFilterActionModifyDate, [self.edtFilterActionBegModifyDate, self.edtFilterActionEndModifyDate]),
            (self.chkFilterActionPayStatus, [self.cmpFilterActionPayStatusCode, self.cmbFilterActionPayStatusFinance]),
            (self.chkTakeIntoAccountProperty, [self.chkFilledProperty, self.chkThresholdPenaltyGrade]),
            (self.chkThresholdPenaltyGrade, [self.edtThresholdPenaltyGrade]),
            (self.chkListProperty, [self.tblListOptionProperty]),
            (self.chkFilledProperty, []),
            (self.chkFilterActionFinance, [self.cmbFilterActionFinance]),
            (self.chkFilterActionPayer, [self.cmbFilterActionPayer, self.btnFilterActionSelectPayer]),
            (self.chkFilterActionContract, [self.edtFilterActionContract, self.btnFilterActionContract])
            ]
        self.__actionTypeIdListByClassPage = [None] * 4
        self.cmbFilterVisitsType.setTable('rbVisitType', False)
        self.cmbFilterExecSetSpecialityVisit.setTable('rbSpeciality', False)
        self.cmbFilterVisitFinance.setTable('rbFinance', True)
        self.cmbFilterExecSetPersonVisit.setOrgId(None)
        self.cmbFilterExecSetPersonVisit.setAddNone(True)
        self.cmbFilterExecSetPersonVisit.addNotSetValue()
        self.cmbFilterVisitAssistant.addNotSetValue()
        self.cmbFilterVisitCreatePersonEx.setSpecialityPresent(False)
        self.cmbFilterVisitModifyPersonEx.setSpecialityPresent(False)
        self.chkListOnVisitsPage = [
            (self.chkFilterVisitType, [self.cmbFilterVisitsType]),
            (self.chkFilterVisitExecDate, [self.edtFilterVisitBegExecDate, self.edtFilterVisitBegExecTime, self.edtFilterVisitEndExecDate, self.edtFilterVisitEndExecTime]),
            (self.chkFilterExecSetSpecialityVisit, [self.cmbFilterExecSetSpecialityVisit]),
            (self.chkFilterExecSetOrgStructureVisit, [self.cmbFilterExecSetOrgStructureVisit]),
            (self.chkFilterExecSetPersonVisit, [self.cmbFilterExecSetPersonVisit]),
            (self.chkFilterVisitAssistant, [self.cmbFilterVisitAssistant]),
            (self.chkFilterVisitFinance, [self.cmbFilterVisitFinance]),
            (self.chkFilterServiceVisit, [self.cmbFilterServiceVisit]),
            (self.chkFilterSceneVisit, [self.cmbFilterSceneVisit]),
            (self.chkFilterVisitCreatePersonEx, [self.cmbFilterVisitCreatePersonEx]),
            (self.chkFilterVisitCreateDateEx, [self.edtFilterVisitBegCreateDateEx, self.edtFilterVisitEndCreateDateEx]),
            (self.chkFilterVisitModifyPersonEx, [self.cmbFilterVisitModifyPersonEx]),
            (self.chkFilterVisitModifyDateEx, [self.edtFilterVisitBegModifyDateEx, self.edtFilterVisitEndModifyDateEx]),
            (self.chkFilterVisitPayStatusEx, [self.cmpFilterVisitPayStatusCodeEx, self.cmbFilterVisitPayStatusFinanceEx])
            ]
        self.chkListOnExpertPage = [
            (self.chkFilterExpertDocType, [self.cmbFilterExpertDocType]),
            (self.chkPrimaryOrDuble, [self.cmbPrimaryOrDuble]),
            (self.chkFilterExpertSerial,  [self.edtFilterExpertSerial]),
            (self.chkFilterExpertNumber,  [self.edtFilterExpertNumber]),
            (self.chkFilterExpertReason,  [self.cmbFilterExpertReason]),
            (self.chkFilterExpertBegDate, [self.edtFilterExpertBegBegDate, self.edtFilterExpertEndBegDate]),
            (self.chkFilterExpertEndDate, [self.edtFilterExpertBegEndDate, self.edtFilterExpertEndEndDate]),
            (self.chkFilterExpertOrgStruct, [self.cmbFilterExpertOrgStruct]),
            (self.chkFilterExpertSpeciality, [self.cmbFilterExpertSpeciality]),
            (self.chkFilterExpertPerson,  [self.cmbFilterExpertPerson]),
            (self.chkFilterExpertMKB,     [self.edtFilterExpertBegMKB,     self.edtFilterExpertEndMKB]),
            (self.chkFilterExpertClosed,  [self.cmbFilterExpertClosed]),
            (self.chkFilterExpertDuration,[self.edtFilterExpertBegDuration, self.edtFilterExpertEndDuration]),
            (self.chkFilterExpertInsuranceOfficeMark,[self.cmbFilterExpertInsuranceOfficeMark]),
            (self.chkFilterExpertLinked,  []),
            (self.chkFilterExpertExternal, [self.cmbFilterExpertExternal]),
            (self.chkFilterExpertCreatePerson, [self.cmbFilterExpertCreatePerson]),
            (self.chkFilterExpertCreateDate,   [self.edtFilterExpertBegCreateDate, self.edtFilterExpertEndCreateDate]),
            (self.chkFilterExpertModifyPerson, [self.cmbFilterExpertModifyPerson]),
            (self.chkFilterExpertModifyDate,   [self.edtFilterExpertBegModifyDate, self.edtFilterExpertEndModifyDate]),
            ]
        self.initFilterExpertExpertiseMC()
        self.chkListOnExpertMCPage = [
            (self.chkFilterExpertExpertiseMC,  [self.cmbFilterExpertExpertiseMC]),
            (self.chkFilterExpertExpertiseTypeMC, [self.cmbFilterExpertExpertiseTypeMC]),
            (self.chkFilterExpertNumberExpertise, [self.edtFilterExpertNumberExpertise]),
            (self.chkFilterExpertNumberMC,     [self.edtFilterExpertNumberMC]),
            (self.chkFilterExpertSetPersonMC,  [self.cmbFilterExpertSetPersonMC]),
            (self.chkFilterExpertOrgStructMC,  [self.cmbFilterExpertOrgStructMC]),
            (self.chkFilterExpertSpecialityMC, [self.cmbFilterExpertSpecialityMC]),
            (self.chkFilterDirectDateOnMC,     [self.edtFilterBegDirectDateOnMC, self.edtFilterEndDirectDateOnMC]),
            (self.chkFilterExecDateMC,         [self.edtFilterBegExecDateMC, self.edtFilterEndExecDateMC]),
            (self.chkFilterExpertClosedMC,     [self.cmbFilterExpertClosedMC]),
            (self.chkFilterExpertExpertiseCharacterMC, [self.cmbFilterExpertExpertiseCharacterMC]),
            (self.chkFilterExpertExpertiseKindMC,      [self.cmbFilterExpertExpertiseKindMC]),
            (self.ghkFilterExpertExpertiseObjectMC,    [self.cmbFilterExpertExpertiseObjectMC]),
            (self.chkFilterExpertExpertiseArgumentMC,  [self.cmbFilterExpertExpertiseArgumentMC]),
            (self.chkExpertIdMC,               [self.cmbExpertIdMC]),
            (self.chkFilterExpertMKBMC,        [self.edtFilterExpertBegMKBMC, self.edtFilterExpertEndMKBMC]),
            (self.chkFilterExpertCreateDateMC, [self.edtFilterExpertBegCreateDateMC, self.edtFilterExpertEndCreateDateMC]),
            (self.chkFilterExpertCreatePersonMC, [self.cmbFilterExpertCreatePersonMC]),
            (self.chkFilterExpertModifyDateMC,   [self.edtFilterExpertBegModifyDateMC, self.edtFilterExpertEndModifyDateMC]),
            (self.chkFilterExpertModifyPersonMC, [self.cmbFilterExpertModifyPersonMC]),
            ]
        self.__tempInvalidDocTypeIdListByTypePage = [None] * 6
        QMetaObject.connectSlotsByName(self) # т.к. в setupUi параметр не self
        for i in range(7):
            self.tblClients.setColumnHidden(6+i, True)
        self.tblActionsStatusProperties.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        self.tblActionsDiagnosticProperties.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        self.tblActionsCureProperties.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        self.tblActionsMiscProperties.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        self.tblExpertDirectionsPropertiesMC.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        self.tblExpertProtocolsPropertiesMC.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        self.tblExpertMSIProperties.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        self.TFAccountingSystemId = QtGui.qApp.TFAccountingSystemId()
        if not self.TFAccountingSystemId:
            self.chkFilterTFUnconfirmed.setEnabled(False)
            self.chkFilterTFConfirmed.setEnabled(False)
        self.__filter = {}
        self.bufferRecordsOptionProperty = []
        self.__filterEventContractId = None
        self.filterEventContractParams = {}
        self.txtClientInfoBrowser.actions.append(self.actEditClient)
        self.txtClientInfoBrowser.actions.append(self.actRelationsClient)
        self.txtClientInfoBrowser.actions.append(self.actOpenClientDocumentTrackingHistory)
        self.txtClientInfoBrowser.actions.append(self.actEditStatusObservationClient)
        self.txtClientInfoBrowser.actions.append(self.actShowContingentsClient)
        self.txtClientInfoBrowserEvents.actions.append(self.actEventEditClient)
        self.txtClientInfoBrowserEvents.actions.append(self.actEventRelationsClient)
        self.txtClientInfoBrowserAmbCard.actions.append(self.actAmbCardEditClient)
        self.txtClientInfoBrowserAmbCard.actions.append(self.actAmbCardRelationsClient)
        self.txtClientInfoBrowserActions.actions.append(self.actActionEditClient)
        self.txtClientInfoBrowserActions.actions.append(self.actActionsRelationsClient)
        self.txtClientInfoBrowserExpert.actions.append(self.actExpertEditClient)
        self.txtClientInfoBrowserExpert.actions.append(self.actExpertRelationsClient)
        self.txtClientInfoBrowserVisits.actions.append(self.actActionEditClient)
        self.txtClientInfoBrowserVisits.actions.append(self.actVisitRelationsClient)
        self.tblClients.createPopupMenu(
                [
                    self.actSimplifiedClientSearch,
                    self.actCreateEvent,
                    self.actListVisitsBySchedules,
                    self.actBatchRegLocatCard,
                    self.actStatusObservationClient,
                    self.actControlDoublesRecordClient,
                    self.actReservedOrderQueueClient,
                    self.actOpenClientVaccinationCard,
                    self.actShowPacsImages,
                    self.actNotify,
                    self.actNotificationLog,
                    self.actShowContingentsClient,
                ]
                )
        self.tblClients.addPopupRecordProperies()

        self.tblEvents.addPopupAction(self.actOpenAccountingByEvent)
        self.tblEvents.addPopupAction(self.actUpdateEventTypeByEvent)
        self.tblEvents.addPopupAction(self.actConcatEvents)
        self.tblEvents.addPopupAction(self.actMakePersonalAccount)
        self.tblEvents.addPopupDelRow()
        self.tblEventActions.addPopupAction(self.actOpenAccountingByAction)
        self.tblEventVisits.addPopupAction(self.actOpenAccountingByVisit)
        self.tblActionsStatus.createPopupMenu([self.actEditActionEvent])
        self.tblActionsStatus.addPopupAction(self.actOpenAccountingBySingleActionStatus)
        self.tblVisits.createPopupMenu([self.actEditVisitEvent])
        self.tblVisits.addPopupAction(self.actOpenAccountingBySingleVisits)
        self.tblActionsDiagnostic.createPopupMenu([self.actEditActionEvent])
        self.tblActionsDiagnostic.addPopupAction(self.actOpenAccountingBySingleActionDiagnostic)
        self.tblActionsCure.createPopupMenu([self.actEditActionEvent])
        self.tblActionsCure.addPopupAction(self.actOpenAccountingBySingleActionCure)
        self.tblActionsMisc.createPopupMenu([self.actEditActionEvent])
        self.tblActionsMisc.addPopupAction(self.actOpenAccountingBySingleActionMisc)
        self.tblExpertTempInvalid.createPopupMenu([self.actExpertTempInvalidNext, self.actExpertTempInvalidPrev, '-', self.actExpertTempInvalidDelete])
        self.tblExpertTempInvalidRelation.createPopupMenu([self.actExpertTempInvalidNext, self.actExpertTempInvalidPrev, '-', self.actExpertTempInvalidDelete])
        self.tblExpertDisability.createPopupMenu([self.actExpertDisabilityNext, self.actExpertDisabilityPrev, '-', self.actExpertDisabilityDelete])
        self.tblExpertDisabilityRelation.createPopupMenu([self.actExpertDisabilityNext, self.actExpertDisabilityPrev, '-', self.actExpertDisabilityDelete])
        self.tblExpertVitalRestriction.createPopupMenu([self.actExpertVitalRestrictionNext, self.actExpertVitalRestrictionPrev, '-', self.actExpertVitalRestrictionDelete])
        self.tblExpertVitalRestrictionRelation.createPopupMenu([self.actExpertVitalRestrictionNext, self.actExpertVitalRestrictionPrev, '-', self.actExpertVitalRestrictionDelete])
        self.tblExpertDirectionsMC.createPopupMenu([self.actExpertMedicalCommissionDelete, '-', self.actExpertMedicalCommissionSelected, self.actExpertMedicalCommissionUpdateGroup, self.actExpertMedicalCommissionClear, '-', self.actExpertMedicalCommissionUpdate, self.actExpertMCUpdateTempInvalid])
        self.tblExpertProtocolsMC.createPopupMenu([self.actExpertMedicalCommissionDelete, '-', self.actExpertMedicalCommissionSelected, self.actExpertMedicalCommissionUpdateGroup, self.actExpertMedicalCommissionClear, '-', self.actExpertMedicalCommissionUpdate, self.actExpertMCUpdateTempInvalid, self.actExpertMedicalCommissionMSI])
        self.tblExpertMedicalSocialInspection.createPopupMenu([self.actExpertMedicalCommissionDelete, '-', self.actExpertMedicalCommissionUpdate, '-', self.actExpertMCUpdateTempInvalid])
        self.tblActionsStatus.addPopupRecordProperies()
        self.tblActionsDiagnostic.addPopupRecordProperies()
        self.tblActionsCure.addPopupRecordProperies()
        self.tblActionsMisc.addPopupRecordProperies()
        self.tblSchedules.createPopupMenu([self.actAmbCreateEvent, self.actAmbDeleteOrder, self.actAmbChangeNotes, self.actAmbPrintOrder, self.actJumpQueuePosition, self.actPrintBeforeRecords, self.actShowPreRecordInfo])
        self.tblVisitsBySchedules.createPopupMenu([self.actShowPreRecordInfoVisitBySchedule])
        self.connect(self.tblEvents.horizontalHeader(), SIGNAL('sectionClicked(int)'), self._setEventsOrderByColumn)
        self.connect(self.modelFilterInfection,
                     SIGNAL('itemCheckingChanged()'),
                     self.on_modelFilterInfection_itemCheckingChanged)
        self.connect(self.modelFilterVaccine,
                     SIGNAL('itemCheckingChanged()'),
                     self.on_modelFilterVaccine_itemCheckingChanged)
        self.connect(self.tblActionsStatus.horizontalHeader(), SIGNAL('sectionClicked(int)'), self._setActionsOrderByColumn)
        self.connect(self.tblActionsDiagnostic.horizontalHeader(), SIGNAL('sectionClicked(int)'), self._setActionsOrderByColumn)
        self.connect(self.tblActionsCure.horizontalHeader(), SIGNAL('sectionClicked(int)'), self._setActionsOrderByColumn)
        self.connect(self.tblActionsMisc.horizontalHeader(), SIGNAL('sectionClicked(int)'), self._setActionsOrderByColumn)
        self.connect(self.tblExpertDirectionsMC.horizontalHeader(), SIGNAL('sectionClicked(int)'), self._setMedicalCommissionActionsOrderByColumn)
        self.connect(self.tblExpertProtocolsMC.horizontalHeader(), SIGNAL('sectionClicked(int)'), self._setMedicalCommissionActionsOrderByColumn)
        self.connect(self.tblExpertMedicalSocialInspection.horizontalHeader(), SIGNAL('sectionClicked(int)'), self._setMedicalCommissionActionsOrderByColumn)
        self.setFocusProxy(self.tabMain)
        self.internal.setFocusProxy(self.tabMain)
        self.tabMain.setFocusProxy(self.tblClients) # на первый взгляд не нужно, но строка ниже не справляется...
        self.tabRegistry.setFocusProxy(self.tblClients)
        self.tabEvents.setFocusProxy(self.tblEvents)
        self.tabMainCurrentPage = 0
        self.currentClientFromEvent = False
        self.currentClientFromAction = False
        self.currentClientFromExpert = False
        self.sortAscendingList = {}
        self.controlSplitter = self.splitterRegistry
        self.tabMain.setCurrentIndex(self.tabMainCurrentPage)
        self.loadDialogPreferences()
        self.cmbFilterAddressCity.setAreaSelectable(True)
        self.cmbFilterAddressCity.setCode(QtGui.qApp.defaultKLADR())
        self.personInfo = CRBModel(self)
        self.personInfo.setTable('vrbPersonWithSpeciality')
        self.connect(QtGui.qApp, SIGNAL('currentClientInfoChanged()'), self.updateQueue)
        self.tabRegistry.addAction(self.actScanBarcode)
        self.tabRegistry.addAction(self.actBatchRegLocatCard)
        self.tabRegistry.addAction(self.actSimplifiedClientSearch)
        self.tblClients.addAction(self.actCreateEvent)
        self.cmbFilterRegionSMO.setAreaSelectable(True)
        self.tblEvents.enableColsHide()
        self.tblEvents.enableColsMove()
        self.tblEventDiagnostics.enableColsHide()
        self.tblEventDiagnostics.enableColsMove()
        self.tblEventActions.enableColsHide()
        self.tblEventActions.enableColsMove()
        self.tblEventVisits.enableColsHide()
        self.tblEventVisits.enableColsMove()
        self.tblActionsStatus.enableColsHide()
        self.tblActionsDiagnostic.enableColsHide()
        self.tblActionsCure.enableColsHide()
        self.tblActionsMisc.enableColsHide()
        self.tblActionsStatus.enableColsMove()
        self.tblActionsDiagnostic.enableColsMove()
        self.tblActionsCure.enableColsMove()
        self.tblActionsMisc.enableColsMove()
        self.tblVisits.enableColsHide()
        self.tblVisits.enableColsMove()
        self.tblExpertDirectionsMC.enableColsHide()
        self.tblExpertDirectionsMC.enableColsMove()
        self.tblExpertProtocolsMC.enableColsHide()
        self.tblExpertProtocolsMC.enableColsMove()
        self.tblExpertMedicalSocialInspection.enableColsHide()
        self.tblExpertMedicalSocialInspection.enableColsMove()
        self.setUserRights()
        self.isAscendingSchedules = False
        self.isAscendingBySchedules = False
        # сканирование штрих-кода; КЭР
        self.addBarcodeScanAction('actScanBarcodeTabExpert')
        self.tabExpert.addAction(self.actScanBarcodeTabExpert)
        self.connect(self.actScanBarcodeTabExpert, SIGNAL('triggered()'), self.on_actScanBarcodeTabExpert)
        self.connect(QtGui.qApp, SIGNAL('scannedReceived(QByteArray)'), self.on_scannedReceived)
        # считывание смарт-карты
        self.connect(QtGui.qApp, SIGNAL('policySmartCardReceived(QVariant)'), self.onPolicySmartCardReceived)
        QObject.connect(self.tblExpertTempInvalid.horizontalHeader(), SIGNAL('sectionClicked(int)'), self.setXXXSort)
        QObject.connect(self.tblExpertTempInvalidRelation.horizontalHeader(), SIGNAL('sectionClicked(int)'), self.setXXXSort)
        QObject.connect(self.tblClients.horizontalHeader(), SIGNAL('sectionClicked(int)'), self.setClientSort)
        QObject.connect(self.tblSchedules.horizontalHeader(), SIGNAL('sectionClicked(int)'), self.setSchedulesSort)
        QObject.connect(self.tblVisitsBySchedules.horizontalHeader(), SIGNAL('sectionClicked(int)'), self.setBySchedulesSort)
        self.loadSortAscendingList()
        self.extendedPolicyInformation = {}
        self.chkFilterEventContract.setChecked(forceBool(QtGui.qApp.preferences.appPrefs.get('Registry_chkFilterEventContract', False)))
        self.__filterEventContractId = forceRef(QtGui.qApp.preferences.appPrefs.get('Registry_filterEventContractId', None))
        self.edtFilterEventContract.setText(forceString(QtGui.qApp.preferences.appPrefs.get('Registry_edtFilterEventContract', '')))
        self.tblExpertTempInvalidRelation.setVisible(False)
        self.tblExpertDisabilityRelation.setVisible(False)
        self.tblExpertVitalRestrictionRelation.setVisible(False)
        self.tblExpertTempInvalidRelationDocumentsEx.setVisible(False)
        self.tblExpertDisabilityRelationDocumentsEx.setVisible(False)
        self.tblExpertVitalRestrictionRelationDocumentsEx.setVisible(False)
        self.initDocumentsTempInvalidTextEdit()
        self.medicalCommissionSelectedRows = []


    def initFilterExpertExpertiseMC(self):
        itemList = []
        for item in expertiseClass:
            itemList.append(item[0])
        self.cmbFilterExpertExpertiseMC.addItems(itemList)
        self.cmbFilterExpertExpertiseMC.setCurrentIndex(0)
        if not self.chkFilterExpertExpertiseMC.isChecked():
            self.on_cmbFilterExpertExpertiseMC_currentIndexChanged(0)


    def initDocumentsTempInvalidTextEdit(self):
        tempInvalidReasonCache = CRBModelDataCache.getData('rbTempInvalidReason', True, '')
        vrbPersonCache = CRBModelDataCache.getData('vrbPerson', True, '')
        tempInvalidDocumentCache = CRBModelDataCache.getData('rbTempInvalidDocument', True, '')
        tempInvalidBreakCache = CRBModelDataCache.getData('rbTempInvalidBreak', True, '')
        tempInvalidResultCache = CRBModelDataCache.getData('rbTempInvalidResult', True, '')
        tempInvalidRegimeCache = CRBModelDataCache.getData('rbTempInvalidRegime', True, '')
        self.edtExpertDocumentsTempInvalid.setDataCache(tempInvalidReasonCache, vrbPersonCache, tempInvalidDocumentCache, tempInvalidBreakCache, tempInvalidResultCache, tempInvalidRegimeCache)
        self.edtExpertDisabilityDocumentsTempInvalid.setDataCache(tempInvalidReasonCache, vrbPersonCache, tempInvalidDocumentCache, tempInvalidBreakCache, tempInvalidResultCache, tempInvalidRegimeCache)
        self.edtExpertVitalRestrictionDocumentsTempInvalid.setDataCache(tempInvalidReasonCache, vrbPersonCache, tempInvalidDocumentCache, tempInvalidBreakCache, tempInvalidResultCache, tempInvalidRegimeCache)


    @pyqtSignature('')
    def on_actRelationsClient_triggered(self):
        clientId = self.currentClientId()
        if not clientId:
            clientId = self.selectedClientId()
        if clientId:
            self.editRelationsClient(clientId)
        self.focusClients()


    @pyqtSignature('')
    def on_actEventRelationsClient_triggered(self):
        clientId = self.currentClientId()
        if not clientId:
            clientId = self.selectedClientId()
        if clientId:
            self.editRelationsClient(clientId)
        self.focusClients()


    @pyqtSignature('')
    def on_actAmbCardRelationsClient_triggered(self):
        clientId = self.currentClientId()
        if not clientId:
            clientId = self.selectedClientId()
        if clientId:
            self.editRelationsClient(clientId)
        self.focusClients()


    @pyqtSignature('')
    def on_actActionsRelationsClient_triggered(self):
        clientId = self.currentClientId()
        if not clientId:
            clientId = self.selectedClientId()
        if clientId:
            self.editRelationsClient(clientId)
        self.focusClients()


    @pyqtSignature('')
    def on_actExpertRelationsClient_triggered(self):
        clientId = self.currentClientId()
        if not clientId:
            clientId = self.selectedClientId()
        if clientId:
            self.editRelationsClient(clientId)
        self.focusClients()


    @pyqtSignature('')
    def on_actVisitRelationsClient_triggered(self):
        clientId = self.currentClientId()
        if not clientId:
            clientId = self.selectedClientId()
        if clientId:
            self.editRelationsClient(clientId)
        self.focusClients()


    def editRelationsClient(self, clientId):
        dialog = CRelationsClientListDialog(self, clientId)
        try:
            if dialog.exec_():
                clientId = dialog.itemId()
        finally:
            dialog.destroy()
            sip.delete(dialog)
            del dialog


    def setSchedulesSort(self, col):
        name = self.modelSchedules.cols()[col].fieldName()
        self.subOrder = name
        header=self.tblSchedules.horizontalHeader()
        header.setSortIndicatorShown(True)
        self.isAscendingSchedules = not self.isAscendingSchedules
        header.setSortIndicator(col, Qt.AscendingOrder if self.isAscendingSchedules else Qt.DescendingOrder)
        self.modelSchedules.sortData(col, self.isAscendingSchedules)


    def setBySchedulesSort(self, col):
        name = self.modelVisitsBySchedules.cols()[col].fieldName()
        self.subOrder = name
        header=self.tblVisitsBySchedules.horizontalHeader()
        header.setSortIndicatorShown(True)
        self.isAscendingBySchedules = not self.isAscendingBySchedules
        header.setSortIndicator(col, Qt.AscendingOrder if self.isAscendingBySchedules else Qt.DescendingOrder)
        self.modelVisitsBySchedules.sortData(col, self.isAscendingBySchedules)


    def on_scannedReceived(self, data):
        if self.focusWidget() == QtGui.qApp.focusWidget():
            d = bytes(data)
            ok, policySerialAndNumber, lastName, firstName, patrName, sex, birthDate = parsePolicyBarCode(self, d)
            if ok:
                self.tabMain.setCurrentIndex(0)
                self.on_buttonBoxClient_reset()
                self.chkFilterLastName.setChecked(True)
                self.edtFilterLastName.setText(lastName)
                self.chkFilterFirstName.setChecked(True)
                self.edtFilterFirstName.setText(firstName)
                self.chkFilterPatrName.setChecked(True)
                self.edtFilterPatrName.setText(patrName)
                self.chkFilterBirthDay.setChecked(True)
                self.edtFilterBirthDay.setDate(birthDate)
                self.chkFilterSex.setChecked(True)
                self.cmbFilterSex.setCurrentIndex(sex)

                if policySerialAndNumber:
                    policyParts = policySerialAndNumber.split()
                    if len(policyParts) == 1:
                        policySerial = ''
                        policyNumber = policyParts[0]
                    else:
                        policySerial = policyParts[0]
                        policyNumber = policyParts[1]

                    self.chkFilterPolicyOnlyActual.setChecked(False)
                    self.edtFilterPolicyActualData.setDate(QDate())
                    self.edtFilterPolicySerial.setText(policySerial)
                    self.edtFilterPolicyNumber.setText(policyNumber)
                    self.cmbFilterPolicyInsurer.setValue(None)
                    self.cmbFilterPolicyType.setCode(u'ОМС') # ОМС
                    self.cmbFilterPolicyKind.setCode('1' if policySerial else '3')
                    self.chkFilterPolicy.setChecked(True)
                self.on_buttonBoxClient_apply()


    def onPolicySmartCardReceived(self, data):
        def normalizeDict(d):
            result = {}
            for key, val in d.iteritems():
                if isinstance(key, QString):
                    key = unicode(key)
                if isinstance(val, QString):
                    val = unicode(val)
                elif isinstance(val, dict):
                    val = normalizeDict(val)
                result[key] = val
            return result

        def decodeDate(d):
            if d and isinstance(d, (tuple, list)) and len(d)==3:
                return QDate(*d)
            return None

        def findInsurer(OGRN, OKATO):
            db = QtGui.qApp.db
            table = db.table('Organisation')
            record = db.getRecordEx(table,
                                    'id',
                                    [ table['deleted'].eq(0),
                                      table['isInsurer'].eq(1),
                                      table['OGRN'].eq(OGRN),
                                      table['OKATO'].eq(OKATO),
                                    ]
                                   )
            if record:
                return forceRef(record.value(0))
            return None

        if self.focusWidget() == QtGui.qApp.focusWidget():
            clientInformation = normalizeDict(data.toPyObject())
            lastName  = clientInformation.get('lastName')
            firstName = clientInformation.get('firstName')
            patrName  = clientInformation.get('patrName')
            sex       = clientInformation.get('sex')
            birthDate = decodeDate(clientInformation.get('birthDate'))
            policyNumber = clientInformation.get('policyNumber')
            insurerId = None
            smo = clientInformation.get('smo')
            if isinstance(smo, dict):
                OGRN  = smo.get('OGRN')
                OKATO = smo.get('OKATO')
                begDate = decodeDate(smo.get('begDate'))
                endDate = decodeDate(smo.get('endDate'))
                if OGRN and OKATO:
                    insurerId = findInsurer(OGRN, OKATO)
            else:
                begDate = decodeDate(clientInformation.get('issueDate'))
                endDate = decodeDate(clientInformation.get('expireDate'))

            self.tabMain.setCurrentIndex(0)
            self.on_buttonBoxClient_reset()
            if lastName:
                self.chkFilterLastName.setChecked(True)
                self.edtFilterLastName.setText(lastName)
            if firstName:
                self.chkFilterFirstName.setChecked(True)
                self.edtFilterFirstName.setText(firstName)
            if patrName:
                self.chkFilterPatrName.setChecked(True)
                self.edtFilterPatrName.setText(patrName)
            if birthDate:
                self.chkFilterBirthDay.setChecked(True)
                self.edtFilterBirthDay.setDate(birthDate)
            if sex:
                self.chkFilterSex.setChecked(True)
                self.cmbFilterSex.setCurrentIndex(sex)

            if policyNumber:
                self.chkFilterPolicyOnlyActual.setChecked(False)
                self.edtFilterPolicyActualData.setDate(QDate())
                self.edtFilterPolicySerial.setText('')
                self.edtFilterPolicyNumber.setText(policyNumber)
                #self.cmbFilterPolicyInsurer.setValue(insurerId)
                self.cmbFilterPolicyInsurer.setValue(None)
                self.cmbFilterPolicyType.setCode(u'ОМС') # ОМС
                self.cmbFilterPolicyKind.setCode('3')
                self.chkFilterPolicy.setChecked(True)
            self.extendedPolicyInformation = { 'number'    : policyNumber,
                                               'insurerId' : insurerId,
                                               'begDate'   : begDate,
                                               'endDate'   : endDate,
                                             }
            self.on_buttonBoxClient_apply()


    def on_actScanBarcodeTabExpert(self):
        self.cmbFilterExpert.setCurrentIndex(2)
        chkWidgetList = [
#                        self.chkFilterExpertDocType,
                        self.chkPrimaryOrDuble,
                        self.chkFilterExpertSerial,
#                        self.chkFilterExpertNumber,
                        self.chkFilterExpertReason,
                        self.chkFilterExpertBegDate,
                        self.chkFilterExpertEndDate,
                        self.chkFilterExpertOrgStruct,
                        self.chkFilterExpertSpeciality,
                        self.chkFilterExpertPerson,
                        self.chkFilterExpertMKB,
                        self.chkFilterExpertClosed,
                        self.chkFilterExpertDuration,
                        self.chkFilterExpertInsuranceOfficeMark,
                        self.chkFilterExpertLinked,
                        self.chkFilterDirectDateOnMC,
                        self.chkFilterExpertExternal,
                        self.chkDateKAK,
                        self.chkExpertIdKAK,
                        self.chkFilterExpertCreatePerson,
                        self.chkFilterExpertCreateDate,
                        self.chkFilterExpertModifyPerson,
                        self.chkFilterExpertModifyDate,
                        ]
        for chkWidget in chkWidgetList:
            chkWidget.setChecked(False)
        self.chkFilterExpertNumber.setChecked(True)


    def setUserRights(self):
        u"""Права доступа по вкладкам"""
        app = QtGui.qApp
        isAdmin = app.userHasRight(urAdmin)
        # чтение
        self.tabRegistry.setEnabled(isAdmin or app.userHasRight(urRegTabReadRegistry) or app.userHasRight(urRegTabWriteRegistry))
        self.tabEvents.setEnabled(isAdmin or app.userHasRight(urRegTabReadEvents))
        self.tabAmbCard.setEnabled(isAdmin or app.userHasRight(urRegTabReadAmbCard))
        self.tabActions.setEnabled(isAdmin or app.userHasRight(urRegTabReadActions))
        self.tabExpert.setEnabled(isAdmin or app.userHasRight(urRegTabReadExpert))
        self.tabVisits.setEnabled(isAdmin or app.userHasRight(urRegTabReadVisits))
        # Запись\изменение
        # Вкладка Картотека: кнопки Редактировать и Регистрация
        self.btnEdit.setEnabled(isAdmin or app.userHasRight(urRegTabWriteRegistry))
        self.btnNew.setEnabled(isAdmin or app.userHasRight(urRegTabWriteRegistry))
        # Вкладка Обращение: кнопки Редактировать и Новый
        self.btnEventEdit.setEnabled(isAdmin or app.userHasRight(urRegTabWriteEvents))
        self.btnEventNew.setEnabled(isAdmin or app.userHasRight(urRegTabWriteEvents))
        # Вкладка Мед. Карта
        #   зарезервировано право urRegTabWriteAmbCard
        # Вкладка Обслуживание: кнопка Редактировать событие
        self.btnActionEventEdit.setEnabled(isAdmin or app.userHasRight(urRegTabWriteEvents))
        # Вкладка Обслуживание: элемент контекстного меню
        self.actEditActionEvent.setEnabled(isAdmin or app.userHasRight(urRegTabWriteActions))
        # Вкладка Обслуживание: кнопка Редактировать
        self.btnActionEdit.setEnabled(isAdmin or app.userHasRight(urRegTabWriteActions))
        # Вкладка КЭР
        #   зарезервировано право urRegTabWriteExpert
        self.setBtnExpertEditEnabled()
        self.actExpertTempInvalidDelete.setEnabled(getRightEditTempInvalid(self.tblExpertTempInvalid.currentItemId()))
        self.actExpertDisabilityDelete.setEnabled(getRightEditTempInvalid(self.tblExpertDisability.currentItemId()))
        self.actExpertVitalRestrictionDelete.setEnabled(getRightEditTempInvalid(self.tblExpertVitalRestriction.currentItemId()))
        self.tabWidgetTempInvalidTypes.setTabEnabled(self.tabWidgetTempInvalidTypes.indexOf(self.tabExpertMedicalCommission), QtGui.qApp.userHasRight(urRegTabEditExpertMC))
        # Вкладка Визиты: кнопка Редактировать
        self.btnVisitEdit.setEnabled(isAdmin or app.userHasRight(urRegTabWriteVisits))
        self.actEditVisitEvent.setEnabled(isAdmin or app.userHasRight(urRegTabWriteVisits))
        # Вкладка  СМП
        #   зарезервировано право urRegTabWriteAmbulance
        # заполнение combobox-ов тип прикрепления
        self.setupCmbFilterAttachTypeCategory(0)
        # Оповещения
        self.actNotify.setEnabled(isAdmin or app.userHasRight(urSendInternalNotifications))
        # перенести в exec_
        self.updateClientsList(self.__filter)


    def setBtnExpertEditEnabled(self):
        tableList = [self.tblExpertTempInvalid, self.tblExpertDisability, self.tblExpertVitalRestriction, self.getCurrentExpertMedicalCommissionTable()]
        self.btnExpertEdit.setEnabled(getRightEditTempInvalid(tableList[self.tabWidgetTempInvalidTypes.currentIndex()].currentItemId()))


    def setupCmbFilterAttachTypeCategory(self, category,
                                         filterDict = { 1: 'temporary=0',
                                                        2: 'temporary',
                                                        3: 'outcome' }
                                        ):
        v = self.cmbFilterAttachType.value()
        self.cmbFilterAttachType.setFilter(filterDict.get(category, ''))
        if v:
            self.cmbFilterAttachType.setValue(v)


    def closeEvent(self, event):
        self.saveDialogPreferences()
        QtGui.qApp.preferences.appPrefs['FilterAccountingSystem'] = toVariant(self.cmbFilterAccountingSystem.value())
        QtGui.qApp.preferences.appPrefs['Registry_chkFilterEventContract'] = toVariant(self.chkFilterEventContract.isChecked())
        QtGui.qApp.preferences.appPrefs['Registry_filterEventContractId'] = toVariant(self._CRegistryWindow__filterEventContractId)
        QtGui.qApp.preferences.appPrefs['Registry_edtFilterEventContract'] = toVariant(self.edtFilterEventContract.text())
        QtGui.qApp.preferences.appPrefs['Registry_filterEventContractParams_financeId'] = toVariant(self.filterEventContractParams.get('financeId', None))
        QtGui.qApp.preferences.appPrefs['Registry_filterEventContractParams_groupingIndex'] = toVariant(self.filterEventContractParams.get('groupingIndex', 0))
        QtGui.qApp.preferences.appPrefs['Registry_filterEventContractParams_grouping'] = toVariant(self.filterEventContractParams.get('grouping', u''))
        QtGui.qApp.preferences.appPrefs['Registry_filterEventContractParams_resolutionIndex'] = toVariant(self.filterEventContractParams.get('resolutionIndex', 0))
        QtGui.qApp.preferences.appPrefs['Registry_filterEventContractParams_resolution'] = toVariant(self.filterEventContractParams.get('resolution', u''))
        QtGui.qApp.preferences.appPrefs['Registry_filterEventContractParams_priceList'] = toVariant(self.filterEventContractParams.get('priceList', 0))
        currentDate = QDate.currentDate()
        QtGui.qApp.preferences.appPrefs['Registry_filterEventContractParams_edtBegDate'] = toVariant(self.filterEventContractParams.get('edtBegDate', QDate(currentDate.year(), 1, 1)))
        QtGui.qApp.preferences.appPrefs['Registry_filterEventContractParams_edtEndDate'] = toVariant(self.filterEventContractParams.get('edtEndDate', QDate(currentDate.year(), 12, 31)))
        QtGui.qApp.preferences.appPrefs['Registry_filterEventContractParams_enableInAccounts'] = toVariant(self.filterEventContractParams.get('enableInAccounts', 0))
        QtGui.QScrollArea.closeEvent(self, event)
        QtGui.qApp.mainWindow.registry = None
        if forceBool(QtGui.qApp.preferences.appPrefs.get('SocCard_SupportEnabled', False)):
            self.cardReader.finalize()


    def syncSplitters(self, nextSplitter):
        if nextSplitter != self.controlSplitter:
            nextSplitter.setSizes(self.controlSplitter.sizes())
            self.controlSplitter = nextSplitter


    def focusClients(self):
        self.tblClients.setFocus(Qt.TabFocusReason)


    def focusEvents(self):
        self.tblEvents.setFocus(Qt.TabFocusReason)


    def showAccountingDialog(self, eventId=None, actionId=None, visitId=None):
        QtGui.qApp.setWaitCursor()
        dlg = CAccountingDialog(self)
        dlg.setWatchingFields(eventId, actionId, visitId)
        QtGui.qApp.restoreOverrideCursor()
        dlg.exec_()


    @pyqtSignature('int')
    def on_cmbFilterContingentType_currentIndexChanged(self, index):
        contingentTypeId = self.cmbFilterContingentType.value()
        if contingentTypeId:
            eventRecordList = CContingentTypeTranslator.getCondRecordList(CContingentTypeTranslator.CTETTName, contingentTypeId)
            actionRecordList = CContingentTypeTranslator.getCondRecordList(CContingentTypeTranslator.CTATTName, contingentTypeId)
            self.cmbFilterContingentEventTypeStatus.setEnabled(bool(eventRecordList))
            self.cmbFilterContingentActionType.setEnabled(bool(actionRecordList))


    def getLeavedHospitalBeds(self, orgStructureIdList):
        currentDate = QDate.currentDate()
        db = QtGui.qApp.db
        tableAPHB = db.table('ActionProperty_HospitalBed')
        tableAPT = db.table('ActionPropertyType')
        tableAP = db.table('ActionProperty')
        tableActionType = db.table('ActionType')
        tableAction = db.table('Action')
        tableEvent = db.table('Event')
        tableOSHB = db.table('OrgStructure_HospitalBed')
        tableOS = db.table('OrgStructure')
        tableClient = db.table('Client')
        orgStructureBedsIdList = db.getDistinctIdList(tableOSHB, [tableOSHB['master_id']], [tableOSHB['master_id'].inlist(orgStructureIdList)])
        clientIdList = []
        if orgStructureBedsIdList:
            queryTable = tableActionType.innerJoin(tableAction, tableActionType['id'].eq(tableAction['actionType_id']))
            queryTable = queryTable.innerJoin(tableEvent, tableAction['event_id'].eq(tableEvent['id']))
            queryTable = queryTable.innerJoin(tableClient, tableEvent['client_id'].eq(tableClient['id']))
            cond = [ tableActionType['flatCode'].like('leaved%'),
                     tableAction['deleted'].eq(0),
                     tableEvent['deleted'].eq(0),
                     tableActionType['deleted'].eq(0),
                     tableClient['deleted'].eq(0),
                     tableEvent['setDate'].le(currentDate),
                     tableAction['begDate'].le(currentDate),
                     tableAction['endDate'].isNull()
                   ]
            stmt = db.selectStmt(queryTable, [tableEvent['id'].alias('eventId'), tableAction['id'].alias('actionId')], cond)
            query = db.query(stmt)
            queryTable = queryTable.innerJoin(tableAPT, tableAPT['actionType_id'].eq(tableActionType['id']))
            queryTable = queryTable.innerJoin(tableAP, tableAP['type_id'].eq(tableAPT['id']))
            queryTable = queryTable.innerJoin(tableAPHB, tableAPHB['id'].eq(tableAP['id']))
            queryTable = queryTable.innerJoin(tableOSHB, tableOSHB['id'].eq(tableAPHB['value']))
            queryTable = queryTable.innerJoin(tableOS, tableOS['id'].eq(tableOSHB['master_id']))
            while query.next():
                record = query.record()
                eventId = forceRef(record.value('eventId'))
                cond = [ tableActionType['flatCode'].like('moving%'),
                         tableAction['deleted'].eq(0),
                         tableEvent['deleted'].eq(0),
                         tableEvent['id'].eq(eventId),
                         tableAP['deleted'].eq(0),
                         tableActionType['deleted'].eq(0),
                         tableAPT['deleted'].eq(0),
                         tableOS['deleted'].eq(0),
                         tableClient['deleted'].eq(0),
                         tableEvent['setDate'].le(currentDate),
                         tableAction['begDate'].le(currentDate),
                         tableAPT['typeName'].like('HospitalBed'),
                         tableAP['action_id'].eq(tableAction['id'])
                       ]
                order = u'Action.begDate DESC'
                cols = [tableClient['id']]
                strListId = ' IN ('+(','.join(str(orgId) for orgId in orgStructureBedsIdList))+')'
                cols.append(u'IF(OrgStructure.id%s, 1, 0) AS boolOrgStructure'%(strListId))
                firstRecord = db.getRecordEx(queryTable, cols, cond, order)
                if firstRecord:
                    if forceBool(firstRecord.value('boolOrgStructure')):
                        clientIdList.append(forceInt(firstRecord.value('id')))
        return clientIdList


    def getHospitalBeds(self, orgStructureIdList, indexFlatCode = 0):
        flatCode = [u'', u'moving', u'leaved', u'planning']
        currentDate = QDate.currentDate()
        db = QtGui.qApp.db
        tableAPHB = db.table('ActionProperty_HospitalBed')
        tableAPT = db.table('ActionPropertyType')
        tableAP = db.table('ActionProperty')
        tableActionType = db.table('ActionType')
        tableAction = db.table('Action')
        tableEvent = db.table('Event')
        tableOSHB = db.table('OrgStructure_HospitalBed')
        tableOS = db.table('OrgStructure')
        tableClient = db.table('Client')
        orgStructureBedsIdList = db.getDistinctIdList(tableOSHB, [tableOSHB['master_id']], [tableOSHB['master_id'].inlist(orgStructureIdList)])
        if orgStructureBedsIdList:
            queryTable = tableActionType.innerJoin(tableAction, tableActionType['id'].eq(tableAction['actionType_id']))
            queryTable = queryTable.innerJoin(tableEvent, tableAction['event_id'].eq(tableEvent['id']))
            queryTable = queryTable.innerJoin(tableClient, tableEvent['client_id'].eq(tableClient['id']))
            queryTable = queryTable.innerJoin(tableAPT, tableAPT['actionType_id'].eq(tableActionType['id']))
            queryTable = queryTable.innerJoin(tableAP, tableAP['type_id'].eq(tableAPT['id']))
            queryTable = queryTable.innerJoin(tableAPHB, tableAPHB['id'].eq(tableAP['id']))
            queryTable = queryTable.innerJoin(tableOSHB, tableOSHB['id'].eq(tableAPHB['value']))
            queryTable = queryTable.innerJoin(tableOS, tableOS['id'].eq(tableOSHB['master_id']))
            cond = [ tableClient['deleted'].eq(0),
                     tableEvent['deleted'].eq(0),
                     tableAction['deleted'].eq(0),
                     tableAP['deleted'].eq(0),
                     tableActionType['deleted'].eq(0),
                     tableOS['deleted'].eq(0),
                     tableOS['id'].inlist(orgStructureBedsIdList),
                     tableEvent['setDate'].le(currentDate),
                     tableAction['begDate'].le(currentDate),
                     tableAPT['typeName'].like('HospitalBed'),
                     tableAP['action_id'].eq(tableAction['id'])
                   ]
            cond.append(db.joinOr([tableEvent['execDate'].isNull(), tableEvent['execDate'].ge(currentDate)]))
            cond.append(db.joinOr([tableAction['endDate'].isNull(), tableAction['endDate'].ge(currentDate)]))
            if indexFlatCode == 1 or indexFlatCode == 3:
                cond.append(tableActionType['flatCode'].like(flatCode[indexFlatCode]))
            else:
                cond.append(db.joinOr([tableActionType['flatCode'].like('moving'), tableActionType['flatCode'].like('planning')]))
            return db.getDistinctIdList(queryTable, [tableClient['id']], cond)
        else:
            return None


    def updateClientsList(self, filter, posToId=None):
        """
            в соответствии с фильтром обновляет список пациентов.
        """
        db = QtGui.qApp.db

        isEventJoined = False
        isSocStatusJoined = False


        def calcBirthDate(cnt, unit):
            result = QDate.currentDate()
            if unit == 3: # дни
                result = result.addDays(-cnt)
            elif unit == 2: # недели
                result = result.addDays(-cnt*7)
            elif unit == 1: # месяцы
                result = result.addMonths(-cnt)
            else: # года
                result = result.addYears(-cnt)
            return result

        def addAddressCond(cond, addrType, addrIdList):
            tableClientAddress = db.table('ClientAddress')
            subcond = [tableClientAddress['client_id'].eqEx('`Client`.`id`'),
                       tableClientAddress['id'].eqEx(u'getClientLocAddressId(Client.id)' if addrType else u'getClientRegAddressId(Client.id)')
                      ]
            if addrIdList is None:
                subcond.append(tableClientAddress['address_id'].isNull())
                subcondNoCAId = u'(SELECT %s IS NULL)'%(u'getClientLocAddressId(Client.id)' if addrType else u'getClientRegAddressId(Client.id)')
            else:
                subcond.append(tableClientAddress['address_id'].inlist(addrIdList))
            if addrIdList is None:
                cond.append(db.joinOr([db.existsStmt(tableClientAddress, subcond), subcondNoCAId]))
            else:
                cond.append(db.existsStmt(tableClientAddress, subcond))

        def addExcludeLeavedCond(cond):
            db = QtGui.qApp.db
            tableCA = db.table('ClientAttach').alias('CA')
            tableCA2 = db.table('ClientAttach').alias('CA2')
            tableRBAT = db.table('rbAttachType').alias('rbAT')
            table = tableCA.leftJoin(tableRBAT, tableRBAT['id'].eq(tableCA['attachType_id']))

            localCond = [tableCA['client_id'].eqEx('Client.id'),
                        tableRBAT['outcome'].eq(1),
                        tableCA['deleted'].eq(0),
                        tableCA['id'].eqEx('(%s)'%db.selectStmt(tableCA2,
                                                                        'MAX(%s)'%tableCA2['id'].name(),
                                                                        [tableCA2['client_id'].eqEx('Client.id')] ))]

            cond.append(db.notExistsStmt(table, localCond))

        def addAttachCond(cond, orgCond, attachCategory, attachTypeId, attachBegDate=QDate(), attachEndDate=QDate(), isStatement = 0):
            outerCond = ['ClientAttach.client_id = Client.id']
            innerCond = ['CA2.client_id = Client.id']
            if isStatement:
                outerCond.append('ClientAttach.isStatement = %d'%(isStatement-1))
            if attachBegDate and attachEndDate:
                outerCond.append(db.joinOr([db.joinAnd(['DATE(ClientAttach.begDate) IS NULL',
                                                        'DATE(ClientAttach.endDate) IS NULL'
                                                        ]),

                                            db.joinAnd(['DATE(ClientAttach.begDate) IS NULL',
                                                        'DATE(ClientAttach.endDate) >= DATE(%s)'%(db.formatDate(attachBegDate))
                                                        ]),

                                            db.joinAnd(['DATE(ClientAttach.endDate) IS NULL',
                                                        'DATE(ClientAttach.begDate) < DATE(%s)'%(db.formatDate(attachEndDate))
                                                        ]),

                                            db.joinAnd(['DATE(ClientAttach.begDate) IS NOT NULL',
                                                        'DATE(ClientAttach.begDate) < DATE(%s)'%(db.formatDate(attachEndDate)),
                                            db.joinAnd(['DATE(ClientAttach.endDate) IS NOT NULL',
                                                        'DATE(ClientAttach.endDate) >= DATE(%s)'%(db.formatDate(attachBegDate))
                                                        ])
                                                        ])
                                            ])
                                )
            elif attachBegDate:
                outerCond.append(db.joinOr([db.joinAnd(['DATE(ClientAttach.begDate) IS NULL',
                                                        'DATE(ClientAttach.endDate) IS NULL'
                                                        ]),
                                            db.joinAnd(['DATE(ClientAttach.endDate) IS NOT NULL',
                                                        'DATE(ClientAttach.endDate) >= DATE(%s)'%(db.formatDate(attachBegDate))
                                                        ])
                                            ])
                                )
            elif attachEndDate:
                outerCond.append(db.joinOr([db.joinAnd(['DATE(ClientAttach.begDate) IS NULL',
                                                        'DATE(ClientAttach.endDate) IS NULL'
                                                        ]),
                                            db.joinAnd(['DATE(ClientAttach.begDate) IS NOT NULL',
                                                        'DATE(ClientAttach.begDate) < DATE(%s)'%(db.formatDate(attachEndDate))
                                                        ])
                                            ])
                                )
                outerCond.append('DATE(ClientAttach.begDate) >= DATE(%s)'%(db.formatDate(attachBegDate)))
                outerCond.append('DATE(ClientAttach.begDate) < DATE(%s)'%(db.formatDate(attachEndDate)))
            if orgCond:
                outerCond.append(orgCond)
            if attachTypeId:
                outerCond.append('attachType_id=%d' % attachTypeId)
                innerCond.append('CA2.attachType_id=%d' % attachTypeId)
            else:
                if attachCategory == 1:
                    innerCond.append('rbAttachType2.temporary=0')
                elif attachCategory == 2:
                    innerCond.append('rbAttachType2.temporary')
                elif attachCategory == 3:
                    innerCond.append('rbAttachType2.outcome')
                elif attachCategory == 0:
                    outerCond.append('rbAttachType.outcome=0')
                    innerCond.append('rbAttachType2.temporary=0')
            stmt = '''EXISTS (SELECT ClientAttach.id
               FROM ClientAttach
               LEFT JOIN rbAttachType ON rbAttachType.id = ClientAttach.attachType_id
               WHERE ClientAttach.deleted=0
               AND %s
               AND ClientAttach.id = (SELECT MAX(CA2.id)
                           FROM ClientAttach AS CA2
                           LEFT JOIN rbAttachType AS rbAttachType2 ON rbAttachType2.id = CA2.attachType_id
                           WHERE CA2.deleted=0 AND %s))'''
            cond.append(stmt % (db.joinAnd(outerCond), db.joinAnd(innerCond)))
        try:
            QtGui.QApplication.setOverrideCursor(QtGui.QCursor(Qt.WaitCursor))
            self.__filter = filter
            tableClient = db.table('Client')
            table = tableClient
            cond = [tableClient['deleted'].eq(0)]
            if 'id' in filter:
                accountingSystemId = filter.get('accountingSystemId',  None)
                if accountingSystemId:
                    tableIdentification = db.table('ClientIdentification')
                    table = table.leftJoin(tableIdentification, tableIdentification['client_id'].eq(tableClient['id']))
                    cond.append(tableIdentification['accountingSystem_id'].eq(accountingSystemId))
                    cond.append(tableIdentification['identifier'].eq(filter['id']))
                    cond.append(tableIdentification['deleted'].eq(0))
                else:
                    cond.append(tableClient['id'].eq(filter['id']))
            addCondLike(cond, tableClient['lastName'],  addDots(filter.get('lastName', '')))
            addCondLike(cond, tableClient['firstName'], addDots(filter.get('firstName', '')))
            addCondLike(cond, tableClient['patrName'],  addDots(filter.get('patrName', '')))
            if 'birthDate' in filter:
                birthDate = filter['birthDate']
                if not birthDate:
                    birthDate = None
                if 'endBirthDate' in filter:
                    endBirthDate = filter['endBirthDate']
                    if not endBirthDate:
                        endBirthDate = None
                    cond.append(tableClient['birthDate'].dateGe(birthDate))
                    cond.append(tableClient['birthDate'].dateLe(endBirthDate))
                else:
                    cond.append(tableClient['birthDate'].eq(birthDate))
            if 'dead' in filter:
                cond.append(tableClient['deathDate'].isNotNull())
                if 'begDeathDate' in filter:
                    begDeathDate = filter['begDeathDate']
                    if begDeathDate:
                        cond.append(tableClient['deathDate'].dateGe(begDeathDate))
                if 'endDeathDate' in filter:
                    endDeathDate = filter['endDeathDate']
                    if endDeathDate:
                        cond.append(tableClient['deathDate'].dateLe(endDeathDate))
            self.addEqCond(cond, tableClient, 'createPerson_id', filter, 'createPersonIdEx')
            self.addDateCond(cond, tableClient, 'createDatetime', filter, 'begCreateDateEx', 'endCreateDateEx')
            self.addEqCond(cond, tableClient, 'modifyPerson_id', filter, 'modifyPersonIdEx')
            self.addDateCond(cond, tableClient, 'modifyDatetime', filter, 'begModifyDateEx', 'endModifyDateEx')
            event = filter.get('event', None)
            if event:
                isEventJoined = True
                tableEvent = db.table('Event')
                tableEventType = db.table('EventType')
                tableEventTypePurpose = db.table('rbEventTypePurpose')
                table = table.innerJoin(tableEvent, tableEvent['client_id'].eq(tableClient['id']))
                table = table.innerJoin(tableEventType, tableEventType['id'].eq(tableEvent['eventType_id']))
                table = table.innerJoin(tableEventTypePurpose, tableEventTypePurpose['id'].eq(tableEventType['purpose_id']))
                cond.append(tableEventTypePurpose['code'].notlike('0'))
                firstEvent, begDate, endDate = event
                if begDate or endDate:
                    cond.append(tableEvent['setDate'].isNotNull())
                    cond.append(tableEvent['deleted'].eq(0))
                if begDate:
                    cond.append(tableEvent['setDate'].dateGe(begDate))
                if endDate:
                    cond.append(tableEvent['setDate'].dateLe(endDate))
                if firstEvent and begDate:
                    cond.append('''NOT EXISTS(SELECT E.id FROM Event AS E WHERE E.client_id = Client.id AND Event.deleted = 0 AND DATE(E.setDate) < DATE(%s))'''%(db.formatDate(begDate)))
            if 'age' in filter:
                lowAge, highAge = filter['age']
                cond.append(tableClient['birthDate'].le(calcBirthDate(*lowAge)))
                cond.append(tableClient['birthDate'].ge(calcBirthDate(*highAge)))
            if 'sex' in filter:
                cond.append(tableClient['sex'].eq(filter['sex']))
            if 'SNILS' in filter:
                cond.append(tableClient['SNILS'].eq(filter['SNILS']))
            if 'contact' in filter:
                tableContact = db.table('ClientContact')
                table = table.leftJoin(tableContact, tableContact['client_id'].eq(tableClient['id']))
                cond.append(tableContact['deleted'].eq(0))
                rep1 = replaceMask('ClientContact.contact', '\'(\'', '\'\'')
                rep2 = replaceMask(rep1, '\')\'', '\'\'')
                rep3 = replaceMask(rep2, '\'+\'', '\'\'')
                rep4 = replaceMask(rep3, '\'-\'', '\'\'')
                rep5 = replaceMask(rep4, '\' \'', '\'\'')
                rep  = replaceMask(rep5, '\'.\'', '\'\'')
                cond.append(rep + ' LIKE \'%s\''%(undotLikeMask(contactToLikeMask(filter['contact']))))
                #cond.append(tableContact['contact'].like(contactToLikeMask(filter['contact'])))
            doc = filter.get('doc', None)
            if doc:
                tableDoc = db.table('ClientDocument')
                table = table.leftJoin(tableDoc,
                                       [tableDoc['client_id'].eq(tableClient['id']), tableDoc['deleted'].eq(0)]
                                      )
                typeId, serial, number = doc
                if typeId or serial or number:
                    if typeId:
                        cond.append(tableDoc['documentType_id'].eq(typeId))
                    if serial:
                        cond.append(tableDoc['serial'].eq(serial))
                    if number:
                        cond.append(tableDoc['number'].eq(number))
                else:
                    cond.append(cond.append(tableDoc['id'].isNull()))
            regionSMO = filter.get('regionSMO', None)
            policy = filter.get('policy', None)
            if policy or regionSMO:
                tablePolicy = db.table('ClientPolicy')
                tableOrganisation = db.table('Organisation')
                table = table.innerJoin(tablePolicy,
                                       [tablePolicy['client_id'].eq(tableClient['id']), tablePolicy['deleted'].eq(0)]
                                      )
            if policy:
                onlyPolicyActual, policyActualData, policyType, policyKind, insurerId, serial, number = policy
                if onlyPolicyActual or policyType or policyKind or insurerId or serial or number:
                    if onlyPolicyActual:
                        if policyActualData:
                            cond.append(tablePolicy['begDate'].dateLe(policyActualData))
                            cond.append(
                                        db.joinOr(
                                                  [
                                                   tablePolicy['endDate'].dateGe(policyActualData),
                                                   tablePolicy['endDate'].isNull()
                                                  ]
                                                 )
                                       )
                        stmtPolicyTypeId = u''
                        stmtPolicyKindId = u''
                        if policyType:
                            stmtPolicyTypeId = u'AND CP.policyType_id = ClientPolicy.policyType_id'
                        if policyKind:
                            stmtPolicyKindId = u'AND CP.policyKind_id = ClientPolicy.policyKind_id'
                        cond.append('''ClientPolicy.id = (SELECT MAX(CP.id) FROM ClientPolicy AS CP
                               WHERE CP.deleted = 0
                               AND   CP.insurer_id = ClientPolicy.insurer_id
                               AND   CP.client_id = Client.id
                               %s
                               %s
                              )'''%(stmtPolicyTypeId, stmtPolicyKindId))
                    if policyType:
                        if policyType == -1:
                            cmbPolicyTypeModel = self.cmbFilterPolicyType.model()
                            if cmbPolicyTypeModel.d:
                                # codes 1, 2 means that policy is compulsory
                                policyTypeIdList = [cmbPolicyTypeModel.getId(cmbPolicyTypeModel.d.getIndexByCode(code)) for code in ('1', '2')]
                                cond.append(tablePolicy['policyType_id'].inlist(policyTypeIdList))
                        else:
                            cond.append(tablePolicy['policyType_id'].eq(policyType))
                    if policyKind:
                        cond.append(tablePolicy['policyKind_id'].eq(policyKind))
                    if insurerId:
                        cond.append(tablePolicy['insurer_id'].eq(insurerId))
                        table = table.innerJoin(tableOrganisation,
                                               [tableOrganisation['id'].eq(tablePolicy['insurer_id']), tableOrganisation['deleted'].eq(0)]
                                              )
                    elif not regionSMO:
                        table = table.leftJoin(tableOrganisation,
                                               [tableOrganisation['id'].eq(tablePolicy['insurer_id']), tableOrganisation['deleted'].eq(0)]
                                              )
                    if serial:
                        cond.append(tablePolicy['serial'].eq(serial))
                    if number:
                        cond.append(tablePolicy['number'].eq(number))
                else:
                    cond.append(tablePolicy['id'].isNull())
            if regionSMO:
                regionTypeSMO, regionSMOCode = regionSMO
                table = table.innerJoin(tableOrganisation,
                                               [tableOrganisation['id'].eq(tablePolicy['insurer_id']), tableOrganisation['deleted'].eq(0)]
                                              )
                if regionSMOCode:
#                    regionCode = unicode(QString(regionSMOCode).left(2)) + u'%'
                    if regionTypeSMO:
                        cond.append(tableOrganisation['area'].notlike(regionSMOCode))
                    else:
                        cond.append(tableOrganisation['area'].like(regionSMOCode))
            orgId = filter.get('orgId', None)
            if orgId:
                tableWork = db.table('ClientWork')
                joinCond = [ tableClient['id'].eq(tableWork['client_id']),
                             'ClientWork.id = (select max(CW.id) from ClientWork as CW where CW.client_id=Client.id)'
                           ]
                table = table.join(tableWork, joinCond)
                cond.append(tableWork['org_id'].eq(orgId))
            if filter.get('socStatuses'):
                isSocStatusJoined = True
                socStatusesCondition = filter.get('socStatusesCondition', None)
                socStatusesClass = filter.get('socStatusesClass', None)
                socStatusesType = filter.get('socStatusesType', None)
                socStatusesBegDate = filter.get('socStatusesBegDate', QDate())
                socStatusesEndDate = filter.get('socStatusesEndDate', QDate())
                tableClientSocStatus = db.table('ClientSocStatus')
                clientFilterTable = db.table('Client')
                socStatusFilterTable = db.table('ClientSocStatus')
                if socStatusesCondition and not socStatusesEndDate and not socStatusesBegDate and not socStatusesType and not socStatusesClass:
                    cond.append(tableClientSocStatus['id'].isNull())
                if socStatusesClass:
                    socStatusClassIdList = db.getDescendants('rbSocStatusClass', 'group_id', socStatusesClass)
                    if socStatusClassIdList:
                        stmtStatusTypeId = u'''SELECT DISTINCT rbSocStatusClassTypeAssoc.type_id
                FROM  rbSocStatusClassTypeAssoc
                WHERE rbSocStatusClassTypeAssoc.class_id IN (%s)'''%(u','.join(str(socStatusClassId) for socStatusClassId in socStatusClassIdList))
                        queryStatusTypeId = db.query(stmtStatusTypeId)
                        resultStatusTypeIdList = []
                        while queryStatusTypeId.next():
                            resultStatusTypeIdList.append(queryStatusTypeId.value(0).toInt()[0])
                            parentSocStatusClassIdList = db.getTheseAndParents('rbSocStatusClass', 'group_id', [socStatusesClass])
                        if socStatusesCondition:
                            clientFilterTable = clientFilterTable.leftJoin(socStatusFilterTable, clientFilterTable['id'].eq(socStatusFilterTable['client_id']))
                            if resultStatusTypeIdList:
                                clientFilterCond = [db.joinOr([socStatusFilterTable['socStatusType_id'].inlist(resultStatusTypeIdList), tableClientSocStatus['id'].isNull()])]
                                if socStatusesBegDate:
                                    clientFilterCond.append(db.joinOr([ socStatusFilterTable['endDate'].dateGe(socStatusesBegDate),
                                                            socStatusFilterTable['endDate'].isNull()
                                                          ]
                                                         )
                                               )
                                if socStatusesEndDate:
                                    clientFilterCond.append(db.joinOr([ socStatusFilterTable['begDate'].dateLe(socStatusesEndDate),
                                                            socStatusFilterTable['begDate'].isNull()
                                                          ]
                                                         )
                                               )
                                clientSocStatusesFilterIdList = db.getIdList(clientFilterTable, clientFilterTable['id'], clientFilterCond,  ['id'])
                                cond.append(tableClient['id'].notInlist(clientSocStatusesFilterIdList))
                            else:
                                clientFilterCond = [db.joinOr([db.joinAnd[(socStatusFilterTable['socStatusType_id'].inlist(parentSocStatusClassIdList), tableClientSocStatus['id'].isNull())], tableClientSocStatus['socStatusType_id'].isNull()])]
                        else:
                            if resultStatusTypeIdList:
                                cond.append(tableClientSocStatus['socStatusType_id'].inlist(resultStatusTypeIdList))
                            else:
                                cond.append(tableClientSocStatus['socStatusClass_id'].inlist(parentSocStatusClassIdList))
                                cond.append(tableClientSocStatus['socStatusType_id'].isNull())
                                cond.append(tableClientSocStatus['deleted'].eq(0))
                    else:
                        if socStatusesCondition:
                            clientFilterTable = clientFilterTable.leftJoin(socStatusFilterTable, clientFilterTable['id'].eq(socStatusFilterTable['client_id']))
                            clientFilterCond = [db.joinOr([socStatusFilterTable['socStatusClass_id'].eq(socStatusesClass), tableClientSocStatus['id'].isNull()])]
                            if socStatusesBegDate:
                                clientFilterCond.append(db.joinOr([ socStatusFilterTable['endDate'].dateGe(socStatusesBegDate),
                                                        socStatusFilterTable['endDate'].isNull()
                                                      ]
                                                     )
                                           )
                            if socStatusesEndDate:
                                clientFilterCond.append(db.joinOr([ socStatusFilterTable['begDate'].dateLe(socStatusesEndDate),
                                                        socStatusFilterTable['begDate'].isNull()
                                                      ]
                                                     )
                                           )
                            clientSocStatusesFilterIdList = db.getIdList(clientFilterTable, clientFilterTable['id'], clientFilterCond,  ['id'])
                            cond.append(tableClient['id'].notInlist(clientSocStatusesFilterIdList))
                        else:
                            cond.append(tableClientSocStatus['socStatusClass_id'].eq(socStatusesClass))
                            cond.append(tableClientSocStatus['deleted'].eq(0))
                if socStatusesType:
                    if socStatusesCondition:
                        if not socStatusesClass:
                            clientFilterTable = clientFilterTable.leftJoin(socStatusFilterTable, clientFilterTable['id'].eq(socStatusFilterTable['client_id']))
                        clientFilterCond = [db.joinOr([clientFilterTable['socStatusType_id'].eq(socStatusesType), clientFilterTable['id'].isNull()])]
                        if socStatusesBegDate:
                            clientFilterCond.append(db.joinOr([ socStatusFilterTable['endDate'].dateGe(socStatusesBegDate),
                                                    socStatusFilterTable['endDate'].isNull()
                                                  ]
                                                 )
                                       )
                        if socStatusesEndDate:
                            clientFilterCond.append(db.joinOr([ socStatusFilterTable['begDate'].dateLe(socStatusesEndDate),
                                                    socStatusFilterTable['begDate'].isNull()
                                                  ]
                                                 )
                                       )
                        clientSocStatusesFilterIdList = db.getIdList(clientFilterTable, clientFilterTable['id'], clientFilterCond,  ['id'])
                        cond.append(tableClient['id'].notInlist(clientSocStatusesFilterIdList))
                    else:
                        cond.append(tableClientSocStatus['socStatusType_id'].eq(socStatusesType))
                        cond.append(tableClientSocStatus['deleted'].eq(0))
                if not socStatusesCondition:
                    if socStatusesBegDate:
                        cond.append(db.joinOr([ tableClientSocStatus['endDate'].dateGe(socStatusesBegDate),
                                                tableClientSocStatus['endDate'].isNull()
                                              ]
                                             )
                                   )
                    if socStatusesEndDate:
                        cond.append(db.joinOr([ tableClientSocStatus['begDate'].dateLe(socStatusesEndDate),
                                                tableClientSocStatus['begDate'].isNull()
                                              ]
                                             )
                                   )
                table = table.leftJoin(tableClientSocStatus, tableClient['id'].eq(tableClientSocStatus['client_id']))
            documentTypeForTracking = filter.get('documentTypeForTracking', None)
            documentLocation = filter.get('documentLocation', None)
            personDocumentLocation = filter.get('personDocumentLocation', None)
            begDateFilterDocumentLocation = filter.get('begDateFilterDocumentLocation', QDate())
            endDateFilterDocumentLocation = filter.get('endDateFilterDocumentLocation', QDate())
            if documentTypeForTracking:
                tableDocumentTypeForTracking = db.table('Client_DocumentTracking')
                cond.append(tableDocumentTypeForTracking['deleted'].eq(0))
                table = table.leftJoin(tableDocumentTypeForTracking, tableClient['id'].eq(tableDocumentTypeForTracking['client_id']))
                cond.append(tableDocumentTypeForTracking['documentTypeForTracking_id'].eq(documentTypeForTracking))
            if documentLocation:
                tableDocumentLocation = db.table('Client_DocumentTrackingItem')
                if not documentTypeForTracking:
                    tableDocumentTypeForTracking = db.table('Client_DocumentTracking')
                    table = table.leftJoin(tableDocumentTypeForTracking, tableClient['id'].eq(tableDocumentTypeForTracking['client_id']))
                table = table.leftJoin(tableDocumentLocation, tableDocumentTypeForTracking['id'].eq(tableDocumentLocation['master_id']))
                if begDateFilterDocumentLocation==QDate.currentDate() and endDateFilterDocumentLocation==QDate.currentDate():
                    tableDocumentLocationLimit = db.table('Client_DocumentTrackingItem').alias('tableDocumentLocationLimit')
                    cond.append(tableDocumentLocation['documentLocation_id'].eq(documentLocation))
                    cond.append(tableDocumentLocation['id'].eqEx('(%s)'%db.selectStmt(tableDocumentLocationLimit, tableDocumentLocationLimit['id'], tableDocumentLocationLimit['master_id'].eq(tableDocumentLocation['master_id']), 'documentLocationDate desc limit 1')))
                else:
                    cond.append(tableDocumentLocation['documentLocation_id'].eq(documentLocation))
                if begDateFilterDocumentLocation and begDateFilterDocumentLocation!=QDate.currentDate():
                    cond.append(tableDocumentLocation['documentLocationDate'].ge(begDateFilterDocumentLocation))
                if endDateFilterDocumentLocation and endDateFilterDocumentLocation!=QDate.currentDate():
                    cond.append(tableDocumentLocation['documentLocationDate'].le(endDateFilterDocumentLocation))
                if personDocumentLocation:
                    cond.append(tableDocumentLocation['person_id'].eq(personDocumentLocation))
            statusObservationType = filter.get('statusObservationType', None)
            if statusObservationType:
                cond.append('''EXISTS(SELECT Client_StatusObservation.id FROM Client_StatusObservation WHERE Client_StatusObservation.deleted = 0 AND Client_StatusObservation.master_id = Client.id AND Client_StatusObservation.statusObservationType_id = %s)'''%(str(statusObservationType)))
            address = filter.get('address', None)
            if address:
                addrType, KLADRCode, Okato, KLADRStreetCode, KLADRStreetCodeList, house, chkCorpus, corpus, flat = address
                tableClientAddress = db.table('ClientAddress')
                tableAddressHouse = db.table('AddressHouse')
                tableAddress = db.table('Address')
                table = table.leftJoin(tableClientAddress, tableClientAddress['client_id'].eq(tableClient['id']))
                table = table.leftJoin(tableAddress, tableAddress['id'].eq(tableClientAddress['address_id']))
                table = table.leftJoin(tableAddressHouse, tableAddressHouse['id'].eq(tableAddress['house_id']))
                cond.append('''ClientAddress.id IN (SELECT MAX(CA.id) FROM ClientAddress AS CA WHERE CA.type=%d
                AND CA.deleted=0 AND CA.client_id=Client.id)''' % addrType)
                if KLADRStreetCode:
                    cond.append(tableAddressHouse['KLADRStreetCode'].eq(KLADRStreetCode))
                    if Okato:
                        # какая-то чёрная магия.
                        # я верю, что это можно улучшить!
                        okatoCond = u'''kladr.DOMA.OCATD LIKE \'%s%%\'''' % Okato if Okato else '''1'''
                        cond.append(u'''(SELECT DISTINCT kladr.STREET.CODE
                        FROM kladr.STREET INNER JOIN kladr.DOMA ON kladr.DOMA.level5 = kladr.STREET.level5
                        WHERE %skladr.STREET.actuality=\'00\'
                        AND kladr.STREET.CODE = \'%s\'
                        AND %s
                        AND kladr.STREET.CODE = AddressHouse.KLADRStreetCode
                        AND (IF(AddressHouse.number != '' AND AddressHouse.number != ' ', FIND_IN_SET(AddressHouse.number,
                        kladr.DOMA.flatHouseList), 1))
                        ORDER BY kladr.STREET.NAME, kladr.STREET.SOCR, kladr.STREET.CODE)'''%((u'kladr.DOMA.CODE LIKE \'%s%%\' AND '%(KLADRCode[0:-2])) if KLADRCode else u'', KLADRStreetCode, okatoCond))
                    else:
                        cond.append(tableAddressHouse['KLADRStreetCode'].eq(KLADRStreetCode))
                elif Okato:
                    cond.append(tableAddressHouse['KLADRStreetCode'].inlist(KLADRStreetCodeList))
                elif KLADRCode:
                    mask = getLikeMaskForRegion(KLADRCode)
                    if mask == KLADRCode:
                        cond.append(tableAddressHouse['KLADRCode'].eq(KLADRCode))
                    else:
                        cond.append(tableAddressHouse['KLADRCode'].like(mask))
                if house:
                    cond.append(tableAddressHouse['number'].eq(house))
                if chkCorpus:
                    cond.append('''TRIM(AddressHouse.corpus) = TRIM('%s')'''%(corpus))
                if flat:
                    cond.append(tableAddress['flat'].eq(flat))
            elif 'addressOrgStructure' in filter:
                addrType, orgStructureId = filter['addressOrgStructure']
                addrIdList = None
                cond2 = []
                if (addrType+1) & 1:
                    addrIdList = getOrgStructureAddressIdList(orgStructureId)
                    addAddressCond(cond2, 0, addrIdList)
                if (addrType+1) & 2:
                    if addrIdList is None:
                        addrIdList = getOrgStructureAddressIdList(orgStructureId)
                    addAddressCond(cond2, 1, addrIdList)
                if ((addrType+1) & 4):
                    if orgStructureId:
                        orgStructureIdList = getOrgStructureDescendants(orgStructureId)
                        addAttachCond(cond2, 'orgStructure_id IN (%s)'%(u','.join(forceString(orgStructureId) for orgStructureId in orgStructureIdList)), 1, None)
                    else:
                        addAttachCond(cond2, 'LPU_id=%d'%QtGui.qApp.currentOrgId(), 1, None)
                if cond2:
                    cond.append(db.joinOr(cond2))
            else:
                if 'regAddressIsEmpty' in filter:
                    addAddressCond(cond, 0, None)
                if 'locAddressIsEmpty'in filter:
                    addAddressCond(cond, 1, None)
            if 'beds' in filter:
                indexFlatCode, orgStructureId = filter['beds']
                if orgStructureId:
                    bedsIdList = getOrgStructureDescendants(orgStructureId)
                else:
                    bedsIdList = getOrgStructures(QtGui.qApp.currentOrgId())
                if indexFlatCode == 2:
                    clientBedsIdList = self.getLeavedHospitalBeds(bedsIdList)
                else:
                    clientBedsIdList = self.getHospitalBeds(bedsIdList, indexFlatCode)
                cond.append(tableClient['id'].inlist(clientBedsIdList))
            if 'attachTo' in filter:
                attachOrgId = filter['attachTo']
                if attachOrgId:
                    addAttachCond(cond, 'LPU_id=%d'%attachOrgId, *filter.get('attachType', (0, None, QDate(), QDate())), isStatement = filter.get('isStatement', 0))
                else:
                    pass # добавить - не имеет пост. прикрепления
            elif 'attachToNonBase' in filter:
                addAttachCond(cond, 'LPU_id!=%d'%QtGui.qApp.currentOrgId(), *filter.get('attachType', (0, None, QDate(), QDate())), isStatement = filter.get('isStatement', 0))
            elif 'attachType' in filter:
                addAttachCond(cond, '', *filter['attachType'], isStatement = filter.get('isStatement', 0))
            if 'tempInvalid' in filter:
                begDate, endDate = filter['tempInvalid']
                if begDate or endDate:
                    tableTempInvalid = db.table('TempInvalid')
                    table = table.leftJoin(tableTempInvalid, tableTempInvalid['client_id'].eq(tableClient['id']))
                    if begDate:
                        cond.append(tableTempInvalid['endDate'].ge(begDate))
                    if endDate:
                        cond.append(tableTempInvalid['begDate'].le(endDate))
            if self.TFAccountingSystemId:
                if filter.get('TFUnconfirmed', False):
                    tableClientIdentification = db.table('ClientIdentification')
                    condClientIdentification  = [ tableClientIdentification['client_id'].eq(tableClient['id']),
                                                  tableClientIdentification['accountingSystem_id'].eq(self.TFAccountingSystemId),
                                                  tableClientIdentification['deleted'].eq(0),
                                                ]
                    cond.append('NOT '+db.existsStmt(tableClientIdentification, condClientIdentification))
                elif 'TFConfirmed' in filter:
                    begDate, endDate = filter['TFConfirmed']
                    tableClientIdentification = db.table('ClientIdentification')
                    condClientIdentification  = [ tableClientIdentification['client_id'].eq(tableClient['id']),
                                                  tableClientIdentification['accountingSystem_id'].eq(self.TFAccountingSystemId),
                                                  tableClientIdentification['deleted'].eq(0),
                                                ]
                    if begDate:
                        condClientIdentification.append(tableClientIdentification['checkDate'].ge(begDate))
                    if endDate:
                        condClientIdentification.append(tableClientIdentification['checkDate'].le(endDate))
                    cond.append(db.existsStmt(tableClientIdentification, condClientIdentification))

            if 'clientConsentTypeId' in filter:
                tableCC = db.table('ClientConsent')
                condClientConsent = [tableCC['client_id'].eq(tableClient['id']), tableCC['deleted'].eq(0)]
                clientConsentValue = filter.get('clientConsentValue', None)
                if clientConsentValue in (1, 2):
                    condClientConsent.append(tableCC['value'].eq(clientConsentValue - 1))
                clientConsentTypeId = filter['clientConsentTypeId']
                if clientConsentTypeId:
                    condClientConsent.append(tableCC['clientConsentType_id'].eq(clientConsentTypeId))
                clientConsentBegDate = filter.get('clientConsentBegDate', None)
                if clientConsentBegDate:
                    condClientConsent.append(db.joinOr([tableCC['endDate'].dateGe(clientConsentBegDate),
                                                        tableCC['endDate'].isNull()]))
                clientConsentEndDate = filter.get('clientConsentEndDate', None)
                if clientConsentEndDate:
                    condClientConsent.append(tableCC['date'].dateLe(clientConsentEndDate))
                clientPersonConsent = filter.get('clientPersonConsent', None)
                if clientPersonConsent:
                    condClientConsent.append(tableCC['modifyPerson_id'].eq(clientPersonConsent))
                if clientConsentValue == 3:
                    cond.append(db.notExistsStmt(tableCC, condClientConsent))
                else:
                    cond.append(db.existsStmt(tableCC, condClientConsent))

            vaccinationClientCond = self.getVaccinationClientCond(filter)
            if vaccinationClientCond:
                cond.append(vaccinationClientCond)

            table = self.appendContingentTypeCond(table, filter, cond, isEventJoined, isSocStatusJoined)

            if filter.get('excludeLeaved'):
                cond.append(tableClient['deathDate'].isNull())

            if 'clientMKBFrom' in filter:
                clientMKBFrom = filter.get('clientMKBFrom', None)
                clientMKBTo = filter.get('clientMKBTo', None)
                tableDiagnosis = db.table('Diagnosis')
                tableDiagnosisType = db.table('rbDiagnosisType')
                table = table.leftJoin(tableDiagnosis, tableDiagnosis['client_id'].eq(tableClient['id']))
                table = table.leftJoin(tableDiagnosisType, tableDiagnosisType['id'].eq(tableDiagnosis['diagnosisType_id']))
                if clientMKBFrom:
                    cond.append(tableDiagnosis['MKB'].ge(clientMKBFrom))
                if clientMKBTo:
                    cond.append(tableDiagnosis['MKB'].le(clientMKBTo))
                if clientMKBFrom or clientMKBTo:
                    cond.append(tableDiagnosisType['code'].inlist(['1', '2', '3', '4']))

            clientsLimit = 10000
            getIdList = db.getDistinctIdList if table != tableClient else db.getIdList
            sort = ['%s %s'%(tableClient['lastName'].name(), self.clientsSortingDest),
                '%s %s'%(tableClient['firstName'].name(), self.clientsSortingDest),
                '%s %s'%(tableClient['patrName'].name(), self.clientsSortingDest)
                ]
            if self.clientsSortingCol == 3:
                sort.insert(0, '%s %s'%(tableClient['birthDate'].name(), self.clientsSortingDest))
            else:
                sort.append('%s %s'%(tableClient['birthDate'].name(), self.clientsSortingDest))
            sort.append('%s %s'%(tableClient['id'].name(), self.clientsSortingDest))
            idList = getIdList(table,
                               tableClient['id'].name(),
                               cond,
                               sort,
                               limit=clientsLimit)

            if len(idList) < clientsLimit:
                clientCount = len(idList)
            else:
                getCount = db.getDistinctCount if table != tableClient else db.getCount
                clientCount = getCount(table, tableClient['id'].name(), cond)

            self.tblClients.setIdList(idList, posToId, clientCount)
#            self.tabMain.setTabEnabled(1, bool(idList))
        finally:
            QtGui.QApplication.restoreOverrideCursor()
        self.focusClients()
        self.updateClientsListRequest = False
        if not idList:
            res = QtGui.QMessageBox.warning(self,
                            u'Внимание',
                            u'Пациент не обнаружен.\nХотите зарегистрировать пациента?',
                            QtGui.QMessageBox.Ok | QtGui.QMessageBox.Cancel,
                            QtGui.QMessageBox.Ok)
            if res == QtGui.QMessageBox.Ok:
                self.editNewClient()
                self.focusClients()
        elif clientCount == 1 and QtGui.qApp.userHasAnyRight([urBatchRegLocatCardProcess]):
            prefs = getPref(QtGui.qApp.preferences.reportPrefs, 'BatchRegLocatCardParams', {})
            batchRegLocatCardProcess = getPrefBool(prefs, 'BatchRegLocatCardProcess', False)
            if batchRegLocatCardProcess:
                self.getParamsBatchRegistrationLocationCard = CGetParamsBatchRegistrationLocationCard(self, idList[0])
                clientLocationCardId = self.getParamsBatchRegistrationLocationCard.registrationLocationCard()
                if not clientLocationCardId:
                    res = QtGui.QMessageBox.warning(self,
                                    u'Внимание',
                                    u'Ошибка регистрации статуса места нахождения карты пациента.\nСтатус не зарегистрирован.',
                                    QtGui.QMessageBox.Cancel,
                                    QtGui.QMessageBox.Cancel)
                else:
                    clientIdUpdate = idList[0]
                    self.showClientInfo(clientIdUpdate)


    def getVaccinationClientCond(self, filter):
        cond = []

        vaccinationBegDate     = filter.get('vaccinationBegDate',     None)
        vaccinationEndDate     = filter.get('vaccinationEndDate',     None)
        vaccinationCalendarId  = filter.get('vaccinationCalendarId',  None)
        infectionIdList        = filter.get('infectionIdList',        None)
        vaccineIdList          = filter.get('vaccineIdList',          None)
        vaccinationSeria       = filter.get('vaccinationSeria',       None)
        vaccinationType        = filter.get('vaccinationType',        None)
        vaccinationContingent  = filter.get('vaccinationContingent',  None)
        medicalExemption       = filter.get('medicalExemption',       None)
        medicalExemptionTypeId = filter.get('medicalExemptionTypeId', None)

        db = QtGui.qApp.db

        if vaccinationContingent: # Привиты
            tableClientVaccination = db.table('ClientVaccination')
            table = tableClientVaccination
            cond.append(tableClientVaccination['client_id'].eq(db.table('Client')['id']))
            if vaccinationBegDate:
                cond.append(tableClientVaccination['datetime'].dateGe(vaccinationBegDate))
            if vaccinationEndDate:
                cond.append(tableClientVaccination['datetime'].dateLe(vaccinationEndDate))
            if vaccineIdList:
                cond.append(tableClientVaccination['vaccine_id'].inlist(vaccineIdList))
            elif infectionIdList or vaccinationCalendarId:
                cond.append(tableClientVaccination['vaccine_id'].inlist(self.modelFilterVaccine.idList()))
            if vaccinationSeria:
                cond.append(tableClientVaccination['seria'].eq(vaccinationSeria))
            if vaccinationType:
                cond.append(tableClientVaccination['vaccinationType'].eq(vaccinationType))
            if medicalExemption is None:
                return db.existsStmt(table, cond)
            else:
                vaccinationStmt = db.existsStmt(table, cond)
        if medicalExemption is not None:
            cond = []
            tableClientMedicalExemption          = db.table('ClientMedicalExemption')
            tableClientMedicalExemptionInfection = db.table('ClientMedicalExemption_Infection')
            table = tableClientMedicalExemption
            cond.append(tableClientMedicalExemption['client_id'].eq(db.table('Client')['id']))
            if vaccinationBegDate:
                cond.append(tableClientMedicalExemption['date'].dateGe(vaccinationBegDate))
            if vaccinationEndDate:
                cond.append(tableClientMedicalExemption['date'].dateLe(vaccinationEndDate))
            if medicalExemptionTypeId:
                cond.append(tableClientMedicalExemption['medicalExemptionType_id'].eq(medicalExemptionTypeId))
            if infectionIdList:
                table = tableClientMedicalExemption.leftJoin(
                                    tableClientMedicalExemptionInfection,
                                    tableClientMedicalExemptionInfection['master_id'].eq(tableClientMedicalExemption['id'])
                                                                                  )
                cond.append(tableClientMedicalExemptionInfection['infection_id'].inlist(infectionIdList))
            elif vaccinationCalendarId:
                table = tableClientMedicalExemption.leftJoin(
                                    tableClientMedicalExemptionInfection,
                                    tableClientMedicalExemptionInfection['master_id'].eq(tableClientMedicalExemption['id'])
                                                                                  )
                cond.append(tableClientMedicalExemptionInfection['infection_id'].inlist(self.modelFilterInfection.idList()))

            if vaccinationContingent:
                if medicalExemption == 0: # Привиты + нет медотводов
                    return vaccinationStmt + u' AND ' + db.notExistsStmt(table, cond)
                elif medicalExemption == 2: # временный
                    cond.append(tableClientMedicalExemption['endDate'].isNotNull())
                elif medicalExemption == 3: # постоянный
                    cond.append(tableClientMedicalExemption['endDate'].isNull())
                return vaccinationStmt + u' AND ' + db.existsStmt(table, cond)
            else:
                if medicalExemption == 0: # нет медотводов
                    return db.notExistsStmt(table, cond)
                elif medicalExemption == 2: # временный
                    cond.append(tableClientMedicalExemption['endDate'].isNotNull())
                elif medicalExemption == 3: # постоянный
                    cond.append(tableClientMedicalExemption['endDate'].isNull())
                return db.existsStmt(table, cond)
        return None


    def appendContingentTypeCond(self, table, filter, cond, isEventJoined, isSocStatusJoined):
        if 'contingentTypeId' not in filter.keys():
            return table
        db = QtGui.qApp.db
        contingentTypeId = filter.get('contingentTypeId', None)
        contingentEventTypeStatus = filter.get('contingentEventTypeStatus', 0)
        contingentActionTypeStatus = filter.get('contingentActionTypeStatus', 0)

        contingentOperation = forceInt(db.translate(CContingentTypeTranslator.CTTName, 'id',
                                                    contingentTypeId, 'contingentOperation'))
        observedOperation = forceInt(db.translate(CContingentTypeTranslator.CTTName, 'id',
                                                    contingentTypeId, 'observedOperation'))
        eventOrActionOperation = forceInt(db.translate(CContingentTypeTranslator.CTTName, 'id',
                                                    contingentTypeId, 'eventOrActionOperation'))

        tableClient = db.table('Client')

        contingentTypeCond = []

        if CContingentTypeTranslator.isSexAgeSocStatusEnabled(contingentOperation):
            tmp = []
            if not isSocStatusJoined:
                tableClientSocStatus = db.table('ClientSocStatus')
                table = table.leftJoin(tableClientSocStatus, tableClient['id'].eq(tableClientSocStatus['client_id']))
            sexAgeCond    = CContingentTypeTranslator.getSexAgeCond(contingentTypeId)
            socStatusCond = CContingentTypeTranslator.getSocStatusCond(contingentTypeId)
            if CContingentTypeTranslator.isSexAgeSocStatusOperationType_OR(contingentOperation):
                if sexAgeCond:
                    tmp.extend(sexAgeCond)
                if socStatusCond:
                    tmp.extend(socStatusCond)
                contingentTypeCond.append(db.joinOr(tmp))
            else:
                if sexAgeCond:
                    tmp.append(db.joinOr(sexAgeCond))
                if socStatusCond:
                    tmp.append(db.joinOr(socStatusCond))
                contingentTypeCond.append(db.joinAnd(tmp))

        if contingentEventTypeStatus  and (CContingentTypeTranslator.isEventTypeMESEnabled(observedOperation) or CContingentTypeTranslator.isHealthGroupEnabled(observedOperation)):
            tmp = []
            tableEvent = db.table('Event').alias('ContingentEvent')
            contingentEventJoinCond = [tableEvent['client_id'].eq(tableClient['id'])]
            eventTypeCond = CContingentTypeTranslator.getEventTypeJoinCond(contingentTypeId)
            MESCond       = CContingentTypeTranslator.getMESJoinCond(contingentTypeId)
            healtGroupCond= CContingentTypeTranslator.getHealtGroupJoinCond(contingentTypeId)
            if CContingentTypeTranslator.isEventTypeMESOperationType_OR(observedOperation):
                if eventTypeCond:
                    tmp.extend(eventTypeCond)
                if MESCond:
                    tmp.extend(MESCond)
                contingentEventJoinCond.append(db.joinOr(tmp))
            else:
                if eventTypeCond:
                    tmp.append(db.joinOr(eventTypeCond))
                if MESCond:
                    tmp.append(db.joinOr(MESCond))
                contingentEventJoinCond.append(db.joinAnd(tmp))

            if CContingentTypeTranslator.isHealthGroupOperationType_OR(observedOperation):
                if eventTypeCond and not CContingentTypeTranslator.isEventTypeMESOperationType_OR(observedOperation):
                    tmp.extend(eventTypeCond)
                if healtGroupCond:
                    tmp.extend(healtGroupCond)
                contingentEventJoinCond.append(db.joinOr(tmp))
            else:
                if eventTypeCond and not CContingentTypeTranslator.isEventTypeMESOperationType_OR(observedOperation):
                    tmp.append(db.joinOr(eventTypeCond))
                if healtGroupCond:
                    tmp.append(db.joinOr(healtGroupCond))
                contingentEventJoinCond.append(db.joinAnd(tmp))
            if contingentEventTypeStatus > 1:
                table = table.innerJoin(tableEvent, contingentEventJoinCond)
            else:
                table = table.leftJoin(tableEvent, contingentEventJoinCond)

            contingentEventCond = CContingentTypeTranslator.getEventStatusCond(contingentEventTypeStatus)
            if contingentEventCond:
                cond.extend(contingentEventCond)

        if contingentActionTypeStatus and CContingentTypeTranslator.isEventTypeActionTypeEnabled(eventOrActionOperation):
            tmp = []
            isEventTypeMES = not contingentEventTypeStatus or not CContingentTypeTranslator.isEventTypeMESEnabled(observedOperation)
            tableAction  = db.table('Action').alias('ContingentAction')
            contingentActionJoinCond = []
            if CContingentTypeTranslator.isEventTypeActionTypeOperationType_OR(eventOrActionOperation):
                if contingentActionTypeStatus == 1:
                    eventTypeCond = CContingentTypeTranslator.getEventTypeSubQueryJoinCond(contingentTypeId)
                    actionTypeCond = CContingentTypeTranslator.getActionTypeSubQueryJoinCond(contingentTypeId)
                    if eventTypeCond and isEventTypeMES:
                        tmp.extend(eventTypeCond)
                    if actionTypeCond:
                        tmp.extend(actionTypeCond)
                    contingentEventActionCond =  [u'''Client.`id` NOT IN (SELECT CE.client_id FROM Action AS CA INNER JOIN Event AS CE ON  CA.`event_id`=CE.`id` WHERE CE.`client_id`= Client.`id` AND  %s)'''%(db.joinOr(tmp))]
                else:
                    if isEventTypeMES:
                        tableEvent = db.table('Event').alias('ContingentEvent')
                        eventTypeCond = CContingentTypeTranslator.getEventTypeJoinCond(contingentTypeId)
                        table = table.innerJoin(tableEvent, tableEvent['client_id'].eq(tableClient['id']))
                    actionTypeCond = CContingentTypeTranslator.getActionTypeJoinCond(contingentTypeId)
                    if eventTypeCond and isEventTypeMES:
                        tmp.extend(eventTypeCond)
                    if actionTypeCond:
                        tmp.extend(actionTypeCond)
                    contingentActionJoinCond.append(tableEvent['id'].eq(tableAction['event_id']))
                    contingentActionJoinCond.append(db.joinOr(tmp))
                    table = table.innerJoin(tableAction, contingentActionJoinCond)
            else:
                if contingentActionTypeStatus == 1:
                    eventTypeCond = CContingentTypeTranslator.getEventTypeSubQueryJoinCond(contingentTypeId)
                    actionTypeCond = CContingentTypeTranslator.getActionTypeSubQueryJoinCond(contingentTypeId)
                    if eventTypeCond and isEventTypeMES:
                        tmp.append(db.joinOr(eventTypeCond))
                    if actionTypeCond:
                        tmp.append(db.joinOr(actionTypeCond))
                    contingentEventActionCond =  [u'''Client.`id` NOT IN (SELECT CE.client_id FROM Action AS CA INNER JOIN Event AS CE ON  CA.`event_id`=CE.`id` WHERE CE.`client_id`= Client.`id` AND  %s)'''%(db.joinAnd(tmp))]
                else:
                    if isEventTypeMES:
                        tableEvent = db.table('Event').alias('ContingentEvent')
                        eventTypeCond = CContingentTypeTranslator.getEventTypeJoinCond(contingentTypeId)
                        table = table.innerJoin(tableEvent, tableEvent['client_id'].eq(tableClient['id']))
                    actionTypeCond = CContingentTypeTranslator.getActionTypeJoinCond(contingentTypeId)
                    if eventTypeCond and isEventTypeMES:
                        tmp.append(db.joinOr(eventTypeCond))
                    if actionTypeCond:
                        tmp.append(db.joinOr(actionTypeCond))
                    contingentActionJoinCond.append(tableEvent['id'].eq(tableAction['event_id']))
                    contingentActionJoinCond.append(db.joinAnd(tmp))
                    table = table.innerJoin(tableAction, contingentActionJoinCond)
            if contingentActionTypeStatus != 1:
                contingentEventActionCond = CContingentTypeTranslator.getActionStatusCond(contingentActionTypeStatus)
            if contingentEventActionCond:
                cond.extend(contingentEventActionCond)
        if contingentTypeCond:
            cond.extend(contingentTypeCond)
        return table


    def getClientFilterAsText(self):
        def formatAge(value):
            unitMap = {0: (u'годa',   u'лет',     u'лет'),   # это родительный падеж, да?
                       1: (u'месяца', u'месяцев', u'месяцев'),
                       2: (u'недели', u'недель',  u'недель'),
                       3: (u'дня',    u'дней',    u'дней')
                      }
            (lowCnt, lowUnit), (highCnt, highUnit) = value
            return u'с %s до %s' %( formatNum(lowCnt,  unitMap.get(lowUnit)  or unitMap.get(0)),
                                     formatNum(highCnt, unitMap.get(highUnit) or unitMap.get(0)),
                                   )

        def formatAddress(value):
            addressType, addressCity, addressOKATO, addressStreet, addressStreetList, addressHouse, chkCorpus, addressCorpus, addressFlat = value
            #TODO: учесть addressOKATO и addressStreetList, да?
            address = smartDict()
            address.update({'KLADRCode':forceString(addressCity),
                            'KLADRStreetCode':forceString(addressStreet),
                            'number':forceString(addressHouse),
                            'corpus':forceString(addressCorpus) if chkCorpus else None,
                            'flat':forceString(addressFlat),
                            'freeInput':None})
            addressText = formatAddressInt(address)
            return {0:u'регистрации ', 1:u'проживания '}.get(addressType, u'регистрации ') + addressText

        filter  = self.__filter
        resList = []
        convertFilterToTextItem(resList, filter, 'id',        u'Код пациента',  unicode)
        convertFilterToTextItem(resList, filter, 'lastName',  u'Фамилия')
        convertFilterToTextItem(resList, filter, 'firstName', u'Имя')
        convertFilterToTextItem(resList, filter, 'patrName',  u'Отчество')
        convertFilterToTextItem(resList, filter, 'birthDate', u'Дата рождения', forceString)
        convertFilterToTextItem(resList, filter, 'age',       u'Возраст',       formatAge)
        convertFilterToTextItem(resList, filter, 'sex',       u'Пол',           formatSex)
        convertFilterToTextItem(resList, filter, 'SNILS',     u'СНИЛС',         formatSNILS)
        convertFilterToTextItem(resList, filter, 'address',   u'Адрес',         formatAddress)
        convertFilterToTextItem(resList, filter, 'doc',       u'документ',      lambda doc: formatDocument(doc[0], doc[1], doc[2]))
        convertFilterToTextItem(resList, filter, 'policy',    u'полис',         lambda policy: formatPolicy(policy[3], policy[4], policy[5]))
        convertFilterToTextItem(resList, filter, 'orgId',     u'занятость',     getOrganisationShortName)
        convertFilterToTextItem(resList, filter, 'regAddressIsEmpty', u'адрес',    lambda dummy: u'пуст')
        return '\n'.join([item[0]+u': '+item[1] for item in resList])


    def showClientInfo(self, id):
        u"""
            показ информации о пациенте в панели наверху
        """
        if id:
            self.txtClientInfoBrowser.setHtml(getClientBanner(id))
        else:
            self.txtClientInfoBrowser.setText('')
        self.actEditClient.setEnabled(bool(id))
        self.actRelationsClient.setEnabled(bool(id))
        self.actOpenClientDocumentTrackingHistory.setEnabled(bool(id) and QtGui.qApp.userHasAnyRight([urRegTabReadLocationCard, urEditLocationCard]))
        self.actEditStatusObservationClient.setEnabled(bool(id))
        self.actShowContingentsClient.setEnabled(bool(id))
        QtGui.qApp.setCurrentClientId(id)


    def clientId(self, index):
        return self.tblClients.itemId(index)


    def selectedClientId(self):
        return self.tblClients.currentItemId()


    def currentClientId(self):
        return QtGui.qApp.currentClientId()


    def editClient(self, clientId):
        if QtGui.qApp.userHasAnyRight([urAdmin, urRegTabWriteRegistry, urRegTabReadRegistry]):
            dialog = CClientEditDialog(self)
            try:
                if clientId:
                    dialog.load(clientId)
                if dialog.exec_():
                    clientId = dialog.itemId()
                    self.updateClientsList(self.__filter, clientId)
            finally:
                dialog.destroy()
                sip.delete(dialog)
                del dialog
    #            self.showClientInfo(clientId)


    def editCurrentClient(self):
        clientId = self.currentClientId()
        self.editClient(clientId)


    def findClient(self, clientId, clientRecord=None):
        self.tabMain.setCurrentIndex(0)
        if not clientRecord:
            db = QtGui.qApp.db
            clientRecord = db.getRecord('Client', 'lastName, firstName, patrName, sex, birthDate, SNILS', clientId)
        if clientRecord:
            filter = {}
            filter['id'] = clientId
            filter['accountingSystemId'] = None
            filter['lastName'] = forceString(clientRecord.value('lastName'))
            filter['firstName'] = forceString(clientRecord.value('firstName'))
            filter['patrName'] = forceString(clientRecord.value('patrName'))
            filter['sex'] = forceInt(clientRecord.value('sex'))
            filter['birthDate'] = forceDate(clientRecord.value('birthDate'))
            filter['SNILS'] = forceString(clientRecord.value('SNILS'))
            self.__filter = self.updateFilterWidgets(filter)
            if not self.__filter:
                self.__filter = self.updateFilterWidgets({'id': clientId, 'accountingSystemId':None}, True)
            self.updateClientsList(self.__filter, clientId)
            self.tabMain.setTabEnabled(1, True)


    def getParamsDialogFilter(self):
        dialogInfo = {}
        if self.chkFilterLastName.isChecked():
            dialogInfo['lastName'] = forceString(self.edtFilterLastName.text())
        if self.chkFilterFirstName.isChecked():
            dialogInfo['firstName'] = forceString(self.edtFilterFirstName.text())
        if self.chkFilterPatrName.isChecked():
            dialogInfo['patrName'] = forceString(self.edtFilterPatrName.text())
        if self.chkFilterBirthDay.isChecked():
            dialogInfo['birthDate'] = forceDate(self.edtFilterBirthDay.date())
        if self.chkFilterSex.isChecked():
            dialogInfo['sex'] = self.cmbFilterSex.currentIndex()
        if self.chkFilterSNILS.isChecked():
            dialogInfo['SNILS'] = self.edtFilterSNILS.text()
        if self.chkFilterDocument.isChecked():
            dialogInfo['docType'] = self.cmbFilterDocumentType.value()
            serial = self.edtFilterDocumentSerial.text()
            for c in '-=/_|':
                serial = serial.replace('c',' ')
            serial = trim(serial).split()
            dialogInfo['serialLeft'] = serial[0] if len(serial)>=1 else ''
            dialogInfo['serialRight'] = serial[1] if len(serial)>=2 else ''
            dialogInfo['docNumber'] = self.edtFilterDocumentNumber.text()
        if self.chkFilterContact.isChecked():
            dialogInfo['contact'] = forceString(self.edtFilterContact.text())
        if self.chkFilterPolicy.isChecked():
            dialogInfo['polisSerial'] = forceString(self.edtFilterPolicySerial.text())
            dialogInfo['polisNumber'] = forceString(self.edtFilterPolicyNumber.text())
            dialogInfo['polisCompany'] = self.cmbFilterPolicyInsurer.value()
            dialogInfo['polisType'] = self.cmbFilterPolicyType.value()
            dialogInfo['polisTypeName'] = self.cmbFilterPolicyType.model().getName(self.cmbFilterPolicyType.currentIndex())
        if self.chkFilterAddress.isChecked():
            dialogInfo['addressType'] = self.cmbFilterAddressType.currentIndex()
            dialogInfo['regCity'] = self.cmbFilterAddressCity.code()
            dialogInfo['regStreet'] = self.cmbFilterAddressStreet.code()
            dialogInfo['regHouse'] = self.edtFilterAddressHouse.text()
            if self.chkFilterAddressCorpus.isChecked():
                dialogInfo['regCorpus'] = self.edtFilterAddressCorpus.text()
            dialogInfo['regFlat'] = self.edtFilterAddressFlat.text()
        return dialogInfo


    def editNewClient(self):
        if QtGui.qApp.userHasAnyRight([urAdmin, urRegTabWriteRegistry]):
            dialog = CClientEditDialog(self)
            dialogInfo = self.getParamsDialogFilter()
            if dialogInfo:
                dialogInfo['extendedPolicyInformation'] = self.extendedPolicyInformation
                dialog.setClientDialogInfo(dialogInfo)
            try:
                if dialog.exec_():
                    clientId = dialog.itemId()
                    clientRecord = dialog.getRecord()
                    self.findClient(clientId, clientRecord)
            finally:
                dialog.destroy()
                sip.delete(dialog)
                del dialog


    def findControlledByChk(self, chk):
        for s in self.chkListOnClientsPage:
            if s[0] == chk:
                return s[1]
        for s in self.chkListOnEventsPage:
            if s[0] == chk:
                return s[1]
        for s in self.chkListOnActionsPage:
            if s[0] == chk:
                return s[1]
        for s in self.chkListOnExpertPage:
            if s[0] == chk:
                return s[1]
        for s in self.chkListOnExpertMCPage:
            if s[0] == chk:
                return s[1]
        for s in self.chkListOnVisitsPage:
            if s[0] == chk:
                return s[1]
        return None


    def activateFilterWdgets(self, alist):
        if alist:
            for s in alist:
                s.setEnabled(True)
                if isinstance(s, (QtGui.QLineEdit, CDateEdit)):
                    s.selectAll()
            alist[0].setFocus(Qt.ShortcutFocusReason)
            alist[0].update()


    def deactivateFilterWdgets(self, alist):
        for s in alist:
            s.setEnabled(False)


    def updateFilterWidgets(self, filter, force=False):
        def intUpdateClientFilterLineEdit(chk, widget, inFilter, name, outFilter):
            val = inFilter.get(name, None)
            if val and (chk.isChecked() or force):
                chk.setChecked(True)
                self.activateFilterWdgets([widget])
                widget.setText(unicode(val))
                outFilter[name] = val

        def intUpdateClientFilterComboBox(chk, widget, inFilter, name, outFilter):
            if name in inFilter and (chk.isChecked() or force):
                val = inFilter[name]
                chk.setChecked(True)
                self.activateFilterWdgets([widget])
                widget.setCurrentIndex(val)
                outFilter[name] = val

        def intUpdateClientFilterRBComboBox(chk, widget, inFilter, name, outFilter):
            if name in inFilter and (chk.isChecked() or force):
                val = inFilter[name]
                chk.setChecked(True)
                self.activateFilterWdgets([widget])
                widget.setValue(val)
                outFilter[name] = val

        def intUpdateClientFilterDateEdit(chk, widget, inFilter, name, outFilter):
            val = inFilter.get(name, None)
            if val and (chk.isChecked() or force):
                chk.setChecked(True)
                self.activateFilterWdgets([widget])
                widget.setDate(val)
                outFilter[name] = val

        outFilter = {}
        for s in self.chkListOnClientsPage:
            chk = s[0]
            if chk == self.chkFilterId:
                intUpdateClientFilterLineEdit(chk, s[1][0], filter, 'id', outFilter)
                intUpdateClientFilterRBComboBox(chk, s[1][1], filter, 'accountingSystemId', outFilter)
            elif chk == self.chkFilterLastName:
                intUpdateClientFilterLineEdit(chk, s[1][0], filter, 'lastName', outFilter)
            elif chk == self.chkFilterFirstName:
                intUpdateClientFilterLineEdit(chk, s[1][0], filter, 'firstName', outFilter)
            elif chk == self.chkFilterPatrName:
                intUpdateClientFilterLineEdit(chk, s[1][0], filter, 'patrName', outFilter)
            elif chk == self.chkFilterSex:
                intUpdateClientFilterComboBox(chk, s[1][0], filter, 'sex', outFilter)
            elif chk == self.chkFilterBirthDay:
                intUpdateClientFilterDateEdit(chk, s[1][0], filter, 'birthDate', outFilter)
            elif chk == self.chkFilterContact:
                intUpdateClientFilterLineEdit(chk, s[1][0], filter, 'contact', outFilter)
            elif chk == self.chkFilterSNILS:
                intUpdateClientFilterLineEdit(chk, s[1][0], filter, 'SNILS', outFilter)
            else:
                chk.setChecked(False)
                self.deactivateFilterWdgets(s[1])
        return outFilter


    def onChkFilterToggled(self, chk, checked):
        controlled = self.findControlledByChk(chk)
        if checked:
            self.activateFilterWdgets(controlled)
        else:
            self.deactivateFilterWdgets(controlled)


    def findEvent(self, eventId):
        self.tabMain.setCurrentIndex(1)
        if eventId in self.modelEvents.idList():
            self.tblEvents.setCurrentItemId(eventId)
        else:
            filter = {}
            self.updateEventsList(filter, eventId)


    def setEventList(self, eventIdList):
        self.tabMain.setCurrentIndex(1)
        self.tblEvents.setIdList(eventIdList)
#            filter = {}
#            self.updateEventsList(filter, eventId)


    def requestNewEvent(self):
        clientId = self.currentClientId()
        if clientId:
            return requestNewEvent({'widget':self, 'clientId':clientId})
        return None


    def requestNewEventQueue(self, typeQueue=-1):
        clientId = self.currentClientId()
#        #WTF? у реестра просят создать событие, но он почему-то делегирует это дальше
#        #правильное решение: создавать самому + слать сигналы, которые учтут изменение в ресурсах.
#        dockFreeQueue = QtGui.qApp.mainWindow.dockFreeQueue
#        if dockFreeQueue and dockFreeQueue.content and dockFreeQueue.content.locked():
#            dockFreeQueue.content.on_actAmbCreateOrder_triggered()
#        elif clientId:
        if clientId:
            return requestNewEvent({'widget':self, 'clientId':clientId, 'typeQueue':typeQueue})
        return None

    #########################################################################

    def setCmbFilterEventTypeFilter(self, eventPurposeId):
        if eventPurposeId:
            filter = 'EventType.purpose_id =%d' % eventPurposeId
        else:
            filter = getWorkEventTypeFilter()
        self.cmbFilterEventType.setFilter(filter)


    def addEqCond(self, cond, table, fieldName, filter, name):
        if name in filter:
            cond.append(table[fieldName].eq(filter[name]))


    def addLikeCond(self, cond, table, fieldName, filter, name):
        if name in filter:
            cond.append(table[fieldName].like(filter[name]))


    def addDateCond(self, cond, table, fieldName, filter, begDateName, endDateName):
        begDate = filter.get(begDateName, None)
        if begDate:
            cond.append(table[fieldName].ge(begDate))
        endDate = filter.get(endDateName, None)
        if endDate:
            cond.append(table[fieldName].lt(endDate.addDays(1)))


    def addDateTimeCond(self, cond, table, fieldName, filter, begDateTimeName, endDateTimeName):
        begDateTime = filter.get(begDateTimeName, None)
        if begDateTime:
            cond.append(table[fieldName].ge(begDateTime))
        endDateTime = filter.get(endDateTimeName, None)
        if endDateTime:
            cond.append(table[fieldName].le(endDateTime))


    def addRangeCond(self, cond, table, fieldName, filter, begName, endName):
        if begName in filter:
            cond.append(table[fieldName].ge(filter[begName]))
        if endName in filter:
            cond.append(table[fieldName].le(filter[endName]))


    def updateEventListAfterEdit(self, eventId):
        if self.tabMain.currentIndex() == 1:
            self.updateEventsList(self.__eventFilter, eventId)
        QtGui.qApp.emitCurrentClientInfoChanged()


    def _setEventsOrderByColumn(self, column):
        self.tblEvents.setOrder(column)
        self.updateEventsList(self.__eventFilter, self.tblEvents.currentItemId())


    def updateEventsList(self, filter, posToId=None):
        # в соответствии с фильтром обновляет список событий.
        self.__eventFilter = filter
        order = self.tblEvents.order() if self.tblEvents.order() else ['Event.setDate DESC', 'Event.id']
        db = QtGui.qApp.db
        table = db.table('Event')
        tableEventType = db.table('EventType')
        tableEventTypePurpose = db.table('rbEventTypePurpose')
        tableVisit = db.table('Visit')
        queryTable = table
        cond = [table['deleted'].eq(0),
                tableEventType['context'].notlike(u'inspection%')
                ]
        queryTable = queryTable.innerJoin(tableEventType, tableEventType['id'].eq(table['eventType_id']))
        queryTable = queryTable.innerJoin(tableEventTypePurpose, tableEventTypePurpose['id'].eq(tableEventType['purpose_id']))

        if 'Client' in order:
            tableClient = db.table('Client')
            queryTable = queryTable.leftJoin(tableClient, tableClient['id'].eq(table['client_id']))

        if 'MES' in order:
            tableMES = db.table('mes.MES')
            queryTable = queryTable.leftJoin(tableMES, tableMES['id'].eq(table['MES_id']))

        if 'Event.execPerson_id' in order or 'specialityId' in filter or 'orgStructureId' in filter:
            tableExecPerson = db.table('Person')
            queryTable = queryTable.leftJoin(tableExecPerson, tableExecPerson['id'].eq(table['execPerson_id']))

        if 'rbResult' in order:
            tableResult = db.table('rbResult')
            queryTable = queryTable.leftJoin(tableResult, tableResult['id'].eq(table['result_id']))

        if 'MKB' in order or 'MKBEx' in order:
            tableDiagnosis = db.table('Diagnosis')
            queryTable = queryTable.leftJoin(tableDiagnosis, 'Diagnosis.id=getEventDiagnosis(Event.id)')

        if u'rbHealthGroup.code' in order:
            tableRBHealthGroup = db.table('rbHealthGroup')
            queryTable = queryTable.leftJoin(tableRBHealthGroup, 'rbHealthGroup.id=getEventHealthGroupDiagnostic(Event.id)')

        if u'rbDiagnosticResult.name' in order:
            tableRBDiagnosticResult = db.table('rbDiagnosticResult')
            queryTable = queryTable.leftJoin(tableRBDiagnosticResult, 'rbDiagnosticResult.id=getEventResultDiagnostic(Event.id)')

        cond.append(tableEventTypePurpose['code'].notlike('0'))
        self.addEqCond(cond, table, 'id', filter, 'id')
        self.addEqCond(cond, table, 'createPerson_id', filter, 'createPersonId')
        self.addDateCond(cond, table, 'createDatetime', filter, 'begCreateDate', 'endCreateDate')
        self.addEqCond(cond, table, 'modifyPerson_id', filter, 'modifyPersonId')
        self.addDateCond(cond, table, 'modifyDatetime', filter, 'begModifyDate', 'endModifyDate')
        if 'clientIds' in filter:
            clientIds = filter['clientIds']
            cond.append(table['client_id'].inlist(clientIds))
        if 'externalId' in filter:
            cond.append(table['externalId'].eq(filter['externalId']))
        eventExpertId = filter.get('eventExpertId', False)
        if eventExpertId:
            cond.append(table['expert_id'].eq(eventExpertId))
        elif eventExpertId is None:
            cond.append(table['expert_id'].isNull())

        if 'begExpertiseDate' in filter:
            begExpertiseDate = filter.get('begExpertiseDate', None)
            endExpertiseDate = filter.get('endExpertiseDate', None)
            if not begExpertiseDate and not endExpertiseDate:
                cond.append(table['expertiseDate'].isNull())
            elif begExpertiseDate:
                cond.append(table['expertiseDate'].ge(begExpertiseDate))
            elif endExpertiseDate:
                cond.append(table['expertiseDate'].le(endExpertiseDate))
        if 'accountSumLimit' in filter:
            accountSumLimit = filter.get('accountSumLimit', 0)
            sumLimitFrom = filter.get('sumLimitFrom', 0)
            sumLimitTo = filter.get('sumLimitTo', 0)
            sumLimitDelta = filter.get('sumLimitDelta', 0)
            tableEventLocalContract = db.table('Event_LocalContract')
            queryTable = queryTable.innerJoin(tableEventLocalContract, tableEventLocalContract['master_id'].eq(table['id']))
            cond.append(tableEventLocalContract['deleted'].eq(0))
            cond.append(tableEventLocalContract['sumLimit'].gt(0))
            if sumLimitFrom or sumLimitTo and sumLimitFrom <= sumLimitTo:
                cond.append(tableEventLocalContract['sumLimit'].ge(sumLimitFrom))
                cond.append(tableEventLocalContract['sumLimit'].le(sumLimitTo))
            if accountSumLimit == 1:
                cond.append(table['totalCost'].gt(tableEventLocalContract['sumLimit']))
                if sumLimitDelta > 0:
                    cond.append(u'Event.totalCost >= (Event_LocalContract.sumLimit + %d)'%(sumLimitDelta))
            elif accountSumLimit == 2:
                cond.append(table['totalCost'].lt(tableEventLocalContract['sumLimit']))
                if sumLimitDelta > 0:
                    cond.append(u'Event.totalCost <= (Event_LocalContract.sumLimit + %d)'%(sumLimitDelta))
        self.addDateTimeCond(cond, table, 'setDate', filter, 'begSetDateTime', 'endSetDateTime')
        if filter.get('emptyExecDate', False):
            cond.append(table['execDate'].isNull())
        self.addDateTimeCond(cond, table, 'execDate', filter, 'begExecDateTime', 'endExecDateTime')
        self.addDateCond(cond, table, 'nextEventDate', filter, 'begNextDate', 'endNextDate')
        eventPurposeId = filter.get('eventPurposeId', None)
        if eventPurposeId:
            cond.append(table['eventType_id'].name()+' IN (SELECT id FROM EventType WHERE EventType.purpose_id=%d)' % eventPurposeId)
        if 'eventTypeId' in filter:
            cond.append(table['eventType_id'].eq(filter['eventTypeId']))
        elif not eventPurposeId:
            cond.append(table['eventType_id'].name()+' IN (SELECT id FROM EventType WHERE'+getWorkEventTypeFilter()+')')
        if 'personId' in filter:
            personId = filter['personId']
            if personId == -1:
                cond.append(table['execPerson_id'].isNull())
            else:
                cond.append(table['execPerson_id'].eq(personId))
        else:
            if 'specialityId' in filter:
                cond.append(tableExecPerson['speciality_id'].eq(filter['specialityId']))
            if 'orgStructureId' in filter:
                orgStructureId = filter.get('orgStructureId', None)
                if orgStructureId:
                    orgStructureIdList = db.getDescendants('OrgStructure', 'parent_id', orgStructureId)
                    if orgStructureIdList:
                        cond.append(tableExecPerson['orgStructure_id'].inlist(orgStructureIdList))
        if 'isPrimary' in filter:
            cond.append(table['isPrimary'].eq(filter['isPrimary']))
        if 'order' in filter:
            cond.append(table['order'].eq(filter['order']))
        if 'relegateOrgId' in filter:
            cond.append(table['relegateOrg_id'].eq(filter['relegateOrgId']))
        if filter.get('dispanserObserved', False):
            condDispanserObserved  = 'EXISTS (SELECT * FROM Diagnostic LEFT JOIN rbDispanser ON rbDispanser.id = Diagnostic.dispanser_id WHERE Diagnostic.event_id = Event.id AND rbDispanser.observed)'
            cond.append(condDispanserObserved)
#            cond.append(table['execDate'].isNull())
        self.addEqCond(cond, table, 'org_id', filter, 'LPUId')
        if filter.get('nonBase', False):
            cond.append(table['org_id'].ne(QtGui.qApp.currentOrgId()))
        mesCode = filter.get('mesCode', None)
        if mesCode is not None:
            if mesCode:
                condMes  = 'MES_id IN (SELECT id FROM mes.MES WHERE code LIKE %s)' % quote(undotLikeMask(mesCode))
                cond.append(condMes)
            else:
                cond.append(table['eventType_id'].name()+' IN (SELECT id FROM EventType WHERE EventType.mesRequired)')
                cond.append(table['MES_id'].isNull())

        self.addEqCond(cond, table, 'result_id', filter, 'eventResultId')
        if filter.get('errorInDiagnostic', False):
            condErrInFinish = '(SELECT COUNT(*) FROM Diagnostic WHERE Diagnostic.event_id = Event.id AND diagnosisType_id=(SELECT id FROM rbDiagnosisType WHERE code=\'1\')) != 1'
            condErrInGroup  = 'EXISTS (SELECT * FROM Diagnostic WHERE Diagnostic.event_id = Event.id AND healthGroup_id IS NULL)'
            cond.append(db.joinOr([condErrInFinish, condErrInGroup]))

        payStatusFinanceCode = filter.get('payStatusFinanceCode', None)
        if payStatusFinanceCode:
            payStatusCode = filter.get('payStatusCode', 0)
            mask  = getPayStatusMaskByCode(payStatusFinanceCode)
            value = getPayStatusValueByCode(payStatusCode, payStatusFinanceCode)
            cond.append('((%s & %d) = %d)' % (table['payStatus'].name(), mask, value))
        if filter['filterVisits']:
            queryTable = queryTable.innerJoin(tableVisit, tableVisit['event_id'].eq(table['id']))
            cond.append(tableVisit['deleted'].eq(0))
            visitScene = filter.get('visitScene', None)
            if visitScene:
                cond.append(tableVisit['scene_id'].eq(visitScene))
            visitType = filter.get('visitType', None)
            if visitType:
                cond.append(tableVisit['visitType_id'].eq(visitType))
            visitProfile = filter.get('visitProfile', None)
            if visitProfile:
                cond.append(tableVisit['service_id'].eq(visitProfile))
        MKBFrom = filter.get('MKBFrom')
        MKBTo = filter.get('MKBTo')
        if MKBFrom or MKBTo:
            if MKBFrom and not MKBTo:
               MKBTo = u'U'
            elif not MKBFrom and MKBTo:
                MKBFrom = u'A'
            condTypeMKB = u''
            if filter.get('MKBisPreliminary'):
                condTypeMKB = u''' OR rbDiagnosisType.code = 7'''
            if filter.get('MKBisConcomitant') and not filter.get('MKBisPreliminary'):
                condTypeMKB = u''' OR rbDiagnosisType.code = 9'''
            elif filter.get('MKBisConcomitant') and filter.get('MKBisPreliminary'):
                condTypeMKB = u''' OR rbDiagnosisType.code = 11'''
            cond.append('''EXISTS(SELECT Diagnosis.id
FROM Diagnosis INNER JOIN Diagnostic ON Diagnostic.diagnosis_id=Diagnosis.id
INNER JOIN rbDiagnosisType ON Diagnostic.diagnosisType_id=rbDiagnosisType.id
WHERE Diagnostic.event_id = Event.id AND Diagnosis.deleted = 0 AND Diagnostic.deleted = 0 %s %s
AND (rbDiagnosisType.code = '1'
OR (rbDiagnosisType.code = '2' AND Diagnostic.person_id = Event.execPerson_id
AND (NOT EXISTS (SELECT DC.id FROM Diagnostic AS DC
INNER JOIN rbDiagnosisType AS DT ON DT.id = DC.diagnosisType_id WHERE DT.code = '1'
AND DC.event_id = Event.id
LIMIT 1)))%s))'''%((u'''AND Diagnosis.MKB LIKE '%s%%' '''% MKBFrom) if len(MKBFrom) == 3 else (u'''AND Diagnosis.MKB >= '%s' ''' % MKBFrom),
(u'''AND Diagnosis.MKB LIKE '%s%%' ''' % MKBTo) if len(MKBTo) == 3 else (u'''AND Diagnosis.MKB <= '%s' ''' % MKBTo), condTypeMKB))
        if 'payStatusCodeVisit' in filter:
            if not filter['filterVisits']:
                queryTable = queryTable.innerJoin(tableVisit, tableVisit['event_id'].eq(table['id']))
                cond.append(tableVisit['deleted'].eq(0))
            payStatusFinanceCodeVisit = filter.get('payStatusFinanceCodeVisit', None)
            if payStatusFinanceCodeVisit:
                payStatusCodeVisit = filter.get('payStatusCodeVisit', 0)
                mask  = getPayStatusMaskByCode(payStatusFinanceCodeVisit)
                value = getPayStatusValueByCode(payStatusCodeVisit, payStatusFinanceCodeVisit)
                cond.append('((%s & %d) = %d)' % (tableVisit['payStatus'].name(), mask, value))
        if 'jobTicketId' in filter:
            jobTicketId = filter['jobTicketId']
            tableAction = db.table('Action')
            tableActionProperty = db.table('ActionProperty')
            tableActionPropertyJobTicket = db.table('ActionProperty_Job_Ticket')
            subTableQuery = tableAction.leftJoin(tableActionProperty,
                                                 tableActionProperty['action_id'].eq(tableAction['id']))
            subTableQuery = subTableQuery.leftJoin(tableActionPropertyJobTicket,
                                                   tableActionPropertyJobTicket['id'].eq(tableActionProperty['id']))
            subCond = [tableAction['deleted'].eq(0),
                       tableActionProperty['deleted'].eq(0),
                       tableActionPropertyJobTicket['value'].eq(jobTicketId),
                       tableAction['event_id'].eq(table['id'])]
            cond.append(db.existsStmt(subTableQuery, subCond))
        payerId = filter.get('payerId')
        if payerId:
            tableContract = db.table('Contract')
            queryTable = queryTable.leftJoin(tableContract, tableContract['id'].eq(table['contract_id']))
            cond.append(tableContract['payer_id'].eq(payerId))
        if 'contractId' in filter:
            cond.append(table['contract_id'].eq(filter['contractId']))
        try:
            QtGui.QApplication.setOverrideCursor(QtGui.QCursor(Qt.WaitCursor))
            idList = db.getDistinctIdList(queryTable,
                           table['id'].name(),
                           cond,
                           order)
#                           ['execDate DESC', 'id'])
            self.tblEvents.setIdList(idList, posToId)
        finally:
            QtGui.QApplication.restoreOverrideCursor()

        hideClientInfo = 'clientIds' in filter and len(filter['clientIds']) == 1
        for i in xrange(4):
            self.tblEvents.setColumnHidden(i, hideClientInfo)
        self.updateClientsListRequest = self.updateClientsListRequest or not hideClientInfo


    def getEventFilterAsText(self):
        db = QtGui.qApp.db
        filter  = self.__eventFilter
        resList = []

        clientIds = filter.get('clientIds', None)
        if clientIds and len(clientIds) == 1:
            resList.append((u'пациент', getClientMiniInfo(clientIds[0])))
        elif clientIds is None:
            resList.append((u'список пациентов', u'полный'))
        else:
            resList.append((u'список пациентов', u'из вкладки'))

        tmpList = [
            ('begSetDateTime', u'Дата и время назначения с', forceString),
            ('endSetDateTime', u'Дата и время назначения по', forceString),
            ('emptyExecDate', u'Пустая дата выполнения', lambda dummy: ''),
            ('begExecDateTime', u'Дата и время выполнения с', forceString),
            ('endExecDateTime', u'Дата и время выполнения по', forceString),
            ('begNextDate', u'Дата следующей явки с', forceString),
            ('endNextDate', u'Дата следующей явки по', forceString),
            ('eventTypeId', u'Тип обращения',
                lambda id: getEventName(id)),
            ('specialityId', u'Специальность',
                lambda id: forceString(db.translate('rbSpeciality', 'id', id, 'name'))),
            ('personId', u'Выполнил',
                lambda id: forceString(db.translate('vrbPersonWithSpeciality', 'id', id, 'name'))),
            ('dispanserObserved', u'Дисп.наблюдение', lambda dummy: ''),
            ('LPUId', u'ЛПУ', getOrganisationShortName),
            ('nonBase', u'Не базовое ЛПУ', lambda dummy: ''),
            ('eventResultId', u'Результат обращения',
                lambda id: forceString(db.translate('rbResult', 'id', id, 'name'))),
            ('errorInDiagnostic', u'Ошибки', lambda dummy: ''),
            ]
        for (key, title, ftm) in tmpList:
            convertFilterToTextItem(resList, filter, key, title, ftm)
        return '\n'.join([ ': '.join(item) for item in resList])


    def eventId(self, index):
        return self.tblEvents.itemId(index)


    def currentEventId(self):
        return self.tblEvents.currentItemId()


    def updateEventInfo(self, eventId):
        db = QtGui.qApp.db
        record = db.getRecord('Event', ['createDatetime','createPerson_id', 'modifyDatetime', 'modifyPerson_id', 'eventType_id', 'externalId', 'client_id', 'payStatus', 'note'], eventId)
        if record:
            createDatetime = dateTimeToString(record.value('createDatetime').toDateTime())
            createPersonId = forceRef(record.value('createPerson_id'))
            modifyDatetime = dateTimeToString(record.value('modifyDatetime').toDateTime())
            modifyPersonId = forceRef(record.value('modifyPerson_id'))
            eventTypeId    = forceRef(record.value('eventType_id'))
            externalId     = forceString(record.value('externalId'))
            clientId       = forceRef(record.value('client_id'))
            note           = forceString(record.value('note'))
            payStatus      = forceInt(record.value('payStatus'))
        else:
            createDatetime = ''
            createPersonId = None
            modifyDatetime = ''
            modifyPersonId = None
            externalId     = ''
            clientId       = None
            note           = ''
            payStatus      = 0

        if not clientId:
            clientId = self.selectedClientId()
        if clientId:
            self.txtClientInfoBrowserEvents.setHtml(getClientBanner(clientId))
        else:
            self.txtClientInfoBrowserEvents.setText('')
        self.actEventEditClient.setEnabled(bool(clientId))
        self.actEventRelationsClient.setEnabled(bool(clientId))
        QtGui.qApp.setCurrentClientId(clientId)

        if eventId:
            table = db.table('Diagnostic')
            idList = db.getIdList(table, 'id', [table['event_id'].eq(eventId),table['deleted'].eq(0)],  ['id'])
            self.tblEventDiagnostics.setIdList(idList)

            tableAction = db.table('Action')
            tableActionType = db.table('ActionType')
            table = tableAction.innerJoin(tableActionType, tableActionType['id'].eq(tableAction['actionType_id']))
            idList = db.getIdList(table, 'Action.id', [tableAction['event_id'].eq(eventId),
                                                tableAction['deleted'].eq(0),
                                                tableActionType['deleted'].eq(0),
                                                tableActionType['flatCode'].notlike(u'temperatureSheet')], ['Action.id'])
            self.tblEventActions.setIdList(idList)

            table = db.table('Visit')
            idList = db.getIdList(table, 'id', [table['event_id'].eq(eventId),table['deleted'].eq(0)], ['id'])
            self.tblEventVisits.setIdList(idList)

            context = getEventContext(eventTypeId)
            additionalCustomizePrintButton(self, self.btnEventPrint, context, self.btnEventPrint_actions)
        else:
            self.tblEventDiagnostics.setIdList([])
            self.tblEventActions.setIdList([])
            self.tblEventVisits.setIdList([])

        self.lblEventIdValue.setText(str(eventId) if eventId else '')
        self.lblEventExternalIdValue.setText(externalId)
        self.lblEventCreateDateTimeValue.setText(createDatetime)
        self.lblEventCreatePersonValue.setText(self.getPersonText(createPersonId))
        self.lblEventModifyDateTimeValue.setText(modifyDatetime)
        self.lblEventModifyPersonValue.setText(self.getPersonText(modifyPersonId))
        self.lblEventNoteValue.setText(note)
        self.lblEventPayStatusValue.setText(payStatusText(payStatus))


    def getPersonText(self, personId):
        if personId:
            index = self.personInfo.searchId(personId)
            if index:
                return u' | '.join([val for val in [forceString(self.personInfo.getCode(index)),
                                                    forceString(self.personInfo.getName(index))] if val])
            else:
                return '{'+str(personId)+'}'
        else:
            return ''

    def updateFilterEventResultTable(self):
        if self.chkFilterEventPurpose.isChecked():
            purposeId = self.cmbFilterEventPurpose.value()
        else:
            purposeId = None
        if purposeId is None:
            if self.chkFilterEventType.isChecked():
                eventTypeId = self.cmbFilterEventType.value()
            else:
                eventTypeId = None
            if eventTypeId:
                purposeId = getEventPurposeId(eventTypeId)
        filter = ('eventPurpose_id=\'%d\'' % purposeId) if purposeId else ''
        self.cmbFilterEventResult.setTable('rbResult', False, filter)


    #########################################################################

    def updateAmbCardInfo(self):
        clientId = self.currentClientId()
        if clientId:
            self.txtClientInfoBrowserAmbCard.setHtml(getClientBanner(clientId))
            self.tabAmbCardContent.setClientId(clientId)
        else:
            self.txtClientInfoBrowserAmbCard.setText('')
        self.actAmbCardEditClient.setEnabled(bool(clientId))
        self.actAmbCardRelationsClient.setEnabled(bool(clientId))
        QtGui.qApp.setCurrentClientId(clientId)

    #########################################################################

    def editAction(self, actionId):
        dialog = CActionEditDialog(self)
        try:
            dialog.load(actionId)
            if dialog.exec_():
                self.updateActionsList(self.__actionFilter, dialog.itemId())
                QtGui.qApp.emitCurrentClientInfoChanged()
                return dialog.itemId()
            else:
                self.updateActionInfo(actionId)
                self.updateClientsListRequest = True
            return None
        finally:
            dialog.destroy()
            sip.delete(dialog)
            del dialog


    def _setActionsOrderByColumn(self, column):
        table = self.getCurrentActionsTable()
        table.setOrder(column)
        self.updateActionsList(self.__actionFilter, table.currentItemId())


    def updateActionsList(self, filter, posToId=None):
#        """
#            в соответствии с фильтром обновляет список действий (мероприятий).
#        """
        self.__actionFilter = filter
        db = QtGui.qApp.db
        tableAction = db.table('Action')
        currentTable = self.getCurrentActionsTable()
        order = currentTable.order() if currentTable.order() else ['execDate DESC', tableAction['id'].name()]
        optionPropertyIdList = []
        tableEvent = db.table('Event')
        tableActionProperty = db.table('ActionProperty')
        tableActionPropertyType = db.table('ActionPropertyType')
        queryTable = tableAction.innerJoin(tableEvent, tableEvent['id'].eq(tableAction['event_id']) )
        cond = [tableAction['deleted'].eq(0),
                tableEvent['deleted'].eq(0),
                tableAction['actionType_id'].notInlist(getActionTypeIdListByFlatCode(u'inspection%'))]
        self.addEqCond(cond, tableAction, 'id', filter, 'id')
        self.addEqCond(cond, tableAction, 'createPerson_id', filter, 'createPersonId')
        self.addDateCond(cond, tableAction, 'createDatetime', filter, 'begCreateDate', 'endCreateDate')
        self.addEqCond(cond, tableAction, 'modifyPerson_id', filter, 'modifyPersonId')
        self.addDateCond(cond, tableAction, 'modifyDatetime', filter, 'begModifyDate', 'endModifyDate')
        if 'clientIds' in filter:
            clientIds = filter.get('clientIds')
            cond.append(tableEvent['client_id'].inlist(clientIds))
        if 'eventIds' in filter:
            eventIds = filter.get('eventIds')
            cond.append(tableAction['event_id'].inlist(eventIds))
        if 'isUrgent' in filter:
#            isUrgent = filter.get('isUrgent')
            cond.append(tableAction['isUrgent'].ne(0))
        if 'actionMKB' in filter:
            if 'actionMKBFrom' in filter:
                actionMKBFrom = filter.get('actionMKBFrom', None)
            if 'actionMKBTo' in filter:
                actionMKBTo = filter.get('actionMKBTo', None)
            if actionMKBFrom or actionMKBTo:
                if actionMKBFrom and not actionMKBTo:
                   actionMKBTo = u'U'
                elif not actionMKBFrom and actionMKBTo:
                    actionMKBFrom = u'A'
                cond.append(tableAction['MKB'].ge(actionMKBFrom))
                cond.append(tableAction['MKB'].le(actionMKBTo))
        self.addDateCond(cond, tableAction, 'directionDate', filter, 'begSetDate', 'endSetDate')
        self.addDateCond(cond, tableAction, 'plannedEndDate', filter, 'begPlannedEndDate', 'endPlannedEndDate')
        self.addDateTimeCond(cond, tableAction, 'endDate', filter, 'begExecDateTime', 'endExecDateTime')
        actionClass = self.tabWidgetActionsClasses.currentIndex()
        if 'actionTypeId' in filter:
            actionTypeIdList = getActionTypeDescendants(filter['actionTypeId'], actionClass)
            cond.append(tableAction['actionType_id'].inlist(actionTypeIdList))
        else:
            cond.append(tableAction['actionType_id'].name()+' IN (SELECT ActionType.id FROM ActionType WHERE ActionType.class=%d)' % actionClass)
# сделано без учета значений value в таблицах ActionProperty_Double, ActionProperty_Action и т.п., учитывается наличие записи в ActionProperty, т.к. считается, что при не заполнении не создается запись в ActionProperty.
        takenTissueJournal = filter.get('takenTissueJournal')
        if takenTissueJournal is not None:
            if takenTissueJournal:
                cond.append(tableAction['takenTissueJournal_id'].isNotNull())
            else:
                cond.append(tableAction['takenTissueJournal_id'].isNull())
        if self.chkTakeIntoAccountProperty.isChecked():
            if self.chkListProperty.isChecked() and not self.chkThresholdPenaltyGrade.isChecked():
                if 'booleanFilledProperty' not in filter:
                    queryTable = queryTable.leftJoin(tableActionPropertyType, tableActionPropertyType['actionType_id'].eq(tableAction['actionType_id']))
                optionPropertyIdList = self.getDataOptionPropertyChecked()
                if optionPropertyIdList != []:
                    cond.append(tableActionPropertyType['id'].inlist(optionPropertyIdList))
            if 'booleanFilledProperty' in filter:
                if filter.get('booleanFilledProperty') == True:
                    queryTable = queryTable.leftJoin(tableActionProperty, tableActionProperty['action_id'].eq(tableAction['id']))
                    queryTable = queryTable.leftJoin(tableActionPropertyType, tableActionPropertyType['id'].eq(tableActionProperty['type_id']))
                    cond.append(tableActionProperty['deleted'].eq(0))
                elif not self.chkThresholdPenaltyGrade.isChecked():
                    queryTable = queryTable.leftJoin(tableActionPropertyType, tableActionPropertyType['actionType_id'].eq(tableAction['actionType_id']))
                    cond.append(u'Action.id NOT IN (SELECT DISTINCT AP.action_id FROM ActionProperty AS AP WHERE AP.deleted = 0)')
            if self.chkThresholdPenaltyGrade.isChecked() and ('thresholdPenaltyGrade' in filter):
                filterPenaltyGrade = filter.get('thresholdPenaltyGrade')
                if filterPenaltyGrade:
                    whereOptionPropertyId = u''
                    if self.chkListProperty.isChecked():
                        optionPropertyIdList = self.getDataOptionPropertyChecked()
                        if optionPropertyIdList != []:
                            strPropertyIdList = u''
                            col =len(optionPropertyIdList) - 1
                            for propertyId in optionPropertyIdList:
                                strPropertyIdList += forceString(propertyId)
                                col -= 1
                                if col > 0:
                                    strPropertyIdList +=  u', '
                            whereOptionPropertyId = u'(APT.id IN (%s)) AND ' % (strPropertyIdList)
                    cond.append(u'SELECT SUM(APT.penalty) >= %s FROM ActionPropertyType AS APT WHERE %s(APT.id NOT IN (SELECT AP.type_id FROM ActionProperty AS AP WHERE AP.action_id = Action.id AND AP.deleted = 0 GROUP BY AP.type_id) AND (APT.actionType_id = Action.actionType_id))' % (filterPenaltyGrade, whereOptionPropertyId))
        addTableSetPerson = False
        addTableExecPerson = False
        if 'setPersonId' in filter:
            cond.append(tableAction['setPerson_id'].eq(filter['setPersonId']))
        else:
            if 'setSpecialityId' in filter:
                if not addTableSetPerson:
                    addTableSetPerson = True
                    tablePerson = db.table('Person')
                    queryTable = queryTable.join(tablePerson, tablePerson['id'].eq(tableAction['setPerson_id']))
                cond.append(tableAction['setPerson_id'].isNotNull())
                cond.append(tablePerson['speciality_id'].eq(filter['setSpecialityId']))
            if 'actionSetOrgStructureId' in filter:
                if not addTableSetPerson:
                    addTableSetPerson = True
                    tablePerson = db.table('Person')
                    queryTable = queryTable.join(tablePerson, tablePerson['id'].eq(tableAction['setPerson_id']))
                actionSetOrgStructureId = filter['actionSetOrgStructureId']
                actionSetOrgStructureIdList = db.getDescendants('OrgStructure', 'parent_id', actionSetOrgStructureId) if actionSetOrgStructureId else []
                cond.append(tableAction['setPerson_id'].isNotNull())
                cond.append(tablePerson['orgStructure_id'].isNotNull())
                cond.append(tablePerson['orgStructure_id'].inlist(actionSetOrgStructureIdList))
        if 'execPersonId' in filter:
            execPersonId = filter['execPersonId']
            if execPersonId == -1:
                cond.append(tableAction['person_id'].isNull())
            else:
                cond.append(tableAction['person_id'].eq(execPersonId))
        else:
            if 'execSetOrgStructureId' in filter:
                    if not addTableExecPerson:
                        addTableExecPerson = True
                        tablePersonExec = db.table('Person').alias('PersonExec')
                        queryTable = queryTable.join(tablePersonExec, tablePersonExec['id'].eq(tableAction['person_id']))
                    execSetOrgStructureId = filter['execSetOrgStructureId']
                    execSetOrgStructureIdList = db.getDescendants('OrgStructure', 'parent_id', execSetOrgStructureId) if execSetOrgStructureId else []
                    cond.append(tableAction['person_id'].isNotNull())
                    cond.append(tablePersonExec['orgStructure_id'].isNotNull())
                    cond.append(tablePersonExec['orgStructure_id'].inlist(execSetOrgStructureIdList))
            if 'execSpecialityId' in filter:
                    if not addTableExecPerson:
                        addTableExecPerson = True
                        tablePersonExec = db.table('Person').alias('PersonExec')
                        queryTable = queryTable.join(tablePersonExec, tablePersonExec['id'].eq(tableAction['person_id']))
                    cond.append(tableAction['person_id'].isNotNull())
                    cond.append(tablePersonExec['speciality_id'].eq(filter['execSpecialityId']))
        if 'assistantId' in filter:
            assistantId = filter['assistantId']
            if assistantId == -1:
                cond.append(tableAction['assistant_id'].isNull())
            else:
                cond.append(tableAction['assistant_id'].eq(assistantId))

        if 'uncoordinated' in filter and filter['uncoordinated']:
            tableActionType = db.table('ActionType')
            queryTable = queryTable.innerJoin(tableActionType, tableActionType['id'].eq(tableAction['actionType_id']))
            cond.append(tableActionType['deleted'].eq(0))
            cond.append(tableActionType['isRequiredCoordination'].eq(1))
            cond.append(tableAction['coordDate'].isNull())
        self.addEqCond(cond, tableAction, 'status', filter, 'status')

        payStatusFinanceCode = filter.get('payStatusFinanceCode', None)
        if payStatusFinanceCode:
            payStatusCode = filter.get('payStatusCode', 0)
            mask  = getPayStatusMaskByCode(payStatusFinanceCode)
            value = getPayStatusValueByCode(payStatusCode, payStatusFinanceCode)
            cond.append('((%s & %d) = %d)' % (tableAction['payStatus'].name(), mask, value))
        if 'actionJobTicketId' in filter:
            jobTicketId = filter['actionJobTicketId']
            locTableActionProperty = db.table('ActionProperty').alias('SubActionProperty')
            tableActionPropertyJobTicket = db.table('ActionProperty_Job_Ticket')
            subTableQuery = locTableActionProperty.leftJoin(tableActionPropertyJobTicket,
                                                   tableActionPropertyJobTicket['id'].eq(locTableActionProperty['id']))
            subCond = [locTableActionProperty['deleted'].eq(0),
                       tableActionPropertyJobTicket['value'].eq(jobTicketId),
                       locTableActionProperty['action_id'].eq(tableAction['id'])]
            cond.append(db.existsStmt(subTableQuery, subCond))
        if 'financeId' in filter:
            financeId = filter['financeId']
            if financeId:
                cond.append(tableAction['finance_id'].eq(financeId))
            else:
                cond.append(tableAction['finance_id'].isNull())

        payerId = filter.get('payerId')
        if payerId:
            tableContract = db.table('Contract')
            queryTable = queryTable.leftJoin(tableContract,
                                             'IF(%s, %s, %s)' % (
                                                tableAction['contract_id'].isNotNull(),
                                                tableContract['id'].eq(tableAction['contract_id']),
                                                tableContract['id'].eq(tableEvent['contract_id']),
                                                                )

                                             )
            cond.append(tableContract['payer_id'].eq(payerId))

        if 'contractId' in filter:
            contractId = filter['contractId']
            if contractId:
                cond.append(tableAction['contract_id'].eq(contractId))
            else:
                cond.append(tableAction['contract_id'].isNull())

        if 'Client' in order:
            tableClient = db.table('Client')
            queryTable = queryTable.leftJoin(tableClient, tableClient['id'].eq(tableEvent['client_id']))

        if 'vrbSetPersonWithSpeciality.name' in order:
            tableSetPerson = db.table('vrbPersonWithSpeciality').alias('vrbSetPersonWithSpeciality')
            queryTable = queryTable.leftJoin(tableSetPerson, tableSetPerson['id'].eq(tableAction['setPerson_id']))

        if 'vrbPersonWithSpeciality.name' in order:
            tablePersonSP = db.table('vrbPersonWithSpeciality')
            queryTable = queryTable.leftJoin(tablePersonSP, tablePersonSP['id'].eq(tableAction['person_id']))

        if 'rbFinance.code' in order:
            tableFinance = db.table('rbFinance')
            queryTable = queryTable.leftJoin(tableFinance, tableFinance['id'].eq(tableAction['finance_id']))

        if 'vrbContract.code' in order:
            tableVRBContract = db.table('vrbContract')
            queryTable = queryTable.leftJoin(tableVRBContract, tableVRBContract['id'].eq(tableAction['contract_id']))

        try:
            QtGui.QApplication.setOverrideCursor(QtGui.QCursor(Qt.WaitCursor))
            idList = db.getDistinctIdList(queryTable,
                           tableAction['id'].name(),
                           cond,
                           order)
            self.setActionsIdList(idList, posToId)
        finally:
            QtGui.QApplication.restoreOverrideCursor()
        hideClientInfo = 'clientIds' in filter and len(filter['clientIds']) == 1
        table = self.getCurrentActionsTable()
        tableMC = self.getCurrentExpertMedicalCommissionTable()
        for i in xrange(3):
            table.setColumnHidden(i, hideClientInfo)
            tableMC.setColumnHidden(i, hideClientInfo)
        self.updateClientsListRequest = self.updateClientsListRequest or not hideClientInfo


    def getActionFilterAsText(self):
        db = QtGui.qApp.db
        filter  = self.__actionFilter
        resList = []

        actionsFilterType = filter['actionsFilterType']
        if actionsFilterType in (0, 1):
            clientIds = filter.get('clientIds', None)
            if clientIds and len(clientIds) == 1:
                resList.append((u'пациент', getClientMiniInfo(clientIds[0])))
            elif clientIds and len(clientIds) > 1:
                resList.append((u'список пациентов', u'из вкладки'))
        elif actionsFilterType in (2, 3):
            clientIds = self.__eventFilter.get('clientIds', None)
            if clientIds and len(clientIds) == 1:
                resList.append((u'пациент', getClientMiniInfo(clientIds[0])))
            elif clientIds and len(clientIds) > 1:
                resList.append((u'список пациентов', u'по списку осмотров'))
        else:
            resList.append((u'список пациентов', u'полный'))

        tmpList = [
            ('begSetDate', u'Дата назначения с', forceString),
            ('endSetDate', u'Дата назначения по', forceString),
            ('actionTypeId', u'Тип мероприятия',
                lambda id: forceString(db.translate('ActionType', 'id', id, 'name'))),
            ('setSpecialityId', u'Специальность назначившего',
                lambda id: forceString(db.translate('rbSpeciality', 'id', id, 'name'))),
            ('personId', u'Назначил',
                lambda id: forceString(db.translate('vrbPersonWithSpeciality', 'id', id, 'name'))),
            ('isUrgent',   u'Срочно', lambda dummy: u'+'),
            ('begPlannedEndDate', u'Плановая дата выполнения с', forceString),
            ('endPlannedEndDate', u'Плановая дата окончания по', forceString),
            ('begExecDateTime', u'Дата и время выполнения с', forceString),
            ('endExecDateTime', u'Дата и время выполнения по', forceString),
            ]
        for (x, y, z) in tmpList:
            convertFilterToTextItem(resList, filter, x, y, z)
        return '\n'.join([ ': '.join(item) for item in resList])


    def getCurrentActionsTable(self):
        index = self.tabWidgetActionsClasses.currentIndex()
        return [self.tblActionsStatus, self.tblActionsDiagnostic, self.tblActionsCure, self.tblActionsMisc][index]


    def getCurrentExpertMedicalCommissionTable(self):
        index = self.tabExpertMedicalCommissionWidget.currentIndex()
        return [self.tblExpertDirectionsMC, self.tblExpertProtocolsMC, self.tblExpertMedicalSocialInspection][index]


    def currentActionId(self):
        return self.getCurrentActionsTable().currentItemId()


    def setActionsIdList(self, idList, posToId):
        self.getCurrentActionsTable().setIdList(idList, posToId)


    def focusActions(self):
        self.getCurrentActionsTable().setFocus(Qt.TabFocusReason)


    def updateActionInfo(self, actionId):
        db = QtGui.qApp.db
        tableAction = db.table('Action')
        tableEvent = db.table('Event')
        table = tableAction.leftJoin(tableEvent, tableEvent['id'].eq(tableAction['event_id']))
        record = db.getRecord(table, ['Event.client_id',], actionId)
#                              ['Action.createDatetime','Action.createPerson_id', 'Action.modifyDatetime', 'Action.modifyPerson_id', 'Event.client_id', 'Action.payStatus', 'Action.note'], actionId)
        if record:
#            createDatetime = dateTimeToString(record.value('createDatetime').toDateTime())
#            createPersonId = forceRef(record.value('createPerson_id'))
#            modifyDatetime = dateTimeToString(record.value('modifyDatetime').toDateTime())
#            modifyPersonId = forceRef(record.value('modifyPerson_id'))
            clientId       = forceRef(record.value('client_id'))
#            note           = forceString(record.value('note'))
#            payStatus      = forceInt(record.value('payStatus'))
        else:
#            createDatetime = ''
#            createPersonId = None
#            modifyDatetime = ''
#            modifyPersonId = None
            clientId       = None
#            note           = ''
#            payStatus      = 0

        if not clientId:
            clientId = self.selectedClientId()
        if clientId:
            self.txtClientInfoBrowserActions.setHtml(getClientBanner(clientId))
        else:
            self.txtClientInfoBrowserActions.setText('')
        self.actActionEditClient.setEnabled(bool(clientId))
        self.actActionsRelationsClient.setEnabled(bool(clientId))
        QtGui.qApp.setCurrentClientId(clientId)
        self.tabAmbCardContent.setClientId(clientId)

#        table = db.table('Action')
#        idList = db.getIdList(table, 'id', table['event_id'].eq(eventId), ['id'])
#        self.tblEventActions.setIdList(idList)

#        self.lblEventIdValue.setText(str(eventId) if eventId else '')
#        self.lblEventExternalIdValue.setText(externalId)
#        self.lblEventCreateDateTimeValue.setText(createDatetime)
#        self.lblEventCreatePersonValue.setText(self.getPersonText(createPersonId))
#        self.lblEventModifyDateTimeValue.setText(modifyDatetime)
#        self.lblEventModifyPersonValue.setText(self.getPersonText(modifyPersonId))
#        self.lblEventNoteValue.setText(note)
#        self.lblEventPayStatusValue.setText(payStatusText(payStatus))


    def updateMedicalCommissionInfo(self, actionId):
        db = QtGui.qApp.db
        tableAction = db.table('Action')
        tableEvent = db.table('Event')
        table = tableAction.leftJoin(tableEvent, tableEvent['id'].eq(tableAction['event_id']))
        record = db.getRecord(table, ['Event.client_id',], actionId)
        if record:
            clientId = forceRef(record.value('client_id'))
        else:
            clientId = None
        if not clientId:
            clientId = self.selectedClientId()
        if clientId:
            self.txtClientInfoBrowserActions.setHtml(getClientBanner(clientId))
        else:
            self.txtClientInfoBrowserActions.setText('')
        QtGui.qApp.setCurrentClientId(clientId)
        self.tabAmbCardContent.setClientId(clientId)


    def updateVisitInfo(self, visitId):
        clientId = None
        db = QtGui.qApp.db
        tableVisit = db.table('Visit')
        tableEvent = db.table('Event')
        table = tableVisit.innerJoin(tableEvent, tableEvent['id'].eq(tableVisit['event_id']))
        record = db.getRecord(table, ['Event.client_id',], visitId)
        if record:
            clientId       = forceRef(record.value('client_id'))
        else:
            clientId       = None
#        if not clientId:
#            clientId = self.selectedClientId()
        if clientId:
            self.txtClientInfoBrowserVisits.setHtml(getClientBanner(clientId))
        else:
            self.txtClientInfoBrowserVisits.setText('')
        self.actActionEditClient.setEnabled(bool(clientId))
        self.actVisitRelationsClient.setEnabled(bool(clientId))
        QtGui.qApp.setCurrentClientId(clientId)

    def setClientSort(self, col):
        if col in (0, 3):
            self.clientsSortingDest = 'ASC' if self.clientsSortingDest == 'DESC' else 'DESC'
            if col != self.clientsSortingCol:
                self.clientsSortingDest = 'ASC'
            self.clientsSortingCol = col
            self.updateClientsList(self.__filter)

    def setXXXSort(self, col):
        if col in (0, 1, 4, 5, 6, 7, 8, 9, 10):
            sortAscendingList = self.getSortAscendingList()
            sortAscending = not sortAscendingList.get(col, True)
            self.setXXXSortAscendingList(col, sortAscending)
            sortOrder = ' ASC' if sortAscending else ' DESC'
            if col == 0:
                order = u'Client.lastName%s, Client.firstName%s, Client.patrName%s' % (sortOrder, sortOrder, sortOrder)
            elif col == 1:
                order = u'Client.birthDate%s' % sortOrder
            if col in (4, 5, 7, 8, 9):
                nameField = self.modelExpertTempInvalid.cols()[col].fields()[0]
                table = self.modelExpertTempInvalid.table()
                order = table[str(nameField)].name() + sortOrder
            elif col == 6:
                order = u'rbTempInvalidReason.code%s, rbTempInvalidReason.name%s' % (sortOrder, sortOrder)
            elif col == 10:
                order = u'vrbPerson.code%s, vrbPerson.name' % sortOrder
            header = self.tblExpertTempInvalid.horizontalHeader()
            header.setXXXSortIndicatorShown(True)
            header.setXXXSortIndicator(col, Qt.AscendingOrder if sortAscending else Qt.DescendingOrder)
            self.renewXXXListAndSetTo(self.currentTempInvalidId(), [order])


    def renewXXXListAndSetTo(self, itemId=None, order=[]):
        self.updateTempInvalidList(self.__expertFilter, itemId, order)


    def setXXXSortAscendingList(self, col, check):
        self.sortAscendingList[col] = check


    def loadSortAscendingList(self):
        cols = self.modelExpertTempInvalid.cols()
        for col, field in enumerate(cols):
            self.setXXXSortAscendingList(col, True)


    def getSortAscendingList(self):
        return self.sortAscendingList


    def updateTempInvalidList(self, filter, posToId=None, order=[]):
        # в соответствии с фильтром обновляет список документов вр. нетрудоспособности.
        self.__expertFilter = filter
        db = QtGui.qApp.db
        if filter.get('linked', False):
            table = db.table('TempInvalid').alias('ti')
            mainTable = db.table('TempInvalid')
            queryTable = mainTable.join(table,
                                        db.joinAnd([mainTable['client_id'].eq(table['client_id']),
                                                    mainTable['caseBegDate'].eq(table['caseBegDate'])
                                                   ]))
        else:
            table = db.table('TempInvalid')
            mainTable = table
            queryTable = table
        tableDiagnosis = db.table('Diagnosis')
        tableTempInvalidPeriod = db.table('TempInvalid_Period')
        tableTempInvalidDocument = db.table('TempInvalidDocument')
        queryTable = queryTable.leftJoin(tableDiagnosis, tableDiagnosis['id'].eq(table['diagnosis_id']))
        tempInvalidType = self.tabWidgetTempInvalidTypes.currentIndex()
        cond = [ mainTable['deleted'].eq(0),
                 mainTable['type'].eq(tempInvalidType)
               ]
        self.addEqCond(cond, table, 'id', filter, 'id')
        self.addEqCond(cond, table, 'doctype_id', filter, 'docTypeId')
        if filter.get('linked', False):
            cond.append(mainTable['prev_id'].isNull())
        if self.chkPrimaryOrDuble.isChecked() or self.chkFilterExpertSerial.isChecked() or self.chkFilterExpertNumber.isChecked():
            queryTable = queryTable.innerJoin(tableTempInvalidDocument, tableTempInvalidDocument['master_id'].eq(table['id']))
        if self.chkPrimaryOrDuble.isChecked():
            cond.append(tableTempInvalidDocument['duplicate'].eq(filter.get('primaryOrDuble', 0)))
        if self.chkFilterExpertSerial.isChecked():
            self.addLikeCond(cond, tableTempInvalidDocument, 'serial', filter, 'serial')
        if self.chkFilterExpertNumber.isChecked():
            self.addLikeCond(cond, tableTempInvalidDocument, 'number', filter, 'number')
        self.addEqCond(cond, table, 'tempInvalidReason_id', filter, 'reasonId')
        self.addDateCond(cond, table, 'begDate', filter, 'begBegDate', 'endBegDate')
        self.addDateCond(cond, table, 'endDate', filter, 'begEndDate', 'endEndDate')
        self.addEqCond(cond, table, 'person_id', filter, 'personId')
        self.addRangeCond(cond, tableDiagnosis, 'MKB', filter, 'begMKB', 'endMKB')
        self.addEqCond(cond, table, 'closed', filter, 'closed')
        if 'expertOrgStructureId' in filter or 'expertOrgStructureId' in filter:
            tablePerson = db.table('Person')
            queryTable = queryTable.leftJoin(tablePerson, tablePerson['id'].eq(table['person_id']))
            cond.append(tablePerson['deleted'].eq(0))
            if 'expertOrgStructureId' in filter:
                expertOrgStructureId = filter.get('expertOrgStructureId', None)
                if expertOrgStructureId:
                    orgStructureIdList = db.getDescendants('OrgStructure', 'parent_id', expertOrgStructureId)
                    if orgStructureIdList:
                        cond.append(tablePerson['orgStructure_id'].inlist(orgStructureIdList))
            if 'expertSpecialityId' in filter:
                expertSpecialityId = filter.get('expertSpecialityId', None)
                if expertSpecialityId:
                    cond.append(tablePerson['speciality_id'].eq(expertSpecialityId))
        if 'begDuration' in filter:
            cond.append(table['duration'].ge(filter['begDuration']))
        if 'endDuration' in filter:
            endDuration = filter['endDuration']
            if endDuration:
                cond.append(table['duration'].le(endDuration))
        self.addEqCond(cond, table, 'insuranceOfficeMark', filter, 'insuranceOfficeMark')
        self.addEqCond(cond, table, 'createPerson_id', filter, 'createPersonId')
        self.addDateCond(cond, table, 'createDatetime', filter, 'begCreateDate', 'endCreateDate')
        self.addEqCond(cond, table, 'modifyPerson_id', filter, 'modifyPersonId')
        self.addDateCond(cond, table, 'modifyDatetime', filter, 'begModifyDate', 'endModifyDate')
        if 'receiverIds' in filter:
            receiverIds = filter['receiverIds']
            cond.append(mainTable['client_id'].inlist(receiverIds))
        elif 'receiverEventIds' in filter:
            receiverEventIds = filter.get('receiverEventIds')
            tableEvent = db.table('Event')
            queryTable = queryTable.innerJoin(tableEvent, tableEvent['client_id'].eq(table['client_id']))
            cond.append(tableEvent['id'].inlist(receiverEventIds))
            cond.append(tableEvent['deleted'].eq(0))
        elif 'clientIds' in filter:
            clientIds = filter['clientIds']
            tableClient = db.table('Client').alias('Client')
            if not self.chkPrimaryOrDuble.isChecked():
                queryTable = queryTable.innerJoin(tableTempInvalidDocument, tableTempInvalidDocument['master_id'].eq(table['id']))
            queryTable = queryTable.innerJoin(tableClient, tableClient['id'].inlist(clientIds))
            cond.append(db.joinOr([tableClient['id'].eq(tableTempInvalidDocument['clientPrimum_id']), tableClient['id'].eq(tableTempInvalidDocument['clientSecond_id'])]))
            cond.append(tableTempInvalidDocument['deleted'].eq(0))
            cond.append(tableClient['deleted'].eq(0))
        elif 'eventIds' in filter:
            eventIds = filter.get('eventIds')
            tableEvent = db.table('Event')
            if not self.chkPrimaryOrDuble.isChecked() and not self.chkFilterExpertSerial.isChecked() and not self.chkFilterExpertNumber.isChecked():
                queryTable = queryTable.innerJoin(tableTempInvalidDocument, tableTempInvalidDocument['master_id'].eq(table['id']))
            queryTable = queryTable.innerJoin(tableEvent, tableEvent['id'].inlist(eventIds))
            cond.append(tableEvent['deleted'].eq(0))
            cond.append(tableTempInvalidDocument['deleted'].eq(0))
            cond.append(db.joinOr([tableEvent['client_id'].eq(tableTempInvalidDocument['clientPrimum_id']), tableEvent['client_id'].eq(tableTempInvalidDocument['clientSecond_id'])]))
            if order:
                tableClient = db.table('Client').alias('Client')
                cond.append(tableClient['deleted'].eq(0))
                queryTable = queryTable.innerJoin(tableClient, tableClient['id'].eq(tableEvent['client_id']))
        if self.chkFilterExpertExternal.isChecked():
            isExternal = filter.get('isExternal', 0)
            if isExternal:
                stmtExternal = u'''EXISTS(SELECT TP.id FROM TempInvalid_Period AS TP
                WHERE TP.master_id = %s AND TP.isExternal = 1 AND TP.begDate = (SELECT TP2.begDate
                FROM TempInvalid_Period AS TP2 WHERE TP2.master_id = %s ORDER BY TP2.begDate ASC LIMIT 1)
                ORDER BY begDate ASC LIMIT 1)'''%(table.name() + '.id', table.name() + '.id')
                cond.append((u'NOT %s'%stmtExternal) if isExternal == 1 else stmtExternal)
        try:
            QtGui.QApplication.setOverrideCursor(QtGui.QCursor(Qt.WaitCursor))
            if table == mainTable and (not 'clientIds' in filter and not 'eventIds' in filter):
                getIdList = db.getIdList
            else:
                getIdList = db.getDistinctIdList
            if order:
                tableVRBPerson = db.table('vrbPerson')
                tableRBTempInvalidReason = db.table('rbTempInvalidReason')
                if not 'clientIds' in filter and not 'eventIds' in filter:
                    tableClient = db.table('Client')
                    queryTable = queryTable.innerJoin(tableClient, tableClient['id'].eq(table['client_id']))
                queryTable = queryTable.leftJoin(tableVRBPerson, tableVRBPerson['id'].eq(table['person_id']))
                queryTable = queryTable.leftJoin(tableRBTempInvalidReason, tableRBTempInvalidReason['id'].eq(table['tempInvalidReason_id']))
                order.append(mainTable['endDate'].name()+' DESC')
            else:
                order.append(mainTable['endDate'].name()+' DESC')
                order.append(mainTable['id'].name())
            idList = getIdList(queryTable,
                           mainTable['id'].name(),
                           cond,
                           order)
            self.setExpertIdList(idList, posToId)
            documentIdList = []
            if idList:
                cond = [tableTempInvalidDocument['master_id'].inlist(idList),
                        tableTempInvalidDocument['deleted'].eq(0),
                        ]
                if self.chkPrimaryOrDuble.isChecked():
                    cond.append(tableTempInvalidDocument['duplicate'].eq(filter.get('primaryOrDuble', 0)))
                if self.chkFilterExpertSerial.isChecked():
                    self.addLikeCond(cond, tableTempInvalidDocument, 'serial', filter, 'serial')
                if self.chkFilterExpertNumber.isChecked():
                    self.addLikeCond(cond, tableTempInvalidDocument, 'number', filter, 'number')
                if filter.get('linked', False):
                    cond.append(tableTempInvalidDocument['prev_id'].isNull())
                documentIdList = db.getDistinctIdList(tableTempInvalidDocument,
                                              [tableTempInvalidDocument['id']],
                                              cond,
                                              u'TempInvalidDocument.idx, TempInvalidDocument.issueDate')
            self.getCurrentExpertRelationDocumentsExTable().setIdList([])
            self.getCurrentTempDocumentsExTable().setIdList(documentIdList)
        finally:
            QtGui.QApplication.restoreOverrideCursor()
        expertFilterType = filter['expertFilterType']
        hideClientInfo = expertFilterType == 0 or expertFilterType == 1 and len(filter['receiverIds']) == 1
        table = self.getCurrentExpertTable()
        for i in range(3):
            table.setColumnHidden(i, hideClientInfo)
        self.getCurrentTempDocumentsTable().setColumnHidden(0, hideClientInfo)
        self.getCurrentTempDocumentsExTable().setColumnHidden(0, hideClientInfo)
        if filter.get('linked', False):
            self.getCurrentExpertRelationDocumentsExTable().setColumnHidden(0, hideClientInfo)
            for i in range(3):
                self.getCurrentExpertRelationTable().setColumnHidden(i, hideClientInfo)


    def updateTempInvalidInfo(self, tempInvalidId, noPrev=False):
        isFilterExpertLinked = self.chkFilterExpertLinked.isChecked()
        db = QtGui.qApp.db
        tableTempInvalid = db.table('TempInvalid')
        tablePeriod = db.table('TempInvalid_Period')
        if isFilterExpertLinked:
            begDateMin = None
            endDateMax = None
            countDays = 0
            currentTable = self.getCurrentExpertRelationTable()
            if noPrev:
                tempInvalidRelationIdList = []
                if tempInvalidId:
                    id = tempInvalidId
                    while id:
                        record = db.getRecordEx(tableTempInvalid, ['id'], [tableTempInvalid['prev_id'].eq(id), tableTempInvalid['deleted'].eq(0)], tableTempInvalid['begDate'].name())
                        id = forceRef(record.value('id')) if record else None
                        if id and id not in tempInvalidRelationIdList:
                            tempInvalidRelationIdList.append(id)
                            recordPeriods = db.getRecordListGroupBy(tablePeriod, ['MIN(begDate) AS begDate, MAX(endDate) AS endDate'], [tablePeriod['master_id'].eq(id)], 'master_id', 'endDate')
                            for recordPeriod in recordPeriods:
                                begDate = forceDate(recordPeriod.value('begDate'))
                                begDateMin = min(begDateMin, begDate) if begDateMin else begDate
                                endDateMax = max(endDateMax, forceDate(recordPeriod.value('endDate')))
                currentTable.setIdList(tempInvalidRelationIdList)
            expertFilterType = self.cmbFilterExpert.currentIndex()
            hideClientInfo = expertFilterType == 0 or expertFilterType == 1 and len(self.modelClients.idList()) == 1
            for i in range(3):
                currentTable.setColumnHidden(i, hideClientInfo)
        self.setClientInfoBrowserExpert(tempInvalidId)
        idList = []
        if isFilterExpertLinked and noPrev:
            if tempInvalidId:
                recordPeriods = db.getRecordList(tablePeriod, ['id, begDate, endDate'], [tablePeriod['master_id'].eq(tempInvalidId)], 'endDate')
                for recordPeriod in recordPeriods:
                    id = forceRef(recordPeriod.value('id'))
                    if id and id not in idList:
                        idList.append(id)
                        begDate = forceDate(recordPeriod.value('begDate'))
                        begDateMin = min(begDateMin, begDate) if begDateMin else begDate
                        endDateMax = max(endDateMax, forceDate(recordPeriod.value('endDate')))
            if begDateMin and endDateMax:
                countDays = begDateMin.daysTo(endDateMax) + 1
            index = self.tabWidgetTempInvalidTypes.currentIndex()
            textCount = u', ВУТ: ' + (begDateMin.toString('dd.MM.yyyy') if begDateMin else u'') + u' - ' + (endDateMax.toString('dd.MM.yyyy') if endDateMax else u'') + u', ' + formatDays(countDays)
            if index == self.tabWidgetTempInvalidTypes.indexOf(self.tabExpertTempInvalidDetalsWidget):
                infoBeds, infoAddressOrgStructure = self.getFilterInfoBed()
                self.lblExpertTempInvalidCount.setText(formatRecordsCount(self.modelExpertTempInvalid.rowCount())+infoBeds+infoAddressOrgStructure + textCount)
            elif index == self.tabWidgetTempInvalidTypes.indexOf(self.tabExpertDisabilityDetailsWidget):
                self.lblExpertDisabilityCount.setText(formatRecordsCount(self.modelExpertDisability.rowCount()) + textCount)
            elif index == self.tabWidgetTempInvalidTypes.indexOf(self.tabExpertVitalRestrictionDetailsWidget):
                self.lblExpertVitalRestrictionCount.setText(formatRecordsCount(self.modelExpertVitalRestriction.rowCount()) + textCount)
        else:
            if tempInvalidId:
                idList = db.getDistinctIdList(tablePeriod, 'id', tablePeriod['master_id'].eq(tempInvalidId), ['endDate'])
        self.getCurrentTempPeriodsTable().setIdList(idList)

        idList = []
        if tempInvalidId:
            table = db.table('TempInvalidDocument')
            cond = [table['master_id'].eq(tempInvalidId),
                    table['deleted'].eq(0)
                    ]
            idList = db.getDistinctIdList(table, 'id', cond, ['id'])
        self.getCurrentTempDocumentsTable().setIdList(idList)


    def getExpertFilterAsText(self):
        db = QtGui.qApp.db
        filter  = self.__expertFilter
        resList = []

        expertFilterType = filter['expertFilterType']
        if expertFilterType in (0, 1):
            clientIds = filter.get('clientIds', None)
            if clientIds and len(clientIds) == 1:
                resList.append((u'Пациент', getClientMiniInfo(clientIds[0])))
            elif clientIds and len(clientIds) > 1:
                resList.append((u'Список пациентов', u'из вкладки'))
        else:
            resList.append((u'Список пациентов', u'полный'))

        tmpList = [
            ('id',         u'Идентифиатор записи', lambda id: '%d'%id),
            ('docTypeId',  u'Тип документа',       lambda id: forceString(db.translate('rbTempInvalidDocument', 'id', id, 'name'))),
            ('serial',     u'Серия',               forceString),
            ('number',     u'Номер',               forceString),
            ('reasonId',   u'Причина нетрудоспособности', lambda id: forceString(db.translate('rbTempInvalidReason', 'id', id, 'name'))),
            ('begBegDate', u'Дата начала с',       forceString),
            ('endBegDate', u'Дата начала по',      forceString),
            ('begEndDate', u'Дата окончания с',    forceString),
            ('endEndDate', u'Дата окончания по',   forceString),
            ('personId',   u'Врач',                lambda id: forceString(db.translate('vrbPersonWithSpeciality', 'id', id, 'name'))),
            ('begMKB',     u'Код МКБ с',           forceString),
            ('endMKB',     u'Код МКБ по',          forceString),
            ('closed',     u'состояние',           lambda closed: [u'открыт', u'закрыт', u'продлён', u'передан'][closed]),
            ('begDuration',u'длительность с',      lambda v: '%d'%v),
            ('endDuration',u'длительность по',     lambda v: '%d'%v),
            ('insuranceOfficeMark', u'отметка страхового стола', lambda i: [u'без отметки', u'с отметкой']),
            ('createPerson_id', u'Автор записи',   lambda id: forceString(db.translate('vrbPersonWithSpeciality', 'id', id, 'name'))),
            ('begCreateDate', u'Дата создания с',  forceString),
            ('endCreateDate', u'Дата создания по', forceString),
            ('modifyPerson_id', u'Запись изменил', lambda id: forceString(db.translate('vrbPersonWithSpeciality', 'id', id, 'name'))),
            ('begModifyDate', u'Дата изменения с', forceString),
            ('endModifyDate', u'Дата изменения по',forceString),
            ]
        for (x, y, z) in tmpList:
            convertFilterToTextItem(resList, filter, x, y, z)
        return '\n'.join([ ': '.join(item) for item in resList])


    def getExpertMCFilterAsText(self):
        db = QtGui.qApp.db
        filter  = self.__expertMCFilter
        resList = []
        actionsFilterType = filter['actionsFilterType']
        if actionsFilterType in (0, 1):
            clientIds = filter.get('clientIds', None)
            if clientIds and len(clientIds) == 1:
                resList.append((u'Пациент', getClientMiniInfo(clientIds[0])))
            elif clientIds and len(clientIds) > 1:
                resList.append((u'Список пациентов', u'из вкладки'))
        else:
            resList.append((u'Список пациентов', u'полный'))
        tmpList = [
            ('id',                   u'Идентифиатор записи',  lambda id: '%d'%id),
            ('expertiseId',          u'Экспертиза',           lambda expertiseId: expertiseClass[expertiseId][1]),
            ('expertiseTypeId',      u'Тип экспертизы',       lambda expertiseTypeId: forceString(expertiseTypeId[1])),
            ('expertNumberExpertise',u'Номер'   ,             forceString),
            ('expertNumber',         u'Номер ЛН',             forceString),
            ('setPersonId',          u'Назначивший',          lambda id: forceString(db.translate('vrbPersonWithSpeciality', 'id', id, 'name'))),
            ('begDirectDate',        u'Направление на ВК с',  forceString),
            ('endDirectDate',        u'Направление на ВК по', forceString),
            ('expertOrgStructureId', u'Подразделение',        lambda id: forceString(db.translate('OrgStructure', 'id', id, 'name'))),
            ('expertSpecialityId',   u'Специальность',        lambda id: forceString(db.translate('rbSpeciality', 'id', id, 'name'))),
            ('begMKB',               u'Код МКБ с',            forceString),
            ('endMKB',               u'Код МКБ по',           forceString),
            ('actionStatus',         u'состояние',            lambda actionStatus: CActionStatus.names[actionStatus]),
            ('expertiseCharacterId', u'Характеристика экспертизы',lambda id: forceString(db.translate('rbMedicalBoardExpertiseCharacter', 'id', id, 'name'))),
            ('expertiseKindId',      u'Вид экспертизы',           lambda id: forceString(db.translate('rbMedicalBoardExpertiseKind', 'id', id, 'name'))),
            ('expertiseObjectId',    u'Предмет экспертизы',       lambda id: forceString(db.translate('rbMedicalBoardExpertiseObject', 'id', id, 'name'))),
            ('expertiseArgumentId',  u'Обоснование экспертизы',   lambda id: forceString(db.translate('rbMedicalBoardExpertiseArgument', 'id', id, 'name'))),
            ('begExecDate',          u'Экспертиза ВК с',      forceString),
            ('endExecDate',          u'Экспертиза ВК по',     forceString),
            ('expertId',             u'Эксперт',              lambda id: forceString(db.translate('vrbPersonWithSpeciality', 'id', id, 'name'))),
            ('createPerson_id',      u'Автор записи',         lambda id: forceString(db.translate('vrbPersonWithSpeciality', 'id', id, 'name'))),
            ('begCreateDate',        u'Дата создания с',      forceString),
            ('endCreateDate',        u'Дата создания по',     forceString),
            ('modifyPerson_id',      u'Запись изменил',       lambda id: forceString(db.translate('vrbPersonWithSpeciality', 'id', id, 'name'))),
            ('begModifyDate',        u'Дата изменения с',     forceString),
            ('endModifyDate',        u'Дата изменения по',    forceString),
            ]
        for (x, y, z) in tmpList:
            convertFilterToTextItem(resList, filter, x, y, z)
        return '\n'.join([ ': '.join(item) for item in resList])


    def getCurrentExpertTypesTabWidget(self):
        index = self.tabWidgetTempInvalidTypes.currentIndex()
        return [self.tabExpertTempInvalidDetalsWidget, self.tabExpertDisabilityDetailsWidget, self.tabExpertVitalRestrictionDetailsWidget, self.getCurrentExpertMedicalCommissionTable()][index]


    def getCurrentExpertTypesTabWidgetDetails(self):
        index = self.tabWidgetTempInvalidTypes.currentIndex()
        return [self.tabExpertTempInvalidDetals, self.tabExpertDisabilityDetails, self.tabExpertVitalRestrictionDetails, self.getCurrentExpertMedicalCommissionTable()][index]


    def getCurrentExpertMedicalCommissionTable(self):
        index = self.tabExpertMedicalCommissionWidget.currentIndex()
        return [self.tblExpertDirectionsMC, self.tblExpertProtocolsMC, self.tblExpertMedicalSocialInspection][index]


    def getCurrentExpertTable(self):
        index = self.tabWidgetTempInvalidTypes.currentIndex()
        return [self.tblExpertTempInvalid, self.tblExpertDisability, self.tblExpertVitalRestriction, self.getCurrentExpertMedicalCommissionTable()][index]


    def getCurrentExpertLabelCount(self):
        index = self.tabWidgetTempInvalidTypes.currentIndex()
        return [self.lblExpertTempInvalidCount, self.lblExpertDisabilityCount, self.lblExpertVitalRestrictionCount, self.getCurrentExpertMedicalCommissionTable()][index]


    def getCurrentExpertRelationTable(self):
        index = self.tabWidgetTempInvalidTypes.currentIndex()
        return [self.tblExpertTempInvalidRelation, self.tblExpertDisabilityRelation, self.tblExpertVitalRestrictionRelation, self.getCurrentExpertMedicalCommissionTable()][index]


    def getCurrentExpertRelationDocumentsExTable(self):
        index = self.tabWidgetTempInvalidTypes.currentIndex()
        return [self.tblExpertTempInvalidRelationDocumentsEx, self.tblExpertDisabilityRelationDocumentsEx, self.tblExpertVitalRestrictionRelationDocumentsEx, self.getCurrentExpertMedicalCommissionTable()][index]


    def getCurrentTempPeriodsTable(self):
        index = self.tabWidgetTempInvalidTypes.currentIndex()
        return [self.tblExpertTempInvalidPeriods, self.tblExpertDisabilityPeriods, self.tblExpertVitalRestrictionPeriods, self.getCurrentExpertMedicalCommissionTable()][index]


    def getCurrentTempDocumentsTable(self):
        index = self.tabWidgetTempInvalidTypes.currentIndex()
        return [self.tblExpertTempInvalidDocuments, self.tblExpertDisabilityDocuments, self.tblExpertVitalRestrictionDocuments, self.getCurrentExpertMedicalCommissionTable()][index]


    def getCurrentTempDocumentsExTable(self):
        index = self.tabWidgetTempInvalidTypes.currentIndex()
        return [self.tblExpertTempInvalidDocumentsEx, self.tblExpertDisabilityDocumentsEx, self.tblExpertVitalRestrictionDocumentsEx, self.getCurrentExpertMedicalCommissionTable()][index]


    def getCurrentTempDocumentsExPeriodsTable(self):
        index = self.tabWidgetTempInvalidTypes.currentIndex()
        return [self.tblExpertTempInvalidPeriodsDocuments, self.tblExpertDisabilityPeriodsDocuments, self.tblExpertVitalRestrictionPeriodsDocuments, self.getCurrentExpertMedicalCommissionTable()][index]


    def currentTempInvalidId(self):
        return self.getCurrentExpertTable().currentItemId()


    def currentTempInvalidDocumentsId(self):
        return self.getCurrentTempDocumentsTable().currentItemId()


    def setExpertIdList(self, idList, posToId):
        self.getCurrentExpertTable().setIdList(idList, posToId)


    @pyqtSignature('bool')
    def on_chkFilterExpertExpertiseMC_toggled(self, checked):
        self.initExpertiseMCComboBox()


    @pyqtSignature('int')
    def on_cmbFilterExpertExpertiseMC_currentIndexChanged(self, index):
        self.initExpertiseMCComboBox()
        self.cmbFilterExpertExpertiseCharacterMC.setTable('rbMedicalBoardExpertiseCharacter', False, filter='class = %s'%(index))


    @pyqtSignature('int')
    def on_cmbFilterExpertExpertiseCharacterMC_currentIndexChanged(self, index):
        self.cmbFilterExpertExpertiseKindMC.setTable('rbMedicalBoardExpertiseKind', False, filter='expertiseCharacter_id=%s'%(self.cmbFilterExpertExpertiseCharacterMC.value()))
        self.cmbFilterExpertExpertiseObjectMC.setTable('rbMedicalBoardExpertiseObject', False, filter='expertiseCharacter_id=%s'%(self.cmbFilterExpertExpertiseCharacterMC.value()))
        self.cmbFilterExpertExpertiseArgumentMC.setTable('rbMedicalBoardExpertiseArgument', False, filter='expertiseCharacter_id=%s'%(self.cmbFilterExpertExpertiseCharacterMC.value()))


    def initExpertiseMCComboBox(self):
        flatCode = u'inspection%'
        if self.chkFilterExpertExpertiseMC.isChecked():
            flatCode = expertiseClass[self.cmbFilterExpertExpertiseMC.currentIndex()][1]
        self.cmbFilterExpertExpertiseTypeMC.clearValue()
        self.cmbFilterExpertExpertiseTypeMC.clear()
        self.cmbFilterExpertExpertiseTypeMC.setTable('ActionType', False, filter='''flatCode LIKE '%s' '''%(flatCode))
#        self.getExpertiseProperty(self.cmbFilterExpertExpertiseCharacterMC, u'Характеристика экспертизы', flatCode)
#        self.getExpertiseProperty(self.cmbFilterExpertExpertiseKindMC, u'Вид экспертизы', flatCode)
#        self.getExpertiseProperty(self.cmbFilterExpertExpertiseObjectMC, u'Предмет экспертизы', flatCode)
#        self.getExpertiseProperty(self.cmbFilterExpertExpertiseArgumentMC, u'Обоснование экспертизы', flatCode)


    def getExpertiseProperty(self, comboBox, name, flatCode = u'inspection%'):
        comboBox._model.clear()
        domain = u'\'не определено\','
        record = self.getExpertiseDomain(flatCode, name)
        if record:
            domainR = QString(forceString(record))
            if u'*' in domainR:
                index = domainR.indexOf(u'*', 0, Qt.CaseInsensitive)
                if domainR[index - 1] != u',':
                    domainR.replace(QString('*'), QString(','))
                else:
                    domainR.remove(QChar('*'), Qt.CaseInsensitive)
            domain += domainR
        comboBox.setDomain(domain)


    def getExpertiseDomain(self, flatCode, name):
        db = QtGui.qApp.db
        tableAPT = db.table('ActionPropertyType')
        tableActionType = db.table('ActionType')
        tableAction = db.table('Action')
        cond =[tableActionType['id'].inlist(getActionTypeIdListByFlatCode(flatCode)),
               tableAPT['name'].like(name),
               tableAPT['typeName'].like(u'String'),
               tableAPT['deleted'].eq(0)
               ]
        queryTable = tableActionType.innerJoin(tableAPT, tableActionType['id'].eq(tableAPT['actionType_id']))
        record = db.getRecordEx(queryTable, [tableAPT['valueDomain']], cond)
        if record:
            return record.value(0)
        return None


    def focusExpertMC(self):
        self.getCurrentExpertMedicalCommissionTable().setFocus(Qt.TabFocusReason)


    def focusExpert(self):
        self.getCurrentExpertTable().setFocus(Qt.TabFocusReason)


    def editTempInvalid(self, tempInvalidId):
        widgetIndex = self.tabWidgetTempInvalidTypes.currentIndex()
        clientIdList = []
        db = QtGui.qApp.db
        clientCache = CTableRecordCache(db, db.forceTable('Client'), u'*', capacity=None)
        dialog = CTempInvalidEditDialog(self, clientCache)
        clientInfo = getClientMiniInfo(self.currentClientId())
        dialog.setWindowTitle(self.tabWidgetTempInvalidTypes.tabText(widgetIndex) + u': ' + clientInfo)
        dialog.load(tempInvalidId)
        tempInvalidNewId = None
        try:
            if dialog.exec_():
                tempInvalidNewId = dialog.itemId()
            self.getCurrentExpertTable().model().updateColumnsCaches(tempInvalidId)
            self.updateTempInvalidList(self.__expertFilter, tempInvalidNewId if tempInvalidNewId else tempInvalidId)
            if self.tabWidgetTempInvalidTypes.currentIndex() == self.tabWidgetTempInvalidTypes.indexOf(self.tabExpertMedicalCommission):
                currentTableMSI = self.getCurrentExpertMedicalCommissionTable()
                actionId = currentTableMSI.currentItemId()
                if actionId:
                    currentTableMSI.model().updateColumnsCaches(actionId)
                else:
                    currentTableMSI.model().clearColumnsCaches()
                self.updateMedicalCommissionList(self.__expertMCFilter)
        finally:
            sip.delete(dialog)
            del dialog


    def editTempInvalidExpert(self):
        widgetIndex = self.tabWidgetTempInvalidTypes.currentIndex()
        clientId = self.currentClientId()
        if clientId:
            db = QtGui.qApp.db
            clientCache = CTableRecordCache(db, db.forceTable('Client'), u'*', capacity=None)
            dialog = CTempInvalidCreateDialog(self, clientId, clientCache)
            if self.tabWidgetTempInvalidTypes.currentIndex() == self.tabExpertTempInvalidDetals.indexOf(self.tabExpertTempInvalid):
                docCode = '1'
            else:
                docCode = ''
            dialog.setType(widgetIndex, docCode, True)
            clientInfo = getClientMiniInfo(self.currentClientId())
            dialog.setWindowTitle(self.tabWidgetTempInvalidTypes.tabText(widgetIndex) + u': ' + clientInfo)
            dialog.createTempInvalidDocument(placeRegistry = True)
            tempInvalidId = None
            try:
                if dialog.exec_():
                    tempInvalidId = dialog.itemId()
                self.updateTempInvalidList(self.__expertFilter, tempInvalidId)
            finally:
                sip.delete(dialog)
                del dialog


    def getExpertPrevDocId(self, table):
        tempInvalidId = table.currentItemId()
        if tempInvalidId:
            model = table.model()
            currentItem = model.recordCache().get(tempInvalidId)
            return forceRef(currentItem.value('prev_id'))
        return None


    def getExpertNextDocId(self, table):
        tempInvalidId = table.currentItemId()
        if tempInvalidId:
            return forceRef(QtGui.qApp.db.translate('TempInvalid', 'prev_id', tempInvalidId, 'id'))
        return None


    def _setMedicalCommissionActionsOrderByColumn(self, column):
        table = self.getCurrentExpertMedicalCommissionTable()
        table.setOrder(column)
        self.updateMedicalCommissionList(self.__expertMCFilter, table.currentItemId())


    def updateMedicalCommissionList(self, filter, posToId = None, order = []):
        # в соответствии с фильтром обновляет список документов ВК.
        self.__expertMCFilter = filter
        currentTable = self.getCurrentExpertMedicalCommissionTable()
        db = QtGui.qApp.db
        table = db.table('Action')
        tableEvent = db.table('Event')
        queryTable = table.innerJoin(tableEvent, tableEvent['id'].eq(table['event_id']) )
        expertiseId = filter.get('expertiseId', None)
        cond = [table['actionType_id'].inlist(getActionTypeIdListByFlatCode(expertiseClass[expertiseId][1] if (expertiseId is not None) else u'inspection%')),
                table['deleted'].eq(0),
                tableEvent['deleted'].eq(0)
               ]
        order = currentTable.order() if currentTable.order() else [table['endDate'].name()+' DESC', table['id'].name()]
        actionTypeMSIIdList = getActionTypeIdListByFlatCode(u'inspection_mse%')
        if self.tabExpertMedicalCommissionWidget.currentIndex() == self.tabExpertMedicalCommissionWidget.indexOf(self.tabExpertDirectionsMC):
            cond.append(table['endDate'].isNull())
            if actionTypeMSIIdList:
                cond.append(table['actionType_id'].notInlist(actionTypeMSIIdList))
        elif self.tabExpertMedicalCommissionWidget.currentIndex() == self.tabExpertMedicalCommissionWidget.indexOf(self.tabExpertProtocolsMC):
            cond.append(table['endDate'].isNotNull())
            if actionTypeMSIIdList:
                cond.append(table['actionType_id'].notInlist(actionTypeMSIIdList))
        elif self.tabExpertMedicalCommissionWidget.currentIndex() == self.tabExpertMedicalCommissionWidget.indexOf(self.tabExpertMedicalSocialInspection):
            cond.append(table['actionType_id'].inlist(actionTypeMSIIdList))
        expertiseTypeId = filter.get('expertiseTypeId', None)
        if bool(expertiseTypeId is not None and expertiseTypeId[0]):
            cond.append(table['actionType_id'].inlist(expertiseTypeId[0]))
        expertNumber = filter.get('expertNumber', None)
        if expertNumber:
            cond.append(getStringProperty(u'Номер ЛН', u'''APS.value %s'''%(updateLIKE(expertNumber))))
        expertNumberExpertise = filter.get('expertNumberExpertise', None)
        if expertNumberExpertise:
            cond.append(getStringProperty(u'Номер', u'''APS.value %s'''%(updateLIKE(expertNumberExpertise))))
        self.addDateCond(cond, table, 'begDate', filter, 'begDirectDate', 'endDirectDate')
        self.addDateCond(cond, table, 'endDate', filter, 'begExecDate', 'endExecDate')
        self.addEqCond(cond, table, 'setPerson_id', filter, 'setPersonId')
        self.addEqCond(cond, table, 'person_id', filter, 'expertId')
        if self.chkFilterExpertMKBMC.isChecked():
            actionMKBFrom = filter.get('begMKB', None)
            actionMKBTo = filter.get('endMKB', None)
            if actionMKBFrom or actionMKBTo:
                if actionMKBFrom and not actionMKBTo:
                   actionMKBTo = u'U'
                elif not actionMKBFrom and actionMKBTo:
                    actionMKBFrom = u'A'
                cond.append(table['MKB'].ge(actionMKBFrom))
                cond.append(table['MKB'].le(actionMKBTo))
        self.addEqCond(cond, table, 'status', filter, 'actionStatus')
        if 'expertOrgStructureId' in filter or 'expertSpecialityId' in filter:
            tablePerson = db.table('Person')
            queryTable = queryTable.leftJoin(tablePerson, tablePerson['id'].eq(table['setPerson_id']))
            cond.append(tablePerson['deleted'].eq(0))
            if 'expertOrgStructureId' in filter:
                expertOrgStructureId = filter.get('expertOrgStructureId', None)
                if expertOrgStructureId:
                    orgStructureIdList = db.getDescendants('OrgStructure', 'parent_id', expertOrgStructureId)
                    if orgStructureIdList:
                        cond.append(tablePerson['orgStructure_id'].inlist(orgStructureIdList))
            if 'expertSpecialityId' in filter:
                expertSpecialityId = filter.get('expertSpecialityId', None)
                if expertSpecialityId:
                    cond.append(tablePerson['speciality_id'].eq(expertSpecialityId))
        self.addEqCond(cond, table, 'createPerson_id', filter, 'createPersonId')
        self.addDateCond(cond, table, 'createDatetime', filter, 'begCreateDate', 'endCreateDate')
        self.addEqCond(cond, table, 'modifyPerson_id', filter, 'modifyPersonId')
        self.addDateCond(cond, table, 'modifyDatetime', filter, 'begModifyDate', 'endModifyDate')
        expertiseCharacterId = filter.get('expertiseCharacterId', None)
        if expertiseCharacterId:
            cond.append(existsPropertyValue(u'ActionProperty_rbMedicalBoardExpertiseCharacter', u'Характеристика экспертизы', expertiseCharacterId))
        expertiseKindId = filter.get('expertiseKindId', None)
        if expertiseKindId:
            cond.append(existsPropertyValue(u'ActionProperty_rbMedicalBoardExpertiseKind', u'Вид экспертизы', expertiseKindId))
        expertiseObjectId = filter.get('expertiseObjectId', None)
        if expertiseObjectId:
            cond.append(existsPropertyValue(u'ActionProperty_rbMedicalBoardExpertiseObject', u'Предмет экспертизы', expertiseObjectId))
        expertiseArgumentId = filter.get('expertiseArgumentId', None)
        if expertiseArgumentId:
            cond.append(existsPropertyValue(u'ActionProperty_rbMedicalBoardExpertiseArgument', u'Обоснование экспертизы', expertiseArgumentId))
        if 'clientIds' in filter:
            clientIds = filter.get('clientIds')
            cond.append(tableEvent['client_id'].inlist(clientIds))
        if 'eventIds' in filter:
            eventIds = filter.get('eventIds')
            cond.append(table['event_id'].inlist(eventIds))
        if 'Client' in order:
            tableClient = db.table('Client')
            queryTable = queryTable.leftJoin(tableClient, tableClient['id'].eq(tableEvent['client_id']))
        if 'vrbSetPersonWithSpeciality.name' in order:
            tableSetPerson = db.table('vrbPersonWithSpeciality').alias('vrbSetPersonWithSpeciality')
            queryTable = queryTable.leftJoin(tableSetPerson, tableSetPerson['id'].eq(table['setPerson_id']))
        if 'vrbPersonWithSpeciality.name' in order:
            tablePersonSP = db.table('vrbPersonWithSpeciality')
            queryTable = queryTable.leftJoin(tablePersonSP, tablePersonSP['id'].eq(table['person_id']))
        if 'expertNumberMC' in order:
            cols = [table['id']]
            cols.append(self.getPropertyValueToOrder(u'Номер ЛН', u'expertNumberMC', u'ActionProperty_String'))
        if 'expertNumberExpertise' in order:
            cols = [table['id']]
            cols.append(self.getPropertyValueToOrder(u'Номер', u'expertNumberExpertise', u'ActionProperty_String'))
        if 'expertDecision' in order:
            cols = [table['id']]
            cols.append(self.getPropertyValueToOrder(u'Решение', u'expertDecision', u'ActionProperty_String'))
        if 'directionDateMSI' in order:
            cols = [table['id']]
            cols.append(self.getPrevActionFieldMSI(u'directionDate', u'directionDateMSI'))
        if 'plannedEndDateMSI' in order:
            cols = [table['id']]
            cols.append(self.getPrevActionFieldMSI(u'plannedEndDate', u'plannedEndDateMSI'))
        if 'endDateMSI' in order:
            cols = [table['id']]
            cols.append(self.getPrevActionFieldMSI(u'endDate', u'endDateMSI'))
        try:
            QtGui.QApplication.setOverrideCursor(QtGui.QCursor(Qt.WaitCursor))
            if 'expertNumberMC' in order or 'expertNumberExpertise' in order or 'expertDecision' in order or 'directionDateMSI' in order or 'plannedEndDateMSI' in order or 'endDateMSI' in order:
                records = db.getRecordList(queryTable,
                               cols,
                               cond,
                               order)
                idList = []
                if not 'clientIds' in filter and not 'eventIds' in filter:
                    for record in records:
                        actionId = forceRef(record.value('id'))
                        if actionId:
                            idList.append(actionId)
                else:
                    for record in records:
                        actionId = forceRef(record.value('id'))
                        if actionId and actionId not in idList:
                            idList.append(actionId)
            else:
                if not 'clientIds' in filter and not 'eventIds' in filter:
                    getIdList = db.getIdList
                else:
                    getIdList = db.getDistinctIdList
                idList = getIdList(queryTable,
                               table['id'].name(),
                               cond,
                               order)
            currentTable.setIdList(idList, posToId)
        finally:
            QtGui.QApplication.restoreOverrideCursor()
        actionsFilterType = filter['actionsFilterType']
        hideClientInfo = 'clientIds' in filter and len(filter['clientIds']) == 1
        for i in range(3):
            currentTable.setColumnHidden(i, hideClientInfo)
        self.updateClientsListRequest = self.updateClientsListRequest or not hideClientInfo


    def getPrevActionFieldMSI(self, fieldName, aliasName):
        return u'''(SELECT ActionMSI.%s
                    FROM Action AS ActionMSI
                    WHERE ActionMSI.prevAction_id = Action.id AND ActionMSI.deleted = 0) AS %s'''%(fieldName, aliasName)


    def getPropertyValueToOrder(self, propertyName, aliasName, tableName):
        return u'''(SELECT APS.value
            FROM ActionProperty AS AP
            INNER JOIN ActionPropertyType AS APT ON AP.type_id = APT.id
            INNER JOIN %s AS APS ON APS.id = AP.id
            WHERE AP.action_id = Action.id AND APT.actionType_id = Action.actionType_id
            AND  Action.id IS NOT NULL
            AND APT.deleted = 0 AND AP.deleted = 0 AND APT.name %s) AS %s'''%(tableName, updateLIKE(propertyName), aliasName)


# ==================== Visit_Page ======================
    def updateVisitsList(self, filter, posToId=None):
#        """
#            в соответствии с фильтром обновляет список визитов.
#        """
        self.__visitFilter = filter
        db = QtGui.qApp.db
        tableVisit = db.table('Visit')
        tableEvent = db.table('Event')
        queryTable = tableVisit.leftJoin(tableEvent, tableEvent['id'].eq(tableVisit['event_id']) )
        cond = [tableVisit['deleted'].eq(0), tableEvent['deleted'].eq(0)]
        self.addEqCond(cond, tableVisit, 'id', filter, 'id')
        self.addEqCond(cond, tableVisit, 'createPerson_id', filter, 'createPersonId')
        self.addDateCond(cond, tableVisit, 'createDatetime', filter, 'begCreateDate', 'endCreateDate')
        self.addEqCond(cond, tableVisit, 'modifyPerson_id', filter, 'modifyPersonId')
        self.addDateCond(cond, tableVisit, 'modifyDatetime', filter, 'begModifyDate', 'endModifyDate')
        if 'clientIds' in filter:
            clientIds = filter.get('clientIds')
            cond.append(tableEvent['client_id'].inlist(clientIds))
        if 'eventIds' in filter:
            eventIds = filter.get('eventIds')
            cond.append(tableVisit['event_id'].inlist(eventIds))
        self.addDateTimeCond(cond, tableVisit, 'date', filter, 'begExecDateTime', 'endExecDateTime')
        if 'visitTypeId' in filter:
            cond.append(tableVisit['visitType_id'].eq(filter['visitTypeId']))
        if 'serviceId' in filter:
            cond.append(tableVisit['service_id'].eq(filter['serviceId']))
        if 'sceneId' in filter:
            cond.append(tableVisit['scene_id'].eq(filter['sceneId']))
        addTableExecPerson = False
        if 'personId' in filter:
            execPersonId = filter['personId']
            if execPersonId == -1:
                cond.append(tableVisit['person_id'].isNull())
            else:
                cond.append(tableVisit['person_id'].eq(execPersonId))
        else:
            if 'orgStructureId' in filter:
                    if not addTableExecPerson:
                        addTableExecPerson = True
                        tablePersonExec = db.table('Person').alias('PersonExec')
                        queryTable = queryTable.join(tablePersonExec, tablePersonExec['id'].eq(tableVisit['person_id']))
                    execSetOrgStructureId = filter['orgStructureId']
                    execSetOrgStructureIdList = db.getDescendants('OrgStructure', 'parent_id', execSetOrgStructureId) if execSetOrgStructureId else []
                    cond.append(tableVisit['person_id'].isNotNull())
                    cond.append(tablePersonExec['orgStructure_id'].isNotNull())
                    cond.append(tablePersonExec['orgStructure_id'].inlist(execSetOrgStructureIdList))
            if 'specialityId' in filter:
                    if not addTableExecPerson:
                        addTableExecPerson = True
                        tablePersonExec = db.table('Person').alias('PersonExec')
                        queryTable = queryTable.join(tablePersonExec, tablePersonExec['id'].eq(tableVisit['person_id']))
                    cond.append(tableVisit['person_id'].isNotNull())
                    cond.append(tablePersonExec['speciality_id'].eq(filter['specialityId']))

        if 'assistantId' in filter:
            assistantId = filter['assistantId']
            if assistantId == -1:
                cond.append(tableVisit['assistant_id'].isNull())
            else:
                cond.append(tableVisit['assistant_id'].eq(assistantId))
        payStatusFinanceCode = filter.get('payStatusFinanceCode', None)
        if payStatusFinanceCode:
            payStatusCode = filter.get('payStatusCode', 0)
            mask  = getPayStatusMaskByCode(payStatusFinanceCode)
            value = getPayStatusValueByCode(payStatusCode, payStatusFinanceCode)
            cond.append('((%s & %d) = %d)' % (tableVisit['payStatus'].name(), mask, value))
        if 'financeId' in filter:
            financeId = filter['financeId']
            if financeId:
                cond.append(tableVisit['finance_id'].eq(financeId))
            else:
                cond.append(tableVisit['finance_id'].isNull())
        try:
            QtGui.QApplication.setOverrideCursor(QtGui.QCursor(Qt.WaitCursor))
            idList = db.getDistinctIdList(queryTable,
                           tableVisit['id'].name(),
                           cond,
                           ['date DESC', tableVisit['id'].name()])
            self.tblVisits.setIdList(idList, posToId)
        finally:
            QtGui.QApplication.restoreOverrideCursor()
        visitsFilterType = self.cmbFilterVisit.currentIndex()
        if visitsFilterType == 0:
            filter['clientIds'] = [ self.selectedClientId() ]
        elif visitsFilterType == 1:
            filter['clientIds'] = self.modelClients.idList()
        elif visitsFilterType == 2:
            filter['eventIds'] = [ self.tblEvents.currentItemId() ]
        elif visitsFilterType == 3:
            filter['eventIds'] = self.modelEvents.idList()
        filter['visitsFilterType']=visitsFilterType
        hideClientInfo = 'clientIds' in filter and len(filter['clientIds']) == 1
        for i in xrange(3):
            self.tblVisits.setColumnHidden(i, hideClientInfo)
        self.updateClientsListRequest = self.updateClientsListRequest or not hideClientInfo

# ====== Visit_Page ===:)


    def enableCardReaderSupport(self):
        QtGui.qApp.startProgressBar(1, u'Поддержка соц. карт...')

        prefs = QtGui.qApp.preferences.appPrefs
        if forceBool(prefs.get('SocCard_Emulation',  False)):
            from Registry.CardReader import CCardReaderEmulator
            self.cardReader = CCardReaderEmulator()
        else:
            from Registry.CardReader import CCardReader
            self.cardReader = CCardReader()

        self.mapSocialCardDocumentCodeToId = {}
        self.mapSocialCardBenefitCodeToId = {}
        self.rbSocialCardDocumentType = None
        self.rbSocialCardBenefitType = None

        nPortNo = forceInt(prefs.get('SocCard_PortNumber', 0))
        diDevId = forceInt(prefs.get('SocCard_DevId',  0))
        listIssBSC = []

        for i in range(0, 6):
            listIssBSC.append(forceInt(prefs.get('SocCard_IssBSC%d' % i,  0)))

        szOperatorId = forceString(prefs.get('SocCard_OperatorId',  ''))

        if self.cardReader.init(nPortNo, diDevId, listIssBSC, szOperatorId):
            QtGui.qApp.stepProgressBar()
            self.addObject('actFilterSocCard',  QtGui.QAction(u'Социальная карта',  self))
            self.addObject('actFilter',  QtGui.QAction(u'Фильтр',  self))
            self.addObject('mnuFilter',  QtGui.QMenu(self))
            self.mnuFilter.addAction(self.actFilterSocCard)
            self.mnuFilter.addAction(self.actFilter)
            self.btnFilter.setMenu(self.mnuFilter)
            self.btnFilter.setShortcut('Alt+F7')

            if forceBool(prefs.get('SocCard_SetAsDefaultSearch',  False)):
                self.actFilterSocCard.setShortcut('F7')
                self.actFilter.setShortcut('Shift+F7')
                self.mnuFilter.setDefaultAction(self.actFilterSocCard)
            else:
                self.actFilterSocCard.setShortcut('Shift+F7')
                self.actFilter.setShortcut('F7')
                self.mnuFilter.setDefaultAction(self.actFilter)

        else:
            rc = self.cardReader.rc()
            message = u'<h4>Ошибка инициализации считывателя карт.</h4>\n' \
                u'Проверьте правильность подключения устройства и его '\
                u'настроек в меню Настойки.Умолчания.<br><br>' \
                u'Параметры соединения:<table>' \
                u'<tr><td>Номер порта:</td><td><b>%d</b></td></tr>' \
                u'<tr><td>Идентификатор устройства:</td><td><b>%d</b></td></tr>'\
                u'<tr><td>Код банка:</td><td><b>%.2d%.2d%.2d%.2d%.2d%.2d</b></td></tr>' \
                u'<tr><td>Идентификатор оператора:</td><td><b>%s</b></td></tr>'\
                u'<tr><td>Ошибка:</td><td><b>%s (код %d)</b></td></tr></table>' % \
                (nPortNo, diDevId, listIssBSC[0], listIssBSC[1], listIssBSC[2], listIssBSC[3],
                listIssBSC[4], listIssBSC[5], szOperatorId,  self.cardReader.errorText(rc),  rc)

            QtGui.QMessageBox.critical(self, u'Внимание!',
                                    message, QtGui.QMessageBox.Ok, QtGui.QMessageBox.Ok)

        QtGui.qApp.stopProgressBar()


    def findClientBySocCardInfo(self,  clientInfo):
        table = QtGui.qApp.db.table('Client')
        cond = []
        cond.append(table['deleted'].eq(0))

        for x in ('lastName', 'firstName', 'patrName',  'sex',  'birthDate'):
            if clientInfo.has_key(x):
                cond.append(table[x].eq(clientInfo[x]))

        QtGui.qApp.startProgressBar(2, u'Поиск пациента в БД...')
        record = QtGui.qApp.db.getRecordEx(table, 'id',  cond, 'id')
        QtGui.qApp.stepProgressBar()
        SNILS = clientInfo.get('SNILS')

        if not record and SNILS:
            record = QtGui.qApp.db.getRecordEx(table, 'id',  [table['deleted'].eq(0),
                                                      table['SNILS'].eq(SNILS),
                                                     ], 'id')
            QtGui.qApp.stepProgressBar()

        return record


    def processSocCard(self):
        if self.cardReader.checkCardType():
            QtGui.qApp.stepProgressBar()
            clientInfo = self.cardReader.socialInfo()
            QtGui.qApp.stepProgressBar()
            benefitInfo = self.cardReader.benefitInfo()
            QtGui.qApp.stepProgressBar()
            vitalInfo = self.cardReader.vitalInfo()
            QtGui.qApp.stepProgressBar()
            record = self.findClientBySocCardInfo(clientInfo) if clientInfo else None

            if record:
                return self.findClient(forceRef(record.value(0)),  record)
            elif clientInfo:
                message = u'Пациент: %s %s %s, пол: %s, дата рождения:' \
                    u' %s, СНИЛС: %s, не найден в БД. Добавить новую '\
                    u'регистрационную карту?' % (clientInfo.get('lastName', ''),
                    clientInfo.get('firstName', ''), clientInfo.get('patrName', ''),
                    formatSex(clientInfo.get('sex', 0)),
                    clientInfo.get('birthDate', QDate()).toString('dd.MM.yyyy'),
                    formatSNILS(clientInfo.get('SNILS')))

                if QtGui.QMessageBox.question(self,
                        u'Добавление нового пациента', message,
                        QtGui.QMessageBox.Yes|QtGui.QMessageBox.No,
                        QtGui.QMessageBox.No) == QtGui.QMessageBox.Yes:
                    self.createNewClientDialogWithInfoFromSocCard(clientInfo,  benefitInfo,  vitalInfo)
            else:
                message = u'Социальная карта не содержит информации о паиценте'
                QtGui.QMessageBox.critical(self, u'Поиск пациента', message,
                        QtGui.QMessageBox.Ok,  QtGui.QMessageBox.Ok)
        else:
            QtGui.QMessageBox.warning(self, u'Внимание!',
                u'Социальная карта не вставлена в считыватель,'\
                u' или имеет неверный формат.', QtGui.QMessageBox.Ok, QtGui.QMessageBox.Ok)


    def createNewClientDialogWithInfoFromSocCard(self,  socialInfo,  benefitInfo,  vitalInfo):
        if QtGui.qApp.userHasAnyRight([urAdmin, urRegTabWriteRegistry]):
            dialog = CClientEditDialog(self)
            self.loadSocialInfo(dialog, socialInfo)
            self.loadBenefitInfo(dialog, benefitInfo)
            self.loadVitalInfo(dialog, vitalInfo)
            try:
                if dialog.exec_():
                    clientId = dialog.itemId()
                    clientRecord = dialog.getRecord()
                    self.findClient(clientId, clientRecord)
            finally:
                dialog.destroy()
                sip.delete(dialog)
                del dialog


    def loadSocialInfo(self, dialog, socialInfo):
        dialog.edtFirstName.setText(socialInfo.get('firstName', ''))
        dialog.edtLastName.setText(socialInfo.get('lastName', ''))
        dialog.edtPatrName.setText(socialInfo.get('patrName', ''))
        dialog.edtBirthDate.setDate(socialInfo.get('birthDate', QDate()))
        dialog.cmbSex.setCurrentIndex(socialInfo.get('sex', 0))
        dialog.edtSNILS.setText(socialInfo.get('SNILS', ''))
        serial = socialInfo.get('docSerial')
        for c in '-=/_|':
            serial = serial.replace('c',' ')
        serial = trim(serial).split()
        serialLeft = serial[0] if len(serial)>=1 else ''
        serialRight = serial[1] if len(serial)>=2 else ''
        dialog.edtDocSerialLeft.setText(serialLeft)
        dialog.edtDocSerialRight.setText(serialRight)
        dialog.edtDocNumber.setText(socialInfo.get('docNumber'))
        docTypeId = self.getSocialCardDocumentTypeId(socialInfo.get('docType'))
        if docTypeId:
            dialog.cmbDocType.setValue(docTypeId)
        notes = u'Социальная карта: серия %s, номер %s. Идентификационный '\
            u'номер социального регистра %s.' % (socialInfo.get('cardNumber','-'),
                socialInfo.get('cardSerial',  '-'),  socialInfo.get('numberSocial','-'))
        dialog.edtNotes.setText(notes)
        dialog.edtCompulsoryPolisSerial.setText(socialInfo.get('policySerial', ''))
        dialog.edtCompulsoryPolisNumber.setText(socialInfo.get('policyNumber', ''))
        notes = socialInfo.get('orgCode',  '')
        policyDate = socialInfo.get('policyDate', None)
        if policyDate:
            notes += u' Дата действия полиса %s.' % policyDate
        dialog.edtCompulsoryPolisNote.setText(notes)


    def getSocialCardDocumentTypeId(self,  code):
        id = self.lookupDocumentTypeId(code)

        if not id:
            if self.rbSocialCardDocumentType is None:
                fileName = forceString(QtGui.qApp.preferences.appPrefs.get(
                    'SocCard_rbDocumentTypeRbFileName', ''))
                from Registry.CardReader import loadDocumentTypeRb
                self.rbSocialCardDocumentType = loadDocumentTypeRb(self,  fileName)

            name = self.rbSocialCardDocumentType.get(code)

            if name:
                from Registry.CardReader import askUserForDocumentTypeId
                id = askUserForDocumentTypeId(self,  code, name)
            else:
                message = u'Неизвестный код типа документа "%s".' % code
                QtGui.QMessageBox.critical(self, u'Импорт документа с социальной карты',
                        message,  QtGui.QMessageBox.Ok,  QtGui.QMessageBox.Ok)

        return id


    def loadBenefitInfo(self,  dialog,  benefitInfo):
        if benefitInfo != []:

            for (code,  begDate,  endDate) in benefitInfo:
                id = self.lookupBenefitId(code)
                if not id:
                    if self.rbSocialCardBenefitType is None:
                        fileName = forceString(QtGui.qApp.preferences.appPrefs.get(
                            'SocCard_rbClientBenefitRbFileName', ''))
                        from Registry.CardReader import loadBenefitTypeRb
                        self.rbSocialCardBenefitType = loadBenefitTypeRb(self,  fileName)

                    name = self.rbSocialCardBenefitType.get(code)

                    if name:
                        from Registry.CardReader import askUserForBenefitId
                        id = askUserForBenefitId(self,  code,  name)

                if id:
                    self.addBenefit(dialog, id,  begDate,  endDate)
                else:
                    message = u'Неизвестный код льготы "%s".' % code
                    QtGui.QMessageBox.critical(self, u'Импорт льгот с социальной карты',
                        message,  QtGui.QMessageBox.Ok,  QtGui.QMessageBox.Ok)


    def addBenefit(self, dialog,  id,  begDate,  endDate):
        record = dialog.modelSocStatuses.getEmptyRecord()
        record.setValue('socStatusType_id', toVariant(id))
        record.setValue('begDate',  toVariant(begDate))
        record.setValue('endDate', toVariant(endDate))
        dialog.modelSocStatuses.insertRecord(0, record)


    def loadVitalInfo(self, dialog, vitalInfo):
        if vitalInfo != 0:
            (allergy,  intolerance,  notes) = self.cardReader.decodeVitalInfo(vitalInfo)

            for x in allergy:
                record = dialog.modelAllergy.getEmptyRecord()
                record.setValue('nameSubstance', toVariant(x))
                record.setValue('createDate',  toVariant(QDate.currentDate()))
                dialog.modelAllergy.insertRecord(0, record)

            for x in intolerance:
                record = dialog.modelIntoleranceMedicament.getEmptyRecord()
                record.setValue('nameMedicament', toVariant(x))
                record.setValue('createDate',  toVariant(QDate.currentDate()))
                dialog.modelIntoleranceMedicament.insertRecord(0, record)

            if notes != []:
                strNotes = forceString(dialog.edtNotes.toPlainText()) + \
                    u' '+ u', '.join([str(et) for et in notes])
                dialog.edtNotes.setText(strNotes)


    def lookupBenefitId(self, code):
        result = self.mapSocialCardBenefitCodeToId.get(code, -1)
        if result == -1:
            db = QtGui.qApp.db
            result = db.translate('rbSocStatusType', 'socCode', code, 'id')
            self.mapSocialCardBenefitCodeToId[code] = result
        return result


    def lookupDocumentTypeId(self,  code):
        result = self.mapSocialCardDocumentCodeToId.get(code, -1)
        if result == -1:
            db = QtGui.qApp.db
            result = forceRef(db.translate('rbDocumentType', 'socCode', code, 'id'))
            self.mapSocialCardDocumentCodeToId[code] = result
        return result

#########################################################################


    def getKerContextData(self):
        clientId = self.currentClientId()
        tempInvalidId = self.currentTempInvalidId()
        data = getClientContextData(clientId)
        context = data['client'].context
        if self.tabWidgetExpertTempInvalidDetails.currentIndex() == 0: #вкладка "Периоды"
            tempInvalidInfo = context.getInstance(CTempInvalidInfo, tempInvalidId)
            data['isDuplicate'] = False
            data['tempInvalid'] = tempInvalidInfo
            data['getEventList'] = lambda begDate, endDate: getEventListByDates(context, clientId, begDate, endDate)
            #tempInvalidPeriodId = self.currentTempInvalidPeriodId() ??????????
        else: # self.tabWidgetExpertTempInvalidDetails.currentIndex() == 1: #вкладка "Дубликаты"
            data['tempInvalid'] = tempInvalidInfo
            data['getEventList'] = lambda begDate, endDate: getEventListByDates(context, clientId, begDate, endDate)
        return data


    def getKerListContextData(self):
        context = CInfoContext()
        idList = self.modelExpertTempInvalid.idList()
        tempInvalidId = self.currentTempInvalidId()
        tempInvalidListInfo = CTempInvalidAllInfoList(context, idList)
        clientId = self.currentClientId()
        tempInvalidInfo = context.getInstance(CTempInvalidInfo, tempInvalidId)
        begDate = endDate = None
        data = getClientContextData(clientId)
        context = data['client'].context
        if self.chkFilterExpertBegDate.isChecked():
            begDate = CDateInfo(self.edtFilterExpertBegBegDate.date())
            endDate = CDateInfo(self.edtFilterExpertEndBegDate.date())
        data['tempInvalidList'] = tempInvalidListInfo
        data['tempInvalid'] = tempInvalidInfo
        data['getEventList'] = lambda begDate, endDate: getEventListByDates(context, clientId, begDate, endDate)
        data['filter'] = {'begDate': begDate, 'endDate': endDate}
        return data

################################################################################


    @pyqtSignature('int')
    def on_tabMain_currentChanged(self, index):
        keyboardModifiers = QtGui.qApp.keyboardModifiers()
        if index == 0: # реестр
            self.syncSplitters(self.splitterRegistry)
#            self.focusClients()
            if self.updateClientsListRequest:
                self.updateClientsList(self.__filter)
            if self.tabMainCurrentPage == 1:
                if keyboardModifiers & Qt.ControlModifier == Qt.ControlModifier:
                    db = QtGui.qApp.db
                    eventId = self.currentEventId()
                    clientId = forceRef(db.translate('Event', 'id', eventId, 'client_id'))
                    self.tblClients.setCurrentItemId(clientId)
                if self.updateClientsListRequest:
                    self.updateClientsList(self.__filter)
            self.currentClientFromEvent = False
            self.currentClientFromAction = False
            self.currentClientFromExpert = False
        elif index == 1: # осмотры
            self.syncSplitters(self.splitterEvents)
            if keyboardModifiers & Qt.AltModifier:
                clientsFilterType = 2
            elif keyboardModifiers & Qt.ControlModifier:
                clientsFilterType = 1
            elif keyboardModifiers & Qt.ShiftModifier:
                clientsFilterType = -1
            else:
                clientsFilterType = 0
            if clientsFilterType == 0 and self.currentClientFromEvent and not self.currentClientFromAction and not self.currentClientFromExpert:
                # возврат из мед. карты или подобного, на одного пациента
                self.updateEventInfo(self.currentEventId())
            else:
                self.updateEventInfo(None)
                if clientsFilterType != -1:
                    self.on_buttonBoxEvent_reset()
                    self.cmbFilterEventByClient.setCurrentIndex(clientsFilterType)
                self.on_buttonBoxEvent_apply()
            self.currentClientFromEvent = True
            self.currentClientFromAction = False
            self.currentClientFromExpert = False
        elif index == 2: # мед.карта
            self.syncSplitters(self.splitterAmbCard)
            self.updateAmbCardInfo()
            self.tabAmbCardContent.resetWidgets()
        elif index == 3: # Обслуживание
            self.syncSplitters(self.splitterActions)
            self.updateActionInfo(None)
            if self.currentClientFromEvent: # переход сквозь осмотры
                cmbFilterActionFlags = QVariant(int(Qt.ItemIsEnabled|Qt.ItemIsSelectable))
                cmbFilterActionDefault = 2
            else:
                cmbFilterActionFlags = QVariant(0)
                cmbFilterActionDefault = 0
            self.cmbFilterAction.setItemData(2, cmbFilterActionFlags, Qt.UserRole-1)
            self.cmbFilterAction.setItemData(3, cmbFilterActionFlags, Qt.UserRole-1)

            if keyboardModifiers & Qt.AltModifier == Qt.AltModifier:
                clientsFilterType = 5
            elif keyboardModifiers & Qt.ControlModifier == Qt.ControlModifier:
                clientsFilterType = cmbFilterActionDefault+1
            elif keyboardModifiers & Qt.ShiftModifier == Qt.ShiftModifier:
                clientsFilterType = -1
            else:
                clientsFilterType = cmbFilterActionDefault
            if clientsFilterType != -1:
                self.on_buttonBoxAction_reset()
                self.cmbFilterAction.setCurrentIndex(clientsFilterType)
            self.on_buttonBoxAction_apply()
            self.currentClientFromEvent = False
            self.currentClientFromAction = True
            self.currentClientFromExpert = False
        elif index == 4: # КЭР
            self.syncSplitters(self.splitterExpert)
            self.updateTempInvalidInfo(None, noPrev=True)
            self.updateDocumentsExInfo(None, noPrev=True)
            if self.currentClientFromEvent: # переход сквозь осмотры
                cmbFilterExpertFlags = QVariant(int(Qt.ItemIsEnabled|Qt.ItemIsSelectable))
                cmbFilterExpertDefault = 2
            else:
                cmbFilterExpertFlags = QVariant(0)
                cmbFilterExpertDefault = 0
            self.cmbFilterExpert.setItemData(2, cmbFilterExpertFlags, Qt.UserRole-1)
            self.cmbFilterExpert.setItemData(3, cmbFilterExpertFlags, Qt.UserRole-1)
            self.cmbFilterExpert.setItemData(6, cmbFilterExpertFlags, Qt.UserRole-1)
            self.cmbFilterExpert.setItemData(7, cmbFilterExpertFlags, Qt.UserRole-1)
            if keyboardModifiers & Qt.AltModifier == Qt.AltModifier:
                clientsFilterType = 5
            elif keyboardModifiers & Qt.ControlModifier == Qt.ControlModifier:
                clientsFilterType = cmbFilterExpertDefault + 1
            elif keyboardModifiers & Qt.ShiftModifier == Qt.ShiftModifier:
                clientsFilterType = -1
            else:
                clientsFilterType = cmbFilterExpertDefault
            if clientsFilterType != -1:
                self.on_buttonBoxExpert_reset()
                self.cmbFilterExpert.setCurrentIndex(clientsFilterType)
            self.on_tabWidgetTempInvalidTypes_currentChanged(self.tabWidgetTempInvalidTypes.currentIndex())
#            self.on_buttonBoxExpert_apply()
            self.currentClientFromEvent = False
            self.currentClientFromAction = False
            self.currentClientFromExpert = True
        elif index == 5: # Визиты
            self.syncSplitters(self.splitterVisit)
            if self.currentClientFromEvent: # переход сквозь осмотры
                cmbFilterVisitFlags = QVariant(int(Qt.ItemIsEnabled|Qt.ItemIsSelectable))
            else:
                cmbFilterVisitFlags = QVariant(0)
            self.cmbFilterVisit.setItemData(2, cmbFilterVisitFlags, Qt.UserRole-1)
            self.cmbFilterVisit.setItemData(3, cmbFilterVisitFlags, Qt.UserRole-1)
            self.on_buttonBoxVisit_apply()
            self.currentClientFromEvent = False
            self.currentClientFromAction = False
            self.currentClientFromExpert = False
            self.updateVisitInfo(self.tblVisits.currentItemId())
        self.tabMainCurrentPage = index


    def getFilterInfoBed(self):
        def getNameOrgStructure(orgStructureId):
            db = QtGui.qApp.db
            if orgStructureId:
                tableOS = db.table('OrgStructure')
                record = db.getRecordEx(tableOS, [tableOS['name']], [tableOS['deleted'].eq(0), tableOS['id'].eq(orgStructureId)])
                if record:
                    nameOrgStructure = forceString(record.value(0))
                else:
                    nameOrgStructure = forceString(None)
            else:
                tableO = db.table('Organisation')
                recordOrg = db.getRecordEx(tableO, [tableO['shortName']], [tableO['deleted'].eq(0), tableO['id'].eq(QtGui.qApp.currentOrgId())])
                if recordOrg:
                    nameOrgStructure = forceString(recordOrg.value(0))
                else:
                    nameOrgStructure = forceString(None)
            return nameOrgStructure
        infoBeds = u''
        if self.chkFilterBeds.isChecked():
            statusBeds = self.cmbFilterStatusBeds.currentText()
            orgStructureId = self.cmbFilterOrgStructureBeds.value()
            infoBeds = u'; по койкам: статус %s подразделение %s'%(statusBeds, getNameOrgStructure(orgStructureId))
        infoAddressOrgStructure = u''
        if self.chkFilterAddressOrgStructure.isChecked():
            orgStructureType = self.cmbFilterAddressOrgStructureType.currentText()
            orgStructureId = self.cmbFilterAddressOrgStructure.value()
            infoAddressOrgStructure = u'; по участкам: тип %s подразделение %s'%(orgStructureType, getNameOrgStructure(orgStructureId))
        return [infoBeds, infoAddressOrgStructure]


    def on_modelFilterInfection_itemCheckingChanged(self):
        infectionIdList = self.modelFilterInfection.getCheckedIdList()
        if infectionIdList:
            self.chkFilterId.setChecked(False)
            db = QtGui.qApp.db
            tableInfectionVaccine = db.table('rbInfection_rbVaccine')
            tableVaccine = db.table('rbVaccine')
            cond = tableInfectionVaccine['infection_id'].inlist(infectionIdList)
            idList = db.getIdList(tableInfectionVaccine, 'vaccine_id', cond)
            self.modelFilterVaccine.setFilter(tableVaccine['id'].inlist(idList))
        else:
            self.modelFilterVaccine.setFilter('')


    def on_modelFilterVaccine_itemCheckingChanged(self):
        if self.modelFilterVaccine.getCheckedIdList():
            self.chkFilterId.setChecked(False)


    @pyqtSignature('int')
    def on_cmbFilterVaccinationCalendar_currentIndexChanged(self, index):
        vaccinationCalendarId = self.cmbFilterVaccinationCalendar.value()
        if vaccinationCalendarId:
            db = QtGui.qApp.db
            idList = db.getIdList('rbVaccinationCalendar_Infection', 'infection_id', 'master_id=%d'%vaccinationCalendarId)
            tableInfection = db.table('rbInfection')
            self.modelFilterInfection.setFilter(tableInfection['id'].inlist(idList))
        else:
            self.modelFilterInfection.setFilter('')


    @pyqtSignature('int')
    def on_modelClients_itemsCountChanged(self, count):
        realItemCount = self.sender().getRealItemCount()
        infoBeds, infoAddressOrgStructure = self.getFilterInfoBed()
        prefs = getPref(QtGui.qApp.preferences.reportPrefs, 'BatchRegLocatCardParams', {})
        batchRegLocatCardProcess = getPrefBool(prefs, 'BatchRegLocatCardProcess', False)
        if batchRegLocatCardProcess:
            self.lblClientsCount.setText(u'ВЫ НАХОДИТЕСЬ В РЕЖИМЕ ИЗМЕНЕНИЯ МЕСТА НАХОЖДЕНИЯ КАРТЫ!')
        else:
            self.lblClientsCount.setText(formatRecordsCount2(count, realItemCount)+infoBeds+infoAddressOrgStructure)


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelClients_currentRowChanged(self, current, previous):
        clientId = self.clientId(current)
        self.showClientInfo(clientId)
        self.tabMain.setTabEnabled(1, current.isValid())
        self.updateQueue()


    def updateQueue(self):
        clientId = self.currentClientId()
        self.modelSchedules.loadData(clientId)
        self.modelVisitsBySchedules.loadData(clientId)


    @pyqtSignature('QModelIndex')
    def on_tblClients_doubleClicked(self, index):
        self.editCurrentClient()
        self.focusClients()


    def showQueuePosition(self, scheduleItemId):
        if QtGui.qApp.mainWindow.dockResources:
            QtGui.qApp.mainWindow.dockResources.content.showQueueItem2(scheduleItemId)


    @pyqtSignature('QModelIndex')
    def on_tblSchedules_doubleClicked(self, index):
        if QtGui.qApp.doubleClickQueueClient() == 1:
            self.on_actAmbCreateEvent_triggered()
        elif QtGui.qApp.doubleClickQueueClient() == 2:
                self.on_actAmbChangeNotes_triggered()
        elif QtGui.qApp.doubleClickQueueClient() == 3:
               self.on_actAmbPrintOrder_triggered()
        elif QtGui.qApp.doubleClickQueueClient() == 4:
                self.on_actJumpQueuePosition_triggered()


    @pyqtSignature('')
    def on_actJumpQueuePosition_triggered(self):
        index = self.tblSchedules.currentIndex()
        if index.isValid():
            row = index.row()
            item = self.modelSchedules.items()[row]
            scheduleItemId = forceRef(item.value('id'))
            if scheduleItemId:
                QtGui.qApp.callWithWaitCursor(self, self.showQueuePosition, scheduleItemId)


    @pyqtSignature('QModelIndex')
    def on_tblVisitsBySchedules_doubleClicked(self, index):
        index = self.tblVisitsBySchedules.currentIndex()
        if index.isValid():
            row = index.row()
            eventId = self.modelVisitsBySchedules.getEventId(row)
            if eventId:
                editEvent(self, eventId)
                self.focusEvents()


    @pyqtSignature('')
    def on_actEditClient_triggered(self):
        self.editCurrentClient()
        self.focusClients()

    @pyqtSignature('')
    def on_actOpenClientDocumentTrackingHistory_triggered(self):
        self.openClientDocumentTrackingHistory()

    @pyqtSignature('')
    def on_actEditStatusObservationClient_triggered(self):
        self.updateStatusObservationClient()

    def openClientDocumentTrackingHistory(self):
        try:
            clientId = self.currentClientId()
            if clientId:
                if QtGui.qApp.userHasAnyRight([urRegTabReadLocationCard, urEditLocationCard]):
                    dialog = CClientDocumentTrackingList(self, clientId)
                    try:
                        dialog.exec_()
                        self.updateClientsList(self.__filter, clientId)
                    finally:
                        dialog.destroy()
                        sip.delete(dialog)
                        del dialog
        except:
            pass

    def updateStatusObservationClient(self):
        try:
            clientId = self.currentClientId()
            if clientId:
                if QtGui.qApp.userHasAnyRight([urAdmin, urEditStatusObservationClient]):
                    dialog = CStatusObservationClientEditor(self, [clientId])
                    try:
                        if dialog.exec_():
                            self.updateClientsList(self.__filter, clientId)
                    finally:
                        dialog.destroy()
                        sip.delete(dialog)
                        del dialog
        except:
            pass


    @pyqtSignature('')
    def on_actShowContingentsClient_triggered(self):
        self.showContingentsClient()


    def showContingentsClient(self):
        try:
            clientId = self.currentClientId()
            if clientId:
                dialog = CShowContingentsClientDialog(self, clientId)
                try:
                    if dialog.exec_():
                        pass
                finally:
                    dialog.destroy()
                    sip.delete(dialog)
                    del dialog
        except:
            pass


    @pyqtSignature('')
    def on_btnEdit_clicked(self):
        self.editCurrentClient()
        self.focusClients()


    @pyqtSignature('')
    def on_btnNew_clicked(self):
        self.editNewClient()
        self.focusClients()


    @pyqtSignature('')
    def on_mnuPrint_aboutToShow(self):
        clientId = self.selectedClientId()
        if clientId:
            self.mnuPrint.setDefaultAction(self.actPrintClient)
            self.actPrintClient.setEnabled(True)
            printer = QtGui.qApp.labelPrinter()
            self.actPrintClientLabel.setEnabled(self.clientLabelTemplate is not None and bool(printer))
        else:
            self.actPrintClient.setEnabled(False)
            self.actPrintClientLabel.setEnabled(False)


    @pyqtSignature('int')
    def on_actPrintClient_printByTemplate(self, templateId):
        clientId = self.selectedClientId()
        if clientId:
            changeClientConsent(self, clientId, templateId)
            data = getClientContextData(clientId)
            QtGui.qApp.call(self, applyTemplate, (self, templateId, data))


    @pyqtSignature('')
    def on_actPrintClientLabel_triggered(self):
        clientId = self.selectedClientId()
        printer = QtGui.qApp.labelPrinter()
        if clientId and self.clientLabelTemplate and printer:
            clientInfo = getClientInfo2(clientId)
            QtGui.qApp.call(self, directPrintTemplate, (self.clientLabelTemplate[1], {'client':clientInfo}, printer))


    @pyqtSignature('')
    def on_actPrintClientList_triggered(self):
        self.tblClients.setReportHeader(u'Список пациентов')
        self.tblClients.setReportDescription(self.getClientFilterAsText())
        mask = [True]*(self.modelClients.columnCount())
        orientation = QtGui.QPrinter.Landscape
        self.tblClients.printContent(mask, orientation)
        self.focusClients()


    @pyqtSignature('')
    def on_actFilterSocCard_triggered(self):
        QtGui.qApp.startProgressBar(4, u'Чтение карты...')
        self.processSocCard()
        QtGui.qApp.stopProgressBar()


    @pyqtSignature('')
    def on_actFilter_triggered(self):
        self.on_btnFilter_clicked()


    @pyqtSignature('')
    def on_btnFilter_clicked(self):
        for s in self.chkListOnClientsPage:
            chk = s[0]
            if chk.isChecked():
                self.activateFilterWdgets(s[1])
                return
        self.chkFilterBirthDay.setChecked(True)


    @pyqtSignature('')
    def on_actScanBarcode_triggered(self):
        self.on_buttonBoxClient_reset()
        self.cmbFilterAccountingSystem.setValue(None)
        self.chkFilterId.setChecked(True)


    @pyqtSignature('')
    def on_actShowPacsImages_triggered(self):
        explorer = CPacsExplorer(self.currentClientId())
        explorer.exec_()


    @pyqtSignature('')
    def on_actNotify_triggered(self):
        CNotifyDialog(self.tblClients.selectedItemIdList(),
                      CNotificationRule.ntRegistry).exec_()


    @pyqtSignature('')
    def on_actNotificationLog_triggered(self):
        CNotificationLogList(self, self.currentClientId()).exec_()


    @pyqtSignature('')
    def on_actSimplifiedClientSearch_triggered(self):
        dialog = CSimplifiedClientSearch(self)
        if dialog.exec_():
            ok, nameParts, date = dialog.getParsedKey()
            if ok:
                self.on_buttonBoxClient_reset()
                chkAndWidgets = ((self.chkFilterLastName,  self.edtFilterLastName),
                                 (self.chkFilterFirstName, self.edtFilterFirstName),
                                 (self.chkFilterPatrName,  self.edtFilterPatrName),
                                )

                for part, (chk, edt) in zip(nameParts, chkAndWidgets):
                    chk.setChecked(True)
                    edt.setText(part)
                if date:
                    self.chkFilterBirthDay.setChecked(True)
                    self.edtFilterBirthDay.setDate(date)
                self.extendedPolicyInformation = {}
                self.on_buttonBoxClient_apply()


    @pyqtSignature('')
    def on_actCreateEvent_triggered(self):
        self.requestNewEvent()


    @pyqtSignature('bool')
    def on_chkFilterExpertLinked_toggled(self, checked):
        self.tblExpertTempInvalidRelation.setVisible(checked)
        self.tblExpertTempInvalidRelationDocumentsEx.setVisible(checked)
        self.tblExpertDisabilityRelation.setVisible(checked)
        self.tblExpertDisabilityRelationDocumentsEx.setVisible(checked)
        self.tblExpertVitalRestrictionRelation.setVisible(checked)
        self.tblExpertVitalRestrictionRelationDocumentsEx.setVisible(checked)
        self.on_buttonBoxExpert_apply()


    @pyqtSignature('bool')
    def on_chkFilterVaccinationPeriod_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterVaccinationCalendar_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
            self.on_cmbFilterVaccinationCalendar_currentIndexChanged(self.cmbFilterVaccinationCalendar.currentIndex())

        else:
            self.modelFilterInfection.setFilter('')
        self.onChkFilterToggled(self.sender(), checked)



    @pyqtSignature('bool')
    def on_chkFilterVaccinationSeria_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)

    @pyqtSignature('bool')
    def on_chkFilterVaccinationType_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)

    @pyqtSignature('bool')
    def on_chkFilterContingent_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)

    @pyqtSignature('bool')
    def on_chkFilterMedicalExemption_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)

    @pyqtSignature('bool')
    def on_chkFilterMedicalExemptionType_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)



    @pyqtSignature('bool')
    def on_chkFilterId_toggled(self, checked):
        self.onChkFilterToggled(self.chkFilterId, checked)
        if checked:
            for chk in (self.chkFilterLastName,
                        self.chkFilterFirstName,
                        self.chkFilterPatrName,
                        self.chkFilterBirthDay,
                        self.chkFilterEndBirthDay,
                        self.chkFilterSex,
                        self.chkFilterContact,
                        self.chkFilterSNILS,
                        self.chkFilterDocument,
                        self.chkFilterPolicy,
                        self.chkFilterRegionSMO,
                        self.chkFilterPolicyOnlyActual,
                        self.chkFilterAddress,
                        self.chkFilterAddressCorpus,
                        self.chkFilterWorkOrganisation,
                        self.chkSocStatuses,
                        self.chkFilterDocumentLocation,
                        self.chkFilterDocumentTypeForTracking,
                        self.chkFilterStatusObservationType,
                        self.chkFilterContingentType,
                        self.chkFilterCreatePerson,
                        self.chkFilterCreateDate,
                        self.chkFilterModifyPerson,
                        self.chkFilterModifyDate,
                        self.chkFilterEvent,
                        self.chkFilterAge,
                        self.chkFilterAddressOrgStructure,
                        self.chkFilterBeds,
                        self.chkFilterRegAddressIsEmpty,
                        self.chkFilterLocAddressIsEmpty,
                        self.chkFilterAttach,
                        self.chkFilterToStatement,
                        self.chkFilterAttachType,
                        self.chkFilterAttachNonBase,
                        self.chkFilterTempInvalid,
                        self.chkFilterTFUnconfirmed,
                        self.chkFilterTFConfirmed,
                        self.chkFilterClientConsent,
                        self.chkFilterVaccinationPeriod,
                        self.chkFilterVaccinationCalendar,
                        self.chkFilterVaccinationSeria,
                        self.chkFilterVaccinationType,
                        self.chkFilterContingent,
                        self.chkFilterMedicalExemption,
                        self.chkFilterMedicalExemptionType
                       ):
                chk.setChecked(False)
            self.tblFilterInfection.uncheckAll()
            self.tblFilterVaccine.uncheckAll()


    @pyqtSignature('int')
    def on_cmbFilterAccountingSystem_currentIndexChanged(self, index):
        self.edtFilterId.setValidator(None)
        if self.cmbFilterAccountingSystem.value():
            self.edtFilterId.setValidator(None)
        else:
            self.edtFilterId.setValidator(self.idValidator)


    @pyqtSignature('bool')
    def on_chkFilterLastName_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterFirstName_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterPatrName_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterBirthDay_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
            self.chkFilterAge.setChecked(False)
            self.edtFilterEndBirthDay.setEnabled(self.chkFilterEndBirthDay.isChecked())
        else:
            self.edtFilterEndBirthDay.setEnabled(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterEndBirthDay_toggled(self, checked):
#        if checked:
#            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterSex_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterContact_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterSNILS_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterDocument_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterPolicy_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)
        if self.chkFilterRegionSMO.isChecked():
            self.cmbFilterPolicyInsurer.setEnabled(False)
            self.edtFilterPolicySerial.setEnabled(False)
            self.edtFilterPolicyNumber.setEnabled(False)


    @pyqtSignature('bool')
    def on_chkFilterRegionSMO_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.cmbFilterPolicyInsurer.setEnabled(self.chkFilterPolicy.isChecked() and not checked)
        self.edtFilterPolicySerial.setEnabled(self.chkFilterPolicy.isChecked() and not checked)
        self.edtFilterPolicyNumber.setEnabled(self.chkFilterPolicy.isChecked() and not checked)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterPolicyOnlyActual_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('QString')
    def on_edtFilterPolicySerial_textEdited(self, text):
        self.cmbFilterPolicyInsurer.setSerialFilter(text)


    @pyqtSignature('bool')
    def on_chkFilterWorkOrganisation_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkSocStatuses_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterDocumentLocation_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)

    @pyqtSignature('bool')
    def on_chkFilterDocumentTypeForTracking_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterStatusObservationType_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterContingentType_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterCreatePerson_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterCreateDate_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterModifyPerson_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterModifyDate_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterEvent_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterAge_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
            self.chkFilterBirthDay.setChecked(False)
            self.chkFilterEndBirthDay.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('')
    def on_btnSelectWorkOrganisation_clicked(self):
        orgId = selectOrganisation(self, self.cmbWorkOrganisation.value(), False)
        self.cmbWorkOrganisation.updateModel()
        if orgId:
            self.cmbWorkOrganisation.setValue(orgId)


    @pyqtSignature('bool')
    def on_chkFilterAddress_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
            self.chkFilterAddressOrgStructure.setChecked(False)
            self.chkFilterRegAddressIsEmpty.setChecked(False)
            self.chkFilterLocAddressIsEmpty.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)
        self.edtFilterAddressCorpus.setEnabled(self.chkFilterAddressCorpus.isChecked() and checked)


    @pyqtSignature('bool')
    def on_chkFilterAddressCorpus_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('int')
    def on_cmbFilterAddressCity_currentIndexChanged(self, index):
        code = self.cmbFilterAddressCity.code()
        self.cmbFilterAddressStreet.setCity(code)
        self.cmbFilterAddressOkato.setKladrCode(code)


    @pyqtSignature('int')
    def on_cmbFilterAddressOkato_currentIndexChanged(self, index):
        okato = self.cmbFilterAddressOkato.value()
        self.cmbFilterAddressStreet.setOkato(okato)


    @pyqtSignature('int')
    def on_cmbSocStatusesClass_currentIndexChanged(self, index):
        socStatusClassId = self.cmbSocStatusesClass.value()
        if socStatusClassId:
            filter = (u'''rbSocStatusType.id IN (SELECT DISTINCT rbSocStatusClassTypeAssoc.type_id
            FROM  rbSocStatusClassTypeAssoc
            WHERE rbSocStatusClassTypeAssoc.class_id = %s)'''%(socStatusClassId))
        else:
            filter = u''
        self.cmbSocStatusesType.setFilter(filter)


    @pyqtSignature('bool')
    def on_chkFilterAddressOrgStructure_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
            self.chkFilterAddress.setChecked(False)
            self.chkFilterAddressCorpus.setChecked(False)
            self.chkFilterRegAddressIsEmpty.setChecked(False)
            self.chkFilterLocAddressIsEmpty.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterBeds_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterRegAddressIsEmpty_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
            self.chkFilterAddress.setChecked(False)
            self.chkFilterAddressCorpus.setChecked(False)
            self.chkFilterAddressOrgStructure.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterLocAddressIsEmpty_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
            self.chkFilterAddress.setChecked(False)
            self.chkFilterAddressCorpus.setChecked(False)
            self.chkFilterAddressOrgStructure.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterAttach_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
            self.chkFilterAttachNonBase.setChecked(False)
            if not self.cmbFilterAttachOrganisation.value():
                self.cmbFilterAttachOrganisation.setValue(QtGui.qApp.currentOrgId())
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterAttachType_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterToStatement_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterAttachNonBase_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
            self.chkFilterAttach.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterTempInvalid_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterTFUnconfirmed_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
            self.chkFilterTFConfirmed.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterClientConsent_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterTFConfirmed_toggled(self, checked):
        if checked:
            self.chkFilterId.setChecked(False)
            self.chkFilterTFUnconfirmed.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('QAbstractButton*')
    def on_buttonBoxClient_clicked(self, button):
        buttonCode = self.buttonBoxClient.standardButton(button)
        if buttonCode == QtGui.QDialogButtonBox.Apply:
            self.on_buttonBoxClient_apply()
        elif buttonCode == QtGui.QDialogButtonBox.Reset:
            self.on_buttonBoxClient_reset()
            self.extendedPolicyInformation = {}


#    @pyqtSignature('')
    def on_buttonBoxClient_apply(self):
        filter = {}
        if self.chkFilterId.isChecked():
            accountingSystemId = self.cmbFilterAccountingSystem.value()
            clientId = forceStringEx(self.edtFilterId.text())
            if not accountingSystemId:
                clientId = parseClientId(clientId)
            if clientId:
                filter['id'] = clientId
                filter['accountingSystemId'] = accountingSystemId

        if self.chkFilterLastName.isChecked():
            tmp = forceStringEx(self.edtFilterLastName.text())
            if tmp:
                filter['lastName'] = tmp
        if self.chkFilterFirstName.isChecked():
            tmp = forceStringEx(self.edtFilterFirstName.text())
            if tmp:
                filter['firstName'] = tmp
        if self.chkFilterPatrName.isChecked():
            tmp = forceStringEx(self.edtFilterPatrName.text())
            if tmp:
                filter['patrName'] = tmp
        if self.chkFilterBirthDay.isChecked():
            filter['birthDate'] = self.edtFilterBirthDay.date()
            if self.chkFilterEndBirthDay.isChecked():
                filter['endBirthDate'] = self.edtFilterEndBirthDay.date()
        if self.chkFilterDead.isChecked():
            filter['dead'] = True
        if self.chkFilterDeathBegDate.isChecked():
            filter['begDeathDate'] = self.edtFilterDeathBegDate.date()
        if self.chkFilterDeathEndDate.isChecked():
            filter['endDeathDate'] = self.edtFilterDeathEndDate.date()
        if self.chkFilterSex.isChecked():
            filter['sex'] = self.cmbFilterSex.currentIndex()
        if self.chkFilterContact.isChecked():
            contact = forceStringEx(self.edtFilterContact.text())
            if contact:
                filter['contact'] = contact
        if self.chkFilterSNILS.isChecked():
            tmp = trim(forceString(self.edtFilterSNILS.text()).replace('-','').replace(' ',''))
            filter['SNILS'] = tmp
        if self.chkFilterDocument.isChecked():
            typeId = self.cmbFilterDocumentType.value()
            serial = forceStringEx(self.edtFilterDocumentSerial.text())
            number = forceStringEx(self.edtFilterDocumentNumber.text())
            filter['doc'] = (typeId, serial, number)
        if self.chkFilterPolicy.isChecked():
            onlyPolicyActual = self.chkFilterPolicyOnlyActual.isChecked()
            policyActualData = self.edtFilterPolicyActualData.date()
            policyType = self.cmbFilterPolicyType.value()
            policyKind = self.cmbFilterPolicyKind.value()
            insurerId = self.cmbFilterPolicyInsurer.value() if not self.chkFilterRegionSMO.isChecked() else None
            serial = forceStringEx(self.edtFilterPolicySerial.text()) if not self.chkFilterRegionSMO.isChecked() else ''
            number = forceStringEx(self.edtFilterPolicyNumber.text()) if not self.chkFilterRegionSMO.isChecked() else ''
            filter['policy'] = (onlyPolicyActual, policyActualData, policyType, policyKind, insurerId, serial, number)
        if self.chkFilterRegionSMO.isChecked():
            filter['regionSMO'] = (self.cmbFilterRegionTypeSMO.currentIndex(), self.cmbFilterRegionSMO.code())
        if self.chkFilterWorkOrganisation.isChecked():
            filter['orgId'] = self.cmbWorkOrganisation.value()
        filter['socStatuses'] = self.chkSocStatuses.isChecked()
        if self.chkSocStatuses.isChecked():
            filter['socStatusesCondition'] = self.chkSocStatusesCondition.isChecked()
            filter['socStatusesClass'] = self.cmbSocStatusesClass.value()
            filter['socStatusesType'] = self.cmbSocStatusesType.value()
            filter['socStatusesBegDate'] = self.edtFilterSocStatusesBegDate.date()
            filter['socStatusesEndDate'] = self.edtFilterSocStatusesEndDate.date()
        if self.chkFilterDocumentTypeForTracking.isChecked():
            filter['documentTypeForTracking'] = self.cmbFilterDocumentTypeForTracking.value()
        if self.chkFilterDocumentLocation.isChecked():
            filter['documentLocation'] = self.cmbFilterDocumentLocation.value()
            filter['personDocumentLocation'] = self.cmbPersonDocumentLocation.value()
            filter['begDateFilterDocumentLocation'] = self.edtBegDateFilterDocumentLocation.date()
            filter['endDateFilterDocumentLocation'] = self.edtEndDateFilterDocumentLocation.date()
        if self.chkFilterStatusObservationType.isChecked():
            filter['statusObservationType'] = self.cmbFilterStatusObservationType.value()
        if self.chkFilterContingentType.isChecked():
            filter['contingentTypeId'] = self.cmbFilterContingentType.value()
            filter['contingentEventTypeStatus'] = self.cmbFilterContingentEventTypeStatus.currentIndex()
            filter['contingentActionTypeStatus'] = self.cmbFilterContingentActionType.currentIndex()
        if self.chkFilterCreatePerson.isChecked():
            filter['createPersonIdEx'] = self.cmbFilterCreatePerson.value()
        if self.chkFilterCreateDate.isChecked():
            filter['begCreateDateEx'] = self.edtFilterBegCreateDate.date()
            filter['endCreateDateEx'] = self.edtFilterEndCreateDate.date()
        if self.chkFilterModifyPerson.isChecked():
            filter['modifyPersonIdEx'] = self.cmbFilterModifyPerson.value()
        if self.chkFilterModifyDate.isChecked():
            filter['begModifyDateEx'] = self.edtFilterBegModifyDate.date()
            filter['endModifyDateEx'] = self.edtFilterEndModifyDate.date()
        if self.chkFilterEvent.isChecked():
            filter['event'] = ( self.chkFilterFirstEvent.isChecked(),
                                self.edtFilterEventBegDate.date(),
                                self.edtFilterEventEndDate.date() )
        if self.chkFilterAge.isChecked():
            filter['age'] = ((self.edtFilterBegAge.value(), self.cmbFilterBegAge.currentIndex()),
                             (self.edtFilterEndAge.value()+1, self.cmbFilterEndAge.currentIndex())
                            )
        if self.chkFilterAddress.isChecked():
            filter['address'] = (self.cmbFilterAddressType.currentIndex(),
                                 self.cmbFilterAddressCity.code(),
                                 self.cmbFilterAddressOkato.value(),
                                 self.cmbFilterAddressStreet.code(),
                                 self.cmbFilterAddressStreet.codeList(),
                                 forceString(self.edtFilterAddressHouse.text()),
                                 self.chkFilterAddressCorpus.isChecked(),
                                 forceString(self.edtFilterAddressCorpus.text()),
                                 forceString(self.edtFilterAddressFlat.text())
                                )
        elif self.chkFilterAddressOrgStructure.isChecked():
            filter['addressOrgStructure'] = (self.cmbFilterAddressOrgStructureType.currentIndex(),
                                             self.cmbFilterAddressOrgStructure.value()
                                            )
        else:
            if self.chkFilterRegAddressIsEmpty.isChecked():
                filter['regAddressIsEmpty'] = True
            if self.chkFilterLocAddressIsEmpty.isChecked():
                filter['locAddressIsEmpty'] = True
        if self.chkFilterBeds.isChecked():
            filter['beds'] = (self.cmbFilterStatusBeds.currentIndex(),
                              self.cmbFilterOrgStructureBeds.value()
                             )
        if self.chkFilterAttach.isChecked():
            filter['attachTo'] = self.cmbFilterAttachOrganisation.value()
        elif self.chkFilterAttachNonBase.isChecked():
            filter['attachToNonBase'] = True
        if self.chkFilterAttachType.isChecked():
            filter['attachType'] = (self.cmbFilterAttachCategory.currentIndex(),
                                    self.cmbFilterAttachType.value(),
                                    self.edtFilterAttachBegDate.date(),
                                    self.edtFilterAttachEndDate.date())
            if self.chkFilterToStatement.isChecked():
                filter['isStatement'] = self.cmbFilterToStatement.currentIndex()
        if self.chkFilterTempInvalid.isChecked():
            filter['tempInvalid'] = (self.edtFilterBegTempInvalid.date(),
                                     self.edtFilterEndTempInvalid.date()
                                    )
        if self.chkFilterTFUnconfirmed.isChecked():
            filter['TFUnconfirmed'] = True
        elif self.chkFilterTFConfirmed.isChecked():
            filter['TFConfirmed'] = (self.edtFilterBegTFConfirmed.date(),
                                     self.edtFilterEndTFConfirmed.date()
                                    )
        if self.chkFilterClientConsent.isChecked():
            filter['clientConsentValue'] = self.cmbFilterClientConsentValue.currentIndex()
            filter['clientConsentTypeId'] = self.cmbFilterClientConsentType.value()
            filter['clientConsentBegDate'] = self.edtFilterClientConsentBegDate.date()
            filter['clientConsentEndDate'] = self.edtFilterClientConsentEndDate.date()
            filter['clientPersonConsent'] = self.cmbFilterPersonConsent.value()
        filter['excludeLeaved'] = self.chkFilterExcludeLeaved.isChecked()
        if self.chkClientMKB.isChecked():
            clientMKBFrom = forceString(self.edtClientMKBFrom.text())
            clientMKBTo = forceString(self.edtClientMKBTo.text())
            filter['clientMKBFrom'] = clientMKBFrom if clientMKBFrom else None
            filter['clientMKBTo'] = clientMKBTo if clientMKBTo else None
        filter.update(self.getVaccinationClientFilter())

        self.updateClientsList(filter)
        self.focusClients()


    def getVaccinationClientFilter(self):
        filter = {}

        if self.chkFilterVaccinationPeriod.isChecked():
            filter['vaccinationBegDate'] = self.edtFilterVaccinationBegDate.date()
            filter['vaccinationEndDate'] = self.edtFilterVaccinationEndDate.date()

        if self.chkFilterVaccinationCalendar.isChecked():
            filter['vaccinationCalendarId'] = self.cmbFilterVaccinationCalendar.value()

        infectionIdList = self.modelFilterInfection.getCheckedIdList()
        if infectionIdList:
            filter['infectionIdList'] = infectionIdList

        vaccineIdList = self.modelFilterVaccine.getCheckedIdList()
        if vaccineIdList:
            filter['vaccineIdList'] = vaccineIdList

        if self.chkFilterVaccinationSeria.isChecked():
            filter['vaccinationSeria'] = trim(self.edtFilterVaccinationSeria.text())

        if self.chkFilterVaccinationType.isChecked():
            filter['vaccinationType'] = trim(self.cmbFilterVaccinationType.text())

        if self.chkFilterContingent.isChecked():
            filter['vaccinationContingent'] = self.cmbFilterContingent.currentIndex()

        if self.chkFilterMedicalExemption.isChecked():
            filter['medicalExemption'] = self.cmbFilterMedicalExemption.currentIndex()

        if self.chkFilterMedicalExemptionType.isChecked():
            filter['medicalExemptionTypeId'] = self.cmbFilterMedicalExemptionType.value()

        return filter



    @pyqtSignature('int')
    def on_cmbFilterAttachCategory_currentIndexChanged(self, index):
        self.setupCmbFilterAttachTypeCategory(index)

    @pyqtSignature('bool')
    def on_chkFilterExcludeLeaved_clicked(self, state):
        if state:
            self.chkFilterDead.setChecked(False)

    @pyqtSignature('bool')
    def on_chkFilterDead_toggled(self, state):
        self.chkFilterDeathBegDate.setEnabled(state)
        self.chkFilterDeathEndDate.setEnabled(state)
        if state:
            self.chkFilterExcludeLeaved.setChecked(False)
        else:
            self.chkFilterDeathBegDate.setChecked(False)
            self.chkFilterDeathEndDate.setChecked(False)

#    @pyqtSignature('')
    def on_buttonBoxClient_reset(self):
        for s in self.chkListOnClientsPage:
            chk = s[0]
            if chk.isChecked():
                self.deactivateFilterWdgets(s[1])
                chk.setChecked(False)
        self.tblFilterInfection.uncheckAll()
        self.tblFilterVaccine.uncheckAll()


#### Events page #################################

    @pyqtSignature('')
    def on_actEventEditClient_triggered(self):
        clientId = self.currentClientId()
        if not clientId:
            clientId = self.selectedClientId()
        if clientId:
            self.editClient(clientId)
            self.updateEventInfo(self.currentEventId())
            self.updateClientsListRequest = True


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelEvents_currentRowChanged(self, current, previous):
        eventId = self.eventId(current)
        self.updateEventInfo(eventId)


    @pyqtSignature('int')
    def on_modelEvents_itemsCountChanged(self, count):
        infoBeds, infoAddressOrgStructure = self.getFilterInfoBed()
        clientIdList =  self.modelEvents.getClientIdList()
        clientCount = len(clientIdList)
        if count:
            labelText =  u'в списке '+ '%d %s' % (count, agreeNumberAndWord(count, (u'обращение', u'обращения', u'обращений'))) + ', %d %s' % (clientCount, agreeNumberAndWord(clientCount, (u'пациент', u'пациента', u'пациентов')))
        else:
            labelText =  u'список пуст'
        self.lblEventsCount.setText(labelText+infoBeds+infoAddressOrgStructure)


    @pyqtSignature('QModelIndex')
    def on_tblEvents_doubleClicked(self, index):
        self.on_btnEventEdit_clicked()


    @pyqtSignature('')
    def on_btnEventEdit_clicked(self):
        eventId = self.currentEventId()
        if eventId:
            eventId = editEvent(self, eventId)
        else:
            eventId = self.requestNewEvent()
        if eventId:
            self.updateEventListAfterEdit(eventId)
        self.focusEvents()


    @pyqtSignature('')
    def on_btnEventNew_clicked(self):
        self.requestNewEvent()
        self.focusEvents()


    @pyqtSignature('')
    def on_actEventPrint_triggered(self):
        self.tblEvents.setReportHeader(u'Список обращений')
        self.tblEvents.setReportDescription(self.getEventFilterAsText())
#        if self.cmbFilterEventByClient.currentIndex() == 0:
#            mask = [False]*3 + [True]*(self.modelEvents.columnCount()-3)
#        else:
#            mask = [True]*(self.modelEvents.columnCount())
#        self.tblEvents.printContent(mask)
        pageFormat = CPageFormat(pageSize=CPageFormat.A4, orientation=CPageFormat.Landscape, leftMargin=1, topMargin=1, rightMargin=1,  bottomMargin=1)
        self.tblEvents.printContent(pageFormat = pageFormat)
        self.focusEvents()


    @pyqtSignature('')
    def on_actVisitPrint_triggered(self):
        self.tblVisits.setReportHeader(u'Список визитов')
        self.tblVisits.setReportDescription(self.getVisitFilterAsText())
        self.tblVisits.printContent()
        self.focusEvents()


    def getVisitFilterAsText(self):
        db = QtGui.qApp.db
        filter  = self.__visitFilter
        resList = []

        visitsFilterType = filter['visitsFilterType']
        if visitsFilterType in (0, 1):
            clientIds = filter.get('clientIds', None)
            if clientIds and len(clientIds) == 1:
                resList.append((u'пациент', getClientMiniInfo(clientIds[0])))
            elif clientIds and len(clientIds) > 1:
                resList.append((u'список пациентов', u'из вкладки'))
        elif visitsFilterType in (2, 3):
            clientIds = self.__eventFilter.get('clientIds', None)
            if clientIds and len(clientIds) == 1:
                resList.append((u'пациент', getClientMiniInfo(clientIds[0])))
            elif clientIds and len(clientIds) > 1:
                resList.append((u'список пациентов', u'по списку осмотров'))
        else:
            resList.append((u'список пациентов', u'полный'))

        tmpList = [
            ('begSetDate', u'Дата назначения с', forceString),
            ('endSetDate', u'Дата назначения по', forceString),
            ('visitTypeId', u'Тип визита',
                lambda id: forceString(db.translate('rbVisitType', 'id', id, 'name'))),
            ('setSpecialityId', u'Специальность назначившего',
                lambda id: forceString(db.translate('rbSpeciality', 'id', id, 'name'))),
            ('personId', u'Назначил',
                lambda id: forceString(db.translate('vrbPersonWithSpeciality', 'id', id, 'name'))),
            ('isPrimary',   u'Признак первичности', lambda dummy: u'+'),
            ('begPlannedEndDate', u'Плановая дата выполнения с', forceString),
            ('endPlannedEndDate', u'Плановая дата окончания по', forceString),
            ('begExecDateTime', u'Дата и время выполнения с', forceString),
            ('endExecDateTime', u'Дата и время выполнения по', forceString),
            ]
        for (x, y, z) in tmpList:
            convertFilterToTextItem(resList, filter, x, y, z)
        return '\n'.join([ ': '.join(item) for item in resList])


    @pyqtSignature('')
    def on_actReportClientServices_triggered(self):
        dialog = CReportClientServices(self, self.selectedClientId(), self.modelEvents.idList())
        dialog.exec_()


    def getEventInfo(self, context):
        eventId = self.currentEventId()
        return context.getInstance(CEmergencyEventInfo, eventId)


    @pyqtSignature('int')
    def on_btnEventPrint_printByTemplate(self, templateId):
        data = self.getEventContextData(self)
        QtGui.qApp.call(self, applyTemplate, (self, templateId, data))


    def getEventContextData(self, editor):
        from Events.TempInvalidInfo import CTempInvalidInfo
        context = CInfoContext()
        eventInfoList = context.getInstance(CEmergencyEventInfoList, tuple(self.tblEvents.model().idList()))
        eventInfo = editor.getEventInfo(context)
        if hasattr(editor, 'getTempInvalidInfo'):
            tempInvalidInfo = editor.getTempInvalidInfo(context)
        else:
            tempInvalidInfo = context.getInstance(CTempInvalidInfo, None)
        eventType = context.getInstance(CEventTypeInfo, forceRef(self.cmbFilterEventType.value()))  if self.chkFilterEventType.isChecked() else None
        eventBegSetDate = CDateInfo(self.edtFilterEventBegSetDate.date()) if self.chkFilterEventSetDate.isChecked() else None
        eventEndSetDate = CDateInfo(self.edtFilterEventEndSetDate.date()) if self.chkFilterEventSetDate.isChecked() else None
        eventBegSetTime = CTimeInfo(self.edtFilterEventBegSetTime.time()) if self.chkFilterEventSetDate.isChecked() else None
        eventEndSetTime = CTimeInfo(self.edtFilterEventEndSetTime.time()) if self.chkFilterEventSetDate.isChecked() else None
        eventBegExecDate = CDateInfo(self.edtFilterEventBegExecDate.date()) if self.chkFilterEventExecDate.isChecked() else None
        eventEndExecDate = CDateInfo(self.edtFilterEventEndExecDate.date()) if self.chkFilterEventExecDate.isChecked() else None
        eventBegExecTime = CTimeInfo(self.edtFilterEventBegExecTime.time()) if self.chkFilterEventExecDate.isChecked() else None
        eventEndExecTime = CTimeInfo(self.edtFilterEventEndExecTime.time()) if self.chkFilterEventExecDate.isChecked() else None
        eventOrgStructure = context.getInstance(COrgStructureInfo, forceRef(self.cmbFilterEventOrgStructure.value())) if self.chkFilterEventOrgStructure.isChecked() else None
        eventSpeciality = context.getInstance(CSpecialityInfo, forceRef(self.cmbFilterEventSpeciality.value())) if self.chkFilterEventSpeciality.isChecked() else None
        eventPerson = context.getInstance(CPersonInfo, forceRef(self.cmbFilterEventPerson.value())) if self.chkFilterEventPerson.isChecked() else None
        filter = {'eventType': eventType if eventType else None,
                    'eventBegSetDate' : eventBegSetDate,
                    'eventBegSetTime' : eventBegSetTime,
                    'eventEndSetDate' : eventEndSetDate,
                    'eventEndSetTime' : eventEndSetTime,
                    'eventBegExecDate' : eventBegExecDate,
                    'eventBegExecTime' : eventBegExecTime,
                    'eventEndExecDate' : eventEndExecDate,
                    'eventEndExecTime' : eventEndExecTime,
                    'eventOrgStructure' : eventOrgStructure if eventOrgStructure else None,
                    'eventSpeciality' : eventSpeciality if eventSpeciality else None,
                    'eventPerson' : eventPerson if eventPerson else None
        }
        data = { 'event' : eventInfo,
                 'eventList' : eventInfoList,
                 'client': eventInfo.client,
                 'tempInvalid': tempInvalidInfo,
                 'filter' : filter
                }
        return data


    @pyqtSignature('')
    def on_btnEventFilter_clicked(self):
        for s in self.chkListOnEventsPage:
            chk = s[0]
            if chk.isChecked():
                self.activateFilterWdgets(s[1])
                return
        self.chkFilterEventType.setChecked(True)

    def onCheckEventToggled(self, checked):
        if checked:
            self.chkFilterEventId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)

    @pyqtSignature('bool')
    def on_chkFilterEventSetDate_toggled(self, checked):
        self.onCheckEventToggled(checked)

    @pyqtSignature('bool')
    def on_chkFilterEventNextDate_toggled(self, checked):
        self.onCheckEventToggled(checked)

    @pyqtSignature('bool')
    def on_chkFilterEventEmptyExecDate_toggled(self, checked):
        if checked:
            self.chkFilterEventId.setChecked(False)
            self.chkFilterEventExecDate.setChecked(False)


    @pyqtSignature('bool')
    def on_chkFilterEventExecDate_toggled(self, checked):
        if checked:
            self.chkFilterEventId.setChecked(False)
            self.chkFilterEventEmptyExecDate.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterEventPurpose_toggled(self, checked):
        if checked:
            self.chkFilterEventId.setChecked(False)
            self.updateFilterEventResultTable()
            self.setCmbFilterEventTypeFilter(self.cmbFilterEventPurpose.value())
        else:
            self.setCmbFilterEventTypeFilter(None)
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('int')
    def on_cmbFilterEventPurpose_currentIndexChanged(self, index):
        self.setCmbFilterEventTypeFilter(self.cmbFilterEventPurpose.value())
        self.updateFilterEventResultTable()


    @pyqtSignature('bool')
    def on_chkFilterEventType_toggled(self, checked):
        if checked:
            self.chkFilterEventId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)
        if checked:
            self.updateFilterEventResultTable()


    @pyqtSignature('int')
    def on_cmbFilterEventType_currentIndexChanged(self, index):
        self.updateFilterEventResultTable()


    @pyqtSignature('bool')
    def on_chkFilterEventSpeciality_toggled(self, checked):
        if checked:
            self.chkFilterEventId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)
        if checked:
            self.cmbFilterEventPerson.setSpecialityId(self.cmbFilterEventSpeciality.value())
        else:
            self.cmbFilterEventPerson.setSpecialityId(None)


    @pyqtSignature('bool')
    def on_chkFilterEventOrgStructure_toggled(self, checked):
        if checked:
            self.chkFilterEventId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)
        if checked:
            self.cmbFilterEventPerson.setOrgStructureId(self.cmbFilterEventOrgStructure.value())
        else:
            self.cmbFilterEventPerson.setOrgStructureId(None)


    @pyqtSignature('int')
    def on_cmbFilterEventSpeciality_currentIndexChanged(self, index):
        self.cmbFilterEventPerson.setSpecialityId(self.cmbFilterEventSpeciality.value())


    @pyqtSignature('bool')
    def on_chkFilterEventPerson_toggled(self, checked):
        self.onCheckEventToggled(checked)

    @pyqtSignature('bool')
    def on_chkFilterEventIsPrimary_toggled(self, checked):
        self.onCheckEventToggled(checked)

    @pyqtSignature('bool')
    def on_chkFilterEventOrder_toggled(self, checked):
        self.onCheckEventToggled(checked)

    @pyqtSignature('bool')
    def on_chkRelegateOrg_toggled(self, checked):
        self.onCheckEventToggled(checked)

    @pyqtSignature('bool')
    def on_chkFilterEventDispanserObserved_toggled(self, checked):
        self.onCheckEventToggled(checked)


    @pyqtSignature('bool')
    def on_chkFilterEventLPU_toggled(self, checked):
        if checked:
            self.chkFilterEventId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)
        if checked:
            db = QtGui.qApp.db
            orgId = forceRef(db.translate('Event', 'id', self.currentEventId(), 'org_id'))
            self.cmbFilterEventLPU.setValue(orgId)
            self.cmbFilterEventPerson.setOrgId(orgId)
            self.chkFilterEventNonBase.setChecked(False)
        else:
            self.cmbFilterEventLPU.setValue(None)
            self.cmbFilterEventPerson.setOrgId(None)


    @pyqtSignature('int')
    def on_cmbFilterEventLPU_currentIndexChanged(self, index):
        self.cmbFilterEventPerson.setOrgId(self.cmbFilterEventLPU.value())


    @pyqtSignature('bool')
    def on_chkFilterEventNonBase_toggled(self, checked):
        if checked:
            self.chkFilterEventId.setChecked(False)
            self.chkFilterEventLPU.setChecked(False)


    @pyqtSignature('bool')
    def on_chkFilterEventMes_toggled(self, checked):
        self.onCheckEventToggled(checked)

    @pyqtSignature('bool')
    def on_chkMKB_toggled(self, checked):
        self.onCheckEventToggled(checked)

    @pyqtSignature('bool')
    def on_chkActionMKB_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)

    @pyqtSignature('bool')
    def on_chkFilterEventResult_toggled(self, checked):
        if checked:
            self.chkFilterEventId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)
        if checked:
            self.updateFilterEventResultTable()


    @pyqtSignature('bool')
    def on_chkFilterEventCreatePerson_toggled(self, checked):
        self.onCheckEventToggled(checked)


    @pyqtSignature('bool')
    def on_chkFilterEventCreateDate_toggled(self, checked):
        self.onCheckEventToggled(checked)


    @pyqtSignature('bool')
    def on_chkFilterEventModifyPerson_toggled(self, checked):
        self.onCheckEventToggled(checked)


    @pyqtSignature('bool')
    def on_chkFilterEventModifyDate_toggled(self, checked):
        self.onCheckEventToggled(checked)


    @pyqtSignature('bool')
    def on_chkFilterEventPayStatus_toggled(self, checked):
        self.onCheckEventToggled(checked)


    @pyqtSignature('bool')
    def on_chkFilterVisitPayStatus_toggled(self, checked):
        self.onCheckEventToggled(checked)


    @pyqtSignature('bool')
    def on_chkErrorInDiagnostic_toggled(self, checked):
        if checked:
            self.chkFilterEventId.setChecked(False)


    @pyqtSignature('bool')
    def on_chkFilterJobTicketId_toggled(self, checked):
        self.onCheckEventToggled(checked)


    @pyqtSignature('bool')
    def on_chkFilterVisits_toggled(self, checked):
        self.onCheckEventToggled(checked)


    @pyqtSignature('bool')
    def on_chkFilterEventPayer_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('')
    def on_btnFilterEventSelectPayer_clicked(self):
        orgId = selectOrganisation(self, self.cmbFilterEventPayer.value(), False)
        self.cmbFilterEventPayer.updateModel()
        if orgId:
            self.cmbFilterEventPayer.setValue(orgId)


    @pyqtSignature('bool')
    def on_chkFilterEventContract_toggled(self, checked):
        self.onCheckEventToggled(checked)


    @pyqtSignature('bool')
    def on_chkFilterEventExpertId_toggled(self, checked):
        self.onCheckEventToggled(checked)


    @pyqtSignature('bool')
    def on_chkFilterEventExpertiseDate_toggled(self, checked):
        self.onCheckEventToggled(checked)


    @pyqtSignature('')
    def on_btnFilterEventContract_clicked(self):
        currentDate = QDate.currentDate()
        self.filterEventContractParams = {}
        self.filterEventContractParams['financeId'] = forceRef(QtGui.qApp.preferences.appPrefs.get('Registry_filterEventContractParams_financeId', None))
        self.filterEventContractParams['groupingIndex'] = forceInt(QtGui.qApp.preferences.appPrefs.get('Registry_filterEventContractParams_groupingIndex', 0))
        self.filterEventContractParams['grouping'] = forceString(QtGui.qApp.preferences.appPrefs.get('Registry_filterEventContractParams_grouping', u''))
        self.filterEventContractParams['resolutionIndex'] = forceInt(QtGui.qApp.preferences.appPrefs.get('Registry_filterEventContractParams_resolutionIndex', 0))
        self.filterEventContractParams['resolution'] = forceString(QtGui.qApp.preferences.appPrefs.get('Registry_filterEventContractParams_resolution', u''))
        self.filterEventContractParams['priceList'] = forceInt(QtGui.qApp.preferences.appPrefs.get('Registry_filterEventContractParams_priceList', 0))
        self.filterEventContractParams['edtBegDate'] = forceDate(QtGui.qApp.preferences.appPrefs.get('Registry_filterEventContractParams_edtBegDate',QDate(currentDate.year(), 1, 1)))
        self.filterEventContractParams['edtEndDate'] = forceDate(QtGui.qApp.preferences.appPrefs.get('Registry_filterEventContractParams_edtEndDate', QDate(currentDate.year(), 12, 31)))
        self.filterEventContractParams['enableInAccounts'] = forceInt(QtGui.qApp.preferences.appPrefs.get('Registry_filterEventContractParams_enableInAccounts', 0))
        (contractId, contractDescr, filterEventContractParams)= selectContract(self, self.__filterEventContractId, self.filterEventContractParams)
        self.__filterEventContractId = contractId
        self.filterEventContractParams = filterEventContractParams
        self.edtFilterEventContract.setText(contractDescr)
        QtGui.qApp.preferences.appPrefs['Registry_filterEventContractParams_financeId'] = toVariant(self.filterEventContractParams.get('financeId', None))
        QtGui.qApp.preferences.appPrefs['Registry_filterEventContractParams_groupingIndex'] = toVariant(self.filterEventContractParams.get('groupingIndex', 0))
        QtGui.qApp.preferences.appPrefs['Registry_filterEventContractParams_grouping'] = toVariant(self.filterEventContractParams.get('grouping', u''))
        QtGui.qApp.preferences.appPrefs['Registry_filterEventContractParams_resolutionIndex'] = toVariant(self.filterEventContractParams.get('resolutionIndex', 0))
        QtGui.qApp.preferences.appPrefs['Registry_filterEventContractParams_resolution'] = toVariant(self.filterEventContractParams.get('resolution', u''))
        QtGui.qApp.preferences.appPrefs['Registry_filterEventContractParams_priceList'] = toVariant(self.filterEventContractParams.get('priceList', 0))
        QtGui.qApp.preferences.appPrefs['Registry_filterEventContractParams_edtBegDate'] = toVariant(self.filterEventContractParams.get('edtBegDate', QDate(currentDate.year(), 1, 1)))
        QtGui.qApp.preferences.appPrefs['Registry_filterEventContractParams_edtEndDate'] = toVariant(self.filterEventContractParams.get('edtEndDate', QDate(currentDate.year(), 12, 31)))
        QtGui.qApp.preferences.appPrefs['Registry_filterEventContractParams_enableInAccounts'] = toVariant(self.filterEventContractParams.get('enableInAccounts', 0))


    @pyqtSignature('bool')
    def on_chkFilterEventId_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)
        if checked:
            for chk in (self.chkFilterEventCreatePerson,
                        self.chkFilterEventCreateDate,
                        self.chkFilterEventModifyPerson,
                        self.chkFilterEventModifyDate,
                        self.chkErrorInDiagnostic,
                        self.chkFilterEventPayStatus,
                        self.chkFilterVisitPayStatus,
                        self.chkFilterEventSetDate,
                        self.chkFilterEventEmptyExecDate,
                        self.chkFilterEventExecDate,
                        self.chkFilterEventNextDate,
                        self.chkFilterEventOrgStructure,
                        self.chkFilterEventSpeciality,
                        self.chkFilterEventPerson,
                        self.chkRelegateOrg,
                        self.chkFilterEventDispanserObserved,
                        self.chkFilterEventLPU,
                        self.chkFilterEventNonBase,
                        self.chkFilterEventMes,
                        self.chkMKB,
                        self.chkFilterEventResult,
                        self.chkFilterExternalId,
                        self.chkFilterAccountSumLimit,
                        self.chkFilterVisits,
                        self.chkFilterEventPayer,
                        self.chkFilterEventContract,
                        self.chkFilterEventExpertId,
                        self.chkFilterEventExpertiseDate,
                        self.chkFilterJobTicketId):
                chk.setChecked(False)
            self.cmbFilterEventByClient.setCurrentIndex(2)


    @pyqtSignature('bool')
    def on_chkFilterExternalId_toggled(self, checked):
        self.onCheckEventToggled(checked)


    @pyqtSignature('bool')
    def on_chkFilterAccountSumLimit_toggled(self, checked):
        if checked:
            self.chkFilterEventId.setChecked(False)
        self.onChkFilterToggled(self.sender(), checked)
        if self.cmbFilterAccountSumLimit.currentIndex() == 0:
            self.edtFilterSumLimitDelta.setValue(0)
            self.edtFilterSumLimitDelta.setEnabled(False)


    @pyqtSignature('int')
    def on_cmbFilterAccountSumLimit_currentIndexChanged(self, index):
        if index == 0:
            self.edtFilterSumLimitDelta.setValue(0)
            self.edtFilterSumLimitDelta.setEnabled(False)
        else:
            self.edtFilterSumLimitDelta.setEnabled(True)


    @pyqtSignature('QAbstractButton*')
    def on_buttonBoxEvent_clicked(self, button):
        buttonCode = self.buttonBoxEvent.standardButton(button)
        if buttonCode == QtGui.QDialogButtonBox.Apply:
            self.on_buttonBoxEvent_apply()
        elif buttonCode == QtGui.QDialogButtonBox.Reset:
            self.on_buttonBoxEvent_reset()


#    @pyqtSignature('')
    def on_buttonBoxEvent_reset(self):
        for s in self.chkListOnEventsPage:
            chk = s[0]
            if chk.isChecked():
                self.deactivateFilterWdgets(s[1])
                chk.setChecked(False)
        self.tblEvents.setOrder(None)
        self.cmbFilterEventByClient.setCurrentIndex(0)


#    @pyqtSignature('')
    def on_buttonBoxEvent_apply(self):
        filter = {}
        if self.chkFilterEventSetDate.isChecked():
            filter['begSetDateTime'] = QDateTime(self.edtFilterEventBegSetDate.date(),  self.edtFilterEventBegSetTime.time())
            filter['endSetDateTime'] = QDateTime(self.edtFilterEventEndSetDate.date(),  self.edtFilterEventEndSetTime.time())
        filter['emptyExecDate'] = self.chkFilterEventEmptyExecDate.isChecked()
        if self.chkFilterEventExecDate.isChecked():
            filter['begExecDateTime'] = QDateTime(self.edtFilterEventBegExecDate.date(), self.edtFilterEventBegExecTime.time())
            filter['endExecDateTime'] = QDateTime(self.edtFilterEventEndExecDate.date(), self.edtFilterEventEndExecTime.time())
        if self.chkFilterEventNextDate.isChecked():
            filter['begNextDate'] = self.edtFilterEventBegNextDate.date()
            filter['endNextDate'] = self.edtFilterEventEndNextDate.date()
        if self.chkFilterEventPurpose.isChecked():
            filter['eventPurposeId'] = self.cmbFilterEventPurpose.value()
        if self.chkFilterEventIsPrimary.isChecked():
            filter['isPrimary'] = self.cmbFilterEventIsPrimary.currentIndex()+1
        if self.chkFilterEventOrder.isChecked():
            filter['order'] = self.cmbFilterEventOrder.currentIndex()+1
        if self.chkFilterEventType.isChecked():
            filter['eventTypeId'] = self.cmbFilterEventType.value()
        if self.chkFilterEventOrgStructure.isChecked():
            filter['orgStructureId'] = self.cmbFilterEventOrgStructure.value()
        if self.chkFilterEventSpeciality.isChecked():
            filter['specialityId'] = self.cmbFilterEventSpeciality.value()
        if self.chkFilterEventPerson.isChecked():
            filter['personId'] = self.cmbFilterEventPerson.value()
        if self.chkRelegateOrg.isChecked():
            filter['relegateOrgId'] = self.cmbRelegateOrg.value()
        filter['dispanserObserved'] = self.chkFilterEventDispanserObserved.isChecked()
        if self.chkFilterEventLPU.isChecked():
            filter['LPUId'] = self.cmbFilterEventLPU.value()
        else:
            filter['nonBase'] = self.chkFilterEventNonBase.isChecked()
        filter['errorInDiagnostic'] = self.chkErrorInDiagnostic.isChecked()
        if self.chkFilterEventMes.isChecked():
            filter['mesCode'] = forceStringEx(self.edtFilterEventMes.text())

        if self.chkMKB.isChecked():
            filter['MKBFrom'] = MKBwithoutSubclassification(unicode(self.edtMKBFrom.text()))
            filter['MKBTo']   = MKBwithoutSubclassification(unicode(self.edtMKBTo.text()))
            filter['MKBisPreliminary'] = self.chkPreliminary.isChecked()
            filter['MKBisConcomitant'] = self.chkConcomitant.isChecked()

        if self.chkFilterEventResult.isChecked():
            filter['eventResultId'] = self.cmbFilterEventResult.value()
#        if self.chkFilterEventTempInvalid.isChecked():
#            filter['begTempInvalidDate'] = self.edtFilterEventBegTempInvalid.date()
#            filter['endTempInvalidDate'] = self.edtFilterEventEndTempInvalid.date()
        if self.chkFilterEventCreatePerson.isChecked():
            filter['createPersonId'] = self.cmbFilterEventCreatePerson.value()
        if self.chkFilterEventCreateDate.isChecked():
            filter['begCreateDate'] = self.edtFilterEventBegCreateDate.date()
            filter['endCreateDate'] = self.edtFilterEventEndCreateDate.date()
        if self.chkFilterEventModifyPerson.isChecked():
            filter['modifyPersonId'] = self.cmbFilterEventModifyPerson.value()
        if self.chkFilterEventModifyDate.isChecked():
            filter['begModifyDate'] = self.edtFilterEventBegModifyDate.date()
            filter['endModifyDate'] = self.edtFilterEventEndModifyDate.date()
        if self.chkFilterEventPayStatus.isChecked():
            filter['payStatusCode'] = self.cmpFilterEventPayStatusCode.currentIndex()
            index = self.cmbFilterEventPayStatusFinance.currentIndex()
            if not 0<=index<5:
                index = 0
            filter['payStatusFinanceCode'] = 5-index
        if self.chkFilterVisitPayStatus.isChecked():
            filter['payStatusCodeVisit'] = self.cmpFilterVisitPayStatusCode.currentIndex()
            index = self.cmbFilterVisitPayStatusFinance.currentIndex()
            if not 0<=index<5:
                index = 0
            filter['payStatusFinanceCodeVisit'] = 5-index
        if self.chkFilterEventPayer.isChecked():
            filter['payerId'] = self.cmbFilterEventPayer.value()
        if self.chkFilterEventContract.isChecked() and self.__filterEventContractId:
            filter['contractId'] = self.__filterEventContractId
        if self.chkFilterEventExpertId.isChecked():
           filter['eventExpertId'] = self.cmbFilterEventExpertId.value()
        if self.chkFilterEventExpertiseDate.isChecked():
           filter['begExpertiseDate'] = self.edtFilterEventBegExpertiseDate.date()
           filter['endExpertiseDate'] = self.edtFilterEventEndExpertiseDate.date()
        if self.chkFilterEventId.isChecked():
            tmp = forceStringEx(self.edtFilterEventId.text())
            if tmp:
                filter['id'] = int(tmp)
        if self.chkFilterJobTicketId.isChecked():
            tmp = forceStringEx(self.edtFilterJobTicketId.text())
            if tmp:
                filter['jobTicketId'] = int(tmp)
        if self.chkFilterExternalId.isChecked():
            tmpExternalId = forceStringEx(self.edtFilterExternalId.text())
            if tmpExternalId:
                filter['externalId'] = tmpExternalId
        if self.chkFilterAccountSumLimit.isChecked():
            filter['accountSumLimit'] = forceInt(self.cmbFilterAccountSumLimit.currentIndex())
            filter['sumLimitFrom'] = forceInt(self.edtFilterSumLimitFrom.value())
            filter['sumLimitTo'] = forceInt(self.edtFilterSumLimitTo.value())
            filter['sumLimitDelta'] = forceInt(self.edtFilterSumLimitDelta.value())
        filter['filterVisits'] = self.chkFilterVisits.isChecked()
        if self.chkFilterVisits.isChecked():
            filter['visitScene'] = self.cmbFilterVisitScene.value()
            filter['visitType'] = self.cmbFilterVisitType.value()
            filter['visitProfile'] = self.cmbFilterVisitProfile.value()
        filterEventByClient = self.cmbFilterEventByClient.currentIndex()
        if filterEventByClient == 0:
            filter['clientIds'] = [ self.selectedClientId() ]
        elif filterEventByClient == 1:
            filter['clientIds'] = self.modelClients.idList()
        filter['filterEventByClient'] = filterEventByClient
        self.updateEventsList(filter)
        self.focusEvents()

#### AmbCard page #################################

    @pyqtSignature('')
    def on_actAmbCardEditClient_triggered(self):
        clientId = self.currentClientId()
        if clientId:
            self.editClient(clientId)
            self.updateAmbCardInfo()
            self.updateClientsListRequest = True

### Actions page ##################################

    @pyqtSignature('')
    def on_actActionEditClients_triggered(self):
        clientId = self.currentClientId()
        if clientId:
            self.editClient(clientId)
            self.updateActionInfo(None)
            self.updateClientsListRequest = True


    @pyqtSignature('int')
    def on_tabWidgetActionsClasses_currentChanged(self, index):
        self.cmbFilterActionType.setClassesPopup([index])
        self.cmbFilterActionType.setClass(index)
        self.cmbFilterActionType.setValue(self.__actionTypeIdListByClassPage[index])
        self.on_buttonBoxAction_apply()


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelVisits_currentRowChanged(self, current, previous):
        visitId = self.tblVisits.currentItemId()
        self.updateVisitInfo(visitId)


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelActionsStatus_currentRowChanged(self, current, previous):
        actionId = self.tblActionsStatus.currentItemId()
        self.updateActionInfo(actionId)
        self.tabAmbCardContent.updateAmbCardPropertiesTable(current, self.tblActionsStatusProperties, previous)


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelActionsDiagnostic_currentRowChanged(self, current, previous):
        actionId = self.tblActionsDiagnostic.currentItemId()
        self.updateActionInfo(actionId)
        self.tabAmbCardContent.updateAmbCardPropertiesTable(current, self.tblActionsDiagnosticProperties, previous)


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelActionsCure_currentRowChanged(self, current, previous):
        actionId = self.tblActionsCure.currentItemId()
        self.updateActionInfo(actionId)
        self.tabAmbCardContent.updateAmbCardPropertiesTable(current, self.tblActionsCureProperties, previous)


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelActionsMisc_currentRowChanged(self, current, previous):
        actionId = self.tblActionsMisc.currentItemId()
        self.updateActionInfo(actionId)
        self.tabAmbCardContent.updateAmbCardPropertiesTable(current, self.tblActionsMiscProperties, previous)


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelExpertDirectionsMC_currentRowChanged(self, current, previous):
        actionId = self.tblExpertDirectionsMC.currentItemId()
        self.updateMedicalCommissionInfo(actionId)
        self.tabAmbCardContent.updateAmbCardPropertiesTable(current, self.tblExpertDirectionsPropertiesMC, previous)


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelExpertProtocolsMC_currentRowChanged(self, current, previous):
        actionId = self.tblExpertProtocolsMC.currentItemId()
        self.updateMedicalCommissionInfo(actionId)
        self.tabAmbCardContent.updateAmbCardPropertiesTable(current, self.tblExpertProtocolsPropertiesMC, previous)


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelExpertMedicalSocialInspection_currentRowChanged(self, current, previous):
        actionId = self.tblExpertMedicalSocialInspection.currentItemId()
        self.updateMedicalCommissionInfo(actionId)
        self.tabAmbCardContent.updateAmbCardPropertiesTable(current, self.tblExpertMSIProperties, previous)


    @pyqtSignature('int')
    def on_modelExpertDirectionsMC_itemsCountChanged(self, count):
        self.lblExpertDirectionsMCCount.setText(formatRecordsCount(count))


    @pyqtSignature('int')
    def on_modelExpertProtocolsMC_itemsCountChanged(self, count):
        self.lblExpertProtocolsMCCount.setText(formatRecordsCount(count))


    @pyqtSignature('int')
    def on_modelExpertMedicalSocialInspection_itemsCountChanged(self, count):
        self.lblExpertMedicalSocialInspectionCount.setText(formatRecordsCount(count))


    @pyqtSignature('int')
    def on_modelActionsStatus_itemsCountChanged(self, count):
        infoBeds, infoAddressOrgStructure = self.getFilterInfoBed()
        self.lblActionsStatusCount.setText(formatRecordsCount(count)+infoBeds+infoAddressOrgStructure)
        self.lblActionsStatusAmountSum.setText(u'Количество: %.2f УЕТ: %.2f'% (self.modelActionsStatus.getAmountSum(), self.modelActionsStatus.getUetSum()))


    @pyqtSignature('int')
    def on_modelVisits_itemsCountChanged(self, count):
        infoBeds, infoAddressOrgStructure = self.getFilterInfoBed()
        self.lblVisitCount.setText(formatRecordsCount(count)+infoBeds+infoAddressOrgStructure)


    @pyqtSignature('int')
    def on_modelActionsDiagnostic_itemsCountChanged(self, count):
        self.lblActionsDiagnosticCount.setText(formatRecordsCount(count))
        self.lblActionsDiagnosticAmountSum.setText(u'Количество: %.2f  УЕТ: %.2f' % (self.modelActionsDiagnostic.getAmountSum(), self.modelActionsDiagnostic.getUetSum()))


    @pyqtSignature('int')
    def on_modelActionsCure_itemsCountChanged(self, count):
        self.lblActionsCureCount.setText(formatRecordsCount(count))
        self.lblActionsCureAmountSum.setText(u'Количество: %.2f  УЕТ: %.2f' % (self.modelActionsCure.getAmountSum(),  self.modelActionsCure.getUetSum()))


    @pyqtSignature('int')
    def on_modelActionsMisc_itemsCountChanged(self, count):
        self.lblActionsMiscCount.setText(formatRecordsCount(count))
        self.lblActionsMiscAmountSum.setText(u'Количество: %.2f  УЕТ: %.2f' % (self.modelActionsMisc.getAmountSum(),  self.modelActionsMisc.getUetSum()))


    @pyqtSignature('QModelIndex')
    def on_tblActionsStatus_doubleClicked(self, index):
        self.on_btnActionEdit_clicked()


    @pyqtSignature('QModelIndex')
    def on_tblActionsDiagnostic_doubleClicked(self, index):
        self.on_btnActionEdit_clicked()


    @pyqtSignature('QModelIndex')
    def on_tblActionsCure_doubleClicked(self, index):
        self.on_btnActionEdit_clicked()


    @pyqtSignature('QModelIndex')
    def on_tblActionsMisc_doubleClicked(self, index):
        self.on_btnActionEdit_clicked()


    @pyqtSignature('')
    def on_actOpenAccountingByEvent_triggered(self):
        eventId = self.tblEvents.currentItemId()
        self.showAccountingDialog(eventId)


    @pyqtSignature('')
    def on_actUpdateEventTypeByEvent_triggered(self):
        eventId = self.tblEvents.currentItemId()
        if eventId:
            if self.tblEvents.eventHasAccountItem(eventId):
                QtGui.QMessageBox.warning(self, u'Внимание!', u'По событию выставлен счёт, поэтому его тип не может быть изменён.', QtGui.QMessageBox.Ok, QtGui.QMessageBox.Ok)
            else:
                self.updateEventTypeByEvent(eventId)


    def updateEventTypeByEvent(self, eventId):
        if QtGui.qApp.userHasAnyRight([urAdmin, urUpdateEventTypeByEvent]):
            db = QtGui.qApp.db
            tableET = db.table('Event')
            record = db.getRecordEx(tableET, '*', [tableET['id'].eq(eventId), tableET['deleted'].eq(0)])
            eventTypeId = forceRef(record.value('eventType_id')) if record else None
            oldPrevEventId = forceRef(record.value('prevEvent_id')) if record else None
            clientId    = forceRef(record.value('client_id')) if record else None
            if eventTypeId:
                tableETE = db.table('EventType_Event')
                cols = [tableETE['eventType_id']
                       ]
                cond = [tableETE['master_id'].eq(eventTypeId)
                       ]
                eventTypeIdList = db.getDistinctIdList(tableETE, cols, cond, 'EventType_Event.id')
                if eventTypeIdList:
                    dialog = CUpdateEventTypeByEvent(self, eventTypeIdList, eventTypeId)
                    try:
                        if dialog.exec_():
                            newEventTypeId = dialog.getNewEventTypeId()
                            if newEventTypeId:
                                record.setValue('eventType_id', toVariant(newEventTypeId))
                                idList = set([])
                                idListParents = set(db.getTheseAndParents(tableET, 'prevEvent_id', [eventId if eventId else self.prevEventId]))
                                idList ^= idListParents
                                idListDescendant = set(db.getDescendants(tableET, 'prevEvent_id', eventId if eventId else self.prevEventId))
                                idList ^= idListDescendant
                                if len(idList) < 2:
                                    prevEventTypeId = getEventPrevEventTypeId(newEventTypeId)
                                    prevEventId = getPrevEventIdByEventTypeId(prevEventTypeId, clientId)
                                    if oldPrevEventId != prevEventId:
                                        record.setValue('prevEvent_id', toVariant(prevEventId))
                                db.updateRecord(tableET, record)
                                self.updateEventListAfterEdit(eventId)
                    finally:
                        dialog.destroy()
                        sip.delete(dialog)
                        del dialog


    @pyqtSignature('')
    def on_actConcatEvents_triggered(self):
        eventId = self.tblEvents.currentItemId()
        self.concatEvents(eventId)


    @pyqtSignature('')
    def on_actMakePersonalAccount_triggered(self):
        eventId = self.tblEvents.currentItemId()
        self.makePersonalAccount(eventId)


    @pyqtSignature('')
    def on_actOpenAccountingByAction_triggered(self):
        actionId = self.tblEventActions.currentItemId()
        self.showAccountingDialog(None, actionId)


    @pyqtSignature('')
    def on_actOpenAccountingBySingleActionStatus_triggered(self):
        actionId = self.tblActionsStatus.currentItemId()
        self.showAccountingDialog(None, actionId)


    @pyqtSignature('')
    def on_actOpenAccountingBySingleVisits_triggered(self):
        visitId = self.tblVisits.currentItemId()
        self.showAccountingDialog(None, None, visitId)


    @pyqtSignature('')
    def on_actOpenAccountingBySingleActionDiagnostic_triggered(self):
        actionId = self.tblActionsDiagnostic.currentItemId()
        self.showAccountingDialog(None, actionId)

    @pyqtSignature('')
    def on_actOpenAccountingBySingleActionCure_triggered(self):
        actionId = self.tblActionsCure.currentItemId()
        self.showAccountingDialog(None, actionId)

    @pyqtSignature('')
    def on_actOpenAccountingBySingleActionMisc_triggered(self):
        actionId = self.tblActionsMisc.currentItemId()
        self.showAccountingDialog(None, actionId)


    @pyqtSignature('')
    def on_actOpenAccountingByVisit_triggered(self):
        visitId = self.tblEventVisits.currentItemId()
        self.showAccountingDialog(None, None, visitId)


    @pyqtSignature('')
    def on_actEditActionEvent_triggered(self):
        self.on_btnActionEventEdit_clicked()


    @pyqtSignature('')
    def on_btnActionEventEdit_clicked(self):
        actionId = self.currentActionId()
        if actionId:
            eventId = forceRef(QtGui.qApp.db.translate('Action', 'id', actionId, 'event_id'))
            if eventId:
                if editEvent(self, eventId):
                   self.on_buttonBoxAction_apply()
        self.focusActions()


    @pyqtSignature('')
    def on_actEditVisitEvent_triggered(self):
        visitId = self.tblVisits.currentItemId()
        if visitId:
            eventId = self.getEventIdForVisit(visitId)
            if eventId:
                if editEvent(self, eventId):
                   self.on_buttonBoxAction_apply()
        self.focusActions()


    @pyqtSignature('QModelIndex')
    def on_tblVisits_doubleClicked(self, index):
        self.on_actEditVisitEvent_triggered()


    def canChangePayStatusAdditional(self, table, id):
        db = QtGui.qApp.db
        tableEvent = db.table('Event')
        tableCan = db.table(table)
        tableQuery = tableEvent.innerJoin(tableCan, tableCan['event_id'].eq(tableEvent['id']))
        record = db.getRecordEx(tableQuery, 'Event.*, %s.payStatus AS canPayStatus'%(table), [tableCan['id'].eq(id),
                                                                              tableEvent['deleted'].eq(0),
                                                                              tableCan['deleted'].eq(0)])
        if record:
            payStatus = forceInt(record.value('payStatus'))
            isClosed  = forceInt(record.value('isClosed'))
            payStatusCan  = forceInt(record.value('canPayStatus'))
            if payStatus or (isClosed and payStatusCan):
                if QtGui.qApp.userHasRight(urEditAfterInvoicingEvent):
                    message = u''
                    if payStatus:
                        message = u'Данное событие выставлено в счёт\nи его данные изменять нежелательно!\nВы настаиваете на изменении?'
                    elif payStatusCan:
                        message = u'%s из закрытого события были выставлены в счет\nи его данные изменять нежелательно!\nВы настаиваете на изменении?'%(u'Действия' if (table.lower() in 'Action'.lower()) else (u'Визиты' if (table.lower() in 'Visit'.lower()) else u''))
                    if not QtGui.QMessageBox.critical(self, u'Внимание!', message, QtGui.QMessageBox.Yes|QtGui.QMessageBox.No, QtGui.QMessageBox.No) == QtGui.QMessageBox.Yes:
                        return False
                else:
                    message = u''
                    if payStatus:
                        message = u'Данное событие выставлено в счёт, редактирование запрещено!'
                    elif payStatusCan:
                        message = u'%s из закрытого события были выставлены в счет, редактирование запрещено!'%(u'Действия' if (table.lower() in 'Action'.lower()) else (u'Визиты' if (table.lower() in 'Visit'.lower()) else u''))
                    QtGui.QMessageBox.critical(self, u'Внимание!', message)
                    return False
        return True


    @pyqtSignature('')
    def on_btnActionEdit_clicked(self):
        actionId = self.currentActionId()
        if actionId and self.canChangePayStatusAdditional('Action', actionId):
            self.editAction(actionId)
        self.focusActions()


    def expertMedicalCommissionActionEdit(self):
        currentTable = self.getCurrentExpertMedicalCommissionTable()
        actionId = currentTable.currentItemId()
        if actionId and self.canChangePayStatusAdditional('Action', actionId):
            dialog = CActionEditDialog(self)
            try:
                dialog.load(actionId)
                if dialog.exec_():
                    self.updateMedicalCommissionList(self.__expertMCFilter, dialog.itemId())
                    currentTable.model().updateColumnsCaches(actionId)
                    if self.tabExpertMedicalCommissionWidget.currentIndex() == self.tabExpertMedicalCommissionWidget.indexOf(self.tabExpertMedicalSocialInspection):
                        self.tblExpertProtocolsMC.model().updateColumnsCaches(actionId)
                    QtGui.qApp.emitCurrentClientInfoChanged()
                    return dialog.itemId()
                else:
                    self.updateMedicalCommissionInfo(actionId)
                    self.updateClientsListRequest = True
                return None
            finally:
                dialog.destroy()
                sip.delete(dialog)
                del dialog
        self.focusExpertMC()


    @pyqtSignature('')
    def on_btnVisitEdit_clicked(self):
        visitId = self.tblVisits.currentItemId()
        if visitId and self.canChangePayStatusAdditional('Visit', visitId):
            eventId = self.getEventIdForVisit(visitId)
            if eventId:
                eventId = editEvent(self, eventId)
        self.tblVisits.setFocus(Qt.TabFocusReason)


    def getEventIdForVisit(self, visitId):
        db = QtGui.qApp.db
        table = db.table('Visit')
        record = db.getRecordEx(table, [table['event_id']], [table['id'].eq(visitId)])
        if record:
            return forceRef(record.value('event_id'))
        return None


    @pyqtSignature('int')
    def on_btnActionPrint_printByTemplate(self, templateId):
        if templateId == -1:
            tblActions = self.getCurrentActionsTable()
            tblActions.setReportHeader(u'Список мероприятий')
            tblActions.setReportDescription(self.getActionFilterAsText())
    #        if self.cmbFilterAction.currentIndex() == 0:
    #            mask = [False]*3 + [True]*(model.columnCount()-3)
    #        else:
    #            mask = [True]*(model.columnCount())
    #        tblActions.printContent(mask)
            tblActions.printContent()
            self.focusActions()
        else:
            tblActions = self.getCurrentActionsTable()
            idList = tblActions.model().idList()
            context = CInfoContext()
            if self.cmbFilterActionType.value():
                actionType = context.getInstance(CActionTypeInfo, (CActionTypeCache.getById(forceRef(self.cmbFilterActionType.value()))))
            else:
                actionType = None
            actionBegSetDate = CDateInfo(self.edtFilterActionBegSetDate.date()) if self.chkFilterActionSetDate.isChecked() else None
            actionEndSetDate = CDateInfo(self.edtFilterActionEndSetDate.date()) if self.chkFilterActionSetDate.isChecked() else None
            actionBegExecDate = CDateInfo(self.edtFilterActionBegExecDate.date()) if self.chkFilterActionExecDate.isChecked() else None
            actionEndExecDate = CDateInfo(self.edtFilterActionEndExecDate.date()) if self.chkFilterActionExecDate.isChecked() else None
            actionBegExecTime = CTimeInfo(self.edtFilterActionBegExecTime.time()) if self.chkFilterActionExecDate.isChecked() else None
            actionEndExecTime = CTimeInfo(self.edtFilterActionEndExecTime.time()) if self.chkFilterActionExecDate.isChecked() else None
            actionExecOrgStructure = context.getInstance(COrgStructureInfo, forceRef(self.cmbFilterExecSetOrgStructure.value()))
            actionExecSpeciality = context.getInstance(CSpecialityInfo, forceRef(self.cmbFilterExecSetSpeciality.value()))
            actionExecSetPerson = context.getInstance(CPersonInfo, forceRef(self.cmbFilterExecSetPerson.value()))
            actionExecSetAssistant = context.getInstance(CPersonInfo, forceRef(self.cmbFilterAssistant.value()))
            filter = {'actionType' : actionType,
                        'actionBegSetDate' : actionBegSetDate,
                        'actionEndSetDate' : actionEndSetDate,
                        'actionBegExecDate' : actionBegExecDate,
                        'actionEndExecDate' : actionEndExecDate,
                        'actionBegExecTime' : actionBegExecTime,
                        'actionEndExecTime' : actionEndExecTime,
                        'actionExecOrgStructure' : actionExecOrgStructure if actionExecOrgStructure else None,
                        'actionExecSpeciality' : actionExecSpeciality if actionExecSpeciality else None,
                        'actionExecSetPerson' : actionExecSetPerson if actionExecSetPerson else None,
                        'actionExecSetAssistant' : actionExecSetAssistant if actionExecSetAssistant else None
                        }
            data = { 'actions': CLocActionInfoList(context, idList),
                            'filter':filter}
            QtGui.qApp.call(self, applyTemplate, (self, templateId, data))


    @pyqtSignature('int')
    def on_btnVisitPrint_printByTemplate(self, templateId):
        if templateId == -1:
            self.tblVisits.setReportHeader(u'Список визитов')
            self.tblVisits.setReportDescription(self.getVisitFilterAsText())
            self.tblVisits.printContent()
            self.focusEvents()
        else:
            context = CInfoContext()
            visitInfo = context.getInstance(CVisitInfo, self.tblVisits.currentItemId())
            visitInfoList = context.getInstance(CVisitInfoListEx, tuple(self.tblVisits.model().idList()))
            data = { 'visitList'    : visitInfoList,
                     'visit'        : visitInfo,
                     'filterVisitBegDate'   : CDateInfo(self.edtFilterVisitBegExecDate.date()) if self.chkFilterVisitExecDate.isChecked() else None,
                     'filterVisitBegTime'   : CTimeInfo(self.edtFilterVisitBegExecTime.time()) if self.chkFilterVisitExecDate.isChecked() else None,
                     'filterVisitEndDate'   : CDateInfo(self.edtFilterVisitEndExecDate.date()) if self.chkFilterVisitExecDate.isChecked() else None,
                     'filterVisitEndTime'   : CTimeInfo(self.edtFilterVisitEndExecTime.time()) if self.chkFilterVisitExecDate.isChecked() else None,
                     'filterVisitExecSetOrgStructure': context.getInstance(COrgStructureInfo, self.cmbFilterExecSetOrgStructureVisit.value() if self.chkFilterExecSetOrgStructureVisit.isChecked() else None),
                     'filterVisitExecSetSpeciality'  : context.getInstance(CSpecialityInfo, self.cmbFilterExecSetSpecialityVisit.value() if self.chkFilterExecSetSpecialityVisit.isChecked() else None),
                     'filterVisitExecSetPerson'      : context.getInstance(CPersonInfo, self.cmbFilterExecSetPersonVisit.value() if self.chkFilterExecSetPersonVisit.isChecked() else None),
                     'filterVisitAssistant'          : context.getInstance(CPersonInfo, self.cmbFilterVisitAssistant.value() if self.chkFilterVisitAssistant.isChecked() else None),
                     'filterVisitFinance'            : context.getInstance(CFinanceInfo, self.cmbFilterVisitFinance.value() if self.chkFilterVisitFinance.isChecked() else None),
                     'filterVisitService'            : context.getInstance(CServiceInfo, self.cmbFilterServiceVisit.value() if self.chkFilterServiceVisit.isChecked() else None),
                     'filterVisitScene'              : context.getInstance(CSceneInfo, self.cmbFilterSceneVisit.value() if self.chkFilterSceneVisit.isChecked() else None),
                     'filterVisitCreatePersonEx'     : context.getInstance(CPersonInfo, self.cmbFilterVisitCreatePersonEx.value() if self.chkFilterVisitCreatePersonEx.isChecked() else None),
                     'filterVisitBegCreateDateEx'    : CDateInfo(self.edtFilterVisitBegCreateDateEx.date()) if self.chkFilterVisitCreateDateEx.isChecked() else None,
                     'filterVisitEndCreateDateEx'    : CDateInfo(self.edtFilterVisitEndCreateDateEx.date()) if self.chkFilterVisitCreateDateEx.isChecked() else None,
                     'filterVisitModifyPersonEx'     : context.getInstance(CPersonInfo, self.cmbFilterVisitModifyPersonEx.value() if self.chkFilterVisitModifyPersonEx.isChecked() else None),
                     'filterVisitBegModifyDateEx'    : CDateInfo(self.edtFilterVisitBegModifyDateEx.date()) if self.chkFilterVisitModifyDateEx.isChecked() else None,
                     'filterVisitEndModifyDateEx'    : CDateInfo(self.edtFilterVisitEndModifyDateEx.date()) if self.chkFilterVisitModifyDateEx.isChecked() else None,
                     'filterVisitPayStatusCodeEx'    : forceString(self.cmpFilterVisitPayStatusCodeEx.currentText()) if self.chkFilterVisitPayStatusEx.isChecked() else '',
                     'filterVisitPayStatusFinanceEx' : forceString(self.cmbFilterVisitPayStatusFinanceEx.currentText()) if self.chkFilterVisitPayStatusEx.isChecked() else ''
                     }
            QtGui.qApp.call(self, applyTemplate, (self, templateId, data))


    @pyqtSignature('int')
    def on_btnPrint_printByTemplate(self, clientId):
        context = CInfoContext()
        clientInfo = context.getInstance(CClientInfo, self.tblClients.currentItemId())
        clientInfoList = context.getInstance(CClientInfoListEx, tuple(self.tblClients.model().idList()))
        data = { 'clientsList': clientInfoList,
                 'client'    : clientInfo }
        QtGui.qApp.call(self, applyTemplate, (self, clientId, data))


    @pyqtSignature('')
    def on_btnActionFilter_clicked(self):
        for s in self.chkListOnActionsPage:
            chk = s[0]
            if chk.isChecked():
                self.activateFilterWdgets(s[1])
                return
        self.chkFilterActionType.setChecked(True)


    @pyqtSignature('')
    def on_btnVisitFilter_clicked(self):
        for s in self.chkListOnVisitsPage:
            chk = s[0]
            if chk.isChecked():
                self.activateFilterWdgets(s[1])
                return
        self.chkFilterVisitType.setChecked(True)


    @pyqtSignature('bool')
    def on_chkFilterActionType_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)
        if not checked or not self.chkTakeIntoAccountProperty.isChecked():
            self.enabledOptionProperty(checked)
            self.chkListProperty.setChecked(False)
            self.chkListProperty.setEnabled(False)
            self.tblListOptionProperty.setEnabled(False)


    @pyqtSignature('bool')
    def on_chkThresholdPenaltyGrade_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkListProperty_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)
        actionTypeId = forceRef(self.cmbFilterActionType.value())
        self.enabledOptionProperty(checked, actionTypeId)


    def makePersonalAccount(self, eventId):
        db = QtGui.qApp.db
        tableEvent = db.table('Event')
        tableContract = db.table('Contract')
        tableFinance = db.table('rbFinance')
        table = tableEvent.innerJoin(tableContract, tableContract['id'].eq(tableEvent['contract_id']))
        table = table.innerJoin(tableFinance, tableFinance['id'].eq(tableContract['finance_id']))
        cond = [tableEvent['id'].eq(eventId)]
        contractRecord = db.getRecordEx(table, 'Contract.*,rbFinance.name as financeName', cond)
        number = forceString(contractRecord.value('number'))
        finance = forceString(contractRecord.value('financeName'))
        mbResult = QtGui.QMessageBox.question(self,
            u'Внимание!', u'Договор номер %s, (%s)\nДействительно сформировать счет?'%(number, finance),
            QtGui.QMessageBox.Yes|QtGui.QMessageBox.No, QtGui.QMessageBox.No)
        if {QtGui.QMessageBox.Yes: True,
                QtGui.QMessageBox.No: False}.get(mbResult, None):
            try:
                from Accounting.InstantAccountDialog import createInstantAccount
                createInstantAccount(eventId, False)
            except:
                QtGui.qApp.logCurrentException()


    def concatEvents(self, eventId):
        mbResult = QtGui.QMessageBox.question(self,
            u'Внимание!', u'Действительно выполнить слияние событий?',
            QtGui.QMessageBox.Yes|QtGui.QMessageBox.No, QtGui.QMessageBox.No)
        if {QtGui.QMessageBox.Yes: True,
                QtGui.QMessageBox.No: False}.get(mbResult, None):
            db = QtGui.qApp.db
            record = self.modelEvents.getRecordById(eventId)
            if record:
                prevId = forceRef(record.value('prevEvent_id'))
                if prevId:
                    prevRecord = self.modelEvents.getRecordById(prevId)
                    prevRecord.setValue('result_id', record.value('result_id'))
                    prevRecord.setValue('execDate', record.value('execDate'))
                    tableAction = db.table('Action')
                    actionList = db.getRecordList(tableAction, '*', tableAction['event_id'].eq(eventId))
                    for action in actionList:
                        action.setValue('event_id', prevId)
                        db.updateRecord(tableAction, action)
                    actionList = db.getRecordList(tableAction, '*', tableAction['event_id'].eq(prevId), 'begDate')
                    for i, action in enumerate(actionList):
                        action.setValue('idx', i)
                        db.updateRecord(tableAction, action)
                    db.updateRecord('Event', prevRecord)
                    self.tblEvents.removeCurrentRow(False)
                else:
                    QtGui.QMessageBox.information(self,
                                              u'Предупреждение!',
                                              u'У данного обращения нет связанных событий!',
                                              QtGui.QMessageBox.Ok,
                                              QtGui.QMessageBox.Ok)


    def enabledOptionProperty(self, checked, actionTypeId = None):
        if actionTypeId and checked:
            self.bufferRecordsOptionProperty = []
            self.tblListOptionProperty.setEnabled(True)
            QtGui.qApp.callWithWaitCursor(self, self.printDataOptiontPropertys, actionTypeId)
        else:
            self.bufferRecordsOptionProperty = []
            self.tblListOptionProperty.selectAll()
            self.tblListOptionProperty.clear()
            self.tblListOptionProperty.setEnabled(False)


    def printDataOptiontPropertys(self, typeActionId = None):
        self.bufferRecordsOptionProperty = []
        self.rows = 0
        if typeActionId:
            db = QtGui.qApp.db
            table = db.table('ActionPropertyType')
            cols = [table['id'], table['name']]
            cond = [table['actionType_id'].eq(typeActionId)]
            records = db.getRecordList(table, cols, cond, u'name')
            for record in records:
                nameProperty = u''
                self.bufferRecordsOptionProperty.append(record)
                nameProperty = forceString(record.value('name'))
                self.tblListOptionProperty.addItem(CFilterPropertyOptionsItem(nameProperty, self.tblListOptionProperty))
                item = self.tblListOptionProperty.item(self.rows)
                self.tblListOptionProperty.scrollToItem(item)
                self.rows += 1


    def getDataOptionPropertyChecked(self):
        optionPropertyIdList = []
        rowCount = 0
        rowCount = self.tblListOptionProperty.count() - 1
        while rowCount >= 0:
            optionPropertyItem = self.tblListOptionProperty.item(rowCount)
            if optionPropertyItem:
                if optionPropertyItem.checkState() == Qt.Checked:
                    record = self.bufferRecordsOptionProperty[rowCount]
                    optionPropertyIdList.append(forceRef(record.value('id')))
            rowCount -= 1
        return optionPropertyIdList


    @pyqtSignature('bool')
    def on_chkFilledProperty_toggled(self, checked):
        if checked:
            self.chkThresholdPenaltyGrade.setChecked(False)
            self.chkThresholdPenaltyGrade.setEnabled(False)
            self.edtThresholdPenaltyGrade.setEnabled(False)
        elif self.chkTakeIntoAccountProperty.isChecked():
             self.chkThresholdPenaltyGrade.setEnabled(True)


    @pyqtSignature('bool')
    def on_chkTakeIntoAccountProperty_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)
        if not checked or not self.chkFilterActionType.isChecked():
            self.enabledOptionProperty(checked)
            self.chkListProperty.setChecked(False)
            self.chkListProperty.setEnabled(False)
            self.tblListOptionProperty.setEnabled(False)
        elif checked and self.chkFilterActionType.isChecked():
              self.chkListProperty.setEnabled(True)
        if not self.chkThresholdPenaltyGrade.isChecked():
            self.edtThresholdPenaltyGrade.setEnabled(False)


    @pyqtSignature('int')
    def on_cmbFilterActionType_currentIndexChanged(self, index):
        index = self.tabWidgetActionsClasses.currentIndex()
        self.__actionTypeIdListByClassPage[index] = self.cmbFilterActionType.value()
        actionTypeId = forceRef(self.cmbFilterActionType.value())
        self.enabledOptionProperty(self.chkListProperty.isChecked(), actionTypeId)

    @pyqtSignature('bool')
    def on_chkFilterActionPayer_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('')
    def on_btnFilterActionSelectPayer_clicked(self):
        orgId = selectOrganisation(self, self.cmbFilterActionPayer.value(), False)
        self.cmbFilterActionPayer.updateModel()
        if orgId:
            self.cmbFilterActionPayer.setValue(orgId)


    @pyqtSignature('bool')
    def on_chkFilterActionContract_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)

    @pyqtSignature('')
    def on_btnFilterActionContract_clicked(self):
        (contractId, contractDescr, filterActionContractParams)= selectContract(self, self.__filterActionContractId)
        self.__filterActionContractId = contractId
        self.edtFilterActionContract.setText(contractDescr)


    @pyqtSignature('bool')
    def on_chkFilterActionSetDate_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterActionSetSpeciality_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)
        if checked:
            self.chkFilterActionSetPerson.setChecked(False)
            self.cmbFilterActionSetPerson.setSpecialityId(self.cmbFilterActionSetSpeciality.value())
        else:
            self.cmbFilterActionSetPerson.setSpecialityId(None)


    @pyqtSignature('bool')
    def on_chkFilterActionSetOrgStructure_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)
        if checked:
            self.chkFilterActionSetPerson.setChecked(False)
            self.cmbFilterActionSetPerson.setOrgStructureId(self.cmbFilterActionSetOrgStructure.value())
        else:
            self.cmbFilterActionSetPerson.setOrgStructureId(None)


    @pyqtSignature('bool')
    def on_chkFilterExecSetOrgStructure_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)
        if checked:
            self.chkFilterExecSetPerson.setChecked(False)
            execSetOrgStructure = self.cmbFilterExecSetOrgStructure.value()
            if not execSetOrgStructure:
                execSetOrgStructure = QtGui.qApp.currentOrgStructureId()
                self.cmbFilterExecSetOrgStructure.setValue(execSetOrgStructure)
            self.cmbFilterActionSetPerson.setOrgStructureId(execSetOrgStructure)
        else:
            self.cmbFilterActionSetPerson.setOrgStructureId(None)


    @pyqtSignature('bool')
    def on_chkFilterExecSetSpeciality_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)
        if checked:
            self.chkFilterExecSetPerson.setChecked(False)
            self.cmbFilterExecSetPerson.setSpecialityId(self.cmbFilterExecSetSpeciality.value())
        else:
            self.cmbFilterExecSetPerson.setSpecialityId(None)


    @pyqtSignature('int')
    def on_cmbFilterActionSetSpeciality_currentIndexChanged(self, index):
        self.cmbFilterActionSetPerson.setSpecialityId(self.cmbFilterActionSetSpeciality.value())


    @pyqtSignature('int')
    def on_cmbFilterExecSetSpeciality_currentIndexChanged(self, index):
        self.cmbFilterExecSetPerson.setSpecialityId(self.cmbFilterExecSetSpeciality.value())


    @pyqtSignature('bool')
    def on_chkFilterActionSetPerson_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)
        self.chkFilterActionSetOrgStructure.setChecked(False)
        self.chkFilterActionSetSpeciality.setChecked(False)


    @pyqtSignature('bool')
    def on_chkFilterExecSetPerson_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)
        self.chkFilterExecSetOrgStructure.setChecked(False)
        self.chkFilterExecSetSpeciality.setChecked(False)


    @pyqtSignature('bool')
    def on_chkFilterAssistant_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterActionFinance_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterActionJobTicketId_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterActionPlannedEndDate_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterActionStatus_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterActionExecDate_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterActionCreatePerson_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterActionCreateDate_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterActionModifyPerson_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterActionModifyDate_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterActionPayStatus_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkTakenTissueJournal_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterActionId_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('QAbstractButton*')
    def on_buttonBoxAction_clicked(self, button):
        buttonCode = self.buttonBoxAction.standardButton(button)
        if buttonCode == QtGui.QDialogButtonBox.Apply:
            self.on_buttonBoxAction_apply()
        elif buttonCode == QtGui.QDialogButtonBox.Reset:
            self.on_buttonBoxAction_reset()


#    @pyqtSignature('')
    def on_buttonBoxAction_reset(self):
        for s in self.chkListOnActionsPage:
            chk = s[0]
            if chk.isChecked():
                self.deactivateFilterWdgets(s[1])
                chk.setChecked(False)
        self.cmbFilterAction.setCurrentIndex(0)


#    @pyqtSignature('')
    def on_buttonBoxAction_apply(self):
        filter = {}
        if self.chkFilterActionType.isChecked():
            filter['actionTypeId'] = self.cmbFilterActionType.value()
        if self.chkFilterActionStatus.isChecked():
            filter['status'] = self.cmbFilterActionStatus.value()
        if self.chkFilterActionSetDate.isChecked():
            filter['begSetDate'] = self.edtFilterActionBegSetDate.date()
            filter['endSetDate'] = self.edtFilterActionEndSetDate.date()
        if self.chkFilterActionSetSpeciality.isChecked():
            filter['setSpecialityId'] = self.cmbFilterActionSetSpeciality.value()
        if self.chkFilterActionSetOrgStructure.isChecked():
            filter['actionSetOrgStructureId'] = self.cmbFilterActionSetOrgStructure.value()
        if self.chkFilterExecSetOrgStructure.isChecked():
            filter['execSetOrgStructureId'] = self.cmbFilterExecSetOrgStructure.value()
        if self.chkFilterActionSetPerson.isChecked():
            filter['setPersonId'] = self.cmbFilterActionSetPerson.value()
        if self.chkFilterExecSetSpeciality.isChecked():
            filter['execSpecialityId'] = self.cmbFilterExecSetSpeciality.value()
        if self.chkFilterExecSetPerson.isChecked():
            filter['execPersonId'] = self.cmbFilterExecSetPerson.value()
        if self.chkFilterAssistant.isChecked():
            filter['assistantId'] = self.cmbFilterAssistant.value()
        if self.chkFilterActionFinance.isChecked():
            filter['financeId'] = self.cmbFilterActionFinance.value()
        if self.chkFilterActionPayer.isChecked():
            filter['payerId'] = self.cmbFilterActionPayer.value()
        if self.chkFilterActionContract.isChecked():
            filter['contractId'] = self.__filterActionContractId
        if self.chkUncoordinated.isChecked():
            filter['uncoordinated'] = self.chkUncoordinated.isChecked()
        if self.chkActionMKB.isChecked():
            filter['actionMKB'] = self.chkActionMKB.isChecked()
            filter['actionMKBFrom'] = MKBwithoutSubclassification(unicode(self.edtActionMKBFrom.text()))
            filter['actionMKBTo'] = MKBwithoutSubclassification(unicode(self.edtActionMKBTo.text()))
        if self.chkFilterActionJobTicketId.isChecked():
            tmp = forceStringEx(self.edtFilterActionJobTicketId.text())
            if tmp:
                filter['actionJobTicketId'] = int(tmp)
        if self.chkFilterActionIsUrgent.isChecked():
            filter['isUrgent'] = True
        if self.chkFilterActionPlannedEndDate.isChecked():
            filter['begPlannedEndDate'] = self.edtFilterActionBegPlannedEndDate.date()
            filter['endPlannedEndDate'] = self.edtFilterActionEndPlannedEndDate.date()
        if self.chkFilterActionExecDate.isChecked():
            filter['begExecDateTime'] = QDateTime(self.edtFilterActionBegExecDate.date(), self.edtFilterActionBegExecTime.time())
            filter['endExecDateTime'] = QDateTime(self.edtFilterActionEndExecDate.date(), self.edtFilterActionEndExecTime.time())
        if self.chkFilterActionCreatePerson.isChecked():
            filter['createPersonId'] = self.cmbFilterActionCreatePerson.value()
        if self.chkFilterActionCreateDate.isChecked():
            filter['begCreateDate'] = self.edtFilterActionBegCreateDate.date()
            filter['endCreateDate'] = self.edtFilterActionEndCreateDate.date()
        if self.chkFilterActionModifyPerson.isChecked():
            filter['modifyPersonId'] = self.cmbFilterActionModifyPerson.value()
        if self.chkFilterActionModifyDate.isChecked():
            filter['begModifyDate'] = self.edtFilterActionBegModifyDate.date()
            filter['endModifyDate'] = self.edtFilterActionEndModifyDate.date()
        if self.chkFilterActionPayStatus.isChecked():
            filter['payStatusCode'] = self.cmpFilterActionPayStatusCode.currentIndex()
            index = self.cmbFilterActionPayStatusFinance.currentIndex()
            if not 0<=index<5:
                index = 0
            filter['payStatusFinanceCode'] = 5-index
        if self.chkTakenTissueJournal.isChecked():
            filter['takenTissueJournal'] = self.cmbTakenTissueJournal.currentIndex() == 0
        actionsFilterType = self.cmbFilterAction.currentIndex()
        if actionsFilterType == 0:
            filter['clientIds'] = [ self.selectedClientId() ]
        elif actionsFilterType == 1:
            filter['clientIds'] = self.modelClients.idList()
        elif actionsFilterType == 2:
            filter['eventIds'] = [ self.tblEvents.currentItemId() ]
        elif actionsFilterType == 3:
            filter['eventIds'] = self.modelEvents.idList()
        filter['actionsFilterType']=actionsFilterType
        filter['booleanFilledProperty'] = self.chkFilledProperty.isChecked() #True = заполнено; False = не заполнено
        if self.chkThresholdPenaltyGrade.isChecked():
            filter['thresholdPenaltyGrade'] = self.edtThresholdPenaltyGrade.text()


        self.updateActionsList(filter)
        self.focusEvents()


### Expert page ##################################

    @pyqtSignature('')
    def on_actExpertEditClient_triggered(self):
        clientId = self.currentClientId()
        if clientId:
            self.editClient(clientId)
            if self.getCurrentExpertTypesTabWidgetDetails().currentIndex():
                self.updateDocumentsExInfo(self.getCurrentTempDocumentsExTable().currentItemId(), noPrev=True)
            else:
                self.updateTempInvalidInfo(self.getCurrentExpertTable().currentItemId(), noPrev=True)
            self.updateClientsListRequest = True


    def setBtnExpertEditEnabled(self):
        tableList = [self.tblExpertTempInvalid, self.tblExpertDisability, self.tblExpertVitalRestriction, self.getCurrentExpertMedicalCommissionTable()]
        self.btnExpertEdit.setEnabled(getRightEditTempInvalid(tableList[self.tabWidgetTempInvalidTypes.currentIndex()].currentItemId()))


    @pyqtSignature('int')
    def on_tabExpertMedicalCommissionWidget_currentChanged(self, index):
        self.on_buttonBoxExpertMC_apply()
        self.btnExpertEdit.setEnabled(QtGui.qApp.userHasRight(urRegTabEditExpertMC))


    @pyqtSignature('int')
    def on_tabWidgetTempInvalidTypes_currentChanged(self, index):
        if index == self.tabWidgetTempInvalidTypes.indexOf(self.tabExpertMedicalCommission):
            self.stwExpertFilter.setCurrentWidget(self.tabMedicalCommissionExpertFilter)
            self.cmbFilterExpertSpecialityMC.setTable('rbSpeciality', True)
            self.on_buttonBoxExpertMC_apply()
            rightEditExpertMC = QtGui.qApp.userHasRight(urRegTabEditExpertMC)
            enable = rightEditExpertMC and bool(len(self.getCurrentExpertMedicalCommissionTable().model().idList()) > 0)
            self.btnExpertEdit.setEnabled(enable)
            self.btnExpertPrint.setEnabled(enable)
            self.btnExpertFilter.setEnabled(enable)
            self.btnExpertNew.setEnabled(False)
        else:
            self.stwExpertFilter.setCurrentWidget(self.tabTempInvalidExpertFilter)
            filter = 'type=%d'%index
            self.cmbFilterExpertDocType.setTable('rbTempInvalidDocument', False, filter)
            self.cmbFilterExpertReason.setTable('rbTempInvalidReason', False, filter)
            self.cmbFilterExpertSpeciality.setTable('rbSpeciality', True)
            self.cmbFilterExpertDocType.setValue(self.__tempInvalidDocTypeIdListByTypePage[index])
            self.on_buttonBoxExpert_apply()
            self.setBtnExpertEditEnabled()
            self.btnExpertNew.setEnabled(True)


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelExpertTempInvalid_currentRowChanged(self, current, previous):
        tempInvalidId = self.tblExpertTempInvalid.currentItemId()
        self.updateTempInvalidInfo(tempInvalidId, noPrev=True)
        self.setBtnExpertEditEnabled()


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelExpertTempInvalidRelation_currentRowChanged(self, current, previous):
        tempInvalidId = self.tblExpertTempInvalidRelation.currentItemId()
        self.updateTempInvalidInfo(tempInvalidId)
        self.setBtnExpertEditEnabled()


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelExpertTempInvalidDocumentsEx_currentRowChanged(self, current, previous):
        self.updateDocumentsExInfo(self.tblExpertTempInvalidDocumentsEx.currentItemId(), noPrev=True)
        self.setBtnExpertEditEnabled()


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelExpertDisabilityDocumentsEx_currentRowChanged(self, current, previous):
        self.updateDocumentsExInfo(self.tblExpertDisabilityDocumentsEx.currentItemId(), noPrev=True)
        self.setBtnExpertEditEnabled()


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelExpertVitalRestrictionDocumentsEx_currentRowChanged(self, current, previous):
        self.updateDocumentsExInfo(self.tblExpertVitalRestrictionDocumentsEx.currentItemId(), noPrev=True)
        self.setBtnExpertEditEnabled()


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelExpertTempInvalidRelationDocumentsEx_currentRowChanged(self, current, previous):
        self.updateDocumentsExInfo(self.tblExpertTempInvalidRelationDocumentsEx.currentItemId())
        self.setBtnExpertEditEnabled()


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelExpertDisabilityRelationDocumentsEx_currentRowChanged(self, current, previous):
        self.updateDocumentsExInfo(self.tblExpertDisabilityRelationDocumentsEx.currentItemId())
        self.setBtnExpertEditEnabled()


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelExpertVitalRestrictionRelationDocumentsEx_currentRowChanged(self, current, previous):
        self.updateDocumentsExInfo(self.tblExpertVitalRestrictionRelationDocumentsEx.currentItemId())
        self.setBtnExpertEditEnabled()


    def getCurrentDocumentsTempInvalid(self):
        index = self.tabWidgetTempInvalidTypes.currentIndex()
        return [self.edtExpertDocumentsTempInvalid, self.edtExpertDisabilityDocumentsTempInvalid, self.edtExpertVitalRestrictionDocumentsTempInvalid][index]


    def updateDocumentsExInfo(self, documentId, noPrev=False):
        isFilterExpertLinked = self.chkFilterExpertLinked.isChecked()
        if documentId:
            db = QtGui.qApp.db
            tableTempInvalidDocument = db.table('TempInvalidDocument')
            tablePeriod = db.table('TempInvalid_Period')
            record = db.getRecordEx(tableTempInvalidDocument, [tableTempInvalidDocument['master_id']], [tableTempInvalidDocument['id'].eq(documentId), tableTempInvalidDocument['deleted'].eq(0)], tableTempInvalidDocument['idx'].name())
            tempInvalidId = forceRef(record.value('master_id')) if record else None
            self.getCurrentDocumentsTempInvalid().loadData(tempInvalidId)
            self.setClientInfoBrowserExpert(tempInvalidId)
            if tempInvalidId:
                idList = db.getDistinctIdList(tablePeriod, 'id', tablePeriod['master_id'].eq(tempInvalidId), 'endDate')
            else:
                idList = []
            self.getCurrentTempDocumentsExPeriodsTable().setIdList(idList)
            if isFilterExpertLinked:
                if noPrev:
                    prevIdList = [documentId]
                    prevIdSet = set([documentId])
                    while prevIdList:
                        childrenIdSet = set(db.getDistinctIdList(tableTempInvalidDocument, ['id'], [tableTempInvalidDocument['prev_id'].inlist(prevIdList), tableTempInvalidDocument['deleted'].eq(0)], tableTempInvalidDocument['idx'].name()))
                        newIdSet = childrenIdSet-prevIdSet
                        prevIdSet |= newIdSet
                        prevIdList = list(newIdSet)
                    documentRelationIdList = list(prevIdSet-set([documentId]))
                    self.getCurrentExpertRelationDocumentsExTable().setIdList(documentRelationIdList)
        else:
            self.getCurrentTempDocumentsExPeriodsTable().setIdList([])
            self.getCurrentDocumentsTempInvalid().loadData(None)
            if isFilterExpertLinked and noPrev:
                self.getCurrentExpertRelationDocumentsExTable().setIdList([])



    def setClientInfoBrowserExpert(self, tempInvalidId):
        clientId = None
        if tempInvalidId:
            db = QtGui.qApp.db
            tableTempInvalid = db.table('TempInvalid')
            record = db.getRecordEx(tableTempInvalid, [tableTempInvalid['client_id']], [tableTempInvalid['id'].eq(tempInvalidId), tableTempInvalid['deleted'].eq(0)])
            clientId = forceRef(record.value('client_id')) if record else None
        if not clientId:
            clientId = self.selectedClientId()
        if clientId:
            self.txtClientInfoBrowserExpert.setHtml(getClientBanner(clientId))
        else:
            self.txtClientInfoBrowserExpert.setText('')
        self.actExpertEditClient.setEnabled(bool(clientId))
        self.actExpertRelationsClient.setEnabled(bool(clientId))
        QtGui.qApp.setCurrentClientId(clientId)


    @pyqtSignature('')
    def on_tblExpertDirectionsMC_popupMenuAboutToShow(self):
        self.expertMC_popupMenuAboutToShow(self.tblExpertDirectionsMC)


    @pyqtSignature('')
    def on_tblExpertProtocolsMC_popupMenuAboutToShow(self):
        self.expertMC_popupMenuAboutToShow(self.tblExpertProtocolsMC)


    @pyqtSignature('')
    def on_ExpertMedicalSocialInspection_popupMenuAboutToShow(self):
        self.expertMC_popupMenuAboutToShow(self.ExpertMedicalSocialinspection)


    def expertMC_popupMenuAboutToShow(self, table):
        currentId = table.currentItemId()
        userRight = QtGui.qApp.userHasRight(urRegTabEditExpertMC)
        prevActionId = self.getPrevActionIdMSI(currentId)
        self.actExpertMedicalCommissionDelete.setEnabled(bool(currentId) and userRight and (not prevActionId))
        self.actExpertMedicalCommissionSelected.setEnabled(bool(currentId) and userRight)
        self.actExpertMedicalCommissionUpdateGroup.setEnabled(bool(currentId) and userRight and len(self.getCurrentExpertMedicalCommissionTable().selectedItemIdList()) > 1 and len(self.medicalCommissionSelectedRows) > 0)
        self.actExpertMedicalCommissionClear.setEnabled(bool(currentId) and userRight)
        self.actExpertMedicalCommissionUpdate.setEnabled(bool(currentId) and userRight)
        self.actExpertMCUpdateTempInvalid.setEnabled(bool(currentId) and userRight)
        self.actExpertMedicalCommissionMSI.setEnabled(bool(currentId) and userRight and (not prevActionId))


    def getPrevActionIdMSI(self, currentId):
        prevActionId = None
        if currentId:
            db = QtGui.qApp.db
            tableAction = db.table('Action')
            record = db.getRecordEx(tableAction, [tableAction['id']], [tableAction['prevAction_id'].eq(currentId), tableAction['deleted'].eq(0)])
            prevActionId = forceRef(record.value('id')) if record else None
        return prevActionId


    @pyqtSignature('')
    def on_tblExpertTempInvalid_popupMenuAboutToShow(self):
        tempInvalidId = self.tblExpertTempInvalid.currentItemId()
        prevId = self.getExpertPrevDocId(self.tblExpertTempInvalid)
        nextId = self.getExpertNextDocId(self.tblExpertTempInvalid)
        self.actExpertTempInvalidNext.setEnabled(bool(nextId))
        self.actExpertTempInvalidPrev.setEnabled(bool(prevId))


    @pyqtSignature('')
    def on_tblExpertDisability_popupMenuAboutToShow(self):
        tempInvalidId = self.tblExpertDisability.currentItemId()
        prevId = self.getExpertPrevDocId(self.tblExpertDisability)
        nextId = self.getExpertNextDocId(self.tblExpertDisability)
        self.actExpertDisabilityNext.setEnabled(bool(nextId))
        self.actExpertDisabilityPrev.setEnabled(bool(prevId))
#        app = QtGui.qApp
        self.actExpertDisabilityDelete.setEnabled(bool(tempInvalidId) and getRightEditTempInvalid(tempInvalidId))


    @pyqtSignature('')
    def on_tblExpertVitalRestriction_popupMenuAboutToShow(self):
        tempInvalidId = self.tblExpertVitalRestriction.currentItemId()
        prevId = self.getExpertPrevDocId(self.tblExpertVitalRestriction)
        nextId = self.getExpertNextDocId(self.tblExpertVitalRestriction)
        self.actExpertVitalRestrictionNext.setEnabled(bool(nextId))
        self.actExpertVitalRestrictionPrev.setEnabled(bool(prevId))
#        app = QtGui.qApp
        self.actExpertVitalRestrictionDelete.setEnabled(bool(tempInvalidId) and getRightEditTempInvalid(tempInvalidId))


    @pyqtSignature('')
    def on_actExpertTempInvalidNext_triggered(self):
        self.onExpertDocNext(self.tblExpertTempInvalid)


    @pyqtSignature('')
    def on_actExpertDisabilityNext_triggered(self):
        self.onExpertDocNext(self.tblExpertDisability)


    @pyqtSignature('')
    def on_actExpertVitalRestrictionNext_triggered(self):
        self.onExpertDocNext(self.tblExpertVitalRestriction)


    def onExpertDocNext(self, table):
        nextId = self.getExpertNextDocId(table)
        if nextId:
            row = table.model().findItemIdIndex(nextId)
            if row>=0:
                table.setCurrentRow(row)
            else:
                QtGui.QMessageBox.information(self, u'Переход к следующему документу', u'Переход невозможен - необходимо изменить фильтр')


    @pyqtSignature('')
    def on_tblClients_popupMenuAboutToShow(self):
        QtGui.qApp = QtGui.qApp
        clientPresent = bool(self.currentClientId())
        self.actCreateEvent.setEnabled(clientPresent)
        self.actShowContingentsClient.setEnabled(clientPresent)
        self.actOpenClientVaccinationCard.setEnabled(clientPresent and QtGui.qApp.userHasAnyRight([urCanReadClientVaccination, urCanEditClientVaccination]))
        self.actListVisitsBySchedules.setEnabled(clientPresent)
        self.actBatchRegLocatCard.setEnabled(QtGui.qApp.userHasAnyRight([urBatchRegLocatCardProcess]))
        self.actStatusObservationClient.setEnabled(clientPresent)
        self.actControlDoublesRecordClient.setEnabled(clientPresent and QtGui.qApp.userHasRight(urRegControlDoubles))
        self.actReservedOrderQueueClient.setEnabled(bool(QtGui.qApp.mainWindow.dockFreeQueue and QtGui.qApp.mainWindow.dockFreeQueue.content and QtGui.qApp.mainWindow.dockFreeQueue.content.locked()))


    @pyqtSignature('')
    def on_tblSchedules_popupMenuAboutToShow(self):
        notEmpty = self.modelSchedules.rowCount() > 0
        self.actAmbCreateEvent.setEnabled(notEmpty and QtGui.qApp.userHasAnyRight([urAdmin, urRegTabWriteEvents]))
        self.actAmbDeleteOrder.setEnabled(notEmpty)
        self.actAmbChangeNotes.setEnabled(notEmpty)
        self.actAmbPrintOrder.setEnabled(notEmpty)
        self.actPrintBeforeRecords.setEnabled(notEmpty)
        self.actShowPreRecordInfo.setEnabled(notEmpty)


    @pyqtSignature('')
    def on_tblVisitsBySchedules_popupMenuAboutToShow(self):
        notEmpty = self.modelVisitsBySchedules.rowCount() > 0
        self.actShowPreRecordInfoVisitBySchedule.setEnabled(notEmpty)


    @pyqtSignature('')
    def on_tblEvents_popupMenuAboutToShow(self):
        self.actOpenAccountingByEvent.setEnabled(self.modelEvents.rowCount()>0)
        eventId = self.tblEvents.currentItemId()
        self.actMakePersonalAccount.setEnabled(self.modelEvents.canMakePersonalAccount(eventId))
        self.actUpdateEventTypeByEvent.setEnabled(self.modelEvents.rowCount()>0 and QtGui.qApp.userHasAnyRight([urUpdateEventTypeByEvent]))


    @pyqtSignature('')
    def on_tblEventActions_popupMenuAboutToShow(self):
        self.actOpenAccountingByAction.setEnabled(self.modelEventActions.rowCount()>0)


    @pyqtSignature('')
    def on_tblEventVisits_popupMenuAboutToShow(self):
        self.actOpenAccountingByVisit.setEnabled(self.modelEventVisits.rowCount()>0)


    @pyqtSignature('')
    def on_actAmbCreateEvent_triggered(self):
        index = self.tblSchedules.currentIndex()
        if index.isValid():
            row = index.row()
            record = self.modelSchedules.items()[row]
            params = {}
            params['widget'] = self
            params['clientId'] = self.currentClientId()
            params['flagHospitalization'] = False
            params['actionTypeId'] = None
            params['valueProperties'] = []
            params['eventTypeFilterHospitalization'] = 0
            params['dateTime'] = forceDateTime(record.value('directionDate'))
            params['personId'] = forceRef(record.value('person_id'))
            requestNewEvent(params)


    @pyqtSignature('')
    def on_actAmbDeleteOrder_triggered(self):
        index = self.tblSchedules.currentIndex()
        if index.isValid():
            row = index.row()
            record = self.modelSchedules.items()[row]
            scheduleItemId = forceRef(record.value('id'))
            recordPersonId = forceRef(record.value('recordPerson_id'))
            clientId = self.currentClientId()
            confirmAndFreeScheduleItem(self, scheduleItemId, recordPersonId, clientId)
            self.updateQueue()
            if QtGui.qApp.mainWindow.dockResources:
                QtGui.qApp.mainWindow.dockResources.content.updateTimeTable()
            QtGui.qApp.emitCurrentClientInfoJLWDeleted(scheduleItemId)


    @pyqtSignature('')
    def on_actAmbChangeNotes_triggered(self):
        index = self.tblSchedules.currentIndex()
        if index.isValid():
            row = index.row()
            item = self.modelSchedules.items()[row]
            scheduleItemId = forceRef(item.value('id'))
            dlg = CComplaintsEditDialog(self)
            dlg.setComplaints(forceString(item.value('complaint')))
            if dlg.exec_():
                db = QtGui.qApp.db
                table = db.table('Schedule_Item')
                record = table.newRecord(['id', 'complaint'])
                record.setValue('id', toVariant(scheduleItemId))
                record.setValue('complaint', toVariant(dlg.getComplaints()))
                db.updateRecord(table, record)
#                self.modelSchedules.setValue(row, 'complaint', record.value('complaint'))
            self.updateQueue()
            if QtGui.qApp.mainWindow.dockResources:
                QtGui.qApp.mainWindow.dockResources.content.updateTimeTable()


    @pyqtSignature('')
    def on_actAmbPrintOrder_triggered(self):
        index = self.tblSchedules.currentIndex()
        if index.isValid():
            row = index.row()
            item = self.modelSchedules.items()[row]
            scheduleItemId = forceRef(item.value('id'))
            printOrderByScheduleItem(self, scheduleItemId, self.currentClientId())


    @pyqtSignature('')
    def on_actPrintBeforeRecords_triggered(self):
        if self.modelSchedules.rowCount():
            self.tblSchedules.setReportHeader(u'Протокол предварительной записи пациента')
            self.tblSchedules.setReportDescription(self.txtClientInfoBrowser.toHtml())
            mask = [False]
            for i in range(1, self.tblSchedules.model().columnCount()): mask.append(True)
            self.tblSchedules.printContent(mask)


    @pyqtSignature('')
    def on_actShowPreRecordInfo_triggered(self):
        row = self.tblSchedules.currentRow()
        if 0<=row<self.modelSchedules.rowCount():
            scheduleItemId = self.modelSchedules.getScheduleItemId(row)
            showScheduleItemInfo(scheduleItemId, self)


    @pyqtSignature('')
    def on_actShowPreRecordInfoVisitBySchedule_triggered(self):
        row = self.tblVisitsBySchedules.currentRow()
        if 0<=row<self.modelVisitsBySchedules.rowCount():
            scheduleItemId = self.modelVisitsBySchedules.getScheduleItemId(row)
            showScheduleItemInfo(scheduleItemId, self)


    @pyqtSignature('')
    def on_actControlDoublesRecordClient_triggered(self):
         clientId = self.currentClientId()
         if clientId:
            CRegistryControlDoubles(self, clientId).exec_()
            self.on_buttonBoxClient_apply()


    @pyqtSignature('')
    def on_actReservedOrderQueueClient_triggered(self):
        dockFreeQueue = QtGui.qApp.mainWindow.dockFreeQueue
        if dockFreeQueue and dockFreeQueue.content and dockFreeQueue.content.locked():
            dockFreeQueue.content.on_actAmbCreateOrder_triggered()


    @pyqtSignature('')
    def on_actOpenClientVaccinationCard_triggered(self):
        clientId = self.currentClientId()
        openClientVaccinationCard(self, clientId)



    @pyqtSignature('')
    def on_actListVisitsBySchedules_triggered(self):
        if QtGui.qApp.mainWindow.dockResources.content:
            personId = QtGui.qApp.mainWindow.dockResources.content.getCurrentPersonId()
        else:
            personId = None

        if personId:
            orgStructureId, specialityId = None, None
        else:
            orgStructureId = QtGui.qApp.currentOrgStructureId
            specialityId = QtGui.qApp.userSpecialityId

        clientId = self.currentClientId()
        CVisitsBySchedulesDialog(self, clientId, orgStructureId, specialityId, personId).exec_()


    @pyqtSignature('')
    def on_actBatchRegLocatCard_triggered(self):
        try:
            if QtGui.qApp.userHasAnyRight([urBatchRegLocatCardProcess]):
                dialog = CSetParamsBatchRegistrationLocationCard(self)
                try:
                    if dialog.exec_():
                        pass
                finally:
                    dialog.destroy()
                    sip.delete(dialog)
                    del dialog
        except:
            pass


    @pyqtSignature('')
    def on_actStatusObservationClient_triggered(self):
        self.updateStatusObservationClient()


    @pyqtSignature('')
    def on_actExpertMedicalCommissionDelete_triggered(self):
        table = self.getCurrentExpertMedicalCommissionTable()
        actionId = table.currentItemId()
        if actionId:
            if QtGui.QMessageBox.question(self,
                            u'Удаление документа', u'Вы действительно хотите удалить документ?',
                            QtGui.QMessageBox.Yes|QtGui.QMessageBox.No,
                            QtGui.QMessageBox.No) == QtGui.QMessageBox.Yes:
                res1, res2 = QtGui.qApp.call(self, self.deleteExpertMedicalCommission, (actionId,))
                if res1 and res2:
                    table.model().removeRow(table.currentRow())
            self.focusExpertMC()


    def deleteExpertMedicalCommission(self, actionId):
        db = QtGui.qApp.db
        table = db.table('Action')
        db.markRecordsDeleted(table, table['id'].eq(actionId))
        return True


    @pyqtSignature('')
    def on_actExpertMedicalCommissionUpdate_triggered(self):
        self.expertMedicalCommissionActionEdit()


    @pyqtSignature('')
    def on_actExpertMCUpdateTempInvalid_triggered(self):
        QtGui.qApp.setWaitCursor()
        try:
            currentTable = self.getCurrentExpertMedicalCommissionTable()
            actionId = currentTable.currentItemId()
            if not actionId:
                return
            db = QtGui.qApp.db
            tableAction = db.table('Action')
            tableEvent = db.table('Event')
            table = tableAction.innerJoin(tableEvent, tableEvent['id'].eq(tableAction['event_id']))
            record = db.getRecordEx(table, [tableEvent['tempInvalid_id'], tableEvent['id']], [tableAction['id'].eq(actionId), tableAction['deleted'].eq(0), tableEvent['deleted'].eq(0)])
            tempInvalidId = forceRef(record.value('tempInvalid_id')) if record else None
            if not tempInvalidId:
                return
            record = db.getRecordEx(table, '*', [tableAction['id'].eq(actionId), tableAction['deleted'].eq(0)])
            action = CAction(record=record) if record else None
            if not action:
                return
        finally:
            QtGui.qApp.restoreOverrideCursor()
        if u'Номер ЛН' in action._actionType._propertiesByName and action[u'Номер ЛН'] and getRightEditTempInvalid(tempInvalidId):
            self.editTempInvalid(tempInvalidId)


    @pyqtSignature('')
    def on_actExpertMedicalCommissionSelected_triggered(self):
        self.medicalCommissionSelectedRows = []
        currentTable = self.getCurrentExpertMedicalCommissionTable()
        actionId = currentTable.currentItemId()
        currentTable.clearSelection()
        if not actionId:
            return
        db = QtGui.qApp.db
        table = db.table('Action')
        tableEvent = db.table('Event')
        queryTable = table.innerJoin(tableEvent, tableEvent['id'].eq(table['event_id']))
        cols = [table['actionType_id'],
                table['status'],
                table['directionDate'],
                table['setPerson_id'],
                tableEvent['tempInvalid_id']]
        cond = [table['deleted'].eq(0),
                tableEvent['deleted'].eq(0),
                table['id'].eq(actionId),
                ]
        record = db.getRecordEx(queryTable, cols, cond, u'Event.id DESC')
        tempInvalidId = forceRef(record.value('tempInvalid_id')) if record else None
        status = forceInt(record.value('status')) if record else None
        setPersonId = forceRef(record.value('setPerson_id')) if record else None
        actionTypeId = forceRef(record.value('actionType_id')) if record else None
        directionDate = forceDate(record.value('directionDate')) if record else None
        actionIdList = []
        if tempInvalidId:
            cond = [table['deleted'].eq(0),
                    tableEvent['deleted'].eq(0),
                    tableEvent['tempInvalid_id'].eq(tempInvalidId),
                    table['status'].eq(status),
                    table['setPerson_id'].eq(setPersonId),
                    table['actionType_id'].eq(actionTypeId),
                    table['directionDate'].dateEq(directionDate),
                    ]
            actionIdList = db.getDistinctIdList(queryTable, u'Action.id', cond, u'Action.id')
        if actionIdList:
            currentTable.setSelectedItemIdList(actionIdList)
            self.medicalCommissionSelectedRows = actionIdList


    @pyqtSignature('')
    def on_actExpertMedicalCommissionUpdateGroup_triggered(self):
        currentTable = self.getCurrentExpertMedicalCommissionTable()
        selectedActionIdList = currentTable.selectedItemIdList()
        if self.tabExpertMedicalCommissionWidget.currentIndex() != self.tabExpertMedicalCommissionWidget.indexOf(self.tabExpertMedicalSocialInspection) and len(selectedActionIdList) > 1 and len(self.medicalCommissionSelectedRows) > 0:
            actionId = currentTable.currentItemId()
            if actionId and self.canChangePayStatusAdditional('Action', actionId):
                dialog = CActionEditDialog(self)
                try:
                    dialog.load(actionId)
                    if dialog.exec_():
                        db = QtGui.qApp.db
                        tableEvent = db.table('Event')
                        action = CAction.getActionById(actionId)
                        selectedActionIdList = list(set(selectedActionIdList) - set([actionId]))
                        for selectedActionId in selectedActionIdList:
                            newAction = CAction.getActionById(selectedActionId)
                            newRecord = newAction.getRecord()
                            actionType = newAction.getType()
                            eventId = forceRef(newRecord.value('event_id'))
                            currentId = forceRef(newRecord.value('id'))
                            idx = forceInt(newRecord.value('idx'))
                            number = ''
                            if u'Номер ЛН' in actionType._propertiesByName:
                                number = trim(newAction[u'Номер ЛН'])
                            copyFields(newRecord, action.getRecord())
                            newAction.updateByAction(action)
                            if number and u'Номер ЛН' in actionType._propertiesByName:
                                newAction[u'Номер ЛН'] = number
                            newAction.getRecord().setValue('id', toVariant(currentId))
                            newAction.getRecord().setValue('event_id', toVariant(eventId))
                            newAction.getRecord().setValue('idx', toVariant(idx))
                            if eventId:
                                recordEvent = db.getRecordEx(tableEvent, '*', [tableEvent['id'].eq(eventId), tableEvent['deleted'].eq(0)])
                            self.saveMedicalCommissionAction(newAction, recordEvent, eventId)
                        self.updateMedicalCommissionList(self.__expertMCFilter, dialog.itemId())
                        currentTable.model().updateColumnsCaches(actionId)
                        for selectedActionId in selectedActionIdList:
                            currentTable.model().updateColumnsCaches(selectedActionId)
                        QtGui.qApp.emitCurrentClientInfoChanged()
                        return dialog.itemId()
                    else:
                        self.updateMedicalCommissionInfo(actionId)
                        self.updateClientsListRequest = True
                    return None
                finally:
                    dialog.destroy()
                    sip.delete(dialog)
                    del dialog


    @pyqtSignature('')
    def on_actExpertMedicalCommissionClear_triggered(self):
        self.medicalCommissionSelectedRows = []
        self.getCurrentExpertMedicalCommissionTable().clearSelection()


    @pyqtSignature('')
    def on_actExpertMedicalCommissionMSI_triggered(self):
        actionId = self.tblExpertProtocolsMC.currentItemId()
        if not actionId:
            return
        db = QtGui.qApp.db
        tableAction = db.table('Action')
        tableEvent = db.table('Event')
        table = tableAction.innerJoin(tableEvent, tableEvent['id'].eq(tableAction['event_id']))
        record = db.getRecordEx(table, [tableEvent['client_id'], tableEvent['id']], [tableAction['id'].eq(actionId), tableAction['deleted'].eq(0), tableEvent['deleted'].eq(0)])
        if record:
            clientId = forceRef(record.value('client_id'))
            eventActionId = forceRef(record.value('id'))
            record = db.getRecordEx(table, '*', [tableAction['id'].eq(actionId), tableAction['deleted'].eq(0)])
            prevAction = CAction(record=record) if record else None
        else:
            eventActionId = None
            clientId = None
            prevAction = None
        if not clientId:
            return
        actionTypeIdList = getActionTypeIdListByFlatCode(u'inspection_mse%')
        if actionTypeIdList:
            actionTypeId = None
            if len(actionTypeIdList) > 1:
                try:
                    dialog = CActionTypeDialogTableModel(self, actionTypeIdList)
                    if dialog.exec_():
                        actionTypeId = dialog.currentItemId()
                finally:
                    dialog.destroy()
                    sip.delete(dialog)
                    del dialog
            else:
                actionTypeId = actionTypeIdList[0]
            if actionTypeId:
                action = None
                dialog = CActionCreateDialog(self)
                try:
                    actionType = CActionTypeCache.getById(actionTypeId)
                    defaultStatus = actionType.defaultStatus
                    defaultOrgId = actionType.defaultOrgId
                    newRecord = tableAction.newRecord()
                    newRecord.setValue('createDatetime', toVariant(QDateTime.currentDateTime()))
                    newRecord.setValue('createPerson_id',toVariant(QtGui.qApp.userId))
                    newRecord.setValue('modifyDatetime', toVariant(QDateTime.currentDateTime()))
                    newRecord.setValue('modifyPerson_id',toVariant(QtGui.qApp.userId))
                    newRecord.setValue('actionType_id',  toVariant(actionTypeId))
                    newRecord.setValue('prevAction_id',  toVariant(actionId))
                    newRecord.setValue('status',          toVariant(defaultStatus))
                    newRecord.setValue('begDate',         toVariant(QDateTime.currentDateTime()))
                    newRecord.setValue('directionDate',   toVariant(QDateTime.currentDateTime()))
                    newRecord.setValue('org_id',          toVariant(defaultOrgId if defaultOrgId else QtGui.qApp.currentOrgId()))
                    newRecord.setValue('setPerson_id',   toVariant(QtGui.qApp.userId))
                    newRecord.setValue('id',             toVariant(None))
                    newAction = CAction(record=newRecord)
                    if prevAction:
                        newAction.updateByAction(prevAction)
                    if not newAction:
                        return
                    if u'Номер' in newAction.getType()._propertiesByName:
                        newAction[u'Номер'] = None
                    dialog.load(newAction.getRecord(), newAction, clientId)
                    dialog.btnPrint.setEnabled(False)
                    if dialog.exec_() and dialog.checkDataEntered(secondTry=True):
                        action = dialog.getAction()
                        dialog.btnPrint.setEnabled(True)
                finally:
                    dialog.destroy()
                    sip.delete(dialog)
                    del dialog
                if action:
                    tableEventType = db.table('EventType')
                    eventId = None
                    if eventActionId:
                        queryTable = tableEvent.innerJoin(tableEventType, tableEventType['id'].eq(tableEvent['eventType_id']))
                        cond = [tableEvent['id'].eq(eventActionId),
                                tableEventType['context'].like(u'inspection%'),
                                tableEvent['execDate'].isNull(),
                                tableEvent['client_id'].eq(clientId),
                                tableEvent['deleted'].eq(0),
                                tableEventType['deleted'].eq(0),
                                ]
                        recordEvent = db.getRecordEx(queryTable, 'Event.*', cond, u'Event.id DESC')
                        eventId = forceRef(recordEvent.value('id')) if recordEvent else None
                    if eventId:
                        actionId = self.saveMedicalCommissionAction(action, recordEvent, eventId)
                    else:
                        recordEventType = db.getRecordEx(tableEventType, [tableEventType['id']], [tableEventType['context'].like(u'inspection%'), tableEventType['deleted'].eq(0)], u'EventType.id')
                        eventTypeId = forceRef(recordEventType.value('id')) if recordEventType else None
                        if eventTypeId:
                            recordEvent = tableEvent.newRecord()
                            recordEvent.setValue('createDatetime', toVariant(QDateTime.currentDateTime()))
                            recordEvent.setValue('createPerson_id',toVariant(QtGui.qApp.userId))
                            recordEvent.setValue('modifyDatetime', toVariant(QDateTime.currentDateTime()))
                            recordEvent.setValue('modifyPerson_id',toVariant(QtGui.qApp.userId))
                            recordEvent.setValue('setDate',        toVariant(QDateTime.currentDateTime()))
                            recordEvent.setValue('eventType_id',   toVariant(eventTypeId))
                            recordEvent.setValue('client_id',      toVariant(clientId))
                            recordEvent.setValue('relegatePerson_id', toVariant(QtGui.qApp.userId))
                            recordEvent.setValue('relegateOrg_id', toVariant(QtGui.qApp.currentOrgId()))
                            recordEvent.setValue('tempInvalid_id', toVariant(tempInvalidId))
                            eventId = db.insertRecord(tableEvent, recordEvent)
                            if eventId:
                                recordEvent.setValue('id', toVariant(eventId))
                                actionId = self.saveMedicalCommissionAction(action, recordEvent, eventId)


    def saveMedicalCommissionAction(self, action, recordEvent = None, eventId = None):
        id = None
        try:
            try:
                db = QtGui.qApp.db
                db.transaction()
                id = action.save(eventId, idx=0, checkModifyDate = False)
                if id:
                    action.getRecord().setValue('id', toVariant(id))
                    checkTissueJournalStatusByActions([(action.getRecord(), action)])
                    if action.getType().closeEvent and recordEvent:
                        eventExecDate = forceDate(recordEvent.value('execDate'))
                        actionEndDate = forceDateTime(action.getRecord().value('endDate'))
                        if not eventExecDate and actionEndDate:
                            recordEvent.setValue('execDate', QVariant(actionEndDate))
                            recordEvent.setValue('isClosed', QVariant(1))
                            db.updateRecord('Event', recordEvent)
                db.commit()
            except:
                db.rollback()
                raise
        except Exception, e:
            QtGui.qApp.logCurrentException()
            QtGui.QMessageBox.critical( self,
                                        u'',
                                        unicode(e),
                                        QtGui.QMessageBox.Close)
        return id


    @pyqtSignature('')
    def on_actExpertTempInvalidDelete_triggered(self):
        self.onExpertDocDelete(self.tblExpertTempInvalid, self.modelExpertTempInvalid)


    @pyqtSignature('')
    def on_actExpertDisabilityDelete_triggered(self):
        self.onExpertDocDelete(self.tblExpertDisability, self.modelExpertDisability)


    @pyqtSignature('')
    def on_actExpertVitalRestrictionDelete_triggered(self):
        self.onExpertDocDelete(self.tblExpertVitalRestriction, self.modelExpertVitalRestriction)


    def onExpertDocDelete(self, table, model):
        tempInvalidId = table.currentItemId()
        if getRightEditTempInvalid(tempInvalidId):
            if QtGui.QMessageBox.question(self,
                            u'Удаление документа', u'Вы действительно хотите удалить документ?',
                            QtGui.QMessageBox.Yes|QtGui.QMessageBox.No,
                            QtGui.QMessageBox.No) == QtGui.QMessageBox.Yes:
                self.getCurrentExpertTable().model().updateColumnsCaches(tempInvalidId)
                res1, res2 = QtGui.qApp.call(self, deleteTempInvalid, (self, tempInvalidId,))
                if res1 and res2:
                    model.removeRow(table.currentRow())
                self.updateTempInvalidList(self.__expertFilter)


    @pyqtSignature('')
    def on_actExpertTempInvalidPrev_triggered(self):
        self.onExpertDocPrev(self.tblExpertTempInvalid)


    @pyqtSignature('')
    def on_actExpertDisabilityPrev_triggered(self):
        self.onExpertDocPrev(self.tblExpertDisability)


    @pyqtSignature('')
    def on_actExpertVitalRestrictionPrev_triggered(self):
        self.onExpertDocPrev(self.tblExpertVitalRestriction)


    def onExpertDocPrev(self, table):
        prevId = self.getExpertPrevDocId(table)
        if prevId:
            row = table.model().findItemIdIndex(prevId)
            if row>=0:
                table.setCurrentRow(row)
            else:
                QtGui.QMessageBox.information(self, u'Переход к предыдущему документу', u'Переход невозможен - необходимо изменить фильтр')


    def onEditTempInvalid(self, tempInvalidId):
        if tempInvalidId:
            if getRightEditTempInvalid(tempInvalidId):
                self.editTempInvalid(tempInvalidId)
                self.focusExpert()
                QtGui.qApp.emitCurrentClientInfoChanged()


    @pyqtSignature('QModelIndex')
    def on_tblExpertTempInvalidDocumentsEx_doubleClicked(self, index):
        documentId = self.tblExpertTempInvalidDocumentsEx.currentItemId()
        tempInvalidId = self.modelExpertTempInvalidDocumentsEx.getCurrentTempInvalidId(documentId)
        self.onEditTempInvalid(tempInvalidId)


    @pyqtSignature('QModelIndex')
    def on_tblExpertTempInvalidRelationDocumentsEx_doubleClicked(self, index):
        documentId = self.tblExpertTempInvalidRelationDocumentsEx.currentItemId()
        tempInvalidId = self.modelExpertTempInvalidRelationDocumentsEx.getCurrentTempInvalidId(documentId)
        self.onEditTempInvalid(tempInvalidId)


    @pyqtSignature('QModelIndex')
    def on_tblExpertDisabilityDocumentsEx_doubleClicked(self, index):
        documentId = self.tblExpertDisabilityDocumentsEx.currentItemId()
        tempInvalidId = self.modelExpertDisabilityDocumentsEx.getCurrentTempInvalidId(documentId)
        self.onEditTempInvalid(tempInvalidId)


    @pyqtSignature('QModelIndex')
    def on_tblExpertDisabilityRelationDocumentsEx_doubleClicked(self, index):
        documentId = self.tblExpertDisabilityRelationDocumentsEx.currentItemId()
        tempInvalidId = self.modelExpertDisabilityRelationDocumentsEx.getCurrentTempInvalidId(documentId)
        self.onEditTempInvalid(tempInvalidId)


    @pyqtSignature('QModelIndex')
    def on_tblExpertVitalRestrictionDocumentsEx_doubleClicked(self, index):
        documentId = self.tblExpertVitalRestrictionDocumentsEx.currentItemId()
        tempInvalidId = self.modelExpertVitalRestrictionDocumentsEx.getCurrentTempInvalidId(documentId)
        self.onEditTempInvalid(tempInvalidId)


    @pyqtSignature('QModelIndex')
    def on_tblExpertVitalRestrictionRelationDocumentsEx_doubleClicked(self, index):
        documentId = self.tblExpertVitalRestrictionRelationDocumentsEx.currentItemId()
        tempInvalidId = self.modelExpertVitalRestrictionRelationDocumentsEx.getCurrentTempInvalidId(documentId)
        self.onEditTempInvalid(tempInvalidId)


    @pyqtSignature('QModelIndex')
    def on_tblExpertTempInvalidDocuments_doubleClicked(self, index):
        tempInvalidDocumentId = self.tblExpertTempInvalidDocuments.currentItemId()
        tempInvalidId = self.modelExpertTempInvalidDocuments.getCurrentTempInvalidId(tempInvalidDocumentId)
        self.onEditTempInvalid(tempInvalidId)


    @pyqtSignature('QModelIndex')
    def on_tblExpertDisabilityDocuments_doubleClicked(self, index):
        tempInvalidDocumentId = self.tblExpertDisabilityDocuments.currentItemId()
        tempInvalidId = self.modelExpertDisabilityDocuments.getCurrentTempInvalidId(tempInvalidDocumentId)
        self.onEditTempInvalid(tempInvalidId)


    @pyqtSignature('QModelIndex')
    def on_tblExpertVitalRestrictionDocuments_doubleClicked(self, index):
        tempInvalidDocumentId = self.tblExpertVitalRestrictionDocuments.currentItemId()
        tempInvalidId = self.modelExpertVitalRestrictionDocuments.getCurrentTempInvalidId(tempInvalidDocumentId)
        self.onEditTempInvalid(tempInvalidId)


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelExpertDisability_currentRowChanged(self, current, previous):
        tempInvalidId = self.tblExpertDisability.currentItemId()
        self.updateTempInvalidInfo(tempInvalidId, noPrev=True)
        self.setBtnExpertEditEnabled()


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelExpertDisabilityRelation_currentRowChanged(self, current, previous):
        tempInvalidId = self.tblExpertDisabilityRelation.currentItemId()
        self.updateTempInvalidInfo(tempInvalidId)
        self.setBtnExpertEditEnabled()


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelExpertVitalRestriction_currentRowChanged(self, current, previous):
        tempInvalidId = self.tblExpertVitalRestriction.currentItemId()
        self.updateTempInvalidInfo(tempInvalidId, noPrev=True)
        self.setBtnExpertEditEnabled()


    @pyqtSignature('QModelIndex, QModelIndex')
    def on_selectionModelExpertVitalRestrictionRelation_currentRowChanged(self, current, previous):
        tempInvalidId = self.tblExpertVitalRestrictionRelation.currentItemId()
        self.updateTempInvalidInfo(tempInvalidId)
        self.setBtnExpertEditEnabled()


    @pyqtSignature('int')
    def on_modelExpertTempInvalid_itemsCountChanged(self, count):
        infoBeds, infoAddressOrgStructure = self.getFilterInfoBed()
        self.lblExpertTempInvalidCount.setText(formatRecordsCount(count)+infoBeds+infoAddressOrgStructure)


    @pyqtSignature('int')
    def on_modelExpertTempInvalidDocumentsEx_itemsCountChanged(self, count):
        self.lblExpertTempInvalidDocumentsExCount.setText(formatRecordsCount(count))


    @pyqtSignature('int')
    def on_modelExpertDisability_itemsCountChanged(self, count):
        self.lblExpertDisabilityCount.setText(formatRecordsCount(count))


    @pyqtSignature('int')
    def on_modelExpertDisabilityDocumentsEx_itemsCountChanged(self, count):
        self.lblExpertDisabilityDocumentsExCount.setText(formatRecordsCount(count))


    @pyqtSignature('int')
    def on_modelExpertVitalRestriction_itemsCountChanged(self, count):
        self.lblExpertVitalRestrictionCount.setText(formatRecordsCount(count))


    @pyqtSignature('int')
    def on_modelExpertVitalRestrictionDocumentsEx_itemsCountChanged(self, count):
        self.lblExpertVitalRestrictionDocumentsExCount.setText(formatRecordsCount(count))


    @pyqtSignature('')
    def on_btnExpertRelationEdit_clicked(self):
        tempInvalidId = self.getCurrentExpertRelationTable().currentItemId()
        if tempInvalidId:
            self.editTempInvalid(tempInvalidId)
        self.focusExpert()
        QtGui.qApp.emitCurrentClientInfoChanged()


    @pyqtSignature('QModelIndex')
    def on_tblExpertTempInvalid_doubleClicked(self, index):
        tempInvalidId = self.tblExpertTempInvalid.currentItemId()
        if getRightEditTempInvalid(tempInvalidId):
            self.on_btnExpertEdit_clicked()


    @pyqtSignature('QModelIndex')
    def on_tblExpertTempInvalidRelation_doubleClicked(self, index):
        tempInvalidId = self.tblExpertTempInvalidRelation.currentItemId()
        if getRightEditTempInvalid(tempInvalidId):
            self.on_btnExpertRelationEdit_clicked()


    @pyqtSignature('QModelIndex')
    def on_tblExpertDisability_doubleClicked(self, index):
        tempInvalidId = self.tblExpertDisability.currentItemId()
        if getRightEditTempInvalid(tempInvalidId):
            self.on_btnExpertEdit_clicked()


    @pyqtSignature('QModelIndex')
    def on_tblExpertDisabilityRelation_doubleClicked(self, index):
        tempInvalidId = self.tblExpertDisabilityRelation.currentItemId()
        if getRightEditTempInvalid(tempInvalidId):
            self.on_btnExpertRelationEdit_clicked()


    @pyqtSignature('QModelIndex')
    def on_tblExpertVitalRestriction_doubleClicked(self, index):
        tempInvalidId = self.tblExpertVitalRestriction.currentItemId()
        if getRightEditTempInvalid(tempInvalidId):
            self.on_btnExpertEdit_clicked()


    @pyqtSignature('QModelIndex')
    def on_tblExpertVitalRestrictionRelation_doubleClicked(self, index):
        tempInvalidId = self.tblExpertVitalRestrictionRelation.currentItemId()
        if getRightEditTempInvalid(tempInvalidId):
            self.on_btnExpertRelationEdit_clicked()


    @pyqtSignature('QModelIndex')
    def on_tblExpertDirectionsMC_doubleClicked(self, index):
        if QtGui.qApp.userHasRight(urRegTabEditExpertMC):
            self.on_btnExpertEdit_clicked()


    @pyqtSignature('QModelIndex')
    def on_tblExpertProtocolsMC_doubleClicked(self, index):
        if QtGui.qApp.userHasRight(urRegTabEditExpertMC):
            self.on_btnExpertEdit_clicked()


    @pyqtSignature('QModelIndex')
    def on_tblExpertMedicalSocialInspection_doubleClicked(self, index):
        if QtGui.qApp.userHasRight(urRegTabEditExpertMC):
            self.on_btnExpertEdit_clicked()


    @pyqtSignature('')
    def on_btnExpertEdit_clicked(self):
        if self.tabWidgetTempInvalidTypes.currentIndex() == self.tabWidgetTempInvalidTypes.indexOf(self.tabExpertMedicalCommission):
            self.expertMedicalCommissionActionEdit()
        else:
            tempInvalidId = self.currentTempInvalidId()
            if tempInvalidId:
                self.editTempInvalid(tempInvalidId)
            self.focusExpert()
        QtGui.qApp.emitCurrentClientInfoChanged()


    @pyqtSignature('')
    def on_actExpertPrint_triggered(self):
        if self.tabWidgetTempInvalidTypes.currentIndex() == self.tabWidgetTempInvalidTypes.indexOf(self.tabExpertMedicalCommission):
            tblExpertMC = self.getCurrentExpertMedicalCommissionTable()
            expertiseType = self.tabWidgetTempInvalidTypes.currentIndex()
            tblExpertMC.setReportHeader(unicode(self.tabWidgetTempInvalidTypes.tabText(expertiseType)))
            tblExpertMC.setReportDescription(self.getExpertMCFilterAsText())
            tblExpertMC.printContent()
            self.focusExpertMC()
        else:
            tblExpert = self.getCurrentExpertTable()
            tempInvalidType = self.tabWidgetTempInvalidTypes.currentIndex()
            tblExpert.setReportHeader(unicode(self.tabWidgetTempInvalidTypes.tabText(tempInvalidType)))
            tblExpert.setReportDescription(self.getExpertFilterAsText())
            tblExpert.printContent()
            self.focusExpert()


    @pyqtSignature('int')
    def on_btnExpertPrint_printByTemplate(self, templateId):
        context = forceString(QtGui.qApp.db.translate('rbPrintTemplate', 'id', templateId, 'context'))
        data = None
        if self.tabWidgetTempInvalidTypes.currentIndex() == self.tabWidgetTempInvalidTypes.indexOf(self.tabExpertMedicalCommission):
            if context == u'medicalCommission':
                currentTable = self.getCurrentExpertMedicalCommissionTable()
                if templateId == -1:
                    expertiseType = self.tabWidgetTempInvalidTypes.currentIndex()
                    currentTable.setReportHeader(unicode(self.tabWidgetTempInvalidTypes.tabText(expertiseType)))
                    currentTable.setReportDescription(self.getExpertMCFilterAsText())
                    currentTable.printContent()
                    self.focusExpertMC()
                else:
                    idList = currentTable.model().idList()
                    context = CInfoContext()
                    data = {'expertise': CLocActionInfoList(context, idList)}
                    QtGui.qApp.call(self, applyTemplate, (self, templateId, data))
        else:
            if context == u'tempInvalid':
                data = self.getKerContextData()
            else:
                data = self.getKerListContextData()
            QtGui.qApp.call(self, applyTemplate, (self, templateId, data))


    @pyqtSignature('')
    def on_btnExpertFilter_clicked(self):
        if self.tabWidgetTempInvalidTypes.currentIndex() == self.tabWidgetTempInvalidTypes.indexOf(self.tabExpertMedicalCommission):
            for s in self.chkListOnExpertMCPage:
                chk = s[0]
                if chk.isChecked():
                    self.activateFilterWdgets(s[1])
                    return
            self.chkFilterExpertExpertiseMC.setChecked(True)
        else:
            for s in self.chkListOnExpertPage:
                chk = s[0]
                if chk.isChecked():
                    self.activateFilterWdgets(s[1])
                    return
            self.chkFilterExpertDocType.setChecked(True)


    @pyqtSignature('')
    def on_btnExpertNew_clicked(self):
        tempInvalidId = getTempInvalidIdOpen(self.currentClientId(), self.tabWidgetTempInvalidTypes.currentIndex())
        if tempInvalidId:
            self.editTempInvalid(tempInvalidId)
        else:
            self.editTempInvalidExpert()
        self.focusExpert()
        QtGui.qApp.emitCurrentClientInfoChanged()


    @pyqtSignature('bool')
    def on_chkFilterExpertDocType_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertSerial_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkPrimaryOrDuble_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertNumber_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertReason_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertBegDate_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterDirectDateOnMC_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertExternal_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkDateKAK_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkExpertIdKAK_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertEndDate_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertPerson_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertOrgStruct_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertSpeciality_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertMKB_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertClosed_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertDuration_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertInsuranceOfficeMark_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertCreatePerson_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertModifyPerson_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertCreateDate_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertExpertiseMC_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertExpertiseTypeMC_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertNumberMC_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertNumberExpertise_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertSetPersonMC_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertExpertiseCharacterMC_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertExpertiseKindMC_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_ghkFilterExpertExpertiseObjectMC_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertExpertiseArgumentMC_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertOrgStructMC_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertSpecialityMC_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertMKBMC_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertClosedMC_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExecDateMC_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkExpertIdMC_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertCreatePersonMC_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertCreateDateMC_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertModifyPersonMC_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExpertModifyDateMC_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('QAbstractButton*')
    def on_buttonBoxExpert_clicked(self, button):
        buttonCode = self.buttonBoxExpert.standardButton(button)
        if buttonCode == QtGui.QDialogButtonBox.Apply:
            self.on_buttonBoxExpert_apply()
        elif buttonCode == QtGui.QDialogButtonBox.Reset:
            self.on_buttonBoxExpert_reset()


#    @pyqtSignature('')
    def on_buttonBoxExpert_reset(self):
        for s in self.chkListOnExpertPage:
            chk = s[0]
            if chk.isChecked():
                self.deactivateFilterWdgets(s[1])
                chk.setChecked(False)
        self.cmbFilterExpert.setCurrentIndex(0)


    @pyqtSignature('QAbstractButton*')
    def on_buttonBoxExpertMC_clicked(self, button):
        buttonCode = self.buttonBoxExpertMC.standardButton(button)
        if buttonCode == QtGui.QDialogButtonBox.Apply:
            self.on_buttonBoxExpertMC_apply()
        elif buttonCode == QtGui.QDialogButtonBox.Reset:
            self.on_buttonBoxExpertMC_reset()


#    @pyqtSignature('')
    def on_buttonBoxExpertMC_reset(self):
        for s in self.chkListOnExpertMCPage:
            chk = s[0]
            if chk.isChecked():
                self.deactivateFilterWdgets(s[1])
                chk.setChecked(False)
        self.cmbFilterExpert.setCurrentIndex(0)


#    @pyqtSignature('')
    def on_buttonBoxExpert_apply(self):
        filter = {}
        if self.chkFilterExpertDocType.isChecked():
            filter['docTypeId'] = self.cmbFilterExpertDocType.value()
        if self.chkFilterExpertSerial.isChecked():
            filter['serial'] = forceStringEx(self.edtFilterExpertSerial.text())
        if self.chkFilterExpertNumber.isChecked():
            filter['number'] = forceStringEx(self.edtFilterExpertNumber.text())
        if self.chkPrimaryOrDuble.isChecked():
            filter['primaryOrDuble'] = forceInt(self.cmbPrimaryOrDuble.currentIndex())
        if self.chkFilterExpertReason.isChecked():
            filter['reasonId'] = self.cmbFilterExpertReason.value()
        if self.chkFilterExpertBegDate.isChecked():
            filter['begBegDate'] = self.edtFilterExpertBegBegDate.date()
            filter['endBegDate'] = self.edtFilterExpertEndBegDate.date()
        if self.chkFilterExpertEndDate.isChecked():
            filter['begEndDate'] = self.edtFilterExpertBegEndDate.date()
            filter['endEndDate'] = self.edtFilterExpertEndEndDate.date()
        if self.chkFilterExpertPerson.isChecked():
            filter['personId'] = self.cmbFilterExpertPerson.value()
        if self.chkFilterExpertOrgStruct.isChecked():
            filter['expertOrgStructureId'] = self.cmbFilterExpertOrgStruct.value()
        if self.chkFilterExpertSpeciality.isChecked():
            filter['expertSpecialityId'] = self.cmbFilterExpertSpeciality.value()
        if self.chkFilterExpertMKB.isChecked():
            filter['begMKB'] = unicode(self.edtFilterExpertBegMKB.text())
            filter['endMKB'] = unicode(self.edtFilterExpertEndMKB.text())
        if self.chkFilterExpertClosed.isChecked():
            filter['closed'] = self.cmbFilterExpertClosed.currentIndex()
        if self.chkFilterExpertDuration.isChecked():
            filter['begDuration'] = self.edtFilterExpertBegDuration.value()
            filter['endDuration'] = self.edtFilterExpertEndDuration.value()
        if self.chkFilterExpertInsuranceOfficeMark.isChecked():
            filter['insuranceOfficeMark'] = self.cmbFilterExpertInsuranceOfficeMark.currentIndex()
        filter['linked'] = self.chkFilterExpertLinked.isChecked()
        if self.chkFilterExpertExternal.isChecked():
            filter['isExternal'] = self.cmbFilterExpertExternal.currentIndex()
        if self.chkFilterExpertCreatePerson.isChecked():
            filter['createPersonId'] = self.cmbFilterExpertCreatePerson.value()
        if self.chkFilterExpertCreateDate.isChecked():
            filter['begCreateDate'] = self.edtFilterExpertBegCreateDate.date()
            filter['endCreateDate'] = self.edtFilterExpertEndCreateDate.date()
        if self.chkFilterExpertModifyPerson.isChecked():
            filter['modifyPersonId'] = self.cmbFilterExpertModifyPerson.value()
        if self.chkFilterExpertModifyDate.isChecked():
            filter['begModifyDate'] = self.edtFilterExpertBegModifyDate.date()
            filter['endModifyDate'] = self.edtFilterExpertEndModifyDate.date()
        expertFilterType = self.cmbFilterExpert.currentIndex()
        if expertFilterType == 0:
            filter['receiverIds'] = [ self.selectedClientId() ]
        elif expertFilterType == 1:
            filter['receiverIds'] = self.modelClients.idList()
        elif expertFilterType == 2:
            filter['receiverEventIds'] = [ self.tblEvents.currentItemId() ]
        elif expertFilterType == 3:
            filter['receiverEventIds'] = self.modelEvents.idList()
        elif expertFilterType == 4:
            filter['clientIds'] = [ self.selectedClientId() ]
        elif expertFilterType == 5:
            filter['clientIds'] = self.modelClients.idList()
        elif expertFilterType == 6:
            filter['eventIds'] = [ self.tblEvents.currentItemId() ]
        elif expertFilterType == 7:
            filter['eventIds'] = self.modelEvents.idList()
        filter['expertFilterType']=expertFilterType
        self.updateTempInvalidList(filter)
        self.focusExpert()


    def on_buttonBoxExpertMC_apply(self):
        filter = {}
        if self.chkFilterExpertExpertiseMC.isChecked():
            filter['expertiseId'] = self.cmbFilterExpertExpertiseMC.currentIndex()
        if self.chkFilterExpertExpertiseTypeMC.isChecked():
            expertiseTypeIdList = []
            expertiseTypeStr = self.cmbFilterExpertExpertiseTypeMC.value().split(',')
            for expertiseType in expertiseTypeStr:
                expertiseTypeId = trim(expertiseType)
                if expertiseTypeId and expertiseTypeId not in expertiseTypeIdList:
                    expertiseTypeIdList.append(expertiseTypeId)
            filter['expertiseTypeId'] = (expertiseTypeIdList, forceString(self.cmbFilterExpertExpertiseTypeMC.currentText()))
        if self.chkFilterExpertNumberMC.isChecked():
            filter['expertNumber'] = forceStringEx(self.edtFilterExpertNumberMC.text())
        if self.chkFilterExpertNumberExpertise.isChecked():
            filter['expertNumberExpertise'] = forceStringEx(self.edtFilterExpertNumberExpertise.text())
        if self.chkFilterExpertSetPersonMC.isChecked():
            filter['setPersonId'] = self.cmbFilterExpertSetPersonMC.value()
        if self.chkFilterExpertOrgStructMC.isChecked():
            filter['expertOrgStructureId'] = self.cmbFilterExpertOrgStructMC.value()
        if self.chkFilterDirectDateOnMC.isChecked():
            filter['begDirectDate'] = self.edtFilterBegDirectDateOnMC.date()
            filter['endDirectDate'] = self.edtFilterEndDirectDateOnMC.date()
        if self.chkFilterExecDateMC.isChecked():
            filter['begExecDate'] = self.edtFilterBegExecDateMC.date()
            filter['endExecDate'] = self.edtFilterEndExecDateMC.date()
        if self.chkFilterExpertSpecialityMC.isChecked():
            filter['expertSpecialityId'] = self.cmbFilterExpertSpecialityMC.value()
        if self.chkFilterExpertSpecialityMC.isChecked():
            filter['expertSpecialityId'] = self.cmbFilterExpertSpecialityMC.value()
        if self.chkFilterExpertMKBMC.isChecked():
            filter['begMKB'] = unicode(self.edtFilterExpertBegMKBMC.text())
            filter['endMKB'] = unicode(self.edtFilterExpertEndMKBMC.text())
        if self.chkFilterExpertClosedMC.isChecked():
            filter['actionStatus'] = self.cmbFilterExpertClosedMC.value()
        if self.chkFilterExpertExpertiseCharacterMC.isChecked():
            filter['expertiseCharacterId'] = unicode(self.cmbFilterExpertExpertiseCharacterMC.value())
        if self.chkFilterExpertExpertiseKindMC.isChecked():
            filter['expertiseKindId'] = unicode(self.cmbFilterExpertExpertiseKindMC.value())
        if self.ghkFilterExpertExpertiseObjectMC.isChecked():
            filter['expertiseObjectId'] = unicode(self.cmbFilterExpertExpertiseObjectMC.value())
        if self.chkFilterExpertExpertiseArgumentMC.isChecked():
            filter['expertiseArgumentId'] = unicode(self.cmbFilterExpertExpertiseArgumentMC.value())
        if self.chkExpertIdMC.isChecked():
            filter['expertId'] = self.cmbExpertIdMC.value()
        if self.chkFilterExpertCreateDateMC.isChecked():
            filter['begCreateDate'] = self.edtFilterExpertBegCreateDateMC.value()
            filter['endCreateDate'] = self.edtFilterExpertEndCreateDateMC.value()
        if self.chkFilterExpertCreatePersonMC.isChecked():
            filter['createPersonId'] = self.cmbFilterExpertCreatePersonMC.value()
        if self.chkFilterExpertModifyDateMC.isChecked():
            filter['begModifyDate'] = self.edtFilterExpertBegModifyDateMC.date()
            filter['endModifyDate'] = self.edtFilterExpertEndModifyDateMC.date()
        if self.chkFilterExpertModifyPersonMC.isChecked():
            filter['modifyPersonId'] = self.cmbFilterExpertModifyPersonMC.value()
        actionsFilterType = self.cmbFilterExpertMC.currentIndex()
        if actionsFilterType == 0:
            filter['clientIds'] = [ self.selectedClientId() ]
        elif actionsFilterType == 1:
            filter['clientIds'] = self.modelClients.idList()
        elif actionsFilterType == 2:
            filter['eventIds'] = [ self.tblEvents.currentItemId() ]
        elif actionsFilterType == 3:
            filter['eventIds'] = self.modelEvents.idList()
        filter['actionsFilterType']=actionsFilterType
        self.updateMedicalCommissionList(filter)
        self.focusExpertMC()

# ===== :)
# ============ Visit_Page ====================================================

    @pyqtSignature('bool')
    def on_chkFilterVisitType_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterVisitExecDate_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterExecSetOrgStructureVisit_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)
        if checked:
            self.chkFilterExecSetPersonVisit.setChecked(False)
            execSetOrgStructure = self.cmbFilterExecSetOrgStructureVisit.value()
            if not execSetOrgStructure:
                execSetOrgStructure = QtGui.qApp.currentOrgStructureId()
                self.cmbFilterExecSetOrgStructureVisit.setValue(execSetOrgStructure)
            self.cmbFilterExecSetPersonVisit.setOrgStructureId(execSetOrgStructure)
        else:
            self.cmbFilterExecSetPersonVisit.setOrgStructureId(None)


    @pyqtSignature('bool')
    def on_chkFilterExecSetSpecialityVisit_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)
        if checked:
            self.chkFilterExecSetPersonVisit.setChecked(False)
            self.cmbFilterExecSetPersonVisit.setSpecialityId(self.cmbFilterExecSetSpecialityVisit.value())
        else:
            self.cmbFilterExecSetPersonVisit.setSpecialityId(None)


    @pyqtSignature('int')
    def on_cmbFilterExecSetSpecialityVisit_currentIndexChanged(self, index):
        self.cmbFilterExecSetPersonVisit.setSpecialityId(self.cmbFilterExecSetSpecialityVisit.value())


    @pyqtSignature('bool')
    def on_chkFilterExecSetPersonVisit_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)
        self.chkFilterExecSetOrgStructureVisit.setChecked(False)
        self.chkFilterExecSetSpecialityVisit.setChecked(False)


    @pyqtSignature('bool')
    def on_chkFilterVisitAssistant_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterServiceVisit_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterSceneVisit_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterVisitFinance_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterVisitCreatePersonEx_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterVisitCreateDateEx_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterVisitModifyPersonEx_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterVisitModifyDateEx_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('bool')
    def on_chkFilterVisitPayStatusEx_toggled(self, checked):
        self.onChkFilterToggled(self.sender(), checked)


    @pyqtSignature('QAbstractButton*')
    def on_buttonBoxVisit_clicked(self, button):
        buttonCode = self.buttonBoxVisit.standardButton(button)
        if buttonCode == QtGui.QDialogButtonBox.Apply:
            self.on_buttonBoxVisit_apply()
        elif buttonCode == QtGui.QDialogButtonBox.Reset:
            self.on_buttonBoxVisit_reset()


    def on_buttonBoxVisit_reset(self):
        for s in self.chkListOnVisitsPage:
            chk = s[0]
            if chk.isChecked():
                self.deactivateFilterWdgets(s[1])
                chk.setChecked(False)
        self.cmbFilterVisit.setCurrentIndex(0)


    def on_buttonBoxVisit_apply(self):
        filter = {}
        if self.chkFilterVisitType.isChecked():
            filter['visitTypeId'] = self.cmbFilterVisitsType.value()
        if self.chkFilterVisitCreatePersonEx.isChecked():
            filter['createPersonId'] = self.cmbFilterVisitCreatePersonEx.currentIndex()
        if self.chkFilterVisitCreateDateEx.isChecked():
            filter['begCreateDate'] = self.edtFilterVisitBegCreateDateEx.date()
            filter['endCreateDate'] = self.edtFilterVisitEndCreateDateEx.date()
        if self.chkFilterVisitModifyPersonEx.isChecked():
            filter['modifyPersonId'] = self.cmbFilterVisitModifyPersonEx.value()
        if self.chkFilterVisitModifyDateEx.isChecked():
            filter['begModifyDate'] = self.edtFilterVisitBegModifyDateEx.date()
            filter['endModifyDate'] = self.edtFilterVisitEndModifyDateEx.date()
        if self.chkFilterVisitPayStatusEx.isChecked():
            filter['payStatusCode'] = self.cmpFilterVisitPayStatusCodeEx.currentIndex()
            index = self.cmbFilterVisitPayStatusFinanceEx.currentIndex()
            if not 0<=index<5:
                index = 0
            filter['payStatusFinanceCode'] = 5-index
        if self.chkFilterVisitExecDate.isChecked():
            filter['begExecDateTime'] = QDateTime(self.edtFilterVisitBegExecDate.date(), self.edtFilterVisitBegExecTime.time())
            filter['endExecDateTime'] = QDateTime(self.edtFilterVisitEndExecDate.date(), self.edtFilterVisitEndExecTime.time())
        if self.chkFilterExecSetOrgStructureVisit.isChecked():
            filter['orgStructureId'] = self.cmbFilterExecSetOrgStructureVisit.value()
        if self.chkFilterExecSetSpecialityVisit.isChecked():
            filter['specialityId'] = self.cmbFilterExecSetSpecialityVisit.value()
        if self.chkFilterExecSetPersonVisit.isChecked():
            filter['personId'] = self.cmbFilterExecSetPersonVisit.value()
        if self.chkFilterVisitAssistant.isChecked():
            filter['assistantId'] = self.cmbFilterVisitAssistant.value()
        if self.chkFilterVisitFinance.isChecked():
            filter['financeId'] = self.cmbFilterVisitFinance.value()
        if self.chkFilterServiceVisit.isChecked():
            filter['serviceId'] = self.cmbFilterServiceVisit.value()
        if self.chkFilterSceneVisit.isChecked():
            filter['sceneId'] = self.cmbFilterSceneVisit.value()
        visitFilterType = self.cmbFilterVisit.currentIndex()
        if visitFilterType == 0:
            filter['clientIds'] = [ self.selectedClientId() ]
        elif visitFilterType == 1:
            filter['clientIds'] = self.modelClients.idList()
        elif visitFilterType == 2:
            filter['eventIds'] = [ self.tblEvents.currentItemId() ]
        elif visitFilterType == 3:
            filter['eventIds'] = self.modelEvents.idList()
        filter['visitFilterType'] = visitFilterType
        self.updateVisitsList(filter)
        self.focusEvents()

# ===== Visit_Page ============ :)

def convertFilterToTextItem(resList, filter, prop, propTitle, format=None):
    val = filter.get(prop, None)
    if val:
        if format:
            resList.append((propTitle, format(val)))
        else:
            resList.append((propTitle, val))


class CIdValidator(QtGui.QRegExpValidator):
    def __init__(self, parent):
        QtGui.QRegExpValidator.__init__(self, QRegExp(r'(\d*)(\.\d*)?'), parent)


def dateTimeToString(value):
#   return value.toString(Qt.DefaultLocaleLongDate)
    return  value.toString('dd.MM.yyyy hh:mm:ss')


def parseClientId(clientIdEx):
    if '.' in clientIdEx:
        miacCode, clientId = clientIdEx.split('.')
    else:
        miacCode, clientId = '', clientIdEx
    if miacCode:
        if forceString(QtGui.qApp.db.translate('Organisation', 'id', QtGui.qApp.currentOrgId(), 'miacCode')) != miacCode.strip():
            return -1
    return int(clientId)


class CSchedulesModel(CRecordListModel):
    def __init__(self,  parent):
        CRecordListModel.__init__(self, parent)
#        activityPref = QtGui.qApp.getGlobalPreference('23')
        self.addCol(CBoolInDocTableCol(u'Отметка',    'checked', 6))
        self.addCol(CEnumInDocTableCol(u'Тип', 'appointmentType', 6, CSchedule.atNames)).setReadOnly()
        self.addCol(CRBInDocTableCol(u'Назначение', 'appointmentPurpose_id', 20, 'rbAppointmentPurpose')).setReadOnly()
        self.addCol(CDateTimeInDocTableCol(u'Дата и время приема', 'time', 20)).setReadOnly()
        self.addCol(CInDocTableCol(u'Каб',          'office', 6)).setReadOnly()
        self.addCol(CRBInDocTableCol(u'Специалист', 'person_id', 20, 'vrbPersonWithSpeciality')).setReadOnly()
        self.addCol(CRBInDocTableCol(u'Записал',    'recordPerson_id', 20, 'vrbPersonWithSpeciality')).setReadOnly()
        self.addCol(CInDocTableCol(u'Жалобы',       'complaint', 6)).setReadOnly()
        self.addCol(CInDocTableCol(u'Примечания',   'note', 6)).setReadOnly()
        self.addCol(CRBInDocTableCol(u'Вид деятельности',  'activity_id',  10,  'rbActivity')).setToolTip(u'Вид деятельности')


    def loadData(self, clientId = None):
        if clientId:
            self.clientId = clientId
            currentDate = forceDateTime(QDate.currentDate())
            db = QtGui.qApp.db
            tableSchedule = db.table('Schedule')
            tableScheduleItem = db.table('Schedule_Item')
            cols = [tableScheduleItem['id'],
                    tableScheduleItem['checked'],
                    tableScheduleItem['time'],
                    tableSchedule['appointmentType'],
                    tableSchedule['appointmentPurpose_id'],
                    tableSchedule['office'],
                    tableSchedule['person_id'],
                    tableSchedule['activity_id'],
                    tableScheduleItem['recordPerson_id'],
                    tableScheduleItem['complaint'],
                    tableScheduleItem['note'],
                   ]
            cond = [tableScheduleItem['client_id'].eq(self.clientId),
                    tableSchedule['date'].ge(currentDate),
                    tableSchedule['deleted'].eq(0),
                    tableScheduleItem['deleted'].eq(0),
                   ]
            tableQuery = tableScheduleItem
            tableQuery = tableQuery.leftJoin(tableSchedule, tableSchedule['id'].eq(tableScheduleItem['master_id']))
            records = db.getRecordList(tableQuery, cols, cond, 'Schedule_Item.time DESC')
            self.setItems(records)
        else:
            self.clearItems()


    def flags(self, index):
        result = Qt.ItemIsSelectable|Qt.ItemIsEnabled
        column = index.column()
        if column == 0:
            row = index.row()
            item = self.items()[row]
            if forceInt(item.value('appointmentType')) == CSchedule.atHome or QtGui.qApp.ambulanceUserCheckable():
                result = result | Qt.ItemIsUserCheckable
        return result


    def setData(self, index, value, role=Qt.EditRole):
        column = index.column()
        row = index.row()
        item = self.items()[row]
        if column == 0 and role == Qt.CheckStateRole:
            scheduleItemId = forceRef(item.value('id'))
            checked = forceBool(item.value('checked'))
            db = QtGui.qApp.db
            stmt = 'UPDATE Schedule_Item SET checked=%(newChecked)d ' \
                   'WHERE id=%(id)d AND client_id=%(clientId)d AND deleted=0' % dict(
                        id         = scheduleItemId,
                        clientId   = self.clientId,
                        newChecked = not checked,
                                                                                )
            r = db.query(stmt)
            item.setValue('checked', toVariant(not checked))
            if r.numRowsAffected()!=1:
                QtGui.qApp.emitCurrentClientInfoChanged()
            if QtGui.qApp.mainWindow.dockResources:
                QtGui.qApp.mainWindow.dockResources.content.updateTimeTable()
            self.emitCellChanged(row, column)
            return True
        return False


    def getScheduleItemId(self, row):
        item = self.items()[row]
        return forceRef(item.value('id'))



class CVisitsBySchedulesModel(CRecordListModel):
    def __init__(self,  parent):
        CRecordListModel.__init__(self, parent)
        self.addCol(CBoolInDocTableCol(u'Выполнено', 'event_id', 6)).setReadOnly()
        self.addCol(CEnumInDocTableCol(u'Тип',       'appointmentType', 6, CSchedule.atNames)).setReadOnly()
        self.addCol(CRBInDocTableCol(u'Назначение', 'appointmentPurpose_id', 20, 'rbAppointmentPurpose')).setReadOnly()
        self.addCol(CDateTimeInDocTableCol(u'Дата и время приема', 'date', 20)).setReadOnly()
        self.addCol(CInDocTableCol(u'Каб',          'office', 6)).setReadOnly()
        self.addCol(CRBInDocTableCol(u'Специалист', 'person_id', 20, 'vrbPersonWithSpeciality')).setReadOnly()
        self.addCol(CRBInDocTableCol(u'Записал',    'recordPerson_id', 20, 'vrbPersonWithSpeciality')).setReadOnly()
        self.addCol(CInDocTableCol(u'Примечания',   'note', 6)).setReadOnly()
        self.addCol(CRBInDocTableCol(u'Вид деятельности',  'activity_id',  10,  'rbActivity')).setToolTip(u'Вид деятельности').setReadOnly()


    def loadData(self, clientId):
        if clientId:
            db   = QtGui.qApp.db
            stmt =  'SELECT '\
                    ' Schedule_Item.id,'                                                            \
                    ' vVisitExt.event_id,'                                                          \
                    ' COALESCE(vVisitExt.date,      Schedule_Item.time) AS date,'                     \
                    ' COALESCE(vVisitExt.person_id, Schedule.person_id) AS person_id,'                \
                    ' Schedule.appointmentType,'                                                    \
                    ' Schedule.appointmentPurpose_id,'                                              \
                    ' Schedule.office,'                                                             \
                    ' Schedule.activity_id,'                                                             \
                    ' Schedule_Item.recordPerson_id,'                                               \
                    ' Schedule_Item.complaint '                                                     \
                    'FROM'                                                                          \
                    ' Schedule_Item'                                                                \
                    ' LEFT JOIN Schedule     ON Schedule.id      = Schedule_Item.master_id'         \
                    ' LEFT JOIN Person       ON Person.id        = Schedule.person_id'              \
                    ' LEFT JOIN vVisitExt    ON vVisitExt.client_id = Schedule_Item.client_id '     \
                                              ' AND Person.speciality_id=vVisitExt.speciality_id'   \
                                              ' AND DATE(vVisitExt.date) = Schedule.date'           \
                    ' WHERE Schedule_Item.client_id =%(clientId)d'                                  \
                      ' AND Schedule_Item.deleted = 0'                                              \
                      ' AND Schedule.deleted = 0'                                                   \
                      ' AND Schedule.date<=%(currentDate)s'                                         \
                    'ORDER BY Schedule_Item.time DESC'
            query = db.query(stmt%dict(clientId    = clientId,
                                       currentDate = db.formatDate(QDate.currentDate())
                                      )
                            )
            items = []
            while query.next():
                items.append(query.record())
            self.setItems(items)
        else:
            self.clearItems()


    def getScheduleItemId(self, row):
        item = self.items()[row]
        return forceRef(item.value('id'))


    def getEventId(self, row):
        item = self.items()[row]
        return forceRef(item.value('event_id'))


def contactToLikeMask(contact):
    m = '...'
    result = m+u''.join(contact.replace('-','').replace(' ','').replace('.','').replace('(','').replace(')','').replace('+',''))+m
    return result

#WTF?
def getKerContext():
    return ['tempInvalid', 'tempInvalidList']

#WTF?
def getActionContext():
    return ['actionsList']

#WTF?
def getVisitContext():
    return ['visitsList']

#WTF?
def getEventContextTemplate():
    return ['eventsList']


def getClientContext():
    return ['clientsList']


def getMedicalCommissionContext():
    return ['medicalCommission']


